'From Smalltalk/X, Version:2.10.8 on 25-nov-1995 at 6:34:29 pm'                 !

SimpleView subclass:#PVMMonitor
	 instanceVariableNames:'listView processes listUpdateDelay updateDelay updateBlock
		listUpdateBlock updateProcess groupName interface'
	 classVariableNames:''
	 poolDictionaries:''
	 category:'contrib-PVM-Tools'
!

!PVMMonitor class methodsFor:'documentation'!

documentation
"
    Abstract SuperClass for PVM Process and GroupProcess visualisation
"
!

examples
"
    examples to be added.
"
!

history
    "Created: 25.11.1995 / 16:30:51 / robert"
!

version
^ '$Header: /cvs/stx/stx/contrib/libPVM/PVMMonitor.st,v 1.2 1997/01/11 19:18:35 cg Exp $'! !

!PVMMonitor methodsFor:'destroying'!

destroy
    "shutdown pvm task"
    interface removeProcess: groupName.

    updateBlock notNil ifTrue:[
	Processor removeTimedBlock: updateBlock.
	Processor removeTimedBlock: listUpdateBlock.
    ] ifFalse:[
	updateProcess notNil ifTrue:[updateProcess terminate]
    ].
    super destroy

    "Modified: 25.11.1995 / 16:32:14 / robert"
! !

!PVMMonitor methodsFor:'drawing'!

titleLine
"/  ^ 'id   name                         cpu state    prio   usedStack  totalStack'.
    ^ 'TID     PTID    pvmdTID pid   name                                        '.
"      0000000 0000000 000000  00000 name max 28char"

    "Modified: 25.11.1995 / 16:32:38 / robert"
!

updateList
    "update list of processes"

    |newList|

    shown ifTrue:[
	"sort by id - take care of nil ids of dead processes"
	self updateStatus
    ].
    updateBlock notNil ifTrue:[
	Processor removeTimedBlock: listUpdateBlock.
	Processor addTimedBlock: listUpdateBlock afterSeconds: listUpdateDelay
    ].

    "Modified: 25.11.1995 / 16:32:45 / robert"
!

updateStatus
    "update status display of processes"

    |oldList list line con |

    shown "true" ifTrue:[

	list := OrderedCollection new.
	list add: self titleLine.
	processes := self getProcessList.

	processes do:[:aPVMTaskInfo |
	    | line nm pt h pid|    
	    line := (aPVMTaskInfo taskID hexPrintString) printStringPaddedTo: 8.
	    pt := (aPVMTaskInfo parentTaskID hexPrintString) printStringPaddedTo: 8.
	    h := (aPVMTaskInfo pvmdTaskID hexPrintString) printStringPaddedTo: 8.    
	    pid := aPVMTaskInfo localUnixProcessID printStringPaddedTo: 6.

	    (nm := aPVMTaskInfo taskFileName) isNil ifFalse:[
		nm := nm printString
	    ] ifTrue:[
		nm := 'not spawned '
	    ].
	    nm size >= 29 ifTrue:[
		nm := (nm contractTo:28) , ' '
	    ] ifFalse:[
		nm := (nm printStringPaddedTo:29).
	    ].
	    line := line, pt, h, pid, nm.
	    list add:line.
	].
	listView setList: list.

"/        ^list
    ].
    "
    PVMProcessMonitor new updateStatus
    "

    "Modified: 25.11.1995 / 16:32:52 / robert"
!

updateView
    self updateList.
    self updateStatus

    "Modified: 25.11.1995 / 16:32:59 / robert"
! !

!PVMMonitor methodsFor:'events'!

canHandle:key
    ^ key == #InspectIt

    "Modified: 25.11.1995 / 16:33:19 / robert"
!

keyPress:key x:x y:y
    key == #InspectIt ifTrue:[
	^ self inspectProcess.
    ].
    ^ super keyPress:key x:x y:y

    "Modified: 25.11.1995 / 16:33:25 / robert"
! !

!PVMMonitor methodsFor:'initialisation'!

initialize
    |v menu|

    super initialize.

    v := ScrollableView for:SelectionInListView in:self.
    v origin:0.0@0.0 corner:1.0@1.0.

    self extent:(font widthOf:self titleLine) + v scrollBar width @ 100.

    listView := v scrolledView.
    listView font: font.
    listView menuHolder:self; menuPerformer: self; menuMessage: #processMenu. 

    listView multipleSelectOk: true.
    listView delegate: (KeyboardForwarder toView: self).

    updateDelay := 5.0.
    listUpdateDelay := 2.

    true 
    ifTrue:[
	updateBlock := [self updateStatus].
	listUpdateBlock := [self updateList].
    ].

    groupName := 'processMonitor'.
    interface := PVMInterface new.
    interface  addProcess: groupName.
    interface isNil ifTrue:[
	self error: 'could not get an Interface'.
    ].

    ^self

    "Modified: 25.11.1995 / 16:33:50 / robert"
!

realize
    super realize.
    self startUpdateProcess.

    "Modified: 25.11.1995 / 16:33:57 / robert"
!

startUpdateProcess
    "return the block which does the work"

    updateBlock notNil ifTrue:[
	Processor addTimedBlock: updateBlock afterSeconds: updateDelay.
	Processor addTimedBlock: listUpdateBlock afterSeconds: listUpdateDelay.
    ] ifFalse:[
	updateProcess := [
	    Process terminateSignal handle:[:ex |
		updateProcess := nil
	    ] do:[
		[true] whileTrue:[
		    (Delay forSeconds:5.0) wait.
		    self updateStatus.

		    (Delay forSeconds:1.0) wait.
		    self updateList.
		]
	    ]
	]  forkAt:(Processor userSchedulingPriority + 1).

	updateProcess name:'monitor [' , 
			   Processor activeProcess id printString ,
			   '] update'.
	"
	 raise my own priority
	"
	Processor activeProcess priority:(Processor userSchedulingPriority + 2)
    ].

    "Modified: 25.11.1995 / 16:34:04 / robert"
! !

!PVMMonitor methodsFor:'menu actions'!

debugProcess
    "open a debugger on the selected process"

    self selectedProcessesDo:[:p |
       Debugger openOn:p
    ]

    "Modified: 25.11.1995 / 16:34:23 / robert"
!

inspectProcess
    "open an inspector on the selected process"

    self selectedProcessesSend:#inspect

    "Modified: 25.11.1995 / 16:34:29 / robert"
!

processMenu
    |labels selectors m|

    labels := resources array:#(
			 'inspect' 
			 'debug'  
			).
    selectors := #(
		    inspectProcess  
		    debugProcess  
		  ).

    m := PopUpMenu labels: labels
		   selectors: selectors.

    ^ m

    "Modified: 25.11.1995 / 16:34:36 / robert"
! !

!PVMMonitor methodsFor:'private'!

selectedProcessDo: aBlock
    |p sel|

    sel := listView selection.
    sel isNil ifTrue:[^ self].
    p := processes at: sel.
    p notNil ifTrue:[
       aBlock value:p
    ].

    "Modified: 25.11.1995 / 16:34:50 / robert"
! !

!PVMMonitor methodsFor:'queries'!

getProcessList
    "return the processList"

    | anArray |

    interface isNil ifTrue:[
	self warn: 'interface must not be nil'.
	^OrderedCollection new.
    ].

    anArray := PVMBasicInterface new pvmTasksInVM.

    ^anArray asOrderedCollection.

    "Modified: 25.11.1995 / 16:35:09 / robert"
!

preferredExtent
    ^ (font widthOf:self titleLine) + 40 @ 100

    "Modified: 25.11.1995 / 16:35:15 / robert"
! !

