"
 COPYRIGHT (c) 1997 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.

 This is a demo example:

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
"

"{ NameSpace: CodingExamples }"

Object subclass:#CModuleDemo
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'examples-C-Modules'
!

!CModuleDemo class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.

 This is a demo example:

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
"
!

documentation
"
    this example demonstrates, how a C object file can be dynamically
    loaded, the function(s) found there be called and finally be
    unloaded.

    This may be useful to implement things like extension interfaces 
    (a la netscape plugIns ;-?) which call third party C-code.

    To demonstrate, compile the C-source found in demo[1234].c
    into appropriate loadable object files
    (i.e.:
        make demo1.o demo2.o demo3.o demo4.o
        (on ELF-based systems, followed by:
            make demo1.so demo2.so demo3.so demo4.so
        )

    then fileIn this class and.

    Notice: watch the xterm window for the output
            (C-code cannot easily send messages to the Transcript)

    demo1 
        contains simple C functions.
    demo2 
        contains a C function; it provides an init function,
        which is automatically called when the module is loaded.
        The initFunctions returnValue indicates a 'sucessful' init.
    demo3
        like demo2, but indicates a failed initialization.
        Its automatically unloaded.

    Notice: the code here is not 'the best coding style';
    the path-stuff & loading should be done in a separate
    method.

    [start with:]
        CModuleDemo demo1
        CModuleDemo demo2
        CModuleDemo demo3

    [author:]
        Claus Gittinger

    [see also:]
        ObjectFileLoader ObjectFileHandle
"
! !

!CModuleDemo class methodsFor:'defaults'!

pathToObjectFiles
    "where are those object files ?"

    |path|

    path := Smalltalk getSystemFileName:'doc/coding/cModules'.
    path isNil ifTrue:[
        ^ self error:'cannot find any c modules (cModules directory)'
    ].
    ^ path

    "
     self pathToObjectFiles 
    "
! !

!CModuleDemo class methodsFor:'demos'!

demo1
    "load a C object file, extract some functions, call them and finally
     unload the object."

    |path ext handle f|

    "/ load the module.
    "/ instead of hardcoding the file-extension,
    "/ ask the system about its preferred extension for object files;
    "/ Thus the following works on NT and ELF based systems as well.

    'loading ...' printCR.
    ext := ObjectFileLoader sharedLibraryExtension.
    path := self pathToObjectFiles.
    handle := ObjectFileLoader 
                loadObjectFile:(path , '/' , 'demo1' , ext).

    handle isNil ifTrue:[
        ^ self error:'object file load failed'.
    ].

    "/
    "/ good; look for the C-function named 'function1' there
    "/

    'extracting function1''s handle ...' printCR.
    f := handle getFunction:'function1'.
    f isNil ifTrue:[
        ObjectFileLoader unloadDynamicObject:handle.
        ^ self error:'function1 not found in object'.    
    ].

    "/
    "/ call it ...
    "/

    'calling the C function ...' printCR.
    f call.

    "/
    "/ unload the object file
    "/

    'unloading ...' printCR.
    ObjectFileLoader unloadDynamicObject:handle

    "
     CModuleDemo demo1
    "
!

demo2
    "load a C object file which contains some auto-init function, 
     extract a function, call it and finally unload the object."

    |path ext handle f|

    "/ load the module.
    "/ instead of hardcoding the file-extension,
    "/ ask the system about its preferred extension for object files;
    "/ Thus the following works on NT and ELF based systems as well.

    'loading ...' printCR.
    ext := ObjectFileLoader sharedLibraryExtension.
    path := self pathToObjectFiles.
    handle := ObjectFileLoader 
                loadObjectFile:(path , '/' , 'demo2' , ext).

    handle isNil ifTrue:[
        ^ self error:'object file load failed'.
    ].

    "/
    "/ good; look for the C-function named 'function1' there
    "/

    'extracting function1''s handle ...' printCR.
    f := handle getFunction:'function1'.
    f isNil ifTrue:[
        ObjectFileLoader unloadDynamicObject:handle.
        ^ self error:'function1 not found in object'.    
    ].

    "/
    "/ call it ...
    "/

    'calling C function ...' printCR.
    f call.

    "/
    "/ unload the object file
    "/

    'unloading ...' printCR.
    ObjectFileLoader unloadDynamicObject:handle

    "
     CModuleDemo demo2
    "
!

demo3
    "load a C object file which contains some auto-init function, 
     (which fails), try to extract a function, call it and 
     finally unload the object.
     Since the initialization fails, the call will not be done."

    |path ext handle f|

    "/ load the module.
    "/ instead of hardcoding the file-extension,
    "/ ask the system about its preferred extension for object files;
    "/ Thus the following works on NT and ELF based systems as well.

    'loading ...' printCR.
    ext := ObjectFileLoader sharedLibraryExtension.
    path := self pathToObjectFiles.
    handle := ObjectFileLoader 
                loadObjectFile:(path , '/' , 'demo3' , ext).

    handle isNil ifTrue:[
        ^ self error:'object file load failed'.
    ].

    "/
    "/ good; look for the C-function named 'function1' there
    "/

    'extracting function1''s handle ...' printCR.
    f := handle getFunction:'function1'.
    f isNil ifTrue:[
        ObjectFileLoader unloadDynamicObject:handle.
        ^ self error:'function1 not found in object'.    
    ].

    "/
    "/ call it ...
    "/

    'calling C function ...' printCR.
    f call.

    "/
    "/ unload the object file
    "/

    'unloading ...' printCR.
    ObjectFileLoader unloadDynamicObject:handle

    "
     CModuleDemo demo3
    "
! !

!CModuleDemo class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/doc/coding/cModules/CModuleDemo.st,v 1.1 1997/01/08 17:31:59 cg Exp $'
! !
