/*
 * demonstration of a c-function module
 *
 * compile it with:
 *      cc -c demo1.c
 * or simply:
 *      make demo1.o
 *
 * on ELF based systems, make it a shared object with:
 *      ld -shared -o demo1.so demo1.o
 * or simply:
 *      make demo2.so
 *
 * in Smalltalk, load it with:
 *      handle := ObjectFileLoader loadDynamicObject:'demo1.o'  ('demo1.so' on ELF systems)
 *
 *
 * get function handles:
 *
 *      f1 := handle getFunction:'function1'.
 *      f2 := handle getFunction:'function2'.
 *
 *
 * call a function:
 *
 *      f1 call.
 *
 *
 * call a function with an argument:
 *
 *      f2 callWith:999.
 *
 *
 * call a function which returns something:
 * (f3 returns a string):
 *
 *      |bytes|
 *
 *      bytes := ExternalBytes address:(f3 callWith:'hello world')
 *   (be careful, the string is void when the module is unloaded;
 *    therefore, it should be immediately extracted and converted to
 *    a smalltalk object:
 *      s := bytes stringAt:1
 *   )
 *
 *
 * call a function with double-argument, which returns something:
 * (f4 expects a double and returns a double):
 *
 *      f4 callWithDouble:(Float pi) returnsDouble:true
 *
 *
 * finally, unload the library:
 *      ObjectFileLoader unloadDynamicObject:handle
 */

void
function1() {
    printf("here is function1\n");
}

int
function2(i) {
    printf("here is function2 i:%d\n", i);
    return i * 2;
}

char *
function3(s) 
    char *s;
{
    printf("here is function3 s:%s\n", s);
    return "a string";
}

double
function4(d)
    double d;
{
    printf("here is function4 d:%f; returning 100.0\n", d);
    return 100.0;
}
