"
    this class has been extracted from the manchester goodies
    library - it is not covered by the Smalltalk/X copyright.

    The original readme text is:

    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1993 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 
"
"{ Package: 'stx:goodies' }"

Collection subclass:#Chain
	instanceVariableNames:'lastLink firstLink'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Linked'
!

Chain comment:'documentation
"
    Chains are a collection of arbitrary elements explicitly ordered by the
    sequence in which elements are added and removed or inserted.  Each element
    is stored using a ChainLink.  These links can be traversed efficiently
    forwards or backwards using nextLink and previousLink.  The ChainLinks are
    linked as a ring but an individual ChainLink can efficiently determine if
    it is the first or last link in the Chain.  So the Chain can act as a ring
    or as non-ring.

    Chain provides many of the same abilities as OrderedCollection except Chain
    is not indexable.  OrderedCollections are much more space efficient but
    Chains are faster for some addition/removal operations.

    Adding a new link before or after any given link or at the beginning or end
    of the Chain is an Order(1) operation.  Removing any given link is an
    Order(1) operation.  For OrderedCollections all these operations (except
    removeLast) are Order(n).

    This class and its private link class ChainLink were implemented by Carl
    Watts (Carl@ParcPlace.com).

    [Instance Variables:]
	firstLink  <ChainLink or self>   The first link in the chain 
					 or self if empty.

	lastLink  <ChainLink or self>    The last link in the chain 
					 or self if empty.

    [author:]
	Carl Watts (Carl@ParcPlace.com)

    [see also:]
	OrderedCollection LinkedList
	ChainLink
"
'
!

!Chain class methodsFor:'documentation'!

copyright
"
    this class has been extracted from the manchester goodies
    library - it is not covered by the Smalltalk/X copyright.

    The original readme text is:

    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1993 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 
"


!

documentation
"
    Originl comments by Carl Watts:

    Chains are a collection of arbitrary elements explicitly ordered by the
    sequence in which elements are added and removed or inserted.  Each element
    is stored using a ChainLink.  These links can be traversed efficiently
    forwards or backwards using nextLink and previousLink.  The ChainLinks are
    linked as a ring but an individual ChainLink can efficiently determine if
    it is the first or last link in the Chain.  
    So the Chain can act as a ring or as non-ring.

    Chain provides many of the same abilities as OrderedCollection except Chain
    is NOT indexable.  
    OrderedCollections are much more space efficient but
    Chains may be faster for some addition/removal operations (in the middle).

    Adding a new link before or after any given link or at the beginning or end
    of the Chain is an Order(1) operation.  Removing any given link is an Order(1) operation.  
    For OrderedCollections all these operations (except removeLast) are Order(n).

    This class and its private link class ChainLink were implemented by Carl
    Watts (Carl@ParcPlace.com).

    Note by claus:
        - OrderedCollections are quite fast in ST/X 
          (try the benchmarks ... OCs are hard to beat.
           At least with small Chains (say, up to 10000 elements), OCs
           are faster.)

        - also, keep in mind that Chains produce quite a lot of additional
          space overhead ...

        - therefore, you have to check for your concrete application, if a Chain    
          is the right collection class for your speed needs.

    [Instance Variables:]
        firstLink  <ChainLink or self>   The first link in the chain 
                                         or self if empty.

        lastLink  <ChainLink or self>    The last link in the chain 
                                         or self if empty.

    [author:]
        Carl Watts (Carl@ParcPlace.com)

    [see also:]
        OrderedCollection LinkedList
        ChainLink
"
!

examples
"
  exampleBenchmark1:
    This benchmark compares an operation that both Chain and
    OrderedCollection are good at, adding at the end.  
    Their times should be comparable 
    (cg: well, OC is roughly 5 times faster).
                                                                        [exBegin]
        Chain runBenchMark: [:collectionClass |
            | aCollection |
            20 timesRepeat: [
                    aCollection := collectionClass new.
                    aCollection addAll: (1 to: 10000)]]
                                                                        [exEnd]
  exampleBenchmark2:
    This benchmark compares operations that both Chain and OrderedCollection
    are good at, adding at the end and enumerating.  
    Their times should be comparable.
    (cg: well, OC is roughly 2-3 times faster).
                                                                        [exBegin]
        Chain runBenchMark: [:collectionClass |
            | aCollection |
            20 timesRepeat: [
                    aCollection := collectionClass new.
                    aCollection addAll: (1 to: 10000).
                    aCollection do: [:each | each + 1]]]
                                                                        [exEnd]
  exampleBenchmark3
     This benchmark shows that Chain is significantly faster when removing
     elements from the middle even though both Chain and OrderedCollection have
     to do a linear search to find the element.
                                                                        [exBegin]
        Chain runBenchMark: [:collectionClass |
            | aCollection |
            8 timesRepeat: [
                    aCollection := collectionClass new.
                    aCollection addAll: (1 to: 10000).
                    aCollection removeAll: (250 to: 7500)]]
                                                                        [exEnd]
  exampleBenchmark3
     same as above, measuring the net removal time alone:
                                                                        [exBegin]
        |chain oc|

        chain := Chain new.
        chain addAll: (1 to: 10000).
        Transcript 
            show:'Chain time ';
            showCR:
                 (Time millisecondsToRun:[
                     chain removeAll: (2500 to: 7500)
                 ]);
            endEntry.

        oc := OrderedCollection new.
        oc addAll: (1 to: 10000).
        Transcript 
            show:'OrderedCollection time ';
            showCR:
                 (Time millisecondsToRun:[
                     oc removeAll: (2500 to: 7500)
                 ]);
            endEntry.
                                                                        [exEnd]
  exampleBenchmark3
     same as above, removing individual elements;
                                                                        [exBegin]
        |chain oc|

        chain := Chain new.
        chain addAll: (1 to: 1000).
        Transcript 
            show:'Chain time ';
            showCR:
                 (Time millisecondsToRun:[
                     250 to:750 do:[:what | chain remove:what] 
                 ]);
            endEntry.

        oc := OrderedCollection new.
        oc addAll: (1 to: 1000).
        Transcript 
            show:'OrderedCollection time ';
            showCR:
                 (Time millisecondsToRun:[
                     250 to:750 do:[:what | oc remove:what] 
                 ]);
            endEntry.
                                                                        [exEnd]
"
! !

!Chain class methodsFor:'instance creation'!

new
	"Answer with a new instance of the receiver."

	^super new initialize
! !

!Chain class methodsFor:'benchmarks'!

runBenchMark: benchMarkBlock
    "Run a benchmark with the given test block.  
     The block will be evaluated with different collection classes 
     as the parameter.  The time required to evaluate the block will be compared."

    | chainTime orderedCollectionTime |

    chainTime := Time millisecondsToRun: [benchMarkBlock value: Chain].
    Transcript cr; cr; nextPutAll: 'Chain time '; print: chainTime; endEntry.
    orderedCollectionTime := Time millisecondsToRun: [
				benchMarkBlock value: OrderedCollection].
    Transcript cr; nextPutAll: 'OrderedCollection time '; 
		   print: orderedCollectionTime; endEntry.
    Transcript cr; nextPutAll: 'Chain is '; 
		   print: orderedCollectionTime / chainTime asFloat; 
		   nextPutAll: ' times faster for this benchmark'; cr;
		   endEntry
! !

!Chain methodsFor:'accessing'!

first
	"Answer the first value;  provide an error notification if the receiver is
empty."

	^self firstLink value
!

firstLink
	"Answer the first link;  provide an error notification if the receiver is
empty."

	self emptyCheck.
	^firstLink
!

last
	"Answer the last value;  provide an error notification if the receiver is
empty."

	^self lastLink value
!

lastLink
	"Answer the last link;  provide an error notification if the receiver is
empty."

	self emptyCheck.
	^lastLink
!

linkAtValue: aValue
	"Answer a link with value = aValue."

	| aLink |
	self isEmpty ifTrue: [^self].
	aLink := firstLink.
	[aValue = aLink value ifTrue: [^aLink].
	 aLink := aLink nextLink.
	 aLink == firstLink] whileFalse.
	self notFoundError
!

size
	"Answer how many elements the receiver contains."

	| tally |
	tally := 0.
	self do: [:each | tally := tally + 1].
	^tally
! !

!Chain methodsFor:'adding'!

add: aValue
	"Add aValue to the end of the receiver.  Answer aValue."

	^self addLast: aValue
!

addFirst: aValue
	"Add aValue to the beginning of the receiver. Answer aValue"

	self addLinkFirst: (ChainLink value: aValue).
	^aValue
!

addLast: aValue
	"Add aValue to the end of the receiver. Answer aValue."

	self addLinkLast: (ChainLink value: aValue).
	^aValue
!

addLink: aLink
	"Add aLink to the end of the receiver.  Answer aLink."

	^self addLinkLast: aLink
!

addLinkFirst: aLink
	"Add aLink to the beginning of the receiver."

	aLink previousLink: self nextLink: firstLink.
	^aLink
!

addLinkLast: aLink
	"Add aLink to the end of the receiver."

	aLink previousLink: lastLink nextLink: self.
	^aLink
! !

!Chain methodsFor:'enumerating'!

do: aBlock
    "Evaluate aBlock with each of the receiver's values as the argument."

    | aLink |

    self isEmpty ifTrue: [^self].

    aLink := firstLink.
    [
	aBlock value: aLink value.
	aLink := aLink nextLink.
	aLink == firstLink
    ] whileFalse
!

linksDo: aBlock
    "Evaluate aBlock with each of the receiver's links as the argument."

    | aLink |

    self isEmpty ifTrue: [^self].
    aLink := firstLink.
    [
	aBlock value: aLink.
	aLink := aLink nextLink.
	aLink == firstLink
    ] whileFalse
! !

!Chain methodsFor:'initialization'!

initialize
    "Initialize the receiver."

    firstLink := lastLink := self
! !

!Chain methodsFor:'private'!

linkAfterPreviousLink
    "Answer the link after the receiver's previous link."
    "Double-Dispatched method from ChainLink.  
     For the receiver answer the first link."

    ^firstLink
!

linkBeforeNextLink
    "Answer the link before the receiver's next link."
    "Double-Dispatched method from ChainLink.  
     For the receiver answer the last link."

    ^ lastLink
!

nextLinkIsFirst
    "Answer if the nextLink is the first link in the chain."
    "Double-dispatched from a ChainLink.  I know I can answer true."

    ^true
!

previousLink: aLink
    "Set the ChainLink before the receiver."
    "Double-dispatched method from a ChainLink.  Set aLink as the first link."

    firstLink := aLink
!

previousLinkIsLast
    "Answer if the previousLink is the last link in the chain."
    "Double-dispatched from a ChainLink.  I know I can answer true."

    ^true
!

setNextLink: aLink
    "Set the ChainLink before the receiver."
    "Double-dispatched method from a ChainLink.  Set aLink as the first link."

    firstLink := aLink
!

setPreviousLink: aLink
    "Set the ChainLink before the receiver."
    "Double-dispatched method from a ChainLink.  Set aLink as the last link."

    lastLink := aLink
! !

!Chain methodsFor:'removing'!

remove: oldObject ifAbsent: anExceptionBlock 
    "Remove oldObject as one of the receiver's elements.  
     Answer oldObject unless no element is equal to oldObject, 
     in which case, evaluate anExceptionBlock."

    (self linkAtValue: oldObject) remove.
    ^oldObject
! !

!Chain methodsFor:'testing'!

isEmpty
    "Answer whether the receiver contains any elements."

    ^firstLink == self
! !

!Chain class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Collections-Chain.st,v 1.8 2009/09/05 10:01:37 cg Exp $'
! !
