"
 This class is not covered by or part of the ST/X licence.


 COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help 
"



"{ Package: 'stx:goodies' }"

SequenceableCollection subclass:#CompressedOrderedCollection
	instanceVariableNames:'runs tally'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Sequenceable'
!

CompressedOrderedCollection comment:'This implements an ordered collection which uses runs to minimise the amount
 of data that it holds. Basically it should be used if you know that an ordered
 collections is giong to contain a lot of runs of eactly the same data. Implemented
 to allow simultation playback, since the ordered collctions which that generates
 are so big that the complier falls over, though most of it is extremely repetetive.
 This should be totally abstracted. The user should not be a ble to see the difference
 between an ordered collection and a ComrpessedOrderedCollection.  This has a
 lot in common with RunArray, and the two should probably share implementation.
 but I could not do some of the things I wanted with the RunArray code, so this
 is all done on its own.
	Some of this could be made faster by adding a cache of the start and finish
 indices of each run, but since I envisage that most additions etc. will be to
 and from the end those are not included. In addition I have implemented the
 bare essentials of this for what I need it for - i.e. add to the end and take
 off the beginning.'
!

!CompressedOrderedCollection class methodsFor:'documentation'!

copyright
"
 This class is not covered by or part of the ST/X licence.


 COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help 
"


!

documentation
"
    This implements an ordered collection which uses runs to minimise the amount
    of data that it holds. Basically it should be used if you know that an ordered
    collections is going to contain a lot of runs of eactly the same data. 
    Implemented to allow simulation playback, since the ordered collections which that 
    generates are so big that the system falls over, though most of it is extremely repetetive.

    This should be totally abstracted. The user should not be able to see the difference
    between an OrderedCollection and a CompressedOrderedCollection.  This has a
    lot in common with RunArray, and the two should probably share implementation.
    but I could not do some of the things I wanted with the RunArray code, so this
    is all done on its own.

    Some of this could be made faster by adding a cache of the start and finish
    indices of each run, but since I envisage that most additions etc. will be to
    and from the end those are not included. In addition I have implemented the
    bare essentials of this for what I need it for - i.e. add to the end and take
    off the beginning.

    No WARRANTY:
        this is an adaption of a PD class.

    Notice:
        there is only a space saving if there are really runs (i.e. multiple
        elements which compare same.
        Otherwise, an OrderedCollection is more compact.


    [info:]
        NAME            CompressedOrderedCollection
        AUTHOR          RMD@cs.man.ac.uk
        FUNCTION        like RunArrays, for OrderedCollections 
        ST-VERSIONS     2.2
        PREREQUISITES    
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1.1
        DATE            22 Jan 1989

    [author:]
        RMD@cs.man.ac.uk
        Claus Gittinger (adding/removing of inner elements & grow)

    [see also:]
        Run OrderedCollection
"
!

examples
"
  this eats up a lot of memory ...
  ... but its relatively fast:
                                                                        [exBegin]
    |coll|

    coll := OrderedCollection new.
    Transcript showCR:(    
        Time millisecondsToRun:[
            100000 timesRepeat:[coll add:'hello'].
            100000 timesRepeat:[coll add:'world'].
        ]
    ).
    coll.
                                                                        [exEnd]

  ... much faster, if we give it an initial size:
                                                                        [exBegin]
    |coll|

    coll := OrderedCollection new:200000.
    Transcript showCR:(    
        Time millisecondsToRun:[
            100000 timesRepeat:[coll add:'hello'].
            100000 timesRepeat:[coll add:'world'].
        ]
    ).
    coll.
                                                                        [exEnd]

  this is very space efficient ...
                                                                        [exBegin]
    |coll|

    coll := CompressedOrderedCollection new.
    Transcript showCR:(    
        Time millisecondsToRun:[
            100000 timesRepeat:[coll add:'hello'].
            100000 timesRepeat:[coll add:'world'].
        ]
    ).
    coll.
                                                                        [exEnd]


  this is very space efficient ...
  ... AND much faster:
                                                                        [exBegin]
    |coll|

    coll := CompressedOrderedCollection new.
    Transcript showCR:(    
        Time millisecondsToRun:[
            coll add:'hello' withOccurrences:100000.
            coll add:'world' withOccurrences:100000.
        ]
    ).
    coll.
                                                                        [exEnd]


  this is very space INEFFICIENT (a real memory pig ;-) ...
  ... and much slower:
                                                                        [exBegin]
    |coll|

    coll := CompressedOrderedCollection new.
    Transcript showCR:(    
        Time millisecondsToRun:[
            1 to:100000 do:[:i | coll add:i].
        ]
    ).
    coll.
                                                                        [exEnd]


  things like this are VERY slow:
                                                                        [exBegin]
    |coll|

    coll := CompressedOrderedCollection new.
    1 to:100000 do:[:i | coll add:i].
    Transcript showCR:(    
        Time millisecondsToRun:[
            100000 to:1 by:-2 do:[:i | coll removeIndex:i].
        ]
    )
                                                                        [exEnd]

  much faster with OCs (although these are not optimal for this, too):
                                                                        [exBegin]
    |coll|

    coll := OrderedCollection new.
    1 to:1000 do:[:i | coll add:i].
    Transcript showCR:(    
        Time millisecondsToRun:[
            1000 to:1 by:-2 do:[:i | coll removeIndex:i].
        ]
    )
                                                                        [exEnd]

  in general, such things are better done by constructing a new collection
  from scratch (use #select: or #collect:)
"
! !

!CompressedOrderedCollection class methodsFor:'instance creation'!

new
    "Answer a new empty instance of OrderedCollection."

    ^ (self basicNew) initialize
!

new:aNumber
    "Answer a new empty instance of OrderedCollection. 
     This is here for compatibility,  it totally ignores the parameter, 
     since it is impossible to predict how many  runs will be required."

     ^ self new
! !

!CompressedOrderedCollection methodsFor:'accessing'!

after:oldObject ifAbsent:exceptionBlock 
    "Answer the element after oldObject.  If the receiver does not contain 
     oldObject or if the receiver contains no elements after oldObject,  
     provide an error notification. This is a dumb thing to do to a 
     compressed collection, so no attempt has been made to make it 
     efficient. It answers with the element immediately after oldObject, 
     which probably means it is in the same run and will have the same 
     value. "

    | index |

    index := self find:oldObject ifAbsent:0.
    index == 0 ifTrue: [^ exceptionBlock value].
    ^ self at: index + 1

    "Created: 10.5.1996 / 14:00:49 / cg"
    "Modified: 10.5.1996 / 14:01:16 / cg"
!

at:anInteger 
    "Answer the element at index anInteger. 
     at: is used by a knowledgeable client to access an existing element 
     This is a pretty dumb thing to do a compressed collection and it is 
     not at all efficient (think of that as a discouragement)."

    | position |

    (anInteger < 1 or: [anInteger > tally]) ifTrue: [
	^ self errorInvalidKey:anInteger
    ].

    position := 1.
    runs do:[:run | 
	|runLen runLast|

	runLen := run length.
	anInteger >= position ifTrue:[
	    runLast := position + runLen - 1.
	    anInteger <= runLast ifTrue:[
		^ run value
	    ].
	].
	position := position + runLen
    ]

    "
     |c|

     c := CompressedOrderedCollection new.
     c add:1; add:1; add:1; add:2; add:2; add:3; add:3; add:4; add:5.
     c at:2. 
     c at:3. 
     c at:4.  
     c at:5. 
     c at:7.
     c at:8.  
     c at:9.  
     c at:10.  
    "

    "Modified: 10.5.1996 / 16:59:03 / cg"
!

at:index put:anObject 
    "Put anObject at element index anInteger.      
     at:put: can not be used to append, front or back, to an ordered      
     collection;  it is used by a knowledgeable client to replace an     
     element. It doesn't make a lot of sense for a compressed collection,    
     and as you can see, the implementation is awful - still if you will    
     insist on using this what can you expect. Basically this just copies 
     itself up to the start point then adds the required element then 
     copies the rest of itself"

    | run runIndex runOffset info len l1 l2 val newRuns prevRun nextRun|

    "first check the range"
    (index < 1 or: [index > tally]) ifTrue: [
	^ self errorInvalidKey:index
    ].

    info := self runIndexAndStartIndexForIndex:index.
    runIndex := info at:1.
    runOffset := info at:2.

    run := runs at:runIndex.

    "/ easiest: value there is the same ...

    (val := run value) = anObject ifTrue:[
	^ anObject
    ].

    "/ or the length is 1 ...

    (len := run length) = 1 ifTrue:[
	run value:anObject.

	"/ check if it can be merged into the next or previous run

	runIndex > 1 ifTrue:[
	    prevRun := runs at:(runIndex - 1).
	    prevRun value = anObject ifTrue:[
		prevRun incrementLength.
		runs removeIndex:runIndex.

		runIndex <= runs size ifTrue:[
		    nextRun := runs at:runIndex.
		    nextRun value = anObject ifTrue:[
			prevRun increaseLength:(nextRun length).
			runs removeIndex:runIndex.
		    ].
		].
		^ anObject
	    ].
	].

	runIndex < runs size ifTrue:[
	    nextRun := runs at:runIndex+1.
	    nextRun value = anObject ifTrue:[
		nextRun incrementLength.
		runs removeIndex:runIndex.
	    ].
	].

	^ anObject
    ].

    "/ sorry - must split the run

    runOffset == index ifTrue:[
	"/ at the beginning ...

	"/ check if its better added to the previous run ...

	runIndex > 1 ifTrue:[
	    prevRun := runs at:(runIndex - 1).
	    prevRun value = anObject ifTrue:[
		prevRun incrementLength.
		run decrementLength.
		^ anObject.
	    ].
	].
        
	run length:(len - 1).
	runs add:(Run length:1 value:anObject) beforeIndex:runIndex.
	^ anObject
    ].

    (runOffset + len - 1) == index ifTrue:[
	"/ at the end ...

	"/ check if its better added to the next run ...

	runIndex < (runs size) ifTrue:[
	    nextRun := runs at:(runIndex + 1).
	    nextRun value = anObject ifTrue:[
		nextRun incrementLength.
		run decrementLength.
		^ anObject.
	    ].
	].
	run length:(len - 1).
	runs add:(Run length:1 value:anObject) beforeIndex:runIndex+1.
	^ anObject
    ].

    "/ hardest - split run into two, insert new run in-between

    newRuns := OrderedCollection new grow:(runs size + 2).
    runIndex > 1 ifTrue:[
	newRuns replaceFrom:1 to:runIndex-1 with:runs.
    ].
    newRuns replaceFrom:runIndex+3 to:(runs size+2) with:runs startingAt:runIndex+1.

    l2 := len - (index - runOffset).
    l1 := len - l2.
    l2 := l2 - 1.

    newRuns at:runIndex   put:(Run length:l1 value:run value).
    newRuns at:runIndex+2 put:(Run length:l2 value:run value).

    "/ insert
    newRuns at:runIndex+1 put:(Run length:1 value:anObject).
    runs := newRuns.
    ^ anObject

    "
     |c|

     c := CompressedOrderedCollection new.
     c add:1; add:1; add:1; add:2; add:2; add:3; add:3; add:4; add:5; yourself.
     Transcript showCR:c.   

     c at:1 put:$a.
     Transcript showCR:c.   
     c.

     c at:3 put:$a.
     Transcript showCR:c.   
     c.

     c at:4 put:$a.   
     Transcript showCR:c.   
     c.

     c at:5 put:$a.   
     Transcript showCR:c.   
     c.

     c at:2 put:$a.   
     Transcript showCR:c.   
     c.

     Transcript showCR:c.   
    "

    "Modified: 11.5.1996 / 12:19:27 / cg"
!

before:oldObject ifAbsent:exceptionBlock
    "Answer the element before oldObject.  If the receiver does not 
     contain oldObject or if the receiver contains no elements before 
     oldObject, provide an error notification. This is a dumb thing to do to 
     a compressed collection, so no attempt has been made to make it 
     efficient. It answers with the element immediately before oldObject, 
     which probably means it is in the same run and will have the same 
     value. "

    |index|

    index := self find:oldObject ifAbsent:0.
    index == 0 ifTrue: [^ exceptionBlock value].
    ^ self at: index - 1

    "Created: 10.5.1996 / 14:05:09 / cg"
    "Modified: 11.5.1996 / 12:20:37 / cg"
!

first
    "Return the value of my first element."

    tally == 0 ifTrue:[
	^ self emptyCollectionError.
    ].
    ^runs first value

    "Modified: 11.5.1996 / 12:19:52 / cg"
!

last
    "Return the value of my last element."

    tally == 0 ifTrue:[
	^ self emptyCollectionError.
    ].
    ^runs last value

    "Modified: 11.5.1996 / 12:20:25 / cg"
!

size
    "Answer how many elements the receiver contains."

    ^ tally.

    "Modified: 11.5.1996 / 12:20:31 / cg"
! !

!CompressedOrderedCollection methodsFor:'adding & removing'!

add:newObject
    "Add newObjectto the end of this collection."

    self add:newObject withOccurrences:1.
    ^ newObject.

    "
     |c|

     c := CompressedOrderedCollection new.
     c add:1; add:1; add:1; add:2; add:2; add:3; add:3; add:4; add:5.
    "

    "Modified: 11.5.1996 / 12:21:22 / cg"
!

add:anObject beforeIndex:index 
    "Add the argument, newObject, as an element of the receiver.  
     Put it in the position just preceding the indexed position.  
     Answer newObject."

    | run runIndex runOffset prevRun info newRuns len l1 l2 val|

    (index < 1 or:[index > (tally + 1)]) ifTrue:[
	^ self errorInvalidKey:index
    ].

    "/ handle special cases

    index == 1 ifTrue:[
	tally == 0 ifTrue:[
	    self add:anObject.
	    ^ anObject
	].
	runIndex := 1.
	runOffset := 1.
    ] ifFalse:[
	index == (tally+1) ifTrue:[
	    self addLast:anObject.
	    ^ anObject
	].
	info := self runIndexAndStartIndexForIndex:index.
	runIndex := info at:1.
	runOffset := info at:2.
    ].

    run := runs at:runIndex.

    "/ check if it can be merged into the run

    (val := run value) = anObject ifTrue:[
	run incrementLength.
	tally := tally + 1.
	^ anObject
    ].

    "/ if its the first, and there is a previous run,
    "/ we can tack it at the end of the previous
        
    runOffset == index ifTrue:[
	runIndex > 1 ifTrue:[
	    prevRun := runs at:runIndex - 1.
	    prevRun value = anObject ifTrue:[
		prevRun incrementLength.
		tally := tally + 1.
		^ anObject
	    ]
	]
    ].

    runOffset == index ifTrue:[
	"/ well - have to insert a new run.

	newRuns := OrderedCollection new grow:(runs size + 1).
	runIndex > 1 ifTrue:[
	    newRuns replaceFrom:1 to:runIndex-1 with:runs.
	].
	newRuns replaceFrom:runIndex+1 to:(runs size+1) with:runs startingAt:runIndex.
	newRuns at:runIndex put:(Run length:1 value:anObject).
    ] ifFalse:[
	"/ well - have to split a run.

	newRuns := OrderedCollection new grow:(runs size + 2).
	runIndex > 1 ifTrue:[
	    newRuns replaceFrom:1 to:runIndex-1 with:runs.
	].
	newRuns replaceFrom:runIndex+3 to:(runs size+2) with:runs startingAt:runIndex+1.

	"/ split
	len := run length.
	l2 := len - (index - runOffset).
	l1 := len - l2.
	newRuns at:runIndex   put:(Run length:l1 value:val).
	newRuns at:runIndex+2 put:(Run length:l2 value:val).

	"/ insert
	newRuns at:runIndex+1 put:(Run length:1 value:anObject).
    ].
    tally := tally + 1.
    runs := newRuns.
    ^ anObject

    "
     |c|

     c := CompressedOrderedCollection new.
     c add:1; add:1; add:1; add:2; add:2; add:3; add:3; add:4; add:5; yourself.
     Transcript showCR:c.

     c add:99 beforeIndex:3; yourself.
     Transcript showCR:c.
     c add:999 beforeIndex:11; yourself.
     Transcript showCR:c.
     c add:1 beforeIndex:4; yourself.
     Transcript showCR:c.
     c add:2 beforeIndex:2; yourself.
     Transcript showCR:c.
    "

    "Modified: 11.5.1996 / 08:28:28 / cg"
!

add:newObject withOccurrences:n
    "Add newObject n times to the end of this collection."

    |lastRun|

    "First see if there is anything in this collection - if not just insert a 
     new run."

    runs isEmpty ifTrue:[
	runs addLast:(Run length:n value:newObject)
    ] ifFalse: [
	"/ If this is the same value as the last run add it to the run, 
	"/ otherwise add a new run

	(lastRun := runs last) value = newObject ifTrue:[
	    lastRun increaseLength:n
	] ifFalse: [
	    runs addLast:(Run length:n value:newObject)
	]
    ].
    tally := tally + n.

    "Modified: 11.5.1996 / 12:12:27 / cg"
    "Created: 11.5.1996 / 12:13:27 / cg"
!

addAll: aCollection
    "Add all elements of the argument, aCollection, at the end of the receiver."

    aCollection do:[:element | self addLast:element]

    "Modified: 11.5.1996 / 12:21:41 / cg"
!

addFirst:newObject 
    "Add newObject to the start of this collection."

    "First see if there is anything in this collection
     - if not just insert a new run."

    runs isEmpty ifTrue: [
	runs addLast: (Run length: 1 value: newObject).
    ] ifFalse: [

	"/ If this is the same value as the first run add it to the run, 
	"/ otherwise add a new run

	runs first value = newObject ifTrue: [
	    runs first incrementLength
	] ifFalse: [
	    runs addFirst:(Run length: 1 value: newObject)
	]
    ].
    tally := tally + 1.

    "Modified: 11.5.1996 / 12:17:59 / cg"
!

grow:howBig
    "grow or shrink the receiver to contain howBig elements.
     If growing, new slots will be filled with nil."

    |info runIndex runOffset|

    howBig == tally ifTrue:[^ self].
    howBig == 0 ifTrue:[
	runs := OrderedCollection new.
	tally := 0.
	^ self.
    ].

    tally == 0 ifTrue:[
	runs := OrderedCollection with:(Run length:howBig value:nil).
	tally := howBig.
	^ self
    ].

    howBig > tally ifTrue:[
	runs addLast:(Run length:(howBig - tally) value:nil).
	tally := howBig.
	^ self
    ].

    "/ shrinking; possibly cut of a run

    info := self runIndexAndStartIndexForIndex:howBig.
    runIndex := info at:1.
    runOffset := info at:2.

    howBig == (runOffset - 1) ifTrue:[
	"/ we are lucky - new size is up-to previous run

	runs removeFromIndex:runIndex toIndex:(runs size).
    ] ifFalse:[
	runs removeFromIndex:runIndex+1 toIndex:(runs size).
	runs last length:(howBig - runOffset + 1)
    ].
    tally := howBig.

    "
     |c|

     c := CompressedOrderedCollection new.
     c addAll:#(1 2 3 4 4 4 4 5 5 5 1 2 3).
     c.    

     c removeIndex:3.
     c.  

     c grow:50.
     c.      

     c grow:50.
     c.        

     c grow:7.
     c.       

     c grow:6.
     c.       

     c grow:0.
     c.       
    "

    "Modified: 10.5.1996 / 18:11:54 / cg"
!

remove: oldObject ifAbsent: anExceptionBlock 
    "Remove oldObject as one of the receiver's elements.  If several of the 
     elements are equal to oldObject, only one is removed. If no element is 
     equal to 
     oldObject, answer the result of evaluating anExceptionBlock.  
     Otherwise, answer the argument, oldObject."

    |index|

    index := self find:oldObject ifAbsent:0.
    index == 0 ifTrue:[
	^ anExceptionBlock value
    ].
    self removeIndex:index.
    ^oldObject

    "Modified: 10.5.1996 / 18:24:06 / cg"
!

removeAllSuchThat:aBlock 
    "Evaluate aBlock for each element of the receiver.  Remove each 
     element for 
     which aBlock evaluates to true.   
     This is easy - just check for the value of each run and if ablock is 
     true remove the run"

    runs removeAllSuchThat:[:aRun | 
	(aBlock value:aRun value) ifTrue:[
	    tally := tally - aRun length.
	    true
	] ifFalse:[
	    false
	]
    ]

    "Modified: 10.5.1996 / 16:57:20 / cg"
!

removeAtIndex: anIndex 
    "Remove the element of the collection at position anIndex.  
     Re-implement rather than inherit so that we don't have to faff 
     about with firstIndex which is not used anywhere else. Answer the 
     object removed."

    |obj|

    obj := self at:anIndex.
    self removeIndex: anIndex.
    ^ obj

    "Modified: 10.5.1996 / 18:23:56 / cg"
!

removeFirst
    "Remove the first element of the receiver.  If the receiver is empty,   
     provide an error notification."

    |returnValue run l|

    tally == 0 ifTrue:[
	^ self emptyCollectionError.
    ].

    run := runs first.
    returnValue := run value.
    (l := run length) == 1 ifTrue:[
	runs removeFirst
    ] ifFalse:[
	run length:(l - 1)
    ].

    tally := tally - 1.
    ^ returnValue

    "Modified: 11.5.1996 / 12:22:15 / cg"
!

removeIndex:index 
    "Remove the element of the collection at position anIndex."

    |info runIndex run l|

    (index < 1 or:[index > (tally + 1)]) ifTrue:[
	^ self errorInvalidKey:index
    ].

    info := self runIndexAndStartIndexForIndex:index.
    runIndex := info at:1.

    tally := tally - 1.

    run := runs at:runIndex.

    l := run length - 1.
    l ~~ 0 ifTrue:[
	run length:l.
	^ self
    ].

    "/ that run is to be removed ...

    runIndex == 1 ifTrue:[
	runs removeFirst.
	^ self
    ].
    runIndex == (runs size) ifTrue:[
	runs removeLast.
	^ self
    ].

    "/ check if the previous run can be merged with the next

    (runs at:(runIndex-1)) value = (runs at:(runIndex+1)) value ifTrue:[
	"/ yes - merge them
	(runs at:(runIndex-1)) increaseLength:(runs at:(runIndex+1)) length.
	runs removeIndex:runIndex+1.
    ].

    runs removeIndex:runIndex.

    "Modified: 10.5.1996 / 18:23:41 / cg"
!

removeLast
    "Remove the Last element of the receiver.  If the receiver is empty,   
     provide an error notification."

    |returnValue run l|

    tally == 0 ifTrue:[
	^ self emptyCollectionError.
    ].

    run := runs first.
    returnValue := run value.
    (l := run length) == 1 ifTrue:[
	runs removeLast
    ] ifFalse:[
	run length:(l - 1)
    ].

    tally := tally - 1.
    ^returnValue

    "Modified: 11.5.1996 / 12:22:19 / cg"
! !

!CompressedOrderedCollection methodsFor:'converting'!

asOrderedCollection
    "Uncompress this collection."

    |newCollection|

    newCollection := OrderedCollection new:tally.
    newCollection addAll: self.
    ^newCollection

    "Modified: 11.5.1996 / 12:18:10 / cg"
! !

!CompressedOrderedCollection methodsFor:'enumerating'!

do:aBlock 
    "Evaluate aBlock with each of the receiver's elements as the 
    argument. "

    |runValue|

    runs do:[:aRun | 
	runValue := aRun value.
	aRun length timesRepeat:[aBlock value: runValue]
    ]

    "Modified: 11.5.1996 / 12:22:53 / cg"
!

reverseDo:aBlock
    "Evaluate aBlock with each of the receiver's elements as the argument, 
     starting with the last element and taking each in sequence up to the first."

    |runValue|

    runs reverseDo:[:aRun | 
	runValue := aRun value.
	aRun length timesRepeat:[aBlock value: runValue]
    ]

    "Modified: 11.5.1996 / 12:22:49 / cg"
! !

!CompressedOrderedCollection methodsFor:'initialization'!

initialize
    "To initialise a new instance. Install the ordered collection that will 
     hold the runs."

    runs := OrderedCollection new.
    tally := 0.

    "Modified: 10.5.1996 / 17:53:01 / cg"
! !

!CompressedOrderedCollection methodsFor:'printing & storing'!

storeOn:aStream 
    "Append to aStream an expression which, if evaluated, will generate   
     an object similar to the receiver."

    aStream nextPutAll: '(CompressedOrderedCollection new'.
    runs do:[:run |
	|l|

	aStream nextPutAll:' add:'; nextPutAll:run value storeString.

	(l := run length) ~~ 1 ifTrue:[
	    aStream nextPutAll:' withOccurrences:'; nextPutAll:l printString.
	].
	aStream nextPutAll:';'
    ].
    aStream nextPutAll: ' yourself)'

    "
     CompressedOrderedCollection new storeString 

     (CompressedOrderedCollection new 
	add:1;
	add:2;
	add:3 withOccurrences:15;
	add:4;
	yourself) storeString  
    "

    "Modified: 11.5.1996 / 12:26:19 / cg"
! !

!CompressedOrderedCollection methodsFor:'private'!

find:oldObject 
    "If I contain oldObject return its index, otherwise create an error   
    notifier. It will answer with the position of the very first element of  
    that value."

    |position|

    position := self find:oldObject ifAbsent:0.
    position ~~ 0 ifTrue:[
	^ position
    ].

    self errorValueNotFound:oldObject

    "
     |c|

     c := CompressedOrderedCollection new.
     c add:1; add:1; add:1; add:2; add:2; add:3; add:3; add:4; add:5.
     c find:2.

     c find:99
    "

    "Modified: 10.5.1996 / 13:39:58 / cg"
!

find:oldObject ifAbsent:exceptionBlock 
    "If I contain oldObject return its index, otherwise execute the 
     exception block. It will answer with the position of the very first 
     element of that value."

    | position |

    position := 1.
    runs do:[:run | 
	run value = oldObject ifTrue:[
	    ^ position
	].
	position := position + run length.
    ].
    ^ exceptionBlock value

    "Modified: 10.5.1996 / 13:54:25 / cg"
!

isEmpty
    "Am I empty or not. Returns a boolean"

    ^ tally == 0

    "Modified: 11.5.1996 / 12:23:32 / cg"
!

runIndexAndStartIndexForIndex:anIndex
    "given a logical index, find the index of that run and the logical index
     of the first item in that run."

    |position|

    position := 1.
    runs keysAndValuesDo:[:idx :run | 
	|runLen runLast|

	runLen := run length.
	anIndex >= position ifTrue:[
	    runLast := position + runLen - 1.
	    anIndex <= runLast ifTrue:[
		^ Array with:idx with:position 
	    ].
	].
	position := position + runLen
    ].

    ^ #(0 0)

    "Created: 10.5.1996 / 17:12:28 / cg"
    "Modified: 10.5.1996 / 17:13:42 / cg"
!

runIndexForIndex:anIndex
    "given a logical index, find the index of that run"

    ^ (self runIndexAndStartIndexForIndex:anIndex) at:1

    "Modified: 10.5.1996 / 17:13:45 / cg"
! !

!CompressedOrderedCollection methodsFor:'user interface'!

inspect
    "Reimplement so that they don't get an ordered collection inspector 
     which would get very confused."

    self basicInspect

    "Modified: 10.5.1996 / 18:24:43 / cg"
! !

!CompressedOrderedCollection class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Collections-CompressedOrderedCollection.st,v 1.20 2006/04/24 08:54:03 cg Exp $'
! !
