"
    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1992 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 
"

"{ Package: 'stx:goodies' }"

Collection subclass:#MultiDimensionalDictionary
	instanceVariableNames:'data dimensions'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Unordered'
!

!MultiDimensionalDictionary class methodsFor:'documentation'!

copyright
"
    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1992 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 
"
!

documentation
"
    this class has been extracted from a Manchester Goodies Benchmark package -
    it may be useful on its own.
    credit goes to: SW@cs.man.ac.uk

    MultiDimensionalDictionary:

    A general facility for an array with fixed dimensions.  
    The indices can be any objects; they need not be integers.


    claus:
        the original name was MultiDimensionalArray, 
        which was a misleading name for that thing;
        so, I renamed it.

    [author:]
        SW@cs.man.ac.uk

    [see also:]
        Array OrderedCollection SparseArray
        Dictionary
"
! !

!MultiDimensionalDictionary class methodsFor:'instance creation'!

new
    "Create a zero-dimensional array."

    ^self new: #()
!

new: anArray
    "Create a new multidimensional dictionary defined by anArray of subscripts."

    ^super new subscripts: anArray fill: nil

    "
     |a|

     a := MultiDimensionalDictionary new: (Array with: (1 to: 3) with: #(a b c)).
     a at:#(1 #a) put:'hello'.
     a at:#(1 #b). 
     a at:#(1 #a) 
    "
!

new: anArray withAll: element
    "Create a new multidimensional dictionary filled with 'element'."

    ^super new subscripts: anArray fill: element

    "
     |a|

     a := MultiDimensionalDictionary new: (Array with: (1 to: 3) with: #(a b c))
                                withAll:#foo.
     a at:#(1 #a) put:'hello'.
     a at:#(1 #b).  
     a at:#(1 #a)   
    "
! !

!MultiDimensionalDictionary methodsFor:'accessing'!

at: anArray
    "Get the element determined by subscript 'anArray'."

    | r |
    dimensions == anArray size ifFalse: [self error: 'Bad subscript'].
    r := data.
    anArray do: [:sub | r := r at: sub].
    ^r
!

at: anArray incrementBy: num
    "Increment by 'num' the element determined by subscript 'anArray'."

    | r |
    dimensions == anArray size ifFalse: [self error: 'Bad subscript'].
    dimensions = 0
	    ifTrue: [^data := data + num]
	    ifFalse: 
		    [r := data.
		    1 to: anArray size - 1 do: [:index | r := r at: (anArray at: index)].
		    ^r at: anArray last put: (r at: anArray last) + num]
!

at: anArray put: element
    "Set the element determined by subscript 'anArray'."

    | r |
    dimensions == anArray size ifFalse: [self error: 'Bad subscript'].
    dimensions = 0
	    ifTrue: [^data := element]
	    ifFalse: 
		    [r := data.
		    1 to: anArray size - 1 do: [:index | r := r at: (anArray at: index)].
		    ^r at: anArray last put: element]
! !

!MultiDimensionalDictionary methodsFor:'adding & removing'!

add: el
    self shouldNotImplement
!

remove: el ifAbsent: aBlock
    self shouldNotImplement
! !

!MultiDimensionalDictionary methodsFor:'enumerating'!

associationsDo: aBlock
    "Perform aBlock for all (index-element) combinations."

    self associationsDo: aBlock data: data sub: (Array new: dimensions) index: 1
!

do: aBlock
    "Perform aBlock for all elements."

    self associationsDo: [:assoc | aBlock value: assoc value]
! !

!MultiDimensionalDictionary methodsFor:'initialize-release'!

subscripts: anArray fill: default
    "Set the subscripts to those in anArray, and fill the array."

    dimensions := anArray size.
    data := self createFrom: anArray index: 1 fill: default
! !

!MultiDimensionalDictionary methodsFor:'printing'!

printOn: aStream 
    "Print the information in the array."

    self class printOn: aStream.
    aStream nextPutAll: ' ('.
    self
	    associationsDo: 
		    [:assoc | 
		    assoc printOn: aStream.
		    aStream space]
	    data: data
	    sub: (Array new: dimensions)
	    index: 1.
    aStream nextPut: $)
! !

!MultiDimensionalDictionary methodsFor:'private'!

associationsDo: aBlock data: info sub: sub index: index 
    index > dimensions
	ifTrue: 
	    [aBlock value: (Association key: sub value: info)]
	ifFalse: [info class == Array
		    ifTrue: [1 to: info size do: 
				[:ind | 
				sub at: index put: ind.
				self
					associationsDo: aBlock
					data: (info at: ind)
					sub: sub
					index: index + 1]]
		    ifFalse: [info
				associationsDo: 
				    [:assoc | 
				    sub at: index put: assoc key.
				    self
					associationsDo: aBlock
					data: assoc value
					sub: sub
					index: index + 1]]]
!

createFrom: anArray index: anInteger fill: default
    | sub r |
    anArray size < anInteger ifTrue: [^default].
    sub := anArray at: anInteger.
    r := sub = (1 to: sub size)
			    ifTrue: [Array new: sub size]
			    ifFalse: [Dictionary new].
    sub do: [:subscript | 
	r at: subscript 
	  put: (self createFrom: anArray index: anInteger + 1 fill: default)].
    ^r
! !

!MultiDimensionalDictionary class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Collections-MultiDimensionalArray.st,v 1.7 2003/05/07 10:03:45 cg Exp $'
! !
