"
 This class is not covered by or part of the ST/X licence.


 COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"
"{ Package: 'stx:goodies' }"

ArrayedCollection subclass:#SparseArray
	instanceVariableNames:'size defaultElement exceptionElements'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Sequenceable'
!

!SparseArray class methodsFor:'documentation'!

copyright
"
 This class is not covered by or part of the ST/X licence.


 COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"
!

documentation
"
    Instances of SparseArray behave like instances of Array in every respect
    but have a very different representation.  Rather than storing each element
    explicitly, SparseArrays have a default element and a list of exceptional
    elements (and their respective indices).  This is an efficient representation
    for arrays that are large and have very few elements that are not the
    default.  Contrast this class with RunArray, which is optimized for the case
    in which consecutive elements are likely to have the same value.
    Notice, that a non-sparsely filled SparseArray is not at all efficient spacewise.

    [Instance Variables:]
        size                <SmallInteger>      maximum number of elements

        defaultElement      <Object>            the value of ''most'' elements

        exceptionElements   <IdentityDictionary of (SmallInteger->Object)>
                                                list of the elements that are not the default, keyed
                                                 by index.

   [info:]
        NAME            SparseArray
        AUTHOR          unknown
        CONTRIBUTOR     ParcBench
        FUNCTION        Implements sparse arrays
        ST-VERSION      2.5
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE     1990
        SUMMARY Sparse arrays (from ParcPlace Newsletter Summer 1990).

    [author:]
        unknown (parcplace newsLetter)

    [see also:]
        Array OrderedCollection Dictionary

    Claus:
        Notice that a sparseArray which is filled more than about 30-40% 
        becomes less space efficient than a normal array.
"

!

examples
"
                                                                        [exBegin]
    |a|

    a := SparseArray new:1000000.
    a at:5 put:'hello'.
    a at:123456 put:'world'.
    1 to:10 do:[:i |
        Transcript show:i; show:' -> '; showCR:(a at:i) 
    ].
    Transcript show:5; show:' -> '; showCR:(a at:5).
    Transcript show:123455; show:' -> '; showCR:(a at:123455).
    Transcript show:123456; show:' -> '; showCR:(a at:123456).
    Transcript show:123457; show:' -> '; showCR:(a at:123457).
                                                                        [exBegin]

                                                                        [exBegin]
    |a|

    a := SparseArray new:1000000.
    a atAllPut:5.
    a at:123456 put:6.
    1 to:10 do:[:i |
        Transcript show:i; show:' -> '; showCR:(a at:i) 
    ].
    Transcript show:5; show:' -> '; showCR:(a at:5).
    Transcript show:123455; show:' -> '; showCR:(a at:123455).
    Transcript show:123456; show:' -> '; showCR:(a at:123456).
    Transcript show:123457; show:' -> '; showCR:(a at:123457).
                                                                        [exBegin]

"
! !

!SparseArray class methodsFor:'instance creation'!

new: size
    "Answer a new instance whose maximum subscript is size."

    ^self new: size withAll: nil

    "Modified: 9.5.1996 / 16:31:22 / cg"
!

new: size withAll: anObject 
    "Answer a new instance whose every element is equal to the argument, 
     anObject."

    ^ self basicNew setSize: size default: anObject

    "Modified: 9.5.1996 / 16:31:28 / cg"
! !

!SparseArray methodsFor:'accessing'!

at: index 
    "Answer the value of an indexable field in the receiver. Signal an error
    if the index is invalid."

    ^ exceptionElements at:index ifAbsent:[
	self checkIndex:index.
	defaultElement
    ]

    "Modified: 9.5.1996 / 16:31:45 / cg"
!

at: index put: value 
    "Store the argument value in the indexable field of the receiver indicated
    by index. Answer the value that was stored. Signal an error if the index
    is invalid."

    value = defaultElement ifTrue: [
	exceptionElements removeKey: index ifAbsent: []
    ] ifFalse:[
	self checkIndex: index.
	exceptionElements at: index put: value
    ].
    ^value

    "Modified: 9.5.1996 / 16:31:50 / cg"
!

atAllPut:value 
    "Store the argument value in all elements
     This is very quickly done here ..."

    defaultElement := value.
    exceptionElements := IdentityDictionary new.
    ^ value
!

size
    "Answer the number of indexable fields in the receiver."

    ^size

    "Modified: 9.5.1996 / 16:31:55 / cg"
! !

!SparseArray methodsFor:'adding'!

grow
    "The receiver becomes larger."
    "Growing simply means accepting larger subscripts."

    size := size + self growSize

    "Modified: 9.5.1996 / 16:32:00 / cg"
!

growToAtLeast: anInteger
    "The receiver becomes at least as large as anInteger."
    "Growing simply means accepting larger subscripts."

    anInteger <= size ifTrue: [^self].
    anInteger class = SmallInteger ifFalse: [
	self error: 'Size must be non-negative SmallInteger.'
    ].
    size := anInteger

    "Modified: 9.5.1996 / 16:32:06 / cg"
! !

!SparseArray methodsFor:'copying'!

copyEmpty: aSize
    "Answer a copy of the receiver that contains no elements."

    ^self class new: aSize withAll: defaultElement

    "Modified: 9.5.1996 / 16:32:11 / cg"
! !

!SparseArray methodsFor:'private'!

checkIndex: index 
    "Signal an error if the argument is an invalid index, that is,
    less than 1 or greater than the array's size."

    index isInteger ifFalse: [^self nonIntegerIndexError: index].
    (index >= 1 and: [index <= size]) ifFalse: [
	^self subscriptBoundsError: index
    ]

    "Modified: 9.5.1996 / 16:32:16 / cg"
!

defaultElement
     "return the default element"

    ^defaultElement

    "Modified: 9.5.1996 / 16:32:28 / cg"
!

setSize: anInteger default: default 
    "Initialize the receiver. Sent when creating new instances; answers the 
    receiver."
    "Indices must be SmallIntegers since we've chosen to use IdentityDictionary

    for the exceptionElements."

    (anInteger class = SmallInteger and: [anInteger >= 0]) ifFalse: [
	self error: 'Size must be non-negative SmallInteger.'
    ].
    size := anInteger.
    defaultElement := default.
    exceptionElements := IdentityDictionary new: 16

    "Modified: 9.5.1996 / 16:32:32 / cg"
! !

!SparseArray methodsFor:'testing'!

includes: anObject 
    "Answer whether anObject is one of the receiver's elements."
    "Overrided for efficiency."

    anObject = defaultElement ifTrue: [^true].
    ^ exceptionElements includes:anObject

    "Modified: 9.5.1996 / 16:32:39 / cg"
!

includesIdentical:anObject 
    "Answer whether anObject is one of the receiver's elements."
    "Overrided for efficiency."

    anObject == defaultElement ifTrue: [^true].
    ^ exceptionElements includesIdentical:anObject

!

occurrencesOf: anObject
    "Answer how many of the receiver's elements are equal to anObject."
    "Overrided for efficiency."

    anObject = defaultElement ifTrue: [^size - exceptionElements size].
    ^exceptionElements occurrencesOf: anObject

    "Modified: 9.5.1996 / 16:32:44 / cg"
! !

!SparseArray class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Collections-SparseArray.st,v 1.10 2001/11/16 10:10:10 cg Exp $'
! !
