"
 COPYRIGHT (c) 1996 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies' }"

AbstractMultidimensionalArray variableSubclass:#TwoDimensionalArray
	instanceVariableNames:'cols'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-MultiDimensional'
!

!TwoDimensionalArray class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1996 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    TwoDimensionalArray allows access to its elements by row/col.
    They are much like arrays, but have to be accessed via:
	atRow:row col:col
    and:
	atRow:row col:col put:element

    [see also:]
	ByteArray BooleanArray FloatArray DoubleArray Array

    [author:]
	Claus Gittinger
"
!

examples
"
                                                        [exBegin]
    |matrix|

    matrix := TwoDimensionalArray rows:4 cols:4.
    1 to:4 do:[:row |
        1 to:4 do:[:col |
            matrix atRow:row col:col put:(row @ col)
        ]
    ].
    matrix inspect
                                                        [exEnd]

                                                        [exBegin]
    |matrix|

    matrix := TwoDimensionalArray rows:4 cols:4.
    1 to:4 do:[:row |
        1 to:4 do:[:col |
            matrix atRow:row col:col put:(row @ col)
        ]
    ].
    Transcript showCR:(matrix rowAt:1).
    Transcript showCR:(matrix colAt:1).
    Transcript showCR:(matrix firstDiagonal).
    Transcript showCR:(matrix secondDiagonal).
                                                        [exEnd]
"
! !

!TwoDimensionalArray class methodsFor:'instance creation'!

rows:nRows cols:nCols
    "create and return a new TwoDimensionalArray holding nRows*nCols
     elements"

    ^ (super new:nRows*nCols) setCols:nCols
!

rows:nRows cols:nCols withAll:initialValue
    "create and return a new TwoDimensionalArray holding nRows*nCols elements"

    ^ (super new:nRows*nCols withAll:initialValue) setCols:nCols

    "Created: / 24-08-2010 / 14:31:20 / cg"
! !

!TwoDimensionalArray methodsFor:'accessing'!

_at:row at:col
    "return the element at row/col. Indexing starts at 1/1."

    ^ self at:(row-1*cols+col)
!

_at:row at:col put:anObject
    "store the argument, anObject at row/col. Indexing starts at 1/1."

    ^ self at:(row-1*cols+col) put:anObject
!

atRow:row col:col
    "return the element at row/col. Indexing starts at 1/1."

    ^ self at:(row-1*cols+col)
!

atRow:row col:col put:anObject
    "store the argument, anObject at row/col. Indexing starts at 1/1."

    ^ self at:(row-1*cols+col) put:anObject
!

colAt:col
    "retrieve a (copy of a) column"

    |nRows idx column|

    nRows := self rows.
    idx := col.

    column := Array new:nRows.
    1 to:nRows do:[:r |
	column at:r put:(self at:idx).
	idx := idx + cols.
    ].
    ^ column
!

firstDiagonal
    "retrieve a (copy of a) diagonal.
     Requires that the matrix is square."

    self assert:(self rows = self cols).
    ^ (1 to:self cols) collect:[:idx | self atRow:idx col:idx]

    "Created: / 14-08-2010 / 12:46:13 / cg"
!

rowAt:row
    "retrieve a (copy of a) row"

    |idx|

    idx := row-1*cols+1.
    ^ (Array new:cols) replaceFrom:1 to:cols with:self startingAt:idx
!

secondDiagonal
    "retrieve a (copy of a) diagonal.
     Requires that the matrix is square."

    self assert:(self rows = self cols).
    ^ (self cols to:1 by:-1) collect:[:idx | self atRow:(cols-idx+1) col:idx]

    "Created: / 14-08-2010 / 12:46:59 / cg"
! !

!TwoDimensionalArray methodsFor:'printing'!

displayString
    ^ self printString
!

printOn:aStream
    1 to: self rows do:[:row |
        aStream nextPutAll:'|'.
        1 to:self cols do:[:col |
            aStream space.
            (self atRow:row col:col) printOn:aStream.
        ].
        aStream nextPutAll:' |'; cr.
    ].
! !

!TwoDimensionalArray methodsFor:'private-initialization'!

setCols:numberOfCols
    "set the number of cols"

    cols := numberOfCols
! !

!TwoDimensionalArray methodsFor:'queries'!

cols
    "return the number of cols of the matrix"

    ^ cols
!

rows
    "return the number of rows of the matrix"

    ^ self basicSize // cols
!

size
    ^ self basicSize
! !

!TwoDimensionalArray class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Collections-TwoDArray.st,v 1.11 2010/08/24 13:40:46 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/Collections-TwoDArray.st,v 1.11 2010/08/24 13:40:46 cg Exp $'
! !
