"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies' }"

Block variableSubclass:#CurryingBlock
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Methods'
!

!CurryingBlock class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Curried functions are functions which, if invoked with less-than-expected
    arguments, return a function which expects the remaining arguments.

    For example, given a function add(a,b),
    which expects two arguments, and returns their sum,
    a curried add(N) would return a function, which expects one argument (b)
    and computes add(N+b).
    I.e.
        (add(a)) (b)  -> add(a,b)

    Here, a CurryingBlock, if evaluated with less-than-required arguments,
    returns a block which expects the remaining args, and provides the initially
    provided arguments.
    I.e.
        ( [:a :b | a + b] beCurryingBlock ) value:10
    returns a block, which expects 1 argument (b) and
    adds it to 10.

    This is an experimental (academic ?) goody for demonstration purposes.

    [author:]
        Claus Gittinger

    [see also:]
"
!

examples
"
                                                                    [exBegin]
    |b blockAdding1 blockAdding10|

    b := [:a :b | a + b] beCurryingBlock.
    blockAdding1 := b value:1.
    blockAdding10 := b value:10.

    blockAdding1 value:2.      
    blockAdding10 value:2.
    b value:10 value:30.
    (b value:10) value:30.
                                                                    [exEnd]

                                                                    [exBegin]
     |b b1 b2 b3|

     b := [:a :b :c | a + b + c] beCurryingBlock.
     b numArgs.    
     b value:1 value:2 value:3.

     b1 := b value:10.
     b1 numArgs.         
     b1 value:2 value:3.    

     b2 := b value:10 value:20.
     b2 numArgs.
     b2 value:3.

     b3 := b1 value:20.
     b3 numArgs.
     b3 value:3.
                                                                    [exEnd]
"
! !

!CurryingBlock methodsFor:'error handling'!

wrongNumberOfArguments:numArgsGiven
    |nMissing argsGiven|

    numArgsGiven < self numArgs ifFalse:[
        "/ more args given than requested
        ^ super wrongNumberOfArguments:numArgsGiven
    ].

    "/ return a block, which evaluates mySelf with

    argsGiven := thisContext sender args.
    nMissing := self numArgs - numArgsGiven.
    nMissing == 1 ifTrue:[
        ^ [:missing | self valueWithArguments:(argsGiven copyWith:missing) ].
    ].
    nMissing == 2 ifTrue:[
        ^ [:m1 :m2 | self valueWithArguments:((argsGiven copyWith:m1) copyWith:m2) ] changeClassTo:CurryingBlock.
    ].

    self halt:'only currying upto 2 missing args (for now)'.

    "
     [:a :b | a + b] beCurryingBlock   
     ( [:a :b | a + b] beCurryingBlock ) value:1
     ( [:a :b | a + b] beCurryingBlock ) value
    "

    "|f|

     f := [:a :b | a + b] beCurryingBlock.
     f value:1.      
     (f value:1) value:3. 
     (f value) value:1.  
     ((f value) value:1) value:3.   
    "
! !

!CurryingBlock methodsFor:'printing'!

printBlockBracketsOn:aStream
    aStream nextPutAll:'[] (currying)'
! !

!CurryingBlock class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/CurryingBlock.st,v 1.5 2010/05/10 10:58:15 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/CurryingBlock.st,v 1.5 2010/05/10 10:58:15 cg Exp $'
! !
