"{ Package: 'stx:goodies' }"

!Date class methodsFor:'general queries'!

easterOfYear:yearInteger
    "return the date representing easter-sunday of a given year"

    |nMonth nDay nMoon nEpact nSunday nGold nCent nCorx nCorz|

    "/ The Golden Number of the year in the 19 year Metonic Cycle:
    nGold := (yearInteger \\ 19) + 1. 
    "/ Calculate the Century: 
    nCent := (yearInteger // 100) + 1. 
    "/ Number of years in which leap year was dropped in order... 
    "/ to keep in step with the sun: 
    nCorx := ((3 * nCent) // 4) - 12. 
    "/ Special correction to syncronize Easter with moon's orbit: 
    nCorz := ((8 * nCent + 5) // 25) - 5. 
    "/ Find Sunday:  
    nSunday := ((5 * yearInteger) // 4) - nCorx - 10. 
    "/ Set Epact - specifies occurrence of full moon:  
    nEpact := (11 * nGold + 20 + nCorz - nCorx) \\ 30. 
    nEpact < 0 ifTrue:[ 
	nEpact := nEpact + 30.
    ].
    (((nEpact = 25) and:[ (nGold > 11) ]) or:[ (nEpact = 24) ]) ifTrue:[ 
	nEpact := nEpact + 1.
    ].
    "/ Find Full Moon: 
    nMoon := 44 - nEpact. 
    nMoon < 21 ifTrue:[ 
	nMoon := nMoon + 30.
    ].
    "/ Advance to Sunday: 
    nMoon := nMoon + 7 - ((nSunday + nMoon) \\ 7). 
    nMoon > 31 ifTrue:[ 
	nMonth := 4. 
	nDay   := nMoon - 31. 
    ] ifFalse:[
	nMonth := 3. 
	nDay   := nMoon. 
    ].
    ^ self newDay:nDay month:nMonth year:yearInteger 

    "
     Date easterOfYear:1992  
     Date easterOfYear:1994  
     Date easterOfYear:1900  
     Date easterOfYear:2002 
    "
! !
