"{ Package: 'stx:goodies' }"

'From Smalltalk/X, Version:2.10.7 on 21-oct-1995 at 19:51:19'                   !

FileStream subclass:#LockedFileStream
	 instanceVariableNames:''
	 classVariableNames:'PermanentlyLockedErrorSignal'
	 poolDictionaries:''
	 category:'Streams-External'
!

!LockedFileStream class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

version
"
$Header: /cvs/stx/stx/goodies/LockedFileStream.st,v 1.3 2000/03/02 14:14:54 cg Exp $
"

    "Created: 21.10.1995 / 19:41:02 / cg"
!

documentation
"
    Like a normal fileStream, but automatically creates a lockfile (pathname.lck)
    to prevent simultaneous access.
    This uses files as locking mechanism and should work on all Unix systems;
    however, it is not very efficient.

    NO WARRANTY: this is an experimental goody. Use at your own risk.
"

    "Created: 21.10.1995 / 19:41:02 / cg"
    "Modified: 21.10.1995 / 19:47:24 / cg"
!

history

    "Created: 21.10.1995 / 19:41:03 / cg"
    "Modified: 21.10.1995 / 19:47:43 / cg"
! !

!LockedFileStream class methodsFor:'initialization'!

initialize
    PermanentlyLockedErrorSignal := StreamErrorSignal newSignalMayProceed:true.
    PermanentlyLockedErrorSignal nameClass:self message:#permanentlyLockedErrorSignal.
    PermanentlyLockedErrorSignal notifierString:'file seems permanently locked '.

    "Created: 21.10.1995 / 19:49:00 / cg"
! !

!LockedFileStream class methodsFor:'Signal constants'!

permanentlyLockedErrorSignal
    "return the signal raised when a lockedFile seems to be permanently locked
     (i.e. the locker forgot to release the lock).
     In this case, you should manually remove the lockfile (filename.lck)"

    ^ PermanentlyLockedErrorSignal

    "Created: 21.10.1995 / 19:50:16 / cg"
! !

!LockedFileStream class methodsFor:'examples'!

examples
"
    without locks (corrupted contents):

	|p1 p2 s1 s2|

	p1 := [
		    1 to:100 do:[:n |
			s1 := FileStream appendingOldFileNamed:'testfile'.
			s1 nextPutAll:'from '.
			s1 synchronizeOutput.
			(Delay forSeconds:0.1) wait.
			s1 nextPutAll:' process1; ' , n printString.
			(Delay forSeconds:0.1) wait.
			s1 cr.
			s1 close.
			(Delay forSeconds:0.1) wait.
		    ].
	      ] forkAt:4.

	p2 := [
		    1 to:100 do:[:n |
			s2 := FileStream appendingOldFileNamed:'testfile'.
			s2 nextPutAll:'from '.
			s2 synchronizeOutput.
			(Delay forSeconds:0.1) wait.
			s2 nextPutAll:' process2; ' , n printString.
			(Delay forSeconds:0.1) wait.
			s2 cr.
			s2 close.
			(Delay forSeconds:0.1) wait.
		    ].
	      ] forkAt:4.


    with locks (correct contents):

	|p1 p2 s1 s2|

	p1 := [
		    1 to:100 do:[:n |
			s1 := LockedFileStream appendingOldFileNamed:'testfile'.
			s1 nextPutAll:'from '.
			s1 synchronizeOutput.
			(Delay forSeconds:0.1) wait.
			s1 nextPutAll:' process1; ' , n printString.
			(Delay forSeconds:0.1) wait.
			s1 cr.
			s1 close.
			(Delay forSeconds:0.1) wait.
		    ].
	      ] forkAt:4.

	p2 := [
		    1 to:100 do:[:n |
			s2 := LockedFileStream appendingOldFileNamed:'testfile'.
			s2 nextPutAll:'from '.
			s2 synchronizeOutput.
			(Delay forSeconds:0.1) wait.
			s2 nextPutAll:' process2; ' , n printString.
			(Delay forSeconds:0.1) wait.
			s2 cr.
			s2 close.
			(Delay forSeconds:0.1) wait.
		    ].
	      ] forkAt:4.
"

    "Created: 21.10.1995 / 19:23:17 / cg"
    "Modified: 21.10.1995 / 19:44:24 / cg"
! !

!LockedFileStream methodsFor:'private'!

openWithMode:openmode
    |counter num lockName haveLock myLockName|

    mode ~~ #readonly ifTrue:[
	num := Processor activeProcess id.

	[myLockName isNil] whileTrue:[
	    myLockName := '.'
			  , OperatingSystem getProcessId printString , 
			  '_' , num printString.
	    (myLockName asFilename exists) ifTrue:[
		myLockName := nil.
		num := num + 1
	    ]
	].
	myLockName asFilename writeStream close.

	lockName := pathName , '.lck'.
	haveLock := false.

	counter := 0.
	[haveLock] whileFalse:[
	    [lockName asFilename exists] whileTrue:[
		counter > 100 ifTrue:[
		    PermanentlyLockedErrorSignal
			raiseRequestWith:pathName , ' seems locked. please remove lock manually'.
		    ^ nil.
		].
		(Delay forSeconds:0.5) wait.
		counter := counter + 1.
	    ].
	    (OperatingSystem linkFile:myLockName to:lockName) ifTrue:[
		haveLock := true.
		OperatingSystem removeFile:myLockName 
	    ].
	].
    ].
    ^ super openWithMode:openmode

    "Created: 21.10.1995 / 19:16:33 / cg"
    "Modified: 21.10.1995 / 19:50:51 / cg"
!

closeFile
    |lockPath|

    mode ~~ #readonly ifTrue:[
	pathName notNil ifTrue:[
	    lockPath := pathName , '.lck'.
	]
    ].

    super closeFile.
    lockPath notNil ifTrue:[
	OperatingSystem removeFile:lockPath
    ].

    "Created: 21.10.1995 / 19:17:17 / cg"
    "Modified: 21.10.1995 / 19:30:42 / cg"
! !

LockedFileStream initialize!
