"
    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1992 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 

    Copyright (c) 1989, ParcPlace Systems, Inc.  All rights reserved.  The
    contents of this file may be copied and used only in accord with the
    provisions of the ParcBench User Agreement.

"

"{ Package: 'stx:goodies' }"

IdentityDictionary subclass:#Bindings
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Unordered'
!

QueryWithoutDefault subclass:#BindingRequest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Bindings
!

Error subclass:#NoBindingError
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Bindings
!

Bindings comment:'Instances of Bindings contain mappings of Symbols to values
which are used as environments in which to evaluate blocks.
The mapping is accessed by sending the #binding message a
Symbol.

The implementation here uses exception-handling, but was
inspired by a context-searching implementation shown in
Peter Deutsch''s Aug ''81 BYTE magazine article.
'
!

!Bindings class methodsFor:'documentation'!

copyright
"
    COPYRIGHT.
    The above file is a Manchester Goodie protected by copyright.
    These conditions are imposed on the whole Goodie, and on any significant
    part of it which is separately transmitted or stored:
	   * You must ensure that every copy includes this notice, and that
	     source and author(s) of the material are acknowledged.
	   * These conditions must be imposed on anyone who receives a copy.
	   * The material shall not be used for commercial gain without the prior
	     written consent of the author(s).
    Further information on the copyright conditions may be obtained by
    sending electronic mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: copyright
    or by writing to The Smalltalk Goodies Library Manager, Dept of
    Computer Science, The University, Manchester M13 9PL, UK

    (C) Copyright 1992 University of Manchester
    For more information about the Manchester Goodies Library (from which 
    this file was distributed) send e-mail:
	   To: goodies-lib@cs.man.ac.uk
	   Subject: help 

    Copyright (c) 1989, ParcPlace Systems, Inc.  All rights reserved.  The
    contents of this file may be copied and used only in accord with the
    provisions of the ParcBench User Agreement.

"
!

documentation
"
    [info:]
	NAME            binding
	AUTHOR          Spring 1990 ParcPlace Newsletter
	CONTRIBUTOR ParcBench
	FUNCTION        non-local variables file-in
	ST-VERSION      
	PREREQUISITES   
	CONFLICTS       
	DISTRIBUTION    world
	VERSION         1
	DATE     1991

    Instances of Bindings contain mappings of Symbols to values
    which are used as environments in which to evaluate blocks.
    The mapping is accessed by sending the #for: or #for:ifAbsent:
    message to the Binding class 
    (or #binding to a Symbol, if the additional Symbol add-ons are loaded).

    The implementation here uses exception-handling, but was
    inspired by a context-searching implementation shown in
    Peter Deutsch''s Aug ''81 BYTE magazine article.

    claus:
	Bindings as implemented here resemble the lisp way of dynamic
	variable scoping. A neat example.
	The implementation here may involve a bit too 
	much overhead and be too slow for heavy use, though.

	I modified the code to do things locally (i.e. not defining
	additional methods in Symbol). The original used <aSymbol> binding
	to access a variables value - changed it to use (Bindings at:<aSymbol>).

	Notice, ST/X already contains a mechanism, which is resembling
	bindings: the QuerySignal mechanism, which can also be used to
	ask for some value. However, QuerySignals are somewhat more general,
	in that they are not only usable to ask for a value, but also
	to provide ignorable (lazy) user notifications.



    [note:]
	This could have been implemented without any need to
	subclass Dictionary - since the key- and value are already available
	in the #with:boundTo:do: method, why not return the value right from
	the handler ?
	Here is an alternative implementation:
	    with:aValue boundTo:aSymbol do:aBlock
		^ BindingRequestSignal
		    handle:[:ex |
				ex parameter == aSymbol ifTrue:[
				    ex proceedWith:aValue
				].
				ex reject
			   ]
		    do: aBlock

    [author:]
	unknown (parcbench newsLetter)

    [see also:]
	Context Signal Exception
	(``Exception handling and signals'': programming/exceptions.html)
"
!

examples
"
  define a binding for the variable `foo'
  and evaluate a block. There, get `foos's binding:
									[exBegin]
    Bindings
	with:'hello' 
	boundTo:#foo 
	do:[
	    Transcript showCR:'foo is ' , (Bindings for:#foo)
	]
									[exEnd]

  a nonExisting binding raises an exception:
									[exBegin]
    Bindings
	with:'hello' 
	boundTo:#foo 
	do:[
	    Transcript showCR:'bar is ' , (Bindings for:#bar)
	]
									[exEnd]

  a nonExisting binding with fallBack value:
									[exBegin]
    Bindings
	with:'hello' 
	boundTo:#foo 
	do:[
	    Transcript showCR:'bar is ' , (Bindings for:#bar ifAbsent:['none'])
	]
									[exEnd]

  binding environments can nest:
									[exBegin]
    Bindings
	with:'hello' 
	boundTo:#foo 
	do:[
	    Transcript showCR:'foo is ' , (Bindings for:#foo).
	    Bindings
		with:'world' 
		boundTo:#foo 
		do:[
		    Transcript showCR:'foo is ' , (Bindings for:#foo)
		].
	    Transcript showCR:'foo is ' , (Bindings for:#foo).
	]
									[exEnd]

  nested bindings, again:
									[exBegin]
    (Bindings
	with: #foo -> 1
	with: #bar -> 'hello'
	with: #ugh -> (99 @ 23)
    ) applyIn: [
	Bindings
	   with:'goodbye'
	   boundTo:#bar
	   do:[
		Transcript showCR:(
		    Array
			with: (Bindings for:#ugh)
			with: (Bindings for:#foo)
			with: (Bindings for:#bar))
	   ]
    ]
									[exEnd]
"
! !

!Bindings class methodsFor:'Signal constants'!

bindingRequestSignal
    "Answer the Signal which indicates a request for a binding value.
     Provided here for backward compatibity with instance based signals.   
     By convention, the signal is raised with the binding symbol as 
     a parameter."

    ^ BindingRequest

    "Modified: 16.1.1997 / 13:23:34 / cg"
!

noBindingSignal
    "Answer the Signal which indicates that no binding is available.
     Provided here for backward compatibity with instance based signals.   
     By convention, the signal is raised with the original binding symbol 
     as a parameter."

    ^ NoBindingError
! !

!Bindings class methodsFor:'class initialization'!

initialize
    "set up the classes signals"

    BindingRequest notifierString: 'Value requested for '.
    NoBindingError notifierString: 'No binding available for '

    "
     Bindings initialize
    "

    "Modified: 16.1.1997 / 13:01:24 / cg"
! !

!Bindings class methodsFor:'evaluation with a binding'!

bind:aValue to:aSymbol in:aBlock
    "evaluate aBlock, with a aValue bound to aSymbol within that block
     and all contexts below that evaluation."

    ^ (self with:(aSymbol -> aValue)) applyIn: aBlock

    "
     Bindings 
        bind:'hello' 
        to:#foo
        in:[
            Transcript show:'1: the binding of foo is ';
                       showCR:(Bindings for:#foo).
            Bindings 
                bind:'world' 
                to:#foo
                in:[
                    Transcript show:'2: the binding of foo is ';
                               showCR:(Bindings for:#foo).
                ]
     ]
    "

    "Created: 9.5.1996 / 16:38:16 / cg"
    "Modified: 30.1.1997 / 15:09:57 / cg"
!

with:aValue boundTo:aSymbol do:aBlock
    "evaluate aBlock, with a aValue bound to aSymbol within that block
     and all contexts below that evaluation."

    ^ (self with:(aSymbol -> aValue)) applyIn: aBlock

    "
     Bindings with:'hello' boundTo:#foo
	do:[
	    Transcript show:'the binding of foo is ';
		       showCR:(Bindings for:#foo).
	    Bindings with:'world' boundTo:#foo
	    do:[
		Transcript show:'the binding of foo is ';
			   showCR:(Bindings for:#foo).
	    ]
     ]
    "

    "Created: 9.5.1996 / 16:38:16 / cg"
    "Modified: 30.1.1997 / 15:09:57 / cg"
! !

!Bindings class methodsFor:'getting a binding'!

bindingFor:aSymbol
    "Retrieve the binding for a symbol in the current binding environment.
     Raises an exception, if no such binding is available."

    ^ self bindingFor:aSymbol ifAbsent:[ NoBindingError raiseRequestWith:aSymbol ]
!

bindingFor:aSymbol ifAbsent:exceptionBlock
    "Retrieve the binding for a symbol in the current binding environment.
     Returns the value of exceptionBlock, if no such binding is available."

    ^ NoHandlerError
        handle:
            [:ex |

            "noHandler exceptions have the original exception as its
             parameter.  If that (unhandled) exception is a bindingRequest,
             then this means there was no binding (and so raise a noBinding
             signal).  Otherwise, let someone else deal with the lack
             of handler (most likely, there is no handler for noBindingSignal)."

            (BindingRequest accepts:(ex parameter signal)) ifTrue:[ 
                ex returnDoing:exceptionBlock
            ] ifFalse:[
                ex reject
            ]
        ] do: [
             "Pass the receiver upward to any handler which might
              have a binding."

            BindingRequest raiseRequestWith:aSymbol
        ]

    "Created: 16.1.1997 / 13:09:08 / cg"
    "Modified: 16.1.1997 / 13:09:26 / cg"
!

for:aSymbol
    "Retrieve the binding for a symbol in the current binding environment.
     Raises an exception, if no such binding is available."

    ^ self bindingFor:aSymbol 
!

for:aSymbol ifAbsent:exceptionBlock
    "Retrieve the binding for a symbol in the current binding environment.
     Returns the value of exceptionBlock, if no such binding is available."

    ^ self bindingFor:aSymbol ifAbsent:exceptionBlock
! !

!Bindings methodsFor:'binding'!

applyIn:aBlock 
    "Evaluate the block using the receiver as dictionary of symbol 
     bindings."

    "/ catch binding-queries, and return a value if there is a binding.
    "/ if there is no binding, let outer binding handlers look for it.
    "/ The outermost handlers reject will lead to an exception which
    "/ is visible to the outside world ...

    ^ BindingRequest
        handle:[:ex |
                    ex proceedWith:(self at:(ex key) ifAbsent: [ex reject])
               ]
        do: aBlock

    "Modified: 16.1.1997 / 13:07:55 / cg"
! !

!Bindings::BindingRequest methodsFor:'accessing'!

key
    ^ parameter
! !

!Bindings::NoBindingError class methodsFor:'queries'!

mayProceed
    ^ true
! !

!Bindings class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Misc-Bindings.st,v 1.12 2002/03/01 15:28:44 cg Exp $'
! !
Bindings initialize!
