"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"


"{ Package: 'stx:goodies' }"

BitBlt subclass:#Pen
	instanceVariableNames:'locationX locationY direction isDown dX dY'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-Support'
!

!Pen class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

!

documentation
"
    This class was added as a goody for playing and for ST-80rel2.x compatibility.
    Notice:
	This class may be no longer present in newer ST-80 releases

    also notice, that ST/X does not use or need Pen in any way.
    This is a demo class - no WARRANTY whatsoever.

    written spring/summer 89 by claus

    [see also:]
	BitBlt Commander
	GraphicsContext

    [author:]
	Claus Gittinger

    [start with:]
	PenDemo open
"
!

examples
"
  logo like drawing:
                                                                [exBegin]
    |v bic|

    v := (View new extent:200@200) openAndWait.

    bic := Pen in:v.
    bic defaultNib:4.
    bic fillColor:Color red.
    bic down.

    bic go:50.
    bic turn:120.
    bic go:50.
    bic turn:120.
    bic go:50.
                                                                [exEnd]

  draws a spiral:
                                                                [exBegin]
    |v bic|

    v := (View new extent:200@200) openAndWait.

    bic := Pen in:v.

    bic defaultNib:4.
    bic down.
    1 to:50 do:[:i | 
        bic go:i*4. bic turn:89.
    ]
                                                                [exEnd]

  draws a mandala on the rootWindow (nostalgia):
  Sorry windows users: only works under X
                                                                [exBegin]
    |bic|

    Screen current rootView clear.
    bic := Pen in:Screen current rootView.
    bic mandala:50 radius:400
                                                                [exEnd]
"
! !

!Pen methodsFor:'accessing'!

direction
    "return the pens current direction (in degrees)"

    ^ direction

    "Modified: 16.5.1996 / 14:56:50 / cg"
!

location
    "return the pens current location"

    ^ locationX @ locationY

    "Modified: 16.5.1996 / 14:57:01 / cg"
! !

!Pen methodsFor:'direction'!

direction:aNumber
    "set the direction in degrees; 0 is east; 90 is south"

    |rads i|

    direction := aNumber.
    aNumber >= 360 ifTrue:[
	direction := aNumber - (((aNumber // 360)) * 360)
    ] ifFalse:[
	aNumber < 0 ifTrue:[
	    direction := aNumber - (((aNumber // 360)) * 360).
	    direction := direction + 360
	].
    ].

"/    direction = (i := direction asInteger) ifTrue:[
"/        direction := i
"/    ].

    "avoid rounding errors and allow integer arithmetic for common cases"
    direction = 0 ifTrue:[
	dX := 1. dY := 0
    ] ifFalse:[
	direction = 90 ifTrue:[
	    dX := 0. dY := 1
	] ifFalse: [
	    direction = 180 ifTrue:[
		dX := -1. dY := 0
	    ] ifFalse: [
		direction = 270 ifTrue:[
		    dX := 0. dY := -1
		] ifFalse: [
		    rads := direction degreesToRadians.
		    dX := rads cos.
		    dY := rads sin
		]
	    ]
	]
    ]

    "Modified: 17.8.1996 / 11:33:13 / cg"
!

east
    "set the direction to east"

    dX := 1.
    dY := 0.
    direction := 0.

    "Modified: 16.8.1996 / 23:02:06 / cg"
!

north
    "set the direction to north"

    dX := 0.
    dY := -1.
    direction := 270.

    "Modified: 16.8.1996 / 23:02:12 / cg"
!

south
    "set the direction to south"

    dX := 0.
    dY := 1.
    direction := 90.

    "Modified: 16.8.1996 / 23:02:16 / cg"
!

turn:aNumber
    "turn clockwise aNumber degrees"

    self direction:(direction + aNumber)

    "Modified: 16.5.1996 / 15:00:04 / cg"
!

west
    "set the direction to west"

    dX := -1.
    dY := 0.
    direction := 180.

    "Modified: 16.8.1996 / 23:02:22 / cg"
! !

!Pen methodsFor:'geometric designs'!

cCurve:level side:sideLength
    "draw a c-curve with the specified level and side length"

    level == 0 ifTrue:[^ self go:sideLength].
    self cCurve:level - 1 side:sideLength. self turn:90.
    self cCurve:level - 1 side:sideLength. self turn:-90

    "(Pen new) turn:-90; cCurve:10 side:4"
!

dragon:n
    "draw a dragon curve with order n"

    (n == 0) ifTrue:[
	self go:10
    ] ifFalse:[
	n > 0 ifTrue:[
	    self dragon: n - 1.
	    self turn:90.
	    self dragon:1 - n
	] ifFalse:[
	    self dragon: -1 - n.
	    self turn: -90.
	    self dragon: 1 + n
	]
    ]
    "(Pen new) dragon:10"

    "Modified: 16.5.1996 / 14:57:20 / cg"
!

hilbert:n side:s
    "draw a hilbert curve with order n and sideLength s"

    |a m negA negM|

    n == 0 ifTrue:[
	self turn:180
    ] ifFalse: [
	n > 0 ifTrue:[
	    a := 90.
	    negA := -90.
	    m := n - 1
	] ifFalse:[
	    a := -90.
	    negA := 90.
	    m := n + 1
	].
	negM := 0 - m.
	self turn:a.
	self hilbert:negM side:s.
	self turn:a.
	self go:s.
	self hilbert:m side:s.
	self turn:negA.
	self go:s.
	self turn:negA.
	self hilbert:m side:s.
	self go:s.
	self turn:a.
	self hilbert:negM side:s.
	self turn:a
    ]
    "(Pen new) hilbert:5 side:15"

    "Modified: 16.5.1996 / 16:16:26 / cg"
!

koch:n side:s
    "draw a koch curve with order n and sideLength s"

    n == 0 ifTrue:[
	self go:s
    ] ifFalse: [
	self koch:(n - 1) side:s.
	self turn:-45.
	self koch:(n - 1) side:s.
	self turn:90.
	self koch:(n - 1) side:s.
	self turn:-45.
	self koch:(n - 1) side:s
    ]
    "(Pen new) koch:5 side:4"

    "Modified: 16.5.1996 / 14:57:45 / cg"
!

mandala:n radius:r
    "draw a mandala curve with n segments and radius of r"

    |px py points deltaAngle angle|

    deltaAngle := 360.0 / n.
    angle := 0.0.
    points := Array new:n.
    1 to:n do:[:index |
	px := (angle degreesToRadians cos * r) asInteger + locationX.
	py := (angle degreesToRadians sin * r) asInteger + locationY.
	points at:index put:(px @ py).
	angle := angle + deltaAngle
    ].
    1 to:n do:[:thisIndex |
	thisIndex to:n do:[:otherIndex |
	    self place:(points at:thisIndex).
	    self goto:(points at:otherIndex)
	]
    ]
    "(Pen new) mandala:20 radius:300"

    "Modified: 16.5.1996 / 14:58:14 / cg"
!

pythagoras:n side:s
    "draw a pythagoras curve with order n sideLength of s"

    |a lX lY d|

    a := (s*s / 2) sqrt.

    self go:s.
    self turn:-45.
    n > 1 ifTrue:[
	self pythagoras:(n-1) side:a.
    ].
    self turn:135; go:s.

    lX := locationX. lY := locationY. d := direction.
    self up; turn:-135; go:a.
    self turn:90; down.
    n > 1 ifTrue:[
	self pythagoras:(n-1) side:a.
    ].
    locationX := lX. locationY := lY. direction := d.

    self turn:90.
    self go:s.
    self turn:90.
    self go:s.
    self turn:90.

    "/ at original location & direction again ...

    "Modified: 16.5.1996 / 16:08:06 / cg"
!

spiral:n angle:a
    "draw a spiral curve with order n sideLength of s"

    1 to:n do:[:i |
	self go:i.
	self turn:a
    ]

    "(Pen new) spiral:300 angle:89"
    "(Pen new) spiral:300 angle:121"
    "(Pen new) spiral:300 angle:59"

    "Modified: 16.5.1996 / 16:16:12 / cg"
! !

!Pen methodsFor:'initialization'!

initialize
    super initialize.
    self defaultNib:1.
    self home.
    self east.
    self down
! !

!Pen methodsFor:'moving'!

go:distance
    "move the pen by given distance, in its current direction"

    |newX newY|

    newX := (dX * distance) + locationX.
    newY := (dY * distance) + locationY.

    isDown ifTrue:[
	self drawFromX:locationX 
		     y:locationY
		   toX:newX 
		     y:newY.
    ].

    locationX := newX.
    locationY := newY

    "Modified: 14.4.1997 / 11:22:58 / cg"
!

goto:newLocation
    "move the pen to some new location"

    |newX newY|

    newX := newLocation x.
    newY := newLocation y.

    isDown ifTrue:[
	self drawFromX:locationX
		     y:locationY
		   toX:newX
		     y:newY
    ].
    locationX := newX.
    locationY := newY

    "Modified: 14.4.1997 / 11:23:12 / cg"
!

home
    "place the pen at the center"

    self place:(destForm extent // 2)

    "Modified: 16.5.1996 / 15:51:59 / cg"
!

place:aPoint
    "place the pen at some position"

    locationX := aPoint x.
    locationY := aPoint y

    "Modified: 16.5.1996 / 15:00:26 / cg"
! !

!Pen methodsFor:'pen up/down'!

down
    "lower the pen - it will draw when moved"

    isDown := true

    "Modified: 16.5.1996 / 15:01:37 / cg"
!

up
    "raise the pen - it will not draw when moved"

    isDown := false

    "Modified: 16.5.1996 / 15:01:45 / cg"
! !

!Pen class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Pen.st,v 1.16 2004/02/19 14:23:21 cg Exp $'
! !
