"{ Package: 'stx:goodies' }"

SimpleView subclass:#ScreenSaver
	instanceVariableNames:'keyboardForm timedBlock random prevX prevY'
	classVariableNames:''
	poolDictionaries:''
	category:'demos-ScreenSaver'
!

!ScreenSaver class methodsFor:'documentation'!

documentation
"
    a simple full-screen view example
    simply blackens the screen and goes away on key- or buttonPress.

    If a bitmap called 'keyboard.xbm' exists (in the bitmaps directory),
    that one is shown at random plcaes on the screen 
    (much like the good-old Xerox screensavers ...)

    [author:]
	Claus Gittinger

    [see also:]
	LightInTheDark LightInTheDark2

    [start with:]
	ScreenSaver open
"
! !

!ScreenSaver class methodsFor:'startup'!

open
    ^ self new open

    "
     ScreenSaver new open
     ScreenSaver open
    "
! !

!ScreenSaver methodsFor:'cleanup'!

destroy
    "stop any background timeout activities"

    Processor removeTimedBlock:timedBlock.
    super destroy

    "Modified: 17.5.1996 / 11:31:23 / cg"
! !

!ScreenSaver methodsFor:'events'!

buttonPress:button x:x y:y
    "close down, when a button is pressed"

    self destroy

    "Modified: 17.5.1996 / 11:31:36 / cg"
!

keyPress:key x:x y:y
    "close down, when a key is pressed"

    self destroy

    "Modified: 17.5.1996 / 11:31:43 / cg"
!

mapped
    "get focus when visible"

    super mapped.
    device setInputFocusTo:drawableId.
! !

!ScreenSaver methodsFor:'initialization'!

initEvents
    super initEvents.
    self saveUnder:true

    "Modified: 17.5.1996 / 11:32:14 / cg"
!

initialize
    "setup for an empty (black) cursor; look for a keyboard.xbm image"

    |cursor|

    super initialize.

    self borderWidth:0; level:0; viewBackground:Black.

    "a black cursor"
    self cursor:(Cursor extent:(16 @ 16)
		     fromArray:#(0 0 0 0 0 0 0 0
				 0 0 0 0 0 0 0 0
				 0 0 0 0 0 0 0 0
				 0 0 0 0 0 0 0 0)
			offset:(0 @ 0)).

    self origin:(0 @ 0)
	 extent:(Display width) @ (Display height).

    keyboardForm := Image fromFile:'keyboard.xbm'.
    timedBlock := [self cycle].
    Processor addTimedBlock:[self cycleAndCollectGarbage] afterSeconds:0.5.
    random := Random new

    "ScreenSaver new open"

    "Modified: 17.5.1996 / 11:33:13 / cg"
! !

!ScreenSaver methodsFor:'private'!

cycle
    "show keyboard-form (if present) at a random place, trigger another cycle"

    |xpos ypos|

    keyboardForm notNil ifTrue:[
	prevX notNil ifTrue:[
	    self paint:Black on:White.
	    self fillRectangleX:prevX y:prevY
			  width:(keyboardForm width)
			 height:(keyboardForm height)
	].

	xpos := (random next * (device width - keyboardForm width)) 
							    rounded asInteger.
	ypos := (random next * (device height - keyboardForm height))
							    rounded asInteger.
	self displayOpaqueForm:keyboardForm x:xpos y:ypos.
	prevX := xpos.
	prevY := ypos.
	timedBlock := [self cycle].
	Processor addTimedBlock:timedBlock afterSeconds:2 
    ]

    "Modified: 17.5.1996 / 11:33:21 / cg"
!

cycleAndCollectGarbage
    "a good place to do a garbage collection ..."

    timedBlock value.
    ObjectMemory garbageCollect
! !

!ScreenSaver methodsFor:'queries'!

createOnTop
    "should this view be created on top of others ? - yes"

    ^ true

    "Modified: 17.5.1996 / 11:28:19 / cg"
!

isPopUpView
    "is this a kind of popupView ? - yes"

    ^ true

    "Modified: 17.5.1996 / 11:28:48 / cg"
! !

!ScreenSaver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ScreenSaver.st,v 1.7 2000/03/02 14:15:04 cg Exp $'
! !
