"{ Package: 'stx:goodies' }"

"
 claus:

 this code was picked up from comp.lang.smalltalk,
 slightly modified, to handle multiple occurrences of the same
 object.
 You may find it useful ...

---- original letter:

Matthias> Dear Netters,

Matthias> I have to implement a quick search in an alphabetic list of words,
Matthias> with the possibility to access the predecessor.
[...]
Matthias> Matthias

Sounds like you can use SortedCollection for that.  Here are some
methods that I wrote to access predecessor and successors of an object
in a SortedCollection, even if the object is not in the collection.

These methods are added directly to SortedCollection.  If you don't
like that sort of thing, feel free to subclass it and add these
methods to the subclass:).

These methods are not perfectly named, since they find elements that
would preceed or succeed an element if it were in the collection, even
if the element isn't there.

A search on a SortedCollection should be just as efficient as a search
on an AVL tree, by the way.

If you're only looking at elements that are in the collection, you
don't even need these methods; just use before: and after:.
"

"--- cut here ---"
!

!SortedCollection methodsFor: 'MODS'!

predecessor: anElement
	^self predecessor: anElement ifNone: [self error: 'There are no elements preceding this one']!

successor: anElement ifNone: aBlock
	| i |
	i := self indexForInserting: anElement.
	"skip multiple occurrences of this one"
	[(i <= lastIndex) and:[(contentsArray at:i) == anElement]] whileTrue:[i := i + 1].
	i > lastIndex ifTrue: [^aBlock value].
	^contentsArray at: i
!

successor: anElement
	^self successor: anElement ifNone: [self error: 'There are no elements succeeding this one']!

predecessor: anElement ifNone: aBlock
	| i |
	i := self indexForInserting: anElement.
	i <= firstIndex ifTrue: [^aBlock value].
	^ contentsArray at: i - 1
! !

"--- end of fileIn ---"!
"
--

	-- Bill Burdick
	burdick@ars.rtp.nc.us
"
