"{ Package: 'stx:goodies' }"

"{ NameSpace: Benchmarks }"

Object subclass:#UCSDJavaBenchmarks
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-UCSD'
!

!UCSDJavaBenchmarks class methodsFor:'documentation'!

documentation
"
    parts of the UCSD Java Benchmark.

    Notice:
        the results are not naively comparable;
        for example, the Java code for addInt silently generates a wrong sum,
        as it uses a 32bit integer for the sum (which is overflowing in the loop).

        Thats the kind of bugs, which make Smalltalk different !!
"
!

originalJavaCode
"
class ArithmeticTest {
  static final int ADDS = 1000000;
  static final int MULTS = 1000000;
  static final int FADDS = 1000000;
  static final int FMULTS = 1000000;
  Timer intaddtimer, intmulttimer;
  Timer doubleaddtimer, doublemulttimer;
  int sum = 0, prod = 1;
  double fsum = 0.0, fprod = 1.0;
  double realnumber = Math.random() * 43213.5752 + 1;  // WGG - don't use int i

  // WGG - note that there are extra adds hidden in the loop counter
  public void start(PrintStream pstream) {
    intaddtimer = new Timer();  
    intmulttimer = new Timer(); 
    doubleaddtimer = new Timer();       
    doublemulttimer = new Timer();      

    intaddtimer.mark();
    for(int i = 0; i < ADDS; i++) {
//      try {
        sum += i;
//        } catch(Exception e) { pstream.println(""int add exception""); }
      }
    intaddtimer.record();

    intmulttimer.mark();
    for(int i = 1; i <= MULTS; i++) {
//      try {
        prod *= i;
//        } catch(Exception e) { pstream.println(""int mult exception""); }
      }
    intmulttimer.record();

    doubleaddtimer.mark();
    for(int i = 0; i < FADDS; i++) {
//      try {
        fsum += realnumber;
//        } catch(Exception e) { pstream.println(""double float add exception""); }
      }
    doubleaddtimer.record();

    doublemulttimer.mark();
    for(int i = 1; i <= FMULTS; i++) {
//     try {
        fprod *= realnumber;
//        } catch(Exception e) { pstream.println(""double float mult exception""); }
      }
    doublemulttimer.record();
  }

"
! !

!UCSDJavaBenchmarks class methodsFor:'tests-arithmetic'!

addDouble1
    |sum|

    sum := 0.0.
    1 to:1000000 do:[:i | 
        sum := sum + i.
    ].
!

addDouble2
    |sum i|

    sum := 0.0.
    i := 1.
    [ i <= 1000000 ] whileTrue:[ 
        sum := sum + i.
        i := i + 1.
    ].
!

addDouble3
    |sum|

    sum := 0.0.
    1000000 timesRepeat:[ 
        sum := sum + 1.0.
    ].
!

addInt1
    "notice: does LargeInteger arithmetic"

    |sum|

    sum := 0.
    1 to:1000000 do:[:i | 
        sum := sum + i.
    ].
!

addInt2
    "notice: does LargeInteger arithmetic"

    |sum i|

    sum := 0.
    i := 1.
    [ i <= 1000000 ] whileTrue:[ 
        sum := sum + i.
        i := i + 1.
    ].
!

addInt3
    "notice: does SmallInteger arithmetic"

    |sum|

    sum := 0.
    1000000 timesRepeat:[ 
        sum := sum + 1.
    ].
!

mulDouble1
    |prod|

    prod := 0.0.
    1 to:1000000 do:[:i | 
        prod := prod + i.
    ].
!

mulDouble2
    |prod i|

    prod := 0.0.
    i := 1.
    [ i <= 1000000 ] whileTrue:[ 
        prod := prod + i.
        i := i + 1.
    ].
!

mulDouble3
    |prod|

    prod := 0.0.
    1000000 timesRepeat:[ 
        prod := prod + 1.0.
    ].
!

mulInt1
    "notice: does SmallInteger arithmetic"

    |prod|

    prod := 0.
    1 to:1000000 do:[:i | 
        prod := prod * i.
    ].
!

mulInt2
    "notice: does SmallInteger arithmetic"

    |prod i|

    prod := 0.
    i := 1.
    [ i <= 1000000 ] whileTrue:[ 
        prod := prod * i.
        i := i + 1.
    ].
!

mulInt3
    "notice: does SmallInteger arithmetic"

    |prod|

    prod := 0.
    1000000 timesRepeat:[ 
        prod := prod * 1.
    ].
!

time_addDouble1
    self addDouble1.
    Transcript showCR:(Time millisecondsToRun:[ self addDouble1 ]).

    "
     self time_addDouble1
    "
!

time_addDouble2
    self addDouble2.
    Transcript showCR:(Time millisecondsToRun:[ self addDouble2 ]).

    "
     self time_addDouble2
    "
!

time_addDouble3
    self addDouble3.
    Transcript showCR:(Time millisecondsToRun:[ self addDouble3 ]).

    "
     self time_addDouble3
    "
!

time_addInt1
    "notice: does LargeInteger arithmetic"

    self addInt1.
    Transcript showCR:(Time millisecondsToRun:[ self addInt1 ]).

    "
     self time_addInt1
    "
!

time_addInt2
    "notice: does LargeInteger arithmetic"

    self addInt2.
    Transcript showCR:(Time millisecondsToRun:[ self addInt2 ]).

    "
     self time_addInt2
    "
!

time_addInt3
    self addInt3.
    Transcript showCR:(Time millisecondsToRun:[ self addInt3 ]).

    "
     self time_addInt3
    "
!

time_mulDouble1
    self mulDouble1.
    Transcript showCR:(Time millisecondsToRun:[ self mulDouble1 ]).

    "
     self time_mulDouble1
    "
!

time_mulDouble2
    self mulDouble2.
    Transcript showCR:(Time millisecondsToRun:[ self mulDouble2 ]).

    "
     self time_mulDouble2
    "
!

time_mulDouble3
    self mulDouble3.
    Transcript showCR:(Time millisecondsToRun:[ self mulDouble3 ]).

    "
     self time_mulDouble3
    "
!

time_mulInt1
    "notice: does SmallInteger arithmetic"

    self mulInt1.
    Transcript showCR:(Time millisecondsToRun:[ self mulInt1 ]).

    "
     self time_mulInt1
    "
!

time_mulInt2
    "notice: does SmallInteger arithmetic"

    self mulInt2.
    Transcript showCR:(Time millisecondsToRun:[ self mulInt2 ]).

    "
     self time_mulInt2
    "
!

time_mulInt3
    self mulInt3.
    Transcript showCR:(Time millisecondsToRun:[ self mulInt3 ]).

    "
     self time_mulInt3
    "
! !

!UCSDJavaBenchmarks class methodsFor:'tests-loops'!

emptyLoop1
    1000000 timesRepeat:[ 
    ].
!

emptyLoop2
    1 to:1000000 do:[:i |
    ].
!

time_emptyLoop1
    self emptyLoop1.
    Transcript showCR:(Time millisecondsToRun:[ self emptyLoop1 ]).

    "
     self time_emptyLoop1
    "
!

time_emptyLoop2
    self emptyLoop2.
    Transcript showCR:(Time millisecondsToRun:[ self emptyLoop2 ]).

    "
     self time_emptyLoop2
    "
! !

!UCSDJavaBenchmarks class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/UCSDJavaBenchmarks.st,v 1.4 2006/02/14 12:29:26 cg Exp $'
! !
