"{ Package: 'stx:goodies' }"

"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"

View subclass:#TestView
	instanceVariableNames:'displayBlock actionBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Testing-Support'
!

TestView comment:'
Copyright (c) 1991-1994
Atsushi Aoki (SRA Boulder),
Katsuhiro Watanabe (SRA Tokyo),
Taiji Matsuoka (SRA Tokyo),
Yoshiyuki Nishinaka (SRA Tokyo) and
Makoto Minagawa (FXIS Tokyo).
Software Research Associates, Inc.
1113 Spruce Street, Suite 502
Boulder, Colorado 80302 the U.S.A.
TEL: +1-303-440-3582
FAX: +1-303-440-3930
'!

!TestView class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"
!

documentation
"
   
 I have a display algorithm and a action algorithm as BlockClosures. 
    I evaluate the display algorithm when displaing the view, 
    and the action algorithm when the action is requested by mouse button. 
    Each blockClosures expects two arguments, to say the view and the graphics context.
    I am easy to use, and convenient for testing and learning the behavior or graphics context.
"
!

examples
"
								[exBegin]
    |anImage|

    anImage := Image fromUser.
    TestView 
	openDisplayBlock: [:view :gstate | anImage displayOn: gstate]
	actionBlock: [:view :gstate | anImage inspect]
								[exEnd]
"


! !

!TestView class methodsFor:'instance creation'!

displayBlock: aBlock 
	^self displayBlock: aBlock actionBlock: [:view :gstate | ]!

displayBlock: displayBlock actionBlock: actionBlock 
	^(self new) displayBlock: displayBlock; actionBlock: actionBlock; yourself! !

!TestView class methodsFor:'utilities'!

openDisplayBlock: aBlock 
	"TestView openDisplayBlock: [:view :gstate | 
	gstate paint: ColorValue red. 
	gstate displayRectangle: (view bounds insetBy: 10)]."

	^self openDisplayBlock: aBlock actionBlock: [:view :gstate | ]!

openDisplayBlock: displayBlock actionBlock: actionBlock 
	| testView topView |
	testView := self displayBlock: displayBlock actionBlock: actionBlock.
	testView origin:0.0@0.0 corner:1.0@1.0.
	topView := StandardSystemView
				model: nil
				label: 'Test'
				minimumSize: 200 @ 200.
	topView addSubView: testView.
	topView open.
	^testView

    "Modified: 2.4.1997 / 12:09:10 / cg"
!

show: anObject 
	"TestView show: Image fromUser."

	DisplayView show: anObject label: 'Test'

    "Modified: 2.4.1997 / 13:31:19 / cg"
!

show: anObject label: labelString 
	"TestView show: Image fromUser label: 'Test'."

	DisplayView show: anObject label: labelString

    "Modified: 2.4.1997 / 13:31:25 / cg"
!

showWhile: aBlock
	"TestView
		showWhile: [:view :gstate | 
			0
				to: 1
				by: 0.01
				do: 
					[:v | 
					| text point |
					text := (v roundTo: 0.01) printString asComposedText.
					point := view bounds center - text bounds center.
					view clearInside.
					text displayOn: gstate at: point.
					Delay waitForSeconds:1.
					]]."

	self showWhile: aBlock label: 'Progress'

    "Modified: 2.4.1997 / 13:52:18 / cg"
!

showWhile: aBlock label: labelString 
	"TestView
		showWhile: [:view :gstate | 
			0
				to: 1
				by: 0.01
				do: 
					[:v | 
					| text point |
					text := (v roundTo: 0.01) printString asComposedText.
					point := view bounds center - text bounds center.
					view clearInside.
					text displayOn: gstate at: point.
					Delay waitForSeconds:1.]]
		label: 'Progress'."

	self
		showWhile: aBlock
		label: labelString
		size: 150 @ 50

    "Modified: 2.4.1997 / 14:00:14 / cg"
!

showWhile: aBlock label: labelString size: sizePoint 
	"TestView
		showWhile: [:view :gstate | 
			0
				to: 1
				by: 0.01
				do: 
					[:v | 
					| text point |
					text := (v roundTo: 0.01) printString asComposedText.
					point := view bounds center - text bounds center.
					view clearInside.
					text displayOn: gstate at: point]]
		label: 'Progress'
		size: 150 @ 50."

	| testView topView openRectangle |
	testView := self displayBlock: [:v :gc | ]
				actionBlock: [:v :gc | ].
	testView origin:0.0@0.0 corner:1.0@1.0.
	topView := StandardSystemView
				model: nil
				label: labelString
				minimumSize: sizePoint.
	topView addSubView: testView.
"/        openRectangle := 0 @ 0 extent: topView minimumSize.
"/        openRectangle := openRectangle align: openRectangle center with: MeiSensorUtility cursorPoint.
"/        topView openIn: openRectangle.
	topView open.
	[topView topComponent sensor hasDamage] whileFalse.
	topView displayOn: topView graphicsContext.
	aBlock value: testView value: testView graphicsContext.
	topView close

    "Modified: 2.4.1997 / 13:32:59 / cg"
! !

!TestView methodsFor:'accessing'!

actionBlock
	actionBlock isNil ifTrue: [^[:view :gstate | ]].
	^actionBlock!

actionBlock: aBlock
	actionBlock := aBlock!

displayBlock
	displayBlock isNil ifTrue: [^[:view :gstate | ]].
	^displayBlock!

displayBlock: aBlock
	displayBlock := aBlock! !

!TestView methodsFor:'controller accessing'!

defaultControllerClass
	^TestController

    "Modified: 2.4.1997 / 11:57:58 / cg"
! !

!TestView methodsFor:'displaying'!

displayOn: graphicsContext 
	self displayBlock value: self value: graphicsContext!

redraw
	self displayOn: self

    "Created: 2.4.1997 / 13:15:55 / cg"
! !

!TestView methodsFor:'wrapper accessing'!

defaultWrapperClass
	^BorderedWrapper! !

!TestView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/Views-TestView.st,v 1.4 2000/03/02 14:15:17 cg Exp $'
! !
