"{ Package: 'stx:goodies' }"

Object subclass:#XBaseFile
	instanceVariableNames:'filterArray lastUpdate setDelete records fileName fileStream
		headerSize recordSize currentRecord deleted updated headerUpdated
		eof bof fieldsInfos fieldsContents version'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-DBase-Interface'
!

!XBaseFile class methodsFor:'documentation'!

documentation
"
    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.

    NAME             XBaseFile
    AUTHOR           criter@tin.it (Antonio d'Avino - NAPLES (ITALY))
    URL              (none)
    FUNCTION         Creating and accessing xBase file (DBIII/IV, Clipper etc.)
    KEYWORDS         DBase sDBase DBIII DBIV CLIPPER
    ST-VERSIONS      Squeak
    PREREQUISITES    (none)
    CONFLICTS        (none known)
    DISTRIBUTION     world
    VERSION          1.00
    DATE             16-Apr-98

    A pure Smalltalk class giving a full access to xDBase files.  

    Fully compatible with CLIPPER(TM) 5.xx and DB III/IV (TM) Files.
    V. 1.00  Feb 13, 1998. 
    Please, feel free to post your messages to andavino@tin.it.


    TODO (NEXT STEPS)

        * Index management
        * Visual xBase files maintenance tool

    ...... stay tuned

    [author:]
        Antonio d'Avino - NAPLES (ITALY) criter@tin.it

    [modifications:]
        Claus Gittinger - adapted to ST/X; test does not run - needs more work

"
! !

!XBaseFile class methodsFor:'Class Testing'!

test 
    "A DBFile class test method"

    | a b |
    b _ Dictionary new.
    Transcript cr; show: 'Building new test xBase file ...'.
    b at: 'abc' put: (Array with: $C with: 5 with: 0 with: nil).
    a _ XBaseFile new: #test fields: b.
    1 to: 10 do: [ : ix | a append;
        at: #abc put: (ix asString)
    ].
    Transcript show: ' Done'.
    Transcript cr; show: 'Records in file ', ( a records asString).
    Transcript cr; show: 'Setting filter on field ABC. Filter expression ''((selfDB at: #abc) asNumber  == 2) not '' ...'.
    a setFilter: '((selfDB at: #abc) asNumber  == 2) not '.
    Transcript show: ' Done'.

    Transcript cr; show: 'Start 1st test: Skipping forward from top ... '.
    a goTop.
      
    [ a eof or: [a bof]] whileFalse: [
        Transcript cr; show: (a record asString), ' -- ', (a at: #abc) . a skip 
    ].
    Transcript show: ' Done'.
    Transcript cr; show: 'Start 2st test: Skipping backward from bottom ... '.
    a goBottom.
      
    [ a eof or: [a bof]] whileFalse: [
        Transcript cr; show: (a record asString), ' -- ', (a at: #abc) . a skip: -1 
    ].
    Transcript show: ' Done'.
    Transcript cr; show: 'Closing xBase test file and exiting ...' .
    a close.
    Transcript show: ' Done'.
    Transcript cr.

    "
     self test
    "

    "Modified: / 13.11.2001 / 19:43:48 / cg"
! !

!XBaseFile class methodsFor:'Instance Creation'!

new: fileName
    "create a DBFile object open on an existing .DBF file named fileName. fileName may               be a string or a symbol."

    ^(super basicNew setName: fileName asString) open.
        
!

new: fileName fields: fieldsDictionary
    "create a DBFile object creating a .DBF file named fileName. 
     fileName may be a string or a symbol. 
     FieldsDictionary is a Dictionary whose key is the field name as string and
     the value an Array of 4 dimensions: 
	1st position is a character indicating the field type
		($C -> Character, $N Numeric, $L Logical, $D Date). 
	2nd  is field length. 
	3rd the field is decimal positions ( is not-significant if field is not Numeric).  
	4th position may be left nil
    "

    ^ (super basicNew setName: fileName asString) create: fieldsDictionary.
            
! !

!XBaseFile methodsFor:'Accessing'!

append
    "append a new, blank record"

    updated ifTrue: [ self saveRecord ].
    (records > 0) ifTrue: [
	self gotoRecord: records.
	fileStream skip: recordSize.
    ] ifFalse: [
	fileStream position: headerSize
    ].
    records _ records+1.
    currentRecord _ records.
    self blankRecord;
	     saveRecord.
    headerUpdated _ true.
        
        
        
!

at: fieldName
    "answer current content of field. Field name maybe a string or a symbol. 
     It returns nil if fieldName is not in file. fieldName is case insenitive"

    ^self unformatField: fieldName
!

at: fieldName put: fieldValue
    " set field  fieldName value to fieldValue. Field name may be a string or a symbol. 
      fieldValue MUST be of the correct type for the field. The methods returns current
      field value or nil if fieldName is no in file. fieldName is case insensitive"

    | old  new |

    old _ self unformatField: fieldName.
    new _ self formatField: fieldName value: fieldValue.
    old isNil ifFalse: [ 
	updated _ true. 
	fieldsContents at: (fieldName asString asUppercase) put: new 
    ].
    ^old
        
!

delete
    "mark as deleted current record"

    deleted ifFalse: [ deleted _ true. updated _ true. ]
!

fieldDec: fieldName
    "returns  field decimal positions for field named fieldName     
     (string or symbol, case insensitive). 
     If field is not numeric this datum is not significant and usually zero. 
     Method returns nil if fieldName is not in file"

    | answ |

    answ _ fieldsInfos at: (fieldName asString asUppercase) ifAbsent: [ nil ].
    ^ answ isNil ifTrue: [ nil ] ifFalse: [ answ at: 2 ]

        
!

fieldInfos: fieldName
    "returns an array three items sized, containing infos for field named fieldName                  (string or symbol, case insensitive).. First array item is
     the field type, second the field length, third field decimal positions, fourth is
     the field offset inside record. Method returns nil if fieldName is not in file"
    
    ^fieldsInfos at: (fieldName asString asUppercase) ifAbsent: [ nil ].

        
!

fieldInfosDictionary
    "returns the field infos dictionary. Any key in dictionary is a field name, value
     is an array four items sized, containing field infos. 
     First array item is the field type, second the field length, third field decimal positions,
     fourth is the field offset inside record."
    
    ^fieldsInfos .

        
!

fieldLen: fieldName
    "returns field length for field named fieldName (string or
     symbol, case insensitive). Method returns nil if fieldName is not in file"

    | answ |

    answ _ fieldsInfos at: (fieldName asString asUppercase) ifAbsent: [ nil ].
    ^ answ isNil ifTrue: [ nil ] ifFalse: [ answ at: 2 ]

        
!

fieldType: fieldName
    "returns a character describing field type for field named fieldName (string or
     symbol, case insensitive). Character $C stands for Character field, $N for 
     Numeric field, $L for logical field, $D for date field. Returns nil if fieldName
     is not in file"

    | answ |

    answ _ fieldsInfos at: (fieldName asString asUppercase) ifAbsent: [ nil ].
    ^ answ isNil ifTrue: [ nil ] ifFalse: [ answ at: 1 ]

        
!

record
    "return current record number"

    ^currentRecord
!

records
    "return records in file"

    ^records
!

restore
    "unmark as deleted current record"

    deleted ifTrue: [deleted _ false. updated _ true]
! !

!XBaseFile methodsFor:'Exiting'!

close
    "Close xDBase file flushing last updates and release object "

    updated ifTrue: [ self saveRecord ].
    headerUpdated ifTrue: [ self saveHeaderInfo ].
    fileStream close.
! !

!XBaseFile methodsFor:'Initializing'!

open
    "Open an existing .DBF file. This methods is called by class methods new: and 
    new:fields:. May be called directly to reopen a closed file"

    self setVersion.
    fileName asFilename exists
"/    (StandardFileStream isAFileNamed: fileName) 
    ifFalse: [ 
        ^ self error: fileName, ': no such .DBF file' 
    ].

"/    fileStream _ StandardFileStream fileNamed: fileName.
    fileStream := fileName asFilename readWriteStream.
    self isDBFFile ifFalse: [ 
        fileStream close.
        ^ self error: fileName, ': is not a .DBF file' 
    ].

    updated _ false.
    setDelete _ false.
    self getDBFInfos.
    self goto: 1.
    ^ self

    "Modified: / 13.11.2001 / 19:43:32 / cg"
! !

!XBaseFile methodsFor:'Positioning'!

goBottom
    "set last record in file as current record. Deleted and Filter settings are influent"

    self 
	goto: records;
	skip: 0 direction: -1
!

goTop
    "set first record in file as current record. Deleted and Filter settings are influent"

    self 
	goto: 1;
	skip: 0 direction: 1
!

goto: rNumber
    "position file on record number rNumber. Deleted and Filter settings are NOT influent"

    | recordNumber |

    eof _ false.
    bof _ false.
    updated ifTrue: [ self saveRecord ].
    (rNumber < 1) ifTrue: [
	recordNumber _ 1. bof _ true. 
    ] ifFalse: [
	recordNumber _ rNumber
    ].
    (recordNumber<=records) ifTrue: [
	self gotoRecord: (currentRecord _ recordNumber).
	self getRecord.
    ] ifFalse: [ 
	currentRecord _ records+1.
	eof _ true.
	self blankRecord.
	deleted _ false.
    ]
                                                                        
!

skip
    "set next record in file as current record. Deleted and Filter settings are influent"

    ^self skip: 1 direction: 1
!

skip: aNumber
    "jump to next aNumber record. 
     Jump maybe forward (aNumber > 0) or backward (aNumber <0). 
     Deleted and Filter settings are influent."

    self skip: aNumber direction: ((aNumber abs) / aNumber)
! !

!XBaseFile methodsFor:'Private'!

blankRecord
    "put empty contents into field contents collection"
    
    | tp k val |

    updated _ false.
    fieldsInfos associationsDo: [ : a  | 
	tp _ a value at: 1.
	k _ a key.
	(tp==$C) ifTrue: [
	    val _ self formatField: k value: ''
	].
	(tp==$N) ifTrue: [val _ self formatField: k value: 0].
	(tp==$L) ifTrue: [val _ false].
	(tp==$D) ifTrue: [val _ self formatField: k value: (Date newDay: 1 month: 1 year: 1)].
	fieldsContents at: k put: val 
    ].
!

char2Num: aString
    "converts hex num in aString to integer"
    
    | aNum ix |

    aNum _ 0.
    ix _ 1.
    aString do: [ : a | aNum _ aNum + ((a  asInteger) * ix). ix _ ix*256].
    ^aNum
!

create: fDictionary

    | pos  ky |

    self setVersion.
    "/ fileStream _ StandardFileStream newFileNamed: fileName.
    fileStream _ fileName asFilename readWriteStream.
    lastUpdate _ Date today.
    records _ 0.
    currentRecord _ 0.
    updated _ false.
    headerUpdated _ false.
    deleted _ false.
    setDelete _ false.
    pos _ 1.
    fieldsContents _ Dictionary new.
    fieldsInfos _ Dictionary new.
    fDictionary associationsDo: [ : a |  
        ky _ a key asString asUppercase. 
        ky _ ky copyFrom: 1 to: (10 min: (ky size)).
        (a value) at: 4 put: pos.  
        fieldsInfos at: ky put: a value.
        pos _ pos+((a value) at:  2).
    ].
    recordSize _ pos.
    headerSize _ 34+((fieldsInfos size) * 32).
    self createHeaderInfo

    "Modified: / 13.11.2001 / 19:43:07 / cg"
!

createHeaderInfo
    "save .DBF file header info's"
 
    headerUpdated _ false.
    fileStream 
        position: fileStream class zeroPosition;
        nextPut: (Character value: 3);
        nextPut: ((self num2Char: ((lastUpdate year) - 1900) size: 1) at: 1);
        nextPut: ((self num2Char: (lastUpdate monthIndex) size: 1) at: 1);
        nextPut: ((self num2Char: (lastUpdate dayOfMonth - 1900) size: 1) at: 1);
        nextPutAll: (self num2Char: records size: 4);
        nextPutAll: (self num2Char: headerSize size: 2);
        nextPutAll: (self num2Char: recordSize size: 2).
    self saveFieldsInfos.
    fileStream 
        nextPut: (Character value: 13);
        nextPut: (Character value: 0);
        nextPut: (Character value: 26).

    "Modified: / 13.11.2001 / 19:42:16 / cg"
!

evaluateFilters
    | answ |

    answ _ true.
    filterArray do: [ : a | answ _ answ and: [ a value: self ]].
    ^ answ not
!

formatField: fieldName value: v
    |  answ k  t l d  i dc is dcs cti sgn comma tmp |

    k _ fieldName asString asUppercase.
    t _ (fieldsInfos at: k ifAbsent: [ ^ nil ]) at: 1.
    l _ (fieldsInfos at: k ) at: 2.
    d _ (fieldsInfos at: k ) at: 3.
    (t == $N) ifTrue: [    
	i _ v asInteger.
	is _ i abs asString.
	sgn _ (i<0) ifTrue: ['-'] ifFalse: [''].
	comma _ (d>0) ifTrue: ['.'] ifFalse: [''].
	cti _ ((tmp _ (l - d - (is size) - (sgn size) - (comma size))) > 0) 
	    ifTrue: [
		  String new: tmp withAll:'0'
	    ] ifFalse: [''].
	(d>0) ifTrue: [       
	    dc _ (v - i) abs.
	    dcs _ dc asFloat asString.
	    dcs _ ((dcs copyFrom: 3 to: dcs size), (String new: d withAll: $0)) copyFrom: 1 to: d.
	] ifFalse: [ dcs _ ''].
	answ _ (sgn,cti,is,comma,dcs).
	answ _ answ copyFrom: ((answ size) - l + 1) to: (answ size)                                                                                
    ].
    (t == $L) ifTrue: [answ _ v ifTrue: [$T] ifFalse: [$F] ].
    (t == $D) ifTrue: [ 
	answ _ v year asString.
	tmp _ '0',(v monthIndex asString).
	tmp _ tmp copyFrom: (tmp size - 1) to: tmp size.
	answ _ answ,tmp.
	tmp _ '0',(v day asString).
	tmp _ tmp copyFrom: (tmp size - 1) to: tmp size.
	answ _ answ,tmp.
    ].
    (t == $C) ifTrue: [ 
	answ _ (v, (String new: l withAll: $  )) copyFrom: 1 to: l
    ].
    ^ answ



!

getDBFInfos
    "get all infos about open DBF file"

    self  getHeaderInfo;
	     getFieldsInfos.
    fieldsContents _ Dictionary new.
    ^self
!

getFieldsInfos
     "get field infos on fieldsInfos array"

     | pos name type len dec fpos  |

     pos _ 32.
     fieldsInfos _ Dictionary new.
     fpos _ 1.
     [ pos < (headerSize - 2)] whileTrue: [
	fileStream position: pos. 
	name _ (fileStream upTo: 0 asCharacter). 
	fileStream position: pos+11. 
	type _ fileStream next.
	fileStream skip: 4.
	dec _ 0.
	( type == $C ) ifTrue:  [
	    len _ self char2Num: (fileStream next: 2)
	] ifFalse:  [
	    len _ self char2Num: (fileStream next) asString.
	    dec _ self char2Num: (fileStream next) asString.
	].
	fieldsInfos at: name put: (Array with: type with: len with: dec with: fpos).
	fpos _ fpos+len.
	pos _ pos+32.
    ].

!

getHeaderInfo
     "get .DBF file header info's"

     | d m y |

     headerUpdated _ false.
     fileStream position: 1.
     y _ (self char2Num: (fileStream next) asString).
     m _ (self char2Num: (fileStream next) asString).
     d _ (self char2Num: (fileStream next) asString).

     lastUpdate _ Date newDay: d month: m year: y.
     records _ self char2Num: (fileStream next: 4).
     headerSize _ self char2Num: (fileStream next: 2).
     recordSize _ self char2Num: (fileStream next: 2).


!

getRecord
    "get current record contents"

    | tmp str k v |

    tmp _ fileStream next: recordSize.
    str _ ReadStream on: tmp from: 1 to: recordSize.
    ((tmp at: 1) == $  ) ifFalse: [ deleted _ true ]
				       ifTrue: [ deleted _ false ].
    fieldsInfos associationsDo: [ : a |  
	k _ a key.
	v _ a value.
	str position: (v at: 4).
	fieldsContents at: k  put: (str next: (v at: 2)).
    ].                                                                                      
    updated _ false.
!

gotoRecord: recordNumber
    "position currentRecord to recordNumber"
 
    ((recordNumber <= records) and: [ recordNumber>0]) ifFalse: [ 
	self error: 'Record number outside file bounds' 
    ] ifTrue: [ 
	fileStream position: headerSize+(((currentRecord _ recordNumber) - 1) * recordSize)
    ].
             
!

num2Char: aNum size: aSize
    "converts aNum hex num to aString"
    
    | tmp answ |

    answ _ String new: aSize withAll: $  .
    tmp _ aNum.
    1 to: aSize do: [ : ix |  
	answ at: ix put: (Character value: (tmp \\ 256)).
	tmp _ tmp//256 
    ].
    ^answ
!

saveFieldsInfos
    "get field infos on fieldsInfos array"

    | pos  |

    pos _ 32.
    fileStream position: 32.
    fieldsInfos associationsDo: [ : a |
	fileStream      
	    position: pos; 
	    nextPutAll: ((a key asUppercase), (Character value:0) asString);
	    position: pos+11;
	    nextPut: ((a value) at: 1);
	    skip: 4.
	(((a value) at: 1) == $C) ifTrue:[
	    fileStream nextPutAll: (self num2Char: ((a value) at: 2) size:2) 
	] ifFalse:[
	    fileStream 
		nextPut: ((self num2Char: ((a value) at: 2) size:1) at: 1);
		nextPut: ((self num2Char: ((a value) at: 3) size:1) at: 1)
	].
	pos _ pos+32.
    ].
    fileStream position: pos.
!

saveHeaderInfo
    "save .DBF file header info's"
 
    headerUpdated _ false.
    fileStream 
	position: 4;
	nextPutAll: (self num2Char: records size: 4).
     
     
!

saveRecord
    | str tmp |
    
    self gotoRecord: currentRecord.
    tmp _ String new: recordSize withAll: $ .
    deleted ifTrue: [tmp at: 1 put: $* ].
    str _ WriteStream on: tmp from: 1 to: recordSize.
    fieldsContents associationsDo: [ : a | 
	str position: ((fieldsInfos at: (a key)) at: 4).
	str nextPutAll: (fieldsContents at: a key).
    ].
    fileStream nextPutAll: tmp.
    updated _ false.
!

setName: aString
    "Set the .DBF file name to aString"

    | ext |

    ext _ ''.
    (aString includesSubstring: '.' caseSensitive: true)    ifFalse: [ ext _ '.DBF' ].
    fileName _ (aString, ext)

!

setVersion
    version _ 'XBaseFile Class Package V. 1.00 by Antonio d''Avino andavino@tin.it'

!

skip: aNumber direction: ofs
    self goto: (currentRecord + aNumber).
    [(((filterArray notNil and: [self evaluateFilters]) 
             or: [setDelete and: [deleted]]) 
     and: [ eof not]) 
     and: [bof not]] whileTrue: [
        self goto: currentRecord + ofs 
    ].

    "Modified: / 13.11.2001 / 19:44:55 / cg"
!

unformatField: fieldName
    |  k  t answ str1 y m dy |

    k _ fieldName asString asUppercase.
    t _ (fieldsInfos at: k ifAbsent: [ ^ nil ]) at: 1.
    (t == $N) ifTrue: [ answ _ (fieldsContents at: k) asNumber ].
    (t == $L) ifTrue: [(answ _ (fieldsContents at: k) == $T) ].
    (t == $D) ifTrue: [ 
	str1 _ ReadStream on: (fieldsContents at: k) from: 1 to: 8.
	y _ ( str1 next: 4) asInteger.
	m _ ( str1 next: 2) asInteger.
	dy _ (str1 next: 2) asInteger.
	answ _ Date newDay: dy month: m year: y.
    ].
    (t == $C) ifTrue: [ answ _ fieldsContents at: k ].
    ^ answ


! !

!XBaseFile methodsFor:'Settings'!

noFilter
    "remove any filter setting for file"

    filterArray _ nil
!

setDeleted
    "inquiry the deleted record filtering status"

    ^setDelete
!

setDeleted: status
    "set (status == true) or reset (status == false) deleted record filter status"

    setDelete _ status
!

setFilter: aBlock
    "set a new filter condition. Any preceding filter condition is lost. aBlock must be a
     string describing a Smalltalk block, returning  a true or false value. If returned 
     value is true record is acceptd, if false record is rejected. Current dbFile object may 
     be referred in aBlock as 'selfDB'. See 'test' class method for a sample of using 
     setFilter message"

    filterArray _ OrderedCollection new.
    filterArray add: (Compiler evaluate: '[ : selfDB | ',(aBlock asString), ']').
!

setFilterAdditive: aBlock
    "Append a new filter condition. Filter conditions are chained by a logical AND. See
     remark of setFilter method for a description of aBlock format"

    filterArray isNil ifTrue: [filterArray _ OrderedCollection new].
    filterArray add: (Compiler evaluate: '[ : selfDB | ',(aBlock asString), ']').
! !

!XBaseFile methodsFor:'Testing'!

bof
    "test a beginning of file condition"

    ^bof
!

eof
    "test an end of file condition"

    ^eof
!

isDBFFile
    "tests if the open file is a .DBF file (signature byte=03)"

    fileStream position: 0.
    ^(fileStream next asInteger = 3)
    

        
!

isDeleted
    "test a deleted condition for current record"

    ^deleted
!

isUpdated
    " the currend record has been updated (return true) or not (return false) ?"

    ^updated
!

version
    "answer current package version"

    ^ version
! !

!XBaseFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/XBaseFile.st,v 1.6 2001/11/13 18:50:18 cg Exp $'
! !
