"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#AbstractDictionaryHelper
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!AbstractDictionaryHelper class methodsFor:'constants'!

new
"/self halt.
^ super new.
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<abstractDictionary>'
! !

!AbstractDictionaryHelper methodsFor:'testing'!

testXaddAllX
	" Collection <abstractDictionary> "
	| abstractDictionary object |
	self messageSelector: #'addAll:'.

	abstractDictionary := testCase emptyCollection.
	object := testCase canonicalObject.
self anomaly: [
	self conformingValue: [abstractDictionary addAll: object]
		should: [:r| abstractDictionary keysDo: [:key |
			(abstractDictionary at: key) = (object at: key)
				ifFalse: [^false].
			^true]].
] for: #(#'DO30' "2000/12/06 Harmon, R. Implementing addAll: in
					Dictionary causes the layout of CategoryPrompter
					to be distorted. Find cause.").
!

testXatX
	" Collection <abstractDictionary> "
	self messageSelector: #'at:'.

	self conformingValue: [testCase canonicalObject at: 1]
		should: [:r| r == 11]
!

testXatXifAbsentPutX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'at:ifAbsentPut:'.

	object := testCase canonicalObject.
	self conformingValue: [object at: 3 ifAbsentPut: [99]]
		should: [:r| r = 33].

	self conformingValue: [object at: 7 ifAbsentPut: [99]]
		should: [:r| r = 99].

	self conformingValue: [object at: 17 ifAbsentPut: [nil]]
		should: [:r| r = nil]
!

testXatXifAbsentX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'at:ifAbsent:'.

	object := testCase canonicalObject.
	self conformingValue: [object at: 3 ifAbsent: [99]]
		should: [:r| r = 33].

	self conformingValue: [object at: 7 ifAbsent: [99]]
		should: [:r| r = 99]
!

testXatXputX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'at:put:'.

	object := testCase canonicalObject.
	self conformingValue: [object at: 3 put: 99]
		should: [:r| r = 99].

	self conformingValue: [object at: 7 put: 7]
		should: [:r| r = 7]
!

testXcollectX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'collect:'.

	"Standard says that collect: returns an object of the same type as
	the receiver, which would be either a Dictionary or an
	IdentityDictionary.  Most Smalltalks return an OrderedCollection."
	object := testCase canonicalObject.
	self conformingValue: [object collect: [:each| each * 2]]
		should: [:r| r keysDo: [:each |
			(r at: each) = ((object at: each) * 2)
				ifFalse: [^false].
			^true]].

	self should: [object collect: [:each| each, '2']]
		raise: TestResult error
!

testXincludesKeyX
	" Collection <abstractDictionary> "
	self messageSelector: #'includesKey:'.

	self conformingValue: [testCase canonicalObject includesKey: 1]
		should: [:r| true]
!

testXkeyAtValueX
	" Collection <abstractDictionary> "
	self messageSelector: #'keyAtValue:'.

	self conformingValue: [testCase canonicalObject keyAtValue: 22]
		should: [:r| r = 2].
!

testXkeyAtValueXifAbsentX
	" Collection <abstractDictionary> "
	self messageSelector: #'keyAtValue:ifAbsent:'.

	self conformingValue: [testCase canonicalObject
			keyAtValue: 22
			ifAbsent: [nil]]
		should: [:r| r = 2].
!

testXkeys
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'keys'.

	object := testCase canonicalObject.
	self conformingValue: [object keys]
		should: [:r| r size = object size]
!

testXkeysAndValuesDoX
	" Collection <abstractDictionary> "
	| object sum |
	self messageSelector: #'keysAndValuesDo:'.

	object := testCase canonicalObject.
	sum := 0.
	self conformingValue: [object keysAndValuesDo: [:key :value|
			sum := sum + key + value]]
		should: [:r| sum = 120].

	self should: [object keysAndValuesDo: [:key :value| key, 'a']]
		raise: TestResult error
!

testXkeysDoX
	" Collection <abstractDictionary> "
	| object sum |
	self messageSelector: #'keysDo:'.

	object := testCase canonicalObject.
	sum := 0.
	self conformingValue: [object keysDo: [:key| sum := sum + key]]
		should: [:r| sum = 10].

	self should: [object keysDo: [:key| key, 'a']]
		raise: TestResult error
!

testXrejectX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'reject:'.

	"Standard says that reject: returns an object of the same type as
	the receiver, which would be either a Dictionary or an
	IdentityDictionary.  Most Smalltalks return an OrderedCollection."
	object := testCase canonicalObject.
	self conformingValue: [object reject: [:each| each > 0]]
		should: [:r| r isEmpty].

	self should: [object reject: [:each| each * '2']]
		raise: TestResult error
!

testXremoveAllKeysX
	" Collection <abstractDictionary> "
	| keys object |
	self messageSelector: #'removeAllKeys:'.

	object := testCase canonicalObject.
	keys := object keys.
	self conformingValue: [object removeAllKeys: keys]
		should: [:r| r isEmpty]
!

testXremoveAllKeysXifAbsentX
	" Collection <abstractDictionary> "
	| keys sum |
	self messageSelector: #'removeAllKeys:ifAbsent:'.

	sum := 0.
	keys := #($a $b).
	self should: [testCase canonicalObject
			removeAllKeys: keys
			ifAbsent: [:key| sum := sum + key]]
		raise: TestResult error
!

testXremoveKeyX
	" Collection <abstractDictionary> "
	self messageSelector: #'removeKey:'.

	self conformingValue: [testCase canonicalObject removeKey: 2]
		should: [:r| r = 22]
!

testXremoveKeyXifAbsentX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'removeKey:ifAbsent:'.

	object := testCase canonicalObject.
	self conformingValue: [object removeKey: 2 ifAbsent: [nil]]
		should: [:r| r = 22].

	object := testCase canonicalObject.
	self conformingValue: [object removeKey: 666 ifAbsent: [nil]]
		should: [:r| r isNil]
!

testXselectX
	" Collection <abstractDictionary> "
	| object |
	self messageSelector: #'select:'.

	"Standard says that select: returns an object of the same type
	as the receiver, which would be either a Dictionary or an
	IdentityDictionary.  Most Smalltalks return an OrderedCollection."
	object := testCase canonicalObject.
	self conformingValue: [object select: [:each| each < 0]]
		should: [:r| r isEmpty].

	self should: [object select: [:each| each , '2']]
		raise: TestResult error
!

testXvalues
	" Collection <abstractDictionary> "
	self messageSelector: #'values'.

	self conformingValue: [testCase canonicalObject values]
		should: [:r| (r inject: 0 into: [:sum :each| sum + each]) = 110].
! !

!AbstractDictionaryHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/AbstractDictionaryHelper.st,v 1.1 2002/06/21 10:27:32 penk Exp $'
! !
