"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#ArrayFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!ArrayFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^ArrayTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Array factory>'
! !

!ArrayFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^ArrayTest returnTypeHasLimitedElementTypes
! !

!ArrayFactoryTest methodsFor:'constants'!

canonicalCollection

	^ArrayTest canonicalCollection
!

canonicalElement

	^ArrayTest canonicalElement
!

canonicalObject

	^Array
! !

!ArrayFactoryTest methodsFor:'testing'!

testXnew
	" Collection <Array factory> "
	self messageSelector: #new.

	self conformingValue: [Array new]
		should: [:r| r isEmpty].
!

testXnewX
	" Collection <Array factory> "
	self messageSelector: #new:.

	"Refinement: <Array factory>
	The parameter count specifies the size of the receiver."
	self conformingValue: [Array new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [Array new: 3]
		should: [:r|
			r size = 3
			"The initial value of each element of the new instance
				of the receiver is nil."
			& (r allSatisfy: [:elem| elem isNil])].

	"Errors: count < 0."
		self should: [Array new: -3]
			raise: TestResult error.
!

testXwithAllX
	" Collection <Array factory> "
	| collection |
	self messageSelector: #withAll:.
	"Refinement: <Array factory>
	If the elements of newElements are ordered then their ordering
	establishing their index positions in the new collection."
	collection := self canonicalCollection.
	self conformingValue: [Array withAll: collection]
		should: [:r|
			1 to: r size do: [:ndx |
				(r at: ndx) = (collection at: ndx) ifFalse: [
					^false]].
			r size = 4].

	"Errors: If any of the elements of newElements do not meet
			the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [Array withAll: illegalElements]
			raise: TestResult error]
!

testXwithX
	" Collection <Array factory> "
	| element |
	self messageSelector: #with:.

	element := self canonicalElement.
	self conformingValue: [Array with: element]
		should: [:r|
			r size = 1
			& r last = element].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		self should: [Array with: (self class limitedElementTypes)]
			raise: TestResult error].
!

testXwithXwithX
	" Collection <Array factory> "
	| collection |
	self messageSelector: #with:with:.

	collection := self canonicalCollection.
	self conformingValue: [
			Array
				with: collection first
				with: collection last]
		should: [:r|
			r size = 2
			& r first = collection first
			& r last = collection last].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Array
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" Collection <Array factory> "
	| collection |
	self messageSelector: #with:with:with:.

	collection := self canonicalCollection
		copyFrom: 1 to: 3.
	self conformingValue: [
			Array
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Array
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" Collection <Array factory> "
	| collection |
	self messageSelector: #with:with:with:with:.

	collection := self canonicalCollection.
	self conformingValue: [
			Array
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)
				with: (collection at: 4)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Array
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!ArrayFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/ArrayFactoryTest.st,v 1.1 2002/06/21 10:29:56 penk Exp $'
! !
