"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#BagFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!BagFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^BagTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Bag factory>'
! !

!BagFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^BagTest returnTypeHasLimitedElementTypes
! !

!BagFactoryTest methodsFor:'constants'!

canonicalCollection

	^BagTest canonicalCollection
!

canonicalElement

	^BagTest canonicalElement
!

canonicalObject

	^Bag
! !

!BagFactoryTest methodsFor:'testing'!

testXnew
	" Collection <Bag factory> "
	self messageSelector: #'new'.

	self conformingValue: [Bag new]
		should: [:r| r isEmpty].
!

testXnewX
	" Collection <Bag factory> "
	self messageSelector: #'new:'.

	"Refinement: <Bag factory>
	The parameter count represents a hint to the implementation as to
	the likely number of elements that may be added to the new collection.
	The new collection initially contains no elements."
	self conformingValue: [Bag new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [Bag new: 3]
		should: [:r| r isEmpty].
!

testXwithAllX
	" Collection <Bag factory> "
	| collection |
	self messageSelector: #'withAll:'.
	"Refinement: <Bag factory>
		The result is unspecified if newElements contains nil."

	collection := self canonicalCollection.
	self conformingValue: [Bag withAll: collection]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the elements of newElements do not meet
			the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [Bag withAll: illegalElements]
			raise: TestResult error]
!

testXwithX
	" Collection <Bag factory> "
	| element |
	self messageSelector: #'with:'.
	"Refinement: <Bag factory>
		The result is undefined if any of the arguments are nil."

	element := self canonicalElement.
	self conformingValue: [Bag with: element]
		should: [:r|
			r size = 1
			& r any = element].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		self should: [Bag with: (self class limitedElementTypes)]
			raise: TestResult error].
!

testXwithXwithX
	" Collection <Bag factory> "
	| collection |
	self messageSelector: #'with:with:'.
	"Refinement: <Bag factory>
		The result is undefined if any of the arguments are nil."

	collection := self canonicalCollection asArray
		copyFrom: 1
		to: 2.
	self conformingValue: [
			Bag
				with: collection first
				with: collection last]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Bag
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" Collection <Bag factory> "
	| collection |
	self messageSelector: #'with:with:with:'.
	"Refinement: <Bag factory>
		The result is undefined if any of the arguments are nil."

	collection := self canonicalCollection asArray
		copyFrom: 1
		to: 3.
	self conformingValue: [
			Bag
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Bag
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" Collection <Bag factory> "
	| collection |
	self messageSelector: #'with:with:with:with:'.
	"Refinement: <Bag factory>
		The result is undefined if any of the arguments are nil."

	collection := self canonicalCollection asArray.
	self conformingValue: [
			Bag
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)
				with: (collection at: 4)]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Bag
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!BagFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/BagFactoryTest.st,v 1.1 2002/06/21 10:30:04 penk Exp $'
! !
