"{ Package: 'stx:goodies/ansiTests/common' }"

CollectionTest subclass:#BagTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!BagTest class methodsFor:'constants'!

canonicalCollection

	^self canonicalObject
!

canonicalElement

	^2
!

canonicalObject

	^Bag
		with: 1
		with: 2
		with: 3
		with: 4
!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^nil            "DO30"
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Bag>'
! !

!BagTest class methodsFor:'public'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^true   "Has restrictions: DO30"
! !

!BagTest methodsFor:'constants'!

canonicalElement

	^self class canonicalElement
!

canonicalObject

	^self class canonicalObject
!

emptyCollection

	^Bag new
! !

!BagTest methodsFor:'testing'!

testXaddAllX
	" Collection <Bag> "
	| bag collection element1 element2 addedCollection |
	self messageSelector: #'addAll:'.

	"Refinement: <Bag>
		The result is undefined if newElements contains nil.
		The traversal order is unspecified."
	bag := Bag new.
	collection := #(666 777).
	self conformingValue: [bag addAll: collection]  
		should: [:r| (bag difference: collection) isEmpty].

	element1 := 555.
	element2 := 444.
	collection := OrderedCollection with: element1 with: element2.
	addedCollection := Bag new
		add: element1;
		add: element2;
		yourself.
	self conformingValue: [bag addAll: collection]  
		should: [:r|
			(bag includes: 444)
			& (bag includes: 555)
			& (bag includes: 666)
			& (bag includes: 777)
			& bag size = 4].
!

testXaddX
	" Collection <Bag> "
	| bag element1 element2 addedCollection |
	self messageSelector: #'add:'.

	"Refinement: <Bag>
The result is undefined if newElement is nil."
	bag  := Bag new.
	element1 := 555.
	self conformingValue: [bag add: element1]       
		should: [:r| "UNSPECIFIED" (bag includes: element1)].

	element2 := 444.
	self conformingValue: [bag add: element2]       
		should: [:r| "UNSPECIFIED"
			(bag includes: element1)
			and: [(bag includes: element2)]].
!

testXaddXwithOccurrencesX
	" Collection <Bag> "
	| it |
	self messageSelector: #'add:withOccurrences:'.

	it := Bag new.
	self conformingValue: [it add: 42 withOccurrences: 3]
		should: [:r| (it occurrencesOf: 42) = 3]
!

testXcollectX
	" Collection <Bag> "
	| bag |
	self messageSelector: #'collect:'.

	bag := self canonicalObject.
	self conformingValue: [bag collect: [:each| each * 2]]
		should: [:r| r size = 4].

	"Duplicates will not be added."
	self conformingValue: [bag collect: [:each| each even]]
		should: [:r| r size = 4].

	"Errors If the elements of the receiver are inappropriate for use
	as arguments to transformer."

	"Errors If the result of evaluating the transformer does not conform
		to any element type restrictions of the collection to be returned."
	self should: [bag collect: [:each| each, 'string']]
		raise: TestResult error.

! !

!BagTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/BagTest.st,v 1.1 2002/06/21 10:38:20 penk Exp $'
! !
