"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#CharacterTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Fundamental'
!


!CharacterTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Character>'
! !

!CharacterTest methodsFor:'constants'!

canonicalObject

	^Character space
! !

!CharacterTest methodsFor:'testing'!

testXasLowercase
	" Fundamental <Character> "
	self messageSelector: #'asLowercase'.

	"If the receiver is equal to the value of a character literal in
	the receiver row of the following table, the result object must
	be equal to the value of the corresponding character literal in
	the result row.

		receiver                $A - $Z
		result          $a - $z

	An implemention may define other #asLowercase mappings.  If the
	receiver does not correspond to a character in the receiver row
	of the table and does not have an implementation defined mapping
	the receiver is returned as the result."

	self conformingMessageSelectorBehavesLike: #( #($A $a) #($b $b) #($1 $1) #($: $:) )
!

testXasString
	" Fundamental <Character> "
	self messageSelector: #'asString'.

	self conformingMessageSelectorBehavesLike: #( #($A 'A') #($B 'B') #($a 'a') #($1 '1') #($: ':') )
!

testXasUppercase
	" Fundamental <Character> "
	self messageSelector: #'asUppercase'.

	"If the receiver is equal to the value of a character literal in the
	receiver row of the following table, the result object must be equal
	to the value of the corresponding character literal in the result row.

		receiver                $a - $z
		result          $A - $Z

	An implemention may define other #asUppercase mappings.  If the
	receiver does not correspond to a character in the receiver row
	of the table and does not have an implementation defined mapping
	the receiver is returned as the result."

	self conformingMessageSelectorBehavesLike: #( #($a $A) #($B $B) #($1 $1) #($: $:) )
!

testXcodePoint
	" Fundamental <Character> "
	self messageSelector: #'codePoint'.

	"The following invariant must hold:

		(charFactory codePoint: x) codePoint = x

	where charFactory is an object that implements <Character factory>
	and x is an <integer>."

	self conformingValue: [$a codePoint]
		should: [:r| (Character codePoint: r) codePoint = r].
!

testXequalityOp
	" Fundamental <Character> "
	| character1 character2 |
	self messageSelector: #'='.
	character1 := $a.
	"Refinement: <Character>
	Two characters are considered equivalent if they have the same code point.  In other words character1 = character2 is true if and only if character1 codePoint = character2 codePoint is also true."

	character2 := character1.
	self conformingValue: [character1 = character2]
		should: [:r| r & character1 codePoint = character2 codePoint].

	character2 := $A.
	self conformingValue: [character1 = character2]
		shouldnt: [:r| r | character1 codePoint = character2 codePoint].
!

testXisAlphaNumeric
	" Fundamental <Character> "
	self messageSelector: #'isAlphaNumeric'.

	"Return true if the receiver is either a letter or digit.  Otherwise
	return false.  In other words character isAlphaNumeric is true if
	and only if either character isLetter is true or character isDigit
	is true."

	self conformingValue: [$A isAlphaNumeric]
		should: [:r| r & ($A isLetter | $A isDigit)].

	self conformingValue: [$B isAlphaNumeric]
		should: [:r| r & ($B isLetter | $B isDigit)].

	self conformingValue: [$a isAlphaNumeric]
		should: [:r| r & ($a isLetter | $a isDigit)].

	self conformingValue: [$1 isAlphaNumeric]
		should: [:r| r & ($1 isLetter | $1 isDigit)].

	self conformingValue: [$: isAlphaNumeric]
		shouldnt: [:r| r & ($: isLetter | $: isDigit)].
!

testXisDigit
	" Fundamental <Character> "
	self messageSelector: #'isDigit'.

	self conformingMessageSelectorBehavesLike: #(#($1 true) #($A false) #($B false) #($b false) #($: false)).
!

testXisLetter
	" Fundamental <Character> "
	| alphaChars |
	self messageSelector: #'isLetter'.

	alphaChars := #( $A $B $C $D $E $F $G $H $I $J $K $L $M
					$N $O $P $Q $R $S $T $U $V $W $X $Y $Z
					$a $b $c $d $e $f $g $h $i $j $k $l $m
					$n $o $p $q $r $s $t $u $v $w $x $y $z ).

	self conformingValue: [$A isLetter]
		should: [:r| r & (alphaChars includes: $A)].

	self conformingValue: [$B isLetter]
		should: [:r| r & (alphaChars includes: $B)].

	self conformingValue: [$a isLetter]
		should: [:r| r & (alphaChars includes: $a)].

	self conformingValue: [$1 isLetter]
		shouldnt: [:r| r & (alphaChars includes: $1)].

	self conformingValue: [$: isLetter]
		shouldnt: [:r| r & (alphaChars includes: $:)].
!

testXisLowercase
	" Fundamental <Character> "
	| lowerChars |
	self messageSelector: #'isLowercase'.

	lowerChars := #( $a $b $c $d $e $f $g $h $i $j $k $l $m
					$n $o $p $q $r $s $t $u $v $w $x $y $z ).

	self conformingValue: [$a isLowercase]
		should: [:r| r & (lowerChars includes: $a)].

	self conformingValue: [$A isLowercase]
		shouldnt: [:r| r & (lowerChars includes: $A)].

	self conformingValue: [$B isLowercase]
		shouldnt: [:r| r & (lowerChars includes: $B)].

	self conformingValue: [$1 isLowercase]
		shouldnt: [:r| r & (lowerChars includes: $1)].

	self conformingValue: [$: isLowercase]
		shouldnt: [:r| r & (lowerChars includes: $:)].
!

testXisUppercase
	" Fundamental <Character> "
	| upperChars |
	self messageSelector: #'isUppercase'.

	upperChars := #( $A $B $C $D $E $F $G $H $I $J $K $L $M
					$N $O $P $Q $R $S $T $U $V $W $X $Y $Z ).

	self conformingValue: [$A isUppercase]
		should: [:r| r & (upperChars includes: $A)].

	self conformingValue: [$B isUppercase]
		should: [:r| r & (upperChars includes: $B)].

	self conformingValue: [$a isUppercase]
		shouldnt: [:r| r & (upperChars includes: $a)].

	self conformingValue: [$1 isUppercase]
		shouldnt: [:r| r & (upperChars includes: $1)].

	self conformingValue: [$: isUppercase]
		shouldnt: [:r| r & (upperChars includes: $:)].
! !

!CharacterTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/CharacterTest.st,v 1.1 2002/06/21 10:31:23 penk Exp $'
! !
