"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#CollectionHelper
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!CollectionHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<collection>'
! !

!CollectionHelper methodsFor:'testing'!

testXallSatisfyX
        " Collection <collection> "
        | collection |
        self messageSelector: #'allSatisfy:'.

        collection := testCase canonicalObject.
        self conformingValue: [collection allSatisfy:[:each| true]]
                should: [:result| result].

        self conformingValue: [collection allSatisfy: [:each| false]]
                shouldnt: [:result| result].

        self conformingValue: [(collection reject: [:each|
                        each = testCase canonicalElement]) allSatisfy: [:each|
                                        each ~= testCase canonicalElement]]
                should: [:result| result].

        self conformingValue: [
                        testCase emptyCollection allSatisfy: [:each| true]]
                should: [:result| result].

self anomaly: [
        self should: [collection allSatisfy: [:each| nil]]
                raise: TestResult error
] for: #(#'STX36' "stc treats non-true as false"
         #'STX41' "stc treats non-true as false").
!

testXanySatisfyX
	" Collection <collection> "
	self messageSelector: #'anySatisfy:'.

	self conformingValue: [testCase canonicalObject anySatisfy: [:each|
			each = testCase canonicalElement]]
		should: [:result| result].

	self conformingValue: [testCase canonicalObject anySatisfy: [:each| false]]
		shouldnt: [:result| result].

	self conformingValue: [(testCase canonicalObject reject: [:each|
			each = testCase canonicalElement]) anySatisfy: [:each|
				each = testCase canonicalElement]]
		shouldnt: [:result| result].
!

testXasArray
	" Collection <collection> "
	self messageSelector: #'asArray'.

	self conformingAssertMessageSelector
!

testXasBag
	" Collection <collection> "
	self messageSelector: #'asBag'.

	self conformingAssertMessageSelector
!

testXasByteArray
	" Collection <collection> "
	self messageSelector: #'asByteArray'.

	self conformingAssertMessageSelector.
!

testXasOrderedCollection
	" Collection <collection> "
	self messageSelector: #'asOrderedCollection'.

	self conformingAssertMessageSelector
!

testXasSet
	" Collection <collection> "
	self messageSelector: #'asSet'.

	self conformingAssertMessageSelector
!

testXasSortedCollection
	" Collection <collection> "
	self messageSelector: #'asSortedCollection'.

	self conformingAssertMessageSelector.
!

testXasSortedCollectionX
	" Collection <collection> "
	self messageSelector: #'asSortedCollection:'.

	self conformingValue: [testCase canonicalObject
			asSortedCollection: [:a :b| b < a]]
		should: [:result| result size = testCase canonicalObject size].

	self conformingValue: [testCase emptyCollection
			asSortedCollection: [:a :b| b < a]]
		should: [:result| result size = testCase emptyCollection size].

	"Test a garbage message for any parameter passed into the block."
	self should: [testCase canonicalObject
			asSortedCollection: [:a :b| a gobbledegook]]
		raise: TestResult error
!

testXcollectX
	" Collection <collection> "
	| collection |
	self messageSelector: #'collect:'.

	collection := testCase canonicalObject.
	self conformingValue: [collection collect: [:each| each]]
		should: [:result| result size = collection size].

	"Test empty class."
	self conformingValue: [testCase emptyCollection collect: [:each| true]]
		should: [:result| result isEmpty].

	testCase class returnTypeHasLimitedElementTypes
		ifTrue: [
			self should: [collection
					collect: [:each| testCase class limitedElementTypes]]
				raise: TestResult error].

	self should: [collection collect: [:each| each gobbledegook]]
		raise: TestResult error.
!

testXdetectX
	" Collection <collection> "
	self messageSelector: #'detect:'.

	self conformingValue: [testCase canonicalObject
			detect: [:each| each = testCase canonicalElement]]
		should: [:result| result = testCase canonicalElement].

	"Test empty class.  What is the appropriate behavior for sending
	detect: to an empty collection?  Both VA and VW raise element
	not found errors."
	self should: [testCase emptyCollection detect: [:each| true]]
		raise: TestResult error.

	self should: [testCase canonicalObject detect: [:each| nil]]
		raise: TestResult error.

	self should: [testCase canonicalObject detect: [:each| false]]
		raise: TestResult error.

	self should: [
			testCase canonicalObject detect: [:each| each gobbledegook]]
		raise: TestResult error
!

testXdetectXifNoneX
	" Collection <collection> "
	self messageSelector: #'detect:ifNone:'.

	self conformingValue: [testCase canonicalObject
			detect: [:each| each = testCase canonicalElement]
			ifNone: [nil]]
		should: [:result| result = testCase canonicalElement].

	"Test empty class.  What is the appropriate behavior for sending
	detect: to an empty collection?  Both VA and VW raise element
	not found errors."
	self conformingValue: [testCase emptyCollection
			detect: [:each| true]
			ifNone: [#'NotFound']]
		should: [:result| result == #'NotFound'].

	self should: [testCase canonicalObject
			detect: [:each| nil]
			ifNone: [nil]]
		raise: TestResult error.

	self conformingValue: [testCase canonicalObject
			detect: [:each| false]
			ifNone: [#'NotFound']]
		should: [:result| result == #'NotFound'].

	self should: [testCase canonicalObject
			detect: [:each| each gobbledegook]
			ifNone: []]
		raise: TestResult error.

	self should: [testCase canonicalObject
			detect: [:each| false]
			ifNone: [self gobbledegook]]
		raise: TestResult error.
!

testXdoX
	" Collection <collection> "
	| collection |
	self messageSelector: #'do:'.

	collection := testCase canonicalObject.
	self conformingValue: [collection do: [:each| ]]
		should: [:result| "UNSPECIFIED" true].

	self conformingValue: [testCase emptyCollection do: [:each| ]]
		should: [:result| "UNSPECIFIED" true].

	"Send message that causes an error in the do: loop."
	self should: [collection do: [:each| each gobbledegook]]
		raise: TestResult error.
!

testXdoXseparatedByX
	" Collection <collection> "
	| collection |
	self messageSelector: #'do:separatedBy:'.

	collection := testCase canonicalObject.
	self conformingValue: [collection
			do: [:each| ]
			separatedBy: []]
		should: [:result| "UNSPECIFIED" true].

	"Send message that causes an error in the do: loop."
	self should: [collection
			do: [:each| each gobbledegook]
			separatedBy: []]
		raise: TestResult error.

	"Send message that causes an error in the separatedBy: loop."
	self should: [collection
			do: [:each| ]
			separatedBy: [self gobbledegook]]
		raise: TestResult error.

	self conformingValue: [testCase emptyCollection
			do: [:each| ]
			separatedBy: []]
		should: [:result| "UNSPECIFIED" true].
!

testXincludesX
	" Collection <collection> "
	self messageSelector: #'includes:'.

	self conformingValue: [testCase canonicalObject
			includes: testCase canonicalElement]
		should: [:result| result].

	self conformingValue: [testCase canonicalObject includes: 'Zornfindel']
		shouldnt: [:result| result].

	self conformingValue: [testCase emptyCollection includes: 'Zornfindel']
		shouldnt: [:result| result].
!

testXinjectXintoX
	" Collection <collection> "
	self messageSelector: #'inject:into:'.

	self conformingValue: [testCase canonicalObject
			inject: 1
			into: [:value :each|
				(each printString at: 1) asInteger * value]]
		should: [:result| result > 0].

	self conformingValue: [testCase emptyCollection
			inject: 1
			into: [:value :each|
				(each printString at: 1) asInteger * value]]
		should: [:result| result > 0].
!

testXisEmpty
	" Collection <collection> "
	self messageSelector: #'isEmpty'.

	self conformingAssertMessageSelector
!

testXnotEmpty
	" Collection <collection> "
	self messageSelector: #'notEmpty'.

	self conformingAssertMessageSelector.
!

testXoccurrencesOfX
	" Collection <collection> "
	self messageSelector: #'occurrencesOf:'.

	self conformingValue: [testCase canonicalObject
			occurrencesOf: testCase canonicalElement]
		should: [:result| result = 1].

	self conformingValue: [testCase canonicalObject
			occurrencesOf: 'Zornfindel']
		should: [:result| result = 0].

	self conformingValue: [testCase emptyCollection
			occurrencesOf: 'Zornfindel']
		should: [:result| result = 0]
!

testXrehash
	" Collection <collection> "
	self messageSelector: #'rehash'.

	self conformingAssertMessageSelector
!

testXrejectX
	" Collection <collection> "
	self messageSelector: #'reject:'.

	self conformingValue: [testCase canonicalObject
			reject: [:each| each = testCase canonicalElement]]
		should: [:result| (result includes: testCase canonicalElement) not].

	"Test empty class."
	self conformingValue: [testCase emptyCollection reject: [:each| true]]
		should: [:result| result isEmpty].

	self should: [testCase canonicalObject reject: [:each| nil]]
		raise: TestResult error.

	self should: [testCase canonicalObject
				reject: [:each| each gobbledegook]]
		raise: TestResult error
!

testXselectX
	" Collection <collection> "
	self messageSelector: #'select:'.

	self conformingValue: [testCase canonicalObject
			select: [:each| each ~= testCase canonicalElement]]
		should: [:result| (result includes: testCase canonicalElement) not].

	"Test empty class."
	self conformingValue: [testCase emptyCollection select: [:each| true]]
		should: [:result| result isEmpty].

	self should: [testCase canonicalObject select: [:each| nil]]
		raise: TestResult error.

	self should: [
			testCase canonicalObject select: [:each| each gobbledegook]]
		raise: TestResult error
!

testXsize
	" Collection <collection> "
	self messageSelector: #'size'.

	self conformingAssertMessageSelector.
! !

!CollectionHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/CollectionHelper.st,v 1.1 2002/06/21 10:37:56 penk Exp $'
! !
