"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#DateAndTimeFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Date and Time'
!


!DateAndTimeFactoryTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<DateAndTime factory>'
! !

!DateAndTimeFactoryTest methodsFor:'constants'!

canonicalObject

	^DateAndTime
! !

!DateAndTimeFactoryTest methodsFor:'testing'!

testXclockPrecision
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'clockPrecision'.

	"Answer a <Duration> such that after that period of time passes,
	#now is guaranteed to give a different result. "
	self conformingValue: [DateAndTime clockPrecision]
		should: [:r| "unspecified" true "???How to test???"].
!

testXnow
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'now'.

	"Answer a <DateAndTime> representing the current date and time
	in the local time specified by the implementation."
	self conformingValue: [DateAndTime now]
		should: [:r| "unspecified" true "???How to test???"].
!

testXyearXdayXhourXminuteXsecondX
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'year:day:hour:minute:second:'.

	"April 26, 1997 CST"
	self conformingValue: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: 3]
		should: [:r|
			(r year = 1997) & (r month = 4) & (r dayOfMonth = 26)
			& (r hour = 1) & (r minute = 2) & (r second = 3)].

	"Errors dayOfYear greater than the number of days in the year year
	of the astronomical Gregorian calendar."
	self should: [DateAndTime
			year: 1997 day: 400
			hour: 1 minute: 2 second: 3]
		raise: TestResult error.

	"Errors hour is not between 0 and 23 inclusive."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: -1 minute: 2 second: 3]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 24 minute: 2 second: 3]
		raise: TestResult error.

	"Errors minute is not between 0 and 59 inclusive."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: -1 second: 3]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 60 second: 3]
		raise: TestResult error.

	"Errors second is not greater than or equal to 0
	and strictly less than 60."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: -1]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: 60]
		raise: TestResult error.

	"Errors the time specified does not exist."
"It is possible that the time specified does not exist in the local time specified by the implementation. If there is a time change such that the local time is set forward and the time specified is in the interregnum, then that time does not exist in the local time. For example if at 02:00 in California on April 26, 1997 there is a time change that sets local time forward one hour, then the local time 02:30  in California does not exist. Conversely if there is a time change that sets the locale time back there are times which are ambiguous. For example if instead of setting the local time forward from 02:00 to 03:00 it is set back to 01:00 the the local time 01:30 in California is ambiguious. The result is the least <DateAndTime> that conforms to the given parameters."
!

testXyearXdayXhourXminuteXsecondXoffsetX
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'year:day:hour:minute:second:offset:'.

	"April 26, 1997 CST"
	self conformingValue: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: 3
			offset: (Duration hours: -6 "CST")]
		should: [:r |
			(r year = 1997) & (r month = 4) & (r dayOfMonth = 26)
			& (r hour = 1) & (r minute = 2) & (r second = 3)].

	"Errors dayOfYear greater than the number of days in the year year
	of the astronomical Gregorian calendar."
	self should: [DateAndTime
			year: 1997 day: 400
			hour: 1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors hour is not between 0 and 23 inclusive."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: -1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 24 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors minute is not between 0 and 59 inclusive."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: -1 second: 3
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 60 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors second is not greater than or equal to 0
	and strictly less than 60."
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: -1
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 day: 116
			hour: 1 minute: 2 second: 60
			offset: Duration zero]
		raise: TestResult error.

	"Errors the time specified does not exist."
"See #testXyearXdayXhourXminuteXsecondX."
!

testXyearXmonthXdayXhourXminuteXsecondX
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'year:month:day:hour:minute:second:'.

	"April 26, 1997 CST"
	self conformingValue: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: 3]
		should: [:r|
			(r year = 1997) & (r month = 4) & (r dayOfMonth = 26)
			& (r hour = 1) & (r minute = 2) & (r second = 3)].

	"Errors month is not between 1 and 12 inclusive."
	self should: [DateAndTime
			year: 1997 month: 0 day: 26
			hour: 1 minute: 2 second: 3]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 13 day: 26
			hour: 1 minute: 2 second: 3]
		raise: TestResult error.

	"Errors dayOfMonth greater than the number of days in the month month of year year of the astronomical Gregorian calendar."
	self should: [DateAndTime
			year: 1997 month: 4 day: 40
			hour: 1 minute: 2 second: 3]
		raise: TestResult error.

	"Errors hour is not between 0 and 23 inclusive."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: -1 minute: 2 second: 3]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 24 minute: 2 second: 3]
		raise: TestResult error.

	"Errors minute is not between 0 and 59 inclusive."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: -1 second: 3]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 60 second: 3]
		raise: TestResult error.

	"Errors second is not greater than or equal to 0
	and strictly less than 60."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: -1]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: 60]
		raise: TestResult error.

	"Errors the time specified does not exist."
"See #testXyearXdayXhourXminuteXsecondX."
!

testXyearXmonthXdayXhourXminuteXsecondXoffsetX
	" Date and Time <DateAndTime factory> "
	self messageSelector: #'year:month:day:hour:minute:second:offset:'.

	"April 26, 1997 CST"
	self conformingValue: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: 3
			offset: (Duration hours: -6 "CST")]
		should: [:r|
			(r year = 1997) & (r month = 4) & (r dayOfMonth = 26)
			& (r hour = 1) & (r minute = 2) & (r second = 3)].

	"Errors month is not between 1 and 12 inclusive."
	self should: [DateAndTime
			year: 1997 month: 0 day: 26
			hour: 1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 13 day: 26
			hour: 1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors dayOfMonth greater than the number of days in the month month of year year of the astronomical Gregorian calendar."
	self should: [DateAndTime
			year: 1997 month: 4 day: 40
			hour: 1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors hour is not between 0 and 23 inclusive."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: -1 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 24 minute: 2 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors minute is not between 0 and 59 inclusive."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: -1 second: 3
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 60 second: 3
			offset: Duration zero]
		raise: TestResult error.

	"Errors second is not greater than or equal to 0
	and strictly less than 60."
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: -1
			offset: Duration zero]
		raise: TestResult error.
	self should: [DateAndTime
			year: 1997 month: 4 day: 26
			hour: 1 minute: 2 second: 60
			offset: Duration zero]
		raise: TestResult error.

	"Errors the time specified does not exist."
"See #testXyearXdayXhourXminuteXsecondX."
! !

!DateAndTimeFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/DateAndTimeFactoryTest.st,v 1.1 2002/06/21 10:30:20 penk Exp $'
! !
