"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#DateAndTimeTest
	instanceVariableNames:'d19970426t8'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Date and Time'
!


!DateAndTimeTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<DateAndTime>'
! !

!DateAndTimeTest methodsFor:'constants'!

canonicalObject
	^DateAndTime
		year: 2000 month: 3 day: 16
		hour: 16 minute: 49 second: 43
!

magnitudeObjects
	| dt |
	dt := DateAndTime year: 2000 month: 3 day: 16
			hour: 16 minute: 49 second: 43.
	^(Dictionary new)
		at:#'small' put: dt - (Duration days: 1);
		at:#'medium' put: dt;
		at:#'large' put: dt + (Duration days: 1);
		yourself
! !

!DateAndTimeTest methodsFor:'others'!

setUp
	super setUp.
	d19970426t8 := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 8 minute: 0 second: 0.
! !

!DateAndTimeTest methodsFor:'testing'!

testXaddOp
	" Date and Time <DateAndTime> "
	self messageSelector: #'+'.

	self conformingValue: [d19970426t8
			+ (Duration days: 1 hours: 0 minutes: 0 seconds: 1)]
		should: [:r| r = (DateAndTime year: 1997 month: 4 day: 27
			hour: 8 minute: 0 second: 1)].

	self conformingValue: [d19970426t8
			+ (Duration days: -1 hours: 0 minutes: 0 seconds: -1)]
		should: [:r| r = (DateAndTime year: 1997 month: 4 day: 25
			hour: 7 minute: 59 second: 59)].
!

testXasLocal
	" Date and Time <DateAndTime> "
	self messageSelector: #'asLocal'.

	"Answer a <DateAndTime> that represents the same UTC time as the
	receiver but in the local time specified by the implementation. "
	self conformingValue: [d19970426t8 asLocal]
		should: [:r | true "???how test???"].
!

testXasUTC
	" Date and Time <DateAndTime> "
	self messageSelector: #'asUTC'.

	"Answer a <DateAndTime> that represents the same absolute time as
	the receiver but in the local time UTC.  The exact meaning of UTC
	local time is specified by the implementation. The UTC local time
	must use the Gregorian calendar. <DateAndTimes> representing UTC
	times prior to the adoption of the Gregorian calendar must use the
	retrospective astronomical Gregorian calendar. It is an invariant
	that: <DateAndTime> asUTC offset = Duration zero."
	self conformingValue: [d19970426t8 asUTC]
		should: [:r| r offset = Duration zero].
!

testXdayOfMonth
	" Date and Time <DateAndTime> "
	self messageSelector: #'dayOfMonth'.

	self conformingValue: [d19970426t8 dayOfMonth]
		should: [:r| r = 26].
!

testXdayOfWeek
	" Date and Time <DateAndTime> "
	self messageSelector: #'dayOfWeek'.

	"Sunday is 1, Monday is 2, and so on."
	self conformingValue: [d19970426t8 dayOfWeek]
		should: [:r| r = 7].
!

testXdayOfWeekAbbreviation
	" Date and Time <DateAndTime> "
	self messageSelector: #'dayOfWeekAbbreviation'.

	"2000/12/08 Harmon, R. Need to accommidate locale differences."
	self conformingValue: [d19970426t8 dayOfWeekAbbreviation]
		should: [:r| r = 'Sat'].
!

testXdayOfWeekName
	" Date and Time <DateAndTime> "
	self messageSelector: #'dayOfWeekName'.

	"2000/12/08 Harmon, R. Need to accommidate locale differences."
	self conformingValue: [d19970426t8 dayOfWeekName]
		should: [:r| r = 'Saturday'].
!

testXdayOfYear
	" Date and Time <DateAndTime> "
	self messageSelector: #'dayOfYear'.

	self conformingValue: [d19970426t8 dayOfYear]
		should: [:r| r = 116].
!

testXequalityOp
	" Date and Time <DateAndTime> "
	| d19970426t8CST d19970426t7MST d19970426t7CST tmp1 tmp2 |
	self messageSelector: #'='.

	"2000/06/23 Harmon, R. Tests need to account for Daylight Savings."
	d19970426t8CST := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 8 minute: 0 second: 0.
	"1997-04-26T08:00:00-06:00 CST"
	self     conformingValue: [d19970426t8 = d19970426t8CST]
		should: [:r| r & (d19970426t8 hash = d19970426t8CST hash)].

	d19970426t8CST := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 8 minute: 0 second: 0.
	"1997-04-26T08:00:00-06:00 CST"
	tmp1 := DateAndTime
		year: 1996 month: 4 day: 26
		hour: 5 minute: 59 second: 50
		offset: (Duration seconds: -10).
	tmp2 := DateAndTime
		year: 1996 month: 4 day: 26
		hour: 6 minute: 0 second: 10
		offset: (Duration seconds: 10).
	self     conformingValue: [tmp1 = tmp2]
		should: [:r| r & (tmp1 hash = tmp2 hash)].

	d19970426t7MST := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 7 minute: 0 second: 0
		offset: (Duration days: 0 hours: -7
		minutes: 0 seconds: 0).
	"1997-04-26T07:00:00-07:00 MST"
	"May fail because d19970426t7MST is Daylight Savings not Standard."
	self     conformingValue: [d19970426t8 = d19970426t7MST]
		should: [:r| r & (d19970426t8 hash = d19970426t7MST hash)].

	d19970426t7CST := DateAndTime
		year: 1996 month: 4 day: 27
		hour: 7 minute: 0 second: 0.
	"1997-04-26T07:00:00-07:00 MST"
	self conformingValue: [d19970426t8 = d19970426t7CST]
		shouldnt: [:r| r & (d19970426t8 hash = d19970426t7CST hash)].
!

testXgreaterThanOp
	" Date and Time <DateAndTime> "
	self messageSelector: #'>'.

	"Answer true if the UTC time represented by operand precedes the UTC
	time represented by the receiver. Answer false otherwise.
	If the offsets of the receiver and operand are the same then their
	order is determined by their lexical order in the sequence #year,
	#month, #day, #hour24, #minute, #second. If their offsets differ then
	result is the same as if receiver asUTC > operand asUTC were evaluated."
	self conformingValue: [(DateAndTime year: 1997 month: 4 day: 27
			hour: 8 minute: 0 second: 1) > d19970426t8]
		should: [:r| r].
	self conformingValue: [(DateAndTime year: 2000 month: 4 day: 27
			hour: 8 minute: 0 second: 0) > d19970426t8]
		should: [:r| r] .

	self conformingValue: [(DateAndTime year: 1996 month: 4 day: 27
			hour: 7 minute: 0 second: 0) > d19970426t8]
		shouldnt: [:r | r].
!

testXhour
	" Date and Time <DateAndTime> "
	| dtTmp |
	self messageSelector: #'hour'.

	"It is implementation defined whether a given local time uses the
	12-hour clock or the 24-hour clock, except that the UTC local time
	must use the 24-hour clock."
	self conformingValue: [d19970426t8 hour]
		should: [:r| r = 8].

	dtTmp := d19970426t8 + (Duration days: 0 hours: 12
			minutes: 0 seconds: 0).
	self conformingValue: [dtTmp hour]
		should: [:r| (r = 8) | (r = 20)].
!

testXhour12
	" Date and Time <DateAndTime> "
	self messageSelector: #'hour12'.

	self conformingValue: [d19970426t8 hour12]
		should: [:r| r = 9].
!

testXhour24
	" Date and Time <DateAndTime> "
	| dtTmp |
	self messageSelector: #'hour24'.

	self conformingValue: [d19970426t8 hour24]
		should: [:r| r = 8 "8 AM"].

	dtTmp := d19970426t8 + (Duration days: 0 hours: 12
			minutes: 0 seconds: 0).
	self conformingValue: [dtTmp hour24]
		should: [:r| r = 20 "8 PM"].

	dtTmp := d19970426t8 - (Duration days: 0 hours: 7 minutes: 59
			seconds: 0).
	self conformingValue: [dtTmp hour24]
		should: [:r| r = 0 "12:01 AM"].

	dtTmp := d19970426t8 + (Duration days: 0 hours: 15 minutes: 59
			seconds: 0).
	self conformingValue: [dtTmp hour24]
		should: [:r| r = 23 "11:59 PM"].
!

testXisLeapYear
	" Date and Time <DateAndTime> "
	self messageSelector: #'isLeapYear'.

	"Answer true if the year, which includes the receiver, in the local
	time of the receiver is a leap year, false otherwise.
	Two <DateAndTime> objects that are equal can give different results
	for #isLeapYear. Equality depends on their UTC time whereas
	#isLeapYear depends on their local time."

	self conformingValue: [d19970426t8 isLeapYear]
		shouldnt: [:r| r].

	self conformingValue: [(DateAndTime year: 1996 month: 2 day: 1
			hour: 0 minute: 0 second: 0 ) isLeapYear]
		should: [:r| r].
!

testXlessThanOp
	" Date and Time <DateAndTime> "
	self messageSelector: #'<'.

	self conformingValue: [d19970426t8 < (DateAndTime
			year: 1997 month: 4 day: 27
			hour: 8 minute: 0 second: 1)]
		should: [:r| r].

	self conformingValue: [d19970426t8 < (DateAndTime
			year: 2000 month: 4 day: 27
			hour: 8 minute: 0 second: 0)]
		should: [:r| r].

	self conformingValue: [d19970426t8 < (DateAndTime
			year: 1996 month: 4 day: 27
			hour: 7 minute: 0 second: 0)]
		shouldnt: [:r| r].
!

testXmeridianAbbreviation
	" Date and Time <DateAndTime> "
	self messageSelector: #'meridianAbbreviation'.
	"2000/12/08 Harmon, R. Need to accommidate locale differences."

	self conformingValue: [d19970426t8 meridianAbbreviation]
		should: [:r| r = 'AM'].

	self conformingValue: [(DateAndTime year: 1996 month: 2 day: 1
			hour: 12 minute: 0 second: 0 ) meridianAbbreviation]
		should: [:r| r = 'PM'].
!

testXminute
	" Date and Time <DateAndTime> "
	self messageSelector: #'minute'.

	self conformingValue: [d19970426t8 minute]
		should: [:r| r = 0].
!

testXmonth
	" Date and Time <DateAndTime> "
	self messageSelector: #'month'.

	self conformingValue: [d19970426t8 month]
		should: [:r| r = 4].
!

testXmonthAbbreviation
	" Date and Time <DateAndTime> "
	self messageSelector: #'monthAbbreviation'.
	"2000/12/08 Harmon, R. Need to accommidate locale differences."

	"Answer a <readableString> that is the abbreviation of the name of
	the month, in the local time of the receiver, which includes the
	receiver."
	self conformingValue: [d19970426t8 monthAbbreviation]
		should: [:r| r = 'Apr'].
!

testXmonthName
	" Date and Time <DateAndTime> "
	self messageSelector: #'monthName'.
	"2000/12/08 Harmon, R. Need to accommidate locale differences."

	"Answer a <readableString> that is the name of the month, in the
	local time of the receiver, which includes the receiver."
	self conformingValue: [d19970426t8 monthName]
		should: [:r| r = 'April'].
!

testXoffset
	" Date and Time <DateAndTime> "
	| d19970426t8Tmp off123 |
	self messageSelector: #'offset'.

	off123 := Duration days: 0 hours: 1 minutes: 2 seconds: 3.
	d19970426t8Tmp := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 0 minute: 0 second: 0
		offset: off123.
	self conformingValue: [d19970426t8Tmp offset]
		should: [:r| r = off123].
!

testXoffsetX
	" Date and Time <DateAndTime> "
	| dtTmp numList |
	self messageSelector: #'offset:'.

	"Answer a <DateAndTime> equivalent to the receiver but with its local
	time being offset from UTC by offset.  The impact of this on any other
	local time property is unspecified.
	Implementations may define a limit to the range of offset, but it must
	be at least -12:00:00 to 12:00:00 inclusive.
	It is an invariant that if x is a <Duration> in range then
	(<DateAndTime> offset: x) offset = x"
	numList := Array
		with: (Duration seconds: 2)
		with: (Duration seconds: 2.0d0)
		with: (Duration seconds: (1/2))
		with: (Duration seconds: (2.0d0 asScaledDecimal: 3)).
	dtTmp := DateAndTime
		year: 1997 month: 4 day: 26
		hour: 0 minute: 0 second: 0
		offset: Duration zero.
	numList do: [:numOffset|
		self conformingValue: [dtTmp offset: numOffset]
			should: [:r|
				r = (DateAndTime
						year: 1997 month: 4 day: 26
						hour: 0 minute: 0 second: 0
						offset: numOffset)
				and: [(dtTmp offset: numOffset) offset = numOffset]]].

	"Errors offset out of range."
	self should: [dtTmp offset: (Duration seconds: -2000000000)]
		raise: TestResult error.
!

testXprintString
	" Date and Time <DateAndTime> "
	self messageSelector: #'printString'.

	"The returned string will represent the UTC time of the receiver
	offset from UTC by the offset of the receiver. All dates are in the
	astronomical Gregorian calendar. The result will be formatted as
	-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z where
		[snipped]
	This format is based on ISO 8601 sections 5.3.3 and 5.4.1."

	self conformingValue: [(DateAndTime year: 1997 month: 4 day: 26
				hour: 1 minute: 2 second: 3
				offset: (Duration days: 0 hours: 1 minutes: 2 seconds: 3)                       ) printString]
		should: [:r| r = ' 1997-04-26T01:02:03+01:02:3'].

	"Example: 8:33:14.321 PM EST January 5, 1200 B.C. 
		'-1199-01-05T20:33:14.321-05:00'"
self anomaly: [
	self conformingValue: [(DateAndTime year: -1199 month: 1 day: 5
				hour: 20 minute: 33 second: 14
				offset: (Duration hours: -5 "EST")
			) printString]
		should: [:r| r = '-1199-01-05T20:33:14.321-05:00'].
] for: #(#'DO30' "2000/12/06 Harmon, R. r -> '-1199-01-05T20:33:14-05:00'").

	"Example: 12 midnight UTC January 1,  2001 A.D. 
			' 2001-01-01T00:00:00+00:00'"
	self conformingValue: [(DateAndTime year: 2001 month: 1 day: 1
				hour: 0 minute: 0 second: 0
				offset: Duration zero
			) printString]
		should: [:r| r = ' 2001-01-01T00:00:00+00:00'].
!

testXsecond
	" Date and Time <DateAndTime> "
	self messageSelector: #'second'.

	self conformingValue: [d19970426t8 second]
		should: [:r| r = 0].
!

testXsubtractOp
	" Date and Time <DateAndTime> "
	self messageSelector: #'-'.

	"If operand is a <DateAndTime>, answer a <Duration>."
	self conformingValue: [d19970426t8 - (DateAndTime
			year: 1997 month: 4 day: 26
			hour: 7 minute: 0 second: 0)]
		should: [:r|
			(r isKindOf: Duration)
			& r = (Duration days: 0 hours: 1 minutes: 0 seconds: 0)].

	self conformingValue: [d19970426t8 - (DateAndTime
			year: 1998 month: 4 day: 26
			hour: 7 minute: 0 second: 0)]
		should: [:r|
			(r isKindOf: Duration)
			& r = (Duration days: -364 hours: -23 minutes: 0 seconds: 0)].

	"If operand is a <Duration>, answer a new <DateAndTime>"
	self conformingValue: [d19970426t8 - (Duration
			days: 1 hours: 0 minutes: 0 seconds: 1)]
		should: [:r|
			(r isKindOf: DateAndTime)
			& r = (DateAndTime year: 1997 month: 4 day: 25
					hour: 7 minute: 59 second: 59)].

	self conformingValue: [d19970426t8 - (Duration
			days: -1 hours: 0 minutes: 0 seconds: -1)]
		should: [:r|
			(r isKindOf: DateAndTime)
			& r = (DateAndTime year: 1997 month: 4 day: 27
					hour: 8 minute: 0 second: 1)].
!

testXtimeZoneAbbreviation
	" Date and Time <DateAndTime> "
	| offTmp |
	self messageSelector: #'timeZoneAbbreviation'.
	"2000/12/08 Harmon, R. Need to accommidate locale differences."

	offTmp := Duration days: 0 hours: -5 minutes: 0 seconds: 0.

self anomaly: [
	self conformingValue: [(d19970426t8 offset: offTmp)
			timeZoneAbbreviation]
		should: [:r| r = 'EST'].
] for: #(#'DO30' "2000/12/06 Harmon, R. r -> 'CST'").

	"May fail because d19970426t7MST is Daylight Savings not Standard."
	offTmp := Duration days: 0 hours: -6 minutes: 0 seconds: 0.
	self conformingValue: [(d19970426t8 offset: offTmp)
		timeZoneAbbreviation]
		should: [:r| r = 'CST'].
!

testXtimeZoneName
	" Date and Time <DateAndTime> "
	| offTmp |
	self messageSelector: #'timeZoneName'.
	"2000/12/08 Harmon, R. Need to accommidate locale differences."

self anomaly: [
	offTmp := Duration days: 0 hours: -5 minutes: 0 seconds: 0.
	self conformingValue: [(d19970426t8 offset: offTmp) timeZoneName]
		should: [:r| r = 'EST'].
] for: #(#'DO30' "2000/12/06 Harmon, R. r -> 'Central Standard Time'").

	"May fail because d19970426t7MST is Daylight Savings not Standard."
	offTmp := Duration days: 0 hours: -6 minutes: 0 seconds: 0.
	self conformingValue: [(d19970426t8 offset: offTmp) timeZoneName]
		should: [:r| r = 'Central Standard Time'].
!

testXyear
	" Date and Time <DateAndTime> "
	self messageSelector: #'year'.

	self conformingValue: [d19970426t8 year]
		should: [:r| r = 1997].
! !

!DateAndTimeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/DateAndTimeTest.st,v 1.1 2002/06/21 10:31:47 penk Exp $'
! !
