"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#DurationTest
	instanceVariableNames:'off0123 smallInt2 largeNegInt2000000000 largePosInt2000000000
		float2 fractionHalf numList'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Date and Time'
!


!DurationTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Duration>'
! !

!DurationTest methodsFor:'constants'!

canonicalObject

	^Duration seconds: 500000000
!

magnitudeObjects

	^(Dictionary new)
		at: #'small' put: (Duration seconds: 100000000);
		at: #'medium' put: (Duration seconds: 500000000);
		at: #'large' put: (Duration seconds: 1000000000);
		yourself
! !

!DurationTest methodsFor:'others'!

setUp
	super setUp.
	off0123 := Duration days: 0 hours: 1 minutes: 2 seconds: 3.
	smallInt2 := 2.
	largeNegInt2000000000 := -2000000000.
	largePosInt2000000000 := 2000000000.
	float2 := 2.0d0.
	fractionHalf := 1/2.
	numList := Array new: 5.
	numList at: 1 put: smallInt2.
	numList at: 2 put: largeNegInt2000000000.
	numList at: 3 put: largePosInt2000000000.
	numList at: 4 put: float2.
	numList at: 5 put: fractionHalf.
! !

!DurationTest methodsFor:'testing'!

testXabs
	" Date and Time <Duration> "
	| absDuration |
	self messageSelector: #'abs'.

	absDuration := Duration days: 0 hours: 1 minutes: 2 seconds: 3.

	self conformingValue: [off0123 abs]
		should: [:r| r = absDuration].

	self conformingValue: [(Duration days: 0 hours: -1 minutes: -2
			seconds: -3) abs]
		should: [:r| r = absDuration].
!

testXaddOp
	" Date and Time <Duration> "
	self messageSelector: #'+'.

	self conformingValue: [off0123 + (Duration
			days: 1 hours: 1 minutes: 1 seconds: 1)]
		should: [:r| r = (Duration days: 1 hours: 2 minutes: 3 seconds: 4)].
!

testXasSeconds
	" Date and Time <Duration> "
	self messageSelector: #'asSeconds'.

	self conformingValue: [off0123 asSeconds]
		should: [:r| r = 3723].
!

testXdays
	" Date and Time <Duration> "
	self messageSelector: #'days'.

	self conformingValue: [off0123 days]
		should: [:r| r = 0].

	self conformingValue: [(Duration days: 9 hours: 0 minutes: 2
			seconds: 0) days]
		should: [:r| r = 9].

	self conformingValue: [(Duration days: -9 hours: -1 minutes: -2
			seconds: -3) days]
		should: [:r| r = -9].
!

testXdivideOp
	" Date and Time <Duration> "
	| retVals |
	self messageSelector: #'/'.

	"#( 2 -2000000000 2000000000 2.0d0 1/2 )"
	retVals := Array new: 5.
	retVals at: 1 put: (Duration days: 0 hours: 0 minutes: 31 seconds: 1.5d0).
	retVals at: 2 put: (Duration seconds: -01.8615d-6).
	retVals at: 3 put: (Duration seconds: 1.8615d-6).
	retVals at: 4 put: (Duration days: 0 hours: 0 minutes: 31 seconds: 1.5d0).
	retVals at: 5 put: (Duration days: 0 hours: 2 minutes: 4 seconds: 6).

	1 to: numList size do: [ :ndx |
		"If operand is a <number> answer a new <Duration>."
		self conformingValue: [off0123 / (numList at: ndx)]
			should: [:r|(r isKindOf: Duration)
						and: [r = (retVals at: ndx)]]].

	retVals at: 1 put: (3723/2).
	retVals at: 2 put: (-3723/2000000000).
	retVals at: 3 put: (3723/2000000000).
	retVals at: 4 put: 1.8615d3.
	retVals at: 5 put: 7.446d3.
	1 to: numList size do: [:ndx|
		"If operand is a <Duration> answer a <number>."
		self conformingValue: [
				off0123 / (Duration seconds: (numList at: ndx))]
			should: [:r| (r isKindOf: Number)
						and: [r = (retVals at: ndx)]]].

	"If operand is <Duration factory> #zero the ZeroDivide exception
	is signaled."
	self should: [off0123 / 0]
		raise: ZeroDivide.
	self should: [off0123 / (Duration zero)]
		raise: ZeroDivide.
!

testXequalityOp
	" Date and Time <Duration> "
	| tmpDuration |
	self messageSelector: #'='.

	tmpDuration := (Duration days: 0 hours: 1 minutes: 2 seconds: 3).
	self conformingValue: [off0123 = tmpDuration]
		should: [:r| r and: [off0123 hash = tmpDuration hash]].

	tmpDuration := (Duration days: 1 hours: 1 minutes: 1 seconds: 1).
	self conformingValue: [off0123 = tmpDuration]
		shouldnt: [:r| r or: [off0123 hash = tmpDuration hash]].
!

testXgreaterThanOp
	" Date and Time <Duration> "
	self messageSelector: #'>'.

	self conformingValue: [(Duration days: 1 hours: 1 minutes: 1
			seconds: 1) > off0123]
		should: [:r| r].

	self conformingValue: [(Duration days: 0 hours: 1 minutes: 2
				seconds: 3) > off0123]
		shouldnt: [:r| r].

	"It is erroneous if the receiver and operand are not comparable."
	self should: [off0123 > 0]
		raise: TestResult error.
!

testXhours
	" Date and Time <Duration> "
	self messageSelector: #'hours'.

	self conformingValue: [off0123 hours]
		should: [:r| r = 1].

	self conformingValue: [(Duration days: 9 hours: 0
			minutes: 2 seconds: 0) hours]
		should: [:r| r = 0].

	self conformingValue: [(Duration days: -9 hours: -1
			minutes: -2 seconds: -3) hours]
		should: [:r| r = -1].
!

testXlessThanOp
	" Date and Time <Duration> "
	self messageSelector: #'<'.

	self conformingValue: [off0123
			< (Duration days: 1 hours: 1 minutes: 1 seconds: 1)]
		should: [:r| r].

	self conformingValue: [off0123
				< (Duration days: 0 hours: 1 minutes: 2 seconds: 3)]
		shouldnt: [:r| r].

	self should: [off0123 < 0]
		raise: TestResult error.
!

testXminutes
	" Date and Time <Duration> "
	self messageSelector: #'minutes'.

	self conformingValue: [off0123 minutes]
		should: [:r| r = 2].

	self conformingValue: [(Duration days: 9 hours: 0
			minutes: 0 seconds: 0) minutes]
		should: [:r| r = 0].

	self conformingValue: [(Duration days: -9 hours: -1
			minutes: -2 seconds: -3) minutes]
		should: [:r| r = -2].
!

testXmultiplyOp
	" Date and Time <Duration> "
	| retVals |
	self messageSelector: #'*'.

	retVals := Array new: 5.                "#( 2 -2000000000 2000000000 2.0d0 1/2 )"
	retVals at: 1 put: (Duration days: 0 hours: 2 minutes: 4 seconds: 6).
	retVals at: 2 put: (Duration days: -86180555 hours: -13
								minutes: -20 seconds: 0).
	retVals at: 3 put: (Duration days: 86180555 hours: 13
								minutes:  20 seconds: 0).
	retVals at: 4 put: (Duration days: 0 hours: 2 minutes: 4 seconds: 6).
	retVals at: 5 put: (Duration days: 0 hours: 0 minutes: 31 seconds: 1.5d0).
        
	1 to: numList size do: [:ndx|
		self conformingValue: [off0123 * (numList at: ndx)]
			should: [:r| r = (retVals at: ndx)]     ].
!

testXnegated
	" Date and Time <Duration> "
	| negatedDuration rcvr |
	self messageSelector: #'negated'.

	negatedDuration := Duration days: 0 hours: -1 minutes: -2 seconds: -3.

	self conformingValue: [off0123 negated]
		should: [:r| r = negatedDuration].

	self conformingValue: [negatedDuration negated]
		should: [:r| r = off0123].

	self conformingValue: [Duration zero negated]
		should: [:r| r = (Duration zero)].
!

testXnegative
	" Date and Time <Duration> "
	self messageSelector: #'negative'.

	self conformingValue: [off0123 negative]
		shouldnt: [:r| r].

	self conformingValue: [(Duration days: 0 hours: -1
			minutes: -2 seconds: -3) negative]
		should: [:r| r].

	self conformingValue: [(Duration zero) negative]
		shouldnt: [:r| r].
!

testXpositive
	" Date and Time <Duration> "
	self messageSelector: #'positive'.

	self conformingValue: [off0123 positive]
		should: [:r| r].

	self conformingValue: [(Duration days: 0 hours: -1
			minutes: -2 seconds: -3) positive]
		shouldnt: [:r| r].

	self conformingValue: [(Duration zero) positive]
		should: [:r| r].
!

testXprintString
	" Date and Time <Duration> "
	self messageSelector: #'printString'.

	self conformingValue: [off0123 printString]
		should: [:r| r = '0:01:02:03'].

	self conformingValue: [(Duration days: 0 hours: -1
		minutes: -2 seconds: -3.5d0) printString]
		should: [:r| r = '-0:01:02:03.5'].

	self conformingValue: [(Duration zero) printString]
		should: [:r| r = '0:00:00:00'].

	"Check milliseconds printing and rounding:"
	self conformingValue: [(Duration seconds: (1/3)) printString]
		should: [:r| r = '0:00:00:00.333333'].

	self conformingValue: [(Duration seconds: (71/10000000)) printString]
		should: [:r| r = '0:00:00:00.000007'].

	self conformingValue: [(Duration seconds: (75/10000000)) printString]
		should: [:r| r = '0:00:00:00.000008'].

	self conformingValue: [(Duration seconds: 1.01d0) printString]
		should: [:r| r = '0:00:00:01.01'].
!

testXseconds
	" Date and Time <Duration> "
	self messageSelector: #'seconds'.

	self conformingValue: [off0123 seconds]
		should: [:r| r = 3].

	self conformingValue: [(Duration days: 9 hours: 0
		minutes: 0 seconds: 0) seconds]
		should: [:r| r = 0].

	self conformingValue: [(Duration days: -9 hours: -1 
			minutes: -2 seconds: -3.5d0) seconds]
		should: [:r| r = -3.5d0].
!

testXsubtractOp
	" Date and Time <Duration> "
	self messageSelector: #'-'.

	self conformingValue: [off0123 - (Duration days: 1 hours: 1
			minutes: 1 seconds: 1)]
		should: [:r| r = (Duration days: -0 hours: -23
			minutes:-58 seconds:-58)].
! !

!DurationTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/DurationTest.st,v 1.1 2002/06/21 10:22:45 penk Exp $'
! !
