"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#FileStreamFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-File Stream'
!


!FileStreamFactoryTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<FileStream factory>'
! !

!FileStreamFactoryTest methodsFor:'constants'!

canonicalObject

	^FileStream
!

fileNameXMustNotExistXjnk
	"Answer a <String> fileID, an external file identifier.
	The syntax of the fileID string is implementation defined."

	^'TestMustNotExist.jnk' copy
!

sampleText

	^'123 abc ABC %$#'
! !

!FileStreamFactoryTest methodsFor:'helpers'!

createTestFileWithContent
	"Create a test file with sample content."
	| fileCreated rStream |
	fileCreated := false.
	[rStream := FileStream write: self fileNameXTestFileXjnk.
	rStream nextPutAll: self sampleText.
	rStream close.
	fileCreated := true.
	] ensure: [
		rStream isNil ifFalse: [rStream close].
		^fileCreated].
!

doTestOfReadType: aType contentTest: testBlk
	"Test #'read:type:'."
	| rStream |
	[self conformingValue: [
			rStream := FileStream
				read: self fileNameXTestFileXjnk
				type: aType]
		should: [:r|
			"future sequence values initially consist of the elements
			of the external file [and] [t]he ordering of the sequence
			values is the same as the ordering within the external file"
			(testBlk value: rStream contents)

			"initially has no past sequence values"
			& rStream position = 0

			"value of fileType determines the external stream type
			and sequence value type of the result object."
			& rStream externalType = aType]

	] ensure: [
		rStream close].
!

is: fileStream1 equivalentTo: fileStream2

	fileStream1 == fileStream2 ifTrue: [
		^true].
	fileStream1 class == fileStream2 class ifTrue: [
		fileStream1 contents = fileStream2 contents ifTrue: [
			fileStream1 externalType = fileStream2 externalType ifTrue: [
				^true]]].
	^false

! !

!FileStreamFactoryTest methodsFor:'testing'!

testXreadX
	" File Stream <FileStream factory> "
	| rStream secStream |
	self messageSelector: #'read:'.

	"The result is the same as if the message #read:type: was sent
	to the receiver with fileId as the first argument and the symbol
	#'text' as the second argument."
	self assert: (self createTestFileWithContent).
	[self conformingValue: [
			rStream := FileStream read: self fileNameXTestFileXjnk]
		should: [:r|
			secStream := FileStream
				read: self fileNameXTestFileXjnk
				type: #'text'.
			self is: rStream equivalentTo: secStream].
	] ensure: [
		rStream close.
		secStream close].


	"Errors: As defined by <FileStream factory> #read:type:
		#read:type: Errors:
			It is an error if the file does not exist, or if the user
			does not have read access to the file."
	rStream := nil.
	[self should: [
			rStream := FileStream read: self fileNameXMustNotExistXjnk]
		raise: Error.
	] ensure: [
		rStream isNil ifFalse: [rStream close]].
!

testXreadXtypeX
	" File Stream <FileStream factory> "
	| rStream secStream |
	self messageSelector: #'read:type:'.

	self assert: (self createTestFileWithContent).
	self doTestOfReadType: #'binary'
		contentTest: [:strmContent|
			strmContent = (self sampleText asByteArray)].
	self doTestOfReadType: #'text'
		contentTest: [:strmContent| strmContent = self sampleText].

	"Errors: As defined by <FileStream factory> #read:type:
		#read:type: Errors:
			It is an error if the file does not exist, or if the user
			does not have read access to the file."
	self should: [
			rStream := FileStream read: self fileNameXMustNotExistXjnk]
		raise: Error.
!

testXwriteX
	" File Stream <FileStream factory> "
	| rStream secStream |
	self messageSelector: #'write:'.

	"The result is the same as if the message #write:mode:check:type:
	was sent to the receiver with fileId as the first argument,
	#'create' as the second argument, false as the third argument,
	and the symbol #'text' as the fourth argument."
	self assert: (self createTestFileWithContent).
	[self conformingValue: [
			rStream := FileStream write: self fileNameXTestFileXjnk]
		should: [:r|
			secStream := FileStream
				write: self fileNameXTestFileXjnk
				mode: #'create'
				check: false
				type: #'text'.
			self is: rStream equivalentTo: secStream].
	] ensure: [
		rStream close.
		secStream close].


	"Errors: As defined by <FileStream factory> #write:mode:check:type:
		#write:mode:check:type: Errors:"
	"N/A: If mode = #create and check = true and the file exists."
	"N/A: If mode = #append and check = true and the file does not exist."
	"N/A: If mode = #truncate and check = true and the file does not exist."
self notDone. "If the user does not have write permissions for the file."
self notDone. "If the user does not have creation permissions for a file
		that is to be created."
!

testXwriteXmodeX
	" File Stream <FileStream factory> "
	| rStream secStream |
	self messageSelector: #'write:mode:'.

	"The result is the same as if the message #write:mode:check:type:
	was sent to the receiver with fileId as the first argument,
	mode as the second argument, false as the third argument,
	and the symbol #'text' as the fourth argument."
	self assert: (self createTestFileWithContent).
self anomaly: [
	#( #'create' #'append' #'truncate') do: [:tstMode|
		[self conformingValue: [
				rStream := FileStream
					write: self fileNameXTestFileXjnk
					mode: tstMode]
			should: [:r|
				secStream := FileStream
					write: self fileNameXTestFileXjnk
					mode: tstMode
					check: false
					type: #'text'.
				self is: rStream equivalentTo: secStream].
		] ensure: [
			rStream close.
			secStream close]].
] for: #(#'DO30' "2001/02/20 Harmon, R. For #'append' contents returns only past (up to position) ignoring future.").



	"Errors: As defined by <FileStream factory> #write:mode:check:type:
		#write:mode:check:type: Errors:"
	"N/A: If mode = #create and check = true and the file exists."
	"N/A: If mode = #append and check = true and the file does not exist."
	"N/A: If mode = #truncate and check = true and the file does not exist."
self notDone. "If the user does not have write permissions for the file."
self notDone. "If the user does not have creation permissions for a file
		that is to be created."
!

testXwriteXmodeXcheckXtypeX
	" File Stream <FileStream factory> "
	| rStream |
	self messageSelector: #'write:mode:check:type:'.

	self notDone. "This should probably check the various combinations."

self anomaly: [
	[self conformingValue: [
			rStream := FileStream
				write: self fileNameXTestFileXjnk
				mode: #'create'
				check: false
				type: #'text']
		should: [:r| true].
	] ensure: [
		rStream close.].


	"Errors: As defined by <FileStream factory> #write:mode:check:type:
		#write:mode:check:type: Errors:"

	"If mode = #create and check = true and the file exists."
	rStream := nil.
	self assert: (self createTestFileWithContent).
	[self should: [
			rStream := FileStream
				write: self fileNameXTestFileXjnk
				mode: #'create'
				check: true
				type: #'text']
		raise: Error.
	] ensure: [
		rStream isNil ifFalse: [rStream close]].

	"If mode = #append and check = true and the file does not exist."
	rStream := nil.
	[self should: [
			rStream := FileStream
				write: self fileNameXMustNotExistXjnk
				mode: #'append'
				check: true
				type: #'text']
		raise: Error.
	] ensure: [
		rStream isNil ifFalse: [rStream close]].

	"If mode = #truncate and check = true and the file does not exist."
	rStream := nil.
	[self should: [
			rStream := FileStream
				write: self fileNameXMustNotExistXjnk
				mode: #'truncate'
				check: true
				type: #'text']
		raise: Error.
	] ensure: [
		rStream isNil ifFalse: [rStream close]].
] for: #(#'SQ27' "2001/04/21 Harmon, R. error - FileStream >> #write:mode:check:type: returns nil").

self notDone. "If the user does not have write permissions for the file."
self notDone. "If the user does not have creation permissions for a file
		that is to be created."


"Definition: <FileStream factory>

Depending upon the values of check and mode, either create a new external file or locate an existing external file that is identified by the value of fileID. The syntax of the fileID string is implementation defined.
Return an object conforming to <writeFileStream>. The external file serves as the stream backing store of the returned object. The returned object is a write-back stream. The value of fileType determines the external stream type and sequence value type of the result object. 

	Valid values for mode are: #'create', #'append', and #'truncate'. The meaning of these values are:

		#'create'       create a new file, with initial position at the beginning.
		#'append'       use an existing file, with initial position at its end.
		#'truncate'     use an existing file, initially truncating it.

	The value of mode determines the initial state of the past sequence values and future sequence values of the result object. If mode is #'create' or #'truncate' the past sequence values and future sequence values are both initially empty. If mode is #'append' the past sequence values initially consist of the elements of the external file and future sequence values is initially empty. The ordering of the sequence values is the same as the ordering within the external file.
The check flag determines whether the file specified by fileID must exist or not exist.
If mode = #'create' and check = false and the file exists, then the existing file is used.
If mode = #'append' and check = false and the file does not exist, then it is created.
If mode = #'truncate' and check = false and the file does not exist, then it is created. 
This operation is undefined if a value other than #'create', #'append' or #'truncate' is used as the mode argument.


From <FileStream> Description:

	The external stream type defines the data translation and the sequence value type for the stream. External stream types are specified using <symbol> objects. The standard defines the following external stream types:

		#'binary'       The external data is treated as sequence of 8-bit bytes. The sequence value type is <integer> with values restricted to the range 0 to 255.

		#'text'         The external data is treated as a sequenced of 8-bit characters encoded using an implementation defined external character set.  The sequence value type is <Character> restricted to those specific characters that may be represented in the external character set.

	Implementations may define other external stream types."
! !

!FileStreamFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/FileStreamFactoryTest.st,v 1.1 2002/06/21 10:38:52 penk Exp $'
! !
