"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#FloatCharacterizationTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Numeric'
!


!FloatCharacterizationTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<floatCharacterization>'
! !

!FloatCharacterizationTest methodsFor:'constants'!

canonicalObject

	^Float
! !

!FloatCharacterizationTest methodsFor:'testing'!

testXdenormalized
	" Numeric <floatCharacterization> "
	self messageSelector: #'denormalized'.

	self conformingValue: [self canonicalObject denormalized]
		should: [:r| true "any boolean is valid" ].
!

testXe
	" Numeric <floatCharacterization> "
	self messageSelector: #'e'.

	self conformingValue: [self canonicalObject e]
		should: [:r| r closeTo: 1 exp].
!

testXemax
	" Numeric <floatCharacterization> "
	self messageSelector: #'emax'.

	"we think it's 308, but refer to ISO/IEC 10967 which we don't have"
	self conformingValue: [self canonicalObject emax]
		should: [:r| true].
!

testXemin
	" Numeric <floatCharacterization> "
	self messageSelector: #'emin'.

	"Can be any implem. defined, but refer to ISO/IEC 10967 which we
	don't have"
	self conformingValue: [self canonicalObject emin]
		should: [:r| true].
!

testXepsilon
	" Numeric <floatCharacterization> "
	| canonicalObject |
	self messageSelector: #'epsilon'.

	"refer to ISO/IEC 10967 which we don't have.
	The return value is specified as:
		self radix raisedTo: (1 - self precision)"
	canonicalObject := self canonicalObject.
	self conformingValue: [canonicalObject epsilon]
		should: [:r|
			r = (canonicalObject radix raisedTo: (1 - canonicalObject precision))].
!

testXfmax
	" Numeric <floatCharacterization> "
	| canonicalObject |
	self messageSelector: #'fmax'.

	"Refer to ISO/IEC 10967 which we don't have."
	canonicalObject := self canonicalObject.
	self conformingValue: [canonicalObject fmax]
		should:         [:r|
			r = (1 - (canonicalObject radix raisedTo: canonicalObject precision negated) * (canonicalObject radix raisedTo: canonicalObject emax))].
!

testXfmin
	" Numeric <floatCharacterization> "
	| canonicalObject |
	self messageSelector: #'fmin'.

	"Refer to ISO/IEC 10967 which we don't have."
	canonicalObject := self canonicalObject.
	self conformingValue: [canonicalObject fmin]
		should:         [:r|
			canonicalObject denormalized
				ifTrue: [r = canonicalObject fminDenormalized]
				ifFalse: [r = canonicalObject fminNormalized]].
!

testXfminDenormalized
	" Numeric <floatCharacterization> "
	| canonicalObject |
	self messageSelector: #'fminDenormalized'.

	"Refer to ISO/IEC 10967 which we don't have."
	canonicalObject := self canonicalObject.
self anomaly: [
	self conformingValue: [canonicalObject fminDenormalized]
		should: [:r|
			canonicalObject denormalized
				ifTrue: [r = (canonicalObject radix raisedTo:                                                   (canonicalObject emin
								- canonicalObject precision))]
				ifFalse: [ true "The result is unspecified
					if denormalized values are not allowed
					by the characterized representation."]].
] for: #(#'DO30' "2000/12/06 Harmon, R. overflow error.").

!

testXfminNormalized
	" Numeric <floatCharacterization> "
	| canonicalObject |
	self messageSelector: #'fminNormalized'.

	"Refer to ISO/IEC 10967 which we don't have.  

Harmon, R. - Draft specifies:
Definition: <floatCharacterization>
Report the minimum normalized value allowed by the characterized floating point object representation. This satisfies the ISO/IEC 10967 floating point characterization requirement fminN, and is equal to
self radix raisedTo: (self emin - 1).

	From Numerical Computation Guide: 2 - IEEE Arithmetic
	min. pos. normal number: 00100000 00000000      2.2250738585072014e-308 
         
	(Float radix raisedTo: (Float emin + 1)) asFloat -> 2.2250738585072e-308 
         
	Changing - 1 to + 1 agrees with Guide.  I will use plus as it appears
	to be correct."
	canonicalObject := self canonicalObject.
	self conformingValue: [canonicalObject fminNormalized]
		should: [:r|
			r = (canonicalObject radix raisedTo: canonicalObject emin + 1)].
!

testXpi
	" Numeric <floatCharacterization> "
	| isoPi |
	self messageSelector: #'pi'.

	"Refer to ISO/IEC 10967 for the value of pi; using made-up isoPi here"
	isoPi := 3.141592653589793d0.
	self conformingValue: [self canonicalObject pi]
		should: [:r| r closeTo: isoPi].

!

testXprecision
	" Numeric <floatCharacterization> "
	self messageSelector: #'precision'.

	"Refer to ISO/IEC 10967 for the required precision."
	self conformingValue: [self canonicalObject precision]
		should: [:r | r >= 2].
!

testXradix
	" Numeric <floatCharacterization> "
	self messageSelector: #'radix'.

	"Refer to ISO/IEC 10967 for the required radix."
	self conformingValue: [self canonicalObject radix]
		should: [:r | r even & r >= 2].
! !

!FloatCharacterizationTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/FloatCharacterizationTest.st,v 1.1 2002/06/21 10:32:45 penk Exp $'
! !
