"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#GettableStreamHelper
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Stream'
!


!GettableStreamHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<gettableStream>'
! !

!GettableStreamHelper methodsFor:'testing'!

testXatEnd
	" Stream <gettableStream> "
	| gettableStream |
	self messageSelector: #'atEnd'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	"Initially has stuff:"
	self conformingValue: [gettableStream atEnd]
		shouldnt: [:r| r].

	gettableStream setToEnd.
	self conformingValue: [gettableStream atEnd]
		should: [:r| r].
!

testXdoX
	" Stream <gettableStream> "
	| gettableStream list list2 |
	self messageSelector: #'do:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	list := OrderedCollection new.
	self conformingValue: [gettableStream do: [:elem| list add: elem]]
		should: [:r| list asArray = testCase canonicalContents asArray].

	gettableStream setToEnd.
	list2 := OrderedCollection new.
	self conformingValue: [gettableStream do: [:elem| list2 add: elem]]
		should: [:r| list2 isEmpty].
!

testXnext
	" Stream <gettableStream> "
	self messageSelector: #'next'.

	self conformingValue: [testCase canonicalObject next]
		should: [:r| r = $t].
!

testXnextLine
	" Stream <gettableStream> "
	| contents ndx line |
	self messageSelector: #'nextLine'.

	contents := testCase canonicalContents.
	ndx := contents
		indexOfSubCollection: self lineDelimiter
		startingAt: 1.
	line := contents copyFrom: 1 to: (ndx - 1).

	self conformingValue: [testCase  canonicalObject nextLine]
		should: [:r| r = line].

	"Errors If any of the future sequence values to be returned
	do not conform to the protocol <Character>."
!

testXnextMatchForX
	" Stream <gettableStream> "
	| gettableStream |
	self messageSelector: #'nextMatchFor:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	self conformingValue: [gettableStream nextMatchFor: $t]
		should: [:r| r].

	self conformingValue: [gettableStream nextMatchFor: $t]
		shouldnt: [:r| r].
!

testXnextX
	" Stream <gettableStream> "
	self messageSelector: #'next:'.

	self conformingValue: [testCase canonicalObject next: 3]
		should: [:r| r asArray = 'thi' asArray].
	"If amount is equal to 0 an empty collection is returned."
	self conformingValue: [testCase canonicalObject next: 0]
		should: [:r| r isEmpty].
!

testXpeek
	" Stream <gettableStream> "
	| gettableStream pos |
	self messageSelector: #'peek'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	"Returns the next object in the receiver's future sequence values
	without advancing the receiver's position."
	self conformingValue: [gettableStream peek]
		should: [:r| r = $t & gettableStream position = 0].

	"Returns nil if the receiver is at end of stream."
	gettableStream setToEnd.
	pos := gettableStream position.
	self conformingValue: [gettableStream peek]
		should: [:r| r = nil & pos = gettableStream position].
!

testXpeekForX
	" Stream <gettableStream> "
	| gettableStream |
	self messageSelector: #'peekFor:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	self conformingValue: [gettableStream peekFor: $t]
		should: [:r| r].

	self conformingValue: [gettableStream peekFor: $X]
		shouldnt: [:r| r].
!

testXskipToX
	" Stream <gettableStream> "
	| gettableStream |
	self messageSelector: #'skipTo:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	self conformingValue: [gettableStream skipTo: $s]
		should: [:r| r & gettableStream position = 4].

	self conformingValue: [gettableStream skipTo: $X]
		should: [:r| r = false & gettableStream atEnd].
!

testXskipX
	" Stream <gettableStream> "
	| gettableStream |
	self messageSelector: #'skip:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	self conformingValue: [gettableStream skip: 3]
		should: [:r| gettableStream next = $s].

self anomaly: [
	"A number of objects equal to the lesser of amount and the size of
	the receiver's future sequence values are removed from the receiver's
	future sequence values and appended, in order, to the end of the
	receiver's past sequence values."
	gettableStream position: (gettableStream contents size - 1).
	self conformingValue: [gettableStream skip: 3]
		should: [:r | gettableStream atEnd].
] for: #(#'DO30' "2000/12/06 Harmon, R. ?not at end?"
	#'SQ27' "2000/12/06 Harmon, R. ?attempt set pos. of PosStrm?").
!

testXupToX
	" Stream <gettableStream> "
	| gettableStream remainder contents |
	self messageSelector: #'upTo:'.

	gettableStream := testCase canonicalObject.
	gettableStream reset.

	self conformingValue: [gettableStream upTo: $ ]
		should: [:r| r = 'this' & gettableStream next = $i].

	contents := testCase canonicalContents.
	remainder := contents copyFrom: 7 to: contents size.
	self conformingValue: [gettableStream upTo: $X]
		should: [:r| r = remainder & gettableStream atEnd].

	self conformingValue: [gettableStream upTo: $a]
		should: [:r | r isEmpty].
! !

!GettableStreamHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/GettableStreamHelper.st,v 1.1 2002/06/21 10:24:52 penk Exp $'
! !
