"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#InstanceExtraInformation
	instanceVariableNames:'extraInfo'
	classVariableNames:''
	poolDictionaries:''
	category:'System-PortableUseful'
!

InstanceExtraInformation comment:'The singleton instance of this class manages extra information for instances of <Object>.

	It is especially useful as a substitute to adding an instance variable to a class.  Take my word for it.  This can be as dangerous and error prone as doing brain surgery on yourself especially if the class is Objector Class.

	As an example one may wish to mark a class as changed if any aspect is changed such as adding a method, or changing its comment, etc.

	A straight forward way is to add an instance variable ''changed'' to Behavior, in the superclass hierarchy of both Class and Metaclass if they are present in your dialect (ANSI does not require there be classes).

	Now deal with all the existing instances.  Whoops.  Getting very dangerous.

	An easier way is to use instance extra information:



Behavior Method

isChanged

	"Answer true if the receiver changed."



	^PU

		extraInfoFor: self

		at: #changed

		ifAbsent: [false]



Behavior Method

isChanged: aBoolean 

	"Set the changed flag for the receiver to aBoolean"



	aBoolean

		ifTrue: [PU

				extraInfoFor: self

				at: #changed

				put: true]

		ifFalse: [PU

				removeExtraInfoFor: self

				at: #changed

				ifAbsent: []]



Class method

comment: aString

	comment := aString.

	self isChanged: true.'
!


!InstanceExtraInformation class methodsFor:'instance creation'!

new

	"Answers an instance of the receiver."



	^super new initialize
! !

!InstanceExtraInformation methodsFor:'accessing'!

extraInfoFor: instance at: extraKey 

	"Answer instance's <Object> extra information at extraKey 

	<Symbol>, or if not found signal an error."



	^self

		extraInfoFor: instance

		at: extraKey

		ifAbsent: [(Smalltalk at: #PU)

				errorNotFoundTag: extraKey in: instance]
!

extraInfoFor: instance at: extraKey ifAbsent: absentBlock 

	"Answer instance's <Object> extra information at extraKey 

	<Symbol>, or the result of evaluating absentBlock if not found."



	| instanceExtraInfo |

	instanceExtraInfo := extraInfo at: instance ifAbsent: [^absentBlock value].

	^instanceExtraInfo at: extraKey ifAbsent: absentBlock
!

extraInfoFor: instance at: extraKey put: extraValue 

	"Set instance's <Object> extra information at extraKey <Symbol> 

	to extraValue <Object>, and answer extraValue."



	| instanceExtraInfo |

	instanceExtraInfo := extraInfo at: instance ifAbsentPut: [IdentityDictionary new].

	^instanceExtraInfo at: extraKey put: extraValue
!

extraInfoFor: instance ifAbsent: absentBlock 

	"Answer instance's <Object> extra information, or the result of 

	evaluating absentBlock if not found."



	^extraInfo at: instance ifAbsent: absentBlock
! !

!InstanceExtraInformation methodsFor:'initializing'!

initialize

        "Private - Initialize the receiver."

        "IdentityDictionary"



        extraInfo := WeakIdentityDictionary new
! !

!InstanceExtraInformation methodsFor:'removing'!

removeAllExtraInfoFor: instance 

	"Remove all instance's <Object> extra information."



	extraInfo removeKey: instance ifAbsent: []
!

removeExtraInfoFor: instance at: extraKey 

	"Remove instance's <Object> extra information at extraKey 

	<Symbol>, or if not found signal an error."



	^self

		removeExtraInfoFor: instance

		at: extraKey

		ifAbsent: [(Smalltalk at: #PU)

				errorNotFoundTag: extraKey in: instance]
!

removeExtraInfoFor: instance at: extraKey ifAbsent: absentBlock 

	"Remove instance's <Object> extra information at extraKey 

	<Symbol>, or the result of evaluating absentBlock if not found."



	| instanceExtraInfo |

	instanceExtraInfo := extraInfo at: instance ifAbsent: [^absentBlock value].

	instanceExtraInfo removeKey: extraKey ifAbsent: absentBlock
! !

!InstanceExtraInformation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/InstanceExtraInformation.st,v 1.1 2002/06/21 10:30:44 penk Exp $'
! !
