"{ Package: 'stx:goodies/ansiTests/common' }"

CollectionTest subclass:#IntervalTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!IntervalTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^self error: 'Has no restrictions.'.    "DO30"
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Interval>'
! !

!IntervalTest class methodsFor:'public'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^false  "Has no restrictions: DO30"
! !

!IntervalTest methodsFor:'constants'!

canonicalElement

	^2
!

canonicalObject

	^1 to: 4
!

emptyCollection

	^1 to: 0
! !

!IntervalTest methodsFor:'testing'!

testXcollectX
	" Collection <Interval> "
	| object |
	self messageSelector: #'collect:'.

	object := self canonicalObject.
	self conformingValue: [object collect: [:each| each > 0]]
		should: [:r| r size = 4].

	self should: [object collect: [:each| each, 'string']]
		raise: TestResult error.
!

testXconcatenateOp
	" Collection <Interval> "
	| object  |
	self messageSelector: #','.

	object := self canonicalObject.
	self conformingValue: [object, object]
		should: [:r| r size = 8].
!

testXcopyFromXtoX
	" Collection <Interval> "
	| object |
	self messageSelector: #'copyFrom:to:'.

	object := self canonicalObject.
	self conformingValue: [object copyFrom: 1 to: 3]
		should: [:r| r size = 3].

	self conformingValue: [object copyFrom: 3 to: 1]
		should: [:r| r size = 0].

	"Error If start < 1 or start > self size."
	self should: [object copyFrom: 0 to: 3]
		raise: TestResult error.

	"Error If start < 1 or start > self size.
		If stop < 1 or stop > self size."
	self should: [object copyFrom: 5 to: 5]
		raise: TestResult error.

	"Error If stop < 1 or stop > self size."
	self should: [object copyFrom: 1 to: 0]
		raise: TestResult error.
	self should: [object copyFrom: 1 to: 5]
		raise: TestResult error.
!

testXcopyReplaceAllXwithX
	" Collection <Interval> "
	| object targetElements replacementElements |
	self messageSelector: #'copyReplaceAll:with:'.


	object := self canonicalObject.
	targetElements := #(2).
	replacementElements := #(9).
	self conformingValue: [object
			copyReplaceAll: targetElements
			with: replacementElements]
		should: [:r| (r at: 2) = 9].

	replacementElements := $a.
	self should: [object
			copyReplaceAll: targetElements
			with: replacementElements]
		raise: TestResult error.
!

testXcopyReplaceFromXtoXwithObjectX
	" Collection <Interval> "
	| object replacementElement |
	self messageSelector: #'copyReplaceFrom:to:withObject:'.

	object := self canonicalObject.
	replacementElement := 9.

	self conformingValue: [object
			copyReplaceFrom: 1
			to: 2
			withObject: replacementElement]
		should: [:r| (r at: 1) = 9].
        
	self conformingValue: [object
			copyReplaceFrom: 1
			to: 0
			withObject: replacementElement]
		should: [:r| (r at: 1) = 9].
        
	self conformingValue: [object
			copyReplaceFrom: 5
			to: 4
			withObject: replacementElement]
		should: [:r| (r at: r size) = 9].
!

testXcopyReplaceFromXtoXwithX
	" Collection <Interval> "
	| object replacementElements |
	self messageSelector: #'copyReplaceFrom:to:with:'.

	object := self canonicalObject.
	replacementElements := #(9).

	self conformingValue: [object
			copyReplaceFrom: 1
			to: 2
			with: replacementElements]
		should: [:r| (r at: 1) = 9].
        
	self conformingValue: [object
			copyReplaceFrom: 1
			to: 0
			with: replacementElements]
		should: [:r| (r at: 1) = 9].
        
	self conformingValue: [object
			copyReplaceFrom: 5
			to: 4
			with: replacementElements]
		should: [:r| (r at: r size) = 9].

	"replacementElements is not a collection."      
	replacementElements := $A.
	self should: [object
			copyReplaceFrom: 1
			to: 2
			with: replacementElements]
		raise: TestResult error.

	"Errors The elements in replacementElements are not suitable for storage in instances of the result."   
!

testXcopyReplacingXwithObjectX
	" Collection <Interval> "
	self messageSelector: #'copyReplacing:withObject:'.

	self conformingValue: [self canonicalObject
			copyReplacing: 3
			withObject: 2]
		should: [:r| (r at: 3) = 2].
!

testXcopyWithX
	" Collection <Interval> "
	| newElement |
	self messageSelector: #'copyWith:'.

	newElement := 9.
	self conformingValue: [self canonicalObject copyWith: newElement]
		should: [:r| r size = 5 and:[(r at: 5) = newElement]].
!

testXcopyWithoutX
	" Collection <Interval> "
	self messageSelector: #'copyWithout:'.

	self conformingValue: [self canonicalObject copyWithout: 2]
		should: [:r| r size = 3 and:[(r at: 2) = 3]].
!

testXrejectX
	" Collection <Interval> "
	| object |
	self messageSelector: #'reject:'.

	object := self canonicalObject.
	self conformingValue: [object reject: [:each| each > 0]]
		should: [:r| r isEmpty].

	"Errors If discriminator evaluates to an object that does not conform
			to the protocol <boolean> for any element of the receiver."
	self should: [object reject: [:each| each, 'string']]
		raise: TestResult error.
	self should: [object reject: [:each| 'string']]
		raise: TestResult error

	"Errors If the elements of the receiver are inappropriate for use as arguments to discriminator."
!

testXreverse
	" Collection <Interval> "
	| object |
	self messageSelector: #'reverse'.

	object := self canonicalObject.
	self conformingValue: [object reverse]
		should: [:r| r size = 4
			and: [(r at: 1) = (object at: 4)
				& ((r at: 2) = (object at: 3))
				& ((r at: 3) = (object at: 2))
				& ((r at: 4) = (object at: 1))]]
!

testXselectX
	" Collection <Interval> "
	| object |
	self messageSelector: #'select:'.

	object := self canonicalObject.
	self conformingValue: [object select: [:each| each > 0]]
		should: [:r| r size = 4].

	"Errors If discriminator evaluates to an object that does not conform
			to the protocol <boolean> for any element of the receiver."
	self should: [object select: [:each| each, 'string']]
		raise: TestResult error.
	self should: [object select: [:each| 'string']]
		raise: TestResult error

	"Errors If the elements of the receiver are inappropriate for use
			as arguments to discriminator."
! !

!IntervalTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/IntervalTest.st,v 1.1 2002/06/21 10:25:32 penk Exp $'
! !
