"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#MagnitudeHelper
	instanceVariableNames:'comparableNumberMagnitudes comparableReadableStringMagnitudes'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Numeric'
!


!MagnitudeHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<magnitude>'
! !

!MagnitudeHelper methodsFor:'helpers'!

createComparableNumberMagnitudes
	"<Float>, <Fraction>, <integer>, & <scaledDecimal>."

	comparableNumberMagnitudes := (Array
		with: FloatTest
		with: FractionTest
		with: IntegerTest
		with: ScaledDecimalTest
	) collect: [:mainTestCls| mainTestCls magnitudeObjects].
!

createComparableReadableStringMagnitudes
	"<string> & <symbol>."

	comparableReadableStringMagnitudes := (Array
		with: StringTest
		with: SymbolTest
	) collect: [:mainTestCls| mainTestCls magnitudeObjects].
!

setUp

	super setUp.
	self createComparableNumberMagnitudes.
	self createComparableReadableStringMagnitudes.
! !

!MagnitudeHelper methodsFor:'testing'!

testXbetweenXandX
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'between:and:'.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [medium between: small and: large]
			should: [:r| r].
	self conformingValue: [medium between: medium and: large]
		should: [:r| r].
	self conformingValue: [medium between: small and: medium]
		should: [:r| r].
	self conformingValue: [medium between: medium and: medium]
		should: [:r| r].

	"Test min & max reversed -> false:"
	self conformingValue: [medium between: large and: small]
		shouldnt: [:r| r].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [medium between: small and: rsLarge]
				should: [:r| r].
			self conformingValue: [medium between: rsLarge and: rsSmall]
				shouldnt: [:r| r]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].

	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
		self conformingValue: [medium between: small and: numLarge]
			should: [:r| r].
		self conformingValue: [medium between: numLarge and: numSmall]
			shouldnt: [:r| r]].

	self conformingValue: [2.0d0 between: 3 and: 1]
		shouldnt: [:r| r].
!

testXgreaterThanOp
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'>'.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [medium > small] should: [:r| r].
	self conformingValue: [large > medium] should: [:r| r].
	self conformingValue: [small > large] shouldnt: [:r| r].
	self conformingValue: [large > large] shouldnt: [:r| r].
	self conformingValue: [small > small] shouldnt: [:r| r].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [medium > rsSmall] should: [:r| r].
			self conformingValue: [rsLarge > medium] should: [:r| r].
			self conformingValue: [rsSmall > medium] shouldnt: [:r| r]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].
	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
		self conformingValue: [medium > numSmall] should: [:r| r].
		self conformingValue: [numLarge > medium] should: [:r| r].
		self conformingValue: [numSmall > medium] shouldnt: [:r| r]].
!

testXgreaterThanOrEqualToOp
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'>='.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [medium >= small] should: [:r| r].
	self conformingValue: [large >= medium] should: [:r| r].
	self conformingValue: [medium >= medium] should: [:r| r].
	self conformingValue: [large >= small] should: [:r| r].
	self conformingValue: [small >= small] should: [:r| r].

	self conformingValue: [small >= medium] shouldnt: [:r| r].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [medium >= rsSmall] should: [:r| r].
			self conformingValue: [rsLarge >= medium] should: [:r| r].
			self conformingValue: [rsSmall >= medium] shouldnt: [:r| r]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].
	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
		self conformingValue: [medium >= numSmall] should: [:r| r].
		self conformingValue: [numLarge >= medium] should: [:r| r].
		self conformingValue: [numSmall >= medium] shouldnt: [:r| r]].

!

testXlessThanOp
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'<'.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [small < medium] should: [:r| r].
	self conformingValue: [medium < large] should: [:r| r].
	self conformingValue: [large < small] shouldnt: [:r| r].
	self conformingValue: [large < large] shouldnt: [:r| r].
	self conformingValue: [small < small] shouldnt: [:r| r].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [rsSmall < medium] should: [:r| r].
			self conformingValue: [medium < rsLarge] should: [:r| r].
			self conformingValue: [medium < rsSmall] shouldnt: [:r| r]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].
	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
		self conformingValue: [numSmall < medium] should: [:r| r].
		self conformingValue: [medium < numLarge] should: [:r| r].
		self conformingValue: [medium < numSmall] shouldnt: [:r| r]].
!

testXlessThanOrEqualToOp
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'<='.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [small <= medium] should: [:r| r].
	self conformingValue: [medium <= large] should: [:r| r].
	self conformingValue: [large <= small] shouldnt: [:r| r].
	self conformingValue: [large <= large] should: [:r| r].
	self conformingValue: [small <= small] should: [:r| r].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [rsSmall <= medium] should: [:r| r].
			self conformingValue: [medium <= rsLarge] should: [:r| r].
			self conformingValue: [medium <= rsSmall] shouldnt: [:r| r]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].
	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
		self conformingValue: [numSmall <= medium] should: [:r| r].
		self conformingValue: [medium <= numLarge] should: [:r| r].
		self conformingValue: [medium <= numSmall] shouldnt: [:r| r]].
!

testXmaxX
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'max:'.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [small max: medium] should: [:r| r == medium].
	self conformingValue: [medium max: large] should: [:r| r == large].
	self conformingValue: [large max: small] should: [:r| r == large].
self anomaly: [
	self conformingValue: [large copy max: large] should: [:r| r == large].
	self conformingValue: [small copy max: small] should: [:r| r == small].
] for: #(#'DO30' "2000/12/06 Harmon, R. Only true if DateAndTime are immutable?").

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [rsSmall max: medium]
				should: [:r| r == medium].
			self conformingValue: [medium max: rsLarge]
				should: [:r| r == rsLarge]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].

	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
			self conformingValue: [numSmall max: medium]
				should: [:r| r == medium].
			self conformingValue: [medium max: numLarge]
				should: [:r| r == numLarge]].
!

testXminX
	" Numeric <magnitude> "
	| magnitudeObjects small medium large |
	self messageSelector: #'min:'.

	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	medium := magnitudeObjects at: #'medium'.
	large := magnitudeObjects at: #'large'.
	self conformingValue: [small min: medium] should: [:r| r == small].
	self conformingValue: [medium min: large] should: [:r| r == medium].
	self conformingValue: [large min: small] should: [:r| r == small].
	self conformingValue: [large copy min: large] should: [:r| r == large].
	self conformingValue: [small copy min: small] should: [:r| r == small].

	"Errors Receiver and operands are not comparable."
	self notDone.

	"Test comparable mixed:"
	(small respondsToProtocolId: '<readableString>') ifTrue: [
		comparableReadableStringMagnitudes do: [:rsMagnitude|
			| rsSmall rsLarge |
			rsSmall := rsMagnitude at: #'small'.
			rsLarge := rsMagnitude at: #'large'.
			self conformingValue: [rsSmall min: medium]
				should: [:r| r == rsSmall].
			self conformingValue: [medium min: rsLarge]
				should: [:r| r == medium]].
		^self].

	(small respondsToProtocolId: '<number>') ifFalse: [
		^self].

	comparableNumberMagnitudes do: [:numMagnitude|
		| numSmall numLarge |
		numSmall := numMagnitude at: #'small'.
		numLarge := numMagnitude at: #'large'.
			self conformingValue: [numSmall min: medium]
				should: [:r| r == numSmall].
			self conformingValue: [medium min: numLarge]
				should: [:r| r == medium]].
! !

!MagnitudeHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/MagnitudeHelper.st,v 1.1 2002/06/21 10:33:16 penk Exp $'
! !
