"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseUsingProtocols subclass:#MainTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SUnit-Protocol-Framework'
!


!MainTestCase class methodsFor:'instance creation'!

selector: selectorSpec protocolId: protocolSpec
	"Answer a new main test case with the initial state set to the message specification, identified by selectorSpec, in the protocol with identifier protocolSpec."

	^self
		new setTestSelector: selectorSpec
		protocolId: protocolSpec
!

suite
	"Answer a new TestSuite containing coverage tests for the receiver's protocol and all protocols it conforms to."
	| protocol newTestSuite coveredProtocolIds coveredTestSelectors |
	newTestSuite := TestSuite new.
#zToDo. "See Note 1 below."
	self isAbstract ifTrue: [
		^newTestSuite].
#zToDo. "This cludge is only required to work around the fact that class MainUsingProtocolsTest is not a regular protocol test class but its methods are used to test using-protocols."
	self protocolId = TestCaseUsingProtocols protocolId ifTrue: [
		^newTestSuite].
	coveredProtocolIds := IdentitySet with: (self protocolId).
	coveredTestSelectors := IdentitySet new.
	self testMethods do: [:testSelector|
		newTestSuite addTest: (self
			selector: testSelector
			protocolId: self protocolId).
		coveredTestSelectors add: testSelector].
	self	
		addHelperConformsToTo: newTestSuite
		attachedTo: self
		coveredTests: coveredTestSelectors
		visited: coveredProtocolIds.
	^newTestSuite

"Note 1: While refactoring the SUnit Protocols Framework and Camp Smalltalk
ANSI tests, I found I questioned the approach above of detecting
descendants of TestCase to select the classes to return a suite of tests.  The following from ''Simple Smalltalk Testing: With Patterns'' by Kent Beck got me thinking:

	Unit testing
	I recommend that developers write their own unit tests, one
	per class. The framework supports the writing of suites of tests,
	which can be attached to a class. I recommend that all classes
	respond to the message ''testSuite'', returning a suite containing
	the unit tests.

That seems a reasonable approach and avoids creating work around solutions when the TestCase descendants are not appropriate--abstract classes of test classes, test using-protocols, etc.

Where test cases are associated with a protocol rather than a class, they could be detirmined by asking the protocol rather than the class.
	2001/01/18 R. A. Harmon"
! !

!MainTestCase class methodsFor:'building suites'!

buildSuiteFromAllSelectors
	"Answer a new TestSuite containing coverage tests for the receiver's protocol and all protocols it conforms to."

	^self suite
! !

!MainTestCase class methodsFor:'testing'!

isAbstract
	"Answer a <boolean>, true if the receiver is an abstract class so no TestCase instances built from it, else false if the receiver is a main test class so it should have a TestCase instance built.
	Assumes that concrete test case classes do not have subclasses."

	^self subclasses isEmpty not
!

zshouldInheritSelectors
	"Answer false, the default NOT to inherit selectors from superclasses"

	^false
! !

!MainTestCase methodsFor:'private'!

setTestSelector: selectorSpec protocolId: protocolSpec
	"Private - Set the receiver's initial state of the message specification, identified by selectorSpec, in the protocol with identifier protocolSpec."
	testSelector := selectorSpec asSymbol.
	protocolId := protocolSpec asSymbol.
! !

!MainTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/MainTestCase.st,v 1.1 2002/06/21 10:25:40 penk Exp $'
! !
