"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#NumberHelper
	instanceVariableNames:'mixedNums'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Numeric'
!


!NumberHelper class methodsFor:'constants'!

defaultMixedNumberObjects

	^OrderedCollection new
		add: -2000000000;
		add: (1/2);
		add: (2.0d0 asScaledDecimal: 3);
		add: 2;
		add: 2.0d0;
		add: 2000000000;
		yourself

!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<number>'
! !

!NumberHelper class methodsFor:'work'!

workValuesOf: testCaseClass fromDo: opBlock
	"I use this to generate the values the return value should match, and then verify them by hand.  Example usage:
	NumberHelper
		workValuesOf: FractionTest
		fromDo: [:canonicalNumber :mixedNumber|
			canonicalNumber + mixedNumber].
"
	| newTestCase newHelper numNdx byNdx numbers values byNumbers |
	newTestCase := testCaseClass new
		setUp.
	numbers := newTestCase numberObjects at: #'numbers'.

	newHelper := NumberHelper new
		setUp.
	byNumbers := newHelper mixedNumberObjects.

	values := Array new: (numbers size).
	numNdx := 0.
	numbers do: [:number |
		numNdx := numNdx + 1.
		byNdx := 0.
		values at: numNdx put: (Array new: (byNumbers size)).
		byNumbers do: [:byNumber | | value |
			byNdx := byNdx + 1.
			value := opBlock
				value: number
				value: byNumber.
			(values at: numNdx) at: byNdx put: value]].
	^values
! !

!NumberHelper methodsFor:'constants'!

mixedNumberObjects

	^mixedNums
! !

!NumberHelper methodsFor:'helpers'!

setUp

	super setUp.
	mixedNums := self class defaultMixedNumberObjects.
!

verifyScaleOf: result receiver: receiver operand: operand
	"If the return value conforms to <scaledDecimal> then the scale of the result is at least the scale of the receiver after conversion if necessary[or operand if the
	receiver does not conforms to <scaledDecimal>] after conversion if
	necessary.
	[1] <integer> op <scaledDecimal> -> <scaledDecimal>
		added on 2000/12/06 by Harmon, R."

	(result respondsToProtocolId: '<scaledDecimal>') ifFalse: [
		^true].
	(receiver respondsToProtocolId: '<scaledDecimal>') ifTrue: [
		^result scale >= receiver scale].
	^result scale >= operand scale
! !

!NumberHelper methodsFor:'testing'!

testXabs
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'abs'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'abs'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number abs]
			should: [:r| r = answer]].
!

testXaddOp
	" Numeric <number> "
	| numberObjects canonicalNumbers answers |
	self messageSelector: #'+'.

	"The protocol and representation of the return value is defined by the
	Default Result Type.
	If the return value conforms to <scaledDecimal> then the scale of the
	result is at least the scale of the receiver [or operand if the
	receiver does not conforms to <scaledDecimal>] after conversion if
	necessary.
	[1] added on 2000/12/06 by Harmon, R."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'+'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [ :canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber + mixedNumber]
				should: [:r|
					(r closeTo: answer) and: [self
						verifyScaleOf: r
						receiver: canonicalNumber
						operand: mixedNumber]]
				ruleReceiver: canonicalNumber
				operand: mixedNumber].
!

testXasFloat
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'asFloat'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'asFloat'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number asFloat]
			should: [:r| r = answer]].
!

testXasFloatD
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'asFloatD'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'asFloatD'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number asFloatD]
			should: [:r| r = answer]].
!

testXasFloatE
        " Numeric <number> "
        | numberObjects numbers answers |
        self messageSelector: #'asFloatE'.

        numberObjects := testCase numberObjects.
        numbers := numberObjects at: #'numbers'.
        answers := numberObjects at: #'asFloatE'.
        self objects: numbers answers: answers do: [:number :answer|
                self conformingValue: [  number asFloatE]
                        should: [ : r | 
                                    Transcript showCR:r.
                                    Transcript showCR:answer.
                                    r = answer]].
!

testXasFloatQ
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'asFloatQ'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'asFloatQ'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number asFloatQ]
			should: [:r| r = answer]].
!

testXasFraction
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'asFraction'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'asFraction'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number asFraction]
			should: [:r| r = answer]].
!

testXasInteger
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'asInteger'.

	numberObjects := testCase numberObjects at: #'asInteger'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number asInteger]
			should: [:r| r = answer]].
!

testXasScaledDecimalX
	" Numeric <number> "
	| numberObjects numbers answers scales |
	self messageSelector: #'asScaledDecimal:'.

	numberObjects := testCase numberObjects at: #'asScaledDecimal:'.
	numbers := numberObjects at: #'numbers'.
	scales := numberObjects at: #'scales'.
	answers := numberObjects at: #'answers'.

	self objects: numbers objects: scales answers: answers do: [
			:number :numberScale :answer|
		self conformingValue: [number asScaledDecimal: numberScale]
			should: [:r|
				r = answer and: [(number respondsToProtocolId: '<integer>')
					ifTrue: [ r scale = 0]
					ifFalse: [ r scale = numberScale]]]].
!

testXceiling
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'ceiling'.

	numberObjects := testCase numberObjects at: #'ceiling'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number ceiling]
			should: [:r| r = answer]].
!

testXdivideOp
	" Numeric <number> "
	| number numberObjects canonicalNumbers answers |
	self messageSelector: #'/'.

	"The protocol and representation of the return value is defined by the
	Default Result Type.
	If the return value conforms to <scaledDecimal> then the scale of the
	result is at least the scale of the receiver [or operand if the
	receiver does not conforms to <scaledDecimal>] after conversion if
	necessary.
	[1] added on 2000/12/06 by Harmon, R."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'/'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber / mixedNumber]
				should: [:r|
					(r closeTo: answer) and: [self
						verifyScaleOf: r
						receiver: canonicalNumber
						operand: mixedNumber]]
				ruleReceiver: canonicalNumber
				operand: mixedNumber].

	number := testCase canonicalObject.
	(number respondsToProtocolId: '<Float>')
		ifTrue: [
			"If either the receiver or operand are of type <Float>
			and the operand has a value of zero, the result is
			implementation defined.  The implementation must either
			signal the ZeroDivide exception or provide a continuation
			value."
			self notDone]
		ifFalse: [
			"For all other numeric representations the ZeroDivide
			exception is signaled."
			self should: [number / 0]
				raise: ZeroDivide].
!

testXequalityOp
	" Numeric <number> "
	| number mixed numberObjects canonicalNumbers answers |
	self messageSelector: #'='.

	"Definition: <Object>
		The meaning of 'equivalent' cannot be precisely defined but the
	intent is that two objects are considered equivalent if they can be
	used interchangeably. 
		If the value of receiver = comparand is true then the receiver
	and comparand must have equivalent hash values.  Or more formally:
		receiver = comparand =>
		receiver hash = comparand hash
	Refinement: <number>
		Numeric equality is defined by implementation defined conventions
	regarding round-off error and representation of numbers, hence
	behavior of this message may differ between platforms."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'='.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber = mixedNumber]
				should: [:r| r
					ifTrue: [canonicalNumber hash = mixedNumber hash]
					ifFalse: [canonicalNumber hash ~= mixedNumber hash]]].

self anomaly: [
	number := 2.5d0.
	mixed := 5/2.
	self conformingValue: [number = mixed]
		should: [:r| r
			ifTrue: [number hash = mixed hash]
			ifFalse: [number hash ~= mixed hash]].

	number := 1/2.
	mixed := 0.5d0.
	self conformingValue: [number = mixed]
		should: [:r| r
			ifTrue: [number hash = mixed hash]
			ifFalse: [number hash ~= mixed hash]].
] for: #(#'DO30' "2000/12/06 Harmon, R. Dolphin = is true but hash not =.
??Not sure?? = is redef. in <Float> I think it should still pass <Object> invariant.").
!

testXfloor
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'floor'.

	numberObjects := testCase numberObjects at: #'floor'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number floor]
			should: [:r| r = answer]].
!

testXfractionPart
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'fractionPart'.

	"Within the limits of representation, the following invariants
	should hold:

		receiver integerPart + receiver fractionPart = receiver
		receiver \\1 = receiver fractionPart (RAH - erroneous, add #'abs')"

	numberObjects := testCase numberObjects at: #'fractionPart'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number fractionPart]
			should: [:r|
				r = answer
				& (number integerPart + number fractionPart = number)
				& (number \\ 1 = number fractionPart abs)]].
!

testXgreaterThanOp
	" Numeric <number> "
	| numberObjects canonicalNumbers answers magnitudeObjects small large |
	self messageSelector: #'>'.

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'>'.
	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	large := magnitudeObjects at: #'large'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber > mixedNumber]
				should: [:r| r = answer].

			self conformingValue: [mixedNumber > small]
				should: [:r| r].
			self conformingValue: [large > mixedNumber]
				should: [:r| r].
			self conformingValue: [mixedNumber > mixedNumber]
				shouldnt: [:r| r].
			self conformingValue: [small > mixedNumber]
				shouldnt: [:r| r].
			self conformingValue: [mixedNumber > large]
				shouldnt: [:r| r]].

	"It is erroneous if the receiver and operand are not comparable."
	self notDone.
!

testXintegerDivideOp
	" Numeric <number> "
	| number numberObjects canonicalNumbers answers zero |
	self messageSelector: #'//'.

	"Answer an integer whose value is the truncated result of dividing
	the receiver by operand, as specified by the ISO/IEC 10967 flooring
	division operation divf. Truncation is towards negative infinity.
	The sign of the result is positive if the receiver and operand have
	the same sign, and negative if the signs are different."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'//'.
	zero := numberObjects at: #'zero'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [ :canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber // mixedNumber]
				should: [:r| r closeTo: answer].

			self conformingValue: [zero // mixedNumber]
				should: [:r| r = 0]].
	"Old Tests:"
	self should: [25.5d0 // 5.1d0 = 5].
	self should: [25.5d0 // 5 = 5].
	self should: [2.55d10 // 5000000000 = 5].
	self should: [-2.55d10 // -5000000000 = 5].
	self should: [25.5d0 // (51 / 10) = 5].
	self should: [25.5d0 // (5.1d0 asScaledDecimal: 1) = 5].

	self should: [(256 / 5) // (50 / 5) = 5].
	self should: [(126 / 5) // 5 = 5].
	self should: [(125000000001 / 5) // 5000000000 = 5].
	self should: [(-125000000001 / 5) // -5000000000 = 5].
	self should: [(131 / 5) // 5.1d0 = 5].
	self should: [(131 / 5) // (5.1d0 asScaledDecimal: 1) = 5].

	self should: [26 // 5.1d0 = 5].
	self should: [26 // 5 = 5].
	self should: [25500000000 // 5000000000 = 5].
	self should: [-25500000000 // -5000000000 = 5].
	self should: [26 // (51 / 10) = 5].
	self should: [26 // (5.1d0 asScaledDecimal: 1) = 5].

	self should: [(25.5d0 asScaledDecimal: 3) // (5.1d0 asScaledDecimal: 3) = 5].
	self should: [(25.5d0 asScaledDecimal: 3) // 5 = 5].
	self should: [(2.55d10 asScaledDecimal: 3) // 5000000000 = 5].
	self should: [(-2.55d10 asScaledDecimal: 3) // -5000000000 = 5].
	self should: [(25.5d0 asScaledDecimal: 3) // (51 / 10) = 5].
	self should: [(25.5d0 asScaledDecimal: 3) // 5.1d0 = 5].

	self notDone.
	"Which error desc. below applies?
	Definition: <number>
		. . .
		If the operand has a value of zero the ZeroDivide exception is
	signaled.
	Errors:
		operand = 0 unless receiver or operand are of type <Float>."
	number := testCase canonicalObject.
	(number respondsToProtocolId: '<Float>')
	| (zero respondsToProtocolId: '<Float>')
		ifTrue: [
self anomaly: [
			self should: [number // zero]
				raise: ZeroDivide
] for: #(#'DO30' "2000/12/06 Harmon, R. May not be needed?")]
		ifFalse: [
			self should: [number // zero]
				raise: ZeroDivide].
!

testXintegerPart
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'integerPart'.

	numberObjects := testCase numberObjects at: #'integerPart'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number integerPart]
			should: [:r| r = answer]
			ruleReceiver: number].
!

testXlessThanOp
	" Numeric <number> "
	| numberObjects canonicalNumbers answers magnitudeObjects small large |
	self messageSelector: #'<'.

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'<'.
	magnitudeObjects := testCase magnitudeObjects.
	small := magnitudeObjects at: #'small'.
	large := magnitudeObjects at: #'large'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer| | mediumMixed |
			self conformingValue: [canonicalNumber < mixedNumber]
				should: [:r| r = answer].

			mediumMixed := mixedNumber.
			self conformingValue: [mediumMixed < small]
				shouldnt: [:r| r].
			self conformingValue: [large < mediumMixed]
				shouldnt: [:r| r].
			self conformingValue: [mediumMixed < mediumMixed]
				shouldnt: [:r| r].
			self conformingValue: [small < mediumMixed]
				should: [:r| r].
			self conformingValue: [mediumMixed < large]
				should: [:r| r]].

	"It is erroneous if the receiver and operand are not comparable."
	self notDone.
!

testXmultiplyOp
	" Numeric <number> "
	| numberObjects canonicalNumbers answers |
	self messageSelector: #'*'.

	"The protocol and representation of the return value is defined by the
	Default Result Type.
	If the return value conforms to <scaledDecimal> then the scale of the
	result is at least the scale of the receiver [or operand if the
	receiver does not conforms to <scaledDecimal>] after conversion if
	necessary.
	[1] added on 2000/12/06 by Harmon, R."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'*'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber * mixedNumber]
				should: [:r|
					(r closeTo: answer) and: [self
						verifyScaleOf: r
						receiver: canonicalNumber
						operand: mixedNumber]]
				ruleReceiver: canonicalNumber
				operand: mixedNumber].
!

testXnegated
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'negated'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'negated'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number negated]
			should: [:r| r = answer]].
!

testXnegative
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'negative'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'negative'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number negative]
			should: [:r| r = answer]].
!

testXpositive
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'positive'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'positive'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number positive]
			should: [:r| r = answer]].
!

testXprintString
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'printString'.

	numberObjects := testCase numberObjects at: #'printString'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		(number respondsToProtocolId: '<scaledDecimal>')
			ifTrue: [ | altAnswer |
				altAnswer := answer, (number scale printString).
				self conformingValue: [number printString]
					should: [:r| (r = answer) | (r = altAnswer)]]
			ifFalse: [
				self conformingValue: [number printString]
					should: [:r| r = answer]]].
!

testXquoX
	" Numeric <number> "
	| number numberObjects canonicalNumbers answers |
	self messageSelector: #'quo:'.

	"Answer the truncated integer quotient resulting from dividing the
	receiver by operand.  Truncation is towards zero."

	self notDone. "2000/12/06 by Harmon, R. Which applies?
	Definition: <number>
	. . .
	The protocol and representation of the return value is defined by the
	Default Result Type.
	If the return value conforms to <scaledDecimal> then the scale of the
	result is at least the scale of the receiver after conversion if
	necessary.
or
	Return Values
		<integer>       unspecified"

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'quo:'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber quo: mixedNumber]
				should: [:r| (r closeTo: answer)
					"r is <integer> I think, so not needed?"
					and: [ self
						verifyScaleOf: r
						receiver: canonicalNumber
						operand: mixedNumber]]].

	number := testCase canonicalObject.
	(number respondsToProtocolId: '<Float>')
		ifTrue: [
			"If either the receiver or operand are of type <Float> and
			the operand has a value of zero, the result is implementation
			defined.  The implementation must either signal the ZeroDivide
			exception or provide a continuation value."
			self notDone]
		ifFalse: [
			"For all other numeric representations the ZeroDivide
			exception is signaled"
			self should: [number quo: 0]
				raise: ZeroDivide].

	"Old Tests:"
	self should: [(25.5d0 quo: 5.1d0) = 5].
	self should: [(25.5d0 quo: 5) = 5].
	self should: [(2.55d10 quo: 5000000000) = 5].
	self should: [(-2.55d10 quo: -5000000000) = 5].
	self should: [(25.5d0 quo: 51 / 10) = 5].
	self should: [(25.5d0 quo: (5.1d0 asScaledDecimal: 1)) = 5].

	self should: [(256 / 5 quo: 50 / 5) = 5].
	self should: [(126 / 5 quo: 5) = 5].
	self should: [(125000000001 / 5 quo: 5000000000) = 5].
	self should: [(-125000000001 / 5 quo: -5000000000) = 5].
	self should: [(131 / 5 quo: 5.1d0) = 5].
	self should: [(131 / 5 quo: (5.1d0 asScaledDecimal: 1)) = 5].

	self should: [(26 quo: 5.1d0) = 5].
	self should: [(26 quo: 5) = 5].
	self should: [(25500000000 quo: 5000000000) = 5].
	self should: [(-25500000000 quo: -5000000000) = 5].
	self should: [(26 quo: 51 / 10) = 5].
	self should: [(26 quo: (5.1d0 asScaledDecimal: 1)) = 5].

	self should: [
			((25.5d0 asScaledDecimal: 3) quo: (5.1d0 asScaledDecimal: 3)) = 5].
	self should: [((25.5d0 asScaledDecimal: 3) quo: 5) = 5].
	self should: [((2.55d10 asScaledDecimal: 3) quo: 5000000000) = 5].
	self should: [((-2.55d10 asScaledDecimal: 3) quo: -5000000000) = 5].
	self should: [((25.5d0 asScaledDecimal: 3) quo: 51 / 10) = 5].
	self should: [((25.5d0 asScaledDecimal: 3) quo: 5.1d0) = 5].
!

testXraisedToIntegerX
	" Numeric <number> "
	| numberObjects numberIntegerAnswerList canonicalNumber zero |
	self messageSelector: #'raisedToInteger:'.

	numberObjects := testCase numberObjects.
	numberIntegerAnswerList := numberObjects at: #'raisedToInteger:'.
	numberIntegerAnswerList do: [:numberIntegerAnswer|
		| number integer answer |
		number := numberIntegerAnswer at: 1.
		integer := numberIntegerAnswer at: 2.
		answer := numberIntegerAnswer at: 3.
		self conformingValue: [number raisedToInteger: integer]
			should: [:r| r closeTo: answer]].

	"It is erroneous if the operand does not conform to the protocol
	<integer>."
	canonicalNumber := testCase canonicalObject.
	self should: [canonicalNumber raisedToInteger: 2.0d0]
		raise: TestResult error.
        
	self notDone."???I think this is a valid test, or may be
	implementation defined case??? 2000/12/06 Harmon, R."
	zero := numberObjects at: #'zero'.
	self should: [zero raisedToInteger: -2]
		raise: ZeroDivide.
!

testXraisedToX
	" Numeric <number> "
	| number zero mixedNumsNoLargeIntegers |
	self messageSelector: #'raisedTo:'.

	"If operand conforms to <integer>, answer the result of sending
	#raisedToInteger: with argument operand to the receiver.
	Otherwise answer: (receiver asFloat ln * operand) exp."

	number := testCase canonicalObject.
	zero := testCase numberObjects at: #'zero'.
	mixedNumsNoLargeIntegers := mixedNums select: [:mixed| mixed abs < 10].
	mixedNumsNoLargeIntegers do: [:mixedNumber|
		self conformingValue: [zero raisedTo: mixedNumber]
			should: [:r| r = zero].

		self conformingValue: [number raisedTo: mixedNumber]
			should: [:r|
				r closeTo: ((mixedNumber respondsToProtocolId: '<integer>')
					ifTrue: [number raisedToInteger: mixedNumber]
					ifFalse: [(number asFloat ln * mixedNumber) exp])]].

	"It is erroneous if the receiver equals zero and the operand is
	less than or equal to zero,"
	self should: [zero raisedTo: -2]
		raise: TestResult error.
	self should: [zero raisedTo: zero]
		raise: TestResult error.

	"or if the receiver is less than zero."
	self should: [number negated raisedTo: number]
		raise: TestResult error.


"Modified old FloatANSITest:
	??Large pos. & neg. Integers are not tested as they take a long time
	to compute.??
	??mixedNums ->#( 2 -2000000000 2000000000 2.0d0 1/2 2.0s3 ).??
	float2 := 2.0d0.
	retVals := (Dictionary new)
		at: #'smallInt2'                                        put: 4.0d0;
		at: #'largeNegInt2000000000'    put: 0;
		at: #'largePosInt2000000000'    put: 0;
		at: #'float2'                                   put: 4.0d0;
		at: #'fractionHalf'                             put: 1.414213562373095d0;
		at: #'sclDec2s3'                                        put: 4.0d0;
		yourself.
	mixedNums keysAndValuesDo: [:typeKey :tstNum|
		((typeKey = #'largeNegInt2000000000')
		| (typeKey = #'largePosInt2000000000')) ifFalse: [
			self conformingValue: [float2 raisedTo: tstNum]
				should: [:r| r closeTo: (retVals at: typeKey)]]].

	retVals := (Dictionary new)
		at: #'smallInt2'                                        put: 0.25d0;
		at: #'largeNegInt2000000000'    put: 0;
		at: #'largePosInt2000000000'    put: 0;
		at: #'float2'                                   put: 0.25d0;
		at: #'fractionHalf'                             put: 0.707106781d0;
		at: #'sclDec2s3'                                        put: 0.25d0;
		yourself.
	mixedNums keysAndValuesDo: [:typeKey :tstNum|
		((typeKey = #'largeNegInt2000000000')
		| (typeKey = #'largePosInt2000000000')) ifFalse: [
			self conformingValue: [
					float2 raisedTo: tstNum negated]
				should: [:r| r closeTo: (retVals at: typeKey)]]].

	self conformingValue: [float2 raisedTo: 0.0] should: [:r| r = 1.0d0].
	self conformingValue: [0.0 raisedTo: float2] should: [:r| r = 0.0].
	self conformingValue: [float2 raisedTo: 1.0d0] should: [:r| r = float2].

	??It is erroneous if the receiver equals zero
		and the operand is less than or equal to zero,??
	self should: [0.0 raisedTo: -2]
		raise: TestResult error.

	??or if the receiver is less than zero.??
"
	self should: [2.0d0 negated raisedTo: 2.0d0]
		raise: TestResult error.
!

testXreciprocal
	" Numeric <number> "
	| numberObjects numbers numberZero |
	self messageSelector: #'reciprocal'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'reciprocal'.

	"Answer the reciprocal of the receiver, which is equal to
	the result of the operation (1/receiver)."
	numbers do: [:number|
		self conformingValue: [number reciprocal]
			should: [:r| r = (1 / number)]
			ruleReceiver: number].

	"Signal a ZeroDivide exception if the receiver is equal to zero."
	numberZero := numberObjects at: #'zero'.
	self should: [numberZero reciprocal]
		raise: ZeroDivide.
!

testXremX
	" Numeric <number> "
	| numberObjects numbers divisors answers number zero |
	self messageSelector: #'rem:'.

	"Answer the remainder with respect to integer division, as specified
	by the ISO/IEC 10967 remainder operation rem.
	The sign of the remainder is the same sign as the receiver.
	Within the limits of representation, the following invariant should
	hold:
	(receiver quo: operand)*operand + (receiver rem: operand) = receiver
	The protocol and representation of the return value is defined by the
	Default Result Type.  If the return value conforms to <scaledDecimal>
	then the scale of the result is at least the scale of the receiver
	[or operand if the receiver does not conforms to <scaledDecimal>]
	after conversion if necessary.
	[1] added on 2000/12/06 by Harmon, R."

	numberObjects := testCase numberObjects.
	numbers := (numberObjects at: #'rem:') at: #'numbers'.
	answers := (numberObjects at: #'rem:') at: #'answers'.
	divisors := #(5 2000000000 -2000000000 5.0d0 0 0).
	divisors
		at: 5 put: (5/2);
		at: 6 put: (5.0d0 asScaledDecimal: 3).

	self numbers: numbers
		byNumbers: divisors
		answers: answers
		do: [:receiver :operand :answer|
			self conformingValue: [receiver rem: operand]
				should: [:r|
					(r closeTo: answer)
					& ((receiver quo: operand) * operand + r
						= receiver)
					& (self
						verifyScaleOf: r
						receiver: receiver
						operand: operand)]
				ruleReceiver: receiver
				operand: operand].

	number := testCase canonicalObject.
	zero := numberObjects at: #'zero'.
	(number respondsToProtocolId: '<Float>')
	| (zero respondsToProtocolId: '<Float>')
		ifTrue: [
			"If either the receiver or operand are of type <Float>
			and the operand has a value of zero, the result is
			implementation defined.  The implementation may signal the
			ZeroDivide exception or provide a continuation value."
			self notDone]
		ifFalse: [
			"For all other numeric representations the ZeroDivide
			exception is signaled."
			self should: [number rem: zero]
				raise: ZeroDivide].
!

testXremainderIntegerDivideOp
	" Numeric <number> "
	| number numberObjects canonicalNumbers negMixedNums answers zero |
	self messageSelector: #'\\'.

	"Answer the remainder after integer division of the receiver
	by the operand.
	The remainder has the same sign as operand.
		The protocol and representation of the return value is defined
	by the Default Result Type.
		If the return value conforms to <scaledDecimal> then the scale
	of the result is at least the scale of the receiver [or operand if
	the receiver does not conforms to <scaledDecimal>] after conversion
	if necessary.
		[1] added on 2000/12/06 by Harmon, R.

		Within the limits of representation, the following invariant
	should hold:
		(receiver // operand) * operand + (receiver \\ operand) = receiver
		If the result value is outside of the range of the common numeric
	representation, the effect of underflow or overflow is implementation
	defined."

	numberObjects := testCase numberObjects.
	zero := numberObjects at: #'zero'.
	canonicalNumbers := (numberObjects at: #'\\') at: #'numbers'.
	answers := (numberObjects at: #'\\') at: #'posAnswers'.
	"Test remainder has the same sign as operand -> positive."
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer| | receiver operand |
			receiver := canonicalNumber.
			operand := mixedNumber.
			self conformingValue: [receiver \\ operand]
				should: [:r|
					(r closeTo: answer)
					& (self
						verifyScaleOf: r
						receiver: receiver
						operand: operand)
					& ((receiver // operand) * operand
						+ (receiver \\ operand) = receiver)]
				ruleReceiver: receiver
				operand: operand].

	"Test remainder has the same sign as operand -> negative."
	negMixedNums := mixedNums collect: [:mixed| mixed negated].
	answers := (numberObjects at: #'\\') at: #'negAnswers'.
	self numbers: canonicalNumbers
		byNumbers: negMixedNums
		answers: answers
		do: [:canonicalNumber :mixedNumber :answer| | receiver operand |
			receiver := canonicalNumber.
			operand := mixedNumber.
			self conformingValue: [receiver \\ operand]
				should: [:r|
					(r closeTo: answer)
					& (self
						verifyScaleOf: r
						receiver: receiver
						operand: operand)
					& ((receiver // operand) * operand
						+ (receiver \\ operand) = receiver)]
				ruleReceiver: receiver
				operand: operand].

	number := testCase canonicalObject.
	(number respondsToProtocolId: '<Float>')
		ifTrue: [
			"If either the receiver or operand is of type <Float> and
			the operand has a value of zero, the result is implementation
			defined.
			The implementation may signal the ZeroDivide exception or
			provide a continuation value."
			self notDone]
		ifFalse: [
			"For all other numeric representations the ZeroDivide
			exception is signaled."
			self should: [number \\ zero]
				raise: ZeroDivide].

	"Old Tests:"
	self should: [26 \\ 5 = 1 & (26 // 5 * 5 + (26 \\ 5) = 26)].
	self should: [26 \\ -5 = -4 & (26 // -5 * -5 + (26 \\ -5) = 26)].
	self should: [0 \\ 2 = 0].
	self should: [0 \\ -2000000000 = 0].
	self should: [0 \\ 2000000000 = 0].

	self should: [
			(26.5d0 asScaledDecimal: 3) \\ (5.0d0 asScaledDecimal: 3)
			= (1.5d0 asScaledDecimal: 3)

			& ((26.5d0 asScaledDecimal: 3) // (5.0d0 asScaledDecimal: 3)
				* (5.0d0 asScaledDecimal: 3)
				+ ((26.5d0 asScaledDecimal: 3) \\ (5.0d0 asScaledDecimal: 3))
			= (26.5d0 asScaledDecimal: 3))].

	self should: [
			(26.5d0 asScaledDecimal: 3) \\ (-5.0d0 asScaledDecimal: 3)
			= (-3.5d0 asScaledDecimal: 3)

			& ((26.5d0 asScaledDecimal: 3) // (-5.0d0 asScaledDecimal: 3)
				* (-5.0d0 asScaledDecimal: 3)
				+ ((26.5d0 asScaledDecimal: 3) \\ (-5.0d0 asScaledDecimal: 3))
			= (26.5d0 asScaledDecimal: 3))].
!

testXroundToX
	" Numeric <number> "
	| receiver answers |
	self messageSelector: #'roundTo:'.

	receiver := testCase canonicalObject.
	answers := testCase numberObjects at: #'roundTo:'.
	self objects: mixedNums answers: answers do: [:operand :answer|
		self conformingValue: [receiver roundTo: operand]
			should: [:r| r = answer]
			ruleReceiver: receiver
			operand: operand].
!

testXrounded
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'rounded'.

	numberObjects := testCase numberObjects at: #'rounded'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number rounded]
			should: [:r| r = answer]].
!

testXsign
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'sign'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'sign'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number sign]
			should: [:r| r = answer]].
!

testXsqrt
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'sqrt'.

	numberObjects := testCase numberObjects at: #'sqrt'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number sqrt]
			should: [:r| r closeTo: answer]].
!

testXsquared
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'squared'.

	numberObjects := testCase numberObjects at: #'squared'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number squared]
			should: [:r| r closeTo: answer]].
!

testXstrictlyPositive
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'strictlyPositive'.

	numberObjects := testCase numberObjects.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'strictlyPositive'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number strictlyPositive]
			should: [:r| r = answer]].
!

testXsubtractOp
	" Numeric <number> "
	| numberObjects canonicalNumbers answers |
	self messageSelector: #'-'.

	"The protocol and representation of the return value is defined by the
	Default Result Type.
	If the return value conforms to <scaledDecimal> then the scale of the
	result is at least the scale of the receiver [or operand if the
	receiver does not conforms to <scaledDecimal>] after conversion if
	necessary.
	[1] added on 2000/12/06 by Harmon, R."

	numberObjects := testCase numberObjects.
	canonicalNumbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'-'.
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [ :canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber - mixedNumber]
				should: [:r|
					(r closeTo: answer) and: [self
						verifyScaleOf: r
						receiver: canonicalNumber
						operand: mixedNumber]]
				ruleReceiver: canonicalNumber
				operand: mixedNumber].
!

testXtoX
	" Numeric <number> "
	| coercing |
	self messageSelector: #'to:'.

	"Answer an interval which represents an arithmetic progression from
	the receiver to stop, using the increment 1 to compute each successive
	element."

	coercing := testCase numberObjects at: #'coercing'.
	mixedNums do: [:stop| | start |
		start := coercing value: (stop - 1).
		self conformingValue: [start to: stop]
			should: [:r|
				r size = 2
				"The elements conform to the receiver's protocol."
				& (r allSatisfy: [:elem|
					elem respondsToProtocolId: (testCase class protocolId)])
				"Note that stop may not be the last element in the
				sequence, which is given by the formula:
					receiver + ((stop - receiver) // 1)."
				& (r last = (start + ((stop - start) // 1)))].

		"The interval answered will be empty if the receiver is greater
		than stop."
		self conformingValue: [stop to: start]
			should: [:r| r isEmpty]].
!

testXtoXbyX
	" Numeric <number> "
	| coerceToNumTypeTesting nonNegMixedNums start stop2 zero |
	self messageSelector: #'to:by:'.

	self notDone. "This needs to also test negative mixedNums.
				Also size can not be detirmined."

	"Answer an interval which represents an arithmetic progression
	from the receiver to stop, using the increment step to compute each
	successive element.  The value of step can be positive or negative,
	but it must be non-zero."
	coerceToNumTypeTesting := testCase numberObjects at: #'coercing'.
	nonNegMixedNums := mixedNums select: [:num| num positive].
self anomaly: [
	nonNegMixedNums do: [:stop|
		nonNegMixedNums do: [:step| | conversionProtocol |
			start := coerceToNumTypeTesting value: (stop - step).
			self conformingValue: [start to: stop by: step]
				should: [:r|
					"The elements conform to either the receiver's
					or step's protocol, according to the Default
					Conversion Table."
					conversionProtocol := SysProProtocolsManager
						defaultReturnProtocolIdReceiver: start
						operand: step.
					(r allSatisfy: [:elem|
						elem respondsToProtocolId: conversionProtocol])

					"Note that stop may not be the last element
					in the sequence, which is given by the formula:
					(((stop - receiver) // step) * step) + receiver"
					& (r last closeTo: ((((stop - start) // step) * step)
									+ start))

					"The size can not be detirmined:
						start := (2.000s - (1/2) -> 3/2) asInteger -> 2
						giving: 2 to: 2.000s by: (1/2)
					& (r size = 2)"]]].
] forDialectAndTestCaseName: #(
	#(#'DO30' "2001/02/20 Harmon, R."
		#(#'ScaledDecimalTest'  "(0s to: (1/2) by: (1/2)) -> #(0s 1/2)
		I read the spec. to say all elements must be #'<rational>'"))).

	"Errors: step = 0."
	zero := testCase numberObjects at: #'zero'.
	start := coerceToNumTypeTesting value: (zero + 1).
	stop2 := coerceToNumTypeTesting value: (start + 1).
self anomaly: [
	self should: [start to: stop2 by: zero]
		raise: TestResult error.
] for: #(#'DO30' "2001/02/20 Harmon, R. Doesn't signal error.").
!

testXtoXbyXdoX
	" Numeric <number> "
	| coerceToNumTypeTesting nonNegMixedNums start step2 stop2 zero |
	self messageSelector: #'to:by:do:'.

	self notDone. "This needs to also test negative mixedNums.
				Also size can not be determined."

	"Evaluate operation for each element of an interval starting at
	the receiver and stopping at stop where each element is step
	greater than the previous. The value of step can be positive
	or negative, but it must be non-zero."
	coerceToNumTypeTesting := testCase numberObjects at: #'coercing'.
	nonNegMixedNums := mixedNums select: [:num| num positive].
self anomaly: [
	nonNegMixedNums do: [:stop|
		nonNegMixedNums do: [:step| | conversionProtocol collctn |
			start := coerceToNumTypeTesting value: (stop - step).
			collctn := OrderedCollection new.
			self conformingValue: [
					start to: stop by: step do: [:each|
						collctn add: each]]
				should: [:r| "UNSPECIFIED"
					"The elements conform to either the receiver's
					or step's protocol, according to the Default
					Conversion Table."
					conversionProtocol := SysProProtocolsManager
						defaultReturnProtocolIdReceiver: start
						operand: step.
					(collctn allSatisfy: [:elem|
						elem respondsToProtocolId: conversionProtocol])

					"Note that stop may not be the last element
					in the sequence, which is given by the formula:
					(((stop - receiver) // step) * step) + receiver"
					& (collctn last closeTo:
						((((stop - start) // step) * step) + start))

					"The size can not be detirmined:
						start := (2.000s - (1/2) -> 3/2) asInteger -> 2
						giving: 2 to: 2.000s by: (1/2)
					& (collctn size = 2)"]]].
] forDialectAndTestCaseName: #(
	#(#'DO30' "2001/02/20 Harmon, R. I read the spec. to say all elements
				must conform to Def. Conv. Table spec. of receiver & step:"
		#(#'FractionTest' "((-3/2) (1/2) 2.0) -> #(-3/2 0.5) <Float>"
		#'IntegerTest' "(-2 (1/2) 2.000s) -> #(-2 0.000s) <scaledDecimal>"
		#'ScaledDecimalTest' "(0s (1/2) (1/2)) -> #(0s 1/2) <rational>"))).

	"No evaluation takes place if:
		1. receiver < stop, and step < 0:"
	zero := testCase numberObjects at: #'zero'.
	start := "1"  coerceToNumTypeTesting value: (zero + 1).
	stop2 := "2"  coerceToNumTypeTesting value: (zero + 2).
	step2 := "-1" coerceToNumTypeTesting value: (zero - 1).
	start to: stop2 by: step2 do: [:each|
		self signalFailure: 'This should not be sent.'].

		"2. receiver > stop, and step > 0:"
	start := "2" coerceToNumTypeTesting value: (zero + 2).
	stop2 := "1" coerceToNumTypeTesting value: (zero + 1).
	step2 := "1" coerceToNumTypeTesting value: (zero + 1).
	start to: stop2 by: step2 do: [:each|
		self signalFailure: 'This should not be sent.'].

	"Errors: step = 0."
self anomaly: [ "self halt should not be sent."
	self should: [start to: stop2 by: zero do: [:each| self halt]]
		raise: TestResult error.
] for: #(#'DO30' "2001/02/20 Harmon, R. Doesn't signal error.").

!

testXtoXdoX
	" Numeric <number> "
	| coerceToNumTypeTesting nonNegMixedNums start step2 stop2 zero |
	self messageSelector: #'to:do:'.

	self notDone. "This needs to also test negative mixedNums.
				Also don't know what elements must all conform to."

	coerceToNumTypeTesting := testCase numberObjects at: #'coercing'.
	nonNegMixedNums := mixedNums select: [:num| num positive].
	nonNegMixedNums do: [:stop| | conversionProtocol collctn |
		start := coerceToNumTypeTesting value: (stop - 1).
		collctn := OrderedCollection new.
		self conformingValue: [
				start to: stop do: [:each| collctn add: each]]
			should: [:r| "UNSPECIFIED"
				"The elements must all conform to the receiver's
				protocol according to the Default Conversion Table"
"I don't know what the above means, unless it is receiver's & operand's???  
				conversionProtocol := SysProProtocolsManager
					defaultReturnProtocolIdReceiver: start
					operand: stop ????.
or just all conform to the receiver's  protocol???
					numericInstanceProtocolId: start.
				(collctn allSatisfy: [:elem|
					elem respondsToProtocolId: conversionProtocol])
"
					"Note that stop may not be the last element in the sequence, which is given by the formula
receiver + ((stop - receiver) // 1)"
				(collctn last closeTo: (start + ((stop - start) // 1)))

				"The size can not be detirmined:
					start := (2.000s - (1/2) -> 3/2) asInteger -> 2
					giving: 2 to: 2.000s by: (1/2)"
				& (collctn size = 2)]].
self anomaly: [
] forDialectAndTestCaseName: #(
	#(#'DO30' "2001/02/20 Harmon, R. I read the spec. to say all elements
				must conform to Def. Conv. Table spec. of receiver & step:"
		#(#'FractionTest' "((-3/2) (1/2) 2.0) -> #(-3/2 0.5) <Float>"
		#'IntegerTest' "(-2 (1/2) 2.000s) -> #(-2 0.000s) <scaledDecimal>"
		#'ScaledDecimalTest' "(0s (1/2) (1/2)) -> #(0s 1/2) <rational>"))).

	"No evaluation takes place if the receiver is greater than stop.:"
	zero := testCase numberObjects at: #'zero'.
	start := "2" coerceToNumTypeTesting value: (zero + 2).
	stop2 := "1" coerceToNumTypeTesting value: (zero + 1).
	start to: stop2 do: [:each|
		self signalFailure: 'This should not be sent.'].
!

testXtruncateToX
	" Numeric <number> "
	| receiver answers |
	self messageSelector: #'truncateTo:'.

	receiver := testCase canonicalObject.
	answers := testCase numberObjects at: #'truncateTo:'.
	self objects: mixedNums answers: answers do: [:operand :answer|
		self conformingValue: [receiver truncateTo: operand]
			should: [:r| r = answer]
			ruleReceiver: receiver
			operand: operand].
!

testXtruncated
	" Numeric <number> "
	| numberObjects numbers answers |
	self messageSelector: #'truncated'.

	numberObjects := testCase numberObjects at: #'truncated'.
	numbers := numberObjects at: #'numbers'.
	answers := numberObjects at: #'answers'.
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number truncated]
			should: [:r| r = answer]].
! !

!NumberHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/NumberHelper.st,v 1.1 2002/06/21 10:23:42 penk Exp $'
! !
