"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#OrderedCollectionFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!OrderedCollectionFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^OrderedCollectionTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<OrderedCollection factory>'
! !

!OrderedCollectionFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^OrderedCollectionTest returnTypeHasLimitedElementTypes
! !

!OrderedCollectionFactoryTest methodsFor:'constants'!

canonicalCollection

	^OrderedCollectionTest canonicalCollection
!

canonicalElement

	^OrderedCollectionTest canonicalElement
!

canonicalObject

	^OrderedCollection
! !

!OrderedCollectionFactoryTest methodsFor:'testing'!

testXnew
	" ANSI-Collection <OrderedCollection factory> "
	self messageSelector: #'new'.

	self conformingValue: [OrderedCollection new]
		should: [:r| r isEmpty].
!

testXnewX
	" ANSI-Collection <OrderedCollection factory> "
	self messageSelector: #'new:'.

	"Refinement: <OrderedCollection factory>
	The parameter count represents a hint for space allocation.  The new
	collection is to optimized to contain count elements.  The new
	collection initially contains no elements."
	self conformingValue: [OrderedCollection new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [OrderedCollection new: 3]
		should: [:r| r isEmpty].

	"Errors: count < 0."
		self should: [OrderedCollection new: -3]
			raise: TestResult error.
!

testXwithAllX
	" ANSI-Collection <OrderedCollection factory> "
	| collection |
	self messageSelector: #'withAll:'.

	"Refinement: <OrderedCollection factory>
	If the elements of newElements are ordered then their ordering
	establishing their index positions in the new collection."
	collection := self canonicalCollection.
	self conformingValue: [OrderedCollection withAll: collection]
		should: [:r|
			1 to: r size do: [:ndx |
				(r at: ndx) = (collection  at: ndx) ifFalse: [
					^false]].
			r size = 4].

	"Errors: If any of the elements of newElements do not meet
			the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [OrderedCollection withAll: illegalElements]
			raise: TestResult error]
!

testXwithXwithX
	" ANSI-Collection <OrderedCollection factory> "
	| collection |
	self messageSelector: #'with:with:'.

	collection := self canonicalCollection.
	self conformingValue: [
			OrderedCollection
				with: collection first
				with: collection last]
		should: [:r|
			r size = 2
			& r first = collection first
			& r last = collection last].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			OrderedCollection
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" ANSI-Collection <OrderedCollection factory> "
	| collection |
	self messageSelector: #'with:with:with:'.

	collection := self canonicalCollection
		copyFrom: 1 to: 3.
	self conformingValue: [
			OrderedCollection
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			OrderedCollection
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" ANSI-Collection <OrderedCollection factory> "
	| collection |
	self messageSelector: #'with:with:with:with:'.

	collection := self canonicalCollection.
	self conformingValue: [
			OrderedCollection
				with: (collection  at: 1)
				with: (collection  at: 2)
				with: (collection  at: 3)
				with: (collection  at: 4)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			OrderedCollection
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!OrderedCollectionFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/OrderedCollectionFactoryTest.st,v 1.1 2002/06/21 10:27:40 penk Exp $'
! !
