"{ Package: 'stx:goodies/ansiTests/common' }"

CollectionTest subclass:#OrderedCollectionTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!OrderedCollectionTest class methodsFor:'constants'!

canonicalCollection

	^self canonicalObject
!

canonicalElement

	^self canonicalObject at: 2
!

canonicalObject

	^OrderedCollection
		with: 1
		with: 2
		with: 3
		with: 4
!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^self error: 'Has no restrictions.'.    "DO30"
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<OrderedCollection>'
! !

!OrderedCollectionTest class methodsFor:'public'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^false  "Has no restrictions: DO30"
! !

!OrderedCollectionTest methodsFor:'constants'!

canonicalElement

	^self class canonicalElement
!

canonicalObject

	^self class canonicalObject
!

emptyCollection

	^OrderedCollection new
! !

!OrderedCollectionTest methodsFor:'testing'!

testXaddAllFirstX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAllFirst:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection addAllFirst: #(111 222)]
	"2000/12/06 Harmon, R. Needs fix??? Draft - Return Values UNSPECIFIED."
		should:[:r| r = #(111 222)
			and: [(orderedCollection at: 1) = 111
			and: [(orderedCollection at: 2) = 222]]]
!

testXaddAllLastX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAllLast:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection addAllLast: #(111 222)]
	"2000/12/06 Harmon, R. Needs fix??? Draft - Return Values UNSPECIFIED."
		should:[:r | r = #(111 222)
			and: [(orderedCollection at: orderedCollection size - 1) = 111
			and: [(orderedCollection last) = 222]]]
!

testXaddAllXafterIndexX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAll:afterIndex:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			addAll: #(333 444)
			afterIndex: 1]
		should: [:r| ((orderedCollection at: 2) = 333)
			and: [(orderedCollection at: 3) = 444]].

	self conformingValue: [orderedCollection
			addAll: #(555 666)
			afterIndex: 4]
		should: [:r| ((orderedCollection at: 5) = 555)
			and: [(orderedCollection at: 6) = 666]].

	self conformingValue: [orderedCollection
			addAll: #(777 888)
			afterIndex: 0]
		should: [:r| ((orderedCollection first) = 777)
			and: [( orderedCollection at: 2) = 888]].

	"Errors If index > receiver's size."
	orderedCollection := self canonicalObject.
	self should: [orderedCollection
			addAll: #(111 222)
			afterIndex: 5]  
		raise: TestResult error.

	"Errors If index < 0."
	self should: [orderedCollection
			addAll: #(111 222)
			afterIndex: -1] 
		raise: TestResult error.
!

testXaddAllXafterX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAll:after:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			addAll: #(333 444)
			after: 1]
		should: [:r| ((orderedCollection at: 2) = 333)
			and: [(orderedCollection at: 3) = 444]].

	self conformingValue: [orderedCollection
			addAll: #(555 666)
			after: 4]
		should: [:r|
			((orderedCollection at: orderedCollection size - 1) = 555)
			and: [(orderedCollection last) = 666]]
.
	self conformingValue: [orderedCollection
			addAll: #(777 888)
			after: 555]
		should: [:r|
			((orderedCollection at: orderedCollection size - 2) = 777)
			and: [( orderedCollection at: orderedCollection size - 1)
					= 888]].

	"Errors If there is no element in the receiver which is equivalent to target."
	self should: [orderedCollection
			addAll: #(111 222)
			after: 5]       
		raise: TestResult error.
!

testXaddAllXbeforeIndexX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAll:beforeIndex:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			addAll: #(222 333)
			beforeIndex: 1]
		should: [:r | (orderedCollection first) = 222
			and: [(orderedCollection at: 2) = 333]].

	self conformingValue: [orderedCollection
			addAll: #(444 555)
			beforeIndex: 4]
		should: [:r | (orderedCollection at: 4) = 444
			and: [(orderedCollection at: 5) = 555]].

	"Errors If index <=0."
	orderedCollection := self canonicalObject.
	self should: [orderedCollection
			addAll: #(111 222)
			beforeIndex: 0] 
		raise: TestResult error.

	"Errors If index > receiver's size + 1."
	self should: [orderedCollection
				addAll: #(111 222)
				beforeIndex: 6] 
		raise: TestResult error.
!

testXaddAllXbeforeX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addAll:before:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			addAll: #(222 333)
			before: 1]
		should: [:r| (orderedCollection first) = 222
			and: [(orderedCollection at: 2) = 333]].

	self conformingValue: [orderedCollection
			addAll: #(444 555)
			before: 4]
		should: [:r|
			(orderedCollection at: orderedCollection size - 2) = 444
			and: [(orderedCollection at: orderedCollection size - 1) = 555]].

	self conformingValue: [orderedCollection
			addAll: #(666 777)
			before: 555]
		should: [:r|
			(orderedCollection at: orderedCollection size - 3) = 666
			and: [(orderedCollection at: orderedCollection size - 2) = 777]].

	"Errors: If there is no element in the receiver
	which is equivalent to target."
	self should: [orderedCollection
			addAll: #(111 222)
			before: 5]      
		raise: TestResult error.
!

testXaddFirstX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addFirst:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection addFirst: 111]
	"2000/12/06 Harmon, R. Needs fix??? Draft - Return Values UNSPECIFIED."
		should:[:r | r = 111
			and: [(orderedCollection at: 1) = 111]]
!

testXaddLastX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'addLast:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection addLast: 111]
	"2000/12/06 Harmon, R. Needs fix??? Draft - Return Values UNSPECIFIED."
		should:[:r| r = 111
			and: [(orderedCollection last) = 111]]
!

testXaddX
	" Collection <OrderedCollection> "
	| orderedCollection sampleValue |
	self messageSelector: #'add:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection add: 777]      
		should: [:r| 
			orderedCollection do: [:each| sampleValue := each].
			sampleValue = 777]
!

testXaddXafterIndexX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'add:afterIndex:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			add: 777
			afterIndex: 0]
		should: [:r| (r = 777)
			and: [(orderedCollection first) = 777]].

	self conformingValue: [orderedCollection
			add: 888
			afterIndex: 5]
		should: [:r| (r = 888)
			and: [(orderedCollection last) = 888]].

	self conformingValue: [orderedCollection
			add: 999
			afterIndex: 1]
		should: [:r| (r = 999)
			and: [(orderedCollection at: 2) = 999]].

	"Errors If index < 0."
	orderedCollection := self canonicalObject.
	self should: [orderedCollection
			add: 666
			afterIndex: -1] 
		raise: TestResult error.

	"Errors If index > receiver's size."
	self should: [orderedCollection
			add: 555
			afterIndex: 5]  
		raise: TestResult error.
!

testXaddXafterX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'add:after:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			add: 777
			after: 1]
		should: [:r| orderedCollection = #(1 777 2 3 4) asOrderedCollection].

	self conformingValue: [orderedCollection
			add: 888
			after: 4]
		should: [:r|
			orderedCollection = #(1 777 2 3 4 888) asOrderedCollection].

	self conformingValue: [orderedCollection
			add: 999
			after: 777]
		should: [:r|
			orderedCollection = #(1 777 999 2 3 4 888) asOrderedCollection].

	"Errors If there is no element in the receiver which is equivalent
			to target."
	self should: [orderedCollection
			add: 777
			after: 5]       
		raise: TestResult error.
!

testXaddXbeforeIndexX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'add:beforeIndex:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			add: 777
			beforeIndex: 1]
		should: [:r | orderedCollection first = 777].

	self conformingValue: [orderedCollection
			add: 888
			beforeIndex: 6]
		should: [:r| orderedCollection last = 888].

	self conformingValue: [orderedCollection
			add: 999
			beforeIndex: 2]
		should: [:r| (orderedCollection at: 2) = 999].

	"Errors If index <=0."
	orderedCollection := self canonicalObject.
	self should: [orderedCollection
			add: 666
			beforeIndex: 0] 
		raise: TestResult error.
	self should: [orderedCollection
			add: 666
			beforeIndex: -1]        
		raise: TestResult error.

	"Errors If index > receiver's size + 1."
	self should: [orderedCollection
			add: 555
			beforeIndex: 6] 
		raise: TestResult error
!

testXaddXbeforeX
	" Collection <OrderedCollection> "
	| orderedCollection |
	self messageSelector: #'add:before:'.

	orderedCollection := self canonicalObject.
	self conformingValue: [orderedCollection
			add: 777
			before: 1]
		should: [:r| orderedCollection first = 777].

	self conformingValue: [orderedCollection
		add: 888
		before: 4]
		should: [:r| (orderedCollection at: 5) = 888].

	self conformingValue: [orderedCollection
			add: 999
			before: 888]
		should: [:r| (orderedCollection at: 5) = 999].

	"Errors: If there is no element in the receiver which is equivalent to target."
	self should: [orderedCollection
			add: 777
			before: 5]      
		raise: TestResult error
! !

!OrderedCollectionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/OrderedCollectionTest.st,v 1.1 2002/06/21 10:34:27 penk Exp $'
! !
