"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#PortableUseful
	instanceVariableNames:''
	classVariableNames:'InstExtraInfoMgr'
	poolDictionaries:''
	category:'System-PortableUseful'
!

PortableUseful comment:'The Portability/Useful Project - VW 5i.4

	It adds the VW5i.4 specific global interface to common cross-dialect solutions that may be useful as future ANSI additions (or not but just useful).

	An instance of this class represents a global interface to common functions to make code more portable and other functions just found to be generally useful. An instance of this class is created at: #''''PU'''' during class initialization.  One may use functions by sending a message to PU.

	It includes stuff like global access to classes, globals, and pools in a dialect independent way.  The CampSmalltalk SUnit folks have factored out a bunch of this stuff. They may also have come up with a portable way to handle namespaces.  I''''d like to untie these from specific applications (SIF, CampSmalltalk SUnit, etc.) and collect all of them into the Portability/Useful Project for use by all applications.

	It also includes a dialect independent interface to manage extra information for instances of <Object>.

	I found message closeTo: for numeric object to be generally useful in testing.

Contact:
	Richard A. Harmon 
	520 Second Avenue West
	Spencer, IA, USA  51301-3803
	(712) 262-5409
	email: harmonra@webname.com

Version.Release History:
0.2A 2002/04/13    DWL
0.2A  2001/04/14   Harmon, R.A.

'
!


!PortableUseful class methodsFor:'initialization'!

initialize
	"Class initialization. Initialize the singleton instance of the global 
	#'PU' entry point to portable or useful methods. 
        
	PortableUseful initialize 
	"

	self initializeGlobalPUEntryPoint.
	self initializeInstExtraInfoMgr.
! !

!PortableUseful class methodsFor:'private'!

initializeGlobalPUEntryPoint
	"Private - Initialize the singleton instance of the global #'PU' entry 
	point to portable or useful methods."

	Smalltalk at: #PU put: self basicNew
!

initializeInstExtraInfoMgr
	"Private - Initialize the singleton manager that handles instance 
	extra information for all objects."

	InstExtraInfoMgr := InstanceExtraInformation new
! !

!PortableUseful methodsFor:'accessing'!

classNamed: clsName ifAbsent: ifAbsentBlk 
	"Answer the class named, clsName, in the current environment 
	(default Smalltalk). If no class is found, answer the result of 
	evaluating the niladic valuable, absentBlk."

	^self class environment at: clsName asSymbol ifAbsent: ifAbsentBlk
!

classNamed: nameString inEnvironment: environmentObject 
	"Answer the class named nameString asSymbol in the 
	environment, environmentObject, name space. The default if no 
	name space is the system dictionary Smalltalk."

	^environmentObject at: nameString asSymbol
!

globalNamed: globalName ifAbsent: ifAbsentBlk 
	"Answer the global named, globalName, in the current 
	environment (default Smalltalk). If no global is found, answer the 
	result of evaluating the niladic valuable, absentBlk."

	^self class environment at: globalName asSymbol ifAbsent: ifAbsentBlk
!

globalNamed: nameString inEnvironment: environmentObject 
	"Answer the global named nameString asSymbol in the 
	environment, environmentObject, name space. The default if no 
	name space is the system dictionary Smalltalk."

	^environmentObject at: nameString asSymbol
!

poolNamed: poolName ifAbsent: ifAbsentBlk 
	"Answer the pool named, poolName, in the current environment 
	(default Smalltalk). If no pool is found, answer the result of 
	evaluating the niladic valuable, absentBlk."

	^self class environment at: poolName asSymbol ifAbsent: ifAbsentBlk
!

poolNamed: nameString inEnvironment: environmentObject 
	"Answer the pool named nameString asSymbol in the 
	environment, environmentObject, name space. The default if no 
	name space is the system dictionary Smalltalk."

	^environmentObject at: nameString asSymbol
! !

!PortableUseful methodsFor:'constants'!

dialectAndVersionString
	"Answer the dialect and version.release.patchLevel <String>."

" XXX DWL: more trouble than it is worth for now. XXX
	| verSubstrs dialectVer lastUpdate |
	verSubstrs := Smalltalk version subStrings.
	dialectVer := verSubstrs at: 1.
	lastUpdate := Smalltalk lastUpdateString.
	^(dialectVer copyFrom: 1 to: 6), ' ',
	(dialectVer copyFrom: 7 to: (dialectVer size)), '.',
	(lastUpdate copyFrom: (lastUpdate size - 3) to: (lastUpdate size))

"
!

dialectAndVersionTag
        "Answer the dialect and version.release tag <symbol>. Dialect 
        specific. 
        Note: I recommend using the same scheme as used for naming 
        SUnit files. 
        
        dialect (versions)                      Suggested Symbol Returned 
        ---------------------------             ------------------------- 
        Dolphin (3.0, 4.0)                      #'DO30' #'DO40' 
        Gemstone (5.x)                          ^#'GS514' 
        ObjectStudio (6.2)                      ^#'OS62' 
        Smalltalk MT (2.6)                      ^#'MT26' 
        Smalltalk/X (3.6.* 4.1.*)               #'STX36' #'STX41'
        Squeak (2.7)                            ^#'SQ27' 
        VisualAge (4.5, 5.0)                    ^#'VA50' 
        VisualSmalltalk (3.1x)          ^#'VS312' 
        VisualWorks (2.5.2, 3.0, 5i, 7) ^#'VW252' ^#'VW30' ^#'VW5i' ^#'VW7'
        "

        ^('STX', Smalltalk majorVersionNr asString, Smalltalk minorVersionNr asString) asSymbol
! !

!PortableUseful methodsFor:'creation'!

newGlobalConstantNamed: globalName ifExists: existsBlk 
	"Create and answer a new global constant named, globalName, in 
	the current environment (default Smalltalk) with an initial value of 
	nil. 
	If the global by that name already exits, answer the result of 
	evaluating the niladic valuable, existsBlk."

	self globalNamed: globalName ifAbsent: [^self class environment at: globalName asSymbol put: nil].
	^existsBlk value
!

newGlobalNamed: globalName ifExists: existsBlk 
	"Create and answer a new global variable named, globalName, in 
	the current environment (default Smalltalk) with an initial value of 
	nil. 
	If the global by that name already exits, answer the result of 
	evaluating the niladic valuable, existsBlk."

	self globalNamed: globalName ifAbsent: [^self class environment at: globalName asSymbol put: nil].
	^existsBlk value
!

newPoolConstantNamed: poolName ifExists: existsBlk 
	"Create and answer a new constant pool named, poolName, in the 
	current environment (default Smalltalk) with an initial value of an 
	empty PoolConstantsDictionary. 
	If the pool by that name already exits, answer the result of 
	evaluating the niladic valuable, existsBlk."

	self poolNamed: poolName ifAbsent: [^self class environment at: poolName asSymbol put: PoolConstantsDictionary new].
	^existsBlk value
!

newPoolNamed: poolName ifExists: existsBlk 
	"Create and answer a new pool named, poolName, in the current 
	environment (default Smalltalk) with an initial value of an empty 
	PoolDictionary. 
	If the pool by that name already exits, answer the result of 
	evaluating the niladic valuable, existsBlk."

	self poolNamed: poolName ifAbsent: [^self class environment at: poolName asSymbol put: PoolDictionary new].
	^existsBlk value
! !

!PortableUseful methodsFor:'error handling'!

errorNotFoundTag: tagObject in: receiverObject 
	"Signal an error with tagObject indicating something was not 
	found in receiverObject."

	^NotFoundError signalReceiver: receiverObject withTag: tagObject
! !

!PortableUseful methodsFor:'initializing'!

globalNamed: globalName initialValue: initialValue ifAbsent: absentBlk 
	"Initialize and answer a new global variable named, globalName, 
	in the current environment (default Smalltalk) with an initial value 
	of initialValue. 
	If no global is found, answer the result of evaluating the niladic 
	valuable, absentBlk."

	self globalNamed: globalName ifAbsent: absentBlk.
	^self class environment at: globalName asSymbol put: initialValue.
! !

!PortableUseful methodsFor:'instance extra information'!

extraInfoFor: instance at: extraKey 
	"Answer instance's <Object> extra information at extraKey 
	<Symbol>, or if not found signal an error."

	^InstExtraInfoMgr extraInfoFor: instance at: extraKey
!

extraInfoFor: instance at: extraKey ifAbsent: absentBlock 
	"Answer instance's <Object> extra information at extraKey 
	<Symbol>, or the result of evaluating absentBlock if not found."

	^InstExtraInfoMgr
		extraInfoFor: instance
		at: extraKey asSymbol
		ifAbsent: absentBlock
!

extraInfoFor: instance at: extraKey put: extraValue 
	"Set instance's <Object> extra information at extraKey <Symbol> 
	to extraValue <Object>, and answer extraValue."

	^InstExtraInfoMgr
		extraInfoFor: instance
		at: extraKey asSymbol
		put: extraValue
!

removeAllExtraInfoFor: instance 
	"Remove all instance's <Object> extra information."

	^InstExtraInfoMgr removeAllExtraInfoFor: instance
!

removeExtraInfoFor: instance at: extraKey 
	"Remove instance's <Object> extra information at extraKey 
	<Symbol>, or if not found signal an error."

	^InstExtraInfoMgr removeExtraInfoFor: instance at: extraKey asSymbol
!

removeExtraInfoFor: instance at: extraKey ifAbsent: absentBlock 
	"Remove instance's <Object> extra information at extraKey 
	<Symbol>, or the result of evaluating absentBlock if not found."

	^InstExtraInfoMgr
		removeExtraInfoFor: instance
		at: extraKey asSymbol
		ifAbsent: absentBlock
! !

!PortableUseful methodsFor:'removing'!

removeClassNamed: className ifAbsent: ifAbsentBlk 
	"Remove the class named, className, from the current 
	environment (default Smalltalk). If no class is found, answer the 
	result of evaluating the niladic valuable, absentBlk. If the class 
	has subclassesthen signal an error"

	| classObj |
	classObj := Smalltalk at: className asSymbol ifAbsent: [].
	classObj isNil ifTrue: [^ifAbsentBlk value].
	classObj subclasses size > 0 ifTrue: [self error: 'Class named: "' , className , '" has subclasses.'].
	classObj removeFromSystem
!

removeGlobalNamed: globalName ifAbsent: ifAbsentBlk 
	"Remove the global named, globalName, from the current 
	environment (default Smalltalk). If no global is found, answer the 
	result of evaluating the niladic valuable, absentBlk."

	^self class environment removeKey: globalName asSymbol ifAbsent: ifAbsentBlk
!

removePoolNamed: poolName ifAbsent: ifAbsentBlk 
	"Remove the pool named, poolName, from the current 
	environment (default Smalltalk). If no pool is found, answer the 
	result of evaluating the niladic valuable, absentBlk."

	^self class environment removeKey: poolName asSymbol ifAbsent: ifAbsentBlk
! !

!PortableUseful class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/PortableUseful.st,v 1.1 2002/06/21 10:30:59 penk Exp $'
! !
PortableUseful initialize!
