"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#ReadableStringHelper
	instanceVariableNames:'subseq'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!ReadableStringHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<readableString>'
! !

!ReadableStringHelper methodsFor:'helpers'!

createSubseqDictionary
	"Create a dictionary of subsequences of the canonical object."
	| canObj canObjCls |
	canObj := testCase canonicalObject.
	canObjCls := canObj species. 
	subseq := Dictionary new.
	subseq at: #'a' put: (canObjCls with: (canObj at: 1)).
	subseq at: #'b' put: (canObjCls with: (canObj at: 2)).
	subseq at: #'c' put: (canObjCls with: (canObj at: 3)).
	subseq at: #'d' put: (canObjCls with: (canObj at: 4)).

	subseq at: #'ab'
		put: (canObjCls
				with: (canObj at: 1)
				with: (canObj at: 2)).
	subseq at: #'bc'
		put: (canObjCls
				with: (canObj at: 2)
				with: (canObj at: 3)).
	subseq at: #'cd'
		put: (canObjCls
				with: (canObj at: 3)
				with: (canObj at: 4)).
	subseq at: #'abc'
		put: (canObjCls
				with: (canObj at: 1)
				with: (canObj at: 2)
				with: (canObj at: 3)).
	subseq at: #'bcd'
		put: (canObjCls
				with: (canObj at: 2)
				with: (canObj at: 3)
				with: (canObj at: 4)).
	subseq at: #'abcd'
		put: (canObjCls
				with: (canObj at: 1)
				with: (canObj at: 2)
				with: (canObj at: 3)
				with:(canObj at: 4)).
!

setUp

	super setUp.
	self createSubseqDictionary.
! !

!ReadableStringHelper methodsFor:'testing'!

testXasLowercase
	" Collection <readableString> "
	self messageSelector: #'asLowercase'.

	self conformingAssertMessageSelector.
!

testXasString
	" Collection <readableString> "
	self messageSelector: #'asString'.

	self conformingAssertMessageSelector.
!

testXasSymbol
	" Collection <readableString> "
	| readableString |
	self messageSelector: #'asSymbol'.

	"Answer a symbol containing the same characters as the receiver,
		in their original order."
	readableString := testCase canonicalObject.
	self conformingValue: [readableString asSymbol]
		should: [:r|
			1 to: r size do: [:ndx |
				(r at: ndx) = (readableString  at: ndx) ifFalse: [
					^false]].
			r size = readableString size].
!

testXasUppercase
	" Collection <readableString> "
	| rString |
	self messageSelector: #'asUppercase'.

	"Answer a new string which contains all of the elements of
	the receiver converted to their upper case equivalents.  Individual
	element of the string are converted as if they were receivers of
	the message #asUppercase."
	rString := testCase canonicalObject.
	self conformingValue: [rString asUppercase]
		should: [:r|
			1 to: r size do: [:ndx |
				(r at: ndx) = ((rString  at: ndx) asUppercase) ifFalse: [
					^false]].
			r size = rString size].
!

testXconcatenateOp
	" Collection <readableString> "
	self messageSelector: #','.

	self conformingValue: [(subseq at: #'ab') , (subseq at: #'cd')]
		should: [:r| r = (subseq at: #'abcd')].
!

testXcopyReplaceAllXwithX
	" Collection <readableString> "
	self messageSelector: #'copyReplaceAll:with:'.

	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceAll:(subseq at: #'bc')
				with: (subseq at: #'bcd')]
		should: [:r|
			r = ((subseq at: #'abcd'), (subseq at: #'d'))].
!

testXcopyReplaceFromXtoXwithX
	" Collection <readableString> "
	self messageSelector: #'copyReplaceFrom:to:with:'.

	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 3
				to: 2
				with: (subseq at: #'bc')]
		should: [:r|
			r = ((subseq at: #'ab'), (subseq at: #'bc'),
					(subseq at: #'cd'))].
 
	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 5
				to: 4
				with: (subseq at: #'bc')]
		should: [:r| r = ((subseq at: #'abcd'), (subseq at: #'bc'))].

	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 1
				to: 3
				with: (subseq at: #'bc')]
		should: [:r| r = (subseq at: #'bcd')].
!

testXcopyReplacingXwithObjectX
	" Collection <readableString> "
	| a b c chrb chrc |
	self messageSelector: #'copyReplacing:withObject:'.

	a := subseq at: #'a'.
	b := subseq at: #'b'.
	c := subseq at: #'c'.
	chrb := b at: 1.
	chrc := c at: 1.
	self conformingValue: [((subseq at: #'ab'), (subseq at: #'bc'))
			copyReplacing: chrb
			withObject: chrc]
		should: [:r| r = (a, c, c, c)].
!

testXcopyWithX
	" Collection <readableString> "
	| chrd|
	self messageSelector: #'copyWith:'.

	chrd := (subseq at: #'d') at: 1.
	self conformingValue: [(subseq at: #'abc') copyWith: chrd]
		should: [:r| r = (subseq at: #'abcd')]
!

testXgreaterThanOp
	" Collection <readableString> "
	| copy |
	self messageSelector: #'>'.

	self conformingValue: [(subseq at: #'abc') > (subseq at: #'ab')]
		should: [:r| r].

	copy := (subseq at: #'abc') copy.
	self conformingValue: [(subseq at: #'abc') > copy]
		shouldnt: [:r| r].
!

testXgreaterThanOrEqualToOp
	" Collection <readableString> "
	| copy |
	self messageSelector: #'>='.

	self conformingValue: [(subseq at: #'bcd') >= (subseq at: #'abc')]
		should: [:r| r].

	copy := (subseq at: #'abc') copy.
	self conformingValue: [(subseq at: #'abc') >= copy]
		should: [:r| r].

	self conformingValue: [(subseq at: #'ab') >= (subseq at: #'bc')]
		shouldnt: [:r| r].
	self conformingValue: [(subseq at: #'ab') >= (subseq at: #'abc')]
		shouldnt: [:r| r].
!

testXlessThanOp
	" Collection <readableString> "
	| copy |
	self messageSelector: #'<'.

	self conformingValue: [(subseq at: #'ab') < (subseq at: #'abc')]
		should: [:r| r].

	copy := (subseq at: #'abc') copy.
	self conformingValue: [(subseq at: #'abc') < copy]
		shouldnt: [:r| r].
!

testXlessThanOrEqualToOp
	" Collection <readableString> "
	| copy |
	self messageSelector: #'<='.

	self conformingValue: [(subseq at: #'abc') <= (subseq at: #'bcd')]
		should: [:r| r].

	copy := (subseq at: #'abc') copy.
	self conformingValue: [(subseq at: #'abc') <= copy]
		should: [:r| r].
	self conformingValue: [(subseq at: #'ab') <= (subseq at: #'bc')]
		should: [:r| r].

	self conformingValue: [(subseq at: #'bc') <= (subseq at: #'ab')]
		shouldnt: [:r| r].
	self conformingValue: [(subseq at: #'abc') <= (subseq at: #'ab')]
		shouldnt: [:r| r].
!

testXsameAsX
	" Collection <readableString> "
	| copy |
	self messageSelector: #'sameAs:'.

	copy := (subseq at: #'abc') copy.
	self conformingValue: [(subseq at: #'abc') sameAs: copy]
		should: [:r| r].

	self conformingValue: [(subseq at: #'ab') sameAs: (subseq at: #'abc')]
		shouldnt: [:r| r]
!

testXsubStringsX
	" Collection <readableString> "
	| aStream |
	self messageSelector: #'subStrings:'.

	aStream := WriteStream with: String new.
	aStream
		nextPutAll: 'sub1'; cr;
		nextPutAll: 'sub2'; nextPut: (Character lf);
		nextPutAll: 'sub3'; space;
		nextPutAll: 'sub4'; tab;
		nextPutAll: 'sub5'.

	self conformingValue: [aStream contents
			subStrings: SysProProtocolsManager whitespaceSeparators]
		should: [:r | r = #('sub1' 'sub2' 'sub3' 'sub4' 'sub5')].

	self conformingValue: ['sub1, sub2 sub3' subStrings: ', ']
		should: [:r | r = #('sub1' 'sub2' 'sub3')].

	"Errors: If the list of separators contains anything other
			than characters."
	self should: [self canonicalObject subStrings: #($, 9 $-)]
		raise: TestResult error.
! !

!ReadableStringHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/ReadableStringHelper.st,v 1.1 2002/06/21 10:35:47 penk Exp $'
! !
