"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#SequencedReadableCollectionHelper
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!SequencedReadableCollectionHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<sequencedReadableCollection>'
! !

!SequencedReadableCollectionHelper methodsFor:'constants'!

canonicalObjects
	"Answer a dictionary of subsequences of the canonical object."
	| can cl |
	can := testCase canonicalObject.
	cl := can species. 
	^(Dictionary new)
		at: #'a' put: (cl with: (can at: 1));
		at: #'b' put: (cl with: (can at: 2));
		at: #'c' put: (cl with: (can at: 3));
		at: #'d' put: (cl with: (can at: 4));
		at: #'ab' put: (cl with: (can at: 1) with: (can at: 2));
		at: #'bc' put: (cl with: (can at: 2) with: (can at: 3));
		at: #'cd' put: (cl with: (can at: 3) with: (can at: 4));
		at: #'abc' put: (cl with: (can at: 1) with: (can at: 2) with: (can at: 3));
		at: #'bcd' put: (cl with: (can at: 2) with: (can at: 3) with: (can at: 4));
		at: #'abcd' put: (cl with: (can at: 1) with: (can at: 2) with: (can at: 3) with:(can at: 4));
		yourself
! !

!SequencedReadableCollectionHelper methodsFor:'testing'!

testXafterX
	" Collection <sequencedReadableCollection> "
	| co |
	self messageSelector: #'after:'.

	co := testCase canonicalObject.
	self conformingValue: [co after: (co at: 2)]
		should: [:r| r = (co at: 3)]
!

testXatX
	" Collection <sequencedReadableCollection> "
	| cos b |
	self messageSelector: #'at:'.

	cos := self canonicalObjects.
	b := (cos at: #'b') at: 1.
	self conformingValue: [(cos at: #'abcd') at: 2]
		should: [:r| r = b]
!

testXatXifAbsentX
	" Collection <sequencedReadableCollection> "
	| cos b |
	self messageSelector: #'at:ifAbsent:'.

	cos := self canonicalObjects.
	b := (cos at: #'b') at: 1.
	self conformingValue: [(cos at: #'abcd') at: 2 ifAbsent: [nil]]
		should: [:r| r = b].

	self conformingValue: [(cos at: #'abcd') at: 47 ifAbsent: [nil]]
		should: [:r| r isNil]
!

testXbeforeX
	" Collection <sequencedReadableCollection> "
	| co |
	self messageSelector: #'before:'.

	co := testCase canonicalObject.
	self conformingValue: [co before: (co at: 3)]
		should: [:r| r = (co at: 2)]
!

testXconcatenateOp
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #','.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'ab') , (cos at: #'cd')]
		should: [:r| r = (cos at: #'abcd')]
!

testXcopyFromXtoX
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #'copyFrom:to:'.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'abcd') copyFrom: 2 to: 3]
		should: [:r| r = (cos at: #'bc')].

	self conformingValue: [(cos at: #'abcd') copyFrom: 3 to: 2]
		should: [:r| r isEmpty]
!

testXcopyReplaceAllXwithX
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #'copyReplaceAll:with:'.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'abcd')
			copyReplaceAll:(cos at: #'bc')
			with: (cos at: #'bcd')]
		should: [:r| r = ((cos at: #'abcd'), (cos at: #'d'))].
!

testXcopyReplaceFromXtoXwithObjectX
	" Collection <sequencedReadableCollection> "
	| cos c |
	self messageSelector: #'copyReplaceFrom:to:withObject:'.

	cos := self canonicalObjects.
	c := (cos at: #'c').
	self conformingValue: [(cos at: #'abcd')
			copyReplaceFrom: 3
			to: 2
			withObject:((cos at: #'b') at: 1)]
		should: [:r| r = ((cos at: #'ab'), (cos at: #'bcd'))].

	self conformingValue: [(cos at: #'abc')
			copyReplaceFrom: 4
			to: 3
			withObject: ((cos at: #'d') at: 1)]
		should: [:r| r = (cos at: #'abcd')].

	self conformingValue: [(cos at: #'abc')
			copyReplaceFrom: 1
			to: 2
			withObject: (c at:1)]
		should: [:r| r = (c, c, c)].
!

testXcopyReplaceFromXtoXwithX
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #'copyReplaceFrom:to:with:'.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'abcd')
			copyReplaceFrom: 3
			to: 2
			with: (cos at: #'bc')]
		should: [:r| r = ((cos at: #'ab'), (cos at: #'bc'), (cos at: #'cd'))].
 
	self conformingValue: [(cos at: #'abcd')
			copyReplaceFrom: 5
			to: 4
			with: (cos at: #'bc')]
		should: [:r| r = ((cos at: #'abcd'), (cos at: #'bc'))].

	self conformingValue: [(cos at: #'abcd')
			copyReplaceFrom: 1
			to: 3
			with: (cos at: #'bc')]
		should: [:r| r = (cos at: #'bcd')].
!

testXcopyReplacingXwithObjectX
	" Collection <sequencedReadableCollection> "
	| cos c a b |
	self messageSelector: #'copyReplacing:withObject:'.

	cos := self canonicalObjects.
	a := cos at: #'a'.
	b := cos at: #'b'.
	c := cos at: #'c'.
	self conformingValue: [((cos at: #'ab'), (cos at: #'bc'))
			copyReplacing: (b at: 1)
			withObject: (c at: 1)]
		should: [:r| r = (a, c, c, c)].
!

testXcopyWithX
	" Collection <sequencedReadableCollection> "
	| cos d |
	self messageSelector: #'copyWith:'.

	cos := self canonicalObjects.
	d := cos at:#d.
	self conformingValue: [(cos at: #'abc') copyWith: (d at:1)]
		should: [:r| r = (cos at: #'abcd')]
!

testXcopyWithoutX
	" Collection <sequencedReadableCollection> "
	| cos c a b |
	self messageSelector: #'copyWithout:'.

	cos := self canonicalObjects.
	a := cos at: #'a'.
	b := cos at: #'b'.
	c := cos at: #'c'.
	self conformingValue: [(cos at: #'abc') copyWithout: (b at: 1)]
		should: [:r| r = (a, c)]
!

testXdoX
	" Collection <sequencedReadableCollection> "
	| collection |
	self messageSelector: #'do:'.

	"Refinement: <sequenceReadableCollection>
		The operation is evaluated with each element of the receiver in
	indexed order starting at 1.  The first element is at index 1, the
	second at index 2, etc.  The index of the last element is equal to
	the receiver's size."
	collection := testCase canonicalObject.
	self conformingValue: [collection do: [:each| ]]
		should: [:result| "UNSPECIFIED" true].

	"No error when sending message that causes an error in the do: loop."
	self conformingValue: [
			testCase emptyCollection do: [:each| each gobbledegook]]
		should: [:result| "UNSPECIFIED" true].

	"Send message that causes an error in the do: loop."
	self should: [collection do: [:each| each gobbledegook]]
		raise: TestResult error.
!

testXequalityOp
	" Collection <sequencedReadableCollection> "
	| copy cos |
	self messageSelector: #'='.

	cos := self canonicalObjects.
	copy := (cos at: #'abc') copy.
	self conformingValue: [(cos at: #'abc') = copy]
		should: [:r| r].

	self conformingValue: [(cos at: #'ab') = copy]
		shouldnt: [:r| r]
!

testXfindFirstX
	" Collection <sequencedReadableCollection> "
	| cos d b it |
	self messageSelector: #'findFirst:'.

	cos := self canonicalObjects.
	b := (cos at: #'b') at: 1.
	d := (cos at: #'d') at: 1.
	it := (cos at: #'ab'), (cos at: #'bc').
	self conformingValue: [it findFirst: [:e| e = b]]
		should: [:r| r = 2].

	self conformingValue: [(cos at: #'abc') findFirst: [:e| e = d]]
		should: [:r| r = 0].
!

testXfindLastX
	" Collection <sequencedReadableCollection> "
	| cos d b it |
	self messageSelector: #'findLast:'.

	cos := self canonicalObjects.
	b := (cos at: #'b') at: 1.
	d := (cos at: #'d') at: 1.
	it := (cos at: #'ab'), (cos at: #'bc').
	self conformingValue: [it findLast: [:e| e = b]]
		should: [:r| r = 3].

	self conformingValue: [(cos at:#'abc') findLast: [:e| e = d]]
		should: [:r| r = 0].
!

testXfirst
	" Collection <sequencedReadableCollection> "
	| cos a |
	self messageSelector: #'first'.

	cos := self canonicalObjects.
	a := (cos at: #'a') at: 1.
	self conformingValue: [(cos at: #'abcd') first]
		should: [:r| r = a].
!

testXfromXtoXdoX
	" Collection <sequencedReadableCollection> "
	| co col ndx |
	self messageSelector: #'from:to:do:'.

	co := testCase canonicalObject.
	col := co copy.
	ndx := 0.
	self conformingValue: [co from: 1 to: co size do: [:e|
			ndx := ndx + 1.
			(col at: ndx) = e]]
		should: [:r| co = col].
!

testXfromXtoXkeysAndValuesDoX
	" Collection <sequencedReadableCollection> "
	| co col ndx |
	self messageSelector: #'from:to:keysAndValuesDo:'.

	co := testCase canonicalObject.
	col := co copy.
	ndx := 0.
	self conformingValue: [co from: 1 to: 4 keysAndValuesDo: [:i :e|
			ndx := ndx + 1.
			(col at: ndx) = e]]
		should: [:r| col = co]
!

testXindexOfSubCollectionXstartingAtX
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #'indexOfSubCollection:startingAt:'.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'abcd')
			indexOfSubCollection:(cos at: #'bc')
			startingAt: 1]
		should: [:r| r = 2].

	self conformingValue: [(cos at: #'abcd')
			indexOfSubCollection: (cos at: #'bc')
			startingAt: 3]
		should: [:r| r = 0].
!

testXindexOfSubCollectionXstartingAtXifAbsentX
	" Collection <sequencedReadableCollection> "
	| cos |
	self messageSelector: #'indexOfSubCollection:startingAt:ifAbsent:'.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'abcd')
			indexOfSubCollection:(cos at: #'bc')
			startingAt: 1
			ifAbsent: [nil]]
		should: [:r| r = 2].

	self conformingValue: [(cos at: #'abcd')
			indexOfSubCollection: (cos at: #'bc')
			startingAt: 3
			ifAbsent: [nil]]
		should: [:r| r isNil].
!

testXindexOfX
	" Collection <sequencedReadableCollection> "
	| cos c |
	self messageSelector: #'indexOf:'.

	cos := self canonicalObjects.
	c := (cos at: #'c') at: 1.
	self conformingValue: [(cos at: #'abcd') indexOf: c]
		should: [:r| r = 3].

	self conformingValue: [(cos at: #'abcd') indexOf: 42]
		should: [:r| r = 0].
!

testXindexOfXifAbsentX
	" Collection <sequencedReadableCollection> "
	| cos c |
	self messageSelector: #'indexOf:ifAbsent:'.

	cos := self canonicalObjects.
	c := (cos at: #'c') at: 1.
	self conformingValue: [(cos at: #'abcd')
			indexOf: c
			ifAbsent:[nil]]
	    should: [:r| r = 3].

	self conformingValue: [(cos at: #'abcd')
			indexOf: 42
			ifAbsent:[nil]]
		should: [:r| r isNil].
!

testXkeysAndValuesDoX
	" Collection <sequencedReadableCollection> "
	| collection valueList |
	self messageSelector: #'keysAndValuesDo:'.

	"Evaluate operation with the index of each element of the receiver,
	in order, together with the element itself."
	collection := testCase canonicalObject.
	valueList := Array new: (collection size).
	self conformingValue: [
			collection keysAndValuesDo: [:key :value|
				valueList
					at: key
					put: value]]
		should: [:r|
			1 to: valueList size do: [:ndx |
				(valueList at: ndx) = (collection  at: ndx) ifFalse: [
					^false]].
			valueList size = collection size].

	"Errors: If the elements of the receiver are inappropriate for use
			as arguments to operation."
	self should: [collection keysAndValuesDo: [:key :value| key, 'a']]
		raise: TestResult error
!

testXlast
	" Collection <sequencedReadableCollection> "
	| cos d |
	self messageSelector: #'last'.

	cos := self canonicalObjects.
	d := (cos at: #'d') at: 1.
	self conformingValue:[(cos at: #abcd) last]
		should: [:r| r = d].
!

testXreverse
	" Collection <sequencedReadableCollection> "
	| cos abc |
	self messageSelector: #'reverse'.

	cos := self canonicalObjects.
	abc := cos at: #'abc'.
	self conformingValue: [abc reverse]
		should: [:r| r size = 3
			and: [((r at: 1) = (abc at: 3))
			and: [(r at: 3) = (abc at: 1)]]]
!

testXreverseDoX
	" Collection <sequencedReadableCollection> "
	| col cos abc |
	self messageSelector: #'reverseDo:'.

	cos := self canonicalObjects.
	abc := cos at: #'abc'.
	col := OrderedCollection new.
	self conformingValue: [abc reverseDo: [:e| col add: e]]
		should: [:r| col = abc reverse asOrderedCollection].
!

testXwithXdoX
	" Collection <sequencedReadableCollection> "
	| col cos abc bcd |
	self messageSelector: #'with:do:'.

	cos := self canonicalObjects.
	abc := cos at: #'abc'.
	bcd := cos at: #'bcd'.
	col := OrderedCollection new.
	self conformingValue: [abc with: bcd do: [:e1 :e2| col add: (e1 = e2)]]
	    should: [:r| col = #(false false false) asOrderedCollection].
! !

!SequencedReadableCollectionHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SequencedReadableCollectionHelper.st,v 1.1 2002/06/21 10:37:15 penk Exp $'
! !
