"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseHelper subclass:#SequencedStreamHelper
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Stream'
!


!SequencedStreamHelper class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<sequencedStream>'
! !

!SequencedStreamHelper methodsFor:'testing'!

testXclose
	" Stream <sequencedStream> "
	self messageSelector: #'close'.

self anomaly: [
	self conformingAssertMessageSelector.
] for: #(#'SQ27' "2001/04/21 Harmon, R. dnu error").

	self notDone."Don't know how to check this:
If the receiver is a write-back stream update its stream backing store as if the message #flush was sent to the receiver. Then eliminate any association between the receiver and its stream backing store. Any system resources associated with the association should be released. The effect of sending any message to the receiver subsequent to this message is undefined.
"
!

testXcontents
	" Stream <sequencedStream> "
	| seqStream list list2 |
	self messageSelector: #'contents'.

	seqStream := testCase canonicalObject.
	seqStream reset.

self anomaly: [
	self conformingValue: [seqStream contents]
		should: [:r| r = testCase canonicalContents
				& r size = seqStream size].
] for: #(#'DO30' "2000/12/06 Harmon, R. Definition: <sequencedStream>
Returns a collection that contains the receiver's past and future sequence values, in order.
contents returns only past (up to position) ignoring future.").
!

testXisEmpty
	" Stream <sequencedStream> "
	self messageSelector: #'isEmpty'.

	self conformingValue: [testCase canonicalObject isEmpty]
		shouldnt: [:r| r].
!

testXposition
	" Stream <sequencedStream> "
	| seqStream |
	self messageSelector: #'position'.

	seqStream := testCase canonicalObject.
	seqStream reset.
	self conformingValue: [seqStream position]
		should: [:r| r = 0].
!

testXpositionX
	" Stream <sequencedStream> "
	| seqStream list list2 |
	self messageSelector: #'position:'.

	seqStream := testCase canonicalObject.
	seqStream reset.

	"Errors: If the receiver has any sequence values and amount
			is greater than or equal to the total number
			of sequence values of the receiver."
self anomaly: [
	self should: [seqStream position: 99]
		raise: TestResult error.
] for: #(#'DO30' "2000/12/06 Harmon, R. Equal to the total number- no error").

	seqStream position: 4.

	"Past sequence values smaller than amount:"
	self conformingValue: [seqStream position: 6]
		should: [:r| true "UNSPECIFIED"].

	"Past sequence values greater than amount:"
	self conformingValue: [seqStream position: 2]
		should: [:r| true "UNSPECIFIED"].

	"Receiver's past sequence values is equal to amount:"
	self conformingValue: [seqStream position: 2]
		should: [:r| true "UNSPECIFIED"].

	"Errors: If amount is negative."
	self should: [seqStream position: -1]
		raise: TestResult error.
!

testXreset
	" Stream <sequencedStream> "
	| seqStream list list2 |
	self messageSelector: #'reset'.

	seqStream := testCase canonicalObject.
	seqStream reset.

	self conformingValue: [seqStream reset]
		should: [:r| seqStream position = 0].

	self conformingValue: [seqStream reset]
		should: [:r| seqStream position = 0].
!

testXsetToEnd
	" Stream <sequencedStream> "
	| seqStream list list2 |
	self messageSelector: #'setToEnd'.

	seqStream := testCase canonicalObject.
	seqStream reset.

	self conformingValue: [seqStream setToEnd]
		should: [:r| seqStream position = seqStream contents size].
! !

!SequencedStreamHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SequencedStreamHelper.st,v 1.1 2002/06/21 10:36:26 penk Exp $'
! !
