"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#SetFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!SetFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^SetTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<Set factory>'
! !

!SetFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^SetTest returnTypeHasLimitedElementTypes
! !

!SetFactoryTest methodsFor:'constants'!

canonicalCollection

	^SetTest canonicalCollection
!

canonicalElement

	^SetTest canonicalElement
!

canonicalObject

	^Set
! !

!SetFactoryTest methodsFor:'testing'!

testXnew
	" ANSI-Collection <Set factory> "
	self messageSelector: #'new'.

	self conformingValue: [Set new]
		should: [:r| r isEmpty].
!

testXnewX
	" ANSI-Collection <Set factory> "
	self messageSelector: #'new:'.

	"Refinement: <Set factory>
	The parameter count represents a hint for space allocation.  The new
	collection is to optimized to contain count elements.  If the value of
	count is zero the collection should be optimize to hold an arbitrary
	number of elements.  The new collection initially contains no elements."
	self conformingValue: [Set new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [Set new: 3]
		should: [:r| r isEmpty].
!

testXwithAllX
	" ANSI-Collection <Set factory> "
	| collection |
	self messageSelector: #'withAll:'.

	"Refinement: <Set factory>
	The result is unspecified if newElements contains nil."
	collection := self canonicalCollection.
	self conformingValue: [Set withAll: collection]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the elements of newElements do not meet
			the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [Set withAll: illegalElements]
			raise: TestResult error]
!

testXwithX
	" ANSI-Collection <Set factory> "
	| element |
	self messageSelector: #'with:'.

	"Refinement: <Set factory>
		The result is undefined if any of the arguments are nil."
	element := self canonicalElement.
	self conformingValue: [Set with: element]
		should: [:r|
			r size = 1
			& r any = element].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		self should: [Set with: (self class limitedElementTypes)]
			raise: TestResult error].
!

testXwithXwithX
	" ANSI-Collection <Set factory> "
	| collection |
	self messageSelector: #'with:with:'.

	"Refinement: <Set factory>
		The result is undefined if any of the arguments are nil."
	collection := self canonicalCollection asArray
		copyFrom: 1
		to: 2.
	self conformingValue: [
			Set
				with: collection first
				with: collection last]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Set
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" ANSI-Collection <Set factory> "
	| collection |
	self messageSelector: #'with:with:with:'.

	"Refinement: <Set factory>
		The result is undefined if any of the arguments are nil."
	collection := self canonicalCollection asArray
		copyFrom: 1
		to: 3.
	self conformingValue: [
			Set
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Set
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" ANSI-Collection <Set factory> "
	| collection |
	self messageSelector: #'with:with:with:with:'.

	"Refinement: <Set factory>
		The result is undefined if any of the arguments are nil."
	collection := self canonicalCollection asArray.
	self conformingValue: [
			Set
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)
				with: (collection at: 4)]
		should: [:r| (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			Set
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!SetFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SetFactoryTest.st,v 1.1 2002/06/21 10:30:28 penk Exp $'
! !
