"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#SortedCollectionFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!SortedCollectionFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^SortedCollectionTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SortedCollection factory>'
! !

!SortedCollectionFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^SortedCollectionTest returnTypeHasLimitedElementTypes
! !

!SortedCollectionFactoryTest methodsFor:'constants'!

canonicalCollection

	^SortedCollectionTest canonicalCollection
!

canonicalElement

	^SortedCollectionTest canonicalElement
!

canonicalObject

	^SortedCollection
! !

!SortedCollectionFactoryTest methodsFor:'testing'!

testXnew
	" ANSI-Collection <SortedCollection factory> "
	self messageSelector: #'new'.

	self conformingValue: [SortedCollection new]
		should: [:r| r isEmpty].
!

testXnewX
	" ANSI-Collection <SortedCollection factory> "
	self messageSelector: #'new:'.

	"Refinement: <SortedCollection factory>
		The parameter count represents an estimate of the maximum number
	of elements in the collection.  The representation may be optimized
	for this size.
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements."
	self conformingValue: [SortedCollection new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [SortedCollection new: 3]
		should: [:r| r isEmpty].
!

testXsortBlockX
	" ANSI-Collection <SortedCollection factory> "
	self messageSelector: #'sortBlock:'.

	"Definition: <SortedCollection factory>
		Return a new sorted collection with sortBlock as the sort block.
	The sortBlock specifies the ordering criteria for the new collection
	and is a two-parameter valuable, which when evaluated with any two
	elements in the receiver, answers true if the first parameter should
	be ordered before the second parameter, and false otherwise.  The
	sort block must obey the following properties:
		1. Given the same two parameters, the sort block must answer
			the same result.
		2. The sort block must obey transitivity. For example, if a
			is before b, and b is before c, then a must be before c."
	self conformingValue: [SortedCollection sortBlock: [:a :b| a > b]]
		should: [:r| r isEmpty].
!

testXwithAllX
	" ANSI-Collection <SortedCollection factory> "
	| collection |
	self messageSelector: #'withAll:'.

	"Refinement: <SortedCollection factory>
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements.  The initial elements are ordered according to this sort
	block."
	collection := self canonicalCollection.
	self conformingValue: [SortedCollection withAll: collection]
		should: [:r|
			1 to: (r size - 1) do: [:ndx |
				(r at: ndx) <= (r at: (ndx + 1)) ifFalse: [
					^false]].
			(r difference: collection) isEmpty
			& r size = 4].

	"Errors: If any element of newElements is not appropriate
			as a parameter to the default sort block."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [SortedCollection withAll: illegalElements]
			raise: TestResult error]
!

testXwithX
	" ANSI-Collection <SortedCollection factory> "
	| element |
	self messageSelector: #'with:'.

	"Refinement: <SortedCollection factory>
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements.  The initial elements are ordered according to this sort
	block."
	element := self canonicalElement.
	self conformingValue: [SortedCollection with: element]
		should: [:r|
			r size = 1
			& r last = element].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		self should: [
				SortedCollection with: (self class limitedElementTypes)]
			raise: TestResult error].
!

testXwithXwithX
	" ANSI-Collection <SortedCollection factory> "
	| collection |
	self messageSelector: #'with:with:'.

	"Refinement: <SortedCollection factory>
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements.  The initial elements are ordered according to this sort
	block."
	collection := self canonicalCollection.
	self conformingValue: [
			SortedCollection
				with: collection first
				with: collection last]
		should: [:r|
			r size = 2
			& r first <= r last
			& (r difference: collection) isEmpty].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			SortedCollection
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" ANSI-Collection <SortedCollection factory> "
	| collection |
	self messageSelector: #'with:with:with:'.

	"Refinement: <SortedCollection factory>
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements.  The initial elements are ordered according to this sort
	block."
	collection := self canonicalCollection
		copyFrom: 1 to: 3.
	self conformingValue: [
			SortedCollection
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r|
			1 to: (r size - 1) do: [:ndx |
				(r at: ndx) <= (r at: (ndx + 1)) ifFalse: [
					^false]].
			(r difference: collection) isEmpty
			& r size = 3].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			SortedCollection
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" ANSI-Collection <SortedCollection factory> "
	| collection |
	self messageSelector: #'with:with:with:with:'.

	"Refinement: <SortedCollection factory>
		A sort block is supplied which guarantees that the elements will
	be sorted in ascending order as specified by the #< message for the
	elements.  The initial elements are ordered according to this sort
	block."
	collection := self canonicalCollection.
	self conformingValue: [
			SortedCollection
				with: (collection  at: 1)
				with: (collection  at: 2)
				with: (collection  at: 3)
				with: (collection  at: 4)]
		should: [:r|
			1 to: (r size - 1) do: [:ndx |
				(r at: ndx) <= (r at: (ndx + 1)) ifFalse: [
					^false]].
			(r difference: collection) isEmpty
			& r size = 4].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			SortedCollection
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!SortedCollectionFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SortedCollectionFactoryTest.st,v 1.1 2002/06/21 10:33:01 penk Exp $'
! !
