"{ Package: 'stx:goodies/ansiTests/common' }"

CollectionTest subclass:#SortedCollectionTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!SortedCollectionTest class methodsFor:'constants'!

canonicalCollection

	^self canonicalObject
!

canonicalElement

	^self canonicalObject at: 2
!

canonicalObject

	^SortedCollection
		with: 1
		with: 2
		with: 3
		with: 4
!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^self error: 'Has no restrictions.'.    "DO30"
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SortedCollection>'
! !

!SortedCollectionTest class methodsFor:'public'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^false  "Has no restrictions: DO30"
! !

!SortedCollectionTest methodsFor:'constants'!

canonicalElement

	^self class canonicalElement
!

canonicalObject

	^self class canonicalObject
!

canonicalObjects
	"Answer a dictionary of subsequences of the canonical object."
	| can cl |
	can := self canonicalObject.
	cl := can species. 
	^(Dictionary new)
		at: #'a' put: (cl with: (can at: 1));
		at: #'b' put: (cl with: (can at: 2));
		at: #'c' put: (cl with: (can at: 3));
		at: #'d' put: (cl with: (can at: 4));
		at: #'ab' put: (cl with: (can at: 1)
					with: (can at: 2));
		at: #'bc' put: (cl with: (can at: 2)
					with: (can at: 3));
		at: #'cd' put: (cl with: (can at: 3)
					with: (can at: 4));
		at: #'abc' put: (cl with: (can at: 1)
					with: (can at: 2)
					with: (can at: 3));
		at: #'bcd' put: (cl with: (can at: 2)
					with: (can at: 3)
					with: (can at: 4));
		at: #'abcd' put: (cl with: (can at: 1)
					with: (can at: 2)
					with: (can at: 3)
					with:(can at: 4));
		yourself
!

emptyCollection

	^SortedCollection new
! !

!SortedCollectionTest methodsFor:'testing'!

testXaddX
	" Collection <SortedCollection> "
	| sortedCollection receiverCollection |
	self messageSelector: #'add:'.

	sortedCollection := self canonicalObjects.
	receiverCollection := sortedCollection at: #'bcd'.
	self conformingValue: [receiverCollection
			add: ((sortedCollection at: #'a') at: 1);
			yourself]
		should: [:r | receiverCollection = (sortedCollection at: #'abcd')].
!

testXasSortedCollection
	" Collection <SortedCollection> "
	self messageSelector: #'asSortedCollection'.

	self conformingAssertMessageSelector.
!

testXcollectX
	" Collection <SortedCollection> "
	| collection |
	self messageSelector: #'collect:'.

	collection := self canonicalObject.
	self conformingValue: [collection collect: [:each| each]]
		should: [:result| result size = collection size].

	"Test empty class."
	self conformingValue: [self emptyCollection collect: [:each| true]]
		should: [:result| result isEmpty].

	self class returnTypeHasLimitedElementTypes
		ifTrue: [
			self should: [collection
					collect: [:each| self class limitedElementTypes]]
				raise: TestResult error].

	self should: [collection collect: [:each| each gobbledegook]]
		raise: TestResult error.
!

testXconcatenateOp
	" Collection <SortedCollection> "
	| cos |
	self messageSelector: #','.

	cos := self canonicalObjects.
	self conformingValue: [(cos at: #'cd'), (cos at: #'ab')]
		should: [:r | r = (cos at: #'abcd')].

	"Errors If the elements of operand cannot be sorted
			using receiver's sort block."
!

testXcopyReplaceAllXwithX
	" Collection <SortedCollection> "
	| subseq |
	self messageSelector: #'copyReplaceAll:with:'.

	subseq := self canonicalObjects.
self anomaly: [
	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceAll:(subseq at: #'bc')
				with: (subseq at: #'bcd')]
		should: [:r| r = ((subseq at: #'abcd'), (subseq at: #'d'))].
] for: #(#'DO30' "2001/02/20 Harmon, R. Signals should not impl. #at:put: error.").
!

testXcopyReplaceFromXtoXwithObjectX
	" Collection <SortedCollection> "
	| subseq c |
	self messageSelector: #'copyReplaceFrom:to:withObject:'.

	subseq := self canonicalObjects.
	c := (subseq at: #'c').
self anomaly: [
	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 3
				to: 2
				withObject:((subseq at: #'b') at: 1)]
		should: [:r| r = ((subseq at: #'ab'), (subseq at: #'bcd'))].

	self conformingValue: [
			(subseq at: #'abc')
				copyReplaceFrom: 4
				to: 3
				withObject: ((subseq at: #'d') at: 1)]
		should: [:r| r = (subseq at: #'abcd')].

	self conformingValue: [
			(subseq at: #'abc')
				copyReplaceFrom: 1
				to: 2
				withObject: (c at:1)]
		should: [:r| r = (c, c, c)].
] for: #(#'DO30' "2001/02/20 Harmon, R. Signals should not impl. #at:put: error.").
!

testXcopyReplaceFromXtoXwithX
	" Collection <SortedCollection> "
	| subseq |
	self messageSelector: #'copyReplaceFrom:to:with:'.

	subseq := self canonicalObjects.
self anomaly: [
	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 3
				to: 2
				with: (subseq at: #'bc')]
		should: [:r|
			r = ((subseq at: #'ab'), (subseq at: #'bc'),
					(subseq at: #'cd'))].
 
	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 5
				to: 4
				with: (subseq at: #'bc')]
		should: [:r| r = ((subseq at: #'abcd'), (subseq at: #'bc'))].

	self conformingValue: [
			(subseq at: #'abcd')
				copyReplaceFrom: 1
				to: 3
				with: (subseq at: #'bc')]
		should: [:r| r = (subseq at: #'bcd')].
] for: #(#'DO30' "2001/02/20 Harmon, R. Signals should not impl. #at:put: error.").
!

testXcopyReplacingXwithObjectX
	" Collection <SortedCollection> "
	| subseq c a b |
	self messageSelector: #'copyReplacing:withObject:'.

	subseq := self canonicalObjects.
	a := subseq at: #'a'.
	b := subseq at: #'b'.
	c := subseq at: #'c'.
self anomaly: [
	self conformingValue: [
			((subseq at: #'ab'), (subseq at: #'bc'))
				copyReplacing: (b at: 1)
				withObject: (c at: 1)]
		should: [:r| r = (a, c, c, c)].
] for: #(#'DO30' "2001/02/20 Harmon, R. #copyReplacing:withObject: returns OrderedCollection.").
!

testXreverse
	" Collection <SortedCollection> "
	| subseq abc |
	self messageSelector: #'reverse'.

	subseq := self canonicalObjects.
	abc := subseq at: #'abc'.
	self conformingValue: [abc reverse]
		should: [:r|
			r size = 3
			and: [((r at: 1) = (abc at: 3))
			and: [(r at: 3) = (abc at: 1)]]]
!

testXsortBlock
	" Collection <SortedCollection> "
	self messageSelector: #'sortBlock'.

	self conformingValue: [self canonicalObject sortBlock]
		should: [:result | true].
!

testXsortBlockX
	" Collection <SortedCollection> "
	| co |
	self messageSelector: #'sortBlock:'.

	co := self canonicalObject.
	self conformingValue: [co sortBlock: [:a :b | a > b]]
		should: [:r| co first = 4].

	"Errors If the elements of the receiver cannot be sorted
			using the discriminator."
!

ztestXaddAllX
	| cos res |
	self messageSelector: #'addAll:'.

	cos := self canonicalObjects.
#'zToDo'. "Set up <SortedCollectionAdded> & move to its test.
	2000/12/06 Harmon, R."
self anomaly: [ "Not found in spec; but SortedCollection needs to refine it.
		????/??/?? unknown"
	self conformingValue: [(cos at: #cd) addAll: (cos at: #ab); yourself]
		should: [:r | r = (cos at: #abcd)]
] for: #(#'DO30' "2000/12/06 Harmon, R.").

	res := (cos at: #cd) addAll: (cos at: #ab); yourself.
	self should: [res = (cos at: #abcd)]


! !

!SortedCollectionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SortedCollectionTest.st,v 1.1 2002/06/21 10:38:05 penk Exp $'
! !
