"{ Package: 'stx:goodies/ansiTests/common' }"

MainTestCase subclass:#StringFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-Collection'
!


!StringFactoryTest class methodsFor:'constants'!

limitedElementTypes
	"Answer an object that can NOT be put in receiver class being tested collection, or signal an error if this dialect has no restrictions."

	^StringTest limitedElementTypes
!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<String factory>'
! !

!StringFactoryTest class methodsFor:'subclass tests'!

returnTypeHasLimitedElementTypes
	"Answer a <boolean> whether the receiver class being tested restricts the objects that can be put in its collection."

	^StringTest returnTypeHasLimitedElementTypes
! !

!StringFactoryTest methodsFor:'constants'!

canonicalCollection

	^StringTest canonicalCollection
!

canonicalElement

	^StringTest canonicalElement
!

canonicalObject

	^String
! !

!StringFactoryTest methodsFor:'testing'!

testXnew
	" ANSI-Collection <String factory> "
	self messageSelector: #'new'.

	self conformingValue: [String new]
		should: [:r| r isEmpty].
!

testXnewX
	" ANSI-Collection <String factory> "
	self messageSelector: #'new:'.

	"Refinement: <String factory>
		The parameter count specifies the size of the receiver.  The
	initial value of each element of the new instance of the receiver
	is unspecified."
	self conformingValue: [String new: 0]
		should: [:r| r isEmpty].

	self conformingValue: [String new: 3]
		should: [:r| r size = 3].

	"Errors: count < 0."
		self should: [String new: -3]
			raise: TestResult error.
!

testXwithAllX
	" ANSI-Collection <String factory> "
	| collection |
	self messageSelector: #'withAll:'.

	"Refinement: <String factory>
		If the elements of newElements are ordered then their ordering
	establishing their index positions in the new collection."
	collection := self canonicalCollection.
	self conformingValue: [String withAll: collection]
		should: [:r|
			1 to: r size do: [:ndx |
				(r at: ndx) = (collection  at: ndx) ifFalse: [
					^false]].
			r size = 4].

	"Errors: If any of the elements of newElements do not meet
			the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElements |
		illegalElements := Array with: self class limitedElementTypes.
		self should: [String withAll: illegalElements]
			raise: TestResult error]
!

testXwithX
	" ANSI-Collection <String factory> "
	| element |
	self messageSelector: #'with:'.

	element := self canonicalElement.
	self conformingValue: [String with: element]
		should: [:r|
			r size = 1
			& r last = element].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		self should: [String with: (self class limitedElementTypes)]
			raise: TestResult error].
!

testXwithXwithX
	" ANSI-Collection <String factory> "
	| collection |
	self messageSelector: #'with:with:'.

	collection := self canonicalCollection.
	self conformingValue: [
			String
				with: collection first
				with: collection last]
		should: [:r|
			r size = 2
			& r first = collection first
			& r last = collection last].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			String
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithX
	" ANSI-Collection <String factory> "
	| collection |
	self messageSelector: #'with:with:with:'.

	collection := self canonicalCollection
		copyFrom: 1 to: 3.
	self conformingValue: [
			String
				with: (collection at: 1)
				with: (collection at: 2)
				with: (collection at: 3)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			String
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
!

testXwithXwithXwithXwithX
	" ANSI-Collection <String factory> "
	| collection |
	self messageSelector: #'with:with:with:with:'.

	collection := self canonicalCollection.
	self conformingValue: [
			String
				with: (collection  at: 1)
				with: (collection  at: 2)
				with: (collection  at: 3)
				with: (collection  at: 4)]
		should: [:r| r = collection].

	"Errors: If any of the arguments do not meet
		the element type constraints of the result object."
	self class returnTypeHasLimitedElementTypes ifTrue: [
		| illegalElement |
		self should: [
			illegalElement := self class limitedElementTypes.
			String
				with: illegalElement
				with: illegalElement
				with: illegalElement
				with: illegalElement]
			raise: TestResult error].
! !

!StringFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/StringFactoryTest.st,v 1.1 2002/06/21 10:32:11 penk Exp $'
! !
