"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#SysProMessageParameter
	instanceVariableNames:'parmName parmProtocolIds parmAliasingAttribute'
	classVariableNames:'ParmAliasingAttributesConst'
	poolDictionaries:''
	category:'System-Protocols'
!


!SysProMessageParameter class methodsFor:'instance creation'!

newParmName: nameIn protocolIds: protocolSpecList aliasing: attributeSpec
	"Answer a new message parameter."
	| attribute protocolIdList |
	attribute := attributeSpec asSymbol.
	(ParmAliasingAttributesConst includes: attribute) ifFalse: [
		^self error: 'Protocol msg. parameter aliasing attribute not valid.'].
	protocolIdList := (protocolSpecList collect: [:p| p asSymbol]) asSet.

	^super new
		setNewParmName: nameIn
		protocolIds: protocolIdList
		aliasing: attribute
! !

!SysProMessageParameter class methodsFor:'constants/aliasing attributes'!

initialize
	"Class initialization.
	 
		SysProMessageParameter initialize
	"
	"Initialize the list of protocol message parameter aliasing attribute constants:"
	ParmAliasingAttributesConst := Set
		with: #'captured'
		with: #'uncaptured'
		with: #'unspecified'.
! !

!SysProMessageParameter methodsFor:'accessing'!

parmAliasingAttribute
	"Answer the receiver's aliasing attribute."

	^parmAliasingAttribute
!

parmName
	"Answer the receiver's name."

	^parmName copy
!

parmProtocolIds
	"Answer the list of identifiers of protocols to which the receiver conforms."

	^parmProtocolIds copy
! !

!SysProMessageParameter methodsFor:'displaying'!

displayOn: target 
	"Write to a stream a textual description of the receiver as a user would want to see it."

	target
		nextPutAll: parmName;
		space.
	self putParmProtocolIdsOn: target.
	target
		space;
		nextPutAll: parmAliasingAttribute
!

displayString
	"Answer a string that describes the receiver as a user would want to see it."
	| aStream |
	aStream := WriteStream with: String new.
	self displayOn: aStream.
	^aStream contents
! !

!SysProMessageParameter methodsFor:'filing'!

storeAsSingleStringOn: target
	"Store the receiver as a single string on target."

	target
		nextPut: $';
		nextPutAll: parmName;
		space.
	self putParmProtocolIdsOn: target.
	target
		space;
		nextPutAll: parmAliasingAttribute;
		nextPut: $'.
! !

!SysProMessageParameter methodsFor:'printing'!

printOn: target 
	"Write to a stream a textual description of the receiver as a developer would want to see it."

	super printOn: target.
	target
		nextPut: $(;
	 nextPutAll: self parmName;
		nextPut: $).

"	 space.
	self parmProtocolIds
		do: [ :protoId | target nextPutAll: protoId]
		separatedBy: [target nextPutAll: '|'].
	target space; nextPutAll: self parmAliasingAttribute; nextPut: $)
"
! !

!SysProMessageParameter methodsFor:'private'!

putParmProtocolIdsOn: target
	"Private - Put the parm. protocol id.s on target."

	parmProtocolIds asSortedCollection
		do: [ :parmProtoId | target nextPutAll: parmProtoId asString]
		separatedBy: [target nextPut: $|].
!

setNewParmName: name protocolIds: protocolIdList aliasing: aliasingAttribute 
	"Private - Set the initial state of the receiver."

	parmName := name.
	parmProtocolIds := protocolIdList.
	parmAliasingAttribute := aliasingAttribute
! !

!SysProMessageParameter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProMessageParameter.st,v 1.1 2002/06/21 11:07:57 penk Exp $'
! !
SysProMessageParameter initialize!
