"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#SysProProtocolDocReader
	instanceVariableNames:'docStream protocolId prvLine lineInd msgParms ansiSw logSw'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Protocols-Doc Reader'
!


!SysProProtocolDocReader class methodsFor:'instance creation'!

protosFromFileloc: fileloc isANSI: ansiFlag isLoggingOn: logFlag
	"Create protocol objects from file locator, fileloc, indicating if ANSI protocols with ansiFlag = true and notifying user if logFlag = true."
	| inStream |
	inStream := FileStream read: fileloc.
	[self basicNew
		protosFromStream: inStream
		isANSI: ansiFlag
		isLoggingOn: logFlag.
	] ensure: [
		inStream close.
	].
! !

!SysProProtocolDocReader methodsFor:'operations-logging'!

logLine: lineText
	"Private - If logging on, put line, lineText, on log and go to new line.  Dialect specific."

	logSw ifTrue: [
		Transcript
			nextPutAll: lineText;
			cr].
!

logNewLine
	"Private - If logging on, go to new line on log.  Dialect specific."

	logSw ifTrue: [
		Transcript cr].
! !

!SysProProtocolDocReader methodsFor:'public'!

addNextGroup
	"Add next proto group extracted from docStream."
	| newProtocol protoGroupDesc protocolGroup secDict |
	protocolGroup := self extractProtoGroup.
	ansiSw ifTrue: [
		protocolGroup := 'ANSI-', protocolGroup].

	protoGroupDesc := self getSectionText.
	secDict := self dictOfSections.
	secDict
		at: 'Description'
		put: protoGroupDesc.
	[lineInd = 'Protocol'] whileTrue: [
		newProtocol := self addNextProtoToGroup: protocolGroup.
		self logLine: newProtocol protocolId].
!

addNextMsgSpecToProtoId: aProtocolId
	"Add next proto message spec. extracted from docStream."
	| newMsgSpec msgSelector secDict returnRuleOrNil returnList |
	msgSelector := self extractMsgSpecSelector.
"Debugging Code:
(aProtocolId asString = '<FileStream factory>' asString
and: [msgSelector asString = 'read:' asString]) ifTrue: [self halt].
"
	self getNextLineUpdateInd.
	[lineInd = nil] whileTrue: [self getNextLineUpdateInd].
	secDict := self dictOfSections.
	secDict := self cleanedupMsgSpecSections: secDict.

	returnRuleOrNil := secDict removeKey: 'ReturnValueRuleOrNil'.
	returnList := secDict removeKey: 'ReturnValueList'.
	returnRuleOrNil notNil
		ifTrue: [
			newMsgSpec := SysProProtocolsManager
				protocol: aProtocolId
				newSelector: msgSelector
				parameters: (secDict removeKey: 'Parameters')
				returnRule: returnRuleOrNil]
		ifFalse: ["No rule, so return list."
			newMsgSpec := SysProProtocolsManager
				protocol: aProtocolId
				newSelector: msgSelector
				parameters: (secDict removeKey: 'Parameters')
				returnValues: returnList].

	self checkMsgParmNames: (newMsgSpec parameterList collect: [:parm|
		parm parmName]).

	secDict keysAndValuesDo: [:secHeading :secText |
		newMsgSpec sectionAt: secHeading put: secText].

	^newMsgSpec
!

addNextProtoToGroup: aProtocolGroup
	"Add next proto extracted from docStream."
	| newProto newMsgSpec pConformsTo nextProtoOrMsgSpec secDict |
	protocolId := self extractProtoId.
	self getNextLineUpdateInd.
	[lineInd = nil] whileTrue: [self getNextLineUpdateInd].
	secDict := self dictOfSections.
	secDict := self cleanedupProtoSections: secDict.
	pConformsTo := secDict
		removeKey: 'ConformsTo'
		ifAbsent: [^self error: 'No ConformsTo in protocol.'].
	newProto := SysProProtocolsManager
		newProtocol: protocolId
		conformsToProtocols: pConformsTo
		protocolGroup: aProtocolGroup.
	secDict keysAndValuesDo: [:secHeading :secText |
		newProto sectionAt: secHeading put: secText].

	[lineInd = 'MessagePattern'] whileTrue: [
		self addNextMsgSpecToProtoId: protocolId].
	^newProto
!

checkMsgParmNames: parmNames
	"Catch doc. typo. where 'Parameters:' parm. spec. name does not match 'Message:' parm. name."
	| messageNames messageSubStrs |
	messageSubStrs := msgParms subStrings: ' '.
	messageNames := OrderedCollection new.
	1 to: (messageSubStrs size) do: [:ndx|
		ndx even ifTrue: [
			messageNames add: (messageSubStrs at: ndx)]].
	messageNames size = parmNames size ifFalse: [
		self error: '"Parameters:" parm. count not = "Message:" parm. count.'].
	1 to: (parmNames size) do: [:ndx|
		(parmNames at: ndx) = (messageNames at: ndx) ifFalse: [
			self error: '"Parameters:" parm. spec. name "',
				(parmNames at: ndx),
				'" does not match "Message:" parm. name "',
				(messageNames at: ndx), '".']].
!

cleanedupMsgSpecSections: secDict
	"Answer the sections dictionary with section values in desired form and unneeded sections removed."
	| errText refinementText aText aStringsList returnValueRuleOrNil |
	aText := secDict at: 'ReturnNote' ifAbsent: [nil].
	aText isNil ifFalse: [
		secDict at: 'Return Note' put: aText.
		secDict removeKey: 'ReturnNote'].

	aText := secDict at: 'RevisionHistory' ifAbsent: [nil].
	aText isNil ifFalse: [
		secDict at: 'Revision History' put: aText.
		secDict removeKey: 'RevisionHistory'].

	refinementText := secDict at: 'Refinement' ifAbsent: [nil].
	refinementText isNil ifFalse: [
		secDict at: 'Definition' put: refinementText.
		secDict removeKey: 'Refinement'].

	aText := secDict at: 'Parameters' ifAbsent: [String new].
	aStringsList := self msgParmStringsFromText: aText.
	secDict at: 'Parameters' put: aStringsList.

	aText := secDict removeKey: 'ReturnValue' ifAbsent: [
		^self error: 'No ReturnValue in protocol message.'].
	returnValueRuleOrNil := self msgReturnRuleOrNilFromText: aText.
	secDict at: 'ReturnValueRuleOrNil' put: returnValueRuleOrNil.
	returnValueRuleOrNil notNil
		ifTrue: [secDict at: 'ReturnValueList' put: (Array new)]
		ifFalse: ["No rule, so get return list:"
			aStringsList := self msgReturnStringsFromText: aText.
			secDict at: 'ReturnValueList' put: aStringsList	].

	errText := secDict at: 'Errors' ifAbsent: [nil].
	(self isMsgSpecErrorsNoneInText: errText)
		ifTrue: [secDict removeKey: 'Errors'].

	^secDict
!

cleanedupProtoSections: secDict
	"Answer the sections dictionary with section values in desired form and unneeded sections removed."
	| aText |
	aText := secDict at: 'RevisionHistory' ifAbsent: [nil].
	aText isNil ifFalse: [
		secDict at: 'Revision History' put: aText.
		secDict removeKey: 'RevisionHistory'].

	aText := secDict at: 'StandardGlobals' ifAbsent: [nil].
	aText isNil ifFalse: [
		secDict at: 'Standard Globals' put: aText.
		secDict removeKey: 'StandardGlobals'].

	secDict removeKey: 'Messages'.
	^secDict
!

dictOfSections
	"Get the set of sections of a group, protocol, or msg. from docStream."
	| specTerm secDict sectionHeading sectionText |
	specTerm :=  #('MessagePattern' 'Protocol' 'Protocols' 'DocEnd').
	secDict := Dictionary new.
	[specTerm includes: lineInd] whileFalse: [
		sectionHeading := lineInd.
		sectionText := self getSectionText.
		secDict
			at: sectionHeading
			put: sectionText].
	^secDict
!

extractMsgSpecSelector
	"Get protocol name:"
	| msgSelector tmpStr |
	tmpStr := prvLine
		copyFrom: ((prvLine indexOf: $[) + 1)
		to: (prvLine size).
	msgSelector := String methodSelector: (tmpStr, ' '). "No blowup."
	msgParms := tmpStr trimBlanks.
	^msgSelector asSymbol
!

extractProtoGroup
	"Now on proto group line."
	| startNdx endNdx |
	startNdx := prvLine indexOf: Character space.
	endNdx := prvLine indexOfSubCollection: 'Protocols['.
	^(prvLine copyFrom: (startNdx + 1) to: (endNdx - 1)) trimBlanks.
!

extractProtoId
	"Now on proto group line."
	| startNdx endNdx |
	startNdx := prvLine indexOf: $<.
	endNdx := prvLine indexOf: $>.
	^(prvLine copyFrom: startNdx to: endNdx) asSymbol.
!

getNextLineUpdateInd
	"Get next line from docStream and updatet the line type indicator if any without trailing [."
	| wordList indHold |
	docStream atEnd ifTrue: [
		prvLine := String new.
		^lineInd := 'DocEnd']. 
	prvLine := docStream nextLine. 

	lineInd := nil.
	(prvLine includes: $[) ifFalse: [
		^self].
	wordList := prvLine subStrings: SysProProtocolsManager whitespaceSeparators.
	indHold := wordList
		detect: [ :word|
			| trimWord |
			trimWord := word trimBlanks.
			trimWord size > 1 and: [ trimWord last = $[ ]]
		ifNone: [^self].
	lineInd := indHold
		copyFrom: 1
		to: indHold size - 1.
!

getSectionText
	"Get all the sections of a group, protocol, or msg. from docStream."
	| textStream |
	textStream := WriteStream with: String new.	
	self getNextLineUpdateInd. 
	[lineInd isNil] whileTrue: [
		textStream nextPutAll: prvLine; cr.
		self getNextLineUpdateInd].
	"This is an ugly platform neutral way to eliminate
	trailing empty line but ensure the last line has line termination."
	textStream := WriteStream with: textStream contents trimBlanks.
	textStream cr.
	^textStream contents
!

isMsgSpecErrorsNoneInText: sectionTextOrNil
	""
	| secLineType tmpStream prvLineAsToken sectionTextHold |
	sectionTextOrNil isNil ifTrue: [
		^false].
	sectionTextHold := sectionTextOrNil trimBlanks.
	sectionTextHold isEmpty ifTrue: [
		^true].
	sectionTextHold size < 6 ifTrue: [
		(sectionTextHold asLowercase indexOfSubCollection: 'none'
		) > 0 ifTrue: [
			^true]].
	^false
!

msgParmStringsFromText: text
	"Now on Parameters Line."
	| parmStringsList aLine textStream |
	parmStringsList := OrderedCollection new.
	textStream := ReadStream on: text.
	[textStream atEnd] whileFalse: [
		aLine := textStream nextLine trimBlanks.
		aLine notEmpty ifTrue: [
			parmStringsList add: aLine]].
	^parmStringsList
!

msgReturnRuleOrNilFromText: text
	"Answer a return value rule, or nil if an array (possibly empty) of strings represents the return value."
	| textHold |
	textHold := text trimBlanks.

	textHold = 'RULE1' ifTrue: [
		^'[ :receiver :operand |
	"If the operands conform to integer then <rational>
	otherwise the Default Result Type."
	| defaultRetType |
	"defaultRetType is Default Conversion Table[rec, op]"
	defaultRetType := SystemProtocolsManager
		defaultReturnProtocolIdReceiver: receiver
		operand: operand.
	"conformsTo:"
	defaultRetType = #''<integer>''
		ifTrue: [#''<rational>'']
		ifFalse:[defaultRetType]]'].

	textHold = 'RULE2' ifTrue: [
		^'[ :receiver |	| returnTypeTable |
	"receiver		| result
	------------------|------------------
	<integer>		| <rational>
	------------------|------------------
	<Fraction>		| <rational>
	------------------|------------------
	<scaledDecimal>	| <scaledDecimal>
	------------------|------------------
	<Float>		| <Float>"

	returnTypeTable := Dictionary new.
	returnTypeTable at: #''<integer>'' put: #''<rational>''.
	returnTypeTable at: #''<Fraction>'' put: #''<rational>''.
	returnTypeTable at: #''<scaledDecimal>'' put: #''<scaledDecimal>''.
	returnTypeTable at: #''<Float>'' put: #''<Float>''.

	returnTypeTable at: (SystemProtocolsManager numericInstanceProtocolId: receiver)]'].

	textHold = 'RULE3' ifTrue: [
		^'[ :receiver |	| returnTypeTable |
	"receiver		| result
	------------------|------------------
	<rational>		|<rational>
	------------------|------------------
	<scaledDecimal>	| <scaledDecimal>
	------------------|------------------
	<Float>		| <Float>"

	returnTypeTable := Dictionary new.
	returnTypeTable at: #''<integer>'' put: #''<rational>''.
	returnTypeTable at: #''<Fraction>'' put: #''<rational>''.
	returnTypeTable at: #''<scaledDecimal>'' put: #''<scaledDecimal>''.
	returnTypeTable at: #''<Float>'' put: #''<Float>''.

	returnTypeTable at: (SystemProtocolsManager numericInstanceProtocolId: receiver)]'].

	textHold = 'The Default Result Type' ifTrue: [
		^'[ :receiver :operand |
	"The Default Result Type"
	SystemProtocolsManager
		defaultReturnProtocolIdReceiver: receiver
		operand: operand]'].

	^nil
!

msgReturnStringsFromText: text
	"Answer an array (possibly empty) of strings represents the return value.  Assumes a return value rule case should already be handled and this method not be subsequently used."
	| returnStringsList aLine textStream textHold endNdx |
	textHold := text trimBlanks.
	textHold isEmpty ifTrue: [
		^Array new].
	((textHold = 'none') | (textHold = 'UNSPECIFIED')) ifTrue: [
		^Array new].

	(textHold indexOfSubCollection: '<RECEIVER>') > 0 ifTrue: [
	endNdx := textHold indexOf: $>.
		^Array with: (protocolId, (textHold
			copyFrom: (endNdx + 1)
			to: (textHold size)))].

	returnStringsList := Set new.
	textStream := ReadStream on: textHold.
	[textStream atEnd] whileFalse: [
		aLine := textStream nextLine trimBlanks.
		aLine notEmpty ifTrue: [
			returnStringsList add: aLine]].
	^returnStringsList asArray
!

protosFromStream: docStreamIn isANSI: ansiFlag isLoggingOn: logFlag
	"Create protocol objects from docStream indicating if ANSI protocols with ansiFlag = true and notifying user if logFlag = true."
	| newProtocol undef|
	docStream := docStreamIn.
	ansiSw := ansiFlag.
	logSw := logFlag.
	self	 logNewLine;
		logLine: 'start';
		logNewLine.

	[lineInd = 'DocEnd' or: [lineInd = '><']] whileFalse: [
		self getNextLineUpdateInd].
	self getNextLineUpdateInd.
	[lineInd = 'Protocols'] whileTrue: [
		self addNextGroup].

	undef := SysProProtocolsManager curMgr undefinedConformsToProtocolIds.
	undef isEmpty
		ifTrue: [self logLine: 'No undefined conforms-to protocols.']
		ifFalse: [self logLine: ('Undefined conforms-to protocol names ', undef printString)].

	self logLine: 'end'.
! !

!SysProProtocolDocReader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolDocReader.st,v 1.1 2002/06/21 11:07:38 penk Exp $'
! !
