"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#SysProProtocolMessage
	instanceVariableNames:'messageSelector parameterList returnValueList returnValueWithRule
		specSections'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Protocols'
!


!SysProProtocolMessage class methodsFor:'instance creation'!

newMessageSelector: selectorSpec parameters: parmList returnRule: returnValueRuleSource
	"Answer a new message with selector selectorSpec, parameters parmList, and a return value rule source returnValueRuleSource."
	| parameters newReturnRule |
	parameters := self parametersFromList: parmList.
	newReturnRule := SysProMessageByRuleReturn
			newRetValRuleSourceCode: returnValueRuleSource.
	^self
		newSelector: selectorSpec
		parmList: parameters
		returnValueList: nil
		orRule: newReturnRule
!

newMessageSelector: selectorSpec  parameters: parmList returnValues: returnValueList
	"Answer a new message with selector selectorSpec, parameters parmList, and return values returnValueList."
	| parameters returnValues |
	parameters := self parametersFromList: parmList.
	returnValues := self returnValuesFromList: returnValueList.
	^self
		newSelector: selectorSpec 
		parmList: parameters
		returnValueList: returnValues
		orRule: nil.
! !

!SysProProtocolMessage class methodsFor:'private'!

checkMessagePatternParmList: parmList toSelector: selector ifError: errorBlock 
	"Private - Check that a message pattern built from the selector and parameter list is reasonable (i.e. binary selector doesn't have parameters).  If not evaluate errorBlock.
	Note: needs more checks like the number of parameters indicated in selector by colons match the size of parameter list."
	| colonCnt |
	selector isUnarySelector ifTrue: [
		parmList isEmpty ifTrue: [
			^self	"OK"].
		^errorBlock value].


	selector isBinarySelector ifTrue: [
		parmList size = 1 ifTrue: [
			^self	"OK"].
		^errorBlock value].

	(selector occurrencesOf: $:) = parmList size ifFalse: [
		^errorBlock value].
!

newSelector: selector parmList: parmList returnValueList: retValList orRule: retValRule

	| newProtocolMsgSpec |
	self
		checkMessagePatternParmList: parmList
		toSelector: selector
		ifError: [
			^self error: 'Mis-matched protocol msg. parms & selector.'].
	newProtocolMsgSpec := super basicNew.
	newProtocolMsgSpec
		setNewSelector: selector
		parmList: parmList
		returnValueList: retValList
		orRule: retValRule.
	^newProtocolMsgSpec

	"Private -"


	"Definition: <SystemProtocolMsg factory> 
	Answer a new protocol message specification with selector, selector, specSections, specSections, a list of specifications for each parameter, parmList, and a list of specifications for each return value, retValList.
	Note: specSections must be a <Dictionary> of <symbol> keys and <readableString> values, parmList must be a <collection> of <protocolMessageParmSpec>s, retValList, a <collection> of <protocolMessageReturnSpec>s.
	Parameters 
	selector			<symbol>			captured 
	specSections		<Dictionary>			captured 
	parmList		<collection 			captured 
	retValList		<collection>			captured 
	Return Values 
	<SystemProtocolMsg>		new
	"
!

parametersFromList: parmStringList 
	"Private - Extract and create a collection of new protocol message parameters from the list of parameter specification strings."
	| newParmList|
	newParmList := parmStringList collect: [ :aString |
		| protoIdList parmName alias protoId strStream endNdx strX |
		endNdx := aString indexOf: $<.
		parmName := (aString copyFrom: 1 to: (endNdx - 1)) trimBlanks.

		strX := (aString copyFrom: endNdx to: (aString size)) reverse.
		endNdx := strX indexOf: $>.
		alias := (strX copyFrom: 1 to: (endNdx - 1)) reverse trimBlanks.

		protoIdList := OrderedCollection new.
		strStream := ReadStream on: ((strX copyFrom: endNdx to: (strX size)) reverse).
		[strStream atEnd] whileFalse: [
			protoId := (strStream upTo: $|) trimBlanks.
			protoIdList add: protoId].
		SysProMessageParameter
			newParmName: parmName
			protocolIds: protoIdList
			aliasing: alias].
	^newParmList
!

returnValuesFromList: returnValueStringList 
	"Private - Extract and create a collection of new protocol message return values from the list of return value specification strings."

	^returnValueStringList collect: [ :aString |
		| retValProName alias endNdx |
		endNdx := aString indexOf: $>.
		retValProName := (aString copyFrom: 1 to: endNdx) trimBlanks.
		alias := (aString copyFrom: (endNdx + 1) to: aString size) trimBlanks.
		SysProMessageReturn
			newRetValProtocolId: retValProName
			aliasing: alias].
! !

!SysProProtocolMessage methodsFor:'accessing'!

messagePattern
	"Answer the message pattern (selector, together with its parameters) of the receiver."
	| aStream colonCnt parmNames |
	parmNames := (self parameterList collect: [ :msgParm |
		msgParm parmName]) asArray.
	(messageSelector includes: $:) ifFalse: [
		parmNames size = 0 ifTrue: [
			^messageSelector asString].
		parmNames size = 1 ifTrue: [
			^messageSelector asString , ' ' , (parmNames at: 1)]].
	aStream := WriteStream with: String new.
	colonCnt := 0.
	messageSelector do: [:char |
		char = $:
			ifTrue: [
				colonCnt := colonCnt + 1.
				aStream nextPutAll: ': '.
				aStream nextPutAll: (parmNames at: colonCnt).
				colonCnt = parmNames size ifFalse: [aStream space]]
			ifFalse: [aStream nextPut: char]].
	^aStream contents
!

messageSelector
	"Answer the selector of the receiver."

	^messageSelector
!

parameterList
	"Answer the parameter list of the receiver."

	parameterList isNil ifTrue: [
		^Array new].
	^parameterList copy
!

returnValueList
	"Answer the return value list of the receiver."

	returnValueList isNil ifTrue: [
		^Array new].
	^returnValueList
!

returnValueWithRule
	"Answer the return value specification-with-rule of the receiver."

	^returnValueWithRule
!

sectionAt: sectionHeadingSpec
	"Answer the text of the receiver's specification section titled, sectionHeadingSpec, or an empty text if sectionHeadingSpec not found."

	specSections isNil ifTrue: [
		^String new].
	^specSections
		at: sectionHeadingSpec asSymbol
		ifAbsent: [String new]
!

sectionAt: sectionHeadingSpec put: sectionText
	"Set the receiver's specification section titled, sectionHeadingSpec, to the text, sectionText."

	specSections isNil ifTrue: [
		specSections := SysProProtocol defaultSpecSectionsCollection].
	specSections
		at: sectionHeadingSpec asSymbol
		put: sectionText
! !

!SysProProtocolMessage methodsFor:'comparing'!

<= comperand 
	"Answer true if the receiver is less than or equal to comperand.  Answer false otherwise."

	^self messageSelector <= comperand messageSelector
!

= comparand
	"Answer true if comparand and the receiver are equivalent, comparand is a <SystemProtocolMsg> and has the same message selector as the receiver, else false."

	self == comparand ifTrue: [
		^true].
	^(comparand class == self class)
	and: [self messageSelector == comparand messageSelector]
!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self messageSelector hash
! !

!SysProProtocolMessage methodsFor:'displaying'!

displayOn: target 
	"Write to a stream a textual description of the receiver as a user would want to see it."

	target nextPutAll: messageSelector
!

displayString
	"Answer a string that describes the receiver as a user would want to see it."
	| aStream |
	aStream := WriteStream with: String new.
	self displayOn: aStream.
	^aStream contents
! !

!SysProProtocolMessage methodsFor:'enumerating'!

sectionHeadingsAndTextsDo: operation
	"Iteratively evaluate operation with each of the receiver's specification section headings and texts."

	specSections isNil ifTrue: [
		^self].
	specSections keysAndValuesDo: operation

! !

!SysProProtocolMessage methodsFor:'filing'!

storeOn: target inProtocolId: protocolSpec
	"Store the receiver, a component of the protocol identified by protocolSpec in chunk format on target.
	Note: I manually add a chunk terminator except for spec. sections.  Only the spec. sections should contain embedded terminators that must be doubled."
	| protocolStr |
	protocolStr := protocolSpec asString.
	target
		nextPutAll: SysProProtocolsManager name;
		tab; nextPutAll: 'protocol: ''';
		nextPutAll: protocolStr;
		nextPut: $'; cr;
		tab; nextPutAll: 'newSelector: ''';
		nextPutAll: messageSelector asString;
		nextPut: $'; cr;
		tab; nextPutAll: 'parameters: #('.
	self parameterList
		do: [:msgParm | msgParm storeAsSingleStringOn: target]
		separatedBy: [target cr; tab; tab].
	target
		nextPut: $); cr.
	self isReturnValueUnspecified ifTrue: [ "No rule & no return parms:"
		target tab; nextPutAll: 'returnValues: #()!!'; cr; cr
	] ifFalse: [
		returnValueWithRule notNil ifTrue: [ "Has rule:"
			target tab; nextPutAll: 'returnRule: '.
			returnValueWithRule returnValueRuleBlockSourceCode printOn: target.
			target nextPut: $!!; cr; cr
		] ifFalse: [ "Has return parms:"
			target tab; nextPutAll: 'returnValues: #('.
			returnValueList
				do: [:msgRet | msgRet storeAsSingleStringOn: target]
				separatedBy: [target cr; tab; tab].
			target
				nextPutAll: ')!!'; cr; cr]].

	specSections isNil ifFalse: [	| sortedHeadings |
		sortedHeadings := specSections keys asSortedCollection
			sortBlock: [:x :y | x asLowercase <= y asLowercase].
		sortedHeadings do: [:secHeading|
			| secText secStream |
			secText := specSections at: secHeading.
			target
				nextPutAll: SysProProtocolsManager name;
				nextPutAll: ' protocol: ''';
				nextPutAll: protocolStr;
				nextPut: $'; cr;
				tab; nextPutAll: 'selector: ''';
				nextPutAll: messageSelector asString;
				nextPut: $'; cr;
				tab; nextPutAll: 'section: ''';
				nextPutAll: secHeading asString;
				nextPut: $'; cr;
				tab; nextPutAll: 'text: '.
			secStream := WriteStream with: String new.	
			"Insure single qoutes doubled up:"
			secText asString printOn: secStream.
			target nextPutChunk: (secStream contents); cr]].

	"Definition: <SystemProtocol> 
	Store the receiver on target so that it can be recreated when read and evaluated. 

	Parameters 
	target		<puttableStream>		uncaptured
	"
! !

!SysProProtocolMessage methodsFor:'printing'!

printOn: target
	"Write to a stream a textual description of the receiver as a developer would want to see it."

	super printOn: target.
	target nextPutAll: '( #''';
		nextPutAll: messageSelector;
		nextPutAll: ''' )'.

"	self parameterList
		do: [:msgParm | target nextPutAll: msgParm parmName]
		separatedBy: [target space].
	target nextPutAll: ')'.
	target nextPutAll: ' retVals('.
	self isReturnValueUnspecified ifTrue: [
		^target nextPutAll: 'Unspecified))'].
	returnValueWithRule notNil ifTrue: [
		^target nextPutAll: 'Rule))'].
	returnValueList
		do: [:retVal| target nextPutAll: retVal returnValueProtocolId]
		separatedBy: [target space].
	target nextPutAll: '))'.
"
! !

!SysProProtocolMessage methodsFor:'private'!

setNewSelector: selectorIn parmList: parmListIn returnValueList: returnValueListIn orRule: returnValueRuleIn
	"Private - Set the initial state of the receiver."

	messageSelector := selectorIn asSymbol.
	parameterList := parmListIn notEmpty
		ifTrue: [Array withAll: parmListIn]
		ifFalse: [nil].
	returnValueRuleIn isNil ifFalse: [
		returnValueWithRule := returnValueRuleIn.
		^returnValueList := nil].
	returnValueWithRule := nil.
	returnValueListIn isNil ifTrue: [
		^returnValueList := nil].
	returnValueListIn isEmpty ifTrue: [
		^returnValueList := nil].
	returnValueList := Array withAll: returnValueListIn
! !

!SysProProtocolMessage methodsFor:'public'!

isReturnValueSpecByRule
	"Answer true if the receiver return value protocol is determined by a rule, else false."

	returnValueWithRule isNil ifTrue: [
		^false].
	^true
!

isReturnValueUnspecified
	"Answer true if the receiver return value is unspecified, else false."

	returnValueWithRule isNil & returnValueList isNil ifTrue: [
		^true].
	^false
! !

!SysProProtocolMessage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolMessage.st,v 1.1 2002/06/21 11:08:08 penk Exp $'
! !
