"{ Package: 'stx:goodies/ansiTests/common' }"

SysProGroup subclass:#SysProProtocolMessageTest
	instanceVariableNames:'numberXnegativeMsg'
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-System Protocols'
!


!SysProProtocolMessageTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SysProProtocolMessage>'
! !

!SysProProtocolMessageTest methodsFor:'constants'!

canonicalObject

	^numberXnegativeMsg
! !

!SysProProtocolMessageTest methodsFor:'helpers'!

setUp

	super setUp.
	numberXnegativeMsg := (ProtoMgr protocolId: #'<number>')
		messageAtSelector: #'negative'.
! !

!SysProProtocolMessageTest methodsFor:'private'!

jnkProtoXselectorParm1Str

	^'SysProProtocolsManager	protocol: ''<JnkProto>''
	newSelector: ''selectorParm1:''
	parameters: #(''operand <UndefProto1>|<UndefProto2> uncaptured'')
	returnValues: #(''<JnkProto> new''
		''<nil> unspecified'')!!

SysProProtocolsManager protocol: ''<JnkProto>''
	selector: ''selectorParm1:''
	section: ''Description''
	text: ''description.''!!

SysProProtocolsManager protocol: ''<JnkProto>''
	selector: ''selectorParm1:''
	section: ''Header''
	text: ''Protocol section text.''!!

'
! !

!SysProProtocolMessageTest methodsFor:'testing'!

testXdisplayOnX
	" System Protocols <SysProProtocolMessage> "
	| aStream |
	self messageSelector: #'displayOn:'.

	aStream := WriteStream with: String new.

	self conformingValue: [numberXnegativeMsg displayOn: aStream]
		should: [:r| aStream contents = 'negative']. 
!

testXdisplayString
	" System Protocols <SysProProtocolMessage> "
	self messageSelector: #'displayString'.

	self conformingValue: [numberXnegativeMsg displayString]
		should: [:r| r = 'negative']. 
!

testXequalityOp
	" System Protocols <SysProProtocolMessage> "
	| fracXprintStringMsg numberProto numXprintStringMsg numberXsqrtMsg |
	self messageSelector: #'='.

	"receiver = comparand	=>
	receiver hash = comparand hash"
	numberProto := ProtoMgr protocolId: #'<number>'.

	"Test if object equals itself:"
	self conformingValue: [numberXnegativeMsg = numberXnegativeMsg]
		should: [:r|r &(numberXnegativeMsg hash = numberXnegativeMsg hash)].

	"Test if different objects with same selector are equal:"
	fracXprintStringMsg := (ProtoMgr protocolId: #'<Fraction>')
		messageAtSelector: #'printString'.
	numXprintStringMsg := numberProto messageAtSelector: #'printString'.
	self conformingValue: [fracXprintStringMsg = numXprintStringMsg]
		should: [:r| r & (fracXprintStringMsg = numXprintStringMsg)].

	"Test if different objects with different selector are equal:"
	numberXsqrtMsg := (numberProto messageAtSelector: 'sqrt').
	self conformingValue: [numberXnegativeMsg = numberXsqrtMsg]
		shouldnt: [:r| r | (numberXnegativeMsg hash = numberXsqrtMsg hash)].
!

testXhash
	" System Protocols <SysProProtocolMessage> "
	| fracXprintStringMsg numberProto numXprintStringMsg numXsqrtMsg |
	self messageSelector: #'hash'.

	"Any two objects that are considered equivalent using the #= message
	must have the same hash value.  More formally:

		receiver = comparand	=>
		receiver hash = comparand hash"
	numberProto := ProtoMgr protocolId: #'<number>'.

	"(true) Test if object hash equals itself:"
	self conformingValue: [numberXnegativeMsg hash]
		should: [:r| (r = numberXnegativeMsg hash)
			&(numberXnegativeMsg = numberXnegativeMsg)].

	"(true) Test if different objects with same selector have equal hashes:"
	fracXprintStringMsg := (ProtoMgr protocolId: #'<Fraction>')
		messageAtSelector: 'printString'.
	numXprintStringMsg := numberProto messageAtSelector: #'printString'.
	self conformingValue: [fracXprintStringMsg hash]
		should: [:r| (r = numXprintStringMsg hash)
			& (fracXprintStringMsg = numXprintStringMsg)].

	"(false) Test if different objects with different selector are equal:"
	numXsqrtMsg := (numberProto messageAtSelector: #'sqrt').
	self conformingValue: [numberXnegativeMsg hash]
		shouldnt: [:r| (r = numXsqrtMsg hash)
			| (numberXnegativeMsg = numXsqrtMsg)].
!

testXisReturnValueSpecByRule
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'isReturnValueSpecByRule'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [
			(numberProto messageAtSelector: #'*') isReturnValueSpecByRule]
		should: [:r| r ]. 

	self conformingValue: [numberXnegativeMsg isReturnValueSpecByRule]
		shouldnt: [:r| r ]. 
!

testXisReturnValueUnspecified
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'isReturnValueUnspecified'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	"(true) Test a unspecified:"
	self conformingValue: [
			(numberProto messageAtSelector: #'to:do:')
				isReturnValueUnspecified]
		should: [:r| r ]. 

	"(false) Test a rule:"
	self conformingValue: [
			(numberProto messageAtSelector: #'*') isReturnValueUnspecified]
		shouldnt: [:r| r ]. 

	"(false) Test a parm. list:"
	self conformingValue: [numberXnegativeMsg isReturnValueUnspecified]
		shouldnt: [:r| r ]. 
!

testXlessThanOrEqualToOp
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'<='.

	numberProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [
			numberXnegativeMsg <= (numberProto messageAtSelector: #'sqrt')]
		should: [:r| r].

	self conformingValue: [numberXnegativeMsg <= numberXnegativeMsg]
		should: [:r| r].

	self conformingValue: [
			numberXnegativeMsg <= (numberProto messageAtSelector: #'abs')]
		shouldnt: [:r| r].
!

testXmessagePattern
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'messagePattern'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	"Test a keyword message:"
	self conformingValue: [
			(numberProto messageAtSelector: #'to:do:') messagePattern]
		should: [:r| r = 'to: stop do: operation']. 

	"Test a binary operator message:"
	self conformingValue: [
			(numberProto messageAtSelector: #'*') messagePattern]
		should: [:r| r = '* operand']. 

	"Test a unary operator message:"
	self conformingValue: [numberXnegativeMsg messagePattern]
		should: [:r| r = 'negative']. 
!

testXmessageSelector
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'messageSelector'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	"Test a keyword message:"
	self conformingValue: [
			(numberProto messageAtSelector: #'to:do:') messageSelector]
		should: [:r| r = #'to:do:']. 

	"Test a binary operator message:"
	self conformingValue: [
			(numberProto messageAtSelector: #'*') messageSelector]
		should: [:r| r = #'*']. 

	"Test a unary operator message:"
	self conformingValue: [numberXnegativeMsg messageSelector]
		should: [:r| r = #'negative']. 
!

testXparameterList
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'parameterList'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	"Test a muliple keyword message parameters:"
	self conformingValue: [
			(numberProto messageAtSelector: #'to:do:') parameterList]
		should: [:r| ((r at: 1) parmName = 'stop')
			& ((r at: 2) parmName = 'operation')]. 

	"Test a binary operator message parameters:"
	self conformingValue: [
			(numberProto messageAtSelector: #'*') parameterList]
		should: [:r| ((r at: 1) parmName = 'operand')]. 

	"Test a unary operator message parameters:"
	self conformingValue: [numberXnegativeMsg parameterList]
		should: [:r| r isEmpty]. 
!

testXprintString
	" System Protocols <SysProProtocolMessage> "
	self messageSelector: #'printString'.

	self conformingValue: [numberXnegativeMsg printString]
		should: [:r| r = 'a SysProProtocolMessage( #''negative'' )'].
!

testXreturnValueList
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'returnValueList'.

	numberProto := ProtoMgr protocolId: '<number>'.

	"Test an unspecified return value:"
	self conformingValue: [
			(numberProto messageAtSelector: #'to:do:') returnValueList]
		should: [:r| r isEmpty]. 

	"Test a return value list:"
	self conformingValue: [numberXnegativeMsg returnValueList]
		should: [:r|	| retVal | retVal := r at: 1.
			(retVal returnValueProtocolId = #'<boolean>')
			& (retVal returnValueAliasingAttribute = #'unspecified')]. 
!

testXreturnValueWithRule
	" System Protocols <SysProProtocolMessage> "
	| numberProto |
	self messageSelector: #'returnValueWithRule'.

	numberProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [
			(numberProto messageAtSelector: #'*') returnValueWithRule]
		should: [:r| r ruleBlock argumentCount = 2]. 
!

testXsectionAtX
	" System Protocols <SysProProtocolMessage> "
	| protoMsg |
	self messageSelector: #'sectionAt:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	protoMsg := (ProtoMgr protocolId: #'<JnkProto>')
		 messageAtSelector: #'selectorParm1:'.

	protoMsg
		sectionAt: #'Description'
		put: 'System Protocols protocol message description.'.
	self conformingValue: [protoMsg sectionAt: #'Description']
		should: [:r| r = 'System Protocols protocol message description.']. 

	self conformingValue: [protoMsg sectionAt: 'Bogus']
		should: [:r| r isEmpty]. 

	] ensure: [self removeJnkProto].
!

testXsectionAtXputX
	" System Protocols <SysProProtocolMessage> "
	| protoMsg |
	self messageSelector: #'sectionAt:put:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	protoMsg := (ProtoMgr protocolId: #'<JnkProto>')
		 messageAtSelector: #'selectorParm1:'.

	self conformingValue: [
			protoMsg
				sectionAt: 'Header'
				put: 'Section text.']
		should: [:r| (protoMsg sectionAt: 'Header') = 'Section text.']. 

	] ensure: [self removeJnkProto].
!

testXsectionHeadingsAndTextsDoX
	" System Protocols <SysProProtocolMessage> "
	| protoMsg results |
	self messageSelector: #'sectionHeadingsAndTextsDo:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	protoMsg := (ProtoMgr protocolId: #'<JnkProto>')
		 messageAtSelector: #'selectorParm1:'.
	protoMsg
		sectionAt: #'Description'
		put: 'description.'.
	protoMsg
		sectionAt: 'Header'
		put: 'Protocol section text.'.
	results := Set new.

	self conformingValue: [
			protoMsg sectionHeadingsAndTextsDo: [:secHeading :secText|
				secHeading = #'Header'
					ifTrue: [results add: (secText = 'Protocol section text.')]
					ifFalse: [secHeading = #'Description'
						ifTrue: [results add: (secText = 'description.')]
						ifFalse: [results add: false]]]]
		shouldnt: [:r| results includes: false]. 

	] ensure: [self removeJnkProto].
!

testXstoreOnXinProtocolIdX
	" System Protocols <SysProProtocolMessage> "
	| resultStream protoMsg |
	self messageSelector: #'storeOn:inProtocolId:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	protoMsg := (ProtoMgr protocolId: #'<JnkProto>')
		 messageAtSelector: #'selectorParm1:'.
	protoMsg
		sectionAt: #'Description'
		put: 'description.'.
	protoMsg
		sectionAt: 'Header'
		put: 'Protocol section text.'.

	resultStream := ReadWriteStream with: String new.
	self conformingValue: [
			protoMsg
				storeOn: resultStream
				inProtocolId: #'<JnkProto>']
		should: [:r|	| targetStream |
			targetStream := ReadStream on: (self jnkProtoXselectorParm1Str).
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]. 

	] ensure: [ self removeJnkProto].
! !

!SysProProtocolMessageTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolMessageTest.st,v 1.1 2002/06/21 10:25:01 penk Exp $'
! !
