"{ Package: 'stx:goodies/ansiTests/common' }"

SysProGroup subclass:#SysProProtocolTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-System Protocols'
!


!SysProProtocolTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SysProProtocol>'
! !

!SysProProtocolTest methodsFor:'constants'!

canonicalObject

	^ProtoMgr protocolId: #'<number>'
! !

!SysProProtocolTest methodsFor:'private'!

jnkProtoDefineStr

	^'SysProProtocolsManager protocol: ''<JnkProto>''
	defineBehaviorOfClasses: ''Array Integer''!!

'
!

jnkProtoFactoryDefineStr

	^'SysProProtocolsManager protocol: ''<JnkProto factory>''
	defineBehaviorOfMetaclassesOfClasses: ''Object''!!

'
!

jnkProtoFactoryStr

	^'SysProProtocolsManager newProtocol: ''<JnkProto factory>''
	conformsToProtocols: ''<Object class>''
	protocolGroup: ''Junk Group''!!

SysProProtocolsManager protocol: ''<JnkProto factory>''
	section: ''Description''
	text: ''Represents a System Protocols SUnits test protocol.
''!!

SysProProtocolsManager	protocol: ''<JnkProto factory>''
	newSelector: ''new''
	parameters: #()
	returnValues: #(''<JnkProto> new'')!!

SysProProtocolsManager protocol: ''<JnkProto factory>''
	selector: ''new''
	section: ''Definition''
	text: ''The object new is junk.
''!!

SysProProtocolsManager protocol: ''<JnkProto factory>''
	selector: ''new''
	section: ''Errors''
	text: ''If junk is not allowed.
''!!

SysProProtocolsManager protocol: ''<JnkProto factory>''
	selector: ''new''
	section: ''Synopsis''
	text: ''Answer a new junk object.
''!!

'
!

jnkProtoStr

	^'SysProProtocolsManager newProtocol: ''<JnkProto>''
	conformsToProtocols: ''<nil>, <UndefProto1>''
	protocolGroup: ''Junk Group''!!

SysProProtocolsManager protocol: ''<JnkProto>''
	section: ''Description''
	text: ''Represents a 2nd. System Protocols SUnits test protocol.
''!!

SysProProtocolsManager	protocol: ''<JnkProto>''
	newSelector: ''selector''
	parameters: #()
	returnRule: ''[ :receiver |	"Sample comment"
	SysProProtocolsManager unaryReturnProtocolIdReceiver: receiver]''!!

SysProProtocolsManager protocol: ''<JnkProto>''
	selector: ''selector''
	section: ''Synopsis''
	text: ''A System Protocols SUnits test protocol message with rule return.
''!!

SysProProtocolsManager	protocol: ''<JnkProto>''
	newSelector: ''selectorParm1:''
	parameters: #(''operand <UndefProto1>|<UndefProto2> uncaptured'')
	returnValues: #(''<JnkProto> new''
		''<nil> unspecified'')!!

SysProProtocolsManager protocol: ''<JnkProto>''
	selector: ''selectorParm1:''
	section: ''Synopsis''
	text: ''A System Protocols SUnits test protocol message with parm.
''!!

SysProProtocolsManager	protocol: ''<JnkProto>''
	newSelector: ''selectorParm1:parm2:''
	parameters: #(''parm1 <collection> unspecified''
		''parm2 <symbol> captured'')
	returnValues: #()!!

SysProProtocolsManager protocol: ''<JnkProto>''
	selector: ''selectorParm1:parm2:''
	section: ''Synopsis''
	text: ''A System Protocols SUnits test protocol message with 2 parms.
''!!

'
!

nilDictionary

	^Dictionary new
	at: #printString	put: #'<nil>';

	at: #=				put: #'<Object>';
	at: #==				put: #'<Object>';
	at: #~~				put: #'<Object>';
	at: #~=				put: #'<Object>';
	at: #class				put: #'<Object>';
	at: #copy				put: #'<Object>';
	at: #doesNotUnderstand:		put: #'<Object>';
	at: #error:				put: #'<Object>';
	at: #hash				put: #'<Object>';
	at: #identityHash			put: #'<Object>';
	at: #isKindOf:			put: #'<Object>';
	at: #isMemberOf:			put: #'<Object>';
	at: #isNil				put: #'<Object>';
	at: #notNil				put: #'<Object>';
	at: #perform:			put: #'<Object>';
	at: #perform:with:		put: #'<Object>';
	at: #perform:with:with:		put: #'<Object>';
	at: #perform:with:with:with:	put: #'<Object>';
	at: #perform:withArguments:	put: #'<Object>';
	at: #printOn:			put: #'<Object>';
	at: #respondsTo:			put: #'<Object>';
	at: #yourself			put: #'<Object>';
	yourself
! !

!SysProProtocolTest methodsFor:'testing'!

testXaddConformsToProtocolIdX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'addConformsToProtocolId:'.

	[self addJnkProto.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [aProto addConformsToProtocolId: #'<addedProto>']
		should: [:r| aProto conformsToProtocolIds asArray
						= #(#'<nil>' #'<UndefProto1>' #'<addedProto>')]. 

	] ensure: [self removeJnkProto].
!

testXaddMessageX
	" System Protocols <SysProProtocol> "
	| aProto newMsg |
	self messageSelector: #'addMessage:'.

	[self addJnkProto.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	newMsg := SysProProtocolMessage
		newMessageSelector: #'newSelector'
		parameters: #()
		returnValues: #().

	self conformingValue: [aProto addMessage: newMsg]
		should: [:r| (aProto messageAtSelector: #'newSelector')
			messageSelector = #'newSelector']. 

	"Errors: The protocol already specifies a message with a selector
	that matches newMessage's selector."
	self should: [aProto addMessage: newMsg]
		raise: TestResult error.

	] ensure: [self removeJnkProto].
!

testXallConformsToProtocolIds
	" System Protocols <SysProProtocol> "
	self messageSelector: #'allConformsToProtocolIds'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<number>')
				allConformsToProtocolIds]
		should: [:r| r asArray = #(#'<magnitude>' #'<Object>' #'<ANY>' #'<objectProtocols>')].

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>')
				allConformsToProtocolIds]
		should: [:r| r asArray = #(#'<ANY>' #'<objectProtocols>')].

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object class>')
				allConformsToProtocolIds]
		should: [:r| r asArray = #(#'<classDescription>' #'<instantiator>' #'<Object>' #'<classDescriptionProtocols>' #'<ANY>' #'<objectProtocols>')].
!

testXallMessageSelectorProtocolDictionary
	" System Protocols <SysProProtocol> "
	self messageSelector: #'allMessageSelectorProtocolDictionary'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<ANY>')
				allMessageSelectorProtocolDictionary]
		should: [:r| r isEmpty]. 

	self conformingValue: [
			(ProtoMgr protocolId: #'<nil>')
				allMessageSelectorProtocolDictionary]
		shouldnt: [:r| r equals: self nilDictionary].
!

testXallMessageSelectors
        " System Protocols <SysProProtocol> "
        self messageSelector: #'allMessageSelectors'.

        self conformingValue: [
                        (ProtoMgr protocolId: #'<ANY>') allMessageSelectors]
                should: [:r| r isEmpty].

        self conformingValue: [
                        (ProtoMgr protocolId: #'<nil>') allMessageSelectors ]
                should: [:r|
                        r asSortedCollection asArray = #(#'=' #'==' #'class'
                        #'copy' #'doesNotUnderstand:' #'error:' #'hash'
                        #'identityHash' #'isKindOf:' #'isMemberOf:' #'isNil'
                        #'notNil' #'perform:' #'perform:with:' #'perform:with:with:'
                        #'perform:with:with:with:' #'perform:withArguments:'
                        #'printOn:' #'printString' #'respondsTo:'
                        #'respondsToProtocolId:' #'yourself' #'~=' #'~~')].
!

testXconformsToProtocolIds
	" System Protocols <SysProProtocol> "
	self messageSelector: #'conformsToProtocolIds'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') conformsToProtocolIds]
		should: [:r| r asArray = #(#'<ANY>' #'<objectProtocols>')].

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object class>')
				conformsToProtocolIds]
		should: [:r|
			r asArray = #(#'<classDescription>' #'<instantiator>')].

	self conformingValue: [
			(ProtoMgr protocolId: #'<ANY>') conformsToProtocolIds]
		should: [:r| r isEmpty].
!

testXdefinesBehaviorOfList
	" System Protocols <SysProProtocol> "
	self messageSelector: #'definesBehaviorOfList'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') definesBehaviorOfList]
		should: [:r| r asArray = (Array with: Object)].

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object class>')
				definesBehaviorOfList]
		should: [:r| r asArray = (Array with: Object class)].
!

testXdisplayOnX
	" System Protocols <SysProProtocol> "
	| aStream |
	self messageSelector: #'displayOn:'.

	aStream := WriteStream with: String new.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') displayOn: aStream]
		should: [:r| aStream contents = '<Object>'].

	aStream := WriteStream with: String new.
	self conformingValue: [
			(ProtoMgr protocolId: (SysProProtocol protocolANYId))
				displayOn: aStream]
		should: [:r| aStream contents = '<ANY>'].
!

testXdisplayString
	" System Protocols <SysProProtocol> "
	self messageSelector: #'displayString'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') displayString]
		should: [:r| r = '<Object>'].

	self conformingValue: [
			(ProtoMgr protocolId: (SysProProtocol protocolANYId))
				displayString]
		should: [:r| r = '<ANY>'].
!

testXlessThanOrEqualToOp
	" System Protocols <SysProProtocol> "
	| numberProto |
	self messageSelector: #'<='.

	numberProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [
			numberProto <= (ProtoMgr protocolId: #'<WriteStream>')]
		should: [:r | r].

	self conformingValue: [
			numberProto <= (ProtoMgr protocolId: #'<number>')]
		should: [:r| r].

	self conformingValue: [
			numberProto <= (ProtoMgr protocolId: #'<Float>')]
		shouldnt: [:r| r].
!

testXmessageAtSelectorX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'messageAtSelector:'.

	aProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [aProto messageAtSelector: #'abs']
		should: [:r | r messageSelector = #'abs']. 

	"Errors: The protocol message specification identified
	by selectorSpec. is not found."
	self should: [aProto messageAtSelector: 'bogus']
		raise: TestResult error.
!

testXmessageAtSelectorXifNoneX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'messageAtSelector:ifNone:'.

	aProto := ProtoMgr protocolId: #'<number>'.

	self conformingValue: [
			aProto
				messageAtSelector: #'abs'
				ifNone: []]
		should: [:r| r messageSelector = #'abs']. 

	"Test not found:"
	self conformingValue: [
			aProto
				messageAtSelector: 'bogus'
				ifNone: [1]]
		should: [:r| r = 1]. 
!

testXmessageSelectors
	" System Protocols <SysProProtocol> "
	self messageSelector: #'messageSelectors'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<String>') messageSelectors]
		should: [:r| r any = #'asString'].

	self conformingValue: [
			(ProtoMgr protocolId: #'<ANY>') messageSelectors]
		should: [:r| r isEmpty].
!

testXmessages
	" System Protocols <SysProProtocol> "
	self messageSelector: #'messages'.

	self conformingValue: [(ProtoMgr protocolId: #'<String>') messages]
		should: [:r| r any messageSelector = #'asString']. 

	self conformingValue: [(ProtoMgr protocolId: #'<ANY>') messages]
		should: [:r| r isEmpty]. 
!

testXprintString
	" System Protocols <SysProProtocol> "
	self messageSelector: #'printString'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') printString]
		should: [:r| r = 'a SysProProtocol( <Object> )'].

	self conformingValue: [
			(ProtoMgr protocolId: '<Object class>') printString]
		should: [:r| r = 'a SysProProtocol( <Object class> )'].

	self conformingValue: [(ProtoMgr protocolId: #'<ANY>') printString]
		should: [:r| r = 'a SysProProtocol( <ANY> )'].
!

testXprotocolGroup
	" System Protocols <SysProProtocol> "
	self messageSelector: #'protocolGroup'.

	self conformingValue: [
			(ProtoMgr protocolId: #'<Object>') protocolGroup]
		should: [:r| r = 'ANSI-Fundamental'].

	self conformingValue: [
			(ProtoMgr protocolId: #'<SysProProtocol>') protocolGroup]
		should: [:r| r = 'System Protocols'].

	self conformingValue: [
			(ProtoMgr protocolId: #'<ANY>') protocolGroup]
		should: [:r| r = 'ANSI-Special'].

	"Test a protocol that has no group:"
	[SysProProtocolsManager
		newProtocol: '<noGroup>'
		conformsToProtocols: '<ANY>'
		protocolGroup: ''.

	self conformingValue: [(ProtoMgr protocolId: '<noGroup>') protocolGroup]
		should: [:r| r = 'Unknown'].

	] ensure: [
		ProtoMgr removeProtocolId: '<noGroup>'].
!

testXprotocolId
	" System Protocols <SysProProtocol> "
	self messageSelector: #'protocolId'.

	self conformingValue: [(ProtoMgr protocolId: #'<Object>') protocolId]
		should: [:r| r = #'<Object>'].
!

testXremoveAllSelectorsX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'removeAllSelectors:'.

	[self addAndDefineBehaviorOfJnkProtos.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [
			aProto removeAllSelectors: #('selector' 'selectorParm1:')]
		should: [:r|
			r do: [:msgSpec|
				((msgSpec messageSelector = #'selector')
				| (msgSpec messageSelector = #'selectorParm1:')) ifFalse: [
					^false]].
			true]. 

	"Errors: Any message specification identified by a selector
	in selectorSpecList is not found."
	self should: [
			aProto removeAllSelectors: #('selectorParm1:parm2:' 'bogus')]
		raise: TestResult error.

	] ensure: [
		self removeJnkProtoFactory.
		self removeJnkProto].
!

testXremoveSelectorX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'removeSelector:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [aProto removeSelector: #'selectorParm1:']
		should: [:r| r messageSelector = #'selectorParm1:']. 

	"Errors: The message specification identified by selectorSpec
	is not found."
	self should: [aProto removeSelector: 'bogus']
		raise: TestResult error.

	] ensure: [self removeJnkProto].
!

testXremoveSelectorXifAbsentX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'removeSelector:ifAbsent:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [
			aProto
				removeSelector: #'selectorParm1:'
				ifAbsent: [1]]
		should: [:r| r messageSelector = #'selectorParm1:']. 

	self conformingValue: [
			aProto
				removeSelector: 'bogus'
				ifAbsent: [1]]
		should: [:r| r = 1].

	] ensure: [self removeJnkProto].
!

testXsectionAtX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'sectionAt:'.

	[self addJnkProto.
	self addDescSecOfJnkProto.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [aProto sectionAt: #'Description']
		should: [:r| r = 'Represents a 2nd. System Protocols SUnits test protocol.
']. 

	self conformingValue: [aProto sectionAt: 'Bogus']
		should: [:r| r isEmpty]. 

	] ensure: [self removeJnkProto].
!

testXsectionAtXputX
	" System Protocols <SysProProtocol> "
	| aProto |
	self messageSelector: #'sectionAt:put:'.

	[self addJnkProto.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.

	self conformingValue: [
			aProto
				sectionAt: 'Header'
				put: 'Section text.']
		should: [:r| (aProto sectionAt: 'Header') = 'Section text.']. 

	] ensure: [self removeJnkProto].
!

testXsectionHeadingsAndTextsDoX
	" System Protocols <SysProProtocol> "
	| aProto results |
	self messageSelector: #'sectionHeadingsAndTextsDo:'.

	[self addJnkProto.
	self addDescSecOfJnkProto.
	aProto := ProtoMgr protocolId: #'<JnkProto>'.
	aProto
		sectionAt: 'Header'
		put: 'Protocol section text.'.
	results := Set new.
	self conformingValue: [
			aProto sectionHeadingsAndTextsDo: [:secHeading :secText|
				secHeading = #'Header'
					ifTrue: [
						results add: (secText = 'Protocol section text.')].
				secHeading = #'Description'
					ifTrue: [
						results add: (secText = 'Represents a 2nd. System Protocols SUnits test protocol.
')].
				(secHeading = #'Header') | (secHeading = #'Description')
					ifFalse: [results add: false]]]
		shouldnt: [:r| results includes: false]. 

	] ensure: [self removeJnkProto].
!

testXstoreDefineBehaviorOnX
	" System Protocols <SysProProtocol> "
	| resultStream aProto |
	self messageSelector: #'storeDefineBehaviorOn:'.

	[self addAndDefineBehaviorOfJnkProtos.

	aProto := ProtoMgr protocolId: #'<JnkProto>'.
	resultStream := ReadWriteStream with: String new.
	self conformingValue: [aProto storeDefineBehaviorOn: resultStream]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: self jnkProtoDefineStr.
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]. 

	aProto := ProtoMgr protocolId: #'<JnkProto factory>'.
	resultStream := ReadWriteStream with: String new.
	self conformingValue: [aProto storeDefineBehaviorOn: resultStream]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: self jnkProtoFactoryDefineStr.
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]. 

	] ensure: [
		self removeJnkProtoFactory.
		self removeJnkProto].
!

testXstoreOnXwithDefineBehaviorX
	" System Protocols <SysProProtocol> "
	| resultStream aProto |
	self messageSelector: #'storeOn:withDefineBehavior:'.

	self notDone. "Do withDefineBehavior: false & true"
	[self addAndDefineBehaviorOfJnkProtos.

	aProto := ProtoMgr protocolId: #'<JnkProto>'.
	resultStream := ReadWriteStream with: String new.
	self conformingValue: [
			aProto
				storeOn: resultStream
				withDefineBehavior: false]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: (self jnkProtoStr).
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]. 

	aProto := ProtoMgr protocolId: #'<JnkProto factory>'.
	resultStream := ReadWriteStream with: String new.
	self conformingValue: [
			aProto
				storeOn: resultStream
				withDefineBehavior: false]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: (self jnkProtoFactoryStr).
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]. 

	] ensure: [
		self removeJnkProtoFactory.
		self removeJnkProto].
! !

!SysProProtocolTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolTest.st,v 1.1 2002/06/21 10:34:11 penk Exp $'
! !
