"{ Package: 'stx:goodies/ansiTests/common' }"

SysProGroup subclass:#SysProProtocolsManagerFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-System Protocols'
!


!SysProProtocolsManagerFactoryTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SysProProtocolsManager factory>'
! !

!SysProProtocolsManagerFactoryTest methodsFor:'constants'!

canonicalObject

	^SysProProtocolsManager class
! !

!SysProProtocolsManagerFactoryTest methodsFor:'testing'!

testXcurMgr
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'curMgr'.

	self conformingValue: [SysProProtocolsManager curMgr]
		should: [:r| r == SysProProtocolsManager curMgr]. 
!

testXdefaultReturnProtocolIdReceiverXoperandX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'defaultReturnProtocolIdReceiver:operand:'.

	self conformingValue: [
			SysProProtocolsManager
				defaultReturnProtocolIdReceiver: 1
				operand: 2]
		should: [:r| r == #'<integer>']. 

	self conformingValue: [
			SysProProtocolsManager
				defaultReturnProtocolIdReceiver: 1
				operand: (1/2)]
		should: [:r| r == #'<rational>'].

	"Errors: If receiver or operand does not conform to
	any numeric protocol."
	self should: [
			SysProProtocolsManager
				defaultReturnProtocolIdReceiver: 'xxx'
				operand: (1/2)]
		raise: TestResult error.
!

testXnewProtocolXconformsToProtocolsXprotocolGroupX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'newProtocol:conformsToProtocols:protocolGroup:'.

	[self conformingValue: [
			SysProProtocolsManager
				newProtocol: '<JnkProto>'
				conformsToProtocols: '<nil>, <UndefProto1>'
				protocolGroup: 'Junk Group']
		should: [:r|
			(r == (ProtoMgr protocolId: #'<JnkProto>'))
			& ProtoMgr undefinedConformsToProtocolIds asArray = #(#'<UndefProto1>')]. 

	"Errors: A protocol with identifier protocolSpec already exists
	in the list of protocols."
	self should: [
			SysProProtocolsManager
				newProtocol: '<JnkProto>'
				conformsToProtocols: '<nil>'
				protocolGroup: 'Junk Group']
		raise: TestResult error.
	] ensure: [self removeJnkProto].

	"Errors: Identifier protocolSpec not bracketed with '>'."
	self should: [
			SysProProtocolsManager
				newProtocol: 'xxx'
				conformsToProtocols: 'yyy'
				protocolGroup: 'Junk Group']
		raise: TestResult error.
!

testXnumericInstanceProtocolIdX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'numericInstanceProtocolId:'.

	self conformingValue: [
			SysProProtocolsManager numericInstanceProtocolId: 2]
		should: [:r| r == #'<integer>']. 

	self conformingValue: [
			SysProProtocolsManager numericInstanceProtocolId: (1/2)]
		shouldnt: [:r| r == #'<rational>'].

	"Errors: If number does not conform to any numeric protocol."
	self should: [SysProProtocolsManager numericInstanceProtocolId: 'xxx']
		raise: TestResult error.
!

testXprotocolXdefineBehaviorOfClassesX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:defineBehaviorOfClasses:'.

	[self addJnkProto.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				defineBehaviorOfClasses: 'Array Integer']
		should: [:r|	| proto |
			proto := ProtoMgr protocolId: #'<JnkProto>'.
			(proto definesBehaviorOfList asArray = (Array with: Array with: Integer))
			& (ProtoMgr
				behavior: Array
				conformsToProtocolId: #'<JnkProto>')
			& (ProtoMgr
				behavior: Integer
				conformsToProtocolId: #'<JnkProto>')]. 

	"Errors: A class named in classNamesList does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				defineBehaviorOfClasses: 'JunkClass']
		raise: TestResult error.

	] ensure: [self removeJnkProto].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<UndefProto>'
				defineBehaviorOfClasses: 'Character Integer']
		raise: TestResult error.
!

testXprotocolXdefineBehaviorOfMetaclassesOfClassesX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:defineBehaviorOfMetaclassesOfClasses:'.

	[self addJnkProtoFactory.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto factory>'
				defineBehaviorOfMetaclassesOfClasses: 'Object']
		should: [:r| | proto |
			proto := ProtoMgr protocolId: #'<JnkProto factory>'.
			(proto definesBehaviorOfList asArray = (Array with: Object class))
			& (ProtoMgr
				behavior: Object class
				conformsToProtocolId: '<JnkProto factory>')]. 

	"Errors: A class named in classNamesList does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto factory>'
				defineBehaviorOfMetaclassesOfClasses: 'JunkClass']
		raise: TestResult error.

	] ensure: [self removeJnkProtoFactory].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<UndefProto>'
				defineBehaviorOfMetaclassesOfClasses: 'Object']
		raise: TestResult error.
!

testXprotocolXnewSelectorXparametersXreturnRuleX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:newSelector:parameters:returnRule:'.

	[self addJnkProto.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector'
				parameters: #()
				returnRule: '[ :receiver |	"Sample comment"
	SysProProtocolsManager unaryReturnProtocolIdReceiver: receiver]']
		should: [:r|
			"Return value is the new message:"
			(r == ((ProtoMgr protocolId: #'<JnkProto>')
					messageAtSelector: #'selector'))
			"Undefined has 2 protocol conforms-to & 1 message parm.:"
			& ProtoMgr undefinedConformsToProtocolIds asArray
				= #(#'<UndefProto1>')]. 

	"Errors: The protocol already has a message with selector selectorSpec."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector'
				parameters: #()
				returnRule: '[ :receiver | self]']
		raise: TestResult error.

	"Errors: The protocol message parameter aliasing attribute
	is not valid."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector2:'
				parameters: #('operand <JunkProto3> junk')
				returnRule: '[ :receiver | self]']
		raise: TestResult error.

	"Errors: The protocol message return value rule block source
	will not compile."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector'
				parameters: #()
				returnRule: '[ :receiver | "Will not compile"
					eceiver = self]']
		raise: TestResult error.

	"Errors: The protocol message return value rule block source
	contains a '^'."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector3:'
				parameters: #('operand <JunkProto3> uncaptured')
				returnRule: '[ :receiver | ^self]']
		raise: TestResult error.

	] ensure: [self removeJnkProto].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [SysProProtocolsManager
			protocol: '<UndefProto>'
			newSelector: 'selector'
			parameters: #()
			returnRule: '[ :receiver | self]']
		raise: TestResult error.
!

testXprotocolXnewSelectorXparametersXreturnValuesX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:newSelector:parameters:returnValues:'.

	[self addJnkProto.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selectorParm1:'
				parameters: #('operand <UndefProto1>|<UndefProto2> uncaptured')
				returnValues: #('<JnkProto>	new'
					'<nil>	unspecified')]
		should: [:r|
			"Return value is the new message:"
			(r == ((ProtoMgr protocolId: #'<JnkProto>')
					messageAtSelector: #'selectorParm1:'))
			"Undefined has 1 protocol conforms-to & 1 message parm.:"
			& ProtoMgr undefinedConformsToProtocolIds 
				asSortedCollection asArray = #(#'<UndefProto1>' #'<UndefProto2>')]. 

	"Errors: The protocol already has a message with selector selectorSpec."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selectorParm1:'
				parameters: #()
				returnValues: #()]
		raise: TestResult error.

	"Errors: The protocol message parameter or return value
	aliasing attribute is not valid."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector2:'
				parameters: #('operand <UndefProto1> junk')
				returnValues: #()]
		raise: TestResult error.

	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'selector3:'
				parameters: #('operand <UndefProto1> uncaptured')
				returnValues: #('<JnkProto>	junk')]
		raise: TestResult error.

	] ensure: [self removeJnkProto].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<UndefProto>'
				newSelector: 'selectorParm1:'
				parameters: #()
				returnValues: #()]
		raise: TestResult error.
!

testXprotocolXsectionXtextX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:section:text:'.

	[self addJnkProto.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				section: 'Description'
				text: 'Represents a 2nd. System Protocols SUnits test protocol.']
		should: [:r|
			((ProtoMgr
				protocolId: #'<JnkProto>') sectionAt: #'Description')
				= 'Represents a 2nd. System Protocols SUnits test protocol.']. 

	] ensure: [self removeJnkProto].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<UndefProto>'
				newSelector: 'selectorParm1:'
				parameters: #()
				returnValues: #()]
		raise: TestResult error.
!

testXprotocolXselectorXsectionXtextX
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'protocol:selector:section:text:'.

	[self addJnkProto.
	self addJnkProtoXselectorParm1.

	self conformingValue: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				selector: 'selectorParm1:'
				section: 'Section Heading'
				text: 'The section text.']
		should: [:r|
			(((ProtoMgr protocolId: #'<JnkProto>')
					messageAtSelector: #'selectorParm1:'
				) sectionAt: 'Section Heading'
			) = 'The section text.']. 

	"Errors: A protocol message with selector, selectorSpec
	does not exist in protocol with identifier protocolSpec."
	self should: [
			SysProProtocolsManager
				protocol: '<JnkProto>'
				newSelector: 'bogusSelector:'
				section: 'Section Heading'
				text: 'The section text.']
		raise: TestResult error.

	] ensure: [self removeJnkProto].

	"Errors: A protocol with identifier protocolSpec does not exist."
	self should: [
			SysProProtocolsManager
				protocol: '<UndefProto>'
				newSelector: 'selectorParm1:'
				parameters: #()
				returnValues: #()]
		raise: TestResult error.
!

testXwhitespaceSeparators
	" System Protocols <SysProProtocolsManager factory> "
	self messageSelector: #'whitespaceSeparators'.

self anomaly: [
	self conformingValue: [ SysProProtocolsManager whitespaceSeparators]
		should: [:r| r allSatisfy: [ :char| char isWhitespace]].
] for: #(#'SQ27' "2001/04/21 Harmon, R. dnu isWhitespace error").
! !

!SysProProtocolsManagerFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolsManagerFactoryTest.st,v 1.1 2002/06/21 10:37:48 penk Exp $'
! !
