"{ Package: 'stx:goodies/ansiTests/common' }"

SysProGroup subclass:#SysProProtocolsManagerTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-ANSI-System Protocols'
!


!SysProProtocolsManagerTest class methodsFor:'constants'!

protocolId
	"Answer the protocol identifier of the protocol the receiver tests."

	^#'<SysProProtocolsManager>'
! !

!SysProProtocolsManagerTest methodsFor:'testing'!

testXallProtocolIds
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'allProtocolIds'.

	self conformingValue: [ProtoMgr allProtocolIds]
		should: [:r| r notEmpty]. 
!

testXallProtocolIdsSorted
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'allProtocolIdsSorted'.

	self conformingValue: [ProtoMgr allProtocolIdsSorted]
		should: [:r| r notEmpty]. 
!

testXallProtocols
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'allProtocols'.

	self conformingValue: [ProtoMgr allProtocols]
		should: [:r| r allSatisfy: [:proto| proto isKindOf: SysProProtocol]]. 
!

testXbehaviorXconformsToProtocolIdX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'behavior:conformsToProtocolId:'.

	self conformingValue: [
			ProtoMgr	"Test class:"
				behavior: 1 class
				conformsToProtocolId: #'<number>']
		should: [:r| r ]. 

	self conformingValue: [
			ProtoMgr	"Test meta-class:"
				behavior: Character class
				conformsToProtocolId: #'<Character factory>']
		should: [:r| r ].

	self conformingValue: [
			ProtoMgr	"Test class:"
				behavior: 1 class
				conformsToProtocolId: #'<Float>']
		shouldnt: [:r| r ].

	self conformingValue: [
			ProtoMgr	"Test meta-class:"
				behavior: Character class
				conformsToProtocolId: #'<symbol>']
		shouldnt: [:r| r ].
!

testXfileOutDefineBehaviorOfProtocolIdsXtoX
	" System Protocols <SysProProtocolsManager> "
	| resultStream |
	self messageSelector: #'fileOutDefineBehaviorOfProtocolIds:to:'.

self anomaly: [
	[self addAndDefineBehaviorOfJnkProtos.

	self conformingValue: [
			ProtoMgr
				fileOutDefineBehaviorOfProtocolIds: self targetProtocolIds
				to: self fileNameXTestFileXjnk]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: self targetDefineString.
			resultStream := FileStream read: self fileNameXTestFileXjnk.
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream].

	] ensure: [
		resultStream isNil ifFalse: [resultStream close].
		self removeJnkProtoFactory.
		self removeJnkProto].
] for: #(#'SQ27' "2001/04/21 Harmon, R. error - FileStream >> #read: returns nil").

	"Errors: A protocol with an identifier in protoIdList does not exist."
	self should: [
			ProtoMgr
				fileOutDefineBehaviorOfProtocolIds: #('<UndefProto>')
				to: self fileNameXTestFileXjnk]
		raise: Error.

	"** Not Checked ** Errors: As defined by <FileStream factory> #write:mode:check:type:."
!

testXfileOutProtocolIdsXwithDefineBehaviorXtoX
	" System Protocols <SysProProtocolsManager> "
	| resultStream |
	self messageSelector: #'fileOutProtocolIds:withDefineBehavior:to:'.

	self notDone. "Do withDefineBehavior: false & true"
self anomaly: [
	[self addAndDefineBehaviorOfJnkProtos.

	self conformingValue: [
			ProtoMgr
				fileOutProtocolIds: self targetProtocolIds
				withDefineBehavior: false
				to: self fileNameXTestFileXjnk]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: self targetProtocolsString.
			resultStream := FileStream read: self fileNameXTestFileXjnk.
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream].

	] ensure: [
		resultStream close.
		self removeJnkProtoFactory.
		self removeJnkProto].
] for: #(#'SQ27' "2001/04/21 Harmon, R. error - FileStream >> #??? returns nil").

	"Errors: A protocol with an identifier in protoIdList does not exist."
	self should: [
			ProtoMgr
				fileOutProtocolIds: #('<UndefProto>')
				withDefineBehavior: false
				to: self fileNameXTestFileXjnk]
		raise: TestResult error.

	"** Not Checked ** Errors: As defined by <FileStream factory> #write:mode:check:type:."
!

testXincludesProtocolIdX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'includesProtocolId:'.

	self conformingValue: [ProtoMgr includesProtocolId: #'<number>']
		should: [:r| r ].

	self conformingValue: [ProtoMgr includesProtocolId: '<UndefProto>']
		shouldnt: [:r| r ].
!

testXprotocolIdX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'protocolId:'.

	self conformingValue: [ProtoMgr protocolId: #'<number>']
		should: [:r| r protocolId = #'<number>' ].

	"Errors: A protocol with protocolSpec does not exist."
	self should: [ProtoMgr protocolId: '<UndefProto>']
		raise: TestResult error.
!

testXprotocolIdXifNoneX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'protocolId:ifNone:'.

	self conformingValue: [
			ProtoMgr
				protocolId: #'<number>'
				ifNone: []]
		should: [:r| r protocolId = #'<number>' ].

	self conformingValue: [
			ProtoMgr
				protocolId: '<UndefProto>'
				ifNone: []]
		should: [:r| r = nil ].
!

testXremoveProtocolIdX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'removeProtocolId:'.

	self addJnkProto.

	self conformingValue: [ProtoMgr removeProtocolId: #'<JnkProto>']
		should: [:r| true ].

	"Errors: The protocol to be removed is protocol <ANY>."
	self should: [ProtoMgr removeProtocolId: #'<ANY>']
		raise: TestResult error.

	"Errors: A protocol with protocolSpec does not exist."
	self should: [ProtoMgr removeProtocolId: '<UndefProto>']
		raise: TestResult error.
!

testXrenameProtocolIdXtoX
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'renameProtocolId:to:'.

	[self addJnkProto.
	self defineJnkProtoBehaviorOfClasses.

	self conformingValue: [
			ProtoMgr
				renameProtocolId: #'<JnkProto>'
				to: '<newNameProto>']
		should: [:r|	| proto |
			proto := ProtoMgr protocolId: '<newNameProto>'.
			((ProtoMgr includesProtocolId: '<JnkProto>') not)
			& (proto definesBehaviorOfList asArray = (Array with: Array with: Integer))
			& (ProtoMgr
				behavior: Array
				conformsToProtocolId: '<newNameProto>')
			& (ProtoMgr
				behavior: Integer
				conformsToProtocolId: '<newNameProto>')].

	] ensure: [ProtoMgr removeProtocolId: '<newNameProto>'].


	"Errors: The protocol to be renamed is protocol <ANY>."
	self should: [
			ProtoMgr
				renameProtocolId: #'<ANY>'
				to: '<FailNameProto>']
		raise: TestResult error.

	"Errors: A protocol with oldProtocolSpec does not exist."
	self should: [
			ProtoMgr
				renameProtocolId: '<UndefProto>'
				to: '<FailNameProto>']
		raise: TestResult error.

	"Errors: A protocol with newProtocolSpec already exists."
	self should: [
			ProtoMgr
				renameProtocolId: #'<SysProProtocolsManager factory>'
				to: #'<SysProProtocolsManager>']
		raise: TestResult error.
!

testXstoreDefineBehaviorOfProtocolIdsXonX
	" System Protocols <SysProProtocolsManager> "
	| resultStream |
	self messageSelector: #'storeDefineBehaviorOfProtocolIds:on:'.

	[self addAndDefineBehaviorOfJnkProtos.
	resultStream := ReadWriteStream with: String new.

	self conformingValue: [
			ProtoMgr
				storeDefineBehaviorOfProtocolIds: self targetProtocolIds
				on: resultStream]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: (self targetDefineString).
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream].
 
	] ensure: [
		self removeJnkProtoFactory.
		self removeJnkProto].

	"Errors: A protocol with an identifier in protoIdList does not exist."
	self should: [
			ProtoMgr
				storeDefineBehaviorOfProtocolIds: #('<UndefProto>')
				on: resultStream]
		raise: TestResult error.
!

testXstoreProtocolIdsXwithDefineBehaviorXonX
	" System Protocols <SysProProtocolsManager> "
	| resultStream |
	self messageSelector: #'storeProtocolIds:withDefineBehavior:on:'.
self anomaly: [
	self notDone. "Do withDefineBehavior: false & true"
	[self addAndDefineBehaviorOfJnkProtos.
	resultStream := ReadWriteStream with: String new.

	self conformingValue: [
			ProtoMgr
				storeProtocolIds: self targetProtocolIds
				withDefineBehavior: false
				on: resultStream]
		should: [:r|	| targetStream |
			targetStream := ReadStream on: (self targetProtocolsString).
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream].

	] ensure: [
		self removeJnkProtoFactory.
		self removeJnkProto].
] for: #(#'SQ27' "2001/04/21 Harmon, R. error - FileStream >> ???").

	"Errors: A protocol with an identifier in protoIdList does not exist."
	self should: [
			ProtoMgr
				storeProtocolIds: #('<UndefProto>')
				withDefineBehavior: false
				on: resultStream]
		raise: TestResult error.
!

testXundefinedConformsToProtocolIds
	" System Protocols <SysProProtocolsManager> "
	self messageSelector: #'undefinedConformsToProtocolIds'.

	self conformingValue: [ProtoMgr undefinedConformsToProtocolIds]
		should: [:r| r isEmpty].

	[self addJnkProto.

	self conformingValue: [ProtoMgr undefinedConformsToProtocolIds]
		should: [:r| r asArray = #(#'<UndefProto1>')]. 

	] ensure: [self removeJnkProto].
! !

!SysProProtocolsManagerTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolsManagerTest.st,v 1.1 2002/06/21 10:34:35 penk Exp $'
! !
