"{ Package: 'stx:goodies/ansiTests/common' }"

TestCaseUsingProtocols subclass:#TestCaseHelper
	instanceVariableNames:'testCase'
	classVariableNames:''
	poolDictionaries:''
	category:'SUnit-Protocol-Framework'
!


!TestCaseHelper class methodsFor:'instance creation'!

selector: selectorSpec attachedTo: aMainTestClass conformsToProtocolId: protocolSpec
	"Answer a new helper test case with the initial state set to testSelector containing the ''ConformsTo'' message specification, identified by selectorSpec, in the ''ConformsTo'' protocol with identifier protocolSpec.  Also testCase contains a new main test case that is an instance of aMainTestClass with testSelector set to the ''ConformsTo'' message and the protocol set to the identifier protocolSpec answered by aMainTestClass.  A main test case is need for concrete information that varies in the helper tests.  An example is a canonical (typical) object like an actual <String> or <symbol> that (<readableString>, #'asLowercase') tests."

	| newMainTestCase |
	newMainTestCase := aMainTestClass
		selector: selectorSpec
		protocolId: aMainTestClass protocolId.
	^self new
		setTestSelector: selectorSpec
		attachTo: newMainTestCase 
		conformsToProtocolId: protocolSpec
! !

!TestCaseHelper methodsFor:'asserting'!

conformingAssertMessageSelector 
	"Record a failure if the result of sending the message being tested to a main test case typical instance (canonicalObject) evaluates to an object that does not conform to the return value specified for the message, messageSelector, in protocol with protocolId.
	See Also: TestCaseUsingProtocols>>#conformingAssertMessageSelector.
	Important: the message selector must first be set.
	Also note that this test really says nothing other than this message is implemented and the result of send it is of the specified type. 
	Example testing ANSI-Collection (<readableString>, #'asLowercase'):

	self messageSelector: #'asLowercase'.
	self conformingAssertMessageSelector."

	self conformingValue: [
			testCase canonicalObject perform: messageSelector]
		should: [:result| true].

! !

!TestCaseHelper methodsFor:'error handling'!

anomaly: anomalousBlock forDialectAndTestCaseName: dialectTestNameList 
	"Depending on SignalAnomalies selectively ignore SUnit test code in anomalousBlock that has some anomaly for dialects and tests indicated in dialectTestNameList.  Each element of dialectTestNameList array should contain a dialect and version <symbol> and an array of associated main test class name <symbol>s that exhibit the anomaly.   See class>>#dialectAndVersion for recommended <symbol> of dialects and versions.
	Options are initially set to signal anomalies.  It is just easier sometimes to ignore non-working tests yet not miss anomalies one has not dealt with in a clutter of known anomalies.  See Also: TestCaseHelper>> #anomaly:for:.  Example:

	self anomaly: [
		obj aMsg
	] forDialectAndTestCaseName: #(
		#(#'DO30' ''2000/12/06 Harmon, R.''
			#(#'ExceptionTest' ''Blows up.''
			#'WarningTest' ''r -> dev. session mgr''))
		#(#'SQ27' ''2000/12/06 Harmon, R.''
			#(#'ExceptionTest' ''Blows up.'')))."

	SignalAnomalies ifTrue: [
		^anomalousBlock value].

	dialectTestNameList do: [:dialectAndListOfTestNames|
		| dialect testCaseNameList |
		dialect := dialectAndListOfTestNames at: 1.
		dialect = DialectAndVersion ifTrue: [
		testCaseNameList := dialectAndListOfTestNames at: 2.
		(testCaseNameList includes: (testCase class name)) ifTrue: [
			^self]]].

	^anomalousBlock value
! !

!TestCaseHelper methodsFor:'printing'!

printOn: aStream
	"Write to a stream a textual description of the receiver as a developer would want to see it.  Note this will show the main test case class, not the helper test case class.  This is to keep all the main and helper tests grouped together.  It also helps to avoid confusion over which path to the helper caused the error."

	aStream
		nextPutAll: testCase class printString;
		nextPutAll: '>>';
		nextPutAll: testCase testSelector
! !

!TestCaseHelper methodsFor:'private'!

setTestSelector: selectorSpec attachTo: aMainTestCase conformsToProtocolId: protocolSpec 
	"Private - "
	testSelector := selectorSpec asSymbol.
	testCase := aMainTestCase.
	protocolId := protocolSpec asSymbol.
! !

!TestCaseHelper methodsFor:'running'!

runCase
	"Override the TestCase method to add the main test set up, run the helper case, and ensure the main test tear down is done."

	testCase setUp.
	[super runCase] sunitEnsure: [
		testCase tearDown]
!

runCaseAsFailure
	"Override the TestCase method to add the main test set up, run the helper case as a failure, and ensure the main test tear down is done."

	testCase setUp.
	[super runCaseAsFailure] sunitEnsure: [
		testCase tearDown]
! !

!TestCaseHelper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/TestCaseHelper.st,v 1.1 2002/06/21 10:30:12 penk Exp $'
! !
