"{ Package: 'stx:goodies/ansiTests/common' }"

TestCase subclass:#TestCaseUsingProtocolsTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tests-SUnitProtocolFramework'
!


!TestCaseUsingProtocolsTest methodsFor:'helpers'!

setUp

	super setUp.

	"Add dummy protocol for testing:"
	self addUsingProto.
	"Set the protocol this class tests:"
	MainUsingProtocolsTest protocolId: #'<usingProtocolsTest>'.
!

tearDown
	"Nil out the protocol this class tests so it will be bypassed
		when asked for suite of tests, and other class inst. vars.:"
	MainUsingProtocolsTest initializeClassInstVars.

	"Remove dummy protocol:"
	(SysProProtocolsManager curMgr
			includesProtocolId:  #'<usingProtocolsTest>'
	) ifTrue: [
		self delUsingProto].
! !

!TestCaseUsingProtocolsTest methodsFor:'private'!

addUsingProto
	"Private - Add <usingProtocolsTest> to use in testing."
	SysProProtocolsManager
		newProtocol: '<usingProtocolsTest>'
		conformsToProtocols: '<ANY>'
		protocolGroup: 'Junk Group'.
!

delUsingProto
	"Private - Remove <usingProtocolsTest> used in testing."
	SysProProtocolsManager curMgr
			removeProtocolId: #'<usingProtocolsTest>'.
! !

!TestCaseUsingProtocolsTest methodsFor:'testing'!

testClsXprotoMgr
	| res |
	res := MainUsingProtocolsTest
			runSel: #'testClsXprotoMgr'.
	self should: [res hasPassed].
!

testClsXprotocolId

	self should: [TestCaseUsingProtocols protocolId == #'<>'].
!

testClsXrunTestCaseX
        | testCaseClass |
        testCaseClass := (Smalltalk at: #'SysProProtocolsManagerFactoryTest').   testCaseClass runTestCase: #'testXcurMgr'.

        self should: [testCaseClass runTestCase: #'bozo']
                raise: TestResult error
!

testClsXsignalAnomaliesX
	| holdSignalAnomalies res |
	holdSignalAnomalies := TestCaseUsingProtocols signalAnomalies.

	[TestCaseUsingProtocols signalAnomalies: true.
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest runSel: #'testClsXsignalAnomaliesX'.
	self should: [res hasPassed].

	TestCaseUsingProtocols signalAnomalies: false.
	MainUsingProtocolsTest shouldValue: false.
	res := MainUsingProtocolsTest runSel: #'testClsXsignalAnomaliesX'.
	self should: [res hasPassed].
	] ensure: [
		TestCaseUsingProtocols signalAnomalies: holdSignalAnomalies].
!

testClsXsignalNotDoneX
	| holdSignalNotDone res |
	holdSignalNotDone := TestCaseUsingProtocols signalNotDone.

	[TestCaseUsingProtocols signalNotDone: true.
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest runSel: #'testClsXsignalNotDoneX'.
	self should: [res hasPassed].

	TestCaseUsingProtocols signalNotDone: false.
	MainUsingProtocolsTest shouldValue: false.
	res := MainUsingProtocolsTest runSel: #'testClsXsignalNotDoneX'.
	self should: [res hasPassed].
	] ensure: [
		TestCaseUsingProtocols signalNotDone: holdSignalNotDone].
!

testClsXtestMethods
	| methodSelectors |
	methodSelectors := MainUsingProtocolsTest testMethods.
	self should: [
			methodSelectors allSatisfy: [:selector|
				(selector copyFrom: 1 to: 4) = 'test']].
!

testXanomalyXforX
	| holdSignalAnomalies res |
	holdSignalAnomalies := TestCaseUsingProtocols signalAnomalies.

	"Anomalies Signal/Ignore Table 
		SignalAnomalies	| true   | false
	dialectList ---------|--------|-------
		this dialect		| signal | ignore
		other dialect	| signal | signal"

	["SignalAnomalies: true & dialectList: this dialect -> signal:"
	TestCaseUsingProtocols signalAnomalies: true.
	MainUsingProtocolsTest forDialectList: (Array with: (PU dialectAndVersionTag)).
	res := MainUsingProtocolsTest runSel: #'testXanomalyXforX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"SignalAnomalies: true & dialectList: other dialect -> signal:"
	MainUsingProtocolsTest forDialectList: #(#'OtherDialect00').
	res := MainUsingProtocolsTest runSel: #'testXanomalyXforX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"SignalAnomalies: false & dialectList: this dialect -> ignore:"
	TestCaseUsingProtocols signalAnomalies: false.
	MainUsingProtocolsTest forDialectList: (Array with: (PU dialectAndVersionTag)).
	res := MainUsingProtocolsTest runSel: #'testXanomalyXforX'.
	self should: [res hasPassed].

	"SignalAnomalies: false & dialectList: other dialect -> signal:"
	MainUsingProtocolsTest forDialectList: #(#'OtherDialect00').
	res := MainUsingProtocolsTest runSel: #'testXanomalyXforX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	] ensure: [
		TestCaseUsingProtocols signalAnomalies: holdSignalAnomalies].
!

testXcompareStreamIgnoringLineTermTargetXresultX
	| res |
	res := MainUsingProtocolsTest
		runSel: #'testXcompareStreamIgnoringLineTermTargetXresultX'.
	self should: [res hasPassed].
!

testXcompareTextIgnoringLineTermTargetXresultX	| res |

	res := MainUsingProtocolsTest
		runSel: #'testXcompareStreamIgnoringLineTermTargetXresultX'.
	self should: [res hasPassed].
!

testXconformingAssertMessageSelector
        | res |
        SysProProtocolsManager
                protocol: '<usingProtocolsTest>'
                newSelector: 'asString'
                parameters: #()
                returnValues: #('<String> unspecified').
        MainUsingProtocolsTest canonicalObject: 'bozo' copy.
        MainUsingProtocolsTest targetSelector: #'asString'.
        "Test sucessful #conformingAssertMessageSelector usage:"
        res := MainUsingProtocolsTest
                        runSel: #'testXconformingAssertMessageSelector'.
        self should: [res hasPassed].

        "Test failure resulting from #valueTrue message returning wrong type:"
        (SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
                removeSelector: #'asString'.
        SysProProtocolsManager
                protocol: '<usingProtocolsTest>'
                newSelector: 'asString'
                parameters: #()
                returnValues: #('<nil> unspecified').
        res := MainUsingProtocolsTest
                        runSel: #'testXconformingAssertMessageSelector'.
        self should: [res failureCount = 1 & res errorCount = 0].

        "Test failure resulting from invalid messageSelector:"
        MainUsingProtocolsTest targetSelector: nil.
        res := MainUsingProtocolsTest
                        runSel: #'testXconformingAssertMessageSelector'.
        self should: [res failureCount = 0 & res errorCount = 1].

        "Test failure resulting from invalid messageSelector:"
        MainUsingProtocolsTest canonicalObject: nil.
        MainUsingProtocolsTest targetSelector: #'asString'.
        self halt.
        res := MainUsingProtocolsTest
                        runSel: #'testXconformingAssertMessageSelector'.
        self should: [res failureCount = 0 & res errorCount = 1].

        "Test failure resulting from missing #asString message selector:"
        (SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
                removeSelector: #'asString'.
        res := MainUsingProtocolsTest
                        runSel: #'testXconformingAssertMessageSelector'.
        self should: [res failureCount = 0 & res errorCount = 1].

        "Test error resulting from no <usingProtocolsTest>:"
        self delUsingProto.
        self should: [MainUsingProtocolsTest suite]
                raise: Error.
!

testXconformingMessageSelectorBehavesLikeX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'isDigit'
		parameters: #()
		returnValues: #('<boolean> unspecified').
	MainUsingProtocolsTest arrayOfTriplets: #(#($1 true) #($A false)).
	MainUsingProtocolsTest targetSelector: #'isDigit'.
	"Test sucessful #conformingMessageSelectorBehavesLikeX usage:"
	res := MainUsingProtocolsTest
			runSel: #'testXconformingMessageSelectorBehavesLikeX'.
	self should: [res hasPassed].

	"Test failure resulting from #valueTrue message returning wrong type:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'isDigit'.
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'isDigit'
		parameters: #()
		returnValues: #('<nil> unspecified').
	res := MainUsingProtocolsTest
			runSel: #'testXconformingMessageSelectorBehavesLikeX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from invalid messageSelector:"
	MainUsingProtocolsTest targetSelector: nil.
	res := MainUsingProtocolsTest
			runSel: #'testXconformingMessageSelectorBehavesLikeX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"Test failure resulting from invalid messageSelector:"
	MainUsingProtocolsTest targetSelector: #'isBozo'.
	res := MainUsingProtocolsTest
			runSel: #'testXconformingMessageSelectorBehavesLikeX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"Test failure resulting from missing #isDigit message selector:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'isDigit'.
	res := MainUsingProtocolsTest
			runSel: #'testXconformingMessageSelectorBehavesLikeX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"Test error resulting from no <usingProtocolsTest>:"
	self delUsingProto.
	self should: [MainUsingProtocolsTest suite]
		raise: Error.
!

testXconformingValueXshouldX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'valueTrue'
		parameters: #()
		returnValues: #('<boolean> unspecified').

	"Test sucessful #conformingValue:should: usage:"
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldX'.
	self should: [res hasPassed].

	"Test failure resulting from #valueTrue message returning wrong type:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'valueTrue'.
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'valueTrue'
		parameters: #()
		returnValues: #('<nil> unspecified').
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from missing #valueTrue message selector:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'valueTrue'.
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test error resulting from no <usingProtocolsTest>:"
	self delUsingProto.
	self should: [MainUsingProtocolsTest suite]
		raise: Error.
!

testXconformingValueXshouldXruleReceiverX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'dummyPart'
		parameters: #()
		returnRule: '[:receiver|
			(receiver isKindOf: Integer)
				ifTrue: [#''<rational>'']
				ifFalse: [#''<scaledDecimal>'']]'.

	"Test sucessful #conformingValue:should:ruleReceiver: usage:"
	MainUsingProtocolsTest targetSelector: #'dummyPart'.
	MainUsingProtocolsTest canonicalObject: 10.
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverX'.
	self should: [res hasPassed].


	"Test failure of shouldBlock:"
	MainUsingProtocolsTest shouldValue: false.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from #dummyPart message returning wrong type
		<Float> will not conform to <scaledDecimal>:"
	MainUsingProtocolsTest canonicalObject: 9.0. 
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from #dummyPart message not specifying rule:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'dummyPart'.
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'dummyPart'
		parameters: #()
		returnValues: #('<nil> unspecified').
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"Test failure resulting from missing #dummyPart message selector:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'dummyPart'.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test error resulting from no <usingProtocolsTest>:"
	self delUsingProto.
	self should: [MainUsingProtocolsTest suite]
		raise: Error.
!

testXconformingValueXshouldXruleReceiverXoperandX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: '*'
		parameters: #('operand <number> unspecified')
		returnRule: '[:receiver :operand|
			(receiver isKindOf: Integer)
				ifTrue: [#''<rational>'']
				ifFalse: [#''<scaledDecimal>'']]'.

	"Test sucessful #conformingValue:should:ruleReceiver:operand: usage:"
	MainUsingProtocolsTest targetSelector: #'*'.
	MainUsingProtocolsTest canonicalObject: 10.
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverXoperandX'.
	self should: [res hasPassed].


	"Test failure of shouldBlock:"
	MainUsingProtocolsTest shouldValue: false.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverXoperandX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from #* message returning wrong type
		<Float> will not conform to <scaledDecimal>:"
	MainUsingProtocolsTest canonicalObject: 9.0. 
	MainUsingProtocolsTest shouldValue: true.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverXoperandX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from #* message not specifying rule:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'*'.
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: '*'
		parameters: #('operand <number> unspecified')
		returnValues: #('<nil> unspecified').
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverXoperandX'.
	self should: [res failureCount = 0 & res errorCount = 1].

	"Test failure resulting from missing #* message selector:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'*'.
	res := MainUsingProtocolsTest
		runSel: #'testXconformingValueXshouldXruleReceiverXoperandX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test error resulting from no <usingProtocolsTest>:"
	self delUsingProto.
	self should: [MainUsingProtocolsTest suite]
		raise: Error.
!

testXconformingValueXshouldntX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'valueTrue'
		parameters: #()
		returnValues: #('<boolean> unspecified').

	"Test sucessful #conformingValue:shouldnt: usage:"
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldntX'.
	self should: [res hasPassed].

	"Test failure resulting from #valueTrue message returning wrong type:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'valueTrue'.
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'valueTrue'
		parameters: #()
		returnValues: #('<nil> unspecified').
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldntX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test failure resulting from missing #valueTrue message selector:"
	(SysProProtocolsManager curMgr protocolId: #'<usingProtocolsTest>')
		removeSelector: #'valueTrue'.
	res := MainUsingProtocolsTest runSel: #'testXconformingValueXshouldntX'.
	self should: [res failureCount = 1 & res errorCount = 0].

	"Test error resulting from no <usingProtocolsTest>:"
	self delUsingProto.
	self should: [MainUsingProtocolsTest suite]
		raise: Error.
!

testXdebug
	| res |
	res := MainUsingProtocolsTest runSel: #'testXdebug'.
	self should: [res failureCount = 0 & res errorCount = 1].
!

testXdebugAsFailure
	| res |
	res := MainUsingProtocolsTest runSel: #'testXdebugAsFailure'.
	self should: [res failureCount = 1 & res errorCount = 0].
!

testXfileNameXTestFileXjnk
	| res |
	res := MainUsingProtocolsTest
			runSel: #'testXfileNameXTestFileXjnk'.
	self should: [res hasPassed].
!

testXlineDelimiter
	| res |
	res := MainUsingProtocolsTest
			runSel: #'testXlineDelimiter'.
	self should: [res hasPassed].
!

testXmessageSelectorX
	| res |
	res := MainUsingProtocolsTest
			runSel: #'testXmessageSelectorX'.
	self should: [res hasPassed].
!

testXnumbersXbyNumbersXanswersXdoX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: '+'
		parameters: #('operand <number> unspecified')
		returnRule: '[ :receiver :operand |
			SysProProtocolsManager
				defaultReturnProtocolIdReceiver: receiver
				operand: operand]'.

	"Test sucessful #conformingValue:should:ruleReceiver:operand: usage:"
	MainUsingProtocolsTest targetSelector: #'+'.
	res := MainUsingProtocolsTest
		runSel: #'testXnumbersXbyNumbersXanswersXdoX'.
	self should: [res hasPassed].
!

testXobjectsXanswersXdoX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'abs'
		parameters: #()
		returnValues: #('<number> unspecified').

	"Test sucessful #objects:answers:do:"
	MainUsingProtocolsTest targetSelector: #'abs'.
	res := MainUsingProtocolsTest
		runSel: #'testXobjectsXanswersXdoX'.
	self should: [res hasPassed].
!

testXobjectsXobjectsXanswersXdoX
	| res |
	SysProProtocolsManager
		protocol: '<usingProtocolsTest>'
		newSelector: 'asScaledDecimal:'
		parameters: #('scale <integer> unspecified')
		returnValues: #('<scaledDecimal> unspecified').

	"Test sucessful #objects:objects:answers:do:"
	MainUsingProtocolsTest targetSelector: #'asScaledDecimal:'.
	res := MainUsingProtocolsTest
		runSel: #'testXobjectsXobjectsXanswersXdoX'.
	self should: [res hasPassed].
!

testXtestSelector
	| res |
	res := MainUsingProtocolsTest
			runSel: #'testXtestSelector'.
	self should: [res hasPassed].
! !

!TestCaseUsingProtocolsTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/TestCaseUsingProtocolsTest.st,v 1.1 2002/06/21 10:35:06 penk Exp $'
! !
