"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/authentication' }"

"{ NameSpace: Authentication }"

Object subclass:#Authenticator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Authentication'
!

!Authenticator class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This is an abstract superclass.
    Authenticators implement the protocol to authenticate an user.
    They generate challenges and check the returned credentials.

    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]

    [class variables:]

    [see also:]
        Authentication::AuthenticationData

"
!

examples
"
                                                                [exBegin]
    |authenticationData|

    authenticationData := Authentication::BasicAuthenticator 
                                newAuthenticationDataFor:nil secret:'passwort'.

    authenticationData storeOn:Transcript. Transcript cr.

    'exception is raised if wrong passwort'.    
    Authentication::BasicAuthenticator new
                        secret:'passwort';
                        authenticateWith:authenticationData
                                                                [exEnd]

                                                                [exBegin]
    |authenticationData|

    authenticationData := Authentication::DigestAuthenticator 
                                newAuthenticationDataFor:'testUser:testRealm' secret:'passwort'.
    authenticationData storeOn:Transcript. Transcript cr.

    authenticationData inspect.
                                                                [exEnd]
"
! !

!Authenticator class methodsFor:'instance creation'!

authenticatorNamed:authenticatorName
    "get the authenticator name for 'Basic', 'Digest', ...
     - if the authenticatorName is unknown, raise an error."

    ^ (self authenticatorClassNamed:authenticatorName) new.


    "
      self authenticatorNamed:#Basic
      self authenticatorNamed:#Digest
      self authenticatorNamed:#blaFasel
    "
! !

!Authenticator class methodsFor:'protocol'!

authenticateForResponse:aResponseString resolveUserVia:aOneArgBlock
    "authenticate using aResponseString. 
     Resolve users via aOneArgBlock that gets the username as argument.
     Raise an exception, it authentication fails.
     Answer the authenticator containing the authentication parameters"

    |authenticator|

    authenticator := self initializeFromResponse:aResponseString.
    authenticator authenticateWithUserResolver:aOneArgBlock.
    ^ authenticator.

    "
      self authenticateForResponse:'Basic abcdefgh==' resolveUserVia:[:userName ]
    "
!

generateChallengeForRealm:aRealmString
    "generate a challenge for a client"

    ^ self subclassResponsibility
!

initializeFromResponse:aResponseString
    "create and initialize an authenticator using aResponseString. 
     Answer the authenticator containing the authentication parameters"

    |index mechanism rest authenticator|

    index := aResponseString indexOfSeparator.

    mechanism := aResponseString copyTo:index-1.
    rest := aResponseString copyFrom:index+1.

    authenticator := self authenticatorNamed:mechanism.
    authenticator initializeWith:rest.
    ^ authenticator.

    "
      self initializeFromResponse:'Basic abcdefgh=='
    "
!

newAuthenticationDataFor:user secret:aSecretString
    "answer the authentication data used by this authenticator"

    ^ self newAuthenticationData user:user secret:aSecretString
! !

!Authenticator class methodsFor:'queries'!

authenticatorClassNamed:authenticatorName
    "get the authenticator class for a 
     mechanism name name like 'Basic', 'Digest', ...
     or raise an error"
    
    |cls|

    cls := Authentication classNamed:(authenticatorName, 'Authenticator').
    cls isNil ifTrue:[
        ParameterError raiseRequestWith:authenticatorName errorString:' - unknown mechanism: ', authenticatorName printString.
        ^ nil.
    ].
    ^ cls


    "
      self authenticatorClassNamed:#Basic
      self authenticatorClassNamed:#Digest
      self authenticatorClassNamed:#blaFasel
    "
!

knownAuthenticators
    "answer the authenticator classes, we know"

    "/ at least, these two must be present
    BasicAuthenticator autoload.
    DigestAuthenticator autoload.

    ^ self subclasses
!

mechanismName
    "answer the name of the mechanism as known in the protocols"

    self subclassResponsibility
! !

!Authenticator methodsFor:'protocol'!

authenticateWithUserResolver:aOneArgBlock
    "authenticate - resolve the username via aOneArgBlock"

    self subclassResponsibility
!

bodyData:aStringOrByteArary
    "pass the body data.
     some authenticators need this (Digest with qop=auth-int)"
!

generateStaleChallenge
    "generate a challenge for a client.
     This is sent when the credentials are stale, i.e. the user has already
     supplied valid username/password and will not be asked again.

     Not all Authenticators support stale challenges"

    ^ self subclassResponsibility
!

initializeWith:aString 
    "inititialize the authentiaction parameters from aString"
    
    self subclassResponsibility
!

requestMethod:methodString
    "pass the request method (like 'GET', 'POST', ...).
     some authenticators need this"
! !

!Authenticator methodsFor:'queries'!

isValidUri:uriString 
    "anser true, if authenticator is valid for uri in uriString"
    
    ^ true
!

mechanismName
    ^ self class mechanismName
! !

!Authenticator methodsFor:'testing'!

isBasic
    ^ false

    "Created: / 25-11-2011 / 13:43:52 / sr"
!

isDigest
    ^ false

    "Created: / 25-11-2011 / 13:43:59 / sr"
! !

!Authenticator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__Authenticator.st,v 1.5 2011/11/25 13:33:37 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__Authenticator.st,v 1.5 2011/11/25 13:33:37 sr Exp $'
! !
