"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/authentication' }"

"{ NameSpace: Authentication }"

Authenticator subclass:#BasicAuthenticator
	instanceVariableNames:'username secret'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Authentication'
!

AuthenticationData subclass:#BasicAuthenticationData
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:BasicAuthenticator
!

!BasicAuthenticator class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This class implements Basic Authentication as defined in RFC 2617.

    Note that Basic Authentication is insecure and should be used only for backward compatibility.

    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]
        username        String      the username as supplied by the client in its credentials
        password        String      the password as supplied by the client in its credentials

    [class variables:]

    [see also:]

"
! !

!BasicAuthenticator class methodsFor:'protocol'!

generateChallengeForRealm:aRealm 
    "generate a challenge for a client"

    ^ 'Basic realm="<1s>"' expandMacrosWith:aRealm.

    "
        self generateChallengeForRealm:'realm'
    "
!

mechanismName
    "answer the name of the mechanism as known in the protocols"

    ^ #Basic
!

newAuthenticationData
    "answer the authentication data used by this authenticator"

    ^ BasicAuthenticationData new
! !

!BasicAuthenticator methodsFor:'accessing'!

secret
    ^ secret
!

secret:something
    secret := something.
!

username
    ^ username
!

username:something
    username := something.
! !

!BasicAuthenticator methodsFor:'initialization'!

initializeWith:authString 
    "initialize the algorithm with the response parameters:"
    
    |nameAndPassword|

    nameAndPassword := (Base64Coder decode:authString) asString 
                asCollectionOfSubstringsSeparatedBy:$:.
    nameAndPassword size < 2 ifTrue:[
        ParameterError raiseRequestWith:authString errorString:'bad authentication data'.
    ].
    username := nameAndPassword at:1.
    secret := nameAndPassword at:2.
! !

!BasicAuthenticator methodsFor:'protocol'!

authenticateWith:authenticationData
    "authenticate - against authenticationData.
     Raise BadCredentialsError if authentication fails"

    (authenticationData checkSecret:secret) ifFalse:[
        BadCredentialsError raiseRequest.
    ].
!

authenticateWithUserResolver:aOneArgBlock
    "authenticate - resolve the username via aOneArgBlock.
     Raise BadCredentialsError if authentication fails"

    |user mechanism|

    user := aOneArgBlock value:username.
    user isNil ifTrue:[
        user := UnknownUserError raiseRequestWith:self.
    ].
    mechanism := user mechanismFor:#Basic.
    mechanism isNil ifTrue:[
        mechanism := UnknownUserError raiseRequestWith:user.
    ].

    "This raises an error if not authentic"
    self authenticateWith:mechanism
!

generateStaleChallenge
    "BasicAuthenticator does not support stale challenges"

    self shouldNotImplement
! !

!BasicAuthenticator methodsFor:'testing'!

isBasic
    ^ true

    "Created: / 25-11-2011 / 13:44:19 / sr"
! !

!BasicAuthenticator::BasicAuthenticationData class methodsFor:'queries'!

mechanismName
    "answer the name of the mechanism as known in the protocols"

    ^ #Basic
! !

!BasicAuthenticator::BasicAuthenticationData methodsFor:'accessing'!

user:user secret:passwordString
    "encode the secret for user and store it.
     Use SSHA1 (salted SHA1 to generate a one-way-hash of the password."

    |salt saltedSecret|

    salt := RandomGenerator new nextCharacters:8.
    saltedSecret := SHA1Stream hashValueOf:(passwordString, salt).
    
    encodedSecret := Base64Coder encode:(saltedSecret, salt).
    encodedSecret := '{SSHA1}', encodedSecret.

    "
      self new user:nil secret:'test'.
    "
! !

!BasicAuthenticator::BasicAuthenticationData methodsFor:'private'!

checkSecret:passwordString
    "answer true, if passwordString matches the encoded secret"

    |salt saltedSecret possibleMatch|

    encodedSecret first == ${ ifTrue:[
        saltedSecret := encodedSecret copyFrom:('{SSHA1}' size + 1).
    ] ifFalse:[
        saltedSecret := encodedSecret.
    ].

    saltedSecret := Base64Coder decode:saltedSecret.
    salt := saltedSecret copyLast:8.

    possibleMatch := SHA1Stream hashValueOf:(passwordString, salt).

    ^ saltedSecret startsWith:possibleMatch.

    "
      self new user:nil secret:'test';
               checkSecret:'test'.

      self new user:nil secret:'test';
               checkSecret:'bla'.
    "
! !

!BasicAuthenticator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__BasicAuthenticator.st,v 1.6 2011/11/25 13:33:45 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__BasicAuthenticator.st,v 1.6 2011/11/25 13:33:45 sr Exp $'
! !
