"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/authentication' }"

"{ NameSpace: Authentication }"

Authenticator subclass:#DigestAuthenticator
	instanceVariableNames:'username realm nonce uri opaque qop algorithm nc cnonce response
		requestMethod bodyData'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Authentication'
!

AuthenticationData subclass:#DigestAuthenticationData
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:DigestAuthenticator
!

!DigestAuthenticator class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This class implements Digest Authentication as defined in RFC 2617.

    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]
        username realm nonce uri opaque qop algorithm nc cnonce response    
                String      credential data supplied by the client

        requestMethod       String      the type of request that should be authenticated (for HTTP: 'GET', 'PUT', 'POST')
        timestamp           Timestamp   the timestamp, when the challenge has been generated
        bodyData            String      the data of the body, used when quop=auth-int.

    [class variables:]

    [see also:]

"
! !

!DigestAuthenticator class methodsFor:'default values'!

defaultReplayTimer
    "answer the time in seconds, until the nonce value used to generate
     a response times out.
     So replay attacks are only possible within this time.
     If <= 0, no checking will be done."

    ^ 300.
! !

!DigestAuthenticator class methodsFor:'protocol'!

generateChallengeForRealm:aRealm 
    "generate a challenge for a client.
     The challenge is Base64(timestamp:Hash(random stuff))"

    |nonceStream timestamp nonce|

    "we need the timestamp, to remember the nonce lifetime.
     If you change the timestamp encoding here, you have to change
     the authenticateWith: method too."
    timestamp := UtcTimestamp now asSeconds digitBytes.

    nonceStream := Base64Coder on:'' writeStream.
    nonceStream 
        nextPut:timestamp size;
        nextPutAll:timestamp;
        nextPutAll:(RandomGenerator random nextBytes:10).

    nonce := nonceStream contents.

    ^ 'Digest realm="<1s>", qop="auth,auth-int", nonce="<2s>"' expandMacrosWith:aRealm with:nonce.

    "
        self generateChallengeForRealm:'realm'
    "
!

mechanismName
    "answer the name of the mechanism as known in the protocols"

    ^ #Digest
!

newAuthenticationData
    "answer the authentication data used by this authenticator"

    ^ DigestAuthenticationData new
! !

!DigestAuthenticator methodsFor:'accessing'!

algorithm
    ^ algorithm
!

algorithm:something
    algorithm := something.
!

bodyData:aStringOrByteArary
    "pass the body data.
     some authenticators need this (Digest with qop=auth-int)"

    bodyData := aStringOrByteArary
!

cnonce
    ^ cnonce
!

cnonce:something
    cnonce := something.
!

nc
    ^ nc
!

nc:something
    nc := something.
!

nonce
    ^ nonce
!

nonce:something
    nonce := something.
!

opaque
    ^ opaque
!

opaque:something
    opaque := something.
!

qop
    ^ qop
!

qop:something
    qop := something.
!

realm
    ^ realm
!

realm:something
    realm := something.
!

requestMethod
    ^ requestMethod
!

requestMethod:something
    requestMethod := something.
!

response
    ^ response
!

response:something
    response := something.
!

uri
    ^ uri
!

uri:something
    uri := something.
!

username
    ^ username
!

username:something
    username := something.
! !

!DigestAuthenticator methodsFor:'initialization'!

initializeWith:authString 
    "initialize the algorithm with the response parameters:"
    
    |authStream|

    authStream := authString readStream.
    [ authStream atEnd ] whileFalse:[
        |key value selector|

        authStream skipSeparators.
        key := authStream upTo:$=.
        authStream skipSeparators = $" ifTrue:[
            authStream
                next;
                skipSeparators.
            value := authStream upTo:$".
            authStream skipSeparators = $, ifTrue:[
                authStream next
            ].
        ] ifFalse:[
            value := authStream upTo:$,.
        ].
        selector := (key copyWith:$:) asSymbolIfInterned.
        (#(algorithm: cnonce: nc: nonce: opaque: qop: realm: uri: username: response:) 
            includes:selector) ifTrue:[
            "RFC 2617 says: ignore unknown parameters"
            self perform:selector with:value
        ].
    ].

    "/   self new initializeFrom:'username="xx", realm="test", nonce="d92e7c45ea2b7c719bf2b7245eecdb7d", uri="/Maintenance", response="6a4508685ce768af0daf8f9f55b5ff58", opaque="6df23dc03f9b54cc38a0fc1483df6e21", qop=auth, nc=00000015, cnonce="e6c33502901a30d2"'
! !

!DigestAuthenticator methodsFor:'protocol'!

authenticateWith:mechanism
    "authenticate - resolve the username via aOneArgBlock.
     Raise BadCredentialsError if authentication fails and
     ExpiredCredentialsError if the nonce used is too old (possible replay)"

    |hashA2 hashedResponse now stream a2 replayTimer timestamp cleartextNonceStream timestampSize|

    "authenticate follows the RFC 2617 specification 
        H() is HashValue of
        for qop=auth or auth-int
            response=H(H(a1):nonce:nc:cnonce:qop:H(a2))
        if qop not present follow RFC 2069
            response=H(H(a1):nonce:H(a2))
        a1=username:realm:password
        for qop=auth
            a2=METHOD:uri
        for qop=auth-int
            a2=METHOD:uri:H(entity-body)
        METHOD is ('GET' 'POST' etc.)"

    stream := MD5Stream new.
    stream 
        nextPut:mechanism encodedSecret;    "H(A1)"
        nextPut:$:; nextPut:nonce; nextPut:$:.

    a2 := requestMethod , ':' , uri.
    qop notNil ifTrue:[
        qop = 'auth-int' ifTrue:[
            a2 := a2, ':', (MD5Stream hashValueOf:bodyData).
        ] ifFalse:[qop = 'auth' ifFalse:[
            "Konqueror returns 'qop='auth,auth-int'' here. This is illegal"          
"/            ^ ParameterError raiseRequestErrorString:' - bad qop'.
        ]].

        stream nextPut:nc; nextPut:$:; 
               nextPut:cnonce; nextPut:$:; 
               nextPut:qop; nextPut:$:.
    ].

    hashA2 := MD5Stream hashValueOf:a2.
    stream nextPut:hashA2 hexPrintString asLowercase.
    hashedResponse := stream hashValue.

    "success, if the computed response = the received response"
    hashedResponse hexPrintString asLowercase ~= response ifTrue:[
        ^ BadCredentialsError raiseRequestWith:self.
    ].

    "now check that timestamp (that I placed in nonce) is still valid"
    replayTimer := self replayTimer.
    replayTimer > 0 ifTrue:[
        cleartextNonceStream := Base64Coder on:nonce readStream.
        timestampSize := cleartextNonceStream next.
        [
            timestamp := cleartextNonceStream next:timestampSize.
        ] on:EndOfStreamError do:[:ex|
            "someone corrupted the nonce"
            ^ BadCredentialsError raiseRequestWith:self.
        ].
        timestamp := UtcTimestamp fromSeconds:(LargeInteger digitBytes:timestamp).
        now := UtcTimestamp now.

        (timestamp between:(now subtractSeconds:replayTimer) and:now) ifFalse:[
            ExpiredCredentialsError raiseRequestWith:self.
        ].
    ].
!

authenticateWithUserResolver:aOneArgBlock
    "authenticate - resolve the username via aOneArgBlock.
     Raise BadCredentialsError if authentication fails and
     ExpiredCredentialsError if the nonce used is too old (possible replay)"

    |user mechanism|

    user := aOneArgBlock value:username.
    user isNil ifTrue:[
        user := UnknownUserError raiseRequestWith:self.
    ].
    mechanism := user mechanismFor:#Digest.
    mechanism isNil ifTrue:[
        mechanism := UnknownUserError raiseRequestWith:user.
    ].

    "This raises an error if not authentic"
    self authenticateWith:mechanism
!

generateStaleChallenge 
    "generate a challenge for a client.
     This is sent when the credentials are stale, i.e. the user has already
     supplied valid username/password and will not be asked again."

    |challenge|

    challenge := self class generateChallengeForRealm:realm.
    challenge := challenge, ', stale=true'.

    ^ challenge
! !

!DigestAuthenticator methodsFor:'protocol parameters'!

replayTimer 
    "answer the time in seconds, until the nonce value used to generate
     a response times out.
     So replay attacks are only possible within this time.
     If < 0, no checking will be done"

    ^ self class defaultReplayTimer
! !

!DigestAuthenticator methodsFor:'queries'!

isValidUri:uriString 
    "anser true, if authenticator is valid for uri in uriString.
     Mozilla and IE (up to 6.0) differ here: Mozilla sends the base URI without parameters,
     where IE sends the URI with all parameters.

     So we declare any uri that starts with the authenticated URI as valid"
    
    ^ uriString isString and:[uriString startsWith:uri].
! !

!DigestAuthenticator methodsFor:'testing'!

isDigest
    ^ true

    "Created: / 25-11-2011 / 13:44:07 / sr"
! !

!DigestAuthenticator::DigestAuthenticationData class methodsFor:'queries'!

mechanismName
    "answer the name of the mechanism as known in the protocols"

    ^ #Digest
! !

!DigestAuthenticator::DigestAuthenticationData methodsFor:'accessing'!

user:user secret:passwordString
    "encode the secret for user and store it.
     We generate the H(A1) value used by the DigestAuthenticator"

    |userString|

    user isString ifTrue:[
        userString := user.
    ] ifFalse:[
        userString := user name , ':' , user realm.
    ].

    encodedSecret := (MD5Stream hashValueOf:(userString, ':' , passwordString)) 
                        hexPrintString asLowercase.
! !

!DigestAuthenticator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__DigestAuthenticator.st,v 1.12 2011/11/25 13:33:43 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/authentication/Authentication__DigestAuthenticator.st,v 1.12 2011/11/25 13:33:43 sr Exp $'
! !
