"{ Package: 'stx:goodies/benchmarks/deltaBlue' }"

"{ NameSpace: Benchmarks }"

Object subclass:#Planner
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-DeltaBlue'
!

Planner class instanceVariableNames:'currentMark'

"
 No other class instance variables are inherited by this class.
"
!

Planner comment:'I embody the DeltaBlue algorithm described in:
	"The DeltaBlue Algorithm: An Incremental Constraint Hierarchy Solver"
	by Bjorn N. Freeman-Benson and John Maloney
See January 1990 Communications of the ACM or University of Washington TR 89-08-06 for further details.'
!

!Planner class methodsFor:'documentation'!

documentation
"
    This package was provided by Urs Hoelzle (thanks),
    formerly member of the Self language team.

    [author:]
        Self language group (?)

    [start with:]
        10 timesRepeat:[Planner ursDBTest: 10 Count: 100]
"
!

results
"
    Run the benchmarks a few times, to avoid measuring
    autoload times, dynamic compilation times and cache effects.

    Run the benchmark by:
        10 timesRepeat:[Planner ursDBTest: 10 Count: 100]
    on fast machines, use:
        10 timesRepeat:[Planner ursDBTest: 10 Count: 1000]
    and divide the number by 10.

    Notice:
        the numbers below could be outdated 
        - we dont update them for every change.
        Also, the BEST time from a number of runs on an unloaded machine
        was taken.
        Do not expect to be able to reproduce them exactly - a variation of
        5-15% is normal, depending on network traffic, memory speed,
        cache fluctuations etc. 
        (especially on PC hardware, the memory bus timing can make a difference)


  bytecode + dynamic compilation:
    DUO 2.1Ghz Vista                     4.6
    DUO 1.8Ghz Vista                     7.8
    P-M 1.7Ghz WinXP                     8
    Athlon64_32 1.8Ghz lx/gcc3.3.5      13              (4)
    Duron 1.2Ghz winXP                  20 
    Athlon 1Ghz lx/egcs                 24 
    P5-celeron/500 lx/egcs              32              (3)
    P6/400 (linux/elf )                 52 40
    Celeron/433 winXP                   65
    21164/433 (osf1)                    78 59      
    P6/266 (linux/elf pci)              69
    Ultra250 296Mhz sol2.6/gcc          91
    P5/200 (linux/elf pci)             115
    P5/200 (linux/elf notebook)        132 149 137?
    P5/200 (win95 notebook)            165 155          (2) (S)
    P5/150 (linux/elf)                 202              (2)
    P5/133 (linux/a.out)               212
    r4000/150 (sgi irix5.3)            292 221 284
    P5/120 (linux/elf)                 373              (2)
    SS10/40 (solaris 2.5)              593   
    21064/233 (osf1)                   727              (1)
    486/66 (unixware)                 1030
    486/50 (linux/elf)                1060     
    68k/33 (NeXTStation mono)         1705
    320H                              1870
    ELC (sunos 4.1)                   2226      

  bytecode interpreter (no just-in-time compilation):
    hp B2000(785) (hpux10.20;cc)       236
    P5/133 (linux/a.out)               913
    hp 735/100 (hpux10.20 newcc)       989
    hp 735/100 (hpux10.20;gcc)        1066
    hp 735/100 (hpux10.20)            1379
    88k (realIX)                      2219
    21064/233 (osf1)                  2762
    r4000/100 (sgi irix5.2)           2903
    68k/33 (NeXTStation mono)         6900
    88k
    320H (aix3.2.5; gcc)              7350
    320H (aix3.2.5)                  10506

  vw2.5.1
    SS10/40 (solaris 2.5)              436

  Notice: those numbers may be outdated by the time you read this.

  (1) running 64bit mode with zero L2 cache.
  (2) a notebook with a very poor memory interface
  (3) a more modern notebook (128k cache)
  (4) compiled with -m32 (i.e. 32bit mode)
  (S): no stack overflow checks
"
! !

!Planner class methodsFor:'add/remove'!

incrementalAdd: c
	"Attempt to satisfy the given constraint and, if successful, incrementally update the dataflow graph."
	"Details: If satifying the constraint is successful, it may override a weaker constraint on its output. The algorithm attempts to resatisfy that constraint using some other method. This process is repeated until either a) it reaches a variable that was not previously determined by any constraint or b) it reaches a constraint that is too weak to be satisfied using any of its methods. The variables of constraints that have been processed are marked with a unique mark value so that we know where we've been. This allows the algorithm to avoid getting into an infinite loop even if the constraint graph has an inadvertent cycle."

	| mark overridden |
	mark _ self newMark.
	overridden _ c satisfy: mark.
	[overridden == nil] whileFalse:
		[overridden _ overridden satisfy: mark].
!

incrementalRemove: c
	"Entry point for retracting a constraint. Remove the given constraint and incrementally update the dataflow graph."
	"Details: Retracting the given constraint may allow some currently unsatisfiable downstream constraint be satisfied. We thus collect a list of unsatisfied downstream constraints and attempt to satisfy each one in turn. This list is sorted by constraint strength, strongest first, as a heuristic for avoiding unnecessarily adding and then overriding weak constraints."
	"Assume: c is satisfied."

	| out unsatisfied |
	out _ c output.
	c markUnsatisfied.
	c removeFromGraph.
	unsatisfied _ self removePropagateFrom: out.
	unsatisfied do: [: u | self incrementalAdd: u].
! !

!Planner class methodsFor:'benchmarks'!

chainTest: n
	"Do chain-of-equality-constraints performance tests, printing the results in the Transcript."
	"Planner chainTest: 100"

	| equalsC vars constraints v1 v2 eqC editConstraint plan |
	"This constraint is slower than the special-purpose EqualityConstraint."
	equalsC _ Constraint
		names: #(a b)
		methods: #('a _ b'   'b _ a').

	self report: 'Built chain of ', n printString, ' equality constraints in' times: 1 run:
		[vars _ (0 to: n) collect: [: i | DBVariable new].
		 constraints _ OrderedCollection new: n.
		 "thread a chain of equality constraints through the variables"
		 1 to: n do:
			[: i |
			 v1 _ (vars at: i).
			 v2 _ (vars at: i + 1).
			 eqC _ v1 requireEquals: v2 "equalsC copy var: v1 var: v2 strength: #required".
			 constraints add: eqC].
		 vars last strongDefaultStay].

	self report: 'Add constraint (case 1):' times: 1 run:
		[editConstraint _ EditConstraint var: (vars first) strength: #preferred].
	self report: 'Make Plan (case 1)' times: 1 run:
		[plan _ Planner extractPlanFromConstraints: (Array with: editConstraint)].
	Transcript show: 'Plan length: ', plan size printString, ' constraints'; cr.
	self report: 'Execute Plan (case 1):' times: 20 run:
		[plan execute].
	self report: 'Remove constraint (case 1):' times: 1 run:
		[editConstraint destroyConstraint].
	self report: 'Setting first node: (case 1a):' times: 1 run:
		[(vars first) setValue: 123 strength: #strongPreferred].
	self report: 'Setting last node: (case 1b):' times: 1 run:
		[(vars last) setValue: 321 strength: #strongPreferred].
	Transcript cr.

	self report: 'Add constraint (case 2):' times: 1 run:
		[editConstraint _ EditConstraint var: (vars first) strength: #default].
	self report: 'Make Plan (case 2):' times: 1 run:
		[plan _ Planner extractPlanFromConstraints: (Array with: editConstraint)].
	Transcript show: 'Plan length: ', plan size printString, ' constraints'; cr.
	self report: 'Execute Plan (case 2):' times: 20 run:
		[plan execute].
	self report: 'Remove constraint (case 2):' times: 1 run:
		[editConstraint destroyConstraint].
	self report: 'Setting first node: (case 2a):' times: 1 run:
		[(vars first) setValue: 1230 strength: #weakDefault].
	self report: 'Setting last node: (case 2b):' times: 1 run:
		[(vars last) setValue: 3210 strength: #weakDefault].
	Transcript cr.

	constraints do: [: c | c release].
	vars do: [: v | v release].
	Transcript cr.
!

projectionTest: n
"This test constructs a two sets of variables related to each other by a simple linear transformation (scale and offset). The time is measured to change a variable on either side of the mapping and to change the scale and offset factors. Results are printed in the Transcript."
	"Planner projectionTest: 100"

	| projectionC constraints scale offset src dst |
	"This constraint is slower than the special-purpose ScaleConstraint."
	projectionC _ Constraint
		names: #(src scale offset dst)
		methods: #('dst _ (src * scale) + offset'   'src _ (dst - offset) // scale').
	constraints _ OrderedCollection new.

	self report: 'Projection test for ', n printString, ' points (Smalltalk):' times: 1 run:
		[scale _ DBVariable value: 10.
		 offset _ DBVariable value: 1000.
		 1 to: n do:
			[: i |
			 src _ DBVariable value: i.
			 dst _ DBVariable value: i.
			 constraints add: (src defaultStay).
			 constraints add:
				("(projectionC copy)"ScaleConstraint
					var: src var: scale var: offset var: dst
					strength: #required)]].
	self reportChange: 'Changing a data point:'
		var: src newValue: 17.
	self reportChange: 'Changing a projected point:'
		var: dst newValue: 1050.
	self reportChange: 'Changing the scale factor:'
		var: scale newValue: 5.
	self reportChange: 'Changing the offset:'
		var: offset newValue: 2000.

	constraints do: [: c | c release].
	Transcript cr.
!

report: string times: count run: aBlock
	"Report the time required to execute the given block."

	| time |
	time _ Time millisecondsToRun: [count timesRepeat: aBlock].
	Transcript show: string, ' ', (time // count) printString, ' milliseconds'; cr.
!

reportChange: title var: aVariable newValue: newValue

	| editConstraint plan |
	Transcript show: title; cr.
	self report: '  Adding Constraint:' times: 1 run:
		[editConstraint _ EditConstraint var: aVariable strength: #preferred].
	self report: '  Making Plan' times: 1 run:
		[plan _ Planner extractPlanFromConstraints: (Array with: editConstraint)].
	Transcript show: '  Plan size: ', plan size printString, ' constraints'; cr.
	self report: '  Executing Plan:' times: 10 run:
		[aVariable value: newValue. plan execute].
	self report: '  Removing Constraint:' times: 1 run:
		[editConstraint destroyConstraint].
!

ursDBTest: n Count: count 
        "Do chain-of-equality-constraints performance tests, printing the results in 
        the Transcript."
        "Planner ursDBTest: 10 Count: 10"
        "Planner ursDBTest: 10 Count: 100"
        "TimeProfiler profile:[Planner ursDBTest: 100 Count: 100]"
        "AllocationProfiler profile:[Planner ursDBTest: 100 Count: 100]"

        | vars constraints v1 v2 eqC editConstraint plan msecs |
        vars := (1 to: n)
                                collect: [:i | DBVariable new].
        constraints := OrderedCollection new: n.
        1 to: n - 1 do: 
                [:i | 
                v1 := vars at: i.
                v2 := vars at: i + 1.
                eqC := v1 requireEquals: v2.
                constraints add: eqC].
        vars last defaultStay.
        msecs := Time millisecondsToRun: [count
                                        timesRepeat: 
                                                [editConstraint := EditConstraint var: vars first strength: #strongDefault.
                                                plan := Planner extractPlanFromConstraints: (Array with: editConstraint).
                                                plan execute.
                                                editConstraint destroyConstraint]].
        constraints do: [:c | c release].
        vars do: [:v | v release].
        Transcript show: 'Self-style DeltaBlue Benchmark (Smalltalk-80)'; cr.
        Transcript show: '  (chain length: ' , n printString , ', count: ' , count printString , ', total time: ' , msecs printString , ')'; cr; cr.
        Transcript endEntry

    "Modified: / 2.3.1998 / 16:12:01 / cg"
! !

!Planner class methodsFor:'class initialization'!

initialize
	"Planner initialize"

	currentMark _ 1.
! !

!Planner class methodsFor:'planning/value propagation'!

extractPlanFromConstraints: constraints
	"Extract a plan for resatisfaction starting from the outputs of the given constraints, usually a set of input constraints."

	| sources |
	sources _ OrderedCollection new.
	constraints do:
		[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]].
	^self makePlan: sources
!

extractPlanFromVariables: variables
	"Extract a plan from the dataflow graph having the given variables. It is assumed that the given set of variables is complete, or at least that it contains all the input variables."

	| sources |
	sources _ OrderedCollection new.
	variables do:
		[: v |
		 (v constraints) do:
			[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]]].
	^self makePlan: sources
!

makePlan: sources
	"Extract a plan for resatisfaction starting from the given source constraints, usually a set of input constraints. This method assumes that stay optimization is desired; the plan will contain only constraints whose output variables are not stay. Constraints that do no computation, such as stay and edit constraints, are not included in the plan."
	"Details: The outputs of a constraint are marked when it is added to the plan under construction. A constraint may be appended to the plan when all its input variables are known. A variable is known if either a) the variable is marked (indicating that has been computed by a constraint appearing earlier in the plan), b) the variable is 'stay' (i.e. it is a constant at plan execution time), or c) the variable is not determined by any constraint. The last provision is for past states of history variables, which are not stay but which are also not computed by any constraint."
	"Assume: sources are all satisfied."

	| mark plan todo c |
	mark _ self newMark.
	plan _ Plan new.
	todo _ sources.
	[todo isEmpty] whileFalse:
		[c _ todo removeFirst.
		 ((c output mark ~= mark) and:          "not in plan already and..."
		  [c inputsKnown: mark]) ifTrue:                "eligible for inclusion"
			[plan addLast: c.
			 c output mark: mark.
			 c _ self addConstraintsConsuming: c output to: todo]].
	^plan
!

propagateFrom: v
	"The given variable has changed. Propagate new values downstream."

	| todo c |
	todo _ OrderedCollection new.
	self addConstraintsConsuming: v to: todo.
	[todo isEmpty] whileFalse:
		[c _ todo removeFirst.
		 c execute.
		 self addConstraintsConsuming: c output to: todo].
! !

!Planner class methodsFor:'private'!

addConstraintsConsuming: v to: aCollection

	| determiningC |
	determiningC _ v determinedBy.
	v constraints do:
		[: c |
		 ((c == determiningC) or: [c isSatisfied not]) ifFalse:
			[aCollection add: c]].
!

addPropagate: c mark: mark
	"Recompute the walkabout strengths and stay flags of all variables downstream of the given constraint and recompute the actual values of all variables whose stay flag is true. If a cycle is detected, remove the given constraint and answer false. Otherwise, answer true."
	"Details: Cycles are detected when a marked variable is encountered downstream of the given constraint. The sender is assumed to have marked the inputs of the given constraint with the given mark. Thus, encountering a marked node downstream of the output constraint means that there is a path from the constraint's output to one of its inputs."

	| todo d |
	todo _ OrderedCollection with: c.
	[todo isEmpty] whileFalse:
		[d _ todo removeFirst.
		 (d output mark = mark) ifTrue:
			[self incrementalRemove: c.
			 ^false].
		 d recalculate.
		 self addConstraintsConsuming: d output to: todo].
	^true
!

constraintsConsuming: v do: aBlock

	| determiningC |
	determiningC _ v determinedBy.
	v constraints do:
		[: c |
		 ((c == determiningC) or: [c isSatisfied not]) ifFalse:
			[aBlock value: c]].
!

newMark
	"Select a previously unused mark value."
	"Details: We just keep incrementing. If necessary, the counter will turn into a LargePositiveInteger. In that case, it will be a bit slower to compute the next mark but the algorithms will all behave correctly. We reserve the value '0' to mean 'unmarked'. Thus, this generator starts at '1' and will never produce '0' as a mark value."

	^currentMark _ currentMark + 1
!

removePropagateFrom: out
	"Update the walkabout strengths and stay flags of all variables downstream of the given constraint. Answer a collection of unsatisfied constraints sorted in order of decreasing strength."

	| unsatisfied todo v nextC |
	unsatisfied _ SortedCollection sortBlock:
		[: c1 : c2 | c1 strength stronger: c2 strength].
	out determinedBy: nil.
	out walkStrength: Strength absoluteWeakest.
	out stay: true.
	todo _ OrderedCollection with: out.
	[todo isEmpty] whileFalse:
		[v _ todo removeFirst.
		 v constraints do:
			[: c | (c isSatisfied) ifFalse: [unsatisfied add: c]].
		 self constraintsConsuming: v do:
			[: c |
			 c recalculate.
			 todo add: c output]].
	^unsatisfied
! !

!Planner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/benchmarks/deltaBlue/Planner.st,v 1.84 2011/09/27 15:04:10 cg Exp $'
! !

Planner initialize!
