"{ NameSpace: Benchmarks }"

AbstractConstraint subclass:#UnaryConstraint
	instanceVariableNames:'output satisfied'
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-DeltaBlue'
!

UnaryConstraint comment:'I am an abstract superclass for constraints having a single possible output variable.

Instance variables:
	output          possible output variable <Variable>
	satisfied               true if I am currently satisfied <Boolean>'!


!UnaryConstraint methodsFor:'add/remove'!

addToGraph
	"Add myself to the constraint graph."

	output addConstraint: self.
	satisfied _ false.!

removeFromGraph
	"Remove myself from the constraint graph."

	(output == nil) ifFalse: [output removeConstraint: self].
	satisfied _ false.! !

!UnaryConstraint methodsFor:'initialize-release'!

release

	strength _ nil.
	output _ nil.
	satisfied _ nil.!

var: aVariable strength: strengthSymbol
	"Initialize myself with the given variable and strength."

	strength _ Strength of: strengthSymbol.
	output _ aVariable.
	satisfied _ false.
	self addConstraint.! !

!UnaryConstraint methodsFor:'planning'!

chooseMethod: mark
	"Decide if I can be satisfied and record that decision."

	satisfied _
		(output mark ~= mark) and:
		[strength stronger: output walkStrength].!

execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility!

inputsDo: aBlock
	"I have no input variables."!

markUnsatisfied
	"Record the fact that I am unsatisfied."

	satisfied _ false.!

output
	"Answer my current output variable."

	^output!

recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay', the value for the current output of this constraint. Assume this constraint is satisfied."

	output walkStrength: strength.
	output stay: (self isInput not).
	(output stay) ifTrue: [self execute].   "stay optimization"! !

!UnaryConstraint methodsFor:'queries'!

isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^satisfied! !

!UnaryConstraint class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/benchmarks/deltaBlue/UnaryConstraint.st,v 1.4 1997/01/03 17:39:42 cg Exp $'
! !
