"{ Package: 'stx:goodies/benchmarks/richards' }"

"{ NameSpace: Benchmarks }"

RBObject subclass:#RichardsBenchmarks
	instanceVariableNames:'taskList currentTask currentTaskIdentity taskTable tracing layout
		queuePacketCount holdCount'
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-RichardsBenchmarks'
!

!RichardsBenchmarks class methodsFor:'documentation'!

documentation
"
    The richardsBenchmark, provided by Urs Hoelzle.

    Run the benchmarks a few times, to avoid measuring
    autoload and/or dynamic compilation times.

    [author:]
        Self language group (?)

    [start with:]
        10 timesRepeat:[RichardsBenchmarks start]
      (or, if you want to start it out of another namespace:)
        Benchmarks::RichardsBenchmarks start
"
!

results
"
    Run the benchmarks a few times, to avoid measuring
    autoload times, dynamic compilation times and cache effects.

        10 timesRepeat:[RichardsBenchmarks start]

    Notice:
        the numbers below could be outdated 
        - we dont update them for every change.
        Also, the BEST time from a number of runs on an unloaded machine
        was taken.
        Do not expect to be able to reproduce them exactly - a variation of
        5-15% is normal, depending on network traffic, memory speed,
        cache fluctuations etc. 
        (especially on PC hardware, the memory bus timing can make a difference)


    results (bytecode + dynamic compiler):
        DUO 1.8Ghz vista            Scheduler time =    15 Total time =    15 
        P-M 1.7Ghz winXP            Scheduler time =    20 Total time =    20 
        Duron 1.2Ghz winXP          Scheduler time =    40 Total time =    40 
        Athlon 1Ghz lx/egcs         Scheduler time =    55 Total time =    55 
        P6/700 lx/egcs              Scheduler time =    74 Total time =    75 
        P5-celeron/500 lx/egcs      Scheduler time =   106 Total time =   116 (3)
        P6/400 linux/egcs/elf       Scheduler time =   118 Total time =   119
        Celeron/433 winXP           Scheduler time =   121 Total time =   121
        P6/266 linux/gcc/elf        Scheduler time =   170 Total time =   171
        Ultra250 296Mhz sol2.6/gcc  Scheduler time =   201 Total time =   202
        21164/433 (osf1)            Scheduler time =   221 Total time =   222 
        P5/200 win95/bcc            Scheduler time =   274 Total time =   274 (S)
        P5/200 linux/gcc/elf        Scheduler time =   295 Total time =   298
        P5/150 linux/gcc/elf        Scheduler time =   437 Total time =   439
        P5/133 linux/gcc/a.out      Scheduler time =   471 Total time =   474
        R4000/150 IRIX5.3/gcc       Scheduler time =   636 Total time =   640
        P5/120 linux/gcc/elf        Scheduler time =   796 Total time =   801 (2)
        SS10/40 sol2.5/gcc          Scheduler time =  1122 Total time =  1130
        21064/233 (osf1)            Scheduler time =  1648 Total time =  1654 (1)
        486/66 unixware             Scheduler time =  2890 Total time =  2990
        486/50 linux/gcc/elf        Scheduler time =  3435 Total time =  3448
        68k/33 (NextStation)        Scheduler time =  5203 Total time =  5218
        320H (aix3.2/cc)            Scheduler time =  5439 Total time =  5453
        ELC (sunos/gcc)             Scheduler time =  6639 Total time =  6649 (*)
        ELC (sunos/cc)              Scheduler time =  7364 Total time =  7378 (*)

    bytecode interpreter (no just-in-time compilation):
        P-M 1.7Ghz winXP                 Scheduler time =   240 Total time =   240 
        hp B2000(785) (hpux10.20/new cc) Scheduler time =  1984 Total time =  1986
        P5/133 linux/gcc/a.out           Scheduler time =  6637 Total time =  6641
        hp735/100 (hpux10.20/newcc)      Scheduler time =  7266 Total time =  7272
        hp735/100 (hpux10.20/gcc)        Scheduler time =  8272 Total time =  8277
        hp735/100 (hpux10.20)            Scheduler time = 10970 Total time = 10976
        88k (realIX/gcc)                 Scheduler time = 12740 Total time = 12750
        21064/233 (osf1)                 Scheduler time = 18551 Total time = 18561 (1)
        R4000/100 (IRIX/cc)              Scheduler time = 22794 Total time = 22807
        68k/33 (NeXTStation mono)        Scheduler time = 57693 Total time = 57718
        320H (aix3.2/cc)                 Scheduler time = 80722 Total time = 93508

    parcplace vw2.5.1
        SS10/40 sol2.5                   Scheduler time =  1430 Total time =  1616

    Notice: those numbers may be outdated my the time your read this.
    (1): the tested machine runs 64bit mode.
    (2): a notebook with a very poor memory interface
    (3): a more modern notebook (128k cache)
    (*): definitely outdated - tested on an old version of the JIT compiler.
    (S): no stack overflow checks
"
! !

!RichardsBenchmarks class methodsFor:'instance creation'!

start
	super new start
! !

!RichardsBenchmarks methodsFor:'creation'!

createDevice: identity priority: priority work: work state: state 
        | data functionWork |
        data := DeviceTaskDataRecord create.
        self
                createTask: identity
                priority: priority
                work: work
                state: state
                function: 
                        [:work :word | 
                        data := word.
                        functionWork := work.
                        NoWork == functionWork
                                ifTrue:
                                        [NoWork == (functionWork := data pending)
                                                ifTrue: [self wait]
                                                ifFalse: 
                                                        [data pending: NoWork.
                                                        self queuePacket: functionWork]]
                                ifFalse: 
                                        [data pending: functionWork.
                                        tracing
                                                ifTrue: 
                                                        [self trace: functionWork datum].
                                        self holdSelf]]
                data: data

    "Modified: / 13.3.1998 / 11:44:28 / cg"
!

createHandler: identity priority: priority work: work state: state 
        | data workPacket count devicePacket |
        data := HandlerTaskDataRecord create.
        self
                createTask: identity
                priority: priority
                work: work
                state: state
                function: 
                        [:work :word | 
                        data := word.
                        NoWork == work
                                ifFalse: [WorkPacketKind == work kind 
                                        ifTrue: [data workInAdd: work]
                                        ifFalse: [data deviceInAdd: work]].
                        NoWork == (workPacket := data workIn)
                                ifTrue: [self wait]
                                ifFalse: 
                                        [count := workPacket datum.
                                        count > 4
                                                ifTrue: 
                                                        [data workIn: workPacket link.
                                                        self queuePacket: workPacket]
                                                ifFalse:
                                                        [NoWork == (devicePacket := data deviceIn)
                                                                ifTrue: [self wait]
                                                                ifFalse: 
                                                                        [data deviceIn: devicePacket link.
                                                                        devicePacket datum: (workPacket data at: count).
                                                                        workPacket datum: count + 1.
                                                                        self queuePacket: devicePacket]]]]
                data: data

    "Modified: / 13.3.1998 / 11:44:34 / cg"
!

createIdler: identity priority: priority work: work state: state 
        | data |
        data := IdleTaskDataRecord create.
        self
                createTask: identity
                priority: priority
                work: work
                state: state
                function: 
                        [:work :word | 
                        data := word.
                        data count: data count - 1.
                        0 = data count
                                ifTrue: [self holdSelf]
                                ifFalse:
                                        [0 = (data control bitAnd: 1)
                                                ifTrue: 
                                                        [data control: data control // 2.
                                                        self release: DeviceA]
                                                ifFalse: 
                                                        [data control: (data control // 2 bitXor: 53256).
                                                        self release: DeviceB]]]
                data: data

    "Modified: / 13.3.1998 / 11:44:39 / cg"
!

createPacket: link identity: identity kind: kind 
	^Packet
		create: link
		identity: identity
		kind: kind
!

createTask: identity priority: priority work: work state: state function: aBlock data: data 
        | t |
        t := TaskControlBlock
                                link: taskList
                                create: identity
                                priority: priority
                                initialWorkQueue: work
                                initialState: state
                                function: aBlock
                                privateData: data.
        taskList := t.
        taskTable at: identity put: t

    "Modified: / 13.3.1998 / 11:44:42 / cg"
!

createWorker: identity priority: priority work: work state: state 
        | data |
        data := WorkerTaskDataRecord create.
        self
                createTask: identity
                priority: priority
                work: work
                state: state
                function: 
                        [:work :word | 
                        data := word.
                        NoWork == work
                                ifTrue: [self wait]
                                ifFalse: 
                                        [data destination: (HandlerA = data destination
                                                        ifTrue: [HandlerB]
                                                        ifFalse: [HandlerA]).
                                        work identity: data destination.
                                        work datum: 1.
                                        1 to: 4 do: 
                                                [:i | 
                                                data count: data count + 1.
                                                data count > 26 ifTrue: [data count: 1].
                                                work data at: i put: $A asInteger + data count - 1].
                                        self queuePacket: work]]
                data: data

    "Modified: / 13.3.1998 / 11:44:48 / cg"
! !

!RichardsBenchmarks methodsFor:'initialize'!

start
        | workQ mark1 mark2 mark3 mark4 |
        self initTrace; initScheduler.
        mark1 := Time millisecondClockValue.
        Transcript show: 'Bench mark starting'; cr.
        self
                createIdler: Idler
                priority: 0
                work: NoWork
                state: TaskState running.
        workQ := self
                                createPacket: NoWork
                                identity: Worker
                                kind: WorkPacketKind.
        workQ := self
                                createPacket: workQ
                                identity: Worker
                                kind: WorkPacketKind.
        self
                createWorker: Worker
                priority: 1000
                work: workQ
                state: TaskState waitingWithPacket.
        workQ := self
                                createPacket: NoWork
                                identity: DeviceA
                                kind: DevicePacketKind.
        workQ := self
                                createPacket: workQ
                                identity: DeviceA
                                kind: DevicePacketKind.
        workQ := self
                                createPacket: workQ
                                identity: DeviceA
                                kind: DevicePacketKind.
        self
                createHandler: HandlerA
                priority: 2000
                work: workQ
                state: TaskState waitingWithPacket.
        workQ := self
                                createPacket: NoWork
                                identity: DeviceB
                                kind: DevicePacketKind.
        workQ := self
                                createPacket: workQ
                                identity: DeviceB
                                kind: DevicePacketKind.
        workQ := self
                                createPacket: workQ
                                identity: DeviceB
                                kind: DevicePacketKind.
        self
                createHandler: HandlerB
                priority: 3000
                work: workQ
                state: TaskState waitingWithPacket.
        self
                createDevice: DeviceA
                priority: 4000
                work: NoWork
                state: TaskState waiting.
        self
                createDevice: DeviceB
                priority: 5000
                work: NoWork
                state: TaskState waiting.
        Transcript show: 'Starting'; cr.
        mark2 := Time millisecondClockValue.
        self schedule.
        mark3 := Time millisecondClockValue.
        Transcript show: 'Finished'; cr.
        Transcript show: 'QueuePacket count = ' , queuePacketCount printString , ' HoldCount = ' , holdCount printString; cr; show: 'These results are ' , (queuePacketCount = 23246 & (holdCount = 9297)
                                ifTrue: ['correct']
                                ifFalse: ['wrong']) printString.
        Transcript cr; show: 'End of run'; cr.
        mark4 := Time millisecondClockValue.
        Transcript show: '*****Scheduler time = ' , (mark3 - mark2) printString , ' Total time = ' , (mark4 - mark1) printString; cr

    "Modified: / 13.3.1998 / 11:45:28 / cg"
! !

!RichardsBenchmarks methodsFor:'private'!

findTask: identity 
        | t |
        t := taskTable at: identity.
        NoTask == t ifTrue: [self error: 'findTask failed'].
        ^t

    "Modified: / 13.3.1998 / 11:44:51 / cg"
!

holdSelf
        holdCount := holdCount + 1.
        currentTask taskHolding: true.
        ^currentTask link

    "Modified: / 13.3.1998 / 11:44:54 / cg"
!

initScheduler
        queuePacketCount := holdCount := 0.
        taskTable := Array new: 6 withAll: NoTask.
        taskList := NoTask

    "Modified: / 13.3.1998 / 11:50:39 / cg"
!

initTrace
        "    BinaryChoice 
        message: 'Trace?' 
        displayAt: Sensor mousePoint 
        centered: true 
        ifTrue: [tracing := true] 
        ifFalse: [tracing := false].
        "
        "does not work in V. 4"

        tracing := false.
        layout := 0

    "Modified: / 13.3.1998 / 11:45:00 / cg"
!

queuePacket: packet 
        | t |
        t := self findTask: packet identity.
        NoTask == t ifTrue: [^NoTask].
        queuePacketCount := queuePacketCount + 1.
        packet link: NoWork.
        packet identity: currentTaskIdentity.
        ^t addInput: packet checkPriority: currentTask

    "Modified: / 13.3.1998 / 11:45:07 / cg"
!

release: identity 
        | t |
        t := self findTask: identity.
        NoTask == t ifTrue: [^NoTask].
        t taskHolding: false.
        t priority > currentTask priority
                ifTrue: [^t]
                ifFalse: [^currentTask]

    "Modified: / 13.3.1998 / 11:45:10 / cg"
!

trace: id 
        layout := layout - 1.
        0 >= layout
                ifTrue: 
                        [Transcript cr.
                        layout := 50].
        Transcript show: id printString

    "Modified: / 13.3.1998 / 11:45:32 / cg"
!

wait 
	currentTask taskWaiting: true.
	^currentTask
! !

!RichardsBenchmarks methodsFor:'scheduling'!

schedule
        currentTask := taskList. 
        [NoTask == currentTask]
                whileFalse: 
                        [currentTask isTaskHoldingOrWaiting
                                ifTrue: [currentTask := currentTask link]
                                ifFalse: 
                                        [currentTaskIdentity := currentTask identity.
                                        tracing ifTrue: [self trace: currentTaskIdentity].
                                        currentTask := currentTask runTask]]

    "Modified: / 13.3.1998 / 11:45:16 / cg"
! !

!RichardsBenchmarks class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/benchmarks/richards/RichardsBenchmarks.st,v 1.105 2007/02/10 16:32:56 cg Exp $'
! !
