"{ Package: 'stx:goodies/benchmarks/self' }"

"{ NameSpace: Benchmarks }"

AbstractBenchmark subclass:#TakBenchmark
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-Self'
!

!TakBenchmark class methodsFor:'documentation'!

documentation
"
   this is a recursion test;
   Except as noted, the following execution times were extracted from
   ``Performance and Evaluation of Lisp Systems'', by Richard P Gabriel, MIT Press

   System               Language                 Time (ms)      Notes

   VAX  11/750          Franz Lisp (slowest)     47600          generic arithmetic
   VAX  11/750          Franz Lisp                3600          fixnum arithmetic
   Symbolics LM2        Lisp                      2905
   VAX  11/750          C                         2400
   68k                  C                         1900          unknown clock; probably 8Mhz
   VAX  11/750          Franz Lisp (best)         1900          fixnum arithmetic (lfc)
   VAX  11/780          C                         1350
   VAX  11/780          Franz Lisp                1100          fixnum arithmetic (lfc)
   68k                  Assembler                  700          unknown clock; probably 8Mhz
   Dorado               Interlisp                  526                
   LMI Lambda           microcompiled Lisp         450             
   Symbolics 3600       Lisp                       430
   i586/100             ST/X interpreted           376          (*)
   SS10/40              ST/X interpreted           174          just in time compiled
   i586/133             ST/X interpreted            98          just in time compiled
   i586/100             ST/X compiled               55          (*)
   Cray-1               PSL                         44
   i586/100             ST/X jitted                 31          (*)
   i586/100             ST/X compiled (-O)          30          (*)
   i586/133             ST/X compiled (-O)          25          (*)
   i586/100             C (gcc2.6.3)                15.5        (*)
   i586/100             C (gcc2.6.3 -O)             12          (*)
   P3/700               ST/X jitted                 4.6         (*)

   (*) my measurement
"
! !

!TakBenchmark methodsFor:'constants'!

cTimes: mach 
        ^nil


!

fixedBase
       ^1


! !

!TakBenchmark methodsFor:'misc'!

blockTakX: x Y: y Z: z
    "uses block evaluation (which does not require a message send"

    "slower (in this particular implementation, because value/3 is not
     inlined by the jitter (as opposed to value/0, value/1 and value/2)"

    | tak |

    tak := [:x :y :z |
        y < x 
            ifFalse: [z]
            ifTrue: [
                tak
                    value: (tak
                                    value: x - 1
                                    value: y
                                    value: z)
                    value: (tak
                                    value: y - 1
                                    value: z
                                    value: x)
                    value: (tak
                                    value: z - 1
                                    value: x
                                    value: y)]].

    ^ tak value:x value:y value:z
!

run
        "TakBenchmark new run"
        "Time millisecondsToRun:[TakBenchmark new run]"
        "Transcript showCR:(Time millisecondsToRun:[TakBenchmark new run])"
        "10 timesRepeat:[
            Transcript showCR:(Time millisecondsToRun:[TakBenchmark new run])
         ]
        "
        "Time millisecondsToRun:[
                100 timesRepeat:[ TakBenchmark new takX:18 Y:12 Z:6] ]"
        "Time millisecondsToRun:[TakBenchmark new takX:18.0 Y:12.0 Z:6.0]"
        "MessageTally spyOn:[100 timesRepeat:[TakBenchmark new run]]"

        | r |
        r := self takX: 18 Y: 12 Z: 6.
        r ~= 7 ifTrue: [ self error: 'tak results are incorrect' ].
!

run2
        "TakBenchmark new run2"
        "Time millisecondsToRun:[TakBenchmark new run2]"
        "Transcript showCR:(Time millisecondsToRun:[TakBenchmark new run2])"
        "10 timesRepeat:[
            Transcript showCR:(Time millisecondsToRun:[TakBenchmark new run2])
         ]
        "
        "Time millisecondsToRun:[
                100 timesRepeat:[ TakBenchmark new blockTakX:18 Y:12 Z:6] ]"
        "Time millisecondsToRun:[TakBenchmark new blockTakX:18.0 Y:12.0 Z:6.0]"
        "MessageTally spyOn:[100 timesRepeat:[TakBenchmark new run2]]"

        | r |
        r := self blockTakX: 18 Y: 12 Z: 6.
        r ~= 7 ifTrue: [ self error: 'tak results are incorrect' ].
!

takX: x Y: y Z: z 
        y < x ifFalse: [^z]
                ifTrue: [^self
                                takX: (self
                                                takX: x - 1
                                                Y: y
                                                Z: z)
                                Y: (self
                                                takX: y - 1
                                                Y: z
                                                Z: x)
                                Z: (self
                                                takX: z - 1
                                                Y: x
                                                Z: y)]
! !

!TakBenchmark class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/benchmarks/self/TakBenchmark.st,v 1.10 2005/03/17 12:44:49 penk Exp $'
! !
