"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/business' }"

Object subclass:#CreditCardValidator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Utilities-Business'
!

!CreditCardValidator class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    A credit card number validator.

    ***** NOTICE & WARNING ****
    This only verifies for a correct number format -
    it does NOT (I repeat: NOT) validate the number for being
    valid w.r.t. the expiration date or if such an account exists
    at all - i.e. it does not ask the institute about that number.

    No warranty: the algorithm below was extracted from Public domain
    information (Harrell W. Stiles) - and is not proven to be correct.
    We do not recommend using that code for real, critical business 
    application. We are not responsible for loss of money ...

    For info, read:
	http://www.beachnet.com/~hstiles/cardtype.html:

    which is:

    Credit Card Validation - Check Digits 

    This document outlines procedures and algorithms for Verifying the accuracy and validity of
    credit card numbers. Most credit card numbers are encoded with a 'Check Digit'. A check digit is
    a digit added to a number (either at the end or the beginning) that validates the authenticity of the
    number. A simple algorithm is applied to the other digits of the number which yields the check
    digit. By running the algorithm, and comparing the check digit you get from the algorithm with
    the check digit encoded with the credit card number, you can verify that you have correctly read all
    of the digits and that they make a valid combination. 

    Possible uses for this information: 

	 When a user has keyed in a credit card number (or scanned it) and you want to validate it
	 before sending it our for debit authorization. 
	 When issuing cards, say an affinity card, you might want to add a check digit using the
	 MOD 10 method. 

    1.Prefix, Length, and Check Digit Criteria 

    Here is a table outlining the major credit cards that you might want to validate. 

     CARD TYPE       Prefix    Length   Check digit
					 algorithm 

     MASTERCARD      51-55     16       mod 10

     VISA              4       13, 16   mod 10

     AMEX             34       15       mod 10

     Diners Club/
     Carte Blanche   300-305
		      36
		      38       14       mod 10

     Discover        6011      16       mod 10

     enRoute         2014
		     2149      15       any

     JCB              3        16       mod 10
     JCB             2131
		     1800      15       mod 10


    2. LUHN Formula (Mod 10) for Validation of Primary
    Account Number 

    The following steps are required to validate the primary account number:

    Step 1: Double the value of alternate digits of the primary account number beginning with the
    second digit from the right (the first right--hand digit is the check digit.) 

    Step 2: Add the individual digits comprising the products obtained in Step 1 to each of the
    unaffected digits in the original number. 

    Step 3: The total obtained in Step 2 must be a number ending in zero (30, 40, 50, etc.) for the
    account number to be validated. 

    For example, to validate the primary account number 49927398716: 

    Step 1: 

	    4 9 9 2 7 3 9 8 7 1 6
	     x2  x2  x2  x2  x2 
    ------------------------------
	     18   4   6  16   2





    Step 2: 4 +(1+8)+ 9 + (4) + 7 + (6) + 9 +(1+6) + 7 + (2) + 6 

    Step 3: Sum = 70 : Card number is validated 

    Note: Card is valid because the 70/10 yields no remainder. 

    The great folks at ICVERIFY are the original source of this data, I only formatted it in HTML. 

    If you are in the market, I wrote a set of FoxPro modules for Windows/Dos that interface nicely
    with ICVERIFY in a multi-user LAN setup. You just set up ICVERIFY on a single station, and all
    stations on the LAN can authorize credit cards with a single FOXBASE function call. Of course,
    you have to license ICVERIFY by the node, but it is very reasonable. I also wrote a couple of
    simple functions to perform pre-authorization, card screening, etc. 

    Here is a Microsoft Excel worksheet that will validate a number for you (useful for understanding
    the algorithm, it is in a .ZIP compressed format) 

    Eliot Jackson has written a set of VB 4 routines here. 

    Horace Vallas made a NeoWebScript (Tcl really) procedure that implements it.
    Check it out at https://enterprise.neosoft.com/secureforms/hav/ 

    Because I get at least a letter a week regarding this routine, here are some additional helpful notes: 

    Make sure that you: 

       1.have started with the rightmost digit (including the check digit) (figure odd and even based
	 upon the rightmost digit being odd, regardless of the length of the Credit Card.) ALWAYS
	 work right to left. 
       2.the check digit counts as digit #1 (assuming that the rightmost digit is the check digit) and is
	 not doubled 
       3.double every second digit (starting with digit # 2 from the right) 
       4.remember that when you double a number over 4, (6 for example) you don't add the result to
	 your total, but rather the sum of the digits of the result (in the above example 6*2=12 so you
	 would add 1+2 to your total (not 12). 
       5.always include the Visa or M/C/ prefix. 

			   E-Mail me at: hstiles@beachnet.com 
			 Also, visit my home page or my POS page.

				 Last revised: 2/25/97
"
! !

!CreditCardValidator class methodsFor:'validation'!

verifyCreditCardNumber:numberStringIn institute:cardTypeString
    "returns true if number is a valid number, false if not.
     ***** NOTICE & WARNING ****
     This only verifies for a correct number format -
     it does NOT (I repeat: NOT) validate the number for being
     valid w.r.t. the expiration date or if such an account exists
     at all - i.e. it does not ask the institute about that number.

     No warranty: the algorithm below was extracted from Public domain
     information (Harrell W. Stiles) - and is not proven to be correct.
     We do not recommend using that code for real, critical business 
     application. We are not responsible for loss of money ...

     cardTypeString: one of 
	'MasterCard'        'MC'
	'VISA'
	'AmericanExpress'   'AMEX'
	'Discover'          'DISC'
	'DinersClub'        'DC'
	'CarteBlanche'      'CB'
	'enRoute'
	'JCB'
    "

    |total tmp number first second third firstTwo firstFour|

    "/ make sure there are only numbers in the string...
    number := numberStringIn select:[:ch | ch isDigit].

    (number size < 13) ifTrue:[
	"/ too short for anything
	^ false
    ].

    first  := number copyFrom:1 to:1.
    second := number copyFrom:2 to:2.
    third  := number copyFrom:3 to:3.
    firstTwo  := number copyFrom:1 to:2.
    firstFour := number copyFrom:1 to:4.

    "/ first, check the credit card suffix and length vs. the type

    ((cardTypeString = 'MC')
    or:[cardTypeString = 'MasterCard']) ifTrue:[
	"/ Master Card prefix must be 51-55
	(first ~= '5' 
	or:[second < '1'
	or:[second > '5']]) ifTrue:[
	     ^ false.   "/ invalid Mastercard prefix
	].
	"Number of digits must be 16 for Master Card"
	(number size ~~ 16) ifTrue:[
	     ^ false    "/ bad length
	].
    ].

    (cardTypeString = 'VISA') ifTrue:[
	"/ Visa prefix must be 4
	(first ~= '4') ifTrue:[ 
	     ^ false.   "/ invalid Visa prefix
	].
	"Number of digits must be 13 or 16 for Visa"
	(number size ~~ 13 and:[number size ~~ 16]) ifTrue:[
	     ^ false    "/ bad length
	].
    ].

    (cardTypeString = 'AMEX'
    or:[cardTypeString = 'AmericanExpress']) ifTrue:[
	"/ American Express prefix must be 34 or 37
	(firstTwo ~= '34' and:[firstTwo ~= '37']) ifTrue:[ 
	     ^ false.   "/ invalid American Express Prefix
	].
	"Number of digits must be 15 for American Express"
	(number size ~~ 15) ifTrue:[
	     ^ false    "/ bad length
	].
    ].

    (cardTypeString = 'DISC'
    or:[cardTypeString = 'Discover']) ifTrue:[
	"/ Discover prefix must be 6011
	(firstFour ~= '6011') ifTrue:[ 
	     ^ false.   "/ invalid Discover Prefix
	].
	"Number of digits must be 16 for Discover "
	(number size ~~ 16) ifTrue:[
	     ^ false    "/ bad length
	].
    ].

    (cardTypeString = 'DC'
    or:[cardTypeString = 'DinersClub'
    or:[cardTypeString = 'CB'
    or:[cardTypeString = 'CarteBlanche']]]) ifTrue:[
	"/ Diner's Club/Carte Blanche prefix must be 36, 38, or 300-305
	(firstTwo ~= '36'
	and:[firstTwo ~= '38'
	and:[firstTwo ~= '30'
	     or:[third < '0' or:[third > '5']]]]) ifTrue:[ 
	     ^ false.   "/ invalid DC/CB Prefix
	].
	"Number of digits must be 14 for Diner's Club/Carte Blanche"
	(number size ~~ 14) ifTrue:[
	     ^ false    "/ bad length
	].
    ].

    (cardTypeString = 'enRoute') ifTrue:[
	"/ enRoute prefix must be 2014 or 2149
	(firstFour ~= '2014'
	and:[firstFour ~= '2149']) ifTrue:[ 
	     ^ false.   "/ invalid enRoute Prefix
	].
	"Number of digits must be 15 for enRoute"
	(number size ~~ 15) ifTrue:[
	     ^ false    "/ bad length
	].
	^ true.    "/ no checksum calculation
    ].

    (cardTypeString = 'JCB') ifTrue:[
	"/ JCB prefix must be 3, 2131 or 1800

	(firstFour ~= '2131'
	and:[firstFour ~= '1800'
	and:[first ~= '3']]) ifTrue:[ 
	     ^ false.   "/ invalid JCB Prefix
	].
	"Number of digits must be 16 for JCB with prefix of 3"
	"Number of digits must be 15 for JCB with prefix of 2131 or 1800"
	first = '3' ifTrue:[
	    (number size ~~ 16) ifTrue:[
		^ false    "/ bad length
	    ].
	] ifFalse:[
	    (number size ~~ 15) ifTrue:[
		^ false    "/ bad length
	    ]
	].
	^ true.    "/ no checksum calculation
    ].


    "/ do the check sum
    total := 0.

    number size - 2 to:0 by:-2 do:[:loc |
	total := total + (number at:(loc + 1 + 1)) digitValue.
	tmp := (number at:(loc + 1)) digitValue.
	tmp > 9 ifTrue:[
	    total := total + 1
	].
	total := total + (tmp \\ 10).
    ].
    number size \\ 2 > 0 ifTrue:[
	total := total + (number at:1) digitValue
    ].
    ^ (total \\ 10) == 0

    "
     self verifyCreditCardNumber:'1234567812345678' institute:'MC' 
     self verifyCreditCardNumber:'1234 5678 1234 5678' institute:'MC' 
     - no, I will not give you an example with a valid number here ;-)
    "


! !

!CreditCardValidator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/business/CreditCardValidator.st,v 1.6 2003/11/18 20:43:34 cg Exp $'
! !
