"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/business' }"

Object subclass:#EuropeanIDCardValidator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Utilities-Business'
!

!EuropeanIDCardValidator class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    A personal ID card validator.
    It should work with German ID cards - I dont know if it also works with other
    european countries...

    ***** NOTICE & WARNING ****
    This only verifies for a correct number format -
    it does NOT (I repeat: NOT) validate the id for being valid.

    No warranty: the algorithm below was extracted from Public domain information 
    - and is not proven to be correct.
    We do not recommend using that code for real, critical business application. 
    We are not responsible for loss of money ...
    We are not responsible for any damage resulting from the use of this code;
    treat it as a guideline or initial implementation sceletton.

    For info, read:
        http://vb-tec.de/persausw.htm

    which is:

    Gesetzliche Grundlagen

        * Gesetz ber die Personalausweise
        * Verordnung zur Bestimmung der Muster der Personalausweise der Bundesrepublik Deutschland

    Weitere Hinweise knnen Sie dem Personalausweis- und Passgesetz entnehmen.
    Mit der unten gezeigten Funktion PersonalausweisOK kann die auf einem deutschen Personalausweis 
    notierte Nummer auf innere Korrektheit geprft werden. 
    Natrlich ist es damit nicht mglich zu prfen, ob der Ausweis wirklich existiert...

    Optional knnen das Geburtsdatum, das Ablaufdatum und/oder die Behrdenkennzahl angegeben werden, 
    so dass diese auf Konsistenz geprft werden. Dies ist mglich, weil die Personalausweis-Nummer (im Prinzip) 
    gerade aus den genannten Komponenten besteht (zuzglich Prfziffern). 
    Hier der schematische Aufbau:

    1220000016D<<4509303<0110052<<<<<<7
    BBBB#####*D  GGGGGG* AAAAAA*      *

    Legende:
      B ......... Behrdenkennziffer,   
      # ......... fortlaufende Nummer
      D ......... Land - deutsch
      G ......... Geburtsdatum 
      A ......... Ablaufdatum
      * ..... ... Prfziffern

    Die Prfziffern werden berechnet, indem die erste Ziffer mit 7, 
    die zweite mit 3, die dritte mit 1, die vierte wieder mit 7 usw. 
    multipliziert und dann summiert wird, wobei nur die Einerstelle relevant ist.
    Beispiel

    Nummer = '1220000016D<<4509303<0110052<<<<<<7'

    B   = 1220
    #   = 00001
    *1  = 6
    D
    G   = 450930  (30-Sep-45)
    *2  = 3
    A   = 011005  (1-Jan-05)
    *3  = 2
    *4  = 7

    Code / Quelltext (VB code)

    Public Function PersonalausweisOK( _
        ByVal Nummer As String, _
        Optional ByVal Geburtsdatum As Date = -1, _
        Optional ByVal Ablaufdatum As Date = -1, _
        Optional ByVal Behrdenkennzahl As Long = -1 _
      ) As Boolean
      // Deklarationen:
      Dim i As Long
      Dim Char As String
      Dim NummerPos As Long

      // 0. Vorab-Checks:
      If Len(Nummer) < 25 _
      Then Exit Function // Nummer zu kurz
      If Mid$(Nummer, 11, 1) <> 'D' _
      Then Exit Function 'kein deutscher Personalausweis

      // 1. Nummer normieren/extrahieren:
      For i = 1 To Len(Nummer)
        Char = Mid$(Nummer, i, 1)
        Select Case Char
        Case '<', ' ', 'D'
          // berspringen

        Case '0' To '9'
          // Nummer merken:
          NummerPos = NummerPos + 1
          If NummerPos > 25 Then Exit Function
          Mid$(Nummer, NummerPos, 1) = Char

        Case Else
          // ungltiges Zeichen:
          Exit Function

        End Select
      Next i

      // 2. Behrdenkennzahl/Seriennummer prfen:
      If Behrdenkennzahl <> -1 _
      Then If Left$(Nummer, 4) <> Format$(Behrdenkennzahl, '0000') _
           Then Exit Function
      If Prffziffer(Left$(Nummer, 9)) <> Mid$(Nummer, 10, 1) _
      Then Exit Function

      // 3. Geburtsdatum prfen:
      If Geburtsdatum <> -1 _
      Then If Mid$(Nummer, 11, 6) <> Format$(Geburtsdatum, 'yymmdd') _
           Then Exit Function
      If Prffziffer(Mid$(Nummer, 11, 6)) <> Mid$(Nummer, 17, 1) _
      Then Exit Function

      // 4. Ablaufdatum prfen:
      If Ablaufdatum <> -1 _
      Then If Mid$(Nummer, 18, 6) <> Format$(Ablaufdatum, 'yymmdd') _
           Then Exit Function
      If Prffziffer(Mid$(Nummer, 18, 6)) <> Mid$(Nummer, 24, 1) _
      Then Exit Function

      // 5. Abschluprfung:
      If Prffziffer(Left$(Nummer, 24)) = Right$(Nummer, 1) _
      Then PersonalausweisOK = True
    End Function

    Prffziffer

    Oben wurde ja bereits erklrt, wie die Prfziffern gebildet werden. 
    Der folgende Code setzt genau dies um, allerdings wurde der Algorithmus deutlich 
    optimiert (die Multiplikationen werden nur einmal zum Schlu gemacht):

    Public Function Prffziffer(ByRef Ziffernfolge As String) As String
      Dim i As Long
      Dim n7 As Long
      Dim n3 As Long
      Dim n1 As Long

      // Quersumme' bilden:
      For i = 1 To Len(Ziffernfolge) Step 3
        n7 = n7 + Val(Mid$(Ziffernfolge, i, 1))
        n3 = n3 + Val(Mid$(Ziffernfolge, i + 1, 1))
        n1 = n1 + Val(Mid$(Ziffernfolge, i + 2, 1))
      Next i

      // Nur Einerstelle beachten:
      Prffziffer = CStr((n7 * 7 + n3 * 3 + n1) Mod 10)
    End Function
"
! !

!EuropeanIDCardValidator class methodsFor:'validation'!

constructIDCardNumberForDepartment:amt runningNr:lfdNr country:countryStr birthDate:birthDate expirationDate:expirationDate
    "returns a valid id-card number string for the given parameters (i.e. generates check sums).

     ***** NOTICE & WARNING ****
     No warranty: the algorithm below was extracted from Public domain
     information (see class documentation) - and is not proven to be correct.
     We do not recommend using that code for real, critical business 
     application. 
     We are not responsible for loss of money ..."

    |t1 t2 t3 s genCheckDigit amtStr lfdNrStr birthDateStr expirationDateStr|

    (amtStr := amt) isString ifFalse:[
        amtStr := amt printString leftPaddedTo:4 with:$0.
    ].
    amtStr size == 4 ifFalse:[ self error:'bad department' ].

    (lfdNrStr := lfdNr) isString ifFalse:[
        lfdNrStr := lfdNr printString leftPaddedTo:5 with:$0.
    ].
    lfdNrStr size == 5 ifFalse:[ self error:'bad running number' ].

    countryStr size == 1 ifFalse:[ self error ].

    (birthDateStr := birthDate) isString ifFalse:[
        birthDateStr := birthDate printStringFormat:'%Y%m%d'.
    ].
    birthDateStr size == 6 ifFalse:[ self error:'bad birthDate string' ].

    (expirationDateStr := expirationDate) isString ifFalse:[
        expirationDateStr := expirationDate printStringFormat:'%Y%m%d'.
    ].
    expirationDateStr size == 6 ifFalse:[ self error:'bad expirationDate string' ].

    genCheckDigit := [:str |
                        |sum factor|

                        sum := 0.
                        factor := 7.
                        str do:[:char |
                            sum := sum + (char digitValue * factor).
                            factor := #(3 1 7) at:( #(7 3 1) indexOf:factor).
                        ].
                        (Character digitValue:(sum \\ 10)) asString.
                     ].

    t1 := amtStr , lfdNrStr.
    t1 := t1 , (genCheckDigit value:t1).
    t2 := birthDateStr , (genCheckDigit value:birthDateStr).
    t3 := expirationDateStr , (genCheckDigit value:expirationDateStr).

    s := t1 , countryStr , '<<' , t2, '<' , t3 , '<<<<<<'.
    s := s , (genCheckDigit value:(t1 , t2, t3)).
    ^ s

    "
     EuropeanIDCardValidator 
        constructIDCardNumberForDepartment:'1234'
        runningNr:'56789'
        country:'D'
        birthDate:'010101'
        expirationDate:'020202'  
     - no, I will not give you an example with a valid number here ;-)
    "
!

verifyIDCardNumber:numberStringIn
    "returns true if numberStringIn is a valid IDCard number, false if not.

     ***** NOTICE & WARNING ****
     This only verifies for a correct format -
     it does NOT (I repeat: NOT) validate the number for being a valid id card
     (i.e. if such a person exists at all).

     No warranty: the algorithm below was extracted from Public domain
     information (see class documentation) - and is not proven to be correct.
     We do not recommend using that code for real, critical business 
     application. 
     We are not responsible for loss of money ..."

    |genCheckDigit amt lfdNr country birthDate expirationDate check1 check2 check3 check4|

    "/                                1         2         3
    "/                       12345678901234567890123456789012345
    (numberStringIn size == 'bbbb#####*D<<YYMMDD*<yymmdd*<<<<<<*' size) ifFalse:[
        "/ bad length of string 
        ^ false
    ].

    amt := numberStringIn copyFrom:1 to:4.
    lfdNr := numberStringIn copyFrom:5 to:9.
    check1 := (numberStringIn at:10).
    country := numberStringIn copyFrom:11 to:11.
    birthDate := numberStringIn copyFrom:14 to:19.
    check2 := (numberStringIn at:20).
    expirationDate := numberStringIn copyFrom:22 to:27.
    check3 := (numberStringIn at:28).
    check4 := (numberStringIn at:35).

    genCheckDigit := [:str |
                        |sum factor|

                        sum := 0.
                        factor := 7.
                        str do:[:char |
                            sum := sum + (char digitValue * factor).
                            factor := #(3 1 7) at:( #(7 3 1) indexOf:factor).
                        ].
                        sum \\ 10.
                     ].

    (genCheckDigit value:(amt , lfdNr)) = check1 digitValue ifFalse:[ ^ false ].
    (genCheckDigit value:(birthDate)) = check2 digitValue ifFalse:[ ^ false ].
    (genCheckDigit value:(expirationDate)) = check3 digitValue ifFalse:[ ^ false ].
    (genCheckDigit value:(amt , lfdNr , check1 , birthDate , check2 , expirationDate , check3)) = check4 digitValue ifFalse:[ ^ false ].

    ^ true.

"/Public Function PersonalausweisOK( _
"/    ByVal Nummer As String, _
"/    Optional ByVal Geburtsdatum As Date = -1, _
"/    Optional ByVal Ablaufdatum As Date = -1, _
"/    Optional ByVal Behrdenkennzahl As Long = -1 _
"/  ) As Boolean
"/  'Deklarationen:
"/  Dim i As Long
"/  Dim Char As String
"/  Dim NummerPos As Long
"/
"/  '0. Vorab-Checks:
"/  If Len(Nummer) < 25 _
"/  Then Exit Function 'Nummer zu kurz
"/  If Mid$(Nummer, 11, 1) <> "D" _
"/  Then Exit Function 'kein deutscher Personalausweis
"/
"/  '1. Nummer normieren/extrahieren:
"/  For i = 1 To Len(Nummer)
"/    Char = Mid$(Nummer, i, 1)
"/    Select Case Char
"/    Case "<", " ", "D"
"/
"/      'berspringen
"/
"/    Case "0" To "9"
"/
"/      'Nummer merken:
"/      NummerPos = NummerPos + 1
"/      If NummerPos > 25 Then Exit Function
"/      Mid$(Nummer, NummerPos, 1) = Char
"/
"/    Case Else
"/
"/      'ungltiges Zeichen:
"/      Exit Function
"/
"/    End Select
"/  Next i
"/
"/  '2. Behrdenkennzahl/Seriennummer prfen:
"/  If Behrdenkennzahl <> -1 _
"/  Then If Left$(Nummer, 4) <> Format$(Behrdenkennzahl, "0000") _
"/       Then Exit Function
"/  If Prffziffer(Left$(Nummer, 9)) <> Mid$(Nummer, 10, 1) _
"/  Then Exit Function
"/
"/  '3. Geburtsdatum prfen:
"/  If Geburtsdatum <> -1 _
"/  Then If Mid$(Nummer, 11, 6) <> Format$(Geburtsdatum, "yymmdd") _
"/       Then Exit Function
"/  If Prffziffer(Mid$(Nummer, 11, 6)) <> Mid$(Nummer, 17, 1) _
"/  Then Exit Function
"/
"/  '4. Ablaufdatum prfen:
"/  If Ablaufdatum <> -1 _
"/  Then If Mid$(Nummer, 18, 6) <> Format$(Ablaufdatum, "yymmdd") _
"/       Then Exit Function
"/  If Prffziffer(Mid$(Nummer, 18, 6)) <> Mid$(Nummer, 24, 1) _
"/  Then Exit Function
"/
"/  '5. Abschluprfung:
"/  If Prffziffer(Left$(Nummer, 24)) = Right$(Nummer, 1) _
"/  Then PersonalausweisOK = True
"/End Function

    "
     EuropeanIDCardValidator verifyIDCardNumber:'1234567812345678'  
     EuropeanIDCardValidator verifyIDCardNumber:'1234010101D<<0101018<0202026<<<<<<8'  
     EuropeanIDCardValidator verifyIDCardNumber:'1234567897D<<0101011<0202022<<<<<<4'   
     - no, I will not give you an example with a valid number here ;-)
    "
! !

!EuropeanIDCardValidator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/business/EuropeanIDCardValidator.st,v 1.2 2003/11/18 20:45:05 cg Exp $'
! !
