'From Smalltalk/X, Version:6.1.2 on 29-08-2010 at 14:04:04'                     !

Object variableSubclass:#ClSymbolicWeight
	instanceVariableNames:''
	classVariableNames:'Zero'
	poolDictionaries:'ClConstants'
	category:'Constraints-Cassovary-Kernel'
!

!ClSymbolicWeight class methodsFor:'uncategorized private'!

initializeAfterLoad
    "ACTION
        Initialize the class variables to the pre-defined strengths.
    "

    Zero := self new.
    1 to: Zero size do: [:i | Zero at: i put: 0.0].

! !

!ClSymbolicWeight class methodsFor:'uncategorized public'!

new
    ^self basicNew: ClStrength nLevels
!

zero
    ^Zero
! !

!ClSymbolicWeight methodsFor:'double dispatching'!

productFromFloat:aFloat
    ^ self * aFloat

    "Created: / 28-08-2010 / 14:36:38 / cg"
! !

!ClSymbolicWeight methodsFor:'uncategorized public'!

* n
    "Multiply this symbolic weight by n (n should be a number)"
        | result |
    n isNumber ifFalse: [ExCLInternalError signal].
    result := self class new.
    1 to: self size do: [:i | result at: i put: n*(self at: i)].
    ^result
!

+ n
    "Add this symbolic weight to n (which must also be a symbolic weight)"
        | result |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    result := self class new.
    1 to: self size do: [:i | result at: i put: (self at: i) + (n at: i)].
    ^result
!

- n
    "Subtract n from this symbolic weight (n must also be a symbolic weight)"
        | result |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    result := self class new.
    1 to: self size do: [:i | result at: i put: (self at: i) - (n at: i)].
    ^result
!

/ n
    "Divide this symbolic weight by n (n should be a number)"
        | result |
    n isNumber ifFalse: [ExCLInternalError signal].
    result := self class new.
    1 to: self size do: [:i | result at: i put: (self at: i)/n].
    ^result
!

< n
        | a b |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    1 to: self size do: [:i | 
        a := self at: i.  
        b := n at: i.
        a<b ifTrue: [^true].
        a>b ifTrue: [^false]
        "they're equal ... keep going"].
    "all elements were equal"
    ^false
!

<= n
        | a b |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    1 to: self size do: [:i | 
        a := self at: i.  
        b := n at: i.
        a<b ifTrue: [^true].
        a>b ifTrue: [^false]
        "they're equal ... keep going"].
    "all elements were equal"
    ^true
!

= n
    n isSymbolicWeight  ifFalse: [^false].
    1 to: self size do: [:i | 
        (self at: i) = (n at: i) ifFalse: [^false]].
    "all elements were equal"
    ^true
!

> n
        | a b |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    1 to: self size do: [:i | 
        a := self at: i.  
        b := n at: i.
        a>b ifTrue: [^true].
        a<b ifTrue: [^false]
        "they're equal ... keep going"].
    "all elements were equal"
    ^false
!

>= n
        | a b |
    n isSymbolicWeight  ifFalse: [ExCLInternalError signal].
    1 to: self size do: [:i | 
        a := self at: i.  
        b := n at: i.
        a>b ifTrue: [^true].
        a<b ifTrue: [^false]
        "they're equal ... keep going"].
    "all elements were equal"
    ^true
!

approxNonNegative
    "return true if this symbolic weight is non-negative.  Allow coefficients that are within epsilon of
        0 to count as 0"
        | a nepsilon |
    nepsilon := 0.0 - ClEpsilon.
    1 to: self size do: [:i | 
        a := self at: i.  
        a<nepsilon ifTrue: [^false].
        a>ClEpsilon ifTrue: [^true]].
    ^true
!

clApprox: s
    "Return true if this symbolic weight is approximately equal to s. 
        The argument s must be another symbolic weight -- we can't
        compare symbolic weights with ordinary numbers."
    s isSymbolicWeight ifFalse: [ExCLInternalError signal].
    1 to: self size do: [:i | 
        ((self at: i) clApprox: (s at: i)) ifFalse: [^false]].
    ^true
!

clApproxZero
    "return true if this symbolic weight is approximately zero"
    ^self clApprox: Zero
!

isSymbolicWeight
    ^true
!

printOn: strm
    strm nextPutAll: '['.
    (self at: 1) printOn: strm.
    2 to: self size do: [:i | strm nextPutAll: ','.  (self at: i) printOn: strm].
    strm nextPutAll: ']'
! !

