"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/communication' }"

Object subclass:#AbstractFTPSession
	instanceVariableNames:'server sessionProcess socket timeOut lastArgument user
		workingDirectory mode dataSocket'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-FTP'
!

!AbstractFTPSession class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Helper class for the FTPServer.

    Common code for FTP serverSessions - add concrete subclass
    The stuff implemented here realizes the protocol;
    real file operations and authorization must be implemented in subclasses.

    Methods which should be redefined are found in the 'must be redefined' category.

    [author:]
        Claus Gittinger (extracted code from FTPSession)

    [start with:]
        FTPServer start.
        FTPServer startOnPort:portNr.
"
! !

!AbstractFTPSession class methodsFor:'defaults'!

ftpVersion
    ^ 'Version 1.0 Smalltalk/X'
! !

!AbstractFTPSession methodsFor:'accessing'!

server:something
    server := something.
! !

!AbstractFTPSession methodsFor:'debugging'!

log:aMessage
    server log:'[' , socket getPeerName , ']: ' , aMessage
!

logCommand:aMessage
    self log:aMessage
! !

!AbstractFTPSession methodsFor:'ftp system'!

HELP
    "send all commands supported by FTP server
    "
    |commands n|

    commands := self class methodDictionary keys select:[:name |
                        name size <= 4 
                        and:[(name contains:[:char | char isLowercase]) not
                        and:[(name includes:$:) not]]
                ].
    socket nextPutLine:'214-The following commands are recognized.'.
    n := 0.

    commands sort do:[:aCommand |
        socket nextPutAll:'   '.
        socket nextPutAll:(aCommand paddedTo:4).

        (n := n + 1) == 8 ifTrue:[
            socket cr.
            n := 0.
        ].
    ].
    n ~~ 0 ifTrue:[
        socket cr.
    ].
    socket nextPutLine:'214 Comments to ca@exept.de.'.
!

NOOP
    "no instruction command
    "

    socket nextPutLine:'200 Okay'.
!

PASS
    "password from user; already readin from USER command
    "
    user isNil ifTrue:[
        FTPServerException loginIncorrect
    ].
    socket nextPutLine:'230 User ', user, ' logged in'.
!

PORT
    |words ipAddress port|

    self closeDataConnection.

    lastArgument notNil ifTrue:[
        words := lastArgument asCollectionOfSubstringsSeparatedBy:$,.
    ].

    words size ~~ 6 ifTrue:[
        FTPServerException wrongNumberOfArguments
    ].

    ipAddress := ByteArray with:((words at:1) asNumber)
                           with:((words at:2) asNumber)
                           with:((words at:3) asNumber)
                           with:((words at:4) asNumber).

    port := (words at:5) asNumber bitShift:8.
    port := port bitOr:((words at:6) asNumber).

    dataSocket := Socket newTCPclientToAddress:ipAddress port:port withTimeout:5000.

    dataSocket isNil ifTrue:[
        socket nextPutLine:'421 cannot open connection'.
        ^ self.
    ].
    socket nextPutLine:'200 PORT command successful.'
!

QUIT
    "shutdown server
    "
    self doReinitialize.
    socket nextPutLine:'221 Goodbye'.
    socket close.

!

REIN
    "reinitialize server
    "
    self doReinitialize.
    socket nextPutLine:'200 Reinitialize'.

!

SITE
    |words time cmd|

    lastArgument isNil ifTrue:[
        FTPServerException wrongNumberOfArguments
    ].

    words := lastArgument asCollectionOfWords.
    cmd := words first asUppercase.

    cmd = 'IDLE' ifTrue:[
        words size == 2 ifTrue:[
            time := Number fromString:(words at:2) onError:nil.
        ].

        time isNil ifTrue:[
            socket nextPutLine:('200 Current IDLE time limit is %1 seconds; max %2.'
                                    bindWith:timeOut with:7200).
            ^ self.
        ].

        (time >= 10 and:[time <= 7200]) ifTrue:[
            timeOut := time.
            socket nextPutLine:('200 Maximum IDLE time set to %1 seconds.' bindWith:timeOut).
            ^ self.
        ].

        socket nextPutLine:('501 Maximum IDLE time must be between 10 and %1 second.'
                                    bindWith:7200).
        ^ self
    ].

    cmd = 'HELP' ifTrue:[
        socket nextPutLine:'214-The following SITE commands are recognized.'.
        socket nextPutLine:'   HELP   IDLE'.
        socket nextPutLine:'214 Comments to ca@exept.de.'.
        ^ self
    ].

    socket nextPutLine:('500 ''SITE %1'': command not understood.' bindWith:cmd).
!

STAT
    socket nextPutLine:'211- ' , OperatingSystem getHostName , ' FTP server status:'.
    socket nextPutLine:('   ', self class ftpVersion).
    socket nextPutLine:'   Connected to ' , socket getPeerName.

    user notNil ifTrue:[
        socket nextPutLine:'   Logged in as ' , user.
    ].
    socket nextPutLine:'   TYPE: ' , mode asUppercase.
    socket nextPutLine:'211 End of status'.
!

SYST
    |type|

    type := 'UNIX'.

	     OperatingSystem isVMSlike       ifTrue:[type := 'VMS']
    ifFalse:[OperatingSystem isMSWINDOWSlike ifTrue:[type := 'MSWINDOWS']].

    socket nextPutLine:('215 %1 Type: L8' bindWith:type).
!

USER
    |newUser|

    self doReinitialize.

    (newUser := lastArgument) isNil ifTrue:[
        FTPServerException wrongNumberOfArguments
    ].

    (self loginUserNeedsPassword:newUser) ifTrue:[
        socket nextPutLine:('331 Password required for ', newUser).

        self nextCommand ~~ #PASS ifTrue:[FTPServerException loginIncorrect].
        lastArgument isNil        ifTrue:[FTPServerException wrongNumberOfArguments].

        (self checkPassword:lastArgument for:newUser) ifFalse:[
            FTPServerException loginIncorrect
        ].
    ].
    self doUserLoggedIn:newUser.
    socket nextPutLine:'230 User ', newUser, ' logged in'.
! !

!AbstractFTPSession methodsFor:'ftp user commands'!

CDUP
    "change to parent directory
    "
    |dir|

    dir := self checkAndGetCWD directory.

    (self canChangeWorkingDirectory:dir) ifFalse:[
        FTPServerException permissionDenied
    ].
    workingDirectory := dir.
    socket nextPutLine:'200 CDUP command successful'.
!

CWD
    "change working directory
    "
    |directory|

    directory := self checkAndGetArgFilename.

    (self canChangeWorkingDirectory:directory) ifFalse:[
        FTPServerException permissionDenied:lastArgument
    ].
    workingDirectory := directory.
    socket nextPutLine:'250 CWD command successful'.
!

DELE
    "delete file or (empty) directory
    "
    self doDELETE:false
!

LIST
    "show directory contents; longFormat
    "
    self doLIST:true
!

MKD
    "make directory
    "
    |file|

    file := self checkAndGetArgFilename.

    self doMKD:file.
!

NLST
    "show directory contents; shortFormat
    "
    self doLIST:false
!

PWD
    "get current working directory
    "
    socket nextPutLine:('257 "%1" is current directory.' bindWith:(self checkAndGetCWD pathName)).
!

RETR
    |file outStream|

    outStream := self checkAndGetDaTaConnection.
    file      := self checkAndGetArgFilename.
    self doRETR:file onTo:outStream.
!

RMD
    "recursive delete directory
    "
    self doDELETE:true
!

STOR
    self doSTORE:false
!

STOU
    self doSTORE:true
!

TYPE
    "set mode: 
	bin  (I or i) := #binary
	ascii(A or a) := #ascii
    "
    |arg|

    lastArgument size == 1 ifFalse:[
	^ socket nextPutLine:('500 Type not supported: ', lastArgument, '.')
    ].

    arg := lastArgument first asUppercase.

	     arg == $A ifTrue:[mode := #ascii]
    ifFalse:[arg == $I ifTrue:[mode := #binary]
    ifFalse:[
	^ socket nextPutLine:('500 Type not supported: ', lastArgument, '.')
    ]].

    socket nextPutLine:('200 Type set to ', lastArgument, '.').    
! !

!AbstractFTPSession methodsFor:'must be redefined'!

canChangeWorkingDirectory:newDirectory 
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
!

checkPassword:arg1 for:arg2 
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
!

defaultDirectoryForUser:userName 
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
!

doDELETE:recursive 
    "raise an error: must be redefined in concrete subclass(es)"

    self log:('not allowed: DELE').

    FTPServerException permissionDenied.
!

doLIST:arg 
    "raise an error: must be redefined in concrete subclass(es)"

    self log:('not allowed: LIST').

    FTPServerException permissionDenied.
!

doMKD:arg 
    "raise an error: must be redefined in concrete subclass(es)"

    self log:('not allowed: MKD').

    FTPServerException permissionDenied.
!

doRETR:file onTo:outStream
    "raise an error: must be redefined in concrete subclass(es)"

    self closeDataConnection.
    FTPServerException permissionDenied:lastArgument
!

doSTORE:arg 
    "raise an error: must be redefined in concrete subclass(es)"

    self log:('not allowed STOR').

    FTPServerException permissionDenied.
!

loginUserNeedsPassword:arg 
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
! !

!AbstractFTPSession methodsFor:'private'!

closeDataConnection

    dataSocket notNil ifTrue:[
	dataSocket close.
	dataSocket := nil.
    ].
!

doReinitialize
    "reinitialize the system; called when:
	- user logged out
	- system before shutdown
	- reinitailze system request
	- during startup
    "
    user             := nil.
    workingDirectory := nil.
    mode             := #ascii.

    self closeDataConnection.
!

doUserLoggedIn:aUser
    "setup user context
    "
    user := aUser.
    workingDirectory := self defaultDirectoryForUser:user.
!

nextCommand
    "reads the next command from socket; if a timeout occurs nil is returned otherwise
     the command as symbol and the lastArgument is set to the received argument
    "
    |line soT eoT cmd|

    lastArgument := nil.

    [socket readWaitWithTimeout:timeOut] whileFalse:[
        line := socket nextLine.
        line isNil ifTrue:[^ nil].

        self logCommand:line.

        (soT := line indexOfNonSeparatorStartingAt:1) ~~ 0 ifTrue:[
            eoT := line indexOfSeparatorStartingAt:soT.

            eoT == 0 ifTrue:[
                cmd := line copyFrom:soT
            ] ifFalse:[
                cmd := line copyFrom:soT to:(eoT - 1).
                soT := line indexOfNonSeparatorStartingAt:eoT.

                soT ~~ 0 ifTrue:[
                    lastArgument := line copyFrom:soT.

                    lastArgument last isSeparator ifTrue:[
                        lastArgument := lastArgument withoutSeparators
                    ]
                ]
            ].
            ^ cmd asUppercase asSymbol
        ]
    ].

    ^ nil
!

serveSocket:aSocket
    |cmd|

    socket  := aSocket.
    timeOut := 900.

    self doReinitialize.

    socket nextPutLine:('220 %1 FTP server (%2) ready'
                         bindWith:(OperatingSystem getHostName)
                             with:(self class ftpVersion)
                       ).

    [socket isOpen] whileTrue:[
        (cmd := self nextCommand) isNil ifTrue:[
            self log:'Timeout on socket'.
            self QUIT
        ] ifFalse:[
            FTPServerException handle:[:ex|
                socket nextPutLine:ex description.
                self closeDataConnection.
            ] do:[
                AbortSignal handle:[
                    self log:'Command aborted'.
                    socket nextPutLine:'502 Internal Server error'.
                ] do:[
                    self perform:cmd ifNotUnderstood:[
                        self log:('Command not implemented: ', cmd, ' <', lastArgument printString, '>').
                        socket nextPutLine:'502 Command not implemented: ' , cmd.
                    ]
                ]
            ]
        ]
    ].
! !

!AbstractFTPSession methodsFor:'private ftp'!

checkAndGetArgFilename
    |file currDir|

    lastArgument isNil ifTrue:[
        FTPServerException wrongNumberOfArguments
    ].
    currDir  := self checkAndGetCWD.

    (file := lastArgument asFilename) isAbsolute ifFalse:[
        file := currDir construct:lastArgument
    ].
    ^ file
!

checkAndGetCWD

    workingDirectory isNil ifTrue:[
        FTPServerException noWorkingDirectory
    ].
    ^ workingDirectory
!

checkAndGetDaTaConnection

    dataSocket isNil ifTrue:[
        FTPServerException noDataConnection
    ].
    ^ dataSocket
! !

!AbstractFTPSession methodsFor:'process'!

clientOn:aSocket
    "process for new client
    "

    aSocket isNil ifTrue:[
        ^ nil
    ].

    sessionProcess := [  
                           [ Exception handle:[:ex|
                                (Object haltSignal accepts:ex signal) ifTrue:[
                                    ex reject
                                ].
                                self log:('error cought: ' , ex description).
"/ during debugging
                                ex reject.
                             ] do:[
                                self serveSocket:aSocket
                             ]
                           ] ensure:[
                                aSocket isOpen ifTrue:[
                                    aSocket shutDown
                                ].
                                self doReinitialize
                           ]
                        ] newProcess.

    sessionProcess priority:(Processor userBackgroundPriority).
    sessionProcess resume.
    ^ sessionProcess
! !

!AbstractFTPSession methodsFor:'queries'!

isBinary
    ^ mode == #binary
! !

!AbstractFTPSession class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/communication/AbstractFTPSession.st,v 1.8 2007/06/13 08:33:55 cg Exp $'
! !
