"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/communication' }"

Object subclass:#FTPServer
	instanceVariableNames:'serverProcess acceptorSocket portNumber sessionClass'
	classVariableNames:'LastServer Verbose'
	poolDictionaries:''
	category:'Net-Communication-FTP'
!

!FTPServer class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    an FTP server - subclass to add/change its functionality.

    the servers functionality is limited (by purpose)
    - see the comment in FTPSession

    [author:]
        Claus Atzkern

    [start with:]
        FTPServer start.

        FTPServer startOnPort:12345.

        (FTPServer forPort:12345)
            sessionClass:SmalltalkFTPSession;
            start
"
! !

!FTPServer class methodsFor:'instance creation'!

forPort:aPort
    "return a server for the given port"

    ^ self basicNew port:aPort.

    "
     (FTPServer forPort:12387) start
    "
!

start
    "start a server on the default port (21).
     Notice: this is a priviledged port, not normally accessable to users."

    ^ self startOnPort:(Socket portOfService:'ftp').

    "
     FTPServer start
    "
!

startOnPort:aPort
    "start a server on the given port"

    ^ self basicNew servePort:aPort.

    "
     FTPServer startOnPort:12387
    "
! !

!FTPServer class methodsFor:'obsolete'!

onPort:aPort
    <resource: #obsolete>
    self obsoleteMethodWarning:'use #startOnPort:'.
    ^ self startOnPort:aPort.

    "
     FTPServer onPort:12387
    "
! !

!FTPServer class methodsFor:'tests'!

testClose
    |server|

    (server := LastServer) notNil ifTrue:[
        LastServer := nil.

        Exception handle:[:ex|]do:[
            server terminateWithAllSubprocesses.
            server waitUntilTerminated.
        ].
    ].
    ^ LastServer
!

testOpen
    |port|

    port := 12345.

    LastServer isNil ifTrue:[
        LastServer := FTPServer onPort:port.
        LastServer addExitAction:[
            Transcript showCR:'close server'.
            LastServer := nil
        ]
    ].
    Transcript showCR:('running on port: ', port printString).
    Transcript showCR:('ftp command: ftp data ', port printString).
    ^ LastServer
! !

!FTPServer methodsFor:'accessing'!

port:aPortNumber
    portNumber := aPortNumber.
!

sessionClass:something
    sessionClass := something.
! !

!FTPServer methodsFor:'debugging'!

log:aMessage
    Verbose == true ifTrue:[
        Transcript show:(Timestamp now); show:': '; showCR:aMessage
    ]
! !

!FTPServer methodsFor:'obsolete'!

onPort:aPort
    <resource: #obsolete>
    "obsolete (backward compatible) entry
    "

    self obsoleteMethodWarning:'use servePort:'.
    self servePort:aPort.
! !

!FTPServer methodsFor:'serving'!

servePort:aPort
    "start an FTP-Server on a port
    "

    self port:aPort.
    self start.
!

start
    "start an FTP-Server"

    acceptorSocket := Socket newTCPserverAtPort:portNumber.
    acceptorSocket isNil ifTrue:[
        self error:'cannot create socket on port: ' , portNumber printString.
    ].

    (acceptorSocket listenFor:2) ifFalse:[
        acceptorSocket shutDown.
        self error:'listen failed on port: ' , portNumber printString.
    ].

    serverProcess := [ 
                        [   
                            self serverLoop
                        ] ensure:[acceptorSocket shutDown].
                      ] newProcess.

    serverProcess priority:(Processor userSchedulingPriority).
    serverProcess name:('FTP Server [<1p>]' expandMacrosWith:portNumber).
    serverProcess resume.
! !

!FTPServer methodsFor:'serving-private'!

serverLoop
    |session clientProcess connection peerName|

    [true] whileTrue:[
        Error handle:[:ex|
            ex description infoPrintCR.
        ] do:[
            self log:'Waiting for a connection'.
            acceptorSocket readWait.
            self log:'Accepting'.
            connection := acceptorSocket accept.
            peerName := connection getPeerName.
            self log:'connection from ' , peerName.

            session := (sessionClass ? FTPSession) new.
            session server:self.
            clientProcess := session clientOn:connection.
            clientProcess notNil ifTrue:[
                clientProcess name:('FTP Session [%1:%2]' bindWith:peerName with:portNumber).
            ]
        ]
    ]
! !

!FTPServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/communication/FTPServer.st,v 1.17 2009/02/26 21:31:17 stefan Exp $'
! !
