"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/communication' }"

ObjectCoder subclass:#JSONPrinter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-JSON'
!

!JSONPrinter class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    I encode a limited subset of smalltalk object types into a JSON representation.

    Allowed are:
        strings,
        numbers
        booleans,
        nil,
        Arrays and OrderedCollections,
        Dictionaries.

    Everything else is not representable. I do not handle recursive data structures.

    Author:
        Claus Gittinger
        Loosely Based upon Public Domain Code from Robin Redeker published in lists.gnu.org
"
!

examples
"
     see examples in JSONReader
"
! !

!JSONPrinter methodsFor:'visiting'!

visitBoolean:anBoolean with:aParameter
   anBoolean printOn: stream

    "Created: / 27-08-2007 / 11:33:08 / cg"
!

visitDate: aDate with: aParameter 
    
    stream 
        nextPutAll: 'new Date(' ;
        nextPutAll: aDate year asString ;
        nextPutAll:',' ;
        nextPutAll: (aDate month - 1) printString;  "/ months start at 0
        nextPutAll:',' ;
        nextPutAll: aDate  dayOfMonth asString;
        nextPutAll:')'.
!

visitDictionary:aDictionary with:aParameter
   | f |
   stream nextPut: ${.
   f := true.
   aDictionary keysAndValuesDo: [ :key :val |
      f ifFalse: [ stream nextPut: $, ].
      key acceptVisitor:self with:aParameter.
      stream nextPut: $:.
      val acceptVisitor:self with:aParameter.
      f := false
   ].
   stream nextPut: $}.

    "Created: / 27-08-2007 / 11:44:06 / cg"
!

visitFloat:aFloat with:aParameter
   stream nextPutAll:
	(aFloat printString copyReplacing: aFloat class exponentCharacter withObject: $e)

    "Created: / 27-08-2007 / 11:32:37 / cg"
!

visitInteger:anInteger with:aParameter
   anInteger printOn: stream

    "Created: / 27-08-2007 / 11:30:10 / cg"
!

visitNilWith:aParameter
   stream nextPutAll: 'null'

    "Created: / 27-08-2007 / 11:37:08 / cg"
!

visitNumber:aNumber with:aParameter
   aNumber asFloat printOn: stream

    "Created: / 27-08-2007 / 11:30:24 / cg"
!

visitSequenceableCollection:aCollection with:aParameter
   | f |

   stream nextPut: $[.
   f := true.
   aCollection do: [ :val |
      f ifFalse: [ stream nextPut: $, ].
      val acceptVisitor:self with:aParameter.
      f := false
   ].
   stream nextPut: $].

    "Created: / 27-08-2007 / 11:19:17 / cg"
!

visitString:aString with:aParameter
   stream nextPut: $".
   aString do: [ :c |
      | i f |

      i := c asInteger.
      (((i = 16r20
	 or: [ i = 16r21 ])
	 or: [ i >= 16r23 and: [ i <= 16r5B ] ])
	 or: [ i >= 16r5D ])
	    ifTrue: [ stream nextPut: c ]
	    ifFalse: [
	       f := false.
	       stream nextPut: $\.
	       i = 16r22 ifTrue: [ f := true. stream nextPut: c ].
	       i = 16r5C ifTrue: [ f := true. stream nextPut: c ].
	       i = 16r2F ifTrue: [ f := true. stream nextPut: c ].
	       i = 16r08 ifTrue: [ f := true. stream nextPut: $b ].
	       i = 16r0C ifTrue: [ f := true. stream nextPut: $f ].
	       i = 16r0A ifTrue: [ f := true. stream nextPut: $n ].
	       i = 16r0D ifTrue: [ f := true. stream nextPut: $r ].
	       i = 16r09 ifTrue: [ f := true. stream nextPut: $t ].
	       f ifFalse: [
		  stream nextPut: $u.
		  stream nextPutAll: (i printStringRadix:16 padTo:4) ].
	    ]
   ].
   stream nextPut: $".

    "Created: / 27-08-2007 / 11:29:21 / cg"
!

visitTimestamp: aTimestamp with: aParameter 

    |nameOfMonth timestampString|

    "Month dd, yyyy hh:mm:ss"
    nameOfMonth := Date abbreviatedNameOfMonth: aTimestamp month.

    timestampString := String streamContents:[:s |
        s nextPutAll: nameOfMonth asUppercaseFirst,' '.
        aTimestamp printOn: s format:'%(day), %(year) %h:%m:%s'.
    ].

    stream 
        nextPutAll: 'new Date("', timestampString, '")'.
! !

!JSONPrinter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/communication/JSONPrinter.st,v 1.10 2011/11/23 16:55:07 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/communication/JSONPrinter.st,v 1.10 2011/11/23 16:55:07 cg Exp $'
! !
