"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/communication' }"

Stream subclass:#JSONReader
	instanceVariableNames:'stream'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-JSON'
!

!JSONReader class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    I read data structures (currently build of OrderedCollection and Dictionary)
    from and to JSON (Java Script Object Notation). 
    Writing is done with the #toJSON: class-method (note: it will behave badly with circular data structures).
    Reading via the #fromJSON: class-method.

    Author:
        Public Domain Code from Robin Redeker published in lists.gnu.org
        adapted to ST/X by Claus Gittinger
"
!

examples
"
                                                                        [exBegin]
     |o1 s o2|
     o1 := #('hello' 123 nil true false (1 'two' 3.0)).
     s := JSONReader toJSON:o1.
     o2 := JSONReader fromJSON:s   
                                                                        [exEnd]

                                                                        [exBegin]
     |o1 s o2|
     o1 := Dictionary withKeysAndValues:#('one' 1 'two' 2 'three' 3.0 'four' 'vier').
     s := JSONReader toJSON:o1.
     o2 := JSONReader fromJSON:s   
                                                                        [exEnd]

                                                                        [exBegin]
     |o1 s o2|
     o1 := Dictionary withKeysAndValues:#('one two three' 123).
     s := JSONReader toJSON:o1.
     o2 := JSONReader fromJSON:s   
                                                                        [exEnd]
"
! !

!JSONReader class methodsFor:'instance creation'!

on: aStream
    ^self basicNew stream: aStream

    "Modified: / 27-08-2007 / 11:35:05 / cg"
! !

!JSONReader class methodsFor:'encoding & decoding'!

fromJSON: string
    "decode the JSON string (as received from JavaScript) to an object."

    ^(self on: string readStream) nextJSONObject

    "Modified: / 07-01-2011 / 12:40:56 / cg"
!

toJSON: anObject
    "return a JSON string which represents the object (can be sent to JavaScript)."

    |printer|

    printer := JSONPrinter on:(WriteStream on:(String new:100)).
    anObject acceptVisitor:printer.
    ^ printer contents.

"/   ^anObject toJSON

    "
     |s1 s2|
     s1 := self toJSON:#('hello' 123 true (1 'two' 3.0)).
     s2 := self fromJSON:s1  
    "

    "Modified: / 07-01-2011 / 12:41:01 / cg"
! !

!JSONReader methodsFor:'private'!

next
    "I'm returning the next non-whitespace character"
    
    |c|

    c := self peek.
    c isNil ifTrue:[
        ^ self error:'expected character but found end of stream'
    ].
    stream next.
    ^ c

    "Modified: / 07-01-2011 / 12:52:17 / cg"
!

nextJSONArray
   "I decode JSON arrays from self and will return an Array for them."

   | c obj value |

   obj := OrderedCollection new.
   self next.
   [ c := self peek.
     (c = $]) ] whileFalse: [
      (c = $,) ifTrue: [ self next. ].
      value := self nextJSONObject.
      obj add: value.
   ].
   self next.
   ^ obj asArray
!

nextJSONDict
   "I decode JSON objects from self and will return a Dictionary containing all the key/value pairs."
   | c obj key value |
   obj := Dictionary new.
   self next.
   [ c := self peek.
     c = $} ] whileFalse: [
      (c = $,) ifTrue: [ self next ].

      key := self nextJSONString.

      c := self next.
      c = $: ifFalse: [
         self error: ('unexpected character found where name-seperator '':'' expected, found: %1' bindWith: c)
      ].

      value := self nextJSONObject.

      obj at: key put: value.
   ].
   self next.
   ^obj
!

nextJSONNumber
   "I'm extracting a number in JSON format from self and return Integer or Float depending on the input."
   | c num sgn int intexp frac exp isfloat |
   num := WriteStream on: (String new).

   isfloat := false.
   sgn     := 1.
   int     := 0.
   intexp  := 1.

   c := self peek.
   (c isNil) ifTrue: [ ^self error: 'expected number or -sign, but found end of self' ].
   c = $- ifTrue: [ sgn := -1. self next. ].

   c := self peek.
   (c isNil) ifTrue: [ ^self error: 'expected number, but found end of self' ].
   (c isDigit or: [ c = $. ]) ifFalse: [ ^self error: 'invalid JSON input' ].

   [ c notNil and: [ c isDigit ] ] whileTrue: [
      self next.
      int := sgn * (c digitValue) + (int * 10).
      c := self peek
   ].
   (c isNil) ifTrue: [ ^int ].

   c = $. ifTrue: [
      self next.
      isfloat := true.
      [ c := self peek. c notNil and: [ c isDigit ] ] whileTrue: [
         sgn := sgn / 10.
         int := sgn * (c digitValue) + int.
         self next
      ]
   ].

   exp := 0.
   ((c = $e) or: [ c = $E ]) ifFalse: [
        ^isfloat ifTrue: [ int asFloat ] ifFalse: [ int ] ].

   self next.
   c := self peek.
   (c isNil) ifTrue: [ ^int ].
   sgn := 1.
   c = $+ ifTrue: [ sgn :=  1. self next ].
   c = $- ifTrue: [ sgn := -1. self next ].

   [ c := self peek. c notNil and: [ c isDigit ] ] whileTrue: [
      exp := (c digitValue) + (exp * 10).
      self next
   ].

   int := int * (10 raisedToInteger: exp * sgn).
   ^int asFloat
!

nextJSONString
   "I'm extracting a JSON string from self and return them as String."

   | c str |

   str := CharacterWriteStream on:(String new).
   self next.
   [
        c := stream next.    "/ not self next here - we want spaces inside the string !!
        c = $"
   ] whileFalse: [
        c = $\ ifTrue: [
            c := stream next.   "/ not self next here - we want spaces inside the string !!
            c isNil ifTrue:[ 
                ^ self error: 'expected character, found end of self' 
            ].
            c = $b ifTrue: [ c := 8 asCharacter ].
            c = $f ifTrue: [ c := 12 asCharacter ].
            c = $n ifTrue: [ c := Character nl ].
            c = $r ifTrue: [ c := Character cr ].
            c = $t ifTrue: [ c := Character tab ].
            c = $u ifTrue: [
                c := Character codePoint:(Integer readFrom: (self next: 4) readStream radix: 16).
            ]
      ].
      str nextPut:c.
   ].

   "Undo the conversion to UnicodeString done above."
   ^str contents asString
!

peek
    "I'm peeking for the next non-whitespace character and will drop all whitespace in front of it"
    
    |c|

    [
        c := stream peek.
        c = (Character space) or:[ c = (Character tab) or:[ c = (Character lf) or:[ c = (Character return) ] ] ]
    ] whileTrue:[ stream next ].
    ^ c

    "Modified: / 07-01-2011 / 12:52:12 / cg"
!

stream: aStream
    stream := aStream
! !

!JSONReader methodsFor:'reading'!

nextJSONObject
    "I decode a json self to a value, which will be one of: 
         nil, true, false, OrderedCollection, Dictionary, String or Number
     (I will return Integer or Float depending on the input)."

    | c |

    c := self peek.
    (c = $n) ifTrue: [ self next: 4. ^nil   ].
    (c = $t) ifTrue: [ self next: 4. ^true  ].
    (c = $f) ifTrue: [ self next: 5. ^false ].
    (c = ${) ifTrue: [ ^self nextJSONDict ].
    (c = $[) ifTrue: [ ^self nextJSONArray  ].
    (c = $") ifTrue: [ ^self nextJSONString ].
    ^self nextJSONNumber

    "Modified: / 07-01-2011 / 12:53:26 / cg"
! !

!JSONReader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/communication/JSONReader.st,v 1.9 2011/01/07 11:53:43 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/communication/JSONReader.st,v 1.9 2011/01/07 11:53:43 cg Exp $'
! !
