"{ Package: 'stx:goodies/compression' }"

Object subclass:#GZipSurrogateStream
	instanceVariableNames:'gZipStream zippedFileStream bufferStream positionThusFar'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Compression'
!

GZipSurrogateStream comment:'A pseudo stream that allows SmartRefStream to write directly to a gzipped file. There are some peculiarities of the project exporting process that require:

1. We ignore #close since the file is closed and may be reopened to continue writing. We implement #reallyClose for when we know that all writing is over.

2. We use a BitBlt to write WordArrayForSegment objects. Bit of a hack, but there it is.

| fileStream wa |

wa _ WordArrayForSegment new: 30000.
1 to: wa size do: [ :i | wa at: i put: i].
fileStream _ GZipSurrogateStream newFileNamed: ''xxx3.gz'' inDirectory: FileDirectory default.
fileStream nextPutAll: ''this is a test''.
fileStream nextPutAll: wa.
fileStream reallyClose.
'
!


!GZipSurrogateStream class methodsFor:'as yet unclassified'!

newFileNamed: fName inDirectory: aDirectory

	^self new newFileNamed: fName inDirectory: aDirectory
! !

!GZipSurrogateStream methodsFor:'as yet unclassified'!

ascii

	self bufferStream ascii
!

binary

	self bufferStream binary
!

bufferStream

	^bufferStream ifNil: [bufferStream _ RWBinaryOrTextStream on: (ByteArray new: 5000)].
!

close
	
	"we don't want to until user is really done"
	

!

closed

	^false
!

command: aString
	"Overridden by HtmlFileStream to append commands directly without translation.  4/5/96 tk"
	"We ignore any HTML commands.  Do nothing"
!

cr

	self bufferStream cr
!

fileOutClass: extraClass andObject: theObject
	"Write a file that has both the source code for the named class and an object as bits.  Any instance-specific object will get its class written automatically."

	| class srefStream |

	self timeStamp.

	extraClass ifNotNil: [
		class _ extraClass.	"A specific class the user wants written"
		class sharedPools size > 0 ifTrue: [
			class shouldFileOutPools ifTrue: [class fileOutSharedPoolsOn: self]
		].
		class fileOutOn: self moveSource: false toFile: 0
	].

	"Append the object's raw data"
	srefStream _ SmartRefStream on: self.
	srefStream nextPut: theObject.  "and all subobjects"
	srefStream close.		"also closes me - well it thinks it does, anyway"
!

flushBuffer

	| data |
	bufferStream ifNil: [^self].
	data _ bufferStream contents asByteArray.
	gZipStream nextPutAll: data.
	positionThusFar _ positionThusFar + data size.
	bufferStream _ nil.
!

header

	"ignore"
!

newFileNamed: fName inDirectory: aDirectory

	positionThusFar _ 0.
	zippedFileStream _ aDirectory newFileNamed: fName.
	zippedFileStream binary; setFileTypeToObject.
		"Type and Creator not to be text, so can be enclosed in an email"
	gZipStream _ GZipWriteStream on: zippedFileStream.
!

next

	^self bufferStream next
!

nextChunkPut: aString

	self bufferStream nextChunkPut: aString
!

nextInt32Put: int32

	^self bufferStream nextInt32Put: int32
!

nextNumber: n put: v 

	^self bufferStream nextNumber: n put: v 
!

nextPut: aByte

	^self bufferStream nextPut: aByte
!

nextPutAll: aString

	^aString writeOnGZIPByteStream: self
!

nextPutAllBytes: aString

	^self bufferStream nextPutAll: aString
!

nextPutAllWordArray: aWordArray

	| ba hackwa hackba blt rowsAtATime sourceOrigin rowsRemaining |

	self flag: #bob.		"do we need to be concerned by bytesPerElement??"
	ba _ nil.
	rowsAtATime _ 2000.		"or 8000 bytes"
	hackwa _ Form new hackBits: aWordArray.
	sourceOrigin _ 0@0.
	[(rowsRemaining _ hackwa height - sourceOrigin y) > 0] whileTrue: [
		rowsAtATime _ rowsAtATime min: rowsRemaining.
		(ba isNil or: [ba size ~= (rowsAtATime * 4)]) ifTrue: [
			ba _ ByteArray new: rowsAtATime * 4.
			hackba _ Form new hackBits: ba.
			blt _ (BitBlt toForm: hackba) sourceForm: hackwa.
		].
		blt 
			combinationRule: Form over;
			sourceOrigin: sourceOrigin;
			destX: 0 destY: 0 width: 4 height: rowsAtATime;
			copyBits.
		self bufferStream nextPutAll: ba.
		self flushBuffer.
		sourceOrigin _ sourceOrigin x @ (sourceOrigin y + rowsAtATime).
	].
!

nextStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	| length |
	(length _ s size) < 192
		ifTrue: [self nextPut: length]
		ifFalse: 
			[self nextPut: (length digitAt: 4)+192.
			self nextPut: (length digitAt: 3).
			self nextPut: (length digitAt: 2).
			self nextPut: (length digitAt: 1)].
	self nextPutAll: s.
	^s
!

nextWordsPutAll: aCollection
	"Write the argument a word-like object in big endian format on the receiver.
	May be used to write other than plain word-like objects (such as ColorArray)."
	^self nextPutAllWordArray: aCollection
!

originalContents

	^''		"used only to determine if we are byte-structured"
!

padToEndWith: aChar
	"We don't have pages, so we are at the end, and don't need to pad."
!

position

	^self bufferStream position + positionThusFar
!

reallyClose

	self flushBuffer.
	gZipStream close.
!

reopen

	"ignore"
!

setFileTypeToObject

	"ignore"
!

setToEnd

	"ignore"
!

skip: aNumber

	^self bufferStream skip: aNumber
!

timeStamp
	"Append the current time to the receiver as a String."
	self bufferStream nextChunkPut:	"double string quotes and !!s"
		(String streamContents: [:s | Smalltalk timeStamp: s]) printString.
	self bufferStream cr
!

trailer

	"ignore"
! !

!GZipSurrogateStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/compression/GZipSurrogateStream.st,v 1.1 2004/10/01 08:19:17 cg Exp $'
! !
