"{ Package: 'stx:goodies/constraints/cassovary' }"

View subclass:#ClDemoWindow
	instanceVariableNames:'shell main form da gc gcw buffer grabPoint dragPoint cs
		errorPixel editX editY'
	classVariableNames:''
	poolDictionaries:'CwConstants CgConstants ClConstants'
	category:'Constraints-Cassovary-Demos'
!


!ClDemoWindow class methodsFor:'uncategorized public'!

makeDemos

    "statements to build various demos"

    ClBoundedLineDemo new open.
    ClElasticBoundedLineDemo new open.
    ClQuadDemo new open.
    ClMidpointDemo new open.
    ClBoundedQuadDemo new open.
    ClCrossedQuadDemo new open.
    ClElasticQuadDemo new open.
    ClTreeDemo1 new open.
    ClTreeDemo2 new open.
    ClBoxesDemo new open.
    ClSequenceDiagramDemo1 new open.
    (ClBinaryTreeDemo height: 4) open.
!

new

    ^super new initialize
! !

!ClDemoWindow methodsFor:'uncategorized public'!

beginManipulation
    "ACTION
        This method is called when a constrainable point is starting to
        be manipulated, i.e., upon mouse down.  The constrainable
        point is stored in the 'dragPoint' instance variable, and the
        corresponding plan for moving that point is stored in the 'plan'
        instance variable.

        The order of messages is:
            1. #beginManipulation
            2-N. #manipulationMoveTo:
            N+1. #endManipulation
    "

    self constraintPointsDo: [:p |
        (dragPoint == p) ifTrue: [
            cs addEditVar: p x strength: ClStrength strong.
            cs addEditVar: p y strength: ClStrength strong.
            cs beginEdit.
            ^nil]].

     "drag point not found"
    self error

    "Modified: / 29-08-2010 / 12:59:21 / cg"
!

button1Motion: aWidget clientData: clientData callData: callData

    dragPoint isNil ifTrue: [^self].
    self manipulationMoveTo: callData point
!

buttonMotion:mask x:x y:y
    mask == 0 ifTrue:[
        self pointerMotion:self clientData:nil callData:(XmCallData new point:(x@y)).
        ^ self.
    ].
    self button1Motion:self clientData:nil callData:(XmCallData new point:(x@y)).

    "Modified: / 29-08-2010 / 13:48:18 / cg"
!

buttonPress: aWidget clientData: clientData callData: callData

    grabPoint isNil ifTrue: [^self].
    self drawGrabBox.
    dragPoint := grabPoint.
    grabPoint := nil.

    self beginManipulation
!

buttonPress:button x:x y:y
    self buttonPress:self "aWidget" clientData:nil "clientData" callData:(XmCallData new point:(x@y))

    "Modified: / 29-08-2010 / 12:50:25 / cg"
!

buttonRelease: aWidget clientData: clientData callData: callData

    dragPoint isNil ifTrue: [^self].
    dragPoint := nil.

    self endManipulation
!

buttonRelease:button x:x y:y
    self buttonRelease:self "aWidget" clientData:nil "clientData" callData:(XmCallData new point:(x@y)).

    "Created: / 29-08-2010 / 12:24:36 / cg"
!

constraintPointsDo: aBlock
    "ACTION
        Evaluate this block for each of the constrainable points in this demo.
        Constrainable points are those that can be manipulated by the mouse.
        A constraint point is a Point object whose x and y instance variables
        hold instances of ClVariable.

    PARAMETERS
        aBlock <[:<Point of <ClVariable>> | ...]>
    "
!

createForm

    da := form createDrawingArea: 'da' argBlock: nil.
    da manageChild.
    da setValuesBlock: [:w |
        w
            bottomAttachment: XmATTACHFORM;
            topAttachment: XmATTACHFORM;
            rightAttachment: XmATTACHFORM;
            leftAttachment: XmATTACHFORM].
    da
        addCallback: XmNexposeCallback
            receiver: self
            selector: #exposeArea:clientData:callData:
            clientData: nil;
        addCallback: XmNresizeCallback
            receiver: self
            selector: #resizeArea:clientData:callData:
            clientData: nil;
        addEventHandler: Button1MotionMask
            receiver: self
            selector: #button1Motion:clientData:callData:
            clientData: nil;
        addEventHandler: ButtonPressMask
            receiver: self
            selector: #buttonPress:clientData:callData:
            clientData: nil;
        addEventHandler: ButtonReleaseMask
            receiver: self
            selector: #buttonRelease:clientData:callData:
            clientData: nil;
        addEventHandler: PointerMotionMask
            receiver: self
            selector: #pointerMotion:clientData:callData:
            clientData: nil
!

createFormSTX
    buffer := gc := Form extent:self extent depth:24 onDevice:Screen current.

    "Created: / 28-08-2010 / 18:05:48 / cg"
    "Modified: / 29-08-2010 / 13:54:40 / cg"
!

createWindow
    Smalltalk isSmalltalkX ifTrue:[
        self createWindowSTX
    ] ifFalse:[
        self createWindowVAge
    ]

    "Modified: / 28-08-2010 / 18:03:31 / cg"
!

createWindowSTX
    self origin:0.0@0.0 corner:1.0@1.0.

    main := StandardSystemView new.
    main add:self.
    main extent:self initialExtent.
    main label:self title.

    self createFormSTX

    "Created: / 28-08-2010 / 18:02:45 / cg"
!

createWindowVAge

    shell :=
        CwTopLevelShell
            createApplicationShell: 'shell'
            argBlock: [:w |
                w
                    title: self title;
                    width: self initialExtent x;
                    height: self initialExtent y].
    main := shell createMainWindow: 'main' argBlock: nil.
    main manageChild.
    form := main createForm: 'form' argBlock: nil.
    form manageChild.
    shell
        addCallback: XmNdestroyCallback
        receiver: self
        selector: #destroyWindow:clientData:callData:
        clientData: nil.
    self createForm

    "Created: / 28-08-2010 / 18:02:54 / cg"
!

destroy
    "ACTION
        This method is called when the window is closing.
    "
!

destroyWindow: aWidget clientData: clientData callData: callData

    gc isNil
        ifFalse: [
            gc freeGC.
            gc := nil].
    gcw isNil
        ifFalse: [
            gcw freeGC.
            gcw := nil].
    buffer isNil
        ifFalse: [
            buffer freePixmap.
            buffer := nil].
    self destroy
!

drawArea
    "ACTION
        Draw the demo.
    "
!

drawBackground
    "ACTION
        Draw the background of the demo.
    "

    Smalltalk isSmalltalkX ifTrue:[
        buffer paint:(Color white).
        buffer fillRectangleX: 0 y: 0 width: buffer width height: buffer height
    ] ifFalse:[
        gc setForeground: 15.
        buffer fillRectangle: gc x: 0 y: 0 width: buffer width height: buffer height
    ]

    "Modified: / 28-08-2010 / 18:21:42 / cg"
!

drawGrabBox

    | r pv |

    pv := grabPoint x value rounded @ grabPoint y value rounded.
    r := (pv - self grabThreshold) corner: (pv + self grabThreshold).
    Smalltalk isSmalltalkX ifTrue:[ 
        self
            lineWidth: 3;
            paint:(Color red).
        self displayRectangleX: r origin x y: r origin y width: r width height: r height.
    ] ifFalse:[
        gcw
            setFunction: GXhighlight;
            setLineAttributes: 3 lineStyle: LineSolid capStyle: CapButt joinStyle: JoinMiter;
            setForeground: 0.
        da window drawRectangle: gcw x: r origin x y: r origin y width: r width height: r height.
        gcw setFunction: GXcopy
    ]

    "Modified: / 29-08-2010 / 12:57:07 / cg"
!

endManipulation
    "ACTION
        This method is called when a constrainable point is ending
        being manipulated, i.e., upon mouse up.  The constrainable
        point is stored in the 'dragPoint' instance variable, and the
        corresponding plan for moving that point is stored in the 'plan'
        instance variable.

        The order of messages is:
            1. #beginManipulation
            2-N. #manipulationMoveTo:
            N+1. #endManipulation
    "
    cs endEdit.
    self redraw.
!

exposeArea: aWidget clientData: clientData callData: callData

    buffer isNil ifTrue: [self resizeArea: aWidget clientData: clientData callData: nil].
    Smalltalk isSmalltalkX ifTrue:[
        self copyFrom:buffer
            x:0 y:0 toX:0 y:0 width:buffer width height:buffer height
    ] ifFalse:[
        buffer
            copyArea: aWidget window
            gc: gcw
            srcX: 0
            srcY: 0
            width: buffer width
            height: buffer height
            destX: 0
            destY: 0.
    ].
    grabPoint := nil

    "Modified: / 28-08-2010 / 18:31:39 / cg"
!

grabThreshold

    ^7 @ 7
!

initialExtent
    "ACTION
        Return the initial size for the overall window.
    "

    ^300 @ 300
!

initialize
    "ACTION
        Initialize the demo structures before the widgets are created.
    "

    super initialize.

    cs := ClSimplexSolver new.
    errorPixel := nil.

    Smalltalk isSmalltalkX ifTrue:[
        self enableMotionEvents
    ]

    "Modified: / 29-08-2010 / 12:53:37 / cg"
!

manipulationMoveTo: aPoint
    "ACTION
        This method is called when a constrainable point is manipulated
        using the mouse, i.e., during mouse motion.  aPoint is the
        location of the mouse and thus the new requested location of
        the constrainable point.  The constrainable
        point is stored in the 'dragPoint' instance variable, and the
        corresponding plan for moving that point is stored in the 'plan'
        instance variable.

        The order of messages is:
            1. #beginManipulation
            2-N. #manipulationMoveTo:
            N+1. #endManipulation

    PARAMETERS
        aPoint <Point of <Number>>
    "
    cs suggestValue: dragPoint x newValue: aPoint x asFloat.
    cs suggestValue: dragPoint y newValue: aPoint y asFloat.
    cs resolve.
    self redraw
!

open

    self createWindow.
    Smalltalk isSmalltalkX ifTrue:[
        main open
    ] ifFalse:[
        shell realizeWidget
    ].

    "Modified: / 28-08-2010 / 18:10:49 / cg"
!

pointerMotion: aWidget clientData: clientData callData: callData

    | r |

self redraw.
    dragPoint isNil
        ifTrue: [
            self constraintPointsDo: [:p | | pv th |
                th := self grabThreshold * 2.
                pv := p x value rounded @ p y value rounded.
                (((pv - th) corner: (pv + th)) containsPoint: callData point)
                    ifTrue: [
                        grabPoint == p ifTrue: [^self].
                        grabPoint notNil ifTrue: [self drawGrabBox].
                        grabPoint := p.
                        self drawGrabBox.
                        ^self].
                grabPoint notNil ifTrue: [self drawGrabBox].
                grabPoint := nil]]

    "Modified: / 29-08-2010 / 12:59:46 / cg"
!

redraw

    self drawBackground.
    self drawArea.
    self exposeArea: da clientData: nil callData: nil
!

resizeArea: aWidget clientData: clientData callData: callData

    buffer isNil
        ifFalse: [
            buffer freePixmap.
            buffer := nil].
    gc isNil
        ifFalse: [
            gc freeGC.
            gc := nil].
    gcw isNil ifTrue: [gcw := aWidget window createGC: None values: nil].
    buffer :=
        aWidget window createPixmap: aWidget width height: aWidget height depth: aWidget window depth.
    gc := buffer createGC: None values: nil.
    self drawBackground.
    self drawArea
!

setForegroundColor: c

    gc setForeground: (errorPixel isNil ifTrue: [c] ifFalse: [errorPixel])

!

title
    "ACTION
        Return the title for the overall window.
    "

    ^'demo'
! !

!ClDemoWindow class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/constraints/cassovary/ClDemoWindow.st,v 1.1 2010/09/05 16:00:07 cg Exp $'
! !
