"{ Package: 'stx:goodies/constraints/cassovary' }"

ClAbstractVariable subclass:#ClVariable
	instanceVariableNames:'value'
	classVariableNames:''
	poolDictionaries:''
	category:'Constraints-Cassovary-Kernel'
!


!ClVariable class methodsFor:'uncategorized public'!

newWith: val
    "ACTION
        Create a new variable with val as its initial value.

    PARAMETERS
        val <Object>

    RETURNS
        new
    "

    ^self new
        value: val;
        yourself
!

newWith: val name: n
    "ACTION
        Create a new variable with val as its initial value and name n.

    PARAMETERS
        val <Object>
        n <String>

    RETURNS
        new
    "

    ^self new
        value: val;
        name: n;
        yourself
! !

!ClVariable methodsFor:'double dispatching'!

differenceFromFloat: aNumber
    ^ self subtractFrom:aNumber

    "Modified: / 28-08-2010 / 14:39:34 / cg"
!

differenceFromInteger: aNumber
    ^ self subtractFrom:aNumber

    "Created: / 28-08-2010 / 14:40:11 / cg"
!

productFromFloat: aNumber
    ^ self * aNumber

    "Created: / 28-08-2010 / 14:39:13 / cg"
!

productFromInteger: aNumber
    ^ self * aNumber

    "Created: / 28-08-2010 / 14:39:50 / cg"
! !

!ClVariable methodsFor:'uncategorized public'!

* expr
    ^self asLinearExpression * expr
!

+ expr
    ^self asLinearExpression + expr
!

- expr
    ^self asLinearExpression - expr
!

/ expr
    ^self asLinearExpression / expr
!

@ obj
    "ACTION
        A convenience method for creating point pairs of
        constrainable variables.  Mostly used in graphic
        constraints where the x and y dimensions are
        constrained separately.

    PARAMETERS
        obj <Object>

    RETURNS
        <Point of <self> obj>
    "

    ^Point x: self y: obj
!

asLinearExpression
    "ACTION
        Coerce this variable to be a linear expression.

    RETURNS
        <ClLinearExpression>
    "
        | expr |
    expr := ClLinearExpression new.
    expr terms at: self put: 1.0.
    ^expr
!

cnEqual: expr
    "ACTION
        Return a linear constraint self=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearEquation>
    "
    ^self cnEqual: expr strength: ClStrength required weight: 1.0

!

cnEqual: expr strength: s
    "ACTION
        Return a linear constraint self=expr of strength s and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearEquation>
    "

    ^self cnEqual: expr strength: s weight: 1.0
!

cnEqual: expr strength: s weight: w
    "ACTION
        Return a linear constraint self=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearEquation new.
    cn expression: self-expr; strength: s; weight: w.
    ^cn
!

cnGEQ: expr
    "ACTION
        Return a linear constraint self>=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnGEQ: expr strength: ClStrength required weight: 1.0
!

cnGEQ: expr strength: s
    "ACTION
        Return a linear constraint self>=expr of strength s and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnGEQ: expr strength: s weight: 1.0
!

cnGEQ: expr strength: s weight: w
    "ACTION
        Return a linear constraint self>=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearInequality new.
    cn expression: self-expr; strength: s; weight: w.
    ^cn
!

cnLEQ: expr
    "ACTION
        Return a linear constraint self<=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnLEQ: expr strength: ClStrength required weight: 1.0

!

cnLEQ: expr strength: s
    "ACTION
        Return a linear constraint self<=expr of strength s and weight 1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnLEQ: expr strength: s weight: 1.0

!

cnLEQ: expr strength: s weight: w
    "ACTION
        Return a linear constraint self<=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearInequality new.
    cn expression: expr-self; strength: s; weight: w.
    ^cn
!

divFrom: aNumber
    "ACTION
        We should return a new linear expression (aNumber/self) -- but since the result must be linear, 
        this is always an error.

    RETURNS
        <ClLinearExpression>
    "
    ExCLNonlinearExpression signal
!

isExternal
    "ACTION
        Return true if this a variable known outside the solver.  (We need to give such variables a value after solving
        is complete.)

    RETURNS
        <Boolean>
    "

    ^true
!

isPivotable
    "ACTION
        Return true if we can pivot on this variable.

    RETURNS
        <Boolean>
    "

    ^false
!

isRestricted
    "ACTION
        Return true if this is a restricted (or slack) variable.  Such variables are constrained to be 
        non-negative and occur only internally to the simplex solver.

    RETURNS
        <Boolean>
    "

    ^false
!

printOn: aStream
    "ACTION
        See superclass method.
    PARAMETERS
        aStream <WriteStream on <String>>
    "

    super printOn: aStream.
    aStream nextPutAll: '['.  self value printOn: aStream.  aStream nextPutAll: ']'
!

subtractFrom: aNumber
    "ACTION
        Return a new linear expression (aNumber-self).

    RETURNS
        <ClLinearExpression>
    "
    ^ -1.0*self + aNumber
!

value
    "ACTION
        Return the current value I hold.

    RETURNS
        <Number>
    "

    ^value
!

value: v
    "ACTION
        Set my value.

    PARAMETERS
        v <Number>
    "

    value := v
! !

!ClVariable class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/constraints/cassovary/ClVariable.st,v 1.1 2010/09/05 16:02:53 cg Exp $'
! !
