"{ Package: 'stx:goodies/constraints/cassovary' }"!

!Float methodsFor:'cassovary'!

approxNonNegative
    ^self > (0.0 - ClSimplexSolver epsilon)
! !

!Float methodsFor:'cassovary'!

clApprox: x
    "ACTION
        Test whether I am approximately equal to the argument.

    PARAMETERS
        x <Float>

    RETURNS
        <Boolean>
    "

    | epsilon |

    epsilon := ClSimplexSolver epsilon.
    self = 0.0 ifTrue: [^x abs < epsilon].
    x = 0.0 ifTrue: [^self abs < epsilon].
    ^(self - x) abs < (self abs * epsilon)
! !

!Float methodsFor:'cassovary'!

clApproxZero
    "ACTION
        Test whether I am approximately zero.

    RETURNS
        <Boolean>
    "

    ^self clApprox: 0.0
! !

!Number methodsFor:'cassovary'!

asLinearExpression
    "ACTION
        Coerce this number to a linear expression.

    RETURNS
        <ClLinearExpression>
    "
        | expr |
    expr := ClLinearExpression new.
    expr constant: self asFloat.
    ^expr
! !

!Number methodsFor:'cassovary'!

cnEqual: expr
    "ACTION
        Return a linear constraint self=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearEquation>
    "
    ^self cnEqual: expr strength: ClStrength required weight: 1.0

! !

!Number methodsFor:'cassovary'!

cnEqual: expr strength: s
    "ACTION
        Return a linear constraint self=expr of strength s and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearEquation>
    "

    ^self cnEqual: expr strength: s weight: 1.0
! !

!Number methodsFor:'cassovary'!

cnEqual: expr strength: s weight: w
    "ACTION
        Return a linear constraint self=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearEquation new.
    cn expression: self-expr; strength: s; weight: w.
    ^cn
! !

!Number methodsFor:'cassovary'!

cnGEQ: expr
    "ACTION
        Return a linear constraint self>=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnGEQ: expr strength: ClStrength required weight: 1.0
! !

!Number methodsFor:'cassovary'!

cnGEQ: expr strength: s
    "ACTION
        Return a linear constraint self>=expr of strength s and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnGEQ: expr strength: s weight: 1.0
! !

!Number methodsFor:'cassovary'!

cnGEQ: expr strength: s weight: w
    "ACTION
        Return a linear constraint self>=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearInequality new.
    cn expression: self-expr; strength: s; weight: w.
    ^cn
! !

!Number methodsFor:'cassovary'!

cnLEQ: expr
    "ACTION
        Return a linear constraint self<=expr with strength=required and weight=1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnLEQ: expr strength: ClStrength required weight: 1.0

! !

!Number methodsFor:'cassovary'!

cnLEQ: expr strength: s
    "ACTION
        Return a linear constraint self<=expr of strength s and weight 1.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        s <ClStrength>

    RETURNS
        <ClLinearInequality>
    "
    ^self cnLEQ: expr strength: s weight: 1.0

! !

!Number methodsFor:'cassovary'!

cnLEQ: expr strength: s weight: w
    "ACTION
        Return a linear constraint self<=expr of strength s and weight w.

    PARAMETERS
        expr <ClLinearExpression | ClVariable | Number>
        st <ClStrength>
        w <Float>

    RETURNS
        <ClLinearEquation>
    "
        | cn |
    cn := ClLinearInequality new.
    cn expression: expr-self; strength: s; weight: w.
    ^cn
! !

!Object methodsFor:'cassovary'!

clApprox: x
    "ACTION
        Test whether I am approximately equal to the argument.

    PARAMETERS
        x <Object>

    RETURNS
        <Boolean>

    NOTES
        The default is just an equality test."

    ^self = x
! !

!Object methodsFor:'cassovary'!

isSymbolicWeight
    ^false
! !

