"{ Package: 'stx:goodies/dialogs' }"

SimpleDialog subclass:#FileSelectionDialog
	instanceVariableNames:'os allowDirectoryResult allowFileResult directoryStrings
		directoryString parentDirectories currentParent
		currentDirectoryString currentDirectory directoryContents tail
		masks mask result numberOfSubdirectories folderIcon labelOffset
		skipUpdates'
	classVariableNames:'UpDirectory Folder NewDirectory'
	poolDictionaries:''
	category:'Views-DialogBoxes'
!

!FileSelectionDialog class methodsFor:'documentation'!

documentation
"This class provides a dialog that allows the user to browse the filesystem to select a file or directory.

Instance Variables:

  os                    <Symbol>                current operating system
  allowDirectoryResult  <Boolean>               true if a directory may be selected as the result of the dialog
  directories           <List of: Filename>     list of directories available for browsing.  A string may represent a valid filename, or a filename concatenated with some label describing the filename.  The label must be trimmed before using the filename.
  directory             <Filename>              closest match from directories list to the current directory being browsed
  currentDirectory      <Filename>              the current directory being browsed
  directoryContents     <List of: Strings>      list of directories / files in directory
  tail                  <String>                filename or mask
  masks                 <List of: String>       list of file masks to filter directoryContents
  mask                  <String>                selected mask
  result                <String>                return value
"
! !

!FileSelectionDialog class methodsFor:'instance creation'!

allowDirectoryResult: dirBoolean allowFileResult: fileBoolean

	^self allowDirectoryResult: dirBoolean
		allowFileResult: fileBoolean
		initialDirectory: Filename defaultDirectoryName
!

allowDirectoryResult: dirBoolean allowFileResult: fileBoolean initialDirectory: dirString

	^self allowDirectoryResult: dirBoolean
		allowFileResult: fileBoolean
		initialDirectory: dirString
		masks: self defaultMaskList
!

allowDirectoryResult: dirBoolean allowFileResult: fileBoolean initialDirectory: dirString masks: maskList

	| result |
	result := ValueHolder new.
	(self new
		allowDirectoryResult: dirBoolean
		allowFileResult: fileBoolean
		directory: dirString
		masks: maskList
		result: result) open.
	^result value
!

allowDirectoryResult: dirBoolean allowFileResult: fileBoolean masks: maskList

	^self allowDirectoryResult: dirBoolean
		allowFileResult: fileBoolean
		initialDirectory: Filename defaultDirectoryName
		masks: maskList
! !

!FileSelectionDialog class methodsFor:'class initialization'!

initialize
"FileSelectionDialog initialize"

        Smalltalk isSmalltalkX ifTrue:[
            Folder := self folderImage.
            UpDirectory := self upDirectoryImage.
            NewDirectory := self newDirectoryImage
        ] ifFalse:[
            Folder := OpaqueImage
                    figure: self folderImage
                    shape: self folderMask.
            UpDirectory := OpaqueImage
                    figure: self upDirectoryImage
                    shape: self upDirectoryMask.
            NewDirectory := OpaqueImage
                    figure: self newDirectoryImage
                    shape: self newDirectoryMask
        ]
! !

!FileSelectionDialog class methodsFor:'defaults'!

defaultMaskList

	^#('*') asList
!

labelPrefix

	^'  < '
!

labelSuffix

	^' >'
! !

!FileSelectionDialog class methodsFor:'interface specs'!

windowSpec
        "UIPainter new openOnClass: self andSelector: #windowSpec"

        <resource: #canvas>
        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #min: #(#Point 575 360 ) 
                        #bounds: #(#Rectangle 0 0 575 360 ) 
                        #isEventDriven: true ) 
                #component: 
                #(#SpecCollection 
                        #collection: #(
                                #(#LabelSpec 
                                        #layout: #(#AlignmentOrigin -10 0.166667 0 0.0625 1 0.5 ) 
                                        #label: 'Look in:' ) 
                                #(#ActionButtonSpec 
                                        #layout: #(#LayoutFrame 0 0.85 10 0 -10 0.925 0 0.1 ) 
                                        #name: #upDirectory 
                                        #model: #upDirectory 
                                        #label: #upDirectoryImage 
                                        #hasCharacterOrientedLabel: false ) 
                                #(#ActionButtonSpec 
                                        #layout: #(#LayoutFrame 0 0.925 10 0 -10 1 0 0.1 ) 
                                        #name: #newDirectory 
                                        #model: #newDirectory 
                                        #label: #newDirectoryImage 
                                        #hasCharacterOrientedLabel: false ) 
                                #(#SequenceViewSpec 
                                        #layout: #(#LayoutFrame 10 0 0 0.225 -10 1 0 0.775 ) 
                                        #name: #directoryContents 
                                        #model: #directoryContents 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #doubleClickSelector: #directoryContentsDoubleClick 
                                                #requestValueChangeSelector: 
                                                #validateFileListSelection: ) 
                                        #useModifierKeys: true 
                                        #selectionType: #highlight ) 
                                #(#LabelSpec 
                                        #layout: #(#AlignmentOrigin -10 0.25 0 0.8375 1 0.5 ) 
                                        #label: 'File name:' ) 
                                #(#LabelSpec 
                                        #layout: #(#AlignmentOrigin -10 0.25 0 0.9375 1 0.5 ) 
                                        #label: 'File mask:' ) 
                                #(#InputFieldSpec 
                                        #layout: #(#LayoutFrame 0 0.25 0 0.8 0 0.75 0 0.875 ) 
                                        #name: #tail 
                                        #model: #tail ) 
                                #(#ComboBoxSpec 
                                        #layout: #(#LayoutFrame 0 0.25 0 0.9 0 0.75 0 0.975 ) 
                                        #name: #mask 
                                        #model: #mask 
                                        #comboList: #masks ) 
                                #(#ActionButtonSpec 
                                        #layout: #(#LayoutFrame 20 0.75 0 0.8 -10 1 0 0.875 ) 
                                        #name: #accept 
                                        #model: #accept 
                                        #label: 'OK' 
                                        #isDefault: true 
                                        #defaultable: true ) 
                                #(#ActionButtonSpec 
                                        #layout: #(#LayoutFrame 20 0.75 0 0.9 -10 1 0 0.975 ) 
                                        #name: #cancel 
                                        #model: #cancel 
                                        #label: 'Cancel' 
                                        #defaultable: true ) 
                                #(#ComboBoxSpec 
                                        #layout: #(#LayoutFrame 0 0.166667 10 0 -10 0.85 0 0.1 ) 
                                        #name: #directoryString 
                                        #model: #directoryString 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #requestValueChangeSelector: 
                                                #volumeExists: ) 
                                        #type: #string 
                                        #comboList: #directoryStrings ) 
                                #(#InputFieldSpec 
                                        #layout: #(#LayoutFrame 10 0 0 0.125 -10 1 0 0.2 ) 
                                        #name: #currentDirectoryString 
                                        #model: #currentDirectoryString 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #requestFocusOutSelector: 
                                                #validateCurrentDirectoryExit: 
                                                #requestValueChangeSelector: 
                                                #validateCurrentDirectory: ) ) ) ) )
! !

!FileSelectionDialog class methodsFor:'resources'!

folder

	^Folder
!

folderImage
    "This resource specification was automatically generated
     by the ImageEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the ImageEditor may not be able to read the specification."

    "
     self folderImage inspect
     ImageEditor openOnClass:self andSelector:#folderImage
    "

    <resource: #image>

    ^Icon
        constantNamed:#'FileSelectionDialog class folderImage'
        ifAbsentPut:[(Depth2Image new) width: 15; height: 14; photometric:(#palette); bitsPerSample:(#(2 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'@@@@@@@@@@@@@@@@A&P@@@@@@@@&Y&Y FY&Y$BY&Y&@Y&Y&PI&Y&XA&Y&Y@&Y&Y FY&Y$@@@@@@b') ; colorMapFromArray:#[0 0 0 255 255 0 255 255 255]; mask:((Depth1Image new) width: 15; height: 14; photometric:(#blackIs0); bitsPerSample:(#(1 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'@@D@@S8A_0G??_???????????????????????0@a') ; yourself); yourself]
!

folderImagePC
	"UIMaskEditor new openOnClass: self andSelector: #folderImagePC"

	<resource: #image>
	^CachedImage on: (Image extent: 15@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue yellow; at: 3 put: ColorValue white; yourself)) usingBits: #[170 170 170 168 170 170 170 168 170 170 170 168 170 170 170 168 160 2 170 168 134 100 170 168 0 0 0 8 38 102 102 96 25 153 153 144 38 102 102 96 25 153 153 144 38 102 102 96 25 153 153 144 38 102 102 96 25 153 153 144 0 0 0 0])
!

folderMask
	"UIMaskEditor new openOnClass: self andSelector: #folderMask"

	<resource: #image>
	^CachedImage on: (Image extent: 15@14 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[0 0 0 0 0 0 0 0 62 0 0 0 127 0 0 0 255 252 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0])
!

folderMaskPC
	"UIMaskEditor new openOnClass: self andSelector: #folderMaskPC"

	<resource: #image>
	^CachedImage on: (Image extent: 15@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 62 0 0 0 127 0 0 0 255 252 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0])
!

newDirectory

	^NewDirectory
!

newDirectoryImage
    "This resource specification was automatically generated
     by the ImageEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the ImageEditor may not be able to read the specification."

    "
     self newDirectoryImage inspect
     ImageEditor openOnClass:self andSelector:#newDirectoryImage
    "

    <resource: #image>

    ^Icon
        constantNamed:#'FileSelectionDialog class newDirectoryImage'
        ifAbsentPut:[(Depth2Image new) width: 15; height: 12; photometric:(#palette); bitsPerSample:(#(2 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'@@@@@@@@@@@@@@@@A&@@@@@@@@@&Y&@@FY&P@BY&X@@Y&Y@@I&Y @A&Y$@@@@@@@') ; colorMapFromArray:#[0 0 0 255 255 0 255 255 255]; mask:((Depth1Image new) width: 15; height: 12; photometric:(#blackIs0); bitsPerSample:(#(1 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'@BDAAS2)_%G?2??1?>''?9_?!!?>G?8_?!!') ; yourself); yourself]
!

newDirectoryMask
	"UIMaskEditor new openOnClass: self andSelector: #newDirectoryMask"

	<resource: #image>
	^CachedImage on: (Image extent: 15@12 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[0 32 0 0 1 4 0 0 60 168 0 0 126 80 0 0 255 202 0 0 255 240 0 0 255 232 0 0 255 228 0 0 255 224 0 0 255 224 0 0 255 224 0 0 255 224 0 0])
!

upDirectory

	^UpDirectory
!

upDirectoryImage
    "This resource specification was automatically generated
     by the ImageEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the ImageEditor may not be able to read the specification."

    "
     self upDirectoryImage inspect
     ImageEditor openOnClass:self andSelector:#upDirectoryImage
    "

    <resource: #image>

    ^Icon
        constantNamed:#'FileSelectionDialog class upDirectoryImage'
        ifAbsentPut:[(Depth2Image new) width: 15; height: 13; photometric:(#palette); bitsPerSample:(#(2 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'@@@@@@Y$@@@@@@@@I&Y&XA&I&Y@&@&Y F@BY$BYFY&@Y"Y&PI$@BXA&Y&Y@&Y&Y @@@@@@@a') ; colorMapFromArray:#[0 0 0 255 255 0 255 255 255]; mask:((Depth1Image new) width: 15; height: 13; photometric:(#blackIs0); bitsPerSample:(#(1 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'O E?@_?=??????????????????????????<b') ; yourself); yourself]
!

upDirectoryMask
	"UIMaskEditor new openOnClass: self andSelector: #upDirectoryMask"

	<resource: #image>
	^CachedImage on: (Image extent: 15@13 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[62 0 0 0 127 0 0 0 255 252 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0 255 254 0 0])
! !

!FileSelectionDialog methodsFor:'actions'!

newDirectory

	| answer newDir |
	answer := Dialog request: 'Please enter the name of the new directory:' initialAnswer: ''.
	answer isEmpty ifTrue: [^self].
	newDir := currentDirectory construct: answer.
	newDir exists
		ifTrue:
			[^(Dialog confirm: 'A file or directory already exists by that name.  Try again?')
				ifTrue: [self newDirectory]
				ifFalse: [self]]
		ifFalse:
			[newDir makeDirectory.
			self getDirectoryContents.]
!

upDirectory

	| up |
	(up := currentDirectory directory) = currentDirectory
		ifFalse:
			[currentDirectory := up.
			currentDirectoryString value: up asString]
! !

!FileSelectionDialog methodsFor:'aspects'!

currentDirectoryString

	^currentDirectoryString
!

directoryContents

	^directoryContents
!

directoryString

	^directoryString
!

directoryStrings

	^directoryStrings
!

mask

	^mask
!

masks

	^masks
!

parentDirectories

	^parentDirectories
!

result

	^result
!

tail

	^tail
! !

!FileSelectionDialog methodsFor:'initialize-release'!

allowDirectoryResult: dirBoolean allowFileResult: fileBoolean directory: dirStr masks: maskList result: resultHolder

	skipUpdates := false.
	os := OSHandle currentOS.
	(os == #win32 or: [os == #os2])
		ifTrue:
			[folderIcon := OpaqueImage
				figure: self class folderImagePC
				shape: self class folderMaskPC.
			labelOffset := -4]
		ifFalse:
			[folderIcon := self class folder.
			labelOffset := -2].
	directoryStrings := List new asValue.
	parentDirectories := List new.
	self setDirectories.
	masks := maskList asValue.
	allowDirectoryResult := dirBoolean.
	allowFileResult := fileBoolean.
	directoryString := String new asValue.
	directoryString onChangeSend: #changedDirectory to: self.
	result := resultHolder.
	mask := maskList first asValue.
	mask onChangeSend: #changedMask to: self.
	currentDirectoryString := String new asValue.
	currentDirectoryString onChangeSend: #changedCurrentDirectory to: self.
	directoryContents := SelectionInList new.
	directoryContents addDependent: self.
	tail := String new asValue.
	tail onChangeSend: #changedTail to: self.
	accept onChangeSend: #acceptResult to: self.
	currentDirectoryString value:
		(dirStr isNil
			ifTrue: [(currentDirectory := Filename defaultDirectory) asString]
			ifFalse: [(currentDirectory := dirStr asFilename). dirStr])
!

getValidDriveLetters

	| drive |
	#('A:\' 'B:\' 'C:\' 'D:\' 'E:\' 'F:\' 'G:\' 'H:\' 'I:\' 'J:\' 
'K:\' 'L:\' 'M:\' 'N:\' 'O:\' 'P:\' 'Q:\' 'R:\' 'S:\' 'T:\' 'U:\' 'V:\' 'W:\' 'X:\' 'Y:\' 'Z:\')
		do: [:driveString |  (drive := self validFilename: driveString) notNil ifTrue: [directoryStrings value add: drive asString. parentDirectories add: drive]]
!

setDirectories

	| label vols drive |
	(os == #win32 or: [os == #os2])
		ifTrue:
			[self getValidDriveLetters.
			vols := Filename volumes.
			1 to: vols size do: [:index |
				drive := directoryStrings value at: index.
				label := vols at: index.
				(label sameAs: drive)
					ifFalse:
						[drive := drive, self class labelPrefix, label, self class labelSuffix.
						directoryStrings value at: index put: drive]]].

	(os == #unix)
		ifTrue:
			[directoryStrings value add: ('/', self class labelPrefix, 'ROOT', self class labelSuffix) asFilename.
			self registerSystemVariable: 'HOME'].

	self registerSystemVariable: 'VISUALWORKS'.
	parentDirectories add: Filename defaultDirectory.
	directoryStrings value add:
		Filename defaultDirectoryName,
		self class labelPrefix,
		Filename currentDirectory asString,
		self class labelSuffix
! !

!FileSelectionDialog methodsFor:'interface opening'!

postBuildWith: aBuilder

        | widget label |
        super postBuildWith: aBuilder.
        widget := (builder componentAt: #directoryContents) widget.
"/        widget
"/                measureWidth: true;
"/                selectedVisualBlock:
"/                        [:view :index |
"/                        | reversingWrapper |
"/                        label := LabelAndIcon with: (view sequence at: index) offset: 4@labelOffset.
"/                        numberOfSubdirectories < index
"/                                ifFalse: [label icon: folderIcon].
"/                        reversingWrapper := ReversingWrapper on: label.
"/                        reversingWrapper reverse setValue: true.
"/                        BoundedWrapper on: reversingWrapper];
"/                visualBlock:
"/                        [:view :index |
"/                        label := LabelAndIcon with: (view sequence at: index) offset: 4@labelOffset.
"/                        numberOfSubdirectories < index
"/                                ifFalse: [label icon: folderIcon].
"/                        BoundedWrapper on: label].
        builder window label: 'Please select a ',
                (allowDirectoryResult
                        ifTrue:
                                [allowFileResult
                                        ifTrue: ['file or directory...']
                                        ifFalse: ['directory...']]
                        ifFalse:
                                ['file...'])
! !

!FileSelectionDialog methodsFor:'notification'!

directoryContentsDoubleClick

	| filename |
	filename := currentDirectory construct: tail value.
	filename isDirectory
		ifTrue:
			[currentDirectory := filename.
			currentDirectoryString value: filename asString]
		ifFalse:
			[^self accept value: true]
! !

!FileSelectionDialog methodsFor:'private'!

acceptResult

	| selection isDirectory trigger |
	trigger := (builder componentAt: #accept) widget model.
	trigger value == true
		ifTrue:
			[self tailIsPattern ifTrue: [tail value: ''.  ^trigger value: false].
			selection := currentDirectory construct: tail value.
			selection exists
				ifTrue:
					[isDirectory := selection isDirectory.
					((isDirectory and: [allowDirectoryResult not]) or: [isDirectory not and: [allowFileResult not]])
						ifTrue:
							[Dialog warn: builder window label.
							^trigger value: false]].
			result value: selection asString]
!

getDirectoryContents

	| tailString pattern list dirList fileList filename tailIsPattern |
	tailString := tail value.
	pattern := (tailIsPattern := self tailIsPattern)
		ifTrue: [tail value]
		ifFalse: [mask value].
	dirList := List new.
	fileList := List new.
	list := currentDirectory filesMatching: pattern.
	list do:
		[:fileString |
		(filename := self validFilename: fileString) isNil
			ifFalse: [filename exists
				ifTrue: [filename isDirectory
					ifTrue: [dirList add: filename tail]
					ifFalse: [fileList add: filename tail]]]].
	numberOfSubdirectories := dirList size.
	list := dirList asSortedStrings asList.
	allowFileResult ifTrue: [list := list, fileList asSortedStrings asList].
	directoryContents list: list.
	directoryContents selection: tailString.
	tailIsPattern
		ifFalse: [directoryContents selectionIndex = 0 ifTrue: [tail value: String new]]
!

isDirectory: aString

	| filename |
	(filename := self validFilename: aString) isNil ifTrue: [^false].
	^[filename isDirectory]
		on: OSErrorHolder inappropriateOperationSignal, OSErrorHolder noPermissionsSignal
		do: [ :ex |  false]
!

nearestParentDirectoryOf: aDirectory
"Return the filename from parentDirectories represented by dirString, or the nearest parent thereof."

	| filename parent parentDir |
	(filename := aDirectory) = currentParent ifTrue: [^currentParent].
	[parent := parentDirectories
			detect: [:dir |  dir = filename]
			ifNone: [nil].
		parent isNil and: [(filename = (parentDir := filename directory)) not]]
		whileTrue: [filename := parentDir].
	^parent
!

registerSystemVariable: varString

	| value |
	(value := SystemUtils getEnvironmentVariable: varString) isEmpty
		ifTrue: [^self].
	parentDirectories add: value asFilename.
	directoryStrings value add: value, self class labelPrefix, varString, self class labelSuffix
!

tailIsPattern

	| tailString |
	tailString := tail value.
	^(tailString includes: $*) or: [tailString includes: $# ]
!

topComponentFor: filenameString

	| filename |
	filename := filenameString asFilename.
	filename isRelative
		ifTrue: [^self topComponentFor: Filename defaultDirectoryName].
	[filename = filename directory]
		whileFalse: [filename := filename directory].
	^filename asString
!

validFilename: aString

	^[aString asFilename]
		on: Filename errorReporter badArgumentsSignal
		do: [ :ex |  nil]
!

volumeFor: aString

	^[aString asFilename]
		on: Filename errorReporter badArgumentsSignal
		do: [ :ex |  nil]
! !

!FileSelectionDialog methodsFor:'updating'!

changedCurrentDirectory

	| parent |
	parent := self nearestParentDirectoryOf: currentDirectory.
	parent = currentParent
		ifFalse:
			[currentParent := parent.
			skipUpdates := true.
			directoryString value: (directoryStrings value at: (parentDirectories indexOf: parent)).
			skipUpdates := false].
	self getDirectoryContents
!

changedDirectory

	| dir dirString |
	skipUpdates ifTrue: [^self].
	dirString := directoryString value.
	currentDirectory = (dir := parentDirectories at: (directoryStrings value indexOf: dirString))
		ifFalse:
			[currentDirectory := dir.
			currentDirectoryString value: currentDirectory asString]
!

changedFileListSelection

	| selection |
	(selection := directoryContents selection) isNil
		ifFalse: [tail value = selection
			ifFalse: [tail value: selection]]
!

changedMask

	(masks value includes: mask value)
		ifFalse: [masks value add: mask value].
	self getDirectoryContents
!

changedTail

	| tailString |
	tailString := tail value.
	self tailIsPattern
		ifTrue:
			[(masks value includes: tailString) ifFalse: [masks value add: tailString].
			mask value = tailString ifFalse: [mask value: tailString].
			self getDirectoryContents]
		ifFalse:
			[directoryContents selection = tailString
				ifFalse: [directoryContents selection: tailString]]
!

update: anAspectSymbol with: aValue from: anObject

	super update: anAspectSymbol with: aValue from: anObject.
	anObject == directoryContents
		ifTrue:
			["anAspectSymbol == #list
				ifTrue: [self changedFileList]."
			anAspectSymbol == #selectionIndex
				ifTrue: [self changedFileListSelection]]
! !

!FileSelectionDialog methodsFor:'validation'!

validateCurrentDirectory: anInputBoxController

	| topComponent dirString |
	anInputBoxController hasEditValue
		ifFalse: [^false].
	dirString := anInputBoxController editValue.
	topComponent :=
		[self topComponentFor: dirString]
			on: Filename errorReporter badArgumentsSignal, Filename errorReporter invalidArgumentsSignal
			do: [ :ex |
				Dialog warn: 'Invalid file name, ', anInputBoxController editValue, '.  Try again.'.
				ex returnWith: nil].
	topComponent isNil ifTrue: [^false].
	directoryStrings value
		detect:
			[:dir | (self topComponentFor: dir) = topComponent]
		ifNone:
			[dirString asFilename exists
				ifTrue:
					[directoryStrings add: dirString.
					^true]
				ifFalse:
					[topComponent asFilename exists
						ifTrue:
							[directoryStrings add: dirString.
							^true]
						ifFalse: [^false]]].
	^true
!

validateCurrentDirectoryExit: anInputBoxController

	| topComponent dirString newDir |
	anInputBoxController hasEditValue
		ifFalse: [^false].
	dirString := anInputBoxController editValue.
	topComponent :=
		[self topComponentFor: dirString]
			on: Filename errorReporter badArgumentsSignal, Filename errorReporter invalidArgumentsSignal
			do: [ :ex |
				newDir := Dialog
					request: 'Invalid file name, ', anInputBoxController editValue, '.  Try again:'
					initialAnswer: dirString.
				newDir isEmpty
					ifTrue: 
						[anInputBoxController cancel; notAccepted.
						ex returnWith: nil]
					ifFalse:
						[ex retryUsing: [self topComponentFor: newDir]]].
	topComponent isNil ifTrue: [^false].
	directoryStrings value
		detect:
			[:dir | (self topComponentFor: dir) = topComponent]
		ifNone:
			[dirString asFilename exists
				ifTrue:
					[directoryStrings add: dirString.
					^true]
				ifFalse:
					[topComponent asFilename exists
						ifTrue:
							[directoryStrings add: dirString.
							^true]
						ifFalse: [^false]]].
	^true
!

validateFileListSelection: aSequenceController

	^(directoryContents selectionIndex = aSequenceController view targetIndex) not
!

volumeExists: aComboBoxButtonController

	| view selection filename |
	(builder componentAt: #cancel) widget controller isControlWanted
		ifTrue:
			[^true].
	view := aComboBoxButtonController view.
	selection := view displayContents asString.
	filename := (selection readStream upToAll: self class labelPrefix) asFilename.
	filename exists
		ifFalse:
			[Dialog warn: 'The selection is not accessible.  Perhaps the 
directory does not exist or a disk must be inserted.'.
			filename exists		"Check to see if the problem was fixed while the warning was displayed."
				ifFalse: [aComboBoxButtonController cancel; notAccepted.  ^false]].
	^true
! !

!FileSelectionDialog class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/dialogs/FileSelectionDialog.st,v 1.1 2002/02/19 10:51:42 cg Exp $'
! !
FileSelectionDialog initialize!
