"{ Package: 'stx:goodies/displayLists' }"

DisplayItem subclass:#DisplayLine
	instanceVariableNames:'width'
	classVariableNames:'LineBlt'
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayLine comment:'Instances of DisplayLine are used for visual lines in display lists.
Instance Variables:
	width		<Number> the object''s line width
Class Variables
	LineBlt		<BitBlt> cached BitBlt for faster display'
!


!DisplayLine class methodsFor:'instance creation'!

origin: oPoint corner: ePoint	
	"Answer an initialized instance."
	^self new origin: oPoint; corner: ePoint
!

origin: oPoint corner: ePoint	 color: color width: width
	"Answer an initialized instance."
	^self new origin: oPoint; corner: ePoint; color: color; width: width
!

origin: oPoint extent: ePoint	
	"Answer an initialized instance."
	^self new origin: oPoint; extent: ePoint
!

origin: oPoint extent: ePoint  color: color width: width
	"Answer an initialized instance."
	^self new origin: oPoint; extent: ePoint; color: color; width: width
! !

!DisplayLine class methodsFor:'class initialization'!

initialize
        "DisplayLine initialize"

        Smalltalk isSqueak ifTrue:[
            LineBlt := BitBlt
                    destForm: Display
                    sourceForm: nil
                    fillColor: nil
                    combinationRule: Form over
                    destOrigin: 0@0
                    sourceOrigin: 0@0
                    extent: 1@1
                    clipRect: Display boundingBox.
        ]
! !

!DisplayLine class methodsFor:'class var accessing'!

lineBlt
	"Answer the cached BitBlt used for drawing lines."
	^LineBlt
! !

!DisplayLine class methodsFor:'examples'!

lineExample
        "DisplayLine lineExample"
        Display fillWhite.
        (DisplayLine linesX: Display width byY: Display height) display.
        (Delay forSeconds: 2) wait.
        ScheduledControllers restore
!

linesX: x byY: y
	"Answer a display list with randomly-placed random-color lines over the given extent."
	"(DisplayLine linesX: Display width byY: Display height) display"
	^self linesX: x byY: y items: 100
!

linesX: x byY: y items: items
        "Answer a display list with randomly-placed random-color lines over the given extent."
        "(DisplayLine linesX: Display width byY: Display height) display"

        | dList rand inset origin extent clr |
        inset := 32.
        dList := DisplayList new.
        rand := Random new.
        items timesRepeat:
                [origin := (rand nextIntFrom: inset to: x-inset) @ (rand nextIntFrom: inset to: y-inset).
                extent := (rand nextIntFrom: -128 to: 128) @ (rand nextIntFrom: -128 to: 128).
                clr := Smalltalk isSqueak ifTrue:[Color random] ifFalse:[Color redByte:(rand nextIntFrom:0 to:255) greenByte:(rand nextIntFrom:0 to:255) blueByte:(rand nextIntFrom:0 to:255)].
                dList add: (DisplayLine origin: origin extent: extent 
                                                color: clr width: (rand nextIntFrom: 1 to: 6))].
        ^dList
! !

!DisplayLine methodsFor:'accessing'!

corner
	"Answer the corner of the receiver."
	^corner
!

corner: aPoint
	"Set the corner of the receiver."
	corner := aPoint
!

operatorForCorner: aPoint
	"Answer the message to send to change the given corner point."
	^(aPoint = origin)
		ifTrue: [#origin:]
		ifFalse: [#corner:]
!

selectionHandles
	"Answer the points at which to display selection handles when the receiver is selected."
	^Array with: origin with: corner
!

slope
	"Answer the first derivative of the receiver (in screen orientation--y running negative)."
	^(corner x = origin x)
		ifFalse: [(corner y - origin y) / (corner x - origin x)]
		ifTrue: [(origin y - corner y) / (corner x - origin x + 1)]
!

width: aVal
	width := aVal
! !

!DisplayLine methodsFor:'displaying'!

displayOn: aDisplayMedium at: aPoint
        "Display a line between startPoint and endPoint."
        "(DisplayList linesX: 600 byY: 400) display"
"       Transcript show: start printString, '   ', end printString; cr."

        Smalltalk isSmalltalkX ifTrue:[
            color == nil
                    ifTrue: [aDisplayMedium paint: Color black]
                    ifFalse: [aDisplayMedium paint: color].
            width == nil
                    ifTrue: [aDisplayMedium lineWidth:1]
                    ifFalse: [aDisplayMedium lineWidth:width].
            aDisplayMedium displayLineFrom: (origin + aPoint) to: (corner + aPoint)
        ] ifFalse:[
            color == nil
                    ifTrue: [LineBlt fillColor: Color black]
                    ifFalse: [LineBlt fillColor: color].
            width == nil
                    ifFalse: [LineBlt width: width; height: width].
            LineBlt setDestForm: aDisplayMedium.
            LineBlt drawFrom: (origin + aPoint) to: (corner + aPoint)
        ]
! !

!DisplayLine methodsFor:'drawing'!

drawOn: aCanvas at: aPoint
	"Display the receiver on the given canvas at the given position."

	aCanvas line: (origin + aPoint) to: (corner + aPoint) 
		width: width color: color
! !

!DisplayLine methodsFor:'printing'!

printInstVarsOn: aStream
	"Print the receiver's relevant instance variables on the given Stream."
	super printInstVarsOn: aStream.
	corner isNil ifFalse: [aStream nextPutAll: ' corner: '. corner printOn: aStream].
	color isNil ifFalse: [aStream nextPutAll: ' color: '. color printOn: aStream].
! !

!DisplayLine methodsFor:'testing'!

isHit0: aPoint within: distance
	"Answer whether the given point is a hit on the receiver to within the given distance."
	| expect |
"	(super isHit: aPoint within: distance)
		ifFalse: [^false].
"	expect := aPoint x - origin x * self slope + origin y.		"expected y value"
	^(expect - aPoint y) abs <= distance
!

isHit: aPoint within: distance
	"Answer whether the given point is a hit on the receiver to within the given distance."
	| expect |
	(super isHit: aPoint within: distance)
		ifFalse: [^false].
	expect := aPoint x - origin x * self slope + origin y.		"expected y value"
	^(expect - aPoint y) abs <= distance
! !

!DisplayLine class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayLine.st,v 1.1 2000/12/01 11:45:57 cg Exp $'
! !
DisplayLine initialize!
