"{ Package: 'stx:goodies/displayLists' }"

DisplayModel subclass:#DisplayList
	instanceVariableNames:'components origin bounds'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayList comment:'Instances of DisplayList are used for representing composed structured graphics. They can be nested (see the class examples) and can display themselves and their components.

Instance Variables:
	components	<OrderedCollection of DisplayModels> the list''s display items
	offset		<Point> the object''s offset distance relative to its container
	bounds		<Rectangle> the list''s bounding box'
!


!DisplayList class methodsFor:'instance creation'!

new
	^super new initialize
! !

!DisplayList class methodsFor:'examples'!

exampleHierarchical
        "Create and answer a large display list with a lines, strings, and visuals as sub-lists in different regions."
        "(DisplayList exampleHierarchical) display"
        "((DisplayList exampleHierarchical) scaledBy: 2@2) display"
        "| d |  Display fillWhite.
        d := DisplayList exampleHierarchical.
        d displayOn: Display.
        (Delay forSeconds: 2) wait.
        Display fillWhite.
        (d scaledBy: 2@2) displayOn: Display.
        (Delay forSeconds: 2) wait.
        ScheduledControllers restore"

        | dl1 dl2 dl3 dl4 dl5 dl6 rand size pt x y w h |
        size := 100.
        dl1 := DisplayList new.
        dl2 := DisplayList new.
        dl3 := DisplayList new.
        dl4 := DisplayList new.
        dl5 := DisplayList new.
        dl6 := DisplayList new.
        rand := Random new.
        10 timesRepeat:
                [pt := (rand next * size) rounded @ ((rand next * size) rounded).
                dl1 add: ((DisplayString string: (pt printString) 
                                origin: pt) color: (DisplayList randomColor))].
        10 timesRepeat:
                [x := (rand next * size) rounded.               y := (rand next * size) rounded.
                w := (rand next * size) rounded.                h :=  (rand next * size) rounded.
                dl2 add: ((DisplayRectangle origin: (x@y) extent: (w@h))
                                color: (DisplayList randomColor); fill: true)].
        10 timesRepeat:
                [x := (rand next * size) rounded.               y := (rand next * size) rounded.
                w := (rand next * size) rounded.                h :=  (rand next * size) rounded.
                dl3 add: ((DisplayRectangle origin: (x@y) extent: (w@h))
                                color: (DisplayList randomColor); fill: true)].
        10 timesRepeat:
                [pt := (rand next * size) rounded @ ((rand next * size) rounded).
                dl4 add: ((DisplayString string: (pt printString) 
                                origin: pt) color: (DisplayList randomColor))].
        dl5 add: dl1; 
                add: dl2 at: (250@0); 
                add: dl3 at: (0@250); 
                add: dl4 at: (250@250).
        dl6 add: dl5 at: (50@50).
        ^dl6
!

exampleHierarchicalStarsX: x byY: y stars: stars
        "Create and answer a large display list with a lines, strings, and visuals as sub-lists in different regions."
        "(DisplayList exampleHierarchicalStarsX: 600 byY: 600 stars: 16) display"

        | list star rand spikes pt ext |
        list := DisplayList new.
        rand := Random new.
        spikes := 10.
        ext := 24.
        stars timesRepeat:
                [star := DisplayList new.
                spikes timesRepeat:
                        [pt := (rand nextIntFrom: ext negated to: ext) @ (rand nextIntFrom: ext negated to: ext).
                        star add: (DisplayLine origin: (0@0) extent: pt  color:  (DisplayList randomColor) width: (rand nextIntFrom: 1 to: 4))].
                list add: star at: ((rand nextIntFrom: ext to: x - ext) @ (rand nextIntFrom: ext to: y - ext))].
        ^list
!

gridExample
	"Create and answer a large display list with lines and strings."
	"(DisplayList gridExample) display"
	"DisplayListView example"
	| dList rand ext step pt |
	ext := 5000.
	step := 50.
	dList := DisplayList new.
	0 to: (ext/step) do:
		[ :i |
		dList add: (DisplayLine origin: (i*step)@0 extent: 0@ext).
		dList add: (DisplayLine origin: 0@(i*step) extent: ext@0)].
	rand := Random new.
	1 to: 1000 do:
		[ :i |
		pt := (rand next * ext) rounded @ ((rand next * ext) rounded).
		dList add: (DisplayString string: (pt printString) origin: pt)].
	^dList
!

gridExampleX: x byY: y
	"Create and answer a large display list with lines and strings."
	"(DisplayList gridExampleX: 3000 byY: 500) display"
	| dList rand step pt |
	step := 50.
	dList := DisplayList new.
	0 to: (x/step) do:
		[ :i |
		dList add: (DisplayLine origin: (i*step)@0 extent: 0@y)].
	0 to: (y/step) do:
		[ :i |
		dList add: (DisplayLine origin: 0@(i*step) extent: x@0)].
	rand := Random new.
	1 to: (x * y / 20000) do:
		[ :i |
		pt := (rand next * x) rounded @ ((rand next * y) rounded).
		dList add: (DisplayString string: (pt printString) origin: pt)].
	^dList
!

randomExample
        "Create and answer a large display list with a lines, strings, and visuals."
        "(DisplayList randomExample) display"
        | dList rand ext size pt pt2 x y w h coll |
        ext := 1500.
        size := ext / 2.
        dList := DisplayList new.
        rand := Random new.
        20 timesRepeat:
                [x := (rand next * size) rounded.
                y := (rand next * size) rounded.
                w := (rand next * size/10) rounded.
                h :=  (rand next * size/10) rounded.
                dList add: ((DisplayRectangle
                                origin: (x@y)
                                extent: (w@h))
                                color: DisplayList randomColor; fill: true)].
        20 timesRepeat:
                [pt := (rand next * size) rounded @ ((rand next * size) rounded).
                pt2 := (rand next * size) rounded @ ((rand next * size) rounded).
                dList add: ((DisplayLine origin: pt2 extent: (pt - pt2))
                                color: DisplayList randomColor)].
        20 timesRepeat:
                [pt := (rand next * size) rounded @ ((rand next * size) rounded).
                dList add: ((DisplayString string: (pt printString) 
                                origin: pt) color: DisplayList randomColor)].
        20 timesRepeat:
                [coll := Cursor allInstances.
                pt := (rand next * ext) rounded @ ((rand next * ext) rounded).
                dList add: ((DisplayVisual visual: (coll atRandom) 
                                origin: pt) color: DisplayList randomColor)].
        ^dList
!

scalingExample
	"Create and scale a simple display list."
	"DisplayList scalingExample"

	| list box list2 |
	list := DisplayRectangle rectanglesX: 160 byY: 160 items: 16.
	box :=  list computeBounds.
	Transcript tab; show: box printString, ' -- ', list items size printString; cr.
	list displayOn: Display at: 32@32.
	list2 := list scaledBy: (2@2).
	box :=  list2 computeBounds.
	Transcript tab; show: box printString, ' -- ', list items size printString; cr.
	list2 displayOn: Display at:400@32.

"	self halt."
!

stringExample
	"Open a display list view with randomly-placed random-color strings over the given extent."
	"DisplayString stringExample"
	(DisplayList stringsX: 600 byY: 600) display
!

testAll
	"Show all of the display items"
	"DisplayList testAll"

	| delay x y d |
	delay := Delay forSeconds: 1.
	x := Display extent x.
	y := Display extent y.
	Display fillWhite.
	'Lines' displayOn: Display at: 10@10.
	(DisplayLine linesX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.
	'Polygons' displayOn: Display at: 10@10.
	(DisplayPolyline polylinesX: x byY: y items: 60) displayOn: Display.
	delay wait.		Display fillWhite.
	'Splines' displayOn: Display at: 10@10.
	(DisplaySpline splinesX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.	
	'Circles' displayOn: Display at: 10@10.
	(DisplayCircle circlesX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.	
	'Rectangles' displayOn: Display at: 10@10.
	(DisplayRectangle rectanglesX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.
	'Strings' displayOn: Display at: 10@10.
	(DisplayString stringsX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.
	'Visuals' displayOn: Display at: 10@10.
	(DisplayVisual visualsX: x byY: y) displayOn: Display.
	delay wait.		Display fillWhite.
	'Hierarchical Lists' displayOn: Display at: 10@10.
	(DisplayList exampleHierarchical) display.
	delay wait.		Display fillWhite.
	'Hierarchical Lists' displayOn: Display at: 10@10.
	(DisplayList exampleHierarchicalStarsX: 800 byY: 600 stars: 64) display.
	delay wait.		Display fillWhite.
	'List Scaling' displayOn: Display at: 10@10.
	d := DisplayList exampleHierarchical.
	d displayOn: Display.
	delay wait.		Display fillWhite.
	(d scaledBy: 1.5@1.5) displayOn: Display.
	delay wait.		Display fillWhite.
	(d scaledBy: 0.6@0.6) displayOn: Display.
	delay wait.
	ScheduledControllers restore
! !

!DisplayList class methodsFor:'support'!

randomColor
    Smalltalk isSmalltalkX ifTrue:[^ Color redByte:(Random nextIntegerBetween:0 and:255) 
                                           greenByte:(Random nextIntegerBetween:0 and:255) 
                                           blueByte:(Random nextIntegerBetween:0 and:255)].
    ^Color random
! !

!DisplayList methodsFor:'accessing'!

components
	"Answer the receiver's sub-components."

	^components
!

flatten
	"Answer a copy of the receiver with its hierarchy flattened."

	| new subnew |
	new := DisplayList new.
	components do:
		[ :item |
		item hasItems
			ifTrue: [subnew := item flatten components.
				subnew do: [ :subitem |
					new add: (subitem translateBy: item origin)]]
			ifFalse: [new add: item]].
	^new
!

flushItems
	"Empty the receiver's sub-components."

	components := OrderedCollection new.
!

items
	"Answer the receiver's sub-components."

	^components
!

nodeAt: aPath
	"Answer the element described by the collection of items in the argument."

	| next path2 |
	components isEmpty ifTrue: [^nil].
	next := components
			detect: [ :i | 
					i "component" model == aPath first]
			ifNone: [nil].
	next == nil
		ifFalse: [aPath size = 1
			ifTrue: [^next]
			ifFalse: [path2 := aPath copy.
				path2 removeFirst.
				^next nodeAt: path2]]
		ifTrue: [^nil]
!

nodeFor: aModel
	"Answer the element whose model is the argument."

	components isEmpty ifTrue: [^nil].
	^components
			detect: [ :i | i model == aModel]
			ifNone: [nil].
!

origin

	^origin
!

origin: aPoint
	"Set the origin of the receiver."

	origin := aPoint
! !

!DisplayList methodsFor:'adding-removing'!

add: aVisualComponent
	"Add the argument to the receiver."

	^self addComponent: aVisualComponent
!

add: aVisual at: aPoint
	"Add aVisual to the receiver's components with its offset set to aPoint."

	aVisual origin: aPoint.
	self add: aVisual.
	bounds := nil
!

add: anItem atPath: aPath
	"Add the first argument at the node described by the second."

	| node |
	(aPath == nil or: [aPath isEmpty])
		ifTrue: [^self add: anItem].
	node := self nodeAt: aPath.
	node == nil
		ifTrue: [self addComponent: anItem]
		ifFalse: [node addComponent: anItem]
!

addComponent: aVisual
	"Add the argument to the receiver."

	components addLast: aVisual.
	aVisual container: self.
	bounds := nil
! !

!DisplayList methodsFor:'bounds accessing'!

boundingBox
	"Answer the receiver's bounds."

	bounds isNil
		ifTrue: [self computeBounds].
	^bounds
!

bounds
	"Answer the receiver's bounds."

	bounds isNil
		ifTrue: [self computeBounds].
	^bounds
!

bounds: aRectangle
	"Set the receiver's bounds to the argument."

	bounds := aRectangle
!

computeBoundingBox
	"Compute the receiver's bounds"

	^self computeBounds
!

computeBounds
	"Compute the receiver's bounds"

	| comp |
	components isEmpty
		ifTrue: [bounds := (0@0) extent: (0@0). ^bounds]
		ifFalse: [bounds := components first boundingBox].
	1 to: components size do:
		[:i |
		comp := components at: i.
		bounds := bounds merge: (comp computeBounds "translateBy: comp origin")].
	bounds origin < (0@0)
		ifTrue: [bounds := bounds translateBy: bounds origin negated].
	^origin == nil
		ifTrue: [bounds "expandedBy: (64@64)"]
		ifFalse: [(bounds translateBy: origin) "expandedBy: (64@64)"]
!

extent
	"Answer the extent of the receiver."

	^self boundingBox extent
! !

!DisplayList methodsFor:'displaying'!

display
        "Open a DisplayListView on the receiver."

        Smalltalk isSmalltalkX ifTrue:[
            ^ self displayOn: Display rootView at: 0@0
        ].
        self displayOn: Display at: 0@0
!

displayOn: aDisplayMedium at: aPoint
	"Display each of the receiver's components."
	"(DisplayList rectanglesX: 400 byY: 400) displayOn: Display"

	1 to: components size do:
		[:i | 
		(components at: i) displayOn: aDisplayMedium at: origin + aPoint].
!

displayOn: aDisplayMedium at: aPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
	"Display each of the receiver's components."
	"(DisplayList rectanglesX: 400 byY: 400) displayOn: Display"

	| item rect |
	rect := 0@0 extent: clipRectangle extent.
	1 to: components size do:
		[:i | 
		item := components at: i.
		(item boundingBox intersects: rect)
			ifTrue: [item displayOn: aDisplayMedium at: origin + aPoint]]
!

edit
	"Open a display list view on the receiver."

	DisplayListView openOnList:  self
!

view
	"Open a display list view on the receiver."

	DisplayListView openOnList:  self
! !

!DisplayList methodsFor:'enumerating'!

do: aBlock
	"Iterate the argument block over the receiver's components."
	components do: aBlock
! !

!DisplayList methodsFor:'hit detection'!

itemWithin: dist of: point
	"Answer the list of items that think the given point and tolerance qualifies as a 'hit.' "

	| item |
	components do: 
		[ :comp |
		comp hasItems
			ifTrue: [(item := comp itemWithin: dist of: (point - comp origin)) == nil
					ifFalse: [^item]]
			ifFalse: [(comp isHit: point within: dist) ifTrue: [^comp]]].
	^nil
!

itemsFromX: x1 toX: x2
	"Answer the list of items whose origins are within the given X range."

	| dList tList |
	dList := DisplayList new.
	components do: 
		[ :i |
		i hasItems
			ifTrue:
				[tList := i itemsFromX: x1 + i origin x toX: x2 + i origin x.
				tList isEmpty ifFalse: [dList add: tList at: i origin]]
			ifFalse:
				[(i origin x between: x1 and: x2) ifTrue: [dList add: i]]].
	^dList
!

itemsIntersecting: rect
	"Answer the list of items whose boundingBoxes intersect the given rectangle. 
	Clip them to the box's border if necessary."

	| dList dList2 |
	dList := DisplayList new.
	components do: 
		[ :i |
		i hasItems
			ifTrue: [dList2 := i itemsIntersecting: (rect translateBy: self origin).
				dList2 == nil
					ifFalse: [dList add: (dList2 origin: i origin)]]
			ifFalse: [(((i boundingBox) translateBy: self origin) 
								intersects: rect)
					ifTrue: [dList add: (i clipTo: (rect translateBy: ((0@0) - self origin)))]]].
	^(dList items isEmpty)
		ifTrue: [nil]
		ifFalse: [(dList items size = 1)
			ifTrue: [dList components first]
			ifFalse: [dList]]
!

itemsWithin: rect
	"Answer the list of items whose boundingBoxes are entirely within the given rectangle."

	| dList movedRect items first |
	movedRect := rect translateBy: self origin negated.
	dList := DisplayList new.
	components do: 
		[ :comp |
		comp hasItems
			ifTrue: [items := comp itemsWithin: movedRect.
				items == nil ifFalse: [dList add: items]]
			ifFalse: [(movedRect containsRect: comp boundingBox) 
					ifTrue: [dList add: comp]]].
	(dList components isEmpty)
		ifTrue: [^nil]
		ifFalse: [(dList components size = 1)
			ifTrue: [first := dList components first.
				first hasItems
					ifTrue: [first container: nil].
				^first copy translateBy: self origin]
			ifFalse: [^dList translateBy: self origin]]
!

itemsWithin: dist of: point
	"Answer the list of items whose boundingBoxes are entirely within the given rectangle."
	| dList |
	dList := DisplayList new.
	components do: 
		[ :comp |
		comp hasItems
			ifTrue: [dList add: 
					(comp itemsWithin: dist of: (point - self origin))]
			ifFalse: [((comp boundingBox expandBy: dist) containsPoint: point)
					ifTrue: [dList add: comp]]].
	^(dList components isEmpty)
		ifTrue: [nil]
		ifFalse: [(dList components size = 1)
			ifTrue: [dList components first]
			ifFalse: [dList]]
!

modelsFromX: x1 toX: x2
	"Answer the list of model-space-items whose origins are within the given X range."
	| dList |
	dList := DisplayList new.
	components do: 
		[ :i |
		((i origin x >= x1) and: [i origin x <= x2])
				ifTrue: [dList add: i]].
	^dList collect: [ :i | i model]
!

selectionHandlesFor: item
	"Answer the transformed set of selection handle points for the argument in the hierarchy of the receiver."

	| pts cont |
	pts := item selectionHandles.
	cont := item container.
	[(cont == self) or: [cont isNil]]
		whileFalse:
			[pts := pts collect: [ :p | p + cont origin].
			cont := cont container].
	^pts
! !

!DisplayList methodsFor:'initialize-release'!

initialize
	"Set up the default state of the receiver"

	super initialize.
	components := OrderedCollection new.
	origin := 0@0
!

release
	"Break the model dependency links of all the receiver's display items."

	super release.
	components do: [ :item | item release]
! !

!DisplayList methodsFor:'printing'!

printCompleteOn: aStream depth: depth
	"Descend the hierarchy printing on the stream."
	aStream cr.
	depth timesRepeat: [aStream tab].
	aStream nextPutAll: '(a DisplayList '.
	origin == nil
		ifFalse: [aStream nextPutAll: ' origin: ', origin printString].
"	model == nil
		ifFalse: [aStream nextPutAll: ' model: ', model printString]."
	components == nil
		ifFalse: [components do: 
			[ :e | 
			e printCompleteOn: aStream depth: depth + 1]].
	aStream nextPut: $)
!

printOn: aStream
	"Print the receiver on the argument using the recursive method."
	^Sensor leftShiftDown 
		ifTrue: [super printOn: aStream]
		ifFalse: [self printCompleteOn: aStream depth: 0]
! !

!DisplayList methodsFor:'testing'!

hasItems
	"Answer whether or not the receiver has items or components."

	^true
!

isEmpty
	"Amswer whether the receiver has any items."

	^components isEmpty
!

isWide
	"Answer whether is much wider than tall."

	| b |
	b := self bounds.
	^b x > (b y * 2)
! !

!DisplayList methodsFor:'transforming'!

scaledBy: aPoint
	"Scale the receiver's sizes by the argument; answer a new display list."

	| dl item |
	dl := self copy.
	dl flushItems.
	dl origin: (origin * aPoint).
	1 to: components size do:
		[ :ind |
		item := components at: ind.
		dl add: (item scaledBy: aPoint)].
	^dl
!

translateBy: aPoint
	"Translate the receiver's origin by the argument."

	self origin: self origin + aPoint.
	bounds := nil
! !

!DisplayList class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayList.st,v 1.2 2000/12/01 15:20:20 cg Exp $'
! !
