"{ Package: 'stx:goodies/displayLists' }"

MouseMenuController subclass:#DisplayListController
	instanceVariableNames:'selection actions keys'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayListViews'
!

DisplayListController comment:'Instances of DisplayListController (or its subclasses) are used by DisplayListViews (or subclass instances) for menu management and selection.

By default, a simple YellowButton menu allows for group/inspect/redraw/zoom.
The RedButton is used for display item selection (by pointing or boxing-in).
 LeftShift extends the selection, and dragging a selection moves it.



Instance Variables:
	selection		<DisplayList> the object''s selected items

Interaction:
	The red mouse botton selects, drags, and rubber-bands items in the display list.
	The yellow button brings up a simple menu of basic functions (only redraw has been tested so far).
	The blue button is used for ''drag-scrolling'' of the view--try it!!
	The cursor keys, page up/down and home/end keys scroll in steps of 1/4 screen.
	The keys + and - zoom in and out; pressing ''='' zooms to scale 1@1.'
!


!DisplayListController methodsFor:'accessing'!

selection
	"Answer the receiver's 'selection'."

	^selection
!

selection: anObject
	"Set the receiver's instance variable 'selection' to be anObject."

	selection := anObject
! !

!DisplayListController methodsFor:'control defaults'!

blueButtonActivity
	"Drag scroll amplified by 2@2 with the blue button."

	| pt1 pt2 |
	pt1 := sensor cursorPoint.
	(Delay forMilliseconds: 250) wait.
	pt2 := sensor cursorPoint.
	(pt2 = pt1) 
		ifTrue: [^view topView controller blueButtonActivity].
	[sensor anyButtonPressed] whileTrue:
		[pt2 := sensor cursorPoint.
		(pt2 = pt1) 
			ifFalse: [view scrollBy: (pt2 - pt1) * 2.
				pt1 := pt2]]
!

controlActivity

	self viewHasCursor
		ifFalse: [^super controlActivity].
	((self sensor redButtonPressed) & (self sensor yellowButtonPressed))
		ifTrue: [^self orangeButtonActivity].
	self sensor redButtonPressed
		ifTrue: [^self redButtonActivity].
	self sensor yellowButtonPressed
		ifTrue: [^self yellowButtonActivity].
	self sensor blueButtonPressed
		ifTrue: [^self blueButtonActivity].
	self sensor keyboardPressed
		ifTrue: [^self processKeyboard]
!

extendSelection
	"Hit-detect items from the display list."

	| pt item |
	model == nil ifTrue: [^nil].
	sensor waitNoButton.
	pt := sensor cursorPoint.
"	Transcript show: 'Select at: ', pt printString; cr."
	item := view list itemWithin: 1 of: pt - view class inset.
"	Transcript show: 'ExtSelect ', item printString; cr."
	item == nil
		ifFalse: [model addSelection: item]
!

orangeButtonActivity
	"Do the default orange-click action."

	self performAction: (keys at: #orangeClick)
!

processKeyboard
	"Map scroll/page keys to view scrolling, +, -, and = to zooming."

	| keyEvent |
	sensor keyboardPressed ifFalse: [^ self].
	keyEvent := sensor keyboard.
"	(keyEvent isAlphaNumeric or: [keyEvent isSpecial or: [keyEvent = $-]])
		ifTrue: [Transcript show: keyEvent printString; cr].
"
	keyEvent == $] ifTrue: [^view zoomIn].		"+ to zoom in by 25%"
	keyEvent == $} ifTrue: [^view zoomIn].		"+ to zoom in by 25%"
	keyEvent == $[ ifTrue: [^view zoomOut].		"- to zoom out 25%"
	keyEvent == ${ ifTrue: [^view zoomOut].		"- to zoom out 25%"
	keyEvent == $\ ifTrue: [^view zoomTo1].		"= to go to scale 1.0"
	keyEvent == $= ifTrue: [^view zoomTo1].		"= to go to scale 1.0"
	keyEvent == $+ ifTrue: [^view zoomTo1].		"= to go to scale 1.0"

	keyEvent := keyEvent asciiValue.
"	Transcript show: keyEvent printString; cr.
"	keyEvent == 31							"down-arrow"
		ifTrue: [^sensor leftShiftDown
			ifTrue: [view pageDown]
			ifFalse: [view scrollDown]].	
	keyEvent == 30							"up arrow"
		 ifTrue: [^sensor leftShiftDown
			ifTrue: [view pageUp]
			ifFalse: [view scrollUp]].
	 keyEvent == 28							"left-arrow"
		ifTrue: [^sensor leftShiftDown
			ifTrue: [view pageLeft]
			ifFalse: [view scrollLeft]].
	keyEvent == 29							"right arrow"
		ifTrue: [^sensor leftShiftDown
			ifTrue: [view pageRight]
			ifFalse: [view scrollRight]].
	keyEvent == 1  ifTrue: [^view scrollHome].			"home"
	keyEvent == 4  ifTrue: [^view scrollEnd].  			"end"
	keyEvent == 11 ifTrue: [^view pageUp].			 "page up"
	keyEvent == 12  ifTrue: [^view pageDown]. 			 "page down"
"
	self error: 'Bad key: ', keyEvent printString.
"
!

redButtonActivity
	"Hit-detect items from the display list or drag a selection."

	(Delay forMilliseconds: 200) wait.		"wait to see if it's a click or a drag."
	self sensor anyButtonPressed
		ifTrue: [self sensor leftShiftDown
				ifTrue: [^self performAction: #shiftDrag].
			self sensor controlKeyPressed
				ifTrue: [^self performAction: #ctrlDrag].
			^self performAction: #drag]
		ifFalse: [self sensor leftShiftDown
				ifTrue: [^self performAction: #shiftClick].
			self sensor controlKeyPressed
				ifTrue: [^self performAction: #ctrlClick].
			^self performAction: #click]
! !

!DisplayListController methodsFor:'initialize-release'!

initialize
	"Set up the receiver's action and key dictionaries"

	super initialize.
	self initializeMenu.
	self initializeKeys.
	self initializeActions.
!

initializeActions
	"Initialize up the receiver's action dictionary."

	actions := Dictionary new.
	actions at: #click put: #select.
	actions at: #doubleClick put: #deselect.
	actions at: #shiftClick put: #extendSelection.
	actions at: #ctrlClick put: #delete.
	actions at: #orangeClick put: #selectAll.
	actions at: #drag put: #drag.
	actions at: #shiftDrag put: #drag.
	actions at: #ctrlDrag put: #drag.
!

initializeKeys
	"Initialize up the receiver's keys dictionary."

	keys := Dictionary new.
	keys at: #Left put: #scrollLeft.
	keys at: #Right put: #scrollRight.
	keys at: #Up put: #scrollUp.
	keys at: #Down put: #scrollDown.
	keys at: #Space put: #delete.
	keys at: #Tab put: #scrollRight.
!

initializeMenu
	"Set up the receiver's YellowButtonMenu"

	self yellowButtonMenu: (PopUpMenu labels: 'group
ungroup
inspect
redraw
zoom' lines: #(2 4))
		yellowButtonMessages: #(group ungroup doInspect redraw zoom)
! !

!DisplayListController methodsFor:'menu messages'!

doInspect
	"Inspect the model (if leftShiftDown) or the model's list."

	sensor leftShiftDown
		ifTrue: [model inspect]
		ifFalse: [model list inspect]
!

group
	"Group the current selections together."

	model group
!

redraw
	"Redraw the view and clear the selection."

	view displayView.
	model == nil
		ifFalse: [model select: nil]
!

ungroup
	"Ungroup the current selection (if possible)."

	model ungroup
!

zoom

	model doZoom

	"Prompt the user for a scale point, zoom the view's scale, and redraw."
"	| pt prompt |
	prompt := (view scaleX == nil)
		ifTrue: ['1.5@1.5']
		ifFalse: [(view scaleX @ view scaleY) printString].
	pt := DialogView request: 'zoom to scale:'
			initialAnswer: prompt.
	pt = '' ifFalse: [pt := Compiler evaluate: pt.
			pt class == Point
				ifTrue: [view scaleX: pt x; scaleY: pt y]]
"
! !

!DisplayListController methodsFor:'private'!

performAction: actionSymbol
	"Do the given action safely."

	(actions includesKey: actionSymbol)
		ifFalse: [^self].
	^self perform: (actions at: actionSymbol)
! !

!DisplayListController methodsFor:'selecting'!

drag
	"Drag the selection."

	| pt0 dl |
	dl := model selection.
	dl == nil ifTrue: [^self].
	pt0 := view relativePointFor: sensor cursorPoint.
	dl selectionHandles do:
		[ :sh |
		(pt0 - sh) r < 10
			ifTrue: [^self rubberBandSelection: dl corner: sh while: [sensor anyButtonPressed]]].
	^self dragSelection: dl while: [sensor anyButtonPressed]
!

dragSelection: dl while: dBlock
	"Drag the selection for as long as the given block is true."

	| pt0 pt1 |
	pt0 := sensor cursorPoint.
	dBlock whileTrue:
		[pt1 := sensor cursorPoint.
		pt0 = pt1
			ifFalse: [dl translateBy: (pt1 - pt0).
				pt0 := pt1.
				view displayView]]
!

rubberBandSelection: dl corner: sh while: dBlock
	"Rubber-band one corner of the selection for as long as the given block is true."

	| pt0 pt1 operator |
	operator := dl operatorForCorner: sh.
	pt0 := view relativePointFor: sensor cursorPoint.
	dBlock whileTrue:
		[pt1 := view relativePointFor: sensor cursorPoint.
		pt0 = pt1
			ifFalse: [operator ifNotNil:
					[dl perform: operator with: pt1].
				pt0 := pt1.
				view displayView]]
!

select
	"Hit-detect items from the display list."

	| pt item |
	model == nil ifTrue: [^nil].
	sensor waitNoButton.
	pt := view relativePointFor: sensor cursorPoint.
"	Transcript show: 'Select: ', pt printString; cr."
	item := view list itemWithin: 4 of: pt.
"	Transcript show: 'Select ', item printString; cr."
	model select: item
!

selectFromPoint: aPoint toPoint: anotherPoint
	"Hit-detect the model for the given range."

	| item |
	(aPoint dist: anotherPoint) < 8
		ifTrue: [item := view list itemWithin: 24 of: aPoint.
			Transcript show: 'Select ', item printString; cr.
			item == nil
				ifTrue: [model select: item]
				ifFalse: [model select: item]]
		ifFalse: [model select:
				(view list itemsWithin: 
						(aPoint corner: anotherPoint))]
!

selectRangeWhile: durationBlock
	"Hit-detect items from the display list or drag a selection as long as the block is true."

	| pt1 pt2 gc color1 color2 |
	gc := view graphicsContext.
	color1 := Color black.
	color2 := Color white.
	pt1 := sensor cursorPoint.
	pt2 := pt1 + (4@4).
	sensor cursorPoint: pt2.
	durationBlock whileTrue:
		[gc paint: color1.
		gc displayRectangularBorder: (pt1 corner: pt2).
		pt2 := sensor cursorPoint.
		gc paint: color2.
		gc displayRectangularBorder: (pt1 corner: pt2)]
! !

!DisplayListController class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayListController.st,v 1.1 2000/12/01 15:19:35 cg Exp $'
! !
