"{ Package: 'stx:goodies/displayLists' }"

Model subclass:#DisplayListEditor
	instanceVariableNames:'selection displayList copyBuffer view controller'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayListViews'
!

DisplayListEditor comment:'An instance of DisplayListEditor is used as the intermediary model for viewing lists.
It handles selection and operation on the list.
It adds itself as a dependent of the list and echoes its change messages (i.e., the update: message says self changed).

Instance Variable:
	list			<DisplayList> the ''subject'' list model
	selection		<DisplayItem orDisplayLIst> the current selection or nil
'
!


!DisplayListEditor class methodsFor:'instance creation'!

on: model
	^self new list: model
! !

!DisplayListEditor methodsFor:'accessing'!

addSelection: sel
	"Add the argument to the receiver's selection."

	| dl |
"	sel == nil ifTrue: [Transcript show: 'clear'; cr]."
	selection hasItems
		ifTrue: [selection add: sel]
		ifFalse: [dl := DisplayList new.
			dl add: selection.
			dl add: sel.
			selection := dl].
	self changed: #selection with: sel
!

displayList
	"Answer the receiver's list."

	^displayList
!

list
	"Answer the receiver's display list."

	^displayList
!

list: aList
	"Set the receiver's display list, removing the dependency to the former list if necessary."

	displayList == nil
		ifFalse: [displayList removeDependent: self].
	displayList := aList.
	displayList addDependent: self
!

moveSelectionTo: newPoint
	"Move the editor's selection to the new point in the displayList."

	| oldOff |
	selection hasItems
		ifFalse: [oldOff := selection origin.
			selection origin: newPoint.
			self changed": #position with: (oldOff corner: newPoint)"]
		ifTrue: [selection components do: [ : i | displayList remove: i].
			selection origin: newPoint.
			displayList add: selection.
			self changed]
!

select: sel
	"Set the receiver's selection."

"	sel == nil ifTrue: [Transcript show: 'clear'; cr]."
	selection := sel.
	self changed: #selection with: sel
!

selection
	"Answer the receiver's selection."

	^selection
! !

!DisplayListEditor methodsFor:'editing'!

copy
	"Copy the selection into the copy buffer."

	copyBuffer := selection
!

doDisplay
	"inspect button"

	self changed
!

doEdit
	"edit button"

	self doInspect
!

doFile
	"file i/o button"

	Transcript show: 'No File.'; cr.
!

doInspect
	"Inspect the editor (if shiftDown) or the model."

	Sensor leftShiftDown
		ifTrue: [self inspect]
		ifFalse: [displayList inspect]
!

doZoom
	"Zoom in or out by 25%."

	Sensor leftShiftDown
		ifTrue: [self changed: #scaleX with: 0.75.
			self changed: #scaleY with: 0.75]
		ifFalse: [self changed: #scaleX with: 1.25.
			self changed: #scaleY with: 1.25].
	self changed
!

group
	"Group the selections together."

	| dl |
	selection == nil
		ifTrue: [^self].
	selection hasItems
		ifFalse: [^self].
	dl := DisplayList new.
	selection do:
		[ :item | dl add: item.
			displayList remove: item].
	displayList add: dl.
	self select: (DisplayList new add: dl)
!

ungroup
	"Ungroup the selection (if possible)."

	selection == nil
		ifTrue: [^self].
	selection hasItems
		ifFalse: [^self].
	displayList remove: selection components first.
	selection components first do:
		[ :item | displayList add: item].
	self select: selection components first
! !

!DisplayListEditor methodsFor:'initialize-release'!

release
	"Remove the dependency to the displayList."

	displayList removeDependent: self.
	displayList release
! !

!DisplayListEditor methodsFor:'updating'!

update: anAspect with: anArg
	"Echo the model's changes, assuming a view is a dependent of the receiver."

	self changed: anAspect with: anArg
! !

!DisplayListEditor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayListEditor.st,v 1.1 2000/12/01 15:19:42 cg Exp $'
! !
