"{ Package: 'stx:goodies/displayLists' }"

DisplayRectangle subclass:#DisplayPolyline
	instanceVariableNames:'vertices'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayPolyline comment:'Aninstance of DisplayPolyline can represent an open or closed polygon.
Instance Variables:
	vertices	<Array of Point> the polygon''s vertices'
!


!DisplayPolyline class methodsFor:'instance creation'!

origin: offPt vertices: arrayOfPoints
	"Answer a new DisplayPolyline with the arguments as its vertices."
	^self basicNew origin: offPt; vertices: arrayOfPoints
!

vertices: arrayOfPoints
	"Answer a new DisplayPolyline with the arguments as its vertices."
	^self basicNew vertices: arrayOfPoints
! !

!DisplayPolyline class methodsFor:'examples'!

polylineExample
	"DisplayPolyline polylineExample"
	Display fillWhite.
	(DisplayPolyline polylinesX: Display width byY: Display height) display.
	(Delay forSeconds: 2) wait.
	ScheduledControllers restore
!

polylinesX: x byY: y
	"Answer a display list with randomly-placed random-color polylines over the given extent."
	"(DisplayPolyline polylinesX: 600 byY: 400) display"
	^self polylinesX: x byY: y items: 40
!

polylinesX: x byY: y items: num
        "Answer a display list with randomly-placed random-color polylines over the given extent."
        "(DisplayPolyline polylinesX: 600 byY: 600 items: 1000) display"
        | dList rand inset scale x0 y0 size pointArray |
        inset := 16.
        scale := 128.
        dList := DisplayList new.
        rand := Random new.
        x0 := rand nextIntFrom: inset to: x-inset.
        y0 := rand nextIntFrom: inset to: y-inset.
        num timesRepeat:
                [size := rand nextIntFrom: 3 to: 8.
                pointArray := Array new: size.
                pointArray at: 1 put: (0@0).
                pointArray at: size put: (0@0).
                x0 := y0 := 0.
                2 to: size-1 do: 
                        [ :i |
                        x0 := x0 + (rand nextIntFrom: scale negated to: scale).
                        y0 := y0 + (rand nextIntFrom: scale negated to: scale).
                        pointArray at: i put: (x0 @ y0)].
                dList add: ((DisplayPolyline
                                origin: ((rand nextIntFrom: inset to: x-inset)
                                                @(rand nextIntFrom: inset to: y-inset)))
                                color: DisplayList randomColor;
                                width: (rand nextIntFrom: 1 to: 4);
                                vertices: pointArray ";
                                fill: (rand next > 0.8)")].
        ^dList
! !

!DisplayPolyline methodsFor:'accessing'!

boundingBox
	"Answer the receiver's bounds." 
	| ext |
	ext := 0@0.
	vertices do: [ :pt | ext := ext max: pt].
	^origin extent: ext
!

vertices: pointArray
	"Set the receiver's vertices."
	vertices := pointArray asArray
! !

!DisplayPolyline methodsFor:'displaying'!

displayOn: aDisplayMedium at: aPoint
	"Draw the receiver's edges on the supplied DisplayMedium."
	| src dest tColor intPt |
	tColor := color isNil
		ifTrue: [Color black]
		ifFalse: [color].
	LineBlt fillColor: tColor.
	width isNil
		ifFalse: [LineBlt width: width; height: width].
	LineBlt setDestForm: aDisplayMedium.
	src := vertices first + origin + aPoint.
	2 to: vertices size do:
		[ :index |
		dest := (vertices at: index) + origin + aPoint.
		LineBlt drawFrom: src truncated to: dest truncated.
		src := dest].
	(fill isNil or: [fill not])
		ifFalse: [intPt := vertices inject: 0@0 into: [ :sum :item | sum + item].
			intPt := intPt / vertices size.  "The average of the vertices is inside."
			aDisplayMedium shapeFill: tColor interiorPoint: intPt + origin + aPoint].
! !

!DisplayPolyline methodsFor:'transforming'!

scaledBy: aPoint
	"Scale the receiver's origin and extent by the argument; answer a copy."
	| new |
	new := self copy.
	self origin == nil
		ifFalse: [new origin: self origin * aPoint].
	new vertices: (vertices collect: [ :vPt | vPt * aPoint]).
	^new
! !

!DisplayPolyline class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayPolyline.st,v 1.2 2000/12/01 15:20:26 cg Exp $'
! !
