"{ Package: 'stx:goodies/displayLists' }"

DisplayLine subclass:#DisplayRectangle
	instanceVariableNames:'fill border'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayRectangle comment:'Instances of DisplayRectangle can be used to display bordered or filled rectangles in display lists. This is a subclass of DisplayLine for reasons of practicality. a "purist" (e.g., David Liebs), would create an intermediate abstract class (e.g., BoundedDisplayItem) for both DisplayLine and DisplayRectangle.
Instanve Variables:
	border	<Number> the border width
	fill		<Boolean or nil> whether or not to fill the receiver on display (no by default)'
!


!DisplayRectangle class methodsFor:'examples'!

rectangleExample
	"DisplayRectangle rectangleExample"

	Display fillWhite.
	(DisplayRectangle rectanglesX: Display width byY: Display height) display.
	(Delay forSeconds: 2) wait.
	ScheduledControllers restore
!

rectanglesX: x byY: y
	"(DisplayRectangle rectanglesX: 600 byY: 400) display"

	^self rectanglesX: x byY: y items: 60
!

rectanglesX: x byY: y items: items
        "Answer a display list with randomly-placed random-color rectangles over the given extent."
        "(DisplayRectangle rectanglesX: 600 byY: 400) display"

        | dList rand x0 y0 x1 y1 inset w h |
        dList := DisplayList new.
        rand := Random new.
        inset := 20.
        x0 := x - (inset * 2).
        y0 := y - (inset * 2).
        items timesRepeat:
                [x1 := inset + (rand next * x0) rounded.
                y1 := inset + (rand next * y0) rounded.
                w := (rand next * 64) rounded.
                h :=  (rand next * 64) rounded.
                dList add: ((DisplayRectangle
                                origin: x1@y1
                                extent: w@h
                                color: DisplayList randomColor)
                                fill: (rand next > 0.4); 
                                width: (rand nextIntFrom: 0 to: 4))].
        ^dList
! !

!DisplayRectangle methodsFor:'accessing'!

border
	"Answer the receiver's border color."
	^border
!

border: aColor
	"Set the receiver's border color."
	border := aColor
!

fill: aBoolean
	"Set the receiver's filling Boolean."
	fill := aBoolean
!

operatorForCorner: aPoint
	"Answer the message to send to change the given corner point."
	^(aPoint = origin)
		ifTrue: [#origin:]
		ifFalse: [aPoint = corner
			ifTrue: [#corner:]
				ifFalse: [aPoint = self topRight
					ifTrue: [#topRight:]
					ifFalse: [#bottomLeft:]]]
!

selectionHandles
	"Answer the points at which to display selection handles when the receiver is selected."
	^self boundingBox corners
!

topRight: aPoint
	"Set the point at the top right corner of the receiver's top horizontal line."
	corner x: aPoint x.
	origin y: aPoint y
! !

!DisplayRectangle methodsFor:'displaying'!

displayOn: aDisplayMedium at: aPoint
	"Display the receiver on the first argument."
	"(DisplayList rectanglesX: 600 byY: 400) display"
"	Transcript show: start printString, '	', end printString; cr."
	(fill == nil or: [fill not])
		ifFalse: [aDisplayMedium fill: (self boundingBox translateBy: aPoint)
					 fillColor: color ]
		ifTrue: [aDisplayMedium border: (self boundingBox translateBy: aPoint)
					width: (width  isNil ifTrue: [1] ifFalse: [width]) fillColor: color ]
! !

!DisplayRectangle methodsFor:'drawing'!

drawOn: aCanvas at: aPoint
	"Display the receiver on the given canvas at the given position."

	fill 
		ifTrue: [aCanvas fillRectangle: (self bounds translatedBy: aPoint) color: color]
		ifFalse: [aCanvas frameRectangle: (self bounds translatedBy: aPoint) 
						width: width color: color]
! !

!DisplayRectangle methodsFor:'testing'!

isHit: aPoint within: distance
	"Answer whether the given point is a hit 'on' the receiver to within the given distance."
	^(self boundingBox expandBy: distance) containsPoint: aPoint
! !

!DisplayRectangle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayRectangle.st,v 1.2 2000/12/01 15:20:06 cg Exp $'
! !
