"{ Package: 'stx:goodies/displayLists' }"

DisplayItem subclass:#DisplayString
	instanceVariableNames:'dText'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayString comment:'Instances of DisplayString are used for visual text items in display lists.
Instance Variables:
	dText	<DisplayText> the object''s string/text/style'
!


!DisplayString class methodsFor:'instance creation'!

new
	"Answer a new initialized instance."

	^super new initialize
!

string: aString
	"Answer a new instance with the argument as its string."

	^self new string: aString
!

string: aString origin: oPoint
	"Answer a new instance with the arguments as its string and origin point."

	^self new string: aString; origin: oPoint
! !

!DisplayString class methodsFor:'examples'!

pointsX: x byY: y items: items
        "Answer a display list with randomly-placed random-color strings over the given extent."
        "(DisplayString pointsX: Display width byY: Display height items: 100) display"

        | pt dList rand inset |
        inset := 16.
        dList := DisplayList new.
        rand := Random new.
        items timesRepeat:
                [pt := (rand nextIntFrom: inset to: x-inset)
                                                @(rand nextIntFrom: inset to: y-inset).
                dList add: ((DisplayString
                                string: pt printString
                                origin: pt)
                                color: DisplayList randomColor)].
        ^dList
!

stringExample
	"DisplayString stringExample"
	Display fillWhite.
	(DisplayString stringsX: Display width byY: Display height) display.
	(Delay forSeconds: 2) wait.
	ScheduledControllers restore
!

stringsX: x byY: y
	"Answer a display list with randomly-placed random-color strings over the given extent."
	"(DisplayString stringsX: 600 byY: 600) display"

	^self stringsX: x byY: y items: 80
!

stringsX: x byY: y items: items
        "Answer a display list with randomly-placed random-color strings over the given extent."
        "(DisplayString stringsX: 800 byY: 600 items: 500) display"

        | dList rand inset xp yp |
        inset := 16.
        dList := DisplayList new.
        rand := Random new.
        Cursor execute showWhile:
                [items timesRepeat:
                        [xp :=rand nextIntFrom: inset to: x - inset.
                        yp := rand nextIntFrom: inset to: y - inset.
                        dList add: ((DisplayString
                                        string: (xp printString, '@', yp printString)
                                        origin: (xp @ yp))
                                        color: DisplayList randomColor)]].
        ^dList
! !

!DisplayString methodsFor:'accessing'!

color: aVal
	"Set the receiver's display color to the argument."

	super color: aVal.
	dText foregroundColor: color backgroundColor: Color transparent
!

emph: emph
	"Set the receiver's emphasis."

	dText text addAttribute: (TextEmphasis perform: emph) from: 1 to: dText string size
!

extent
	"Answer a rectangle that circumscribes the receiver."

	^self boundingBox extent
!

font
	"Answer the receiver's font."

	^dText textStyle
!

font: aFont
	"Set the receiver's font."

	dText textStyle: aFont
!

string
	"Answer the receiver's string."

	^dText text string
!

string: aString
	"Set the receiver's string."

	dText text: aString asText
!

text
	"Answer the receiver's text."

	^dText text
!

textStyle
	"Always answer the default."

	^TextStyle default
! !

!DisplayString methodsFor:'bounds accessing'!

boundingBox
	"Answer the receiver's actual bounding rectangle."

	^dText boundingBox translateBy: self origin
! !

!DisplayString methodsFor:'displaying'!

displayOn: aDisplayMedium at: aPoint
	"Display the receiver on the argument."

	dText displayOn: aDisplayMedium at: origin + aPoint
		clippingBox: aDisplayMedium boundingBox
		rule: Form paint
		fillColor: color
! !

!DisplayString methodsFor:'drawing'!

displayOn: aCanvas using: aScanner at: aPoint
	"Display the receiver on the given canvas at the given position."

	| line style |
	style := TextStyle default.
	line := (TextLine start: 1 stop: dText string size
					internalSpaces: 0 paddingWidth: 0)
				rectangle: (0@0 extent: 10@style lineGrid);
				lineHeight: style lineGrid baseline: style baseline.
	aScanner displayLine: line offset: aPoint leftInRun: 0
!

drawOn: aCanvas at: aPoint
	"Display the receiver on the given canvas at the given position."

	aCanvas paragraph: self bounds: (self bounds translateBy: aPoint) color: color
! !

!DisplayString methodsFor:'initialize-release'!

initialize
	"Initialize the instance variables of the receiver."

	super initialize.
	dText := DisplayText new textStyle: TextStyle default; offset: 0@0
! !

!DisplayString methodsFor:'printing'!

printInstVarsOn: aStream
	"Print the receiver's string..."
	super printInstVarsOn: aStream.
	self string printOn: aStream.
! !

!DisplayString class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayString.st,v 1.2 2000/12/01 15:20:14 cg Exp $'
! !
