"{ Package: 'stx:goodies/displayLists' }"

DisplayItem subclass:#DisplayVisual
	instanceVariableNames:'visual'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayVisual comment:'Instances of DisplayVisual are used for displaying arbitrary visual objects (e.g., cached images) in display lists.
Instance Variables:
	visual		<Form> the object''s visual object, typically an image'
!


!DisplayVisual class methodsFor:'instance creation'!

model: aM visual: aV origin: aPt
	"Answer an instance of DisplayItem with the given instance variables."
	^self new model: aM; visual: aV; origin: aPt
!

visual: aV
	"Answer an instance of DisplayItem with the given instance variables."
	^(super new initialize) visual: aV
!

visual: aV origin: aPt
	"Answer an instance of DisplayItem with the given instance variables."
	^(super new initialize) visual: aV; origin: aPt
! !

!DisplayVisual class methodsFor:'examples'!

visualExample
	"DisplayVisual visualExample"
	Display fillWhite.
	(DisplayVisual visualsX: Display width byY: Display height) display.
	(Delay forSeconds: 2) wait.
	ScheduledControllers restore
!

visualsX: x byY: y
	"Answer a display list with randomly-placed random-color visuals over the given extent."
	"(DisplayVisual visualsX: 600 byY: 600) display"
	^self visualsX: x byY: y items: 80
!

visualsX: x byY: y items: items
        "Answer a display list with randomly-placed random-color visuals over the given extent."
        "(DisplayVisual visualsX: 600 byY: 600) display"
        | dList rand inset cursors |
        inset := 16.
        dList := DisplayList new.
        rand := Random new.
        cursors := Cursor allInstances.
        items timesRepeat:
                [dList add: ((DisplayVisual
                                visual: (cursors atRandom: rand)
                                origin: ((rand nextIntFrom: inset to: x-inset)
                                                @(rand nextIntFrom: inset to: y-inset)))
                                color: DisplayList randomColor)].
        ^dList
! !

!DisplayVisual methodsFor:'accessing'!

boundingBox
	"Answer the visual's bounds translated by the receiver's translation."
	^origin == nil
		ifTrue: [visual boundingBox]
		ifFalse: [visual boundingBox translateBy: origin]
!

visual
	"Answer the receiver's visual."
	^visual
!

visual: aVisual
	"Set the receiver's visual."
	visual := aVisual.
"	aVisual palette == nil
		ifTrue: [aVisual palette: MappedPalette colorDefault]"
! !

!DisplayVisual methodsFor:'displaying'!

displayOn: aDisplayMedium
	"Display the receiver's visual on the given graphics context."
	self displayOn: aDisplayMedium at: origin
!

displayOn: aDisplayMedium at: aPoint
	"Display the receiver's visual on the given display medium."

	visual displayOn: aDisplayMedium at: origin + aPoint 
			clippingBox: aDisplayMedium boundingBox
			rule: Form oldPaint 
			fillColor: color
!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm

	visual displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
! !

!DisplayVisual methodsFor:'drawing'!

drawOn: aCanvas at: aPoint
	"Display the receiver on the given canvas at the given position."

	aCanvas image: visual at: (self boundingBox origin + aPoint)
! !

!DisplayVisual methodsFor:'printing'!

printInstVarsOn: aStream
	"Print the receiver's relevant inst vars on the given Stream."
	super printInstVarsOn: aStream.
	visual == nil ifFalse: [aStream nextPutAll: ' visual: '. visual printOn: aStream].
! !

!DisplayVisual class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayVisual.st,v 1.2 2000/12/01 15:20:28 cg Exp $'
! !
