"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFModel subclass:#RTFCellModel
	instanceVariableNames:'container contents borders shading verticalAlignment
		textAlignment rightBoundary'
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFCellModel class methodsFor:'instance creation'!

in: aContainer
       "Returns an instance of the receiver contained by aContainer."

        ^self new
              container: aContainer;
                 yourself
! !

!RTFCellModel methodsFor:'accessors'!

borders
         "Returns the receiver's borders."

     borders isNil ifTrue: [ self initializeBorders ].
      ^borders
!

container
  "Returns the receiver's container."

   ^container
!

container: aContainer
  "Set the receiver's container."

       container := aContainer
!

contents
       "Returns the receiver's contents."

    contents isNil ifTrue: [ self initializeContents ].
    ^contents
!

rightBoundary
    "Returns the receiver's rightBoundary."

       rightBoundary isNil ifTrue: [ self initializeRightBoundary ].
  ^rightBoundary
!

rightBoundary: twips
    "Sets the receiver's rightBoundary."

  rightBoundary := twips
!

rightBoundary: aNumber in: units
         "Sets the receiver's rightBoundary."

  ^self rightBoundary: (self measure: aNumber in: units)
!

shading
        "Returns the receiver's shading."

     shading isNil ifTrue: [ self initializeShading ].
      ^shading
!

shading: aRTFShadingModel
         "Sets the receiver's shading."

        shading := aRTFShadingModel
!

textAlignment
     "Returns the receiver's textAlignment."

       textAlignment isNil ifTrue: [ self initializeTextAlignment ].
  ^textAlignment
!

textAlignment: aRTFCellTextAlignment
  "Sets the receiver's textAlignment."

  (aRTFCellTextAlignment isKindOf: Symbol) ifTrue: [
             ^self textAlignment: (RTFCellTextAlignment new perform: aRTFCellTextAlignment)
         ].
     textAlignment := aRTFCellTextAlignment
!

verticalAlignment
  "Returns the receiver's verticalAlignment."

   verticalAlignment isNil ifTrue: [ self initializeVerticalAlignment ].
  ^verticalAlignment
!

verticalAlignment: aRTFCellTextAlignment
  "Sets the receiver's verticalAlignment."

      (aRTFCellTextAlignment isKindOf: Symbol) ifTrue: [
             ^self verticalAlignment: (RTFCellTextAlignment new perform: aRTFCellTextAlignment)
     ].
     verticalAlignment := aRTFCellTextAlignment
! !

!RTFCellModel methodsFor:'contents alignment'!

alignContents: aSymbol
       "Add the command to alter contents alignment."

        self isEmpty ifFalse: [
                self addFirst: (RTFParagraphAlignment new perform: aSymbol)
    ]
!

centerContents
    "Add the command to alter contents alignment."

        self alignContents: #center
! !

!RTFCellModel methodsFor:'filling'!

add: anObject
     " Add anObject to the receiver's contents. "

  anObject notNil ifTrue: [
              self contents add: anObject.
   ].
!

addFirst: anObject
         " Add anObject to the receiver's contents. "

  anObject notNil ifTrue: [
              self contents addFirst: anObject.
      ].
!

fillWith: anObject
    " Fill the receiver's contents with anObject. "

       self initializeContents.
       self add: anObject.
! !

!RTFCellModel methodsFor:'initialization'!

initializeBorders
   " Private - Initializes the receiver's borders. "

     borders := Array new: 4.
!

initializeContents
       " Private - Initializes the receiver's contents. "

    contents := OrderedCollection new.
!

initializeRightBoundary
     " Private - Initializes the receiver's rightBoundary. "

       rightBoundary := nil "aNumber in twips or nil"
!

initializeShading
  " Private - Initializes the receiver's shading. "

     shading := nil. "an RTFShadingModel or nil"
!

initializeTextAlignment
      " Private - Initializes the receiver's textAlignment. "

       textAlignment := nil "RTFCellTextAlignment defaultTextAlignment"
!

initializeVerticalAlignment
      " Private - Initializes the receiver's verticalAlignment. "

   verticalAlignment := nil "RTFCellTextAlignment defaultVerticalAlignment"
! !

!RTFCellModel methodsFor:'printing'!

rtfContents
     " Private - Returns the contents of the receiver asRTF. "

     | result add |
         add := [:each|
                 result := result isNil
                         ifTrue: [each]
                         ifFalse: [ result , each ]
     ].
     self contents do: [:element|
           (element isKindOf: RTFElement)
                 ifTrue: [ add value: (RTFText with: element) ]
                 ifFalse: [ add value: (RTFText from: element) ]
        ].
     result isNil ifTrue: [ ^RTFText empty ].
       ^result
!

rtfSpec
      " Private - Returns the RTF replacement specification of the receiver. "

      ^RTFCell with: self rtfContents
! !

!RTFCellModel methodsFor:'query'!

hasBorders
  " Returns true if borders is set. "

   ^borders notNil
!

hasRightBoundary
   " Returns true if rightBoundary is set. "

     ^rightBoundary notNil
!

hasShading
         " Returns true if shading is set. "

   ^shading notNil
!

hasTextAlignment
       " Returns true if textAlignment is set. "

     ^textAlignment notNil
!

hasVerticalAlignment
       " Returns true if verticalAlignment is set. "

         ^verticalAlignment notNil
!

isEmpty
        " Returns true if the receiver isEmpty. "

     ^contents size = 0
! !

!RTFCellModel methodsFor:'shading'!

backColor: aColor
   " Set a cololred shade onto the receiver. "

   self shading: (RTFCellShadingModel new backColor: aColor)
!

gray: percent
      " Set a gray shade onto the receiver. "

       self shading: (RTFCellShadingModel gray: percent)
! !

!RTFCellModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFCellModel.st,v 1.2 2011/01/28 14:27:39 cg Exp $'
! !
