"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFElement subclass:#RTFComposite
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFComposite class methodsFor:'instance creation'!

with: anElement
       "Returns an instance of the receiver with anElement."

         ^self new
              add: anElement;
                yourself
!

with: element1
with: element2
     "Returns an instance of the receiver with elements."

  ^self new
              add: element1;
                 add: element2;
                 yourself
!

with: element1
with: element2
with: element3
   "Returns an instance of the receiver with elements."

  ^self new
              add: element1;
                 add: element2;
                 add: element3;
                 yourself
!

with: element1
with: element2
with: element3
with: element4
      "Returns an instance of the receiver with elements."

  ^self new
              add: element1;
                 add: element2;
                 add: element3;
                 add: element4;
                 yourself
!

withAll: elements
  "Returns an instance of the receiver with elements."

  ^self new
              addAll: elements;
              yourself
! !

!RTFComposite methodsFor:'accessing'!

first
      "Returns the first element."

  ^self elements first
!

firstElement
    "Returns the first rtfElement in the receiver."

       ^self detect: [:one| one isKindOf: RTFElement ]
!

last
         "Returns the last element."

   ^self elements last
!

token
    "Returns the token to be used when printing the receiver."

    ^self hasRTFElements ifTrue: [ self firstElement token ]
! !

!RTFComposite methodsFor:'element management'!

, anElement
     "Add anElement into the receiver.
      Returns the receiver."

        self add: anElement
!

add: anElement
      "Add anElement into the receiver."

    ^self elements add: anElement
!

addAll: aCollection
        "Add element in aCollection into the receiver."

       ^aCollection do: [:each| self add: each ]
!

collect: aBlock
    "Perform a collect into the elements of the receiver."

        ^self elements collect: aBlock
!

detect: aBlock
       "Perform a detect into the elements of the receiver."

         ^self elements detect: aBlock
!

detect: aBlock ifNone: bBlock
  "Perform a detect into the elements of the receiver."

         ^self elements detect: aBlock ifNone: bBlock
!

do: aBlock
  "Evaluate aBlock for all the elements in the receiver."

       ^self elements do: aBlock
!

includes: anElement
     "Returns true if the receiver includes anElement."

    ^self elements includes: anElement
!

oneOf: aKind
  "Returns an object of species aKind(aClass) includes in the receiver.
  If absent, creates a new open and add it to the receiver."

    ^self
          oneOf: aKind
           ifNone: [ self add: aKind new ]
!

oneOf: aKind ifNone: aBlock
      "Returns an object of species aKind(aClass) includes in the receiver.
  If absent, creates a new open and add it to the receiver."

    ^self
          detect: [:one| one isKindOf: aKind ]
           ifNone: aBlock
! !

!RTFComposite methodsFor:'printing'!

basicPrintRTFOn: aStream
     "Must write the receiver in RTF source format onto aStream."

  self printRTFElementsOn: aStream.
!

printNonRTFElement: anObject on: aStream
       " Prints a non-RTF element onto aStream.
       This method can be refined by subclasses."

    (RTFText from: anObject) printRTFOn: aStream
!

printRTFElementsOn: aStream
        "Must write the receiver's elements in RTF source format onto aStream."

       self elements do: [:each|
              (each respondsTo: #printRTFOn:)
                ifTrue: [ each printRTFOn: aStream ]
           ifFalse: [ self printNonRTFElement: each on: aStream ]
         ].
! !

!RTFComposite methodsFor:'query'!

hasRTFElements
         "Returns true if the receiver has a rtfElement."

      self elements
          detect: [:one| one isKindOf: RTFElement ]
              ifNone: [ ^false ].
    ^true
!

isComposite
  "Returns true if the receiver is a composite."

        ^true
!

isEmpty
      "Returns true if the receiver is empty."

      ^self elements isEmpty
! !

!RTFComposite class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFComposite.st,v 1.2 2011/01/28 14:28:58 cg Exp $'
! !
