"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

Object subclass:#RTFElement
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFElement class methodsFor:'colors management'!

colorNames
  " Default the default colorNames table. "

     ^#(
            #black
                 #blue
          (0 255 255)
            #green
                 (255 0 255)
            #red
           (255 255 0)
            #white
                 #darkBlue
              (0 128 128)
            #darkGreen
             (128 0 128)
            #darkRed
               (128 128 0)
            #darkGray
              #gray
  )
!

colorTable
         " Default the default color table. "

  ^#(
            (0 0 0)
                (0 0 255)
              (0 255 255)
            (0 255 0)
              (255 0 255)
            (255 0 0)
              (255 255 0)
            (255 255 255)
          (0 0 128)
              (0 128 128)
            (0 128 0)
              (128 0 128)
            (128 0 0)
              (128 128 0)
            (128 128 128)
          (192 192 192)
  )
! !

!RTFElement class methodsFor:'query'!

supportedTokens
   "Returns the collection of tokens supported by the receiver.
   The default action is to return an empty collection.
   This method can be refined by subclasses."

    ^#()
!

supports: aToken
    "Returns true if the receiver supports aToken. "

      ^self supportedTokens includes: aToken
!

whoSupports: aToken
  "Returns the subclass of the receiver who supports aToken.
     Returns nil if not found/implemented.
  Examples:
              RTFElement whoSupports: 'intbl'
                RTFElement whoSupports: 'cell'
                 RTFElement whoSupports: 'par'
  "

     ^self withAllSubclasses
                detect: [:one| one supports: aToken ]
          ifNone: []
! !

!RTFElement methodsFor:'colors management'!

color: aColor
    "Returns the index for color aColor.
   Generates an error if not found."

     ^self
          color: aColor
          ifAbsent: [ self error: 'Invalid color values.' ]
!

color: aColor ifAbsent: aBlock
       "Returns the index for color aColor.
   aColor can be:
                 #(red,green,blue) of rgb values (must be in colorTable)
                #index  aColor index (anInteger)
               nil for auto color
             aSymbol for a color name
       Evaluates aBlock if color not found."

         aColor isNil ifTrue: [ ^self color: #auto ].
   aColor isInteger ifTrue: [ ^aColor ].

         (aColor isKindOf: Array)
       ifTrue: [
              ^self
                  colorRed: (aColor at: 1) green: (aColor at: 2) blue: (aColor at: 3)
                    ifAbsent: aBlock
       ].

    ^self colorNamed: aColor ifAbsent: aBlock
!

colorNamed: aColor
         "Returns the index for color named aColor.
     Generates an error if not found."

     ^self
          colorNamed: aColor
             ifAbsent: [ self error: 'Invalid color values.' ]
!

colorNamed: aColor ifAbsent: aBlock
         "Returns the index for color named aColor or evaluates aBlock if not found."

  | colorTable |
         aColor == #auto ifTrue: [ ^0 ].
        colorTable := self colorNames.
         1 to: colorTable size do: [:index|
             (colorTable at: index) = aColor ifTrue: [ ^index ].
    ].
     ^aBlock value
!

colorNames
        " Returns the colorNames table known by the receiver.
  The default action is to return the default colorNames.
        This method can be refined by subclasses."

    ^self class colorNames
!

colorRed: red green: green blue: blue
        "Returns the index for color (red,green,blue)
  Generates an eror if color not found."

        ^self
          colorRed: red green: green blue: blue
          ifAbsent: [ self error: 'Invalid color values.' ]
!

colorRed: red green: green blue: blue ifAbsent: aBlock
       "Returns the index for color (red,green,blue)."

       | colorTable aColor |
  aColor := Array with: red with: green with: blue.
      colorTable := self colorTable.
         1 to: colorTable size do: [:index|
             (colorTable at: index) = aColor ifTrue: [ ^index ].
    ].
     ^aBlock value
!

colorTable
  " Returns the color table known by the receiver.
       The default action is to return the default colorTable.
        This method can be refined by subclasses."

    ^self class colorTable
! !

!RTFElement methodsFor:'composing'!

, anElement
       "Returns a composite element with the receiver and anElement."

        ^RTFOrderedComposite
           with: self
             with: anElement
! !

!RTFElement methodsFor:'converting'!

asByteArray
         "Must return the receiver as aByteArray in RTF source format."

        ^self asRTF asByteArray
!

asRTF
         "Must return the receiver as aString in RTF source format."

   | stream |
     stream := WriteStream on: (String new: 170).
   self printRTFOn: stream.
       ^stream contents
! !

!RTFElement methodsFor:'printing'!

printRTFOn: aStream
   "Must write the receiver in RTF source format onto aStream."

  self isGroup
   ifTrue: [
              aStream nextPut: ${.
           self canBeIgnored ifTrue: [ aStream nextPutAll: '\*' ].
                self basicPrintRTFOn: aStream.
                 aStream nextPut: $}.
   ]
      ifFalse: [
             self canBeIgnored ifTrue: [ aStream nextPutAll: '\*' ].
                self basicPrintRTFOn: aStream.
         ]
! !

!RTFElement methodsFor:'query'!

canBeIgnored
     "Must return true when the receiver can be ignored when reading."

     ^false
!

isComposite
      "Returns true if the receiver is a composite."

        ^false
!

isFontInfoItem
        "Must return true when the receiver is a font info element."

  ^false
!

isGroup
       "Must return true when the receiver is a group."

      ^false
!

isHeader
   "Returns true if the receiver is a header."

   ^false
!

isTextElement
         "Must return true when the receiver is a text element."

       ^false
!

paranoia
         "Returns true for paranoic(secure) debuggin."

         ^false
! !

!RTFElement methodsFor:'twips conversions'!

measure: aNumber in: aSymbol
      " Convert aNumber to twips. "

         (#( #cm #centimeters #centimeter) includes: aSymbol) ifTrue: [ ^self measure: aNumber inCentimeters: true ].
   (#( #twips #twip ) includes: aSymbol) ifTrue: [ ^aNumber ].
    (#( #in #inches #inch ) includes: aSymbol) ifTrue: [ ^self measure: aNumber inInches: true ].
  ^self error: 'Invalid measure.'
!

measure: aNumber inCentimeters: inCentimeters
    " Convert aNumber to twips. "

         ^inCentimeters
         ifTrue: [ aNumber  * 1440 *  0.3937  "There are 1440 twips per inch" ]
         ifFalse: [ aNumber ]
!

measure: aNumber inInches: inInches
       " Convert aNumber to twips. "

         ^inInches
      ifTrue: [ aNumber  * 1440 "There are 1440 twips per inch" ]
    ifFalse: [ aNumber ]
! !

!RTFElement class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFElement.st,v 1.2 2011/01/28 14:29:49 cg Exp $'
! !
