"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFOrderedComposite subclass:#RTFParagraphText
	instanceVariableNames:''
	classVariableNames:'Mappings'
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFParagraphText class methodsFor:'instance creation'!

empty
      "Returns an empty instance of the receiver."

  ^self new
!

from: anObject
        " Returns an instance of the receiver from anObject. "

        (anObject isKindOf: String) ifFalse: [
                 (anObject respondsTo: #value) ifTrue: [
                        ^self from: anObject value
             ].
             ^self from: anObject printString
       ].

    ^self new , anObject
! !

!RTFParagraphText class methodsFor:'converting'!

initializeMappings
    " Private - Initializes the mapping dictionary to be used for reserved characters in RTF. "

   Mappings := Dictionary new.

   #( $\ ${ $} ) do: [:c|
                 Mappings at: c put: (String with: $\ with: c).
         ].

    Mappings
               at: Character tab put: '\tab';
                 at: Character cr put: '\par',(String with: Character cr);
              yourself.
!

mappings
     " Private - Returns a mapping dictionary to be used for reserved characters in RTF. "

         Mappings isNil ifTrue: [ self initializeMappings ].
    ^Mappings
!

replacementFor: aCharacter using: replacements
       " Returns a replacement string for character aCharacter. "

    | result |
     result := replacements at: aCharacter ifAbsent: [ ^String with: aCharacter ].
  " result isSymbol ifTrue: [ ^self perform: result ]. "
         ^result
! !

!RTFParagraphText methodsFor:'converting'!

convertStringToRTF: aString
    " Converts aString to raw RTF text, transforming special characters. "

        | stream replacements converter |
      converter := self class.
       replacements := converter mappings.
    stream := WriteStream on: (String new: aString size+ 100).
     aString do: [:c|
               (replacements includesKey: c)
          ifTrue: [ stream nextPutAll: (converter replacementFor: c using: replacements) ]
               ifFalse: [ stream nextPut: c ]
         ].
     "
      Transcript
             cr;show: 'Converting to rtf:',aString;
                 cr;show: 'Converted:',stream contents.
         "
      ^stream contents
! !

!RTFParagraphText methodsFor:'element management'!

, anElement
       "Add anElement into the receiver.
      Returns the receiver."

        (anElement isKindOf: String)
   ifTrue: [ self elements add: anElement ]
       ifFalse: [ super , anElement ].
        ^self
!

add: anElement
       "Add anElement into the receiver."

    (self paranoia and: [ anElement isTextElement not ])
   ifTrue: [ ^self error: 'Invalid element.' ].

  ^super add: anElement
! !

!RTFParagraphText methodsFor:'printing'!

printNonRTFElement: anObject on: aStream
         " Prints a non-RTF element onto aStream. "

    (anObject isKindOf: String) ifFalse: [
                 ^self printNonRTFElement: anObject printString on: aStream
     ].

    aStream space.
         aStream nextPutAll: (self convertStringToRTF: anObject)
! !

!RTFParagraphText methodsFor:'query'!

isTextElement
      "Must return true when the receiver is a text element."

       ^true
! !

!RTFParagraphText class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFParagraphText.st,v 1.2 2011/01/28 14:36:51 cg Exp $'
! !
