"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFModel subclass:#RTFShadingModel
	instanceVariableNames:'pattern lineColor backColor value'
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFShadingModel class methodsFor:'instance creation'!

backwardDiagonal
        "Returns an instance of the receiver."

        ^self pattern: #backwardDiagonal
!

color: aColor percent: percent
  "Returns an instance of the receiver."

        ^self new
              color: aColor percent: percent;
                yourself
!

cross
   "Returns an instance of the receiver."

        ^self pattern: #cross
!

darkBackwardDiagonal
   "Returns an instance of the receiver."

        ^self pattern: #darkBackwardDiagonal
!

darkCross
  "Returns an instance of the receiver."

        ^self pattern: #darkCross
!

darkDiagonalCross
       "Returns an instance of the receiver."

        ^self pattern: #darkDiagonalCross
!

darkForwardDiagonal
      "Returns an instance of the receiver."

        ^self pattern: #darkForwardDiagonal
!

darkHorizontal
  "Returns an instance of the receiver."

        ^self pattern: #darkHorizontal
!

darkVertical
  "Returns an instance of the receiver."

        ^self pattern: #darkVertical
!

diagonalCross
     "Returns an instance of the receiver."

        ^self pattern: #diagonalCross
!

forwardDiagonal
    "Returns an instance of the receiver."

        ^self pattern: #forwardDiagonal
!

gray: percent
         "Returns an instance of the receiver."

        ^self new
              gray: percent;
                 yourself
!

horizontal
         "Returns an instance of the receiver."

        ^self pattern: #horizontal
!

pattern: aPattern
        "Returns an instance of the receiver."

        ^self new
              pattern: aPattern;
             yourself
!

percent: aNumber
       "Returns an instance of the receiver."

        ^self new
              percent: aNumber;
              yourself
!

style: aPattern
         "Returns an instance of the receiver."

        ^self pattern: aPattern
!

value: aNumber
         "Returns an instance of the receiver."

        ^self new
              value: aNumber;
                yourself
!

vertical
        "Returns an instance of the receiver."

        ^self pattern: #vertical
! !

!RTFShadingModel methodsFor:'accessors'!

backColor
    "Returns the receiver's backColor."

   backColor isNil ifTrue: [ self initializeBackColor ].
  ^backColor
!

backColor: aNumber
         "Sets the receiver's backColor."

      backColor := self color: aNumber
!

lineColor
     "Returns the receiver's lineColor."

   lineColor isNil ifTrue: [ self initializeLineColor ].
  ^lineColor
!

lineColor: aNumber
   "Sets the receiver's lineColor."

      lineColor := self color: aNumber
!

pattern
       "Returns the receiver's pattern."

     pattern isNil ifTrue: [ self initializePattern ].
      ^pattern
!

pattern: aPattern
   "Sets the receiver's pattern."

        (aPattern isKindOf: Symbol) ifTrue: [
          ^self pattern: (self patternClass new perform: aPattern; yourself)
     ].
     pattern := aPattern
!

percent: aNumber
      "Sets the receiver's value with aNumber (in percent)."

        self value: (aNumber * 100) truncated
!

value
        "Returns the receiver's value (or nil)."

      value isNil ifTrue: [ self initializeValue ].
  ^value
!

value: aNumber
   "Sets the receiver's value."

  value := aNumber
! !

!RTFShadingModel methodsFor:'initialization'!

initializeBackColor
  " Private - Initializes the receiver's backColor.  "

  backColor := nil.
!

initializeLineColor
   " Private - Initializes the receiver's lineColor.  "

  lineColor := nil.
!

initializePattern
        " Private - Initializes the receiver's pattern.  "

    pattern := nil.
!

initializeValue
    " Private - Initializes the receiver's value.  "

      value := nil.
! !

!RTFShadingModel methodsFor:'printing'!

rtfSpec
       " Private - Returns the RTF replacement specification of the receiver. "

      | result add |
         add := [:aValue|
               aValue notNil ifTrue: [
                        result := result isNil
                                 ifTrue: [ aValue ]
                             ifFalse: [ result , aValue ]
           ]
      ].

    add value: self pattern.
       self hasBackColor ifTrue: [
            add value: (self backgroundBackColorClass new value: self backColor)
   ].
     self hasLineColor ifTrue: [
            add value: (self backgroundLineColorClass new value: self lineColor)
   ].
     self hasValue ifTrue: [
                add value: (self shadingClass new value: self value)
   ].
     ^result
! !

!RTFShadingModel methodsFor:'query'!

hasBackColor
     "Returns true if the receiver has the backColor defined."

     ^backColor notNil
!

hasLineColor
         "Returns true if the receiver has the lineColor defined."

     ^lineColor notNil
!

hasPattern
     "Returns true if the receiver has the pattern defined."

       ^pattern notNil
!

hasValue
   "Returns true if the receiver has the value defined."

         ^value notNil
! !

!RTFShadingModel methodsFor:'settings'!

backwardDiagonal
        "Set the receiver with this format."

  ^self pattern: #backwardDiagonal
!

color: aColor percent: percent
         "Set the receiver with this format."

  self
           backColor: aColor;
             percent: percent
!

cross
   "Set the receiver with this format."

  ^self pattern: #cross
!

darkBackwardDiagonal
       "Set the receiver with this format."

  ^self pattern: #darkBackwardDiagonal
!

darkCross
         "Set the receiver with this format."

  ^self pattern: #darkCross
!

darkDiagonalCross
    "Set the receiver with this format."

  ^self pattern: #darkDiagonalCross
!

darkForwardDiagonal
    "Set the receiver with this format."

  ^self pattern: #darkForwardDiagonal
!

darkHorizontal
     "Set the receiver with this format."

  ^self pattern: #darkHorizontal
!

darkVertical
        "Set the receiver with this format."

  ^self pattern: #darkVertical
!

diagonalCross
  "Set the receiver with this format."

  ^self pattern: #diagonalCross
!

forwardDiagonal
        "Set the receiver with this format."

  ^self pattern: #forwardDiagonal
!

gray: percent
      "Set the receiver with this format."

  self
           backColor: 8;
          percent: percent
!

horizontal
   "Set the receiver with this format."

  ^self pattern: #horizontal
!

vertical
        "Set the receiver with this format."

  ^self pattern: #vertical
! !

!RTFShadingModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFShadingModel.st,v 1.2 2011/01/28 14:38:10 cg Exp $'
! !
