"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFComposite subclass:#RTFSortedComposite
	instanceVariableNames:'elements'
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFSortedComposite methodsFor:'accessing'!

elements
  "Must return all the elements of the receiver in a valid order."

      elements isNil ifTrue: [ self initializeElements ].
    ^elements
!

elements: aCollection
       " Private - Sets the receiver's elements of the receiver to aCollection."

     elements := aCollection
! !

!RTFSortedComposite methodsFor:'element management'!

add: anElement
  "Add anElement into the receiver."

    self elements: (self elementsWith: anElement).
         ^anElement
! !

!RTFSortedComposite methodsFor:'initialization'!

initializeElements
  " Private - Initializes the receiver's elements. "

    elements := OrderedCollection new.
! !

!RTFSortedComposite methodsFor:'sorting contents'!

basicSortOrderFor: anElement
       " Private - Returns the sort order for anElement (or nil)."

   | alternatives alternative |
   alternatives := self bnfAlternatives.
  1 to: alternatives size do: [:index|
           alternative := alternatives at: index.
                 (alternative isKindOf: Symbol) ifTrue: [ "class name assumed"
                  alternative := Smalltalk at: alternative ifAbsent: [ alternative "not implemented" ].
                  ((alternative isKindOf: Class) and: [ anElement isKindOf: alternative ])
                               ifTrue: [ ^index ].
            ].
             anElement token = alternative ifTrue: [ ^index ].
      ].
     ^self defaultSortOrderFor: anElement
!

bnfAlternatives
  " Private - Returns the array of extended BNF alternatives for receiver's contents."

  ^self class bnfAlternatives
!

defaultSortOrderFor: anElement
    " Private - Return the default sort order (or nil).
    Default answer is nil, generating an error when trying to sort invalid elements.
       This method can be refined by subclasses to support dynamic element insertion."

       ^nil
!

elementsWith: anElement
  " Private - Returns a copy of the receiver's elements with anElement added. "

         | stream result order |
        order := self sortOrderFor: anElement.
         result := self elements species new.
   stream := ReadStream on: self elements.
        [stream atEnd not and: [ (self sortOrderFor: stream peek) <= order ]]
  whileTrue: [
           result add: stream next.
               stream atEnd ifTrue: [
                         result add: anElement.
                         ^result
                ].
     ].
     result add: anElement.
         [stream atEnd] whileFalse: [ result add: stream next ].
        ^result
!

sortOrderFor: anElement
         "Must return the sort order for anElement."

   | result |
     result := self basicSortOrderFor: anElement.
   result isNil ifTrue: [ self error: 'Invalid element.' ].
       ^result
! !

!RTFSortedComposite class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFSortedComposite.st,v 1.2 2011/01/28 14:38:23 cg Exp $'
! !
