"{ Package: 'stx:goodies/docFormats/rtf' }"

"{ NameSpace: RTF1 }"

RTFModel subclass:#RTFTableModel
	instanceVariableNames:'cells spaceBetweenRows leftMargin borders columnsBorders
		columnsShadings columnsWidths labelRows'
	classVariableNames:''
	poolDictionaries:''
	category:'RTF-Dist'
!


!RTFTableModel class methodsFor:'instance creation'!

sending: selectors to: objects
   "Returns an instance of the receiver filled with cells resulting of sending selectors to objects."

    ^self sending: selectors to: objects labels: nil
!

sending: selectors to: objects labels: labels
      "Returns an instance of the receiver filled with cells resulting of sending selectors to objects."

    ^self sending: selectors to: objects labels: labels grayPercent: nil
!

sending: selectors to: objects labels: labels grayPercent: gray
        "Returns an instance of the receiver filled with cells resulting of sending selectors to objects."

    | result height |
      height := labels size > 0
              ifTrue: [ objects size + 1 ]
           ifFalse: [ objects size ].
     result := self size: selectors size @ height.
  result fillCellsSending: selectors to: objects labels: labels grayPercent: gray.
       ^result
!

size: size
      "Returns an instance of the receiver with size=size."

         (size isKindOf: Point) ifTrue: [
               ^self new
                      resizeTo: size;
                        yourself
       ].
     ^self size: size@size
!

width: width height: height
  "Returns an instance of the receiver with width@height elements."

     ^self size: width@height
! !

!RTFTableModel methodsFor:'accessors'!

borders
   "Returns the receiver's borders."

     borders isNil ifTrue: [ self initializeBorders ].
      ^borders
!

borders: aRectangle
    "Sets the receiver's borders."

        borders := aRectangle
!

bordersForColumn: aColumn
     "Returns the border for column aColumn."

      aColumn isInteger ifTrue: [
            ^self bordersForColumn: (self column: aColumn)
         ].
     ^self columnsBorders at: aColumn ifAbsent: [
           self columnsBorders at: aColumn put: self defaultColumnBorder
  ]
!

cellAt: aPoint
      "Returns the cell at aPoint."

         ^(self cells at: aPoint x) at: aPoint y
!

cells
      "Returns the receiver's cells."

       cells isNil ifTrue: [ self initializeCells ].
  ^cells
!

cells: aCollection
    "Sets the receiver's cells."

  cells := aCollection
!

column: index
    "Returns the receiver's column at index (as a collection)."

   ^self cells at: index
!

columns
    "Returns the receiver's columns in a collection."

     ^(1 to: self width) collect: [:index| self column: index ]
!

columnsBorders
     "Returns the receiver's columnsBorders."

      columnsBorders isNil ifTrue: [ self initializeColumnsBorders ].
        ^columnsBorders
!

columnsShadings
        "Returns the receiver's columnsShadings."

     columnsShadings isNil ifTrue: [ self initializeColumnsShadings ].
      ^columnsShadings
!

columnsWidths
  "Returns the receiver's columnsWidths."

       columnsWidths isNil ifTrue: [ self initializeColumnsWidths ].
  ^columnsWidths
!

extent
       "Returns the receiver's extent."

      ^self width @ self height
!

height
        "Returns the receiver's height."

      self cells size = 0 ifTrue: [ ^0 ].
    ^self cells first size
!

indexOfRow: aRow
   "Returns the index location of aRow.
   Computed from first cell."

    aRow size = 0 ifTrue: [ ^0 ].
  ^(self column: 1) indexOf: aRow first ifAbsent: [ 0 ]
!

labelRows
         "Returns the receiver's labelRows."

   labelRows isNil ifTrue: [ self initializeLabelRows ].
  ^labelRows
!

leftMargin
    "Returns the receiver's leftMargin."

  leftMargin isNil ifTrue: [ self initializeLeftMargin ].
        ^leftMargin
!

leftMargin: aValue
         "Sets the receiver's leftMargin."

     leftMargin := aValue
!

row: index
   "Returns the receiver's row at index (as a collection)."

      ^(1 to: self width) collect: [:x| (self cells at: x) at: index ].
!

rows
  "Returns the receiver's rows in a collection."

        ^(1 to: self height) collect: [:index| self row: index ]
!

shadingForColumn: aColumn
  "Returns the shading defined for aColumn."

    aColumn isInteger ifTrue: [
            ^self shadingForColumn: (self column: aColumn)
         ].
     ^self columnsShadings at: aColumn ifAbsent: [
          self columnsShadings at: aColumn put: self defaultColumnShading
        ]
!

spaceBetweenRows
       "Returns the receiver's spaceBetweenRows."

    spaceBetweenRows isNil ifTrue: [ self initializeSpaceBetweenRows ].
    ^spaceBetweenRows
!

spaceBetweenRows: aValue
        "Sets the receiver's spaceBetweenRows."

       spaceBetweenRows := aValue
!

width
    "Returns the receiver's width."

       ^self cells size
!

widthForColumn: aColumn
       "Returns the width for aColumn."

      aColumn isInteger ifTrue: [
            ^self widthForColumn: (self column: aColumn)
   ].
     ^self columnsWidths
            at: aColumn
            ifAbsent: [ self defaultWidthForColumn: aColumn ]
!

widthForColumn: aColumn put: twips
    "Sets the width for aColumn."

         aColumn isInteger ifTrue: [
            ^self
                  widthForColumn: (self column: aColumn)
                         put: twips
     ].
     twips isNil ifTrue: [
          ^self columnsWidths removeKey: aColumn
         ].
     ^self columnsWidths at: aColumn put: twips
!

widthForColumns: aCollection
     "Sets the widths of receiver's columns to aCollection (magnitudes in twips).
   nil to be set to default value."

      ^self widthForColumns: aCollection in: #twips
!

widthForColumns: aCollection in: aSymbol
         "Sets the widths of receiver's columns to aCollection (magnitudes in twips or centimeters).
    nil to be set to default value."

      | aValue |
     1 to: aCollection size do: [:index|
            aValue := aCollection at: index.
               self widthForColumn: index put: (self measure: aValue in: aSymbol).
    ].
! !

!RTFTableModel methodsFor:'borders, grids and shadings'!

bordersWidth: twips
  "Set the receiver's borders width to twips."

  self borders width: twips.
!

clearBorders
   "Clear the receiver's  borders settings."

     borders := nil
!

clearColumnsBorders
        "Clear the receiver's column borders settings."

       columnsBorders := nil
!

clearColumnsShadings
  "Clear the receiver's shadings settings."

     columnsShadings := nil
!

grayColumn: aColumn percent: percent
     "Gray cells in collumn aColumn."

      self withCellsInColumn: aColumn do: [:aCell| aCell gray: percent ].
!

grayRow: aRow percent: percent
     "Gray cells in row aRow."

     self withCellsInRow: aRow do: [:aCell| aCell gray: percent ].
!

grid
        "Grid the receiver with borders of default width."

    ^self gridWidth: self gridWidthDefault
!

gridWidth: twips
    "Grid the receiver with borders of width twips."

      ^self gridWidth: twips in: #twips
!

gridWidth: twips in: units
    "Grid the receiver with borders of width twips."

      self columns do: [:column|
             (self bordersForColumn: column) width: twips in: units
         ].
!

gridWidthDefault
     "Returns the default grid width in twips."

    ^10
!

shadingFor: aCell inColumn: aColumn
   "Returns the border for aCell in column aColumn."

     ^aCell hasShading
      ifTrue: [ aCell shading ]
      ifFalse: [ self shadingForColumn: aColumn ]
! !

!RTFTableModel methodsFor:'filling cells'!

fillCellsSending: selectors to: objects labels: labels
  " Fill receiver's contents with the objects resulting of sending selectors to objects. "

      ^self fillCellsSending: selectors to: objects labels: labels grayPercent: nil
!

fillCellsSending: selectors to: objects labels: labels grayPercent: gray
  " Fill receiver's contents with the objects resulting of sending selectors to objects. "

      | selector receiver offset aCell |
     self fillWith: nil.
    labels size > 0
        ifTrue: [
              1 to: labels size do: [:i|
                     aCell := self cellAt: i @ 1.
                   aCell fillWith: (labels at: i).
                        gray notNil ifTrue: [
                          aCell gray: gray
                       ].
             ].
             offset := 1.
           self labelRows add: 1.
         ]
      ifFalse: [ offset := 0 ].
      1 to: objects size do: [:y|
            receiver := objects at: y.
             receiver notNil ifTrue: [
                      1 to: selectors size do: [:x|
                          selector := selectors at: x.
                           (self cellAt: x@ (y + offset))
                                         fillWith: (self perform: selector on: receiver)
                        ].
             ].
     ].
!

fillDoing: aBlock
    " Fill receiver's contents with evaluating aBlock for each cell in the receiver. "

    self columns do: [:column|
             column do: [:cell| cell fillWith: (aBlock value: cell) ].
      ].
!

fillWith: anObject
         " Fill receiver's contents with anObject. "

   self columns do: [:column|
             column do: [:cell| cell fillWith: anObject ].
  ].
!

perform: actions on: aReceiver
       " Retruns the result of evaluating action(s) on aReceiver. "

  | receiver |
   actions isNil ifTrue: [ ^nil ].
        receiver := aReceiver.
         (actions isKindOf: Symbol) ifTrue: [ ^receiver perform: actions ].
     (actions respondsTo: #value:) ifTrue: [ ^actions value: receiver ].
    (actions isKindOf: String) ifTrue: [ ^actions ].
       actions do: [:each|
            receiver := self perform: each on: receiver.
   ].
     ^receiver
! !

!RTFTableModel methodsFor:'initialization'!

defaultColumnBorder
  " Private - Returns default borders for a column. "

   ^RTFColumnBordersModel new
!

defaultColumnShading
       " Private - Returns default borders for a column. "

   ^RTFColumnShadingModel new
!

defaultWidthForColumn: aColumn
  " Private - Returns default width for a column (in twips). "

  ^1725 "twips"
!

initializeBorders
     " Private - Initializes the receiver's Borders. "

     borders := RTFRowBordersModel new
!

initializeCells
   " Private - Initializes the receiver's cells. "

       "initialized with only a cell."
        cells := OrderedCollection with: (self newColumn: 1).
!

initializeColumnsBorders
        " Private - Initializes the receiver's columnsBorders. "

      columnsBorders := Dictionary new.
!

initializeColumnsShadings
       " Private - Initializes the receiver's columnsShadings. "

     columnsShadings := Dictionary new.
!

initializeColumnsWidths
       " Private - Initializes the receiver's columnsWidths. "

       columnsWidths := Dictionary new.
!

initializeLabelRows
       " Private - Initializes the receiver's labelRows. "

   labelRows := Set new.
!

initializeLeftMargin
  " Private - Initializes the receiver's leftMargin. "

  leftMargin := -70 "twips".
!

initializeSpaceBetweenRows
     " Private - Initializes the receiver's spaceBetweenRows. "

    spaceBetweenRows := 70 "twips".
!

newCell
       " Private - Returns a new cell. "

     ^RTFCellModel in: self
!

newColumn: size
   " Private - Returns a new column of size size. "

      | column |
     column := OrderedCollection new: size.
         size timesRepeat: [ column add: self newCell ].
        ^column
! !

!RTFTableModel methodsFor:'iterations'!

withCellsInColumn: aColumn do: aBlock
        "Evaluates aBlock with cells in aColumn."

     aColumn isInteger ifTrue: [
            ^self withCellsInColumn: (self column: aColumn) do: aBlock
     ].
     aColumn do: aBlock.
!

withCellsInRow: aRow do: aBlock
   "Evaluates aBlock with cells in aRow."

        aRow isInteger ifTrue: [
               ^self withCellsInRow: (self row: aRow) do: aBlock
      ].
     aRow do: aBlock.
!

withColumn: aColumn do: aBlock
     "Evaluates aBlock with cells in aColumn."

     ^self withCellsInColumn: aColumn do: aBlock
!

withRow: aRow do: aBlock
  "Evaluates aBlock with cells in aRow."

        ^self withCellsInRow: aRow do: aBlock
! !

!RTFTableModel methodsFor:'printing'!

bordersFor: aCell inColumn: aColumn
      "Returns the border for aCell in column aColumn."

     ^aCell hasBorders
      ifTrue: [ aCell borders ]
      ifFalse: [ self bordersForColumn: aColumn ]
!

boundaryFor: aCell in: aRow andColumn: aColumn
     "Returns the RTF cell width for column aColumn."

      ^RTFRightBoundary value: (self rightBoundaryFor: aCell in: aRow andColumn: aColumn)
!

boundaryForColumn: aColumn
      "Returns the RTF cell width for column aColumn."

      ^RTFRightBoundary value: (self rightBoundaryForColumn: aColumn)
!

cellAlignmentFor: aCell inColumn: aColumn
         " Private - Returns the alignment definition of aCell in column aColumn. "

    ^aCell hasVerticalAlignment
    ifTrue: [ aCell verticalAlignment ]
    ifFalse: [ self cellAlignmentForColumn: aColumn ]
!

cellAlignmentForColumn: aColumn
    " Private - Returns the definition of cell alignment for column aColumn. "

    ^aColumn first verticalAlignment "we will use the vertical alignment for the first cell in the column"
!

paragraphInTableDefinition
      " Private - Returns the definition of RTF elements definin that the next paragraph will be defined in table context. "

        ^RTFInlineElement value: '\pard \nowidctlpar\widctlpar\intbl\adjustright '

    "       \pard \nowidctlpar\widctlpar\intbl\adjustright
         ^RTFParagraphReset new
         ,RTFParagraphNoWidowControl new
        ,RTFParagraphWidowControl new
  ,RTFParagraphInTable new
       "
!

rightBoundaryFor: aCell in: aRow andColumn: aColumn
   " Returns the right boundary (in twips) for aCell in aRow at column aColumn. "

        aCell hasRightBoundary ifTrue: [ ^aCell rightBoundary ].
       ^self rightBoundaryForColumn: aColumn in: aRow
!

rightBoundaryForColumn: aColumn
    " Returns the right boundary (in twips) for the column aColumn. "

     | result |
     result := 0.
   self columns do: [:each|
               result := result + (self widthForColumn: each).
                each == aColumn ifTrue: [ ^result ].
   ].
     ^result
!

rightBoundaryForColumn: aColumn in: aRow
       " Returns the right boundary (in twips) for the column aColumn at aRow. "

     ^self rightBoundaryForColumn: aColumn
!

rtfSpec
         " Private - Returns the RTF replacement specification of the receiver. "

      | result add |
         add := [:each|
                 result := result isNil
                         ifTrue: [each]
                         ifFalse: [ result , each ]
     ].
     self rows do: [:row| add value: (self rtfSpecForRow: row) ].
   ^result

" Please see RTF specification 'Table Definitions'  Page 43 Sec. 1
    There is no RTF table group; instead, tables are specified as paragraph properties.
    A table is represented as a sequence of table rows. A table row is a continuous
         sequence of paragraphs partitioned into cells. The table row begins with
       the \trowd control word and ends with the \row control word.
  Every paragraph that is contained in a table row must have the \intbl control
   word specified or inherited from the previous paragraph. A cell may have more
          than one paragraph in it; the cell is terminated by a cell mark (the \cell control word),
      and the row is terminated by a row mark (the \row control word). Table rows can also
   be positioned. In this case, every paragraph in a table row must have the same
         positioning controls (see the <apoctl> controls on page 29 of this Application Note).
         Table properties may be inherited from the previous row; therefore, a series of table
   rows may be introduced by a single <tbldef>.
"
!

rtfSpecForCellsContentsIn: aRow
        " Private - Returns the definition of receiver's cells in aRow. "

     | result add |
         add := [:each|
                 result := result isNil
                         ifTrue: [each]
                         ifFalse: [ result , each ]
     ].
     aRow do: [:aCell|
              add value: RTFParagraphReset new.
              add value: RTFParagraphInTable new.
            add value: aCell rtfSpec.
      ].
     add value: RTFParagraphReset new.
      add value: RTFParagraphInTable new.
    add value: RTFTableRowElement new.
     ^result
!

rtfSpecForCellsDefinitionIn: aRow
        " Private - Returns the definition of cells in aRow. "

        | result add stream column |
   add := [:each|
                 each notNil ifTrue: [
                  result := result isNil
                                 ifTrue: [each]
                                 ifFalse: [ result , each ]
             ].
     ].
     stream := ReadStream on: self columns.
         aRow do: [:aCell|
              column := stream next.
                 add value: (self cellAlignmentFor: aCell inColumn: column).
            add value: ((self hasBordersFor: aCell inColumn: column)
                                                        ifTrue: [self bordersFor: aCell inColumn: column]).
           add value: ((self hasShadingFor: aCell inColumn: column)
                                                        ifTrue: [self shadingFor: aCell inColumn: column]).
           add value: (self textAlignmentFor: aCell inColumn: column).
            add value: (self boundaryFor: aCell in: aRow andColumn: column).
       ].
     ^result
!

rtfSpecForRow: aRow
        " Private - Returns the RTF specification for the row in the receiver. "

      | row |
        row := RTFRow new.
     row
            add: RTFTableRowDefaults new;
          add: (RTFCellsGap twips: self spaceBetweenRows // 2);
          add: (RTFRowLeftPosition twips: self leftMargin);
              yourself.
      (self isTableRowHeader: aRow) ifTrue: [
                row add: RTFTableRowHeader new.
        ].
     self hasBorders ifTrue: [ row add: self borders ].
     row
            add: (self rtfSpecForCellsDefinitionIn: aRow);
                 add: (self rtfSpecForCellsContentsIn: aRow);
           yourself.
      ^row
!

textAlignmentFor: aCell inColumn: aColumn
        " Private - Returns the text alignment definition of aCell in column aColumn. "

       ^aCell hasTextAlignment
        ifTrue: [ aCell textAlignment ]
        ifFalse: [ self textAlignmentForColumn: aColumn ]
!

textAlignmentForColumn: aColumn
       " Private - Returns the definition of text alignment for column aColumn. "

    ^aColumn first textAlignment
! !

!RTFTableModel methodsFor:'query'!

hasBorders
        "Returns true if the receiver is bordered."

   ^borders notNil
!

hasBordersFor: aCell inColumn: aColumn
     "Returns true if the receiver has borders defined for acell in column aColumn."

       ^aCell hasBorders or: [ self hasBordersForColumn: aColumn ]
!

hasBordersForColumn: aColumn
         "Returns true if the receiver has borders defined for column aColumn."

        aColumn isInteger ifTrue: [
            ^self hasBordersForColumn: (self column: aColumn)
      ].
     ^self columnsBorders includesKey: aColumn
!

hasShadingFor: aCell inColumn: aColumn
       "Returns true if the receiver has shading defined for aCell in column aColumn."

       ^aCell hasShading or: [ self hasShadingForColumn: aColumn ]
!

hasShadingForColumn: aColumn
  "Returns true if the receiver has shading defined for column aColumn."

        aColumn isInteger ifTrue: [
            ^self hasShadingForColumn: (self column: aColumn)
      ].
     ^self columnsShadings includesKey: aColumn
!

isTableRowHeader: aRow
  "Returns true if aRow is considered a row header."

    ^self labelRows includes: (self indexOfRow: aRow)
!

isTextElement
         "Must return true when the receiver is a text element."

       ^true
! !

!RTFTableModel methodsFor:'resizing'!

resizeTo: aPoint
       " Private - Resize the receiver to aPoint. "

  | column |
     self cells size > aPoint x ifTrue: [ "crop columns"
            self cells: (self cells copyFrom: 1 to: aPoint x)
      ].
     [self cells size < aPoint x ] whileTrue: [ "grow columns"
              self cells add: (self newColumn: aPoint y)
     ].

    1 to: self cells size do: [:index|
             column := self cells at: index.
                column size > aPoint y ifTrue: [ "crop rows"
                   self cells at: index put: (column copyFrom: 1 to: aPoint y)
            ].
             [column size < aPoint y ] whileTrue: [ "grow rows"
                     column add: self newCell
               ].
     ].
! !

!RTFTableModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/docFormats/rtf/RTFTableModel.st,v 1.2 2011/01/28 14:39:12 cg Exp $'
! !
