"{ Package: 'stx:goodies/gitocello' }"

Object subclass:#GCCallout
	instanceVariableNames:'localPath'
	classVariableNames:'CalloutDict GitBinaryPath'
	poolDictionaries:''
	category:'Gitocello-Core'
!


!GCCallout class methodsFor:'callout'!

apiLinuxLibc5Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.5'>
	^ self externalCallFailed 
!

apiLinuxLibc6Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.6'>
	^ self externalCallFailed 
!

apiMacOSXCallout: aCommand
	<cdecl: long 'system' (char*) module: 'libSystem.dylib'>
	| errCode |
	errCode := ExternalFunction getLastError. 
	^ errCode = 13 "Unable to find function address. dylibs are not searched"
		ifTrue: [self error: 'Cannot call libSystem.dylib. 
			Please set "SqueakPluginsBuiltInOrLocalOnly" in your VM''s Info.plist to false']
		ifFalse: [self externalCallFailed]
!

apiWindowsCallout: aCommand
	<apicall: long 'system' ( char* ) module: 'msvcrt.dll'>
	^ self externalCallFailed
!

callout: aCommand

	^ (Smalltalk classNamed: 'OSProcess')
		ifNil: [(self calloutDict 
				at: SmalltalkImage current platformName asLowercase
				ifAbsent: [[:cmd | self error: 'Callouts not implemented for your platform!!']]) 
					value: aCommand]
		ifNotNilDo: [:class | class waitForCommand: aCommand].
!

calloutDict

	CalloutDict ifNil: [CalloutDict := Dictionary new
		at: 'unix' put: [:cmd | self linuxCallout: cmd];
		at: 'win32' put: [:cmd | self windowsCallout: cmd];
		at: 'mac os' put: [:cmd | self apiMacOSXCallout: cmd];
		yourself].
	^ CalloutDict
!

linuxCallout: aCommand
	"Try with new libc6 and older libc5"
	[self apiLinuxLibc6Callout: aCommand] 
		on: Error
		do: [self apiLinuxLibc5Callout: aCommand]
!

windowsCallout: aCommand
	"Escape newlines for Windows CmdLine"
	| escapedCommand |
	escapedCommand := ((aCommand 
		copyReplaceAll: Character cr asString with: ' ') 
			copyReplaceAll: Character lf asString with: ' ').
	self apiWindowsCallout: escapedCommand.
! !

!GCCallout class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCCallout.st,v 1.1 2011/08/31 15:56:37 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCCallout.st,v 1.1 2011/08/31 15:56:37 cg Exp $'
! !
