"{ Package: 'stx:goodies/gitocello' }"

Object subclass:#GCMapper
	instanceVariableNames:'systemOrganizer packageName repoDir files'
	classVariableNames:''
	poolDictionaries:''
	category:'Gitocello-Core'
!

GCMapper comment:'I am used to file out a package to a repository. I do not care at all what kind of repository. 
''repoDir'' and ''packageName'' are the only options that should be set on me.
You can use my ''files'' field to find out which files I have filed out during my lifetime.

I just map packages and classes to files. Like so:
The packages and classes
		Package (Class1)
		Package-Core (Class2)
		Package-Base (Class3, Class4)
become
		Package/
			|- Core/
				|- Class2.st
			|- Base/
				|- Class3.st
				|- Class4.st
			|- Class1.st'
!


!GCMapper class methodsFor:'instance creation'!

newFor: aPackage

	^ self basicNew
		packageName: aPackage;
		initialize;
		yourself
! !

!GCMapper methodsFor:'accessing'!

files

	^ files
!

packageName
	"Answer the value of packageName"

	^ packageName
!

packageName: anObject
	"Set the value of packageName"

	packageName := anObject
!

repoDir

	^ repoDir
!

repoDir: aFileDirectory

	aFileDirectory assureExistence.
	repoDir := aFileDirectory
! !

!GCMapper methodsFor:'fileOut'!

createFolder: aSubfolder

	(repoDir directoryExists: aSubfolder)
		ifFalse: [repoDir createDirectory: aSubfolder]
!

fileOutCategory: aCategory
	"This creates a sub-folder in the repository for the categories classes"
	| subfolder |
	subfolder := (aCategory asString findBetweenSubStrs: '-') last.
	self createFolder: subfolder.
	(systemOrganizer superclassOrder: aCategory) do: [:each | 
		self 
			fileOutClass: each 
			in: (repoDir directoryNamed: subfolder) pathName]
!

fileOutClass: aClass in: aPath
	"This files out the class in the according sub-folder"
	| stream path |
	stream := WriteStream on: (String new: 100).
	aClass fileOutOn: stream moveSource: false toFile: 0.
	path := (aPath, FileDirectory pathNameDelimiter asString, aClass name).
	self writeSourceCodeFrom: stream to: path
!

fileOutExtensions: extensionMethods
	"For each extended class collect the extension Methods and file them out"
	| dict |
	self createFolder: 'Extensions'.
	dict := Dictionary new.
	extensionMethods do: [:each | 
		dict at: each classSymbol ifAbsentPut: OrderedCollection new.
		(dict at: each classSymbol) add: each methodSymbol].
	dict keysDo: [:key || stream file |
		stream := WriteStream on: (String new: 1000).
		file := (repoDir directoryNamed: 'Extensions') pathName, 
				FileDirectory pathNameDelimiter asString, 
				key asString.
		(dict at: key) do: [:method |
			(Smalltalk classNamed: key) 
				printMethodChunk: method withPreamble: true
				on: stream moveSource: false toFile: 0].
		self writeSourceCodeFrom: stream to: file]
!

fileOutPackage
	"Files out the entire package into a folder structure"
	| packageInfo |
	packageInfo := PackageInfo named: self packageName.
	packageInfo systemCategories do: [:category | self fileOutCategory: category].
	packageInfo extensionMethods 
		ifNotEmpty: [self fileOutExtensions: packageInfo extensionMethods].
	
!

writeSourceCodeFrom: aStream to: aFile
	"This writes the source code and adds to files"
	| converter f fileName |
	aStream contents isAsciiString 
		ifTrue: [converter := MacRomanTextConverter new]
		ifFalse: [converter := UTF8TextConverter new].
	fileName := aFile, (FileDirectory dot, FileStream st).
	f := MultiByteFileStream new open: fileName forWrite: true.
	f ifNil: [^ self error: 'Cannot open file'].
	f lineEndConvention: #lf.
	(converter isMemberOf: UTF8TextConverter)
		ifTrue: [f binary.
			UTF8TextConverter writeBOMOn: f].
	f text.
	f converter: converter.
	f nextPutAll: aStream contents.
	f close.
	files add: fileName
! !

!GCMapper methodsFor:'initialize-release'!

initialize

	super initialize.
	files := Set new.
	systemOrganizer := Smalltalk organization
! !

!GCMapper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCMapper.st,v 1.1 2011/08/31 15:57:15 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCMapper.st,v 1.1 2011/08/31 15:57:15 cg Exp $'
! !
