"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Query subclass:#AbstractReadQuery
	instanceVariableNames:'resultClass whereClause base limit offset proxyType
		readsOneObject shouldRefresh tracing absentBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!AbstractReadQuery class methodsFor:'instance creation'!

newQuery
	^SimpleQuery new.
!

read: aClass limit: anInteger
	^self returningManyOf: aClass limit: anInteger.
!

read: aClass where: anExpression limit: anInteger
	^self
		returningManyOf: aClass
		where: anExpression
		limit: anInteger.
!

readManyOf: aClass limit: anInteger
	^self returningManyOf: aClass limit: anInteger.
!

readManyOf: aClass where: anExpression limit: anInteger
	^self
		returningManyOf: aClass
		where: anExpression
		limit: anInteger.
!

returningManyOf: aClass
	^self returningManyOf: aClass where: nil.
!

returningManyOf: aClass limit: anInteger
	^self
		returningManyOf: aClass
		where: nil
		limit: anInteger.
!

returningManyOf: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression limit: nil.
!

returningManyOf: aClass where: anExpression limit: anInteger
	^(self newQuery)
		limit: anInteger;
		initResultClass: aClass
			whereClause: anExpression
			singleObject: false;
		yourself.
!

returningOneOf: aClass
	^self returningOneOf: aClass where: nil.
!

returningOneOf: aClass where: anExpression
	^self newQuery
		initResultClass: aClass
		whereClause: anExpression
		singleObject: true.
! !

!AbstractReadQuery class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!AbstractReadQuery methodsFor:'accessing'!

absentBlock

	absentBlock == nil ifTrue: [^[nil]].
	^absentBlock.
!

baseExpression
	^base.
!

baseExpression: aBaseExpression
	base := aBaseExpression.
!

defaultTracing
	| defaultTracing |
	defaultTracing := Tracing new.
	defaultTracing base: base.
	^defaultTracing.
!

descriptor
	^session descriptorFor: resultClass.
!

limit
	^limit.
!

limit: anInteger
	limit := anInteger.
!

offset
	^offset
!

offset: anObject
	offset := anObject.
!

proxyType
	"Answer the type of proxies the receiver should build when instantiating objects:
		nil - Do not build proxies.
		TimedProxy - Build TimedProxies.
		Proxy - Build Proxies."

	^proxyType
!

proxyType: aClass
	
	proxyType := aClass.
!

readsOneObject

	^readsOneObject.
!

readsOneObject: aBoolean 
	readsOneObject := aBoolean
!

resultClass
	^resultClass
!

returnProxies
	^proxyType notNil
!

returnProxies: aBoolean
	
	aBoolean ifTrue: [proxyType := Proxy] ifFalse: [proxyType := nil].
!

returnTimedProxies: aBoolean

	aBoolean ifTrue: [proxyType := TimedProxy] ifFalse: [proxyType := nil]
!

setUpBase
	whereClause isGlorpExpression ifTrue: [base := whereClause queryLevelBaseExpression] ifFalse: [base := BaseExpression new].
!

shouldRefresh
	^shouldRefresh
!

shouldRefresh: aBoolean
	shouldRefresh := aBoolean
!

tracing
	tracing isNil ifTrue: [tracing := self defaultTracing].
	^tracing.
!

tracing: aTracing
	tracing := aTracing.
	tracing updateBase: base.
	tracing setup.
!

whereClause
	^whereClause
!

whereClause: anExpression
	whereClause := anExpression.
	self setUpBase.
! !

!AbstractReadQuery methodsFor:'boolean operations'!

AND: anExpression
	"Allow you to send AND: or OR: directly to a query to build up a query dynamically without needing to mess with the criteria explicitly"

	whereClause := (anExpression
		asGlorpExpressionOn: base) AND: ((whereClause asGlorpExpressionOn: base) asGeneralGlorpExpression).
!

OR: anExpression
	"Allow you to send AND: or OR: directly to a query to build up a query dynamically without needing to mess with the criteria explicitly"

	whereClause := (anExpression asGlorpExpressionOn: base)
		OR: (whereClause asGlorpExpressionOn: base) asGeneralGlorpExpression.
!

except: aQuery
	^self minus: aQuery.
!

intersect: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^aQuery].
	self shortCircuitEmptyReturn ifTrue: [^self].
	^CompoundQuery newIntersect addQuery: self; addQuery: aQuery.
!

minus: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	self shortCircuitEmptyReturn ifTrue: [^aQuery].
	^CompoundQuery newMinus addQuery: self; addQuery: aQuery.
!

unionAll: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	self shortCircuitEmptyReturn ifTrue: [^aQuery].
	^CompoundQuery newUnionAll addQuery: self; addQuery: aQuery.
! !

!AbstractReadQuery methodsFor:'caching'!

checkCacheWithParameters: aDictionary ifPresent: aBlock
	| primaryKey result filteredResult |
	readsOneObject ifFalse: [^nil].
	self shouldRefresh ifTrue: [^nil].
	primaryKey := self primaryKeyFrom: aDictionary.
	"We can't figure out the primary key"
	primaryKey isNil ifTrue: [^nil].
	"The primary key is invalid (i.e. has nulls in it). So we've found the entry, and we know it's nil"
	primaryKey = #() ifTrue: [^aBlock value: nil].
	"If it's expired, make sure we do the read but still refresh"
  	(session hasExpired: resultClass key: primaryKey) ifTrue: [
		self shouldRefresh: true.
		^nil].
	result := session 
		cacheAt: primaryKey
		forClass: resultClass
		ifNone: [nil].
	filteredResult := session filterDeletionFrom: result.
	^filteredResult isNil ifTrue: [nil] ifFalse: [
		(session system shouldResetExpiryTimeForObjectOfClass: filteredResult class readFrom: self)
			ifTrue: [self session markAsCurrentOfClass: filteredResult class key: primaryKey].
		aBlock value: filteredResult].
!

primaryKeyFrom: aDictionary
	"Construct a primary key from the given parameters."

	self prepare.
	(aDictionary
		anySatisfy:
			[:each | self session system shouldForeignKeyValueBeTreatedAsNull: each])
		ifTrue: [^#()].
	^self whereClause primaryKeyFromDictionary: aDictionary.
! !

!AbstractReadQuery methodsFor:'converting'!

asFullQuery
	^self.
! !

!AbstractReadQuery methodsFor:'copying'!

postCopy
	super postCopy.
	base := BaseExpression new.
	whereClause := whereClause asGlorpExpressionOn: base.
	session isNil ifFalse: [self setUpExpressions].
! !

!AbstractReadQuery methodsFor:'executing'!

executeWithParameters: parameterArray in: aSession 
	| |
	session := aSession.
	self requiresFullQuery 
		ifTrue: [^self asFullQuery executeWithParameters: parameterArray in: aSession].
	self checkValidity.
	self setUpExpressions.
	self checkCacheWithParameters: parameterArray ifPresent: [:hit | session register: hit. ^hit].
	self setupTracing.
	^self 
		readFromDatabaseWithParameters: parameterArray.
!

readFromDatabaseWithParameters: anArray
	| answerStream result |
	answerStream := self rowsFromDatabaseWithParameters: anArray.
	^readsOneObject
		ifTrue: [result := answerStream next. answerStream release. result]
		ifFalse: [self resultCollectionFor: answerStream].
!

rowsFromDatabaseWithParameters: anArray
	self subclassResponsibility.
!

setUpExpressions
	whereClause isGlorpExpression
		ifTrue:
			[base := whereClause queryLevelBaseExpression.
			base hasDescriptor
				ifFalse: [base descriptor: (session descriptorFor: resultClass)]]
		ifFalse:
			[base isNil ifTrue: [base := BaseExpression new].
			base descriptor: (session descriptorFor: resultClass).
			whereClause := whereClause asGlorpExpressionOn: base in: self].
	ordering isNil
		ifFalse:
			[ordering := ordering
				collect:
					[:each | (self expressionBlockFor: each) asGlorpExpressionOn: base in: self]].
	tracing isNil ifFalse: [tracing updateBase: base].
!

validateWhereClause
	resultClass glorpCanHaveDescriptorDirectly ifFalse: [self error: 'Invalid result class for query'].
	whereClause isJoin ifFalse: [
		whereClause do: [:each | each validate]].
! !

!AbstractReadQuery methodsFor:'initialize'!

initResultClass: aClass whereClause: theCriteria singleObject: aBoolean 
	resultClass := aClass.
	self whereClause: ((theCriteria isNil or: [theCriteria = true or: [theCriteria = false]]) ifTrue: [EmptyExpression on: theCriteria] ifFalse: [theCriteria]).
	readsOneObject := aBoolean.
	aBoolean ifTrue: [self limit: 1].
!

initialize
	
	super initialize.
	proxyType := nil.
	shouldRefresh := false.
! !

!AbstractReadQuery methodsFor:'preparing'!

shouldExpressionBeJoin: anExpression
	"Return true if the expression should be a join in our context (which means it refers only to tables we have in our from list) or if it should be ANDed into the where clause). Only relevant if we're using ANSI joins"
	^anExpression
		allSatisfy:
			[:eachNode | 
			| queryBase |
			queryBase := eachNode queryLevelBaseExpression.
			queryBase = self baseExpression].
! !

!AbstractReadQuery methodsFor:'private'!

privateAnd: anExpression
	"AND these two together without checking for common bases, rebuilding, etc. Everything had better be right"
	whereClause := whereClause AND: anExpression.
! !

!AbstractReadQuery methodsFor:'specifying retrievals'!

alsoFetch: anExpression
	self tracing alsoFetch: (self expressionBlockFor: anExpression).
!

retrieve: anExpression
	self tracing retrieve: (self expressionBlockFor: anExpression).
!

retrieveAvg: anExpression
        self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #avg).
!

retrieveMax: anExpression
	self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #max).
!

retrieveMin: anExpression
	self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #min).
! !

!AbstractReadQuery methodsFor:'testing'!

hasLimit
	^self limit notNil.
!

hasOffset
	^self offset notNil and: [self offset > 0].
!

hasTracing
	"Return true if we've given this query a tracing already"
	^false.
!

limitMakesSense
	^self hasLimit not or: [self limit > 0].
!

requiresFullQuery
	^false.
!

returnTimedProxies
	^proxyType == TimedProxy
!

shortCircuitEmptyReturn
	"If we have a literal false for criteria, we never need to go to the database.
	We also don't need to go to database if limit doesnt make sense in terms of knowing beforehand that no results will be returned"

	^(whereClause class == EmptyExpression and: [whereClause isFalse])
		or: [self limitMakesSense not].
! !

!AbstractReadQuery methodsFor:'tracing'!

setupTracing
	self setUpExpressions.
	self tracing setup.
	self tracing additionalExpressions do: [:each |
		each hasDescriptor ifTrue: [
			each descriptor trace: self tracing context: each]].
! !

!AbstractReadQuery methodsFor:'validation'!

checkValidity
	resultClass glorpCanHaveDescriptorDirectly ifFalse: [self error: 'resultClass must be a class'].
	(ordering notNil and: [self descriptor supportsOrdering not]) ifTrue: [self error: 'The descriptor for ', self resultClass name, ' does not support ordering in queries'].
! !

!AbstractReadQuery class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/AbstractReadQuery.st,v 1.1 2006/07/25 16:32:03 boris Exp $'
! !
