"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

ObjectExpression subclass:#BaseExpression
	instanceVariableNames:'descriptor availableSubSelectNumber'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!BaseExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!BaseExpression methodsFor:'accessing'!

additionalExpressions
	| expressions |
	expressions := OrderedCollection new.
	self descriptor typeMapping addTypeMappingCriteriaTo: expressions in: self.
	expressions addAll: self multipleTableExpressions.
	^expressions
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. We don't have a base, but we *are* a base, so return true"
	^true.
!

descriptor

	^descriptor
!

descriptor: aDescriptor
	descriptor := aDescriptor
!

hasDescriptor

	^self descriptor notNil.
!

mappedFields
	^self descriptor mappedFields collect: [:each | self translateField: each].
!

multipleTableExpressions
	^self descriptor multipleTableJoin 
		collect: [:each | each asExpressionJoiningSource: self toTarget: self]
!

system
	self descriptor isNil ifTrue: [^nil].
	^self descriptor system.
!

table

	self error: 'trying to get a single table from a base expression'
!

tables

	^self descriptor tables.
!

targetDescriptor
	self halt
! !

!BaseExpression methodsFor:'api'!

base

	^nil.
!

getParameter: aDatabaseField 
	^ParameterExpression forField: aDatabaseField basedOn: self.
! !

!BaseExpression methodsFor:'mapping'!

valueInBuilder: anElementBuilder
	"If we ever get here, this is because we're being used as the reverse of a direct to-many mapping, so we expect to have a descriptor with only one mapping, named #myValue"
	| mapping |
	mapping := self descriptor mappingForAttributeNamed: #myValue.
	^mapping valueInBuilder: anElementBuilder as: (self get: #myValue) field.
! !

!BaseExpression methodsFor:'navigating'!

queryLevelBaseExpression

	^self
!

ultimateBaseExpression

	^self
! !

!BaseExpression methodsFor:'preparing'!

additionalExpressionsOn: anExpression
	^#().
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	^source.
!

availableSubSelectNumber
	availableSubSelectNumber := availableSubSelectNumber isNil ifTrue: [1] ifFalse: [availableSubSelectNumber + 1].
	^availableSubSelectNumber.
!

controlsTables

	^true.
!

fieldsForSelectStatement
	^self descriptor mappedFields.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	anExpression isNil ifTrue: [^aBaseExpression].
	anExpression == self ifTrue: [^aBaseExpression].
	^ultimateBaseExpression isNil ifTrue: [self] ifFalse: [ultimateBaseExpression].
!

tablesToPrint
	"We derive the base's tables from the fields that are being selected, but make sure that at least the primary table is listed."
	^Array with: (self aliasedTableFor: self descriptor primaryTable).
! !

!BaseExpression methodsFor:'printing'!

className

	^'Base'.
!

printOn: aStream 
	aStream
		nextPutAll: self className;
		nextPut: $(.
	self printTreeOn: aStream.
	aStream nextPut: $)
!

printOnlySelfOn: aStream 
	self descriptor isNil ifTrue: [aStream nextPutAll: 'Empty Base'. ^self].
	aStream print: self descriptor describedClass.
	self printTableAliasesOn: aStream
!

printSQLOn: aStream withParameters: aDictionary 
	"Do nothing. We're not printable"
!

printSelectSQLOn: aStream withParameters: aDictionary 
	"Normally, we're not printable, but we might be if we're inverting a direct to-many mapping. In that case, there should only be one mapped field"
	self descriptor mappedFields do: [:each | (self translateField: each) printSQLOn: aStream withParameters: aDictionary].
!

printTreeOn: aStream 
	aStream 
		print: (self descriptor isNil ifTrue: [nil] ifFalse: [self descriptor describedClass])
! !

!BaseExpression methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	^true.
!

mapsSimpleObject
	"Try to do a fast test for this, to look for the pseudo-descriptors that direct to-many mappings use."
	^self descriptor mappings size = 1 and: [(self descriptor mappingForAttributeNamed: #myValue) notNil].
! !

!BaseExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/BaseExpression.st,v 1.1 2006/07/26 08:29:22 boris Exp $'
! !
