"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Cache
	instanceVariableNames:'items policy mainCache extraReferences'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

Cache comment:'This is the per-class cache of instances read from the database.

Instance Variables:
	items	<Dictionary from: Object to: Object>	The cached items, keyed by their primary key values
	policy	<CachePolicy>	The settings for this cache.

'
!


!Cache class methodsFor:'instance creation'!

new

	^super new initialize.
!

newFor: aClass in: aCacheManager 
	| newCache descriptor |
	descriptor := aCacheManager session isNil ifFalse: [aCacheManager system descriptorFor: aClass].
	newCache := Cache new.
	newCache mainCache: aCacheManager.
	descriptor isNil 
		ifTrue: [newCache cachePolicy: CachePolicy default]
		ifFalse: [newCache cachePolicy: descriptor cachePolicy].
	^newCache.
! !

!Cache class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Cache methodsFor:'accessing'!

cachePolicy: aCachePolicy
	policy := aCachePolicy.
	self initializeCache.
!

mainCache
	^mainCache
!

mainCache: aCacheManager
	mainCache := aCacheManager
!

numberOfElements
	^items size.
!

session
	^mainCache session.
!

willRemoveItems
	^policy willRemoveItems.
! !

!Cache methodsFor:'finalization'!

mournKeyOf: anEphemeron
	policy takeExpiryActionForKey: anEphemeron key withValue: anEphemeron value in: self.
! !

!Cache methodsFor:'initialize'!

initialize
!

initializeCache

	items := policy newItemsIn: self.
	extraReferences := policy collectionForExtraReferences.
!

release
	policy release: self.
	extraReferences := nil.
! !

!Cache methodsFor:'lookup'!

at: key ifAbsent: aBlock 
	| item value |
	item := self basicAt: key ifAbsent: [^aBlock value].
	value := policy contentsOf: item.
	(self hasItemExpired: item) 
		ifTrue: 
			[policy takeExpiryActionForKey: key withValue: value in: self. 
			(items includesKey: key) ifFalse: [^aBlock value]].
	^value.
!

at: key ifAbsentPut: aBlock 
	| item |
	item := self at: key ifAbsent: [nil].
	^item isNil 
		ifTrue: [ | newItem|
			newItem := policy cacheEntryFor: aBlock value.
			self markEntryAsCurrent: newItem.
			items at: key put: newItem]
		ifFalse: [self markEntryAsCurrent: item. item].
!

expireAll
	items do: [:each | policy expire: each].
!

expireKey: key ifAbsent: aBlock
	| item |
	item := self basicAt: key ifAbsent: [^aBlock value].
	policy expire: item.
!

hasExpired: key
	| item |
	item := self basicAt: key ifAbsent: [^false].
	^self hasItemExpired: item.
!

hasItemExpired: anItem
	^(policy hasExpired: anItem) and: [(mainCache session isRegistered: (policy contentsOf: anItem)) not].
!

includesKey: key
	"Return true if we include the object. Don't listen to any expiry policy"
	self basicAt: key ifAbsent: [^false].
	^true.
!

includesKey: key as: anObject
	"Return true if we include the object, and it matches the given object. If we include a different object with the same key, raise an exception. Don't listen to any expiry policy"
	| item value |
	item := self basicAt: key ifAbsent: [^false].
	value := policy contentsOf: item.
	value == anObject ifFalse: [self error: 'Duplicate objects with same primary key'].
	^true.
!

includesKey: key withClass: aClass
	"Return true if we include the object, and it matches our class. Don't listen to any expiry policy"
	| item value |
	item := self basicAt: key ifAbsent: [^false].
	value := policy contentsOf: item.
	^value isKindOf: aClass.
!

removeKey: key ifAbsent: aBlock

	^items removeKey: key ifAbsent: aBlock.
! !

!Cache methodsFor:'private'!

basicAt: anObject ifAbsent: aBlock

	^items at: anObject ifAbsent: aBlock.
!

do: aBlock
	items do: aBlock.
!

expiredInstanceFor: key
	"Return the expired instance. Used for refreshing so that we don't recursively try and refresh when we get the instance to be refreshed"
	| item value |
	item := self basicAt: key ifAbsent: [self error: 'No expired instance found'].
	value := policy contentsOf: item.
	(self hasItemExpired: item) ifFalse: [self error: 'No expired instance found'].
	^value.
!

markAsCurrentAtKey: key
	| item |
	item := self basicAt: key ifAbsent: [^false].
	^policy markEntryAsCurrent: item in: self.
!

markEntryAsCurrent: anItem
	"The policy has told us to mark an item as current. This is only really useful for weak policies, which tell us to keep an additional pointer to the object in a (presumably) fixed-size collection"
	extraReferences isNil ifFalse: [extraReferences add: anItem].
! !

!Cache class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Cache.st,v 1.1 2006/07/26 07:59:06 boris Exp $'
! !
