"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#CacheManager
	instanceVariableNames:'subCaches session'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

CacheManager comment:'This is the entire cache for a session, consisting of multiple sub-caches, one per class.

Instance Variables:
	session	<Session>	The containing session.
	subCaches	<Dictionary from: Class to: Cache>	The per-class caches.

'
!


!CacheManager class methodsFor:'instance creation'!

forSession: aSession 
	^self new session: aSession.
!

new

	^super new initialize.
! !

!CacheManager class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!CacheManager methodsFor:'accessing'!

numberOfElements
	^subCaches inject: 0 into: [:sum :each |
		sum + each numberOfElements].
!

session

	^session.
!

session: aSession 
	session := aSession.
!

system

	^self session system.
! !

!CacheManager methodsFor:'adding'!

at: aKey insert: anObject

	| subCache |
	subCache := self cacheForClass: anObject class.
	subCache at: aKey ifAbsentPut: [anObject].
! !

!CacheManager methodsFor:'initialize/release'!

cacheForClass: aClass
	^subCaches at: aClass
		ifAbsentPut: [self makeCacheFor: aClass]
!

initialize

	subCaches := IdentityDictionary new: 100.
!

makeCacheFor: aClass 
	| rootClass cache |
	rootClass := session isNil ifTrue: [aClass] ifFalse: [session lookupRootClassFor: aClass].
	cache := subCaches at: rootClass ifAbsentPut: [ Cache newFor: rootClass in: self ].
	subCaches at: aClass put: cache.
	^cache
!

release
	subCaches do: [:each | each release].
! !

!CacheManager methodsFor:'private/caching'!

cacheFor: anObject
	"Get the cache for a particular object. Since this could conceivably be passed a proxy, check for that. The cache for an uninstantiated proxy is kind of ambiguous, treat it as nil.  This could also be a class"

	| nonMetaClass |
	nonMetaClass := anObject isBehavior ifTrue: [anObject] ifFalse: [anObject class].
	^nonMetaClass == Proxy
		ifTrue: [anObject isInstantiated ifTrue: [self cacheFor: anObject getValue] ifFalse: [nil]]
		ifFalse: [self cacheForClass: nonMetaClass].
!

expiredInstanceOf: aClass key: aKey

	^(self cacheForClass: aClass) expiredInstanceFor: aKey.
! !

!CacheManager methodsFor:'querying'!

containsObject: anObject key: aKey

	| cache |
	cache := self cacheForClass: anObject class.
	^cache includesKey: aKey as: anObject.
!

containsObjectForClass: aClass key: aKey

	| cache |
	cache := self cacheForClass: aClass.
	^cache includesKey: aKey.
!

expireAll
	subCaches do: [:each | each expireAll].
!

expireClass: aClass key: aKey

	^self expireClass: aClass key: aKey ifAbsent: [self error: 'Object not in cache'].
!

expireClass: aClass key: aKey ifAbsent: failBlock 
	| cache |
	cache := self cacheForClass: aClass.
	(cache includesKey: aKey withClass: aClass) ifFalse: [^failBlock value].
	cache expireKey: aKey ifAbsent: [ failBlock value ].
!

hasExpired: anObject
	| key cache |
	key := (session descriptorFor: anObject) primaryKeyFor: anObject.
	cache := self cacheFor: anObject.
	cache isNil ifTrue: [^false].  "We have an uninstantiated proxy."
	^cache hasExpired: key.
!

hasExpired: aClass key: key
	| cache |
	cache := self cacheFor: aClass.
	^cache hasExpired: key.
!

hasObjectExpiredOfClass: aClass withKey: key
	| cache |
	cache := self cacheForClass: aClass.
	^cache hasExpired: key.
!

lookupClass: aClass key: aKey

	^self lookupClass: aClass key: aKey ifAbsent: [self error: 'cache miss'].
!

lookupClass: aClass key: aKey ifAbsent: failBlock

	| object |
	object := (self cacheForClass: aClass) at: aKey ifAbsent: failBlock.
	^(object isKindOf: aClass)
		ifTrue: [ object ]
		ifFalse: [ failBlock value ].
!

lookupClass: aClass key: aKey ifAbsentPut: failBlock

	^(self cacheForClass: aClass) at: aKey ifAbsentPut: failBlock.
!

markAsCurrentOfClass: aClass key: key
	| cache |
	aClass == Proxy ifTrue: [^self].
	cache := self cacheForClass: aClass.
	cache markAsCurrentAtKey: key.
!

removeClass: aClass key: aKey

	^self removeClass: aClass key: aKey ifAbsent: [self error: 'Object not in cache'].
!

removeClass: aClass key: aKey ifAbsent: failBlock 
	| cache |
	cache := self cacheForClass: aClass.
	(cache includesKey: aKey withClass: aClass) ifFalse: [^failBlock value].
	cache removeKey: aKey ifAbsent: [ failBlock value ].
! !

!CacheManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/CacheManager.st,v 1.1 2006/07/26 08:00:31 boris Exp $'
! !
