"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#CachePolicy
	instanceVariableNames:'expiryAction numberOfElements'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

CachePolicy comment:'A CachePolicy implements the different possible policies we might use for caching. The superclass implements the trivial policy of keeping all objects forever.

The policy also controls what we store in the cache. In general, it''s assumed to be a cache entry of some sort, and the policy is responsible for wrapping and unwrapping objects going to and from the cache. The default policy is that the objects themselves are the cache entry (saving one object per cached object in overhead).

Instance Variables:
	size	<Number>	The minimum cache size we want to use.
	expiryAction <Symbol> What to do when an object has expired. Currently hard-coded as one of #remove, #notify, #refresh, #notifyAndRemove.

'
!


!CachePolicy class methodsFor:'instance creation'!

default
	"Use of the WeakVWCache is tied in with Ephemera which we do not expect to get working in VW3."

	Dialect isVWWithNameSpaces ifTrue: [^WeakVWCachePolicy new].
	^self new
!

new

	^super new initialize.
! !

!CachePolicy class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!CachePolicy methodsFor:'accessing'!

dictionaryClass
	^Dictionary.
!

expiryAction
	^expiryAction
!

expiryAction: aSymbol
	"See class comment for possible values"
	expiryAction := aSymbol
!

numberOfElements
	^numberOfElements
!

numberOfElements: anInteger
	numberOfElements := anInteger
! !

!CachePolicy methodsFor:'expiry'!

expire: anItem
	"Force anItem to be expired. This may be ignored if the policy doesn't respect this, and is ignored by default"
!

notifyOfExpiry: anObject in: aCache 
	anObject glorpNoticeOfExpiryIn: aCache session.
!

release: aCache
	(expiryAction == #notify or: [expiryAction == #notifyAndRemove])
		ifTrue: [
			aCache do: [:each |
				(self contentsOf: each)  glorpNoticeOfExpiryIn: aCache session]].
!

takeExpiryActionForKey: key withValue: anObject in: aCache
	expiryAction == #refresh
		ifTrue: [aCache session refresh: anObject].
	(#(#notify #notifyAndRemove) includes: expiryAction) ifTrue: [
		self notifyOfExpiry: anObject in: aCache].
	(#(#remove #notifyAndRemove) includes: expiryAction) ifTrue: [
		aCache removeKey: key ifAbsent: []].
!

willRemoveItems
	^(expiryAction == #notifyAndRemove or: [expiryAction == #remove])
! !

!CachePolicy methodsFor:'initialize'!

collectionForExtraReferences
	^nil.
!

initialize

	numberOfElements := 100.
	expiryAction := #remove.
!

newItemsIn: aCache
	^self dictionaryClass new: 20.
! !

!CachePolicy methodsFor:'wrap/unwrap'!

cacheEntryFor: anObject

	^anObject.
!

contentsOf: aCacheEntry

	^aCacheEntry.
!

hasExpired: aCacheEntry

	^false.
!

markEntryAsCurrent: aCacheEntry in: aCache

	^self.
! !

!CachePolicy class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/CachePolicy.st,v 1.1 2006/07/26 08:00:21 boris Exp $'
! !
