"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Mapping subclass:#ConditionalMapping
	instanceVariableNames:'conditionalField conditionalMethod cases otherwiseCase
		conditionalFieldMapping'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!ConditionalMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ConditionalMapping methodsFor:'accessing'!

attributeName
	^self representativeMapping attributeName.
!

cases
	^cases.
!

conditionalField: aField

	(conditionalField notNil and: [conditionalField ~~ aField]) ifTrue: [self error: 'You can only test one field in a conditional mapping'].
	conditionalField := aField.
!

conditionalFieldMapping
	^conditionalFieldMapping
!

conditionalFieldMapping: aMapping
	"This is a write-only mapping for the conditional field value, which writes out the result of performing the conditional method"
	conditionalFieldMapping := aMapping.
	descriptor isNil ifFalse: [aMapping descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].
!

conditionalMethod: aSymbol
	(conditionalMethod notNil and: [conditionalMethod ~~ aSymbol]) ifTrue: [self error: 'You can only test one attribute in a conditional mapping'].
	conditionalMethod := aSymbol.
!

descriptor: aDescriptor

	super descriptor: aDescriptor.
	cases do: [:each | each value descriptor: aDescriptor].
	otherwiseCase isNil ifFalse: [otherwiseCase descriptor: aDescriptor].
	conditionalFieldMapping isNil ifFalse: [conditionalFieldMapping descriptor: aDescriptor].
!

mappedFields

	| all |
	all := Set new.
	conditionalFieldMapping isNil 
		ifTrue: [all add: conditionalField]
		ifFalse: [all addAll: conditionalFieldMapping mappedFields].
	cases do: [:each |
		all addAll: each value mappedFields].
	^all.
!

otherwise
	^otherwiseCase.
!

referenceClass
	^self representativeMapping referenceClass.
!

representativeMapping
	"At some points we may need to have this mapping behave some way without knowing yet which of its cases is satisfied. We assume that there is some basic level of similarity so that we can use one mapping as a representative to discover, e.g. what the reference class is. This works all right for cases where we have a mapping or a constant. It wouldn't work very well if we have a one to one mapping to one class in case (a) but a many-many to an entirely different class in the other"
	^cases first value.
! !

!ConditionalMapping methodsFor:'conditions'!

forField: aField attribute: attributeBlock if: testBlock useMapping: aMapping
	self conditionalField: aField.
	self conditionalMethod: attributeBlock.
	self if: testBlock then: aMapping.
!

if: conditionSelector then: aMapping

	cases add: (Association key: conditionSelector value: aMapping).
	descriptor isNil ifFalse: [aMapping descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].
!

otherwise: aMapping

	otherwiseCase := aMapping.
	descriptor isNil ifFalse: [otherwiseCase descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].
!

trace: aTracing context: anExpression
	"To make a join, we need to look at all of our possible cases"

	conditionalFieldMapping isNil ifFalse: [
		conditionalFieldMapping trace: aTracing context: anExpression].
	cases do: [:each |
		each value trace: aTracing context: anExpression].
! !

!ConditionalMapping methodsFor:'initialize/release'!

initialize
	
	super initialize.
	cases := OrderedCollection new.
! !

!ConditionalMapping methodsFor:'mapping'!

applicableMappingForConditionalValue: rowValue
	cases
		do:
			[:each | 
			((Dialect isBlock: each key)
				ifTrue: [each key value: rowValue]
				ifFalse: [self descriptor system perform: each key with: rowValue])
				ifTrue: [^each value]].
	^otherwiseCase.
!

applicableMappingForObject: anObject
	| conditionalValue |
	conditionalValue := self conditionalValueFor: anObject.
	^self applicableMappingForConditionalValue: conditionalValue.
!

applicableMappingForRow: anArray in: anElementBuilder
	"If a conditional field has been specified, then pass in the extracted value. Otherwise pass in the whole row"
	| rowValue |
	rowValue := conditionalField isNil
		ifFalse: [anElementBuilder valueOfField: conditionalField in: anArray]
		ifTrue: [anArray].
	^self applicableMappingForConditionalValue: rowValue.
!

conditionalValueFor: anObject
	^(Dialect isBlock: conditionalMethod)
		ifTrue: [conditionalMethod value: anObject]
		ifFalse: [anObject perform: conditionalMethod].
!

mapFromObject: anObject intoRowsIn: aRowMap 
	canWrite ifFalse: [^self].
	(self applicableMappingForObject: anObject) 
		mapFromObject: anObject
		intoRowsIn: aRowMap.
	conditionalFieldMapping isNil ifTrue: [^self].
	conditionalFieldMapping 
		mapFromObject: (self conditionalValueFor: anObject)
		intoRowsIn: aRowMap
!

mapObject: anObject inElementBuilder: anElementBuilder
	(self applicableMappingForRow: anElementBuilder row in: anElementBuilder)
		mapObject: anObject
		inElementBuilder: anElementBuilder.
!

newMapping: aMappingClass
	"Implement this so that we can create our sub-mappings without having them added to the parent descriptor as independent mappings"
	| mapping |
	mapping := aMappingClass new.
	mapping descriptor: self descriptor.
	descriptor resetMappedFields.
	^mapping.
!

referencedIndependentObjectsFrom: anObject

	| allReferencedObjects |
	allReferencedObjects := OrderedCollection new.
	cases do: [:each | 
		allReferencedObjects addAll: (each value referencedIndependentObjectsFrom: anObject)].
	^allReferencedObjects.
! !

!ConditionalMapping methodsFor:'preparing'!

joinExpressionFor: anExpression

	^self representativeMapping joinExpressionFor: anExpression.
! !

!ConditionalMapping methodsFor:'printing'!

printMappingInformationOn: aTextStream

  aTextStream nextPutAll: conditionalField printString;
              nextPutAll: ' -> ';
              nextPutAll: cases size printString;
              nextPutAll: ' cases '.
! !

!ConditionalMapping methodsFor:'testing'!

controlsTables
	^self cases first value controlsTables.
!

isRelationship

	^false
! !

!ConditionalMapping methodsFor:'validation'!

validateAttribute
	cases do: [:each | each value validateAttribute].
	otherwiseCase validateAttribute.
	conditionalFieldMapping isNil ifFalse: [conditionalFieldMapping validateAttribute].
! !

!ConditionalMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ConditionalMapping.st,v 1.1 2006/07/26 07:59:51 boris Exp $'
! !
