"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#ConstantExpression
	instanceVariableNames:'value alias'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!ConstantExpression class methodsFor:'instance creation'!

for: anObject

	^self new value: anObject.
! !

!ConstantExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ConstantExpression methodsFor:'As yet unclassified'!

mappedFields
	^Array with: self.
!

table
	^nil.
!

valueInBuilder: aBuilder as: anExpression
	^value.
! !

!ConstantExpression methodsFor:'accessing'!

alias
	^alias
!

alias: anObject
	alias := anObject
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^false.
!

ultimateBaseExpression
	^nil.
!

value
	^value
!

value: anObject
	value := anObject
!

valueIn: aDictionary
	^value
! !

!ConstantExpression methodsFor:'api'!

get: aSymbol withArguments: anArray
	| functionExpression |
	functionExpression := self getFunction: aSymbol withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	^RelationExpression
		named: aSymbol
		basedOn: self
		withArguments: anArray
! !

!ConstantExpression methodsFor:'converting'!

asGlorpExpressionForDescriptor: aDescriptor
	"This should only get sent to us if we're the top level of an expression. And since we don't have children, we're also the bottom level. This is only useful for the case where the expression is [:each | true] or [:each | false]."

	^(EmptyExpression on: value)
		rebuildOn: (BaseExpression new descriptor: aDescriptor).
!

asGlorpExpressionOn: aBaseExpression
	^self.
! !

!ConstantExpression methodsFor:'mapping'!

valueInBuilder: aBuilder
	^value.
! !

!ConstantExpression methodsFor:'preparing'!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	^self.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	^self.
! !

!ConstantExpression methodsFor:'printing'!

printOnlySelfOn: aStream

	aStream print: value
!

printSQLOn: aStream withParameters: aDictionary
	self value glorpPrintSQLOn: aStream.
!

printSelectSQLOn: aStream withParameters: aDictionary
	self printSQLOn: aStream withParameters: aDictionary.
	alias notNil ifTrue: [aStream nextPutAll: ' AS '; nextPutAll: alias].
!

printTreeOn: aStream 
    (value isKindOf: String)
        ifTrue: [value storeOn: aStream]
        ifFalse: [aStream print: value]
!

printUnqualifiedSQLOn: aStream withParameters: aDictionary
	self printSQLOn: aStream withParameters: aDictionary.
! !

!ConstantExpression methodsFor:'testing'!

canBind
	"Return true if this represents a value that can be bound into a prepared statement"
	^true.
!

isConstantExpression
	^true.
! !

!ConstantExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ConstantExpression.st,v 1.1 2006/07/26 08:29:37 boris Exp $'
! !
