"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Mapping subclass:#ConstantMapping
	instanceVariableNames:'constantValue valueIsSession'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

ConstantMapping comment:'Sometimes you just want a constant value to be set, either in the row, the object or both. And sometimes you just want a non-mapping (e.g. with a ConditionalMapping where one
of the conditions means "this isn''t mapped"). This mapping represents these situations.
It also handles the special case where it''s useful to have access to the session inside a
domain object, by allowing you to map it to an instance variable.

So far only the case of mapping to an inst var is implemented.

Instance Variables:
'
!


!ConstantMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ConstantMapping methodsFor:'accessing'!

constantValue
	^constantValue.
!

constantValue: anObject

	constantValue := anObject.
!

constantValueIn: aSession

	^valueIsSession
		ifTrue: [aSession]
		ifFalse: [constantValue].
!

constantValueIsSession

	valueIsSession := true.
!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^#().
! !

!ConstantMapping methodsFor:'api'!

getValueFrom: anObject

	^constantValue.
! !

!ConstantMapping methodsFor:'initialize/release'!

initialize

	super initialize.
	valueIsSession := false.
! !

!ConstantMapping methodsFor:'mapping'!

mapFromObject: anObject intoRowsIn: aRowMap
!

mapObject: anObject inElementBuilder: anElementBuilder
	| value |
	value := anElementBuilder isNil
		ifTrue: [constantValue]
		ifFalse: [self constantValueIn: anElementBuilder session].
	self setValueIn: anObject to: value.
!

referencedIndependentObjectsFrom: anObject

	^#().
!

trace: aTracing context: anExpression

	^self.
! !

!ConstantMapping methodsFor:'printing'!

printMappingInformationOn: aTextStream

  aTextStream nextPutAll: attribute name;
              nextPutAll: ' -> ';
              nextPutAll: constantValue printString.
! !

!ConstantMapping methodsFor:'testing'!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false
!

isRelationship
	^false
! !

!ConstantMapping methodsFor:'validation'!

validateAttribute
	^self.
! !

!ConstantMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ConstantMapping.st,v 1.1 2006/07/26 07:59:37 boris Exp $'
! !
