"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DatabaseField
	instanceVariableNames:'table name isPrimaryKey position type isNullable isUnique
		isLockKey defaultValue'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!DatabaseField class methodsFor:'instance creation'!

named: aString

	^self error: 'type needed'
!

named: aString type: dbType

	^super new initialize
		name: aString;
		type: dbType
!

new

	^self error: 'dbType needed'
! !

!DatabaseField class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DatabaseField methodsFor:'accessing'!

defaultValue
	^defaultValue
!

impliedSmalltalkType
	"Return the default Smalltalk type corresponding to our database type"

	^self type impliedSmalltalkType.
!

name
	"Private - Answer the value of the receiver's ''name'' instance variable."

	^name
!

name: anObject
	"Private - Set the value of the receiver's ''name'' instance variable to the argument, anObject."

	name := anObject
!

platform
	^self type platform.
!

position
	^position
!

position: anObject
	position := anObject
!

table
	^table
!

table: anObject

	table := anObject
!

tablesToPrint
	^Array with: table
!

type

	^type
! !

!DatabaseField methodsFor:'comparing'!

< aField
        ^self name < aField name

  " added for SmalltalkX to support the conversion of collection into
    a SortedCollection. (The default sort block in SmalltalkX is
    [:x :y | x < y].)  "
!

<= aField
	^self name <= aField name
! !

!DatabaseField methodsFor:'configuring'!

beIndexed
	self table addIndex: (DatabaseIndex forField: self).
!

beLockKey

	isLockKey := true.
	self table isNil ifFalse: [self table addAsLockKeyField: self]
!

beNullable: aBoolean

	self isPrimaryKey ifFalse: [ isNullable := aBoolean ]
!

bePrimaryKey
	isPrimaryKey := true.
	isNullable := false.
	self table isNil ifFalse: [self table addAsPrimaryKeyField: self]
!

defaultValue: anObject
	defaultValue := anObject
!

isUnique: aBoolean
	isUnique := aBoolean.
!

type: aDatabaseType

	type := aDatabaseType.
! !

!DatabaseField methodsFor:'converting'!

asDistinctFieldOn: anExpression

	^(anExpression getField: self) asDistinctFieldOn: anExpression.
!

asField
	^self.
!

asGlorpExpression

	^ParameterExpression forField: self basedOn: nil.
!

asGlorpExpressionOn: anExpression

	^ParameterExpression forField: self basedOn: anExpression.
!

converterForStType: aClass

	^self type converterForStType: (aClass isBehavior ifTrue: [aClass] ifFalse: [aClass class]).
! !

!DatabaseField methodsFor:'copying'!

isSameAliasedFieldAs: aField
	"Return true if we are the same as another aliased field. For non-aliased fields, identity is assured, so this is redundant. Note that we == the string. This will work because we know these must both be shallow copies of the same thing"
	self class == aField class ifFalse: [^false].
	^self table == aField table and: [self name == aField name].
!

withTable: aTable
	"Return a copy of ourselves, with the table set to aTable. Presumably aTable is an aliased version of our table"
	^self copy table: aTable.
! !

!DatabaseField methodsFor:'database'!

typeString

	^type typeString
! !

!DatabaseField methodsFor:'initializing'!

initialize
	isPrimaryKey := false.
	isNullable := true.
	isUnique := false.
	isLockKey := false.
!

postInitializeIn: aDescriptorSystem
	"Any initialization that has to be delayed until we're in the table"
	type initializeForField: self in: aDescriptorSystem.
! !

!DatabaseField methodsFor:'printing'!

asConstraintReferenceString

	^table name, ' (', self name, ')'.
!

printForConstraintNameOn: aStream maxLength: maxLength 
	| constraintName |
	constraintName := table name , '_' , name.
	constraintName size > maxLength 
		ifTrue: [constraintName := constraintName copyFrom: 1 to: maxLength].
	aStream nextPutAll: constraintName
!

printNameOn: aStream withParameters: anArray 
	aStream nextPutAll: self name
!

printOn: aStream
	
	aStream nextPutAll: 'Field'.
	aStream 
		nextPutAll: '(';
		nextPutAll: (table isNil ifTrue: [''] ifFalse: [table name]);
		nextPutAll: '.';
		nextPutAll: name;
		nextPutAll: ')'.
!

printQualifiedSQLOn: aStream withParameters: aDictionary 
	aStream nextPutAll: self qualifiedName
!

printSQLOn: aStream withParameters: anArray 
	aStream nextPutAll: self quotedQualifiedName.
!

printSelectSQLOn: aStream withParameters: anArray 
	self printSQLOn: aStream withParameters: anArray.
!

printUnqualifiedSQLOn: aStream withParameters: anArray 
	aStream nextPutAll: self name.
! !

!DatabaseField methodsFor:'querying'!

qualifiedName

	^table isNil 
		ifTrue: [self name]
		ifFalse: [self table qualifiedName, '.', self name].
!

quotedQualifiedName

	| basicName |
	basicName := self platform nameForColumn: self name.
	^table isNil 
		ifTrue: [basicName]
		ifFalse: [self table qualifiedName, '.', basicName].
! !

!DatabaseField methodsFor:'testing'!

isDistinct
	^false.
!

isGenerated

	^type isGenerated or: [defaultValue notNil].
!

isIndexed
	^self table hasIndexForField: self.
!

isLockKey

	^isLockKey
!

isMappable
	"Return true if we're something that can be returned in a select statement. Always true for fields"
	^true.
!

isNullable
	
	^isNullable.
!

isPrimaryKey
	"Private - Answer the value of the receiver's ''isPrimaryKey'' instance variable."

	^isPrimaryKey
!

isUnique
	^isUnique.
! !

!DatabaseField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DatabaseField.st,v 1.1 2006/07/26 07:59:58 boris Exp $'
! !
