"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DatabasePlatform
	instanceVariableNames:'types converters useBinding reservedWords functions
		characterEncoding'
	classVariableNames:'UseBindingIfSupported'
	poolDictionaries:''
	category:'Glorp'
!

DatabasePlatform class instanceVariableNames:'converterRepository'

"
 No other class instance variables are inherited by this class.
"
!


!DatabasePlatform class methodsFor:'*eoglorp'!

createLoginFromConnectionDictionary: aDict 
	self subclassResponsibility
!

loginWithConnectionDictionary: aDict 
	| platformClass |
	platformClass := self glorpAllSubclasses
				detect: [:cls | cls understandsConnectionDictionary: aDict].
	^ platformClass isNil
		ifFalse: [platformClass createLoginFromConnectionDictionary: aDict] ifTrue: [nil]
!

understandsConnectionDictionary: aDict

	^false
! !

!DatabasePlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DatabasePlatform class methodsFor:'accessing'!

useBindingIfSupported
	UseBindingIfSupported isNil ifTrue: [UseBindingIfSupported := false].
	^UseBindingIfSupported
!

useBindingIfSupported: aBoolean 
	UseBindingIfSupported := aBoolean
! !

!DatabasePlatform methodsFor:'accessing'!

characterEncoding
	"We hope that this will be set appropriately for the connection, but make sure the default is fairly harmless, and will in most cases complain rather than write characters incorrectly"
	characterEncoding isNil ifTrue: [characterEncoding := #ascii].
	^characterEncoding
!

characterEncoding: anEncodingName
	characterEncoding := anEncodingName.
!

vwConnectionClass
	^self isODBCPlatform 
			ifTrue: [self vwODBCConnectionClass]
			ifFalse: [self vwEXDIConnectionClass]
!

vwEXDIConnectionClass
	^self subclassResponsibility
!

vwODBCConnectionClass
	^'ODBCThreadedConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'ODBCThapiEXDI'
! !

!DatabasePlatform methodsFor:'binding'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	self subclassResponsibility.
!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once"
	^aCollectionOfDatabaseRows size.
! !

!DatabasePlatform methodsFor:'constants'!

areSequencesExplicitlyCreated

	self subclassResponsibility.
!

batchWriteStatementTerminatorString
	"^<String> This statement return the string to be used to devide several statement during batch write ..."

	^';'
!

capitalWritingOfColumnName
	"^<Boolean> This method returns true, if the dbms wants to have column 
	names written in capital letters"

	^false
!

capitalWritingOfCreatorName
	"^<Boolean> This method returns true, if the dbms wants to have column 
	names written in capital letters"

	^true
!

capitalWritingOfDatabaseName
	"^<Boolean>"

	^true
!

capitalWritingOfSQLCommands
	"^<Boolean>"

	^true
!

capitalWritingOfTableName
	"^<Boolean>"

	^true
!

charactersThatNeedEscaping
	^#($' $" $\)
!

columnNameSeparatorString
	"^<String> This statement return the string to be used to devide several columns ..."

	^','
!

deleteViewWithTableSyntax

	^false
!

hasSubtransaction
	"^<Boolean> This method returns true, if the used dbms is able to execute multiple sql-statements
	transferred via a command line transmitted from client to server - otherwise I return false"

	^true
!

initializeReservedWords
	reservedWords := Dictionary new.
!

maxLengthOfDatabaseName
	"^<Integer>I return the max. length of a database name"

	^8
!

maxSQLBufferLength
	"^<Integer> I return the maximum length of a sql command stream"

	^8192
!

maximumLengthOfColumnName
	"^<Integer> I return the max. length of a column name"

	^18
!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^18
!

parenthesizeCommandsInCompoundQuery
	^true
!

postfixTableNameBeforeDeleting

	^false
!

prefixQualifierBeforeCreatingAndDeleting

	^true
!

prefixQualifierSeparatorString
	"^<String> This statement return the string to be used to separate the qualifier and the table/column name"

	^'.'
!

prefixTableNameBeforeDeleting

	^false
!

primaryKeysAreAutomaticallyUnique
	"Return false if, in addition to specifying something as a primary key, we must separately specify it as unique"

	^false.
!

requiresEscapeFor: aCharacter
	^self charactersThatNeedEscaping includes: aCharacter.
!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^true.
!

reservedWords
	reservedWords isNil ifTrue: [self initializeReservedWords].
	^reservedWords.
!

sqlTextForBeginTransaction
	"comment"

	^'BEGIN'
!

sqlTextForNOTNULLAttributeConstraint
	"^<String>"

	^'NOT NULL'
!

sqlTextForNOTNULLWithDefaultAttributeConstraint
	"^<String>"

	^'NOT NULL WITH DEFAULT'
!

sqlTextForNOTUNIQUEAttributeConstraint
	^''.
!

sqlTextForNULLAttributeConstraint
	"^<String>"

	^'NULL'
!

sqlTextForUNIQUEAttributeConstraint
	^'UNIQUE'.
!

sqlWildcardForMultipleCharacters
	"^<String> This method returns the used wildcard string for multiple characters"
	
	^'%'
!

sqlWildcardForSingleCharacter
	"^<String> This method returns the used wildcard string for single characters"
	
	^'_'
!

supportsAuthentication
	^true
!

supportsConstraints
	"Return true if we support integrity constraints"
	^true.
!

supportsMillisecondsInTimes
	self subclassResponsibility.
!

supportsVariableSizedNumerics
	"Return true if this platform can support numbers with a varying size and number of decimal places. Access, notably, doesn't seem to be able to"
	^true.
!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^false.
!

usesNullForFalse
	"Return true if this database is likely to use nil as an empty string value"
	^false.
! !

!DatabasePlatform methodsFor:'conversion-boolean'!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToDBBoolean:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.
!

booleanToIntegerConverter
	^DelegatingDatabaseConverter
		named: #booleanToInteger
		hostedBy: self
		fromStToDb: #convertBooleanToInteger:for:
		fromDbToSt: #convertIntegerToBoolean:for:.
!

booleanToStringTFConverter
	^DelegatingDatabaseConverter
		named: #booleanToString
		hostedBy: self
		fromStToDb: #convertBooleanToTF:for:
		fromDbToSt: #convertTFToBoolean:for:.
!

convertBooleanToDBBoolean: aBoolean for: aType
	(self usesNullForFalse and: [aBoolean isNil]) ifTrue: [^false].
	^aBoolean.
!

convertBooleanToInteger: aBoolean for: aType
	aBoolean isNil ifTrue: [^nil].
	^aBoolean ifTrue: [1] ifFalse: [0].
!

convertBooleanToTF: aBoolean for: aType
	aBoolean isNil ifTrue: [^aBoolean].
	^aBoolean ifTrue: ['T'] ifFalse: ['F'].
!

convertDBBooleanToBoolean: aBoolean for: aType
	^aBoolean.
!

convertIntegerToBoolean: anInteger for: aType
	anInteger isNil ifTrue: [^anInteger].
	anInteger = 1 ifTrue: [^true].
	anInteger = 0 ifTrue: [^false].
	self error: 'invalid boolean conversion'.
!

convertTFToBoolean: aString for: aType
	aString isNil ifTrue: [^aString].
	aString = 'T' ifTrue: [^true].
	aString = 'F' ifTrue: [^false].
	self error: 'invalid boolean conversion'.
! !

!DatabasePlatform methodsFor:'conversion-null'!

nullConversion: anObject for: aType
	^anObject.
!

nullConverter
	^NullConverter new
		name: #null.
! !

!DatabasePlatform methodsFor:'conversion-numbers'!

convertToDouble: aNumber for: type 

	aNumber isNil ifTrue: [^nil].
	^Dialect coerceToDoublePrecisionFloat: aNumber
!

convertToFloat: aNumber for: aType 
	aNumber isNil ifTrue: [^nil].
	^aNumber asFloat.
!

convertToInteger: aNumber for: aType 
	^aNumber isNil ifTrue: [aNumber] ifFalse: [aNumber asNumber asInteger].
!

convertToNumber: aNumber for: aType
	"Convert to an arbitrary numeric type. Make sure our precision matches. *Hate* floating point"

	| exponent result |
	aNumber isNil ifTrue: [^nil].
	aType precision isNil ifTrue: [^aNumber].
	(Dialect doesPrecisionOf: aNumber equal: aType precision) ifTrue: [^aNumber].
	exponent := Dialect
		coerceToDoublePrecisionFloat: (10 raisedToInteger: aType precision).
	result := (aNumber * exponent) rounded / exponent.
	^aNumber isInteger ifTrue: [result rounded] ifFalse: [result].
!

numberToDoubleConverter
	^DelegatingDatabaseConverter
		named: #numberToDouble
		hostedBy: self
		fromStToDb: #convertToDouble:for:
		fromDbToSt: #convertToDouble:for:.
!

numberToFloatConverter
	^DelegatingDatabaseConverter
		named: #numberToFloat
		hostedBy: self
		fromStToDb: #convertToFloat:for:
		fromDbToSt: #convertToFloat:for:.
!

numberToIntegerConverter
	^DelegatingDatabaseConverter
		named: #numberToInteger
		hostedBy: self
		fromStToDb: #convertToInteger:for:
		fromDbToSt: #convertToInteger:for:.
!

numberToNumberConverter
	^DelegatingDatabaseConverter
		named: #numberToNumber
		hostedBy: self
		fromStToDb: #convertToNumber:for:
		fromDbToSt: #convertToNumber:for:.
! !

!DatabasePlatform methodsFor:'conversion-strings'!

byteArrayToStringConverter
	^DelegatingDatabaseConverter
		named: #byteArrayToString
		hostedBy: self
		fromStToDb: #convertStringToByteArray:for:
		fromDbToSt: #convertByteArrayToString:for:.
!

convertByteArrayToString: aByteArray for: aType
	^aByteArray asString.
!

convertStringToByteArray: aString for: aType
	^aString asByteArray.
!

escapeFor: aCharacter
	^String with: $\ with: aCharacter.
!

padString: aString for: aType 
	| padding trimmed |
	aString isNil ifTrue: [^nil].
	aString isString ifFalse: [self error: 'Invalid data type'].
	(self usesNullForEmptyStrings and: [aString isEmpty]) ifTrue: [^nil].
	trimmed := self trimString: aString for: aType.
				
	(aType isVariableWidth or: [trimmed size = aType width]) ifTrue: [^trimmed].
	padding := String new: aType width - aString size.
	padding atAllPut: self paddingCharacter.
	^aString , padding
!

paddingCharacter
	^1 asCharacter.
!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: 'X'''.
	aByteArray do: [:each |
		each printOn: aStream paddedWith: $0 to: 2 base: 16].
	aStream nextPut: $'.
!

stringToStringConverter
	^DelegatingDatabaseConverter
		named: #stringToString
		hostedBy: self
		fromStToDb: #padString:for:
		fromDbToSt: #unpadString:for:.
!

stringToSymbol: aString for: aType

	^(self unpadString: aString for: aType) asSymbol
!

symbolToString: aSymbol for: aType

	^self padString: aSymbol asString for: aType.
!

symbolToStringConverter
	^DelegatingDatabaseConverter
		named: #symbolToString
		hostedBy: self
		fromStToDb: #symbolToString:for:
		fromDbToSt: #stringToSymbol:for:.
!

trimString: aString for: aType 
	| trimmedString length excess |
	self measuresStringFieldWidthsInBytes ifFalse: [
		^aString size > aType width ifTrue: [aString copyFrom: 1 to: aType width] ifFalse: [aString]].
	trimmedString := aString.
	[length := (trimmedString asByteArrayEncoding: self characterEncoding) size.
	excess := length - aType width.
	excess > 0 
		"Try to be careful and trim as little as possible, while still keeping the number of steps reasonable for large excesses"
		ifTrue: [trimmedString := trimmedString copyFrom: 1 to: trimmedString size - ((excess // 2) max: 1)]
		ifFalse: [^trimmedString].
	true] whileTrue.
!

unpadString: aString for: aType

	aString isNil ifTrue: [^nil].
	^aType isVariableWidth 
		ifTrue: [aString]
		ifFalse: [(ReadStream on: aString) upTo: 1 asCharacter].
! !

!DatabasePlatform methodsFor:'conversion-times'!

dateConverter
	^DelegatingDatabaseConverter
		named: #date
		hostedBy: self
		fromStToDb: #nullConversion:for:
		fromDbToSt: #readDate:for:.	"#printDate:for:"
!

printDate: aTimestamp isoFormatOn: stream
	"Print the date as yyyy-mm-dd"
	| monthNumber dayOfMonth |
	aTimestamp isNil ifTrue: [^'NULL'].
	aTimestamp year printOn: stream.
	stream nextPut: $-.
	monthNumber := aTimestamp monthIndex.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: monthNumber).
	stream nextPut: $-.
	dayOfMonth := aTimestamp dayOfMonth.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: dayOfMonth).
!

printTime: aTime isoFormatOn: aStream 

	self
		printTime: aTime
		isoFormatOn: aStream
		milliseconds: self supportsMillisecondsInTimes.
	^self
!

printTime: aTimestamp isoFormatOn: stream milliseconds: aBoolean
	"Print the time as hh:mm:ss.mmm"
	| ms |
	aTimestamp isNil ifTrue: [^nil].
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp hours).
	stream nextPut: $:.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp minutes).
	stream nextPut: $:.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp seconds).
	aBoolean ifFalse: [^self].
	ms := aTimestamp milliseconds.
	ms = 0 ifTrue: [^self].
	stream nextPut: $..
	stream nextPutAll: (GlorpDatabaseType padToThreeDigits: ms).
!

printTimestamp: aTimestamp on: aStream for: aType
	aTimestamp isNil ifTrue: [aTimestamp glorpPrintSQLOn: aStream. ^self].

	aStream nextPut: $'; nextPutAll: aTimestamp printString; nextPut: $'.
!

readDate: anObject for: aType
	"format '2003-03-13"
	anObject isNil ifTrue: [^nil].
	anObject class == Date ifTrue: [^anObject].
	anObject isString ifTrue: [
		^self readDateFromStream: (ReadStream on: anObject) for: aType].
	^anObject asDate.
!

readDateFromStream: aStream for: aType
	"Seems like we get to do this ourselves, in a lowest common denominator kind of way. Translate into GMT if we've got a timezone."
	"assumes ISO format.
	self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil

"
	|  years months days |
	years := (aStream upTo: $-) asNumber.
	months := (aStream upTo: $-) asNumber.
	days := (aStream upTo: $ ) asNumber.
	^Dialect newDateWithYears: years months: months days: days.
!

readTime: anObject for: aType
	"format 15:29:28.337-05  (timezone optional)"
	anObject isNil ifTrue: [^nil].
	anObject class == Time ifTrue: [^anObject].
	anObject isString ifTrue: [
		^self readTimeFromStream: (ReadStream on: anObject) for: aType].
	^anObject asTime.
!

readTimeFromStream: aStream for: aType
        "Seems like we get to do this ourselves, in a lowest common denominator kind of way. Ignore timezones right now"
        "assumes ISO format.
        self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
        self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil

"
        | hours minutes seconds milliseconds timeZoneOffset millisecondAccumulator |
        hours := (aStream upTo: $:) asNumber.
        minutes := (aStream upTo: $:) asNumber.
        seconds := (aStream next: 2) asNumber.
        (aStream peek = $.) 
                ifTrue: [
                        aStream next.
                        millisecondAccumulator := WriteStream on: String new.
                        [aStream atEnd not and: [aStream peek isDigit]] whileTrue: [
                                millisecondAccumulator nextPut: aStream next].
                        milliseconds := millisecondAccumulator contents asNumber]
                ifFalse: [milliseconds := 0].
        timeZoneOffset := Number readFromString: aStream upToEnd onError: [0].
        ^Dialect newTimeWithHours: hours minutes: minutes seconds: seconds milliseconds: milliseconds.
        "^Dialect addSeconds: (timeZoneOffset * -1* 60 * 60) to: aTime."
!

readTimestamp: anObject for: aType
	"Seems like we get to do this ourselves, in a lowest common denominator kind of way. Translate into GMT if we've got a timezone."

	"assumes ISO format.
	self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil"

	anObject isNil ifTrue: [^nil].
	anObject class == Dialect timestampClass ifTrue: [^anObject].
	anObject isString
		ifTrue:
			[| stream |
			stream := ReadStream on: anObject.
			^self readTimestampFromStream: stream for: aType].
	^anObject asTimestamp.
!

readTimestampFromStream: aStream for: aType

        | years months days hours minutes seconds millisecondAccumulator milliseconds timeZoneOffset |
        years := (aStream upTo: $-) asNumber.
        months := (aStream upTo: $-) asNumber.
        days := (aStream upTo: $ ) asNumber.
        hours := (aStream upTo: $:) asNumber.
        minutes := (aStream upTo: $:) asNumber.
        seconds := (aStream next: 2) asNumber.
        (aStream peek = $.) 
                ifTrue: [
                        aStream next.
                        millisecondAccumulator := WriteStream on: String new.
                        [aStream atEnd not and: [aStream peek isDigit]] whileTrue: [
                                millisecondAccumulator nextPut: aStream next].
                        milliseconds := millisecondAccumulator contents asNumber]
                ifFalse: [milliseconds := 0].
        timeZoneOffset := Number readFromString: aStream upToEnd onError: [0]. " for compatibility with st-80"
        timeZoneOffset isNil ifTrue: [timeZoneOffset := 0].
        timeZoneOffset := timeZoneOffset * 60 * 60.
        ^Dialect 
                newTimestampWithYears: years
                months: months
                days: days
                hours: hours
                minutes: minutes
                seconds: seconds
                milliseconds: milliseconds
                offset:timeZoneOffset.
!

timeConverter
	^DelegatingDatabaseConverter
		named: #time
		hostedBy: self
		fromStToDb: #nullConversion:for:
		fromDbToSt: #readTime:for:.	"#printTime:for:"
!

timestampConverter
	^DelegatingDatabaseConverter
		named: #timestamp
		hostedBy: self
		fromStToDb: #nullConversion:for:
		fromDbToSt: #readTimestamp:for:.	"#printTimestamp:for:"
! !

!DatabasePlatform methodsFor:'database-specific'!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	^aSymbol.
!

printOffset: anInteger on: aCommand
	"By default, do nothing"
!

printPostLimit: anInteger on: aCommand
	"By default, do nothing"
!

printPreLimit: anInteger on: aCommand
	"By default, do nothing"
!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	^self.
! !

!DatabasePlatform methodsFor:'exdi specific'!

exdiTypeForDates
	^#Date.
! !

!DatabasePlatform methodsFor:'functions'!

functions
	functions isNil ifTrue: [self initializeFunctions].
	^functions.
!

initializeFunctions
	"Subclasses should override to add database-specific functions"
	functions := FunctionExpression createBasicFunctions.
! !

!DatabasePlatform methodsFor:'general services'!

predefinedKeywords
	"
		^<OrderdCollection of: String> This method returns a list of preserved keyword, which should
		not be used in database-, table or column names or any othe names in the platform system
	"

	^OrderedCollection new
! !

!DatabasePlatform methodsFor:'sequences'!

databaseSequenceClass
	self subclassResponsibility.
!

newDatabaseSequenceNamed: aString
	"Return a sequence of the type we use, with the given name"
	^self databaseSequenceClass named: aString.
! !

!DatabasePlatform methodsFor:'services columns'!

printColumnsSpecificationFor: aGlorpDatabaseTable on: sqlStatementStream
	aGlorpDatabaseTable fields isEmpty not
		ifTrue:
			[| sepFlag |
			sqlStatementStream
				space;
				nextPut: $(.
			sepFlag := false.
			aGlorpDatabaseTable fields
				do:
					[:eachGlorpDatabaseField | 
					sepFlag ifTrue: [sqlStatementStream nextPutAll: self columnNameSeparatorString].
					sqlStatementStream
						nextPutAll: (self nameForColumn: eachGlorpDatabaseField name);
						space;
						nextPutAll: eachGlorpDatabaseField typeString;
						space;
						nextPutAll:
								(eachGlorpDatabaseField isNullable
										ifTrue: [self sqlTextForNULLAttributeConstraint]
										ifFalse: [self sqlTextForNOTNULLAttributeConstraint]);
						space;
						nextPutAll:
								(eachGlorpDatabaseField isUnique
										ifTrue: [self sqlTextForUNIQUEAttributeConstraint]
										ifFalse: [self sqlTextForNOTUNIQUEAttributeConstraint]).
					sepFlag := true]].
!

printFormattedColumnsSpecificationFor: aGlorpDatabaseTable on: sqlStatementStream
        aGlorpDatabaseTable fields isEmpty not
                ifTrue:
                        [| sepFlag |
                        sqlStatementStream
                                space;
                                nextPut: $(.
                        sepFlag := false.
                        aGlorpDatabaseTable fields
                                do:
                                        [:eachGlorpDatabaseField | 
                                        sepFlag ifTrue: [sqlStatementStream nextPutAll: self columnNameSeparatorString; cr; nextPutAll: '      '.].
                                        sqlStatementStream
                                                nextPutAll: (self nameForColumn: eachGlorpDatabaseField name);
                                                space;
                                                nextPutAll: eachGlorpDatabaseField typeString;
                                                space;
                                                nextPutAll:
                                                                (eachGlorpDatabaseField isNullable
                                                                                ifTrue: [self sqlTextForNULLAttributeConstraint]
                                                                                ifFalse: [self sqlTextForNOTNULLAttributeConstraint]);
                                                space;
                                                nextPutAll:
                                                                (eachGlorpDatabaseField isUnique
                                                                                ifTrue: [self sqlTextForUNIQUEAttributeConstraint]
                                                                                ifFalse: [self sqlTextForNOTUNIQUEAttributeConstraint]).
                                        sepFlag := true]].
! !

!DatabasePlatform methodsFor:'services tables'!

createFormattedTableStatementStringFor: aGLORPDatabaseTable 
        "^<String> This method returns a string which can be used to create a database table ..."

        | sqlStatementStream tmpString |
        tmpString := 'create table'.
        sqlStatementStream := WriteStream on: String new.
        sqlStatementStream
                nextPutAll: (self capitalWritingOfSQLCommands 
                                        ifTrue: [tmpString asUppercase]
                                        ifFalse: [tmpString]);
                space.
        self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.

        "Now print the columns specification for each field in the table ..."
        self printFormattedColumnsSpecificationFor: aGLORPDatabaseTable
                on: sqlStatementStream.
        (self supportsConstraints 
                and: [aGLORPDatabaseTable hasPrimaryKeyConstraints]) 
                        ifTrue: 
                                [sqlStatementStream nextPutAll: ', '; cr; nextPutAll: '     '.
                                self printPrimaryKeyConstraintsOn: sqlStatementStream
                                        for: aGLORPDatabaseTable].

        sqlStatementStream
                nextPut: $).
        ^sqlStatementStream contents
!

createTableFKContraintsStatementStringsFor: aGLORPDatabaseTable
	| commandString commands addString |
	commands := OrderedCollection new.
	commandString := 'alter table'.
	addString := 'add'.
	self capitalWritingOfSQLCommands ifTrue: [	commandString := commandString asUppercase.
												addString := addString asUppercase].
	self supportsConstraints
			ifTrue: [aGLORPDatabaseTable foreignKeyConstraintsForCreation
						do: [:eachKeyField |
								| sqlStatementStream |
								sqlStatementStream := WriteStream on: String new.
								sqlStatementStream
									nextPutAll: commandString;
									space.
								self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.
								sqlStatementStream
									space;
									nextPutAll: addString;
									space;
									nextPutAll: eachKeyField creationString.
								commands add: sqlStatementStream contents]].
	^commands
!

createTableIndexStatementStringsFor: aGLORPDatabaseTable
	^aGLORPDatabaseTable indexes
		collect:
			[:idx | 
			| sqlStatementStream |
			sqlStatementStream := WriteStream on: String new.
			sqlStatementStream nextPutAll: 'CREATE INDEX '.
			sqlStatementStream nextPutAll: idx name.
			sqlStatementStream nextPutAll: ' ON '.
			self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.
			idx printStatementOn: sqlStatementStream.
			sqlStatementStream contents].
!

createTableStatementStringFor: aGLORPDatabaseTable 
	"^<String> This method returns a string which can be used to create a database table ..."

	| sqlStatementStream tmpString |
	tmpString := 'create table'.
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll: (self capitalWritingOfSQLCommands 
					ifTrue: [tmpString asUppercase]
					ifFalse: [tmpString]);
		space.
	self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.

	"Now print the columns specification for each field in the table ..."
	self printColumnsSpecificationFor: aGLORPDatabaseTable
		on: sqlStatementStream.
	(self supportsConstraints 
		and: [aGLORPDatabaseTable hasPrimaryKeyConstraints]) 
			ifTrue: 
				[sqlStatementStream nextPutAll: ', '.
				self printPrimaryKeyConstraintsOn: sqlStatementStream
					for: aGLORPDatabaseTable].

	sqlStatementStream
		nextPut: $).
	^sqlStatementStream contents
!

endColumnQuote
	^'"'.
!

nameForColumn: aColumnString
	| upper |
	upper := self capitalWritingOfColumnName ifTrue: [aColumnString asUppercase] ifFalse: [aColumnString].
	(self reservedWords includes: aColumnString asLowercase) ifFalse: [^upper].
	^self startColumnQuote, upper, self endColumnQuote.
!

printDDLTableNameFor: aGLORBDatabaseTable on: sqlStatementStream
	"This method just writes the name of a table to a stream"
	
	(aGLORBDatabaseTable schema asString isEmpty not 
		and: [ self prefixQualifierBeforeCreatingAndDeleting ]) ifTrue:[
			sqlStatementStream
				nextPutAll: (self capitalWritingOfCreatorName 
										ifTrue:[  aGLORBDatabaseTable schema asUppercase ] 
										ifFalse:[aGLORBDatabaseTable schema ]) ;
				nextPutAll: self prefixQualifierSeparatorString.
	].
	sqlStatementStream
		nextPutAll: (self capitalWritingOfTableName 
								ifTrue:[ aGLORBDatabaseTable name asUppercase ] 
								ifFalse:[ aGLORBDatabaseTable name ]).
!

printPrimaryKeyConstraintsOn: sqlStatementStream for: aTable 
	"This method print the constraint specification on sqlStatementStream"

	| sepFlag |
	aTable primaryKeyFields isEmpty ifTrue: [^self].
	sqlStatementStream
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: aTable primaryKeyConstraintName;
		nextPutAll: ' PRIMARY KEY  ('.
	sepFlag := false.
	aTable primaryKeyFields do: 
			[:eachPrimaryKeyField | 
			sepFlag ifTrue: [sqlStatementStream nextPutAll: ','].
			sqlStatementStream nextPutAll: eachPrimaryKeyField name.
			sepFlag := true].
	sqlStatementStream nextPut: $).
	self primaryKeysAreAutomaticallyUnique ifTrue: [^self].
	sqlStatementStream
		nextPutAll: ',';
		cr;
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: aTable primaryKeyUniqueConstraintName;
		nextPutAll: ' UNIQUE  ('.
	sepFlag := false.
	aTable primaryKeyFields do: 
			[:eachPrimaryKeyField | 
			sepFlag ifTrue: [sqlStatementStream nextPutAll: ','].
			sqlStatementStream nextPutAll: eachPrimaryKeyField name.
			sepFlag := true].
	sqlStatementStream nextPut: $)
!

startColumnQuote
	^'"'.
!

validateTableName: tableNameString
	" <Boolean> I return true, if the choosen tableNameString is valid for the platform"

 	^( tableNameString size <= self maximumLengthOfTableName ) 
			and: [ (self predefinedKeywords includes: tableNameString asLowercase) not ]
! !

!DatabasePlatform methodsFor:'testing'!

canBind: aValue to: aType
	(Dialect unbindableClassNames includes: aValue class name) ifTrue: [^false].
	^true.
!

isDB2Platform

	^false
!

isODBCPlatform

	^false
!

isOcelotPlatform

	^false
!

isOraclePlatform

	^false
!

isPostgreSQLPlatform

	^false
!

measuresStringFieldWidthsInBytes
	"Return true if we measure the width of a string in bytes rather than characters. So, a 255-character field can hold 255 bytes worth of string, not 255 characters. This is important with encodings"
	^false.
!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^false.
!

supportsAliasInCompoundQuery
	"Return true if, when we have a subselect, we can (and/or must) alias the result with a table name. Access doesn't like the table name"
	^true.
!

supportsBinding
	"Return true if this platform supports binding parameters rather than printing them as strings into the SQL statement"
	^false.
!

supportsCaseInsensitiveLike
	^false.
!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^true.
!

supportsGroupWritingFor: aCommand
	"Return true if this platform can do array binding or an equivalent for this command"
	^false.
!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^false.
!

supportsMultipleOpenCursors
	"Can this database support multiple open cursors at once"
	^true.
!

supportsOffset
	"Do we support anything analogous to the postgresql OFFSET, skipping the first N rows"
	^false.
!

supportsRowCount
	^true.
!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^false
!

useBinding
	"Return true if we should use binding"

	self supportsBinding ifFalse: [^false].
	useBinding isNil ifTrue: [useBinding := self class useBindingIfSupported].
	^useBinding
!

useBinding: aBoolean
	useBinding := aBoolean.
!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins"
	^false.
!

useOracleOuterJoins
	"Return true if we use the old Oracle x = y (+) syntax for outer joins"
	^false.
!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^self subclassResponsibility.
!

usesIdentityColumns
	^false.
! !

!DatabasePlatform methodsFor:'type converters'!

converterNamed: aSymbol

	^self converters at: aSymbol ifAbsentPut: [
		| converter |
		converter := self perform: (aSymbol, 'Converter') asSymbol.
		converter name: aSymbol]
!

converters
	converters isNil ifTrue: [converters := IdentityDictionary new].
	^converters.
! !

!DatabasePlatform methodsFor:'types'!

blob
	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new].
!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].
!

char: anInteger
	^self char width: anInteger.
!

character
	^self char
!

clob
	^self typeNamed: #blob ifAbsentPut: [GlorpClobType new].
!

datetime
	^self timestamp
!

inMemorySequence
	^self typeNamed: #inMemorySequence ifAbsentPut: [
		GlorpInMemorySequenceDatabaseType representedBy: self int4].
!

int4

	^self subclassResponsibility
!

integer

	^self typeNamed: #integer ifAbsentPut: [GlorpIntegerType new].
!

sequence

	^self subclassResponsibility
!

serial
	self subclassResponsibility.
!

smallint

	^self typeNamed: #smallint ifAbsentPut: [GlorpSmallIntType new].
!

text
	^self typeNamed: #text ifAbsentPut: [GlorpClobType new].
!

timestamp
	self subclassResponsibility.
!

typeNamed: aSymbol ifAbsentPut: aBlock

	| type |
	type := self types at: aSymbol ifAbsentPut: [
		| newType |
		newType := aBlock value.
		newType platform: self].
	type hasParameters ifTrue: [type := type copy].
	^type.
!

types
	types == nil ifTrue: [types := IdentityDictionary new].
	^types.
!

varChar
	^self varchar.
!

varChar: anInt

	^self varchar width: anInt.
!

varbinary
	^self subclassResponsibility.
!

varbinary: anInteger
	^self varbinary width: anInteger.
!

varchar
	^self subclassResponsibility.
!

varchar: anInt

	^self varchar width: anInt.
!

versionFieldFor: aType
	^self typeNamed: #versionNumber ifAbsentPut: [VersionType underlyingType: aType].
! !

!DatabasePlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DatabasePlatform.st,v 1.1 2006/07/26 07:59:14 boris Exp $'
! !
