"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DatabaseTable
	instanceVariableNames:'name schema fields foreignKeyConstraints indexes parent
		lockKeyFields primaryKeyFields isImaginary replacementSubSelect'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!DatabaseTable class methodsFor:'instance creation'!

named: aString

	^self new name: aString.
!

new

	^super new initialize.
! !

!DatabaseTable class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DatabaseTable methodsFor:'As yet unclassified'!

replacementSubSelect: aQuery
	replacementSubSelect := aQuery.
! !

!DatabaseTable methodsFor:'accessing'!

creator
	^self schema
!

creator: aString
	"For backward-compatibility. Use schema: instead."
	self schema: aString.
!

fields

	^fields
!

foreignKeyConstraints
	"Private - Answer the value of the receiver's ''foreignKeyConstraints'' instance variable."

	^foreignKeyConstraints
!

foreignKeyConstraintsForCreation
	"Return the foreign key constraints that should actually be created in the database. Omit those that refer to imaginary tables"
	^foreignKeyConstraints select: [:each |
		each isImaginary not and: [each shouldCreateInDatabase]].
!

isAliased
	^parent notNil.
!

isImaginary
	^isImaginary.
!

isImaginary: aBoolean
	isImaginary := aBoolean.
!

lockKeyFields

	^lockKeyFields
!

name

	^(schema isNil or: [schema isEmpty]) ifTrue: [name] ifFalse: [schema, '.', name].
!

name: anObject

	name := anObject
!

nonPrimaryKeyFields

	^self fields reject: [:each | each isPrimaryKey].
!

parent

	^parent.
!

parent: aDatabaseTable

	parent := aDatabaseTable.
!

platform
	fields do: [:each | ^each type platform].
	^nil.
!

primaryKeyFields

	^primaryKeyFields.
!

qualifiedName

	^self name.
!

schema
	^schema
!

schema: aString

	schema := aString.
	parent isNil ifTrue: [
		self sequences do: [:each | each schema: aString]].
!

sequences
	"Return a collection (which is likely either empty or of size 1) of all the sequences we use."
	| sequences |
	sequences := OrderedCollection new: 2.
	self primaryKeyFields do: [:each | each type hasSequence ifTrue: [sequences add: each type sequence]].
	^sequences.
!

shouldBeWritten
	^isImaginary not.
! !

!DatabaseTable methodsFor:'comparing'!

< aTable
        ^self name < aTable name.
!

<= aTable
	^self name <= aTable name.
! !

!DatabaseTable methodsFor:'create/delete in db'!

creationStringFor: aDatabaseAccessor 
	| creationStream |
	creationStream := WriteStream on: (String new: 1000).
	creationStream
		nextPutAll: 'CREATE TABLE ';
		nextPutAll: self name;
		nextPutAll: ' ( ';
		cr.
	self printFieldsOn: creationStream for: aDatabaseAccessor.
	aDatabaseAccessor platform supportsConstraints ifTrue: [
		self hasPrimaryKeyConstraints 
			ifTrue: [self printDelimiterOn: creationStream].
		self printPrimaryKeyConstraintsOn: creationStream for: aDatabaseAccessor.
		"self hasForeignKeyConstraints ifTrue: [self printDelimiterOn: creationStream].
		self printForeignKeyConstraintsOn: creationStream for: aDatabaseAccessor"].
	creationStream nextPutAll: ')'.
	^creationStream contents
!

dropForeignKeyConstraintsFromAccessor: aDatabaseAccessor 
	self foreignKeyConstraints 
		do: [:each | aDatabaseAccessor dropConstraint: each]
!

dropFromAccessor: aDatabaseAccessor
        (aDatabaseAccessor platform supportsConstraints
            and: [aDatabaseAccessor platform supportsImplicitDroppingOfConstraints not])
                ifTrue: [self dropPrimaryKeyConstraintsFromAccessor: aDatabaseAccessor].
        aDatabaseAccessor dropTableNamed: self name.
!

dropPrimaryKeyConstraintsFromAccessor: aDatabaseAccessor
	self primaryKeyFields isEmpty
		ifFalse:
			[aDatabaseAccessor
				execute:
					[aDatabaseAccessor
						executeSQLStringNoResult:
							'ALTER TABLE ' , self name , ' DROP CONSTRAINT '
								, self primaryKeyUniqueConstraintName]
				ifError: [:ex | Transcript show: ex messageText. ex return: nil].
			aDatabaseAccessor
				execute:
					[aDatabaseAccessor
						executeSQLStringNoResult:
							'ALTER TABLE ' , self name , ' DROP CONSTRAINT '
								, self primaryKeyConstraintName]
				ifError: [:ex | Transcript show: ex messageText. ex return: nil]].
!

primaryKeyConstraintName

	^self name, '_PK'.
!

primaryKeyUniqueConstraintName

	^self name, '_UNIQ'.
!

printDelimiterOn: aStream
	
	aStream
		nextPut: $,;
		cr
!

printFieldsOn: creationStream for: aDatabaseAccessor 
	GlorpHelper 
		do: [:each | each printCreationStringFor: aDatabaseAccessor on: creationStream]
		for: fields
		separatedBy: [self printDelimiterOn: creationStream]
!

printPrimaryKeyConstraintsOn: aStream for: aDatabaseAccessor 
	self primaryKeyFields isEmpty ifTrue: [^self].
	aStream nextPutAll: 'CONSTRAINT '.
	aStream nextPutAll: self primaryKeyConstraintName.
	aStream nextPutAll: ' PRIMARY KEY  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $).

	aStream
		nextPutAll: ',';
		cr.

	aStream nextPutAll: 'CONSTRAINT '.
	aStream nextPutAll: self primaryKeyUniqueConstraintName.
	aStream nextPutAll: ' UNIQUE  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $)
! !

!DatabaseTable methodsFor:'fields'!

addField: aField

	fields add: aField.
	aField isPrimaryKey ifTrue: [
		self addAsPrimaryKeyField: aField].
	aField table: self.
	aField position: fields size.
	^aField.
!

addForeignKeyFrom: sourceField to: targetField
	^self
		addForeignKeyFrom: sourceField
		to: targetField
		suffixExpression: nil.
!

addForeignKeyFrom: sourceField to: targetField from: sourceField2 to: targetField2
	^self
		addForeignKeyFromAll: (Array with: sourceField with: sourceField2)
		toAll: (Array with: targetField with: targetField2)
		suffixExpression: nil.
!

addForeignKeyFrom: sourceField to: targetField from: sourceField2 to: targetField2 from: sourceField3 to: targetField3
	^self
		addForeignKeyFromAll: (Array with: sourceField with: sourceField2 with: sourceField3)
		toAll: (Array with: targetField with: targetField2 with: targetField3)
		suffixExpression: nil.
!

addForeignKeyFrom: sourceField to: targetField suffixExpression: suffixExpression
	| newFK |
	newFK := ForeignKeyConstraint
					sourceField: sourceField
					targetField: targetField
					suffixExpression: suffixExpression.
	newFK name: newFK name , (foreignKeyConstraints size + 1) printString.
	foreignKeyConstraints add: newFK.
	^newFK.
!

addForeignKeyFromAll: sourceFields toAll: targetFields
	^self addForeignKeyFromAll: sourceFields toAll: targetFields suffixExpression: nil.
!

addForeignKeyFromAll: sourceFields toAll: targetFields suffixExpression: suffixExpression
	| newFK |
	newFK := ForeignKeyConstraint
					sourceFields: sourceFields
					targetFields: targetFields
					suffixExpression: suffixExpression.
	newFK name: newFK name , (foreignKeyConstraints size + 1) printString.
	^foreignKeyConstraints add: newFK.
!

createFieldNamed: aString type: dbType

	| existingField |
	existingField := fields detect: [:each | each name = aString] ifNone: [nil].
	(existingField notNil)
	ifTrue: [self error: 'field ', aString, ' already exists'].
	^self addField: (DatabaseField named: aString type: dbType)
!

fieldNamed: aString

	^fields detect: [:each | each name = aString]
!

fieldNamed: aString ifAbsent: aBlock

	^fields detect: [:each | each name = aString] ifNone: aBlock.
!

newFieldNamed: aString

	^self error: 'use #createFieldNamed:type:'
! !

!DatabaseTable methodsFor:'indexes'!

addIndex: aDatabaseIndex
	^indexes add: aDatabaseIndex.
!

addIndexForField: aField
	^indexes add: (DatabaseIndex forField: aField).
!

addIndexForField: aField and: anotherField
	^self addIndexForFields: (Array with: aField with: anotherField).
!

addIndexForField: aField and: anotherField and: thirdField
	^self addIndexForFields: (Array with: aField with: anotherField with: thirdField).
!

addIndexForFields: aFieldCollection
	^indexes add: (DatabaseIndex forFields: aFieldCollection).
!

hasIndexForField: aField
	^(indexes detect: [:each | each indexesField: aField] ifNone: [nil]) notNil.
!

indexes
	^indexes.
! !

!DatabaseTable methodsFor:'initialize'!

initialize

	schema := ''.
	fields := OrderedCollection new.
	primaryKeyFields := #().
	foreignKeyConstraints := OrderedCollection new: 4.
	lockKeyFields := OrderedCollection new.
	indexes := OrderedCollection new.
	isImaginary := false.
!

postInitializeIn: aDescriptorSystem
	"Any initialization that happens after all the fields have been added"

	fields do: [:each | each postInitializeIn: aDescriptorSystem].
! !

!DatabaseTable methodsFor:'printing'!

imaginaryReplacementName
	| stream |
	stream := WriteStream on: (String new: 100).
	stream nextPut: $(.
	stream nextPutAll: (replacementSubSelect sqlWith: #()) sqlString.
	stream nextPutAll: ') '.
	^stream contents.
!

printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: '(';
		nextPutAll: (name isNil ifTrue: [''] ifFalse: [name]);
		nextPutAll: ')'.
!

printSQLOn: aWriteStream withParameters: aDictionary
	aWriteStream nextPutAll: self name
!

sqlString

	^name.
!

sqlTableName
	"Our name, as appropriate for the list of tables in a SQL statement. Take into account aliasing"

	^parent isNil
		ifTrue:
			[self isImaginary
				ifTrue: [self imaginaryReplacementName]
				ifFalse: [self name]]
		ifFalse: [parent sqlTableName , ' ' , self name].
! !

!DatabaseTable methodsFor:'private/fields'!

addAsLockKeyField: aField 

	lockKeyFields add: aField
!

addAsPrimaryKeyField: aField

	(fields includes: aField) ifFalse: [self error: 'Primary key field not part of the table'].
	aField isPrimaryKey ifFalse: ["Somebody is calling this as API, since there's an example out on the net that does it that way - just do the right thing instead"
		^aField bePrimaryKey].
	primaryKeyFields := primaryKeyFields, (Array with: aField)
! !

!DatabaseTable methodsFor:'testing'!

hasCompositePrimaryKey

	^primaryKeyFields size > 1.
!

hasConstraints
	^self hasForeignKeyConstraints or: [self hasPrimaryKeyConstraints]
!

hasConstraintsTo: aTable
	self foreignKeyConstraints
		do: [:each | (each refersToTable: aTable) ifTrue: [^true]].
	^false.
!

hasConstraintsToAnyOf: aCollectionOfTables
	aCollectionOfTables do: [:each |
		(self hasConstraintsTo: each) ifTrue: [^true]].
	^false.
!

hasFieldNamed: aString

	^fields contains: [:each | each name = aString]
!

hasForeignKeyConstraints
	^foreignKeyConstraints isEmpty not
!

hasPrimaryKeyConstraints
	^self primaryKeyFields isEmpty not.
! !

!DatabaseTable methodsFor:'validation'!

validateIn: aDescriptorSystem
	"Do any tests we can to make sure that this definition makes sense in this context"
! !

!DatabaseTable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DatabaseTable.st,v 1.1 2006/07/26 08:03:34 boris Exp $'
! !
