"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DescriptorSystem
	instanceVariableNames:'session platform descriptors tables sequences typeResolvers
		cachePolicy allClasses useDirectAccessForMapping classModels
		useJoinsForAnySatisfy'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!DescriptorSystem class methodsFor:'instance creation'!

forPlatform: dbPlatform

	^super new initialize;
		platform: dbPlatform
!

new

	^super new initialize.
! !

!DescriptorSystem class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DescriptorSystem methodsFor:'As yet unclassified'!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once. Normally we delegate this to the platform, but we allow the system to intervene if desired"
	^platform maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows.
! !

!DescriptorSystem methodsFor:'accessing'!

allClassModels
	^self allClasses collect: [:each | 
		self classModelFor: each].
!

allClasses
	allClasses isNil ifTrue: [allClasses := self constructAllClasses].
	^allClasses
!

allDescriptors

	^self allClasses collect: [:each | 
		self descriptorFor: each].
!

allSequences
	sequences isEmpty ifFalse: [^sequences].
	self allTables do: [:each | 
		each fields do: [:eachField |
			eachField type hasSequence ifTrue: [
				sequences at: eachField type sequence name put: eachField type sequence]]].
	^sequences.
!

allTableNames

	self subclassResponsibility.
!

allTables
	^self allTableNames collect: [:each | 
		self tableNamed: each asString].
!

cachePolicy
	"Return the default cache policy that will be used for descriptors that don't specify their own policy"
	cachePolicy isNil ifTrue: [cachePolicy := CachePolicy default].
	^cachePolicy.
!

cachePolicy: aCachePolicy
	cachePolicy := aCachePolicy.
!

constructAllClasses
	^IdentitySet new
!

defaultTracing

	^Tracing new
!

platform

	^platform
!

platform: dbPlatform

	platform := dbPlatform
!

session
	^session
!

session: anObject
	session := anObject
!

useDirectAccessForMapping
	^useDirectAccessForMapping
!

useDirectAccessForMapping: anObject
	useDirectAccessForMapping := anObject
! !

!DescriptorSystem methodsFor:'api'!

addClassModelFor: aClass
	^classModels
		at: aClass
		ifAbsentPut: [self newClassModelFor: aClass]
!

classModelFor: aClass
	(self allClasses includes: aClass) ifFalse: [^nil].
	^classModels
		at: aClass
		ifAbsentPut: [self newClassModelFor: aClass]
!

createMappingsFromClassModelIn: aDescriptor 
	aDescriptor classModel attributesDo: 
			[:each | 
			(aDescriptor mappingForAttributeNamed: each name) isNil 
				ifTrue: 
					[self createMappingIn: aDescriptor
						fromClassModelForAttributeNamed: each name]]
!

databaseSequenceNamed: aString

	^sequences at: aString asUppercase ifAbsentPut: [platform newDatabaseSequenceNamed: aString].
!

databaseSequenceNamed: aString ifAbsentPut: aBlock

	^sequences at: aString asUppercase ifAbsentPut: aBlock.
!

descriptorFor: aClassOrObject 
        | theClass |
        aClassOrObject == Proxy ifTrue: [self error: 'Cannot find descriptor for the class Proxy. Pass in the instance'].
        theClass := aClassOrObject glorpCanHaveDescriptorDirectly 
                                ifTrue: [aClassOrObject]
                                ifFalse: [aClassOrObject isGlorpProxy 
                                                        ifTrue: [aClassOrObject getValue class]
                                                        ifFalse: [aClassOrObject class]].
  theClass isNil ifTrue: [self halt.].
        ^descriptors
                at: theClass
                ifAbsent: [
                        (self allClasses includes: theClass)" ifFalse: [self halt]" ifTrue: [
                                descriptors at: theClass put: (self newDescriptorFor: theClass)]].
!

existingTableNamed: aString

	^tables at: aString asUppercase ifAbsent: [self error: 'missing table'].
!

flushAllClasses
	allClasses := nil
!

hasDescriptorFor: aClassOrObject 

	^(self descriptorFor: aClassOrObject) notNil.
!

tableNamed: aString

	^tables at: aString asString asUppercase ifAbsent: [
		| newTable|
		newTable := DatabaseTable new.
		newTable name: aString.
		tables at: aString asUppercase put: newTable.
		self initializeTable: newTable.
		newTable].
!

typeResolverFor: aClassOrObject 

	| theClass |
	aClassOrObject == Proxy ifTrue: [self error: 'Cannot find type resolver for the class Proxy. Pass in the instance'].
	theClass := aClassOrObject glorpCanHaveDescriptorDirectly 
				ifTrue: [aClassOrObject]
				ifFalse: [aClassOrObject isGlorpProxy ifTrue: [aClassOrObject getValue class] ifFalse: [aClassOrObject class]].
	^typeResolvers at: theClass ifAbsentPut: [self newTypeResolverFor: theClass]
! !

!DescriptorSystem methodsFor:'hooks'!

additionalRelatedRowsFor: aRow in: aRowMap
	"This method serves as a hook. It is called when sorting rows before writing to the database, and can be used to return a list of additional related rows that aren't found by the normal foreign key constraints. These rows will act as an ordering constraint, and will have to be written before aRow."
	^nil.
!

fieldNameForAttributeName: aSymbol 
	^aSymbol asString asUppercase
! !

!DescriptorSystem methodsFor:'initialization'!

setUpDefaults
	"For systems that are configurable, set them up for testing configuration"
! !

!DescriptorSystem methodsFor:'optimization parameters'!

shouldUpdateOnlyChangedFields
	"Return true if we should write only changed fields out in an update statement, or if we should write all of them. The tradeoff is that writing only the changed ones will produce less SQL, but make more variations of the statement, and require us to keep more prepared statements around if we're using binding. By default, use this option only if we're not using binding by default"
	^self platform useBinding.
! !

!DescriptorSystem methodsFor:'private'!

converterBetweenStType: aClass andField: aField 
	| selector |
	selector := ('converterBetween' , aClass name , 'AndField:') asSymbol.
	(self respondsTo: selector) ifTrue: [^self perform: selector with: aField].
	^aField converterForStType: aClass
!

createMappingIn: aDescriptor fromClassModelForAttributeNamed: aSymbol 
	| attribute referenceModel |
	attribute := aDescriptor classModel attributeNamed: aSymbol.
	referenceModel := self classModelFor: attribute type.
	referenceModel isNil 
		ifTrue: 
			[^(aDescriptor newMapping: DirectMapping) from: aSymbol
				to: (aDescriptor table 
						fieldNamed: (self fieldNameForAttributeName: aSymbol))]
		ifFalse: 
			[attribute isCollectionAttribute 
				ifTrue: [^(aDescriptor newMapping: ToManyMapping) attributeName: aSymbol]
				ifFalse: [^(aDescriptor newMapping: OneToOneMapping) attributeName: aSymbol]]
!

initialize
	descriptors := Dictionary new.
	tables := Dictionary new.
	classModels := Dictionary new.
	typeResolvers := Dictionary new.
	sequences := Dictionary new.
	useDirectAccessForMapping := true
!

initializeClassModel: aClassModel
        | selector simpleClassName idx |
        simpleClassName := aClassModel describedClass name.
        (simpleClassName includes: $:) ifTrue: [
           idx := simpleClassName indexOf: $:.
           simpleClassName := simpleClassName copyFrom: idx + 2 to: simpleClassName size.
        ].

        selector := ('classModelFor' , simpleClassName , ':')
                asSymbol.

        (self respondsTo: selector)
                ifTrue: [^self perform: selector with: aClassModel].
        (aClassModel describedClass respondsTo: #glorpSetupClassModel:forSystem:)
                ifTrue:
                        [^aClassModel describedClass
                                glorpSetupClassModel: aClassModel
                                forSystem: self].
        "Otherwise, just initialize it dynamically, so that old code with only descriptors will still work"
        ^aClassModel.
!

initializeDescriptor: aDescriptor 
        | selector simpleClassName idx |
        simpleClassName := aDescriptor describedClass name.
        (simpleClassName includes: $:) ifTrue: [
           idx := simpleClassName indexOf: $:.
           simpleClassName := simpleClassName copyFrom: idx + 2 to: simpleClassName size.
        ].
        selector := ('descriptorFor' , simpleClassName , ':') asSymbol.
        (self respondsTo: selector) 
                ifTrue: [self perform: selector with: aDescriptor]
                ifFalse: [aDescriptor describedClass
                                            glorpSetupDescriptor: aDescriptor
                                            forSystem: self]
!

initializeTable: newTable
	self perform: ('tableFor' , (newTable name asUppercase) , ':')  asSymbol with: newTable.
	newTable postInitializeIn: self.
!

newClassModelFor: aClass

	| newClassModel |
	newClassModel := GlorpClassModel new.
	newClassModel system: self.
	newClassModel describedClass: aClass.
	self initializeClassModel: newClassModel.
	^newClassModel.
!

newDescriptorFor: aClass

	| newDescriptor |
	(self allClasses includes: aClass) ifFalse: [^nil].
	newDescriptor := Descriptor new.
	newDescriptor system: self.
	newDescriptor classModel: (self classModelFor: aClass).
	self initializeDescriptor: newDescriptor.
	^newDescriptor.
!

newTypeResolverFor: aClass 
        | selector simplifiedClassName idx |
        (self allClasses includes: aClass) ifFalse: [^nil].
        simplifiedClassName := aClass name.
        (simplifiedClassName includes: $:) ifTrue: [
           idx := simplifiedClassName indexOf: $:.
           simplifiedClassName := simplifiedClassName copyFrom: idx + 2 to: simplifiedClassName size.
        ].

        selector := ('typeResolverFor' , simplifiedClassName) asSymbol.
        ^(self respondsTo: selector) 
                ifTrue: [self perform: selector]
                ifFalse: [aClass glorpTypeResolver]
!

privateAndBasicAddDescriptorFor: anObject as: aDescriptor
	"Just add the descriptor and don't try to validate. anObject may not actually even be a class. Assume the descriptor is already properly initialized"
	descriptors at: anObject put: aDescriptor.
	aDescriptor system: self.
! !

!DescriptorSystem methodsFor:'settings'!

shouldForeignKeyValueBeTreatedAsNull: aValue
	"Return true if this foreign key value should be treated as null, meaning there's no relationship. Normally this means just nil, but descriptor systems can override if they have special values which they use (e.g. 0) that indicate a missing relationship. This can improve performance by avoiding unnecessary queries to the database for things we know will never return a result"
	^aValue isNil.
!

shouldResetExpiryTimeForObjectOfClass: aClass readFrom: aQuery
	"Return true if, when an object is read from a query, even if it's not refreshing. we 'touch' it, resetting its expiry time. By default, we do this if the cache policy for that type of object will remove from the cache. The rationale is that this means we're using the object. Removing an object in use is a very bad thing, and can cause loss of identity and other bad things. However, if expiry only indicates a need to refresh, we don't need to touch the object. Subclasses can override to provide different behaviour."

	aQuery shouldRefresh ifTrue: [^true].
	aQuery returnProxies ifTrue: [^false].
	^(self session cacheForClass: aClass)  willRemoveItems
!

useJoinsForAnySatisfy
	useJoinsForAnySatisfy isNil ifTrue: [useJoinsForAnySatisfy := false].
	^useJoinsForAnySatisfy.
!

useJoinsForAnySatisfy: aBoolean
	useJoinsForAnySatisfy := aBoolean.
! !

!DescriptorSystem methodsFor:'testing'!

canUseGroupedWrites
	"We can use grouped writes as long as there are no identity columns in use"
	self platform usesIdentityColumns ifFalse: [^true].
	^(self allTables anySatisfy: [:each |
		each primaryKeyFields anySatisfy: [:eachField |
			eachField type isSerial and: [eachField type sequence isIdentityColumn]]]) not.
!

canUseGroupedWritesFor: aTable
	"We can use grouped writes as long as there are no identity columns in use"
	self platform usesIdentityColumns ifFalse: [^true].
	^(aTable primaryKeyFields anySatisfy: [:eachField |
			eachField type isSerial and: [eachField type sequence isIdentityColumn]]) not.
! !

!DescriptorSystem methodsFor:'validation'!

validate
	self allTables do: [:each | each validateIn: self].
	self allClassModels do: [:each | each validateIn: self].
	self allDescriptors do: [:each | each validate].
!

validateRows: aUnitOfWork
	"This is a hook to allow the system to do application-specific validation of the rows to be written. I suppose it's also possible to modify the rows here. Subclasses can override if they have useful work to do here. This will be called after the rows are generated, pre-write notification has been sent, rows have been sorted for writing, and and just before sequence numbers (if pre-write) are assigned, and writing starts"
!

validateRowsPostWrite: aUnitOfWork
	"Validate after writing. This is a bit late, but some things may not be catchable ahead of time"
! !

!DescriptorSystem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DescriptorSystem.st,v 1.1 2006/07/26 08:04:12 boris Exp $'
! !
