"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Mapping subclass:#DirectMapping
	instanceVariableNames:'field converter'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!DirectMapping class methodsFor:'instance creation'!

from: attributeName to: field

	^self from: attributeName
		type: field impliedSmalltalkType
		to: field
!

from: attributeName type: aClass to: field

	^self new
		attributeName: attributeName;
		field: field;
		type: aClass;
		setConverterBetween: aClass and: field
!

fromPseudoVariable: attributeName to: field

	^self fromPseudoVariable: attributeName
		type: field impliedSmalltalkType
		to: field
!

fromPseudoVariable: attributeName type: aClass to: field

	^self new
		attributeName: attributeName;
		field: field;
		setConverterBetween: aClass and: field;
		beForPseudoVariable.
! !

!DirectMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DirectMapping methodsFor:'accessing'!

converter
	^converter.
!

converter: aDatabaseConverter
	converter := aDatabaseConverter.
!

field
	^field
!

field: aDatabaseFieldOrExpression
	field := aDatabaseFieldOrExpression.
!

type: aClass
	self attribute type: aClass.
! !

!DirectMapping methodsFor:'enumerating'!

referencedIndependentObjectsFrom: anObject

	^#().
! !

!DirectMapping methodsFor:'initialize-release'!

from: anAttributeName to: aField

	| type |
	self attributeName: anAttributeName.
	self field: aField.
	type := self attribute type isNil ifTrue: [aField impliedSmalltalkType] ifFalse: [self attribute type].
	self type: type.
	self setConverterBetween: type and: aField.
!

from: anAttributeName type: aClass to: aField

	self
		attributeName: anAttributeName;
		field: aField;
		type: aClass;
		setConverterBetween: aClass and: aField
!

fromPseudoVariable: anAttributeName to: aField

	^self fromPseudoVariable: anAttributeName
		type: aField impliedSmalltalkType
		to: aField
!

fromPseudoVariable: anAttributeName type: aClass to: aField

	self from: anAttributeName type: aClass to: aField.
	self beForPseudoVariable.
!

setConverterBetween: aClass and: aDbField 
	aClass isNil ifTrue: [^self].
	converter := descriptor ifNotNil: 
					[descriptor system converterBetweenStType: aClass andField: aDbField]
				ifNil: [aDbField converterForStType: aClass]
! !

!DirectMapping methodsFor:'mapping'!

convertValueToDatabaseForm: aValue
	
	converter isNil ifTrue: [^aValue].
	^converter convertedDbValueOf: aValue.
!

expressionFor: anObject basedOn: anExpression relation: aSymbol
	"Return our expression using the object's values. e.g. if this was a direct mapping from id->ID and the object had id: 3, then return TABLE.ID=3"

	| value |
	value := anObject isNil
		ifTrue: [nil]
		ifFalse:
			[anObject isGlorpExpression
				ifTrue: [anObject get: self attributeName]
				ifFalse: [anObject glorpIsCollection
					ifTrue: [anObject collect: [:each | attribute getValueFrom: each]]
					ifFalse: [attribute getValueFrom: anObject]]].
	^(anExpression get: self attribute name) get: aSymbol withArguments: (Array with: value).
!

mapFromObject: anObject intoRowsIn: aRowMap

	^self mapFromObject: anObject toTarget: anObject puttingRowsIn: aRowMap.
!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap

	| dbValue value row |
	canWrite ifFalse: [^self].
	value := self getValueFrom: anObject.
	dbValue := self convertedDbValueOf: value.
	row := aRowMap findOrAddRowForTable: self field table withKey: target.
	row at: field put: dbValue.
!

mapObject: anObject inElementBuilder: anElementBuilder
	| value |
	value := self valueInBuilder: anElementBuilder.
	self setValueIn: anObject to: value.
!

readBackNewRowInformationFor: anObject fromRowsIn: aRowMap 
	| value row |
	field isGenerated ifFalse: [^self].
	canWrite ifFalse: [^false].
	row := aRowMap findOrAddRowForTable: self field table withKey: anObject.
	value := self convertedStValueOf: (row at: field ifAbsent: [^self]).
	attribute setValueIn: anObject to: value
!

trace: aTracing context: anExpression
	^self.
!

valueIn: anElementBuilder as: anExpression
	| dbValue valueToGet |
	valueToGet := anExpression canTranslateFields 
		ifTrue: [anExpression translateField: field] 
		ifFalse: [anExpression].
	dbValue := anElementBuilder
		valueOfField: valueToGet.
	^self convertedStValueOf: dbValue.
!

valueInBuilder: anElementBuilder
	^self valueInBuilder: anElementBuilder as: field.
!

valueInBuilder: anElementBuilder as: anExpression
	| dbValue |
	dbValue := anElementBuilder valueOfField: anExpression.
	^self convertedStValueOf: dbValue.
!

valueOfField: aField fromObject: anObject
	field = aField ifFalse: [self error: 'Mapping doesn''t describe field'].
	^self convertedDbValueOf: (self getValueFrom: anObject).
! !

!DirectMapping methodsFor:'printing'!

printMappingInformationOn: aTextStream

  aTextStream nextPutAll: attribute name;
              nextPutAll: ' -> ';
              nextPutAll: field printString.
! !

!DirectMapping methodsFor:'printing SQL'!

printSQLOn: aStream withParameters: aDictionary
	self field printSQLOn: aStream withParameters:aDictionary.
! !

!DirectMapping methodsFor:'private'!

convertedDbValueOf: stValue 
	^converter isNil
		ifTrue: [stValue]
		ifFalse: [converter convert: stValue toDatabaseRepresentationAs: self field type]
!

convertedStValueOf: dbValue 
	^(converter isNil)
		ifTrue: [dbValue]
		ifFalse: [converter convert: dbValue fromDatabaseRepresentationAs: self field type]
! !

!DirectMapping methodsFor:'testing'!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false
!

isRelationship
	"True when the mapping associates different persistent classes."

	^false
!

isStoredInSameTable

	^true
!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^Array with: self field
!

mapsSimpleObject
	^true.
! !

!DirectMapping methodsFor:'validation'!

validate
	super validate.
	(field isGlorpExpression and: [self readOnly not]) ifTrue: [self error: 'Mappings to functions cannot be written'].
! !

!DirectMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DirectMapping.st,v 1.1 2006/07/26 08:05:20 boris Exp $'
! !
