"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

OneToOneMapping subclass:#EmbeddedValueOneToOneMapping
	instanceVariableNames:'fieldTranslation'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

EmbeddedValueOneToOneMapping comment:'This represents a one-to-one mapping in which the referenced object is stored as part of the same table as the containing object.
'''
!


!EmbeddedValueOneToOneMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!EmbeddedValueOneToOneMapping methodsFor:'accessing'!

fieldTranslation
	^fieldTranslation
!

fieldTranslation: aPrimaryKeyExpression 
	fieldTranslation := aPrimaryKeyExpression
!

hasFieldTranslation

	^fieldTranslation notNil.
!

join
	^nil.
!

join: anExpression
	self error: 'Embedded mappings do not use joins'.
! !

!EmbeddedValueOneToOneMapping methodsFor:'internal'!

fieldsForSelectStatement
	"Return a collection of fields that this mapping will read from a row"
	"Return nothing, because our sub-objects will take care of adding their own fields, translated correctly through us."
	^#().
!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	fieldTranslation isNil ifFalse: [^fieldTranslation allSourceFields].
	^self referenceDescriptor mappedFields.
! !

!EmbeddedValueOneToOneMapping methodsFor:'mapping'!

knitFromBuilder: sourceBuilder toBuilder: targetBuilder 
	sourceBuilder requiresPopulating ifFalse: [^self].
	super knitFromBuilder: sourceBuilder toBuilder: targetBuilder
!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap 

	self transformationExpression 
		mapFromSource: anObject
		andTarget: target
		intoRowsIn: aRowMap.

	self transformationExpression allTables do: [:eachTable |	
		aRowMap rowForTable: eachTable withKey: target ifPresent: [  :each |
			each shouldBeWritten: false]].
!

mapObject: anObject inElementBuilder: anElementBuilder
	| myTraceNode myBuilder |
	"If the object already has a value in my slot, then this it got a cache hit, the embedded value was carried along for the ride, and we don't need to assign anything"
	(self getValueFrom: anObject) isNil ifFalse: [^self].	"Otherwise, we need to look up the trace node that corresponds to this mapping, and get its instance"
	myTraceNode := anElementBuilder expression get: self attributeName.
	myBuilder := anElementBuilder query elementBuilderFor: myTraceNode.
	self setValueIn: anObject to: myBuilder instance.
!

reverseMapping
	^self class new
		referenceClass: self descriptor describedClass;
		descriptor: self descriptor.
!

translateFields: anOrderedCollection 
	fieldTranslation isNil ifTrue: [^anOrderedCollection].
	^anOrderedCollection collect: [:each |
		fieldTranslation sourceForTarget: each].
!

translateTable: aTable
	"If we have a field translation, return the translated (i.e. real) table to use. This ideally allows us to use either the real table in which something is embedded, or the imaginary table"

	fieldTranslation isNil ifTrue: [^aTable].
	1 to: fieldTranslation allTargetFields size
		do:
			[:i | 
			(fieldTranslation allTargetFields at: i) table == aTable
				ifTrue: [^(fieldTranslation allSourceFields at: i) table]].
	^aTable.
!

valueOfField: aField fromObject: anObject ifNone: aBlock
	| targetField target |
	targetField := self transformationExpression targetForSource: aField.
	targetField isNil ifTrue: [^aBlock value].
	target := self getValueFrom: anObject.
	^self referenceDescriptor valueOfField: targetField fromObject: target.
! !

!EmbeddedValueOneToOneMapping methodsFor:'preparing'!

joinExpressionFor: targetExpression
	"We're looking for the object represented by this mapping, and we know the object represented by its source. Use our mapping criteria to construct a join that traverses that instance of this relationship.
	Embedded values never induce a join."

	^nil.
! !

!EmbeddedValueOneToOneMapping methodsFor:'printing'!

printMappingInformationOn: aTextStream
       | mf attributeClassModel subnames  |
  mf := self mappedFields.
  attributeClassModel := attribute classModel system classModelFor: attribute type.
  subnames := OrderedCollection new.
  attributeClassModel attributesDo:
       [:item |
         subnames add: item name.
       ].

  subnames with: mf do:
    [:name :mappedField |
      aTextStream nextPutAll: attribute name; nextPut: $.; nextPutAll: name;
              nextPutAll: ' -> ';
              nextPutAll: mappedField printString; cr;
              nextPutAll: '    '.
    ].
! !

!EmbeddedValueOneToOneMapping methodsFor:'testing'!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false
!

isStoredInSameTable
	^true
!

shouldProxy

	^false.
! !

!EmbeddedValueOneToOneMapping methodsFor:'transformations'!

defaultTransformationExpressionFor: aDescriptor 
	"If there's no transformation, get all the mapped fields from the other descriptor and construct a transformation of each onto itself. This lets us unify the fields in my row with the fields in its row"

	| fields transform |
	fields := IdentitySet new.
	aDescriptor mappings do: [:each | fields addAll: each mappedFields].
	transform := Join new.
	fields do: [:each | transform addSource: each target: each].
	^transform
!

hasTransformation

	^false.
!

transformationExpression

	^self hasFieldTranslation
		ifTrue: [fieldTranslation]
		ifFalse: [fieldTranslation := self defaultTransformationExpressionFor: (self referenceDescriptor)].
! !

!EmbeddedValueOneToOneMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/EmbeddedValueOneToOneMapping.st,v 1.1 2006/07/26 08:03:20 boris Exp $'
! !
