"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#FieldExpression
	instanceVariableNames:'field base alias'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!FieldExpression class methodsFor:'instance creation'!

forField: aField basedOn: anObjectExpression

	^self new field: aField base: anObjectExpression; yourself
! !

!FieldExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!FieldExpression methodsFor:'As yet unclassified'!

convertedDbValueOf: anObject
	"We don't do any conversion"
	^anObject
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	| result |
	self == anExpression ifTrue: [^aBaseExpression].
	result := (base rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression) getField: field.
	result alias: self alias.
	^result.
!

tableForANSIJoin
	^self field table.
! !

!FieldExpression methodsFor:'accessing'!

alias
	^alias
!

alias: aString
	"What column name, if anything, should we print for 'as X' when retrieving"
	alias := aString
!

base

	^base.
!

basicField
	^field.
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

converterForStType: aClass
	^self field converterForStType: aClass.
!

field
	^base translateField: field
!

fieldExpression
	^self.
!

impliedSmalltalkType
	^self field impliedSmalltalkType.
!

name
	^field name.
!

table
	^self field table.
!

type
	^self field type.
! !

!FieldExpression methodsFor:'api'!

get: aSymbol withArguments: anArray

	| functionExpression |
	functionExpression := self getFunction: aSymbol withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	anArray isEmpty ifTrue: [self error: 'Field expressions do not have attributes'].
	^RelationExpression named: aSymbol basedOn: self withArguments: anArray.
! !

!FieldExpression methodsFor:'copying'!

setTable: aTable
	"Modify (do not copy) ourselves to use aTable. Presumably aTable is an aliased version of our table"
	field := field withTable: aTable.
!

withTable: aTable
	"Modify (do not copy) ourselves to use aTable. Presumably aTable is an aliased version of our table"
	field := field withTable: aTable.
! !

!FieldExpression methodsFor:'initializing'!

field: aField base: anObjectExpression

	field := aField.
	base := anObjectExpression.
! !

!FieldExpression methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree"

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	aBlock value: self.
! !

!FieldExpression methodsFor:'mapping'!

mappedFields
	^Array with: self field.
!

valueInBuilder: anElementBuilder
	^self valueInBuilder: anElementBuilder as: self field.
!

valueInBuilder: anElementBuilder as: anExpression
	^anElementBuilder valueOfField: anExpression.
! !

!FieldExpression methodsFor:'navigating'!

queryLevelBaseExpression
	^base queryLevelBaseExpression.
!

ultimateBaseExpression
	^base ultimateBaseExpression.
! !

!FieldExpression methodsFor:'preparing'!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."
	| newTarget |
	newTarget := (target tables includes: field table) 
		ifTrue: [target]
		ifFalse: [base asExpressionJoiningSource: source toTarget: target].
	^newTarget getField: field.
!

tables

	^base tables.
!

tablesToPrint

	^Array with: self table
! !

!FieldExpression methodsFor:'printing'!

printOnlySelfOn: aStream 
	base printsTable
		ifTrue: [field printUnqualifiedSQLOn: aStream withParameters: #()]
		ifFalse: [field printSQLOn: aStream withParameters: #()]
!

printTreeOn: aStream 
	base printOn: aStream.
	aStream nextPut: $..
	base printsTable 
		ifTrue: [field printUnqualifiedSQLOn: aStream withParameters: #()]
		ifFalse: [field printSQLOn: aStream withParameters: #()]
! !

!FieldExpression methodsFor:'printing SQL'!

printSQLOn: aStream withParameters: aDictionary
	self field printSQLOn: aStream withParameters:aDictionary.
!

printSelectSQLOn: aStream withParameters: aDictionary
	self printSQLOn: aStream withParameters:aDictionary.
	alias notNil ifTrue: [aStream nextPutAll: ' AS '; nextPutAll: alias].
!

printUnqualifiedSQLOn: aStream withParameters: aDictionary
	self field printUnqualifiedSQLOn: aStream withParameters:aDictionary.
! !

!FieldExpression methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	^true.
! !

!FieldExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/FieldExpression.st,v 1.1 2006/07/26 08:28:52 boris Exp $'
! !
