"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#FieldValueWrapper
	instanceVariableNames:'contents hasValue containedBy'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!FieldValueWrapper class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!FieldValueWrapper class methodsFor:'public'!

new

	^super new initialize.
! !

!FieldValueWrapper methodsFor:'accessing'!

allInnerWrappers
	^#().
!

basicContents
	"Return the un-transformed contents"
	^self contents.
!

innerWrapper
	^self.
! !

!FieldValueWrapper methodsFor:'containing'!

isNowContainedBy: aRow and: aField
	"Optimize this for memory consumption with the common case of one container. If we're contained by nothing, make it a single association. Otherwise, use a dictionary"
	| shortCircuit thisFieldsEntries |
	shortCircuit := false.
	containedBy == nil ifTrue: [containedBy := aField -> ((IdentitySet new: 2) add: aRow; yourself). ^self].
	containedBy class == Association 
		ifTrue: [
		containedBy key == aField
			ifTrue: [thisFieldsEntries := containedBy value]
			ifFalse: [ | newContainedBy |
				newContainedBy := IdentityDictionary new. 
				newContainedBy at: containedBy key put: containedBy value.
				containedBy := newContainedBy]].
	thisFieldsEntries isNil ifTrue: [
		thisFieldsEntries := containedBy at: aField ifAbsentPut: [shortCircuit := true. (IdentitySet new: 2) add: aRow; yourself]].
	shortCircuit ifTrue: [^self].
	(thisFieldsEntries includes: aRow) ifFalse: [thisFieldsEntries add: aRow].
! !

!FieldValueWrapper methodsFor:'public'!

containedBy

	^containedBy.
!

contents

	^contents.
!

contents: anObject

	(hasValue and: [contents ~= anObject]) ifTrue: [self error: 'Inconsistent values in field'].
	self privateSetContents: anObject.
!

hasValue

	^hasValue.
!

initialize

	hasValue := false.
!

printOn: aStream

	aStream 
		nextPutAll: '<<'.
	self hasValue ifTrue: [aStream print: self contents].
	aStream
		nextPutAll: '>>'.
!

privateSetContents: anObject
	contents := anObject.
	hasValue := true.
! !

!FieldValueWrapper methodsFor:'unifying'!

containsWrapper: aFieldValueWrapper
	^self == aFieldValueWrapper.
!

replaceWith: aFieldValueWrapper
	containedBy isNil ifTrue: [^nil].
	containedBy
		keysAndValuesDo:
			[:eachField :eachListOfRows | eachListOfRows do: [:eachRow | eachRow wrapperAt: eachField put: aFieldValueWrapper]].
!

unifyWith: targetWrapper
	(self containsWrapper: targetWrapper) ifTrue: [^self].
	(targetWrapper containsWrapper: self) ifTrue: [^self].
	targetWrapper hasValue ifFalse: [^targetWrapper replaceWith: self].
	self hasValue ifFalse: [^self replaceWith: targetWrapper].
	self contents = targetWrapper contents
		ifFalse: [self error: 'Conflicting values in rows'].
	targetWrapper replaceWith: self.
! !

!FieldValueWrapper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/FieldValueWrapper.st,v 1.1 2006/07/26 08:05:48 boris Exp $'
! !
