"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#ForeignKeyConstraint
	instanceVariableNames:'name suffixExpression sourceFields targetFields
		shouldCreateInDatabase'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!ForeignKeyConstraint class methodsFor:'instance creation'!

new
	^super new initialize.
!

sourceField: aDatabaseField targetField: anotherDatabaseField
	^self
		sourceField: aDatabaseField
		targetField: anotherDatabaseField
		suffixExpression: nil.
!

sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpression
	^(self new)
		sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpression;
		yourself.
!

sourceFields: sourceFields targetFields: targetFields
	^self
		sourceFields: sourceFields
		targetFields: targetFields
		suffixExpression: nil.
!

sourceFields: sourceFields targetFields: targetFields suffixExpression: suffixExpression
	^(self new)
		sourceFields: sourceFields targetFields: targetFields suffixExpression: suffixExpression;
		yourself.
! !

!ForeignKeyConstraint class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ForeignKeyConstraint methodsFor:'accessing'!

addSourceField: aField
	(sourceFields allSatisfy: [:each | aField table = each table]) ifFalse: [self error: 'Fields in foreign key constraints must all have the same source table'].
	sourceFields add: aField.
!

addTargetField: aField
	(targetFields allSatisfy: [:each | aField table = each table]) ifFalse: [self error: 'Fields in foreign key constraints must all have the same target table'].
	targetFields add: aField.
!

name
	name isNil ifTrue: [name := self generateName].
	^name.
!

name: aString
	name := aString.
!

shouldCreateInDatabase
	^shouldCreateInDatabase
!

shouldCreateInDatabase: aBoolean
	"If false, then this is a constraint that we use for in-memory ordering, but it should not be created in the database, possibly for performance, or because it might cause a cycle"
	shouldCreateInDatabase := aBoolean
!

someSourceField
	"Return any one of our source fields"
	^sourceFields first.
!

sourceField: aField
	sourceFields isEmpty ifFalse: [self error: 'Adding a field to a non-empty constraint'].
	sourceFields add: aField.
!

sourceFields
	^sourceFields
!

sourceFields: aCollection
	sourceFields := aCollection.
!

sourceTable
	"All the source fields have to come from the same table, so just pick one"
	^sourceFields first table.
!

suffixExpression
	^suffixExpression
!

suffixExpression: anObject
	suffixExpression := anObject.
!

table
	^self sourceTable.
!

targetField: aField
	targetFields isEmpty ifFalse: [self error: 'Adding a field to a non-empty constraint'].
	targetFields add: aField.
!

targetFields
	^targetFields
!

targetFields: aCollection
	targetFields := aCollection.
!

targetTable
	"All the target fields have to come from the same table, so just pick one"
	^targetFields first table.
! !

!ForeignKeyConstraint methodsFor:'converting'!

asJoin
	^Join fromAll: sourceFields toAll: targetFields.
!

asReverseJoin
	^Join fromAll: targetFields toAll: sourceFields.
! !

!ForeignKeyConstraint methodsFor:'initializing'!

initialize
	sourceFields := OrderedCollection new: 2.
	targetFields := OrderedCollection new: 2.
	shouldCreateInDatabase := true.
!

sourceField: aDatabaseField targetField: anotherDatabaseField
	self
		sourceField: aDatabaseField
		targetField: anotherDatabaseField
		suffixExpression: nil.
!

sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpressionString
	self
		sourceField: aDatabaseField;
		targetField: anotherDatabaseField;
		suffixExpression: suffixExpressionString.
!

sourceFields: sourceFieldCollection targetFields: targetFieldCollection suffixExpression: suffixExpressionString
	self
		sourceFields: sourceFieldCollection;
		targetFields: targetFieldCollection;
		suffixExpression: suffixExpressionString.
! !

!ForeignKeyConstraint methodsFor:'printing'!

creationString
	| ws |
	shouldCreateInDatabase ifFalse: [^''].
	ws := WriteStream on: (String new: 50).
	ws
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: self name;
		nextPutAll: ' FOREIGN KEY ('.
	GlorpHelper print: [:each | each name] on: ws for: self sourceFields separatedBy: ','.
	ws 	nextPutAll: ') REFERENCES ';
		nextPutAll: self targetTable name;
		nextPutAll: ' ('.
	GlorpHelper print: [:each | each name] on: ws for: self targetFields separatedBy: ','.
	ws nextPutAll: ')'.
	self suffixExpression isNil
		ifFalse:
			[ws
				space;
				nextPutAll: self suffixExpression].
	^ws contents.
!

dropString
	| ws |
	shouldCreateInDatabase ifFalse: [^''].
	ws := WriteStream on: (String new: 50).
	^ws
		nextPutAll: 'ALTER TABLE ';
		nextPutAll: self table sqlString;
		nextPutAll: ' DROP CONSTRAINT ';
		nextPutAll: self name;
		contents.
!

generateName
	| stream maxLength tableLength roomToBeVerbose shortTableLength longTableLength |
	sourceFields isEmpty ifTrue: [^'INVALID CONSTRAINT'].
	"Figure out if we have room for verbose separators or if we should maximize the space for the table/field names"
	maxLength := sourceFields first table platform maximumLengthOfTableName.
	shortTableLength := ((maxLength - 8) // 2) min: 10.
	longTableLength := (maxLength // 2) - 1.
	roomToBeVerbose := shortTableLength >= 10.
	tableLength := roomToBeVerbose ifTrue: [shortTableLength] ifFalse: [longTableLength].

	stream := WriteStream on: (String new: 100).
	sourceFields first printForConstraintNameOn: stream maxLength: tableLength.
	roomToBeVerbose ifTrue: [stream nextPutAll: '_TO_'].
	targetFields first printForConstraintNameOn: stream maxLength: tableLength.
	stream nextPutAll: (roomToBeVerbose ifTrue: ['_REF'] ifFalse: ['R']).
	^stream contents.
!

printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self name;
		nextPut: $).
! !

!ForeignKeyConstraint methodsFor:'relationships'!

rowsRelatedTo: aRow
	| myWrapper relatedRows sourceField targetField allWrappers |
	"Note that we don't care which field, as presumably all of them must point the same place for a composite key, so grab the first pair"
	sourceField := sourceFields first.
	targetField := targetFields first.
	myWrapper := aRow wrapperAt: sourceField ifAbsent: [nil].
	myWrapper isNil ifTrue: [^#()].
	relatedRows := Set new: 5.
	allWrappers := OrderedCollection with: myWrapper.
	allWrappers addAll: myWrapper allInnerWrappers.
	allWrappers
		do: [:eachWrapper |
			(eachWrapper containedBy glorpAtIdentity: targetField ifAbsent: [#()])
				do: [:eachRow | eachRow == aRow ifFalse: [relatedRows add: eachRow]]].
	^relatedRows.
! !

!ForeignKeyConstraint methodsFor:'testing'!

isImaginary
	^self targetTable isImaginary or: [self sourceTable isImaginary].
!

refersToTable: aTable
	^self targetTable == aTable.
! !

!ForeignKeyConstraint class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ForeignKeyConstraint.st,v 1.1 2006/07/26 08:04:48 boris Exp $'
! !
