"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpExpression
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!GlorpExpression class methodsFor:'instance creation'!

new

	^super new initialize.
! !

!GlorpExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpExpression methodsFor:'accessing'!

beOuterJoin
	^self.
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^self subclassResponsibility.
!

field
	^nil.
!

hasDescriptor

	^false.
!

printsTable

	^false.
!

type
	^self field type.
!

valueIn: aDictionary
	"Return the value associated with this expression given the parameters in aDictionary. Only meaningful for ParameterExpressions"

	^self.
! !

!GlorpExpression methodsFor:'api'!

, anExpression
	^self get: #, withArguments: (Array with: anExpression asGlorpExpression).
!

AND: anExpression
	"This method doesn't really have to exist, because it would be inferred using operationFor:, but it's included here for efficiency and to make it a little less confusing how relation expression get created. Note that the two expression must already be  built on the same base!!"

	anExpression isNil ifTrue: [^self].
	anExpression == true ifTrue: [^self].
	^RelationExpression named: #AND basedOn: self withArguments: (Array with: anExpression).
!

AND: anExpression AND: otherExpression
	"A convenience method to allow easy conjunction of multiple ANDs at the same level"
	^(self AND: anExpression) AND: otherExpression.
!

AND: anExpression AND: otherExpression AND: thirdExpression
	"A convenience method to allow easy conjunction of multiple ANDs at the same level"
	^((self AND: anExpression) AND: otherExpression) AND: thirdExpression.
!

AND: anExpression AND: otherExpression AND: thirdExpression AND: fourthExpression
	"A convenience method to allow easy conjunction of multiple ANDs at the same level"
	^(((self AND: anExpression) AND: otherExpression) AND: thirdExpression) AND: fourthExpression.
!

OR: anExpression
	"This method doesn't really have to exist, because it would be inferred using operationFor:, but it's included here for efficiency and to make it a little less confusing how relation expression get created.  Note that the two expression must already be  built on the same base!!"
	anExpression isNil ifTrue: [^self].
	^RelationExpression named: #OR basedOn: self withArguments: (Array with: anExpression).
!

asGlorpExpression

	^self.
!

base

	self subclassResponsibility.
!

equals: anExpression

	^RelationExpression named: #= basedOn: self withArguments: (Array with: anExpression).
!

get: aSymbol withArguments: anArray

	self subclassResponsibility.
!

getFunction: aSymbol withArguments: anArray

	| expression |
	expression := FunctionExpression for: aSymbol withArguments: anArray basedOn: self.
	expression isNil ifTrue: [^nil].
	^expression.
!

parameter: aConstantExpression
	"Create a parameter expression with the given name. But note that the name doesn't have to be a string. Database fields, symbols, and integers are all plausible"
	| value |
	value := aConstantExpression class == ParameterExpression ifTrue: [aConstantExpression field] ifFalse: [aConstantExpression value].
	^ParameterExpression forField: value basedOn: self.
! !

!GlorpExpression methodsFor:'converting'!

asDistinctFieldOn: anExpression
	^(self asGlorpExpressionOn: anExpression) get: #distinct withArguments: #().
!

asField
	"If we correspond to a single field, return it"
	^self field.
!

asGeneralGlorpExpression
	"Convert the result to a general (tree-format) expression, if it's the more limited join expression"
	^self.
!

asGeneralGlorpExpressionWithFields
	"Convert the result to a general (tree-format) expression, if it's the more limited join expression."
	^self.
!

asGlorpExpressionForDescriptor: aDescriptor

	self ultimateBaseExpression descriptor: aDescriptor.
!

asGlorpExpressionOn: aBaseExpression
	aBaseExpression ultimateBaseExpression == self ultimateBaseExpression
		ifTrue: [^self].
	^self rebuildOn: aBaseExpression.
!

asGlorpExpressionOn: aBaseExpression basedOn: oldBaseExpression withUltimateBase: ultimateBaseExpression
	^self
		rebuildOn: aBaseExpression
		startingFrom: oldBaseExpression
		withOuterScopeBase: ultimateBaseExpression.
!

asGlorpExpressionOn: aBaseExpression in: aQuery

	^self asGlorpExpressionOn: aBaseExpression.
!

asGlorpExpressionOn: anExpression withUltimateBase: aBaseExpression
	"Build the expression, knowing that some parts of it may refer to an outer context based on aBaseExpression. This is primarily useful for subselect blocks"
	^self rebuildOn: anExpression startingFrom: self queryLevelBaseExpression withOuterScopeBase: aBaseExpression.
!

asIndependentJoins
	"If this is an ANDed clause, split it into independent joins"
	^Array with: self.
!

asJoin
	self subclassResponsibility.
!

asNonDistinctField
	^self
!

generalExpressionPart
	"If we have a mixture of joins and general expressions, return only the general part"
	^self.
!

inverseJoin
	"We assume that only the parts that are actually joins ought to invert"
	| new |
	new := self asGlorpExpressionOn: BaseExpression new.
	new do: [:each |
		each replaceJoinsWithInverse].
	^new.
!

rebuildOn: aBaseExpression
	^self rebuildOn: aBaseExpression startingFrom: nil.
!

sourceForTarget: aField

	^nil.
! !

!GlorpExpression methodsFor:'debugging'!

sqlString
	| command |
	command := QuerySelectCommand new
		useBinding: false;
		platform: self ultimateBaseExpression descriptor session platform.
	self printSQLOn: command withParameters: #().
	^command contents.
! !

!GlorpExpression methodsFor:'initialize'!

in: anExpression

	^RelationExpression named: #IN basedOn: self withArguments: (Array with: anExpression).
!

initialize
! !

!GlorpExpression methodsFor:'inspecting'!

inspectorHierarchies
	| hierarchy |
	hierarchy := ((Smalltalk at: #Tools ifAbsent: [^#()])
		at: #Trippy ifAbsent: [^#()])
		at: #Hierarchy ifAbsent: [^#()].
	^Array with: (hierarchy
			id: #expression
			label: 'Expression Tree'
			parentBlock: [:each | nil]
			childrenBlock: [:each | each inspectorChildren])
! !

!GlorpExpression methodsFor:'iterating'!

allSatisfy: aBlock
	"Answer true if aBlock answers true for any element of the receiver.
	 An empty collection answers false."

	self do: [:each| (aBlock value: each) ifFalse: [^false]].
	^true
!

anySatisfy: aBlock
	"Answer true if aBlock answers true for any element of the receiver.
	 An empty collection answers false."

	self do: [:each| (aBlock value: each) ifTrue: [^true]].
	^false
!

collect: aBlock

	| newCollection |
	newCollection := OrderedCollection new.
	self do: [:each | newCollection add: (aBlock value: each)].
	^newCollection.
!

detect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	^self detect: aBlock ifNone: [self notFoundError]
!

detect: aBlock ifNone: exceptionBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	self do: [:each | (aBlock value: each) ifTrue: [^each]].
	^exceptionBlock value
!

do: aBlock
	"Iterate over the expression tree"

	self do: aBlock skipping: Dialect identitySetClass new.
!

do: aBlock skipping: aSet
	"Iterate over the expression tree. Keep track of who has already been visited, so we don't get trapped in cycles or visit nodes twice."

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	aBlock value: self.
!

inject: anObject into: aBlock

	| sum |
	sum := anObject.
	self do: [:each | sum := aBlock value: sum value: each].
	^sum
!

select: aBlock

	| newCollection |
	newCollection := OrderedCollection new.
	self do: [:each | (aBlock value: each) ifTrue: [newCollection add: each]].
	^newCollection
! !

!GlorpExpression methodsFor:'navigating'!

queryLevelBaseExpression
	^self ultimateBaseExpression
!

ultimateBaseExpression
	self subclassResponsibility.
! !

!GlorpExpression methodsFor:'preparing'!

addBoundExpressionsIn: aQuery to: boundExpressions
!

additionalExpressions

	^#().
!

additionalExpressionsIn: aQuery 
	"Return the collection of additional expressions (representing joins) that this expression tree requires"

	| allExpressions |
	allExpressions := ExpressionGroup with: self.
	allExpressions addAll: aQuery ordering.
	allExpressions addAll: aQuery tracing additionalExpressions.
	^allExpressions
		inject: OrderedCollection new 
		into: [:sum :each | 
			each queryLevelBaseExpression == aQuery baseExpression ifTrue: [
				sum addAll: each additionalExpressions].
			sum].
!

allRelationsFor: rootExpression do: aBlock andBetweenDo: anotherBlock
	"In any normal relationship, there's only one thing. Just do it"
	aBlock value: rootExpression leftChild value: rootExpression rightChild.
!

allTables

	^self inject: Set new into: [:sum :each | 
		sum addAll: each tables. sum].
!

allTablesToPrint

	^self inject: Set new into: [:sum :each | 
		sum addAll: each tablesToPrint. sum].
!

allTablesToPrintStoppingAt: aSubSelectBaseExpression

	^self inject: Set new into: [:sum :each | 
		each queryLevelBaseExpression == aSubSelectBaseExpression ifTrue: [
			sum addAll: each tablesToPrint]. sum].
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	self subclassResponsibility.
!

assignTableAliasesStartingAt: anInteger prefix: aString
	^anInteger.
!

bindingType
	^self field type.
!

condensePrimaryKeyComparison
	^self.
!

condensePrimaryKeyComparisons
	^self.
!

hasBindableExpressionsIn: aCommand
	^false.
!

prepareIn: aQuery
	"A little weird, because what we end up asking for additional expressions from, might not be us"

	aQuery adjustWhereClause.
	(aQuery whereClause additionalExpressionsIn: aQuery)
		do:
			[:each | 
			(aQuery shouldExpressionBeJoin: each)
				ifTrue: [aQuery addJoin: each]
				ifFalse: [aQuery privateAnd: each]].
!

rebuildOn: aBaseExpression startingFrom: anExpression
	"Rebuild the expression on the new base. If we encounter anExpression, treat that as the base, and build only from that point up. e.g. if we have a.b.c.d and we rebuildOn: z startingFrom: b, then the result will be z.c.d"
	^self rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: nil.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	"Rebuild the expression on the new base. If we encounter anExpression, treat that as the base, and build only from that point up. e.g. if we have a.b.c.d and we rebuildOn: z startingFrom: b, then the result will be z.c.d. Pieces that aren't based on anExpression should be built onto ultimateBaseExpression instead of aBaseExpression. They represents parts of outer scope trees that are referenced within us.
  Note that anExpression can be nil, but if it is, then ultimateBaseExpression is effectively ignored."
	self subclassResponsibility.
!

replaceJoinsWithInverse
	"Replace any actual Join objects we have with their inverse. Only useful if we have something that has mixed joins and general expressions, where we assume we don't want to mess with the general ones, because they're not field-level"
	^self.
!

rewriteEquality
	^self.
!

rewriteEqualityExpressions
	^self.
!

tableForANSIJoin
	"Which table will we join to."
	^nil.
!

tables

	^#().
!

tablesToPrint

	^#().
!

validate
! !

!GlorpExpression methodsFor:'primary keys'!

primaryKeyFromDictionary: aDictionary
	"Given a set of parameters, return a primary key suitable for retrieving our target. We can't do this for general expressions, so indicate failure by returning nil"
	^nil.
! !

!GlorpExpression methodsFor:'printing'!

className

	^self class name.
!

displayString

	| stream |
	stream := String new writeStream.
	self printOnlySelfOn: stream.
	^stream contents.
!

printOn: aStream 
	self printTreeOn: aStream
!

printOnlySelfOn: aStream
	self subclassResponsibility.
!

printTreeOn: aStream

	self subclassResponsibility.
! !

!GlorpExpression methodsFor:'private/accessing'!

requiresDistinct
	^false.
! !

!GlorpExpression methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	^false.
!

canBind
	"Return true if this represents a value that can be bound into a prepared statement"
	^false.
!

canKnit
	"Return true if, when building objects, we can knit the object corresponding to this expression to a related object. Roughly speaking, is this a mapping expression"
	^false.
!

canTranslateFields
	"Return true if we are a mapping expression and can thus translate from fields into their aliased version for a particular query."
	^false.
!

hasImpliedClauses
	"Return true if this implies additional SQL clauses beyond just a single field expression"
	^false.
!

hasProvidedAdditionalExpressions
	"Return true if we have already provided additional expressions to some query. Only important when we're preparing a subselect and don't want to bring in joins that are part of the main query"
	^true.
!

hasTableAliases
	^false.
!

isConstantExpression
	^false.
!

isDistinct
	^false.
!

isEmptyExpression
	^false.
!

isGlorpExpression
	^true.
!

isJoin
	^false.
!

isMappable
	"Return true if we're something that can be returned in a select statement."
	^true.
!

representsDerivedObject
	^false.
!

returnsDictionary
	"Return true if we representa  ampping to a dictionary"
	^false.
! !

!GlorpExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/GlorpExpression.st,v 1.1 2006/07/26 08:29:15 boris Exp $'
! !
