"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Mapping subclass:#AdHocMapping
	instanceVariableNames:'fromDbMappingBlock toDbMappingBlock mappedFields shouldProxy'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

AdHocMapping comment:'AdHocMapping is a configurable sort of mapping, done via two blocks.

The protocol for this is still ugly because the users will have to explicitly make use of the field positions, and probably need to use the elementBuilder''s translation. This should be automated.

'
!


!AdHocMapping class methodsFor:'instance creation'!

forAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection 
	^super new 
		setAttribute: aSymbol
		fromDb: fromBlock
		toDb: toBlock
		mappingFields: aFieldCollection
! !

!AdHocMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!AdHocMapping methodsFor:'accessing'!

shouldProxy

	^shouldProxy.
!

shouldProxy: aBoolean

	shouldProxy := aBoolean.
! !

!AdHocMapping methodsFor:'initialize/release'!

forAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection
	self
		setAttribute: aSymbol
		fromDb: fromBlock
		toDb: toBlock
		mappingFields: aFieldCollection.
!

initialize

	super initialize.
	self shouldProxy: false.
!

setAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection
 
	self attributeName: aSymbol.
	fromDbMappingBlock := fromBlock.
	toDbMappingBlock := toBlock.
	mappedFields := aFieldCollection.
! !

!AdHocMapping methodsFor:'mapping'!

field
	| fields |
	fields := self mappedFields.
	fields size = 1 ifFalse: [self error: 'This mapping does not correspond to exactly one field'].
	^fields first.
!

mappedFields

	^mappedFields.
!

referencedIndependentObjectsFrom: anObject
	"Assumes that the only object this might affect is our primary attribute. That's probably valid. I think."
	| object otherDescriptor |
	object := self getValueFrom: anObject.
	otherDescriptor := self system descriptorFor: object.
	^otherDescriptor isNil ifTrue: [#()] ifFalse: [Array with: object].
!

trace: aTracing context: anExpression
	^self.
!

valueIn: anElementBuilder as: anExpression
	^fromDbMappingBlock
		value: anElementBuilder row
		value: anElementBuilder
		value: anExpression.
! !

!AdHocMapping methodsFor:'printing SQL'!

allRelationsFor: rootExpression
	"We may have multiple relationships."
	| tables rows result rightChild rightHandSide |
	tables := self mappedTables.
	rows := Dictionary new.
	tables do: [:each | rows at: each put: (DatabaseRow newForTable: each)].
	rightChild := rootExpression rightChild.
	rightHandSide := rightChild isConstantExpression ifTrue: [rightChild value] ifFalse: [rightChild].
	toDbMappingBlock value: rows value: rightHandSide value: Dictionary new. 
	result := nil.
	rows do: [:eachRow |
		| table |
		table := rootExpression leftChild base getTable: eachRow table.
		eachRow fieldsAndValidValuesDo: [:eachField :eachValue |  | newExp |
			newExp := (table getField: eachField) get: rootExpression relation withArguments: (Array with: eachValue).
			result := newExp AND: result.]].
	^result.
!

convertedDbValueOf: anObject
	| tables rows |
	tables := self mappedTables.
	rows := Dictionary new.
	tables do: [:each | rows at: each put: (DatabaseRow newForTable: each)].
	toDbMappingBlock value: rows value: anObject.
	rows keysAndValuesDo: [:eachTable :eachRow |
		^eachRow at: mappedFields first].
! !

!AdHocMapping methodsFor:'public'!

mapFromObject: anObject intoRowsIn: aRowMap

	| target sourceRows targetRows valueDescriptor |
	canWrite ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target.
	target := aRowMap mementoFor: target.
	sourceRows := Dictionary new: 3.
	targetRows := Dictionary new: 3.
	descriptor tables do: [:each |
		sourceRows at: each put: (aRowMap findOrAddRowForTable: each withKey: anObject)].
	valueDescriptor := self system descriptorFor: target.
	valueDescriptor isNil ifFalse: [
		valueDescriptor tables do: [:each |
			targetRows at: each put: (aRowMap findOrAddRowForTable: each withKey: target)]].
	toDbMappingBlock value: sourceRows value: target value: targetRows.
!

mapObject: anObject inElementBuilder: anElementBuilder

	| readBlock row |
	row := anElementBuilder row.
	readBlock := [fromDbMappingBlock value: row value: anElementBuilder value: BaseExpression new].
	self shouldProxy 
		ifTrue: [self setValueIn: anObject to: (AdHocProxy readBlock: readBlock)]
		ifFalse: [self setValueIn: anObject to: readBlock value]
! !

!AdHocMapping methodsFor:'testing'!

controlsTables
	^false
!

hasImpliedClauses
	"We may imply more than one clause, or a clause which is different from the one directly implied by the relationship"
	^true
!

isRelationship
	"True when the mapping associates different persistent classes."

	^false
!

isStoredInSameTable
	^true.
! !

!AdHocMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__AdHocMapping.st,v 1.1 2006/07/25 16:57:00 boris Exp $'
! !
