"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Query subclass:#CompoundQuery
	instanceVariableNames:'queries operation primaryQuery'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpQueries'
!


!CompoundQuery class methodsFor:'instance creation'!

newIntersect
	^self new operation: #INTERSECT.
!

newMinus
	^self new operation: #EXCEPT.
!

newUnionAll
	^self new operation: #'UNION ALL'.
! !

!CompoundQuery methodsFor:'As yet unclassified'!

resultClass
	^primaryQuery resultClass.
!

shortCircuitEmptyReturn
	^false.
! !

!CompoundQuery methodsFor:'accessing'!

addQuery: aQuery
	queries isEmpty ifTrue: [primaryQuery := aQuery].
	queries add: aQuery.
!

joins
	^queries inject: #() into: [:sum :each | sum, each joins].
!

mightHaveDuplicateRows
	^primaryQuery mightHaveDuplicateRows.
!

operation: aString
	operation := aString.
!

primaryQuery: aQuery
	primaryQuery := aQuery.
!

proxyType
	^primaryQuery proxyType.
!

queries
	^queries
!

queries: aCollectionOfQueries
	queries := aCollectionOfQueries
!

readsOneObject
	^false.
!

tracing
	^primaryQuery tracing.
!

whereClause
	| allCriteria |
	allCriteria := queries inject: #() into: [:sum :each | sum, each whereClause].
	^ExpressionGroup new addAll: allCriteria.
! !

!CompoundQuery methodsFor:'boolean operations'!

except: aQuery
	^self minus: aQuery.
!

minus: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	^CompoundQuery newMinus addQuery: self; addQuery: aQuery.
!

unionAll: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	^self isUnionAll
		ifTrue:
			[queries add: aQuery.
			self]
		ifFalse:
			[(CompoundQuery newUnionAll)
				addQuery: self;
				addQuery: aQuery].
! !

!CompoundQuery methodsFor:'copying'!

postCopy

	super postCopy.
	queries := queries collect: [:each | each copy].
	primaryQuery := queries first.
! !

!CompoundQuery methodsFor:'executing'!

executeWithParameters: parameterArray in: aSession
	session := aSession.
	^self 
		readFromDatabaseWithParameters: parameterArray.
!

readFromDatabaseWithParameters: anArray
	| answerStream |
	answerStream := self rowsFromDatabaseWithParameters: anArray.
	^self resultCollectionFor: answerStream.
!

rowsFromDatabaseWithParameters: anArray
	^session accessor
		executeCommand: (self sqlWith: anArray)
		returnCursor: true.
!

sqlWith: aDictionary

	self prepare.
	session platform queryWithUnsupportedOperationsEliminatedFrom: self do: [:newQuery | 
		^newQuery
			prepare;
			sqlWith: aDictionary].
	^CompoundSelectCommand
		forQuery: self
		parameters: aDictionary
		useBinding: session useBinding
		platform: session platform.
! !

!CompoundQuery methodsFor:'initializing'!

initialize
	super initialize.
	queries := OrderedCollection new: 3.
! !

!CompoundQuery methodsFor:'preparing'!

copyOfBuilders 
	^primaryQuery copyOfBuilders.
!

prepare
	| |
	queries do: [:each | each session: session].
	queries do: [:each | each prepare].
	ordering isNil ifFalse: [
		ordering := ordering collect: [:each |
			(self expressionBlockFor: each) asGlorpExpressionOn: queries first baseExpression]].
! !

!CompoundQuery methodsFor:'printing'!

printCompoundSelectAliasesOn: aStream
	"We only want one table. This is a somewhat convoluted way of getting 't1' uniformly appended"
	aStream nextPutAll: ' '.
	self session platform supportsAliasInCompoundQuery ifTrue: [
		primaryQuery tablesToPrint first printSQLOn: aStream withParameters: nil].
!

printOperationOn: aCommand
	aCommand space.
	aCommand nextPutAll: (self platform compoundOperationFor: operation).
	aCommand space.
!

sqlString
	"Mostly for debugging"
	^(self sqlWith: #()) sqlString.
!

tablesToPrint
	^self queries first tablesToPrint.
! !

!CompoundQuery methodsFor:'sql generation'!

rewriteExcept
	"Rewrite any EXCEPT operation we have into a NOT EXISTS test with a correlated subselect. Only a CompoundQuery would have an EXCEPT"
	
	| |
	operation == #EXCEPT ifFalse: [^self].
	queries := queries collect: [:each | each rewriteExcept].
	^self rewriteSetOperationUsing: #notExists:
!

rewriteIntersect
	"Rewrite any INTERSECT operation we have into an EXISTS test for a correlated subselect. Only a CompoundQuery would have an intersect"
	
	| |
	operation == #INTERSECT ifFalse: [^self].
	queries := queries collect: [:each | each rewriteIntersect].
	^self rewriteSetOperationUsing: #exists:
!

rewriteSetOperationUsing: aSelector
	"Rewrite any INTERSECT OR EXCEPT operation we have into a correlated EXISTS or NOT EXISTS test, respectively. We take the sub-query, and AND into it a constraint that its base expression is equal to the first queries base expression, and then do either an exists: or a notExists:, depending on aSelector."
	
	| newQuery |
	newQuery := queries first copy.
	2 to: queries size do: [:i | | q expression mainExpression |
		q := (queries at: i) copy.
		expression := RelationExpression new outerJoin: false; relation: #=; leftChild: newQuery baseExpression; rightChild: q baseExpression.
		mainExpression := RelationExpression new outerJoin: false; relation: #AND; leftChild: q whereClause; rightChild: expression.
		q whereClause: mainExpression.
		newQuery AND: [:each | each perform: aSelector with: q]].
	^newQuery.
! !

!CompoundQuery methodsFor:'testing'!

isMinus
	^operation = 'EXCEPT'.
!

isUnionAll
	^operation = 'UNION ALL'.
! !

!CompoundQuery class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__CompoundQuery.st,v 1.1 2006/07/26 08:51:01 boris Exp $'
! !
