"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Dialect
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Support'
!

Dialect class instanceVariableNames:'dialectName timestampClass identitySetClass lookedUpNames
	isNamespaceAware'

"
 No other class instance variables are inherited by this class.
"
!


!Dialect class methodsFor:'LICENSE'!

LICENSE

	^ 'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Dialect class methodsFor:'binding'!

unbindableClassNames

	self isVisualWorks ifTrue: [^ #()].
	^ #()
! !

!Dialect class methodsFor:'collections'!

needsCollectionMementoFor: aCollection
	^true.
! !

!Dialect class methodsFor:'dates'!

addSeconds: seconds to: aTime

        self isVisualWorks ifTrue: [^ Time fromSeconds: aTime asSeconds + seconds].
        self isDolphin ifTrue: [^ self addTimeForDolphin: aTime seconds: seconds].
        self isGNU ifTrue: [^ aTime addSeconds: seconds].
        self isVisualAge ifTrue: [^Time fromSeconds: aTime asSeconds + seconds].
        self isSmalltalkX ifTrue: [^Time fromSeconds: aTime asSeconds + seconds].
        self error: 'not implemented'.
        ^ self
!

addTimeForDolphin: aTime seconds: seconds

	"Dolphin's time/date arithmetic is pretty weak, especially for timestamps. Hack around it. This is likely only to work for seconds <24 hours"

	| result |

	^ aTime class == Time
		ifTrue: [Time fromMilliseconds: (aTime asMilliseconds + (seconds * 1000)) \\ 86400000.]
		ifFalse: [
			result := self timestampClass date: aTime date
				time: (self addTimeForDolphin: aTime time seconds: seconds).
			(seconds > 0 and: [result time < aTime time.])
				ifTrue: [result date: (result date addDays: 1).].
			(seconds < 0 and: [result time > aTime time.])
				ifTrue: [result date: (result date addDays: -1).].
			^ result
		]
!

newDateWithYears: years months: months days: days

        (self isVisualWorks or: [self isSqueak]) ifTrue: [^ Date newDay: days monthNumber: months year: years].
        self isGNU ifTrue: [^ Date newDay: days monthIndex: months year: years].
        self isVisualAge ifTrue: [^Date newDay: days monthIndex: months year: years].
        self isSmalltalkX ifTrue: [^Date newDay: days monthIndex: months year: years].
        self isObjectStudio ifTrue: [^Date newDay: days imonth: months year: years].
        self error: 'not implemented'.
        ^ self
!

newTimeWithHours: hours minutes: minutes seconds: seconds milliseconds: milliseconds

        self isGNU ifTrue: [^ Time fromSeconds: hours * 60 * 60 + (minutes * 60) + seconds].
        (self isVisualWorks or: [self isSqueak]) ifTrue: [^ Time fromSeconds: hours * 60 * 60 + (minutes * 60) + seconds].
        self isVisualAge
                ifTrue: [
                        ^ Time fromMilliseconds: ((hours * 60 * 60) + (minutes * 60) + seconds) * 1000 + milliseconds
                ].
        self isObjectStudio ifTrue: [^Time hour: hours minute: minutes second: seconds millisecond: milliseconds].
        self isSmalltalkX ifTrue: [^ Time fromSeconds: hours * 60 * 60 + (minutes * 60) + seconds].
        self error: 'Not implemented yet'.
        ^ self
!

newTimestampWithYears: years
        months: months
        days: days
        hours: hours
        minutes: minutes
        seconds: seconds
        milliseconds: millis
        offset: offset

        | date time ts |

        self isGNU
                ifTrue: [
                        ^ self timestampClass year: years
                                month: months
                                day: days
                                hour: hours
                                minute: minutes
                                second: seconds
                                offset: ((Dialect smalltalkAt: #Duration) fromSeconds: offset)
                ].
        date := self newDateWithYears: years months: months days: days.
        time := self newTimeWithHours: hours minutes: minutes seconds: seconds milliseconds: millis.
        self isVisualWorks  | self isSmalltalkX
                ifTrue: [^ (self timestampClass fromDate: date andTime: time) addMilliseconds: millis].
        self isSqueak
                ifTrue: [ts := self timestampClass date: date time: time.
                        ts time addSeconds: (millis / 1000) asInteger.
                                ^ts]. 
        self isVisualAge
                ifTrue: [
                        ^ ((Dialect smalltalkAt: #AbtTimestamp) date: date time: time)
                                milliSeconds: millis;
                                yourself
                ].
        self isObjectStudio ifTrue: [^self timestampClass newDate: date time: time].
        self error: 'not implemented'.
        ^ self
!

readFixedPointFrom: aString

	self isVisualWorks ifTrue: [^ (self smalltalkAt: #FixedPoint) readFrom: (ReadStream on: aString)].
	(self isDolphin or: [self isSqueak]) ifTrue: [^ Number readFrom: (ReadStream on: aString , 's')].
	self isGNU
		ifTrue: [
			^ (Number readFrom: (ReadStream on: aString)) asScaledDecimal:
				aString size - (aString indexOf: $. ifAbsent: [aString size.])
		].
	self isVisualAge ifTrue: [ ^(self smalltalkAt: #Decimal) fromString: aString].
	self isObjectStudio ifTrue: [^(self smalltalkAt: #Decimal) newString: aString].
	self error: 'not implemented'.
	^ self
!

supportsMillisecondsInTimeStamps

        self isGNU ifTrue: [^ false].
        self isVisualWorks ifTrue: [^ true].
        self isDolphin ifTrue: [^ true].
        self isVisualAge ifTrue: [^ true].
        self isObjectStudio ifTrue: [^true].
        self isSmalltalkX ifTrue: [^true].
        self error: 'not yet implemented'.
        ^ self
!

supportsMillisecondsInTimes

        self isGNU ifTrue: [^ false].
        self isVisualWorks ifTrue: [^ false].
        self isSqueak ifTrue: [^false].
        self isSmalltalkX ifTrue: [^ false].
        self isDolphin ifTrue: [^ true].
        self isVisualAge ifTrue: [^ true].
        self isObjectStudio ifTrue: [^true]. "Sort of. Supports hundredths of a second"
        ^ self
!

timeOffsetFromGMT

	self isGNU ifTrue: [Time timezoneBias / (60 * 60).].
	self isVisualWorks ifTrue: [^ (self smalltalkAt: #TimeZone) default secondsFromGMT / (60 * 60)].
	^ 0
!

timestampClass

        timestampClass == nil ifFalse: [^ timestampClass].
        Dialect isGNU ifTrue: [^ timestampClass := self smalltalkAt: #DateTime].
        (Dialect isSqueak or: [Dialect isDolphin.])
                ifTrue: [^ timestampClass := self smalltalkAt: #TimeStamp].
        Dialect isVisualWorks ifTrue: [^ timestampClass := self smalltalkAt: #Timestamp].
        Dialect isVisualAge ifTrue: [^ timestampClass := self smalltalkAt: #Timestamp].
        Dialect isObjectStudio ifTrue: [^timestampClass := self smalltalkAt: #Timestamp].
        Dialect isSmalltalkX ifTrue: [^(timestampClass := self smalltalkAt: #Timestamp)].
        self error: 'Not yet implemented'.
        ^ self
!

timestampNow

        Dialect isGNU ifTrue: [^ self timestampClass dateAndTimeNow].
        Dialect isSqueak ifTrue: [^ self timestampClass current].
        Dialect isVisualWorks ifTrue: [^ self timestampClass now].
        Dialect isDolphin ifTrue: [^ self timestampClass current].
        Dialect isVisualAge ifTrue: [^ self timestampClass now].
        Dialect isObjectStudio ifTrue: [^self timestampClass now].
        Dialect isSmalltalkX ifTrue: [^self timestampClass now].
        self error: 'Not yet implemented'.
        ^ self
!

timestampNowUTC

        Dialect isGNU ifTrue: [^ self timestampClass utcDateAndTimeNow].
        Dialect isVisualWorks ifTrue: [^ (self smalltalkAt: #Timestamp) fromSeconds: Time secondClock].
        Dialect isDolphin ifTrue: [self error: 'not supported'.].
        Dialect isSmalltalkX ifTrue: [^ self timestampClass now].  " this is not UTC, but it is a timestamp "
        self error: 'Not yet implemented'.
        ^ self
!

totalSeconds

	self isGNU ifTrue: [^ Time utcSecondClock].
	self isVisualAge ifTrue: [^ (self smalltalkAt: #AbtTimestamp) now totalSeconds].
	^ Time totalSeconds
! !

!Dialect class methodsFor:'files'!

contentsOfFileNamed: aString 
	"Given a text file name, read its contents"

	| stream contents fileClass |
	Dialect isVisualWorks ifTrue: [^aString asFilename contentsOfEntireFile].
	Dialect isSqueak ifTrue: [
		stream := (Dialect smalltalkAt: #CrLfFileStream) oldFileNamed: aString.
		[contents := stream contents] ensure: [stream close].
		^contents].

	fileClass := Dialect isVisualAge 
				ifTrue: [self smalltalkAt: #CfsReadFileStream]
				ifFalse: 
					[Dialect isDolphin ifTrue: [self smalltalkAt: #File] ifFalse: [self halt]].
	stream := fileClass read: aString.
	[contents := stream contents] ensure: [stream close].
	^contents
! !

!Dialect class methodsFor:'forward references'!

glorpConstantExpressionClass
        ^Glorp::ConstantExpression.
! !

!Dialect class methodsFor:'general portability'!

argumentCountFor: aBlock

        Dialect isGNU ifTrue: [^ self error: 'not implemented yet'].
        Dialect isVisualWorks ifTrue: [^ aBlock numArgs].
        Dialect isVisualAge ifTrue: [^ aBlock argumentCount].
        Dialect isDolphin ifTrue: [^ aBlock argumentCount].
        Dialect isSqueak ifTrue: [^self error: 'not implemented yet'].
        Dialect isObjectStudio ifTrue: [^aBlock numArgs].
        Dialect isSmalltalkX ifTrue: [^aBlock argumentCount].
         self error: 'not implemented yet'.
        ^ self
!

error
    "For
        aTestCaseInstance should: aSomeBlock raise: Dialect error
    "
	^Dialect isVisualAge ifTrue: [^Error, (Dialect smalltalkAt: #ExError)] ifFalse: [Error].
!

fitHashIntoSmallInteger: aNumber
	"Truncate the number to fit into smallinteger range. Mostly useful for ObjectStudio where hashes have to be 16 bits or less"
	^Dialect isObjectStudio ifTrue: [aNumber hash] ifFalse: [aNumber].
!

garbageCollect

        Dialect isGNU ifTrue: [^ ObjectMemory globalGarbageCollect].
        Dialect isVisualWorks ifTrue: [^ ObjectMemory quickGC].
        Dialect isVisualAge ifTrue: [^ (self smalltalkAt: #System) globalGarbageCollect].
        Dialect isSqueak ifTrue: [^Smalltalk garbageCollect].
        Dialect isSmalltalkX ifTrue: [^Smalltalk garbageCollect].
        Dialect isObjectStudio ifTrue: [^(self smalltalkAt: #System) garbageCollect].
        self error: 'not implemented yet'.
        ^ self
!

identitySetClass
	identitySetClass == nil
		ifTrue:
			[Dialect isVisualAge
				ifTrue: [identitySetClass := self smalltalkAt: #EsIdentitySet]
				ifFalse: [identitySetClass := self smalltalkAt: #IdentitySet]].
	^identitySetClass.
!

instVarNameFor: aName

	Dialect isGNU ifTrue: [^ aName asSymbol].
	^ aName asString
!

isBlock: anObject
        "Return true if this is a block"
        self isVisualAge ifTrue: [^anObject isKindOf: (self smalltalkAt: #Block)].
        self isSmalltalkX ifTrue: [^anObject isKindOf: (self smalltalkAt: #Block)].
        ^anObject class == [] class.
!

newIdentitySet
	^self identitySetClass new.
!

quitTheImage
	"Quit the entire image"

	self isGNU ifTrue: [^ self error: 'Not implemented yet'.].
	self isVisualWorks ifTrue: [^(self smalltalkAt: #ObjectMemory) quit].
	self isSqueak ifTrue: [^(self smalltalkAt: #SmalltalkImage) current snapshot: false andQuit: true].
	self isDolphin ifTrue: [^(self smalltalkAt: #SessionManager) current quit].
	self isVisualAge ifTrue: [^  self error: 'Not implemented yet'.].
	self isObjectStudio ifTrue: [^ self error: 'Not implemented yet'.].
	self error: 'not implemented yet'.
	^ self
!

smalltalkAssociationAt: aName

	^ self smalltalkAssociationAt: aName ifAbsent: [self error: 'element not found'.]
!

smalltalkAssociationAt: aName ifAbsent: aBlock

	self isVisualWorks ifTrue: [^ aName asQualifiedReference].
	^ Smalltalk associationAt: aName asSymbol ifAbsent: aBlock
!

smalltalkAt: aName

	^ self smalltalkAt: aName ifAbsent: [self error: 'element not found'].
!

smalltalkAt: aName ifAbsent: aBlock
	"We may look these names up a lot, so cache them in a small, local dictionary"
	| cached value |
	cached := self lookedUpNames at: aName ifAbsent: [nil].
	cached isNil ifFalse: [^cached].
	value := self isVisualWorks
				ifTrue: [aName asQualifiedReference valueOrDo: aBlock]
				ifFalse: [Smalltalk at: aName asSymbol ifAbsent: aBlock].
	self lookedUpNames
		at: aName
		put: value.
	^value.
!

tokensBasedOn: aString in: stringToTokenize

        self isGNU ifTrue: [^ stringToTokenize subStrings: aString first].
        self isVisualWorks ifTrue: [^ stringToTokenize tokensBasedOn: aString first].
        self isSqueak ifTrue: [^ stringToTokenize findTokens: aString].
        self isDolphin ifTrue: [^ stringToTokenize subStrings: aString].
        self isVisualAge ifTrue: [^ (stringToTokenize subStrings: aString first)].
        self isObjectStudio ifTrue: [^(stringToTokenize asArrayOfSubstringsDelimiter: aString first)].
        self isSmalltalkX ifTrue: [^(stringToTokenize subStrings: aString first)].
        self error: 'not implemented yet'.
        ^ self
!

weakValueDictionaryClass

        Dialect isVisualWorks ifTrue: [^self smalltalkAt: #EphemeralValueDictionary].
        Dialect isSqueak ifTrue: [^self smalltalkAt: #WeakValueDictionary].
        Dialect isSmalltalkX ifTrue: [^self smalltalkAt: #WeakValueDictionary].
        self error: 'Not yet implemented for this dialect'.
! !

!Dialect class methodsFor:'identifying'!

dialectName

	dialectName isNil ifTrue: [self determineDialect.].
	^ dialectName
!

isDolphin

	^ self dialectName = #Dolphin
!

isGNU

	^ self dialectName = #GNU
!

isNamespaceAware
	isNamespaceAware isNil ifTrue: [isNamespaceAware := self basicIsVisualWorks].
	^isNamespaceAware
!

isObjectStudio

	^ self dialectName = #ObjectStudio
!

isSmalltalkX

        ^ self dialectName = #SmalltalkX
!

isSqueak

	^ self dialectName = #Squeak
!

isVWWithNameSpaces
	"For cases where our hacked backports of asQualifiedReference and etc. to VW3 are not sufficient."

	^self isVisualWorks and: [self isNamespaceAware]
!

isVisualAge

	^ self dialectName = #VisualAge
!

isVisualWorks

	^ self dialectName = #VisualWorks
! !

!Dialect class methodsFor:'numbers'!

coerceToDoublePrecisionFloat: aNumber

	self isGNU ifTrue: [^ aNumber asFloatD].
	self isVisualWorks ifTrue: [^ aNumber asDouble].
	self isVisualAge ifTrue: [^ aNumber asDouble].
	self isSqueak ifTrue: [^aNumber asFloat].
	^ aNumber
!

doesPrecisionOf: aNumber equal: aPrecision
	^aNumber class == self fixedPointClass and: [aNumber scale = aPrecision].
!

doublePrecisionFloatClass

	self isGNU ifTrue: [^ self smalltalkAt: #FloatD].
	self isVisualWorks ifTrue: [^ self smalltalkAt: #Double].
	^ Float
!

fixedPointClass
	Dialect isVisualWorks ifTrue: [^self smalltalkAt: #FixedPoint].
!

isFloatingPoint: aNumber
	^aNumber class == self doublePrecisionFloatClass or: [aNumber class == Float].
!

singlePrecisionFloatClass
	^ Float
! !

!Dialect class methodsFor:'private'!

basicIsDolphin
	^Smalltalk includesKey: #DolphinSplash.
!

basicIsGNU
	^Smalltalk includesKey: #BindingDictionary.
!

basicIsObjectStudio
	^Smalltalk class name == #SmalltalkClass.
!

basicIsSmalltalkX
        ^(Smalltalk respondsTo:#isSmalltalkX) and:[Smalltalk isSmalltalkX].
!

basicIsSqueak

	^ (Smalltalk respondsTo: #vmVersion) and: [(Smalltalk vmVersion copyFrom: 1 to: 6) = 'Squeak'.]
!

basicIsVisualAge

	^ Smalltalk class name == #EsSmalltalkNamespace
	"	| sys |
	sys := Smalltalk at: #System ifAbsent: [^false].
	(sys respondsTo: #vmType) ifFalse: [^false].
	^sys vmType = 'ES'"
!

basicIsVisualWorks

	^ Smalltalk class name == #NameSpace
	"Smalltalk class selectors do: [ :s | 
		(s == #versionName and: [ (Smalltalk versionName copyFrom: 1 to: 11) = 'VisualWorks']) 
		    ifTrue: [^true]].
	^false"
!

basicIsVisualWorksWithoutNamespaces
	^Smalltalk class name == #SystemDictionary
!

determineDialect
        self basicIsDolphin ifTrue: [^ dialectName := #Dolphin].
        self basicIsGNU ifTrue: [^ dialectName := #GNU].
        self basicIsVisualAge ifTrue: [^ dialectName := #VisualAge].
        self basicIsVisualWorks ifTrue: [^ dialectName := #VisualWorks].
        self basicIsSmalltalkX ifTrue: [^ dialectName := #SmalltalkX].
        self basicIsSqueak ifTrue: [^ dialectName := #Squeak].
        self basicIsObjectStudio ifTrue: [^dialectName := #ObjectStudio].
        self basicIsVisualWorksWithoutNamespaces ifTrue: [^dialectName := #VisualWorks].
        self error: 'I don''t know what dialect this is'.
        ^ self
!

lookedUpNames
	lookedUpNames isNil ifTrue: [lookedUpNames := IdentityDictionary new].
	^lookedUpNames.
!

reset
	lookedUpNames := nil.
	identitySetClass := nil.
	timestampClass := nil.
	dialectName := nil.
! !

!Dialect class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__Dialect.st,v 1.1 2006/07/26 09:26:48 boris Exp $'
! !
