"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DescriptorSystem subclass:#DynamicDescriptorSystem
	instanceVariableNames:'attributeNameToFieldNameBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

DynamicDescriptorSystem comment:'This is a descriptor system whose descriptors and tables are created dynamically rather than out of generated code. Note that identity is extremely important, so care is required to set these up properly.
'
!


!DynamicDescriptorSystem class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DynamicDescriptorSystem methodsFor:'accessing'!

addEmptyDescriptorFor: aClass 
	| descriptor |
	descriptor := Descriptor new.
	descriptor system: self.
	self privateDescriptorAt: aClass put: descriptor.
	descriptor classModel: (self classModelFor: aClass).
	^descriptor
!

addEmptyTableNamed: aString 
	| table |
	table := DatabaseTable named: aString.
	self privateTableAt: aString put: table.
	^table
!

allClasses
	^descriptors keys
!

allTableNames
	^tables keys
!

attributeNameToFieldNameBlock: aBlock
	attributeNameToFieldNameBlock := aBlock
!

initializeClassModel: aClassModel
	"Assume we'll initialize it either before or after this"
! !

!DynamicDescriptorSystem methodsFor:'hooks'!

fieldNameForAttributeName: aSymbol 
	attributeNameToFieldNameBlock isNil  
		ifTrue: [^super fieldNameForAttributeName: aSymbol].
	^attributeNameToFieldNameBlock value: aSymbol
! !

!DynamicDescriptorSystem methodsFor:'private'!

privateDescriptorAt: aClass put: aDescriptor
	"Normally you don't want to be setting tables explicitly, as it may defeat the identity management but it's here if needed"
	descriptors at: aClass put: aDescriptor.
!

privateTableAt: aString put: aTable
	"Normally you don't want to be setting tables explicitly, as it may defeat the identity management but it's here if needed"
	tables at: aString asUppercase put: aTable.
! !

!DynamicDescriptorSystem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__DynamicDescriptorSystem.st,v 1.1 2006/07/26 08:04:31 boris Exp $'
! !
