"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#ExpressionGroup
	instanceVariableNames:'children'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!

ExpressionGroup comment:'This isn''t really an expression, in that it can never occur due to parsing. It''s a way of grouping several expressions together so that we can process them together, essentially making sure that the iteration methods will loop over all the expressions, but only do each node once, even if it occurs in multiple expressions.  This is used in processing order expressions to figure out what tables and join expressions we need.

Because it is only used in transient ways, it probably doesn''t implement all the required operations for normal expression usage.
'
!


!ExpressionGroup class methodsFor:'instance creation'!

with: anExpression
	^self new add: anExpression.
! !

!ExpressionGroup class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ExpressionGroup methodsFor:'accessing'!

add: anExpression
	children add: anExpression.
!

addAll: anExpressionCollection
	anExpressionCollection isNil ifTrue: [^self].
	children addAll: anExpressionCollection.
!

children
	^children.
!

queryLevelBaseExpression
	^nil.
! !

!ExpressionGroup methodsFor:'initialize'!

initialize
	super initialize.
	children := OrderedCollection new.
! !

!ExpressionGroup methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree. Keep track of who has already been visited, so we don't get trapped in cycles or visit nodes twice."

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	children do: 	[:each | each do: aBlock skipping: aSet].
	aBlock value: self.
! !

!ExpressionGroup methodsFor:'printing'!

printTreeOn: aStream

	children do: [:each |
		aStream nextPut: $(.
		each printOn: aStream.
		aStream nextPut: $).
		aStream cr].
! !

!ExpressionGroup class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__ExpressionGroup.st,v 1.1 2006/07/26 08:28:19 boris Exp $'
! !
