"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

TypeMapping subclass:#FilteredTypeMapping
	instanceVariableNames:'field key keyDictionary'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpMappings'
!

FilteredTypeMapping comment:'EnumeratedMapping knows what type an object should be based on the value of a single row.'
!


!FilteredTypeMapping class methodsFor:'instance creation'!

to: field keyedBy: key
	^self new
		field: field
		keyedBy: key
! !

!FilteredTypeMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!FilteredTypeMapping methodsFor:'accessing'!

field
	"If we don't have this, look it up from one of our sub-mappings"
	| concrete |
	field isNil ifTrue: [
		concrete := self descriptor typeResolver concreteMembers detect: [:each | each typeMapping field notNil].
		field := concrete typeMapping field].
	^field.
!

keyDictionary
	keyDictionary isNil ifTrue: [ self buildKeyDictionary ].
	^keyDictionary
!

keyedBy
	^key
!

keyedBy: aKey
	key := aKey
!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^Array with: self field.
! !

!FilteredTypeMapping methodsFor:'initialize-release'!

buildKeyDictionary
	| subclassDescriptor |
	keyDictionary := Dictionary new.
	key isNil ifFalse: [keyDictionary at: key put: descriptor describedClass].
	descriptor describedClass glorpAllSubclasses do: 
			[:each | 
			subclassDescriptor := descriptor system descriptorFor: each.
			(subclassDescriptor isNil 
				or: [subclassDescriptor typeMapping keyedBy isNil]) 
					ifFalse: 
						[keyDictionary at: subclassDescriptor typeMapping keyedBy
							put: subclassDescriptor describedClass]]
!

field: aField keyedBy: aKey
	field := aField.
	key := aKey.
! !

!FilteredTypeMapping methodsFor:'mapping'!

addTypeMappingCriteriaTo: collection in: base
	| singleRightValue r l |
	singleRightValue := self keys size = 1.
	r := ConstantExpression for: (singleRightValue
											ifTrue: [self keys asArray first]
											ifFalse: [self keys]).
	l := FieldExpression forField: self field basedOn: base.
	collection add: (singleRightValue ifTrue: [l equals: r] ifFalse: [l in: r]).
!

mapFromObject: anObject intoRowsIn: aRowMap

	| row |
	canWrite ifFalse: [^self].
	row := aRowMap findOrAddRowForTable: self field table withKey: anObject.
	row at: field put: key.
!

trace: aTracing context: anExpression
	^self.
! !

!FilteredTypeMapping methodsFor:'types'!

describedConcreteClassFor: aRow withBuilder: builder

	^self keyDictionary at: (aRow atIndex: (builder translateFieldPosition: field))
!

keys
	^self keyDictionary keys
! !

!FilteredTypeMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__FilteredTypeMapping.st,v 1.1 2006/07/26 08:49:04 boris Exp $'
! !
