"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

OcelotPlatform subclass:#FirebirdPlatform
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDatabase'
!

FirebirdPlatform comment:'This is a Glorp platform for the InterBase and Firebird databases. It''s currently subclassed from Ocelot because both claim to be highly standard-compliant. It would probably be better if both inherited from some sort of abstract SQL92Platform.'
!


!FirebirdPlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!FirebirdPlatform methodsFor:'accessing'!

vwEXDIConnectionClass
	^'InterBaseConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'IBEXDI'
! !

!FirebirdPlatform methodsFor:'binding'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.
! !

!FirebirdPlatform methodsFor:'constants'!

areSequencesExplicitlyCreated
	^true
!

initializeReservedWords
	reservedWords := Set new.
	reservedWords 
		add: 'type';
		add: 'value';
		add: 'password';
		add: 'unique';
		add: 'date';
		add: 'current';
		add: 'order';
		add: 'active';
		add: 'action';
		add: 'database';
		add: 'table';
		add: 'timestamp';
		add: 'position'.
!

maximumLengthOfColumnName
	"^<Integer> I return the max. length of a column name"

	^31
!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^31
!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^false.
!

supportsMillisecondsInTimes
	^false.
! !

!FirebirdPlatform methodsFor:'conversion'!

blobDataToStreamConverter
	^DelegatingDatabaseConverter
		named: #blobDataToStream
		hostedBy: self
		fromStToDb: #convertBlobData:for:
		fromDbToSt: #nullConversion:for:.
!

convertBlobData: aData for: aType
	"From standard Store getBlobData called in writeSession;  Firebird seems to need it.  Glorp needs to think that two such streams on the same data are equal."

	^EqualOnContentReadStream on: aData
! !

!FirebirdPlatform methodsFor:'sequences'!

databaseSequenceClass
	^FirebirdSequence.
! !

!FirebirdPlatform methodsFor:'testing'!

supportsBinding
	"Return true if this platform supports binding parameters rather than printing them as strings into the SQL statement"
	^true.
! !

!FirebirdPlatform methodsFor:'types'!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'blob sub_type text'].
!

sequence

	^self typeNamed: #sequence ifAbsentPut: [GlorpSerialType new typeString: (self integer typeString)].
! !

!FirebirdPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__FirebirdPlatform.st,v 1.1 2006/07/26 08:42:47 boris Exp $'
! !
