"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#FunctionExpression
	instanceVariableNames:'function base alias symbol'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!

FunctionExpression class instanceVariableNames:'functions'

"
 No other class instance variables are inherited by this class.
"
!

FunctionExpression comment:'This represents a database function or other modifier. For example, conversion to upper or lower case, or the ascending/descending modifier in order by clauses. At the moment it is hard-coded to to handle only the descending modifier and does not handle e.g. function arguments, functions that differ between databases, functional syntax ( as opposed to postfix). One would probably define subclasses to handle these cases, but this is the simplest thing that could possibly work for the current functionality.

Instance Variables:
	function	<String>	The string we print to the database
	base <GlorpExpression> The thing we''re a function of.
	alias <String> If we''re to be selected as a field, what should we be selcted as. Think SELECT t1.A + t1.B AS FRED.
	symbol <Symbol> The original symbol used to create us. Helpful if recreating onto a different platform where the DB string can be different.
	
'
!


!FunctionExpression class methodsFor:'instance creation'!

for: aSymbol withArguments: anArray basedOn: anExpression
	| functionTemplate newFunction system base |
	base := anExpression ultimateBaseExpression.
	system := base isNil ifTrue: [nil] ifFalse: [base system].	
	functionTemplate := (system isNil
		ifTrue: [self basicFunctions]
		ifFalse: [system platform functions]) at: aSymbol ifAbsent: [^nil].
	newFunction := functionTemplate
		createFor: anArray
		withBase: anExpression
		symbol: aSymbol.
	^newFunction.
! !

!FunctionExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!FunctionExpression class methodsFor:'private'!

basicFunctions
	functions isNil ifTrue: [functions := self createBasicFunctions].
	^functions.
!

createBasicFunctions
	^IdentityDictionary new
		at: #descending put: (PostfixFunction named: 'DESC');
		at: #distinct put: (PrefixFunction named: 'DISTINCT');
		at: #max put: (PrefixFunction named: 'MAX');
		at: #min put: (PrefixFunction named: 'MIN');
		at: #sum put: (PrefixFunction named: 'SUM');
		at: #avg put: (PrefixFunction named: 'AVG');
		at: #average put: (PrefixFunction named: 'AVG');	
		at: #toChar: put: (PrefixFunction named: 'TO_CHAR');
		at: #not put: (PrefixFunction named: 'NOT');
		at: #, put: (InfixFunction named: '||');
		at: #countStar put: (StandaloneFunction named: 'COUNT(*)');
		at: #count put: (PrefixFunction named: 'COUNT');
		at: #isNil put: (RelationExpression new relation: #=; rightChild: nil asGlorpExpression);
		at: #notNil put: (RelationExpression new relation: #<>; rightChild: nil asGlorpExpression);
		at: #isNIL put: (RelationExpression new relation: #=; rightChild: nil asGlorpExpression);
		at: #notNIL put: (RelationExpression new relation: #<>; rightChild: nil asGlorpExpression);
		at: #exists: put: (InfixFunction named: 'EXISTS');
		at: #notExists: put: (InfixFunction named: 'NOT EXISTS');
		at: #asUppercase put: (PrefixFunction named: 'UPPER');
		at: #asLowercase put: (PrefixFunction named: 'LOWER');
		at: #replace:with: put: (PrefixFunction named: 'REPLACE');
		at: #cast: put: ((PrefixFunction named: 'CAST') separator: ' AS ');
		yourself.
!

initialize

	self resetFunctions.
!

named: aString
	"Used for creating template instances only"

	^self new function: aString.
!

resetFunctions
	"self resetFunctions"
	functions := nil.
! !

!FunctionExpression methodsFor:'As yet unclassified'!

valueInBuilder: anElementBuilder as: anExpression
	^self base valueInBuilder: anElementBuilder as: anExpression.
! !

!FunctionExpression methodsFor:'accessing'!

alias
	^alias
!

alias: aString
	alias := aString
!

arguments: anArray
	^self.
!

base
	^base.
!

base: anExpression
	base := anExpression.
!

canHaveBase
	^true.
!

convertedDbValueOf: anObject
	"Not clear what should happen here, but let's just pass it through for the moment"
	^base convertedDbValueOf: anObject.
!

converterForStType: aClass
	^self field converterForStType: aClass.
!

defaultType
	"We don't know our type. Return a basic numeric type as being as good a guess as any."
	^base system platform numeric
!

field

	^base field.
!

fieldExpression

	^base fieldExpression.
!

function: aString

	function := aString.
!

function: aString arguments: anArray

	self function: aString.
!

impliedSmalltalkType
	^self field impliedSmalltalkType.
!

mappedFields
	^Array with: self.
!

symbol
	^symbol
!

symbol: aSymbol
	symbol := aSymbol
!

table
	^self field table.
!

tablesToPrint
	^#().
!

type
	| field |
	field := self field.
	^field isNil ifTrue: [self defaultType] ifFalse: [field type].
! !

!FunctionExpression methodsFor:'api'!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	| newBase |
	newBase := base asExpressionJoiningSource: source toTarget: target.
	^self copy function: function; base: newBase.
!

get: aSymbol withArguments: anArray
	| functionExpression |
	functionExpression := self getFunction: aSymbol withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	^RelationExpression
		named: aSymbol
		basedOn: self
		withArguments: anArray
! !

!FunctionExpression methodsFor:'compatibility'!

translateField: aField
	^base translateField: aField.
! !

!FunctionExpression methodsFor:'converting'!

asDistinctFieldOn: anExpression
	self isDistinct ifTrue: [^self asGlorpExpressionOn: anExpression].
	^super asDistinctFieldOn: anExpression.
!

asNonDistinctField
	^self isDistinct ifTrue: [base] ifFalse: [self].
! !

!FunctionExpression methodsFor:'copying'!

createFor: arguments
	| new |
	new := self copy.
	new arguments: arguments.
	^new.
!

createFor: arguments withBase: anExpression symbol: aSymbol
	| new |
	new := self copy.
	new arguments: arguments.
	new base: anExpression.
	new symbol: aSymbol.
	^new.
!

withTable: aTable
	"Make a copy of ourselves that uses aTable. Presumably aTable is an aliased version of our table"

	| copy |
	copy := self rebuildOn: BaseExpression new.
	copy fieldExpression setTable: aTable.
	^copy.
! !

!FunctionExpression methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree. Keep track of who has already been visited, so we don't get trapped in cycles or visit nodes twice."

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	aBlock value: self.
! !

!FunctionExpression methodsFor:'mapping'!

valueInBuilder: anElementBuilder
	^self base valueInBuilder: anElementBuilder as: self.
! !

!FunctionExpression methodsFor:'navigating'!

queryLevelBaseExpression

	^base queryLevelBaseExpression.
!

ultimateBaseExpression

	^base ultimateBaseExpression.
! !

!FunctionExpression methodsFor:'preparing'!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	| rebuilt |
	self == anExpression ifTrue: [^aBaseExpression].
	rebuilt := (base rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression) get: self symbol withArguments: #().
	rebuilt alias: self alias.
	^rebuilt.
! !

!FunctionExpression methodsFor:'printing'!

printOnlySelfOn: aStream
	aStream nextPutAll: function.
!

printSQLOn: aStream withParameters: aDictionary
	
	self subclassResponsibility.
!

printSelectSQLOn: aStream withParameters: aDictionary
	self printSQLOn: aStream withParameters: aDictionary.
	alias notNil ifTrue: [aStream nextPutAll: ' AS '; nextPutAll: alias].
!

printTreeOn: aStream 
	aStream 
		nextPutAll: function
! !

!FunctionExpression methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	^true.
!

isDistinct
	^function = 'DISTINCT'.
!

isGenerated
	^false.
!

printsIntoJoins
	"Certain functions should be printed into joins as well as into select lists (e.g. UPPER). However, there are others that don't (at least I don't think so) make any sense in joins, e.g. DISTINCT. Differentiate the two. Right now the only one I can think of that doesn't make sense is distinct. It's not clear to me that I'm not just hacking my way through this"
	^self isDistinct not.
! !

!FunctionExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__FunctionExpression.st,v 1.1 2006/07/26 08:29:08 boris Exp $'
! !

FunctionExpression initialize!
