"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#GeneralSubSelectExpression
	instanceVariableNames:'query base'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpExpressions'
!

GeneralSubSelectExpression comment:'This represents an arbitrary sub-select.

Instance Variables:
	query	<Query>	The query that we subselect.

'
!


!GeneralSubSelectExpression class methodsFor:'instance creation'!

basedOn: anExpression
	"The defining characteristic of a general subselect is that it doesn't care what it's based on. The base is just a convenient place to hang an independent expression. So make sure we're built on a true base, which won't print confusing stuff for us"
	^self new base: anExpression ultimateBaseExpression.
! !

!GeneralSubSelectExpression methodsFor:'As yet unclassified'!

descriptor
	^base system descriptorFor: query resultClass.
!

get: aSymbol
	"This is rather a hack on the use of get:, but I think it should be all right because we should never run into one of these in normal expression generation. The issue is that if we do an expression of the form (x = aQuery) where the query returns an object of the same class as X. We want to rewrite that into a primary key comparison for the database, so we want to do a get: against aQuery. But the only thing that seems to sensibly translate into is a retrieve: clause and return the query. That will only work for a single level get:, but I think that a multi-level one, from converting a block, shouldn't ever get to a general sub-select."
	query retrieve: aSymbol.
!

getField: aField
	"This is rather a hack on the use of get:, but I think it should be all right because we should never run into one of these in normal expression generation. The issue is that if we do an expression of the form (x = aQuery) where the query returns an object of the same class as X. We want to rewrite that into a primary key comparison for the database, so we want to do a get: against aQuery. But the only thing that seems to sensibly translate into is a retrieve: clause and return the query. That will only work for a single level get:, but I think that a multi-level one, from converting a block, shouldn't ever get to a general sub-select."
	query retrieve: [:each | each getField: aField].
! !

!GeneralSubSelectExpression methodsFor:'accessing'!

base: anExpression
	base := anExpression.
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

field
	^query fields first.
!

mappedFields
	"Let's try this and see if it works returning empty"
	^#().
!

query
	^query
!

query: anObject
	query := anObject
! !

!GeneralSubSelectExpression methodsFor:'api'!

base
	^base
!

get: aSymbol withArguments: anArray 

	^RelationExpression
		named: aSymbol
		basedOn: self
		withArguments: anArray
! !

!GeneralSubSelectExpression methodsFor:'converting'!

asJoin
	" *** This method was defined by Glorp.GlorpExpression as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'
!

convertedDbValueOf: anObject
	^anObject.
! !

!GeneralSubSelectExpression methodsFor:'navigating'!

ultimateBaseExpression
	^base ultimateBaseExpression.
! !

!GeneralSubSelectExpression methodsFor:'preparing'!

addBoundExpressionsIn: aCommand to: boundExpressions
	self query session: aCommand query session.
	self query prepare.
	self query addBoundExpressionsIn: aCommand to: boundExpressions.
!

asExpressionJoiningSource: source toTarget: target
	" *** This method was defined by Glorp.GlorpExpression as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	self == anExpression ifTrue: [^aBaseExpression].
	^query asGlorpExpressionOn: aBaseExpression.
! !

!GeneralSubSelectExpression methodsFor:'printing'!

printOnlySelfOn: aStream
	" *** This method was defined by Glorp.GlorpExpression as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'
!

printSQLOn: aStream withParameters: aDictionary 
	| command |
	aStream nextPutAll: '('.
	self query session: aStream query session.
	command := self query sqlWith: aDictionary.
	aStream nextPutAll: command sqlString.
	aStream nextPutAll: ')'.
!

printTreeOn: aStream
	aStream nextPutAll: 'subselect('.
	query printOn: aStream.
	aStream nextPutAll: ')'.
! !

!GeneralSubSelectExpression methodsFor:'testing'!

isMappable
	"Return true if we're something that can be returned in a select statement."
	^false.
! !

!GeneralSubSelectExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GeneralSubSelectExpression.st,v 1.1 2006/07/26 08:48:11 boris Exp $'
! !
