"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpAttributeModel
	instanceVariableNames:'name type collectionType keyType classModel attributeIndex
		useDirectAccess setSelector'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpMappings'
!


!GlorpAttributeModel class methodsFor:'instance creation'!

named: aSymbol
	^self new name: aSymbol.
!

named: aSymbol collection: aCollectionClass of: aClass
	^(self new)
		name: aSymbol;
		type: aClass;
		collectionType: aCollectionClass.
!

named: aSymbol collectionOf: aClass
	^self named: aSymbol collection: OrderedCollection of: aClass.
!

named: aSymbol dictionary: dictionaryClass from: keyClass to: valueClass
	^(self new)
		name: aSymbol;
		type: valueClass;
		keyType: keyClass;
		collectionType: dictionaryClass.
!

named: aSymbol dictionaryFrom: keyClass to: valueClass
	^self named: aSymbol dictionary: Dictionary from: keyClass to: valueClass.
!

named: aSymbol type: aClass

        ^self new name: aSymbol; type: aClass.
! !

!GlorpAttributeModel methodsFor:'As yet unclassified'!

isCollectionAttribute
	^collectionType notNil.
!

newCollection
	^self collectionType new.
! !

!GlorpAttributeModel methodsFor:'accessing'!

classModel
	^classModel
!

classModel: aGlorpClassModel
	classModel := aGlorpClassModel
!

collectionType
	^collectionType
!

collectionType: aClass
	collectionType := aClass
!

keyType
	^keyType
!

keyType: aClass
	keyType := aClass
!

name
	^name isNil ifTrue: [''] ifFalse: [name]
!

name: aSymbol
	| symbol |
	symbol := aSymbol isNil ifTrue: [aSymbol] ifFalse: [aSymbol asSymbol].
	name := symbol.
!

system
	classModel isNil ifTrue: [^nil].
	^classModel system.
!

type
	^type
!

type: aClass
	type := aClass
!

useDirectAccess: aBoolean
	useDirectAccess := aBoolean.
! !

!GlorpAttributeModel methodsFor:'copying'!

copyFrom: anAttributeModel
	"Copy over the attributes if it defines them and we don't"
	anAttributeModel isNil ifTrue: [^self].
	self type isNil ifTrue: [self type: anAttributeModel type].
	self collectionType isNil
		ifTrue: [self collectionType: anAttributeModel collectionType].
	self privateUseDirectAccess isNil
		ifTrue: [useDirectAccess := anAttributeModel privateUseDirectAccess].
! !

!GlorpAttributeModel methodsFor:'get/set'!

checkType: anObject
	(self classModel matches: anObject)
		ifFalse: [self error: 'Wrong type of object'].
!

getValueFrom: anObject
	| real |
	real := anObject yourSelf.
	self checkType: real.
	self useDirectAccess ifTrue: [^self directGetValueFrom: real].
	^real perform: self name.
!

setValueIn: anObject to: aValue 
	| real |
	self isValid ifFalse: [^self].
	real := anObject yourSelf.
	self checkType: real.
	self useDirectAccess ifTrue: [^self directSetValueIn: real to: aValue].
	^real perform: self setSelector with: aValue
! !

!GlorpAttributeModel methodsFor:'knitting'!

add: anObject to: aCollection in: ourObject
	"If this is an array we can't just add to it, we must concatenate and re-set the value"
	| newCollection |
	aCollection class == Array 
		ifFalse: [^aCollection add: anObject].

	newCollection := aCollection, (Array with: anObject).
	self setValueIn: ourObject to: newCollection.
!

knit: ourObject to: anotherObject 
	"Set up the relationship from our object to another one, indicated by our mapping."

	self isCollectionAttribute 
		ifTrue: 
			[| collection |
			collection := self getValueFrom: ourObject.
			(collection isGlorpProxy and: [collection isInstantiated not]) 
				ifTrue: 
					[collection := self newCollection.
					self setValueIn: ourObject to: collection.
					self 
						add: anotherObject
						to: collection
						in: ourObject.
					^self].
			(collection includes: anotherObject) 
				ifFalse: 
					[self 
						add: anotherObject
						to: collection
						in: ourObject]]
		ifFalse: [self setValueIn: ourObject to: anotherObject]
! !

!GlorpAttributeModel methodsFor:'mapping'!

isDictionaryAttribute
	^collectionType includesBehavior: Dictionary.
!

loopOver: mightBeACollection doing: aBlock
	| index |
	self isCollectionAttribute ifFalse: [^aBlock value: 1 value: mightBeACollection].
	mightBeACollection isNil ifTrue: [^nil].
	self isDictionaryAttribute ifTrue: [^mightBeACollection keysAndValuesDo: aBlock].
	index := 1.
	mightBeACollection
		do:
			[:each | 
			aBlock value: index value: each.
			index := index + 1].
! !

!GlorpAttributeModel methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: self classModel printString, '>>', self name.
! !

!GlorpAttributeModel methodsFor:'private'!

directGetValueFrom: anObject 
	| index |
	index := self instVarIndexIn: anObject.
	index = 0 ifTrue: [self raiseInvalidAttributeError].
	^anObject instVarAt: index
!

directSetValueIn: anObject to: aValue 
	| index |
	index := self instVarIndexIn: anObject.
	index = 0 ifTrue: [self raiseInvalidAttributeError].
	^anObject instVarAt: index put: aValue
!

instVarIndex
	| soughtName |
	attributeIndex == nil ifFalse: [^attributeIndex].
	soughtName := Dialect instVarNameFor: name.
	^attributeIndex := self classModel describedClass allInstVarNames 
				indexOf: soughtName
!

instVarIndexIn: anObject 
	^self instVarIndex
!

isValid
	^name notNil and: [classModel notNil].
!

privateUseDirectAccess
	^useDirectAccess.
!

raiseInvalidAttributeError
	self error: 'Invalid attribute'
!

setSelector
	setSelector  == nil ifTrue: [setSelector := (name, ':') asSymbol].
	^setSelector.
! !

!GlorpAttributeModel methodsFor:'testing'!

useDirectAccess
	useDirectAccess isNil ifFalse: [^useDirectAccess].
	self system isNil ifTrue: [^true].  "A global default, although only applicable if we're not in a valid setup"
	^self system useDirectAccessForMapping.
! !

!GlorpAttributeModel methodsFor:'validation'!

isReadable
	^self useDirectAccess 
		ifTrue: [self instVarIndex > 0]
		ifFalse: [self classModel describedClass canUnderstand: self name]
!

isWritable
	^self useDirectAccess 
		ifTrue: [self instVarIndex > 0]
		ifFalse: [self classModel describedClass canUnderstand: self setSelector]
!

validate
	name isEmpty ifTrue: [self error: 'Invalid attribute name']
!

validateCanRead
   name = #flagBits ifTrue: [self halt].
        self isReadable ifFalse: [self raiseInvalidAttributeError]
!

validateCanWrite
	self isWritable ifFalse: [self raiseInvalidAttributeError]
! !

!GlorpAttributeModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpAttributeModel.st,v 1.1 2006/07/26 08:49:35 boris Exp $'
! !
