"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpClassModel
	instanceVariableNames:'system describedClass subclasses attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpMappings'
!


!GlorpClassModel class methodsFor:'instance creation'!

for: aClass
	^self new describedClass: aClass.
!

new
	^super new initialize.
! !

!GlorpClassModel methodsFor:'accessing'!

addAttribute: anAttributeDescription
	attributes add: anAttributeDescription.
	anAttributeDescription classModel: self.
	^anAttributeDescription.
!

attributeNamed: aSymbol
	^attributes detect: [:each | each name = aSymbol] ifNone: [self newAttributeNamed: aSymbol].
!

attributeNamed: aSymbol type: aClass
	^attributes
		detect:
			[:each | each name = aSymbol and: [each type isNil or: [each type = aClass]]]
		ifNone: [self newAttributeNamed: aSymbol type: aClass].
!

attributeNamed: aSymbol type: aClass collectionType: collectionClass
	^attributes
		detect:
			[:each | 
			| collectionTypeMatches typeMatches nameMatches |
			nameMatches := each name = aSymbol.
			typeMatches := each type isNil or: [each type = aClass].
			collectionTypeMatches := each collectionType isNil
				or: [each collectionType = collectionClass].
			nameMatches & typeMatches & collectionTypeMatches]
		ifNone: [self newAttributeNamed: aSymbol type: aClass].
!

describedClass
	^describedClass
!

describedClass: aClass
	describedClass := aClass.
	subclasses := aClass allSubclasses isEmpty ifTrue: [nil] ifFalse: [aClass allSubclasses asArray].
!

hasAttributeNamed: aSymbol

	attributes do: [:attribute | attribute name = aSymbol ifTrue: [^true]].
	^false
!

system
	^system
!

system: aDescriptorSystem
	system := aDescriptorSystem
! !

!GlorpClassModel methodsFor:'attributes'!

attributesDo: aBlock
	attributes do: aBlock
!

newAttributeNamed: aSymbol
	^self addAttribute: (GlorpAttributeModel named: aSymbol).
!

newAttributeNamed: aSymbol collection: collectionClass of: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol collection: collectionClass of: aClass).
!

newAttributeNamed: aSymbol collectionOf: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol collectionOf: aClass).
!

newAttributeNamed: aSymbol dictionaryFrom: keyClass to: valueClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol dictionaryFrom: keyClass to: valueClass).
!

newAttributeNamed: aSymbol type: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol type: aClass).
! !

!GlorpClassModel methodsFor:'initializing'!

initialize
	attributes := OrderedCollection new.
! !

!GlorpClassModel methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: '~~'.
	self describedClass isNil ifFalse: [aStream nextPutAll: self describedClass name].
	aStream nextPutAll: '~~'.
! !

!GlorpClassModel methodsFor:'validation'!

matches: anObject
	"Try to optimize this check as much as possible. We cache the list of subclasses, and inline the check for isKindOf:"
	| theClass |
	theClass := anObject class.
	describedClass == theClass ifTrue: [^true].
	nil == subclasses ifTrue: [^false].
	1 to: subclasses size do: [:i | theClass == (subclasses at: i)
		ifTrue: [^true]].
	^false.
!

validateIn: aDescriptorSystem 
	attributes do: [:each | each validate]
! !

!GlorpClassModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpClassModel.st,v 1.1 2006/07/26 08:49:11 boris Exp $'
! !
