"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Stream subclass:#GlorpCursoredStream
	instanceVariableNames:'command cursor mightHaveDuplicateRows builders
		buildersThatReturnResults databaseAccessor peekBuffer
		rawPeekBuffer statement'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpQueries'
!


!GlorpCursoredStream class methodsFor:'instance creation'!

on: aStream
	^self basicNew cursor: aStream.
!

on: aStream accessor: aDatabaseAccessor
	^(self on: aStream)
		databaseAccessor: aDatabaseAccessor.
!

on: aStream for: aCommand
	^(self on: aStream)
		command: aCommand.
!

on: aStream for: aCommand accessor: aDatabaseAccessor
	^(self on: aStream)
		command: aCommand;
		databaseAccessor: aDatabaseAccessor.
! !

!GlorpCursoredStream methodsFor:'accessing'!

command
	^command
!

command: aDatabaseCommand
	command := aDatabaseCommand.
	aDatabaseCommand isNil ifTrue: [^self].
	aDatabaseCommand query isNil ifTrue: [^self].
	databaseAccessor := aDatabaseCommand query session accessor.
	builders := self query copyOfBuilders.
	builders do: [:each | each cursor: self].
!

contents
	self shouldNotImplement.
!

cursor
	^cursor
!

cursor: aStream
	cursor := aStream
!

databaseAccessor: aDatabaseAccessor
	databaseAccessor := aDatabaseAccessor
!

flush
	^self.
!

mightHaveDuplicateRows
	mightHaveDuplicateRows isNil ifTrue: [mightHaveDuplicateRows := self query isNil ifTrue: [false] ifFalse: [self query mightHaveDuplicateRows]].
	^mightHaveDuplicateRows.
!

next
	| answer session |
	self atEnd ifTrue: [^nil].
	session := self session.

	[self atEnd ifTrue: [^nil].
	answer := self nextAnswer.
	answer == nil] 
		whileTrue.

	[self hasCompleteAnswerFor: answer] whileFalse: [self nextAnswer].
	session == nil ifFalse: [session registerAsOld: answer].
	^answer.
!

next: anInteger
	| result |
	Dialect isObjectStudio ifFalse: [^super next: anInteger].
	result := OrderedCollection new: anInteger.
	anInteger timesRepeat: [result add: self next].
	^result.
!

nextPut: anObject
	self shouldNotImplement.
!

peek
	peekBuffer = nil ifFalse: [^peekBuffer].
	peekBuffer := self nextAnswer.
	^peekBuffer.
!

query
	command isNil ifTrue: [^nil].
	^command query.
!

query: x
!

session
	^self query isNil ifTrue: [nil] ifFalse: [self query session].
!

statement
	^statement.
!

statement: aGlorpPreparedStatement
	statement := aGlorpPreparedStatement.
!

upToEnd
	| result |
	result := WriteStream on: (Array new: 100).
	[self atEnd] whileFalse: [
			| nextValue |
			"Stupid VA's cursors don't deal with atEnd or isEmpty nicely, so check for nil"
			nextValue := self next.
			nextValue == nil ifFalse: [	result nextPut: nextValue]].
	^result contents
! !

!GlorpCursoredStream methodsFor:'building objects'!

buildObjectsForRow: aRow
	self query isNil ifTrue: [^aRow].
	builders
		do: [:each | each findInstanceForRow: aRow proxyType: self query proxyType].
	builders do: [:each | each buildObjectFrom: aRow].
	builders do: [:each | 
		"Yes, the == false is correct"
		(self session sendPostFetchValidateEventTo: each instance) == false
			ifTrue: [each instance: nil]].
	builders do: [:each | each instance == nil ifFalse: [each knitResultIn: self]].
	builders do: [:each | 	self session sendPostFetchEventTo: each instance].
	^self buildersThatReturnResults size = 1
		ifTrue: [self buildersThatReturnResults first returnValueIn: self]
		ifFalse: [self buildersThatReturnResults collect: [:each | each returnValueIn: self]].
!

buildersThatReturnResults
	buildersThatReturnResults isNil ifTrue: [
		buildersThatReturnResults := OrderedCollection new.
		builders
			do: [:each | 	
				| expression shouldBeRetrieved |
				"For some bizarre reason, includes: fails on ObjectStudio for this"
				expression := each expression.
				shouldBeRetrieved := (self query tracing retrievalExpressions detect: [:foo |
					foo == expression] ifNone: [nil]) notNil.
				(shouldBeRetrieved and: [
					(buildersThatReturnResults anySatisfy: [:eachBuilder | eachBuilder expression == expression]) not])
						ifTrue: [buildersThatReturnResults add: each]].
		buildersThatReturnResults := buildersThatReturnResults asArray].
	^buildersThatReturnResults.
!

collectionTypeFor: aCollectionType
	"Make sure that we can reasonably return aCollectionType. If not, default to something we can"

	aCollectionType class == GlorpDictionaryDescription ifFalse: [^aCollectionType].
	self buildersThatReturnResults size = 1 ifTrue: [^aCollectionType].
	^Array.
!

elementBuilderFor: anExpression 
	^builders detect: [:each | each expression == anExpression] ifNone: [nil].
!

hasCompleteAnswerFor: objects
	| nextAnswer |
	self atEnd ifTrue: [^true].
	self mightHaveDuplicateRows ifFalse: [^true].
	nextAnswer := self peek.
	^objects ~= nextAnswer
!

nextAnswer
	| result |
	peekBuffer isNil ifFalse: [
		result := peekBuffer.
		peekBuffer := nil.
		^result].
	^self nextAnswerFromUnderlyingCursor.
!

nextAnswerFromUnderlyingCursor
	| objects |
	rawPeekBuffer isNil ifTrue: [rawPeekBuffer := cursor next].
	rawPeekBuffer isNil ifTrue: [^nil].
	objects := self buildObjectsForRow: rawPeekBuffer.
	rawPeekBuffer := nil.
	^objects.
!

nextObjectFrom: aRow
	| objects |
	objects := self buildObjectsForRow: aRow.
	self mightHaveDuplicateRows ifFalse: [
		self registerInstances.
		^objects].
!

objectBuilderFor: anExpression 
	"Return the builder for this expression, but only those that directly build compound objects. Most importantly, exclude dictionary builders"
	^builders detect: [:each | 
		each buildsObjects and: [each expression == anExpression]] ifNone: [nil].
!

registerInstances
	self session isNil ifTrue: [^self].
	builders do: [:each | self session register: each instance].
! !

!GlorpCursoredStream methodsFor:'debugging'!

rows
	"This will extract all the rows and replace the internal stream with a regular stream on the rows. This is useful for debugging, but in real use rather destroys the purpose of the cursored stream"
	| rows |
	rows := cursor upToEnd.
	self releaseCursor.
	cursor := rows readStream.
	^rows.
! !

!GlorpCursoredStream methodsFor:'initializing'!

initialize
!

release
	super release.
	self releaseCursor.
	cursor := nil.
	rawPeekBuffer := nil.
	peekBuffer := nil.
	statement isNil
		ifFalse:
			[databaseAccessor reusePreparedStatements
				ifTrue: [statement busy: false]
				ifFalse: [statement release].
			statement := nil].
	databaseAccessor := nil.
!

releaseCursor
	self accessor isNil ifFalse: [self accessor releaseCursor: cursor in: statement].
! !

!GlorpCursoredStream methodsFor:'iterating'!

collect: aBlock 
	| newStream |
	newStream := WriteStream on: (Array new: 100).
	[self atEnd] whileFalse: [newStream nextPut: (aBlock value: self next)].
	^newStream contents
!

detect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	^self detect: aBlock ifNone: [self error: 'not found']
!

detect: aBlock ifNone: exceptionBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	self do: [:each | (aBlock value: each) ifTrue: [^each]].
	^exceptionBlock value
!

do: aBlock 
	"Evaluate aBlock for each of the elements of the receiver."

	[self atEnd]
		whileFalse: [aBlock value: self next]
! !

!GlorpCursoredStream methodsFor:'private'!

rawPeek
	"Peek at the underlying rows, but don't build objects"
	rawPeekBuffer isNil ifFalse: [^rawPeekBuffer].
	rawPeekBuffer := cursor next.
	^rawPeekBuffer.
! !

!GlorpCursoredStream methodsFor:'testing'!

accessor
	^databaseAccessor.
!

atEnd
	"Release ourselves fairly aggressively when we hit the end"
	| accessor |
	peekBuffer isNil ifFalse: [^false].
	rawPeekBuffer isNil ifFalse: [^false].
	cursor isNil ifTrue: [^true].
	"For test purposes, fabricate an accessor if we don't have one, it's not really needed for a non-db stream"
	accessor := self accessor isNil ifTrue: [DatabaseAccessor new] ifFalse: [self accessor].
	^([accessor isCursorAtEnd: cursor] on: Dialect error do: [:ex | databaseAccessor handleError: ex for: command])
		ifTrue:
			[self release.
			true]
		ifFalse: [	"We have to do this, because VA won't answer true to atEnd until we've read past end"
			self rawPeek isNil].
!

isReadable
	^true.
!

isWritable
	^false.
! !

!GlorpCursoredStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpCursoredStream.st,v 1.1 2006/07/26 08:51:12 boris Exp $'
! !
