"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpDatabaseType
	instanceVariableNames:'name platform typeString'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!GlorpDatabaseType class methodsFor:'instance creation'!

instance

	^super new
!

new
	^super new initialize.
! !

!GlorpDatabaseType class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDatabaseType class methodsFor:'printing'!

padToThreeDigits: anInteger

	| string |
	string := anInteger truncated printString.
	string size = 3 ifTrue: [^string].
	string size = 2 ifTrue: [^'0', string].
	string size = 1 ifTrue: [^'00', string].
!

padToTwoDigits: anInteger

	| string |
	string := anInteger truncated printString.
	^string size = 1 ifTrue: ['0', string] ifFalse: [string].
! !

!GlorpDatabaseType methodsFor:'SQL'!

glorpPrintSQLOn: aCommand
	aCommand nextPutAll: self typeName.
!

platform

	^platform
!

platform: aDatabasePlatform

	platform := aDatabasePlatform.
!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow
!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: aSession
!

preWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor
	aDatabaseField defaultValue isNil ifTrue: [^self].
	(aDatabaseRow hasValueFor: aDatabaseField) ifTrue: [^self].
	"Default values will not write if this field is constrained to a primary key generator"
	(aDatabaseRow expectsToGetValueFor: aDatabaseField) ifTrue: [^self].
	aDatabaseRow at: aDatabaseField put: (aDatabaseField defaultValue).
!

print: aValue on: aStream
	aValue glorpPrintSQLOn: aStream.
!

printCollection: aCollection on: aStream
	aCollection glorpPrintSQLOn: aStream for: self.
!

typeName
	"An alias that can be used to print the non-parameterized name"
	^typeString.
!

typeString

	^typeString.
! !

!GlorpDatabaseType methodsFor:'accessing'!

precision: anInteger

	^self error: ((self class name asString), ' is not a variable precision type.')
!

queryType
	"If we can't pass this type in as an argument to a query (e.g. a blob) then return a type we can use instead"
	^self.
!

scale: anInteger

	^self error: ((self class name asString), ' is not a variable scale type.')
!

sequence
	^nil
!

size: anInteger

	^self error: ((self class name asString), ' is not a variable sized type.')
!

typeString: aString

	typeString := aString
! !

!GlorpDatabaseType methodsFor:'converting'!

converterForStType: aClass

	^self platform nullConverter.
!

impliedSmalltalkType
	"Return the Smalltalk type which most commonly corresponds to our database type. By default, Object if we don't have any more specific information."
	^Object.
! !

!GlorpDatabaseType methodsFor:'exdi specific'!

exdiType
	"Return a symbol that the VisualWorks EXDI uses to indicate the database type -- this is a symbol which is often the name of a Smalltalk class, but with variations for multiple db types that correspond to the same Smalltalk symbol. It also varies by Database. This is equivalent to the values returned by methods like #odbcConversion or #oracleConversion"
	self subclassResponsibility.
! !

!GlorpDatabaseType methodsFor:'initialize'!

initialize
!

initializeForField: aDatabaseField in: aDescriptorSystem
! !

!GlorpDatabaseType methodsFor:'testing'!

generatesOverExistingValues
	"Do we generate our value even if there's alread one present in the field"

	^false.
!

hasParameters
	"Return true if this has modifiable parameters. That is, when we return one of these, should we return a copy rather than trying to save space be re-using instances"
	^false.
!

hasSequence
	^false.
!

isGenerated

	^false
!

isIdentityColumn
	^false.
!

isSerial
	"answer if this is a database-generated primary key column"

	^false
!

isStringType
	"Return true if the type of values this stores are strings"
	^false.
!

isVariable
	^false.
!

isVariableWidth
	"Return true if this type allows varying length data within a particular instance. e.g., this is true for a varchar, but false for a fixed size character field"
	^false.
! !

!GlorpDatabaseType class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpDatabaseType.st,v 1.1 2006/07/26 08:04:55 boris Exp $'
! !
