"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpRowSorter
	instanceVariableNames:'childSelector queue sorted current items childCache'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDatabase'
!


!GlorpRowSorter class methodsFor:'instance creation'!

new
	^super new initialize.
! !

!GlorpRowSorter methodsFor:'accessing'!

getChildrenVia: aSelector
	childSelector := aSelector.
! !

!GlorpRowSorter methodsFor:'initializing'!

initialize
	queue := OrderedCollection new: 100.
	sorted := OrderedCollection new: 100.
	childCache := IdentityDictionary new: 100.
! !

!GlorpRowSorter methodsFor:'sorting'!

childrenOf: anItem
	^childCache at: anItem ifAbsentPut: [
		childSelector isSymbol
			ifTrue: [anItem perform: childSelector]
			ifFalse: [childSelector value: anItem]].
!

currentChildren
	^self childrenOf: current.
!

currentItemIsDone
	^current status = #done.
!

currentItemIsInProgress
	^current status = #visitedOnce.
!

currentItemIsNew
	^current status = #notVisited.
!

hasSomethingToProcess
	self next.
	^current notNil.
!

initialVisit
	| children |
	children := self currentChildren select: [:each  | each status == #notVisited].
	children do: [:each | each parent: current].
	queue addAll: children.
	current status: #visitedOnce.
	children isEmpty ifTrue: [queue add: current].
!

next
	queue isEmpty ifTrue: [
		| candidate |
		candidate := items detect: [:each | each status == #notVisited] ifNone: [nil].
		candidate isNil ifTrue: [
			"We're probably at the end, but check again for part-way done nodes, which can happen if there are heavy cycles"
			items reset. 
			candidate := items detect: [:each | each status == #visitedOnce] ifNone: [nil]].
		candidate isNil ifFalse: [queue add: candidate]].
	current := queue isEmpty ifTrue: [nil] ifFalse: [queue removeFirst].
!

processCurrent
	self currentItemIsNew ifTrue: [^self initialVisit].
	self currentItemIsInProgress ifTrue: [^self secondVisit].
	self currentItemIsDone ifTrue: [^self].
	self error: 'Unknown item status'.
!

secondVisit
	sorted add: current.
	current status: #done.
	current parent isNil
		ifFalse:
			[((self childrenOf: current parent)
				allSatisfy: [:each | each status == #done])
				ifTrue: [queue add: current parent].
			current parent: nil].
!

sort: aCollection
	items := aCollection readStream.
	aCollection do: [:each | each status: #notVisited].
	[self hasSomethingToProcess] whileTrue: [self processCurrent].
	^sorted.
! !

!GlorpRowSorter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpRowSorter.st,v 1.1 2006/07/26 08:41:26 boris Exp $'
! !
