"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#GlorpSession
	instanceVariableNames:'system currentUnitOfWork cache accessor applicationData reaper
		errorAction'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

GlorpSession comment:'This class has not yet been commented.  The comment should state the purpose of the class, what messages are subclassResponsibility, and the type and purpose of each instance and class variable.  The comment should also explain any unobvious aspects of the implementation.

Instance Variables:

	system	<ClassOfVariable>	description of variable''s function
	currentUnitOfWork	<ClassOfVariable>	description of variable''s function
	cache	<ClassOfVariable>	description of variable''s function
	accessor	<ClassOfVariable>	description of variable''s function
	application	<ClassOfVariable>	application-specific data'
!


!GlorpSession class methodsFor:'instance creation'!

forSystem: aSystem 
	^self new system: aSystem
!

new

	^super new initialize.
! !

!GlorpSession class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpSession methodsFor:'As yet unclassified'!

cacheFor: anObject
	^cache cacheFor: anObject.
!

cacheForClass: aClass
	^cache cacheForClass: aClass.
!

errorAction
	^errorAction
!

errorAction: aThreeArgumentBlockExceptionSessionCommand
	errorAction := aThreeArgumentBlockExceptionSessionCommand.
! !

!GlorpSession methodsFor:'accessing'!

accessor
	^accessor
!

accessor: aDatabaseAccessor 
	accessor := aDatabaseAccessor.
	system isNil ifFalse: [system platform: (accessor platform)].
!

addToReaper: aProxy

	aProxy class == TimedProxy ifFalse: [^self].
	self reaper add: aProxy.
!

applicationData
	^applicationData
!

applicationData: anObject	
	applicationData := anObject
!

platform
	^self system platform.
!

reaper

	reaper isNil ifTrue: [self initializeReaper].
	^reaper.
!

reusePreparedStatements: aBoolean
	accessor reusePreparedStatements: aBoolean.
!

system
	^system
! !

!GlorpSession methodsFor:'api'!

descriptorFor: aClassOrInstance

	^system descriptorFor: aClassOrInstance.
!

hasDescriptorFor: aClass

	^system hasDescriptorFor: aClass.
!

register: anObject 
	| realObject |
	currentUnitOfWork isNil ifTrue: [^self].
	realObject := self realObjectFor: anObject ifNone: [^self].

	(self isNew: realObject) 
		ifTrue: [currentUnitOfWork registerAsNew: realObject]
		ifFalse: [currentUnitOfWork register: realObject]
!

registerAll: aCollection
	aCollection do: [:ea | self register: ea].
!

registerAsNew: anObject 
	currentUnitOfWork isNil ifTrue: [^self].
	currentUnitOfWork registerAsNew: anObject.
!

registerAsOld: anObject 
	| realObject |
	currentUnitOfWork isNil ifTrue: [^self].
	realObject := self realObjectFor: anObject ifNone: [^self].

	currentUnitOfWork register: realObject
!

system: aSystem 
	aSystem session: self.
	accessor isNil ifFalse: [aSystem platform: (accessor currentLogin database)].
	system := aSystem.
	system validate.
! !

!GlorpSession methodsFor:'api/queries'!

count: aClass
	^self execute: (SimpleQuery count: aClass).
!

count: aClass where: anExpression
	^self execute: (SimpleQuery count: aClass where: anExpression).
!

delete: anObject
	| realObject |
	"Get the real object, instantiating if necessary"
	realObject := anObject glorpRealObject.
	self hasUnitOfWork
		ifTrue: [currentUnitOfWork delete: realObject]
		ifFalse:
			[self inUnitOfWorkDo: [
				currentUnitOfWork delete: realObject]].
!

deleteAll: aCollection

 	aCollection do: [:each | self delete: each].
!

execute: aQuery
	^aQuery executeWithParameters: #() in: self.
!

hasExpired: anObject 
	^cache hasExpired: anObject.
!

read: aClass
	^self execute: (SimpleQuery returningManyOf: aClass).
!

read: aClass limit: anInteger
	^self execute: (SimpleQuery returningManyOf: aClass limit: anInteger).
!

read: aClass orderBy: aCriteria
	| query |
	query := SimpleQuery returningManyOf: aClass.
	query orderBy: aCriteria.
	^self execute: query.
!

read: aClass where: aBlock
	^self execute: (SimpleQuery returningManyOf: aClass where: aBlock).
!

read: aClass where: aBlock limit: anInteger
	^self
		execute:
			(SimpleQuery
				returningManyOf: aClass
				where: aBlock
				limit: anInteger).
!

readManyOf: aClass
	^self execute: (SimpleQuery returningManyOf: aClass).
!

readManyOf: aClass limit: anInteger
	^self execute: (SimpleQuery returningManyOf: aClass limit: anInteger).
!

readManyOf: aClass orderBy: aCriteria
	| query |
	query := SimpleQuery returningManyOf: aClass.
	query orderBy: aCriteria.
	^self execute: query.
!

readManyOf: aClass where: aBlock
	^self execute: (SimpleQuery returningManyOf: aClass where: aBlock).
!

readManyOf: aClass where: aBlock limit: anInteger
	^self
		execute:
			(SimpleQuery
				returningManyOf: aClass
				where: aBlock
				limit: anInteger).
!

readOneOf: aClass
	^self execute: (SimpleQuery returningOneOf: aClass).
!

readOneOf: aClass where: aBlock
	^self execute: (Query returningOneOf: aClass where: aBlock).
!

refresh: anObject
	| exp query realObject descriptor |
	realObject := self realObjectFor: anObject ifNone: [^self].
	descriptor := self descriptorFor: realObject.
	descriptor isNil
		ifTrue: [self error: 'Cannot refresh an object with no descriptor'].
	exp := descriptor
		primaryKeyExpressionFor: realObject
		basedOn: BaseExpression new
		relation: #=.
	query := Query returningOneOf: realObject class where: exp.
	query shouldRefresh: true.
	^self execute: query.
! !

!GlorpSession methodsFor:'api/transactions'!

beginTransaction
	accessor beginTransaction.
!

beginUnitOfWork

	self hasUnitOfWork ifTrue: [self error: 'Cannot nest units of work yet'].
	currentUnitOfWork := UnitOfWork new.
	currentUnitOfWork session: self.
!

commitTransaction
	accessor commitTransaction
!

commitUnitOfWork
	[self isInTransaction
		ifTrue: [currentUnitOfWork commit]
		ifFalse: [self inTransactionDo: [currentUnitOfWork commit]]] 
	ifCurtailed: [self rollbackUnitOfWork].
	currentUnitOfWork := nil.
!

commitUnitOfWorkAndContinue
	"Commit the current unit of work, but then keep going with the same set of registered objects, with their state updated to reflect current values."
	| registeredObjects |	
	currentUnitOfWork isNil ifTrue: [^self error: 'Not in unit of work'].
	registeredObjects := currentUnitOfWork registeredObjects.
	currentUnitOfWork deletedObjects do: [:each | registeredObjects remove: each].
	self commitUnitOfWork.
	self beginUnitOfWork.
	self registerAll: registeredObjects.
!

hasUnitOfWork

	^currentUnitOfWork notNil.
!

inTransactionDo: aBlock
	| alreadyInTransaction result |
	[ 
	alreadyInTransaction := self isInTransaction.
	alreadyInTransaction ifFalse: [self beginTransaction].
	result := aBlock numArgs = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	alreadyInTransaction ifFalse: [self commitTransaction]]
		ifCurtailed:
			[alreadyInTransaction ifFalse: [self rollbackTransaction]].
	^result.
!

inUnitOfWorkDo: aBlock
	| result |
	[
	self beginUnitOfWork.
	result := aBlock numArgs = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	self commitUnitOfWork]
		ifCurtailed:
			[self rollbackUnitOfWork].
	^result.
!

isInTransaction
	^accessor isInTransaction.
!

modify: anObject in: aBlock
	"Evaluate aBlock inside a unit of work. Start a database transaction at the beginning and commit it at the end. If we don' terminate normally, roll everything back. Register anObject before starting"
	self transact: [
		self register: anObject.
		aBlock value].
!

rollbackTransaction
	accessor execute: [ accessor rollbackTransaction].
!

rollbackUnitOfWork

	currentUnitOfWork isNil ifTrue: [^self].
	currentUnitOfWork abort.
	currentUnitOfWork := nil.
!

rollbackUnitOfWorkFor: anObject
	currentUnitOfWork abort: anObject
!

saveAndContinue
	"A shorter synonym"
	^self commitUnitOfWorkAndContinue.
!

transact: aBlock
	"Evaluate aBlock inside a unit of work. Start a database transaction at the beginning and commit it at the end. If we don' terminate normally, roll everything back.  This might more consistently be  called inUnitOfWorkWithTransactionDo:, but that's too verbose"
	| alreadyInTransaction alreadyInUnitOfWork result |
	[[
	alreadyInUnitOfWork := false.
	alreadyInTransaction := self isInTransaction.
	alreadyInTransaction ifFalse: [self beginTransaction].
	alreadyInUnitOfWork := self hasUnitOfWork.
	alreadyInUnitOfWork ifFalse: [self beginUnitOfWork].
	result := aBlock numArgs = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	alreadyInUnitOfWork ifFalse: [self commitUnitOfWork].
	alreadyInTransaction ifFalse: [self commitTransaction]]
		ifCurtailed:
			[alreadyInUnitOfWork ifFalse: [self rollbackUnitOfWork]]]
			ifCurtailed: [alreadyInTransaction ifFalse: [self rollbackTransaction]].
	^result.
! !

!GlorpSession methodsFor:'caching'!

cacheAt: aKey  forClass: aClass ifNone: failureBlock

	^cache lookupClass: aClass key: aKey ifAbsent: failureBlock.
!

cacheAt: keyObject put: valueObject 
	^cache at: keyObject insert: valueObject
!

cacheContainsObject: anObject key: aKey 
	"Just test containment, don't return the result or trigger anything due to expiration"
	aKey isNil ifTrue: [^false].
	^cache
		containsObject: anObject
		key: aKey.
!

cacheContainsObjectForClass: aClass key: aKey 
	"Just test containment, don't return the result or trigger anything due to expiration"
	aKey isNil ifTrue: [^false].
	^cache
		containsObjectForClass: aClass
		key: aKey.
!

cacheContainsObjectForRow: aDatabaseRow
	^self 
		cacheContainsObjectForClass: aDatabaseRow owner class
		key: aDatabaseRow primaryKey
!

cacheLookupForClass: aClass key: aKey 
	^self 
		cacheAt: aKey
		forClass: aClass
		ifNone: [nil]
!

cacheLookupObjectForRow: aDatabaseRow
	^self 
		cacheLookupForClass: aDatabaseRow owner class
		key: aDatabaseRow primaryKey
!

cacheRemoveObject: anObject
	| key |
	key := (self descriptorFor: anObject) primaryKeyFor: anObject.
	cache removeClass: anObject class key: key ifAbsent: [].
!

hasExpired: aClass key: key
	^cache hasExpired: aClass key: key.
!

hasObjectExpiredOfClass: aClass withKey: key
	^cache hasObjectExpiredOfClass: aClass withKey: key.
!

isRegistered: anObject

	currentUnitOfWork isNil ifTrue: [^false].
	^currentUnitOfWork isRegistered: anObject.
!

lookupRootClassFor: aClass
	| descriptor |
	descriptor := self system descriptorFor: aClass.
	^descriptor notNil
			ifTrue: [ descriptor typeMappingRootDescriptor describedClass ]
			ifFalse: [ aClass ]
! !

!GlorpSession methodsFor:'copying'!

copy

	^self shallowCopy postCopy.
!

postCopy

	super postCopy.
	self initializeCache.
	currentUnitOfWork := nil.
	accessor := accessor copy.
! !

!GlorpSession methodsFor:'events'!

sendPostFetchEventTo: anObject
	^anObject glorpPostFetch: self.
!

sendPostFetchValidateEventTo: anObject
	^anObject glorpPostFetchValidate: self.
!

sendPostWriteEventTo: anObject
	anObject glorpPostWrite: self.
!

sendPreWriteEventTo: anObject
	anObject glorpPreWrite: self.
! !

!GlorpSession methodsFor:'initialize'!

initialize
	
	self initializeCache.
!

initializeCache

	cache release.
	cache := CacheManager forSession: self.
!

initializeReaper

	self releaseReaper.
	reaper := TimedProxyReaper new.
!

releaseReaper

	reaper release.
!

reset

	self initializeCache.
	self releaseReaper.
	accessor reset.
	currentUnitOfWork := nil.
!

resetPermissions
	accessor isNil ifFalse: [accessor resetPermissions].
! !

!GlorpSession methodsFor:'internal/writing'!

commandForHomogeneousRows: aCollectionOfDatabaseRows
	"Create the command for this set of rows. These are assumed to be homogeneous, in that they are all for the same table, and are all the same type of operation (i.e. all insert or all update)"
	| representativeRow |
	representativeRow := aCollectionOfDatabaseRows first.
	representativeRow forDeletion ifTrue: [self error: 'invalid way of handling deletes (this should never happen)'].
	^(self shouldInsert: representativeRow)
		ifTrue: [^InsertCommand forRows: aCollectionOfDatabaseRows useBinding: self useBinding platform: self platform]
		ifFalse: [^UpdateCommand forRows: aCollectionOfDatabaseRows useBinding: self useBinding platform: self platform].
!

commandForRow: aDatabaseRow 
	
	(aDatabaseRow forDeletion) ifTrue: [^DeleteCommand forRow: aDatabaseRow useBinding: self useBinding platform: self platform].
	^(self shouldInsert: aDatabaseRow)
		ifTrue: [^InsertCommand forRow: aDatabaseRow useBinding: self useBinding platform: self platform]
		ifFalse: [^UpdateCommand forRow: aDatabaseRow useBinding: self useBinding platform: self platform].
!

createDeleteRowsFor: anObject in: rowMap
	"Create records for rows that require deletion"

	(self descriptorFor: anObject) createDeleteRowsFor: anObject in: rowMap.
!

createRowsFor: anObject in: rowMap
	| descriptor |
	(descriptor := self descriptorFor: anObject class) isNil
		ifFalse: [descriptor createRowsFor: anObject in: rowMap].
!

shouldInsert: aDatabaseRow 
	(currentUnitOfWork notNil and: [currentUnitOfWork forcesUpdateFor: aDatabaseRow]) ifTrue: [^false].
	^(self cacheContainsObjectForRow: aDatabaseRow) not.
!

tablesInCommitOrder

	^(TableSorter for: system allTables) sort.
! !

!GlorpSession methodsFor:'login'!

isLoggedIn
	^accessor isLoggedIn.
!

login
	| result |
	result := accessor login.
	system platform characterEncoding: accessor encoding.
	^result.
!

loginIfError: aBlock
	| result |
	result := self accessor loginIfError: aBlock.
	system platform characterEncoding: accessor encoding.
	^result.
!

logout
	^self accessor logout.
! !

!GlorpSession methodsFor:'private'!

embeddedObjectsFor: anObject
	"Return all the objects embedded into this one"
	| embedded descriptor |
	descriptor := self descriptorFor: anObject.
	descriptor isNil ifTrue: [^#()].
	embedded := OrderedCollection new: 3.
	descriptor mappings do: [:each |
		(each class == EmbeddedValueOneToOneMapping) ifTrue: [
			embedded add: (each getValueFrom: anObject)]].
	^embedded.
!

expiredInstanceOf: aClass key: key

	^cache expiredInstanceOf: aClass key: key.
!

markAsCurrentOfClass: aClass key: key

	cache markAsCurrentOfClass: aClass key: key.
!

privateGetCache

	^cache.
!

privateGetCurrentUnitOfWork

	^currentUnitOfWork.
!

realObjectFor: anObject 
	"If this is a proxy, return the contents (if available). Otherwise, return nil"
	^self realObjectFor: anObject ifNone: [nil].
!

realObjectFor: anObject ifNone: aBlock
	"If this is a proxy, return the contents (if available). Otherwise, evalute the block"
	^anObject isGlorpProxy 
		ifTrue: [anObject isInstantiated ifTrue: [anObject getValue] ifFalse: [aBlock value]]
		ifFalse: [anObject]
! !

!GlorpSession methodsFor:'read/write'!

filterDeletionFrom: anObject

	self hasUnitOfWork ifFalse: [^anObject].
	currentUnitOfWork shouldFilterDeletions ifFalse: [^anObject].
	(currentUnitOfWork willDelete: anObject)
		ifTrue: [^nil].
	^anObject.
!

groupWrite: aCollectionOfDatabaseRows
	| command |
	aCollectionOfDatabaseRows
		do: [:each | each preWriteAssignSequencesUsing: self accessor].
	command := self commandForHomogeneousRows: aCollectionOfDatabaseRows.
	accessor executeCommand: command.
	^command succeeded
		ifTrue: [true]
		ifFalse:
			[((GlorpWriteFailure new)
				command: command;
				object: (aCollectionOfDatabaseRows collect: [:each | each owner]))
				signal].
!

writeHomogeneousRows: aCollectionOfDatabaseRows
	"Write out a collection of rows belonging to a single table"

	| representativeRow succeeded maximumSize start |

	aCollectionOfDatabaseRows isEmpty ifTrue: [^true].
	representativeRow := aCollectionOfDatabaseRows first.
	representativeRow shouldBeWritten ifFalse: [^true].
	succeeded := true.
	maximumSize := self system maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows.
	start := 1.
	[start > aCollectionOfDatabaseRows size] whileFalse: [
		| end |
		end := (start + maximumSize - 1) min: aCollectionOfDatabaseRows size.
		succeeded := succeeded
			& (self writeHomogeneousRows: aCollectionOfDatabaseRows from: start to: end).
		start := end + 1].
	^succeeded.
!

writeHomogeneousRows: aCollectionOfDatabaseRows from: startIndex to: endIndex
	"Write out the specified range from a collection of rows belonging to a single table"

	| succeeded |
	succeeded := true.
	^(self system 	
			canUseGroupedWritesFor: (aCollectionOfDatabaseRows at: startIndex) table)
		ifTrue: [self groupWrite: (aCollectionOfDatabaseRows copyFrom: startIndex to: endIndex)]
		ifFalse:
			[startIndex to: endIndex do: [:i |
				 |eachRow rowSucceeded |
				eachRow := aCollectionOfDatabaseRows at: i.
				rowSucceeded := self writeRow: eachRow.
				succeeded := succeeded & rowSucceeded].
			succeeded].
!

writeRow: aDatabaseRow
	| command success |
	aDatabaseRow shouldBeWritten ifFalse: [^true].
	aDatabaseRow preWriteAssignSequencesUsing: self accessor.
	command := self commandForRow: aDatabaseRow.
	accessor executeCommand: command.
	success := true.
	command succeeded
		ifFalse:
			[success := (GlorpWriteFailure new)
				command: command;
				object: (Array with: aDatabaseRow owner);
				signal].
	^success.
! !

!GlorpSession methodsFor:'testing'!

isNew: anObject 
	"When registering, do we need to add this object to the collection of new objects? New objects are treated specially when computing what needs to be written, since we don't have their previous state"
	"This will break for composite keys (Really? Why? Do we need to test that if the key is a collection with a nil value, rather than just nil?)"

	| key descriptor |
	(currentUnitOfWork isRegistered: anObject) ifTrue: [^false].
	descriptor := self descriptorFor: anObject.
	descriptor isNil ifTrue: [^false].
	"For embedded values we assume that they are not new. This appears to work. I can't really justify it."
	self needsWork: 'cross your fingers'.
	descriptor mapsPrimaryKeys ifFalse: [^false].

	key := descriptor primaryKeyFor: anObject.
	key isNil ifTrue: [^true].
	^(self cacheContainsObject: anObject key: key) not.
!

isUninstantiatedProxy: anObject
	^anObject isGlorpProxy and: [anObject isInstantiated not].
!

reusePreparedStatements
	^accessor reusePreparedStatements
!

useBinding
	^self accessor useBinding.
! !

!GlorpSession methodsFor:'virtual collections'!

virtualCollectionOf: aClass
	^GlorpVirtualCollection on: aClass in: self.
! !

!GlorpSession class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpSession.st,v 1.1 2006/07/26 08:03:46 boris Exp $'
! !
