"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Collection subclass:#GlorpVirtualCollection
	instanceVariableNames:'query session realObjects parameters'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpQueries'
!

GlorpVirtualCollection comment:'This represents a virtual collection, i.e. one that we haven''t really read into memory yet. It responds to a reasonable subset of collection protocol, and will read the elements into memory only when necessary. So, e.g. a select: operation takes a query block, and is equivalent to AND:ing that query block to the main query.

To create a virtual collection, ask the session for one. e.g. session virtualCollectionOf: AClass.

This is an initial version which will read in the objects fairly eagerly. An optimization might be to defer certain types of operations depending on whether the block can be evaluated into SQL or not. e.g.
  collect: [:each | each name]
can be turned into a retrieve: operation. But 
  collect: [:each | each printString]
cannot. We could try to check the block for operations like collect: and detect:, deferring the point at which the objects will be read in.

Handling of ordering is also a little bit funny. The blocks we like for ordering aren''t compatible with sortedCollection type blocks. It''d be nice to be more compatible.

'
!


!GlorpVirtualCollection class methodsFor:'instance creation'!

on: aClass in: aSession
	^self new on: aClass in: aSession.
! !

!GlorpVirtualCollection methodsFor:'accessing'!

orderBy: aBlockOrExpression
	query orderBy: aBlockOrExpression.
!

parameters: aCollection
	parameters := aCollection
!

setQuery: aQuery
	"Semi-private. Set the internal query"
	query := aQuery.
!

setSession: aSession
	"Semi-private. Set the session"
	session := aSession.
!

size
	^self realObjects size.
! !

!GlorpVirtualCollection methodsFor:'adding'!

add: newObject
	self readOnlyError.
! !

!GlorpVirtualCollection methodsFor:'copying'!

copy
	^self shallowCopy postCopy.
!

copyAndGrowBy: anInteger
	"For VA compatibility"
	^super glorpShallowCopy.
!

postCopy
	query := query copy.
	realObjects := nil.
! !

!GlorpVirtualCollection methodsFor:'enumerating'!

collect: aBlock
	^self realObjects collect: aBlock
!

do: aBlock
	self realObjects do: aBlock.
!

reject: aBlock
	^self copy AND: [:each | (aBlock value: each) not].
!

select: aBlock
	^self isInstantiated ifTrue: [self realObjects select: aBlock] ifFalse: [self copy AND: aBlock].
! !

!GlorpVirtualCollection methodsFor:'initialize-release'!

on: aClass in: aSession
	query := Query returningManyOf: aClass.
	session := aSession.
! !

!GlorpVirtualCollection methodsFor:'printing'!

printOn: aStream
	self isInstantiated
		ifTrue: [super printOn: aStream]
		ifFalse:
			[aStream nextPutAll: 'a virtual collection of '.
			query notNil ifTrue: [aStream nextPutAll: query resultClass name]].
! !

!GlorpVirtualCollection methodsFor:'private'!

AND: aBlock
	query AND: aBlock.
!

getQuery
	^query.
!

readOnlyError
	self error: 'Virtual collections are read-only'.
!

realObjects
	realObjects isNil ifTrue: [	realObjects := query executeWithParameters: parameters in: session].
	^realObjects.
! !

!GlorpVirtualCollection methodsFor:'removing'!

remove: oldObject ifAbsent: anExceptionBlock
	self readOnlyError.
! !

!GlorpVirtualCollection methodsFor:'testing'!

isEmpty
	^self realObjects isEmpty.
!

isInstantiated
	^realObjects notNil.
! !

!GlorpVirtualCollection class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__GlorpVirtualCollection.st,v 1.1 2006/07/26 08:51:39 boris Exp $'
! !
