"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

FunctionExpression subclass:#InfixFunction
	instanceVariableNames:'argument'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpExpressions'
!


!InfixFunction methodsFor:'accessing'!

argument: anObject
	argument := anObject.
!

arguments: anArray
	anArray isEmpty ifFalse: [argument := anArray first].
! !

!InfixFunction methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree. Keep track of who has already been visited, so we don't get trapped in cycles or visit nodes twice."

	(aSet includes: self) ifTrue: [^self].
	super do: aBlock skipping: aSet.
	argument do: aBlock skipping: aSet.
! !

!InfixFunction methodsFor:'preparing'!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."
	
	| newArgument |
	newArgument := argument isNil ifTrue: [nil] ifFalse: [argument asExpressionJoiningSource: source toTarget: target].
	^(super asExpressionJoiningSource: source toTarget: target) argument: newArgument.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	| rebuilt |
	self == anExpression ifTrue: [^aBaseExpression].
	rebuilt := (base rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression) get: self symbol withArguments: (Array with: (argument rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression)).
	rebuilt alias: self alias.
	^rebuilt.
! !

!InfixFunction methodsFor:'printing'!

printSQLOn: aStream withParameters: aDictionary
	base printSQLOn: aStream withParameters: aDictionary.
	self printsIntoJoins ifFalse: [^self].
	aStream 
		nextPut: $ ;
		nextPutAll: function;
		nextPut: $ .
	argument printSQLOn: aStream withParameters: aDictionary.
!

printTreeOn: aStream 
	base printOn: aStream.
	aStream
		nextPutAll: ' ', function, ' '.
	argument printOn: aStream.
!

printUnqualifiedSQLOn: aStream withParameters: aDictionary
	base printUnqualifiedSQLOn: aStream withParameters: aDictionary.
	aStream 
		nextPut: $ ;
		nextPutAll: function;
		nextPut: $ .
	argument printUnqualifiedSQLOn: aStream withParameters: aDictionary.
! !

!InfixFunction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__InfixFunction.st,v 1.1 2006/07/26 08:47:57 boris Exp $'
! !
