"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

NamedSequence subclass:#JustSelectTheMaximumSequenceValueAndAddOne
	instanceVariableNames:'field table'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

JustSelectTheMaximumSequenceValueAndAddOne comment:'This is a sequence that just does a select max(primaryKeyFieldName) for the table in question and adds one to it. This is, um, less-than-perfectly efficient, and I''m not at all clear that it''ll work for a multi-user system. But it''s what Store does on SQL Server, so we''d like to be able to mimic it. And it''s actually easier to preallocate for than identity columns, which is nice. This isn''t really named in the same sense as the others, but it''s useful to inherit the named preallocation behaviour, and we store the field to select against in the name.

Instance Variables:
	tableName	<DatabaseTable>	the table we sequence.
	field <DatabaseField> the field we sequence.'
!


!JustSelectTheMaximumSequenceValueAndAddOne class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2003 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!JustSelectTheMaximumSequenceValueAndAddOne methodsFor:'accessing'!

field
	field isNil ifTrue: [field := table primaryKeyFields first].
	^field.
!

field: aDatabaseField
	field := aDatabaseField.
!

table: aTable
	table := aTable.
! !

!JustSelectTheMaximumSequenceValueAndAddOne methodsFor:'other'!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession

	| stream rows value |
	"I repeat: ick"
	stream := WriteStream on: (String new: 50).
	stream nextPutAll: 'SELECT MAX('.
	aField printSQLOn: stream withParameters: #().
	stream nextPutAll: ') FROM '.
	aRow table printSQLOn: stream withParameters: #().
	rows := aSession accessor executeSQLString: stream contents.
	value := rows first first isNil ifTrue: [1] ifFalse: [rows first first + 1].
	aRow at: aField put: value.
	^value.
!

reserveSequenceNumbers: anInteger in: anAccessor for: aTable
	| numberNeeded firstNumber fakeRow |
	numberNeeded := anInteger - self availableReservedNumbers.
	numberNeeded <= 0 ifTrue: [^self].
	fakeRow := DatabaseRow newForTable: self field table.
	firstNumber := self
		getSequenceValueFromDatabaseFor: self field
		in: fakeRow
		using: anAccessor.
	self reservedNumbers addAll: (firstNumber to: firstNumber + numberNeeded - 1).
! !

!JustSelectTheMaximumSequenceValueAndAddOne class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__JustSelectTheMaximumSequenceValueAndAddOne.st,v 1.1 2006/07/26 08:15:09 boris Exp $'
! !
