"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#MultipleRowMapKey
	instanceVariableNames:'keys'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpUnitOfWork'
!

MultipleRowMapKey comment:'This is a special (and rarely needed) form of row map key that allows an arbitrary number of objects to participate in it.'
!


!MultipleRowMapKey class methodsFor:'instance creation'!

new
	"Answer a newly created and initialized instance."

	^super new initialize
!

with: key1 with: key2 with: key3
	(key1 isNil or: [key2 isNil or: [key3 isNil]]) ifTrue: [self error: 'Nil key in row map, probably indicates an invalid object structure'].
	^self new 
		addKey: key1;
		addKey: key2;
		addKey: key3;
		yourself.
!

withAll: aCollection
	
	^self new 
		addAllKeys: aCollection;
		yourself.
! !

!MultipleRowMapKey methodsFor:'accessing'!

addAllKeys: aKeyCollection
	keys addAll: aKeyCollection.
!

addKey: aKey
	keys add: aKey.
!

keys
	^keys.
!

keys: aCollection
	keys := (aCollection asSortedCollection: self sortBlock).
! !

!MultipleRowMapKey methodsFor:'adjusting'!

adjustIn: aCorrespondenceMap
	"Adjust ourselves to refer to the copies rather than the originals"
	| adjusted |
	adjusted := self copy.
	adjusted keys: (keys collect: [:each | aCorrespondenceMap at: each ifAbsent: [each]]).
	^adjusted.
!

reverseAdjustIn: aRowMapForMementos
	"Adjust ourselves to refer to the originals rather than the copies"
	| adjusted |
	adjusted := self copy.
	adjusted keys: (keys collect: [:each | 
		aRowMapForMementos reversedCorrespondenceMap at: each ifAbsent: [each]]).
	^adjusted.
! !

!MultipleRowMapKey methodsFor:'comparing'!

= aRowMapKey

	aRowMapKey class == self class ifFalse: [^false].
	aRowMapKey keys size = keys size ifFalse: [^false].
	"We can't rely on the order being the same because of the obscure but real case of two different related objects with the same identityHash"
	aRowMapKey keys do: [:each |
		(keys includes: each) ifFalse: [^false]].
	^true.
!

hash
	"Note: the two things with the same identity hash problem is OK here because if they do then the hashes are (by definition) the same, and the order doesn't matter"
	| increment hash |
	increment := 14 // keys size.
	hash := 0.
	keys doWithIndex: [:each :index |
		hash := hash bitXor: (each identityHash bitShift: increment * index)].
	^hash.
! !

!MultipleRowMapKey methodsFor:'initialize-release'!

initialize
	keys := (SortedCollection new: 4) sortBlock: self sortBlock.
!

sortBlock
        ^[:a :b | a identityHash < b identityHash].
! !

!MultipleRowMapKey methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: 'KEY('.
	self hash printOn: aStream.
	aStream nextPutAll: '):'.
	keys printOn: aStream.
! !

!MultipleRowMapKey class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__MultipleRowMapKey.st,v 1.1 2006/07/26 08:53:40 boris Exp $'
! !
