"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

ElementBuilder subclass:#ObjectBuilder
	instanceVariableNames:'queryCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

ObjectBuilder comment:'This builds full-blown persistent objects with descriptors. This is the most common type of builder.'
!


!ObjectBuilder class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ObjectBuilder methodsFor:'accessing'!

cachedQueryFor: aMapping
	^self queryCache at: aMapping ifAbsent: [nil].
!

cachedQueryFor: aMapping is: aQuery
	"Save a query which we will use when constructing filtered queries"
	self queryCache at: aMapping put: aQuery.
!

canCauseDuplicateRows
	^expression class == MappingExpression and: [expression mapping isToManyRelationship].
!

command
	^cursor command.
!

descriptor
	
	^expression descriptor.
!

instance: anObject
	instance := anObject
!

key
	^self primaryKey.
!

primaryKey
	"We use self as a special guard value to indicate that the value hasn't changed"
	key == self ifFalse: [^key].
	self canCache ifFalse: [^nil].
	key := self descriptor keyFields
		collect: [:each | self valueOfField: each].
	key size = 1 ifTrue: [key := key first].
	^key.
!

queryCache
	queryCache isNil ifTrue: [queryCache := IdentityDictionary new: 5].
	^queryCache.
!

requiresPopulating
	^requiresPopulating and: [self returnProxies not].
!

requiresPopulating: aBoolean
	requiresPopulating := aBoolean
!

returnProxies
	^query returnProxies.
!

returnValueIn: aCursoredStream
	"If we're a to-many relationship, what we retrieve: is the collection containing us, not the individual items"
	| relatedBuilder |
	expression canKnit ifFalse: [^self instance].
	expression mapping isToManyRelationship ifFalse: [^self instance].
	relatedBuilder := aCursoredStream elementBuilderFor: expression base.
	(relatedBuilder isNil or: [relatedBuilder instance isNil]) ifTrue: [^self instance].
	^expression mapping getValueFrom: relatedBuilder instance.
! !

!ObjectBuilder methodsFor:'building objects'!

buildObjectFrom: anArray
	self row: anArray.
	self requiresPopulating ifTrue: [self populateInstance].
	(self system shouldResetExpiryTimeForObjectOfClass: instance class readFrom: self query)
		ifTrue: [self session markAsCurrentOfClass: instance class key: self key].
!

canBuild
	"If we have a regular object with a nil primary key, or if we have an embedded object whose values are all nil, we can't build anything (probably due to an outer join)"
	^self descriptor mapsPrimaryKeys
		ifTrue: [self key notNil]
		ifFalse: [true].
!

canCache
	| keyFields |
	keyFields := self descriptor keyFields.
	^keyFields notNil and: [keyFields notEmpty].
!

findInstanceForRow: aRow proxyType: proxyType
	instance := nil.
	self row: aRow.
	self canBuild ifFalse: [^self].
	(self shouldProxyInstanceFor: proxyType)
		ifTrue: [^self newProxyWithNewQueryForClass: proxyType].
	self lookupCachedObject.
	self instanceExistsButIsDeleted
		ifTrue:
			[instance := nil.
			^self].
	instance isNil
		ifFalse:
			[requiresPopulating := requiresPopulating | query shouldRefresh.
			^self].
	requiresPopulating := true.
	instance := (expression descriptor
		describedConcreteClassFor: self row
		withBuilder: self) basicNew.
	self canCache ifTrue: [self session cacheAt: self key put: instance].
!

instanceDescriptor
	"Return the descriptor for the instance we're going to build. Note that we have to look it up by instance in order to find subclass. "
	| basicDescriptor |
	instance isNil ifTrue: [^nil].
	basicDescriptor := self system descriptorFor: instance.
	basicDescriptor isNil ifFalse: [^basicDescriptor].
	^expression descriptor.
!

instanceExistsButIsDeleted
	instance isNil ifTrue: [^false].
	instance := self session filterDeletionFrom: instance.
	^instance isNil.
!

knitResultIn: aCursoredStream 
	"Connect up our built object with any other objects that reference it. Used if we retrieve more than one thing in the same query"

	| relatedBuilder |
	expression canKnit ifFalse: [^self].
	relatedBuilder := aCursoredStream objectBuilderFor: expression base.
	(relatedBuilder isNil or: [relatedBuilder instance isNil]) 
		ifFalse: 
			[expression mapping knitFromBuilder: relatedBuilder toBuilder: self]
!

lookupCachedObject
	| resultClass |
	self canBuild ifFalse: [^self].
	self canCache
		ifTrue:
			[
			resultClass := expression descriptor describedClass.
			(self session hasExpired: resultClass key: self key)
				ifTrue:
					[instance := self session expiredInstanceOf: resultClass key: self key.
					requiresPopulating := true.
					isExpired := true]
				ifFalse:
					[instance := self session cacheLookupForClass: resultClass key: self key.
					requiresPopulating := instance isNil]].
!

newProxyWithNewQueryForClass: aClass
	| parameters |
	parameters := IdentityDictionary new.
	self descriptor primaryTable primaryKeyFields
		do:
			[:eachField | parameters at: eachField put: (self valueOfField: eachField in: row)].
	instance := aClass new
		session: self session;
		parameters: parameters;
		query: self queryForNewProxy.
	self session addToReaper: instance.
	^self.
!

populateInstance
	key isNil ifTrue: [^self].
	self instanceDescriptor populateObject: instance inBuilder: self.
!

queryForNewProxy
	"Create a proxy with a primary key query in which the parameters are the primary key fields. We can't just reuse a mapping's query, it has to be a new primary key one, because the mapping's query has just run and been told to return proxies. This kind of stinks"
	| whereExpression |
	whereExpression := Join new.
	self descriptor primaryTable primaryKeyFields do: [:eachField | 
		whereExpression addSource: eachField target: eachField].
	^Query
		returningOneOf: query resultClass 
		where: whereExpression
!

shouldProxyInstanceFor: proxyType
	^proxyType notNil.
! !

!ObjectBuilder methodsFor:'copying'!

postCopy
	super postCopy.
	queryCache := nil.
! !

!ObjectBuilder methodsFor:'initializing'!

initialize

	requiresPopulating := false.
	isExpired := false.
! !

!ObjectBuilder methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	expression printOn: aStream.
	aStream nextPut: $).
! !

!ObjectBuilder methodsFor:'selecting fields'!

fieldsForSelectStatement
	^self fieldsFromThePerspectiveOfTheMainSelect: (self fieldsFromMyPerspective).
!

fieldsFromMyPerspective
	^query returnProxies 
		ifTrue: [self descriptor table primaryKeyFields]
		ifFalse: [self descriptor fieldsForSelectStatement]
!

fieldsFromThePerspectiveOfTheMainSelect: aCollection
	^expression translateFields: aCollection.
! !

!ObjectBuilder methodsFor:'testing'!

buildsObjects
	^true.
! !

!ObjectBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__ObjectBuilder.st,v 1.1 2006/07/26 08:18:49 boris Exp $'
! !
