"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabasePlatform subclass:#OcelotPlatform
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!OcelotPlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!OcelotPlatform methodsFor:'constants'!

areSequencesExplicitlyCreated

^false
!

databaseSequenceClass
	^InMemorySequence.
!

primaryKeysAreAutomaticallyUnique

	^true.
!

sqlTextForNULLAttributeConstraint
	"^<String>"

	^''
!

supportsConstraints

	^true
!

supportsMillisecondsInTimes
	"I'm guessing here"
	^true.
!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^false.
! !

!OcelotPlatform methodsFor:'conversion times'!

printDate: aDate for: aType
	| stream |
	aDate isNil ifTrue: [^nil].
	stream := WriteStream on: String new.
	stream nextPutAll: 'date '''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''''.
	^stream contents.
!

printTime: aTime for: aType
	| stream |
	aTime isNil ifTrue: [^nil].
	stream := WriteStream on: String new.
	stream nextPutAll: 'time '''.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPutAll: ''''.
	^stream contents.
!

printTimestamp: aTimestamp on: stream for: aType

	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: 'timestamp '''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $ .
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	"	milliseconds: true."
	stream nextPutAll: ''''.
! !

!OcelotPlatform methodsFor:'conversion-strings'!

padString: aString for: aType 
	| padding |
	aString isNil ifTrue: [^nil].
	(self usesNullForEmptyStrings and: [aString isEmpty]) ifTrue: [^nil].
	aString size > aType width 
				ifTrue: [^aString copyFrom: 1 to: aType width].
				
	aType isVariableWidth ifTrue: [^aString].
	padding := String new: aType width - aString size.
	padding atAllPut: 32 asCharacter.
	^aString , padding
!

paddingCharacter
	^32 asCharacter.
!

unpadString: aString for: aType

	aString isNil ifTrue: [^nil].
	^aType isVariableWidth 
		ifTrue: [aString]
		ifFalse: [(ReadStream on: aString) upTo: 32 asCharacter].
! !

!OcelotPlatform methodsFor:'testing'!

isODBCPlatform

	^true
!

isOcelotPlatform

	^true
!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"

	^true
!

usesArrayBindingRatherThanGrouping
	^false.
! !

!OcelotPlatform methodsFor:'types'!

boolean
	^self typeNamed: #boolean ifAbsentPut: [GlorpBooleanType new typeString: 'boolean'].
!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'clob'].
!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new].
!

double
	^self float8
!

float
	^self float4.
!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new typeString: 'float'].
!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new typeString: 'double precision'].
!

int2
	^self smallint.
!

int4

	^self integer.
!

int8

	^self integer
!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].
!

sequence

	^self inMemorySequence
!

serial

	^self sequence
!

text
	^self clob.
!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time'].
!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'timestamp'].
!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].
! !

!OcelotPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__OcelotPlatform.st,v 1.1 2006/07/26 08:16:08 boris Exp $'
! !
