"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

NamedSequence subclass:#OracleSequence
	instanceVariableNames:'count sequenceIncrement accessProtect'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!OracleSequence class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!OracleSequence methodsFor:'accessing'!

name: aString
	name := aString copyFrom: 1 to: (aString size min: OraclePlatform new maximumLengthOfTableName).
!

sequenceIncrement
	^sequenceIncrement
!

sequenceIncrement: aNumber 
	sequenceIncrement := aNumber
! !

!OracleSequence methodsFor:'initalize-release'!

initialize
	super initialize.
	count := 0.
	sequenceIncrement := 1.
	"This seems to hang on the semaphore in ObjectStudio. Fake around it"
	accessProtect := Dialect isObjectStudio ifTrue: [nil] ifFalse: [Semaphore forMutualExclusion].
! !

!OracleSequence methodsFor:'private'!

backupReserveCommand: anInteger in: aSession
	| command stream |
	stream := WriteStream on: (String new: 200).
	stream nextPutAll: 'select '.
	stream nextPutAll: self qualifiedName.
	stream nextPutAll: '.nextval from SYS.ALL_OBJECTS '.
	stream nextPutAll: ' WHERE rownum <= '.
	stream
		nextPutAll:
			(aSession useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
	command := SQLStringSelectCommand new.
	command setSQLString: stream contents.
	command parameters: (Array with: anInteger).
	command platform: aSession platform.
	^command.
!

nextSequenceNumber: anAccessor 
	accessProtect critical: 
			[count := count + 1.
			^(self alwaysGetFromDb or: [count \\ sequenceIncrement = 1]) 
				ifTrue: 
					[count := (anAccessor 
								executeSQLString: self nextSequenceNumberSelectString) first 
								first]
				ifFalse: [count]]
!

nextSequenceNumberSelectString
	^'SELECT ' , self qualifiedName , '.NEXTVAL FROM DUAL'
!

reserveViaBackupMechanism: anInteger in: aSession
	"The regular mechanism didn't work, (probably because there weren't enough rows in the table), try a different technique. Oracle doesn't repeatedly evaluate nextval if we do it multiple times on a line, so instead guess at a table we know should have lots of rows in Oracle and use it"

	| rows command |
	command := self backupReserveCommand: anInteger in: aSession.
	command blockFactor: anInteger.
	rows := aSession accessor executeCommand: command.
	rows do: [:each | self reservedNumbers add: each first].
!

reserveViaTableSelect: anInteger in: aSession for: aTable
	"In Oracle, there can be performance issues selecting against our tables, which we don't seem to get if we just use SYS.ALLOBJECTS. I'm suspicious the problem is if we have fragmentation of the file containing the table, but regardless, ALLOBJECTS seems consistently fast, so always use it"
	self reserveViaBackupMechanism: anInteger in: aSession.

"	| rows command |
	command := self
		tableSelectCommandFor: aTable
		in: aSession
		reserving: anInteger.
	command parameters: (Array with: anInteger).
	rows := aSession accessor executeCommand: command.
	rows do: [:each | self reservedNumbers add: each first]."
!

tableSelectCommandFor: aTable in: aSession reserving: anInteger
	| stream |
	(tableSelectCommand notNil and: [tableSelectCommand parameters includes: anInteger]) ifFalse:
			[stream := WriteStream on: (String new: 200).
			stream nextPutAll: 'select '.
			stream nextPutAll: self qualifiedName.
			stream nextPutAll: '.nextval from '.
			stream nextPutAll: aTable name.
			stream nextPutAll: ' WHERE rownum <= '.
			stream nextPutAll: (aSession useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
			tableSelectCommand := SQLStringSelectCommand new.
			tableSelectCommand parameters: (Array with: anInteger).
			tableSelectCommand setSQLString: stream contents.
			tableSelectCommand blockFactor: anInteger.
			tableSelectCommand platform: aSession platform].
	^tableSelectCommand.
! !

!OracleSequence methodsFor:'public'!

creationString

	^'create sequence ', self qualifiedName.
!

getSequenceValueFromDatabaseFor: aField in: aRow using: anAccessor 
	aRow at: aField put: (self nextSequenceNumber: anAccessor)
!

postWriteAssignSequenceValueFor: aField in: aRow using: aSession
! !

!OracleSequence methodsFor:'testing'!

alwaysGetFromDb
	^sequenceIncrement = 1
! !

!OracleSequence class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__OracleSequence.st,v 1.1 2006/07/26 08:15:16 boris Exp $'
! !
