"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

FunctionExpression subclass:#PrefixFunction
	instanceVariableNames:'arguments separator'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!PrefixFunction class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!PrefixFunction methodsFor:'accessing'!

arguments: aCollection
	arguments := aCollection.
!

separator
	^separator
!

separator: aString
	separator := aString
! !

!PrefixFunction methodsFor:'converting'!

convertedStValueOf: anObject
	"This assumes that functions that do conversions have already had their effect in the database, and all we're concerned with is the fundamental data type conversion"
	^base convertedStValueOf: anObject.
! !

!PrefixFunction methodsFor:'initializing'!

initialize
	super initialize.
	arguments := #().
	separator := ','.
! !

!PrefixFunction methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree. Keep track of who has already been visited, so we don't get trapped in cycles or visit nodes twice."

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	arguments do: [:each | each do: aBlock skipping: aSet].
	aBlock value: self.
! !

!PrefixFunction methodsFor:'printing'!

printBasicSQLOn: aStream withParameters: aDictionary forSelect: aBoolean
	aStream
		nextPutAll: function;
		nextPut: $(.
	aBoolean
		ifTrue: [base printSelectSQLOn: aStream withParameters: aDictionary]
		ifFalse: [base printSQLOn: aStream withParameters: aDictionary].
	arguments
		do:
			[:each | 
			aStream nextPutAll: separator.
			aBoolean 
				ifTrue: [each printSelectSQLOn: aStream withParameters: aDictionary]
				ifFalse: [each printSQLOn: aStream withParameters: aDictionary]].
	aStream nextPut: $).
	alias notNil
		ifTrue:
			[aStream
				nextPutAll: ' AS ';
				nextPutAll: alias].
!

printSQLOn: aStream withParameters: aDictionary
	self printsIntoJoins
		ifTrue: [self printBasicSQLOn: aStream withParameters: aDictionary forSelect: false]
		ifFalse: [base printSQLOn: aStream withParameters: aDictionary].
!

printSelectSQLOn: aStream withParameters: aDictionary
	self printBasicSQLOn: aStream withParameters: aDictionary forSelect: true.
!

printTreeOn: aStream 
	aStream
		nextPutAll: function, '('.
	base printOn: aStream.
	aStream nextPutAll: ')'.
! !

!PrefixFunction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__PrefixFunction.st,v 1.1 2006/07/26 08:30:06 boris Exp $'
! !
