"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

RowMap subclass:#RowMapForMementos
	instanceVariableNames:'correspondenceMap reversedCorrespondenceMap collectionMementos'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

RowMapForMementos comment:'This is a specialized version of RowMap for creating rowmaps out of the mementos in the undo/correspondence map. When doing partial writes we create a rowmap for the current state of the objects, then a rowmap for the original state, and difference the two.

The tricky part is that the mementos refer back to the original objects, so when we establish unification constraints between rows, they would establish them to original objects. This is wrong, and not trivial to debug.

So this rowmap keeps the correspondence map and knows that it has to compensate and get the memento for any related objects.
'
!


!RowMapForMementos class methodsFor:'instance creation'!

withCorrespondenceMap: aDictionary

	^self new
		correspondenceMap: aDictionary.
! !

!RowMapForMementos class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!RowMapForMementos methodsFor:'accessing'!

collectionMementos: aDictionary
	collectionMementos := aDictionary.
!

correspondenceMap
	^correspondenceMap
!

correspondenceMap: anObject
	correspondenceMap := anObject
! !

!RowMapForMementos methodsFor:'api'!

adjustForMementos: objects
	"We may be being passed row map keys that refer to originals when they should refer to mementos. Fix.This is horribly ugly."

	1 to: objects size do: [:i | | possibleRowMapKey |
		possibleRowMapKey := objects at: i.
		(self isRowMapKey: possibleRowMapKey) ifTrue: [
			objects at: i put: (possibleRowMapKey adjustIn: correspondenceMap)]].
	^objects.
!

deleteRowForTable: aTable withKey: aKey ifAbsent: aBlock
	^super deleteRowForTable: aTable withKey: (self mementoFor: aKey) ifAbsent: aBlock.
!

findOrAddRowForTable: aTable withKey: aKey 
	^super findOrAddRowForTable: aTable withKey: (self mementoFor: aKey).
!

mementoFor: aKey
	^(self isRowMapKey: aKey)
		ifTrue: [aKey adjustIn: correspondenceMap]
		ifFalse: [correspondenceMap at: aKey ifAbsent: [aKey]].
!

reversedCorrespondenceMap
	(reversedCorrespondenceMap isNil
		or: [reversedCorrespondenceMap size ~= correspondenceMap size])
		ifTrue:
			[reversedCorrespondenceMap := IdentityDictionary
				new: correspondenceMap size.
			correspondenceMap
				keysAndValuesDo: [:k :v | reversedCorrespondenceMap at: v put: k]].
	^reversedCorrespondenceMap.
!

rowForTable: aTable withKey: aKey ifAbsent: aBlock
	^super
		rowForTable: aTable
		withKey: (self mementoFor: aKey)
		ifAbsent: aBlock.
! !

!RowMapForMementos methodsFor:'private/mapping'!

collectionMementoFor: aCollection

	aCollection glorpIsCollection ifFalse: [^aCollection].
	^collectionMementos at: aCollection ifAbsent: [correspondenceMap at: aCollection].
!

originalObjectFor: anObject

	^correspondenceMap at: anObject.
!

reverseLookup: anObject

      (self isRowMapKey: anObject) ifTrue: [
            ^anObject reverseAdjustIn: self].
      ^ self reversedCorrespondenceMap at: anObject.
! !

!RowMapForMementos class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__RowMapForMementos.st,v 1.1 2006/07/26 08:14:18 boris Exp $'
! !
