"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabaseCommand subclass:#SelectCommand
	instanceVariableNames:'parameters blockFactor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpCore'
!

SelectCommand comment:'This represents a SELECT query to the database. This can either be based on a GLORP query, or on a hard-coded SQL string.

Subclasses must implement the following messages:
	accessing
		printSQL

Instance Variables:
	blockFactor	<SmallInteger>	The blocking factor to use for this query. This is most likely to make a difference for these kind of operations, and so is directly settable here.
	parameters	<Dictionary from: DatabaseField to: Object>	The parameters to the query.

'
!


!SelectCommand class methodsFor:'instance creation'!

forQuery: aQuery parameters: aDictionary
	^(self new)
		query: aQuery;
		parameters: aDictionary;
		yourself.
!

forQuery: aQuery parameters: aDictionary useBinding: aBoolean platform: aDatabasePlatform
	^(self new)
		query: aQuery;
		parameters: aDictionary;
		useBinding: aBoolean;
		platform: aDatabasePlatform;
		yourself.
! !

!SelectCommand class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!SelectCommand methodsFor:'accessing'!

blockFactor: anInteger
	blockFactor := anInteger.
!

parameterTypeSignature
	| result |
	result := WriteStream on: String new.
	self bindings do: [:each | result nextPutAll: each class name].
	^result contents.
!

parameters
	^parameters
!

parameters: aDictionary
	parameters := aDictionary
!

printSQL
	self subclassResponsibility.
! !

!SelectCommand methodsFor:'testing'!

canBind: aValue to: aType
	aValue isNil ifTrue: [^false].
	^super canBind: aValue to: aType.
!

isReadCommand
	^true.
!

succeeded

	^true.
! !

!SelectCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__SelectCommand.st,v 1.1 2006/07/26 08:37:55 boris Exp $'
! !
