"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabaseAccessor subclass:#SmalltalkXDatabaseAccessor
	instanceVariableNames:'isInTransaction returnRowCount rowsRead'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-SmalltalkX'
!


!SmalltalkXDatabaseAccessor methodsFor:'accessing'!

returnRowCount: aBoolean

   returnRowCount := aBoolean.
! !

!SmalltalkXDatabaseAccessor methodsFor:'executing'!

basicExecuteSQLString: aString

    aString withBlanksTrimmed isEmpty ifTrue: [self halt].
    ^self basicExecuteSQLString: aString binding: nil.
!

basicExecuteSQLString: aString binding: aBindingArray 
        ^self basicExecuteSQLString: aString returnResult: true binding: aBindingArray doing: [].
!

basicExecuteSQLString: aString returnResult: returnResult binding: aBindingArray doing: aBlock
      "Duplicate some of the logic of command execution, primarily because
       of the need to do special treatment depending if we want a result or
       not, and to execute the block. Don't cache the prepared statement,
       but do put it into the driverSesssion (ugh) so that we can check the
       rowCount if necessary"


        | sqlStatement cursor row fetchedRecord rows|
  aString withBlanksTrimmed isEmpty ifTrue: [^nil].
Transcript show: aString; cr.

        [sqlStatement := connection newStatementFor: aString.]
                on: Error
                do: [:ex | Transcript cr; show: '**basicExecuteSQLString** failed:'; cr;
                                show: aString; cr;cr.
                        ex reject].

        [cursor := sqlStatement execute]
                on: Error
                do: [:ex | 
                        cursor notNil ifTrue: [cursor release].
                        sqlStatement release.
  Transcript show: aString; cr.
                        Transcript show: '**>> error in statement execution'; cr.
                        ex reject.].
          " sqlStatement class == SQL::ODBCStatement
            cursor class == SQL::ODBCCursor "

        [aBlock value]
          on: Error
          do: [:ex | " do nothing, just catch the exception "].

        returnResult
          ifFalse: [
            cursor release.
            sqlStatement release.
            ^nil.
          ].

        rows := WriteStream on: (Array new: 10).
        rowsRead := 0.
        [(fetchedRecord := cursor next) notNil]
          whileTrue:
            [row := fetchedRecord rowAsArray.
             rows nextPut: row.
             rowsRead := rowsRead + 1.
            ].

        cursor release.
        sqlStatement release.
        "connection commit." " needed for CREATE TABLE, DROP TABLE and the like "

        ^"Glorp::GlorpCursoredStream" ReadStream on: rows contents.
!

disconnect

        ^connection release.
!

executeCommandUnbound: aCommand
        | answerStream |
        self log: aCommand sqlString.
        answerStream := self basicExecuteSQLString: aCommand sqlString.
        aCommand needsRowCount
                ifTrue:
                        [aCommand
                                rowCount:
                                        (self platform supportsRowCount
                                                ifTrue: [answerStream rowCount]
                                                ifFalse: [1])].
        ^self cursorFor: answerStream command: aCommand.
!

releaseCursor: aCursor in: aPreparedStatement
        "Release the cursor. We're trapping exceptions, so return true or false to indicate success/failure"
        aCursor isNil ifTrue: [^self].
        ^[aCursor release. true] on: Dialect error do: [:ex | ex return: false].
!

rowCount

       ^rowsRead
! !

!SmalltalkXDatabaseAccessor methodsFor:'initialize'!

initialize
        super initialize.
        isInTransaction := returnRowCount := false.
! !

!SmalltalkXDatabaseAccessor methodsFor:'login'!

connectionArgsFromCurrentLogin

        "connect string looks like  hostname:portNumber_db-name. :portNumber is optional"
        | str portIndex index dbIndex |
        str := self currentLogin connectString.
        portIndex := index := str indexOf: $:.
        dbIndex := str indexOf: $_.
        index = 0 ifTrue: [index := dbIndex].
self halt.
        "^(Dialect smalltalkAt: #PGConnectionArgs)
                hostname: (str copyFrom: 1 to: index - 1)
                portno: (portIndex = 0 ifTrue: [5432] ifFalse: [(str copyFrom: portIndex + 1 to: dbIndex - 1) asNumber])
                databaseName: (str copyFrom: dbIndex + 1 to: str size)
                userName: self currentLogin username
                password: self currentLogin password   "
!

connectionClassForLogin: aLogin 
        aLogin database class == PostgreSqlODBCPlatform
                ifTrue: [^SQL::ODBCSession].
        " add connection classes for other platforms here.  "
        self error: 'Unknown database: ' , aLogin database printString
!

isLoggedIn

        ^connection notNil and: [connection isConnected].
!

loginIfError: aBlock 

        self log: 'Login'.
        connection := self connectionClass new.
          " connect class == SQL::ODBCSession "

        [connection connectWithUsername: currentLogin username
                password: currentLogin password
                dbname: currentLogin name.]
        on: Error
        do: aBlock.
        connection isConnected
          ifFalse: [
                self log: 'login was unsucessful'.
                ^nil
          ].

        self log: 'Login finished'.
        connection autocommit: 'off'.
!

logout

        ^connection disconnect.
! !

!SmalltalkXDatabaseAccessor methodsFor:'transactions'!

beginTransaction
        self log: 'Begin Transaction'.

        isInTransaction := true.
     "   self basicExecuteSQLStringNoResult: 'BEGIN TRANSACTION' doing: [self halt].
     "
!

clearAutocommit

  connection autocommit: 'off'.
!

commitTransaction

       " self executeSQLCommand: 'ROLLBACK TRANSACTION'  "
       isInTransaction
        ifTrue: [
        isInTransaction := false.
        connection commit.
        ]
!

executeSQLCommand: aString
 " | sqlStatement |
        sqlStatement := connection newStatementFor: aString.
        sqlStatement execute.
        sqlStatement release.    "
!

isInTransaction
        ^ isInTransaction
!

rollbackTransaction
       " self executeSQLCommand: 'ROLLBACK TRANSACTION'  "

       isInTransaction
        ifTrue: [
        isInTransaction := false.
        connection rollback.
        ]
!

setAutocommit

  connection autocommit: 'on'.
! !

!SmalltalkXDatabaseAccessor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__SmalltalkXDatabaseAccessor.st,v 1.1 2006/07/26 08:34:52 boris Exp $'
! !
