"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabaseAccessor subclass:#SqueakDatabaseAccessor
	instanceVariableNames:'isInTransaction'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-VW3'
!


!SqueakDatabaseAccessor class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!SqueakDatabaseAccessor methodsFor:'executing'!

basicExecuteSQLString: aString
	| result rowCollection |
	self log: aString.
	result := connection execute: aString.
	result errorResponse notNil
		ifTrue:
			[self externalDatabaseErrorSignal signal: result errorResponse value].
	rowCollection := OrderedCollection new.
	result rows do: [:ea | rowCollection add: ea data asArray].
 	^rowCollection asArray
!

basicExecuteSQLStringNoResult: aString doing: aBlock

	| a |
	a := self basicExecuteSQLString: aString.
	aBlock value.
	^a
!

disconnect
	^ connection disconnect
!

executeCommandUnbound: aCommand
	| answerStream |
	self log: aCommand sqlString.
	answerStream := ReadStream on: (self basicExecuteSQLString: aCommand sqlString).
	aCommand needsRowCount
		ifTrue:
			[aCommand
				rowCount:
					(self platform supportsRowCount
						ifTrue: [answerStream rowCount]
						ifFalse: [1])].
	^self cursorFor: answerStream command: aCommand.
!

externalDatabaseErrorSignal
	^ Dialect error
!

innerExceptionFor: exception
	"I don't know how to get the inner error in Squeak"
	^exception messageText.
!

releaseCursor: aCursor in: aPreparedStatement
        "Release the cursor. We're trapping exceptions, so return true or false to indicate success/failure"
        aCursor isNil ifTrue: [^self].
        ^[aCursor close. true] on: Dialect error do: [:ex | ex return: false].
!

rowCount

	|count|
	count := 0.
	connection result resultSets
		do: [:each| count := count + 
			(each completedResponse value subStrings: (Array with: Character space)) last asInteger].
	^count
! !

!SqueakDatabaseAccessor methodsFor:'initialize'!

initialize
	super initialize.
	isInTransaction := false
! !

!SqueakDatabaseAccessor methodsFor:'login'!

connectionArgsFromCurrentLogin

	"connect string looks like  hostname:portNumber_db-name. :portNumber is optional"
	| str portIndex index dbIndex |
	str := self currentLogin connectString.
	portIndex := index := str indexOf: $:.
	dbIndex := str indexOf: $_.
	index = 0 ifTrue: [index := dbIndex].
	^(Dialect smalltalkAt: #PGConnectionArgs)
		hostname: (str copyFrom: 1 to: index - 1)
		portno: (portIndex = 0 ifTrue: [5432] ifFalse: [(str copyFrom: portIndex + 1 to: dbIndex - 1) asNumber])
		databaseName: (str copyFrom: dbIndex + 1 to: str size)
		userName: self currentLogin username
		password: self currentLogin password
!

connectionClassForLogin: aLogin 
	aLogin database class == PostgreSQLPlatform
		ifTrue: [^ Smalltalk at: #PGConnection].
	self error: 'Unknown database: ' , aLogin database
!

isLoggedIn
	connection isNil
		ifTrue: [^ false].
	^ connection isConnected
!

loginIfError: aBlock 
	self log: 'Login'.
	connection := self connectionClass new.
	connection connectionArgs: self connectionArgsFromCurrentLogin.
	self
		execute: [connection startup]
		ifError: aBlock.
	self log: 'Login finished'
!

logout
	self isLoggedIn ifFalse: [^self].
	self log: 'Logout'.
	self execute: [connection terminate].
	self log: 'Logout finished'
!

showDialog: aString 
	"Do 'Smalltalk at: #PopUpMenu' so that the code will compile in other dialects, even if it won't run"

	(Smalltalk at: #PopUpMenu) inform: aString
! !

!SqueakDatabaseAccessor methodsFor:'transactions'!

beginTransaction
	self log: 'Begin Transaction'.
	isInTransaction := true.
	connection execute: 'BEGIN TRANSACTION'
!

commitTransaction
	self log: 'Commit Transaction'.
	isInTransaction := false.
	connection execute: 'COMMIT TRANSACTION'
!

isInTransaction
	^ isInTransaction
!

rollbackTransaction
	self log: 'Rollback Transaction'.
	isInTransaction := false.
	connection execute: 'ROLLBACK TRANSACTION'
! !

!SqueakDatabaseAccessor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__SqueakDatabaseAccessor.st,v 1.1 2006/07/26 08:35:42 boris Exp $'
! !
