"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

RelationshipMapping subclass:#ToManyMapping
	instanceVariableNames:'orderBy shouldWriteTheOrderField'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!ToManyMapping methodsFor:'api'!

collectionType
	^self attribute collectionType.
!

collectionType: aClass
	self attribute collectionType: aClass.
!

orderBy
	^orderBy.
!

orderBy: aBlockOrSelector
	orderBy isNil ifTrue: [orderBy := OrderedCollection new].
	orderBy add: aBlockOrSelector.
!

shouldWriteTheOrderField
	^shouldWriteTheOrderField
!

shouldWriteTheOrderField: aBoolean
	shouldWriteTheOrderField := aBoolean.
!

writeTheOrderField
	shouldWriteTheOrderField := true.
! !

!ToManyMapping methodsFor:'initialize/release'!

defaultCollectionType
	^OrderedCollection.
!

initialize
	super initialize.
	shouldWriteTheOrderField := false.
!

initializeAttribute
	"This is where we can initialize aspects of the attribute, if required. The motivating example is initializing the collection type of the attribute if it wasn't set. We need it to have some type, but we don't want to initialize it while we're doing basic initialization and aren't yet properly attached to a descriptor or class model"
	super initializeAttribute.
	self attribute collectionType isNil
		ifTrue: [self attribute collectionType: self defaultCollectionType].
! !

!ToManyMapping methodsFor:'mapping'!

buildGeneralQueryIn: anObjectBuilder
	| newQuery |
	newQuery := super buildGeneralQueryIn: anObjectBuilder.
	orderBy isNil ifFalse: [orderBy do: [:each | newQuery orderBy: each]].
	^newQuery.
!

buildQuery
	query := super buildQuery.
	orderBy isNil ifFalse: [orderBy do: [:each | query orderBy: each]].
	^query.
!

keyForOrderingFrom: anObject to: target
	self usesLinkTable ifFalse: [^target].
	^self orderField table == self linkTable ifTrue: [target] ifFalse: [anObject].
!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject atIndex: index
	shouldWriteTheOrderField
		ifTrue:
			[| keyForOrdering |
			keyForOrdering := self keyForOrderingFrom: sourceObject to: rowMapKey.
			aRowMap
				at: self orderField
				key: keyForOrdering
				put: index].
! !

!ToManyMapping methodsFor:'printing'!

printMappingInformationOn: aTextStream


  aTextStream nextPutAll: attribute name;
              nextPutAll: ' -> ';
              nextPutAll: self relatedTables printString.
! !

!ToManyMapping methodsFor:'private/expressions'!

linkTable
	| referenceKeys |
	self usesLinkTable ifFalse: [^nil].
	referenceKeys := (self join) targetKeys asOrderedCollection.
	^referenceKeys first table.
!

orderField
	| exp |
	exp := Query new expressionBlockFor: orderBy first.
	^(exp asGlorpExpressionOn: (BaseExpression new descriptor: self referenceDescriptor)) field.
! !

!ToManyMapping methodsFor:'testing'!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection. Inline the superclass implementation to avoid fetching the value twice."
	| value |
	(super hasValueThatCanProvideField: aField from: anObject) ifFalse: [^false].
	value := self getValueFrom: anObject.
	^value isEmpty not.
! !

!ToManyMapping methodsFor:'validation'!

validate

	| targetClassModel |
	(self collectionType = GlorpCursoredStream and: [self shouldUseFilteredRead]) ifTrue: [self error: 'Filtered mappings can cause duplicates and so cannot return cursors'].

	orderBy isNil ifTrue: [^self].
	targetClassModel := self system classModelFor: attribute type.

	"Note that this will only validate when the orderBy is a symbol - not a block, expression, or field"
	orderBy do: [:attributeName |
		attributeName isSymbol ifTrue: [
			(targetClassModel hasAttributeNamed: attributeName) ifFalse: [
				self error: attribute printString, ' is attempting to order by #', attributeName, ' which is not a mapped attribute of ', targetClassModel describedClass name]]].
! !

!ToManyMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__ToManyMapping.st,v 1.1 2006/07/26 08:16:30 boris Exp $'
! !
