"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

RowBasedCommand subclass:#UpdateCommand
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpCore'
!


!UpdateCommand class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!UpdateCommand methodsFor:'accessing'!

arrayBindings
	"I don't think we can array bind updates. There may be values in the row that we haven't read, and if we attempt to write all the fields we'll be overwriting them with nulls. It's possible that there's a set of rows with all the same fields and we could update them all the same way, but that isn't done yet"
	self error: 'cannot array bind an update'.
!

arrayBoundFields
	^row table nonPrimaryKeyFields asArray , row table primaryKeyFields asArray.
!

blockFactor
	^1.
!

fieldsNeedingUpdate
	^self shouldUpdateOnlyChangedFields 
		ifTrue: [row fieldsNeedingUpdate]
		ifFalse: [row nonPrimaryKeyFields]
!

needsRowCount
	^row lockKeyFields isEmpty not.
! !

!UpdateCommand methodsFor:'bound values'!

bindingTypes
	^super bindingTypes, (row lockKeyFields collect: [:each | each type]).
!

singleRowBindings
	| basic lockFields lockValues |
	basic := super singleRowBindings.
	lockFields := row lockKeyFields.
	lockFields isEmpty ifTrue: [^basic].
	lockValues := OrderedCollection new: 2.
	row lockKeyFields do: [:each | 	
			| fieldValue |
			fieldValue := row oldVersionAt: each ifAbsent: [nil].
			(self canBind: fieldValue to: each type) ifTrue: [lockValues add: fieldValue]].
	^basic, lockValues.
!

singleRowBoundFields
	"Note: We will actually have to bind one more thing than this, the optimistic lock fields (if present). That's handled in the caller"
	| basicFields |
	basicFields := self fieldsNeedingUpdate.
	^basicFields , row table primaryKeyFields asArray
! !

!UpdateCommand methodsFor:'printing'!

printSQL
	| bodyFields |
	self nextPutAll: 'UPDATE '.
	row table printSQLOn: self withParameters: #().
	self nextPutAll: ' SET '.
	bodyFields := self fieldsNeedingUpdate.
	GlorpHelper 
		do: [:field | row printEqualityTemplateForField: field on: self]
		for: bodyFields
		separatedBy: [self nextPut: $,].
	self nextPutAll: ' WHERE '.
	row printPrimaryKeyTemplateOn: self.
	row lockKeyFields notEmpty
		ifTrue: 
			[self nextPutAll: ' AND '.
			GlorpHelper
				do: [:field | row printLockEqualityForField: field on: self]
				for: row lockKeyFields
				separatedBy: [self nextPutAll: ' AND ']]
! !

!UpdateCommand methodsFor:'testing'!

shouldUpdateOnlyChangedFields
	"Return true if we should write only changed fields out in an update statement, or if we should write all of them. The tradeoff is that writing only the changed ones will produce less SQL, but make more variations of the statement, and require us to keep more prepared statements around if we're using binding. By default, use this option only if we're not using binding by default. It would be nice if we could delegate this decision to somewhere the user has control over, e.g. the descriptor system, but we don't actually know about it"
	^self useBinding not.
! !

!UpdateCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__UpdateCommand.st,v 1.1 2006/07/26 08:38:10 boris Exp $'
! !
