"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Mapping
	instanceVariableNames:'descriptor canRead canWrite attribute debugRead debugWrite'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!Mapping class methodsFor:'instance creation'!

new

	^super new initialize.
! !

!Mapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Mapping methodsFor:'accessing'!

allTables

	self subclassResponsibility.
!

attribute
	attribute isNil ifTrue: [attribute := GlorpAttributeModel new].
	^attribute.
!

attributeAccessor
	^self attribute.
!

attributeName
	^self attribute name.
!

attributeName: aSymbol
	| oldAttribute |
	oldAttribute := attribute.
	attribute := self classModel attributeNamed: aSymbol asSymbol.
	oldAttribute isNil
		ifTrue: [self initializeAttribute]
		ifFalse: [attribute copyFrom: oldAttribute].
!

beForPseudoVariable
	canRead := false.
	canWrite := false.
!

canRead
	^canRead.
!

canWrite
	^canWrite.
!

classModel
	"If we don't have one, just make a temporary one. We'll fix it later"
	descriptor isNil ifTrue: [^GlorpClassModel new].
	^self descriptor classModel.
!

descriptor
	^descriptor
!

descriptor: aDescriptor
	descriptor := aDescriptor.
	attribute isNil ifFalse: [self copyAttributeAttributes].
!

fieldsForSelectStatement
	"Return a collection of fields that this mapping will read from a row"
	canRead ifFalse: [^#()].
	^self mappedFields
!

isForPseudoVariable
	self attributeAccessor isValid not.
!

readOnly
	^canWrite not.
!

readOnly: aBoolean
	canWrite := aBoolean not.
!

session
	
	^self descriptor session.
!

system
	
	^self descriptor system.
! !

!Mapping methodsFor:'api'!

beExclusive
	self error: 'Only relationship mappings can be exclusive'.
!

isExclusive
	^false.
!

isExclusive: aBoolean
	aBoolean ifTrue: [self error: 'Only relationship mappings can be exclusive'].
! !

!Mapping methodsFor:'debugging'!

debugRead
	^debugRead
!

debugRead: aBoolean
	debugRead := aBoolean
!

debugWrite
	^debugWrite
!

debugWrite: aBoolean
	debugWrite := aBoolean
!

stopIfDebuggingRead
	debugRead ifTrue: [self halt].
!

stopIfDebuggingWrite
	debugWrite ifTrue: [self halt].
! !

!Mapping methodsFor:'initialize/release'!

copyAttributeAttributes
	"We have an attributemodel that was created before we actually had a descriptor. It's possible that it had properties set. Make sure we copy them. If necessary, initialize things that require it."
	| oldAttribute |
	oldAttribute := attribute.
	attribute := self classModel attributeNamed: oldAttribute name type: oldAttribute type collectionType: oldAttribute collectionType.
	attribute copyFrom: oldAttribute.
	self initializeAttribute.
!

initialize
	canRead := true.
	canWrite := true.
	debugRead := false.
	debugWrite := false.
!

initializeAttribute
	"This is where we can initialize aspects of the attribute, if required. The motivating example is initializing the collection type of the attribute if it wasn't set. We need it to have some type, but we don't want to initialize it while we're doing basic initialization and aren't yet properly attached to a descriptor or class model"
! !

!Mapping methodsFor:'iterating'!

loopOverAttributeOf: anObject doing: aTwoArgumentBlock
	"Loop over the target, which we extract from anObject, and iterate over it evaluating the block. For things that aren't collections, iterating means just run once. Note that the argument is expected to be an instance of our descriptor, as opposed to the attribute method, which expects the target object"
	| target |
	target := self getValueFrom: anObject.
	self attribute loopOver: target doing: aTwoArgumentBlock.
! !

!Mapping methodsFor:'mapping'!

applicableMappingForObject: anObject 
	"For polymorphism with conditional mappings"
	^self
!

createDeleteRowFor: anObject in: aRowMap
	"Only applies to mappings that write into link tables"
!

expressionFor: anObject
	"Return an expression representing the value of the object. This can be nil, an object value or values, an expression, or a collection of expressions (for a composite key, if we're passed an expression)"

	anObject isNil ifTrue: [^#(nil)].
	anObject isGlorpExpression ifFalse: [
		^self mappedFields collect: [:each | 	
			self valueOfField: each fromObject: anObject]].
	^self mappedFields
		collect: [:each | (anObject getField: each)].
!

expressionFor: anObject basedOn: anExpression relation: aSymbol
	"Return our expression using the object's values. e.g. if this was a direct mapping from id->ID and the object had id: 3, then return TABLE.ID=3. Used when rewriting object=object into field=field"

	| myValue result |
	myValue := self expressionFor: anObject.
	result := nil.
	myValue with: self join allTargetFields do: [:eachValue :eachField |
		| source |
		source := anExpression get: self attribute name.
		source hasDescriptor ifTrue: [source := source getField: eachField].
		result := (source get: aSymbol withArguments: (Array with: eachValue)) AND: result].
	^result.
!

mapFromObject: anObject intoRowsIn: aRowMap

	self subclassResponsibility.
!

mapObject: anObject inElementBuilder: anObject1
	self subclassResponsibility.
!

mappedTables
	^(self mappedFields collect: [:each | each table]) asSet.
!

readBackNewRowInformationFor: anObject fromRowsIn: aRowMap 
	"
	self subclassResponsibility. ?"
!

referencedIndependentObjectsFrom: anObject

	self subclassResponsibility.
!

trace: aTracing context: anExpression

	self subclassResponsibility.
!

translateFields: anOrderedCollection 
	"Normal mappings don't translate"
	^anOrderedCollection.
!

translateTable: aTable
	"If we have a field translation, return the translated (i.e. real) table to use. This ideally allows us to use either the real table in which something is embedded, or the imaginary table"
	^aTable.
!

valueOfField: aField fromObject: anObject
	self error: 'Can''t get value of field'.
! !

!Mapping methodsFor:'preparing'!

joinExpressionFor: anExpression

	^nil.
!

multipleTableExpressionsFor: anExpression

	^#().
! !

!Mapping methodsFor:'printing'!

printMappingInformationOn: aTextStream
! !

!Mapping methodsFor:'printing SQL'!

allRelationsFor: rootExpression do: aBlock andBetweenDo: anotherBlock
	"Normal mappings just operate on a single expression"
	aBlock value: rootExpression leftChild value: rootExpression rightChild.
! !

!Mapping methodsFor:'public'!

getValueFrom: anObject

	^self attribute getValueFrom: anObject.
!

printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: '(';
		nextPutAll: (attribute isNil ifTrue: [''] ifFalse: [self attributeName]) ;
		nextPutAll: ')'.
!

setValueIn: anObject to: aValue
	canRead ifFalse: [^self].
	self attribute setValueIn: anObject to: aValue.
! !

!Mapping methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	self isRelationship ifFalse: [^true].
	^self isToManyRelationship not.
!

controlsTables
	"Return true if this type of mapping 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	self subclassResponsibility
!

hasAttribute
	"Return true if this maps an attribute, which is normally true"
	^true
!

hasImpliedClauses
	"Return true if this implies multiple sql clauses"
	^false.
!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection."
	^true.
!

includesSubFieldsInSelectStatement
	^false
!

isRelationship
	"True when the mapping associates different persistent classes."

	^self subclassResponsibility
!

isStoredInSameTable
	"True when the mapping is between two objects that occupy the same table, e.g. an embedded mapping."

	^self subclassResponsibility
!

isToManyRelationship
	^false.
!

isTypeMapping
	^false
!

mappedFields
	self subclassResponsibility
!

mapsPrimaryKeys
	^(self mappedFields detect: [:each | each isPrimaryKey] ifNone: [nil]) notNil.
!

mapsSimpleObject
	"Return true if we know that we map something without a descriptor. For some mappings, like ad hoc or conditional, we don't really know, so we default to false"
	^false.
! !

!Mapping methodsFor:'validation'!

validate
	self validateAttribute
!

validateAttribute
	canWrite ifTrue: [self attribute validateCanRead].
	canRead ifTrue: [self attribute validateCanWrite]
! !

!Mapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Mapping.st,v 1.1 2006/07/26 08:17:36 boris Exp $'
! !
