"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

ObjectExpression subclass:#MappingExpression
	instanceVariableNames:'name base outerJoin mapping'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!MappingExpression class methodsFor:'instance creation'!

named: aSymbol basedOn: anExpression

	^self new
		named: aSymbol
		basedOn: anExpression.
! !

!MappingExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!MappingExpression methodsFor:'accessing'!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

descriptor
	| system |
	system := self system.
	system isNil ifTrue: [^nil].
	^system descriptorFor: self mapping referenceClass.
!

field

	self mapping isNil ifTrue: [self error: '"', name, '" is not a mapped property name in ', base descriptor describedClass name].
	self hasDescriptor ifTrue: [
		"If we have a descriptor, then either we're a mapping to a full object, in which case we can't answer this properly, or we're a direct to-many mapping, in which case we can."
		self mapping mapsSimpleObject
			ifTrue: [^self translateField: self mapping field]
			ifFalse: [self error: '"',name, 
				'" is not an attribute that resolves to a field in the mapped tables for ', 
					base descriptor describedClass name]].
	^base translateField: self mapping field.
!

hasDescriptor
	"Does the object that we describe have its own descriptor"
	| myMapping |
	myMapping := self mapping.
	^myMapping notNil and: [myMapping isRelationship]
!

join
	self mapping isNil ifTrue: [^nil].
	^self mapping join.
!

mappedFields
	
	self mapping isNil ifTrue: [self error: '"', name, '" is not a mapped property name in ', base descriptor describedClass name].
	self mapping isRelationship ifTrue: [self error: '"',name, '" is not an attribute that resolves to a field in the mapped tables for ', base descriptor describedClass name].
	^self mapping mappedFields collect: [:each | base translateField: each].
!

multipleTableExpressions
	^self multipleTableExpressionsOn: self.
!

multipleTableExpressionsOn: anExpression
	^self mapping multipleTableExpressionsFor: anExpression.
!

name

	^name.
!

sourceDescriptor
	^base descriptor.
!

system
	^base system.
!

table
	self hasDescriptor ifTrue: [^self descriptor primaryTable].
	^self field table.
!

tables

	| set |
	self controlsTables ifFalse: [^#()].
	self descriptor isNil ifTrue: [^#()].
	set := self descriptor tables asSet.
	^set.
! !

!MappingExpression methodsFor:'api'!

asOuterJoin
	outerJoin := true.
!

base

	^base.
!

sqlSelect: anExpression
	"Only useful for mappings that represent collections. This restricts the range of values in a variable. Most useful as part of an aggregation. e.g. 
    read: SomeClass where: [:each | (each collection select: [:eachItem | eachItem attribute = someValue]) sqlCount > 10]"

	"We may be passed an expression corresponding to either a field, or to a full object, depending on the aggregate function. In either case, build backwards from the first full object"
	| objectExpression newMapping previousJoin |
	objectExpression := self mapping isRelationship 
		ifTrue: [self] ifFalse: [self base].
	newMapping := self mapping class new.
	newMapping referenceClass: self mapping referenceClass.
	newMapping descriptor: self mapping descriptor.
	newMapping relevantLinkTableFields: self mapping relevantLinkTableFields.
	newMapping usesLinkTable: self mapping usesLinkTable.
	previousJoin := self mapping join.
	newMapping join: ((anExpression rebuildOn: previousJoin ultimateBaseExpression startingFrom: self) AND: previousJoin asGlorpExpression).

	^objectExpression base getMapping: newMapping named: ( 'a temporary restricted mapping based on ', self mapping attributeName).
! !

!MappingExpression methodsFor:'converting'!

convertedDbValueOf: anObject

	^self mapping convertedDbValueOf: anObject.
!

convertedStValueOf: anObject

	^self mapping convertedStValueOf: anObject.
!

valueInBuilder: anElementBuilder
	^self valueInBuilder: anElementBuilder as: self base.
!

valueInBuilder: anElementBuilder as: anExpression
	^self mapping valueIn: anElementBuilder as: anExpression.
! !

!MappingExpression methodsFor:'fields'!

aliasedTableFor: aDatabaseTable 
	^self controlsTables 
		ifTrue: [super aliasedTableFor: aDatabaseTable]
		ifFalse: [base aliasedTableFor: (self mapping translateTable: aDatabaseTable)]
!

aliasedTableFor: aDatabaseTable ifAbsent: aBlock
	^self controlsTables 
		ifTrue: [super aliasedTableFor: aDatabaseTable ifAbsent: aBlock]
		ifFalse: [base aliasedTableFor: aDatabaseTable ifAbsent: aBlock]
!

controlsTables
	self mapping isNil ifTrue: [^true].
	^self mapping controlsTables
! !

!MappingExpression methodsFor:'internal'!

allRelationsFor: rootExpression
	^self mapping allRelationsFor: rootExpression.
!

mapping
	mapping isNil ifTrue: [
		| descriptor |
		descriptor := self sourceDescriptor.
		descriptor isNil ifTrue: [^nil].
		mapping := descriptor mappingForAttributeNamed: name].
	^mapping.
! !

!MappingExpression methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree"

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	aBlock value: self.
! !

!MappingExpression methodsFor:'navigating'!

queryLevelBaseExpression

	^base queryLevelBaseExpression.
!

ultimateBaseExpression

	^base ultimateBaseExpression.
! !

!MappingExpression methodsFor:'preparing'!

additionalExpressions
	^self additionalExpressionsOn: self.
!

additionalExpressionsOn: anExpression
	"We can get hijacked, so pass in the expression to use rather than just using self"
	| exp |
	exp := self mapping joinExpressionFor: anExpression.
	outerJoin ifTrue: [exp beOuterJoin].
	^(self multipleTableExpressionsOn: anExpression), (exp isNil ifTrue: [#()] ifFalse: [Array with: exp]).
!

allRelationsFor: rootExpression do: aBlock andBetweenDo: anotherBlock
	"We might have multiple clauses to print, depending on our mapping"
	self mapping 
		allRelationsFor: rootExpression
		do: aBlock
		andBetweenDo: anotherBlock.
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	| newBase |
	newBase := base asExpressionJoiningSource: source toTarget: target.
	^self class named: name basedOn: newBase.
!

fieldsForSelectStatement
	
	^self mapping fieldsForSelectStatement.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	| expression newBase |
	self == anExpression ifTrue: [^aBaseExpression].
	newBase := base rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression.

	"For normal use, this is an optimization to avoid looking up the mapping by name, but for phantom mappings it's essential"
	expression := mapping isNil
		ifTrue: [newBase get: name]
		ifFalse: [newBase getMapping: self mapping named: name].
	outerJoin ifTrue: [expression asOuterJoin].
	^expression.
!

tablesToPrint

	self hasDescriptor ifFalse: [^#()].
	^self tables collect: [:each |
		self aliasedTableFor: each].
!

translateField: aDatabaseField
	| translatedField |
	translatedField := (self mapping translateFields: (Array with: aDatabaseField)) first.
	^super translateField: (translatedField isNil ifTrue: [aDatabaseField] ifFalse: [translatedField]).
!

translateFields: anOrderedCollection 
	"Ugh. Unify these mechnisms"
	^super translateFields: (self mapping translateFields: anOrderedCollection).
!

validate
	self mapping isNil ifTrue: [self error: 'no mapping for ', self printString].
! !

!MappingExpression methodsFor:'printing'!

printOnlySelfOn: aStream

	aStream nextPutAll: name.
	self printTableAliasesOn: aStream.
!

printTreeOn: aStream 
	aStream
		print: base;
		nextPut: $.;
		nextPutAll: name
! !

!MappingExpression methodsFor:'printing SQL'!

printSQLOn: aStream withParameters: aDictionary 
	self field asField printSQLOn: aStream withParameters: aDictionary.
!

printSelectSQLOn: aStream withParameters: aDictionary 
	self printSQLOn: aStream withParameters: aDictionary.
!

printUnqualifiedSQLOn: aStream withParameters: aDictionary 
	self field asField printUnqualifiedSQLOn: aStream withParameters: aDictionary.
! !

!MappingExpression methodsFor:'private/accessing'!

requiresDistinct: aBoolean
	super requiresDistinct: aBoolean.
	base requiresDistinct: aBoolean.
! !

!MappingExpression methodsFor:'private/initialization'!

mapping: aMapping
	mapping := aMapping.
!

named: aSymbol basedOn: anExpression

	name := aSymbol.
	base := anExpression.
	outerJoin := false.
! !

!MappingExpression methodsFor:'testing'!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	^self mapping canBeUsedForRetrieve.
!

canKnit
	"Return true if, when building objects, we can knit the object corresponding to this expression to a related object. Roughly speaking, is this a mapping expression"
	^true.
!

hasImpliedClauses
	^self mapping notNil and: [self mapping hasImpliedClauses].
!

mapsSimpleObject
	^self mapping mapsSimpleObject.
!

representsDerivedObject
	^self mapping isRelationship.
!

returnsDictionary
	"Return true if we representa  ampping to a dictionary"
	^self mapping isToManyRelationship and: [self mapping collectionType == Dictionary].
! !

!MappingExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/MappingExpression.st,v 1.1 2006/07/26 08:29:51 boris Exp $'
! !
