"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

ProtoObject subclass:#MessageArchiver
	instanceVariableNames:'mySelector myArguments myReceiver query'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!MessageArchiver class methodsFor:'instance creation'!

receiver: aMessageCollector selector: aSymbol arguments: aCollection

	^self new
		receiver: aMessageCollector
		selector: aSymbol
		arguments: aCollection.
! !

!MessageArchiver class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!MessageArchiver methodsFor:'debugging'!

asText
	^self basicPrintString asText.
!

basicPrintString

	^self printString.
!

class
        ^Glorp::MessageArchiver
!

halt
	"Support this so that we can debug inside query blocks. For portability, send it to a different object so that we don't have to care how halt is implemented"

	(Array with: self) halt.
!

inspect

	"Not exactly the intended semantics, but should be portable"
	(Array with: self) inspect.
!

inspectorExtraAttributes
	^#()
!

inspectorSize
  ^2
!

mustBeBoolean
	nil error: 'mustBeBoolean - this probably indicates you are trying to use an optimized message like and: or ifTrue: inside a Glorp expression block. Use AND: or & instead.'
!

printOn: aStream

	aStream nextPutAll: self printString.
!

printString

	"Hard-code this for maximum dialect portability"
	^'a MessageArchiver'.
! !

!MessageArchiver methodsFor:'doesNotUnderstand'!

= anObject
	"Needed because VA's abtObservableWrapper implements =. Should be portable."
	^MessageArchiver 
		receiver: self
		selector: #= 
		arguments: (Array with: anObject).
!

basicDoesNotUnderstand: aMessage 
	"Invoke this to avoid infinite recursion in the case of internal errors. We want a dialect-independent way of getting a walkback window, so we'll invoke it against a different object"


	(Array with: self) doesNotUnderstand: aMessage.
!

basicDoesNotUnderstand: sel args: args 
	"Invoke this to avoid infinite recursion in the case of internal errors. We want a dialect-independent way of getting a walkback window, so we'll invoke it against a different object"

	(Array with: self) doesNotUnderstand: sel args: args.
!

doesNotUnderstand: aMessage 
	| sel |
	sel := aMessage selector.
	sel == #doesNotUnderstand: ifTrue: [self basicDoesNotUnderstand: aMessage].
	(sel size >= 8 and: [(sel copyFrom: 1 to: 8) = 'perform:']) 
		ifTrue: 
			[^self get: aMessage arguments first
				withArguments: (aMessage arguments copyFrom: 2 to: aMessage arguments size)].
	^self get: sel withArguments: aMessage arguments.
!

doesNotUnderstand: sel args: args 
	"This is the variant for ObjectStudio"
	sel == #doesNotUnderstand: ifTrue: [self basicDoesNotUnderstand: sel args: args].
	(sel size >= 8 and: [(sel copyFrom: 1 to: 8) = 'perform:']) 
		ifTrue: 
			[^self get: args first
				withArguments: (args copyFrom: 2 to: args size)].
	^self get: sel withArguments: args.
! !

!MessageArchiver methodsFor:'expression creation'!

asGlorpExpression
	^self asGlorpExpressionOn: BaseExpression new
!

asGlorpExpressionOn: aBaseExpression 
	| arguments |
	myReceiver == nil ifTrue: [^query isNil ifTrue: [aBaseExpression] ifFalse: [query baseExpression]].
	arguments := myArguments collect: [:each | each asGlorpExpressionOn: aBaseExpression].
	^(myReceiver asGlorpExpressionOn: aBaseExpression) get: mySelector
		withArguments: arguments.
!

asGlorpExpressionOn: aBaseExpression basedOn: aMessageArchiver withUltimateBase: ultimateBaseExpression
	"We know we are built from aMessageArchiver, but parts of us may refer to an outer scope. Those parts should be built on ultimateBaseExpression rather than aBaseExpression. e.g. consider 
 [:each | each amount amount = ((Query readManyOf: GlorpBankTransaction
			where: [:eachTrans | eachTrans owner = each owner]) 
				retrieve: [:eachTrans | eachTrans amount amount max])]."

	| arguments |
	self == aMessageArchiver ifTrue: [^aBaseExpression].
	myReceiver == nil ifTrue: [^query isNil ifTrue: [ultimateBaseExpression] ifFalse: [query baseExpression]].
	arguments := myArguments
		collect:
			[:each | 
			each
				asGlorpExpressionOn: aBaseExpression
				basedOn: aMessageArchiver
				withUltimateBase: ultimateBaseExpression].
	^(myReceiver
		asGlorpExpressionOn: aBaseExpression
		basedOn: aMessageArchiver
		withUltimateBase: ultimateBaseExpression)
		get: mySelector
		withArguments: arguments.
! !

!MessageArchiver methodsFor:'expression protocol'!

between: anObject and: anotherObject

	^(self >= anObject) & (self <= anotherObject).
!

get: aSymbol 
	^MessageArchiver receiver: self
		selector: aSymbol
		arguments: #().
!

get: aSymbol withArguments: anArray 
	| arguments |
	arguments := anArray collect: [:each | each glorpIsBlock 
		ifTrue: [(Dialect argumentCountFor: each) = 1
			ifTrue: [each value: self]
			ifFalse: [each value]]
		ifFalse: [each]].
	^MessageArchiver receiver: self
		selector: aSymbol
		arguments: arguments.
! !

!MessageArchiver methodsFor:'initialize'!

receiver: aMessageCollector selector: aSymbol arguments: aCollection

	myReceiver := aMessageCollector.
	mySelector := aSymbol.
	myArguments := aCollection.
! !

!MessageArchiver methodsFor:'private/accessing'!

privateGlorpQuery
	^query.
!

privateGlorpQuery: aQuery
	query := aQuery.
!

privateGlorpReceiver

	^myReceiver
!

privateGlorpSelector

	^mySelector
! !

!MessageArchiver methodsFor:'testing'!

glorpIsBlock
	^false.
! !

!MessageArchiver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/MessageArchiver.st,v 1.1 2006/07/26 08:28:05 boris Exp $'
! !
