"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#ObjectExpression
	instanceVariableNames:'mappingExpressions requiresDistinct tableAliases fieldAliases'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!ObjectExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ObjectExpression methodsFor:'accessing'!

fieldAliases

	fieldAliases isNil ifTrue: [fieldAliases := IdentityDictionary new].
	^fieldAliases.
!

mappingExpressions
	mappingExpressions isNil ifTrue: [mappingExpressions := IdentityDictionary new].
	^mappingExpressions.
!

mappingExpressions: anObject
	mappingExpressions := anObject
!

system
	self subclassResponsibility.
!

table
	self subclassResponsibility.
!

tableAliases
	tableAliases isNil ifTrue: [
		tableAliases := IdentityDictionary new: 3].
	^tableAliases
! !

!ObjectExpression methodsFor:'api'!

get: aSymbol 
	"Return the mapping expression corresponding to the named attribute"

	| reallyASymbol functionExpression |
	reallyASymbol := aSymbol asSymbol.
	functionExpression := self getFunction: aSymbol withArguments: #().
	functionExpression isNil ifFalse: [^functionExpression].
	^self mappingExpressions at: reallyASymbol
		ifAbsentPut: [MappingExpression named: reallyASymbol basedOn: self]
!

get: aSymbol withArguments: anArray 

	| specialExpression functionExpression |
	functionExpression := self getFunction: aSymbol withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	specialExpression := self performSpecialExpression: aSymbol withArguments: anArray.
	specialExpression isNil ifFalse: [^specialExpression].
	anArray isEmpty ifTrue: [^self get: aSymbol].
	^RelationExpression
		named: aSymbol
		basedOn: self
		withArguments: anArray
!

getConstant: anObject
	"Allows us to refer to a constant object easily. This is most useful for subselects, where I can already write something like
   [:each | (each count: subQuery) = 5]
but since the subselect part might be very verbose, I might prefer to put the constant value first. I can't write
   [:each | 5 = (each count: subQuery)]
because then the left-hand side expression doesn't refer back to the block argument, and we can't build the expression correctly. So I can write
   [:each | (each getConstant: 5) = (each count: subQuery)]"
	^anObject asGlorpExpressionOn: self.
!

getField: aField
	| realField |
	realField := aField isString
		ifTrue: [self findFieldNamed: aField]
		ifFalse: [aField].
	"This might be an expression, most notably a constant expression, in which case it either contains a string or a field. This is awfully ugly"
	realField isGlorpExpression
		ifTrue:
			[realField class == ConstantExpression
				ifTrue:
					[realField := realField value isString
						ifTrue: [realField := self findFieldNamed: realField value]
						ifFalse: [realField]]].
	^self getFieldKnownNotToBeAName: realField.
!

getFieldKnownNotToBeAName: aField
	"The public getField method may, for convenience, be passed a string or a constant expression with a string that is a field name. This is a mess, and can be confused with a constant value. This method does not allow that."

	| realField |
	realField := aField.
	realField isGlorpExpression
		ifTrue:
			[realField class == ConstantExpression ifTrue: [^realField].
			realField class == ParameterExpression
				ifTrue: [realField := realField field]].
	^self getFieldThatsAnActualFieldObject: realField.
!

getFieldThatsAnActualFieldObject: aField
	"If the field isn't from our table, do a getTable first. If we don't know, assume we can get it directly."

	self controlsTables ifFalse: [^self base getFieldThatsAnActualFieldObject: (self translateField: aField)].
	^((self hasDescriptor not) or: [self tables includes: aField table])
		ifTrue:
			[self mappingExpressions
				at: aField
				ifAbsentPut: [self newFieldExpressionFor: aField]]
		ifFalse: [(self getTable: aField table) getField: aField].
!

getMapping: aMapping named: aString
	"Get a mapping expression given the actual mapping (which presumably isn't named or in our descriptor). Only if you really know what you're doing!!"

	^self mappingExpressions at: aString
		ifAbsentPut: [ | expression |
			expression := MappingExpression named: aString basedOn: self.
			expression mapping: aMapping.
			expression].
!

getParameter: aDatabaseField 
	^ParameterExpression forField: aDatabaseField basedOn: self.
!

getTable: aTable
	"This can take a string, a constantExpression containing a string, or a table object"

	| realTable |
	realTable := aTable isString
		ifTrue: [self system tableNamed: aTable]
		ifFalse: [aTable].	"This might be an expression, most notably a constant expression, in which case it either contains a string or a field"
	realTable isGlorpExpression
		ifTrue:
			[realTable value isString
				ifTrue: [realTable := self system tableNamed: realTable value]
				ifFalse: [realTable := realTable value]].
	^self mappingExpressions
		at: realTable
		ifAbsentPut: [TableExpression forTable: realTable basedOn: self].
! !

!ObjectExpression methodsFor:'api-aggregates'!

aggregate: anExpression as: aggregateFunctionName

	^self aggregate: anExpression as: aggregateFunctionName where: nil.
!

aggregate: anExpression as: aggregateFunctionName where: whereExpression
	"Make a subselect of the expression that is our argument, returning an aggregated value. e.g.
session read: GlorpCustomer where: 	[:each |  
	(each aggregate: eachCust accounts as: #countStar where: [:acct | acct price > 1000]]) = 1].
	This is the most general form of aggregation convenience method, and everything else is implemented in terms of it"

	| q expression next objectExpression retrievalExpression basicExpression |
	"We may be passed an expression corresponding to either a field, or to a full object, depending on the aggregate function. In either case, build backwards from the first full object"
	objectExpression := anExpression mapping isRelationship 
		ifTrue: [anExpression] ifFalse: [anExpression base].
	q := Query readManyOf: objectExpression mapping referenceClass.
	expression := q baseExpression.
	next := objectExpression.
	[next base isNil] whileFalse: [
		expression := next mapping addPhantomMappingExpressionOn: expression.
		next := next base].
	"We have to force construct the equality expression, because we don't want 'self' rebuilt onto the other base"
	expression := RelationExpression new outerJoin: false; relation: #=; leftChild: expression; rightChild: self.
	q whereClause: expression.
	"We need to rebuild the thing we're using to retrieve in the right context. Also, if we're called directly, the symbol may be wrapped as a constantExpression. Unwrap it"
	basicExpression := anExpression rebuildOn: q baseExpression startingFrom: objectExpression.
	retrievalExpression := (basicExpression get: aggregateFunctionName asGlorpExpression value).
	q retrieve: retrievalExpression.
	whereExpression == nil ifFalse: [
		q AND: (whereExpression asGlorpExpressionOn: basicExpression)].
	^q asGlorpExpressionOn: self ultimateBaseExpression.
!

average: anExpression
	^self aggregate: anExpression as: #average.
!

count: anExpression
	"Make a subselect of the expression that is our argument, returning a count(*). e.g.
session read: GlorpCustomer where: 	[:each |  (each count: [:eachCust | eachCust accounts]) = 1]."
	^self aggregate: anExpression as: #countStar.
!

count: anExpression where: filterExpression
	"Make a subselect of the expression that is our argument, returning a count(*). e.g.
session read: GlorpCustomer where: 	[:each |  (each count: [:eachCust | eachCust accounts]) = 1]."
	^self aggregate: anExpression as: #countStar where: filterExpression.
!

max: anExpression
	^self aggregate: anExpression as: #max.
!

min: anExpression
	^self aggregate: anExpression as: #min.
!

sqlAverage
	"Make a subselect of the expression that is our argument, returning an avg(). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts balance sqlAverage > 100]."
	^self queryLevelBaseExpression average: self.
!

sqlAvg
	"Make a subselect of the expression that is our argument, returning an avg(). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts balance sqlAvg > 100]."
	^self queryLevelBaseExpression average: self.
!

sqlCount
	"Make a subselect of the expression that is our argument, returning a count(). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts sqlCount = 1]."
	^self queryLevelBaseExpression count: self.
!

sqlCountWhere: filterExpression
	"Make a subselect of the expression that is our argument, returning a count(). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts sqlCount = 1]."
	^self queryLevelBaseExpression count: self where: filterExpression.
!

sqlMax
	"Make a subselect of the expression that is our argument, returning a max(). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts balance sqlMax = 1]."
	^self queryLevelBaseExpression max: self.
!

sqlMin
	"Make a subselect of the expression that is our argument, returning a minimum. e.g.
session read: GlorpCustomer where: 	[:each |  each accounts balance sqlMin = 1]."
	^self queryLevelBaseExpression min: self.
!

sqlSum
	"Make a subselect of the expression that is our argument, returning a count(*). e.g.
session read: GlorpCustomer where: 	[:each |  each accounts balance sqlSum = 1]."
	^self queryLevelBaseExpression sum: self.
!

sum: anExpression
	"Make a subselect of the expression that is our argument, returning a sum. e.g.
session read: GlorpCustomer where: 	[:each |  (each sum: [:eachCust | eachCust accounts id]) > 5]."
	^self aggregate: anExpression as: #sum.
! !

!ObjectExpression methodsFor:'fields'!

aliasTable: aDatabaseTable to: aString 
	| newTable |
	newTable := aDatabaseTable copy.
	newTable name: aString.
	newTable parent: aDatabaseTable.
	newTable schema: ''.
	self tableAliases at: aDatabaseTable put: newTable
!

aliasedTableFor: aDatabaseTable 
	tableAliases isNil ifTrue: [^aDatabaseTable].
	aDatabaseTable isAliased ifTrue: [^aDatabaseTable].
	^tableAliases 
		at: aDatabaseTable 
		ifAbsent: [
			(self tables includes: aDatabaseTable) 	
				ifTrue: [self error: 'Bug. Expression has only partially aliased tables']
				ifFalse: [
					| tableExpression |
					tableExpression := self mappingExpressions at: aDatabaseTable ifAbsent: [nil].
					tableExpression isNil 
						ifTrue: [aDatabaseTable]
						ifFalse: [tableExpression aliasedTableFor: aDatabaseTable]]].
!

aliasedTableFor: aDatabaseTable  ifAbsent: aBlock
	tableAliases isNil ifTrue: [^aBlock value].
	aDatabaseTable isAliased ifTrue: [^aDatabaseTable].
	^tableAliases at: aDatabaseTable ifAbsent: [aBlock value].
!

controlsTables

	self subclassResponsibility.
!

findFieldNamed: aString
	| fields |
	fields := OrderedCollection new: 1.
	self tables do: [:each |
		| field |
		field := each fieldNamed: aString ifAbsent: [nil].
		field isNil ifFalse: [fields add: field]].
	fields isEmpty ifTrue: [self error: 'No field named ', aString, ' in ', self printString].
	fields size > 1 ifTrue: [self error: 'Ambiguous field ', aString, ' in ', self printString].
	^fields at: 1.
!

newFieldExpressionFor: aField

	^FieldExpression forField: aField basedOn: self
!

translateField: aDatabaseField
	| newTable rebuilt |
	rebuilt := aDatabaseField isGlorpExpression
		ifTrue: [aDatabaseField asGlorpExpressionOn: self ultimateBaseExpression]
		ifFalse: [aDatabaseField].
	newTable := self aliasedTableFor: rebuilt table.
	newTable == rebuilt table ifTrue: [^rebuilt].
	^self fieldAliases at: rebuilt ifAbsentPut: [rebuilt withTable: newTable].
!

translateFields: anOrderedCollection 
	^anOrderedCollection collect: [:each | self translateField: each]
! !

!ObjectExpression methodsFor:'initialize'!

initialize

	super initialize.
	requiresDistinct := false.
! !

!ObjectExpression methodsFor:'internal'!

anySatisfyDefault: anExpression 
	^(self system isNil or: [self system useJoinsForAnySatisfy])
		ifTrue: [self anySatisfyJoin: anExpression]
		ifFalse: [self anySatisfyExists: anExpression].
!

anySatisfyExists: anExpression 
	| newExpression |
	newExpression := SubSelectExpression 
		named: #anySatisfy:
		basedOn: self
		withArguments: (Array with: anExpression).
	^newExpression.
!

anySatisfyJoin: anExpression 
	| newExpression |
	self base requiresDistinct: true.
	newExpression := CollectionExpression 
		named: #anySatisfy:
		basedOn: self
		withArguments: (Array with: anExpression).
	self base removeMappingExpression: self.
	^newExpression.
!

noneSatisfy: anExpression 
	| newExpression |
	newExpression := SubSelectExpression 
		named: #noneSatisfy:
		basedOn: self
		withArguments: (Array with: anExpression).
	^newExpression.
!

noneSatisfyExists: anExpression 
	| newExpression |
	newExpression := SubSelectExpression 
		named: #noneSatisfy:
		basedOn: self
		withArguments: (Array with: anExpression).
	^newExpression.
!

oneArgumentSpecialSelectors
	^#(#anySatisfyDefault: #anySatisfyJoin: #anySatisfyExists: #getTable: #getField: #fieldNamed: #parameter: #noneSatisfy: #noneSatisfyExists: #getConstant: #count: #sum: #min: #max: #average: #sqlSelect:)
!

performSpecialExpression: aSymbol withArguments: anArray
	| symbolToPerform |
	symbolToPerform := self specialSymbolToPerformFor: aSymbol.
	(self zeroArgumentSpecialSelectors includes: symbolToPerform) ifTrue: [^self perform: symbolToPerform].
	(self oneArgumentSpecialSelectors includes: symbolToPerform) ifTrue: [^self perform: symbolToPerform with: anArray first ].
	(self twoArgumentSpecialSelectors includes: symbolToPerform) ifTrue: [^self perform: symbolToPerform with: (anArray at: 1) with: (anArray at: 2)].
	^nil.
!

specialSymbolToPerformFor: aSymbol
	"Avoid polymorphism between things you can send to an expression inside a query block and things you can do to an expression to iterate over an expression true"
	aSymbol == #anySatisfy: ifTrue: [^#anySatisfyDefault:].
	aSymbol == #select: ifTrue: [^#sqlSelect:].
	^aSymbol.
!

twoArgumentSpecialSelectors
	^#(#aggregate:as:)
!

zeroArgumentSpecialSelectors
	^#(#asOuterJoin #sqlCount #sqlSum #sqlMin #sqlMax #sqlAverage #sqlAvg).
! !

!ObjectExpression methodsFor:'preparing'!

assignTableAliasesStartingAt: anInteger prefix: aString
	| tableNumber prefix |
	self controlsTables ifFalse: [^anInteger].
	self hasTableAliases ifTrue: [^anInteger].
	tableNumber := anInteger.
	prefix := aString isNil ifTrue: ['t'] ifFalse: [aString].
	self tables
		do:
			[:each | 
			self aliasTable: each to: prefix , tableNumber printString.
			tableNumber := tableNumber + 1].
	^tableNumber.
! !

!ObjectExpression methodsFor:'printing'!

printTableAliasesOn: aStream 
	self hasTableAliases 
		ifTrue: 
			[aStream nextPutAll: ' '.
			tableAliases keysAndValuesDo: [:eachKey :eachValue | 
				aStream nextPutAll: eachKey name, '->', eachValue name , ' ']]
! !

!ObjectExpression methodsFor:'private/accessing'!

removeMappingExpression: anExpression
	"Private. Normally you would never do this, but in the case of an anySatisfy: or allSatisfy: we want to have each of them as distinct joins, so we will remove the entry from the mappingExpression of the base, making sure that relationship will not be used for anything else. Since any/allSatisfy: is the only valid use of a collection relationship, we don't have to worry about whether it was used for something else earlier."

	self mappingExpressions removeKey: anExpression name.
!

requiresDistinct
	^requiresDistinct
!

requiresDistinct: aBoolean 
	requiresDistinct := aBoolean
! !

!ObjectExpression methodsFor:'testing'!

canTranslateFields
	"Return true if we are a mapping expression and can thus translate from fields into their aliased version for a particular query. More generally, do we respond to translateField:"
	^true.
!

hasProvidedAdditionalExpressions
	"Return true if we have already provided additional expressions to some query. Only important when we're preparing a subselect and don't want to bring in joins that are part of the main query"
	^self hasTableAliases.
!

hasTableAliases
	^tableAliases notNil
! !

!ObjectExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ObjectExpression.st,v 1.1 2006/07/26 08:28:28 boris Exp $'
! !
