"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabasePlatform subclass:#OraclePlatform
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!OraclePlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!OraclePlatform methodsFor:'SQL'!

isOraclePlatform

	^true
!

supportsConstraints

	^true.
! !

!OraclePlatform methodsFor:'accessing'!

vwEXDIConnectionClass
	^'OracleThreadedConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'OracleThapiEXDI'
! !

!OraclePlatform methodsFor:'binding'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand arrayBindings.
! !

!OraclePlatform methodsFor:'constants'!

areSequencesExplicitlyCreated
	^true.
!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^30
!

primaryKeysAreAutomaticallyUnique
	"Return false if, in addition to specifying something as a primary key, we must separately specify it as unique"

	^true.
!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^false.
!

supportsMillisecondsInTimes
	^false.
!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^true.
! !

!OraclePlatform methodsFor:'conversion-strings'!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: 'HEXTORAW('''.
	aByteArray do: [:each |
		each printOn: aStream paddedWith: $0 to: 2 base: 16].
	aStream nextPutAll: ''')'.
! !

!OraclePlatform methodsFor:'conversion-times'!

printDate: aDate for: aType
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: 'to_date('''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'YYYY-MM-DD'.
	stream nextPutAll: ''')'.
	^stream contents.
!

printTime: aTime for: aType
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: 'to_date('''.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'HH24:MI:SS'.
	stream nextPutAll: ''')'.
	^stream contents.
!

printTimestamp: aTimestamp on: stream for: aType

	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: 'to_date('''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $ .
	self
		printTime: aTimestamp
		isoFormatOn: stream
		milliseconds: false.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'YYYY-MM-DD HH24:MI:SS'.
	stream nextPutAll: ''')'.
! !

!OraclePlatform methodsFor:'database-specific'!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	aSymbol == #EXCEPT ifTrue: [^'MINUS'].
	^aSymbol.
! !

!OraclePlatform methodsFor:'sequences'!

databaseSequenceClass
	^OracleSequence.
! !

!OraclePlatform methodsFor:'table creation'!

printNumericFieldOfSize: anInteger on: aStream

	aStream 
		nextPutAll: 'number('.
	anInteger printOn: aStream.
	aStream nextPutAll: ')'.
!

printStringFieldOfSize: anInteger on: aStream

	aStream 
		nextPutAll: 'varchar2('.
	anInteger printOn: aStream.
	aStream nextPutAll: ')'.
! !

!OraclePlatform methodsFor:'testing'!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.
!

supportsGroupWritingFor: aCommand
	^self supportsBinding and: [self useBinding and: [aCommand supportsGroupWriting]].
!

supportsTableOwners
	^true
!

useOracleOuterJoins
	"Return true if we use the old Oracle x = y (+) syntax for outer joins"
	^true.
!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^true.
! !

!OraclePlatform methodsFor:'types'!

boolean
	^self int2.
!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].
!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'clob'].
!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new typeString: 'date'].
!

double
	^self float8
!

float
	^self float4.
!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new typeString: 'float'].
!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new typeString: 'double precision'].
!

int2

	^self integer
!

int4

	^self integer
!

int8

	^self integer
!

number

	^self typeNamed: #number ifAbsentPut: [GlorpNumericType new typeString: 'number'].
!

numeric
	^self number.
!

sequence

	^self serial.
!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: (self integer typeString)].
!

text
	^self clob.
!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'date'].
!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'date'].
!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new typeName: 'varchar2'].
! !

!OraclePlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/OraclePlatform.st,v 1.1 2006/07/26 08:18:37 boris Exp $'
! !
