"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#ParameterExpression
	instanceVariableNames:'field base'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!ParameterExpression class methodsFor:'instance creation'!

forField: aField basedOn: anObjectExpression

	^self new field: aField base: anObjectExpression; yourself
! !

!ParameterExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ParameterExpression methodsFor:'accessing'!

base

	^base
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

field
	^field
! !

!ParameterExpression methodsFor:'api'!

get: aSelector withArguments: anArray
	| functionExpression |
	functionExpression := self getFunction: aSelector withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	^RelationExpression
		named: aSelector
		basedOn: self
		withArguments: anArray
! !

!ParameterExpression methodsFor:'initialize/release'!

field: aDatabaseField base: aBaseExpression 
	field := aDatabaseField.
	base := aBaseExpression
! !

!ParameterExpression methodsFor:'iterating'!

do: aBlock skipping: aSet

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	aBlock value: self.
! !

!ParameterExpression methodsFor:'navigating'!

queryLevelBaseExpression
	^base queryLevelBaseExpression.
!

ultimateBaseExpression
	^base ultimateBaseExpression.
! !

!ParameterExpression methodsFor:'preparing'!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	^source getField: field.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	self == anExpression ifTrue: [^aBaseExpression].
	self base == ultimateBaseExpression ifTrue: [^self].
	^aBaseExpression getParameter: field.
! !

!ParameterExpression methodsFor:'printing'!

convertedDbValueOf: anObject
	"We don't do any conversion"
	^anObject
!

printOn: aStream 
	aStream nextPutAll: 'Parameter('.
	self printTreeOn: aStream.
	aStream nextPut: $)
!

printOnlySelfOn: aStream

	field printSQLOn: aStream withParameters: #()
!

printSQLOn: aStream withParameters: aDictionary
	self field type print: (self valueIn: aDictionary) on: aStream.
!

printTreeOn: aStream 
	[field printSQLOn: aStream withParameters: #()] on: Error do: [:ex | field printOn: aStream].
!

valueIn: aDictionary 
	^aDictionary at: field.
! !

!ParameterExpression methodsFor:'testing'!

canBind
	"Return true if this represents a value that can be bound into a prepared statement"
	^true.
! !

!ParameterExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ParameterExpression.st,v 1.1 2006/07/26 08:29:44 boris Exp $'
! !
