"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabasePlatform subclass:#PostgreSQLPlatform
	instanceVariableNames:'doubleClass'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!PostgreSQLPlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!PostgreSQLPlatform methodsFor:'SQL'!

isPostgreSQLPlatform
	^true.
!

supportsConstraints
	^true.
! !

!PostgreSQLPlatform methodsFor:'accessing'!

vwEXDIConnectionClass
	^'PostgreSQLEXDIConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'PostgreSQLEXDI'
! !

!PostgreSQLPlatform methodsFor:'binding'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.
! !

!PostgreSQLPlatform methodsFor:'constants'!

areSequencesExplicitlyCreated
	^false.
!

maximumLengthOfColumnName
	"^<Integer> I return the max. length of a table name"

	^31
!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^31
!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^false.
!

supportsMillisecondsInTimes
	"I think this depends on settings, but it's easier right now to just turn it off"
	^false.
! !

!PostgreSQLPlatform methodsFor:'conversion-strings'!

byteArrayToSQLStringConverter

	^DelegatingDatabaseConverter
		named: #byteArrayToSQLString
		hostedBy: self
		fromStToDb: #nullConversion:for:
		fromDbToSt: #convertSQLStringToByteArray:for:.
!

convertSQLStringToByteArray: aString for: aType

	| aStream str | 
	aString isNil ifTrue: [^nil].
	aStream := (ByteArray new: aString size // 4) writeStream.
	str := aString readStream.
	[str atEnd] whileFalse: [ |nextChar|
		nextChar := str next.
		aStream nextPut: (nextChar = $\
								ifTrue: [str peek = $\
											ifTrue: [str next asInteger]
											ifFalse: [Number readFrom: (str next: 3) base: 8]]
								ifFalse: [nextChar asInteger])
	].
	aStream close.
	^aStream contents
!

printBlob: aByteArray on: aStream for: aType
	| base64Stream |
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	Dialect isSqueak ifTrue: [^self squeakPrintBlob: aByteArray on: aStream for: aType].
	aStream nextPut: $'.
	base64Stream := (Dialect smalltalkAt: #Base64EncodingWriteStream) onStream: aStream.
	base64Stream lineBreaksEnabled: false.
	base64Stream nextPutAll: aByteArray.
	base64Stream emitClosingCharacters.
	aStream nextPut: $'.
!

squeakPrintBlob: aByteArray on: aStream for: aType

	aStream nextPut: $'.
	aByteArray do: [:each |
		aStream nextPutAll: '\\'.
		each asInteger printOn: aStream paddedWith: $0 to: 3 base: 8].
	aStream nextPutAll: '''::bytea'.
! !

!PostgreSQLPlatform methodsFor:'conversion-times'!

printDate: aDate for: aType
	"Print a date (or timestamp) as yyyy-mm-dd"
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $'.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPut: $'.
	^stream contents.
!

printTime: aTime for: aType
	"Print a time (or timestamp) as hh:mm:ss.fff"
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $'.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPut: $'.
	^stream contents.
!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPut: $'.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $'.
! !

!PostgreSQLPlatform methodsFor:'database-specific'!

printOffset: anInteger on: aCommand
	aCommand
		nextPutAll: ' OFFSET '.
	anInteger printOn: aCommand.
!

printPostLimit: anInteger on: aCommand
	aCommand
		nextPutAll: ' LIMIT '.
	anInteger printOn: aCommand.
! !

!PostgreSQLPlatform methodsFor:'sequences'!

databaseSequenceClass
	^PGSequence.
! !

!PostgreSQLPlatform methodsFor:'testing'!

canBind: aValue to: aType
	"The Postgres EXDI seems to have trouble binding date formats"
	aType = self date ifTrue: [^false].
	aType = self time ifTrue: [^false].
	aType = self timestamp ifTrue: [^false].
	aType = self timetz ifTrue: [^false].
	(Dialect isVisualWorks and: [aValue class == Dialect doublePrecisionFloatClass]) ifTrue: [^false].
	(Dialect isVisualWorks and: [aValue class == Dialect fixedPointClass]) ifTrue: [^false].
	^super canBind: aValue to: aType.
!

doubleClass
	"cache this, because otherwise we need to look it up on every parameter bind"
	doubleClass == nil ifTrue: [doubleClass := Dialect smalltalkAt: #Double].
	^doubleClass.
!

measuresStringFieldWidthsInBytes
	"Return true if we measure the width of a string in bytes rather than characters. So, a 255-character field can hold 255 bytes worth of string, not 255 characters. This is important with encodings"
	^true.
!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax."
	^true.
!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.
!

supportsCaseInsensitiveLike
	^true.
!

supportsGroupWritingFor: aCommand
	^aCommand supportsGroupWriting.
!

supportsImplicitDroppingOfConstraints

        ^true
!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^true.
!

supportsOffset
	"Do we support anything analogous to the postgresql OFFSET, skipping the first N rows"
	^true.
!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^false.
! !

!PostgreSQLPlatform methodsFor:'types'!

bigint
	^self int8
!

blob

	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'bytea'].
!

boolean
	^self typeNamed: #boolean ifAbsentPut: [GlorpBooleanType new typeString: 'boolean'].
!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'text'].
!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new typeString: 'date'].
!

decimal
	^self numeric.
!

double
	^self float8
!

float
	^self float4.
!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new].
!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new].
!

int
	^self int4
!

int2
	^self typeNamed: #int2 ifAbsentPut: [GlorpIntegerType new typeString: 'int2'].
!

int4
	^self typeNamed: #int4 ifAbsentPut: [GlorpIntegerType new typeString: 'int4'].
!

int8
	^self typeNamed: #int8 ifAbsentPut: [GlorpIntegerType new typeString: 'int8'].
!

integer

	^self int8.
!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].
!

real
	^self float4
!

sequence

	^self serial
!

serial
	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'serial'].
!

smallint
	^self int2
!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time'].
!

timeStampTypeString
	^'timestamp'.
!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'timestamp'].
!

timetz
	^self typeNamed: #timetz ifAbsentPut: [GlorpPGTimeWithTimeZoneType new].
!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].
! !

!PostgreSQLPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/PostgreSQLPlatform.st,v 1.1 2006/07/26 08:14:37 boris Exp $'
! !
