"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Query
	instanceVariableNames:'session prepared expectedRows collectionType ordering'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!Query class methodsFor:'instance creation'!

count: aClass
	^self count: aClass where: nil.
!

count: aClass where: anExpression
	| query |
	query := self returningOneOf: aClass where: anExpression.
	query retrieve: #countStar.
	^query.
!

new

	^super new initialize.
!

read: aClass
	"readManyOf: and returningManyOf: are synonyms. This now seems more natural to me, to be consistent with session API"
	^self returningManyOf: aClass where: nil.
!

read: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression.
!

readManyOf: aClass
	"readManyOf: and returningManyOf: are synonyms. This now seems more natural to me, to be consistent with session API"
	^self returningManyOf: aClass where: nil.
!

readManyOf: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression.
!

readOneOf: aClass
	^self returningOneOf: aClass.
!

readOneOf: aClass where: anExpression
	^self returningOneOf: aClass where: anExpression.
!

returningManyOf: aClass
	^self returningManyOf: aClass where: nil.
!

returningManyOf: aClass where: anExpression
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningManyOf: aClass where: anExpression.
!

returningOneOf: aClass
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningOneOf: aClass.
!

returningOneOf: aClass where: anExpression
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningOneOf: aClass where: anExpression.
! !

!Query class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Query methodsFor:'accessing'!

collectionType
	"Note that queries default the collection type to array, while mappings default to OrderedCollection. I think it makes sense"
	collectionType isNil ifTrue: [collectionType := Array].
	^collectionType.
!

collectionType: aClass
	collectionType := aClass.
!

expectedRows
	"How many rows do we think it's likely this query will bring back. Used for tweaking things like block factor"
	^expectedRows isNil
		ifTrue: [expectedRows := self readsOneObject ifTrue: [1] ifFalse: [100]]
		ifFalse: [expectedRows].
!

expectedRows: anInteger
	"How many rows do we think it's likely this query will bring back. Used for tweaking things like block factor"
	expectedRows := anInteger
!

platform
	^session system platform.
!

readsOneObject
	self subclassResponsibility.
!

session
	^session
!

session: aSession 
	session := aSession
! !

!Query methodsFor:'copying'!

copy
	^self shallowCopy postCopy.
!

postCopy

	prepared := false.
! !

!Query methodsFor:'executing'!

executeIn: aSession

	^self executeWithParameters:#() in: aSession.
!

executeWithParameters: parameterArray in: aSession 
	self subclassResponsibility.
!

resultCollectionFor: aCursoredStream
        | results finalAnswer |
        collectionType isNil ifTrue: [^aCursoredStream upToEnd].
        collectionType == aCursoredStream class ifTrue: [^aCursoredStream].
       " results := ((aCursoredStream collectionTypeFor: collectionType) new: self expectedRows writeStream."
        results := WriteStream on: (OrderedCollection new: self expectedRows).
        results nextPutAll: aCursoredStream upToEnd.
        finalAnswer := (aCursoredStream collectionTypeFor: collectionType) withAll: results contents.
        session register: finalAnswer.
        ^finalAnswer.
! !

!Query methodsFor:'initialize'!

initialize

	prepared := false.
! !

!Query methodsFor:'ordering'!

expressionBlockFor: anOrderingCriteria
	"Allow us to use symbols interchangeably with simple blocks for ordering, so 
	#firstName is equivalent to [:each | each firstName]. Also, allow chains of symbols, so #(owner firstName)"

	anOrderingCriteria isGlorpExpression ifTrue: [^anOrderingCriteria].
	anOrderingCriteria glorpIsBlock ifTrue: [^anOrderingCriteria].

	anOrderingCriteria isSymbol 
		ifTrue: [^[:each | each perform: anOrderingCriteria]].
	
	"otherwise, we assume it's a collection of symbols, the only other valid case"
	anOrderingCriteria do: [:each | each isSymbol ifFalse: [self error: 'invalid ordering criteria']].
	^[:each | anOrderingCriteria inject: each into: [:sum :eachExpression |
		sum perform: eachExpression]].
!

orderBy: aBlock
	ordering isNil 
		ifTrue: [ordering := Array with: aBlock]
		ifFalse: [ordering := ordering , (Array with: aBlock)].
!

ordering
	^ordering.
!

setOrdering: aCollection
	ordering := aCollection.
! !

!Query methodsFor:'printing'!

printOn: aStream
	"I hate the printing with namespaces in vw"
      | title |
        title := self class name.
        aStream nextPutAll:
                ((title at: 1) isVowel ifTrue: ['an '] ifFalse: ['a ']).
        aStream nextPutAll: title
! !

!Query methodsFor:'specifying retrievals'!

retrieveDictionaryWithKey: keyExpression value: valueExpression

	self collectionType: (GlorpDictionaryDescription new keyExpression: keyExpression; valueExpression: valueExpression).
!

returnsDictionary

	^collectionType class == GlorpDictionaryDescription.
! !

!Query methodsFor:'sql generation'!

printCompoundSelectAliasesOn: aStream
	^self.
!

printOrderingOn: aStream
	ordering isNil ifTrue: [^self].
	aStream nextPutAll: ' ORDER BY '.
	GlorpHelper
		do: [:each | each printSQLOn: aStream withParameters: nil]
		for: ordering
		separatedBy: [aStream nextPutAll: ', '].
!

rewriteExcept
	"Rewrite any EXCEPT operation we have into a NOT EXISTS tests. Only a CompoundQuery would have such an operation"
!

rewriteIntersect
	"Rewrite any INTERSECT operation we have into an EXISTS tests. Only a CompoundQuery would have such an operation"
! !

!Query class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Query.st,v 1.1 2006/07/26 08:18:25 boris Exp $'
! !
