"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

AbstractReadQuery subclass:#ReadQuery
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

ReadQuery comment:'This represents a general read query. By general we mean that it might require more than one trip to the database. It computes a "tracing" indicating which groups of objects can be read simultaneously, then constructs a group of corresponding SimpleQuery instances and executes them.'
!


!ReadQuery class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ReadQuery methodsFor:'accessing'!

readFromDatabaseWithParameters: anArray 
	| col |
	col := OrderedCollection new.
	self descriptor classesRequiringIndependentQueries do: 
			[:aClass | 
			| simpleQuery result |
			simpleQuery := self asSimpleQueryFor: aClass.
			result := simpleQuery readFromDatabaseWithParameters: anArray.
			simpleQuery readsOneObject 
				ifTrue: [col add: result]
				ifFalse: [col addAll: result]].
	^col
! !

!ReadQuery methodsFor:'converting'!

asSimpleQueryFor: aClass
	| newQuery newCriteria newBase |
	"Rebuild the expression, because this means a full query is being split into multiple sub-queries, e.g. for an inheritance read. The expression may get prepared differently in each case (e.g. table aliases), so we can't share"
	newBase := BaseExpression new.
	newCriteria := whereClause rebuildOn: newBase.
	newQuery := SimpleQuery new
		initResultClass: aClass
		whereClause: newCriteria
		singleObject: readsOneObject.
	newQuery session: session.
	newQuery proxyType: self proxyType.
	newQuery shouldRefresh: self shouldRefresh.
	newQuery setOrdering: ordering.
	newQuery collectionType: collectionType.
	newQuery setUpExpressions.
	newQuery tracing: tracing.
	newQuery readsOneObject: readsOneObject.
	newQuery expectedRows: expectedRows.
	^newQuery.
! !

!ReadQuery class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ReadQuery.st,v 1.1 2006/07/26 08:17:22 boris Exp $'
! !
