"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

GlorpExpression subclass:#RelationExpression
	instanceVariableNames:'relation leftChild rightChild outerJoin'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!RelationExpression class methodsFor:'instance creation'!

named: aSymbol basedOn: anExpression withArguments: anArray

	^self new
		named: aSymbol
		basedOn: anExpression
		withArguments: anArray.
! !

!RelationExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!RelationExpression methodsFor:'As yet unclassified'!

allTargetFields
	"Assume that part of us is a join, and the rest doesn't count"
	^self inject: OrderedCollection new into: [:sum :each |
		each isJoin ifTrue: [sum addAll: each allTargetFields].
		sum].
!

canUseBinding
	"Return true if we can use binding for our right child's value"
	^self expectsCollectionArgument not.
!

expectsCollectionArgument
	^self relationsWithCollectionArguments includes: relation.
!

primaryKeyFromDictionary: aDictionary
	"Given a set of parameters, return a primary key suitable for retrieving our target. Do this only if the expression is for a primary key, and has no other conditions than the primary key one"

	| left right field |
	relation = #AND ifTrue: [
		left := leftChild primaryKeyFromDictionary: aDictionary.
		right := rightChild primaryKeyFromDictionary: aDictionary.
		^(left notNil and: [right notNil]) ifTrue: [Array with: left with: right] ifFalse: [nil]].
	relation = #= ifFalse: [^nil].
	field := leftChild field.
	field isNil ifTrue: [^nil].
	field isGlorpExpression ifTrue: [^nil].
	^(field isPrimaryKey)
		ifTrue: [rightChild valueIn: aDictionary]
		ifFalse: [nil].
!

printForANSIJoinTo: table on: aCommand
	"Print ourselves as table JOIN otherTable USING (criteria). Return the table we joined"

	self outerJoin
		ifTrue: [aCommand nextPutAll: ' LEFT OUTER JOIN ']
		ifFalse: [aCommand nextPutAll: ' INNER JOIN '].
	aCommand nextPutAll: table sqlTableName.
	aCommand nextPutAll: ' ON '.
	self printSQLOn: aCommand withParameters: aCommand parameters.
	^table.
!

relationsWithCollectionArguments
	^#(#IN #'NOT IN')
!

tableToJoinFrom: aTableCollection
	"Return a table we could legitimately use at this point to write as an ANSI join - meaning that all but one must be  already used and thus available, and one that isn't. aTableCollection lists the used ones"
	| possibleTables |
	possibleTables := self tablesForANSIJoin asArray
		select: [:each | (aTableCollection includes: each) not].
	^possibleTables size = 1 ifTrue: [possibleTables first] ifFalse: [nil].
!

tablesForANSIJoin
	"Which tables will we join. Assumes this is a single-level join"
	^self inject: Set new into: [:sum :each |
		each tableForANSIJoin isNil ifFalse: [sum add: each tableForANSIJoin].
		sum].
!

targetKeys
	"Assume that part of us is a join, and the rest doesn't count"
	^self inject: OrderedCollection new into: [:sum :each |
		each isJoin ifTrue: [sum addAll: each targetKeys].
		sum].
!

useBindingFor: aValue to: aType in: aCommand
	"Return true if we can use binding for our right child's value, in the context of this command"
	aCommand useBinding ifFalse: [^false].
	self expectsCollectionArgument ifTrue: [^false].
	^aCommand canBind: aValue to: aType.
! !

!RelationExpression methodsFor:'accessing'!

beOuterJoin
	outerJoin := true.
	leftChild beOuterJoin.
	rightChild beOuterJoin.
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

isOuterJoin
	outerJoin isNil ifTrue: [outerJoin := false].
	^outerJoin.
!

leftChild
	^leftChild
!

leftChild: anExpression 
	leftChild := anExpression
!

outerJoin
	outerJoin isNil ifTrue: [outerJoin := false].
	^outerJoin.
!

outerJoin: aBoolean
	outerJoin := aBoolean.
!

relation

	^relation.
!

relation: aSymbol

	relation := aSymbol.
!

rightChild
	^rightChild
!

rightChild: anExpression
	rightChild := anExpression
! !

!RelationExpression methodsFor:'api'!

get: aSymbol withArguments: anArray
	"We treat NOT as a function, so we have to check for functions here"
	| functionExpression |
	functionExpression := self getFunction: aSymbol withArguments: anArray.
	functionExpression isNil ifFalse: [^functionExpression].
	^anArray size = 1
		ifTrue: [RelationExpression named: aSymbol basedOn: self withArguments: anArray]
		ifFalse: ["This is either an error, or something defined as a selector on relation expressions"
			self perform: aSymbol withArguments: anArray.].
! !

!RelationExpression methodsFor:'converting'!

asGeneralGlorpExpression
	"Convert the result to a general (tree-format) expression, if it's the more limited join expression."
	| result |
	result := self copy.
	result leftChild: leftChild asGeneralGlorpExpression.
	result rightChild: rightChild asGeneralGlorpExpression.
	^result.
!

asGeneralGlorpExpressionWithFields
	"Convert the result to a general (tree-format) expression, if it's the more limited join expression."
	| result |
	result := self copy.
	result leftChild: leftChild asGeneralGlorpExpressionWithFields.
	result rightChild: rightChild asGeneralGlorpExpressionWithFields.
	^result.
!

asIndependentJoins
	| |
	"If this is an ANDed clause, referring to two different tables split it into independent joins"
	relation == #AND ifFalse: [^Array with: self].
	leftChild tablesForANSIJoin asSortedCollection = rightChild tablesForANSIJoin asSortedCollection
		ifTrue: [^Array with: self].
	^(Array with: leftChild with: rightChild) inject: OrderedCollection new into: [:sum :each |
		sum addAll: each asIndependentJoins. sum].
!

asJoin
	| leftField rightField |
	leftField := self leftChild field.
	rightField := self rightChild field.
	(leftField notNil and: [rightField notNil]) ifTrue: [^Join from: leftField to: rightField].
	^leftChild asJoin, rightChild asJoin.
!

generalExpressionPart
	| left right |
	left := leftChild generalExpressionPart.
	right := rightChild generalExpressionPart.
	(left isNil and: [right isNil]) ifTrue: [^nil].
	left isNil ifTrue: [^right].
	right isNil ifTrue: [^left].
	^self.
!

sourceForTarget: aField

	| left |
	relation = #= ifFalse: [
		left := leftChild sourceForTarget: aField.
		left isNil ifFalse: [^left].
		^rightChild sourceForTarget: aField].
	rightChild field == aField ifTrue: [^leftChild field].
	leftChild field == aField ifTrue: [^rightChild field].
	^nil.
! !

!RelationExpression methodsFor:'copying'!

createFor: arguments withBase: anExpression symbol: aSymbol
	"Create a copy of our expression with the given parameters. Used in function creation, for 'functions' like isNil that really just turn into relations."

	^self class
		named: relation
		basedOn: anExpression
		withArguments: (Array with: rightChild).
! !

!RelationExpression methodsFor:'iterating'!

convertValueOf: anObject in: aDictionary
	| translated convertedValue |

	translated := anObject isGlorpExpression 
		ifTrue: [anObject valueIn: aDictionary]
		ifFalse: [anObject].
	translated isGlorpExpression ifTrue: [^translated].
	convertedValue := self expectsCollectionArgument 
		ifTrue: [translated collect: [:each | self leftChild convertedDbValueOf: each]]
		ifFalse: [self leftChild convertedDbValueOf: translated].
	^convertedValue.
!

do: aBlock skipping: aSet
	| clauses |
	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	leftChild hasImpliedClauses
		ifTrue:
			[clauses := leftChild allRelationsFor: self.
			clauses do: [:each | each do: aBlock skipping: aSet]]
		ifFalse:
			[leftChild do: aBlock skipping: aSet.
			rightChild do: aBlock skipping: aSet.
			aBlock value: self.].
! !

!RelationExpression methodsFor:'navigating'!

queryLevelBaseExpression

	^leftChild canHaveBase ifTrue: [leftChild queryLevelBaseExpression] ifFalse: [rightChild queryLevelBaseExpression].
!

ultimateBaseExpression

	^leftChild canHaveBase ifTrue: [leftChild ultimateBaseExpression] ifFalse: [rightChild ultimateBaseExpression].
! !

!RelationExpression methodsFor:'preparing'!

addBoundExpressionsIn: aCommand to: boundExpressions
	"If our right-child can be used for binding, add it to the list. We need to do this at this level because the expressions themselves don't know what type they'll be matched against"

	| translated |
	rightChild canBind ifFalse: [^self].
	(boundExpressions includes: self) ifTrue: [^self].
	translated := self convertValueOf: rightChild in: aCommand parameters.
	(self useBindingFor: translated to: leftChild field type in: aCommand) ifTrue: [boundExpressions add: self].
!

additionalExpressions

	^#().
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	| left right |
	left := leftChild asExpressionJoiningSource: source toTarget: target.
	right := rightChild asExpressionJoiningSource: source toTarget: target.
	^self class new 
		relation: relation;
		leftChild: left;
		rightChild: right.
!

bindingIn: aCommand

	^self convertValueOf: rightChild in: aCommand parameters.
!

bindingType
	"If we're to be used for binding, return the field type to be bound to"

	^self leftChild bindingType.
!

condensePrimaryKeyComparison
	"This tries to condense expressions of the form
         something relatedObject id = x
      with 
          something foreignKeyToRelatedObjectId = x, saving a join (and making the thing work at all, in the case where x = nil"

	| fieldBeingCompared baseObjectExpression join sourceField |
	(relation = #= or: [relation = #<>]) ifFalse: [^self].
	self leftChild mappedFields size = 1 ifFalse: [^self].
	fieldBeingCompared := self leftChild field.
	fieldBeingCompared class == DatabaseField ifFalse: [^self].
	fieldBeingCompared isPrimaryKey ifFalse: [^self].
	baseObjectExpression := self leftChild base.
	baseObjectExpression representsDerivedObject ifFalse: [^self].
	join := baseObjectExpression join asGlorpExpressionOn: baseObjectExpression base.
	sourceField := join sourceForTarget: fieldBeingCompared.
	sourceField isNil ifTrue: [^self].

	"OK, we've got a match, replace ourselves with the shortcut"
	^(self leftChild base base getField: sourceField) get: relation withArguments: (Array with: self rightChild).
!

condensePrimaryKeyComparisons
	"Because we're replacing the expression, we need to do it at the parent and replace the previous child"
	leftChild := leftChild condensePrimaryKeyComparison.
	rightChild := rightChild condensePrimaryKeyComparison.
!

hasBindableExpressionsIn: aCommand
	"Return true if our right-child can be used for binding. We need to do this at this level because the expressions themselves don't know what type they'll be matched against"

	| translated |
	rightChild canBind ifFalse: [^false].
	translated := self convertValueOf: rightChild in: aCommand parameters.
	^self useBindingFor: translated to: leftChild field type in: aCommand.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	| expression |
	self == anExpression ifTrue: [^aBaseExpression].
	expression := (leftChild rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression)
		get: relation
		withArguments:
			(Array
				with:
					(self rightChildToRebuild rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression)).
	self isOuterJoin ifTrue: [expression beOuterJoin].
	^expression.
!

replaceJoinsWithInverse
	"Replace any actual Join objects we have with their inverse. Only useful if we have something that has mixed joins and general expressions, where we assume we don't want to mess with the general ones, because they're not field-level"
	leftChild isJoin ifTrue: [leftChild := leftChild inverseJoin].
	rightChild isJoin ifTrue: [rightChild := rightChild inverseJoin].
!

rewriteEquality
	| keyEquality |
	leftChild hasDescriptor ifFalse: [^self].
	(relation = #= | (relation = #<>) | (relation = #IN)) ifFalse: [^self].
	(relation = #IN and: [leftChild descriptor primaryTable primaryKeyFields size > 1]) ifTrue: [
		(GlorpInvalidExpressionError signal: 'Cannot convert an object comparison to an IN comparison when the table uses a composite primary key')].
	keyEquality := leftChild descriptor
		primaryKeyExpressionFor: rightChild
		basedOn: leftChild
		relation: relation.
	^keyEquality.
!

rewriteEqualityExpressions
	leftChild := leftChild rewriteEquality.
	rightChild := rightChild rewriteEquality.
!

rightChildToRebuild
	^rightChild.
! !

!RelationExpression methodsFor:'printing'!

printOnlySelfOn: aStream

	aStream nextPutAll: relation.
!

printTreeOn: aStream 
	aStream
		print: leftChild;
		space;
		nextPutAll: relation;
		space;
		print: rightChild
! !

!RelationExpression methodsFor:'printing SQL'!

printBasicSQLOn: aCommand withParameters: aDictionary 
	aCommand platform parenthesizeCommandsInCompoundQuery ifTrue: [aCommand nextPut: $(].
	leftChild printSQLOn: aCommand withParameters: aDictionary.
	self 
		printComparisonTo: rightChild
		withParameters: aDictionary
		on: aCommand.
	self printOracleOuterJoinOn: aCommand.
	aCommand platform parenthesizeCommandsInCompoundQuery ifTrue: [aCommand nextPut: $)]
!

printComparisonTo: value withParameters: aDictionary on: aStream
	"Horribly convoluted logic to handle the cases where the value might be a constant, an expression that results in a value (constant or parameter) or a regular expression, with the caveat that any value that turns out to be null has to be printed with IS NULL rather than = NULL."

	| translated |
	translated := self convertValueOf: value in: aDictionary.
	translated isGlorpExpression
		ifTrue:
			[
			translated isEmptyExpression ifFalse: [
				self printRelationOn: aStream.
				translated
					printSQLOn: aStream
					withParameters: aDictionary]]
		ifFalse:
			[self
				printSimpleValueComparisonTo: translated
				on: aStream].
!

printMicrosoftOuterJoinOn: aCommand
	self isOuterJoin ifFalse: [^self].
	aCommand platform useMicrosoftOuterJoins ifTrue: [
		aCommand nextPutAll: '*'].
!

printOracleOuterJoinOn: aCommand
	self isOuterJoin ifFalse: [^self].
	(self leftChild class == self class and: [self leftChild isOuterJoin]) ifTrue: [^self].
	(self rightChild class == self class and: [self rightChild isOuterJoin]) ifTrue: [^self].
	aCommand platform useOracleOuterJoins ifTrue: [
		aCommand nextPutAll: ' (+) '].
!

printRelationOn: aStream
	aStream space.
	self printMicrosoftOuterJoinOn: aStream.
	aStream 
		nextPutAll: self relation;
		space.
!

printSQLOn: aStream withParameters: aDictionary
	self leftChild hasImpliedClauses
		ifTrue:
			[| impliedClauses |
			impliedClauses := self leftChild allRelationsFor: self.
			impliedClauses outerJoin: self outerJoin.
			impliedClauses
				printSQLOn: aStream
				withParameters: aDictionary]
		ifFalse:
			[self
				printBasicSQLOn: aStream
				withParameters: aDictionary].
!

printSimpleValueComparisonTo: value on: aStream
	value isNil
		ifTrue: [self printWithNullOn: aStream]
		ifFalse:
			[self printRelationOn: aStream.
			self printValue: value on: aStream].
!

printValue: value on: aCommand
	| type |
	type := self leftChild type.
	(self useBindingFor: value to: type in: aCommand) ifTrue: [^aCommand nextPutAll: '?'].
	self expectsCollectionArgument
		ifTrue:
			[type
				printCollection: value
				on: aCommand]
		ifFalse:
			[type
				print: value
				on: aCommand].
!

printWithNullOn: aStream
	aStream nextPutAll: ' IS '.
	self relation = #<> ifTrue: [
		aStream nextPutAll: 'NOT '].
	aStream nextPutAll: 'NULL'.
! !

!RelationExpression methodsFor:'private/initializing'!

named: aSymbol basedOn: anExpression withArguments: anArray

	| base right rightBase |
	outerJoin := false.
	relation := self operationFor: aSymbol.
	leftChild := anExpression.

	"The only time we don't expect anExpression to have a base is if it's a constant, in which case the other side should be a variable expression and thus have a base."
	base := anExpression canHaveBase ifTrue: [anExpression ultimateBaseExpression] ifFalse: [anArray first ultimateBaseExpression].
	right := anArray first.
	right isGlorpExpression ifFalse: [rightChild := right asGlorpExpressionOn: base. ^self].
	rightBase := right ultimateBaseExpression.
	rightChild := rightBase ~= base
		ifTrue: [
			right asGlorpExpressionOn: ((rightBase notNil and: [rightBase hasDescriptor]) ifTrue: ["Correlated subselect" rightBase] ifFalse: [base])]
		ifFalse: [right].
!

operationFor: aSymbol
	"Simple translation of operators"

	aSymbol == #AND: ifTrue: [^#AND].
	aSymbol == #& ifTrue: [^#AND].
	aSymbol == #OR: ifTrue: [^#OR].
	aSymbol == #| ifTrue: [^#OR].
	aSymbol == #~= ifTrue: [^#<>].
	 aSymbol == #like: ifTrue: [^#LIKE].
	 aSymbol == #ilike: ifTrue: [^#ILIKE].		"Case-insensitive variant of LIKE. Only supported on PostgreSQL at the moment"
	 aSymbol == #in: ifTrue: [^#IN].
	 aSymbol == #notIn: ifTrue: [^#'NOT IN'].

	^aSymbol.
! !

!RelationExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/RelationExpression.st,v 1.1 2006/07/26 08:28:43 boris Exp $'
! !
