"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Mapping subclass:#RelationshipMapping
	instanceVariableNames:'query join usesLinkTable shouldProxy relevantLinkTableFields
		linkTableJoin targetTableJoin reverseJoin shouldUseFilteredRead
		rowMapKeyConstructorBlock controlsTables isExclusive'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

RelationshipMapping comment:'This is an abstract superclass for relationships between one mapped object and another (or a collection of others).

Instance Variables:
	join	<Join>	The join between our table and the target. Note that if we use a link table, this will be a join between our table and the link table.
	targetTableJoin	<Join>	If we''re using a link table, it may be desirable to also join to attributes of the main table. This optional extra join allows that.

	query	<SimpleQuery>	The query that will return the other side of the relationship, given parameters describing our side.
	relevantLinkTableFields	<Collection>	When there''s a link table, we sometimes needs to know the other side of the relationship, from link table to target. Usually we can figure it out based on the target table and the foreign key constraints, but sometimes (e.g. if there are multiple relationships to the same table) we need to be told which fields are the ones that apply to us
	reverseJoin	<Join>	caches the other end of the relationship, as above.
	rowMapKeyConstructorBlock	<BlockClosure>	Usually row map keys are constructed automatically, but it''s possible that in weird cases we  might need to construct complicated ones (e.g. if there are 3 things that need to participate instead of just two, as in Store''s use of the same link table and the same row to indicate the relationship from package to class and package to metaclass).
	shouldProxy	<Boolean>	Should we proxy or directly construct the referenced object
	shouldUseFilteredRead	<Boolean>	should we use a filtered read.
	usesLinkTable	<Boolean>	Do we use a link table, or is it a direct join to the target.
	isExclusive <Boolean> Is this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted.

'
!


!RelationshipMapping class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!RelationshipMapping methodsFor:'accessing'!

allTables

	(self join) isNil ifTrue: [^#()].
	^(self join) allTables.
!

basicReferenceClass  
        | attr type |
        attr := self attribute.
        type := attr type.
"type isNil ifTrue: [self halt]. 
Transcript show: type printString; cr. "
        "^self attribute type. "
        ^type
!

basicReferenceDescriptor

        ^self system descriptorFor: self basicReferenceClass.
!

collectionType
	^nil.
!

controlsTables: aBoolean
	"We can indicate that this relationship does not actually control its tables. This is going to be true if the parent controls the same tables and coexists with us. Think StoreVersionlessPundle and Pundle"
	controlsTables := aBoolean.
!

join
	join isNil ifTrue: [self initializeJoin].
	^join.
!

join: aJoin
	join := aJoin.
	[join mapping: self] on: MessageNotUnderstood do: [:ex | ex | ex return].
!

linkTableJoin
	linkTableJoin isNil ifTrue: [^self join].
	^linkTableJoin
!

linkTableJoin: aJoin
	linkTableJoin := aJoin.
	[aJoin mapping: self] on: MessageNotUnderstood do: [:ex | ex | ex return].
!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^self join allSourceFields select: [:each | each isMappable].
!

mappingCriteria
	^self join.
!

mappingCriteria: aJoin
	self join: aJoin.
!

referenceClass
	^self attribute type.
!

referenceClass: aClass
	self attribute type: aClass.
!

referenceDescriptor

	^self system descriptorFor: self referenceClass.
!

reverseJoin: aJoin
	"Set the reverse join. Normally this will be calculated based on the foreign key constraints, but in rare cases (e.g. a foreign key that may be to more than one table) you may want to set it explicitly"

	reverseJoin := aJoin.
!

shouldProxy

	^shouldProxy.
!

shouldProxy: aBoolean

	shouldProxy := aBoolean.
!

targetTableJoin
	^targetTableJoin
!

targetTableJoin: aJoin
	"Set the join from the source table to the target table. Note that is is only required if you are using a link table, and in addition to the link table join need to connect from the source table directly to the target table as well.  This is *NOT* supposed to be a join from the link table to the target table. That's reverseJoin, and is calculated automatically. See relevantLinkTableFields if you need to give a hint, or linkTableJoin: if you want to set it explicitly."
	targetTableJoin := aJoin.
	aJoin mapping: self.
!

useLinkTable
	usesLinkTable := true.
!

usesLinkTable
	^usesLinkTable
!

usesLinkTable: anObject
	usesLinkTable := anObject
! !

!RelationshipMapping methodsFor:'api'!

beExclusive
	"Make this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	isExclusive := true.
!

constructRowMapKeyAs: aBlock
	"Give us the opportunity to construct a custom row map key. This is useful if you need to force two relationships to share a link table entry"
	rowMapKeyConstructorBlock := aBlock.
!

isExclusive
	"Return whether this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	^isExclusive.
!

isExclusive: aBoolean
	"Set whether this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	isExclusive := aBoolean.
!

referencedIndependentObjectsFrom: anObject

	| value |
	canWrite ifFalse: [^#()].
	value := self getValueFrom: anObject.
	value == nil ifTrue: [^#()].
	^Array with: value.
!

relevantLinkTableFields
	^relevantLinkTableFields.
!

relevantLinkTableFields: aCollection
	relevantLinkTableFields := aCollection.
!

shouldUseFilteredRead
	^shouldUseFilteredRead.
!

useFilteredRead
	shouldUseFilteredRead := true.
! !

!RelationshipMapping methodsFor:'initializing'!

constraintsFrom: oneTableSet to: anotherTableSet
	^(self oneWayConstraintsFrom: oneTableSet to: anotherTableSet), 
		(self oneWayConstraintsFrom: anotherTableSet to: oneTableSet)
!

initialize

	super initialize.
	shouldProxy := true.
	shouldUseFilteredRead := false.
	usesLinkTable := false.
	controlsTables := true.
	isExclusive := false.
!

initializeJoin
	"We haven't been given a join. There might be a compound one, or else see if we can make an educated guess at one"
	linkTableJoin notNil ifTrue: [join := linkTableJoin, targetTableJoin. ^self].
	self initializeJoinTo: self relatedTables.
!

initializeJoinTo: otherTables
	"We haven't been giving a mapping criteria. See if we can make an educated guess at one"
	| toConstraints fromConstraints numberOfConstraints |
	toConstraints := self oneWayConstraintsFrom: self descriptor tables to: otherTables.
	fromConstraints :=  self oneWayConstraintsFrom: otherTables to: self descriptor tables.
	numberOfConstraints := toConstraints size + fromConstraints size.
	numberOfConstraints = 0 ifTrue: [self error: 'Cannot calculate the join, no constraints found'].
	numberOfConstraints > 1 ifTrue: [self error: 'Cannot calculate the join, too many possibilities'].
	self join: (toConstraints isEmpty ifTrue: [fromConstraints first asReverseJoin] ifFalse: [toConstraints first asJoin]).
!

oneWayConstraintsFrom: oneTableSet to: anotherTableSet
	| matches |
	matches := OrderedCollection new: 3.
	oneTableSet do: [:eachFromTable |
		 anotherTableSet do: [:eachToTable |
			matches addAll:  (eachFromTable foreignKeyConstraints
					select: [:eachConstraint | eachConstraint refersToTable: eachToTable])]].
	^matches.
!

relatedTables
	| otherTables possibleLinkTables |
	otherTables := self basicReferenceDescriptor tables.
	usesLinkTable ifFalse: [^otherTables].

	possibleLinkTables := self system allTables
		select:
			[:each | 
			(each hasConstraintsToAnyOf: self descriptor tables)
				and: [each hasConstraintsToAnyOf: otherTables]].
	possibleLinkTables size ~= 1
		ifTrue: [self error: 'Link table not found or ambiguous. Specify an explicit join'].
	^possibleLinkTables.
! !

!RelationshipMapping methodsFor:'mapping'!

addPhantomMappingExpressionOn: base 
	| phantomMapping phantomMappingName phantomExpression |
	phantomMapping := self reverseMapping.
	phantomMappingName := 'glorp_internal_' , self attributeName , '_to_' 
				, self descriptor describedClass name.
	phantomExpression := base getMapping: phantomMapping
				named: phantomMappingName.
	^phantomExpression
!

buildGeneralQueryIn: anObjectBuilder
	"For filtered queries. We need to build this in terms of the parent query"

	| base parentCriteria fullCriteria phantomExpression newQuery |
	base := BaseExpression new.
	base descriptor: self referenceDescriptor.
	phantomExpression := self addPhantomMappingExpressionOn: base.
	parentCriteria := anObjectBuilder query whereClause.
	fullCriteria := parentCriteria
		rebuildOn: phantomExpression.
	newQuery := Query returningManyOf: self referenceClass.
	"newQuery collectionType: self collectionType."
	fullCriteria isEmptyExpression ifFalse: [newQuery where: fullCriteria].
	self
		setGeneralQueryRetrievalIn: newQuery
		basedOn: base
		phantom: phantomExpression.
	^newQuery.
!

buildQuery
	| toMany |
	toMany := self attribute isCollectionAttribute.
	query := toMany
		ifTrue: [Query returningManyOf: self referenceClass where: self join]
		ifFalse: [Query returningOneOf: self referenceClass where: self join].
	query collectionType: self collectionType.
	self usesLinkTable ifTrue: [
		query AND: (self expressionFromLinkToReferenceTableWithBase: query baseExpression)].
	^query.
!

buildRowMapKeyForSource: source target: target index: index
	^(RowMapKey new)
		key1: target yourSelf;
		key2: source yourSelf.
!

createDeleteRowFor: anObject in: aRowMap
	"This is like firing the mapping, but instead of creating the data, we just need to check if there are link table entries, and if so, make sure they are deleted"
	| target mementoizedTarget|
	canWrite ifFalse: [^self].
	self usesLinkTable ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target ifNone: [^self].
	(self isValidTarget: target) ifTrue: [
		mementoizedTarget := aRowMap collectionMementoFor: target.
		self createDeleteRowForLinkFrom: anObject toTarget: mementoizedTarget in: aRowMap].
!

createDeleteRowForLinkFrom: anObject toTarget: target in: aRowMap
	| linkTable |
	self usesLinkTable ifFalse: [^false].
	linkTable := self linkTableJoin allTables asArray first.
	self attribute
		loopOver: target
		doing:
			[:index :eachTarget | 
			| intermediate |
			intermediate := self
				rowMapKeyForSource: anObject
				target: eachTarget
				index: index.
			aRowMap
				rowForTable: linkTable
				withKey: intermediate
				ifPresent:
					[:linkRow | linkRow isEmpty ifFalse: [linkRow forDeletion: true]]].
!

extendedJoin
	"In order to do a many-to-many read we need more information than just the write, we need to know 
	the relationship to the other table. Construct that based on the table information"

	| generalMappingCriteria base |
	self usesLinkTable ifFalse: [^self join].
	generalMappingCriteria := self join asGeneralGlorpExpression.
	base := generalMappingCriteria ultimateBaseExpression.
	^(self expressionFromLinkToReferenceTableWithBase: base) AND: generalMappingCriteria.
!

isValidTarget: anObject

	^anObject isGlorpProxy
		ifTrue: [anObject isInstantiated]
		ifFalse: [true].
!

knitFromBuilder: sourceBuilder toBuilder: targetBuilder 
	self attribute knit: sourceBuilder instance to: targetBuilder instance
!

mapFromObject: anObject intoRowsIn: aRowMap 
	"Our target is a collection. The tricky bit is that if we're building rows into a RowMapForMementos, then the collection we contain isn't the one we want to use. We want the old version. Ask the row map to give it to us. If it's a normal row map, we'll just get the same thing back" 
	| target mementoizedTarget|
	canWrite ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target ifNone: [^self].
	(self isValidTarget: target) ifTrue: [
		mementoizedTarget := aRowMap collectionMementoFor: target.
		self mapFromObject: anObject toTarget: mementoizedTarget puttingRowsIn: aRowMap].
!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap
	self attribute
		loopOver: target
		doing:
			[:index :eachTarget | 
			| intermediate |
			intermediate := self
				rowMapKeyForSource: anObject
				target: eachTarget
				index: index.
			self
				mapOneFromObject: anObject
				toTarget: eachTarget
				intermediate: intermediate
				atPosition: index
				puttingRowsIn: aRowMap].
!

mapObject: anObject inElementBuilder: anElementBuilder
	self
		setValueIn: anObject
		to:
			(self shouldProxy
				ifTrue:
					[(self newProxyIn: anElementBuilder)
						session: descriptor session;
						parameters: (self parametersIn: anElementBuilder)]
				ifFalse:
					[(self queryFor: anElementBuilder)
						executeWithParameters: (self parametersIn: anElementBuilder)
						in: descriptor session]).
!

mapOneFromObject: sourceObject toTarget: targetObject intermediate: aRowMapKey atPosition: index puttingRowsIn: aRowMap
	self linkTableJoin
		mapFromSource: sourceObject
		andTarget: aRowMapKey
		intoRowsIn: aRowMap.
	self targetTableJoin isNil
		ifFalse:
			[self targetTableJoin
				mapFromSource: sourceObject
				andTarget: (self mapsSimpleObject ifTrue: [aRowMapKey] ifFalse: [targetObject])
				intoRowsIn: aRowMap].
	self usesLinkTable
		ifTrue:
			[self reverseJoin
				mapFromSource: aRowMapKey
				andTarget: (self mapsSimpleObject ifTrue: [aRowMapKey] ifFalse: [targetObject])
				intoRowsIn: aRowMap].
	self
		writeExtraInformationInto: aRowMap
		atKey: aRowMapKey
		forSourceObject: sourceObject
		targetObject: targetObject
		atIndex: index.
!

parametersIn: anElementBuilder
	| parameters |
	self shouldUseFilteredRead ifTrue: [^anElementBuilder command parameters].
	parameters := IdentityDictionary new.
	self join
		mappableFieldsDo:
			[:eachSource :eachTarget | parameters at: eachSource put: (anElementBuilder valueOfField: eachSource)].
	^parameters.
!

query

	query isNil ifTrue: [self buildQuery].
	^query.
!

referenceTableFieldFor: aField
	^self usesLinkTable
		ifTrue: [nil]
		ifFalse: [self join targetForSource: aField].
!

reverseJoin

	reverseJoin isNil ifTrue: [reverseJoin := self joinFromLinkToReferenceTable].
	^reverseJoin.
!

reverseMapping
	"Construct the backwards mapping that corresponds to us. If it's using a link table, then we are at least potentially a toMany mapping. Otherwise, we can point backwards only to a single object."
	| mapping |
	^self usesLinkTable
		ifTrue: 
			[mapping := (ManyToManyMapping new)
				referenceClass: self descriptor describedClass;
				join: self reverseJoin inverseJoin;
				descriptor: self descriptor.
			mapping reverseJoin: self join inverseJoin]
		ifFalse: 
			[(OneToOneMapping new)
				referenceClass: self descriptor describedClass;
				join: self join inverseJoin;
				descriptor: self descriptor]
!

rowMapKeyForSource: source target: target index: index
	"Create the row map key. Make sure we're holding onto the real objects, so proxies don't time out on us"

	self needsSpecialRowMapKey ifFalse: [^target].
	(target yourSelf isNil or: [source yourSelf isNil]) ifTrue: [^nil].
	^rowMapKeyConstructorBlock isNil
		ifTrue:
			[self
				buildRowMapKeyForSource: source
				target: target
				index: index]
		ifFalse:
			[rowMapKeyConstructorBlock value: source yourSelf value: target yourSelf].
!

setGeneralQueryRetrievalIn: newQuery basedOn: base phantom: phantomExpression
	self descriptor keyFields
		do: [:eachField | newQuery retrieve: (phantomExpression getField: eachField)].
	newQuery retrieve: base.
!

valueOfField: aField fromObject: anObject
	^self valueOfField: aField fromObject: anObject ifNone: [self error: 'Cannot find value'].
!

valueOfField: aField fromObject: anObject ifNone: aBlock
	"Return the value of a given field in our target object. Used in extracting primary keys when they are mapped via relationships"
	| targetField target |
	targetField := self referenceTableFieldFor: aField.
	targetField isNil ifTrue: [^aBlock value].
	target := self getValueFrom: anObject.
	(target isGlorpProxy and: [target isInstantiated not])
		ifTrue: [^target parameters at: aField].
	self attribute 	
		loopOver: target 
		doing: [:index :eachTarget |
			^self referenceDescriptor valueOfField: targetField fromObject: eachTarget].
!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject  atIndex: index
! !

!RelationshipMapping methodsFor:'preparing'!

joinExpressionFor: targetExpression
	"We're looking for the object represented by this mapping, and we know the object represented by its source. Use our mapping criteria to construct a join that traverses that instance of this relationship"

	| sourceExpression |
	(self controlsTables not and: [self classModel describedClass == Association]) ifTrue: [^nil].
	sourceExpression := targetExpression base.
	^self extendedJoin asExpressionJoiningSource: sourceExpression toTarget: targetExpression.
!

multipleTableExpressionsFor: anExpression
	^self referenceDescriptor multipleTableJoin
		collect: [:each | each asExpressionJoiningSource: anExpression toTarget: anExpression]
! !

!RelationshipMapping methodsFor:'private/expressions'!

constraints
	| referenceKeys linkTable referenceTables allConstraints relevantConstraints |
	referenceKeys := (self join) targetKeys asOrderedCollection.
	linkTable := referenceKeys first table.
	"If we haven't been told the relevant link table fields, assume we can find them by looking at all the ones that aren't the ones from our source to the link, and all the rest will be from the link to the target"
	allConstraints := linkTable foreignKeyConstraints.
	relevantConstraints := relevantLinkTableFields isNil 
		ifTrue: [allConstraints reject: [:each |  each sourceFields anySatisfy: [:eachSourceField |
			(referenceKeys includes: eachSourceField)]]]
		ifFalse: [allConstraints select: [:each | each sourceFields anySatisfy: [:eachSourceField |
			relevantLinkTableFields includes: eachSourceField]]].

	"Validate that we can handle this case"
	referenceTables := (relevantConstraints collect: [:each | each targetTable]) asSet.
	referenceTables size > 1 ifTrue: [self error: 'Cannot handle this general a case'].
	referenceTables size = 0 ifTrue: [self error: 'No tables found. Did you set up foreign key references in the table definitions?'].

	^relevantConstraints.
!

expressionFromLinkToReferenceTableWithBase: base

	| reverse |
	reverse := self reverseJoin asGlorpExpressionOn: base.
	^reverse  asGeneralGlorpExpressionWithFields.
"	| expression |
	expression := nil.
	self constraints do: [:eachConstraint | 
		eachConstraint sourceFields with: eachConstraint targetFields do: 
			[:eachSource :eachTarget |
			|src target |
			src := (base getTable: eachConstraint sourceTable) getField: eachSource.
			target := (base getTable: eachConstraint targetTable) getField: eachTarget.
			expression := expression isNil 
				ifTrue: [src equals: target]
				ifFalse: [expression AND: (src equals: target)]]].
	^expression."
!

joinFromLinkToReferenceTable
	"Generate the inverse mapping expression, i.e. the one connecting the link table to the reference table, using the foreign key constraints."

	| expression generalPart |
	expression := Join new.
	self constraints do: [:eachConstraint | 
		eachConstraint sourceFields with: eachConstraint targetFields do: 
			[:eachSource :eachTarget |
				expression addSource: eachSource target: eachTarget]].
	generalPart := self join generalExpressionPart.
	^generalPart isNil ifTrue: [expression] ifFalse: [generalPart AND: expression].
! !

!RelationshipMapping methodsFor:'processing'!

trace: aTracing context: anExpression
	"Currently we don't trace relationships across tables, so all we do here
is accumulate the list of embedded mappings"
	| newContext |
	(aTracing tracesThrough: self) ifFalse: [^self].
	newContext := anExpression get: self attributeName.
	aTracing addExpression: newContext.
	self referenceDescriptor trace: aTracing context: newContext.
! !

!RelationshipMapping methodsFor:'proxies'!

newProxyIn: anObjectBuilder 
	| proxy |
	proxy := Proxy new.
	proxy query: (self queryFor: anObjectBuilder).
	^proxy
!

queryFor: anObjectBuilder 
	"If we have a specific query we should use in this case, rather than our general one, construct it. Right now that only applies if we're using filtered reads"
	| basicQuery collection primaryKey newQuery cachedQuery filterBlock |
	self shouldUseFilteredRead 
		ifFalse: [^self query].
	cachedQuery := anObjectBuilder cachedQueryFor: self.
	cachedQuery isNil 
		ifTrue: 
			[basicQuery := self buildGeneralQueryIn: anObjectBuilder.
			self query tracing alsoFetchExpressions 
				do: [:each | basicQuery alsoFetch: each].
			collection := GlorpVirtualCollection new setQuery: basicQuery]
		ifFalse: [collection := cachedQuery collection].
	primaryKey := anObjectBuilder primaryKey.
	filterBlock := self descriptor keyFields size = 1 
				ifTrue: [[:objects :parameters | (objects at: 1) = primaryKey]]
				ifFalse: 
					[
					[:objects :parameters | 
					(1 to: primaryKey size) inject: true
						into: [:sum :i | sum and: [(primaryKey at: i) = (objects at: i)]]]].
	newQuery := FilteringQuery on: collection filter: filterBlock.
	newQuery collectBlock: [:each | each last].
	newQuery readsOneObject: self attribute isCollectionAttribute not.
	newQuery collectionType: self collectionType.
	anObjectBuilder cachedQueryFor: self is: newQuery.
	^newQuery
! !

!RelationshipMapping methodsFor:'testing'!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^controlsTables
!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection. Assumes that aField is one that we map."
	| value |
	value := self getValueFrom: anObject.
	value isGlorpProxy ifFalse: [^true].
	value isInstantiated ifTrue: [^true].
	^value parameters includesKey: aField.
!

isRelationship
	"True when the mapping associates different persistent classes."

	^true
!

isStoredInSameTable
	"Normally, a relationship mapping isn't stored in the same table, but if we're part of a dictionary mapping and the table is mapped to an association, then we want to be treated that way. The test is a horrible hack"
	^self classModel describedClass == Association.
!

isToManyRelationship
	^self attribute isCollectionAttribute.
!

needsSpecialRowMapKey
	"Return true if we have to use a special key object for the row map, rather than just our target"
	^self usesLinkTable.
! !

!RelationshipMapping methodsFor:'validation'!

targetTables
	^((self usesLinkTable ifTrue: [self reverseJoin] ifFalse: [self join]) allTargetFields collect: [:each | each table]) asSet.
!

validate
	| targetJoinSourceFields targetJoinTargetTables |
	super validate.
	self join isNil ifTrue: [^self].
	targetJoinSourceFields := targetTableJoin isNil ifTrue: [#()] ifFalse: [targetTableJoin allSourceFields].
	targetJoinTargetTables :=  targetTableJoin isNil ifTrue: [#()] ifFalse: [targetTableJoin allTargetFields collect: [:each | each table]].
	(self join allSourceFields asArray, targetJoinSourceFields)
		do:
			[:each | 
			(each class == DatabaseField
				and: [self descriptor tables includes: each table])
				ifFalse:
					[self
						error:
							'Joins should go from the table(s) associated with this descriptor to the other table(s)']].
	(self targetTables asArray, targetJoinTargetTables)
		do:
			[:each | 
			((self system descriptorFor: self referenceClass) tables includes: each)
				ifFalse:
					[self
						error:
							'Joins should go to the table(s) associated with the reference class of the mapping']].
! !

!RelationshipMapping class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/RelationshipMapping.st,v 1.1 2006/07/26 08:17:55 boris Exp $'
! !
