"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#RowMap
	instanceVariableNames:'rowDictionary hasDeletions'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!RowMap class methodsFor:'instance creation'!

forRows: aCollection
	"Convenience - construct a row map from a given set of rows"
	| map |
	map := self new.
	aCollection do: [:each | map addRow: each].
	^map.
!

new

	^super new initialize.
!

new: size

	^super new initialize: size.
! !

!RowMap class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!RowMap methodsFor:'api'!

at: aField key: anObject put: aValue
	(self findOrAddRowForTable: aField table withKey: anObject) at: aField put: aValue.
!

findOrAddRowForTable: aTable withKey: aKey
	| submap row |
	submap := self subMapForTable: aTable withKey: aKey.
	row := submap
		at: aKey
		ifAbsentPut: [DatabaseRow newForTable: aTable withOwner: aKey].
	^(self isDeleted: row)
		ifTrue:
			[| newRow |
			newRow := DatabaseRow newForTable: aTable withOwner: aKey.
			submap at: aKey put: newRow.
			newRow]
		ifFalse: [row].
!

isRowMapKey: anObject
	^anObject class == RowMapKey or: [anObject class == MultipleRowMapKey].
!

mementoFor: anObject
	^anObject.
!

unifyField: eachSourceField key: sourceObject withField: eachTargetField key: targetObject
	| sourceRow targetRow |
	sourceRow := self findOrAddRowForTable: eachSourceField table withKey: sourceObject.
	targetRow := self findOrAddRowForTable: eachTargetField table withKey: targetObject.
	sourceRow unifyField: eachSourceField withField: eachTargetField inRow: targetRow.
!

unifyField: sourceField key: sourceObject withField: targetField key: targetObject transformationForward: forwardTransformation inverse: inverseTransformation
	| sourceRow targetRow existingWrapper newWrapper |
	sourceRow := self findOrAddRowForTable: sourceField table withKey: sourceObject.
	targetRow := self findOrAddRowForTable: targetField table withKey: targetObject.
	sourceRow unifyField: sourceField withField: targetField inRow: targetRow.
	existingWrapper := targetRow wrapperAt: targetField.
	(existingWrapper class == TransformingFieldValueWrapper and: [
		existingWrapper forwardTransformation == forwardTransformation and: [
			existingWrapper inverseTransformation == inverseTransformation]])
				ifTrue: [^self].
	newWrapper := (TransformingFieldValueWrapper new)
		innerWrapper: existingWrapper;
		forwardTransformation: forwardTransformation;
		inverseTransformation: inverseTransformation.
	targetRow wrapperAt: targetField put: newWrapper.
! !

!RowMap methodsFor:'counting'!

numberOfEntries
	self compress.
	^rowDictionary inject: 0 into: [:sum :each | sum + each size]
!

numberOfEntriesForTable: aTable
	self compress.
	^(self subMapForTable: aTable) size.
! !

!RowMap methodsFor:'initialize/release'!

initialize
      self initialize: 17
!

initialize: size
	rowDictionary := IdentityDictionary new: size.
	hasDeletions := false.
! !

!RowMap methodsFor:'iterating'!

keysAndValuesDo: aBlock 
	self tables do: [:each | 
		(self subMapForTable: each) keysAndValuesDo: aBlock].
!

objects
	| objects |
	objects := IdentitySet new.
	self tables do: [:each | objects addAll: (self subMapForTable: each) keys].
	^objects
!

objectsAndRowsDo: aTwoArgumentBlock
	rowDictionary
		do:
			[:eachObjectToRowDictionary | 
			eachObjectToRowDictionary
				keysAndValuesDo:
					[:key :value | 
					(self isDeleted: value)
						ifFalse: [aTwoArgumentBlock value: key value: value]]].
!

objectsAndRowsForTable: aTable do: aTwoArgumentBlock
	^(self subMapForTable: aTable)
		keysAndValuesDo:
			[:key :value | 
			(self isDeleted: value)
				ifFalse: [aTwoArgumentBlock value: key value: value]].
!

objectsDo: aBlock
	self tables
		do:
			[:each | 
			(self subMapForTable: each)
				keysAndValuesDo:
					[:key :value | (self isDeleted: value) ifFalse: [aBlock value: key]]].
!

rowsDo: aBlock 
	self tables do: [:each | self rowsForTable: each do: aBlock]
!

rowsForTable: aTable do: aBlock
	^(self subMapForTable: aTable)
		keysAndValuesDo:
			[:key :value | (self isDeleted: value) ifFalse: [aBlock value: value]].
! !

!RowMap methodsFor:'lookup'!

addRow: aRow
	^self addRow: aRow forTable: aRow table withKey: aRow owner.
!

addRow: aRow forTable: aTable withKey: aKey 
	| submap |
	aKey isGlorpProxy ifTrue: [self halt].
	submap := self subMapForTable: aTable withKey: aKey.
	submap at: aKey put: aRow.
	^aRow.
!

deleteRowForTable: aTable withKey: aKey ifAbsent: aBlock
	"Don't actually delete, the fixup of the hash tables is too expensive. Mark invisible instead, overloading the status variable that we know we're not using until later"

	| row |
	row := (self subMapForTable: aTable) at: aKey ifAbsent: aBlock.
	row isNil ifFalse: [self privateDeleteRow: row].
!

includesRowForTable: aTable withKey: aKey

	| row |
	row := (self subMapForTable: aTable ifAbsent: [^false]) at: aKey ifAbsent: [^false].
	^(self isDeleted: row) not.
!

rowForTable: aTable withKey: aKey 
	^self rowForTable: aTable withKey: aKey ifAbsent: [self error: 'missing element'].
!

rowForTable: aTable withKey: aKey ifAbsent: aBlock
	| row |
	row := (self subMapForTable: aTable) at: aKey ifAbsent: aBlock.
	^(self isDeleted: row) ifTrue: [aBlock value] ifFalse: [row].
!

rowForTable: aTable withKey: aKey ifPresent: aBlock
	^(self subMapForTable: aTable)
		at: aKey
		ifPresent:
			[:row | (self isDeleted: row) ifTrue: [nil] ifFalse: [aBlock value: row]].
! !

!RowMap methodsFor:'private/deletion'!

collapseMatchingInsertsAndDeletes
	^rowDictionary
		inject: Dialect identitySetClass new
		into:
			[:sum :eachTablesRowList | 
			sum addAll: (self collapseMatchingInsertsAndDeletesFor: eachTablesRowList).
			sum].
!

collapseMatchingInsertsAndDeletesFor: eachTablesRows
	"Find any inserts and deletes for these rows which have the same primary key. We expect these to be link table rows, because for rows that map to objects, object identity should ensure this never happens. Force the insert to be an update instead (by returning it to the calling method, which will force that to happen) and remove the deletion"
	| deletions updates |
	(self isSuitableForCollapsing: eachTablesRows) ifFalse: [^#()].
	deletions := Dictionary new: 50.
	updates := OrderedCollection new.
	eachTablesRows do: [:each | each forDeletion ifTrue: [
		deletions at: (each primaryKey) put: each]].
	deletions isEmpty ifTrue: [^#()].
	eachTablesRows do: [:each |
		| pk |
		pk := each primaryKey.
		((deletions includesKey: pk) and: [each forDeletion not]) ifTrue: [
			each numberOfFields = each table fields size ifFalse: [self error: 'link table entry requiring update is only partially populated'].
			updates add: each.
			self privateDeleteRow: (deletions at: pk)]].
	^updates
!

compress
	"Remove all deleted rows and rebuild"
	| newDictionary |
	hasDeletions ifFalse: [^self].
	Transcript cr; show: 'compressing'.
	hasDeletions := false.
	newDictionary := IdentityDictionary new: rowDictionary size.
	rowDictionary keysAndValuesDo: [:eachTable :eachRowDict |
		| newRowDict |
		newRowDict := IdentityDictionary new: eachRowDict size.
		eachRowDict keysAndValuesDo: [:eachObject :eachRow |
			(self isDeleted: eachRow) ifFalse: [
				newRowDict at: eachObject put: eachRow]].
		newDictionary at: eachTable put: newRowDict].
	rowDictionary := newDictionary.
!

isDeleted: row
	^row status == #invisible.
!

isSuitableForCollapsing: eachTablesRows
	"Return true if this set of rows are suitable for collapsing the inserts and deletes together. It's suitable if this is a link table, and if there is a primary key defined which we can use to decide if the rows match or not"
	eachTablesRows
		keysAndValuesDo:
			[:eachKey :eachRow | 
				(self isRowMapKey: eachKey) ifFalse: [^false].
				eachRow table primaryKeyFields isEmpty ifTrue: [^false].
				^true ].
!

privateDeleteRow: row
	row status: #invisible.
	hasDeletions := true.
! !

!RowMap methodsFor:'private/mapping'!

adjustForMementos: objects
	^self.
!

collectionMementoFor: aCollection

	^aCollection.
!

dictionaryClassRequiredForKey: anObject

	^(self isRowMapKey: anObject)
		ifTrue: [Dictionary]
		ifFalse: [IdentityDictionary].
!

reverseLookup: anObject

	^anObject.
!

rowsForKey: aKey
	"Return a collection of all rows for any table which are keyed by aKey"

	| rowsForKey |
	rowsForKey := OrderedCollection new: 5.
	rowDictionary
		do:
			[:each | 
			| row |
			row := each at: aKey ifAbsent: [nil].
			(row notNil and: [(self isDeleted: row) not]) ifTrue: [rowsForKey add: row]].
	^rowsForKey.
!

subMapForTable: aTable
	
	^self subMapForTable: aTable withKey: nil.
!

subMapForTable: aTable ifAbsent: aBlock 
	^rowDictionary at: aTable ifAbsent: aBlock
!

subMapForTable: aTable withKey: anObject 
	anObject isGlorpProxy ifTrue: [self halt].
	^rowDictionary at: aTable
		ifAbsentPut: [(self dictionaryClassRequiredForKey: anObject) new]
!

tables

	^rowDictionary keys.
! !

!RowMap methodsFor:'set operations'!

additiveDifferencesFrom: aRowMap into: differencesMap
	"Add everything which is in us, but not in aRowMap into differencesMap"
	| newRow |
	self objectsAndRowsDo: [:object :row |
		| correspondingRow |
		correspondingRow := aRowMap
			rowForTable: row table
			withKey: object
			ifAbsent: [DatabaseRow new].
		(row equals: correspondingRow)
			ifFalse:
				[newRow := differencesMap
					addRow: (row withAllFieldsIn: correspondingRow)
					forTable: row table
					withKey: object.
				newRow forDeletion: row forDeletion]].
!

differenceFrom: aMementoRowMap
	| differencesMap |
	differencesMap := RowMap new.
	self	additiveDifferencesFrom: aMementoRowMap into: differencesMap.
	self	subtractiveDifferencesFrom: aMementoRowMap into: differencesMap.
	^differencesMap.
!

subtractiveDifferencesFrom: aRowMap into: differencesMap
	"Figure out which things are in aRowMap but not in us. These should be flagged as delete rows. Since we have no further use for aRowMap after this, we can compute this destructively by removing everything that's in us"

	self objectsAndRowsDo: [:object :row |
		aRowMap
			deleteRowForTable: row table
			withKey: object
			ifAbsent: []].
	aRowMap
		objectsAndRowsDo:
			[:object :row | 
			| adjustedObject |
			adjustedObject := (aRowMap reverseLookup: object).
			row forDeletion: true.
			differencesMap
				addRow: row
				forTable: row table
				withKey: adjustedObject].
! !

!RowMap methodsFor:'testing'!

isEmpty
	self compress.
	self rowsDo: [:each | (self isDeleted: each) ifFalse: [^false]].
	^true.
!

notEmpty
	^self isEmpty not
! !

!RowMap class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/RowMap.st,v 1.1 2006/07/26 08:19:01 boris Exp $'
! !
