"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#RowMapKey
	instanceVariableNames:'key1 key2'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

RowMapKey comment:'This class serves as a key for a dictionary containing two sub-keys, where we want to be able to look up based on the identity of both sub-keys paired together. This is used primarily for many-to-many mappings indexing into rowmaps, where we want to key the row by the identity of the object that determines it, but there are two of them.

Instance Variables:

key1	<Object>	One sub-key.
key2	<Object>	The other sub-key.'''
!


!RowMapKey class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!RowMapKey methodsFor:'accessing'!

key1
	^key1
!

key1: anObject
	key1 := anObject
!

key2
	^key2
!

key2: anObject
	key2 := anObject
! !

!RowMapKey methodsFor:'adjusting'!

adjustIn: aCorrespondenceMap
	"Adjust ourselves to refer to the copies rather than the originals"
	| adjusted |
	adjusted := self copy.
	adjusted key1: (aCorrespondenceMap at: key1 ifAbsent: [key1]).
	adjusted key2: (aCorrespondenceMap at: key2 ifAbsent: [key2]).
	^adjusted.
!

reverseAdjustIn: aRowMapForMementos
	"Adjust ourselves to refer to the copies rather than the originals"
	| adjusted |
	adjusted := self copy.
	adjusted key1: (aRowMapForMementos reversedCorrespondenceMap at: key1 ifAbsent: [key1]).
	adjusted key2: (aRowMapForMementos reversedCorrespondenceMap at: key2 ifAbsent: [key2]).
	^adjusted.
! !

!RowMapKey methodsFor:'comparing'!

= aRowMapKey

	aRowMapKey class == self class ifFalse: [^false].
	^(key1 == aRowMapKey key1 and: [key2 == aRowMapKey key2]) or: [
		key2 == aRowMapKey key1 and: [key1 == aRowMapKey key2]].
!

hash
	| hash1 hash2 hash1Temp longHash |
	hash1 := key1 identityHash.
	hash2 := key2 identityHash.
	hash1 > hash2 ifTrue: [
		hash1Temp := hash1.
		hash1 := hash2.
		hash2 := hash1Temp].
	longHash := (hash1 bitShift: 8) bitXor: hash2.
	^Dialect fitHashIntoSmallInteger: longHash.
! !

!RowMapKey class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/RowMapKey.st,v 1.1 2006/07/26 08:14:51 boris Exp $'
! !
