"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabasePlatform subclass:#SQLServerPlatform
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!SQLServerPlatform class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!SQLServerPlatform methodsFor:'binding'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.
!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once"
	^aCollectionOfDatabaseRows size min: 250.
! !

!SQLServerPlatform methodsFor:'constants'!

areSequencesExplicitlyCreated

^false
!

databaseSequenceClass
	^SQLServerSequence.
!

maximumLengthOfColumnName
	"^<Integer> I return the max. length of a column name"

	^128
!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^128
!

supportsConstraints

	^true
!

supportsMillisecondsInTimes
	"I'm guessing here"
	^true.
! !

!SQLServerPlatform methodsFor:'conversion-boolean'!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToInteger:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.
! !

!SQLServerPlatform methodsFor:'conversion-strings'!

charactersThatNeedEscaping
	"There seem to be all kind of contradictory bits of information about what sql server does/requires for escaped characters, all of which differ from standard sql. Empirically the only thing that requires escaping appears to be single quote"
	^#($' ).
!

escapeFor: aCharacter
	^String with: $' with: aCharacter.
"	^'\', (aCharacter asInteger printStringRadix: 16)."
!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: '0x'.
	aByteArray do: [:each |
		each printOn: aStream paddedWith: $0 to: 2 base: 16].
! !

!SQLServerPlatform methodsFor:'conversion-times'!

dateConverter
	"SQL server doesn't have plain dates, and doesn't accept them"
	^DelegatingDatabaseConverter
		named: #date
		hostedBy: self
		fromStToDb: #dateToTimestampConversion:for:
		fromDbToSt: #readDate:for:.	"#printDate:for:"
!

dateToTimestampConversion: aDate for: aType
	aDate isNil ifTrue: [^aDate].
	^aDate asTimestamp.
!

printDate: aDate for: aType
	"Print a date (or timestamp) as yyyy-mm-dd"
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: '{ d '''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''' }'.
	^stream contents.
!

printTime: aTime for: aType
	"Print a time (or timestamp) as hh:mm:ss.fff"
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: '{ t '''.
	self
		printTime: aTime
		isoFormatOn: stream
		milliseconds: self supportsMillisecondsInTimes.
	stream nextPutAll: ''' }'.
	^stream contents.
!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: '{ ts '''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ''' }'.
! !

!SQLServerPlatform methodsFor:'database-specific'!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	aSymbol == #INTERSECT ifTrue: [^'WHERE EXISTS'].
	aSymbol == #MINUS ifTrue: [^'WHERE NOT EXISTS'].
	^aSymbol.
!

printPreLimit: anInteger on: aCommand
	aCommand
		nextPutAll: ' TOP '.
	anInteger printOn: aCommand.
	aCommand nextPutAll: ' '.
!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	| newQuery |
	newQuery := aQuery rewriteIntersect.
	newQuery := newQuery rewriteExcept.
	newQuery == aQuery ifFalse: [aBlock value: newQuery].
! !

!SQLServerPlatform methodsFor:'exdi specific'!

exdiTypeForDates
	^#Timestamp.
! !

!SQLServerPlatform methodsFor:'functions'!

initializeFunctions
	super initializeFunctions.
	functions at: #, put: (InfixFunction named: '+').
! !

!SQLServerPlatform methodsFor:'testing'!

isODBCPlatform

	^true
!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^true.
!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.
!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^false.
!

supportsGroupWritingFor: aCommand
	^aCommand supportsGroupWriting.
!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^true.
!

supportsMultipleOpenCursors
	"Can this database support multiple open cursors at once"
	^false.
!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^true
!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins"
	^false.
!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^false.
!

usesIdentityColumns
	^true.
! !

!SQLServerPlatform methodsFor:'types'!

bit
	^self typeNamed: #bit ifAbsentPut: [GlorpBooleanType new typeString: 'bit'].
!

blob

	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'image'; queryType: (self varbinary)].
!

boolean
	^self bit.
!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].
!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'text'].
!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new typeString: 'datetime'].
!

decimal
	^self numeric.
!

double
	^self float.
!

float

	^self typeNamed: #float ifAbsentPut: [GlorpMSSQLFloatType new].
!

float4
	^self float.
!

float8
	^self float.
!

int
	^self integer.
!

int2
	^self smallint.
!

int4

	^self integer.
!

int8

	^self numeric.
!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].
!

real
	^self float.
!

sequence
	^self serial.
!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'int  IDENTITY (1, 1) '].
!

text
	^super text queryType: self varchar.
!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'datetime'].
!

timeStampTypeString
	^'datetime'.
!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'datetime'].
!

tinyint
	^self typeNamed: #tinyInt ifAbsentPut: [GlorpIntegerType new typeString: 'tinyint'].
!

varbinary
	^self typeNamed: #varbinary ifAbsentPut: [GlorpVarBinaryType new].
!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].
! !

!SQLServerPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/SQLServerPlatform.st,v 1.1 2006/07/26 08:15:33 boris Exp $'
! !
