"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

AbstractReadQuery subclass:#SimpleQuery
	instanceVariableNames:'builders fields joins tablePrefix'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

SimpleQuery comment:'This is a query that is directly executable. A single query might be more than we can do in a single database read, so we might have to break it down into simple queries. But at the moment we just break anything down into an equivalent single query.

Instance Variables:
	builders	<OrderedCollection of: ElementBuilder)>	The builders that will assemble the object from the row that this query returns.
	fields	<OrderedCollection of: DatabaseField>	The fields being selected.
	traceNodes	<Collection of: GlorpExpression>	 These describe the graph of the objects to be read, so we can specify customer, customer address and customer account all in one read.


'
!


!SimpleQuery class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!SimpleQuery methodsFor:'As yet unclassified'!

queries
	"A hack so you can write things that manipulate both this and compound queries the same way"
	^Array with: self.
! !

!SimpleQuery methodsFor:'accessing'!

addBoundExpressionsIn: aCommand to: aCollection
	self joins , (Array with: self whereClause)
		do:
			[:eachBigExpression | 
			eachBigExpression
				do:
					[:eachIndividualExpressionNode | 
					eachIndividualExpressionNode
						addBoundExpressionsIn: aCommand
						to: aCollection]].
!

builders

	^builders.
!

elementBuilderFor: anExpression 
	^builders detect: [:each | each expression == anExpression] ifNone: [nil].
!

fields
	^fields
!

joins
	^joins.
!

tablePrefix
	^tablePrefix
!

tablePrefix: aString
	tablePrefix := aString.
!

where: anExpression
	whereClause := anExpression.
	self setUpBase.
! !

!SimpleQuery methodsFor:'converting'!

asFullQuery
	| newQuery |
	newQuery := ReadQuery new
		initResultClass: resultClass whereClause: whereClause singleObject: readsOneObject.
	newQuery returnProxies: self returnProxies.
	newQuery shouldRefresh: self shouldRefresh.
	newQuery setOrdering: ordering.
	newQuery collectionType: collectionType.
	^newQuery.
!

asGlorpExpression
	^self asGlorpExpressionOn: BaseExpression new.
!

asGlorpExpressionOn: anExpression
        "If we're being converted into an expression, that means we're a
         subselect inside an expression block. We don't actually have
         any relationship to anExpression, so create our own base and make
         a pseudo-mapping with no join to describe the relationship.
         Don't even give it a string-name, just use ourselves as the key"
        
        | expression localBase phantomMapping phantomExpression |
        phantomMapping := OneToOneMapping new
                referenceClass: self resultClass;
                join: Join new;
                descriptor: (anExpression system descriptorFor: self resultClass).
        phantomExpression := anExpression getMapping: phantomMapping named: self.
        expression := GeneralSubSelectExpression 
                basedOn: phantomExpression.
        expression query: self.
        localBase := SubSelectBaseExpression on: phantomExpression.
        " this addition reverts an earlier version "
        self whereClause: (self whereClause asGlorpExpressionOn: localBase withUltimateBase: anExpression).
         " ---  end of modification --- "
        self tablePrefix: ('s', anExpression ultimateBaseExpression availableSubSelectNumber printString, 't').
        ^expression.
!

asGlorpExpressionOn: anExpression basedOn: aMessageArchiver withUltimateBase: aBaseExpression
	^self asGlorpExpressionOn: aBaseExpression.
! !

!SimpleQuery methodsFor:'copying'!

postCopy
	super postCopy.
	fields := OrderedCollection new.
	joins := OrderedCollection new.
	builders := nil.
! !

!SimpleQuery methodsFor:'executing'!

computeFields
	builders 
		do: [:each | self computeFieldsFor: each]
!

computeFieldsFor: anElementBuilder 
	| translatedFields |
	translatedFields := self 
		addFields: anElementBuilder fieldsForSelectStatement
		returningTranslationForFields: anElementBuilder fieldsFromMyPerspective
		distinct: anElementBuilder requiresDistinct.
	anElementBuilder fieldTranslations: translatedFields.
!

rowsFromDatabaseWithParameters: anArray

	| alreadyTriedAction command |
	self shortCircuitEmptyReturn ifTrue: [^session accessor cursorFor: (ReadStream on: #()) command: (self sqlWith: anArray)].
	alreadyTriedAction := false.
	command := self sqlWith: anArray.
	^[session accessor
		executeCommand: command
		returnCursor: true] 
	on: GlorpDatabaseReadError 
	do: [:ex | 
		alreadyTriedAction ifTrue: [ex pass] ifFalse: [alreadyTriedAction := true].
		session errorAction isNil 
			ifTrue: [ex pass] 
			ifFalse: [
				session errorAction value: ex value: session value: self.	
				ex retry]].
!

setUpExpressions
	super setUpExpressions.
	self validateWhereClause.
! !

!SimpleQuery methodsFor:'fields'!

addFields: aliasedFields returningTranslationForFields: originalFields distinct: isDistinct
	"The query has computed a set of fields the way the mappings see them, which are then transformed to account for field aliasing in embedded mappings. Add those to our collection, and set up the translation which knows which fields are at which index in the resulting row. If necessary, note that those fields are selected as distinct"

	| translation |
	translation := IdentityDictionary new.
	aliasedFields
		with: originalFields
		do:
			[:aliased :original | 
			| position fieldToAdd|
			position := self findPositionofAliasedField: aliased.
			position = 0
				ifTrue:  
					[fieldToAdd := isDistinct ifTrue: [aliased asDistinctFieldOn: self baseExpression] ifFalse: [aliased].
					fields add: fieldToAdd .
					position := fields size].
			translation at: original put: position].
	^translation.
!

findPositionofAliasedField: aField
	1 to: fields size do: [:i |
		| existingField |
		existingField := fields at: i.
		(existingField asField isSameAliasedFieldAs: aField)
			ifTrue: [^i]].
	^0.
! !

!SimpleQuery methodsFor:'initialize'!

initResultClass: aClass whereClause: anExpression singleObject: aBoolean 
	super 
		initResultClass: aClass
		whereClause: anExpression
		singleObject: aBoolean.
	prepared := false.
	fields := OrderedCollection new.
	joins := OrderedCollection new.
! !

!SimpleQuery methodsFor:'preparing'!

addJoin: anExpression
	joins addAll: anExpression asIndependentJoins.
!

addOrderingForBuilder: aBuilder
	| expression alreadyThere fieldExpressions |
	expression := aBuilder expression.
	fieldExpressions := expression descriptor primaryTable primaryKeyFields collect: [:each | expression getField: each].	
	fieldExpressions do: [:each |
		alreadyThere := ordering notNil and: [ordering includes: each].
		alreadyThere ifFalse: [self orderBy: each]].
!

addOrderingForToManyJoins
	"If we have a join to a to-many relation, then we need for all the joined bits to be grouped together in order in the result set"
	| problemBuilders baseBuilders |
	problemBuilders := builders select: [:each | each canCauseDuplicateRows].
	baseBuilders := problemBuilders
		collect:
			[:each | 
			builders
				detect: [:eachBuilder | eachBuilder expression == each expression base]
				ifNone: [nil]].
	baseBuilders do: [:each | 
		each isNil ifFalse: [self addOrderingForBuilder: each]].
!

adjustWhereClause
	"Make any adjustments to the where clause. At the moment, that just consists of adjusting references of the form object = anotherObject (or nil) to refer to the primary key expressions instead. Other rewrites are possible"

	whereClause := whereClause rewriteEquality. "Special because otherwise we'd be throwing away the top-level one"
	whereClause do: [:each | each rewriteEqualityExpressions].
	whereClause do: [:each | each condensePrimaryKeyComparisons].
	whereClause := whereClause condensePrimaryKeyComparison.
!

assignTableAliases
	| tableNumber allExpressions |
	whereClause isJoin ifTrue: [^self].
	tableNumber := 1.
	allExpressions := ExpressionGroup with: whereClause.
	allExpressions addAll: ordering.
	allExpressions addAll: joins.
	builders do: [:each | allExpressions add: each expression].
	allExpressions
		do:
			[:each | tableNumber := each assignTableAliasesStartingAt: tableNumber prefix: self tablePrefix].
	"We've aliased tables, joins may now become independent"
	joins := joins inject: OrderedCollection new into: [:sum :each | (sum addAll: each asIndependentJoins). sum].
!

copyOfBuilders 
	"Copy the builders, but make sure that subBuilders also get matched up"
	| newBuilders |
	newBuilders := builders collect: [:each | each copy].
	newBuilders do: [:each | each reconcileWithBuilders: builders andCopy: newBuilders].
	^newBuilders.
!

fixJoins
	| pseudoJoins realJoins |
	pseudoJoins := joins select: [:each | each tablesForANSIJoin size < 2].
	pseudoJoins do: [:each | whereClause := each AND: whereClause].
	realJoins :=  joins select: [:each| each tablesForANSIJoin size >= 2].
	joins := realJoins.
!

isPrepared
	^prepared.
!

prepare
	prepared ifTrue: [^self].
	self setupTracing.
	whereClause prepareIn: self.
	self fixJoins.
	self assignTableAliases.
	self computeFields.
	prepared := true.
!

setupTracing
	| |
	builders isNil ifFalse: [^self].	"Already been done"
	super setupTracing.
	builders := tracing allTracings asArray
		inject: #()
		into:
			[:sum :each | sum , (ElementBuilder for: each in: self) withAllSubBuilders].
	self addOrderingForToManyJoins.
!

traceExpressionInContextFor: anExpression
	^anExpression rebuildOn: base.
! !

!SimpleQuery methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' for '.
	aStream nextPutAll: (resultClass isNil ifTrue: ['nil'] ifFalse: [resultClass name]).
! !

!SimpleQuery methodsFor:'sql generation'!

printANSITablesOn: aCommand
	"Print ourselves using the JOIN... USING syntax. Note that we have to put the joins in the right order because we're not allowed to refer to tables not mentioned yet. Great syntax. Reminds me of Pascal. And so easy to deal with."

	| printer |
	printer := JoinPrinter for: self.
	printer printJoinsOn: aCommand.
!

printJoinsOn: aCommand
	| noLeadIn|
	self platform supportsANSIJoins ifTrue: [^self].
	joins isEmpty ifTrue: [^self].
	noLeadIn := whereClause isEmptyExpression.
	noLeadIn ifFalse: [aCommand nextPutAll: ' AND ('].
	GlorpHelper 
			do: [:each | each printSQLOn: aCommand withParameters: aCommand parameters]
			for: joins
			separatedBy: [aCommand nextPutAll: ' AND '].
	noLeadIn ifFalse: [aCommand nextPut: $)].
!

printNormalTablesOn: aCommand
	self printNormalTablesOn: aCommand excluding: #().
!

printNormalTablesOn: aCommand excluding: aCollection
	| tablesToPrint |
	tablesToPrint := self tablesToPrint.
	aCollection do: [:each | tablesToPrint remove: each].
	GlorpHelper
		print: [:table | table sqlTableName]
		on: aCommand
		for: tablesToPrint
		separatedBy: ', '.
	^tablesToPrint.
!

printOffsetOn: aCommand
	self hasOffset ifFalse: [^self].
	self platform printOffset: self offset on: aCommand.
!

printPostLimitOn: aCommand
	self hasLimit ifFalse: [^self].
	self platform printPostLimit: self limit on: aCommand.
!

printPreLimitOn: aCommand
	self hasLimit ifFalse: [^self].
	self platform printPreLimit: self limit on: aCommand.
!

printSelectFields: aCollection on: stream 
	| fieldsToPrint |
	(aCollection anySatisfy: [:each | each isDistinct]) 
		ifTrue: [
			fieldsToPrint := aCollection collect: [:each | each asField].
			stream nextPutAll: 'DISTINCT ']
		ifFalse: [fieldsToPrint := aCollection].

	"For SQL Server, the TOP has to come after the distinct, but before the fields"
	self printPreLimitOn: stream.
			
	GlorpHelper 
		do: [:field | field printSelectSQLOn: stream withParameters: nil. ]
		for: fieldsToPrint
		separatedBy: [stream nextPutAll: ', '].
!

printSelectFieldsOn: stream 
     self printSelectFields: fields on: stream
!

printTablesOn: aCommand
	aCommand
		cr;
		nextPutAll: ' FROM '.
	self useANSIJoins
		ifTrue: [self printANSITablesOn: aCommand]
		ifFalse: [self printNormalTablesOn: aCommand].
!

printWhereClauseOn: aCommand
	self hasEmptyWhereClause
		ifFalse:
			[aCommand
				cr;
				nextPutAll: ' WHERE '.
			whereClause printSQLOn: aCommand withParameters: aCommand parameters].
!

signature
	session useBinding ifFalse: [^''].
	^self sqlWith: Dictionary new.
!

sqlString
	"Mostly for debugging"
	^(self sqlWith: #()) sqlString.
!

sqlWith: aDictionary
	self prepare.
	^QuerySelectCommand
		forQuery: self
		parameters: aDictionary
		useBinding: session useBinding
		platform: session platform.
!

tablesToPrint
	| allTables stoppingPoint |
	allTables := Set new.
	fields do: [:each | allTables addAll: each tablesToPrint].
	stoppingPoint := base.
	allTables
		addAll:
			(stoppingPoint isNil
				ifTrue: [whereClause allTablesToPrint]
				ifFalse: [whereClause allTablesToPrintStoppingAt: stoppingPoint]).
	joins
		do:
			[:eachJoin | 
			allTables
				addAll:
					(stoppingPoint isNil
						ifTrue: [eachJoin allTablesToPrint]
						ifFalse: [eachJoin allTablesToPrintStoppingAt: stoppingPoint])].
	ordering isNil
		ifFalse: [ordering do: [:each | allTables add: each field table]].
	self tracing allTracings
		do: [:each | allTables addAll: each allTablesToPrint].
	^allTables asSortedCollection.
! !

!SimpleQuery methodsFor:'testing'!

hasEmptyWhereClause
	"If we have regular where clause entries, or if we have joins that aren't going to be printed in the tables portion, then we're not empty"
	whereClause isEmptyExpression ifFalse: [^false].
	self useANSIJoins ifTrue: [^true].
	^joins isEmpty.
!

hasTracing
	"Return true if we've given this query a tracing already"
	^builders notNil.
!

mightHaveDuplicateRows

	^builders anySatisfy: [:each | each canCauseDuplicateRows].
!

requiresFullQuery
	^self descriptor classesRequiringIndependentQueries size > 1
!

useANSIJoins
	^self session platform supportsANSIJoins.
! !

!SimpleQuery class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/SimpleQuery.st,v 1.1 2006/07/26 08:14:29 boris Exp $'
! !
