"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

ObjectExpression subclass:#TableExpression
	instanceVariableNames:'table base'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-Expressions'
!


!TableExpression class methodsFor:'instance creation'!

forTable: aDatabaseTable basedOn: aBaseExpression

	^self new table: aDatabaseTable base: aBaseExpression; yourself
! !

!TableExpression class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!TableExpression methodsFor:'accessing'!

base

	^base.
!

canHaveBase
	"Return true if this type of expression can have a base expression on which other things can be built. Doesn't say whether we actually have a valid one or not."
	^true.
!

printsTable

	^true.
!

queryLevelBaseExpression
	^base queryLevelBaseExpression.
!

table

	^table.
!

tables

	^Array with: table.
!

ultimateBaseExpression
	^base ultimateBaseExpression.
! !

!TableExpression methodsFor:'api'!

getFieldThatsAnActualFieldObject: aField

	^self mappingExpressions at: aField ifAbsentPut: [self newFieldExpressionFor: aField].
!

getTable: aTable
	self error: 'getting a table from a table expression doesn''t make sense'.
! !

!TableExpression methodsFor:'initialize/release'!

table: aDatabaseTable base: aBaseExpression

	table := aDatabaseTable.
	base := aBaseExpression.
! !

!TableExpression methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Iterate over the expression tree"

	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	base do: aBlock skipping: aSet.
	aBlock value: self.
! !

!TableExpression methodsFor:'preparing'!

aliasedTableFor: aDatabaseTable 
	^self controlsTables 
		ifTrue: [super aliasedTableFor: aDatabaseTable]
		ifFalse: [base aliasedTableFor: aDatabaseTable]
!

aliasedTableFor: aDatabaseTable ifAbsent: aBlock
	^self controlsTables 
		ifTrue: [super aliasedTableFor: aDatabaseTable ifAbsent: aBlock]
		ifFalse: [base aliasedTableFor: aDatabaseTable ifAbsent: aBlock]
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	^target getTable: table.
!

controlsTables
	"We can end up with a table expression built on top of a base that has the same table. If so, we don't count as controlling that table"

	base isNil ifTrue: [^true].
	base hasDescriptor ifFalse: [^true].
	^(base descriptor tables includes: table) not.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	self == anExpression ifTrue: [^aBaseExpression].
	^(base rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression) getTable: table.
!

tablesToPrint

	self controlsTables ifFalse: [^#()].
	^Array with: (self aliasedTableFor: table).
! !

!TableExpression methodsFor:'printing'!

printOnlySelfOn: aStream

	table printSQLOn: aStream withParameters: #().
	self printTableAliasesOn: aStream.
!

printTreeOn: aStream 
	base printOn: aStream.
	aStream nextPut: $..
	table printSQLOn: aStream withParameters: #()
! !

!TableExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/TableExpression.st,v 1.1 2006/07/26 08:28:36 boris Exp $'
! !
