"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#TableSorter
	instanceVariableNames:'orderedTables tables visitedTables'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!


!TableSorter class methodsFor:'instance creation'!

for: tables

	| sorter |
	sorter := self new.
	tables do: [:each |
		sorter addTable: each].
	^sorter.
!

new

	^super new initialize.
! !

!TableSorter class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!TableSorter methodsFor:'accessing'!

addTable: aTable 
	tables add: aTable
!

hasBeenVisited: aTable

	^visitedTables includes: aTable.
!

markVisited: aTable

	visitedTables add: aTable.
! !

!TableSorter methodsFor:'initializing'!

initialize
	tables := OrderedCollection new: 100.
	visitedTables := IdentitySet new: 100.
! !

!TableSorter methodsFor:'sorting'!

sort
	orderedTables := OrderedCollection new: tables size.
	tables do: [:each | self visit: each].
	^orderedTables
!

visit: aTable 
	"The essential bit of topological sort. Visit each node in post-order, traversing dependencies, based on foreign key constraints to database-generated fields. "

	(self hasBeenVisited: aTable) ifTrue: [^self].
	self markVisited: aTable.
	self visitDependentTablesFor: aTable.
	orderedTables add: aTable
!

visitDependentTablesFor: aTable 
	aTable foreignKeyConstraints do: [:eachConstraint | 
          self visit: eachConstraint targetTable.]
! !

!TableSorter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/TableSorter.st,v 1.1 2006/07/26 08:19:13 boris Exp $'
! !
