"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Tracing
	instanceVariableNames:'base allTracings retrievalExpressions alsoFetchExpressions query'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

Tracing comment:'A tracing is a collection of expressions representing the graph of other objects which
are to be read at the same time as the root object.

Instance Variables:

	base	<Expression>	The base expression representing the root object. Same as the parameter to the query block
	allTracings	<Collection of: Expression>	The expressions representing each of the associated objects. e.g. base accounts, base amount serviceCharge .
	alsoFetchExpressions	<(Collection of: GlorpExpression)>	Objects to also retrieve, but not included in the result set, just knitted together with the other related objects.
	retrievalExpressions	<(Collection of: GlorpExpression)>	Objects to also retrieve, and to include in teh result set

'
!


!Tracing class methodsFor:'instance creation'!

for: aQuery
	^self new base: aQuery whereClause ultimateBaseExpression.
!

new

	^super new initialize.
! !

!Tracing class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Tracing methodsFor:'accessing'!

addExpression: anExpression
	self addExpression: anExpression andDo: [:ignore |].
!

addExpression: anExpression andDo: aBlock
	| exp |
	exp := anExpression asGlorpExpressionOn: self base in: query.
	(allTracings includes: exp) ifFalse: [
		allTracings add: exp.
		aBlock value: exp].
!

additionalExpressions
	| all |
	alsoFetchExpressions isEmpty ifTrue: [^retrievalExpressions].
	all := OrderedCollection new.
	all addAll: self retrievalExpressions.
	all addAll: self alsoFetchExpressions.
	^all.
!

allTracings

	^allTracings.
!

alsoFetchExpressions
	^alsoFetchExpressions
!

base
	^query isNil ifTrue: [base] ifFalse: [query baseExpression].
!

base: anExpression 
	base := anExpression
!

retrievalExpressions
	^retrievalExpressions.
! !

!Tracing methodsFor:'api'!

alsoFetch: anExpression 
	"Add the expression as something which will be explicitly retrieved and knit together with other results, but NOT included in the result list"

	self addExpression: anExpression andDo: [:exp |
		alsoFetchExpressions add: exp].
!

retrieve: anExpression 
	"Add the expression as something which will be explicitly retrieved and knit together with other results, and included in the result list"
	self addExpression: anExpression andDo: [:exp |
		retrievalExpressions add: exp].
! !

!Tracing methodsFor:'initialize'!

initialize

	base := BaseExpression new.
	allTracings := OrderedCollection new: 2.
	retrievalExpressions := Set new: 3.
	alsoFetchExpressions := Set new: 3.
! !

!Tracing methodsFor:'querying'!

tracesThrough: aMapping

	^aMapping isStoredInSameTable.
! !

!Tracing methodsFor:'setup'!

setup
	"We have been put into a query. If we aren't to trace anything else, trace the base"
	retrievalExpressions isEmpty ifTrue: [
		allTracings addFirst: base.
		retrievalExpressions add: base].
!

updateBase: aBaseExpression
	"Make sure we have the same base as the query"
	| transformed |
	transformed := IdentityDictionary new.
	base == aBaseExpression ifTrue: [^self].
	base := aBaseExpression.
	allTracings := allTracings collect: [:each | | new | new := each asGlorpExpressionOn: base.
		transformed at: each put: new.
		new].
	retrievalExpressions := retrievalExpressions collect: [:each | 
		transformed at: each].
	alsoFetchExpressions := alsoFetchExpressions collect: [:each | 
		transformed at: each].
! !

!Tracing class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Tracing.st,v 1.1 2006/07/26 08:19:31 boris Exp $'
! !
