"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabaseAccessor subclass:#VA55DatabaseAccessor
	instanceVariableNames:'isInTransaction'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDatabase'
!

VA55DatabaseAccessor class instanceVariableNames:'databaseErrorSignal'

"
 No other class instance variables are inherited by this class.
"
!


!VA55DatabaseAccessor class methodsFor:'initialization'!

initializeDatabaseErrorSignal

	databaseErrorSignal := ((Smalltalk at: #SystemExceptions) at: 'ExError') newChild.
	databaseErrorSignal markReadOnly: false;
		description: 'Database error';
		resumable: true;
		defaultHandler: nil;
		markReadOnly: true
! !

!VA55DatabaseAccessor class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!VA55DatabaseAccessor class methodsFor:'accessing'!

externalDatabaseErrorSignal

	databaseErrorSignal isNil ifTrue: [self initializeDatabaseErrorSignal ].
	^databaseErrorSignal
! !

!VA55DatabaseAccessor methodsFor:'accessing'!

returnRowCount: aBoolean

	connection isNil ifFalse: [connection queryRowCount: true].
!

rowCount
	^connection rowCount.
! !

!VA55DatabaseAccessor methodsFor:'cursors'!

cursorFor: aStream
	aStream onFinalizeDo: #close.
	^super cursorFor: aStream.
!

isCursorAtEnd: aCursor
	"Return true if aCursor is at its end."

	^aCursor actuallyAtEnd.
!

releaseCursor: aCursor in: aPreparedStatement
	aCursor close.
! !

!VA55DatabaseAccessor methodsFor:'development'!

todo: aString

	^self
! !

!VA55DatabaseAccessor methodsFor:'executing'!

basicExecuteSQLString: aString
	"^<(OrderedCollection of: ??) | self> This method executes a general SQL command against the connection"

	| resultTable |
	self log: aString.
	(aString copyFrom: 1 to: (6 min: aString size)) asUppercase = 'SELECT'
		ifFalse:
			[connection
				executeSQLStatement: aString
				ifError: [:err | GlorpError signalWith: err].
				^#() readStream.]
		ifTrue:
			[resultTable := connection
				resultTableFromQuerySpec:
					((Smalltalk at: #AbtQuerySpec) new statement: aString)
				ifError: [:err | GlorpError signalWith: err].
			resultTable isAbtError 
				ifTrue: [GlorpError signalWith: resultTable] 
				ifFalse: [^resultTable]].
!

executeCommandUnbound: aCommand
	| answerStream |
	self log: aCommand sqlString.
	answerStream := self basicExecuteSQLString: aCommand sqlString.
	aCommand needsRowCount
		ifTrue:
			[aCommand
				rowCount:
					(self platform supportsRowCount
						ifTrue: [answerStream rowCount]
						ifFalse: [1])].
	^self cursorFor: answerStream command: aCommand.
!

externalDatabaseErrorSignal

	^self class externalDatabaseErrorSignal.
!

innerExceptionFor: exception
	^exception exception.
! !

!VA55DatabaseAccessor methodsFor:'initialize'!

initialize
	super initialize.
	isInTransaction := false.
! !

!VA55DatabaseAccessor methodsFor:'login'!

connectionClassForLogin: aLogin 
	aLogin database isDB2Platform ifTrue: [^ Smalltalk at: #AbtIbmCliDatabaseManager].
	aLogin database class == OraclePlatform 
		ifTrue: [^Smalltalk at: #AbtOracle8DatabaseManager].
	aLogin database isODBCPlatform 
		ifTrue: [^Smalltalk at: #AbtOdbcDatabaseManager].
	aLogin database isPostgreSQLPlatform 
		ifTrue: [self error: 'PostgreSQL is not yet supported under VA'].
	self error: 'Unknown database platform' , aLogin database printString
!

isLoggedIn

	^connection notNil
!

logStream
	^Transcript
!

loginIfError: aBlock 
	"
		This method creates a NEW database connection.
	"

	| dbMgrClass anAbtDatabaseLogonSpec anAbtDatabaseConnectionSpec aliasNameString baseAliasName |
	self log: 'Login'.
	aliasNameString := baseAliasName := currentLogin connectString.

	"
		We try to generate a unique alias name to get a new connection and to 
		prevent the normal VAST behaviour to reuse a connection
	"
	
	[((Smalltalk at: #AbtDbmSystem) 
		activeDatabaseConnectionWithAlias: aliasNameString) notNil] 
			whileTrue: 
				[aliasNameString := baseAliasName , Time millisecondClockValue printString].


	"
		Get the native VA class responsible for doing the work against the
		three special connections available in VA
	"
	dbMgrClass := self connectionClass.

	"
		Get the logon specification for the database ... do not use
		any server information
	"
	anAbtDatabaseLogonSpec := (Smalltalk at: #AbtDatabaseLogonSpec) 
				id: currentLogin username
				password: currentLogin password
				server: nil.

	"	
		Get the specification for the connection ... 
   "
	anAbtDatabaseConnectionSpec := (Smalltalk at: #AbtDatabaseConnectionSpec) 
				forDbmClass: dbMgrClass
				dataSourceName: currentLogin connectString.
	connection := anAbtDatabaseConnectionSpec 
				connectUsingAlias: aliasNameString
				logonSpec: anAbtDatabaseLogonSpec
				ifError: 
					[:error | 
					"throw away the connection ..."
					connection := nil.
					aBlock value: error].
	connection isNil ifFalse: [
		connection databaseMgr errorBlock: [:err | self externalDatabaseErrorSignal signalWith: err]].
!

logout
	self isLoggedIn ifFalse: [^self].
	self log: 'Logout'.
	"disconnectIfError: doesn't properly catch errors."
	[
	[connection disconnectIfError: [:ex | self logError: ex]]
		ensure:	[connection := nil]]
		  on: Dialect error
		  do: [:ex | ex return: nil].
	self log: 'Logout finished'
!

showDialog: aString

	(Smalltalk at: #CwMessagePrompter) warn: aString.
! !

!VA55DatabaseAccessor methodsFor:'transactions'!

beginTransaction
	self log: 'Begin Transaction'.
	isInTransaction := true.
	self connection beginUnitOfWork.
!

commitTransaction
	self log: 'Commit Transaction'.
	isInTransaction := false.
	self connection commitUnitOfWorkIfError: 
			[:err |
			self log: 'Commit Transaction failed.'. 
			self connection autoCommit: true.
			self todo: 'Need to handle this error somehow....'.
			"errorBlock value: err"].
!

isInTransaction
	^isInTransaction
!

rollbackTransaction
	self log: 'Rollback Transaction'.
	isInTransaction := false.
	self connection rollbackUnitOfWork.
! !

!VA55DatabaseAccessor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/VA55DatabaseAccessor.st,v 1.1 2006/07/26 08:42:34 boris Exp $'
! !
