"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

DatabaseAccessor subclass:#VWDatabaseAccessor
	instanceVariableNames:'driverSession preparedStatements'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-VW3'
!


!VWDatabaseAccessor class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!VWDatabaseAccessor class methodsFor:'exception handling'!

errorsToCatch
	| pgError |
	pgError := Smalltalk at: #PostgreSQLEXDIException ifAbsent: [^Error].
	^Error, pgError.
! !

!VWDatabaseAccessor methodsFor:'accessing'!

createSession

	| session |
	session := connection getSession.
	(session respondsTo: #maxLongBytes:) ifTrue: [
		session maxLongBytes: 32767].  "Why does this not default larger for Oracle?"
	session blockFactor: 100.  "We don't know, so make it big"
	^session.
!

createSessionFor: aCommand

	| session types |
	session := connection getSession.
	(session respondsTo: #maxLongBytes:) ifTrue: [
		session maxLongBytes: 32767].  "Why does this not default larger for Oracle?"
	session blockFactor: aCommand blockFactor.
	types := aCommand bindingTypesForEXDI.
	types isEmpty ifFalse: [session bindTemplate: types].
	^session.
!

driverSession
	driverSession isNil ifTrue: [driverSession := self createSession].
	^driverSession.
!

encoding
	^connection encoding.
!

numberOfPreparedStatements
	^preparedStatements numberOfElements.
!

rowCount
	"GACK. Kind of assumes we know that we just prepared the last statement. And if the driver doesn't support it, just fake it. Very ugly. "
	self platform supportsRowCount ifFalse: [^1].
	^self driverSession rowCount.
! !

!VWDatabaseAccessor methodsFor:'copying'!

postCopy
	super postCopy.
	self initializePreparedStatementCache.
! !

!VWDatabaseAccessor methodsFor:'cursors'!

cursorFor: answerStream
	answerStream == #noAnswerStream ifTrue: [^#() readStream].
	answerStream == #noMoreAnswers ifTrue: [^#() readStream].
	^GlorpCursoredStream on: answerStream accessor: self.
!

cursorFor: answerStream command: aCommand
	| stream |
	stream := answerStream.
	answerStream == #noAnswerStream ifTrue: [
		stream := ReadStream on: #()].
	answerStream == #noMoreAnswers ifTrue: [
		stream := ReadStream on: #()].
	^aCommand isNil
		ifTrue: [GlorpCursoredStream on: stream accessor: self]
		ifFalse: [GlorpCursoredStream on: stream for: aCommand accessor: self].
!

releaseCursor: aCursor in: aPreparedStatement
	"Release the cursor. We're trapping exceptions, so return true or false to indicate success/failure"
	aCursor isNil ifTrue: [^self].
	aPreparedStatement isNil ifTrue: [
		 ^(aCursor class ~~ ReadStream) ifTrue: [self halt] ifFalse: [self]].
	[aPreparedStatement statement isNil or: [aPreparedStatement statement answer == #noMoreAnswers]] whileFalse.
	^[aCursor close. true] on: Dialect error do: [:ex | ex return: false].
! !

!VWDatabaseAccessor methodsFor:'executing'!

basicExecuteSQLString: aString
	^self basicExecuteSQLString: aString binding: nil.
!

basicExecuteSQLString: aString binding: aBindingArray 
	^self basicExecuteSQLString: aString returnResult: true binding: aBindingArray doing: [].
!

basicExecuteSQLString: aString returnResult: returnResult binding: aBindingArray doing: aBlock
	"Duplicate some of the logic of command execution, primarily because of the need to do special treatment depending if we want a result or not, and to execute the block. Don't cache the prepared statement, but do put it into the driverSesssion (ugh) so that we can check the rowCount if necessary"
	| answerStream command statement session |
	answerStream := nil.
	command := SQLStringCommand new.
	command setSQLString: aString.
	command parameters: aBindingArray.
	(aBindingArray isNil or: [aBindingArray class ~= Association and: [aBindingArray isEmpty]]) ifTrue: [command useBinding: false].
	statement := self newPreparedStatementFor: command.
	session := statement statement.
	driverSession := session.
	aBindingArray isNil ifFalse: [
		session bindInput: aBindingArray].
	session execute.
	"Always do this, because at least with ODBC, if we don't ask for the answer, we don't get errors, and subsequent statements get them"
	answerStream := session answer.
	answerStream == #noAnswerStream ifTrue: [answerStream := ReadStream on: #()].
	answerStream == #noMoreAnswers ifTrue: [answerStream := ReadStream on: #()].
	aBlock value.
	returnResult 
		ifTrue: [ | cursor |
			cursor := GlorpCursoredStream on: answerStream accessor: self.
			cursor statement: statement.
			^cursor]
		ifFalse: [session disconnect].
!

basicExecuteSQLStringNoResult: aString doing: aBlock
	^self basicExecuteSQLString: aString returnResult: false binding: nil doing: aBlock.
!

disconnect

	self reset.
	^connection disconnect
!

errorsToCatch
	^self class errorsToCatch.
!

executeCommand: aCommand inDBSession: aSession
	"Execute the command, given a database session which has already been prepared for this command"
	| answerStream cursor |
	aSession execute.
	answerStream := aSession answer.
	aCommand needsRowCount
		ifTrue:
			[aCommand
				rowCount:
					(self platform supportsRowCount
						ifTrue: [aSession rowCount]
						ifFalse: [1])].
	cursor := self cursorFor: answerStream command: aCommand.
	cursor statement: (GlorpPreparedStatement new statement: aSession).
	^cursor.
!

executeCommandBound: aCommand
	| statement cursor time block |
	self log: aCommand sqlString , '  ' , aCommand bindings printString.
	block := [
		statement := self statementFor: aCommand.
		statement statement bindInput: aCommand bindings.
		cursor := self executeCommand: aCommand inDBSession: statement statement.
		cursor statement: statement].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse: [block value].
	self log: ['(' , (time / 1000.0) printString , ' s)'].
	^cursor.
!

executeCommandUnbound: aCommand
	| statement cursor block time |
	self log: aCommand sqlString.
	block := [
		statement := self statementFor: aCommand.
		cursor := self executeCommand: aCommand inDBSession: statement statement.
		cursor statement: statement].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse: [block value].
	self log: ['(' , (time / 1000.0) printString , ' s)'].	
	^cursor.
!

externalDatabaseErrorSignal

	^connection class externalDatabaseErrorSignal.
!

newPreparedStatementFor: aCommand
	| newSession |
	newSession := self createSessionFor: aCommand.
	aCommand useBinding
		ifTrue: [newSession prepare: aCommand sqlString]
		ifFalse: [newSession prepareWithoutBinding: aCommand sqlString].
	^(GlorpPreparedStatement new)
		signature: aCommand signature;
		statement: newSession;
		busy: true.
!

preparedStatementFor: aCommand
	| wasMarkedBusyForUs statement |
	wasMarkedBusyForUs := false.
	statement := preparedStatements
		lookupClass: GlorpPreparedStatement
		key: aCommand signature
		ifAbsentPut:
			[wasMarkedBusyForUs := true.
			self newPreparedStatementFor: aCommand].
	(wasMarkedBusyForUs not and: [statement busy])
		ifTrue: [statement := self newPreparedStatementFor: aCommand].
	^statement.
!

statementFor: aCommand
	^self reusePreparedStatements
		ifTrue: [self preparedStatementFor: aCommand]
		ifFalse: [self newPreparedStatementFor: aCommand].
! !

!VWDatabaseAccessor methodsFor:'initialize'!

initializePreparedStatementCache

	| policy cache |
	policy := CachePolicy default.
	policy numberOfElements: 20.
	policy expiryAction: #notifyAndRemove.
	preparedStatements := CacheManager new.
	cache := preparedStatements cacheForClass:GlorpPreparedStatement.
	cache cachePolicy: policy.
!

reset
	super reset.	
	preparedStatements release.
	self initializePreparedStatementCache.
	driverSession := nil.
! !

!VWDatabaseAccessor methodsFor:'initializing'!

initialize
	super initialize.
	reusePreparedStatements := true.
	self initializePreparedStatementCache.
! !

!VWDatabaseAccessor methodsFor:'login'!

connectionClassForLogin: aLogin 
	^aLogin database vwConnectionClass
!

isLoggedIn

	connection isNil ifTrue: [^false].
	^connection isConnected
!

loginIfError: aBlock 
	self log: 'Login'.
	connection := self connectionClass new.
	connection username: currentLogin username.
	connection environment: currentLogin connectString.
	self execute: [connection connect: currentLogin password] ifError: aBlock.
	self log: 'Login finished'
!

logout
	self isLoggedIn ifFalse: [^self].
	self log: 'Logout'.
	self execute: [connection disconnect. self reset] ifError: [:ex | ex return: nil].
	self log: 'Logout finished'
!

reLogin
	"Attempt to re-login to a database that's not responding. Give it a maximum of 5 attempts"
	| attempts |
	attempts := 0.
	[attempts < 5 and: [self isLoggedIn not]] whileTrue: [
		attempts := attempts + 1.
		self loginIfError: [:ex | nil]].
!

showDialog: aString

	(Smalltalk at: #Dialog) warn: aString.
! !

!VWDatabaseAccessor methodsFor:'transactions'!

beginTransaction
	self log: 'Begin Transaction'.
	connection begin
!

commitTransaction
	self log: 'Commit Transaction'.
	connection commit
!

isInTransaction

	^connection inTransactionMode
!

rollbackTransaction
	self log: 'Rollback Transaction'.
	connection rollback
! !

!VWDatabaseAccessor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/VWDatabaseAccessor.st,v 1.1 2006/07/26 08:35:56 boris Exp $'
! !
