"{ Package: 'stx:goodies/glorp0_4' }"
!

!Association methodsFor:'accessing'!

glorpAtIdentity: aKey ifAbsent: aBlock
	"A cheat to let use use this like an identity dictionary"
	^key == aKey ifTrue: [value] ifFalse: aBlock.
! !

!Association methodsFor:'accessing'!

keysAndValuesDo: aBlock
	"A memory allocation hack. Let us use a single association in place of a dictionary"
	aBlock value: key value: value.
! !

!Block methodsFor:'glorp'!

asGlorpExpression

        ^self asGlorpExpressionOn: Glorp::BaseExpression new.
! !

!Block methodsFor:'glorp'!

asGlorpExpressionForDescriptor: aDescriptor

        | base |
        base := Glorp::BaseExpression new.
        base descriptor: aDescriptor.
        ^self asGlorpExpressionOn: base.
! !

!Block methodsFor:'glorp'!

asGlorpExpressionOn: anExpression 
        ^(self value: Glorp::MessageArchiver new) asGlorpExpressionOn: anExpression
! !

!Block methodsFor:'glorp'!

asGlorpExpressionOn: aBaseExpression in: aQuery

        ^self asGlorpExpressionOn: aBaseExpression.
! !

!Block methodsFor:'glorp'!

asGlorpExpressionOn: anExpression withUltimateBase: aBaseExpression
        "Build the expression, knowing that some parts of it may refer to an outer context based on aBaseExpression. This is primarily useful for subselect blocks"
        | archiver tree |
        archiver := Glorp::MessageArchiver new.
        tree := self value: archiver.
        ^tree asGlorpExpressionOn: anExpression basedOn: archiver withUltimateBase: aBaseExpression
! !

!Block methodsFor:'glorp'!

glorpIsBlock
        ^true.
! !

!ByteArray methodsFor:'testing'!

glorpIsCollection
	"For our purposes, these aren't collections, but rather a simple database type"
	^false.
! !

!Collection methodsFor:'glorp'!

assertEqual: aCollection
	"A convenience method for testing"
	self asSortedCollection with: aCollection asSortedCollection do: [:a :b |
		a assertEqual: b].
! !

!Collection methodsFor:'adding'!

glorpAddAll: aCollection
	^self addAll: aCollection.
! !

!Collection methodsFor:'adding'!

glorpAddAllToDictionary: aDictionary
	"Assumes that we are a collection of associations"
	self do: [:each | aDictionary add: each].
! !

!Collection methodsFor:'enumerating'!

glorpGroupedBy: aBlock
	"Same as VW groupedBy:. This is too useful not to be able to use, but we don't want to conflict with other dialects/extensions that might implement, so make our own prefixed version"

	"Return a dictionary whose keys are the result of evaluating aBlock for all elements in
	 the collection, and the value for each key is the collection of elements that evaluated
	 to that key. e.g.
	     #(1 2 3 4 5) groupedBy: [:each | each odd]
	   a Dictionary  
	     true ---> #( 1 3 5)
	     false --> #(2 4)"

	| result |
	result := Dictionary new.
	self do:
		[:each | | key collection |
		key := aBlock value: each.
		collection := result at: key ifAbsentPut: [OrderedCollection new].
		collection add: each].
	self species ~~ OrderedCollection ifTrue:
		["Convert the result collections to be the right type.
		  Note that it should be safe to modify the dictionary
		  while iterating because we only replace values for existing keys"
		result keysAndValuesDo:
			[:key :value | result at: key put: (self species withAll: value)]].

	^result
! !

!Collection methodsFor:'testing'!

glorpIsCollection

	^true.
! !

!Collection methodsFor:'glorp'!

glorpPrintSQLOn: aCommand
        aCommand nextPut: $(.
        self isEmpty
                ifTrue: [aCommand nextPutAll: 'NULL']
                ifFalse:
                        [Glorp::GlorpHelper
                                do: [:each | each glorpPrintSQLOn: aCommand]
                                for: self
                                separatedBy: [aCommand nextPutAll: ', ']].
        aCommand nextPut: $).
! !

!Collection methodsFor:'glorp'!

glorpPrintSQLOn: aStream for: aType
        aStream nextPut: $(.
        self isEmpty
                ifTrue: [aStream nextPutAll: 'NULL']
                ifFalse:
                        [Glorp::GlorpHelper
                                do: [:each | aType print: each on: aStream]
                                for: self
                                separatedBy: [aStream nextPutAll: ', ']].
        aStream nextPut: $).
! !

!Collection methodsFor:'glorp'!

glorpRegisterCollectionInternalsIn: anObjectTransaction 
	"Explicitly register any internal structures (e.g. a VW identity dictionary's valueArray) with the transaction. Assume we can safely register everything inside the collection reflectively. The obvious exceptions would be dependents and sortblocks. This is a cheat, and for peculiar cases you'll need to override this in the subclass"

	| names |
	names := self class allInstVarNames.
	(1 to: names size) do: 
			[:index | 
			(#('dependents' 'sortBlock') includes: (names at: index)) 
				ifFalse: [anObjectTransaction register: (self instVarAt: index)]]
! !

!Date methodsFor:'printing'!

glorpPrintSQLOn: aCommand
	"Print the date in ISO format. 'yyyy-mm-dd'  Don't rely on any dialect-specific formatting or padding mechanisms"
	| monthString dayString |
	aCommand 
		nextPut: $';
		print: self year;
		nextPut: $-.
	monthString := self monthIndex printString.
	monthString size = 1 ifTrue: [aCommand nextPut: $0 ].
	aCommand nextPutAll: monthString.
	aCommand nextPut: $-.
	dayString := self dayOfMonth printString.
	dayString size = 1 ifTrue: [aCommand nextPut: $0 ].
	aCommand nextPutAll: dayString.
	aCommand nextPut: $'.
! !

!Dictionary methodsFor:'adding'!

glorpAddAll: aCollection
	"An addAll: that can work for either a dictionary argument or a collection of associations"
	aCollection glorpAddAllToDictionary: self.
! !

!IdentityDictionary methodsFor:'accessing'!

glorpAtIdentity: aKey ifAbsent: aBlock
	^self at: aKey ifAbsent: aBlock.
! !

!Object methodsFor:'glorp'!

asGlorpExpression
        ^Glorp::GlorpHelper glorpConstantExpressionClass for: self.
! !

!Object methodsFor:'glorp'!

asGlorpExpressionOn: anExpression

	^self asGlorpExpression.
! !

!Object methodsFor:'glorp'!

asGlorpExpressionOn: anExpression basedOn: aMessageArchiver withUltimateBase: aBaseExpression

	^self asGlorpExpression.
! !

!Object methodsFor:'glorp'!

glorpBasicSize
	^self basicSize.
! !

!Object methodsFor:'glorp'!

glorpCanHaveDescriptorDirectly
	^self isBehavior.
! !

!Object methodsFor:'glorp'!

glorpEndOfIndexedInstVars
        "Return the index at which indexed instance variables start. The same as basicSize for most dialects, but funny for ObjectStudio"

        ^self basicSize.
! !

!Object methodsFor:'glorp'!

glorpIsBlock
        ^false.
! !

!Object methodsFor:'glorp'!

glorpIsCollection

	^false.
! !

!Object methodsFor:'glorp'!

glorpPostFetch: aSession
! !

!Object methodsFor:'glorp'!

glorpPostFetchValidate: aSession
	"This allows us to do post-read notification of the objects. Note that if this method explicitly returns a false, then we will treat that as meaning that the object is invalid and should not be read. Yes, this is kind of a hack."
! !

!Object methodsFor:'glorp'!

glorpPostWrite: aSession
! !

!Object methodsFor:'glorp'!

glorpPreWrite: aSession
! !

!Object methodsFor:'glorp'!

glorpPrintSQLOn: aCommand

	self printOn: aCommand.
! !

!Object methodsFor:'glorp'!

glorpRealObject
	"Return the real object. Will force instantiation of a proxy, even on compilers which inline yourself"
	^self.
! !

!Object methodsFor:'glorp'!

glorpStartOfIndexedInstVars
        "Return the index at which indexed instance variables start. 1 for most dialects, but funny for ObjectStudio"
        ^1.
! !

!Object methodsFor:'glorp'!

isGlorpExpression
	^false.
! !

!Object methodsFor:'glorp'!

isGlorpProxy
	^false.
! !

!Object methodsFor:'glorp'!

needsWork: aString

	^self.
! !

!Object methodsFor:'glorp'!

todo
	"marker"
! !

!Object methodsFor:'glorp'!

yourSelf
	"Return the real object. Will force instantiation of a proxy, even on compilers which inline yourself"
	^self.
! !

!Object class methodsFor:'glorp'!

glorpAllSubclasses
        "We need an isolation layer here because ObjectStudio's allSubclasses returns symbols"
        ^self allSubclasses.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

asGlorpExpression
        ^Glorp::GlorpHelper glorpConstantExpressionClass for: self.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

asGlorpExpressionOn: anExpression

        ^self asGlorpExpression.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

asGlorpExpressionOn: anExpression basedOn: aMessageArchiver withUltimateBase: aBaseExpression

        ^self asGlorpExpression.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

isGlorpExpression
        ^false.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

isGlorpProxy
        ^false.
! !

!ProtoObject methodsFor:'doesNotUnderstand'!

isKindOf: aClass
        ^self class includesBehavior: aClass.
! !

!ReadStream methodsFor:'glorp'!

actuallyAtEnd
	"For working around a VA cursor problem"
	^self atEnd
! !

!ReadStream methodsFor:'glorp'!

collect: aBlock 
	| newStream |
	newStream := WriteStream on: collection species new.
	[self atEnd] whileFalse: [newStream nextPut: (aBlock value: self next)].
	^newStream contents
! !

!ReadStream methodsFor:'glorp'!

collectionTypeFor: aClass
	^aClass.
! !

!ReadStream methodsFor:'glorp'!

detect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	^self detect: aBlock ifNone: [self error: 'not found']
! !

!ReadStream methodsFor:'glorp'!

detect: aBlock ifNone: exceptionBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	self do: [:each | (aBlock value: each) ifTrue: [^each]].
	^exceptionBlock value
! !

!ReadStream methodsFor:'glorp'!

includes: anObject
	self do: [:each | each = anObject ifTrue: [^true]].
	^false.
! !

!ReadStream methodsFor:'glorp'!

select: aBlock 
	| newStream |
	newStream := WriteStream on: collection species new.
	[self atEnd] whileFalse: [
		| current |
		current := self next.
		(aBlock value: current) ifTrue: [newStream nextPut: current]].
	^newStream contents
! !

!ReadStream methodsFor:'glorp'!

statement: aGlorpPreparedStatement
	"This is the statement used to create us. If it's coming back to a simple read stream, then we must have already used all its data, and can thus reuse it"
	aGlorpPreparedStatement busy: false.
! !

!SequenceableCollection methodsFor:'*glorp'!

atIndex: anIndex 
        ^self at: anIndex
! !

!String methodsFor:'glorp'!

asByteArrayEncoding: encoding
        "Answer an instance of ByteArray encoded in encoding whose elements
        are the elements of the receiver."

        ^self asByteArray
! !

!String methodsFor:'glorp'!

glorpIsCollection
	"For our purposes, these aren't collections, but rather a simple database type"
	^false.
! !

!String methodsFor:'glorp'!

glorpPrintSQLOn: aCommand
	| platform requiresEscape |
	platform := [aCommand platform] on: MessageNotUnderstood do: [:ex | ex return: nil].
	aCommand nextPut: $'.
	1
		to: self size
		do:
			[:i | 
			| char |
			char := self at: i.
			requiresEscape := platform isNil ifTrue: [false] ifFalse: [platform requiresEscapeFor: char].
			requiresEscape
				ifTrue: [aCommand nextPutAll: (aCommand platform escapeFor: char)]
				ifFalse: [aCommand nextPut: char]].
	aCommand nextPut: $'.
! !

!String methodsFor:'glorp'!

like: wildcardString
        " 'abcde' like: 'abc%'  "
        ^(wildcardString copyReplaceAll: $% with: $*) match: self.
! !

!Time methodsFor:'printing'!

glorpPadToTwoDigits: anInteger
	| string |
	string := anInteger truncated printString.
	^string size = 1 ifTrue: ['0', string] ifFalse: [string].
! !

!Time methodsFor:'printing'!

glorpPrintSQLOn: aCommand
	"Print as 24 hour time"

	aCommand 
		nextPut: $';
		nextPutAll: 	(self glorpPadToTwoDigits: self hours);
		nextPut: $:;
		nextPutAll: (self glorpPadToTwoDigits: self minutes);
		nextPut: $:;
		nextPutAll: (self glorpPadToTwoDigits: self seconds);
		nextPut: $'.
! !

!UndefinedObject methodsFor:'glorp'!

assertEqual: anObject
        Glorp::GlorpTestModelClass new assert: anObject = nil.
! !

!UndefinedObject methodsFor:'printing'!

glorpPrintSQLOn: aCommand

	aCommand nextPutAll: 'NULL'.
! !

