"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestDescriptorSystem subclass:#GlorpDemoDescriptorSystem
	instanceVariableNames:''
	classVariableNames:'Default'
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDemoDescriptorSystem class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDemoDescriptorSystem class methodsFor:'accessing'!

default
	Default isNil ifTrue: [Default := self new].
	^Default
!

dropDefault
        " Glorp::GlorpDemoDescriptorSystem dropDefault "
        Default := nil.
! !

!GlorpDemoDescriptorSystem methodsFor:'descriptors/airline'!

classModelForGlorpAirline: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
!

classModelForGlorpItinerary: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #reservation type: GlorpReservation.
!

classModelForGlorpPassenger: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #frequentFlyerMiles.
	aClassModel newAttributeNamed: #airline type: GlorpAirline.
!

classModelForGlorpReservation: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #passengers collectionOf: GlorpPassenger.
	aClassModel newAttributeNamed: #passenger type: GlorpPassenger.
!

descriptorForGlorpAirline: aDescriptor
	| table |
	table := self tableNamed: 'AIRLINE'.
	aDescriptor table: (self tableNamed: 'AIRLINE').
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	^aDescriptor
!

descriptorForGlorpItinerary: aDescriptor
	| table |
	table := self tableNamed: 'ITINERARY'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #reservation.
!

descriptorForGlorpPassenger: aDescriptor
	| passTable ffTable ffMiles airlineMapping |
	passTable := self tableNamed: 'PASSENGER'.
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	aDescriptor table: passTable.
	aDescriptor addTable: ffTable.
	aDescriptor addMultipleTableJoin: (
		Join
			from: (passTable fieldNamed: 'ID')
			to: (ffTable fieldNamed: 'ID')) beOuterJoin.
	(aDescriptor newMapping: DirectMapping) from: #id to: (passTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (passTable fieldNamed: 'NAME').

	ffMiles := aDescriptor newMapping: ConditionalMapping.
	ffMiles
		forField: (ffTable fieldNamed: 'POINTS')
		attribute: [:object | object frequentFlyerPoints]
		if: [:x | x notNil]
		useMapping: ((ffMiles newMapping: DirectMapping) from: #frequentFlyerMiles to: (ffTable fieldNamed: 'POINTS')).
	ffMiles otherwise: (ffMiles newMapping: ConstantMapping).

	airlineMapping := aDescriptor newMapping: ConditionalMapping.
	airlineMapping
		forField: (ffTable fieldNamed: 'AIRLINE_ID')
		attribute: [:object | object airline]
		if: [:x | x notNil]
		useMapping: ((airlineMapping newMapping: OneToOneMapping) attributeName: #airline).
	airlineMapping otherwise: (airlineMapping newMapping: ConstantMapping).
!

descriptorForGlorpReservation: aDescriptor
	| table |
	table := self tableNamed: 'RESERVATION'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	"The res->passenger relationship is actually 1-1, but map it as both 1-1 and 1-many so that we can more easily verify that only one object comes back, i.e. that joins are being done correctly"
	(aDescriptor newMapping: OneToManyMapping)
			attributeName: #passengers.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #passenger.
! !

!GlorpDemoDescriptorSystem methodsFor:'descriptors/bank'!

classModelForGlorpBankAccount: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #accountNumber type: GlorpBankAccountNumber.
	aClassModel newAttributeNamed: #accountHolders collectionOf: GlorpCustomer.
!

classModelForGlorpBankAccountNumber: aClassModel

	aClassModel newAttributeNamed: #bankCode.
	aClassModel newAttributeNamed: #branchNumber.
	aClassModel newAttributeNamed: #accountNumber.
!

classModelForGlorpBankBranch: aClassModel
	aClassModel newAttributeNamed: #branchNumber.
	aClassModel newAttributeNamed: #accounts collectionOf: GlorpBankAccount.
!

classModelForGlorpBankTransaction: aClassModel

        aClassModel newAttributeNamed: #id.
        aClassModel newAttributeNamed: #owner type: Glorp::GlorpCustomer.
        aClassModel newAttributeNamed: #amount type: Glorp::GlorpMoney.
        aClassModel newAttributeNamed: #serviceCharge type: Glorp::GlorpServiceCharge.
!

classModelForGlorpCustomer: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #transactions collectionOf: GlorpBankTransaction.
	aClassModel newAttributeNamed: #accounts collectionOf: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedById collectionOf: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedByIdDescending collectionOf: GlorpBankAccount.
!

classModelForGlorpEmailAddress: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #user.
	aClassModel newAttributeNamed: #host.
!

classModelForGlorpMoney: aClassModel

	aClassModel newAttributeNamed: #currency.
	aClassModel newAttributeNamed: #amount.
!

classModelForGlorpPerson: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #address type: GlorpAddress.
	aClassModel newAttributeNamed: #emailAddresses collectionOf: GlorpEmailAddress.
!

classModelForGlorpServiceCharge: aClassModel

	aClassModel newAttributeNamed: #description.
	aClassModel newAttributeNamed: #amount type: GlorpMoney.
!

descriptorForGlorpBankAccount: aDescriptor

	| table |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
				to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: ToManyMapping)
				attributeName: #accountHolders;
				useLinkTable.
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #accountNumber.
!

descriptorForGlorpBankAccountNumber: aDescriptor

	| table |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
"       aDescriptor addMapping: ((DirectMapping from: #bankCode to: (table fieldNamed: 'BANK_CODE')) type: Integer)."
	(aDescriptor newMapping: DirectMapping) from: #bankCode to: (table fieldNamed: 'BANK_CODE').
	(aDescriptor newMapping: DirectMapping) from: #branchNumber to: (table fieldNamed: 'BRANCH_NO').
	(aDescriptor newMapping: DirectMapping) from: #accountNumber to: (table fieldNamed: 'ACCT_NO').
!

descriptorForGlorpBankBranch: aDescriptor

	| table distinctBranchNumber mapping |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
	distinctBranchNumber := [:each | ((each getTable: table) getField: (table fieldNamed: 'BRANCH_NO')) distinct] asGlorpExpression.
	mapping := aDescriptor newMapping: DirectMapping.
	mapping from: #branchNumber to: distinctBranchNumber.
	mapping readOnly: true.

	aDescriptor keyFields: (Array with: distinctBranchNumber).

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #accounts;
		readOnly: true;
		join:
						(Join
								from: distinctBranchNumber
								to: (table fieldNamed: 'BRANCH_NO')).
!

descriptorForGlorpBankTransaction: aDescriptor
	| table |
	table := self tableNamed: 'BANK_TRANS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
				to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: OneToOneMapping)
				attributeName: #owner.
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #amount;
				fieldTranslation: ((Join new)
							addSource: (table fieldNamed: 'AMT_AMT')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'AMOUNT');
							addSource: (table fieldNamed: 'AMT_CURR')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'CURRENCY');
							yourself).
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #serviceCharge.
!

descriptorForGlorpCustomer: aDescriptor

	| table |
	table := self tableNamed: 'GR_CUSTOMER'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #transactions.
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #accounts;
			useLinkTable.

	"Two additional relationships, there to test ordering within a mapping, where the order is determined by a field in the link table"
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #accountsSortedById;
			readOnly: true;
			useLinkTable;
			orderBy: [:each | (each getTable: 'CUSTOMER_ACCT_LINK') getField: 'ACCT_ID'].
	(aDescriptor newMapping: ManyToManyMapping)
			attributeName: #accountsSortedByIdDescending;
			readOnly: true;
			useLinkTable;
			orderBy: [:each | ((each getTable: 'CUSTOMER_ACCT_LINK') getField: 'ACCT_ID') descending].
!

descriptorForGlorpEmailAddress: aDescriptor

	| table |
	table := self tableNamed: 'EMAIL_ADDRESS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #user to: (table fieldNamed: 'USER_NAME').
	(aDescriptor newMapping: DirectMapping) from: #host to: (table fieldNamed: 'HOST_NAME').
!

descriptorForGlorpMoney: aDescriptor

	| table |
	table := self tableNamed: 'MONEY_IMAGINARY_TABLE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #currency type: Symbol to: (table fieldNamed: 'CURRENCY').
	(aDescriptor newMapping: DirectMapping) from: #amount to: (table fieldNamed: 'AMOUNT').
	^aDescriptor.
!

descriptorForGlorpPerson: aDescriptor

	| table |
	table := self tableNamed: 'PERSON'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: DirectMapping) fromPseudoVariable: #addressId to: (table fieldNamed: 'ADDRESS_ID').
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #address.
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #emailAddresses;
			orderBy: #id.
!

descriptorForGlorpServiceCharge: aDescriptor

	| table |
	table := self tableNamed: 'BANK_TRANS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #description
				to: (table fieldNamed: 'SRVC_DESC').
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #amount;
				fieldTranslation: ((Join new)
							addSource: (table fieldNamed: 'SRVC_AMT_AMT')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'AMOUNT');
							addSource: (table fieldNamed: 'SRVC_AMT_CURR')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'CURRENCY');
							yourself).
! !

!GlorpDemoDescriptorSystem methodsFor:'descriptors/other'!

descriptorForGlorpCompressedMoney: aDescriptor
	| table currencyField amountField |
	table := self tableNamed: 'COMPRESSED_MONEY_TABLE'.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	currencyField := table fieldNamed: 'CURRENCY_NAME'.
	amountField := table fieldNamed: 'AMOUNT'.
	aDescriptor table: table.
	aDescriptor addMapping: (AdHocMapping
				forAttribute: #array
				fromDb:
					[:row :elementBuilder :context|
					Array with: (elementBuilder valueOfField: (context translateField: currencyField) in: row)
						with: (elementBuilder valueOfField: (context translateField: amountField) in: row)]
				toDb:
					[:rows :attribute :attributeRows |
					(rows at: table) at: currencyField put: (attribute at: 1).
					(rows at: table) at: amountField put: (attribute at: 2)]
				mappingFields: (Array with: currencyField with: amountField)).
	"Note that position won't work if we have a join. We need to take the elementbuilder into account"
	^aDescriptor
!

descriptorForGlorpTransformedTime: aDescriptor
	| table timeField |
	table := self tableNamed: 'TRANSFORMED_TIME'.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	timeField := table fieldNamed: 'TIMEFIELD'.
	aDescriptor table: table.
	aDescriptor addMapping: (AdHocMapping
				forAttribute: #time
				fromDb:
					[:row :elementBuilder :context|
					Time fromSeconds: (elementBuilder valueOfField: (context translateField: timeField) in: row) ]
				toDb:
					[:rows :attribute :attributeRows |
					(rows at: table) at: timeField put: attribute asSeconds]
				mappingFields: (Array with: timeField)).
	"Note that position won't work if we have a join. We need to take the elementbuilder into account"
	^aDescriptor
! !

!GlorpDemoDescriptorSystem methodsFor:'descriptors/tree nodes'!

classModelForGlorpTreeNode: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #children collectionOf: GlorpTreeNode.
!

descriptorForGlorpTreeNode: aDescriptor
	| table |
	table := self tableNamed: 'TREE_NODE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #children;
		useLinkTable;
		join:
				(Join
						from: (table fieldNamed: 'ID')
						to: ((self tableNamed: 'TREE_NODE_LINK') fieldNamed: 'PARENT')).
!

tableForTREE_NODE: aTable

	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
!

tableForTREE_NODE_LINK: aTable

	| parent child |
	parent := (aTable createFieldNamed: 'PARENT' type: platform int4) bePrimaryKey.
	(aTable addForeignKeyFrom: parent
		to: ((self tableNamed: 'TREE_NODE') fieldNamed: 'ID')) shouldCreateInDatabase: false  .
	child := (aTable createFieldNamed: 'CHILD' type: platform int4) bePrimaryKey.
	(aTable addForeignKeyFrom: child
		to: ((self tableNamed: 'TREE_NODE') fieldNamed: 'ID')) shouldCreateInDatabase: false.
! !

!GlorpDemoDescriptorSystem methodsFor:'examples'!

exampleAccountRow1

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 9874.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '1'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 2.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '3'.
	^row.
!

exampleAccountRow2

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 6.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '2'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 3.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '4'.
	^row.
!

exampleAccountRow3

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 22.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '2'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 712.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '5551212'.
	^row.
!

exampleAddressRow
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Paseo Montril'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row
!

exampleAddressRowForOrdering1
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 1.
	row at: (addressTable fieldNamed: 'STREET') put: 'Alpha'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '300'.
	^row
!

exampleAddressRowForOrdering2
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 2.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '200'.
	^row
!

exampleAddressRowForOrdering3
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 3.
	row at: (addressTable fieldNamed: 'STREET') put: 'Gamma'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '100'.
	^row
!

exampleAddressRowForOrdering4
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 6.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '120'.
	^row
!

exampleAddressRowForOrdering5
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 4.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '130'.
	^row
!

exampleAddressRowWithDifferentStreet
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Garden of the Gods'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row
!

exampleBankTransactionRow
	| table row |
	table := self tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.

	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'OWNER_ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'CDN'.
	row atFieldNamed: 'AMT_AMT' put: 7.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'USD'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.

	^row
!

exampleBankTransactionRow2
	| table row |
	table := self tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.

	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'OWNER_ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'AMT_AMT' put: 45.
	row atFieldNamed: 'SRVC_DESC' put: 'deposit'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 1.

	^row
!

exampleCALinkRow1

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 9874.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 27.
	^row.
!

exampleCALinkRow2

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 6.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 27.
	^row.
!

exampleCALinkRow3

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 22.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 28.
	^row.
!

exampleCALinkRow4

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 6.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 28.
	^row.
!

exampleCompressedMoneyRow
	| table row |
	table := self  tableNamed: 'COMPRESSED_MONEY_TABLE'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 123.
	row at: (table fieldNamed: 'AMOUNT') put: 12.
	row at: (table fieldNamed: 'CURRENCY_NAME') put: 'CDN'.
	^row
!

exampleCompressedMoneyRow2
	| table row |
	table := self  tableNamed: 'COMPRESSED_MONEY_TABLE'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 124.
	row at: (table fieldNamed: 'AMOUNT') put: 15.
	row at: (table fieldNamed: 'CURRENCY_NAME') put: 'CDN'.
	^row
!

exampleCustomerRow1

	| customerTable row |
	customerTable := self tableNamed: 'GR_CUSTOMER'.
	row := DatabaseRow newForTable: customerTable.
	row at: (customerTable fieldNamed: 'ID') put: 27.
	row at: (customerTable fieldNamed: 'NAME') put: 'aCustomer'.
	^row.
!

exampleCustomerRow2

	| customerTable row |
	customerTable := self tableNamed: 'GR_CUSTOMER'.
	row := DatabaseRow newForTable: customerTable.
	row at: (customerTable fieldNamed: 'ID') put: 28.
	row at: (customerTable fieldNamed: 'NAME') put: 'anotherCustomer'.
	^row.
!

exampleEmailAddressRow1

	| personTable row |
	personTable := self tableNamed: 'EMAIL_ADDRESS'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 42.
	row at: (personTable fieldNamed: 'USER_NAME') put: 'alan'.
	row at: (personTable fieldNamed: 'HOST_NAME') put: 'objectpeople.com'.
	row at: (personTable fieldNamed: 'PERSON_ID') put: 3.

	^row.
!

exampleEmailAddressRow2

	| personTable row |
	personTable := self tableNamed: 'EMAIL_ADDRESS'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 54321.
	row at: (personTable fieldNamed: 'USER_NAME') put: 'johnson'.
	row at: (personTable fieldNamed: 'HOST_NAME') put: 'cs.uiuc.edu'.
	row at: (personTable fieldNamed: 'PERSON_ID') put: 3.
	^row.
!

exampleFrequentFlyerRow
	| ffTable row |
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	row := DatabaseRow newForTable: ffTable.
	row at: (ffTable fieldNamed: 'ID') put: 1.
	row at: (ffTable fieldNamed: 'POINTS') put: 10000.
	row at: (ffTable fieldNamed: 'AIRLINE_ID') put: nil.
	^row.
!

exampleFrequentFlyerRow2
	| ffTable row |
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	row := DatabaseRow newForTable: ffTable.
	row at: (ffTable fieldNamed: 'ID') put: 2.
	row at: (ffTable fieldNamed: 'POINTS') put: 7.
	row at: (ffTable fieldNamed: 'AIRLINE_ID') put: nil.
	^row.
!

exampleModifiedAddressRow
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Something Else'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row
!

examplePassengerRow
	| passengerTable row |
	passengerTable := self tableNamed: 'PASSENGER'.
	row := DatabaseRow newForTable: passengerTable.
	row at: (passengerTable fieldNamed: 'ID') put: 1.
	row at: (passengerTable fieldNamed: 'NAME') put: 'Some Passenger'.
	^row.
!

examplePassengerRow2
	| passengerTable row |
	passengerTable := self tableNamed: 'PASSENGER'.
	row := DatabaseRow newForTable: passengerTable.
	row at: (passengerTable fieldNamed: 'ID') put: 2.
	row at: (passengerTable fieldNamed: 'NAME') put: 'Another Passenger'.
	^row.
!

examplePersonRow1

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 3.
	row at: (personTable fieldNamed: 'NAME') put: 'aPerson'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 123.
	^row.
!

examplePersonRow2

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 4.
	row at: (personTable fieldNamed: 'NAME') put: 'aPerson'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: nil.
	^row.
!

examplePersonRowForOrdering1

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 86.
	row at: (personTable fieldNamed: 'NAME') put: 'person1'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 2.
	^row.
!

examplePersonRowForOrdering2

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 87.
	row at: (personTable fieldNamed: 'NAME') put: 'person2'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 3.
	^row.
!

examplePersonRowForOrdering3

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 88.
	row at: (personTable fieldNamed: 'NAME') put: 'person3'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 1.
	^row.
! !

!GlorpDemoDescriptorSystem methodsFor:'other'!

allTableNames
	^#('GR_ADDRESS' 'PERSON' 'GR_CUSTOMER' 'BANK_TRANS' 'BANK_ACCT' 'CUSTOMER_ACCT_LINK' 'EMAIL_ADDRESS' 'STUFF' 'PASSENGER' 'AIRLINE' 'FREQUENT_FLYER' 'COMPRESSED_MONEY_TABLE' 'RESERVATION' 'ITINERARY' 'TRANSFORMED_TIME' 'FKCONTACT' 'FKADDRESS' 'GLORP_OWNER' 'GLORP_SLAVE' 'TREE_NODE' 'TREE_NODE_LINK').
!

constructAllClasses
	^(super constructAllClasses)
		add: GlorpPerson;
		add: GlorpAddress;
		add: GlorpCustomer;
		add: GlorpBankTransaction;
		add: GlorpBankAccount;
		add: GlorpMoney;
		add: GlorpCompressedMoney;
		add: GlorpServiceCharge;
		add: GlorpBankAccountNumber;
		add: GlorpEmailAddress;
		add: GlorpPassenger;
		add: GlorpAirline;
		add: GlorpReservation;
		add: GlorpItinerary;
		add: GlorpTransformedTime;
		add: GlorpBankBranch;
		add: GlorpOwner;
		add: GlorpSlave;
		add: GlorpTreeNode;
		yourself.
! !

!GlorpDemoDescriptorSystem methodsFor:'owner'!

descriptorForGlorpOwner: aDescriptor
	| table slaveTable |
	table := self tableNamed: 'GLORP_OWNER'.
	slaveTable := self tableNamed: 'GLORP_SLAVE'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID'));
		addMapping:
				((OneToManyMapping new)
						attributeName: #slaves;
						referenceClass: GlorpSlave;
						collectionType: OrderedCollection;
						orderBy:
								[:each | (each getTable: 'GLORP_SLAVE') getField: 'POSITION_IN_OWNER'];
						writeTheOrderField;
						mappingCriteria:
								(Join
										from: (table fieldNamed: 'ID')
										to: (slaveTable fieldNamed: 'OWNER_ID')));
		addMapping:
				((OneToOneMapping new)
						attributeName: #slave;
						referenceClass: GlorpSlave;
						mappingCriteria:
								(Join
										from: (table fieldNamed: 'SLAVE_ID')
										to: (slaveTable fieldNamed: 'ID'))).
!

descriptorForGlorpSlave: aDescriptor
	| table |
	table := self tableNamed: 'GLORP_SLAVE'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
!

tableForGLORP_OWNER: aTable
	| field |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	field := aTable createFieldNamed: 'SLAVE_ID' type: platform int4.
	aTable
		addForeignKeyFrom: field
		to: ((self tableNamed: 'GLORP_SLAVE') fieldNamed: 'ID').
!

tableForGLORP_SLAVE: aTable
	| field constraint |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'POSITION_IN_OWNER' type: platform int4.
	field := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	constraint := aTable
		addForeignKeyFrom: field
		to: ((self tableNamed: 'GLORP_OWNER') fieldNamed: 'ID').
	constraint shouldCreateInDatabase: false.
! !

!GlorpDemoDescriptorSystem methodsFor:'support'!

writeCustomer1Rows
	| linkRow2 id2 id1 accountRow1 customerRow accountRow2 linkRow1 |
	customerRow := self exampleCustomerRow1.
	accountRow1 := self exampleAccountRow1.
	accountRow2 := self exampleAccountRow2.
	linkRow1 := self exampleCALinkRow1.
	linkRow2 := self exampleCALinkRow2.
	session writeRow: customerRow.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	id1 := accountRow1 at: (accountRow1 table fieldNamed: 'ID').
	id2 := accountRow2 at: (accountRow2 table fieldNamed: 'ID').
	^Array with: id1 with: id2.
!

writeCustomerWithDualAccountOwners
	| linkRow2 id2 id1 accountRow1 accountRow2 linkRow1 customer1Row customer2Row linkRow4 |
	customer1Row := self exampleCustomerRow1.
	customer2Row := self exampleCustomerRow2.
	accountRow1 := self exampleAccountRow1.
	accountRow2 := self exampleAccountRow2.
	linkRow1 := self exampleCALinkRow1.
	linkRow2 := self exampleCALinkRow2.
	linkRow4 := self exampleCALinkRow4.
	session writeRow: customer1Row.
	session writeRow: customer2Row.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	session writeRow: linkRow4.
	id1 := accountRow1 at: (accountRow1 table fieldNamed: 'ID').
	id2 := accountRow2 at: (accountRow2 table fieldNamed: 'ID').
	^Array with: id1 with: id2.
! !

!GlorpDemoDescriptorSystem methodsFor:'tables'!

tableForBANK_ACCT: aTable

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'BANK_CODE' type: (platform varChar: 10).
	aTable createFieldNamed: 'BRANCH_NO' type: (platform int4).
	aTable createFieldNamed: 'ACCT_NO' type: (platform varChar: 10).
!

tableForBANK_TRANS: aTable
	| ownerId |
	(aTable createFieldNamed: 'ID' type: platform serial)
		bePrimaryKey.
	ownerId := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	aTable addForeignKeyFrom: ownerId
		to: ((self tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	aTable createFieldNamed: 'AMT_CURR' type: (platform varChar: 5).
	aTable createFieldNamed: 'AMT_AMT' type: platform int4.
	aTable createFieldNamed: 'SRVC_DESC' type: (platform varChar: 30).
	aTable createFieldNamed: 'SRVC_AMT_CURR' type: (platform varChar: 5).
	aTable createFieldNamed: 'SRVC_AMT_AMT' type: platform int4.
!

tableForCOMPRESSED_MONEY_TABLE: aTable
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'CURRENCY_NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'AMOUNT' type: platform int4.
!

tableForCUSTOMER_ACCT_LINK: aTable

	| customerId accountId |
	customerId := (aTable createFieldNamed: 'CUSTOMER_ID' type: platform int4).
	aTable addForeignKeyFrom: customerId to: ((self tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	accountId := aTable createFieldNamed: 'ACCT_ID' type: platform int4.
	aTable addForeignKeyFrom: accountId to: ((self tableNamed: 'BANK_ACCT') fieldNamed: 'ID').
	aTable addIndexForFields: (Array with: customerId with: accountId).
!

tableForEMAIL_ADDRESS: aTable

	| personId |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	(aTable createFieldNamed: 'USER_NAME' type: (platform varChar: 20)) beIndexed.
	aTable createFieldNamed: 'HOST_NAME' type: (platform varChar: 20).
	personId := aTable createFieldNamed: 'PERSON_ID' type: platform int4.
	personId beIndexed.
	(aTable addForeignKeyFrom: personId to: ((self tableNamed: 'PERSON') fieldNamed: 'ID')).
!

tableForFKADDRESS: aTable
	| contact |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	contact := aTable createFieldNamed: 'CONTACT_ID' type: platform int4.
	aTable addForeignKeyFrom: contact
		to: ((self tableNamed: 'FKCONTACT') fieldNamed: 'ID')
!

tableForFKCONTACT: aTable
	| address |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	address := aTable createFieldNamed: 'ADDRESS_ID' type: platform int4.
	aTable addForeignKeyFrom: address
		to: ((self tableNamed: 'FKADDRESS') fieldNamed: 'ID')
!

tableForGR_ADDRESS: aTable

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'STREET' type: (platform varChar: 20).
	aTable createFieldNamed: 'HOUSE_NUM' type: (platform varChar: 20)
!

tableForGR_CUSTOMER: aTable
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)
!

tableForMONEY_IMAGINARY_TABLE: aTable

	aTable createFieldNamed: 'CURRENCY' type: (platform varChar: 5).
	aTable createFieldNamed: 'AMOUNT' type: platform int4
!

tableForPERSON: aTable
	| addrId|
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	addrId := aTable createFieldNamed: 'ADDRESS_ID' type: platform int4.
	aTable addForeignKeyFrom: addrId to: ((self tableNamed: 'GR_ADDRESS') fieldNamed: 'ID').
!

tableForSTUFF: aTable

	(aTable createFieldNamed: 'ID' type: platform inMemorySequence)
		bePrimaryKey.
	aTable createFieldNamed: 'THING' type: (platform varChar: 20)
!

tableForTRANSFORMED_TIME: aTable
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'TIMEFIELD' type: platform int4.
! !

!GlorpDemoDescriptorSystem methodsFor:'tables/airline'!

tableForAIRLINE: aTable

	(aTable createFieldNamed: 'ID' type: platform inMemorySequence) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)
!

tableForAIRLINE_MEAL: aTable

	aTable createFieldNamed: 'ID' type: platform int4.
	aTable createFieldNamed: 'DESCR' type: (platform varChar: 20).
	aTable createFieldNamed: 'FLIGHT_ID' type: platform int4
!

tableForFLIGHT: aTable

	aTable name: 'FLIGHT'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'FLIGHT_NUM' type: platform int4
!

tableForFLIGHT_PASS: aTable

	| flight pass airline |
	aTable name: 'FLIGHT_PASS'.
	flight := aTable createFieldNamed: 'FLIGHT_ID' type: platform int4.
	aTable addForeignKeyFrom: flight
		to: ((self tableNamed: 'FLIGHT') fieldNamed: 'ID').
	pass := aTable createFieldNamed: 'PASS_ID' type: platform int4.
	aTable addForeignKeyFrom: pass
		to: ((self tableNamed: 'PASSENGER') fieldNamed: 'ID').
	airline := aTable createFieldNamed: 'AIRLINE_ID' type: platform int4.
	aTable addForeignKeyFrom: airline
		to: ((self tableNamed: 'AIRLINE') fieldNamed: 'ID').
!

tableForFREQUENT_FLYER: aTable
	| airlineId |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'POINTS' type: platform int4.
	airlineId := aTable createFieldNamed: 'AIRLINE_ID' type: platform int4.
	aTable addForeignKeyFrom: airlineId to: ((self tableNamed: 'AIRLINE') fieldNamed: 'ID').
!

tableForITINERARY: aTable

	| res |
	(aTable createFieldNamed: 'ID' type: platform serial)
		bePrimaryKey.
	res := aTable createFieldNamed: 'RES_ID' type: (platform int4).
	aTable addForeignKeyFrom: res
		to: ((self tableNamed: 'RESERVATION') fieldNamed: 'ID').
!

tableForPASSENGER: aTable
	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)
!

tableForRESERVATION: aTable

	| pass |
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	pass := aTable createFieldNamed: 'PASS_ID' type: (platform int4).
	aTable addForeignKeyFrom: pass
		to: ((self tableNamed: 'PASSENGER') fieldNamed: 'ID').
! !

!GlorpDemoDescriptorSystem methodsFor:'test support'!

testIDLessThan5: aValue
	"A test method, used for conditional mapping tests"
	^aValue < 5.
!

testIDMoreThan5: aValue
	"A test method, used for conditional mapping tests"
	^aValue > 5.
! !

!GlorpDemoDescriptorSystem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/GlorpDemoDescriptorSystem.st,v 1.1 2006/07/26 09:10:14 boris Exp $'
! !
