"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpCompositeKeyTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpCompositeKeyTest methodsFor:'setup'!

setUp
	super setUp.
	system := (GlorpCompositeKeyDescriptorSystem forPlatform: session platform).
	session system: system.
	session beginTransaction.
	session transact: [session register: system example1].
	session reset.
!

tearDown
	session rollbackTransaction.
	session resetPermissions.
	super tearDown.
! !

!GlorpCompositeKeyTest methodsFor:'tests- basic reading'!

testAlsoFetch
	| users |
	users := session readManyOf: GlorpUser.
!

testBasicReadFolder
	| folders |
	folders := (session readManyOf: GlorpFolder) asSortedCollection: [:a :b | a name <= b name].
	self assert: folders size = 3.
	self assert: folders first name = 'One'.
	self assert: folders last name = 'Two'.
!

testBasicReadMessage
	| messages |
	messages := (session readManyOf: GlorpMessage) asSortedCollection: [:a :b | a subject <= b subject].
	self assert: messages size = 3.
	self assert: messages first subject = 'also goes in folder 1'.
	self assert: (messages at: 2) subject = 'goes in folder 1'.
	self assert: messages last subject = 'goes in folder 2'.
!

testBasicReadUser
	| users |
	users := (session readManyOf: GlorpUser) asSortedCollection: [:a :b | a id <= b id].
	self assert: users size = 2.
	self assert: users first name = 'User One'.
	self assert: users last name = 'User Two'.
!

testFaulting
	| users user1 sortedFolders user2 |
	users := session readManyOf: GlorpUser.
	user1 := users detect: [:each | each id = 1].
	self assert: user1 folders size = 2.
	sortedFolders := user1 folders asSortedCollection: [:a :b | a name < b name].
	self assert: sortedFolders first name = 'One'.
	user2 := users detect: [:each | each id = 2].
	self assert: user2 folders first messages first subject = 'goes in folder 2'.
!

testFolderIdentity
	| folders |
	folders := session readManyOf: GlorpFolder.
	folders do: [:each | | individualFolders individualFolder otherIndividualFolder|
		individualFolders := session readManyOf: GlorpFolder where: [:eachFolder | eachFolder = each].
		self assert: individualFolders size = 1.
		individualFolder := individualFolders first.
		otherIndividualFolder := session readOneOf: GlorpFolder where: [:eachFolder | eachFolder = each].
		self assert: individualFolder == each.
		self assert: otherIndividualFolder == individualFolder].
!

testMessageIdentity
	| messages |
	messages := session readManyOf: GlorpMessage.
	messages do: [:each | | individualMessages individualMessage otherIndividualMessage|
		individualMessages := session readManyOf: GlorpMessage where: [:eachMessage | eachMessage = each].
		self assert: individualMessages size = 1.
		individualMessage := individualMessages first.
		otherIndividualMessage := session readOneOf: GlorpMessage where: [:eachMessage | eachMessage = each].
		self assert: individualMessage == each.
		self assert: otherIndividualMessage == individualMessage].
!

testReadWithCacheHitFromExpression
	| message |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	self assert: message folder name = 'One'.
	session accessor permitNothing.
	session readOneOf: GlorpFolder where: [:each | each user id = 1 & (each name = 'One')].
!

testReadWithCacheHitFromJoin
	| message |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	session readOneOf: GlorpFolder where: [:each | each user id = 1 & (each name = 'One')].
	session accessor permitNothing.
	self assert: message folder name = 'One'.
!

testReadWithCacheHitFromParameterizedExpression
	| message query |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	self assert: message folder name = 'One'.
	session accessor permitNothing.
	query := Query readOneOf: GlorpFolder where: [:each | each user id = (each parameter: 1)  & (each name = (each parameter: 2))].
	query executeWithParameters: #(1 'One') in: session.
!

testReadWithJoins
	| users |
	users := session readManyOf: GlorpUser where: [:each | each folders anySatisfyJoin: [:eachFolder |
		eachFolder messages anySatisfyJoin: [:eachMessage | eachMessage subject = 'goes in folder 1']]].
	self assert: users size = 1.
	self assert: users first id = 1.
!

testReadWithSubselect
	| users |
	users := session readManyOf: GlorpUser where: [:each | each folders anySatisfyExists: [:eachFolder |
		eachFolder messages anySatisfyExists: [:eachMessage | eachMessage subject = 'goes in folder 1']]].
	self assert: users size = 1.
	self assert: users first id = 1.
!

testReadWithSubselectAndINClause
	| users |
	self should: [
		users := session readManyOf: GlorpUser where: [:each |
			each folders anySatisfyExists: [:eachFolder |
				| q |
				q := Query read: GlorpFolder.
				eachFolder in: q]]]
		raise: GlorpInvalidExpressionError.
!

testUserIdentity
	| users |
	users := session readManyOf: GlorpUser.
	users
		do:
			[:each |
			| individualUser otherIndividualUser individualUsers |
			individualUsers := session
				readManyOf: GlorpUser
				where: [:eachIndividual | eachIndividual id = each id].
			self assert: individualUsers size = 1.
			individualUser := individualUsers first.
			otherIndividualUser := session
				readOneOf: GlorpUser
				where: [:eachIndividual | eachIndividual id = each id].
			self assert: individualUser == each.
			self assert: otherIndividualUser == individualUser].
!

testUserIdentityViaMessage
	| users |
	users := session readManyOf: GlorpUser.
	users
		do:
			[:each |
			each folders
				do:
					[:eachFolder |
					eachFolder messages
						do: [:eachMessage |
							self assert: eachMessage folder yourSelf == eachFolder.
							self assert: eachMessage user yourSelf == each]]].
!

testUserIdentityViaMessageInUnitOfWork
	session beginUnitOfWork.
	[self testUserIdentityViaMessage] ensure: [session rollbackUnitOfWork].
! !

!GlorpCompositeKeyTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpCompositeKeyTest.st,v 1.1 2006/07/26 08:59:24 boris Exp $'
! !
