"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpConstantValueInRelationshipTest
	instanceVariableNames:'session system'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpConstantValueInRelationshipTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpConstantValueInRelationshipTest class methodsFor:'resources'!

resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.
! !

!GlorpConstantValueInRelationshipTest methodsFor:'support'!

idsFor: aCollection

	^(aCollection collect: [:each | each id]) asSortedCollection asArray.
!

sampleWorker

	| worker job3 |
	worker := GlorpWorker new.
	worker id: 1234.
	worker name: 'Some Worker'.
	worker pendingJobs add: (GlorpJob new id: 1; description: 'job 1').
	worker pendingJobs add: (GlorpJob new id: 2; description: 'job 2').
	worker finishedJobs add: (job3 := GlorpJob new id: 3; description: 'job 3').
	worker finishedJobs add: (GlorpJob new id: 4; description: 'job 4').
	worker priorityJobs add: job3.
	^worker.
!

setUp
	system := GlorpWorkerDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database.
	session := GlorpSessionResource current newSession.
	session system: system.
!

writeTestData

	session writeRow: system exampleWorkerRow.
	session writeRow: (system exampleJobRow: 1 finished: false).
	session writeRow: (system exampleJobRow: 2 finished: false).
	session writeRow: (system exampleJobRow: 3 finished: true).
	session writeRow: (system exampleJobRow: 4 finished: true).
	session writeRow: system exampleLinkRow1.
	session writeRow: system exampleLinkRow2.
! !

!GlorpConstantValueInRelationshipTest methodsFor:'tests'!

testRead

	| worker |
	[session beginTransaction.
	self writeTestData.
	worker := session execute: (Query returningOneOf: GlorpWorker where: [:each | each id = 1234]).

	self assert: (self idsFor: worker pendingJobs) = #(1 2).
	self assert: (self idsFor: worker finishedJobs) = #(3 4).
	self assert: (self idsFor: worker priorityJobs) = #(3).

	] ensure: [session rollbackTransaction].
!

testReadWithJoin
	| worker |
	[session beginTransaction.
	self writeTestData.
	worker := session
		execute:
			(Query returningOneOf: GlorpWorker where: [:each | each pendingJobs id = 1]).
	self assert: worker id = 1234] ensure: [session rollbackTransaction].
!

testWrite

	| worker sampleWorker |
	[session beginTransaction.
	session beginUnitOfWork.
	sampleWorker := self sampleWorker.
	session register: sampleWorker.
	session commitUnitOfWork.
	session reset.
	worker := session execute: (Query returningOneOf: GlorpWorker where: [:each | each id = 1234]).

	self assert: (self idsFor: worker pendingJobs) = #(1 2).
	self assert: (self idsFor: worker finishedJobs) = #(3 4).
	self assert: (self idsFor: worker priorityJobs) = #(3).

	] ensure: [session rollbackTransaction].
! !

!GlorpConstantValueInRelationshipTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpConstantValueInRelationshipTest.st,v 1.1 2006/07/26 09:13:38 boris Exp $'
! !
