"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpCursoredCollectionDBTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpCursoredCollectionDBTest methodsFor:'support'!

setUp
	session := GlorpSessionResource current newSession.
	session beginTransaction.
	self writePersonRows.
!

tearDown
	session rollbackTransaction.
	super tearDown.
!

writePersonRows
	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.
! !

!GlorpCursoredCollectionDBTest methodsFor:'tests'!

testParallelStreams
	| query person query2 stream1 stream2 address person2 address2 |
	session platform supportsMultipleOpenCursors ifFalse: [^self].
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each id].
	query collectionType: GlorpCursoredStream.
	query2 := Query returningManyOf: GlorpAddress.
	query2 orderBy: [:each | each id].
	query2 collectionType: GlorpCursoredStream.
	stream1 := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	stream2 := session execute: query2.
	person := stream1 next.
	self assert: session privateGetCache numberOfElements = 1.
	self assert: person id = 86.
	address := stream2 next.
	self assert: address id = 1.
	self assert: address street = 'Alpha'.
	self assert: session privateGetCache numberOfElements = 2.
	person2 := stream1 next.
	address2 := stream2 next.
	self assert: person2 class == GlorpPerson.
	self assert: address2 class == GlorpAddress.
	self assert: address2 street = 'Beta'.
!

testReadAsStream
	| query stream person restOfPeople |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each id].
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 1.
	self assert: person id = 86.
	restOfPeople := stream upToEnd.
	self assert: restOfPeople size = 2.
	self assert: session privateGetCache numberOfElements = 3.
!

testReadEmptyStream
	| query stream person |
	query := Query returningManyOf: GlorpPerson where: [:each | each id = 999].
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 0.
	self assert: person isNil.
	self assert: stream atEnd.
!

testReadLiteralEmptyStream
	| query stream person |
	query := Query returningManyOf: GlorpPerson where: false.
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 0.
	self assert: person isNil.
	self assert: stream atEnd.
! !

!GlorpCursoredCollectionDBTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpCursoredCollectionDBTest.st,v 1.1 2006/07/26 08:58:48 boris Exp $'
! !
