"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpDatabaseBasicTest
	instanceVariableNames:'system'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDatabaseBasicTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDatabaseBasicTest class methodsFor:'resources'!

resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.
! !

!GlorpDatabaseBasicTest methodsFor:'accessing'!

accessor
	^self databaseLoginResource accessor
! !

!GlorpDatabaseBasicTest methodsFor:'support'!

databaseLoginResource
	^GlorpDatabaseLoginResource current.
!

setUp

	super setUp.
	system := GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database.
! !

!GlorpDatabaseBasicTest methodsFor:'tests'!

testBeginTransactionWithCommit
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor commitTransaction.
	self assert: self accessor isInTransaction not
!

testBeginTransactionWithRollback
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor rollbackTransaction.
	self assert: self accessor isInTransaction not
!

testCreateTable
	| selectResult |
	[[self accessor beginTransaction.
	self accessor
		executeSQLStringNoResult: 'CREATE TABLE GLORP_TEST_CREATE (ID varchar(4))']
		ensure: [self accessor commitTransaction].
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM GLORP_TEST_CREATE'.
	self assert: selectResult isEmpty]
		ensure:
			[[self accessor
				beginTransaction;
				dropTableNamed: 'GLORP_TEST_CREATE'
					ifAbsent: [:ex | self assert: false]]
				ensure: [self accessor commitTransaction]].
!

testDropMissingTable
	| absentFlag |
	absentFlag := false.
	[self accessor beginTransaction.
	self accessor
		dropTableNamed: 'GLORP_TEST_DROP'
		ifAbsent: [:ex  | absentFlag := true. ex sunitExitWith: nil]]
	ensure: [self accessor rollbackTransaction].
	self assert: absentFlag
!

testNameBinding
	| selectResult |
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :key OR ID = :value'
			binding: 10->30.

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].
!

testPositionalBinding
	| selectResult |
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :2 OR ID = :17'
			binding: #(nil 10 nil nil nil nil nil nil nil nil nil nil nil nil nil nil 20).

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].
!

testReadEmpty
	| results |
	results := self accessor
				executeSQLString: 'SELECT * FROM PERSON'.
	self assert: results size = 0
!

testReadStatement
	| results |
	results := self accessor
				executeSQLString: 'SELECT * FROM STUFF ORDER BY ID'.
	self assert: results size = 5.
	self assert: results first size = 2.
	self assert: results first last =  'abc'
!

testRollbackRemovesData
	"Just to make sure I'm not losing my mind"
	| numAddresses newNumAddresses|
	numAddresses := (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.
	self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO GR_ADDRESS (ID,STREET,HOUSE_NUM)  VALUES (111,''Main Street'',''77777'')'.
	newNumAddresses := (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.
	self assert: newNumAddresses = (numAddresses + 1).
	self accessor rollbackTransaction.
	self assert: numAddresses = (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.
! !

!GlorpDatabaseBasicTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDatabaseBasicTest.st,v 1.1 2006/07/26 09:12:52 boris Exp $'
! !
