"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

TestResource subclass:#GlorpDatabaseLoginResource
	instanceVariableNames:'accessor login'
	classVariableNames:'DefaultLogin'
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDatabaseLoginResource class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDatabaseLoginResource class methodsFor:'accessing'!

constructEventsTriggered
	^(super constructEventsTriggered)
		add: #changedDefaultLogin;
		yourself
!

defaultAccessLogin
	"To set the default database login for Access, execute the following statement."

	"DefaultLogin := self defaultAccessLogin."
	"TestResource withAllSubclasses do: [:each | each reset]."

	^(Login new)
		database: AccessPlatform new;
		username: 'whatever';
		password: '';
		connectString: 'access'
!

defaultFirebirdLogin
	"To set the default database login to Firebird, execute the following statement."
	"DefaultLogin := self defaultFirebirdLogin."

	^(Login new)
		database: FirebirdPlatform new;
		username: 'BaseSystem';
		password: 'BaseSyst';
		connectString: 'firebirdGlorpTestDB'
!

defaultLogin
	"Return the default Login."

	DefaultLogin isNil ifTrue: [^DefaultLogin := self defaultOracleLogin2].
	DefaultLogin database useBinding: DatabasePlatform useBindingIfSupported.
	^DefaultLogin
!

defaultLogin: aLogin
	DefaultLogin := aLogin.
	self triggerEvent: #changedDefaultLogin
!

defaultMysqlLogin
	"To set the default database login to MySQL, execute the following statement."
	"DefaultLogin := self defaultMysqlLogin."

	^(Login new)
		database: MySQLPlatform new;
		username: '';
		password: '';
		connectString: 'test'.
!

defaultOracleLiteLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLiteLogin."

	^(Login new)
		database: OraclePlatform new;
		username: 'System';
		password: 'password';
		connectString: 'odbc:polite'
!

defaultOracleLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLogin."

	^(Login new)
		database: OraclePlatform new;
		username: 'System';
		password: 'manager';
		connectString: ''
!

defaultOracleLogin2
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLogin."

	^(Login new)
		database: OraclePlatform new;
		username: 'System';
		password: 'password';
		connectString: ''
!

defaultOracleODBCLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleODBCLogin."

	^(Login new)
		database: OracleODBCPlatform new;
		username: 'System';
		password: 'manager';
		connectString: 'oracle'
!

defaultPersonalOracleLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultPersonalOracleLogin."

	^(Login new)
		database: OraclePlatform new;
		username: 'system';
		password: 'manager';
		connectString: ''
!

defaultPlatform
	^self defaultLogin database.
!

defaultPostgreSQLInternetLogin
	"To set the default database login to PostgreSQL, execute the following statement."
	"DefaultLogin := self defaultPostgreSQLInternetLogin."

	^(Login new)
		database: PostgreSQLPlatform new;
		username: 'guest';
		password: 'guest';
		connectString: 'store.cincomsmalltalk.com:5432_glorptest'.
!

defaultPostgreSQLLocalLogin
        "To set the default database login to PostgreSQL, execute the following statement."
        "DefaultLogin := self defaultPostgreSQLLocalLogin."

        ^(Login new)
                database: PostgreSQLPlatform new;
                username: 'etas';
                password: '';
                connectString: 'localhost_mydb';
                name: 'PostgreSQL ANSI'.
        " ODBC does not use the connect string, it uses the data source name "
!

defaultPostgreSQLLogin
	"To set the default database login to PostgreSQL, execute the following statement."
	"DefaultLogin := self defaultPostgreSQLLogin."

	^(Login new)
		database: PostgreSQLPlatform new;
		username: 'glorp-test';
		password: 'simple';
		connectString: '192.168.0.18_glorp-test'.
!

defaultPostgreSQLLoginForGlorpStore
	"To set the default database login to PostgreSQL, execute the following statement."
	"DefaultLogin := self defaultPostgreSQLLoginForGlorpStore."

	^(Login new)
		database: PostgreSQLPlatform new;
		username: 'store-glorp-test';
		password: 'simple';
		connectString: '192.168.0.18_store-glorp-test'.
!

defaultPostgreSQLODBCLogin
        "To set the default database login to PostgreSQL that is accessed via ODBC, execute the following statement."
        "DefaultLogin := self defaultPostgreSQLODBCLogin."

        ^(Login new)
                database: PostgreSqlODBCPlatform new;
                username: 'etas';
                password: '';
                connectString: 'localhost_mydb';
                name: 'PostgreSQL ANSI'.
        " ODBC does not use the connect string, it uses the data source name "
!

defaultSQLServerLogin
	"To set the default database login for Dolphin, execute the following statement."

	"DefaultLogin := self defaultSQLServerLogin."

	^(Login new)
		database: SQLServerPlatform new;
		username: 'glorptest';
		password: 'password';
		connectString: 'glorptest'
! !

!GlorpDatabaseLoginResource methodsFor:'accessing'!

accessor
	^accessor
!

accessor: anObject
	accessor := anObject
!

login
	^login
!

login: anObject
	login := anObject
!

platform
	^login database
! !

!GlorpDatabaseLoginResource methodsFor:'initialize/release'!

setUp
	Transcript show: self class name asString, ' setUp'; cr.
	super setUp.
	login := self class defaultLogin.
	accessor := DatabaseAccessor forLogin: login.
	accessor login.
!

tearDown

	Transcript show: self class name asString, ' tearDown'; cr.
	accessor notNil ifTrue: [[accessor logout] on: Dialect error do: [:ex | ]].
! !

!GlorpDatabaseLoginResource class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDatabaseLoginResource.st,v 1.1 2006/07/26 09:13:05 boris Exp $'
! !
