"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpDatabasePlatformTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDatabasePlatformTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDatabasePlatformTest methodsFor:'tests'!

addToString: aString
	"Append a couple of special characters. The result will utf-8 encode to be 3 characters longer than its number of characters"
	^aString, (String with: (Character value: 16r2022) "bullet point" with: (Character value: 16r0131) "Turkish i with no dot")
!

helpTestPrintTimestamp: aTimestampString

        | aStream ts timestamp |
        ts := DatabasePlatform new readTimestamp: aTimestampString for: nil.
        aStream := WriteStream on: String new.
        DatabasePlatform new printDate: ts asDate isoFormatOn: aStream.
        aStream nextPut: Character space.
        DatabasePlatform new printTime: ts isoFormatOn: aStream milliseconds: true.
        timestamp := aStream contents.
        self assert: timestamp = aTimestampString.
!

string20
	| platform |
	platform := PostgreSQLPlatform new.
	platform characterEncoding: #'utf-8'.
	^platform varchar: 20.
!

testNumericCoercion

	| type platform num |
	type := GlorpNumericType new scale: 5; precision: 2.
	platform := DatabasePlatform new.
	num := Dialect coerceToDoublePrecisionFloat: (123456 / 100).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123450 / 100).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123450 / 1000).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123456 / 1000).
	self deny: (platform convertToNumber: num for: type) = num.
!

testPrintTimestamp

	| strings |
	strings := #( '2005-01-01 21:29:28.337'  '2005-01-01 01:02:00.037'  '2005-01-01 21:29:28.002'  '2005-01-01 21:29:28.001').
	strings do: [:each |
		self helpTestPrintTimestamp: each].
!

testReadTimestamp

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self assert: ts asSeconds = 3224158168.
	self assert: ([ts asMilliseconds = 3224158168337 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #asMilliseconds ]).

	ts := DatabasePlatform new readTimestamp: '2003-03-13 15:29:28.337-05' for: nil.
	self assert: ts asSeconds = 3225022168.
	self assert: ([ts asMilliseconds = 3225022168337]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #asMilliseconds ])
!

testReadTimestampNoMS

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28-05' for: nil.
	self assert: ts year = 2003.
	self assert: ts monthIndex = 3.
	self assert: ts dayOfMonth = 3.
	self assert: ts hour = 15.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 0 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])
!

testReadTimestampNoMSNoTZ

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28' for: nil.
	self assert: ts year = 2003.
	self assert: ts monthIndex = 3.
	self assert: ts dayOfMonth = 3.
	self assert: ts hour = 15.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 0 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])
!

testReadTimestampNoTZ

	| ts |
	ts := DatabasePlatform new readTimestamp: '1957-08-13 21:29:28.337' for: nil.
	self assert: ts year = 1957.
	self assert: ts monthIndex = 8.
	self assert: ts dayOfMonth = 13.
	self assert: ts hour = 21.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds =  337]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])
!

testReadTimestampOverflowDays

	| ts |
	ts := DatabasePlatform new readTimestamp: '1957-08-13 21:29:28.337-05' for: nil.
	self assert: ts year = 1957.
	self assert: ts monthIndex = 8.
	self assert: ts dayOfMonth = 13.
	self assert: ts hour = 21.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 337 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])
!

testTrimString1
	"Normal ascii"
	| type plat |
	type := self string20.
	plat := type platform.
	self assert: (plat trimString: 'abcdefghi' for: type) = 'abcdefghi'.
	self assert: (plat trimString: 'abcdefghijklmnopqrst' for: type) = 'abcdefghijklmnopqrst'.
	self assert: (plat trimString: 'abcdefghijklmnopqrstu' for: type) = 'abcdefghijklmnopqrst'.
	self assert: (plat trimString: 'abcdefghijklmnopqrstuvwxy' for: type) = 'abcdefghijklmnopqrst'.
!

testTrimString2
	"Some special characters"
	| type plat string |
	type := self string20.
	plat := type platform.

	string := self addToString: 'abcdefghi'.
	self assert: (plat trimString: string for: type) = string.
	string := self addToString: 'abcdefghijklmno'.
	self assert: (plat trimString: string for: type) = string.
	string := self addToString: 'abcdefghijklmnop'.
	self assert: (plat trimString: string for: type) = (string copyFrom: 1 to: string size -1).
	string := self addToString: 'abcdefghijklmnopq'.
	self assert: (plat trimString: string for: type) = (string copyFrom: 1 to: string size -1).
	string := (String new: 100000) atAllPut: $a.
	self assert: (plat trimString: string for: type) = 'aaaaaaaaaaaaaaaaaaaa'
! !

!GlorpDatabasePlatformTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDatabasePlatformTest.st,v 1.1 2006/07/26 09:17:39 boris Exp $'
! !
