"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpDatabaseSequenceTest
	instanceVariableNames:'sequence table row2 row1'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpDatabaseSequenceTest methodsFor:'setup'!

setUp
	super setUp.
	table := session system tableNamed: 'BANK_TRANS'.
	sequence := (table fieldNamed: 'ID') type sequence.
	row1 := DatabaseRow newForTable: table.
	row2 := DatabaseRow newForTable: table.
! !

!GlorpDatabaseSequenceTest methodsFor:'tests'!

assignSequences
	| accessor |
	accessor := session isNil ifTrue: [nil] ifFalse: [session accessor].
	row1 preWriteAssignSequencesUsing: accessor.
	row1 postWriteAssignSequencesUsing: accessor.
	row2 preWriteAssignSequencesUsing: accessor.
	row2 postWriteAssignSequencesUsing: accessor.
!

row1Value
	^row1 atFieldNamed: 'ID'.
!

row2Value
	^row2 atFieldNamed: 'ID'.
!

testGetNextValue
	session platform usesIdentityColumns ifTrue: [^self].
	self assignSequences.
	self assert: self row2Value = (self row1Value + 1).
!

testReserveEmpty
	session platform usesIdentityColumns ifTrue: [^self].
	sequence reserveSequenceNumbers: 2 in: session for: table.
	session := nil.
	self assignSequences.
	self assert: self row1Value + 1 = self row2Value.
!

testReserveUsingTable
	session platform usesIdentityColumns ifTrue: [^self].
	[session beginTransaction.
	self writeTransactionRow: 65.
	self writeTransactionRow: 66.
	self writeTransactionRow: 67.
	sequence
		reserveSequenceNumbers: 7
		in: session
		for: table] ensure: [session rollbackTransaction].
	session := nil.
	self assignSequences.
	self assert: self row1Value + 1 = self row2Value.
!

testReservedNumberAllocation
	session := nil.
	sequence reservedNumbers add: 23; add: 24.
	self assignSequences.
	self assert: self row1Value = 23.
	self assert: self row2Value = 24.
!

writeTransactionRow: anInteger
	| tempRow |
	tempRow := DatabaseRow newForTable: table.
	session platform usesIdentityColumns ifFalse: [
		tempRow atFieldNamed: 'ID' put: anInteger].
	tempRow atFieldNamed: 'AMT_CURR' put: 'CDN'.
	session writeRow: tempRow.
! !

!GlorpDatabaseSequenceTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDatabaseSequenceTest.st,v 1.1 2006/07/26 08:56:45 boris Exp $'
! !
