"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpDatabaseTypeIndividualDBTests
	instanceVariableNames:'type stType connection session table'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp-DBTests'
!


!GlorpDatabaseTypeIndividualDBTests class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDatabaseTypeIndividualDBTests class methodsFor:'resources'!

resources
	^Array with: GlorpDatabaseLoginResource with: GlorpSessionResource.
! !

!GlorpDatabaseTypeIndividualDBTests methodsFor:'accessing'!

platform
	^connection platform
! !

!GlorpDatabaseTypeIndividualDBTests methodsFor:'helpers'!

compareFloatInModel: read with: original
	read isNil ifTrue: [^false].
	read test = original ifTrue: [^true].
	read test isNil ifTrue: [^false].
	^(read test - original) abs <= 0.00001.
!

helpTestExplicitReadBackFor: dbOutValue
	"Read back the row with the given value, using an explicit where clause."
	| queryStream result |
	queryStream := SQLStringSelectCommand sqlString: String new parameters: #() useBinding: false platform: session platform.
	queryStream nextPutAll: 'SELECT TEST,ID FROM TYPETESTS WHERE TEST '.
	queryStream nextPutAll: (dbOutValue isNil ifTrue: [' IS '] ifFalse: [' = ']).
	type print: dbOutValue on: queryStream.
	result := connection executeSQLString: queryStream contents.
	result isEmpty ifTrue: [^nil].
	^(result atIndex: 1) atIndex: 1.
!

helpTestFloat: anObject
	"Don't try and read back an equal float, it'll likely fail on precision issues"

	self
		helpTestValue: anObject
		compareModelWith:
			[:read :original | self compareFloatInModel: read with: original]
		compareWith:
			[:read :original | original isNil or: [(read - original) abs <= 0.00001]].
!

helpTestInvalidValue: anObject
	self helpTestValue: anObject
		compareModelWith: [:read :original | read isNil or: [ read test ~= original ]]
		compareWith: [:read :original | read ~= original]
!

helpTestValue: anObject
	"Don't try and read back an equal float, it'll likely fail on precision issues"

	self
		helpTestValue: anObject
		compareModelWith:
			[:read :original |
			read notNil and:
					[(original isKindOf: Float) or:
							[original class == Dialect doublePrecisionFloatClass
								or: [read test = original]]]]
		compareWith: [:read :original | read = original]
!

helpTestValue: anObject compareModelWith: modelBlock compareWith: aBlock
	self
		helpTestValueWithSQLWrite: anObject
		compareModelWith: modelBlock
		compareWith: aBlock.
	self
		helpTestValueWithUnitOfWorkWrite: anObject
		compareModelWith: modelBlock
!

helpTestValue: anObject compareWith: aBlock
	self
		helpTestValue: anObject
		compareModelWith: [:read :original | true]
		compareWith: aBlock
!

helpTestValueWithSQLWrite: anObject compareModelWith: modelBlock compareWith: aBlock

	| dbInValue readObject row converter dbOutValue typeTestModel system dbInValue2 readObject2 |

	system := self systemFor: table.
	session system: system.
	row := DatabaseRow newForTable: table.
	row owner: GlorpTypeTestsModelClass new.
	converter := type converterForStType: (stType isNil ifTrue: [anObject class] ifFalse: [stType]).
	dbOutValue := converter convert: anObject toDatabaseRepresentationAs: type.
	row atFieldNamed: 'test' put: dbOutValue.
	[
	session beginTransaction.
	session writeRow: row.

	dbInValue := (connection executeSQLString: 'SELECT TEST, ID FROM TYPETESTS') first atIndex: 1.
	readObject := converter convert: dbInValue fromDatabaseRepresentationAs: type.

	(Dialect isFloatingPoint: dbOutValue) ifFalse: [
		dbInValue2 := self helpTestExplicitReadBackFor: dbOutValue.
		readObject2 := converter convert: dbInValue2 fromDatabaseRepresentationAs: type].

	typeTestModel := self readBackModelWithValue: anObject.

	self assert: (modelBlock value: typeTestModel value: anObject)] ensure: [
		session rollbackTransaction].
	self assert: (aBlock value: readObject value: anObject).
	readObject2 isNil ifFalse: [
		self assert: (aBlock value: readObject2 value: anObject)].
!

helpTestValueWithUnitOfWorkWrite: anObject compareModelWith: aBlock
	| typeTestModel system model |
	system := self systemFor: table.
	session system: system.
	[session beginTransaction.
	session beginUnitOfWork.
	model := GlorpTypeTestsModelClass new test: anObject.
	session register: model.
	session commitUnitOfWork.
	session reset.
	typeTestModel := self readBackModelWithValue: anObject.
	self assert: (aBlock value: typeTestModel value: anObject)]
		ensure: [session rollbackTransaction].
!

readBackModelWithValue: anObject
	^session
		readOneOf: GlorpTypeTestsModelClass
		where: [:each | each test = anObject].
! !

!GlorpDatabaseTypeIndividualDBTests methodsFor:'initializing'!

initialize
! !

!GlorpDatabaseTypeIndividualDBTests methodsFor:'setUp'!

createTypeTestTable
	| system |
	self platform requiresTransactionForTableOperations ifTrue: [connection beginTransaction].
	connection
		dropTableNamed: 'TYPETESTS'
		ifAbsent: [:ex | ex return: nil].
	table := DatabaseTable named: 'TYPETESTS'.
	table
		createFieldNamed: 'test'
		type: type.
	(table createFieldNamed: 'id' type: session system platform inMemorySequence) bePrimaryKey.
	connection
		createTable: table
		ifError:
			[:ex |
			Transcript show: 'CANNOT CREATE TABLE'.
			self platform requiresTransactionForTableOperations ifTrue: [connection rollbackTransaction].
		self signalFailure: ex messageText.
			ex return: nil].
	self platform requiresTransactionForTableOperations ifTrue: [connection commitTransaction].
	system := self systemFor: table.
	session system: system.
	^table.
!

defaultDatabaseType
	self subclassResponsibility.
!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	connection := session accessor.
	type := self defaultDatabaseType.
	table := self createTypeTestTable.
!

systemFor: aTable
	| system descriptor mapping model |
	system := DynamicDescriptorSystem new.
	model := system addClassModelFor: GlorpTypeTestsModelClass.
	model newAttributeNamed: #id.
	model newAttributeNamed: #test type: stType.
	system privateTableAt: aTable name put: aTable.
	descriptor := Descriptor new.
	descriptor system: system.
	descriptor classModel: model.
	descriptor table: aTable.
	descriptor
		addMapping: (DirectMapping from: #id to: (aTable fieldNamed: 'id')).
	stType isNil
		ifTrue:
			[mapping := DirectMapping from: #test to: (aTable fieldNamed: 'test')]
		ifFalse:
			[mapping := DirectMapping
				from: #test
				type: stType
				to: (aTable fieldNamed: 'test')].
	descriptor addMapping: mapping.
	system privateDescriptorAt: GlorpTypeTestsModelClass put: descriptor.
	^system.
!

tearDown
	super tearDown.
	self platform requiresTransactionForTableOperations ifTrue: [connection beginTransaction].
	connection
		dropTableNamed: 'TYPETESTS'
		ifAbsent: [:ex | ex return: nil].
	self platform requiresTransactionForTableOperations ifTrue: [connection commitTransaction].

	session reset.
	session := nil.
! !

!GlorpDatabaseTypeIndividualDBTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDatabaseTypeIndividualDBTests.st,v 1.1 2006/07/26 08:55:24 boris Exp $'
! !
