"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpDeleteInUnitOfWorkTest
	instanceVariableNames:'unitOfWork'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDeleteInUnitOfWorkTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDeleteInUnitOfWorkTest methodsFor:'As yet unclassified'!

system
	^unitOfWork session system.
!

tableNamed: aString
	^self system tableNamed: aString.
! !

!GlorpDeleteInUnitOfWorkTest methodsFor:'support'!

setUp
	| session |
	session := GlorpMockSession new.
	session beginUnitOfWork.
	unitOfWork := session privateGetCurrentUnitOfWork.
	session system: (GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database)
!

tearDown

	unitOfWork := nil.
! !

!GlorpDeleteInUnitOfWorkTest methodsFor:'tests'!

testCommit

	unitOfWork delete: GlorpAddress example1.
	unitOfWork commit.
	self assert: (unitOfWork numberOfRows) = 1.
!

testDeleteRegistration
	| obj |
	obj := Object new.
	unitOfWork delete: obj.
	self assert: (unitOfWork willDelete: obj).
	self deny: (unitOfWork willDelete: 3).
!

testDeletesComeAfterUpdates
	unitOfWork delete: GlorpAddress example1.
	unitOfWork register: GlorpCustomer example1.
	unitOfWork commit.
	self assert: unitOfWork session rows last table == (self tableNamed: 'GR_ADDRESS').
!

testDeletesInReverseOrder
	"Not that good a test, because it could be luck with only two tables. Should test this at a lower level"
	| cust trans |
	cust := GlorpCustomer example2.
	trans := cust transactions first.
	unitOfWork delete: cust.
	unitOfWork delete: trans.
	unitOfWork commit.
	self assert: unitOfWork session rows last owner == cust.
	self assert: (unitOfWork session rows reversed at: 2) owner == trans.
!

testGeneratingDeleteRows
	unitOfWork delete: GlorpAddress example1.
	unitOfWork createRows.
	self assert: unitOfWork privateGetRowMap numberOfEntries = 1.
	unitOfWork rowsForTable: (self tableNamed: 'GR_ADDRESS')
		do: [:each | self assert: each forDeletion]
! !

!GlorpDeleteInUnitOfWorkTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDeleteInUnitOfWorkTest.st,v 1.1 2006/07/26 09:10:51 boris Exp $'
! !
