"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpDeleteTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDeleteTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDeleteTest methodsFor:'support'!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
!

setUpFolders

	session system: (GlorpCompositeKeyDescriptorSystem forPlatform: session platform).
	session beginTransaction.
	session transact: [session register: session system example1].
	session reset.
!

tearDown
	super tearDown.
	session reset.
	session := nil.
! !

!GlorpDeleteTest methodsFor:'tests'!

checkCustomerDeletedInDatabase
	| result |
	result := session accessor
				executeSQLString: 'SELECT * FROM GR_CUSTOMER WHERE ID=1'.
	self assert: result isEmpty
!

checkCustomerNotInCache
	self assert: (session cacheLookupForClass: GlorpCustomer key: 1) isNil
!

checkCustomerStillInCache
	self assert: (session cacheLookupForClass: GlorpCustomer key: 1) notNil
!

checkFoldersDeletedInDatabase
	| result generalResult |
	result := session accessor
				executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result isEmpty.

	generalResult := session accessor
				executeSQLString: 'SELECT * FROM GR_FOLDER'.
	self assert: generalResult size = 1.
!

checkFoldersExistInDatabase
	| result |
	result := session accessor
				executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result size = 2.
!

checkFoldersNotInCache
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One')) isNil.
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One-A')) isNil.
!

checkMessagesDeletedInDatabase
	| result generalResult |
	result := session accessor
				executeSQLString: 'SELECT * FROM GR_MESSAGE WHERE USER_ID = 1'.
	self assert: result isEmpty.

	generalResult := session accessor
				executeSQLString: 'SELECT * FROM GR_MESSAGE'.
	self assert: generalResult size = 1.
!

checkPersonDeletedInDatabase
	| result |
	result := session accessor
				executeSQLString: 'SELECT * FROM PERSON WHERE ID=1'.
	self assert: result isEmpty
!

setUpCustomer
	session beginTransaction.
	session accessor
		executeSQLStringNoResult: 'INSERT INTO GR_CUSTOMER VALUES (1,''Fred Flintstone'')'.
	^session readOneOf: GlorpCustomer where: [:each | each id = 1].
!

setUpPersonWithAddress
	session beginTransaction.
	session accessor
		executeSQLStringNoResult: 'INSERT INTO GR_ADDRESS VALUES (2,''Paseo Montril'', ''999'')'.
	session accessor
		executeSQLStringNoResult: 'INSERT INTO PERSON VALUES (1,''Fred Flintstone'', 2)'.
	^session readOneOf: GlorpPerson where: [:each | each id = 1].
!

testCascadingDeleteAcrossNil
	| person addressRows personRows |
	session inTransactionDo: [session accessor dropConstraint: (session system tableNamed: 'PERSON') foreignKeyConstraints first].
	[
	((session system descriptorFor: GlorpPerson) mappingForAttributeNamed: #address) beExclusive.
	self setUpPersonWithAddress .

	session accessor executeSQLString: 'delete from GR_ADDRESS where ID = 2'.
	session reset.
	person := session readOneOf: GlorpPerson.
	"First, make sure this doesn't walkback"
	[session beginUnitOfWork.
	session delete: person] ensure: [session commitUnitOfWork].
	"Now make sure the address is left"
	addressRows := session accessor executeSQLString: 'SELECT * FROM GR_ADDRESS'.
	self assert: addressRows isEmpty.
	personRows :=  session accessor executeSQLString: 'SELECT * FROM PERSON'.
	self assert: personRows isEmpty.

	]
		ensure: [
			session rollbackTransaction.
			session inTransactionDo: [session accessor createTableFKConstraints: (session system tableNamed: 'PERSON') ifError: [self halt]]]
!

testCascadingDeleteInstantiating
	| user |

	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	user folders do: [:each | each printString].
	session delete: user.
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase.
	self checkMessagesDeletedInDatabase]
			ensure: [session rollbackTransaction]
!

testCascadingDeleteWithoutInstantiating
	| user |

	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	session delete: user.
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase]
			ensure: [session rollbackTransaction]
!

testDeleteMultipleTable
	| passengerRow1 passengerRow2 passenger rows frequentFlyerRow1 frequentFlyerRow2 |

	[
	session beginTransaction.
	passengerRow1 := session system examplePassengerRow.
	session writeRow: passengerRow1.
	frequentFlyerRow1 := session system exampleFrequentFlyerRow.
	session writeRow: frequentFlyerRow1.
	passengerRow2 := session system examplePassengerRow2.
	session writeRow: passengerRow2.
	frequentFlyerRow2 := session system exampleFrequentFlyerRow2.
	session writeRow: frequentFlyerRow2.
	passenger := session readOneOf: GlorpPassenger.
	session delete: passenger.
	rows := session accessor executeSQLString: 'SELECT * FROM FREQUENT_FLYER'.
	self assert: rows size = 1.
	rows := session accessor executeSQLString: 'SELECT * FROM PASSENGER'.
	self assert: rows size = 1.
	]
			ensure: [session rollbackTransaction]
!

testDeleteMultipleTableWithOneMissing
	| passengerRow1 passengerRow2 passenger rows frequentFlyerRow2 |

	[
	session beginTransaction.
	passengerRow1 := session system examplePassengerRow.
	session writeRow: passengerRow1.
	passengerRow2 := session system examplePassengerRow2.
	session writeRow: passengerRow2.
	frequentFlyerRow2 := session system exampleFrequentFlyerRow2.
	session writeRow: frequentFlyerRow2.
	passenger := session readOneOf: GlorpPassenger where: [:each | each id = 1].
	session delete: passenger.
	rows := session accessor executeSQLString: 'SELECT * FROM FREQUENT_FLYER'.
	self assert: rows size = 1.
	rows := session accessor executeSQLString: 'SELECT * FROM PASSENGER'.
	self assert: rows size = 1.
	]
			ensure: [session rollbackTransaction]
!

testNormalDeleteOnFolders
	| user |

	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	session delete: user.
	user folders do: [:each |
		session delete: each.
		each messages do: [:eachMessage |
			session delete: eachMessage]].
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase.
	self checkMessagesDeletedInDatabase]
			ensure: [session rollbackTransaction]
!

testUnitOfWorkDelete
	| customer result |

	[customer := self setUpCustomer.
	session beginUnitOfWork.
	session delete: customer.
	result := session accessor
				executeSQLString: 'SELECT * FROM GR_CUSTOMER WHERE ID=1'.
	self assert: result size = 1.
	self
		assert: (session readOneOf: GlorpCustomer where: [:each | each id = 1]) isNil.
	self checkCustomerStillInCache.
	session commitUnitOfWork.
	self checkCustomerNotInCache.
	self checkCustomerDeletedInDatabase]
			ensure: [session rollbackTransaction]
!

testUnitOfWorkDeleteOrder
	| person |

	[person := self setUpPersonWithAddress.
	session beginUnitOfWork.
	session delete: person.
	session delete: person address.
	session commitUnitOfWork.
	self checkPersonDeletedInDatabase.]
			ensure: [session rollbackTransaction]
! !

!GlorpDeleteTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDeleteTest.st,v 1.1 2006/07/26 09:16:26 boris Exp $'
! !
