"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpDatabaseBasedTest subclass:#GlorpDescriptorTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDescriptorTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDescriptorTest methodsFor:'support'!

errorSignal

	^Dialect error.
! !

!GlorpDescriptorTest methodsFor:'tests'!

testAllClassesAndNames
	| identity1 identity2 identity3 |
	system flushAllClasses.
	identity1 := system allClasses.
	identity2 := system allClasses.
	system flushAllClasses.
	identity3 := system allClasses.
	self assert: identity1 == identity2.
	self assert: identity1 ~~ identity3.
	self should: [system allClassNames] raise: Dialect error.
!

testAllMappingsForField
	| descriptor mappings |
	descriptor := system descriptorFor: GlorpCustomer.
	mappings := descriptor
				allMappingsForField: ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	self assert: mappings first attributeName = #id
!

testBuildBankTransactionAndDependentsFromRow
	| transactionDescriptor object row moneyDescriptor money1 table translations session builder |
	session := GlorpMockSession new.
	session system: system.
	transactionDescriptor := system descriptorFor: GlorpBankTransaction.
	object := GlorpBankTransaction new.
	row := #(99 nil 'CDN' 98 'service charge' 'USD' 97).
	builder := ObjectBuilder new.
	builder row: row.
	transactionDescriptor populateObject: object inBuilder: builder.
	self assert: object id = 99.
	moneyDescriptor := system descriptorFor: GlorpMoney.
	money1 := GlorpMoney new.
	table := system tableNamed: 'MONEY_IMAGINARY_TABLE'.
	translations := IdentityDictionary new.
	translations at: (table fieldNamed: 'CURRENCY') put: 3.
	translations at: (table fieldNamed: 'AMOUNT') put: 4.
	builder := ElementBuilder new fieldTranslations: translations; row: row.
	moneyDescriptor
		populateObject: money1
		inBuilder: builder.
	self assert: money1 amount = 98.
	self assert: money1 currency = #CDN.
!

testBuildPersonFromRow
	| descriptor object address session builder |
	session := GlorpMockSession new.
	session system: system.
	address := GlorpAddress new.
	session cacheAt: 127 put: address.
	descriptor := system descriptorFor: GlorpPerson.
	system tableNamed: 'PERSON'.
	object := GlorpPerson new.
	builder := ObjectBuilder new.
	builder row: #(456 'Ralph' 127).
	descriptor populateObject: object inBuilder: builder.
	self assert: object class = GlorpPerson.
	self assert: object id = 456.
	self assert: object name = 'Ralph'.
	self assert: object address getValue == address.
!

testClassLookup

	self assert: (Dialect smalltalkAt: 'Object') == Object.

	Dialect isVWWithNameSpaces ifTrue: [
		"Sneaky meta-tricks to avoid things that won't compile in non-VW dialects"
		self assert: (Dialect smalltalkAt: 'XProgramming.SUnit.TestCase') == (Compiler evaluate: 'XProgramming.SUnit.TestCase')].
!

testDescriptorIdentity

	| descriptor |
	descriptor := system descriptorFor: GlorpCustomer.
	self assert: descriptor == (system descriptorFor: GlorpCustomer).
!

testDescriptorWithNamespace

	| descriptor testCaseClass |
	Dialect isVWWithNameSpaces ifFalse: [^self].
	system := GlorpDescriptorSystemWithNamespaces new.
	testCaseClass := 'GlorpTestNamespace.GlorpTestClassInNamespace' asQualifiedReference value.
	descriptor := system descriptorFor: testCaseClass.
	self assert: descriptor describedClass == testCaseClass.
!

testMappedFields

	| descriptor |
	descriptor := system descriptorFor: GlorpBankTransaction.
	self assert: (descriptor mappedFields) = (descriptor table fields).
!

testMappingForField
	| descriptor mapping |
	descriptor := system descriptorFor: GlorpCustomer.
	mapping := descriptor
				directMappingForField: ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	self assert: mapping attributeName = #id
!

testPrimaryKeyExpressionFor
	| descriptor trans exp |
	descriptor := system descriptorFor: GlorpBankTransaction.
	trans := GlorpBankTransaction new.
	trans id: 42.
	exp := descriptor
		primaryKeyExpressionFor: trans
		basedOn: BaseExpression new
		relation: #=.
	self assert: exp relation = #=.
	self assert: exp rightChild value = 42.
!

testPrimaryKeyExpressionForFailing
	| descriptor trans |
	descriptor := system descriptorFor: GlorpBankTransaction.
	trans := GlorpCustomer new.
	self
		should:
			[descriptor
				primaryKeyExpressionFor: trans
				basedOn: BaseExpression new
				relation: #=]
		raise: self errorSignal.
!

testPrimaryKeyExpressionForWithCompositeKey

	self unfinished.
! !

!GlorpDescriptorTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDescriptorTest.st,v 1.1 2006/07/26 09:16:44 boris Exp $'
! !
