"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

TestCase subclass:#GlorpDescriptorValidationTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTests'
!


!GlorpDescriptorValidationTest methodsFor:'testing'!

testJoinDirection
	| descriptor table1 table2 mapping system descriptor2 |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	descriptor2 := system addEmptyDescriptorFor: GlorpVideoStore.
	table1 := DatabaseTable named: 'Table1'.
	table1 createFieldNamed: 'one' type: nil.
	table2 := DatabaseTable named: 'Table2'.
	table2 createFieldNamed: 'two' type: nil.
	descriptor addTable: table1.
	descriptor2  addTable: table2.
	mapping := descriptor newMapping: OneToOneMapping.
	mapping attributeName: #accounts.
	mapping referenceClass: GlorpVideoStore.
	mapping join: (Join from: (table1 fieldNamed: 'one') to: (table2 fieldNamed: 'two')).
	mapping validate.

	mapping := descriptor newMapping: OneToOneMapping.
	mapping attributeName: #bar.
	mapping referenceClass: GlorpVideoStore.
	mapping join: (Join from: (table2 fieldNamed: 'two') to: (table1 fieldNamed: 'one')).
	self should: [mapping validate] raise: Error.
!

testValidateDirectInstvarAccessExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #id to: field.
	mapping validate
!

testValidateDirectInstvarAccessNonExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #bar to: field.
	self should: [mapping validate] raise: Error
!

testValidateSelectorAccessExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #id to: field.
	mapping attribute useDirectAccess: false.
	mapping validate
!

testValidateSelectorAccessNonExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #foo to: field.
	mapping attribute useDirectAccess: false.
	self should: [mapping validate] raise: Error
!

testValidatingExistingDescriptors
	| session |
	session := GlorpSessionResource current newSession.
	GlorpTestDescriptorSystem glorpAllSubclasses do: [:each |
		session system: (each forPlatform: (OraclePlatform new)).
		session system validate].
! !

!GlorpDescriptorValidationTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDescriptorValidationTest.st,v 1.1 2006/07/26 09:16:38 boris Exp $'
! !
