"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpMappingDBTest subclass:#GlorpDirectMappingDBTest
	instanceVariableNames:'person personId'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpDirectMappingDBTest class methodsFor:'As yet unclassified'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpDirectMappingDBTest methodsFor:'support'!

writeAddress
	| addressRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
!

writeHomefulPerson
	| personRow |
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.
!

writeHomelessPerson
	| personRow |
	personRow := session system examplePersonRow2.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.
! !

!GlorpDirectMappingDBTest methodsFor:'tests'!

readPerson
	| results query |
	query := Query
		returningManyOf: GlorpPerson
		where: [:pers | pers id = personId].
	results := query executeIn: session.
	self assert: results size = 1.
	person := results first.
!

testModify
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		session inUnitOfWorkDo: [
			session register: person.
			person name: 'something else'].
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = 'something else'].
!

testReadWithDirectFieldReference

	| peopleWithAddresses peopleWithoutAddresses addressIdField |
	[session beginTransaction.
	self writeAddress.
	self writeHomelessPerson.
	self writeHomefulPerson.
	addressIdField := (session system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID'.
	peopleWithAddresses := session readManyOf: GlorpPerson where: [:each | (each getField: addressIdField) notNIL].
	peopleWithoutAddresses := session readManyOf: GlorpPerson where: [:each | (each getField: addressIdField) isNIL].
	self assert: peopleWithAddresses size = 1.
	self assert: peopleWithoutAddresses size = 1.
	self assert: peopleWithAddresses first id = 3.
	self assert: peopleWithoutAddresses first id = 4.
	 ]
		ensure: [session rollbackTransaction].
!

testReadWithPseudoVariableMapping

	| peopleWithAddresses peopleWithoutAddresses |
	[session beginTransaction.
	self writeAddress.
	self writeHomelessPerson.
	self writeHomefulPerson.
	peopleWithAddresses := session readManyOf: GlorpPerson where: [:each | each addressId notNIL].
	peopleWithoutAddresses := session readManyOf: GlorpPerson where: [:each | each addressId isNIL].
	self assert: peopleWithAddresses size = 1.
	self assert: peopleWithoutAddresses size = 1.
	self assert: peopleWithAddresses first id = 3.
	self assert: peopleWithoutAddresses first id = 4.]
		ensure: [session rollbackTransaction].
! !

!GlorpDirectMappingDBTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpDirectMappingDBTest.st,v 1.1 2006/07/26 09:05:36 boris Exp $'
! !
