"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpDatabaseBasedTest subclass:#GlorpExpressionJoiningTest
	instanceVariableNames:'source target base'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpExpressionJoiningTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpExpressionJoiningTest methodsFor:'support'!

setUp
	super setUp.
	source := BaseExpression new.
	target := source get: #relation.
	base := BaseExpression new.
!

tearDown

	source := nil.
	target := nil.
	base := nil.
	system := nil.
! !

!GlorpExpressionJoiningTest methodsFor:'tests'!

join: exp
	^exp asExpressionJoiningSource: source toTarget: target
!

resultOfJoiningFieldFor: aTable toExpressionBuiltOn: anotherTable
	| exp table |
	base descriptor: (system descriptorFor: GlorpCustomer).
	exp := FieldExpression forField: (aTable fieldNamed: 'ID')
				basedOn: BaseExpression new.
	table := base getTable: anotherTable.

	^exp asExpressionJoiningSource: base toTarget: table.
!

testBase
	| result |
	result := self join: base.
	self assert: result == source.
!

testConstant
	| exp |
	exp := ConstantExpression for: 42.
	self assert: (self join: exp) == exp
!

testField
	| exp  result |
	exp := FieldExpression forField: (DatabaseField named: 'test' type: system platform int4)
				basedOn: base.
	result := self join: exp.
	self assert: result base == source.
	self assert: result field == exp field
!

testFieldBuiltOnDifferentTable
	| result custTable |
	custTable := system tableNamed: 'GR_CUSTOMER'.
	result := self
		resultOfJoiningFieldFor: custTable
		toExpressionBuiltOn: custTable.
	self assert: result base == (base getTable: custTable).
	self assert: result field == ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
!

testFieldBuiltOnSameTable
	| exp result base2 table custTable |
	custTable := system tableNamed: 'GR_CUSTOMER'.
	base2 := BaseExpression new.
	base2 descriptor: (system descriptorFor: GlorpCustomer).
	table := base2 getTable: custTable.
	exp := FieldExpression forField: (custTable fieldNamed: 'ID')
				basedOn: base.
	result := exp asExpressionJoiningSource: base2 toTarget: table.
	self assert: result base == table.
	self assert: result field == exp field.
!

testMapping
	| result exp |
	exp := base get: #foo.
	result := self join: exp.
	self assert: result base == source.
	self assert: result name = #foo.
!

testParameter
	| result exp table field |
	table := DatabaseTable named: 'T'.
	field := DatabaseField named: 'F' type: system platform int4.
	table addField: field.
	exp := base getParameter: field.
	result := self join: exp.
	self assert: result base == source.
	self assert: result class == FieldExpression.
	self assert: result field == field.
!

testRelation
	| result exp |
	exp := [:a | a foo = 3] asGlorpExpressionOn: base.
	result := self join: exp.

	self assert: result class == RelationExpression.
	self assert: result rightChild == exp rightChild.
	self assert: result leftChild base == source.
!

testRelation2
	| result exp field |
	field := DatabaseField named: 'fred' type: system platform int4.
	exp := [:a | a foo = field] asGlorpExpressionOn: base.
	result := self join: exp.
	self assert: result class == RelationExpression.
	self assert: result rightChild class == FieldExpression.
	self assert: result rightChild field == field.
	self assert: result leftChild base == source.
!

testSelfJoinWithPrimaryKeyExpression
	"This tests a join of a class to itself, in this case customers who have other customers associated with them. Useful for hierarchies"
	| pkExpression field result |
	field := (system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID'.
	pkExpression := Join
		from: field
		to: field.
	result := self join: pkExpression.
	self assert: result leftChild basicField == field.
	self assert: result rightChild basicField == field.
	self assert: result rightChild base name = #relation.
!

testTable
	| result exp table |
	table := DatabaseTable named: 'T'.
	exp := base getTable: table.
	result := self join: exp.
	self assert: result base == target.
	self assert: result table == table.
! !

!GlorpExpressionJoiningTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpExpressionJoiningTest.st,v 1.1 2006/07/26 09:19:47 boris Exp $'
! !
