"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpDatabaseBasedTest subclass:#GlorpExpressionTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpExpressionTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpExpressionTest methodsFor:'support'!

assertIdentityOf: aBlock and: anotherBlock

	| base |
	base := BaseExpression new.
	self assert: (aBlock asGlorpExpressionOn: base) == (anotherBlock asGlorpExpressionOn: base)
!

denyIdentityOf: aBlock and: anotherBlock

	| base |
	base := BaseExpression new.
	self deny: (aBlock asGlorpExpressionOn: base) == (anotherBlock asGlorpExpressionOn: base)
! !

!GlorpExpressionTest methodsFor:'tests'!

testAndOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) & (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #AND.
!

testAndOperation2
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) AND: (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #AND.
!

testAnySatisfyExists

	| expression base |
	system useJoinsForAnySatisfy: false.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	expression := [:a | a items anySatisfy: [:each | each id = 7]] asGlorpExpressionOn: base.
	self assert: expression class == SubSelectExpression.
!

testAnySatisfyJoin

	| expression base |
	system useJoinsForAnySatisfy: true.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	expression := [:a | a items anySatisfy: [:each | each id = 7]] asGlorpExpressionOn: base.
	self assert: expression class == CollectionExpression.
	self assert: expression leftChild == expression rightChild leftChild base.
!

testAnySatisfyPrintExists
	| expression |
	system useJoinsForAnySatisfy: false.
	expression := [:cust | cust transactions anySatisfy: [:each | each id = 7]]
				asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: expression sqlString = ' EXISTS (SELECT s1t1.ID
 FROM BANK_TRANS s1t1
 WHERE ((s1t1.ID = ?) AND (s1t2.ID = s1t1.OWNER_ID)))'
!

testAnySatisfyPrintJoin
	| expression |
	system useJoinsForAnySatisfy: true.
	expression := [:cust | cust transactions anySatisfy: [:each | each id = 7]]
				asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: expression sqlString = '(BANK_TRANS.ID = 7)'
!

testBetweenAnd
	| expression base |
	base := BaseExpression new.
	expression := [:a | a between: 3 and: 4] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #AND.
	self assert: expression leftChild relation == #>=.
	self assert: expression leftChild rightChild value == 3.
	self assert: expression rightChild relation == #<=.
	self assert: expression rightChild rightChild value == 4.
!

testEqualityOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | a firstName = fred] asGlorpExpressionOn: base.
	self assert: expression leftChild == (base get: #firstName).
	self assert: expression rightChild class == ConstantExpression.
	self assert: expression rightChild value == fred.
	self assert: expression relation == #=
!

testFindingMapping
	| base baseDescriptor |
	baseDescriptor := system descriptorFor: GlorpBankTransaction.
	base := BaseExpression new descriptor: baseDescriptor.
	self assert: (base get: #serviceCharge) mapping == (baseDescriptor mappingForAttributeNamed: #serviceCharge).
	self assert: (base get: #serviceCharge) sourceDescriptor == (system descriptorFor: GlorpBankTransaction).
	self assert: (base get: #serviceCharge) descriptor == (system descriptorFor: GlorpServiceCharge)
!

testGetField
	| expression |
	expression := [:cust | cust getField: 'ID'] asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: expression sqlString = 'GR_CUSTOMER.ID'.
	self assert: expression class = FieldExpression.
	self assert: expression field table = (system tableNamed: 'GR_CUSTOMER').
!

testGetFieldAmbiguous
	| |
	self
		should: [[:pass | pass getField: 'ID']
			asGlorpExpressionForDescriptor: (system descriptorFor: GlorpPassenger)]
		raise: Error.
!

testGetFieldFromSecondaryTable
	| expression |
	expression := [:pass | pass getField: 'POINTS']
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpPassenger).
	self assert: expression sqlString = 'FREQUENT_FLYER.POINTS'.
	self assert: expression class = FieldExpression.
	self assert: expression field table = (system tableNamed: 'FREQUENT_FLYER').
!

testIsNullPrint

	| expression stream |
	expression := [:cust | cust id = nil]
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	stream := DatabaseCommand new platform: system platform.
	expression printSQLOn: stream withParameters: Dictionary new.
	self assert: stream contents = '(GR_CUSTOMER.ID IS NULL)'.
!

testJoinOperation
	| userExpression base expression addressTable personTable query field1 field2 join |
	addressTable := system tableNamed: 'GR_ADDRESS'.
	personTable := system tableNamed: 'PERSON'.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpPerson).
	userExpression := [:aPerson | aPerson address number = 12]
				asGlorpExpressionOn: base.
	query := SimpleQuery returningOneOf: GlorpPerson where: userExpression.
	query session: (GlorpSession new system: system).
	self assert: (userExpression additionalExpressionsIn: query) size = 1.
	query prepare.
	expression := query whereClause.
	self assert: query joins size = 1.
	join := query joins first.
	self
		assert: expression == userExpression;
		assert: expression relation == #=.
	field1 := join leftChild field.
	self assert: field1 table parent == personTable.
	self assert: field1 name = 'ADDRESS_ID'.
	field2 := join rightChild field.
	self assert: field2 table parent == addressTable.
	self assert: field2 name = 'ID'.
!

testMappingBase

	| base |
	base := BaseExpression new.
	self assert: (base get: #someAttribute) base == base
!

testMappingExpressionIdentity

	self assertIdentityOf: [:a | a someAttribute] and: [:a | a someAttribute].
!

testMappingExpressionIdentity2

	self assertIdentityOf: [:a | a perform: #someAttribute] and: [:a | a someAttribute].
!

testMappingExpressionIdentity3

	self assertIdentityOf: [:a | a get: #someAttribute] and: [:a | a someAttribute].
!

testMappingExpressionIdentity4

	self denyIdentityOf: [:a | a get: #someAttribute] and: [:a | a someOtherAttribute].
!

testNotNullPrint

	| expression stream |
	expression := [:cust | cust id ~= nil]
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	stream := DatabaseCommand new platform: system platform.
	expression printSQLOn: stream withParameters: Dictionary new.
	self assert: stream contents = '(GR_CUSTOMER.ID IS NOT NULL)'.
!

testOrOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) | (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #OR.
!

testOrOperation2
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) OR: (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #OR.
!

testTwoLevelMappingExpressionIdentity

	self assertIdentityOf: [:a | a someAttribute someAttribute] and: [:a | a someAttribute someAttribute].
	self denyIdentityOf: [:a | a someAttribute someAttribute] and: [:a | a someAttribute].
! !

!GlorpExpressionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpExpressionTest.st,v 1.1 2006/07/26 09:12:16 boris Exp $'
! !
