"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpHorizontalInheritanceTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpHorizontalInheritanceTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpHorizontalInheritanceTest class methodsFor:'resources'!

resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.
! !

!GlorpHorizontalInheritanceTest methodsFor:'support'!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	session system: (GlorpInheritanceDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
!

tearDown
	super tearDown.
	session reset.
!

writeTestHarness

	session register: (GlorpPerishableItem new id: 1; name: 'squash'; age: 10; yourself).
	session register: (GlorpPerishableItem new id: 2; name: 'zucchini'; age: 14; yourself).
	session register: (GlorpPerishableItem new id: 3; name: 'apples'; age: 4; yourself).
	session register: (GlorpNonperishableItem new id: 4; name: 'TV'; serialNumber: 56893; yourself).
	session register: (GlorpNonperishableItem new id: 5; name: 'Fridge'; serialNumber: 12345; yourself).
	session register: (GlorpUnassembledItem new id: 6; name: 'Bicycle'; serialNumber: 83754; assemblyCost: 100; yourself).
	session register: (GlorpUnassembledItem new id: 7; name: 'Wagon'; serialNumber: 99958; assemblyCost: 20; yourself).
! !

!GlorpHorizontalInheritanceTest methodsFor:'tests'!

testClassSelection

	| classes |
	classes := (session system descriptorFor: GlorpInventoryItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 3.
	#(#GlorpNonperishableItem #GlorpPerishableItem #GlorpUnassembledItem) do: [:name |
		self assert: (classes includes: name) ].

	classes := (session system descriptorFor: GlorpPerishableItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 1.
	#(#GlorpPerishableItem) do: [:name |
		self assert: (classes includes: name) ].

	classes := (session system descriptorFor: GlorpNonperishableItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 2.
	#(#GlorpNonperishableItem #GlorpUnassembledItem) do: [:name |
		self assert: (classes includes: name)].
!

testDirectQuery

	| items query |
	session beginTransaction.

	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	session initializeCache.
	query := Query readManyOf: GlorpInventoryItem
				where: [:each | each name = 'TV'].
	items := session execute: query.
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpNonperishableItem]) size = 1.

	session initializeCache.
	items := session readManyOf: GlorpInventoryItem
				where: [:each | each name = 'Bicycle'].
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpUnassembledItem]) size = 1.

	session initializeCache.
	items := session readManyOf: GlorpPerishableItem
				where: [:each | each name = 'Bicycle'].
	self assert: items size = 0.
	]
			ensure: [session rollbackTransaction]
!

testOrderBy
	"We can't use database-level ordering in horizontal inheritance because it does multiple queries. We could, I suppose, sort after the fact, but we don't right now"

	| items query errored |
	session beginTransaction.

	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	query := Query returningManyOf: GlorpInventoryItem
				where: [:each | each id <= 4].
	query orderBy: #name.
	errored := false.
	self should: [items := session execute: query] raise: Dialect error.
	]
			ensure: [session rollbackTransaction]
! !

!GlorpHorizontalInheritanceTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpHorizontalInheritanceTest.st,v 1.1 2006/07/26 09:01:22 boris Exp $'
! !
