"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpInsertUpdateTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpInsertUpdateTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpInsertUpdateTest class methodsFor:'resources'!

resources

	^Array with: GlorpSessionResource.
! !

!GlorpInsertUpdateTest methodsFor:'support'!

rowFor: anObject

	| rowMap rows |
	rowMap := RowMap new.
	session createRowsFor: anObject in: rowMap.
	rows := rowMap rowsForKey: anObject.
	self assert: rows size = 1.
	^rows first.
!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
! !

!GlorpInsertUpdateTest methodsFor:'tests'!

testFunctionalInsertUpdateForInsert

	| testObject |
	[session beginTransaction.
	session beginUnitOfWork.
	testObject := GlorpCustomer example1.
	testObject id: 876.
	session register: testObject.
	session commitUnitOfWork.
	self assert: (testObject seenPreWrite = true).
	self assert: (testObject seenPostWrite = true).

	session beginUnitOfWork.
	session register: testObject.
	testObject name: 'Change of name'.
	session commitUnitOfWork]
		ensure: [session rollbackTransaction]
!

testRowOwnership

	| aCustomer rowMap |
	aCustomer := GlorpCustomer new.
	rowMap := RowMap new.
	(session descriptorFor: GlorpCustomer) createRowsFor: aCustomer in: rowMap.
	rowMap rowsDo: [:each |
		self assert: each owner = aCustomer].
!

testShouldInsertForInsert

	| testObject row |
	testObject := GlorpCustomer example1.
	testObject id: 876.
	row :=self rowFor: testObject.
	self assert: (session shouldInsert: row).
!

testShouldInsertForUpdate

	| testObject row |
	session beginUnitOfWork.
	testObject := GlorpCustomer example1.
	testObject id: 876.
	session cacheAt: 876 put: testObject.
	row :=self rowFor: testObject.
	self deny: (session shouldInsert: row).
! !

!GlorpInsertUpdateTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpInsertUpdateTest.st,v 1.1 2006/07/26 09:06:09 boris Exp $'
! !
