"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestDescriptorSystem subclass:#GlorpLockingDescriptorSystem
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTestModels'
!


!GlorpLockingDescriptorSystem methodsFor:'descriptors'!

descriptorForGlorpBook: aDescriptor

	| table |
	table := self tableNamed: 'BOOK'.
	aDescriptor table: table.
	aDescriptor addMapping: (	
		DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (	
		DirectMapping from: #title to: (table fieldNamed: 'TITLE')).
	aDescriptor addMapping: (	
		DirectMapping from: #copiesInStock to: (table fieldNamed: 'COPIES_IN_STOCK')).
	aDescriptor addMapping: (	
		DirectMapping from: #version to: (table fieldNamed: 'VERSION')).
!

descriptorForGlorpBookstoreCustomer: aDescriptor

	| table |
	table := self tableNamed: 'BOOK_CUSTOMER'.
	aDescriptor table: table.
	aDescriptor addMapping: (	
		DirectMapping from: #accountNumber to: (table fieldNamed: 'ACCOUNT_NUMBER')).
	aDescriptor addMapping: (	
		DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (	
		ManyToManyMapping new
			attributeName: #booksOnOrder;
			referenceClass: GlorpBook).
! !

!GlorpLockingDescriptorSystem methodsFor:'examples'!

bookExample1
	^(GlorpBook new)
		title: 'Mastering ENVY/Developer';
		author: 'Knight/Pelrine/Cho';
		description:
				'Absolutely the best book written on ENVY/Developer by a set of authors all living on different continents';
		copiesInStock: 2000;
		version: 1.
!

bookExample2
	^(GlorpBook new)
		title: 'The Bad Beginning';
		author: 'Lemony Snicket';
		description: 'Very fine description of the EJB standards process';
		copiesInStock: 3;
		version: 5.
!

customerExample1
	^(GlorpBookstoreCustomer new)
		accountNumber: '123456789ABCDEF';
		name: 'Aunt Colof'.
!

customerExample2
	^(GlorpBookstoreCustomer new)
		accountNumber: '7';
		name: 'Veronica Filbert Davidson'.
!

example1
	"Return an example start set of books, customers and orders."

	| cust1 cust2 book1 book2 |
	cust1 := self customerExample1.
	cust2 := self customerExample2.
	book1 := self bookExample1.
	book2 := self bookExample2.
	^Array
		with: book1
		with: book2
		with: cust1
		with: cust2.
! !

!GlorpLockingDescriptorSystem methodsFor:'other'!

allTableNames

	^#('BOOK' 'BOOK_CUSTOMER' 'CUSTOMER_BOOK_LINK').
!

constructAllClasses
	^(super constructAllClasses) 
		add: GlorpBook;
		add: GlorpBookstoreCustomer;
		yourself.
! !

!GlorpLockingDescriptorSystem methodsFor:'tables'!

tableForBOOK: aTable

	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 100).
	aTable createFieldNamed: 'DESCRIPTION' type: (platform varChar: 255).
	aTable createFieldNamed: 'COPIES_IN_STOCK' type: (platform int4).
	(aTable createFieldNamed: 'VERSION' type: (platform versionFieldFor: platform int4)) beLockKey.
!

tableForBOOK_CUSTOMER: aTable

	(aTable createFieldNamed: 'ACCOUNT_NUMBER' type: (platform varChar: 15)) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 100).
!

tableForCUSTOMER_BOOK_LINK: aTable

	| custKey bookKey |
	custKey := aTable createFieldNamed: 'CUSTOMER_ACCT_NUM' type: (platform varChar: 15).
	aTable addForeignKeyFrom: custKey
		to: ((self tableNamed: 'BOOK_CUSTOMER') fieldNamed: 'ACCOUNT_NUMBER').
	bookKey := aTable createFieldNamed: 'BOOK_ID' type: (platform int4).
		aTable addForeignKeyFrom: bookKey
		to: ((self tableNamed: 'BOOK') fieldNamed: 'ID').
! !

!GlorpLockingDescriptorSystem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpLockingDescriptorSystem.st,v 1.1 2006/07/26 09:12:28 boris Exp $'
! !
