"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpMappingDBTest subclass:#GlorpManyToManyDBTest
	instanceVariableNames:'customer customerId accountId1 accountId2 accountId3'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpManyToManyDBTest class methodsFor:'As yet unclassified'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpManyToManyDBTest methodsFor:'support'!

checkAccounts

	| sorted numberOfAccounts |
	numberOfAccounts := (accountId1 isNil ifTrue: [0] ifFalse: [1]) + (accountId2 isNil ifTrue: [0] ifFalse: [1]) + (accountId3 isNil ifTrue: [0] ifFalse: [1]).
	self assert: customer accounts size = numberOfAccounts.
	sorted := customer accounts asSortedCollection: [:a :b | a id <= b id].
	accountId1 isNil ifFalse: [self assert: sorted first id = accountId1].
	accountId2 isNil ifFalse: [self assert: (sorted at: 2) id = accountId2].
	accountId3 isNil ifFalse: [self assert: sorted last id = accountId3].
	self assert: (customer accounts collect: [:each | each id]) asSet size = customer accounts size.
!

inUnitOfWorkDo: aBlock initializeWith: initBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	initBlock value.
	session beginUnitOfWork.
	self readCustomer.
	aBlock value.
	session commitUnitOfWork.
	session reset.
!

readCustomer
	| results query |
	query := Query
		returningManyOf: GlorpCustomer
		where: [:cust | cust id = customerId].
	results := query executeIn: session.
	self assert: results size = 1.
	customer := results first.
!

writeCustomerWithAccounts
	| customerRow accountRow1 accountRow2 linkRow1 linkRow2 customerRow2 accountRow3 linkRow3 |

	customerRow := session system exampleCustomerRow1.
	customerId := customerRow atFieldNamed: 'ID'.
	customerRow2 := session system exampleCustomerRow2.
	accountRow1 := session system exampleAccountRow1.
	accountId2 := accountRow1 atFieldNamed: 'ID'.
	accountRow2 := session system exampleAccountRow2.
	accountId1 := accountRow2 atFieldNamed: 'ID'.
	accountRow3 := session system exampleAccountRow3.
	linkRow1 := session system exampleCALinkRow1.
	linkRow2 := session system exampleCALinkRow2.
	linkRow3 := session system exampleCALinkRow3.
	session writeRow: customerRow.
	session writeRow: customerRow2.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: accountRow3.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	session writeRow: linkRow3.
! !

!GlorpManyToManyDBTest methodsFor:'tests-join'!

testReadCustomerOrderByLinkTableField

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session readManyOf: GlorpCustomer.
			customers do: [:each | | sortedAccounts |
				sortedAccounts := each accounts asSortedCollection: [:a :b | a id <= b id].
				self assert: each accountsSortedById asArray = sortedAccounts asArray]].
!

testReadCustomerOrderByLinkTableFieldDescending

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session readManyOf: GlorpCustomer.
			customers do: [:each | | sortedAccounts |
				sortedAccounts := each accounts asSortedCollection: [:a :b | a id <= b id].
				self assert: each accountsSortedByIdDescending asArray = sortedAccounts asArray reversed]].
!

testReadCustomerWithJoinToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount accountNumber bankCode = '2']].
			self assert: customers size = 2].
!

testReadCustomerWithJoinToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount accountNumber branchNumber = 3]].
			self assert: customers size = 1].
!

testReadCustomerWithSimpleJoinToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount id <> 12]].
			self assert: customers size = 2].
! !

!GlorpManyToManyDBTest methodsFor:'tests-nodes'!

testReadNode
	| allNodes |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		allNodes := session readManyOf: GlorpTreeNode orderBy: #id.
		self assert: (allNodes collect: [:each | each id]) asArray = #( 1 2 3).
		self assert: allNodes first children size = 2.
		self assert: (allNodes first children collect: [:each | each id]) asSortedCollection asArray = #( 2 3)].
!

testReadNodeAndDeleteChild
	| node1 node3 links nodes |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		node3 := (node1 children asSortedCollection: [:a :b | a id <= b id]) last.
		session transact: [
			session register: node1.
			session delete: node3.
			node1 children remove: node3].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 1.
		self assert: links first first = 1.
		self assert: links first last = 2.
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self deny: (nodes anySatisfy: [:each | each first = node3 id]).
		self assert: nodes size = 2
		].
!

testReadNodeAndDeleteChildWithSaveAndContinue
	"This exercises two things. First, there's a bug in save and continue with deleted objects resurrecting. Second, make sure that if we delete both parent and child, the link goes away"
	| node1 node3 links nodes node2 |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		node2 := (node1 children asSortedCollection: [:a :b | a id <= b id]) first.
		node3 := (node1 children asSortedCollection: [:a :b | a id <= b id]) last.
		session beginUnitOfWork.
			[session register: node1.
			session delete: node3.
			node1 children remove: node3] ensure: [session saveAndContinue].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 1.
		self assert: links first first = 1.
		self assert: links first last = 2.
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self deny: (nodes anySatisfy: [:each | each first = node3 id]).
		self assert: nodes size = 2.

			[session register: node1.
			session delete: node2.
			session delete: node1] ensure: [session saveAndContinue].
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self assert: nodes isEmpty.
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links isEmpty.
		].
!

testReadNodeAndDeleteParent
	| node1 links |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		session transact: [
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		session delete: node1.
		node1 children: #()].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 0].
!

writeNodeWithChildren
	| node1 node2 node2a |
	node1 := GlorpTreeNode new id: 1.
	node2 := GlorpTreeNode new id: 2.
	node2a := GlorpTreeNode new id: 3.
	node1 children add: node2.
	node1 children add: node2a.
	session inUnitOfWorkDo: [session register: node1].
! !

!GlorpManyToManyDBTest methodsFor:'tests-read'!

checkNumberOfAccounts: anInteger
	| accountRows |
	accountRows := session accessor
		executeSQLString: 'SELECT * FROM BANK_ACCT'.
	self assert: accountRows size = anInteger.
!

checkNumberOfLinkRows: anInteger
	| linkRows |
	linkRows := session accessor
		executeSQLString: 'SELECT * FROM CUSTOMER_ACCT_LINK'.
	self assert: linkRows size = anInteger.
!

testReadCustomerAndAddAccount
	self
		inTransactionDo:
			[
			self
				inUnitOfWorkDo: [customer addAccount: (GlorpBankAccount new id: 77473)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId3 := 77473.
			self checkAccounts.
			self checkNumberOfLinkRows: 4].
!

testReadCustomerAndDeleteAccount
	"Note that delete, without also changing the relationships in memory, leaves a dangling link row. Some databases will fail the operation as a result. The VW oracle connect seems to have a problem here that this will sometimes leave subsequent tests looking at invalid data. The transaction appears to rollback, and the database appears fine, but somehow I'm getting erratic constraint failures. Ignoring for the moment, and just commenting out this test"
	self
		inTransactionDo:[
			[self
				inUnitOfWorkDo: [ | account|
					account := customer accounts detect: [:each | each id = 9874].
					session delete: account]
				initializeWith: [self writeCustomerWithAccounts]]
				on: Dialect error
				do: [:ex | Transcript show: 'integrity violation'; cr.^self].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 3.
			self checkNumberOfAccounts: 2].
!

testReadCustomerAndDeleteAccountProperly
	"Do both the delete and the patching up of relationships"
	self
		inTransactionDo:
			[
			self
				inUnitOfWorkDo: [ | account|
					account := customer accounts detect: [:each | each id = 9874].
					session delete: account.
					customer removeAccount: account]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 2].
!

testReadCustomerAndRemoveAccount
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := customer accounts detect: [:each | each id = 9874].
					customer accounts remove: account]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2].
!

testReadCustomerAndReplaceAccounts
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := GlorpBankAccount new id: 99999.
					customer accounts do: [:each | each accountHolders remove: customer].
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := 99999.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 4].
!

testReadCustomerAndReplaceAccountsWithRemoval
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := customer accounts detect: [:each | each id = 6].
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2].
!

testReadCustomerAndReplaceAccountsWithoutInstantiatingHolders
	"This works, but only fortuitously. If the accounts haven't been read into memory, we don't have to remove their object-level references to the account holder, because changing one side of the relationship in memory is enough to cause the link rows to be deleted"
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := GlorpBankAccount new id: 99999.
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := 99999.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 4].
!

testReadCustomerAndReplaceInstantiatedAccountsWithEmpty
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[customer accounts yourSelf.
					customer accounts: #()]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 1.
			self checkNumberOfAccounts: 3].
!

testReadCustomerAndReplaceUninstantiatedAccountsWithEmpty
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[customer accounts: #()]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 1.
			self checkNumberOfAccounts: 3].
! !

!GlorpManyToManyDBTest methodsFor:'tests-subselect'!

testReadCustomerWithAverageSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each average: [:eachCust | eachCust accounts id]) = ((9874 + 6) / 2)].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each accounts id sqlAverage) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].
!

testReadCustomerWithCountSubselect

	| customersWithOneAccount customersWithTwoAccounts customersWithLessThanFiveAccounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customersWithOneAccount := session
				read: GlorpCustomer
				where:
					[:each |
					(each getConstant: 1) = (each count: [:eachCust | eachCust accounts])].
			customersWithTwoAccounts := session
				read: GlorpCustomer
				where:
					[:each |
					(each count: [:eachCust | eachCust accounts]) = 2].
			"fully general syntax. doesn't require any helper methods, just the existence of the function"
			customersWithLessThanFiveAccounts := session
				read: GlorpCustomer
				where: [:each | (each aggregate: each accounts as: #countStar) < 5].
		self assert: customersWithOneAccount size = 1.
		self assert: customersWithOneAccount first id = 28.
		self assert: customersWithTwoAccounts size = 1.
		self assert: customersWithTwoAccounts first id = 27.
		self assert: customersWithLessThanFiveAccounts size = 2].
!

testReadCustomerWithCountSubselectAlternateSyntax

	| customersWithOneAccount customersWithTwoAccounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customersWithOneAccount := session
				read: GlorpCustomer
				where:
					[:each |
					(each getConstant: 1) = (each accounts sqlCount)].
			customersWithTwoAccounts := session
				read: GlorpCustomer
				where:
					[:each |
					each accounts sqlCount = 2].
		self assert: customersWithOneAccount size = 1.
		self assert: customersWithOneAccount first id = 28.
		self assert: customersWithTwoAccounts size = 1.
		self assert: customersWithTwoAccounts first id = 27].
!

testReadCustomerWithEqualityComparisonToAccountsUsingInClause
	"Test doing a primary key rewrite of the where clause in a subselect, to the result of another subselect"

	| customers accounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			accounts := session read: GlorpBankAccount where: [:each | each id = 22].
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount in: accounts]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]]].
!

testReadCustomerWithEqualitySubSelectToAccountsUsingInClause
	"Test doing a primary key rewrite of the where clause in a subselect, to the result of another subselect"

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount in: (Query read: GlorpBankAccount where: [:accts2 | accts2 id <> 12])]].
			self assert: customers size = 2.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount |
							eachAccount in: (Query read: GlorpBankAccount where: [:accts2 | accts2 id = 22])]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]]].
!

testReadCustomerWithFilteredSubSelectToAccounts1

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					(eachCustomer accounts select: [:eachAccount | eachAccount accountNumber bankCode = '2']) sqlCount > 0].
			self assert: customers size = 2].
!

testReadCustomerWithFilteredSubSelectToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					(eachCustomer accounts select: [:eachAccount | eachAccount accountNumber bankCode = '2']) sqlCount > 1].
			self assert: customers size = 0].
!

testReadCustomerWithFilteredSubSelectToAccounts3

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					(eachCustomer accounts select: [:eachAccount | eachAccount id > 5 ]) sqlCount <= 1].
			self assert: customers size = 1].
!

testReadCustomerWithMaxSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each max: [:eachCust | eachCust accounts id]) = 9874].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each accounts id sqlMax) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].
!

testReadCustomerWithMinSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each min: [:eachCust | eachCust accounts id]) = 6].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each accounts id sqlMin) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].
!

testReadCustomerWithSimpleSubSelectToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount id <> 12]].
			self assert: customers size = 2].
!

testReadCustomerWithSubSelectToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount accountNumber bankCode = '2']].
			self assert: customers size = 2].
!

testReadCustomerWithSubSelectToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount accountNumber branchNumber = 3]].
			self assert: customers size = 1].
!

testReadCustomerWithSubSelectToAccountsUsingInClause
	"Test doing an IN clause with the result of another subselect"

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount |
							| q |
							q := Query read: GlorpBankAccount where: [:accts2 | accts2 id <> 12].
							q retrieve: #id.
							eachAccount id in: q]].
			self assert: customers size = 2.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer |
					eachCustomer accounts
						anySatisfyExists: [:eachAccount |
							| q |
							q := Query read: GlorpBankAccount where: [:accts2 | accts2 id = 22].
							q retrieve: #id.
							eachAccount id in: q]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]].
].
!

testReadCustomerWithSumSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each sum: [:eachCust | eachCust accounts accountNumber branchNumber]) < 6].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each |
					(each accounts accountNumber branchNumber sqlSum) > 6].
		self assert: customers size = 1.
		self assert: customers first id = 28.].
! !

!GlorpManyToManyDBTest methodsFor:'tests-write'!

testWriteCustomerWithAccounts
	| newCustomer |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			newCustomer accounts: (OrderedCollection with: (GlorpBankAccount new id: 223)).
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := 223.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfAccounts: 4.
			self checkNumberOfLinkRows: 4].
!

testWriteCustomerWithNoAccounts
	| newCustomer |
	self
		inTransactionDo:
			[session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfAccounts: 0.
			self checkNumberOfLinkRows: 0].
!

testWriteCustomerWithTwoAccounts
	| newCustomer |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			newCustomer accounts: (OrderedCollection with: (GlorpBankAccount new id: 223)).
			newCustomer accounts add: (GlorpBankAccount new id: 224).
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := 223.
			accountId2 := 224.
			self checkAccounts.
			self checkNumberOfAccounts: 5.
			self checkNumberOfLinkRows: 5].
! !

!GlorpManyToManyDBTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpManyToManyDBTest.st,v 1.1 2006/07/26 09:13:32 boris Exp $'
! !
