"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpObjectComparisonJoinsTest
	instanceVariableNames:'itinerary1 itinerary2'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpObjectComparisonJoinsTest methodsFor:'support'!

setUp
	super setUp.
	session system useJoinsForAnySatisfy: true.
	session beginTransaction.
	itinerary1 := GlorpItinerary example1.
	itinerary2 := GlorpItinerary example2.
	itinerary2 reservation passenger: nil.
	itinerary2 reservation passengers: #().

	session transact: [
		session register: itinerary1.
		session register: itinerary2].
!

tearDown

	session rollbackTransaction.
	itinerary1 := nil.
	itinerary2 := nil.
	super tearDown.
! !

!GlorpObjectComparisonJoinsTest methodsFor:'tests'!

testBasicEqualNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passenger isNIL].
	self assert: reservations size = 1.
	self assert: reservations first id = 7.
!

testBasicNotNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passenger notNIL].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
!

testEqualNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger isNIL]].
	self assert: reservations size = 1.
	self assert: reservations first id = 7.
!

testEqualToIdsInExpression
	"Just to verify that this works when not directly comparing objects"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger id = each passenger id]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
!

testEqualToObjectInExpression
	"A somewhat contrived example of comparing two relationships that should end up with the same object, because they're actually mapped the same way. But will fail if the passenger is nil"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger = each passenger]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
!

testEqualToObjectInMemory
	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger = itinerary1 reservation passenger]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
!

testNotEqualToObjectInExpression
	"A somewhat contrived example of comparing two relationships that should end up with the same object, because they're actually mapped the same way. But will fail if the passenger is nil, because nil ~= nil"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger ~= each passenger]].
	self assert: reservations isEmpty.
!

testNotEqualToObjectInMemory
	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger ~= (GlorpPassenger new id: 7)]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
!

testNotNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger notNIL]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.
! !

!GlorpObjectComparisonJoinsTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpObjectComparisonJoinsTest.st,v 1.1 2006/07/26 08:58:36 boris Exp $'
! !
