"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpObjectMappedToImaginaryTableTest
	instanceVariableNames:'example example2'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpObjectMappedToImaginaryTableTest methodsFor:'support'!

setUp
	super setUp.
	system := GlorpVideoDescriptorSystem forPlatform: session platform.
	session system: system.
	session beginTransaction.
	example := system exampleStore1.
	example2 := system exampleStore2.
	session inUnitOfWorkDo: [session register: example. session register: example2.].
	session reset.
!

tearDown
	session rollbackTransaction.
	super tearDown.
! !

!GlorpObjectMappedToImaginaryTableTest methodsFor:'tests'!

queryForComplexSubselectRead2: anInteger
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table. The gist of the query is stores where more than anInteger different customers have made rentals"

	| q1 q2 q3 |
	q1 := Query
		readManyOf: GlorpVideoStore
		where: [:store |
			q2 := Query
				count: GlorpVideoCustomer
				where: [:customer |
					q3 := Query
						readManyOf: GlorpVideoRental
						where: [:rental | rental customer = customer AND: (rental store = store)].
					q3 retrieve: [:trade | trade customerName].
					customer name in: q3].
			(store getConstant: anInteger) <= q2].
	^q1.
!

queryForComplexSubselectRead: anInteger
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table. The gist of the query is stores where more than anInteger different customers have made rentals"

	| q1 q2 q3 |
	q1 := Query
		readManyOf: GlorpVideoStore
		where: [:store |
			q2 := Query
				count: GlorpVideoCustomer
				where: [:customer |
					q3 := Query
						readManyOf: GlorpVideoRental
						where: [:rental | rental store = store].
					q3 retrieve: [:trade | trade customerName].
					customer name in: q3].
			(store getConstant: anInteger) <= q2].
	^q1.
!

testComplexSubselectRead
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table"

	| |
	self assert: (session execute: (self queryForComplexSubselectRead: 3)) size = 1.
	self assert: (session execute: (self queryForComplexSubselectRead: 4)) size = 0.
!

testComplexSubselectRead2
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table"

	| |
	self assert: (session execute: (self queryForComplexSubselectRead2: 1)) size = 1.
	self assert: (session execute: (self queryForComplexSubselectRead2: 0)) size = 2.
!

testReadBackAllCustomer
	| customers |
	customers := session readManyOf: GlorpVideoCustomer.
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian' 'Bob' 'Carol' 'David' 'Eleanor' 'Francois' 'George' 'Hieronymous').
!

testReadBackEntireExample
	| store |
	store := session readOneOf: GlorpVideoStore where: [:each | each id = 1].
	example assertEqual: store.
!

testReadCustomerWhere
	| customers |
	customers := session readManyOf: GlorpVideoCustomer where: [:each | each name like: '%an%'].
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian'  'Eleanor' 'Francois').
!

testReadCustomerWhereWithJoin
	| customers |
	customers := session readManyOf: GlorpVideoCustomer where: [:each | each onlinePurchases anySatisfy: [:eachPurchase | eachPurchase price > 2]].
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian'  'George' ).
!

testWrite
	"Write and validate that it basically got written correctly, in terms of number and keys of db rows"

	| storeRows storeRow purchaseRows purchaseKeys rentalRows rentalKeys |
	storeRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_STORE'.
	storeRows := storeRows asSortedCollection: [:a :b | a first <= b first].
	self assert: storeRows size = 2.
	storeRow := storeRows first.
	self assert: storeRow size = 2.
	self assert: storeRow first = 1.
	self assert: storeRow last = 'Als Video'.
	storeRow := storeRows last.
	self assert: storeRow first = 2.
	self assert: storeRow last = 'Bonzai Video'.
	purchaseRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_PURCHASE'.
	self assert: purchaseRows size = 10.
	purchaseKeys := (purchaseRows collect: [:each | each atIndex: 1])
		asSortedCollection.
	self assert: purchaseKeys asArray = #(1 2 3 101 102 103 104 105 106 107).
	rentalRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_RENTAL'.
	self assert: rentalRows size = 4.
	rentalKeys := (rentalRows collect: [:each | each atIndex: 1])
		asSortedCollection.
	self assert: rentalKeys asArray = #(1001 1002 1003 1004).
! !

!GlorpObjectMappedToImaginaryTableTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpObjectMappedToImaginaryTableTest.st,v 1.1 2006/07/26 08:57:46 boris Exp $'
! !
