"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpObjectTransactionTest
	instanceVariableNames:'transaction objects'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpObjectTransactionTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpObjectTransactionTest methodsFor:'support'!

setUp

	transaction := ObjectTransaction new
! !

!GlorpObjectTransactionTest methodsFor:'tests'!

testArray
	| object |
	object := #(1 2 3 4 5) copy.
	transaction begin.
	transaction register: object.
	object
		at: 1 put: #one;
		at: 2 put: object.
	transaction abort.
	self
		assert: (object at: 1) == 1;
		assert: (object at: 2) == 2
!

testBecome
        | object |
        object := 'hello' copy.
        transaction begin.
        transaction register: object.
        object become: "Set new" Array new.
        transaction abort.
        self
                assert: object class == '' class;
                assert: object = 'hello'.
!

testCommit
	| array |
	array := #(1 2 3 4 5) copy.
	transaction begin.
	transaction register: array.
	array
		at: 1 put: #one;
		at: 2 put: array.
	transaction commit.
	self
		assert: (array at: 1) == #one;
		assert: (array at: 2) == array
!

testHashedCollection
	| object originalMembers |
	object := Set new.
	originalMembers := #(#one #two #three 'four' 5 'vi' (1 2 3 4 5 6 7)) collect: [:each | each copy].
	object addAll: originalMembers.
	transaction begin.
	transaction register: object.
	object remove: #one; remove: (originalMembers at: 4).
	object add: 1.
	originalMembers last at: 7 put: 'seven'.
	transaction abort.
	self
		assert: object size = originalMembers size;
		assert: (object includes: originalMembers first);
		assert: (object includes: (originalMembers at: 4));
		assert: object size = (object rehash; size).
	originalMembers do: [:each | self assert: (object includes: each)]
!

testRegisteredObjectsDoRecursively

	| link |
	link := 1 -> (2 -> (3 -> (4 ->5))).
	transaction register: link.
	self assert: transaction undoMap size = 1.
	transaction registeredObjectsDo: [:each |
		transaction register: each value].
	self assert: transaction undoMap size = 4.
!

testString
	| object |
	object := 'Hello, World!!' copy.
	transaction begin.
	transaction register: object.
	object
		at: 1 put: $h;
		at: 2 put: $E.
	transaction abort.
	self
		assert: object first  == $H;
		assert: (object at: 2) == $e
! !

!GlorpObjectTransactionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpObjectTransactionTest.st,v 1.1 2006/07/26 09:07:43 boris Exp $'
! !
