"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpMappingDBTest subclass:#GlorpOneToOneDBTest
	instanceVariableNames:'person personId'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpOneToOneDBTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpOneToOneDBTest methodsFor:'support'!

checkAddress
	self assert: person address isGlorpProxy.
	self assert: person address getValue id = 123.
	^self assert: person address getValue class == GlorpAddress.
!

checkNoAddress
	self assert: person address isGlorpProxy.
	self assert: person address getValue == nil.
!

checkNoAddressesInDB
	| addresses addressKeys |
	addresses := session accessor executeSQLString: 'SELECT * FROM GR_ADDRESS'.
	self assert: addresses isEmpty.
	addressKeys := session accessor executeSQLString: 'SELECT ADDRESS_ID FROM PERSON'.
	self assert: (addressKeys size = 1).
	self assert: (addressKeys first atIndex: 1) = nil.
!

checkPerson
	self assert: person class = GlorpPerson.
	self assert: person id = personId.
	self assert: person name = 'aPerson'.
!

inUnitOfWorkDo: aBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	self inUnitOfWorkDo: aBlock initializeWith: [self writePersonWithAddress].
!

inUnitOfWorkDo: aBlock initializeWith: initBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	initBlock value.
	session beginUnitOfWork.
	self readPerson.
	aBlock value.
	session commitUnitOfWork.
	session reset.
!

readPerson
	| results query |
	query := Query
		returningManyOf: GlorpPerson
		where: [:pers | pers id = personId].
	results := query executeIn: session.
	self assert: results size = 1.
	person := results first.
!

writeAddress
	| addressRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
!

writeHomefulPerson
	| personRow |
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.
!

writeHomelessPerson
	| personRow |
	personRow := session system examplePersonRow2.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.
!

writePersonWithAddress
	self writeAddress.
	self writeHomefulPerson.
! !

!GlorpOneToOneDBTest methodsFor:'tests-join'!

testReadPersonWithJoinToAddress
	| people |
	self inTransactionDo: [
		self writePersonWithAddress.
		people := session readManyOf: GlorpPerson where: [:eachPerson | eachPerson address street = 'Paseo Montril'].
		self assert: people size = 1.
		person := people first.
		self assert: person address street = 'Paseo Montril'.
		self assert: person address id = 123].
! !

!GlorpOneToOneDBTest methodsFor:'tests-link table'!

testLinkTableRead
	| customers |
	system := GlorpLinkTableAnywhereDescriptorSystem forPlatform: system platform.
	session system: system.
	self inTransactionDo: [
		self writeCustomersWithOneAccountEach.
		session reset.
		customers := session readManyOf: GlorpCustomer.
		self assert: ((customers collect: [:each | each accounts id]) asSortedCollection asArray = #(22 9874))].
!

testLinkTableWrite
	system := GlorpLinkTableAnywhereDescriptorSystem
		forPlatform: system platform.
	session system: system.
	self
		inTransactionDo:
			[| rows row |
			self
				inUnitOfWorkDo:
					[| cust1 acct1 |
					cust1 := GlorpCustomer new.
					cust1 id: 2.
					cust1 name: 'Violet Baudelaire'.
					acct1 := GlorpBankAccount new.
					acct1 id: 3.
					cust1 accounts: acct1.
					session register: cust1].
			session reset.
			rows := session accessor
				executeSQLString:
					'select * from GR_CUSTOMER t1, BANK_ACCT t2, CUSTOMER_ACCT_LINK t3 WHERE t3.ACCT_ID = t2.ID AND t3.CUSTOMER_ID = t1.ID'.
			self assert: rows size = 1.
			row := rows first.
			self assert: (row asArray = (OrderedCollection new add: 2; add: 'Violet Baudelaire'; add: 3; add: nil; add: nil; add: nil; add: 2; add: 3; yourself) asArray)].
!

writeCustomersWithOneAccountEach
	| customerRow accountRow1 accountRow2 linkRow1 linkRow2 customerRow2 |

	customerRow := session system exampleCustomerRow1.
	customerRow2 := session system exampleCustomerRow2.
	accountRow1 := session system exampleAccountRow1.
	accountRow2 := session system exampleAccountRow3.
	linkRow1 := session system exampleCALinkRow1.
	linkRow2 := session system exampleCALinkRow3.
	session writeRow: customerRow.
	session writeRow: customerRow2.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
! !

!GlorpOneToOneDBTest methodsFor:'tests-read'!

additionalTests

	"It would be good to have tests here for a foreign key 'pointing' the other direction. Also composite keys (once those work)"
!

testReadPersonAndAddAddress
	self inTransactionDo: [
		self
			inUnitOfWorkDo: [person address: (GlorpAddress new id: 5555; street: 'hello'; number: 'world')]
			initializeWith: [self writeHomelessPerson].
		self readPerson.
		self checkPerson].
!

testReadPersonAndRemoveAddress
	self inTransactionDo: [
		self inUnitOfWorkDo: [
			session delete: person address.
			person address: nil.].
		self readPerson.
		self checkPerson.
		self checkNoAddress.
		self checkNoAddressesInDB].
!

testReadPersonAndReplaceAddress
	self inTransactionDo:
		[self inUnitOfWorkDo:
			[person address: ((GlorpAddress new) id: 12; street: 'foo'; number: '1234')].
		self readPerson.
		self checkPerson.
		self assert: person address isGlorpProxy.
		self assert: person address getValue id = 12.
		self assert: person address getValue street = 'foo'.].
!

testReadPersonAndReplaceAdressWithDifferentProxy
	| proxy |
	self inTransactionDo:
			[
			session writeRow: session system exampleAddressRowForOrdering1.
			self inUnitOfWorkDo:
					[ | field |
					proxy := Proxy new.
					proxy query: person address query copy.
					field := (session system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID'.
					proxy parameters: (IdentityDictionary new at: field put: 1; yourself).
					proxy session: person address session.
					person address: proxy].
			self readPerson.
			self checkPerson.
			self assert: person address isGlorpProxy.
			self assert: person address getValue notNil.
			self assert: person address getValue id = 1]
!

testReadPersonAndReplaceAdressWithProxy
	| proxy |
	self inTransactionDo:
			[self inUnitOfWorkDo:
					[proxy := Proxy new.
					proxy query: person address query copy.
					proxy parameters: person address parameters copy.
					proxy session: person address session.
					person address: proxy].
			self readPerson.
			self checkPerson.
			self assert: person address isGlorpProxy.
			self assert: person address getValue notNil.
			self assert: person address getValue id = 123]
!

testReadPersonAndUpdateName
	"Test to make sure that if we never instantiate the address, we don't have problems with that relationship"
	self inTransactionDo:
		[self inUnitOfWorkDo:
			[person name: 'new name'].
		self deny: person address isInstantiated.
		session reset.
		self readPerson.
		self assert: person name = 'new name'.
		self assert: person address isGlorpProxy.
		self assert: person address getValue id = 123.
		self assert: person address getValue street = 'Paseo Montril'.].
!

testReadPersonWithAddress
	self inTransactionDo: [
		self writePersonWithAddress.
		self readPerson.
		self checkPerson.
		self checkAddress].
!

testReadPersonWithoutAddress
	self inTransactionDo: [
		self writeHomelessPerson.
		self writeAddress.
		self readPerson.
		self checkPerson.
		self checkNoAddress].
!

testReadWithCacheHitFromExpression
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[self assert: person address yourSelf id = 123.
					session accessor permitNothing.
					session readOneOf: GlorpAddress where: [:each | each id = 123]]].
!

testReadWithCacheHitFromJoin
	self inTransactionDo: [
		self
			inUnitOfWorkDo: [
				session readOneOf: GlorpAddress.
				session accessor permitNothing.
				self assert: (person address yourSelf id = 123)]].
! !

!GlorpOneToOneDBTest methodsFor:'tests-write'!

testWritePersonWithAddress
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = newPerson name.
		self assert: person address id = newPerson address id.
		self assert: person address street = newPerson address street].
!

testWritePersonWithoutAddress
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		newPerson address: nil.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = newPerson name.
		self assert: person address yourSelf == nil.
		self checkNoAddressesInDB].
! !

!GlorpOneToOneDBTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpOneToOneDBTest.st,v 1.1 2006/07/26 09:15:31 boris Exp $'
! !
