"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpOptimisticLockingTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTests'
!


!GlorpOptimisticLockingTest methodsFor:'support'!

setUp
	super setUp.
	session system: GlorpLockingDescriptorSystem new.
	session beginTransaction.
	session transact: [session register: session system example1].
!

tearDown
	session rollbackTransaction.
	super tearDown.
! !

!GlorpOptimisticLockingTest methodsFor:'tests'!

changeABookFrom: books
	^session
		transact:
			[session register: books.
			books first copiesInStock: books first copiesInStock - 1].
!

testSuccessfulUpdate
	| books copies |
	books := session readManyOf: GlorpBook.
	copies := books collect: [:each | each copiesInStock].
	session transact: [
		session register: books.
		books do: [:each | each copiesInStock: (each copiesInStock - 1)]].
	session reset.
	books := session readManyOf: GlorpBook orderBy: #id.
	self assert: (books collect: [:each | each copiesInStock]) = (copies collect: [:each | each - 1]).
!

testSuccessfulUpdateOntoNullVersionField
	| books copies someBook |
	someBook := session readOneOf: GlorpBook.
	session accessor executeSQLString: 'UPDATE BOOK SET VERSION = NULL WHERE ID = ', someBook id printString.
	session reset.
	books := session read: GlorpBook orderBy: #id.
		copies := books collect: [:each | each copiesInStock].
	session transact: [
		session register: books.
		books do: [:each | each copiesInStock: (each copiesInStock - 1)]].
	session reset.
	books := session readManyOf: GlorpBook orderBy: #id.
	self assert: (books collect: [:each | each copiesInStock]) = (copies collect: [:each | each - 1]).
!

testUnsuccessfulUpdateProceedingAndFailing
        | books |
        books := session readManyOf: Glorp::GlorpBook.
        session accessor
                executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
        self should: [[self changeABookFrom: books] on: Glorp::GlorpWriteFailure do: [:ex | ex resume: false]] raise: Glorp::GlorpTransactionFailure.

        session reset.
        books := session readManyOf: Glorp::GlorpBook.
        self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).
!

testUnsuccessfulUpdateProceedingAnyway
	| books |
	books := session readManyOf: GlorpBook.
	session accessor
		executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
	[self changeABookFrom: books] on: GlorpWriteFailure do: [:ex | ex resume: true].

	session reset.
	books := session readManyOf: GlorpBook.
	self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).
!

testUnsuccessfulUpdateWithNoAction
        "This is a little quirky. SUnit's action in should:raise: is to exit the exception with true. So the thing just short circuits, and we don't get a transaction failure because we killed the whole unit of work. This is, in general, what will happen if we handle and don't resume the write failures."
        | books |
        books := session readManyOf: Glorp::GlorpBook.
        session accessor
                executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
        self shouldnt: [
                self should: [self changeABookFrom: books] raise: Glorp::GlorpWriteFailure]
                raise: Glorp::GlorpTransactionFailure.
        session reset.
        books := session readManyOf: Glorp::GlorpBook.
        self assert: (books collect: [:each | each copiesInStock]) = #(7 7).
! !

!GlorpOptimisticLockingTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpOptimisticLockingTest.st,v 1.1 2006/07/26 09:17:27 boris Exp $'
! !
