"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpQueryUnionTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!

GlorpQueryUnionTest comment:'This tests the ability to do a union operation on queries, returning results corresponding to multiple different queries.'
!


!GlorpQueryUnionTest methodsFor:'support'!

setUp
	super setUp.
	session beginTransaction.
	self writeAddressOrderingRows.
!

tearDown
	session rollbackTransaction.
	super tearDown.
!

writeAddressOrderingRows

	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
!

writePeople

	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
! !

!GlorpQueryUnionTest methodsFor:'tests-functional'!

testBasicExcept
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 1.
!

testBasicExcept2
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 1.
	self assert: result first id = 2.
!

testBasicExcept3
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each street = 'Alpha'].
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 0.
!

testBasicIntersect
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 0.
!

testBasicIntersect2
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 1.
!

testBasicIntersect3
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each street = 'Alpha'].
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 1.
!

testBasicUnion
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result size = 2.
	self assert: result first id = 1.
	self assert: result last id = 2.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).
!

testOrderByInUnion
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id > 1].
	query1 retrieve: #id.
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: [:each | each id].
	union := query1 unionAll: query2.
	union orderBy: [:each | each id descending].
	result := session execute: union.
	self assert: result = #(87 86 3 2).
!

testThreeWayUnion
	| query1 query2 union result query3 |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	query3 := Query returningManyOf: GlorpAddress where: [:each | each id = 3].

	union := (query1 unionAll: query2) unionAll: query3.
	result := session execute: union.
	self assert: result size = 3.
	self assert: result first id = 1.
	self assert: result last id = 3.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).
!

testUnionWithEmpty
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: false.
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result size = 1.
	self assert: result first id = 2.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).
!

testUnionWithParameters
	| query1 query2 union result field1 dict |
	field1 := (system tableNamed: 'GR_ADDRESS') fieldNamed: 'ID'.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = (each parameter: field1)].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = ((each parameter: field1) + 1)].
	union := query1 unionAll: query2.
	dict := Dictionary new at: field1 put: 2; yourself.
	result := union executeWithParameters: dict in: session.
	self assert: result size = 2.
	self assert: result first id = 2.
	self assert: result last id = 3.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).
!

testUnionWithRetrieveDisaparateResults
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress.
	query1 retrieve: #id.
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: #id.
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #(1 2 3 86 87).
!

testUnionWithRetrieveThoroughlyDisaparateResults
	"Fields from mismatched tables"
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress.
	query1 retrieve: [:each | each street].
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: [:each | each name].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #('Alpha' 'Beta' 'Gamma' 'person1' 'person2').
!

testUnionWithRetrieveThoroughlyDisaparateResults2
	"The tables will get aliased differently, make sure that works"

	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	query1 retrieve: [:each | each street].
	query2 := Query returningManyOf: GlorpPerson where: [:each | each id = 87].
	query2 retrieve: [:each | each address street].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #('Beta' 'Gamma').
! !

!GlorpQueryUnionTest methodsFor:'tests-unit'!

testCompoundCommandCreate
	| query1 union command |
	query1 := Query returningManyOf: GlorpAddress.
	union := query1 unionAll: query1.
	union session: session.
	command := union sqlWith: #().
	self assert: command class == CompoundSelectCommand.
!

testCreateCompoundQuery
	| query1 union |
	query1 := Query returningManyOf: GlorpAddress.
	union := query1 unionAll: query1.
	self assert: union class == CompoundQuery.
!

testUnionSQL
	| query1 union command |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	union := query1 unionAll: query1.
	union session: session.
	command := union sqlWith: #().
	self assert: ('SELECT * FROM ((SELECT *
 FROM GR_ADDRESS t1
 WHERE (t1.ID = *)) UNION ALL (SELECT *
 FROM GR_ADDRESS t1
 WHERE (t1.ID = *)))*' match: command sqlString).
! !

!GlorpQueryUnionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpQueryUnionTest.st,v 1.1 2006/07/26 09:00:52 boris Exp $'
! !
