"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpReadingDifferentCollectionsThroughMappingsTest
	instanceVariableNames:'system session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpReadingDifferentCollectionsThroughMappingsTest class methodsFor:'resources'!

resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource with: GlorpSessionResource.
! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor:'setup'!

descriptorClass
	^GlorpCollectionTypesDescriptorSystem.
!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system := self descriptorClass forPlatform: session platform.
	session system: system.
	session beginTransaction.
!

tearDown
	super tearDown.
	session rollbackTransaction.
!

writeMore

	| other |
	session transact: [
		session register: GlorpThingWithLotsOfDifferentCollections example1.
		other := GlorpThingWithLotsOfDifferentCollections example1.
		other name: 'barney'.
		session register: other].
!

writeRows

	session transact: [
		session register: GlorpThingWithLotsOfDifferentCollections example1].
! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor:'tests'!

testReadBack
        | thing list bool |
        self writeRows.
        session reset.
        list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
        self assert: list size = 1.
        thing := list first.
        self assert: thing array size = 3.
        self
                assert:
                        (self validateFor: thing array against: #('array1' 'array2' 'array3')).
 bool := thing array yourself class = Array.
        self assert: thing array yourself class = Array.
        self assert: thing set size = 2.
        self assert: thing set yourself class = Set.
        self assert: (self validateFor: thing set against: #('set1' 'set2')).
        self assert: thing orderedCollection size = 2.
        self assert: thing orderedCollection yourself class = OrderedCollection.
        self
                assert:
                        (self
                                validateFor: thing orderedCollection
                                against: #('orderedCollection1' 'orderedCollection2')).
        self assert: thing orderedCollection first name = 'orderedCollection1'.
        self assert: thing bag size = 2.
        self assert: thing bag yourself class = Bag.
        self assert: (self validateFor: thing bag against: #('bag1' 'bag2')).
        self assert: thing sortedCollection size = 4.
        self assert: thing sortedCollection yourself class = SortedCollection.
        self assert:
                        (thing sortedCollection collect: [:each | each name]) asArray
                                = #('sorted1' 'sorted2' 'sorted3' 'sorted4').
!

testReadBackOneOfSeveral
        | thing list array set oc bag sc |
        self writeMore.
        session reset.
        list := session readManyOf: GlorpThingWithLotsOfDifferentCollections where: [:each | each name = 'fred'].
        self assert: list size = 1.
        thing := list first.
        self assert: thing array size = 3.
        self
                assert:
                        (self validateFor: thing array against: #('array1' 'array2' 'array3')).
        array := thing array.
        self assert:  array class = Array.
        self assert: thing set size = 2.
        self assert: thing set yourself class = Set.
        self assert: (self validateFor: thing set against: #('set1' 'set2')).
        self assert: thing orderedCollection size = 2.
        self assert: thing orderedCollection yourself class = OrderedCollection.
        self
                assert:
                        (self
                                validateFor: thing orderedCollection
                                against: #('orderedCollection1' 'orderedCollection2')).
        self assert: thing bag size = 2.
        self assert: thing bag yourself class = Bag.
        self assert: (self validateFor: thing bag against: #('bag1' 'bag2')).
        self assert: thing sortedCollection size = 4.
        self assert: thing sortedCollection yourself class = SortedCollection.
        self assert:
                        (thing sortedCollection collect: [:each | each name]) asArray
                                = #('sorted1' 'sorted2' 'sorted3' 'sorted4').
!

testReadCollectionWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing orderedCollection size = 6.
	self
		assert:
			(thing orderedCollection collect: [:each | each name]) asArray = #('oc6' 'oc5' 'oc4' 'oc3' 'oc7' 'oc8')
!

testReadManyToManyWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing orderedCollection size = 6.
	self
		assert:
			(thing orderedCollection collect: [:each | each name]) asArray = #('oc6' 'oc5' 'oc4' 'oc3' 'oc7' 'oc8')
!

testReadOneToManyWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing array size = 6.
	self
		assert:
			(thing array collect: [:each | each name]) asArray = #('a1' 'a2' 'a3' 'a9' 'a8' 'a7')
!

validateFor: aCollection against: expectedArrayContents
	^(aCollection collect: [:each | each name]) asSortedCollection asArray
		= expectedArrayContents.
! !

!GlorpReadingDifferentCollectionsThroughMappingsTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpReadingDifferentCollectionsThroughMappingsTest.st,v 1.1 2006/07/26 09:18:20 boris Exp $'
! !
