"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpDatabaseBasedTest subclass:#GlorpSessionTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpSessionTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpSessionTest methodsFor:'support'!

addCustomerToCache
	| customer |
	customer := GlorpCustomer example1.
	customer id: 3.
	session cacheAt: 3 put: customer.
	^customer
!

setUp
	super setUp.
	session := GlorpSession new.
	session system: system
! !

!GlorpSessionTest methodsFor:'tests'!

helpTestUpdateWithExpiredExistingEntry: aCachePolicy
	| customer customer2 row table unitOfWork |
	(session system descriptorFor: GlorpCustomer)
		cachePolicy: aCachePolicy.
	customer := self addCustomerToCache.
	customer2 := GlorpCustomer new.
	customer2 id: customer id.
	customer2 name: 'Barney Rubble'.
	row := DatabaseRow newForTable: (table := system tableNamed: 'GR_CUSTOMER').
	row at: (table fieldNamed: 'ID') put: customer id.
	unitOfWork := UnitOfWork new.
	unitOfWork session: session.
	"Since there's already an object there, this shouldn't do anything"
	unitOfWork updateSessionCacheFor: customer2 withRow: row.
	self assert: (session expiredInstanceOf: GlorpCustomer key: 3) == customer.
!

helpTestUpdateWithoutExpiredExistingEntry: aCachePolicy
	| customer2 row table unitOfWork |
	(session system descriptorFor: GlorpCustomer)
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer2 := GlorpCustomer new.
	customer2 id: 3.
	customer2 name: 'Barney Rubble'.
	row := DatabaseRow newForTable: (table := system tableNamed: 'GR_CUSTOMER').
	row at: (table fieldNamed: 'ID') put: 3.
	unitOfWork := UnitOfWork new.
	unitOfWork session: session.
	unitOfWork updateSessionCacheFor: customer2 withRow: row.
	self assert: (session expiredInstanceOf: GlorpCustomer key: 3) == customer2.
!

testAddingDescriptors

	self assert: session system == system.
	self assert: system session == session.
	(session descriptorFor: GlorpAddress) session == session.
!

testDuplicateDetection
	| customer customer2 |
	session beginUnitOfWork.
	customer := self addCustomerToCache.
	customer2 := customer copy.
	self should: [session register: customer2] raise: Dialect error.
	session rollbackUnitOfWork.
!

testExecuteQuery
	| q result |
	q := GlorpQueryStub new result: 3.
	result := session execute: q.
	self assert: result = 3
!

testHasExpired1
	| customer |
	customer := self addCustomerToCache.
	self deny: (session hasExpired: customer)
!

testHasExpired2
	| customer |
	(session system descriptorFor: GlorpCustomer)
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer := self addCustomerToCache.
	self assert: (session hasExpired: customer)
!

testHasExpired3
	| customer |
	(session system descriptorFor: GlorpCustomer)
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer := self addCustomerToCache.
	self assert: (session hasExpired: customer)
!

testHasObjectOfClassExpired1

	self addCustomerToCache.
	self deny: (session hasObjectExpiredOfClass: GlorpCustomer withKey: 3).
!

testHasObjectOfClassExpired2

	(session system descriptorFor: GlorpCustomer)
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	self addCustomerToCache.
	self assert: (session hasObjectExpiredOfClass: GlorpCustomer withKey: 3).
!

testSQLDeleteStringFor
	| row table string |
	table := session system tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 12.
	row at: (table fieldNamed: 'STREET') put: 'some street'.
	string := (DeleteCommand forRow: row useBinding: true platform: session system platform) sqlString.
	self assert: string = 'DELETE FROM GR_ADDRESS WHERE ID = ?'.
	string := (DeleteCommand forRow: row useBinding: false platform: session system platform) sqlString.
	self assert: string = 'DELETE FROM GR_ADDRESS WHERE ID = 12'.
!

testUpdateWithExpiredExistingEntry
	self helpTestUpdateWithExpiredExistingEntry: (TimedExpiryCachePolicy new timeout: 0; expiryAction: #refresh).
!

testUpdateWithExpiredExistingEntry2
	self helpTestUpdateWithExpiredExistingEntry: (TimedStrongCachePolicy new timeout: 0; expiryAction: #refresh).
!

testUpdateWithoutExpiredExistingEntry
	self helpTestUpdateWithoutExpiredExistingEntry: (TimedExpiryCachePolicy new timeout: 0).
!

testUpdateWithoutExpiredExistingEntry2
	self helpTestUpdateWithoutExpiredExistingEntry: (TimedStrongCachePolicy new timeout: 0).
! !

!GlorpSessionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpSessionTest.st,v 1.1 2006/07/26 09:20:06 boris Exp $'
! !
