"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpSimpleQueryTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpSimpleQueryTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpSimpleQueryTest class methodsFor:'resources'!

resources

	^Array with: GlorpSessionResource.
! !

!GlorpSimpleQueryTest methodsFor:'support'!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
!

tearDown
	super tearDown.
	session reset.
	session := nil.
! !

!GlorpSimpleQueryTest methodsFor:'tests'!

setUpQueryBasic: query
	| |
	query session: session.
	query setUpExpressions.
	query setupTracing.
!

setUpQueryFields: query
	self setUpQueryBasic: query.
"       query setupTracing."
	query computeFields.
!

setUpQueryFully: query
	self setUpQueryBasic: query.
	query prepare
!

skipToString
	^session platform supportsANSIJoins ifTrue: ['join '] ifFalse: ['where '].
!

testCaseInsensitiveQuery
        | result |
        session platform supportsCaseInsensitiveLike ifFalse: [^self].
        [session beginUnitOfWork.
        session beginTransaction.
        session register: GlorpAddress example1.
        session commitUnitOfWork.
        result := session accessor execute: 
                     [session
                       readOneOf: GlorpAddress
                       where: [:address | address street ilike: 'WeSt%']
                     ]
                   ifError: [:ex | ex return: nil].
        self assert: result notNil.
        self assert: result street = 'West 47th Ave']
                        ensure: [session rollbackTransaction]
!

testComputingFieldsForDirectMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpAddress where: [:each | each id = 1].
	self setUpQueryFields: query.

	table := session system tableNamed: 'GR_ADDRESS'.
	self assert: (query fields = table fields).
!

testComputingFieldsForReferenceMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpPerson where: [:each | each id = 1].
	self setUpQueryFields: query.

	table := session system tableNamed: 'PERSON'.
	self assert: (query fields = table fields).
!

testDescriptorAssignmentToCriteria

	| query |
	query := SimpleQuery returningOneOf: GlorpAddress where: [:each | each id = 1].
	query session: session.
	query setUpExpressions.
	self assert: query whereClause ultimateBaseExpression descriptor == (session descriptorFor: GlorpAddress).
!

testFieldAliasingForEmbeddedMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpBankTransaction where: [:each | each id = 1].
	self setUpQueryFields: query.
	table := session system tableNamed: 'BANK_TRANS'.
	self assert: (query fields = table fields).
	self assert: (query builders first translateFieldPosition: (table fieldNamed: 'ID')) = 1.
	self assert: (query builders first translateFieldPosition: (table fieldNamed: 'OWNER_ID')) = 2.
!

testJoinExpressionWithMultipleTables
        | query sql sqlStream result command |
        query := SimpleQuery returningOneOf: GlorpPassenger where: [:each | each id = 1].
        query session: session.
        query prepare.
        command := query sqlWith: Dictionary new.
        command useBinding: false.
        sql := command sqlString.
        sqlStream := ReadStream on: sql asLowercase.
        sqlStream skipToAll: self skipToString.
        Dialect isVisualWorks ifTrue: [sqlStream skip: self skipToString size]. "<Grumble grumble> stupid incompatibilities"
        result := sqlStream upToEnd.
        session platform supportsANSIJoins ifTrue: [
                self assert: ('join frequent_flyer t2 on (t1.id = t2.id))
 where (t1.id = 1)*' match: result)]
                ifFalse: [self assert:  ('*(t1.id = 1) and ((t1.id = t2.id*' match: result)].
! !

!GlorpSimpleQueryTest methodsFor:'tests-ordering'!

testDoubleOrderSQL

	| query sql |
	query := SimpleQuery returningManyOf: GlorpAddress.
	query orderBy: [:each | each id].
	query orderBy: [:each | each number].
	self setUpQueryFully: query.
	sql := (query sqlWith: Dictionary new) sqlString.
	self assert: ('* from gr_address t1 order by t1.id, t1.house_num' match: sql asLowercase).
!

testOrderSQL

	| query sql |
	query := SimpleQuery returningManyOf: GlorpAddress.
	query orderBy: [:each | each id].
	self setUpQueryFully: query.
	sql := (query sqlWith: Dictionary new) sqlString.
	self assert: ('* from gr_address t1 order by t1.id' match: sql asLowercase).
! !

!GlorpSimpleQueryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpSimpleQueryTest.st,v 1.1 2006/07/26 09:21:31 boris Exp $'
! !
