"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpStXDatabaseBasicTest
	instanceVariableNames:'system session accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpStXDatabaseBasicTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpStXDatabaseBasicTest methodsFor:'accessing'!

accessor
        ^accessor
! !

!GlorpStXDatabaseBasicTest methodsFor:'support'!

databaseLoginResource
	^GlorpDatabaseLoginResource current.
!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.

  session := GlorpSession new.
  session system: (system := GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'ALTER TABLE STUFF DROP CONSTRAINT STUFF_PK'.]
     ifError: [:ex | ex return: nil].

  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'DROP TABLE STUFF'.]
     ifError: [:ex | ex return: nil].
  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'ALTER TABLE EMPTY_TABLE DROP CONSTRAINT EMPTY_TABLE_PK'.]
     ifError: [:ex | ex return: nil].

  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'DROP TABLE EMPTY_TABLE'.]
     ifError: [:ex | ex return: nil].

  self setUpDatabase.
  accessor clearAutocommit.
!

setUpDatabase
   accessor executeSQLStringNoResult: 
     'CREATE TABLE STUFF
        (ID integer NOT NULL,
         THING varchar(20) NULL,
         CONSTRAINT STUFF_PK PRIMARY KEY (ID)
         )'.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (12,''abc'')'.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (13, ''hey nonny nonny'')'.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (42, ''yabba dabba doo'')'.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (9625, ''the band played on'')'.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (113141, ''Smalltalk'')'.
   accessor executeSQLStringNoResult: 
     'CREATE TABLE EMPTY_TABLE
        (ID integer NOT NULL,
         THING varchar(20) NULL,
         CONSTRAINT EMPTY_TABLE_PK PRIMARY KEY (ID)
         )'.
!

tearDown
       " super tearDown.  "

        session reset.
        accessor logout. 
        session := nil.
        system := nil.
! !

!GlorpStXDatabaseBasicTest methodsFor:'tests'!

testBeginTransactionWithCommit
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor commitTransaction.
	self assert: self accessor isInTransaction not
!

testBeginTransactionWithRollback
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor rollbackTransaction.
	self assert: self accessor isInTransaction not
!

testCreateTable
        | selectResult |

        [[accessor beginTransaction.
        accessor
                executeSQLStringNoResult: 'CREATE TABLE GLORP_TEST_CREATE (ID varchar(4))']
                ensure: [accessor commitTransaction].
        selectResult := accessor
                executeSQLString: 'SELECT * FROM GLORP_TEST_CREATE'.
        self assert: selectResult isEmpty]
                ensure:
                        [[accessor
                                beginTransaction;
                                dropTableNamed: 'GLORP_TEST_CREATE'
                                        ifAbsent: [:ex | self assert: false]]
                                ensure: [accessor commitTransaction]].
!

testDropMissingTable
        | absentFlag |
        absentFlag := false.
        [self accessor beginTransaction.
        self accessor
                dropTableNamed: 'GLORP_TEST_DROP_X'
                ifAbsent: [:ex  | absentFlag := true. ex return: nil]]
        ensure: [self accessor rollbackTransaction].
        self assert: absentFlag
!

testNameBinding
	| selectResult |
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :key OR ID = :value'
			binding: 10->30.

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].
!

testPositionalBinding
	| selectResult |
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :2 OR ID = :17'
			binding: #(nil 10 nil nil nil nil nil nil nil nil nil nil nil nil nil nil 20).

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].
!

testReadEmpty
        | results |
        results := 
          accessor execute: 
             [accessor executeSQLString: 'SELECT * FROM EMPTY_TABLE'.]
          ifError: [:ex | ex return: nil.].
        self assert: results notNil.
        self assert: results size = 0
!

testReadStatement
        | results |
        results := 
          accessor execute: 
             [accessor executeSQLString: 'SELECT * FROM STUFF ORDER BY ID'.]
          ifError: [:ex | ex return: nil.].
        self assert: results notNil.

        self assert: results size = 5.
        self assert: results first size = 2.
        self assert: results first last =  'abc'
!

testRollbackRemovesData
        "Just to make sure I'm not losing my mind"
        | numAddresses newNumAddresses|
        numAddresses := (accessor executeSQLString: 'SELECT * FROM STUFF') size.
        accessor beginTransaction.
        accessor executeSQLStringNoResult: 'INSERT INTO STUFF (ID,THING)  VALUES (111,''Main Street'')'.
        newNumAddresses := (accessor executeSQLString: 'SELECT * FROM STUFF') size.
        self assert: newNumAddresses = (numAddresses + 1).
        accessor rollbackTransaction.
        self assert: numAddresses = (accessor executeSQLString: 'SELECT * FROM STUFF') size.
! !

!GlorpStXDatabaseBasicTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpStXDatabaseBasicTest.st,v 1.1 2006/07/26 09:14:51 boris Exp $'
! !
