"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpStXDictionaryMappingTest
	instanceVariableNames:'accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpStXDictionaryMappingTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpStXDictionaryMappingTest methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.

  session := GlorpSession new.
  session system: (system := GlorpEncyclopediaDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  self setUpDatabase.
  accessor clearAutocommit.
!

setUpDatabase

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [self  setUpSystem: (GlorpEncyclopediaDescriptorSystem forPlatform: accessor platform) setUpDefaults.

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: sys 
        | errorBlock |
        accessor isLoggedIn ifFalse: [accessor login].
        accessor dropTables: ((TableSorter for: sys allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        sys platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: sys allSequences.
                sys allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        sys allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        sys allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        sys allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
!

tearDown
       " super tearDown.  "
        session reset.
        accessor logout. 
        session := nil.
        system := nil.
! !

!GlorpStXDictionaryMappingTest methodsFor:'tests'!

testCasesToWrite

	"How des a dictionary mapping relate to 1:many vs many:many.
	dictionary of strings to strings
	dictionary of strings to objects
	dictionary of objects to objects
	keys always have to be related to values somehow, because I can't extract the association otherwise. Both might also be associated to source.
    You should be able to use the topological sort to determine the create/delete order of tables as well"
!

testCollectionKeyQuery

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
"       matches := session read: GlorpEncyclopedia       where: [:each | each entries anySatisfy: [:eachEntry |
		((eachEntry getTable: 'ENCYC_ENTRY_LINK') getField: 'REFERENCE_NO') = 'unique']]."
	matches := session read: GlorpEncyclopedia       where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry key = 'unique']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].
!

testCollectionValueQuery

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry value title = 'Vehicular Fish Decay']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].
!

testCollectionValueQueryWithImpliedValue

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry title = 'Vehicular Fish Decay']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].
!

testCollectionValueQueryWithImpliedValueAndObjectEquality

	| encyclopedias matches entry |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	entry := session readOneOf: GlorpEncyclopediaEntry where: [:each | each title = 'Vehicular Fish Decay'].
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry = entry]].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].
!

testModify

	| encyclopedias canadian allEntryRows allLinkRows |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	session accessor denyCommands: (Array with: DeleteCommand).
	session inUnitOfWorkDo: [
		| newEntry |
		canadian := session readOneOf: GlorpEncyclopedia where: [:each | each name like: 'The Ca%'].
		newEntry := GlorpEncyclopediaEntry new
			id: 'seven';
			title: 'The Latest Entry';
			text: 'This is all about the very newest stuff'.
		canadian entries at: 'unique' put: newEntry].
	self assert: (canadian entries at: 'abcde') id = 'two'.
	self assert: (canadian entries at: '12345') id = 'one'.
	self assert: (canadian entries at: 'unique') id = 'seven'.
	self assert: (canadian entries at: 'unique') title = 'The Latest Entry'.
	allEntryRows := session accessor executeSQLString: 'select * from ENCYC_ENTRY'.
	"We didn't actually delete the 'three' entry"
	self assert: allEntryRows size = 4.
	allLinkRows := session accessor executeSQLString: 'select * from ENCYC_ENTRY_LINK'.
	"But we should have only our 3 link table rows, plus 2 for the other encyclopedia"
	self assert: allLinkRows size = 5.
!

testQueryWithDictionaryType

	| encyclopedias query dict linkTable entryTable refField linkTableField idField |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	linkTable := session system tableNamed: 'ENCYC_ENTRY_LINK'.
	entryTable := session system tableNamed: 'ENCYC_ENTRY'.
	refField := linkTable fieldNamed: 'REFERENCE_NO'.
	linkTableField := linkTable fieldNamed: 'ENCYC_ENTRY_ID'.
	idField := entryTable fieldNamed: 'ID'.
	query := Query readManyOf: GlorpEncyclopediaEntry.
	query where: [:each | ((each getTable: linkTable) getField: linkTableField) = ((each getTable: entryTable) getField: idField)].
	query retrieveDictionaryWithKey: [:each | (each getTable: linkTable) getField: refField] value: [:each | each].
	dict := session execute: query.
	self assert: dict class == Dictionary.
	self assert: dict size = 5.
	self assert: (dict at: 'abcde') id = 'two'.
	self assert: (dict at: 'abcde') == (dict at: 'edcba').
!

testWriteAndReadBack

	| encyclopedias query readBackEncyclopedias canadian plagiarists |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	query := Query readManyOf: GlorpEncyclopedia.
	query orderBy: [:each | each name].
	readBackEncyclopedias := session execute: query.
	self assert: readBackEncyclopedias size = 3.
	canadian := readBackEncyclopedias first.
	self assert: canadian name = 'The Canadian Encyclopedia'.
	self assert: canadian entries size = 3.
	self assert: (canadian entries at: 'abcde') id = 'two'.
	self assert: (canadian entries at: '12345') id = 'one'.
	self assert: (canadian entries at: 'unique') id = 'three'.
	plagiarists := readBackEncyclopedias at: 2.
	self assert: plagiarists entries size = 2.
	self assert: (plagiarists entries at: '54321') = (canadian entries at: '12345').
	self assert: (plagiarists entries at: 'edcba') = (canadian entries at: 'abcde').
	self assert: readBackEncyclopedias last entries isEmpty.
! !

!GlorpStXDictionaryMappingTest methodsFor:'tests-key in different tables'!

testWriteAndReadBackKeyInLinkTable

	| query readBackPublishers publishers withEmps |
	publishers := session system examplePublishers.
	session transact: [session register: publishers].
	session reset.
	query := Query readManyOf: GlorpPublisher.
	query orderBy: [:each | each name].
	readBackPublishers := session execute: query.
	self assert: readBackPublishers size = 3.
	withEmps := readBackPublishers at: 2.
	self assert: withEmps name = 'Cambridge'.
	self assert: withEmps differentBunchOfEmployeesById size = 2.
	self assert: withEmps differentBunchOfEmployeesById keys asSortedCollection asArray = #(3 4).
	self assert: (withEmps differentBunchOfEmployeesById at: 3) name = 'Contractor 1'.
	self assert: (withEmps differentBunchOfEmployeesById at: 4) name = 'Contractor 2'.
!

testWriteAndReadBackKeyInTargetTable

	| query readBackPublishers publishers withEmps |
	publishers := session system examplePublishers.
	session transact: [session register: publishers].
	session reset.
	query := Query readManyOf: GlorpPublisher.
	query orderBy: [:each | each name].
	readBackPublishers := session execute: query.
	self assert: readBackPublishers size = 3.
	withEmps := readBackPublishers at: 2.
	self assert: withEmps name = 'Cambridge'.
	self assert: withEmps employees size = 2.
	self assert: withEmps employees keys asSortedCollection asArray = #('CEO' 'Editor').
	self assert: (withEmps employees at: 'CEO') name = 'The Boss'.
	self assert: (withEmps employees at: 'Editor') name = 'Somebody'.
! !

!GlorpStXDictionaryMappingTest methodsFor:'tests-simple reference classes'!

testDeleteSimpleTypeOneToMany
	| publishers publisher |
	self assert: (session read: GlorpPublisher) isEmpty.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titles: (publisher titles copyWithout: 'Mastering ENVY/Developer')].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: publisher titles size = 2.
	self assert: publisher titles asSortedCollection = (session system examplePublisher1 titles copyWithout: 'Mastering ENVY/Developer') asSortedCollection.
!

testSimpleTypeOneToMany
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id = 1].
	publishers do: [:each |
		each titles do: [:eachTitle | self assert: eachTitle isString]].
	self assert: publishers first titles size = 3.
	self assert: publishers first titles asSortedCollection = session system examplePublisher1 titles asSortedCollection.
!

testSimpleTypeOneToManyComplexQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titles sqlCount = 3].
	self assert: publishers size = 1.
	self assert: publishers first titles size = 3.
	self assert: (publishers first titles includes: 'Mastering ENVY/Developer').
!

testSimpleTypeOneToManyQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titles anySatisfy: [:x | x like: 'Master%' ]].
	self assert: publishers size = 1.
	self assert: (publishers first titles anySatisfy: [:each | each like: 'Master%']).
!

testSimpleTypeOneToManyQueryingWithFilteredReads
	| publishers publisherOne publisher2 |
	((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titles) useFilteredRead.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id < 3].
	self assert: publishers size = 2.
	publisherOne := publishers detect: [:each | each id = 1].
	self assert: publisherOne titles size = 3.
	self assert: (publisherOne titles includes: 'Mastering ENVY/Developer').
	session accessor denyReads.
	publisher2 := publishers detect: [:each | each id = 2].
	self assert: publisher2 titles first = 'A book'.
!

testUpdateSimpleTypeOneToMany
	| publishers publisher |
	self assert: (session read: GlorpPublisher) isEmpty.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titles: (Array with: 'New Improved Book'), publisher titles].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: publisher titles size = 4.
	self assert: publisher titles asSortedCollection = (session system examplePublisher1 titles , (Array with: 'New Improved Book')) asSortedCollection.
! !

!GlorpStXDictionaryMappingTest methodsFor:'tests-simple reference dictionary'!

testSimpleTypeDictionary
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id = 1].
	publishers do: [:each |
		each titlesWithNumberInStock keysAndValuesDo: [:eachTitle :eachNumber |
			self assert: eachTitle isString.
			self assert: eachNumber = 2]].
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: publishers first titlesWithNumberInStock keys asSortedCollection = session system examplePublisher1 titles asSortedCollection.

"       publishers do: [:each |
		each titlesWithNumberInStockLink keysAndValuesDo: [:eachTitle :eachNumber |
			self assert: eachTitle isString.
			self assert: eachNumber = 2]].
	self assert: publishers first titlesWithNumberInStockLink size = 3.
	self assert: publishers first titlesWithNumberInStockLink keys asSortedCollection = session system examplePublisher1 titles asSortedCollection."
!

testSimpleTypeDictionaryComplexQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock sqlCount = 3].
	self assert: publishers size = 1.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer').

"       publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStockLink sqlCount > 0].
	publishers := publishers asSortedCollection: [:a :b | a name <= b name].
	self assert: publishers size = 2.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer')."
!

testSimpleTypeDictionaryQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:x | x key like: 'Master%']].
	self assert: publishers size = 1.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer').

"       publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:x | x value  = 2]].
	self assert: publishers size = 1.
	self assert: publishers first id = 1."
!

testSimpleTypeDictionaryQueryingWithFilteredReads
	| publishers publisherOne publisher2 |
	((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titlesWithNumberInStock) useFilteredRead.
	"((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titlesWithNumberInStockLink) useFilteredRead."

	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	publishers := session read: GlorpPublisher where: [:each | each id < 3].
	self assert: publishers size = 2.
	publisherOne := publishers detect: [:each | each id = 1].
	self assert: publisherOne titlesWithNumberInStock size = 3.
	self assert: (publisherOne titlesWithNumberInStock includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStock values asArray = #(2 2 2)).
"       self assert: publisherOne titlesWithNumberInStockLink size = 3.
	self assert: (publisherOne titlesWithNumberInStockLink includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStockLink values asArray = #(2 2 2)).
"
	session accessor denyReads.
	publisher2 := publishers detect: [:each | each id = 2].
	self assert: publisher2 titlesWithNumberInStock keys asArray first = 'A book'.
"       self assert: publisher2 titlesWithNumberInStockLink keys asArray first = 'A book'."
	self assert: (publisher2 titlesWithNumberInStock at: 'A book') = 200.
"       self assert: (publisher2 titlesWithNumberInStockLink at: 'A book') = 200."
!

testSimpleTypeDictionaryRead
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:eachTitleAssociation |
		eachTitleAssociation value = 2]].
	self assert: publishers size = 1.
!

testSimpleTypeDictionaryReadWithImpliedValue
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:eachTitleAssociation |
		eachTitleAssociation = 2]].
	self assert: publishers size = 1.
!

testUpdateSimpleTypeDictionaryAdding
	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStock at: 'New Book' put: 1].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStock at: 'New Book') = 1.
	self assert: publisher titlesWithNumberInStock size = 4.
!

testUpdateSimpleTypeDictionaryDeleting
	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStock removeKey: 'Mastering ENVY/Developer'].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer' ifAbsent: [999]) = 999.
	self assert: publisher titlesWithNumberInStock size = 2.
!

testUpdateSimpleTypeDictionaryLinkModifying
"       | publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer' put: 123].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer') = 123."
	"It can't find how to delete properly, because it doesn't know the primary key of the old record, and we don't have it recorded anywhere in the image"
	"self assert: (session accessor executeSQLString: 'SELECT * FROM PUBLISHER_TITLE2') size = 4.
	self assert: (session accessor executeSQLString: 'SELECT * FROM PUB_TITLE_LINK') size = 4."
!

testUpdateSimpleTypeDictionaryLinkedAdding
"       | publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink at: 'New Book' put: 1].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'New Book') = 1.
	self assert: publisher titlesWithNumberInStockLink size = 4."
!

testUpdateSimpleTypeDictionaryLinkedDeleting
"       | publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink removeKey: 'Mastering ENVY/Developer'].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer' ifAbsent: [999]) = 999.
	self assert: publisher titlesWithNumberInStockLink size = 2."
!

testUpdateSimpleTypeDictionaryModifying
        | publishers publisher |
        publishers := session system examplePublishers.
        session modify: publishers in: [].
        session reset.
        publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].

        session modify: publisher in: [
                publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer' put: 123].
        session reset.
        publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
        self assert: (publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer') = 123.
! !

!GlorpStXDictionaryMappingTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpStXDictionaryMappingTest.st,v 1.1 2006/07/26 09:09:44 boris Exp $'
! !
