"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpStXOptimisticLockingTest
	instanceVariableNames:'accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTests'
!


!GlorpStXOptimisticLockingTest methodsFor:'support'!

fillDatabase
  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'ALTER TABLE BOOK DROP CONSTRAINT BOOK_PK'.]
     ifError: [:ex | ex return: nil].

  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'ALTER TABLE BOOK DROP CONSTRAINT BOOK_UNIQ'.]
     ifError: [:ex | ex return: nil].

  accessor
     execute:
        [ accessor executeSQLStringNoResult: 'DROP TABLE BOOK'.]
     ifError: [:ex | ex return: nil].

     accessor execute:
       [accessor executeSQLStringNoResult: 
     'CREATE TABLE BOOK (ID serial NOT NULL,
      TITLE varchar(100) NULL,
      DESCRIPTION varchar(255) NULL ,
      COPIES_IN_STOCK integer NULL ,
      VERSION integer NULL ,
      CONSTRAINT BOOK_PK PRIMARY KEY  (ID),
      CONSTRAINT BOOK_UNIQ UNIQUE  (ID))
']
     ifError: [:ex | ex return: nil].
!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp

   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.

  session := GlorpSession new.
  session system: (system := GlorpLockingDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.

  self setUpDatabase.
 " self fillDatabase.  "

  session transact: [session register: session system example1].

  accessor clearAutocommit.  
!

setUpDatabase

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [GlorpTestDescriptorSystem glorpAllSubclasses do: 
                        [:eachSystemClass | 
                        self 
                                setUpSystem: (eachSystemClass forPlatform: accessor platform) setUpDefaults].

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: sys 
        | errorBlock |
        accessor isLoggedIn ifFalse: [self halt. accessor login].
        accessor dropTables: ((TableSorter for: sys allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        sys platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: sys allSequences.
                sys allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        sys allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        sys allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        sys allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
!

tearDown
	session rollbackTransaction.
	super tearDown.
! !

!GlorpStXOptimisticLockingTest methodsFor:'tests'!

changeABookFrom: books
	^session
		transact:
			[session register: books.
			books first copiesInStock: books first copiesInStock - 1].
!

testSuccessfulUpdate
        | books copies |  self halt.
        books := session readManyOf: GlorpBook.
        copies := books collect: [:each | each copiesInStock].
        session transact: [
                session register: books.
                books do: [:each | each copiesInStock: (each copiesInStock - 1)]].
        session reset.
        books := session readManyOf: GlorpBook orderBy: #id.
        self assert: (books collect: [:each | each copiesInStock]) = (copies collect: [:each | each - 1]).
!

testSuccessfulUpdateOntoNullVersionField
        | books copies someBook |   self halt.
        someBook := session readOneOf: GlorpBook.
        session accessor executeSQLString: 'UPDATE BOOK SET VERSION = NULL WHERE ID = ', someBook id printString.
        session reset.
        books := session read: GlorpBook orderBy: #id.
                copies := books collect: [:each | each copiesInStock].
        session transact: [
                session register: books.
                books do: [:each | each copiesInStock: (each copiesInStock - 1)]].
        session reset.
        books := session readManyOf: GlorpBook orderBy: #id.
        self assert: (books collect: [:each | each copiesInStock]) = (copies collect: [:each | each - 1]).
!

testUnsuccessfulUpdateProceedingAndFailing
        | books |      self halt.
        books := session readManyOf: GlorpBook.
        session accessor
                executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
        self should: [[self changeABookFrom: books] on: GlorpWriteFailure do: [:ex | ex resume: false]] raise: GlorpTransactionFailure.

        session reset.
        books := session readManyOf: GlorpBook.
        self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).
!

testUnsuccessfulUpdateProceedingAnyway
        | books |    self halt.
        books := session readManyOf: GlorpBook.
        session accessor
                executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
        [self changeABookFrom: books] on: GlorpWriteFailure do: [:ex | ex resume: true].

        session reset.
        books := session readManyOf: GlorpBook.
        self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).
!

testUnsuccessfulUpdateWithNoAction
        "This is a little quirky. SUnit's action in should:raise: is to exit
         the exception with true. So the thing just short circuits, and we
         don't get a transaction failure because we killed the whole unit of
         work. This is, in general, what will happen if we handle and don't
         resume the write failures."
        | books |     self halt.
        books := session readManyOf: GlorpBook.
        session accessor
                executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
        self shouldnt: [
                self should: [self changeABookFrom: books] raise: GlorpWriteFailure]
                raise: GlorpTransactionFailure.
        session reset.
        books := session readManyOf: GlorpBook.
        self assert: (books collect: [:each | each copiesInStock]) = #(7 7).
! !

!GlorpStXOptimisticLockingTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpStXOptimisticLockingTest.st,v 1.1 2006/07/26 09:19:53 boris Exp $'
! !
