"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpStXReadingQueryTest
	instanceVariableNames:'system session accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpStXReadingQueryTest methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.

  session := GlorpSession new.
  session system: (system := GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  self setUpDatabase.
  accessor clearAutocommit.

  session beginTransaction.
  self writeAddressDoubleOrderingRows.
  self writePersonOrderingRows.
!

tearDown
       " super tearDown.  "
        session rollbackTransaction.
        
        session reset.
        accessor logout. 
        session := nil.
        system := nil.
!

validateDoubleOrderFor: query
	| result sortedResult |
	result := session execute: query.
	sortedResult := result asSortedCollection:
					[:a :b |
					a street = b street
						ifTrue: [a number <= b number]
						ifFalse: [a street < b street]].
	^self assert: sortedResult asArray = result asArray
!

validateDoubleOrderMixedFor: query
	| result sortedResult |
	result := session execute: query.
	sortedResult := result asSortedCollection:
					[:a :b |
					a street = b street
						ifTrue: [a number <= b number]
						ifFalse: [a street > b street]].
	^self assert: sortedResult asArray = result asArray
!

verifyOrderFor: result byAttribute: aBlock
	1 to: result size - 1
		do:
			[:i |
			| a b |
			a := result at: i.
			b := result at: i + 1.
			self assert: (aBlock value: a) <= (aBlock value: b)]
!

writeAddressDoubleOrderingRows

	self writeAddressOrderingRows.
	session writeRow: session system exampleAddressRowForOrdering4.
	session writeRow: session system exampleAddressRowForOrdering5.
!

writeAddressOrderingRows

	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
!

writePersonOrderingRows

	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.
! !

!GlorpStXReadingQueryTest methodsFor:'table creation'!

setUpDatabase

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [GlorpTestDescriptorSystem glorpAllSubclasses do: 
                        [:eachSystemClass | 
                        self 
                                setUpSystem: (eachSystemClass forPlatform: accessor platform) setUpDefaults].

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: sys 
        | errorBlock |
        accessor isLoggedIn ifFalse: [accessor login].
        accessor dropTables: ((TableSorter for: sys allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        sys platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: sys allSequences.
                sys allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        sys allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        sys allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        sys allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
! !

!GlorpStXReadingQueryTest methodsFor:'tests'!

testCriteriaSetup
	| query |
	query := Query returningOneOf: GlorpAddress where: [:each | each id = 12].
	query session: session.
	query setUpExpressions.
	self assert: query whereClause class == RelationExpression.
	self assert: query whereClause ultimateBaseExpression descriptor
				== (session descriptorFor: GlorpAddress)
!

testIn
	| query result |

	query := Query returningManyOf: GlorpAddress where: [:each | each street in: #('Beta' 'Alpha')].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | #('Beta' 'Alpha') includes: each street]).
	self assert: result size = 4.
!

testInEmpty
	| query result |

	query := Query returningManyOf: GlorpAddress where: [:each | each street in: #()].
	result := session execute: query.
	self assert: result isEmpty.
!

testInInteger
	| query result |

	query := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | #(1 2) includes: each id]).
	self assert: result size = 2.
!

testInSymbol
	| query result transRow transRow2 |
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	transRow2 := session system exampleBankTransactionRow2.
	session writeRow: transRow2.
	query := Query returningManyOf: GlorpBankTransaction where: [:each | each amount currency in: #(#USD #CDN)].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each amount currency = #CDN]).
	self assert: result size = 1.
	query := Query returningManyOf: GlorpBankTransaction where: [:each | each amount currency in: #(#USD #DM)].
	result := session execute: query.
	self assert: result isEmpty.
!

testLike
	| query result |

	query := Query returningManyOf: GlorpAddress where: [:each | each street like: 'Be%'].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each street = 'Beta']).
	self assert: result size = 3.
!

testReadMultipleObjects

	| query result tracing addressReadSeparately personReadSeparately allResults |
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each ].
	tracing retrieve: [:each | each address].
	query tracing: tracing.
	query orderBy: #id.
	allResults := query executeIn: session.
	self assert: allResults size = 3.
	result := allResults first.
	self assert: (result at: 1) id = 86.
	self assert: ((result at: 1) class == GlorpPerson).
	self assert: (result at: 1) address == (result at: 2).
	self assert: ((result at: 2) class == GlorpAddress).
	self assert: ((result at: 2) id = 2).
	addressReadSeparately := session readOneOf: GlorpAddress where: [:each | each id = 2].
	self assert: result last == addressReadSeparately.
	personReadSeparately := session readOneOf: GlorpPerson where: [:each | each id = 86].
	self assert: personReadSeparately == result first.
	self assert: personReadSeparately address yourSelf == result last.
! !

!GlorpStXReadingQueryTest methodsFor:'tests-data reading'!

testReadDataItemsFromEmbeddedObject

	| query result transRow id |
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	id := transRow atFieldNamed: 'ID'.
	query := Query returningManyOf: GlorpBankTransaction.
	query retrieve: [:each | each id].
	query retrieve: [:each | each serviceCharge description].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = (Array with: id with: 'additional overcharge').
!

testReadDistinctIds

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id distinct].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).
!

testReadDistinctIdsWithWhereClause

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id distinct].
	query whereClause: [:each | each id ~= 423421].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).
!

testReadDistinctRelatedAttribute

	| query result allStreets |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each address street distinct].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #('Alpha' 'Beta' 'Gamma')).
	allStreets := (session readManyOf: GlorpAddress) collect: [:each | each street].
	self assert: allStreets size = 5.
!

testReadObjectsAndData

	| query result tracing addressReadSeparately personReadSeparately |
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each address].
	query tracing: tracing.
	query orderBy: #id.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: (result first at: 1) = 86.
	self assert: ((result first at: 2) class == GlorpAddress).
	self assert: ((result first at: 2) id = 2).
	addressReadSeparately := session readOneOf: GlorpAddress where: [:each | each id = 2].
	self assert: result first last == addressReadSeparately.
	personReadSeparately := session readOneOf: GlorpPerson where: [:each | each id = 86].
	self assert: personReadSeparately address yourSelf == result first last.
!

testReadOneWithObjects

	| query result |
	query := Query returningOneOf: GlorpPerson where: [:each | each id = 86].
	query retrieve: [:each | each].
	query retrieve: [:each | each address].
	query orderBy: [:each | each id].
	result := query executeIn: session.
	self assert: result first id = 86.
	self assert: (result first address yourSelf == result last).
!

testReadOnlyPrimaryKeys

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).
!

testReadTwoDataItems

	| query result tracing|
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each name].
	query tracing: tracing.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: result first = #(86 'person1').
	self assert: (result at: 2) = #(87 'person2').
	self assert: result last = #(88 'person3').
!

testReadTwoDataItemsFromDifferentObjects

	| query result tracing|
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each address street].
	query tracing: tracing.
	query orderBy: #id.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: result first = #(86 'Beta').
	self assert: (result at: 2) = #(87 'Gamma').
	self assert: result last = #(88 'Alpha').
!

testReadTwoObjects

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each].
	query retrieve: [:each | each address].
	query orderBy: [:each | each id].
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: (result first atIndex: 1) id = 86.
	self assert: ((result first atIndex: 1) address yourSelf == result first last).
! !

!GlorpStXReadingQueryTest methodsFor:'tests-ordering'!

helpTestSingleOrderingBy: aBlock setup: setupBlock

	| query result realBlock |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: aBlock.
	realBlock := query expressionBlockFor: aBlock.
	result := session execute: query.
	self verifyOrderFor: result byAttribute: realBlock.
!

testDescendingSingleOrdering
	| query result sortedResult |

	query := Query returningManyOf: GlorpAddress where: [:each | each street ~= 'Beta'].
	query orderBy: [:each | each street descending].
	result := session execute: query.
	sortedResult := result
				asSortedCollection: [:a :b | a street > b street].
	self assert: sortedResult asArray = result asArray.
!

testDoubleOrderingAddress
	| query |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: [:each | each street].
	query orderBy: [:each | each number].
	self validateDoubleOrderFor: query.
!

testMixedDoubleOrderingAddress
	| query |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: [:each | each street descending].
	query orderBy: [:each | each number].
	self validateDoubleOrderMixedFor: query.
!

testOrderingByRelatedObjectAttribute
	| query result |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each address street].
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].
!

testOrderingWithNonEmptyWhereClause

	| query result |
	query := Query returningManyOf: GlorpPerson where: [:each | each id ~= 12].
	query orderBy: #(#address #street).
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].
!

testSingleOrderingAddress1

	self
		helpTestSingleOrderingBy: [:each | each street]
		setup: [self writeAddressOrderingRows].
!

testSingleOrderingAddress2

	self
		helpTestSingleOrderingBy: [:each | each number]
		setup: [self writeAddressOrderingRows].
!

testSingleOrderingBySymbol

	self
		helpTestSingleOrderingBy: #street
		setup: [self writeAddressOrderingRows].
!

testSymbolsOrderingByRelatedObjectAttribute

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: #(#address #street).
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].
! !

!GlorpStXReadingQueryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpStXReadingQueryTest.st,v 1.1 2006/07/26 09:06:55 boris Exp $'
! !
