"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpTestODBCInterface
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest-ODBC'
!

!GlorpTestODBCInterface class methodsFor:'documentation'!

documentation

"
   The tests of this test case are designed to verify that
   NULL values are properly written and read.
"
! !

!GlorpTestODBCInterface methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
! !

!GlorpTestODBCInterface methodsFor:'tests'!

testIntegerNULL1
  " verify that an omitted nullable number value is stored and retrieved as NULL "

   | login accessor result |
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.
  accessor setAutocommit.

  accessor execute:
        [ accessor executeSQLString: 'ALTER TABLE TESTNULLS DROP CONSTRAINT TESTNULLS_PKEY'.]
        ifError: [:ex | ex return: nil].

  accessor execute:
        [ accessor executeSQLString: 'DROP TABLE TESTNULLS'.]
        ifError: [:ex | ex return: nil].
  accessor execute: 
        [accessor executeSQLString: 'CREATE TABLE TESTNULLS
   (ID integer NOT NULL,
    NUMVAL integer NULL,
    STRINGVAL varchar(20) NULL,
    CONSTRAINT TESTNULLS_PKEY PRIMARY KEY (ID)
   )']
        ifError: [:ex | self halt.  ex return: nil.]. 

  accessor execute: 
        [  accessor executeSQLString: 'INSERT INTO TESTNULLS (ID, NUMVAL, STRINGVAL)
  VALUES (1, 20, ''my_string'')']
        ifError: [:ex | ex return: nil.].
  accessor execute: 
        [  accessor executeSQLString: 'INSERT INTO TESTNULLS (ID, STRINGVAL)
  VALUES (2, ''my_string'')']
        ifError: [:ex | ex return: nil.].

  result := accessor executeSQLString: 'SELECT * FROM TESTNULLS WHERE NUMVAL IS NULL'.  

  accessor executeSQLString: 'DROP TABLE TESTNULLS'.

  accessor logout.
  self assert: result size = 1.
  self assert: (result first at: 2) isNil.
!

testIntegerNULL2
  " verify that an explicitely written numeric NULL value is stored and retrieved as NULL "

   | login accessor result |
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.
  accessor setAutocommit.

  accessor execute:
        [ accessor executeSQLString: 'ALTER TABLE TESTNULLS DROP CONSTRAINT TESTNULLS_PKEY'.]
        ifError: [:ex | ex return: nil].
  accessor execute:
        [ accessor executeSQLString: 'DROP TABLE TESTNULLS'.]
        ifError: [:ex | ex return: nil].

  accessor executeSQLString: 'CREATE TABLE TESTNULLS
   (ID integer NOT NULL,
    NUMVAL integer NULL,
    STRINGVAL varchar(20) NULL,
    CONSTRAINT TESTNULLS_PKEY PRIMARY KEY (ID)
   )'. 

  accessor executeSQLString: 'INSERT INTO TESTNULLS (ID, NUMVAL, STRINGVAL)
  VALUES (1, 20, ''my_string'')'.
  accessor executeSQLString: 'INSERT INTO TESTNULLS (ID, NUMVAL, STRINGVAL)
  VALUES (2, NULL, ''my_string'')'.

  result := accessor executeSQLString: 'SELECT * FROM TESTNULLS WHERE NUMVAL IS NULL'.  

  accessor executeSQLString: 'DROP TABLE TESTNULLS'.

  accessor logout.

  self assert: result size = 1.
  self assert: (result first at: 2) isNil.
!

testStringNULL1
  " verify that an omitted nullable string value is stored and retrieved as NULL "

   | login accessor result |
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.
  accessor setAutocommit.

  accessor execute:
        [ accessor executeSQLString: 'ALTER TABLE TESTNULLS DROP CONSTRAINT TESTNULLS_PKEY'.]
        ifError: [:ex | ex return: nil].

  accessor execute:
        [ accessor executeSQLString: 'DROP TABLE TESTNULLS'.]
        ifError: [:ex | ex return: nil].

  accessor executeSQLString: 'CREATE TABLE TESTNULLS
   (ID integer NOT NULL,
    NUMVAL integer NULL,
    STRINGVAL varchar(20) NULL,
    CONSTRAINT TESTNULLS_PKEY PRIMARY KEY (ID)
   )'. 

  accessor executeSQLString:
        'INSERT INTO TESTNULLS (ID, NUMVAL, STRINGVAL)
                VALUES (1, 20, ''my_string'')'.
  accessor executeSQLString:
        'INSERT INTO TESTNULLS (ID, NUMVAL)
                VALUES (2, 20)'.

  result := accessor executeSQLString:
        'SELECT * FROM TESTNULLS WHERE STRINGVAL IS NULL'.  

  accessor executeSQLString: 'DROP TABLE TESTNULLS'.

  accessor logout.
  self assert: result size = 1.
  self assert: (result first at: 3) isNil.
! !

!GlorpTestODBCInterface class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpTestODBCInterface.st,v 1.1 2006/07/26 09:05:19 boris Exp $'
! !
