"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpTimedProxyReaperTest
	instanceVariableNames:'reaper'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpTimedProxyReaperTest methodsFor:'support'!

setUp
	Dialect isVisualAge ifTrue: [^self].
	Dialect isObjectStudio ifTrue: [^self].

	reaper := TimedProxyReaper new.
	reaper delay: 0.5.
!

tearDown

	reaper release.
	reaper := nil.
! !

!GlorpTimedProxyReaperTest methodsFor:'tests'!

README

	"Unfortunately, these tests are timing sensitive. They may intermittently fail, when in fact they are working as advertised.
	Rerun failed tests individually. If a test consistently fails, then it likely indicates a bug."
!

testAddObjects

	| proxies |
	Dialect isVisualAge ifTrue: [^self].
	Dialect isObjectStudio ifTrue: [^self].
	proxies := OrderedCollection new.
	1 to: 10 do: [:index |
		proxies add: (TimedProxy returningOneOf: Object where: nil in: nil).
		proxies last glorpSecondsToLive: 1000.
		reaper add: proxies last].

	(Delay forSeconds: 1) wait.
	proxies do: [:proxy | self should: [reaper includes: proxy]].
!

testExpireObjects

	| proxies |
	Dialect isVisualAge ifTrue: [^self].
	Dialect isObjectStudio ifTrue: [^self].

	proxies := OrderedCollection new.
	1 to: 10 do: [:index |
		proxies add: (TimedProxy returningOneOf: Object where: nil in: nil).
		proxies last glorpSecondsToLive: 1.
		reaper add: proxies last].

	(Delay forSeconds: 5) wait.
	proxies do: [:proxy | self shouldnt: [reaper includes: proxy]].
!

testTouchAndExpireObjectsWhileOtherObjectsAreBeingReaped

	| expiringProxy proxy |
	Dialect isVisualAge ifTrue: [^self].
	Dialect isObjectStudio ifTrue: [^self].

	reaper delay: 0.25.
	expiringProxy := TimedProxy returningOneOf: Object where: nil in: nil.
	expiringProxy glorpSecondsToLive: 1.
	proxy := TimedProxy returningOneOf: Object where: nil in: nil.
	proxy glorpSecondsToLive: 2.

	reaper add: proxy; add: expiringProxy.
	self assert: (reaper includes: proxy).
	self assert: (reaper includes: expiringProxy).
	(Delay forSeconds: 1) wait.
	proxy glorpTouch.
	(Delay forSeconds: 1.5) wait.
	self assert: (reaper includes: proxy).
	self deny: (reaper includes: expiringProxy).
	(Delay forSeconds: 2) wait.
	self deny: (reaper includes: proxy).
!

testTouchObjects

	| proxy |
	Dialect isVisualAge ifTrue: [^self].
	Dialect isObjectStudio ifTrue: [^self].

	reaper delay: 0.25.
	proxy := TimedProxy returningOneOf: Object where: nil in: nil.
	proxy glorpSecondsToLive: 2.
	reaper add: proxy.
	(Delay forSeconds: 1) wait.
	proxy glorpTouch.
	(Delay forSeconds: 1.1) wait.
	self should: [reaper includes: proxy].
	(Delay forSeconds: 2) wait.
	self shouldnt: [reaper includes: proxy]
! !

!GlorpTimedProxyReaperTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpTimedProxyReaperTest.st,v 1.1 2006/07/26 08:57:06 boris Exp $'
! !
