"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpTutorialTests
	instanceVariableNames:'system session accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTutorial'
!


!GlorpTutorialTests methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor loginIfError:
        [:ex | (ex class == SQL::ODBCError and: [ex  state = '08001'])
                ifTrue: ["unable to connect. do something here"].
        ].
  Transcript show: 'login completed'; cr.
  session := GlorpSession new.
  session system: (system := GlorpTutorialExampleDescriptor forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  self setUpDatabase.
  accessor clearAutocommit.
!

tearDown
       " super tearDown.  "
        session reset.
        accessor logout. 
        session := nil.
        system := nil.
! !

!GlorpTutorialTests methodsFor:'table creation'!

setUpDatabase
             Transcript show: 'setup database'; cr.
        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [self setUpSystem: (GlorpTutorialExampleDescriptor forPlatform: accessor platform) setUpDefaults.

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
  Transcript show: 'setup database curtailed'; cr.
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: sys 
        | errorBlock |
        accessor isLoggedIn ifFalse: [accessor login].  
        accessor dropTables: ((TableSorter for: sys allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        sys platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: sys allSequences.
                sys allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        sys allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        sys allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        sys allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
! !

!GlorpTutorialTests methodsFor:'tests'!

test1
    | instance result |
 session beginUnitOfWork.
  instance := OpenSourceProject new
                title: 'GLORP';
                webpage: 'www.glorp.org';
                projectManager:
                  (GTPerson new
                       name: 'Knight';
                       givenName: 'Alan';
                       affiliation: 
                         (Company new
                           shortName: 'Cincom';
                           fullName: 'Cincom Systems';
                           webpage: 'www.cincom.com'
                         )
                   ). 
  session register: instance.
  session commitUnitOfWork.

  result := session readManyOf: OpenSourceProject
           where: [:each | each projectManager name = 'Knight'].
  self assert: result size = 1.
  self assert: (result first isKindOf: OpenSourceProject).
!

test2
    | instance result |
 session beginUnitOfWork.
  instance :=  
             Company new
                           shortName: 'eXept';
                           fullName: 'Exept software ag';
                           webpage: 'www.exept.com'. 
  session register: instance.
  session commitUnitOfWork.

  result := session readManyOf: Company
                    where: [:record | record shortName = 'eXept'].

  self assert: result size = 1.
  self assert: (result first isKindOf: Company).
!

test3
    | instance result |
 session beginUnitOfWork.
  instance := OpenSourceProject new
                title: 'GLORP';
                webpage: 'www.glorp.org'. 

  session register: instance.
  session commitUnitOfWork.

  result := session readManyOf: OpenSourceProject
           where: [:each | each title = 'GLORP'].
  self assert: result size = 1.
  self assert: (result first isKindOf: OpenSourceProject).
!

test4
    | instance result record |
 session beginUnitOfWork.
  instance := OpenSourceProject new
                title: 'GLORP';
                webpage: 'www.glorp.org';
                projectManager:
                  (GTPerson new
                       name: 'Knight';
                       givenName: 'Alan';
                       affiliation: 
                         (Company new
                           shortName: 'Cincom';
                           fullName: 'Cincom Systems';
                           webpage: 'www.cincom.com'
                         )
                   ). 
  session register: instance.
  session commitUnitOfWork.

  result := session readManyOf: OpenSourceProject
           where: [:each | each projectManager name = 'Knight'].
  self assert: result size = 1.
  self assert: (result first isKindOf: OpenSourceProject).

  session beginUnitOfWork.
  record :=
     session readOneOf: OpenSourceProject
             where: [:each | each title = 'GLORP'
                             & (each projectManager affiliation shortName = 'Cincom')
                    ].
  record projectManager
     affiliation:
       (Company new
          shortName: 'eXept';
          fullName: 'eXept software ag';
          webpage: 'www.exept.de'
       ).
  session commitUnitOfWork.

  result := session readManyOf: OpenSourceProject
           where: [:each | each projectManager affiliation shortName = 'Cincom'].
  self assert: result size = 0.

  result := session readManyOf: OpenSourceProject
           where: [:each | each projectManager affiliation shortName = 'eXept'].
  self assert: result size = 1.

  self assert: (result first isKindOf: OpenSourceProject).
! !

!GlorpTutorialTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpTutorialTests.st,v 1.1 2006/07/26 09:17:21 boris Exp $'
! !
