"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpTutorialTests2
	instanceVariableNames:'system session accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTutorial'
!


!GlorpTutorialTests2 methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor loginIfError:
        [:ex | (ex class == SQL::ODBCError and: [ex  state = '08001'])
                ifTrue: ["unable to connect. do something here"].
        ].
  Transcript show: 'login completed'; cr.
  session := GlorpSession new.
  session system: (system := GTEmbeddedValueExampleDescriptor forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  self setUpDatabase.
  accessor clearAutocommit.
!

tearDown
       " super tearDown.  "
        session reset.
        accessor logout. 
        session := nil.
        system := nil.
! !

!GlorpTutorialTests2 methodsFor:'table creation'!

setUpDatabase
             Transcript show: 'setup database'; cr.
        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [self setUpSystem: (GTEmbeddedValueExampleDescriptor forPlatform: accessor platform) setUpDefaults.

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
  Transcript show: 'setup database curtailed'; cr.
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: sys 
        | errorBlock |
        accessor isLoggedIn ifFalse: [accessor login].  
        accessor dropTables: ((TableSorter for: sys allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        sys platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: sys allSequences.
                sys allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        sys allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        sys allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        sys allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
! !

!GlorpTutorialTests2 methodsFor:'tests'!

test1
    | curve result |
 session beginUnitOfWork.
  curve := PlanarCurve new.
  curve addPoint: 12@3;
        addPoint: 4@76;
        addPoint: 3@8.
  session register: curve.
  session commitUnitOfWork.

  result := session readManyOf: PlanarCurve.
  self assert: result size = 1.
  self assert: (result first isKindOf: PlanarCurve).
  self assert: result first points size = 3.
!

test2
    | curve1 curve2 result |
 session beginUnitOfWork.
  curve1 := PlanarCurve new.
  curve1 addPoint: 12@3;
        addPoint: 4@76;
        addPoint: 3@8.
  curve2 := PlanarCurve new.
  curve2 addPoint: 12@3;
        addPoint: 4@76;
        addPoint: 3@8;
        addPoint: 76@88.

  session register: curve1;
          register: curve2.
  session commitUnitOfWork.

  result := session readManyOf: PlanarCurve.
  self assert: result size = 2.
  self assert: (result allSatisfy: [:item | item isKindOf: PlanarCurve]).
!

test3
  " check that duplicate points are stored "

    | curve result fetchedCurve |
 session beginUnitOfWork.
  curve := PlanarCurve new.
  curve addPoint: 12@3;
        addPoint: 4@76;
        addPoint: 3@8;
        addPoint: 12@3.
  session register: curve.
  session commitUnitOfWork.

  result := session readManyOf: PlanarCurve.
  self assert: result size = 1.
  self assert: (result first isKindOf: PlanarCurve).
  self assert: result first points size = 4.
  fetchedCurve := result first.
  self assert: fetchedCurve points first point = fetchedCurve points last point.
!

test4
  " check that duplicate points are stored "

    | curve result fetchedCurve p1 p2 p3|
 session beginUnitOfWork.
  curve := PlanarCurve new.
  p1 := CurvePoint fromPoint:12@3.
  p2 := CurvePoint fromPoint:4@76.
  p3 := CurvePoint fromPoint:3@8.
  curve addCurvePoint: p1;
        addCurvePoint: p2;
        addCurvePoint: p3;
        addCurvePoint: p1.
  session register: curve.
  session commitUnitOfWork.

  result := session readManyOf: PlanarCurve.
  self assert: result size = 1.
  self assert: (result first isKindOf: PlanarCurve).
  self assert: result first points size = 4.
  fetchedCurve := result first.
  self assert: fetchedCurve points first point = fetchedCurve points last point.
  self assert: fetchedCurve points first point == fetchedCurve points last point.
! !

!GlorpTutorialTests2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpTutorialTests2.st,v 1.1 2006/07/26 09:15:13 boris Exp $'
! !
