"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpUnitOfWorkTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest'
!


!GlorpUnitOfWorkTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpUnitOfWorkTest class methodsFor:'resources'!

resources

	^Array with: GlorpSessionResource.
! !

!GlorpUnitOfWorkTest methodsFor:'support'!

exampleCustomerProxy
	| p |
	p := Proxy new.
	p session: session.
	p query: (GlorpQueryStub returningOneOf: GlorpCustomer where: [:a | a id = 3]).
	p query result: (GlorpCustomer new id: 3).
	^p
!

exampleCustomerWithTransactionsProxy
	| customer |
	customer := GlorpCustomer new.
	customer transactions: self exampleTransactionsProxy.
	^customer
!

exampleTransactionWithCustomerProxy
	| transaction |
	transaction := GlorpBankTransaction example1.
	transaction owner: self exampleCustomerProxy.
	^transaction
!

exampleTransactionsProxy
	| p |
	p := Proxy new.
	p session: session.
	p query: (GlorpQueryStub returningOneOf: GlorpBankTransaction where: [:a | a id ~= 0]).
	p query result: (Array with: GlorpBankTransaction example1 with: GlorpBankTransaction example1).
	^p
!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
!

tearDown
	super tearDown.
	session reset.
	session := nil.
! !

!GlorpUnitOfWorkTest methodsFor:'tests'!

testAutomaticRegistrationOnRead
	| p c |
	p := self exampleCustomerProxy.
	c := p getValue.
	session beginUnitOfWork.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: c).
!

testCheckIfInstantiationRequiredForDirectMapping
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	c1 name: proxy.
	session beginUnitOfWork.
	session register: c1.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #name.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).
!

testCheckIfInstantiationRequiredForRelationshipInstantiatedProxy
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy query: (GlorpQueryStub new session: session; result: 'foo').
	proxy session: session.
	proxy yourSelf.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session register: c1.
	c1 accounts: #().
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).
!

testCheckIfInstantiationRequiredForRelationshipNoChange
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session registerAsOld: c1.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).
!

testCheckIfInstantiationRequiredForRelationshipNoProxy
	| c1 mapping |
	c1 := GlorpCustomer new.
	c1 accounts: #().
	session beginUnitOfWork.
	session register: c1.
	c1 accounts: nil.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).
!

testCheckIfInstantiationRequiredForRelationshipWithChange
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy session: session.
	proxy query: (GlorpQueryStub new result: 'foo').
	c1 accounts: proxy.
	session beginUnitOfWork.
	session registerAsOld: c1.
	c1 accounts: #().
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self assert: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).
!

testCheckIfInstantiationRequiredForToManyRelationshipFromNew
	| c1 proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy session: session.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session registerAsNew: c1.
	self assert: (proxy isInstantiated).
!

testCommitOrderAtSessionLevel

	| tables |
	tables := session tablesInCommitOrder.
	tables first name = 'CUSTOMER'.
	self unfinished.
!

testOriginalValueFor
	| c1 mapping |
	c1 := GlorpCustomer new.
	c1 name: 'fred'.
	session beginUnitOfWork.
	session register: c1.
	c1 name: 'barney'.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #name.
	self assert: (session privateGetCurrentUnitOfWork originalValueFor: c1 mapping: mapping) = 'fred'.
!

testRollbackOneToManyAfterAdd
	| customer t1 t2 transList amount |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	amount := t1 amount.
	session beginUnitOfWork.
	session register: customer.
	20 timesRepeat: [customer addTransaction: (GlorpBankTransaction new)].
	customer transactions first amount: 65543.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.
	self assert: t1 amount == amount.
!

testRollbackOneToManyAfterReplace
	| customer t1 t2 transList |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	session beginUnitOfWork.
	session register: customer.
	customer transactions: OrderedCollection new.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.
!

testRollbackOneToManyProxy
	| customer t1 t2 transList |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	session beginUnitOfWork.
	session register: customer.
	customer transactions: OrderedCollection new.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.
!

testRollbackOneToManyWithList
	"Check that dependents aren't being registered for the collection"
	| customer marker |
	"Lists only exist in VW"
	Dialect isVisualWorks ifFalse: [^self].
	marker := Object new.
	customer := GlorpCustomer example1.
	customer transactions: (customer transactions asList).
	customer transactions addDependent: marker.
	session beginUnitOfWork.
	session register: customer.
	20 timesRepeat: [customer addTransaction: (GlorpBankTransaction new)].
	session rollbackUnitOfWork.
	self assert: customer transactions class == (Dialect smalltalkAt: #List).
	self assert: customer transactions size = 2.
	self should: [customer transactions privateAt: 3] raise: Object subscriptOutOfBoundsSignal.
	self assert: (customer transactions dependents includes: marker).
	self deny: (session isRegistered: marker).
!

testRollbackOneToOne
	| transaction customer |
	transaction := GlorpBankTransaction new.
	customer := GlorpCustomer new.
	transaction owner: customer.
	session beginUnitOfWork.
	session register: transaction.
	transaction owner: GlorpCustomer new.
	session rollbackUnitOfWork.
	self assert: transaction owner == customer.
!

testRollbackOneToOneWithProxy
	| transaction customerProxy |
	transaction := self exampleTransactionWithCustomerProxy.
	customerProxy := transaction owner.
	session beginUnitOfWork.
	session register: transaction.
	transaction owner: GlorpCustomer new.
	session rollbackUnitOfWork.
	self assert: transaction owner == customerProxy
!

testWriteObjectWithNilCollection
	| c1 query customer |
	c1 := GlorpCustomer new.
	c1 transactions: nil.
	c1 id: 9999.
	[session beginTransaction.
	session beginUnitOfWork.
	session register: c1.
	session commitUnitOfWork.
	query := Query returningOneOf: GlorpCustomer where: [:each | each id = 9999].
	query shouldRefresh: true.
	customer := session execute: query.
	self assert: customer transactions notNil.
	self assert: customer transactions isEmpty.
	]
		ensure: [session rollbackTransaction].
! !

!GlorpUnitOfWorkTest methodsFor:'tests-registration'!

testPostRegister
	| c1 t1 t2 |
	c1 := GlorpCustomer example2.
	[session beginTransaction.
	session beginUnitOfWork.
	t1 := GlorpBankTransaction new.
	t2 := GlorpBankTransaction new.
	c1 addTransaction: t1.
	c1 addTransaction: t2.
	session register: c1.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: t1).
	self assert: (session isRegistered: t2).
	session commitUnitOfWork]
		ensure: [session rollbackTransaction].
	"Need some assertions on what was written"
	self unfinished.
!

testPreRegister

	| c1 t1 t2 trans |
	c1 := GlorpCustomer example2.
	[session beginTransaction.
	session beginUnitOfWork.
	session register: c1.
	t1 := GlorpBankTransaction new.
	t2 := GlorpBankTransaction new.
	c1 addTransaction: t1.
	c1 addTransaction: t2.
	trans := session privateGetCurrentUnitOfWork privateGetTransaction.
	session commitUnitOfWork.
	self assert: (trans isRegistered: c1).
	self assert: (trans isRegistered: t1).
	self assert: (trans isRegistered: t2).]
		ensure: [session rollbackTransaction].
	"Need some assertions on what got written"
	self unfinished.
!

testRegisterAll
	| c1 c2 collection |
	c1 := GlorpCustomer new.
	c2 := GlorpCustomer new.
	session beginUnitOfWork.
	collection := Array with: c1 with: c2.
	session registerAll: collection.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: c2).
	self deny: (session isRegistered: collection).
!

testRegisterCollection
	| c1 c2 collection |
	c1 := GlorpCustomer new.
	c2 := GlorpCustomer new.
	session beginUnitOfWork.
	collection := Array with: c1 with: c2.
	session register: collection.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: collection).
!

testRegisterEmbeddedInTargetDontRefetch
	" This tests for a problem with embedded objects suddenly becoming nil. This can happen when an object is part of a to-many relation. If the relation is followed during the registering phase the query will wrongly replace the embedded object even though requiresPopulating is false. When the object itself is already registered the replaced embedded object won't be registered."

	| transactionId customer |

	[session beginTransaction.
	session inUnitOfWorkDo:
			[customer := GlorpCustomer example1.
			customer id: 42.
			session register: customer].
	transactionId := customer transactions first id.
	customer := nil.
	session initializeCache.
	session inUnitOfWorkDo:
			[| transaction amount |
			transaction := session readOneOf: GlorpBankTransaction
						where: [:each | each id = transactionId].
			amount := transaction amount.
			transaction owner transactions getValue.
			self assert: amount == transaction amount]]
			ensure: [session rollbackTransaction]
!

testRegisterExistingCollection
	| c1 |
	c1 := GlorpCustomer new.
	session beginUnitOfWork.
	session register: c1.
	session register: c1 transactions.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: c1 transactions).
	self deny: (session isNew: c1 transactions).
!

testRegisterInstantiatedProxy
	| p c |
	p := self exampleCustomerProxy.
	c := p getValue.
	session beginUnitOfWork.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: c).
!

testRegisterObjectWithCollectionProxyThenInstantiate
	| customer transactions |
	customer := self exampleCustomerWithTransactionsProxy.
	session beginUnitOfWork.
	session registerAsOld: customer.
	self deny: customer transactions isInstantiated.
	transactions := customer transactions getValue.
	self assert: customer transactions isInstantiated.
	session register: transactions.
	self assert: (session isRegistered: transactions first).
	self assert: (session isRegistered: customer).
	self assert: (session isRegistered: transactions).
	self assert: (session isRegistered: customer transactions).
!

testRegisterObjectWithInstantiatedProxy
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	customer := transaction owner getValue.
	session beginUnitOfWork.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: customer).
	self assert: transaction owner isInstantiated.
!

testRegisterObjectWithNilCollection
	| c1 |
	c1 := GlorpCustomer new.
	c1 transactions: nil.
	session beginUnitOfWork.
	session register: c1.
	self assert: (session isRegistered: c1).
!

testRegisterObjectWithProxy
	| transaction |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session registerAsOld: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: transaction).
	self deny: transaction owner isInstantiated.
!

testRegisterObjectWithProxyThenInstantiate
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session registerAsOld: transaction.
	self deny: transaction owner isInstantiated.
	customer := transaction owner getValue.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: customer).
	self assert: transaction owner isInstantiated
!

testRegisterObjectWithProxyThenInstantiateAndReregister
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session register: transaction.
	customer := transaction owner getValue.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: customer).
	self assert: transaction owner isInstantiated
!

testRegisterProxy
	| p |
	p := self exampleCustomerProxy.
	session beginUnitOfWork.
	session register: p.
	self deny: (session isRegistered: p).
	self deny: (session isRegistered: p query result).
	p getValue.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: p query result).
!

testRegisterProxyThenInstantiateAndReregister
	| p c |
	p := self exampleCustomerProxy.
	session beginUnitOfWork.
	session register: p.
	c := p getValue.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction
				isRegistered: c)
! !

!GlorpUnitOfWorkTest methodsFor:'tests-transaction wrappers'!

testInTransactionDoSuccessful
	"This has to test that a transaction completed successfully, so unlike most other tests, we have to clean up the evidence afterwards"

	| result endResult |
	[session inTransactionDo: [
		session writeRow: session system exampleAddressRow].
	result := session readManyOf: GlorpAddress.
	self assert: result size = 1.
	self assert: result first id = 123]
		ensure: [session inTransactionDo: [session accessor executeSQLStringNoResult: 'DELETE FROM GR_ADDRESS']].
	endResult := session readManyOf: GlorpAddress.
	self assert: endResult size = 0.
!

testInTransactionDoUnsuccessful

	| result initial|
	initial := session readManyOf: GlorpAddress.
	self assert: initial isEmpty.
	[session inTransactionDo: [
		session writeRow: session system exampleAddressRow.
		self error: 'no you don''t']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpAddress.
	self assert: result size = 0.
!

testTransactSuccessful
	"This has to test that a transaction completed successfully, so unlike most other tests, we have to clean up the evidence afterwards"

	| result |
	[session transact: [
		session register: (GlorpReservation new id: 345)].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 1.
	self assert: result first id = 345]
		ensure: [session transact: [session accessor executeSQLStringNoResult: 'DELETE FROM RESERVATION']].
!

testTransactUnsuccessful

	| result |
	[session transact: [
		session register: (GlorpReservation new id: 345).
		self error: 'didn''t work']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 0.
!

testinUnitOfWorkSuccessful

	| result |
	[session beginTransaction.
	session inUnitOfWorkDo: [
		session register: (GlorpReservation new id: 345)].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 1.
	self assert: result first id = 345]
		ensure: [session rollbackTransaction].
!

testinUnitOfWorkUnsuccessful

	| result |
	[session beginTransaction.
	[session inUnitOfWorkDo: [
		session register: (GlorpReservation new id: 345).
		self error: 'aaaagh']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 0]
		ensure: [session rollbackTransaction].
! !

!GlorpUnitOfWorkTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpUnitOfWorkTest.st,v 1.1 2006/07/26 09:21:13 boris Exp $'
! !
