"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#GlorpVirtualCollectionBasicTest
	instanceVariableNames:'vc'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTests'
!


!GlorpVirtualCollectionBasicTest methodsFor:'support'!

realObjects
	"Get the real objects from a virtual collection without resorting to any of its mechanisms except do:, so we can validate against more complex things."
	^self realObjectsFrom: vc.
!

realObjectsFrom: aVirtualCollection
	"Get the real objects from a virtual collection without resorting to any of its mechanisms except do:, so we can validate against more complex things."
	| all |
	all := OrderedCollection new.
	aVirtualCollection do: [:each | all add: each].
	^all.
!

setUp
	super setUp.
	session beginTransaction.
	self writePersonRows.
	vc := session virtualCollectionOf: GlorpPerson.
!

tearDown
	session rollbackTransaction.
	super tearDown.
!

writePersonRows
	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.
! !

!GlorpVirtualCollectionBasicTest methodsFor:'tests'!

testCollect
	| ids |
	ids := vc collect: [:each | each id].

	self assert: ids size = 3.
	ids do: [:each | self assert: each isInteger].
!

testCreation

	self assert: vc notNil.
	self should: [vc isKindOf: GlorpVirtualCollection].
!

testDo

	| all |
	all := OrderedCollection new.
	vc do: [:each |
		self assert: (each isKindOf: GlorpPerson).
		all add: each.
		].
	self assert: all size = 3.
	self assert: all asSet size = 3.
	self assert: (all collect: [:each | each id]) asSortedCollection asArray = #(86 87 88).
!

testInject
	| sumofIds |
	sumofIds := vc inject: 0 into: [:sum :each | sum + each id].
	self assert: sumofIds = (86 + 87 + 88).
!

testIsEmpty
	| vc2 |
	self deny: vc isEmpty.
	vc2 := vc select: [:each | each id = 98].
	self assert: vc2 isEmpty.
!

testReject
	| vc2 |
	vc2 := vc reject: [:each | each id > 87 ].
	self deny: vc isInstantiated.
	self deny: vc2 isInstantiated.
	self assert: vc2 size = 2.
	self deny: vc isInstantiated.
	self assert: vc size = 3.
	self assert: (self realObjectsFrom: vc2) size = 2.
!

testSelect
	| vc2 |
	vc2 := vc select: [:each | each id <= 87 ].
	self deny: vc isInstantiated.
	self deny: vc2 isInstantiated.
	self assert: vc2 size = 2.
	self deny: vc isInstantiated.
	self assert: vc size = 3.
	self assert: (self realObjectsFrom: vc2) size = 2.
! !

!GlorpVirtualCollectionBasicTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpVirtualCollectionBasicTest.st,v 1.1 2006/07/26 09:09:02 boris Exp $'
! !
