"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpCacheTest subclass:#GlorpWeakCacheTest
	instanceVariableNames:'mourned'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTests'
!


!GlorpWeakCacheTest class methodsFor:'instance creation'!

new
	"Answer a newly created and initialized instance."

	^super new initialize
! !

!GlorpWeakCacheTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpWeakCacheTest methodsFor:'initializing'!

initialize
! !

!GlorpWeakCacheTest methodsFor:'support'!

checkCacheExhaustivelyFor: aBlock
	"Check to make sure the cache satisfies the criteria. Since this relies on unpredictable finalization, do a full 10 garbage collect then wait iterations. Don't return early, because we're checking to see that finalization *doesn't* happen"
	self assert: (self doesCacheExhaustivelySatisfy: aBlock).
!

checkCacheFor: aBlock
	"Check to make sure the cache satisfies the criteria. Since this relies on unpredictable finalization, do up to 10 garbage collect then wait iterations. If it's true before that, return early, but if it's not true at the end, fail"
	self assert: (self doesCacheSatisfy: aBlock).
!

doesCacheExhaustivelySatisfy: aBlock
	| result |
	result := false.
	10
		timesRepeat:
			[Dialect garbageCollect.
			(Delay forMilliseconds: 100) wait.
			result := aBlock value].
	^result.
!

doesCacheSatisfy: aBlock
	10
		timesRepeat:
			[Dialect garbageCollect.
			(Delay forMilliseconds: 100) wait.
			aBlock value ifTrue: [^true]].
	^false.
!

mournKeyOf: anEphemeron
	mourned := true.
!

setUp
	super setUp.
	system cachePolicy: WeakVWCachePolicy new.
	mourned := false.
! !

!GlorpWeakCacheTest methodsFor:'tests'!

testEphemeralValue
	| value ephemeron |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	ephemeron := (Dialect smalltalkAt: #EphemeralValue) key: 'abc' value: value.
	ephemeron manager: self.
	Dialect garbageCollect.
	value := nil.
	self should: [10 timesRepeat: [
		mourned ifFalse: [Dialect garbageCollect.
		(Delay forMilliseconds: 100) wait]].
		mourned].
!

testEphemeralValueDictionary
	| value dict done |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	dict := WeakVWCachePolicy new dictionaryClass new.
	dict at: 'abc' put: value.
	Dialect garbageCollect.
	value := nil.
	done := false.
	self should:
		[10     timesRepeat:
			[done ifFalse: [
				Dialect garbageCollect.
				(Delay forMilliseconds: 100) wait.
				done := dict size = 0]].
			 done].
!

testEphemeron
	| value ephemeron |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	ephemeron := (Dialect smalltalkAt: #Ephemeron) key: value value: 'abc'.
	ephemeron manager: self.
	Dialect garbageCollect.
	value := nil.
	self should: [10 timesRepeat: [
		mourned ifFalse: [Dialect garbageCollect.
		(Delay forMilliseconds: 100) wait]].
		mourned].
!

testEphemeronDictionary
	| value dict done |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	dict := (Dialect smalltalkAt: #EphemeronDictionary) new.
	dict at: value put: 'abc'.
	Dialect garbageCollect.
	value := nil.
	done := false.
	self should:
		[10     timesRepeat:
			[done ifFalse: [
				Dialect garbageCollect.
				(Delay forMilliseconds: 100) wait.
				done := dict size = 0]].
			 done].
!

testLargeWeakDictionary
	| elements size customers newElements |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	size := 2 raisedTo: 14.
	customers := OrderedCollection new: size.
	1 to: size do: [:i | | cust |
		cust := GlorpCustomer new.
		customers add: cust.
		cache at: i insert: cust].
	elements := cache numberOfElements.
	self assert: elements = (2 raisedTo: 14).
	customers := nil.
	Dialect garbageCollect.
	(Delay forSeconds: 2) wait.
	Dialect garbageCollect.
	(Delay forSeconds: 1) wait.
	newElements := cache numberOfElements.
	self assert: newElements = 1.
!

testUnreferencedExcessObjectsAreRemoved
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	cache at: 3 insert: GlorpCustomer new.
	cache at: 4 insert: GlorpCustomer new.
	self checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 3) not].
	self assert: (cache containsObjectForClass: GlorpCustomer key: 4).
!

testUnreferencedObjectsAreRemoved

	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 0.
	cache at: 3 insert: GlorpCustomer new.
	self
		checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 3) not].
!

testUnreferencedObjectsAreRemovedInTheRightOrder
	| customer |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	cache at: 3 insert: GlorpCustomer new.
	cache at: 4 insert: GlorpCustomer new.
	customer := cache lookupClass: GlorpCustomer key: 3.
	self deny: customer isNil.
	cache at: 3 insert: customer.
	self checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 4) not].
	self assert: (cache containsObjectForClass: GlorpCustomer key: 3).
!

testUnreferencedObjectsNotRemovedDueToExtraReferences
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	cache at: 3 insert: GlorpCustomer new.
	self checkCacheExhaustivelyFor: [(cache containsObjectForClass: GlorpCustomer key: 3)].
! !

!GlorpWeakCacheTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpWeakCacheTest.st,v 1.1 2006/07/26 09:16:14 boris Exp $'
! !
