"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#GlorpWritingTest
	instanceVariableNames:'session'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!GlorpWritingTest class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!GlorpWritingTest methodsFor:'support'!

setUp
	super setUp.
	session := GlorpSessionResource current newSession.
!

tearDown
	super tearDown.
	session reset.
	session := nil.
! !

!GlorpWritingTest methodsFor:'tests'!

testLargeWrite
	"This is really just a performance test. No assertions. It should take on the order of 10 seconds. If it starts taking 10 minutes, it indicates a potential problem...."
	| customer |
	session beginTransaction.
	[customer := GlorpCustomer new.
	customer name: 'foo'.
     customer id: 123.
	1 to: 5000 do: [:i | | transaction |
		transaction := GlorpBankTransaction new.
		transaction amount amount: i.
		customer addTransaction: transaction].
	session beginUnitOfWork.
	session register: customer.
	session commitUnitOfWork]
		ensure: [session rollbackTransaction].
!

testMultipleWritesAndReads
	"Trying to narrow down on a problem with SQLiteEXDI"

	| names customers |
	session beginTransaction.
	[session beginUnitOfWork.
	names := OrderedCollection new.
	1 to: 10
		do:
			[:i |
			| nm |
			nm := 'John Doe ' , i printString.
			names add: nm.
			session register: ((GlorpCustomer new)
									id: i;
									name: nm;
									yourself)].
	session commitUnitOfWork.
	customers := session readManyOf: GlorpCustomer.
	self assert: (names asSortedCollection isSameSequenceAs: (customers collect: [:ea | ea name]) asSortedCollection).
	]
			ensure: [session rollbackTransaction]
!

testRegistrationFromWrittenObject
	| customer trans transactions |
	session beginTransaction.
	[customer := GlorpCustomer new.
	customer name: 'foo'.
     customer id: 123.
	session beginUnitOfWork.
	session register: customer.
	session commitUnitOfWork.
	trans := GlorpBankTransaction new.
	session beginUnitOfWork.
	session readOneOf: GlorpCustomer where: [:each | each id = customer id].
	customer addTransaction: trans.
	session commitUnitOfWork.
	transactions := session accessor executeSQLString: 'SELECT ID FROM BANK_TRANS WHERE OWNER_ID = ', customer id printString.
	self assert: transactions size = 1.
	self assert: (trans id = (transactions first atIndex: 1))]
		ensure: [session rollbackTransaction].
! !

!GlorpWritingTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__GlorpWritingTest.st,v 1.1 2006/07/26 08:58:05 boris Exp $'
! !
