"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpTestCase subclass:#SimpleStudentExampleTest2
	instanceVariableNames:'session accessor'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpTest-StudentDatabase'
!


!SimpleStudentExampleTest2 methodsFor:'support'!

loginToUse

   ^Glorp::Login new
        database: Glorp::PostgreSqlODBCPlatform new;
        username: 'etas';
        password: '';
        connectString: 'localhost:5432_mydb';
        name: 'PostgreSQL ANSI'.
!

setUp
   | login |
Transcript show: 'start setup'; cr.
  login := self loginToUse.
  accessor := Glorp::DatabaseAccessor forLogin: login.
  accessor login.

  session := GlorpSession new.
  session system: (GlorpStudentExampleDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
  session accessor: accessor.

  accessor setAutocommit.
  self setUpDatabase.
  accessor clearAutocommit.
!

tearDown

        super tearDown.
    "    session rollbackTransaction.   "
        session accessor logout.
        session reset.
        session := nil.
! !

!SimpleStudentExampleTest2 methodsFor:'table creation'!

setUpDatabase

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor beginTransaction].
        [GlorpStudentTestDescriptorSystem glorpAllSubclasses do: 
                        [:eachSystemClass | 
                        self 
                                setUpSystem: (eachSystemClass forPlatform: accessor platform) setUpDefaults].

        accessor platform requiresTransactionForTableOperations ifTrue: [accessor commitTransaction]] ifCurtailed: [
                accessor platform requiresTransactionForTableOperations ifTrue: [accessor rollbackTransaction]].
!

setUpSystem: system 
        | errorBlock |
        accessor isLoggedIn ifFalse: [accessor login].
        accessor dropTables: ((TableSorter for: system allTables) sort reversed).
        errorBlock := [:ex | Transcript show: ex description; cr. ex pass].
        system platform areSequencesExplicitlyCreated ifTrue: [
                accessor dropSequences: system allSequences.
                system allSequences do: [:each |
                        accessor createSequence: each ifError: errorBlock]].
        system allTables do: [:each | 
                        accessor createTable: each
                                ifError: errorBlock].
        system allTables do: [:each |
                        accessor createTableIndexes: each ifError: errorBlock]. 
        system allTables do: [:each | 
                        accessor createTableFKConstraints: each ifError: errorBlock].
! !

!SimpleStudentExampleTest2 methodsFor:'table population'!

writeAllTables
   Transcript show: 'start writing tables ...'; cr.
  self writeStudentTable;
       writeTeacherTable;
       writeCoursesTable;
       writeSectionsTable;
       writeEnrollsTable.
!

writeCoursesTable

 session system exampleCourseRows do:
     [ :aRow | session writeRow: aRow].
!

writeEnrollsTable

 session system exampleEnrollRows do:
     [ :aRow | session writeRow: aRow].
!

writeSectionsTable

 session system exampleSectionRows do:
     [ :aRow | session writeRow: aRow].
!

writeStudentTable

 session system exampleStudentRows do:
     [ :aRow | session writeRow: aRow].
!

writeTeacherTable

 session system exampleTeacherRows do:
     [ :aRow | session writeRow: aRow].
! !

!SimpleStudentExampleTest2 methodsFor:'tests'!

testCorrelatedSelect01
    " page 138:
     which students are not enrolled in any course? "
        | query result |

    [session beginTransaction.
     self writeAllTables.
        query := Query returningManyOf: GlorpSDBStudents
                    where: [:each |
                 each notExists:
                   (Query readManyOf: GlorpSDBEnrolls where: [:eachEn | eachEn studentId = each])
               ].

        result := session execute: query.

        self assert: (result size = 1).
    ]
        ensure: [session rollbackTransaction].

  " this is quite tricky "
!

testCreateAndReadRecord

        | teacher result |

  session beginUnitOfWork.
  teacher := GlorpSDBTeachers new.
  teacher id: 1;
        name: 'Meier';
        phone: '321 777';
        salary: 32000.00.
  session register: teacher.
  session commitUnitOfWork.
  result := session readManyOf: GlorpSDBTeachers.
  self assert: result size = 1.
!

testDirectSQL

        | result |

  [session beginTransaction.

        session accessor basicExecuteSQLString:
         'INSERT INTO TEACHERS (TEACHER_ID,TEACHER_NAME,PHONE,SALARY)  VALUES (303,''Dr. Horn'',''257-3049'',27540.0)'.

        result := session accessor basicExecuteSQLString: 'SELECT * FROM TEACHERS'.

         self assert: result size = 1.
   ]
        ensure: [session rollbackTransaction].
!

testSelectFromJoinedTableEnrolls01
    " page 99 "
        | query result |

    [session beginTransaction.
        self writeAllTables.
        query := Query returningManyOf: GlorpSDBEnrolls
                    where: [:each |
                 each grade > 2
                & ( each studentId state = 'CA' | (each studentId state = 'IL'))
               ].
        result := session execute: query.
        self assert: (result size = 1).
    ]
        ensure: [session rollbackTransaction].

  " this is quite tricky "
!

testSelectFromTableCourses01
    " page 24 "
        | query result |
    [session beginTransaction.
        self writeAllTables.
        query := Query returningManyOf: GlorpSDBCourses where: [:each | each numCredits = 3].
        result := session execute: query.
        self assert: (result size = 3).
    ]
        ensure: [session rollbackTransaction].
!

testSelectFromTableCourses02
    " page 29 "
        | query result |

    [session beginTransaction. 
        self writeAllTables.
        query := Query returningManyOf: GlorpSDBCourses where: [:each | each department = 'Math' & (each numCredits >= 3)].
        result := session execute: query.
        self assert: (result size = 1).
        self assert: result first numCredits = 4.
    ]
        ensure: [session rollbackTransaction].
!

testSelectFromTableStudents01
    " page 26 "
        | query result |

    [session beginTransaction.
        self writeAllTables.
        query := Query returningManyOf: GlorpSDBStudents where: [:each | each sex = 'M'].
        result := session execute: query.
        self assert: (result size = 6).
    ]
        ensure: [session rollbackTransaction].
!

testSelectFromTableStudents02
    " page 28 "
        | result |


    [session beginTransaction.
     self writeAllTables.
        result := session readManyOf: GlorpSDBStudents where: [:each | each sex = 'F' & (each city = 'Chicago')].
        self assert: (result size = 1).
    ]
        ensure: [session rollbackTransaction].
!

testSelectFromTableTeatchers01
    " page 25 "
        | query result |

    [session beginTransaction.
        self writeAllTables.
        query := Query returningManyOf: GlorpSDBTeachers where: [:each | each salary > 30000.00].
        result := session execute: query.
        self assert: (result size = 4).
    ]
        ensure: [session rollbackTransaction].
! !

!SimpleStudentExampleTest2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__SimpleStudentExampleTest2.st,v 1.1 2006/07/26 09:05:54 boris Exp $'
! !
