"{ Package: 'stx:goodies/glorp0_4/tests' }"

"{ NameSpace: Glorp }"

GlorpSessionBasedTest subclass:#TestCommandPermission
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpDBTests'
!


!TestCommandPermission methodsFor:'support'!

writeTestRows
	| addressRow personRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
! !

!TestCommandPermission methodsFor:'tests'!

testAlsoDeny
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: InsertCommand.
	session accessor alsoDenyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testDenyReads
	| person address |
	[session beginTransaction.
	self writeTestRows.
	person := session readOneOf: GlorpPerson where: nil.
	address := person address yourSelf.
	session accessor denyReads.
	self should: [session readOneOf: GlorpPerson where: nil] raise: GlorpIllegalCommand.
	session inUnitOfWorkDo: [
		session register: person.
		session delete: address.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].
!

testForbiddenDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person address.
					person address: nil]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testForbiddenDeleteAllowsUpdateAndInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	session inUnitOfWorkDo: [
		session register: person.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].
!

testInsertOnlyAllowsInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor insertOnly.
	person := session readOneOf: GlorpPerson where: nil.
	session
		inUnitOfWorkDo:
			[session register: person.
			session register: (GlorpPerson new id: 4)]]
		ensure: [session rollbackTransaction].
!

testInsertOnlyForbidsDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor insertOnly.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testInsertOnlyForbidsUpdate
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person name: 'Quigley Quagmire']]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testPermitEverything
	session accessor permitEverything.
	[session beginTransaction.
	self writeTestRows.
	session inUnitOfWorkDo: [ | person |
		person := session readOneOf: GlorpPerson where: nil.
		session delete: person address.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].
!

testPermitNothing
	session accessor permitNothing.
	self should: [session readManyOf: GlorpPerson] raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: InsertCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: UpdateCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: DeleteCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor
				executeCommand: SQLStringSelectCommand new]
		raise: GlorpIllegalCommand.
!

testReadOnlyForbidsDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testReadOnlyForbidsInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person address: (GlorpAddress new id: 7)]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
!

testReadOnlyForbidsUpdate
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person name: 'Quigley Quagmire']]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].
! !

!TestCommandPermission class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/tests/Glorp__TestCommandPermission.st,v 1.1 2006/07/26 08:58:24 boris Exp $'
! !
