"{ Package: 'stx:goodies/gofer' }"

Object subclass:#GoferReference
	instanceVariableNames:'name'
	classVariableNames:''
	poolDictionaries:''
	category:'Gofer-Core'
!

GoferReference comment:'A GoferReference is an abstract superclass for various kinds of references to Monticello packages and versions.'
!


!GoferReference class methodsFor:'instance creation'!

name: aString
	^ self basicNew initializeName: aString
!

new
	self error: 'Use #name: to initialize the receiver.'
! !

!GoferReference methodsFor:'accessing'!

name
	"Answer the name of this reference."
	
	^ name
!

packageName
	"Answer the package name."
	
	self subclassResponsibility
! !

!GoferReference methodsFor:'comparing'!

= aReference
	^ self class = aReference class and: [ self name = aReference name ]
!

hash
	^ self name hash
! !

!GoferReference methodsFor:'initialization'!

initializeName: aString
	name := aString
! !

!GoferReference methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' name: '; print: self name
! !

!GoferReference methodsFor:'private'!

matches: aResolvedReference
	"Answer true if the receiver matches aResolvedReference."

	self subclassResponsibility
! !

!GoferReference methodsFor:'querying'!

resolveAllWith: aGofer
	"Answer a sorted collection of all resolved references within aGofer."

	^ aGofer allResolved select: [ :each | self matches: each ]
!

resolveWith: aGofer
	"Answer a single resolved reference with aGofer configuration, throw an error if the version can't be found.'"

	| references |
	references := self resolveAllWith: aGofer.
	^ references isEmpty
		ifTrue: [ self error: 'Unable to resolve ' , self name ]
		ifFalse: [ references last ]
!

workingCopy
	"Answer a working copy or throw an error if not present."

	^ MCWorkingCopy allManagers
		detect: [ :each | self packageName = each packageName ]
		ifNone: [ self error: 'Working copy for ' , self name , ' not found' ]
! !

!GoferReference class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gofer/GoferReference.st,v 1.1 2011/08/20 16:23:46 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gofer/GoferReference.st,v 1.1 2011/08/20 16:23:46 cg Exp $'
! !
