"{ Package: 'stx:goodies/gofer' }"

GoferVersionReference subclass:#GoferResolvedReference
	instanceVariableNames:'repository'
	classVariableNames:''
	poolDictionaries:''
	category:'Gofer-Core'
!

GoferResolvedReference comment:'A GoferVersionReference refers to a specific version of a Monticello package in a particular repository. This class is the only one that can actually load the version, because it is the only one knowing where to find it.'
!


!GoferResolvedReference class methodsFor:'instance creation'!

name: aString repository: aRepository
	^ self basicNew initializeName: aString repository: aRepository
! !

!GoferResolvedReference methodsFor:'accessing'!

repository
	"Answer the repository of the receiver."
	
	^ repository
!

version
	"Answer a Monticello version of the receiver."

	^ self repository goferVersionFrom: self
! !

!GoferResolvedReference methodsFor:'comparing'!

<= aResolvedReference
	"Sort versions according to:

		1. package name
		2. branch name, list versions without branch last
		3. version number
		4. author name
		5. repository priority"
	
	self packageName = aResolvedReference packageName
		ifFalse: [ ^ self packageName <= aResolvedReference packageName ].
	self branch = aResolvedReference branch ifFalse: [ 
		^ (self branch isEmpty or: [ aResolvedReference branch isEmpty ])
			ifTrue: [ self branch size > aResolvedReference branch size ]
			ifFalse: [ self branch <= aResolvedReference branch ] ].
	self versionNumber = aResolvedReference versionNumber
		ifFalse: [ ^ self versionNumber <= aResolvedReference versionNumber ].
	self author = aResolvedReference author
		ifFalse: [ ^ self author <= aResolvedReference author ].
	self repository goferPriority = aResolvedReference repository goferPriority
		ifFalse: [ ^ self repository goferPriority <= aResolvedReference repository goferPriority ].
	^ true
! !

!GoferResolvedReference methodsFor:'initialization'!

initializeName: aString repository: aRepository
	self initializeName: aString.
	repository := aRepository
! !

!GoferResolvedReference class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gofer/GoferResolvedReference.st,v 1.1 2011/08/20 16:24:52 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gofer/GoferResolvedReference.st,v 1.1 2011/08/20 16:24:52 cg Exp $'
! !
