"{ Package: 'stx:goodies/gofer/tests' }"

GoferTest subclass:#GoferApiTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Gofer-Tests'
!


!GoferApiTest methodsFor:'testing'!

testInitialReferences
	self assert: gofer references isEmpty
!

testInitialRepositories
	gofer := Gofer new.
	self assert: (gofer repositories size = 1).
	self assert: (gofer repositories first isKindOf: MCCacheRepository default class)
! !

!GoferApiTest methodsFor:'testing-references'!

testConstraintReference
	gofer 
		repository: self monticelloRepository; 
		package: 'GoferBar' constraint: [ :ref | ref branch = 'branch' ].
	self assert: (gofer resolved size = 1).
	self assert: (gofer resolved first isKindOf: GoferResolvedReference).
	self assert: (gofer resolved first packageName = 'GoferBar').
	self assert: (gofer resolved first author = 'lr').
	self assert: (gofer resolved first branch = 'branch').
	self assert: (gofer resolved first versionNumber = 2).
	self assert: (gofer resolved first repository = self monticelloRepository)
!

testPackageReference
	gofer 
		repository: self monticelloRepository;
		package: 'GoferFoo'.
	self assert: (gofer resolved size = 1).
	self assert: (gofer resolved first isKindOf: GoferResolvedReference).
	self assert: (gofer resolved first packageName = 'GoferFoo').
	self assert: (gofer resolved first author = 'lr').
	self assert: (gofer resolved first branch isEmpty).
	self assert: (gofer resolved first versionNumber = 4).
	self assert: (gofer resolved first repository = self monticelloRepository)
!

testVersionReference
	gofer 
		repository: self monticelloRepository; 
		version: 'GoferFoo-lr.2'.
	self assert: (gofer resolved size = 1).
	self assert: (gofer resolved first isKindOf: GoferResolvedReference).
	self assert: (gofer resolved first packageName = 'GoferFoo').
	self assert: (gofer resolved first author = 'lr').
	self assert: (gofer resolved first branch isEmpty).
	self assert: (gofer resolved first versionNumber = 2).
	self assert: (gofer resolved first repository = self monticelloRepository)
! !

!GoferApiTest methodsFor:'testing-repositories'!

testCustomRepository
	gofer repository: self monticelloRepository.
	self assert: gofer repositories: (Array with: self monticelloRepository description).
	self assert: (gofer repositories first = self monticelloRepository)
!

testDirectoryRepository
	gofer directory: FileDirectory default pathName.
	self assert: gofer repositories: (Array with: FileDirectory default pathName).
	self assert: (gofer repositories first isKindOf: MCDirectoryRepository)
!

testFtpRepository
	Smalltalk globals
		at: #MCFtpRepository
		ifPresent: [ :ftpRepositoryClass | 
			gofer url: 'ftp://squeakfoundation.org/39a' username: 'foo' password: 'bar'.
			self assert: gofer repositories: #('ftp://foo@squeakfoundation.org/39a').
			self assert: (gofer repositories first isKindOf: ftpRepositoryClass) ]
!

testHttpRepository
	gofer url: 'http://source.lukas-renggli.ch/pier' username: 'foo' password: 'bar'.
	self assert: gofer repositories: #('http://source.lukas-renggli.ch/pier').
	self assert: (gofer repositories first isKindOf: MCHttpRepository)
!

testSubDirectoryRepository
	Smalltalk globals
		at: #MCSubDirectoryRepository
		ifPresent: [ :subDirectoryRepositoryClass | 
			gofer directory: FileDirectory default pathName , FileDirectory slash , '*'.
			self assert: gofer repositories: (Array with: FileDirectory default pathName , FileDirectory slash , '*').
			self assert: (gofer repositories first isKindOf: subDirectoryRepositoryClass) ]
! !

!GoferApiTest methodsFor:'testing-repositories-options'!

testPackageCache
	gofer squeaksource: 'r1'; squeaksource: 'r2'.
	
	gofer enablePackageCache.
	self assert: gofer repositories: (Array with: MCCacheRepository default description) , #('http://www.squeaksource.com/r1' 'http://www.squeaksource.com/r2').
	
	gofer disablePackageCache.
	self assert: gofer repositories: #('http://www.squeaksource.com/r1' 'http://www.squeaksource.com/r2')
!

testRepositoryErrors
	gofer 
		url: 'http://www.google.com';
		repository: self monticelloRepository.
	gofer package: 'GoferFoo'.
	
	gofer enableRepositoryErrors.
	self should: [ gofer resolved ] raise: GoferRepositoryError.
	
	gofer disableRepositoryErrors.
	self shouldnt: [ gofer resolved ] raise: GoferRepositoryError
! !

!GoferApiTest methodsFor:'testing-repositories-places'!

testBlueplane
	gofer blueplane: 'SIXX'.
	self assert: gofer repositories: #('http://squeaksource.blueplane.jp/SIXX')
!

testCroquet
	gofer croquet: 'Hermes'.
	self assert: gofer repositories: #('http://hedgehog.software.umn.edu:8888/Hermes')
!

testGemsource
	gofer gemsource: 'Seaside29'.
	self assert: gofer repositories: #('http://seaside.gemstone.com/ss/Seaside29')
!

testImpara
	gofer impara: 'Tweak'.
	self assert: gofer repositories: #('http://source.impara.de/Tweak')
!

testRenggli
	gofer renggli: 'pier'.
	self assert: gofer repositories: #('http://source.lukas-renggli.ch/pier')
!

testSaltypickle
	gofer saltypickle: 'GraphViz'.
	self assert: gofer repositories: #('http://squeak.saltypickle.com/GraphViz')
!

testSqueakfoundation
	gofer squeakfoundation: '39a'.
	self assert: gofer repositories: #('http://source.squeakfoundation.org/39a')
!

testSqueaksource
	gofer squeaksource: 'Seaside29'.
	self assert: gofer repositories: #('http://www.squeaksource.com/Seaside29')
!

testWiresong
	gofer wiresong: 'ob'.
	self assert: gofer repositories: #('http://source.wiresong.ca/ob')
! !

!GoferApiTest methodsFor:'utilities'!

assert: aGofer repositories: anArray
	self assert: aGofer repositories size = anArray size.
	aGofer repositories with: anArray
		do: [ :first :second | self assert: first description = second ]
! !

!GoferApiTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gofer/tests/GoferApiTest.st,v 1.2 2011/09/06 18:02:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gofer/tests/GoferApiTest.st,v 1.2 2011/09/06 18:02:20 cg Exp $'
! !
