"{ Package: 'stx:goodies/gofer/tests' }"

GoferTest subclass:#GoferOperationTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Gofer-Tests'
!


!GoferOperationTest methodsFor:'running'!

runCase
	^ SystemChangeNotifier uniqueInstance doSilently: [ super runCase ]
!

setUp
	super setUp.
	gofer repository: self monticelloRepository
!

tearDown
	(self hasPackage: 'GoferFoo')
		ifTrue: [ Gofer new package: 'GoferFoo'; unload ].
	(self hasPackage: 'GoferBar')
		ifTrue: [ Gofer new package: 'GoferBar'; unload ]
! !

!GoferOperationTest methodsFor:'testing'!

expectedFailures

	^#(testRevert) "Squeak ends up removing the class GoferBar instead of changing it's category ... go figure ... this is a Monticello bug"
!

testCleanup
	| class |
	gofer package: 'GoferFoo'; load.
	class := Smalltalk globals classNamed: #GoferFoo.
	Smalltalk globals organization addCategory: #'GoferFoo-Empty'.
	class organization addCategory: #empty.
	class class organization addCategory: #empty.
	gofer cleanup.
	self deny: (Smalltalk organization categories includes: #'GoferFoo-Empty').
	self deny: (class organization categories includes: #'GoferFoo-Empty').
	self deny: (class class organization categories includes: #'GoferFoo-Empty')
!

testCommit
	| repository |
	repository := MCDictionaryRepository new.
	gofer package: 'GoferFoo'; load.
	gofer := Gofer new.
	gofer disablePackageCache.
	gofer repository: repository.
	gofer package: 'GoferFoo'.
	self shouldnt: [ gofer commit: 'A test commit' ] raise: Error.
	self assert: repository allVersionInfos size = 1
!

testFetch
	gofer package: 'GoferFoo'.
	self shouldnt: [ gofer fetch ] raise: Error
!

testLoad
	gofer version: 'GoferFoo-lr.1'.
	self shouldnt: [ gofer load ] raise: Error.
	self assert: (self hasVersion: 'GoferFoo-lr.1').
	self assert: (self hasClass: #GoferFoo)
!

testLocalChanges
	| changes |
	gofer package: 'GoferBar'; load.
	(Smalltalk globals classNamed: #GoferBar) compile: 'foo'.
	self shouldnt: [ changes := gofer localChanges ] raise: Error.
	self assert: changes operations size = 1
!

testMerge
	| initial |
	initial := gofer copy.
	initial version: 'GoferBar-jf.1'; load.
	gofer package: 'GoferBar'; load.
	(Smalltalk globals classNamed: #GoferBar) compile: 'foo'.
	gofer merge.
	self assert: (self hasClass: #GoferBar selector: #foo)
!

testPush
	| repository |
	gofer := Gofer new.
	gofer disablePackageCache.
	gofer repository: (repository := MCDictionaryRepository new).
	gofer package: 'GoferFoo'.
	self shouldnt: [ gofer push ] raise: Error
!

testRecompile
	gofer package: 'Gofer-Core'.
	self shouldnt: [ gofer recompile ] raise: Error
!

testReinitialize
	| class |
	gofer package: 'GoferFoo'; load.
	class := Smalltalk classNamed: #GoferFoo.
	class addClassVarName: 'InstanceSide'; addClassVarName: 'ClassSide'.
	class compile: 'initialize InstanceSide := true'.
	class class compile: 'initialize ClassSide := true'.
	self assert: (class classPool at: #InstanceSide) isNil.
	self assert: (class classPool at: #ClassSide) isNil.
	self shouldnt: [ gofer reinitialize ] raise: Error.
	self assert: (class classPool at: #InstanceSide) isNil.
	self assert: (class classPool at: #ClassSide)
!

testRemoteChanges
	| changes |
	gofer package: 'GoferBar'; load.
	(Smalltalk globals classNamed: #GoferBar) compile: 'foo'.
	self shouldnt: [ changes := gofer remoteChanges ] raise: Error.
	self assert: changes operations size = 1
!

testRevert
	gofer
		package: 'GoferFoo';
		package: 'GoferBar';
		load.
	(Smalltalk globals classNamed: #GoferBar) category: 'GoferFoo'.
	gofer revert.
	self assert: (Smalltalk globals classNamed: #GoferFoo) category asSymbol = #GoferFoo.
	self assert: (Smalltalk globals classNamed: #GoferBar) category asSymbol = #GoferBar
!

testUnload
	gofer package: 'GoferFoo'; load.
	self shouldnt: [ gofer unload ] raise: Error.
	self deny: (self hasPackage: 'GoferFoo').
	self deny: (self hasClass: #GoferFoo)
!

testUpdate
	| initial |
	initial := gofer copy.
	initial version: 'GoferFoo-lr.1'; load.
	gofer package: 'GoferFoo'.
	gofer update.
	self assert: (self hasVersion: 'GoferFoo-lr.4')
! !

!GoferOperationTest methodsFor:'utilities'!

hasClass: aSymbol
	^ Smalltalk globals includesKey: aSymbol
!

hasClass: aSymbol selector: aSelector
	^ (Smalltalk globals classNamed: aSymbol) includesSelector: aSelector
!

hasPackage: aString
	| package |
	package := MCWorkingCopy allManagers
		detect: [ :each | each packageName = aString ]
		ifNone: [ nil ].
	^ package notNil
!

hasVersion: aString
	| version |
	version := MCWorkingCopy allManagers
		detect: [ :each | each ancestry ancestorString = aString ]
		ifNone: [ nil ].
	^ version notNil
! !

!GoferOperationTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gofer/tests/GoferOperationTest.st,v 1.2 2011/09/06 18:02:54 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gofer/tests/GoferOperationTest.st,v 1.2 2011/09/06 18:02:54 cg Exp $'
! !
