"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

EllipseFigure subclass:#ArcFigure
	instanceVariableNames:'startAngle stopAngle'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

ArcFigure comment:'ArcFigure is a figure that represents an EllipticalArc.

Instance Variables:
	startAngle      <Integer> start angle for our ellipse
	stopAngle       <Integer> start angle for our ellipse
'
!


!ArcFigure methodsFor:'accessing'!

handles
	| handles |
	handles := OrderedCollection withAll: super handles.
	handles
		add: ((TrackHandle on: self at: #startAnglePosition)
					moveBlock: [:aPoint | self trackStartAngle: aPoint];
					yourself);
		add: ((TrackHandle on: self at: #stopAnglePosition)
					moveBlock: [:aPoint | self trackStopAngle: aPoint];
					yourself).
	^handles
!

startAngle
	^startAngle min: stopAngle
!

startAngle: anInteger
	startAngle := anInteger.
	self changed
!

startAnglePosition
	^self positionForAngle: startAngle
!

stopAngle
	^stopAngle max: startAngle
!

stopAngle: anInteger
	stopAngle := anInteger.
	self changed
!

stopAnglePosition
	^self positionForAngle: stopAngle
!

sweepAngle
	^stopAngle = startAngle
		ifTrue: [360]
		ifFalse: [self stopAngle - self startAngle]
!

trackStartAngle: aPoint
	self startAngle: (self convertThetaToAngle: (self normalizePoint: aPoint) theta)
!

trackStopAngle: aPoint
	self stopAngle: (self convertThetaToAngle: (self normalizePoint: aPoint) theta)
! !

!ArcFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	super displayFigureOn: aGraphicsContext.
	self lineWidth > 0 ifFalse: [^self].
	aGraphicsContext
		lineWidth: self lineWidth;
		paint: self lineColor.
	(LineSegment from: self center to: self startAnglePosition)
		displayStrokedOn: aGraphicsContext.
	(LineSegment from: self center to: self stopAnglePosition)
		displayStrokedOn: aGraphicsContext
! !

!ArcFigure methodsFor:'initialize-release'!

ellipse: aRect
	super ellipse: aRect.
	startAngle := 0.
	stopAngle := 360
! !

!ArcFigure methodsFor:'private'!

convertAngleToTheta: anAngle
	^anAngle / 360.0 * (2 * Double pi)
!

convertThetaToAngle: aNumber
	^(aNumber * 360 / (2 * Double pi)) rounded
!

normalizePoint: aPoint
	| vector extent |
	extent := self extent.
	(extent x <= 0 or: [extent y <= 0]) ifTrue: [^0 @ 0].
	vector := aPoint - self center.
	vector := (vector x asFloat / extent x) @ (vector y asFloat / extent y).
	0 @ 0 = vector ifTrue: [^0 @ 0].
	^vector
!

positionForAngle: anAngle
	| theta |
	theta := self convertAngleToTheta: anAngle.
	^self center
		+ ((Point r: self extent x / 2.0 theta: theta) x
				@ (Point r: self extent y / 2.0 theta: theta) y) rounded
! !

!ArcFigure methodsFor:'testing'!

containsPoint: aPoint
	"Overriden, because regionIntersects: has a bug for small arcs"

	| outer angle strtAngle |
	self isOpaque ifFalse: [^super containsPoint: aPoint].
	self center = aPoint ifTrue: [^true].
	angle := self
				convertThetaToAngle: (self normalizePoint: aPoint rounded) theta.
	strtAngle := self startAngle.
	(angle between: strtAngle and: strtAngle + self sweepAngle)
		ifFalse: [^false].
	outer := self positionForAngle: angle.
	^aPoint rounded between: (self center min: outer)
		and: (self center max: outer)
! !

!ArcFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ArcFigure.st,v 1.1 2006/03/13 16:56:18 cg Exp $'
! !
