"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

PolylineFigure subclass:#BezierFigure
	instanceVariableNames:'bezier'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

BezierFigure comment:'BezierFigure represents a bezier curve.

Instance Variables:
	bezier  <Bezier>  our curve

'
!


!BezierFigure class methodsFor:'instance creation'!

createAt: aPoint
	^self start: aPoint stop: aPoint
!

start: startPoint stop: stopPoint
	| diff |
	diff := stopPoint - startPoint.
	^self withPoints: (Array
				with: startPoint
				with: diff / 3.0 + startPoint
				with: diff * 2 / 3.0 + startPoint
				with: stopPoint)
! !

!BezierFigure methodsFor:'accessing'!

handles
	^(1 to: points size) collect: [:i | TrackHandle pointAtIndex: i of: self]
!

pointAt: anIndex put: aValue
	super pointAt: anIndex put: aValue.
	bezier := nil
! !

!BezierFigure methodsFor:'copying'!

postCopy
	super postCopy.
	bezier := nil
! !

!BezierFigure methodsFor:'private'!

asGeometric
	^self bezier
!

bezier
	^bezier isNil
		ifTrue: [bezier := self computeBezier]
		ifFalse: [bezier]
!

computeBezier
	^Bezier
		start: points first
		end: points last
		controlPoint1: (points at: 2)
		controlPoint2: (points at: 3)
!

computePreferredBounds
	| rect |
	rect := self bezier bounds.
	^rect origin - (self lineWidth // 2)
		corner: rect corner + (self lineWidth // 2)
! !

!BezierFigure methodsFor:'transforming'!

basicTranslateBy: aPoint
	super basicTranslateBy: aPoint.
	bezier := bezier translatedBy: aPoint
! !

!BezierFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__BezierFigure.st,v 1.1 2006/03/13 16:56:30 cg Exp $'
! !
