"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#CachedFigure
	instanceVariableNames:'cache'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

CachedFigure comment:'CachedFigure is a figure that holds a pixmap of its display. It can be used for complicated figures to cache their display so that they display faster. However, as graphics cards become faster, this figure is less useful.

Subclasses must implement the following messages:
	private
		fillCache       returns a pixmap of our contents

Instance Variables:
	cache   <Pixmap>  what we display when we are drawn

'
!


!CachedFigure methodsFor:'accessing'!

handles
	^Handle allCornersOf: self
!

origin: aPoint
	bounds := aPoint extent: self cache extent
! !

!CachedFigure methodsFor:'displaying'!

displayFigureOn: aGC
	self cache displayOn: aGC at: self origin rounded
! !

!CachedFigure methodsFor:'initialize-release'!

flushCaches
	super flushCaches.
	cache := nil
!

initialize
	super initialize.
	self opaque
! !

!CachedFigure methodsFor:'private'!

cache
	^cache isNil
		ifTrue: [cache := self fillCache]
		ifFalse: [cache]
!

fillCache
	"Draw each of my component Figures onto my cache."

	^self subclassResponsibility
! !

!CachedFigure methodsFor:'transforming'!

computePreferredBounds
	^bounds isNil
		ifTrue: [0 @ 0 extent: self cache extent]
		ifFalse: [bounds]
! !

!CachedFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__CachedFigure.st,v 1.1 2006/03/13 16:53:32 cg Exp $'
! !
