"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#CanvasFigure
	instanceVariableNames:'pixmap width'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-HotPaint'
!

CanvasFigure comment:'A CanvasFigure is a concrete subclass that can be painted with painting tools
like the airbrush and paintbrush.  It uses color, too.'
!


!CanvasFigure class methodsFor:'instance creation'!

createAt: aPoint
	^self rectangle: (aPoint corner: aPoint)
!

rectangle: aRectangle
	^self new initialize: aRectangle
! !

!CanvasFigure methodsFor:'accessing'!

menuAt: aPoint
	| mb |
	mb := MenuBuilder new.
	mb beginSubMenuLabeled: 'Tool Width'.
	#('1 Smallest' '2 Small' '3 Medium' '4 Big' '5 Biggest') with: (1 to: 5)
		do: [:label :i | mb add: label -> [self width: i]].
	mb endSubMenu.
	mb beginSubMenuLabeled: 'Tool Color'.
	#(#black #cyan #red #green #blue #yellow #pink)
		do: [:each | mb add: each -> [self fillColor: (ColorValue perform: each)]].
	mb endSubMenu.
	^mb menu
!

width
	^width
!

width: anInteger
	width := anInteger.
	self changed
! !

!CanvasFigure methodsFor:'damage control'!

changed: anAspectSymbol with: aParameter
	self resetBounds.
	super changed: anAspectSymbol with: aParameter
! !

!CanvasFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	| originForPixmap |
	originForPixmap := self origin.
	self lineWidth > 0
		ifTrue:
			[originForPixmap := originForPixmap + self lineWidth.
			aGraphicsContext lineWidth: self lineWidth.
			aGraphicsContext paint: self lineColor.
			aGraphicsContext displayRectangularBorder: self bounds].
	pixmap displayOn: aGraphicsContext at: originForPixmap
! !

!CanvasFigure methodsFor:'initialize-release'!

initialize: aRectangle
	bounds := aRectangle.
	width := 2.
	self
		lineColor: ColorValue black;
		fillColor: ColorValue black;
		lineWidth: 1;
		resetBounds;
		opaque
! !

!CanvasFigure methodsFor:'painting'!

paintAt: aPoint
	| aRectangle gc extent |
	extent := self brushWidth @ self brushWidth.
	aRectangle := aPoint - (extent // 2) - self origin extent: extent.
	gc := pixmap graphicsContext paint: self fillColor.
	gc
		displayWedgeBoundedBy: aRectangle
		startAngle: 0
		sweepAngle: 360.
	self changed
!

paintAt: aPoint brushMask: aMask
	pixmap graphicsContext paint: self fillColor.
	pixmap graphicsContext displayImage: aMask
		at: aPoint - (8 @ 8) - self origin.
	self changed
!

paintFrom: startPoint to: endPoint
	| gc |
	gc := pixmap graphicsContext.
	gc paint: self fillColor.
	gc lineWidth: self brushWidth.
	gc displayLineFrom: startPoint - self origin to: endPoint - self origin.
	self changed
! !

!CanvasFigure methodsFor:'private'!

brushWidth
	^self width ** 2
!

resetBounds
	| newPixmap extent |
	extent := self bounds extent - (self lineWidth * 2).
	(extent x <= 0 or: [extent y <= 0]) ifTrue: [extent := 0 @ 0].
	(pixmap notNil and: [extent = pixmap extent]) ifTrue: [^self].
	newPixmap := Pixmap extent: extent.
	pixmap isNil
		ifFalse: [pixmap displayOn: newPixmap graphicsContext at: 0 @ 0].
	pixmap := newPixmap
! !

!CanvasFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__CanvasFigure.st,v 1.1 2006/03/13 16:53:20 cg Exp $'
! !
