"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Controller subclass:#DrawingController
	instanceVariableNames:'tool keyboardProcessor'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Framework'
!

DrawingController comment:'DrawingController is a Controller for a Drawing view.

Instance Variables:
	keyboardProcessor       <KeyboardProcessor>       the keyboard processor for our window
	tool    <Tool>    the current tool selected

'
!


!DrawingController methodsFor:'accessing'!

currentTool
	^tool value
! !

!DrawingController methodsFor:'changing'!

changedTool
	view clearSelections.
	(self currentTool)
		controller: self;
		selected.
	self currentTool cursor show.
	self setAsCurrentConsumer
! !

!DrawingController methodsFor:'event driven'!

desiresFocus
	^true
!

handleEvent: anEvent
	anEvent key == #enter ifTrue: [self currentTool cursor show].
	anEvent key == #exit ifTrue: [Cursor normal show].
	(anEvent isMouseEvent or: [anEvent isKeyboard])
		ifTrue:
			[(self currentTool handleEvent: anEvent)
				ifTrue: [self setAsCurrentConsumer]]
!

keyboardProcessor
	^keyboardProcessor
!

keyboardProcessor: aKeyboardProcessor
	keyboardProcessor := aKeyboardProcessor
!

setAsCurrentConsumer
	| ctrl |
	keyboardProcessor isNil ifTrue: [^self].
	ctrl := keyboardProcessor currentConsumer.
	ctrl ~= self ifTrue: [keyboardProcessor requestActivationFor: self]
! !

!DrawingController methodsFor:'events'!

enterEvent: anEvent
	self currentTool cursor show.
	self handleEvent: anEvent.
	^super enterEvent: anEvent
!

exitEvent: anEvent
	Cursor normal show.
	self handleEvent: anEvent.
	^super exitEvent: anEvent
! !

!DrawingController methodsFor:'initialize-release'!

initialize
	super initialize.
	tool := Tool selectionTool asValue.
	tool value controller: self
!

tool: aValueModel
	tool := aValueModel.
	tool onChangeSend: #changedTool to: self.
	self changedTool
! !

!DrawingController methodsFor:'menu processing'!

localMenuItem: aSymbol
	"Answer an Array of Symbols that represent the menu messages that should be sent to my
	View as opposed to my Model."

	^#(#cut #paste #copySelection #group #ungroup) includes: aSymbol
!

processMenuAt: globalPoint local: localPoint for: aFigure
	| valueResult |
	valueResult := self
				trackMenu: (aFigure menuAt: localPoint)
				at: globalPoint
				centered: true.
	valueResult isNil ifTrue: [^self].
	(valueResult isKindOf: MenuItem)
		ifTrue: [valueResult := valueResult value].
	valueResult isSymbol
		ifTrue:
			[(self localMenuItem: valueResult)
				ifTrue:
					[aFigure isDrawing ifFalse: [view selection: aFigure].
					view perform: valueResult]
				ifFalse:
					[(aFigure model notNil and: [aFigure model respondsTo: valueResult])
						ifTrue: [aFigure model perform: valueResult]
						ifFalse: [aFigure perform: valueResult]]]
		ifFalse: [valueResult value]
! !

!DrawingController class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__DrawingController.st,v 1.1 2006/03/13 16:52:28 cg Exp $'
! !
