"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#EllipseFigure
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

EllipseFigure comment:'EllipseFigure is a figure that represents ellipses.'
!


!EllipseFigure class methodsFor:'instance creation'!

createAt: aPoint
	^self ellipse: (aPoint extent: 0 @ 0)
!

ellipse: anEllipse
	^self new ellipse: anEllipse
! !

!EllipseFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	"Display an ellipse on aGraphicsContext. If the ellipse figure is not transparent then
	display a filled ellipse using the current fillColor."

	self isOpaque ifTrue: [self displayFilledOn: aGraphicsContext].
	self lineWidth > 0 ifTrue: [self displayOutlineOn: aGraphicsContext]
!

displayFilledOn: aGraphicsContext
	aGraphicsContext paint: self fillColor.
	aGraphicsContext
		displayWedgeBoundedBy: self bounds
		startAngle: self startAngle
		sweepAngle: self sweepAngle
!

displayOutlineOn: aGraphicsContext
	aGraphicsContext lineWidth: self lineWidth.
	aGraphicsContext paint: self lineColor.
	aGraphicsContext
		displayArcBoundedBy: self bounds
		startAngle: self startAngle
		sweepAngle: self sweepAngle
! !

!EllipseFigure methodsFor:'initialize-release'!

ellipse: aRectangle
	bounds := aRectangle.
	self opaque
! !

!EllipseFigure methodsFor:'private'!

startAngle
	^0
!

sweepAngle
	^360
! !

!EllipseFigure methodsFor:'testing'!

containsPoint: aPoint
	^self isOpaque
		ifTrue:
			[(EllipticalArc boundingBox: self preferredBounds
				startAngle: self startAngle
				sweepAngle: self sweepAngle) regionIntersects: (aPoint - 1 extent: 2 @ 2)]
		ifFalse:
			[(EllipticalArc boundingBox: self preferredBounds
				startAngle: self startAngle
				sweepAngle: self sweepAngle) outlineIntersects: (aPoint - 1 extent: 2 @ 2)]
! !

!EllipseFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__EllipseFigure.st,v 1.1 2006/03/13 16:55:57 cg Exp $'
! !
