"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Object subclass:#EndToolState
	instanceVariableNames:'name command'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Tools'
!

EndToolState comment:'EndToolState is a state for tools. When this state is entered, the tool evaluates its command, and then returns to the initial state of the tool.

EndToolState is also the superclass of ToolState. While it might be better to have an abstract class that is the superclass for both these classes. The abstract class and EndToolState would be essentially the same. The only difference would be that EndToolState would have its isEndState return true.

Instance Variables:
	command <BlockClosure>    the command that is executed when we are entered, it takes two arguments, the tool and the event
	name    <String>  our state name
'
!


!EndToolState class methodsFor:'instance creation'!

name: aString command: aBlock
	^(self new) name: aString;
		command: aBlock;
		yourself
! !

!EndToolState methodsFor:'accessing'!

command
	^command
!

command: anObject
	command := anObject
!

name
	^name
!

name: anObject
	name := anObject
! !

!EndToolState methodsFor:'printing'!

printOn: aStream
	aStream
		nextPutAll: self class name;
		nextPut: $(;
		nextPutAll: self name;
		nextPut: $)
! !

!EndToolState methodsFor:'testing'!

isEndState
	^true
! !

!EndToolState methodsFor:'tool-accessing'!

evaluateIn: aTool event: anEvent
	command
		value: aTool
		value: anEvent
!

nextStateForEvent: anEvent tool: aTool
	^nil
! !

!EndToolState class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__EndToolState.st,v 1.1 2006/03/13 16:53:45 cg Exp $'
! !
