"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#Handle
	instanceVariableNames:'owner toolState'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Handles'
!

Handle comment:'Handle is a special figure that is displayed when another figure in the drawing is selected. Handles aren''t permant figures, but are added and removed whenever another figure is selected/deselected. They are useful for defining special manipulation actions such as resizing, connecting figures, etc.

Instance Variables:
	owner   <Figure>  the figure that we are for
	toolState       <EndToolState>    whenever the tool presses us, goto this state

'
!


!Handle class methodsFor:'instance creation'!

connectionOn: aFigure at: aSymbol
	^(self on: aFigure at: aSymbol)
		toolState: (Tool stateFor: 'Connection State');
		yourself
!

on: aFigure at: aSymbol
	| handle |
	handle := self new.
	handle
		owner: aFigure;
		constrain: #center to: (MessageSend receiver: aFigure selector: aSymbol).
	^handle
!

on: aFigure at: aSymbol offset: aPoint
	| handle |
	handle := self new.
	handle
		owner: aFigure;
		constrain: #center
			to: (MessageSend
					receiver: aFigure
					selector: aSymbol
					arguments: (Array with: aPoint)).
	^handle
! !

!Handle class methodsFor:'accessing'!

buildImage
	| size |
	size := 6 @ 6.
	image := CompositePart new.
	image
		add: ((GraphicsAttributesWrapper
					on: (FillingWrapper on: (0 @ 0 extent: size)))
						attributes: (GraphicsAttributes new paint: ColorValue black));
		add: ((GraphicsAttributesWrapper
					on: (FillingWrapper on: (size // 2 - 1 extent: 2 @ 2)))
						attributes: (GraphicsAttributes new paint: ColorValue white)).
	^image
!

image
	^image isNil
		ifTrue: [image := self buildImage]
		ifFalse: [image]
! !

!Handle class methodsFor:'corner handles'!

allCornersOf: aFigure
	^OrderedCollection
		with: (self topLeftOf: aFigure)
		with: (self topRightOf: aFigure)
		with: (self bottomRightOf: aFigure)
		with: (self bottomLeftOf: aFigure)
!

bottomLeftOf: aFigure
	^self on: aFigure at: #bottomLeft
!

bottomRightOf: aFigure
	^self on: aFigure at: #bottomRight
!

topLeftOf: aFigure
	^self on: aFigure at: #topLeft
!

topRightOf: aFigure
	^self on: aFigure at: #topRight
! !

!Handle methodsFor:'accessing'!

menuAt: aPoint
	^Menu new
!

owner
	^owner
!

owner: aFigure
	owner := aFigure
!

toolState
	^toolState
!

toolState: aToolState
	toolState := aToolState
! !

!Handle methodsFor:'copying'!

postCopy
	super postCopy.
	owner := nil
! !

!Handle methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	self class image displayOn: aGraphicsContext at: self origin
! !

!Handle methodsFor:'initialize-release'!

initialize
	super initialize.
	self selectable: false.
	bounds := 0 @ 0 extent: self class image bounds extent.
	toolState := EndToolState name: 'No-op' command: [:tool :event | ]
! !

!Handle methodsFor:'testing'!

isHandle
	^true
! !

!Handle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__Handle.st,v 1.1 2006/03/13 16:58:46 cg Exp $'
! !
