"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

VisualPart subclass:#LineAnnotation
	instanceVariableNames:'positionSymbol directionSymbol isFilled'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

LineAnnotation comment:'LineAnnotation is a visual component that when added to a LineFigure allows you draw special symbols on the end points of the line (e.g., arrows, circles, etc.).

Subclasses must implement the following messages:
	private
		graphic returns a Geometric object that can be displayed

Instance Variables:
	directionSymbol <#startDirection | #stopDirection>        a symbol that when performed on our container, returns a unit vector of our direction
	isFilled        <Boolean> are we filled?
	positionSymbol  <#startPoint | #stopPoint>        where should we be displayed at (when performed on our container returns our position)

'
!


!LineAnnotation class methodsFor:'instance creation'!

forStart
	^(self new)
		direction: #startDirection;
		position: #startPoint;
		yourself
!

forStop
	^(self new)
		direction: #stopDirection;
		position: #stopPoint;
		yourself
! !

!LineAnnotation methodsFor:'accessing'!

isFilled
	^isFilled
!

isFilled: aBoolean
	isFilled := aBoolean.
	self changed
! !

!LineAnnotation methodsFor:'bounds accessing'!

bounds
	^self graphic bounds
! !

!LineAnnotation methodsFor:'displaying'!

displayOn: aGraphicsContext
	aGraphicsContext
		lineWidth: container lineWidth;
		paint: container lineColor.
	isFilled
		ifTrue: [self graphic displayFilledOn: aGraphicsContext]
		ifFalse: [self graphic displayStrokedOn: aGraphicsContext]
! !

!LineAnnotation methodsFor:'initialize-release'!

initialize
	super initialize.
	positionSymbol := #stopPoint.
	directionSymbol := #stopDirection.
	isFilled := true
! !

!LineAnnotation methodsFor:'private'!

direction: aSymbol
	directionSymbol := aSymbol
!

directionVector
	^container perform: directionSymbol
!

graphic
	^self subclassResponsibility
!

location
	^container perform: positionSymbol
!

position: aSymbol
	positionSymbol := aSymbol
! !

!LineAnnotation methodsFor:'testing'!

containsPoint: aPoint
	^isFilled
		ifTrue: [self graphic regionIntersects: (aPoint - 1 corner: aPoint + 1)]
		ifFalse: [self graphic outlineIntersects: (aPoint - 1 corner: aPoint + 1)]
! !

!LineAnnotation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__LineAnnotation.st,v 1.1 2006/03/13 16:55:18 cg Exp $'
! !
