"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

PolylineFigure subclass:#LineFigure
	instanceVariableNames:'annotations'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

LineFigure comment:'LineFigure is a special PolylineFigure that can add annotations to its start and end points. These annotations can be arrows, circles, etc.

Instance Variables:
	annotations     <SequenceableCollection of: LineAnnotation>       the annotations to display on our line

'
!


!LineFigure class methodsFor:'instance creation'!

connect: aFigure to: anotherFigure
	| figure |
	figure := super connect: aFigure to: anotherFigure.
	figure addDependent: aFigure.
	figure addDependent: anotherFigure.
	figure addStopArrow.
	^figure
! !

!LineFigure methodsFor:'accessing'!

addAnnotation: aLineAnnotation
	annotations := annotations copyWith: aLineAnnotation.
	aLineAnnotation container: self.
	self recomputePreferredBounds
!

addStartArrow
	^self addAnnotation: ArrowAnnotation forStart
!

addStopArrow
	^self addAnnotation: ArrowAnnotation forStop
!

menuAt: aPoint
	| menu submenu selectedAnnotation |
	submenu := MenuBuilder new.
	LineAnnotation allSubclasses do:
			[:each |
			| name |
			name := each name asString.
			(name copyFrom: (name size - 9 max: 1) to: name size) = 'Annotation'
				ifTrue: [name := name copyFrom: 1 to: name size - 10].
			submenu
				add: ('Start ' , name) -> [self addAnnotation: each forStart];
				add: ('Stop ' , name) -> [self addAnnotation: each forStop]].
	selectedAnnotation := annotations
				detect: [:each | each containsPoint: aPoint]
				ifNone: [nil].
	selectedAnnotation notNil
		ifTrue:
			[submenu
				line;
				add: 'Remove annotation' -> [self removeAnnotation: selectedAnnotation]].
	menu := super menuAt: aPoint.
	menu addItem: ((MenuItem labeled: 'Annotations') submenu: submenu menu).
	^menu
!

removeAnnotation: aLineAnnotation
	annotations := annotations copyWithout: aLineAnnotation.
	aLineAnnotation container: nil.
	self recomputePreferredBounds
!

startDirection
	| direction |
	direction := points first - (points at: 2).
	^direction = (0 @ 0)
		ifTrue: [1 @ 0]
		ifFalse: [direction unitVector]
!

stopDirection
	| direction |
	direction := points last - (points at: points size - 1).
	^direction = (0 @ 0)
		ifTrue: [1 @ 0]
		ifFalse: [direction unitVector]
! !

!LineFigure methodsFor:'displaying'!

displayOn: aGraphicsContext
	super displayOn: aGraphicsContext.
	annotations do: [:each | each displayOn: aGraphicsContext]
! !

!LineFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	annotations := #()
! !

!LineFigure methodsFor:'private'!

computePreferredBounds
	^annotations inject: super computePreferredBounds
		into: [:sum :each | sum merge: each bounds]
! !

!LineFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__LineFigure.st,v 1.1 2006/03/13 16:57:43 cg Exp $'
! !
