"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

AnimatedDrawing subclass:#NetworkDrawing
	instanceVariableNames:'nodes edgeWeights forces'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Network Drawing'
!


!NetworkDrawing methodsFor:'accessing'!

addNode: aName
	| aNode anOC |
	aNode := NetworkNode name: aName.
	aNode origin: self center.
	nodes add: aNode.
	self add: aNode.
	edgeWeights do: [:oc | oc add: 0].
	anOC := OrderedCollection new: nodes size.
	nodes size timesRepeat: [anOC add: 0].
	edgeWeights add: anOC.
	forces add: 0
!

edgeWeight: aNum from: name1 to: name2
	| i j figure |
	i := self indexOfNode: name1.
	j := self indexOfNode: name2.
	(((edgeWeights at: i) at: j) == 0 and: [aNum ~~ 0])
		ifTrue:
			["Add an edge in the drawing"

			figure := (nodes at: i)
						connectFromPoint: (nodes at: i) center
						to: (nodes at: j)
						at: (nodes at: j) center.
			self add: figure].
	aNum == 0 ifTrue: [self error: 'Need code to delete edge'].
	(edgeWeights at: i) at: j put: aNum.
	^(edgeWeights at: j) at: i put: aNum
!

indexOfNode: name1
	1 to: nodes size do: [:i | (nodes at: i) name = name1 ifTrue: [^i]].
	self error: name1 , ' is not a node'
!

removeNode: aName
	| index |
	index := self indexOfNode: aName.
	self remove: (nodes at: index)
! !

!NetworkDrawing methodsFor:'animation'!

step
	"Move nodes. Each node repulses others with a force that
	is inverse to their distance. Some nodes have springs, and
	the spring force is constant. Not realistic, but it works."

	| center repulsiveConstant stepSize |
	nodes isEmpty ifTrue: [^self].
	center := self center.
	repulsiveConstant := (200000 / nodes size sqrt) truncated.
	1 to: forces size do: [:i | forces at: i put: 0 @ 0].
	1 to: nodes size
		do:
			[:i |
			| n1 sum center1 |
			n1 := nodes at: i.
			center1 := n1 center.   "Make all nodes attracted to the center of the picture"
			sum := forces at: i.
			center ~= center1
				ifTrue: [sum := sum + ((center - center1) unitVector * 10)].
			i + 1 to: nodes size
				do:
					[:j |
					| n2 delta |
					n2 := nodes at: j.
					delta := center1 - n2 center.
					delta = (0 @ 0)
						ifFalse:
							[| spring pairForce distance |
							distance := delta x * delta x + (delta y * delta y).
							pairForce := delta unitVector * repulsiveConstant / distance.
							spring := (edgeWeights at: i) at: j.
							spring ~~ 0
								ifTrue: [pairForce := pairForce - (delta unitVector * spring) truncated].
							sum := sum + pairForce.
							forces at: j put: (forces at: j) - pairForce]].
			forces at: i put: sum].
	stepSize := forces inject: 0
				into: [:max :elem | (elem x abs max: elem y abs) max: max].
	stepSize < 1 ifTrue: [^self].
	nodes with: forces do: [:node :force | node translateBy: force / 10]
! !

!NetworkDrawing methodsFor:'deleting'!

remove: aFigure
	| index |
	super remove: aFigure.
	aFigure class = NetworkNode ifFalse: [^self].
	self changed: #removeNode with: aFigure name.
	index := self indexOfNode: aFigure name.
	nodes removeAtIndex: index.
	edgeWeights do: [:each | each removeAtIndex: index]
! !

!NetworkDrawing methodsFor:'initialize-release'!

initialize
	super initialize.
	nodes := OrderedCollection new.
	forces := OrderedCollection new.
	edgeWeights := OrderedCollection new
! !

!NetworkDrawing class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__NetworkDrawing.st,v 1.1 2006/03/13 16:56:51 cg Exp $'
! !
