"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

DrawingEditor subclass:#NetworkEditor
	instanceVariableNames:'nodes edgeWeights nodeList2 nodeList1 textEditor'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Network Drawing'
!

NetworkEditor comment:'Network editors are a good example of embedding HotDraw in a larger
MVC application, and of how to do animation in HotDraw.  The idea
is that you create a set of nodes and can assign weights to the edges
between the nodes.  An edge with weight zero doesn''''t exist.  The
drawing rearranges itself so that edges act like springs, and nodes
with high-weight edges are closer together than nodes with low-weight
edges.

nodes   <Collection of: NetworkNode>
edgeWeights <Dictionary from: Figure to: Number>
node1, node2 <NetworkNode>  the selected nodes

NetworkEditor open'
!


!NetworkEditor class methodsFor:'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#{UI.FullSpec}
		#window:
		#(#{UI.WindowSpec}
			#label: 'Network'
			#min: #(#{Core.Point} 40 20 )
			#bounds: #(#{Graphics.Rectangle} 512 384 982 707 )
			#isEventDriven: true )
		#component:
		#(#{UI.SpecCollection}
			#collection: #(
				#(#{UI.ArbitraryComponentSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 0 0 0 1 -2 0.65 )
					#name: #drawing
					#flags: 0
					#component: #drawing )
				#(#{UI.SequenceViewSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 2 0.65 0 0.333333 0 1 )
					#name: #nodeList1
					#model: #nodeList1
					#callbacksSpec:
					#(#{UI.UIEventCallbackSubSpec}
						#valueChangeSelector: #changedNode )
					#menu: #nodeMenu1
					#useModifierKeys: true
					#selectionType: #highlight )
				#(#{UI.SequenceViewSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0.333333 2 0.65 0 0.666666 0 1 )
					#name: #nodeList2
					#model: #nodeList2
					#callbacksSpec:
					#(#{UI.UIEventCallbackSubSpec}
						#valueChangeSelector: #changedNode )
					#menu: #nodeMenu2
					#useModifierKeys: true
					#selectionType: #highlight )
				#(#{UI.TextEditorSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0.666666 2 0.65 0 1 0 1 )
					#name: #textEditor
					#model: #textEditor
					#menu: #textMenu
					#tabRequiresControl: true )
				#(#{UI.ResizingSplitterSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 -2 0.65 0 1 2 0.65 )
					#name: #ResizingSplitter1
					#horizontal: true
					#minAboveSize: 30
					#minBelowSize: 30
					#aboveWidgets: 'drawing'
					#belowWidgets: 'nodeList1 nodeList2 textEditor' ) ) ) )
! !

!NetworkEditor methodsFor:'aspects'!

nodeList1
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^nodeList1 isNil
		ifTrue:
			[nodeList1 := SelectionInList new]
		ifFalse:
			[nodeList1]
!

nodeList2
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^nodeList2 isNil
		ifTrue:
			[nodeList2 := SelectionInList new]
		ifFalse:
			[nodeList2]
!

textEditor
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^textEditor isNil
		ifTrue:
			[textEditor := String new asValue]
		ifFalse:
			[textEditor]
! !

!NetworkEditor methodsFor:'drawing description'!

drawingClass
	^NetworkDrawing
!

windowName
	^'Network Editor'
! !

!NetworkEditor methodsFor:'initialize-release'!

initialize
	super initialize.
	nodes := SortedCollection new.
	edgeWeights := Dictionary new.
	self updateLists
! !

!NetworkEditor methodsFor:'nodes'!

addNode
	"Get name of node from user."

	| name |
	name := Dialog request: 'enter name of new node'.
	name isNil ifTrue: [^nil].
	nodes add: name.
	drawing addNode: name.
	self updateLists
!

deleteNode1
	self node1 isNil ifTrue: [^self].
	nodes remove: self node1 ifAbsent: [].
	drawing removeNode: self node1.
	self updateLists
!

deleteNode2
	self node2 isNil ifTrue: [^self].
	nodes remove: self node2 ifAbsent: [].
	drawing removeNode: self node2.
	self updateLists
!

nodeMenu1
	"Answer a Menu for the node list panes."

	^Menu
			labels: 'add node\delete node' withCRs
			lines: #( )
			values: #(#addNode #deleteNode1 )
!

nodeMenu2
	"Answer a Menu for the node list panes."

	^Menu
			labels: 'add node\delete node' withCRs
			lines: #( )
			values: #(#addNode #deleteNode2 )
!

updateLists
	| selection |
	selection := self nodeList1 selection.
	self nodeList1 list: nodes asList.
	self nodeList1 selection: selection.
	selection := self nodeList2 selection.
	self nodeList2 list: nodes asList.
	self nodeList2 selection: selection
! !

!NetworkEditor methodsFor:'private'!

getTableFor: name
	^edgeWeights at: name ifAbsentPut: [Dictionary new]
! !

!NetworkEditor methodsFor:'updating'!

changedNode
	textEditor value: self weight printString
!

update: anAspectSymbol with: aParameter from: aSender
	anAspectSymbol == #removeNode
		ifTrue:
			[nodes remove: aParameter ifAbsent: [].
			edgeWeights removeKey: aParameter ifAbsent: [].
			edgeWeights values do: [:value | value removeKey: aParameter ifAbsent: []].
			self updateLists].
	^super
		update: anAspectSymbol
		with: aParameter
		from: aSender
! !

!NetworkEditor methodsFor:'weights'!

acceptText: aString from: what
	self weight: aString asNumber.
	^true
!

node1
	^self nodeList1 selection
!

node2
	^self nodeList2 selection
!

textMenu
	"Answer a Menu for the node list panes."

	^Menu
			labels: 'again\undo\copy\cut\paste\accept\cancel' withCRs
			lines: #(2 5 )
			values: #(#again #undo #copySelection #cut #paste #acceptText:from: #cancel )
!

weight
	^(edgeWeights at: self node1 ifAbsent: [^0])
		at: self node2 ifAbsent: [0]
!

weight: number
	| node1 node2 |
	node1 := self node1.
	node2 := self node2.
	node1 isNil | node2 isNil ifTrue: [^self].
	(self getTableFor: node1)
		at: node2 put: number.
	(self getTableFor: node2)
		at: node1 put: number.
	drawing
		edgeWeight: number
		from: node1
		to: node2
! !

!NetworkEditor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__NetworkEditor.st,v 1.1 2006/03/13 16:53:58 cg Exp $'
! !
