"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

CachedFigure subclass:#ObjectFigure
	instanceVariableNames:'object inEdges outEdges'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Drawing Inspector'
!

ObjectFigure comment:'ObjectFigure is a Figure that represents an object in a DrawingInspector.  ObjectFigures are wrappers for all other classes of Objects in Smalltalk so that they can be inspected inside a DrawingInspector.

Instance Variables :

object  <Object>  The object for which this is the wrapper.'
!


!ObjectFigure class methodsFor:'instance creation'!

object: anObject
	^self object: anObject at: 0 @ 0
!

object: anObject at: aPoint
	| figure |
	figure := self new.
	^figure
		setObject: anObject origin: aPoint;
		yourself
! !

!ObjectFigure class methodsFor:'parcel load/unload/save'!

obsolete
	[Tool reinitialize] on: Error do: [:ex | ex return: nil].
	super obsolete
!

postLoadActionFor: aParcel
	Tool reinitialize
! !

!ObjectFigure methodsFor:'accessing'!

handles
	| yloc variableNames xloc height refHandles classHandle dependentHandle referencedObject |
	yloc := 'a' asComposedText height * 3 // 2.
	height := 'a' asComposedText height.
	(self shouldDisplayAsString
		or: [(variableNames := self allSlotNames) isEmpty])
			ifTrue: [^Array with: (Handle on: self at: #origin)].
	refHandles := OrderedCollection new.
	variableNames do:
			[:varName |
			referencedObject := self instVar: varName.
			referencedObject notNil
				ifTrue:
					[| xloc1 |
					xloc1 := varName asComposedText width.
					refHandles
						add: (self handleFor: referencedObject offset: (xloc1 + 6) @ yloc)].
			yloc := yloc + height].
	xloc := object class name asComposedText width.
	classHandle := self handleFor: object class offset: (xloc + 4) @ 8.
	dependentHandle := self handleFor: object dependents
				offset: (self extent x // 2) @ 0.
	^refHandles , (Array with: classHandle with: dependentHandle)
!

inspectObject
	self object inspect
!

localInspectObject
	| inspector |
	inspector := Inspector inspect: object.
	self drawing add: (ViewAdapterFigure
				view: (inspector allButOpenInterface: #windowSpec) window component
				in: (10 @ 10 corner: 250 @ 250))
!

menuAt: aPoint
	| mb |
	mb := MenuBuilder new.
	mb
		add: 'inspect...' -> #inspectObject;
		add: 'local inspect' -> #localInspectObject;
		line;
		add: 'other' -> (super menuAt: aPoint).
	^mb menu
!

object
	^object
!

origin: aPoint
	bounds := aPoint extent: self extent
!

title
	^object class name
! !

!ObjectFigure methodsFor:'connection'!

connectFromPoint: myPoint to: aFigure at: figurePoint
	| newFigure |
	newFigure := super
				connectFromPoint: myPoint
				to: aFigure
				at: figurePoint.
	self addOutEdge: newFigure.
	aFigure addInEdge: newFigure.
	newFigure
		addDependent: self;
		addDependent: aFigure.
	newFigure model: (ObjectConnectionModel new
				start: self
				offset: myPoint
				stop: aFigure).
	^newFigure
!

createStartConnectionConstraintFor: newFigure at: myPoint
	PositionConstraint
		send: #startPoint:
		to: newFigure
		with: (MessageSend
				receiver: self
				selector: #offOrigin:
				argument: myPoint)
! !

!ObjectFigure methodsFor:'damage control'!

deletionUpdateFrom: aFigure
	self removeDependent: aFigure.
	aFigure removeDependent: self.
	inEdges remove: aFigure ifAbsent: [].
	outEdges remove: aFigure ifAbsent: []
! !

!ObjectFigure methodsFor:'displaying'!

delete
	self reconnectIndirectReferences.
	super delete
!

displayFigureOn: aGraphicsContext
	self shouldDisplayAsString
		ifTrue: [self displayName displayOn: aGraphicsContext at: self origin]
		ifFalse: [super displayFigureOn: aGraphicsContext]
!

displayName
	| name |
	name := object printString.
	name size > 50 ifTrue: [name := (name copyFrom: 1 to: 50) , '...'].
	^name asComposedText
!

fillCache
	| title vars aGC rect |
	title := self title asComposedText.
	vars := self variableLabels asComposedText.
	cache := Pixmap
				extent: 0 @ title height + (title extent max: vars extent) + 4.
	aGC := cache graphicsContext.
	aGC medium background: LookPreferences defaultForWindows backgroundColor.
	aGC clear.
	title displayOn: aGC at: 0 @ 0.
	aGC paint: ColorValue white.
	rect := 0 @ title height extent: cache extent - (0 @ title height) - 1.
	aGC displayRectangle: rect.
	aGC paint: ColorValue black.
	aGC displayRectangularBorder: rect.
	vars displayOn: aGC at: 2 @ title height.
	^cache asImage
! !

!ObjectFigure methodsFor:'edges'!

addInEdge: anEdge
	inEdges add: anEdge
!

addOutEdge: anEdge
	outEdges add: anEdge
!

createIndirectReferenceFrom: aLineFigure to: outLineFigure
	| stopFigure startFigure edge |
	startFigure := aLineFigure model start.
	stopFigure := outLineFigure model stop.
	edge := startFigure
				connectFromPoint: aLineFigure model offset
				to: stopFigure
				at: stopFigure origin.
	edge lineColor: ColorValue gray.
	edge lineWidth: 2.
	self drawing add: edge
!

reconnectIndirectReferences
	"When an object is deleted, reconnect each input to each output."

	| fromFigures toFigures |
	fromFigures := (inEdges collect: [:each | each model start]) asSet.
	toFigures := (outEdges collect: [:each | each model stop]) asSet.
	fromFigures remove: self ifAbsent: [].
	toFigures remove: self ifAbsent: [].
	inEdges do:
			[:aLineFigure |
			(fromFigures includes: aLineFigure model start)
				ifTrue:
					[fromFigures remove: aLineFigure model start.
					outEdges do:
							[:outLineFigure |
							(toFigures includes: outLineFigure model stop)
								ifTrue:
									[self createIndirectReferenceFrom: aLineFigure to: outLineFigure]]]]
! !

!ObjectFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	inEdges := Set new.
	outEdges := Set new
! !

!ObjectFigure methodsFor:'private'!

allSlotNames
	| showing max |
	showing := 20.
	max := object basicSize.
	^object class allInstVarNames , (object class isVariable
				ifTrue:
					[(max <= showing
						ifTrue: [1 to: max]
						ifFalse: [(1 to: showing // 3 * 2) , (max - (showing // 3) to: max)])
							collect: [:i | i printString]]
				ifFalse: [#()])
!

handleFor: anObject offset: aPoint
	| handle |
	handle := Handle
				on: self
				at: #offOrigin:
				offset: aPoint.
	handle model: (ObjectHandleModel for: anObject offset: aPoint).
	handle toolState: (Tool stateFor: 'Object Figure Handle').
	^handle
!

instVar: aString
	| index |
	index := aString first isDigit
				ifTrue: [Integer readFromString: aString]
				ifFalse: [object class allInstVarNames indexOf: aString].
	^aString first isDigit
		ifTrue: [object basicAt: index]
		ifFalse: [object instVarAt: index]
!

setObject: anObject origin: aPoint
	object := anObject.
	self fillCache.
	self origin: aPoint
!

variableLabels
	| aStream |
	aStream := WriteStream on: String new.
	self allSlotNames do:
			[:each |
			aStream
				nextPutAll: each;
				cr].
	aStream isEmpty ifTrue: [^object printString] ifFalse: [aStream skip: -1].
	^aStream contents
! !

!ObjectFigure methodsFor:'testing'!

shouldDisplayAsString
	^object isLiteral or: [#(#Point #Rectangle) includes: object class name]
! !

!ObjectFigure methodsFor:'transforming'!

computePreferredBounds
	^bounds isNil
		ifTrue: [0 @ 0 extent: self extent]
		ifFalse: [bounds origin extent: self extent]
!

extent
	^self shouldDisplayAsString
		ifTrue: [self displayName extent]
		ifFalse: [self cache extent]
! !

!ObjectFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ObjectFigure.st,v 1.1 2006/03/13 16:57:18 cg Exp $'
! !
