"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Object subclass:#ObjectParser
	instanceVariableNames:'bindings drawingEditor'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-ObjectWorld'
!

ObjectParser comment:'This is the parser and controller for the StringHolder part of ObjectWorld.   Hacks have been added to ''''access'''' and doItProcessing to allow user''''s to enter text such as ''''fred moveRight!!!!'''' and have the ''''!!!!'''' cause a doIt.

Instance Variables
bindings :-
	A dictionary of all of the named objects that we currently know about.  This dictionary always contains a binding to map ''''self'''' to our model (i.e. an ObjectWorld object)

'
!


!ObjectParser methodsFor:'initialize-release'!

drawingEditor: aModel
	bindings at: 'self' put: aModel.
	drawingEditor := aModel
!

initialize
	| table |
	super initialize.
	bindings := Dictionary new.
	table := self dispatchTable.
	table
		bindValue: #selectAndDoItKey:
		to: ESC
		followedBy: ESC.
	dispatchTable := table
!

initializeMenu
	menuHolder := (Menu
				labelList: #(#(#again #undo) #(#copy #cut #paste) #(#browse) #('do it' 'print it' 'inspect') #(#accept #cancel) #(#hardcopy))
				values: #(#again #undo #copySelection #cut #paste #browse #doIt #printIt #inspectIt #accept #cancel #hardcopy))
					asValue
! !

!ObjectParser methodsFor:'private'!

browse
	drawingEditor browse
!

evaluateSelection
	"Evaluate the current text selection as an expression "

	| result selectionStart oldTextSize selection |
	selectionStart := self selectionStartIndex.
	oldTextSize := self text size.
	selection := self selection.
	result := WorkspaceCompiler new
				evaluate: self selectionAsStream
				inWorkspace: bindings
				receiver: drawingEditor
				notifying: self
				ifFail: [].
	self selection asString = selection asString
		ifFalse:
			[self selectFrom: selectionStart
				to: selectionStart + selection size - 1 + (self text size - oldTextSize)        "Reselect doIt range after compiler interaction"].      "Smalltalk logChange: self selection string."
	^result
!

selectAndDoItKey: aKey
	"Select the text that has been typed recently and DoIt"

	self selectCurrentTypeInKey: aKey.
	self evaluateSelection
! !

!ObjectParser methodsFor:'registration'!

keys
	"Return all of the keys we have bindings for"

	^bindings keys
!

register: anObject as: aString
	"Register aString as a pointer to anObject"

	bindings at: aString asSymbol put: anObject
!

unregister: anObject
	"Remove anObject from the binding list"

	bindings removeKey: anObject model name asSymbol
! !

!ObjectParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ObjectParser.st,v 1.1 2006/03/13 16:54:30 cg Exp $'
! !
