"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

DrawingEditor subclass:#ObjectWorld
	instanceVariableNames:'parser commandText'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-ObjectWorld'
!

ObjectWorld comment:'I am the model in an ObjectWorld application.  I am like a subclass of both ValueHolder and DrawingEditor, because one of my views is a ComposedTextView and the other one is a DrawingView.  However, I copy code from DrawingEditor and am a subclass of ValueHolder.

In ObejctWorld, if a user types ''''self doSomething'''' the message gets sent to me.  Things users are allowed to do are all contained in my ''''public'''' protocol.  Special commands are trapped in the ''''private'''' protocol (i.e. ''''self remove'''' which does *not* remove ObjectWorld).

ObjectParser and ObjectWorldMetaFigure are the other important classes of the ObjectWorld application.  ObjectParser is the controller of the text pane, and it parses user input and directs messages to the ObjectWorld, to the drawing, and to the individual figures in the drawing.  ObjectWorldMetaFigure implements ObjectWorld-specific protocol for figures.

Instance variables:
parser :-
	The object parsing ObjectWorld commands is also the controller of our text pane.
drawingView     The view of our drawing, needed for flashing, etc.
drawing The ObjectWorldDrawing.

tools, currentTool, fileName:   variables from DrawingEditor'
!


!ObjectWorld class methodsFor:'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#{UI.FullSpec}
		#window:
		#(#{UI.WindowSpec}
			#label: 'Object World'
			#min: #(#{Core.Point} 40 20 )
			#bounds: #(#{Graphics.Rectangle} 557 381 1044 820 )
			#isEventDriven: true )
		#component:
		#(#{UI.SpecCollection}
			#collection: #(
				#(#{UI.TextEditorSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 0 0 0 1 0 0.3 )
					#name: #commandText
					#model: #commandText
					#tabRequiresControl: true )
				#(#{UI.ArbitraryComponentSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 28 0.3 0 1 0 1 )
					#colors:
					#(#{UI.LookPreferences}
						#setBackgroundColor: #(#ColorValue #white ) )
					#component: #drawing )
				#(#{UI.ArbitraryComponentSpec}
					#layout: #(#{Graphics.LayoutFrame} 0 0 0 0.3 0 1 28 0.3 )
					#flags: 0
					#component: #toolbar ) ) ) )
! !

!ObjectWorld methodsFor:'ObjectParser protocol'!

register: anObject as: aString
	parser register: anObject as: aString
!

unregister: anObject
	parser unregister: anObject
! !

!ObjectWorld methodsFor:'accessing'!

parser
	^parser
! !

!ObjectWorld methodsFor:'aspects'!

commandText
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^commandText isNil
		ifTrue:
			[commandText := String new asValue]
		ifFalse:
			[commandText]
!

subcanvas
	^self
! !

!ObjectWorld methodsFor:'drawing description'!

drawingClass
	^ObjectWorldDrawing
!

windowName
	^'Object World'
! !

!ObjectWorld methodsFor:'interface opening'!

postBuildWith: aBuilder
	| editor oldController |
	super postBuildWith: aBuilder.
	editor := (builder componentAt: #commandText) widget.
	oldController := editor controller.
	editor controller: (parser := (ObjectParser new)
						drawingEditor: self;
						keyboardProcessor: oldController keyboardProcessor;
						yourself)
! !

!ObjectWorld methodsFor:'private'!

remove
	"Catch an attempt by the user to do a 'self remove!!!!' from the
	ObjectWorld text window."

	parser view flash
! !

!ObjectWorld methodsFor:'public'!

clearAnimation
	"Clear list of figures being animated"

	drawing stopAll
!

pause
	"Temporarily pause animation"

	drawing pause
!

refresh
	"If the screen has gotten out of sync refresh it"

	drawing repairDamage
!

resume
	"Restart animation"

	drawing resume
!

setFont: aFontSize
	"set a new font for everything in the Universe"

	TextAttributes setDefaultTo: aFontSize asSymbol.
	TextAttributes resetViews
!

who
	"Show the objects we know about"

	| mb |
	mb := MenuBuilder new.
	parser keys do: [:each | mb add: each -> each].
	mb menu startUp
! !

!ObjectWorld methodsFor:'user interface'!

browse
	"Browse only the methods users are allowed to see"

	((#{Browser.BrowserEnvironment}
		valueOrDo: [^self error: 'No Browser Loaded']) new
		forClass: self class
		protocols: #(#public)) openEditor
! !

!ObjectWorld class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ObjectWorld.st,v 1.1 2006/03/13 16:58:23 cg Exp $'
! !
