"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

ViewAdapterFigure subclass:#PERTEventFigure
	instanceVariableNames:'outgoing'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-PERTChart'
!

PERTEventFigure comment:'PERTEventFigure represents an event in the drawing. It simply adapts a PERTEventAppModel to the figure protocol.

Instance Variables:
	outgoing        <IdentityDictionary key: LineFigure value: PERTEventFigure>       mapping of lines to the figure they connect, when a line is deleted we must update the figure its connected to

'
!


!PERTEventFigure methodsFor:'accessing'!

handles
	| handles |
	handles := OrderedCollection withAll: (TrackHandle allCornersOf: self).
	handles add: (Handle connectionOn: self at: #rightCenter).
	^handles
! !

!PERTEventFigure methodsFor:'changing'!

deletionUpdateFrom: aFigure
	model disconnectFrom: (outgoing at: aFigure ifAbsent: [^self]) model.
	outgoing removeKey: aFigure.
	super deletionUpdateFrom: aFigure
! !

!PERTEventFigure methodsFor:'connection'!

canConnectFromFigure: aFigure
	^self class == aFigure class
!

canConnectToFigure: aFigure
	self == aFigure ifTrue: [^false].
	self class == aFigure class ifFalse: [^false].
	^(aFigure isConnectedTo: self) not
		and: [(self isConnectedTo: aFigure) not]
!

connectFromPoint: myPoint to: aFigure at: figurePoint
	| newFigure |
	newFigure := super
				connectFromPoint: myPoint
				to: aFigure
				at: figurePoint.
	outgoing at: newFigure put: aFigure.
	newFigure addDependent: self.
	self model connectTo: aFigure model.
	^newFigure
! !

!PERTEventFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	super displayFigureOn: aGraphicsContext copy.
	aGraphicsContext
		paint: self lineColor;
		lineWidth: self lineWidth.
	self bounds displayStrokedOn: aGraphicsContext
! !

!PERTEventFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	model := PERTEventAppModel new.
	self view: (model allButOpenInterface: #windowSpec) window component
		in: (0 @ 0 corner: 100 @ 90).
	self opaque.
	outgoing := IdentityDictionary new
! !

!PERTEventFigure methodsFor:'testing'!

isConnectedTo: aFigure
	^self == aFigure
		or: [outgoing contains: [:each | each isConnectedTo: aFigure]]
! !

!PERTEventFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__PERTEventFigure.st,v 1.1 2006/03/13 16:59:14 cg Exp $'
! !
