"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Object subclass:#PositionConstraint
	instanceVariableNames:'receiver position message isUpdating'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Constraints'
!

PositionConstraint comment:'PositionConstraint represents a constraint between a position (a message send) and a message (symbol) of a receiver (figure). Whenever either figure changes, the constraint is re-evaluated, and the receiver is moved to the position.

Instance Variables:
	isUpdating      <Boolean> are we currently updating the figure (avoid infinite recursion)
	message <Symbol>  the symbol that is performed on receiver to set its position
	position        <MessageSend>     a message send that when evaluated, returns the new position for receiver
	receiver        <Figure>  the figure to be constrained

'
!


!PositionConstraint class methodsFor:'instance creation'!

send: aSelector to: anObject with: aMessageSend
	^(self new) location: aMessageSend
			receiver: anObject
			sending: aSelector;
		evaluate;
		yourself
! !

!PositionConstraint methodsFor:'initialize-release'!

location: aMessageSend receiver: anObject sending: aSelector
	position := aMessageSend.
	(position receiver isKindOf: Figure)
		ifTrue: [position receiver addDependent: self].
	receiver := anObject.
	receiver addDependent: self.
	message := aSelector.
	isUpdating := false
!

release
	position receiver removeDependent: self.
	receiver removeDependent: self.
	super release
! !

!PositionConstraint methodsFor:'updating'!

evaluate
	message last == $:
		ifTrue: [receiver perform: message with: position value]
		ifFalse: [receiver translateBy: position value - (receiver perform: message)]
!

update: anAspectSymbol with: aParameter from: aSender
	"If one of our figures was removed from the drawing, disconnect ourself from the figures."

	anAspectSymbol == #deleted ifTrue: [^self release].
	aSender == receiver
		ifTrue:
			[isUpdating
				ifFalse:
					[isUpdating := true.
					[self evaluate] valueNowOrOnUnwindDo: [isUpdating := false]]]
		ifFalse: [self evaluate]
! !

!PositionConstraint class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__PositionConstraint.st,v 1.1 2006/03/13 16:59:00 cg Exp $'
! !
