"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

RectangleFigure subclass:#RoundedRectangleFigure
	instanceVariableNames:'inset'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

RoundedRectangleFigure comment:'RoundedRectangleFigure is a rectangle that has rounded corners.

Instance Variables:
	inset   <Point>   the offset for the ellipse that draws the rounded corners (if greater than our extent // 2, then we are an ellipse)

'
!


!RoundedRectangleFigure methodsFor:'accessing'!

handles
	| handles |
	handles := OrderedCollection withAll: super handles.
	handles add: ((TrackHandle on: self at: #insetPosition)
				moveBlock: [:aPoint | self inset: (aPoint - self origin max: 0 @ 0)];
				yourself).
	^handles
! !

!RoundedRectangleFigure methodsFor:'displaying'!

displayFilledOn: aGraphicsContext
	| xinset yinset |
	aGraphicsContext paint: self fillColor.
	xinset := (self inset x - 1) @ 0.
	(self origin + xinset corner: self corner - xinset)
		displayFilledOn: aGraphicsContext.
	yinset := 0 @ (self inset y - 1).
	(self origin + yinset corner: self corner - yinset)
		displayFilledOn: aGraphicsContext.
	self arcs do: [:each | each displayFilledOn: aGraphicsContext]
!

displayOutlineOn: aGraphicsContext
	| xinset yinset |
	aGraphicsContext
		lineWidth: self lineWidth;
		paint: self lineColor.
	xinset := (self inset x - 1) @ 0.
	(LineSegment from: self origin + xinset to: self topRight - xinset)
		displayStrokedOn: aGraphicsContext.
	(LineSegment from: self bottomLeft + xinset to: self corner - xinset)
		displayStrokedOn: aGraphicsContext.
	yinset := 0 @ (self inset y - 1).
	(LineSegment from: self origin + yinset to: self bottomLeft - yinset)
		displayStrokedOn: aGraphicsContext.
	(LineSegment from: self topRight + yinset to: self corner - yinset)
		displayStrokedOn: aGraphicsContext.
	self arcs do: [:each | each displayStrokedOn: aGraphicsContext]
! !

!RoundedRectangleFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	inset := 20 @ 20
! !

!RoundedRectangleFigure methodsFor:'private'!

arcs
	(inset x <= 0 or: [inset y <= 0]) ifTrue: [^#()].
	^Array
		with: (EllipticalArc
				boundingBox: (self origin corner: self origin + (self inset * 2))
				startAngle: 180
				sweepAngle: 90)
		with: (EllipticalArc
				boundingBox: (self corner - (self inset * 2) corner: self corner)
				startAngle: 0
				sweepAngle: 90)
		with: (EllipticalArc
				boundingBox: (Rectangle
						vertex: self bottomLeft + (self inset x @ self inset y negated * 2)
						vertex: self bottomLeft)
				startAngle: 90
				sweepAngle: 90)
		with: (EllipticalArc
				boundingBox: (Rectangle
						vertex: self topRight + (self inset x negated @ self inset y * 2)
						vertex: self topRight)
				startAngle: 270
				sweepAngle: 90)
!

inset
	inset x <= 0 ifTrue: [^0 @ 0].
	inset y <= 0 ifTrue: [^0 @ 0].
	^inset min: self extent // 2
!

inset: aPoint
	inset := aPoint.
	self changed
!

insetPosition
	^self origin + inset
! !

!RoundedRectangleFigure methodsFor:'testing'!

containsPoint: aPoint
	(super containsPoint: aPoint) ifFalse: [^false].
	(aPoint x between: self origin x + self inset x
		and: self corner x - self inset x) ifTrue: [^true].
	(aPoint y between: self origin y + self inset y
		and: self corner y - self inset y) ifTrue: [^true].
	^self arcs
		contains: [:each | each regionIntersects: (aPoint - 1 extent: 2 @ 2)]
! !

!RoundedRectangleFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__RoundedRectangleFigure.st,v 1.1 2006/03/13 16:53:39 cg Exp $'
! !
