"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

EndToolState subclass:#ToolState
	instanceVariableNames:'keyPress keyRelease mouseMove redButtonPress redButtonRelease
		yellowButtonPress yellowButtonRelease doubleClick immediate'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Tools'
!

ToolState comment:'ToolState is a state in a Tool''s state machine. It inherits its action from its superclass, and has several transition tables. One for each type of event that can be processed.

Instance Variables:
	doubleClick     <TransitionTable> where to go to for a double click event
	keyPress        <TransitionTable> ...
	keyRelease      <TransitionTable>
	mouseMove       <TransitionTable>
	redButtonPress  <TransitionTable>
	redButtonRelease        <TransitionTable>
	yellowButtonPress       <TransitionTable>
	yellowButtonRelease     <TransitionTable>

'
!


!ToolState methodsFor:'accessing'!

doubleClick
	^doubleClick
!

doubleClick: aTransitionTable
	doubleClick := aTransitionTable
!

immediate
	^immediate
!

immediate: aTransitionTable
	immediate := aTransitionTable
!

keyPress
	^keyPress
!

keyPress: aTransitionTable
	keyPress := aTransitionTable
!

keyRelease
	^keyRelease
!

keyRelease: aTransitionTable
	keyRelease := aTransitionTable
!

mouseMove
	^mouseMove
!

mouseMove: aTransitionTable
	mouseMove := aTransitionTable
!

name
	^name isNil
		ifTrue: [self identityHash printString]
		ifFalse: [name]
!

redButtonPress
	^redButtonPress
!

redButtonPress: aTransitionTable
	redButtonPress := aTransitionTable
!

redButtonRelease
	^redButtonRelease
!

redButtonRelease: aTransitionTable
	redButtonRelease := aTransitionTable
!

yellowButtonPress
	^yellowButtonPress
!

yellowButtonPress: aTransitionTable
	yellowButtonPress := aTransitionTable
!

yellowButtonRelease
	^yellowButtonRelease
!

yellowButtonRelease: aTransitionTable
	yellowButtonRelease := aTransitionTable
! !

!ToolState methodsFor:'events'!

blueButtonPressedEvent: aMouseButtonEvent
	^nil
!

blueButtonReleasedEvent: aMouseButtonEvent
	^nil
!

boundsEvent: anEvent
	^nil
!

closeEvent: aCloseEvent
	^nil
!

doubleClickEvent: anEvent
	^doubleClick
!

enterEvent: anEvent
	^nil
!

exitEvent: anEvent
	"Treat exiting a drawing as releasing the red button."

	^redButtonRelease
!

expandEvent: anEvent
	^nil
!

exposeEvent: anEvent
	^nil
!

immediateEvent: anEvent
	^immediate
!

keyPressedEvent: anEvent
	^keyPress
!

keyReleasedEvent: anEvent
	^keyRelease
!

mouseMovedEvent: aMouseMovedEvent
	^mouseMove
!

mouseWheelEvent: anEvent
	^nil
!

moveEvent: aMoveEvent
	^nil
!

pseudoEvent: aPseudoEvent
	^nil
!

redButtonPressedEvent: aMouseButtonEvent
	^redButtonPress
!

redButtonReleasedEvent: aMouseButtonEvent
	^redButtonRelease
!

resizeEvent: anEvent
	^nil
!

unknownEvent: anEvent
	^nil
!

windowEnterEvent: anEvent
	^nil
!

windowExitEvent: anEvent
	^nil
!

yellowButtonPressedEvent: aMouseButtonEvent
	^yellowButtonPress
!

yellowButtonReleasedEvent: aMouseButtonEvent
	^yellowButtonRelease
! !

!ToolState methodsFor:'testing'!

isEndState
	^false
! !

!ToolState methodsFor:'tool-accessing'!

immediateStateForEvent: anEvent tool: aTool
	| transitionTable |
	transitionTable := self immediateEvent: anEvent.
	transitionTable isNil ifTrue: [^nil].
	^transitionTable nextStateForTool: aTool event: anEvent
!

nextStateForEvent: anEvent tool: aTool
	| transitionTable |
	transitionTable := anEvent dispatchTo: self.
	transitionTable isNil ifTrue: [^nil].
	^transitionTable nextStateForTool: aTool event: anEvent
! !

!ToolState class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ToolState.st,v 1.1 2006/03/13 16:52:39 cg Exp $'
! !
