"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

EllipseFigure subclass:#ToolStateFigure
	instanceVariableNames:''
	classVariableNames:'NameIndex'
	poolDictionaries:''
	category:'HotDraw-Tool Creation GUI'
!


!ToolStateFigure class methodsFor:'instance creation'!

createAt: aPoint
	| stateName |
	NameIndex := NameIndex + 1.
	stateName := 'state ' , NameIndex printString.
	^(self new)
		name: stateName at: aPoint;
		yourself
!

fromModel: aToolStateModel
	| figure |
	figure := self new.
	figure
		model: aToolStateModel;
		resetFigure.
	^figure
! !

!ToolStateFigure methodsFor:'accessing'!

menuAt: aPoint
	| mb isEndState isExternal |
	isEndState := model isEndState.
	isExternal := model isExternal.
	mb := MenuBuilder new.
	mb
		add: 'Rename...' -> #rename;
		add: 'Edit command...' -> #editCommand;
		line;
		add: (isEndState ifTrue: ['Normal state'] ifFalse: ['End state'])
					-> [model isEndState: isEndState not];
		add: (isExternal ifTrue: ['Define here'] ifFalse: ['Define elsewhere'])
					-> [model isExternal: isExternal not].
	^mb menu
!

name
	^self model name
! !

!ToolStateFigure methodsFor:'actions'!

editCommand
	(ToolStateCommandEditor state: model) open
!

rename
	| newName |
	newName := Dialog request: 'Enter state name:'
				initialAnswer: self model name.
	newName isEmpty ifTrue: [^self].
	self model name: newName
! !

!ToolStateFigure methodsFor:'changing'!

changed
	model position: self origin.
	super changed
! !

!ToolStateFigure methodsFor:'connection'!

canConnectFromFigure: aFigure
	^self class = aFigure class
!

canConnectToFigure: aFigure
	^self canBeConnected and: [self class = aFigure class]
!

connectFromPoint: myPoint to: aFigure at: figurePoint
	| newFigure |
	newFigure := super
				connectFromPoint: myPoint
				to: aFigure
				at: figurePoint.
	self == aFigure
		ifTrue:
			[newFigure
				addPoint: self rightCenter + (10 @ 10) beforeIndex: 2;
				addPoint: self rightCenter + (10 @ -10) beforeIndex: 2].
	(newFigure model)
		from: self model;
		to: aFigure model.
	^newFigure
!

connectionFigureClass
	^StateTransitionFigure
! !

!ToolStateFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	| text |
	super displayFigureOn: aGraphicsContext.
	text := self name asComposedText.
	text displayOn: aGraphicsContext
		at: (self extent - text extent) // 2 + self origin
! !

!ToolStateFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	self model: ToolStateModel new.
	bounds := 0 @ 0 extent: 0 @ 0.
	self opaque
!

name: aString at: aPoint
	bounds := aPoint extent: 0 @ 0.
	(self model)
		name: aString;
		position: aPoint
! !

!ToolStateFigure methodsFor:'testing'!

isStateFigure
	^true
! !

!ToolStateFigure methodsFor:'updating'!

computePreferredBounds
	| newExtent |
	newExtent := self model name asComposedText extent + (10 @ 10).
	^self origin extent: newExtent
!

resetEndState
	self lineWidth: (self model isEndState ifTrue: [2] ifFalse: [1])
!

resetExternal
	model isExternal
		ifTrue:
			[self lineWidth: 1.
			self lineColor: ColorValue red]
		ifFalse:
			[self lineColor: ColorValue black.
			self resetEndState]
!

resetFigure
	self
		computePreferredBounds;
		translateTo: model position;
		resetEndState;
		resetExternal
!

update: anAspectSymbol with: aParameter from: aSender
	anAspectSymbol == #position ifTrue: [^self].
	anAspectSymbol == #name ifTrue: [^self recomputePreferredBounds].
	anAspectSymbol == #isEndState ifTrue: [^self resetEndState].
	anAspectSymbol == #isExternal ifTrue: [^self resetExternal].
	^super
		update: anAspectSymbol
		with: aParameter
		from: aSender
! !

!ToolStateFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ToolStateFigure.st,v 1.1 2006/03/13 16:58:53 cg Exp $'
! !
