"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

DrawingEditor subclass:#ToolStateMachineEditor
	instanceVariableNames:'states connections'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Tool Creation Models'
!


!ToolStateMachineEditor class methodsFor:'instance creation'!

openOnModel: aToolStateMachineModel
	^self openWithStates: aToolStateMachineModel states
		connections: aToolStateMachineModel connections
!

openWithStates: toolStateModels connections: toolStateTransitionModels
	| figureMapping figures editor |
	figures := OrderedCollection new.
	figureMapping := IdentityDictionary new.
	toolStateModels do:
			[:each |
			| figure |
			figure := ToolStateFigure fromModel: each.
			figureMapping at: each put: figure.
			figures add: figure].
	toolStateTransitionModels do:
			[:each |
			figures add: (StateTransitionFigure
						fromModel: each
						connecting: (figureMapping at: each from)
						to: (figureMapping at: each to))].
	editor := self new.
	editor drawing addAll: figures.
	editor open.
	^editor
!

states: toolStateModels connections: toolStateTransitionModels
	^self new states: toolStateModels connections: toolStateTransitionModels
! !

!ToolStateMachineEditor methodsFor:'accessing'!

addConnection: aToolStateTransitionMetafigure
	connections add: aToolStateTransitionMetafigure
!

addState: aToolStateMetafigure
	states add: aToolStateMetafigure
!

connections
	^connections copy
!

removeConnection: aToolStateTransitionMetafigure
	connections remove: aToolStateTransitionMetafigure ifAbsent: []
!

removeState: aToolStateMetafigure
	states remove: aToolStateMetafigure
!

states
	^states copy
! !

!ToolStateMachineEditor methodsFor:'actions'!

checkForTypeConflictsIn: transitions
	^transitions contains:
			[:transition |
			transitions
				contains: [:each | transition ~~ each and: [transition type conflictsWith: each type]]]
!

compileForClass: aClass selector: aSymbol ifFail: aBlock
	| codeStream string |
	self verifyIfFail:
			[:error |
			aBlock value: error.
			^self].
	codeStream := String new writeStream.
	codeStream
		nextPutAll: aSymbol;
		cr.
	self writeCommentOn: codeStream.
	self writeStateDefinitionsOn: codeStream.
	self writeTransitionsOn: codeStream.
	string := codeStream contents.
	aClass
		compile: string
		classified: #'tool states'
		notifying: nil
!

transitionBuckets
	| connectionBuckets |
	connectionBuckets := Dictionary new.
	connections do:
			[:each |
			((connectionBuckets at: each from ifAbsentPut: [Dictionary new])
				at: each type event
				ifAbsentPut:
					[SortedCollection sortBlock: [:a :b | a type priority < b type priority]])
					add: each].
	^connectionBuckets
!

verifyIfFail: aBlock
	| names connectionBuckets |
	names := Set new.
	states do:
			[:each |
			(names includes: each name)
				ifTrue: [aBlock value: each name , ' is defined multiple times']
				ifFalse: [names add: each name]].
	connectionBuckets := self transitionBuckets.
	connectionBuckets keysAndValuesDo:
			[:state :eventTable |
			eventTable keysAndValuesDo:
					[:event :transitions |
					(self checkForTypeConflictsIn: transitions)
						ifTrue:
							[aBlock value: state name , ' has transition conflicts for event ' , event]]]
! !

!ToolStateMachineEditor methodsFor:'compilation'!

writeCommentOn: codeStream
	codeStream
		nextPutAll: '   "This method was automatically generated by the HotDraw ToolStateMachineEditor.';
		cr;
		nextPutAll: '   To edit this method, evaluate the comment below"';
		cr;
		cr.
	self writeDrawingDefinitionOn: codeStream
!

writeDrawingDefinitionOn: codeStream
	codeStream
		nextPutAll: '   "| states transitions stateTable |';
		cr;
		nextPutAll: '   states := '.
	states storeOn: codeStream.
	codeStream
		nextPut: $.;
		cr;
		nextPutAll: '   stateTable := Dictionary new.';
		cr;
		nextPutAll: '   states do: [:each | stateTable at: each name put: each].';
		cr;
		cr;
		nextPutAll: '   transitions := OrderedCollection new.';
		cr.
	connections do:
			[:each |
			codeStream nextPutAll: '        transitions add: (ToolStateTransitionModel new'.
			each writeStoreStringOn: codeStream.
			codeStream
				nextPutAll: ').';
				cr].
	codeStream
		nextPutAll: '   ToolStateMachineEditor openWithStates: states connections: transitions"';
		cr;
		cr
!

writeStateDefinitionsOn: codeStream
	states do: [:each | each writeDefinitionOn: codeStream]
!

writeTransitions: transitions on: codeStream
	transitions first writeDefinitionOn: codeStream.
	2 to: transitions size
		do: [:i | (transitions at: i) writeContinuedDefinitionOn: codeStream].
	codeStream nextPutAll: '; yourself'
!

writeTransitionsOn: codeStream
	self transitionBuckets keysAndValuesDo:
			[:state :events |
			events keysAndValuesDo:
					[:event :transitions |
					codeStream
						nextPutAll: '   (Tool stateFor: ''';
						nextPutAll: state name;
						nextPutAll: ''') ';
						nextPutAll: event;
						nextPutAll: ': ('.
					self writeTransitions: transitions on: codeStream.
					codeStream
						nextPutAll: ').';
						cr]]
! !

!ToolStateMachineEditor methodsFor:'drawing description'!

drawingClass
	^ToolStateMachineDrawing
!

windowName
	^'Tool State Machine Editor'
! !

!ToolStateMachineEditor methodsFor:'initialize-release'!

initialize
	super initialize.
	states := OrderedCollection new.
	connections := OrderedCollection new
!

states: toolStateModels connections: toolStateTransitionModels
	states := toolStateModels.
	connections := toolStateTransitionModels
! !

!ToolStateMachineEditor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ToolStateMachineEditor.st,v 1.1 2006/03/13 16:57:04 cg Exp $'
! !
