"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Controller subclass:#ToolbarController
	instanceVariableNames:'pressing pressingItem'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Toolbar'
!

ToolbarController comment:'ToolbarController is the controller for the ToolbarView.

Instance Variables:
	pressing        <Boolean> are we pressing a button?
	pressingItem    <ButtonDescription | nil> the items we''re pressing

'
!


!ToolbarController methodsFor:'basic control sequence'!

controlTerminate
	pressing ifFalse: [view over: nil].
	^super controlTerminate
! !

!ToolbarController methodsFor:'control defaults'!

controlActivity
	self viewHasCursor
		ifTrue:
			[self sensor redButtonPressed
				ifTrue: [self redButtonActivity]
				ifFalse: [self mouseMovedOver: self sensor cursorPoint]].
	^super controlActivity
!

endSelectionAt: point
	(pressingItem notNil and: [pressingItem == (view itemUnder: point)])
		ifTrue: [self requestValueChange ifTrue: [model selection: pressingItem]].
	pressingItem := nil.
	view selecting: nil.
	pressing := false
!

mouseMovedOver: point
	| item |
	item := view itemUnder: point.
	(item notNil and: [item isEnabled])
		ifTrue: [view over: item]
		ifFalse: [view over: nil]
!

moveSelectionTo: point
	pressingItem == (view itemUnder: point)
		ifTrue: [view selecting: pressingItem]
		ifFalse: [view selecting: nil]
!

redButtonActivity
	self startSelectionAt: self sensor cursorPoint.
	[self sensor redButtonPressed]
		whileTrue:
			[self moveSelectionTo: self sensor cursorPoint.
			view repairDamage].
	self endSelectionAt: self sensor cursorPoint
!

startSelectionAt: point
	pressing := true.
	pressingItem := view itemUnder: point.
	(pressingItem notNil and: [pressingItem isEnabled])
		ifTrue: [view over: pressingItem]
		ifFalse: [pressingItem := nil].
	pressingItem notNil ifTrue: [view selecting: pressingItem]
! !

!ToolbarController methodsFor:'events'!

exitEvent: event
	super exitEvent: event.
	self endSelectionAt: -1 @ -1.
	self mouseMovedOver: -1 @ -1
!

mouseMovedEvent: anEvent
	pressing
		ifTrue: [self moveSelectionTo: (self sensor mousePointFor: anEvent)]
		ifFalse: [self mouseMovedOver: (self sensor mousePointFor: anEvent)]
!

redButtonPressedEvent: event
	self startSelectionAt: (self sensor mousePointFor: event)
!

redButtonReleasedEvent: event
	self endSelectionAt: (self sensor mousePointFor: event)
! !

!ToolbarController methodsFor:'initialize-release'!

initialize
	super initialize.
	pressing := false
! !

!ToolbarController class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ToolbarController.st,v 1.1 2006/03/13 16:56:24 cg Exp $'
! !
