"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

View subclass:#ToolbarView
	instanceVariableNames:'horizontal over selecting'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Toolbar'
!

ToolbarView comment:'ToolbarView is the view that displays the toolbar.

Instance Variables:
	horizontal      <Boolean> description of horizontal
	over    <ButtonDescription>       the item the mouse cursor is over
	selecting       <ButtonDescription>       the item we''re tentatively selecting

'
!


!ToolbarView methodsFor:'accessing'!

buttons
	^self model list
!

horizontal
	^horizontal
!

horizontal: aBoolean
	horizontal := aBoolean.
	self invalidate
!

invalidateOverIcon
	over notNil
		ifTrue:
			[self
				invalidateRectangle: ((self boundsFor: over) expandedBy: self spacer)
				repairNow: false]
!

invalidateSelectedIcon
	selecting notNil
		ifTrue:
			[self
				invalidateRectangle: ((self boundsFor: selecting) expandedBy: self spacer)
				repairNow: false]
!

over
	^over
!

over: anObject
	over == anObject ifTrue: [^self].
	self invalidateOverIcon.
	over := anObject.
	self invalidateOverIcon
!

selecting: anObject
	selecting == anObject ifTrue: [^self].
	self invalidateSelectedIcon.
	selecting := anObject.
	self invalidateSelectedIcon
! !

!ToolbarView methodsFor:'controller accessing'!

defaultControllerClass
	^ToolbarController
!

itemUnder: aPoint
	self buttonsAndPositionsDo:
			[:each :point |
			((point extent: each icon extent) containsPoint: aPoint) ifTrue: [^each]].
	^nil
!

mouseMovedOver: aPoint
	self over: (self itemUnder: aPoint)
! !

!ToolbarView methodsFor:'displaying'!

display: each at: point on: aGraphicsContext
        | inset |
        inset := selecting == each or: [self isSelected: each].

        each icon
                displayOn: aGraphicsContext
                at: point + (inset ifTrue: [1] ifFalse: [0]).
Smalltalk isSmalltalkX ifTrue:[^ self ].
        (each == over and: [inset not])
                ifTrue:
                        [self overBorder
                                displayOn: aGraphicsContext
                                forDisplayBox: ((point extent: each icon extent) expandedBy: 3 @ 3)
                                using: self].
        inset
                ifTrue:
                        [self selectedBorder
                                displayOn: aGraphicsContext
                                forDisplayBox: ((point extent: each icon extent) expandedBy: 3 @ 3)
                                using: self]
!

displayBackgroundOn: aGraphicsContext
	aGraphicsContext paint: self backgroundColor.
	aGraphicsContext displayRectangle: aGraphicsContext clippingBounds
!

displayForegroundOn: aGraphicsContext
	self buttonsAndPositionsDo:
			[:each :point |
			self
				display: each
				at: point
				on: aGraphicsContext]
!

displayOn: aGraphicsContext
	self displayBackgroundOn: aGraphicsContext.
	self displayForegroundOn: aGraphicsContext
!

redraw
    self displayOn: self
! !

!ToolbarView methodsFor:'initialize-release'!

initialize
	super initialize.
	horizontal := true
! !

!ToolbarView methodsFor:'private'!

boundsFor: aButton
	self buttonsAndPositionsDo:
			[:each :point |
			each == aButton ifTrue: [^point extent: each icon extent]].
	^1 @ 1 extent: 0 @ 0
!

buttonsAndPositionsDo: aBlock
	self buttons inject: self startPosition
		into:
			[:pos :each |
			aBlock value: each value: pos.
			pos + (horizontal
						ifTrue: [(each icon extent x + self spacer) @ 0]
						ifFalse: [0 @ (each icon extent y + self spacer)])]
!

isSelected: anObject
	^model selection == anObject
!

overBorder
	^BeveledBorder raised
!

selectedBorder
	^BeveledBorder inset
!

spacer
	^5
!

startPosition
	^self spacer @ self spacer
! !

!ToolbarView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ToolbarView.st,v 1.1 2006/03/13 16:59:49 cg Exp $'
! !
