"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#ViewAdapterFigure
	instanceVariableNames:'component'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

ViewAdapterFigure comment:'ViewAdapterFigure adapts VisualCompoents to the Figure protocol, allowing any VisualComponent to be inserted into a Drawing.

Instance Variables:
	component       <VisualComponent> the visual component that is adapted

'
!


!ViewAdapterFigure class methodsFor:'instance creation'!

view: aView in: aRectangle
	^self new view: aView in: aRectangle
! !

!ViewAdapterFigure methodsFor:'accessing'!

component
	^component
!

component: aVisualComponent
	component := aVisualComponent
!

container: aContainer
	super container: aContainer.
	aContainer isNil ifTrue: [^self].
	self visualComponentsIn: self
		do: [:each | (each isKindOf: ComposedTextView) ifTrue: [each initializeSelection]]
! !

!ViewAdapterFigure methodsFor:'bounds changing'!

changedPreferredBounds: aRectangle
	component bounds: self bounds.
	super changedPreferredBounds: aRectangle
! !

!ViewAdapterFigure methodsFor:'broadcast'!

downcastEvent: aKey with: aParameter from: anInitiator
	"This is a message passed down the view structure to all subparts."

	super downcastEvent: aKey
		with: aParameter
		from: anInitiator.
	component downcastEvent: aKey
		with: aParameter
		from: anInitiator
!

flushCoordinateCaches
	"Flush caches that relate to coordinate translations between this component
	and its container"

	super flushCoordinateCaches.
	component flushCoordinateCaches
!

newGraphicsDevice: aGraphicsDevice
	"Forward the newGraphicsDevice: aGraphicsDevice messages
	to the receiver's components."

	super newGraphicsDevice: aGraphicsDevice.
	component newGraphicsDevice: aGraphicsDevice
! !

!ViewAdapterFigure methodsFor:'changing'!

delete
	| topComponent |
	topComponent := self topComponent.
	topComponent isNil ifTrue: [^self].
	self visualComponentsIn: self
		do:
			[:each |
			topComponent keyboardProcessor keyboardConsumers remove: each ifAbsent: []].
	super delete
! !

!ViewAdapterFigure methodsFor:'control'!

objectWantingControl
	"Answer the first component that wishes to take control.  Answer nil if there is
	no component wanting control."

	^component objectWantingControl
! !

!ViewAdapterFigure methodsFor:'copying'!

postCopy
	super postCopy.
	component := component copy
! !

!ViewAdapterFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	self isOpaque ifTrue:
			[aGraphicsContext paint: self fillColor;
				displayRectangle: self bounds;
				paint: self lineColor].
	aGraphicsContext intersectClip: self bounds.
	aGraphicsContext translateBy: self origin.
	component displayOn: aGraphicsContext
! !

!ViewAdapterFigure methodsFor:'event driven'!

handlerForMouseEvent: anEvent
	^component handlerForMouseEvent: anEvent
! !

!ViewAdapterFigure methodsFor:'initialize-release'!

release
	component release.
	super release
!

view: aView in: aRectangle
	bounds := aRectangle.
	component := aView.
	component container: self.
	component bounds: (0 @ 0 extent: aRectangle extent)
! !

!ViewAdapterFigure methodsFor:'private'!

changedBounds: oldBounds forComponent: aVisualComponent
	"The bounds of aVisualComponent has changed from oldBounds.
	Do nothing since the receiver has its own notion of bounds."

	^self
!

changedContainer
	super changedContainer.
	container isNil ifTrue: [^self].
	self setKeyboardProcessor: self topComponent keyboardProcessor.
	self setOccluded
!

changedPreferredBounds: oldPreferredBounds forComponent: aVisualComponent
	"The preferred bounds of aVisualComponent has changed from
	oldPreferredBounds."

	"Do nothing since the receiver has its own notion of bounds."

	^self
!

compositionBoundsFor: aVisualPart
	"The receiver is a container for aVisualPart.
	An actual bounding rectangle is being searched for by
	aVisualPart. Forward to the receiver's container."

	^container isNil ifTrue: [self bounds] ifFalse: [0 @ 0 extent: self extent]
!

downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator
	"This is a message passed down the view structure to some single
	part. Answer true if we accepted the event, or false if it should be
	passed on to whatever's behind us."

	^super
		downcastLocalEvent: aKey
		with: aParameter
		at: (aPoint translatedBy: self origin negated)
		from: anInitiator
!

graphicsContextFor: aVisualComponent
	"Answer a GraphicsContext set-up for aVisualComponent."

	| gc |
	gc := container graphicsContextFor: self.
	gc intersectClip: self bounds.
	gc translateBy: self origin.
	^gc
!

invalidateRectangle: aRectangle repairNow: aBoolean forComponent: aVisualComponent
	"Propagate damage up the hierarchy after translating and clipping."

	| rect |
	container isNil ifTrue: [^self].
	rect := aRectangle translatedBy: self origin.
	(self intersects: rect)
		ifTrue:
			[container
				invalidateRectangle: (self bounds intersect: rect)
				repairNow: aBoolean
				forComponent: self]
!

localPointToGlobal: aPoint
	"Convert a point in local coordinates to a point in the top window's coordinate system."

	^aPoint translatedBy: (container localPointToGlobal: self origin)
!

setComponentBoundsTo: newBounds
	component bounds: (0 @ 0 extent: newBounds extent)
!

setKeyboardProcessor: aKeyboardProcessor
	self visualComponentsIn: self
		do:
			[:each |
			(each respondsTo: #controller)
				ifTrue: [aKeyboardProcessor sendKeyboardTo: each]]
!

setOccluded
	self downcastEvent: #useInvalidation
		with: true
		from: self
!

visualComponentsIn: aVisualComponent do: aBlock
	"This is a hack to go over the components of aVisualComponent"

	aBlock value: aVisualComponent.
	(aVisualComponent respondsTo: #component)
		ifTrue: [self visualComponentsIn: aVisualComponent component do: aBlock].
	(aVisualComponent respondsTo: #components) ifTrue:
			[aVisualComponent components
				do: [:each | self visualComponentsIn: each component do: aBlock]]
! !

!ViewAdapterFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ViewAdapterFigure.st,v 1.1 2006/03/13 16:56:11 cg Exp $'
! !
