"{ Package: 'stx:goodies/iniFile' }"

Object subclass:#INIFile
	instanceVariableNames:'sectionDictionaries'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Support-FileFormats'
!

!INIFile class methodsFor:'documentation'!

documentation
"
    Parser for Windows-style .INI files

    This package is a port of a Squeak package called INIFile (Author: Julian Fitzell). 
" 
! !

!INIFile class methodsFor:'instance creation'!

readFrom: aStream

    | section line ini caseOptions block |
    section := nil.
    ini := self new.
    
    [aStream atEnd] whileFalse:[
        line := aStream nextLine withBlanksTrimmed.
        line isEmpty ifFalse:[
            caseOptions := Dictionary newFrom: (Array 
                with: ($[ -> [section := self newSectionIn: ini from: line])
                with: ($; -> ["ignore comment line"])).
            block := caseOptions at: line first ifAbsent:nil.
            block isNil 
                ifTrue:[section ifNotNil: [self addSettingFrom: line to: section]]
                ifFalse:[block value].
        ]
    ].

    ^ ini
! !

!INIFile class methodsFor:'private'!

addSettingFrom: lineString to: section

    | stream key value |
    stream := lineString readStream.
    key := (stream upTo: $=) withBlanksTrimmed.
    value := (stream upToEnd) withBlanksTrimmed.
    section at: key put: (value isEmpty ifTrue: [nil] ifFalse: [value]).
!

newSectionIn: anINIFile from: aString

    | stream |
    stream := aString readStream.
    stream upTo: $[.
    ^ anINIFile section: (stream upTo: $])
! !

!INIFile methodsFor:'accessing'!

section: sectionString

    ^ INIFileSection named: sectionString on: self
!

section: label at: keyString ifAbsent: absentBlock

    ^ (self sectionDictionaries at: label ifAbsent: [OrderedDictionary new])
            at: keyString ifAbsent: absentBlock
!

section: label at: keyString ifAbsent: absentBlock ignoreCase: ignoreCase

    ^ ignoreCase ifTrue:[         
        self sectionIgnoringCase: label at: keyString ifAbsent: absentBlock.
    ] ifFalse:[
        self section: label at: keyString ifAbsent: absentBlock 
    ].
!

section: label at: keyString put: valueString

    ^ (self ensureSectionDictionary: label)
            at: keyString put: valueString
!

sectionIgnoringCase: label at: keyString ifAbsent: absentBlock

    | sectionKey associationKey associationsDictionary|

    sectionKey := self sectionDictionaries findFirstKey:[:sectionlabel | label sameAs: sectionlabel].
    sectionKey isNil ifTrue:[^ absentBlock value].
    associationsDictionary := self sectionDictionaries at: sectionKey ifAbsent: [OrderedDictionary new].
    associationKey := associationsDictionary findFirstKey:[:key | keyString sameAs: key].
    associationKey isNil ifTrue:[^ absentBlock value].
    ^ associationsDictionary at: associationKey ifAbsent: absentBlock.
!

sections

    ^ Array streamContents:
            [:stream | self sectionsDo: [:ea | stream nextPut: ea]]
!

sectionsDo: aBlock

    self sectionDictionaries keysDo:
            [:key | aBlock value: (INIFileSection on: self label: key)]
! !

!INIFile methodsFor:'private'!

ensureSectionDictionary: label

    ^ self sectionDictionaries at: label ifAbsentPut: [OrderedDictionary new]
!

section: label associationsDo: aBlock

    (self sectionDictionaries at: label ifAbsent: [OrderedDictionary new])
            associationsDo: aBlock
!

sectionDictionaries

    sectionDictionaries isNil ifTrue: [sectionDictionaries := OrderedDictionary new].
    ^ sectionDictionaries
! !

!INIFile methodsFor:'testing'!

includesSection: label

    ^ self sectionDictionaries includesKey: label
! !

!INIFile methodsFor:'writing'!

writeOn: aStream

    self sections
            do: [:ea | ea writeOn: aStream]
            separatedBy: [aStream cr]
! !

!INIFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/iniFile/INIFile.st,v 1.6 2008/06/20 09:45:51 cg Exp $'
! !
