"{ Package: 'stx:goodies/iniFile' }"

TestCase subclass:#INIFileTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'System-Support-FileFormats'
!


!INIFileTest methodsFor:'tests'!

testBatchAddition

    | ini dict |
    ini := INIFile new.
    dict := Dictionary withAll: (Array with: ('key1'->'val1') with: ('key2'->'val2')).
    (ini section: 'foo') addAll: dict.
    
    self assert: ((ini section: 'foo') at: 'key1') = 'val1'.
    self assert: ((ini section: 'foo') at: 'key2') = 'val2'.
!

testComplexRead

    | content in |
    content := '

[bar]
;comment

key1 = val1

key2 = val2
;key2 = oldvalue

;foo heading comment
[foo]
key1=    val1
key2= val with a space in it

'.

    in := INIFile readFrom: content readStream.
    self assert: ((in section: 'foo') at: 'key1') = 'val1'.
    self assert: ((in section: 'foo') at: 'key2') = 'val with a space in it'.
    self assert: ((in section: 'bar') at: 'key1') = 'val1'.
    self assert: ((in section: 'bar') at: 'key2') = 'val2'.
!

testEmptyValueOnRead

    | content in |
    content := '[foo]
key1 =      '.

    in := INIFile readFrom: content readStream.
    self assert: ((in section: 'foo') at: 'key1') isNil
!

testHasSection

    | ini section |
    ini := INIFile new.
    section := ini section: 'foo'.
    self deny: (ini includesSection: 'foo').
    section at: 'bar' put: 'baz'.
    self assert: (ini includesSection: 'foo').
!

testIteration

    | ini |
    ini := INIFile new.
    (ini section: 'foo') at: 'key1' put: 'val1'.
    (ini section: 'foo') at: 'key2' put: 'val2'.
    (ini section: 'bar') at: 'key1' put: 'val1'.
    (ini section: 'bar') at: 'key2' put: 'val2'.
    
    self assert: ini sections size = 2.
    ini sectionsDo: [:ea | self assert: ea associations size = 2].
!

testMultipleSections

    | stream out in |
    out := INIFile new.
    (out section: 'foo') at: 'key1' put: 'val1'.
    (out section: 'foo') at: 'key2' put: 'val2'.
    (out section: 'bar') at: 'key1' put: 'val1'.
    (out section: 'bar') at: 'key2' put: 'val2'.
    stream := WriteStream on: String new.
    out writeOn: stream.
    
    in := INIFile readFrom: stream contents readStream.
    self assert: ((in section: 'foo') at: 'key1') = 'val1'.
    self assert: ((in section: 'foo') at: 'key2') = 'val2'.
    self assert: ((in section: 'bar') at: 'key1') = 'val1'.
    self assert: ((in section: 'bar') at: 'key2') = 'val2'.
!

testNilValueOnWrite

    | out expected stream |
    out := INIFile new.
    (out section: 'foo') at: 'key1' put: nil.
    stream := WriteStream on: String new.
    out writeOn: stream.
    
    expected := '[foo]
key1=
'.
        
    self assert: expected = stream contents
!

testSectionIsEmpty

    | ini section |
    ini := INIFile new.
    section := ini section: 'foo'.
    self assert: section isEmpty.
    section at: 'bar' put: 'baz'.
    self deny: section isEmpty.
!

testSingleSection

    | out stream foo |
    out := INIFile new.
    (out section: 'foo') at: 'key1' put: 'val1'.
    (out section: 'foo') at: 'key2' put: 'val2'.
    stream := WriteStream on: String new.
    out writeOn: stream.
    
    foo := (INIFile readFrom: stream contents readStream) section: 'foo'.
    self assert: (foo at: 'key1') = 'val1'.
    self assert: (foo at: 'key2') = 'val2'.
! !

!INIFileTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/iniFile/INIFileTest.st,v 1.2 2008/06/20 09:45:56 cg Exp $'
! !
