"{ Package: 'stx:goodies/lisp' }"

"{ NameSpace: Lisp }"

ApplicationModel subclass:#LispInterpreter
	instanceVariableNames:'lispTable bindStack failBlock textValue textCollector'
	classVariableNames:'QuestionMenu'
	poolDictionaries:''
	category:'Languages-Lisp'
!

!LispInterpreter class methodsFor:'documentation'!

documentation
"
    Lisp
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"
!

example03
        "LispInterpreter example03."
        "Transcript showCR:LispInterpreter example03."

        | aList |
        aList := LispInterpreter evaluateFrom: '

        "abc"

        '.
        ^aList

    "Modified: 20.6.1997 / 15:29:41 / cg"
!

example20
	"LispInterpreter example20."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(if
			(= (send {Dialog} `confirm: "Are you happy?") {true})
			then
			(send {Transcript} `cr)
			(send {Transcript} `show: "yes")
			else 
			(send {Transcript} `cr)
			(send {Transcript} `show: "no")))

	'.
	^aList
!

examples
"
   a lisp view - top pane allows lispExpressions to be entered and
   evaluated. Result is shown in bottom panel.
                                                                [exBegin]
        LispInterpreter open
                                                                [exEnd]


   passing strings containing a lisp expression 
   to a lisp interpreter

                                                                [exBegin]
        |result|

        result := LispInterpreter evaluateFrom: ' nil '.
        ^ result   
                                                                [exEnd]

                                                                [exBegin]
        |result|

        result := LispInterpreter evaluateFrom: ' 123 '.
        ^ result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

                        (cons 3 4)

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (quote (3 4))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (car (quote (3 4)))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (cdr (quote (3 4)))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (cons (car `(1 2 3)) `(3 4))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (+ 1 2 3 4 5 6 7 8 9 10)

        '.
        ^result  
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (setq x 100)
                (setq y 200 z 300)
                (+ x y z))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (defun plus (x y) (+ x y))
                (plus 3 4))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (defun concat (x y) 
                        (cond
                                ((atom x) y)
                                (t (cons
                                                (car x)
                                                (concat (cdr x) y)))))
                (concat `(1 2 3) `(4 5)))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (defun plus nlambda (x) (plus1 x))
                (defun plus1 lambda (x)
                        (cond
                                ((null x) 0)
                                (t (+ (car x) (plus1 (cdr x))))))
                (plus 1 2 3 4 5 6 7 8 9 10))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        ((lambda (x y) (cons x (cons y nil))) 3 4)

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        ((nlambda (x) x) 1 2 3 4 5 6 7 8 9 10)

        '.
        ^result
                                                                [exEnd]

    mixing Smalltalk with lisp:
                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (do
                (image view point)
                (setq image {Image fromUser})
                (setq view {StandardSystemView new openAndWait})
                (setq point {50@50})
                (send image `displayOn:at: view point))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (setq x 100)
                (setq y 200)
                (do
                        (x y)
                        (setq x 10000)
                        (setq y 20000)
                        (send {Transcript} `cr)
                        (send {Transcript} `show: (send x `printString))
                        (send {Transcript} `cr)
                        (send {Transcript} `show: (send y `printString)))
                (send {Transcript} `cr)
                (send {Transcript} `show: (send x `printString))
                (send {Transcript} `cr)
                (send {Transcript} `show: (send y `printString))
                (send {Transcript} `cr)
        )

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (progn
                (setq x 1000000)
                (setq y 2000000)
                (do
                        (rslt)
                        (setq rslt (* x y))
                        (send {Transcript} `show: (send x `printString))
                        (send {Transcript} `show: `* )
                        (send {Transcript} `show: (send y `printString))
                        (send {Transcript} `show: `= )
                        (send {Transcript} `show: (send rslt `printString))
                        (send {Transcript} `cr)
                )
        )

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (do
                (count)
                (setq count 1)
                (while
                        (<= count 100)
                        do
                        (send {Transcript} `cr)
                        (send {Transcript} `show: (send count `printString))
                        (setq count (+ count 1)))
                (send {Transcript} `cr))

        '.
        ^result
                                                                [exEnd]

                                                                [exBegin]
        | result |
        result := LispInterpreter evaluateFrom: '

        (do
                (count)
                (setq count 1)
                (repeat
                        (send {Transcript} `cr)
                        (send {Transcript} `show: (send count `printString))
                        (setq count (+ count 1))
                        until
                        (<= count 100))
                (send {Transcript} `cr))

        '.
        ^result
                                                                [exEnd]

"

    "Created: 31.7.1997 / 19:34:03 / cg"
    "Modified: 31.7.1997 / 19:48:54 / cg"
! !

!LispInterpreter class methodsFor:'instance creation'!

new
	^super new initialize
! !

!LispInterpreter class methodsFor:'class initialization'!

flushMenus
	"LispInterpreter flushMenus"

	QuestionMenu := nil
!

initialize
	"LispInterpreter initialize."

	self initializeFont
!

initializeFont
        "LispInterpreter initializeFont."

"/ XXX the ST/X emulation of Text- and characterAttributes is not yet
"/ complete ...

        | defaultFontName defaultFontSize fontDescription characterAttributes textAttributes deviceFont |
        defaultFontName := #('Courier*' 'courier*' '**').
        defaultFontSize := 12.
        fontDescription := FontDescription new.
        fontDescription family: defaultFontName.
        fontDescription pixelSize: defaultFontSize.
        fontDescription boldness: 0.5.
        fontDescription italic: false.
        fontDescription outline: false.
        fontDescription shadow: false.
        fontDescription underline: false.
        fontDescription fixedWidth: true.
        fontDescription serif: false.
        fontDescription strikeout: false.
"/        fontDescription encodings: Locale current preferredEncodings.

	CharacterAttributes notNil ifTrue:[
	    Autoload autoloadFailedSignal catch:[
	 	CharacterAttributes autoload.
	    ].
	    CharacterAttributes isLoaded ifTrue:[
                characterAttributes := CharacterAttributes newWithDefaultAttributes.
                characterAttributes setDefaultQuery: fontDescription.
                textAttributes := TextAttributes new.
                textAttributes setCharacterAttributes: characterAttributes.
"/                deviceFont := Screen default defaultFontPolicy findFont: fontDescription.
"/                textAttributes lineGrid: deviceFont height.
"/                textAttributes baseline: deviceFont ascent.
                TextAttributes styleNamed: #lispDefault put: textAttributes
	    ]
	]

    "Modified: 20.6.1997 / 15:05:56 / cg"
! !

!LispInterpreter class methodsFor:'interface specs'!

original_windowSpec
        "UIPainter new openOnClass: self andSelector: #original_windowSpec."

        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #label: 'Lisp' 
                        #min: #(#Point 250 300 )
                        #bounds: #(#Rectangle 100 100 350 400 ) ) 
                #component: 
                #(#SpecCollection 
                        #collection: #(
                                #(#TextEditorSpec 
                                        #layout: #(#LayoutFrame 1 0 1 0 -1 1 -1 0.3 ) 
                                        #name: #textValue 
                                        #model: #textValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #focusOutSelector: #textValueChange: 
                                                #valueChangeSelector: #textValueChange: )
                                        #tabable: true 
                                        #menu: #textMenu  
                                         ) 
                                #(#ArbitraryComponentSpec 
                                        #layout: #(#LayoutFrame 1 0 0 0.3 -1 1 -1 1 ) 
                                        #name: #textCollectorView 
                                        #flags: 9 
                                        #component: #textCollectorView ) ) ) )

    "Created: 31.7.1997 / 19:08:47 / cg"
!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:Lisp::LispInterpreter andSelector:#windowSpec
     Lisp::LispInterpreter new openInterface:#windowSpec
     Lisp::LispInterpreter open
    "

    <resource: #canvas>

    ^
     
       #(#FullSpec
          #name: #windowSpec
          #window: 
           #(#WindowSpec
              #name: 'Lisp'
              #layout: #(#LayoutFrame 22 0 386 0 617 0 788 0)
              #level: 0
              #label: 'Lisp'
              #bounds: #(#Rectangle 22 386 618 789)
              #usePreferredExtent: false
              #returnIsOKInDialog: true
              #escapeIsCancelInDialog: true
          )
          #component: 
           #(#SpecCollection
              #collection: 
               #(
                 #(#VariableVerticalPanelSpec
                    #name: 'variableVerticalPanel1'
                    #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
                    #component: 
                     #(#SpecCollection
                        #collection: 
                         #(
                           #(#TextEditorSpec
                              #name: #textValue
                              #tabable: true
                              #model: #textValue
                              #menu: #textMenu
                              #hasHorizontalScrollBar: true
                              #hasVerticalScrollBar: true
                              #miniScrollerHorizontal: true
                              #callbacksSpec: 
                               #(#UIEventCallbackSubSpec
                                  #valueChangeSelector: #textValueChange:
                                  #focusOutSelector: #textValueChange:
                              )
                          )
                           #(#ArbitraryComponentSpec
                              #name: #textCollectorView
                              #flags: 9
                              #hasHorizontalScrollBar: true
                              #hasVerticalScrollBar: true
                              #miniScrollerHorizontal: true
                              #component: #textCollectorView
                              #hasBorder: false
                          )
                        )
                    )
                    #handles: #(#Any 0.5 1.0)
                )
              )
          )
      )
! !

!LispInterpreter class methodsFor:'printing'!

printString: anObject 
	"LispInterpreter printString: 'string'."
	"LispInterpreter printString: #symbol."
	"LispInterpreter printString: 123."
	"LispInterpreter printString: 123.456."
	"LispInterpreter printString: 123.456e7."
	"LispInterpreter printString: LispNil null."
	"LispInterpreter printString: nil."

	| aStream |
	aStream := WriteStream on: (String new: 32).
	(anObject isKindOf: LispCons)
		ifTrue: [anObject printOn: aStream level: 0]
		ifFalse: [LispCons new printOn: aStream object: anObject].
	^aStream contents
! !

!LispInterpreter class methodsFor:'public access'!

evaluate: sExpression 
	^self new evaluateTopLevel: sExpression ifFail: [:errorMessage | self error: errorMessage]
!

evaluate: sExpression ifFail: aBlock 
	^self new evaluateTopLevel: sExpression ifFail: aBlock
!

evaluateFrom: aStream 
	^self new evaluateTopLevel: (LispParser parse: aStream)
		ifFail: [:errorMessage | self error: errorMessage]
!

evaluateFrom: aStream ifFail: aBlock 
	^self new evaluateTopLevel: (LispCons parse: aStream ifFail: aBlock)
		ifFail: aBlock
! !

!LispInterpreter methodsFor:'accessing'!

textCollector
	^textCollector
! !

!LispInterpreter methodsFor:'adaptor'!

questionMenu
	"LispInterpreter flushMenus."

	| menuBlock mainMenu subMenu |
	QuestionMenu isNil
		ifTrue: 
			[menuBlock := 
					[:list | 
					| labels lines values index |
					labels := OrderedCollection new.
					lines := OrderedCollection new.
					values := OrderedCollection new.
					index := 0.
					list do: [:assoc | assoc isNil
							ifTrue: [lines add: index]
							ifFalse: 
								[labels add: assoc key.
								values add: assoc value.
								index := index + 1]].
					(Menu
						labelArray: labels asArray
						lines: lines asArray
						values: values asArray) yourself].
			subMenu := OrderedCollection new.
			subMenu add: 'gc' -> #subrGc.
			subMenu add: 'investigate' -> #investigate.
			mainMenu := OrderedCollection new.
			mainMenu add: 'find...' -> #find.
			mainMenu add: 'replace...' -> #replace.
			mainMenu add: nil.
			mainMenu add: 'undo' -> #undo.
			mainMenu add: nil.
			mainMenu add: 'copy' -> #copySelection.
			mainMenu add: 'cut' -> #cut.
			mainMenu add: 'paste' -> #paste.
			mainMenu add: nil.
			mainMenu add: 'do it' -> #doIt:from:.
			mainMenu add: 'inspect' -> #inspectIt:from:.
			mainMenu add: nil.
			mainMenu add: 'lisp' -> (menuBlock value: subMenu).
			QuestionMenu := menuBlock value: mainMenu].
	^QuestionMenu
!

textCollectorView
        | textCollectorView |
        "/ ST/X
        ^ textCollector := TextCollector new.

        "/ ST80
        textCollectorView := TextCollectorView new model: self textCollector.
        textCollectorView controller: TextEditorController new.
        textCollectorView controller keyboardProcessor: builder keyboardProcessor.
        textCollectorView textStyle: (TextAttributes styleNamed: #lispDefault).
        ^textCollectorView

    "Modified: 21.6.1997 / 13:38:49 / cg"
!

textMenu
	^[self questionMenu]
!

textValue
	textValue isNil ifTrue: [textValue := Text new asValue].
	^textValue
!

textValueChange: aController 
	aController accepted ifFalse: [aController accept]
! !

!LispInterpreter methodsFor:'error handling'!

fatal: message 
	bindStack reverseDo: [:assoc | assoc key notNil ifTrue: [assoc value notNil
				ifTrue: [self
						putprop: assoc key
						key: #apval
						value: assoc value]
				ifFalse: [self remprop: assoc key key: #apval]]].
	^failBlock value: '*** Error: ' , message
! !

!LispInterpreter methodsFor:'evaluating'!

evaluate: sExpression 
        | apval |

        (sExpression isKindOf: LispList)
                ifTrue: [^self listEvaluate: sExpression].

        (sExpression isKindOf: Symbol)
                ifTrue: 
                        [sExpression = #t ifTrue: [^#t].
                        lispTable intern: sExpression.
                        apval := lispTable getprop: sExpression key: #apval.
                        apval isNil ifTrue: [^self fatal: (self printString: sExpression)
                                                , ' is unbound atom'].
                        ^apval].
        ^sExpression
!

evaluateTopLevel: sExpression 
	| result |
	failBlock := 
			[:errorMessage | 
			textCollector show: errorMessage; cr.
			Cursor normal show.
			^LispNil null].
	Cursor execute showWhile: [result := self evaluate: sExpression].
	^result
!

evaluateTopLevel: sExpression ifFail: aBlock 
	failBlock := aBlock.
	^self evaluate: sExpression
!

listEvaluate: sExpression 
	| funcName arguList funcBody |
	sExpression null ifTrue: [^sExpression].
	funcName := sExpression head.
	arguList := sExpression tail.
	(funcName isKindOf: LispCons)
		ifTrue: 
			[funcBody := funcName.
			funcBody head = #lambda
				ifTrue: 
					[funcBody := LispCons head: #lambda tail: funcBody.
					^self exprEval: funcBody arguList: arguList].
			funcBody head = #nlambda
				ifTrue: 
					[funcBody := LispCons head: #nlambda tail: funcBody.
					^self fexprEval: funcBody arguList: arguList].
			^self fatal: 'unexpected function ' , (self printString: funcBody)].
	(funcName isKindOf: Symbol)
		ifFalse: [^self fatal: 'null function ' , (self printString: funcName)].
	funcBody := self getprop: funcName key: #fexpr.
	funcBody = LispNil null ifFalse: [^self fexprEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #expr.
	funcBody = LispNil null ifFalse: [^self exprEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #fsubr.
	funcBody = LispNil null ifFalse: [^self fsubrEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #subr.
	funcBody = LispNil null ifFalse: [^self subrEval: funcBody arguList: arguList].
	^self fatal: 'undefined function ' , (self printString: funcName)
! !

!LispInterpreter methodsFor:'executing'!

execute: aStream 
        | start list end string result |

        self skipSeparators: aStream.
        start := aStream position1Based.
        list := LispParser parse: aStream.
        end := aStream position1Based - 1.
        string := aStream contents copyFrom: start to: end.
        textCollector show: '> ' , string.
        textCollector cr.
        result := self evaluateTopLevel: list.
        textCollector show: (self printString: result).
        textCollector cr.
        ^result

    "Modified: / 18-04-2007 / 12:48:58 / cg"
! !

!LispInterpreter methodsFor:'fsubr functions'!

fsubrAdd: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for +'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v + a]].
	^v
!

fsubrCond: arguList 
	| result |
	arguList do: [:each | (self evaluate: each head)
			~= LispNil null
			ifTrue: 
				[result := LispNil null.
				(each tail isKindOf: LispCons)
					ifTrue: [each tail do: [:expr | result := self evaluate: expr]].
				^result]].
	^LispNil null
!

fsubrDefun: arguList 
	| funcName funcType |
	funcName := arguList head.
	(funcName isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected function name ' , (self printString: funcName) , ' for defun'].
	funcType := arguList tail head.
	funcType = #lambda
		ifTrue: 
			[self
				putprop: funcName
				key: #expr
				value: arguList.
			^funcName].
	funcType = #nlambda
		ifTrue: 
			[self
				putprop: funcName
				key: #fexpr
				value: arguList.
			^funcName].
	self
		putprop: funcName
		key: #expr
		value: (LispCons head: funcName tail: (LispCons head: #lambda tail: arguList tail)).
	^funcName
!

fsubrDiv: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for /'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v / a]].
	^v
!

fsubrDo: arguList 
	| locals executions result |
	locals := arguList head.
	executions := arguList tail.
	self bindMark.
	locals do: [:lvar | self bind: lvar value: LispNil null].
	result := LispNil null.
	executions do: [:each | result := self evaluate: each].
	self unbind.
	^result
!

fsubrIdiv: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for //'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v // a]].
	^v
!

fsubrIf: arguList 
	| predicate then list truePart falsePart bool result |
	predicate := arguList head.
	then := arguList tail head.
	list := arguList tail tail.
	truePart := LispNil null.
	falsePart := LispNil null.
	bool := true.
	list do: [:each | each = #else
			ifTrue: [bool := false]
			ifFalse: [bool
					ifTrue: [truePart := LispCons head: each tail: truePart]
					ifFalse: [falsePart := LispCons head: each tail: falsePart]]].
	then = #then ifFalse: [^self fatal: 'unexpected format for if'].
	truePart := truePart reverse.
	falsePart := falsePart reverse.
	result := LispNil null.
	(self evaluate: predicate)
		= LispNil null
		ifTrue: [falsePart do: [:each | result := self evaluate: each]]
		ifFalse: [truePart do: [:each | result := self evaluate: each]].
	^result
!

fsubrMlt: arguList 
        | v a |
        v := LispNil null.
        arguList
                do: 
                        [:each | 
                        a := self evaluate: each.
                        (a isKindOf: Number)
                                ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for *'].
                        v = LispNil null
                                ifTrue: [v := a]
                                ifFalse: [v := v * a]].
        ^v
!

fsubrMod: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for \\'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v \\ a]].
	^v
!

fsubrProgn: arguList 
	| result |
	result := LispNil null.
	arguList do: [:each | result := self evaluate: each].
	^result
!

fsubrQuote: arguList
	^arguList head
!

fsubrRead: arguList 
	| message default answer |
	message := '          Please type S expression.          '.
	default := String new.
	arguList length >= 1
		ifTrue: 
			[message := self evaluate: (arguList nth: 1).
			(message isKindOf: String)
				ifFalse: [message := self printString: message]].
	arguList length >= 2
		ifTrue: 
			[default := self evaluate: (arguList nth: 2).
			(default isKindOf: String)
				ifFalse: [default := self printString: default]].
	answer := Dialog request: message initialAnswer: default.
	(answer isNil or: [answer isEmpty])
		ifTrue: [^LispNil null].
	^LispParser parse: (ReadStream on: answer)
!

fsubrRepeat: arguList 
	| reverse predicate until executions result |
	reverse := arguList reverse.
	predicate := reverse head.
	until := reverse tail head.
	executions := reverse tail tail reverse.
	until = #until ifFalse: [^self fatal: 'unexpected format for repeat'].
	result := LispNil null.
	executions do: [:each | result := self evaluate: each].
	[(self evaluate: predicate)
		= LispNil null]
		whileFalse: [executions do: [:each | result := self evaluate: each]].
	^result
!

fsubrSend: arguList 
	| list receiver selector arguments result |
	list := arguList.
	receiver := self evaluate: list head.
	list := list tail.
	selector := self evaluate: list head.
	(selector isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected selector ' , (self printString: selector) , ' for send'].
	list := list tail.
	arguments := OrderedCollection new.
	[list isKindOf: LispCons]
		whileTrue: 
			[arguments add: (self evaluate: list head).
			list := list tail].
	result := receiver perform: selector withArguments: arguments asArray.
	^result
!

fsubrSetq: arguList 
	| list a1 a2 |
	list := arguList.
	a2 := LispNil null.
	[list isKindOf: LispCons]
		whileTrue: 
			[a1 := list head.
			(a1 isKindOf: Symbol)
				ifFalse: [^self fatal: 'unexpected variable ' , (self printString: a1) , ' for setq'].
			list := list tail.
			a2 := self evaluate: list head.
			self
				putprop: a1
				key: #apval
				value: a2.
			list := list tail].
	^a2
!

fsubrSub: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for -'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v - a]].
	^v
!

fsubrWhile: arguList 
	| predicate do executions result |
	predicate := arguList head.
	do := arguList tail head.
	executions := arguList tail tail.
	do = #do ifFalse: [^self fatal: 'unexpected format for while'].
	result := LispNil null.
	[(self evaluate: predicate)
		= LispNil null]
		whileFalse: [executions do: [:each | result := self evaluate: each]].
	^result
! !

!LispInterpreter methodsFor:'func eval'!

exprEval: funcBody arguList: arguList 
	| expression funcName lvarList lvar result |
	expression := funcBody.
	funcName := expression head.
	expression := expression tail.
	expression := expression tail.
	lvarList := expression head.
	expression := expression tail.
	arguList length ~= lvarList length ifTrue: [^self fatal: 'too few or many arguments ' , (self printString: arguList) , ' vs ' , (self printString: lvarList) , ' for ' , funcName].
	self bindMark.
	arguList
		do: 
			[:each | 
			lvar := lvarList head.
			self bind: lvar value: (self evaluate: each).
			lvarList := lvarList tail].
	expression do: [:each | result := self evaluate: each].
	self unbind.
	^result
!

fexprEval: funcBody arguList: arguList 
	| expression funcName lvarList lvar result |
	expression := funcBody.
	funcName := expression head.
	expression := expression tail.
	expression := expression tail.
	lvarList := expression head.
	expression := expression tail.
	lvarList length ~= 1 ifTrue: [^self fatal: 'too few or many arguments ' , (self printString: lvarList) , ' for ' , (self printString: funcName)].
	self bindMark.
	lvar := lvarList head.
	self bind: lvar value: arguList.
	expression do: [:each | result := self evaluate: each].
	self unbind.
	^result
!

fsubrEval: funcBody arguList: arguList 
	| messageSelector |
	messageSelector := funcBody tail head.
	^self perform: messageSelector with: arguList
!

subrEval: funcBody arguList: arguList 
	| funcName messageSelector arguCount arguBuffer list |
	funcName := funcBody head.
	messageSelector := funcBody tail head.
	arguCount := funcBody tail tail head.
	arguBuffer := OrderedCollection new.
	list := arguList.
	arguCount
		timesRepeat: 
			[list null ifTrue: [^self fatal: 'too few arguments for ' , (self printString: funcName)].
			arguBuffer add: (self evaluate: list head).
			list := list tail].
	list null not ifTrue: [^self fatal: 'too mary arguments for ' , (self printString: funcName)].
	arguBuffer isEmpty
		ifTrue: [^self perform: messageSelector]
		ifFalse: [^self perform: messageSelector with: arguBuffer asArray]
! !

!LispInterpreter methodsFor:'initialize-release'!

initialize
        lispTable := LispTable new.
        bindStack := OrderedCollection new.
        failBlock := [:errorMessage | self error: errorMessage].
        self initializeSubrFunctions.
        self initializeFsubrFunctions.
        self initializeExprFunctions.
        self initializeFexprFunctions.
        "/ textCollector := TextCollector new

    "Modified: 21.6.1997 / 13:36:05 / cg"
!

initializeExprFunctions
	self evaluateTopLevel: (LispParser parse: '

	% Expr Functions
	(progn

		% ++
		(defun ++ lambda (x)
			(+ x 1))

		% --
		(defun -- lambda (x)
			(- x 1))

		% assoc
		(defun assoc lambda (x a)
			(cond
				((null a) nil)
				((equal x (car (car a))) (car a))
				(t (assoc x (cdr a)))))

		% copy
		(defun copy lambda (x)
			(cond
				((null x) nil)
				(t (cons (car x) (copy (cdr x))))))

		% mapc
		(defun mapc lambda (f x)
			(cond
				((null x) nil)
				(t (progn
					(eval (cons f (cons `(car x) nil)))
					(mapc f (cdr x))))))

		% mapcar
		(defun mapcar lambda (f x)
			(cond
				((null x) nil)
				(t (cons
					(eval (cons f (cons `(car x) nil)))
					(mapcar f (cdr x))))))
			
		) % end

	')
!

initializeFexprFunctions
	self evaluateTopLevel: (LispParser parse: '

	% Expr Functions
	(progn

		% and
		(defun and nlambda (x)
			(do	(list)
				(setq list x)
				(while
					(if (null list) then nil else (eval (car list)))
					do
					(setq list (cdr list)))
				(if (null list) then t else nil)))

		% list
		(defun list nlambda (x)
			(mapcar `eval x))

		% or
		(defun or nlambda (x)
			(do	(list)
				(setq list x)
				(while
					(if (null list) then nil else (not (eval (car list))))
					do
					(setq list (cdr list)))
				(if (null list) then nil else t)))

		) % end

	')
!

initializeFsubrFunctions
	self setFsubrFunc: #(#* #fsubrMlt:).
	self setFsubrFunc: #(#+ #fsubrAdd:).
	self setFsubrFunc: #(#- #fsubrSub:).
	self setFsubrFunc: #(#/ #fsubrDiv:).
	self setFsubrFunc: #(#// #fsubrIdiv:).
	self setFsubrFunc: #(#cond #fsubrCond:).
	self setFsubrFunc: #(#defun #fsubrDefun:).
	self setFsubrFunc: #(#do #fsubrDo:).
	self setFsubrFunc: #(#if #fsubrIf:).
	self setFsubrFunc: #(#progn #fsubrProgn:).
	self setFsubrFunc: #(#quote #fsubrQuote:).
	self setFsubrFunc: #(#read #fsubrRead:).
	self setFsubrFunc: #(#repeat #fsubrRepeat:).
	self setFsubrFunc: #(#send #fsubrSend:).
	self setFsubrFunc: #(#setq #fsubrSetq:).
	self setFsubrFunc: #(#while #fsubrWhile:).
	self setFsubrFunc: #(#\\ #fsubrMod:).
!

initializeSubrFunctions
	self setSubrFunc: #(#< #subrLt: 2).
	self setSubrFunc: #(#<= #subrLe: 2).
	self setSubrFunc: #(#= #subrEqual: 2).
	self setSubrFunc: #(#== #subrEq: 2).
	self setSubrFunc: #(#> #subrGt: 2).
	self setSubrFunc: #(#>= #subrGe: 2).
	self setSubrFunc: #(#append #subrAppend: 2).
	self setSubrFunc: #(#atom #subrAtom: 1).
	self setSubrFunc: #(#car #subrCar: 1).
	self setSubrFunc: #(#cdr #subrCdr: 1).
	self setSubrFunc: #(#cons #subrCons: 2).
	self setSubrFunc: #(#consp #subrConsp: 1).
	self setSubrFunc: #(#dtpr #subrConsp: 1).
	self setSubrFunc: #(#doublep #subrDoublep: 1).
	self setSubrFunc: #(#eq #subrEq: 2).
	self setSubrFunc: #(#equal #subrEqual: 2).
	self setSubrFunc: #(#eval #subrEval: 1).
	self setSubrFunc: #(#exprs #subrExprs 0).
	self setSubrFunc: #(#fexprs #subrFexprs 0).
	self setSubrFunc: #(#floatp #subrFloatp: 1).
	self setSubrFunc: #(#fsubrs #subrFsubrs 0).
	self setSubrFunc: #(#gc #subrGc 0).
	self setSubrFunc: #(#gensym #subrGensym 0).
	self setSubrFunc: #(#getprop #subrGetprop: 2).
	self setSubrFunc: #(#integerp #subrIntegerp: 1).
	self setSubrFunc: #(#last #subrLast: 1).
	self setSubrFunc: #(#length #subrLength: 1).
	self setSubrFunc: #(#listp #subrListp: 1).
	self setSubrFunc: #(#member #subrMember: 2).
	self setSubrFunc: #(#memq #subrMemq: 2).
	self setSubrFunc: #(#nconc #subrNconc: 2).
	self setSubrFunc: #(#neq #subrNeq: 2).
	self setSubrFunc: #(#nequal #subrNequal: 2).
	self setSubrFunc: #(#not #subrNull: 1).
	self setSubrFunc: #(#nth #subrNth: 2).
	self setSubrFunc: #(#null #subrNull: 1).
	self setSubrFunc: #(#numberp #subrNumberp: 1).
	self setSubrFunc: #(#oblist #subrOblist 0).
	self setSubrFunc: #(#pp #subrPp: 1).
	self setSubrFunc: #(#princ #subrPrinc: 1).
	self setSubrFunc: #(#print #subrPrint: 1).
	self setSubrFunc: #(#putprop #subrPutprop: 3).
	self setSubrFunc: #(#remprop #subrRemprop: 2).
	self setSubrFunc: #(#reverse #subrReverse: 1).
	self setSubrFunc: #(#rplaca #subrRplaca: 2).
	self setSubrFunc: #(#rplacd #subrRplacd: 2).
	self setSubrFunc: #(#stringp #subrStringp: 1).
	self setSubrFunc: #(#subrs #subrSubrs 0).
	self setSubrFunc: #(#symbolp #subrSymbolp: 1).
	self setSubrFunc: #(#terpri #subrTerpri 0).
	self setSubrFunc: #(#~= #subrNequal: 2).
	self setSubrFunc: #(#~~ #subrNeq: 2)
! !

!LispInterpreter methodsFor:'interface opening'!

postOpenWith: aBuilder 
        | textCollectorView controller |
        textCollectorView := (aBuilder componentAt: #textCollectorView) widget.
        (controller := textCollectorView controller) notNil ifTrue:[
            controller deselect
        ]

    "Modified: 20.6.1997 / 14:49:35 / cg"
! !

!LispInterpreter methodsFor:'menu messages'!

doIt: aText from: aController 
        | stream selection result |
        "/ ST/X
        selection := (builder componentAt:#textValue) selection.
        "/ ST80
        "/ selection := aController selection.

        stream := ReadStream on: selection asString.
        result := LispNil null.
        [stream atEnd]
                whileFalse: 
                        [result := self execute: stream.
                        self skipSeparators: stream].
        ^result

    "Modified: 20.6.1997 / 14:52:48 / cg"
!

inspectIt: aText from: aController 
	| result |
	result := self doIt: aText from: aController.
	result inspect.
	^result
!

investigate
	self inspect
! !

!LispInterpreter methodsFor:'printing'!

printString: anObject 
	^self class printString: anObject
! !

!LispInterpreter methodsFor:'private'!

setFsubrFunc: bodyArray 
	self
		putprop: (bodyArray at: 1) asSymbol
		key: #fsubr
		value: (LispCons list: bodyArray)
!

setSubrFunc: bodyArray 
	self
		putprop: (bodyArray at: 1) asSymbol
		key: #subr
		value: (LispCons list: bodyArray)
!

skipSeparators: stream 
	| char |
	[stream atEnd not
		and: 
			[char := stream peek.
			(LispScannerTable scannerTable at: char asInteger)
				= #xDelimiter]]
		whileTrue: [stream next]
! !

!LispInterpreter methodsFor:'property access'!

getprop: identifier key: key 
	| value |
	lispTable intern: identifier.
	value := lispTable getprop: identifier key: key.
	value isNil ifTrue: [^LispNil null].
	^value
!

putprop: identifier key: key value: value 
	lispTable intern: identifier.
	^lispTable
		putprop: identifier
		key: key
		value: value
!

remprop: identifier key: key 
	| value |
	lispTable intern: identifier.
	value := lispTable remprop: identifier key: key.
	value isNil ifTrue: [^LispNil null].
	^value
! !

!LispInterpreter methodsFor:'shallow binding'!

bind: symbol value: value 
	| saveValue assoc |
	lispTable intern: symbol.
	saveValue := lispTable getprop: symbol key: #apval.
	assoc := Association key: symbol value: saveValue.
	bindStack addLast: assoc.
	self
		putprop: symbol
		key: #apval
		value: value
!

bindMark
	| assoc |
	assoc := Association key: nil value: nil.
	bindStack addLast: assoc
!

unbind
	| assoc |
	
	[assoc := bindStack removeLast.
	assoc key notNil]
		whileTrue: [assoc value notNil
				ifTrue: [self
						putprop: assoc key
						key: #apval
						value: assoc value]
				ifFalse: [self remprop: assoc key key: #apval]]
! !

!LispInterpreter methodsFor:'subr functions'!

subrAppend: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for append'].
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for append'].
	^a1 append: a2
!

subrAtom: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons) ifFalse: [^#t].
	^LispNil null
!

subrCar: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list head].
	(list isKindOf: LispNil)
		ifTrue: [^LispNil null].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for car'
!

subrCdr: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list tail].
	(list isKindOf: LispNil)
		ifTrue: [^LispNil null].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for cdr'
!

subrCons: arguArray 
	^LispCons head: (arguArray at: 1)
		tail: (arguArray at: 2)
!

subrConsp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons)
		ifTrue: [^#t].
	^LispNil null
!

subrDoublep: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Double)
		ifTrue: [^#t].
	^LispNil null
!

subrEq: arguArray 
	| bool |
	(arguArray at: 1)
		== (arguArray at: 2)
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool
!

subrEqual: arguArray 
	| bool |
	(arguArray at: 1)
		= (arguArray at: 2)
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool
!

subrEval: arguArray 
	^self evaluate: (arguArray at: 1)
!

subrExprs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #expr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list
!

subrFexprs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #fexpr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list
!

subrFloatp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Float)
		ifTrue: [^#t].
	^LispNil null
!

subrFsubrs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #fsubr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list
!

subrGc
	ObjectMemory globalCompactingGC.
	^#t
!

subrGe: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for >='].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for >='].
	a1 >= a2 ifTrue: [^#t].
	^LispNil null
!

subrGensym
	| clock |
	(Delay forMilliseconds: 1) wait.
	clock := Time millisecondClockValue.
	^('id' , clock printString) asSymbol
!

subrGetprop: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for getprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for getprop'].
	^self getprop: a1 key: a2
!

subrGt: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for >'].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for >'].
	a1 > a2 ifTrue: [^#t].
	^LispNil null
!

subrIntegerp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Integer)
		ifTrue: [^#t].
	^LispNil null
!

subrLast: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list last].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for last'
!

subrLe: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for <='].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for <='].
	a1 <= a2 ifTrue: [^#t].
	^LispNil null
!

subrLength: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list length].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for length'
!

subrListp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispList)
		ifTrue: [^#t].
	^LispNil null
!

subrLt: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for <'].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for <'].
	a1 < a2 ifTrue: [^#t].
	^LispNil null
!

subrMember: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for member'].
	^a2 member: a1
!

subrMemq: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for memq'].
	^a2 memq: a1
!

subrNconc: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for nconc'].
	a1 nconc: a2.
	^a1
!

subrNeq: arguArray 
	| bool |
	((arguArray at: 1)
		== (arguArray at: 2)) not
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool
!

subrNequal: arguArray 
	| bool |
	((arguArray at: 1)
		= (arguArray at: 2)) not
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool
!

subrNth: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Number)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for nth'].
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for nth'].
	^a2 nth: a1
!

subrNull: arguArray 
	((arguArray at: 1)
		isMemberOf: LispNil)
		ifTrue: [^#t].
	^LispNil null
!

subrNumberp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Number)
		ifTrue: [^#t].
	^LispNil null
!

subrOblist
	| list |
	list := LispNil null.
	lispTable identifiers reverseDo: [:each | list := LispCons head: each tail: list].
	^list
!

subrPp: arguArray 
	| a pretty |
	a := arguArray at: 1.
	pretty := a ppString.
	textCollector show: pretty.
	^a
!

subrPrinc: arguArray 
	| a |
	a := arguArray at: 1.
	(a isKindOf: String)
		ifTrue: [textCollector show: a]
		ifFalse: [textCollector show: (self printString: a)].
	^a
!

subrPrint: arguArray 
	| a |
	a := self subrPrinc: arguArray.
	textCollector cr.
	^a
!

subrPutprop: arguArray 
	| a1 a2 a3 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	a3 := arguArray at: 3.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for putprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for putprop'].
	^self
		putprop: a1
		key: a2
		value: a3
!

subrRead
	| answer |
	answer := Dialog request: '          Please type S expression.          ' initialAnswer: ''.
	(answer isNil or: [answer isEmpty])
		ifTrue: [^LispNil null].
	^LispParser parse: (ReadStream on: answer)
!

subrRemprop: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for remprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for remprop'].
	^self remprop: a1 key: a2
!

subrReverse: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for reverse'].
	^a1 reverse
!

subrRplaca: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for rplaca'].
	a1 rplaca: a2.
	^a1
!

subrRplacd: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for rplacd'].
	a1 rplacd: a2.
	^a1
!

subrStringp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	((a1 isKindOf: String)
		and: [(a1 isKindOf: Symbol) not])
		ifTrue: [^#t].
	^LispNil null
!

subrSubrs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #subr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list
!

subrSymbolp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Symbol) ifTrue: [^#t].
	^LispNil null
!

subrTerpri
	textCollector cr.
	^#t
! !

!LispInterpreter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/lisp/LispInterpreter.st,v 1.17 2007/04/18 11:39:55 cg Exp $'
! !

LispInterpreter initialize!
