"{ Package: 'stx:goodies/lisp' }"

"{ NameSpace: Lisp }"

LispScanner subclass:#LispParser
	instanceVariableNames:'prevMark prevToken prevTokenType'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Lisp'
!

!LispParser class methodsFor:'documentation'!

documentation
"
    Lisp
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"

! !

!LispParser class methodsFor:'examples'!

example1
	"LispParser example1."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 8 9) 10)'.
	^list
!

example2
	"LispParser example2."

	| list |
	list := LispParser parse: '
			(10 (1 2)
			 20 (3 4 . 100)
			 30 (5 6) . 200)
	'.
	^list
!

example3
	"LispParser example3."

	| list |
	list := LispParser parse: '
			(PetriNet JunAoki
				(Place
					(p1 . {100@100})
					(p2 . {200@200})
					(p3 . {300@300})
					(p4 . {400@400})
					(p5 . {500@500}))
				(Transition
					(t1 . {100@100})
					(t2 . {200@200})
					(t3 . {300@300})
					(t4 . {400@400})
					(t5 . {500@500}))
				(InputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(OutputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(Marking {#(1 2 3 4 5)}))'.
	Transcript cr; show: list saveString.
	list := LispParser parse: list saveString.
	^list
!

example4
	"LispParser example4."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ~ 8 9) 10)'.
	^list
!

example5
	"LispParser example5."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 {100@100 8 9) 10)'.	"error"
	^list
!

example6
	"LispParser example6."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ''aaaaa 8 9) 10)'.	"error"
	^list
!

example7
	"LispParser example7."

	| list |
	list := LispParser parse: ' `(`(1 2 `3) . `4) '.	"`(`(1 2 `3) . `4) -> (quote ((quote (1 2 (quote 3))) quote 4))"
	^list
! !

!LispParser class methodsFor:'private'!

makeStream: aStream 
	| stream |
	((aStream isKindOf: String)
		or: [aStream isKindOf: Text])
		ifTrue: [stream := ReadStream on: aStream asString]
		ifFalse: [stream := aStream].
	^stream
! !

!LispParser class methodsFor:'utilities'!

parse: aStream 
	^self new parse: (self makeStream: aStream)
!

parse: aStream ifFail: aBlock 
	^self new parse: (self makeStream: aStream)
		ifFail: aBlock
! !

!LispParser methodsFor:'parsing'!

parse: sourceStream 
	| label string |
	^self parse: sourceStream
		ifFail: 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 20).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: label , '\' withCRs , string.
			^LispNil null]
!

parse: sourceStream ifFail: aBlock 
	| result |
	self init: sourceStream ifFail: aBlock.
	result := self scan.
	^result
! !

!LispParser methodsFor:'private'!

init: sourceStream ifFail: aBlock 
	super on: sourceStream.
	failBlock := aBlock
! !

!LispParser methodsFor:'scan and parse'!

scan
	source atEnd ifTrue: [^LispCons null].
	^self scanList
!

scanList
	| expression |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #number ifTrue: [^token].
	tokenType == #string ifTrue: [^token].
	tokenType == #object ifTrue: [^token].
	tokenType == #symbol ifTrue: [^token].
	tokenType == #quote
		ifTrue: 
			[expression := LispCons head: self scanList tail: LispCons null.
			^LispCons head: #quote tail: expression].
	tokenType == #leftParenthesis ifTrue: [^self scanListAux].
	^failBlock value: 'Syntax error'
!

scanListAux
	| cdr |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #rightParenthesis ifTrue: [^LispCons null].
	tokenType == #leftParenthesis ifTrue: [^LispCons head: self scanListAux tail: self scanListAux].
	tokenType == #number ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #string ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #object ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #symbol ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #period
		ifTrue: 
			[cdr := self scanList.
			self nextToken.
			tokenType == #rightParenthesis
				ifTrue: [^cdr]
				ifFalse: [^failBlock value: 'Syntax error']].
	tokenType == #quote
		ifTrue: 
			[cdr := LispCons head: self scanList tail: LispCons null.
			cdr := LispCons head: #quote tail: cdr.
			^LispCons head: cdr tail: self scanListAux].
	self unNextToken.
	^failBlock value: 'Syntax error'
! !

!LispParser methodsFor:'scanning'!

nextToken
	prevMark := mark.
	prevToken := token.
	prevTokenType := tokenType.
	^super nextToken
!

unNextToken
	super unNextToken.
	mark := prevMark.
	token := prevToken.
	tokenType := prevTokenType
! !

!LispParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/lisp/LispParser.st,v 1.6 2002/11/22 20:06:34 cg Exp $'
! !
