"{ Package: 'stx:goodies/lisp' }"

"{ NameSpace: Lisp }"

Object subclass:#LispScanner
	instanceVariableNames:'source mark token tokenType typeTable failBlock compiler'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Lisp'
!

!LispScanner class methodsFor:'documentation'!

documentation
"
    Lisp
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"

! !

!LispScanner class methodsFor:'instance creation'!

new
	^super new initScanner
! !

!LispScanner methodsFor:'initialize-release'!

initScanner
	| label string |
	typeTable := LispScannerTable scannerTable.
	failBlock := 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 10).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: label , '\' withCRs , string].
	compiler := Compiler new
!

on: inputStream 
	source := inputStream.
	mark := source position
! !

!LispScanner methodsFor:'private'!

nextChar
	| char |
	char := source next.
	char = Character lf ifTrue: [char := Character cr].
	char = Character cr
		ifTrue: 
			[source peek = Character lf ifTrue: [source next].
			char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char
!

peekChar
	| char |
	char := source peek.
	char = Character lf ifTrue: [char := Character cr].
	char = Character cr ifTrue: [char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char
!

unNextChar
	source skip: -1
! !

!LispScanner methodsFor:'reading'!

numberFrom: aStream 
	^Number readFrom: aStream
!

objectFrom: aStream 
	| buffer char |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	
	[char := aStream next.
	char = $}]
		whileFalse: 
			[char == nil ifTrue: [^failBlock value: 'Syntax error unmatched ${'].
			buffer nextPut: char].
	^compiler
		evaluate: buffer contents
		in: nil
		to: nil
		notifying: nil
		ifFail: []
!

stringFrom: aStream 
	| buffer char string |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	char = $" ifTrue: [
		[char := aStream peek.
		char ~~ nil]
			whileTrue: 
				[char = $"
					ifTrue: 
						[aStream next.
						char := aStream peek.
						char = $" ifFalse: [^String fromString: buffer contents]].
				buffer nextPut: aStream next]].
	string := aStream upToEnd.
	string size > 100 ifTrue: [string := string copyFrom: 1 to: 100].
	^failBlock value: 'Syntax error unmatched $'''
!

symbolFrom: aStream 
	| buffer char type |
	buffer := WriteStream on: (String new: 20).
	char := aStream peek.
	[char notNil and: [(type := typeTable at: char asInteger) == #xSymbol or: [type == #xDigit or: [type == #xSign]]]]
		whileTrue: 
			[buffer nextPut: aStream next.
			char := aStream peek].
	buffer contents = 'nil' ifTrue: [^LispNil null].
	^Symbol intern: buffer contents
! !

!LispScanner methodsFor:'scanning'!

multiChar: type 
	self perform: type
!

nextToken
	| char |
	mark := source position.
	char := self peekChar.
	tokenType := typeTable at: char asInteger.
	[tokenType == #xDelimiter]
		whileTrue: 
			[self nextChar.
			char := self peekChar.
			tokenType := typeTable at: char asInteger].
	(tokenType at: 1)
		= $x
		ifTrue: [self multiChar: tokenType]
		ifFalse: [self singleChar: tokenType].
	^token
!

singleChar: type 
	self nextChar.
	token := type
!

unNextToken
	source position: mark
! !

!LispScanner methodsFor:'x'!

xBinary
	^failBlock value: 'Syntax error ' , source peek printString
!

xBrace
	tokenType := #object.
	token := self objectFrom: source
!

xComment
	| char |
	[(char := self nextChar) = Character cr]
		whileFalse: [char == nil ifTrue: [^self nextToken]].
	^self nextToken
!

xDigit
	tokenType := #number.
	token := self numberFrom: source
!

xDoubleQuote
	tokenType := #string.
	token := self stringFrom: source
!

xSign
	| char sign |
	sign := self nextChar.
	char := self peekChar.
	char isDigit
		ifTrue: 
			[tokenType := #number.
			token := self numberFrom: source.
			sign == $- ifTrue: [token := token negated]]
		ifFalse: 
			[self unNextChar.
			tokenType := #symbol.
			token := self symbolFrom: source]
!

xSymbol
	tokenType := #symbol.
	token := self symbolFrom: source
! !

!LispScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/lisp/LispScanner.st,v 1.6 2002/11/22 20:06:48 cg Exp $'
! !
