"
 COPYRIGHT (c) 2005-2011 by Jan Vrany & Martin Dvorak

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the 'Software'), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/loggia' }"

"{ NameSpace: Loggia }"

Object subclass:#Logger
	instanceVariableNames:'facilities'
	classVariableNames:'DefaultLogger PreviousLogger ConfiguredTransports'
	poolDictionaries:''
	category:'Loggia'
!

!Logger class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2005-2011 by Jan Vrany & Martin Dvorak

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the 'Software'), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
"
! !

!Logger class methodsFor:'instance creation'!

new

    ^self basicNew initialize
! !

!Logger class methodsFor:'accessing'!

addTransport: aLogTransport

    (self transports includes: aLogTransport)
        ifFalse: [self transports add: aLogTransport].

    ^aLogTransport

    "Modified: / 05-05-2005 / 15:28:42 / masca"
!

default

    DefaultLogger ifNil: [DefaultLogger := self new].
    ^DefaultLogger
!

defaultTransport

    ^self transports
        detect: [:e | e = TranscriptTransport default]
        ifNone: [
            TranscriptTransport default name: 'Transcript (default)'.
            self addTransport: TranscriptTransport default]

    "Created: / 05-05-2005 / 14:04:50 / masca"
!

inMemoryTransports

    ^self transports select:[:t|t isInMemoryTransport]

    "Created: / 17-11-2005 / 12:15:23 / janfrog"
!

nullTransport

    ^self transports 
        detect:
            [:t|t isNullTransport]
        ifNone:
            [|null|
            null := NullTransport new.
            null name:'NULL transport [default]'.
            self addTransport: null.
            null].

    "
        self nullTransport
    "

    "Created: / 27-09-2007 / 08:14:43 / janfrog"
!

removeTransport: aLogTransport

    aLogTransport close.
    self transports
        remove: aLogTransport
        ifAbsent: [self error: 'Given log transport not registered.']
!

systemFacility

    ^#system
!

transports

    ConfiguredTransports ifNil: [ConfiguredTransports := Set new].
    ^ConfiguredTransports
! !

!Logger class methodsFor:'class initialization'!

initialize

    (Smalltalk at:#Logger) ifNil:[Smalltalk at:#Logger put: self default].

    "Created: / 01-09-2011 / 12:26:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

install

    Logger == self default ifTrue:[ ^ self ].    
    PreviousLogger :=  Smalltalk at: #Logger.
    Smalltalk at: #Logger put: self default

    "
        Loggia::Logger install
    "

    "Created: / 02-11-2011 / 12:36:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

uninstall

    Smalltalk at: #Logger put: (PreviousLogger ? MiniLogger instance)

    "
        Loggia::Logger install
    "

    "Created: / 08-04-2012 / 09:53:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Logger class methodsFor:'compatibility'!

for: originatorObject withSeverity: severitySymbol log: messageString

    self log: messageString at: severitySymbol for: originatorObject with: nil
! !

!Logger class methodsFor:'processing'!

commit: severitySymbol

    self
        commit: severitySymbol
        for: thisContext sender receiver
!

commit: severitySymbol for: originatorObject

    self default
        commit: severitySymbol
        for: originatorObject
!

log: messageString at: severitySymbol
    "Log the given message at the given severity. To be called directly as the
    originator object is obtained from the message context."

    self
        log: messageString
        at: severitySymbol
        for: thisContext sender receiver
        with: nil
!

log: messageString at: severitySymbol for: originatorObject
    "Log the given message at the given severity. Set message originator."

    self
        log: messageString
        at: severitySymbol
        for: originatorObject
        with: nil
!

log: messageString at: severitySymbol for: originatorObject with: attachedObject
    "Log the given message at the given severity with the given object as log originator.
    Attach the specified object to the message (if a transport can make use of it)."

    self default
        log: messageString
        at: severitySymbol
        for: originatorObject
        with: attachedObject
!

log: messageString at: severitySymbol with: attachment
    "Log the given message at the given severity. To be called directly as the
    originator object is obtained from the message context."

    self
        log: messageString
        at: severitySymbol
        for: thisContext sender receiver
        with: attachment

    "Created: / 01-09-2011 / 13:54:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

reset: severitySymbol

    self
        reset: severitySymbol
        for: thisContext sender receiver
!

reset: severitySymbol for: originatorObject

    self default
        reset: severitySymbol
        for: originatorObject
! !

!Logger class methodsFor:'testing'!

hasTransportNamed: aString

    ^self transports contains: [:e | e name = aString]
!

transportNamed: aString

    ^self transports detect: [:e | e name = aString] ifNone:[nil].

    "Created: / 07-04-2012 / 09:00:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Logger class methodsFor:'utilities'!

flushLogger

    self flushTransports.
    self default reinitialize

    "
        Logger flushLogger
    "

    "Created: / 30-05-2005 / 15:27:20 / masca"
!

flushTransports

    ConfiguredTransports ifNotNil: [
        ConfiguredTransports do: [:e | e close].
        ConfiguredTransports := nil]

    "
        Logger flushTransports
    "
! !

!Logger methodsFor:'accessing'!

addFacility: aLogFacility as: facilitySymbol

    facilities
        at: facilitySymbol
        put: aLogFacility
!

facilities

    ^facilities
!

registeredFacilities

    ^facilities
!

removeFacilityAt: facilitySymbol

    facilitySymbol = self systemFacility
        ifTrue: [self error: 'System facility cannot be removed.'].

    facilities
        removeKey: facilitySymbol
        ifAbsent: [self error: 'Facility not registered.']
! !

!Logger methodsFor:'initialization'!

initialize

    super initialize.
    self reinitialize

    "Modified: / 30-05-2005 / 15:24:33 / masca"
!

reinitialize

    facilities := IdentityDictionary new.
    self setupSystemFacility

    "Created: / 30-05-2005 / 15:24:21 / masca"
!

setFacilities: facilityDictionary

    facilities := facilityDictionary.
    "Log some message - used to proof that everyting is 
     OK"
    self 
        log: 'Facilities set directly using #setFacilities:'
        at: #warn 
        for: Logger default with: nil

    "Created: / 27-09-2007 / 07:56:54 / janfrog"
! !

!Logger methodsFor:'logging'!

log: message

    self log: message severity: #debug originator: thisContext sender receiver

    "Created: / 15-09-2011 / 10:27:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message facility: facility

    self log: message severity: #debug facility: facility originator: thisContext sender receiver

    "Created: / 14-09-2011 / 21:22:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity

    self log: message severity: #debug originator: thisContext sender receiver

    "Created: / 15-09-2011 / 10:25:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity attachment: attachment

    | originator |
    originator := thisContext sender receiver.

    self log: message severity: #debug facility: (self facilityOf: originator) originator: originator attachment: attachment

    "Created: / 15-09-2011 / 11:47:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity facility: facility

    self log: message severity: severity facility: facility originator: nil

    "Created: / 14-09-2011 / 21:20:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity facility: facility originator: originator

    self log: message severity: severity facility: facility originator: originator attachment: nil

    "Created: / 14-09-2011 / 21:20:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity facility: facility originator: originator attachment: attachment

    self process: (LogMessage
        message: message
        at: severity
        from: originator
        with: attachment)

    "Created: / 14-09-2011 / 21:18:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: message severity: severity originator: originator

    self log: message severity: #debug facility: nil originator: originator

    "Created: / 15-09-2011 / 10:26:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Logger methodsFor:'logging-obsolete'!

log: messageString at: severitySymbol
    "Log the given message at the given severity. To be called directly as the
    originator object is obtained from the message context."

    <resource: #obsolete>

    self
        log: messageString
        at: severitySymbol
        for: thisContext sender receiver
        with: nil

    "Modified: / 15-09-2011 / 10:32:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: messageString at: severitySymbol for: originatorObject
    "Log the given message at the given severity. Set message originator."

    <resource: #obsolete>

    self
        log: messageString
        at: severitySymbol
        for: originatorObject
        with: nil

    "Modified: / 15-09-2011 / 10:32:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: messageString at: severitySymbol for: originatorObject with: attachedObject

    <resource: #obsolete>

    self log: messageString severity: severitySymbol facility: nil originator: originatorObject attachment: attachedObject

    "Modified: / 15-09-2011 / 10:32:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

log: messageString at: severitySymbol with: attachment
    "Log the given message at the given severity. To be called directly as the
    originator object is obtained from the message context."

    <resource: #obsolete>

    self
        log: messageString
        at: severitySymbol
        for: thisContext sender receiver
        with: attachment

    "Created: / 01-09-2011 / 13:54:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Logger methodsFor:'private'!

facilityFor: aSymbol

    ^facilities
        at: aSymbol
        ifAbsent: [facilities
            at: self systemFacility
            ifAbsent: [self error: 'System logging facility undefined, something goes wrong...']]
!

setupSystemFacility
    "Set up a system facility for general messages."

    self addFacility: LogFacility new as: self systemFacility
!

systemFacility

    ^self class systemFacility
! !

!Logger methodsFor:'processing'!

commit: severitySymbol

    self
        commit: severitySymbol
        for: thisContext sender receiver
!

commit: severitySymbol for: originatorObject

    (self facilityFor: originatorObject logFacility) commitSeverity: severitySymbol
!

process: aLogMessage

    (self facilityFor: aLogMessage facility) log: aLogMessage

    "Created: / 15-09-2011 / 10:28:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

reset: severitySymbol

    self
        reset: severitySymbol
        for: thisContext sender receiver
!

reset: severitySymbol for: originatorObject

    (self facilityFor: originatorObject logFacility) resetSeverity: severitySymbol
! !

!Logger methodsFor:'queries'!

allFacilities

    ^facilities keys
!

allSeverities

    ^facilities
        inject: Set new
        into: [:set :facility | set addAll: facility levels keys; yourself]
!

facilityNameFor: aLogFacility

    ^facilities
        keyAtIdentityValue: aLogFacility
        ifAbsent: ['unknown']
!

facilityOf: originator

    ^originator class 
        perform: #logFacility
        ifNotUnderstood: [self class systemFacility]

    "
        Logger facilityOf: Object
        Logger facilityOf: Expecco::Browser
    "

    "Created: / 15-09-2011 / 10:20:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

usesTransport: aLogTransport
    "Answer whether the given transport is in use by some facility
    or multiplex transport. Used to notify the user when removing
    transports."

    ^((self class transports select: [:e | e isMultiplexTransport])
        anySatisfy: [:e | e transports includes: aLogTransport])
        or: [facilities anySatisfy: [:e | e levels includes: aLogTransport]]

    "Modified: / 05-05-2005 / 14:28:47 / masca"
! !

!Logger methodsFor:'utilities'!

configurationString
    "Answer a string that, when evaluated, configures the Logger and brings
    it to the same state as it's now."

    | config |
    config := String new writeStream.
    self storeConfigurationOn: config.
    ^config contents

    "Created: / 05-05-2005 / 15:27:16 / masca"
    "Modified: / 05-05-2005 / 16:47:44 / masca"
!

storeConfigurationOn: aStream
    "The contents of the stream will, when evaluated, configure the Logger and bring
    it to the same state as it's now."

    | transports |
    transports := Dictionary new.
    
    self class transports do: [:e | transports at: e put: ('t' , e name select: [:c | c isLetter])].
    aStream nextPutAll: '| '.        
    transports do: [:e | aStream nextPutAll: e] separatedBy: [aStream space].
    aStream nextPutAll: ' facility |'; cr.

    aStream nextPutAll: 'Logger flushLogger.'; cr; cr.

    transports keysAndValuesDo: [:transport :variable |
        aStream
            nextPutAll: variable;
            nextPutAll: ' := ';
            nextPutAll: transport class name;
            space;
            nextPutAll: (transport class isSingleton ifTrue: ['default'] ifFalse: ['new']);
            nextPut: $.; cr.
        transport storeConfigurationOf: variable on: aStream.
        aStream
            nextPutAll: 'Logger addTransport: ';
            nextPutAll: variable;
            nextPut: $.; cr; cr.
    ].

    self facilities keysAndValuesDo: [:name :facility |
        aStream
            nextPutAll: 'facility := LogFacility new.';
            cr.
        facility levels keysAndValuesDo: [:severity :transport |
            aStream
                nextPutAll: 'facility for: #';
                nextPutAll: severity printString;
                nextPutAll: ' use: ';
                nextPutAll: (transports at: transport ifAbsent: ['Logger defaultTransport']);
                nextPut: $.; cr
        ].
        facility colors keysAndValuesDo: [:severity :color |
            color ifNotNil: [
                aStream
                    nextPutAll: 'facility for: #';
                    nextPutAll: severity printString;
                    nextPutAll: ' useColor: (Color red: ';
                    nextPutAll: color red printString;
                    nextPutAll: ' green: ';
                    nextPutAll: color green printString;
                    nextPutAll: ' blue: ';
                    nextPutAll: color blue printString;
                    nextPutAll: ').'; cr
        ]].
        aStream
            nextPutAll: 'Logger default addFacility: facility as: #';
            nextPutAll: name printString;
            nextPut: $.; cr; cr.
    ].

    aStream nextPut: $!!; cr

    "Created: / 05-05-2005 / 16:45:50 / masca"
    "Modified: / 02-06-2005 / 15:54:55 / masca"
! !

!Logger class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/loggia/Loggia__Logger.st,v 1.5 2012/04/13 00:38:49 vrany Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/loggia/Loggia__Logger.st,v 1.5 2012/04/13 00:38:49 vrany Exp $'
!

version_SVN
    ^ 'Id: Loggia__Logger.st 24 2011-10-04 22:47:00Z vranyj1 '
! !

Logger initialize!
