"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#MatrixReducer
	instanceVariableNames:'matrix pivots detSign'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algorithms'
!

MatrixReducer comment:'A MatrixReducer consists in a matrix, as an instance variable, which will be reduced. For example: MatrixReducer aMatrix. You can ask for the reduced matrix an it''s determinant

        #reduce guides the process, and #nextPivot and #killColumn makes the real job. By overriding only this two methods we''ve made an EuclideanMatrixReducer. 
'
!


!MatrixReducer methodsFor:'accessing'!

det
	"public - I have some determinant stored, if pivots size = matrix rows it's OK,
	if not determinant must be 0"
	| answer |
	self reduce.
	answer _ self privateDetSign.
	1 to: self matrix rowDim do: [:each |
		answer _ answer * (self matrix at: each@each)].
	^ answer
!

matrix: aValue
	"assigns the value of the internal variable matrix and returns self"
	self initialize.
	self privateMatrix: aValue copy.
	^self
!

reducedMatrix
    " public - returns the reduced matrix (compute if needed) "
    self reduce.
    ^self matrix.
!

unusedRows
    " Return a Set of rows where there are no pivots at the moment"
    ^(1 to: self matrix colDim) asSet removeAll: self usedRows ; yourself
!

usedRows
    " Return a Set of rows where there are pivots at the moment"
    ^(self pivots collect: [:eachPivot | eachPivot y]) asSet.
! !

!MatrixReducer methodsFor:'initialization'!

initialize
	"Private - initialize the reducer "
	self initializePivots.
	self privateDetSign: 1.
!

initializePivots
	"Private - initializa the pivots "
    pivots _ OrderedCollection new.
! !

!MatrixReducer methodsFor:'printing'!

printOn: aStream
    self matrix printOn: aStream.
! !

!MatrixReducer methodsFor:'private-accessing'!

matrix
    "returns the value of the internal variable matrix"
    ^matrix
!

pivot
    " private - returns the current pivot"
    ^self pivots isEmpty
        ifTrue: []
        ifFalse: [self pivots last]
!

pivots
    "returns the value of the internal variable pivots"
	pivots isNil ifTrue: [self initializePivots].
    ^pivots
!

pivots: aValue
    "assigns the value of the internal variable pivots
    and returns self"
    pivots := aValue.
    ^self
!

privateDetSign
	^ detSign
!

privateDetSign: aNumber
	detSign _ aNumber
!

privateMatrix: aMatrix
    "assigns the value of the internal variable matrix"
    matrix _ aMatrix
! !

!MatrixReducer methodsFor:'private-proccess'!

killColumn
	" private - kills a column of the matrix using the current pivot "
	| pivotRow |
	pivotRow _ (self matrix row: self pivot y) * (self matrix at: self pivot) reciprocal.
	pivotRow _ pivotRow negated.
	((1 to: self matrix colDim) asSet remove: self pivot y;	yourself)
		do: [:eachRow |
		self matrix
			row: eachRow 
			add: pivotRow * (self matrix at: self pivot x@eachRow)]
!

nextPivot
	" private - Returns the next free pivot"
	| col unusedRows |
	unusedRows _ self unusedRows.
	col _ (self pivot isNil
				ifTrue: [0]
				ifFalse: [self pivot x])
				+ 1.
	[col <= self matrix rowDim]
		whileTrue: 
			[unusedRows do: [:eachRow | (self matrix at: col @ eachRow)
					isNull not ifTrue: [^ col @ eachRow]].
			col _ col + 1].
	^ nil 
!

reduce
	" private - Directs the reducer algorithm"
	| pivot |
	[(pivot _ self nextPivot) isNil not] whileTrue: [
		self pivots add: pivot.
		self killColumn.
		self updateMorph].
	self triangulate.
!

triangulate
	" private - In this implementation when the algorithm finish the
	 matrix is already triangulated"
	1 to: self matrix rowDim do: [:each |
		self matrix
			permuteRow: ((each to: self matrix rowDim) detect: [:another |
				(self matrix at: each@another) isNull not] ifNone: [
					self privateDetSign: self privateDetSign negated.
					each])
			withRow: each.
		self privateDetSign: self privateDetSign negated]
		
! !

!MatrixReducer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/MatrixReducer.st,v 1.1 2002/11/22 19:48:26 cg Exp $'
! !
