"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#MonicMonomial
	instanceVariableNames:'variables'
	classVariableNames:'SortBlock'
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!


!MonicMonomial class methodsFor:'initialization'!

initialize
	"MonicMonomial initialize"
	SortBlock _ [:a :b | (b lexSmallerThan: a) not]
! !

!MonicMonomial class methodsFor:'instance creation'!

fromVariable: aVariable
	" answer a MonicMonomial representing aVariable"
	^ self new add: aVariable; yourself
!

variable: aString 
	" public - answers a MonicMonomial that represents a single variable, 
	  it's usefull for cerating Polynomials as in: 
	| x y | 
	x _ MonicMonomial variable: #x. 
	y _ MonicMonomial variable: #y. 
	x*x*(x*y*y*y) "
	^ self new add: (RaisedVariable name: aString); yourself
!

variable: aSymbol deg: anInteger
	" public - answers a MonicMonomial that represents a single variable elevated to anInteger power, 
	  it's usefull for cerating literals as in: 
	| y2 x3 | 
	x3 _ MonicMonomial variable: #x deg: 3. 
	y2 _ MonicMonomial variable: #x deg: 2.
	x3+y2"
	^ self new add: (RaisedVariable name: aSymbol deg: anInteger); yourself
! !

!MonicMonomial class methodsFor:'accessing'!

sortBlock
	" my SortBlock determines how MonicMonomials and Monomials must be sorted when they are stored in a collection.
	Simply by changing my #initialize you can change the order in which a Polynomial is displayed"
	^ SortBlock
! !

!MonicMonomial methodsFor:'MonicMonomial functions'!

deg
	" public - answer my degree. It's the addition of my variables' degrees"
	| answer |
	answer _ 0.
	self do: [:eachVar | answer _ answer + eachVar deg].
	^ answer
!

derivativeIn: aVariable
	" public - answer my derivative in aVariable"
	^ Monomial
		monicMonomial: (self copy at: aVariable put: ((self at: aVariable) - 1))
		coef: (self at: aVariable)
!

raisedToInteger: anInteger 
	"Public - gives the resulting Monomial after elevating me to the anIntegerth power"
	| mid |
	anInteger < 0 ifTrue: [^ (self raisedToInteger: anInteger negated) reciprocal].
	anInteger = 0 ifTrue: [^ 1].
	anInteger = 1 ifTrue: [^ self].
	mid _ anInteger // 2.
	^ (self raisedToInteger: anInteger - mid)
		* (self raisedToInteger: mid)
!

value: aVariable at: anObject
	" public - evaluates myself when aVariable is anObject"
	| var |
	^ ((var _ self variableAt: aVariable ifNone: [nil])
		ifNil: [self]
		ifNotNil: [((self copy remove: aVariable)*(var valueAt: anObject))]) asSimplePolynomial
!

valueAt: anObject
	" public - evaluates myself at anObject"
	^ self variables isEmpty
		ifTrue: [self asSimplePolynomial]
		ifFalse: [self value: (self variables first) at: anObject]
! !

!MonicMonomial methodsFor:'accessing'!

add: aVariable
	"private - add myself a Variable "
	aVariable deg ~= 0 ifTrue: [self variables add: aVariable]
!

at: aVariable
	" public - answers degree of aVariable in self "
	^(self variableAt: aVariable ifNone: [^0]) deg
!

at: aVariable put: anInteger 
	" public - sets the degree of aVariable"
	self remove: aVariable.
	^ self add: (aVariable clone deg: anInteger)
!

name
	" public - answer my name (it's my variables' name concatenated)"
	| answer |
	answer _ String new.
	self variables do: [:eachVar | answer _ answer , eachVar name].
	^ answer asSymbol
!

remove: aVariable 
	" public - removes aVariable"
	self variables remove: (self variableAt: aVariable ifNone: [nil]) ifAbsent: [nil]
!

removeAllVariables
	^ self initializeVariables
!

size
	" public - answers how many variables I have "
	^ self variables size
!

variables
	variables ifNil: [self initializeVariables].
	^ variables
! !

!MonicMonomial methodsFor:'arithmetic'!

* anObject
	" public - answer a new MonicMonomial that is myself * anObject"
	| answer |
	^ (anObject isMonicMonomial or: [self isMonomial])
		ifTrue: [
			answer _ self copy.
			anObject do: [:eachVariable |
				answer
					at: eachVariable
					put: (self at: eachVariable) + (anObject at: eachVariable)].
			answer]
		ifFalse: [(anObject adaptMonicMonomial: self)*(anObject adaptToMonicMonomial)]
!

+ aMonicMonomial
	^ self asMonomial + aMonicMonomial
!

- anObject
	" public - obvious"
	^ self + anObject negated
!

/ aMonicMonomial 
	" public - answer a new MonicMonomial that is myself / aMonicMonomial"
	^ self * aMonicMonomial reciprocal
!

// anObject
	^ self asPolynomial // anObject
!

\\ anObject
	^ self asPolynomial \\ anObject
!

negated
	" public - answer a Monomial representing my additive inverse"
	^ self asMonomial negated
!

reciprocal
	" public - answer my reciprocal. It's a Monomial containing the my variables' reciprocals"
	| answer |
	answer _ self class new .
	self do: [:eachVar | answer add: eachVar reciprocal].
	^ answer
! !

!MonicMonomial methodsFor:'comparing'!

= aMonicMonomial
	" public - answer whether aMonicMonomial is equal to myself"
	^ self class == aMonicMonomial class and: [
		self variables = aMonicMonomial variables]
!

hash
	" public - answer my hash"
	| answer |
	answer _ 0.
	self do: [:eachVar | answer _ answer + eachVar hash].
	^ answer hash.
!

isMonicMonomial
	^ true
!

isNull

	^ false
!

lexSmallerThan: aMonicMonomial 
	" public - answer whether I'm smaller than aMonicMonomial, lexicografical order "
	self deg = aMonicMonomial deg
		ifFalse: [^ self deg < aMonicMonomial deg].
	^ self variables < aMonicMonomial variables.
! !

!MonicMonomial methodsFor:'converting'!

, anObject
	"Public - answer a Tuple with myself and anObject"
	^ (Tuple new add: self ; add: anObject)
!

adaptFloat: aNumber
	^ self adaptNumber: aNumber
!

adaptFraction: aNumber
	^ self adaptNumber: aNumber
!

adaptInteger: aNumber
	^ self adaptNumber: aNumber
!

adaptMonomial: aMonomial
	"If a Monomial is involved in arithmetic with me answer the Monomial represented in the smaller ring containing both rings involved (my own and Monomial's: Monomial's)"
	^ aMonomial
!

adaptNumber: aNumber
	^ Monomial number: aNumber
!

adaptPolynomial: aPolynomial
	"If a Polynomial is involved in arithmetic with me answer the Polynomial represented in the smaller ring containing both rings involved (my own and Polynomial's: Polynomial's)"
	^ aPolynomial
!

adaptToFloat
	^ self adaptToNumber
!

adaptToFraction
	^ self adaptToNumber
!

adaptToInteger
	^ self adaptToNumber
!

adaptToMonomial
	"If I am involved in arithmetic with a Monomial answer myself represented in the smaller ring containing both rings involved (my own and Monomial's): Monomial"
	^ self asMonomial
!

adaptToNumber
	^ Monomial monic: self
!

adaptToPolynomial
	"If I am involved in arithmetic with a Polynomial answer myself represented in the smaller ring containing both rings involved (my own and Polynomial's): Polynomial"
	^ self asPolynomial
!

adaptToVariable
	" answer a Monomial that represents the given number"
	^ self
!

adaptVariable: aVariable
	" answer a Monomial that represents the given number"
	^ aVariable asMonomial
!

asMonomial
	" answer a Monomial representing me"
	^ Monomial fromMonicMonomial: self
!

asMonomialIn: aVariable
	" answer myself coverted to a Monomial expressend in aVariable"
	^ Monomial
		monicMonomial: (MonicMonomial fromVariable: (self variableAt: aVariable ifNone: [MonicMonomial new]))
		coef: (self copy remove: aVariable)
!

asPolynomial
	" answer a Polynomial representing me"
	^ Polynomial with: self asMonomial
!

asPolynomialIn: aVariable
	" answer a Polynomial representing me expressend in aVariable"
	^ self asPolynomial asPolynomialIn: aVariable
! !

!MonicMonomial methodsFor:'copying'!

copy
	^ self clone variables: self variables copy
! !

!MonicMonomial methodsFor:'enumerating'!

do: aBlock 
	" public - with each Variable do aBlock"
	self variables do: aBlock
!

monomialsDo: aBlock
	" public - added so a MonicMonomial can be easly added to a Polynomial"
	aBlock value: self asMonomial
! !

!MonicMonomial methodsFor:'initialization'!

initializeVariables
	" variables stores all my variables"
	self variables: (SortedCollection sortBlock: RaisedVariable sortBlock)
! !

!MonicMonomial methodsFor:'printing'!

printOn: aStream
	"public - my printed representation is a concatenation of my variables' printed representation"
	| first |
	first _ true.
	self variables do: 
		[:eachVariable | 
		first ifTrue: [first _ false] ifFalse: [aStream nextPut: $*].
		eachVariable printOn: aStream].
	first ifTrue: [1 printOn: aStream].
! !

!MonicMonomial methodsFor:'private'!

variableAt: aVariable ifNone: exceptionBlock 
	" private - answers the variable whos name matches aVariable name if none value exceptionBlock"
	| index |
	index _ self variables findFirst: [:eachVar | eachVar sameAs: aVariable].
	^ index ~= 0
		ifTrue: [ self variables at: index]
		ifFalse: [ exceptionBlock value]
!

variables: aSortedCollection
	variables _ aSortedCollection
! !

!MonicMonomial class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/MonicMonomial.st,v 1.1 2002/11/22 19:47:59 cg Exp $'
! !

MonicMonomial initialize!
