"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

MonicMonomial subclass:#Monomial
	instanceVariableNames:'coeficient'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

Monomial comment:'A Monomial is a MonicMonomial multiplied by a coeficient.
It''s interesting to see how the implementation of #value:at: in MonicMonomial also solves #value:at: for Monomials
'
!


!Monomial class methodsFor:'instance creation'!

fromMonicMonomial: aMonicMonomial
	^ self monicMonomial: aMonicMonomial coef: 1
!

fromNumber: aNumber 
	" public - answer a Monomial that represents a Number"
	^ self monicMonomial: MonicMonomial new coef: aNumber
!

generalNull
	" public - answer a monomial that represents zero with 0 as coeficient.
	  It may not be the apropiate for some type of monomials: when coeficient is not a Number"
	^ self fromNumber: 0
!

literal: aLiteral coef: aCoeficient 
	^ self new variables: aLiteral variables;
	 coef: aCoeficient
!

monic: aMonicMonomial
	^ self monicMonomial: aMonicMonomial coef: 1
!

monicMonomial: aMonicMonomial coef: aCoeficient 
	^ Monomial new variables: aMonicMonomial variables;
	 coef: aCoeficient
!

number: aNumber
	" public - answer a Monomial that represents a Number"
	^ self fromNumber: aNumber
!

variable: aString
	" public - answer a Monomial that represents a Variable whos name is aString"
	^ (super variable: aString)
		coef: 1
!

variable: aString deg: anInteger
	" public - answer a Monomial that represents a Variable whos name is aString"
	^ (super variable: aString deg: anInteger)
		coef: 1
! !

!Monomial methodsFor:'accessing'!

coef
	coeficient ifNil: [self initializeCoeficient].
	^ coeficient
!

coef: aCoeficient 
	coeficient _ aCoeficient.
	self coef isNull ifTrue: [self removeAllVariables]
!

lc
	^ self coef asSimplePolynomial
! !

!Monomial methodsFor:'arithmetic'!

* anObject
	"public - answer the product of myself and anObject"
	^ anObject isMonomial
		ifTrue: [(super * anObject) coef: (self coef * anObject coef)]
		ifFalse: [(anObject adaptMonomial: self)*(anObject adaptToMonomial)]
!

+ anObject
	" public - if my MonicMonomial = aMonomial's MonicMonomial answer a monomial, if not answer a Polynomial"
	^ anObject isMonomial
		ifTrue: [
			super = anObject
				ifTrue: [self class monicMonomial: self coef: (self coef + anObject coef)]
				ifFalse: [Polynomial with: self with: anObject]]
		ifFalse: [(anObject adaptMonomial: self)+(anObject adaptToMonomial)]
!

negated
	" public - answers my additive inverse "
	^ self copy coef: self coef negated
!

reciprocal

	^ super reciprocal coef: self coef reciprocal
! !

!Monomial methodsFor:'comparing'!

= aMonomial
	^ super = aMonomial and: [self coef = aMonomial coef]
!

hash
	^ (super hash * self coef hash) hash
!

isMonic
	^ self coef * self coef = self coef
!

isMonicMonomial
	^ false
!

isNull
	^ self coef isNull
!

negative
	" public - answers whether my coeficient is negative or not"
	^ self coef negative
! !

!Monomial methodsFor:'converting'!

adaptFloat: aNumber
	" answer a Monomial that represents the given number"
	^ self adaptNumber: aNumber
!

adaptFraction: aNumber
	" answer a Monomial that represents the given number"
	^ self adaptNumber: aNumber
!

adaptInteger: aNumber
	" answer a Monomial that represents the given number"
	^ self adaptNumber: aNumber
!

adaptLiteral: aLiteral
	" answer a Monomial that represents the given number"
	^ Monomial monic: aLiteral
!

adaptMonicMonomial: aMonicMonomial
	" answer a Monomial that represents the given MonicMonomial"
	^ aMonicMonomial asMonomial
!

adaptNumber: aNumber
	" answer a Monomial that represents the given number"
	^ Monomial fromNumber: aNumber
!

adaptToFloat
	" answer myself, becouse the integer is what shuld be converted to a Monomial"
!

adaptToFraction
	" answer myself, becouse the integer is what should be converted to a Monomial"
!

adaptToInteger
	" answer myself, becouse the integer is what shuld be converted to a Monomial"
!

adaptToMonicMonomial
	"  answer myself, becouse the Monic Monomial is what should be converted to a Monomial"
	^ self
!

adaptToNumber
	" answer myself, becouse the integer is what shuld be converted to a Monomial"
!

asMonomial
	^ self
!

asSimplePolynomial
	| answer |
	answer _ self coef.
	answer isNumber ifFalse: [answer _ answer asSimplePolynomial].
	self do: [:eachVar |
		answer _ answer * eachVar].
	^ answer
!

isMonomial
	^ true
	
		
! !

!Monomial methodsFor:'copying'!

copy
	^ super copy coef: self coef copy
! !

!Monomial methodsFor:'initialization'!

initializeCoeficient
	self coef: 0
! !

!Monomial methodsFor:'monomial functions'!

derivativeIn: aVariable
	" public - answer my derivative in aVariableOrMonicMonomial"
	| answer |
	answer _ (super derivativeIn: aVariable).
	answer coef: (answer coef*self coef).
	^ answer
! !

!Monomial methodsFor:'printing'!

printOn: aStream 
	self variables size = 0
		ifTrue: [self coef printOn: aStream]
		ifFalse: [
			self coef ~= 1 ifTrue: [
				self coef printOn: aStream.
				aStream nextPut: $*].
			super printOn: aStream]
! !

!Monomial class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Monomial.st,v 1.1 2002/11/22 19:48:15 cg Exp $'
! !
