"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#OpenedInterval
	instanceVariableNames:'left right'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Geometry'
!

OpenedInterval comment:'My instances are mathematical Opened Intervals, they know how to intersect each other and if a point is included or not in them. They also know the result of intersecting each other, how to be translated, its center and to be negated.

        To create one use:
                OpenedInterval left: -6 right: -1/2
                OpenedInterval center: 5 delta: 2.2

        I have one subclass: ClosedInterval, to create SemiClosedInterval or SemiOpenedInterval you just need to override #printOn: and #includes:
'
!


!OpenedInterval class methodsFor:'instance creation'!

center: center delta: delta
	" public - answer a new interval with center in middle and length delta*2.
		ClosedInterval center: 100 delta: 22:"
	^ self left: center - delta right: center+delta
!

left: aNumber right: bNumber
	^ self new
		left: aNumber;
		right: bNumber.
! !

!OpenedInterval methodsFor:'accessing'!

left
	^left
!

left: anObject
	left _ anObject
!

length
	" public - answer my length"
	^ right - left
!

right
	^right
!

right: anObject
	right _ anObject
! !

!OpenedInterval methodsFor:'arithmetic'!

+ aNumber
	" public - answer an interval which is my translation by aNumber.
	It's not necesary to changed this if implementing another kind of Intervals"

	^ self class left: self left+aNumber right: self right + aNumber
!

center
	" public - answer my center.
	It's not necesary to changed this if implementing another kind of Intervals"

	^ self left + self right / 2
!

intersectionWith: anInterval
	" public - answer the intersection between myself and anInterval, nil if it's empty
	It's not necesary to changed this if implementing another kind of Intervals"

	^ (self intersects: anInterval)
		ifFalse: []
		ifTrue: [
			self class
				left: (self left max: anInterval left)
				right: (self right min: anInterval right)]
!

negated
	" public - answer anInterval that is simetrical to me respecting 0
	I's not necesary to changed this if implementing another kind of Intervals"

	^ self class new
		left: self right negated;
		right: self left negated.
! !

!OpenedInterval methodsFor:'printing'!

printOn: aStream
	aStream nextPut: $(.
	self left printOn: aStream.
	aStream nextPut: $,.
	self right printOn: aStream.
	aStream nextPut: $).

! !

!OpenedInterval methodsFor:'testing'!

includes: aNumber
	"public - answer if I include aNumber.
	This should be changed if implementing another kind of Intervals, as OpenInterval"
	^ (self left < aNumber) and: [self right > aNumber]
!

intersects: anInterval
	" public - answer wheader I intersect anInterval.
	It's not necesary to changed this if implementing another kind of Intervals"

	^ ((self right > anInterval left) and: [self left < anInterval right]) or: [
		(anInterval includes: self left) or: [anInterval includes: self right]]
! !

!OpenedInterval class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/OpenedInterval.st,v 1.1 2002/11/22 19:47:50 cg Exp $'
! !
