"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#Sturn
	instanceVariableNames:'polynomials'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algorithms'
!

Sturn comment:'My instances know how to compute the Sturn''s sucesion, and counting sign changes calculates the number of roots in an interval (for doing that, I take the polynomial squareFreed). Then the number of roots is really the number of different roots in the interval. For example: (3 + x + 2*x^2 + x^3) sturnIn:  [-3,0].
        For more examples you can play in the Morphic Word.


| x |
x _ Variable name: #x.
x*x*x + (2*x*x)+x+3;;,f1,f1P|x|
x _ Variable name: #x.
(Sturn new polynomial: (x*x*x+(2*x*x)+x+3)) sturnIn: (ClosedInterval left: (-3) right: 0)
'
!


!Sturn methodsFor:'accesing'!

polynomials

	"Public - answer the current value of the instance variable"
	polynomials ifNil: [self initializePolynomials].
	^ polynomials
!

sturnIn: anInterval

	"Public - answer the number of roots of the polynomial in the interval [a,b]"
	| left right count last |
	self polynomials isNumber ifTrue: [ self polynomials isNull ifTrue: 										[self error: ' tiene infinitas raices']
									ifFalse: [^0]].
	left _ self polynomials collect: [:eachPol | eachPol valueAt: anInterval left].
	right _ self polynomials collect: [:eachPol | eachPol valueAt: anInterval right].
	count _ 0.
	last _ 0.
	left do: [:each |
		count _ count + ((each sign-last) abs // 2).
		(each sign = 0) ifFalse: [last _ each sign]].
	last _ 0.
	right do: [:each |
		count _ count - ((each sign-last) abs // 2).
		(each sign = 0) ifFalse: [last _ each sign]].
	^ count
! !

!Sturn methodsFor:'initializing'!

initializePolynomials
	"Private - initialize the method"
	self polynomials: OrderedCollection new
! !

!Sturn methodsFor:'printing'!

printOn: aStream
	self polynomials printOn: aStream
	
! !

!Sturn methodsFor:'private - accesing'!

polynomial: aPolynomial

	"Private - redefines the instance variable"
	| r1 r2 aux |
	aPolynomial isNumber ifTrue: [^ self polynomials: aPolynomial].
	r1 _ aPolynomial squareFree.
	r2 _ r1 derivative.
	[r1 deg = 0] whileFalse: [
		self polynomials add: r1.
		aux _ (r1 \\ r2) negated.
		r1 _ r2.
		r2 _ aux].
	self polynomials add: r1.
!

polynomials: aCollection
	" private - instance variable polynomials is Sturn's sucession"
	polynomials _ aCollection
! !

!Sturn class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Sturn.st,v 1.1 2002/11/22 19:48:48 cg Exp $'
! !
