"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#Tuple
	instanceVariableNames:'elements'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

Tuple comment:'(Tuple fromString: ''1 2 3 4 5'') * (Tuple fromString: ''1 2 0 0 5'') =
'
!


!Tuple class methodsFor:'instance creation'!

canonic: anInteger dim: bInteger
    "Public - answer a Tuple with all zeros except the anInteger place"

    ^ (Tuple zeros: bInteger) at: anInteger put: 1
!

fromString: aString 
    "public - returns a new Tuple filled in with values from a String like 
     Tuple fromString: '(1 5 2 6 8)' or '1 5 7 4 2' or (1,4,6,4,3) or 5,7,4,3,6
    "
    ^ self newFrom: ((aString findTokens: #($  $, $( $) $    ))
                    collect: [:each | Number readFrom: each])
!

new: n
    self error:'Tuples cannot be created this way'
!

newFrom: anObject
    ^ (anObject isString)
            ifTrue: [self fromString: anObject]
            ifFalse: [self new elements: anObject copy].
!

zeros: anInteger
    "Public - creates a tuple with anInteger zeros"

    | answer |

    answer _ Tuple new.
    "/ anInteger timesRepeat: [ answer add: 0].
    answer elements:(OrderedCollection new:anInteger withAll:0).
    ^ answer

    "
     Tuple zeros:5 
     Tuple zeros:10 
    "
! !

!Tuple methodsFor:'accessing'!

at: anIndex
        "returns value at index anIndex"
    ^self elements at: anIndex
!

at: anIndex put: aValue
        "returns value at index anIndex"
    self elements at: anIndex put: aValue.
    ^self
!

size
    "returns the number of elements of this Tuple"

    ^ elements size
! !

!Tuple methodsFor:'adding'!

add:anElement
    self elements add: anElement.
    ^ self
! !

!Tuple methodsFor:'arithmetic'!

* aTupleOrInteger
    "returns a new Tuple that is the myself * aTupleOrInteger"

    |answer|

    answer _ self copy.
    (aTupleOrInteger isKindOf: Tuple)
        ifTrue: [ self indexesDo: [:each | answer at: each put: ((self at: each) * (aTupleOrInteger at: each))]]
        ifFalse: [answer _ self collect: [:each | each * aTupleOrInteger]].
    ^answer
!

** aTuple
    "answers the inner product between me and aTuple"
    |answer|
    answer _ 0.
    self indexesDo: [:each | answer _ answer + ((self at: each) * (aTuple at: each))].
    ^answer
!

+ aTuple
    "returns a new Tuple that is the addition of self + aTuple"
    |ans|
    ans _ self class new.
    1 to: self size do: [:each | ans add: ((self at: each) + (aTuple at: each))].
    ^ans
!

- aTuple
    "public - returns a new Tuple that is the addition of self + aTuple"
    ^ self + aTuple negated
!

/ aNumber
    "returns a new Tuple that each of my elements / aNumber"
    ^ self class new: (self elements collect: [:each|each / aNumber])
!

// aNumber
    "returns a new Tuple that each of my elements // aNumber"
    ^ self class new: (self elements collect: [:each | each // aNumber])
!

negated
    "public - returns a negated copy of the Tuple"
    |ans|
    ans _ self class newFrom: (self elements collect: [:anElement | anElement negated]).
    ^ ans
!

shift
    "Public - moves all of my elements one place to the right, losing the last one "
    | answer |
    answer _ self copy.
    2 to: (self size) do: [ :each | answer at: each put: (self at: (each - 1))].
    answer at: 1 put: 0.
    ^ answer
! !

!Tuple methodsFor:'comparing'!

= aTuple
    1 to: self size do: [:eachIndex |
            (self at: eachIndex) = (aTuple at: eachIndex) ifFalse: [^false]].
    ^true
!

hash
    |h|

    h := self size.
    self do:[:eachElement | h := ((h bitShift:4) + (eachElement hash)) bitAnd:16r3FFFFFFF].
    ^ h
! !

!Tuple methodsFor:'copying'!

, anElement
    "returns a new Tuple that is self appended with anElement"
    ^(self class newFrom: self elements) add: anElement.
!

copy
    ^ self class newFrom: self elements copy
!

| aTuple
    ^ Matrix new
            addRow: self;
            addRow: aTuple;
            yourself
! !

!Tuple methodsFor:'enumerating'!

collect: aBlock
    " public - answer a new Tuple with aBlock value: eachElement as elements"

    ^ self class newFrom: (self elements collect: aBlock)
!

do: aBlock
    "evaluates aBlock with values"

    elements notNil ifTrue:[
        elements do:aBlock
    ].
    ^self
!

indexesDo: aBlock
    "evaluates aBlock with indexes as values"

    1 to: self size do: [:each | aBlock  value: each].
    ^self
! !

!Tuple methodsFor:'initialization'!

initializeElements
    "assign empty array to elements"

    self elements: OrderedCollection new.
! !

!Tuple methodsFor:'printing'!

printOn: aStream
    |isFirst|

    isFirst := true.
    1 to: self size do: [:eachIndex | 
        aStream 
            nextPut: 
                (isFirst
                    ifTrue: [isFirst := false. $(]
                    ifFalse: [$,]).
        (self at: eachIndex) printOn: aStream
    ].
    aStream nextPut: $)
! !

!Tuple methodsFor:'private'!

elements
    "returns the elements of this Tuple"
    elements isNil ifTrue: [self initializeElements].
    ^elements
!

elements: anArray
    "assigns to the elements of this Tuple"
    elements _ anArray.
    ^self
! !

!Tuple methodsFor:'testing'!

isNull
    self elements do: [:eachElem | eachElem isNull ifFalse: [^ false]].
    ^ true
!

isTuple
    ^ true
! !

!Tuple methodsFor:'tuple functions'!

reverse
    "Public - answer the reverse of me, that is me from end to the biggining " 
    | answer size |
    answer _ self copy.
    size _ self size + 1.
    self indexesDo: [ :each | answer at: each put: ( self at: (size - each))].
    ^ answer
!

reversed
    "Public - answer a reversed copy of me, that is me from end to the biggining " 
    | answer size |
    answer _ self copy.
    size _ self size + 1.
    self indexesDo: [ :each | answer at: each put: ( self at: (size - each))].
    ^ answer
! !

!Tuple class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Tuple.st,v 1.1 2002/11/22 19:48:04 cg Exp $'
! !
