"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#Variable
	instanceVariableNames:'name'
	classVariableNames:'SortBlock'
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

Variable comment:'My instances represent named variables.
Originally I''m planned to be superclass for smarter kind of variables, as RaisedVariables.
My instances have a name: (Variable name: #x) inspect and they know how to ve evaluated at an Object.
'
!


!Variable class methodsFor:'initialization'!

initialize
	"Variable initialize"
	SortBlock _ [:a :b | b lexSmallerOrEqualThan: a]
! !

!Variable class methodsFor:'instance creation'!

name: aString
	^ self new name: aString.
! !

!Variable class methodsFor:'accessing'!

sortBlock
	^ SortBlock
! !

!Variable methodsFor:'accessing'!

name
	" public - answer my name "
	^ name
!

name: aSymbol 
	" public - set my name"
	name _ aSymbol.
! !

!Variable methodsFor:'arithmetic'!

* anObject
	"public - to multiply a Variable by anything, all must be adaped"
	^ (anObject adaptVariable: self) * (anObject adaptToVariable)
!

+ anObject
	"public - to add anything to a Variable, all must be adaped"
	^ (anObject adaptVariable: self) + (anObject adaptToVariable)
!

- anObject
	^ self + anObject negated
!

/ anObject
	^ self * anObject reciprocal
!

// anObject
	"public - to divide a Variable by anything, all must be adaped"
	^ (anObject adaptVariable: self) // (anObject adaptToVariable)
!

\\ anObject
	"public - to divide a Variable by anything, all must be adaped"
	^ (anObject adaptVariable: self) \\ (anObject adaptToVariable)
!

negated
	"public - the only way to negate a Variable is multiplying it by -1 so it will be converted to a Monomial"
	^ self asMonomial negated
! !

!Variable methodsFor:'comparing'!

= aVariable
	^ (self class == aVariable class)
		and: [self name = aVariable name].
!

hash
	^ self name hash
!

isNull

	^ false
!

isVariable
	^ true
!

lexSmallerOrEqualThan: aVariable 
	"public - answer if my name is lexicografically smaller or equal than aVariable's."
	^ (self lexSmallerThan: aVariable) or: [ self = aVariable ].
!

lexSmallerThan: aVariable
	"public - answer if my name is lexicografically smaller than aVariable's."
	^ self name < aVariable name
!

sameAs: aVariable
	"public - answer if I'm reprecenting the same variable as aVariable.
	Variables are identified only by it's name"
	^ self name = aVariable name.
! !

!Variable methodsFor:'converting'!

, anObject
	"Public - answer a Tuple with myself and anObject"
	^ (Tuple new add: self ; add: anObject)
!

adaptAlgebraicNumber: aNumber
	^ Monomial fromNumber: aNumber
!

adaptFloat: aNumber
	^ Monomial fromNumber: aNumber
!

adaptFraction: aFraction
	^ Monomial fromNumber: aFraction
!

adaptInteger: anInteger
	^ Monomial fromNumber: anInteger
!

adaptMonicMonomial: aMonicMonomial
	^ aMonicMonomial
!

adaptMonomial: aMonomial
	^ aMonomial
!

adaptPolynomial: aPolynomial
	^ aPolynomial
!

adaptToAlgebraicNumber
	^ self asMonomial
!

adaptToFloat
	^ self asMonomial
!

adaptToFraction
	^ self asMonomial
!

adaptToInteger
	^ self asMonomial
!

adaptToMonicMonomial
	^ self asMonicMonomial
!

adaptToMonomial
	^ self asMonomial
!

adaptToPolynomial
	^ self asPolynomial
!

adaptToVariable
	^ self asMonicMonomial
!

adaptVariable: aVariable
	^ aVariable asMonicMonomial
!

asMonicMonomial
	" anwer a MonicMonomial representing me"
	^ MonicMonomial fromVariable: self copy
!

asMonomial
	" anwer a MonicMonomial representing me"
	^ Monomial fromMonicMonomial: self asMonicMonomial
!

asPolynomial
	" anwer a MonicMonomial representing me"
	^ Polynomial fromMonicMonomial: self asMonicMonomial
! !

!Variable methodsFor:'evaluating'!

valueAt: anObject
	"public - my evaluation method is:"
	^ anObject
! !

!Variable methodsFor:'printing'!

printOn: aStream
	self name printOn: aStream.
! !

!Variable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Variable.st,v 1.1 2002/11/22 19:48:21 cg Exp $'
! !

Variable initialize!
