"{ Package: 'stx:goodies/math/dhbNumeric' }"

Object subclass:#DhbErfApproximation
	instanceVariableNames:'constant series norm'
	classVariableNames:'UniqueInstance'
	poolDictionaries:''
	category:'Math-DHB Numerical'
!


!DhbErfApproximation class methodsFor:'creation'!

new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance
! !

!DhbErfApproximation methodsFor:'information'!

normal: aNumber
		"Computes the value of the Normal distribution for aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "

	^[ ( aNumber squared * -0.5d) exp * norm]
			on: Error do: [ :signal | signal return: 0]
!

value: aNumber
		"Answer erf( aNumber) using an approximation from Abramovitz and Stegun, Handbook of Mathematical Functions.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	| t |
	aNumber = 0
		ifTrue: [ ^0.5d].
	aNumber > 0
		ifTrue: [ ^1- ( self value: aNumber negated)].
	aNumber < -20
		ifTrue: [ ^0].
	t := 1 / (1 - (constant * aNumber)).
	^( series value: t) * t * (self normal: aNumber)
! !

!DhbErfApproximation methodsFor:'initialization'!

initialize
		"Private - Initialize constants needed to evaluate the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	constant := 0.2316419d.
	norm := 1 / ( Double pi * 2) sqrt.
	series := DhbPolynomial coefficients: #( 0.31938153d -0.356563782d 1.781477937d -1.821255978d 1.330274429d).
! !

!DhbErfApproximation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/dhbNumeric/DhbErfApproximation.st,v 1.3 2006/10/04 13:57:25 cg Exp $'
! !
