"{ Package: 'stx:goodies/math/dhbNumeric' }"

Object subclass:#DhbLanczosFormula
	instanceVariableNames:'coefficients sqrt2Pi'
	classVariableNames:'UniqueInstance'
	poolDictionaries:''
	category:'Math-DHB Numerical'
!


!DhbLanczosFormula class methodsFor:'creation'!

new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance
! !

!DhbLanczosFormula methodsFor:'information'!

gamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) exp * ( self series: aNumber) * sqrt2Pi / aNumber
!

leadingFactor: aNumber
		"Private - Answers the log of the leading factor in Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| temp |
	temp := aNumber + 5.5d.
	^( temp ln * ( aNumber + 0.5d) - temp)
!

logGamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) + ( ( self series: aNumber) * sqrt2Pi / aNumber) ln
!

series: aNumber
		"Private - Answer the value of the series of Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| term |
	term := aNumber.
	^coefficients inject: 1.000000000190015d
						into: [ :sum :each | term := term + 1. each / term + sum]
! !

!DhbLanczosFormula methodsFor:'initialization'!

initialize
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	sqrt2Pi := ( Double pi * 2) sqrt.
	coefficients := #( 76.18009172947146d -86.50532032941677d 24.01409824083091d -1.231739572450155d 0.1208650973866179d-2 -0.5395239384953d-5).
	^self
! !

!DhbLanczosFormula class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/dhbNumeric/DhbLanczosFormula.st,v 1.3 2006/10/04 13:59:13 cg Exp $'
! !
