"{ Package: 'stx:goodies/math/dhbNumeric' }"
!

!Collection methodsFor:'converting'!

asVector
	"Convert the receiver to a vector.
	(c) Copyrights Didier BESSET, 1999, all rights reserved.
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector
! !

!Integer methodsFor:'information'!

gamma
		"Compute the Gamma function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	self > 0
		ifFalse:[ ^self error: 'Attempt to compute the Gamma function of a non-positive integer'].
	^( self - 1) factorial
! !

!Integer methodsFor:'information'!

random
		"Answer a random integer between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^DhbMitchellMooreGenerator new integerValue: self
! !

!Number methodsFor:'operation'!

addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self
! !

!Number methodsFor:'converting'!

asLimitedPrecisionReal
	"Convert the receiver to an instance of
	some subclass of LimitedPrecisionReal.
	This method defines what the default is."

	^self asDouble
! !

!Number methodsFor:'information'!

beta: aNumber
		"Computes the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self logBeta: aNumber) exp
! !

!Number methodsFor:'operation'!

dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)
! !

!Number methodsFor:'information'!

equalsTo: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	^self relativelyEqualsTo: aNumber upTo: DhbFloatingPointMachine new defaultNumericalPrecision
! !

!Number methodsFor:'information'!

errorFunction
		"Answer the error function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^DhbErfApproximation new value: self
! !

!Number methodsFor:'information'!

gamma
	"Compute the Gamma function for the receiver.
	(c) Copyrights Didier BESSET, 1999, all rights reserved.
	Initial code: 11/2/99"

	^self > 1
		ifTrue: [^DhbLanczosFormula new gamma: self]
		ifFalse: [self < 0
				ifTrue: [Double pi / ((Double pi * self) sin * (1 - self) gamma)]
				ifFalse: [(DhbLanczosFormula new gamma: self + 1)
						/ self]]
! !

!Number methodsFor:'information'!

logBeta: aNumber
		"Computes the logarithm of the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self logGamma + aNumber logGamma - ( self + aNumber) logGamma
! !

!Number methodsFor:'information'!

logGamma
		"Computes the log of the Gamma function (for positive numbers only)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self > 1
		ifTrue: [ DhbLanczosFormula new logGamma: self]
		ifFalse:[ self > 0
						ifTrue: [ ( DhbLanczosFormula new logGamma: ( self + 1) ) - self ln ]
						ifFalse: [ ^self error: 'Argument for the log gamma function must be positive']
					]
! !

!Number methodsFor:'operation'!

productWithMatrix: aMatrix
		"Answer a new matrix, product of aMatrix with the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix class rows: ( aMatrix rowsCollect: [ :each | each * self])
! !

!Number methodsFor:'operation'!

productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]
! !

!Number methodsFor:'information'!

random
		"Answers a random number distributed between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^self class random * self
! !

!Number methodsFor:'information'!

relativelyEqualsTo: aNumber upTo: aSmallNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| norm |
	norm := self abs max: aNumber abs.
	^norm <= DhbFloatingPointMachine new defaultNumericalPrecision
		or: [ (self - aNumber) abs < ( aSmallNumber * norm)]
! !

!Number methodsFor:'operation'!

subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated
! !

!Number methodsFor:'operation'!

timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: self
! !

!Number class methodsFor:'information'!

random
		"Answers a random number between 0 and the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^DhbMitchellMooreGenerator new floatValue
! !

!Point methodsFor:'dhb'!

extentFromBottomLeft: aPoint
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^Rectangle origin: ( self - ( 0 @ ( aPoint isInteger ifTrue: [ aPoint]
														 ifFalse:[ aPoint y])))
			   extent: aPoint
! !

!Point methodsFor:'dhb'!

extentFromBottomRight: aPoint
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^Rectangle origin: ( self - aPoint) extent: aPoint
! !

!Point methodsFor:'dhb'!

extentFromCenter: aPoint
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^Rectangle origin: ( self - ( aPoint // 2)) extent: aPoint
! !

!Point methodsFor:'dhb'!

extentFromTopLeft: aPoint
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^Rectangle origin: self extent: aPoint
! !

!Point methodsFor:'dhb'!

extentFromTopRight: aPoint
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^Rectangle origin: ( self - ( ( aPoint isInteger ifTrue: [ aPoint]
													 ifFalse:[ aPoint x]) @ 0))
			   extent: aPoint
! !

!Rectangle methodsFor:'dhb'!

positiveRectangle
		"(c) Copyrights Didier BESSET, 1998, all rights reserved
		 Initial code: 21/4/98 "
	^( origin min: corner) corner: ( origin max: corner)
! !
