"{ Package: 'stx:goodies/math/dhbNumeric/tests' }"

TestCase subclass:#DhbTestIterators
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-DHB Numerical SUnits'
!


!DhbTestIterators methodsFor:'integration'!

integratorTest: anIntegratorClass
	| a |
	a:= anIntegratorClass function:[:x | x sin ] from:0 to:0.
	self assert: (a evaluate equalsTo: 0).
	a:= anIntegratorClass function:[:x | x sin ] from:0 to: Double pi.
	self assert: ((a evaluate - 2) < 0.00001).
	a:= anIntegratorClass function:[:x | x sin ] from:Double pi negated to: 0.
	self assert: ((a evaluate + 2) abs < 0.00001).
	a:= anIntegratorClass function:[:x | x sin ] from: Double pi negated to: Double pi.
	self assert: (a evaluate equalsTo: 0).
!

integratorTestInversedBounds: anIntegratorClass
	| a |
	"Integrator cannot deal with nversed bounds"
	a:= anIntegratorClass function:[:x | x sin ] from:Double pi to: 0.
	self deny: (a evaluate equalsTo: -2).
!

testRomberg
	self integratorTest: DhbRombergIntegrator
!

testRombergInversedBounds

	"Integrator cannot deal with nversed bounds"
	self integratorTestInversedBounds: DhbRombergIntegrator
!

testSimpson
	self integratorTest: DhbSimpsonIntegrator
!

testSimpsonInversedBounds

	"Integrator cannot deal with nversed bounds"
	self integratorTestInversedBounds: DhbSimpsonIntegrator
!

testTrapeze
	self integratorTest: DhbTrapezeIntegrator
!

testTrapezeInversedBounds

	"Integrator cannot deal with nversed bounds"
	self integratorTestInversedBounds: DhbTrapezeIntegrator
! !

!DhbTestIterators methodsFor:'polynomial'!

testRootFind
	| a roots |
	a := DhbPolynomial coefficients: #(0 1 1).
	roots := a roots asSortedCollection.
	self 
		assert: roots size = 2;
		assert: (roots  first equalsTo: -1);
		assert: (roots  last equalsTo: 0)
!

testRootFindComplex
	"Does not support complex roots"
	| a roots |
	a := DhbPolynomial coefficients: #(1 0 1).
	roots := a roots.
	self 
		assert: roots size = 0.
! !

!DhbTestIterators methodsFor:'zero finders'!

testBisectionZeroFinder
	| finder |
	finder := DhbBisectionZeroFinder function: [:x | x + 1].
	finder 
		setNegativeX: -70;
		setPositiveX: 10.
	finder evaluate.
	self 
		assert: finder hasConverged;
		assert:( finder result equalsTo: -1)
!

testBisectionZeroFinderNoZero
	| finder |
	finder := DhbBisectionZeroFinder function: [:x | x*x + 1].
	self 
		should: 
			[finder 
				findNegativeXFrom: -30
				range:  20]
		raise: Error
!

testBisectionZeroFinderSquared
	| finder |
	finder := DhbBisectionZeroFinder function: [:x | x*x - 1].
	finder 
		setNegativeX: -0.9;
		setPositiveX: 10.
	finder evaluate.
	self 
		assert: finder hasConverged;
		assert:( finder result equalsTo: 1)
!

testNewtonZeroFinder
	| finder |
	finder := DhbNewtonZeroFinder 
		function: [:x | x + 1]
		derivative: [:x | 1].
	finder evaluate.
	self 
		assert: finder hasConverged;
		assert:( finder result equalsTo: -1)
!

testNewtonZeroFinderSquared
	| finder |
	finder := DhbNewtonZeroFinder 
		function: [:x | x*x - 1]
		derivative: [:x | 2*x].
	finder evaluate.
	self 
		assert: finder hasConverged;
		assert:( finder result abs equalsTo: 1)
! !

!DhbTestIterators class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/dhbNumeric/tests/DhbTestIterators.st,v 1.3 2006/10/04 14:04:17 cg Exp $'
! !
