"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"

"{ Package: 'stx:goodies/math/distributions' }"

"{ NameSpace: Distributions }"

BernoulliDistribution subclass:#BinomialDistribution
	instanceVariableNames:'numberOfTimes'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Distributions-Instances'
!

!BinomialDistribution class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"
!

documentation
"
    instances generate random numbers with a binomial probability distribution.
"

! !

!BinomialDistribution class methodsFor:'instance creation'!

events: n mean: m 
	n truncated <= 0 ifTrue: [self error: 'number of events must be > 0'].
	^self new events: n mean: m
! !

!BinomialDistribution class methodsFor:'examples'!

example1
	"BinomialDistribution example1."

	| distribution bag set value point |
	TestView openDisplayBlock: [:view :gstate | ]
		actionBlock: 
			[:view :gstate | 
			gstate clear.
			distribution := BinomialDistribution events: 100 mean: 0.25.
			bag := Bag new.
			set := Set new.
			200
				timesRepeat: 
					[value := distribution next rounded.
					bag add: value.
					set add: value.
					set
						do: 
							[:each | 
							point := each * 50 @ (bag occurrencesOf: each).
							gstate displayLineFrom: point to: point]]]
! !

!BinomialDistribution methodsFor:'private'!

events: n mean: m 
	numberOfTimes := n truncated.
	self setParameter: m / numberOfTimes
! !

!BinomialDistribution methodsFor:'probability functions'!

density: x 
	(x between: 0 and: numberOfTimes)
		ifTrue: [^(self computeSample: x outOf: numberOfTimes)
				/ (self computeSample: x outOf: x) * (prob raisedTo: x) * (1.0 - prob raisedTo: numberOfTimes - x)]
		ifFalse: [^0.0]
! !

!BinomialDistribution methodsFor:'random sampling'!

next
	| t |
	t := 0.
	numberOfTimes timesRepeat: [t := t + super next].
	^t
! !

!BinomialDistribution class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/distributions/Distributions-BinomialDistribution.st,v 1.5 2006/10/04 13:04:46 cg Exp $'


! !
