"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"

"{ Package: 'stx:goodies/math/distributions' }"

"{ NameSpace: Distributions }"

ContinuousProbability subclass:#ExponentialDistribution
	instanceVariableNames:'mu'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Distributions-Instances'
!

!ExponentialDistribution class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"
!

documentation
"
    instances generate random numbers with a exponential probability distribution.
"


! !

!ExponentialDistribution class methodsFor:'instance creation'!

mean: p 
	^self parameter: 1.0 / p
!

parameter: p 
	p > 0.0
		ifTrue: [^self new setParameter: p]
		ifFalse: [self error: 'The probability parameter must be greater than 0.0']
! !

!ExponentialDistribution class methodsFor:'examples'!

example1
        "ExponentialDistribution example1."

        | distribution bag set value point |
        TestView openDisplayBlock: [:view :gstate | ]
                actionBlock: 
                        [:view :gstate | 
                        gstate clear.
                        distribution := ExponentialDistribution mean: 100.
                        bag := Bag new.
                        set := Set new.
                        5000
                                timesRepeat: 
                                        [value := distribution next rounded.
                                        bag add: value.
                                        set add: value.
                                        set
                                                do: 
                                                        [:each | 
                                                        point := each @ (bag occurrencesOf: each).
                                                        gstate displayLineFrom: point to: point]]]
! !

!ExponentialDistribution methodsFor:'accessing'!

mean
	^1.0 / mu
!

variance
	^1.0 / (mu * mu)
! !

!ExponentialDistribution methodsFor:'private'!

inverseDistribution: x 
	^x ln negated / mu
!

setParameter: p 
	mu := p
! !

!ExponentialDistribution methodsFor:'probability functions'!

density: x 
	x > 0.0
		ifTrue: [^mu * (mu * x) negated exp]
		ifFalse: [^0.0]
!

distribution: anInterval 
	anInterval last <= 0.0
		ifTrue: [^0.0]
		ifFalse: [^1.0 - (mu * anInterval last) negated exp - (anInterval first > 0.0
					ifTrue: [self distribution: (0.0 to: anInterval first)]
					ifFalse: [0.0])]
! !

!ExponentialDistribution class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/distributions/Distributions-ExponentialDistribution.st,v 1.6 2006/10/04 13:04:46 cg Exp $'


! !
