"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"

"{ Package: 'stx:goodies/math/distributions' }"

"{ NameSpace: Distributions }"

ExponentialDistribution subclass:#GammaDistribution
	instanceVariableNames:'nu'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Distributions-Instances'
!

!GammaDistribution class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"
!

documentation
"
    instances generate random numbers with a gamma probability distribution.
"



! !

!GammaDistribution class methodsFor:'instance creation'!

events: k mean: p 
	| events |
	events := k truncated.
	events > 0
		ifTrue: [^(self parameter: events / p)
				setEvents: events]
		ifFalse: [self error: 'the number of events must be greater than 0']
! !

!GammaDistribution class methodsFor:'examples'!

example1
        "GammaDistribution example1."

        | distribution bag set value point |
        TestView openDisplayBlock: [:view :gstate | ]
                actionBlock: 
                        [:view :gstate | 
                        gstate clear.
                        distribution := GammaDistribution events: 10 mean: 100.
                        bag := Bag new.
                        set := Set new.
                        3000
                                timesRepeat: 
                                        [value := distribution next rounded.
                                        bag add: value.
                                        set add: value.
                                        set
                                                do: 
                                                        [:each | 
                                                        point := each @ (bag occurrencesOf: each).
                                                        gstate displayLineFrom: point to: point]]]
! !

!GammaDistribution methodsFor:'accessing'!

mean
	^super mean * nu
!

variance
	^super variance * nu
! !

!GammaDistribution methodsFor:'private'!

gamma: n 
	| t |
	t := n - 1.0.
	^self computeSample: t outOf: t
!

setEvents: events 
	nu := events
! !

!GammaDistribution methodsFor:'probability functions'!

density: x 
	| t |
	x > 0.0
		ifTrue: 
			[t := mu * x.
			^(mu raisedTo: nu)
				/ (self gamma: nu) * (x raisedTo: nu - 1) * t negated exp]
		ifFalse: [^0.0]
! !

!GammaDistribution class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/distributions/Distributions-GammaDistribution.st,v 1.6 2006/10/04 13:04:46 cg Exp $'


! !
