"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"

"{ Package: 'stx:goodies/math/distributions' }"

"{ NameSpace: Distributions }"

ContinuousProbability subclass:#NormalDistribution
	instanceVariableNames:'mu sigma distribution'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Distributions-Instances'
!

!NormalDistribution class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1991-1994
    Atsushi Aoki (SRA Boulder),
    Katsuhiro Watanabe (SRA Tokyo),
    Taiji Matsuoka (SRA Tokyo),
    Yoshiyuki Nishinaka (SRA Tokyo) and
    Makoto Minagawa (FXIS Tokyo).
    Software Research Associates, Inc.
    1113 Spruce Street, Suite 502
    Boulder, Colorado 80302 the U.S.A.
    TEL: +1-303-440-3582
    FAX: +1-303-440-3930
"
!

documentation
"
    instances generate random numbers with a normal probability distribution.
"





! !

!NormalDistribution class methodsFor:'instance creation'!

mean: a deviation: b 
	b > 0.0
		ifTrue: [^self new setMean: a standardDeviation: b]
		ifFalse: [self error: 'standard deviation must be greater than 0.0']
! !

!NormalDistribution class methodsFor:'examples'!

example1
        "NormalDistribution example1."

        | distribution bag set value point |
        TestView openDisplayBlock: [:view :gstate | ]
                actionBlock: 
                        [:view :gstate | 
                        gstate clear.
                        distribution := NormalDistribution mean: 100 deviation: 15.
                        bag := Bag new.
                        set := Set new.
                        10000
                                timesRepeat: 
                                        [value := distribution next rounded.
                                        bag add: value.
                                        set add: value.
                                        set
                                                do: 
                                                        [:each | 
                                                        point := each @ (bag occurrencesOf: each).
                                                        gstate displayLineFrom: point to: point]]]

    "Modified: 2.4.1997 / 16:09:01 / cg"
! !

!NormalDistribution methodsFor:'accessing'!

mean
	^mu
!

variance
	^sigma squared
! !

!NormalDistribution methodsFor:'private'!

setMean: m standardDeviation: s 
	mu := m.
	sigma := s.
	distribution := UniformDistribution from: -1.0 to: 1.0
! !

!NormalDistribution methodsFor:'probability functions'!

density: x 
	| twoPi t |
	twoPi := 2 * 3.14159.
	t := x - mu / sigma.
	^(-0.5 * t squared) exp / (sigma * twoPi sqrt)
! !

!NormalDistribution methodsFor:'random sampling'!

next
	| v1 v2 s u |
	[v1 := distribution next.
	v2 := distribution next.
	s := v1 squared + v2 squared.
	s >= 1] whileTrue.
	u := (-2.0 * s ln / s) sqrt.
	^mu + (sigma * v1 * u)
! !

!NormalDistribution class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/distributions/Distributions-NormalDistribution.st,v 1.7 2006/10/04 13:04:46 cg Exp $'


! !
