"{ Package: 'stx:goodies/math/examples/fibonacci' }"
!

!Integer methodsFor:'misc math'!

fib
    "compute the fibionacci number for the receiver.
        fib(0) := 0
        fib(1) := 1
        fib(n) := fib(n-1) + fib(n-2)"

    self <= 0 ifTrue:[
        self == 0 ifTrue:[^ 0].
    ].
    ^ self fib_fastUsingDict

    "
     30 fib           
     60 fib
     1000 fib
    "
! !

!Integer methodsFor:'misc math'!

fib_binet
    "compute the fibionacci number for the receiver.
        fib(0) := 0
        fib(1) := 1
        fib(n) := fib(n-1) + fib(n-2)
     This uses Binets formula, which is non recursive, fast, but limited to
     receivers up to roughly 1000.
    "

    |sqrt5|

    self <= 0 ifTrue:[
        self == 0 ifTrue:[^ 0].
    ].

    sqrt5 := 5 sqrt.
    ^ ((((1+sqrt5)/2) raisedTo:self)-(((1-sqrt5)/2) raisedTo:self)) / sqrt5

    "
     10 fib
     10 fib_binet           
     30 fib           
     30 fib_binet           
     60 fib       
     60 fib_binet           
     1000 fib       
     1000 fib_binet           
     1500 fib        
     1500 fib_binet           
     Time millisecondsToRun:[ 1000 fib ]       
     Time millisecondsToRun:[ 1000 fib_binet ]          
    "

    "Created: / 04-10-2006 / 17:34:38 / cg"
! !

!Integer methodsFor:'misc math'!

fib_fast
    "compute the fibionacci number for the receiver.

        Fib(n) = Fib(n-1) + Fib(n-2)

    Knuth:
        Fib(n+m) = Fib(m) * Fib(n+1) + Fib(m-1) * Fib(n)
    "

    |n|

    self <= 20 ifTrue:[
       "/ 0 1 2 3 4 5 6  7  8  9 10 11  12  13  14  15  16   17   18   19   20
        ^ #(1 1 2 3 5 8 13 21 34 55 89 144 233 377 610 987 1597 2584 4181 6765) at:self
    ].

    n := self // 2.
    self odd ifTrue:[
        "/ m is set to n+1; therefore:
        "/ Fib(x) = Fib(n+n+1)      ; x odd; setting n = (x-1)/2
        "/ using Knuth:
        "/ Fib(n+n+1) = Fib(n+1) * Fib(n+1) + Fib(n+1-1) * Fib(n)
        "/            = (Fib(n+1) ^ 2) + (Fib(n) ^ 2)
        ^ n fib_fast squared + (n+1) fib_fast squared
    ].
    "/ as 
    "/    Fib(n+1) = Fib(n) + Fib(n-1)
    "/ therefore:
    "/    Fib(n) = Fib(n+1) - Fib(n-1)
    "/ and, since n is even, n+1 and n-1 are odd, and can be computed as above.
    "/ This gives us:
    "/    Fib(x) = Fib(x+1) - Fib(x-1)      ; x even; setting n = x/2
    "/           = Fib(n+n+1) - Fib(n+n-1)
    "/           = Fib(n+n+1) - Fib((n-1)+(n-1)+1)
    "/           = ((Fib(n+1)^2) + (Fib(n)^2)) - ((Fib((n-1)+1)^2) + (Fib((n-1))^2))   
    "/           = (Fib(n+1)^2) + (Fib(n)^2) - (Fib(n)^2) - (Fib((n-1))^2)   
    "/           = (Fib(n+1)^2) - (Fib((n-1))^2)   
    ^ (n+1) fib_fast squared - (n-1) fib_fast squared

    "
     Time millisecondsToRun:[ 1000 timesRepeat:[50 fib_recursive]]        
     Time millisecondsToRun:[ 1000 timesRepeat:[50 fib_fast]]         

     Time millisecondsToRun:[501 fib_recursive]         
     Time millisecondsToRun:[501 fib_fast]  

     Time millisecondsToRun:[5001 fib_recursive]      
     Time millisecondsToRun:[5001 fib_fast]  

     Time millisecondsToRun:[50001 fib_recursive]     
     Time millisecondsToRun:[50001 fib_fast] 
    "
! !

!Integer methodsFor:'misc math'!

fib_fastUsingDict
    "compute the fibionacci number for the receiver.

        Fib(n) = Fib(n-1) + Fib(n-2)

    Knuth:
        Fib(n+m) = Fib(m) * Fib(n+1) + Fib(m-1) * Fib(n)
    "

    |fibUsingDict dict|

    dict := Dictionary new:100.

    fibUsingDict := [:x |
        |n fib fibN fibNp1 fibNm1 fibXm1 fibXm2 fibXp1|

        x <= 30 ifTrue:[
                "/ 0 1 2 3 4 5 6  7  8  9 10 11  12  13  14  15  16   17   18   19   20    21    22    23    24    25     26     27     28     29     30
            fib := #(1 1 2 3 5 8 13 21 34 55 89 144 233 377 610 987 1597 2584 4181 6765 10946 17711 28657 46368 75025 121393 196418 317811 514229 832040
                     ) at:x
        ] ifFalse:[
            fib := dict at:x ifAbsent:nil.
            fib isNil ifTrue:[
                fibXm1 := dict at:(x-1) ifAbsent:nil.
                fibXm1 notNil ifTrue:[
                    fibXm2 := dict at:(x-2) ifAbsent:nil.
                    fibXm2 notNil ifTrue:[
                        fib := fibXm1 + fibXm2.
                    ] ifFalse:[
                        fibXp1 := dict at:(x+1) ifAbsent:nil.
                        fibXp1 notNil ifTrue:[
                            fib := fibXp1 - fibXm1.
                        ]
                    ]
                ].

                fib isNil ifTrue:[
                    n := x // 2.
                    x odd ifTrue:[
                        "/ m is set to n+1; therefore:
                        "/ Fib(x) = Fib(n+n+1)      ; x odd; setting n = (x-1)/2
                        "/ using Knuth:
                        "/ Fib(n+n+1) = Fib(n+1) * Fib(n+1) + Fib(n+1-1) * Fib(n)
                        "/            = (Fib(n+1) ^ 2) + (Fib(n) ^ 2)
                        fibN   := fibUsingDict value:n.
                        fibNp1 := fibUsingDict value:(n+1).
                        fib := fibN squared + fibNp1 squared
                    ] ifFalse:[
                        "/ as 
                        "/    Fib(n+1) = Fib(n) + Fib(n-1)
                        "/ therefore:
                        "/    Fib(n) = Fib(n+1) - Fib(n-1)
                        "/ and, since n is even, n+1 and n-1 are odd, and can be computed as above.
                        "/ This gives us:
                        "/    Fib(x) = Fib(x+1) - Fib(x-1)      ; x even; setting n = x/2
                        "/           = Fib(n+n+1) - Fib(n+n-1)
                        "/           = Fib(n+n+1) - Fib((n-1)+(n-1)+1)
                        "/           = ((Fib(n+1)^2) + (Fib(n)^2)) - ((Fib((n-1)+1)^2) + (Fib((n-1))^2))   
                        "/           = (Fib(n+1)^2) + (Fib(n)^2) - (Fib(n)^2) - (Fib((n-1))^2)   
                        "/           = (Fib(n+1)^2) - (Fib((n-1))^2)   
                        fibNm1 := fibUsingDict value:(n-1). 
                        fibNp1 := fibUsingDict value:(n+1).
                        fib := fibNp1 squared - fibNm1 squared
                    ].
                ].
                dict at:x put:fib. 
            ]
        ].
        fib
    ].

    ^ fibUsingDict value:self

    "
     Time millisecondsToRun:[50000 fib_iterative]     
     Time millisecondsToRun:[50000 fib_fast]          
     Time millisecondsToRun:[50000 fib_fastUsingDict]  

     Time millisecondsToRun:[100000 fib_iterative]     
     Time millisecondsToRun:[100000 fib_fast]          
     Time millisecondsToRun:[100000 fib_fastUsingDict]  

     1 to:100 do:[:i | self assert:(i fib_iterative = i fib_fast) ]       
     1 to:100 do:[:i | self assert:(i fib_iterative = i fib_fastUsingDict) ]
     1 to:100 do:[:i | self assert:(i fib_iterative = i fib) ]
    "
! !

!Integer methodsFor:'misc math'!

fib_iterative
    "compute the fibionacci number for the receiver.
     (same result as returned by #recursiveFib.
      This method shows how a changed algorithm can
      change things much more drastic than tuning ...
      ... compare 30 recursiveFib with 30 fib;
      and dont even try 60 recursiveFib)"

    |a1 a2 an|

    a1 := 1.
    a2 := 0.
    an := self.
    (self - 1) timesRepeat:[
        an := a1 + a2.
        a2 := a1.
        a1 := an
    ].
    ^ an

    "
     Transcript showCR:(Time millisecondsToRun:[30 fib_iterative]).            
     Transcript showCR:(Time millisecondsToRun:[30 fib_recursive]). 

     Transcript showCR:(Time millisecondsToRun:[60 fib_iterative]).
     Transcript showCR:(Time millisecondsToRun:[200 fib_iterative]).
     Transcript showCR:(Time millisecondsToRun:[1500 fib_iterative]).
     Transcript showCR:(Time millisecondsToRun:[5000 fib_iterative]).
     Transcript showCR:(Time millisecondsToRun:[10000 fib_iterative]).

     Transcript showCR:(Time millisecondsToRun:[10000 fib]).
    "
! !

!Integer methodsFor:'misc math'!

fib_recursive
    "compute the fibionacci number for the receiver.
     WARNING:
         Dont use this method if you need fibionacci numbers -
         this method is for benchmarking purposes only.
         (use #fib instead and dont ever try 60 fib_recursive ...)"

    (self > 1) ifTrue:[
        ^ (self - 1) fib_recursive + (self - 2) fib_recursive
    ].
    ^ self

    "
     30 fib
     30 fib_recursive

     Transcript showCR:(Time millisecondsToRun:[30 fib_recursive]) 
     Transcript showCR:(Time millisecondsToRun:[30 fib_iterative])
     Transcript showCR:(Time millisecondsToRun:[30 fib])

     Transcript showCR:(Time millisecondsToRun:[6000 fib_iterative])
     Transcript showCR:(Time millisecondsToRun:[6000 fib])
    "

    "Modified: 18.7.1996 / 12:47:19 / cg"
! !

