"{ Package: 'stx:goodies/math/fibonacci' }"

"{ NameSpace: Math }"

FibonacciCalculator subclass:#RelativeFibonacciCalculator
	instanceVariableNames:'fn1 fn n'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Algorithms'
!

!RelativeFibonacciCalculator class methodsFor:'documentation'!

documentation
"
(See #positiveF: comment if you don't know how I do my job)

        I'm not not a very fast implementation for computing a random acces Fibonacci
        Sequence, but if you know that numbers will be requested in order, or from one
        request to the next the distance is not very large, I recomend myself as a good 
        FibonacciCalculator.

        For example to compute an intervale of the sequence,

                | fc |
                fc _ RelativeFibonacciCalculator new.
                ((0 to: 100) collect: [:eachIndex | fc f: eachIndex]) inspect

        or to compute muh and lambda (See FibonacciCalculator's #lambdaMuh)

                | fc |
                fc _ RelativeFibonacciCalculator new.
                ('Calculating 100 lambdas'
                        displayProgressAt: Sensor cursorPoint from: 1 to: 100 during: 
                                [:bar | (1 to: 100) collect: [:eachIndex | bar value: eachIndex. fc m: eachIndex. fc lambda]]) inspect

        Did you inspect the last chunk? Did you notice something? Lets call Ln this sequence of
        lambdas, I saw (just saw) that for any not prime n, Ln = lcm { divisors of n }. What's that?
        Together with Ariel Pacetti we have already proved this statement.

        Sometimes this is a fast implementation: 

                | fc a b |
                ('Timing two implementations'
                        displayProgressAt: Sensor cursorPoint from: 1 to: 100 during: [:bar |
                                fc _ RelativeFibonacciCalculator new.
                                a_(Time millisecondsToRun: [(1 to: 50) collect: [:eachIndex |
                                                fc m: eachIndex.
                                                fc lambda.
                                                bar value: eachIndex]]).
                                fc _ Split2FibonacciCalculator new.
                                b_(Time millisecondsToRun: [(1 to: 50) collect: [:eachIndex |
                                                fc m: eachIndex.
                                                fc lambda.
                                                bar value: 50+eachIndex]]).
                                (Array with: a with: b)]) inspect
"
! !

!RelativeFibonacciCalculator methodsFor:'accessing'!

fn1: anInteger 
	^ fn1 _ self normalize: anInteger
!

fn: anInteger
	^ fn _ self normalize: anInteger.
!

m: anInteger
	super m: anInteger.
	self n: 1
!

n: anInteger
	| fc |
	n _ anInteger.
	fc _ BinaryFibonacciCalculator new m: self m.
	self fn1: (fc f: n-1).
	^ self fn: (fc f: n).
! !

!RelativeFibonacciCalculator methodsFor:'computing'!

positiveF: anIndex 
	" Computes the anIdnexth number in Fibonacci sequence.  
	  My instances know a given n, Fn and Fn-1 (n,fn,fn1), when this message is sent  
	  I use Fn+1=Fn+Fn-1 or Fn-2=Fn-Fn-1 whether anIndex > n or anIndex < n to compute  
	  the requested Fn, then I save this n, Fn and Fn-1 as current state."
	| fnew |
"	(anIndex - n) abs > (4*(anIndex - n) abs highBit)"
"		ifTrue: [^ self n: anIndex]."
	self initializeIfNotDone.
	[n < anIndex]
		whileTrue: 
			[fnew _ fn + fn1.
			self fn1: fn.
			self fn: fnew.
			n _ n + 1].
	[n > anIndex]
		whileTrue: 
			[fnew _ fn - fn1.
			self fn: fn1.
			self fn1: fnew.
			n _ n - 1].
	^ fn
! !

!RelativeFibonacciCalculator methodsFor:'copying'!

copy
	^ self class new m: self m
! !

!RelativeFibonacciCalculator methodsFor:'initialization'!

initializeIfNotDone
	n ifNil: [self n: 1].
! !

!RelativeFibonacciCalculator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/fibonacci/RelativeFibonacciCalculator.st,v 1.1 2002/11/22 16:49:31 cg Exp $'
! !
