"{ Package: 'stx:goodies/math/graph' }"

"{ NameSpace: Math }"

Object subclass:#Graph
	instanceVariableNames:'vertices'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Graph'
!

!Graph class methodsFor:'documentation'!

documentation
"
    This class has been ported from Squeak.
    It is provided *AS IS* without any warranty.

    Original comment:

    My instances are Graphs that can be shown in a morphic world.

    The graphical representation is achieved by thinking that each pair of nodes is connected
    with a contractile or expansive spring, for connected or unconnected nodes, respectively.
    This makes connected nodes stay close to each other, and unconnected nodes away.
    [not finished yet]
    Mis instancias son grafos que pueden ser vistos en un morphic world.

    La representacion grafica es lograda pensando que cada par de nodos esta conectado con un resorte contractil o expansivo, dependiendo si los nodos estan conectados o desconectados.
    Esto produce que los nodos conectados traten de mantenerse cerca unos de los otros, y que los
    desconectados se alejen. [todavia no esta terminado]
"
!

examples
"
    |graph 
     stuttgart muenchen frankfurt berlin|

    graph := Graph new.
    stuttgart := Vertex name:'stuttgart'.
    muenchen := Vertex name:'muenchen'.
    frankfurt := Vertex name:'frankfurt'.
    berlin := Vertex name:'berlin'.

    graph addArrowFrom:stuttgart to:muenchen.
    graph addArrowFrom:stuttgart to:frankfurt.
    graph addArrowFrom:muenchen to:berlin.
    graph addArrowFrom:frankfurt to:berlin.
    graph inspect.
"
! !

!Graph class methodsFor:'instance creation'!

for: anObject 
	^ self for: anObject depth: -1
!

for: anObject depth: anInteger
	^ self for: anObject depth: anInteger addTo: (self new)
!

for: anObject depth: anInteger addTo: aGraph
        | objectName |
        anInteger = 0 ifFalse: [
                objectName := anObject name asSymbol.
                (self subpartsOf: anObject) do: [:each |
                        aGraph addArrowFrom: objectName to: each name asSymbol.
                        self for: each depth: anInteger - 1 addTo: aGraph]].
        ^ aGraph
!

hierarchyOf: anObject
        | answer object |
        answer := self new.
        object := anObject.
        [object superclass isNil] whileFalse: [
                answer addArrowFrom: object superclass name asSymbol to: object name asSymbol.
                object := object superclass].
        ^ self hierarchyOf: anObject addTo: answer
!

hierarchyOf: anObject addTo: aGraph
        | objectName |
        objectName := anObject name asSymbol.
        anObject subclasses do: [:eachSubclass |
                aGraph addArrowFrom: objectName to: eachSubclass name asSymbol.
                self hierarchyOf: eachSubclass addTo: aGraph].
        ^ aGraph
!

new
	^ super new initialize
!

subpartsOf: anObject
        | subparts |
        subparts := OrderedCollection new.
        1 to: anObject class instSize do: [ :each | subparts add: (anObject instVarAt: each)].
        1 to: anObject basicSize do: [ :each | subparts add: (anObject basicAt: each)].
        ^ subparts
! !

!Graph methodsFor:'accessing'!

edges
        " public - answer aCollection of all my edges"
        | answer |
        answer := OrderedCollection new.
        self vertices do: [:each | answer addAll: each edges].
        ^ answer
!

includes: aVertex
        ^ (self vertexForName: aVertex) notNil
!

vertexForName: aVertexOrSymbol 
	" public - answer a node whos name matches aVertexOrSymbol's.
	  If it doesn't exist answer nil"
	^ self vertexForName: aVertexOrSymbol ifNone: [nil]
!

vertexForName: aVertexOrSymbol ifNone: aBlock
        " public - answer a node whos name matches aVertexOrSymbol's.
          If it doesn't exist do aBlock"
        | aVertexName |
        aVertexName := aVertexOrSymbol name asSymbol.
        ^ self vertices
                detect: [:one | one name = aVertexName]
                ifNone: [aBlock value]
!

vertexForNameIfNoneAdd: aVertexOrSymbol
	" public - answer a vertex whos name matches aVertexOrSymbol's
	  If it doesn't exist create one (and add myself it)"
	^ self vertexForName: aVertexOrSymbol
		ifNone: [^ self addVertex: (Vertex name: aVertexOrSymbol name asSymbol)]
!

vertices
	^vertices
!

vertices: anObject
        vertices := anObject
!

weights
        " public - answer a set with all the wieghts I have"
        | answer |
        answer := (self edges collect: [:eachArrow | eachArrow weight]).
        answer remove: nil ifAbsent: [].
        ^ answer asSet
! !

!Graph methodsFor:'adding - removing'!

addAllVertices: aVerticesCollection
	" public - add aVertex to my vertices"
	aVerticesCollection do: [:eachVertex | self vertices add: eachVertex].
	self updateMorph.
	^ aVerticesCollection
!

addArcFrom: aVertexOrSymbol to: bVertexOrSymbol
	" public - add an arc from aVertex to bVertex.
	  adds two arrows"
	self addArrowFrom: aVertexOrSymbol to: bVertexOrSymbol.
	self addArrowFrom: bVertexOrSymbol to: aVertexOrSymbol
!

addArrowFrom: aVertexOrSymbol to: bVertexOrSymbol
	" public - add an arraw from node whos name is aVertexOrSybol name
	  to node whos name is bVertexOrSymbol"
	self addArrowFrom: aVertexOrSymbol to: bVertexOrSymbol weighted: nil.
	^bVertexOrSymbol
!

addArrowFrom: aVertexOrSymbol to: bVertexOrSymbol weighted: anObject
        " public - add an arraw from node whos name is aVertexOrSybol name
          to node whos name is bVertexOrSymbol with weight anObject"
        (self vertexForNameIfNoneAdd: aVertexOrSymbol)
                addArrowTo: (self vertexForNameIfNoneAdd: bVertexOrSymbol) weight: anObject.
"/        self updateMorph
        self changed:#edges.
!

addEdge: anEdge
	" public - add an edge to my edges
	            will add an Arrow to propper vertex"
	self addArrowFrom: anEdge from to: anEdge to weighted: anEdge weight
!

addVertex: aVertex
        " public - add aVertex to my vertices"
        self vertices add: aVertex.
"/        self updateMorph.
        self changed:#vertices.
        ^ aVertex
!

removeArrowFrom: aVertexOrSymbol to: bVertexOrSymbol
	" public - remove an arrow from aVertex* to bVertex*"
	(self vertexForName: aVertexOrSymbol) removeArrowTo: bVertexOrSymbol.
	self updateMorph
!

removeVertex: aVertex
	" public - add aVertex to my vertices"
	self vertices remove: aVertex ifAbsent: [].
	self updateMorph.
	^ aVertex
! !

!Graph methodsFor:'converting'!

asMatrix
        | answer |

        self halt:'unimplemented'.

        answer := Math::Matrix zeros: self vertices size.
        1 to: self vertices size do: [:each |
                (self vertices at: each) adjacentVertices do: [:eachVertex |
                        answer at: each@(self vertices indexOf: eachVertex) put: 1]].
        ^ answer
!

asSingleEdgeGraph
        " public - answer a new SingleEdgedGraph with same edges as myself"
        ^ SingleEdgedGraph fromGraph: self
!

morphicWrapperClass
        ^ GraphMorphicWrapper
! !

!Graph methodsFor:'graph functions'!

addSubGraphFor: aVertex to: aGraph
	aVertex arrows do: [:eachArrow |
		(aGraph includes: eachArrow to)
			ifFalse: [
				aGraph addArrowFrom: aVertex to: eachArrow to.
				self addSubGraphFor: eachArrow to to: aGraph]
			ifTrue: [
				aGraph addArrowFrom: aVertex to: eachArrow to]].
	^ aGraph
!

hasLoop
        ^ self vertices contains:[:aVertex | (self hasLoopOn: aVertex)].
!

hasLoopOn: aVertex
        ^ (self vertexForName: aVertex) arrows 
            contains: [:anArrow | (self subGraphFor: anArrow to) includes: aVertex].
!

squared
        " public - answer a new Graph wich is my squared graph.
          That's a new Graph whos edges are (a,c) if and only if
          I have an edge (a,b) and an edge (b,c) for some node b.
                This notion can be extended to multilpication of Graphs, see #*"
        | answer |
        answer := self species new.
        self edges do: [:eachEdge |
                eachEdge to adjacentVertices do: [:eachTherd |
                                        answer addArrowFrom: eachEdge from to: eachTherd]].
        ^ answer
!

subGraphFor: aVertex
        | answer |
        answer := self species new.
        ^ self addSubGraphFor: (self vertexForName: aVertex) to: answer.
!

transposed
        " public - answer my transposed Graph.
          see [ALG] Exercise 23.1-3.
                The running time of this implementation is O(V+E),
                more accurate, it's O(V+(c1*E+c2*E) as edges (arrows) are walked once to answer
                #edges (c1), and once again to add each reversed Edge (c2).
                An implementation of an O(V+c*E) algorithm (as insinuated in [ALG])
                requires to add reversed Edges at same time than collecting edges, but this
                breakes out factorization, how ever, the total running time is the same (c=c1+c2),
                as all operations performed in one implementation should be implemented in the
                other (you have to ADD each new edge, COLLET each edge and WALK each vertex)"
        | answer |
        answer := self species new.
        self edges do: [:eachEdge |
                answer addArrowFrom: eachEdge to to: eachEdge from weighted: eachEdge weight].
        ^ answer
! !

!Graph methodsFor:'initialization'!

initialize
	self vertices: OrderedCollection new
! !

!Graph methodsFor:'printing'!

printOn: aStream
	" my printString is a list of all my vertices and their adjacencyList"
	self vertices do: [:each |
		each printOn: aStream. 
		aStream cr]
! !

!Graph methodsFor:'testing'!

hasArrowFrom: aVertexOrSymbol to: bVertexOrSymbol
	" public - answer whether I have an arrow from aVertexOrSymbol to bVertexOrSymbol"
	^ (self hasVertex: (aVertexOrSymbol)) and: [
		(self vertexForName: aVertexOrSymbol) hasAnArrowTo: bVertexOrSymbol]
!

hasVertex: aVertexOrSymbol
        " public - answer whether I have the Vertex aVertexOrSymbol"
        ^ (self vertexForName: aVertexOrSymbol) notNil 
! !

!Graph class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/graph/Graph.st,v 1.2 2003/03/26 12:21:34 cg Exp $'
! !
