"{ Package: 'stx:goodies/math/graph' }"

"{ NameSpace: Math }"

Squeak::MorphicWrapper subclass:#GraphMorphicWrapper
	instanceVariableNames:'deltaTotal desiredDistance'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Graph'
!

!GraphMorphicWrapper class methodsFor:'documentation'!

documentation
"
My instances are Graphs that can be shown in a morphic world.

The graphical representation is achieved by thinking that each pair of nodes is connected
with a contractile or expansive spring, for connected or unconnected nodes, respectively.
This makes connected nodes stay close to each other, and unconnected nodes away.
[not finished yet]
Mis instancias son grafos que pueden ser vistos en un morphic world.

La representacion grafica es lograda pensando que cada par de nodos esta conectado con un resorte contractil o expansivo, dependiendo si los nodos estan conectados o desconectados.
Esto produce que los nodos conectados traten de mantenerse cerca unos de los otros, y que los
desconectados se alejen. [todavia no esta terminado]
"
! !

!GraphMorphicWrapper methodsFor:'accesing'!

desiredDistance
	" answer the distance I like to be my vertices"
	^ desiredDistance
!

desiredDistance: anInteger
	" set the distance I like to be my vertices"
	desiredDistance _ anInteger.
	self updateContents
! !

!GraphMorphicWrapper methodsFor:'delegation'!

edges
	^ self object edges
!

vertices
	^ self object vertices
! !

!GraphMorphicWrapper methodsFor:'drawing'!

drawOn: aCanvas
	" draw edges"
	| to |
	super drawOn: aCanvas.
	self object edges do: [:eachEdge |
		aCanvas
			line: eachEdge from asMorph center
			to: (to _ eachEdge to asMorph center)
			width: 1
			color: Color red.
"		aCanvas					""This lines draw arrow's head""
			fillOval: ((to -(10@3)) corner: (to + 3))
			color: Color red"].
! !

!GraphMorphicWrapper methodsFor:'event handling'!

aboutToRemove: aMorph
	self object removeVertex: aMorph object.
!

acceptDroppingMorph: aMorph event: evt
	" if it's a Vertex, just accept it... this is to allow vertices rearranging. It have to be diferent"
	(aMorph isKindOf: VertexMorphicWrapper) ifTrue: [
		self object addVertex: aMorph object.
		aMorph dxdy: 0@0.
		self addMorph: aMorph.
		^ self startStepping.].
	^ super acceptDroppingMorph: aMorph event: evt.
!

addCustomMenuItems: aCustomMenu hand: aHandMorph
	"r++ 5/16/1998 19:02
		added startStepping"
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu
		addLine;
		add: 'shuffle' action: #shuffle
!

allowSubmorphExtraction
	" allow extraction so submorphs can be grabbed and dragged to a new position.
	  usefull to rearrange the vertices manually"
	^ true
!

shuffle
	" random move all submorphs"
	self submorphs do: [:eachMorph |
		eachMorph dxdy: 101 atRandom@101 atRandom - 50].
	self startStepping
! !

!GraphMorphicWrapper methodsFor:'initialization'!

initialize
	super initialize.
	desiredDistance _ 40.
! !

!GraphMorphicWrapper methodsFor:'stepping'!

aproachConnectedNodes
	" connected nodes are tied with a rubber band', they aproach each other"
	| len f from to |
	self edges do: [:eachArc |
		from _ eachArc from asMorph.
		to _ eachArc to asMorph.
		(len _ (from center - to center) r) = 0 ifFalse: [
			f _ (self desiredDistance - len) asFloat / (len * 4).
			to dxdy: to dxdy - (from center - to center * f).
			from dxdy: from dxdy +  (from center - to center * f)]]
!

separateAllNodesAndCenter
	" will set dxdy of each node according to the computation of the separation.
	  the graph is centered in my bounds"
	| dxdy c1 c2 inBounds |
	inBounds _ (self submorphs at: 1 ifAbsent: [^ self]) center.
	inBounds _ inBounds corner: inBounds.
	self vertices do: [:node1 |
		dxdy _ 0@0.
		inBounds _ inBounds encompass: (c1 _ node1 asMorph center).
		self vertices do: [:node2 |
			c2 _ node2 asMorph center.
			(node1 == node2 or: [(c1 - c2) r > 80]) ifFalse: [
				c1 = c2
					ifTrue: [dxdy _ dxdy + (3 atRandom@3 atRandom)]
					ifFalse: [dxdy _ dxdy + ((c1 - c2) / (c1 - c2) r squared)]]].
		dxdy r > 0 ifTrue: [
			node1 asMorph dxdy: (node1 asMorph dxdy + (dxdy / dxdy r * 2) min: 10@10)]].
	self vertices do: [:each | "center"
		each asMorph dxdy: each asMorph dxdy - inBounds center + self bounds center].
	self bounds: (inBounds expandBy: 5).
!

step
	" will calculate a new position for every submorph, based on gravitanional low.
	  it's not working as desired, must be changed for some other aproach"
	deltaTotal _ 0.
	self vertices do: [:eachVertex |
		(self submorphs includes: eachVertex asMorph) ifFalse: [
			eachVertex asMorph position: self center.
			self addMorph: eachVertex asMorph]].
	self aproachConnectedNodes; separateAllNodesAndCenter.
	self vertices do: [:each |
		deltaTotal _ deltaTotal max: each asMorph doMove.
		each asMorph dxdy: each asMorph dxdy / 2].
	deltaTotal < 0.5 ifTrue: [self stopStepping].
	self changed	
!

stepTime
	"r++ 12/25/97 00:07
	as fast as possible"
	^ 0		
!

updateContents
	self startStepping
! !

!GraphMorphicWrapper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/graph/GraphMorphicWrapper.st,v 1.1 2002/11/22 16:46:51 cg Exp $'
! !
