"{ Package: 'stx:goodies/math/graph' }"

"{ NameSpace: Math }"

Object subclass:#Vertex
	instanceVariableNames:'name arrows'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Graph'
!

!Vertex class methodsFor:'documentation'!

documentation
"
    This class has been ported from Squeak.
    It is provided *AS IS* without any warranty.

    Original comment:
"
! !

!Vertex class methodsFor:'instance creation'!

name: aSymbol
	" creates and answers a new Vertex whos name is aSymbol"
	^ self new name: aSymbol
!

new
	^ super new initialize
! !

!Vertex methodsFor:'accessing'!

adjacentVertices
	" public - answer all the verteces I have in my adjacencyList"
	^ self arrows collect: [:each | each to]
!

arrows
	^arrows
!

name
	"answer my name"
	^ name
!

name: aSymbol
	"set my name"
	name _ aSymbol
! !

!Vertex methodsFor:'adding - removing'!

addArrowTo: aVertex
	" public - will add an Edge from myself to aVertex to my adjacency list"
	self addArrowTo: aVertex weight: nil
!

addArrowTo: aVertex weight: anObject
	" public - will add an Edge from myself to aVertex with weight anObject to my adjacency list"
	self arrows add: (Arrow to: aVertex weight: anObject)
!

removeArrowTo: aVertexOrSymbol
	" public - remove the Arrow to aVertex* from my arrows, if it exists"
	self arrows
		remove: (Arrow to: (Vertex name: aVertexOrSymbol name asSymbol)) ifAbsent: [nil]
! !

!Vertex methodsFor:'comparing'!

= aVertex
	" public - Vertices are identified by its name"
	^ self name asSymbol = aVertex name asSymbol
!

hash
        " public - Vertices are identified by its name"
        ^ self name hash
! !

!Vertex methodsFor:'initialization'!

initialize
	" my adjacencyList will be an OrderedCollection"
	arrows _ OrderedCollection new
!

morphicWrapperClass
        ^ VertexMorphicWrapper
! !

!Vertex methodsFor:'printing'!

printOn: aStream
        "my text representation is my name followed of my adjacecyList"

        |first|

        self name printOn: aStream.
        aStream nextPutAll:' ~> ('.
        self adjacentVertices do: [:each |
            first 
                ifTrue:[first := false]
                ifFalse:[aStream space.].
            each name printOn: aStream.
        ].
        aStream nextPut: $)
! !

!Vertex methodsFor:'vertex functions'!

edges
        " public - answer a Collection of all the edges that coneccts me with my adjacent nodes"
        ^ self arrows collect: [:each | Edge from: self arrow: each]
!

hasAnArrowTo: aVertexOrSymbol
	" public - answer whether I have an arrow to aVertexOrSymbol or not"
	^ self adjacentVertices includes: (Vertex name: aVertexOrSymbol name asSymbol)
! !

!Vertex class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/graph/Vertex.st,v 1.2 2003/03/26 12:21:29 cg Exp $'
! !
