"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearEquation
	instanceVariableNames:'linearForm value'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearEquation class methodsFor:'documentation'!

documentation
"
    My instances represent linear equations, i.e., expresions of the form
    $$
        a_1 x_1 + \cdots + a_nx_n = b.
    $$
    where left side is a linear form and the right one is a (constant) value.

    For an example try this:

        LinearEquation linearForm: (3, 2, 5, -1, 0, 8) asLinearForm value: -1
"
! !

!LinearEquation class methodsFor:'instance creation'!

from: aCollection
    "Answer a Linear Equation from the Collection aColection."

    ^self
        linearForm: (LinearForm from: (aCollection copyFrom: 1 to: aCollection size - 1))
        value: aCollection last
        
!

linearForm: aLinearForm value: aNumber
    "Answer a new instance of the receiver consisting of aLinearFrom = aNumber."

    ^self new
            linearForm: aLinearForm;
            value: aNumber
! !

!LinearEquation methodsFor:'ST/V methods'!

asLinearSystem
    "Answer the LinearSystem consisting of the single equation represented by self."

    ^LinearSystem new add: self; yourself
!

asTuple
    "Public - Answer a Tuple with the coefficientes of the receiver."

    ^self linearForm asTuple, value
!

isSolution: aTuple
    "Answer true if aTuple is solution of the receiver."

    ^(self linearForm valueAt: aTuple) = self value
!

kernel
    "Answer the kernel of the reciever (a LinearSubspace)."

    ^self linearForm kernel
!

linearForm
    "Answer the linear form of the receiver (i.e., the left side)."

    ^linearForm
!

linearForm: aLinearForm
    "Change the linear form of the receiver (i.e., the left side)."

    linearForm := aLinearForm
!

particular
    "Answer a particular solution of the receiver."

    ^self asLinearSystem particular
!

printOn: aStream
    "Append the String representation of the receiver on aStream."

    self linearForm printOn: aStream.
    aStream
        nextPutAll: ' = ';
        nextPutAll: self value printString

    
!

value
    "Answer the right side (independent term) of the receiver."

    ^value

!

value: aNumber
    "Change the right side (independent term) of the receiver."

    value := aNumber

!

verifies: aTuple
    "Answer true if aTuple is solution of the receiver."

    ^self isSolution: aTuple
! !

!LinearEquation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearEquation.st,v 1.1 2002/11/22 19:44:25 cg Exp $'
! !
