"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Tuple subclass:#LinearForm
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearForm class methodsFor:'documentation'!

documentation
"
    My instances represent linear forms, i.e., expresions of the form
    $$
        a_1 x_1 + \cdots + a_nx_n.
    $$

    For an example try this:

        | x1 x2 x3 f |
        x1 := LinearForm x: 1 dim: 3.
        x2 := LinearForm x: 2 dim: 3.
        x3 := LinearForm x: 3 dim: 3.
        f := -1 * x1 + (7 * x2) + (4 * x3).
        f inspect.
        f valueAt: (-2, 7, 1)
"
! !

!LinearForm class methodsFor:'instance creation'!

x: indexInteger dim: dimInteger
    "Answer the canonical linear form of dimInteger coordinates with index indexInteger."

    ^self e: indexInteger dim: dimInteger
! !

!LinearForm class methodsFor:'defaults'!

indeterminate
    "Answer the String to be used as the indeterminate name."

    ^'x'
! !

!LinearForm methodsFor:'ST/V methods'!

asEquation
    "Public - Answer the equation consisting of the receiver equal 0."

    ^self eq: 0
!

asLinearForm
    "Answer a LinearForm with the coefficients of the receiver."

    ^self
!

asLinearTransformation
	"Answer the LinearTransformation defined by the receiver."

	^LinearTransformation new addForm: self; yourself
!

asTuple
    "Answer a Tuple from the coefficients of the receiver."

    ^Tuple from: self entries
!

domain
	"Answer the domain of the receiver."

	^LinearSubspace dim: self dim
!

dual
    "Answer the LinearForm whose coefficients are the coordinates of the receiver."

    ^Tuple from: self entries
!

eq: aNumber
    "Answer the Equation consisting of the receiver equal aNumber."

    ^LinearEquation
        linearForm: self
        value: aNumber
!

kernel
    "Answer the kernel of the receiver (a LinearSubspace)."

    | sys |
    sys := LinearSystem new.
    sys add: self asEquation.
    ^sys kernel
!

printOn: aStream
    "Append the String representation of the receiver on aStream."

    | first |
    first := true.
    self isNull ifTrue: [
        aStream nextPut: $0.
        ^self].
    1 to: self dim do: [:each |
        | entry |
        entry := self at: each.
        entry ~= 0
            ifTrue: [
                first
                    ifTrue: [
                        first := false.
                        entry = -1
                            ifTrue: [aStream nextPut: $-]
                            ifFalse: [    entry = 1 ifFalse: [aStream nextPutAll: entry printString]]]
                    ifFalse: [
                        aStream nextPutAll: (entry < 0
                            ifTrue: [' - ']
                            ifFalse: [' + ']).
                        entry abs = 1 ifFalse: [aStream nextPutAll: entry abs printString]].
                aStream
                    nextPutAll: self class indeterminate, '_';
                    nextPutAll: (each > 9
                        ifFalse: [each printString]
                        ifTrue: ['{', each printString, '}'])
        ]
    ]
!

valueAt: aTuple
    "Public - Answer the result of evaluating the receiver at aTuple."

    ^self scalar: aTuple
! !

!LinearForm class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearForm.st,v 1.1 2002/11/22 19:43:32 cg Exp $'
! !
