"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#Segment
	instanceVariableNames:'origin destiny'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!Segment class methodsFor:'documentation'!

documentation
"
        My instances represent line segments in an ambient space of
        any given (but arbitrary) dimension.

        Each instance is oriented in the sense that it has an origin and
        a destiny.

        For an example try this:

        | seg |
        seg := Segment origin: (3, 0, -1) destiny: (1, 1, 1).
        seg length.
        seg direction.
        seg middle.
        seg line
"
! !

!Segment class methodsFor:'instance creation'!

origin: aVector destiny: bVector
	"Public - Answer a new instance of the receiver going from aVector to bVector."

	^self new
		origin: aVector;
		destiny: bVector
! !

!Segment methodsFor:'ST/V methods'!

ambient
	"Answer the ambient space containing the receiver."

	self origin ambient
!

destiny
	"Answer the destiny of the receiver."

	^destiny
!

destiny: aVector
	"Change the destiny of the receiver."

	destiny := aVector
!

direction
	"Answer the direction of the receiver."

	^self destiny - self origin
!

includes: aTuple
	"Answer true if aTuple is on the receiver. Else answer false."

	| m |
	(self line includes: aTuple) ifFalse: [^false].
	^((m := self middle) distance2To: aTuple) <= (m distance2To: self origin)
!

length
	"Public - Answer the length of the receiver."

	^self length2 sqrt
!

length2
	"Public - Answer the squared length of the receiver."

	^self origin distance2To: self destiny
!

line
	"Answer the LinearVariety underlying the receiver."

	^LinearVariety subspace: self direction asSubspace translation: self origin
!

middle
	"Public - Answer the middle point of the receiver."

	^self origin + self destiny / 2
!

origin
	"Answer the origin of the receiver."

	^origin
!

origin: aVector
	"Change the origin of the receiver."

	origin := aVector
!

printOn: aStream
	"Append a String representation of the receiver on aStream."

	aStream nextPut: $[.
	self origin printOn: aStream.
	aStream nextPutAll: ' , '.
	self destiny printOn: aStream.
	aStream nextPut: $]
!

transformWith: aLinearTransformation
        "Answer the image of the receiver by aLinearTranformation."

        ^Segment
                origin: (aLinearTransformation valueAt: self origin)
                destiny: (aLinearTransformation valueAt: self destiny)
! !

!Segment class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/Segment.st,v 1.1 2002/11/22 19:43:59 cg Exp $'
! !
