"
    Copyright (c) 2001 David N. Smith.  

    See PiCalculator class>>license for license text.
    Ported to Dolphin Smalltalk by Bob Jarvis.
"

"{ Package: 'stx:goodies/math/misc' }"

"{ NameSpace: Math }"

Object subclass:#PiCalculator
	instanceVariableNames:'pi digits'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-Algorithms'
!

PiCalculator comment:'nil'
!

!PiCalculator class methodsFor:'documentation'!

copyright
"
    Copyright (c) 2001 David N. Smith.  

    See PiCalculator class>>license for license text.
    Ported to Dolphin Smalltalk by Bob Jarvis.
"
!

documentation
"
    I compute Pi to arbitrary precision using a brute force implementation of Machin's formula:

        pi/4 = 4 arctan(1/5) - arctan(1/239)

    As the number of digits desired increases, the computation speed decreases 
    since the implementation computes all digits at once. 
    Thus one might see me compute and print 200 digits in well under a second, 
    you might see 10000 digits take a very long time. 
    See below for some benchmarks. 
    The time to format the result into decimal may exceed the computational time.

    I answer a fraction holding the value, or a string holding a printable version of the value.

    I have been been tested to 10000 digits by comparing my output to supposedly correct values from 
    http://www.cecm.sfu.ca/projects/ISC/data/pi.html. 

    Try http://www.cecm.sfu.ca/projects for a higher level view of what the site is about.

    Try:

        PiCalculator new computeToDigits: 100; print  
        (PiCalculator new computeToDigits: 25) asFloat = Float pi
        PiCalculator new computeToDigits: 1000; print 
        PiCalculator new computeToDigits: 20; print  


    Benchmarks from a PowerBook G3 (February 2000 model) running Squeak 3.0 showing times in seconds:

        Digits   Compute     +Print
          100     0.018         0.027
          500     0.11          0.3
         1000     0.29          1.0
         2000     0.84          3.9
         3000     1.6           8.1
         5000     4.0          21.9
        10000    14.5          87.3

    Time millisecondsToRun: [ 
        10 timesRepeat: [PiCalculator new computeToDigits: 5000] 
    ]

    Time millisecondsToRun: [ 
        10 timesRepeat: [PiCalculator new computeToDigits: 5000; print] 
    ]
"
!

license
	"PLEASE READ THE FOLLOWING NOTICE AND DISCLAIMER CAREFULLY BEFORE DOWNLOADING
	THIS SOFTWARE.  BY DOWNLOADING THIS SOFTWARE YOU ARE AGREEING TO BE BOUND BY THESE
	TERMS.  IF YOU DO NOT AGREE TO THE TERMS, DO NOT DOWNLOAD.

	This software (Software), provided by IBM Corporation, may contain, or be derived from, code provided by
	Apple Computer, Inc., and is provided subject to the provisions of the Apple Computer, Inc. Software License
	for the SQUEAK Smalltalk System which can be viewed at [SITE ADDRESS].  This Software is provided AS IS,
	and IBM Corporation DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING, WITHOUT 
	LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
	AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.  IBM DOES NOT WARRANT THAT THE 
	FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET THE USERS' REQUIREMENTS, THAT
	THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT
	DEFECTS IN THE SOFTWARE WILL BE CORRECTED.  UNDER NO CIRCUMSTANCES SHALL IBM 
	CORPORATION BE LIABLE FOR INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES
	ARISING FROM OR RELATING TO USE OF THIS SOFTWARE.

	The user of this Software agrees to indemnify and hold IBM Corporation harmless from any and all damages, 
	liabilities, costs and expenses (including attorney's fees) incurred by IBM Corporation as a result of any claim, 
	proceeding or judgment to the extent it arises out of or is connected in any manner with the operation, use, 
	distribution or modification of the Software, or the combination of the Software or modified Software with
	other programs."
! !

!PiCalculator methodsFor:'private'!

arctan: denom scale: scale
	| denom2 k result top |

	denom2 := denom squared.
	k := 3.
	result := scale.
	result := result quo: denom.
	top := result.
	[	"Calculate subtractive term"
		top := top quo: denom2.
		result := result - (top quo: k).
		"Calculate additive term"
		top := top quo: denom2.
		result := result + (top quo: k+2).
		"When top = 0, all further terms are zero"
		top > 0 ]
	whileTrue: [
		k := k + 4 ].
	^result
! !

!PiCalculator methodsFor:'public'!

computeToDigits: maxDigits
	"Compute pi to maxDigits precision.
	 Uses Machin's formula: pi/4 = 4 arctan(1/5) - arctan(1/239)
	 Answer a fraction which represents the result."
	| tot scale |

	digits := maxDigits.
	scale := 10 raisedToInteger: maxDigits+3.
	"Add 3 to make sure we get maxDigits correct digits"

	tot := self arctan: 5 scale: scale.
	tot := tot * 4.
	tot := tot - (self arctan: 239 scale: scale).
	tot := tot * 4.
	^pi := tot / scale
!

print
	" Prints the last value of PI computed by this instance.
	  NOTE: Must be sent to the instance of PiCalculator, not the numeric result. "
	| value str |

	value := pi numerator * (10 raisedTo: digits) // pi denominator.
	str := value printString.
	^ str first asString, '.', (str copyFrom: 2)
! !

!PiCalculator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/misc/PiCalculator.st,v 1.3 2003/06/16 09:43:43 cg Exp $'
! !
