"{ Package: 'stx:goodies/math/stmath' }"

StMIntegratorInterface subclass:#StMDAEIntegrator
	instanceVariableNames:'neqn ncon'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Integrator'
!


!StMDAEIntegrator methodsFor:'accessing'!

iterMax
	^system iterMax
!

neqn
	^neqn
!

t: aDouble 
	^system t: aDouble
!

time: aDouble 
	^system time: aDouble
!

tnew: aDouble 
	self halt: 'Check if used.'.
	^system tnew: aDouble
!

y
	^system y
!

y: aColumn 
	^system y: aColumn
!

ydot
	^system ydot
!

ydot: aColumn 
	^system ydot: aColumn
! !

!StMDAEIntegrator methodsFor:'initialize-release'!

initialize
	super initialize.
	integrator := StMStartingBasicDAEIntegrator new.
	integrator system: self
!

initializeGlobally
	super initializeGlobally.
	neqn := system neqn
! !

!StMDAEIntegrator methodsFor:'run'!

checkForDiscontinuity
	system checkForDiscontinuity
!

checkForOutputThrough: t 
	"Inclusive of t."

	| tlimit thereIsOutput yout ydotout yddotout |
	tlimit := t.
	thereIsOutput := false.
	tend <= tlimit
		ifTrue: 
			[integrator continue: false.
			tlimit := tend + (0.1d * hout)].
	[tout <= tlimit]
		whileTrue: 
			[thereIsOutput := true.
			yout := integrator yDeriv: 0 at: tout.
			ydotout := integrator yDeriv: 1 at: tout.
			yddotout := integrator yDeriv: 2 at: tout.
			system time: tout.
			system y: yout.
			system ydot: ydotout.
			system yddot: yddotout.
			self preDAEOutput.
			system output.
			tout := tout + hout].
	thereIsOutput
		ifTrue: 
			["Reset system to integrator time."
			system time: integrator t.
			system y: integrator y.
			system ydot: integrator ydot.
			self postDAEOutput]
!

fillF: col 
	^system fillF: col
!

fillY: aColumn 
	^system fillY: aColumn
!

fillYdot: aColumn 
	^system fillYdot: aColumn
!

fillpFpy: matrix 
	^system fillpFpy: matrix
!

fillpFpydot: mat 
	^system fillpFpydot: mat
!

postDAECorrector
	system postDAECorrector
!

postDAECorrectorIteration
	system postDAECorrectorIteration
!

postDAEFirstStep
	system postDAEFirstStep
!

postDAEOutput
	system postDAEOutput
!

postDAEPredictor
	system postDAEPredictor
!

postDAEStep
	system postDAEStep.
	integrator istep > 0 ifTrue: ["Noise make checking at the start unreliable."
		self checkForDiscontinuity].
	self checkForOutputThrough: integrator t
!

postRun
	system postDAEIntegrator
!

preDAECorrector
	system preDAECorrector
!

preDAECorrectorIteration
	system preDAECorrectorIteration
!

preDAEFirstStep
	system preDAEFirstStep
!

preDAEOutput
	system preDAEOutput
!

preDAEPredictor
	system preDAEPredictor
!

preDAEStep
	system preDAEStep
!

preRun
	system preDAEIntegrator
!

reportDAEStepStats: stats 
	system reportDAEStepStats: stats
!

reportStats
	system reportDAEStats: statistics
!

reportTrialStepStats: stats 
	system reportDAETrialStepStats: stats
!

suggestSmallerOrAcceptFirstStepSize: hnew 
	| hnew2 |
	hnew2 := system suggestSmallerOrAcceptFirstStepSize: hnew.
	hnew2 > hmax
		ifTrue: 
			[hnew2 := hmax.
			self logString: #'StM: \Step size is at user specified maximum.'].
	hnew2 < hmin
		ifTrue: 
			[self logString: #'StM: \Step size %1 < %2 user specified minimum.' withArguments: (Array with: hnew2 asFloat printString with: hmin asFloat printString).
			StMTooSmallStepSizeError raiseSignal].
	^hnew2
!

suggestSmallerOrAcceptStepSize: hnew 
	| hnew2 |
	hnew2 := system suggestSmallerOrAcceptStepSize: hnew.
	hnew2 > hmax
		ifTrue: 
			[hnew2 := hmax.
			self logString: #'StM: \Step size is at user specified maximum.'].
	hnew2 < hmin
		ifTrue: 
			[self logString: #'StM: \Step size %1 < %2 user specified minimum.' withArguments: (Array with: hnew2 asFloat printString with: hmin asFloat printString).
			StMTooSmallStepSizeError raiseSignal].
	^hnew2
!

updateForDAECorrector
	system updateForDAECorrector
! !

!StMDAEIntegrator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMDAEIntegrator.st,v 1.6 2006/10/04 13:00:52 cg Exp $'
! !
