"{ Package: 'stx:goodies/math/stmath' }"

StMSolver subclass:#StMDifferentialAlgebraicEquationSolver
	instanceVariableNames:'n t y ydot aF pFpy pFpydot tstart tend hout hmin hmax orderMax
		iterMax relativeTol absoluteTol touts youts ydotouts'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Integrator'
!


!StMDifferentialAlgebraicEquationSolver class methodsFor:'instance creation'!

new: size 
	^super new initialize: size
! !

!StMDifferentialAlgebraicEquationSolver methodsFor:'accessing'!

aFAt: i put: eqn 
	aF at: i put: eqn
!

absoluteTol: double 
	absoluteTol := double
!

endTime
	^tend
!

firstOutputTime
	^tstart
!

hmax: double 
	hmax := double
!

hmin: double 
	hmin := double
!

hout: double 
	hout := double
!

iterMax
	^iterMax
!

iterMax: integer 
	iterMax := integer
!

maxStepSize
	^hmax
!

minStepSize
	^hmin
!

n
	^n
!

neqn
	^n
!

orderMax
	^orderMax
!

orderMax: integer 
	orderMax := integer
!

outputStepSize
	^hout
!

relativeTol: double 
	relativeTol := double
!

startTime
	^tstart
!

t: var 
	t := var
!

tend: double 
	tend := double
!

tstart: double 
	tstart := double
!

yAt: i put: var 
	y at: i put: var
!

ydotAt: i put: var 
	ydot at: i put: var
! !

!StMDifferentialAlgebraicEquationSolver methodsFor:'initialize-release'!

initialize: size 
	n := size.
	y := StMFullColumn new: size.
	ydot := StMFullColumn new: size.
	aF := StMFullColumn new: size.
	pFpy := StMSparseMatrix new: size by: size.
	pFpydot := StMSparseMatrix new: size by: size.
	touts := OrderedCollection new.
	youts := OrderedCollection new.
	ydotouts := OrderedCollection new
! !

!StMDifferentialAlgebraicEquationSolver methodsFor:'output'!

logString: string 
	Transcript show: string mbdMessage
!

logString: string withArgument: string1 
	Transcript show: (string mbdMessage expandWithArgument: string1)
!

logString: string withArguments: arrayOfStrings 
	Transcript show: (string mbdMessage expandWithArguments: arrayOfStrings)
!

outputOnFile: aString 
	| writeStream block numberString |
	writeStream := aString asFilename writeStream.
	block := 
			[:var :string | 
			| varname |
			varname := var name.
			varname := varname isNil
						ifTrue: [string]
						ifFalse: [varname asString].
			writeStream nextPutAll: varname; nextPut: $,].
	block value: t value: 't'.
	1 to: n do: [:j | block value: (y at: j)
			value: 'y' , j printString].
	1 to: n do: [:j | block value: (ydot at: j)
			value: 'ydot' , j printString].
	writeStream cr.
	block := 
			[:number | 
			numberString := number asDouble printString.
			numberString last == $d
				ifTrue: [numberString replaceAll: $d with: Character space]
				ifFalse: [numberString replaceAll: $d with: $e].
			writeStream nextPutAll: numberString.
			writeStream nextPut: $,].
	1 to: touts size
		do: 
			[:i | 
			| youti ydotouti |
			block value: (touts at: i).
			youti := youts at: i.
			ydotouti := ydotouts at: i.
			1 to: n do: [:j | block value: (youti at: j)].
			1 to: n do: [:j | block value: (ydotouti at: j)].
			writeStream cr].
	writeStream flush; close
! !

!StMDifferentialAlgebraicEquationSolver methodsFor:'solving'!

checkForDiscontinuity
	"Do nothing."
!

correctorAbsoluteTolerance
	^self toleranceVectorFrom: absoluteTol
!

correctorRelativeTolerance
	^self toleranceVectorFrom: relativeTol
!

fillF: col 
	1 to: n do: [:i | col at: i put: (aF at: i) value asDouble]
!

fillY: col 
	1 to: n do: [:i | col at: i put: (y at: i) value asDouble]
!

fillYdot: col 
	1 to: n do: [:i | col at: i put: (ydot at: i) value asDouble]
!

fillpFpy: mat 
	| rowi pFipy |
	1 to: n
		do: 
			[:i | 
			rowi := mat at: i.
			pFipy := pFpy at: i.
			pFipy keysAndValuesDo: [:j :pFipyj | rowi at: j put: pFipyj value asDouble]]
!

fillpFpydot: mat 
	| rowi pFipydot |
	1 to: n
		do: 
			[:i | 
			rowi := mat at: i.
			pFipydot := pFpydot at: i.
			pFipydot keysAndValuesDo: [:j :pFipydotj | rowi at: j put: pFipydotj value asDouble]]
!

integrationAbsoluteTolerance
	"Answer integrationAbsoluteTolerance vector with nil for algebraic equations."

	^self integrationToleranceVectorFrom: absoluteTol
!

integrationRelativeTolerance
	"Answer integrationRelativeTolerance vector with nil for algebraic equations."

	^self integrationToleranceVectorFrom: relativeTol
!

integrationToleranceVectorFrom: tol 
	"Answer integrationTolerance vector with nil for algebraic equations."

	| answer |
	answer := self toleranceVectorFrom: tol.
	1 to: n do: [:i | (pFpydot at: i) isZero ifTrue: [answer at: i put: nil]].
	^answer
!

output
	"Do nothing."

	touts add: t value.
	youts add: (y collect: [:yi | yi value]).
	ydotouts add: (ydot collect: [:ydoti | ydoti value])
!

postDAECorrector
	"Do nothing."
!

postDAEFirstStep
	"Do nothing."
!

postDAEIntegrator
	"Do nothing."
!

postDAEOutput
	"Do nothing."
!

postDAEPredictor
	"Do nothing."
!

postDAEStep
	"Do nothing."
!

preDAECorrector
	"Do nothing."
!

preDAEFirstStep
	"Do nothing."
!

preDAEIntegrator
        (y detect: [:yi | yi value isLimitedPrecisionReal not]
                ifNone: [false])
                ifTrue: [self error: 'Variables must have initial values.'].
        (aF detect: [:aFi | aFi value isLimitedPrecisionReal not]
                ifNone: [false])
                ifTrue: [self error: 'Equations must be valid.'].
        1 to: n
                do: 
                        [:i | 
                        | pFipy pFipydot aFi |
                        pFipy := pFpy at: i.
                        pFipydot := pFpydot at: i.
                        aFi := aF at: i.
                        1 to: n
                                do: 
                                        [:j | 
                                        | pFipyj pFipydotj |
                                        pFipyj := aFi partialDifferentiateWRT: (y at: j).
                                        pFipyj isZero ifFalse: [pFipy at: j put: pFipyj].
                                        pFipydotj := aFi partialDifferentiateWRT: (ydot at: j).
                                        pFipydotj isZero ifFalse: [pFipydot at: j put: pFipydotj]]]
!

preDAEOutput
	"Do nothing."
!

preDAEPredictor
	"Do nothing."
!

preDAEStep
	"Do nothing."
!

reportDAEStats: stats 
	"Do nothing."
!

reportDAEStepStats: stats 
	"Do nothing."
!

reportDAETrialStepStats: stats 
        | array |
        array := Array new: 6.
        array at: 1 put: (stats at: #istep) printString.
        array at: 2 put: (stats at: #t) asFloat printString.
        array at: 3 put: (stats at: #order) printString.
        array at: 4 put: (stats at: #h) asFloat printString.
        array at: 5 put: (stats at: #corIterNo ifAbsent:nil) printString.
        array at: 6 put: (stats at: #truncError) asFloat printString.
        self logString: #'ADM: \%1 , t=%2 , o=%3 , h=%4 , it=%5 , e=%6' withArguments: array
!

run
	| daeIntegrator |
	daeIntegrator := StMDAEIntegrator new.
	daeIntegrator system: self.
	daeIntegrator run
!

suggestSmallerOrAcceptFirstStepSize: hnew 
	"Default is return hnew."
	"Best to do nothing so as not to disrupt the starting algorithm."

	^hnew
!

suggestSmallerOrAcceptStepSize: hnew 
	"Default is return hnew."
	"Best to do nothing so as not to disrupt the algorithm."

	^hnew
!

time: aDouble 
	t x: aDouble
!

toleranceVectorFrom: tol 
        "Answer tolerance vector."
        "tol may be number or array."
        "nil elements are not used in tolerance calculations."

        | answer toli |
        answer := StMFullColumn new: n.
        tol isLimitedPrecisionReal
                ifTrue: [answer atAllPut: tol asDouble]
                ifFalse: [tol isSequenceable
                                ifTrue: [tol size == n
                                                ifTrue: [1 to: n
                                                                do: 
                                                                        [:i | 
                                                                        toli := tol at: i.
                                                                        toli isLimitedPrecisionReal ifTrue: [answer at: i put: toli asDouble]]]
                                                ifFalse: [self error: 'Tolerance vector must have equal dimension as equation set.']]
                                ifFalse: [self error: 'Tolerance value is not properly defined.']].
        ^answer
!

updateForDAECorrector
	"Do nothing."
!

y: col 
	1 to: n do: [:i | (y at: i)
			value: (col at: i)]
!

yddot: col 
	"Do nothing."
!

ydot: col 
	1 to: n do: [:i | (ydot at: i)
			value: (col at: i)]
! !

!StMDifferentialAlgebraicEquationSolver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMDifferentialAlgebraicEquationSolver.st,v 1.8 2006/10/04 13:00:52 cg Exp $'
! !
