"{ Package: 'stx:goodies/math/stmath' }"

StMFullVector variableSubclass:#StMFullColumn
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Array'
!


!StMFullColumn methodsFor:'accessing'!

ncol
	^1
!

nrow
        "Answer the number of indexable fields in the receiver. This value is the 
        same as the largest legal subscript. Primitive is specified here to override 
        SequenceableCollection size."

"/        <primitive: 62>
        ^self basicSize
! !

!StMFullColumn methodsFor:'basic ops'!

minusFullColumn: aFullColumn 
	^super minusFullVector: aFullColumn
!

minusFullColumn: aFullColumn times: aNumber 
	^super minusFullVector: aFullColumn times: aNumber
!

plusFullColumn: aFullColumn 
	^super plusFullVector: aFullColumn
!

plusFullColumn: aFullColumn times: aNumber 
	^super plusFullVector: aFullColumn times: aNumber
!

times: beta minusFullColumn: aFullColumn 
	^super times: beta minusFullVector: aFullColumn
!

times: beta minusFullColumn: aFullColumn times: alpha 
	^super
		times: beta
		minusFullVector: aFullColumn
		times: alpha
!

times: alpha plusFullColumn: aFullColumn 
	^super times: alpha plusFullVector: aFullColumn
!

times: beta plusFullColumn: aFullColumn times: alpha 
	^super
		times: beta
		plusFullVector: aFullColumn
		times: alpha
!

timesFullColumn: aFullColumn 
	self halt: 'Column times: Column is not a valid operation.'
!

timesFullMatrix: aFullMatrix 
	"a*b = a(i)b(1,j)"

	^aFullMatrix nrow == 1
		ifTrue: [self timesFullRow: (aFullMatrix at: 1)]
		ifFalse: [self error: 'StM: Incompatible row size.']
!

timesFullRow: aFullRow 
	"a*b = a(i)b(j)"

	| nrow answer |
	nrow := self nrow.
	answer := StMFullMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (aFullRow timesNumber: (self at: i))].
	^answer
!

timesFullRow: aFullRow times: double 
	"a*b*scalar = a(i)b(j)*scalar"

	| nrow answer |
	nrow := self nrow.
	answer := StMFullMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (aFullRow timesNumber: double * (self at: i))].
	^answer
!

timesSparseMatrix: aSparseMatrix 
	"a*b = a(i)b(1,j)"

	^aSparseMatrix nrow == 1
		ifTrue: [self timesSparseRow: (aSparseMatrix at: 1)]
		ifFalse: [self error: 'StM: Incompatible row size.']
!

timesSparseRow: aSparseRow 
	"a*b = a(i)b(j)"

	| nrow answer |
	nrow := self nrow.
	answer := StMSparseMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (aSparseRow timesNumber: (self at: i))].
	^answer
!

timesSparseRow: aSparseRow times: double 
	"a*b*scalar = a(i)b(j)*scalar"

	| nrow answer |
	nrow := self nrow.
	answer := StMSparseMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (aSparseRow timesNumber: double * (self at: i))].
	^answer
!

timesTransposeFullColumn: aFullColumn 
	"a*bT = a(i)b(j)"

	^self timesFullRow: aFullColumn transpose
!

timesTransposeFullColumn: aFullColumn times: double 
	"a*bT*num = a(i)b(j)*num"

	^self timesFullRow: aFullColumn transpose times: double
!

timesTransposeFullMatrix: aFullMatrix 
	"a*bT = a(i)b(j,1)"

	^self timesTransposeFullColumn: aFullMatrix asFullColumn
!

timesTransposeSparseColumn: aSparseColumn 
	"a*bT = a(i)b(j)"

	^self timesSparseRow: aSparseColumn transpose
!

timesTransposeSparseMatrix: aSparseMatrix 
	"a*bT = a(i)b(j,1)"

	^self timesTransposeSparseColumn: aSparseMatrix asSparseColumn
!

transpose
	"Return a row with the same elements."
	"This method is much faster than the obvious way."

	^self shallowCopy changeClassToThatOf: (StMFullRow new: 0)
!

transposeTimesFullColumn: aFullColumn 
	"colT*col = row*col."

	^self dot: aFullColumn
!

transposeTimesFullMatrix: aFullMatrix 
	^self transpose timesFullMatrix: aFullMatrix
!

transposeTimesSparseMatrix: aSparseMatrix 
	^self transpose timesSparseMatrix: aSparseMatrix
! !

!StMFullColumn methodsFor:'converting'!

asFullColumn
	"Do nothing."
!

asFullMatrix
	| nrow answer |
	nrow := self nrow.
	answer := StMFullMatrix new: nrow by: 1.
	1 to: nrow do: [:i | (answer at: i)
			at: 1 put: (self at: i)].
	^answer
!

asZeroColumn
	^StMZeroColumn new: self nrow
!

value: x 
	"self is a column of block formulas."
	"Return evaluation at x."

	| nrow answer |
	nrow := self nrow.
	answer := StMFullColumn new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				value: x)].
	^answer
! !

!StMFullColumn methodsFor:'ops on self'!

at: i minusFullColumn: aFullColumn 
	super at: i minusFullVector: aFullColumn
!

at: i minusFullColumn: aFullColumn times: aNumber 
	super
		at: i
		minusFullVector: aFullColumn
		times: aNumber
!

at: i plusFullColumn: aFullColumn 
	super at: i plusFullVector: aFullColumn
!

at: i plusFullColumn: aFullColumn times: aNumber 
	super
		at: i
		plusFullVector: aFullColumn
		times: aNumber
!

at: i1 putFullColumn: aFullColumn 
	super at: i1 putFullVector: aFullColumn
!

at: i1 putFullColumn: aFullColumn times: aNumber 
	super
		at: i1
		putFullVector: aFullColumn
		times: aNumber
!

equalFullColumn: aFullColumn 
	super equalFullVector: aFullColumn
!

equalFullColumn: aFullColumn at: ii1 
	super equalFullVector: aFullColumn at: ii1
!

equalFullColumn: aFullColumn times: aNumber 
	super equalFullVector: aFullColumn times: aNumber
!

equalFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self size do: [:i | self at: i put: (aFullMatrix at: iiOffset + i and: jj1)]
!

equalSelfMinusFullColumn: aFullColumn 
	super equalSelfMinusFullVector: aFullColumn
!

equalSelfMinusFullColumn: aFullColumn at: ii1 
	super equalSelfMinusFullVector: aFullColumn at: ii1
!

equalSelfMinusFullColumn: aFullColumn times: aNumber 
	super equalSelfMinusFullVector: aFullColumn times: aNumber
!

equalSelfMinusFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self size do: [:i | self at: i put: (self at: i)
				- (aFullMatrix at: iiOffset + i and: jj1)]
!

equalSelfMinusSparseColumn: aSparseColumn 
	super equalSelfMinusSparseVector: aSparseColumn
!

equalSelfMinusSparseColumn: aSparseColumn times: aNumber 
	super equalSelfMinusSparseVector: aSparseColumn times: aNumber
!

equalSelfPlusFullColumn: aFullColumn 
	super equalSelfPlusFullVector: aFullColumn
!

equalSelfPlusFullColumn: aFullColumn at: ii1 
	super equalSelfPlusFullVector: aFullColumn at: ii1
!

equalSelfPlusFullColumn: aFullColumn times: aNumber 
	super equalSelfPlusFullVector: aFullColumn times: aNumber
!

equalSelfPlusFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self size do: [:i | self at: i put: (self at: i)
				+ (aFullMatrix at: iiOffset + i and: jj1)]
!

equalSelfPlusSparseColumn: aSparseColumn 
	super equalSelfPlusSparseVector: aSparseColumn
!

equalSelfPlusSparseColumn: aSparseColumn times: aNumber 
	super equalSelfPlusSparseVector: aSparseColumn times: aNumber
!

equalSelfTimes: alpha minusFullColumn: aFullColumn times: beta 
	super
		equalSelfTimes: alpha
		minusFullVector: aFullColumn
		times: beta
!

equalSelfTimes: alpha plusFullColumn: aFullColumn times: beta 
	super
		equalSelfTimes: alpha
		plusFullVector: aFullColumn
		times: beta
! !

!StMFullColumn class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMFullColumn.st,v 1.7 2006/10/04 13:00:52 cg Exp $'
! !
