"{ Package: 'stx:goodies/math/stmath' }"

StMFullVector variableSubclass:#StMFullRow
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Array'
!


!StMFullRow methodsFor:'accessing'!

ncol
        "Answer the number of indexable fields in the receiver. This value is the 
        same as the largest legal subscript. Primitive is specified here to override 
        SequenceableCollection size."

"/        <primitive: 62>
        ^self basicSize
!

nrow
	^1
! !

!StMFullRow methodsFor:'basic ops'!

minusFullRow: aFullRow 
	^super minusFullVector: aFullRow
!

minusFullRow: aFullRow times: aNumber 
	^super minusFullVector: aFullRow times: aNumber
!

plusFullRow: aFullRow 
	^super plusFullVector: aFullRow
!

plusFullRow: aFullRow times: aNumber 
	^super plusFullVector: aFullRow times: aNumber
!

times: alpha minusFullRow: aFullRow 
	^super times: alpha minusFullVector: aFullRow
!

times: beta minusFullRow: aFullRow times: alpha 
	^super
		times: beta
		minusFullVector: aFullRow
		times: alpha
!

times: alpha plusFullRow: aFullRow 
	^super times: alpha plusFullVector: aFullRow
!

times: beta plusFullRow: aFullRow times: alpha 
	^super
		times: beta
		plusFullVector: aFullRow
		times: alpha
!

timesFullColumn: aFullColumn 
	| answer |
	answer := (self at: 1)
				* (aFullColumn at: 1).
	2 to: self ncol do: [:j | answer := answer + ((self at: j)
						* (aFullColumn at: j))].
	^answer
!

timesFullMatrix: aFullMatrix 
	"Similar to inner dot product."

	| answer |
	answer := (aFullMatrix at: 1)
				timesNumber: (self at: 1).
	2 to: self ncol do: [:j | answer equalSelfPlusFullRow: (aFullMatrix at: j)
			times: (self at: j)].
	^answer
!

timesFullRow: aFullRow 
	self halt: 'Row times: Row is not a valid operation.'
!

timesSparseMatrix: aSparseMatrix 
	| answer |
	answer := self class new: aSparseMatrix ncol.
	answer zeroSelf.
	1 to: self ncol do: [:j | answer equalSelfPlusSparseRow: (aSparseMatrix at: j)
			times: (self at: j)].
	^answer
!

timesTransposeFullMatrix: aFullMatrix 
	"a*bT = a(1,j)b(k,j)"

	| ncol answer |
	ncol := aFullMatrix nrow.
	answer := StMFullRow new: ncol.
	1 to: ncol do: [:k | answer at: k put: (self dot: (aFullMatrix at: k))].
	^answer
!

timesTransposeSparseMatrix: aSparseMatrix 
	"a*bT = a(1,j)b(k,j)"

	| ncol answer |
	ncol := aSparseMatrix nrow.
	answer := StMFullRow new: ncol.
	1 to: ncol do: [:k | answer at: k put: ((aSparseMatrix at: k)
				dot: self)].
	^answer
!

transpose
	"Return a row with the same elements."
	"This method is much faster than the obvious way."

	^self shallowCopy changeClassToThatOf: (StMFullColumn new: 0)
! !

!StMFullRow methodsFor:'converting'!

asFullColumn
	^self shallowCopy changeClassToThatOf: (StMFullColumn new: 0)
!

asSparseRow
	| ncol answer |
	ncol := self ncol.
	answer := StMSparseRow new: ncol.
	1 to: ncol
		do: 
			[:i | 
			| ai |
			ai := self at: i.
			ai isZero ifFalse: [answer at: i put: ai]].
	^answer
! !

!StMFullRow methodsFor:'ops on self'!

at: i minusFullRow: aFullRow 
	super at: i minusFullVector: aFullRow
!

at: i minusFullRow: aFullRow times: aNumber 
	super
		at: i
		minusFullVector: aFullRow
		times: aNumber
!

at: i plusFullRow: aFullRow 
	super at: i plusFullVector: aFullRow
!

at: i plusFullRow: aFullRow times: aNumber 
	super
		at: i
		plusFullVector: aFullRow
		times: aNumber
!

at: j1 putFullRow: aFullRow 
	super at: j1 putFullVector: aFullRow
!

at: j1 putFullRow: aFullRow times: aNumber 
	super
		at: j1
		putFullVector: aFullRow
		times: aNumber
!

equalFullRow: aFullRow 
	super equalFullVector: aFullRow
!

equalFullRow: aFullRow at: j1 
	super equalFullVector: aFullRow at: j1
!

equalFullRow: aFullRow times: aNumber 
	super equalFullVector: aFullRow times: aNumber
!

equalSelfMinusFullRow: aFullRow 
	super equalSelfMinusFullVector: aFullRow
!

equalSelfMinusFullRow: aFullRow at: jj1 
	super equalSelfMinusFullVector: aFullRow at: jj1
!

equalSelfMinusFullRow: aFullRow times: aNumber 
	super equalSelfMinusFullVector: aFullRow times: aNumber
!

equalSelfMinusSparseRow: aSparseRow 
	super equalSelfMinusSparseVector: aSparseRow
!

equalSelfMinusSparseRow: aSparseRow times: aNumber 
	super equalSelfMinusSparseVector: aSparseRow times: aNumber
!

equalSelfPlusFullRow: aFullRow 
	super equalSelfPlusFullVector: aFullRow
!

equalSelfPlusFullRow: aFullRow at: jj1 
	super equalSelfPlusFullVector: aFullRow at: jj1
!

equalSelfPlusFullRow: aFullRow times: aNumber 
	super equalSelfPlusFullVector: aFullRow times: aNumber
!

equalSelfPlusSparseRow: aSparseRow 
	super equalSelfPlusSparseVector: aSparseRow
!

equalSelfPlusSparseRow: aSparseRow times: aNumber 
	super equalSelfPlusSparseVector: aSparseRow times: aNumber
!

equalSelfTimes: alpha minusFullRow: aFullRow times: beta 
	super
		equalSelfTimes: alpha
		minusFullVector: aFullRow
		times: beta
!

equalSelfTimes: alpha plusFullRow: aFullRow times: beta 
	super
		equalSelfTimes: alpha
		plusFullVector: aFullRow
		times: beta
! !

!StMFullRow class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMFullRow.st,v 1.7 2006/10/04 13:00:52 cg Exp $'
! !
