"{ Package: 'stx:goodies/math/stmath' }"

Object subclass:#StMFunction
	instanceVariableNames:'dependents value'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!


!StMFunction class methodsFor:'instance creation'!

new
	^super new initialize
! !

!StMFunction methodsFor:'accessing'!

addDependent: aFunction 
	dependents add: aFunction
!

conditionArgument: arg 
        | answer |
        answer := arg isLimitedPrecisionReal
                                ifTrue: [StMConstant with: arg]
                                ifFalse: [arg].
        answer addDependent: self.
        ^answer
!

removeDependency
	self subclassResponsibility
!

removeDependent: aFunction 
	dependents remove: aFunction ifAbsent: [nil]
!

value
	value == nil ifTrue: [self setValue].
	^value
! !

!StMFunction methodsFor:'changing'!

changed
	value := nil.
	dependents do: [:dependent | dependent changed]
!

substitute: var1 with: var2 
	self subclassResponsibility
! !

!StMFunction methodsFor:'copying'!

postCopy
	dependents := IdentitySet new.
	value := nil
! !

!StMFunction methodsFor:'evaluating'!

differentiateWRT: variable 
	self subclassResponsibility
!

partialDifferentiateWRT: variable 
	self subclassResponsibility
!

setValue
	self subclassResponsibility
!

simplified
	self subclassResponsibility
! !

!StMFunction methodsFor:'initialize-release'!

initialize
	dependents := IdentitySet new
! !

!StMFunction methodsFor:'message handling'!

argsPerform: aSymbol 
	self subclassResponsibility
!

argsPerform: selector withArguments: anArray 
	self subclassResponsibility
!

perform: aSymbol 
	(self respondsTo: aSymbol)
		ifTrue: [^super perform: aSymbol]
		ifFalse: [^self argsPerform: aSymbol]
!

perform: aSymbol with: anObject 
	(self respondsTo: aSymbol)
		ifTrue: [^super perform: aSymbol withArguments: (Array with: anObject)]
		ifFalse: [^self argsPerform: aSymbol withArguments: (Array with: anObject)]
!

perform: aSymbol with: firstObject with: secondObject 
	(self respondsTo: aSymbol)
		ifTrue: [^super perform: aSymbol withArguments: (Array with: firstObject with: secondObject)]
		ifFalse: [^self argsPerform: aSymbol withArguments: (Array with: firstObject with: secondObject)]
!

perform: aSymbol with: firstObject with: secondObject with: thirdObject 
	(self respondsTo: aSymbol)
		ifTrue: [^super perform: aSymbol withArguments: (Array
					with: firstObject
					with: secondObject
					with: thirdObject)]
		ifFalse: [^self argsPerform: aSymbol withArguments: (Array
					with: firstObject
					with: secondObject
					with: thirdObject)]
!

perform: selector withArguments: anArray 
	(self respondsTo: selector)
		ifTrue: [^super perform: selector withArguments: anArray]
		ifFalse: [^self argsPerform: selector withArguments: anArray]
! !

!StMFunction methodsFor:'ops'!

* func 
	^StMProduct with: self times: func
!

+ func 
	^StMSum with: self plus: func
!

- func 
	^StMSum with: self plus: func negated
!

/ func 
	^StMProduct with: self times: func reciprocal
!

abs
	^StMAbs with: self
!

arcTan
	^StMArcTan with: self
!

cos
	^StMCos with: self
!

exp
	^StMExp with: self
!

lg
	^StMLg with: self
!

ln
	^StMLn with: self
!

minus: func 
	^StMSum with: self plus: func negated
!

negated
	^StMNegative with: self
!

plus: func 
	^StMSum with: self plus: func
!

raisedTo: exponent 
	^StMPower with: self raisedTo: exponent
!

reciprocal
	^StMPower with: self raisedTo: -1
!

sin
	^StMSin with: self
!

sqrt
	^StMSqrt with: self
!

tan
	^StMTan with: self
!

times: func 
	^StMProduct with: self times: func
! !

!StMFunction methodsFor:'printing'!

outputOn: filename interval: interval 
	| writeStream |
	Transcript cr; show: 'Tabulated data is in ' , filename.
	writeStream := filename asFilename writeStream.
	writeStream nextPut: $x; nextPut: $,; nextPutAll: 'func'; cr.
	interval
		do: 
			[:x | 
			| fvalue |
			writeStream print: x asFloat; nextPut: $,.
			self x: x.
			fvalue := self value.
			fvalue isSequenceable
				ifTrue: [fvalue do: [:fvaluei | writeStream print: fvaluei asFloat; nextPut: $,]]
				ifFalse: [writeStream print: fvalue asFloat].
			writeStream cr].
	writeStream flush; close
! !

!StMFunction methodsFor:'testing'!

isConstant
	^false
!

isNegative
	^false
!

isOne
	^false
!

isZero
	^false
! !

!StMFunction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMFunction.st,v 1.5 2006/10/04 13:00:53 cg Exp $'
! !
