"{ Package: 'stx:goodies/math/stmath' }"

StMFunctionFromData subclass:#StMGeneralSpline
	instanceVariableNames:'derivs degree index delta'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!


!StMGeneralSpline class methodsFor:'instance creation'!

with: var order: order xs: xxs ys: yys 
	^super new
		x: var
		order: order
		xs: xxs
		ys: yys
! !

!StMGeneralSpline methodsFor:'accessing'!

degree
	^degree
!

degree: integer 
	degree := integer.
	(degree even and: [self isCyclic])
		ifTrue: [self error: 'KOH001: Even degree cyclic spline is not possible.']
!

derivs
	^derivs
!

xs
	^xs
!

xs: array 
	array isIncreasing ifFalse: [self error: #'KOH: x series must be in increasing order.'].
	xs := array
!

ys
	^ys
!

ys: array 
	ys := array.
	(degree notNil and: [degree even and: [self isCyclic]])
		ifTrue: [self error: 'KOH001: Even degree cyclic spline is not possible.']
! !

!StMGeneralSpline methodsFor:'calculation'!

calcCyclicIndexAndDelta
	| xFirst xLast |
	xFirst := xs first.
	xLast := xs last.
	xvalue := xvalue - xFirst \\ (xLast - xFirst) + xFirst.
	self calcIndexAndDelta
!

calcIndexAndDelta
	(index < xs size and: [(xs at: index)
			< xvalue & (xvalue < (xs at: index + 1))])
		ifFalse: [self searchIndexFrom: 1 to: xs size].
	delta := xvalue - (xs at: index)
!

calcIndexAndDeltaFor: xx 
	xvalue := xx.
	self isCyclic
		ifTrue: [self calcCyclicIndexAndDelta]
		ifFalse: [self calcNonCyclicIndexAndDelta]
!

calcNonCyclicIndexAndDelta
	| xFirst xLast |
	xFirst := xs first.
	xLast := xs last.
	xvalue < xFirst
		ifTrue: 
			[index := 1.
			delta := xvalue - xFirst]
		ifFalse: [xvalue > xLast
				ifTrue: 
					[index := xs size.
					delta := xvalue - xLast]
				ifFalse: [self calcIndexAndDelta]]
!

computeDerivatives
	"See derivation in 9spline.doc."

	| n p np matrix bvector y1 zero offset h dum jj ii derivsVector |
	n := xs size.
	p := degree.
	np := n * p.
	matrix := StMSparseMatrix new: np by: np.
	bvector := (StMFullColumn new: np).
	y1 := ys at: 1.
	(y1 isKindOf: Array) 
		ifTrue: [zero := (y1 class new: y1 size) atAllPut: Double zero] 
		ifFalse: [zero := y1 class zero].
	bvector atAllPut: zero.
	1 to: n - 1 do: [:i | 
			offset := i - 1 * p.
			h := (xs at: i + 1) - (xs at: i).
			2 to: p do: [:j | 
					matrix at: offset + j and: offset + j - 1 put: 1.0d.
					matrix at: offset + j and: offset + j - 1 + p put: -1.0d].
			dum := 1.0d.
			1 to: p do: [:j | 
					dum := dum * h / j.
					j to: p do: [:k | matrix at: offset + k - j + 1 and: offset + k put: dum]].
			bvector at: offset + 1 put: (ys at: i + 1) - (ys at: i)].
	offset := n - 1 * p.	"Filling in the boundary conditions."
	self isCyclic
		ifTrue: 
			[p odd 
				ifTrue: 
					[1 to: p - 1 do: [:j | 
						jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np and: np put: 1.0d] 
				ifFalse: 
					[1 to: p - 2 do: [:j | jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np - 1 and: p put: 1.0d.		"(pEven - 1) continuity at 1 and n is not guaranteed."
					matrix at: np and: np put: 1.0d]]		"pEven derivative at 1 and n are zeros."
		ifFalse: 
			[ii := offset.
			p // 2 + 1 to: p - 1 do: [:j | 
							ii := ii + 1.
							matrix at: ii and: j put: 1.0d.
							ii := ii + 1.
							matrix at: ii and: offset + j put: 1.0d].
			p even ifTrue: [matrix at: np - 1 and: p put: 1.0d].
			matrix at: np and: np put: 1.0d].
	derivsVector := StMGESpMatParPvMarko3 new solve: matrix with: bvector saveOriginal: false.
	derivs := StMFullMatrix new: n by: p.
	1 to: n do: [:i | (derivs at: i) equalFullRow: derivsVector at: i - 1 * p + 1]
!

computeDerivatives1
	"See derivation in 9spline.doc."

	| n p np matrix bvector y1 zero offset h dum jj ii derivsVector |
	n := xs size.
	p := degree.
	np := n * p.
	matrix := StMSparseMatrix new: np by: np.
	bvector := (StMFullColumn new: np).
	y1 := ys at: 1.
	(y1 isKindOf: Array) 
		ifTrue: [zero := (y1 class new: y1 size) atAllPut: Double zero] 
		ifFalse: [zero := y1 class zero].
	bvector atAllPut: zero.
	1 to: n - 1 do: [:i | 
			offset := i - 1 * p.
			h := (xs at: i + 1) - (xs at: i).
			2 to: p do: [:j | 
					matrix at: offset + j and: offset + j - 1 put: 1.0d.
					matrix at: offset + j and: offset + j - 1 + p put: -1.0d].
			dum := 1.0d.
			1 to: p do: [:j | 
					dum := dum * h / j.
					j to: p do: [:k | matrix at: offset + k - j + 1 and: offset + k put: dum]].
			bvector at: offset + 1 put: (ys at: i + 1) - (ys at: i)].
	offset := n - 1 * p.	"Filling in the boundary conditions."
	self isCyclic
		ifTrue: 
			[p odd 
				ifTrue: 
					[1 to: p - 1 do: [:j | 
						jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np and: np put: 1.0d] 
				ifFalse: 
					[1 to: p - 2 do: [:j | jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np - 1 and: p put: 1.0d.		"(pEven - 1) continuity at 1 and n is not guaranteed."
					matrix at: np and: np put: 1.0d]]		"pEven derivative at 1 and n are zeros."
		ifFalse: 
			[ii := offset.
			p // 2 + 1 to: p - 1 do: [:j | 
							ii := ii + 1.
							matrix at: ii and: j put: 1.0d.
							ii := ii + 1.
							matrix at: ii and: offset + j put: 1.0d].
			p even ifTrue: [matrix at: np - 1 and: p put: 1.0d].
			matrix at: np and: np put: 1.0d].
	derivsVector := StMGESpMatParPvMarko3 new solve: matrix with: bvector.
	derivs := StMFullMatrix new: n by: p.
	1 to: n do: [:i | (derivs at: i) equalFullRow: derivsVector at: i - 1 * p + 1]
!

computeDerivatives2
	"See derivation in 9spline.doc."

	| n p np matrix bvector y1 zero offset h dum jj ii derivsVector |
	n := xs size.
	p := degree.
	np := n * p.
	matrix := StMSparseMatrix new: np by: np.
	bvector := (StMFullColumn new: np).
	y1 := ys at: 1.
	(y1 isKindOf: Array) 
		ifTrue: [zero := (y1 class new: y1 size) atAllPut: Double zero] 
		ifFalse: [zero := y1 class zero].
	bvector atAllPut: zero.
	1 to: n - 1 do: [:i | 
			offset := i - 1 * p.
			h := (xs at: i + 1) - (xs at: i).
			2 to: p do: [:j | 
					matrix at: offset + j and: offset + j - 1 put: 1.0d.
					matrix at: offset + j and: offset + j - 1 + p put: -1.0d].
			dum := 1.0d.
			1 to: p do: [:j | 
					dum := dum * h / j.
					j to: p do: [:k | matrix at: offset + k - j + 1 and: offset + k put: dum]].
			bvector at: offset + 1 put: (ys at: i + 1) - (ys at: i)].
	offset := n - 1 * p.	"Filling in the boundary conditions."
	self isCyclic
		ifTrue: 
			[p odd 
				ifTrue: 
					[1 to: p - 1 do: [:j | 
						jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np and: np put: 1.0d] 
				ifFalse: 
					[1 to: p - 2 do: [:j | jj := offset + j.
						matrix at: jj and: j put: 1.0d.
						matrix at: jj and: jj put: -1.0d].
					matrix at: np - 1 and: p put: 1.0d.		"(pEven - 1) continuity at 1 and n is not guaranteed."
					matrix at: np and: np put: 1.0d]]		"pEven derivative at 1 and n are zeros."
		ifFalse: 
			[ii := offset.
			p // 2 + 1 to: p - 1 do: [:j | 
							ii := ii + 1.
							matrix at: ii and: j put: 1.0d.
							ii := ii + 1.
							matrix at: ii and: offset + j put: 1.0d].
			p even ifTrue: [matrix at: np - 1 and: p put: 1.0d].
			matrix at: np and: np put: 1.0d].
	derivsVector := StMGESpMatParPvMarko3 new solve: matrix with: bvector.
	derivs := StMFullMatrix new: n by: p.
	1 to: n do: [:i | (derivs at: i) equalFullRow: derivsVector at: i - 1 * p + 1]
!

searchIndexFrom: first to: last 
	"Assume xs(first) < xvalue < xs(last)."

	| middle |
	first + 1 == last
		ifTrue: [index := first]
		ifFalse: 
			[middle := first + last // 2.
			xvalue < (xs at: middle)
				ifTrue: [self searchIndexFrom: first to: middle]
				ifFalse: [self searchIndexFrom: middle to: last]]
! !

!StMGeneralSpline methodsFor:'evaluating'!

d2ydx2: xx 
	^self derivative: 2 at: xx
!

d3ydx3: xx 
	^self derivative: 3 at: xx
!

derivative: n at: xx 
	"dydx(x) := dydxi + d2ydx2i*hi + d3ydx3i*hi^2/2!! +"
	"d2ydx2(x) := d2ydx2i + d3ydx3i*hi + d4ydx4i*hi^2/2!! +"

	| derivsi d1 sum |
	self calcIndexAndDeltaFor: xx.
	derivsi := derivs at: index.
	d1 := derivsi at: 1.
	(d1 isKindOf: Array)
		ifTrue: [sum := (d1 class new: d1 size) atAllPut: Double zero]
		ifFalse: [sum := d1 class zero].
	degree to: n + 1 by: -1 do: [:j | sum := sum + (derivsi at: j) * delta / (j - n) asDouble].
	^(derivsi at: n) + sum
!

differentiateWRTx
	^StMDifferentiatedGeneralSpline with: self derivativeOrder: 1
!

dydx: xx 
	^self derivative: 1 at: xx
!

setValue
	value := self y: x value
!

y: xx 
	"y(x) := yi + dydxi*hi + d2ydx2i*hi^2/2!! + d3ydx3i*hi^3/3!! +"

	| derivsi d1 sum |
	self calcIndexAndDeltaFor: xx.
	derivsi := derivs at: index.
	d1 := derivsi at: 1.
	(d1 isKindOf: Array)
		ifTrue: [sum := (d1 class new: d1 size) atAllPut: Double zero]
		ifFalse: [sum := d1 class zero].
	degree to: 1 by: -1 do: [:j | sum := sum + (derivsi at: j) * delta / j asDouble].
	^(ys at: index) + sum
! !

!StMGeneralSpline methodsFor:'initialize-release'!

initialize
	super initialize.
	index := 1
!

x: arg order: order xs: xarray ys: yarray 
	x := self conditionArgument: arg.
	degree := order asInteger.
	xs := StMArray newWithElements: xarray.
	xs isIncreasing ifFalse: [self error: #'StM: x series must be in increasing order.'].
	ys := StMArray newWithElements: yarray.
	(degree notNil and: [degree even and: [self isCyclic]])
		ifTrue: [self error: 'KOH001: Even degree cyclic spline is not possible.'].
	self computeDerivatives
! !

!StMGeneralSpline methodsFor:'testing'!

isCyclic
	"Answer whether the receiver is cyclic, i.e., folds back on itself."

	^ys size > 3 and: [ys first = ys last]
! !

!StMGeneralSpline class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMGeneralSpline.st,v 1.6 2006/10/04 13:00:53 cg Exp $'
! !
