"{ Package: 'stx:goodies/math/stmath' }"

StMIntegrator subclass:#StMIntegratorInterface
	instanceVariableNames:'tout hout hmin hmax tstart tend integrator'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Integrator'
!


!StMIntegratorInterface methodsFor:'accessing'!

changeTime: aDouble 
	system time: aDouble
!

hmax
	^hmax
!

hmax: aDouble 
	hmax := aDouble
!

hmin
	^hmin
!

hmin: aDouble 
	hmin := aDouble
!

hout
	^hout
!

hout: aDouble 
	hout := aDouble.
	hout < 0
		ifTrue: 
			[self logString: #'StM: \The output step size must be positive. \The integrator cannot proceed backwards.'.
			self error: '']
!

istep
	^integrator istep
!

opBDF
	^integrator opBDF
!

orderMax
	^system orderMax
!

time
	^integrator t
!

tout
	^tout
!

tstart
	^tstart
! !

!StMIntegratorInterface methodsFor:'initialize-release'!

initializeGlobally
	tstart := system startTime.
	self hout: system outputStepSize.
	hmax := system maxStepSize.
	hmin := system minStepSize.
	tout := system firstOutputTime.
	tend := system endTime
! !

!StMIntegratorInterface methodsFor:'run'!

checkForDiscontinuity
	self subclassResponsibility
!

checkForOutputThrough: t 
	self subclassResponsibility
!

correctorAbsoluteTolerance
	^system correctorAbsoluteTolerance
!

correctorRelativeTolerance
	^system correctorRelativeTolerance
!

incrementTime: aDouble 
	system time: aDouble
!

integrationAbsoluteTolerance
	^system integrationAbsoluteTolerance
!

integrationRelativeTolerance
	^system integrationRelativeTolerance
!

interpolateAt: tArg 
	"Interpolate for system state at tArg and leave system in that state."

	| yout ydotout yddotout |
	yout := integrator yDeriv: 0 at: tArg.
	ydotout := integrator yDeriv: 1 at: tArg.
	yddotout := integrator yDeriv: 2 at: tArg.
	system time: tArg.
	system y: yout.
	system ydot: ydotout.
	system yddot: yddotout.
	system simUpdateAll
!

postFirstStep
	system postFirstStep.
	integrator istep > 0 ifTrue: ["Noise make checking at the start unreliable."
		self checkForDiscontinuity].
	self checkForOutputThrough: integrator t
!

postRun
	"system postRun"

	self subclassResponsibility
!

postStep
	system postStep.
	integrator istep > 0 ifTrue: ["Noise make checking at the start unreliable."
		self checkForDiscontinuity].
	self checkForOutputThrough: integrator t
!

preRun
	"system preRun"

	self subclassResponsibility
!

reportStats
	self subclassResponsibility
!

run
	self preRun.
	self initializeLocally.
	self initializeGlobally.
	(hout > StMMachineAccuracy OfDoubleTimesFour and: [tout < (tend + (0.1d * hout))])
		ifTrue: [integrator run].
	self finalize.
	self reportStats.
	self postRun
!

suggestSmallerOrAcceptFirstStepSize: hnew 
	self subclassResponsibility
!

suggestSmallerOrAcceptStepSize: hnew 
	self subclassResponsibility
! !

!StMIntegratorInterface class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMIntegratorInterface.st,v 1.6 2006/10/04 13:00:53 cg Exp $'
! !
