"{ Package: 'stx:goodies/math/stmath' }"

StMLeastSquaresFit subclass:#StMLeastSquaresFitPolynomial
	instanceVariableNames:'order polynomial'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!


!StMLeastSquaresFitPolynomial methodsFor:'accessing'!

removeDependency
	super removeDependency.
	polynomial removeDependent: self
!

x: arg 
	| newx |
	newx := self conditionArgument: arg.
	x isNil
		ifFalse: 
			[x removeDependent: self.
			polynomial substitute: x with: newx].
	x := newx.
	self changed
! !

!StMLeastSquaresFitPolynomial methodsFor:'calc'!

calcCoeffs
	| orderPlusOne twoOrder xspower xspowerSum matrix vector rowi power element coeffs |
	self conditionData.
	orderPlusOne := order + 1.
	twoOrder := 2 * order.
	xspower := StMFullMatrix new: twoOrder.
	xspower at: 1 put: xs.
	2 to: twoOrder do: [:i | xspower at: i put: (xs elementTimesElementOf: (xspower at: i - 1))].
	xspowerSum := StMFullColumn new: twoOrder.
	1 to: twoOrder do: [:i | xspowerSum at: i put: (xspower at: i) sumOfElements].
	matrix := StMFullMatrix new: orderPlusOne by: orderPlusOne.
	vector := StMFullColumn new: orderPlusOne.
	1 to: orderPlusOne
		do: 
			[:i | 
			rowi := matrix at: i.
			1 to: orderPlusOne
				do: 
					[:j | 
					power := i - 1 + (j - 1).
					element := power == 0
								ifTrue: [xs size asDouble]
								ifFalse: [xspowerSum at: power].
					rowi at: j put: element]].
	1 to: orderPlusOne
		do: 
			[:i | 
			element := i == 1
						ifTrue: [ys sumOfElements]
						ifFalse: [ys dot: (xspower at: i - 1)].
			vector at: i put: element].
	[coeffs := StMGEFullMatParPv3 new
				solve: matrix
				with: vector
				saveOriginal: false]
		on: StMSingularMatrixError do: [:ex | ex resume].
	polynomial := StMPolynomial with: (x minus: xOffset)
				coeffs: coeffs
!

conditionData
	"Shift origin to midpoint of xs to avoid ill conditioning."

	| minMax |
	xs size == ys size ifFalse: [self error: 'x series and y series must be of the same size.'].
	minMax := xs minAndMaxValues.
	xOffset := (minMax at: 1)
				+ (minMax at: 2) / 2.0d.
	xs := xs minusNumber: xOffset
!

derivative: n at: xx 
	^polynomial derivative: n at: xx - xOffset
! !

!StMLeastSquaresFitPolynomial methodsFor:'changing'!

substitute: var1 with: var2 
	super substitute: var1 with: var2.
	polynomial == var1
		ifTrue: 
			[var1 removeDependent: self.
			var2 addDependent: self.
			polynomial := var2.
			self changed]
		ifFalse: [polynomial substitute: var1 with: var2]
! !

!StMLeastSquaresFitPolynomial methodsFor:'evaluating'!

differentiateWRT: variable 
	| deriv |
	deriv := polynomial differentiateWRT: variable.
	^StMCompositeFunctionX with: x copy expression: deriv
!

partialDifferentiateWRT: variable 
	| deriv |
	deriv := polynomial partialDifferentiateWRT: variable.
	^StMCompositeFunctionX with: x copy expression: deriv
!

setValue
	value := polynomial value
! !

!StMLeastSquaresFitPolynomial methodsFor:'initialize-release'!

x: arg order: integer xs: xarray ys: yarray 
	x := self conditionArgument: arg.
	xs := StMArray newWithElements: xarray.
	ys := StMArray newWithElements: yarray.
	order := integer asInteger.
	self calcCoeffs
! !

!StMLeastSquaresFitPolynomial class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMLeastSquaresFitPolynomial.st,v 1.6 2006/10/04 13:00:54 cg Exp $'
! !
