"{ Package: 'stx:goodies/math/stmath' }"

StMSolver subclass:#StMMatrixSolver3
	instanceVariableNames:'m n matrixA answerX rightHandSideB rowOrder colOrder rowScalings
		pivotValues singularPivotTolerance millisecondsToRun'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-MatrixSolver'
!

StMMatrixSolver3 comment:'Abstract class of all linear equations solvers.

Instance Variables:

m	<Integer>	row size of matrix.
n	<Integer>	column size of matrix.
matrixA	<StMRowTypeMatrix>	a kind of row type matrix.
answerX	<StMFullColumn>	solution.
rightHandSideB	<StMFullColumn>	right hand side vector.
rowOrder	<StMArray>	given row position as index, answer the original matrix row number.
colOrder	<StMArray>	given column position as index, answer the original matrix column number.
rowScalings	<StMFullColumn>	scaling factors to normalize each row so aij <= 1.0d.
pivotValues	<StMFullColumn>	values of pivots used.
singularPivotTolerance	<Double>	
millisecondsToRun	<Integer>	'
!


!StMMatrixSolver3 class methodsFor:'testing'!

isAbstractClass
	^true
! !

!StMMatrixSolver3 methodsFor:'accessing'!

colOrder
	^colOrder
!

millisecondsToRun
	^millisecondsToRun
!

pivotValues
	^pivotValues
!

rhsZeroElement
	| rhs1 |
	rhs1 := rightHandSideB at: 1.
	^rhs1 isSequenceable
		ifTrue: [rhs1 zeroCopy]
		ifFalse: [0.0d]
!

rowOrder
	^rowOrder
! !

!StMMatrixSolver3 methodsFor:'pivoting'!

doPivoting: p 
	"Matrices consist of rows of elements."
	"Row pivoting is actually carried out."
	"Column pivoting is done in-place."

	self subclassResponsibility
! !

!StMMatrixSolver3 methodsFor:'scaling'!

findScalingsForRows: i1 to: i2 
	"Row element * scaling <= 1.0."

	| maxRowElement |
	rowScalings := StMFullColumn new: m.
	i1 to: i2
		do: 
			[:i | 
			maxRowElement := (matrixA at: i) maxElement.
			maxRowElement = 0 ifTrue: [StMSingularMatrixError raiseSignal].
			rowScalings at: i put: maxRowElement reciprocal]
! !

!StMMatrixSolver3 methodsFor:'solution'!

basicSolve: aMatrix with: aVector saveOriginal: saveOriginal 
	self subclassResponsibility
!

postSolve
	self subclassResponsibility
!

preSolve: aMatrix with: aVector saveOriginal: saveOriginal 
	self subclassResponsibility
!

solve: aMatrix with: aVector saveOriginal: saveOriginal 
	self
		timedSolve: aMatrix
		with: aVector
		saveOriginal: saveOriginal.
	millisecondsToRun > 500 ifTrue: [ObjectMemory garbageCollect].
	^answerX
!

timedSolve: aMatrix with: aVector saveOriginal: saveOriginal 
	millisecondsToRun := nil.
	millisecondsToRun := Time millisecondsToRun: [self
					basicSolve: aMatrix
					with: aVector
					saveOriginal: saveOriginal].
	"Transcript cr; show: millisecondsToRun printString."
	^answerX
! !

!StMMatrixSolver3 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMMatrixSolver3.st,v 1.6 2006/10/04 13:00:54 cg Exp $'
! !
