"{ Package: 'stx:goodies/math/stmath' }"

StMSolver subclass:#StMNewtonRaphson2
	instanceVariableNames:'system xold x dx dxNorm dxNorms dxTol y yNorm yNormOld yNorms
		yNormTol pypx increNo increMax nDivergence nBackTracking twoAlp
		lam'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-NewtonRaphson'
!

StMNewtonRaphson2 comment:'Based on Numerical Recipes in C, 2nd Edition, page 383.
"Globally Convergent Methods for Nonlinear Systems of Equations."
y(x) = 0
y(xold) + pypx(xold)*dx + ... = 0
pypx*dx = -y
dx = -pypxInverse*y
xnew = xold + lam*dx              0 < lam =< 1

How to choose lam?
Define scalar function
func = 0.5*|y|^2 = 0.5*yT*y
grad(func) = yT*pypx
grad(func)*dx = (yT*pypx)*(-pypxInverse*y) = -yT*y = -2*func(xold)

Let lam = 1
Acceptance criterion
func(xnew) =< func(xold) + alp*grad(func)*(xnew - xold)               alp = 1.0d-4
or
func(xnew) =< func(xold) + alp*grad(func)*(lam*dx)
func(xnew) =< func(xold) + alp*lam*grad(func)*dx
func(xnew) =< func(xold) - 2*alp*lam*func(xold)
func(xnew)/func(xold) =< 1 - 2*alp*lam

If not accepted
lamPast = lam
Define scalar function
g(lam) = func(xold + lam*dx)
gp(lam) = dgdlam(lam) = grad(func)*dx = -2*func(xold)
g(0) = func(xold)
g(1) = func(xold + dx)
gp(lam) = -2*g(0)
Form quadratic 
g(lam) ~= (g(1) - g(0) - gp(0))*lam^2 + gp(0)*lam + g(0)
g(lam) ~= (g(1) + g(0))*lam^2 - 2*g(0)*lam + g(0)
At minimum
lam = -gp(0)/(2*(g(1) - g(0) - gp(0)))
lam = g(0)/(g(1) + g(0))
Limit 
0.1 =< lam =< 0.5
Apply acceptance criterion, if not accepted,
lamPast = lam
Form cubic
g(lam) = a*lam^3 + b*lam^2 + gp(0)*lam + g(0)
g(lam) = a*lam^3 + b*lam^2 + 2*g(0)*lam + g(0)
Solve for a and b
rhs1 = g(lam) - gp(0)*lam - g(0)
rhs2 = g(lamPast) - gp(0)*lamPast - g(0)
rhs1 = g(lam) + 2*g(0)*lam - g(0) = g(lam) + (2*lam - 1)*g(0)
rhs2 = g(lamPast) + 2*g(0)*lamPast - g(0) = g(lamPast) + (2*lamPast - 1)*g(0)
factor = 1/(lam - lamPast)
a = factor*(rhs1/lam^2 - rhs2/lamPast^2)
b = factor*(-rhs1*lamPast/lam^2 + rhs2*lam/lamPast^2)
At minimum
lam = (-b + sqrt(b^2 - 3*a*gp(0)))/(3*a)
lam = (-b + sqrt(b^2 + 6*a*g(0)))/(3*a)
Limit 
0.1*lamPast =< lam =< 0.5*lamPast

Instance Variables:

n	<ClassOfVariable>	description of variable''s function
xold	<ClassOfVariable>	description of variable''s function
x	<ClassOfVariable>	description of variable''s function
mdx	<ClassOfVariable>	description of variable''s function
y	<ClassOfVariable>	description of variable''s function
pypx	<ClassOfVariable>	description of variable''s function
system	<ClassOfVariable>	description of variable''s function
dxNorms	<ClassOfVariable>	description of variable''s function
yNorms	<ClassOfVariable>	description of variable''s function
iterNo	<ClassOfVariable>	description of variable''s function
iterMax	<ClassOfVariable>	description of variable''s function
nDivergence	<ClassOfVariable>	description of variable''s function
nConvergence	<ClassOfVariable>	description of variable''s function
errorTol	<ClassOfVariable>	description of variable''s function
glam0	<ClassOfVariable>	description of variable''s function
gplam0	<ClassOfVariable>	description of variable''s function
glam1	<ClassOfVariable>	description of variable''s function
glam2	<ClassOfVariable>	description of variable''s function
lam	<ClassOfVariable>	description of variable''s function
lam1	<ClassOfVariable>	description of variable''s function
lam2	<ClassOfVariable>	description of variable''s function'
!


!StMNewtonRaphson2 methodsFor:'accessing'!

dxTol: tol 
	dxTol := tol
!

increMax: integer 
	increMax := integer
!

system: sys 
	system := sys
!

yNormTol: tol 
	yNormTol := tol
! !

!StMNewtonRaphson2 methodsFor:'initialize-release'!

initialize
	super initialize.
	dxNorms := OrderedCollection new.
	dxTol := StMMachineAccuracy OfDoubleTimesFour.
	yNorms := OrderedCollection new.
	yNormTol := StMMachineAccuracy OfDoubleTimesFour.
	increMax := 100.
	twoAlp := 2.0d-4
!

initializeLocally
	"setIndices is a fast way of clearing anOrderedCollection."

	increNo := 0.
	nDivergence := 0.
	nBackTracking := 0.
	dxNorms setIndices.
	yNorms setIndices.
	yNormOld := StMMachineAccuracy OfLargestDouble
! !

!StMNewtonRaphson2 methodsFor:'solving'!

askSystemToUpdate
	system updateForNewtonRaphson
!

calcdxNorm
	self subclassResponsibility
!

calcyNorm
	self subclassResponsibility
!

doBackTracking
	| g0 glam lamPast glamPast rhs1 rhs2 factor lamSq lamPastSq a b lamMin lamMax |
	g0 := yNormOld.
	glam := yNorm.
	lamPast := 1.0d.
	nBackTracking := nBackTracking + 1.
	lam := g0 / (glam + g0).
	lam := lam max: 0.1d.
	self xEqualxoldPlusdxTimeslam.
	self passRootToSystem.
	self askSystemToUpdate.
	self fillY.
	self calcyNorm.
	[yNorm / yNormOld <= (1.0d - (twoAlp * lam))]
		whileFalse: 
			[glamPast := glam.
			glam := yNorm.
			rhs1 := glam + (2.0d * lam - 1.0d * g0).
			rhs2 := glamPast + (2.0d * lamPast - 1.0d * g0).
			factor := 1.0d / (lam - lamPast).
			lamSq := lam * lam.
			lamPastSq := lamPast * lamPast.
			a := factor * (rhs1 / lamSq - (rhs2 / lamPastSq)).
			b := factor * (0.0d - rhs1 * lamPast / lamSq + (rhs2 * lam / lamPastSq)).
			lamPast := lam.
			nBackTracking := nBackTracking + 1.
			lam := 0.0d - b + (b * b + (6.0d * a * g0)) sqrt / (3.0d * a).
			lamMin := 0.1d * lamPast.
			lamMax := 0.5d * lamPast.
			lam < lamMin
				ifTrue: [lam := lamMin]
				ifFalse: [lam > lamMax ifTrue: [lam := lamMax]].
			self xEqualxoldPlusdxTimeslam.
			self passRootToSystem.
			self askSystemToUpdate.
			self fillY.
			self calcyNorm].
	self dxEqualdxTimeslam
!

dxEqualdxTimeslam
	self subclassResponsibility
!

fillPyPx
	self subclassResponsibility
!

fillY
	self subclassResponsibility
!

improveRoot
	xold := x.
	self xEqualxoldPlusdx.
	self passRootToSystem.
	self askSystemToUpdate.
	self fillY.
	self calcyNorm.
	self calcdxNorm.
	dxNorms add: dxNorm.
	yNorms add: yNorm.
	yNormOld := yNorm
!

improveRootAndMinimizeYNorm
	xold := x.
	self xEqualxoldPlusdx.
	self passRootToSystem.
	self askSystemToUpdate.
	self fillY.
	self calcyNorm.
	yNorm > yNormTol ifTrue: [yNorm / yNormOld <= (1.0d - twoAlp) ifFalse: [self doBackTracking]].
	self calcdxNorm.
	dxNorms add: dxNorm.
	yNorms add: yNorm.
	yNormOld := yNorm
!

isConverged
	^self isConvergedToNumericalLimit
!

isConvergedToNumericalLimit
	"worthIterating is less stringent with IterNo."
	"nDivergenceMax is the number of small divergences allowed."

	| tooLargeTol smallEnoughTol nDecade dxNormIterNo dxNormIterNoOld farTooLargeError worthIterating stillConverging nDivergenceMax |
	tooLargeTol := 1.0d.
	smallEnoughTol := 1.0d-18.
	nDecade := (tooLargeTol / smallEnoughTol) log.
	nDivergenceMax := 3.
	dxNormIterNo := dxNorms at: increNo.
	^increNo > 1
		ifTrue: 
			[dxNormIterNoOld := dxNorms at: increNo - 1.
			farTooLargeError := dxNormIterNo > tooLargeTol.
			worthIterating := dxNormIterNo > (smallEnoughTol * (10.0d raisedTo: increNo - 1 / increMax * nDecade)).
			stillConverging := dxNormIterNo < dxNormIterNoOld
						ifTrue: [true]
						ifFalse: 
							[farTooLargeError ifFalse: [nDivergence := nDivergence + 1].
							nDivergence < nDivergenceMax].
			(farTooLargeError | (worthIterating & stillConverging)) not]
		ifFalse: 
			[worthIterating := dxNormIterNo > smallEnoughTol.
			worthIterating not]
!

iterate
	self fillY.
	self calcyNorm.
	yNorms add: yNorm.
	yNorm < yNormTol ifTrue: ["Initial x is root."
		^self].
	increNo := 0.
	
	[increNo := increNo + 1.
	increNo > increMax
		ifTrue: 
			[self reportStats.
			StMMaximumIterationError raiseSignal].
	self fillPyPx.
	self solveEquations.
	self improveRoot.
	self isConverged] whileFalse
!

iterateWithBackTracking
	self fillY.
	self calcyNorm.
	yNorms add: yNorm.
	yNorm < yNormTol ifTrue: ["Initial x is root."
		^self].
	increNo := 0.
	
	[increNo := increNo + 1.
	increNo > increMax
		ifTrue: 
			[self reportStats.
			StMMaximumIterationError raiseSignal].
	self fillPyPx.
	self solveEquations.
	self improveRootAndMinimizeYNorm.
	self isConverged] whileFalse
!

passRootToSystem
	system dx: dx x: x
!

postRun
	system postNewtonRaphson
!

preRun
	system preNewtonRaphson
!

reportStats
	statistics at: #increNo put: increNo.
	system reportNewtonRaphsonStats: statistics
!

run
	self preRun.
	self initializeLocally.
	self initializeGlobally.
	self iterate.
	self finalize.
	self reportStats.
	self postRun.
	^x
!

runWithBackTracking
	self preRun.
	self initializeLocally.
	self initializeGlobally.
	self iterateWithBackTracking.
	self finalize.
	self reportStats.
	self postRun.
	^x
!

solveEquations
	self subclassResponsibility
!

xEqualxoldPlusdx
	self subclassResponsibility
!

xEqualxoldPlusdxTimeslam
	self subclassResponsibility
! !

!StMNewtonRaphson2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMNewtonRaphson2.st,v 1.4 2006/10/04 13:00:54 cg Exp $'
! !
