"{ Package: 'stx:goodies/math/stmath' }"

StMSolver subclass:#StMNonlinearEquationSolver2
	instanceVariableNames:'n vars eqns jacobian'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-NewtonRaphson'
!


!StMNonlinearEquationSolver2 class methodsFor:'instance creation'!

new: size 
	^super new initialize: size
! !

!StMNonlinearEquationSolver2 methodsFor:'accessing'!

eqnAt: i put: eqn 
	eqns at: i put: eqn
!

n
	^n
!

varAt: i put: var 
	vars at: i put: var
! !

!StMNonlinearEquationSolver2 methodsFor:'initialize-release'!

initialize: size 
	n := size.
	vars := StMFullColumn new: size.
	eqns := StMFullColumn new: size.
	jacobian := StMFullMatrix new: size by: size
! !

!StMNonlinearEquationSolver2 methodsFor:'solving'!

dx: dx x: x 
	n == 1
		ifTrue: [(vars at: 1)
				x: x]
		ifFalse: [1 to: n do: [:i | (vars at: i)
					x: (x at: i)]]
!

fillErrorVector: y 
	1 to: n do: [:i | y at: i put: (eqns at: i) value asDouble]
!

fillJacobian: pypx 
	| pypxrow jacobianrow |
	1 to: n
		do: 
			[:i | 
			pypxrow := pypx at: i.
			jacobianrow := jacobian at: i.
			1 to: n do: [:j | pypxrow at: j put: (jacobianrow at: j) value asDouble]]
!

fillVarVector: x 
	1 to: n do: [:i | x at: i put: (vars at: i) value asDouble]
!

postNewtonRaphson
	"Do nothing."
!

postNewtonRaphsonIteration
	"Do nothing."
!

preNewtonRaphson
	| rowi eqni partialij |
	(vars detect: [:var | var value isReal not]
		ifNone: [false])
		ifTrue: [self error: 'Variables must have initial values.'].
	(eqns detect: [:eqn | eqn value isReal not]
		ifNone: [false])
		ifTrue: [self error: 'Equations must be valid.'].
	1 to: n
		do: 
			[:i | 
			rowi := jacobian at: i.
			eqni := eqns at: i.
			1 to: n
				do: 
					[:j | 
					partialij := eqni partialDifferentiateWRT: (vars at: j).
					rowi at: j put: partialij]]
!

preNewtonRaphsonIteration
	"Do nothing."
!

pypx
	^n == 1
		ifTrue: [(jacobian at: 1 and: 1) value asDouble]
		ifFalse: [jacobian collect: [:row | row collect: [:aij | aij value asDouble]]]
!

reportNewtonRaphsonStats: stats 
	"Do nothing."
!

run
	| newtonRaphson |
	n == 1
		ifTrue: [newtonRaphson := StMScalarNewtonRaphson2 new]
		ifFalse: 
			[newtonRaphson := StMVectorNewtonRaphson2 new.
			newtonRaphson x: (StMFullColumn new: n).
			newtonRaphson y: (StMFullColumn new: n).
			newtonRaphson pypx: (StMFullMatrix new: n by: n).
			newtonRaphson matrixSolver: StMGEFullMatParPv3 new].
	newtonRaphson system: self.
	newtonRaphson run
!

runWithBackTracking
	| newtonRaphson |
	n == 1
		ifTrue: [newtonRaphson := StMScalarNewtonRaphson2 new]
		ifFalse: 
			[newtonRaphson := StMVectorNewtonRaphson2 new.
			newtonRaphson x: (StMFullColumn new: n).
			newtonRaphson y: (StMFullColumn new: n).
			newtonRaphson pypx: (StMFullMatrix new: n by: n).
			newtonRaphson matrixSolver: StMGEFullMatParPv3 new].
	newtonRaphson system: self.
	newtonRaphson runWithBackTracking
!

updateForNewtonRaphson
	"Do nothing."
!

updateNewtonRaphsonErrorVectorOnly
	"Do nothing."
!

x
	^n == 1
		ifTrue: [(vars at: 1) value asDouble]
		ifFalse: [vars collect: [:var | var value asDouble]]
!

y
	^n == 1
		ifTrue: [(eqns at: 1) value asDouble]
		ifFalse: [eqns collect: [:eqn | eqn value asDouble]]
! !

!StMNonlinearEquationSolver2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMNonlinearEquationSolver2.st,v 1.6 2006/10/04 13:00:55 cg Exp $'
! !
