"{ Package: 'stx:goodies/math/stmath' }"

StMFunctionManyArgs subclass:#StMPiecewiseFunction
	instanceVariableNames:'functions transitions'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!

StMPiecewiseFunction comment:'PiecewiseFunction.
The piecewise functions must reference the same x so that everything is consistent. Otherwise the value returned will not be correct when the x''s are changed independently.

Instance Variables:

x	<StMArgumentHolder>	The holder is used in functions so that change is argument is reflected in functions.
functions	<OrderedCollection>	func1, func2, .., funcn.
transitions	<OrderedCollection>	-inf < x1 < x2 < x3 ...< inf.

When 
	(x1 <= keyFunction < x2) 
then 
	function := functions at: 2.

To get 
	(x1 <= keyFunction <= x2), 
use 
	(x1 <= keyFunction < x2+) 
where
	x2+ :=  x2 * (1.0d + machinePrecision).
'
!


!StMPiecewiseFunction class methodsFor:'instance creation'!

with: var functions: funcs transitions: trans 
	^super new
		x: var
		functions: funcs
		transitions: trans
! !

!StMPiecewiseFunction methodsFor:'accessing'!

x: arg 
	| newx |
	newx := self conditionArgument: arg.
	x isNil
		ifFalse: 
			[x removeDependent: self.
			functions do: [:function | function substitute: x with: newx].
			transitions do: [:transition | transition substitute: x with: newx]].
	x := newx.
	self changed
! !

!StMPiecewiseFunction methodsFor:'changing'!

substitute: var1 with: var2 
	super substitute: var1 with: var2.
	functions do: [:function | function substitute: var1 with: var2].
	transitions do: [:transition | transition substitute: var1 with: var2]
! !

!StMPiecewiseFunction methodsFor:'copying'!

postCopy
	super postCopy.
	functions := functions
				collect: 
					[:function | 
					| newFunction |
					newFunction := function copy.
					newFunction addDependent: self.
					newFunction].
	transitions := transitions
				collect: 
					[:transition | 
					| newTransition |
					newTransition := transition copy.
					newTransition addDependent: self.
					newTransition]
! !

!StMPiecewiseFunction methodsFor:'evaluating'!

differentiateWRT: variable 
	| derivs answer |
	derivs := functions collect: [:function | function differentiateWRT: variable].
	answer := StMPiecewiseFunction
				with: x copy
				functions: derivs
				transitions: transitions.
	^StMCompositeFunctionX with: x copy expression: answer
!

partialDifferentiateWRT: variable 
	| derivs answer |
	derivs := functions collect: [:function | function partialDifferentiateWRT: variable].
	answer := StMPiecewiseFunction
				with: x copy
				functions: derivs
				transitions: transitions.
	^answer
!

setValue
	| nTransitions |
	nTransitions := transitions size.
	1 to: nTransitions do: [:i | x value < (transitions at: i) value
			ifTrue: 
				[value := (functions at: i) value.
				^self]].
	value := (functions at: nTransitions + 1) value
!

simplified
	x := x simplified.
	1 to: functions size do: [:i | functions at: i put: (functions at: i) simplified].
	1 to: transitions size do: [:i | transitions at: i put: (transitions at: i) simplified]
! !

!StMPiecewiseFunction methodsFor:'initialize-release'!

removeDependency
	"function, transition are not StMArgumentHolders."

	super removeDependency.
	functions do: [:function | function removeDependent: self].
	transitions do: [:transition | transition removeDependent: self]
!

x: var functions: funcs transitions: trans 
	x := self conditionArgument: var.
	functions := funcs collect: [:func | self conditionArgument: func].
	transitions := trans collect: [:tran | self conditionArgument: tran]
! !

!StMPiecewiseFunction methodsFor:'message handling'!

argsPerform: aSymbol 
	x perform: aSymbol.
	functions do: [:function | function perform: aSymbol].
	transitions do: [:transition | transition perform: aSymbol]
!

argsPerform: selector withArguments: anArray 
	x perform: selector withArguments: anArray.
	functions do: [:function | function perform: selector withArguments: anArray].
	transitions do: [:transition | transition perform: selector withArguments: anArray]
! !

!StMPiecewiseFunction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMPiecewiseFunction.st,v 1.6 2006/10/04 13:00:55 cg Exp $'
! !
