"{ Package: 'stx:goodies/math/stmath' }"

StMFunctionManyArgs subclass:#StMPolynomial
	instanceVariableNames:'coeffs'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!


!StMPolynomial class methodsFor:'instance creation'!

with: var coeffs: funcs 
	^super new x: var coeffs: funcs
! !

!StMPolynomial methodsFor:'accessing'!

coeffs: collection 
	coeffs := collection collect: [:coeff | self conditionArgument: coeff]
!

order
	^coeffs size - 1
! !

!StMPolynomial methodsFor:'changing'!

substitute: var1 with: var2 
	super substitute: var1 with: var2.
	coeffs do: [:coeff | coeff substitute: var1 with: var2]
! !

!StMPolynomial methodsFor:'copying'!

postCopy
	super postCopy.
	coeffs := coeffs
				collect: 
					[:coeff | 
					| newCoeff |
					newCoeff := coeff copy.
					newCoeff addDependent: self.
					newCoeff]
! !

!StMPolynomial methodsFor:'evaluating'!

differentiateWRT: variable 
	| dfuncdx dxdvar dcoeffsdvar |
	self == variable ifTrue: [^StMConstant with: 1].
	dfuncdx := self differentiateWRTx.
	dxdvar := x differentiateWRT: variable.
	dcoeffsdvar := StMPolynomial with: x copy coeffs: (coeffs collect: [:coeff | coeff differentiateWRT: variable]).
	^(StMCompositeFunctionX with: x copy expression: ((dfuncdx times: dxdvar)
			plus: dcoeffsdvar)) simplified
!

differentiateWRTx
	| n newCoeffs |
	n := coeffs size.
	n < 2 ifTrue: [^StMConstant with: 0].
	n == 2 ifTrue: [^coeffs at: 2].
	newCoeffs := Array new: n - 1.
	2 to: n do: [:i | newCoeffs at: i - 1 put: ((coeffs at: i)
				times: i - 1)].
	^StMPolynomial with: x copy coeffs: newCoeffs
!

partialDifferentiateWRT: variable 
	| dfuncdx dxdvar dcoeffsdvar |
	self == variable ifTrue: [^StMConstant with: 1].
	dfuncdx := self differentiateWRTx.
	dxdvar := x partialDifferentiateWRT: variable.
	dcoeffsdvar := self class with: x copy coeffs: (coeffs collect: [:coeff | coeff partialDifferentiateWRT: variable]).
	^((dfuncdx times: dxdvar)
		plus: dcoeffsdvar) simplified
!

setValue
	| xvalue xpower |
	xvalue := x value.
	xpower := xvalue.
	value := (coeffs at: 1) value.
	2 to: coeffs size
		do: 
			[:i | 
			value := value + ((coeffs at: i) value * xpower).
			xpower := xpower * xvalue]
!

simplified
	| newCoeffs highestDegreeFound coeffi |
	x := x simplified.
	x isZero
		ifTrue: 
			[self removeDependency.
			^StMConstant with: 0].
	coeffs := coeffs collect: [:coeff | coeff simplified].
	newCoeffs := OrderedCollection new.
	highestDegreeFound := false.
	coeffs size
		to: 1
		by: -1
		do: [:i | highestDegreeFound
				ifTrue: [newCoeffs addFirst: (coeffs at: i)]
				ifFalse: 
					[coeffi := coeffs at: i.
					coeffi isZero
						ifTrue: [coeffi removeDependency]
						ifFalse: 
							[newCoeffs addFirst: coeffi.
							highestDegreeFound := true]]].
	coeffs := newCoeffs
! !

!StMPolynomial methodsFor:'initialize-release'!

removeDependency
	super removeDependency.
	coeffs do: [:coeff | coeff removeDependent: self]
!

x: arg coeffs: funcs 
	x := self conditionArgument: arg.
	coeffs := funcs collect: [:func | self conditionArgument: func]
! !

!StMPolynomial methodsFor:'message handling'!

argsPerform: aSymbol 
	x perform: aSymbol.
	coeffs do: [:coeff | coeff perform: aSymbol]
!

argsPerform: selector withArguments: anArray 
	x perform: selector withArguments: anArray.
	coeffs do: [:coeff | coeff perform: selector withArguments: anArray]
! !

!StMPolynomial methodsFor:'testing'!

isZero
	coeffs do: [:coeff | coeff isZero ifFalse: [^false]].
	^true
! !

!StMPolynomial class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMPolynomial.st,v 1.6 2006/10/04 13:00:55 cg Exp $'
! !
