"{ Package: 'stx:goodies/math/stmath' }"

StMFunctionManyTerms subclass:#StMProduct
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-FunctionSymbolic'
!


!StMProduct class methodsFor:'instance creation'!

with: arg1 times: arg2 
	| instance |
	instance := self new.
	instance times: arg1.
	instance times: arg2.
	^instance
!

with: arg1 times: arg2 times: arg3 
	| instance |
	instance := self new.
	instance times: arg1.
	instance times: arg2.
	instance times: arg3.
	^instance
! !

!StMProduct methodsFor:'accessing'!

* arg 
	| term |
	term := self conditionArgument: arg.
	terms add: term.
	self changed
!

times: arg 
	| term |
	term := self conditionArgument: arg.
	terms add: term.
	self changed
! !

!StMProduct methodsFor:'evaluating'!

differentiateWRT: variable 
	"Apply chain rule of differentiation."
	"(xyz)' := x'yz + xy'z + xyz'."

	| derivatives derivativeTerms derivative newTermFunctions newTerm answer |
	derivatives := terms collect: [:term | term differentiateWRT: variable].
	derivativeTerms := OrderedCollection new.
	1 to: terms size
		do: 
			[:i | 
			derivative := derivatives at: i.
			newTermFunctions := terms shallowCopy.
			newTermFunctions at: i put: derivative.
			newTerm := StMProduct new.
			newTerm terms: newTermFunctions.
			derivativeTerms add: newTerm].
	answer := StMSum new.
	answer terms: derivativeTerms.
	^answer simplified
!

partialDifferentiateWRT: variable 
	"Apply chain rule of differentiation."
	"(xyz)' := x'yz + xy'z + xyz'."

	| derivatives derivativeTerms derivative newTermFunctions newTerm answer |
	derivatives := terms collect: [:term | term partialDifferentiateWRT: variable].
	derivativeTerms := OrderedCollection new.
	1 to: terms size
		do: 
			[:i | 
			derivative := derivatives at: i.
			newTermFunctions := terms shallowCopy.
			newTermFunctions at: i put: derivative.
			newTerm := StMProduct new.
			newTerm terms: newTermFunctions.
			derivativeTerms add: newTerm].
	answer := StMSum new.
	answer terms: derivativeTerms.
	^answer simplified
!

setValue
	value := 1.0d.
	terms do: [:term | value := value * term value]
!

simplified
	| newTerms term newSize answer |
	newTerms := OrderedCollection new.
	1 to: terms size
		do: 
			[:i | 
			term := (terms at: i) simplified.
			term isZero
				ifTrue: 
					[self removeDependency.
					^StMConstant with: 0].
			term isOne
				ifTrue: [term removeDependency]
				ifFalse: [newTerms add: term]].
	newSize := newTerms size.
	newSize == 0
		ifTrue: 
			[self removeDependency.
			^StMConstant with: 1].
	newSize == 1
		ifTrue: 
			[answer := newTerms at: 1.
			self removeDependency.
			dependents do: [:dependent | answer addDependent: dependent].
			^answer].
	terms := newTerms
! !

!StMProduct class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMProduct.st,v 1.6 2006/10/04 13:00:55 cg Exp $'
! !
