"{ Package: 'stx:goodies/math/stmath' }"

StMArray variableSubclass:#StMRowTypeMatrix
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-Array'
!

StMRowTypeMatrix comment:'Second dimension is a kind of Row which in turn can store Double''s only.
A matrix is a stack of rows.

GE	Gauss Elimination
LU	LU decomposition
SC	Scaling
FP	Full pivoting
PP	Partial pivoting
MT	Markowitz criterion and threshold (stops when threshold fails)
AM 	Criterion is Aij divided by (2.0d raisedTo: MarkowitzProduct * integerExp)

GE:SC:PV:AM:
GE:FPAM:
GE:PPAM:

LU:SC:PV:AM:
LUAM:			Factorized self and return row pivoting. Ready for use with different b vectors.
LU:SC:PV:		Scale and pivot b vector, then forward and back substitute with LU for x.'
!


!StMRowTypeMatrix class methodsFor:'instance creation'!

new: nrow by: ncol 
	"Creates the row dimension of a matrix and send a message to create the column 
	dimension."
	"StMFullMatrix new: 3 by: 4."

	^(super new: nrow)
		by: ncol
! !

!StMRowTypeMatrix methodsFor:'accessing'!

at: i and: j 
	"Return element aij."

	^(self at: i)
		at: j
!

at: i and: j put: value 
	"Set element aij."

	(self at: i)
		at: j put: value
!

fullColumn: j 
	"Returns the fullColumn of a matrix."

	| nrow fullColumn |
	nrow := self nrow.
	fullColumn := StMFullColumn new: nrow.
	1 to: nrow do: [:i | fullColumn at: i put: (self at: i and: j)].
	^fullColumn
!

matrixSolverClass
	self subclassResponsibility
!

ncol
	^(self at: 1) ncol
!

nrow
        "Answer the number of indexable fields in the receiver. This value is the 
        same as the largest legal subscript. Primitive is specified here to override 
        SequenceableCollection size."

"/        <primitive: 62>
        ^self basicSize
!

numberOfElements
	"Answer the number of elements in the full vector or matrix."

	^self nrow * self ncol
!

row: i 
	^(self at: i) copy
!

secondDimensionClass
	self subclassResponsibility
!

sparseColumn: j 
	"Returns the sparseColumn of a matrix."

	| nrow sparseColumn aij |
	nrow := self nrow.
	sparseColumn := StMSparseColumn new: nrow.
	1 to: nrow
		do: 
			[:i | 
			aij := self at: i and: j.
			(aij isNil or: [aij isZero])
				ifFalse: [sparseColumn at: i put: aij]].
	^sparseColumn
! !

!StMRowTypeMatrix methodsFor:'basic ops'!

conditionedByRow
	| nrow answer |
	nrow := self nrow.
	answer := self class new: nrow.
	1 to: nrow do: [:i | answer at: i put: (self at: i) conditioned].
	^answer
!

identity
	self nrow == self ncol ifFalse: [self error: 'Matrix is not square'].
	self zeroSelf.
	1 to: self nrow do: [:i | self at: i and: i put: 1.0d]
!

scaledWith: scalingVector forRows: i1 to: i2 
	i1 to: i2 do: [:i | (self at: i)
			magnifySelf: (scalingVector at: i)]
!

scalingVectorForRows: i1 to: i2 
	"Row element * scaling <= 1.0."

	| scalingVector maxElement |
	scalingVector := StMFullColumn new: self nrow.
	i1 to: i2
		do: 
			[:i | 
			maxElement := (self at: i) maxElement.
			scalingVector at: i put: 1.0d / maxElement].
	^scalingVector
!

timesFullColumn: aFullColumn 
	"a*b = a(i,j)b(j) sum j."

	| nrow answer |
	nrow := self nrow.
	answer := StMFullColumn new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				timesFullColumn: aFullColumn)].
	^answer
!

timesFullMatrix: aFullMatrix 
	"a*b = a(i,j)b(j,k)"

	| nrow answer |
	nrow := self nrow.
	answer := StMFullMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				timesFullMatrix: aFullMatrix)].
	^answer
!

timesFullRow: aFullRow 
	"a*b = a(i,1)b(1,j) sum j."

	| nrow answer |
	self ncol == 1 ifFalse: [self error: 'Matrix has wrong dimension for product with row.'].
	nrow := self nrow.
	answer := StMFullMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (aFullRow timesNumber: (self at: i and: 1))].
	^answer
!

timesSparseMatrix: aSparseMatrix 
	"a*b = a(i,j)b(j,k)"

	| nrow answer |
	nrow := self nrow.
	answer := StMSparseMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				timesSparseMatrix: aSparseMatrix)].
	^answer
!

timesTransposeFullColumn: aFullColumn 
	"a*bT = a(i,1)b(j)"

	^self timesFullRow: aFullColumn transpose
!

timesTransposeFullMatrix: aFullMatrix 
	"a*bT = a(i,j)b(k,j)"

	| nrow answer |
	nrow := self nrow.
	answer := StMFullMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				timesTransposeFullMatrix: aFullMatrix)].
	^answer
!

timesTransposeSparseMatrix: aSparseMatrix 
	"a*bT = a(i,j)b(k,j)"

	| nrow answer |
	nrow := self nrow.
	answer := StMSparseMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: ((self at: i)
				timesTransposeSparseMatrix: aSparseMatrix)].
	^answer
!

trace
	"No checking."

	| trace |
	trace := self at: 1 and: 1.
	2 to: self nrow do: [:i | trace := trace + (self at: i and: i)].
	^trace
!

transposeTimesFullColumn: aFullColumn 
	"aT*b = a(i,j)b(i) sum i."

	self subclassResponsibility
!

transposeTimesFullColumn: aFullColumn times: aNumber 
	"aT*b = a(i,j)b(i) sum i."

	| nrow ncol answer |
	nrow := self nrow.
	ncol := self ncol.
	answer := (StMFullColumn new: ncol) zeroSelf.
	1 to: nrow do: [:i | answer equalSelfPlusFullColumn: (self at: i)
			times: (aFullColumn at: i)
					* aNumber].
	^answer
!

transposeTimesFullMatrix: aFullMatrix 
	^self transpose timesFullMatrix: aFullMatrix
!

transposeTimesNumber: aNumber 
	self subclassResponsibility
!

transposeTimesSparseMatrix: aSparseMatrix 
	^self transpose timesSparseMatrix: aSparseMatrix
! !

!StMRowTypeMatrix methodsFor:'converting'!

asDiagonalMatrix
	| nrow answer |
	nrow := self nrow.
	answer := StMDiagonalMatrix new: nrow.
	1 to: nrow do: [:i | answer at: i put: (self at: i and: i)].
	^answer
!

asFullColumn
	self ncol == 1 ifFalse: [self error: 'Matrix cannot be converted into a column.'].
	^self fullColumn: 1
!

asIdentityMatrix
	^StMIdentityMatrix new: self nrow
!

asSparseColumn
	self ncol == 1 ifFalse: [self error: 'Matrix cannot be converted into a column.'].
	^self sparseColumn: 1
!

asZeroMatrix
	^StMZeroMatrix new: self nrow by: self ncol
! !

!StMRowTypeMatrix methodsFor:'initialize-release'!

validateElements
	"Check that all the arrays have the same second dimension size."

	| ncol secondDimensionClass |
	ncol := self ncol.
	secondDimensionClass := self secondDimensionClass.
	1 to: self nrow
		do: 
			[:i | 
			| array newArray |
			array := self at: i.
			newArray := (array isMemberOf: secondDimensionClass)
						ifTrue: [array]
						ifFalse: [array shallowCopy changeClassToThatOf: (secondDimensionClass new: 0)].
			newArray size == ncol ifFalse: [self error: 'StM: Matrix must have newArrays equal to the second dimension.'].
			newArray validateElements.
			self at: i put: newArray]
! !

!StMRowTypeMatrix methodsFor:'instance creation'!

by: ncol 
	"Add second dimension to matrix. Used by new:by: only."

	1 to: self nrow do: [:i | self at: i put: (self secondDimensionClass new: ncol)]
! !

!StMRowTypeMatrix methodsFor:'norms'!

rowSumNorm
	| rowsum answer |
	answer := (self at: 1) sumOfAbsElements.
	2 to: self size
		do: 
			[:i | 
			rowsum := (self at: i) sumOfAbsElements.
			answer < rowsum ifTrue: [answer := rowsum]].
	^answer
! !

!StMRowTypeMatrix methodsFor:'ops on self'!

at: i1 and: j1 minusDiagonalMatrix: aDiagonalMatrix 
	| n iOffset jOffset |
	n := aDiagonalMatrix nrow.
	iOffset := i1 - 1.
	jOffset := j1 - 1.
	1 to: n do: [:ii | (self at: iOffset + ii)
			at: jOffset + ii minusNumber: (aDiagonalMatrix at: ii)]
!

at: i1 and: j1 minusFullColumn: aFullColumn 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullColumn nrow do: [:ii | (self at: iOffset + ii)
			at: j1 minusNumber: (aFullColumn at: ii)]
!

at: i1 and: j1 minusFullColumn: aFullColumn times: aNumber 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullColumn nrow do: [:ii | (self at: iOffset + ii)
			at: j1 minusNumber: (aFullColumn at: ii)
				* aNumber]
!

at: i1 and: j1 minusFullMatrix: aFullMatrix 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1 minusFullRow: (aFullMatrix at: ii)]
!

at: i1 and: j1 minusFullMatrix: aFullMatrix times: aNumber 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1
			minusFullRow: (aFullMatrix at: ii)
			times: aNumber]
!

at: i1 and: j1 minusFullRow: aFullRow 
	(self at: i1)
		at: j1 minusFullRow: aFullRow
!

at: i1 and: j1 minusFullRow: aFullRow times: aNumber 
	(self at: i1)
		at: j1
		minusFullRow: aFullRow
		times: aNumber
!

at: i1 and: j1 minusSparseMatrix: aSparseMatrix 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aSparseMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1 minusSparseRow: (aSparseMatrix at: ii)]
!

at: i1 and: j1 minusTransposeFullMatrix: aFullMatrix 
	| jOffset |
	jOffset := j1 - 1.
	1 to: aFullMatrix nrow do: [:jj | self
			at: i1
			and: jOffset + jj
			minusFullColumn: (aFullMatrix at: jj)]
!

at: i1 and: j1 minusTransposeFullMatrix: aFullMatrix times: aNumber 
	| jOffset |
	jOffset := j1 - 1.
	1 to: aFullMatrix nrow do: [:jj | self
			at: i1
			and: jOffset + jj
			minusFullColumn: (aFullMatrix at: jj)
			times: aNumber]
!

at: i1 and: j1 plusDiagonalMatrix: aDiagonalMatrix 
	| n iOffset jOffset |
	n := aDiagonalMatrix nrow.
	iOffset := i1 - 1.
	jOffset := j1 - 1.
	1 to: n do: [:ii | (self at: iOffset + ii)
			at: jOffset + ii plusNumber: (aDiagonalMatrix at: ii)]
!

at: i1 and: j1 plusFullColumn: aFullColumn 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullColumn nrow do: [:ii | (self at: iOffset + ii)
			at: j1 plusNumber: (aFullColumn at: ii)]
!

at: i1 and: j1 plusFullColumn: aFullColumn times: aNumber 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullColumn nrow do: [:ii | (self at: iOffset + ii)
			at: j1 plusNumber: (aFullColumn at: ii)
				* aNumber]
!

at: i1 and: j1 plusFullMatrix: aFullMatrix 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1 plusFullRow: (aFullMatrix at: ii)]
!

at: i1 and: j1 plusFullMatrix: aFullMatrix times: aNumber 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1
			plusFullRow: (aFullMatrix at: ii)
			times: aNumber]
!

at: i1 and: j1 plusFullRow: aFullRow 
	(self at: i1)
		at: j1 plusFullRow: aFullRow
!

at: i1 and: j1 plusFullRow: aFullRow times: aNumber 
	(self at: i1)
		at: j1
		plusFullRow: aFullRow
		times: aNumber
!

at: i and: j plusNumber: aNumber 
	^(self at: i)
		at: j plusNumber: aNumber
!

at: i1 and: j1 plusSparseMatrix: aSparseMatrix 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aSparseMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1 plusSparseRow: (aSparseMatrix at: ii)]
!

at: i1 and: j1 plusTransposeFullMatrix: aFullMatrix 
	| jOffset |
	jOffset := j1 - 1.
	1 to: aFullMatrix nrow do: [:jj | self
			at: i1
			and: jOffset + jj
			plusFullColumn: (aFullMatrix at: jj)]
!

at: i1 and: j1 plusTransposeFullMatrix: aFullMatrix times: aNumber 
	| jOffset |
	jOffset := j1 - 1.
	1 to: aFullMatrix nrow do: [:jj | self
			at: i1
			and: jOffset + jj
			plusFullColumn: (aFullMatrix at: jj)
			times: aNumber]
!

at: i1 and: j1 putFullColumn: aFullColumn 
	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullColumn nrow do: [:ii | self
			at: iOffset + ii
			and: j1
			put: (aFullColumn at: ii)]
!

at: i1 and: j1 putFullMatrix: aFullMatrix 
	"Put aFullMatrix into a submatrix of self."

	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1 putFullRow: (aFullMatrix at: ii)]
!

at: i1 and: j1 putFullMatrix: aFullMatrix times: aNumber 
	"Put aFullMatrix into a submatrix of self."

	| iOffset |
	iOffset := i1 - 1.
	1 to: aFullMatrix nrow do: [:ii | (self at: iOffset + ii)
			at: j1
			putFullRow: (aFullMatrix at: ii)
			times: aNumber]
!

at: i1 and: j1 putFullRow: aFullRow 
	(self at: i1)
		at: j1 putFullRow: aFullRow
!

conditionSelfByRow
	1 to: self nrow do: [:i | self at: i put: (self at: i) conditioned]
!

equalFullMatrix: aFullMatrix 
	1 to: self nrow do: [:i | (self at: i)
			equalFullRow: (aFullMatrix at: i)]
!

equalFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self nrow do: [:i | (self at: i)
			equalFullRow: (aFullMatrix at: iiOffset + i)
			at: jj1]
!

equalFullMatrix: aFullMatrix times: aNumber 
	1 to: self nrow do: [:i | (self at: i)
			equalFullRow: (aFullMatrix at: i)
			times: aNumber]
!

equalSelfMinusFullMatrix: aFullMatrix 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfMinusFullRow: (aFullMatrix at: i)]
!

equalSelfMinusFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self nrow do: [:i | (self at: i)
			equalSelfMinusFullRow: (aFullMatrix at: iiOffset + i)
			at: jj1]
!

equalSelfMinusFullMatrix: aFullMatrix times: aNumber 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfMinusFullRow: (aFullMatrix at: i)
			times: aNumber]
!

equalSelfPlusFullMatrix: aFullMatrix 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfPlusFullRow: (aFullMatrix at: i)]
!

equalSelfPlusFullMatrix: aFullMatrix at: ii1 and: jj1 
	| iiOffset |
	iiOffset := ii1 - 1.
	1 to: self nrow do: [:i | (self at: i)
			equalSelfPlusFullRow: (aFullMatrix at: iiOffset + i)
			at: jj1]
!

equalSelfPlusFullMatrix: aFullMatrix times: aNumber 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfPlusFullRow: (aFullMatrix at: i)
			times: aNumber]
!

equalSelfTimes: alpha minusFullMatrix: aFullMatrix times: beta 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfTimes: alpha
			minusFullRow: (aFullMatrix at: i)
			times: beta]
!

equalSelfTimes: alpha plusFullMatrix: aFullMatrix times: beta 
	1 to: self nrow do: [:i | (self at: i)
			equalSelfTimes: alpha
			plusFullRow: (aFullMatrix at: i)
			times: beta]
!

symLowerWithUpper
	"Upper triangle is given."
	"Fill lower triangle with upper triangle to make matrix symmetric."

	| nrow ncol |
	nrow := self nrow.
	ncol := self ncol.
	nrow == ncol ifFalse: [self error: 'StM: Matrix must be square to be made symmetric.'].
	1 to: ncol
		do: 
			[:i | 
			| row |
			row := self at: i.
			i + 1 to: ncol do: [:j | self
					at: j
					and: i
					put: (row at: j)]]
! !

!StMRowTypeMatrix methodsFor:'printing'!

outputOnFile: aString 
	| writeStream aij aijString |
	writeStream := aString asFilename writeStream.
	1 to: self nrow
		do: 
			[:i | 
			1 to: self ncol
				do: 
					[:j | 
					aij := self at: i and: j.
					aij == nil ifTrue: [aij := 0.0d].
					aijString := aij printString.
					aijString last == $d
						ifTrue: [aijString replaceAll: $d with: Character space]
						ifFalse: [aijString replaceAll: $d with: $e].
					writeStream nextPutAll: aijString.
					writeStream space].
			writeStream cr].
	writeStream flush; close
! !

!StMRowTypeMatrix methodsFor:'testing'!

isDiagonal
	| m n |
	m := self nrow.
	n := self ncol.
	m == n ifFalse: [^false].
	1 to: m
		do: 
			[:i | 
			| arrayi |
			arrayi := self at: i.
			1 to: n do: [:j | i == j ifFalse: [(arrayi at: j)
						= 0 ifFalse: [^false]]]].
	^true
!

isIdentity
	| n identicalElement m |
	m := self nrow.
	n := self ncol.
	m == n ifFalse: [^false].
	identicalElement := self at: 1 and: 1.
	1 to: m
		do: 
			[:i | 
			| arrayi |
			arrayi := self at: i.
			1 to: n do: [:j | i == j
					ifTrue: [(arrayi at: j)
							= identicalElement ifFalse: [^false]]
					ifFalse: [(arrayi at: j)
							= 0 ifFalse: [^false]]]].
	^true
! !

!StMRowTypeMatrix class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMRowTypeMatrix.st,v 1.7 2006/10/04 13:00:55 cg Exp $'
! !
