"{ Package: 'stx:goodies/math/stmath' }"

StMSparseVector variableSubclass:#StMSparseRow
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-ArraySparse'
!


!StMSparseRow methodsFor:'accessing'!

ncol
	"Answer the number of elements in the full vector or matrix."

	^fullsize
!

nrow
	^1
! !

!StMSparseRow methodsFor:'basic ops'!

transpose
	"Return a row with the same elements."
	"This method is much faster than the obvious way."

	^self copy changeClassToThatOf: (StMSparseColumn new: 0)
! !

!StMSparseRow methodsFor:'basic ops *'!

timesFullColumn: aFullColumn 
	| answer key |
	answer := 0.0d.
	1 to: self basicSize
		do: 
			[:index | 
			key := self basicAt: index.
			key == nil ifFalse: [answer := answer + ((valueArray at: index)
								* (aFullColumn at: key))]].
	^answer
!

timesFullMatrix: aFullMatrix 
	"Similar to inner dot product."

	| aFullRow answer i |
	aFullRow := aFullMatrix at: 1.
	answer := aFullRow class new: aFullRow size withAll: 0.0d.
	1 to: self basicSize
		do: 
			[:index | 
			i := self basicAt: index.
			i == nil ifFalse: [answer equalSelfPlusFullRow: (aFullMatrix at: i)
					times: (valueArray at: index)]].
	^answer
!

timesSparseMatrix: aSparseMatrix 
	"Similar to inner dot product."

	| aSparseRow answer i |
	aSparseRow := aSparseMatrix at: 1.
	answer := aSparseRow class new: aSparseRow ncol.
	1 to: self basicSize
		do: 
			[:index | 
			i := self basicAt: index.
			i == nil ifFalse: [answer equalSelfPlusSparseRow: (aSparseMatrix at: i)
					times: (valueArray at: index)]].
	^answer
!

timesTransposeSparseMatrix: aSparseMatrix 
	"a*bT = a(1,j)b(k,j)"

	| ncol answer ak |
	ncol := aSparseMatrix nrow.
	answer := StMSparseRow new: ncol.
	1 to: ncol
		do: 
			[:k | 
			ak := (aSparseMatrix at: k)
						dot: self.
			ak isZero ifFalse: [answer at: k put: ak]].
	^answer
! !

!StMSparseRow methodsFor:'basic ops +'!

plusSparseRow: aSparseRow 
	^super plusSparseVector: aSparseRow
!

plusSparseRow: aSparseRow times: beta 
	^super plusSparseVector: aSparseRow times: beta
!

times: alpha plusSparseRow: aSparseRow 
	^super times: alpha plusSparseVector: aSparseRow
!

times: alpha plusSparseRow: aSparseRow times: beta 
	^super
		times: alpha
		plusSparseVector: aSparseRow
		times: beta
! !

!StMSparseRow methodsFor:'basic ops -'!

minusSparseRow: aSparseRow 
	^super minusSparseVector: aSparseRow
!

minusSparseRow: aSparseRow times: beta 
	^super minusSparseVector: aSparseRow times: beta
!

times: alpha minusSparseRow: aSparseRow 
	^super times: alpha minusSparseVector: aSparseRow
!

times: alpha minusSparseRow: aSparseRow times: beta 
	^super
		times: alpha
		minusSparseVector: aSparseRow
		times: beta
! !

!StMSparseRow methodsFor:'converting'!

asFullRow
	| answer key |
	answer := StMFullRow new: self ncol.
	answer zeroSelf.
	1 to: self basicSize do: [:index | (key := self basicAt: index) == nil ifFalse: [answer at: key put: (valueArray at: index)]].
	^answer
!

asSparseRow
	^self
! !

!StMSparseRow methodsFor:'ops on self'!

at: i1 putFullRow: aFullRow 
	super at: i1 putFullVector: aFullRow
!

at: i1 putFullRow: aFullRow times: number 
	super
		at: i1
		putFullVector: aFullRow
		times: number
! !

!StMSparseRow methodsFor:'ops on self +'!

at: i plusFullRow: aFullRow 
	^super at: i plusFullVector: aFullRow
!

at: i plusFullRow: aFullRow times: aNumber 
	^super
		at: i
		plusFullVector: aFullRow
		times: aNumber
!

at: i1 plusSparseRow: aSparseRow 
	super at: i1 plusSparseVector: aSparseRow
!

equalSelfPlusSparseRow: aSparseRow 
	super equalSelfPlusSparseVector: aSparseRow
!

equalSelfPlusSparseRow: aSparseRow times: aNumber 
	super equalSelfPlusSparseVector: aSparseRow times: aNumber
! !

!StMSparseRow methodsFor:'ops on self -'!

at: i minusFullRow: aFullRow 
	super at: i minusFullVector: aFullRow
!

at: i minusFullRow: aFullRow times: aNumber 
	^super
		at: i
		minusFullVector: aFullRow
		times: aNumber
!

at: i1 minusSparseRow: aSparseRow 
	super at: i1 minusSparseVector: aSparseRow
!

equalSelfMinusSparseRow: aSparseRow 
	super equalSelfMinusSparseVector: aSparseRow
!

equalSelfMinusSparseRow: aSparseRow times: aNumber 
	super equalSelfMinusSparseVector: aSparseRow times: aNumber
! !

!StMSparseRow class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMSparseRow.st,v 1.6 2006/10/04 13:00:55 cg Exp $'
! !
