"{ Package: 'stx:goodies/math/stmath' }"

IdentityDictionary subclass:#StMSparseVector
	instanceVariableNames:'fullsize'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-ArraySparse'
!

StMSparseVector comment:'Used by SparseMatrix. The second dimension size (size2) is stored in SparseMatrix and not by individual IdentityDictionarys.'
!


!StMSparseVector class methodsFor:'instance creation'!

new: aInteger 
	^(super new: aInteger)
		fullsize: aInteger
! !

!StMSparseVector methodsFor:'accessing'!

at2: key 
        "Behaves like IdentityDictionary>at: but specialized for numbers."

        | length index continue value |
        length := keyArray "self"  basicSize.
        index := key \\ length + 1.
        continue := true.
        
        [| probe |
        probe := keyArray "self" basicAt: index.
        probe == nil
                ifTrue: 
                        [value := nil.
                        continue := false]
                ifFalse: [probe == key
                                ifTrue: 
                                        [value := valueArray at: index.
                                        continue := false]
                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]].
        continue] whileTrue.
        ^value
!

at: key 
        "Behaves like IdentityDictionary>at: but specialized for numbers."

        | length index |
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == nil
                                ifTrue: [^nil]
                                ifFalse: [probe == key
                                                ifTrue: [^valueArray at: index]
                                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
!

at: key put: aNumber 
        "Behaves like IdentityDictionary>at:put: but specialized for numbers."

        | length index |
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == DeletedEntry ifTrue:[probe := nil].
                        probe == nil
                                ifTrue: 
                                        [tally := tally + 1.
                                        keyArray "self" basicAt: index put: key.
                                        valueArray at: index put: aNumber.
                                        length - tally <= (length // 4) ifTrue: [self grow].
                                        ^aNumber]
                                ifFalse: [probe == key
                                                ifTrue: 
                                                        [valueArray at: index put: aNumber.
                                                        ^aNumber]
                                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
!

atIndex: index 
	^valueArray at: index
!

fullsize: aInteger 
	fullsize := aInteger
!

ncol
	self subclassResponsibility
!

nrow
	self subclassResponsibility
!

numberOfElements
	"Answer the number of elements in the full vector or matrix."

	^fullsize
!

tally
	^tally
!

validIndicesWithoutKey: discardKey 
        | indices |
        indices := OrderedCollection new: self size.
        1 to: keyArray "self" basicSize
                do: 
                        [:index | 
                        | key |
                        key := keyArray "self" basicAt: index.
                        key == DeletedEntry ifTrue:[key := nil].
                        key == nil ifFalse: [key == discardKey ifFalse: [indices add: index]]].
        ^indices
!

valueArray
	^valueArray
! !

!StMSparseVector methodsFor:'adding'!

grow
	| oldfullsize |
	oldfullsize := fullsize.
	super grow.
	fullsize := oldfullsize
! !

!StMSparseVector methodsFor:'basic ops'!

conditioned
	"Answer a fresh sparse vector of significant elements."

	| tol |
	tol := self maxElement * StMMachineAccuracy OfDouble.
	^self conditionedWithTol: tol
!

conditionedWithTol: tol 
	"Answer a fresh sparse vector of significant elements."

	| answer |
	answer := self class new.
	self keysAndValuesDo: [:key :value | value abs >= tol ifTrue: [answer at: key put: value]].
	^answer
!

dot: aVector 
        "Inner dot product."

        | answer |
        answer := 0.0d.
        1 to: keyArray "self" basicSize
                do: 
                        [:index | 
                        | key |
                        key := keyArray "self" basicAt: index.
                        key == DeletedEntry ifTrue:[key := nil].
                        key == nil ifFalse: [answer := (valueArray at: index) * (aVector at: key) + answer]].
        ^answer
!

negated
	^self timesNumber: -1.0d
!

times: double conditionedWithTol: tol 
	"Answer a fresh sparse vector of significant elements."

	| answer newValue |
	answer := self class new.
	self
		keysAndValuesDo: 
			[:key :value | 
			newValue := value * double.
			newValue abs >= tol ifTrue: [answer at: key put: newValue]].
	^answer
! !

!StMSparseVector methodsFor:'basic ops *'!

* aNumber 
	| answer |
	answer := self copy.
	^answer magnifySelf: aNumber
!

timesNumber: aNumber 
	| answer |
	answer := self copy.
	^answer magnifySelf: aNumber
! !

!StMSparseVector methodsFor:'basic ops +'!

plusSparseVector: aSparseVector 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector copy.
			self keysAndValuesDo: [:i :value | answer at: i plusNumber: value]]
		ifFalse: 
			[answer := self copy.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i plusNumber: value]].
	^answer
!

plusSparseVector: aSparseVector times: beta 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector timesNumber: beta.
			self keysAndValuesDo: [:i :value | answer at: i plusNumber: value]]
		ifFalse: 
			[answer := self copy.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i plusNumber: value * beta]].
	^answer
!

times: alpha plusSparseVector: aSparseVector 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector copy.
			self keysAndValuesDo: [:i :value | answer at: i plusNumber: value * alpha]]
		ifFalse: 
			[answer := self timesNumber: alpha.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i plusNumber: value]].
	^answer
!

times: alpha plusSparseVector: aSparseVector times: beta 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector timesNumber: beta.
			self keysAndValuesDo: [:i :value | answer at: i plusNumber: value * alpha]]
		ifFalse: 
			[answer := self timesNumber: alpha.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i plusNumber: value * beta]].
	^answer
! !

!StMSparseVector methodsFor:'basic ops -'!

minusSparseVector: aSparseVector 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector copy.
			self keysAndValuesDo: [:i :value | answer at: i minusNumber: value]]
		ifFalse: 
			[answer := self copy.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i minusNumber: value]].
	^answer
!

minusSparseVector: aSparseVector times: beta 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector timesNumber: beta.
			self keysAndValuesDo: [:i :value | answer at: i minusNumber: value]]
		ifFalse: 
			[answer := self copy.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i minusNumber: value * beta]].
	^answer
!

times: alpha minusSparseVector: aSparseVector 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector copy.
			self keysAndValuesDo: [:i :value | answer at: i minusNumber: value * alpha]]
		ifFalse: 
			[answer := self timesNumber: alpha.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i minusNumber: value]].
	^answer
!

times: alpha minusSparseVector: aSparseVector times: beta 
	| answer |
	self tally < aSparseVector tally
		ifTrue: 
			[answer := aSparseVector timesNumber: beta.
			self keysAndValuesDo: [:i :value | answer at: i minusNumber: value * alpha]]
		ifFalse: 
			[answer := self timesNumber: alpha.
			aSparseVector keysAndValuesDo: [:i :value | answer at: i minusNumber: value * beta]].
	^answer
! !

!StMSparseVector methodsFor:'norms'!

length
	"Euclidean norm."
	"Square root of the sum of squares."

	^self sumOfSquares sqrt
!

maxElement
        "abs is not efficient."

        | answer |
        answer := 0.0d.
        1 to: keyArray "self" basicSize
                do: 
                        [:index | 
                        | num |
                        num := valueArray at: index.
                        num == nil
                                ifFalse: 
                                        [num < 0.0d ifTrue: [num := 0.0d - num].
                                        answer < num ifTrue: [answer := num]]].
        ^answer
!

minAndMaxValues
        | min max num |
        min := 1.0d300.
        max := min negated.
        1 to: keyArray "self" basicSize
                do: 
                        [:index | 
                        num := valueArray at: index.
                        num == nil
                                ifFalse: 
                                        [min < num ifFalse: [min := num].
                                        max > num ifFalse: [max := num]]].
        ^Array with: min with: max
!

normL: number 
	^(self sumOfElementsRaisedTo: number)
		raisedTo: number reciprocal
!

rootMeanSquare
	"(sumOfSquares / numberOfElements) sqrt"

	^(self sumOfSquares / self numberOfElements) sqrt
!

sumOfAbsElements
        | sum |
        sum := 0.
        1 to: keyArray "self" basicSize do: 
                [:index | 
                | num |
                (num := valueArray at: index) == nil ifFalse: [sum := num > 0.0d
                                                ifTrue: [sum + num]
                                                ifFalse: [sum - num]]].
        ^sum
!

sumOfElements
        | sum |
        sum := 0.
        1 to: keyArray "self" basicSize do: 
                [:index | 
                | num |
                (num := valueArray at: index) == nil ifFalse: [sum := num + sum]].
        ^sum
!

sumOfElementsRaisedTo: number 
        "For each element abs it, raised it to the power of number."
        "Then sum them all."

        | sum |
        sum := 0.
        1 to: keyArray "self" basicSize do: 
                [:index | 
                | num |
                (num := valueArray at: index) == nil
                        ifFalse: 
                                [num < 0.0d ifTrue: [num := 0.0d - num].
                                sum := sum + (num raisedTo: number)]].
        ^sum
!

sumOfSquares
        "When sum is integer 0, num * num + sum is more efficient than sum + num * num."

        | sum |
        sum := 0.
        1 to: keyArray "self" basicSize do: 
                [:index | 
                | num |
                (num := valueArray at: index) == nil ifFalse: [sum := num * num + sum]].
        ^sum
! !

!StMSparseVector methodsFor:'ops on self'!

at: i putFullVector: aFullVector 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii put: (aFullVector at: ii)]
!

at: i putFullVector: aFullVector times: number 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii put: (aFullVector at: ii)
				* number]
!

conditionSelf
	"self become new sparse vector with significant elements only."

	self become: self conditioned
!

conditionSelfWithTol: tol 
	"self become new sparse vector with significant elements only."

	| newSelf |
	newSelf := self conditionedWithTol: tol.
	self become: newSelf
!

magnifySelf: double 
        double = 1 ifTrue: [^self].
        1 to: keyArray "self" basicSize do: [:index | |probe| 
            probe := (keyArray "self" basicAt: index).
            probe == DeletedEntry ifTrue:[probe := nil].
            probe == nil ifFalse: [valueArray at: index put: (valueArray at: index) * double]]
!

zeroSelf
        "Make non zero elements zero. Do not remove them."

        1 to: keyArray "self" basicSize do: [:index | 
            |probe|

            probe := (keyArray "self" basicAt: index).
            probe == DeletedEntry ifTrue:[probe := nil].
            probe == nil ifFalse: [valueArray at: index put: 0.0d]]
! !

!StMSparseVector methodsFor:'ops on self *'!

at: key timesNumber: aNumber 
        "Efficient method for row at: key put: ((row at: key) * aNumber)"

        | length index |
        aNumber = 1 ifTrue: [^aNumber].
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == DeletedEntry ifTrue:[probe := nil].
                        probe == nil ifFalse: [probe == key
                                        ifTrue: 
                                                [valueArray at: index put: (valueArray at: index)
                                                                * aNumber.
                                                ^aNumber]
                                        ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
! !

!StMSparseVector methodsFor:'ops on self +'!

at: i plusFullVector: aFullVector 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii plusNumber: (aFullVector at: ii)]
!

at: i plusFullVector: aFullVector times: aNumber 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii plusNumber: (aFullVector at: ii)
				* aNumber]
!

at: key plusNumber: aNumber 
        "Efficient method for row at: key put: ((row at: key) + aNumber)"

        | length index answer |
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == DeletedEntry ifTrue:[probe := nil].
                        probe == nil
                                ifTrue: 
                                        [tally := tally + 1.
                                        keyArray "self" basicAt: index put: key.
                                        valueArray at: index put: aNumber.
                                        length - tally <= (length // 4) ifTrue: [self grow].
                                        ^aNumber]
                                ifFalse: [probe == key
                                                ifTrue: 
                                                        [answer := (valueArray at: index)
                                                                                + aNumber.
                                                        valueArray at: index put: answer.
                                                        ^aNumber]
                                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
!

at: i1 plusSparseVector: aSparseVector 
	| ioffset |
	ioffset := i1 - 1.
	aSparseVector keysAndValuesDo: [:ii :value | self at: ioffset + ii plusNumber: value]
!

equalSelfPlusSparseVector: aSparseVector 
	aSparseVector keysAndValuesDo: [:i :value | self at: i plusNumber: value]
!

equalSelfPlusSparseVector: aSparseVector times: aNumber 
	aSparseVector keysAndValuesDo: [:i :value | self at: i plusNumber: value * aNumber]
! !

!StMSparseVector methodsFor:'ops on self -'!

at: i minusFullVector: aFullVector 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii minusNumber: (aFullVector at: ii)]
!

at: i minusFullVector: aFullVector times: aNumber 
	| iOffset |
	iOffset := i - 1.
	1 to: aFullVector size do: [:ii | self at: iOffset + ii minusNumber: (aFullVector at: ii)
				* aNumber]
!

at: key minusNumber: aNumber 
        "Efficient method for row at: key put: ((row at: key) + aNumber)"

        | length index answer |
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == DeletedEntry ifTrue:[probe := nil].
                        probe == nil
                                ifTrue: 
                                        [tally := tally + 1.
                                        keyArray "self" basicAt: index put: key.
                                        valueArray at: index put: 0.0d - aNumber.
                                        length - tally <= (length // 4) ifTrue: [self grow].
                                        ^aNumber]
                                ifFalse: [probe == key
                                                ifTrue: 
                                                        [answer := (valueArray at: index)
                                                                                - aNumber.
                                                        valueArray at: index put: answer.
                                                        ^aNumber]
                                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
!

at: i1 minusSparseVector: aSparseVector 
	| ioffset |
	ioffset := i1 - 1.
	aSparseVector keysAndValuesDo: [:ii :value | self at: ioffset + ii minusNumber: value]
!

equalSelfMinusSparseVector: aSparseVector 
	aSparseVector keysAndValuesDo: [:i :value | self at: i minusNumber: value]
!

equalSelfMinusSparseVector: aSparseVector times: aNumber 
	aSparseVector keysAndValuesDo: [:i :value | self at: i minusNumber: value * aNumber]
! !

!StMSparseVector methodsFor:'printing'!

outputOnFile: aString 
	| writeStream aij aijString |
	writeStream := aString asFilename writeStream.
	1 to: self nrow
		do: 
			[:i | 
			aij := self at: i.
			aij == nil ifTrue: [aij := 0.0d].
			aijString := aij printString.
			aijString last == $d
				ifTrue: [aijString replaceAll: $d with: Character space]
				ifFalse: [aijString replaceAll: $d with: $e].
			writeStream nextPutAll: aijString.
			writeStream cr].
	writeStream flush; close
!

printOn: aStream 
        "Append to the argument aStream a sequence of characters that identifies the receiver."

        | tooMany |
        tooMany := aStream position + self maxPrint.
        aStream print: self class.
        aStream space.
        aStream print: tally / keyArray "self" basicSize.
        aStream nextPutAll: ' ('.
        self
                associationsDo: 
                        [:element | 
                        aStream position > tooMany
                                ifTrue: 
                                        [aStream nextPutAll: '...etc...)'.
                                        ^self].
                        element printOn: aStream.
                        aStream space].
        aStream nextPut: $)
! !

!StMSparseVector methodsFor:'private'!

findKeyOrNil: key 
        "Look for the key in the receiver. If it is found, answer 
        the index of the association containting the key, otherwise 
        answer the index of the first unused slot."
        "key is anInteger."

        | index length |
        length := keyArray "self" basicSize.
        index := key \\ length + 1.
        [true]
                whileTrue: 
                        [| probe |
                        probe := keyArray "self" basicAt: index.
                        probe == DeletedEntry ifTrue:[^ index].
                        probe == nil
                                ifTrue: [^index]
                                ifFalse: [probe == key
                                                ifTrue: [^index]
                                                ifFalse: [(index := index + 1) > length ifTrue: [index := 1]]]]
! !

!StMSparseVector methodsFor:'removing'!

removeKey: key 
        "Remove key from the receiver. If key is not in the receiver, 
        do nothing and return nil. Otherwise, answer the value 
        associated with key."

        | index value k|

        index := self findKeyOrNil: key.
        Smalltalk isSmalltalkX ifTrue:[
            k := keyArray basicAt: index.
            k == DeletedEntry ifTrue:[k := nil].
        ] ifFalse:[
            k := self basicAt: index.
        ].
        ^ k == nil
                ifTrue: [nil]
                ifFalse: 
                        [value := valueArray at: index.
                         Smalltalk isSmalltalkX ifTrue:[
                            keyArray basicAt:index put:DeletedEntry
                         ] ifFalse:[
                            self removeAndCleanUpAtIndex: index.
                         ].
                        value]
! !

!StMSparseVector methodsFor:'testing'!

= otherCollection 
        "Answer whether the two sparse vectors are equal even in structure."

        | basicSize |
        self species == otherCollection species ifFalse: [^false].
        self tally = otherCollection tally ifFalse: [^false].
        self numberOfElements = otherCollection numberOfElements ifFalse: [^false].
        (basicSize := keyArray "self"  basicSize) = otherCollection keyArray basicSize ifFalse: [^false].
        1 to: basicSize do: [:index | (self at: index)
                        = (otherCollection at: index) ifFalse: [^false]].
        self valueArray = otherCollection valueArray ifFalse: [^false].
        ^true
!

isSequenceable
	"Answers true."

	^true
!

isZero
        | value |
        1 to: keyArray "self" basicSize
                do: 
                        [:index | 
                        value := valueArray at: index.
                        (value == nil or: [value isZero])
                                ifFalse: [^false]].
        ^true
! !

!StMSparseVector class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMSparseVector.st,v 1.8 2006/10/04 13:00:55 cg Exp $'
! !
