"{ Package: 'stx:goodies/math/stmath' }"

StMBackwardDifference subclass:#StMStartingBDF
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-StMath-DifferenceOperator'
!


!StMStartingBDF methodsFor:'accessing'!

iStep: integer 
	"iStep is the current step of interest."
	"iStep must increase consecutively."

	| iStepNew |
	iStepNew := iStep + 1.
	iStepNew == integer
		ifTrue: 
			[iStep := iStepNew.
			order := iStep + 1.
			nrow := iStep + 2]
		ifFalse: [self error: 'Not appropriate iStep.']
!

order: integer 
	"order is controlled by iStep."

	order == integer ifFalse: [self error: 'iStep and order must be consistent.']
! !

!StMStartingBDF methodsFor:'calc'!

derivative: i present: y past: ypast presentDerivative: ydot pastDerivative: ydotpast 
	| series |
	i == 0 ifTrue: [^y copy].
	series := StMArray new: nrow.
	series at: 1 put: y.
	2 to: nrow - 1 do: [:j | series at: j put: (ypast at: j - 1)].
	series at: nrow put: (ydotpast at: order - 1).
	^(operatorMatrix at: i + 1)
		dot: series
!

formTaylorMatrix
	"
	For method order 3: 
	|	1		0		0				0		|	|q(t)	|	=	|q(t)	| 
	|	1	(t1 - t)	(t1 - t)^2/2!!		(t1 - t)^3/3!!	|	|qd(t)	|		|q(t1)	| 
	|	1	(t2 - t)	(t2 - t)^2/2!!		(t2 - t)^3/3!!	|	|qdd(t)	|		|q(t2)	| 
	|	0		1		(t2 - t)		(t2 - t)^2/2!!	|	|qddd(t)|		|qd(t2)	| 
	"

	self instanciateTaylorMatrix.
	self formDegenerateTaylorRow: 1.
	2 to: nrow - 1 do: [:i | self
			formTaylorRow: i
			withTimeNode: i - 1
			derivative: 0].
	self
		formTaylorRow: nrow
		withTimeNode: order - 1
		derivative: 1
! !

!StMStartingBDF methodsFor:'initialize-release'!

initialize
	super initialize.
	iStep := 0.
	order := iStep + 1.
	nrow := iStep + 2
!

initializeLocally
	iStep := 0.
	order := iStep + 1.
	nrow := iStep + 2
! !

!StMStartingBDF class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/stmath/StMStartingBDF.st,v 1.6 2006/10/04 13:00:56 cg Exp $'
! !
