"{ Package: 'stx:goodies/math/stmath' }"
!

!Float class methodsFor:'StMath'!

arcTan0to2piY: y overX: x 
	"(y/x) arcTan in the range 0 to 2*pi."
	"Double arcTan0to2piY: 1.0d overX: 1.0d."

	^x > 0
		ifTrue: [y >= 0
				ifTrue: ["First quadrant."
					(y / x) arcTan]
				ifFalse: ["Forth quadrant."
					2.0d * Double pi + (y / x) arcTan]]
		ifFalse: [x < 0
				ifTrue: ["Second and Third quadrants."
					Double pi + (y / x) arcTan]
				ifFalse: ["x = 0"
					y > 0
						ifTrue: [Double pi / 2.0d]
						ifFalse: [y < 0
								ifTrue: [1.5d * Double pi]
								ifFalse: [self error: '0/0 arcTan is not defined.']]]]
! !

!Float class methodsFor:'StMath'!

arcTanMpiTOpiY: y overX: x 
	"(y/x) arcTan in the range -pi to pi."
	"Double arcTanMpiTOpiY: 1.0d overX: 1.0d."

	^x > 0
		ifTrue: ["First and Fourth quadrants."
			(y / x) arcTan]
		ifFalse: [x < 0
				ifTrue: [y >= 0
						ifTrue: ["Second quadrant."
							(y / x) arcTan + Double pi]
						ifFalse: ["Third quadrant."
							(y / x) arcTan - Double pi]]
				ifFalse: ["x = 0"
					y = 0
						ifTrue: [self error: '0/0 arcTan is not defined.']
						ifFalse: [y sign * Double pi / 2.0d]]]
! !

!Object methodsFor:'StMath'!

isReal
	"Answer whether the receiver is a Real number. This 
	means essentially that it's an instance of a subclass of 
	Number. Since many mathematical operations in the 
	system are designed only to work with the Real numbers, 
	it's convenient to be able to test for this property."

	^false
! !

!Object methodsFor:'StMath'!

raiseTranscript
        Transcript topView raise
! !

!SequenceableCollection methodsFor:'StMath'!

isDecreasing
	| next previous |
	self size < 2 ifTrue: [self error: 'Vector has less than two elements.'].
	next := self at: 1.
	2 to: self size
		do: 
			[:i | 
			previous := next.
			next := self at: i.
			next < previous ifFalse: [^false]].
	^true
! !

!SequenceableCollection methodsFor:'StMath'!

isDecreasingIfExceptionsAreLessThan: double 
	"Test if elements are decreasing."
	"Ok if spoilers are less than double."

	| next previous |
	self size < 2 ifTrue: [self error: 'Vector has less than two elements.'].
	next := self at: 1.
	2 to: self size
		do: 
			[:i | 
			previous := next.
			next := self at: i.
			next < previous ifFalse: [next < double ifFalse: [^false]]].
	^true
! !

!SequenceableCollection methodsFor:'StMath'!

isIncreasing
	| next previous |
	self size < 2 ifTrue: [self error: 'Vector has less than two elements.'].
	next := self at: 1.
	2 to: self size
		do: 
			[:i | 
			previous := next.
			next := self at: i.
			next > previous ifFalse: [^false]].
	^true
! !

!String methodsFor:'StMath'!

cleanSourceFromSelf
	"Return a version of aString which has a cr line-end convention imposed upon it. 
	If the stream actually contains lf, cr-lf or (worse) lf-cr line-ends, replace each 
	single occurrence of a line-end with a single cr. This is to help alleviate problems 
	caused by moving source files between machines with different line-end conventions."

	| lf cr cleanStream prev |
	cleanStream := (String new: self size) writeStream.
	lf := Character lf.
	cr := Character cr.
	prev := nil.
	1 to: self size
		do: 
			[:i | 
			| ch |
			ch := self at: i.
			lf == ch
				ifTrue: [cr == prev
						ifTrue: [prev := nil]
						ifFalse: 
							[cleanStream nextPut: cr.
							prev := lf]]
				ifFalse: [cr == ch
						ifTrue: [lf == prev
								ifTrue: [prev := nil]
								ifFalse: [prev := cleanStream nextPut: ch]]
						ifFalse: [prev := cleanStream nextPut: ch]]].
	^cleanStream contents
! !

!String methodsFor:'StMath'!

expandWithArgument: string 
	"Substitute %1 with string."

	| stream buffer |
	stream := self readStream.
	buffer := (String new: 256) writeStream.
	[stream atEnd]
		whileFalse: 
			[| char |
			char := stream next.
			char = $%
				ifTrue: [(Integer readFrom: stream)
						== 1 ifTrue: [buffer nextPutAll: string]]
				ifFalse: [buffer nextPut: char]].
	^buffer contents
! !

!String methodsFor:'StMath'!

expandWithArguments: arrayOfStrings 
	"Substitute %i with the ith elements of arrayOfStrings."

	| stream buffer |
	stream := self readStream.
	buffer := (String new: 256) writeStream.
	[stream atEnd]
		whileFalse: 
			[| char |
			char := stream next.
			char = $%
				ifTrue: 
					[| i |
					i := Integer readFrom: stream.
					i > 0 & (i <= arrayOfStrings size)
						ifTrue: 
							[| string |
							string := arrayOfStrings at: i.
							buffer nextPutAll: string]]
				ifFalse: [buffer nextPut: char]].
	^buffer contents
! !

!String methodsFor:'StMath'!

format: string 
	"Return string with FORTRAN format."

	| stream number answer nBlanks |
	stream := string readStream.
	stream next == $A ifFalse: [self error: 'Wrong format for string.'].
	number := Integer readFrom: stream.
	answer := self.
	nBlanks := number - answer size.
	nBlanks < 0
		ifTrue: 
			[answer := ''.
			number timesRepeat: [answer := answer , '*']]
		ifFalse: [nBlanks timesRepeat: [answer := answer , ' ']].
	^answer
! !

!String methodsFor:'StMath'!

mbdMessage
	"Detect prefix 'MbD' or 'ADM'."
	"Then skip pass the first blank space."
	"Then convert all \'s to cr's."

	| prefix stream buffer char |
	^(self size > 2
		and: 
			[prefix := self copyFrom: 1 to: 3.
			(prefix = 'ADM' or: [prefix = 'MbD'])
				or: [prefix = 'StM']])
		ifTrue: 
			[stream := self readStream.
			stream skipThrough: Character space.
			buffer := (String new: 256) writeStream.
			[stream atEnd]
				whileFalse: 
					[char := stream next.
					buffer nextPut: (char = $\
							ifTrue: [Character cr]
							ifFalse: [char])].
			buffer contents]
		ifFalse: [self withCRs]
! !

!String methodsFor:'StMath'!

mbdMessageWithArgument: string 
	"Remove prefix 'MbD'."
	"Substitute %1 with string."
	"And convert \ to cr."

	| prefix |
	^(self size > 2
		and: 
			[prefix := self copyFrom: 1 to: 3.
			(prefix = 'ADM' or: [prefix = 'MbD'])
				or: [prefix = 'StM']])
		ifTrue: 
			[| stream buffer |
			stream := self readStream.
			stream skipThrough: Character space.
			buffer := (String new: 256) writeStream.
			[stream atEnd]
				whileFalse: 
					[| char |
					char := stream next.
					char = $%
						ifTrue: [(Integer readFrom: stream)
								= 1 ifTrue: [buffer nextPutAll: string]]
						ifFalse: [buffer nextPut: (char = $\
									ifTrue: [Character cr]
									ifFalse: [char])]].
			buffer contents]
		ifFalse: [self withCRs expandWithArgument: string]
! !

!String methodsFor:'StMath'!

mbdMessageWithArguments: arrayOfStrings 
	"Remove prefix 'MbD'."
	"Substitute %i with the ith elements of arrayOfStrings."
	"Then convert \ to cr."

	| prefix |
	^(self size > 2
		and: 
			[prefix := self copyFrom: 1 to: 3.
			(prefix = 'ADM' or: [prefix = 'MbD'])
				or: [prefix = 'StM']])
		ifTrue: 
			[| stream buffer |
			stream := self readStream.
			stream skipThrough: Character space.
			buffer := (String new: 100) writeStream.
			[stream atEnd]
				whileFalse: 
					[| char |
					char := stream next.
					char = $%
						ifTrue: 
							[| i |
							i := Integer readFrom: stream.
							i > 0 & (i <= arrayOfStrings size) ifTrue: [buffer nextPutAll: (arrayOfStrings at: i)]]
						ifFalse: [buffer nextPut: (char = $\
									ifTrue: [Character cr]
									ifFalse: [char])]].
			buffer contents]
		ifFalse: [self withCRs expandWithArguments: arrayOfStrings]
! !

!String methodsFor:'StMath'!

rcMessage
	"Detect prefix 'ADM' or 'MbD' or 'StM'."
	"Then skip pass the first blank space."
	"Then convert all \'s to \n's."

	| prefix stream buffer char |
	prefix := self copyFrom: 1 to: 4.
	((prefix = 'ADM:' or: [prefix = 'MbD:'])
		or: [prefix = 'StM:'])
		ifTrue: 
			[stream := self readStream.
			stream skipThrough: Character space.
			buffer := (String new: 100) writeStream.
			[stream atEnd]
				whileFalse: 
					[char := stream next.
					char = $\
						ifTrue: [buffer nextPutAll: '\n']
						ifFalse: [buffer nextPut: char]].
			^buffer contents]
		ifFalse: [self error: 'Invalid message.']
! !

