"{ Package: 'stx:goodies/math/turingMachine' }"

"{ NameSpace: Math }"

Object subclass:#TuringAction
	instanceVariableNames:'write move state'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-TuringMachine'
!

!TuringAction class methodsFor:'documentation'!

documentation
"
    My instances are actions that may be performed by a Turing machine. 
    Each action consists of a value to write, a state to change to 
    and a movement to right or to the left of the tape's head.
"
! !

!TuringAction class methodsFor:'instance creation'!

write: anObject state: aSymbol move: anInteger
	^self new write: anObject; state: aSymbol; move: anInteger
! !

!TuringAction methodsFor:'accessing'!

move
	^move
!

move: anInteger
	move _ anInteger
!

state
	^state
!

state: aSymbol
	state _ aSymbol
!

write
	^write
!

write: aSymbol
	write _ aSymbol
! !

!TuringAction methodsFor:'comparing'!

= anAction
	" public - two actions are the same if they match in write value, state and movement"
	^self write = anAction write
	and: [self state = anAction state
	and: [self move = anAction move]]
!

hash
	^(self write hash + self state hash + self move hash) hash
! !

!TuringAction methodsFor:'printing'!

printOn: aStream
	" public - will show as '(write: aValue; state: aState; move: aMovement)'"
	aStream
		nextPutAll: '(write: ';
		nextPutAll: self write printString;
		nextPutAll: '; state: ';
		nextPutAll: self state printString;
		nextPutAll: '; move: ';
		nextPutAll: self move printString;
		nextPut: $) 
! !

!TuringAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/turingMachine/TuringAction.st,v 1.1 2002/11/22 16:45:34 cg Exp $'
! !
