"{ Package: 'stx:goodies/math/turingMachine' }"

"{ NameSpace: Math }"

TuringProgram subclass:#TuringMachine
	instanceVariableNames:'tape state'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-TuringMachine'
!

!TuringMachine class methodsFor:'documentation'!

documentation
"
    My instances are Turing machines. 
    A Turing machine may be programmed and the program may be run on some input. 
    The programming language consists of rule. 
    A program may loop forever or end by change movng into state #halt.
    Once a program is loaded in the machine, a set of data can be used as input 
    by sending the message #input: to the machine.

    see example and adder for some examples on how to program a TuringMachine.

    Or if you want to Type TuringMachine example (on en WrapperWorldMorph) and press enter.
"
! !

!TuringMachine class methodsFor:'examples'!

adder
        ^ self new
                when: (TuringCondition state: #start value: 0)
                do: (TuringAction write: 0 state: 0 move: 1);
                
                when: (TuringCondition state: #start value: 1)
                do: (TuringAction write: 1 state: 1 move: 1);
                
                when: (TuringCondition state: 0 value: 0)
                do: (TuringAction write: 0 state: 2 move: 1);
                
                when: (TuringCondition state: 0 value: 1)
                do: (TuringAction write: 1 state: 3 move: 1 );

                when: (TuringCondition state: 1 value: 0)
                do: (TuringAction write: 0 state: 3 move: 1);

                when: (TuringCondition state: 1 value: 1)
                do: (TuringAction write: 1 state: 4 move: 1);

                when: (TuringCondition state: 2 value: #nil)
                do: (TuringAction write: 0 state: #halt move: 1);

                when: (TuringCondition state: 3 value: #nil)
                do: (TuringAction write: 1 state: #halt move: 1);

                when: (TuringCondition state: 4 value: #nil)
                do: (TuringAction write: 0 state: 3 move: 1);

                yourself.
!

example
        ^self new
                when: (TuringCondition state: #start value: $a)
                do: (TuringAction write: $A state: 2 move: 1);

                when: (TuringCondition state: #start value: #nil)
                do: (TuringAction write: #nil state: #halt move: 1);

                when: (TuringCondition state: 2 value: $b)
                do: (TuringAction write: $B state: 3 move: -1);

                when: (TuringCondition state: 2 value: $B)
                do: (TuringAction write: $B state: 2 move: 1);

                when: (TuringCondition state: 2 value: $a)
                do: (TuringAction write: $a state: 2 move: 1);

                when: (TuringCondition state: 3 value: $A)
                do: (TuringAction write: $A state: 5 move: 1);

                when: (TuringCondition state: 3 value: $B)
                do: (TuringAction write: $B state: 3 move: -1);

                when: (TuringCondition state: 3 value: $a)
                do: (TuringAction write: $a state: 4 move: -1);

                when: (TuringCondition state: 4 value: $a)
                do: (TuringAction write: $a state: 4 move: -1);

                when: (TuringCondition state: 4 value: $A)
                do: (TuringAction write: $A state: #start move: 1);

                when: (TuringCondition state: 5 value: #nil)
                do: (TuringAction write: #equalSizes state: #halt move: -1);

                when: (TuringCondition state: 5 value: $b)
                do: (TuringAction write: #differentSizes state: #halt move: -1);

                when: (TuringCondition state: 5 value: $B)
                do: (TuringAction write: $B state: 5 move: 1);

                input: 'aabbb';

                yourself
!

showExample
	| turing |
	turing _ TuringMachine example.
	turing input: 'aaaabbbb'.
	turing showDelay: 500.
! !

!TuringMachine methodsFor:'accessing'!

morphicWrapperClass
	" public - outliner interface. Answer my best outliner's Class"
	^ Smalltalk at: #TuringMachineMorphicWrapper
!

state
	" public - answer my current state"
	state ifNil: [self initializeState].
	^state
!

state: aSymbol
	" set my current state. My be using for debugging a TuringProgram"
	state _ aSymbol
!

tape
	tape ifNil: [self initializeTape].
	^ tape
!

tape: aTape
	tape _ aTape
! !

!TuringMachine methodsFor:'converting'!

asProgram
	^ TuringProgram new vertices: self vertices
! !

!TuringMachine methodsFor:'data loading'!

input: aCollection
        " public - load data in my Tape, and change state to #start, so step method will work fine"
        self tape: (TuringTape on: aCollection).
        self state: (self vertexForName: #start)
! !

!TuringMachine methodsFor:'executing'!

move: anInteger
	" public - Move my Tape's read/write head
	  forward the message to my Tape"
	self tape move: anInteger
!

read
	" public - Read what my Tape's read/write head is pointing to.
	  forward the message to my Tape"
	^self tape read
!

step
        | action |
        action _ self apply: (TuringCondition value: self read state: self state).
        self
                write: action write;
                state: action state;
                move: action move.
!

write: anObject
	" public - Write an Object to my Tape's.
	  forward the message to my Tape"
	self tape write: anObject
! !

!TuringMachine methodsFor:'initialization'!

initializeState
	" start in #halt state"
	self state: (self vertexForName: #halt).
!

initializeTape
	" start with an almost empty Tape.
	  The #nil (insted) of nil is becouse Set (for #dictionary) doesn't allow addition of nil"
	self input: #( #nil )
! !

!TuringMachine methodsFor:'printing'!

printOn: aStream
	" public - I will show up in two lines, first containing 'state: aState',
	  and second showing my Tape"
	aStream nextPutAll: 'state: ', self state printString; cr.
	self tape printOn: aStream 
!

showDelay: aDelayTime
	| process delay |
	delay _ Delay forMilliseconds: aDelayTime.
	process _ 
		[[Transcript show: self printString; cr.
		self state = #halt] whileFalse: [delay wait. self step].
		delay unschedule] newProcess.
	process priority: Processor userInterruptPriority.
	process resume
!

showOn: aStream delay: aDelayTime
	| process delay |
	delay _ Delay forMilliseconds: aDelayTime.
	process _ 
		[[aStream nextPutAll: self printString; cr.
		self state = #halt] whileFalse: [delay wait. self step].
		delay unschedule] newProcess.
	process priority: Processor userInterruptPriority.
	process resume
! !

!TuringMachine class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/turingMachine/TuringMachine.st,v 1.1 2002/11/22 16:45:29 cg Exp $'
! !
