"{ Package: 'stx:goodies/math/turingMachine' }"

"{ NameSpace: Math }"

SingleEdgedGraph subclass:#TuringProgram
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Math-TuringMachine'
!

!TuringProgram class methodsFor:'documentation'!

documentation
"
    My instances are programs for Turing machines. 
    You can ask them it's states and dictionary
"
! !

!TuringProgram methodsFor:'accessing'!

actions
	" public - answer a Set off al the actions I can take"
	^ self weights collect: [:eachWeight | eachWeight action]
!

conditions
        " public - answer a Set off al the conditions I understand"
        ^ (self edges collect: [:eachEdge | TuringCondition state: eachEdge from name value: eachEdge weight readValue]) asSet
!

dictionary
	" public - answer the Set of all output and input values I can produce and understand"
	^ self inputDictionary addAll: self outputDictionary ; yourself
!

inputDictionary
	" public - answer the Set of all input values I understand"
	^ Set new addAll: (self weights collect: [:eachWeight | eachWeight readValue]).
!

inputStates
	" public - answer the Set of all posible states from which I can take an action from"
	^ (self conditions collect: [:eachCondition | eachCondition state]) asSet.
!

outputDictionary
	" public - answer the Set of all output values I can produce"
	^ Set new addAll: (self weights collect: [:eachWeight | eachWeight writeValue]).
!

outputStates
	" public - answer the Set of all posible state which I can take an action to"
	^ (self actions collect: [:eachActions | eachActions state]) asSet.
!

states
	" public - answer all posible states in which I can be, that's all I understand as a Condition
	  or all I can answer as an Action"
	^ self inputStates addAll: self outputStates; yourself
! !

!TuringProgram methodsFor:'executing'!

apply: aCondition
        " public - if there's a proper rule, answer the Action corresponding to aCondition,
          otherway answer nil"
        | competentArrow |
        aCondition state = #halt ifTrue: [^ TuringAction write: aCondition value state: #halt move: 0].
        competentArrow _ (self vertexForName: aCondition state) arrows detect: [:one |
                 one weight readValue = aCondition value].
        ^TuringAction
                write: competentArrow weight writeValue
                state: competentArrow to
                move: competentArrow weight movement.
! !

!TuringProgram methodsFor:'programing'!

when: aCondition do: anAction
	" public - add the given rule to the program.	r++ 1/17/98 16:26"
	^ self addArrowFrom: aCondition state
		to: anAction state
		weighted: (
			TuringWeight
				whenRead: aCondition value
				do: anAction)
! !

!TuringProgram class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/turingMachine/TuringProgram.st,v 1.1 2002/11/22 16:45:01 cg Exp $'
! !
