"{ Package: 'stx:goodies/math/turingMachine' }"

"{ NameSpace: Math }"

Object subclass:#TuringTape
	instanceVariableNames:'cell contents origin'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-TuringMachine'
!

!TuringTape class methodsFor:'documentation'!

documentation
"
    My instances are recording tapes. A tape has cells and a read/write head. 
    The head acts on the current cell by reading or writing objects on that cell. 
    The tape can grow to the right and to the left. 
"
! !

!TuringTape class methodsFor:'instance creation'!

new
	^super new initialize
!

on: aCollection
	^self new contents: aCollection asOrderedCollection
! !

!TuringTape methodsFor:'printing'!

printOn: aStream
	" public - I will whow up as 'anElement bElement [cElement] dElement ...',
	  the read/write head is pointing to cElement"
	| first |
	aStream nextPut: $(.
	first _ true.
	1 to: self contents size do: [:each |
		first ifTrue: [first _ false] ifFalse: [aStream nextPut: $ ].
		each = self cellPosition
			ifTrue:
				[aStream nextPut: $[.
				(self contents at: each) printOn: aStream.
				aStream nextPut: $]]
			ifFalse: [(self contents at: each) printOn: aStream]].
	aStream nextPut: $)
! !

!TuringTape methodsFor:'private'!

cell
	^cell
!

cell: anInteger
	cell _ anInteger
!

cellPosition
	" private - answer the position of my cell in my content's Collection"
	^self cell + self origin
!

contents
	^contents
!

contents: aCollection 
	contents _ aCollection isEmpty
				ifTrue: [aCollection species with: nil]
				ifFalse: [aCollection]
!

initialize
	self origin: 1.
	self rewind
!

origin
	" private - origin is the index of the starting position of my read/write head
	  in my content's collection, as elements are prepended to my contents, the origin
	  is incremented"
	^origin
!

origin: anInteger
	origin _ anInteger
! !

!TuringTape methodsFor:'public'!

backward
	" public - move backward one position my read/write head"
	self cellPosition = 1
		ifTrue:
			[self contents addFirst: #nil.
			self origin: self origin + 1].
	self cell: self cell - 1
!

foreward
	" public - move foreward one position my read/write head"
	self cellPosition = self contents size ifTrue: [self contents add: #nil].
	self cell: self cell + 1
!

move: anInteger
	" public - 1 moves foreward, -1 moves backward, 0 no movement is done"
	anInteger = 0 ifTrue: [^ self].
	anInteger > 0
		ifTrue: [anInteger timesRepeat: [self foreward]]
		ifFalse: [anInteger negated timesRepeat: [self backward]]
!

playOn: aStream tempo: someMilliseconds
	| delay process note |
	delay _ Delay forMilliseconds: someMilliseconds.
	process _ [
		[(note _ self read) isNil] whileFalse:
			[delay wait.
			aStream nextPut: note.
			self foreward].
		delay unschedule].
	process forkAt: Processor userInterruptPriority
!

read
	" public - answer what my read write head is pointing at"
	^self contents at: self cellPosition
!

rewind
	" public - move my read write head to the starting position"
	self cell: 0
!

showOnTranscriptTempo: someMilliseconds
	| delay process note |
	delay _ Delay forMilliseconds: someMilliseconds.
	process _ [
		[(note _ self read) isNil] whileFalse:
			[delay wait.
			Transcript nextPut: note; endEntry.
			self foreward].
		delay unschedule].
	process forkAt: Processor userInterruptPriority
!

write: anObject
	" public - write anObject to the place my read/write head is pointing to"
	^self contents at: self cellPosition put: anObject
! !

!TuringTape class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/turingMachine/TuringTape.st,v 1.1 2002/11/22 16:45:23 cg Exp $'
! !
