"{ Package: 'stx:goodies/measurement' }"

Unit subclass:#DerivedUnit
	instanceVariableNames:'name baseUnit conversion'
	classVariableNames:''
	poolDictionaries:''
	category:'Measurements'
!


!DerivedUnit class methodsFor:'class initialization'!

addUnitNamed: aString conversion: aNumber base: aBaseUnitString 
	UnitAllocationLock critical: [self addUnit: (DerivedUnit new
				setName: aString
				key: self nextPrime
				conversion: aNumber
				base: (Unit of: aBaseUnitString))]
!

installAreas
	self
		addUnitNamed: 'acre'
		conversion: 40468564224 / 10000000
		base: 'm2'.
	self
		addUnitNamed: 'are'
		conversion: 100
		base: 'm2'.
	self
		addUnitNamed: 'hectare'
		conversion: 10000
		base: 'm2'.
	self
		addUnitNamed: 'rood'
		conversion: 10117141056 / 10000000
		base: 'm2'.
	self
		addUnitNamed: 'square'  "of timber"
		conversion: 9290304 / 1000000
		base: 'm2'.
	self
		addUnitNamed: 'township'
		conversion: 93239571972 / 1000
		base: 'm2'
!

installDerivedUnits
	self installLengths; installTimes; installMasses; installTemperatures; installVolumes; installAreas; installEnergies; installForces; installPowers; installPressures; installSpeeds; installRates; installElectrics; installPrefixedUnits
!

installElectrics
	self
		addUnitNamed: 'V'
		conversion: 1
		base: 'kg.m2/s3.A'.
	self
		addUnitNamed: 'ohm'
		conversion: 1
		base: 'kg.m2/s3.A2'.
	self
		addUnitNamed: 'F'
		conversion: 1
		base: 's4.A2/kg.m2'.
	self
		addUnitNamed: 'C'
		conversion: 1
		base: 'A.s'.
	self
		addUnitNamed: 'Wb'
		conversion: 1
		base: 'kg.m2/s2.A'.
	self
		addUnitNamed: 'H'
		conversion: 1
		base: 'kg.m2/s2.A2'.
	self
		addUnitNamed: 'T'
		conversion: 1
		base: 'kg/s2.A'
!

installEnergies
	self
		addUnitNamed: 'J'
		conversion: 1
		base: 'kg.m2/s2'.
	self
		addUnitNamed: 'bTU'
		conversion: 100 / 105575
		base: 'kg.m2/s2'.
	self
		addUnitNamed: 'cal'
		conversion: 10000 / 41868
		base: 'kg.m2/s2'.
	self
		addUnitNamed: 'erg'
		conversion: 1 / 10000000
		base: 'kg.m2/s2'.
	self
		addUnitNamed: 'therm'
		conversion: 105575000
		base: 'kg.m2/s2'.
	self
		addUnitNamed: 'cHU'
		conversion: 19004 / 10
		base: 'kg.m2/s2'
!

installForces
	self
		addUnitNamed: 'N'
		conversion: 1
		base: 'kg.m/s2'.
	self
		addUnitNamed: 'dyne'
		conversion: 1 / 100000
		base: 'kg.m/s2'.
	self
		addUnitNamed: 'lbf'
		conversion: 4448222 / 1000000
		base: 'kg.m/s2'.
	self
		addUnitNamed: 'kip'
		conversion: 4448222 / 1000
		base: 'kg.m/s2'.
	self
		addUnitNamed: 'poundal'
		conversion: 138255 / 1000000
		base: 'kg.m/s2'
!

installLengths
	self
		addUnitNamed: 'yd'
		conversion: 9144 / 10000
		base: 'm'. self addUnitNamed: 'nmi' conversion: '1852' base: 'm'.
	self
		addUnitNamed: 'ft'
		conversion: 3048 / 10000
		base: 'm'.
	self
		addUnitNamed: 'fathom'
		conversion: 18288 / 10000
		base: 'm'.
	self
		addUnitNamed: 'furlong'
		conversion: 201168 / 1000
		base: 'm'.
	self
		addUnitNamed: 'in'
		conversion: 254 / 10000
		base: 'm'.
	self
		addUnitNamed: 'lightyear'
		conversion: 9460500000000000
		base: 'm'.
	self
		addUnitNamed: 'mi'
		conversion: 1609344 / 1000
		base: 'm'.
	self
		addUnitNamed: 'parsec'
		conversion: 30856770000000000
		base: 'm'.
	self
		addUnitNamed: 'angstrom'
		conversion: 1 / 10000000000
		base: 'm'.
	self
		addUnitNamed: 'AU'
		conversion: 149598550000
		base: 'm'.
	self
		addUnitNamed: 'barleycorn'
		conversion: 8467 / 1000000
		base: 'm'.
	self
		addUnitNamed: 'chain'
		conversion: 201168 / 10000
		base: 'm'.
	self
		addUnitNamed: 'cubit'
		conversion: 45 / 100
		base: 'm'.
	self
		addUnitNamed: 'ell'
		conversion: 875 / 1000
		base: 'm'.
	self
		addUnitNamed: 'hand'
		conversion: 106 / 1000
		base: 'm'.
	self
		addUnitNamed: 'link'
		conversion: 201168 / 1000000
		base: 'm'.
	self
		addUnitNamed: 'point'
		conversion: 352777 / 1000000000
		base: 'm'.
	self
		addUnitNamed: 'pica'
		conversion: 4233333 / 1000000000
		base: 'm'.
	self
		addUnitNamed: 'perch'
		conversion: 50292 / 10000
		base: 'm'.
	self
		addUnitNamed: 'rod'
		conversion: 50292 / 10000
		base: 'm'.
	self
		addUnitNamed: 'pole'
		conversion: 50292 / 10000
		base: 'm'
!

installMasses
	self
		addUnitNamed: 'g'
		conversion: 1 / 1000
		base: 'kg'.
	self
		addUnitNamed: 'lbm'
		conversion: 45359237 / 100000000
		base: 'kg'.
	self
		addUnitNamed: 'oz'
		conversion: 28349523125 / 1000000000000
		base: 'kg'.
	self
		addUnitNamed: 'carat'
		conversion: 2 / 10000
		base: 'kg'.
	self
		addUnitNamed: 'grain'
		conversion: 6479891 / 100000000000
		base: 'kg'.
	self
		addUnitNamed: 'stone'
		conversion: 635029318 / 100000000
		base: 'kg'.
	self
		addUnitNamed: 'slug'
		conversion: 68521 / 1000000
		base: 'kg'.
	self
		addUnitNamed: 'ton'
		conversion: 90718474 / 100000
		base: 'kg'.
	self
		addUnitNamed: 'tonne'
		conversion: 1000
		base: 'kg'
!

installPowers
	self
		addUnitNamed: 'W'
		conversion: 1
		base: 'kg.m2/s3'.
	self
		addUnitNamed: 'hp'
		conversion: 7457 / 10
		base: 'kg.m2/s3'
!

installPrefixedUnits
	| prefixes |
	prefixes := self unitPrefixes.
	#('m' 's' 'A' 'mol' 'Hz' 'J' 'W' 'N' 'Pa' 'V') do: [:each | prefixes
			keysAndValuesDo: 
				[:pre :scale | 
				| unit |
				unit := Unit of: each.
				self
					addUnitNamed: pre , each
					conversion: scale * unit conversion
					base: unit baseUnit name]].
	(prefixes removeKey: 'k'; yourself) keysAndValuesDo: [:pre :scale | self
			addUnitNamed: pre , 'g'
			conversion: scale / 1000
			base: 'kg']
!

installPressures
	self
		addUnitNamed: 'Pa'
		conversion: 1
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'atm'
		conversion: 101325
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'bar'
		conversion: 100000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'cm-Hg'
		conversion: 133322 / 100
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'ft-H20'
		conversion: 298906692 / 100000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'in-H20'
		conversion: 24908891 / 100000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'in-Hg'
		conversion: 3386388 / 1000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'mm-Hg'
		conversion: 133322 / 1000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'psi'
		conversion: 6894757 / 1000
		base: 'kg/m.s2'.
	self
		addUnitNamed: 'ksi'
		conversion: 6894757
		base: 'kg/m.s2'
!

installRates
	self
		addUnitNamed: 'Hz'
		conversion: 1
		base: '/s'
!

installSpeeds
	self
		addUnitNamed: 'mph'
		conversion: 44704 / 100000
		base: 'm/s'.
	self
		addUnitNamed: 'Mach'
		conversion: 331.5
		base: 'm/s'.
	self
		addUnitNamed: 'knot'
		conversion: 514444 / 1000000
		base: 'm/s'
!

installTemperatures
	self
		addUnitNamed: 'C'
		conversion: 1
		base: 'K'.
	self
		addUnitNamed: 'F'
		conversion: 5 / 9
		base: 'K'.
	self
		addUnitNamed: 'R'
		conversion: 5 / 9
		base: 'K'
!

installTimes
	self
		addUnitNamed: 'min'
		conversion: 60
		base: 's'.
	self
		addUnitNamed: 'hr'
		conversion: 3600
		base: 's'.
	self
		addUnitNamed: 'd'
		conversion: 24 * 3600
		base: 's'.
	self
		addUnitNamed: 'wk'
		conversion: 24 * 3600 * 7
		base: 's'.
	self
		addUnitNamed: 'y'
		conversion: 315569259747 / 10000
		base: 's'.
	self
		addUnitNamed: 'decade'
		conversion: 315569259747 / 1000
		base: 's'.
	self
		addUnitNamed: 'century'
		conversion: 315569259747 / 100
		base: 's'.
	self
		addUnitNamed: 'millennium'
		conversion: 315569259747 / 10
		base: 's'
!

installVolumes
	"where conflicts go with US rather than UK, and liquid instead of dry. The extra / 1000 is because the tables list them relevant to Liters"

	self
		addUnitNamed: 'L'
		conversion: 1 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'barrel'
		conversion: 158987294928 / 1000000000 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'bushel'
		conversion: 3523907016688 / 100000000000 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'fl-oz'
		conversion: 295735345625 / 10000000000000 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'gal'
		conversion: 3785411784 / 1000000000 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'pt'
		conversion: 473176473 / 1000000000 / 1000
		base: 'm3'.
	self
		addUnitNamed: 'qt'
		conversion: 946352946 / 1000000000 / 1000
		base: 'm3'
!

unitPrefixes
	| dict |
	dict := Dictionary new.
	dict at: 'Y' put: 10 ** 24.
	dict at: 'Z' put: 10 ** 21.
	dict at: 'E' put: 10 ** 18.
	dict at: 'P' put: 10 ** 15.
	dict at: 'T' put: 10 ** 12.
	dict at: 'G' put: 10 ** 9.
	dict at: 'M' put: 10 ** 6.
	dict at: 'k' put: 10 ** 3.
	dict at: 'm' put: 10 ** -3.
	dict at: 'u' put: 10 ** -6.
	dict at: 'n' put: 10 ** -9.
	dict at: 'p' put: 10 ** -12.
	dict at: 'f' put: 10 ** -15.
	dict at: 'a' put: 10 ** -18.
	dict at: 'z' put: 10 ** -21.
	dict at: 'y' put: 10 ** -24.
	^dict
! !

!DerivedUnit methodsFor:'accessing'!

baseUnit
	^baseUnit
!

conversion
	^conversion
!

key
	^key
!

name
	^name
! !

!DerivedUnit methodsFor:'printing'!

printOn: aStream 
	aStream
		nextPutAll: 'Unit of: ';
		print: name
! !

!DerivedUnit methodsFor:'private-initialize'!

setName: aString key: aPrimeInteger conversion: aFactor base: aBaseUnit 
	name := aString.
	key := aPrimeInteger.
	conversion := aFactor.
	baseUnit := aBaseUnit
! !

!DerivedUnit class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/DerivedUnit.st,v 1.3 2002/11/15 12:17:27 cg Exp $'
! !
