"{ Package: 'stx:goodies/measurement' }"

ArithmeticValue subclass:#Measurement
	instanceVariableNames:'magnitude unit'
	classVariableNames:''
	poolDictionaries:''
	category:'Measurements'
!


!Measurement class methodsFor:'instance creation'!

magnitude: aNumeric unit: aUnit 
	^self basicNew setMagnitude: aNumeric unit: aUnit
!

readFrom: aStream onError: aBlock
	| num unitString |
	num := Number readFrom: aStream onError: [^aBlock value].
	unitString := aStream upToEnd reject: [:char | char isSeparator ].
	^self magnitude: num unit: (Unit of: unitString)
! !

!Measurement methodsFor:'accessing'!

baseMagnitude
	^magnitude * self canonicalUnit conversion
!

baseUnit
	^unit baseUnit
!

canonicalUnit
	^unit canonicalUnit
!

magnitude
	^magnitude
!

unit
        ^unit
! !

!Measurement methodsFor:'arithmetic'!

* aMeasurement 
        ^Measurement magnitude: magnitude * aMeasurement magnitude
                unit: unit * aMeasurement unit

        "
         (2 ~> 'm') * 3    
         2 * (3 ~> 'm')    
         (2 ~> 'm') * (3 ~> 'm')    
        "
!

+ aMeasurement 
        | myUnit hisUnit |
        unit = aMeasurement unit 
                ifTrue: 
                        [^Measurement magnitude: magnitude + aMeasurement magnitude unit: unit].
        myUnit := unit canonicalUnit.
        hisUnit := aMeasurement canonicalUnit.
        myUnit baseUnit == hisUnit baseUnit 
                ifFalse: [^self incompatibleUnitsError].
        ^Measurement magnitude: magnitude 
                                + (aMeasurement magnitude * hisUnit conversion / myUnit conversion)
                unit: myUnit

        "
         (2 ~> 'm') + (3 ~> 'm')    
         (2 ~> 'm') + (3 ~> 'ft')    
        "
!

- aMeasurement 
        | myUnit hisUnit |
        unit = aMeasurement unit 
                ifTrue: 
                        [^Measurement magnitude: magnitude - aMeasurement magnitude unit: unit].
        myUnit := unit canonicalUnit.
        hisUnit := aMeasurement canonicalUnit.
        myUnit baseUnit == hisUnit baseUnit 
                ifFalse: [^self incompatibleUnitsError].
        ^Measurement magnitude: magnitude 
                                - (aMeasurement magnitude * hisUnit conversion / myUnit conversion)
                unit: myUnit

        "
         (3 ~> 'm') - (2 ~> 'm')    
         (2 ~> 'm') - (3 ~> 'ft')    
        "
!

/ aMeasurement 
        ^Measurement magnitude: magnitude / aMeasurement magnitude
                unit: unit / aMeasurement unit

        "
         (2 ~> 'm') / 3             
         2 / (3 ~> 'm')             
         (2 ~> 'm') / (3 ~> 'm')   
        "
!

negated
	^Measurement magnitude: magnitude negated unit: unit
!

reciprocal
	^Measurement magnitude: magnitude reciprocal unit: unit reciprocal
! !

!Measurement methodsFor:'comparing'!

< aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude < aMeasurement magnitude].
	unit canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^self incompatibleUnitsError].
	^self baseMagnitude < aMeasurement baseMagnitude
!

<= aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude <= aMeasurement magnitude].
	unit canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^self incompatibleUnitsError].
	^self baseMagnitude <= aMeasurement baseMagnitude
!

= aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude = aMeasurement magnitude].
	self canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^false].
	^self baseMagnitude = aMeasurement baseMagnitude
!

> aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude > aMeasurement magnitude].
	unit canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^self incompatibleUnitsError].
	^self baseMagnitude > aMeasurement baseMagnitude
!

>= aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude >= aMeasurement magnitude].
	unit canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^self incompatibleUnitsError].
	^self baseMagnitude >= aMeasurement baseMagnitude
!

hash
	^self baseMagnitude hash + self baseUnit hash
!

~= aMeasurement 
	unit = aMeasurement unit ifTrue: [^magnitude ~= aMeasurement magnitude].
	unit canonicalUnit baseUnit == aMeasurement canonicalUnit baseUnit 
		ifFalse: [^self incompatibleUnitsError].
	^self baseMagnitude ~= aMeasurement baseMagnitude
! !

!Measurement methodsFor:'converting'!

@ aNumeric 
	^Point x: self y: aNumeric
!

asPoint
	^self @ self
!

~> aUnitString 
	^(Unit of: aUnitString) convert: self
! !

!Measurement methodsFor:'double dispatching stuff'!

coerce: anArithmeticValue 
        ^Measurement magnitude: anArithmeticValue unit: Unit unitless
!

generality
	"Sandwich it in between complex and points"

	^165
! !

!Measurement methodsFor:'mathematical functions'!

raisedToInteger: anInteger 
	^Measurement magnitude: (magnitude raisedToInteger: anInteger)
		unit: (KeyOnlyUnit key: (unit key raisedToInteger: anInteger))
! !

!Measurement methodsFor:'printing'!

displayString
	| ws |
	ws := String new writeStream.
	ws print: magnitude; space; nextPutAll: unit name.
	^ws contents
!

printOn: aStream 
	aStream
		print: magnitude;
		nextPutAll: ' ~> ';
		print: unit name
! !

!Measurement methodsFor:'private'!

incompatibleUnitsError
	^IncompatibleUnitsError raiseSignal
! !

!Measurement methodsFor:'private-initialize'!

setMagnitude: aNumeric unit: aUnit 
	magnitude := aNumeric.
	unit := aUnit
! !

!Measurement methodsFor:'temperature-double dispatching'!

addedToTemperature: aTemperature 
	self temperatureUnitCheck.
	^aTemperature class new setMagnitude: aTemperature magnitude + (self ~> aTemperature unitString) magnitude
!

asAbsoluteTemperature
	self temperatureUnitCheck.
	^Temperature readFromString: magnitude printString , ' ' , unit name
!

subtractedFromTemperature: aTemperature 
	self temperatureUnitCheck.
	^aTemperature class new setMagnitude: aTemperature magnitude - (self ~> aTemperature unitString) magnitude
!

temperatureUnitCheck
	self baseUnit == (Unit of: 'K') ifFalse: [self incompatibleUnitsError]
! !

!Measurement methodsFor:'testing'!

isZero
	^magnitude isZero
!

negative
	^magnitude negative
!

positive
	^magnitude positive
!

sign
	^magnitude sign
!

strictlyPositive
	^magnitude strictlyPositive
! !

!Measurement methodsFor:'time-double dispatching'!

addedToDate: aDate 
	| days |
	days := (self ~> 'd') magnitude.
	^days rounded = days
		ifTrue: [Date fromDays: aDate asDays + days]
		ifFalse: [self addedToTimestamp: aDate asTimestamp]
!

addedToTime: aTime 
	^Time fromSeconds: aTime asSeconds + (self ~> 's') magnitude rounded \\ 86400
!

addedToTimestamp: aTimestamp 
	self timeUnitCheck.
	^aTimestamp fromMilliseconds: aTimestamp asMilliseconds + (self ~> 'ms') magnitude rounded
!

subtractedFromDate: aDate 
	| days |
	days := (self ~> 'd') magnitude.
	^days rounded = days
		ifTrue: [Date fromDays: aDate asDays - days]
		ifFalse: [self subtractedFromTimestamp: aDate asTimestamp]
!

subtractedFromTime: aTime 
	^Time fromSeconds: aTime asSeconds - (self ~> 's') magnitude rounded \\ 86400
!

subtractedFromTimestamp: aTimestamp 
	self timeUnitCheck.
	^aTimestamp fromMilliseconds: aTimestamp asMilliseconds - (self ~> 'ms') magnitude rounded
!

timeUnitCheck
	self baseUnit == (Unit of: 's') ifFalse: [^self incompatibleUnitsError]
! !

!Measurement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/Measurement.st,v 1.4 2003/10/07 21:24:39 cg Exp $'
! !
