"{ Package: 'stx:goodies/measurement' }"

Magnitude subclass:#Temperature
	instanceVariableNames:'magnitude'
	classVariableNames:''
	poolDictionaries:''
	category:'Measurements'
!


!Temperature class methodsFor:'instance creation'!

readFrom: aStream onError:exceptionBlock
	| num unit cl |
	num := Number readFrom: aStream onError:[^ exceptionBlock value].
	unit := aStream upToEnd reject: [:char | char isSeparator].
	cl := self classFor: unit.
	cl isNil ifTrue: [self error: 'Unknown temperature unit'].
	^cl new setMagnitude: num
!

unitString
	^self subclassResponsibility
! !

!Temperature class methodsFor:'private'!

classFor: aUnitString 
	^self subclasses detect: [:each | each unitString = aUnitString]
		ifNone: [nil]
! !

!Temperature methodsFor:'accessing'!

magnitude
	^magnitude
!

unitString
	^self class unitString
! !

!Temperature methodsFor:'arithmetic'!

+ anObject 
	^anObject addedToTemperature: self
!

- anObject 
	^anObject subtractedFromTemperature: self
! !

!Temperature methodsFor:'comparing'!

< aTemperature 
	^magnitude < (self convert: aTemperature) magnitude
!

= aTemperature 
	^magnitude = (self convert: aTemperature) magnitude
!

hash
	^self asKelvin hash
! !

!Temperature methodsFor:'converting'!

as: aUnitString 
	| cl |
	(cl := Temperature classFor: aUnitString) isNil ifTrue: [^self error: 'Unknown unit type'].
	^cl new convert: self
!

asCelsius
	^self subclassResponsibility
!

asFarenheit
	^self subclassResponsibility
!

asKelvin
	^self subclassResponsibility
!

asRankine
	^self subclassResponsibility
! !

!Temperature methodsFor:'measurements-double dispatching'!

subtractedFromTemperature: aTemperature 
	^aTemperature magnitude - (aTemperature convert: self) magnitude ~> aTemperature unitString
! !

!Temperature methodsFor:'printing'!

displayString
	| ws |
	ws := String new writeStream.
	ws print: magnitude; space; nextPutAll: self unitString.
	^ws contents
!

printOn: aStream 
	super printOn: aStream.
	aStream space; nextPut: $(; nextPutAll: self displayString; nextPut: $)
! !

!Temperature methodsFor:'private'!

convert: aTemperature 
	^self subclassResponsibility
! !

!Temperature methodsFor:'private-initialize'!

setMagnitude: aNumber 
	magnitude := aNumber
! !

!Temperature class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/Temperature.st,v 1.3 2002/11/15 12:17:37 cg Exp $'
! !
