"{ Package: 'stx:goodies/measurement' }"
!

!ArithmeticValue methodsFor:'measurements'!

canonicalUnit
	^Unit unitless
! !

!ArithmeticValue methodsFor:'measurements'!

magnitude
	^self
! !

!ArithmeticValue methodsFor:'measurements'!

of: aUnitString
	^self ~> aUnitString
! !

!ArithmeticValue methodsFor:'measurements'!

unit
	^Unit unitless
! !

!ArithmeticValue methodsFor:'measurements'!

~> aUnitString
	^Measurement magnitude: self unit: (Unit of: aUnitString)
! !

!Bag methodsFor:'adding'!

addBag: aBag
	aBag
		valuesAndCountsDo: [:value :count | self add: value withOccurrences: count]
! !

!Collection methodsFor:'measurements'!

magnitude
	^self
! !

!Collection methodsFor:'measurements'!

unit
	^Measurements.Unit unitless
! !

!Date methodsFor:'measurements-double dispatching'!

subtractedFromDate: aDate
	^aDate asDays - self asDays ~> 'd'
! !

!Fraction methodsFor:'measurements'!

unitNameFromKey
	| ws |
	ws := String new writeStream.
	ws
		nextPutAll: numerator unitNameFromKey;
		nextPut: $/;
		nextPutAll: denominator unitNameFromKey.
	^ws contents
! !

!Integer methodsFor:'measurements'!

primeFactors
	self isPrime
		ifTrue: [^Bag with: self]
		ifFalse:
			[self \\ 2 = 0
				ifTrue:
					[^((self // 2) primeFactors)
						add: 2;
						yourself]
				ifFalse:
					[3 to: self
						by: 2
						do: [:i | self \\ i = 0 ifTrue: [^i primeFactors addBag: (self // i) primeFactors]].
					self error: 'Number must be greater than 1!!']]
! !

!Integer methodsFor:'measurements'!

unitNameFromKey
	| ws parts dict |
	self = 1 ifTrue: [^''].
	ws := String new writeStream.
	parts := self primeFactors.
	dict := Unit unitsByKey.
	parts valuesAndCountsDo:
			[:prime :count |
			ws nextPutAll: (dict at: prime) name.
			count > 1 ifTrue: [count printOn: ws].
			ws nextPut: $.].
	ws position: ws position - 1.
	^ws contents
! !

!PeekableStream methodsFor:'accessing'!

nextUntil: block1
	"Answer the next elements for which <block1 value: ch> evaluates to false."

	| stream |
	stream := self contentsSpecies new writeStream.
	^self class endOfStreamSignal handle: [:ex | ex returnWith: stream contents]
		do:
			[| value |
			[block1 value: (value := self next)]
				whileFalse: [stream nextPut: value].    "We will not reach here if the end-of-stream is encountered"
			self skip: -1.
			stream contents]
! !

!PeekableStream methodsFor:'accessing'!

skipWhile: block1
	"Move the receiver's position past the next elements for which <block1 value: ch> evaluates to true."

	self class endOfStreamSignal handle: [:ex | ex return]
		do:
			[[block1 value: self next] whileTrue.   "We will not reach here if the end-of-stream is encountered"
			self skip: -1]
! !

!Time methodsFor:'arithmetic'!

+ aMeasurement
	aMeasurement isNumber ifTrue:[^ super + aMeasurement].
	^aMeasurement addedToTime: self
! !

!Time methodsFor:'arithmetic'!

- aMeasurement
	aMeasurement isNumber ifTrue:[^ super - aMeasurement].
	^aMeasurement subtractedFromTime: self
! !

!Time methodsFor:'measurements-double dispatching'!

subtractedFromTime: aTime
	| s |
	s := aTime asSeconds - self asSeconds.
	^s \\ 60 = 0
		ifTrue: [s \\ 3600
				ifTrue: [s // 3600 ~> 'hr']
				ifFalse: [s // 60 ~> 'min']]
		ifFalse: [s ~> 's']
! !

!Timestamp methodsFor:'arithmetic'!

+ aMeasurement
	aMeasurement isNumber ifTrue:[^ super + aMeasurement].
	^aMeasurement addedToTimestamp: self
! !

!Timestamp methodsFor:'arithmetic'!

- aMeasurement
	aMeasurement isNumber ifTrue:[^ super - aMeasurement].
	^aMeasurement subtractedFromTimestamp: self
! !

!Timestamp methodsFor:'converting'!

fromMilliseconds: totalMilliseconds
	self setMilliseconds: totalMilliseconds
"/        self fromSeconds: totalMilliseconds // 1000.
"/        millisecond := totalMilliseconds \\ 1000
! !

!Timestamp methodsFor:'measurement-double dispatching'!

subtractedFromTimestamp: aTimestamp
	| ms conversions units |
	ms := aTimestamp asMilliseconds - self asMilliseconds.
	conversions := #(1000 60000 3600000 86400000).
	units := #('s' 'min' 'hr' 'd').
	4 to: 1 do: [:i | ms \\ (conversions at: i) = 0 ifTrue: [^ms // (conversions at: i) ~> (units at: i)]].
	^ms ~> 'ms'
! !
