"{ Package: 'stx:goodies/measurement/tests' }"

TestCase subclass:#MeasurementTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Measurements-Tests'
!


!MeasurementTests methodsFor:'running'!

setUp
	Unit installUnits
!

testAddition
	| a b c |
	a := 10 ~> 's'.
	b := -2 ~> 'hr'.
	c := 13 ~> 'min'.
	self
		assert: a + b = (-7190 ~> 's');
		assert: b + a = (a + b);
		assert: a + c = (790 ~> 's');
		assert: c + a = (a + c);
		assert: b + c = (-107 ~> 'min');
		assert: c + b = (b + c);
		assert: a + a = (20 ~> 's');
		should: [a + 2] raise: IncompatibleUnitsError
!

testComparisons
	| a b |
	a := 1 ~> 'furlong'.
	b := 1 ~> 'm'.
	self
		assert: a ~= b;
		deny: a ~= a copy;
		assert: b = b copy;
		deny: a = b;
		assert: a > b;
		deny: b > a;
		assert: a >= a;
		assert: a >= b;
		deny: b >= a;
		assert: b <= a;
		assert: b <= b;
		deny: a <= b;
		assert: b < a;
		deny: a < b
!

testComplexAddition
	| a b c failed |
	a := 143 ~> 's2'.
	b := 2 ~> 'min2'.
	c := 1 ~> 'ft/s2'.
	failed := false.
	[a + c] on: Error do: [:ex | failed := true].
	self
		assert: failed;
		assert: a + b = (7343 ~> 's2')
!

testConversion
	| a |
	a := 12 ~> 'in'.
	a := a ~> 'ft'.
	self assert: a unit = (Unit of: 'ft'); assert: a magnitude = 1.
	a := a * a / a ~> 'in'.
	self assert: a unit = (Unit of: 'in'); assert: a magnitude = 12.
	self should: [a ~> 's']
		raise: IncompatibleUnitsError
!

testCreation
	| m |
	m := 1 ~> 'm'.
	self assert: m magnitude = 1; assert: m unit == (Unit of: 'm').
	m := Measurement readFromString: '3.141529 kg.m/s2'.
	self assert: m magnitude = 3.141529; assert: m unit = (Unit of: 'kg.m/s2')
!

testDateInteroperation
	self assert: (Date readFromString: '12/25/2002')
			- (4 ~> 'd') = (Date readFromString: '12/21/2002').
	self assert: (Date readFromString: '01/01/2000')
			- (1 ~> 'hr') = (Timestamp readFromString: '12/31/1999 23:00').
	self assert: (Date readFromString: '01/01/2000')
			- (Date readFromString: '2/02/2000') = (-32 ~> 'd').
	self assert: (Date readFromString: '09/19/1970')
			- (Date readFromString: '9/13/1970') = (6 ~> 'd').
	self assert: (Date readFromString: '12/25/2005')
			+ (6 ~> 'd') = (Date readFromString: '12/31/2005').
	self assert: (Date readFromString: '01/01/2000')
			+ (1 ~> 'hr') = (Timestamp readFromString: '1/1/2000 1:00')
!

testDivision
	| a b c |
	a := 10 ~> 'm'.
	b := 2 ~> 's2'.
	self assert: a / a = (1 ~> '').
	c := a / b.
	self
		assert: c = (5 ~> 'm/s2');
		assert: (10 ~> 'in2') / (2 ~> 'in') = (5 ~> 'in')
!

testMixedDivision
	| m |
	m := 12 ~> 'kg'.
	self
		assert: m / 2 = (6 ~> 'kg');
		assert: m / 0.5 = (24.0 ~> 'kg');
		assert: m / 10.0d = (1.2d ~> 'kg');
		assert: m / (4 / 3) = (9 ~> 'kg');
		assert: m / (2 @ 3) = (6 @ 4 ~> 'kg');
		assert: 36 / m = (3 ~> '/kg');
		assert: 6.0 / m = (0.5 ~> '/kg');
		assert: 18.0d / m = (1.5d ~> '/kg');
		assert: 2 / 3 / m = (1 / 18 ~> '/kg');
		assert: 6 @ 18 / m = ((1 / 2 ~> '/kg') @ (3 / 2 ~> '/kg'))
!

testMixedMultiplication
	| a |
	a := 10 ~> 'ft'.
	self
		assert: a * 2 = (20 ~> 'ft');
		assert: -3 * a = (-30 ~> 'ft');
		assert: -1.0 * a = (-10.0 ~> 'ft');
		assert: -1.0d * a = (-10.0d ~> 'ft');
		assert: 2 @ 4 * a = ((20 ~> 'ft') @ (40 ~> 'ft'));
		assert: a * (4 @ 5) = (40 @ 50 ~> 'ft');
		assert: 1 / 2 * a = (5 ~> 'ft')
!

testMultiplication
	| a b |
	a := 10 ~> 'm'.
	b := a * a.
	self
		assert: b magnitude = 100;
		assert: b unit = (Unit of: 'm2')
!

testNegated
	| a |
	a := (13 ~> 'm') negated.
	self assert: a unit = (Unit of: 'm'); assert: a magnitude = -13
!

testReciprocal
	| a b |
	a := 10 ~> 's'.
	b := a * a / a.
	a := a reciprocal.
	b := b reciprocal.
	self assert: a unit = (Unit of: '/s'); assert: a magnitude = (1 / 10).
	self assert: b unit = (Unit of: '/s'); assert: b magnitude = (1 / 10)
!

testSubtraction
	| a b |
	a := 10 ~> 'min'.
	b := 13 / 2 ~> 'min'.
	self
		assert: a - b = (7 / 2 ~> 'min');
		assert: b - a = (-7 / 2 ~> 'min').
	a := -2 ~> 'hr'.
	b := 600 ~> 's'.
	self
		assert: a - b = (-130 ~> 'min');
		assert: b - b = (0 ~> 's')
!

testTemperatureInteroperation
	| a b c |
	self assert: (Temperature readFromString: '4 K')
			+ (45 ~> 'C') = (Temperature readFromString: '49 K').
	self assert: (Temperature readFromString: '212 F')
			- (12 ~> 'R') = (Temperature readFromString: '200 F').
	self should: [(Temperature readFromString: '212 F')
			+ (13 ~> 'ft')]
		raise: IncompatibleUnitsError.
	a := Temperature readFromString: '100 C'.
	b := a - (50 ~> 'K').
	self assert: b = (Temperature readFromString: '50 C').
	c := a - b.
	self assert: c = (50 ~> 'C')
!

testTimeInteroperation
	self assert: (Time readFromString: '15:30')
			- (4 ~> 'min') = (Time readFromString: '15:26').
	self assert: (Time readFromString: '15:30')
			- (3 / 2 ~> 'd') = (Time readFromString: '3:30').
	self assert: (Time readFromString: '7.00')
			- (Time readFromString: '8:17:32') = (-4652 ~> 's').
	self assert: (Time readFromString: '9.00')
			- (Time readFromString: '8:17:32') = (2548 ~> 's').
	self assert: (Time readFromString: '21.00')
			+ (4 ~> 'hr') = (Time readFromString: '1.00').
	self assert: (Time readFromString: '9.00')
			+ (4 ~> 'hr') = (Time readFromString: '13.00')
!

testTimestampInteroperation
	self assert: (Timestamp readFromString: '4/5/1982 15:00')
			- (45 ~> 'ms') = (Timestamp readFromString: '4/5/1982 14:59:59.955').
	self assert: (Timestamp readFromString: '9/17/1970 12:00')
			- (Timestamp readFromString: '9/17/1970 13:00') = (-1 ~> 'hr').
	self assert: (Timestamp readFromString: '9/17/1970 12:00')
			- (Timestamp readFromString: '9/17/1970 11:55') = (5 ~> 'min').
	self assert: (Timestamp readFromString: '4/5/1982 15:00')
			+ (45 ~> 'min') = (Timestamp readFromString: '4/5/1982 15:45')
! !

!MeasurementTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/tests/MeasurementTests.st,v 1.4 2011/02/10 17:50:37 cg Exp $'
! !
