"{ Package: 'stx:goodies/measurement/tests' }"

TestCase subclass:#UnitsTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Measurements-Tests'
!


!UnitsTest methodsFor:'running'!

setUp
	Unit installUnits
!

testComplexBaseUnit
	| unit base |
	unit := Unit of: 'N'.
	base := unit baseUnit.
	self assert: (base isKindOf: BaseUnit).
	self assert: base = (Unit of: 'kg.m/s2').
	self assert: base key = ((Unit of: 'kg') key * (Unit of: 'm') key / (Unit of: 's') key squared)
!

testCompoundBaseUnitInference
	| a |
	a := BaseUnit fromString: 'ft2'.
	self
		assert: a isBaseUnit;
		assert: a = (Unit of: 'm2').
	a := BaseUnit fromString: 's3'.
	self
		assert: a isBaseUnit;
		assert: a = (Unit of: 's3')
!

testInvalidUnitCreation
	| failed |
	failed := false.
	[Unit of: 'yuck'] on: Error do: [:ex | failed := true].
	self assert: failed
!

testPoweredUnitParsing
	self
		assert: (Unit basicUnitPartIn: 'ft') = 'ft';
		assert: (Unit basicUnitPartIn: 's3') = 's';
		assert: (Unit unitPowerPartIn: 'yd2') = 2;
		assert: (Unit unitPowerPartIn: 'in') = 1
!

testPrimeNumbers
	| factors |
	self
		assert: ((1 to: 30) select: [:i | i isPrime])
					= #(2 3 5 7 11 13 17 19 23 29);
		assert: ((0 to: 30) collect: [:i | i nextPrime])
					= #(2 2 3 5 5 7 7 11 11 11 11 13 13 17 17 17 17 19 19 23 23 23 23 29 29 29 29 29 29 31 31).
	factors := 9 primeFactors.
	self
		assert: factors size = 2;
		assert: (factors occurrencesOf: 3) = 2.
	factors := (17 * 11) primeFactors.
	self
		assert: factors size = 2;
		assert: (factors occurrencesOf: 11) = 1;
		assert: (factors occurrencesOf: 17) = 1.
	factors := 29 primeFactors.
	self
		assert: factors size = 1;
		assert: (factors occurrencesOf: 29) = 1.
	factors := 1024 primeFactors.
	self
		assert: factors size = 10;
		assert: factors asSet asArray first = 2
!

testUnitCreation
	| a b c |
	a := Unit of: 'm'.
	self
		assert: a isBaseUnit;
		assert: a conversion = 1.
	b := Unit of: 'yd'.
	self
		deny: b isBaseUnit;
		assert: b baseUnit == a;
		assert: b conversion = (254 * 36 / 10000).
	c := Unit of: 'ft3'.
	self assert: c key = ((Unit of: 'ft') key ** 3).
	c := Unit of: '/s2'.
	self assert: c key = (Unit of: 's') key squared reciprocal.
	c := Unit of: 'ft3/ft2'.
	self assert: c key = (Unit of: 'ft') key
!

testUnitDenominatorParsing
	| res |
	res := Unit denominatorPartIn: '/m'.
	self
		assert: res size = 1;
		assert: (res occurrencesOf: 'm') = 1.
	res := Unit denominatorPartIn: 'kg.m/s2'.
	self
		assert: res size = 2;
		assert: (res occurrencesOf: 's') = 2.
	res := Unit denominatorPartIn: 'kg.m/s2.rad'.
	self
		assert: res size = 3;
		assert: (res occurrencesOf: 's') = 2;
		assert: (res occurrencesOf: 'rad') = 1.
	res := Unit denominatorPartIn: 'm.s2'.
	self assert: res size = 0
!

testUnitNumeratorParsing
	| res |
	res := Unit numeratorPartIn: 'm'.
	self
		assert: res size = 1;
		assert: (res occurrencesOf: 'm') = 1.
	res := Unit numeratorPartIn: 'kg.m/s2'.
	self
		assert: res size = 2;
		assert: (res occurrencesOf: 'm') = 1;
		assert: (res occurrencesOf: 'kg') = 1.
	res := Unit numeratorPartIn: 'kg.m/s2.rad'.
	self
		assert: res size = 2;
		assert: (res occurrencesOf: 'm') = 1;
		assert: (res occurrencesOf: 'kg') = 1.
	res := Unit numeratorPartIn: 'kg.m2/s2.rad/m'.
	self
		assert: res size = 4;
		assert: (res occurrencesOf: 'm') = 3;
		assert: (res occurrencesOf: 'kg') = 1.
	res := Unit numeratorPartIn: '/m'.
	self assert: res size = 0
!

testUnitTokenParsing
	| bag |
	bag := Unit parseUnitTokensIn: 'm'.
	self
		assert: bag size = 1;
		assert: (bag occurrencesOf: 'm') = 1.
	bag := Unit parseUnitTokensIn: 'm2'.
	self
		assert: bag size = 2;
		assert: (bag occurrencesOf: 'm') = 2.
	bag := Unit parseUnitTokensIn: 'm2.s'.
	self
		assert: bag size = 3;
		assert: (bag occurrencesOf: 'm') = 2;
		assert: (bag occurrencesOf: 's') = 1
! !

!UnitsTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/tests/UnitsTest.st,v 1.4 2011/02/10 17:50:38 cg Exp $'
! !
