"{ Package: 'stx:goodies/metacello' }"

MetacelloSpec subclass:#MetacelloAbstractPackageSpec
	instanceVariableNames:'name requires includes answers'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-Core-Specs'
!

MetacelloAbstractPackageSpec comment:''
!


!MetacelloAbstractPackageSpec methodsFor:'*metacello-mc'!

forceUpdatePackageSpec: updatedSpecs using: anMCLoader

	^self updatePackageSpec: updatedSpecs using: anMCLoader
!

loadUsing: aLoader gofer: gofer

	^self subclassResponsibility
!

packageRepository
	
	^nil
!

packagesNeedSavingVisited: visitedProjects using: repos into: aCollection
	"noop by default"
!

repositorySpecs

	^#()
!

resolveToAllPackagesIn: aVersionSpec into: packages visited: visited

	visited
		visit: self
		doing: [:spec |
			self visitingWithPackages: packages.
			(spec includes, spec requires) do: [:pkgName |
				(aVersionSpec packageNamed: pkgName)
					projectDo: [:prj | 
						(prj resolveToAllPackagesIn: aVersionSpec visited: visited) do: [:pkg | 
							packages at: pkg name put: pkg ]] 
					packageDo: [:pkg | 
						packages at: pkg name put: pkg.
						(pkg resolveToAllPackagesIn: aVersionSpec visited: visited) do: [:rpkg | 
							packages at: rpkg name put: rpkg ] ] 
					groupDo: [:grp | grp resolveToAllPackagesIn: aVersionSpec into: packages  visited: visited ]]]
!

resolveToAllPackagesIn: aVersionSpec visited: visited

	| packages |
	packages := Dictionary new.
	self resolveToAllPackagesIn: aVersionSpec into: packages visited: visited.
	^packages values asOrderedCollection
!

resolveToLoadableSpec

	^self
!

resolveToPackagesIn: aVersionSpec visited: visited

	^self subclassResponsibility
!

updateForSpawnMethod: sourceSpec
	"This means that this spec was used in a baseline and will be used in a version .... drop all information that isn't useful"

	answers := name := requires := includes := nil
!

updatePackageRepositoriesFor: aVersionSpec
	"noop by default"
	
	^true
!

updatePackageSpec: updatedSpecs using: anMCLoader
	"Add pkg copy to updatedSpecs if the file in current image is different from the receiver's file"
!

visitingWithPackages: packages
	"noop"
! !

!MetacelloAbstractPackageSpec methodsFor:'*metacello-mc-querying'!

file 
	"MetacelloPackageSpec compatibility"
	
	^nil
!

isPackageLoaded

	^false
!

repository
	
	^nil
!

version 
	"MetacelloPackageSpec compatibility"
	
	^nil
! !

!MetacelloAbstractPackageSpec methodsFor:'accessing'!

answers: aListOfPairs

	self setAnswers: aListOfPairs
!

includes: aCollection

	aCollection setIncludesInMetacelloPackage: self
!

name: aString

	name := aString
!

referencedSpec

	^self
!

requires: aCollection

	aCollection setRequiresInMetacelloPackage: self
! !

!MetacelloAbstractPackageSpec methodsFor:'adding'!

addToMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec addMember 
			name: self name;
			spec: self;
			yourself)
! !

!MetacelloAbstractPackageSpec methodsFor:'copying'!

postCopy

	super postCopy.
	requires := requires copy.
	includes := includes copy.
	answers := answers copy.
! !

!MetacelloAbstractPackageSpec methodsFor:'merging'!

mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec mergeMember 
			name: self name;
			spec: self;
			yourself)
!

mergeMap

	| map |
	map := super mergeMap.
	map at: #requires put: requires.
	map at: #includes put: includes.
	map at: #answers put: answers.
	^map
!

mergeSpec: anotherSpec

	| newSpec map anotherRequires anotherIncludes anotherAnswers |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	anotherSpec name ~~ nil 
		ifTrue: [ newSpec name: anotherSpec name ].
	(anotherRequires := map at: #requires) ~~ nil
		ifTrue: [ newSpec setRequires: self requires, anotherRequires ].
	(anotherIncludes := map at: #includes) ~~ nil
		ifTrue: [ newSpec setIncludes: self includes, anotherIncludes ].
	(anotherAnswers := map at: #answers) ~~ nil
		ifTrue: [ newSpec setAnswers: self answers, anotherAnswers ].
	^newSpec
	
!

nonOverridable

	^#( includes requires answers )
! !

!MetacelloAbstractPackageSpec methodsFor:'printing'!

configMethodBodyOn: aStream hasName: hasName cascading: cascading indent: indent

	| hasCascading hasRequires hasIncludes hasAnswers |
	hasCascading := cascading.
	hasRequires := self requires isEmpty not.
	hasIncludes := self includes isEmpty not.
	hasAnswers := self answers isEmpty not.
	hasRequires
		ifTrue: [ 
			hasName | hasIncludes | hasAnswers | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'requires: #('.
			self requires do: [:str | aStream nextPutAll: str printString, ' ' ].
			hasIncludes | hasAnswers | hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].
	hasIncludes
		ifTrue: [ 
			hasName | hasRequires | hasAnswers | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'includes: #('.
			self includes do: [:str | aStream nextPutAll: str printString, ' ' ].
			hasAnswers | hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].
	hasAnswers
		ifTrue: [ 
			hasName | hasRequires | hasIncludes | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'supplyingAnswers: #( '.
			self answers do: [:ar | 
				aStream nextPutAll: '#( '.
				ar do: [:val | 
					(val isString or: [ val isNumber or: [ val isSymbol or: [ val isCharacter ]]])
						ifTrue: [  aStream nextPutAll: val printString, ' ' ].
					val == true
						ifTrue: [  aStream nextPutAll: 'true ' ].
					val == false
						ifTrue: [  aStream nextPutAll: 'false ' ]].
				aStream nextPutAll: ') ' ].
			hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].
!

configMethodCascadeOn: aStream member: aMember last: lastCascade indent: indent

	self subclassResponsibility
!

configMethodOn: aStream for: aValue selector: selector cascading: cascading cascade: cascade indent: indent

	| valuePrintString |
	aValue == nil ifTrue: [ ^self ].
	cascading ifTrue: [ aStream cr; tab: indent ].
	valuePrintString := aValue value isSymbol
		ifTrue: [ '#' , aValue value asString printString ]
		ifFalse: [ aValue value printString ].
	aStream  nextPutAll: selector, valuePrintString.
	cascade ifTrue: [ aStream nextPut: $; ]
!

label

	^self name
! !

!MetacelloAbstractPackageSpec methodsFor:'private'!

includesForPackageOrdering

	^#()
!

setAnswers: aCollection

	answers := aCollection
!

setIncludes: aCollection

	includes := aCollection
!

setRequires: aCollection

	requires := aCollection
! !

!MetacelloAbstractPackageSpec methodsFor:'querying'!

answers

	answers == nil ifTrue: [ answers := #() ].
	^answers
!

includes

	includes == nil ifTrue: [ includes := #() ].
	^includes
!

name

	^name
!

requires

	requires == nil ifTrue: [ requires := #() ].
	^requires
! !

!MetacelloAbstractPackageSpec methodsFor:'removing'!

removeFromMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec removeMember 
			name: self name;
			spec: self;
			yourself)
! !

!MetacelloAbstractPackageSpec methodsFor:'visiting'!

projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	self subclassResponsibility
! !

!MetacelloAbstractPackageSpec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloAbstractPackageSpec.st,v 1.1 2011/08/20 18:06:22 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloAbstractPackageSpec.st,v 1.1 2011/08/20 18:06:22 cg Exp $'
! !
