"{ Package: 'stx:goodies/metacello' }"

MetacelloAbstractConstructor subclass:#MetacelloAbstractVersionConstructor
	instanceVariableNames:'root configuration project attributeMap attributeOrder
		symbolicVersion currentContext'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-Core-Constructors'
!

MetacelloAbstractVersionConstructor comment:''
!


!MetacelloAbstractVersionConstructor class methodsFor:'method generation'!

symbolicMethodSelectorAndPragma: selector symbolicVersionSymbol: symbolicVersionSymbol on: strm
	strm
		nextPutAll: selector asString , ' spec';
		cr;
		tab;
		nextPutAll: '<symbolicVersion: #' , symbolicVersionSymbol asString printString;
		nextPutAll: '>';
		cr
! !

!MetacelloAbstractVersionConstructor methodsFor:'*metacello-mc-accessing'!

project

	project == nil ifTrue: [ project := MetacelloMCProject new].
	^project
! !

!MetacelloAbstractVersionConstructor methodsFor:'accessing'!

addAttribute: anAttribute

	self attributeOrder add: anAttribute
!

attributeMap

	attributeMap == nil ifTrue: [ attributeMap := Dictionary new ].
	^attributeMap
!

attributeOrder

	attributeOrder == nil ifTrue: [ attributeOrder := OrderedCollection new ].
	^attributeOrder
!

configuration

	^configuration
!

configuration: aConfig

	configuration := aConfig
!

configurationClass

	^self configuration class
!

root

	^root
!

root: aMetacelloSpec

	root := aMetacelloSpec
!

setProject: aProject

	project := aProject
!

symbolicVersion

	^symbolicVersion
!

symbolicVersion: aSymbol

	symbolicVersion := aSymbol
! !

!MetacelloAbstractVersionConstructor methodsFor:'api'!

author: aBlockOrString

	aBlockOrString setAuthorInMetacelloConfig: self
!

blessing: aBlockOrString

	aBlockOrString setBlessingInMetacelloConfig: self
!

className: anObject

	self root className: anObject
!

description: aBlockOrString

	aBlockOrString setDescriptionInMetacelloConfig: self
!

file: anObject

	self root file: anObject
!

for: attribute do: aBlock
	"conditional version support"

	| blockList |
	blockList := self attributeMap
		at: attribute
		ifAbsent: [ self attributeMap at: attribute put: (blockList := OrderedCollection new) ].
	blockList add: aBlock.
	self addAttribute: attribute
!

for: attribute version: aString
	"conditional symbolicVersion support"

	self attributeMap at: attribute put: aString.
	self addAttribute: attribute
!

group: aString overrides: aStringOrCollection

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			includes: aStringOrCollection;
			yourself.
	self root packages add: spec.
!

group: aString with: aStringOrCollection

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			includes: aStringOrCollection;
			yourself.
	self root packages merge: spec.
!

includes: anObject

	self root includes: anObject
!

loads: anObject

	self root loads: anObject
!

name: anObject

	self root name: anObject
!

operator: anObject

	self root operator: anObject
!

package: aString

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages add: spec.
!

package: aString overrides: aBlock

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages add: spec.
	self with: spec during: aBlock
!

package: aString with: aBlockOrString

	aBlockOrString setPackage: aString withInMetacelloConfig: self
!

postLoadDoIt: aSymbol

	self validateDoItSelector: aSymbol.
	self root postLoadDoIt: aSymbol
!

preLoadDoIt: aSymbol

	self validateDoItSelector: aSymbol.
	self root preLoadDoIt: aSymbol
!

project: aString

	self project: aString with: ''
!

project: aString copyFrom: oldSpecName with: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages copy: oldSpecName to: spec.
	self with: projectSpec during: aBlock
!

project: aString overrides: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	projectSpec projectPackage: self project packageSpec.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages add: spec.
	self with: projectSpec during: aBlock
	
!

project: aString with: aBlockOrString

	aBlockOrString setProject: aString withInMetacelloConfig: self
!

projectPackage: aBlock

	| spec |
	(spec := self root projectPackage) == nil
		ifTrue: [ 
			spec := self project packageSpec.
			self root projectPackage: spec ].
	self with: spec during: aBlock
!

removeGroup: aString

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			yourself.
	self root packages remove: spec.
!

removePackage: aString

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages remove: spec.
!

removeProject: aString

	| spec |
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			yourself.
	self root packages remove: spec.
!

repositories: aBlock

	self with: self root repositories during: aBlock
!

repository: anObject

	self root repository: anObject
!

repository: description username: username password: password

	self root repository: description username: username password: password
!

requires: anObject

	self root requires: anObject
!

supplyingAnswers: aCollection

	self root answers: aCollection
!

timestamp: aBlockOrString

	aBlockOrString setTimestampInMetacelloConfig: self
!

value: anObject

	self root value: anObject
!

version: anObject

	self root versionString: anObject
!

versionString: anObject

	self root versionString: anObject
! !

!MetacelloAbstractVersionConstructor methodsFor:'api callbacks'!

setAuthorWithBlock: aBlock

	| spec |
	(spec := self root getAuthor) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setAuthor: spec ].
	self with: spec during: aBlock
!

setAuthorWithString: aString

	self root author: aString
!

setBlessingWithBlock: aBlock

	| spec |
	(spec := self root getBlessing) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setBlessing: spec ].
	self with: spec during: aBlock
!

setBlessingWithString: aString

	self root blessing: aString
!

setDescriptionWithBlock: aBlock

	| spec |
	(spec := self root getDescription) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setDescription: spec ].
	self with: spec during: aBlock
!

setDescriptionWithString: aString

	self root description: aString
!

setPackage: aString withBlock: aBlock

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages merge: spec.
	self with: spec during: aBlock
!

setPackage: aString withString: aFile

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			file: aFile;
			yourself.
	self root packages merge: spec.
!

setProject: aString withBlock: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	projectSpec projectPackage: self project packageSpec.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages merge: spec.
	self with: projectSpec during: aBlock
!

setProject: aString withString: versionString

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			versionString: versionString;
			yourself.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages merge: spec.
!

setTimestampWithBlock: aBlock

	| spec |
	(spec := self root getTimestamp) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setTimestamp: spec ].
	self with: spec during: aBlock
!

setTimestampWithString: aString

	self root timestamp: aString
! !

!MetacelloAbstractVersionConstructor methodsFor:'initialization'!

reset

	attributeMap := attributeOrder := nil
! !

!MetacelloAbstractVersionConstructor methodsFor:'private'!

evaluatePragma: pragma
	currentContext := pragma.
	[ self configuration perform: pragma selector with: self ] ensure: [ currentContext := nil ]
!

with: aMetacelloSpec during: aBlock 

	| previousRoot |
	previousRoot := self root.
	self root: aMetacelloSpec.
	aBlock
		ensure: [self root: previousRoot]
! !

!MetacelloAbstractVersionConstructor methodsFor:'validation'!

validateDoItSelector: anObject

	anObject == nil ifTrue: [ ^ self ].
	anObject isSymbol ifFalse: [ self error: 'Invalid message selector for doit: ', anObject printString ].
! !

!MetacelloAbstractVersionConstructor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloAbstractVersionConstructor.st,v 1.1 2011/08/20 18:14:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloAbstractVersionConstructor.st,v 1.1 2011/08/20 18:14:20 cg Exp $'
! !
