"{ Package: 'stx:goodies/metacello' }"

MetacelloSpecLoader subclass:#MetacelloCommonMCSpecLoader
	instanceVariableNames:'operator loaderPolicy disablePackageCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Loaders'
!

MetacelloCommonMCSpecLoader comment:''
!


!MetacelloCommonMCSpecLoader class methodsFor:'utilities'!

nameComponentsFrom: aVersionName
	| ar |
	ar := (aVersionName last isDigit and: [ (aVersionName indexOf: $.) > 0 ])ifFalse: [Array with: aVersionName with: '' with: 0 with: aVersionName with: self]
		ifTrue: [ | vrsn str |
			str := ((aVersionName copyAfterLast: $-) copyAfterLast: $.).
			vrsn := str isEmpty
				ifTrue: [0]
				ifFalse: [str asInteger].
			Array
				with: (aVersionName copyUpToLast:  $-)								"base pkg name"
				with: ((aVersionName copyAfterLast: $-) copyUpTo: $.)				"user"
				with:  vrsn "version"
				with: aVersionName
				with: self ].
	^ar
! !

!MetacelloCommonMCSpecLoader methodsFor:'accessing'!

fetchingSpecLoader

	^self
!

ignoreImage

	^self loaderPolicy ignoreImage
!

loaderPolicy

	loaderPolicy == nil ifTrue: [ loaderPolicy := MetacelloLoaderPolicy new ].
	^loaderPolicy
!

loaderPolicy: anObject
	loaderPolicy := anObject
!

loadingSpecLoader

	^self
!

operator

	operator == nil ifTrue: [ ^#= ].
	^operator
!

operator: aSymbol

	operator := aSymbol
!

recordingSpecLoader

	^(MetacelloNullRecordingMCSpecLoader on: self spec)
		shouldDisablePackageCache: self shouldDisablePackageCache;
		loaderPolicy: self loaderPolicy copy;
		yourself
!

repositoryMap

	^self loaderPolicy repositoryMap
!

shouldDisablePackageCache

	disablePackageCache == nil ifTrue: [ disablePackageCache := false ].
	^ disablePackageCache
!

shouldDisablePackageCache: anObject
	disablePackageCache := anObject
! !

!MetacelloCommonMCSpecLoader methodsFor:'actions'!

doLoad

	self subclassResponsibility
!

doingLoads: aBlock
	"escape mechanism for recording and null loaders to skip doing loaderlike things"
	
	aBlock value
!

linearLoadPackageSpec: packageSpec gofer: gofer

	self subclassResponsibility
!

load
	| repos |
	repos := self repositoriesFrom: self spec repositorySpecs.
	^ self loadType == #atomic
		ifTrue: [self atomicLoadPackageSpecs: self spec packageSpecsInLoadOrder repositories: repos]
		ifFalse: ["assume #linear"
			self linearLoadPackageSpecs: self spec packageSpecsInLoadOrder repositories: repos ]
!

loadPackageDirective: aPackageLoadDirective gofer: aGofer

	| packageSpec |
	packageSpec := aPackageLoadDirective spec.
	MetacelloPlatform current
		do:  [ | loadBlock goferLoad answers resolvedReference |
			aGofer disablePackageCache. "for good luck:)"
			resolvedReference := self resolvePackageSpec: packageSpec gofer: aGofer.
			resolvedReference isNil ifTrue: [ "Package version already loaded into image" ^self ].
			loadBlock := [
				"mcLoader preLoad: packageSpec."
				goferLoad := MetacelloGoferLoad on: MetacelloGofer new.
				goferLoad addResolved: resolvedReference.
				goferLoad execute.
				MetacelloPlatform current clearCurrentVersionCache.
				"mcLoader postLoad: packageSpec" ].
			(answers := packageSpec answers) notEmpty
				ifTrue: [ loadBlock valueSupplyingMetacelloAnswers: answers ]
				ifFalse: [ loadBlock value].
			resolvedReference workingCopy repositoryGroup addRepository:  aPackageLoadDirective repository.
			Transcript cr; show: 'Loaded -> ', resolvedReference name, ' --- ', aPackageLoadDirective repository description, ' --- ', resolvedReference repository description  ]
		displaying: 'Loading ', packageSpec file
!

loadPackageDirectives: pkgLoads gofer: aGofer

	MetacelloPlatform current
		do:  [ | goferLoad loadBlock answers |
			goferLoad := MetacelloGoferLoad on: aGofer.
			answers := OrderedCollection new.
			pkgLoads do: [:packageLoadDirective | | resolvedReference |
				aGofer disablePackageCache. "for good luck:)"
				(resolvedReference := self 
						resolvePackageSpec: packageLoadDirective spec 
						gofer: aGofer) ~~ nil
					ifTrue: [ 
						goferLoad addResolved: resolvedReference.
						answers addAll: packageLoadDirective spec answers.
						packageLoadDirective resolvedReference:  resolvedReference ]].
			Transcript cr; show: 'Starting atomic load'.
			loadBlock := [
				"pkgLoads do: [:packageLoadDirective |  mcLoader preLoad: packageLoadDirective spec ]."
				goferLoad execute.
				pkgLoads do: [:packageLoadDirective |
					packageLoadDirective resolvedReference == nil
						ifTrue: [ Transcript cr; tab; show: 'Already Loaded -> ', packageLoadDirective file ]
						ifFalse: [ Transcript cr; tab; show: 'Loaded -> ', packageLoadDirective file, ' --- ',  packageLoadDirective repository description, ' --- ', packageLoadDirective resolvedReference repository description.
					packageLoadDirective resolvedReference workingCopy repositoryGroup addRepository: packageLoadDirective repository ]].
				MetacelloPlatform current clearCurrentVersionCache.
				"pkgLoads do: [:packageLoadDirective | 
					packageLoadDirective resolvedReference ~~ nil
						ifTrue: [mcLoader postLoad: packageLoadDirective spec ]]"].
			answers notEmpty
				ifTrue: [ loadBlock valueSupplyingMetacelloAnswers: answers ]
				ifFalse: [ loadBlock value].
			Transcript cr; show: 'Finished atomic load' ]
		displaying: 'Atomic Load...'.
! !

!MetacelloCommonMCSpecLoader methodsFor:'doits'!

postLoad: packageOrVersionSpec
	"subclassResponsibility, but it gets called during an upgrade, so leave it as NOOP"
!

preLoad: packageOrVersionSpec

	self subclassResponsibility
! !

!MetacelloCommonMCSpecLoader methodsFor:'initialize-release'!

initialize

	self loaderPolicy
! !

!MetacelloCommonMCSpecLoader methodsFor:'packages'!

nameComponentsFrom: aVersionName

	^self class nameComponentsFrom: aVersionName
! !

!MetacelloCommonMCSpecLoader methodsFor:'private'!

linearLoadPackageSpecs: packageSpecs repositories: repositories
	
	| gofer |
	gofer := MetacelloGofer new.
	repositories do: [:repo | gofer repository: repo ].
	packageSpecs do: [:pkg | pkg loadUsing: self gofer: gofer ].
!

resolvePackageSpec: packageSpec gofer: gofer

	|  references resolvedReference mcVersion loadedVersionInfos |
	references := self retryingResolvePackageSpecReferences: packageSpec gofer: gofer.
	resolvedReference := references last asMetacelloCachingResolvedReference.
	mcVersion := resolvedReference version.
	(loadedVersionInfos := self ancestorsFor: packageSpec) ~~ nil
		ifTrue: [ 
			loadedVersionInfos do: [:info | info name = mcVersion info name 
				ifTrue: [ 
					"package already loaded, don't load again"
					^nil ]]].
	^resolvedReference
!

resolvePackageSpecReferences: packageSpec gofer: gofer

	|  versionReference references localGofer |
	localGofer := gofer.
	self hasRepositoryOverrides not
		ifTrue: [
			packageSpec repositorySpecs notEmpty
				ifTrue: [ 
					localGofer := MetacelloGofer new.
					(self repositoriesFrom: packageSpec repositorySpecs) do: [:repo | localGofer repository: repo ]]].
	(packageSpec getFile == nil or: [ self shouldDisablePackageCache ])
		ifTrue: [ 
			"don't use package-cache when trying to get latest version"
			localGofer disablePackageCache ].
	versionReference := packageSpec goferLoaderReference.
	references := versionReference resolveAllWith: localGofer.
	localGofer enablePackageCache.
	^references
!

retryingResolvePackageSpecReferences: packageSpec gofer: gofer

	| retryCount references repositoryError |
	retryCount := 0.
	references := #().
	[references isEmpty
		and: [retryCount < 3]]
		whileTrue: [retryCount > 0
				ifTrue: [Transcript cr; show: '...RETRY->', packageSpec file ].
			references := [self resolvePackageSpecReferences: packageSpec gofer: gofer]
						on: Error , GoferRepositoryError
						do: [:ex | 
							retryCount >= 2
								ifTrue: [(ex isKindOf: GoferRepositoryError)
										ifTrue: [
											"ignore repository errors at this point, in case an alternate repository is in the list "
											Transcript cr; show: 'gofer repository error: ', ex description printString, '...ignoring'. 
											repositoryError := ex.
											ex resume: #()]
										ifFalse: [ex pass]].
							ex return: #()].
			retryCount := retryCount + 1].
	references isEmpty 
		ifTrue: [ | repositoryString |
			repositoryString := ''.
			gofer repositories do: [:repo | repositoryString := repositoryString, ' ', repo description ].
			Transcript cr; show: '...FAILED->', packageSpec file.
			self error: 'Could not resolve: ', packageSpec name, ' [', packageSpec file, ']', ' in', repositoryString,
				(repositoryError == nil 
					ifTrue: [''] 
					ifFalse: [
						"report repository error to user here, since failure here is likely to be due to earlier repository error"
						' ERROR: ', repositoryError description printString ]) ].
	^references
! !

!MetacelloCommonMCSpecLoader methodsFor:'repositories'!

repositoriesFrom: aMetacelloMVRepositorySpecs

	^self repositoriesFrom: aMetacelloMVRepositorySpecs ignoreOverrides: false
!

repositoriesFrom: aMetacelloMVRepositorySpecs ignoreOverrides: ignoreOverrides

	| repositories repos |
	(ignoreOverrides not and: [self hasRepositoryOverrides]) ifTrue: [ ^self loaderPolicy overrideRepositories ].
	repositories := MCRepositoryGroup default repositories.
	repos := OrderedCollection new.
	aMetacelloMVRepositorySpecs do: [:aSpec | | description repo |
		description := aSpec description.
		(repo := repositories detect: [:rep | rep description = description ] ifNone: [ aSpec createRepository ]) ~~ nil
			ifTrue: [ repos add: repo ]].
	^repos
! !

!MetacelloCommonMCSpecLoader methodsFor:'testing'!

ensureForDevelopment

	^true
!

hasRepositoryOverrides

	^self loaderPolicy hasRepositoryOverrides
! !

!MetacelloCommonMCSpecLoader methodsFor:'versionInfo'!

ancestorsFor: packageSpec

	| cacheKey vi |
	cacheKey := packageSpec file.
	^MetacelloPlatform current
		stackCacheFor: #ancestors
		at: cacheKey
		doing: [ :cache | 
			vi := packageSpec ancestors.
			cache at: cacheKey put: vi ].
!

currentVersionInfoFor: packageSpec

	| cacheKey vi |
	cacheKey := packageSpec file.
	^MetacelloPlatform current
		stackCacheFor: #currentVersionInfo
		at: cacheKey
		doing: [ :cache | 
			vi := packageSpec currentVersionInfo.
			cache at: cacheKey put: vi ].
! !

!MetacelloCommonMCSpecLoader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloCommonMCSpecLoader.st,v 1.1 2011/08/20 17:57:44 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloCommonMCSpecLoader.st,v 1.1 2011/08/20 17:57:44 cg Exp $'
! !
