"{ Package: 'stx:goodies/metacello' }"

MetacelloConfigAbstractAPITestCase subclass:#MetacelloConfigAPIToolBoxTestCase
	instanceVariableNames:'gofer'
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloConfigAPIToolBoxTestCase comment:''
!


!MetacelloConfigAPIToolBoxTestCase methodsFor:'accessing'!

monticelloRepository
	"MetacelloMonticelloResource reset"
	^ MetacelloMonticelloResource current monticelloRepository
!

projectWith: projectAttributes

	| project |
	"Construct Metacello project"
	project := MetacelloMCProject new.
	project projectAttributes: projectAttributes.
	MetacelloVersionConstructor on: (Smalltalk at: #MetacelloTestConfigurationOfProjectToolBox) new project: project.
	project loader: MetacelloNullRecordingMCSpecLoader new.
	project loader evalDoits: true.
	^project
! !

!MetacelloConfigAPIToolBoxTestCase methodsFor:'running'!

configurationRepository
	"MetacelloConfigurationResource reset"

	^ MetacelloConfigurationResource current monticelloRepository
!

runCase
	| original |
	original := MetacelloPlatform current bypassGoferLoadUpdateCategories.
	[ 
	MetacelloPlatform current bypassGoferLoadUpdateCategories: true.
	^ SystemChangeNotifier uniqueInstance doSilently: [ super runCase ] ]
		ensure: [ MetacelloPlatform current bypassGoferLoadUpdateCategories: original ]
!

setUp
	| repo |
	super setUp.
	MetacelloConfigurationResource projectAttributes: nil.
	repo := self monticelloRepository.
	gofer := Gofer new.
	gofer disablePackageCache.
	gofer repository: repo.
	Smalltalk at: #'Metacello_Gofer_Test_Repository' put: repo.
	repo := self configurationRepository.
	gofer repository: repo.
	Smalltalk at: #'Metacello_Configuration_Test_Repository' put: repo.
	gofer package: 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer metacelloLoad.
	Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result' put: false.
	Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result' put: false.
!

tearDown
	| aGofer |
	super tearDown.
	aGofer := Gofer new.
	(self hasPackage: 'MetacelloTestConfigurationOfProjectToolBox')
		ifTrue: [ aGofer package: 'MetacelloTestConfigurationOfProjectToolBox' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectIssue115')
		ifTrue: [ aGofer package: 'MetacelloTestConfigurationOfProjectIssue115' ].
	aGofer references notEmpty ifTrue: [ aGofer metacelloUnload ].
	Smalltalk removeKey: #'Metacello_Gofer_Test_Repository' ifAbsent: [].
	Smalltalk removeKey: #'Metacello_Configuration_Test_Repository' ifAbsent: [].
	Smalltalk removeKey: #'Metacello_Configuration_Test_DoIt_Result' ifAbsent: [].
	Smalltalk removeKey: #'Metacello_Configuration_Test_POST_DoIt_Result' ifAbsent: [].
! !

!MetacelloConfigAPIToolBoxTestCase methodsFor:'test nested for:do:'!

testNestedForDoOrder1
	"evaluation of nested for:do: blocks is in breadth first order"

	| project version string expected |
	project := self projectWith: #(#'level1_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ].'.
	self assert: string = expected.
!

testNestedForDoOrder2

	| project version string expected |
	project := self projectWith: #(#'level1_1' #'level2_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder3

	| project version string expected |
	project := self projectWith: #(#'level1_1' #'level2_1' #'level3_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/5''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder4

	| project version string expected |
	project := self projectWith: #(#'level1_1' #'level2_1' #'level2_2').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder5

	| project version string expected |
	project := self projectWith: #(#'level1_1' #'level2_2' #'level2_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder6

	| project version string expected |
	project := self projectWith: #(#'level1_1' #'level2_1' #'level2_2' #'level3_2').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/7''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder7

	| project version string expected |
	project := self projectWith: #( #'level1_1' #'level2_1' #'level2_2' #'level3_2' #'level3_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/7''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/5''. ].'.
	self assert: string = expected.
	
!

testNestedForDoOrder8

	| project version string expected |
	project := self projectWith: #( #'level1_1' #'level2_2' #'level3_2' #'level2_1').
	version := project version: '1.3.1-baseline'.
	string := version spec printString.
	expected := 'spec blessing: #''baseline''.
spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/7''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ].'.
	self assert: string = expected.
	
! !

!MetacelloConfigAPIToolBoxTestCase methodsFor:'test package'!

testPackagePreloadOverride

	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.6-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
!

testPackagePreloadPostload
	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.5-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load.
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
! !

!MetacelloConfigAPIToolBoxTestCase methodsFor:'test project'!

testProjectCopyPackageRepositories
	| project version ref repositoryDescriptions string pkg expected |
	project := self project.

	version := project version: '1.3.0-baseline'.
	ref := version packageNamed: 'Copy Project'.
	repositoryDescriptions := ref projectPackage repositorySpecs collect: [:repoSpec | repoSpec description ].
	self assert: repositoryDescriptions size = 2.
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ob').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/or').
	string := ref printString.
	expected := 'spec 
	name: ''Copy Project'';
	className: ''MetacelloExampleProjectConfig'';
	versionString: ''1.0-baseline'';
	preLoadDoIt: #''preloadDoIt'';
	postLoadDoIt: #''postloadDoIt'';
	repository: ''http://www.example.com/ob'';
	repository: ''http://www.example.com/or''.'.
	self assert: string = expected
!

testProjectCopyPreloadOverride

	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.9-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load: 'Copy Project'.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
!

testProjectCopyPreloadPostload
	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.9-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load: 'Example Project'.
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
!

testProjectPreloadOverride

	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.8-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load: 'Example Project'.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
!

testProjectPreloadPostload
	| project versionSpec version |
	project := self project.

	versionSpec := (version := project version: '1.2.7-baseline') spec.
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
	version load: 'Example Project'.
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result').
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_POST_DoIt_Result').
! !

!MetacelloConfigAPIToolBoxTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigAPIToolBoxTestCase.st,v 1.1 2011/08/20 18:16:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigAPIToolBoxTestCase.st,v 1.1 2011/08/20 18:16:28 cg Exp $'
! !
