"{ Package: 'stx:goodies/metacello' }"

TestCase subclass:#MetacelloConfigAbstractAPITestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-TestsMC'
!

MetacelloConfigAbstractAPITestCase comment:''
!


!MetacelloConfigAbstractAPITestCase class methodsFor:'testing'!

isAbstract
	"Override to true if a TestCase subclass is Abstract and should not have
	TestCase instances built from it"

	^self name = #MetacelloConfigAbstractAPITestCase
			
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'accessing'!

project

	^self projectWith: #()
!

projectWith: projectAttributes

	self subclassResponsibility
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'private'!

conditionResult: resultString 
	"GemStone unconditionally prints Symbols with quotes, Squeak and Pharo do not"

	^(((resultString 
		copyReplaceAll: '#preloadForSupplyingAnswers' with: '#''preloadForSupplyingAnswers''')
			copyReplaceAll: '#postloadForSupplyingAnswers' with: '#''postloadForSupplyingAnswers''')
				copyReplaceAll: '#default' with: '#''default''')
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'test baselines'!

testNestedBaseline

	| project version packageNames |
	project := self projectWith: #().
	version := project version: '1.1.1-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 1.
	self assert: (packageNames includes: 'Example-Core').

	project := self projectWith: #(#extra).
	version := project version: '1.1.1-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 2.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Test').

	project := self projectWith: #(#nested).
	version := project version: '1.1.1-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 2.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Base').

	project := self projectWith: #(nested extra).
	version := project version: '1.1.1-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 3.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Base').
	self assert: (packageNames includes: 'Example-Test').
!

testNestedBaselineOrder

	| project version packageNames |
	project := self projectWith: #().
	version := project version: '1.1.2-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 1.
	self assert: (packageNames includes: 'Example-Core').

	project := self projectWith: #(#extra).
	version := project version: '1.1.2-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 2.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Test').

	project := self projectWith: #(#nested).
	version := project version: '1.1.2-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 3.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Base').
	self assert: (packageNames includes: 'Example-Test-Nested').

	project := self projectWith: #(nested extra).
	version := project version: '1.1.2-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 4.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Base').
	self assert: (packageNames includes: 'Example-Test').
	self assert: (packageNames includes: 'Example-Test-Nested').

	project := self projectWith: #(extra nested).
	version := project version: '1.1.2-baseline'.
	packageNames := version packages collect: [:pkg | pkg name].
	self assert: packageNames size = 4.
	self assert: (packageNames includes: 'Example-Core').
	self assert: (packageNames includes: 'Example-Base').
	self assert: (packageNames includes: 'Example-Test').
	self assert: (packageNames includes: 'Example-Test-Nested').
!

testNestedVersionOrder

	| project version fileNames |
	project := self projectWith: #(outer).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: fileNames size = 2.
	self assert: (fileNames includes: 'Example-Core-dkh.1').
	self assert: (fileNames includes: 'Example-Test-dkh.1').

	project := self projectWith: #(extra).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: fileNames size = 1.
	self assert: (fileNames includes: 'Example-Core-dkh.3').

	project := self projectWith: #(extra nested).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: fileNames size = 1.
	self assert: (fileNames includes: 'Example-Core-dkh.3').

	project := self projectWith: #(outer extra).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: fileNames size = 2.
	self assert: (fileNames includes: 'Example-Core-dkh.3').
	self assert: (fileNames includes: 'Example-Test-dkh.1').

	project := self projectWith: #(outer #nested).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: (fileNames includes: 'Example-Core-dkh.2').

	project := self projectWith: #(outer nested extra).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: (fileNames includes: 'Example-Core-dkh.3').

	project := self projectWith: #(outer extra nested).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: (fileNames includes: 'Example-Core-dkh.2').

	project := self projectWith: #(extra nested outer ).
	version := project version: '1.1.3'.
	fileNames := version packages collect: [:pkg | pkg file].
	self assert: (fileNames includes: 'Example-Core-dkh.1').
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'test package'!

testPackageIncludes
	| project versionSpec version pkg string expected |
	project := self project.

	versionSpec := (version := project version: '1.0.9-baseline') spec.
	pkg := version packageNamed: 'Example-Core'.
	string := pkg printString.
	expected := 'spec name: ''Example-Core'''.
	self assert: string = expected.
	pkg := version packageNamed: 'Example-Tests'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Tests'';
	includes: #(''Example-Core'' ).'.
	self assert: string = expected
!

testPackageIncludesRequiresSupplyingAnswers
	| project versionSpec version pkg string expected |
	project := self project.

	versionSpec := (version := project version: '1.1.0-baseline') spec.
	pkg := version packageNamed: 'Example-Test1'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Test1'';
	requires: #(''Example-Base'' );
	includes: #(''Example-Core'' ).'.
	self assert: string = expected.

	pkg := version packageNamed: 'Example-Test2'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Test2'';
	includes: #(''Example-Core'' );
	supplyingAnswers: #( #( ''captionMatch'' ''answer'' ) ).'.
	self assert: string = expected.

	pkg := version packageNamed: 'Example-Test3'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Test3'';
	requires: #(''Example-Base'' );
	supplyingAnswers: #( #( ''captionMatch'' ''answer'' ) ).'.
	self assert: string = expected.

	pkg := version packageNamed: 'Example-Test4'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Test4'';
	requires: #(''Example-Base'' );
	includes: #(''Example-Core'' );
	supplyingAnswers: #( #( ''captionMatch'' ''answer'' ) ).'.
	self assert: string = expected.
!

testPackageRepositories
	| project versionSpec version pkg repositoryDescriptions string expected |
	project := self project.

	versionSpec := (version := project version: '1.0.6-baseline') spec.
	pkg := version packageNamed: 'Example-Core'.
	repositoryDescriptions := pkg repositorySpecs collect: [:repoSpec | repoSpec description ].
	self assert: repositoryDescriptions size = 2.
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ob').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/or').
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Core'';
	repositories: [
		spec
			repository: ''http://www.example.com/ob'';
			repository: ''http://www.example.com/or'' ].'.
	self assert: string = expected
!

testPackageRepository
	| project versionSpec version pkg |
	project := self project.

	versionSpec := (version := project version: '1.0.5-baseline') spec.
	pkg := version packageNamed: 'Example-Core'.
	self assert: pkg repositorySpecs first description = 'http://www.example.com/ob'
!

testPackageRequires
	| project versionSpec version pkg string expected |
	project := self project.

	versionSpec := (version := project version: '1.0.8-baseline') spec.
	pkg := version packageNamed: 'Example-Core'.
	string := pkg printString.
	expected := 'spec name: ''Example-Core'''.
	self assert: string = expected.
	pkg := version packageNamed: 'Example-Tests'.
	string := pkg printString.
	expected := 'spec 
	name: ''Example-Tests'';
	requires: #(''Example-Core'' ).'.
	self assert: string = expected
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'test project'!

testProjectPackage
	| project version loader |
	project := self project.

	version := project version: '1.1.4-baseline'.
	loader := version record.
	loader loadDirective versionDirectivesDo: [:directive | | versionSpec |
		(versionSpec := directive spec) ~~ nil
			ifTrue: [ 
				versionSpec versionString = '1.1.4-baseline'
					ifTrue: [ | projectSpec |
						projectSpec := (versionSpec packageNamed: 'Example Project' ifAbsent: [ self assert: false]) projectReference.
						self assert: projectSpec projectPackage file =  projectSpec projectPackage file ]]].
!

testProjectPackageRepositories
	| project version ref repositoryDescriptions string pkg expected |
	project := self project.

	version := project version: '1.2.3-baseline'.
	ref := version packageNamed: 'Example Project'.
	repositoryDescriptions := ref projectPackage repositorySpecs collect: [:repoSpec | repoSpec description ].
	self assert: repositoryDescriptions size = 2.
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ob').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/or').
	string := ref printString.
	expected := 'spec 
	name: ''Example Project'';
	className: ''MetacelloExampleProjectConfig'';
	repository: ''http://www.example.com/ob'';
	repository: ''http://www.example.com/or''.'.
	self assert: string = expected
!

testProjectPackageRepository
	| project version ref string expected |
	project := self project.

	version := project version: '1.2.2-baseline'.
	ref := version packageNamed: 'Example Project'.
	self assert: ref projectPackage repositorySpecs first description = 'http://www.example.com/ob'.
	string := ref printString.
	expected := 'spec 
	name: ''Example Project'';
	className: ''MetacelloExampleProjectConfig'';
	repository: ''http://www.example.com/ob''.'.
	self assert: string = expected
! !

!MetacelloConfigAbstractAPITestCase methodsFor:'test version'!

testBlessing
	| project version |
	project := self project.

	version := project version: '1.0-baseline'.
	self assert: version blessing = #testBlessing
!

testBlessingBlock
	| project version |
	project := self project.

	version := project version: '1.0.1-baseline'.
	self assert: version blessing = #testBlessing
!

testIssue100
	"Issue 100:	last block wins when there are duplicate attribute blocks"

	| project version repositoryDescriptions |
	project := self project.

	version := project version: '1.2.4-baseline'.
	self assert: version blessing = #baseline.
	repositoryDescriptions := version spec repositorySpecs collect: [:repoSpec | repoSpec description ].
	self assert: repositoryDescriptions size = 2.
	self assert: (repositoryDescriptions includes: 'http://www.example.com/aa').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ab').

	version := project version: '1.2.4'.
	self assert: version blessing = #baseline.
	repositoryDescriptions := version spec repositorySpecs collect: [:repoSpec | repoSpec description ].
	self assert: repositoryDescriptions size = 3.
	self assert: (repositoryDescriptions includes: 'http://www.example.com/aa').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ab').
	self assert: (repositoryDescriptions includes: 'http://www.example.com/ac').
!

testVersionErrors
	| project  |
	project := self project.
	
	self should: [project version: 'does not exist' ] raise: MetacelloVersionDoesNotExistError.
	self should: [project version: #'does not exist' ] raise: MetacelloSymbolicVersionDoesNotExistError.

! !

!MetacelloConfigAbstractAPITestCase methodsFor:'utilities'!

hasPackage: aString
	| package |
	package := MCWorkingCopy allManagers
		detect: [ :each | each packageName = aString ]
		ifNone: [ nil ].
	^ package notNil
! !

!MetacelloConfigAbstractAPITestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigAbstractAPITestCase.st,v 1.1 2011/08/20 17:58:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigAbstractAPITestCase.st,v 1.1 2011/08/20 17:58:28 cg Exp $'
! !
