"{ Package: 'stx:goodies/metacello' }"

Object subclass:#MetacelloConfigTemplate
	instanceVariableNames:'project'
	classVariableNames:'LastVersionLoad'
	poolDictionaries:''
	category:'Metacello-MC-Model'
!

MetacelloConfigTemplate comment:'Copy me to create a new configuration or edit and evaluate the following doits.
        "Create configuration class and initial baseline method"
        MetacelloToolBox 
                createBaseline: ''1.0-baseline''
                for: ''MyProject''
                repository: ''http://www.example.com/MyProjectRepository''
                requiredProjects: #(''Gofer'')
                packages: #(''MyProject-Core'' ''MyProject-Tests'')
                dependencies:
                        {(''MyProject-Core'' -> #(''Gofer'')).
                         (''MyProject-Tests'' -> #(''MyProject-Core''))}
                groups:
                        {(''default'' -> #(''Core'')).
                        (''Core'' -> #(''MyProject-Core'')).
                        (''Tests'' -> #(''MyProject-Tests''))}.
	   "create initial development method from the baseline"
         MetacelloToolBox
               createDevelopment: ''1.0''
               for: ''MyProject''
                importFromBaseline: ''1.0-baseline''
                description: ''initial version''.
'
!


!MetacelloConfigTemplate class methodsFor:'accessing'!

project

	^self new project
! !

!MetacelloConfigTemplate class methodsFor:'development support'!

DevelopmentSupport

"See the methods in the 'development support' category on the class-side of MetacelloBaseConfiguration. Decide what development support methods you would like to use and copy them the the class-side of your configuration."
	<apiDocumentation>
!

validate
	"Check the configuration for Errors, Critical Warnings, and Warnings (see class comment for MetacelloMCVersionValidator for more information). 
	Errors identify specification issues that will result in unexpected behaviour when you load the configuration. 
	Critical Warnings identify specification issues that may result in unexpected behavior when you load the configuration.
	Warnings identify specification issues that are technically correct, but are worth take a look at."

	"self validate"

	<apiDocumentation>
	self ensureMetacello.
	^ ((Smalltalk at: #MetacelloToolBox) validateConfiguration: self debug: #() recurse: false) explore
! !

!MetacelloConfigTemplate class methodsFor:'loading'!

load
	"Load the #stable version defined for this platform. The #stable version is the version that is recommended to be used on this platform."

	"self load"

	<apiDocumentation>
	^(self project version: #stable) load
!

loadBleedingEdge
	"Load the latest versions of the mcz files defined for this project. It is not likely that the #bleedingEdge has been tested."

	"self loadBleedingEdge"

	<apiDocumentation>
	^(self project version: #bleedingEdge) load
!

loadDevelopment
	"Load the #development version defined for this platform. The #development version will change over time and is not expected to be stable."

	"self loadDevelopment"

	<apiDocumentation>
	^(self project version: #development) load
! !

!MetacelloConfigTemplate class methodsFor:'metacello tool support'!

isMetacelloConfig
	"Answer true and the Metacello tools will operate on you"
	
	^true
! !

!MetacelloConfigTemplate class methodsFor:'private'!

baseConfigurationClassIfAbsent: aBlock

	^Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			self ensureMetacelloBaseConfiguration.
			Smalltalk at: #MetacelloBaseConfiguration ifAbsent: aBlock ].
!

ensureMetacello

	(self baseConfigurationClassIfAbsent: []) ensureMetacello
!

ensureMetacelloBaseConfiguration

	Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			| repository version |
			repository := MCHttpRepository location: 'http://seaside.gemstone.com/ss/metacello' user: '' password: ''.
			repository
				versionReaderForFileNamed: 'Metacello-Base-DaleHenrichs.2.mcz'
				do: [ :reader | 
					version := reader version.
					version load.
					version workingCopy repositoryGroup addRepository: repository ] ]
! !

!MetacelloConfigTemplate methodsFor:'accessing'!

project

	^ project ifNil: [ | constructor |
		"Bootstrap Metacello if it is not already loaded"
		(self class baseConfigurationClassIfAbsent: []) ensureMetacello.
		"Construct Metacello project"
		constructor := (Smalltalk at: #MetacelloVersionConstructor) on: self.
		project := constructor project.
		project loadType: #linear. "change to #atomic if desired"
		project ]
! !

!MetacelloConfigTemplate class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigTemplate.st,v 1.2 2011/08/20 18:15:33 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloConfigTemplate.st,v 1.2 2011/08/20 18:15:33 cg Exp $'
! !
