"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloDetermineCurrentVersionForLoadTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloDetermineCurrentVersionForLoadTest comment:''
!


!MetacelloDetermineCurrentVersionForLoadTest methodsFor:'running'!

setUp

	| repo |
	super setUp.
	repo := self configurationRepository.
	self tempRepositories add: repo.
	gofer repository: repo
!

tearDownPackages: aGoferInstance

	super tearDownPackages: aGoferInstance.
	(self hasPackage: 'MetacelloTestConfigurationOfFan')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfFan' ].
	(self hasPackage: 'MetacelloTestConfigurationOfFoo')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfFoo' ].
! !

!MetacelloDetermineCurrentVersionForLoadTest methodsFor:'tests'!

test001

	| version foo fan wc |
	gofer
		package: 'MetacelloTestConfigurationOfFan'; 
		package: 'MetacelloTestConfigurationOfFoo'; 
		load.
	fan := (Smalltalk at: #MetacelloTestConfigurationOfFan) new project.
	foo := (Smalltalk at: #MetacelloTestConfigurationOfFoo) new project.

	version := foo version: '3.0'.
	version load: #( 'GoferBar' 'GoferFaux').

	"When we load version 2.0 of fan, we recognize that '3.0' of foo is the currentVersion (#allLoadedToSpec 
	 or #loadedToSpec and latestVersion), so we load the packages from version 3.0 of foo instead" 
	version := fan version: '2.0'.
	version load: 'Foo'.

	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
!

test002

	| version foo fan wc |
	gofer
		package: 'MetacelloTestConfigurationOfFan'; 
		package: 'MetacelloTestConfigurationOfFoo'; 
		load.
	fan := (Smalltalk at: #MetacelloTestConfigurationOfFan) new project.
	foo := (Smalltalk at: #MetacelloTestConfigurationOfFoo) new project.

	version := foo version: '5.0'.
	version load: #( 'GoferBar' 'GoferFaux').

	"load older version of 1 package (that doesn't match spec of earlier version)"
	gofer
		version: 'GoferFaux-tg.33'; 
		load.

	version := fan version: '2.0'.
	self flag: 'revisit this'.
	"self 
		should: [ version load: 'Foo' ]
		raise: MetacelloProjectSpecLoadConflict"
	version load: 'Foo'

!

test003

	| version foo fan wc |
	gofer
		package: 'MetacelloTestConfigurationOfFan'; 
		package: 'MetacelloTestConfigurationOfFoo'; 
		load.
	fan := (Smalltalk at: #MetacelloTestConfigurationOfFan) new project.
	foo := (Smalltalk at: #MetacelloTestConfigurationOfFoo) new project.

	version := foo version: '5.0'.
	version load: #( 'GoferBar' 'GoferFaux').

	"load later version of 1 package beyond the last version of foo 5.0"
	gofer
		version: 'GoferFaux-tg.35'; 
		load.

	version := fan version: '2.0'.
	version load: 'Foo'.

	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.55').
!

test004

	| version foo fan wc |
	gofer
		package: 'MetacelloTestConfigurationOfFan'; 
		package: 'MetacelloTestConfigurationOfFoo'; 
		load.
	fan := (Smalltalk at: #MetacelloTestConfigurationOfFan) new project.
	foo := (Smalltalk at: #MetacelloTestConfigurationOfFoo) new project.

	version := foo version: '5.0'.
	version load: #( 'GoferBar' 'GoferFaux').

	version := fan version: '2.0'.
	version load: 'Far'.

	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.55').
!

testIssue112

	| version foo wc |
	gofer
		package: 'MetacelloTestConfigurationOfFoo'; 
		load.
	foo := (Smalltalk at: #MetacelloTestConfigurationOfFoo) new project.

	version := foo version: '3.0'.
	version load: #( 'GoferBar' 'GoferFaux').

	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.30').
	self should: [(GoferPackageReference name: 'GoferBeau') workingCopy] raise: Error.

	"A load of GoferBeau for version 3.5, should also load the correct package for GoferFaux (GoferFaux-tg.31)" 
	version := foo version: '3.5'.
	version load: 'GoferBeau'.

	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.1').
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.54').
	"this is the crux of Issue 112"
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.31').
! !

!MetacelloDetermineCurrentVersionForLoadTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloDetermineCurrentVersionForLoadTest.st,v 1.1 2011/08/20 18:16:22 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloDetermineCurrentVersionForLoadTest.st,v 1.1 2011/08/20 18:16:22 cg Exp $'
! !
