"{ Package: 'stx:goodies/metacello' }"

TestCase subclass:#MetacelloDevelopmentCycleTutorialTests
	instanceVariableNames:'authorName tempRepositories'
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloDevelopmentCycleTutorialTests comment:''
!


!MetacelloDevelopmentCycleTutorialTests methodsFor:'accessing'!

tempRepositories

	tempRepositories ifNil: [ tempRepositories := OrderedCollection new ].
	^tempRepositories
! !

!MetacelloDevelopmentCycleTutorialTests methodsFor:'lessons'!

createConfiguration

	MetacelloToolBox
		createBaseline: '1.0-baseline'
		for: self configurationName asString
		repository: 'dictionary://', self repositoryName asString
		requiredProjects: #('MetacelloTestConfigurationOfFoo')
		packages: #('GeauxFaux' 'GeauxBeau')
		dependencies:
			{('GeauxFaux' -> #('MetacelloTestConfigurationOfFoo')).
			('GeauxBeau' -> #('GeauxFaux'))}
		groups:
			{('default' -> #('Core')).
			('Core' -> #('GeauxFaux')).
			('Tests' -> #('GeauxBeau')).
			('Core Tests' -> #('Core' 'Tests'))}.
!

createDevelopment

	MetacelloToolBox
		createDevelopment: '1.0'
		for: self configurationName asString
		importFromBaseline: '1.0-baseline'
		description: 'initial version'.
!

createNewBaseline

	MetacelloToolBox
		createNewBaselineVersionIn: (Smalltalk at: self configurationName)
		description: ''.
!

createNewDevelopment

	MetacelloToolBox
		createNewDevelopmentVersionIn: (Smalltalk at: self configurationName)
		description: ''.
!

releaseDevelopment

	MetacelloToolBox 
		releaseDevelopmentVersionIn: (Smalltalk at: self configurationName)
		description: '- release version 1.0'.
	MetacelloToolBox 
		copyConfiguration: (Smalltalk at: self configurationName) 
		to: 'dictionary://', self alternatRepositoryName asString
!

validateAndSave

	| wc |
	MetacelloToolBox validateConfiguration: (Smalltalk at: self configurationName).
	PackageInfo registerPackageName: self configurationName asString.
	"register workingCopy for gofer to use"
	wc := MCWorkingCopy forPackage: (MetacelloTestsMCPackage new name: self configurationName asString).
	(wc repositoryGroup respondsTo: #disableCache)
		ifTrue: [wc repositoryGroup perform: #disableCache].
	Gofer new
		disablePackageCache;
		repository: (Smalltalk at: self repositoryName);
		package: self configurationName asString;
		commit: 'Initial configuration'.
	"fix it so that the directory repository is used for subsequent saves"
	wc := (Smalltalk at: self configurationName) project projectPackage workingCopy.
	wc repositoryGroup addRepository: (Smalltalk at: self repositoryName).
	MetacelloToolBox saveConfigurationPackageFor: self configurationName asString description: 'commit message'.
! !

!MetacelloDevelopmentCycleTutorialTests methodsFor:'private'!

alternatRepositoryName

	^#'Metacello_Dev_Cycle_Alternate_Repository'
!

configurationName

	^#'ConfigurationOfMetacello_Dev_Cycle'
!

hasPackage: aString
	| package |
	package := MCWorkingCopy allManagers
		detect: [ :each | each packageName = aString ]
		ifNone: [ nil ].
	^ package notNil
!

repositoryName

	^#'Metacello_Dev_Cycle_Repository'
!

version10CommonVersionSpecString

	^'
spec description: ''initial version''.
spec author: ''TesterBob''.
spec timestamp: ''''.
spec repository: ''dictionary://Metacello_Dev_Cycle_Repository''.
spec 
	project: ''MetacelloTestConfigurationOfFoo'' with: [
		spec
			className: ''MetacelloTestConfigurationOfFoo'';
			versionString: #''bleedingEdge'';
			repository: ''cache'' ];
	project: ''MetacelloTestConfigurationOfFoo'' with: ''4.0''.
spec 
	package: ''GeauxFaux'' with: [
		spec requires: #(''MetacelloTestConfigurationOfFoo'' ). ];
	package: ''GeauxBeau'' with: [
		spec requires: #(''GeauxFaux'' ). ];
	package: ''GeauxFaux'' with: ''GeauxFaux-tg.32'';
	package: ''GeauxBeau'' with: ''GeauxBeau-dkh.55''.
spec 
	group: ''default'' with: #(''Core'' );
	group: ''Core'' with: #(''GeauxFaux'' );
	group: ''Tests'' with: #(''GeauxBeau'' );
	group: ''Core Tests'' with: #(''Core'' ''Tests'' ).'
!

version10DevelopmentVersionSpecString

	^'spec blessing: #''development''.', self version10CommonVersionSpecString
!

version10ReleaseVersionSpecString

	^'spec blessing: #''release''.', self version10CommonVersionSpecString
!

version11CommonVersionSpecString

	^'
spec description: ''''.
spec author: ''TesterBob''.
spec timestamp: ''''.
spec repository: ''dictionary://Metacello_Dev_Cycle_Repository''.
spec 
	project: ''MetacelloTestConfigurationOfFoo'' with: [
		spec
			className: ''MetacelloTestConfigurationOfFoo'';
			versionString: #''bleedingEdge'';
			repository: ''cache'' ];
	project: ''MetacelloTestConfigurationOfFoo'' with: ''4.0''.
spec 
	package: ''GeauxFaux'' with: [
		spec requires: #(''MetacelloTestConfigurationOfFoo'' ). ];
	package: ''GeauxBeau'' with: [
		spec requires: #(''GeauxFaux'' ). ];
	package: ''GeauxFaux'' with: ''GeauxFaux-tg.32'';
	package: ''GeauxBeau'' with: ''GeauxBeau-dkh.55''.
spec 
	group: ''default'' with: #(''Core'' );
	group: ''Core'' with: #(''GeauxFaux'' );
	group: ''Tests'' with: #(''GeauxBeau'' );
	group: ''Core Tests'' with: #(''Core'' ''Tests'' ).'
!

version11DevelopmentVersionSpecString

	^'spec blessing: #''development''.', self version11CommonVersionSpecString
!

versionCommonBaselineSpecString

	^'spec blessing: #''baseline''.
spec repository: ''dictionary://Metacello_Dev_Cycle_Repository''.
spec project: ''MetacelloTestConfigurationOfFoo'' with: [
		spec
			className: ''MetacelloTestConfigurationOfFoo'';
			versionString: #''bleedingEdge'';
			repository: ''cache'' ].
spec 
	package: ''GeauxFaux'' with: [
		spec requires: #(''MetacelloTestConfigurationOfFoo'' ). ];
	package: ''GeauxBeau'' with: [
		spec requires: #(''GeauxFaux'' ). ].
spec 
	group: ''default'' with: #(''Core'' );
	group: ''Core'' with: #(''GeauxFaux'' );
	group: ''Tests'' with: #(''GeauxBeau'' );
	group: ''Core Tests'' with: #(''Core'' ''Tests'' ).'
! !

!MetacelloDevelopmentCycleTutorialTests methodsFor:'running'!

runCase
	| original |
	original := MetacelloPlatform current bypassGoferLoadUpdateCategories.
	[ 
	MetacelloPlatform current bypassGoferLoadUpdateCategories: true.
	^ SystemChangeNotifier uniqueInstance doSilently: [ super runCase ] ]
		ensure: [ MetacelloPlatform current bypassGoferLoadUpdateCategories: original ]
!

setUp
	| repo gofer |
	super setUp.
	repo := MCDictionaryRepository new.
	repo description: 'dictionary://', self repositoryName asString.
	self tempRepositories add: repo.
	Smalltalk at: self repositoryName put: repo.
	repo := MCDictionaryRepository new.
	repo description: 'dictionary://', self alternatRepositoryName asString.
	self tempRepositories add: repo.
	Smalltalk at: self alternatRepositoryName put: repo.
	Smalltalk at: self configurationName ifPresent: [:cl | cl removeFromSystem ].

	repo := MetacelloConfigurationResource current monticelloRepository.
	self tempRepositories add: repo.
	Smalltalk at: #'Metacello_Configuration_Test_Repository' put: repo.
	gofer := Gofer new
		repository: repo;
		package: 'MetacelloTestConfigurationOfProjectFee'; 
		metacelloLoad.
	repo := MetacelloMonticelloResource current monticelloRepository.
	repo description: 'dictionary://Metacello_Gofer_Test_Repository'.
	self tempRepositories add: repo.
	Smalltalk at: #'Metacello_Gofer_Test_Repository' put: repo.

	(Smalltalk at: #MetacelloTestConfigurationOfProjectFee) new project load: '4.0'.
	
	authorName := MetacelloPlatform current authorName.
	MetacelloPlatform current authorName: 'TesterBob'.
!

tearDown
	| aGofer |
	super tearDown.
	aGofer := Gofer new.
	self tearDownPackages: aGofer.
	aGofer references notEmpty ifTrue: [ aGofer metacelloUnload ].
	Smalltalk removeKey: self repositoryName ifAbsent: [  ].
	Smalltalk removeKey: self alternatRepositoryName  ifAbsent: [  ].
	Smalltalk removeKey: #'Metacello_Gofer_Test_Repository' ifAbsent: [].
	Smalltalk removeKey: #'Metacello_Configuration_Test_Repository' ifAbsent: [].
	Smalltalk at: self configurationName ifPresent: [ :cl | cl removeFromSystem ].
	self tempRepositories do: [:repo | MCRepositoryGroup default removeRepository: repo ].
	(self hasPackage: self configurationName asString)
		ifTrue: [ 
			(Gofer new)
				package: self configurationName asString;
				metacelloUnload ].

	MetacelloPlatform current authorName: authorName.
!

tearDownPackages: aGoferInstance

	(self hasPackage: 'GoferFoo')
		ifTrue: [ aGoferInstance package: 'GoferFoo' ].
	(self hasPackage: 'GoferBar')
		ifTrue: [ aGoferInstance package: 'GoferBar' ].
	(self hasPackage: 'GoferFaux')
		ifTrue: [ aGoferInstance package: 'GoferFaux' ].
	(self hasPackage: 'GoferBeau')
		ifTrue: [ aGoferInstance package: 'GoferBeau' ].
	(self hasPackage: 'GeauxFaux')
		ifTrue: [ aGoferInstance package: 'GeauxFaux' ].
	(self hasPackage: 'GeauxBeau')
		ifTrue: [ aGoferInstance package: 'GeauxBeau' ].
	(self hasPackage: 'MetacelloTestConfigurationOfFoo')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfFoo' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectFee')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfProjectFee' ].
! !

!MetacelloDevelopmentCycleTutorialTests methodsFor:'tests'!

testCreateConfiguration
	| expected output |
	self createConfiguration.
	expected := self versionCommonBaselineSpecString.
	Smalltalk
		at: self configurationName
		ifPresent: [ :cl | 
			output := (cl project version: '1.0-baseline') versionSpec printString.
			self assert: output = expected ]
!

testCreateDevelopment
	| expected output |
	self
		createConfiguration;
		createDevelopment;
		yourself.
	expected := self version10DevelopmentVersionSpecString.
	Smalltalk
		at: self configurationName
		ifPresent: [ :cl | | versionSpec |
			versionSpec := (cl project version: '1.0') versionSpec.
			versionSpec timestamp: ''.
			output :=  versionSpec printString.
			self assert: output = expected.
			versionSpec := (cl project version: #development) versionSpec.
			versionSpec timestamp: ''.
			output :=  versionSpec printString.
			self assert: output = expected ]
!

testCreateNewBaseline
	| expected output |
	self
		createConfiguration;
		createDevelopment;
		validateAndSave;
		releaseDevelopment;
		createNewDevelopment;
		createNewBaseline;
		yourself.
	expected := self versionCommonBaselineSpecString.
	Smalltalk
		at: self configurationName
		ifPresent: [ :cl | 
			output := (cl project version: '1.1-baseline') versionSpec printString.
			self assert: output = expected ]
!

testCreateNewDevelopment
	| expected output cl versionSpec report |
	self
		createConfiguration;
		createDevelopment;
		validateAndSave;
		releaseDevelopment;
		createNewDevelopment;
		yourself.
	cl := Smalltalk at: self configurationName.
	expected := self version10ReleaseVersionSpecString.
	versionSpec := (cl project version: '1.0') versionSpec.
	versionSpec timestamp: ''.
	output := versionSpec printString.
	self assert: output = expected.

	expected := self version11DevelopmentVersionSpecString.
	versionSpec := (cl project version: '1.1') versionSpec.
	versionSpec timestamp: ''.
	output := versionSpec printString.
	self assert: output = expected.

	versionSpec := (cl project version: #development) versionSpec.
	versionSpec timestamp: ''.
	output := versionSpec printString.
	self assert: output = expected.
	
	report := MetacelloToolBox compareVersionsIn: cl.
	#(additions modifications removals) do: [:key |
		self assert: (report perform: key) keys size = 0 ].
	self assert: (report configuration) = self configurationName asString.
	self assert: (report from) = '1.0'.
	self assert: (report to) = '1.1'.
!

testReleaseDevelopment
	| repo repoVersionsSize altRepo altRepoVersionsSize output expected |
	repo := Smalltalk at: self repositoryName.
	repoVersionsSize := repo allVersionInfos size.
	altRepo := Smalltalk at: self alternatRepositoryName.
	altRepoVersionsSize := altRepo allVersionInfos size.
	self
		createConfiguration;
		createDevelopment;
		validateAndSave;
		releaseDevelopment;
		yourself.
	self assert: repo allVersionInfos size = (repoVersionsSize + 2).
	self assert: altRepo allVersionInfos size = (altRepoVersionsSize + 1).
	expected := self version10ReleaseVersionSpecString.
	Smalltalk
		at: self configurationName
		ifPresent: [ :cl | | versionSpec |
			versionSpec := (cl project version: '1.0') versionSpec.
			versionSpec timestamp: ''.
			output :=  versionSpec printString.
			self assert: output = expected.
			self should: [ cl project version: #development ] raise: Error.
			versionSpec := (cl project version: #stable) versionSpec.
			versionSpec timestamp: ''.
			output :=  versionSpec printString.
			self assert: output = expected ]
!

testValidateAndSave
	| repo allVersionsSize |
	repo := Smalltalk at: self repositoryName.
	allVersionsSize := repo allVersionInfos size.
	self
		createConfiguration;
		createDevelopment;
		validateAndSave;
		yourself.
	self assert: (MetacelloToolBox validateConfiguration: (Smalltalk at: self configurationName)) isEmpty.
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
! !

!MetacelloDevelopmentCycleTutorialTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloDevelopmentCycleTutorialTests.st,v 1.1 2011/08/20 18:17:58 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloDevelopmentCycleTutorialTests.st,v 1.1 2011/08/20 18:17:58 cg Exp $'
! !
