"{ Package: 'stx:goodies/metacello' }"

MetacelloCommonMCSpecLoader subclass:#MetacelloFetchingMCSpecLoader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Loaders'
!

MetacelloFetchingMCSpecLoader comment:''
!


!MetacelloFetchingMCSpecLoader methodsFor:'accessing'!

actionLabel

	^'Fetching '
!

cacheRepository

	^self loaderPolicy cacheRepository
!

ensuredMap

	^self loaderPolicy ensuredMap
!

loadData

	^self loaderPolicy loadData
!

loadDirective

	^self loaderPolicy loadDirective
!

loadingSpecLoader

	^(MetacelloLoadingMCSpecLoader on: self spec)
		shouldDisablePackageCache: self shouldDisablePackageCache;
		loaderPolicy: self loaderPolicy copy;
		yourself
! !

!MetacelloFetchingMCSpecLoader methodsFor:'actions'!

doLoad

	self loaderPolicy copy load
!

linearLoadPackageSpec: packageSpec gofer: gofer

	MetacelloPlatform current
		do:  [ | references nearestReference cachedReference externalReference mcVersion loadedVersionInfos |
			"check to see if mcz file is already in cacheRepository"
			self flag: #cleanup. "validate the comment"
			"packageSpec name = 'OmniBrowser' ifTrue: [self halt: packageSpec file ]."
			cachedReference := self resolvePackageSpec: packageSpec cachedGofer: self loaderPolicy cacheGofer.
			(cachedReference ~~ nil and: [ packageSpec getFile ~~ nil ])
				ifTrue: [	 cachedReference name = packageSpec file ifTrue: [ ^self ]].
			"look up mcz file"
			references := self retryingResolvePackageSpecReferences: packageSpec gofer: gofer.
			nearestReference := references last asMetacelloCachingResolvedReference.
			"If the mcz is already in the cacheRepository, no need to copy"
			(cachedReference ~~ nil and: [ cachedReference name = nearestReference name ])
				ifTrue: [ ^self ].
			"If the mcz is already loaded into the image, no need to copy"
			(self ignoreImage not 
				and: [ (loadedVersionInfos := self ancestorsFor: packageSpec) ~~ nil ])
					ifTrue: [ loadedVersionInfos do: [:info | info name = nearestReference name ifTrue: [ ^self ]]].
			externalReference := (references select: [:ref | ref name = nearestReference name]) first asMetacelloCachingResolvedReference.
			self repositoryMap at: externalReference name put: externalReference repository.
			(self resolveDependencies: externalReference nearest: nearestReference into: (OrderedCollection with: nearestReference)) do: [:reference | | pSpec l |
				mcVersion := reference version.
				(l := (GoferVersionReference name: reference name) resolveAllWith: self loaderPolicy cacheGofer) isEmpty 
					ifTrue:  [ 
						self cacheRepository storeVersion: mcVersion.
				reference == nearestReference
					ifTrue: [ pSpec := packageSpec ]
					ifFalse: [ 
						pSpec :=  packageSpec project packageSpec.
						pSpec name: mcVersion info ].
				self loadData 
					addVersion: mcVersion 
					versionInfo: mcVersion info 
					resolvedReference: reference 
					packageSpec: pSpec ]].
			self loaderPolicy resetCacheGofer.
			self preLoad: packageSpec.
			(MetacelloDirective loadPackage: packageSpec externalReference: externalReference loader: self) addTo: self loadDirective.
			self postLoad: packageSpec.
			Transcript cr; show: 'Fetched -> ', externalReference name, ' --- ', externalReference repository description, ' --- ', nearestReference repository description ]
		displaying: 'Fetching ', packageSpec file
! !

!MetacelloFetchingMCSpecLoader methodsFor:'doits'!

postLoad: packageOrVersionSpec

	(MetacelloDirective postLoadSpec: packageOrVersionSpec loader: self)  addTo: self loadDirective

!

preLoad: packageOrVersionSpec

	(MetacelloDirective preLoadSpec: packageOrVersionSpec loader: self) addTo: self loadDirective
! !

!MetacelloFetchingMCSpecLoader methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	self loadDirective printOn: aStream.
	aStream nextPut: $)
! !

!MetacelloFetchingMCSpecLoader methodsFor:'private'!

atomicLoadPackageSpecs: packageSpecs repositories: repositories

	self loaderPolicy 
		pushAtomicLoadDirectivesDuring: [ super linearLoadPackageSpecs: packageSpecs repositories: repositories ]
		for: self
!

explicitLoadPackageSpecs: packageSpecs repositories: repositories
	
	| directive |
	directive := self loaderPolicy 
		pushExplicitLoadDirectivesDuring: [ super linearLoadPackageSpecs: packageSpecs repositories: repositories ]
		for: self.
	directive explicitLoadWithPolicy: self loaderPolicy.
!

linearLoadPackageSpecs: packageSpecs repositories: repositories
	
	self loaderPolicy 
		pushLinearLoadDirectivesDuring: [ super linearLoadPackageSpecs: packageSpecs repositories: repositories ]
		 for: self
!

resolveDependencies: aResolvedReference nearest: nearestReference into: aCollection

	| block retryCount coll notDone |
	self flag: 'Not used, but retained to avoid upgrade issues'.
	block := [ :dependency | | reference |
		reference := MetacelloCachingGoferResolvedReference
			name: dependency versionInfo name
			repository: aResolvedReference repository.
		coll add: reference.
		Transcript cr; show: 'Fetched dependency -> ', reference name, ' --- ', reference repository description.
		self resolveDependencies: reference nearest: reference into: coll].
	retryCount := 0.
	notDone := true.
	coll := OrderedCollection new.
	[ notDone and: [ retryCount < 3 ]]
		whileTrue: [
			retryCount > 0 ifTrue: [ Transcript cr; show: '...RETRY' ].
			[ 	
				"ensure that all resolved references have cached their version while wrapped by error handler"
				aCollection do: [:each | each version ]. 
				nearestReference version dependencies do: block.
				notDone := false ]
					on: Error 
					do: [:ex | 
						retryCount := retryCount + 1.
						retryCount >= 3 ifTrue: [ ex pass ].
						coll := OrderedCollection new ]].
	aCollection addAll: coll.
	^aCollection
!

resolvePackageSpec: packageSpec cachedGofer: gofer

	|  versionReference references |
	versionReference := packageSpec goferLoaderReference.
	(references := versionReference resolveAllWith: gofer) isEmpty 
		ifTrue: [ ^nil ].
	^references last asMetacelloCachingResolvedReference.
! !

!MetacelloFetchingMCSpecLoader methodsFor:'versionInfo'!

ancestorsFor: packageSpec

	^self loadData
		ancestorsFor: packageSpec 
		ifAbsent: [ super ancestorsFor: packageSpec ]
!

currentVersionInfoFor: packageSpec

	^self loadData
		currentVersionInfoFor: packageSpec 
		ifAbsent: [ super currentVersionInfoFor: packageSpec ]
! !

!MetacelloFetchingMCSpecLoader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloFetchingMCSpecLoader.st,v 1.1 2011/08/20 18:04:23 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloFetchingMCSpecLoader.st,v 1.1 2011/08/20 18:04:23 cg Exp $'
! !
