"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloGoferBasedLoaderTest
	instanceVariableNames:'postloadAnswer postLoadExecuted preLoadExecuted'
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloGoferBasedLoaderTest comment:''
!


!MetacelloGoferBasedLoaderTest methodsFor:'baselines'!

baseline10: spec 
	<version: '1.0-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo';
			package: 'GoferBar';
			yourself ].
!

baseline11: spec 
	<version: '1.1-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo' with: [
				spec 
					preLoadDoIt: #preloadExecute;
					postLoadDoIt: #postloadExecute ];
			yourself ].
!

baselineIssue40: spec 
	<version: 'Issue40-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo';
			package: 'GoferBar' with: [
				spec repository: 'dictionary://Metacello_Configuration_Test_Alternate_Repository' ];
			yourself ].
!

baselineIssue77: spec 
	<version: 'Issue77-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: 'B' with: [
				spec
					className: 'MetacelloTestConfigurationOfIssue77B';
					versionString: '1.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			project: 'D' with: [
				spec
					className: 'MetacelloTestConfigurationOfIssue77D';
					versionString: '1.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself ].
!

baselineIssue95: spec 
	<version: 'Issue95-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: 'Issue95' with: [
				spec
					className: 'MetacelloTestConfigurationOfProjectIssue95';
					versionString: '4.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself ].
!

baselineMoose: spec 
	<version: '1.0-Moose'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFaux' with: [
				spec file: 'GoferFaux-' ];
			yourself ].
! !

!MetacelloGoferBasedLoaderTest methodsFor:'doits'!

postloadExecute

	postLoadExecuted := true
!

postloadSupplyingAnswers

	postloadAnswer := ProvideAnswerNotification signal: 'postload'.
!

preloadExecute

	preLoadExecuted := true
! !

!MetacelloGoferBasedLoaderTest methodsFor:'running'!

setUp

	super setUp.
	Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result' put: false.
	preLoadExecuted := postLoadExecuted := false.
!

tearDown
	super tearDown.
	Smalltalk removeKey: #'Metacello_Configuration_Test_DoIt_Result' ifAbsent: [].
	MetacelloConfigurationResource projectAttributes: nil.
!

tearDownPackages: aGoferInstance

	super tearDownPackages: aGoferInstance.
	(self hasPackage: 'GeauxBar')
		ifTrue: [ aGoferInstance package: 'GeauxBar' ].
	(self hasPackage: 'GeauxBeau')
		ifTrue: [ aGoferInstance package: 'GeauxBeau' ].
	(self hasPackage: 'MetacelloTestConfigurationOfFix')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfFix' ].
	(self hasPackage: 'MetacelloTestConfigurationOfIssue77B')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfIssue77B' ].
	(self hasPackage: 'MetacelloTestConfigurationOfIssue77C')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfIssue77C' ].
	(self hasPackage: 'MetacelloTestConfigurationOfIssue77D')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfIssue77D' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectLoop')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfProjectLoop' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectInfinite')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfProjectInfinite' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectIssue95')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfProjectIssue95' ].
	(self hasPackage: 'MetacelloTestConfigurationOfProjectToolBox')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfProjectToolBox' ].
! !

!MetacelloGoferBasedLoaderTest methodsFor:'tests'!

testBaseline11PrePostLoadFetch

	| project version |
	project := self project.
	version := project version: '1.1-baseline'.
	self deny: preLoadExecuted.
	self deny: postLoadExecuted.
	version fetch.
	self deny: preLoadExecuted.
	self deny: postLoadExecuted.
!

testBaseline11PrePostLoadLoad

	| project version loader |
	project := self project.
	version := project version: '1.1-baseline'.
	self deny: preLoadExecuted.
	self deny: postLoadExecuted.
	version load.
	self assert: preLoadExecuted.
	self assert: postLoadExecuted.
!

testBaselineFetch

	| project version loader files |
	project := self project.
	version := project version: '1.0-baseline'.
	loader := version fetch.
	files := #('GoferFoo-lr.4' 'GoferBar-lr.1') asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | 
		files remove: pkgDirective file ifAbsent: [ self assert: false ]].
	loader loadDirective versionDirectivesDo: [:vrsnDirective |
		vrsnDirective loader ~~ nil
			ifTrue: [ | spec |
				spec := vrsnDirective spec.
				self assert: (spec isKindOf: MetacelloVersionSpec) ]].
	self assert: files isEmpty
!

testBaselineLoad

	| project version wc |
	project := self project.
	version := project version: '1.0-baseline'.
	version load.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferBar-lr.1'
!

testBaselineRecord

	| project version loader files |
	project := self project.
	version := project version: '1.0-baseline'.
	loader := version record.
	files := #('GoferFoo' 'GoferBar') asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | 
		files remove: pkgDirective packageName ifAbsent: [ self assert: false ]].
	self assert: files isEmpty
!

testIssue40Baseline

	| project version wc1 wc2 |
	project := self project.
	version := project version: 'Issue40-baseline'.
	version load.
	wc1 := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: wc1 repositoryGroup repositories size = 2.
	self assert: (wc1 repositoryGroup includes: (Smalltalk at: #'Metacello_Gofer_Test_Repository')).
	self assert: (wc1 ancestry ancestors first name = 'GoferFoo-lr.4').
	wc2 := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc2 ancestry ancestors first name = 'GoferBar-dkh.25').
	self assert: wc2 repositoryGroup repositories size = 2.
	self assert: (wc2 repositoryGroup includes: (Smalltalk at: #'Metacello_Configuration_Test_Alternate_Repository')).
!

testIssue49PartialLoadA

	| project version wc |
	project := self project.
	version := project version: '4.0'.
	version load: '1'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.2').
	self should: [ (GoferPackageReference name: 'GoferFaux') workingCopy ] raise: Error.
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue49PartialLoadB

	| project version wc |
	project := self project.
	version := project version: '5.0'.
	version load: '2'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	self should: [ (GoferPackageReference name: 'GoferBar') workingCopy ] raise: Error.
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue49PartialLoadC

	| project version wc |
	project := self project.
	version := project version: '4.0'.
	version load: '1'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.2').
	self should: [ (GoferPackageReference name: 'GoferFaux') workingCopy ] raise: Error.
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
	version := project version: '5.0'.
	version load: '2'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue49PartialLoadD

	| project version wc |
	project := self project.
	version := project version: '5.0'.
	version load: '2'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	self should: [ (GoferPackageReference name: 'GoferBar') workingCopy ] raise: Error.
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
	version := project version: '4.0'.
	version load: '1'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue49PartialLoadE

	| project version wc |
	project := self project.
	version := project version: '5.0'.
	version load: '2'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	self should: [ (GoferPackageReference name: 'GoferBar') workingCopy ] raise: Error.
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
	version := project version: '4.0'.
	version load: '3'. "should get error, since there is no GoferBeau package in 5.0"
!

testIssue49PartialLoadF

	| project version wc |
	project := self project.
	version := project version: '5.0'.
	version load: '4'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
	version := project version: '4.0'.
	version load: '4'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue49PartialLoadG

	| project version wc |
	project := self project.
	version := project version: '4.0'.
	version load: '4'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.2').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.32').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
	version := project version: '5.0'.
	version load: '4'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.2').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.34').
	self should: [ (GoferPackageReference name: 'GoferBeau') workingCopy ] raise: Error.
!

testIssue59PackageSpecRepositoryOverrides
	"test for Issue 59: http://code.google.com/p/metacello/issues/detail?id=59"
	
	| project version wc repoSpec pkg loader |
	project := self project.
	version := project version: '1.0'.
	repoSpec := version spec repositorySpecs first.
	pkg := version packageNamed: 'GoferBar'.
	pkg repository: repoSpec.
	loader := pkg loader.
	loader loaderPolicy: (MetacelloLoaderPolicy overrideRepositories: (OrderedCollection with: MCDictionaryRepository new)).
	self should: [ pkg load ] raise: Error.
	loader loaderPolicy: (MetacelloLoaderPolicy overrideRepositories: (OrderedCollection with: (Smalltalk at: #'Metacello_Gofer_Test_Repository'))).
	pkg load.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
!

testIssue77
	"The following project structure is loaded:
		project A 1.0
	   		project B 1.0
	     		project C 1.2
	   		project D 1.0 [without a direct load reference for C, but specifying version 1.3 of C]
	     		project C 1.3
	 and with the bug, project C version 1.3 is _not_ loaded.
	"

	| project version loader wc |
	project := self project.
	version := project version: 'Issue77-baseline'.
	loader := version load.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	self 
		should: [ (GoferPackageReference name: 'GeauxBeau') workingCopy ]
		raise: Error
!

testIssue95
	| project version wc |
	project := self project.
	version := project version: 'Issue95-baseline'.
	gofer
		version: 'GoferFaux-tg.30';
		version: 'GeauxBeau-dkh.55';
		version: 'GeauxBar.branch-lr.2';
		load.
	version load.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.30').
	wc := (GoferPackageReference name: 'GeauxBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GeauxBeau-dkh.55').
	wc := (GoferPackageReference name: 'GeauxBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GeauxBar.branch-lr.2').
!

testMczFileExplicitLoad
	"Older files will be loaded if directly referenced from the project being loaded (explicit version request)"
	
	| project version wc |
	project := self project.
	version := project version: '6.2'.
	version load.

	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.35').

	version := project version: '6.3'.
	version load.

	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	
	"explicit load older mcz file IS loaded ... see testMczFileImplicitLoad for alternate case"
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.31').
!

testMczFileImplicitLoad
	"when loading a project reference, do not load mcz files that are older than the currently loaded mcz files (implicit version request)"
	
	| project version wc |
	project := self project.
	version := project version: '6.0'.
	version load.

	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.35').

	version := project version: '6.1'.
	version load.

	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.4').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-lr.1').
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	
	"implicit load older mcz file NOT loaded ... see testMczFileExplicitLoad for alternate case"
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.35').
!

testMooseBaselineLoad

	| project version wc test |
	project := self project.
	version := project version: '1.0-Moose'.
	version load.
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.35').
!

testNestedLoad1

	| project version loader wc configurationName |
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer repository: self configurationRepository.
	gofer package: configurationName.
	gofer load.
	MetacelloConfigurationResource projectAttributes: #(#'level1_1' #'level2_2' #'level3_2' #'level2_1').
	project := (Smalltalk at: configurationName asSymbol) project.
	version := project version: '1.3.3-baseline'.
	loader := version load.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
	wc := (GoferPackageReference name: 'GeauxBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GeauxBar.branch-lr.2').
	wc := (GoferPackageReference name: 'GeauxBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GeauxBeau-dkh.55').
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	self 
		should: [ (GoferPackageReference name: 'GoferFaux') workingCopy ]
		raise: Error
!

testNestedLoad2

	| project version loader wc configurationName |
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer repository: self configurationRepository.
	gofer package: configurationName.
	gofer load.
	MetacelloConfigurationResource projectAttributes: #(#'level1_1' #'level2_2' #'level3_2' #'level2_1').
	project := (Smalltalk at: configurationName asSymbol) project.
	version := project version: '1.3.2-baseline'.
	loader := version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.25').
!

testNestedLoad3

	| project version loader wc configurationName |
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer repository: self configurationRepository.
	gofer package: configurationName.
	gofer load.
	MetacelloConfigurationResource projectAttributes: #(#'level1_1' #'level2_2' #'level2_1' #'level3_2' ).
	project := (Smalltalk at: configurationName asSymbol) project.
	version := project version: '1.3.2-baseline'.
	loader := version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.55').
!

testProjectFetch

	| project version wc files loader |
	project := self project.
	version := project version: '3.0'.
	loader := version fetch.
	files := #('MetacelloTestConfigurationOfFoo-dkh.1' 'GoferFoo-lr.1' 'GoferBar.branch-lr.1' 'GoferFaux-tg.30' 'GoferBeau-dkh.53' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | file |
		file := pkgDirective file.
		self assert: (files indexOf: file) = 1.
		files remove: file ifAbsent: [ self assert: false ]].
	loader loadDirective versionDirectivesDo: [:vrsnDirective |
		vrsnDirective loader ~~ nil
			ifTrue: [ | spec |
				spec := vrsnDirective spec.
				self assert: (spec isKindOf: MetacelloVersionSpec) ]].
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result')
!

testProjectLoad

	| project version wc |
	project := self project.
	version := project version: '3.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.branch-lr.1').
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFaux-tg.30').
	self assert: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result')
!

testProjectPrePostLoad

	| project version |
	project := self project.
	version := project version: '3.1'.
	self deny: preLoadExecuted.
	self deny: postLoadExecuted.
	version load.
	self assert: preLoadExecuted.
	self assert: postLoadExecuted.
!

testProjectRecord

	| project version names loader |
	project := self project.
	version := project version: '3.0'.
	loader := version record.
	names := #('MetacelloTestConfigurationOfFoo' 'GoferFoo' 'GoferBar' 'GoferFaux' 'GoferBeau' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | name |
		name := pkgDirective packageName.
		self assert: (names indexOf: name) = 1.
		names remove: name ifAbsent: [ self assert: false ]].
	self deny: (Smalltalk at: #'Metacello_Configuration_Test_DoIt_Result')
!

testRepositoryOverrides

	| project version wc |
	project := self project.
	version := project version: '1.0'.
	version repositoryOverrides: (OrderedCollection with: MCDictionaryRepository new).
	self 
		should: [ version load ]
		raise: Error.
	version repositoryOverrides: (OrderedCollection with: (Smalltalk at: #'Metacello_Gofer_Test_Repository')).
	version load.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
!

testRepositoryOverridesFetch

	| project version loader names |
	project := self project.
	version := project version: '1.0'.
	version repositoryOverrides: (OrderedCollection with: MCDictionaryRepository new).
	self 
		should: [ version fetch ]
		raise: Error.
	version repositoryOverrides: (OrderedCollection with: (Smalltalk at: #'Metacello_Gofer_Test_Repository')).
	loader := version fetch.
	names := #('GoferFoo-lr.1' 'GoferBar-jf.1' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | name |
		name := pkgDirective file.
		self assert: (names indexOf: name) = 1.
		names remove: name ifAbsent: [ self assert: false ]].
!

testRepositoryOverridesRecord

	| project version loader names |
	project := self project.
	version := project version: '1.0'.
	version repositoryOverrides: (OrderedCollection with: MCDictionaryRepository new).
	self 
		shouldnt: [ version record ]
		raise: Error.
	version repositoryOverrides: (OrderedCollection with: (Smalltalk at: #'Metacello_Gofer_Test_Repository')).
	loader := version record.
	names := #('GoferFoo' 'GoferBar' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | name |
		name := pkgDirective packageName.
		self assert: (names indexOf: name) = 1.
		names remove: name ifAbsent: [ self assert: false ]].
!

testSupplyingAnswerLoad

	| project version wc |
	project := self project.
	version := project version: '1.1'.
	self deny: postloadAnswer = 'postload answer'.
	version load.
	self assert: postloadAnswer = 'postload answer'.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
!

testVersionFetch

	| project version loader names |
	project := self project.
	version := project version: '1.0'.
	loader := version fetch.
	names := #('GoferFoo-lr.1' 'GoferBar-jf.1' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | name |
		name := pkgDirective file.
		self assert: (names indexOf: name) = 1.
		names remove: name ifAbsent: [ self assert: false ]].
	loader loadDirective versionDirectivesDo: [:vrsnDirective |
		vrsnDirective loader ~~ nil
			ifTrue: [ | spec |
				spec := vrsnDirective spec.
				self assert: (spec isKindOf: MetacelloVersionSpec) ]].
!

testVersionLoad

	| project version wc |
	project := self project.
	version := project version: '1.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferFoo') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferFoo-lr.1').
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
!

testVersionRecord

	| project version loader names |
	project := self project.
	version := project version: '1.0'.
	loader := version record.
	names := #('GoferFoo' 'GoferBar' ) asOrderedCollection.
	loader loadDirective packageDirectivesDo: [:pkgDirective | | name |
		name := pkgDirective packageName.
		self assert: (names indexOf: name) = 1.
		names remove: name ifAbsent: [ self assert: false ]].
! !

!MetacelloGoferBasedLoaderTest methodsFor:'versions'!

version10: spec
	<version: '1.0' imports: #('1.0-baseline' )>
	
	spec for: #common do: [
		spec 
			package: 'GoferFoo' with: 'GoferFoo-lr.1';
			package: 'GoferBar' with: 'GoferBar-jf.1';
			yourself. ].
!

version11: spec
	<version: '1.1' imports: #('1.0-baseline' )>
	
	spec for: #common do: [
		spec 
			package: 'GoferFoo' with: [
				spec
					file: 'GoferFoo-lr.1';
					supplyingAnswers: #(#('postload' 'postload answer'));
					postLoadDoIt: #postloadSupplyingAnswers ];
			package: 'GoferBar' with: 'GoferBar-jf.1';
			yourself. ].
!

version30: spec 
	<version: '3.0'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: '1' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '3.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself. ].
!

version31: spec 
	<version: '3.1'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: '1' with: [
				spec
					preLoadDoIt: #preloadExecute;
					postLoadDoIt: #postloadExecute;
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '3.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself. ].
!

version40: spec 
	<version: '4.0'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: '1' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '4.0';
					loads: #('1');
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			project: '2' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '4.0';
					loads: #('2');
					file: 'MetacelloTestConfigurationOfFoo';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			project: '3' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '4.0';
					loads: #('GoferBeau');
					file: 'MetacelloTestConfigurationOfFoo';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself.

		spec
			group: '4' with: #('1' '2') ].
!

version50: spec 
	<version: '5.0'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: '1' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '5.0';
					loads: #('1');
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			project: '2' with: [
				spec
					className: 'MetacelloTestConfigurationOfFoo';
					versionString: '5.0';
					loads: #('2');
					file: 'MetacelloTestConfigurationOfFoo';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself.

		spec
			group: '4' with: #('1' '2') ].
!

version60: spec
	<version: '6.0'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: 'Foo' with: [
				spec
					className: 'MetacelloTestConfigurationOfFix';
					versionString: '6.0';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself ].
!

version61: spec
	<version: '6.1'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			project: 'Foo' with: [
				spec
					className: 'MetacelloTestConfigurationOfFix';
					versionString: '6.1';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself ].
!

version62: spec
	<version: '6.2'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo' with: 'GoferFoo-lr.4';
			package: 'GoferBar' with: 'GoferBar-lr.1';
			package: 'GoferFaux' with: 'GoferFaux-tg.35';
			yourself. ].
!

version63: spec
	<version: '6.3'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo' with: 'GoferFoo-lr.4';
			package: 'GoferBar' with: 'GoferBar-lr.1';
			package: 'GoferBeau' with: 'GoferBeau-dkh.53';
			package: 'GoferFaux' with: 'GoferFaux-tg.31';
			yourself. ].
! !

!MetacelloGoferBasedLoaderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferBasedLoaderTest.st,v 1.1 2011/08/20 18:11:50 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferBasedLoaderTest.st,v 1.1 2011/08/20 18:11:50 cg Exp $'
! !
