"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloGoferFunctionalTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloGoferFunctionalTest comment:''
!


!MetacelloGoferFunctionalTest methodsFor:'accessing'!

repositorySpec

	| project version |
	project := self project.
	version := project version: '1.0-baseline'.
	^version spec repositorySpecs first.
! !

!MetacelloGoferFunctionalTest methodsFor:'baselines'!

baseline10: spec 
	<version: '1.0-baseline'>
	
	spec for: #common do: [
	spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo';
			package: 'GoferBar';
			package: 'GoferFaux';
			yourself ].
! !

!MetacelloGoferFunctionalTest methodsFor:'running'!

doSilently

	^(#(#testCommitPackageSpec #testCommitPackageVersion #testPackagesNeedSaving ) includes: testSelector asSymbol) not
!

tearDown
	super tearDown.
	MetacelloMonticelloResource reset. "testCommitPackageSpec modifies the repository resource"
! !

!MetacelloGoferFunctionalTest methodsFor:'tests'!

testBasics
	| repoSpec repo |
	repoSpec := self repositorySpec.
	repo := repoSpec createRepository.
	self assert: repoSpec type = 'dictionary'.
	self assert: repo class == MCDictionaryRepository.
	
!

testCommitNewPackageSpec
	| repoSpec repo project packagesNeedSaving allVersionsSize packageName |
	project := self project.
	repoSpec := self repositorySpec.
	repo := repoSpec createRepository.
	allVersionsSize := repo allVersionInfos size.
	gofer repository: repo.
	gofer 
		package: 'GoferFoo'; 
		package: 'GoferBar'; 
		metacelloLoad.
	"Create new package"
	packageName := 'GoferFaux'.
	PackageInfo registerPackageName: packageName.
	MCWorkingCopy forPackage: (MetacelloTestsMCPackage new name: packageName).
	(MCClassDefinition 
		name: (packageName copyWithout: $-) asSymbol 
		superclassName: #Object 
		category: packageName asSymbol 
		instVarNames: #() 
		comment: '') createClass.
	self assert: (packagesNeedSaving := project packagesNeedSaving: '1.0-baseline') notEmpty.
	packagesNeedSaving do: [:assoc | | gof |
		gof := Gofer new.
		gof disablePackageCache.
		gof repository: assoc value.
		gof package: assoc key name.
		gof commit: 'testing' ].
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
!

testCommitPackageSpec
	| repoSpec repo project packagesNeedSaving allVersionsSize |
	project := self project.
	repoSpec := self repositorySpec.
	repo := repoSpec createRepository.
	allVersionsSize := repo allVersionInfos size.
	gofer repository: repo.
	gofer 
		package: 'GoferFoo'; 
		package: 'GoferBar'; 
		metacelloLoad.
	(Smalltalk classNamed: #GoferBar) compile: 'foo'.
	self assert: (packagesNeedSaving := project packagesNeedSaving: '1.0-baseline') notEmpty.
	packagesNeedSaving do: [:assoc | | gof |
		gof := Gofer new.
		gof disablePackageCache.
		gof repository: assoc value.
		gof package: assoc key name.
		gof commit: 'testing' ].
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
!

testCommitPackageVersion
	| repoSpec repo project packagesNeedSaving allVersionsSize |
	project := self project.
	repoSpec := self repositorySpec.
	repo := repoSpec createRepository.
	allVersionsSize := repo allVersionInfos size.
	gofer repository: repo.
	gofer 
		package: 'GoferFoo'; 
		package: 'GoferBar'; 
		metacelloLoad.
	(Smalltalk classNamed: #GoferBar) compile: 'foo'.
	self assert: (packagesNeedSaving := project packagesNeedSaving: '1.0') notEmpty.
	packagesNeedSaving do: [:assoc | | gof |
		gof := Gofer new.
		gof disablePackageCache.
		gof repository: assoc value.
		gof package: assoc key name.
		gof commit: 'testing' ].
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
!

testComparison

	| blk |
	blk := [:file1 :operator :file2 | | ref1 ref2 |
		ref1 := GoferResolvedReference name: file1.
		ref2 := GoferResolvedReference name: file2.
		ref1 compare: ref2 using: operator].
	self assert: (blk value: 'GoferFaux-tg.34' value: #>= value: 'GoferFaux-tg.32').
	self assert: (blk value: 'GoferFaux-tg.32' value: #<= value: 'GoferFaux-tg.34').
	self assert: (blk value: 'GoferBar.branch-lr.2' value: #> value: 'GoferBar.branch-lr.1').
	self assert: (blk value: 'GoferBar-lr.1' value: #> value: 'GoferBar.branch-lr.1').
	self assert: (blk value: 'GoferBar-lr.1' value: #> value: 'GoferBar-jf.1').
	self assert: (blk value: 'Bootstrap.232-dkh.163' value: #< value: 'Bootstrap-dkh.178').
!

testComparison01

	| ref1 ref2 |
	true ifTrue: [ ^self ].

	"The currently recommended way to deal with this kind of branch is to do something like:
		spec package: 'Bootstrap' with: [
			spec file: 'Bootstrap.232 ]."

	ref1 := GoferResolvedReference name: 'Bootstrap.232-dkh.163'.
	ref2 := GoferResolvedReference name: 'Bootstrap-dkh.178'.
	"'Bootstrap' <= 'Bootstrap.232'"
	self deny: (ref1 <= ref2).
	self assert: (ref2 <= ref1).
	"163 <= 178"
	self assert: (ref1 compare: ref2 using: #<=).
	self assert: (ref2 compare: ref1 using: #>=).

	ref1 := GoferResolvedReference name: 'Bootstrap.232-dkh.163'.
	ref2 := GoferResolvedReference name: 'Bootstrap-dkh.163'.
	"'Bootstrap' <= 'Bootstrap.232'"
	self deny: (ref1 <= ref2).
	self assert: (ref2 <= ref1).
	"163 <= 163"
	self assert: (ref1 compare: ref2 using: #<=).
	self assert: (ref2 compare: ref1 using: #>=)
!

testIssue40PackageResolution

	|  ref pkgBar pkgFoo |
	pkgBar := GoferPackageReference name: 'GoferFoo'.
	ref := pkgBar resolveWith: gofer.
	self assert: ref repository == self monticelloRepository.
	self assert: ref versionNumber = 4.
	gofer repository: self alternateRepository.
	pkgBar := GoferPackageReference name: 'GoferBar'.
	ref := pkgBar resolveWith: gofer.
	self assert: ref repository == self alternateRepository.
	self assert: ref versionNumber = 25.
	self assert: ref author = 'dkh'.
	pkgFoo := GoferPackageReference name: 'GoferFoo'.
	ref := pkgFoo resolveWith: gofer.
	self assert: ref repository == self alternateRepository.
	self assert: ref versionNumber = 104.
	gofer
		package: 'GoferBar';
		package: 'GoferFoo';
		load.
	self assert: pkgBar workingCopy ancestry ancestors first name = 'GoferBar-dkh.25'.
	self assert: pkgFoo workingCopy ancestry ancestors first name = 'GoferFoo-dkh.104'.
!

testIssue40VersionResolution

	|  ref pkgBar pkgFoo |
	pkgBar := GoferVersionReference name: 'GoferFoo-lr.2'.
	ref := pkgBar resolveWith: gofer.
	self assert: ref repository == self monticelloRepository.
	self assert: ref versionNumber = 2.
	gofer repository: self alternateRepository.
	pkgBar := GoferVersionReference name: 'GoferBar-jf.1'.
	ref := pkgBar resolveWith: gofer.
	self assert: ref repository == self monticelloRepository.
	self assert: ref versionNumber = 1.
	self assert: ref author = 'jf'.
	ref := (GoferVersionReference name: 'GoferBar-dkh.24') resolveWith: gofer.
	self assert: ref repository == self alternateRepository.
	self assert: ref versionNumber = 24.
	self assert: ref author = 'dkh'.
	pkgFoo := GoferVersionReference name: 'GoferFoo-dkh.75'.
	ref := pkgFoo resolveWith: gofer.
	self assert: ref repository == self alternateRepository.
	self assert: ref versionNumber = 75.
	gofer
		version: 'GoferBar-jf.1';
		version: 'GoferFoo-dkh.75';
		load.
	self assert: pkgBar workingCopy ancestry ancestors first name = 'GoferBar-jf.1'.
	self assert: (pkgBar workingCopy repositoryGroup includes: self monticelloRepository).
	self assert: pkgBar workingCopy repositoryGroup repositories size = 3.
	self assert: pkgFoo workingCopy ancestry ancestors first name = 'GoferFoo-dkh.75'.
	self assert: (pkgFoo workingCopy repositoryGroup includes: self alternateRepository).
	self assert: pkgFoo workingCopy repositoryGroup repositories size = 3.
!

testMoosePackageResolution

	|  ref pkg |
	pkg := GoferPackageReference name: 'GoferFaux'.
	ref := pkg resolveWith: gofer.
	self assert: ref repository == self monticelloRepository.
	self assert: ref versionNumber = 35.
	pkg := MetacelloGoferPackage name: 'GoferFaux' packageFilename: 'GoferFaux-'.
	ref := pkg resolveWith: gofer.
	self assert: ref repository == self monticelloRepository.
	self assert: ref versionNumber = 35.
	pkg := GoferPackageReference name: 'GoferFaux-'.
	self should: [ pkg resolveWith: gofer ] raise: Error.
	pkg := GoferVersionReference name: 'GoferFaux-'.
	self should: [ pkg resolveWith: gofer ] raise: Error.
!

testPackageResolution
	|  project version repo pkg ref |
	project := self project.
	version := project version: '1.0-baseline'.
	repo := version spec repositorySpecs first createRepository.
	gofer repository: repo.
	pkg := GoferPackageReference name: 'GoferFoo'.
	ref := pkg resolveWith: gofer.
	self assert: ref versionNumber = 4.
	pkg := GoferPackageReference name: 'GoferBar'.
	ref := pkg resolveWith: gofer.
	self assert: ref versionNumber = 1.
	self assert: ref author = 'lr'.
!

testPackagesNeedSaving
	| repoSpec repo project |
	project := self class new project.
	repoSpec := self repositorySpec.
	repo := repoSpec createRepository.
	gofer repository: repo.
	gofer package: 'GoferFoo'; package: 'GoferBar'; load.
	self assert: (project packagesNeedSaving: '1.0-baseline') isEmpty.
	(Smalltalk classNamed: #GoferBar) compile: 'foo'.
	self assert: (project packagesNeedSaving: '1.0-baseline') notEmpty.
! !

!MetacelloGoferFunctionalTest methodsFor:'versions'!

version10: spec
	<version: '1.0' imports: #('1.0-baseline' )>
	
	spec for: #common do: [
		spec 
			package: 'GoferFoo' with: 'GoferFoo-lr.1';
			package: 'GoferBar' with: 'GoferBar-jf.1';
			package: 'Faux-Smalltalk' with: 'Faux-Smalltalk-tg.32';
			yourself. ].
! !

!MetacelloGoferFunctionalTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferFunctionalTest.st,v 1.1 2011/08/20 18:09:32 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferFunctionalTest.st,v 1.1 2011/08/20 18:09:32 cg Exp $'
! !
