"{ Package: 'stx:goodies/metacello' }"

GoferPackageReference subclass:#MetacelloGoferPackage
	instanceVariableNames:'packageFilename version workingCopy'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Gofer'
!

MetacelloGoferPackage comment:''
!


!MetacelloGoferPackage class methodsFor:'instance creation'!

name: aString packageFilename: packageFilename
	^ self basicNew initializeName: aString packageFilename: packageFilename
! !

!MetacelloGoferPackage class methodsFor:'package name matching'!

packageFileName: pkgFileName matchesPackageName: wcPkgName

	^(pkgFileName beginsWith: wcPkgName) 
		ifTrue: [ 
			(pkgFileName size = wcPkgName size) 
			or: [ ((pkgFileName at: wcPkgName size + 1) = $-) 
				or: [ ((pkgFileName at: wcPkgName size + 1) = $.) 
					or: [ (pkgFileName at: wcPkgName size + 1) isDigit ]]]]
		ifFalse: [
			pkgFileName size >= wcPkgName size ifTrue: [ ^false ].
			(wcPkgName beginsWith: pkgFileName) ifFalse: [ ^false ].
			^(wcPkgName at: pkgFileName size + 1) = $. ]
! !

!MetacelloGoferPackage methodsFor:'accessing'!

packageFilename

	^packageFilename
!

workingCopy
	
	workingCopy == nil ifTrue: [ workingCopy := self findWorkingCopy ].
	^workingCopy
! !

!MetacelloGoferPackage methodsFor:'initialization'!

initializeName: aString packageFilename: packagefilename

	name := aString.
	packageFilename := packagefilename
! !

!MetacelloGoferPackage methodsFor:'private'!

findWorkingCopy
	"Answer a working copy, or nil if the package is not loaded."

	| wcs |
	wcs := MCWorkingCopy allManagers select: [ :each | self matchesWorkingCopy: each ].
	wcs isEmpty ifTrue: [ ^nil ].
	^wcs detectMax: [:ea | ea package name size ]
!

matches: aLoadableReference
	"does Monticello-style #versionInfoFromVersionNamed: matching"
	
	| pFilename |
	(((pFilename := self packageFilename) == nil) or: [ self name = self packageFilename] ) ifTrue: [ ^super matches: aLoadableReference ].
	^aLoadableReference name beginsWith: pFilename 
!

matchesWorkingCopy: aWorkingCopy
	"check that the working copy package name matches the package file name and that the first ancestor's package file name
	 matches the packageName"
	
	| pFilename |
	(pFilename := self packageFilename) == nil ifTrue: [ ^self error: 'cannot match working copy' ].
	(self class 
		packageFileName: pFilename 
		matchesPackageName: aWorkingCopy package name)
			ifTrue: [
				aWorkingCopy ancestry ancestors isEmpty ifTrue: [ ^true ].
				^self class 
					packageFileName: aWorkingCopy ancestry ancestors first name 
					matchesPackageName: self packageName ].
	^false
! !

!MetacelloGoferPackage methodsFor:'querying'!

ancestors

	| wc |
	(wc := self workingCopy) ~~ nil
		ifTrue: [ 
			wc ancestry ancestors isEmpty not
				ifTrue: [ ^wc ancestry ancestors ]].
	^nil
!

currentVersionInfo

	| wc |
	(wc := self workingCopy) ~~ nil
		ifTrue: [ 
			wc ancestry ancestors isEmpty not
				ifTrue: [ ^wc ancestry ancestors first ]].
	^nil
! !

!MetacelloGoferPackage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferPackage.st,v 1.1 2011/08/20 18:13:42 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloGoferPackage.st,v 1.1 2011/08/20 18:13:42 cg Exp $'
! !
