"{ Package: 'stx:goodies/metacello' }"

TestCase subclass:#MetacelloIssue108TestCase
	instanceVariableNames:'tempRepositories'
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloIssue108TestCase comment:''
!


!MetacelloIssue108TestCase methodsFor:'accessing'!

project

	| constructor project |
	"Construct Metacello project"
	constructor := MetacelloVersionConstructor on: self.
	project := constructor project.
	project loader: 
		((project loaderClass new)
			shouldDisablePackageCache: true;
			yourself).
	project loadType: #linear.
	^project
!

tempRepositories

	tempRepositories ifNil: [ tempRepositories := OrderedCollection new ].
	^tempRepositories
! !

!MetacelloIssue108TestCase methodsFor:'baselines'!

baseline10: spec 
	<version: '1.0-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Repository'.
		spec 
			package: 'GoferBar';
			yourself ].
!

baseline20: spec 
	<version: '2.0-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Repository'.
		spec 
			package: 'GoferBar' with: [
				spec file: 'GoferBar.gemstone' ];
			yourself ].
! !

!MetacelloIssue108TestCase methodsFor:'private'!

hasPackage: aString
	| package |
	package := MCWorkingCopy allManagers
		detect: [ :each | each packageName = aString ]
		ifNone: [ nil ].
	^ package notNil
!

repositoryName

	^#'Metacello_Repository'
! !

!MetacelloIssue108TestCase methodsFor:'running'!

runCase
	| original |
	original := MetacelloPlatform current bypassGoferLoadUpdateCategories.
	[ 
	MetacelloPlatform current bypassGoferLoadUpdateCategories: true.
	^ SystemChangeNotifier uniqueInstance doSilently: [ super runCase ] ]
		ensure: [ MetacelloPlatform current bypassGoferLoadUpdateCategories: original ]
!

setUp
	| repo  |
	super setUp.
	repo := MetacelloIssue108Resource current monticelloRepository.
	self tempRepositories add: repo.
	Smalltalk at: self repositoryName put: repo.
!

tearDown
	| aGofer |
	super tearDown.
	aGofer := Gofer new.
	Smalltalk removeKey: self repositoryName ifAbsent: [  ].
	self tempRepositories do: [:repo | MCRepositoryGroup default removeRepository: repo ].
! !

!MetacelloIssue108TestCase methodsFor:'tests'!

expectedFailures

	true ifTrue: [ ^#() ].
	^#(
		testMore "place holder to complete test coverage"
	)
!

testBaseline10
	| wc project loader |
	project := self class new project.
	loader := project load: '1.0-baseline'.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-dkh.70').
!

testBaseline20
	| wc project loader |
	project := self class new project.
	loader := project load: '2.0-baseline'.
	wc := (GoferVersionReference name: 'GoferBar.gemstone') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar.gemstone-dkh.70').
!

testLatestPackage
	"illustrate different behavior for GoferPackageReference, GoferVersionReference and MetacelloGoferPackage"
	
	| gofer repositorySpecs aString |
	gofer := Gofer new.
	gofer disablePackageCache.
	gofer repository: (Smalltalk at: self repositoryName).

	"GoferPackageReference"
	self assert: 'GoferBar-dkh.70' = ((GoferPackageReference name: 'GoferBar') resolveWith: gofer) name.
	self should: [ (GoferPackageReference name: 'GoferBar.gemstone') resolveWith: gofer ] raise: Error.

	"GoferVersionReference"
	self should: [ (GoferVersionReference name: 'GoferBar') resolveWith: gofer ] raise: Error.
	self should: [ (GoferVersionReference name: 'GoferBar.gemstone') resolveWith: gofer ] raise: Error.

	"MetacelloGoferPackage"
	self assert: 'GoferBar-dkh.70' = ((MetacelloGoferPackage name: 'GoferBar' packageFilename: 'GoferBar') resolveWith: gofer) name.
	self assert: 'GoferBar.gemstone-dkh.70' = ((MetacelloGoferPackage name: 'GoferBar' packageFilename: 'GoferBar.gemstone') resolveWith: gofer) name
! !

!MetacelloIssue108TestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloIssue108TestCase.st,v 1.1 2011/08/20 18:12:59 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloIssue108TestCase.st,v 1.1 2011/08/20 18:12:59 cg Exp $'
! !
