"{ Package: 'stx:goodies/metacello' }"

TestCase subclass:#MetacelloIssue115TestCase
	instanceVariableNames:'gofer'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-TestsMC'
!

MetacelloIssue115TestCase comment:''
!


!MetacelloIssue115TestCase methodsFor:'accessing'!

monticelloRepository
	"MetacelloMonticelloResource reset"
	^ MetacelloMonticelloResource current monticelloRepository
!

project

	^self projectWith: #()
!

projectClass

	^(Smalltalk at: #MetacelloTestConfigurationOfProjectIssue115)
!

projectWith: projectAttributes

	| project |
	"Construct Metacello project"
	project := MetacelloMCProject new.
	project projectAttributes: projectAttributes.
	MetacelloVersionConstructor on: self  projectClass new project: project.
	project loader: MetacelloNullRecordingMCSpecLoader new.
	project loader evalDoits: true.
	^project
! !

!MetacelloIssue115TestCase methodsFor:'running'!

configurationRepository
	"MetacelloConfigurationResource reset"

	^ MetacelloConfigurationResource current monticelloRepository
!

runCase
	| original |
	original := MetacelloPlatform current bypassGoferLoadUpdateCategories.
	[ 
	MetacelloPlatform current bypassGoferLoadUpdateCategories: true.
	^ SystemChangeNotifier uniqueInstance doSilently: [ super runCase ] ]
		ensure: [ MetacelloPlatform current bypassGoferLoadUpdateCategories: original ]
!

setUp
	| repo |
	super setUp.
	repo := self monticelloRepository.
	gofer := Gofer new.
	gofer disablePackageCache.
	gofer repository: repo.
	Smalltalk at: #'Metacello_Gofer_Test_Repository' put: repo.
	repo := self configurationRepository.
	gofer repository: repo.
	Smalltalk at: #'Metacello_Configuration_Test_Repository' put: repo.
!

tearDown
	| aGofer |
	super tearDown.
	aGofer := Gofer new.
	(self hasPackage: 'MetacelloTestConfigurationOfProjectIssue115')
		ifTrue: [ aGofer package: 'MetacelloTestConfigurationOfProjectIssue115' ].
	aGofer references notEmpty ifTrue: [ aGofer metacelloUnload ].
	Smalltalk removeKey: #'Metacello_Gofer_Test_Repository' ifAbsent: [].
	Smalltalk removeKey: #'Metacello_Configuration_Test_Repository' ifAbsent: [].
! !

!MetacelloIssue115TestCase methodsFor:'tests'!

testIssue115a
	| project versionSpec version validations |
	gofer version: 'MetacelloTestConfigurationOfProjectIssue115-dkh.1'.
	gofer metacelloLoad.
	project := self project.
	self shouldnt: [versionSpec := (version := project version: '1.0') spec] raise: Error.
	validations := MetacelloToolBox validateConfiguration: self projectClass.
	self assert: validations size = 1.
	validations detect: [ :validation | validation reasonCode == #cannotResolveVersion ] ifNone: [ self assert: false ]
!

testIssue115b
	| project versionSpec version validations |
	gofer version: 'MetacelloTestConfigurationOfProjectIssue115-dkh.2'.
	gofer metacelloLoad.
	project := self project.
	self shouldnt: [versionSpec := (version := project version: '1.1') spec] raise: Error.
	validations := MetacelloToolBox validateConfiguration: self projectClass.
	self assert: validations size = 2.
	validations do: [ :validation | self assert: validation reasonCode == #cannotResolveVersion ]
! !

!MetacelloIssue115TestCase methodsFor:'utilities'!

hasPackage: aString
	| package |
	package := MCWorkingCopy allManagers
		detect: [ :each | each packageName = aString ]
		ifNone: [ nil ].
	^ package notNil
! !

!MetacelloIssue115TestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloIssue115TestCase.st,v 1.1 2011/08/20 17:56:53 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloIssue115TestCase.st,v 1.1 2011/08/20 17:56:53 cg Exp $'
! !
