"{ Package: 'stx:goodies/metacello' }"

Object subclass:#MetacelloLoaderPolicy
	instanceVariableNames:'overrideRepositories repositoryMap ensuredMap cacheRepository
		cacheGofer ignoreImage loadData loadDirective silently'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Loaders'
!

MetacelloLoaderPolicy comment:''
!


!MetacelloLoaderPolicy class methodsFor:'instance creation'!

overrideRepositories: aCollection

	^self new overrideRepositories: aCollection
! !

!MetacelloLoaderPolicy methodsFor:'accessing'!

cacheGofer

	cacheGofer == nil 
		ifTrue: [ 
			"don't use a caching Gofer here, since we expect the contents to change during a fetch operation"
			cacheGofer := Gofer new. 
			cacheGofer disablePackageCache.
			cacheGofer repository: self cacheRepository. ].
	^ cacheGofer
!

cacheRepository

	cacheRepository == nil ifTrue: [ cacheRepository := MCDictionaryRepository new ].
	^ cacheRepository
!

cacheRepository: anMCRepository

	cacheRepository := anMCRepository.
	"getting a new repository, so wipe out the cacheGofer and ensureMap"
	ensuredMap := cacheGofer := nil
!

ensuredMap

	ensuredMap == nil ifTrue: [ ensuredMap := Dictionary new ].
	^ensuredMap
!

ensuredMap: anObject
	ensuredMap := anObject
!

ignoreImage
	^ ignoreImage
!

ignoreImage: anObject
	ignoreImage := anObject
!

loadData

 	loadData == nil ifTrue: [ loadData := MetacelloLoadData new ].
	^loadData
!

loadDirective

	loadDirective == nil ifTrue: [ loadDirective := MetacelloLinearLoadDirective new ].
	^ loadDirective
!

overrideRepositories
	^ overrideRepositories
!

overrideRepositories: anObject
	overrideRepositories := anObject
!

repositoryMap

	repositoryMap == nil ifTrue: [ repositoryMap := Dictionary new ].
	^repositoryMap
!

repositoryMap: anObject
	repositoryMap := anObject
!

resetCacheGofer

	cacheGofer := nil
!

silently

	silently == nil ifTrue: [ silently := false ].
	^ silently
!

silently: anObject
	silently := anObject
! !

!MetacelloLoaderPolicy methodsFor:'actions'!

load

	overrideRepositories := Array with: self cacheRepository. "ensure that hasRepositoryOverrides is true"
	self loadDirective loadWithPolicy: self
!

pushAtomicLoadDirectivesDuring: aBlock for: aLoader

	self pushLoadDirective: (MetacelloAtomicLoadDirective loader: aLoader) during: aBlock.
!

pushExplicitLoadDirectivesDuring: aBlock for: aLoader

	| directive |
	directive := MetacelloExplicitLoadDirective loader: aLoader.
	self pushLoadDirective: directive during: aBlock.
	^directive
!

pushLinearLoadDirectivesDuring: aBlock for: aLoader

	self pushLoadDirective: (MetacelloLinearLoadDirective loader: aLoader) during: aBlock.
! !

!MetacelloLoaderPolicy methodsFor:'initialize-release'!

initialize

	self 
		repositoryMap; 
		cacheRepository;
		ensuredMap.
	ignoreImage := false
! !

!MetacelloLoaderPolicy methodsFor:'private'!

pushLoadDirective: aLoaderDirective during: aBlock

	|  oldRoot |
	self loadDirective add: aLoaderDirective.
	oldRoot := loadDirective.
	loadDirective := aLoaderDirective.
	aBlock ensure: [ loadDirective := oldRoot ].
! !

!MetacelloLoaderPolicy methodsFor:'testing'!

hasRepositoryOverrides

	^self overrideRepositories ~~ nil
! !

!MetacelloLoaderPolicy class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloLoaderPolicy.st,v 1.1 2011/08/20 18:09:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloLoaderPolicy.st,v 1.1 2011/08/20 18:09:20 cg Exp $'
! !
