"{ Package: 'stx:goodies/metacello' }"

MetacelloCommonMCSpecLoader subclass:#MetacelloLoadingMCSpecLoader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Loaders'
!

MetacelloLoadingMCSpecLoader comment:''
!


!MetacelloLoadingMCSpecLoader methodsFor:'accessing'!

actionLabel

	^'Loading '
!

fetchingSpecLoader

	^(MetacelloFetchingMCSpecLoader on: self spec)
		shouldDisablePackageCache: self shouldDisablePackageCache;
		loaderPolicy: self loaderPolicy copy;
		yourself
! !

!MetacelloLoadingMCSpecLoader methodsFor:'actions'!

doLoad
	"NOOP"
! !

!MetacelloLoadingMCSpecLoader methodsFor:'development support'!

copySpec: pkgSpec from: repositorySpecs to: repository

	| gofer |
	gofer := MetacelloGofer new.
	(self repositoriesFrom: repositorySpecs) do: [:repo | gofer repository: repo ].
	^self copySpec: pkgSpec with: gofer to: repository
!

copySpec: pkgSpec with: gofer to: aRepository

	| repository resolvedReference |
	[resolvedReference := pkgSpec goferLoaderReference resolveWith: gofer]
		on: Error
		do: [:ignored | ^nil ].
	repository := MCRepositoryGroup default repositories
		detect: [ :each | each = aRepository ]
		ifNone: [ aRepository ].
	repository storeVersion: resolvedReference version.
	^resolvedReference repository
!

goferCommitBranchPackage: branchName using: repositorySpecs commitMessage: commitMessage
	| gofer repoSpecs wc |
	repoSpecs := self spec repositorySpecs notEmpty
		ifTrue: [ self spec repositorySpecs ]
		ifFalse: [ repositorySpecs ].
	gofer := MetacelloGofer new.
	gofer disablePackageCache.
	wc := self spec workingCopy.
	repositorySpecs
		do: [ :repoSpec | 
			| repo |
			repo := repoSpec createRepository.
			(wc possiblyNewerVersionsIn: repo) notEmpty
				ifTrue: [ 
					self
						notify:
							'There are possibly newer versions of the package ' , self spec name printString , ' in the repository '
								, repo description printString
								, '. Cancel and manually merge if you want to pick up the changes from the later version.' ].
			gofer repository: repo ].
	gofer package: self spec name.
	[ gofer interactiveCommit ]
		on: MCVersionNameAndMessageRequest
		do: [ :ex | 
			| ref |
			ref := GoferVersionReference name: ex suggestedName.
			ex
				resume:
					{(ref packageName , '.' , branchName , '-' , ref author , '.' , ref versionNumber printString).
					commitMessage} ].
	^ true
!

goferCommitPackageUsing: repositorySpecs commitMessage: commitMessage

	| gofer repoSpecs wc |
	repoSpecs := self spec repositorySpecs notEmpty
				ifTrue: [ self spec repositorySpecs ]
				ifFalse: [ repositorySpecs ].
	gofer := MetacelloGofer new.
	gofer disablePackageCache.
	wc := self spec workingCopy.
	repositorySpecs do: [:repoSpec | | repo |
		repo := repoSpec createRepository.
		(wc possiblyNewerVersionsIn: repo) notEmpty
			ifTrue: [ self notify: 'There are possibly newer versions of the package ', self spec name printString, ' in the repository ', repo description printString, '. Cancel and manually merge if you want to pick up the changes from the later version.' ].
		gofer repository: repo ].
	gofer package: self spec name.
	gofer commit: commitMessage.
	^true
!

packagesNeedSavingUsing: repositorySpecs into: aCollection

	| wc repoSpecs repo |
	(wc := self spec workingCopy) == nil ifTrue: [ ^self ].
	(wc ancestry ancestors notEmpty and: [ wc modified not])
		ifTrue: [ ^self ].
	repoSpecs := self spec repositorySpecs notEmpty
				ifTrue: [ self spec repositorySpecs ]
				ifFalse: [ repositorySpecs ].
	repo := (self spec getFile == nil or: [ wc ancestry ancestors isEmpty ])
		ifTrue: [ (self repositoriesFrom: repoSpecs ignoreOverrides: true) first ]
		ifFalse: [
			([ self resolveSpec: self spec from: repoSpecs ] 
				on: Error 
				do: [:ignored | ^self ]) repository ].
	aCollection add: self spec -> repo
!

repositoryFor: pkgSpec from: repositorySpecs

	^([self resolveSpec: pkgSpec from: repositorySpecs]
		on: Error
		do: [:ignored | ^nil ]) repository
!

repositoryFor: pkgSpec with: gofer

	^([self resolveSpec: pkgSpec with: gofer]
		on: Error
		do: [:ignored | ^nil ]) repository
!

resolveSpec: pkgSpec from: repositorySpecs

	| gofer |
	gofer := MetacelloGofer new.
	gofer disablePackageCache.
	(self repositoriesFrom: repositorySpecs ignoreOverrides: true) 
		do: [:repo | gofer repository: repo ].
	^self resolveSpec: pkgSpec with: gofer
!

resolveSpec: pkgSpec with: gofer

	^pkgSpec goferLoaderReference resolveWith: gofer
!

savePackageUsing: repositorySpecs

	| wc repo repoSpecs newVersion |
	(wc := self spec workingCopy) == nil ifTrue: [ ^false ].
	(wc ancestry ancestors notEmpty and: [ wc modified not])
		ifTrue: [ ^false ].
	repoSpecs := self spec repositorySpecs notEmpty
				ifTrue: [ self spec repositorySpecs ]
				ifFalse: [ repositorySpecs ].
	(self spec getFile == nil or: [ wc ancestry ancestors isEmpty ])
		ifTrue: [ repo := (self repositoriesFrom: repoSpecs ignoreOverrides: true) first ]
		ifFalse: [
			[ 	| newer |
				repo := (self resolveSpec: self spec from: repoSpecs) repository.
				newer := wc possiblyNewerVersionsIn: repo.
				newer isEmpty not 
					ifTrue: [
						(MetacelloPlatform current confirm: 'CAUTION!! These versions in the repository may be newer:', 
							String cr, newer printString, String cr,
							'Do you really want to save this version?') ifFalse: [ ^false ]] ] 
				on: Error 
				do: [:ignored | ^false ] ].
	(newVersion :=  wc newVersion) == nil ifTrue: [ ^false ].
	repo storeVersion: newVersion.
	^true
! !

!MetacelloLoadingMCSpecLoader methodsFor:'doits'!

postLoad: packageOrVersionSpec

	| block |
	(block := packageOrVersionSpec postLoadDoItBlock) ~~ nil
		ifTrue: [ block valueWithPossibleArgs: { self. packageOrVersionSpec. } ]
!

preLoad: packageOrVersionSpec

	| block |
	(block := packageOrVersionSpec preLoadDoItBlock) ~~ nil
		ifTrue: [ block valueWithPossibleArgs: { self. packageOrVersionSpec. } ]
! !

!MetacelloLoadingMCSpecLoader methodsFor:'packages'!

latestPackage: aString fromRepository: repositorySpecs

	| gofer |
	gofer := MetacelloGofer new.
	gofer disablePackageCache.
	(self repositoriesFrom: repositorySpecs) do: [:repo | gofer repository: repo ].
	^([(GoferPackageReference name: aString) resolveWith: gofer]
		on: Error
		do: [:ignored | ^ nil ]) name
! !

!MetacelloLoadingMCSpecLoader methodsFor:'private'!

linearLoadPackageSpec: packageSpec gofer: gofer

	MetacelloPlatform current
		do:  [ | loadBlock goferLoad answers resolvedReference repo |
			resolvedReference := self resolvePackageSpec: packageSpec gofer: gofer.
			resolvedReference isNil ifTrue: [ "Package version already loaded into image" ^self ].
			loadBlock := [
				self preLoad: packageSpec.
				goferLoad := MetacelloGoferLoad on: MetacelloGofer new.
				goferLoad addResolved: resolvedReference.
				goferLoad execute.
				MetacelloPlatform current clearCurrentVersionCache.
				self postLoad: packageSpec ].
			(answers := packageSpec answers) notEmpty
				ifTrue: [ loadBlock valueSupplyingMetacelloAnswers: answers ]
				ifFalse: [ loadBlock value].
			repo := resolvedReference repository.
			self hasRepositoryOverrides
				ifTrue: [
					repo := self loaderPolicy repositoryMap 
								at: resolvedReference name 
								ifAbsent: [ resolvedReference repository ].
					resolvedReference workingCopy repositoryGroup addRepository:  repo ]
				ifFalse: [ resolvedReference workingCopy repositoryGroup addRepository: resolvedReference repository ].
			Transcript cr; show: 'Loaded -> ', resolvedReference name, ' --- ', repo description, ' --- ', resolvedReference repository description  ]
		displaying: 'Loading ', packageSpec file
! !

!MetacelloLoadingMCSpecLoader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloLoadingMCSpecLoader.st,v 1.1 2011/08/20 17:57:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloLoadingMCSpecLoader.st,v 1.1 2011/08/20 17:57:06 cg Exp $'
! !
