"{ Package: 'stx:goodies/metacello' }"

Object subclass:#MetacelloMCPartiallyLoadedStatus
	instanceVariableNames:'hasNoPackage hasNoProject aProjectIsLoaded aPackageIsLoaded
		aLoadedProjectIsExact aLoadedPackageIsExact
		aLoadedProjectIsCurrent aLoadedPackageIsCurrent
		aLoadedProjectIsNotCurrent aLoadedPackageIsNotCurrent
		aProjectNotLoaded aPackageNotLoaded vrsnStatus abort'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Specs'
!


!MetacelloMCPartiallyLoadedStatus class methodsFor:'instance creation'!

new
	^ self basicNew initialize
! !

!MetacelloMCPartiallyLoadedStatus methodsFor:'accessing'!

aLoadedPackageIsCurrent
	^ aLoadedPackageIsCurrent
!

aLoadedPackageIsCurrent: aBoolean
	aLoadedPackageIsCurrent := aBoolean
!

aLoadedPackageIsExact
	^ aLoadedPackageIsExact
!

aLoadedPackageIsExact: aBoolean
	aLoadedPackageIsExact := aBoolean
!

aLoadedPackageIsNotCurrent
	^ aLoadedPackageIsNotCurrent
!

aLoadedPackageIsNotCurrent: aBoolean
	aLoadedPackageIsNotCurrent := aBoolean
!

aLoadedProjectIsCurrent: aBoolean
	aLoadedProjectIsCurrent := aBoolean
!

aLoadedProjectIsExact
	^ aLoadedProjectIsExact
!

aLoadedProjectIsExact: aBoolean
	aLoadedProjectIsExact := aBoolean
!

aLoadedProjectIsNotCurrent
	^ aLoadedProjectIsNotCurrent
!

aLoadedProjectIsNotCurrent: aBoolean
	aLoadedProjectIsNotCurrent := aBoolean
!

aPackageIsLoaded
	^aPackageIsLoaded
!

aPackageIsLoaded: aBoolean
	aPackageIsLoaded := aBoolean
!

aPackageNotLoaded
	^aPackageNotLoaded
!

aPackageNotLoaded: aBoolean
	aPackageNotLoaded := aBoolean
!

aProjectIsLoaded
	^aProjectIsLoaded
!

aProjectIsLoaded: aBoolean
	aProjectIsLoaded := aBoolean
!

aProjectNotLoaded
	^aProjectNotLoaded
!

aProjectNotLoaded: aBoolean
	aProjectNotLoaded := aBoolean
!

abort
	^ abort
!

abort: aBoolean
	abort := aBoolean
!

hasNoPackage
	^ hasNoPackage
!

hasNoPackage: aBoolean
	hasNoPackage := aBoolean
!

hasNoProject
	^ hasNoProject
!

hasNoProject: aBoolean
	hasNoProject := aBoolean
!

vrsnStatus
	^vrsnStatus
! !

!MetacelloMCPartiallyLoadedStatus methodsFor:'evaulation'!

evaluateStatus: validStatusList
        | valid |
        self abort
                ifTrue: [ ^false ]
                ifFalse: [ 
                        ^ (self hasNoProject or: [ self vrsnStatus isEmpty ])
                                ifTrue: [ true ]
                                ifFalse: [ 
                                        valid := true.
                                        vrsnStatus
                                                do: [ :status | 
                                                        (validStatusList includes: status)
                                                                ifFalse: [ valid := false ] ].
                                        valid ] ]

    "Modified: / 21-08-2011 / 13:00:47 / cg"
! !

!MetacelloMCPartiallyLoadedStatus methodsFor:'initialization'!

initialize
	super initialize.
	hasNoPackage := hasNoProject := true.
	aProjectIsLoaded := aPackageIsLoaded := false.
	aLoadedProjectIsExact := aLoadedPackageIsExact := false.
	aLoadedProjectIsCurrent := aLoadedPackageIsCurrent := false.
	aLoadedProjectIsNotCurrent := aLoadedPackageIsNotCurrent := false.
	aProjectNotLoaded := aPackageNotLoaded := false.
	vrsnStatus := Set new.
	abort := false
! !

!MetacelloMCPartiallyLoadedStatus methodsFor:'testing'!

isAllLoadedToSpec: matchBlock
        "all projects and packages are loaded and match specification"

	(self evaluateStatus: #(#allLoadedToSpec))
		ifTrue: [ 
			((hasNoPackage
				or: [ 
					aPackageIsLoaded & aPackageNotLoaded not & aLoadedPackageIsExact & aLoadedPackageIsNotCurrent not
						& aLoadedPackageIsCurrent not ])
				and: [ 
					hasNoProject
						or: [ 
							aProjectIsLoaded & aProjectNotLoaded not & aLoadedProjectIsExact & aLoadedProjectIsNotCurrent not
								& aLoadedProjectIsCurrent not ] ])
				ifTrue: [ matchBlock value: #allLoadedToSpec ] ]
!

isLoadedMatchConstraints: matchBlock
	"all loaded projects and packages match constraints (at least one package loaded)"

	| matchStatus |
	matchStatus := #loadedMatchConstraints.
	(self evaluateStatus: #(#allLoadedToSpec #loadedToSpec #loadedMatchConstraints))
		ifTrue: [ 
			aPackageIsLoaded & aProjectIsLoaded
				ifTrue: [ 
					(aLoadedPackageIsExact | aLoadedPackageIsCurrent & aLoadedPackageIsNotCurrent not
						and: [ aLoadedProjectIsExact | aLoadedProjectIsCurrent & aLoadedProjectIsNotCurrent not ])
						ifTrue: [ matchBlock value: matchStatus ] ]
				ifFalse: [ 
					aPackageIsLoaded
						ifTrue: [ 
							aLoadedPackageIsExact | aLoadedPackageIsCurrent & aLoadedPackageIsNotCurrent not
								ifTrue: [ matchBlock value: matchStatus ] ]
						ifFalse: [ 
							hasNoPackage & (aLoadedProjectIsExact | aLoadedProjectIsCurrent) & aLoadedProjectIsNotCurrent not
								ifTrue: [ matchBlock value: matchStatus ] ] ] ]
!

isLoadedToSpec: matchBlock
	"all loaded projects and packages match specifications (at least one package loaded)"

	| matchStatus |
	matchStatus := #loadedToSpec.
	(self evaluateStatus: #(#allLoadedToSpec #loadedToSpec))
		ifTrue: [ 
			aPackageIsLoaded & aProjectIsLoaded
				ifTrue: [ 
					(aLoadedPackageIsExact & aLoadedPackageIsCurrent not & aLoadedPackageIsNotCurrent not
						and: [ aLoadedProjectIsExact & aLoadedProjectIsCurrent not & aLoadedProjectIsNotCurrent not ])
						ifTrue: [ matchBlock value: matchStatus ] ]
				ifFalse: [ 
					aPackageIsLoaded
						ifTrue: [ 
							aLoadedPackageIsExact & aLoadedPackageIsCurrent not & aLoadedPackageIsNotCurrent not
								ifTrue: [ matchBlock value: matchStatus ] ]
						ifFalse: [ 
							hasNoPackage & aLoadedProjectIsExact & aLoadedProjectIsCurrent not & aLoadedProjectIsNotCurrent not
								ifTrue: [ matchBlock value: matchStatus ] ] ] ]
!

isSomethingLoaded: somethingLoadedBlock
	"at least one package loaded"

	(self evaluateStatus: #(#allLoadedToSpec #loadedToSpec #loadedMatchConstraints #somethingLoaded))
		ifTrue: [ 
			aPackageIsLoaded
				ifTrue: [ somethingLoadedBlock value: #somethingLoaded ] ]
! !

!MetacelloMCPartiallyLoadedStatus class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCPartiallyLoadedStatus.st,v 1.2 2011/08/21 11:01:43 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCPartiallyLoadedStatus.st,v 1.2 2011/08/21 11:01:43 cg Exp $'
! !
