"{ Package: 'stx:goodies/metacello' }"

MetacelloProject subclass:#MetacelloMCProject
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Model'
!

MetacelloMCProject comment:''
!


!MetacelloMCProject methodsFor:'development support'!

fetchProject
	"fetch the latest version of the configuration package"
	
	^self fetchProject: MetacelloLoaderPolicy new
!

fetchProject: aLoaderPolicy
	"fetch the latest version of the configuration package"
	
	| mcLoader |
	(mcLoader := self loader) == nil
		ifTrue: [ mcLoader := self project loaderClass on: nil ].
	mcLoader loaderPolicy: aLoaderPolicy.
	mcLoader doingLoads: [ 
		MCWorkingCopy
			managersForClass: self configuration class
			do: [:mgr | | pkg |
				pkg := self packageSpec.
				mgr repositoryGroup repositories do: [:repo | pkg repositories repository: (repo asRepositorySpecFor: self) ].
				pkg name: mgr packageName.
				pkg fetchUsing: mcLoader.
				^true ]].
	^true
!

goferBranch: branchName project: commitMessage

	| pkgSpec |
	(pkgSpec := self projectPackage) == nil ifTrue: [ ^false ].
	pkgSpec file: pkgSpec name, '.', branchName.
	^pkgSpec goferBranchPackage: branchName message: commitMessage
!

goferCommitProject: commitMessage

	| pkgSpec |
	(pkgSpec := self projectPackage) == nil ifTrue: [ ^false ].
	^pkgSpec goferCommitPackage: commitMessage
!

packagesNeedSaving: aVersionString
	"Answer a collection of associations (package -> repository) representing the packages 
	 reachable from this project that need to be saved"

	| packages |
	packages := Set new.
	(self version: aVersionString) spec 
		packagesNeedSavingVisited: (Set with: self configuration class name asString)
		into: packages.
	^packages
!

projectPackage

	MCWorkingCopy
		managersForClass: self configuration class
		do: [:mgr | | pkgSpec repo |
			pkgSpec := 
				self packageSpec
					name: mgr packageName;
					yourself.
			mgr ancestors notEmpty
				ifTrue: [ pkgSpec file: mgr ancestors first name ].
			repo := mgr repositoryGroup repositories 
				detect: [:each | each ~~ MCCacheRepository default ] 
				ifNone: [ 
					Transcript cr; show: 'Using cache repository for ', self label, ' project package'.
					MCCacheRepository default ].
			pkgSpec repository: repo description.
			^pkgSpec].
	^nil
!

saveProject

	| pkgSpec |
	(pkgSpec := self projectPackage) == nil ifTrue: [ ^false ].
	^pkgSpec savePackage
!

updateProject
	"load the latest version of the configuration package"

	"WARNING: don't forget to refresh your project instance after doing an #updateProject, 
	 otherwise your instance won't reflect the info in the freshly loaded configuration"
	
	^self updateProject: MetacelloLoaderPolicy new
!

updateProject: aLoaderPolicy
	"load the latest version of the configuration package"

	"WARNING: don't forget to refresh your project instance after doing an #updateProject, 
	 otherwise your instance won't reflect the info in the freshly loaded configuration"

	| mcLoader |
	(mcLoader := self loader) == nil
		ifTrue: [ mcLoader := self project loaderClass on: nil ].
	mcLoader loaderPolicy: aLoaderPolicy.
	mcLoader
		doingLoads: [ 
			MCWorkingCopy
				managersForClass: self configuration class
				do: [ :mgr | 
					| pkg ar |
					pkg := self packageSpec.
					mgr repositoryGroup repositories do: [ :repo | pkg repositories repository: (repo asRepositorySpecFor: self) ].
					ar := mgr metacelloPackageNameWithBranch.
					pkg name: (ar at: 1).
					(ar at: 2) notEmpty
						ifTrue: [ pkg file: (ar at: 2) ].
					pkg load.
					^ true ] ].
	^ true
! !

!MetacelloMCProject methodsFor:'private'!

pragmaKeywords

	^super pragmaKeywords, #(projectPackage:attribute: packages:attribute: repositories:attribute: )
! !

!MetacelloMCProject methodsFor:'repository creation'!

createRepository: aRepositorySpec

	^ MetacelloPlatform current createRepository: aRepositorySpec
! !

!MetacelloMCProject methodsFor:'repository updating'!

updatePackageRepositoriesFor: versionString

	| versionSpec |
	(versionSpec := (self version: versionString) versionSpec) packageSpecsInLoadOrder do: [:pkgSpec |
		pkgSpec updatePackageRepositoriesFor: versionSpec ].
	^true
! !

!MetacelloMCProject methodsFor:'spec classes'!

defaultLoaderClass

	^MetacelloLoadingMCSpecLoader
!

packageSpec

	^self packageSpecClass for: self
!

packageSpecClass

	^MetacelloPackageSpec
!

projectSpecClass

	^MetacelloMCProjectSpec
!

repositoriesSpec

	^self repositoriesSpecClass for: self
!

repositoriesSpecClass

	^MetacelloRepositoriesSpec
!

repositorySpec

	^self repositorySpecClass for: self
!

repositorySpecClass

	^MetacelloRepositorySpec
!

versionSpecClass

	^MetacelloMCVersionSpec
! !

!MetacelloMCProject class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCProject.st,v 1.1 2011/08/20 18:07:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCProject.st,v 1.1 2011/08/20 18:07:06 cg Exp $'
! !
