"{ Package: 'stx:goodies/metacello' }"

MetacelloProjectSpec subclass:#MetacelloMCProjectSpec
	instanceVariableNames:'projectPackage'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Specs'
!

MetacelloMCProjectSpec comment:''
!


!MetacelloMCProjectSpec methodsFor:'accessing'!

className: aString

	super className: aString.
	(projectPackage ~~ nil and: [ projectPackage getFile == nil ])
		ifTrue: [ projectPackage file: aString ]
!

file: aString

	self projectPackage 
		name: aString;
		file: aString
!

loadPackageList

	| vrsn pkgs |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^#() ].
	pkgs := OrderedCollection new.
	(self loadListForVersion: vrsn) do: [:nm |
		pkgs addAll: ((vrsn packagesForSpecNamed: nm ) collect: [:each | each name ])].
	^pkgs
!

loadedPackageNames: aLoader

	| vrsn pkgs |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^#() ].
	pkgs := OrderedCollection new.
	(self loadListForVersion: vrsn) do: [:nm |
		(vrsn packagesForSpecNamed: nm ) do: [:pkg |
			(pkg isPackageLoaded: aLoader) ifTrue: [ pkgs add: pkg name ]]].
	^pkgs
!

packageFileSpecFor: aMetacelloPackagesSpec

	^(aMetacelloPackagesSpec project projectReferenceSpec)
			name: self name;
			projectReference: self copy;
			yourself.
!

projectPackage: anObject
	projectPackage := anObject
!

repository: aString

	self projectPackage repository: aString
!

repository: aString username: username password: password

	self projectPackage repository: aString username: username password: password
! !

!MetacelloMCProjectSpec methodsFor:'copying'!

postCopy

	super postCopy.
	projectPackage := projectPackage copy.
! !

!MetacelloMCProjectSpec methodsFor:'development support'!

updateForSpawnMethod: sourceSpec
	"This means that this spec was used in a baseline and will be used in a version .... drop all information that isn't useful"
	
	className := operator := loads := projectPackage := nil.
	sourceSpec ~~ nil ifTrue: [ versionString := sourceSpec versionString ].
!

updatePackageSpec: updatedSpecs
	"Add project copy to updatedSpecs if the current version of the project 
	 is different from the receiver's version"
	
	|  prj currentVersion spec |
	self className == nil ifTrue: [ ^self ].
	prj := self projectClassProject.
	(currentVersion := prj currentVersion) = self versionOrNil
		ifTrue: [ ^self ].
	currentVersion == nil ifTrue: [ ^self ].
	spec := self copy.
	spec versionString: currentVersion versionString.
	updatedSpecs at: spec name put: spec
! !

!MetacelloMCProjectSpec methodsFor:'loading'!

determineCurrentVersionForLoad
	"don't use self currentVersion, because we are interested in the currentVersion of the project as loaded in image, not the current version relative to our load list"
	
	| prjct version currentVersion packageAndProjectNames cvs |
	self projectClass == nil ifTrue: [ ^nil ].
	(version := self versionOrNil) == nil ifTrue: [ ^nil ].
	version blessing == #baseline ifTrue: [ ^version ].
	self loader ignoreImage ifTrue: [ ^version ].
	prjct := self projectClass new project.
	prjct loader: self loader.
	(currentVersion := prjct currentVersion) == nil ifTrue: [ ^nil ].
	(cvs := currentVersion versionStatus) == #somethingLoaded ifTrue: [ ^nil ].
	(#(allLoadedToSpec loadedToSpec loadedMatchConstraints) includes: (cvs := currentVersion versionStatus)) 
		ifTrue: [ 
			(currentVersion perform: self operator with: version)
				ifTrue: [ "load currentVersion" ^currentVersion ].
			"load version" ^nil ].
	version = currentVersion ifTrue: [ ^currentVersion ].
	(MetacelloProjectSpecLoadConflict projectSpec: self) 
		signal: 
			'Project load conflict for', 
			prjct label printString,  
			' between current version ',
			currentVersion printString, 
			'(', 
			cvs asString, 
			') and specified version ', 
			version printString,
			'. Press resume to continue with load anyway'.
	^nil
!

ensureLoadedForDevelopmentUsing: mcLoader
	"for #development projects, always need latest version of package when contemplating a load"

	| ensured |
	ensured := mcLoader ensuredMap at: self name ifAbsent: [ nil ].
	self projectClass ~~ nil
		ifTrue: [  | vrsn |
			vrsn := self versionOrNil.
			(vrsn ~~ nil and: [ vrsn blessing == #development ])
				ifTrue: [ 
					ensured ~~ #latest
						ifTrue: [
							mcLoader ensureForDevelopment
								ifTrue: [ self projectPackage loadUsing: mcLoader ]
								ifFalse: [ self projectPackage fetchUsing: mcLoader ].
							mcLoader ensuredMap at: self name put: #latest ].
					^self]].
	ensured == nil
		ifTrue:  [ 
			mcLoader ensureForDevelopment
				ifTrue: [ self projectPackage loadUsing: mcLoader ]
				ifFalse: [ self projectPackage fetchUsing: mcLoader ].
			mcLoader ensuredMap at: self name put: #present ]
!

ensureProjectLoaded
	"Ensure that the MetacelloProject is loaded in image. 
	 projectClass == nil or requested version non-existent warrants a project package load."

	(self projectClass == nil or: [ self versionOrNil == nil ])
		ifTrue: [ 
			self projectPackage load.
			^ true ].
	^ false
!

load

	| displayString |
	displayString := 'Project: ', self name.
	self versionString ~~ nil ifTrue: [ displayString := displayString, ' ', self versionString ].
	Transcript cr; show: displayString.
	self ensureProjectLoaded.
	self loadVersion: self determineCurrentVersionForLoad
!

loadVersion: aVersionOrNil
	"Load the correct version of the project"

	| vrsn mcLoader list |
	vrsn := aVersionOrNil .
	vrsn == nil
		ifTrue: [ 
			(vrsn := self versionOrNil) == nil
				ifTrue: [
					 ^(MetacelloProjectSpecLoadError projectSpec: self) 
						signal: 'No version found for ', 
									self versionString printString, 
									' of ', 
									self className asString ]].
	mcLoader := self loader copy.
	mcLoader operator: self operator.
	vrsn loader: mcLoader.
	list := (mcLoader ignoreImage 
		ifTrue: [ self loadListForVersion: vrsn ]
		ifFalse: [ vrsn packageAndProjectNamesToLoad: (self loadListForVersion: vrsn) loader: mcLoader ]) asSet.
	MetacelloPlatform current 
		useStackCacheDuring: [:dict | | projectCache cachedList |
			projectCache := dict 
					at: self projectClass 
					ifAbsent: [ dict at: self projectClass put: Dictionary new].
			(cachedList := projectCache at: vrsn ifAbsent: []) == nil
				ifTrue: [  projectCache at: vrsn put: list ]
				ifFalse: [ 
					(cachedList size = list size and: [ cachedList includesAllOf: list ]) 
						ifTrue: [ 
							"no need to refetch list ... recursion stoppper (Issue 95)" 
							^self ]
						ifFalse: [  projectCache at: vrsn put: list ]].
			mcLoader preLoad: self.
			vrsn fetchRequiredFromArray: list.
			mcLoader postLoad: self.
			] 
		defaultDictionary: Dictionary new.
! !

!MetacelloMCProjectSpec methodsFor:'merging'!

mergeMap

	| map |
	map := super mergeMap.
	map at: #projectPackage put: projectPackage.
	^map
!

mergeSpec: anotherSpec

	| newSpec anotherProjectPackage map |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	(anotherProjectPackage := map at: #projectPackage) ~~ nil
		ifTrue: [ 
			newSpec projectPackage: 
				(newSpec projectPackage == nil
					ifTrue: [ anotherProjectPackage ]
					ifFalse: [ newSpec projectPackage mergeSpec: anotherProjectPackage ])].
	^newSpec
!

nonOverridable

	^super nonOverridable, #( projectPackage )
! !

!MetacelloMCProjectSpec methodsFor:'private'!

loadListForVersion: vrsn 

	^ (self loads == nil
			or: [self loads isEmpty])
		ifTrue: [vrsn spec defaultPackageNames]
		ifFalse: [self loads]
!

resolveToAllPackagesIn: aVersionSpec visited: visited
	| vrsn |
	visited
		pushProject: [ 
			visited
				visit: self
				doing: [ :spec | 
					spec ensureProjectLoaded.
					(vrsn := spec versionOrNil) == nil
						ifTrue: [ (MetacelloVersionDoesNotExistError project: self project versionString: self versionString) signal ].
					^ vrsn allPackagesForSpecNamed: (self loadListForVersion: vrsn) ifAbsent: [ self error: 'invalid loads: spec' ] ] ].
	^ #()
!

resolveToLoadableSpec

	^self copy
! !

!MetacelloMCProjectSpec methodsFor:'querying'!

file
	
	^self projectPackage file
!

packageRepository
	
	^nil
!

projectClass

	self className == nil ifTrue: [ ^nil ].
	^Smalltalk at: self className asSymbol  ifAbsent: []
!

projectClassProject
	"indirection needed when projectClass is _not_ a subclass of MetacelloProject"
	
	^self projectClass new project
!

projectPackage
	^ projectPackage
!

relativeCurrentVersion
	"currentVersion calculated relative to the loadList"
	
	| vrsn expanded loadList |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^nil ].
	expanded := [ vrsn expandToLoadableSpecNames: (loadList := self loadListForVersion: vrsn) ] 
					on: Error 
					do: [:ex |
						vrsn blessing == #development
							ifTrue: [ 
								self projectPackage load.
								vrsn := self versionOrNil.
								ex return: (vrsn expandToLoadableSpecNames: loadList)].
						ex pass].
	^self projectClassProject currentVersionAgainst: expanded
		
!

repository
	
	^self projectPackage packageRepository
!

version
	"Empty version string means use latestVersion or #bleedingEdge"

	self projectClass == nil
		ifTrue: [ ^ nil ].
	^ self versionString == nil
		ifTrue: [ 
			| vrsn |
			"Eventually it will become an error to not specify a project reference version as default: #stable is the preferred default"
			"self deprecated: 'Must specify a project reference version.'."
			self flag: 'deprecate after version 1.0'.
			(vrsn := self projectClassProject latestVersion) == nil
				ifTrue: [ self projectClassProject version: #bleedingEdge ]
				ifFalse: [ vrsn ] ]
		ifFalse: [ self projectClassProject version: self versionString ]
!

versionOrNil

	^[ self version ] on: MetacelloVersionDoesNotExistError do: [:ex | ^nil ].
! !

!MetacelloMCProjectSpec methodsFor:'testing'!

compareCurrentVersion: anOperator targetVersionStatus: targetVersionStatus using: anMCLoader

	| cv vrsn prjct |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^false ].
	prjct := self projectClassProject.
	prjct loader: anMCLoader.
	(cv := prjct currentVersion) == nil ifTrue: [ ^false ].
	(targetVersionStatus includes: cv versionStatus)
		ifTrue: [ ^cv perform: anOperator with: vrsn ].
	^false
	
 
!

compareRelativeCurrentVersion: anOperator targetVersionStatus: targetVersionStatus using: anMCLoader

	| cv vrsn |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^false ].
	(cv := self relativeCurrentVersion) == nil ifTrue: [ ^false ].
	(targetVersionStatus includes: cv versionStatus)
		ifTrue: [ ^cv perform: anOperator with: vrsn ].
	^false
	
 
!

hasNonVersionStringField
	| hasVersionString hasOperator hasProjectPackage hasLoads hasClassName hasPreLoadDoIt hasPostLoadDoIt |
	hasClassName := self className ~~ nil.
	hasVersionString := self versionString ~~ nil.
	hasOperator := operator ~~ nil.
	hasProjectPackage := self projectPackage ~~ nil
		and: [ self projectPackage name ~~ nil or: [ self projectPackage packageRepository ~~ nil ] ].
	hasLoads := self loads ~~ nil.
	hasPreLoadDoIt := self getPreLoadDoIt ~~ nil.
	hasPostLoadDoIt := self getPostLoadDoIt ~~ nil.
	^ hasClassName | hasOperator | hasProjectPackage | hasLoads | hasPreLoadDoIt | hasPostLoadDoIt
!

isPartiallyLoaded: aLoader

	| vrsn |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^false ].
	(self loadListForVersion: vrsn) do: [:nm |
		(vrsn packagesForSpecNamed: nm ) do: [:pkg |
			(pkg isPackageLoaded: aLoader) ifTrue: [ ^true ]]].
	^false
!

isPossibleBaseline

	| vrsn |
	(vrsn := self versionOrNil) == nil ifTrue: [ ^false ].
	(vrsn allPackagesForSpecNamed: (self loadListForVersion: vrsn)) do: [:pkg |
		pkg workingCopy == nil ifTrue: [ ^false ]].
	^true
! !

!MetacelloMCProjectSpec methodsFor:'visiting'!

projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	projectBlock value: self
! !

!MetacelloMCProjectSpec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCProjectSpec.st,v 1.1 2011/08/20 18:16:09 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCProjectSpec.st,v 1.1 2011/08/20 18:16:09 cg Exp $'
! !
