"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloMCSpecLoaderTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloMCSpecLoaderTest comment:''
!


!MetacelloMCSpecLoaderTest methodsFor:'baselines'!

baseline10: spec 
	<version: '1.0-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec 
			package: 'GoferFoo';
			package: 'GoferBar';
			package: 'GoferFaux';
			yourself ].
!

baseline20: spec 
	<version: '2.0-baseline'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec repository: 'dictionary://Metacello_Configuration_Test_Alternate_Repository'.
		spec 
			package: 'GoferFoo';
			package: 'GoferBar';
			yourself ].
! !

!MetacelloMCSpecLoaderTest methodsFor:'running'!

doSilently

	^(#(#testPackageSave ) includes: testSelector asSymbol) not
!

tearDown
	super tearDown.
	MetacelloMonticelloResource reset. "testCommitPackageSpec modifies the repository resource"
! !

!MetacelloMCSpecLoaderTest methodsFor:'tests'!

testCopySpec

	| project version loader repo fakePackage |
	project := self project.
	version := project version: '1.0-baseline'.
	loader := version spec loader.
	repo := MCDictionaryRepository new.
	self assert: repo allVersionInfos size = 0.
	loader 
		copySpec: (version packageNamed: 'GoferFoo')
		from: version spec repositorySpecs
		to: repo.
	self assert: repo allVersionInfos size = 1.
	
	"error condition"
	fakePackage := version packageNamed: 'GoferFoo'.
	fakePackage name: 'MetacelloTestConfigurationOfFoo'.
	self assert: (loader 
		copySpec: fakePackage
		from: version spec repositorySpecs
		to: repo) == nil
!

testGoferCommit

	| project version repoSpec repo allVersionsSize pkg |
	project := self project.
	version := project version: '1.0'.
	repoSpec := version spec repositorySpecs first.
	repo := repoSpec createRepository.
	allVersionsSize := repo allVersionInfos size.
	version load.
	(Smalltalk classNamed: #GoferBar) compile: 'foo'.
	pkg := version packageNamed: 'GoferBar'.
	pkg repository: repoSpec.
	pkg goferCommitPackage: 'commit message'.
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
!

testLatestPackage

	| project version loader latestFile |
	project := self project.
	version := project version: '1.0-baseline'.
	loader := version spec loader.
	latestFile := loader 
		latestPackage: 'GoferFoo' 
		fromRepository: version spec repositorySpecs.
	self assert: latestFile = 'GoferFoo-lr.4'.

	latestFile := loader 
		latestPackage: 'GoferBar' 
		fromRepository: version spec repositorySpecs.
	self assert: latestFile = 'GoferBar-lr.1'.

	latestFile := loader 
		latestPackage: 'GoferFaux' 
		fromRepository: version spec repositorySpecs.
	self assert: latestFile = 'GoferFaux-tg.35'.

	"error condition"
	latestFile := loader 
		latestPackage: 'MetacelloTestConfigurationOfFoo' 
		fromRepository: version spec repositorySpecs.
	self assert: latestFile == nil.
!

testPackageSave

	| project version repoSpec repo allVersionsSize pkg |
	project := self project.
	version := project version: '1.0'.
	repoSpec := version spec repositorySpecs first.
	repo := repoSpec createRepository.
	allVersionsSize := repo allVersionInfos size.
	version load.
	(Smalltalk classNamed: #GoferBar) compile: 'foo'.
	pkg := version packageNamed: 'GoferBar'.
	pkg repository: repoSpec.
	[ pkg savePackage ]
		on: MCVersionNameAndMessageRequest
		do: [ :notifcation | notifcation resume: (Array with: notifcation suggestedName with: 'xxx') ].
	self assert: repo allVersionInfos size = (allVersionsSize + 1)
!

testPackageSpecLoad

	| project version repoSpec pkg wc |
	project := self project.
	version := project version: '1.0'.
	repoSpec := version spec repositorySpecs first.
	pkg := version packageNamed: 'GoferBar'.
	pkg repository: repoSpec.
	pkg load.
	wc := (GoferPackageReference name: 'GoferBar') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBar-jf.1').
!

testRepositoryFor

	| project version loader pkgSpec repo1 repo2 |
	project := self project.
	version := project version: '2.0-baseline'.
	loader := version spec loader.
	pkgSpec := version packageNamed: 'GoferBar'.
	repo1 := self monticelloRepository.
	repo2 := self alternateRepository.

	pkgSpec file: 'GoferBar.branch-lr.1'.
	self assert: 
		(loader 
			repositoryFor: pkgSpec 
			from: version spec repositorySpecs) == repo1.
	pkgSpec file: 'GoferBar-dkh.25'.
	self assert: 
		(loader 
			repositoryFor: pkgSpec 
			from: version spec repositorySpecs) == repo2.

	"error condition"
	pkgSpec file: 'MetacelloTestConfigurationOfFoo-dkh.1'.
	self assert: 
		(loader 
			repositoryFor: pkgSpec 
			from: version spec repositorySpecs) == nil.
! !

!MetacelloMCSpecLoaderTest methodsFor:'versions'!

version10: spec
	<version: '1.0' imports: #('1.0-baseline' )>
	
	spec for: #common do: [
		spec 
			package: 'GoferFoo' with: 'GoferFoo-lr.1';
			package: 'GoferBar' with: 'GoferBar-jf.1';
			package: 'GoferFaux' with: 'GoferFaux-tg.32';
			yourself. ].
! !

!MetacelloMCSpecLoaderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCSpecLoaderTest.st,v 1.1 2011/08/20 18:14:14 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCSpecLoaderTest.st,v 1.1 2011/08/20 18:14:14 cg Exp $'
! !
