"{ Package: 'stx:goodies/metacello' }"

MetacelloVersionSpec subclass:#MetacelloMCVersionSpec
	instanceVariableNames:'packages repositories'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Specs'
!

MetacelloMCVersionSpec comment:''
!


!MetacelloMCVersionSpec methodsFor:'accessing'!

computeVersionStatus: matchBlock
	self computeVersionStatus: (self expandToLoadableSpecNames: #('ALL')) matchBlock: matchBlock
!

computeVersionStatus: resolvedPackageAndProjectNames matchBlock: matchBlock
	| status |
	status := resolvedPackageAndProjectNames isNil
		ifTrue: [ self isPartiallyCurrent ]
		ifFalse: [ self isPartiallyCurrentAgainst: resolvedPackageAndProjectNames ].
	status isAllLoadedToSpec: matchBlock.
	status isLoadedToSpec: matchBlock.
	status isLoadedMatchConstraints: matchBlock.
	status isSomethingLoaded: matchBlock
!

packages

	packages == nil ifTrue: [ packages := self project packagesSpec ].
	^ packages
!

packages: anObject
	packages := anObject
!

packagesSpec

	^self packages
!

repositories
	repositories == nil ifTrue: [ repositories := self project repositoriesSpec ].
	^ repositories
!

repositories: anObject
	repositories := anObject
!

repositoriesSpec

	^self repositories
!

repository: aString
	self repositoriesSpec add: aString
!

repository: aString username: username password: password
	self repositoriesSpec repository: aString username: username password: password
! !

!MetacelloMCVersionSpec methodsFor:'copying'!

postCopy

	super postCopy.
	packages := packages copy.
	repositories := repositories copy.
	
! !

!MetacelloMCVersionSpec methodsFor:'development support'!

forceUpdatedPackageSpecs

	| updatedSpecs mcLoader |
	updatedSpecs := Dictionary new.
	mcLoader := self loader.
	self packages map valuesDo: [:pkg | pkg forceUpdatePackageSpec: updatedSpecs using: mcLoader].
	^updatedSpecs
!

packagesNeedSavingVisited: visitedProjects into: aCollection

	self packages map valuesDo: [:pkg | pkg packagesNeedSavingVisited: visitedProjects using: self repositories map values into: aCollection ].
!

updateForSpawnMethod: sourceSpec
	"This means that this spec was used in a baseline and will be used in a version .... drop all information that isn't useful"
	
	repositories := preLoadDoIt := postLoadDoIt := nil.
!

updatedPackageSpecs

	| updatedSpecs mcLoader |
	updatedSpecs := Dictionary new.
	mcLoader := self loader.
	self packages map valuesDo: [:pkg | pkg updatePackageSpec: updatedSpecs using: mcLoader].
	^updatedSpecs
! !

!MetacelloMCVersionSpec methodsFor:'enumerating'!

projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	self packageSpecsInLoadOrder do: [:pkgSpec |
		pkgSpec projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock ]
!

specListProjectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	self packages specListDo: [:pkgSpec |
		pkgSpec projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock ]
!

specsNamed: packageAndProjectNames projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	| map |
	map := self packages map.
	packageAndProjectNames do: [:name | | pkgSpec |
		(pkgSpec := map at: name ifAbsent: [ ]) ~~ nil
			ifTrue: [ pkgSpec projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock ]]
! !

!MetacelloMCVersionSpec methodsFor:'loading'!

defaultPackageNames
	"if there is a package named 'default' (a group) then it defines the default package names,
	 otherwise answer a list of all of the package names in this version"

	self packages packageNamed: 'default' ifAbsent: [ ^self packageNames ].
	^#('default')
!

expandToLoadableSpecNames: nameList

	| cacheKey names |
	cacheKey := Array with: self label with: nameList.
	^MetacelloPlatform current
		stackCacheFor: #loadableSpecNames
		at: cacheKey
		doing: [ :cache | 
			names := (self resolveToLoadableSpecs: nameList) collect: [:spec | spec name ].
			cache at: cacheKey put: names ].
!

repositorySpecs

	^self repositories map values
!

resolveToLoadableSpec: aString forMap: map packages: packageMap 
	| package |
	package := self packageNamed: aString forMap: map ifAbsent: [ ^self error: 'Name not found: ' , aString ].
	packageMap at: package name put: package.
	^ {package}
!

resolveToLoadableSpecs: nameList
	"Resolves names in namelist to the  list of packages and projects in the version that would be loaded. Projects are 
	not traversed during the transitive closure. The scope is that of the version itself.
	   If the spec is a package, answer a list including the package and the transitive closure on 
			its #requires: and #includes: fields.
	   If the spec is a project, answer the project.
	   If the spec is a group, answers the list of packages in the #includes: field of the group. 
			Groups in the #includes: field are expanded following the transitive closure on groups"

	| map |
	map := Dictionary new.
	self resolveToLoadableSpecs: nameList map: map.
	^map values
!

resolveToLoadableSpecs: required map: packageMap 

	| reqd allReqd map newReqd spec |
	reqd := required copy.
	allReqd := Set new.
	map := self packages map.
	[ reqd isEmpty ] whileFalse: [
		newReqd := Set new.
		reqd do: [:req | 
			(self resolveToLoadableSpec: req forMap: map packages: packageMap) 
				do: [:loadableSpec | 
					newReqd addAll: loadableSpec requires.
					newReqd addAll: loadableSpec includes ]].
		allReqd addAll: reqd.
		newReqd removeAllFoundIn: allReqd.
		reqd := newReqd ].
	packageMap keys do: [:pkgName |
		(spec := (packageMap at: pkgName) resolveToLoadableSpec) == nil
			ifTrue: [ packageMap removeKey: pkgName ]
			ifFalse: [ packageMap at: pkgName put: (packageMap at: pkgName) resolveToLoadableSpec ]]
! !

!MetacelloMCVersionSpec methodsFor:'merging'!

mergeMap

	| map |
	map := super mergeMap.
	map at: #packages put: self packages.
	map at: #repositories put: self repositories.
	^map
!

mergeSpec: anotherSpec

	| newSpec map anotherPackages anotherRepositories |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	(anotherPackages := map at: #packages) isEmpty not 
		ifTrue: [ 
			newSpec 
				packages: (self packages isEmpty
					ifTrue: [ anotherPackages ]
					ifFalse: [ self packages mergeSpec: anotherPackages ]) ].
	(anotherRepositories := map at: #repositories) isEmpty not
		ifTrue: [ 
			newSpec 
				repositories: (self repositories isEmpty
					ifTrue: [ anotherRepositories ]
					ifFalse: [ self repositories mergeSpec: anotherRepositories ]) ].
	^newSpec
!

nonOverridable

	^super nonOverridable, #( packages repositories)
! !

!MetacelloMCVersionSpec methodsFor:'printing'!

configMethodOn: aStream last: last indent: indent
	| spec hasRepositories hasPackageSpecs |
	hasRepositories := (spec := self repositoriesSpec) ~~ nil and: [ spec list isEmpty not ].
	hasPackageSpecs := false.
	self packagesSpec list
		do: [ :member | 
			member spec
				projectDo: [ :proj | 
					member spec name ~~ nil
						ifTrue: [ hasPackageSpecs := true ] ]
				packageDo: [ :package | 
					member spec name ~~ nil
						ifTrue: [ hasPackageSpecs := true ] ]
				groupDo: [ :group | 
					member spec name ~~ nil
						ifTrue: [ hasPackageSpecs := true ] ] ].
	super configMethodOn: aStream last: (hasRepositories | hasPackageSpecs) not indent: indent.
	hasRepositories
		ifTrue: [ 
			spec map values size = 1
				ifTrue: [ 
					aStream
						tab: indent;
						nextPutAll: 'spec repository: ';
						nextPutAll: spec map values first description printString , '.'.
					hasPackageSpecs ifTrue: [ aStream cr ]]
				ifFalse: [ 
					self
						configMethodOn: aStream
						for: spec
						selector: 'repositories:'
						last: hasPackageSpecs not 
						indent: indent ] ].
	self configPackagesSpecMethodOn: aStream indent: indent.
	last
		ifFalse: [ aStream cr ]
!

configPackagesSpecMethodOn: aStream indent: indent

	| projectSpecs packageSpecs groupSpecs |
	projectSpecs := OrderedCollection new.
	packageSpecs := OrderedCollection new.
	groupSpecs := OrderedCollection new.
	self packagesSpec list do: [:member |
		member spec 
			projectDo: [:proj | member spec name ~~ nil ifTrue: [ projectSpecs add: member ]]
			packageDo: [:package | member spec name ~~ nil ifTrue: [ packageSpecs add: member ]]
			groupDo: [:group | member spec name ~~ nil ifTrue: [ groupSpecs add: member ]]].
	projectSpecs isEmpty not
		ifTrue: [
			aStream tab: indent; nextPutAll: 'spec '.
			projectSpecs size > 1 ifTrue: [ aStream cr; tab: indent + 1 ].
			1 to: projectSpecs size  do: [:index | 
				(projectSpecs at: index) configMethodCascadeOn: aStream last: index == projectSpecs size indent: indent + 1.
				index ~= projectSpecs size ifTrue: [ aStream tab: indent + 1 ]]].
	packageSpecs isEmpty not
		ifTrue: [
			projectSpecs isEmpty not ifTrue: [ aStream cr ].
			aStream tab: indent; nextPutAll: 'spec '.
			packageSpecs size > 1 ifTrue: [ aStream cr; tab: indent + 1 ].
			1 to: packageSpecs size  do: [:index | 
				(packageSpecs at: index) configMethodCascadeOn: aStream last: index == packageSpecs size indent: indent + 1.
				index ~= packageSpecs size ifTrue: [ aStream tab: indent + 1 ]]].
	groupSpecs isEmpty not
		ifTrue: [
			projectSpecs isEmpty not | packageSpecs isEmpty not ifTrue: [ aStream cr ].
			aStream tab: indent; nextPutAll: 'spec '.
			groupSpecs size > 1 ifTrue: [ aStream cr; tab: indent + 1 ].
			1 to: groupSpecs size  do: [:index | 
				(groupSpecs at: index) configMethodCascadeOn: aStream last: index == groupSpecs size indent: indent + 1.
				index ~= groupSpecs size ifTrue: [ aStream tab: indent + 1 ]]].
!

configSpawnMethodOn: aStream indent: indent

	super configSpawnMethodOn: aStream indent: indent.
	self configPackagesSpecMethodOn: aStream indent: indent.
! !

!MetacelloMCVersionSpec methodsFor:'private'!

versionClass

	^MetacelloMCVersion
! !

!MetacelloMCVersionSpec methodsFor:'querying'!

currentlyLoadedClassesInVersion
	| classes |
	classes := Set new.
	self
		projectDo: [ :ignored |  ]
		packageDo: [ :packageSpec | 
			| wc |
			wc := [ packageSpec workingCopy ] on: Error do: [ :ex | ex return: nil ].
			wc ~~ nil
				ifTrue: [ classes addAll: wc packageInfo classes ] ]
		groupDo: [ :ignored |  ].
	^ classes
!

currentlyLoadedExtensionClassesInVersion
	| classes |
	classes := Dictionary new.
	self
		projectDo: [ :ignored |  ]
		packageDo: [ :packageSpec | 
			| wc |
			wc := [ packageSpec workingCopy ] on: Error do: [ :ex | ex return: nil ].
			wc ~~ nil
				ifTrue: [ wc packageInfo extensionClasses do: [:cl |
						classes at: cl put: (wc packageInfo extensionCategoriesForClass: cl)]] ]
		groupDo: [ :ignored |  ].
	^ classes
!

difference: otherVersionSpec
	"Return a a dictionary of additions, removals and modifications"

	| report myProjectSpecs myPackageSpecs otherProjectSpecs otherPackageSpecs |
	report := MetacelloMCVersionDiffReport new.
	myProjectSpecs := Dictionary new.
	myPackageSpecs := Dictionary new.
	self
		projectDo: [ :projectSpec | myProjectSpecs at: projectSpec name put: projectSpec ]
		packageDo: [ :packageSpec | myPackageSpecs at: packageSpec name put: packageSpec ]
		groupDo: [ :ignored |  ].
	otherProjectSpecs := Dictionary new.
	otherPackageSpecs := Dictionary new.
	otherVersionSpec
		projectDo: [ :projectSpec | otherProjectSpecs at: projectSpec name put: projectSpec ]
		packageDo: [ :packageSpec | otherPackageSpecs at: packageSpec name put: packageSpec ]
		groupDo: [ :ignored |  ].
	myProjectSpecs
		valuesDo: [ :myProjectSpec | 
			| otherProjectSpec |
			otherProjectSpec := otherProjectSpecs at: myProjectSpec name ifAbsent: [  ].
			otherProjectSpec == nil
				ifTrue: [ 
					report removals
						at: myProjectSpec name
						put:
							{(myProjectSpec versionString).
							''} ]
				ifFalse: [ 
					myProjectSpec versionString = otherProjectSpec versionString
						ifFalse: [ 
							report modifications
								at: myProjectSpec name
								put:
									{(myProjectSpec versionString).
									(otherProjectSpec versionString)} ] ] ].
	otherProjectSpecs
		valuesDo: [ :otherProjectSpec | 
			(myProjectSpecs at: otherProjectSpec name ifAbsent: [  ]) == nil
				ifTrue: [ 
					report additions
						at: otherProjectSpec name
						put:
							{''.
							(otherProjectSpec versionString)} ] ].
	myPackageSpecs
		valuesDo: [ :myPackageSpec | 
			| otherPackageSpec |
			otherPackageSpec := otherPackageSpecs at: myPackageSpec name ifAbsent: [  ].
			otherPackageSpec == nil
				ifTrue: [ 
					report removals
						at: myPackageSpec name
						put:
							{(myPackageSpec file).
							''} ]
				ifFalse: [ 
					myPackageSpec file = otherPackageSpec file
						ifFalse: [ 
							report modifications
								at: myPackageSpec name
								put:
									{(myPackageSpec file).
									(otherPackageSpec file)} ] ] ].
	otherPackageSpecs
		valuesDo: [ :otherPackageSpec | 
			(myPackageSpecs at: otherPackageSpec name ifAbsent: [  ]) == nil
				ifTrue: [ 
					report additions
						at: otherPackageSpec name
						put:
							{''.
							(otherPackageSpec file)} ] ].
	^report
!

packageNamed: aString

	^self packageNamed: aString ifAbsent: [ nil ]
!

packageNamed: aString forMap: map ifAbsent: aBlock

	| package |
	package := map
				at: aString
				ifAbsent: [].
	package == nil
		ifTrue: [
			(aString = 'default' or: [ aString = 'ALL'])
				ifTrue: [ 
					package := self project groupSpec name: aString;
				 		includes: self packageNames;					 
						yourself ]
				ifFalse: [ ^aBlock value ]].
	^package
!

packageNamed: aString ifAbsent: aBlock

	^self packageNamed: aString forMap: self packages map ifAbsent: aBlock
!

packageNames

	packages == nil ifTrue: [ ^#() ].
	^self packages map keys asSet 
!

packageSpecsInLoadOrder

	^self packages packageSpecsInLoadOrder
! !

!MetacelloMCVersionSpec methodsFor:'testing'!

isAllLoadedToSpec
	"all projects and packages are loaded and match specification"

	self isPartiallyCurrentAgainst isAllLoadedToSpec: [ :ignored | ^ true ].
	^ false
!

isAllLoadedToSpecAgainst: resolvedPackageAndProjectNames
	"all projects and packages are loaded and match specification"

	(self isPartiallyCurrentAgainst: resolvedPackageAndProjectNames) isAllLoadedToSpec: [ :ignored | ^ true ].
	^ false
!

isLoadedMatchConstraints
	"all loaded projects and packages match constraints (at least one package loaded)"
	
	self isPartiallyCurrentAgainst isLoadedMatchConstraints: [ :ignored | ^ true ].
	^ false
!

isLoadedMatchConstraintsAgainst: resolvedPackageAndProjectNames
	"all loaded projects and packages match constraints (at least one package loaded)"

	(self isPartiallyCurrentAgainst: resolvedPackageAndProjectNames) isLoadedMatchConstraints: [ :ignored | ^ true ].
	^ false
!

isLoadedToSpec
	"all loaded projects and packages match specifications (at least one package loaded)"
	
	self isPartiallyCurrentAgainst isLoadedToSpec: [ :ignored | ^ true ].
	^ false
!

isLoadedToSpecAgainst: resolvedPackageAndProjectNames
	"all loaded projects and packages match specifications (at least one package loaded)"

	(self isPartiallyCurrent: resolvedPackageAndProjectNames) isLoadedToSpec: [ :ignored | ^ true ].
	^ false
!

isPartiallyCurrent

	^self isPartiallyCurrentAgainst: (self expandToLoadableSpecNames: #('ALL'))
!

isPartiallyCurrentAgainst: resolvedPackageAndProjectNames
	| mcLoader status |
	status := MetacelloMCPartiallyLoadedStatus new.
	mcLoader := self loader.
	self
		specsNamed: resolvedPackageAndProjectNames
		projectDo: [ :prj | 
			| vrsn currentVersion |
			status hasNoProject: false.
			vrsn := prj versionOrNil.
			vrsn ~~ nil
				ifTrue: [ 
					(currentVersion := prj relativeCurrentVersion) ~~ nil
						ifTrue: [ status vrsnStatus add: currentVersion versionStatus ] ].
			currentVersion ~~ nil
				ifTrue: [ 
					status aProjectIsLoaded: true.
					(currentVersion perform: #= with: vrsn)
						ifTrue: [ status aLoadedProjectIsExact: true ]
						ifFalse: [ 
							(currentVersion perform: prj projectReference operator with: vrsn)
								ifTrue: [ status aLoadedProjectIsCurrent: true ]
								ifFalse: [ status aLoadedProjectIsNotCurrent: true ] ] ]
				ifFalse: [ status aProjectNotLoaded: true ] ]
		packageDo: [ :pkg | 
			status hasNoPackage: false.
			pkg
				currentPackageLoaded: [ :versionInfos :file | 
					| wcName wcRef fileRef exact current |
					status aPackageIsLoaded: true.
					versionInfos isEmpty
						ifTrue: [ status aLoadedPackageIsNotCurrent: true ]
						ifFalse: [ 
							exact := current := false.
							versionInfos
								do: [ :vi | 
									wcName := vi name.
									fileRef := GoferResolvedReference name: file.
									wcRef := GoferResolvedReference name: wcName.
									(wcRef compare: fileRef using: #=)
										ifTrue: [ exact := true ] ].
							exact
								ifTrue: [ status aLoadedPackageIsExact: true ]
								ifFalse: [ 
									versionInfos
										do: [ :vi | 
											wcName := vi name.
											fileRef := GoferResolvedReference name: file.
											wcRef := GoferResolvedReference name: wcName.
											(wcRef compare: fileRef using: #>=)
												ifTrue: [ current := true ] ].
									current
										ifTrue: [ status aLoadedPackageIsCurrent: true ]
										ifFalse: [ status aLoadedPackageIsNotCurrent: true ] ] ] ]
				notLoaded: [ status aPackageNotLoaded: true ]
				using: mcLoader ]
		groupDo: [ :ignoredGroup | 
			"if we encounter a group, trouble"
			status abort: true.
			^ status ].
	^ status
!

isPossibleBaseline

	self 
		projectDo: [:prj | prj isPossibleBaseline ifFalse: [ ^false ]] 
		packageDo: [:pkg | pkg isPackageLoaded ifFalse: [ ^false ]] 
		groupDo: [:ignored | ].
	^true
!

isSomethingLoaded
	"at least one package loaded"
	
	self isPartiallyCurrentAgainst isSomethingLoaded: [ :ignored | ^ true ].
	^ false
!

isSomethingLoadedAgainst: resolvedPackageAndProjectNames
	"at least one package loaded"

	(self isPartiallyCurrent: resolvedPackageAndProjectNames) isSomethingLoaded: [ :ignored | ^ true ].
	^ false
! !

!MetacelloMCVersionSpec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCVersionSpec.st,v 1.1 2011/08/20 18:16:35 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloMCVersionSpec.st,v 1.1 2011/08/20 18:16:35 cg Exp $'
! !
