"{ Package: 'stx:goodies/metacello' }"

MetacelloAbstractPackageSpec subclass:#MetacelloProjectReferenceSpec
	instanceVariableNames:'projectReference'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-Core-Specs'
!

MetacelloProjectReferenceSpec comment:''
!


!MetacelloProjectReferenceSpec methodsFor:'*metacello-mc'!

loadUsing: aLoader gofer: ignored

	| required |
	required := self resolveToLoadableSpec.
	required loader: aLoader.
	^required load
!

packagesNeedSavingVisited: visitedProjects using: repos into: aCollection
	
	| prjct clsName vrsn |
	prjct := self resolveToLoadableSpec.
	(visitedProjects includes: (clsName := prjct className)) ifTrue: [ ^self ].
	visitedProjects add: clsName.
	(vrsn := self versionOrNil) == nil ifTrue: [ ^self ].
	vrsn spec 
		packagesNeedSavingVisited: visitedProjects 
		into: aCollection
!

resolveProjectSpec

	^self projectReference
!

resolveToAllPackagesIn: aVersionSpec into: packages visited: visited

	(self resolveProjectSpec resolveToAllPackagesIn: aVersionSpec visited: visited) do: [:pkg |
		packages at: pkg put: pkg ]
!

resolveToLoadableSpec

	^self resolveProjectSpec resolveToLoadableSpec
!

resolveToPackagesIn: aVersionSpec visited: visited

	^#()
!

updateForSpawnMethod: sourceSpec
	"This means that this spec was used in a baseline and will be used in a version .... drop all information that isn't useful"

	| nm |
	nm := name.
	self projectReference updateForSpawnMethod: sourceSpec.
	super updateForSpawnMethod: sourceSpec.
	name := nm.
!

updatePackageSpec: updatedSpecs using: anMCLoader
	"Add project copy to updatedSpecs if the current version of the project 
	 is different from the receiver's version"
	
	self projectReference updatePackageSpec: updatedSpecs
!

versionString

	^self projectReference versionString
! !

!MetacelloProjectReferenceSpec methodsFor:'accessing'!

answers: aListOfPairs

	self shouldNotImplement
!

includes: aCollection

	self shouldNotImplement
!

projectName
	^self projectReference name
!

projectReference: anObject
	projectReference := anObject
!

referencedSpec

	^self projectReference
!

requires: aCollection

	self shouldNotImplement
! !

!MetacelloProjectReferenceSpec methodsFor:'copying'!

postCopy

	super postCopy.
	projectReference := projectReference copy.
! !

!MetacelloProjectReferenceSpec methodsFor:'merging'!

mergeMap

	| map |
	map := super mergeMap.
	map at: #projectReference put: projectReference.
	^map
!

mergeSpec: anotherSpec

	| newSpec map anotherProjectReference |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	(anotherProjectReference := map at: #projectReference) ~~ nil
		ifTrue: [ 
			newSpec projectReference: 
				(newSpec projectReference == nil
					ifTrue: [ anotherProjectReference ]
					ifFalse: [ newSpec projectReference mergeSpec: anotherProjectReference ])].
	^newSpec
!

nonOverridable

	^super nonOverridable, #( projectReference )
! !

!MetacelloProjectReferenceSpec methodsFor:'printing'!

configMethodCascadeOn: aStream member: aMember last: lastCascade indent: indent
	aMember methodUpdateSelector == #remove:
		ifTrue: [ aStream nextPutAll: 'removeProject: ' , self name printString ]
		ifFalse: [ 
			self projectReference == nil
				ifTrue: [ ^ self ].
			aStream nextPutAll: 'project: ' , self projectName printString , ' '.
			(aMember methodUpdateSelector == #copy: and: [ self projectReference hasNonVersionStringField ])
				ifTrue: [ aStream nextPutAll: 'copyFrom: ' , aMember sourceName printString , ' ' ].
			self projectReference configShortCutMethodOn: aStream member: aMember indent: indent + 1 ].
	lastCascade
		ifTrue: [ aStream nextPut: $. ]
		ifFalse: [ 
			aStream
				nextPut: $;;
				cr ]
!

configMethodOn: aStream indent: indent

	aStream 
		tab: indent; nextPutAll: 'spec '; cr;
		tab: indent + 1; nextPutAll: 'name: ', self name printString; nextPut: $;; cr;
		tab: indent + 1; nextPutAll: 'projectReference: '; nextPut: $[; cr.
	aStream 
		tab: indent + 2; nextPutAll: 'spec'.
	self projectReference configMethodBodyOn: aStream indent: indent + 2.
	aStream nextPutAll: ' ].'
! !

!MetacelloProjectReferenceSpec methodsFor:'querying'!

projectReference
	^ projectReference
!

relativeCurrentVersion

	^self projectReference relativeCurrentVersion
!

version

	^self projectReference version
!

versionOrNil

	^self projectReference versionOrNil
!

versionString: aString

	^self projectReference versionString: aString
! !

!MetacelloProjectReferenceSpec methodsFor:'visiting'!

projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	projectBlock value: self
! !

!MetacelloProjectReferenceSpec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloProjectReferenceSpec.st,v 1.1 2011/08/20 18:01:02 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloProjectReferenceSpec.st,v 1.1 2011/08/20 18:01:02 cg Exp $'
! !
