"{ Package: 'stx:goodies/metacello' }"

MetacelloPlatform subclass:#MetacelloSTXPlatform
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-Platform'
!

!MetacelloSTXPlatform class methodsFor:'documentation'!

documentation   
"
    for now, simply a copy of the pharo-platform code
"
! !

!MetacelloSTXPlatform methodsFor:'attributes'!

defaultPlatformAttributes
        | attributes versionString |
        attributes := OrderedCollection with: #squeakCommon with: #pharo.
        Smalltalk
                at: #SystemVersion
                ifPresent: [ :cl | 
                        versionString := cl current version.
                        (((versionString beginsWith: 'Pharo-1') or: [ versionString beginsWith: 'PharoCore1' ])
                                or: [ versionString beginsWith: 'Pharo1' ])
                                ifTrue: [ 
                                        attributes add: #'pharo1.x'.
                                        ((versionString beginsWith: 'Pharo-1.0') or: [ versionString beginsWith: 'PharoCore1.0' ])
                                                ifTrue: [ attributes add: #'pharo1.0.x' ]
                                                ifFalse: [ 
                                                        ((versionString beginsWith: 'Pharo-1.1') or: [ versionString beginsWith: 'Pharo1.1' ])
                                                                ifTrue: [ attributes add: #'pharo1.1.x' ]
                                                                ifFalse: [ 
                                                                        ((versionString beginsWith: 'Pharo-1.2') or: [ versionString beginsWith: 'Pharo1.2' ])
                                                                                ifTrue: [ attributes add: #'pharo1.2.x' ]
                                                                                ifFalse: [ 
                                                                                        (versionString beginsWith: 'Pharo1.3')
                                                                                                ifTrue: [ attributes add: #'pharo1.3.x' ] ] ] ] ] ].
        ^ attributes

    "Created: / 20-08-2011 / 22:34:43 / cg"
! !

!MetacelloSTXPlatform methodsFor:'initialize-release'!

initialize
        "implmented to force initialize on load"

        super initialize

    "Created: / 20-08-2011 / 22:36:16 / cg"
! !

!MetacelloSTXPlatform methodsFor:'notification'!

collection: aCollection do: aBlock displaying: aString

        self bypassProgressBars ifTrue: [ ^super collection: aCollection do: aBlock displaying: aString ].
        aCollection 
                do: aBlock
                displayingProgress: aString

    "Created: / 20-08-2011 / 22:34:54 / cg"
!

do: aBlock displaying: aString

        self bypassProgressBars ifTrue: [ ^super do: aBlock displaying: aString ].
        aString
                displayProgressAt: Sensor cursorPoint
                from: 0 to: 2
                during: [:bar |
                        bar value: 1.
                        aBlock value.
                        bar value: 2 ]

    "Created: / 20-08-2011 / 22:35:45 / cg"
! !

!MetacelloSTXPlatform methodsFor:'reflection'!

copyClass: oldClass as: newName inCategory: newCategoryName
        | copysName class newDefinition |
        copysName := newName asSymbol.
        copysName = oldClass name
                ifTrue: [ ^ oldClass ].
        (Smalltalk includesKey: copysName)
                ifTrue: [ ^ self error: copysName , ' already exists' ].
        newDefinition := oldClass definition copyReplaceAll: '#' , oldClass name asString with: '#' , copysName asString printString.
        newDefinition := newDefinition
                copyReplaceAll: 'category: ' , (SystemOrganization categoryOfElement: oldClass name) asString printString
                with: 'category: ' , newCategoryName printString.
        class := Compiler evaluate: newDefinition logged: true.
        class class instanceVariableNames: oldClass class instanceVariablesString.
        class copyAllCategoriesFrom: oldClass.
        class class copyAllCategoriesFrom: oldClass class.
        class category: newCategoryName.
        ^ class

    "Created: / 20-08-2011 / 22:35:36 / cg"
! !

!MetacelloSTXPlatform methodsFor:'repository creation'!

createRepository: aRepositorySpec

        | type |
        type := aRepositorySpec type.
        type = 'ftp'
                ifTrue: [ | description headerSize index host directory |
                        description := aRepositorySpec description.
                        headerSize := 'ftp://' size.
                        index := description indexOf: $/ startingAt: headerSize + 1.
                        host := description copyFrom: headerSize + 1 to: index - 1.
                        directory := description copyFrom: index + 1 to: description size.
                        ^ MCFtpRepository
                                host: host 
                                directory: directory 
                                user: aRepositorySpec username
                                password: aRepositorySpec password].
        ^ super createRepository: aRepositorySpec

    "Created: / 20-08-2011 / 22:35:55 / cg"
!

extractTypeFromDescription: description

        (description beginsWith: 'ftp://') ifTrue: [ ^'ftp' ].
        ^super extractTypeFromDescription: description

    "Created: / 20-08-2011 / 22:34:28 / cg"
! !

!MetacelloSTXPlatform methodsFor:'utilities'!

authorName

        ^Author fullName

    "Created: / 20-08-2011 / 22:35:00 / cg"
!

authorName: aString

        Author fullName: aString

    "Created: / 20-08-2011 / 22:36:03 / cg"
!

timestamp

        ^Date today mmddyyyy, ' ',
                ((String streamContents: [:s | Time now print24: true on: s]) copyFrom: 1 to: 5)

    "Created: / 20-08-2011 / 22:35:22 / cg"
! !

!MetacelloSTXPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloSTXPlatform.st,v 1.1 2011/08/20 20:37:56 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloSTXPlatform.st,v 1.1 2011/08/20 20:37:56 cg Exp $'
! !
