"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloSymbolicVersionTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-TestsMC'
!

MetacelloSymbolicVersionTest comment:''
!


!MetacelloSymbolicVersionTest methodsFor:'baselines'!

baseline20: spec 
	<version: '2.0-baseline'>
	
	spec for: #common do: [
		spec blessing: #baseline.
		spec 
			project: 'Symbolic' with: [
				spec
					className: 'MetacelloTestConfigurationOfSymbolic';
					repository: 'dictionary://Metacello_Configuration_Test_Repository' ];
			yourself ].
! !

!MetacelloSymbolicVersionTest methodsFor:'running'!

setUp

	| repo |
	super setUp.
	repo := self configurationRepository.
	self tempRepositories add: repo.
	gofer repository: repo
!

tearDown

	super tearDown.
	MetacelloConfigurationResource projectAttributes: nil
!

tearDownPackages: aGoferInstance

	super tearDownPackages: aGoferInstance.
	(self hasPackage: 'MetacelloTestConfigurationOfSymbolic')
		ifTrue: [ aGoferInstance package: 'MetacelloTestConfigurationOfSymbolic' ].
! !

!MetacelloSymbolicVersionTest methodsFor:'tests stable'!

testBasicStableVersion

	| project version wc |
	project := self projectWith: #(#'platformVersion1.x').
	version := project version: #stable.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.25').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.31'.
!

testBleedingEdgeMethod

	| project version |
	project := self projectWith: #(#'platformVersion1.x').
	self assert: (project version: #bleedingEdge) = project bleedingEdge.
!

testStableVersion

	| project version wc |
	project := self project.
	version := project version: '2.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.32'.
!

testStableVersion10x

	| project version wc |
	MetacelloConfigurationResource projectAttributes: #(#'platformVersion1.x' #'platformVersion1.0.x' ).
	project := self project.
	version := project version: '2.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.25').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.31'
!

testStableVersion11x

	| project version wc |
	MetacelloConfigurationResource projectAttributes: #(#'platformVersion1.x' #'platformVersion1.1.x' ).
	project := self project.
	version := project version: '2.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.53').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.32'
!

testStableVersion12x

	| project version wc |
	MetacelloConfigurationResource projectAttributes: #(#'platformVersion1.x' #'platformVersion1.2.x' ).
	project := self project.
	version := project version: '2.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.54').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.33'
!

testStableVersion1x

	| project version wc |
	MetacelloConfigurationResource projectAttributes: #(#'platformVersion1.x').
	project := self project.
	version := project version: '2.0'.
	version load.
	wc := (GoferPackageReference name: 'GoferBeau') workingCopy.
	self assert: (wc ancestry ancestors first name = 'GoferBeau-dkh.15').
	wc := (GoferPackageReference name: 'GoferFaux') workingCopy.
	self assert: wc ancestry ancestors first name = 'GoferFaux-tg.30'
!

testStableVersionMethod

	| project version |
	project := self projectWith: #(#'platformVersion1.x').
	self assert: (project version: #stable) = project stableVersion.
! !

!MetacelloSymbolicVersionTest methodsFor:'versions'!

bleedingEdge: spec 
	<symbolicVersion: #bleedingEdge>
	
	spec for: #'platformVersion1.x' version: '2.0-baseline'.
!

stableVersion: spec 
	<symbolicVersion: #stable>
	
	spec for: #'platformVersion1.x' version: '1.1'.
!

version10: spec 
	<version: '1.0'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec blessing: #release.
		spec 
			package: 'GoferBeau' with: 'GoferBeau-dkh.15';
			package: 'GoferFaux' with:'GoferFaux-tg.30';
			yourself ].
!

version11: spec 
	<version: '1.1'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec blessing: #release.
		spec 
			package: 'GoferBeau' with: 'GoferBeau-dkh.25';
			package: 'GoferFaux' with:'GoferFaux-tg.31';
			yourself ].
!

version12: spec 
	<version: '1.2'>
	
	spec for: #common do: [
		spec repository: 'dictionary://Metacello_Gofer_Test_Repository'.
		spec blessing: #release.
		spec 
			package: 'GoferBeau' with: 'GoferBeau-dkh.53';
			package: 'GoferFaux' with:'GoferFaux-tg.32';
			yourself ].
!

version20: spec 
	<version: '2.0' imports: #('2.0-baseline')>
	
	spec for: #common do: [ spec project: 'Symbolic' with:  #stable ].
! !

!MetacelloSymbolicVersionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloSymbolicVersionTest.st,v 1.1 2011/08/20 18:02:29 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloSymbolicVersionTest.st,v 1.1 2011/08/20 18:02:29 cg Exp $'
! !
