"{ Package: 'stx:goodies/metacello' }"

MetacelloDictionaryRepositoryTest subclass:#MetacelloToolBoxConstructorTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-TestsMC'
!

MetacelloToolBoxConstructorTestCase comment:''
!


!MetacelloToolBoxConstructorTestCase methodsFor:'private'!

assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
	| cl original modified constructor pragma |
	constructor := MetacelloToolBoxConstructor new.
	constructor configuration: (cl := Smalltalk at: configurationName asSymbol) new.
	pragma := (constructor extractAllVersionPragmas at: versionString) at: 1.
	original := (cl sourceCodeAt: pragma selector) asString.
	[ 
	(MetacelloToolBox configurationNamed: configurationName)
		modifyVersionMethodForVersion: versionString;
		commitMethod ] on: MetacelloCannotUpdateReleasedVersionError do: [ :ex | ex resume ].
	modified := (cl sourceCodeAt: pragma selector) asString.
	self assert: original = modified.
!

assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
	| original modified |
	original := ((Smalltalk at: configurationName asSymbol) project version: versionString) versionSpec printString.
	[ 
	(MetacelloToolBox configurationNamed: configurationName)
		modifyVersionMethodForVersion: versionString;
		commitMethod ] on: MetacelloCannotUpdateReleasedVersionError do: [ :ex | ex resume ].
	modified := ((Smalltalk at: configurationName asSymbol) project version: versionString) versionSpec printString.
	self assert: original = modified.
!

atomicConfigurationRepository

	^MetacelloAtomicConfigurationResource current monticelloRepository
!

configurationNameList
	^ #(  'MetacelloTestConfigurationOfIssue77B' 'MetacelloTestConfigurationOfIssue77C' 'MetacelloTestConfigurationOfIssue77D' 'MetacelloTestConfigurationOfAtomicFan' 'MetacelloTestConfigurationOfAtomicFoo' 'MetacelloTestConfigurationOfFan' 'MetacelloTestConfigurationOfFeaux' 'MetacelloTestConfigurationOfFix' 'MetacelloTestConfigurationOfFoo' 'MetacelloTestConfigurationOfFum' 'MetacelloTestConfigurationOfLinearFoo' 'MetacelloTestConfigurationOfProjectFee' 'MetacelloTestConfigurationOfProjectFie' 'MetacelloTestConfigurationOfProjectFoe' 'MetacelloTestConfigurationOfProjectInfinite' 'MetacelloTestConfigurationOfProjectIssue86' 'MetacelloTestConfigurationOfProjectIssue95' 'MetacelloTestConfigurationOfProjectLoop' 'MetacelloTestConfigurationOfUmbrellaA' 'MetacelloTestConfigurationOfUmbrellaB')
!

fullConfigurationNameList

	^self configurationNameList, #('MetacelloTestConfigurationOfProjectToolBox')
! !

!MetacelloToolBoxConstructorTestCase methodsFor:'running'!

setUp
	| repo |
	super setUp.
	repo := MetacelloAtomicConfigurationResource current monticelloRepository.
	self tempRepositories add: repo.
	Smalltalk at: #'Metacello_Atomic_Configuration_Test_Repository' put: repo.
!

tearDown
	| aGofer |
	super tearDown.
	Smalltalk removeKey: #'Metacello_Atomic_Configuration_Test_Repository' ifAbsent: [].
	MetacelloConfigurationResource projectAttributes: nil
!

tearDownPackages: aGoferInstance
	super tearDownPackages: aGoferInstance.
	self fullConfigurationNameList
		do: [ :configurationName | 
			(self hasPackage: configurationName)
				ifTrue: [ aGoferInstance package: configurationName ] ]
! !

!MetacelloToolBoxConstructorTestCase methodsFor:'tests'!

testConfigurationResources
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	self configurationNameList do: [ :configurationName | gofer package: configurationName ].
	gofer load.
	self configurationNameList
		do: [ :configurationName | 
			((Smalltalk at: configurationName asSymbol) new project versions collect: [ :each | each versionString ])
				do: [ :versionString | self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString ] ]
!

testProjectToolBox
	| configurationName rejects |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	rejects := #('1.0.4-baseline' '1.0.6-baseline' '1.0.7-baseline' '1.1.0-baseline' '1.1.4-baseline' '1.1.6-baseline' '1.2.8-baseline' '1.2.9-baseline' ).
	(((Smalltalk at: configurationName asSymbol) new project versions collect: [ :each | each versionString ])
		reject: [ :each | rejects includes: each ])
		do: [ :versionString | self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString ]
!

testProjectToolBox104Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.0.4-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString.
!

testProjectToolBox106Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.0.6-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox107Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.0.7-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox110Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.0-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox114Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.4-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox116Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.6-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox128Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.2.8-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBox129Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.2.9-baseline'.
	self assertVersionSpecFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison
	| configurationName rejects |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	rejects := #('1.0.6-baseline' '1.1.1-baseline' '1.1.2-baseline' '1.1.3' '1.3.0-baseline' '1.3.1-baseline' '1.3.2-baseline' '1.3.3-baseline').
	(((Smalltalk at: configurationName asSymbol) new project versions collect: [ :each | each versionString ])
		reject: [ :each | rejects includes: each ])
		do: [ :versionString | self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString ]
!

testProjectToolBoxSourceComparison106Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.0.6-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison111Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.1-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison112Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.2-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison113
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.1.3'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString.
!

testProjectToolBoxSourceComparison130Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.3.0-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison131Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.3.1-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison132Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.3.2-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxSourceComparison133Baseline
	| configurationName versionString |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.3.3-baseline'.
	self assertMethodSourceFor: configurationName isEqualAfterModifyingVersion: versionString
!

testProjectToolBoxTraversal131Baseline
	| configurationName versionString expected toolbox attributes count |
	gofer
		repository: self atomicConfigurationRepository;
		repository: self alternateRepository;
		repository: self configurationRepository.
	configurationName := 'MetacelloTestConfigurationOfProjectToolBox'.
	gofer package: configurationName.
	gofer load.
	versionString := '1.3.1-baseline'.
	attributes := #(#'level1_1' #'level2_2' #'level3_2' #'level2_1').
	MetacelloConfigurationResource projectAttributes: attributes.
	toolbox := MetacelloToolBox new project: (Smalltalk at: configurationName asSymbol) project.
	expected := Dictionary new.
	expected
		at: #(#'level1_1' #'level2_1')
		put:
			'spec package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/4''. ].'.
	expected
		at: #(#'level1_1' #'level2_2' #'level3_2')
		put:
			'spec package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/7''. ].'.
	expected
		at: #(#'level1_1' #'level2_2')
		put:
			'spec package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/6''. ].'.
	expected
		at: #'level1_1'
		put:
			'spec 
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/1''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/2''. ];
	package: ''Example-Core'' with: [
		spec repository: ''http://www.example.com/3''. ].'.
	count := 4.
	toolbox
		updateVersionMethodForVersion: versionString
		updateProjects: false
		updatePackages: false
		versionSpecsDo: [ :attributeOrPath :versionSpec | 
			attributeOrPath ~~ #common
				ifTrue: [ 
					| string |
					string := versionSpec printString.
					self assert: string = (expected at: attributeOrPath).
					attributeOrPath isArray
						ifTrue: [ self assert: (attributes indexOf: attributeOrPath last) = count ]
						ifFalse: [ self assert: (attributes indexOf: attributeOrPath) = count ].
					count := count - 1 ].
			false ]
! !

!MetacelloToolBoxConstructorTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloToolBoxConstructorTestCase.st,v 1.1 2011/08/20 18:00:12 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloToolBoxConstructorTestCase.st,v 1.1 2011/08/20 18:00:12 cg Exp $'
! !
