"{ Package: 'stx:goodies/metacello' }"

Magnitude subclass:#MetacelloVersion
	instanceVariableNames:'spec versionNumber importedVersions versionStatus'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-Core-Model'
!

MetacelloVersion comment:''
!


!MetacelloVersion class methodsFor:'instance creation'!

fromSpec: aMetacelloVersionSpec

	^(self fromString: aMetacelloVersionSpec versionString) 
		spec: aMetacelloVersionSpec;
		yourself
!

fromString: aString

	^self new versionNumber: aString asMetacelloVersionNumber
! !

!MetacelloVersion methodsFor:'accessing'!

importedVersions

	importedVersions == nil ifTrue: [ importedVersions := #() ].
	^importedVersions
!

importedVersions: aCollection

	importedVersions := aCollection
!

projectLabel

	^self spec projectLabel
!

spec

	^spec
!

spec: aMetacellVersionSpec

	spec := aMetacellVersionSpec
!

versionNumber: aVersionNumber

	versionNumber := aVersionNumber
!

versionSpec

	^self spec
!

versionStatus

	versionStatus == nil ifTrue: [ versionStatus := self computeVersionStatus].
	^versionStatus
!

versionStatus: aSymbol

	versionStatus := aSymbol
! !

!MetacelloVersion methodsFor:'actions'!

load

	^self subclassResponsibility
!

load: required

	^self subclassResponsibility
!

unload

	^self spec unload
! !

!MetacelloVersion methodsFor:'comparing'!

< aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber < aMetacelloVersion versionNumber
!

= aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber = aMetacelloVersion versionNumber
!

hash

	^self versionNumber hash
!

~> aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber ~> aMetacelloVersion versionNumber
! !

!MetacelloVersion methodsFor:'printing'!

printOn: aStream
	| label vs |
	self blessing ~~ #broken
		ifTrue: [ 
			(vs := self versionStatus) == #somethingLoaded
				ifTrue: [ aStream nextPutAll: '<>' ].
			vs == #loadedMatchConstraints
				ifTrue: [ aStream nextPutAll: '>=' ].
			vs == #loadedToSpec
				ifTrue: [ aStream nextPut: $~ ] ].
	self versionNumber printOn: aStream.
	self spec ~~ nil
		ifTrue: [ 
			(label := self spec projectLabel) isEmpty
				ifFalse: [ aStream nextPutAll: ' [' , label , ']' ] ]
! !

!MetacelloVersion methodsFor:'private'!

computeVersionStatus

	"all projects and packages are loaded and match specification"
	self isAllLoadedToSpec ifTrue: [ ^#allLoadedToSpec ].
	"all loaded projects and packages match specifications (at least one package loaded)"
	self isLoadedToSpec ifTrue: [ ^#loadedToSpec ].
	"all loaded projects and packages match constraints (at least one package loaded)"
	self isLoadedMatchConstraints ifTrue: [ ^#loadedMatchConstraints ].
	"at least one package loaded"
	self isSomethingLoaded ifTrue: [ ^#somethingLoaded ].
	^#noStatus
! !

!MetacelloVersion methodsFor:'querying'!

author

	^self spec author value
!

blessing

	^self spec blessing value
!

description

	^self spec description value
!

loader

	^self spec loader
!

project

	^self spec project
!

timestamp

	^self spec timestamp value
!

versionKey
	"version printString, without the versionStatus decorations, suitable for using as a unique key for the 
	 receiver in a dictionary"

	^ String streamContents: [:aStream| |label|
		self versionNumber printOn: aStream.
		self spec ~~ nil
			ifTrue: [(label := self spec projectLabel) isEmpty
					ifFalse: [aStream nextPutAll: ' [' , label , ']']] ]
!

versionNumber

	^versionNumber
!

versionString

	^self versionNumber versionString 
! !

!MetacelloVersion methodsFor:'testing'!

isAllLoadedToSpec
	"all projects and packages are loaded and match specification"
	
	^self spec isAllLoadedToSpec
!

isLoadedMatchConstraints
	"all loaded projects and packages match constraints"
	
	^self spec isLoadedMatchConstraints
!

isLoadedToSpec
	"all loaded projects and packages match specifications"
	
	^self spec isLoadedToSpec
!

isPossibleBaseline

	^self spec isPossibleBaseline
!

isSomethingLoaded
	"at least one project or package has been loaded"
	
	^self spec isSomethingLoaded
!

name

	^self versionString
! !

!MetacelloVersion class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersion.st,v 1.1 2011/08/20 18:05:32 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersion.st,v 1.1 2011/08/20 18:05:32 cg Exp $'
! !
