"{ Package: 'stx:goodies/metacello' }"

MetacelloDirective subclass:#MetacelloVersionLoadDirective
	instanceVariableNames:'loadDirectives'
	classVariableNames:''
	poolDictionaries:''
	category:'Metacello-MC-Directives'
!

MetacelloVersionLoadDirective comment:''
!


!MetacelloVersionLoadDirective methodsFor:'accessing'!

loadDirectives

	loadDirectives == nil ifTrue: [ loadDirectives := OrderedCollection new ].
	^ loadDirectives
!

loadDirectives: anObject
	loadDirectives := anObject
!

spec
	"Expected to be a MetacelloVersionSpec"
	
	(spec == nil  and: [  self loader ~~ nil ]) 
		ifTrue: [ 
			^[ self loader spec versionSpec ] 
				on: MessageNotUnderstood
				do: [:ex | ex return: self loader spec ]].
	^spec
! !

!MetacelloVersionLoadDirective methodsFor:'actions'!

add: aDirective

	self loadDirectives add: aDirective
!

finalizeLoad: aGofer
	"nothing special for linear loads"
!

loadAtomicLoadDirective: aLoaderDirective gofer: aGofer

	aLoaderDirective loadDirectives do: [:directive | directive loadUsing: aLoaderDirective gofer: aGofer ].
	aLoaderDirective finalizeLoad: aGofer.
!

loadExplicitLoadDirective: aLoaderDirective gofer: aGofer
	"load has already been performed, no need to load again"
!

loadLinearLoadDirective: aLoaderDirective gofer: aGofer

	aLoaderDirective loadDirectives do: [:directive | directive loadUsing: aLoaderDirective gofer: aGofer ].
	aLoaderDirective finalizeLoad: aGofer.
!

loadWithPolicy: aLoadPolicy

	| gofer |
	gofer := MetacelloGofer new.
	gofer disablePackageCache.
	gofer repository: aLoadPolicy cacheRepository.
	self loadUsing: self gofer: gofer
! !

!MetacelloVersionLoadDirective methodsFor:'enumerating'!

directivesDo: aBlock

	aBlock value: self.
	self loadDirectives do: [:directive | directive directivesDo: aBlock ].
!

packageDirectivesDo: aBlock

	self loadDirectives do: [:directive | directive packageDirectivesDo: aBlock ].
!

packagesDo: aBlock

	self loadDirectives do: [:directive | directive packageDo: aBlock ].
!

prepostLoadDirectivesDo: aBlock

	self loadDirectives do: [:directive | directive prepostLoadDirectivesDo: aBlock ].
!

prepostLoadsDo: aBlock

	self loadDirectives do: [:directive | directive prepostLoadDo: aBlock ].
!

versionDirectivesDepthFirstDo: aBlock

	self loadDirectives do: [:directive | directive versionDirectivesDepthFirstDo: aBlock ].
	aBlock value: self.
!

versionDirectivesDo: aBlock

	aBlock value: self.
	self loadDirectives do: [:directive | directive versionDirectivesDo: aBlock ].
!

versionDo: aBlock

	aBlock value: self.
!

versionsDo: aBlock

	self loadDirectives do: [:directive | directive versionDo: aBlock ].
! !

!MetacelloVersionLoadDirective methodsFor:'loading'!

loadPackageDirective: aPackageLoadDirective gofer: aGofer

	aPackageLoadDirective loader loadingSpecLoader 
		loadPackageDirective: aPackageLoadDirective 
		gofer: aGofer
!

loadPostloadDirective: aPostloadDirective 

	| block |
	(block :=  aPostloadDirective spec postLoadDoItBlock) ~~ nil
		ifTrue: [
			aPostloadDirective evaluateSupplyingAnswers: [ block valueWithPossibleArgs: 
					(Array 
						with: aPostloadDirective loader 
						with: aPostloadDirective spec) ].
			Transcript cr; show: 'Evaluated -> ', aPostloadDirective spec label, ' >> ', aPostloadDirective spec postLoadDoIt value asString]
!

loadPreloadDirective: aPreloadDirective 

	| block |
	(block :=  aPreloadDirective spec preLoadDoItBlock) ~~ nil
		ifTrue: [
			aPreloadDirective evaluateSupplyingAnswers: [ block valueWithPossibleArgs: 
					(Array 
						with: aPreloadDirective loader 
						with: aPreloadDirective spec) ].
			Transcript cr; show: 'Evaluated -> ', aPreloadDirective spec label, ' >> ', aPreloadDirective spec preLoadDoIt value asString]
! !

!MetacelloVersionLoadDirective methodsFor:'printing'!

label

	self spec == nil ifTrue: [ ^'' ].
	^self spec label
!

printLoadDirectivesOn: aStream indent: indent

	self loadDirectives do: [:each |
		aStream cr. 
		each printOn: aStream indent: indent + 1 ].
!

printOn: aStream indent: indent

	super printOn: aStream indent: indent.
	self printLoadDirectivesOn: aStream indent: indent
! !

!MetacelloVersionLoadDirective methodsFor:'testing'!

isExplicit

	^false
! !

!MetacelloVersionLoadDirective class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersionLoadDirective.st,v 1.1 2011/08/20 18:15:44 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersionLoadDirective.st,v 1.1 2011/08/20 18:15:44 cg Exp $'
! !
