"{ Package: 'stx:goodies/metacello' }"

TestCase subclass:#MetacelloVersionQueryTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* obsolete *'
!

MetacelloVersionQueryTestCase comment:''
!


!MetacelloVersionQueryTestCase methodsFor:'accessing'!

exampleProject

	^MetacelloVersionQueryExampleConfig new project
!

masterProject

	^MetacelloVersionQueryMasterConfig new project
!

projectProject

	^MetacelloVersionQueryProjectConfig new project
! !

!MetacelloVersionQueryTestCase methodsFor:'test latest filter'!

testLatestFilter
	| project |
	project := MetacelloVersionQueryLatestVersionConfig new project.

	self assert: (project latestVersionMatching: '*.*') versionString = '1.2'.
	self assert: (project latestVersionMatching: '*.*.*') versionString = '1.1.2'.
	self assert: (project 
		latestVersionMatching: '*.*'
		includedBlessings: #(#development)) versionString = '1.3'.
	self assert: (project 
		latestVersionMatching: '*.*.*'
		includedBlessings: #(#development)) versionString = '1.3.2'.
	self assert: (project 
		latestVersionMatching: '*.*'
		excludedBlessings: #()) versionString = '1.3'.
	self assert: (project 
		latestVersionMatching: '*.?'
		includedBlessings: #(#development)) versionString = '1.3.2'.
	self assert: (project 
		latestVersionMatching: '*.?'
		excludedBlessings: #()) versionString = '1.3.2'.
	self assert: (project latestVersionMatching: '*') = nil
! !

!MetacelloVersionQueryTestCase methodsFor:'test resolveToLoadableSpecs'!

testBaseline10ResolveToLoadableSpecs01
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.

	expected := #('Example-Core' 'UTF8').
	list := version resolveToLoadableSpecs: #('Example-Core').
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].

	expected := #('Example-AddOn' 'Example-Core' 'UTF8').
	list := version resolveToLoadableSpecs: #('Example-AddOn').
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].

	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'UTF8').
	list := version resolveToLoadableSpecs: #('No UI').
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30ResolveToLoadableSpecs01
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.

	expected := #('Master-Core' 'Master-Platform' 'Project Core' 'XML').
	list := version resolveToLoadableSpecs: #('Core').
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].


! !

!MetacelloVersionQueryTestCase methodsFor:'tests'!

testBaseline10Groups
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Core' 'Tests' 'No UI' 'UI').
	list := version groups.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10Load
	| loadedPackages project loader |
	project := self exampleProject.
	loader := (project version: '1.0-baseline') load: 'UI'.
	loadedPackages := loader loadedPackages asArray.
	self assert: (loadedPackages indexOf: 'Example-Core')
			> 0.
	self assert: (loadedPackages indexOf: 'Example-AddOn')
			> (loadedPackages indexOf: 'Example-Core').
	self assert: (loadedPackages indexOf: 'Example-Tests')
			> (loadedPackages indexOf: 'Example-AddOn').
	self assert: (loadedPackages indexOf: 'Example-UI')
			> (loadedPackages indexOf: 'Example-AddOn').
	self assert: (loadedPackages indexOf: 'Example-TestsUI')
			> (loadedPackages indexOf: 'Example-UI').
	self assert: (loadedPackages indexOf: 'Example-TestsUI')
			> (loadedPackages indexOf: 'Example-Tests').
!

testBaseline10Packages
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-UI' 'Example-TestsUI').
	list := version packages.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed01
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core').
	list := version packagesForSpecNamed: 'Example-Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed02
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #().
	list := version packagesForSpecNamed: 'None existent'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed03
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-UI' 'Example-TestsUI').
	list := version packagesForSpecNamed: 'ALL'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed04
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' ).
	list := version packagesForSpecNamed: 'Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed05
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' ).
	list := version packagesForSpecNamed: 'No UI'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed06
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-TestsUI').
	list := version packagesForSpecNamed: 'UI'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed07
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-UI' 'Example-TestsUI').
	list := version packagesForSpecNamed: 'default'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10PackagesForSpecNamed08
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-TestsUI' 'Example-UI' 'Misc-UTF8' ).
	list := version allPackagesForSpecNamed: { 'No UI'. 'UI'. 'Tests'. }.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline10Projects
	| project version expected list |
	project := self exampleProject.
	version := project version: '1.0-baseline'.
	expected := #('UTF8').
	list := version projects.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20Groups
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Core' 'Tests').
	list := version groups.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20Load
	| project version |
	project := self exampleProject.
	version := project currentVersionAgainst: #('Core') "in 2.1 'Core' is a package and in 2.0 'Core' is a group ... no errors"
!

testBaseline20Packages
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Project-Core' 'Project-Tests').
	list := version packages.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20PackagesForSpecNamed01
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Project-Core').
	list := version packagesForSpecNamed: 'Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20PackagesForSpecNamed02
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Project-Core' 'Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-UI' 'Example-TestsUI' 'Misc-UTF8').
	list := version allPackagesForSpecNamed: 'Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20PackagesForSpecNamed03
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Project-Core' 'Project-Tests').
	list := version packagesForSpecNamed: 'ALL'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20PackagesForSpecNamed04
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Project-Core' 'Project-Tests' 'Example-Core' 'Example-AddOn' 'Example-Tests' 'Example-UI' 'Example-TestsUI' 'Misc-UTF8').
	list := version allPackagesForSpecNamed: 'ALL'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline20Projects
	| project version expected list |
	project := self projectProject.
	version := project version: '2.0-baseline'.
	expected := #('Example ALL').
	list := version projects.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline21PackagesForSpecNamed01
	| project version expected list |
	project := self projectProject.
	version := project version: '2.1-baseline'.
	expected := #('Project-Core' 'Example-Core' 'Example-AddOn' 'Misc-UTF8' 'Misc-MD5').
	list := version allPackagesForSpecNamed: 'Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline21PackagesForSpecNamed02
	| project version expected list |
	project := self projectProject.
	version := project version: '2.1-baseline'.
	expected := #('Project-Core' 'Example-Core' 'Example-AddOn' 'Project-Tests' 'Example-Tests' 'Misc-UTF8' 'Misc-MD5').
	list := version allPackagesForSpecNamed: 'Tests'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30Groups
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Core' 'Tests').
	list := version groups.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30Packages
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Master-Core' 'Master-Tests' 'Master-Platform' ).
	list := version packages.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30PackagesForSpecNamed01
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Master-Core' 'Master-Platform' 'Project-Core' 'Example-Core' 'Example-AddOn' 'Misc-UTF8' 'Misc-MD5' 'Misc-XML').
	list := version allPackagesForSpecNamed: 'Core'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30PackagesForSpecNamed02
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Master-Core' 'Master-Platform' 'Project-Core' 'Example-Core' 'Example-AddOn' 'Project-Tests' 'Example-Tests' 'Master-Tests' 'Misc-UTF8' 'Misc-MD5' 'Misc-XML').
	list := version allPackagesForSpecNamed: 'Tests'.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30PackagesForSpecNamed03
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Master-Core' 'Master-Platform' 'Project-Core' 'Example-Core' 'Example-AddOn' 'Project-Tests' 'Example-Tests' 'Master-Tests' 'Misc-UTF8' 'Misc-MD5' 'Misc-XML').
	list := version allPackagesForSpecNamed: { 'Tests'. }.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline30Projects
	| project version expected list |
	project := self masterProject.
	version := project version: '3.0-baseline'.
	expected := #('Project Core' 'Project Tests' 'XML').
	list := version projects.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
!

testBaseline31PackagesForSpecNamed03
	| project version expected list |
	project := self masterProject.
	version := project version: '3.1-baseline'.
	expected := #('Master-Core' 'Master-Platform' 'Project-Core' 'Example-Core' 'Example-AddOn' 'Project-Tests' 'Example-Tests' 'Master-Tests' 'Misc-UTF8' 'Misc-MD5' 'Misc-XML').
	list := version allPackagesForSpecNamed: { 'Tests'. }.
	self assert: (list size = expected size).
	list do: [:pkg |
		self assert: (expected includes: pkg name) ].
! !

!MetacelloVersionQueryTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersionQueryTestCase.st,v 1.1 2011/08/20 18:14:07 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/MetacelloVersionQueryTestCase.st,v 1.1 2011/08/20 18:14:07 cg Exp $'
! !
