"{ Package: 'stx:goodies/metacello' }"!

!Block methodsFor:'*metacello-core'!

setAuthorInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setAuthorWithBlock: self
! !

!Block methodsFor:'*metacello-core'!

setBlessingInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setBlessingWithBlock: self
! !

!Block methodsFor:'*metacello-core'!

setDescriptionInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setDescriptionWithBlock: self
! !

!Block methodsFor:'*metacello-core'!

setPackage: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setPackage: aString withBlock: self
! !

!Block methodsFor:'*metacello-core'!

setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withBlock: self
! !

!Block methodsFor:'*metacello-core'!

setTimestampInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setTimestampWithBlock: self
! !

!Block methodsFor:'*metacello-mc'!

valueSupplyingMetacelloAnswers: aListOfPairs
	"evaluate the block using a list of questions / answers that might be called upon to
	automatically respond to Object>>confirm: or FillInTheBlank requests"

	^ [self value] 
		on: ProvideAnswerNotification
		do: 
			[:notify | | answer caption |
			
			caption := notify messageText withSeparatorsCompacted. "to remove new lines"
			answer := aListOfPairs
				detect: 
					[:each | caption = each first or:
						[(caption includesSubstring: each first caseSensitive: false) or:
						[(each first match: caption) or:
						[(String includesSelector: #matchesRegex:) and: 
						[ [ caption matchesRegex: each first ] on: Error do: [:ignored | false ]]]]]]
					ifNone: [nil].
			answer
				ifNotNil: [notify resume: answer second]
				ifNil: 
					[ | outerAnswer |
					outerAnswer := ProvideAnswerNotification signal: notify messageText.
					outerAnswer 
						ifNil: [notify resume] 
						ifNotNil: [notify resume: outerAnswer]]]
! !

!Collection methodsFor:'*metacello-core'!

addToMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each addToMetacelloPackages: aMetacelloPackagesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

addToMetacelloRepositories: aMetacelloRepositoriesSpec

	self do: [:each | each addToMetacelloRepositories: aMetacelloRepositoriesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

fetchRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doFetchRequiredFromArray: self.
! !

!Collection methodsFor:'*metacello-mc'!

loadRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doLoadRequiredFromArray: self.
! !

!Collection methodsFor:'*metacello-core'!

mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each mergeIntoMetacelloPackages: aMetacelloPackagesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

mergeIntoMetacelloRepositories: aMetacelloRepositoriesSpec

	self do: [:each | each mergeIntoMetacelloRepositories: aMetacelloRepositoriesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

recordRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doRecordRequiredFromArray: self.
! !

!Collection methodsFor:'*metacello-core'!

removeFromMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each removeFromMetacelloPackages: aMetacelloPackagesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

removeFromMetacelloRepositories: aMetacelloRepositoriesSpec

	self do: [:each | each removeFromMetacelloRepositories: aMetacelloRepositoriesSpec ]
! !

!Collection methodsFor:'*metacello-mc'!

resolvePackageSpecsNamedForMetacelloMCVersion: aMetacelloMCVersion visited: visited ifAbsent: aBlock 

	^ aMetacelloMCVersion
		allPackagesForSpecs: (self collect: [:ea | 
			aMetacelloMCVersion spec packageNamed: ea ifAbsent: aBlock])
		visited: visited
! !

!Collection methodsFor:'*metacello-core'!

setIncludesInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setIncludes: self asArray.
! !

!Collection methodsFor:'*metacello-core'!

setLoadsInMetacelloProject: aMetacelloPackageSpec

	aMetacelloPackageSpec setLoads: self asArray.
! !

!Collection methodsFor:'*metacello-core'!

setRequiresInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setRequires: self asArray.
! !

!Gofer methodsFor:'*metacello-testsmc'!

metacelloCleanup
	"Cleans the specified packages."
	
	^ self execute: MetacelloTestsGoferCleanup
! !

!Gofer methodsFor:'*metacello-testsmc'!

metacelloLoad
	"Load the specified packages into the image."
	
	^ self execute: MetacelloGoferLoad
! !

!Gofer methodsFor:'*metacello-testsmc'!

metacelloUnload
	"Unload the specified packages."
	
	^ self execute: MetacelloTestsGoferUnload
! !

!GoferResolvedReference methodsFor:'*metacello-mc'!

asMetacelloCachingResolvedReference

	^MetacelloCachingGoferResolvedReference name: self name repository: self repository
! !

!GoferResolvedReference methodsFor:'*metacello-mc'!

compare: aLoadableReference using: aComparisonOperator
	"Compare versions using <aComparisonOperator>. package names #= then compare based upon:

		1. branch name, list versions without branch last
		2. version number
		3. author name

	 If packageNames are not equal then the comparison returns false.
		"
		
	self packageName = aLoadableReference packageName
		ifFalse: [ ^false ].
	self branch = aLoadableReference branch ifFalse: [ 
		^ (self branch isEmpty or: [ aLoadableReference branch isEmpty ])
			ifTrue: [ aLoadableReference branch size perform: aComparisonOperator with: self branch size ]
			ifFalse: [ self branch perform: aComparisonOperator with: aLoadableReference branch ] ].
	self versionNumber = aLoadableReference versionNumber
		ifFalse: [ ^ self versionNumber perform: aComparisonOperator with: aLoadableReference versionNumber ].
	^ self author perform: aComparisonOperator with: aLoadableReference author
! !

!GoferVersionReference methodsFor:'*metacello-mc'!

metacelloPackageNameWithBranch
	"answer array with package name and package name with branch name"
	self branch isEmpty
		ifTrue: [ 
			^ {(self packageName).
			(self packageName)} ].
	^ {(self packageName).
	(self packageName , '.' , self branch)}
! !

!Integer methodsFor:'*metacello-core'!

metacelloIntegerLessThanSelf: anInteger

	^anInteger < self
! !

!Integer methodsFor:'*metacello-core'!

metacelloStringLessThanSelf: aString
	"string version components are always '<' integer component"
	
	^true
! !

!Integer methodsFor:'*metacello-core'!

metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^aMetacelloVersonComponent metacelloIntegerLessThanSelf: self
! !

!MCDictionaryRepository methodsFor:'*metacello-mc'!

asRepositorySpecFor: aMetacelloMCProject

	^(aMetacelloMCProject repositorySpec)
		description:  'dictionary://Metacello_Dictionary';
	 	type: 'dictionary';
		yourself
! !

!MCDictionaryRepository methodsFor:'*metacello-mc'!

versionInfoFromVersionNamed: aString

	| versions |
	versions := self dictionary values select: [:version | version info name beginsWith: aString ].
	versions isEmpty ifTrue: [ ^ nil ].
	versions := versions asSortedCollection: [ :a :b |
		([ (a info name copyAfterLast: $.) asNumber ] on: Error do: [:ex | ex return: 0 ]) <= 
			([ (b info name copyAfterLast: $.) asNumber ] on: Error do: [:ex | ex return: 0 ]) ].
	^ versions last info
! !

!MCDirectoryRepository methodsFor:'*metacello-mc'!

asRepositorySpecFor: aMetacelloMCProject

	^(aMetacelloMCProject repositorySpec)
		description:  directory fullName;
	 	type: 'directory';
		yourself
! !

!MCFileBasedRepository methodsFor:'*metacello-mc'!

versionInfoFromVersionNamed: aString

	| versions |
	versions := self allVersionNames
		select: [ :each | each beginsWith: aString ].
	versions isEmpty ifTrue: [ ^ nil ].
	versions := versions asSortedCollection: [ :a :b |
		([ (a copyAfterLast: $.) asNumber ] on: Error do: [:ex | ex return: 0 ]) <= 
			([ (b copyAfterLast: $.) asNumber ] on: Error do: [:ex | ex return: 0 ]) ].
	^ self versionInfoFromFileNamed: versions last , '.mcz'
! !

!MCHttpRepository methodsFor:'*metacello-mc'!

asRepositorySpecFor: aMetacelloMCProject

	^(aMetacelloMCProject repositorySpec)
		description:  self description;
	 	type: 'http';
		yourself
! !

!MCRepository methodsFor:'*metacello-mc'!

asRepositorySpecFor: aMetacelloMCProject

	self subclassResponsibility
! !

!MCWorkingCopy methodsFor:'*metacello-mc'!

metacelloPackageNameWithBranch
	"answer array with package name and package name with branch name"
	ancestry ancestors isEmpty
		ifTrue: [ 
			^ {(package name).
			(package name)} ].
	^ (GoferVersionReference name: self ancestors first name) metacelloPackageNameWithBranch
! !

!Object methodsFor:'*metacello-core'!

metacelloIntegerLessThanSelf: anInteger

	^self error: 'Invalid Metacello verson component - should be String or Integer.'
! !

!Object methodsFor:'*metacello-core'!

metacelloStringLessThanSelf: anInteger

	^self error: 'Invalid Metacello verson component - should be String or Integer.'
! !

!Object methodsFor:'*metacello-core'!

metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^self error: 'Invalid Metacello verson component - should be String or Integer.'
! !

!Object class methodsFor:'*metacello-mc'!

lastMetacelloVersionLoad
	"Answer the last version loaded and the list of packages loaded for that version.
	 See MetacelloConfigTemplate."

	^nil -> 'default'
! !

!Object class methodsFor:'*metacello-mc'!

metacelloVersion: versionString loads: anArrayOrString
	"Stash the last version loaded and the list of packages loaded for that version. The list
	 of packages will be used by the tools when doing 'Load Package Version'.
	See MetacelloConfigTemplate for example"
	
	"noop by default"
! !

!String methodsFor:'*metacello-core'!

addToMetacelloPackages: aMetacelloPackagesSpec

	| spec |
	spec := 
		(aMetacelloPackagesSpec project packageSpec)
			file: self;
			yourself.
	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec addMember 
			name: spec name;
			spec: spec;
			yourself)
! !

!String methodsFor:'*metacello-mc'!

addToMetacelloRepositories: aMetacelloRepositoriesSpec

	| spec |
	spec := 
		(aMetacelloRepositoriesSpec project repositorySpec)
			description: self;
			yourself.
	aMetacelloRepositoriesSpec addMember: 
		(aMetacelloRepositoriesSpec addMember 
			name: spec name;
			spec: spec;
			yourself)
! !

!String methodsFor:'*metacello-core'!

asMetacelloVersionNumber

	^MetacelloVersionNumber fromString: self
! !

!String methodsFor:'*metacello-mc'!

fetchRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doFetchRequiredFromArray: (Array with: self).
! !

!String methodsFor:'*metacello-mc'!

loadRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doLoadRequiredFromArray: (Array with: self).
! !

!String methodsFor:'*metacello-core'!

mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	| spec |
	spec := 
		(aMetacelloPackagesSpec project packageSpec)
			file: self;
			yourself.
	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec mergeMember 
			name: spec name;
			spec: spec;
			yourself)
! !

!String methodsFor:'*metacello-mc'!

mergeIntoMetacelloRepositories: aMetacelloRepositoriesSpec

	| spec |
	spec := 
		(aMetacelloRepositoriesSpec project repositorySpec)
			description: self;
			yourself.
	aMetacelloRepositoriesSpec addMember: 
		(aMetacelloRepositoriesSpec mergeMember 
			name: spec name;
			spec: spec;
			yourself)
! !

!String methodsFor:'*metacello-core-version comparison'!

metacelloIntegerLessThanSelf: anInteger
	"integer version components are always '>' string component"

	^false
! !

!String methodsFor:'*metacello-core-version comparison'!

metacelloStringLessThanSelf: aString

	^aString < self
! !

!String methodsFor:'*metacello-core-version comparison'!

metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^aMetacelloVersonComponent metacelloStringLessThanSelf: self
! !

!String methodsFor:'*metacello-mc'!

packageFileSpecFor: aMetacelloPackagesSpec

	^(aMetacelloPackagesSpec project packageSpec)
			file: self;
			yourself
! !

!String methodsFor:'*metacello-mc'!

recordRequiredForMetacelloMCVersion: aMetacelloMCVersion

	^aMetacelloMCVersion doRecordRequiredFromArray: (Array with: self).
! !

!String methodsFor:'*metacello-core'!

removeFromMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec removeMember 
			name: self;
			yourself)
! !

!String methodsFor:'*metacello-mc'!

removeFromMetacelloRepositories: aMetacelloRepositoriesSpec

	aMetacelloRepositoriesSpec addMember: 
		(aMetacelloRepositoriesSpec removeMember 
			name: self;
			yourself)
! !

!String methodsFor:'*metacello-mc'!

resolvePackageSpecsNamedForMetacelloMCVersion: aMetacelloMCVersion visited: visited ifAbsent: aBlock 

	^ aMetacelloMCVersion 
		allPackagesForSpecs:  {aMetacelloMCVersion spec packageNamed: self ifAbsent: aBlock}
		visited: visited
! !

!String methodsFor:'*metacello-core'!

setAuthorInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setAuthorWithString: self
! !

!String methodsFor:'*metacello-core'!

setAuthorInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setAuthor: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)
! !

!String methodsFor:'*metacello-core'!

setBlessingInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setBlessingWithString: self
! !

!String methodsFor:'*metacello-core'!

setBlessingInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setBlessing: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)
! !

!String methodsFor:'*metacello-core'!

setDescriptionInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setDescriptionWithString: self
! !

!String methodsFor:'*metacello-core'!

setDescriptionInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setDescription: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)
! !

!String methodsFor:'*metacello-core'!

setIncludesInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setIncludes: { self }.
! !

!String methodsFor:'*metacello-core'!

setLoadsInMetacelloProject: aMetacelloPackageSpec

	aMetacelloPackageSpec setLoads: { self }.
! !

!String methodsFor:'*metacello-core'!

setPackage: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setPackage: aString withString: self
! !

!String methodsFor:'*metacello-core'!

setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withString: self
! !

!String methodsFor:'*metacello-core'!

setRequiresInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setRequires: { self }.
! !

!String methodsFor:'*metacello-core'!

setTimestampInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setTimestampWithString: self
! !

!String methodsFor:'*metacello-core'!

setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)
! !

!Symbol methodsFor:'*metacello-core'!

setPostLoadDoItInMetacelloSpec: aMetacelloSpec

	aMetacelloSpec setPostLoadDoIt: 
		(aMetacelloSpec project valueHolderSpec
			value: self;
			yourself)
! !

!Symbol methodsFor:'*metacello-core'!

setPreLoadDoItInMetacelloSpec: aMetacelloSpec

	aMetacelloSpec setPreLoadDoIt: 
		(aMetacelloSpec project valueHolderSpec
			value: self;
			yourself)
! !

!Timestamp methodsFor:'*metacello-core'!

setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self printString;
			yourself)
! !

!UndefinedObject methodsFor:'*metacello-core'!

asMetacelloVersionNumber

	^MetacelloVersionNumber fromString: ''
! !

!UndefinedObject methodsFor:'*metacello-core'!

setAuthorInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setAuthor: self
! !

!UndefinedObject methodsFor:'*metacello-core'!

setBlessingInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setBlessing: self
! !

!UndefinedObject methodsFor:'*metacello-core'!

setDescriptionInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setDescription: self
! !

!UndefinedObject methodsFor:'*metacello-core'!

setPostLoadDoItInMetacelloSpec: aMetacelloSpec

	aMetacelloSpec setPostLoadDoIt: 
		(aMetacelloSpec project valueHolderSpec
			value: self;
			yourself)
! !

!UndefinedObject methodsFor:'*metacello-core'!

setPreLoadDoItInMetacelloSpec: aMetacelloSpec

	aMetacelloSpec setPreLoadDoIt: 
		(aMetacelloSpec project valueHolderSpec
			value: self;
			yourself)
! !

!UndefinedObject methodsFor:'*metacello-core'!

setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withString: self
! !

!UndefinedObject methodsFor:'*metacello-core'!

setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: self
! !

!stx_goodies_metacello class methodsFor:'documentation'!

extensionsVersion_CVS
    ^ '$Header: /cvs/stx/stx/goodies/metacello/extensions.st,v 1.3 2011/08/20 17:55:29 cg Exp $'
! !