"{ Package: 'stx:goodies/moreCollections/btree' }"

Collection subclass:#BTree
	instanceVariableNames:'root'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-BTree'
!


!BTree class methodsFor:'as yet unclassified'!

keys: aBTreeKeys
	^ self basicNew initializeWithKeys: aBTreeKeys
!

new
	^ self order: 5
!

order: aNumber
	^ self keys: (BTreeKeysArray new: aNumber)
! !

!BTree methodsFor:'accessing'!

at: aMagnitude
	^ self at: aMagnitude ifAbsent: [self error: aMagnitude printString, ' not found']
!

at: aMagnitude ifAbsent: errorBlock
       | leaf |
       leaf _ root existingLeafForKey: aMagnitude.
       leaf ifNil: [^ errorBlock value].
       ^ leaf valueForKey: aMagnitude ifAbsent: errorBlock
!

depth
	^ root depth
!

keys
	^ Array streamContents:
		[:s |
		self keysDo: [:k | s nextPut: k]]
!

order
	^ root size
!

values
	^ Array streamContents:
		[:s |
		self keysAndValuesDo: [:k :v | s nextPut: v]]
! !

!BTree methodsFor:'adding'!

at: aMagnitude put: anObject
	| leaf |
	leaf _ root leafForKey: aMagnitude.
	leaf insertKey: aMagnitude value: anObject.
	root _ leaf root.
	^ anObject
!

removeKey: aMagnitude
	| leaf |
	leaf _ root leafForKey: aMagnitude.
	leaf removeKey: aMagnitude.
	root _ leaf root
! !

!BTree methodsFor:'enumerating'!

commonKeysWith: aTree keysAndValuesDo: aBlock
	^ aTree depth < self depth
		ifTrue: [aTree root commonKeysWith: root keysAndValuesDo: aBlock flip: true]
		ifFalse: [root commonKeysWith: aTree root keysAndValuesDo: aBlock flip: false]
!

do: aBlock
	root allLeavesDo: [:ea | ea valuesDo: aBlock]
!

from: start to: end do: aBlock
	self from: start to: end keysAndValuesDo: [:k :v | aBlock value: v]
!

from: start to: end keysAndValuesDo: aBlock
	root leavesFrom: start to: end do:
		[:ea |
		ea keysAndValuesDo:
			[:k :v |
			(k between: start and: end) ifTrue:
				[aBlock value: k value: v]]]
!

keysAndValuesDo: aBlock
	root allLeavesDo: [:ea | ea keysAndValuesDo: aBlock]
!

keysDo: aBlock
	root allLeavesDo: [:ea | ea keysDo: aBlock]
! !

!BTree methodsFor:'initialize-release'!

initializeWithKeys: aBTreeKeys
	aBTreeKeys size > 3 ifFalse: [self error: 'The BTree order must be at least 4'].
	root _ BTreeLeafNode keys: aBTreeKeys
! !

!BTree methodsFor:'private'!

root
	^ root
! !

!BTree class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/moreCollections/btree/BTree.st,v 1.1 2006/11/20 21:02:11 cg Exp $'
! !
