"{ Package: 'stx:goodies/moreCollections/btree' }"

Object variableSubclass:#BTreeNode
	instanceVariableNames:'parent keys'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-BTree'
!


!BTreeNode class methodsFor:'as yet unclassified'!

keys: anArray
	^ (self new: (anArray size)) keys: anArray
! !

!BTreeNode methodsFor:'accessing'!

children
	^ Array streamContents: [:s | self childrenDo: [:ea | s nextPut: ea]]
!

depth
	^ parent ifNil: [1] ifNotNil: [1 + parent depth]
!

firstKey
	^ keys first
!

parent
	^ parent
!

parent: aBTreeNode
	parent _ aBTreeNode
!

root
	^ parent
		ifNil: [self]
		ifNotNil: [parent root]
!

values
	^ Array streamContents: [:s | self valuesDo: [:ea | s nextPut: ea]]
! !

!BTreeNode methodsFor:'enumerating'!

allChildrenDo: aBlock
	self childrenDo:
		[:ea |
		aBlock value: ea.
		ea allChildrenDo: aBlock]
!

allLeavesDo: aBlock
	self withAllChildrenDo: [:ea | ea isLeaf ifTrue: [aBlock value: ea]]
!

childrenDo: aBlock
	self subclassResponsibility
!

keysAndValuesDo: aBlock
	keys withIndexDo:
		[:key :i |
		key ifNotNil: [aBlock value: key value: (self at: i)]]
!

keysDo: aBlock
	keys withIndexDo:
		[:key :i |
		key ifNotNil: [aBlock value: key] ifNil: [^ self]]
!

leavesFrom: start to: end do: aBlock
	self subclassResponsibility
!

valuesDo: aBlock
	self keysAndValuesDo: [:k :v | aBlock value: v]
!

withAllChildrenDo: aBlock
	aBlock value: self.
	self allChildrenDo: aBlock.
! !

!BTreeNode methodsFor:'inserting'!

insertKey: aMagnitude value: anObject
	| index key |
	index _ keys findIndexForKey: aMagnitude.
	index = 0 ifTrue:
		[self canGrow
			ifTrue:
				[self shiftRightFrom: 1.
				^ self insertKey: aMagnitude value: anObject at: 1]
			ifFalse:
				[self split.
				^ (parent childForKey: aMagnitude) insertKey: aMagnitude value: anObject]].
	
	key _ keys at: index.
	key = aMagnitude ifTrue:
		[^ self insertKey: aMagnitude value: anObject at: index].
	index < self size ifTrue:
		[key _ keys at: index + 1.
		key
			ifNil: [^ self insertKey: aMagnitude value: anObject at: index+1]
			ifNotNil:
				[self canGrow ifTrue:
					[self shiftRightFrom: index+1.
					^ self insertKey: aMagnitude value: anObject at: index+1]]].

	"otherwise"
	self split.
	^ (parent childForKey: aMagnitude) insertKey: aMagnitude value: anObject
! !

!BTreeNode methodsFor:'private'!

ensureParent
	parent ifNil:
		 [parent _ BTreeInteriorNode keys: keys emptyCopy.
		parent insertKey: self firstKey value: self].
	^ parent
!

grow
        | sibling |
        parent ifNotNil:
                [sibling _ parent nextSiblingForChild: self.
                sibling ifNil: ["we're the new root" parent _ nil. ^ self].
                sibling canShrink
                        ifTrue: [self stealFrom: sibling]
                        ifFalse: [self mergeWith: sibling]]

    "Created: / 20-11-2006 / 13:53:45 / cg"
!

insertKey: aMagnitude value: anObject at: index
	keys at: index put: aMagnitude.
	self at: index put: anObject
!

keys: anArray
	keys _ anArray
!

mergeWith: aNode		
	| oldKey |
	oldKey _ self firstKey.
	aNode keysAndValuesDo:
		[:k :v |
		self insertKey: k value: v].
	parent removeKey: aNode firstKey.
	parent updateKey: oldKey to: self firstKey.
!

shiftLeftTo: index
	keys shiftLeftTo: index.
	index to: self size - 1 by: 1 do:
		[:i |
		self at: i put: (self at: i+1)].
	self at: self size put: nil.
!

shiftRightFrom: index
	keys shiftRightFrom: index.
	self size to: index+1 by: -1 do:
		[:i |
		self at: i put: (self at: i-1)]
!

split
	| other midpoint |
	other _ self class keys: keys emptyCopy.
	midpoint _ self size // 2 + 1.
	midpoint to: self size do:
		[:i |
		other insertKey: (keys at: i) value: (self at: i) at: (i - midpoint + 1).
		keys at: i put: nil.
		self at: i put: nil].
	
	self ensureParent insertKey: other firstKey value: other
!

stealFrom: aNode
	| key value |
	aNode firstKey > self firstKey
		ifTrue: [value _ aNode at: 1. key _ aNode firstKey]
		ifFalse:
			[aNode keysAndValuesDo: [:k :v | key _ k. value _ v].
			parent ifNotNil: [parent updateKey: self firstKey to: key]].
	self insertKey: key value: value.
	aNode removeKey: key
! !

!BTreeNode methodsFor:'removing'!

removeKey: aMagnitude
	| index key |
	self canShrink ifFalse: [self grow].
	
	index _ keys findIndexForKey: aMagnitude.
	key _ keys at: index.
	key = aMagnitude ifFalse: [^ self error: 'No such key'].
	
	self shiftLeftTo: index.
	
	index = 1 ifTrue: [parent ifNotNil: [parent updateKey: key to: self firstKey]]
! !

!BTreeNode methodsFor:'testing'!

canGrow
	^ keys canGrow
!

canShrink
	^ keys canShrink
!

isLeaf
	self subclassResponsibility
! !

!BTreeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/moreCollections/btree/BTreeNode.st,v 1.1 2006/11/20 21:02:45 cg Exp $'
! !
