"{ Package: 'stx:goodies/moreCollections/btree' }"

BTreeKeys subclass:#BTreeStringKeys
	instanceVariableNames:'keys prefix abbreviations'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-BTree'
!


!BTreeStringKeys class methodsFor:'as yet unclassified'!

new
	^ self new: 8
!

new: aNumber
	^ self basicNew initializeWithSize: aNumber
! !

!BTreeStringKeys methodsFor:'as yet unclassified'!

abbreviationSize
	^ 3
!

abbreviationsAndIndicesDo: aBlock
	| stream |
	stream _ abbreviations readStream.
	1 to: self size do:
		[:i |
		stream atEnd
			ifFalse: [aBlock value: prefix, (stream next: self abbreviationSize) value: i]
			ifTrue: [aBlock value: nil value: i]]
!

at: aNumber
	^ keys at: aNumber
!

at: aNumber put: aString
	keys at: aNumber put: aString.
	prefix _ self nilPrefix.
!

buildAbbreviationsFrom: readStreams
	| nextChars |
	1 to: self abbreviationSize do:
		[:i |
		nextChars _ readStreams collect: [:ea | ea next ifNil: [Character value: 0]].
		nextChars withIndexDo:
			[:c :j |
			abbreviations at: (j-1 * self abbreviationSize) + i put: c]].
	^ abbreviations
!

extractPrefixFrom: readStreams
	| prefixStream nextChars |
	prefixStream _ String new writeStream.
	
	[readStreams anySatisfy: [:ea | ea atEnd]] whileFalse:
		[nextChars _ readStreams collect: [:ea | ea next].
		(nextChars allSatisfy: [:ea | ea = nextChars first])
				ifTrue: [prefixStream nextPut: nextChars first]
				ifFalse: [readStreams do: [:ea | ea skip: -1]. ^ prefixStream contents]].
	^ prefixStream contents
!

findIndexForKey: aString
	| stream str diff |
	prefix = self nilPrefix ifTrue: [self rebuildAbbreviations].
	stream _ aString readStream.
	str _ stream next: prefix size + self abbreviationSize.
	diff _ prefix size + self abbreviationSize - str size.
	str _ str, (String new: diff).
	self abbreviationsAndIndicesDo:
		[:abbr :i |
		abbr ifNil: [^ i - 1].
		str < abbr ifTrue: [^ i - 1].
		str = abbr ifTrue: [^ super findIndexForKey: aString]].
	^ self size
!

initializeWithSize: aNumber
	keys _ Array new: aNumber.
	prefix _ self nilPrefix.
!

nilPrefix
	^ '^^^'
!

rebuildAbbreviations
	| keyStreams filled |
	filled _ keys count: [:ea | ea notNil].
	abbreviations _ String new: (filled * self abbreviationSize).
	filled = 0 ifTrue: [prefix _ ''. ^ self ].
	keyStreams _ (1 to: filled) collect: [:i | (keys at: i) readStream].
	
	prefix _ self extractPrefixFrom: keyStreams.
	abbreviations _ self buildAbbreviationsFrom: keyStreams.
!

size
	^ keys size
! !

!BTreeStringKeys class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/moreCollections/btree/BTreeStringKeys.st,v 1.1 2006/11/20 21:02:29 cg Exp $'
! !
