"{ Package: 'stx:goodies/moreCollections/btree' }"

Collection subclass:#TSTree
	instanceVariableNames:'root'
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-BTree'
!


!TSTree methodsFor:'accessing'!

at: aString
	^ self at: aString ifAbsent: [self error: aString printString, ' not found']
!

at: aString ifAbsent: exceptionBlock
	aString isEmpty ifTrue: [^ exceptionBlock value].
	root ifNil: [^ exceptionBlock value].
	
	^ (root lookupString: aString startingAt: 1) ifNil: exceptionBlock
!

at: aString ifAbsentPut: exceptionBlock
	^ self at: aString ifAbsent: [self at: aString put: exceptionBlock value]
!

at: aString put: anObject
	aString isEmpty ifTrue: [self error: 'Keys cannot be empty'].
	root ifNil: [root _ TSTreeNode key: aString first].
	root lookupString: aString startingAt: 1 insert: anObject.
	^ anObject
!

removeKey: aString
	^ self removeKey: aString ifAbsent: [self error: 'Could not find key ', aString printString]
!

removeKey: aString ifAbsent: errorBlock
	| val |
	val _ root removeString: aString startingAt: 1.
	root canBeCulled ifTrue: [root _ nil].
	^ val ifNil: errorBlock
!

values
	^ Array streamContents: [:s | self do: [:ea | s nextPut: ea]]
! !

!TSTree methodsFor:'as yet unclassified'!

matchesForPrefix: aString
	^ Array streamContents:
		[:s |
		self matchesForPrefix: aString do:
			[:match |
			s nextPut: match]]
!

matchesForPrefix: aString do: aBlock
	aString isEmpty
		ifTrue: [self do: aBlock]
		ifFalse: [root ifNotNil: [root matchesForPrefix: aString startingAt: 1 do: aBlock]]
!

matchesForString: aString distance: aNumber
	^ Array streamContents:
		[:s |
		self matchesForString: aString distance: aNumber do:
			[:match |
			s nextPut: match]]
!

matchesForString: aString distance: aNumber do: aBlock
	self matchesForString: aString distance: aNumber limitNodes: nil do: aBlock
!

matchesForString: aString distance: aNumber limitNodes: maxNodes do: aBlock
	| nodeCount |
	nodeCount _ 0.
	aString isEmpty ifTrue: [^ self].
	root isNil ifTrue: [^ self].
	root matchesForString: aString startingAt: 1 distance: aNumber do: aBlock nodesDo:
		[:ea |
		nodeCount _ nodeCount + 1.
		nodeCount = maxNodes ifTrue: [^ self]]
! !

!TSTree methodsFor:'enumerating'!

do: aBlock
	root ifNotNil: [root do: aBlock]
! !

!TSTree class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/moreCollections/btree/TSTree.st,v 1.1 2006/11/20 21:01:56 cg Exp $'
! !
