"{ Package: 'stx:goodies/moreCollections/btree/test' }"

TestCase subclass:#BTreeTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-BTree'
!


!BTreeTest class methodsFor:'helpers'!

shuffled:aCollection
    |dstIndex remaining shuffledCollection element|

    shuffledCollection := aCollection copyEmpty:(aCollection size).

    dstIndex := 1.
    remaining := OrderedCollection withAll:aCollection.
    [remaining notEmpty] whileTrue:[
        element := remaining removeAtIndex:(Random nextIntegerBetween:1 and:remaining size).
        shuffledCollection at:dstIndex put:element.
        dstIndex := dstIndex + 1.
    ].
    ^ shuffledCollection

    "
     self shuffled:#(1 2 3 4 5 6 7 8 9 10)
    "

    "Created: / 20-11-2006 / 16:54:31 / cg"
! !

!BTreeTest methodsFor:'as yet unclassified'!

assertConsistent: aBTree
	| depths root |
	root _ aBTree root.
	
 " http://www.cs.umbc.edu/~woodcock/cmsc341/btree/defn.html

A B-Tree of order m is a search tree, where the data 
  (or pointers to it) is stored at the leaves, such that:

1) the root is either a leaf (as well--i.e. the tree is 
     a single node) or has between 2 and m children"
	
	self assert: (root isLeaf or: [root values size between: 2 and: aBTree order]).
	
"  2) all other (non-root) nodes have at least m/2 children,
     but no more than m children"

	root allChildrenDo: [:ea | self assert: (ea values size between: (aBTree order // 2) and: aBTree order)]. 

"  3) all leaves are at the same depth"

	depths _ Set new.
	root withAllChildrenDo: [:ea | ea isLeaf ifTrue: [depths add: ea depth]].
	self assert: depths size = 1.
!

assertSorted: aTree
	| last |
	last _ nil.
	aTree keysAndValuesDo:
		[:k :v |
		last ifNotNil: [self assert: last < k].
		last _ k]
!

checkTree: tree array: pairArray
        pairArray withIndexDo:
                [:pair :i |
                tree at: pair first put: pair second.
                self assert: tree size = i.
                "self assertConsistent: tree.
                self assertSorted: tree"].
        (self class shuffled:pairArray) do:
                [:pair |
                self assert: (tree at: pair first) = pair second].
        self assert: tree size = pairArray size.
        self assertConsistent: tree.
        self assertSorted: tree.
        ^ tree

    "Modified: / 20-11-2006 / 16:59:23 / cg"
!

checkTree: tree items: count
        | pairArray  |

        
        pairArray _ self class shuffled:((1 to: count) collect: [:i | Array with: i asString with: i]).
        self checkTree: tree array: pairArray

    "Modified: / 20-11-2006 / 16:57:09 / cg"
!

checkTreeWithOrder: order items: count
	self checkTree: (BTree order: order) items: count
!

testAddAndRetrieveWithSplit
	self checkTreeWithOrder: 4 items: 5
!

testAddAndRetrieveWithTwoSplits
	self checkTreeWithOrder: 4 items: 17
!

testCommonKeysDifferentValues
	| large small expectedResults results |
	large _ BTree new.
	1 to: 1000 by: 2 do: [:i | large at: i put: i].
	small _ BTree new.
	2 to: 2000 by: 11 do: [:i | small at: i put: i*2].
	
	expectedResults _ ((-9 to: 1000 by: 22) allButFirst collect: [:i | Array with: i with: i with: i*2]) asOrderedCollection.
	
	results _ OrderedCollection new.
	large commonKeysWith: small keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v1 with: v2)].
	self assert: results = expectedResults.
	
	results _ OrderedCollection new.
	small commonKeysWith: large keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v2 with: v1)].
	self assert: results = expectedResults
!

testCommonKeysSameValues
	| large small expectedResults results |
	large _ BTree new.
	1 to: 1000 by: 2 do: [:i | large at: i put: i].
	small _ BTree new.
	2 to: 2000 by: 11 do: [:i | small at: i put: i].
	
	expectedResults _ ((-9 to: 1000 by: 22) allButFirst collect: [:i | Array with: i with: i with: i]) asOrderedCollection.
	
	results _ OrderedCollection new.
	large commonKeysWith: small keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v1 with: v2)].
	self assert: results = expectedResults.
	
	results _ OrderedCollection new.
	small commonKeysWith: large keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v2 with: v1)].
	self assert: results = expectedResults
!

testEmptyTree
	self checkTreeWithOrder: 5 items: 0
!

testLargeTree
	"self checkTreeWithOrder: 8 items: 5000"
	
!

testMissing
	| tree |
	tree _ BTree new.
	tree at: 1 put: 'foo'.
	tree at: 2 put: 'bar'.
	self assert: (tree at: 1 ifAbsent: [42]) = 'foo'.
	self assert: (tree at: 2 ifAbsent: [42]) = 'bar'.
	self assert: (tree at: 3 ifAbsent: [42]) = 42.
!

testMultipleAddAndRetrieve
	self checkTreeWithOrder: 5 items: 3
!

testRange
	| tree keys |
	tree _ BTree new.
	1 to: 100 by: 2 do: [:ea | tree at: ea put: ea * 2].
	self assert: tree size = 50.
	keys _ OrderedCollection new.
	tree from: 12 to: 19 keysAndValuesDo:
		[:k :v |
		self assert: k * 2 = v.
		keys add: k].
	self assert: keys asArray = #(13 15 17 19).
	self assertSorted: tree.
	self assertConsistent: tree.
!

testRemoval
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	tree removeKey: #d.
	self assert: tree size = 11.
	self assert: (tree at: #d ifAbsent: [32]) = 32.
	tree removeKey: #k.
	self assert: tree size = 10.
	self assert: (tree at: #k ifAbsent: [32]) = 32.
	self assertSorted: tree.
	self assertConsistent: tree.
!

testRemoval2
	| tree |
	tree := BTree new.
	1 to: 15 do: [:i | tree at: i put: i].
	1 to: 15 do: [:i | tree removeKey: i].
	self assert: tree isEmpty
!

testRemoveAll
        | tree |
        tree _ BTree new.
        #(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
        self assert: tree size = 12.
        (self class shuffled:#(a b c d e f g h i j k l)) do: [:ea | tree removeKey: ea].
        self assert: tree size = 0.

        self assertSorted: tree.
        self assertConsistent: tree.

    "Modified: / 20-11-2006 / 16:59:53 / cg"
!

testRemoveAllFailing
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	#(#c #j #g #i #k #l #f #e #d #h #a #b) do: [:ea | tree removeKey: ea].
	self assert: tree size = 0.

	self assertSorted: tree.
	self assertConsistent: tree.
!

testRemoveLots
        | tree |
        tree _ BTree new.
        #(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
        self assert: tree size = 12.
        (self class shuffled:#(a c e g i k)) do: [:ea | tree removeKey: ea].
        self assert: tree size = 6.

        self assertSorted: tree.
        self assertConsistent: tree.

    "Modified: / 20-11-2006 / 17:00:05 / cg"
!

testSimpleAddAndRetrieve
	self checkTreeWithOrder: 5 items: 1
!

testStoreNil
	self checkTree: BTree new array: {{'x'. 1}. {'y'. nil}}
!

testStringKeys
	self checkTree: (BTree keys: BTreeStringKeys new) items: 100
! !

!BTreeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/moreCollections/btree/test/BTreeTest.st,v 1.2 2006/11/20 21:23:44 cg Exp $'
! !
