"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

SingleValueWithParametersField subclass:#ContentDispositionField
	instanceVariableNames:''
	classVariableNames:'DefaultContentDisposition'
	poolDictionaries:''
	category:'NetClients-MIME'
!

ContentDispositionField comment:'This class represents content-disposition field as defined by RFC2183.
Content-disposition field designates Mime entity as ''inline'' or ''''attachment''. ''Inline'' parts can be
displayed automatically whereas ''attachments'' are located externally and require some action from the user. These can be displayed as iconized images. Content-disposition field also provides fiel name and essential file attributes to help in saving file and determining its Mime type of its contents.

Default setting is ''attachment'' and can be reset as
	Settings defaultContentDisposition: ''inline''.

This class provides accessor methods for standard parameters of this field. These methods are also provided in MimeEntity class (category content-disposition)

Excerpts from RFC2183:

   MIME specifies a standard format for encapsulating multiple pieces of
   data into a single Internet message. That document does not address
   the issue of presentation styles; it provides a framework for the
   interchange of message content, but leaves presentation issues solely
   in the hands of mail user agent (MUA) implementors.

   Two common ways of presenting multipart electronic messages are as a
   main document with a list of separate attachments, and as a single
   document with the various parts expanded (displayed) inline. The
   display of an attachment is generally construed to require positive
   action on the part of the recipient, while inline message components
   are displayed automatically when the message is viewed. A mechanism
   is needed to allow the sender to transmit this sort of presentational
   information to the recipient; the Content-Disposition header provides
   this mechanism, allowing each component of a message to be tagged
   with an indication of its desired presentation semantics.

   Tagging messages in this manner will often be sufficient for basic
   message formatting. However, in many cases a more powerful and
   flexible approach will be necessary. The definition of such
   approaches is beyond the scope of this memo; however, such approaches
   can benefit from additional Content-Disposition values and
   parameters, to be defined at a later date.

   In addition to allowing the sender to specify the presentational
   disposition of a message component, it is desirable to allow her to
   indicate a default archival disposition; a filename. The optional
   "filename" parameter provides for this.  Further, the creation-date,
   modification-date, and read-date parameters allow preservation of
   those file attributes when the file is transmitted over MIME email.

2.  The Content-Disposition Header Field

   Content-Disposition is an optional header field. In its absence, the
   MUA may use whatever presentation method it deems suitable.

   It is desirable to keep the set of possible disposition types small
   and well defined, to avoid needless complexity. Even so, evolving
   usage will likely require the definition of additional disposition
   types or parameters, so the set of disposition values is extensible;
   see below.

   In the extended BNF notation of [RFC 822], the Content-Disposition
   header field is defined as follows:

     disposition := "Content-Disposition" ":"
		    disposition-type
		    *(";" disposition-parm)

     disposition-type := "inline"
		       / "attachment"
		       / extension-token
		       ; values are not case-sensitive

     disposition-parm := filename-parm
		       / creation-date-parm
		       / modification-date-parm
		       / read-date-parm
		       / size-parm
		       / parameter

     filename-parm := "filename" "=" value

     creation-date-parm := "creation-date" "=" quoted-date-time

     modification-date-parm := "modification-date" "=" quoted-date-time

     read-date-parm := "read-date" "=" quoted-date-time

     size-parm := "size" "=" 1*DIGIT

     quoted-date-time := quoted-string
		      ; contents MUST be an RFC 822 `date-time''
		      ; numeric timezones (+HHMM or -HHMM) MUST be used

'
!

!ContentDispositionField class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!ContentDispositionField class methodsFor:'defaults'!

default
	^self fromLine: 'Content-disposition: ', self defaultContentDisposition
!

defaultContentDisposition
	^DefaultContentDisposition isNil
		ifTrue: [ DefaultContentDisposition := 'attachment' ]
		ifFalse: [DefaultContentDisposition ]
!

defaultContentDisposition: aString
	DefaultContentDisposition := aString
!

defaultSeparators
	^#($; $; nil  )
! !

!ContentDispositionField class methodsFor:'parsing'!

fieldNames
	^#('content-disposition')
! !

!ContentDispositionField methodsFor:'accessing'!

disposition
	^self localValue
!

disposition: aString
" Disposition should be 'inline' or 'attachment' "
	self localValue: aString
!

fileName
	^self parameterAt: 'filename'
!

fileName: aString
	^self parameterAt: 'filename' put: aString
!

size
	Integer readFrom: (self parameterAt: 'size' ifAbsent: [^nil]) readStream
!

size: anInteger
	self parameterAt: 'size' put: anInteger printString
! !

!ContentDispositionField methodsFor:'testing'!

isAttachment
" Answer true if content disposition is 'attachments' "
	^self disposition asLowercase = 'attachment'
!

isInline
" Answer true if content disposition is 'inline' "
	^self disposition asLowercase = 'inline'
! !

!ContentDispositionField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ContentDispositionField.st,v 1.2 2009/09/29 20:09:08 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ContentDispositionField.st,v 1.2 2009/09/29 20:09:08 cg Exp $'
! !
