"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

SingleValueWithParametersField subclass:#ContentTypeField
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

ContentTypeField comment:'This class represents content-type field in MIME and HTTP protocols.
RFC2045 implements content-type field syntax defined in RFC2045.

The field performs parsing/compisition for content-type field and exposes
accessor methods to access individual components:
	contetType (''type/subtype'', example ''text/html'')
	type
	subType
	charset
	boundary

Other parameters can be accessed using #parameterAt:, #parameterAt:put:

Syntax (abbreviated) from RFC2045 (ftp.uu.net/inet/rfcs/rfc2045.Z):

     content := "Content-Type" ":" type "/" subtype
		*(";" parameter)
		; Matching of media type and subtype
		; is ALWAYS case-insensitive.

     type := discrete-type / composite-type

     discrete-type := "text" / "image" / "audio" / "video" /
		      "application" / extension-token

     composite-type := "message" / "multipart" / extension-token

     parameter := attribute "=" value

Instance Variables:

	type    <String>                        type as defined above
	subtype <String>                        subtype'
!

!ContentTypeField class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!ContentTypeField class methodsFor:'instance creation'!

applicationOctetStream
	" self applicationOctetStream "
	^self contentType: MimeTypeDescriptor contentTypeApplicationOctetStream
!

contentType: aString
	^self contentType: aString charset: nil
!

contentType: aString charset: cString
	^self new
		name: 'content-type';
		contentType: aString;
		charset: cString;
		yourself
!

messageRfc822
	" self messageRfc822 "
	^self contentType: MimeTypeDescriptor contentTypeMessageRfc822 charset: self defaultCharset
!

textPlain
	" self textPlain "
	^self contentType: MimeTypeDescriptor contentTypePlainText charset: self defaultCharset
! !

!ContentTypeField class methodsFor:'defaults'!

default
	^self iso8859x1Default
!

defaultCharset
	^self iso8859x1
!

defaultContentType
	^MimeTypeDescriptor defaultContentTypeString
!

defaultSeparators
	^#($; $; nil  )
!

defaultUnsupportedContentType
	^MimeTypeDescriptor defaultUnsupportedContentType
!

iso8859x1
	^MimeTypeDescriptor iso8859x1
!

iso8859x1Default
	^self new
		name: 'content-type';
		contentType: self defaultContentType;
		charset: self iso8859x1;
		yourself
!

usAscii
	^MimeTypeDescriptor usAscii
!

usAsciiDefault
	^self new
		name: 'content-type';
		contentType: self defaultContentType;
		charset: self usAscii;
		yourself
! !

!ContentTypeField class methodsFor:'parsing'!

fieldNames
	^#('content-type')
! !

!ContentTypeField methodsFor:'accessing'!

boundary
	^self parameterAt: 'boundary'
!

boundary: aString
	^self parameterAt: 'boundary' put: aString
!

charset
	| charset |
	charset := self parameterAt: 'charset' ifAbsent: [nil].
	^(charset == nil
		ifTrue:
			[ ('text*' match: self contentType)
				ifTrue: [ self class defaultCharset]
				ifFalse: [ nil ]]
		ifFalse: [ charset asLowercase ])
!

charset: aName

	^aName ~~ nil
		ifTrue: [self parameterAt: 'charset' put: aName ]
		ifFalse: [ nil]
!

charsetIfNone: aBlock

	^(self parameterAt: 'charset' ifAbsent: [aBlock value]) asLowercase
!

contentType

	^self localValue
!

contentType: aString
	self localValue:  aString
!

fileName
	^(self parameterAt: 'name' ifAbsent: [^String new ]) asLowercase
!

fileName: aName
	^self parameterAt: 'name' put: aName
!

subtype
	^(self contentType tokensBasedOn: $/) last.
!

subtype: aString
	self type: self type subtype: aString.
!

type
	^(self contentType tokensBasedOn: $/) first.
!

type: aString
	self type: aString subtype: self subtype
!

type: typeString subtype: subtypeString
	self  localValue: (typeString, '/', subtypeString)
! !

!ContentTypeField methodsFor:'testing'!

isMultipart
	^self type = 'multipart'
! !

!ContentTypeField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ContentTypeField.st,v 1.2 2009/09/29 20:08:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ContentTypeField.st,v 1.2 2009/09/29 20:08:41 cg Exp $'
! !
