"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

PositionableStream subclass:#CrLfStream
	instanceVariableNames:'stream readStatus eatLf'
	classVariableNames:'Lf Cr'
	poolDictionaries:''
	category:'NetClients-Framework'
!

CrLfStream comment:'A CrLfStream acts as a pipe which transforms incoming data into LF-separated
lines, and outgoing data into CRLF-separated lines.'
!

!CrLfStream class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!CrLfStream class methodsFor:'instance creation'!

initialize
    Cr := Character return "cr".
    Lf := Character nl.

    "   
     self initialize
    "
!

new
    "have to re-allow new - it was disabled in Stream"
    ^ self basicNew
!

on: aStream
    ^self new on: aStream
! !

!CrLfStream methodsFor:'initializing'!

on: aStream
    stream := aStream.
    eatLf := false.
    readStatus := #none
! !

!CrLfStream methodsFor:'stream'!

atEnd
    ^stream atEnd and: [ readStatus == #none ]
!

close
    stream close
!

flush
    stream flush
!

next
    | result |
    readStatus == #none ifFalse: [
	readStatus == Cr ifTrue: [ stream peekFor: Lf ].
	readStatus := #none.
	^Lf ].

    result := stream next.
    ^(result == Cr or: [ result == Lf ])
	ifTrue: [ readStatus := result. Cr ]
	ifFalse: [ result ]
!

nextLine
    | line |
    line := self upTo: Cr.
    self next.  "Eat line feed"
    ^line
!

nextPut: aCharacterArg
    |aCharacter|

    (aCharacter := aCharacterArg) isCharacter ifFalse:[
        aCharacter := Character value:aCharacter.
    ].
    eatLf
        ifTrue: [
            eatLf := false.
            aCharacter == Lf ifTrue: [ ^self ] ]
        ifFalse: [
            aCharacter == Lf ifTrue: [
                stream nextPut: Cr; nextPut: Lf; flush.
                ^self ] ].

    stream nextPut: aCharacter.
    aCharacter == Cr ifTrue: [
        stream nextPut: Lf; flush.
        eatLf := true.
    ].
!

peek
    | result |
    readStatus == #none ifFalse: [
	readStatus == Cr ifTrue: [ stream peekFor: Lf ].
	readStatus := Lf. "peek for LF just once"
	^Lf ].

    result := stream peek.
    ^result == Lf
	ifTrue: [ Cr ]
	ifFalse: [ result ]
!

peekFor: aCharacter
    | result success |
    readStatus == #none ifFalse: [
	readStatus == Cr ifTrue: [ stream peekFor: Lf ].
	success := aCharacter == Lf.
	readStatus := success ifTrue: [ #none ] ifFalse: [ Lf ]. "peek for LF just once"
	^success
    ].
    result := stream peek.
    (result == Cr or: [ result == Lf ]) ifTrue: [
	success := aCharacter == Cr.
	success ifTrue: [ readStatus := stream next ].
	^success
    ].
    success := aCharacter == result.
    success ifTrue: [ stream next ].
    ^success
!

species
    ^stream species
!

stream
    ^stream
! !

!CrLfStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__CrLfStream.st,v 1.3 2009/09/29 20:07:09 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__CrLfStream.st,v 1.3 2009/09/29 20:07:09 cg Exp $'
! !

CrLfStream initialize!
