"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

NetClient subclass:#IMAPClient
	instanceVariableNames:'state handler'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-IMAP'
!

IMAPClient comment:'IMAPClient implements user agent interface to IMAP services. These all are in category. All methods in this category support synchromous execution of a specific IMAP request. They creates an instance of command, queues it for IMAP protocol interpreter and waits for command''s completion. All methods answer IMAPCommand. Utility category provides higher level requests that extract data from a response, i.e. after fetch request.
IMAP Client uses state objects to validate if a request is valid in a given state and to handle state transitions.
IMAPClient is also responsible for a number of policies such as automatic reconnect, idle timeout, etc.


Instance Variables:

	state   <IMAPState>     The IMAP4rev1 server state.
	handler <IMAPClientHandler>     An instance of IMAPClientHolder handles unsolicited server messages'
!

!IMAPClient class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Please see
        http://www.linux-magazin.de/Heft-Abo/Ausgaben/2007/06/Facharbeiten-mit-Verstand
"
! !

!IMAPClient class methodsFor:'class initialization'!

initialize
	super initialize
! !

!IMAPClient class methodsFor:'constants'!

serverType
	^#IMAP
! !

!IMAPClient class methodsFor:'defaults'!

defaultPortNumber
	^143
! !

!IMAPClient class methodsFor:'examples'!

examples

    |imapClient list|

    imapClient := IMAPClient loginToHost: 'exept' port: nil asUser: 'penk' withPassword: 'schnuppe'.
    list := imapClient list:'*'  mailbox: '*'.

"
    self examples
"
! !

!IMAPClient class methodsFor:'testing'!

isIncomingMailClient
	^true
!

isMailClient
	^true
!

isOutgoingMailClient
	^false
! !

!IMAPClient methodsFor:'accessing'!

handler
	^handler isNil
		ifTrue: [handler := self defaulfHandler]
		ifFalse: [handler]
!

handler: imapClientHandler

	handler := imapClientHandler
!

protocolInterpreter
    ^ IMAPProtocolInterpreter
!

protocolInterpreterClass
	^IMAPProtocolInterpreter
!

state
	^state
!

state: aState
	state := aState.
	state client: self
! !

!IMAPClient methodsFor:'commands'!

append: aString to: aMailboxName

	^self state
		append: aString
		to: aMailboxName
		flags: nil
		date: nil
!

append: aString to: aMailboxName flags: flags date: dateString

	^self state
		append: aString
		to: aMailboxName
		flags: flags
		date: dateString
!

authenticate: aString
	| comm |
	^(comm := self state authenticate: aString) failed
		ifTrue: [NetClientError signalWith: #authenticate message: comm completionResponse text]
		ifFalse: [comm ]
!

capability

	^ self state capability
!

check
    ^ self state check
!

close

	^ self state close
!

copy: messageNumbers to: aMailboxName
	^self state copy: messageNumbers to: aMailboxName
!

create: aMailBoxName

	^ self state create: aMailBoxName
!

delete: aMailBoxName

	^ self state delete: aMailBoxName
!

examine: aMailBoxName

	^ self state examine: aMailBoxName
!

expunge
    "The method deletes the messages marked for deletion."

    ^ self state expunge
!

fetch: aCriteria
" Fetch messages based on specified criteria
	client fetch: '1:3 (uid Body.Peek[Header.Fields (Message-Id From)])'
	client fetch: '1:2 (flags internaldate uid RFC822)'. "

	^ self state fetch: aCriteria
!

fetch: messageNumbers retrieve: criteria

	^ self state fetch: messageNumbers retrieve: criteria
!

list: aMailbox
" client list: 'inbox' "

	^ self state list: #quotedText->'' mailbox: aMailbox
!

list: refName mailbox: name
    " client list: 'test/'  mailbox: 'mail/*' "

    ^ self state list: refName mailbox: name
!

login
	| comm |
	^(comm := self state login) failed
		ifTrue: [NetClientError signalWith: #login message: comm completionResponse text]
		ifFalse: [comm ]
!

logout
    ^ self state logout.
!

lsub: refName mailbox: name

	^ self state lsub: refName mailbox: name
!

noop

    ^self state noop
!

rename: oldMailBox newName: newMailBox

    ^self state rename: oldMailBox newName: newMailBox
!

search: aCriteria

    ^self state search: aCriteria
!

select: aMailBoxName

    ^self state select: aMailBoxName
!

status: aMailBoxName
" Obtain status of messages in specified mailbox.
	client status: 'inbox' "

	^ self status: aMailBoxName criteria: 'messages'
!

status: aMailBoxName criteria: anArrayOfStatusData
" Obtain status of specified mailbox using specified criteria. criterial
may be either string or array or strings if multiple criteria is given

	client status: 'inbox' criteria: #('messages' 'unseen').
	client status: 'sent items' criteria: 'messages'. "

	^ self state status: aMailBoxName criteria: anArrayOfStatusData
!

store: args

	^ self state store: args
!

subscribe: aMailBoxName

	^ self state subscribe: aMailBoxName
!

uid: aString

	^ self state uid: aString
!

unsubscribe: aMailBoxName

	^ self state unsubscribe: aMailBoxName
! !

!IMAPClient methodsFor:'connection'!

connect
	"Establish a connection to the host "
	| nRetry |
	nRetry := self retries.
	self clientPI client: self.

	[super connect.
	self state: IMAPNonAuthenticatedState new.
	]
	on: OSErrorHolder peerFaultSignal, OsInaccessibleError
	do:     [:ex |
		nRetry := nRetry - 1.
		(ex getSignal = OSErrorHolder peerFaultSignal
		and: [(self handlePeerFaultError: ex retry: nRetry)])
			ifTrue:
				[ NetClientError signalWith: #connection message: 'Unable to connect to the server.'].
		ex class = OsInaccessibleError
			ifTrue:
				[ NetClientError signalWith: #connection message: 'Server is not accessible.']]
!

connectToHost: aString port: aNumber
    "Establish a connection to the host <aString>."

    super connectToHost: aString port: aNumber.
    self state: IMAPNonAuthenticatedState new
!

isConnected
	^self clientPI closed not
!

loginToHost

	self login completedSuccessfully ifFalse: [ ^nil].
! !

!IMAPClient methodsFor:'create&execute command'!

commandClassFor: cmdName
	^self class commandClassFor: cmdName
!

createCommand: aString

	^ self createCommand: aString arguments: nil
!

createCommand: aString arguments: anArray
	^IMAPCommand
		forClient: clientPI
		name: aString
		arguments: anArray
!

execute: cmd arguments: args changeStateTo: aStateBlock
    ^self
	execute: [self createCommand: cmd arguments: args ]
	changeStateTo: aStateBlock
!

execute: aBlock changeStateTo: aStateBlock
    | command |
    command := aBlock value.
    self executeCommand: command.
    command wait.
    command completedSuccessfully ifTrue: [self state: aStateBlock value ].
    ^command
!

executeAndWait: aString
    ^self executeAndWait: aString arguments: nil
!

executeAndWait: aString arguments: anArray
	| command |
	command := self createCommand: aString arguments: anArray.
	self executeCommand: command.
	command waitFor: self timeout.
	^ command
!

executeAndWait: aString arguments: anArray for: ms
	| command |
	command := self createCommand: aString arguments: anArray.
	self executeCommand: command.
	command waitFor: ms.
	^ command
!

executeAndWait: aString for: ms
	^self
		executeAndWait: aString
		arguments: nil
		for: ms
!

executeCommand: aCommand
    ^self clientPI closed
            ifTrue: [NetClientError signalWith: #connection message: 'Server is not accessible.']
            ifFalse: [self clientPI executeCommand: aCommand]
! !

!IMAPClient methodsFor:'defaults'!

defaulfHandler

	^ IMAPClientHandler new
!

defaultMailbox
	^'inbox'
! !

!IMAPClient methodsFor:'private'!

canonicalizeMailboxName: aMailboxName
" Mailbox names are encoded in UTF-7 format. "
        | str |
        Smalltalk isSmalltalkX ifTrue:[
            str := CharacterEncoder encodeString:aMailboxName from:#iso8859 into:#utf7.
        ] ifFalse:[
            str := (ByteArray new withEncoding: #UTF_7IMAP) writeStream.
            str nextPutAll: aMailboxName.
            str close.
            str := (str encodedContents withEncoding: #ASCII) readStream contents
        ].
        ^IMAPScanner stringAsAssociation:str

!

handlePeerFaultError: exception retry: retry
	"The Server we are connecting to might be too busy to handle our
	connection request.  In this case, delay for a little while and try again.
	Once we have hit our connection retry limit, return true to the client so
	that it can handle the error as it sees fit."

	retry > 0 ifTrue:
		[(Delay forSeconds: self delaySeconds) wait.
		self log: ['Failed to connect to <1s>. Will try <2p> more times.' expandMacrosWith: self client hostName with: retries]
			level: #IMAPClient.
		exception retry].
	^true
!

messageSetAsString: messageNumbers
	| stream |
	stream := (String new: 64) writeStream.
	messageNumbers
			do: [:messageNumber | stream nextPutAll: messageNumber printString]
			separatedBy: [stream nextPut: $,].
	^stream contents
! !

!IMAPClient methodsFor:'responses'!

handle: aResponse
	"Ask IMAPClientHandler to handle responses that must be recorded by the client"

	^self handler handle: aResponse for: self
! !

!IMAPClient methodsFor:'utility'!

examineMailbox
" Examine default mailbox (inbox) "
	^ self examineMailbox: self defaultMailbox
!

examineMailbox: mailbox

	^ (self examine: mailbox) responses.
!

fetch: anArrayOfNumbers with: qualifierOrListOfQualifiers
" Returns an OrderedCollection of  associations which key is the message number and value the message"
        | args result dict data messageNumber |

        anArrayOfNumbers isEmpty ifTrue:[^ #() ].
        args := qualifierOrListOfQualifiers isCharacters
                                ifTrue: [Array with: qualifierOrListOfQualifiers]
                                ifFalse: [qualifierOrListOfQualifiers].
        result := self fetch: anArrayOfNumbers retrieve: args.
        dict := Dictionary new.
        ^result successful
                ifFalse: [self error: result completionResponse text. nil]
                ifTrue: [   
                            result commandResponses
                                collect:
                                        [:response |
                                        messageNumber := response value.
                                        data := response parameters first.
                                        1 to: data size by: 2
                                                do: [:idx | dict at: (data at: idx) asUppercase put: (data at: idx + 1)].
                                        messageNumber -> (qualifierOrListOfQualifiers collect: [:tag | dict at: tag asUppercase])]]
!

fetchMessage: messageNumber
" Returns an association which key is the message number and value the message"
	| result |
	result := self fetch: (Array with: messageNumber) with: #('RFC822').
	^result notNil
		ifTrue: [result first]
		ifFalse: [nil]
!

fetchMessageHeaders: anArrayOfNumbers
" Returns an OrderedCollection of  associations which key is the message number and value the message header"
	^self fetch: anArrayOfNumbers with: #('RFC822.HEADER')
!

fetchMessages: anArrayOfNumbers
" Returns an OrderedCollection of  associations which key is the message number and value the message"

	^self fetch: anArrayOfNumbers with: #('RFC822')
!

fetchRFC822MessageHeaders: anArrayOfNumbers
" Returns an OrderedCollection of  associations which key is the message number and value the message header"
	^self fetchMessageHeaders: anArrayOfNumbers
!

fetchRFC822Messages: anArrayOfNumbers
" Returns an OrderedCollection of  associations which key is the message number and value the message"
	^self fetchMessages: anArrayOfNumbers
!

markAsSeen: aCollectionOfNumbers
"Sets \Seen  flags for the specified message number"
	| string |
	string := self messageSetAsString: aCollectionOfNumbers.
	^ (self store: (string, ' +FLAGS (\Seen)')) successful
!

markForDelete: anArrayOfNumbers
"Sets deleted flags for the specified message numbers"
	| string |
	string := self messageSetAsString: anArrayOfNumbers.
	^ (self store: (string, ' +FLAGS (\Deleted)')) successful
!

messageCount
	"Returns the number of messages in the default mailbox (inbox).
	See messageCount: to retrieve message count from any mailbox "
	^self messageCount: self defaultMailbox
!

messageCount: mailbox
	"Returns the number of messages in specified mailbox "
	| resps |
	resps := self examineMailbox: mailbox.
	^(resps detect: [:each | each cmdName = 'EXISTS']) value asNumber
!

newMessages
	"Returns the number of recent messages in in the default mailbox (inbox).
	See newMessages: to retrieve new messages from any mailbox "
	^self newMessages: self defaultMailbox
!

newMessages: mailbox
	"Returns the number of recent messages in specified mailbox "
	| resps |
	resps := self examineMailbox: mailbox.
	^(resps detect: [:each | each cmdName = 'RECENT']) value asNumber
!

searchMessages: aCriteria
" Returns a collection of message numbers that match the given searching criteria"
        |  result  resp |

        result := self search: aCriteria.
        ^ result successful
                ifFalse: [ self error: result asMessage. nil]
                ifTrue:
                        [  (resp := result commandResponse parameters) notEmpty
                                ifTrue: [ resp collect: [ :each | each asNumber]]
                                ifFalse: [ resp ]
                        ]
!

searchMessagesForUids: aCollectionOfUidStrings
" Returns a collection of message numbers that match the message uids"
	| stream |
	stream := (String new: 64) writeStream.
	stream nextPutAll: 'uid '.
	aCollectionOfUidStrings
			do: [:uid | stream nextPutAll: uid]
			separatedBy: [stream nextPut: $,].

	^ self searchMessages: stream contents
!

status: anArrayOfStatusData forMailbox: aName
"Returns a list of message numbers"
" For example the command:
client status: (Array with: 'messages' with: 'unseen') forMailbox: 'inbox'.
returns:  #('23' '3') "

	| resp result |

	resp :=  (self status: aName criteria: anArrayOfStatusData)
				commandResponse
				parameters at: 2.
	result := List new: anArrayOfStatusData size.
	2 to: resp size by: 2 do: [ :each  | result add: ( resp at: each) ].
	^ result.
!

uidFor: anArrayOfMessageNumbers

	^(self fetch: anArrayOfMessageNumbers with: #('UID') )
		collect: [ :assoc |  assoc  value first ].
!

unmarkAsSeen: aCollectionOfNumbers
"Sets \Seen  flags for the specified message number"
	| string |
	string := self messageSetAsString: aCollectionOfNumbers.
	^ (self store: (string, ' -FLAGS (\Seen)')) successful
!

unmarkForDelete: anArrayOfNumbers
"Sets deleted flags for the specified message numbers"
	| string |
	string := self messageSetAsString: anArrayOfNumbers.
	^ (self store: (string, ' -FLAGS (\Deleted)')) successful
!

unseenMessages
	"Returns the number of unseen messages in inbox "
	^ self searchMessages: 'unseen'
! !

!IMAPClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPClient.st,v 1.8 2010/03/09 12:18:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPClient.st,v 1.8 2010/03/09 12:18:28 cg Exp $'
! !

IMAPClient initialize!
