"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#IMAPCommand
	instanceVariableNames:'client sequenceID name arguments status responses
		completionResponse promise'
	classVariableNames:'ResponseRegistry'
	poolDictionaries:''
	category:'NetClients-IMAP'
!

IMAPCommand comment:'IMAPCommand is implemented to create IMAP4rev1 command, execute it and extract the response data.

Instance Variables:

	client  <IMAPProtocolInterpreter>       The client manages IMAP server connection and and provides interface for sending IMAP commands and handling IMAP responses.
	sequenceID      <IMAPCommandSequencer>  Creates next command identifier (tag)
	name    <String>        the command name
	arguments       <Array>  An array of arguments
	status  <Symbol>        The command status as: #sent #done
	responses       <Collection>    The IMAPResponses collection
	completionResponse      <IMAPResponseTagged>    An instance of the IMAPResponseTagged like: "xxx OK FETCH completed"
	promise <TimedPromise>  An instance of TimedPromise

IMAPCommand support Promise pattern. Each command represents a promise to deliver responses in the future. Once command is queued for IMAP protocol, interpreter, control is returned to the calling code immediately. In order to wait till command execution is complete, send command message #wait. This gives application an easy and flexible means to control the level of concurrency with IMAP server.

Most of IMAP API methods execute synchronously -- that is, they create new command, queue it, wait for command''s completion and answer the command. Aplication that needs asynchronous execution should create a new command and queue it via IMAPClient>>executeCommand:.


IMAPCommand and its responses'
!

!IMAPCommand class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!IMAPCommand class methodsFor:'instance creation'!

forClient: anIMAPPI name: aString arguments: arguments
" The intention here is to let users specify the complete string of command arguments. Because this string may contain atom-specials like $(, etc., this line may be sent as quoted string, which would be wrong. So we fool the printing logic to view this string as an atom. It is a hack, but seems like a convenient one "

	| args |
	args := arguments isCharacters
				ifTrue: [#atom -> arguments]
				ifFalse: [arguments].
	^self new
		forClient: anIMAPPI
		name: aString
		arguments: args
!

login: aNameString password: aPassString
	^self name: 'login' arguments: (Array with: (#string->aNameString) with: (#string->aPassString))
!

new
    ^self basicNew initialize
!

parse: scanner
    "Read and parse next command from a stream. This is mainly useful for testing previously stored
    exchange logs"

    ^self new parse: scanner
!

readFrom: aStream
" Read and parse next command from a stream. This is mainly useful for testing previously stored exchange logs "

    ^self parse: (IMAPScanner on: aStream)
! !

!IMAPCommand class methodsFor:'class initialization'!

initialize
    "IMAPCommand initialize"

    (ResponseRegistry := Dictionary new)
	at: 'FETCH' put: #('FETCH' 'OK' 'NO' 'BAD');
	at: 'SEARCH' put: #('SEARCH' 'OK' 'NO' 'BAD');
	at: 'SELECT' put: #('FLAGS' 'EXISTS' 'RECENT' 'OK' 'NO' 'BAD');
	at: 'EXAMINE' put: #('FLAGS' 'EXISTS' 'RECENT' 'OK' 'NO' 'BAD');
	at: 'LIST' put: #('LIST' 'OK' 'NO' 'BAD');
	at: 'LSUB' put: #('LSUB' 'OK' 'NO' 'BAD');
	at: 'STATUS' put: #('STATUS');
	at: 'EXPUNGE' put: #('EXPUNGE' 'OK' 'NO' 'BAD');
	at: 'STORE' put: #('FETCH' 'OK' 'NO' 'BAD');
	at: 'UID' put: #('FETCH' 'SEARCH' 'OK' 'NO' 'BAD');
	at: 'CAPABILITY' put: #('CAPABILITY' 'OK' 'BAD');
	at: 'STORE' put: #('FETCH');
	at: 'LOGOUT' put: #('BYE' 'OK' 'BAD');
	at: 'CLOSE' put: #('OK' 'NO' 'BAD');
	at: 'CHECK' put: #('OK' 'NO');
	at: 'APPEND' put: #('OK' 'NO' 'BAD');
	at: 'SUBSCRIBE' put: #('OK' 'NO' 'BAD');
	at: 'RENAME' put: #('OK' 'NO' 'BAD');
	at: 'DELETE'  put: #('OK' 'NO' 'BAD');
	at: 'CREATE'  put: #('OK' 'NO' 'BAD');
	at: 'LOGIN'  put: #('OK' 'NO' 'BAD');
	at: 'AUTHENTICATE' put: #('OK' 'NO' 'BAD');
	at: 'NOOP'  put: #('OK' 'BAD')
! !

!IMAPCommand class methodsFor:'defined responses'!

definedResponsesAt: aName

	^self responseRegistry at: aName asUppercase ifAbsentPut: [IdentityDictionary new].
!

responseRegistry
	^ResponseRegistry
! !

!IMAPCommand methodsFor:'accessing'!

arguments
    ^arguments
!

arguments: anObject
    arguments := anObject
!

client
    ^client
!

client: anObject
    client := anObject
!

completionResponse: anObject

	completionResponse := anObject.
!

name
    ^name
!

name: anObject
    name := anObject
!

sequenceID
    ^sequenceID
!

sequenceID: anObject
    sequenceID := anObject
! !

!IMAPCommand methodsFor:'command termination'!

terminateWithException: exception
	self terminateWithResponse: (IMAPErrorResponse newFor: self exception: exception)
!

terminateWithResponse: response
	self completionResponse: response.
	self beDone
! !

!IMAPCommand methodsFor:'completion response'!

completionResponse
    ^completionResponse
! !

!IMAPCommand methodsFor:'execute'!

execute
	"Send the gived command to the server."

	self send.
	self beSent
!

wait

	^ self promise value
!

waitFor: ms
        [
            ms notNil ifTrue: [promise := TimedPromise forMilliseconds: ms].
            ^[self promise value]
                    on: OsNeedRetryError
                    do:     [:ex |
                            self terminateWithException: ex.
                            NetClientError signalWith: #timeout message: 'Timeout']
        ] valueUninterruptably
! !

!IMAPCommand methodsFor:'handle responses'!

definedResponses
	^self class definedResponsesAt: self name asUppercase
!

handle: aResponse
	(aResponse terminatesCommand: self) ifTrue:
		[self terminateWithResponse: aResponse.
		^aResponse hasTag: self sequenceID].
	(self isDefinedResponse: aResponse)
		ifTrue:
			[self responses add: aResponse.
			^true].
	^false
!

isDefinedResponse: aResponse
	^((self name asLowercase = aResponse cmdName asLowercase)
		or: [self definedResponses includes: aResponse cmdName])
!

needsClientNotification: aResponse
    ^false
"    ^client isInterestedIn: aResponse"
!

notifyClientIfNeeded: aResponse
    ^(self needsClientNotification: aResponse)
	ifTrue: [client handle: aResponse]
	ifFalse: [false]
!

registerResponse: aResponse

    (aResponse isCompletionResponse)
	ifTrue: [ self completionResponse: aResponse ]
	ifFalse: [ self responses add: aResponse ]
!

responses

    ^responses notNil
	ifTrue: [ responses ]
	ifFalse: [ responses := OrderedCollection new ]
! !

!IMAPCommand methodsFor:'initialization'!

forClient: anIMAPPI name: aString arguments: args

	self client: anIMAPPI.
	self name: aString.
	self arguments: (self canonicalizeArguments: args).
!

initialize
        responses := OrderedCollection new: 1.
        promise := Promise new
! !

!IMAPCommand methodsFor:'obsolete'!

completedSuccessfully

	^self successful
! !

!IMAPCommand methodsFor:'parsing'!

parse: scanner
	"Read and parse next command from a stream. This is mainly useful for testing previously stored
	exchange logs"

	| tokens |

	tokens := scanner deepTokenizeAsAssociation.
	self
		sequenceID: tokens first value;
		name: (tokens at: 2) value;
		arguments: (tokens copyFrom: 3 to: tokens size).
! !

!IMAPCommand methodsFor:'printing'!

printCompletionResponseOn: aStream indent: level

    self completionResponse notNil
	ifTrue: [ self completionResponse printOn: aStream indent: level]
!

printOn: aStream
	self scanner
		printTokenList: self asTokenList on: aStream.
!

printResponseOn: aStream indent: level

    (self responces isNil or: [ self responces isEmpty ])  ifTrue: [ ^String new].
    self responses do: [ : eachResponse |
	aStream nl. eachResponse printOn: aStream indent: level
	]
!

scanner
	^IMAPScanner
!

sendOn: aClient

    "aClient is a IMAPProtocolInterpreter"

    self client sendTokenList: self asTokenList
! !

!IMAPCommand methodsFor:'private'!

asTokenList
    | list |
    list := OrderedCollection with: (#atom->self sequenceID) with: (#atom->name).
    self arguments notNil
	ifTrue: [list addAll: self arguments].
    ^list
!

canonicalizeArguments: args
" Arguments can one of: integer, string or array of thereof, potentially nested. Scalars are
converted into array with this scalar as a sole element "

	args isNil ifTrue: [^Array new].
	(args isCharacters or: [args isSequenceable not])
		ifTrue: [^Array with: args]
		ifFalse: [^ args ]
!

promise
        ^promise 
!

send

	self client sendTokenList: self asTokenList.
	self client connectionStream cr.
! !

!IMAPCommand methodsFor:'responses'!

commandResponse
    | coll |
    ^(coll := self commandResponses) isEmpty
	ifTrue: [ nil ]
	ifFalse: [coll first]
!

commandResponseValue
    | resp |
    ^(resp := self commandResponse) isNil
	ifTrue: [ nil ]
	ifFalse: [resp value]
!

commandResponses
    ^self responses select: [:resp | resp cmdName match: self name ignoreCase:true ]
!

responseNamed: aName
	^self responsesNamed: aName
!

responsesNamed: aName
	^self responses select: [:resp | resp cmdName = aName ]
!

statusResponse
	^self statusResponses first
!

statusResponses
	^self responses select: [:resp | resp isKindOf: IMAPResponseStatus]
! !

!IMAPCommand methodsFor:'status'!

beDone

    self status: #done.
    self client commandIsDone: self.
    self value: self completionResponse
!

beSent

    self status: #sent.
    self client commandIsInProgress: self
!

status

    ^status
!

status: anObject

    status := anObject
!

value

    ^promise value
!

value: anObject

	self promise value: status.
! !

!IMAPCommand methodsFor:'testing'!

failed
	^self successful not
!

isDone

	^ self status = #done
!

isSent

	^ self status = #sent
!

successful

	^ self isDone
		and: [ self completionResponse isOK ].
! !

!IMAPCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPCommand.st,v 1.4 2009/09/29 20:06:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPCommand.st,v 1.4 2009/09/29 20:06:21 cg Exp $'
! !

IMAPCommand initialize!
