"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#IMAPFetchedItemSectionSpecification
	instanceVariableNames:'specName parameters span rawContent'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-IMAP'
!


!IMAPFetchedItemSectionSpecification class methodsFor:'instance creation'!

readFrom: tokenStream

    | specName |

    specName := tokenStream next.
    specName isNil ifTrue: [ specName := 'Empty' ].

    ^(self properSubclassFor: specName) new
	specName: specName ;
	readFrom: tokenStream
! !

!IMAPFetchedItemSectionSpecification class methodsFor:'matching'!

canBe: aName

    ^#(
	'TEXT'
	'MIME'
       ) includes: aName asUppercase
!

defaultClass

    ^IMAPFetchedItemSectionSpecification
!

properSubclassFor: aName

    ^IMAPFetchedItemSectionSpecification withAllSubclasses
	detect: [ : each | each canBe: aName ]
	ifNone: [ self defaultClass ]
! !

!IMAPFetchedItemSectionSpecification methodsFor:'accessing'!

specName
    ^specName
!

specName: aName

    specName := aName
! !

!IMAPFetchedItemSectionSpecification methodsFor:'content'!

extractContentFrom: tokenStream

    "
    Check for a partial fetch- this would include a range specification given in angle brackets.
    Otherwise, there should only be a single token containing the requested content.
    "

    |  peekStream |

    peekStream := tokenStream peek readStream.

    (peekStream peek = $<)
	ifTrue: [ self extractSpannedContentSpanFrom: tokenStream ]
	ifFalse: [ rawContent := tokenStream next ]
!

extractSpannedContentSpanFrom: tokenStream

"we've lost some information- we need the bytecount, but it is gone.  Must revisit this!!"

    | startPoint |

    startPoint := (tokenStream next readStream
	next ;
	upTo: $>) asNumber.

    rawContent := tokenStream next.

"we're going to try to simply use the length of the raw content as the span length- however, this is not actually correct, though it is close."

    span := (startPoint) @ (rawContent size)
!

rawContent

    ^rawContent
! !

!IMAPFetchedItemSectionSpecification methodsFor:'instance creation'!

readFrom: tokenStream

    "
    The section spec will be either numeric (if the message is MIME this is oK) or one of the following:
	'HEADER'
	'HEADER.FIELDS'
	'HEADER.FIELDS.NOT'
	'MIME'
	'TEXT'

    Some examples would be:

    1
    1.HEADER

    HEADER
    HEADER.FIELDS

    3.2.3.5.HEADER.FIELDS (to fetch header fields for part 3.2.3.5)
    "

"the numeric part could be pulled out at this point as the position spec, followed by the section spec, then followed by optional? parameters."

    "positionSpec := ?"

    parameters := tokenStream next
! !

!IMAPFetchedItemSectionSpecification methodsFor:'span'!

pvtFullSpan

    ^0 to: (self rawContent size)
!

span

    "Items are not always requested in their entirety.  The span tells us which part of the desired content was retrieved."

    ^span notNil
	ifTrue: [ span ]
	ifFalse: [ self pvtFullSpan ]
!

span: anInterval

    "Items are not always requested in their entirety.  The span tells us which part of the desired content was retrieved."

    span := anInterval
! !

!IMAPFetchedItemSectionSpecification class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPFetchedItemSectionSpecification.st,v 1.1 2004/11/12 10:43:09 penk Exp $'
! !
