"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Mailbox subclass:#IMAPMailbox
	instanceVariableNames:'name'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-IMAP'
!

!IMAPMailbox class methodsFor:'documentation'!

examples
    |mailBox headers queue user|

    queue := Queue new.
    mailBox := self new.
    user := NetUser username: 'penk' password: 'schnuppe'.
    mailBox
            user: user;
            server: 'mail.bh.exept.de';
            deliveryQueue: queue;
            loadSettings.
    headers := mailBox allHeaders.
    ^ headers.

    "
     self examples
    "
! !

!IMAPMailbox class methodsFor:'debugging'!

debugLabelsAndValues
        ^List with: ('IMAP Mailbox Check'-> #IMAPMailboxCheck)
              with: ('IMAP Mailbox Download'-> #IMAPMailboxDownload).
!

menuItemName

	^'IMAPMailbox Messages'
! !

!IMAPMailbox methodsFor:'accessing'!

name
        ^name isNil
                ifTrue: [name := 'Inbox']
                ifFalse: [name]
!

name: aValue

	name := aValue
!

serverType
	^'IMAP4'
! !

!IMAPMailbox methodsFor:'commands'!

allHeaders
        | numbers headers|
        ^self
                safelyExecute:
                        [ 
                          (numbers := self messageNumbersFor: 'all') isEmpty
                                ifTrue: [ numbers ]
                                ifFalse:
                                        [connection select: self name.
                                        headers := connection fetchRFC822MessageHeaders: numbers.
                                        headers collect:
                                                [:header |
                                                MailIncrementNotification raiseSignal.
                                                MimeEntity readFrom: header value first readStream
                                                ]]]
!

allMessages
        | numbers headers|
        ^self
                safelyExecute:
                        [ 
                          (numbers := self messageNumbersFor: 'all') isEmpty
                                ifTrue: [ numbers ]
                                ifFalse:
                                        [connection select: self name.
                                        headers := connection fetchRFC822Messages: numbers.
                                        headers collect:
                                                [:header |
                                                MailIncrementNotification raiseSignal.
                                                MimeEntity readFrom: header value first readStream
                                                ]]]
!

allUnseenMessages
        | numbers headers|
        ^self
                safelyExecute:
                        [ 
                          (numbers := self messageNumbersFor: 'unseen') isEmpty
                                ifTrue: [ numbers ]
                                ifFalse:
                                        [connection select: self name.
                                        headers := connection fetchRFC822Messages: numbers.
                                        headers collect:
                                                [:header |
                                                MailIncrementNotification raiseSignal.
                                                MimeEntity readFrom: header value first readStream
                                                ]]]
!

anyNewMail
	"Any new mail to get?"

	self log: ['Checking <1s> for mail' expandMacrosWith: server] level: #IMAPMailboxCheck.
	^ (self safelyExecute: [ connection unseenMessages ]) size > 0
!

deleteMessageNumbers: aSequenceOfUIDs
	"Delete all the letters on the server.  Ignore non-existent ones"
	| res |
	res := self safelyExecute: [connection searchMessagesForUids: aSequenceOfUIDs ].
	self safelyExecute: [connection markForDelete: res; expunge. ].
	^self messageCount.
!

getMessage: number
	"Get the <number> message from the IMAP server"
	| coll |
	^(coll := self getMessagesByNumbers: (Array with: number)) notEmpty
		ifTrue: [ coll first ]
		ifFalse: [nil ]
!

messageCount
	"How many messages are on the server?"

	^self safelyExecute:    [connection messageCount]
!

messageNumbersFor: criteria
        "How many messages are on the server?"
        ^self safelyExecute: [connection searchMessages: criteria].
!

unseenMessages
        "Any new mail to get?"

        self log: ['Checking <1s> for mail' expandMacrosWith: server] level: #IMAPMailboxCheck.
        ^ (self safelyExecute: [ connection unseenMessages ]) 
! !

!IMAPMailbox methodsFor:'debugging'!

logMessages: messages position: position from: total time: downloadTime

	| n messageSize |
	n := position.
	messages size
		timesRepeat:
			[n := n + 1.
			n \\ 10 = 0
				ifTrue: [self log: ['dowloaded <1p> messages from <2s>' expandMacrosWith: n with: server]
						level: #IMAPMailboxDownload]].
	self
		debug: [messageSize := messages inject: 0 into: [:sum :message | sum + message size].
			self class processTransferRate: downloadTime ofSize: messageSize fromServer: server]
		level: #IMAPTransferRate
! !

!IMAPMailbox methodsFor:'mail delivery'!

deliverAllMail
	"Queue up all new messages onto the queue. Clients of this
	method should be aware that the state should be such that there
	is new mail to download."

	| numbers |
	numbers := self messageNumbersFor: 'all'.
	numbers size > 0
		ifTrue: [self deliverMessages: numbers ]
!

deliverNewMail
	"Queue up all new messages onto the queue. Clients of this
	method should be aware that the state should be such that there
	is new mail to download."

	| numbers |
	numbers := self messageNumbersFor: 'unseen'.
	numbers size > 0
		ifTrue: [self deliverMessages: numbers ]
! !

!IMAPMailbox methodsFor:'private'!

clientSpecies
	^IMAPClient
!

createConnection
    |executedCommand|

    super createConnection.
    executedCommand := connection select: self name.
    executedCommand successful ifTrue:[
        ^ connection
    ] ifFalse:[
        self logout.
        NetClientError
                signalWith: #select
                message: 'Unable to select the mailbox:  ', self name.
    ].
!

deliverMessages: numbers
	"Queue up all new messages onto the queue. Clients of this
	method should be aware that the state should be such that there
	is new mail to download."

	| messages bundlingFactor n stream msec subSet |

	numbers isEmpty ifTrue: [^self].

	bundlingFactor := 5.
	n := 0.
	self
		safelyExecute:
			[self log: ['Dowloading <1p> messages from <2s>' expandMacrosWith: numbers size with: server]
				level: #IMAPMailboxDownload.
			stream := numbers readStream.
			[stream atEnd]
				whileFalse:
					[subSet := stream nextAvailable: bundlingFactor.
					msec := Time millisecondsToRun:
								[messages := self getMessagesByNumbers: subSet ].
					self logMessages: messages position: n from: numbers size time: msec.
					messages do: [ :ms | self deliver: ms].
					n := n + messages size]].
	self lastMessage: (self lastMessage + n).
	self log: ['Done; dowloaded <1p> messages from <2s>' expandMacrosWith: n with: server]
		level: #IMAPMailboxDownload
!

getMessageUID: arrayOfMessageNumbers
" Read specified messages from IMAP server. Returns an arrray strings representing message uids "

	^self safelyExecute: [ connection uidFor: arrayOfMessageNumbers ].
!

getMessagesByNumbers: arrayOfMessageNumbers
" Read specified messages from IMAP server. arrayOfMessageNumbers is
an array of integers or strings representing message numbers "
        | fetchedMessages messages uids n |
        fetchedMessages := self safelyExecute: [ connection fetchRFC822Messages: arrayOfMessageNumbers ].
        uids := self getMessageUID: arrayOfMessageNumbers.
        messages := List new.
        n := self lastMessage.
        fetchedMessages notEmpty
                ifTrue:
                        [ fetchedMessages
                                doWithIndex: [ :each :index |
                                        n := n + 1.
                                        messages add:
                                                (LetterInTransit new
                                                                        letterText: each value first;
                                                                        uid: (uids at: index);
                                                                        server: self server;
                                                                        serverType: self serverType;
                                                                        deliveryAction: [self saveSettings: n])]].
        ^messages
!

getMessagesFor: criteria
	"Read messages matching specified criteria from the mailbox"

	| numbers |
	numbers := self messageNumbersFor: criteria.
	^self getMessagesByNumbers: numbers
!

handleResult: result
	result successful ifFalse: [ self error: 'to do: implement exceptions' ].
	^result.
!

notifyServerMessageRead: uid
	| res |
	res := self safelyExecute: [connection searchMessagesForUids: (Array with: uid) ].
	^self safelyExecute: [ connection markAsSeen: res ].
! !

!IMAPMailbox class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPMailbox.st,v 1.7 2010/03/09 12:10:01 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPMailbox.st,v 1.7 2010/03/09 12:10:01 cg Exp $'
! !
