"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

TestCase subclass:#IMAPTest
	instanceVariableNames:'client'
	classVariableNames:'Password'
	poolDictionaries:''
	category:'NetClients-IMAP'
!

!IMAPTest class methodsFor:'documentation'!

documentation
"
    Please see
        http://www.linux-magazin.de/Heft-Abo/Ausgaben/2007/06/Facharbeiten-mit-Verstand
"
! !

!IMAPTest methodsFor:'Running'!

getPassword
    "
     Password := nil
    "
    Password isNil ifTrue:[
        Password := Dialog request:'Password for cg on exeptn:'.
    ].
    ^ Password
!

getUser
    "
     Password := nil
    "
    ^ 'cg'
!

imapServer
    "
     Password := nil
    "
    ^ 'exeptn'
!

login

    "establish a socket connection to the IMAP server and log me in"
    client := IMAPClient loginToHost:  'exeptn' asUser:(self getUser) withPassword:(self getPassword).
    self assert: (client isKindOf: IMAPClient).

   client create: 'inbox.text'.
"/    client create: 'test'.
"/    client create: 'user.test.1'.

    "
     self new login
    "
!

logout

    client disconnect.
!

mailBoxName
    ^ 'inbox.test'.
    ^ 'user.test'
! !

!IMAPTest methodsFor:'test-client'!

testAppend
    | message |
    self login.
    message :=
'Date: Mon, 7 Nov 2004 21:52:25 -0800 (PST)
From: Christian Penk <penk@exept.de>
Subject: IMAPTest
To: test@exept.de
Message-Id: <B27397-0100000@exept.de>
MIME-Version: 1.0
Content-Type: TEXT/PLAIN; CHARSET=US-ASCII

Hello 
this is a test message from stx imap client'.

    client append: message to: (self mailBoxName).
    self logout

"
    self new testAppend
"
!

testCreateRenameDelete
    | comm box box1 |

    box := 'inbox.mybox'.
    box1 := 'inbox.myBoxRenamed'.
    self login.
    [comm := client create: box.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: comm completedSuccessfully.
    comm := client rename: box newName: box1.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: comm completedSuccessfully]
        ensure:
            [client delete: box1. self logout]
"
    self new testCreateRenameDelete
"
!

testExamine
    | box comm |
    self login.
    box := 'inbox'.
    comm := client examine: box.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: (comm completedSuccessfully).
    self logout

"
    self new testExamine
"
!

testList
    | box comm |
"    box := nil.
    box isNil ifTrue:[ ^nil].
"
    self login.
    [
        box := 'Inbox.news'.
        comm := client create: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client list: 'Inbox.'  mailbox: 'news' .
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).
        self assert: (((comm responses first value at:3) asUppercase) = box asUppercase).
    ] ensure: [
        comm := client delete: box.
    ].
    self logout


"
    self new testList
"
!

testListAllMailBoxes

    self login.
    client list:'*'  mailbox: '*'.
    self logout.

"
    self new testListAllMailBoxes
"
!

testLogin
    self login.

"
    self new testLogin
"
!

testNoopCapability
    | comm |
    self login.
    comm := client noop.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: (comm completedSuccessfully).
    comm := client capability.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: (comm completedSuccessfully).
    self logout

"
    self new testNoopCapability
"
!

testSelectCheck
    | box comm |
"    box := nil.
    box isNil ifTrue:[ ^nil].
"
    self login.
    [
        box := (self mailBoxName),'.new'.
        comm := client create: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client select: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client check.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

    ] ensure: [
        comm := client delete: box.
    ]

"
    self new testSelectCheck
"
!

testSelectClose
    | box comm |
"    box := nil.
    box isNil ifTrue:[ ^nil].
"
    self login.
    [
        box := (self mailBoxName),'.box'.
        comm := client create: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client select: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client close.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

    ] ensure: [
        comm := client delete: box.
    ]

"
    self new testSelectClose
"
!

testSelectExpunge
" Test case doesn't return untagged response: EXPUNGE as expected"
    | box comm |
"    box := nil.
    box isNil ifTrue:[ ^nil].
"
    self login.
    box := 'inbox' asString.
        comm := client select: box.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully).

        comm := client expunge.
        self assert: (comm isKindOf: IMAPCommand).
        self assert: (comm completedSuccessfully)

"
    self new testSelectExpunge
"
!

testSelectFetch

    | box comm |
    self login.
    box := (self mailBoxName).
    client select: box.
"/    comm := client fetch: '2:3 (flags internaldate uid RFC822)'.
    comm := client fetch: '1:4 full'.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: (comm completedSuccessfully).



"    comm := client fetch: '2,4 (flags internaldate uid BODY.PEEK[header])'."
"    client fetch: '1:4 (uid Body.Peek[Header.Fields (Subject Date From Message-Id)])'."
"    client fetch: '1:2 (flags internaldate uid RFC822)'."
"    client fetch: '1 (Body.Peek[header])'."
"    comm := client fetch: '3 (BodyStructure)'."


"    client fetch: '2 full'."
    self logout

"
    self new testSelectFetch
"
!

testSelectFetch1

    | box comm |
    self login.
    box := (self mailBoxName).
    client select: box.
    client fetch: '1:4 (uid Body.Peek[Header.Fields (Subject Date From Message-Id)])'.
    comm := client fetch: '1:* body[text]'.
    self assert: (comm isKindOf: IMAPCommand).
    self assert: (comm completedSuccessfully).



"    comm := client fetch: '2,4 (flags internaldate uid BODY.PEEK[header])'."
"    client fetch: '1:4 (uid Body.Peek[Header.Fields (Subject Date From Message-Id)])'."
"    client fetch: '1:2 (flags internaldate uid RFC822)'."
"    client fetch: '1 (Body.Peek[header])'."
"    comm := client fetch: '3 (BodyStructure)'."


"    client fetch: '2 full'."
    self logout

"
    self new testSelectFetch1
"
!

testSelectFetch2

    | box result|
    self login.
    box := (self mailBoxName).
    client select: box.
    result := client fetchMessage:3.
    self logout


"
    self new testSelectFetch2
"
!

testSelectSearch
    | box   |

"    box := nil.
    box isNil ifTrue: [ ^box].
"

    self login.
    box := 'inbox'.
    client select: box.

    client search: 'undeleted unanswered from "Kogan, Tamara"'.

    self logout

"
    self new testSelectSearch
"
!

testSelectStore
    | box |

    self login.
    box := (self mailBoxName).
    self assert: ((client select: box) successful).
    self assert:((client markForDelete: (Array with:1))).
    self assert:((client store: ('1:1 -FLAGS (\Deleted)')) successful).
    self logout.

"
    self new testSelectStore
"
!

testSelectUnseen
    | box |

    self login.
    box := (self mailBoxName).
    self assert: ((client select: box) successful).
    self assert:((client store: ('19 -FLAGS (\Seen)')) successful).

    self logout.

"
    self new testSelectUnseen
"
! !

!IMAPTest methodsFor:'test-mailbox'!

testIMAPMailbox
    |mailBox headers queue user anyNewMail message messageCount|

    self login; logout.

    queue := Queue new.
    mailBox := IMAPMailbox new.
    mailBox name: (self mailBoxName).
    user := NetUser username: (self getUser) password: (self getPassword).
    mailBox
            user: user;
            server: (self imapServer);
            deliveryQueue: queue;
            loadSettings.
    headers := mailBox allHeaders.
    messageCount := mailBox messageCount.
    anyNewMail := mailBox anyNewMail.
    message := mailBox getMessage: 1.
"/    mailBox deleteMessageNumbers:#('5').
"
    self new testIMAPMailbox
"
!

testIMAPMailboxGetMessagesFor
    |mailBox queue user messages|

    queue := Queue new.
    mailBox := IMAPMailbox new.
    mailBox name: (self mailBoxName).
    user := NetUser username: (self getUser) password: (self getPassword).
    mailBox
            user: user;
            server: (self imapServer);
            deliveryQueue: queue;
            loadSettings.
    messages := mailBox getMessagesFor: 'unseen'.
"/    messages halt.
"
    self new testIMAPMailbox
"
! !

!IMAPTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPTest.st,v 1.12 2009/07/20 12:09:17 cg Exp $'
! !
