"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

SimpleScanner subclass:#INIScanner
	instanceVariableNames:''
	classVariableNames:'TokenMask CommentChars'
	poolDictionaries:''
	category:'NetClients-Base'
!


!INIScanner class methodsFor:'class initialization'!

initClassificationTable

      super initClassificationTable.
       self initClassificationTableWith: TokenMask when:
	   [:c | c > Character space and: [ ('[]=' includes: c) not ] ].
       self initClassificationTableWith: EndOfLineMask when:
	   [:c | c == Character return or: [ c == Character lf ] ].
!

initialize
	"INIScanner initialize"

	self initializeConstants; initClassificationTable
!

initializeConstants
	CommentChars := Array with: Character return with: $# with: Character lf.
	TokenMask := 16384.
! !

!INIScanner methodsFor:'private'!

nextIniToken
	| char |

	self skipWhiteSpace.
	char := self peek.
	char isNil              "end of input"
		ifTrue: [       tokenType := #doIt.
			^ token := nil].
	token := self scanToken.

	" Tokens must be arranged in name = value pairs.
		where value is allowed to be empty. "
	token isEmpty
		ifTrue: [       tokenType := #none.
			^ token := nil].
	^ token.
! !

!INIScanner methodsFor:'tokenization'!

nextNVPair
	| pos key |

	self skipComments.
	pos := self position.
	key := self nextToken.
	key notNil ifTrue: [
		self skipWhiteSpace.
		(self peekFor: $= )
			ifTrue: [       ^ key -> (self skipWhiteSpace; nextLine) trimBlanks ].
		self position: pos.
		self warnOnSkippedLine ].
	^ nil.
!

nextSection

	| key |
	(key := self nextSectionHeader) notNil
		ifTrue: [ ^ key -> self scanSectionBody ].
	^ nil.
!

nextSectionHeader

	"Section headers must start at the beginning of the line."
	[self atEnd not and: [self peek ~~ $[]]
		whileTrue: [ self nextLine ].

	self atEnd ifTrue: [
		tokenType := #doIt.
		^ token := nil ].

	token := self
		scanToken: [ self matchCharacterType: TokenMask ]
		delimitedBy: '[]'
		notify: (#errMalformedSectionHeader << #dialogs >> 'Malformed section header').
	self nextLine.
	^ token.
!

nextToken

	^self nextIniToken
!

scanSectionBody
	"Answer a Dictionary containing the section's key/value pairs.
	 Empty value is allowed.
	 No protection against duplicate keys - last entry in stream is installed."
	| section nv |

	section := Dictionary new.
	[self skipComments.
	self atEnd not and: [self peek ~~ $[]]
		whileTrue: [
			(nv := self nextNVPair) notNil
				ifTrue: [section add: nv].
			self skipComments ].
	^ section.
!

scanToken
	" token  =  1*<any CHAR except $[, $], $=, SPACE and CTLs>. "

	token := self scanTokenMask: TokenMask.
	tokenType := #token.
	^token
!

sectionNamed: aString

	| name |
	[ name := self nextSectionHeader.
	name notNil and: [name asLowercase ~= (aString asLowercase)] ] whileTrue.
	^ self scanSectionBody.
!

skipComments

	self skipWhiteSpace.
	[CommentChars includes: self peek]
		whileTrue: [self nextLine].
!

tokenizeAsDictionary
	| ini section |

	ini := Dictionary new.
	[ self atEnd ]
		whileFalse: [
			(section := self nextSection) notNil ifTrue: [ ini add: section ] ].
	^ ini.
!

warnOnSkippedLine

	| skipLine |
	skipLine := self nextLine.
	Transcript cr; show: 'Scanned line ''', skipLine, ''' skipped.'.
! !

!INIScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__INIScanner.st,v 1.1 2004/11/12 10:46:55 penk Exp $'
! !

INIScanner initialize!
