"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

NetworkEntityDescriptor subclass:#MailGroupDescriptor
	instanceVariableNames:'addresses'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MailGroupDescriptor comment:'This class represents mail group specification as defined in rfc822. It is capable of parsing and composing group specs to/from a stream and provides access to group components. A group spec may be parsed from a source stream, or created by invoking mutator methods and then written on a stream. Mime-encoded fields are handled transparently

Examples:
    Reply-To: The Committee: Jones@Host.Net,
				     Smith@Other.Org,
				     Doe@Somewhere-Else;

    Reply-To :  Sam.Irving@Reg.Organization
       :  George Jones <Group@Some-Reg.An-Org>,
	  Al.Neuman@MAD.Publisher
       :  Important folk:
	    Tom Softwood <Balsa@Tree.Root>,
	    "Sam Irving"@Other-Host;,
	  Standard Distribution:
		   /main/davis/people/standard@Other-Host,
		   "<Jones>standard.dist.3"@Tops-20-Host>;

Group syntax from rfc822 is given below

     group       =  phrase ":" [#mailbox] ";"

     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec

     route-addr  =  "<" [route] addr-spec ">"

     route       =  1#("@" domain) ":"           ; path-relative

     addr-spec   =  local-part "@" domain        ; global address

     local-part  =  word *("." word)             ; uninterpreted
						 ; case-preserved

     domain      =  sub-domain *("." sub-domain)

     sub-domain  =  domain-ref / domain-literal

     domain-ref  =  atom                         ; symbolic reference



Instance Variables:
	addresses       <SequencableCollection>  Collection of NetworkAddressDescriptors constituting group'
!


!MailGroupDescriptor methodsFor:'accessing'!

addresses
	^addresses
!

addresses: anArray
	addresses := anArray
!

alias
	^alias
!

alias: aString
	alias := aString
! !

!MailGroupDescriptor methodsFor:'composing'!

writeCanonicalValueOn: stream encoding: aString
	self writeAliasOn: stream encoding: aString.
	self printAddressesOn: stream
! !

!MailGroupDescriptor methodsFor:'initialization'!

initialize
	addresses := Array new.
! !

!MailGroupDescriptor methodsFor:'printing'!

printAddressesOn: stream

	stream nextPut: $:.
	self addresses do: [ :address | address printOn: stream ] separatedBy: [ stream nextPut: $, ].
	stream nextPut: $;
!

printCanonicalValueOn: stream
	self printAliasOn: stream.
	self printAddressesOn: stream
! !

!MailGroupDescriptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MailGroupDescriptor.st,v 1.1 2004/11/12 10:44:34 penk Exp $'
! !
