"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

RFC822Message subclass:#MailMessage
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MailMessage comment:'Mail message is a specialization
of generic rfc822 and Mime messages. A mail message has text and zero or more attachments. Mail message''s
text can have a number of different representations as follows:

-Simple message. This message has no attachments and its contents is text of the message. The message
should have text content type (''text/*''); its subtype defines text formatting (''plain'', ''rtf'', ''html'').
-Multipart message. This message has multipart type (''multipart/*''). Multipart message contains zero or more
parts. One of the subparts is the text part. It may be simple or multipart/alternative part.
Multipart may contain multiple alternative representations of message text. This is done by enclosing
all alternative representations into multipart/alternative part.
Mail message itself may be multipart/alternative. In this case the message has no attachments, just alternative
representations of its text. Otherwise, top-level message has type multipart/mixed. The first part of this message
is its text part. It is either simple text part, or multipart/alternative part.

This class exposes methods to access message text (part) and attachments, etc. and to
perform common transformations (add/remove attachment, replace message text, mutate message from
multipart into single part and vice versa) in such a way to preserve message structural consistency. This
includes changing and/or mutating message body, fields and contents

Message text-related methods allow to specify a list of preferences. This is useful when client can display text in more
than one format. Preferences can be specified as array or desired formats i.e. #(''html'' ''plain'' ''*''). The default
is #(''plain'' ''*'')

MailMessage implements little logic of its own. Most functions are actually performed by an associated ''bot'' --
a MailMessageBot'
!


!MailMessage class methodsFor:'constants'!

defaultBuilderType
	^MailBuildHandler
! !

!MailMessage methodsFor:'accessing'!

allAlternativeTextFormats
" Answer all alternative text representation formats available in this message "
	^(self bot allTextPartsFor: self) collect: [ :each | each subtype ]
!

allTextParts
	^self bot allTextPartsFor: self
!

bccSendStrings
	^self encodedStringAt: 'bcc'
!

ccSendStrings
	^self encodedStringAt: 'cc'
!

dateSendString
	^self encodedStringAt: 'cc'
!

fromSendStrings
	^self encodedStringAt: 'from'
!

inReplyToSendString
	^self encodedStringAt: 'in-reply-to'
!

replyToSendStrings
	^self encodedStringAt: 'reply-to'
!

resentBccSendStrings
	^self encodedStringAt: 'resent-bcc'
!

resentCcSendStrings
	^self encodedStringAt: 'resent-cc'
!

resentFromSendStrings
	^self encodedStringAt: 'resent-from'
!

resentReplyToSendStrings
	^self encodedStringAt: 'resent-reply-to'
!

resentToSendStrings
	^self encodedStringAt: 'resent-to'
!

subjectSendString
	^self encodedStringAt: 'subject'
!

text
	^self bot textFor: self
!

text: aString
	^self bot replaceTextFor: self with: aString
!

textInPreferenceOrder: preferences
	^self bot textFor: self preferences: preferences
!

textPart
	^self bot textPartFor: self
!

textPartInPreferenceOrder: preferences
	^self bot textPartFor: self preferences: preferences
!

toSendStrings
	^self encodedStringAt: 'to'
! !

!MailMessage methodsFor:'composing'!

writeOn: aStream
	self bot checkStructure: self.
	super writeOn: aStream
! !

!MailMessage methodsFor:'defaults'!

defaultBot
	^MailMessageBot new
! !

!MailMessage methodsFor:'send'!

send

	NetClient defaultOutgoingMailClient
			sendMessage:  self
! !

!MailMessage methodsFor:'utility'!

applyTransferEncoding
	^self bot applyTransferEncodingTo: self
!

checkStructure
	^self bot checkStructure: self
!

prepareForTransport
	self checkStructure.
	self applyTransferEncoding.
!

replaceTextWith: aString
	^self bot replaceTextFor: self with: aString
!

saveTextOn: aStream
	^self bot writeContentsOf: self textPart on: aStream
!

saveTextOn: aStream inPreferenceOrder: preferences
	^self bot writeContentsOf: (self textPartInPreferenceOrder: preferences) on: aStream
! !

!MailMessage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MailMessage.st,v 1.1 2004/11/12 10:41:38 penk Exp $'
! !
