"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#MessageElement
	instanceVariableNames:'source parent value'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-Base'
!


!MessageElement class methodsFor:'instance creation'!

new
        ^self basicNew initialize
! !

!MessageElement class methodsFor:'parsing'!

fromLine: aString
" For compatibility with Swazoo "
	self subclassResponsibility
!

readFrom: aStream
" Each message element has responsibility to read itself from input stream. Reading usually involves parsing, so implementations of this method create an instance of lexical scanner and invoke a parser (see explanation for parse: method) "

	^self new readFrom: aStream
!

scannerOn: aStream
	^(aStream isKindOf: self scannerType)
		ifTrue: [aStream]
		ifFalse: [self scannerType on: aStream asStream]
!

scannerType
	self subclassResponsibility
! !

!MessageElement methodsFor:'accessing'!

clearSource
	source := nil
!

parent
	^parent
!

parent: anObject
	parent := anObject
!

source
	^source
!

source: anObject
	source := anObject
!

sourceContents
	^self source contents
!

sourceSize
	^self source size
!

value
	^self getValue
!

value: anObject
	value := anObject
! !

!MessageElement methodsFor:'composing'!

sendOn: aStream
	self writeOn: aStream
!

writeOn: aStream
" Writing can be different from printing. Printing may be indented, abbreviated
and otherwise formatted for readability. Writing is used to actually send message.
Default implementation is the same "

	self printOn: aStream
! !

!MessageElement methodsFor:'copying'!

postCopy
	super postCopy.
	parent := nil.
! !

!MessageElement methodsFor:'initialization'!

initialize
! !

!MessageElement methodsFor:'parsing'!

parse
	^self parse: (self scannerOn: self source)
!

parse: scanner
" Each message element has responsibility to parse itself. The argument is an appropriate scanner. Scanners for RFC822, Mime and HTTP messages are stream wrappers, so they can be used to read and tokenize input stream "

	self subclassResponsibility
!

parseBody: scanner

	self subclassResponsibility
!

readBodyFrom: aStream
" Each message element has responsibility to read itself from input stream. Reading usually involves parsing, so implementations of this method typically create an instance of lexical scanner and invoke a parser (see explanation for parse: method) "

	self source: aStream.
	^self parseBody: (self scannerOn: aStream)
!

readFrom: aStream
" Each message element has responsibility to read itself from input stream. Reading usually involves parsing, so implementations of this method typically create an instance of lexical scanner and invoke a parser (see explanation for parse: method) "

	self source: aStream.
	^self parse: (self scannerOn: aStream)
!

scannerOn: aStream
" Each element should know what the underlying syntax is. For example, structured fields would mostly use MIME syntax and tokenize input streams into MIME 'tokens' while <address-spec> which is part of many standards, has to be tokenized using RFC822 syntax (using RFC822 'atoms') "
	^self class scannerOn: aStream
!

valueFrom: aString
" Swazoo compatibility"

	^self readFrom: aString readStream
! !

!MessageElement methodsFor:'printing'!

printLine: aString on: aStream indent: level
	aStream
		tab: level;
		nextPutAll: aString;
		cr
!

printOn: aStream
        self printOn: aStream indent: 0.
!

printOn: aStream indent: level
	aStream tab: level
!

storeOn: aStream
	self printOn: aStream
! !

!MessageElement methodsFor:'private'!

doParse: rfc822Stream
	self subclassResponsibility
!

getValue

	^(value isNil and: [self source notNil])
		ifTrue: [[value := self doParse: (self scannerOn: self source)]
				ensure: [self source close]]
		ifFalse: [value]
! !

!MessageElement methodsFor:'testing'!

isAddressDescriptor
	^false
! !

!MessageElement methodsFor:'visiting'!

acceptVisit: aVisitor with: argument
" Override by invoking appropriate message on the visitor "
! !

!MessageElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageElement.st,v 1.3 2010/06/14 10:53:00 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageElement.st,v 1.3 2010/06/14 10:53:00 cg Exp $'
! !
