"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#MessageHandler
	instanceVariableNames:'dispatcher context headerOnly'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessageHandler comment:'This class is strategy class for MessageDispatcher. When message dispatcher traverses message, it calls back its message handler notifying it where it is in the message. All possible callbacks are declared in ''action callbacks'' protocol. Subclasses override these methods to implement such tasks as building, printing and searching of messages.
The principle and division of work is largely similar to SAX driver for XML.

Because of callback nature, handler usually has to maintain some kind of context. Since messages can be nested, context should reflect this. See methods in ''accessing''.
	#context answers global context, which is an IdentityDictionary.
	#entityContext answers context of the current entity. new entity context is pushed on the context stack when new entity is visited (#startEntity: callback) and popped when this entity is left (#endEntity: callback).
	#entity answers current entity
	#level answers nesting level

''dispatcher interface'' is low-level interface between handler and dispatcher.

Instance Variables:

	dispatcher      <MessageDispatcher>     my dispatcher. Handler typically initialtes work by invoking dispatcher''s method visit:. Dispatcher then executes series of callbacks to the hahdler
	context <IdentityDictionary>    contains context of the work being done (see above)
	headerOnly      <Boolean>       if true, dispatcher will stop after processing message header'
!


!MessageHandler class methodsFor:'instance creation'!

new
	^super new initialize
! !

!MessageHandler methodsFor:'accessing'!

context
	^context
!

dispatcher
	^dispatcher isNil
		ifTrue: [ dispatcher := self defaultDispatcher ]
		ifFalse: [ dispatcher ]
!

dispatcher: aMessageDispatcher
	aMessageDispatcher handler: self.
	dispatcher := aMessageDispatcher.
!

entity
	^self entityContext at: #entity ifAbsentPut: [ nil ]
!

entity: aMimeEntity
	self entityContext at: #entity put: aMimeEntity
!

entityContext
	^self entityContextList notEmpty
		ifTrue: [self entityContextList first ]
!

entityContextAt: aSymbol ifAbsent: aNiladicBlock
" Answer specified entity context element. If not found, try global context.
This failing, answer result of the niladic block. Normally, the same element should not appear in both entity and global context but it may be convenient in certain scenarios "

	^self entityContext
		at: aSymbol
		ifAbsent: [ self context at: aSymbol ifAbsent: aNiladicBlock ]
!

entityContextAt: aSymbol put: anObject
	self entityContext at: aSymbol  put: anObject
!

headerOnly
	^headerOnly
!

headerOnly: aBoolean
	headerOnly := aBoolean
!

level
	^self entityContextList size - 1
!

parentContext
	^self entityContextList size > 1
		ifTrue: [self entityContextList at: 2]
		ifFalse: [nil]
! !

!MessageHandler methodsFor:'action callbacks'!

endBody: aMessageBody
	^true
!

endBodyPart: aMimeEntity
!

endEntity: aMimeEntityOrNil
	| entity |
	entity := self entity.
	self popEntityContext.
	^entity
!

endHeader: aMessageHeader
	^self entity
!

getSimpleBodyStream: aSimpleBody
	^aSimpleBody source
!

startBody: aMimeEntity
	^self headerOnly not
!

startBodyPart: aMimeEntity
	^true
!

startBodyStream: aStream
	^self startBodyStream: aStream length: nil
!

startBodyStream: aStream length: anIntegerOrNil
!

startEntity: aMimeEntityOrNil
	self
		pushEntityContext: aMimeEntityOrNil;
		entity: aMimeEntityOrNil.
	^true
!

startHeader: aMessageHeader
	^true
! !

!MessageHandler methodsFor:'defaults'!

defaultDispatcher
	^self dispatcherClass handler: self
!

dispatcherClass
	^MessageDispatcher
! !

!MessageHandler methodsFor:'dispatcher interface'!

action: aSymbol with: argument
	^self perform: aSymbol with: argument
!

action: aSymbol with: argument with: argument2
	^self perform: aSymbol with: argument with: argument2
!

action: aSymbol withArguments: argumentArray
	^self perform: aSymbol withArguments: argumentArray
!

visit: anObject
	self dispatcher visit: anObject
! !

!MessageHandler methodsFor:'initialization'!

initialize
	context := IdentityDictionary new.
	headerOnly := false.
	self initializeEntityContext
!

initializeEntityContext
	self context removeKey: #entityContextList ifAbsent: [ ]
! !

!MessageHandler methodsFor:'private'!

entityContextList
	^self context at: #entityContextList ifAbsentPut: [List new: 1].
!

popEntityContext
	^self entityContextList removeFirst
!

pushEntityContext
	^self entityContextList addFirst: (IdentityDictionary new: 4)
!

pushEntityContext: aMimeEntity
	self pushEntityContext; entity: aMimeEntity
! !

!MessageHandler methodsFor:'testing'!

isRootContext
	^self parentContext isNil
! !

!MessageHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageHandler.st,v 1.1 2004/11/12 10:43:59 penk Exp $'
! !
