"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

SimpleScanner subclass:#MimeEncodedWordCoDec
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MimeEncodedWordCoDec comment:'I am responsible for scanning tokens for the presence of MIME ''encoded words''. MIME uses encoded word to allow non-ascii characters to be used in message headers. Encoded words can occur inside MIME extension fields (ones starting with X-) as well as in field bodies. An encoded word may occur everywhere in the body in place of text'', ''word'', ''comment'' or ''phrase'' token. Encoded word specifies charset, encoding mechanism and encoded text itself'
!


!MimeEncodedWordCoDec class methodsFor:'parsing'!

decode: word
	^self decode: word using: (self encodingParametersOf: word).
!

decode: word using: arr
	^arr notNil
		ifTrue: [self
				decodeEncodedWord: (arr at: 3)
				charset: arr first
				encoding: (arr at: 2)]
		ifFalse: [word]
!

decodeComment: commentString
	^self new decodeComment: commentString
!

decodePhrase: words
" decode phrase word by word; concatenate decoded words and answer concatenated string "
	| output |
	output := (String new: words size) writeStream.
	self decodePhrase: words printOn: output.
	^output contents
!

decodePhrase: words printOn: stream
	| params lastParams lastWord |
	lastWord := nil.
	words do:
		[:word |
			lastParams := params.
			params := self encodingParametersOf: word.
			(lastWord notNil and: [params isNil or: [lastParams isNil]])
				ifTrue: [stream space].
			stream nextPutAll: (lastWord := self decode: word using: params)].
!

decodeText: text
	^self new decodeText: text
!

encodingParametersOf: word
	| mark1 mark2 |
	word isEmpty ifTrue: [^nil].
	^(word first == $= and: [word last == $= and:
				[(word at: 2) == $? and: [(word at: word size - 1) == $? and:
				[(mark1 := word         nextIndexOf: $?         from: 3 to: word size - 2) > 0 and:
				[(mark2 := word         nextIndexOf: $?         from: mark1 + 1 to: word size - 2) > (mark1 + 1)]]]]])
		ifTrue: [Array
					with: (word copyFrom: 3 to: mark1 - 1) asLowercase
					with: (word copyFrom: mark1 + 1 to: mark2 - 1) asLowercase
					with:  (word copyFrom: mark2 + 1 to: word size - 2)]
		ifFalse: [nil ]
! !

!MimeEncodedWordCoDec class methodsFor:'queries'!

encodeText:aString  asBase64Using:encoding
    "encode a string"

    |lineCounter inStream outStream|

    encoding = 'base64' ifTrue:[
        ^ Base64Coder encode:aString.
    ].

    "we do not really want quoted-printable for RFC822 headers"
    "kludge for now"

    ^ aString.


"/    lineCounter := 0.
"/
"/    inStream := aString readStream.
"/    outStream := WriteStream on:(String new:aString size + 20).
"/
"/    [inStream atEnd] whileFalse:[
"/        |char cp|
"/
"/        char := inStream next.
"/        (char == Character return and:[inStream peek == Character lf]) ifTrue:[
"/            outStream nextPut:char; nextPut:inStream next.
"/            lineCounter := 0.
"/        ] ifFalse:[
"/            cp := char codePoint.
"/
"/            lineCounter > 73 ifTrue:[
"/                outStream nextPut:$=; return; lf.
"/                lineCounter := 0.        
"/            ].
"/
"/            (cp >= 127 or:[cp < 32 or:[cp == 61]]) ifTrue:[
"/                outStream nextPut:$=.
"/                cp printOn:outStream base:16 size:2 fill:$0.
"/                lineCounter := lineCounter + 3.
"/            ] ifFalse:[
"/                outStream nextPut:char.
"/                lineCounter := lineCounter + 1.
"/            ].
"/        ].                               
"/    ].
"/
"/    ^ outStream contents
!

shouldBeEncoded:aString
    ^ aString contains:[:eachChar| |cp|
        cp := eachChar codePoint.
        cp >= 127 or:[cp <= 31 or:[cp == 61]]
    ].
! !

!MimeEncodedWordCoDec class methodsFor:'text processing'!

decodeEncodedWord: contents charset: charset encoding: encodingString
	| encoding outStream |
	encoding := encodingString asLowercase.
	(#('b' base64) includes: encoding)
		ifTrue: " Decode into bytes, then impose desired charset encoding on the result "
			[outStream := (ByteArray new: contents size * 2 // 3 + 1) writeStream.
			MimeScanner decodeBase64From: contents readStream to: outStream.
			^(outStream contents
				withEncoding: (MimeTypeDescriptor encodingForCharset: charset))
				readStream contents].
	(#('q' 'quoted-printable') includes: encoding) ifTrue: [^MimeScanner
			decodeQuotedPrintableFrom: 1
			to: contents size
			in: contents].
	(#('uue' 'uuencode' 'x-uue' 'x-uuencode') includes: encoding)
		ifTrue: [^MimeScanner
				decodeUUEncodedFrom: 1
				to: contents size
				in: contents].
	^nil.                                                                   " Failed to decode "
! !

!MimeEncodedWordCoDec methodsFor:'parsing'!

decode: word
	^self class decode: word
!

decodeComment: text
	| output word params spaces lastParams lastWord |

	" First, quick check if we possibly have an encoded word "
	(text indexOfSubCollection: '=?' startingAt: 1)
		= 0 ifTrue: [^text].    "We suspect there might be an encoded word inside, do the legwork"

	self on: text readStream.
	output := (String new: text size) writeStream.
	spaces := String new.
	params := lastWord := nil.

	[lastParams := params.
	self atEnd]
		whileFalse:
			[word := self scanWhile: [(self matchCharacterType: WhiteSpaceMask) not].
			params := self class encodingParametersOf: word.
			(lastWord notNil and: [params isNil or: [lastParams isNil]])
				ifTrue: [output nextPutAll: spaces].
			output nextPutAll: (lastWord := self class decode: word using: params).
			spaces := self scanWhile: [self matchCharacterType: WhiteSpaceMask]].
	^output contents
!

decodePhrase: words
	^self class decodePhrase: words
!

decodeText: text
" Decoding of text is similar to decoding of comment, but RFC2047 requires that an encoded word that appears in in *text token MUST be separated from any adjacent encoded word or text by a linear-white-space"
	| output word |

	" First, quick check if we possibly have an encoded word "
	(text indexOfSubCollection: '=?' startingAt: 1)
		= 0 ifTrue: [^text].    "We suspect there might be an encoded word inside, do the legwork"

	self on: text readStream.
	output := (String new: text size) writeStream.
	[self atEnd]
		whileFalse:
			[word := self scanWhile: [(self matchCharacterType: WhiteSpaceMask) not].
			output
					nextPutAll: (self decode: word);
					nextPutAll: (self scanWhile: [self matchCharacterType: WhiteSpaceMask])].
	^output contents
!

encodingParametersOf: word
	^self class encodingParametersOf: word
! !

!MimeEncodedWordCoDec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MimeEncodedWordCoDec.st,v 1.2 2009/01/13 15:24:31 stefan Exp $'
! !
