"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageElement subclass:#MimeMessageBody
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MimeMessageBody comment:'This represents is an abstract class for Mime message bodies. Message bodies can be simple or multi-part.
Mail messages can be complex and deeply nested and their parts can be very large. In many cases we
may not need contents of one or more message parts, so it makes sense not to parse messages too
deep and only read data in memory when we actually need it. For this reason Parsing and value initialization
is done lazily. Source stream is remembered so that parsing can be done at any time.
Messages can also have external parts where data can be retrieved from external source via FTP or other means. Because streams are a universal mean of accessing external resources, storing source stream also encapsulates part access logic.

Instance Variables:

	value   <String or SequencableCollection>       body contents'
!


!MimeMessageBody class methodsFor:'instance creation'!

bodyTypeFor: aMimeEntity

	^aMimeEntity isMultipart
		ifTrue: [MultipartBody]
		ifFalse: [SimpleBody]
!

newBodyFor: aMimeEntity
	^(self bodyTypeFor: aMimeEntity) new
		parent: aMimeEntity;
		yourself
!

readFrom: stream parent: aMimeEntity
	^(self newBodyFor: aMimeEntity)
		readFrom: stream;
		yourself
! !

!MimeMessageBody class methodsFor:'parsing'!

builder
	^self defaultBuilder
!

defaultBuilder
	^MimeEntity builder
!

defaultPrinter
	^MimeEntity printer
!

printer
	^self defaultPrinter
!

scannerType
	^MimeScanner
! !

!MimeMessageBody methodsFor:'accessing'!

decodedValue
	^self value
!

decodedValueWith: aSymbol
	^self value
!

mimeType
	^self parent mimeType
!

partAt: anInteger
	self subclassResponsibility
!

removePart: aMimeEntity
	self subclassResponsibility
!

removePartAt: anInteger
	self subclassResponsibility
!

size
	^ self sourceSize
!

sourceLineEndConvention
	^nil
! !

!MimeMessageBody methodsFor:'parsing'!

builder
	^self parent notNil
		ifTrue: [self parent builder]
		ifFalse: [self defaultBuilder]
!

defaultBuilder
	^self class defaultBuilder
!

defaultPrinter
	^self class defaultPrinter
!

parse: scanner
!

printer
	^self parent notNil
		ifTrue: [self parent printer]
		ifFalse: [self defaultPrinter]
! !

!MimeMessageBody methodsFor:'printing'!

conclusion
	^nil
!

preamble
	^nil
!

printForDisplayOn: aStream
	self printOn: aStream
!

printOn: aStream indent: level
	self printer printMessageElement: self on: aStream indent: level
! !

!MimeMessageBody methodsFor:'testing'!

isBinary
	^self parent isBinary
!

isBinaryContentType
	^self parent isBinaryContentType
!

isEmpty
	^self notEmpty not
!

isMultipart
	^false
!

isSimple
	^true
!

notEmpty
	^self size > 0
! !

!MimeMessageBody methodsFor:'writing'!

writeContentsOn: aStream
" Write your contents on a stream. Used to save to a file. etc. "
	self subclassResponsibility
! !

!MimeMessageBody class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MimeMessageBody.st,v 1.1 2004/11/12 10:44:59 penk Exp $'
! !
