"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#NetEnvironment
	instanceVariableNames:'debugStream debugCategories debugClasses trace logFileName'
	classVariableNames:'Environment'
	poolDictionaries:''
	category:'NetClients-Framework'
!


!NetEnvironment class methodsFor:'accessing'!

default

	^ Environment isNil
		ifTrue: [ Environment := self new ]
		ifFalse: [ Environment ].
! !

!NetEnvironment methodsFor:'accessing'!

debugCategories

	debugCategories isNil
		ifTrue: [ debugCategories := Set new.
				debugCategories add: #general ].

	^debugCategories
!

debugClasses

	^debugClasses isNil
		ifTrue: [ debugClasses  := Set new.]
		ifFalse: [ debugClasses ].
!

debugStream

        ^debugStream ? Transcript
!

debugStream: aStream

	debugStream := aStream
!

logFileName

	logFileName isNil
		ifTrue: [ logFileName  := 'NetClientLog.txt'].
	^ logFileName
!

logFileName: aString

	logFileName := aString
!

trace

	trace isNil
		ifTrue: [ trace := false].
	^ trace
!

trace: aBoolean

	 trace := aBoolean
! !

!NetEnvironment methodsFor:'debugging'!

debug: aBlock level: aLevel
	( self trace and: [ self debugCategories includes: aLevel ] ) ifTrue: [ aBlock value]
!

log: aStringOrBlock
	
	self log: aStringOrBlock level: #general
!

log: aStringOrBlock level: aLevel
	| stream i briefMsg aMsg |

	self 
		debug:
			[(stream := self debugStream) == nil ifTrue: [^self].
			(aStringOrBlock isKindOf: BlockClosure)
				ifTrue: [ aMsg := aStringOrBlock value]
				ifFalse: [ aMsg := aStringOrBlock ].
			i := aMsg size.
			[i > 0 and: [(aMsg at: i) isSeparator]] whileTrue: [i := i - 1].
			briefMsg := aMsg copyFrom: 1 to: i.
			stream cr; nextPutAll: briefMsg; flush]
		level: aLevel
!

printTrace: aString
	| stream |
	(stream := self debugStream) == nil ifTrue: [^self].
	stream cr; cr; nextPutAll: ' **** ' asString.
	Date today printOn: stream.
	stream nextPutAll: ' '.	
	Time now printOn: stream.
	stream nextPutAll: ' ', aString, ' ****'; flush.
!

traceOff

	self printTrace: 'Stop Trace'.
	self trace: false.
!

traceOn

	self trace: true.
	self printTrace: 'Start Trace'.
! !

!NetEnvironment methodsFor:'private'!

addDebugCategory: symbol 

	self debugCategories add: symbol
!

removeDebugCategory: symbol 

	self debugCategories remove: symbol
!

reset

	self resetDebugClasses.
	self resetDebugCategories.
!

resetDebugCategories

	debugCategories := nil
!

resetDebugClasses

	debugClasses := nil
! !

!NetEnvironment methodsFor:'registration'!

addToDebug: aClass

	self debugClasses add: aClass.
! !

!NetEnvironment class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetEnvironment.st,v 1.1 2004/11/12 10:47:46 penk Exp $'
! !
