"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#NetProtocolInterpreter
	instanceVariableNames:'client connectionStream'
	classVariableNames:'LineEndCRLF LineEndLF LineEndCR LineEndTransparent'
	poolDictionaries:'Net::TCP'
	category:'NetClients-Framework'
!


!NetProtocolInterpreter class methodsFor:'initialization'!

initialize
    LineEndCR := #cr.
    LineEndLF := #nl.
    LineEndCRLF := #crnl.
    LineEndTransparent := #yourself.



"
    self initialize
"
! !

!NetProtocolInterpreter class methodsFor:'instance creation'!

client: aNetClient
    ^self new client: aNetClient
!

new
    ^self basicNew initialize
! !

!NetProtocolInterpreter class methodsFor:'debugging'!

log: aString level: aLevel

	NetEnvironment default log: aString level: aLevel
!

registerToDebug

	NetEnvironment default addToDebug: self
! !

!NetProtocolInterpreter class methodsFor:'private-attributes'!

defaultResponseClass
    ^NetResponse
! !

!NetProtocolInterpreter methodsFor:'accessing'!

client
    ^client
!

receiveMessageUntilPeriod
    "Receive and answer a message until period line."

    | write |
    write := WriteStream on: (String new: 4 * 1024).
    self receiveMessageUntilPeriodInto: write.
    ^write contents
!

receiveMessageUntilPeriodInto: aStream
    "Receive a message until period line into aStream."

    self connectIfClosed.
    MIME.MimeEntity new parseSimpleBodyFrom: self onto: aStream
!

reporter
    ^self client reporter
!

sendMessageWithPeriod: aStream
    "Send aStream as a message with period."

    self connectIfClosed.
    (PrependDotStream to: self)
	nextPutAll: aStream;
	flush
!

skipMessageUntilPeriod
    "Skip a message until period line."

    self connectIfClosed.
    MIME.MimeEntity new skipSimpleBodyFrom: self
! !

!NetProtocolInterpreter methodsFor:'connection'!

binary
    connectionStream class == CrLfStream
	ifTrue: [ connectionStream := connectionStream stream ]
!

close
    self closed ifFalse: [connectionStream close. connectionStream := nil].

    self liveAcrossSnapshot
	ifTrue: [ObjectMemory removeDependent: self].
!

closed
    ^connectionStream == nil
!

connect
    | connection messageText |
    connection := [ Socket newTCPclientToHost: client hostName port: client portNumber ]
	on: Error
	do: [ :ex | messageText := ex messageText. ex return: nil ].

    connection isNil ifTrue: [^self connectionFailedError: messageText].
    self connectionStream: connection
!

connectIfClosed
    self closed ifTrue: [self connect].
!

connectionStream
    ^connectionStream
!

connectionStream: aSocket
    connectionStream := aSocket.
    self text.
    self liveAcrossSnapshot
	ifTrue: [ObjectMemory addDependent: self]
!

isBinary
    self halt.
    ^connectionStream class ~~ CrLfStream
!

reconnect
    self close; connect
!

text
    self binary.
    self lineEndConvention = LineEndCRLF ifTrue: [
	connectionStream lineEndTransparent
	" connectionStream := CrLfStream on: connectionStream " ].
! !

!NetProtocolInterpreter methodsFor:'encoding'!

decode: aString
    ^aString
!

encode: aString
    ^aString
! !

!NetProtocolInterpreter methodsFor:'initialize-release'!

client: aNetClient
    client := aNetClient
!

initialize
!

release
    self close.
! !

!NetProtocolInterpreter methodsFor:'private'!

checkResponse
    self checkResponse: self getResponse
!

checkResponse: response
    self
	checkResponse: response
	ifError: [self protocolError: response statusMessage]
!

checkResponse: reponse ifError: errorBlock
!

connectionClosedError: messageText
    ^ConnectionClosedError new
	tag: messageText;
	signal: 'Connection closed: ', messageText
!

connectionFailedError: messageText
    ^ConnectionFailedError new
	tag: messageText;
	signal: 'Connection failed: ', messageText
!

getResponse
    ^self class defaultResponseClass fromClient: self
!

loginIncorrectError: messageText
    ^LoginIncorrectError new
	tag: messageText;
	signal: 'Login incorrect: ', messageText
!

protocolError: messageText
    ^ProtocolError new
	tag: messageText;
	signal: 'Protocol error: ', messageText
! !

!NetProtocolInterpreter methodsFor:'private-attributes'!

lineEndConvention
    ^LineEndCRLF
!

liveAcrossSnapshot
    ^false
! !

!NetProtocolInterpreter methodsFor:'stream accessing'!

atEnd
    ^connectionStream atEnd
!

contents
    ^self decode: connectionStream contents.
!

cr
    | conv |
    conv := self lineEndConvention.
    (conv = LineEndCR or: [conv = LineEndTransparent])
	ifTrue: [^connectionStream nextPut: Character cr].
    conv = LineEndLF
	ifTrue: [^connectionStream nextPut: Character nl].
    conv = LineEndCRLF
	ifTrue: [^connectionStream nextPut: Character cr; nextPut: Character nl].

    self error: 'Undefined line-end convention'.
!

flush
    connectionStream flush
!

next
    ^connectionStream next
!

next: anInteger
    ^self decode: (connectionStream next: anInteger).
!

nextAvailable: anInteger
    ^self decode: (connectionStream nextAvailable: anInteger).
!

nextLine
    | write byte |
    write := WriteStream on: (String new: 128).
    [connectionStream atEnd] whileFalse: [
	byte := connectionStream next.
	byte == Character cr ifTrue:
		[connectionStream peekFor: Character nl.
		 ^self decode: write contents].
	byte == Character nl ifTrue:
		[^self decode: write contents].
	write nextPut: byte].
    ^self decode: write contents
!

nextPut: aCharacter
    connectionStream nextPutAll: (self encode: (String with: aCharacter))
!

nextPutAll: aString
    aString isEmpty ifTrue: [^self].
    connectionStream nextPutAll: (self encode: aString)
!

nl
    | conv |
    conv := self lineEndConvention.
    conv = LineEndCR
	ifTrue: [^connectionStream nextPut: Character cr].
    (conv = LineEndLF or: [conv = LineEndTransparent])
	ifTrue: [^connectionStream nextPut: Character nl].
    conv = LineEndCRLF
	ifTrue: [^connectionStream nextPut: Character cr; nextPut: Character nl].

    self error: 'Undefined line-end convention'.
!

species
    ^connectionStream species
!

upTo: aCharacter
    | byte |
    aCharacter = Character cr ifTrue: [^self nextLine].
    byte := self encode: (String with: aCharacter).
    byte size = 1
	ifTrue: [^self decode: (connectionStream upTo: byte)]
	ifFalse:        [^self decode: (connectionStream upToAll: byte)]
! !

!NetProtocolInterpreter methodsFor:'updating'!

update: aSymbol
    "Dependents of ObjectMemory are sent update:
    #returnFromSnapshot when a snapshot is started."

    self liveAcrossSnapshot
	ifTrue:
		[aSymbol == #returnFromSnapshot
			ifTrue: [self close].
		"(aSymbol == #aboutToSnapshot or: [aSymbol == #aboutToQuit])
			ifTrue: [self close]."].

    super update: aSymbol
! !

!NetProtocolInterpreter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetProtocolInterpreter.st,v 1.3 2008/09/03 08:51:30 cg Exp $'
! !

NetProtocolInterpreter initialize!
