"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

RFC822Scanner subclass:#NetworkAddressParser
	instanceVariableNames:'descriptor'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-Framework'
!

NetworkAddressParser comment:'This class parses string representing mailbox and group addresses and answers a instances of address descriptors -- either NetworkAddressDescriptor or MailGroupDescriptor -- depending on the nature of address. .

Instance Variables:
	descriptor      <NetwordAddressDescriptor or MailGroupDescriptor>  descriptor to put parsed info in
'
!


!NetworkAddressParser class methodsFor:'instance creation'!

parse: string
	^self new parse: string
! !

!NetworkAddressParser class methodsFor:'utility'!

addressFrom: stream
	"self addressFrom: 'kyasu@crl.fujixerox.co.jp'."
	"self addressFrom: 'Kazuki Yasumatsu <kyasu@crl.fujixerox.co.jp>'."
	"self addressFrom: 'kyasu@crl.fujixerox.co.jp (Kazuki Yasumatsu)'."

	^(self on: stream) parseAddress
!

addressesFrom: stream
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp' readStream."
	"self addressesFrom: 'Kazuki Yasumatsu <kyasu@crl.fujixerox.co.jp>' readStream."
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp (Kazuki Yasumatsu)' readStream."
	"self addressesFrom: ' kyasu1, kyasu2, Kazuki Yasumatsu <kyasu3>, kyasu4 (Kazuki Yasumatsu)' readStream."
	"self addressesFrom: ' foo bar, kyasu1, ,  Kazuki Yasumatsu <kyasu2> <kyasu3> (<foo> (foo bar), bar)' readStream."

	^(self on: stream) parseAddressesSeparatedBy: $,
! !

!NetworkAddressParser methodsFor:'accessing'!

descriptor
	^descriptor
!

descriptor: aValue

	descriptor := aValue
! !

!NetworkAddressParser methodsFor:'converting'!

phraseAsString: phrase
	| stream |
	stream := (String new: 40) writeStream.
	MimeScanner printPhrase: phrase on: stream.
	^stream contents.
! !

!NetworkAddressParser methodsFor:'initialize-release'!

initialize
	super initialize.
	descriptor := self newAddressDescriptor
! !

!NetworkAddressParser methodsFor:'private'!

completeScanOfAddressSpecWith: partial
" addr-spec   =  local-part <@> domain        ; global address
  local-part = word *(<.> word) ; uninterpreted, case-preserved
First local-part token was already scanned; we are now scanning *(<.> word) group and domain part.
Partial is an array of tokens already read "

	| stream pos |
	stream := partial readWriteStream.
	stream setToEnd.
	self descriptor localPart: (self scanLocalAddressPartTo: stream ).
	pos := self position.
	self nextRFC822Token == $@
		ifTrue: [self descriptor domain: self scanDomain]
		ifFalse:[self position: pos].
!

newAddressDescriptor
	^NetworkAddressDescriptor new
!

parseGroupSpecWith: phrase
	"group = phrase <:> [#mailbox] <;>"

	| group mailboxes phrasex comment stream |

	mailboxes := self tokenizeList: [self parseAddress]
				separatedBy: [token == $,].
	self nextRFC822Token == $; ifFalse: [^self notify: 'Group descriptor should be terminated by <:>'].
	group := MailGroupDescriptor new.

	" If phrase is non-empty, an alias was specified "
	phrasex := phrase isEmpty
				ifTrue: [nil]
				ifFalse: [self phraseAsString: phrase].

	comment := currentComment isNil
				ifTrue: [nil]
				ifFalse:
					[stream := (String new: 40) writeStream.
					currentComment do: [:part | stream nextPutAll: part]
						separatedBy: [stream space].
					stream contents].

	group alias: phrasex; addresses: mailboxes; comment: comment.
	^group
!

parseMailboxSpecWith: phrasex
"     address     =  mailbox                      ; one addressee
				 /  group                        ; named list
     group       =  phrase <:> [#mailbox] <;>
     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec
     route-addr  =  <<> [route] addr-spec <>>
     route       =  1#(<@> domain) <:>           ; path-relative"

	| phrase tok local stream comment |
	phrase := phrasex.
	tok := self nextRFC822Token.
	self descriptor: self newAddressDescriptor.

	"Variations of mailbox spec"
	tok = $< ifTrue:                                                                "Phil Campbell<philc@acme.com>"
		[self stepBack; scanRouteAndAddress]
	ifFalse: [
		('.@' includes: tok)
			ifTrue:
				["These ones should have a non-empty local part to the left of delimiter"
				phrase isEmpty ifTrue: [self error: 'Invalid network address'].
				local := Array with: phrase last.
				phrase := phrase copyFrom: 1 to: phrase size - 1.       "Extract the part we already scanned"
				tok = $. ifTrue:                                                        "phil.campbell.wise@acme.com>"
					[self stepBack; completeScanOfAddressSpecWith: local].
				tok = $@                                                                "philc@acme.com>"
					ifTrue:
						[self descriptor localPart: local.
						self descriptor domain: self scanDomain]]
				ifFalse: [self stepBack]].
	" If phrase is non-empty, an alias was specified "
	phrase := phrase isEmpty
				ifTrue: [phrase := nil]
				ifFalse: [self phraseAsString: phrase].
	self descriptor alias: phrase.
	comment := currentComment isNil
				ifTrue: [nil]
				ifFalse:
					[stream := (String new: 40) writeStream.
					currentComment do: [:part | stream nextPutAll: part]
						separatedBy: [stream space].
					stream contents].
	self descriptor comment: comment.
	^self descriptor
!

scanLocalAddressPartTo: stream
	"local-part = word *(<.> word) ; uninterpreted, case-preserved
	Part of local part may have been scanned already, it's in localPart of the descriptor"

	self
		tokenizeWhile: [token == $.]
		do: [stream nextPut: self scanWord].
	^stream contents
!

tryScanSubdomain
	self nextRFC822Token.
	tokenType = #atom ifTrue: [ ^true ].
	(token = $[) ifTrue: [ self stepBack; scanDomainText. ^true ].
	^false.
! !

!NetworkAddressParser methodsFor:'public'!

addressesFrom: stream
	^(self on: stream) parseAddressesSeparatedBy: $,
!

parse: aString
	^self on: aString readStream; parseAddress
!

parseAddress
"     address     =  mailbox                      ; one addressee
				 /  group                        ; named list
     group       =  phrase <:> [#mailbox] <;>
     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec
     route-addr  =  <<> [route] addr-spec <>>
     route       =  1#(<@> domain) <:>           ; path-relative"

	| phrase |
	phrase := self scanPhrase.
	(phrase isEmpty and: [self atEnd]) ifTrue: [ ^nil ].

	^self nextRFC822Token = $:
		ifTrue: [self parseGroupSpecWith: phrase]
		ifFalse: [self stepBack; parseMailboxSpecWith: phrase].
!

parseAddressesSeparatedBy: separatorChar

	^self tokenizeList: [self parseAddress]
				separatedBy: [token == separatorChar].
!

scanDomain
	"domain = sub-domain *(<.> sub-domain)"
	"Answers an array of domain seqments, from least significant to most significant"

	^self tokenizeList: [self nextRFC822Token.
		tokenType = #atom
			ifTrue: [token]
			ifFalse: [token = $[
					ifTrue: [self stepBack; scanDomainText]
					ifFalse: [^self notify: 'Invalid domain specification']]]
		separatedBy: [token == $.]
!

scanLocalAddress
	"local-part = word *(<.> word) ; uninterpreted, case-preserved"

	^self
		tokenizeList:
			[self nextRFC822Token.
			(#(#quotedText #atom) includes: tokenType)
				ifFalse: [^self notify: 'Local part can only include words'].
			token]
		separatedBy: [token == $.]
!

scanRoute
	"route = 1#(<@> domain) <:> ; path-relative"

	| stream |
	stream := (Array new: 2) writeStream.
	[self nextRFC822Token == $@]
		whileTrue:
			[stream nextPut: self scanDomain.
			self nextToken = $: ifFalse: [self error: 'Invalid route spec']].
	stream size = 0 ifTrue: [self error: 'Invalid route spec'].
	^stream contents
!

scanRouteAndAddress
"     route-addr  =  <<> [route] addr-spec <>>"

	self mustMatch: $< notify: 'Invalid route address spec'.
	self nextRFC822Token == $@ ifTrue: [ self stepBack. self descriptor route: self scanRoute ].
	self completeScanOfAddressSpecWith: (Array with: token).
	self mustMatch: $> notify: 'Invalid route address spec'.
! !

!NetworkAddressParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetworkAddressParser.st,v 1.1 2004/11/12 10:46:45 penk Exp $'
! !
