"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageElement subclass:#NetworkEntityDescriptor
	instanceVariableNames:'alias comment'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

NetworkEntityDescriptor comment:'This class is an abstract class for mailbox address and mail group address descriptors as defined in RFC822. All addresses may have an address part and alias part. Address part defines the actual networ address while alias is a human-readable name associated with the address.
Main purpose of descriptors is to be able to parse and compose address string into indivudual components, check syntax, provide canonical value and decode non-ascii characters in alias.
While address syntax is defined in rfc822, the individual tokens (namely, phrase representing alias) are further developed in Mime (rfc2048) to provide for non-ascii characters. Therefore, message is parsed using rfc822 syntactic rules including rfc822 character table. Once parsed, individual lexical tokens are scanned using Mime rules.

Subclasses must implement the following messages:
	printing
	printCanonicalValueOn:

Instance Variables:
	alias   <OrderedCollection>  Collection of words comprising alias. Access alias using #aliasString
	comment <String>  Comment string if present as part of the address
'
!


!NetworkEntityDescriptor class methodsFor:'parsing'!

scannerType
	^NetworkAddressParser
! !

!NetworkEntityDescriptor methodsFor:'accessing'!

alias

	^alias
!

alias: aValue

	alias := aValue
!

comment

	^comment
!

comment: aValue

	comment := aValue
!

stringWithEncoding: aString
	| stream |
	stream := (String new: 256) writeStream.
	self writeOn: stream encoding: aString.
	^stream contents
! !

!NetworkEntityDescriptor methodsFor:'composing'!

writeAliasOn: stream encoding: aString
	"Encoding is required if alias contains non-ASCII chars."

	alias notNil
		ifTrue:
			[(MimeEncodedWordCoDec shouldBeEncoded: alias)
				ifTrue: [ stream nextPutAll: (MimeEncodedWordCoDec
									encodeText: alias
									asQuotedPrintableUsing: aString)]
				ifFalse:
					[(self scannerType shouldBeQuoted: alias)
						ifTrue: [stream nextPut: $"; nextPutAll: alias; nextPut: $"]
						ifFalse: [stream nextPutAll: alias]]]
!

writeCanonicalValueOn: stream encoding: aString
	self subclassResponsibility
!

writeCommentOn: stream encoding: aString

	self comment ~~ nil
		ifTrue:
			[stream nextPutAll:
				(MimeEncodedWordCoDec
					encodeComment: self comment
					asBase64Using: aString)]
!

writeOn: stream encoding: aString
	self writeCanonicalValueOn: stream encoding: aString.
"       comment notNil ifTrue: [self writeCommentOn: stream encoding: aString] "
! !

!NetworkEntityDescriptor methodsFor:'parsing'!

scannerType
	^self class scannerType
! !

!NetworkEntityDescriptor methodsFor:'printing'!

printAliasOn: stream
	alias notNil
		ifTrue:
			[(self scannerType shouldBeQuoted: alias)
				ifTrue: [ stream nextPut: $"; nextPutAll: alias; nextPut: $" ]
				ifFalse: [stream nextPutAll: alias]].
!

printCanonicalValueOn: stream
	self subclassResponsibility
!

printCommentOn: stream
	comment notNil ifTrue: [
		stream nextPut: $(.
		comment
			do:
				[:char |
				(RFC822Scanner isCommentChar: char)
					ifFalse: [stream nextPut: $\].
				stream nextPut: char].
		stream nextPut: $).
	]
!

printOn: stream indent: level
	super printOn: stream indent: 0.
	self printCanonicalValueOn: stream.
	comment notNil ifTrue: [self printCommentOn: stream]
! !

!NetworkEntityDescriptor methodsFor:'testing'!

isAddressDescriptor
	^true
! !

!NetworkEntityDescriptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetworkEntityDescriptor.st,v 1.1 2004/11/12 10:44:21 penk Exp $'
! !
