"
 COPYRIGHT (c) 2009 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

NetClient subclass:#Pop3Client
	instanceVariableNames:'connection stream state serverResponse'
	classVariableNames:'DelaySeconds TerminationOctet TerminationSequence DebugStream
		MaxRetries LastServer DebugLevels CRLF DebugDetail'
	poolDictionaries:''
	category:'NetClients-POP3'
!

Object subclass:#Pop3State
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Pop3Client
!

Pop3Client::Pop3State subclass:#Pop3AuthorizationState
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Pop3Client
!

Pop3Client::Pop3State subclass:#Pop3TransactionState
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Pop3Client
!

Pop3Client comment:'This class is Pop3 client. It executes Pop3 commands. Because most commands are only valid in certain states, the state variable holds and instance of (subclass of) Pop3State. All commands are delegated to the state which either rejects them as inappropriate in the given state, or calls client back to send the command to Pop3 server.


Instance Variables:
	host    <String>  Host address
	user    <String>  User name
	connection      <ExternalConnection>  Socket
	stream  <Stream>  Socket stream
	state   <Pop3State>  Current state
	serverResponse  <String>  comment
	retries <Integer>  comment


Shared Variables:
	CRLF    <String>  comment
	DelaySeconds    <Integer>  comment
	LastServer      <type>  Last server -- if we don''t care
	MaxRetries      <Integer>  Max retries before we give up
	TerminationOctet        <Character>  comment
	TerminationSequence     <String>  comment
'
!

!Pop3Client class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2009 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

examples
"
    |mailBox headers queue user|

    queue := Queue new.
    mailBox := Net::Pop3Mailbox new.
    user := Net::NetUser username: 'stefan.reise@gmx.de' password: 'XXXXX'.
    mailBox
	    user: user;
	    server: 'pop.gmx.net';
	    deliveryQueue: queue;
	    loadSettings.
    headers := mailBox allHeaders.
    ^ headers.
"
! !

!Pop3Client class methodsFor:'instance creation'!

new
	^super new initialize
! !

!Pop3Client class methodsFor:'class initialization'!

initialize
	self initializeVariables.
	super initialize.
	self registerToDebug.
!

initializeVariables
    DelaySeconds := 10.
    TerminationOctet := $. "16r2E".
    TerminationSequence := String with:$. with:(Character return) with:(Character lf).
    MaxRetries := 5.
    DebugLevels := Set new.
    CRLF := String with:(Character return) with:(Character lf).
    DebugDetail := 1.
!

registerClass
	self mailRegistry at: self serverType put: self name
! !

!Pop3Client class methodsFor:'constants'!

description
	^'incoming mail server'
!

serverType
	^#POP3
! !

!Pop3Client class methodsFor:'debugging'!

debugLabelsAndValues
	^(List new)
		add: ('Pop3 Client Messages'->#POP3Client);
		add: 'Pop3 Server Messages'->#POP3ConnServer;
		add: 'Pop3 Connection Retries'->#POP3ConnRetries;
		yourself
!

menuItemName

	^'POP3 Client Messages'
! !

!Pop3Client class methodsFor:'private'!

update: anAspect with: arguments from: anObject
! !

!Pop3Client class methodsFor:'testing'!

isIncomingMailClient
	^true
!

isMailClient
	^true
!

isOutgoingMailClient
	^false
! !

!Pop3Client class methodsFor:'utilities'!

messageCount
	"How many messages are on the server?"
	"self messageCount"

	| connection |
	connection := self promptForConnection.
	^[connection status messages]
		ensure: [connection disconnect]
!

promptForConnection
	"Pop3Connection promptForConnection"

	| connection user  host |
	user := NetUser request.
	user isNil ifTrue: [^self].
	host := Dialog
		request: 'Pop server on:'
		initialAnswer: (LastServer isNil ifTrue: ['postoffice.pacbell.net'] ifFalse: [LastServer])
		onCancel: [^nil].
	LastServer := host.
	connection := self host: host.
	connection user: user.
	connection
		connect;
		login.
	^connection
!

showProgress: aString complete: anInteger while: aBlock
	"Show a progress indicator window while executing a block of code.
	anInteger is the completeness count (ie = 100%).
	IncrementNotification is raised to denote progress."
	"self  showProgress: 'X' complete: 10000 while:
		[ 10000 timesRepeat: [ IncrementNotification raiseSignal ] ]"

	| increment progress pw res |
	increment := 1.0 / anInteger.
	progress := 0.0 asValue.
	pw := ProgressWidgetView progressOpenOn: progress label: aString asText allBold.
	[ [ Cursor wait showWhile: [ res := aBlock value ] ] on: IncrementNotification do:
		[ :not |
		progress value: ( progress value + ( not ticks * increment ) ).
		not resume.
		 ]
	] ensure: [ pw closeAndUnschedule ].
	^res
!

timestampFormat
	"This is simply moronic."
	|policy|
	policy := List new
		add: #ddd;
		add: Character space;
		add: #mmm;
		add: Character space;
		add: #d;
		add: Character space;
		add: #h;
		add: $: ;
		add: #mm;
		add: Character space;
		add: #yyyy;
		yourself.
	^policy asArray
! !

!Pop3Client methodsFor:'accessing'!

serverResponse
	^serverResponse isNil
		ifTrue: [serverResponse := String new]
		ifFalse: [serverResponse]
!

state
	^state
!

state: aState
	state := aState
! !

!Pop3Client methodsFor:'commands'!

apop: aStringName digest: aStringDigest

	self state apopName: aStringName digest: aStringDigest for: self.
	self hasPositiveResponse ifFalse: [^false].
	self state: Pop3TransactionState new.
	^true
!

delete: message
	^self state delete: message for: self
!

deleteMessageIds: aCollectionOfUniqueIDs
	"Delete all messages which have unique ids in the argument.
	 Answer the number of messages remaining on the server."
	^self state deleteMessageIds: aCollectionOfUniqueIDs for: self
!

disconnect
	self sendQuit.
	self state: Pop3State new.
	stream := nil
!

list
	^self state list: self
!

list: messageNumber
	^self state list: messageNumber for: self
!

login
	self state user: self.
	self hasPositiveResponse ifFalse: [^false].
	self state pass: self.
	self hasPositiveResponse ifFalse: [^false].
	self state: Pop3TransactionState new.
	^true
!

noop
	^self state noop: self
!

quit
	self disconnect
!

reset
	^self state reset: self
!

retrieveMessage: number
	^self state retrieveMessage: number for: self
!

retrieveMessageID: number
	^self state retrieveMessageID: number for: self
!

retrieveMessageLines: number
	^self state retrieveMessageLines: number for: self
!

status
	^self state stat: self
!

top: lines of: messageID
	^self state top: lines of: messageID for: self
!

uidl: aNumber
	^self retrieveMessageID: aNumber
! !

!Pop3Client methodsFor:'connection'!

connect
	self state: Pop3AuthorizationState new.

	(stream := self createSocket) isNil ifTrue: [^NetClientError signalWith: #connection].
	stream lineEndTransparent.
	self waitForResponse
!

isConnected
	^connection notNil
! !

!Pop3Client methodsFor:'debugging'!

log: aStringOrBlock level: aLevel
	self class log: aStringOrBlock level: aLevel
! !

!Pop3Client methodsFor:'defaults'!

defaultPortNumber
	"The default port a Pop3Server will listen on"

	^110
! !

!Pop3Client methodsFor:'initialize-release'!

initialize
	self state: Pop3State new
! !

!Pop3Client methodsFor:'private'!

createSocket
	"Return a SocketAccessor instance or nil if a connection can not be made."

	retries := self retries.        "Just in case our server is busy, prepare for an error."

	^[(SocketAccessor ? Socket) newTCPclientToHost: self hostName port: self portNumber]
		on: OSErrorHolder peerFaultSignal, OsInaccessibleError
		do:
			[:ex |
			(ex getSignal = OSErrorHolder peerFaultSignal
			and: [(self handlePeerFaultError: ex)]) ifTrue:
				[^nil "Unable to connect to the server."].
			ex class = OsInaccessibleError ifTrue:
				[^nil "Pop3Server is not accessible."]]
!

getNextLine
	^stream throughAll: CRLF
!

handlePeerFaultError: exception
	"The Pop3Server we are connecting to might be too busy to handle our
	connection request.  In this case, delay for a little while and try again.
	Once we have hit our connection retry limit, return true to the client so
	that it can handle the error as it sees fit."

	retries := retries - 1.
	retries > 0 ifTrue:
		[(Delay forSeconds: self delaySeconds) wait.
		self log: ['Failed to connect to <1s>. Will try <2p> more times.' expandMacrosWith: self hostName with: retries]
			level: #POP3ConnRetries.
		exception retry].
	^true
!

retrieveMultiLineResponseDo: block
	| result |
	[result := self getNextLine.
	 result isEmpty or: [result = TerminationSequence]] whileFalse:
		[result first = TerminationOctet ifTrue:
			[result := result copyFrom: 2 to: result size].
		self serverResponse: result.
		block value: result]
!

retrieveMultiLineResponseInto: aStream process: block
	| result |
	result := self getNextLine.
	[result isEmpty or: [result = TerminationSequence]]
		whileFalse:
			[result := result first = TerminationOctet
						ifTrue: [result copyFrom: 2 to: result size]
						ifFalse: [result].
			self serverResponse: result.
			aStream nextPutAll: (block value: result).
			result := self getNextLine]
!

sendApopName: aStringName digest: aStringDigest
	self sendCommand: ('APOP ', aStringName, ' ', aStringDigest).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError signalWith: #apop message: self serverResponse].
!

sendCommand: aString

	self log: ['C: <2s> S: <1s>' expandMacrosWith: aString with: self hostName] level: #POP3Client.
	stream
		nextPutAll: aString;
		nextPutAll: CRLF;
		commit.
!

sendDeleteMessage: number
	self sendCommand: ('DELE <1p>' expandMacrosWith: number).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse]
!

sendDeleteMessageIds: aSequenceOfUniqueIDs
	"Delete all messages whose unique ids appear in aSequenceOfUniqueIDs.
	 Do so by doing a UIDL list, computing matching message numbers, and
	 deleting by message number."
	| count map |
	count := self sendStat messages.
	map := Dictionary new: count * 2.
	self sendCommand: 'UIDL'.
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	self retrieveMultiLineResponseDo:
			[:line | | msgnum |
			msgnum := Integer readFrom: (ReadStream on: line).
			map at: (line copyFrom: (line lastIndexOf: Character space) + 1 to: line size - 2) put: msgnum].
	aSequenceOfUniqueIDs do:
		[:uid| | msgnum |
		((msgnum := map at: uid ifAbsent: []) notNil
		and: [self sendCommand: ('DELE <1p>' expandMacrosWith: msgnum).
			  self waitForResponse.
			  self hasPositiveResponse])
			ifFalse: [Transcript cr; nextPutAll: 'failed to delete id '; nextPutAll: uid; flush]].
	^self sendStat messages
!

sendList
	| messageStatuses |
	self sendCommand: 'LIST'.
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^self].

	messageStatuses := OrderedCollection new: 30.
	self
		retrieveMultiLineResponseInto: '' writeStream
		process:
			[:line | |result|
			IncrementNotification signalWith: line size.
			result := line tokensBasedOn: Character space.
			messageStatuses add: (Pop3Status new
				messages: (Number readFrom: (result at: 1) readStream);
				octets: (Number readFrom: (result at: 2) readStream);
				yourself).
			line].

	^messageStatuses
!

sendList: messageNumber
	| result |
	self sendCommand: ('LIST <1p>' expandMacrosWith: messageNumber).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	result := self serverResponse tokensBasedOn: Character space.
	^Pop3Status new
		messages: (Number readFrom: (result at: 2) readStream);
		octets: (Number readFrom: (result at: 3) readStream);
		yourself
!

sendNoop
	self sendCommand: 'NOOP'.
	self waitForResponse.
	"It should always be positive"
	self hasPositiveResponse ifFalse: [^self].
!

sendPassword
	self sendCommand: ('PASS <1s>' expandMacrosWith: self user password).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError signalWith: #login message: self serverResponse].
!

sendQuit
	self sendCommand: 'QUIT'.
	stream ioConnection isNil ifTrue: [^self].
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	stream close
!

sendReset
	self sendCommand: 'RSET'.
	self waitForResponse.
	"Should never be false"
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse]
!

sendRetrieveMessage: number
	| lines size result resultStream |
	self sendCommand: ('RETR <1p>' expandMacrosWith: number).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	lines := OrderedCollection new.
	size := 0.
	self retrieveMultiLineResponseDo:
			[:line | | lineSize |
			lineSize := line size.
			size := size + lineSize - 1.
			IncrementNotification signalWith: lineSize.
			lines add: (line)].
	result := String new: size.
	resultStream := result writeStream.
	lines do: [:lv| | line | line := lv value. resultStream next: line size - 1 putAll: line startingAt: 1].
	(result size ~= size
	or: [resultStream position ~= size]) ifTrue:
		[self error: 'wrong number of elements written to resultStream'].
	^result
!

sendRetrieveMessageID: number
	"Return a message's unique id as a String, or nil if the server does not support the UIDL command."
	| response |
	self sendCommand: ('UIDL <1p>' expandMacrosWith: number).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^nil].
	^(response := self serverResponse) isEmpty ifFalse:
		[response copyFrom: (response lastIndexOf: Character space) + 1 to: response size - 2]
!

sendRetrieveMessageLines: number
	"Return a message as an OrderedCollection of lines with CRLF termination."
	| lines count result |
	self sendCommand: ('RETR <1p>' expandMacrosWith: number).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	lines := LinkedList new.
	count := 0.
	self retrieveMultiLineResponseDo:
		[:line |
		count := count + 1.
		IncrementNotification signalWith: line size.
		lines addLast: (LinkValue value: line)].
	result := OrderedCollection new: count + 10.
	lines do: [:lv| result addLast: lv value].
	^result
!

sendStat
	| result |
	self sendCommand: 'STAT'.
	self waitForResponse.
	"It should always be positive"
	self hasPositiveResponse ifFalse: [^self].
	result := (serverResponse tokensBasedOn: Character space).
	^Pop3Status new
		messages: (Number readFrom: (result at: 2) readStream);
		octets: (Number readFrom: (result at: 3) readStream);
		yourself
!

sendTop: number of: msg
	| lines size result resultStream |
	self sendCommand: ('TOP <1p> <2p>' expandMacrosWith: msg with: number).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError raiseSignal: self serverResponse].
	lines := OrderedCollection new.
	size := 0.
	self retrieveMultiLineResponseDo:
			[:line | | lineSize |
			lineSize := line size.
			size := size + lineSize - 1.
			"/ IncrementNotification signalWith: lineSize.
			lines addLast: line].
	result := String new: size.
	resultStream := result writeStream.
	lines do: [:lv| | line | line := lv value. resultStream next: line size - 1 putAll: line startingAt: 1].
	(result size ~= size
	or: [resultStream position ~= size]) ifTrue:
		[self error: 'wrong number of elements written to resultStream'].
	^result
!

sendUser
	self sendCommand: ('USER <1s>' expandMacrosWith: self user username).
	self waitForResponse.
	self hasPositiveResponse ifFalse: [^NetClientError signalWith: #login message: self serverResponse].
!

serverResponse: aString

	self log: ['S: <2s> R: <1s>' expandMacrosWith: aString with: self hostName]
		level: #POP3ConnServer.
	serverResponse := aString
!

waitForResponse
	| result |

	stream readWait.
	result := stream throughAll: CRLF.
	self serverResponse: result
! !

!Pop3Client methodsFor:'testing'!

hasNegativeResponse
	^(self serverResponse indexOfSubCollection: '-ERR' startingAt: 1) > 0
!

hasPositiveResponse
	^(self serverResponse indexOfSubCollection: '+OK' startingAt: 1) > 0
!

successful
	^self hasPositiveResponse
! !

!Pop3Client::Pop3State methodsFor:'commands'!

apopName: aStringName digest: aStringDigest for: aConnection
	Pop3StateError raiseSignal
!

delete: message for: connection
	Pop3StateError raiseSignal
!

deleteMessageIds: aSequenceOfUniqueIDs for: connection
	Pop3StateError raiseSignal
!

list: number for: connection
	Pop3StateError raiseSignal
!

listMessage: number for: connection
	Pop3StateError raiseSignal
!

noop: aConnection
	Pop3StateError raiseSignal
!

pass: aConnection
	Pop3StateError raiseSignal
!

quit: aConnection
	aConnection sendQuit
!

reset: aConnection
	Pop3StateError raiseSignal
!

retrieveMessage: number for: connection
	Pop3StateError raiseSignal
!

retrieveMessageID: number for: connection
	Pop3StateError raiseSignal
!

retrieveMessageLines: number for: connection
	Pop3StateError raiseSignal
!

stat: aConnection
	Pop3StateError raiseSignal
!

top: lines of: msg for: connection
	Pop3StateError raiseSignal
!

user: aConnection
	Pop3StateError raiseSignal
! !

!Pop3Client::Pop3AuthorizationState methodsFor:'commands'!

apopName: aStringName digest: aStringDigest for: aConnection
	^aConnection sendApopName: aStringName digest: aStringDigest
!

pass: aConnection
	aConnection sendPassword
!

user: aConnection
	aConnection sendUser
! !

!Pop3Client::Pop3TransactionState methodsFor:'commands'!

delete: message for: connection
	^connection sendDeleteMessage: message
!

deleteMessageIds: aSequenceOfUniqueIDs for: connection
	^connection sendDeleteMessageIds: aSequenceOfUniqueIDs
!

list: aConnection
	^aConnection sendList
!

list: number for: connection
	^connection sendList: number
!

listMessage: number for: connection
	^connection sendList: number
!

noop: aConnection
	^aConnection sendNoop
!

reset: connection
	^connection sendReset
!

retrieveMessage: number for: connection
	^connection sendRetrieveMessage: number
!

retrieveMessageID: number for: connection
	^connection sendRetrieveMessageID: number
!

retrieveMessageLines: number for: connection
	^connection sendRetrieveMessageLines: number
!

stat: aConnection
	^aConnection sendStat
!

top: lines of: msg for: connection
	^connection sendTop: lines of: msg
! !

!Pop3Client class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Pop3Client.st,v 1.5 2010/03/19 11:31:12 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Pop3Client.st,v 1.5 2010/03/19 11:31:12 cg Exp $'
! !

Pop3Client initialize!
