"
 COPYRIGHT (c) 2009 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Mailbox subclass:#Pop3Mailbox
	instanceVariableNames:''
	classVariableNames:'MaxTransferRate MinTransferRate'
	poolDictionaries:''
	category:'NetClients-POP3'
!

!Pop3Mailbox class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2009 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

examples
"
    |mailBox headers queue user|

    queue := Queue new.
    mailBox := self new.
    user := NetUser username: 'foo' password: 'foobar'.
    mailBox
            user: user;
            server: 'pop.bar.com';
            deliveryQueue: queue;
            loadSettings.
    headers := mailBox allHeaders.
    ^ headers.
"
! !

!Pop3Mailbox class methodsFor:'debugging'!

debugLabelsAndValues
        ^List new
                add: ('Pop3Mailbox Downloads' -> #POP3Download);
                add: ('Pop3Mailbox Server Checking' -> #POP3Check);
                add: ('Pop3Mailbox Transfer Rate' -> #POP3TransferRate);
                yourself
!

initialize
        self registerToDebug
!

menuItemName
        ^'POP3 Mailbox Messages'
! !

!Pop3Mailbox class methodsFor:'utilities'!

processTransferRate: milliseconds ofSize: size fromServer: server 
        "self processTransferRate: 32 ofSize: 1836 fromServer: 'sulu.cincom.com' "

        | bytesPerSecond kiloBytesPerSecond maxTransferRate minTransferRate |

        bytesPerSecond := (size / milliseconds) * 1000.
        kiloBytesPerSecond := (bytesPerSecond / 1024) asFloat.
        maxTransferRate := (MaxTransferRate at: server ifAbsentPut: [0]) max: kiloBytesPerSecond.
        maxTransferRate = kiloBytesPerSecond ifTrue: 
                [MaxTransferRate at: server put: kiloBytesPerSecond].
        minTransferRate := (MinTransferRate at: server ifAbsentPut: [1024*1024]) min: kiloBytesPerSecond.
        minTransferRate = kiloBytesPerSecond ifTrue: 
                [MinTransferRate at: server put: kiloBytesPerSecond].

        self log: ['Transfer Rate of <1p> Kbps from <2s>. Max: <3p> Min: <4p>' 
                                        expandMacrosWith: kiloBytesPerSecond
                                        with: server
                                        with: maxTransferRate
                                        with: minTransferRate]
                level: #POP3TransferRate.
!

promptForConnection
        "Pop3Mailbox promptForConnection"

        | connection user  host |
        user := NetUser request.
        user isNil ifTrue: [^self].
        host := Dialog 
                request: 'Pop server on:' 
                initialAnswer: (LastServer isNil ifTrue: ['postoffice.pacbell.net'] ifFalse: [LastServer]) 
                onCancel: [^nil].
        LastServer := host.
        connection := Pop3Client host: host.
        connection user: user.
        ^connection
!

request
        "Pop3Mailbox request"

        | username password  host cancel |
        username := (NetUser lastUser isNil
                                        ifTrue: ['']
                                        ifFalse: [NetUser lastUser]) asValue.
        password := '' asValue.
        host := (LastServer isNil 
                                ifTrue: ['postoffice.pacbell.net'] 
                                ifFalse: [LastServer]) asValue.
        cancel := (SimpleDialog initializedFor: nil)
                                setInitialGap;
                                addMessage: 'What POP3 Server and login?' centered: true;
                                addGap;
                                addMessage: 'Username:' textLine: username boundary: 0.4;
                                addGap;
                                addMessage: 'Password:' textLine: password type: #password boundary: 0.4;
                                addGap;
                                addMessage: 'POP3 Server:' textLine: host boundary: 0.4;
                                addGap;
                                addOK: [true];
                                openDialog;
                                cancel.
        cancel value ifTrue: [^nil].
        ^self
                user:  (NetUser username: username value password: password value)
                server: host value
! !

!Pop3Mailbox methodsFor:'accessing'!

serverType
        ^'POP3'
! !

!Pop3Mailbox methodsFor:'commands'!

allHeaders
        | messageCount |

        ^self connectedDo: 
                [messageCount := connection status messages.
                (1 to: messageCount)
                        collect: 
                                [:i | 
                                MailIncrementNotification raiseSignal.
                                MimeEntity readHeaderFrom: (connection top: 10 of: i) readStream]]
!

anyNewMail
        "Any new mail to get?"

        ^self messageCount > lastMessage
!

getMessage: number 
        "Get the <number> message from the Pop3server"

        ^self safelyExecute:    [self getMessageNumber: number]
!

messageCount
        "How many messages are on the server?"

        ^self safelyExecute:
                                [connection status messages].
! !

!Pop3Mailbox methodsFor:'mail delivery'!

deliverAllMail
        "Queue up all new messages onto the queue. Clients of this 
        method should be aware that the state should be such that there 
        is new mail to download."

        self safelyExecute:
                [|  messageCount |
                messageCount := connection status messages.
                self deliverFrom: 1 to: messageCount.
                self log: ['Done; dowloaded <1p> messages from <2s>' expandMacrosWith: messageCount with: server]
                        level: #POP3Download
                ]
!

deliverMail
        "Queue up a message onto the queue."
        | n |
        self deliver: (self getMessage: (n := lastMessage + 1)).
        lastMessage := n.
!

deliverNewMail
        "Queue up all new messages onto the queue. Clients of this 
        method should be aware that the state should be such that there 
        is new mail to download."
        self safelyExecute:
                [|   startCount messageCount |
                messageCount := connection status messages - (startCount := lastMessage).
                self deliverFrom: startCount to: messageCount.
                self log: ['Done; dowloaded <1p> messages from <2s>' expandMacrosWith: (lastMessage - startCount) with: server]
                        level: #POP3Download
                ]
! !

!Pop3Mailbox methodsFor:'private'!

clientSpecies
        ^Pop3Client
!

deleteMessageNumbers: aSequenceOfUids
        ^self connectedDo: [lastMessage := connection deleteMessageIds: aSequenceOfUids]
!

deliverFrom: startCount to: messageCount

                self log: ['Dowloading <1p> messages from <2s>' expandMacrosWith: messageCount with: server] level: #IPOP3Download.
        messageCount timesRepeat: 
                [ | message n milliseconds |
                 milliseconds := Time millisecondsToRun: [message := self getMessageNumber: (n := lastMessage + 1)].
                 lastMessage := n.
                 n - startCount \\ 10 = 0 
                        ifTrue: 
                                [self log: ['dowloaded <1p> messages from <2s>'expandMacrosWith: n - startCount with: server] 
                                        level: #POP3Download].
                        self deliver: message.
                        self debug: [self class processTransferRate: milliseconds ofSize: message size fromServer: server] 
                                level: #POP3TransferRate.
                        Processor yield "be nice to other mail polling processes" ]
!

getMessageNumber: number 
        "Get the <number> message from a Pop3 server on connection and return the message as a string."

        | lines id  stream |
        IncrementNotification raiseSignal. 
        lines := connection retrieveMessageLines: number.
        id := connection retrieveMessageID: number.
        stream := (String new: 512) writeStream.
        lines do: [ :each |  stream nextPutAll: (each copyFrom: 1 to: ( each size - 1)) ].
        ^LetterInTransit new
                letterText: stream contents;
                uid: id;
                server: self server;
                serverType: self serverType;
                deliveryAction: [self saveSettings: number];
                yourself
! !

!Pop3Mailbox class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Pop3Mailbox.st,v 1.3 2009/09/29 20:06:58 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Pop3Mailbox.st,v 1.3 2009/09/29 20:06:58 cg Exp $'
! !

Pop3Mailbox initialize!
