"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MimeMultipartEntity subclass:#RFC822Message
	instanceVariableNames:'date'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

RFC822Message comment:'This class represents top-level RFC822 messages. It adds accessor methods for all fields postulated inrfc822 (to, from, cc, bcc, data, etc.)

Instance Variables:

	date    <Timestamp>     internal message date'
!


!RFC822Message class methodsFor:'defaults'!

defaultCharset

        ^ MimeTypeDescriptor iso8859x1
! !

!RFC822Message methodsFor:'accessing'!

bcc
	^self getValueFor: 'bcc' ifAbsent: [ OrderedCollection new ]
!

bcc: aStringOrList
	self setAddressesAt: 'bcc' to: aStringOrList
!

cc
	^self getValueFor: 'cc' ifAbsent: [ OrderedCollection new ]
!

cc: aStringOrList
	self setAddressesAt: 'cc' to: aStringOrList
!

comments
	^self getValueFor: 'comments'
!

date
	^self fieldValueAt: 'date'
!

date: aTimestampOrString
	(self getFieldAt: 'date') value: aTimestampOrString
!

destinationAddresses
" Answer collection of destination addresses. These addresses have been preprocessed
to make them good for SMTP transport. This means removing everything but address-spec and
encoding alias based on the message charset "
	| coll f |
	coll := OrderedCollection new.
	self destinationFields
		do: [ :fn |
			(f := self fieldAt: fn ifAbsent: [nil]) ~~ nil
				ifTrue: [ coll addAll: (f addressesEncodedWith: self headerCharset)]].
	^coll
!

destinationFields
" Answer collection of fields that contain message destination addresses "

	^#('to' 'cc' 'bcc')
!

encrypted
	^self getValueFor: 'encrypted'
!

from
	^self getValueFor: 'from' ifAbsent: [ OrderedCollection new ]
!

from: aStringOrList
	self setAddressesAt: 'from' to: aStringOrList
!

id
	^self fieldValueAt: 'message-id'
!

id: aString
	(self getFieldAt: 'message-id') value: aString
!

inReplyTo
	^self getValueFor: 'in-reply-to' ifAbsent: [ OrderedCollection new ]
!

inReplyTo: aStringOrList
	self setAddressesAt: 'in-reply-to' to: aStringOrList
!

keywords
	^self getValueFor:  'keywords'
!

messageID
	^self getValueFor: 'message-id'
!

reference
	^self getValueFor: 'reference'
!

replyTo
	^self getValueFor: 'reply-to' ifAbsent: [ OrderedCollection new ]
!

replyTo: aStringOrList
	self setAddressesAt: 'reply-to' to: aStringOrList
!

resentBcc
	^self getValueFor: 'resent-bcc' ifAbsent: [ OrderedCollection new ]
!

resentBcc: aStringOrList
	self setAddressesAt: 'resent-bcc' to: aStringOrList
!

resentCc
	^self getValueFor: 'resent-cc' ifAbsent: [ OrderedCollection new ]
!

resentCc: aStringOrList
	self setAddressesAt: 'resent-cc' to: aStringOrList
!

resentDate
	^self getValueFor: 'resent-date'
!

resentFrom
	^self getValueFor: 'resent-from'
!

resentFrom: aStringOrList
	self setAddressesAt: 'resent-from' to: aStringOrList
!

resentMessageID
	^self getValueFor: 'resent-message-id'
!

resentReplyTo
	^self getValueFor: 'resent-reply-to'
!

resentReplyTo: aStringOrList
	self setAddressesAt: 'resent-reply-to' to: aStringOrList
!

resentSender
	^self getValueFor: 'resent-sender'
!

resentSender: aStringOrList
	self setAddressesAt: 'resent-sender' to: aStringOrList
!

resentTo
	^self getValueFor: 'resent-to' ifAbsent: [ OrderedCollection new ]
!

resentTo: aStringOrList
	self setAddressesAt: 'resent-to' to: aStringOrList
!

returnPath
	^self getValueFor: 'return-path'
!

sender
	^self getValueFor: 'sender'
!

setAddressesAt: fieldName to: aStringOrList
	| addresses |

	addresses := self addressesFrom: aStringOrList.
	^(self getFieldAt: fieldName) addresses: addresses
!

status
	^self getValueFor: 'status'
!

subject
	^self getValueFor: 'subject'
!

subject: aString
	^self fieldValueAt: 'subject' put: aString
!

timestamp
	^(self fieldAt: 'date' ifAbsent: [^nil]) timestamp
!

timestamp: aTimestamp
	^(self getFieldAt: 'date') timestamp: aTimestamp
!

to
	^self getValueFor: 'to' ifAbsent: [ OrderedCollection new ]
!

to: aStringOrList
	self setAddressesAt: 'to' to: aStringOrList
! !

!RFC822Message methodsFor:'parsing'!

addressesFrom: aListOrString
" RFC822Message new addressesFrom: 'George Jones <Group@Some-Reg.An-Org>,
		 Al.Neuman@MAD.Publisher' "

	" Canonicalize addresses "
	| coll addresses |
	coll := aListOrString isString
		ifTrue: [ OrderedCollection with: aListOrString]
		ifFalse: [ aListOrString ].
	addresses := OrderedCollection new.
	coll do:
		[ :addr |
		addr isString
			ifTrue:
				[addresses addAll:
					(NetworkAddressDescriptor addressesFrom: (self scannerOn: addr readStream))]
			ifFalse: [addresses add: addr]].
	^addresses
! !

!RFC822Message methodsFor:'private'!

encodedStringAt: fieldName
	^(self getFieldAt: fieldName) addressesEncodedWith: self headerCharset
! !

!RFC822Message methodsFor:'transfer encodings'!

transferFieldOrder
" rfc822 recommends field order (if present) to be as follows. Order for remaining fiedls does not matter "
	^#('return-path' 'received' 'date' 'from' 'subject' 'sender' 'to' 'cc' 'mime-version' 'content-type' 'content-length' 'content-transfer-encoding' )
! !

!RFC822Message class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__RFC822Message.st,v 1.2 2009/01/13 15:24:43 stefan Exp $'
! !
