"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

ScalarField subclass:#SingleMailboxField
	instanceVariableNames:'address'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

SingleMailboxField comment:'This class represents field whose value is a mailbox-spec as defined by RFC822. Several field
types in RFC822 messages are single mailbox fields. Mailbox allows to specify address-spec
as well as an alias. MIME allows to encode non-ASCII characters in the alias part.

Mailbox examples:
	Alfred Neuman <Neuman@BBN-TENEXA>
	"George, Ted" <Shared@Group.Arpanet>

The formal grammar is as follows:

     address     =  mailbox                      ; one addressee
		 /  group                        ; named list

     group       =  phrase ":" [#mailbox] ";"

     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec

     route-addr  =  "<" [route] addr-spec ">"

     route       =  1#("@" domain) ":"           ; path-relative

     addr-spec   =  local-part "@" domain        ; global address

     local-part  =  word *("." word)             ; uninterpreted
						 ; case-preserved

     domain      =  sub-domain *("." sub-domain)

     sub-domain  =  domain-ref / domain-literal

     domain-ref  =  atom                         ; symbolic reference

'
!


!SingleMailboxField class methodsFor:'parsing'!

fieldNames
	^#('sender' 'resent-sender' 'return-path' )
! !

!SingleMailboxField methodsFor:'accessing'!

address
	^address
!

address: aNetworkAddressDescriptor
	address := aNetworkAddressDescriptor.
	self value: (self sanitizeAddress: aNetworkAddressDescriptor)
!

addresses
	^self address
! !

!SingleMailboxField methodsFor:'parsing'!

doParse: rfc822Stream
" HeaderField fromLine: 'Sender :               Phil Campbell (The great) <philc@yahoo.com>' "
	address := NetworkAddressDescriptor addressFrom: rfc822Stream asStream.
	^self sanitizeAddress: address
! !

!SingleMailboxField methodsFor:'printing'!

addressString
	^self asString
!

asString
	^self address printString
! !

!SingleMailboxField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__SingleMailboxField.st,v 1.1 2004/11/12 10:44:05 penk Exp $'
! !
