"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

HeaderField subclass:#StructuredHeaderField
	instanceVariableNames:'comment'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

StructuredHeaderField comment:'This class is an abstract class for all structured fields. Each specific structured
field type has to implement its own parsing by overriding #parse: method. This
class only provides some utility method to help parse some common constructs
such as parameters.

Instance Variables:

	comment <String>        '
!


!StructuredHeaderField methodsFor:'accessing'!

canonicalValue
" Canonical value removes all white space and comments from the source "
	^self tokenizedValueFrom: (self scannerOn: self source readStream)
!

comment
	^comment
!

comment: aString
	comment := aString
! !

!StructuredHeaderField methodsFor:'composing'!

writeCommentOn: aStream encoding: aString
	self comment ~~ nil
		ifTrue:
			[aStream nextPutAll:
				(MimeEncodedWordCoDec
					encodeComment: self comment
					asBase64Using: aString)]
!

writeStructureOn: aStream encoding: aString
" Default implementation is the same as inherited. Subclasses can override it "
	self  printValueOn: aStream.
	self writeCommentOn: aStream encoding: aString
!

writeValueOn: aStream encoding: aString
	self writeStructureOn: aStream encoding: aString
! !

!StructuredHeaderField methodsFor:'printing'!

printCommentOn: aStream
	self class scannerType printComment: self comment on: aStream
!

printStructureOn: aStream
" Default implementation is the same as inherited. Subclasses can override it "
	super printValueOn: aStream.
	self printCommentOn: aStream
!

printValueOn: aStream
	self printStructureOn: aStream
! !

!StructuredHeaderField methodsFor:'private-utility'!

sanitizeAddress: addressOrString
	"Make addresses is strings"

	^(addressOrString isKindOf: NetworkEntityDescriptor)
		ifTrue: [addressOrString printString]
		ifFalse: [addressOrString]
!

sanitizeAddress: addressOrString encoding: aString
	"Make addresses as strings"
	^(addressOrString isKindOf: NetworkEntityDescriptor)
		ifTrue: [addressOrString stringWithEncoding: aString ]
		ifFalse:
			[MimeEncodedWordCoDec encodeIfRequired: addressOrString using: aString]
!

tokenize: rfc822Stream
	"Scan field value token by token. Answer an array of tokens"

	| result token |
	result := (Array new: 2) writeStream.
	[rfc822Stream atEnd or: [rfc822Stream peek == Character cr or: [(token := rfc822Stream nextToken) isNil]]]
		whileFalse: [result nextPut: token].
	^result contents
!

tokenizedValueFrom: rfc822Stream
	"Scan field value token by token. Answer a string that is a concatenation of all elements in the array. One can view this as a canonicalized field value because this operation eliminates all white space and comments "

	| result tokens |
	result := (String new: 32) writeStream.
	tokens := self tokenize: rfc822Stream.
	tokens
		do: [:token | token isString
				ifTrue: [result nextPutAll: token]
				ifFalse: [result nextPut: token]
].
	^result contents
! !

!StructuredHeaderField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__StructuredHeaderField.st,v 1.1 2004/11/12 10:41:43 penk Exp $'
! !
