"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

StructuredHeaderField subclass:#ValueWithParametersField
	instanceVariableNames:'separators'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

ValueWithParametersField comment:'This class is an  abstract class that provides infrastructure for Mime and HTTP fields that have value and optional parameters. The class defines logic for parsing and composition. Subclasses may perform more
detailed parsing of field value. Subclasses also add accessor methods to access specific parameters (such as ''charset'' or ''boundary'' in content-type field or ''fileName'' in content-disposition field

For example, the syntax of content-type field as defined in rfc2045 is as follows:

  content := "Content-Type" ":" type "/" subtype
	     *(";" parameter)
	     ; Matching of media type and subtype
	     ; is ALWAYS case-insensitive.

  parameter := attribute "=" value
  value := token / quoted-string

Some of the most important fields, namely content-type and content-disposition types are value with parameters fields.


'
!


!ValueWithParametersField class methodsFor:'instance creation'!

newWithSeparators: anArray
	^self new
		separators: anArray;
		yourself
! !

!ValueWithParametersField class methodsFor:'defaults'!

defaultSeparators
	^self subclassResponsibility
! !

!ValueWithParametersField methodsFor:'accessing'!

canMerge
	^true
!

defaultSeparators
	^self class defaultSeparators
!

element
	self first isNil
		ifTrue: [ self value add: (ValueWithParams newWithSeparators: self separators)].
	^self first
!

elementParams
	^self element params
!

first
	^self value first
!

groupSeparator
	^self separators at: 3
!

parametersSeparator
	^self separators at: 2
!

separators
	separators isNil
		ifTrue: [ separators := self defaultSeparators].
	^separators
!

separators: anArray
	separators := anArray
!

value
	^value isNil
		ifTrue: [ value := OrderedCollection new]
		ifFalse: [value]
!

valueSeparator
	^self separators first
! !

!ValueWithParametersField methodsFor:'adding'!

add: aValueWithParametersFiels
	self value addAll: aValueWithParametersFiels value
! !

!ValueWithParametersField methodsFor:'composing'!

writeValueOn: aStream encoding: aString
	self printValueOn: aStream
! !

!ValueWithParametersField methodsFor:'parsing'!

doParse: scanner
	| list |
	list := OrderedCollection new: 5.
	[scanner atEnd ]
		whileFalse:
			[list add:
				((ValueWithParams newWithSeparators: self separators)
						readFrom: scanner;
						yourself ).
			(scanner atEnd not and: [self groupSeparator ~~ Character space])
				ifTrue:
					[scanner skipWhiteSpace.
					(scanner next == self groupSeparator)
						ifFalse:
							[Transcript cr; show: (#DetectedInvalidField << #net >> 'Detected invalid field:'); cr; show: scanner contents.
							scanner upTo: self groupSeparator]]].
	^list
! !

!ValueWithParametersField methodsFor:'printing'!

printValueOn: aStream
	self value notNil
		ifTrue:
			[self value
				do: [ :param |  param printOn: aStream]
				separatedBy: [aStream nextPut: self groupSeparator].
			aStream cr.]
		ifFalse: [ aStream nextPutAll: self class name, ' value: nil']
! !

!ValueWithParametersField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ValueWithParametersField.st,v 1.1 2004/11/12 10:41:28 penk Exp $'
! !
