"{ Package: 'stx:goodies/net/ldap' }"

"{ NameSpace: LDAP }"

Object subclass:#LDAPRequest
	instanceVariableNames:'worker messageId waitingSemaphore isDone result isAbandoned'
	classVariableNames:''
	poolDictionaries:''
	category:'LDAP-Core'
!


!LDAPRequest methodsFor:'as yet unclassified'!

abandon
	"RFC says we MUST NOT send abandon requests for the same operation multiple times"
	isAbandoned ifFalse: [
		LDAPAbandonRequest new initializeWith: worker abandonId: messageId.
		worker delTargetFor: messageId.
		isAbandoned := true ]
	
!

checkForExceptions
        "Check to see if a response caused an exception and if so, raise it"
        self isDone ifFalse:[
            "LDAP timeout"
            ^ (LDAPException newWithCode:85) raiseRequest
        ].
        result checkForExceptions.
!

defaultTimeout
	"Answer the default timout for LDAP queries"
	^30 "seconds"
!

hasResult
	^ isDone
!

initializeWith: anLDAPWorker
	worker := anLDAPWorker.
	messageId := worker nextMessageId.
	waitingSemaphore := Semaphore new.
	isAbandoned := false.
	isDone := false.

	worker addTarget: self for: messageId.

	
!

isDone
	^ isDone
!

processResponse: aMessage
	isDone := true. 
	result := aMessage.
	waitingSemaphore signal.
!

result
	self wait.
	^ result
!

wait
        ^self waitTimeoutMSecs:self defaultTimeout * 1000
!

waitTimeoutMSecs: msecs
	result ifNil: [ waitingSemaphore waitTimeoutMSecs: msecs ].
	self checkForExceptions.
!

waitTimeoutSeconds: seconds
    ^ self waitTimeoutMSecs:seconds*1000
!

wantsMoreResponses
	"you most likely not want to implement this in a subclass, but processResponse:"
	^ self isDone not
! !

!LDAPRequest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/LDAP__LDAPRequest.st,v 1.2 2009/12/03 13:52:07 stefan Exp $'
! !
