"{ Package: 'stx:goodies/net/ldap' }"

"{ NameSpace: LDAP }"

LDAPRequest subclass:#LDAPSearchRequest
	instanceVariableNames:'partialResults iterator gotNewResultSemaphore searchResult
		resultCount'
	classVariableNames:''
	poolDictionaries:''
	category:'LDAP-Core'
!


!LDAPSearchRequest methodsFor:'as yet unclassified'!

checkForExceptions
        "Check to see if a response caused an exception and if so, raise it"
        self isDone ifFalse:[
            "LDAP timeout"
            ^ (LDAPException newWithCode:85) raiseRequest
        ].
        searchResult checkForExceptions.
!

initializeWith: anLDAPWorker base: aBase scope: aSearchScope deref: aDerefBehaviour filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly
	| encoded |
	partialResults := OrderedCollection new.
	iterator :=  0.
	resultCount := 0.
	gotNewResultSemaphore := Semaphore new.
	self initializeWith: anLDAPWorker.
	encoded := LDAPEncoder searchRequest: messageId  base: aBase scope: aSearchScope deref: aDerefBehaviour filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly.
	anLDAPWorker send: encoded.
!

next
	^self nextTimeout: self defaultTimeout.
!

nextTimeout: seconds
        isDone ifFalse:[
            "As long as the request isn't done, synchronously wait for more results"
            gotNewResultSemaphore waitTimeoutSeconds: seconds.
            (resultCount = iterator and:[isDone not]) ifTrue:[ 
                "LDAP timeout"
                ^ (LDAPException newWithCode:85) raiseRequest
            ].
        ].
        resultCount > iterator
                ifTrue:[^partialResults at: (iterator := iterator + 1)]
                ifFalse:[^nil]
!

processResponse: aSearchResultEntry
	aSearchResultEntry class isEndMarker ifTrue:[
		searchResult := aSearchResultEntry.
		isDone := true. 
		result := partialResults. 
		waitingSemaphore signal.
	] ifFalse:[
		partialResults addLast: aSearchResultEntry.
		resultCount := resultCount + 1.
	].
	gotNewResultSemaphore signal.
! !

!LDAPSearchRequest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/LDAP__LDAPSearchRequest.st,v 1.2 2009/12/03 13:52:20 stefan Exp $'
! !
