"{ Package: 'stx:goodies/net/ldap' }"

"{ NameSpace: LDAP }"

Object subclass:#LDAPWorker
	instanceVariableNames:'sockStream hostname port nextMessageId lock targets running'
	classVariableNames:''
	poolDictionaries:''
	category:'LDAP-Core'
!


!LDAPWorker class methodsFor:'as yet unclassified'!

newTo: aHostname port: aPort
	^ self basicNew initialize: aHostname port: aPort
! !

!LDAPWorker methodsFor:'*Cobalt-Router-CoLocalSecretAuthService'!

connectTLS
        | aSocketStream |

        Smalltalk isSmalltalkX ifTrue:[
            sockStream := SSL::SSLSocket newConnection:hostname port: port.
            sockStream isNil ifTrue:[
                OpenError raiseWith:self errorString:'cannot connect to LDAP server'
            ].
            sockStream
                binary;
                signalAtEnd:true.
        ] ifFalse:[
            aSocketStream := TLSSocketStream.
            [ sockStream := 
                    aSocketStream openConnectionToHostNamed: hostname port: port ]
                    on: Exception do: [:ex | sockStream := nil. self error: ex messageText].
            sockStream 
                    binary;
                    autoFlush: false;
                    noTimeout
        ].
! !

!LDAPWorker methodsFor:'as yet unclassified'!

addTarget: aRequest for: aMessageId
        lock critical: [
                | data |
                targets at: aMessageId ifPresent: [
                        self error: 'messageId already present -- shouldnt happen'. 
                ].
                data := Dictionary new.
                data    at: #request put: aRequest.
                targets at: aMessageId put: data.
        ]
!

connect
        Smalltalk isSmalltalkX ifTrue:[
            sockStream := Socket newTCPclientToHost:hostname port: port.
            sockStream isNil ifTrue:[
                OpenError raiseWith:self errorString:'cannot connect to LDAP server'
            ].
        ] ifFalse:[
            sockStream := SocketStream openConnectionToHostNamed: hostname port: port 
        ].

        sockStream 
                binary;
                signalAtEnd:true;
                "/ autoFlush: false;
                "/ noTimeout;
                yourself
!

delTargetFor: aMessageId
	lock critical: [
		targets at: aMessageId
			ifAbsent: [ ^ self ].
		 targets removeKey: aMessageId
	
	]
!

disconnect
	self stop
	
!

dispatchMessage: aMessage
	| target id who |
	id := aMessage id.

	"this is an unsolicited notification -- as it is advisory, we can ignore it for now"
	id = 0 ifTrue: [ ^ self ].

	"RFC says we MUST be prepared to recieve replies to aborted requests -- so we eat it silently"
	target :=targets at: id
		ifAbsent: [ ^ self ].
	
	target ifNil: [ self delTargetFor: id. ^ self ].
	
	"extended results need some kind of handling -- RFC2251 says they are the result of a message with a control,
	which we don't have implemented, so there's no real need yet"
	
	who := target at: #request.
	who processResponse: aMessage.
	
	who wantsMoreResponses
		ifFalse: [ self delTargetFor: id ]

!

initialize: aHostname port: aPort
	hostname := aHostname.
	port := aPort.
	nextMessageId := 0.
	lock := Semaphore forMutualExclusion.
	"looks like if we use the WVD here we wont see exceptions thrown.. hrmmm"
	"targets := WeakValueDictionary new."
	targets := Dictionary new.
	running := false.
!

nextMessageId
	"here we should check wether we're past the message id limit, and if so reconnect and bind"
	nextMessageId := nextMessageId + 1.
	^ nextMessageId
!

running
	^ running
!

running: aBoolean
	running := aBoolean
!

send: someData
	lock critical: [
		sockStream 
			nextPutAll: someData;
			flush.
	]
!

start
        | process |
        running := true.
        process := [ 
                [running] whileTrue: [
                        |  element  |
                        element := [LDAPMessage newFrom: sockStream] on: Error do:[:ex|
                                Transcript cr; show: ex description.
                                running := false.
                                "catching all errors is BAD!!!!!!"
                                ex reject.
                        ].
                        "Transcript show: '********* READ ONE OF:  ', element asString; cr."
                        running ifTrue:[self dispatchMessage: element].
                ]
        ] fork.
        process name:'LDAP worker'.
        ^ process
!

stop
	running := false.
! !

!LDAPWorker class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/LDAP__LDAPWorker.st,v 1.6 2011/11/24 14:24:35 stefan Exp $'
! !
