"{ Package: 'stx:goodies/net/ldap/tests' }"

"{ NameSpace: LDAP }"

TestCase subclass:#LDAPTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'LDAP-Tests'
!


!LDAPTest class methodsFor:'settings'!

baseDN
"/        ^ 'dc=linalco,dc=test'
        ^ 'dc=exept,dc=de'
!

bindDN
        ^ 'cn=Administrator,', self baseDN.
"/        ^ 'uid=stefan, ou=People,', self baseDN
!

hostname
"       ^ '10.2.0.12'"
        ^ 'exeptn.bh.exept.de'.
!

newSuite
        | obj |
        obj := TestSuite named: 'LDAP'.
        obj addTest: (LDAPTest selector: #testAll).
        ^ obj
!

password
        ^ 'please set the password'.
!

port
        ^ 389.
!

testDN
        ^ 'o=stxLDAPTest,', self baseDN.
! !

!LDAPTest class methodsFor:'tests'!

allTestSelectors
    "SUnit workaround"

    ^ self testSelectors
!

testAll
        "This test expects a single entry in the tree:
                dn: dc=linalco, dc=test
                dc: linalco
                objectClass: dcObject
                objectClass: organization
                o: linalco

         self testAll       
        "

        | l |
        l := LDAPTest new.

        self testSelectors do:[:eachTestSelector|
            l perform:eachTestSelector.
        ].
!

testSelectors
    ^ #(
                testBind
                testAnonymousBind
                testDoubleBind
                testAddWithCollectionValues 
                testSearch 
                testSearchWithNext 
                testSearchWithFilters 
                testDel 
                testAddWithStringValues 
                testSearch 
                testCompare 
                testModify 
                testSearch 
                testDel
     )
! !

!LDAPTest methodsFor:'as yet unclassified'!

checkSearchResults: aResult expecting: expectedRDNs
        aResult size  = expectedRDNs size
                ifFalse: [ ^ false ].
        
        expectedRDNs do: [:eachExpected | | qualified isPresent |
                eachExpected
                        ifNil: [ qualified := self class testDN ]
                        ifNotNil: [ qualified := eachExpected, ',', self class testDN ].
                isPresent := false.
                aResult do: [ :eachResult |
                        eachResult dn = qualified
                                ifTrue: [ isPresent := true ]  ].
                isPresent ifFalse: [^ false].
        ].
        ^ true
!

connect
        | req conn |
        conn := LDAPConnection to: self class hostname port: self class port.
        req := conn bindAs: self class bindDN credentials: self class password.
        [
            req wait.
        ] ifCurtailed:[
            conn disconnect.
        ].
        ^ conn
! !

!LDAPTest methodsFor:'setup'!

setUp
    self class password = 'ChangeThisToRealPassword' ifTrue:[
        self warn:'Please set the password first. Aborting...'.
        AbortAllOperationRequest raiseWith:self errorString:'Please set the password first'.
    ].
! !

!LDAPTest methodsFor:'tests'!

testAddWithCollectionValues
        | conn req attrs |

        conn := self connect.
        [
            (1 to: 5) do: [ :each | |name|
                    name := 'test', each asString.
            
                    attrs := Dictionary new
                            at: 'objectClass' put: (OrderedCollection new add: 'person'; yourself);
                            at: 'cn' put: (OrderedCollection new add: name; yourself);
                            at: 'sn' put: (OrderedCollection new add: name; yourself);
                            yourself.
                    
                    req := conn addEntry: 'cn=',name,',', self class testDN attrs: attrs.
                    req wait.
            ].
        ] ensure:[
            conn disconnect.
        ].
!

testAddWithStringValues
        | conn req attrs |
        conn := self connect.
        [
            (1 to: 5) do: [ :each | |name|
                    name := 'test', each asString.
            
                    attrs := Dictionary new
                            at: 'objectClass' put: 'person';
                            at: 'cn' put: name;
                            at: 'sn' put: name;
                            yourself.
                    
                    req := conn addEntry: 'cn=',name,',', self class testDN attrs: attrs.
                    req wait.
            ].
        ] ensure:[
            conn disconnect.
        ].
!

testAnonymousBind
        | conn req |
        conn := self connect.
        [
            req := conn bindAs:'' credentials:''.
            req wait.
        ] ensure:[
            conn disconnect.
        ].
!

testBind
	| conn |
	conn := self connect.
	conn disconnect.
!

testCompare
        | conn req |
        conn := self connect.
        [
            req :=conn compare: 'cn=test1,', self class testDN using: (LDAPFilterEquality new setType: 'cn' value: 'test1').
            req wait.
            self assert: (req result == true).

            req :=conn compare: 'cn=test1,', self class testDN using: (LDAPFilterEquality new setType: 'cn' value: 'test2').
            req wait.
            self deny: (req result == true).
        ] ensure:[
            conn disconnect.
        ].
!

testDel
        | conn req |
        conn := self connect.
        [
            (1 to: 5) do: [ :each | |name|
                    name := 'test', each asString.
                    req := conn delEntry: 'cn=',name,',', self class testDN.
                    req wait.
            ].
        ] ensure:[
            conn disconnect.
        ].
!

testDoubleBind
        | conn req |
        conn := self connect.
        [
            req := conn bindAs: self class bindDN credentials: self class password.
            req wait.
        ] ensure:[
            conn disconnect.
        ].
!

testModify
        | conn req rr ops |
        conn := self connect.
        [
            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: (LDAPFilter with: 'cn' equalTo: 'test1') attrs: nil wantAttrsOnly: false.
            req wait.
            self assert:(req result notEmpty).
            rr := req result at: 1.
            
            ops :=  { 
                            LDAPAttrModifier set: 'sn' to: { 'test5sn' . 'foo' . 'bar'}.
                            LDAPAttrModifier addTo: 'description' values: {'rchueo'} }.

            req := conn modify: (rr dn) with: ops.
            req wait.
            
            req :=conn compare: 'cn=test1,', self class testDN using: (LDAPFilterEquality new setType: 'sn' value: 'test5sn').
            req wait.
            self assert: (req result).
        ] ensure:[
            conn disconnect.
        ].
!

testSearch
        | conn req rr |
        conn := self connect.
        [
            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: nil attrs: nil wantAttrsOnly: false.
            rr := req result.
            self assert: (self checkSearchResults: rr expecting: { nil . 'cn=test1'. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5' } ).
        ] ensure:[
            conn disconnect.
        ].
!

testSearchWithFilters
        | conn req rr |
        conn := self connect.
        [
            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: (LDAPFilter with: 'objectclass') attrs: nil wantAttrsOnly: false.
            rr := req result.
            self assert: (self checkSearchResults: rr expecting: { nil . 'cn=test1'. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5' }).

            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: (LDAPFilter with: 'cn' equalTo: 'test1') attrs: nil wantAttrsOnly: false.
            rr := req result.
            self assert: (self checkSearchResults: rr expecting: { 'cn=test1'} ).

            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: (LDAPFilter not: (LDAPFilter with: 'cn' equalTo: 'test1'))  attrs: nil wantAttrsOnly: false.
            rr := req result.
            self assert: (self checkSearchResults: rr expecting: { nil. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5' }).

            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter:
            (LDAPFilter andOf: (OrderedCollection new
                                                    add: (LDAPFilter not: (LDAPFilter with: 'cn' equalTo: 'test1')); 
                                                    add: (LDAPFilter not: (LDAPFilter with: 'sn' equalTo: 'test2'));
                                                    yourself ))
                                              attrs: nil wantAttrsOnly: false.
            rr := req result.
            self assert: (self checkSearchResults: rr expecting: { nil . 'cn=test3' .  'cn=test4'. 'cn=test5' }).
        ] ensure:[
            conn disconnect.
        ].
!

testSearchWithNext
        | conn req rr r |
        conn := self connect.
        [
            req := conn newSearch: (self class testDN) scope: (LDAPConnection wholeSubtree) deref: (LDAPConnection derefNever) filter: nil attrs: nil wantAttrsOnly: false.
            rr := OrderedCollection new.
            [(r := req next) == nil] whileFalse: [rr add: r ].
        ] ensure:[
            conn disconnect.
        ].
! !

!LDAPTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/tests/LDAP__LDAPTest.st,v 1.4 2009/12/03 14:07:16 stefan Exp $'
! !
