"{ NameSpace: NN }"

NeuralNetworkTrainer subclass:#BackPropagationTrainer
	instanceVariableNames:'rate momentum units outputs'
	classVariableNames:''
	poolDictionaries:''
	category:'Neural Networks'
!

BackPropagationTrainer comment:'My instances are back-propagation training algorithms, which are used to train multilayer-perceptron networks. See the comment in the PerceptronNetwork class.

My implementation is not the most naive of all. See the comment in BackPropagationUnit class.
'!


!BackPropagationTrainer class methodsFor:'examples'!

xor
	"Answer a back propagation trainer to solve the XOR problem."
	| i1 i2 n1 n2 output network |
	i1 _ NeuronInput new. i2 _ NeuronInput new.
	n1 _ Neuron new
		inputs: (Array with: i1 with: i2); function: SigmoidFunction new.
	n2 _ Neuron new
		inputs: (Array with: i1 with: i2); function: SigmoidFunction new.
	output _ Neuron new
		inputs: (Array with: n1 with: n2); function: SigmoidFunction new.
	network _ PerceptronNetwork inputs: (Array with: i1 with: i2) outputs: (Array with: output).
	network randomize.
	^ self network: network

"Try something like this on a workspace:

""create the initial network, the trainer and the samples:""
trainer _ BackPropagationTrainer xor.
network _ trainer network.
samples _ OrderedCollection new.
samples
	add: #(0.0 0.0) -> #(0.0);
	add: #(1.0 1.0) -> #(0.0);
	add: #(0.0 1.0) -> #(1.0);
	add: #(1.0 0.0) -> #(1.0).

""do 200 epoches and take a look at the resulting network:
(do this many times to see how the network evolves)""
trainer epoch: samples times: 200.
#((1 1) (0 0) (1 0) (0 1)) collect: [ :each | network valueAt: each]
"! !

!BackPropagationTrainer methodsFor:'accessing'!

momentum
	"Answer the momentum parameter of the receiver."
	^ momentum!

outputs
	outputs isNil ifTrue: [self initializeOutputs].
	^ outputs!

rate
	"Answer the learning-rate of the receiver."
	^ rate!

sample: aCollection desired: anotherCollection
	self units do: [ :each | each reset].
	self network valueAt: aCollection.
	1 to: self outputs size do: [ :each |
		(self outputs at: each) desired: (anotherCollection at: each)].
	self units do: [ :each | each adjust]!

units
	units isNil ifTrue: [self initializeUnits].
	^ units! !

!BackPropagationTrainer methodsFor:'accessing-private'!

momentum: aNumber
	momentum _ aNumber!

outputs: aCollection
	outputs _ aCollection!

rate: aNumber
	rate _ aNumber!

units: aCollection
	units _ aCollection! !

!BackPropagationTrainer methodsFor:'initialization'!

initialize
	super initialize.
	self rate: 0.75.
	self momentum: 1.0 - 0.75!

initializeOutputs
	outputs _ self network outputs
		collect: [ :each | BackPropagationUnit neuron: each rate: self rate]!

initializeUnits
	units _ self outputs copy asOrderedCollection.
	self network hiddenLayers reverseDo: [ :each |
		each do: [ :other |
			units add: (BackPropagationUnit neuron: other rate: self rate momentum: self momentum)]].
	units do: [ :each |
		each neuron inputs do: [ :other |
			other class == NeuronInput
				ifFalse: [(units detect: [ :one | one neuron = other]) outputs add: each]]].
	units do: [ :each | each outputs: (units select: [ :other | other neuron inputs includes: each neuron])]! !

!BackPropagationTrainer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/neuralNetworks/BackPropagationTrainer.st,v 1.1 1999/07/26 14:42:29 cg Exp $'
! !
