"{ NameSpace: NN }"

Object subclass:#BackPropagationUnit
	instanceVariableNames:'neuron outputs gradient deltas rate momentum'
	classVariableNames:''
	poolDictionaries:''
	category:'Neural Networks'
!

BackPropagationUnit comment:'My instances are back-propagation units, parts of the back-propagation algorithm. See the comment in BackPropagationTrainer class.

My implementation is not the most naive of all. I know a learning-ratio parameter and a moment parameter.'!


!BackPropagationUnit class methodsFor:'instance creation'!

neuron: aNeuron rate: aNumber
	"Answer a new instance of the reciever with
	neuron aNeuron and learning-rate aNumber."

	^ self neuron: aNeuron rate: aNumber momentum: 1.0 - aNumber!

neuron: aNeuron rate: aNumber momentum: anotherNumber
	"Answer a new instance of the reciever with neuron aNeuron,
	learning-rate aNumber and momentum parameter anotherNumber."

	^ self new neuron: aNeuron; rate: aNumber; momentum: anotherNumber!

new
	^ super new initialize! !

!BackPropagationUnit methodsFor:'accessing'!

adjust
	"Adjust the synaptic weights of the neuron of the receiver."
	| weights inputs delta n |
	weights _ self neuron weights.
	inputs _ self neuron inputs.
	1 to: weights size do: [ :each |
		delta _ self rate * self gradient * (inputs at: each) signal +
			    (self momentum * (self deltas at: each)).
		self deltas at: each put: delta.
		weights at: each put: delta + (weights at: each)].
	n _ weights size + 1.
	delta _ self momentum * (self deltas at: n) - (self rate * self gradient).
	self deltas at: n put: delta.
	self neuron threshold: delta + self neuron threshold!

deltas
	"Answer the deltas of the last adjustment."
	deltas isNil ifTrue: [deltas _ (1 to: self neuron fanIn + 1) collect: [ :each | 0.0]].
	^ deltas!

desired: aNumber
	"Compute the gradient of the receiver knowing that the
	desired response for the neuron of the receiver is aNumber."
	| error |
	error _ aNumber - self neuron signal.
	self gradient: error * (self neuron function
							derivativeValueAt: self neuron sum)!

gradient
	"Answer the local gradient of the receiver."
	gradient isNil ifTrue: [self computeGradient].
	^ gradient!

momentum
	"Answer the momentum of the receiver."
	^ momentum!

neuron
	"Answer the neuron of the receiver."
	^ neuron!

outputs
	"Answer the outputs of the receiver."
	^ outputs!

rate
	"Answer the learning-rate of the receiver."
	^ rate!

reset
	self gradient: nil!

weightFor: aBackPropagationUnit
	"Answer the weight for the argument."
	^ self neuron weights at: (self neuron inputs indexOf: aBackPropagationUnit neuron)! !

!BackPropagationUnit methodsFor:'accessing-private'!

deltas: aCollection
	deltas _ aCollection!

gradient: aNumber
	gradient _ aNumber!

momentum: aNumber
	momentum _ aNumber!

neuron: aNeuron
	neuron _ aNeuron!

outputs: aCollection
	outputs _ aCollection!

rate: aNumber
	rate _ aNumber! !

!BackPropagationUnit methodsFor:'initialization'!

initialize
	self outputs: OrderedCollection new! !

!BackPropagationUnit methodsFor:'private'!

computeGradient
	| sum |
	sum _ 0.0.
	self outputs do: [ :each |
		sum _ each gradient * (each weightFor: self) + sum].
	self gradient: sum * (self neuron function derivativeValueAt: self neuron sum)! !

!BackPropagationUnit class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/neuralNetworks/BackPropagationUnit.st,v 1.1 1999/07/26 14:42:41 cg Exp $'
! !
