"{ NameSpace: NN }"

Object subclass:#NeuralNetwork
	instanceVariableNames:'inputs hidden outputs'
	classVariableNames:''
	poolDictionaries:''
	category:'Neural Networks'
!

NeuralNetwork comment:'I am an abstract class. My subclasses are concrete implementations of neural networks.'!


!NeuralNetwork class methodsFor:'instance creation'!

inputs: aCollection outputs: anotherCollection
	"Answer a new instance of the receiver with
	inputs and outputs given by the arguments."

	^ self new inputs: aCollection; outputs: anotherCollection! !

!NeuralNetwork class methodsFor:'examples'!

xor
	"NeuralNetwork xor."

	| inputs n1 n2 output |
	inputs _ Array with: NeuronInput new with: NeuronInput new.
	n1 _ Neuron new
			inputs: inputs;
			weights: #(1 1);
			threshold: 3/2;
			function: ThresholdFunction new.
	n2 _ Neuron new
			inputs: inputs;
			weights: #(1 1);
			threshold: 1/2;
			function: ThresholdFunction new.
	output _ Neuron new
			inputs: (Array with: n1 with: n2);
			weights: #(-2 1);
			threshold: 1/2;
			function: ThresholdFunction new.
	^ self inputs: inputs outputs: (Array with: output)! !

!NeuralNetwork methodsFor:'accessing'!

hidden
	"Answer the hidden layer of the receiver."
	hidden isNil ifTrue: [self computeHiddenLayer].
	^ hidden!

inputs
	"Answer the input layer of the receiver."
	^ inputs!

neurons
	^ self hidden copy; addAll: self outputs; yourself!

outputs
	"Answer the output layer of the receiver."
	^ outputs!

randomize
	| random |
	random _ Random new.
	self do: [ :each | each randomizeWith: random]!

reset
	self hidden do: [ :each | each reset].
	self outputs do: [ :each | each reset]!

step
	"Perform one step of the receiver."
	self reset. self outputs do: [ :each | each step]!

trainer
	"Answer an appropiate trainer for the receiver."
	self subclassResponsibility!

valueAt: aCollection
	"Answer the output corresponding to the inputs given by the argument."
	self reset.
	1 to: aCollection size do: [ :each |
		(self inputs at: each) signal: (aCollection at: each)].
	^ self outputs collect: [ :each | each signal]! !

!NeuralNetwork methodsFor:'accessing-private'!

hidden: aCollection
	hidden _ aCollection!

inputs: aCollection
	inputs _ aCollection!

outputs: aCollection
	outputs _ aCollection! !

!NeuralNetwork methodsFor:'enumerating'!

do: aBlock
	"Enumerate the neurons of the receiver."
	self hidden do: aBlock.
	self outputs do: aBlock! !

!NeuralNetwork methodsFor:'private'!

computeHiddenLayer
	| layer |
	layer _ Set new.
	self outputs do: [ :each |
		each formers do: [ :other |
			((self inputs includes: other) or: [self outputs includes: other])
				ifFalse: [layer add: other]]].
	self hidden: layer! !

!NeuralNetwork class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/neuralNetworks/NeuralNetwork.st,v 1.1 1999/07/26 14:42:37 cg Exp $'
! !
