"{ NameSpace: NN }"

Object subclass:#Neuron
	instanceVariableNames:'inputs weights threshold function signal'
	classVariableNames:''
	poolDictionaries:''
	category:'Neural Networks'
!

Neuron comment:'My instances are neurons.

Each neuron knows a set of imputs (other Neurons or NeuronInputs), a list of wheights, a threshold, and an activation function; so, synapsis and axon are implicitly modeled in the neuron.

Each neuron keeps the last state (it''s last output); to reach it, send a neuron the message #signal, and to reset the state send it the message #reset.
 '!


!Neuron methodsFor:'accessing'!

at: anInteger
	^ self weights at: anInteger!

at: anInteger put: anObject
	^ self weights at: anInteger put: anObject!

depth
	| idepth |
	idepth _ 0.
	self inputs do: [ :each | idepth _ idepth max: each depth].
	^ idepth + 1!

fanIn
	"Answer the fan-in of the receiver."
	^ self inputs size!

formers
	"Answer the former neurons of the receiver."
	^ self formersExceptThoseOf: Set new!

function
	"Answer the activation function of the receiver."
	^ function!

inputs
	"Answer the inputs of the receiver."
	^ inputs!

randomize
	"Choose small random values for the weights and threshold."
	self randomizeWith: Random new!

randomizeWith: aRandom
	"Choose small random values for the weights and threshold
	using the random number generator aRandom."
	| scale |
	scale _ 2.0 * 2.4 / self fanIn. "heuristical good choise"
	self weights: ((1 to: self size) collect: [ :each | (aRandom next - 0.5) * scale]).
	self threshold: (aRandom next - 0.5) * scale!

reset
	self signal: nil!

signal
	"Answer the signal of the receiver."
	signal isNil ifTrue: [self step].
	^ signal!

size
	^ self inputs size!

step
	"Perform one step of the receiver."
	self signal: (self function valueAt: self sum)!

sum
	"Answer the sum associated with the receiver."
	| sum |
	sum _ 0.0.
	1 to: self inputs size do: [ :each |
		sum _ (self inputs at: each) signal * (self weights at: each) + sum].
	^ sum - self threshold!

threshold
	"Answer the threshold of the receiver."
	^ threshold!

weights
	"Answer the synaptic weights of the receiver."
	^ weights! !

!Neuron methodsFor:'accessing-private'!

function: anObject
	function _ anObject!

inputs: aCollection
	inputs _ aCollection!

signal: anObject
	signal _ anObject!

threshold: aNumber
	threshold _ aNumber!

weights: aCollection
	weights _ aCollection! !

!Neuron methodsFor:'private'!

formersExceptThoseOf: aCollection
	| answer |
	answer _ Set new.
	(aCollection includes: self) ifTrue: [^ answer].
	aCollection add: self.
	answer addAll: self inputs.
	self inputs do: [ :each | answer addAll: (each formersExceptThoseOf: aCollection)].
	^ answer! !

!Neuron class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/neuralNetworks/Neuron.st,v 1.1 1999/07/26 14:42:25 cg Exp $'
! !
