"{ Package: 'stx:goodies/oMeta2' }"

OMeta2Base subclass:#OMeta2
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'OMeta2'
!

OMeta2 comment:'TODOS:
* implement OMeta -> Squeak translator
* implement Squeak parser
* implement OMeta/Squeak "compiler", make it OMeta2''s compilerClass
* rewrite #char, #digit, #empty, #end, #exactly, #firstAndRest, #fromTo, #letter, #letterOrDigit, #listOf, #lower, #notLast, #number, #range, #space, #spaces, #string, #symbol, #token, and #upper in OMeta syntax
* consider implementing position-related functionality (see http://www.tinlizzie.org/ometa-js/ometa-base.js)
* consider the optimization suggestions in the comments of OMeta2Lib''s methods'
!


!OMeta2 class methodsFor:'*additions'!

firstPrecodeCommentFor:selector 
    "For compatibility with squeak dev"
    
    ^ nil

    "Modified: / 30-01-2011 / 00:50:03 / cg"
! !

!OMeta2 methodsFor:'rules'!

char
    |x|

    x := self apply:#anything.
    self pred:x isCharacter.
    ^ x

    "Modified: / 30-01-2011 / 00:48:51 / cg"
!

digit
    |x|

    x := self apply:#char.
    self pred:x isDigit.
    ^ x

    "Modified: / 30-01-2011 / 00:48:54 / cg"
!

empty
    ^ true

    "Modified: / 30-01-2011 / 00:48:57 / cg"
!

end
    " may want to inline the sends below, since this rule gets used a lot "
    
    ^ self not:[ self apply:#anything ]

    "Modified: / 30-01-2011 / 00:48:59 / cg"
!

firstAndRest
    |first rest|

    first := self apply:#anything.
    rest := self apply:#anything.
    ^ self 
        genericMany:[ self apply:rest ]
        into:(OrderedCollection with:(self apply:first))

    "Modified: / 30-01-2011 / 00:49:11 / cg"
!

fromTo
    |x y|

    x := self apply:#string.
    y := self apply:#string.
    self apply:#seq withArgs:{ x }.
    self 
        many:[
            self not:[ self apply:#seq withArgs:{ y } ].
            self apply:#char
        ].
    self apply:#seq withArgs:{ y }

    "Modified: / 30-01-2011 / 00:49:14 / cg"
!

letter
    |x|

    x := self apply:#char.
    self pred:x isLetter.
    ^ x

    "Modified: / 30-01-2011 / 00:49:18 / cg"
!

letterOrDigit
    |x|

    x := self apply:#char.
    self pred:x isAlphaNumeric.
    ^ x

    "Modified: / 30-01-2011 / 00:49:20 / cg"
!

lower
    |x|

    x := self apply:#char.
    self pred:x isLowercase.
    ^ x

    "Modified: / 30-01-2011 / 00:49:28 / cg"
!

number
    |x|

    x := self apply:#anything.
    self pred:x isNumber.
    ^ x

    "Modified: / 30-01-2011 / 00:49:34 / cg"
!

space
    |x|

    x := self apply:#char.
    self pred:x asciiValue <= 32.
    ^ x

    "Modified: / 30-01-2011 / 00:49:41 / cg"
!

spaces
    ^ self many:[ self apply:#space ]

    "Modified: / 30-01-2011 / 00:49:43 / cg"
!

string
    |x|

    x := self apply:#anything.
    self pred:x isString.
    ^ x

    "Modified: / 30-01-2011 / 00:49:45 / cg"
!

symbol
    |x|

    x := self apply:#anything.
    self pred:x isSymbol.
    ^ x

    "Modified: / 30-01-2011 / 00:49:49 / cg"
!

upper
    |x|

    x := self apply:#char.
    self pred:x isUppercase.
    ^ x

    "Modified: / 30-01-2011 / 00:49:57 / cg"
! !

!OMeta2 methodsFor:'rules-meta'!

exactly
    |wanted got|

    wanted := self apply:#anything.
    got := self apply:#anything.
    self pred:wanted = got.
    ^ wanted

    "Modified: / 30-01-2011 / 00:49:09 / cg"
!

listOf
    |rule delim|

    rule := self apply:#anything.
    delim := self apply:#anything.
    ^ self ometaOr:
            { [
            |x|

            x := self apply:rule.
            self 
                genericMany:[
                    self apply:#token withArgs:{ delim }.
                    self apply:rule
                ]
                into:(OrderedCollection with:x)
        ].
            [ #() ] }

    "Modified: / 30-01-2011 / 00:49:23 / cg"
!

notLast
    |rule ans|

    rule := self apply:#anything.
    ans := self apply:rule.
    self lookahead:[ self apply:rule ].
    ^ ans

    "Modified: / 30-01-2011 / 00:49:32 / cg"
!

range
    |a b x|

    a := self apply:#anything.
    b := self apply:#anything.
    x := self apply:#anything.
    self pred:((a <= x & x) <= b).
    ^ x

    "Modified: / 30-01-2011 / 00:49:38 / cg"
!

token
    |t|

    t := self apply:#anything.
    self apply:#spaces.
    ^ self apply:#seq withArgs:{ t }

    "Modified: / 30-01-2011 / 00:49:54 / cg"
! !

!OMeta2 class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/oMeta2/OMeta2.st,v 1.2 2011/01/30 00:36:11 cg Exp $'
! !
