"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

BitStream subclass:#BitReadStream
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!BitReadStream methodsFor:'accessing'!

next
	"^an Integer
I return an integer which contains the next bit from
my underlying stream."

	^self next: 1
			
!

next: requiredBits 
	"^an Integer
I return an integer which contains the next anInteger bits from
my underlying stream."

	| code |
	code := 0.
	requiredBits to: 1
		by: -1
		do: 
			[:targetBit | 
			self nextBit == 1 
				ifTrue: [code := code bitOr: (1 bitShift: targetBit - 1)]].
	^code
			
!

skipToStartOfNextByte
	"^self
I skip up to the next 'start of byte', which means I end up pointing at the first bit of a byte. If I'm already pointing at the first bit, I do nothing."

	(self currentBitPosition == self firstBitIndex) ifFalse: [self nextByte].
	^self
			
! !

!BitReadStream methodsFor:'initialize-release'!

initialize
	"^self
I set my internal values.  In the case of currentBitPosition I explicitly set
the value (see self>>currentBitPosition for the reason)."

	super initialize.
	self byteStream reset.
	self nextByte.
	^self
			
! !

!BitReadStream methodsFor:'private'!

nextBit
	"^an Integer
I return the next bit from my underlying byte array (either a 1 or a 0)."

	| nextBit |
	self atEndOfByte ifTrue: [self nextByte].
	nextBit := self currentByte bitAt: self currentBitPosition.
	self incrementCurrentBitPosition.
	^nextBit
			
!

nextByte
	"^self
I read the next byte from my byteStream.  I set the current
bit position to 8."

	self byteStream atEnd ifTrue: [BitStreamOutOfBounds signal].
	self currentByte: self byteStream next.
	self resetCurrentBitPosition.
	^self
			
! !

!BitReadStream methodsFor:'testing'!

atEnd
	"^a Boolean
I return true if my underlying stream is at end."

	^self byteStream atEnd
			
! !

!BitReadStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__BitReadStream.st,v 1.1 2006/09/07 11:11:46 sr Exp $'
! !
