"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#BitStream
	instanceVariableNames:'byteStream currentByte currentBitPosition highOrderBitFirst'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!BitStream class methodsFor:'instance creation'!

onByteStream: aByteStream 
	"^aBitReadStream
I return an instance of myself initialized to read/write
bytes to/from aByteStream.  The diretion depends on 
which of my subclasses you are instantiating."

	^self new onByteStream: aByteStream
			
! !

!BitStream methodsFor:'accessing'!

currentByte
	"^an Integer
I return the integer which is the value of the current byte I'm working
on (reading from / writing to).  I do not use lazy initialization here because
this is a *really* heavily used method.  See self>>initialize."

	^currentByte
			
!

currentByte: anInteger 
	"^self
I record my current byte. c.f. self>>currentByte."

	currentByte := anInteger.
	^self
			
!

position
	"^an Integer
I return my position (in bits, of course)."

	^self highOrderBitIsFirst 
		ifTrue: [self byteStream position * 8 + (9 - self currentBitPosition)]
		ifFalse: [self byteStream position * 8 + self currentBitPosition]
			
!

reset
	"^self
I reset my underlying stream.  The position set back so that the
next bit read will be the first in my underlying byte array."

	self initialize.
	^self
			
!

setHighOrderBitFirst
	"^self
I set myself to have the high order bit as the first bit within
a byte.  So, I'll read/write bit 2**7 first, and bit 2**0 last."

	highOrderBitFirst := true.
	^self
			
!

setLowOrderBitFirst
	"^self
I set myself to have the low order bit as the first bit within
a byte.  So, I'll read/write bit 2**0 first, and bit 2**7 last.
I reset myself to the begining of my underlying array."

	highOrderBitFirst := false.
	self reset.
	^self
			
! !

!BitStream methodsFor:'initialize-release'!

initialize
	"^self
There is no abstract initialization.  My sbclasses must implement this, though."

	^self
			
!

onByteStream: aByteStream 
	"^aBitReadStream
I return myself on  the underlying byte stream on aByteArray.
I'm initially positioned at the first bit in the stream."

	byteStream := aByteStream.
	self initialize.
	^self
			
! !

!BitStream methodsFor:'private'!

byteStream
	"^a ByteStream
I return the ByteStream on my underlying encoded byte array."

	^byteStream
			
!

currentBitPosition
	"^an Integer
I return the number of the current target bit in the current byte.  By default, bytes are written 'left to right' and the left most bit is bit number 8 and the right most bit is bit number 1.  This ordering may be reversed in some situations.  See self>>initialize for more on bit ordering.  I don't use lazy initialization because this is such a heavily used method that the 'ifTrue' has a significant impact on performance (as shown during profiling tests)."

	^currentBitPosition
			
!

currentBitPosition: anInteger 
	"^self
I record the current bit offset within the current byte.
c.f. self>>currentBitPosition"

	currentBitPosition := anInteger.
	^self
			
!

incrementCurrentBitPosition
	"^self
I move to the next bit position.  I either add 1 or -1 to the
current bit position depending on"

	self highOrderBitIsFirst 
		ifTrue: [self currentBitPosition: self currentBitPosition - 1]
		ifFalse: [self currentBitPosition: self currentBitPosition + 1].
	^self
			
!

resetCurrentBitPosition
	"^self
I move the next bit position to the first bit in the current byte."

	self currentBitPosition: self firstBitIndex.
	^self
			
! !

!BitStream methodsFor:'services'!

atEndOfByte
	"^a Boolean
I return true if I'm currently positioned beyond the end of the
current byte.  I just check that my current bit position is between
1 and 8."

	^(self currentBitPosition between: 1 and: 8) not
			
!

firstBitIndex
	"^an Integer
I return the index of the 'first' bit within a byte."

	^self highOrderBitIsFirst ifTrue: [8] ifFalse: [1]
			
!

highOrderBitIsFirst
	"^a Boolean
I return true if the first bit in a byte is the high order bit, that
is the 2**7 bit.  Otherwise, I return false.  The default is true."

	highOrderBitFirst isNil ifTrue: [highOrderBitFirst := true].
	^highOrderBitFirst
			
!

lastBitIndex
	"^an Integer
I return the index of the 'last' bit within a byte."

	^self highOrderBitIsFirst ifTrue: [1] ifFalse: [8]
			
!

lowOrderBitIsFirst
	"^a Boolean
I return true if the first bit in a byte is the high order bit, that
is the 2**1 bit.  Otherwise, I return false."

	^self highOrderBitIsFirst not
			
! !

!BitStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__BitStream.st,v 1.1 2006/09/07 11:13:50 sr Exp $'
! !
