"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

BitStream subclass:#BitWriteStream
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!BitWriteStream methodsFor:'accessing'!

close
	"^self
I write out my current byte and close my underlying byte stream.  I also
set my underlying bytestream to nil, just to be safe."

	self currentByte isNil ifFalse: [self byteStream nextPut: self currentByte].
	byteStream close.
	byteStream := nil.
	^self
			
!

contents
	"^a ByteArray
I return the byte array in my stream."

	^self byteStream contents
			
! !

!BitWriteStream methodsFor:'initialize-release'!

initialize
	"^self
I initialize myself.  This just involves initializing my byte."

	super initialize.
	self initializeByte.
	^self
			
!

initializeByte
	"^self
I set my internal values.  In the case of currentBitPosition I explicitly set
the value (see self>>currentBitPosition for the reason)."

	self currentBitPosition: 8.
	self currentByte: 0.
	^self
			
! !

!BitWriteStream methodsFor:'writing'!

flushByte
	"^self
I write my current byte if it has been written to.  So, if the current
bit count is 8, nothing has been put in this byte, so I won't flush
it.  Otherwise, I will."

	self currentBitPosition == 8 
		ifFalse: 
			[self byteStream nextPut: self currentByte.
			self initializeByte].
	^self
			
!

next: anInteger put: aBit 
	"^self
I write anInteger occurances of aBit to my underlying stream.  Initially
this was implemented as a simple anInteger timesRepeat deal, but as
what's wanted is a run of 1600 1's this is very ineficient.  So, if this is
a request to write out enough bits to make one whole byte (or more) I now
write out the remaining bits for the current byte, write the byte, then write
out whole bytes (either 0 or 255) followed by any remaining bits in a new
byte.  Anyway, that's the plan."

	| outstandingBits byteValue |
	anInteger > self currentBitPosition 
		ifTrue: 
			[outstandingBits := anInteger - self currentBitPosition.
			self currentBitPosition timesRepeat: [self nextPut: aBit].
			byteValue := aBit == 1 ifTrue: [255] ifFalse: [0].
			self byteStream next: outstandingBits // 8 put: byteValue.
			outstandingBits \\ 8 timesRepeat: [self nextPut: aBit]]
		ifFalse: [anInteger timesRepeat: [self nextPut: aBit]].
	^self
			
!

nextPut: aBit 
	"^self
I write aBit to my underlying stream.
NOTE:  This method *directly* sets instance variables.  This is a performance compromise due to the extremely heavy use of the method."

	| bitPosition |
	bitPosition := self currentBitPosition.
	aBit == 1 
		ifTrue: [currentByte := self currentByte bitOr: (1 bitShift: bitPosition - 1)].
	bitPosition := bitPosition - 1.
	bitPosition < 1 
		ifTrue: 
			[self byteStream nextPut: self currentByte.
			currentByte := 0.
			bitPosition := 8].
	currentBitPosition := bitPosition.
	^self
			
! !

!BitWriteStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__BitWriteStream.st,v 1.1 2006/09/07 11:09:48 sr Exp $'
! !
