"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#CCITTDecodeTree
	instanceVariableNames:'root'
	classVariableNames:'BlackDecodingTree ITUxT6DecodingTree WhiteDecodingTree'
	poolDictionaries:''
	category:'SPDF-Images'
!


!CCITTDecodeTree class methodsFor:'instance creation'!

forBlackDecoding
	"^a CCITTDecodeTree
I return an instance of myself suited to decoding black run-lengths."

	BlackDecodingTree isNil 
		ifTrue: 
			[BlackDecodingTree := self new.
			BlackDecodingTree initializeAsBlackDecodeTree].
	^BlackDecodingTree
			
!

forITUxT6Decoding
	"^a CCITTDecodeTree
I return an instance of myself suited to decoding ITU T.6 mode codes."

	ITUxT6DecodingTree isNil 
		ifTrue: 
			[ITUxT6DecodingTree := self new.
			ITUxT6DecodingTree initializeAsT6CodeTree].
	^ITUxT6DecodingTree
			
!

forWhiteDecoding
	"^a CCITTDecodeTree
I return an instance of myself suited to decoding white run-lengths."

	WhiteDecodingTree isNil 
		ifTrue: 
			[WhiteDecodingTree := self new.
			WhiteDecodingTree initializeAsWhiteDecodeTree].
	^WhiteDecodingTree
			
! !

!CCITTDecodeTree class methodsFor:'initialize-release'!

initialize
	"^self
I initialize my class variables.  This will cause the tables to be
re-built the next time they are requested."

	"CCITTDecodeTree initialize"

	BlackDecodingTree := nil.
	WhiteDecodingTree := nil.
	ITUxT6DecodingTree := nil.
	^self
			
! !

!CCITTDecodeTree methodsFor:'accessing'!

add: anEntry 
	"^ self
I add an entry to myself.  In fact, I delegate this to my root."

	self root add: anEntry.
	^self
			
!

code
	"^a String
I return my code.  As I am the root of the tree, this is an empty string."

	^''
			
!

root
	"^a CCITTDecodeTreeBranch
I return my root, which oddly enough is a branch.  I'm
made up of branches and codes (which are the leaves of
this particular tree)."

	root isNil ifTrue: [root := CCITTDecodeTreeBranch representingCode: ByteArray new].
	^root
			
! !

!CCITTDecodeTree methodsFor:'decoding'!

nextEntryFrom: aBitStream 
	"^a CCITTCodeEntry
I return the next code entry from aBitStream.  In fact, I
just delegate this to my root."

	^self root nextEntryFrom: aBitStream
			
! !

!CCITTDecodeTree methodsFor:'initialize-release'!

initializeAsBlackDecodeTree
	"^a CCITTCodeTree
I build and return a new run-decode tree for black code-runs."

	self addBlackT4TerminatingCodes.
	self addBlackT4MakeUpCodes.
	self addCommonMakeUpCodesForColor: 0.
	self addEOLCode.
	^self
			
!

initializeAsT6CodeTree
	"^a CCITTDeodeTree
I build and return a new decode tree ITU/CCITT T.6 mode codes."

	self addT6Codes.
	^self
			
!

initializeAsWhiteDecodeTree
	"^a CCITTDeodeTree
I build and return a new run-decode tree for white code-runs."

	self addWhiteT4TerminatingCodes.
	self addWhiteT4MakeUpCodes.
	self addCommonMakeUpCodesForColor: 1.
	self addEOLCode.
	^self
			
! !

!CCITTDecodeTree methodsFor:'private'!

addBlackT4MakeUpCodes
	"^a CCITTCodeTree
I add the black-specific code-run terminating codes."

	
	[:runLength :code | 
	self add: (ITUxT4MakeUpCodeEntry 
				withRunlength: runLength
				color: 0
				andCode: code)]
		value: 64 value: #(0 0 0 0 0 0 1 1 1 1) asByteArray;
		value: 128 value: #(0 0 0 0 1 1 0 0 1 0 0 0) asByteArray;
		value: 192 value: #(0 0 0 0 1 1 0 0 1 0 0 1) asByteArray;
		value: 256 value: #(0 0 0 0 0 1 0 1 1 0 1 1) asByteArray;
		value: 320 value: #(0 0 0 0 0 0 1 1 0 0 1 1) asByteArray;
		value: 384 value: #(0 0 0 0 0 0 1 1 0 1 0 0) asByteArray;
		value: 448 value: #(0 0 0 0 0 0 1 1 0 1 0 1) asByteArray;
		value: 512 value: #(0 0 0 0 0 0 1 1 0 1 1 0 0) asByteArray;
		value: 576 value: #(0 0 0 0 0 0 1 1 0 1 1 0 1) asByteArray;
		value: 640 value: #(0 0 0 0 0 0 1 0 0 1 0 1 0) asByteArray;
		value: 704 value: #(0 0 0 0 0 0 1 0 0 1 0 1 1) asByteArray;
		value: 768 value: #(0 0 0 0 0 0 1 0 0 1 1 0 0) asByteArray;
		value: 832 value: #(0 0 0 0 0 0 1 0 0 1 1 0 1) asByteArray;
		value: 896 value: #(0 0 0 0 0 0 1 1 1 0 0 1 0) asByteArray;
		value: 960 value: #(0 0 0 0 0 0 1 1 1 0 0 1 1) asByteArray;
		value: 1024 value: #(0 0 0 0 0 0 1 1 1 0 1 0 0) asByteArray;
		value: 1088 value: #(0 0 0 0 0 0 1 1 1 0 1 0 1) asByteArray;
		value: 1152 value: #(0 0 0 0 0 0 1 1 1 0 1 1 0) asByteArray;
		value: 1216 value: #(0 0 0 0 0 0 1 1 1 0 1 1 1) asByteArray;
		value: 1280 value: #(0 0 0 0 0 0 1 0 1 0 0 1 0) asByteArray;
		value: 1344 value: #(0 0 0 0 0 0 1 0 1 0 0 1 1) asByteArray;
		value: 1408 value: #(0 0 0 0 0 0 1 0 1 0 1 0 0) asByteArray;
		value: 1472 value: #(0 0 0 0 0 0 1 0 1 0 1 0 1) asByteArray;
		value: 1536 value: #(0 0 0 0 0 0 1 0 1 1 0 1 0) asByteArray;
		value: 1600 value: #(0 0 0 0 0 0 1 0 1 1 0 1 1) asByteArray;
		value: 1664 value: #(0 0 0 0 0 0 1 1 0 0 1 0 0) asByteArray;
		value: 1728 value: #(0 0 0 0 0 0 1 1 0 0 1 0 1) asByteArray.
	^self
			
!

addBlackT4TerminatingCodes
	"^a CCITTCodeTree
I add the black-specific code-run terminating codes."

	
	[:runLength :code | 
	self add: (ITUxT4TerminatingCodeEntry 
				withRunlength: runLength
				color: 0
				andCode: code)]
		value: 0 value: #(0 0 0 0 1 1 0 1 1 1) asByteArray;
		value: 1 value: #(0 1 0) asByteArray;
		value: 2 value: #(1 1) asByteArray;
		value: 3 value: #(1 0) asByteArray;
		value: 4 value: #(0 1 1) asByteArray;
		value: 5 value: #(0 0 1 1) asByteArray;
		value: 6 value: #(0 0 1 0) asByteArray;
		value: 7 value: #(0 0 0 1 1) asByteArray;
		value: 8 value: #(0 0 0 1 0 1) asByteArray;
		value: 9 value: #(0 0 0 1 0 0) asByteArray;
		value: 10 value: #(0 0 0 0 1 0 0) asByteArray;
		value: 11 value: #(0 0 0 0 1 0 1) asByteArray;
		value: 12 value: #(0 0 0 0 1 1 1) asByteArray;
		value: 13 value: #(0 0 0 0 0 1 0 0) asByteArray;
		value: 14 value: #(0 0 0 0 0 1 1 1) asByteArray;
		value: 15 value: #(0 0 0 0 1 1 0 0 0) asByteArray;
		value: 16 value: #(0 0 0 0 0 1 0 1 1 1) asByteArray;
		value: 17 value: #(0 0 0 0 0 1 1 0 0 0) asByteArray;
		value: 18 value: #(0 0 0 0 0 0 1 0 0 0) asByteArray;
		value: 19 value: #(0 0 0 0 1 1 0 0 1 1 1) asByteArray;
		value: 20 value: #(0 0 0 0 1 1 0 1 0 0 0) asByteArray;
		value: 21 value: #(0 0 0 0 1 1 0 1 1 0 0) asByteArray;
		value: 22 value: #(0 0 0 0 0 1 1 0 1 1 1) asByteArray;
		value: 23 value: #(0 0 0 0 0 1 0 1 0 0 0) asByteArray;
		value: 24 value: #(0 0 0 0 0 0 1 0 1 1 1) asByteArray;
		value: 25 value: #(0 0 0 0 0 0 1 1 0 0 0) asByteArray;
		value: 26 value: #(0 0 0 0 1 1 0 0 1 0 1 0) asByteArray;
		value: 27 value: #(0 0 0 0 1 1 0 0 1 0 1 1) asByteArray;
		value: 28 value: #(0 0 0 0 1 1 0 0 1 1 0 0) asByteArray;
		value: 29 value: #(0 0 0 0 1 1 0 0 1 1 0 1) asByteArray;
		value: 30 value: #(0 0 0 0 0 1 1 0 1 0 0 0) asByteArray;
		value: 31 value: #(0 0 0 0 0 1 1 0 1 0 0 1) asByteArray;
		value: 32 value: #(0 0 0 0 0 1 1 0 1 0 1 0) asByteArray;
		value: 33 value: #(0 0 0 0 0 1 1 0 1 0 1 1) asByteArray;
		value: 34 value: #(0 0 0 0 1 1 0 1 0 0 1 0) asByteArray;
		value: 35 value: #(0 0 0 0 1 1 0 1 0 0 1 1) asByteArray;
		value: 36 value: #(0 0 0 0 1 1 0 1 0 1 0 0) asByteArray;
		value: 37 value: #(0 0 0 0 1 1 0 1 0 1 0 1) asByteArray;
		value: 38 value: #(0 0 0 0 1 1 0 1 0 1 1 0) asByteArray;
		value: 39 value: #(0 0 0 0 1 1 0 1 0 1 1 1) asByteArray;
		value: 40 value: #(0 0 0 0 0 1 1 0 1 1 0 0) asByteArray;
		value: 41 value: #(0 0 0 0 0 1 1 0 1 1 0 1) asByteArray;
		value: 42 value: #(0 0 0 0 1 1 0 1 1 0 1 0) asByteArray;
		value: 43 value: #(0 0 0 0 1 1 0 1 1 0 1 1) asByteArray;
		value: 44 value: #(0 0 0 0 0 1 0 1 0 1 0 0) asByteArray;
		value: 45 value: #(0 0 0 0 0 1 0 1 0 1 0 1) asByteArray;
		value: 46 value: #(0 0 0 0 0 1 0 1 0 1 1 0) asByteArray;
		value: 47 value: #(0 0 0 0 0 1 0 1 0 1 1 1) asByteArray;
		value: 48 value: #(0 0 0 0 0 1 1 0 0 1 0 0) asByteArray;
		value: 49 value: #(0 0 0 0 0 1 1 0 0 1 0 1) asByteArray;
		value: 50 value: #(0 0 0 0 0 1 0 1 0 0 1 0) asByteArray;
		value: 51 value: #(0 0 0 0 0 1 0 1 0 0 1 1) asByteArray;
		value: 52 value: #(0 0 0 0 0 0 1 0 0 1 0 0) asByteArray;
		value: 53 value: #(0 0 0 0 0 0 1 1 0 1 1 1) asByteArray;
		value: 54 value: #(0 0 0 0 0 0 1 1 1 0 0 0) asByteArray;
		value: 55 value: #(0 0 0 0 0 0 1 0 0 1 1 1) asByteArray;
		value: 56 value: #(0 0 0 0 0 0 1 0 1 0 0 0) asByteArray;
		value: 57 value: #(0 0 0 0 0 1 0 1 1 0 0 0) asByteArray;
		value: 58 value: #(0 0 0 0 0 1 0 1 1 0 0 1) asByteArray;
		value: 59 value: #(0 0 0 0 0 0 1 0 1 0 1 1) asByteArray;
		value: 60 value: #(0 0 0 0 0 0 1 0 1 1 0 0) asByteArray;
		value: 61 value: #(0 0 0 0 0 1 0 1 1 0 1 0) asByteArray;
		value: 62 value: #(0 0 0 0 0 1 1 0 0 1 1 0) asByteArray;
		value: 63 value: #(0 0 0 0 0 1 1 0 0 1 1 1) asByteArray.
	^self
			
!

addCommonMakeUpCodesForColor: aColor 
	"^self
I add in the codes which are common between the black and white tables."

	
	[:runLength :code | 
	self add: (ITUxT4MakeUpCodeEntry 
				withRunlength: runLength
				color: aColor
				andCode: code)]
		value: 1792 value: #(0 0 0 0 0 0 0 1 0 0 0) asByteArray;
		value: 1856 value: #(0 0 0 0 0 0 0 1 1 0 0) asByteArray;
		value: 1920 value: #(0 0 0 0 0 0 0 1 1 0 1) asByteArray;
		value: 1984 value: #(0 0 0 0 0 0 0 1 0 0 1 0) asByteArray;
		value: 2048 value: #(0 0 0 0 0 0 0 1 0 0 1 1) asByteArray;
		value: 2112 value: #(0 0 0 0 0 0 0 1 0 1 0 0) asByteArray;
		value: 2176 value: #(0 0 0 0 0 0 0 1 0 1 0 1) asByteArray;
		value: 2240 value: #(0 0 0 0 0 0 0 1 0 1 1 0) asByteArray;
		value: 2304 value: #(0 0 0 0 0 0 0 1 0 1 1 1) asByteArray;
		value: 2368 value: #(0 0 0 0 0 0 0 1 1 1 0 0) asByteArray;
		value: 2432 value: #(0 0 0 0 0 0 0 1 1 1 0 1) asByteArray;
		value: 2496 value: #(0 0 0 0 0 0 0 1 1 1 1 0) asByteArray;
		value: 2560 value: #(0 0 0 0 0 0 0 1 1 1 1 1) asByteArray.
	^self
			
!

addEOLCode
	"^self
I add the 'EOL' code to myself.  In fact, I also add codings to take into account situations where the EOL code is byte aligned.  This means that the EOL code can have up to 7 additional leading zeros."

	self
		add: (ITUxT4EndOfLineCodeEntry withCode: #(0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry 
					withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry 
					withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray);
		add: (ITUxT4EndOfLineCodeEntry 
					withCode: #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray).
	^self
			
!

addT6Codes
	"^self
I build and return a new decode tree ITU/CCITT T.6 mode codes."

	self
		add: (ITUxT6PassCodeEntry withCode: #(0 0 0 1) asByteArray);
		add: (ITUxT6HorizontalCodeEntry withCode: #(0 0 1) asByteArray);
		add: (ITUxT6VerticalCodeEntry withCode: #(1) asByteArray andBias: 0);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 1 1) asByteArray andBias: 1);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 0 0 0 1 1) asByteArray andBias: 2);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 0 0 0 0 1 1) asByteArray andBias: 3);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 1 0) asByteArray andBias: -1);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 0 0 0 1 0) asByteArray andBias: -2);
		add: (ITUxT6VerticalCodeEntry withCode: #(0 0 0 0 0 1 0) asByteArray andBias: -3);
		add: (ITUxT6EnterUncompressedModeCodeEntry withCode: #(0 0 0 0 0 0 1 1 1 1) asByteArray);
		add: (ITUxT6EndOfFaximileBlock withCode: #(0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 1) asByteArray).
	^self
			
!

addWhiteT4MakeUpCodes
	"^a CCITTCodeTree
I add the white-specific make-up codes."

	
	[:runLength :code | 
	self add: (ITUxT4MakeUpCodeEntry 
				withRunlength: runLength
				color: 1
				andCode: code)]
		value: 64 value: #(1 1 0 1 1) asByteArray;
		value: 128 value: #(1 0 0 1 0) asByteArray;
		value: 192 value: #(0 1 0 1 1 1) asByteArray;
		value: 256 value: #(0 1 1 0 1 1 1) asByteArray;
		value: 320 value: #(0 0 1 1 0 1 1 0) asByteArray;
		value: 384 value: #(0 0 1 1 0 1 1 1) asByteArray;
		value: 448 value: #(0 1 1 0 0 1 0 0) asByteArray;
		value: 512 value: #(0 1 1 0 0 1 0 1) asByteArray;
		value: 576 value: #(0 1 1 0 1 0 0 0) asByteArray;
		value: 640 value: #(0 1 1 0 0 1 1 1) asByteArray;
		value: 704 value: #(0 1 1 0 0 1 1 0 0) asByteArray;
		value: 768 value: #(0 1 1 0 0 1 1 0 1) asByteArray;
		value: 832 value: #(0 1 1 0 1 0 0 1 0) asByteArray;
		value: 896 value: #(0 1 1 0 1 0 0 1 1) asByteArray;
		value: 960 value: #(0 1 1 0 1 0 1 0 0) asByteArray;
		value: 1024 value: #(0 1 1 0 1 0 1 0 1) asByteArray;
		value: 1088 value: #(0 1 1 0 1 0 1 1 0) asByteArray;
		value: 1152 value: #(0 1 1 0 1 0 1 1 1) asByteArray;
		value: 1216 value: #(0 1 1 0 1 1 0 0 0) asByteArray;
		value: 1280 value: #(0 1 1 0 1 1 0 0 1) asByteArray;
		value: 1344 value: #(0 1 1 0 1 1 0 1 0) asByteArray;
		value: 1408 value: #(0 1 1 0 1 1 0 1 1) asByteArray;
		value: 1472 value: #(0 1 0 0 1 1 0 0 0) asByteArray;
		value: 1536 value: #(0 1 0 0 1 1 0 0 1) asByteArray;
		value: 1600 value: #(0 1 0 0 1 1 0 1 0) asByteArray;
		value: 1664 value: #(0 1 1 0 0 0) asByteArray;
		value: 1728 value: #(0 1 0 0 1 1 0 1 1) asByteArray.
	^self
			
!

addWhiteT4TerminatingCodes
	"^a CCITTCodeTree
I add the white-specific terminating code-runs."

	
	[:runLength :code | 
	self add: (ITUxT4TerminatingCodeEntry 
				withRunlength: runLength
				color: 1
				andCode: code)]
		value: 0 value: #(0 0 1 1 0 1 0 1) asByteArray;
		value: 1 value: #(0 0 0 1 1 1) asByteArray;
		value: 2 value: #(0 1 1 1) asByteArray;
		value: 3 value: #(1 0 0 0) asByteArray;
		value: 4 value: #(1 0 1 1) asByteArray;
		value: 5 value: #(1 1 0 0) asByteArray;
		value: 6 value: #(1 1 1 0) asByteArray;
		value: 7 value: #(1 1 1 1) asByteArray;
		value: 8 value: #(1 0 0 1 1) asByteArray;
		value: 9 value: #(1 0 1 0 0) asByteArray;
		value: 10 value: #(0 0 1 1 1) asByteArray;
		value: 11 value: #(0 1 0 0 0) asByteArray;
		value: 12 value: #(0 0 1 0 0 0) asByteArray;
		value: 13 value: #(0 0 0 0 1 1) asByteArray;
		value: 14 value: #(1 1 0 1 0 0) asByteArray;
		value: 15 value: #(1 1 0 1 0 1) asByteArray;
		value: 16 value: #(1 0 1 0 1 0) asByteArray;
		value: 17 value: #(1 0 1 0 1 1) asByteArray;
		value: 18 value: #(0 1 0 0 1 1 1) asByteArray;
		value: 19 value: #(0 0 0 1 1 0 0) asByteArray;
		value: 20 value: #(0 0 0 1 0 0 0) asByteArray;
		value: 21 value: #(0 0 1 0 1 1 1) asByteArray;
		value: 22 value: #(0 0 0 0 0 1 1) asByteArray;
		value: 23 value: #(0 0 0 0 1 0 0) asByteArray;
		value: 24 value: #(0 1 0 1 0 0 0) asByteArray;
		value: 25 value: #(0 1 0 1 0 1 1) asByteArray;
		value: 26 value: #(0 0 1 0 0 1 1) asByteArray;
		value: 27 value: #(0 1 0 0 1 0 0) asByteArray;
		value: 28 value: #(0 0 1 1 0 0 0) asByteArray;
		value: 29 value: #(0 0 0 0 0 0 1 0) asByteArray;
		value: 30 value: #(0 0 0 0 0 0 1 1) asByteArray;
		value: 31 value: #(0 0 0 1 1 0 1 0) asByteArray;
		value: 32 value: #(0 0 0 1 1 0 1 1) asByteArray;
		value: 33 value: #(0 0 0 1 0 0 1 0) asByteArray;
		value: 34 value: #(0 0 0 1 0 0 1 1) asByteArray;
		value: 35 value: #(0 0 0 1 0 1 0 0) asByteArray;
		value: 36 value: #(0 0 0 1 0 1 0 1) asByteArray;
		value: 37 value: #(0 0 0 1 0 1 1 0) asByteArray;
		value: 38 value: #(0 0 0 1 0 1 1 1) asByteArray;
		value: 39 value: #(0 0 1 0 1 0 0 0) asByteArray;
		value: 40 value: #(0 0 1 0 1 0 0 1) asByteArray;
		value: 41 value: #(0 0 1 0 1 0 1 0) asByteArray;
		value: 42 value: #(0 0 1 0 1 0 1 1) asByteArray;
		value: 43 value: #(0 0 1 0 1 1 0 0) asByteArray;
		value: 44 value: #(0 0 1 0 1 1 0 1) asByteArray;
		value: 45 value: #(0 0 0 0 0 1 0 0) asByteArray;
		value: 46 value: #(0 0 0 0 0 1 0 1) asByteArray;
		value: 47 value: #(0 0 0 0 1 0 1 0) asByteArray;
		value: 48 value: #(0 0 0 0 1 0 1 1) asByteArray;
		value: 49 value: #(0 1 0 1 0 0 1 0) asByteArray;
		value: 50 value: #(0 1 0 1 0 0 1 1) asByteArray;
		value: 51 value: #(0 1 0 1 0 1 0 0) asByteArray;
		value: 52 value: #(0 1 0 1 0 1 0 1) asByteArray;
		value: 53 value: #(0 0 1 0 0 1 0 0) asByteArray;
		value: 54 value: #(0 0 1 0 0 1 0 1) asByteArray;
		value: 55 value: #(0 1 0 1 1 0 0 0) asByteArray;
		value: 56 value: #(0 1 0 1 1 0 0 1) asByteArray;
		value: 57 value: #(0 1 0 1 1 0 1 0) asByteArray;
		value: 58 value: #(0 1 0 1 1 0 1 1) asByteArray;
		value: 59 value: #(0 1 0 0 1 0 1 0) asByteArray;
		value: 60 value: #(0 1 0 0 1 0 1 1) asByteArray;
		value: 61 value: #(0 0 1 1 0 0 1 0) asByteArray;
		value: 62 value: #(0 0 1 1 0 0 1 1) asByteArray;
		value: 63 value: #(0 0 1 1 0 1 0 0) asByteArray.
	^self
			
! !

!CCITTDecodeTree class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__CCITTDecodeTree.st,v 1.1 2006/09/07 11:07:43 sr Exp $'
! !

CCITTDecodeTree initialize!
