"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

CCITTDecodeTreeNode subclass:#CCITTDecodeTreeBranch
	instanceVariableNames:'zeroBranch oneBranch'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!CCITTDecodeTreeBranch methodsFor:'accessing'!

oneBranch
	"^a CCITTDecodeTreeNode
I return my one branch, that is the part of the tree which represents my
code + 1.  For example, if my code is 0011 my zero node has the code
00111.  This is initialized to a CCITTDecodeTreeLeaf (see self initialize), though
the leaf may be replaced by a branch as the table is built.  Lazy initialization
is not used here because this method is very heavily used."

	^oneBranch
			
!

oneBranch: aBranch 
	"^self
I record my one branch.  c.f. self>>oneBranch."

	oneBranch := aBranch.
	^self
			
!

zeroBranch
	"^a CCITTDecodeTreeNode
I return my zero branch, that is the part of the tree which represents my
code + 0.  For example, if my code is 0011 my zero node has the code
00110.    This is initialized to a CCITTDecodeTreeLeaf (see self initialize), though
the leaf may be replaced by a branch as the table is built.  Lazy initialization
is not used here because this method is very heavily used."

	^zeroBranch
			
!

zeroBranch: aBranch 
	"^self
I record my zero branch.  c.f. self>>zeroBranch."

	zeroBranch := aBranch.
	^self
			
! !

!CCITTDecodeTreeBranch methodsFor:'adding-removing'!

add: anEntry 
	"^self
I add anEntry to myself.  This may invlove just assigning the entry to
either my zeroBranch or oneBranch, or it may involve delegating
to a sub branch.  It's OK for me to have an empty branch."

	(self zeroBranch isTheCorrectPathFor: anEntry) 
		ifTrue: [self addToZeroBranch: anEntry]
		ifFalse: [self addToOneBranch: anEntry].
	^self
			
!

addToOneBranch: anEntry 
	"^self
I add anEntry to my one branch.  This may result in a currently
assigned leaf being replaced by a branch."

	self oneBranch: (self oneBranch add: anEntry).
	^self
			
!

addToZeroBranch: anEntry 
	"^self
I add anEntry to my zero branch.  This may result in a currently
assigned leaf being replaced by a branch."

	self zeroBranch: (self zeroBranch add: anEntry).
	^self
			
! !

!CCITTDecodeTreeBranch methodsFor:'decoding'!

nextEntryFrom: aBitStream 
	"^a CCITTCodeEntry
I return the next code entry from aBitStream. I read the
next bit and delegate this to either my zero branch or one
branch depending upon what I get."

	^aBitStream next == 1 
		ifTrue: [self oneBranch nextEntryFrom: aBitStream]
		ifFalse: [self zeroBranch nextEntryFrom: aBitStream]
			
! !

!CCITTDecodeTreeBranch methodsFor:'initialize-release'!

initialize
	"^self
By default, I do nothing, but my subclasses may override this."

	super initialize.
	self oneBranch: (CCITTDecodeTreeLeaf representingCode: self code , #(1) asByteArray).
	self zeroBranch: (CCITTDecodeTreeLeaf representingCode: self code , #(0) asByteArray).
	^self
			
! !

!CCITTDecodeTreeBranch class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__CCITTDecodeTreeBranch.st,v 1.1 2006/09/07 11:20:59 sr Exp $'
! !
