"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

CodeReadStream subclass:#ITUxT6CodeReadStream
	instanceVariableNames:'currentLine rowWidth numberOfRows'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!

ITUxT6CodeReadStream comment:'This is something of a confused class.  Instances represent the T6 codes which describe a rectangle within an image.  The rectangle may be an entire image, or may be just a strip within an image.'
!


!ITUxT6CodeReadStream class methodsFor:'instance creation'!

onByteStream: aByteStream rowWidth: aRowWidth numberOfRows: aNumberOfRows 
	"^a CODEReadStream
I return a new instance of myself initialied to read codes of a particular scheme from aByteStream.  For T.6 encoding, I also need the width of the row (line) and the number of rows.  Because T6 encoded data is broken into strips, the number of rows in this context is the number of rows per strip, rather than the total number of lines in an image."

	^self new 
		onByteStream: aByteStream
		rowWidth: aRowWidth
		numberOfRows: aNumberOfRows
			
! !

!ITUxT6CodeReadStream methodsFor:'accessing'!

currentLine
	"^an ITUxT6Line
I return my current line.  I lazily initialize this as the reference line.  The first call of self>>next will create the first 'real' line."

	currentLine isNil ifTrue: [currentLine := ITUxT6Line asReferenceFor: self].
	^currentLine
			
!

currentLine: aValue

	currentLine := aValue
			
!

next
	"^an ITUxT6Line
Since the T.6 codes come in lines it turns out I'm really a Line Stream.  So essentially I treat the line like a big code and delegate the decoding to it."

	self currentLine: (ITUxT6Line following: self currentLine for: self).
	^self currentLine
			
!

numberOfRows
	"^an Integer
I return the number of rows in the image rectangle described by my codes."

	^numberOfRows
			
!

numberOfRows: aValue 
	"^self
c.f. self >> numberOfRows. "

	numberOfRows := aValue
			
!

rowWidth
	"^an Integer
I return the number of pixels in a row (line) of the image my code describe."

	^rowWidth
			
!

rowWidth: anInteger 
	"^self
I record the with of the row (line) in the image that my codes represent."

	rowWidth := anInteger
			
!

xT6DecodeTree
	"^an ITUDecodeTree
I return my decode tree which is a ITUDecodeTree populated with the pre-defined T.6 mode codes."

	^CCITTDecodeTree forITUxT6Decoding
			
! !

!ITUxT6CodeReadStream methodsFor:'decoding'!

decodeTo: aByteStream 
	"^self
I decode the contents of my underlying byte array to aByteStream.  I keep track of the line number only because it's handy for debugging.  If it's a performance problem, it can be removed."

	| outputStream |
	self reset.
	outputStream := BitWriteStream onByteStream: aByteStream.
	
	[1 to: self numberOfRows
		do: 
			[:lineNumber | 
			self next decodeTo: outputStream for: self.
			outputStream flushByte]] 
			on: ReachedEFOB
			do: [:ex | outputStream flushByte].
	^self
			
! !

!ITUxT6CodeReadStream methodsFor:'initialize-release'!

onByteStream: aByteStream rowWidth: aRowWidth numberOfRows: aNumberOfRows 
	"^a CODEReadStream
I return a new instance of myself initialied to read codes of a particular scheme from aByteStream.  For T.6 encoding, I also need the width of the row, and the number of rows in the rectangle described by my codes."

	self onByteStream: aByteStream.
	self rowWidth: aRowWidth.
	self numberOfRows: aNumberOfRows.
	^self
			
!

reset
	"^self
I reset myself so that the next thing read will be the first element in my underlying collection."

	super reset.
	self currentLine: nil.
	^self
			
! !

!ITUxT6CodeReadStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__ITUxT6CodeReadStream.st,v 1.1 2006/09/07 11:06:39 sr Exp $'
! !
