"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#LZWByteArray
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!LZWByteArray class methodsFor:'instance creation'!

onEncodedBytes: aByteArray 
	"^a LZWByteArray
I create and return a new instance of myself initialized to represent
the (presumably) LZW encoded contents of aByteArray."

	^self new onStream: aByteArray readStream
			
! !

!LZWByteArray class methodsFor:'constants'!

clearCode
	"^an Integer
I return the code which indicates that the code table must
be reset.   As per the TIFF 6.0 spec on page 59, this number
is 256. (of course) "

	^256
			
!

endOfInputCode
	"^an Integer
I return the code which indicates end of input.  As per the TIFF 6.0
spec on page 59, this number is 257. (of course) "

	^257
			
! !

!LZWByteArray methodsFor:'constants'!

clearCode
	"^an Integer
I return the code which indicates that the code table must
be reset. "

	^self class clearCode
			
!

endOfInputCode
	"^an Integer
I return the code which indicates end of input. "

	^self class endOfInputCode
			
! !

!LZWByteArray methodsFor:'converting'!

decompressedBytes
	"^a ByteArray
I return the decompressed form of my bytes.  This method is coded to resemble the algorythm as given in the TIFF 6.0 spec on page 61.  This may well mean you find it ugly.  Ah, well."

	| codeStream stringTable outputStream code oldCode newString |
	oldCode := nil.
	codeStream := LZWCodeStream on: self.
	stringTable := LZWStringTable new.
	outputStream := ByteArray new writeStream.
	[codeStream atEnd] whileFalse: 
			[code := codeStream next.
			code == self clearCode 
				ifTrue: 
					[stringTable initialize.
					code := codeStream next.
					code == self endOfInputCode ifTrue: [^self].
					outputStream nextPutAll: (stringTable stringForCode: code)]
				ifFalse: 
					[(stringTable containsCode: code) 
						ifTrue: 
							[outputStream nextPutAll: (stringTable stringForCode: code).
							stringTable addNewStringCombining: oldCode and: code]
						ifFalse: 
							[newString := stringTable addNewStringCombining: oldCode and: oldCode.
							outputStream nextPutAll: newString]].
			oldCode := code].
	^self
			
! !

!LZWByteArray class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__LZWByteArray.st,v 1.1 2006/09/07 11:09:36 sr Exp $'
! !
