"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#LZWCodeStream
	instanceVariableNames:'byteArray bitStream nextCode position codeSize'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!LZWCodeStream class methodsFor:'instance creation'!

on: aByteArray 
	"^a LZWCodeStream
I return a new instance of myself on aByteArray which is
presumed to contain LZW codes."

	^self new on: aByteArray
			
! !

!LZWCodeStream methodsFor:'accessing'!

byteArray
"^an LZWByteArray
I return my source LZWByteArray."

	^byteArray
			
!

endOfInputCode
	"^an Integer
I return the code which indicates end of input. "

	^self byteArray endOfInputCode
			
!

next
	"^an Integer
I read the next code from the byte stream.  This involves reading
the next n bits, where n is the current code size."

	| code |
	code := self nextCode.
	nextCode := nil.
	^code
			
! !

!LZWCodeStream methodsFor:'initialize-release'!

on: anLZWByteArray 
	"^self
I return myself on anLZWByteArray.  I'm initially positioned
at code zero."

	byteArray := anLZWByteArray.
	^self
			
! !

!LZWCodeStream methodsFor:'private'!

bitStream
	"^an LZWBitStream
I return my LXWBitStream on my underlying encoded byte array."

	bitStream isNil ifTrue: [bitStream := BitReadStream on: self byteArray].
	^bitStream
			
!

codeSize
	"^an Integer
I return the size in bits of the current code available from me.  I lazily
initialize my self to 9 - the starting code size for TIFF 6.0 LZW."

	codeSize isNil ifTrue: [codeSize := 9].
	^codeSize
			
!

incrementCodeSize
	"^an Integer
I return the size in bits of the current code available from me."

	codeSize := self codeSize + 1.
	^self codeSize
			
!

incrementPosition
	"^an Integer
I return the position of the current code.  The first code is #1."

	position := self position + 1.
	(position > ((2 ** self codeSize) - 1)) ifTrue: [self incrementCodeSize].
	^self position
			
!

nextCode
	"^an Integer
I return the next code from the compressed byteStream.  This
is a sort-of cached peek.  I read ahead and cache the next code.
Reading the next code involves getting the next n bits (where n is the
current code size) from by bit stream."

	nextCode isNil 
		ifTrue: 
			[self bitStream atEnd 
				ifTrue: [^nil]
				ifFalse: 
					[nextCode := self bitStream next: self codeSize.
					self incrementPosition]].
	^nextCode
			
!

position
	"^an Integer
I return my position"

	position isNil ifTrue: [position := 0].
	^position
			
! !

!LZWCodeStream methodsFor:'testing'!

atEnd
	"^a Boolean
I return true if there are no more codes available from the stream
otherwise, I return false."

	^self nextCode == self endOfInputCode
			
! !

!LZWCodeStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__LZWCodeStream.st,v 1.1 2006/09/07 11:07:31 sr Exp $'
! !
