"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#LZWStringTable
	instanceVariableNames:'strings'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!LZWStringTable methodsFor:'accessing'!

strings
	"^an OrderedCollection
I return my string table.  I lazily initialize it if necessary as specified in the
TIFF spec p59.   To quote:
InitializeStringTable() initializes the string table to contain all possible single-character
strings. There are 256 of them, numbered 0 through 255, since our characters
are bytes.
In addition, I add the clearCode and endOfInformation code."

	strings isNil 
		ifTrue: 
			[strings := OrderedCollection new: 1024.
			1 to: 255
				do: [:characterCode | strings add: (ByteArray with: characterCode)].
			strings add: self clearCode.
			strings add: self endOfInputCode].
	^strings
			
! !

!LZWStringTable methodsFor:'constants'!

clearCode
	"^an Integer
I return the code which indicates that the code table must
be reset. "

	^LZWByteArray clearCode
			
!

endOfInputCode
	"^an Integer
I return the code which indicates end of input. "

	^LZWByteArray endOfInputCode
			
! !

!LZWStringTable methodsFor:'initialize-release'!

initialize
	"^self
I initialize myself.  This means throwing away my current contents and returning
to my initial state.  The initial state of the table is lazily set in self>>strings."

	strings := nil.
	^self
			
! !

!LZWStringTable methodsFor:'services'!

addNewStringCombining: baseCode and: extentionCode 
	"^a String
I create a new entry for myself.  The new string is composed
of the string for baseCode plus the first character of extentionCode."

	| newString |
	newString := ByteArray new writeStream.
	newString nextPutAll: (self stringForCode: baseCode).
	newString nextPut: (self stringForCode: extentionCode) first.
	^self addString: newString contents
			
!

addString: aString 
	"^a String
I add aString to my collection of strings.  I return aString."

	self strings add: aString.
	^aString
			
!

containsCode: anInteger 
	"^a Boolean
I return true if I have an entry for anInteger (a code), otherwise,
I return false."

	^(self strings at: anInteger) notNil
			
!

stringForCode: anInteger 
	"^a ByteArray
Although the selector suggests that you'll get a string, in fact
this is a lie.  You'll get a ByteArray and like it.  The reason for this
is that the TIFF implementation of LZW maps character to byte, and
therefore string to byte array.  I may clean the names up, but then
this inconsistancy does reflect the inconsistency in the spec.  Sigh."

	^anInteger == 0 ifTrue: [0] ifFalse: [self strings at: anInteger]
			
! !

!LZWStringTable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__LZWStringTable.st,v 1.1 2006/09/07 11:22:57 sr Exp $'
! !
