"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

DecodeLine subclass:#ModifiedHuffmanLine
	instanceVariableNames:'position'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!ModifiedHuffmanLine methodsFor:'accessing'!

incrementPositionBy: anInteger 
	"^self
I'm being notified that anInteger pels have just been decoded and written.  I update my position accordingly."

	self position: self position + anInteger.
	^self
			
!

position
	"^an Integer
I return mu current position along the line.  The first pel in the line is at position 1"

	position isNil ifTrue: [self position: 0].
	^position
			
!

position: aValue

	position := aValue
			
! !

!ModifiedHuffmanLine methodsFor:'decoding'!

decodeTo: outputStream 
	"^self
I process the next encoded line terminated size.  This means decoding first a white run, followed by a black run, followed by a white until self>>lineWidth bytes have been written. Each run may consist of a number of codes. Once the line is finished, I flush the current byte from the outputStream: this ensures that every line starts on a fresh, clean byte boundary. According to the TIFF Spec, I should not encounter an EOL, but in fact it seems that I do - right at the very end of the image.  In this case, I just discard the remainder of the input stream and return."

	[self atEnd] whileFalse: 
			[(self whiteDecodeTree nextEntryFrom: self bitStream) 
				decodeTo: outputStream
				for: self.
			self atEnd 
				ifTrue: 
					[self position = self client lineLength 
						ifFalse: 
							[SPDFCompressionException 
								raiseSignal: 'Line length inconsistency detected.'].
					^self].
			(self blackDecodeTree nextEntryFrom: self bitStream) 
				decodeTo: outputStream
				for: self].
	self position = self client lineLength 
		ifFalse: 
			[SPDFCompressionException 
				raiseSignal: 'Line length inconsistency detected at end.'].
	^self
			
! !

!ModifiedHuffmanLine methodsFor:'testing'!

atEnd
	"^a Boolean
I return true if I'm at or past the end of the line, otherwise I return false."

	^self position >= self client lineLength
			
! !

!ModifiedHuffmanLine class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__ModifiedHuffmanLine.st,v 1.1 2006/09/07 11:18:12 sr Exp $'
! !
