"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#OffsetReadStream
	instanceVariableNames:'baseStream firstElementPosition lastElementPosition'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-StreamUtilities'
!


!OffsetReadStream class methodsFor:'instance creation'!

onCollection: aCollection offset: anOffset length: aLength 
	"^an OffsetStream
I return a new instance of myself initialised to represent a stream over a subset of aCollection."

	^self new 
		onCollection: aCollection
		firstElementPosition: anOffset
		lastElementPosition: anOffset + aLength - 1
			
!

onStream: aStream firstElementPosition: firstElementPosition lastElementPosition: lastElementPosition 
	"^an OffsetStream
I return a new instance of myself initialised to represent a stream over a subset of aStream."

	^self new 
		onStream: aStream
		firstElementPosition: firstElementPosition
		lastElementPosition: lastElementPosition
			
!

onStream: aStream offset: anOffset length: aLength 
	"^an OffsetStream
I return a new instance of myself initialised to represent a stream over a subset of aStream."

	^self new 
		onStream: aStream
		firstElementPosition: anOffset
		lastElementPosition: anOffset + aLength - 1
			
! !

!OffsetReadStream methodsFor:'accessing'!

baseStream

	^baseStream
			
!

baseStream: aValue

	baseStream := aValue
			
!

firstElementPosition

	^firstElementPosition
			
!

firstElementPosition: aValue

	firstElementPosition := aValue
			
!

lastElementPosition

	^lastElementPosition
			
!

lastElementPosition: aValue

	lastElementPosition := aValue
			
!

next
	"^an Object
I return my next element."

	^self baseStream next
			
!

next: anInteger 
	"Answer the next anInteger elements of the receiver."

	^self baseStream next: anInteger
			
! !

!OffsetReadStream methodsFor:'initialize-release'!

onCollection: aCollection firstElementPosition: first lastElementPosition: last 
	"^an OffsetStream
I initialize and return myself.  I'll represent a stream over a subset of aCollection.  I record the basic facts, validate them and prepare myself for work (I reset myself)."

	self baseStream: aCollection readStream.
	self firstElementPosition: first.
	self lastElementPosition: last.
	self validate.
	self reset.
	^self
			
!

onStream: aStream firstElementPosition: first lastElementPosition: last 
	"^an OffsetStream
I initialize and return myself.  I'll represent a stream over a subset of aSteam's collection.  I record the basic facts, validate them and prepare myself for work (I reset myself)."

	self baseStream: aStream.
	self firstElementPosition: first.
	self lastElementPosition: last.
	self validate.
	self reset.
	^self
			
!

validate
	"^self
I make sure that my internal values are sane.  In particular, the underlying collection must actually contain elements at the specified first and last positions.  I position my underlying stream to check this out - if there are any problems an out-of-bounds exception will be raised."

	self baseStream position: self firstElementPosition.
	self baseStream position: self lastElementPosition.
	^self
			
! !

!OffsetReadStream methodsFor:'services'!

contents
	"^a Collection 
	I return the contents of my baseStream copied from my first element offset to 
	my last element offset."

	| currentPosition myContent |
	currentPosition := self baseStream position.
	self baseStream position: self firstElementPosition - 1.
	myContent := self baseStream next: self lastElementPosition - (self firstElementPosition - 1).
	self baseStream position: currentPosition.
	^myContent
			
!

peek
	"^a String
I delegate this to my underlying stream."

	^self baseStream peek
			
!

position
	"^an Integer
I return my current position.  My first element is at position 1, and when I'm reset, I'm at position 0."

	^self baseStream position - self firstElementPosition
			
!

reset
	"^self
I reset myself.  This means setting my position to just before the the first element, such that the next element read will be the first element."

	self baseStream position: self firstElementPosition - 1.
	^self
			
!

skipThrough: anObject 
	"^a String
I delegate this to my underlying stream."

	^self baseStream skipThrough: anObject
			
!

skipThroughAll: aCollection 
	"^a String
I delegate this to my underlying stream."

	^self baseStream skipThroughAll: aCollection
			
!

throughAll: aCollection 
	"^a String
I delegate this to my underlying stream."

	^self baseStream throughAll: aCollection
			
!

upTo: anObject 
	"^a String
I delegate this to my underlying stream."

	^self baseStream upTo: anObject
			
!

upToEnd
	"^a Collection 
	I return the elements from my current position up to my end."

	^self baseStream next: self lastElementPosition - self position - self firstElementPosition
			
! !

!OffsetReadStream methodsFor:'testing'!

atEnd
	"^a Boolean
I return true if I've reached the end of the subset of my basicStream I'm interested in.  Otherwise, I return false."

	^self baseStream position >= self lastElementPosition
			
! !

!OffsetReadStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__OffsetReadStream.st,v 1.1 2006/09/07 11:14:21 sr Exp $'
! !
