"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFBodyObject subclass:#PDFCatalog
	instanceVariableNames:'fontPalette pagesRoot resources xObjects'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFCatalog comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent a PDF catalog.  The catalog is the root of the objects within the body of a PDF file.
See the PDF Reference Manual V1.2 Section 6.2 (p. 73)'
!


!PDFCatalog methodsFor:'accessing'!

fontPalette
	"^a PDFFontPalette 
	I return my font palette. I lazilily initialize this and put in the default font, which 
	is PDFFont.type1Courier()."

	fontPalette isNil ifTrue: [fontPalette := PDFFontPalette for: self].
	^fontPalette
			
!

pagesRoot
	"^a PDFPages 
	I return the PDFPages object which is the root of the document's Pages 
	tree."

	pagesRoot isNil ifTrue: [pagesRoot := PDFPages for: self].
	^pagesRoot
			
!

resources
	"^a PDFResources 
	Resources define things like the fonts for a given page.."

	resources isNil ifTrue: [resources := PDFResources for: self].
	^resources
			
!

xObjects
	"^an OrderedCollection
I return my collection of X Objects.  These are 'large' objects (e.g. images)
which are represented outside of the page descriptions."

	xObjects isNil ifTrue: [xObjects := Set new].
	^xObjects
			
! !

!PDFCatalog methodsFor:'emitting'!

writeFontsOn: aWriter 
	"^self 
	I initialte the writing of all of the fonts to the PDF file."

	self fontPalette fontDeclarations do: [:aFontDeclaration | aFontDeclaration writeOn: aWriter].
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. First I must make sure that the header 
	has been written out, then I can get the objects inside me to write 
	themselves out. Checking the header is probably repeating what 
	the PDFBody already did, but better safe than sorry."

	self id writeDeclarationOn: aWriter.
	aWriter newLine.
	aWriter writeLine: '<<'.
	aWriter writeLine: '/Type /Catalog'.
	self writePagesClauseOn: aWriter.
	aWriter writeLine: '>>'.
	aWriter writeLine: 'endobj'.
	self writePagesTreeOn: aWriter.
	self resources writeOn: aWriter.
	self writeFontsOn: aWriter.
	self writeXObjectsOn: aWriter.
	^self
			
!

writePagesClauseOn: aWriter 
	"^self 
	I write the /Pages clause to aWriter. This is the PDF reference to 
	the root of the document's pages tree."

	aWriter write: '/Pages '.
	self pagesRoot id writeReferenceOn: aWriter.
	aWriter newLine.
	^self
			
!

writePagesTreeOn: aWriter 
	"^self 
	I initialte the writing of the entire pages tree by getting the root 
	of the pages tree to write itself out."

	self pagesRoot writeOn: aWriter.
	^self
			
!

writeXObjectsOn: aWriter 
	"^self 
	I initialte the writing of all of the X Objects to the PDF file."

	self xObjects do: [:anXObject | anXObject writeOn: aWriter].
	^self
			
! !

!PDFCatalog methodsFor:'services'!

addXObject: anXObject 
	"^self
I add anXObject to my collection of X objects.  These
objects are represented outside of page descriptions.  I 
also name the XObjects as I add them to the collection."

	anXObject name: 'IM' , (self xObjects size + 1) printString.
	self xObjects add: anXObject.
	^self
			
!

catalog
	"^self 
	I return the PDF catalog for the PDF file of which I am a part. 
	It's me!!"

	^self
			
!

defaultFont
	"^a PDFFont 
	I return the default font for this PDF document."

	^self fontPalette defaultFont
			
!

defaultPage
	"^a PDFPage 
	I return the default page within the PDF structure I 
	represent."

	^self pagesRoot defaultPage
			
! !

!PDFCatalog methodsFor:'writing'!

newPage
	"^a PDFPage 
	I create a new page in the PDFDocument which follows the existing 
	pages. I return the new page."

	^self pagesRoot newPage
			
! !

!PDFCatalog class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFCatalog.st,v 1.1 2006/09/07 11:14:02 sr Exp $'
! !
