"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFFrame subclass:#PDFCharacterFrame
	instanceVariableNames:'width justification leftMarginWidth rightMarginWidth font
		fontPitch textLeading grayFill value'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFCharacterFrame class methodsFor:'instance creation'!

width: aWidth 
	"^a PDFCharacterFrame 
	I cerate and initialize a new instance of myself."

	^self new withWidth: aWidth
			
! !

!PDFCharacterFrame methodsFor:'accessing'!

font
	"^a Font 
	I return my font."

	^font
			
!

font: aFont 
	"^self 
	I record my font."

	font := aFont.
	^self
			
!

fontPitch
	"^an Integer 
	I return my font pitch."

	fontPitch isNil ifTrue: [fontPitch := 10].
	^fontPitch
			
!

fontPitch: anInteger 
	"^self 
	I record my font pitch."

	fontPitch := anInteger.
	^self
			
!

grayFill
	"^an Integer 
	I return my gray fill."

	grayFill isNil ifTrue: [self grayFill: 0].
	^grayFill
			
!

grayFill: anInteger 
	"^self 
	I record my gray fill."

	grayFill := anInteger.
	^self
			
!

justification
	"^a String 
	I return the my justification."

	justification isNil ifTrue: [justification := 'L'].
	^justification
			
!

justification: aString 
	"^self 
	I record the my justification."

	justification := aString.
	^self
			
!

leftMarginWidth
	"^an Integer 
	I return the my integer width of my left margin."

	leftMarginWidth isNil ifTrue: [leftMarginWidth := 0].
	^leftMarginWidth
			
!

leftMarginWidth: aNumberOfCharacters 
	"^self 
	Sets the left margin width."

	leftMarginWidth := aNumberOfCharacters.
	^self
			
!

rightMarginWidth
	"^an Integer 
	I return the my integer width of my right margin."

	rightMarginWidth isNil ifTrue: [rightMarginWidth := 0].
	^rightMarginWidth
			
!

rightMarginWidth: aNumberOfCharacters 
	"^self 
	Sets the right margin width."

	rightMarginWidth := aNumberOfCharacters.
	^self
			
!

textLeading
	"^an Integer 
	I record my text leading."

	textLeading isNil
		ifTrue: [^self fontPitch]
		ifFalse: [^textLeading]
			
!

textLeading: anInteger 
	"^self 
	I record my text leading."

	textLeading := anInteger.
	^self
			
!

value
	"^a String 
	I return my current value."

	value isNil ifTrue: [value := String new].
	^value
			
!

value: aString 
	"^self 
	I record my current value."

	value := aString.
	^self
			
!

width
	"^an Integer 
	I return the my integer width."

	^width
			
!

width: anInteger 
	"^self 
	I record the my integer width."

	width := anInteger.
	^self
			
! !

!PDFCharacterFrame methodsFor:'emitting'!

appendLeftHandPaddingTo: aStream 
	"^self 
	Writes the left hand padding spaces to aStream."

	self appendPaddingOf: self paddingWidthOnLeft to: aStream.
	^self
			
!

appendRightHandPaddingTo: aStream 
	"^self 
	Writes the right hand padding spaces to aStream."

	self appendPaddingOf: self paddingWidthOnRight to: aStream.
	^self
			
!

asString
	"^a String 
	I return the string representation of my underlying value. The string 
	is justified and padded/truncated as needed (and quietly, too. So be 
	careful, you might get truncation and not notice)"

	| frameStream |
	frameStream := WriteStream on: String new.
	self appendLeftHandPaddingTo: frameStream.
	frameStream nextPutAll: self truncatedValueAsString.
	self appendRightHandPaddingTo: frameStream.
	^frameStream contents
			
!

paddingWidthOnLeft
	"^an Integer 
	Returns the number of characters which must appear before the first character 
	of >>stringValue(). Of course this number may not be any larger than the width 
	of the frame. 
	This algorythm *seems* to work, but it also looks rather ugly. Any suggestions?"

	| justificationPadding |
	justificationPadding := 0.
	self justification = 'L' 
		ifFalse: 
			[justificationPadding := self width - self leftMarginWidth 
						- (self truncatedValueAsString size + self rightMarginWidth).
			self justification = 'C' 
				ifTrue: [justificationPadding := justificationPadding // 2].
			justificationPadding := justificationPadding max: 0].
	justificationPadding := self leftMarginWidth + justificationPadding.
	justificationPadding := justificationPadding min: self width.
	^justificationPadding
			
!

paddingWidthOnRight
	"^an Integer 
	Returns the number of characters which must appear after the last character 
	of >>stringValue(). 
	This algorythm *seems* to work, but it also looks rather ugly. Any suggestions?"

	| leftPaddingAndDataWidth justificationPadding |
	leftPaddingAndDataWidth := self paddingWidthOnLeft 
				+ self truncatedValueAsString size.
	justificationPadding := self width - self rightMarginWidth 
				- leftPaddingAndDataWidth.
	justificationPadding := justificationPadding max: 0.
	justificationPadding := justificationPadding + rightMarginWidth.
	leftPaddingAndDataWidth + justificationPadding > self width 
		ifTrue: [justificationPadding := self width - leftPaddingAndDataWidth].
	^justificationPadding
			
!

writePositioningWithin: aPlacement 
	"^self 
	Writes the text positioning to the placement. This sets the starting position 
	for the text within the page. Character frames simply go into the next available 
	position within aPlacement, which means that the top left of the 
	rectangle occupied by this frame will be the top right of the used rectangle of 
	aPlacement."

	| targetTextObject parentInsertPoint myInsertPoint |
	targetTextObject := aPlacement textObject.
	parentInsertPoint := aPlacement currentInsertPoint.
	myInsertPoint := PDFPoint x: parentInsertPoint x
				y: parentInsertPoint y - self textLeading.
	targetTextObject addOperator: ((PDFTextPositioning for: targetTextObject) 
				coordinate: myInsertPoint).
	^self
			
!

writeTextStateTo: aPlacement 
	"^self 
	Writes the text state to aPlacement. Only does this if a valid text state can 
	be created."

	| textObject textState |
	self font isNil 
		ifFalse: 
			[textObject := aPlacement textObject.
			textState := PDFTextState for: textObject.
			textState font: self font.
			textState fontPitch: self fontPitch.
			textState textLeading: self textLeading.
			textState grayFill: self grayFill.
			textObject addOperator: textState].
	^self
			
!

writeTextStateWithin: aPlacement 
	"^self
Writes the text state to the current page.  Only does this if a valid text state can
be created."

	self font isNil ifFalse: [self writeTextStateTo: aPlacement].
	^self
			
!

writeTextWithin: aPlacement 
	"^self 
	Writes the text state to the current page. Only does this if a valid text state can 
	be created."

	aPlacement filePage write: self asString.
	^self
			
! !

!PDFCharacterFrame methodsFor:'initialize-release'!

withWidth: aWidth 
	"^a PDFCharacterFrame 
	I initialize and return myself."

	self width: aWidth.
	^self
			
! !

!PDFCharacterFrame methodsFor:'private'!

appendPaddingOf: numberOfPaddingCharactersNeeded to: aStream 
	"^self 
	Writes the numberOfPaddingCharactersNeeded to aBuffer. A padding character is 
	a space."

	numberOfPaddingCharactersNeeded timesRepeat: [aStream space].
	^self
			
!

truncated: sourceString to: maximumAllowedWidth 
	"^a String
I return the truncated form of aSourceString.  Rules for this vary by subclass."

	^self subclassResponsibility
			
!

truncatedValueAsString
	"^self 
	Returns the string as it will appear when rendered. This may involve truncation 
	if there is not enough space between the margins for >>valueAsString. 
	Truncation will be *from the right* except when right justification is used. 
	This method is a monolithic mess - needs review."

	| sourceString maximumAllowedWidth targetString |
	sourceString := self valueAsString.
	maximumAllowedWidth := self width - self leftMarginWidth 
				- self rightMarginWidth.
	targetString := maximumAllowedWidth > 0 
				ifTrue: 
					[sourceString size > maximumAllowedWidth 
						ifTrue: [self truncated: sourceString to: maximumAllowedWidth]
						ifFalse: [sourceString]]
				ifFalse: [String new].
	^targetString
			
! !

!PDFCharacterFrame methodsFor:'services'!

justifyCenter
	"^self 
	Sets the justification to center, so the sting value will appear half 
	way between the margins but tending to the left if necessary."

	self justification: 'C'.
	^self
			
!

justifyLeft
	"^self 
	Sets the justification to left, so the sting value will start in the first 
	character after the left hand margin."

	self justification: 'L'.
	^self
			
!

justifyRight
	"^self 
	Sets the justification to left, so the sting value will end in the last 
	character before the right hand margin."

	self justification: 'R'.
	^self
			
!

marginWidth: aNumberOfCharacters 
	"^self 
	Sets the left and right margins to be the same number of characters."

	self leftMarginWidth: aNumberOfCharacters.
	self rightMarginWidth: aNumberOfCharacters.
	^self
			
!

rectangle
	"^a PDFRectangle 
	I return a rectangle which bounds my text."

	^PDFRectangle
		x: 0
		y: self fontPitch
		width: self width
		height: self fontPitch
			
!

valueAsString
	self subclassResponsibility
			
! !

!PDFCharacterFrame methodsFor:'writing'!

setValue: aString 
	"^self 
	I take aString as my current value."

	self value: aString.
	^self
			
!

writeWithin: aPlacement 
	"^self 
	I write the text of myself within the space defined by aPlacement.
This involves writing out three distinct pieces of PDF information, a positioning, a text state and the text itself. "

	| targetPlacement |
	targetPlacement := aPlacement ensureSpaceFor: self.
	self writePositioningWithin: targetPlacement.
	self writeTextStateWithin: targetPlacement.
	self writeTextWithin: targetPlacement.
	^self
			
! !

!PDFCharacterFrame class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFCharacterFrame.st,v 1.1 2006/09/07 11:20:50 sr Exp $'
! !
