"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFCharacterWriter
	instanceVariableNames:'outputStream'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFCharacterWriter comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

This class manages the writing of characters to an underlying Writer and simply adds the capability to know the current character offset (>>getPosition).'
!


!PDFCharacterWriter class methodsFor:'instance creation'!

on: aStream 
	"^a PDFCharacterWriter 
	I return a new instance of myself initialized to write to 
	aStream"

	^self new outputStream: aStream
			
! !

!PDFCharacterWriter methodsFor:'accessing'!

outputStream
	"^a Stream 
	I return the output stream that I am to write to."

	^outputStream
			
!

outputStream: aStream 
	"^self 
	I record the output stream that I am to write to. Once 
	set, my output stream may not be changed."

	outputStream isNil ifTrue: [outputStream := aStream].
	^self
			
!

position
	"^an Integer 
	I Return the current byte (character) offset. Starts at zero."

	^self outputStream position
			
! !

!PDFCharacterWriter methodsFor:'emitting'!

newLine
	"^self 
	I write a PDF style new line sequence to my underlying stream and 
	update my position accordingly."

	self outputStream nextPut: 10.
	^self
			
!

next: anInteger put: aByte 
	"^self 
	This makes me look a little more like a stream.  It's useful for things like
	Image processing where images will be written to me."

	self outputStream next: anInteger put: aByte.
	^self
			
!

nextPut: aByte 
	"^self 
	This makes me look a little more like a stream.  It's useful for things like
	Image processing where images will be written to me one byte at a time."

	self outputStream nextPut: aByte.
	^self
			
!

nextPutAll: aCollection 
	"^self 
	This makes me look a little more like a stream.  It's useful for things like
	Image processing."

	self outputStream nextPutAll: aCollection.
	^self
			
!

space
	"^self 
	I Write a single space character."

	self write: ' '.
	^self
			
!

write: someBytes 
	"^self 
	I Write bytes to my underlying writer and keep track of the current byte 
	(character) offset."

	self outputStream nextPutAll: someBytes asByteArray.
	^self
			
!

writeLine: someCharacters 
	"^self 
	I Write someCharacters to my underlying stream and then also terminate the line. Of 
	course, I keep my current position up to date."

	self write: someCharacters.
	self newLine.
	^self
			
! !

!PDFCharacterWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFCharacterWriter.st,v 1.1 2006/09/07 11:18:59 sr Exp $'
! !
