"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFCollatedStream
	instanceVariableNames:'clientStream writer byteStream encodedStream'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFCollatedStream comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFCollatedStream class methodsFor:'instance creation'!

for: aClient
	"^a PDFCollatedStream 
	I create and return a new instance of myself. I set aClientStream 
	as the parent of the new instance."

	^self new for: aClient
			
! !

!PDFCollatedStream methodsFor:'accessing'!

byteStream
	"^a Stream 
	I return my underlying byte stream."

	byteStream isNil 
		ifTrue: [self byteStream: (WriteStream on: ByteArray new)].
	^byteStream
			
!

byteStream: aStream 
	"^self 
	I record my byte stream."

	byteStream := aStream.
	^self
			
!

clientStream
	"^a PDFStream 
	I return my client stream."

	^clientStream
			
!

clientStream: aPDFStream 
	"^self 
	I record my client stream."

	clientStream := aPDFStream.
	^self
			
!

writer
	"^aPDFCharaterWriter 
	I return my underlying writer. I use this writer to keep track 
	of my contents prior to writing them out."

	writer isNil ifTrue: [writer := PDFCharacterWriter on: self encodedStream].
	^writer
			
! !

!PDFCollatedStream methodsFor:'emitting'!

writeFilterClauseOn: aWriter
"^self
I do nothing as the compression and encoding don't work yet."

"		if ((this.file().useASCIIEncodingIsOn()) |  (this.file().compressionIsOn()))
		{
			aWriter.write('/Filter [');
			if (this.file().compressionIsOn()) {aWriter.write('/FlateDecode ');}
			if (this.file().useASCIIEncodingIsOn()) {aWriter.write('/ASCII85Decode ');}
			aWriter.writeLine(']');
		}
	}
"
^self
			
!

writeLengthClauseOn: aWriter 
	"^self 
	I write out the '/Length' clause."

	aWriter write: '/Length '.
	aWriter writeLine: self byteStream size printString.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. This means working out and writing my 
	length clause, and then writing out each of my text objects. To do 
	this, I must first write all of my text objects to a local PDFWriter (which 
	will compress as appropriate), then I get the size of the combined 
	text objects, then I can write out the contents of the local 
	PDFWriter."

	self collateStreamText.
	aWriter writeLine: '<<'.
	self writeLengthClauseOn: aWriter.
	self writeFilterClauseOn: aWriter.
	aWriter writeLine: '>>'.
	aWriter writeLine: 'stream'.
	self writeTextObjectsOn: aWriter.
	aWriter newLine.
	aWriter writeLine: 'endstream'.
	^self
			
!

writeTextObjectsOn: aWriter 
	"^self 
	I write the the collated content of the stream to a target PDF file."

	aWriter write: self byteStream contents.
	^self
			
! !

!PDFCollatedStream methodsFor:'initialize-release'!

for: aClientStream 
	"^self 
	I initialize myself. I set aClientStream as my client."

	self clientStream: aClientStream.
	^self
			
! !

!PDFCollatedStream methodsFor:'private'!

collateStreamText
	"^self 
	This is what I live for. I take the contents of my client (_stream) 
	and bring it all together in a single byte array. I may also offer some 
	compression in the future. 
	I walk the graphic objects in my client stream, and get them to write 
	themselves to my PDFWriter."

	self clientStream graphicObjects do: [:aGraphicObject | aGraphicObject writeOn: self writer].
	self finishEncoding.
	^self
			
!

encodedStream
	"^a Stream 
	I return my encoded stream. If I don't do any encoding (such 
	as compression) then I just return my byte stream.
	** Note - the compression is not implemented in Smalltalk yet."

	encodedStream isNil ifTrue: [self file compressionIsOn
			ifTrue: [encodedStream := self byteStream]
			ifFalse: [encodedStream := self byteStream]].
	^encodedStream
			
!

file
	"^a PDFFile 
	I return the file I'm working for."

	^self clientStream file
			
!

finishEncoding
"^self
I finish off the encoding.
Note: The current version does not do compression!!  So there is nothing to do here."
"		self file compressionIsOn ifTrue:
		[
			((DeflaterOutputStream) this.getEncodedStream()).finish();
		].
		if (this.file().useASCIIEncodingIsOn())
		{
			PDFASCII85EncodedByteArray encoder;
			encoder = new PDFASCII85EncodedByteArray(this.getByteStream().toByteArray());
			this.setByteStream(encoder.encodedByteArrayOutputStream());
"
^self
			
! !

!PDFCollatedStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFCollatedStream.st,v 1.1 2006/09/07 11:17:00 sr Exp $'
! !
