"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFFileElement subclass:#PDFCrossReference
	instanceVariableNames:'entries'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFCrossReference comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent the cross reference section in a PDF file.
See the PDF Reference Manual V1.2 Section 5.4 (p. 63)'
!


!PDFCrossReference methodsFor:'accessing'!

entries
	"^an OrderedCollection 
	I return my collection of entries. I lazily initialize."

	entries isNil ifTrue: [entries := OrderedCollection new].
	^entries
			
! !

!PDFCrossReference methodsFor:'emitting'!

predecessor
	"^a PDFElement 
	I return my predecessor if I have one. This is used to make sure that 
	the elements of the PDF file are written in the correct order. My predecessor 
	Is the PDF body."

	^self file body
			
!

writeCrossReferenceSizeClauseOn: aWriter 
	"^self 
	I write the two items which define the size and scope of the 
	cross reference to aWriter. 
	NOTE: For now, the first object in the XRef is always assumed to be zero"

	aWriter write: '0 '.
	aWriter writeLine: (self entries size + 1) printString.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. First I must make sure that the body 
	has been written out, then I can write out my table."

	aWriter writeLine: 'xref'.
	self writeCrossReferenceSizeClauseOn: aWriter.
	aWriter writeLine: '0000000000 65535 f '.
	self entries do: [:objectId | objectId writeCrossReferenceEntryOn: aWriter].
	^self
			
! !

!PDFCrossReference methodsFor:'services'!

objectIdFor: aBodyObject 
	"^a PDFObjectId 
	I return a PDFObjectId which has a unique number. 
	For now, I just create an object ID with the next number 
	and give it a generation number of zero."

	| previousObjectNumber newObjectId |
	previousObjectNumber := self entries size.
	newObjectId := PDFObjectId
				for: aBodyObject
				withId: previousObjectNumber + 1
				generation: 0.
	self entries add: newObjectId.
	^newObjectId
			
! !

!PDFCrossReference class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFCrossReference.st,v 1.1 2006/09/07 11:06:54 sr Exp $'
! !
