"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFElement
	instanceVariableNames:'offset'
	classVariableNames:'ImmutablePDFElementSignal'
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFElement comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFElement class methodsFor:'exceptions'!

immutablePDFElementSignal
	"^a Signal"
 
	ImmutablePDFElementSignal isNil 
		ifTrue: 
			[ImmutablePDFElementSignal := (Object errorSignal 
						newSignalMayProceed: false)
						notifierString: 'PDF element mutation error';
						nameClass: self message: #immutablePDFElementSignal].
	^ImmutablePDFElementSignal
			
! !

!PDFElement methodsFor:'accessing'!

offset
	"^an Integer or nil 
	I return my offset. My offset is only determined at the point I am written 
	out."

	^offset
			
!

offset: anInteger 
	"^self 
	I record my offset from the begining of the PDF file."

	offset := anInteger.
	^self
			
! !

!PDFElement methodsFor:'emitting'!

ensurePredecessorAlreadyWrittenTo: aWriter 
	"^self 
	I make sure that my predecessors have been writen out before I am."

	| predecessor |
	predecessor := self predecessor.
	predecessor notNil ifTrue: [predecessor writeOn: aWriter].
	^self
			
!

predecessor
	"^a PDFElement 
	It is the responsibility of my subclasses to respond with their 
	immediate predecessor."

	^self subclassResponsibility
			
!

writeOn: aWriter 
	"^self 
	I write myself to aWriter. Once written, I am immutable."

	| myStartingOffset |
	self isImmutable
		ifFalse: 
			[self ensurePredecessorAlreadyWrittenTo: aWriter.
			myStartingOffset := aWriter position.
			self writePDFSyntaxOn: aWriter.
			self offset: myStartingOffset]
			
!

writeWhenReady
	"^self 
	I write myself out immediately if appropriate."

	self file parent hasWriteSoonerPolicy ifTrue: [self writeOn: self file characterWriter].
	^self
			
! !

!PDFElement methodsFor:'exceptions'!

immutablePDFElementSignal
	"^a Signal"

	^self class immutablePDFElementSignal
			
! !

!PDFElement methodsFor:'services'!

file
	"^a PDFFile 
	Returns the file of which this element is a part."

	^self subclassResponsibility
			
! !

!PDFElement methodsFor:'testing'!

ensureMutability: anActionDescription 
	"^self 
	I make sure that I am no immutable. If I am, I throw an exception."

	self isImmutable 
		ifTrue: 
			[self immutablePDFElementSignal 
				raiseWith: anActionDescription , ' Element found to be immutable.'].
	^self
			
!

isImmutable
	"^a Boolean 
	I return true if I am an immutable PDFElement. Immutable elements 
	have already been written out and so may no longer be modified. I can tell 
	if I have been written out by seeing if I have an offset yet (I only get a definite 
	offset once I've been written out)."

	^(self offset isNil) not
			
! !

!PDFElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFElement.st,v 1.1 2006/09/07 11:14:57 sr Exp $'
! !
