"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFFile
	instanceVariableNames:'header body crossReference trailer parent useCompression
		useASCIIEncoding'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFFile comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

OK, so I''m not really a PDF file, but I do represent one.  I know my header, body cross reference and trailer.  I''m just the root of the information that will be written onto a Stream (probably a file stream) in PDF format.'
!


!PDFFile class methodsFor:'instance creation'!

for: aParent 
	"^a PDFFile 
	I create and return a new instance of myself which has the parent aParent."

	^self new parent: aParent
			
! !

!PDFFile methodsFor:'accessing'!

body
	"^a PDFBody 
	I return the body of the PDF file I represent. If this is the 
	first time I've been asked, I lazily initialize myself and create 
	a new PDFBody."

	body isNil ifTrue: [body := PDFBody for: self].
	^body
			
!

crossReference
	"^a PDFCrossReference 
	I return my cross reference."

	crossReference isNil ifTrue: [crossReference := PDFCrossReference for: self].
	^crossReference
			
!

header
	"^a PDFHeader 
	I return my header."

	header isNil ifTrue: [header := PDFHeader for: self].
	^header
			
!

parent
	"^a PDFWriter?? 
	I return my parent."

	^parent
			
!

parent: aParent 
	"^self 
	I record my parent. This value many only be assigned once."

	parent isNil ifTrue: [parent := aParent].
	^self
			
!

trailer
	"^a PDFTrailer 
	I return my trailer."

	trailer isNil ifTrue: [trailer := PDFTrailer for: self].
	^trailer
			
!

useCompression
	"^a Boolean 
	I return aBoolean which indicates whether compression should be used to 
	reduce the size of the PDF file."

	useCompression isNil ifTrue: [useCompression := true].
	^useCompression
			
!

useCompression: aBoolean 
	"^self 
	I record aBoolean which indicates whether compression should be used to 
	reduce the size of the PDF file."

	useCompression := aBoolean.
	^self
			
! !

!PDFFile methodsFor:'control'!

compressionOff
	"^self 
	I set compression off."

	self useCompression: false.
	^self
			
! !

!PDFFile methodsFor:'services'!

characterWriter
	"^a PDFCharacterWriter 
	I return the character writer I'm destined to be written to."

	^self parent characterWriter
			
!

compressionIsOff
	"^a Boolean 
	Returns true if the file compression policy is false, othrwise returns true."

	^self compressionIsOn not
			
!

compressionIsOn
	"^a Boolean 
	Returns true if the file compression policy is true, othrwise returns false."

	^self useCompression
			
!

defaultPage
	"^a PDFPage 
	I return the default active page within my body. If there 
	is not a default page yet, one will be created. If I create one, 
	the new page will follow any existing pages."

	^self body defaultPage
			
! !

!PDFFile methodsFor:'writing'!

newPage
	"^a PDFPage 
	I add a new page to the PDF document and make this the 
	current page. I fact, I delegate this to my PDFBody."

	^self body newPage
			
! !

!PDFFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFFile.st,v 1.1 2006/09/07 11:12:54 sr Exp $'
! !
