"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFFont
	instanceVariableNames:'baseFontName descriptor widthArray'
	classVariableNames:'Type1Courier Type1Helvetica Type1HelveticaBold Type1Times
		Type1TimesBold'
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFFont comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent fonts which may appear in PDF files.  Whenever one of my instances appears in a PDFFile it has a wrapping PDFFontDeclaration instance.'
!


!PDFFont class methodsFor:'instance creation'!

named: aString 
	"^a PDFFont 
	I create and initialize a new instance of myself."

	^self new named: aString
			
! !

!PDFFont class methodsFor:'accessing'!

type1Courier
	"^a PDFFont 
	I return an instance which represents the Type 1 Courier 
	font."

	Type1Courier isNil ifTrue: [Type1Courier := PDFBase14Type1Font named: 'Courier'].
	^Type1Courier
			
!

type1Helvetica
	"^a PDFFont 
	I return an instance which represents the Type 1 Helvetica 
	font."

	Type1Helvetica isNil ifTrue: [Type1Helvetica := PDFBase14Type1Font named: 'Helvetica'].
	^Type1Helvetica
			
!

type1HelveticaBold
	"^a PDFFont 
	I return an instance which represents the Type 1 HelveticaBold 
	font."

	Type1HelveticaBold isNil ifTrue: [Type1HelveticaBold := PDFBase14Type1Font named: 'Helvetica-Bold'].
	^Type1HelveticaBold
			
!

type1Times
	"^a PDFFont 
	I return an instance which represents the Type 1 Times 
	font."

	Type1Times isNil 
		ifTrue: [ Type1Times := PDFBase14Type1Font named: 'Times-Roman' ].
	^Type1Times
			
!

type1TimesBold
	"^a PDFFont 
	I return an instance which represents the Type 1 Times 
	font."

	Type1TimesBold isNil 
		ifTrue: [ Type1TimesBold := PDFBase14Type1Font named: 'Times-Bold' ].
	^Type1TimesBold
			
! !

!PDFFont methodsFor:'accessing'!

baseFontName
	"^a String 
	I return the string which is my base font name."

	^baseFontName
			
!

baseFontName: aString 
	"^self
	I record the string which is my base font name."

	baseFontName := aString.
	^self
			
!

descriptor
	"^a PDFFontDescriptor 
	I return my descriptor."

	^descriptor
			
!

widthArray
	"^a PDFFontWidthArray 
	This is (or would be) the array of widths associated with a font. The 
	defaut is that this is an Instance of PDFFontWidthArray, but some fonts 
	(notably the base 14) don't have such an array."

	widthArray isNil ifTrue: [widthArray := PDFFontWidthArray new].
	^widthArray
			
! !

!PDFFont methodsFor:'emitting'!

writeBaseFontClauseOn: aWriter 
	"^self 
	I write my base font clause to aWriter."

	aWriter write: '/BaseFont /'.
	aWriter writeLine: self baseFontName.
	^self
			
!

writeOn: aWriter 
	"^self 
	I write myself to aWriter. I use this technique just so I work in a way which 
	is consistent with PDFElement and it's subclasses."

	self writePDFSyntaxOn: aWriter.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write the declaration of the font I represent to aWriter. This 
	involves writing out the font, it's descriptor and widths."

	self writeSubTypeClauseOn: aWriter.
	self writeBaseFontClauseOn: aWriter.
	self writeWidthClausesOn: aWriter.
	^self
			
!

writeSubTypeClauseOn: aWriter 
	"^self 
	I write my subtype to aWriter. This is specific to my subclasses, so 
	I insist that they implement this."

	^self subclassResponsibility
			
!

writeWidthClausesOn: aWriter 
	"^self 
	I write my width clauses to aWriter. I delegate this to my width array. This 
	method may be overridden by classes which do not need to define their 
	width details in the PDF file (I'm thinking, of course, of the 14 base fonts)"

	self widthArray writeOn: aWriter.
	^self
			
! !

!PDFFont methodsFor:'initialize-release'!

named: aString 
	"^a PDFFont 
	I initialize and return myself."

	self baseFontName: aString.
	^self
			
! !

!PDFFont class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFFont.st,v 1.1 2006/09/07 11:22:04 sr Exp $'
! !
