"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFBodyObject subclass:#PDFFontDeclaration
	instanceVariableNames:'descriptorDeclaration font name'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFFontDeclaration comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

Instances of this class represent the declaration of a font within a PDF file.  This is distinct from the font itself which may appear in many PDF files.'
!


!PDFFontDeclaration class methodsFor:'instance creation'!

for: aCatalog on: aFont named: aString 
	"^a PDFFontDeclaration 
	I initialize and return a new instance of myself."

	^self new
		for: aCatalog
		on: aFont
		named: aString
			
! !

!PDFFontDeclaration methodsFor:'accessing'!

font
	"^a PDFFont 
	I return my font."

	^font
			
!

name
	"^a String
	I return my name."

	^name
			
! !

!PDFFontDeclaration methodsFor:'emitting'!

writeDescriptorClauseOn: aWriter 
	"^self 
	I write the descriptor clause, which is really just a reference to the 
	descriptor object in the PDF file."

	self font descriptor notNil
		ifTrue: 
			[aWriter write: '/FontDescriptor '.
			aWriter writeLine: 'Ref to descriptor should go here - PDFFontDeclaration>>writeDescriptorClause'].
	^self
			
!

writeFontEncoding: aWriter
	aWriter write: '/Encoding /MacRomanEncoding'.
	aWriter newLine.
	^self
!

writePDFSyntaxOn: aWriter
	"^self 
	I write the declaration of the font I represent to aWriter. This 
	involves writing out the font itself, and (if appropriate) the 
	font descriptor."

	self id writeDeclarationOn: aWriter.
	aWriter newLine.
	aWriter writeLine: '<<'.
	aWriter writeLine: '/Type /Font'.
	self font writeOn: aWriter.
	self writeDescriptorClauseOn: aWriter.
	self writeFontEncoding: aWriter.
	aWriter writeLine: '>>'.
	aWriter writeLine: 'endobj'.
	self font descriptor notNil ifTrue: [self descriptorDeclaration writeOn: aWriter].
	^self
			
!

writeResourceReferenceOn: aWriter 
	"^self 
	I write my reference as it appears in the resources enty of the catalog object 
	of the PDF file."

	aWriter write: '/' , self name , ' '.
	self id writeReferenceOn: aWriter.
	^self
			
! !

!PDFFontDeclaration methodsFor:'initialize-release'!

for: aCatalog on: aFont named: aString 
	"^self 
	I initialize and return myself."

	self parent: aCatalog.
	font := aFont.
	name := aString.
	^self
			
! !

!PDFFontDeclaration class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFFontDeclaration.st,v 1.1 2006/09/07 11:10:00 sr Exp $'
! !
