"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFFontPalette
	instanceVariableNames:'defaultFontDeclaration fontDeclarations catalog'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFFontPalette comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent the palette of available fonts within a given PDF file.  One of the fonts in the palette may be nominated as the default font.'
!


!PDFFontPalette class methodsFor:'instance creation'!

for: aCatalog 
	"^aPDFFontPalette 
	I create and initialize a new instance of myself."

	^self new for: aCatalog
			
! !

!PDFFontPalette methodsFor:'accessing'!

catalog
	"^self 
	I return the PDFCatalog that I am the font palette for."

	^catalog
			
!

catalog: aPDFCatalog 
	"^self 
	I record the PDFCatalog that I am the font palette for."

	catalog := aPDFCatalog.
	^self
			
!

defaultFontDeclaration
	"^a PDFFontDeclaration 
	I return my default font declaration."

	defaultFontDeclaration isNil ifTrue: [self addFont: PDFFont type1Courier].
	^defaultFontDeclaration
			
!

defaultFontDeclaration: aFontDeclaration 
	"^self 
	I record my default font declaration."

	defaultFontDeclaration := aFontDeclaration.
	^self
			
!

fontDeclarations
	"^an OrderedCollection 
	I return my collection of font declarations."

	fontDeclarations isNil ifTrue: [fontDeclarations := OrderedCollection new].
	^fontDeclarations
			
! !

!PDFFontPalette methodsFor:'initialize-release'!

for: aCatalog 
	"^self 
	I initialize myself."

	self catalog: aCatalog.
	^self
			
! !

!PDFFontPalette methodsFor:'services'!

addFont: aFont 
	"^self 
	I add aFont to myself. This means creating a wrapping font declaration 
	and adding that to my underlying collection of font declarations."

	| fontDeclaration |
	fontDeclaration := PDFFontDeclaration
				for: self catalog
				on: aFont
				named: 'F' , (self fontDeclarations size + 1) printString.
	self fontDeclarations add: fontDeclaration.
	self defaultFontDeclaration: fontDeclaration.
	^fontDeclaration
			
!

defaultFont
	"^a PDFFont 
	I return my default font."

	^self defaultFontDeclaration font
			
!

fontDeclarationFor: aFont 
	"^a PDFFontDeclaration 
	I return the font declaration for aFont. First I look through 
	my existing declarations, then, if need be, I create a new one."

	^self fontDeclarations detect: [:suspect | suspect font == aFont]
		ifNone: [self addFont: aFont]
			
! !

!PDFFontPalette class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFFontPalette.st,v 1.1 2006/09/07 11:17:30 sr Exp $'
! !
