"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFGroupingFrame subclass:#PDFMediaFrame
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!

PDFMediaFrame comment:'PDFMediaFrame instances are the top of the frame tree and represent the media (the page) itself.  Each report contains a single media frame.'
!


!PDFMediaFrame class methodsFor:'instance creation'!

within: aReport 
	^self new within: aReport
			
! !

!PDFMediaFrame methodsFor:'initialize-release'!

within: aReport 
	"^self
	I initialze myself as the media frame for aReport.  I set the default media rectangle too."

	parent := aReport.
	self setOrientationToLandscape.
	^self
			
! !

!PDFMediaFrame methodsFor:'private'!

rectangle: aRectangle 
	"^self
I must allow people to set the media box, and must notify the PDFFile of the change too."

	rectangle := aRectangle.
	self report writer mediaBox: aRectangle.
	^self
			
! !

!PDFMediaFrame methodsFor:'services'!

report
	"^a PDFReport 
	I return the PDFReport I'm working on writing to.  Because I'm the media frame, I know that my parent is the report."

	^parent
			
!

setOrientationToLandscape
	"^self 
	Sets the media for the report to landscape letter."

	self rectangle: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 792 612)).
	^self
			
!

setOrientationToPortrait
	"^self 
	Sets the media for the report to portrait letter."

	self rectangle: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 612 792)).
	^self
			
!

setOrientationToPortraitA4
	"^self 
	Sets the media for the report to portrait A4."

	self rectangle: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 595 836)).
	^self
			
!

updateCurrentInsertPoint: aRectangle 
	"^self 
	Updates the point at which the next frame will be written. The point is updated given 
	a rectangle which represents the space taken up by a frame this method assumes 
	has just been written. The insertion point is moved along the x axis by the width of the 
	rectangle, and the maximum text depth is updated is this is the deepest rectangle so far."

	self currentPlacement updateCurrentInsertPoint: aRectangle.
	^self
			
! !

!PDFMediaFrame methodsFor:'testing'!

isMediaFrame
	^true
			
! !

!PDFMediaFrame class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFMediaFrame.st,v 1.1 2006/09/07 11:14:15 sr Exp $'
! !
