"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFGroupingFrame subclass:#PDFMetaFrame
	instanceVariableNames:'name hasBorder'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFMetaFrame methodsFor:'accessing'!

hasBorder
	"^a Boolean 
	I return true if I have a visible border, otherwise I return false. I 
	default to false."

	hasBorder isNil ifTrue: [hasBorder := false].
	^hasBorder
			
!

hasBorder: aBoolean 
	"^self 
	I record whether I have a visible border or not."

	hasBorder := aBoolean.
	^self
			
!

name
	"^a String
The name of this meta frame.  This is really a debugging aid.  The idea is that it  is handy to know a frame by some meaningful name when developing an even vaguely complex report."

	name isNil ifTrue: [name := 'Unnamed'].
	^name
			
!

name: aString 
	name := aString.
	^self
			
! !

!PDFMetaFrame methodsFor:'emiting'!

writeBorder
	"^self 
	Writes the border of this frame to the current page, if this frame 
	is supposed to have a border."

	| stream pathObject path |
	1 halt: 'We can only write out the border once the placement is closed'.
	self hasBorder
		ifTrue: 
			[stream := self currentPageDescription.
			pathObject := PDFPathObject for: stream.
			stream addGraphicObject: pathObject.
			path := PDFPathRectangle for: pathObject.
			path rectangle: self absoluteRectangle.
			pathObject addOperator: path].
	^self
			
! !

!PDFMetaFrame methodsFor:'printing'!

printOn: aStream 
	aStream nextPutAll: 'Meta-Frame: '.
	aStream nextPutAll: self name.
	^self
			
! !

!PDFMetaFrame methodsFor:'private'!

hasAbsoluteRectangle
	"^a Boolean 
	Returns true if the absolute rectangle has been computed for this 
	frame. Otherwise returns false."

	^self currentPlacement hasAbsoluteRectangle
			
! !

!PDFMetaFrame methodsFor:'services'!

currentInsertPoint
	"^a PDFPoint 
	Returns the point at which the next frame to be written will start. This 
	value is initialized to the top left hand corner of my rectangle. I return 
	an absolute coordianate with respect to the page."

	^self currentPlacement currentInsertPoint
			
!

currentPage
	"^a PDFPage 
	Returns the current page within the report. That is, the page 
	that will be the target of any write commands."

	^self report currentPage
			
!

currentPageDescription
	"^a PDFPageDescription 
	Returns the current pageDescription within the report. That is, the pageDescription 
	that will be the target of any write commands. (It's simply the pageDescription 
	of the current page)."

	^self currentPage pageDescription
			
!

maximumExtent
	"^a PDFPoint
	I return a PDFPoint which represents the theoretical maximum extent possible for
	me.  e.g. the extent I could expand to on a blank page."

	| containingExtent myTopLeft |
	self rectangle extent isComplete ifTrue: [^self rectangle extent].
	containingExtent := self parent maximumExtent.
	myTopLeft := self rectangle topLeft.
	myTopLeft isComplete 
		ifTrue: 
			[1 halt.
			^PDFPoint x: containingExtent x - myTopLeft x
				y: containingExtent y - myTopLeft y]
		ifFalse: [^containingExtent]
			
!

setBorderOff
	"^self 
	I set myself to have an invisible border."

	self hasBorder: false.
	^self
			
!

setBorderOn
	"^self 
	I set myself to have a visible border."

	self hasBorder: true.
	^self
			
!

updateCurrentInsertPoint: aRectangle 
	"^self 
	Updates the point at which the next frame will be written. The point is updated given 
	a rectangle which represents the space taken up by a frame this method assumes 
	has just been written. The insertion point is moved along the x axis by the width of the 
	rectangle, and the maximum text depth is updated is this is the deepest rectangle so far."

	self currentPlacement updateCurrentInsertPoint: aRectangle.
	^self
			
! !

!PDFMetaFrame methodsFor:'writing'!

close
	"^self 
	Closes off the current placement of this frame after first closing all my sub frames."

	self subFrames do: [:subFrame | subFrame close].
	self currentPlacement close.
	^self
			
!

newLine
	"^self 
	Moves to a new line in the frame. Sets my x coordinate back to 
	zero, and moves the y coordinate down the page by the current 
	y axis extreme value (furthest value down the page)."

	self currentPlacement moveToNewLine.
	^self
			
!

write
	"^self 
	I simply force the computation of my absolute rectangle, which will cause 
	any borders I have to be painted."

	self absoluteRectangle.
	^self
			
!

write: aFrame 
	"^self 
	'Writes' aFrame into this meta frame. This means getting the character 
	frame to write itself out at my next available position. I expect the character frame will 
	want to get my current position so it can work out where to place itself. Once the frame 
	is written, I update my current text position on the basis of the rectangle of aFrame.
* should really make sure that aFrame can fit into the biggest possible placement for this meta frame.  If it can't, there is no way it can be printed, so an exception should be thrown."

	aFrame writeWithin: self currentPlacement.
	self updateCurrentInsertPoint: aFrame rectangle.
	^self
			
!

writeLine: aFrame 
	"^self 
	'Writes' a CharacterFrame into this meta frame. This means adding aFrame to the 
	current LineFrame."

	self write: aFrame.
	self newLine.
	^self
			
!

writeOnly
	"^self 
	'Writes' this meta frame (forcing out the border) then closes the frame 
	at its current absolute location. See close()"

	self write.
	self close.
	^self
			
!

writeOnly: aFrame 
	"^self 
	'Writes' this meta frame (forcing out the border) then closes the frame 
	at its current absolute location. See close()"

	self write: aFrame.
	self close.
	^self
			
! !

!PDFMetaFrame class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFMetaFrame.st,v 1.1 2006/09/07 11:12:10 sr Exp $'
! !
