"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFCharacterFrame subclass:#PDFMoneyCharacterFrame
	instanceVariableNames:'stringValueCache decimalPlaces'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFMoneyCharacterFrame methodsFor:'accessing'!

decimalPlaces
	"^an Integer 
	I return the number of decimal places I am to show 
	I my string representations. The default is 2."

	decimalPlaces isNil ifTrue: [decimalPlaces := 2].
	^decimalPlaces
			
!

decimalPlaces: anInteger 
	"^self 
	I record the number of decimal places I am to show 
	I my string representations."

	decimalPlaces := anInteger.
	^self
			
!

stringValueCache
	"^a String
	I return my cached string value."

	^stringValueCache
			
!

stringValueCache: aString 
	"^self 
	I record my cached string value."

	stringValueCache := aString.
	^self
			
!

value: aString 
	"^self 
	I record my current value and reset my cache."

	super value: aString.
	self stringValueCache: nil.
	^self
			
! !

!PDFMoneyCharacterFrame methodsFor:'private'!

truncated: aSourceString to: aWidth 
	"^a String
This is money we're talking about here.  We can't just cop off numbers.  I just return a new string with aWidth number of $# characters.  This is what a spreadsheet would do in the same situation. "

	^(String new: aWidth) atAllPut: $#
			
! !

!PDFMoneyCharacterFrame methodsFor:'services'!

valueAsString
	"^a String 
	Returns my underlying value so it's presented like money, 
	something like $12,345.67 for 1234567. 
	First, the number is given any necessary leading zeros, so one cent 
	would need '00' on the front to make $0.01 after the '$' , ',' and '.''s are 
	added. 
	Then I put in the ',''s and '.''s 
	Last, I stick on the '$' 
	Again, this is a horrible algorythm. Suggestions, please. 
	Note: This value is a little tricky to compute, so I cache it. Cache discarded with new value."

	| source sourcePosition sourceLength commaFrequency prePunctuationLead requiredCommas target |
	self stringValueCache notNil ifTrue: [^self stringValueCache].
	source := self value printString.
	sourcePosition := 1.
	sourceLength := source size.
	commaFrequency := 3.
	(self decimalPlaces + 1 - sourceLength max: 0) 
		timesRepeat: [source := '0' , source].
	sourceLength := source size.
	prePunctuationLead := (sourceLength - self decimalPlaces) 
				\\ commaFrequency.
	requiredCommas := (sourceLength - self decimalPlaces) // commaFrequency.
	target := WriteStream on: String new.
	target nextPut: $$.
	prePunctuationLead > 0 
		ifTrue: 
			[target nextPutAll: (source copyFrom: sourcePosition
						to: sourcePosition + prePunctuationLead - 1).
			sourcePosition := sourcePosition + prePunctuationLead]
		ifFalse: 
			[target nextPutAll: (source copyFrom: sourcePosition
						to: sourcePosition + commaFrequency - 1).
			sourcePosition := sourcePosition + commaFrequency.
			requiredCommas := requiredCommas - 1].
	requiredCommas timesRepeat: 
			[target nextPut: $,.
			target nextPutAll: (source copyFrom: sourcePosition
						to: sourcePosition - 1 + commaFrequency).
			sourcePosition := sourcePosition + commaFrequency].
	target nextPut: $..
	target nextPutAll: (source copyFrom: sourcePosition
				to: sourcePosition + decimalPlaces - 1).
	self stringValueCache: target contents.
	^target contents
			
! !

!PDFMoneyCharacterFrame class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFMoneyCharacterFrame.st,v 1.1 2006/09/07 11:10:12 sr Exp $'
! !
