"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFObjectId
	instanceVariableNames:'bodyObject objectNumber generationNumber'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFObjectId comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFObjectId class methodsFor:'instance creation'!

for: aBodyObject withId: anObjectNumber generation: aGenerationNumber 
	"^a PDFObjectId 
	I create and initialize a new instance of myself."

	^self new
		for: aBodyObject
		withId: anObjectNumber
		generation: aGenerationNumber
			
! !

!PDFObjectId methodsFor:'accessing'!

bodyObject
	"^a PDFBodyObject 
	I return the body object for which I am the ID."

	^bodyObject
			
!

bodyObject: aBodyObject 
	"^self 
	I record the body object for which I am the ID."

	bodyObject := aBodyObject.
	^self
			
!

generationNumber
	"^an Integer 
	I return the generation number of the Object ID I represent. This 
	is like a version number, and starts at 0 for the first generation 
	of a given object."

	^generationNumber
			
!

generationNumber: anInteger 
	"^self 
	I record my generation number."

	generationNumber := anInteger.
	^self
			
!

objectNumber
	"^an Integer 
	I return the number of the body object I represent. There may be a number 
	of versions of my body object, and they all share the same object number, 
	but have unique generation numbers."

	^objectNumber
			
!

objectNumber: anInteger 
	"^self 
	I record my object number."

	objectNumber := anInteger.
	^self
			
! !

!PDFObjectId methodsFor:'emitting'!

writeCrossReferenceEntryOn: aWriter 
	"^self 
	I write a line to aWriter in the format suitable for the cross reference 
	section."

	| offsetValue rawReference |
	offsetValue := self bodyObject offset printString.
	rawReference := '0000000000' , offsetValue.
	aWriter write: (rawReference copyFrom: rawReference size - 9 to: rawReference size).
	aWriter writeLine: ' 00000 n '.
	^self
			
!

writeDeclarationOn: aWriter 
	"^self 
	I write my 'o g obj' style reference to aWriter."

	self writeObjectAndGenerationNumberOn: aWriter.
	aWriter write: ' obj'.
	^self
			
!

writeObjectAndGenerationNumberOn: aWriter 
	"^self 
	I return a representation of myself in the form 'o g', where o 
	is my object number, g is my generation number. This is 
	useful when forming object definitions and object references."

	aWriter write: self objectNumber printString.
	aWriter write: ' '.
	aWriter write: self generationNumber printString.
	^self
			
!

writeReferenceOn: aWriter 
	"^self 
	I write my 'o g R' style reference to aWriter."

	self writeObjectAndGenerationNumberOn: aWriter.
	aWriter write: ' R'.
	^self
			
! !

!PDFObjectId methodsFor:'initialize-release'!

for: aBodyObject withId: anObjectNumber generation: aGenerationNumber 
	"^self 
	I initialize myself."

	self bodyObject: aBodyObject; objectNumber: anObjectNumber; generationNumber: aGenerationNumber.
	^self
			
! !

!PDFObjectId class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFObjectId.st,v 1.1 2006/09/07 11:09:30 sr Exp $'
! !
