"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFPageTreeObject subclass:#PDFPage
	instanceVariableNames:'contents'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFPage comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

This is the class of PDF pages.  A PDFPage represents a page in the final document.'
!


!PDFPage methodsFor:'accessing'!

contents
	"^a PDFContents 
	I return my contents object."

	contents isNil ifTrue: [contents := PDFContents for: self].
	^contents
			
! !

!PDFPage methodsFor:'emitting'!

writeContentsClauseOn: aWriter 
	"^self 
	I write the /Contents clause to aWriter. This is just a reference to 
	a contents object in the PD file."

	aWriter write: '/Contents ['.
	self contents id writeReferenceOn: aWriter.
	aWriter writeLine: ' ]'.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. Once written (after this), I am immutable."

	self id writeDeclarationOn: aWriter.
	aWriter newLine.
	aWriter writeLine: '<<'.
	aWriter writeLine: '/Type /Page'.
	self writeParentClauseOn: aWriter.
	self writeResourcesClauseOn: aWriter.
	self writeContentsClauseOn: aWriter.
	aWriter writeLine: '>>'.
	aWriter writeLine: 'endobj'.
	self contents writeOn: aWriter.
	^self
			
!

writeParentClauseOn: aWriter 
	"^self 
	I write the /Parent clause to aWriter. This is the PDF reference to 
	the immediate parent of this page."

	aWriter write: '/Parent '.
	self parent id writeReferenceOn: aWriter.
	aWriter newLine.
	^self
			
!

writeResourcesClauseOn: aWriter 
	"^self 
	I write the /Resources clause to aWriter. This clause describes 
	the resources required by this page (fonts etc.)."

	aWriter write: '/Resources '.
	self root resources id writeReferenceOn: aWriter.
	aWriter newLine.
	^self
			
! !

!PDFPage methodsFor:'services'!

mediaBox
	"^a PDFRectangle
I return the rectangle which defines the dimensions of
the media I'll appear on."

	^self parent mediaBox
			
!

page
"^a PDFPage
I return the page upon which I'll appear.
Well, erm.  That's me."
	^self
			
!

pageDescription
	"^a PDFPageDescription 
	I return my pageDescription. This is the collections of all the objects and operators 
	which will paint my content."

	^self contents pageDescription
			
!

stream
	"^a PDFPageDescription 
	I return my stream. This is the collections of all the objects and operators 
	which will paint my content."
nil halt: 'use ..pageDescription instead'.
	^self pageDescription
			
! !

!PDFPage methodsFor:'writing'!

addTextObject: aTextObject 
	"^self 
	I add aText object to my contents."

	self contents addTextObject: aTextObject.
	^self
			
!

addTextOperator: aTextOperator 
	"^self 
	I add aText operator to my contents. The text operator may specify 
	the location of text, the look of the text or the text to be shown."

	self contents addTextOperator: aTextOperator.
	^self
			
!

newLine
	"^self 
	I just make a new line in the PDF document."

	self contents newLine.
	^self
			
!

write: aString 
	"^self 
	I write aString into myself at the next available location."

	self contents write: aString.
	^self
			
!

writeLine: aString 
	"^self 
	I write aString into myself at the next available location and move to 
	the next line afterwards."

	self contents writeLine: aString.
	^self
			
! !

!PDFPage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFPage.st,v 1.1 2006/09/07 11:08:33 sr Exp $'
! !
