"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFPageTreeObject subclass:#PDFPages
	instanceVariableNames:'currentPage kids mediaBox'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFPages comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

This is a collection of either other PDFPages objects or leaf PDFPage objects.
NOTE: at this time I''l only hold PDFPages (to make V1.0 easy)'
!


!PDFPages methodsFor:'accessing'!

currentPage
	"^a PDFPage 
	I return the current page if there is one."

	^currentPage
			
!

currentPage: aPage 
	"^self 
	I record the current default page within the PDF structure I 
	represent."

	currentPage := aPage.
	^self
			
!

kids
	"^an OrderedCollection 
	I return my collection of 'kids'. These are either child PDFPages which may 
	contain more kids, or real leaf pages."

	kids isNil ifTrue: [kids := OrderedCollection new].
	^kids
			
!

mediaBox
	"^a PDFRectangle 
	By default, my media box is set to landscape letter size. This is just 
	because the first reports we did needed that format."

	mediaBox isNil ifTrue: [mediaBox := PDFRectangle fromPDFStandardIntegerArray: #(0 0 792 612)].
	^mediaBox
			
!

mediaBox: aPDFRectangle 
	"^self 
	Records my media box."

	mediaBox := aPDFRectangle.
	^self
			
! !

!PDFPages methodsFor:'emitting'!

balancePagesTree
	"^self 
	It's not efficient for me to have more than 10 kids, so if I find that I do, 
	I create new Pages objects and make these my kids. 
	For now, the algorythm just makes a binary tree of pages objects, with 
	the leaves of the tree having up to 10 kids max. 
	
	Actually, this is rather more tricky. The pages that are written must already know 
	their parent, so I'll have to build up a collection of leaf PSFPages objects (which 
	can be the parents of the written pages) and then balance these at the end. 
	For now, I'll just return void and keep the flat wide structure"

	^self
			
!

writeCountClauseOn: aWriter 
	"^self 
	I write the /Count clause to aWriter."

	aWriter write: '/Count '.
	aWriter write: self kids size printString.
	aWriter newLine.
	^self
			
!

writeKidsClauseOn: aWriter 
	"^self 
	I write the /Kids clause to aWriter. This referrs to the pages (or 
	groups of pages in the form of child Pages objects) in the order 
	in which they will appear in the document."

	aWriter write: '/Kids ['.
	self kids
		do: 
			[:pageTreeObject | 
			aWriter write: ' '.
			pageTreeObject id writeReferenceOn: aWriter].
	aWriter writeLine: ' ]'.
	^self
			
!

writeKidsOn: aWriter 
	"^self 
	I get my kids to write themselves to aWriter."

	self kids do: [:pageTreeObject | pageTreeObject writeOn: aWriter].
	^self
			
!

writeMediaBoxClauseOn: aWriter 
	"^self 
	I write the /MediaBox clause to aWriter. This clause describes 
	the the physical dimensions of the target media for this page."

	| bottomLeft topRight |
	aWriter write: '/MediaBox ['.
	bottomLeft := self mediaBox bottomLeft.
	topRight := self mediaBox topRight.
	aWriter write: bottomLeft x printString , ' ' , bottomLeft y printString , ' '.
	aWriter write: topRight x printString , ' ' , topRight y printString.
	aWriter writeLine: ']'.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. Once written (after this), I am immutable."

	self balancePagesTree.
	self id writeDeclarationOn: aWriter.
	aWriter newLine.
	aWriter writeLine: '<<'.
	aWriter writeLine: '/Type /Pages'.
	self writeMediaBoxClauseOn: aWriter.
	self writeKidsClauseOn: aWriter.
	self writeCountClauseOn: aWriter.
	aWriter writeLine: '>>'.
	aWriter writeLine: 'endObj'.
	self writeKidsOn: aWriter.
	^self
			
! !

!PDFPages methodsFor:'private'!

addPage: aPage 
	"^self 
	I add aPage as to the end of my kids. Don't worry, I'll look after 
	producing a balanced tree of Pages and Page objects when I 
	write myself out. 
	Note, this balancing means I must be written after the last of the 
	Page objects. For this reason, no pages may be added once I'm 
	immutable."

	self ensureMutability: 'Trying to add a new page to the Pages object.'.
	self kids add: aPage.
	aPage parent: self.	"Is this overkill???"
	^self
			
!

makeNewPage
	"^a PDFPage 
	I simply create a new page, add it to my kids and make 
	it the current page."

	| newPage |
	newPage := PDFPage for: self.
	self currentPage: newPage.
	self addPage: newPage.
	^newPage
			
! !

!PDFPages methodsFor:'services'!

defaultPage
	"^a PDFPage 
	I return the default page within the PDF structure I 
	represent. I assume when you call this that you *want* a 
	page, so I'll make sure you get one through lazy initialization 
	if necessary, and if I do initialize it will only ever happen for the 
	first page in the document (of course)."

	self currentPage isNil ifTrue: [self makeNewPage].
	^self currentPage
			
! !

!PDFPages methodsFor:'writing'!

newPage
	"^a PDFPage 
	I create a new page in the PDFDocument which follows the existing 
	pages. I return the new page."

	self currentPage notNil
		ifTrue: 
			[self currentPage writeWhenReady.
			self makeNewPage].
	^self defaultPage
			
! !

!PDFPages class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFPages.st,v 1.1 2006/09/07 11:06:21 sr Exp $'
! !
