"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFPoint
	instanceVariableNames:'x y'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFPoint comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent a point in the plane of the page of a PDF document.  The interesting thing about a PDF point is that one or both of the values can be null (unknown).  This is useful for relative positioning of PDF Objects.'
!


!PDFPoint class methodsFor:'instance creation'!

x: xCoordinate y: yCoordinate 
	"^a PDFPoint 
	I create and initialize an instance of myself to represent the 
	point with xCoordinate and yCoordinate."

	^self new x: xCoordinate y: yCoordinate
			
! !

!PDFPoint methodsFor:'accessing'!

x
	"^an Integer 
	I return my x coordinate."

	^x
			
!

y
	"^an Integer 
	I return my y coordinate."

	^y
			
! !

!PDFPoint methodsFor:'initialize-release'!

x: xCoordinate y: yCoordinate 
	"^self 
	I initialize myself to represent the point with xCoordinate and yCoordinate.  I am immutable
	so I don't use mutator methods to set my values."

	x := xCoordinate.
	y := yCoordinate.
	^self
			
! !

!PDFPoint methodsFor:'printing'!

printOn: aStream 
	"^self 
	I write a more friendly representation of myself to aStream."

	aStream nextPutAll: 'PDFPoint: '.
	aStream nextPutAll: self x printString.
	aStream nextPutAll: ' @ '.
	aStream nextPutAll: self y printString.
	^self
			
! !

!PDFPoint methodsFor:'services'!

plusExtent: aPoint 
	"^a PDFPoint 
	I return a point which represents the bottom right hand corner 
	of a rectangle whose top right is at my location and whose 
	extent is aPoint. 
	Note, in PDF land ... 
	bottom right x = top left x + extent x. 
	bottom right y = top left y - extent y. 
	also note that null coordinates will result from any calculation 
	involving null coordinates."

	| targetX targetY |
	(self x isNil or: [aPoint x isNil])
		ifTrue: [targetX := nil]
		ifFalse: [targetX := self x + aPoint x].
	(self y isNil or: [aPoint y isNil])
		ifTrue: [targetY := nil]
		ifFalse: [targetY := self y - aPoint y].
	^PDFPoint x: targetX y: targetY
			
! !

!PDFPoint methodsFor:'testing'!

isComplete
	"^a Boolean
	I return true if I have explicit vlaues for my x and y, otherwise I return false."

	^(self x isNil or: [self y isNil]) not
			
! !

!PDFPoint class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFPoint.st,v 1.1 2006/09/07 11:07:25 sr Exp $'
! !
