"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFRectangle
	instanceVariableNames:'topLeft extent'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFRectangle comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFRectangle class methodsFor:'instance creation'!

fromPDFStandardIntegerArray: anIntegerArray 
	"^a PDFRectangle 
	The PDF Spec says that a rectangle is an array of four numbers which 
	define the lower left x and y and the upper right x and y. This constructor 
	converts that to the top left x y the width then height.
	I create and initialize a new instance of myself."

	^self new fromPDFStandardIntegerArray: anIntegerArray
			
!

topLeft: topLeft extent: extent 
	"^a PDFRectangle 
	I initialize and return a new instance of myself."

	^self new topLeft: topLeft extent: extent
			
!

x: topLeftX y: topLeftY width: extentX height: extentY 
	"^a PDFRectangle 
	I create and initialize a new instance of myself."

	^self topLeft: (PDFPoint x: topLeftX y: topLeftY)
		extent: (PDFPoint x: extentX y: extentY)
			
! !

!PDFRectangle methodsFor:'accessing'!

extent
	"^a PDFPoint 
	I return my extent."

	^extent
			
!

extent: aPoint 
	"^self 
	I record my extent. It's OK to give me nil 
	but I convert that to a point with nil for x and y."

	aPoint isNil
		ifTrue: [extent := PDFPoint x: nil y: nil]
		ifFalse: [extent := aPoint].
	^self
			
!

topLeft
	"^a PDFPoint 
	I return my top left hand point."

	^topLeft
			
!

topLeft: aPoint 
	"^self 
	I record my top left hand point. It's OK to give me nil 
	but I convert that to a point with nil for x and y."

	aPoint isNil
		ifTrue: [topLeft := PDFPoint x: nil y: nil]
		ifFalse: [topLeft := aPoint].
	^self
			
! !

!PDFRectangle methodsFor:'initialize-release'!

fromPDFStandardIntegerArray: anIntegerArray 
	"^a PDFRectangle 
	I initialize and return myself."

	| topLeftX topLeftY width height |
	topLeftX := anIntegerArray at: 1.
	topLeftY := anIntegerArray at: 4.
	width := (anIntegerArray at: 3)
				- (anIntegerArray at: 1).
	height := (anIntegerArray at: 4)
				- (anIntegerArray at: 2).
	self topLeft: (PDFPoint x: topLeftX y: topLeftY).
	self extent: (PDFPoint x: width y: height).
	^self
			
!

topLeft: topLeftPoint extent: extentPoint 
	"^a PDFRectangle 
	I create and initialize a new instance of myself."

	self topLeft: topLeftPoint.
	self extent: extentPoint.
	^self
			
! !

!PDFRectangle methodsFor:'printing'!

printOn: aStream 
	aStream nextPutAll: 'topLeft: '.
	self topLeft printOn: aStream.
	aStream nextPutAll: '  extent: '.
	self extent printOn: aStream.
	^self
			
! !

!PDFRectangle methodsFor:'services'!

bottom
	^self y - self height
			
!

bottomLeft
	"^a PDFPoint 
	I return the point which is the bottom left hand corner of 
	rectangle I represent."

	^PDFPoint x: self topLeft x y: self bottomRight y
			
!

bottomRight
	"^a PDFPoint 
	I return the point which is the bottom right hand corner of 
	rectangle I represent. 
	Note, in PDF land ... 
	bottom right x = top left x + extent x. 
	bottom right y = top left y - extent y."

	^self topLeft plusExtent: self extent
			
!

height
	"^an Integer 
	This is a convinience method for getting the y coordinate of 
	my extent as an int."

	^self extent y
			
!

left
	^self x
			
!

offsetBy: baseRectangle 
	"^a PDFRectangle 
	I return a new rectangle which represents myself offset by the base Rectangle, so 
	this calculation assumes that I am *inside* the base Rectangle. 
	I insist that the base Rectangle has no unknown points (I'll fail if it does!!). If any 
	of my coordinates are null, I just take the values of the base rectangle."

	| newX newY newWidth newHeight |
	self x isNil
		ifTrue: [newX := baseRectangle x]
		ifFalse: [newX := baseRectangle x + self x].
	self y isNil
		ifTrue: [newY := baseRectangle y]
		ifFalse: [newY := baseRectangle y - baseRectangle height + self y].
	self width isNil
		ifTrue: [newWidth := baseRectangle width]
		ifFalse: [newWidth := self width].
	self height isNil
		ifTrue: [newHeight := baseRectangle height]
		ifFalse: [newHeight := self height].
	^PDFRectangle
		x: newX
		y: newY
		width: newWidth
		height: newHeight
			
!

right
	^self x + self width
			
!

top
	^self y
			
!

topRight
	"^a PDFPoint 
	I return the point which is the top right hand corner of 
	rectangle I represent."

	^PDFPoint x: self bottomRight x y: self topLeft y
			
!

width
	"^an Integer 
	This is a convinience method for getting the x coordinate of 
	my extent an an int."

	^self extent x
			
!

x
	"^an Integer 
	I return the x coordinate of my top left hand corner."

	^self topLeft x
			
!

y
	"^an Integer 
	I return the y coordinate of my top left hand corner."

	^self topLeft y
			
! !

!PDFRectangle methodsFor:'testing'!

isComplete
	"^a Boolean
	I return true if all my coordianates are known (i.e. none are nil), otherwise I return false."

	^self topLeft isComplete and: [self extent isComplete]
			
! !

!PDFRectangle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFRectangle.st,v 1.1 2006/09/07 11:22:48 sr Exp $'
! !
