"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFReportElement subclass:#PDFReportPageHeader
	instanceVariableNames:'lines currentLine'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFReportPageHeader methodsFor:'accessing'!

currentLine
	"^self 
	Returns the current line in the header, that is, the lane that would 
	be written to given a >>write() message send. The current line is lazily 
	initialized as a PDFReportLine, and this new line is added to my lines."

	currentLine isNil ifTrue: [self currentLine: self buildNewLine].
	^currentLine
			
!

currentLine: aPDFReportLine 
	"^self 
	Records the current line in the header."

	currentLine := aPDFReportLine.
	^self
			
!

lines
	"^an OrderedCollection 
	Returns the lines which make up the header. This is a Vector with each 
	element being one line."

	lines isNil ifTrue: [lines := OrderedCollection new].
	^lines
			
! !

!PDFReportPageHeader methodsFor:'emitting'!

writeOn: aWriter 
	"^self 
	Writes the text of the header to aWriter."

	self lines
		do: 
			[:line | 
			line writeOn: aWriter.
			aWriter newLine].
	self deferredFrames do: [:frame | frame value: self report].
	^self
			
! !

!PDFReportPageHeader methodsFor:'private'!

buildNewLine
	"^self 
	Builds a new report line and adds it to the collection of lines 
	in this header."

	| newLine |
	newLine := PDFReportLine for: self.
	self lines add: newLine.
	^newLine
			
! !

!PDFReportPageHeader methodsFor:'services'!

report
	"^aPDFReport 
	I return my parent report. This is just a convinience method."

	^self parent report
			
! !

!PDFReportPageHeader methodsFor:'writing'!

newLine
	"^self 
	Moves to a new line in the header. Simply sets a new line as the current line."

	self currentLine: self buildNewLine.
	^self
			
!

write: aString 
	"^self 
	Adds aString to the current line of the header. Note, there is no 
	wrapping, so if too much is written to one line, it will just over-run."

	self currentLine write: aString asString.
	^self
			
!

writeLine: aString 
	"^self 
	Adds a string to the current header line, and then moves to the next line."

	self write: aString.
	self newLine.
	^self
			
!

writePageNumber: width 
	"^self 
	Adds aString to the vector of elements."

	self currentLine writePageNumber: width.
	^self
			
! !

!PDFReportPageHeader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFReportPageHeader.st,v 1.1 2006/09/07 11:14:51 sr Exp $'
! !
