"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFReportPages
	instanceVariableNames:'pages currentPageNumber parent currentReportPage'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFReportPages class methodsFor:'instance creation'!

for: aParent 
	"^a PDFReportPages 
	I create and initialize a new instance of myself."

	^self new for: aParent
			
! !

!PDFReportPages methodsFor:'accessing'!

currentPageNumber
	"^self 
	I return the current page number."

	^currentPageNumber
			
!

currentPageNumber: anInteger 
	"^self 
	I record the current page number."

	currentPageNumber := anInteger.
	^self
			
!

currentReportPage
	"^a PDFReportPage 
	I return the wrapper arount the current page in the PDFFile."

	currentReportPage isNil ifTrue: [self newPage].
	^currentReportPage
			
!

currentReportPage: aPDFReportPage 
	"^self 
	I record the wrapper arount the current page in the PDFFile."

	currentReportPage := aPDFReportPage.
	^self
			
!

pages
	"^an OrderedCollection 
	Returns a Vector of PDFReportPage s. Each PDFReportPage knows 
	PDFPage and any current position information needed by the frames."

	pages isNil ifTrue: [pages := OrderedCollection new].
	^pages
			
!

parent
	"^a PDFReport 
	I return my parent."

	^parent
			
!

parent: aParent 
	"^self 
	I record my parent."

	parent := aParent.
	^self
			
! !

!PDFReportPages methodsFor:'initialize-release'!

for: aParent 
	"^a PDFReportPages 
	I initialize and return myself."

	self parent: aParent.
	^self
			
! !

!PDFReportPages methodsFor:'services'!

current
	"^a PDFReportPage 
	I return the current page."

	^self currentReportPage page
			
!

currentPage
	"^a PDFPage 
	I return the current page in the PDFFile."

1 halt: 'Use >>current instead'.
	^self currentReportPage page
			
!

currentPagePlacementFor: aFrame 
	"^a PDFReportFrameNotes 
	Returns the notes object for the current frame for the current 
	page."

	^self currentReportPage framePlacementFor: aFrame
			
! !

!PDFReportPages methodsFor:'writing'!

newPage
	"^a PDFReportPage 
	Creates a new page at the end of the report and makes 
	it the new current page. The current page number is set. 
	I get my parent to write out the header stuff."

	| newPage newReportPage |
	newPage := self parent writer newPage.
	newReportPage := PDFReportPage on: newPage.
	self pages add: newReportPage.
	self currentPageNumber: self pages size.
	self currentReportPage: newReportPage.
	self parent writeHeader.
	^newReportPage
			
! !

!PDFReportPages class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFReportPages.st,v 1.1 2006/09/07 11:10:43 sr Exp $'
! !
