"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

SPDFSelfTest subclass:#PDFReportingSelfTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Testing'
!


!PDFReportingSelfTest class methodsFor:'deferred frames'!

testHeader
	"^a Block 
	I return a block which produces the headers."

	| hFont frame1 frame2 header footer waterMark |
	^
	[:aReport | 
	hFont := PDFFont type1HelveticaBold.
	frame1 := PDFStringCharacterFrame width: 15.
	frame1 marginWidth: 1.
	frame1 justifyLeft.
	frame1 font: hFont.
	frame1 fontPitch: 12.
	frame2 := PDFStringCharacterFrame width: 15.
	frame2 font: hFont.
	frame2 fontPitch: 110.
	frame2 grayFill: 90.
	header := PDFMetaFrame 
				within: aReport mediaFrame
				x: 400
				y: 666
				width: 100
				height: 50.
	header name: 'header'.
	footer := PDFMetaFrame 
				within: aReport mediaFrame
				x: 100
				y: 160
				width: 100
				height: 20.
	footer
		setBorderOn;
		name: 'footer'.
	waterMark := PDFMetaFrame 
				within: aReport mediaFrame
				x: 20
				y: 450
				width: 400
				height: 110.
	waterMark name: 'waterMark'.
	header writeLine: (frame1 setValue: 'The Header').
	footer writeLine: (frame1 setValue: 'The Footer').
	waterMark write: (frame2 setValue: 'WaterMark')]
			
! !

!PDFReportingSelfTest class methodsFor:'testing'!

test
	"^self 
	Run the particular tests to be done this time."

	"PDFReportingSelfTest test"
	PDFSelfTest test.

	self test01.
	self test02.
	self test03.
	self test04.
	self test05. "This slows down testing.  You may wish to skip it."
	self test06. 
	self test07. 
	self test08. 
	self test09.
	self test10.
	^self
			
!

test01
	"^self 
	This method is just for testing. 
	It writes the first small part of a PDF file to c:\tmp\qaz.pdf by simply 
	pumping out the text. It don't even use the PDF streaming classes."

	| myStream report |
	myStream := (self testFileNamed: 'spdfreporting01.pdf') writeStream.
	report := PDFReport on: myStream.
	report newLine.
	report write: 'Page No. '.
	report newLine.
	report newLine.
	report writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	report writeLine: '||========||========||========||========||========||'.
	report newLine.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report write: '  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  YYYxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report close.
	myStream close.
	^self
			
!

test02
	"^self
This method is just for testing.
This test uses a header for the first time, and produces a small one
page report."

	| myStream report header |
	myStream := (self testFileNamed: 'spdfreporting02.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header writeLine: 'Page No. '.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	header writeLine: '||========||========||========||========||========||'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report write: '  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  YYYxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	report close.
	myStream close.
	^self
			
!

test03
	"^self 
	This method is just for testing. 
	This test uses a header for the first time, and produces a small one 
	page report."

	| myStream report header |
	myStream := (self testFileNamed: 'spdfreporting03.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header write: 'Page No. [I don''t know yet] '.
	header newLine.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	header writeLine: '||========||========||========||========||========||'.
	header newLine.
	500
		timesRepeat: 
			[report write: '  xxxxxxxx  '.
			report writeLine: 'xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'].
	report close.
	myStream close.
	^self
			
!

test04
	"^self 
	This method is just for testing. 
	This tests the page numbers built into the header."

	| myStream report header |
	myStream := (self testFileNamed: 'spdfreporting04.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header write: 'Page No. '.
	header writePageNumber: 5.
	header newLine.
	header newLine.
	header writeLine: 'This is com.splwg.spdfreporting>>test4()'.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	header writeLine: '||========||========||========||========||========||'.
	header newLine.
	100
		timesRepeat: 
			[report write: '  xxxxxxxx  '.
			report writeLine: 'xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'].
	report close.
	myStream close.
	^self
			
!

test05
	"^self 
	This method is just for testing. 
	Thisis just the same as test 4, but produces a 1000 page report."

	| myStream report header |
	myStream := (self testFileNamed: 'spdfreporting05.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header write: 'Page No. '.
	header writePageNumber: 5.
	header newLine.
	header newLine.
	header writeLine: 'This is com.splwg.spdfreporting>>test5()'.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	header writeLine: '||========||========||========||========||========||'.
	header newLine.
	45020
		timesRepeat: 
			[report write: '  xxxxxxxx  '.
			report writeLine: 'xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'].
	report close.
	myStream close.
	^self
			
!

test06
	"^self 
	This method is just for testing. 
	This test uses a header for the first time, and produces a small 
	report using Character Frames."

	| myStream report header stringFrame |
	myStream := (self testFileNamed: 'spdfreporting06.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header write: 'Page No. '.
	header writePageNumber: 5.
	header newLine.
	header newLine.
	header writeLine: 'This is com.splwg.spdfreporting>>test6()'.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	header writeLine: '||========||========||========||========||========||'.
	header newLine.
	stringFrame := PDFStringCharacterFrame width: 11.
	200
		timesRepeat: 
			[stringFrame setValue: 'xxxxxxxx'.
			report write: stringFrame.
			report writeLine: 'xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'].
	report close.
	myStream close.
	^self
			
!

test07
	"^self 
	This method is just for testing. 
	The objective here is to torture frame justification and 
	truncation in various ways."

	| myStream report header padding frame1 frame2 frame3 frame4 frame5 frame6 frame7 frame8 frame9 |
	myStream := (self testFileNamed: 'spdfreporting07.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header write: 'Page No. '.
	header writePageNumber: 5.
	header newLine.
	header newLine.
	header writeLine: 'This is com.splwg.spdfreporting>>test7()'.
	header newLine.
	header writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5  Header 6  Header 7  Header 8  Header 9'.
	header writeLine: '||========||========||========||========||========||========||========||========||========||'.
	header newLine.
	padding := PDFStringCharacterFrame width: 1.
	frame1 := PDFStringCharacterFrame width: 10.
	frame2 := PDFStringCharacterFrame width: 10.
	frame2 marginWidth: 1.
	frame3 := PDFStringCharacterFrame width: 10.
	frame3 marginWidth: 1.
	frame3 justifyLeft.
	frame4 := PDFStringCharacterFrame width: 10.
	frame4 marginWidth: 1.
	frame4 justifyCenter.
	frame5 := PDFStringCharacterFrame width: 10.
	frame5 rightMarginWidth: 1.
	frame5 justifyRight.
	frame6 := PDFStringCharacterFrame width: 10.
	frame6 marginWidth: 3.
	frame7 := PDFStringCharacterFrame width: 10.
	frame7 marginWidth: 6.
	frame7 justifyLeft.	"Truncate left, so justify right"
	frame8 := PDFStringCharacterFrame width: 10.
	frame8 marginWidth: 1.
	frame8 justifyRight.	"Truncate right, so justify left"
	frame9 := PDFStringCharacterFrame width: 10.
	frame9 marginWidth: 1.
	frame9 justifyLeft.
	1000
		timesRepeat: 
			[report write: padding.
			report write: (frame1 setValue: 'Default').
			report write: (frame2 setValue: 'M1').
			report write: (frame3 setValue: 'M1L').
			report write: (frame4 setValue: 'M1C').
			report write: (frame5 setValue: 'M1 & Too large, should be truncated').
			report write: (frame6 setValue: 'Wide Margins').
			report write: (frame7 setValue: 'Margins too wide').
			report write: (frame8 setValue: '1234567890').
			report writeLine: (frame9 setValue: '1234567890')].
	report close.
	myStream close.
	^self
			
!

test08
	"^self 
	This method is just for testing. 
	Tests the money frame."

	"self test8"

	| myStream report header padding frame1 frame2 frame3 frame4 frame5 frame6 frame7 frame8 frame9 headerFrame1 headerFrame2 frameA frameB frameC |
	myStream := (self testFileNamed: 'spdfreporting08.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	headerFrame1 := PDFStringCharacterFrame width: 10.
	headerFrame1 justifyRight.
	headerFrame2 := PDFStringCharacterFrame width: 10.
	headerFrame2 justifyRight.
	header write: (headerFrame1 setValue: 'Page:').
	header writePageNumber: 10.
	header newLine.
	header write: (headerFrame1 setValue: '~~~~~').
	header write: (headerFrame2 setValue: '^^^^^^^^^^').
	header newLine.
	header writeLine: 'This is com.splwg.spdfreporting>>test8'.
	header newLine.
	header 
		writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5  Header 6  Header 7  Header 8  Header 9  Header 0  Header A    HeaderB'.
	header 
		writeLine: '||========||========||========||========||========||========||========||========||========||========||==========||========||'.
	header newLine.
	padding := PDFStringCharacterFrame width: 1.
	frame1 := PDFStringCharacterFrame width: 10.
	frame2 := PDFStringCharacterFrame width: 10.
	frame2 marginWidth: 1.
	frame3 := PDFStringCharacterFrame width: 10.
	frame3 marginWidth: 1.
	frame3 justifyLeft.
	frame4 := PDFStringCharacterFrame width: 10.
	frame4 marginWidth: 1.
	frame4 justifyCenter.
	frame5 := PDFStringCharacterFrame width: 10.
	frame5 rightMarginWidth: 1.
	frame5 justifyRight.
	frame6 := PDFStringCharacterFrame width: 10.
	frame6 marginWidth: 3.
	frame7 := PDFStringCharacterFrame width: 10.
	frame7 marginWidth: 6.
	frame7 justifyLeft.	"Truncate left, so justify right"
	frame8 := PDFStringCharacterFrame width: 10.
	frame8 marginWidth: 1.
	frame8 justifyRight.	"Truncate right, so justify left"
	frame9 := PDFStringCharacterFrame width: 10.
	frame9 marginWidth: 1.
	frame9 justifyLeft.
	frameA := PDFMoneyCharacterFrame width: 10.
	frameA marginWidth: 1.
	frameA justifyLeft.
	frameB := PDFMoneyCharacterFrame width: 12.
	frameB marginWidth: 1.
	frameB justifyRight.
	frameC := PDFMoneyCharacterFrame width: 10.
	frameC marginWidth: 1.
	frameC justifyRight.
	1000 timesRepeat: 
			[report write: padding.
			report write: (frame1 setValue: 'Default').
			report write: (frame2 setValue: 'M1').
			report write: (frame3 setValue: 'M1L').
			report write: (frame4 setValue: 'M1C').
			report write: (frame5 
						setValue: 'M1 & Too large. The left of this text should be cut.').
			report write: (frame6 setValue: 'Wide Margins').
			report write: (frame7 setValue: 'Margins too wide').
			report write: (frame8 setValue: '1234567890').
			report write: (frame9 setValue: '1234567890').
			report write: (frameA setValue: 1).
			report write: (frameB setValue: 123456).
			report writeLine: (frameC setValue: 123456789)].
	report close.
	myStream close.
	^self
			
!

test09
	"^self 
	This method is just for testing. 
	Tests multiple font reports which also means using meta-frames for 
	the first time. (in a very simple way)."

	| myStream report font1 font2 font3 bodyFrame padding frame1 frame2 frame3 frame4 frame5 frame6 frame7 frame8 frame9 frame10 frame11 frame12 |
	myStream := (self testFileNamed: 'spdfreporting09.pdf') writeStream.
	report := PDFReport on: myStream.
	report setOrientationToPortrait.
	font1 := PDFFont type1Helvetica.
	font2 := PDFFont type1Courier.
	font3 := PDFFont type1HelveticaBold.
	bodyFrame := PDFMetaFrame
				within: report mediaFrame
				x: 30
				y: 762
				width: 470
				height: 732.
	bodyFrame setBorderOn.
	padding := PDFStringCharacterFrame width: 1.
	frame1 := PDFStringCharacterFrame width: 10.
	frame2 := PDFStringCharacterFrame width: 10.
	frame2 marginWidth: 1.
	frame3 := PDFStringCharacterFrame width: 10.
	frame3 marginWidth: 1.
	frame3 justifyLeft.
	frame3 font: font3.
	frame3 fontPitch: 14.
	frame4 := PDFStringCharacterFrame width: 10.
	frame4 marginWidth: 1.
	frame4 justifyCenter.
	frame5 := PDFStringCharacterFrame width: 10.
	frame5 rightMarginWidth: 1.
	frame5 justifyRight.
	frame6 := PDFStringCharacterFrame width: 10.
	frame6 marginWidth: 3.
	frame6 font: font1.
	frame7 := PDFStringCharacterFrame width: 10.
	frame7 marginWidth: 6.
	frame7 justifyLeft.
	frame7 font: font2.
	frame8 := PDFStringCharacterFrame width: 10.
	frame8 marginWidth: 1.
	frame8 justifyRight.
	frame9 := PDFStringCharacterFrame width: 10.
	frame9 marginWidth: 1.
	frame9 justifyLeft.
	frame9 font: font3.
	frame9 fontPitch: 6.
	frame10 := PDFMoneyCharacterFrame width: 10.
	frame10 marginWidth: 1.
	frame10 justifyLeft.
	frame10 font: font2.
	frame11 := PDFMoneyCharacterFrame width: 12.
	frame11 marginWidth: 1.
	frame11 justifyRight.
	frame12 := PDFMoneyCharacterFrame width: 10.
	frame12 marginWidth: 1.
	frame12 justifyRight.
	20
		timesRepeat: 
			[bodyFrame write: padding.
			bodyFrame write: (frame1 setValue: 'Default').
			bodyFrame write: (frame2 setValue: 'M1').
			bodyFrame write: (frame3 setValue: 'M1L').
			bodyFrame write: (frame4 setValue: 'M1C').
			bodyFrame write: (frame5 setValue: 'M1 & Too large, should be truncated').
			bodyFrame write: (frame6 setValue: 'Wide Margins').
			bodyFrame write: (frame7 setValue: 'Margins too wide').
			bodyFrame write: (frame8 setValue: '1234567890').
			bodyFrame write: (frame9 setValue: '1234567890').
			bodyFrame write: (frame10 setValue: 1).
			bodyFrame write: (frame11 setValue: 123456).
			bodyFrame writeLine: (frame12 setValue: 12345)].
	report close.
	myStream close.
	^self
			
!

test10
	"^self 
	This method is just for testing. 
	The objective here is to test multiply nested meta-frames."
	"PDFReportingSelfTest test10"

	| myStream report header font1 font2 font3 frame1 frame2 fA mediumExtent fAA fAD fAAB fAAC lineFrame itemFrame1 itemFrame2 itemFrame3 itemFrame4 itemFrame5 |
	myStream := (self testFileNamed: 'spdfreporting10.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header addDeferredFrame: self testHeader.
	report setOrientationToPortrait.
	font1 := PDFFont type1Times.
	font2 := PDFFont type1Courier.
	font3 := PDFFont type1HelveticaBold.
	frame1 := PDFStringCharacterFrame width: 20.
	frame1 marginWidth: 1.
	frame1 justifyLeft.
	frame1 font: font3.
	frame1 fontPitch: 14.
	frame2 := PDFStringCharacterFrame width: 10.
	frame2 marginWidth: 2.
	frame2 justifyLeft.
	frame2 font: font1.
	frame2 fontPitch: 10.
	fA := PDFMetaFrame
				within: report mediaFrame
				x: 18
				y: 756
				width: 576
				height: 720.
	fA setBorderOn; name: 'fA'.
	mediumExtent := PDFPoint x: 288 y: 360.
	fAA := PDFMetaFrame
				within: fA
				topLeft: nil
				extent: mediumExtent.
	fAD := PDFMetaFrame
				within: fA
				topLeft: (PDFPoint x: 288 y: 360)
				extent: mediumExtent.
	fAD setBorderOn; name: 'fAD'.
	fAAB := PDFMetaFrame
				within: fAA
				x: 144
				y: nil
				width: 144
				height: 180.
	fAAB setBorderOn; name: 'fAAB'.
	fAAC := PDFMetaFrame
				within: fAA
				x: 0
				y: 180
				width: 144
				height: 180.
	fAAC setBorderOn; name: 'fAAC'.
	lineFrame := PDFMetaFrame
				within: fAD
				x: 0
				y: nil
				width: 288
				height: 12.
	lineFrame setBorderOn; name: 'lineFrame'.
	itemFrame1 := PDFMetaFrame
				within: lineFrame
				x: nil
				y: nil
				width: 60
				height: 12.
	itemFrame1 name: 'itemFrame1'.
	itemFrame2 := PDFMetaFrame
				within: lineFrame
				x: nil
				y: nil
				width: 60
				height: 12.
	itemFrame2 setBorderOn; name: 'itemFrame2'.
	itemFrame3 := PDFMetaFrame
				within: lineFrame
				x: 0
				y: 12
				width: 60
				height: 12.
	itemFrame3 name: 'itemFrame3'.
	itemFrame4 := PDFMetaFrame
				within: lineFrame
				x: 60
				y: 12
				width: 60
				height: 12.
	itemFrame4 name: 'itemFrame4'.
	itemFrame5 := PDFMetaFrame
				within: lineFrame
				x: 120
				y: 12
				width: 60
				height: 12.
	itemFrame5 setBorderOn; name: 'itemFrame5'.
	fAAB writeLine: (frame1 setValue: 'PDFReporting').
	fAAB newLine.	"Does not make a blank line because newLine drops by the *USED* amount & we just did a new line!!"
	fAAB writeLine: (frame1 setValue: 'SelfTest...').
	fAAB writeLine: (frame1 setValue: 'test10').
	frame2 font: font2.
	frame2 fontPitch: 10.
	10
		timesRepeat: 
			[itemFrame1 writeOnly: (frame2 setValue: 'IFrame1A').
			itemFrame1 writeOnly: (frame2 setValue: 'IFrame1B').
			itemFrame2 writeOnly: (frame2 setValue: 'IFrame2A').
			lineFrame close.
			fAD newLine].
	5
		timesRepeat: 
			[itemFrame1 writeOnly.
			itemFrame1 writeOnly.
			itemFrame2 writeOnly.
			lineFrame close.
			fAD newLine].
	lineFrame setBorderOff.
	frame2 font: font1.
	frame2 fontPitch: 8.
	15
		timesRepeat: 
			[itemFrame3 writeOnly: (frame2 setValue: 'IFrame3A').
			itemFrame4 writeOnly: (frame2 setValue: 'IFrame4A').
			itemFrame5 writeOnly: (frame2 setValue: 'IFrame5A').
			lineFrame close.
			fAD newLine].
	fAAC write.	"Write the out the outer frame - but why?"
	fA close.
	report newPage.
	fAAC write: (frame1 setValue: 'Just me :-)').
	report close.
	myStream close.
	^self
			
!

test11
	"^self 
	This method is just for testing. 
	The objective here is to test multiply nested meta-frames."
	"PDFReportingSelfTest test11"

	| myStream report header font1 font2 font3 frame1 frame2 fA mediumExtent fAA fAD fAAB fAAC lineFrame itemFrame1 itemFrame2 itemFrame3 itemFrame4 itemFrame5 |
	myStream := (self testFileNamed: 'spdfreporting11.pdf') writeStream.
	report := PDFReport on: myStream.
	header := report pageHeader.
	header addDeferredFrame: self testHeader.
	report setOrientationToPortrait.
	font1 := PDFFont type1Helvetica.
	font2 := PDFFont type1Courier.
	font3 := PDFFont type1HelveticaBold.
	frame1 := PDFStringCharacterFrame width: 20.
	frame1 marginWidth: 1.
	frame1 justifyLeft.
	frame1 font: font3.
	frame1 fontPitch: 14.
	frame2 := PDFStringCharacterFrame width: 10.
	frame2 marginWidth: 2.
	frame2 justifyLeft.
	frame2 font: font1.
	frame2 fontPitch: 10.
	fA := PDFMetaFrame
				within: report mediaFrame
				x: 18
				y: 756
				width: 576
				height: 720.
	fA setBorderOn.
	mediumExtent := PDFPoint x: 288 y: 360.
	fAA := PDFMetaFrame
				within: fA
				topLeft: nil
				extent: mediumExtent.
	fAD := PDFMetaFrame
				within: fA
				topLeft: (PDFPoint x: 288 y: 360)
				extent: mediumExtent.
	fAD setBorderOn.
	fAAB := PDFMetaFrame
				within: fAA
				x: 144
				y: nil
				width: 144
				height: 180.
	fAAB setBorderOn.
	fAAC := PDFMetaFrame
				within: fAA
				x: 0
				y: 180
				width: 144
				height: 180.
	fAAC setBorderOn.
	lineFrame := PDFMetaFrame
				within: fAD
				x: 0
				y: nil
				width: 288
				height: 12.
	lineFrame setBorderOn.
	itemFrame1 := PDFMetaFrame
				within: lineFrame
				x: nil
				y: nil
				width: 60
				height: 12.
	itemFrame2 := PDFMetaFrame
				within: lineFrame
				x: nil
				y: nil
				width: 60
				height: 12.
	itemFrame2 setBorderOn.
	itemFrame3 := PDFMetaFrame
				within: lineFrame
				x: 0
				y: 12
				width: 60
				height: 12.
	itemFrame4 := PDFMetaFrame
				within: lineFrame
				x: 60
				y: 12
				width: 60
				height: 12.
	itemFrame5 := PDFMetaFrame
				within: lineFrame
				x: 120
				y: 12
				width: 60
				height: 12.
	itemFrame5 setBorderOn.
	fAAB writeLine: (frame1 setValue: 'PDFReporting').
	fAAB newLine.	"Does not make a blank line because newLine drops by the *USED* amount & we just did a new line!!"
	fAAB writeLine: (frame1 setValue: 'SelfTest...').
	fAAB writeLine: (frame1 setValue: 'test10').
	frame2 font: font2.
	frame2 fontPitch: 10.
	10
		timesRepeat: 
			[itemFrame1 writeOnly: (frame2 setValue: 'IFrame1A').
			itemFrame1 writeOnly: (frame2 setValue: 'IFrame1B').
			itemFrame2 writeOnly: (frame2 setValue: 'IFrame2A').
			lineFrame close.
			fAD newLine].
	5
		timesRepeat: 
			[itemFrame1 writeOnly.
			itemFrame1 writeOnly.
			itemFrame2 writeOnly.
			lineFrame close.
			fAD newLine].
	lineFrame setBorderOff.
	frame2 font: font1.
	frame2 fontPitch: 8.
	15
		timesRepeat: 
			[itemFrame3 writeOnly: (frame2 setValue: 'IFrame3A').
			itemFrame4 writeOnly: (frame2 setValue: 'IFrame4A').
			itemFrame5 writeOnly: (frame2 setValue: 'IFrame5A').
			lineFrame close.
			fAD newLine].
	fAAC write.	"Write the out the outer frame - but why?"
	fA close.
	report newPage.
	fAAC write: (frame1 setValue: 'Just me :-)').
	report close.
	myStream close.
	^self
			
! !

!PDFReportingSelfTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFReportingSelfTest.st,v 1.1 2006/09/07 11:13:12 sr Exp $'
! !
