"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

SPDFSelfTest subclass:#PDFSelfTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Testing'
!

PDFSelfTest comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFSelfTest class methodsFor:'testing'!

test
	"^self 
	I run through all of the tests."

	"PDFSelfTest test"

	self test01.
	self test02.
	self test03.
	self test04.
	self test05.
	self test06.
	self test07.
	self test08.
	self test09.
	^self
			
!

test01
	"^self 
	This method is just for testing. 
	I write the first small part of a PDF file to c:\tmp\qaz.pdf by simply 
	pumping out the text. I don't even use the PDF streaming classes. 
	NOTE: This does *not* create a legal PDF file."
	"PDFSelfTest test01"

	| myStream |
	myStream := (self testFileNamed: 'spdf01.pdfish') writeStream.
	myStream nextPutAll: '%PDF-1.2'; cr.
	myStream nextPutAll: '1 0 obj'; cr.
	myStream nextPutAll: '<<'; cr.
	myStream nextPutAll: '/Type /Catalog'; cr.
	myStream nextPutAll: '/Pages 3 0 R'; cr.
	myStream nextPutAll: '/Outlines 2 0 R'; cr.
	myStream nextPutAll: '>>'; cr.
	myStream nextPutAll: 'endobj'; cr.
	myStream close.
	^self
			
!

test02
	"^self 
	This tests PDFWriter by making a 'Hello, World' PDF file."
	"PDFSelfTest test02"

	| myStream pdfWriter |
	myStream := (self testFileNamed: 'spdf02.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pdfWriter write: 'Hello, World (From PDFTester)'.
	pdfWriter close.
	myStream close.
	^self
			
!

test03
	"^self 
	This tests PDFWriter by writing a few lines to a PDF document."
	"PDFSelfTest test03"

	| myStream pdfWriter |
	myStream := (self testFileNamed: 'spdf03.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pdfWriter writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
	pdfWriter writeLine: '||========||========||========||========||========||'.
	pdfWriter newLine.
	pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	pdfWriter write: '  xxxxxxxx  xxxxxxxx'.
	pdfWriter writeLine: '  YYYxxxxx  xxxxxxxx  xxxxxxxx'.
	pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
	pdfWriter close.
	myStream close.
	^self
			
!

test04
	"^self 
	This tests PDFWriter by writing a few pages to a PDF document."
	"PDFSelfTest test04"

	| myStream pdfWriter pageNumber |
	myStream := (self testFileNamed: 'spdf04.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pageNumber := 0.
	[pageNumber < 5]
		whileTrue: 
			[pageNumber := pageNumber + 1.
			pdfWriter newPage.
			pdfWriter newLine.
			pdfWriter write: 'Page No. '.
			pdfWriter writeLine: pageNumber printString.
			pdfWriter newLine.
			pdfWriter newLine.
			pdfWriter writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
			pdfWriter writeLine: '||========||========||========||========||========||'.
			pdfWriter newLine.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter write: '  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  YYYxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx  xxxxxxxx'].
	pdfWriter close.
	myStream close.
	^self
			
!

test05
	"^self 
	This tests PDFWriter by writing some tricky characters to the PDF document."
	"PDFSelfTest test05"

	| myStream pdfWriter pageNumber |
	myStream := (self testFileNamed: 'spdf05.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pageNumber := 0.
	[pageNumber < 5]
		whileTrue: 
			[pageNumber := pageNumber + 1.
			pdfWriter newPage.
			pdfWriter newLine.
			pdfWriter write: 'Page No. '.
			pdfWriter writeLine: pageNumber printString.
			pdfWriter newLine.
			pdfWriter newLine.
			pdfWriter writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
			pdfWriter writeLine: '||========||========||========||========||========||'.
			pdfWriter newLine.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter write: '  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  YYYxxxxx( xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx \xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx\\xxxxxxxx) xxxxxxxx  xxxxxxxx'].
	pdfWriter close.
	myStream close.
	^self
			
!

test06
	"^self 
This test is aimed at proving the Pages structure by simply increasing the number of
test pages above 10, forcing a re-structure of the pages (when this gets implemented)"

	"PDFSelfTest test06"

	| myStream pdfWriter pageNumber |
	myStream := (self testFileNamed: 'spdf06.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pageNumber := 0.
	[pageNumber < 20] whileTrue: 
			[pageNumber := pageNumber + 1.
			pdfWriter newPage.
			pdfWriter newLine.
			pdfWriter write: 'Page No. '.
			pdfWriter writeLine: pageNumber printString.
			pdfWriter newLine.
			pdfWriter newLine.
			pdfWriter writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
			pdfWriter 
				writeLine: '||========||========||========||========||========||'.
			pdfWriter newLine.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter write: '  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  YYYxxxxx( xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx \xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			pdfWriter writeLine: '  xxxxxxxx  xxxxxxxx\\xxxxxxxx) xxxxxxxx  xxxxxxxx'].
	pdfWriter close.
	myStream close.
	^self
			
!

test07
	"^self 
	This test fiddles with font pitches."
	"PDFSelfTest test07"

	| myStream pdfWriter pageNumber fontPitch font1 font2 currentPage textState textPositioning currentText |
	myStream := (self testFileNamed: 'spdf07.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pageNumber := 0.
	fontPitch := 7.
	font1 := PDFFont type1Helvetica.
	font2 := PDFFont type1Courier.
	[pageNumber < 20]
		whileTrue: 
			[pageNumber := pageNumber + 1.
			fontPitch := fontPitch + 1.
			currentPage := pdfWriter newPage.
			currentText := PDFTextObject for: currentPage pageDescription.
			textState := PDFTextState for: currentText.
			textState fontPitch: fontPitch.
			textState font: font1.
			currentText addOperator: textState.
			textPositioning := PDFTextPositioning for: currentText.
			textPositioning coordinate: (PDFPoint x: 30 y: 500).
			currentText addOperator: textPositioning.
			currentText newLine.
			currentText write: 'Page No. '.
			currentText writeLine: pageNumber printString.
			currentText writeLine: 'Test 7'.
			currentText newLine.	"Here's a second text state which changes the font"
			textState := PDFTextState for: currentText.
			textState fontPitch: fontPitch.
			textState font: font2.
			currentText addOperator: textState.
			currentText writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
			currentText writeLine: '||========||========||========||========||========||'.
			currentText newLine.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			currentText write: '  xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  YYYxxxxx( xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx \xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx\\xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			currentPage addTextObject: currentText].
	pdfWriter close.
	myStream close.
	^self
			
!

test08
	"^self 
	This test fiddles with paths."
	"PDFSelfTest test08"

	| myStream pdfWriter stream pathObject path |
	myStream := (self testFileNamed: 'spdf08.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	stream := pdfWriter defaultPage pageDescription.
	pathObject := PDFPathObject for: stream.
	stream addGraphicObject: pathObject.
	path := PDFPathRectangle for: pathObject.
	path rectangle: (PDFRectangle
			x: 30
			y: 600
			width: 500
			height: 500).
	pathObject addOperator: path.
	pdfWriter close.
	myStream close.
	^self
			
!

test09
	"^self 
	This test fiddles with font pitches."
	"PDFSelfTest test09"

	| myStream pdfWriter pageNumber fontPitch font1 font2 currentPage currentText textState textPositioning |
	myStream := (self testFileNamed: 'spdf09.pdf') writeStream.
	pdfWriter := PDFWriter on: myStream.
	pdfWriter compressionOff.
	pageNumber := 0.
	fontPitch := 7.
	font1 := PDFFont type1Helvetica.
	font2 := PDFFont type1Courier.
	[pageNumber < 20]
		whileTrue: 
			[pageNumber := pageNumber + 1.
			fontPitch := fontPitch + 1.
			currentPage := pdfWriter newPage.
			currentText := PDFTextObject for: currentPage pageDescription.

			textState := PDFTextState for: currentText.
			textState fontPitch: 215.
			textState font: font1.
			textState grayFill: 90.
			textPositioning := PDFTextPositioning for: currentText.
			textPositioning coordinate: (PDFPoint x: 30 y: 250).
			currentText addOperator: textPositioning.
			currentText addOperator: textState.
			currentText write: 'DRAFT'.

			textState := PDFTextState for: currentText.
			textState fontPitch: fontPitch.
			textState font: font1.
			currentText addOperator: textState.
			textPositioning := PDFTextPositioning for: currentText.
			textPositioning coordinate: (PDFPoint x: 30 y: 500).
			currentText addOperator: textPositioning.
			currentText newLine.
			currentText write: 'Page No. '.
			currentText writeLine: pageNumber printString.
			currentText writeLine: 'Test 7'.
			currentText newLine.	"Here's a second text state which changes the font"
			textState := PDFTextState for: currentText.
			textState fontPitch: fontPitch.
			textState font: font2.
			currentText addOperator: textState.
			currentText writeLine: '  Header 1  Header 2  Header 3  Header 4  Header 5'.
			currentText writeLine: '||========||========||========||========||========||'.
			currentText newLine.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx (xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			currentText write: '  xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  YYYxxxxx( xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx \xxxxxxxx  xxxxxxxx  xxxxxxxx'.
			currentText writeLine: '  xxxxxxxx  xxxxxxxx\\xxxxxxxx) xxxxxxxx  xxxxxxxx'.
			currentPage addTextObject: currentText].
	pdfWriter close.
	myStream close.
	^self
			
! !

!PDFSelfTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFSelfTest.st,v 1.1 2006/09/07 11:09:12 sr Exp $'
! !
