"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFGraphicObject subclass:#PDFTextObject
	instanceVariableNames:'currentTextState currentTextPositioning'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFTextObject comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.


 * Ref Figure 8.1 PDF Reference Manual  1.2
 * A text object consists of one or more operators (as per the figure).
 *
 * The same text object may not appear twice in a PDF File, though
 * of course the same text may be represented many times by the use
 * of many PDFText objects.'
!


!PDFTextObject methodsFor:'accessing'!

currentTextPositioning
	"^a PDFTextPositioning 
	I return my current text positioning if I have one."

	^currentTextPositioning
			
!

currentTextPositioning: aTextPositioning 
	"^self 
	I record my current text positioning."

	currentTextPositioning := aTextPositioning.
	^self
			
!

currentTextState
	"^a PDFTextState 
	I return my current text state if I have one."

	^currentTextState
			
!

currentTextState: aTextState 
	"^self 
	I record my current text state."

	currentTextState := aTextState.
	^self
			
! !

!PDFTextObject methodsFor:'emitting'!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. I write the BT/ET pair, and between 
	that I write my operators and strings."

	aWriter writeLine: 'BT'.
	self writeOperatorsOn: aWriter.
	aWriter writeLine: 'ET'.
	^self
			
! !

!PDFTextObject methodsFor:'initialize-release'!

applyDefaultOperatorsIfNecessary
	"^self 
	I add default text positioning and text state operators if I currently don't 
	have any."

	self currentTextPositioning isNil ifTrue: [self addOperator: self defaultTextPositioning].
	self currentTextState isNil ifTrue: [self addOperator: self defaultTextState].
	^self
			
!

defaultTextPositioning
	"^a PDFTextPositioning 
	I return the default text positioning operator which simply sets the postion 
	to 30 points from the top and 30 points from the side of the media box."

	| newTextPositioning |
	newTextPositioning := PDFTextPositioning for: self.
	newTextPositioning coordinate: (PDFPoint x: 30 y: 550).
	^newTextPositioning
			
!

defaultTextState
	"^a PDFTextState 
	I return the default text state operator which defines how text will 
	look. This sets the font, pitch, text leading etc..."

	| newTextState |
	newTextState := PDFTextState for: self.
	newTextState font: self catalog defaultFont.
	newTextState fontPitch: 8.
	newTextState textLeading: 10.
	^newTextState
			
! !

!PDFTextObject methodsFor:'services'!

addOperator: anOperator 
	"^self 
	I add anOperator to my vector of text operators."

	anOperator applyTo: self.
	^self
			
! !

!PDFTextObject methodsFor:'writing'!

newLine
	"^self 
	I throw a new line in the PDF document."

	| newTextString |
	newTextString := PDFTextString on: '' for: self.
	newTextString showOnNewLine.
	self addOperator: newTextString.
	^self
			
!

write: aString 
	"^self 
	I write a string to my collection of strings."

	self addOperator: (PDFTextString on: aString for: self).
	^self
			
!

writeLine: aString 
	"^self 
	I write aString to my text and have it followed by a new line."

	self write: aString.
	self newLine.
	^self
			
! !

!PDFTextObject class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFTextObject.st,v 1.1 2006/09/07 11:13:18 sr Exp $'
! !
