"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFTextOperator subclass:#PDFTextState
	instanceVariableNames:'fontPitch textLeading fontDeclaration grayFill'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFTextState comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances represent the text-wide parameters for a particular PDFText object.  Things like font and starting point.'
!


!PDFTextState methodsFor:'accessing'!

fontDeclaration
	"^a PDFFontDeclaration 
	I return my font declaration."

	^fontDeclaration
			
!

fontDeclaration: aFontDeclaration 
	"^self 
	I record my font declaration."

	fontDeclaration := aFontDeclaration.
	^self
			
!

fontPitch
	"^an Integer 
	I return my font pitch."

	fontPitch isNil ifTrue: [fontPitch := 8].
	^fontPitch
			
!

fontPitch: anInteger 
	"^self 
	I record my font pitch."

	fontPitch := anInteger.
	^self
			
!

grayFill
	"^an Integer 
	I return my gray fill which is a number between 0 (black) and 100 (white)."

	grayFill isNil ifTrue: [grayFill := 0].
	^grayFill
			
!

grayFill: anInteger 
	"^self 
	I record my gray fill which is a number between 0 (black) and 100 (white)."

	grayFill := anInteger.
	^self
			
!

textLeading
	"^an Integer 
	I return my text leading.  By default this will be the same as the
	font pitch."

	textLeading isNil ifTrue: [textLeading := self fontPitch].
	^textLeading
			
!

textLeading: anInteger 
	"^self 
	This sets my text leading (how high each line is). Once set, it may not be changed!! 
	c.f. >>setFont()"

	textLeading isNil ifTrue: [textLeading := anInteger].
	^self
			
! !

!PDFTextState methodsFor:'emitting'!

writeFontStatementOn: aWriter 
	"^self 
	I write my font statement to a writer. This takes the form: 
	'/FontName pitchSize Tf', e.g. '/F1 8 Tf'"

	aWriter write: '/' , self fontDeclaration name.
	aWriter write: ' ' , self fontPitch printString.
	aWriter writeLine: ' Tf'.
	^self
			
!

writeGrayFillStatementOn: aWriter 
	"^self 
	If necessary, I write out a gray fill statement."

	aWriter write: (self grayFill / 100.0) printString.
	aWriter writeLine: ' g'.
	^self
			
!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. This means witing out all of my text state parameters 
	such as font & font size, starting point for text etc."

	self writeFontStatementOn: aWriter.
	self writeTextLeadingStatementOn: aWriter.
	self writeGrayFillStatementOn: aWriter.
	^self
			
!

writeTextLeadingStatementOn: aWriter 
	"^self 
	I write my text leading statement to aWriter"

	aWriter write: self textLeading printString.
	aWriter writeLine: ' TL'.
	^self
			
! !

!PDFTextState methodsFor:'private'!

applyTo: aTextObject 
	"^self 
	I add myself as an operator within aTextObject by adding myself to the 
	operators vector."

	| currentTextState |
	currentTextState := aTextObject currentTextState.
	(self definesSameStateAs: currentTextState)
		ifFalse: 
			[aTextObject operators add: self.
			aTextObject currentTextState: self].
	^self
			
!

definesSameStateAs: aTextState 
	"^a Boolean 
	I return true if I define the same font, pitch etc.. as aTextState."

	aTextState isNil ifFalse: [aTextState fontDeclaration == self fontDeclaration ifTrue: [aTextState fontPitch = self fontPitch ifTrue: [aTextState textLeading = self textLeading ifTrue: [^true]]]].
	^false
			
! !

!PDFTextState methodsFor:'services'!

font: aFont 
	"^self 
	This sets my font. Once set, it may not be changed!! (after all, 
	you wouldn't want to change the font of somethig you's already 
	printed - would you?)"

	self fontDeclaration: (self catalog fontPalette fontDeclarationFor: aFont).
	^self
			
! !

!PDFTextState class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFTextState.st,v 1.1 2006/09/07 11:07:07 sr Exp $'
! !
