"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFTextOperator subclass:#PDFTextString
	instanceVariableNames:'string startNewLine'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFTextString comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

Ref 8.7.5 PDF Reference Manual  1.2
A text string operator consists of a single string which may be "shown" either on the current line or on a new line.'
!


!PDFTextString class methodsFor:'instance creation'!

on: aString for: aParent 
	"^a PDFTextString 
	I create and initialize a new instance of myself. I set the parent of my new 
	instance to aParent."

	^self new on: aString for: aParent
			
! !

!PDFTextString methodsFor:'accessing'!

startNewLine
	"^a Boolean 
	I return true if the text I represent should appear at the start of a new 
	line, othewise, I return false. I lazily initialize myself to false."

	startNewLine isNil ifTrue: [startNewLine := false].
	^startNewLine
			
!

startNewLine: aBoolean 
	"^self 
	I record wether I should appear at the start of a new line."

	startNewLine := aBoolean.
	^self
			
!

string
	"^a String 
	I return the string I represent."

	^string
			
!

string: aString 
	"^self 
	I record the string I represent."

	string := PDFString on: aString.
	^self
			
! !

!PDFTextString methodsFor:'emitting'!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter."

	aWriter write: self string asEscapedString.
	self startNewLine
		ifTrue: [aWriter writeLine: '''']
		ifFalse: [aWriter writeLine: 'Tj'].
	^self
			
! !

!PDFTextString methodsFor:'initialize-release'!

on: aString for: aParent 
	"^self 
	I initialize myself to represent a string within the context of aParent."

	self string: aString.
	self parent: aParent.
	self parent applyDefaultOperatorsIfNecessary.
	^self
			
! !

!PDFTextString methodsFor:'private'!

applyTo: aTextObject 
	"^self 
	I add myself as an operator within aTextObject by adding myself to the 
	operators vector."

	aTextObject applyDefaultOperatorsIfNecessary.
	aTextObject operators add: self.
	^self
			
! !

!PDFTextString methodsFor:'services'!

showOnNewLine
	"^self 
	I set myself to appaer at the start of a new line."

	self startNewLine: true.
	^self
			
!

showOnSameLine
	"^self 
	I set myself to follow on on the current line ."

	self startNewLine: false.
	^self
			
! !

!PDFTextString class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFTextString.st,v 1.1 2006/09/07 11:22:33 sr Exp $'
! !
