"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFWriter
	instanceVariableNames:'file characterWriter writeSooner'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFWriter comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.

My instances are the chaps that looks after writing the PDF syntax out to the underlying writer in order to have the strings specified by my client appear in the final PDF document.  They use a PDFCharacterWriter to keep tabs on the current position (i.e. offset) with the written file.  This is vital for PDF files as they depend heavily on internal direct reference by character offset.'
!


!PDFWriter class methodsFor:'instance creation'!

on: aStream 
	"^a PDFWriter 
	I return a new instance of myself which is initialized to 
	write to aStream.  I check to see if this is a file stream,
	and if it is, I set it to boolean."

	| newWriter |
	newWriter := self new.
	(aStream respondsTo: #binary) ifTrue: [aStream binary].
	newWriter characterWriter: (PDFCharacterWriter on: aStream).
	^newWriter
			
! !

!PDFWriter methodsFor:'accessing'!

characterWriter
	"^a CharacterWriter 
	I return my character writer."

	^characterWriter
			
!

characterWriter: aCharacterWriter 
	"^self 
	I record my character writer."

	characterWriter := aCharacterWriter.
	^self
			
!

file
	"^a PDFFile 
	I return my PDFFile."

	file isNil ifTrue: [file := PDFFile for: self].
	^file
			
!

file: aFile 
	"^self 
	I record my PDFFile."

	file := aFile.
	^self
			
!

writeSooner
	"^a Boolean 
	I return whether (or not) I have a write sooner policy."

	writeSooner isNil ifTrue: [self writeSooner: true].
	^writeSooner
			
!

writeSooner: aBoolean 
	"^self 
	I record whether (or not) I have a write sooner policy."

	writeSooner := aBoolean.
	^self
			
! !

!PDFWriter methodsFor:'control'!

close
	"^self 
	I flush the remaining PDF information to my >>characterWriter and then 
	close it. To achieve this feat, I simply ask the trailer of my PDF file to 
	write itself out. This in turn causes all of the other bits of the PDF file 
	to be written first. This concludes my activities, so I null out my 
	>>characterWriter and my >>file"

	self file trailer writeOn: self characterWriter.
	self characterWriter: nil.
	self file: nil.
	^self
			
!

compressionOff
	"^self 
	Compression will not be used."

	self file compressionOff.
	^self
			
! !

!PDFWriter methodsFor:'services'!

defaultPage
	"a PDFPage 
	I return the current page in the PDFFile."

	^self file defaultPage
			
!

hasWriteLaterPolicy
	"^a Boolean 
	I return true if it is my poilicy to avoid writing to the PDF file until 
	the last possible moment. Otherwise, I return false."

	^self writeSooner not
			
!

hasWriteSoonerPolicy
	"^a Boolean 
	I return true if it is my poilicy toe write to the PDF file as soon 
	as possible (as opposed to waiting until the last possible moment). 
	Otherwise, I return false."

	^self writeSooner
			
!

mediaBox: aPDFRectangle 
	"^self 
	I set the MediaBox (paper size) to aRectangle."

	self file body catalog pagesRoot mediaBox: aPDFRectangle.
	^self
			
! !

!PDFWriter methodsFor:'writing'!

newLine
	"^self 
	I simply move to the next line in the PDF document on the default 
	page. Note that this does necesarily mean a new line in the PDF 
	file."

	self file defaultPage newLine.
	^self
			
!

newPage
	"^a PDFPage 
	I move to a new page in the PDF document. This new page will become 
	the current page, and subsequent write(), writeLine() and newLine() calls 
	will affect this new page."

	^self file newPage
			
!

write: aString 
	"^self 
	I just write aString to the default page of the PDF document. Note that there 
	is no new line or anything like that."

	self file defaultPage write: aString.
	^self
			
!

writeLine: aString 
	"^self 
	I write aString to the default page of the PDF document and then move to the 
	next line."

	self file defaultPage writeLine: aString.
	^self
			
! !

!PDFWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFWriter.st,v 1.1 2006/09/07 11:17:54 sr Exp $'
! !
