"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

PDFBodyObject subclass:#PDFXObject
	instanceVariableNames:'image name imageSizeObject'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Core'
!

PDFXObject comment:'Copyright (c) 1998 SPL WorldGroup, Inc. ("SPL WorldGroup").  All rights reserved. Protected by the copyright laws of the United States and international treaties.  This software may only be used in accordance with the terms and conditions of the "SPL WorldGroup, Inc. Software Web Site Agreement" which can be viewed on SPL WorldGroup''s web site, at www.splwg.com.'
!


!PDFXObject class methodsFor:'instance creation'!

onImage: anImage for: aPDFStructureElement 
	"^a PDFXImage
I return a new instance of myself on anImage."

	| newMe |
	newMe := self for: aPDFStructureElement.
	newMe onImage: anImage.
	^newMe
			
! !

!PDFXObject methodsFor:'accessing'!

image
	^image
			
!

imageSizeObject
	"^a PDBIntegerObject
I return the integer object I'll use to specify the size of my image in
bytes.  I make refernce to this object in my stream's /Length clause (which
preceeds the image bytes), and then set the value and write the image object
once I've written the image out and know it's size."

	imageSizeObject isNil 
		ifTrue: [imageSizeObject := PDFIntegerObject for: self].
	^imageSizeObject
			
!

name
	"^a String
I return my name.  This is used when I am being refered to as
a resourse."

	^name
			
!

name: aString 
	"^self
I record my name.  I can only be named once, after that I ignore
further requests to name me."

	name isNil ifTrue: [name := aString].
	^self
			
! !

!PDFXObject methodsFor:'emitting'!

writePDFSyntaxOn: aWriter 
	"^self 
	I write myself to aWriter. "

	self id writeDeclarationOn: aWriter.
	aWriter newLine.
	aWriter writeLine: '<<'.
	aWriter writeLine: '/Type /XObject'.
	aWriter writeLine: '/Subtype /Image'.
	aWriter 
		writeLine: '/Width ' , self image width printString.
	aWriter 
		writeLine: '/Height ' , self image height printString.
	aWriter writeLine: '/BitsPerComponent ' 
				, self image depth printString.
	aWriter writeLine: '/ColorSpace /DeviceGray'.
	"aWriter writeLine: '/Filter [/LZWDecode]'."
	aWriter writeLine: '/Length '.
	self imageSizeObject id writeReferenceOn: aWriter.
	aWriter writeLine: '>>'.
	self writeStreamBodyOn: aWriter.
	aWriter writeLine: 'endobj'.
	self imageSizeObject writeOn: aWriter.
	^self
			
!

writeResourceReferenceOn: aWriter 
	"^self 
	I write my reference as it appears in the resources enty of the catalog object 
	of the PDF file."

	aWriter write: '/' , self name , ' '.
	self id writeReferenceOn: aWriter.
	^self
			
!

writeStreamBodyOn: aWriter 
	"^self
I write the body of the stream to my writer."

	| startByte |
	aWriter writeLine: 'stream'.
	startByte := aWriter position.
	self image writeRawImageBytesTo: aWriter.
	self imageSizeObject integer: (aWriter position - startByte).
	aWriter newLine.
	aWriter writeLine: 'endstream'.
	^self
			
! !

!PDFXObject methodsFor:'initialize-release'!

onImage: anImage 
	"^self
I return myself initialized to represent anImage."

	image := anImage.
	self catalog addXObject: self.
	^self
			
! !

!PDFXObject class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFXObject.st,v 1.1 2006/09/07 11:15:52 sr Exp $'
! !
