"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

TIFFElement subclass:#TIFFBytes
	instanceVariableNames:'byteArray firstByteOffset'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFBytes class methodsFor:'instance creation'!

on: aByteArray firstByteOffset: anInteger 
	"^a TIFFBytes
I return a new instance of myself which wraps the byte array for
a TIFF image."

	^self new on: aByteArray firstByteOffset: anInteger
			
!

within: aTIFFImage on: aByteArray firstByteOffset: anInteger 
	"^a TIFFBytes
I return a new instance of myself which wraps the byte array for
a TIFF image."
 
	| newMe |
	newMe := self within: aTIFFImage.
	newMe on: aByteArray firstByteOffset: anInteger.
	^newMe
			
! !

!TIFFBytes methodsFor:'accessing'!

byteArray
	"^a ByteArray
I return my byte array."

	^byteArray
			
!

firstByteOffset
	"^an Integer
I return the offset of my first byte.  I expect this to be zero for a TIFF
file (not that I rely on that).  As the first element in Smalltalk collections
is always one (1) I must always take this into account when reading the
bytes at a given offset."

	^firstByteOffset
			
! !

!TIFFBytes methodsFor:'converting'!

readStream
	"^a ReadStream
Answer a readStream on the receiver.  Subclasses can provide their own kinds of Stream here."

	^ReadStream on: self byteArray
			
! !

!TIFFBytes methodsFor:'initialize-release'!

on: aByteArray firstByteOffset: anInteger 
	"^self
I initialize myself to wrap the byte array for a TIFF image."

	byteArray := aByteArray asByteArray.
	firstByteOffset := anInteger.
	^self
			
! !

!TIFFBytes methodsFor:'private'!

offsetBias
	"^an Integer
I return the integer which I add to any offsets I'm given to convert
the external offset to my internal offset (which always starts from 1).
All TIFF references are counted from zero (i.e. the first byte is number 0)."

	^self firstByteOffset + 1
			
! !

!TIFFBytes methodsFor:'services'!

asHexStringFromByte: start to: end 
	"^a String
I return a string which shows the specified bytes of the TIFF file as a radix 16 (Hex.)
string."

	| outputStream |
	outputStream := String new writeStream.
	(self readBytesFrom: start to: end) 
		do: [:aByte | aByte printOn: outputStream	" base: 16"].
	^outputStream contents
			
!

readBytesFrom: requestedFirst to: requestedLast 
	"^an Integer
I return the bytes from the first to the last bytes.  The bytes I
return will be shifted internally by my offsetBias."

	| first last |
	first := requestedFirst + self offsetBias.
	last := requestedLast + self offsetBias.
	^self byteArray copyFrom: first to: last
			
!

readIntegerFrom: first to: last 
	"^an Integer
I return integer value of the bytes from the first to the last bytes.  This
will be affected by the 'endian' setting for this tiff image."

	| result integerBytes |
	integerBytes := (self readBytesFrom: first to: last).
	self image header isBigEndian ifTrue: [integerBytes := integerBytes reverse].
	result := 0.
	1 to: last - first + 1
		do: 
			[:position | 
			result := result 
						+ ((integerBytes at: position) * (2 ** ((position - 1) * 8)))].
	^result
			
!

readStringFrom: first to: last 
	"^a String
I return the String value of the bytes from the first to the last bytes."

	^(self readBytesFrom: first to: last) asString
			
! !

!TIFFBytes class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFBytes.st,v 1.1 2006/09/07 11:21:17 sr Exp $'
! !
