"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

TIFFObject subclass:#TIFFField
	instanceVariableNames:'tag valueType numberOfValues values'
	classVariableNames:'ValueBlocks'
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFField class methodsFor:'initialization'!

initialize
	"TIFFField initialize"

	ValueBlocks := nil.
	^self

! !

!TIFFField class methodsFor:'instance creation'!

classForTag: anInteger
	"^a Class (a subclass of me)
	I return the appropriate class for the given tag, anInteger.  I use some
	meta-behaviour to do this.  Let me know if you think of a better way of
	doing this."

	^self allSubclasses detect: [:aFieldClass| aFieldClass supportsTagNumber: anInteger]
		ifNone:
			[nil halt: 'No class is ready to handle tag #' , anInteger printString.
			nil]

!

ofAppropriateClassWithin: aTIFFImage at: anOffset
	"^a TIFFField
	I return a new instance of one of my subclasses.  The subclass
	depends upon the tag value of the field."

	|new targetClass |
	targetClass := self classForTag: (aTIFFImage bytes readIntegerFrom: anOffset to: anOffset + 1).
	new := targetClass within: aTIFFImage at: anOffset.
	^new

! !

!TIFFField class methodsFor:'static information'!

supportsTagNumber: anInteger
	"^a Boolean
	Mostly this means that my tag must be == anInteger, but some cases
	are more tricky, in which case this method may be overidden."

	^self tag == anInteger

!

tag
	self subclassResponsibility

!

tagName
	self subclassResponsibility

! !

!TIFFField class methodsFor:'valueCalculation'!

shortTypeBlock
	"^an Integer
I return my value as a SHORT / 2 Byte int.  This is type 3.  The value
comes from the first two bytes of the value/offset part of the field."

	^[:aField | aField image bytes readIntegerFrom: aField offset + 8
						to: aField offset + 9]

! !

!TIFFField methodsFor:'accessing'!

numberOfValues
	numberOfValues isNil
		ifTrue:
			[numberOfValues := self bytes readIntegerFrom: self offset + 4
						to: self offset + 7].
	^numberOfValues

!

tag
	tag isNil
		ifTrue: [tag := self bytes readIntegerFrom: self offset to: self offset + 1].
	^tag

!

tagName
	^self class tagName

!

value
	"^an Object or nil
	I return either a single TIFFValue, or nil.  If self>>numberOfValues is > 1
	I return nil, otherwise, I return the single value."

	^self numberOfValues > 1 ifTrue: [nil] ifFalse: [self values first]

!

valueType
	valueType isNil
		ifTrue:
			[valueType := self bytes readIntegerFrom: self offset + 2
						to: self offset + 3].
	^valueType

!

values
	"^an Object
	I return either a single TIFFValue, or an OrderedCollection of values depending upon
	self>>numberOfValues."

	values isNil ifTrue: [values := TIFFValue allValuesOfAppropriateTypeFor: self].
	^values

! !

!TIFFField methodsFor:'testing'!

writeSummaryTo: aStream
"^self
I write a textual summary of myself to aStream."
	aStream
		nextPutAll: '>>Tag: ';
		nextPutAll: self tag printString;
		nextPutAll: ' (';
		nextPutAll: self tagName;
		nextPutAll: ').  Type: ';
		nextPutAll: self valueType printString;
		nextPutAll: '.  Number of values: '     ;
		nextPutAll: self numberOfValues printString; cr.
	self writeValueSummaryTo: aStream.
	self values do: [:value| value writeSummaryTo: aStream].
	^self

!

writeValueSummaryTo: aStream
	"^self
I don't do anything here, but my subclasses may choose to override this method and display a more human friendly version of their values.  For example, if the values add up to a string, showing each character on a new line is not so nice."

	^self

! !

!TIFFField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFField.st,v 1.2 2006/09/07 11:37:06 sr Exp $'
! !

TIFFField initialize!
