"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#TIFFImage
	instanceVariableNames:'bytes header directories'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFImage class methodsFor:'instance creation'!

fromByteArray: aByteArray 
	"^a TIFFImage
I create a new instance of myself to represent the image presumed to
be in the contents of aByteArray."

	| newMe |
	newMe := self new.
	newMe fromByteArray: aByteArray.
	^newMe
			
!

fromFile: aFilename 
	"^a TIFFImage
I create a new instance of myself to represent the image presumed to
be in the contents of a file named aFilename."

	| readStream |
	readStream := aFilename readStream.
	readStream binary.
	^self fromByteArray: readStream contents
			
!

fromFileNamed: aString 
	"^a TIFFImage
I create a new instance of myself to represent the image presumed to
be in the contents of a file named aString."

	^self fromFile: (Filename named: aString)
			
! !

!TIFFImage methodsFor:'accessing'!

bytes
	"^a TIFFBytes
I return my bytes, the data that comprises the TIFF file
I represent."

	^bytes
			
!

directories
	"^an OrderedCollection
	I return my collection of IFDs ( image file directories).  There is one IFD
	for each image in the TIFF file I represent.  There must be at least one."

	| currentIFD |
	directories isNil 
		ifTrue: 
			[directories := OrderedCollection new.
			currentIFD := TIFFImageFileDirectory within: self
						at: self header firstIFDOffset.
			[currentIFD notNil] whileTrue: 
					[directories add: currentIFD.
					currentIFD := currentIFD nextImageFileDirectory]].
	^directories
			
!

header
	"^a TIFFImageFileHeader 
I return my header.  My header knows things like the byte ordering and the offset
of the first directory (IFD)."

	header isNil ifTrue: [header := TIFFImageFileHeader within: self at: 0].
	^header
			
! !

!TIFFImage methodsFor:'converting'!

asNativeImage
	"^an Image
I return an Image which is a conversion of the first image I represent."

	^self directories first asNativeImage
			
!

writeRawImageBytesTo: aWriter 
	self directories first writeRawImageBytesTo: aWriter.
	^self
			
! !

!TIFFImage methodsFor:'initialize-release'!

fromByteArray: aByteArray 
	bytes := TIFFBytes 
				within: self
				on: aByteArray
				firstByteOffset: 0.
	^self
			
! !

!TIFFImage methodsFor:'printing'!

writeSummaryTo: aStream 
	"^a String
	I write a summary report of my structure to aStream."

	aStream
		nextPutAll: 'TIFF file contains ';
		nextPutAll: self directories size printString;
		nextPutAll: ' images.';
		cr.
	self header writeSummaryTo: aStream.
	self directories do: [:directory | directory writeSummaryTo: aStream].
	^self
			
! !

!TIFFImage methodsFor:'services'!

fileSizeInBytes
	^self bytes byteArray size
			
! !

!TIFFImage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFImage.st,v 1.1 2006/09/07 11:18:47 sr Exp $'
! !
