"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

TIFFObject subclass:#TIFFImageFileHeader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFImageFileHeader methodsFor:'accessing'!

byteOrder
	"^a String
The byte order used within the file. Legal values are:
'II' (4949.H)
'MM' (4D4D.H)
In the 'II' format, byte order is always from the least significant byte to the most
significant byte, for both 16-bit and 32-bit integers This is called little-endian byte
order. In the 'MM' format, byte order is always from most significant to least
significant, for both 16-bit and 32-bit integers. This is called big-endian byte
order."

	^self bytes readStringFrom: 0 to: 1
			
!

firstIFDOffset
	"^an Integer
I return the offset of the first Image File Directory (IFD) in the TIFF
file for which I am the header."

	^self bytes readIntegerFrom: 4 to: 7
			
!

idNumber
	"^an Integer
An arbitrary but carefully chosen number (42) that further identifies the file as a
TIFF file.
The byte order depends on the value of Bytes 0-1."

	^self bytes readIntegerFrom: 2 to: 3
			
! !

!TIFFImageFileHeader methodsFor:'services'!

isBigEndian
	"^a Boolan
I return true if the TIFF file I represent is uses Big-Endian
byte ordering, otherwise I return false."

	^self byteOrder = 'MM'
			
!

isLittleEndian
	"^a Boolan
I return true if the TIFF file I represent is uses Little-Endian
byte ordering, otherwise I return false."

	^self isBigEndian not
			
! !

!TIFFImageFileHeader methodsFor:'testing'!

writeSummaryTo: aStream
"^self
I write a textual summary of myself to aStream."
	aStream
		cr;
		nextPutAll: '*** Header ***'; cr;
		nextPutAll: 'Byte ordering is '.
	self isBigEndian
		ifTrue: [aStream nextPutAll: '"Big Endian" (Motorola).']
		ifFalse:  [aStream nextPutAll: '"Little Endian" (Intel).'].
	aStream
		cr;
		nextPutAll: 'The "Arbitrary but carefully chosen number" is ';
		nextPutAll: self idNumber printString;
		cr.
	^self
			
! !

!TIFFImageFileHeader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFImageFileHeader.st,v 1.1 2006/09/07 11:20:32 sr Exp $'
! !
