"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

TIFFObject subclass:#TIFFValue
	instanceVariableNames:'value'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFValue class methodsFor:'instance creation'!

allValuesFor: aField 
	"^an OrderedCollection
I return an ordered collection of my instances for aTIFF field."

	| values numberOfValues valueOffset |
	numberOfValues := aField numberOfValues.
	values := OrderedCollection new: numberOfValues.
	numberOfValues * self valueLength <= 4 
		ifTrue: 
			[valueOffset := aField offset + 8.
			values add: (self within: aField image at: valueOffset)]
		ifFalse: 
			[valueOffset := aField image bytes readIntegerFrom: aField offset + 8
						to: aField offset + 11.
			numberOfValues timesRepeat: 
					[values add: (self within: aField image at: valueOffset).
					valueOffset := valueOffset + self valueLength]].
	^values
			
!

allValuesOfAppropriateTypeFor: aTIFFField 
	"^an OrderedCollection
I return an ordered collection of my instances for aTIFF field."

	| targetClass |
	targetClass := self classForType: aTIFFField valueType.
	^targetClass allValuesFor: aTIFFField
			
!

classForType: valueType
	"^a Class (a subclass of me)
	I return the appropriate class for the given value type (an Integer).  I use some
	meta-behaviour to do this.  Let me know if you think of a better way of
	doing this."

	^self allSubclasses detect: [:aValueClass| aValueClass type == valueType]
		ifNone: 
			[nil halt: 'No class is ready to handle value type #' , valueType printString.
			nil]
			
! !

!TIFFValue class methodsFor:'static information'!

type
	^self subclassResponsibility
			
!

typeName
	^self subclassResponsibility
			
!

valueLength
	^self subclassResponsibility
			
! !

!TIFFValue methodsFor:'accessing'!

typeName
	^self class typeName
			
!

value
	^self subclassResponsibility
			
!

valueLength
	^self class valueLength
			
! !

!TIFFValue methodsFor:'testing'!

writeSummaryTo: aStream 
	"^self
I write a textual summary of myself to aStream."

	aStream
		tab;
		nextPutAll: '>>> Value (';
		nextPutAll: self typeName;
		nextPutAll: ') : ';
		nextPutAll: self value printString;
		nextPutAll: '.';
		cr.
	^self
			
! !

!TIFFValue class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFValue.st,v 1.1 2006/09/07 11:22:26 sr Exp $'
! !
