"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#TestReport13
	instanceVariableNames:'targetStream report bodyFrame sectionFrame sectionCharacterFrame
		headerFont'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Testing'
!


!TestReport13 class methodsFor:'testing'!

test
	"^self
	Write myself to the test directory at pdfreporting13.pdf."

	"TestReport13 test"

	| fileStream |
	fileStream := (PDFReportingSelfTest testFileNamed: 'spdfreporting13.pdf') 
				writeStream.
	[self new writeTo: fileStream] ensure: [fileStream close].
	^self
			
! !

!TestReport13 methodsFor:'accessing'!

bodyFrame
	"On an A4 portrait page, this frame is 50 points in from the edge all round."

	bodyFrame isNil 
		ifTrue: 
			[bodyFrame := PDFMetaFrame 
						within: self report mediaFrame
						x: 50
						y: 786
						width: self bodyWidth
						height: 736.
			bodyFrame setBorderOff.
			bodyFrame name: 'bodyFrame'].
	^bodyFrame
			
!

bodyWidth
	"^an Integer
	The width, in points, of the body of the report.  50 points in from each side."

	^ 485
			
!

headerFont
	headerFont isNil 
		ifTrue: [headerFont := PDFBase14Type1Font named: 'Times-Bold'].
	^headerFont
			
!

report
	"^a PDFReport
	Initialized to be A4 portrait (585x836 points)."

	report isNil 
		ifTrue: 
			[report := PDFReport on: targetStream.
			report 
				mediaBox: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 595 836))].
	^report
			
!

sectionCharacterFrame
	sectionCharacterFrame isNil 
		ifTrue: 
			[sectionCharacterFrame := (PDFStringCharacterFrame width: 100)
						marginWidth: 1;
						justifyLeft;
						font: PDFFont type1Helvetica;
						fontPitch: 12;
						yourself].
	^sectionCharacterFrame
			
! !

!TestReport13 methodsFor:'writing'!

writeReport
	self prepareHeader.
	20 timesRepeat: 
			[self writeSection.
			self writeSpaceFrame].
	^self
			
!

writeTo: aStream 
	targetStream := aStream.
	report := self report.
	self writeReport.
	report close.
	^self
			
! !

!TestReport13 methodsFor:'writing-private'!

prepareHeader
	self report pageHeader addDeferredFrame: 
			[:aReport | 
			| textFrame headerFrame footerFrame watermarkFrame watermarkTextFrame |
			textFrame := (PDFStringCharacterFrame width: 60)
						marginWidth: 1;
						justifyCenter;
						font: self headerFont;
						fontPitch: 10.
			watermarkTextFrame := (PDFStringCharacterFrame width: 60)
						marginWidth: 1;
						justifyCenter;
						font: self headerFont;
						fontPitch: 30;
						grayFill: 90.
			headerFrame := PDFMetaFrame 
						within: self report mediaFrame
						x: 60
						y: 820
						width: self bodyWidth
						height: 50.
			headerFrame name: 'headerFrame'.
			footerFrame := PDFMetaFrame 
						within: self report mediaFrame
						x: 180
						y: 30
						width: 200
						height: 20.
			footerFrame name: 'footerFrame'.
			watermarkFrame := PDFMetaFrame 
						within: self report mediaFrame
						x: 65
						y: 490
						width: self bodyWidth
						height: 100.
			watermarkFrame name: 'watermarkFrame'.
			headerFrame writeLine: (textFrame setValue: 'TestReport13 - Header').
			footerFrame writeLine: (textFrame setValue: 'TestReport13 - Footer').
			watermarkFrame write: (watermarkTextFrame setValue: 'TestReport13')]
			
!

writeSection
	| frame |
	frame := PDFMetaFrame 
				within: self bodyFrame
				x: nil
				y: nil
				width: self bodyWidth
				height: nil.
	frame name: 'section frame'.
	frame setBorderOn.
	15 timesRepeat: 
			[frame writeLine: (self sectionCharacterFrame setValue: 'Body line')].
	frame close.
	self bodyFrame newLine.
	^self
			
!

writeSpaceFrame
	| frame |
	frame := PDFMetaFrame 
				within: self bodyFrame
				x: nil
				y: nil
				width: nil
				height: 10.
	frame setBorderOff.
	frame name: 'spaceFrame'.
	frame close.
	self bodyFrame newLine.
	^self
			
! !

!TestReport13 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TestReport13.st,v 1.1 2006/09/07 11:22:17 sr Exp $'
! !
