"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).

 For more information about the Manchester Goodies Library (from which
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help
"

"{ Package: 'stx:goodies/postscript' }"

Object subclass:#PSFontFamily
	instanceVariableNames:'fullName widths underlinePosition underlineThickness'
	classVariableNames:'Encodings PSFonts'
	poolDictionaries:''
	category:'Graphics-Postscript'
!

PSFontFamily comment:'I hold useful information from the font metric ("afm") files.  My metaclass
 contains code to parse the afm file and build instances.

Instance Variables:
	fullName                <String>        The name from the afm file.
	widths                  <Dictionary>    The width info from  afm file
						 -- assumes 1000 points in coordinate system.
	underlinePosition       <Number>        From the afm file.
	underlineThickness      <Number>        From the afm file.

Class Variables:
	Encodings               <Dictionary>    Maps character values to PostScript names
						for those characters

						Each character may have multiple names if
						different fonts refer to it by different names.
	PSFonts                 <Dictionary>    Holds the font declarations.
'!

!PSFontFamily class methodsFor:'documentation'!

copyright
"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).

 For more information about the Manchester Goodies Library (from which
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help
"


!

documentation
"
    I hold useful information from the font metric ('afm') files.
    My metaclass contains code to parse the afm file and build instances.

    [Instance Variables:]

	    fullName                <String>        The name from the afm file.
	    widths                  <Dictionary>    The width info from  afm file
						     -- assumes 1000 points in coordinate system.
	    underlinePosition       <Number>        From the afm file.
	    underlineThickness      <Number>        From the afm file.

    [Class Variables:]

	    Encodings               <Dictionary>    Maps character values to PostScript names
						    for those characters

						    Each character may have multiple names if
						    different fonts refer to it by different names.
	    PSFonts                 <Dictionary>    Holds the font declarations.

    Claus:
	since most standard installations do not have those afm files,
	I hardcoded some standard fonts (times, courier & helvetica).
	You may want to change things back to the original (see #XXinitialize).
	The new default settings work (more or less) on all printers we tried so far.

    [See also:]
	PSGraphicsContext PSFont PostscriptPrinterStream
	Font FontDescription
"
! !

!PSFontFamily class methodsFor:'initialization'!

fixNameInternal: string
	"Massage the font name,  getting rid of spaces.  Also,  map 'Oblique'
	to 'Italic',  'medium' to '',  'Light' to '',  'Demi' to 'Bold',  'Roman'
	to '',  for internal use... this is necessary to deal with some of the
	Adobe special naming choices."

	| new foundIndex1 foundIndex2 c |
	foundIndex1 := string findString: 'Book' startingAt: 1.
	foundIndex2 := string findString: 'Bookman' startingAt: 1.
	foundIndex2 == 0 ifTrue: [foundIndex1 == 0 ifFalse: [string
				replaceFrom: foundIndex1
				to: foundIndex1 + 3
				with: '    ']].
	foundIndex1 := string findString: 'Demi' startingAt: 1.
	foundIndex1 == 0 ifFalse: [string
			replaceFrom: foundIndex1
			to: foundIndex1 + 3
			with: 'Bold'].
	foundIndex1 := string findString: 'Oblique' startingAt: 1.
	foundIndex1 == 0 ifFalse: [string
			replaceFrom: foundIndex1
			to: foundIndex1 + 6
			with: 'Italic '].
	foundIndex1 := string findString: 'Light' startingAt: 1.
	foundIndex1 == 0 ifFalse: [string
			replaceFrom: foundIndex1
			to: foundIndex1 + 4
			with: '     '].
	foundIndex1 := string findString: 'Medium' startingAt: 1.
	foundIndex1 == 0 ifFalse: [string
			replaceFrom: foundIndex1
			to: foundIndex1 + 5
			with: '      '].
	foundIndex1 := string findString: 'Roman' startingAt: 1.
	foundIndex1 == 0 ifFalse: [string
			replaceFrom: foundIndex1
			to: foundIndex1 + 4
			with: '     '].
	new := WriteStream on: (String new: 1).
	1 to: string size do:
		[:i |
		c := string at: i.
		c == 32 asCharacter ifFalse: [new nextPut: c]].
	^new contents
!

getDirName
	"Prompt for and return the base directory name of the font metric files."

	^DialogView
		request: 'Where are the Font Metric files?'
		initialAnswer: ((Filename currentDirectory construct: 'utils')
							construct: 'FontMets') asString
!

processAdobeFiles: aCollection
	"Load in the Adobe Font Metric (AFM) files,  and massage them into a
	set of internal font descriptions..."

	| key f s i c width widths  named desc underlineThickness
		underlinePosition  psName charName |

	aCollection do:[:aName |
	    Transcript cr; show: 'Processing ' , aName.
	    key := aName.
	    widths := IdentityDictionary new: 300.
	    f := aName asFilename.
	    s := f contentsOfEntireFile.
	    named := self extractFieldNamed: 'FullName' from: s.
	    psName := self extractFieldNamed: 'FontName' from: s.
	    key := (self fixNameInternal: named) asSymbol.
	    underlineThickness := Number readFrom: (ReadStream on: (self extractFieldNamed: 'UnderlineThickness' from: s)).
	    underlinePosition := Number readFrom: (ReadStream on: (self extractFieldNamed: 'UnderlinePosition' from: s)).
	    i := s indexOfSubCollection: 'StartCharMetrics' startingAt: 1.
	    i := s indexOfSubCollection: 'C ' startingAt: i.
	    f := s readStream.
	    f position: i.

	    [
		f next = Character space ifFalse: [self afmReadError].
		[f next = $W] whileFalse.
		f next = $X ifFalse: [self afmReadError].
		f next = Character space ifFalse: [self afmReadError].
		width := Integer readFrom: f.
		[f next = $N] whileFalse.
		f next = Character space ifFalse: [self afmReadError].
		charName := (f upTo: Character space) asSymbol.
		widths at: charName put: width.
		f skipThrough: 13 asCharacter.
		c := f next.
		c = $C
	    ] whileTrue.
	    f close.
	    desc := (self new) fullName: psName;
			       widths: widths;
			       underlineThickness: underlineThickness;
			       underlinePosition: underlinePosition.
	    PSFonts at: key put: desc
	].
	^PSFonts
!

processEncodings: fName

	| codes codeStream |
	codes := fName asFilename readStream fileIn.
	(codes class == Array and: [codes size even])
		ifFalse:        [self error: 'Character code file may be bad'].
	codeStream := codes readStream.
	Encodings := IdentityDictionary new.
	[codeStream atEnd]
		whileFalse:
			[| charName charVal |
			charName := codeStream next.
			charVal := codeStream next.
			(charName isString and: [charVal isInteger])
				ifTrue: [charName := charName asSymbol.
						Encodings at: charVal
								put: ((Encodings
										at: charVal
										ifAbsent: [#()])
									copyWith: charName)]
				ifFalse:        [self error: 'Character code file may be bad']].! !

!PSFontFamily class methodsFor:'accessing'!

named: aSymbol
	"Return the font description corresponding to the given name."

	^PSFonts at: aSymbol ifAbsent:[ PSFonts at: #Helvetica ].
! !

!PSFontFamily class methodsFor:'class initialization'!

XXinitialize
	"Process the default fonts for the LaserWrite-Plus."
	"PSFontFamily initialize."

	| dirList directoryNameString directoryName |

	PSFonts := IdentityDictionary new: 50.
	[directoryNameString := self getDirName.
	directoryNameString isEmpty or:
		[directoryName := directoryNameString asFilename.
		(Filename errorReporter inaccessibleSignal
				handle: [:ex | ex returnWith: true]
				do: [directoryName isDirectory not])
			or: [(dirList := directoryName filesMatching: '*AFM') isEmpty]]]
		whileTrue:
			[(DialogView confirm: '"', directoryName asString, '" is missing, empty or not a directory.
				Try again?') ifFalse: [^self]].
	self processAdobeFiles: dirList.
	self processEncodings: (directoryName construct: 'PScodes')
!

initialize
	"create some dummy fonts if you dont have the afm-files."
	"PSFontFamily initialize."

	PSFonts := IdentityDictionary new: 50.
	#('Times-Roman' 'Times' 'Courier' 'Helvetica' 'Helv') do:[:psName |
	    |desc|

	    desc := (self new) fullName: psName;
						widths: nil;
						underlineThickness: 1;
						underlinePosition: 0.
	    PSFonts at: psName asSymbol put: desc
	].
! !

!PSFontFamily class methodsFor:'private'!

afmReadError
	self error: 'Can''t parse font metrics file'
!

extractFieldNamed: field from: string
	"Find a named field in a string,  and return it."

	| i result |
	i := string indexOfSubCollection: field startingAt: 1.
	i := string indexOfSubCollection: ' ' startingAt: i.
	result := string copyFrom: i + 1 to: (string indexOfSubCollection: ' ' startingAt: i) - 1.
	^result
! !

!PSFontFamily methodsFor:'accessing'!

encode: aCharacter

	| umlaute |


	"convert german Umlaut to ESC Code"
	umlaute := Dictionary new.
	umlaute at: $ put: ('\224').
	umlaute at: $ put: ('\204').
	umlaute at: $ put: ('\201').
	umlaute at: $ put: ('\231').
	umlaute at: $ put: ('\216').
	umlaute at: $ put: ('\232').
	umlaute at: $ put: ('ss').    "/ mhmh - almost

	(umlaute includesKey: aCharacter) ifTrue:[ ^ umlaute at: aCharacter ].
	^65535

"
	This is the original Code - but I need an encoding only for special country specific characters

	names := Encodings at: aCharacter asInteger ifAbsent: [^65535].
	1 to: names size do:[:i |
	    (widths includesKey: (names at: i)) ifTrue: [^names at: i]
	].
	^65535
"
!

fullName
	"return the full name of the current font."

	^fullName
!

fullName: string
	"Set the current full name of the current font."

	fullName := string
!

underlinePosition
	"Return where the underline goes in the current font."

	^underlinePosition
!

underlinePosition: number
	"Set where the underline goes in the current font."

	underlinePosition := number
!

underlineThickness
	"Return how thick an underline is done in the current font."

	^underlineThickness
!

underlineThickness: number
	"Set how thick the underline should be in the current font.."

	underlineThickness := number
!

widths
	"Return an array of the widths of all the characters in the current
	font."

	^widths
!

widths: array
	"Set the widths of all the characters in the current font from the
	contents of the given array."

	widths := array
! !

!PSFontFamily class methodsFor:'documentation'!

version
   ^ '$Header: /cvs/stx/stx/goodies/postscript/PSFontFamily.st,v 1.10 2007/11/08 14:33:40 cg Exp $'
! !
PSFontFamily initialize!
