"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"
"{ Package: 'stx:goodies/postscript' }"

DeviceGraphicsContext subclass:#PSGraphicsContext
	instanceVariableNames:'currentFontStyle title titleFont fontPolicy medium established
		imageResolver pageName pageCounter pageEndAction pageStartAction
		layout pageOffset psScale psTrans psLineStyle psLineWidth
		printPageNumbers pageString'
	classVariableNames:'CurrentContext'
	poolDictionaries:''
	category:'Graphics-Postscript'
!

!PSGraphicsContext class methodsFor:'documentation'!

copyright
"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"
!

documentation
"
    I output PostScript commands representing the graphical objects displayed on
    me to a file (via a PSMedium).

    There are several bugs. The worst is that the printed output is sometimes improperly
    clipped, which probably happens because the bounding boxes are incorrect. Also,
    if you print the System Workspace, you will see that some strings are mirror
    images of what they should be, and that extraneous spaces appear. (These are
    probably all simple Postscript problems, but I don''t know Postscript. -- Carl
    McConnell 7-21-93). 

    If you make a copy of me, change my characteristics (i.e. color or font or offset
    or clipping bounds), and then go back to the original graphics context, I must
    generate Postscript to undo your change or the picture will be messed up. (An
    example of a place where this occurs is the method displayFromCharacter:to:startX:forTranslation:on:
    in ComposedText. ) To avoid this kind of problem, we keep the graphics context
    previously used in a class variable. At the beginning of each public display
    method, I check to see if I am the previously used graphics context. If not,
    I send my state information like line width, paint color, font, and clipping
    bounds to the medium to ensure the printer is in the proper state, and then
    store myself in the class variable.

    This class is copyrighted by Luis Tavera and Thorr Einarsson. However, it may
    be freely distributed and modified with the condition that Prof. Ralph Johnson
    (johnson@cs.uiuc.edu) be sent a copy of the changes. You should also direct
    any comments to him (*).

    [author:]
        Luis Tavera & Thorr Einarsson -- main authors
        Markus Geltz -- made modifications to produce encapsulated Postscript
        Carl McConnell -- fixed the copied graphics context problem
        Claus Gittinger -- added depth8 bitmaps, and adapted things for ST/X
        (*) class is published within the free demo version of ST/X ...
        Claus Atzkern -- many additions, page numbering, transformations & scaling


    [Instance variables:]
        medium                  <PSMedium>              where to output PostScript commands
        currentFontStyle        <FontDescription>       font being used

    [Class variables:]
        CurrentContext          <PSGraphicsContext>     last instance used

    [see also:]
        PostscriptPrinterStream
        GraphicsContext
"
!

examples
"
    create a postscript file.
    draw line - just as it would be in a view:
                                                                [exBegin]
    | file gc |

    file := DialogView 
                requestFileName: 'Name of file for Postscript:' 
                default: 'view.ps'.
    file isEmpty ifTrue: [^ self].

    Cursor write showWhile:[
        gc := PSGraphicsContext fileName:file extent:200@200.
        gc printPageNumbers:false.
        gc displayLineFrom:0@0 to:100@100.
        gc close
    ]
                                                                [exEnd]


    sending it to directly to a postscript printer:
                                                                [exBegin]
    | printerStream gc |

    printerStream := Printer newNative. 

    Cursor write showWhile:[
        gc := PSGraphicsContext on:printerStream extent:50@50.
        gc printPageNumbers:false.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(1 @ 2).
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(2 @ 1).
        gc displayLineFrom:0@0 to:100@100.
        gc translation:50@0.
        gc paint:Color red.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:1.
        gc paint:Color blue.
        gc lineStyle:#dashed.
        gc displayLineFrom:10@10 to:90@10.
        gc close
    ].
    
                                                                [exEnd]



    getting the postscript code:
                                                                [exBegin]
    | dummyPrinterStream gc postscript |

    dummyPrinterStream := '' writeStream.

    Cursor write showWhile:[
        gc := PSGraphicsContext on:dummyPrinterStream extent:50@50.
        gc printPageNumbers:false.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(1 @ 2).
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(2 @ 1).
        gc displayLineFrom:0@0 to:100@100.
        gc translation:50@0.
        gc paint:Color red.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:1.
        gc paint:Color blue.
        gc lineStyle:#dashed.
        gc displayLineFrom:10@10 to:90@10.
        gc close
    ].
    Transcript showCR:dummyPrinterStream contents.
                                                                [exEnd]



    the corresponding code to draw into a view::
                                                                [exBegin]
    | view gc |

    view := (View extent:200@200) openAndWait.

    Cursor write showWhile:[
        gc := view.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(1 @ 2).
        gc displayLineFrom:0@0 to:100@100.
        gc scale:(2 @ 1).
        gc displayLineFrom:0@0 to:100@100.
        gc translation:50@0.
        gc paint:Color red.
        gc displayLineFrom:0@0 to:100@100.
        gc scale:1.
        gc paint:Color blue.
        gc lineStyle:#dashed.
        gc displayLineFrom:10@10 to:90@10.
    ]
                                                                [exEnd]


    default: with page number:
                                                                [exBegin]
    |printerStream gc|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc displayLineFrom:0@0 to:100@100.
    gc displayLineFrom:100@0 to:0@100.
    gc close
                                                                [exEnd]


    changing the page-number format:
                                                                [exBegin]
    |printerStream gc|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc pageNumberFormat:'Seite %1'.
    gc displayLineFrom:0@0 to:100@100.
    gc displayLineFrom:100@0 to:0@100.
    gc close
                                                                [exEnd]

    disabling page-number printing:
                                                                [exBegin]
    |printerStream gc|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream extent:200@200.
    gc printPageNumbers:false.
    gc displayLineFrom:0@0 to:100@100.
    gc displayLineFrom:100@0 to:0@100.
    gc close
                                                                [exEnd]

    disabling page-number printing:
                                                                [exBegin]
    |printerStream gc|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc printPageNumbers:false.
    gc displayLineFrom:0@0 to:100@100.
    gc displayLineFrom:100@0 to:0@100.
    gc close
                                                                [exEnd]

    adding an end-page hook, which prints a frame around
    (could add corporate-logo etc.):
                                                                [exBegin]
    |printerStream gc|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc pageEndAction:[:pageNumber |
        gc displayRectangle:(100@100 corner:400@400).
    ].
    gc displayLineFrom:0@0 to:100@100.
    gc displayLineFrom:100@0 to:0@100.
    gc close
                                                                [exEnd]

    including an image and string:
                                                                [exBegin]
    |printerStream gc img|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    img := Image fromFile:'../../goodies/bitmaps/xpmBitmaps/misc_logos/stx_32x32.xpm'.
    gc displayImage:img at:400@400.
    gc displayString:'Hello World' at:400@380.
    gc close
                                                                [exEnd]

    scaling:
                                                                [exBegin]
    |printerStream gc img|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc psScale:(gc psScale * 2).
    gc translation:-300@-300.
    img := Image fromFile:'../../goodies/bitmaps/xpmBitmaps/misc_logos/stx_32x32.xpm'.
    gc displayImage:img at:400@400.
    gc displayString:'Hello World' at:400@380.
    gc close
                                                                [exEnd]

    including a scaled-down image:
                                                                [exBegin]
    |printerStream gc img|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    img := Image fromFile:'banner8.xpm'.
    gc displayImage:img at:400@400 scaledBy:0.25.
    gc close
                                                                [exEnd]
                                                                [exBegin]
    |printerStream gc img|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    img := Image fromFile:'banner8.xpm'.
    img := img asMonochromeFormOn:gc device.
    gc displayImage:img at:300@300.
    img := Image fromFile:'banner8.xpm'.
    img := img asMonochromeFormOn:gc device.
    gc displayImage:img at:100@100 scaledBy:0.25.
    gc close
                                                                [exEnd]

                                                                [exBegin]
    |printerStream gc img|

    printerStream := Printer newNative. 
    gc := PSGraphicsContext on:printerStream.
    gc font:(Font family:'times' size:48).
    gc displayString:'Hello World' x:300 y:300.
    gc close
                                                                [exEnd]
"
! !

!PSGraphicsContext class methodsFor:'instance creation'!

fileName:aString 
    "Answer an instance that will write to the named file."

    ^ self fileName:aString extent:nil

    "Modified: 1.6.1996 / 12:29:09 / cg"
!

fileName:aString extent:anExtent 
    "Answer an instance that will write to the named file."

    ^self on:(Filename named:aString) writeStream extent:anExtent

    "Modified: 1.6.1996 / 10:51:44 / cg"
!

on:aStream
    "Answer an instance that will write to the argument, aStream."

    ^ self on:aStream origin:(0 @ 0) corner:(1.0 @ 1.0)
!

on:aStream extent:anExtent 
    "Answer an instance that will write to the argument, aStream.
    "
    ^ self on:aStream origin:(0 @ 0) corner:anExtent
!

on:aStream origin:anOrigin
    ^ self on:aStream origin:anOrigin corner:(1.0 @ 1.0)
!

on:aStream origin:anOrigin corner:aCorner 
    "Answer an instance that will write to the argument, aStream.
    "
    |psgc|

    psgc := super onDevice:(PSMedium on:aStream).
    psgc origin:anOrigin corner:aCorner.

    ^ psgc
!

on:aStream origin:anOrigin extent:anExtent 
    "Answer an instance that will write to the argument, aStream.
    "
    |psgc|

    psgc := super onDevice:(PSMedium on:aStream).
    psgc origin:anOrigin extent:anExtent.

    ^ psgc
! !

!PSGraphicsContext class methodsFor:'constants'!

capButt
    "Answer the integer constant representing the butt cap style."
    ^0
!

capProjecting
        "Answer the integer constant representing the projecting cap style."
        ^2
!

capRound
        "Answer the integer constant representing the round cap style."
        ^1
!

joinBevel
        "Answer the integer constant representing the bevel join style."
        ^2
!

joinMiter
        "Answer the integer constant representing the miter join style."
        ^0
!

joinRound
        "Answer the integer constant representing the round join style."
        ^1
! !

!PSGraphicsContext class methodsFor:'constants-operators'!

closePath
        ^'closepath'
!

fill
        ^'fill'
!

grestore
        ^ 'grestore'.
!

gsave
        ^ 'gsave'.
!

lineTo
        ^'lineto'
!

moveTo
        ^'moveto'
!

newPath
        ^'newpath'
!

scale
        ^'scale'
!

setRGBColor
        ^'setrgbcolor'
!

setgray
        ^'setgray'
!

show
        ^'show'
!

showpage
        ^'showpage'
!

stroke
        ^'stroke'
!

translate
        ^'translate'
! !

!PSGraphicsContext class methodsFor:'constants-procedures'!

drawEllipse
    "Answer the drawellipse procedure in postscript"

    ^'DrawEllipse'
!

lineDef
    ^'Line'


!

pathDef
    ^'PathDef'


!

rectFill
        ^'*RF'
!

rectStroke
        ^'*RS'
! !

!PSGraphicsContext class methodsFor:'procedure definitions'!

defCTMsave
    "Answer the definition of a Postscript procedure that saves the current transformation matrix"

    ^ '/origCTM matrix currentmatrix def'.

    "Modified: 1.6.1996 / 10:50:43 / cg"
!

defDrawEllipse
    "Answer the definition of a Postscript procedure that draws an ellipse"

        ^ '/DrawEllipse          % stack startAngle sweepAngle ... -> --
{ /m matrix currentmatrix def
  % newpath
  4 2 roll translate scale 1 -1 scale
  0 0 1 5 3 roll arc
  % closepath
  m setmatrix
} bind def'

    "Modified: 4.6.1996 / 14:03:49 / cg"
!

defErrorHandler
    "Answer the definition of a PostScript procedure that prints error messages
     on the printer"

        ^ 'errordict begin
  /handleerror {
  systemdict begin
    initgraphics

    /Courier findfont 12 scalefont setfont
    72 720 moveto
    (Postscript problem:) show
    $error /errorname get
    =string cvs show
    72 700 moveto
    (Command: ) show
    $error /command get
    =string cvs show
    systemdict /showpage get exec
    end % systemdict
  } def
end
'.

    "Modified: 1.6.1996 / 10:50:25 / cg"
!

defLineDef
    "Answer the definition of a Postscript procedure that draws a line"

        ^'/Line {
  moveto lineto stroke
} bind def
'.
!

defPathDef
    "Answer the definition of a Postscript procedure that creates an open
     path
    "
    ^'/PathDef {
  dup 0 2 getinterval aload pop newpath moveto
  dup length 1 sub 2 2 3 -1 roll
  {
        exch dup 3 -1 roll 2 getinterval aload pop lineto
  }
  for pop

} bind def
'


!

defReEncodeSmall
    "Answer the definition of a Postscript procedure that reencodes german special characters"

^'/Reencsmalldict 12 dict def
/ReEncodeSmall
{  Reencsmalldict begin         %% written by PSGraphicsContext class defReEncodeSmall
   /NewCodesAndNames exch def
   /NewFontName exch def
   /BaseFontName exch def
   /BaseFontDict
   BaseFontName findfont def
   /NewFont BaseFontDict
    maxlength dict def

   BaseFontDict
   { exch dup /FID ne
     { dup /Encoding eq
       { exch dup length array copy
         NewFont 3 1 roll put
       }
       { exch NewFont 3 1 roll put
       } ifelse
     }
     { pop pop } ifelse
   } forall

   NewFont
   /FontName NewFontName put
    NewCodesAndNames aload pop 
    NewCodesAndNames
    length 2 idiv
    { NewFont /Encoding get
      3 1 roll put
    } repeat

    NewFontName NewFont
    definefont pop
    end
 } def

 /GermanVec
 [ 8#201 /udieresis
   8#204 /adieresis
   8#216 /Adieresis
   8#224 /odieresis
   8#231 /Odieresis
   8#232 /Udieresis
   8#341 /germandbls
 ] def

'

"usage"
"/ /Currier
"/ /Currier-German
"/ GermanVec ReEncodeSmall
"/ /Currier-German findfont
"/ 18 scalefont setfont

"/ 200 400 moveto
"/ (\232berm\204\372iger Lik\224rkonsum in der ) show
"/ showpage

    "Modified: 1.6.1996 / 10:49:49 / cg"
!

defRectFill
    "Answer the definition of a Postscript procedure that draws a filled rectangle"


        ^'/*RF {
  gsave newpath BuildRectPath fill grestore
} bind def
'

    "Modified: 3.6.1996 / 17:48:50 / cg"
!

defRectPath
    "Answer the definition of a Postscript procedure that builds a rectangular path"

        ^'/BuildRectPath {
  dup type dup /integertype eq exch /realtype eq or { %ifelse
    4 -2 roll moveto
    dup 0 exch rlineto exch 0 rlineto neg 0 exch rlineto closepath
  } {
    dup length 4 sub 0 exch 4 exch 
     {
        1 index exch 4 getinterval aload pop
        BuildRectPath
     } for
     pop
  } ifelse
} bind def
'.

    "Modified: 3.6.1996 / 17:48:57 / cg"
!

defRectStroke
    "Answer the definition of a Postscript procedure that draws a rectangle"

        ^'/*RS {
  gsave newpath BuildRectPath stroke grestore
} bind def
'

    "Modified: 3.6.1996 / 17:49:02 / cg"
!

defTextHandling
    "Last programmer: Markus Geltz (18 March 1993, 3:35:02 pm)" 
    "Creator: Markus Geltz (18 March 1993, 10:36:35 am)" 

    "Answer the definition of Postscript procedures that handle the text" 

^"%%initgraphics"
'
/F % Fast font setup.
{
  findfont exch scalefont setfont
} bind def

/ISOshow % show iso encoded string
{
  dup
  /oldfont currentfont def
  currentfont dup length dict /newdict exch def 
  {1 index /FID ne
      {newdict 3 1 roll put}
      {pop pop}
    ifelse
  } forall
  newdict /Encoding get 256 array copy
  newdict exch /Encoding exch put
  newdict /Encoding get exch 248 exch put
  /TFNT newdict definefont pop
  newdict setfont
  (\370) newShow
  oldfont setfont
} bind def

/textShow        % stack: text -> --
{
  /m matrix currentmatrix def
  1 -1 scale show
  m setmatrix
} bind def
'

    "Modified: 3.6.1996 / 17:49:09 / cg"
!

landScape
    "write postscriptCode for Landscape printing an do mirroring"

    | stream |

    stream := ReadWriteStream on: String new.

    stream nextPutAll: '%% Page 1 1';cr.
    stream nextPutAll: '90 rotate';cr.
    "p := Point x: 0.92 y: -0.9."     "negative scaling means mirroring!!"
    stream nextPutAll: '0.92 -0.9 scale';cr.
    "p:= Point x: 10 y: 10."
    stream  nextPutAll: '10 10 translate';cr.

    ^ stream contents

    "Modified: 18.11.1994 / 17:13:16 / R.Sailer"
    "Modified: 1.6.1996 / 10:47:36 / cg"
!

psPrologCommentExtent:anExtent
    "Last programmer: Markus Geltz (22 March 1993, 1:28:25 pm)" 
    "Creator: Markus Geltz (22 March 1993, 1:28:19 pm)" 

    "Answer the Postscript prologue for Encapsulated Postscript" 

    |s xs ys|

    s := '%!!PS-Adobe-2.0
%%Creator: Smalltalk/X
%%CreationDate: ' , Timestamp now printString ,'
'.

    anExtent notNil ifTrue:[
        xs := anExtent x.
        ys := anExtent y.
        s := s , '%%BoundingBox: ' , 20 printString , ' ' , 772 printString 
                                   , ' ' , (20 + xs) printString , ' ' , (772 - ys) printString , '
'.
    ].

    s := s , '%%Pages: (atend)
%%EndComments'.

    ^ s

    "Created: 7.9.1996 / 16:34:22 / cg"
    "Modified: 7.9.1996 / 16:35:15 / cg"
! !

!PSGraphicsContext methodsFor:'accessing'!

basicFont: aFont 
    "Set the default font that I use to draw strings."

    | thisFont changed underlined |
    changed := underlined := false.

    currentFontStyle := aFont.
    self setFont.
    font := aFont.

    true ifTrue:[ ^aFont].

    thisFont := (aFont isKindOf: Font)
                            ifTrue: [aFont] "If this is a font description, no parsing is needed."
                            ifFalse: [
                                    "aFont class parse: aFont name"].


    currentFontStyle isNil
            ifTrue: 
                    [currentFontStyle := thisFont.
                    changed := true].

    thisFont size ~= currentFontStyle size
            ifTrue: 
                    [currentFontStyle size: (thisFont size).
                    changed := true].
    thisFont italic ~= currentFontStyle italic
            ifTrue: 
                    [currentFontStyle italic: thisFont italic.
                    changed := true].
"        thisFont serif ~= currentFontStyle serif
            ifTrue: 
                    [currentFontStyle serif: thisFont serif.
                    changed := true].
"
     thisFont fixedWidth ~= currentFontStyle fixedWidth
            ifTrue: 
                    [currentFontStyle fixedWidth: (thisFont fixedWidth).
                    changed := true].
"        thisFont boldness ~= currentFontStyle boldness
            ifTrue: 
                    [currentFontStyle boldness: thisFont boldness.
                    changed := true].
" 
"        underlined := thisFont underline isNil
                            ifTrue: [false]
                            ifFalse: [thisFont underline].
    underlined
            ifTrue: 
                    [self startUnderlining.
                    changed := true].
"
    changed ifTrue: [self setFont]

    "Created: 6.3.1996 / 18:17:57 / cg"
    "Modified: 3.6.1996 / 18:04:05 / cg"
!

capStyle:aStyleSymbol
    "Set the line endCap style; the argument must be one of
     #butt, #round or #projecting"

    |s|

    (s := aStyleSymbol ? #butt) ~~ capStyle ifTrue:[
        device setCapStyle:(capStyle := s)
    ]
!

function:aFunctionSymbol
    "/ for now, this is ignored (only supporting #copy)

    "Created: 4.2.1996 / 22:17:15 / cg"
!

joinStyle:aSymbol
    "set the style in which 2 lines are connected in polygons -
     aSymbol may be #miter, #bevel, #round
    "
    |s|

    (s := aSymbol ? #miter) ~~ joinStyle ifTrue:[
        self device setJoinStyle:(joinStyle := s)
    ]
!

lineStyle:aStyleSymbol
    "Set the line style; the argument must be one of #solid or #dashed
    "
    |lineStyleSize|

    lineStyle := aStyleSymbol ? #solid.

    lineStyle == #solid ifTrue:[
        lineStyleSize := nil
    ] ifFalse:[
        lineStyleSize := 4 / self scale x
    ].

    lineStyleSize ~= psLineStyle ifTrue:[
        device setLineStyle:(psLineStyle := lineStyleSize)
    ]
!

lineWidth: width
    "Set the default width that I use to draw lines of unspecified width."

    lineWidth := width.
    self setLineWidth.
!

mask:aMask
    "CLAUS: TO BE DONE"

    aMask isNil ifTrue:[
        self lineStyle:#solid
    ] ifFalse:[
        self lineStyle:#dashed
    ]

!

pageCounter
    "returns current page number
    "
    pageCounter ~~ 0 ifTrue:[^ pageCounter].
    ^ 1
!

pageName:aString
    "set the current pages name, which is sent to the
     printer at the page-end as a %%Page: xxx comment.
     Useful when text is previewed by ghostview.
     The pageName is reset to the current pages pageNumber
     after printing."

    pageName := aString

    "Created: 7.9.1996 / 16:41:49 / cg"
!

pageOffset
    ^ pageOffset
!

pageOffset:aPoint
    pageOffset := aPoint
!

paint:aColor
    "Set the paint color"

    self establish.

    aColor ~= paint ifTrue:[
        self putCR:'% paint: ' , aColor displayString.

        super paint:aColor.

        aColor isColor ifTrue:[
            self setPaintToColor:aColor
        ] ifFalse:[
            self setPaintToCoverage:aColor
        ].
    ].
!

paint:fgColor on:bgColor
    "set the paint and backgroundPaint, used for text and bitmaps.
     Both colors may be dithered colors"

    fgColor ~~ paint ifTrue:[
        self paint:fgColor.
    ].
    bgPaint := bgColor

    "Modified: 12.5.1996 / 19:40:43 / cg"
    "Created: 4.6.1996 / 14:24:26 / cg"
!

title
    "returns title or nil
    "
    ^ title

    "Modified: 7.9.1996 / 17:05:33 / cg"
!

title:aTitle
    "should write title to stream"

    title := aTitle.

    "Created: 5.9.1996 / 18:18:51 / cg"
    "Modified: 5.9.1996 / 21:04:31 / cg"
!

titleFont
    "returns font used during endPageAction
    "
    ^ titleFont
!

titleFont:aFont
    "set font used during endPageAction
    "
    titleFont := aFont
! !

!PSGraphicsContext methodsFor:'accessing - colors'!

viewBackground
    "view-protocol compatibility"

    ^ Color white

    "Modified: 5.9.1996 / 21:34:19 / cg"
!

viewBackground:aColor
    "view-protocol compatibility"

    "Modified: 5.9.1996 / 21:34:27 / cg"
! !

!PSGraphicsContext methodsFor:'accessing - hooks'!

pageEndAction:aPageEndAction
    "set pageEndAction - if non-nil, that will be called before
     an EndPage is emmitted. The parameter to that block will be
     the current page number
    "
    pageEndAction := aPageEndAction
!

pageNumberFormat:aFormatString
    "set the pageNumber format - the default is 'page %1'"

    pageString := aFormatString.
!

pageStartAction:aPageStartActionBlock
    "set pageStartAction - if non-nil, that will be called before
     %%Page and StartPage is emmitted. The parameter to that block will be
     the current page number
    "
    pageStartAction := aPageStartActionBlock

    "Created: 7.9.1996 / 17:03:45 / cg"
!

printPageNumbers:aBoolean
    "enable/disable printing of page numbers - the default is on"

    printPageNumbers := aBoolean.
! !

!PSGraphicsContext methodsFor:'accessing dimensions'!

bottomMargin
    "return the papers bottom margin"

    ^ device bottomMargin.

    "Created: 5.9.1996 / 21:46:54 / cg"
    "Modified: 5.9.1996 / 21:46:59 / cg"
!

extent
    "return the paper extent
    "
    ^ layout extent
!

height
    "return the paper height
    "
    ^ layout height
!

leftMargin
    "return the papers left margin"

    ^ device leftMargin.

    "Created: 5.9.1996 / 21:46:50 / cg"
    "Modified: 5.9.1996 / 21:47:04 / cg"
!

origin:anOrigin corner:aCorner 
    |h w l r t b o c otrans|

    h := device height.
    w := device width.
    o := (anOrigin ? 0.0) asPoint.
    c := (aCorner  ? 1.0) asPoint.

    l := (o x isInteger) ifTrue:[o x] ifFalse:[(w * o x) truncated].
    t := (o y isInteger) ifTrue:[o y] ifFalse:[(h * o y) truncated].
    r := (c x isInteger) ifTrue:[c x] ifFalse:[(w * c x) truncated].
    b := (c y isInteger) ifTrue:[c y] ifFalse:[(h * c y) truncated].

    layout  := Rectangle left:l right:(r min:w) top:t bottom:(b min:h).
    otrans  := psTrans.
    psTrans := l @ t.

    established ifTrue:[
        self putAbsPoint:((otrans - psTrans) / psScale);
              putCommand:#translate.
    ] ifFalse:[
        lineWidth  := 1.
        capStyle   := GraphicsContext capButt.
        joinStyle  := GraphicsContext joinMiter.
        pageOffset := 0@0.
        psScale    := 1@1.

        self putCommand: #psPrologCommentExtent: withArgs: (Array with:(layout extent)).
        self putSetup.

        currentFontStyle := font.
        titleFont        := titleFont onDevice:device.
        self setFont.
        device put:'gsave\' withCRs.
    ]
!

origin:anOrigin extent:anExtent 
    |h w l r t b o c|

    (anOrigin isNil or:[anExtent isNil or:[anOrigin = 0 or:[anExtent = 1]]]) ifTrue:[
        ^ self origin:anOrigin corner:anExtent
    ].

    h := device height.
    w := device width.
    o := anOrigin asPoint.
    c := anExtent asPoint.

    l := (o x isInteger) ifTrue:[o x] ifFalse:[(w * o x) truncated].
    t := (o y isInteger) ifTrue:[o y] ifFalse:[(h * o y) truncated].
    r := (c x isInteger) ifTrue:[c x] ifFalse:[(w * c x) truncated].
    r := (r + l) min:w.
    b := (c y isInteger) ifTrue:[c y] ifFalse:[(h * c y) truncated].
    b := (b + t) min:h.

    self origin:(l @ t) corner:(r @ b).
!

rightMargin
    "return the papers right margin"

    ^ device rightMargin.

    "Created: 5.9.1996 / 21:46:50 / cg"
    "Modified: 5.9.1996 / 21:47:04 / cg"
!

topMargin
    "return the papers top margin"

    ^ device topMargin.

    "Created: 5.9.1996 / 21:46:36 / cg"
    "Modified: 5.9.1996 / 21:47:09 / cg"
!

width
    "return the width"

    ^ layout width
! !

!PSGraphicsContext methodsFor:'accessing transformations'!

clippingRectangle:aRectangle
    "Set the PostScript clipping bounds
    "
    (aRectangle isNil and:[clipRect isNil]) ifFalse:[
        self establish.
        device setClippingRectangle:(clipRect := aRectangle).
    ]
!

psScale
    "return the current overall postscript scaling.
     This includes two factors:
        the psScale, which scales from display-coordinates to PS-coordinates
        and the graphicsContexts scale, which further scales all drawings (i.e. this
        is the actual logical scale as seen by the programmer)"

    ^ self scale * psScale
!

psScale:aScaleFactor
    "set the current overall postscript scaling.
     This scale is responsible to scale from screen resolution to
     printer resolution (72x72)."

    psScale := aScaleFactor / self scale
!

scale:aScalePoint
    "scale all further drawing.
     Since postscript multiplies new scales into the current scale,
     first undo any previous scale (i.e. switch back to 1:1 display-resolution
     scaling), then scale to the given factor"

    |oldScale s t o|

    self establish.
    oldScale := self scale.

    super scale:aScalePoint.
    s := self scale asPoint.

    oldScale ~= 1 ifTrue:[
        "/ first, scale back to 1:1 psScale
        s := s * (1 / oldScale).
    ].

    self putAbsPoint:(s x asFloat) @ (s y asFloat); putCommand: #scale.

    (t := self translation asPoint) ~= 0 ifTrue:[
        o := t / oldScale.
        t := t / s.
        t := t - o.
        self putAbsPoint:(t x asFloat) @ (t y asFloat); putCommand: #translate.
    ].
    self setLineWidth.
    self lineStyle:lineStyle

!

transformX:x
    "transforms  a x-coordinate to an absolut x value dependant on the scale and
     transformation factor. This should only be used during the endPageAction is
     running.
    "
  ^ ((x * psScale x) + (device leftMargin))
!

transformation:aTransformation 
    "set the transformation"

    |scale|

    self translation:0@0.
    scale := aTransformation notNil ifTrue:[aTransformation scale]
                                   ifFalse:[1@1].

    self scale:scale.
    aTransformation notNil ifTrue:[
        self translation:aTransformation translation.
    ].
    super transformation:aTransformation.
!

translateBy:aPointOrInt
    "add to the translation offset of the transformation
     Since postscript multiplies the scale into the translation,
     but graphicContext assumes absolute translation, the given offset
     must be inverse-scaled towards postscript"

    |offsetPoint|

    offsetPoint := aPointOrInt asPoint.
    offsetPoint isZero ifTrue:[^ self].

    self translation:(self translation + offsetPoint).
!

translation:offsetPoint
    "translate all further drawing.
     Since postscript multiplies the scale into the translation,
     but graphicContext assumes absolute translation, the given offset
     must be inverse-scaled towards postscript"


    |o s oldTrans|

    self establish.
    oldTrans := self translation.
    
    o := (offsetPoint ? 0) asPoint.
    super translation:o.
    s := self scale.

    o := (o - oldTrans) / s.
    self putAbsPoint:(o x asFloat) @ (o y asFloat); putCommand: #translate.
! !

!PSGraphicsContext methodsFor:'displaying'!

displayForm:aForm x:x y:y
    "display a form or image"

    ^ self displayImage:aForm at:x@y

    "Created: 4.2.1996 / 23:45:45 / cg"
    "Modified: 1.6.1996 / 12:31:26 / cg"
!

displayImage:anImage at:aPoint
    "display the specified image, translated by aPoint."

    self displayImage:anImage at:aPoint scaledBy:1
!

displayImage:anImage at:aPoint scaledBy:imageScale
    "display the specified image, translated by aPoint."

    |devImage imgDepth|

    anImage device ~~ device ifTrue:[
        devImage := anImage onDevice:device.
    ] ifFalse:[
        devImage := anImage.
    ].
    devImage := devImage asFormOn:device.
    self establish.
    self putCommand:#gsave; putPoint:aPoint; putCommand:#translate.

    imgDepth := devImage depth.
    imgDepth == 1 ifTrue:[
        device putImage:devImage cmd:#image scaledBy:imageScale.
    ] ifFalse:[
        imgDepth == 8 ifTrue:[
            device putImage:devImage cmd:#image8 scaledBy:imageScale
        ] ifFalse:[
            imgDepth == 24 ifTrue:[
                device putImage:devImage cmd:#image24 scaledBy:imageScale
            ] ifFalse:[
                'PSGC [warning]: unsupported depth - using monochrome' errorPrintCR.
                device putImage:(devImage asMonochromeFormOn:self) cmd:#image scaledBy:imageScale
            ]
        ]
    ].

    self putCommand:#grestore.

    "Modified: 28.5.1997 / 13:35:47 / cg"
!

displayLineFrom: startPoint to: endPoint 
    "Display a line between startPoint and endPoint."

    self establish.
    self putPoint:startPoint;
         putPoint:endPoint;
         putCommand:#lineDef
!

displayMappedString: aString from: startIndex to: endIndex at: aPoint withMap: map
    "Display the substring of aString from startIndex to endIndex. Place 
     the left end of the text baseline at aPoint. Use the default font and 
     paint.
     This is an ST-80rel2.x backward compatibility method and may vanish."

    self establish.
    self putPoint: aPoint;  putCommand: #moveTo.
    device show:(aString copyFrom: startIndex to: endIndex).

    "Modified: 1.6.1996 / 12:32:15 / cg"
!

displayMask:aMask at:aPoint 
    "Display the specified mask, translated by aPoint. Use the default 
     paint.
     This is an ST-80rel2.x backward compatibility method and may vanish."

    self establish.
    self putCommand: #gsave; putPoint: aPoint; putCommand: #translate.
    device putImage: aMask asImage cmd: #mask.
    self putCommand: #grestore.

    "Modified: 1.6.1996 / 12:32:20 / cg"
!

displayOpaqueForm:aForm x:x y:y
    "display a form or image as opaque.
     Here, the image is drawn non-opaque; i.e. pixels which have been
     already drawn before will not be overwritten."

    "/ for now, display it non-opaque
    self displayImage:aForm at:x@y

    "Created: 4.2.1996 / 23:43:09 / cg"
    "Modified: 1.6.1996 / 12:33:20 / cg"
!

displayPolygon:pointCollection at:aPoint
    "Fill the polygon whose vertices are specified by pointCollection, translated
     by aPoint, with the current default paint."

    self fillPolygon:pointCollection at:aPoint
!

displayPolyline:pointCollection at:aPoint 
    "Display the polyline whose vertices are specified by pointCollection,
     translated by aPoint. Like a polygon but the start and end points are
     not connected.
    "
    self establish.
    self putPolyline:pointCollection at:aPoint.

    pointCollection last == pointCollection first ifTrue:[
        self putCommand:#closePath
    ].
    self putCommand:#stroke.
!

displayString: aString from: startIndex to: endIndex at: aPoint 
    "Display the substring of aString from startIndex to endIndex. Place 
    the left end of the text baseline at aPoint. Use the default font and 
    paint."

    self establish.
    self putPoint: aPoint;  putCommand: #moveTo.
    device show:(aString copyFrom: startIndex to: endIndex).
!

displayString:aString x:x y:y angle:drawAngle opaque:opaque
    "angles other than 0 is not yet supported"

    |angle|

    angle := drawAngle.

    angle >= 360 ifTrue:[
        angle := angle - (((angle // 360)) * 360)
    ] ifFalse:[
        angle < 0 ifTrue:[
            angle := angle - (((angle // 360)) * 360).
            angle := angle + 360.
            angle >= 360 ifTrue:[
                angle := angle - (((angle // 360)) * 360)
            ]
        ].
    ].
    angle == 0 ifTrue:[
        super displayString:aString x:x y:y angle:drawAngle opaque:false.
    ].
!

displayStringAtLastPosition: aString
    "Display aString to the right of the previous string. 
     Use the default font and paint."

    self establish.
    device show:(aString).

    "Created: 5.9.1996 / 18:18:45 / cg"
!

fillPolygon:pointCollection at:aPoint
    "Fill the polygon whose vertices are specified by pointCollection, translated
     by aPoint, with the current default paint."

    self establish.
    self putPolyline:pointCollection at:aPoint.

    self putCommand:#closePath;
         putCommand:#fill.
! !

!PSGraphicsContext methodsFor:'initialize-release'!

close
    "Send postscript epilogue, then flush and close device."

    self endPage.
    self putTrailer.

    self flush.
    device close.
    CurrentContext := nil

    "Modified: 7.9.1996 / 16:05:58 / cg"
!

disposed
    ^ self

    "Created: 4.2.1996 / 23:44:09 / cg"
!

initialize
    "initialize font sizes and page number
    "
    super initialize.

    established   := false.
    printPageNumbers := true.
    Language == #de ifTrue:[
        pageString := 'Seite %1'.
    ] ifFalse:[
        pageString := 'page %1'.
    ].
    pageCounter   := 0.
    title         := nil.
    titleFont     := Font family:'times' face:'medium' style:'roman' size:12.
    psScale := (72 @ -72) / Screen current resolution.

    "Modified: 7.9.1996 / 16:53:19 / cg"
! !

!PSGraphicsContext methodsFor:'misc - pages'!

endPage
    "end of page.
     If a pageEndAction is defined, that is invoked, with the pageCounter as
     argument.
     Otherwise, if printPageNumbers is true, the pageNumber is displayed
     at the lower-right of the page.
    "
    |string cornerY cornerX|

    self putCR:'EndPage'.
    device setFont:titleFont.

    pageEndAction notNil ifTrue:[
        pageEndAction value:pageCounter
    ] ifFalse:[
        printPageNumbers ifTrue:[
            string  := pageString bindWith:pageCounter printString.

            cornerY := (self bottomMargin + titleFont height + titleFont ascent) // 2.
            cornerX := (self width) - (titleFont widthOf:string) - (self rightMargin).

            self displayString:string x:(self transformX:cornerX) y:cornerY.

            title notNil ifTrue:[
                self displayString:title x:(self transformX:0) y:cornerY
            ]
        ]
    ].
    self putCR:'showpage'.
!

startPage
    "start a new page"

    |font pN o trans|

    pageCounter ~~ 0 ifTrue:[
        font := device currentFont.
        self endPage.
        device setFont:font.
    ].
    pageCounter := pageCounter + 1.
    pageStartAction notNil ifTrue:[pageStartAction value:pageCounter].

    pageName isNil ifTrue:[
        pN := pageCounter printString
    ] ifFalse:[
        pN := pageName
    ].
    self putCR:'%%Page: ' , pN , ' ' , pageCounter printString.
    pageName := nil.

    self putCR:'StartPage'.
    device landscape ifTrue:[
        self putCR:'Landscape'.
    ].

    pageCounter == 1 ifTrue:[
        device setFont:font.
    ].

    self putAbsPoint:((psTrans / psScale) + ((device leftMargin) @ (device height-(device topMargin)))); 
         putCommand: #translate.

    self putAbsPoint:psScale; putCommand: #scale.

    self setLineWidth.
    self lineStyle:lineStyle.

! !

!PSGraphicsContext methodsFor:'private'!

establish
    "If I am not the current context, then make myself the current 
     context, and establish any necessary state associated with me."

    | other |

    established ifTrue:[^ self].
    established := true.

    self startPage.

"/    other := CurrentContext.
"/    other == self ifTrue: [^self].
"/
"/    CurrentContext := self.
"/
"/    other isNil ifTrue: [
"/        "If the other is nil, then there was no previous context,
"/         and the  state established when I was initialized should still be valid."
"/        ^self
"/    ].
"/
"/    other font = self font ifFalse: [self setFont].
"/    other lineWidth = self lineWidth ifFalse: [self setLineWidth].
"/    other paint = self paint ifFalse: [self setPaint]

    "Modified: 7.9.1996 / 17:04:24 / cg"




!

put:aCommandString
    "put string on device"

    device put:aCommandString; space

    "Created: 4.6.1996 / 13:29:06 / cg"
    "Modified: 4.6.1996 / 13:31:04 / cg"
!

putAbsPoint:aPoint 
    "put aPoint on device as a string of two numbers ready to be placed on stack"

    |x y|

    x := aPoint x.
    y := aPoint y.
    x isInteger ifFalse:[
        x := x asFloat.
    ].
    y isInteger ifFalse:[
        y := y asFloat
    ].
    device put:x printString; space; put:y printString; space

    "Modified: 4.6.1996 / 18:09:28 / cg"
!

putCR:aCommandString
    "put string on device"

    device put:aCommandString; cr

    "Modified: 3.6.1996 / 17:50:07 / cg"
    "Created: 4.6.1996 / 13:30:58 / cg"
!

putCommand: aCommand
    "put string for aCommand (operator or procedure) on device"

    device put: (PSGraphicsContext perform: aCommand) asString; cr

    "Modified: 3.6.1996 / 17:50:07 / cg"
!

putCommand: aCommand withArgs: collectionOfArgs 
    "Put string for aCommand (operator or procedure) on device."

    device put: (self class perform: aCommand withArguments: collectionOfArgs) asString; cr

    "Modified: 3.6.1996 / 17:50:10 / cg"
!

putNumber: aNumber 
    "put aNumber on device"

    |n|

    n := aNumber.
    n isInteger ifFalse:[
        n := n asFloat
    ].
    device put:n printString; space

    "Modified: 4.6.1996 / 18:08:09 / cg"
!

putPoint:aPoint 
    "put aPoint on device as a string of two numbers ready to be placed on stack"

    self putAbsPoint:(aPoint + pageOffset)

    "Modified: 4.6.1996 / 18:07:26 / cg"
!

putPolyline:pointCollection at:aPoint

    |lnSz tab size|

    tab  := '    '.
    lnSz := 0.
    size := pointCollection size.

    device put:'['; cr; put:tab.

    pointCollection last == pointCollection first ifTrue:[
        size := size - 1
    ].
    pointCollection from:1 to:size do:[:itm|
        (lnSz := lnSz + 1) > 10 ifTrue:[
            lnSz := 1.
            device cr; put:tab.
        ].
        self putPoint:(itm translatedBy:aPoint).
    ].
    device cr; put:']'; cr.

    self putCommand:#pathDef.
!

putProlog
    |prolog|

prolog :=
'
/CBF    {/Courier-Bold } def
/CIF    {/Courier-Oblique } def
/CF     {/Courier } def
/CBIF   {/Courier-BoldOblique } def
/TBF    {/Times-Bold} def
/TIF    {/Times-Italic} def
/TRF    {/Times-Roman} def
/TBIF  {/Times-BoldItalic } def
/HBF   {/Helvetica-Bold} def
/HIF   {/Helvetica-Oblique} def
/HBIF  {/Helvetica-BoldOblique} def
/HRF   {/Helvetica} def

/reencodeISO {
dup dup findfont dup length dict begin
{ 1 index /FID ne { def }{ pop pop } ifelse } forall
/Encoding ISOLatin1Encoding def
currentdict end definefont
} def

/ISOLatin1Encoding [
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/space/exclam/quotedbl/numbersign/dollar/percent/ampersand/quoteright
/parenleft/parenright/asterisk/plus/comma/minus/period/slash
/zero/one/two/three/four/five/six/seven/eight/nine/colon/semicolon
/less/equal/greater/question/at/A/B/C/D/E/F/G/H/I/J/K/L/M/N
/O/P/Q/R/S/T/U/V/W/X/Y/Z/bracketleft/backslash/bracketright
/asciicircum/underscore/quoteleft/a/b/c/d/e/f/g/h/i/j/k/l/m
/n/o/p/q/r/s/t/u/v/w/x/y/z/braceleft/bar/braceright/asciitilde
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef
/.notdef/dotlessi/grave/acute/circumflex/tilde/macron/breve
/dotaccent/dieresis/.notdef/ring/cedilla/.notdef/hungarumlaut
/ogonek/caron/space/exclamdown/cent/sterling/currency/yen/brokenbar
/section/dieresis/copyright/ordfeminine/guillemotleft/logicalnot
/hyphen/registered/macron/degree/plusminus/twosuperior/threesuperior
/acute/mu/paragraph/periodcentered/cedilla/onesuperior/ordmasculine
/guillemotright/onequarter/onehalf/threequarters/questiondown
/Agrave/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla
/Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute/Icircumflex
/Idieresis/Eth/Ntilde/Ograve/Oacute/Ocircumflex/Otilde/Odieresis
/multiply/Oslash/Ugrave/Uacute/Ucircumflex/Udieresis/Yacute
/Thorn/germandbls/agrave/aacute/acircumflex/atilde/adieresis
/aring/ae/ccedilla/egrave/eacute/ecircumflex/edieresis/igrave
/iacute/icircumflex/idieresis/eth/ntilde/ograve/oacute/ocircumflex
/otilde/odieresis/divide/oslash/ugrave/uacute/ucircumflex/udieresis
/yacute/thorn/ydieresis
] def
[CBF CIF CF CBIF TBF TIF TRF TBIF HBF HIF HRF HBIF] {reencodeISO def} forall

/StartPage{
/svpg save def
/newShow{textShow}def
1 dup scale
}def

/EndPage{
svpg restore
/newShow{show}def
}def

/SUL {gsave currentpoint currentfont /FontInfo get /UnderlinePosition get neg
 0 exch currentfont /FontMatrix get dtransform exch pop add newpath moveto
 dup stringwidth rlineto stroke grestore newShow} def

/SSO {gsave currentpoint -300
 0 exch currentfont /FontMatrix get dtransform exch pop add 
 newpath moveto dup stringwidth rlineto stroke grestore newShow} def

'.

    self putCR:prolog.

    self putCR:'
/Landscape{90 rotate 0 -' , device height printString , ' translate}def
'.


    "Created: 7.9.1996 / 16:05:07 / cg"
    "Modified: 25.3.1997 / 16:59:56 / cg"
!

putSetup
    "Put the Postscript Setup to the device"

    psScale := (72 @ -72) / Screen current resolution.
    self putProlog.

"
    self putCR:''.
    self putCR:'/StartPage{/svpg save def 1 dup scale}def'.
    self putCR:'/EndPage{svpg restore showpage}def'.
    self putCR:''.
"
    self 
        putCommand:#defDrawEllipse;
        putCommand:#defTextHandling;
        putCommand:#defRectPath; 
        putCommand:#defRectFill; 
        putCommand:#defRectStroke;
        putCommand:#defLineDef;
        putCommand:#defPathDef.

    self putCR:'%%EndProlog'.
"/    self putStartOfPage.

    "Modified: 7.9.1996 / 16:52:37 / cg"
!

putTrailer
    self putCR:'%%Trailer'.
    self putCR:'%%Pages: ' , pageCounter printString.
    self putCR:'%%EOF'.

    "Created: 7.9.1996 / 16:05:38 / cg"
    "Modified: 7.9.1996 / 16:36:32 / cg"
!

setFont
    "Let the device know the new font style"

    device setFont: currentFontStyle

    "Modified: 3.6.1996 / 17:56:02 / cg"
!

setLineWidth
    "Since postscript multiplies the scale into the line width,
     but graphicContext assumes absolute line width, the given
     width must be inverse-scaled towards postscript
    "
    |w|

    lineWidth <= 1 ifTrue:[w := 0]
                  ifFalse:[w := (lineWidth / self scale y) asFloat abs].

    (psLineWidth isNil or:[psLineWidth ~= w]) ifTrue:[
        device setLineWidth:(psLineWidth := w)
    ]
!

setPaint
    paint installOn: self

    "Modified: 3.6.1996 / 17:56:11 / cg"
! !

!PSGraphicsContext methodsFor:'private-initialize-release'!

setPaintToColor:aColor 
    "Set the default device paint to the specified Color."

    |g|

    device supportsColor ifTrue:[
        self putNumber: aColor red / 100.0; 
             putNumber: aColor green / 100.0; 
             putNumber: aColor blue / 100.0;
             putCommand: #setRGBColor.
    ] ifFalse:[
        g := aColor brightness.
        
        device supportsGreyscale ifFalse:[
            g := (g >= 0.5 ifTrue:[1.0] ifFalse:[0.0])
        ].
        self putNumber:g; putCommand:#setgray.
    ]

    "Modified: / 15.6.1998 / 13:00:31 / cg"
!

setPaintToCoverage: aCoverageValue 
    "Set the default device paint to the specified CoverageValue."

"/    self error: 'This functionality is unimplemented'
    'PSGC: coverage paint is not implemented' errorPrintCR

    "Modified: 3.6.1996 / 18:06:24 / cg"
! !

!PSGraphicsContext methodsFor:'private-primitives'!

primDisplayArcBBoxOrigin:originPoint extent:extentPoint startAngle:startAngle sweepAngle:sweepAngle 
    "Display an elliptical arc defined by the bounding rectangle of the 
    ellipse and by the starting angle and angle of sweep for the arc."

    |rec|

    rec := Rectangle origin:originPoint extent:extentPoint.

    self establish.

    self putCR:'% primDisplayArcBBoxOrigin:' 
             , originPoint printString
             , ' extent:'
             , extentPoint printString
             , ' startAngle:'
             , startAngle printString
             , ' sweepAngle:'
             , sweepAngle printString.

    self put:'newpath';
         putNumber: startAngle; 
         putNumber: startAngle + sweepAngle; 
         putPoint: rec center; 
         putNumber: (rec width asFloat / 2); 
         putNumber: (rec height asFloat / 2);
         put:'DrawEllipse'; 
         putCR:'stroke'.

    "Modified: 4.6.1996 / 14:45:00 / cg"
!

primDisplayRectangleOrigin:originPoint extent:extentPoint 
    "Display the rectangle with the top left corner extent. Use the 
    default paint."

    | rec |

    self establish.
    rec := Rectangle origin: originPoint extent:extentPoint.

    self putCR:'% primDisplayRectangleOrigin:' 
             , originPoint printString
             , ' extent:'
             , extentPoint printString.

    self 
        putPoint: originPoint; 
        putNumber: rec width; 
        putNumber: rec height;
        putCommand: #rectFill.

    "Modified: 4.6.1996 / 14:45:04 / cg"
!

primDisplayRectangularBorderOrigin:originPoint extent:extentPoint 
    "Display the rectangular border with the specified top left corner 
     and extent. Use the default line width, cap style, join style, and paint."

    | rec |

    self establish.
    rec := Rectangle origin: originPoint extent: extentPoint.

    self putCR:'% primDisplayRectangularBorderOrigin:' 
             , originPoint printString
             , ' extent:'
             , extentPoint printString.

    self 
        putPoint: originPoint; 
        putNumber: rec width; 
        putNumber: rec height;
        putCommand: #rectStroke.

    "Modified: 4.6.1996 / 14:45:07 / cg"
!

primDisplayWedgeBBoxOrigin:originPoint extent:extentPoint startAngle:startAngle sweepAngle:sweepAngle 
    "Display an elliptical wedge defined by the bounding rectangle of the ellipse
     and by the starting angle and angle of sweep for the arc."

    |rec|

    self establish.

    rec := Rectangle origin:originPoint extent:extentPoint.

    self putCR:'% primDisplayWedgeBBoxOrigin:' 
             , originPoint printString
             , ' extent:'
             , extentPoint printString
             , ' startAngle:'
             , startAngle printString
             , ' sweepAngle:'
             , sweepAngle printString.

    self
        put:'newpath';
        putPoint: rec center; 
        put:'moveto';
        putNumber: startAngle; 
        putNumber: startAngle+sweepAngle; 
        putPoint: rec center; 
        putNumber: (rec width asFloat / 2); 
        putNumber: (rec height asFloat / 2); 
        put:'DrawEllipse'; 
        put:'closepath';
        putCR:'fill'.

    "Modified: 5.9.1996 / 18:19:01 / cg"
! !

!PSGraphicsContext methodsFor:'queries'!

documentTitle
    ^ self title
!

is3D
    ^ false

    "Created: 5.9.1996 / 18:19:03 / cg"
! !

!PSGraphicsContext methodsFor:'stx drawing'!

displayArcX:x y:y width:width height:height from:startAngle angle:angle
    "draw an arc"

    self putCR:'% displayArcX:' 
             , x printString
             , ' y:'
             , y printString
             , ' width:'
             , width printString
             , ' height:'
             , height printString
             , ' startAngle:'
             , startAngle printString
             , ' angle:'
             , angle printString.

    self primDisplayArcBBoxOrigin: x@y
                           extent: width@height
                       startAngle: startAngle 
                       sweepAngle: angle

    "Modified: 4.6.1996 / 15:49:09 / cg"
!

displayLineFromX:x1 y:y1 toX:x2 y:y2
    "Display a line between startPoint and endPoint."

    self displayLineFrom:x1@y1 to:x2@y2

    "Modified: 4.6.1996 / 14:16:01 / cg"
!

displayOpaqueString:aString x:x y:y 
    "Display the substring of aString from startIndex to endIndex. Place 
    the left end of the text baseline at x@y. Use the default font and 
    paint."

    "CLAUS: TO BE DONE"

    self displayString:aString x:x y:y

!

displayPolygon:aPolygon
    "draw a polygons outline"

    self displayPolyline:aPolygon at:(0@0)

    "Modified: 1.6.1996 / 12:31:13 / cg"
!

displayRectangle:aRectangle
    self primDisplayRectangularBorderOrigin:aRectangle origin extent:aRectangle extent 
!

displayRectangleOrigin:origin extent:extent
    self primDisplayRectangularBorderOrigin:origin extent:extent 
!

displayRectangleX:x y:y width:w height:h
    self primDisplayRectangularBorderOrigin:(x@y) extent:(w@h)
!

displayString: aString from: startIndex to: endIndex x: x y: y 
    "Display the substring of aString from startIndex to endIndex. Place 
    the left end of the text baseline at x@y. Use the default font and 
    paint."

    self displayString: aString from: startIndex to: endIndex at: x@y

!

displayString: aString x: x y: y 
    "Display the substring of aString from startIndex to endIndex. Place 
    the left end of the text baseline at x@y. Use the default font and 
    paint."

    self displayString: aString from: 1 to: aString size at: x@y

!

drawPolygon:aPolygon
    "draw a polygons outline
    "
    self displayPolygon:aPolygon
!

drawRectangle:aRectangle
    self primDisplayRectangularBorderOrigin:aRectangle origin extent:aRectangle extent 
!

drawRectangleOrigin:origin extent:extent
    self primDisplayRectangularBorderOrigin:origin extent:extent 
!

fillArcX:x y:y width:width height:height from:startAngle angle:angle
    "fill an arc"

    self primDisplayWedgeBBoxOrigin: x@y
                             extent: width@height
                         startAngle: startAngle 
                         sweepAngle: angle

    "Modified: 1.6.1996 / 12:30:23 / cg"
!

fillPolygon:aPolygon
    "fill a polygon"

    self fillPolygon:aPolygon at:(0@0)

    "Modified: 1.6.1996 / 12:30:42 / cg"
!

fillRectangleX:x y:y width:width height:height
    self primDisplayRectangleOrigin: x@y extent: width@height
!

underline:underLineFlag strikeout:strikeoutFlag
    "set underline and strikeout mode
    "
    device underline:underLineFlag strikeout:strikeoutFlag
! !

!PSGraphicsContext class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/postscript/PSGraphicsContext.st,v 1.62 2011/04/08 10:27:45 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/postscript/PSGraphicsContext.st,v 1.62 2011/04/08 10:27:45 stefan Exp $'
! !
