"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"
"{ Package: 'stx:goodies/postscript' }"

Object subclass:#PSMedium
	instanceVariableNames:'stream currentFont extent isUnderline isStrikeout
		currentFontResolution hexStrings'
	classVariableNames:'DitherImagesToMonochrome'
	poolDictionaries:''
	category:'Graphics-Postscript'
!

!PSMedium class methodsFor:'documentation'!

copyright
"
 This is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"

!

documentation
"
    This is a helper class for PSGraphicsContext, which
    generates Postscript on a stream instead of displaying on a window. 
    It is a wonderful idea, because it means that you can print anything that you
    can display.

    PSMedium is a PSGraphicsContexts device (i.e. it plays a similar role
    to a PSGraphicsContexts to what DeviceWorkstation plays to a View).
    To allow proper inheritance of GC methods (which passes many queries to its device), 
    this one implements some of those queries.

    This code has a few bugs that are described in the class comment for PSGraphicsContext.
    However, this code is useful in spite of these bugs, especially for drawing
    a single view. 

    [Instance variables:]
        stream                  <WriteStream>           where the PostScript commands go
        currentFont             <PSFont>                the current PostScript font



    Original README text:

        NAME        Postscript
        AUTHOR      Luis Tavera & Thorr Einarsson, but send e-mail to johnson@cs.uiuc.edu

        FUNCTION    A subclass of GraphicsContext that generates Postscript

        ST-VERSIONS      Objectworks 4.1
        PREREQUISITES   
        DISTRIBUTION      world
        VERSION ID           0.2
        VERSION DATE    July 23, 1993

     Notice:
        This is an adaption of a public domain class;
        Parts of it have been rewritten for ST/X, minor bugs fixed and additional
        drawing capabilities were added.
        It should be moved under the HostGraphicsDevice hierarchy, to inherit
        more functionality which is provides as abstract methods there.
        This will be done in one of the next releases.

   [author:]
        Luis Tavera & Thorr Einarsson
        Claus Gittinger [some fixes and ST/X changes]

   [see also:]
        PSGraphicsContext PSFont PostscriptPrinterStream
        DeviceWorkstation GraphicsContext
"

    ^ '$Header: /cvs/stx/stx/goodies/postscript/PSMedium.st,v 1.63 2011/07/24 06:31:35 cg Exp $'
! !

!PSMedium class methodsFor:'instance creation'!

on: aStream
	"Answer a PSMedium with output to aStream."

	^self new on: aStream
! !

!PSMedium methodsFor:'color stuff'!

bitsBlue
    "return the number of valid bits in the blue component"

    ^ 8

    "Created: 16.4.1997 / 14:48:58 / cg"
!

bitsGreen
    "return the number of valid bits in the green component"

    ^ 8

    "Created: 16.4.1997 / 14:48:53 / cg"
!

bitsRed
    "return the number of valid bits in the red component"

    ^ 8

    "Created: 16.4.1997 / 14:48:47 / cg"
!

blackpixel
    "white is all-zeros"

    ^ 0

    "Modified: 28.5.1997 / 12:06:32 / cg"
!

colorScaledRed:red scaledGreen:green scaledBlue:blue
    "return an id for a color"

    |redByte greenByte blueByte greyByte|

    redByte := red bitShift:-8.
    greenByte := green bitShift:-8.
    blueByte := blue bitShift:-8.

    self supportsColor ifTrue:[
        ^ ((redByte bitShift:16)
          bitOr:(greenByte bitShift:8))
          bitOr:blueByte
    ].

    greyByte := (redByte + greenByte + blueByte) // 3.
    ^ greyByte

    "Modified: 16.4.1997 / 14:48:12 / cg"
!

deviceColors
    "dummy: no colors are ever allocated on the printer"

    ^ #()

    "Created: 28.2.1997 / 23:18:55 / cg"
    "Modified: 28.5.1997 / 12:07:20 / cg"
!

freeColor:aColor
    "dummy: no colors are ever allocated on the printer"

    "/ nothing to be done here

    "Modified: 28.5.1997 / 12:07:17 / cg"
!

registerColor:aFont
    "dummy: no colors are ever allocated on the printer"

    "Created: 28.2.1997 / 23:19:17 / cg"
    "Modified: 28.5.1997 / 12:07:14 / cg"
!

shiftBlue
    "return the number of bits by which the blue component is to be shifted
     in a compound rgb pixel value"

    ^ 0

    "Created: 16.4.1997 / 14:50:05 / cg"
!

shiftGreen
    "return the number of bits by which the green component is to be shifted
     in a compound rgb pixel value"

    ^ 8

    "Created: 16.4.1997 / 14:51:02 / cg"
!

shiftRed
    "return the number of bits by which the red component is to be shifted
     in a compound rgb pixel value"

    ^ 16

    "Created: 16.4.1997 / 14:49:50 / cg"
!

unregisterColor:aFont
    "dummy: no colors are ever allocated on the printer"

    "Created: 28.2.1997 / 23:20:21 / cg"
    "Modified: 28.5.1997 / 12:07:32 / cg"
!

whitepixel
    "white is all-ones"

    ^ 16rFF

    "Modified: 28.5.1997 / 12:06:02 / cg"
! !

!PSMedium methodsFor:'font stuff'!

ascentOfFont:aFont
    "return the ascent in pixels of a string in a specific font

     This is a fallback solution, taking the display-font as a base
     for computation and scaling the values. This is of course inexact.
     The real solution is to parse the AFM (font metric) files here 
     .... and take the fallBack only if no AFM files are present.
    "
    |f w|

    f := aFont onDevice:Screen current.
    currentFontResolution isNil ifTrue:[
        currentFontResolution := f getFontResolution.
    ].

    w := f ascent.
    w := w * (72 / (currentFontResolution y)).
    ^ w rounded
!

descentOfFont:aFont
    "return the descent in pixels of a string in a specific font

     This is a fallback solution, taking the display-font as a base
     for computation and scaling the values. This is of course inexact.
     The real solution is to parse the AFM (font metric) files here 
     .... and take the fallBack only if no AFM files are present.
    "
    |f w|

    f := aFont onDevice:Screen current.
    currentFontResolution isNil ifTrue:[
        currentFontResolution := f getFontResolution.
    ].

    w := f descent.
    w := w * (72 / (currentFontResolution y)).
    ^ w rounded
!

deviceFonts
    ^ #()

    "Created: 28.2.1997 / 23:18:52 / cg"
!

encodingOf:aFont
    "return the characterEncoding of a font.
     Here, we assume an ISO8859 (=ANSI) encoding"

    ^ aFont encoding ? 'ISO8859-1'
!

fontMetricsOf:aFont
    "return a fonts metrics

     This is a fallback solution, taking the display-font and
     scaling the values. The real solution is to parse the
     AFM (font metric) files here .... and take the fallBack only
     if no AFM files are present.
    "

    |f display info res factorX factorY|

    display := Screen current.

    f := aFont onDevice:display.
    info := f getFontMetrics.
    res :=  f getFontResolution.

    factorX := (72 / res x).
    factorY := (72 / res y).

    info
      ascent:(info ascent * factorY) rounded
      descent:(info descent * factorY) rounded
      maxAscent:(info maxAscent * factorY) rounded
      maxDescent:(info maxDescent * factorY) rounded
      minWidth:(info minWidth * factorX) rounded
      maxWidth:(info maxWidth * factorX) rounded
      avgWidth:(info averageWidth * factorX) rounded
      minCode:nil
      maxCode:nil
      direction:nil
      encoding:info encoding.

    ^ info

    "Modified: 11.9.1996 / 22:02:15 / cg"
!

getFontWithFamily:family face:face style:style size:size sizeUnit:sizeUnit encoding:encoding
    "try to get the specified font, return font."

    ^ Font family:family face:face style:style size:size sizeUnit:sizeUnit encoding:encoding.
!

heightOfFont:aFont
    "return the height in pixels of a string in a specific font

     This is a fallback solution, taking the display-font as a base
     for computation and scaling the values. This is of course inexact.
     The real solution is to parse the AFM (font metric) files here 
     .... and take the fallBack only if no AFM files are present.
    "
    |f w|

    f := aFont onDevice:Screen current.
    currentFontResolution isNil ifTrue:[
        currentFontResolution := f getFontResolution.
    ].

    w := f height.
    w := w * (72 / (currentFontResolution y)).
    ^ w rounded
!

registerFont:aFont

    "Created: 28.2.1997 / 23:19:40 / cg"
!

releaseFont:aFont
    "nothing to do here"


!

unregisterFont:aFont

    "Created: 28.2.1997 / 23:20:17 / cg"
!

widthOfFont:aFont
    "return the width in pixels of a string in a specific font

     This is a fallback solution, taking the display-font as a base
     for computation and scaling the values. This is of course inexact.
     The real solution is to parse the AFM (font metric) files here 
     .... and take the fallBack only if no AFM files are present.
    "
    |f w|

    f := aFont onDevice:Screen current.
    currentFontResolution isNil ifTrue:[
        currentFontResolution := f getFontResolution.
    ].

    w := f width.
    w := w * (72 / (currentFontResolution x)).
    ^ w rounded
! !

!PSMedium methodsFor:'gc simulation'!

destroyGC:gcID
    gcID ~~ 0 ifTrue:[self halt]. "/ not for an image ?
    ^ self
!

gcFor:someDrawable
    ^ 0
!

gcForBitmap:someDrawable
    ^ 0
!

getBitsFrom:drawableId x:x0 y:y0 width:width height:height into:inData
    inData replaceFrom:1 to:drawableId size with:drawableId

    "Created: 30.5.1996 / 17:52:58 / cg"
!

setForeground:fgId background:bgId in:gcId
    gcId ~~ 0 ifTrue:[self halt]. "/ not for an image ?
     ^ self
!

setGraphicsExposures:aBoolean in:gcId
    gcId ~~ 0 ifTrue:[self halt]. "/ not for an image ?
     ^ self
!

setLineWidth:lineWidth style:lineStyle cap:capStyle join:joinStyle in:gcId
    gcId ~~ 0 ifTrue:[self halt]. "/ not for an image ?
    ^ self

    "Created: 5.2.1996 / 00:00:19 / cg"
! !

!PSMedium methodsFor:'images'!

createBitmapFromArray:bytes width:w height:h
     "use the data itself as id"

     ^ bytes copy

    "Modified: 5.2.1996 / 00:25:56 / cg"
!

createPixmapWidth:w height:h depth:d
     "use the data itself as id"

     |i n|

     d == 24 ifTrue:[
        n := w * h * 3
     ] ifFalse:[
        d == 8 ifTrue:[
            n := w * h
        ] ifFalse:[
            "/ should not happen
            self halt:'unsupported pixmap depth'.
            ^ nil
        ]
     ].
     i := ByteArray new:n.
     ^ i

    "Created: 4.2.1996 / 23:09:53 / cg"
    "Modified: 16.4.1997 / 15:26:21 / cg"
!

destroyPixmap:drawableId
    "/ nothing done here (garbage collector destroys the image)
!

drawBits:imageBits bitsPerPixel:bitsPerPixel depth:d padding:pad
                          width:width height:height 
                              x:x y:y
                           into:formId 
                              x:dstX y:dstY 
                          width:fwidth height:fheight 
                           with:formGCId

    d ~~ bitsPerPixel ifTrue:[
        "/ should not happen
        self halt:'unsupported operation'.
        ^ self.
    ].
    pad ~~ 8 ifTrue:[
        "/ should not happen
        self halt:'unsupported operation'.
        ^ self.
    ].

    ^ self
        drawBits:imageBits 
        depth:d 
        width:width height:height
        x:x y:y 
        into:formId x:dstX y:dstY 
        width:fwidth height:fheight with:formGCId

    "Modified: / 16.4.1997 / 15:06:25 / cg"
    "Created: / 30.1.1998 / 12:31:40 / cg"
!

drawBits:imageBits bitsPerPixel:bitsPerPixel depth:d 
                          width:width height:height 
                              x:x y:y
                           into:formId 
                              x:dstX y:dstY 
                          width:fwidth height:fheight 
                           with:formGCId

    d ~~ bitsPerPixel ifTrue:[
        "/ should not happen
        self halt:'unsupported operation'.
        ^ self.
    ].

    ^ self
        drawBits:imageBits 
        depth:d 
        width:width height:height
        x:x y:y 
        into:formId x:dstX y:dstY 
        width:fwidth height:fheight with:formGCId

    "Modified: 16.4.1997 / 15:06:25 / cg"
!

drawBits:imageBits depth:d padding:pad
                          width:width height:height 
                              x:x y:y
                           into:formId 
                              x:dstX y:dstY 
                          width:fwidth height:fheight 
                           with:formGCId

    pad ~~ 8 ifTrue:[
        "/ should not happen
        self halt:'unsupported operation'.
        ^ self.
    ].

    ^ self
        drawBits:imageBits 
        depth:d 
        width:width height:height
        x:x y:y 
        into:formId x:dstX y:dstY 
        width:fwidth height:fheight with:formGCId

    "Modified: / 16.4.1997 / 15:06:25 / cg"
    "Created: / 30.1.1998 / 12:41:22 / cg"
!

drawBits:greyBits depth:d width:width height:height
                       x:x y:y 
                    into:formId x:dstX y:dstY 
                   width:fwidth height:fheight with:formGCId

    (x ~~ 0
    or:[y ~~ 0
    or:[dstX ~~ 0
    or:[dstY ~~ 0
    or:[fwidth ~~ width
    or:[fheight ~~ height
    or:[formGCId ~~ 0]]]]]]) ifTrue:[
        "/ should not happen
        self halt:'unsupported operation'.
        ^ self
    ].
    "/ the passed formId is the image created before (its a dummy placeholder)
    formId replaceFrom:1 with:greyBits.

    "Created: 5.2.1996 / 00:19:40 / cg"
    "Modified: 16.4.1997 / 16:15:18 / cg"
! !

!PSMedium methodsFor:'initialize-release'!

close
    stream close

    "Modified: 3.6.1996 / 17:29:44 / cg"
!

on: aStream
    "Set the output stream to aStream"
    |fmt|

    stream      := aStream.
    isUnderline := false.
    isStrikeout := false.

    stream isPrinterStream ifTrue:[
        fmt := stream pageFormat
    ] ifFalse:[
        "/ mhmh - sending to a file ...
        fmt := (Printer ? PrinterStream) pageFormat
    ].

    extent := Point x:(UnitConverter convert:1 from:(fmt, 'W') to:#point)
                    y:(UnitConverter convert:1 from:(fmt, 'H') to:#point).

    "Rotate extent point if the page format is Landscape"
    Printer landscape ifTrue:[
        extent := Point x:extent y
                        y:extent x.
    ].

    "Modified: / 29.5.1998 / 17:13:33 / cg"
! !

!PSMedium methodsFor:'private'!

currentTransformation: a b: b c: c d: d tx: e ty: f 
    "Output the PostScript code to set the current graphics coordinate 
     transformation matrix."

    stream nextPut: $[.
    a asFloat printOn: stream.
    stream space.
    b asFloat printOn: stream.
    stream space.
    c asFloat printOn: stream.
    stream space.
    d asFloat printOn: stream.
    stream space.
    e asFloat printOn: stream.
    stream space.
    f asFloat printOn: stream.
    stream nextPutAll: ']'; cr

    "Modified: 3.6.1996 / 17:29:54 / cg"
!

imageWidth:width height:height bits:bits cmd:imageCmd
    "Emit the PostScript code needed to print a bitmap...  
     any needed scaling, translation, etc. is done externally."

    |byte bytesPerLine stride lastByte s x0 dI invert lineBuffer|

    hexStrings isNil ifTrue:[
        hexStrings := (16rFF to:0 by:-1) collect:[:i |
                        i < 16r10 ifTrue:[
                            '0' , i hexPrintString
                        ] ifFalse:[
                            i hexPrintString
                        ]
                      ].
    ].

"/ original:
"/    stride := width + 31 // 32 * 4. "Smalltalk image stride is 32 bits"

    invert := false.
    imageCmd == #image8 ifTrue:[
        "/ an 8-bit grayscale image

        stride := width.                    "ST/X image stride is a byte"
        bytesPerLine := width.              "PostScript file stride is a byte"
    ] ifFalse:[
        imageCmd == #image24 ifTrue:[
            "/ a 24-bit trueColor image

            stride := width * 3.            "ST/X image stride is a byte"
            bytesPerLine := width * 3.      "PostScript file stride is a byte"
            invert := true.
        ] ifFalse:[
            "/ a 1-bit monochrome image

            stride := width + 7 // 8.       "ST/X image stride is a byte"
            bytesPerLine := width + 7 // 8. "PostScript file stride is a byte"
        ]
    ].

    stream nextPutAll: '/picstr ', bytesPerLine printString, ' string def'; cr.

    width printOn: stream.
    stream space.
    height printOn: stream.

    imageCmd == #mask ifTrue: [
        stream nextPutAllUntranslated: ' false ' 
    ] ifFalse: [
        imageCmd == #image8 ifTrue: [
            stream nextPutAllUntranslated: ' 8 '
        ] ifFalse:[
            imageCmd == #image24 ifTrue:[
                stream nextPutAllUntranslated: ' 8 '
            ] ifFalse:[
                stream nextPutAllUntranslated: ' 1 '
            ]
        ]
    ].

    self
        currentTransformation: width
        b: 0
        c: 0
        d: height "negated"
        tx: 0
        ty: 0 "height".


    stream nextPutAllUntranslated: ' { currentfile picstr readhexstring pop } '.
    imageCmd == #image24 ifTrue:[
        stream nextPutAllUntranslated:'false 3 '.
        stream nextPutAllUntranslated:'colorimage'.
    ] ifFalse:[
        stream nextPutAllUntranslated:'image'.
        imageCmd == #mask ifTrue: [stream nextPutAllUntranslated: 'mask'].
    ].
    stream cr; cr.

    lastByte := 0.
    s := 'FF'.

    x0 := 1.
    lineBuffer := String new:(bytesPerLine * 2).

    0 to: height - 1 do:[:y |
        dI := 1.
        0 to: bytesPerLine-1 do: [:x |
            byte := bits at: (x0 + x).
            invert ifTrue:[
                byte := 255 - byte
            ].
            byte ~~ lastByte ifTrue:[
                s := hexStrings at:byte+1.
                lastByte := byte.
            ].
            lineBuffer replaceFrom:dI to:dI+1 with:s startingAt:1.
            dI := dI + 2.
        ].
        stream nextPutAllUntranslated:lineBuffer.
        stream cr. "/ stream space
        stream flush.
        x0 := x0 + stride.
    ].
    stream cr.

    "Modified: 28.5.1997 / 12:09:34 / cg"

!

psFontNameOf:font
    "converts font to a postscript font
    "
    |family style face|

    family := font family.
    style  := font style.
    face   := font face.

    family = 'times' ifTrue:[
        style = 'italic' ifTrue:[
            face = 'bold' ifTrue:[^ 'Times-BoldItalic']
                         ifFalse:[^ 'Times-Italic']
        ].
        face = 'bold' ifTrue:[^ 'Times-Bold']
                     ifFalse:[^ 'Times-Roman']
    ].

    family = 'courier' ifTrue:[
        style = 'oblique' ifTrue:[
            face = 'bold' ifTrue:[^ 'Courier-BoldOblique']
                         ifFalse:[^ 'Courier-Oblique']
        ].
        face = 'bold' ifTrue:[^ 'Courier-Bold']
                     ifFalse:[^ 'Courier']
    ].

    family = 'helvetica' ifTrue:[
        style = 'oblique' ifTrue:[
            face = 'bold' ifTrue:[^ 'Helvetica-BoldOblique']
                         ifFalse:[^ 'Helvetica-Oblique']
        ].
        face = 'bold' ifTrue:[^ 'Helvetica-Bold']
                     ifFalse:[^ 'Helvetica' ]
    ].

    family := family copyFrom:1.
    family at:1 put:((family at:1) asUppercase).
"/    self halt.

    ^ family


!

scale:aPoint 
    "Emit the PostScript code to scale the coordinate system...  
     'aPoint' is not really a point, but rather the x and y scale factors."

    aPoint x asFloat printOn:stream.
    stream space.
    aPoint y asFloat printOn:stream.
    stream nextPutAll:' scale'; cr

    "Modified: 4.6.1996 / 12:14:32 / cg"
! !

!PSMedium methodsFor:'queries'!

canBePaintedWith: aPaint
    "Answer whether aPaint is appropriate for me."

    ^true

    "Modified: 1.6.1996 / 11:25:51 / cg"
!

depth
    "mimicri a depth of 24 (if color) or 8 (if no color)"

    self hasColors ifTrue:[
        ^ 24  "/ simulate 24bit rgb
    ].
    ^ 8 "/ simulate 8bit greyscale

    "Modified: 28.5.1997 / 12:04:22 / cg"
!

drawingResolution
    "return a point consisting of the actual drawing resolution
     in pixel-per-inch horizontally and vertically.
     Since we dont know, assume 300dpi, which is ok for all printers"

    ^ 300 @ 300

    "Created: 4.6.1996 / 15:21:26 / cg"
!

hasColors
    "return true, if I support colors (this is aquired from the printers settings)"

    ^ self supportsColor.

"/    ^ false "/ for now simulate 8bit greyscale
"/            "/ once the color image printing is finished,
"/            "/ change this to ^ Printer supportsColors

    "Modified: 28.5.1997 / 13:40:20 / cg"
!

hasGrayscales
    "return true, if I support grayScales - always true"

    "/ returning false will force dithering of images to monochrome
    "/ which may produce better looking images on some printers ...

    DitherImagesToMonochrome == true ifTrue:[
        ^ false
    ].
    ^ true

    "
     DitherImagesToMonochrome := true
     DitherImagesToMonochrome := false
    "

    "Modified: 28.5.1997 / 13:40:36 / cg"
!

isPersistentInSnapshot
    ^ true
!

resolution
    "return a point consisting of the logical resolution
     this is a fixed 72@72, resulting from postscripts coordinate
     system, which measures in 1/72th of an inch"

    ^ 72 @ 72

    "Created: 4.6.1996 / 15:43:57 / cg"
!

supportedImageFormats
    "return an array with supported image formats; each array entry
     is another array, consisting of depth and bitsPerPixel values.
     Here, we return 1,8 and 24 bit/pixel records"

    |arr info|

    arr := OrderedCollection new:2.

    info := IdentityDictionary new.
    info at:#depth put:1.
    info at:#bitsPerPixel put:1.
    info at:#padding put:8.
    arr add:info.

    info := IdentityDictionary new.
    self hasColors ifTrue:[
        info at:#depth put:24.
        info at:#bitsPerPixel put:24.
        info at:#padding put:8.
    ] ifFalse:[
        info at:#depth put:8.
        info at:#bitsPerPixel put:8.
        info at:#padding put:8.
    ].
    arr add:info.

    ^  arr

    "Modified: / 24-07-2011 / 08:30:06 / cg"
!

supportsColor
    "Answer if the printer supports color printing"

    stream isPrinterStream ifTrue:[
        ^ stream supportsColor
    ].
    ^ (Printer ? PrinterStream) supportsColor

    "Modified: / 29.5.1998 / 17:12:57 / cg"
!

supportsGreyscale
    stream isPrinterStream ifTrue:[
        ^ stream supportsGreyscale
    ].
    ^ (Printer ? PrinterStream) supportsGreyscale
!

supportsRetainedForms
     "no, you cannot create a bitmap in the printer"

     ^ false

    "Created: 4.2.1996 / 23:13:30 / cg"
    "Modified: 28.5.1997 / 12:05:31 / cg"
!

visualType
    "mimicri a visual of TrueColor (if color) or StaticGray (if no color)"

    self supportsColor ifTrue:[
        ^ #TrueColor  "/ simulate 24bit rgb
    ].
    ^ #StaticGray     "/ simulate 8bit greyscale

    "Created: 4.2.1996 / 23:09:17 / cg"
    "Modified: 28.5.1997 / 12:05:54 / cg"
!

widthOf:aString inFont:aFont
    "return the width in pixels of a string in a specific font

     This is a fallback solution, taking the display-font as a base
     for computation and scaling the values. This is of course inexact.
     The real solution is to parse the AFM (font metric) files here 
     .... and take the fallBack only if no AFM files are present.
    "
    |f w|

    f := aFont onDevice:Screen current.
    currentFontResolution isNil ifTrue:[
        currentFontResolution := f getFontResolution.
    ].

    w := f widthOf:aString.
    w := w * (72 / (currentFontResolution x)).
    ^ w rounded

    "Modified: 16.4.1997 / 14:12:41 / cg"
! !

!PSMedium methodsFor:'queries - page'!

bottomMargin
    "Answer the bottomMargin of the postscript page (in points).
     All drawing is inset by that amount"

    stream isPrinterStream ifTrue:[
        ^ 72 * stream bottomMargin
    ].
    "/ mhmh - sending to a file ...
    ^ 72 * (Printer ? PrinterStream) bottomMargin

    "Modified: / 29.5.1998 / 17:13:38 / cg"
!

extent
    "Answer the extent of the postscript page (in points)
    "
    ^ extent
!

height
    "Answer the height of the postscript page (in points)
    "
    ^ extent y
!

landscape
    "Answer true if the postscript page is printed landscape"

    stream isPrinterStream ifTrue:[
        ^ stream landscape
    ].
    "/ mhmh - sending to a file ...
    ^ (Printer ? PrinterStream) landscape

    "Modified: / 29.5.1998 / 17:13:42 / cg"
!

leftMargin
    "Answer the leftMargin of the postscript page (in points).
     All drawing is inset by that amount
    "
    stream isPrinterStream ifTrue:[
        ^ 72 * stream leftMargin
    ].
     "/ mhmh - sending to a file ...
    ^ 72 * (Printer ? PrinterStream) leftMargin

    "Modified: / 29.5.1998 / 17:15:25 / cg"
!

rightMargin
    "Answer the rightMargin of the postscript page (in points).
     This is only used by programs, which wrap text (i.e. not hard-clipping
     is done at this boundary). The PostscriptPrinterStream ignores this (for example).
    "
    stream isPrinterStream ifTrue:[
        ^ 72 * stream rightMargin
    ].
    "/ mhmh - sending to a file ...
    ^ 72 * (Printer ? PrinterStream) rightMargin

    "Modified: / 29.5.1998 / 17:13:47 / cg"
!

topMargin
    "Answer the topMargin of the postscript page (in points).
     All drawing is inset by that amount
    "
    stream isPrinterStream ifTrue:[
        ^ 72 * stream topMargin
    ].
    "/ mhmh - sending to a file ...
    ^ 72 * (Printer ? PrinterStream) topMargin

    "Modified: / 29.5.1998 / 17:13:49 / cg"
!

width
    "Answer the width of the postscript page (in points)
    "
    ^ extent x
! !

!PSMedium methodsFor:'requests'!

cr
    "Redirect cr to the stream."

    stream cr.

    "Modified: 3.6.1996 / 17:57:59 / cg"
!

currentFont
    "returns current font
    "
    ^ currentFont
!

flush
    "Flush the stream"

    stream flush

    "Modified: 3.6.1996 / 17:57:55 / cg"
!

mapSerif:isSerif fixed:isFixed size:size italic:isItalic boldness:boldness
    "Map a Smalltalk font family onto a PostScript face, and set the type size."

    | newFamily newFace |

    boldness > 0.5 ifTrue: [
        newFace := 'Bold'
    ] ifFalse: [
        newFace := ''
    ].
    isItalic ifTrue: [newFace := newFace , 'Italic'].
    isSerif ifTrue: [
        "newFace = '' ifTrue: [newFace := #Roman]]"
        newFamily := #Times
    ] ifFalse: [
        isFixed ifTrue: [
            newFamily := #Courier
        ] ifFalse: [
            newFamily := #Helvetica
        ]
    ].
    ^Array with: (newFamily , newFace) asSymbol with: size

    "Modified: 3.6.1996 / 17:57:51 / cg"
!

put:aString
    "Put aString on the stream."

    stream nextPutAll: aString.

    "Modified: 3.6.1996 / 17:57:36 / cg"
!

putImage:anImage cmd:imageCmd 
    "Put an image on the medium"

    |bits newBits form mustInvert|

    self scale:(anImage width asFloat @ anImage height asFloat).
    bits := anImage id.

    mustInvert := false.

    anImage depth == 8 ifTrue:[
        "/ PS represents white as 0
        "/ and black as FF
        mustInvert := true.
    ] ifFalse:[
        anImage depth == 1 ifTrue:[
            "/ expect 0 to be white, 1 to be black; 
            "/ invert if this is not the case
            anImage colorMap notNil ifTrue:[
                (anImage colorMap at:1) brightness < 0.5 ifTrue:[
                    (anImage colorMap at:2) brightness > 0.5 ifTrue:[
                        mustInvert := true
                    ]
                ]
            ].
        ]
    ].                          

    mustInvert ifTrue:[
        bits := bits copy.
        bits invert.
    ].

    self imageWidth: anImage width
             height: anImage height
               bits: bits
                cmd: imageCmd

    "Modified: 28.5.1997 / 13:42:16 / cg"
!

putImage:anImage cmd:imageCmd scaledBy:imageScale
    "Put an image on the medium"

    |bits newBits form mustInvert|

    self scale:(anImage width asFloat @ anImage height asFloat).
    imageScale ~= 1 ifTrue:[
        self scale:imageScale asPoint.
    ].
    bits := anImage id.

    mustInvert := false.

    anImage depth == 8 ifTrue:[
        "/ PS represents white as 0
        "/ and black as FF
        mustInvert := true.
    ] ifFalse:[
        anImage depth == 1 ifTrue:[
            "/ expect 0 to be white, 1 to be black; 
            "/ invert if this is not the case
            anImage colorMap notNil ifTrue:[
                (anImage colorMap at:1) brightness < 0.5 ifTrue:[
                    (anImage colorMap at:2) brightness > 0.5 ifTrue:[
                        mustInvert := true
                    ]
                ]
            ].
        ]
    ].                          

    mustInvert ifTrue:[
        bits := bits copy.
        bits invert.
    ].

    self 
        imageWidth:(anImage width) height:(anImage height)
        bits:bits cmd:imageCmd

    "Modified: 28.5.1997 / 13:42:16 / cg"
!

setCapStyle:aSymbol
    "Set line style for drawing
    "
    |n|

    aSymbol == #round ifTrue:[n := '1']
                     ifFalse:[n := (aSymbol == #projecting) ifTrue:['2'] ifFalse:['0']].

    stream nextPutAll:n, ' setlinecap'; cr.
!

setClippingRectangle:aRectangle
    "Make a rectangular path at the current origin, and use it as the 
    current clip path
    "
    |origin|

    stream nextPutAll:'initclip'; cr.   "/ remove clipping path

    aRectangle isNil ifTrue:[           "/ no more clipping
        ^ self
    ].
    stream nextPutAll:'newpath '.

    aRectangle left asFloat printOn: stream.
    stream space.
    aRectangle top asFloat printOn: stream.
    stream nextPutAll: ' moveto 0 '.
    aRectangle height asFloat printOn: stream.
    stream nextPutAll: ' rlineto '.
    aRectangle width asFloat printOn: stream.
    stream nextPutAll: ' 0 rlineto 0 '.
    aRectangle height negated asFloat printOn: stream.
    stream nextPutAll: ' rlineto closepath clip newpath'; cr
!

setFont:aFontStyle 
    "Let the medium know of a new font style. If fontStyle is nil, the
     current font is set
    "
    |height family|

    aFontStyle notNil ifTrue:[
        currentFont ~= aFontStyle ifTrue:[
            currentFont := aFontStyle.
            currentFontResolution := nil
        ]
    ] ifFalse:[
        currentFont isNil ifTrue:[ ^ self ]
    ].

    family := '/', (self psFontNameOf:currentFont).
    height := currentFont size.
    stream nextPutAll:(height printString, ' ', family, ' F'); cr

    "Modified: 11.9.1996 / 22:00:23 / cg"
!

setJoinStyle:aSymbol
    "Set line join for drawing
    "
    |n|

    aSymbol == #round ifTrue:[n := '1']
                     ifFalse:[n := (aSymbol == #bevel) ifTrue:['2'] ifFalse:['0']].

    stream nextPutAll:n, ' setlinejoin'; cr.
!

setLineStyle:aNumberOrNil
    "Set line style for drawing"

    |n|

    n := aNumberOrNil isNil ifTrue:['']
                           ifFalse:[aNumberOrNil abs printString].

    stream nextPutAll: '[', n, '] 0 setdash'; cr

!

setLineWidth:aNumber
    "Set line width for drawing"

    aNumber printOn:stream.
    stream nextPutAll: ' setlinewidth'; cr.

    "Modified: 3.6.1996 / 18:02:29 / cg"
!

show:string
    "Emit the PostScript code to print a string of characters.
    "
    | index length c code |

    stream nextPut: $(.
    index := 0.
    length := string size.
    [(index := index + 1) <= length] whileTrue:[ 
        c := string at: index.
        c == $( ifTrue: [
            stream nextPutAll: '\050'
        ] ifFalse: [
            c == $) ifTrue: [
                stream nextPutAll: '\051'
            ] ifFalse: [
                c == $\ ifTrue: [
                    stream nextPutAll: '\\'
                ] ifFalse: [
                    code := c asInteger.
                    code > 127 ifTrue:[
                        stream nextPut:$\.
                        stream nextPutAll:(code printStringRadix:8)
                    ] ifFalse: [
                        stream nextPut: c
                    ]
                ]
            ]
        ]
    ].

    isUnderline ifTrue:[
        stream nextPutAll: ')SUL'
    ] ifFalse:[
        isStrikeout ifTrue:[
            stream nextPutAll: ')SSO'
        ] ifFalse:[
            stream nextPutAll: ')newShow'
        ]
    ].
    stream cr.

    "Modified: 6.8.1997 / 15:01:19 / cg"
!

space
    "Redirect space to the stream."

    stream space.

    "Modified: 3.6.1996 / 17:58:36 / cg"
!

translate:aPoint 
    "Translate PostScript coordinate system by aPoint"

    aPoint x printOn: stream.
    stream space.
    aPoint y printOn: stream.
    stream nextPutAll: ' translate'; cr

    "Modified: 3.6.1996 / 17:58:42 / cg"
!

underline:underLineFlag strikeout:strikeoutFlag
    "set underline and strikeout mode
    "
    isStrikeout := strikeoutFlag.
    isUnderline := underLineFlag.
! !

!PSMedium class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/postscript/PSMedium.st,v 1.63 2011/07/24 06:31:35 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/postscript/PSMedium.st,v 1.63 2011/07/24 06:31:35 cg Exp $'
! !
