"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologClass subclass:#Diff2
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog-Examples'
!


!Diff2 methodsFor:'prolog-clauses'!

%******************************************************
%  clauses for differentiation & simplification
%******************************************************

diff(Y, X, DY)  :- d(Y, X, Z),
                   s(Z, DY).


%"

  diff ( pow(x,2) , x , D)      
  diff ( +(*(-5, pow(x,2)), *(3,x)) , x , D)      
  diff ( sin(x) , x , D)      
  diff ( sin(*(5,x)) , x , D)      
  diff ( sin(*(5,x)) , x , D) , infix(D, DI)      

  infix( sin(*(5,x)) , F)   

  trace, s( *(1,cos(x)) , S)    
  s( *(1,x) , S)  
  s( cos(x) , S)    
  s ( sin(x) , D)       

"%
!

tdiff(Y, X, DY)  :- cputime(Start),
                    d(Y, X, Z),
                    s(Z, DY),
                    cputime(End),
                    is(T, -(End,Start)),
                    write('execution time: '),
                    write(T),
                    writeln('ms').

%"

  diff ( pow(x,2) , x , D)      
  tdiff ( pow(x,2) , x , D)      

"%


! !

!Diff2 methodsFor:'prolog-differentiation'!

%******************************************************
%  clauses for differentiation
%******************************************************
d(X,X,1).

d(C,X,0)                :- atomic(C), \=(C,X).

d(-(U),X,-(D))          :- d(U,X,D).

d(-(U,V),X,-(D1,D2))    :- d(U,X,D1), d(V,X,D2).

  d(+(C,U),X,D)         :- atomic(C), \=(C,X), d(U,X,D), !! .
  d(+(U,C),X,D)         :- atomic(C), \=(C,X), d(U,X,D), !! .

d(+(U,V),X,+(D1,D2))    :- d(U,X,D1), d(V,X,D2).

  d(-(C,U),X,-(D))      :- atomic(C), \=(C,X), d(U,X,D), !! .
  d(-(U,C),X, D)        :- atomic(C), \=(C,X), d(U,X,D), !! .

d(-(U,V),X,-(D1,D2))    :- d(U,X,D1), d(V,X,D2).

  d(*(C,U),X,*(C,D))    :- atomic(C), \=(C,X), d(U,X,D), !! .
  d(*(U,C),X,*(C,D))    :- atomic(C), \=(C,X), d(U,X,D), !! .


d(*(U,V), X, +(*(D2,U) , *(D1,V))) :- d(U,X,D1), d(V,X,D2).

  d(/(1,U),X, -(/(D,pow(U,2))) ) :- d(U,X,D), !! .
  d(/(C,U),X, *(C,DD))           :- atomic(C), \=(C,X), d(/(1,U),X,DD), !! .
  d(/(U,C),X, /(D,C))            :- atomic(C), \=(C,X), d(U,X,D), !! .

d(/(U,V), X, /(-(*(V,D1),*(U,D2)),pow(V,2)) ) :- d(U,X,D1), d(V,X,D2).

  d(pow(U,C), X, *(*(C,D), pow(U,-(C,1))) ) :- atomic(C), \=(C,X), d(U,X,D), !! .
%  d(pow(U,C), X, *(*(C,D), pow(U,-(C,1))) ) :- =(C, -(C1)), atomic(C1), \=(C1,X), d(U,X,D), !! .
  d(pow(U,-(C)), X, Z)                      :-  d( /(1,pow(U,C)), X, Z), !! .
  d(pow(U,/(A,B)), X, *(*(/(A,B),D), pow(U,/(A,-(B,1)))) ) 
                                            :- atomic(A), atomic(B),
                                               \=(A,X), \=(B,X),
                                               d(U,X,D), !! .

  d(pow(U,-(/(A,B))),X, Z)         :- d(/(1,pow(U,/(A,B))) , X, Z), !! .

d(pow(U,V), X, *(pow(U,V),+(/(*(V,D1),U),*(D2,ln(U)))) ) :- d(U,X,D1), d(V,X,D2).


d( sin(W), X, *(Z, cos(W)) ) :- d( W, X, Z).

d( exp(W), X, *(Z, exp(W)) ) :- d( W, X, Z ).

d( log(W), X, /(Z,W) ):- d( W, X, Z).

d( cos(W), X, -(*(Z,sin(W))) ):- d( W, X, Z).

d (ln(U) , X , *(/(1,U),U1)) :- d(U,X,U1).

d (tan(U), X, *( pot(sec(U),2),U1)) :- d(U,X,U1).

%"
    d(x,x,DY)           
    d(pow(x, 2),x,DY)    
    d(*(x, x),x,DY)      

    d(I,x,+(*(1,x),*(1,x))) 
    d(I,x,+(*(1,x),*(1,x))) , s(I, I2)

    d(sin(x),x,DY)            
    self new query:'trace , d(sin(x),x,DY)'        
"%



! !

!Diff2 methodsFor:'prolog-parsing'!

parse(L, Expr, Rest) :- parseExpr(L, Expr, Rest).


%"

parse( [ 5 , a , b , c] , E , R )      
parse( [ 5 , '+' , 10] , E , [] )      
parse( [ 5 , '*' , 10] , E , [] )      
parse( [ 5 , '+' , 7 , '*' , 10] , E , [] )      
parse( [ 5 , '*' , 7 , '+' , 10] , E , [] )      

is(In, [ x , '*' , x , '+' , x , '*' , 10]),
    parse( In, E, [] ), diff(E, x, D), infix(D, Out)  

"%



!

parseAddExpr(L, Op(A, B), Rest) :- parseMulExpr(L, A, R1),
                               parseAddOp(R1, Op, R2),
                               parseAddExpr(R2, B, Rest), !!.

parseAddExpr(L, Expr, Rest) :- parseMulExpr(L, Expr, Rest).

%"

parseAddExpr( [ 5 ] , P , R )      
parseAddExpr( [ 5 , '*' , x] , P , R )      
parseAddExpr( [ 5 , '*' , x , '*' , a] , P , R )      
parseAddExpr( [ 5 , '+' , x , '*' , a] , P , R )      

"%



!

parseAddOp([Op|Rest], Op, Rest) :- =(Op, '+').
parseAddOp([Op|Rest], Op, Rest) :- =(Op, '-').



!

parseBinOp([Op|Rest], Op, Rest) :- =(Op, '+').
parseBinOp([Op|Rest], Op, Rest) :- =(Op, '*').

%"

parse( [ 5 , a , b , c] , P , R )      


"%



!

parseExpr(L, Expr, Rest) :- parseAddExpr(L, Expr, Rest).


%"

parse( [ 5 , a , b , c] , P , R )      
parse( [ 5 , '+' , 10] , P , [] )      


"%



!

parseMulExpr(L, Op(A, B), Rest) :- parsePrimary(L, A, R1),
                                   parseMulOp(R1, Op, R2),
                                   parseMulExpr(R2, B, Rest), !!.

parseMulExpr(L, Expr, Rest) :- parsePrimary(L, Expr, Rest).

%"

parseMulExpr( [ 5 ] , P , R )      
parseMulExpr( [ 5 , '*' , x] , P , R )      
parseMulExpr( [ 5 , '*' , x , '*' , a] , P , R )      

"%



!

parseMulOp([Op|Rest], Op, Rest) :- =(Op, '*').
parseMulOp([Op|Rest], Op, Rest) :- =(Op, '/').




!

parsePrimary(['(' | Rest1], Expr, Rest) :- parseExpr(Rest1, Expr, Rest2),
                                           is(Rest2, [')' | Rest]), !!.

parsePrimary([Primary|Rest], Primary, Rest) :- atomic(Primary).


%"

parsePrimary( [ 5 , a , b , c] , E , R )      
trace, parsePrimary( [ '(' , 5 , '+' , 10 , ')'] , E , R )      

"%
! !

!Diff2 methodsFor:'prolog-printing'!

infix(X,Y) :- atomic(X), is(Y, [ X ]), !! .

%"
 infix(5, W) 
 infix(x, W) 
"%

infix(X,Y) :- =..(X, [Op,A]),
                 infix(Op, A, Y),
                 !!.

infix(X,Y) :- =..(X, [Op,A,B]), 
                 infix(Op, A, B, Y),
                 !!.

infix('-', A, W) :- atomic(A),
                 is(['-' , A ], W),
                 !!.

%"
 infix(-(5), W)        
 infix(-(x), W)        
"%


infix('-', Op(A), W) :- infix(Op, A, A1),
                        append(['-'] , A1, W),
                        !!.

%"
 infix(-(-(5)), W)        
 infix(-(sin(x)), W)        
 infix('-' , sin(x), W)        
"%

infix(Op, A, W) :- atomic(A),
                 is([Op , '(' , A , ')' ], W),
                 !!.

infix(Op, A, W) :-
                 infix(A,A1),
                 append([ Op , '(' ], A1, T),
                 append(T, [ ')' ], W),
                 !!.

%"
 infix( sin(x), W)        
 infix( cos(sin(x)), W)        
"%

infix(Op, A, B, W) :-
                 atomicOrUnary(A), atomicOrUnary(B),
                 infix(A, A1),
                 infix(B, B1),
                 append(A1, [ Op ], T1),
                 append(T1, B1, W),
                 !!.

%"
 infix( +(x, x), W)        
 infix( +(x, 5), W)        
 infix( *(x, 5), W)        
 infix( *(-(x), 5), W)        
"%

infix(Op, A, B, W) :-
                 atomicOrUnary(A),
                 infix(A,A1),
                 infix(B,B1),
                 append(A1, [ Op , '('], T4),
                 append(T4, B1, T5),
                 append(T5, [ ')' ], W),
                 !!.

%"
 infix( *(x, -(x) ), W)        
 infix( *(x, sin(x) ), W)        
"%

infix(Op, A, B, W) :-
                 atomicOrUnary(B),
                 infix(A,A1),
                 infix(B,B1),
                 append([ '(' ], A1 , T1),
                 append(T1 , [ ')', Op ], T2),
                 append(T2, B1, W),
                 !!.

%"
 infix( *( -(x) , x ), W)        
 infix( *(sin(x) ,x  ), W)        
"%


infix(Op, A, B, W) :-
                 infix(A,A1),
                 infix(B,B1),
                 append([ '(' ], A1 , T1),
                 append(T1 , [ ')' ], T2),
                 append(T2, [ Op ], T3),
                 append(T3 , [ '(' ], T4),
                 append(T4, B1, T5),
                 append(T5, [ ')' ], W),
                 !!.
%"

infix( sin(*(-(5),x)) , F)      
infix( sin(*(5,x)) , F)      
infix( +(5,x) , F)        
infix( +(5,+(10,x)) , F)  
trace, infix( +(5,+(10,x)) , F)  
infix( *(5,x) , F)           
infix( +(*(5,x), *(3,y)) , F)    
infix( -(sin(x)) , F)    

"%


! !

!Diff2 methodsFor:'prolog-queries'!

atomicOrUnary(A) :- atomic(A).
atomicOrUnary(A) :- unary(A).

!

unary(Op(A)).


! !

!Diff2 methodsFor:'prolog-reducing'!

%******************************************************
%  basic arithmetic rules 
%******************************************************

% addition

b('+', X, 0,    X)    :- !! .           %  X + 0   -> X
b('+',-(X), 0 , -(X)) :- !! .           %  -X + 0  -> X
b('+',0, X,     X)    :- !! .           %  0 + X   -> X
b('+',0,-(X),   -(X)) :- !! .           %  0 + -X  -> X
b('+',X,-(X),   0)    :- !! .           %  X + -X  -> 0
b('+',-(X), X,  0)    :- !! .           %  -X + X  -> 0

b('+',X, Y,     Z)    :- number(X),    %  C1 + C2 -> C   compute
                         number(Y),
                         is(Z, +(X,Y)), 
                         !! .

% catch all clause to cover cases not covered
% by the previous clauses      

b('+',X, Y ,   +(X,Y) ). 


% subtraction

b('-', X, 0, X)     :- !! .              % X - 0    -> X
b('-',-(X), 0,-(X)) :- !! .              % -X - 0   -> -X
b('-',0, X,-(X))    :- !! .              % 0 - X    -> -X
b('-',0,-(X), X)    :- !! .              % 0 - -X   -> X
b('-',-(X),-(X), 0) :- !! .              % -X - -X  -> 0
b('-',X, X, 0)      :- !! .              % X - X    -> 0

b('-', X,Y,Z)       :- number(X), 
                       number(Y),
                       is(Z, -(X,Y)), 
                       !! . 

% catch all clause to cover cases not covered
% by the previous clauses      

b('-' , X,Y,  -(X,Y)). 

% multiplication


b('*', 0, X, 0)      :- !! .
b('*', 0,-(X), 0)    :- !! .
b('*', X, 0, 0)      :- !! .
b('*', -(X), 0,0)    :- !! .

b('*', 1, X, X)     :- !! .
b('*', 1,-(X),-(X)) :- !! .
b('*', -(1), X,-(X)) :- !! .
b('*', -(1),-(X), X) :- !! .
b('*', X, 1, X)     :- !! .
b('*', -(X), 1,-(X)) :- !! .
b('*', X,-(1),-(X)) :- !! .
b('*', -(X),-(1), X) :- !! .

b('*', /(X,Y), Y, X)   :- !! .
b('*',  Y, /(X,Y), X) :- !! .

b('*',X,Y,Z)           :- number(X), number(Y),
                          is(Z, *(X,Y)), !! . 

% catch all clause to cover cases not covered
% by the previous clauses      
b('*', X, Y, *(X,Y)). 

% division 


b('/',0,0, 'ERROR - indefinite form 0/0') :- !! .   % /* indefinite form */
b('/',X,0, 'ERROR - division by 0      ') :- !! .   % /* division by 0   */

b('/',0, X,0) :- !! .
b('/',0,-(X),0) :- !! .

b('/', X,1, X) :- !! .
b('/',-(X),1,-(X)) :- !! .

b('/', 1,X,    /(1,X)) :- !! .
b('/',-(1),X, -(/(1,X))) :- !! .

b('/', X,-(1),-(X)) :- !! .
b('/',-(X),-(1), X) :- !! .

b('/', 1,  -(X),-(/(1,X))) :- !! .
b('/',-(1),  -(X),   /(1,X)) :- !! .
b('/', 1, /(1,X),     X) :- !! .
b('/',-(1), /(1,X),    -(X)) :- !! .

b('/', X,-(X),-(1)) :- !! .
b('/',-(X), X,-(1)) :- !! .
b('/',-(X),-(X), 1) :- !! .
b('/', X, X, 1)     :- !! .

% catch all clause to cover cases not covered
% by the previous clauses      
b('/',X,Y,/(X,Y)). 

% exponentiation 


b('pow',1,X,1) :- !! .

b('pow',0, 0,   'ERROR - indefinite form 0^0       ') :- !! .
b('pow',0,-(1), 'ERROR - indefinite form 0^-1 = 1/0') :- !! .
b('pow',0,-(K), 'ERROR - indefinite form 0^-K = 1/0') :- atomic(K), !! .

b('pow',-(X),1,-(X)) :- !! .
b('pow',X,1, X)      :- !! .
b('pow',X,0, 1)      :- !! .


% recursive clauses to
% calculate the n-th power
% of positive and negative
% integers   

b('pow',X,N,Y) :- integer(X), integer(N),
                  is(M, -(N,1)),
                  b(pow(X,M),Z),
                  is(Y, *(Z,X)), !! .

b('pow',-(X),N,Y) :- integer(X), integer(N),
                     is(R, mod(N, 2)),
                     \=(R, 0),
                     b(pow(X,N),Z), 
                     =(Y, -(Z)), !! .

b('pow',-(X),N,Y) :- integer(X), integer(Y),
                     is(R, mod(N, 2)), 
                     =(R, 0),
                     b(pow(X,N),Z), 
                     =(Y,Z), !! .

b('pow',-(X),Y, -(pow(X,Y))) :- !! .

% catch all clause to cover cases not covered
% by the previous clauses      
b('pow',X,Y, pow(X,Y)). 


% catch all clause to cover cases not covered
% by the previous clauses      
b(OP,X, Y ,   OP(X,Y) ). 

%"

   b( +(0, 5), R)    
   b( -(0, 5), R)    

   b( +(exp(x,2), -(exp(x,2)) ), R)    
"%


!

%******************************************************
%  clauses for reduction
%******************************************************

% addition 

r('+',-(X),-(X), Z) :- b('*',2, X, W) , u('-', W,Z), !! .
r('+',X,X,       Z) :- b('*',2,X, Z), !! .
r('+',X,-(Y),    Z) :- b('-',X,Y ,Z), !! .
r('+',-(X),Y,    Z) :- b('-',Y,X ,Z), !! .
r('+',-(X),-(Y), Z) :- b('+',X,Y ,W), u('-', W,Z), !! .

r('+',X, /(Y,Z), W) :- integer(X), integer(Y), integer(Z),
                       is(T, +(*(Z,X),Y)),
                       b('/',T,Z, W), !! .

r('+',/(X,Z), Y, W) :- integer(X), integer(Y), integer(Z),
                       is(T, +(X,*(Y,Z))),
                       b('/',T,Z,W), !! .

r('+',X, +(Y,Z), W) :- b('+',Y,Z,T), b('+',X,T,W), !! .
r('+',+(X,Y), Z, W) :- b('+',X,Y,T), b('+',T,Z,W), !! .

r('+',*(X,Y),*(Z,Y),W) :- b('+',X,Z,T), b('*',Y,T,W), !! .
r('+',*(X,Y),*(Y,Z),W) :- b('+',X,Z,T), b('*',Y,T,W), !! .
r('+',*(Y,X),*(Z,Y),W) :- b('+',X,Z,T), b('*',Y,T,W), !! .
r('+',*(Y,X),*(Y,Z),W) :- b('+',X,Z,T), b('*',Y,T,W), !! .

r('+',X,Y, Z)          :- integer(Y), b('+',Y,X,Z), !! .


% subtraction


r('-',X,-(X),Z)    :- b('*',2,X,Z), !! .
r('-',-(X), X,Z)   :- b('*',2,X,W), u('-', W,Z), !! .

r('-',X,-(Y),Z)    :- b('+',X,Y,Z), !! .
r('-',-(X), Y,Z)   :- b('+',X,Y,W), u('-', W,Z), !! .
r('-',-(X),-(Y),Z) :- b('-',Y,X,Z), !! .

r('-',X, /(Y,Z), W) :- integer(X), integer(Y), integer(Z),
                       is(T, -(*(X,Z),Y)),
                       b('/',T,Z,W), !! .

r('-',/(X,Z), Y, W) :- integer(X), integer(Y), integer(Z),
                       is(T, -(X,*(Y,Z))),
                       b('/',T,Z,W), !! .

r('-',X, -(Y,Z), W) :- b('-',Y,Z,T), b('-',X,T,W), !! .
r('-',-(X,Y), Z, W) :- b('-',X,Y,T), b('-',T,Z,W), !! .

r('-',*(X,Y), *(Z,Y), W) :- b('-',X,Z,T), b('*',Y,T,W), !! .
r('-',*(X,Y), *(Y,Z), W) :- b('-',X,Z,T), b('*',Y,T,W), !! .
r('-',*(Y,X), *(Z,Y), W) :- b('-',X,Z,T), b('*',Y,T,W), !! .
r('-',*(Y,X), *(Y,Z), W) :- b('-',X,Z,T), b('*',Y,T,W), !! .


% multiplication  


r('*',X, X,Z)      :- b('pow',X,2,Z), !! .
r('*',-(X),-(X),Z) :- b('pow',X,2,Z), !! .
r('*',-(X), X,Z)   :- b('pow',X,2,W), u('-', W,Z), !! .
r('*', X,-(X),Z)   :- b('pow',X,2,W), u('-', W,Z), !! .

r('*',X, pow(X,-(1)), Z) :- b('/',X,X,Z), !! .
r('*',pow(X,-(1)), X, Z) :- b('/',X,X,Z), !! .

r('*',   X, /(1,X), Z) :- b('/',X,X,Z), !! .
r('*', /(1,X),   X, Z) :- b('/',X,X,Z), !! .
r('*',   X, /(1,Y), Z) :- b('/',X,Y,Z), !! .
r('*', /(1,X),   Y, Z) :- b('/',Y,X,Z), !! .

r('*',   M, /(N,X), Z) :- atomic(M), atomic(N),
                          b('*',M,N,S), b('/',S,X,Z), !! .
r('*', /(M,X),   N, Z) :- atomic(M), atomic(N),
                          b('*',M,N,S), b('/',S,X,Z), !! .

r('*',  X, /(N,Y), Z) :- atomic(N), b('/',X,Y,S), b('*',N,S,Z), !! .
r('*',/(N,Y),   X, Z) :- atomic(N), b('/',X,Y,S), b('*',N,S,Z), !! .

r('*',     X,pow(Y,-(1)),Z) :- b('/',X,Y,Z), !! .
r('*',     X,pow(X,-(Y)),Z) :- b('-',Y,1,S), b('pow',X,S,T), b('/',1,T,Z), !! .
r('*',pow(X,-(1)),     Y,Z) :- b('/',Y,X,Z), !! .
r('*',pow(X,-(Y)),     X,Z) :- b('-',Y,1,S), b('pow',X,S,T), b('/',1,T,Z), !! .

r('*', X,pow(X,Y),Z)  :- b('+',1,Y,S), b('pow',X,S,Z), !! .
r('*', pow(X,Y), X,Z) :- b('+',Y,1,S), b('pow',X,S,Z), !! .

r('*',-(X),-(Y),Z) :- b('*',X,Y,Z), !! .
r('*', X,-(Y),Z)   :- b('*',X,Y,W), u('-', W,Z), !! .
r('*',-(X), Y,Z)   :- b('*',X,Y,W), u('-', W,Z), !! .

r('*',pow(Z,-(X)),pow(Z,-(Y)),W) :- b('+',X,Y,S), b('pow',Z,S,T), b('/',1,T,W), !! .
r('*',pow(Z,-(X)),pow(Z,Y),W)    :- b('-',Y,X,S), b('pow',Z,S,W), !! .
r('*',pow(Z,X),pow(Z,-(Y)),W)    :- b('-',X,Y,S), b('pow',Z,S,W), !! .
r('*',pow(Z,X), pow(Z,Y),W)      :- b('+',X,Y,T), b('pow',Z,T,W), !! .
r('*',pow(X,-(Z)),pow(Y,-(Z)),W) :- b('*',X,Y,S), b('pow',S,Z,T), b('/',1,T,W), !! .
r('*',pow(X,Z),pow(Y,-(Z)),W)    :- b('/',X,Y,S), b('pow',S,Z,W), !! .
r('*',pow(X,-(Z)), pow(Y,(Z)),W) :- b('/',Y,X,S), b('pow',S,Z,W), !! .
r('*',pow(X,Z), pow(Y,Z),W)      :- b('*',X,Y,T), b('pow',T,Z,W), !! .

r('*', *(X,Y),   Y, Z) :- b('pow',Y,2,S), b('*',X,S,Z), !! .
r('*', *(Y,X),   Y, Z) :- b('pow',Y,2,S), b('*',X,S,Z), !! .
r('*',   Y, *(X,Y), Z) :- b('pow',Y,2,S), b('*',X,S,Z), !! .
r('*',   Y, *(Y,X), Z) :- b('pow',Y,2,S), b('*',X,S,Z), !! .

r('*', *(X,Y), *(X,Z), W) :- b('*',Y,Z,S), b('pow',X,2,T), b('*',T,S,W), !! .
r('*', *(Y,X), *(X,Z), W) :- b('*',Y,Z,S), b('pow',X,2,T), b('*',T,S,W), !! .
r('*', *(X,Y), *(Z,X), W) :- b('*',Y,Z,S), b('pow',X,2,T), b('*',T,S,W), !! .
r('*', *(Y,X), *(Z,X), W) :- b('*',Y,Z,S), b('pow',X,2,T), b('*',T,S,W), !! .

r('*',  M, *(N,X), W) :- atomic(M), atomic(N),
                         b('*',M,N,P), b('*',P,X,W), !! .
r('*',*(M,X),   N, W) :- atomic(M), atomic(N),
                         b('*',M,N,P), b('*',P,X,W), !! .

r('*',   X, *(Y,Z), W) :- b('*',Y,Z,T), b('*',X,T,W), !! .
r('*', *(X,Y),   Z, W) :- b('*',X,Y,T), b('*',T,Z,W), !! .

r('*',X,Y,Z) :- integer(Y), b('*',Y,X,Z), !! .

% division
% (division is never actually performed)


r('/',1, /(X,Y), Z)    :- b('/',Y,X,Z), !! .
r('/',-(1), /(X,Y), Z) :- b('/',Y,X,W), u('-', W,Z), !! .

r('/',-(X),-(Y),Z) :- b('/',X,Y,Z), !! .
r('/', X,-(Y),Z)   :- b('/',X,Y,W), u('-', W,Z), !! .
r('/',-(X), Y,Z)   :- b('/',X,Y,W), u('-', W,Z), !! .

r('/',      X, pow(Y,-(1)), Z) :- b('*',X,Y,Z), !! .
r('/', pow(X,-(1)),      Y, Z) :- b('*',X,Y,W), b('/',1,W,Z), !! .

r('/',   X, /(Y,Z), W) :- b('*',X,Z,T), b('/',T,Y,W), !! .
r('/', /(X,Y),   Z, W) :- b('*',Y,Z,T), b('/',X,T,W), !! .

r('/',     X,pow(Y,-(Z)),W) :- b('pow',Y,Z,T), b('*',X,T,W), !! .
r('/',pow(X,-(Z)),     Y,W) :- b('pow',X,Z,S), b('*',S,Y,T), b('/',1,T,W), !! .

r('/',X,pow(X,-(Y)),Z) :- b('+',1,Y,S), b('pow',X,S,Z), !! .
r('/',X,   pow(X,Y),Z) :- b('-',Y,1,S), b('pow',X,S,T), b('/',1,T,Z), !! .
r('/',pow(X,-(Y)),X,Z) :- b('+',Y,1,S), b('pow',X,S,T), b('/',1,T,Z), !! .

r('/',pow(X,Y),     X,Z)         :- b('-',Y,1,S), b('pow',X,S,Z), !! .
r('/',pow(X,N), pow(X,-(M)),Z)   :- b('+',N,M,S), b('pow',X,S,Z), !! .
r('/',pow(X,-(N)),pow(X,M),Z)    :- b('+',N,M,S), b('pow',X,S,T), b('/',1,T,Z), !! .
r('/',pow(X,-(N)),pow(X,-(M)),Z) :- b('-',M,N,S), b('pow',X,S,Z), !! .
r('/',pow(X,N),   pow(X,M),Z)    :- b('-',N,M,W), b('pow',X,W,Z), !! .

r('/',pow(X,-(Z)),pow(Y,Z),W)     :- b('*',X,Y,S), b('pow',S,Z,T), b('/',1,T,W), !! .
r('/',pow(X,Z),pow(Y,-(Z)),W)     :- b('*',X,Y,S), b('pow',S,Z,W), !! .
r('/',pow(X,-(Z)), pow(Y,-(Z)),W) :- b('/',Y,X,S), b('pow',S,Z,W), !! .
r('/',pow(X,Z),   pow(Y,Z),W)     :- b('/',X,Y,T), b('pow',T,Z,W), !! .



% simplification clauses for exponentiation 


r('pow',X,-(1),Y) :- b('/',1,X,Y), !! .

r('pow',X,-(Y),Z) :- b('pow',X,Y,W), b('/',1,W,Z), !! .

r('pow',pow(X,Y),Z,W) :- b('*',Y,Z,T), b('pow',X,T,W), !! .


% catch all clause to cover cases not covered
% by the previous clauses      

r(OP,X,Y,R) :- b(OP,X,Y,R).

%"



"%

!

s(X,Y) :- simplify(X,Z), simplify(Z,Y).


%"

  s ( *(*(2,1),pow(x,-(2,1))) , F )  

"%
!

simplify(X,X) :- atomic(X), !! .

simplify(X,Y) :- =..(X, [Op,Z]), 
                 simplify(Z,Z1), 
                 u(Op,Z1,Y), !! .

simplify(X,Y) :- =..(X, [Op,Z,W]), 
                 simplify(Z,Z1),
                 simplify(W,W1),
                 r(Op,Z1,W1,Y), !! .


!

%******************************************************
%  basic rules for unary operator
%******************************************************

u('-' , 0, 0)    :- !! .
u('-' , -(X), X) :- !! .
u('-' , X, -(X)) :- !! .

u('-' , X,R)     :- number(X), is( -(0,X), R), !! .

u('sin' , X,R)   :- number(X), sin(X,R), !! .
u('cos' , X,R)   :- number(X), cos(X,R), !! .
u('sqrt' , X,R)  :- number(X), sqrt(X,R), !! .
u('tan' , X,R)   :- number(X), tan(X,R), !! .
u('ln' , X,R)    :- number(X), ln(X,R), !! .
u('exp' , X,R)   :- number(X), exp(X,R), !! .

% catch all clause to cover cases not covered
% by the previous clauses      

u(OP,X,  OP(X) ). 

%"

   u( '-' , 6 , R)       
   u( '-' , -(6) , R)       

   u( '-' , x , R)       
   u( '-' , -(x) , R)       

   u( 'sin' , x , R)       
   u( 'sin' , 5 , R)       

"%


! !

!Diff2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/Diff2.st,v 1.7 2002/11/26 18:31:17 cg Exp $'
! !
