"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologClass subclass:#KalahGame
	instanceVariableNames:'tries answers infoTextHolder'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog-Examples'
!


!KalahGame methodsFor:'prolog-clauses'!

alpha_beta(0, Pos, Alpha, Beta, Move, Value) :-
	!!, value(Pos, Value).

alpha_beta(D, Pos, Alpha, Beta, Move, Value) :-
	findall(M, move(Pos, M), Moves),
	-(0, Beta, Alpha1),
	-(0, Alpha, Beta1),
	-(D, 1, D1),
	evaluate_and_choose(Moves, Pos, D1, Alpha1, Beta1, [], (Move,Value)).
        
!

announce(opponent) :- write('You won!! Congratulations.'), nl.

announce(computer) :- write('I won.'), nl.

announce(draw) :- write('The game is a draw.'), nl.
!

check_capture(N,Stones,Hs,Hs1,Ys,Ys1,Pieces) :-
    +(N, Stones, FinishingHole),
%    nth_member(FinishingHole,Hs,1),
    -(7, FinishingHole, OppositeHole),
    nth_member(OppositeHole,Ys,Y),
    >(Y, 0), !!,
    n_substitute(OppositeHole,Hs,0,Hs1),
    n_substitute(FinishingHole,Ys,0,Ys1),
%    n_substitute(OppositeHole,Ys,0,Ys1),
%    n_substitute(FinishingHole,Hs,0,Hs1),
    +(Y, 1, Pieces).

check_capture(N,Stones,Hs,Hs,Ys,Ys,0) :- !!.

!

check_if_finished(board(Hs,K,Ys,L),board(Hs,K,Hs,L1)) :-
  zero(Hs), !!, sumlist(Ys,YsSum), +(L, YsSum, L1).

check_if_finished(board(Hs,K,Ys,L),board(Ys,K1,Ys,L)) :-
  zero(Ys), !!, sumlist(Hs,HsSum), +(K, HsSum, K1).

check_if_finished(Board,Board) :- !!.
!

choose_move(Pos, computer, Move) :-
	lookahead(Depth),
	alpha_beta(Depth, Pos, -40, 40, Move, Value),
	nl,
	write(Move),
	nl.

choose_move(Pos, opponent, Move) :-
	read(Move, 'please make move'),
	legal(Move).

!

cutoff(Move, Value, D, Alpha, Beta, Moves, Pos, Record, (Move, Value)) :-
	>=(Value, Beta), !!.

cutoff(Move,Value,D,Alpha,Beta,Moves,Position,Move1,BestMove) :- 
	<(Alpha, Value), <(Value, Beta), 
	evaluate_and_choose(Moves,Position,D,Value,Beta,Move,BestMove).

cutoff(Move,Value,D,Alpha,Beta,Moves,Position,Move1,BestMove) :-
	=<(Value, Alpha), 
	evaluate_and_choose(Moves,Position,D,Alpha,Beta,Move1,BestMove).



!

distribute(0,Hs,Hs) :- !!.

distribute(N,[H|Hs],[H1|Hs1]) :-
	>(N, 0), !!, -(N, 1, N1), +(H, 1, H1), distribute(N1,Hs,Hs1).

distribute(N,[],[]) :- !!.


!

distribute_my_holes(Stones,N,board(Hs,K,Ys,L),board(Hs1,K1,Ys,L),Stones1) :-
    -(7, N, N7), 
    >(Stones, N7), !!, 
    pick_up_and_distribute(N,Stones,Hs,Hs1),
    +(K, 1 , K1), 
    +(Stones, N , StonesN), 
    -(StonesN, 7, Stones1).

distribute_my_holes(Stones,N,board(Hs,K,Ys,L),Board,0) :-
    pick_up_and_distribute(N,Stones,Hs,Hs1),
    check_capture(N,Stones,Hs1,Hs2,Ys,Ys1,Pieces),
    update_kalah(Pieces,N,Stones,K,K1),
    check_if_finished(board(Hs2,K1,Ys1,L),Board).
!

%  distribute_stones(Stones,Hole,Board,Board1) :-
%        Board1 is the result of distributing the number of stones,
%        Stones, from Hole from the current Board.
%        It consists of two stages: distributing the stones in the player's
%        holes, distribute_my_holes, and distributing the stones in 
%        the opponent's holes, distribute_your_holes.
%

distribute_stones(Stones,Hole,Board,FinalBoard) :-
   distribute_my_holes(Stones,Hole,Board,Board1,Stones1),
   distribute_your_holes(Stones1,Board1,FinalBoard).

!

distribute_your_holes(0,Board,Board) :- !!.

distribute_your_holes(Stones,board(Hs,K,Ys,L),board(Hs,K,Ys1,L)) :-
  =<( 1, Stones), =<(Stones, 6), 
  non_zero(Hs), !!, 
  distribute(Stones,Ys,Ys1).

distribute_your_holes(Stones,board(Hs,K,Ys,L),Board) :-
  >(Stones, 6), !!, 
  distribute(6,Ys,Ys1),
  -(Stones, 6, Stones1),
  distribute_stones(Stones1,0,board(Hs,K,Ys1,L),Board).

distribute_your_holes(Stones,board(Hs,K,Ys,L),board(Hs,K,Hs,L1)) :-
  zero(Hs), !!, sumlist(Ys,YsSum), 
  +(YsSum, L, T),
  +(Stones, T, L1).


!

evaluate_and_choose([ Move | Moves], Pos, D, Alpha, Beta, Record, BestMove) :-
	move(Move, Pos, Pos1),
	alpha_beta(D, Pos1, Alpha, Beta, MoveX, Value),
	-(0, Value, Value1),
	cutoff(Move, Value1, D, Alpha, Beta, Moves, Pos, Record, BestMove),
	!!.

evaluate_and_choose([], Pos, D, Alpha, Beta, Move, (Move,Value)) .

!

extend_move(Stones,M,Board,[]) :-
	-(7, M, M7), mod(M7, 13, M7mod13), \==(Stones, M7mod13), !!.

extend_move(Stones,M,Board,Ms) :- 
	-(7, M, M7), mod(M7, 13, M7mod13), \==(Stones, M7mod13), !!,
%        Stones =:= (7-M) mod 13, !!, 
	distribute_stones(Stones,M,Board,Board1),
		  move(Board1,Ms).





!

game_over(board(0,N,0,N),Player,draw) :-
	pieces(K), *(6, K, K6) , \=(N, K6), !!.

game_over(board(H,K,Y,L),Player,Player) :- 
	pieces(N), *(6, N, N6), >(K, N6), !!.

game_over(board(H,K,Y,L),Player,Opponent) :-
	pieces(N), *(6, N, N6), >(L , N6), next_player(Player,Opponent).
!

initialize(kalah,board([N,N,N,N,N,N],0,[N,N,N,N,N,N],0),opponent) :-
	pieces(N).

!

legal([N|Ns]) :-  
	<(0, N), <(N, 7), legal(Ns).

legal([]).

%"
  legal( [ 1 , 2 , 3 ])
  legal( [ 1 , 8 , 3 ])  
  legal( 1)  
"%
!

lookahead(2).

!

move(Board,[M|Ms]) :- 
	member(M,[1,2,3,4,5,6]), 
	stones_in_hole(M,Board,N),
	extend_move(N,M,Board,Ms).

move(board([0,0,0,0,0,0],K,Ys,L),[]).


move([N|Ns],Board,FinalBoard) :- 
       stones_in_hole(N,Board,Stones),
       distribute_stones(Stones,N,Board,Board1),
       move(Ns,Board1,FinalBoard).

move([],Board1,Board2) :-
	swap(Board1,Board2).
!

next_player(computer,opponent).    
next_player(opponent,computer).

!

non_zero(Hs) :- 
	\=(Hs , [0,0,0,0,0,0]).


!

pick_up_and_distribute(0,N,Hs,Hs1) :-
  !!, distribute(N,Hs,Hs1).

pick_up_and_distribute(1,N,[H|Hs],[0|Hs1]) :-
  !!, distribute(N,Hs,Hs1).

pick_up_and_distribute(K,N,[H|Hs],[H|Hs1]) :- 
  >(K, 1), !!, -(K, 1, K1), pick_up_and_distribute(K1,N,Hs,Hs1).
!

pieces(6).

!

stones_in_hole(M,board(Hs,K,Ys,L),Stones) :-
	nth_member(M,Hs,Stones), >(Stones, 0).
!

swap(board(Hs,K,Ys,L),board(Ys,L,Hs,K)).

!

update_kalah(0,Stones,N,K,K) :- 
	-(7, N, N7), <(Stones, N7), !!.

update_kalah(0,Stones,N,K,K1) :- 
	-(7, N, N7), \=(Stones, N7), !!, +(K,1,K1).

update_kalah(Pieces,Stones,N,K,K1) :- 
	>(Pieces, 0), !!, +(K,Pieces,K1).
!

value(board(H,K,Y,L),Value) :- 
	-(K, L, Value).
!

zero([0,0,0,0,0,0]).
! !

!KalahGame methodsFor:'prolog-main'!

play(Game) :- 
	initialize(Game,Position,Player), 
	display_game(Position,Player),
	play(Position,Player,Result).

play(Position,Player,Result) :- 
	game_over(Position,Player,Result), !!, announce(Result).

play(Position,Player,Result) :-
	choose_move(Position,Player,Move),
	move(Move,Position,Position1),
	display_game(Position1,Player), 
	next_player(Player,Player1),
	!!, 
	play(Position1,Player1,Result).

%"
    clear, trace , 
    play(kalah)
"%
! !

!KalahGame methodsFor:'prolog-output'!

display_game(Position,computer) :-
	show(Position).

display_game(Position,opponent) :-
	swap(Position,Position1), show(Position1).



!

display_holes([H|Hs]) :-
	write_pile(H), display_holes(Hs).

display_holes([]) :- nl.
!

show(board(H,K,Y,L)) :-
	reverse(H,HR),  
	write_stones(HR), write_kalahs(K,L), write_stones(Y).



!

write_kalahs(K,L) :- 
	write(K), tab(34), write(L), nl.
!

write_pile(N) :- 
	<(N, 10), write(N), tab(4).

write_pile(N) :- 
	>=(N, 10), write(N), tab(3).
!

write_stones(H) :- 
	nl, tab(5), display_holes(H).

! !

!KalahGame methodsFor:'prolog-utilities'!

mod(A, B, C) :- \\(A, B, C).

%"

   mod(4, 2, R)
   mod(4, 3, R) 

"%
!

n_substitute(1,[X|Xs],Y,[Y|Xs]) :- !!.

n_substitute(N,[X|Xs],Y,[X|Xs1]) :- 
	>(N , 1), !!, 
	-(N, 1, N1),
	n_substitute(N1,Xs,Y,Xs1).

%"
  n_substitute(5, [ a , b , c , d , e , f , g ], eee , R)
"%

!

nth_member(N,[H|Hs],K) :-
	>(N, 1), !!, 
	-(N, 1, N1), 
	nth_member(N1,Hs,K).

nth_member(1,[H|Hs],H).

%"
  nth_member(5, [ a , b , c , d , e , f , g ], R)
  nth_member(9, [ a , b , c , d , e , f , g ], R)
"%
!

sumlist([],0).

sumlist([A|R],S) :-
	sumlist(R,S1),
	+(A, S1, S).

%"
   sumlist( [ 1 , 2 , 3,  4,  5,  6 ], S) 
"%



! !

!KalahGame class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/KalahGame.st,v 1.3 2002/11/26 18:31:13 cg Exp $'
! !
