"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

ApplicationModel subclass:#PrologClass
	instanceVariableNames:'myPrologInterpreter'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog-ClassWrapper'
!


!PrologClass class methodsFor:'initialization'!

initialize
    PrologInterpreter initialize
! !

!PrologClass class methodsFor:'instance creation'!

new
    "trap instance creation"

    ^ super new setupInterpreter; yourself.

    "
     Prolog::Family new
    "
! !

!PrologClass class methodsFor:'installing prolog clauses'!

addChangeRecordForClause:aClause category:categoryString
    "{ Pragma: +optSpace }"

    |mthd|

    "add a methodCategory-change-record to the changes file"

    Class updateChangeFileQuerySignal query ifTrue:[
        self writingChangeDo:[:aStream |
            self printClassNameOn:aStream.
            aStream nextPutAll:(' addClause:' , aClause storeString).
            aStream nextPutAll:(' category:' , categoryString storeString).
            aStream nextPutChunkSeparator.
        ].
"/
"/        "this test allows a smalltalk without Projects/ChangeSets"
"/        Project notNil ifTrue:[
"/            Project addChangeForClause:aClause category:categoryString in:self
"/        ]
    ]
!

addClause:somePrologCode category:category 
    "add some clause"

    self 
        install:somePrologCode 
        inCategory:category 
        onError:[:errMsg :start :stop |
                    self error:errMsg
                ].

!

install:somePrologCode inCategory:category notifying:requestor
    "test the code, by creating a new prolog parser, and letting him
     parse the piece of given code ..."

    |m|

    m := self 
        install:somePrologCode 
        inCategory:category 
        onError:[:errMsg :start :stop |
                   requestor
                        error:errMsg position:start to:(start+10)"stop" from:nil.
                    ^ nil
                ].
"/    self addChangeRecordForClause:somePrologCode category:category.
    self addChangeRecordForMethod:m.
    ^ m
!

install:somePrologCode inCategory:category onError:errorBlock
    "test the code, by creating a new prolog parser, and letting him
     parse the piece of given code ..."

    |tester source m node clause hornClause functor
     functorSelector functorString|

    source := somePrologCode readStream.

    tester := Prolog::PrologParser new.
    [
        node := tester 
                parse: source 
                ifFail: 
                   [:errorMessage | |label string errString token| 
                       token := tester currentToken.
                       (token isKindOf:PrologEntity) ifTrue:[
                            string := token prologPrintString
                       ] ifFalse:[
                            string := token printString
                       ].
                       label := errorMessage , ' near ' , string.
                       string := source upToEnd.
                       string size > 100 ifTrue: [string := string copyFrom: 1 to: 100].
                       errString := label , '\' withCRs , (string = ''
                                               ifTrue: ['-- end of file --']
                                               ifFalse: ['--> ' , string]).
                       errorBlock 
                            value:errString
                            value:tester tokenPosition 
                            value:source position.
                       ^ nil.
                   ].
        node == #eof
    ] whileFalse:[
        hornClause := node cdr.
        node car ifTrue:[
            "/ a query
            errorBlock
                value:'queries not yet supported'
                value:tester tokenPosition 
                value:source position.
           ^ nil.
        ] ifFalse:[
            functor := hornClause car car.
            functorString := functor string.
            functorSelector isNil ifTrue:[
                functorSelector := functorString
            ] ifFalse:[
                functorSelector ~= functorString ifTrue:[
                    errorBlock
                        value:'only definitions for the same functor (', functorSelector, ') allowed here'
                        value:tester tokenPosition 
                        value:source position.
                ]
            ].
"/            program := tracePredicates at: functor ifAbsent: [userPredicates at: functor ifAbsent: []].
"/            program == nil
"/                    ifTrue: 
"/                            [program := PrologDefinition fromList: (hornClause cons: nil).
"/                            userPredicates at: functor put: program]
"/                    ifFalse: [program nconc: (hornClause cons: nil)]]].
        ].
    ].

    "/ if we arrive here,
    "/ install it as a dummy (source only method)
    m := PrologMethod new.
    m isClause:true.
    m source:somePrologCode.
    m category:category.
    m mclass:self.
    self primAddSelector:functorSelector asSymbol withMethod:m.
    ^ m
! !

!PrologClass class methodsFor:'mimicri'!

categories
    |allOfThem|

    allOfThem := super categories asSet.
    self == PrologClass ifFalse:[
        allOfThem addAll:#( #'prolog-clauses' )
    ].
    ^ allOfThem


!

compilerClass
    self == PrologClass ifTrue:[
        ^ super compilerClass
    ].
    ^ PrologMethodInstaller
!

evaluatorClass
    self name = PrologClass name ifTrue:[
        ^ super evaluatorClass
    ].
    ^ PrologEvaluator
!

parserClass
    self == PrologClass ifTrue:[
        ^ super parserClass
    ].
    ^ nil "/ PrologMethodInterpreter
!

syntaxHighlighterClass
    self == PrologClass ifTrue:[
        ^ super syntaxHighlighterClass
    ].
    ^ nil
! !

!PrologClass class methodsFor:'private'!

convertFromProlog:aPrologEntity
    aPrologEntity isPrologVariable ifTrue:[
        self error:'result is a variable'.
    ].
    (aPrologEntity isKindOf:PrologEntity) ifTrue:[
        (aPrologEntity isKindOf:PrologSymbol) ifTrue:[
            ^ aPrologEntity string asSymbol
        ].
        ^ aPrologEntity prologPrintString
    ].
    ^ aPrologEntity

! !

!PrologClass class methodsFor:'queries'!

isVisualStartable
    "return true, if this application can be started via #open.
     (to allow start of a change browser via double-click in the browser)"

    (self == ApplicationModel) ifTrue:[^ false "I am abstract"].
    ((self respondsTo:#open) or:[self class includesSelector:#openInterface]) ifFalse:[^ false].
    (self respondsTo:#windowSpec) ifFalse:[^ false].
    ^ true
! !

!PrologClass methodsFor:'debugging'!

trace:aBoolean
    aBoolean ifTrue:[
        myPrologInterpreter textCollector:Transcript.
        myPrologInterpreter trace.
    ] ifFalse:[
        myPrologInterpreter notrace
    ]
! !

!PrologClass methodsFor:'initialization'!

addClausesFrom:aClass to:interpreter
"/    Transcript showCR:'adding from: ', aClass name.
    aClass methodDictionary do:[:aPrologMethod |
        (aPrologMethod perform:#isClause ifNotUnderstood:false) ifTrue:[
"/            Transcript showCR:aPrologMethod source.
            interpreter
                    refute:(aPrologMethod source)
                    action:[:someResult | ].
        ]
    ].



!

setupInterpreter
    |interpreter cls|

    interpreter := myPrologInterpreter := PrologInterpreter new.
    interpreter textCollector: nil.

    "/ add facts & rules

    interpreter
        refute:'?- remove. ' action:[:someResult | ].

    cls := self class.
    [cls notNil and:[cls ~~ Prolog::PrologClass]] whileTrue:[
        self addClausesFrom:cls to:interpreter.
        cls := cls superclass.
    ].
    interpreter instance:self.
    ^ interpreter

    "
     Prolog::Family basicNew
    "
! !

!PrologClass methodsFor:'prolog queries'!

doesNotUnderstand:aMessage
    "see if there is a 0-arity clause named alike,
     and invoke it if so."

"/    |selector|
"/
"/    selector := aMessage selector.
"/    selector numArgs == 0 ifTrue:[
"/        ^ self queryValue:selector
"/    ].
"/    selector numArgs == 1 ifTrue:[
"/        ^ self queryValue:(selector copyWithoutLast:1) , '(' , (aMessage arguments at:1) printString , ')'
"/    ].
    ^ super doesNotUnderstand:aMessage

"
    SendMoneyPuzzle new puzzle

    SendMoneyPuzzle new digit:0     
    SendMoneyPuzzle new digit:#a    
    SendMoneyPuzzle new digit:#A    
    SendMoneyPuzzle new query:'digit(0)'     
    SendMoneyPuzzle new queryValue:'digit(A)'
"
!

query:aQuery
    |results r n nMax|

    results := Set new.
    n := 0.
    nMax := 1000.
    r := self
        query:aQuery 
        into:[:result | 
                results add:result. 
                n := n + 1.
                n > nMax ifTrue:[
                    (self confirm:('Are you shure that your rules are not recursive.\\(Proceed after %1 results)' bindWith:n) withCRs)
                    ifFalse:[
                        ^ results
                    ].
                    nMax := nMax * 2.
                ]
             ] 
        stopAfterFirst:false.

     n == 0 ifTrue:[^ r].
    ^ results.

    "
     Prolog::Family new
        query:'brother(sofia,X).'
    "

!

query:aQuery allInto:aBlock
    ^ self
        query:aQuery 
        into:aBlock 
        stopAfterFirst:false

    "
     Prolog::Family new 
        query:'brother(X,Y).' 
        allInto:[:result | Transcript showCR:result]        
    "

    "
     Prolog::Family new 
        queryValues:'brother(X,Y).' 
        allInto:[:x :y | Transcript showCR:x,' is a brother of ',y]
    "

!

query:aQuery firstInto:aBlock
    ^ self
        query:aQuery 
        into:aBlock 
        stopAfterFirst:true

    "
     Prolog::Family new
        query:'brother(jrgen,X).' 
        firstInto:[:x | Transcript showCR:x]
    "


!

query:aQuery into:aBlock stopAfterFirst:stopAfterFirst
    |queryString|

    queryString := aQuery.
    (queryString startsWith:'?-') ifFalse:[
        queryString := '?- ' , queryString
    ].
    (queryString endsWith:'.') ifFalse:[
        queryString := queryString , '.'
    ].

    myPrologInterpreter isNil ifTrue:[
        self setupInterpreter.
    ].
    myPrologInterpreter textCollector:Transcript.

    ^ myPrologInterpreter
        refute:queryString
        action:[:answer | |convertedAnswer|
                    convertedAnswer := Dictionary new.
                    answer keysAndValuesDo:[:var :value |
                        convertedAnswer at:var put:(self class convertFromProlog:value)
                    ].
                    aBlock value:convertedAnswer.
                    stopAfterFirst     "/ want more
               ]

    "
     Prolog::Family new
        query:'brother(claus,X).' 
        into:[:x | Transcript showCR:x]
        stopAfterFirst:false
    "


!

queryFirst:aQuery
    self
        query:aQuery 
        into:[:result | ^result] 
        stopAfterFirst:false.

    ^ nil.

    "
     Prolog::Family new
        queryFirst:'brother(sofia,X).'       
    "

    "
     Prolog::Family new
        query:'brother(sofia,X).
        stopAfter:4'       
    "

!

queryValue:aQuery
    |results r n nMax|

    results := Set new.
    n := 0.
    nMax := 1000.
    r := self
        queryValues:aQuery 
        into:[:result | 
                results add:result. 
                n := n + 1.
                n > nMax ifTrue:[
                    (self confirm:('Are you shure that your rules are not recursive.\\(Proceed after %1 results)' bindWith:n) withCRs)
                    ifFalse:[
                        ^ results
                    ].
                    nMax := nMax * 2.
                ]
             ] 
        stopAfterFirst:false.

    n == 0 ifTrue:[^ r].
    ^ results.

    "
     Prolog::Family new
        queryValue:'brother(sofia,X).'
    "

!

queryValueFirst:aQuery
    self
        queryValues:aQuery 
        into:[:result | ^result] 
        stopAfterFirst:false.

    ^ nil.

    "
     Prolog::Family new
        queryValueFirst:'brother(claudia,X)'       
    "

    "
     Prolog::Family new
        queryValue:'brother(claudia,X)'  
    "

!

queryValues:aQuery allInto:aBlock
    ^ self
        queryValues:aQuery 
        into:aBlock 
        stopAfterFirst:false

    "
     Prolog::Family new 
        queryValues:'brother(claudia,X).' 
        allInto:[:x | Transcript showCR:x]
    "


!

queryValues:aQuery into:aBlock stopAfterFirst:stopAfterFirst
    |queryString|

    queryString := aQuery.
    (queryString startsWith:'?-') ifFalse:[
        queryString := '?- ' , queryString
    ].
    (queryString endsWith:'.') ifFalse:[
        queryString := queryString , '.'
    ].

    myPrologInterpreter isNil ifTrue:[
        self setupInterpreter.
    ].

    ^ myPrologInterpreter
        refute:queryString
        action:[:answer | |numVars|
                    numVars := answer size.
                    numVars == 1 ifTrue:[
                        aBlock value:(self class convertFromProlog:answer first).
                    ] ifFalse:[
                        aBlock numArgs == 1 ifTrue:[
                            aBlock value:answer
                        ] ifFalse:[
                            aBlock valueWithArguments:(answer values collect:[:val | self class convertFromProlog:val]) asArray.
                        ]
                    ].
                    stopAfterFirst     "/ want more
               ]

    "
     Prolog::Family new
        queryValues:'brother(claudia,X).' 
        into:[:x | Transcript showCR:x]
        stopAfterFirst:false
    "


! !

!PrologClass class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologClass.st,v 1.16 2004/07/02 08:51:34 ca Exp $'
! !

PrologClass initialize!
