"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

ApplicationModel subclass:#PrologInterpreter
	instanceVariableNames:'systemPredicates userPredicates status definition question
		questionEnv clause clauseEnv queue queueEnv envCounter valueEnv
		unbindList cutBack backTrack resolveAction definitionStack
		tracePredicates traceCounter textValue textCollector verbose
		listValue predicate instance cutSymbol trueSymbol failSymbol
		sendSymbol varSymbol'
	classVariableNames:'EditorMenu PredEditMenu PredListMenu QuestionMenu
		SystemPredicates'
	poolDictionaries:''
	category:'Languages-Prolog'
!

PrologInterpreter comment:'

Prolog
Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
Copyright (C) 1995-1997 AOKI Atsushi
1997/02/21

'
!

!PrologInterpreter class methodsFor:'documentation'!

documentation
"
    Prolog
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"

! !

!PrologInterpreter class methodsFor:'instance creation'!

new
        ^super new initialize
! !

!PrologInterpreter class methodsFor:'class initialization'!

flushMenus
        "PrologInterpreter flushMenus."

        QuestionMenu := nil.
        EditorMenu := nil.
        PredListMenu := nil.
        PredEditMenu := nil
!

flushOthers
        "PrologInterpreter flushOthers."

        self flushSystemPredicates
!

flushSystemPredicates
        "PrologInterpreter flushSystemPredicates."

        SystemPredicates := nil
!

initialize
        "PrologInterpreter initialize."

        self initializeFont.
        self initializeExtensions.
!

initializeExtensions
        "PrologInterpreter initializeExtensions."

        | source stream | 

        (Object compiledMethodAt:#isPrologVariable) notNil ifTrue:[^  self].

        source := '
!!UndefinedObject methodsFor: ''prolog''!!

car
        ^nil!!

cdr
        ^nil!!

isPrologEntity
        ^true!! !!

!!Object methodsFor: ''prolog''!!

car
        self == nil ifTrue: [^nil].
        self error: ''send message car to atom''!!

cdr
        self == nil ifTrue: [^nil].
        self error: ''send message cdr to atom''!!

cons: anObject 
        ^Prolog::PrologList car: self cdr: anObject!!

consp
        ^false!!

isPrologEntity
        self == nil ifTrue: [^true].
        (self isKindOf: Number)
                ifTrue: [^true].
        ^false!!

isPrologVariable
        ^false!!

printPrologOn: aStream 
        self == nil
                ifTrue: 
                        [aStream nextPutAll: ''[]''.
                        ^self].
        (self isKindOf: Number)
                ifTrue: 
                        [self printOn: aStream.
                        ^self].
        aStream nextPut: ${.
        self printOn: aStream.
        aStream nextPut: $}!!

printPrologOn: aStream level: anInteger 
        self printPrologOn: aStream!!

printPrologString
        | aStream |
        aStream := WriteStream on: (String new: 20).
        self printPrologOn: aStream.
        ^aStream contents
!! !!
'.
"/        stream := ReadWriteStream
"/                                on: source
"/                                from: 0
"/                                to: source size.
        stream := ReadWriteStream
                                on: source
                                from: 1
                                to: source size.
        Class withoutUpdatingChangesDo:[
            Class packageQuerySignal answer:self package do:[
                stream fileIn
            ]
        ]
    "Modified: / 17.4.1998 / 14:29:57 / cg"
!

initializeFont
        "PrologInterpreter initializeFont."

"/ XXX the ST/X emulation of Text- and characterAttributes is not yet
"/ complete ...

        | defaultFontName defaultFontSize fontDescription characterAttributes textAttributes |
        defaultFontName := #('Courier*' 'courier*' '**').
        defaultFontSize := 12.
        fontDescription := FontDescription new.
        fontDescription family: defaultFontName.
        fontDescription pixelSize: defaultFontSize.
        fontDescription boldness: 0.5.
        fontDescription italic: false.
        fontDescription outline: false.
        fontDescription shadow: false.
        fontDescription underline: false.
        fontDescription fixedWidth: true.
        fontDescription serif: false.
        fontDescription strikeout: false.
"/        fontDescription encodings: Locale current preferredEncodings.

        CharacterAttributes notNil ifTrue:[
            Autoload autoloadFailedSignal catch:[
                characterAttributes := CharacterAttributes newWithDefaultAttributes.
                characterAttributes setDefaultQuery: fontDescription.

                textAttributes := TextAttributes new.
                textAttributes setCharacterAttributes: characterAttributes.
"/                deviceFont := Screen default defaultFontPolicy findFont: fontDescription.
"/                textAttributes lineGrid: deviceFont height.
"/                textAttributes baseline: deviceFont ascent.
                TextAttributes styleNamed: #prologDefault put: textAttributes
            ].
        ]
!

reinitialize
        "PrologInterpreter reinitialize."

        PrologSymbol initialize.
        PrologVariable initialize.
        self flushSystemPredicates.
        self initialize.
!

removeExtensions
        "PrologInterpreter removeExtensions."

        Class withoutUpdatingChangesDo:[
            (Array 
                with:Object
                with:UndefinedObject)
            do:[:cls |
                cls methodDictionary do:[:eachMethod |
                    eachMethod package = self package ifTrue:[
                        Object removeSelector:eachMethod selector
                    ].
                ].
            ].
        ].

! !

!PrologInterpreter class methodsFor:'examples'!

example
        "PrologInterpreter example."

        PrologInterpreter open
!

example01
        "PrologInterpreter example01."

        | string prolog result |
        string := '

        ?- remove.         % remove all predicates in PrologDataBase.
        likes(john,mary).    % assert fact.
        likes(john,wine).    % assert fact.
        likes(mary,wine).    % assert fact.
        likes(mary,john).    % assert fact.
        ?- likes(X,Y).       % question goal.
        ?- likes(john,X), likes(mary,X). % question goal.

        '.
        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog).

        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:14:08 / cg"
!

example01b
        "PrologInterpreter example01b."

        | string prolog result |
        string := '

        ?- remove.         % remove all predicates in PrologDataBase.
        schwester(vincent, sofia).
        bruder(sofia, vincent).
        bruder(claus, jurgen).
        bruder(jurgen, claus).
        schwester(claus, claudi).
        schwester(betti, steffi).
        mutter(claus, rosmarie).
        mutter(sofia, betti).
        mutter(betti, brbel).
        ehemann(rosmarie, herbert).
        ehemann(betti, claus).
        ehemann(brbel, gnther).
        geburtstag(sofia, [8 , april, 1992] ).

        liebt(claus,betti).    % assert fact.
        liebt(claus,sofia).    % assert fact.
        liebt(claus,vincent).    % assert fact.

        liebt(betti,sofia).    % assert fact.
        liebt(betti,claus).    % assert fact.
        liebt(betti,vincent).    % assert fact.

        mutter(X, M) :- schwester(X, Y), mutter(Y, M).
        mutter(P, M) :- bruder(P, I), mutter(I, M).

        vater(Person, V) :- ehemann(IrgendEineFrau, V), mutter(Person, IrgendEineFrau).

        ?- geburtstag(Person,[ 8 , april, 1992] ). % question goal.
        ?- geburtstag(sofia,Tag). % question goal.
        ?- vater(claus,Vater). % question goal.

        '.
        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog).

        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:14:08 / cg"
!

example02
        "PrologInterpreter example02."

        | string prolog result |
        string := '

        ?- remove.         % remove all predicates in PrologDataBase.
        on(hen,cat).
        on(cat,dog).
        on(dog,donkey).
        above(X,Y) :-
            on(X,Y).
        above(X,Y) :-
            on(X,Z), above(Z,Y).
        ?- on(X,dog).
        ?- above(X,dog).

        '.
        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog
                    refute: string
                    action:(self exampleActionFor:prolog).
        "/ Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:24:04 / cg"
!

example03
        "PrologInterpreter example03."

        | string prolog result |
        string := '

        ?- remove.      % remove all predicates in PrologDataBase.
        hanoi(N) :-     % This is Hanoi Tower Prolgram.
            move(N,left,right,center).
            move(0,X,Y,Z) :- !!.
        move(N,A,B,C) :-
            -(N,1,M),
            move(M,A,C,B),
            inform(A,B),
            move(M,C,B,A).
        inform(X,Y) :-
            write([move,disc,from,X,to,Y]),
            nl.
        ?- hanoi(3).     % question goal.

        '.

        Transcript showCR:'Towers of Hanoi in Prolog:'.
        Transcript showCR:'=========================='.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog).

"/        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:23:27 / cg"
!

example04
        "PrologInterpreter example04."

        | string prolog result |

        string := '?- append(X,Y,[a,b,c,d]).'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).

        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:30:27 / cg"
!

example04b
        "PrologInterpreter example04b."

        | string prolog result |

        string := '?- append([a,b,c],X,[a,b,c,d,e,f]).'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).

        Transcript show: result printString; cr

    "Created: / 20.4.1998 / 12:50:44 / cg"
    "Modified: / 20.4.1998 / 12:51:12 / cg"
!

example04c
        "PrologInterpreter example04c."

        | string prolog result |

        string := '?- append([a,b,c],[d,e,f],X).'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).

        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:30:27 / cg"
    "Created: / 20.4.1998 / 12:51:07 / cg"
!

example04d
        "PrologInterpreter example04d."

        | string prolog result |

        string := '?- append(X, Y, [a,b,c,d]).'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).

        Transcript show: result printString; cr

!

example05
        "PrologInterpreter example05."

        | string prolog result |
        string := '

        image({Image fromUser}).
        graphicsContext({(View extent:300@300) openAndWait}).
        point({100@100}).
        ?- image(X), graphicsContext(Y), point(Z), send(X,displayOn:at:,[Y,Z]).

        '.
        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action: 
                            [:answer | 
                                answer keys asSortedCollection
                                    do: 
                                            [:s | 
                                            Transcript show: s.
                                            Transcript show: ' = '.
                                            Transcript show: (answer at: s) printPrologString.
                                            Transcript cr].
                                prolog confirm: 'All right?'
                            ].
        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:25:52 / cg"
!

example06
        "PrologInterpreter example06."

        | string prolog result |

        string := '?- send(123,+,[456],X).   % X := 123 + 456.'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).

        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:31:30 / cg"
!

example07
        "PrologInterpreter example07."

        | string prolog result |
        string := '
?- is(X,+(3,4)).
?- is(X,F(3,4)).'.

        prolog := PrologInterpreter new.
        prolog textCollector: Transcript.
        result := prolog 
                    refute: string
                    action:(self exampleActionFor:prolog confirmTitle:string).
        Transcript show: result printString; cr

    "Modified: / 20.4.1998 / 12:32:26 / cg"
!

exampleActionFor:aPrologInterpreter
    "claus: extracted this - its common to all examples"

    ^ self exampleActionFor:aPrologInterpreter confirmTitle:nil

    "Modified: / 20.4.1998 / 12:49:12 / cg"
!

exampleActionFor:aPrologInterpreter confirmTitle:t
    "claus: extracted this - its common to all examples"

    ^
        [:answer | 

            |str msg|

            str := '' writeStream.
            answer keys asSortedCollection do:[:s |
                str nextPutAll:s; nextPutAll:'='; nextPutAll:(answer at: s) printPrologString.
                str cr.
            ].

            Transcript show: str contents.

            msg := str contents , '\\All right ?' withCRs.
            t size > 0 ifTrue:[
                msg := t , '\\' withCRs , msg
            ].

            "/ want more answers ?
            aPrologInterpreter confirm:msg
        ]

    "Created: / 20.4.1998 / 12:13:44 / cg"
    "Modified: / 20.4.1998 / 12:29:32 / cg"
! !

!PrologInterpreter class methodsFor:'interface specs'!

consultingWindowSpec
        "UIPainter new openOnClass: self andSelector: #consultingWindowSpec."

        <resource: #canvas>

        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #label: 'Prolog Consultant' 
                        #min: #(#Point 200 200 )
                        #bounds: #(#Rectangle 100 100 300 300 ) ) 
                #component: 
                #(#SpecCollection 
                        #collection: #( 
                                #(#TextEditorSpec 
                                        #layout: #(#LayoutFrame 1 0 1 0 -1 1 -1 1 ) 
                                        #name: #consultingTextValue 
                                        #model: #consultingTextValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #consultingTextValueChange: )
                                        #tabable: true 
                                        #menu: #consultingTextMenu  
                ) ) ) )

    "Modified: / 18.4.1998 / 19:50:22 / cg"
!

editingWindowSpec
        "UIPainter new openOnClass: self andSelector: #editingWindowSpec."

        <resource: #canvas>

        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #label: 'Prolog Editor' 
                        #min: #(#Point 300 200 )
                        #bounds: #(#Rectangle 100 100 400 300 ) ) 
                #component: 
                #(#SpecCollection 
                        #collection: #( 
                                #(#SequenceViewSpec 
                                        #layout: #(#LayoutFrame 1 0 1 0 -1 0.3 -1 1 ) 
                                        #name: #editingListValue 
                                        #flags: 15 
                                        #model: #editingListValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #editingListValueChange: ) 
                                        #menu: #editingListMenu 
                                        #useModifierKeys: true 
                                        #selectionType: #highlight  
                                )
                                #(#TextEditorSpec 
                                        #layout: #(#LayoutFrame 1 0.3 1 0 -1 1 -1 1 ) 
                                        #name: #editingTextValue 
                                        #model: #editingTextValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #editingTextValueChange: )
                                        #tabable: true 
                                        #menu: #editingTextMenu  
                                         ) ) ) )

    "Modified: / 18.4.1998 / 19:50:15 / cg"
!

listingWindowSpec
        "UIPainter new openOnClass: self andSelector: #listingWindowSpec."

        <resource: #canvas>

        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #label: 'Prolog Lister' 
                        #min: #(#Point 300 200 )
                        #bounds: #(#Rectangle 100 100 400 300 ) ) 
                #component: 
                #(#SpecCollection 
                        #collection: #( 
                                #(#SequenceViewSpec 
                                        #layout: #(#LayoutFrame 1 0 1 0 -1 0.3 -1 1 ) 
                                        #name: #listingListValue 
                                        #flags: 15 
                                        #model: #listingListValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #listingListValueChange: ) 
                                        #menu: #listingListMenu 
                                        #useModifierKeys: true 
                                        #selectionType: #highlight  
                                        )
                                #(#TextEditorSpec 
                                        #layout: #(#LayoutFrame 1 0.3 1 0 -1 1 -1 1 ) 
                                        #name: #listingTextValue 
                                        #model: #listingTextValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #listingTextValueChange: )
                                        #tabable: true 
                                        #menu: #listingTextMenu  
                                        ) ) ) )

    "Modified: / 18.4.1998 / 19:50:11 / cg"
!

old_windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:Prolog::PrologInterpreter andSelector:#windowSpec
     Prolog::PrologInterpreter new openInterface:#windowSpec
     Prolog::PrologInterpreter open
    "

    <resource: #canvas>

    ^
     
       #(#FullSpec
          #window: 
           #(#WindowSpec
              #name: 'Prolog'
              #layout: #(#LayoutFrame 176 0 150 0 425 0 449 0)
              #label: 'Prolog'
              #min: #(#Point 250 300)
              #max: #(#Point 1024 768)
              #bounds: #(#Rectangle 176 150 426 450)
              #usePreferredExtent: false
          )
          #component: 
           #(#SpecCollection
              #collection: 
               #(
                 #(#TextEditorSpec
                    #name: #textValue
                    #layout: #(#LayoutFrame 1 0 1 0 -1 1 -1 0.3)
                    #tabable: true
                    #model: #textValue
                    #menu: #textMenu
                    #hasHorizontalScrollBar: true
                    #hasVerticalScrollBar: true
                    #miniScrollerHorizontal: true
                    #callbacksSpec: 
                     #(#UIEventCallbackSubSpec
                        #valueChangeSelector: #textValueChange:
                        #focusOutSelector: #textValueChange:
                    )
                )
                 #(#ArbitraryComponentSpec
                    #name: #textCollectorView
                    #layout: #(#LayoutFrame 1 0 0 0.3 -1 1 -1 1)
                    #flags: 9
                    #hasHorizontalScrollBar: true
                    #hasVerticalScrollBar: true
                    #miniScrollerHorizontal: true
                    #component: #textCollectorView
                    #hasBorder: false
                )
              )
          )
      )

    "Modified: / 18.4.1998 / 20:03:33 / cg"
    "Created: / 19.4.1998 / 12:59:19 / cg"
!

reconsultingWindowSpec
        "UIPainter new openOnClass: self andSelector: #reconsultingWindowSpec."

        <resource: #canvas>

        ^#(#FullSpec 
                #window: 
                #(#WindowSpec 
                        #label: 'Prolog Reconsultant' 
                        #min: #(#Point 200 200 )
                        #bounds: #(#Rectangle 100 100 300 300 ) ) 
                #component: 
                #(#SpecCollection 
                        #collection: #( 
                                #(#TextEditorSpec 
                                        #layout: #(#LayoutFrame 1 0 1 0 -1 1 -1 1 ) 
                                        #name: #reconsultingTextValue 
                                        #model: #reconsultingTextValue 
                                        #callbacksSpec: 
                                        #(#UIEventCallbackSubSpec 
                                                #valueChangeSelector: #reconsultingTextValueChange: )
                                        #tabable: true 
                                        #menu: #reconsultingTextMenu  
                                         ) ) ) )

    "Modified: / 18.4.1998 / 19:50:06 / cg"
!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:Prolog::PrologInterpreter andSelector:#windowSpec
     Prolog::PrologInterpreter new openInterface:#windowSpec
     Prolog::PrologInterpreter open
    "

    <resource: #canvas>

    ^
     
       #(#FullSpec
          #window: 
           #(#WindowSpec
              #name: 'Prolog'
              #layout: #(#LayoutFrame 62 0 54 0 311 0 353 0)
              #label: 'Prolog'
              #min: #(#Point 250 300)
              #max: #(#Point 1024 768)
              #bounds: #(#Rectangle 62 54 312 354)
              #usePreferredExtent: false
          )
          #component: 
           #(#SpecCollection
              #collection: 
               #(
                 #(#VariableVerticalPanelSpec
                    #name: 'VariableVerticalPanel1'
                    #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
                    #component: 
                     #(#SpecCollection
                        #collection: 
                         #(
                           #(#TextEditorSpec
                              #name: #textValue
                              #tabable: true
                              #model: #textValue
                              #menu: #textMenu
                              #hasHorizontalScrollBar: true
                              #hasVerticalScrollBar: true
                              #miniScrollerHorizontal: true
                              #callbacksSpec: 
                               #(#UIEventCallbackSubSpec
                                  #valueChangeSelector: #textValueChange:
                                  #focusOutSelector: #textValueChange:
                              )
                          )
                           #(#ArbitraryComponentSpec
                              #name: #textCollectorView
                              #flags: 9
                              #hasHorizontalScrollBar: true
                              #hasVerticalScrollBar: true
                              #miniScrollerHorizontal: true
                              #component: #textCollectorView
                              #hasBorder: false
                          )
                        )
                    )
                    #handles: #(#Any 0.5 1.0)
                )
              )
          )
      )

    "Modified: / 19.4.1998 / 13:03:16 / cg"
! !

!PrologInterpreter class methodsFor:'public access'!

refute: stringOrStream 
        ^self new refute: stringOrStream
!

refute: stringOrStream action: aBlock 
        ^self new refute: stringOrStream action: aBlock
! !

!PrologInterpreter methodsFor:'accessing'!

instance
        ^instance
!

instance: someObject 
        instance := someObject
!

textCollector
        ^textCollector
!

textCollector: aTextCollector 
        textCollector := aTextCollector
! !

!PrologInterpreter methodsFor:'adaptor'!

acceptConsulting: aText 
        | readStream |
        textCollector show: '<consult from user>\' withCRs.
        readStream := ReadStream on: aText asString.
        self consult: readStream action: nil.
        ^true
!

acceptEditing: aText 
        | readStream |
        textCollector showCR: '<reconsult from editor>'.
        readStream := ReadStream on: aText asString.
        self withReadCursorDo:[
            self reconsult: readStream action: nil
        ].
        self updatePredList.
        ^true

    "Modified: / 19.4.1998 / 13:35:57 / cg"
!

acceptQuestion: aText 
        ^true
!

acceptReconsulting: aText 
        | readStream |
        textCollector showCR: '<reconsult from user>'.
        readStream := ReadStream on: aText asString.
        self withReadCursorDo:[
            self reconsult: readStream action: nil
        ].
        ^true

    "Modified: / 19.4.1998 / 13:36:11 / cg"
!

consultingMenu
        ^self editingMenu
!

consultingTextMenu
        ^[self consultingMenu]
!

consultingTextValue
        textValue isNil ifTrue: [textValue := Text new asValue].
        ^textValue
!

consultingTextValueChange: aController 
        self acceptConsulting: aController text
!

editingListMenu
        ^[self predEditMenu]
!

editingListValue
        listValue isNil ifTrue: [listValue := SelectionInList with: self predList].
        ^listValue
!

editingListValueChange: aController 
        self pred: self editingListValue selection
!

editingMenu
        "PrologInterpreter flushMenus."

        EditorMenu == nil ifTrue: [EditorMenu := Menu
                                        labels: 'find...\replace...\undo\copy\cut\paste\accept\cancel' withCRs
                                        lines: #(2 3 6)
                                        values: #(#find #replace #undo #copySelection #cut #paste #accept #cancel)].
        ^EditorMenu
!

editingText
        | stream def |
        predicate == nil ifTrue: [^Text new].
        stream := WriteStream on: (String new: 200).
        def := userPredicates at: predicate ifAbsent: [^Text new].
        [def consp]
                whileTrue: 
                        [def car prettyPrintPrologOn: stream.
                        stream cr.
                        def := def cdr].
        ^stream contents asText
!

editingTextMenu
        ^[self editingMenu]
!

editingTextValue
        textValue isNil ifTrue: [textValue := Text new asValue].
        ^textValue
!

editingTextValueChange: aController 
        self acceptEditing: aController text
!

ignoreMenu
        ^nil
!

listingListMenu
        ^[self predListMenu]
!

listingListValue
        listValue isNil ifTrue: [listValue := SelectionInList with: self predList].
        ^listValue
!

listingListValueChange: aController 
        self pred: self listingListValue selection
!

listingText
        | stream def |
        predicate == nil ifTrue: [^Text new].
        stream := WriteStream on: (String new: 200).
        def := userPredicates at: predicate ifAbsent: [nil].
        def isNil ifTrue:[
            def := systemPredicates at: predicate ifAbsent: [^Text new].
        ].
        [def consp]
                whileTrue: 
                        [def car prettyPrintPrologOn: stream.
                        stream cr.
                        def := def cdr].
        ^stream contents asText

    "Modified: / 19.4.1998 / 15:54:13 / cg"
!

listingTextMenu
        ^[self ignoreMenu]
!

listingTextValue
        textValue isNil ifTrue: [textValue := Text new asValue].
        ^textValue
!

listingTextValueChange: aController
        ^self
!

pred
        ^predicate
!

pred: selection 
        selection isNil
                ifTrue: [predicate := nil]
                ifFalse: [predicate := PrologSymbol install: selection].
        self listingTextValue value: self listingText
!

predEditMenu
        "PrologInterpreter flushMenus."

        PredEditMenu == nil ifTrue: [PredEditMenu := Menu
                                        labels: 'save\remove\update' withCRs
                                        lines: #(2)
                                        values: #(#savePred #removePred #updatePredList)].
        ^PredEditMenu
!

predList
        | collection |
        collection := SortedCollection new.
        userPredicates associationsDo: [:assoc | collection add: assoc key string].
"/        systemPredicates associationsDo: [:assoc | collection add: assoc key string].
        ^collection asArray

    "Modified: / 19.4.1998 / 17:24:48 / cg"
!

predListMenu
        "PrologInterpreter flushMenus."

        PredListMenu == nil ifTrue: [PredListMenu := Menu
                                        labels: 'update'
                                        lines: #()
                                        values: #(#updatePredList)].
        ^PredListMenu
!

questionMenu
        "PrologInterpreter flushMenus."

        | menuBlock mainMenu subMenu |
        QuestionMenu isNil
                ifTrue: 
                        [menuBlock := 
                                        [:list | 
                                        | labels lines values index |
                                        labels := OrderedCollection new.
                                        lines := OrderedCollection new.
                                        values := OrderedCollection new.
                                        index := 0.
                                        list do: [:assoc | assoc isNil
                                                        ifTrue: [lines add: index]
                                                        ifFalse: 
                                                                [labels add: assoc key.
                                                                values add: assoc value.
                                                                index := index + 1]].
                                        (Menu
                                                labelArray: labels asArray
                                                lines: lines asArray
                                                values: values asArray) yourself].
                        subMenu := OrderedCollection new.
                        subMenu add: 'listing' -> #listing.
                        subMenu add: 'editing' -> #editing.
                        subMenu add: nil.
                        subMenu add: 'consulting' -> #consulting.
                        subMenu add: 'reconsulting' -> #reconsulting.
                        subMenu add: 'file edit' -> #fileEdit.
                        subMenu add: 'saving' -> #saving.
                        subMenu add: nil.
                        subMenu add: 'gc' -> #gc.
                        subMenu add: 'investigate' -> #investigate.
                        mainMenu := OrderedCollection new.
                        mainMenu add: 'find...' -> #find.
                        mainMenu add: 'replace...' -> #replace.
                        mainMenu add: nil.
                        mainMenu add: 'undo' -> #undo.
                        mainMenu add: nil.
                        mainMenu add: 'copy' -> #copySelection.
                        mainMenu add: 'cut' -> #cut.
                        mainMenu add: 'paste' -> #paste.
                        mainMenu add: nil.
                        mainMenu add: 'do it' -> #doIt:from:.
                        mainMenu add: 'inspect' -> #inspectIt:from:.
                        mainMenu add: nil.
                        mainMenu add: 'prolog' -> (menuBlock value: subMenu).
                        QuestionMenu := menuBlock value: mainMenu].
        ^QuestionMenu
!

reconsultingMenu
        ^self editingMenu
!

reconsultingTextMenu
        ^[self reconsultingMenu]
!

reconsultingTextValue
        textValue isNil ifTrue: [textValue := Text new asValue].
        ^textValue
!

reconsultingTextValueChange: aController 
        self acceptReconsulting: aController text
!

textCollectorView
^ textCollector.
"/        | textCollectorView |
"/
"/        textCollectorView := TextCollectorView new model: self textCollector.
"/        textCollectorView controller: TextEditorController new.
"/        textCollectorView controller keyboardProcessor: builder keyboardProcessor.
"/        textCollectorView textStyle: (TextAttributes styleNamed: #prologDefault).
"/        ^textCollectorView

    "Modified: / 18.4.1998 / 20:33:58 / cg"
!

textMenu
        ^[self questionMenu]
!

textValue
        textValue isNil ifTrue: [textValue := Text new asValue].
        ^textValue
!

textValueChange: aController 
        aController accepted ifFalse: [aController accept].
        ^self acceptQuestion: aController text
! !

!PrologInterpreter methodsFor:'binding and unbinding'!

bind: x env: xEnv and: y env: yEnv 
        |xl list |

        list := PrologList car:(self fetchValue: y env: yEnv) cdr: valueEnv.
        list := PrologList car:(PrologList car:x cdr: list) cdr:(xEnv cdr). 
        xEnv cdr: list. 
        unbindList := PrologList car:xEnv cdr: unbindList

"/        list := Prolog::PrologList car:(self fetchValue: y env: yEnv) cdr: valueEnv.
"/        xl := Prolog::PrologList car:x cdr:list.
"/        list := Prolog::PrologList car:xl cdr: xEnv cdr.
"/        xEnv cdr: list. 
"/        unbindList := Prolog::PrologList car:xEnv cdr: unbindList
!

binding: x env: xEnv 
        | assocList |
        assocList := xEnv cdr.
        assocList == nil ifTrue: [^nil].
        ^assocList assoc: x
!

fetchValue: x env: xEnv 
        | xx xxEnv assoc |

        xx := x.
        xxEnv := xEnv.
        [true] whileTrue:[ 
            valueEnv := xxEnv.
            xx class == Prolog::PrologVariable 
            "/ xx isPrologVariable 
            ifFalse:[
                ^ xx
            ].
            assoc := self binding: xx env: xxEnv.
            assoc == nil ifTrue: [^xx].
            assoc := assoc cdr.
            xx := assoc car.
            xxEnv := assoc cdr
        ]
!

noValue: x env: xEnv 
        | assoc |
        x isPrologVariable
                ifTrue: 
                        [assoc := self binding: x env: xEnv.
                        assoc == nil ifTrue: [^true].
                        assoc := assoc cdr.
                        ^self noValue: assoc car env: assoc cdr].
        ^false
!

nullEnv
        envCounter := envCounter + 1.
"/        ^ Prolog::PrologList car:envCounter.
        ^ PrologList car:envCounter cdr: nil
!

unbindFrom: start to: end 
        | list env assocList |
        list := start.
        [list ~~ end]
                whileTrue: 
                        [env := list car.
                        assocList := env cdr.
                        assocList car cdr cdr: nil.
                        env cdr: assocList cdr.
                        list := list cdr]
! !

!PrologInterpreter methodsFor:'copying'!

postCopy
        super postCopy.
        textValue := nil.
        listValue := nil.
        predicate := nil
! !

!PrologInterpreter methodsFor:'initialize-release'!

initialize
        systemPredicates := Dictionary new.
        userPredicates := Dictionary new.
        tracePredicates := Dictionary new.
        textCollector := TextCollector new.
        verbose := false.
        cutSymbol := PrologSymbol cutSymbol.
        trueSymbol := PrologSymbol trueSymbol.
        failSymbol := PrologSymbol failSymbol.
        sendSymbol := PrologSymbol sendSymbol.
        varSymbol := PrologSymbol varSymbol.
        self makeSystemPredicates
! !

!PrologInterpreter methodsFor:'interface opening'!

postOpenWith: aBuilder 
        | aSpecWrapper textCollectorView listView |
"/ ST/X change:
aBuilder componentAt: #textCollectorView put:textCollector.
^ self.
        aSpecWrapper := aBuilder componentAt: #textCollectorView.
        aSpecWrapper isNil
                ifFalse: 
                        [textCollectorView := aSpecWrapper widget.
                        textCollectorView controller deselect].
        aSpecWrapper := aBuilder componentAt: #listingTextValue.
        aSpecWrapper isNil
                ifFalse: 
                        [listView := aSpecWrapper widget.
                        listView controller: Controller new]

    "Modified: / 18.4.1998 / 20:32:04 / cg"
! !

!PrologInterpreter methodsFor:'kernel predicates'!

builtInCut
        backTrack := cutBack car.
        clause := clause cdr.
        status := #loop
!

builtInFail
        status := #back
!

builtInPredicate: functor 
        functor isPrologCut ifTrue: [^self builtInCut].
        functor isPrologTrue ifTrue: [^self builtInTrue].
        functor isPrologFail ifTrue: [^self builtInFail].
        functor isPrologSend ifTrue: [^self builtInSend].
        functor isPrologVar ifTrue: [^self builtInVar].
        self error: 'Unexpected built-in predicate'
!

builtInSend
        | list receiverSymbol selectorSymbol argumentList unifyTerm result |
        list := self expression: clause car cdr env: clauseEnv.
        receiverSymbol := list car.
        list := list cdr.
        selectorSymbol := list car.
        list := list cdr.
        argumentList := list car.
        list := list cdr.
        list == nil
                ifTrue: [unifyTerm := nil]
                ifFalse: [list cdr == nil
                                ifTrue: [unifyTerm := clause car cdr nth: 4]
                                ifFalse: 
                                        [status := #back.
                                        ^self]].
        result := self
                                receiver: receiverSymbol
                                selector: selectorSymbol
                                arguments: argumentList.
        result = cutSymbol
                ifTrue: 
                        [self builtInCut.
                        status := #back.
                        ^self].
        unifyTerm isNil
                ifTrue: 
                        [result = false ifFalse: [result := true].
                        result
                                ifTrue: 
                                        [clause := clause cdr.
                                        status := #loop.
                                        ^self]
                                ifFalse: 
                                        [status := #back.
                                        ^self]]
                ifFalse: [(self
                                unify: unifyTerm
                                env: clauseEnv
                                and: result
                                env: self nullEnv)
                                ifTrue: 
                                        [clause := clause cdr.
                                        status := #loop.
                                        ^self]
                                ifFalse: 
                                        [status := #back.
                                        ^self]].
        ^self
!

builtInTrue
        clause := clause cdr.
        status := #loop
!

builtInVar
        | var |
        var := clause car cdr car.
        (self noValue: var env: clauseEnv)
                ifTrue: 
                        [clause := clause cdr.
                        status := #loop]
                ifFalse: [status := #back]
!

call: aPrologStructure 
        | structure functor |
        aPrologStructure cdr == nil
                ifTrue: 
                        [self callVariable: aPrologStructure car.
                        ^self].
        structure := self expression: aPrologStructure env: clauseEnv.
        (self
                unify: aPrologStructure
                env: clauseEnv
                and: structure
                env: clauseEnv)
                ifFalse: [self error: 'Unexpected unify structure'].
        functor := structure car.
        functor isPrologVariable
                ifTrue: 
                        [status := #back.
                        ^self].
        clause := clause cdr.
        clause := PrologList car:structure cdr: clause.
        status := #loop
!

callVariable: aPrologVariable 
        | horn functor |
        horn := self expression: aPrologVariable env: clauseEnv.
        (self
                unify: aPrologVariable
                env: clauseEnv
                and: horn
                env: clauseEnv)
                ifFalse: [self error: 'Unexpected unify horn clause'].
        horn consp ifFalse: [horn := PrologStructure functor: horn terms: nil].
        horn car consp ifFalse: [horn := PrologClause head: horn body: nil].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        functor := horn car car.
        functor isPrologVariable
                ifTrue: 
                        [status := #back.
                        ^self].
        clause := clause cdr.
        horn := horn reverse.
        [horn consp]
                whileTrue: 
                        [clause := PrologList car:horn car cdr: clause.
                        horn := horn cdr].
        status := #loop
!

receiver: receiverSymbol selector: selectorSymbol arguments: argumentList 
        | receiver size selector arguments list result index |

        (receiverSymbol isKindOf: PrologSymbol) ifTrue:[
            (receiverSymbol isPrologSelf) ifTrue: [
                receiver := self
            ] ifFalse: [
                (receiverSymbol isPrologInstance) ifTrue: [
                    receiver := self instance
                ] ifFalse: [
                    receiver := receiverSymbol
                ]
            ].
        ] ifFalse: [
            receiver := receiverSymbol
        ].
        (receiver isKindOf: PrologObject)
                ifTrue: [receiver := receiver object].
        size := selectorSymbol size.
        selector := String new: size.
        1 to: size do: [:i | selector at: i put: (selectorSymbol at: i)].
        selector := selector asSymbol.
        list := argumentList.
        list == nil
                ifTrue: 
                        [arguments := Array new.
                        result := receiver perform: selector]
                ifFalse: 
                        [size := list length.
                        arguments := Array new: size.
                        1 to: size do:[:i | |car|
                                car := list car.
                                arguments at: i put: ((car isKindOf: PrologObject)
                                                ifTrue: [car object]
                                                ifFalse: [car]).
                                list := list cdr
                        ].
                        result := receiver perform: selector withArguments: arguments].
        ((result = true 
        or: [result = false])
        or: [result isPrologEntity])
                ifTrue: [ "result := result yourself" ]
                ifFalse: 
                        [list := PrologList car: receiverSymbol cdr: argumentList.
                        index := (Array with: receiver)
                                                , arguments findFirst: [:each | each = result].
                        index = 0
                                ifTrue: [result := PrologObject source: result printString object: result]
                                ifFalse: [result := list nth: index]].
        ^result
! !

!PrologInterpreter methodsFor:'menu messages'!

consulting
        | fileName |
        fileName := Dialog requestFileName: 'Please type a name for consulting file.' default: String new.
        (fileName isNil or: [fileName asString isEmpty or: [fileName asString = 'user']])
                ifTrue: [^self consult]
                ifFalse: [^self consultFile: fileName asString]
!

doIt: aText from: aController 
        | stream sel |
        sel := aController selection.
        sel isNil ifTrue:[^ nil].

        stream := ReadStream on: sel asString.
        ^self consult: stream action: nil

    "Modified: / 18.4.1998 / 20:09:03 / cg"
!

fileEdit
        | fileName |
        fileName := Dialog requestFileName: 'Please type a name for editting file.' default: String new.
        (fileName isNil or: [fileName isEmpty])
                ifTrue: [^false].
        (Filename named: fileName asString) edit.
        ^true
!

inspectIt: aText from: aController 
        | result stream sel |

sel := aController selection.
sel isNil ifTrue:[^ nil].

        stream := ReadStream on: aController selection asString.
        self consult: stream
                action: 
                        [:dict | 
                        result := dict.
                        self outputVariables: dict].
        result inspect.
        ^result

    "Modified: / 18.4.1998 / 20:05:35 / cg"
!

investigate
        self inspect
!

reconsulting
        | fileName |
        fileName := Dialog requestFileName: 'Please type a name for reconsulting file.' default: String new.
        (fileName isNil or: [fileName asString isEmpty or: [fileName asString = 'user']])
                ifTrue: [^self reconsult]
                ifFalse: [^self reconsultFile: fileName asString]
!

removePred
        predicate == nil ifTrue: [^self].
        (self confirm: 'Do you really want to remove?')
                ifTrue: 
                        [self remove: predicate.
                        self updatePredList]
!

savePred
        predicate == nil ifTrue: [^self].
        self saving: predicate
!

updatePredList
        self listingListValue list: self predList.
        self pred isNil
                ifFalse: 
                        [self listingListValue selection: self pred string.
                        self pred: self pred string]
! !

!PrologInterpreter methodsFor:'outputting'!

outputAnswer: dict 
        | anArray string assoc |
        anArray := dict associations asSortedCollection.
        1 to: anArray size do: [:i | 
                assoc := anArray at: i.
                i = 1
                        ifTrue: [string := '']
                        ifFalse: [string := ',\' withCRs].
                string := string , assoc key.
                string := string , ' = '.
                string := string , assoc value printPrologString.
                string := string , ' '.
                textCollector show: string; endEntry]

    "Modified: / 19.4.1998 / 13:32:52 / cg"
!

outputTime: msec 
        | goal string |
        verbose
                ifTrue: 
                        [goal := envCounter - 1.
                        string := '<'.
                        string := string , msec printString , ' milliseconds, '.
                        string := string , goal printString , ' goals'.
                        string := string , '>\' withCRs.
                        textCollector show: string asText]
!

outputVariables: aDictionary
        | bool |
        self outputAnswer: aDictionary.
        bool := self confirm: 'All right ?'.
        bool
                ifTrue: [textCollector cr]
                ifFalse: [textCollector show: ';'; cr].
textCollector endEntry.
        ^bool

    "Modified: / 19.4.1998 / 13:34:21 / cg"
! !

!PrologInterpreter methodsFor:'private'!

deallocateEnv: env
        | assocList assoc nextEnv |  
        assocList := env cdr.
        [assocList consp]
                whileTrue: 
                        [assoc := assocList car.
                        nextEnv := assoc cdr cdr.
                        assoc cdr cdr: nil.
                        nextEnv consp ifTrue: [self deallocateEnv: nextEnv].
                        assocList := assocList cdr]
!

getCondition
        | condition |
        condition := Array new: 14.
        condition at: 1 put: status.
        condition at: 2 put: definition.
        condition at: 3 put: question.
        condition at: 4 put: questionEnv.
        condition at: 5 put: clause.
        condition at: 6 put: clauseEnv.
        condition at: 7 put: queue.
        condition at: 8 put: queueEnv.
        condition at: 9 put: valueEnv.
        condition at: 10 put: unbindList.
        condition at: 11 put: cutBack.
        condition at: 12 put: backTrack.
        condition at: 13 put: resolveAction.
        condition at: 14 put: definitionStack.
        ^condition
!

makeSystemPredicates
        SystemPredicates isNil
                ifTrue: 
                        [self systemPredicatesNo0.
                        self systemPredicatesNo1.
                        self systemPredicatesNo2.
                        self systemPredicatesNo3.
                        self systemPredicatesNo3b.
                        self systemPredicatesNo4.
                        self systemPredicatesNo5.
                        self systemPredicatesNo6.
                        self systemPredicatesNo7.
                        self systemPredicatesNo8.
                        self systemPredicatesNo9.
                        SystemPredicates := systemPredicates]
                ifFalse: [systemPredicates := SystemPredicates]
!

putCondition: condition 
        status := condition at: 1.
        definition := condition at: 2.
        question := condition at: 3.
        questionEnv := condition at: 4.
        clause := condition at: 5.
        clauseEnv := condition at: 6.
        queue := condition at: 7.
        queueEnv := condition at: 8.
        valueEnv := condition at: 9.
        unbindList := condition at: 10.
        cutBack := condition at: 11.
        backTrack := condition at: 12.
        resolveAction := condition at: 13.
        definitionStack := condition at: 14
!

systemPredicatesNo0
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                !! :- builtin.
                true :- builtin.
                fail :- builtin.
                var(X) :- builtin.
                send(X,Y,Z) :- builtin.
                send(X,Y,Z,A) :- builtin.
        ')
!

systemPredicatesNo1
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                repeat.
                repeat :- repeat.
                nonvar(X) :- var(X), !!, fail.
                nonvar(X).
                integer(X) :- send(self,integer:,[X]).
                float(X) :- send(self,float:,[X]).
                double(X) :- send(self,double:,[X]).
                fraction(X) :- send(self,fraction:,[X]).
                number(X) :- send(self,number:,[X]).
                symbol(X) :- send(self,symbol:,[X]).
                string(X) :- send(self,string:,[X]).
                list(X) :- send(self,list:,[X]).
                dotp(X) :- send(self,dotp:,[X]).
                atom(X) :- symbol(X).
                atom(X) :- nonvar(X), =(X,[]).
                atom(X) :- string(X).
                atomic(X) :- atom(X).
                atomic(X) :- number(X).
                structure(X) :- nonvar(X), not(atomic(X)).
        ')
!

systemPredicatesNo2
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                ==(X,Y) :- send(X,=,[Y]).
                \==(X,Y) :- ==(X,Y), !!, fail.
                \==(X,Y).
                =(X,X).
                \=(X,Y) :- =(X,Y), !!, fail.
                \=(X,Y).
                >(X,Y) :- send(X,>,[Y]).
                >=(X,Y) :- send(X,>=,[Y]).
                <(X,Y) :- send(X,<,[Y]).
                =<(X,Y) :- send(X,<=,[Y]).
        ')
!

systemPredicatesNo3
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                +(X,Y,Z) :- nonvar(X), nonvar(Y), send(X,+,[Y],Z).
                +(X,Y,Z) :- nonvar(X), nonvar(Z), send(Z,-,[X],Y).
                +(X,Y,Z) :- nonvar(Y), nonvar(Z), send(Z,-,[Y],X).
                +(X,X,Z) :- nonvar(Z), send(Z,/,[2],X).

                -(X,Y,Z) :- nonvar(X), nonvar(Y), send(X,-,[Y],Z).
                -(X,Y,Z) :- nonvar(X), nonvar(Z), send(X,-,[Z],Y).
                -(X,Y,Z) :- nonvar(Y), nonvar(Z), send(Z,+,[Y],X).

                *(X,Y,Z) :- nonvar(X), nonvar(Y), send(X,*,[Y],Z).
                *(X,Y,Z) :- nonvar(X), nonvar(Z), send(Z,/,[X],Y).
                *(X,Y,Z) :- nonvar(Y), nonvar(Z), send(Z,/,[Y],X).
                *(X,X,Z) :- nonvar(Z), send(Z,sqrt,[], X).

                //(X,Y,Z) :- nonvar(X), nonvar(Y), send(X,//,[Y],Z).
%                //(X,Y,Z) :- nonvar(X), nonvar(Z), send(X,//,[Z],Y).
                /(X,Y,Z) :- nonvar(X), nonvar(Y), send(X,/,[Y],Z).
                /(X,Y,Z) :- nonvar(X), nonvar(Z), send(X,/,[Z],Y).
                /(X,Y,Z) :- nonvar(Y), nonvar(Z), send(Y,*,[Z],X).
                \\(X,Y,Z) :- send(X,\\,[Y],Z).

                is(Z,+(X,Y)) :- nonvar(Z), nonvar(X), nonvar(Y), send(X,+,[Y],Z).
                is(Z,+(X,Y)) :- var(Z), nonvar(X), nonvar(Y), send(X,+,[Y],Z).
                is(Z,+(X,Y)) :- var(X), nonvar(Y), nonvar(Z), send(Z,-,[Y],X).
                is(Z,+(X,Y)) :- var(Y), nonvar(Z), nonvar(X), send(Z,-,[X],Y).

                is(Z,-(X,Y)) :- nonvar(Z), nonvar(X), nonvar(Y), send(X,-,[Y],Z).
                is(Z,-(X,Y)) :- var(Z), nonvar(X), nonvar(Y), send(X,-,[Y],Z).
                is(Z,-(X,Y)) :- var(X), nonvar(Y), nonvar(Z), send(Z,+,[Y],X).
                is(Z,-(X,Y)) :- var(Y), nonvar(Z), nonvar(X), send(X,-,[Z],Y).

                is(Z,*(X,Y)) :- nonvar(Z), nonvar(X), nonvar(Y), send(X,*,[Y],Z).
                is(Z,*(X,Y)) :- var(Z), nonvar(X), nonvar(Y), send(X,*,[Y],Z).
                is(Z,*(X,Y)) :- var(X), nonvar(Y), nonvar(Z), send(Z,/,[Y],X).
                is(Z,*(X,Y)) :- var(Y), nonvar(Z), nonvar(X), send(Z,/,[X],Y).

                is(Z,/(X,Y)) :- nonvar(Z), nonvar(X), nonvar(Y), send(X,/,[Y],Z).
                is(Z,/(X,Y)) :- var(Z), nonvar(X), nonvar(Y), send(X,/,[Y],Z).
                is(Z,/(X,Y)) :- var(X), nonvar(Y), nonvar(Z), send(Z,*,[Y],X).
                is(Z,/(X,Y)) :- var(Y), nonvar(Z), nonvar(X), send(X,/,[Z],Y).

                is(X,Y) :- =(X,Y).
        ')
!

systemPredicatesNo3b
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                sin(X,Y) :- nonvar(X), send(X,sin,[],Y).
                sin(X,Y) :- nonvar(Y), send(Y,arcSin,[],X).

                cos(X,Y) :- nonvar(X), send(X,cos,[],Y).
                cos(X,Y) :- nonvar(Y), send(Y,arcCos,[],X).

                tan(X,Y) :- nonvar(X), send(X,tan,[],Y).
                tan(X,Y) :- nonvar(Y), send(Y,arcTan,[],X).

                ln(X,Y) :- nonvar(X), send(X,ln,[],Y).
                ln(X,Y) :- nonvar(Y), send(Y,exp,[],X).

                exp(X,Y) :- nonvar(X), send(X,exp,[],Y).
                exp(X,Y) :- nonvar(Y), send(Y,ln,[],X).

                sqrt(X,Y) :- nonvar(X), send(X,sqrt,[],Y).
                sqrt(X,Y) :- nonvar(Y), send(Y,*,[Y],X).
        ')
!

systemPredicatesNo4
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                listing :- send(self,userListing,[]).
                listing(X) :- send(self,userListing:,[X]).
                systemListing :- send(self,systemListing,[]).
                systemListing(X) :- send(self,systemListing:,[X]).
                editing :- send(self,editting,[]).
                consult :- send(self,consult,[]).
                consult(X) :- nonvar(X), send(self,consultFile:,[X]).
                reconsult :- send(self,reconsult,[]).
                reconsult(X) :- nonvar(X), send(self,reconsultFile:,[X]).
                saving :- send(self,saving,[]).
                saving(X) :- send(self,saving:,[X]).
                userPredicates(X) :- send(self,userPredicates,[],X).
                systemPredicates(X) :- send(self,systemPredicates,[],X).
                predicates([X|Y]) :- userPredicates(X), systemPredicates(Y).
                functor(T,F,A) :- nonvar(T), !!, send(self,functorArityOf:,[T],[F|A]).
                functor(T,F,A) :- number(F), !!, =(0,A),=(T,F).
                functor(T,F,A) :- atom(F), =<(0,A), ''~addvar''([F],A,L), =..(T,L).
                ''~addvar''(L,0,M) :- !!, =(L,M).
                ''~addvar''(L,NVars,M) :- -(NVars, 1, N), append(L,[FreeV],LV), ''~addvar''(LV,N,M).
                arg(Nth,S,T) :- integer(Nth), <(0,Nth), structure(S), =..(S,[F|L]), nth(L,Nth,T).
                =..(X,Y) :- send(self,univ:,[[''X''|''Y'']]).
                name(X,Y) :- atomic(X), list(Y), !!, send(self,symToList:,[X],Y).
                name(X,Y) :- atomic(X), !!, send(self,symToStr:,[X],Y).
                name(X,Y) :- var(X), !!, nonvar(Y), send(self, strToSym:,[Y],X).
                remove :- send(self,remove,[]).
                remove(X) :- send(self,remove:,[X]).
                clause(X) :- send(self,clauseSet:,[X]), repeat, send(self,clause:,[X],X).
                asserta(X) :- send(self,asserta:,[X]).
                assert(X) :- send(self,assertz:,[X]).
                assertz(X) :- send(self,assertz:,[X]).
                retract(X) :- repeat, send(self,retract:,[X],X).
        ')
!

systemPredicatesNo5
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                call(G) :- G.
                not(G) :- G, !!, fail.
                not(G).
                or(X,Y) :- call(X).
                or(X,Y) :- call(Y).
                and(X,Y) :- call(X), call(Y).
        ')
!

systemPredicatesNo6
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                read(X) :- send(self,read,[],X).
                read(X,M) :- send(self,read:,[M],X).
                readln(X) :- send(self,readln,[],X).
                readln(X,M) :- send(self,readln:,[M],X).
                write(X) :- send(self,write:,[X]).
                nl :- send(self,nl,[]).
                writeln(X) :- send(self,writeln:,[X]).
                tab(X) :- number(X), send(self,tab:,[X]).
                display(X) :- send(self,display:,[X]).
                clear :- send(self,clear,[]).
        ')
!

systemPredicatesNo7
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                clock(X) :- send({Time},millisecondClockValue,[],X).
                verbose(X) :- send(self,verbose:,[X]).
                gc :- send(self,gc,[]).
                inspect(X) :- send(X,inspect,[]).
                spy(X) :- send(self,spy:,[X]).
                nospy(X) :- send(self,nospy:,[X]).
                trace :- send(self,trace,[]).
                notrace :- send(self,notrace,[]).
        ')
!

systemPredicatesNo8
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                append([],X,X).
                append([A|X],Y,[A|Z]) :- append(X,Y,Z).
                member(X,[X|Y]).
                member(X,[Y|Z]) :- member(X,Z).
                reverse([],[]).
                reverse([H|T],L) :- reverse(T,Z), append(Z,[H],L).
                length(X,Y) :- send(self,length:,[X],Y).
                nth([X|Y],1,Z) :- !!, =(X,Z).
                nth([X|Y],N,Z) :- -(N,1,PN), nth(Y,PN,Z).
                printlist(L) :- send(self,listPrint:,[L]).
                lispAppend(X,Y,Z) :- send(X,append:,[Y],Z).
                lispReverse(X,Y) :- send(X,reverse,[],Y).
                lispMember(X,Y) :- send(Y,member:,[X],A), \=(A,[]).
                lispMember(X,Y,Z) :- send(Y,member:,[X],Z).
                lispAssoc(X,Y) :- send(Y,assoc:,[X],A), \=(A,[]).
                lispAssoc(X,Y,Z) :- send(Y,assoc:,[X],Z).
                lispNconc(X,Y,Z) :- send(X,nconc:,[Y],Z).
        ')
!

systemPredicatesNo9
        "PrologInterpreter flushSystemPredicates."

        self systemReconsult: (ReadStream on: '
                %
                %       User System Predicates
                %
        ')
! !

!PrologInterpreter methodsFor:'public accessing'!

cleanup
    self resolveInitialize
!

confirm: messageString 
        ^Dialog confirm: messageString initialAnswer: false
!

refute: stringOrStream
        ^self refute: stringOrStream action: [:answer | ^answer]
!

refute: stringOrStream action: aBlock 
        | stream result |
        (stringOrStream isKindOf: Stream)
                ifTrue: [stream := ReadStream on: stringOrStream contents]
                ifFalse: [stream := ReadStream on: stringOrStream].

        self withExecuteCursorDo:[
            result := self consult: stream action: aBlock
        ].
        ^ result
! !

!PrologInterpreter methodsFor:'representation'!

collectVariables
        | dict | 
        dict := Dictionary new.
        self collectVariables: question to: dict.
        ^dict
!

collectVariables: x to: dict
        | key value |  
        (x isPrologVariable and: [(x at: 1) ~= $~])
                ifTrue:
                        [key := String fromString: x printPrologString.
                        (dict at: key ifAbsent: []) ~~ nil
                                ifTrue: [^self]
                                ifFalse: 
                                        [value := self represent: x env: questionEnv.
                                        dict at: key put: value]].
        (x consp)
                ifFalse: [^self].
        self collectVariables: x car to: dict.
        self collectVariables: x cdr to: dict
!

expression: x env: xEnv 
        | assoc a d |
        x isPrologVariable
                ifTrue: 
                        [assoc := self binding: x env: xEnv.
                        assoc == nil ifTrue: [^x].
                        assoc := assoc cdr.
                        ^self expression: assoc car env: assoc cdr].
        (x consp) ifFalse: [^x].
        a := self expression: x car env: xEnv.
        d := self expression: x cdr env: xEnv.
        ^x class car: a cdr: d
!

represent: xIn env: xEnvIn
        | assoc a d x xEnv|

        x := xIn.
        xEnv := xEnvIn.

        [x isPrologVariable] whileTrue:[ 
            assoc := self binding: x env: xEnv.
            assoc == nil ifTrue: [^ self variableRepresent: x env: xEnv].
            assoc := assoc cdr.
            "/ tail recursion ...
            x := assoc car.
            xEnv := assoc cdr.
"/            ^ self represent: assoc car env: assoc cdr
        ].
        (x consp) ifFalse: [^x].
        a := self represent: x car env: xEnv.
        d := self represent: x cdr env: xEnv.
        ^x class car: a cdr: d
!

variableRepresent: x env: xEnv 
        ^PrologVariable install: x string , xEnv car printString
! !

!PrologInterpreter methodsFor:'resolution'!

resolve: goal 
        ^self resolve: goal action: nil
!

resolve: goal action: actionBlock
        question := clause := goal.
        actionBlock == nil
                ifTrue: [resolveAction := [:dict | self outputVariables: dict]]
                ifFalse: [resolveAction := actionBlock].

        self resolveInitialize.
        self resolveLoop.
        ^ self resolveTerminate
!

resolveInitialize
        status := #loop.
        envCounter := 0.
        questionEnv := clauseEnv := self nullEnv.
        queue := queueEnv := valueEnv := nil.
        unbindList := cutBack := backTrack := nil.
        definitionStack := nil.
        traceCounter := 0
!

resolveLoop
        | totalTime time answer |

        totalTime := 0.
        time := Time millisecondClockValue.
        [true] whileTrue:[
            status == #loop ifTrue: [self loop].
            status == #next ifTrue: [self next].
            status == #back ifTrue: [self back].
            status == #succ ifTrue: [ 
                verbose ifTrue:[
                    time := Time millisecondClockValue - time.
                    self outputTime: (totalTime := totalTime + time).
                ].
                answer := self collectVariables.
                answer isEmpty ifTrue: [^true].
                (resolveAction value: answer) ifTrue: [^true].
                status := #back.
                verbose ifTrue:[
                    time := Time millisecondClockValue.
                ]
            ].
            status == #fail ifTrue:[    
                verbose ifTrue:[
                    time := Time millisecondClockValue - time.
                    self outputTime: (totalTime := totalTime + time).
                ].
                ^false
            ]
        ]
!

resolveTerminate
        self deallocateEnv: questionEnv.
        status == #succ ifTrue:[
            textCollector notNil ifTrue:[
                textCollector show: 'yes'.
                textCollector cr.
            ].
            ^true
        ].
        status == #fail ifTrue:[
            textCollector notNil ifTrue:[
                textCollector show: 'no'.
                textCollector cr.
            ].
            ^false
        ].
        self error: 'Unexpected status'

    "Modified: / 20.4.1998 / 12:22:01 / cg"
! !

!PrologInterpreter methodsFor:'resolve modules'!

back
        | array |
        backTrack == nil ifTrue:[
            status := #fail.
            ^self
        ].
        array := backTrack car.
        backTrack := backTrack cdr.
        clause := array at: 1.
        clauseEnv := array at: 2.
        queue := array at: 3.
        queueEnv := array at: 4.
        cutBack := array at: 5.
        definition := array at: 6.
        array := array at: 7.
        self unbindFrom: unbindList to: array.
        unbindList := array.
        status := #next
!

loop
        | structure functor |

        clause == nil ifTrue:[ 
            queue == nil ifTrue:[ 
                status := #succ.
                ^ self
            ].
            clause := queue car.
            clauseEnv := queueEnv car.
            queue := queue cdr.
            queueEnv := queueEnv cdr.
            cutBack := cutBack cdr.

            "/ cg: optimized: stay here.
            thisContext restart.
"/            status := #loop.
"/            ^ self
        ].
        structure := clause car.
        structure consp ifFalse:[ 
            structure := PrologStructure functor: structure terms: nil.
            clause car: structure
        ].
        functor := structure car.
        functor isPrologVariable ifTrue:[
            self call: clause car.
            ^ self
        ].
"/        functor isBuiltInPredicate ifTrue:[ 
"/            self builtInPredicate: functor.
"/            self error: 'Unexpected built-in predicate'.
"/            ^ self
"/        ].
        "/ cg: inlined; no need to compare twice.
        functor == cutSymbol ifTrue: [self builtInCut. ^ self].
        functor == trueSymbol ifTrue: [self builtInTrue. ^ self].
        functor == failSymbol ifTrue: [self builtInFail. ^ self].
        functor == sendSymbol ifTrue: [self builtInSend. ^ self].
        functor == varSymbol ifTrue: [self builtInVar. ^ self].

"/        functor = cutSymbol ifTrue: [self halt].
"/        functor = trueSymbol ifTrue: [self halt].
"/        functor = failSymbol ifTrue: [self halt].
"/        functor = sendSymbol ifTrue: [self halt].
"/        functor = varSymbol ifTrue: [self halt].

        definition := userPredicates at: functor ifAbsent:nil.
        definition isNil ifTrue:[
            definition := systemPredicates at: functor ifAbsent:nil.
            definition isNil ifTrue:[
                Transcript showCR:'no predicate: ', functor.
                status := #back.
                ^ self
            ]
        ].

"/        definition := userPredicates at: functor 
"/                        ifAbsent: [systemPredicates at: functor
"/                                        ifAbsent: 
"/                                                [status := #back.
"/                                                ^self]].
        status := #next
!

next
        | definitionEnv saveBackTrack array |
        definitionEnv := self nullEnv.
        definition cdr == nil
                ifTrue: 
                        [(self
                                unify: clause car cdr
                                env: clauseEnv
                                and: definition car car cdr
                                env: definitionEnv)
                                ifFalse: 
                                        [status := #back.
                                        ^self].
                        queue := PrologList car:(clause cdr) cdr: queue.
                        queueEnv := PrologList car:clauseEnv cdr: queueEnv.
                        clause := definition car cdr.
                        clauseEnv := definitionEnv.
                        cutBack := PrologList car:backTrack cdr: cutBack.
                        status := #loop.
                        ^self].
        saveBackTrack := backTrack.
        array := Array new: 7.
        array at: 1 put: clause.
        array at: 2 put: clauseEnv.
        array at: 3 put: queue.
        array at: 4 put: queueEnv.
        array at: 5 put: cutBack.
        array at: 6 put: definition cdr.
        array at: 7 put: unbindList.
        backTrack := PrologList car:array cdr: saveBackTrack.
        (self
                unify: clause car cdr
                env: clauseEnv
                and: definition car car cdr
                env: definitionEnv)
                ifFalse: 
                        [status := #back.
                        ^self].
        queue := PrologList car:clause cdr cdr: queue.
        queueEnv := PrologList car:clauseEnv cdr: queueEnv.
        clause := definition car cdr.
        clauseEnv := definitionEnv.
        cutBack := PrologList car:saveBackTrack cdr: cutBack.
        status := #loop
! !

!PrologInterpreter methodsFor:'system predicates'!

asserta: aPrologClause 
        | horn def functor |
        horn := aPrologClause.
        horn consp ifFalse: [horn := PrologStructure functor: horn terms: nil].
        horn car consp ifFalse: [horn := PrologClause head: horn body: nil].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        functor := horn car car.
        def := userPredicates at: functor ifAbsent: [nil].
        def == nil
                ifTrue: 
                        [def := PrologDefinition fromList: (horn cons: nil).
                        userPredicates at: functor put: def]
                ifFalse: 
                        [def := PrologDefinition fromList: (horn cons: def).
                        userPredicates at: functor put: def].
        ^true
!

assertz: aPrologClause 
        | horn def functor |
        horn := aPrologClause.
        horn consp ifFalse: [horn := PrologStructure functor: horn terms: nil].
        horn car consp ifFalse: [horn := PrologClause head: horn body: nil].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        functor := horn car car.
        def := userPredicates at: functor ifAbsent: [nil].
        def == nil
                ifTrue: 
                        [def := PrologDefinition fromList: (horn cons: nil).
                        userPredicates at: functor put: def]
                ifFalse: [def nconc: (PrologDefinition fromList: (horn cons: nil))].
        ^true
!

clause: aPrologClause 
        | count horn def result |
        count := 0.
        horn := aPrologClause.
        horn consp
                ifFalse: 
                        [horn := PrologStructure functor: horn terms: nil.
                        count := count + 1].
        horn car consp
                ifFalse: 
                        [horn := PrologClause head: horn body: nil.
                        count := count + 1].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        def := definitionStack car.
        [def consp]
                whileTrue: [(self
                                unify: horn
                                env: self nullEnv
                                and: def car
                                env: self nullEnv)
                                ifTrue: 
                                        [result := def car.
                                        count timesRepeat: [result := result car].
                                        def := def cdr.
                                        definitionStack car: def.
                                        ^result]
                                ifFalse: 
                                        [def := def cdr.
                                        definitionStack car: def]].
        definitionStack := definitionStack cdr.
        ^ cutSymbol
!

clear
        textCollector clear.
        ^true
!

consult
        | aModel |
        aModel := self copy.
        aModel openInterface: #consultingWindowSpec.
        ^true
!

consultFile: fileName 
        | aFilename aFileStream |
        (fileName respondsTo: #asString)
                ifFalse: [^false].
        aFilename := Filename named: fileName asString.
        aFilename exists
                ifTrue: 
                        [textCollector show: '<consult from ' , fileName asString , '>\' withCRs.
                        aFileStream := aFilename readStream.
                        [self consult: aFileStream action: nil]
                                valueNowOrOnUnwindDo: [aFileStream close].
                        ^true]
                ifFalse: [^false]
!

display: anObject
    |s|

    (anObject respondsTo: #asString) ifTrue: [
        s := anObject asString
    ]  ifFalse: [
        s := anObject printPrologString
    ].
"/    (anObject respondsTo: #printPrologString) ifTrue: [
"/        s := anObject printPrologString
"/    ]  ifFalse: [
"/        s := anObject asString
"/    ].
    self information:s.
    ^ true
!

dotp: x 
        ^x consp
!

double: x
        ^x isKindOf: Double
!

editing
        | aModel |
        aModel := self copy.
        aModel openInterface: #editingWindowSpec.
        ^true
!

float: x
        ^x isKindOf: Float
!

fraction: x
        ^x isKindOf: Fraction
!

functorArityOf: aPrologStructure 
        aPrologStructure isNil ifTrue: [^PrologList car:nil cdr: 0].
        (self number: aPrologStructure)
                ifTrue: [^PrologList car:aPrologStructure cdr: 0].
        (aPrologStructure respondsTo: #functor)
                ifFalse: [^nil].
        ^PrologList car:(aPrologStructure functor) cdr:(aPrologStructure arity)
!

gc
        ObjectMemory globalCompactingGC.
        ^true
!

integer: x
        ^x isInteger
!

length: aPrologDotPair 
        aPrologDotPair consp ifFalse: [^0].
        ^aPrologDotPair length
!

list: x 
        x == nil ifTrue: [^true].
        ^self dotp: x
!

listPrint: aPrologDotPair 
        | list |
        aPrologDotPair consp ifFalse: [^false].
        list := aPrologDotPair.
        [list consp]
                whileTrue: 
                        [textCollector show: list car printPrologString , ' '.
                        list := list cdr].
        ^true
!

listing
        | aModel |
        aModel := self copy.
        aModel openInterface: #listingWindowSpec.
        ^true
!

nl
        (textCollector ? Transcript) cr.
        ^true
!

nospy: aPrologSymbol 
        | def |
        def := tracePredicates at: aPrologSymbol ifAbsent: [nil].
        def == nil ifTrue: [^false].
        userPredicates at: aPrologSymbol put: def.
        tracePredicates removeKey: aPrologSymbol.
        ^true
!

notrace
        userPredicates associationsDo: [:assoc | self nospy: assoc key].
        ^true
!

number: x
        ^x isKindOf: Number
!

read
        ^self read: 'Type prolog expression followed by carriage return.'
!

read: aMessage 
        | parser stream answer |

        (instance notNil
        and:[instance window notNil]) ifTrue:[
            answer := Dialog 
                        request:aMessage asString 
                        displayAt:(instance window topRight) 
                        centered:false 
                        action:nil 
                        initialAnswer:String new
                        onCancel:''
        ] ifFalse:[
            answer := Dialog request: aMessage asString initialAnswer: String new.
        ].
        answer isNil ifTrue:[
            "/ cancel pressed; simulate a fail.
            ^ false
        ].
        
        parser := PrologParser new.
        stream := ReadStream on: answer asString.
        parser init: stream ifFail: [:errorMessage | ^self error: errorMessage].
        ^ parser scanExpression
!

readln
        ^self readln: 'Enter a line:'
!

readln: aMessage 
        | answer |
        answer := Dialog request: aMessage asString initialAnswer: String new.
        ^ PrologString fromString: answer
!

reconsult
        | aModel |
        aModel := self copy.
        aModel openInterface: #reconsultingWindowSpec.
        ^true
!

reconsultFile: fileName 
        | aFilename aFileStream |
        (fileName respondsTo: #asString)
                ifFalse: [^false].
        aFilename := Filename named: fileName asString.
        aFilename exists
                ifTrue: 
                        [textCollector showCR: '<reconsult from ' , fileName asString , '>'.
                        aFileStream := aFilename readStream.
                        [self reconsult: aFileStream action: nil]
                                valueNowOrOnUnwindDo: [aFileStream close].
                        ^true]
                ifFalse: [^false]

    "Modified: / 19.4.1998 / 13:36:36 / cg"
!

remove
        userPredicates keys do: 
                [:aPrologSymbol | 
                userPredicates removeKey: aPrologSymbol ifAbsent: [].
                tracePredicates removeKey: aPrologSymbol ifAbsent: []].
        ^true
!

remove: aPrologSymbol
        userPredicates removeKey: aPrologSymbol ifAbsent: [^false].
        tracePredicates removeKey: aPrologSymbol ifAbsent: [].
        ^true
!

retract: aPrologClause 
        | count horn def functor prev result |
        count := 0.
        horn := aPrologClause.
        horn consp
                ifFalse: 
                        [horn := PrologStructure functor: horn terms: nil.
                        count := count + 1].
        horn car consp
                ifFalse: 
                        [horn := PrologClause head: horn body: nil.
                        count := count + 1].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        functor := horn car car.
        prev := def := userPredicates at: functor ifAbsent: [^ cutSymbol].
        [def consp]
                whileTrue: [(self
                                unify: horn
                                env: self nullEnv
                                and: def car
                                env: self nullEnv)
                                ifTrue: 
                                        [result := def car.
                                        count timesRepeat: [result := result car].
                                        prev == def
                                                ifTrue: 
                                                        [def cdr isNil
                                                                ifTrue: [userPredicates removeKey: functor ifAbsent: [nil]]
                                                                ifFalse: [userPredicates at: functor put: def cdr].
                                                        ^result]
                                                ifFalse: 
                                                        [prev cdr: def cdr.
                                                        ^result]]
                                ifFalse: 
                                        [prev := def.
                                        def := def cdr]].
        ^cutSymbol
!

saving
        | fileName |
        fileName := Dialog requestNewFileName: 'Please type a name for saving file.' default: String new.
        (fileName isNil or: [fileName isEmpty])
                ifTrue: [^false].
        self withCursor:Cursor write do:[
            self saveOn: fileName asString
        ].
        ^true
!

saving: aPrologSymbol 
        | def fileName |
        def := tracePredicates at: aPrologSymbol ifAbsent: [userPredicates at: aPrologSymbol ifAbsent: []].
        def == nil ifTrue: [^false].
        fileName := Dialog requestNewFileName: 'Please type a name for saving file.' default: String new.
        (fileName isNil or: [fileName isEmpty])
                ifTrue: [^false].
        self saveOn: fileName predicateName: aPrologSymbol.
        ^true
!

spy: aPrologSymbol 
        | def reverseClone tracedef |
        def := userPredicates at: aPrologSymbol ifAbsent: [nil].
        def == nil ifTrue: [^false].
        (tracePredicates at: aPrologSymbol ifAbsent: [nil]) isNil ifFalse: [^true].
        tracePredicates at: aPrologSymbol put: def.
        reverseClone := nil.
        [def consp]
                whileTrue: 
                        [reverseClone := def car cons: reverseClone.
                        def := def cdr].
        tracedef := nil.
        [reverseClone consp]
                whileTrue: 
                        [tracedef := (self traceFailClause: reverseClone car car)
                                                cons: tracedef.
                        tracedef := (self traceExitClause: reverseClone car)
                                                cons: tracedef.
                        reverseClone cdr consp
                                ifTrue: [tracedef := (self traceRedoClause: reverseClone car car)
                                                                cons: tracedef]
                                ifFalse: [tracedef := (self traceCallClause: reverseClone car car)
                                                                cons: tracedef].
                        reverseClone := reverseClone cdr].
        userPredicates at: aPrologSymbol put: (PrologDefinition fromList: tracedef).
        ^true
!

strToSym: listOrString 
        | stream string token s |

        listOrString consp ifTrue: [
            s := WriteStream on: (String new: 16).
            listOrString do: [:char | s nextPut: (Character value: char)].
            string := s contents
        ] ifFalse: [
            string := listOrString string
        ].

        stream := ReadStream on: string.
        token := (PrologScanner new on: stream) nextToken.
        ((self number: token) and: [stream atEnd])
            ifTrue: [^token].
        string = '[]'
            ifTrue: [^nil].
        ^PrologSymbol install: string
!

string: x
        ^x class == PrologString
!

symToList: numOrSym 
        | pstring |
        pstring := self symToStr: numOrSym.
        ^PrologList list: pstring string asIntegerArray
!

symToStr: numOrSym 
        numOrSym isNil ifTrue: [^PrologString fromString: '[]'].
        (self number: numOrSym)
                ifTrue: [^PrologString fromString: numOrSym printString].
        ^PrologString fromString: numOrSym string
!

symbol: x
        ^x class == PrologSymbol
!

systemListing
        | predicateName |
        systemPredicates associations asSortedCollection
                do: 
                        [:assoc | 
                        predicateName := assoc key.
                        ((predicateName respondsTo: #asString)
                                and: [predicateName string isEmpty not and: [predicateName string first = $~]])
                                ifFalse: [self systemListing: predicateName]].
        ^true
!

systemListing: aPrologSymbol
        | def |
        def := systemPredicates at: aPrologSymbol ifAbsent: [nil].
        def == nil ifTrue: [^false].
        [def consp]
                        whileTrue: 
                                [textCollector show: def car prettyPrintPrologString , '\' withCRs.
                                def := def cdr].
        ^true
!

systemPredicates
        | collection list |
        collection := systemPredicates associations asSortedCollection.
        list := nil.
        collection reverseDo: [:each | ((each key respondsTo: #asString)
                        and: [each key string isEmpty not and: [each key string first = $~]])
                        ifTrue: ['not adding: invisible predicate' yourself]
                        ifFalse: [list := each key cons: list]].
        ^list
!

tab: aNumber 
        | spaces |
        (aNumber isKindOf: Number)
                ifFalse: [^false].
        spaces := String new.
        aNumber asInteger timesRepeat: [spaces := spaces , ' '].
        textCollector show: spaces.
        ^true
!

trace
        userPredicates associationsDo: [:assoc | self spy: assoc key].
        ^true
!

univ: aPrologList 
        | termv listv termValue termEnv listVal listEnv |
        termv := PrologVariable install: aPrologList car string.
        listv := PrologVariable install: aPrologList cdr string.
        termValue := self fetchValue: termv env: clauseEnv.
        termEnv := valueEnv.
        listVal := self fetchValue: listv env: clauseEnv.
        listEnv := valueEnv.
        termValue isPrologVariable
                ifTrue: 
                        ["construct a term from fixed length list."
                        | functor functorEnv terms newterms newStruct |
                        listVal isPrologVariable ifTrue: [^false].
                        listVal consp ifFalse: [^false].
                        functor := self fetchValue: listVal car env: listEnv.
                        functorEnv := valueEnv.
                        terms := self fetchValue: listVal cdr env: listEnv.
                        listEnv := valueEnv.
                        (self number: functor)
                                ifTrue: 
                                        [terms isNil ifFalse: [^false].
                                        self
                                                bind: termValue
                                                env: termEnv
                                                and: functor
                                                env: listEnv.
                                        ^true].
                        functor consp ifTrue: ["It's a structure or a list"
                                ^false].
                        terms isNil
                                ifTrue: 
                                        [self
                                                bind: termValue
                                                env: termEnv
                                                and: functor
                                                env: functorEnv.
                                        ^true].
                        terms consp ifFalse: ["Not a list, but an illegal dot pair."
                                ^false].
                        newterms := nil.
                        [terms isNil]
                                whileFalse: 
                                        [| car t |
                                        car := terms car.
                                        terms := self fetchValue: terms cdr env: listEnv.
                                        terms isPrologVariable ifTrue: ["The length of the list has not been fixed yet."
                                                ^false].
                                        t := PrologTerms car: car cdr: nil.
                                        newterms isNil
                                                ifTrue: [newterms := t]
                                                ifFalse: [newterms := newterms nconc: t]].
                        newStruct := PrologStructure functor: functor terms: newterms.
                        ^self
                                unify: termValue
                                env: termEnv
                                and: newStruct
                                env: listEnv]
                ifFalse: 
                        ["disasemble a term into a list"
                        | list |
                        termValue consp
                                ifTrue: ["It's a list or a structure."
                                        list := termValue structureList]
                                ifFalse: ["primitive data like symbol, number, nil, or string."
                                        list := PrologList car:termValue cdr: nil].
                        ^self
                                unify: list
                                env: termEnv
                                and: listVal
                                env: listEnv]
!

userListing
        | predicateName |
        (userPredicates associations asSortedCollection)
                do: 
                        [:assoc | 
                        predicateName := assoc key.
                        self userListing: predicateName].
        ^true
!

userListing: aPrologSymbol 
        | def |
        def := userPredicates at: aPrologSymbol ifAbsent: [nil].
        def == nil ifTrue: [^false].
        [def consp]
                whileTrue: 
                        [textCollector show: def car prettyPrintPrologString , '\' withCRs.
                        def := def cdr].
        ^true
!

userPredicates
        | collection list |
        collection := userPredicates associations asSortedCollection.
        list := nil.
        collection reverseDo: [:each | ((each key respondsTo: #string)
                        and: [each key string isEmpty not and: [each key string first = $~]])
                        ifTrue: ['not adding: invisible predicate' yourself]
                        ifFalse: [list := each key cons: list]].
        ^list
!

verbose: aPrologSymbol 
        aPrologSymbol isPrologTrue
                ifTrue: [verbose := true]
                ifFalse: [verbose := false].
        ^true
!

write: anObject
    |s|

    (anObject respondsTo: #asString) ifTrue: [
        s := anObject asString
    ]  ifFalse: [
        s := anObject printPrologString
    ].
"/    (anObject respondsTo: #printPrologString) ifTrue: [
"/        s := anObject printPrologString
"/    ]  ifFalse: [
"/        s := anObject asString
"/    ].
    (textCollector ? Transcript) show:s.
    ^true
!

writeln: anObject 
        self write: anObject.
        (textCollector ? Transcript) cr.
        ^true
! !

!PrologInterpreter methodsFor:'system support'!

clauseSet: aPrologClause 
        | horn functor def |
        horn := aPrologClause.
        horn consp ifFalse: [horn := PrologStructure functor: horn terms: nil].
        horn car consp ifFalse: [horn := PrologClause head: horn body: nil].
        horn := PrologClause head: horn car body: (PrologBody fromList: horn cdr).
        functor := horn car car.
        def := userPredicates at: functor ifAbsent: [^false].
        definitionStack := def cons: definitionStack.
        ^true
!

consult: readStream action: aBlock
        | condition parser node hornClause program functor stream result |
        condition := self getCondition.
        parser := PrologParser new.
        result := true.
        [
            node := parser parse: readStream.
            node == #eof        
        ] whileFalse:[ 
            hornClause := node cdr.
            node car ifTrue:[
                stream := WriteStream on: (String new: 20).
                hornClause questionPrintPrologOn: stream.
                textCollector notNil ifTrue:[
                    textCollector show: stream contents.
                    textCollector cr.
                ].
                result := self resolve: hornClause action: aBlock
            ] ifFalse:[
                functor := hornClause car car.
                program := tracePredicates at: functor ifAbsent: [userPredicates at: functor ifAbsent: []].
                program == nil
                    ifTrue: 
                            [program := PrologDefinition fromList: (PrologList car:hornClause cdr: nil).
                            userPredicates at: functor put: program]
                    ifFalse: [program nconc: (PrologList car:hornClause cdr: nil)]
            ]
        ].
        self putCondition: condition.
        ^result

    "Modified: / 20.4.1998 / 12:21:13 / cg"
!

reconsult: readStream action: aBlock
        | condition newPredicates parser node hornClause program functor stream result |
        condition := self getCondition.
        newPredicates := Dictionary new.
        parser := PrologParser new.
        result := true.
        [node := parser parse: readStream.
        node == #eof]
                whileFalse: 
                        [hornClause := node cdr.
                        node car
                                ifTrue: 
                                        [stream := WriteStream on: (String new: 20).
                                        hornClause questionPrintPrologOn: stream.
                                        textCollector show: stream contents.
                                        textCollector cr.
                                        result := self resolve: hornClause action: aBlock]
                                ifFalse: 
                                        [functor := hornClause car car.
                                        program := newPredicates at: functor ifAbsent: [].
                                        program == nil
                                                ifTrue: 
                                                        [program := PrologDefinition fromList: (hornClause cons: nil).
                                                        newPredicates at: functor put: program]
                                                ifFalse: [program nconc: (hornClause cons: nil)].
                                        userPredicates at: functor put: program]].
        newPredicates keys
                do: [:eachPredicate |
                        userPredicates at: eachPredicate put: (newPredicates at: eachPredicate).
                        tracePredicates removeKey: eachPredicate ifAbsent: []].
        self putCondition: condition.
        ^result
!

saveOn: fileName 
        | aFileStream def |
        aFileStream := (Filename named: fileName) writeStream.
        [(userPredicates associations asSortedCollection)
                do: 
                        [:assoc | 
                        def := assoc value.
                        [def consp]
                                whileTrue: 
                                        [aFileStream nextPutAll: def car prettyPrintPrologString , '\' withCRs.
                                        def := def cdr]]]
                valueNowOrOnUnwindDo: [aFileStream close]
!

saveOn: fileName predicateName: aPrologSymbol 
        | aFileStream def |
        aFileStream := (Filename named: fileName) writeStream.
        
        [def := userPredicates at: aPrologSymbol ifAbsent: [].
        def == nil ifFalse: [[def consp]
                        whileTrue: 
                                [aFileStream nextPutAll: def car prettyPrintPrologString , '\' withCRs.
                                def := def cdr]]]
                valueNowOrOnUnwindDo: [aFileStream close]
!

systemConsult: readStream 
        | condition parser node hornClause program functor |
        condition := self getCondition.
        parser := PrologParser new.
        [node := parser parse: readStream.
        node == #eof]
                whileFalse: 
                        [hornClause := node cdr.
                        node car
                                ifTrue: [self resolve: hornClause]
                                ifFalse: 
                                        [functor := hornClause car car.
                                        program := systemPredicates at: functor ifAbsent: [].
                                        program == nil
                                                ifTrue: 
                                                        [program := PrologDefinition fromList: (hornClause cons: nil).
                                                        systemPredicates at: functor put: program]
                                                ifFalse: [program nconc: (hornClause cons: nil)]]].
        self putCondition: condition.
        ^true
!

systemReconsult: readStream 
        | condition newPredicates parser node hornClause program functor |
        condition := self getCondition.
        newPredicates := Dictionary new.
        parser := PrologParser new.
        [node := parser parse: readStream.
        node == #eof]
                whileFalse: 
                        [hornClause := node cdr.
                        node car
                                ifTrue: [self resolve: hornClause]
                                ifFalse: 
                                        [functor := hornClause car car.
                                        program := newPredicates at: functor ifAbsent: [].
                                        program == nil
                                                ifTrue: 
                                                        [program := PrologDefinition fromList: (hornClause cons: nil).
                                                        newPredicates at: functor put: program]
                                                ifFalse: [program nconc: (hornClause cons: nil)].
                                        systemPredicates at: functor put: program]].
        newPredicates keys do: [:eachPredicate | systemPredicates at: eachPredicate put: (newPredicates at: eachPredicate)].
        self putCondition: condition.
        ^true
!

trace: para head: head arguments: terms 
        | headString numberString structure arguString |
        headString := (self expression: head env: clauseEnv) printPrologString.
        para = 0
                ifTrue: 
                        [traceCounter := traceCounter + 1.
                        numberString := String new: traceCounter withAll: $|.
                        structure := PrologStructure functor: head car terms: terms.
                        arguString := (self expression: structure env: clauseEnv) printPrologString.
                        headString := headString , ' ~ ' , arguString.
                        (textCollector ? Transcript)
                            show: numberString , ' CALL : ' , headString , '\' withCRs.
                        ^self].
        para = 1
                ifTrue: 
                        [traceCounter := traceCounter + 1.
                        numberString := String new: traceCounter withAll: $|.
                        structure := PrologStructure functor: head car terms: terms.
                        arguString := (self expression: structure env: clauseEnv) printPrologString.
                        headString := headString , ' ~ ' , arguString.
                        (textCollector ? Transcript)
                            show: numberString , ' REDO : ' , headString , '\' withCRs.
                        ^self].
        para = 2
                ifTrue: 
                        [numberString := String new: traceCounter withAll: $|.
                        (textCollector ? Transcript)
                            show: numberString , ' FAIL : ' , headString , '\' withCRs.
                        traceCounter := traceCounter - 1.
                        ^self].
        para = 3
                ifTrue: 
                        [numberString := String new: traceCounter withAll: $|.
                        (textCollector ? Transcript)
                            show: numberString , ' EXIT : ' , headString , '\' withCRs.
                        ^self]
!

traceCallClause: head
        ^self traceClause: head flag: 0
!

traceClause: head flag: anInteger 
        | arguVar headStruct failStruct traceStruct |
        arguVar := PrologVariable install: 'Arguments'.
        headStruct := PrologStructure functor: head car terms: arguVar.
        failStruct := PrologStructure functor: (PrologSymbol install: 'fail')
                                terms: nil.
        traceStruct := arguVar cons: nil.
        traceStruct := head cons: traceStruct.
        traceStruct := anInteger cons: traceStruct.
        traceStruct := traceStruct cons: nil.
        traceStruct := (PrologSymbol install: 'trace:head:arguments:')
                                cons: traceStruct.
        traceStruct := (PrologSymbol install: 'self')
                                cons: traceStruct.
        traceStruct := PrologTerms fromList: traceStruct.
        traceStruct := PrologStructure functor: (PrologSymbol install: 'send')
                                terms: traceStruct.
        anInteger = 3 ifTrue: [^PrologClause head: headStruct body: (PrologBody fromList: (traceStruct cons: nil))].
        ^PrologClause head: headStruct body: (PrologBody fromList: (traceStruct cons: (failStruct cons: nil)))
!

traceExitClause: exitClause
        | traceExitClause reverseClone| 
        traceExitClause := self traceExitClauseAux: exitClause car.
        traceExitClause := traceExitClause cdr.
        reverseClone := exitClause reverse.
        [reverseClone consp]
                whileTrue:
                        [traceExitClause := reverseClone car cons: traceExitClause.
                        reverseClone := reverseClone cdr].
        ^PrologClause head: traceExitClause car body: (PrologBody fromList: traceExitClause cdr)
!

traceExitClauseAux: head
        ^self traceClause: head flag: 3
!

traceFailClause: head
        ^self traceClause: head flag: 2
!

traceRedoClause: head
        ^self traceClause: head flag: 1
! !

!PrologInterpreter methodsFor:'unification'!

unify: xIn env: xEnvIn and: yIn env: yEnvIn 
        | xx yy xEnv yEnv assoc |

        xEnv := xEnvIn.
        yEnv := yEnvIn.
        xx := xIn.
        yy := yIn.
        [true] whileTrue:[ 
            xEnv == yEnv ifTrue: [
                xx = yy ifTrue: [^true]
            ].
            xx class == Prolog::PrologVariable 
            "/ xx isPrologVariable 
            ifTrue: [
                (assoc := self binding: xx env: xEnv) ~~ nil ifTrue:[ 
                    valueEnv := xEnv.
                    assoc := assoc cdr.
                    xx := self fetchValue: assoc car env: assoc cdr.
                    "/ cg: tail recursion eliminated
                    "/ ^ self unify: xx env: valueEnv and: yy env: yEnv
                    xEnv := valueEnv.
                    "/ continue looping
                ] ifFalse:[ 
                    yy class == Prolog::PrologVariable 
                    "/ yy isPrologVariable 
                    ifTrue: [
                        (assoc := self binding: yy env: yEnv) ~~ nil ifTrue:[
                            valueEnv := yEnv.
                            assoc := assoc cdr.
                            yy := self fetchValue: assoc car env: assoc cdr.
                            "/ cg: tail recursion eliminated
                            "/ ^ self unify: xx env: xEnv and: yy env: valueEnv
                            yEnv := valueEnv.
                            "/ continue looping
                        ] ifFalse:[
                            self bind: xx env: xEnv and: yy env: yEnv.
                            ^ true
                        ]
                    ] ifFalse:[
                        self bind: xx env: xEnv and: yy env: yEnv.
                        ^ true
                    ]
                ]
            ] ifFalse:[
                yy class == Prolog::PrologVariable 
                "/ yy isPrologVariable 
                ifTrue:[
                    (assoc := self binding: yy env: yEnv) ~~ nil ifTrue:[
                        valueEnv := yEnv.
                        assoc := assoc cdr.
                        yy := self fetchValue: assoc car env: assoc cdr.
                        "/ cg: tail recursion eliminated
                        "/ ^ self unify: xx env: xEnv and: yy env: valueEnv
                        yEnv := valueEnv.
                        "/ continue looping
                    ] ifFalse:[
                        self bind: yy env: yEnv and: xx env: xEnv.
                        ^true
                    ]
                ] ifFalse:[
                    "/ (xx class == Prolog::PrologList)
                    xx consp
                    ifFalse: [^ xx = yy].

                    "/ (yy class == Prolog::PrologList)
                    yy consp 
                    ifFalse: [^ yy = xx].

                    (self unify: xx car env: xEnv and: yy car env: yEnv) ifFalse: [^false].
                    xx := xx cdr.
                    yy := yy cdr
                ]
            ]
        ]
! !

!PrologInterpreter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologInterpreter.st,v 1.25 2004/04/15 08:55:42 stefan Exp $'
! !

PrologInterpreter initialize!
