"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologEntity subclass:#PrologList
	instanceVariableNames:'carPart cdrPart'
	classVariableNames:'PrologDotPairPrintHorizontalLevel PrologDotPairPrintVerticalLevel'
	poolDictionaries:''
	category:'Languages-Prolog'
!

PrologList comment:'

Prolog
Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
Copyright (C) 1995-1997 AOKI Atsushi
1997/02/21

'
!

!PrologList class methodsFor:'documentation'!

documentation
"
    Prolog
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"


! !

!PrologList class methodsFor:'instance creation'!

car: carObject
        ^ self basicNew car: carObject
!

car: carObject cdr: cdrObject 
        ^self basicNew car: carObject cdr: cdrObject
!

list: anArray
	| size list |
	size := anArray size.
	list := nil.
	size
		to: 1
		by: -1
		do: [:i | list := self car: (anArray at: i) cdr: list].
	^list
! !

!PrologList class methodsFor:'class initialization'!

initialize
	"PrologList initialize"

	PrologDotPairPrintVerticalLevel := 10.
	PrologDotPairPrintHorizontalLevel := 100
! !

!PrologList methodsFor:'accessing'!

car
	^carPart
!

car: carObject 
	carPart := carObject
!

cdr
	^cdrPart
!

cdr: cdrObject 
	cdrPart := cdrObject
!

nth: anInteger 
	| count list |
	anInteger <= 0 ifTrue: [^self].
	count := 1.
	list := self.
	[list consp]
		whileTrue: 
			[count >= anInteger ifTrue: [^list car].
			count := count + 1.
			list := list cdr].
	^nil
! !

!PrologList methodsFor:'converting'!

asPrologList
	| list new tail |
	new := self car cons: nil.
	tail := new.
	list := self cdr.
	[list consp]
		whileTrue: 
			[tail cdr: (list car cons: nil).
			tail := tail cdr.
			list := list cdr].
	tail cdr: list.
	^new
! !

!PrologList methodsFor:'enumerating'!

do: aBlock
	| list |
	list := self.
	[list consp] whileTrue: [
		aBlock value: list car.
		list := list cdr]
! !

!PrologList methodsFor:'functions'!

append: aPrologDotPair 
	(cdrPart consp)
		ifFalse: [^carPart cons: aPrologDotPair].
	^carPart cons: (cdrPart append: aPrologDotPair)
!

arity
	^2
!

assoc: anObject 
        | list assoc |

        assoc := carPart.
        anObject = assoc car ifTrue: [^assoc].
        list := cdrPart.

        [
          "/ list consp
            list class == Prolog::PrologList
        ] whileTrue:[ 
            assoc := list car.
            anObject = assoc car ifTrue: [^assoc].
            list := list cdr
        ].
        ^nil
!

functor
	^PrologSymbol fromString: '.'
!

length
	| list count |
	list := self.
	count := 0.
	[list consp]
		whileTrue: 
			[count := count + 1.
			list := list cdr].
	^count
!

member: anObject 
	| list |
	list := self.
	[list consp]
		whileTrue: 
			[anObject = list car ifTrue: [^list].
			list := list cdr].
	^nil
!

nconc: aPrologDotPair 
	| list |
	list := self.
	[list consp]
		whileTrue: [(list cdr consp)
				ifTrue: [list := list cdr]
				ifFalse: 
					[list cdr: aPrologDotPair.
					^self]].
	^aPrologDotPair
!

reverse
	| list revlist mark |
	list := self.
	revlist := nil.
	[list consp]
		whileTrue: 
			[revlist == nil
				ifTrue: [revlist := mark := list car cons: revlist]
				ifFalse: [revlist := list car cons: revlist].
			list := list cdr].
	mark cdr: list.
	^revlist
!

structureList
	"disassemble prolog structure of myself into a list."
	^self functor cons: ((self car) cons: ((self cdr) cons: nil))
! !

!PrologList methodsFor:'printing'!

prettyPrintPrologOn: aStream 
	| head body list |
	head := self car.
	body := self cdr.
	head printPrologOn: aStream.
	body ~~ nil
		ifTrue: 
			[aStream nextPutAll: ' :- '.
			list := body.
			aStream cr; tab.
			[list cdr consp]
				whileTrue: 
					[list ~~ body
						ifTrue: 
							[aStream nextPutAll: ', '.
							aStream cr; tab].
					list car printPrologOn: aStream.
					list := list cdr].
			list ~~ self
				ifTrue: 
					[aStream nextPutAll: ', '.
					aStream cr; tab].
			list car printPrologOn: aStream].
	aStream nextPut: $.
!

prettyPrintPrologString
	| aStream |
	aStream := WriteStream on: (String new: 20).
	self prettyPrintPrologOn: aStream.
	^aStream contents
!

printOn: aStream 
	aStream nextPutAll: self class printString.
	aStream nextPutAll: '('.
	aStream nextPutAll: self printPrologString.
	aStream nextPutAll: ')'
!

printPrologOn: aStream 
	self printPrologOn: aStream level: 1
!

printPrologOn: aStream cdr: tail level: anInteger 
	| d count |
	d := tail.
	count := 1.
	[d consp]
		whileTrue: 
			[count >= PrologDotPairPrintHorizontalLevel
				ifTrue: 
					[aStream nextPutAll: ' ... ]'.
					^self].
			aStream nextPut: $,.
			d car consp
				ifTrue: [d car printPrologOn: aStream level: anInteger + 1]
				ifFalse: [d car printPrologOn: aStream].
			"d car printPrologOn: aStream level: anInteger + 1."
			count := count + 1.
			d := d cdr].
	d isNil
		ifTrue: [aStream nextPut: $]]
		ifFalse: 
			[aStream nextPut: $|.
			d printPrologOn: aStream.
			aStream nextPut: $]]
!

printPrologOn: aStream level: anInteger 
	anInteger > PrologDotPairPrintVerticalLevel
		ifTrue: 
			[aStream nextPutAll: ' ... '.
			^self].
	aStream nextPutAll: '['.
	carPart consp
		ifTrue: [carPart printPrologOn: aStream level: anInteger + 1]
		ifFalse: [carPart printPrologOn: aStream].
	self
		printPrologOn: aStream
		cdr: cdrPart
		level: anInteger
! !

!PrologList methodsFor:'private'!

car: carObject cdr: cdrObject 
	carPart := carObject.
	cdrPart := cdrObject
! !

!PrologList methodsFor:'testing'!

= anObject 
	anObject consp ifFalse: [^false].
	self car = anObject car ifTrue: [^self cdr = anObject cdr].
	^false
!

consp
	^true
! !

!PrologList class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologList.st,v 1.5 2002/11/22 20:06:46 cg Exp $'
! !

PrologList initialize!
