"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologScanner subclass:#PrologParser
	instanceVariableNames:'prevMark prevToken prevTokenType failBlock compiler'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog'
!

PrologParser comment:'

Prolog
Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
Copyright (C) 1995-1997 AOKI Atsushi
1997/02/21

'
!

!PrologParser class methodsFor:'documentation'!

documentation
"
    Prolog
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"


! !

!PrologParser methodsFor:'accessing'!

currentToken
    ^  token
! !

!PrologParser methodsFor:'parsing'!

parse: sourceStream 
        | label string |
        ^self parse: sourceStream
                ifFail: 
                        [:errorMessage | 
                        label := errorMessage , ' near ' , token printString.
                        string := source upToEnd.
                        string size > 100 ifTrue: [string := string copyFrom: 1 to: 100].
                        self syntaxError: label , '\' withCRs , (string = ''
                                                ifTrue: ['-- end of file --']
                                                ifFalse: ['--> ' , string])]


    "
     self new parse:'inOrder( [] ).' readStream
    "
!

parse: sourceStream ifFail: aBlock
        | result |
        self init: sourceStream ifFail: aBlock.
        result := self scan.
        ^result

        "
         self new parse:'in Order( [] ).' readStream ifFail:[:msg | self halt:msg].
        "
! !

!PrologParser methodsFor:'private'!

compileAndEvaluate: string 
	compiler isNil ifTrue: [compiler := Compiler new].
	^compiler
		evaluate: string
		in: nil
		to: nil
		notifying: nil
		ifFail: []
!

init: sourceStream ifFail: aBlock 
	super on: sourceStream.
	failBlock := aBlock.
	PrologVariable setZero
!

syntaxError:errMsg
    SyntaxErrorSignal raiseErrorString:errMsg
! !

!PrologParser methodsFor:'scan'!

scan
        source atEnd ifTrue: [^#eof].
        self nextToken.
        tokenType == #eof ifTrue: [^#eof].
        tokenType == #neck ifTrue: [^ PrologList car:true cdr:(self scanClause)]. "resolve clause"
        self unNextToken.
        ^ PrologList car:false cdr:(self scanClause) "assert clause"
!

scanBody
        | list |
        list := nil.
        [tokenType ~~ #eof]
                whileTrue: 
                        [list := PrologList car:(self scanStructure) cdr:list.
                        self nextToken.
                        tokenType == #rightParenthesis ifTrue: [^PrologBody fromReverseList: list].
                        tokenType == #period ifTrue: [^PrologBody fromReverseList: list].
                        tokenType ~~ #comma ifTrue: [failBlock value: 'Syntax error']].
        failBlock value: 'Unexpected eof'
!

scanClause
	| head |
	head := self scanStructure.
	self nextToken.
	tokenType == #neck ifTrue: [^PrologClause head: head body: self scanBody].
	tokenType == #comma ifTrue: [^PrologClause head: head body: self scanBody].
	tokenType == #period ifTrue: [^PrologClause head: head body: nil].
	tokenType == #rightParenthesis ifTrue: [^PrologClause head: head body: nil].
	self unNextToken.
	failBlock value: 'Syntax error'
!

scanExpression
	| symbol |
	self nextToken.
	(tokenType == #eof or: [tokenType == #period])
		ifTrue: [^nil].
	tokenType == #number ifTrue: [^token].
	tokenType == #string ifTrue: [^token].
	tokenType == #object ifTrue: [^PrologObject source: token object: (self compileAndEvaluate: token)].
	tokenType == #leftBracket ifTrue: [^self scanList].
	tokenType == #leftParenthesis ifTrue: [^self scanClause].
	(tokenType == #symbol or: [tokenType == #variable])
		ifTrue: 
			[symbol := token.
			self nextToken.
			tokenType == #leftParenthesis
				ifTrue: 
					[self unNextToken.
					self unNextToken.
					^self scanStructure]
				ifFalse: 
					[self unNextToken.
					^symbol]].
	self unNextToken.
	failBlock value: 'Syntax error'
!

scanList
        | expression |
        self nextToken.
        tokenType == #comma ifTrue: [self nextToken].
        tokenType == #neck ifTrue: [self nextToken].
        tokenType == #rightBracket ifTrue: [^nil].
        tokenType == #leftBracket ifTrue: [^ PrologList car:(self scanList) cdr:(self scanList) ].
        tokenType == #verticalBar
                ifTrue: 
                        [expression := self scanExpression.
                        self nextToken.
                        tokenType == #rightBracket
                                ifTrue: [^expression]
                                ifFalse: [failBlock value: 'Syntax error']].
        self unNextToken.
        expression := self scanExpression.
        ^ PrologList car:expression cdr:(self scanList)
!

scanStructure
	| functor terms |
	functor := self nextToken.
	(tokenType == #symbol or: [tokenType == #variable])
		ifTrue: 
			[terms := self scanTerms.
			^PrologStructure functor: functor terms: terms].
	failBlock value: 'Syntax error'
!

scanTerms
        | list |
        self nextToken.
        tokenType == #leftParenthesis
                ifTrue: 
                        [self nextToken.
                        (tokenType == #verticalBar or: [tokenType == #rightParenthesis])
                                ifTrue:
                                        [tokenType == #rightParenthesis
                                                ifTrue: [^nil].
                                        list := self scanExpression.
                                        self nextToken.
                                        tokenType == #rightParenthesis
                                                ifTrue: [^list]
                                                ifFalse: [failBlock value: 'Syntax error']]
                                ifFalse: [self unNextToken].
                        list := nil.
                        [tokenType ~~ #eof]
                                whileTrue: 
                                        [list := PrologList car:(self scanExpression) cdr: list.
                                        self nextToken.
                                        token == #rightParenthesis ifTrue: [^PrologTerms fromReverseList: list].
                                        tokenType == #verticalBar
                                                ifTrue:
                                                        [list := list reverse.
                                                        list := list nconc: self scanExpression.
                                                        self nextToken.
                                                        tokenType == #rightParenthesis
                                                                ifTrue: [^PrologTerms fromList: list]
                                                                ifFalse: [failBlock value: 'Syntax error']].
                                        tokenType ~~ #comma ifTrue: [failBlock value: 'Syntax error']].
                        failBlock value: 'Unexpected eof']
                ifFalse: 
                        [self unNextToken.
                        ^nil]
! !

!PrologParser methodsFor:'scanning'!

nextToken
	prevMark := mark.
	prevToken := token.
	prevTokenType := tokenType.
	^super nextToken
!

unNextToken
	super unNextToken.
	mark := prevMark.
	token := prevToken.
	tokenType := prevTokenType
! !

!PrologParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologParser.st,v 1.10 2003/01/28 12:58:25 cg Exp $'
! !
