"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

Object subclass:#PrologScanner
	instanceVariableNames:'source mark token tokenType typeTable endChar'
	classVariableNames:'EndChar SyntaxErrorSignal'
	poolDictionaries:''
	category:'Languages-Prolog'
!

PrologScanner comment:'

Prolog
Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
Copyright (C) 1995-1997 AOKI Atsushi
1997/02/21

'
!

!PrologScanner class methodsFor:'documentation'!

documentation
"
    Prolog
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"


! !

!PrologScanner class methodsFor:'instance creation'!

new
	^super new initScanner
! !

!PrologScanner class methodsFor:'Signal constants'!

syntaxErrorSignal
    ^ SyntaxErrorSignal
! !

!PrologScanner class methodsFor:'class initialization'!

initialize
    "PrologScanner initialize"

"/    EndChar := $^.
    EndChar := Character value:0.

    SyntaxErrorSignal isNil ifTrue:[
        SyntaxErrorSignal := Signal new mayProceed:false.
        SyntaxErrorSignal notifierString:'syntax error'.
        SyntaxErrorSignal nameClass:self message:#syntaxErrorSignal.
    ]
! !

!PrologScanner methodsFor:'accessing'!

tokenPosition
    ^ mark
! !

!PrologScanner methodsFor:'initialize-release'!

initScanner
	typeTable := PrologScannerTable scannerTable.
	endChar := EndChar
!

on: inputStream 
	source := inputStream.
	mark := source position
! !

!PrologScanner methodsFor:'private'!

nextChar
	| char |
	char := source next.
	char = Character lf ifTrue: [char := Character cr].
	char = Character cr
		ifTrue: 
			[source peek =  Character lf ifTrue: [source next].
			char := Character cr].
	char == nil ifTrue: [char := endChar].
	^char
!

peekChar
	| char |
	char := source peek.
	char = Character lf ifTrue: [char := Character cr].
	char = Character cr ifTrue: [char := Character cr].
	char == nil ifTrue: [char := endChar].
	^char
!

unNextChar
	source skip: -1
! !

!PrologScanner methodsFor:'scanning'!

multiChar: type 
	self perform: type
!

nextToken
        | char |
        mark := source position.
        char := self peekChar.
        char == endChar ifTrue:[
            tokenType := #eof
        ] ifFalse:[
            tokenType := typeTable at: char asInteger.
        ].
        [tokenType == #xDelimiter] whileTrue:[ 
            self nextChar.
            char := self peekChar.
            char == endChar ifTrue:[
                tokenType := #eof
            ] ifFalse:[
                tokenType := typeTable at: char asInteger
            ].
        ].
        (tokenType at: 1) = $x
                ifTrue:[self multiChar: tokenType]
                ifFalse:[self singleChar: tokenType].
        ^token
!

singleChar: type 
	self nextChar.
	token := type.
	tokenType == #leftBrace ifFalse: [^self].
	tokenType := #object.
	token := PrologObject readFrom: source
!

unNextToken
	source position: mark
!

xBinary
	NotifierView
		openContext: thisContext
		label: 'Syntax error ' , source peek printString , '\' withCRs , source upToEnd
		proceedable: false
!

xColon
	| char |
	source next.
	char := source peek.
	char = $-
		ifTrue: 
			[tokenType := #neck.
			self singleChar: tokenType]
		ifFalse: 
			[self unNextToken.
			tokenType := #symbol.
			token := PrologSymbol readFrom: source]
!

xComment
        | char |

        self nextChar.
        self peekChar == $" ifTrue:[
            "/ inline or multiline comment
            self nextChar.
            [((char := self nextChar) = $") and:[self peekChar = $% ]]
                whileFalse: [char == endChar
                                    ifTrue: 
                                            [tokenType := #eof.
                                            ^self]].
        ] ifFalse:[
            "/ EOL comment

            [(char := self nextChar) = Character cr]
                    whileFalse: [char == endChar
                                    ifTrue: 
                                            [tokenType := #eof.
                                            ^self]].
        ].
        self nextToken
!

xDigit
        tokenType := #number.
        token := Number readFrom: source

        "
         Number readFrom:'1234,4'. 
         Number readFrom:'1234.4'.   
        "
!

xDollar
	self xBinary
!

xDoubleQuote
	tokenType := #string.
	token := PrologString readFrom: source
!

xSign
	| char sign |
	sign := self nextChar.
	char := self peekChar.
	char isDigit
		ifTrue: 
			[tokenType := #number.
			token := Number readFrom: source.
			sign == $- ifTrue: [token := token negated]]
		ifFalse: 
			[self unNextChar.
			tokenType := #symbol.
			token := PrologSymbol readFrom: source]
!

xSingleQuote
	tokenType := #symbol.
	token := PrologSymbol readFrom: source
!

xSymbol
	tokenType := #symbol.
	token := PrologSymbol readFrom: source
!

xVariable
	tokenType := #variable.
	token := PrologVariable readFrom: source
! !

!PrologScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologScanner.st,v 1.9 2006/04/26 10:54:31 cg Exp $'
! !

PrologScanner initialize!
