"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologString subclass:#PrologSymbol
	instanceVariableNames:''
	classVariableNames:'PrologCut PrologFail PrologSelf PrologSend PrologSymbolTable
		PrologTrue PrologVar PrologInstance'
	poolDictionaries:''
	category:'Languages-Prolog'
!

PrologSymbol comment:'

Prolog
Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
Copyright (C) 1995-1997 AOKI Atsushi
1997/02/21

'
!

!PrologSymbol class methodsFor:'documentation'!

documentation
"
    Prolog
    Goodies for VisualWorks 2.5/2.5J and VisualWave 1.0
    (and ST/X)

    Copyright (C) 1995-1997 AOKI Atsushi
    1997/02/21


    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"


! !

!PrologSymbol class methodsFor:'instance creation'!

install: aString 
        | symbol key |

        symbol := PrologSymbolTable at: aString ifAbsent: [nil].
        symbol == nil ifTrue:[ 
            "/ key := self fromString: aString.
            symbol := self fromString: aString.
            PrologSymbolTable at: aString put: symbol
        ].
        ^ symbol
!

readFrom: aStream 
        | map buffer char |

        map := PrologCharMap symbolMap.
        buffer := WriteStream on: (String new: 20).
        char := aStream peek.
        char == $' ifTrue:[
            aStream next.
            
            [
                char := aStream peek.
                char ~~ nil
            ] whileTrue:[ 
                char = $' ifTrue:[ 
                    aStream next.
                    char := aStream peek.
                    char = $' ifFalse: [^self install: buffer contents]
                ].
                buffer nextPut: aStream next
            ].
            NotifierView
                    openContext: thisContext
                    label: 'Syntax error near $'' unmatched $''\-- end of file --' withCRs
                    proceedable: false
        ].
        (self expect: '=..' on: aStream)
        ifTrue: ["Non standard (or ad hoc) parsing patterns."
            ^self install: '=..'
        ].
        [char ~~ nil and: [map at: char asInteger]]
        whileTrue:[ 
            buffer nextPut: aStream next.
            char := aStream peek
        ].
        ^ self install: buffer contents
! !

!PrologSymbol class methodsFor:'class initialization'!

initialize
        "PrologSymbol initialize"

        PrologSymbolTable := Dictionary new.
        PrologCut := self install: '!!'.
        PrologTrue := self install: 'true'.
        PrologFail := self install: 'fail'.
        PrologSend := self install: 'send'.
        PrologVar := self install: 'var'.
        PrologSelf := self install: 'self'.
        PrologInstance := self install: 'instance'.

! !

!PrologSymbol class methodsFor:'constants'!

cutSymbol
        ^PrologCut
!

failSymbol
        ^PrologFail
!

instanceSymbol
        ^PrologInstance
!

selfSymbol
        ^PrologSelf
!

sendSymbol
        ^PrologSend
!

trueSymbol
        ^PrologTrue
!

varSymbol
        ^PrologVar
! !

!PrologSymbol class methodsFor:'private'!

expect: aString on: aStream
	"If aStream contains aString at the current position, then return true and set the position of aStream to the next char of the string.
	else return false and rewind the position."

	| pos |
	pos := aStream position.
	aString = (aStream nextAvailable: aString size)
		ifTrue: [^true]
		ifFalse: [
			aStream position: pos.
			^false]
! !

!PrologSymbol methodsFor:'accessing'!

arity
	^0
!

functor
	^self
! !

!PrologSymbol methodsFor:'printing'!

printPrologOn: aStream 
	| map flag firstChar i length x |
	map := PrologCharMap symbolMap.
	flag := false.
	self do: [:each | (map at: each asInteger)
			ifFalse: [flag := true]].
	string isEmpty
		ifTrue: 
			[aStream nextPutAll: ''''.
			aStream nextPutAll: '''']
		ifFalse: 
			[firstChar := string at: 1.
			((firstChar >= $A and: [firstChar <= $Z])
				or: [flag])
				ifTrue: 
					[aStream nextPutAll: ''''.
					i := 0.
					length := string size.
					[(i := i + 1) <= length]
						whileTrue: 
							[aStream nextPut: (x := string at: i).
							x == $' ifTrue: [aStream nextPut: x]].
					aStream nextPutAll: '''']
				ifFalse: 
					[i := 0.
					length := string size.
					[(i := i + 1) <= length]
						whileTrue: 
							[aStream nextPut: (x := string at: i).
							x == $' ifTrue: [aStream nextPut: x]]]]
!

printPrologOn: aStream level: anInteger 
	self printPrologOn: aStream
! !

!PrologSymbol methodsFor:'testing'!

isBuiltInPredicate
	self isPrologCut ifTrue: [^true].
	self isPrologTrue ifTrue: [^true].
	self isPrologFail ifTrue: [^true].
	self isPrologSend ifTrue: [^true].
	self isPrologVar ifTrue: [^true].
	^false
!

isPrologCut
	^self = PrologCut
!

isPrologFail
	^self = PrologFail
!

isPrologInstance
        ^self = PrologInstance
!

isPrologSelf
	^self = PrologSelf
!

isPrologSend
	^self = PrologSend
!

isPrologTrue
	^self = PrologTrue
!

isPrologVar
	^self = PrologVar
! !

!PrologSymbol class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/PrologSymbol.st,v 1.6 2002/11/22 20:06:44 cg Exp $'
! !

PrologSymbol initialize!
