"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologClass subclass:#TowersOfHanoiApplication
	instanceVariableNames:'poles offPole delayTime numberOfDisks numberOfMoves'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog-Examples'
!

View subclass:#TowersOfHanoiView
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:TowersOfHanoiApplication
!

!TowersOfHanoiApplication class methodsFor:'documentation'!

documentation
"
    This example demonstrates, how to combine prolog logic
    with a smalltalk graphical user interface.
    Although not a good example for prolog (the code is really not
    much more in smalltalk or any other language), this shows
    how to pass values/actions between prolog and smalltalk.
    Also, notice that the prolog code is visible in the browser
    almost like regular smalltalk code 
    (yes, you can edit those clauses and install them with 'accept')

    Prolog::TowersOfhanoiApplication open

    [author:]
        Claus Gittinger
        (the hanoi algorithm is found everywhere - not much work there ..;-)

    [see also:]
        PrologClass PrologInterpreter ApplicationModel
"
! !

!TowersOfHanoiApplication class methodsFor:'interface specs'!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:Prolog::TowersOfHanoiApplication andSelector:#windowSpec
     Prolog::TowersOfHanoiApplication new openInterface:#windowSpec
     Prolog::TowersOfHanoiApplication open
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpec
        #window: 
       #(#WindowSpec
          #label: 'Towers Of Hanoi'
          #name: 'Prolog::TowersOfHanoiApplication'
          #min: #(#Point 520 300)
          #max: #(#Point 520 300)
          #bounds: #(#Rectangle 16 46 536 346)
          #menu: #mainMenu
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#ArbitraryComponentSpec
              #name: 'TowersView'
              #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
              #model: #yourself
              #component: #TowersOfHanoiView
            )
           )
         
        )
      )
! !

!TowersOfHanoiApplication class methodsFor:'menu specs'!

mainMenu
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:Prolog::TowersOfHanoiApplication andSelector:#mainMenu
     (Menu new fromLiteralArrayEncoding:(Prolog::TowersOfHanoiApplication mainMenu)) startUp
    "

    <resource: #menu>

    ^ 
     #(#Menu
        #(
         #(#MenuItem
            #label: 'Game'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Reset'
                  #translateLabel: true
                  #value: #resetGame
                )
               #(#MenuItem
                  #label: 'Start'
                  #translateLabel: true
                  #value: #startGame
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Exit'
                  #translateLabel: true
                  #value: #closeRequest
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Settings'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Number of disks...'
                  #translateLabel: true
                  #value: #setNumberOfDisks
                )
               #(#MenuItem
                  #label: 'Animation Speed...'
                  #translateLabel: true
                  #value: #setAnimationSpeed
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Help'
            #translateLabel: true
            #startGroup: #right
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'How to Play...'
                  #translateLabel: true
                  #value: #doHowToPlay
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'About...'
                  #translateLabel: true
                  #value: #doAbout
                )
               )
              nil
              nil
            )
          )
         )
        nil
        nil
      )
! !

!TowersOfHanoiApplication methodsFor:'accessing'!

offPole
    ^ offPole
!

poles
    ^ poles
! !

!TowersOfHanoiApplication methodsFor:'calls from prolog'!

canPutTo:poleNr
    offPole isNil ifTrue:[^ false].
    (poles at:poleNr) isEmpty ifTrue:[^ true].
    ^ (poles at:poleNr) last > offPole


!

canTakeFrom:poleNr
    ^ (poles at:poleNr) notEmpty


!

moveFrom:fromPole to:toPole delay:doDelay
    "move a disk"

    numberOfMoves := numberOfMoves + 1.

    "/ take a disk from ...
    self takeFrom:fromPole.
    doDelay ifTrue:[
        self changed.
        Delay waitForSeconds:((delayTime / 10) max:0.05).
    ].

    "/ put that disk to ...
    self putTo:toPole.
    doDelay ifTrue:[
        self changed.
        Delay waitForSeconds:(delayTime).
    ]
!

prologMoveFrom:prologFrom to:prologTo
    "called by prolog, to move a disk"

    |from to|

    from := prologFrom string.
    from := #('left' 'center' 'right') indexOf:from.

    to := prologTo string.
    to := #('left' 'center' 'right') indexOf:to.

    self moveFrom:from to:to delay:true

!

putTo:toPole
    "/ put that disk to ...
    (poles at:toPole) addLast:offPole.
    offPole := nil.
!

takeFrom:fromPole
    "take a disk"

    "/ take a disk from ...
    offPole := (poles at:fromPole) removeLast.
! !

!TowersOfHanoiApplication methodsFor:'initialization'!

initialDiskSetup
    poles := Array 
                with:OrderedCollection new
                with:OrderedCollection new
                with:OrderedCollection new.

    (poles at:1) addAll:(numberOfDisks to:1 by:-1)

!

initialize
    delayTime := 0.2.
    numberOfDisks := 5.

    self initialDiskSetup.

    super initialize
! !

!TowersOfHanoiApplication methodsFor:'initialization & release'!

closeDownViews
    "This is a hook method generated by the Browser.
     It will be invoked when your app/dialog-window is really closed."

    "/ change the code below as required ...
    "/ This should cleanup any leftover resources
    "/ (for example, temporary files)
    "/ super closeRequest will initiate the closeDown

    "/ add your code here

    "/ do not remove the one below ...
    ^ super closeDownViews
!

postBuildWith:aBuilder
    super postOpenWith:aBuilder.
    (builder componentAt:#TowersView) model:self.
! !

!TowersOfHanoiApplication methodsFor:'menu actions'!

checkResonableNumberOfDisks
    |nMoves tOneMove secondsToRun hoursToRun minutesToRun daysToRun monthsToRun yearsToRun|

    nMoves := (2 raisedTo:numberOfDisks) - 1.
    tOneMove := 0.1 max:(delayTime ? 0).
    secondsToRun := tOneMove * nMoves.
    secondsToRun > 60 ifTrue:[
      minutesToRun := secondsToRun / 60.
      minutesToRun > 60 ifTrue:[
        hoursToRun := secondsToRun / 3600.
        hoursToRun > 24 ifTrue:[        
          daysToRun := hoursToRun / 24.
          daysToRun > 30 ifTrue:[
            monthsToRun := daysToRun / 30.
            daysToRun > 365 ifTrue:[
              yearsToRun := daysToRun / 365.
              yearsToRun > 70 ifTrue:[
                Transcript show:'Hanoi estimated runTime:'; show:yearsToRun truncated; showCR: ' years'.
                yearsToRun > 1000 ifTrue:[
                  yearsToRun > 1000000 ifTrue:[
                    yearsToRun > 1000000000 ifTrue:[
                      yearsToRun > 4000000000 ifTrue:[
                        yearsToRun > 13000000000 ifTrue:[
                          ^ self information:'that will take longer than the universes lifetime ...'
                        ].
                        ^ self information:'that will take longer than the earths lifetime ...'
                      ].
                      ^ self information:'come back in ' 
                          , ((yearsToRun / 1000000000) asFixedPoint:1) printString
                          , ' billion years to see the endGame ...'
                    ].
                    ^ self information:'come back in ' 
                        , ((yearsToRun / 1000000) asFixedPoint:1) printString
                        , ' million years to see the endGame ...'
                  ].
                  ^ self information:'come back in ' 
                      , ((yearsToRun / 1000) asFixedPoint:1) printString
                      , ' thousand years to see the endGame ...'
                ].
                yearsToRun > 100 ifTrue:[
                    ^ self information:'come back in ' 
                      , ((yearsToRun / 100) asFixedPoint:1) printString
                      , ' hundred years to see the endGame ...'
                ].
                ^ self information:'takes longer than an average persons lifetime...' 
              ].
              Transcript show:'Hanoi estimated runTime:'; show:((yearsToRun) asFixedPoint:1); showCR: ' years'.
              yearsToRun > 10 ifTrue:[
                  ^ self information:'come back in ' 
                    , ((yearsToRun / 10) asFixedPoint:1) printString
                    , ' decades to see the endGame ...'
              ].
              ^ self information:'come back in ' 
                , ((yearsToRun) asFixedPoint:1) printString
                , ' years to see the endGame ...'
            ].
            Transcript show:'Hanoi estimated runTime:'; show:((monthsToRun) asFixedPoint:1); showCR: ' month'.
            ^ self information:'expect me to waste the next ' 
              , ((monthsToRun) asFixedPoint:1) printString
              , ' months cpu cycles with hanoi playing ...'
          ].
          Transcript show:'Hanoi estimated runTime:'; show:((daysToRun) asFixedPoint:1); showCR: ' days'.
          ^ self information:'do you really want me to spend the next ' 
            , ((daysToRun) asFixedPoint:1) printString
            , ' days wasting my cpu cycles with hanoi playing ...'
        ].
        Transcript show:'Hanoi estimated runTime:'; show:((hoursToRun) asFixedPoint:1); showCR: ' hours'.
        ^ self information:'dont you think there are more useful things to do in the next ' 
          , ((hoursToRun) asFixedPoint:1) printString
          , ' hours ?'
      ].
      Transcript show:'Hanoi estimated runTime:'; show:minutesToRun truncated; showCR: ' minutes'.
      minutesToRun > 30 ifTrue:[
        ^ self information:'how about going for a meal ? I estimate shuffling disks for about ' 
          , ((minutesToRun) truncated) printString
          , ' minutes ?'
      ].
      minutesToRun > 5 ifTrue:[
        ^ self information:'how about a coffe break ? - I estimate shuffling disks for about ' 
          , ((minutesToRun) truncated) printString
          , ' minutes ?'
      ].
    ].
      Transcript show:'Hanoi estimated runTime:'; show:secondsToRun truncated; showCR: ' seconds'.

!

doAbout
    |box|

    box := AboutBox new.
    box label:'About Towers ...'.
    box title:'TowersOfHanoi (V' 
               , self class revision 
               , ' of '
               , self class revisionDateString
               , ')

an example to demonstrate how Prolog logic 
can be combined with a smalltalk application.


Have fun.
'.
    box show



!

doHowToPlay
    |box|

    box := InfoBox new.
    box label:'HowToPlay TowersOfHanoi ...'.
    box title:'How to play Towers Of Hanoi:

The goal is to move all disks from the left
tower to the right tower. But you may not
place a disk onto a smaller one.

You can play for yourself, by clicking on
a disk to pick it and clicking on the tower
where you want to drop it.

Or, you can let me shufle the disks, by
activating the Start menu-item.

Have fun.
'.
    box show



!

resetGame
    self initialDiskSetup.
    self changed.
!

setAnimationSpeed
    |answer num|

    answer := Dialog request:'Delay between disk moves (seconds):'.
    answer size ~~ 0 ifTrue:[
        num := Number readFrom:answer onError:nil.
        num notNil ifTrue:[
            delayTime := num
        ]
    ]
!

setNumberOfDisks
    |answer num|

    answer := Dialog request:'Number of disks:'.
    answer size ~~ 0 ifTrue:[
        num := Integer readFrom:answer onError:nil.
        num notNil ifTrue:[
            numberOfDisks := num.
            self resetGame.
            self checkResonableNumberOfDisks.
        ]
    ]
!

startGame
    self resetGame.

    self information:'You can stop the animation by pressing CTRL-y in the towers view'.

    numberOfMoves := 0.
    Transcript show:'Hanoi startTime:'; showCR:Time now.

    [
        self goal.
    ] valueNowOrOnUnwindDo:[
        myPrologInterpreter cleanup
    ].
    Transcript show:'Number of moves: '; showCR:numberOfMoves.
    Transcript show:'Hanoi endTime:'; showCR:Time now.

! !

!TowersOfHanoiApplication methodsFor:'prolog-clauses'!

% Start the Hanoi Tower Program; aquire the number of
% disks from the application instance.

goal :- 
        send(instance, numberOfInitialPoles, [], N),
        hanoi(N).                       % question goal.

% goal
!

% This is Hanoi Tower Program.

hanoi(N) :-     
        move(N,left,right,center).

!

% inform the application that a disk should be moved.

inform(X,Y) :-
%        writeln([move,disc,from,X,to,Y]),
        send(instance, prologMoveFrom:to:, [X,Y]).

!

% the actual recursive algorithm.

move(0,X,Y,Z) :- !!.
move(N,A,B,C) :-
        -(N,1,M),
        move(M,A,C,B),
        inform(A,B),
        move(M,C,B,A).

! !

!TowersOfHanoiApplication methodsFor:'queries from prolog'!

numberOfInitialPoles
    ^ (poles at:1) size.
! !

!TowersOfHanoiApplication::TowersOfHanoiView methodsFor:'change & update'!

update:something with:aParameter from:changedObject
    self redraw


! !

!TowersOfHanoiApplication::TowersOfHanoiView methodsFor:'drawing'!

drawDiskSize:diskSize atX:cX y:topY height:h
    |wHalf|

    wHalf := diskSize // 2.
    self paint:(Color grey:30).
    self fillRectangleX:cX-wHalf y:topY width:wHalf+wHalf height:h.

    self paint:Color white.
    self displayLineFromX:cX-wHalf y:topY toX:cX+wHalf y:topY.
    self displayLineFromX:cX-wHalf y:topY toX:cX-wHalf y:topY+h.
    self displayLineFromX:cX-wHalf y:topY+h toX:cX+wHalf y:topY+h.
    self displayLineFromX:cX+wHalf y:topY toX:cX+wHalf y:topY+h.



!

drawEmptyPoleAtX:x topY:topY
    self paint:(Color grey:80).
    self fillRectangleX:x-2 y:topY width:4 height:height-topY.

    self displayLineFromX:x-2 y:topY toX:x-2 y:height.
    self displayLineFromX:x+2 y:topY toX:x+2 y:height.
    self displayLineFromX:x-2 y:topY toX:x+2 y:topY.


!

redraw
    |poles wEach hEach nPoles nDisks maxDiskSize offPole|

    self viewBackground:Color black.
    self clear.

    self paint:Color white.

    poles := model poles.
    offPole := model offPole.

    nPoles := poles size.

    nDisks := poles inject:0 into:[:sumSoFar :eachPole | eachPole size + sumSoFar].
    maxDiskSize := poles inject:0 into:[:maxSoFar :eachPole | 
                        eachPole inject:maxSoFar into:[:maxSoFar :eachDisk | eachDisk max: maxSoFar]].
    offPole notNil ifTrue:[
        maxDiskSize := maxDiskSize max:offPole.
        nDisks := nDisks + 1.
    ].

    hEach := self height // (nDisks + 1).
    wEach := self width // nPoles.

    poles keysAndValuesDo:[:poleIndex :pole |
        self drawEmptyPoleAtX:(wEach//2 + (poleIndex-1*wEach)) topY:hEach//2.

        pole keysAndValuesDo:[:diskIndex :diskSize |
            |sz|

            sz := diskSize / maxDiskSize * wEach.
            self 
                drawDiskSize:sz*0.9 
                atX:(wEach//2 + (poleIndex-1*wEach)) 
                y:(height-(diskIndex*hEach))
                height:hEach.
        ].
    ].

    "/ draw the poles...





! !

!TowersOfHanoiApplication::TowersOfHanoiView methodsFor:'event handling'!

buttonPress:button x:x y:y
    |nP xW poleNr|

    nP := model poles size.
    xW := width / nP.
        
    poleNr := x // xW + 1.

    model offPole isNil ifTrue:[
        (model canTakeFrom:poleNr) ifTrue:[
            model takeFrom:poleNr
        ] ifFalse:[
            self flash; beep
        ]
    ] ifFalse:[
        (model canPutTo:poleNr) ifTrue:[
            model putTo:poleNr
        ] ifFalse:[
            self flash; beep
        ]
    ].
    self redraw.
! !

!TowersOfHanoiApplication class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/TowersOfHanoiApplication.st,v 1.8 2002/11/26 18:31:15 cg Exp $'
! !
