"{ Package: 'stx:goodies/prolog' }"

"{ NameSpace: Prolog }"

PrologClass subclass:#TrianglePuzzle
	instanceVariableNames:'pieces boardView'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Prolog-Examples'
!

DisplayObject subclass:#Piece
	instanceVariableNames:'container image origin owningApplication'
	classVariableNames:''
	poolDictionaries:''
	privateIn:TrianglePuzzle
!

!TrianglePuzzle class methodsFor:'documentation'!

documentation
"
    this example solves a little puzzle, which my doughter wanted me to solve (I am impatient ;-)
    This puzzle is sold by: DaMert Company - San Leandro, CA.

    Description:
       the puzzle consists of a magnetic plate, with 9 triangular pieces;
       the pieces have different pictures on each of its 3 sides:
            BugTail       (Bt),
            BugHead       (Bh)
            BlueFrogTail  (BFt)
            BlueFrogHead  (BFh)
            RedFrogTail   (RFt)
            RedFrogHead   (RFh)
            ButterflyTail (Ft)
            ButterflyHead (Fh)
            AntTail       (At)
            AntHead       (Ah)

    piece1:                           piece2:                       piece3:                         
                 /\                               /\                             /\                          
                /  \                             /  \                           /  \                          
               /    \                           /    \                         /    \                      
              /      \                         /      \                       /      \                       
             /        \                       /        \                     /        \                   
            /BugH      \                     /AntT      \                   /AntT      \                
           /            \                   /            \                 /            \                
          /     BlueFrogT\                 /          BugH\               /     BlueFrogT\               
         /                \               /                \             /                \            
        /    ButterflyH    \             /     BlueFrogT    \           /     RedFrogT     \         
        --------------------             --------------------           --------------------           

    piece4:                           piece5:                       piece6:                         
                 /\                               /\                             /\                          
                /  \                             /  \                           /  \                          
               /    \                           /    \                         /    \                      
              /      \                         /      \                       /      \                       
             /        \                       /        \                     /        \                   
            /AntT      \                     /AntH      \                   /BlueFrogH \                
           /            \                   /            \                 /            \                
          /    ButterflyH\                 /    ButterflyH\               /    ButterflyT\               
         /                \               /                \             /                \            
        /     RedFrogH     \             /     BlueFrogH    \           /     RedFrogT     \         
        --------------------             --------------------           --------------------           

    piece7:                           piece8:                       piece9:                         
                 /\                               /\                             /\                          
                /  \                             /  \                           /  \                          
               /    \                           /    \                         /    \                      
              /      \                         /      \                       /      \                       
             /        \                       /        \                     /        \                   
            /RedFrogH  \                     /AntH      \                   /ButterflyT\                
           /            \                   /            \                 /            \                
          /          BugH\                 /      RedFrogT\               /          BugH\               
         /                \               /                \             /                \            
        /    BlueFrogT     \             /       BugT       \           /    BlueFrogT     \         
        --------------------             --------------------           --------------------           

    The goal of the puzzle is to arange the pieces into a bigger triangle:
                           /\
                          /  \
                         /    \
                        /      \
                       /        \
                      /          \
                     *------------*
                    /\            /\
                   /  \          /  \
                  /    \        /    \
                 /      \      /      \
                /        \    /        \
               /          \  /          \
              *------------**------------*
             /\            /\            /\
            /  \          /  \          /  \
           /    \        /    \        /    \
          /      \      /      \      /      \
         /        \    /        \    /        \
        /          \  /          \  /          \
       *------------**------------**------------*

    such that the pictures fit (BugHead <-> BugTail, RedFrogHead <-> RedFrogTail , ...).
"
!

examples
"
    the puzzle is solved by:
                                                                                                [exBegin]
    self new 
        queryValues:'solve(P1, P2, P3, P4, P5, P6, P7, P8, P9).' 
        allInto:[:x | Transcript showCR:x]
                                                                                                [exEnd]
"
! !

!TrianglePuzzle class methodsFor:'image specs'!

board
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/board.gif' forClass:self

    "
     self board inspect
    "
!

piece1
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece1.gif' forClass:self

    "
     self piece1 inspect
    "
!

piece2
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece2.gif' forClass:self

    "
     self piece2 inspect
    "
!

piece3
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece3.gif' forClass:self

    "
     self piece3 inspect
    "
!

piece4
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece4.gif' forClass:self

    "
     self piece4 inspect
    "
!

piece5
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece5.gif' forClass:self

    "
     self piece5 inspect
    "
!

piece6
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece6.gif' forClass:self

    "
     self piece6 inspect
    "
!

piece7
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece7.gif' forClass:self

    "
     self piece7 inspect
    "
!

piece8
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece8.gif' forClass:self

    "
     self piece8 inspect
    "
!

piece9
    <resource: #programImage>

    ^ Smalltalk imageFromFileNamed:'bitmaps/piece9.gif' forClass:self

    "
     self piece9 inspect
    "
! !

!TrianglePuzzle class methodsFor:'interface specs'!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:Prolog::TrianglePuzzle andSelector:#windowSpec
     Prolog::TrianglePuzzle new openInterface:#windowSpec
     Prolog::TrianglePuzzle open
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpec
        #window: 
       #(#WindowSpec
          #label: 'TrianglePuzzle'
          #name: 'TrianglePuzzle'
          #min: #(#Point 10 10)
          #max: #(#Point 1280 1024)
          #bounds: #(#Rectangle 16 46 847 643)
          #menu: #mainMenu
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#ViewSpec
              #name: 'Box1'
              #layout: #(#LayoutFrame 0 0.0 0 0.0 583 0.0 581 0.0)
              #postBuildCallback: #boardViewCreated:
            )
           #(#ViewSpec
              #name: 'Box2'
              #layout: #(#LayoutFrame 584 0.0 0 0.0 0 1.0 0 1.0)
              #postBuildCallback: #galleryCreated:
            )
           )
         
        )
      )
! !

!TrianglePuzzle class methodsFor:'menu specs'!

mainMenu
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:Prolog::TrianglePuzzle andSelector:#mainMenu
     (Menu new fromLiteralArrayEncoding:(Prolog::TrianglePuzzle mainMenu)) startUp
    "

    <resource: #menu>

    ^ 
     #(#Menu
        #(
         #(#MenuItem
            #label: 'File'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Exit'
                  #translateLabel: true
                  #value: #closeRequest
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Puzzle'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Shuffle'
                  #translateLabel: true
                  #value: #menuShufflePuzzle
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Solve'
                  #translateLabel: true
                  #value: #menuSolvePuzzle
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Help'
            #translateLabel: true
            #startGroup: #right
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Documentation'
                  #translateLabel: true
                  #value: #openDocumentation
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'About this Application'
                  #translateLabel: true
                  #value: #openAboutThisApplication
                )
               )
              nil
              nil
            )
          )
         )
        nil
        nil
      )

    "Modified: / 23.8.2001 / 16:15:15 / cg"
! !

!TrianglePuzzle methodsFor:'UI initialization'!

boardViewCreated:aView
    boardView := aView.
    boardView viewBackground:(self class board)

    "Modified: / 23.8.2001 / 16:28:24 / cg"
!

galleryCreated:aView
    pieces := Dictionary new.

    #( piece1 piece2 piece3 piece4 piece5 piece6 piece7 piece8 piece9)
    with:#( (0 -10) 
                (100 50) 
            (0 100) 
                (100 160) 
            (0 210) 
                (100 270)
            (0 320) 
                (100 380) 
            (0 430)  )
    do:[:nm :pos |
        |img piece|

        img := self class perform:nm.
        piece := Piece new 
                    image:img; 
                    moveTo:(pos changeClassTo:Point); 
                    owningApplication:self.

        aView addComponent:piece.
        pieces at:nm put:piece.
    ].

    "Modified: / 23.8.2001 / 16:26:33 / cg"
! !

!TrianglePuzzle methodsFor:'drawing'!

showPieceNr:index piece:pieceVar rotation:rotation
    |pos pieceName piece rotatedImage|

    pos := self slotPosition:index.
    pieceName := pieceVar string.
    piece := pieces at:pieceName.
    piece container removeComponent:piece.

    "/ pos is the position of the top corner of the triangle,
    "/ or or the top baseLines center;
    "/ translate as required for pieces bitmap
    pos := pos - ((piece width // 2)@0).

    rotatedImage := piece image.
    (self slotIsFlipped:index) ifTrue:[
        rotatedImage := (rotatedImage rotated:180).    
    ].
    (rotation ~= 0) ifTrue:[
        rotatedImage := (rotatedImage rotated:(rotation*120)).    
    ].
    piece image:rotatedImage.

    piece moveTo:pos.
    boardView addComponent:piece.

    Transcript show:pieceName; show:'    '; show:rotation; cr.
    index == 9 ifTrue:[
        Transcript cr.
    ].

    "
     self new query:'solve()'
    "

    "Modified: / 23.8.2001 / 16:38:03 / cg"
! !

!TrianglePuzzle methodsFor:'prolog-clauses'!

%
% only a check, to detect typing errors (in piece*)
% Must return P1=piece1 ..... P9=piece9
%

checkPieces(P1,P2,P3,P4,P5,P6,P7,P8,P9) :-
    rotatedPiece(P1, bugHead, blueFrogTail, butterFlyHead),
    rotatedPiece(P2, antTail, bugHead, blueFrogTail),
    rotatedPiece(P3, antTail, blueFrogTail, redFrogTail),
    rotatedPiece(P4, antTail, butterFlyHead, redFrogHead),
    rotatedPiece(P5, antHead, butterFlyHead, blueFrogHead),
    rotatedPiece(P6, blueFrogHead, butterFlyTail, redFrogTail),
    rotatedPiece(P7, redFrogHead, bugHead, blueFrogTail),
    rotatedPiece(P8, antHead, redFrogTail, bugTail),
    rotatedPiece(P9, bugHead, blueFrogTail, butterFlyTail).

%
% checkPieces(P1,P2,P3,P4,P5,P6,P7,P8,P9)           
%
% checkPieces(piece1,piece2,piece3,piece4,piece5,piece6,piece7,piece8,piece9)           
%
!

%
% fit(A,B) - true, if A-B is a legal pair (bidirectional) (i.e. antHead-antTail) 
%

fit(A , B) :- qfit(A , B).
fit(A , B) :- qfit(B , A).

%
% fit(A,B)  
%
!

%
% isPiece(P) - enumerate pieces / true, if P is a piece 
%
isPiece(piece1).
isPiece(piece2).
isPiece(piece3).
isPiece(piece4).
isPiece(piece5).
isPiece(piece6).
isPiece(piece7).
isPiece(piece8).
isPiece(piece9).

%
% isPiece(P)    
% isPiece(piece1)  
%
!

%
% one of the pieces
%

% piece1(bugHead, blueFrogTail, butterFlyHead).
piece1(bugHead, blueFrogTail, butterFlyTail).
!

%
% one of the pieces
%

piece2(redFrogTail , antTail, blueFrogTail).
!

%
% one of the pieces
%

piece3(blueFrogHead , antHead , butterFlyHead).
!

%
% one of the pieces
%

piece4(blueFrogTail, butterFlyHead , bugHead).
!

%
% one of the pieces
%

% piece5(antHead , butterFlyHead , blueFrogHead).
piece5(blueFrogTail , redFrogHead , bugHead).
!

%
% one of the pieces
%

piece6(butterFlyTail , redFrogTail, blueFrogHead).
!

%
% one of the pieces
%

% piece5(antHead , butterFlyHead , blueFrogHead).
% piece7(blueFrogTail , redFrogHead , bugHead).

piece7(butterFlyHead, redFrogHead, antTail).
!

%
% one of the pieces
%

piece8(antHead , redFrogTail , bugTail).
!

%
% one of the pieces
%

piece9(antTail , bugHead , blueFrogTail).
!

%
% qfit(A,B) - true, if A-B is a legal pair (single direction)
%

qfit(bugHead , bugTail).
qfit(redFrogHead , redFrogTail).
qfit(blueFrogHead , blueFrogTail).
qfit(butterFlyHead , butterFlyTail).
qfit(antHead , antTail).

%
% qfit(A,B)  
%
!

%
% qrotatedPiece(P , A,B,C, R) - generate rotations of a given piece; leave rot-count in R (optional).
%

qrotatedPiece(P , A,B,C) :- qrotatedPiece(P , A,B,C,X).

qrotatedPiece(P , A,B,C,0) :- P(A,B,C).
qrotatedPiece(P , A,B,C,1) :- P(C,A,B).
qrotatedPiece(P , A,B,C,2) :- P(B,C,A).


%
% qrotatedPiece(piece1, A, B, C, R)
% qrotatedPiece(piece1, A, B, C)
%
!

%
% rotatedPiece(P , A,B,C) - generate pieces and rotations
%

rotatedPiece(P , A,B,C)    :- isPiece(P), qrotatedPiece(P , A,B,C,X).

%
% rotatedPiece(piece1, A, B, C)           
% rotatedPiece(P, A, B, C)           
% rotatedPiece(P, bugHead, blueFrogTail, butterFlyTail)  
% rotatedPiece(P, butterFlyTail, bugHead, blueFrogTail)  
% rotatedPiece(P, blueFrogTail, butterFlyTail, bugHead)  
% rotatedPiece(P, blueFrogTail, blueFrogHead, bugHead)   
%
%
% are there two equal pieces ?
%    rotatedPiece(P1, A, B, C) , rotatedPiece(P2, A, B, C) , \=(P1, P2)
!

%
% solve - solve the puzzle
%

solve() :- solve( P1, P2, P3, P4, P5, P6, P7, P8, P9 ).

solve( P1, P2, P3, P4, P5, P6, P7, P8, P9 ) :-
%
%                               
%                           /\
%                          /  \
%                         /    \
%                        /A7  B7\
%                       /        \
%                      /    C7    \
%                     *------------*
%                    /\     A1     /\
%                   /  \          /  \
%                  /    \        /    \
%                 /A6  B6\C1  B1/A2  B2\
%                /        \    /        \
%               /    C6    \  /    C2    \
%              *------------**------------*
%             /\     A5     /\     A3     /\
%            /  \          /  \          /  \
%           /    \        /    \        /    \
%          /A9  B9\C5  B5/A4  B4\C3  B3/A8  B8\
%         /        \    /        \    /        \
%        /    C9    \  /    C4    \  /    C8    \
%       *------------**------------**------------*

        isPiece(P1),                                            % take any piece
        isPiece(P2), \=(P2 , P1),                               % and another one
        qrotatedPiece( P1 , A1,B1,C1 , R1 ),                    % try rotations
        qrotatedPiece( P2 , A2,B2,C2 , R2 ),                    % try rotations
        fit(B1, A2),                                            % so that A-side of 2nd piece fits B-side of 1st piece 

        isPiece(P3), \=(P3 , P1), \=(P3 , P2),                  % take another piece

        qrotatedPiece( P3 , A3,B3,C3 , R3 ),                    % rotate
        fit(C2, A3),                                            % so that A-side of 3rd piece fits C-side of 2nd piece

        isPiece(P4), \=(P4 , P1), \=(P4 , P2), \=(P4 , P3),     % take another piece
        qrotatedPiece( P4 , A4,B4,C4 , R4 ),                    % rotate
        fit(B4, C3),                                            % so that C-side of 4th piece fits B-side of 3rd piece

        isPiece(P5), \=(P5 , P1), \=(P5 , P2), \=(P5 , P3), \=(P5 , P4), 
        qrotatedPiece( P5 , A5,B5,C5 , R5 ),
        fit(B5, A4),

        isPiece(P6), \=(P6 , P1), \=(P6 , P2), \=(P6 , P3), \=(P6 , P4), \=(P6 , P5),
        qrotatedPiece( P6 , A6,B6,C6 , R6 ),
        fit(C6, A5),
        fit(B6, C1),

        isPiece(P7), \=(P7 , P1), \=(P7 , P2), \=(P7 , P3), \=(P7 , P4), \=(P7 , P5), \=(P7 , P6),
        qrotatedPiece( P7 , A7,B7,C7 , R7 ),
        fit(C7, A1),

        isPiece(P8), \=(P8 , P1), \=(P8 , P2), \=(P8 , P3), \=(P8 , P4), \=(P8 , P5), \=(P8 , P6), \=(P8 , P7),
        qrotatedPiece( P8 , A8,B8,C8 , R8 ),
        fit(B3, A8),

        isPiece(P9), \=(P9 , P1), \=(P9 , P2), \=(P9 , P3), \=(P9 , P4), \=(P9 , P5), \=(P9 , P6), \=(P9 , P7), \=(P9 , P8),
        qrotatedPiece( P9 , A9,B9,C9 , R9 ),
        fit(B9, C5),

        % here we are - show the result.
        send(instance, showPieceNr:piece:rotation:, [1, P7, R1] ) ,
        send(instance, showPieceNr:piece:rotation:, [2, P6, R6] ) ,
        send(instance, showPieceNr:piece:rotation:, [3, P1, R1] ) ,
        send(instance, showPieceNr:piece:rotation:, [4, P2, R2] ) ,
        send(instance, showPieceNr:piece:rotation:, [5, P9, R9] ) ,
        send(instance, showPieceNr:piece:rotation:, [6, P5, R5] ) ,
        send(instance, showPieceNr:piece:rotation:, [7, P4, R4] ) ,
        send(instance, showPieceNr:piece:rotation:, [8, P3, R3] ) ,
        send(instance, showPieceNr:piece:rotation:, [9, P8, R8] )
        .
        
%"
 trace , solve(P1, P2, P3, P4, P5, P6, P7, P8, P9)

 self new query:'solve(P1, P2, P3, P4, P5, P6, P7, P8, P9)'

 self new 
        queryValues:'solve(P1, P2, P3, P4, P5, P6, P7, P8, P9).' 
        allInto:[:x | Transcript showCR:x]
"%
! !

!TrianglePuzzle methodsFor:'ui stuff'!

menuSolvePuzzle
    self 
        queryValues:'solve(P1, P2, P3, P4, P5, P6, P7, P8, P9).' 
        allInto:[:x | ^ self]

    "Created: / 23.8.2001 / 16:16:07 / cg"
    "Modified: / 23.8.2001 / 16:34:55 / cg"
!

slotIsFlipped:idx
    ^ #( 
          true    "/ 3
          false   "/ 4
          true    "/ 8
          false   "/ 7
          true    "/ 6
          false   "/ 2
          false   "/ 1
          false   "/ 9
          false   "/ 5
      ) at:idx

    "Created: / 23.8.2001 / 16:33:17 / cg"
!

slotPosition:idx
    ^ (#( 
          (288 221)   "/ 3
          (369 223)   "/ 4
          (370 367)   "/ 8
          (290 372)   "/ 7
          (200 369)   "/ 6
          (203 224)   "/ 2
          (288  77)   "/ 1
          (459 368)   "/ 9
          (118 443)   "/ 5
        

"/          (288  77) 
"/
"/          (203 224) 
"/          (288 221)
"/          (369 223) 
"/
"/          (118 443)
"/          (200 369)
"/          (290 372)
"/          (370 367)
"/          (459 368)
      ) at:idx) changeClassTo:Point

    "Created: / 23.8.2001 / 14:59:39 / cg"
    "Modified: / 23.8.2001 / 16:22:23 / cg"
! !

!TrianglePuzzle::Piece methodsFor:'accessing'!

computeBoundingBox
    "recompute the frame - must be done after every size/position change.
     The frame may be larger than the actual real estate used 
     but NEVER smaller for correct clipping"

    ^ Rectangle origin:origin corner:(origin + image extent)
!

container
    ^ container
!

container:aComponent
    container := aComponent
!

image
    "return the value of the instance variable 'image' (automatically generated)"

    ^ image
!

image:something
    "set the value of the instance variable 'image' (automatically generated)"

    image := something.
!

moveTo:newOrigin
    origin := newOrigin.
    frame := self computeBoundingBox.
!

owningApplication
    "return the value of the instance variable 'owningApplication' (automatically generated)"

    ^ owningApplication
!

owningApplication:something
    "set the value of the instance variable 'owningApplication' (automatically generated)"

    owningApplication := something.
!

parent:aComponent
    self container:aComponent
! !

!TrianglePuzzle::Piece methodsFor:'drawing'!

drawIn:aView offset:anOffset
    aView displayForm:image x:(anOffset x + origin x) y:(anOffset y + origin y)
! !

!TrianglePuzzle::Piece methodsFor:'events'!

buttonPress:button x:cx y:cy
    |prevContainer dropObject v|

Transcript showCR:origin + (cx@cy).

    dropObject := DropObject newImage:image.

    prevContainer := container.
    container removeComponent:self.
    prevContainer invalidate:frame repairNow:true.

    DragAndDropManager startDrag:(Array with:dropObject)
                            from:prevContainer
                          offset:(cx@cy)
                           atEnd:[:targetView :targetViewId :screenPoint :targetPoint|
                                    |newContainer|

                                    ((targetView ~~ container)
                                    and:[targetView topView == owningApplication window])
                                    ifTrue:[
                                        Transcript showCR:'target  view: ', targetView   printString.
                                        Transcript showCR:'target    id: ', targetViewId printString.
                                        Transcript showCR:'point screen: ', screenPoint  printString.
                                        Transcript showCR:'point target: ', targetPoint  printString.
                                        "/ targetView invalidate:frame.

                                        self moveTo:targetPoint - (cx@cy).
                                        newContainer := targetView.
                                    ] ifFalse:[
                                        targetView device beep.
                                        newContainer := prevContainer.
                                    ].
                                    newContainer addComponent:self.
                                 ]
                         display:image.

    "Modified: / 23.8.2001 / 16:05:53 / cg"
!

buttonRelease:button x:cx y:cy
!

containsPointX:xView y:yView
    |mask xImg yImg|

    mask := image mask.
    mask isNil ifTrue:[
        ^ true
    ].

    xImg := xView - origin x.
    yImg := yView - origin y.
    ^ (mask valueAtX:xImg y:yImg) == 1
! !

!TrianglePuzzle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prolog/TrianglePuzzle.st,v 1.5 2002/11/26 18:31:09 cg Exp $'
! !
