"{ Package: 'stx:goodies/prototypeBehavior' }"

Behavior subclass:#PrototypeBehavior
	instanceVariableNames:'shared'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Prototypes'
!

PrototypeBehavior comment:'PrototypeBehavior implements the behavior of PrototypeObjects. A PrototypeBehavior has simplified methods for creating and removing methods and slot accessors.

For good storage economy, PrototypeBehaviors are shared between structurally similar PrototypeObjects. When a PrototypeObject is cloned, its class is set to shared.

When a PrototypeObject changes structure (by adding/removing slots or methods) it checks to see whether its class is shared. If it is, a new PrototypeBehavior is created for this now structurally different object. This new PrototypeBehavior is owned, so further structural changes don''t create new PrototypeBehaviors.

Implementation detail: Methods stored in PrototypeBehaviors are specially marked to note whether they are slot accessors or normal methods. The marking mechanism has a slight chance of colliding with the temp name storage mechanism if a lot of temp names are used. Normal methods are stored with temp names, but their source is not recorded in the changes file as there would not be a named class to associate it with.'
!


!PrototypeBehavior methodsFor:'accessing'!

beOwned

	shared _ false
!

beShared

	shared _ true
!

classPool

	^superclass classPool
!

cloneForModifiedObject

	^self copy beOwned
!

delegateNames
        ^(self selectors select: [:each | 
                "/ (self compiledMethodAt: each) last = 252
                (self compiledMethodAt: each) literals last = 252
        ]) asSortedCollection
!

fromSuperclass: aSuperclass

        flags _ aSuperclass flags.
        instSize _ aSuperclass instSize.
        superclass _ aSuperclass.
        methodDictionary _ MethodDictionary new
!

isShared

	^shared == true
!

methodNames
        ^(self selectors select: [:each | 
                "/ (self compiledMethodAt: each) last < 250
                (self compiledMethodAt: each) literals last < 250
        ]) asSortedCollection
!

sharedPools

	^superclass sharedPools
!

slotNames

        ^(self selectors select: [:each | 
                "/ (self compiledMethodAt: each) last = 250
                (self compiledMethodAt: each) literals last = 250
        ]) asSortedCollection
! !

!PrototypeBehavior methodsFor:'compiling'!

compile: aString

	^self
		compile: aString
		withMarker: 0
!

compile: code withMarker: anInteger
        "Simplified version for prototypes"
        | methodNode selector method |

        methodNode _ self compilerClass new
                                compile: code
                                in: self
                                notifying: nil
                                ifFail: [:error | self error: error].

        selector _ methodNode selector.
        method _ methodNode.
        "/ methodNode encoder requestor: nil.  "Why was this not preserved?"
        "/ method _ methodNode generate: (#(0 0 0) copyWith: anInteger).
        method := methodNode class new:(methodNode literals size + 1).
        method cloneInstanceVariablesFrom:methodNode.
        method literals:((methodNode literals ? #()) copyWith:anInteger).

        "/ anInteger = 0 ifTrue: [method _ method copyWithTempNames: methodNode tempNames].
        self addSelector: selector withMethod: method.

        ^selector
!

compileDelegate: aString index: delegateIndex

	self
		compile: aString, '^delegates at:', delegateIndex printString
		withMarker: 252.

	self
		compile: aString, ':obj ^delegates at:', delegateIndex printString, ' put:obj'
		withMarker: 253.

	
!

compileSlot: aString index: slotIndex

	self
		compile: aString, '^slots at:', slotIndex printString
		withMarker: 250.

	self
		compile: aString, ':obj ^slots at:', slotIndex printString, ' put:obj'
		withMarker: 251.

	
! !

!PrototypeBehavior methodsFor:'instance creation'!

basicNew: anInteger

	self error: 'disallowed'
!

new

	self error: 'disallowed'
!

new: anInteger

	self error: 'disallowed'
! !

!PrototypeBehavior class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prototypeBehavior/PrototypeBehavior.st,v 1.1 2003/09/09 09:45:41 cg Exp $'
! !
