"{ Package: 'stx:goodies/prototypeBehavior' }"

InspectorView subclass:#PrototypeInspector
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'System-Prototypes'
!

PrototypeInspector comment:'This special inspector serves as a user interface for PrototypeObjects. It displays slots and methods in an intuitive way. Note that it does not work very well in the Morphic environment.'
!


!PrototypeInspector methodsFor:'as yet unclassified'!

accept: aString
	| slotName selector result |

	slotName _ self selectedSlotName.

	((object methodNames includes: slotName) or: [slotName = self methodsHeading])
		ifTrue: [
			selector _ object addMethod: aString.
			selector = slotName ifFalse: [
				self changed: #fieldList.
				self toggleIndex: (self fieldList indexOf: selector)].
			^true].

	((object slotNames includes: slotName) or: [slotName = self slotsHeading])
		ifTrue: [
			result _ self doItReceiver class evaluatorClass new
				evaluate: (ReadStream on: aString)
				in: self doItContext
				to: self doItReceiver
				notifying: nil	"fix this"
				ifFail:  [^ false].

			result == #failedDoit ifFalse: 
				[contents _ result printString.
				self replaceSelectionValue: result.	"may put contents back"
				self changed: #contents.
			^ true]].
	^false
!

contentsForSlotName: slotName

	slotName = 'self' ifTrue: [^object printString].

	slotName = self slotsHeading
		ifTrue: [^'Enter slot value to create a new slot'].

	slotName = self delegatesHeading
		ifTrue: [^'List of delegates to this object'].

	slotName = self methodsHeading
		ifTrue: [^'method

	"New method in Smalltalk syntax. Remember to access slots via message sends!!"' asText makeSelectorBoldIn: object class].

	(object methodNames includes: slotName)
		ifTrue: [^(object methodSourceAt: slotName) asText makeSelectorBoldIn: object class].

	^(object perform: slotName) printString
!

contentsIsString
	"Hacked so contents empty when deselected and = long printString when item 2"
	| slotName |

	selectionIndex = 0 ifTrue: [^true].

	slotName _ self selectedSlotName.

	^ (object methodNames includes: slotName) | (self methodsHeading = slotName) | (self slotsHeading = slotName)
!

delegatesHeading

	^'--- delegates ---' asText allBold
!

fieldList

	^OrderedCollection new
		add: 'self';
		add: self slotsHeading;
		addAll: object slotNames;
		add: self delegatesHeading;
		addAll: object delegateNames;
		add: self methodsHeading;
		addAll: object methodNames;
		yourself
!

fieldListMenu: aMenu

	^ aMenu labels: 'inspect
inspect references
clone
remove slot/method
update
'
	lines: #(3 4)
	selections: #(inspectSelection objectReferencesToSelection inspectClone  removeSlot updateFieldList).

!

inspectClone

	^self selection clone inspect
!

methodsHeading

	^'--- methods ---' asText allBold
!

removeSlot
	| slotName |

	slotName _ self selectedSlotName.

	(object slotNames includes: slotName)
		ifTrue: [object removeSlot: slotName]
		ifFalse: [(object methodNames includes: slotName)
					ifTrue: [object removeMethod: slotName]].

	selectionIndex _ 0.
	self changed: #fieldList
!

replaceSelectionValue: anObject 
	| slotName |

	slotName _ self selectedSlotName.

	(object slotNames includes: slotName)
		ifTrue: [object perform: (slotName, ':') asSymbol with: anObject]
		ifFalse: [
			slotName _ FillInTheBlank request: 'Name of slot to store into (empty to ignore)?'.
			slotName isEmpty ifFalse: [
				object addSlot: slotName withValue: anObject.
				self changed: #fieldList.
				self toggleIndex: (self fieldList indexOf: slotName asSymbol)]]
!

selectedSlotName

	selectionIndex = 0 ifTrue: [^nil].

	^self fieldList atPin: selectionIndex
!

selection
	| slotName |

	selectionIndex = 0 ifTrue: [^nil].
	selectionIndex = 1 ifTrue: [^object].
	slotName _ self selectedSlotName.
	slotName = self slotsHeading
		ifTrue: [^'Enter slot value to create a new slot'].

	slotName = self delegatesHeading
		ifTrue: [^'List of delegates to this object'].

	slotName = self methodsHeading
		ifTrue: [^'method

	"New method in Smalltalk syntax. Remember to access slots via message sends!!"' asText makeSelectorBoldIn: object class].

	(object methodNames includes: slotName)
		ifTrue: [^(object methodSourceAt: slotName) asText makeSelectorBoldIn: object class].

	^object perform: slotName
!

slotsHeading

	^'--- slots ---' asText allBold
!

updateFieldList
	| slotName |

	slotName _ self selectedSlotName.
	selectionIndex _ 0.
	self changed: #fieldList.
	self toggleIndex: (self fieldList indexOf: slotName)
! !

!PrototypeInspector class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prototypeBehavior/PrototypeInspector.st,v 1.1 2003/09/09 09:45:32 cg Exp $'
! !
