"{ Package: 'stx:goodies/rST' }"

Object subclass:#RSTKeyManager
	instanceVariableNames:'lastKey dictionary mutex onNewKey'
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Core'
!


!RSTKeyManager methodsFor:'accessing'!

at: keyObject
	"anser the object that the key was asigned before"
^ self at:keyObject ifAbsent:[self error:'Object no found for key:', keyObject asString]
!

at: keyObject ifAbsent: absentBlock 
	"anser the object that the key was asigned before"
	| value |
	mutex
		critical: [self flag: #TODO.
			"TODO: be carefull when WeakDictionary will be used"
			value := dictionary
						at: keyObject
						ifAbsent: [].
			value
				ifNil: [value := absentBlock value]].
	^ value
!

at: keyObject put: anObject
	"asign keyObject to anObject"
	^ dictionary at: keyObject put: anObject
!

hasKey: anObject 
	"answer if anObject has a key"
	anObject isNil
		ifTrue: [^ self error: 'can store a nil'].

	mutex
		critical: [dictionary
				keyAtIdentityValue: anObject
				ifAbsent: [^ false]].
	^ true
!

keyFor: anObject 
	"gives a key for anObject, the key will be the same in future calls"
	| key |
	anObject isNil
		ifTrue: [^ self error: 'can store a nil'].
	mutex
		critical: [key := dictionary
						keyAtIdentityValue: anObject
						ifAbsent: [self newKeyFor: anObject]].
	^ key
!

onNewKey: aBlock
	onNewKey := aBlock
!

size
	"answer an estimated size"
	^ mutex
		critical: [dictionary size]
! !

!RSTKeyManager methodsFor:'initialization'!

initialize
	lastKey := 0.
	mutex := Semaphore forMutualExclusion.
	self initializeDictionary.
	onNewKey := nil
!

initializeDictionary
	"hook method"
	dictionary := Dictionary new
! !

!RSTKeyManager methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: ' last key=';
		 nextPutAll: lastKey asString;
		 nextPutAll: ', objects count=';
		 nextPutAll: dictionary size asString
! !

!RSTKeyManager methodsFor:'private'!

depurate
	"PRIVATE: This method is called from a critical section, be carefull to call 
	it without synchonization"
	
!

newKeyFor: anObject 
	"PRIVATE: This method is called from a critical section, be carefull to call 
	it without synchronization"
	| lastKeyAsString |
	lastKey := lastKey + 1.
	lastKeyAsString := 'OID-'
				, (self printHexFrom: 65535 atRandom)
				, (self printHexFrom: lastKey)
				, (self printHexFrom: 65535 atRandom).
	dictionary at: lastKeyAsString put: anObject.
	onNewKey isNil ifFalse:[onNewKey value: anObject value: lastKeyAsString].
	(lastKey isDivisibleBy: 25)
		ifTrue: [self depurate].
	^ lastKeyAsString
!

printHexFrom: anInteger
	"Noury Bouraqadi 9/22/2006 02:41"
	
	^(anInteger printPaddedWith: $0 to: 4 base: 16) allButFirst: 3
! !

!RSTKeyManager methodsFor:'removing'!

removeKey: keyObject 
mutex
		critical: [
	^ dictionary removeKey: keyObject]
! !

!RSTKeyManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTKeyManager.st,v 1.1 2011/12/28 22:58:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTKeyManager.st,v 1.1 2011/12/28 22:58:28 cg Exp $'
! !
