"{ Package: 'stx:goodies/rST' }"

Object subclass:#RSTRememberer
	instanceVariableNames:'remembered mutex'
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Core'
!


!RSTRememberer methodsFor:'accessing'!

allNamed: nameObject 
	"mp 4/20/2006 16:25"
	"answer all objects named nameObject"
	
	| named |
	named := Bag new.
	mutex critical: [
		remembered keysAndValuesDo: [:object :names | 
			(names keys includes: nameObject) 
				ifTrue:[named add: object]]].
	^ named
! !

!RSTRememberer methodsFor:'initialization'!

initialize
	remembered := IdentityDictionary new.
	mutex := Semaphore forMutualExclusion
! !

!RSTRememberer methodsFor:'maintenance'!

maintenance
	"within time and time, check for objects to forget"
	
	mutex critical: [
		| current toForget | 
		current := Time totalSeconds.
		remembered := remembered select: [:eachRemembered | 
			eachRemembered keysAndValuesRemove: [:name :expires | 
				expires notNil and: [expires <= current]].
			eachRemembered notEmpty]]
! !

!RSTRememberer methodsFor:'remembering & forgetting'!

forget: anObject named: nameObject 
	"forget anObject"
	
	mutex critical: [
		| names | 
		names := remembered 
						at: anObject 
						ifAbsent: [^ anObject].
		names 
			removeKey: nameObject 
			ifAbsent: [].
		names isEmpty ifTrue: [remembered removeKey: anObject]]
!

forgetAllNamed: nameObject 
	"forget anObject"
	
	| named | 
	named := self allNamed: nameObject.
	named do: [:object | self forget: object named: nameObject]
!

remember: anObject forSeconds: anInteger naming: nameObject 
	"remember anObject at least for anInteger seconds"
	
	mutex critical: [
		| names expires | 
		names := remembered
					at: anObject
					ifAbsentPut: [Dictionary new].
		expires := anInteger ifNotNil: [Time totalSeconds + anInteger].
		names at: nameObject put: expires]
!

rememberForEver: anObject naming: nameObject 
	"remember anObject for ever"
	^ self
		remember: anObject
		forSeconds: nil
		naming: nameObject
! !

!RSTRememberer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRememberer.st,v 1.1 2011/12/28 22:58:59 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRememberer.st,v 1.1 2011/12/28 22:58:59 cg Exp $'
! !
