"{ Package: 'stx:goodies/rST' }"

Object subclass:#RSTRemoteObjectID
	instanceVariableNames:'brokerID objectID'
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Core'
!


!RSTRemoteObjectID class methodsFor:'instance creation'!

brokerID: brokerIDString objectID: objectIDString 
	"creates an instance with brokerIDString and objectIDString"
	^ self new initializeBrokerID: brokerIDString objectID: objectIDString
!

fromString: aString 
	"creates an instance from aString with the format objectID@brokerID"
	| atPos |
	atPos := aString indexOf: $@.
	atPos isZero
		ifTrue: [self error: 'invalid format'].
	^ self
		brokerID: (aString allButFirst: atPos)
		objectID: (aString first: atPos - 1)
! !

!RSTRemoteObjectID methodsFor:'accessing'!

brokerID
	^ brokerID
!

objectID
	^ objectID
! !

!RSTRemoteObjectID methodsFor:'comparing'!

= anObject 
	self class == anObject class
		ifFalse: [^ false].
	^ brokerID = anObject brokerID
		and: [objectID = anObject objectID]
!

hash
	^ brokerID hash bitXor: objectID hash
! !

!RSTRemoteObjectID methodsFor:'converting'!

convertLocalhostAddressIn: brokerIDString 
	| portString ipStream ipString brokerIDStream |
	brokerIDStream := ReadStream on: brokerIDString.
	ipStream := brokerIDStream upTo: $:.
	(ipStream contents = 'localhost' or: [ipStream contents = '127.0.0.1']) 
		ifTrue: [ipString := NetNameResolver localAddressString]
		ifFalse: [ipString := ipStream contents].
	portString := brokerIDStream upToEnd contents.
	^ ipString , ':' , portString
! !

!RSTRemoteObjectID methodsFor:'initialization'!

initializeBrokerID: brokerIDString objectID: objectIDString 
	brokerID := self convertLocalhostAddressIn: brokerIDString.
	objectID := objectIDString
! !

!RSTRemoteObjectID methodsFor:'printing'!

printOn: aStream 
	aStream nextPutAll: objectID asString;
		 nextPut: $@;
		 nextPutAll: brokerID asString
! !

!RSTRemoteObjectID methodsFor:'resolving objects'!

asLocalObject
	"convert to an apropiate local object, could be a proxy or the real object"
	^ RSTBroker instance localObjectFor: self
!

asProxy
	"convert me to a proxy"
	^ RSTBroker instance proxyFor: self
! !

!RSTRemoteObjectID class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRemoteObjectID.st,v 1.1 2011/12/28 22:59:23 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRemoteObjectID.st,v 1.1 2011/12/28 22:59:23 cg Exp $'
! !
