"{ Package: 'stx:goodies/rST' }"

ProtoObject subclass:#RSTRemoteObjectProxy
	instanceVariableNames:'remoteObjectID haltOnce errorOnce hashCode remoteClassName'
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Core'
!


!RSTRemoteObjectProxy class methodsFor:'instance creation'!

remoteObjectID: aRemoteObjectID
	^ self new initializeRemoteObjectID: aRemoteObjectID
! !

!RSTRemoteObjectProxy methodsFor:'accessing'!

hash
	^hashCode
!

xxxRemoteObjectID
	^ remoteObjectID
! !

!RSTRemoteObjectProxy methodsFor:'class membership'!

className
	^remoteClassName
!

isKindOf: aClass
	"We make the assumption that 2 classes with the same name and located in 2 different images are identical (code replication)"
	^self className = aClass name
! !

!RSTRemoteObjectProxy methodsFor:'comparing'!

= anObject
	self == anObject ifTrue: [^true].
	self className = anObject className ifFalse: [^false].
	^self xxxSend: (Message selector: #= argument: anObject)
! !

!RSTRemoteObjectProxy methodsFor:'error handling'!

clearErrorOnce
	"Turn off the error once flag."
	errorOnce := false
!

clearHaltOnce
	"Turn off the halt once flag."
	haltOnce := false
!

error: aString 
	"Throw a generic Error exception."
	self errorOnce: aString
!

errorOnce: aString 
	"Throw a generic Error exception."
	
	self errorOnceEnabled ifTrue: [
		self clearErrorOnce.
		^ Error signal: aString]
!

halt
	"This is the typical message to use for inserting breakpoints during  
	debugging. It behaves like halt:, but does not call on halt: in order to  
	avoid putting this message on the stack. Halt is especially useful when  
	the breakpoint message is an arbitrary one."
	self haltOnce
!

haltOnce
	"Halt unless we have already done it once."
	
	self haltOnceEnabled ifTrue: [
		self clearHaltOnce.
		Halt signal]
!

setErrorOnce
	"Turn on the error once flag."
	errorOnce := true
!

setHaltOnce
	"Turn on the halt once flag."
	haltOnce := true
! !

!RSTRemoteObjectProxy methodsFor:'initialization'!

initialize
	super initialize.
	self setHaltOnce.
	self setErrorOnce
!

initializeHashCode
	hashCode := self xxxSend: (Message selector: #hash)
!

initializeRemoteClassName
	remoteClassName := self xxxSend: (Message selector: #className)
!

initializeRemoteObjectID: aRemoteObjectID 
	remoteObjectID := aRemoteObjectID.
	self initializeHashCode.
	self initializeRemoteClassName 
! !

!RSTRemoteObjectProxy methodsFor:'printing'!

printString
	"Noury Bouraqadi 9/21/2006 16:20"
	
	self isReachable ifTrue: [^self xxxSend: (Message selector: #printString)].
	^'rST proxy of unreachable remote object'
! !

!RSTRemoteObjectProxy methodsFor:'system primitives'!

become: otherObject 
	"when remote objects participates, only remote objects that live in the 
	same image could be becomed"
	otherObject isRemote
		ifFalse: [^ self error: 'invalid #become: (the receiver is remote and the other is local)'].
	(self xxxSameImage: otherObject)
		ifFalse: [^ self error: 'invalid #become: (the receiver lives in a diferent image than the other)'].
	self
		xxxSend: (Message selector: #become: argument: otherObject)
!

doesNotUnderstand: aMessage
	^ self xxxSend: aMessage
!

xxxSend: aMessage 
	"send this message to the remote object represented by me"
	^ RSTBroker instance sendMessage: aMessage to: self xxxRemoteObjectID
! !

!RSTRemoteObjectProxy methodsFor:'testing'!

errorOnceEnabled
	^ errorOnce
!

haltOnceEnabled
	^ haltOnce
!

isReachable
	^ [self xxxSend: (Message selector: #isReachable)] 
		 on: RSTRemoteObjectUnreachable, RSTObjectNotFound 
		 do: [false]
!

isRemote
	"answer if the object is a remote one"
	^ true
!

xxxSameImage: aRemoteObjectProxy 
	"answer if both objects live in the same Image"
	^ self xxxRemoteObjectID brokerID = aRemoteObjectProxy xxxRemoteObjectID brokerID
! !

!RSTRemoteObjectProxy methodsFor:'user interface'!

basicInspect
	"Create and schedule an Inspector in which the user can examine the  
	receiver's variables. This method should not be overriden."
	Inspector openOn: self withEvalPane: false
!

defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the 
	receiver. "
	| remoteLabel |
	remoteLabel := self
				xxxSend: (Message selector: #defaultLabelForInspector).
	^ 'Remote Object: ' , remoteLabel , ' (' , remoteObjectID asString , ')'
!

inspect
	"Open an RSTRemoteInspector on the receiver. Use basicInspect to get a normal (less useful) type of inspector."

	self inspectorClass openOn: self withEvalPane: true
!

inspectWithLabel: aLabel 
	"Open an RSTRemoteInspector on the receiver. Use basicInspect to get a 
	normal (less useful) type of inspector."

	self inspectorClass 
		openOn: self
		withEvalPane: true
		withLabel: aLabel
!

inspectorClass
	^RSTRemoteInspector
! !

!RSTRemoteObjectProxy class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRemoteObjectProxy.st,v 1.1 2011/12/28 22:59:29 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTRemoteObjectProxy.st,v 1.1 2011/12/28 22:59:29 cg Exp $'
! !
