"{ Package: 'stx:goodies/rST' }"

Object subclass:#RSTSocketStreamPool
	instanceVariableNames:'address port freeStreams busyStreams streamAccessLock'
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Core'
!


!RSTSocketStreamPool class methodsFor:'instance creation'!

addressAndPortFromBrokerID: brokerIDString 
	^self new
		addressAndPortFromBrokerID: brokerIDString;
		yourself
! !

!RSTSocketStreamPool methodsFor:'accessing'!

address
	^ address
!

address: anObject
	address := anObject
!

addressString
	^NetNameResolver stringFromAddress: self address
!

port
	^ port
!

port: anObject
	port := anObject
! !

!RSTSocketStreamPool methodsFor:'connections'!

releaseStream: stream
	stream close.
!

socketStream
	^ self newSocketStream.
! !

!RSTSocketStreamPool methodsFor:'connections-private'!

freeStream
	| stream |
	freeStreams ifEmpty: [^self newSocketStream].
	stream := freeStreams anyOne.
	freeStreams remove: stream.
	stream isConnected ifFalse: [
		[stream close] fork.
		^self freeStream].
	^ stream
!

newSocketStream
	| socket |
	socket := nil.
	3 timesRepeat: 
			[socket ifNotNil: [socket closeAndDestroy].
			socket := Socket newTCP.
			
			[socket 
				connectTo: self address
				port: self port
				waitForConnectionFor: self secondsToWaitForSocketConnection] 
					on: ConnectionTimedOut
					do: 
						[:timeOutEx | 
						RSTBroker instance logException: timeOutEx.
						Processor yield].
			socket isConnected ifTrue: [^ (SocketStream on: socket) timeout: 3; yourself.]].
	RSTRemoteObjectUnreachable 
		signal: 'Cannot connect to: ' , self addressString , ':' 
				, self port asString
!

secondsToWaitForSocketConnection
	^ 3
! !

!RSTSocketStreamPool methodsFor:'initialize-release'!

addressAndPortFromBrokerID: brokerIDString 
	| colonPos serverAddress serverPort |
	colonPos := brokerIDString findString: ':'.
	colonPos isZero 
		ifTrue: 
			[serverAddress := brokerIDString.
			serverPort := '9999']
		ifFalse: 
			[serverAddress := brokerIDString first: colonPos - 1.
			serverPort := brokerIDString allButFirst: colonPos].
	self address: (NetNameResolver addressForName: serverAddress).
	self port: serverPort asInteger
!

closeAndDestroy
	| closeBlock |
	closeBlock := [:stream| stream close].
	streamAccessLock critical: [
		freeStreams do: closeBlock.
		busyStreams do: closeBlock.
		self reset]
!

initialize
	super initialize.
	streamAccessLock := Semaphore forMutualExclusion.
	self reset
!

reset
	busyStreams := IdentitySet new.
	freeStreams := IdentitySet new
! !

!RSTSocketStreamPool class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTSocketStreamPool.st,v 1.1 2011/12/28 22:59:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/RSTSocketStreamPool.st,v 1.1 2011/12/28 22:59:41 cg Exp $'
! !
