"{ Package: 'stx:goodies/rST/samples' }"

Object subclass:#RSTSamples
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'rST-Samples'
!


!RSTSamples class methodsFor:'address configuration'!

clientPort
	^ 8888
!

serverAddress
	^ NetNameResolver localAddressString
!

serverBrokerID
	^ self serverAddress asString , ':' , self serverPort asString
!

serverPort
	^ 9999
! !

!RSTSamples class methodsFor:'private'!

collectionToString: aCollection
	| stream |
	stream := String new writeStream.
	stream nextPutAll: '('.
	aCollection
		do: [:each | stream nextPutAll: each asString]
		separatedBy: [stream nextPutAll: ', '].
	stream nextPutAll: ')'.
	^ stream contents
! !

!RSTSamples class methodsFor:'running client side'!

runClient
	"RSTSamples runClient"
	"Initialize an broker in the port 8888 and get a remote reference to a  
	Date named today"
	Transcript show: 'running client side rST example...';
		 cr.
	RSTBroker
		startOnPort: self clientPort
		logging: false.
	self runClientTodayTests.
	self runClientCollectionTests.
	self runClientTranscriptTests.
	self runClientPersonTests.
	self runClientPerformanceTests.
	Transcript show: 'everything is ok!!';
		 cr
!

runClientCollectionTests
	"in this example remoteCollection receives 2 remote references to local  
	blocks, then the remote collection send #value: and #value to these  
	locals blocks"
	| remoteCollection localCollectionAsString remoteCollectionAsString remoteCollectionAsStringConverter |
	remoteCollection := ('collection@' , self serverBrokerID) asLocalObject.
	remoteCollectionAsStringConverter := ('collectionAsStringConverter@' , self serverBrokerID) asLocalObject.
	remoteCollectionAsString := remoteCollectionAsStringConverter value.
	localCollectionAsString := self collectionToString: remoteCollection.
	Transcript show: 'remoteCollection=';
		 show: localCollectionAsString;
		 cr.
	localCollectionAsString = remoteCollectionAsString
		ifFalse: [self error: 'something wrong!!']
!

runClientPerformanceTests
	| remoteToday localToday remoteTomorrow |
	localToday := Date today.
	""
	remoteToday := ('today@' , self serverBrokerID) asLocalObject.
	remoteTomorrow := remoteToday addDays: 1.
	MessageTally
		spyOn: [100
				timesRepeat: [remoteTomorrow subtractDate: localToday]]
!

runClientPersonTests
	""
	| remotePerson1 remotePerson2 vc1 vc2 |
	remotePerson1 := ('person1@' , self serverBrokerID) asLocalObject.
	remotePerson2 := ('person2@' , self serverBrokerID) asLocalObject.
	""
	vc1 := RSTPersonViewController on: remotePerson1.
	vc2 := RSTPersonViewController on: remotePerson2.
	""
	vc1 test
		ifFalse: [self error: 'something wrong!!'].
	vc2 test
		ifFalse: [self error: 'something wrong!!']
!

runClientTodayTests
	"Initialize an broker in the port 8888 and get a remote reference to a  
	Date named today"
	| remoteToday localToday remoteTomorrow localTomorrow elapsed |
	localToday := Date today.
	localTomorrow := localToday addDays: 1.
	""
	"you must change the IP if you run the example in diferents machines"
	remoteToday := ('today@' , self serverBrokerID) asLocalObject.
	remoteToday inspect.
	" 
	hash return an integer so i receive a copy and the #= is a local  
	message"
	localToday hash = remoteToday hash
		ifFalse: [self error: 'something wrong!!'].
	" 
	remoteTomorrow is an remote object"
	remoteTomorrow := remoteToday addDays: 1.
	remoteTomorrow isRemote
		ifFalse: [self error: 'something wrong!!'].
	localTomorrow julianDayNumber = remoteTomorrow julianDayNumber
		ifFalse: [self error: 'something wrong!!'].
	"pass a reference to a local object (localTomorrow)"
	elapsed := remoteTomorrow subtractDate: localTomorrow.
	elapsed isRemote
		ifTrue: [self error: 'something wrong!!'].
	elapsed isZero
		ifFalse: [self error: 'something wrong!!']
!

runClientTranscriptTests
	| remoteTranscript |
	remoteTranscript := ('Transcript@' , self serverBrokerID) asLocalObject.
	remoteTranscript show: 'everything is ok!! (from client side)';
		 cr
! !

!RSTSamples class methodsFor:'running server side'!

serverStartup
	"RSTSamples serverStartup"
	"Initialize an broker in the port 9999 and export some objects for testing"
	| today collection |
	RSTBroker
		startOnPort: self serverPort
		logging: true.
	RSTBroker export: Transcript named: 'Transcript'.
	today := Date today.
	collection := {1. 'aString'. #aSymbol. today}.
	RSTBroker export: today named: 'today'.
	RSTBroker export: collection named: 'collection'.
	RSTBroker
		export: [self collectionToString: collection]
		named: 'collectionAsStringConverter'.
	RSTBroker
		export: (RSTPerson name: 'name1' eMail: 'eMail1')
		named: 'person1'.
	RSTBroker
		export: (RSTPerson name: 'name2' eMail: 'eMail2')
		named: 'person2'
! !

!RSTSamples class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rST/samples/RSTSamples.st,v 1.1 2011/12/28 23:04:01 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rST/samples/RSTSamples.st,v 1.1 2011/12/28 23:04:01 cg Exp $'
! !
