"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/rdoit' }"

Object subclass:#RDoItClient
	instanceVariableNames:'socket needResult isErrorResponse resultData'
	classVariableNames:'Logging ErrorLogging DefaultPortNrOrPath'
	poolDictionaries:''
	category:'Net-Communication-Smalltalk'
!

!RDoItClient class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This is the Smalltalk version of the rdoit command,
    as found in goodies/rdoit/rdoit.c.

    It sends a request to an rdoit server, bypassing the command execution
    overhead which is involved if the rdoit is called for that.

    [author:]
        Claus Gittinger         

    [see also:]
        RDoItServer Socket
"
!

examples
"
    if no return value is needed:
                                                        [exBegin]
        RDoItClient
            request:'Transcript flash'
            onHost:'felixm'
            port:8004.
                                                        [exEnd]

    with error:
                                                        [exBegin]
        RDoItClient
            request:'Foo flash'
            onHost:'felixm'
            port:8004.
                                                        [exEnd]

    normal:
                                                        [exBegin]
        RDoItClient
            evaluate:'1+2'
            onHost:'felixm'
            port:8004.  
                                                        [exEnd]
                                                        [exBegin]
        RDoItClient
            evaluate:'100 factorial'
            onHost:'felixm'
            port:8004.  
                                                        [exEnd]
                                                        [exBegin]
        RDoItClient
            request:'WorkspaceApplication open'
            onHost:'felixm'
            port:8004.
                                                        [exEnd]

                                                        [exBegin]
     Time millisecondsToRun:[
        100 timesRepeat:[
            RDoItClient
                request:'10 factorial'
                onHost:'felixm'
                port:8004.
        ]
     ]
                                                        [exEnd]
"
! !

!RDoItClient class methodsFor:'defaults'!

defaultConnectTimeoutMS
    " 10 seconds"

    ^ 10*1000
!

defaultPortNumber
    ^ self myServerClass defaultPortNumber
!

myServerClass
    ^ RDoItServer
! !

!RDoItClient class methodsFor:'public api'!

evaluate:expressionString onHost:aHostName port:portNr
    "send a request; return value is the printString of the result"

    ^ self new evaluate:expressionString onHost:aHostName port:portNr
!

request:expressionString onHost:aHostName port:portNr
    "send a request; no return value expected"

    ^ self new request:expressionString onHost:aHostName port:portNr
! !

!RDoItClient methodsFor:'private'!

closeSocket
    socket close
!

connectToHost:hostname
    self connectToHost:hostname port:(self class defaultPortNumber).
!

connectToHost:hostname port:portNr
    socket := Socket newTCPclientToHost:hostname port:portNr withTimeout:(self connectTimeoutMS)
!

receiveResult
    |part start stop|

"/ Transcript showCR:'x'.
    socket readWaitWithTimeoutMs:(self commandTimeoutMS).

    resultData := ''.
    [socket atEnd] whileFalse:[
        part := socket nextAvailable:1024.
        part notEmptyOrNil ifTrue:[
            resultData := resultData , part
        ].
    ].
    resultData size < 2 ifTrue:[
        resultData := nil.
        isErrorResponse := true.
        ^ nil.
    ].

    start := 2.
    stop := resultData size.

    ((resultData at:stop) == (Character nl)) ifTrue:[
        stop := stop - 1.
        ((resultData at:stop) == (Character return)) ifTrue:[
            stop := stop - 1.
        ].
    ].

    isErrorResponse := (resultData at:1) == $-.
    resultData := resultData copyFrom:start to:stop.
    ^ resultData
!

sendExpression:expressionString
    self sendExpression:expressionString needResult:needResult
!

sendExpression:expressionString needResult:needResult
    |prefix|

    socket isNil ifTrue:[ 
        self error:('cannot send expression - no connection') 
    ].
    needResult == false ifTrue:[
        prefix := ' '
    ] ifFalse:[
        prefix := '?'
    ].
    self sendExpressionLine:(prefix, expressionString).
!

sendExpressionLine:line
    socket nextPutAll:line.
    socket nextPut:(Character nl).
! !

!RDoItClient methodsFor:'public api'!

evaluate:expressionString onHost:aHostName port:portNr
    "returns the printString of the result - sigh"

    self request:expressionString onHost:aHostName port:portNr.

    isErrorResponse == true ifTrue:[
        ^ nil
    ].
    ^ resultData

    "
     self evaluate:'1+2' onHost:'felixm' port:8004       
     self evaluate:'100 factorial' onHost:'felixm' port:8004       
     self evaluate:' ''hello'',''world'' ' onHost:'felixm' port:8004  
    "
!

request:expressionString onHost:aHostName port:portNr
    self connectToHost:aHostName port:portNr.
    socket isNil ifTrue:[ 
        self error:('cannot connect to %1' bindWith:aHostName) 
    ].

    self sendExpression:expressionString.
    self receiveResult.
    self closeSocket.

    isErrorResponse == true ifTrue:[
        RDoitError 
            raiseRequestWith:resultData
            errorString:'Remote Doit Error'.
    ].

    "
     self request:'Tools::NewSystemBrowser open' onHost:'felixm' port:8004
     self request:'Foo open' onHost:'felixm' port:8004
    "
! !

!RDoItClient methodsFor:'queries'!

commandTimeoutMS
    ^ nil
!

connectTimeoutMS
    ^ self class defaultConnectTimeoutMS
! !

!RDoItClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rdoit/RDoItClient.st,v 1.4 2008/07/09 20:56:45 cg Exp $'
! !
