static char revision[] = "$Header: /cvs/stx/stx/goodies/rdoit/rdoit.c,v 1.46 2009/09/29 18:28:32 cg Exp $";

/* rdoit -- Remote doIt for st80 r4.  */
/* See class comment for RDoItServer. */

/* Written by Bill Voss <voss@cs.uiuc.edu> June 14th 1991 */
/* This is just a quick hack, not very well tested.       */

/* Modifications:
 *      claus [9-Jul-2003]:
 *              allow for unix domain sockets
 *
 *      claus:
 *              -p print option
 *               prints result of smalltalk expression on
 *               stdout; error text on stderr.
 *               Without '-print', command is simply sent as a
 *               datagram; without any feedback information.
 *
 *      claus:
 *              cleanup exitCodes:
 *                      2 usage / invalid argument
 *                      5 expression too long
 *                      6 internal (cannot happen)
 *                      7 socket open failed
 *                      8 socket write failed
 *                      9 socket read failed
 *                     10 script open failed
 *                     11 script read failed
 *                     12 connect timeout
 *                     13 response timeout
 *                      1 ST-expression error (only with -print)
 *      claus:
 *              -q option to suppress error messages
 *
 *      claus:
 *              -wait option to wait until the evaluation
 *                    has finished.
 *
 *      claus:
 *              if the cmd-name is stscript, check if its executable and send to stx for fileIn.
 *                    and then go into passMode (stdIn -> stx -> stdout/stderr).
 *
 *      claus:
 *              -t option to allow for a connect timeout setting
 *
 *      claus:
 *              changes to compile under win32 (bcc32)
 */

#undef DEBUG
#define DEBUG

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/types.h>
#include <setjmp.h>

#define DEFAULT_HOST                    "localhost"
#define DEFAULT_PORT                    "8004"
#define DEFAULT_CONNECT_TIMEOUT         "30"
#define DEFAULT_RESPONSE_TIMEOUT        "0"

#ifdef WIN32
# ifdef __BORLANDC__
#  include <windows.h>
# endif
# include <winsock.h>
# include <stddef.h>
#else
# include <sys/socket.h>
# include <sys/wait.h>
# include <netinet/in.h>
# include <sys/un.h>
# include <netdb.h>
# define SOCKET int
#endif

#define BUFSIZE 4096
char Buffer[BUFSIZE];
char ScriptFile[BUFSIZE];
#define MAXNAME 255
char Hostname[MAXNAME];
char DisplayName[MAXNAME];
char Scratch[BUFSIZE];
int ConnectTimeout;
int ResponseTimeout = 0;
int PortNumber;
char *UDSocketPath;

int printResult = 0;
int waitForCompletion = 0;
int scriptMode = 0;
int quiet = 0;
int boUMLMode = 0;

#ifdef DEBUG
# define DPRINTF(x)     errprintf x
#else
# define DPRINTF(x)     /* empty */
# endif

char *getenv();

#define SOCK_ERROR          -1
#define SOCK_TIMEOUT        -2
#define SOCK_CONNREFUSED    -3

#define ERR_ST_EXPRESSION       1       /* ST-expression error (only with -print) */
#define ERR_USAGE               2       /* usage / invalid argument */
#define ERR_EXPRESSION_TOO_LONG 5       /* expression too long      */
#define ERR_CONNECT             7       /* socket open/connect failed */
#define ERR_SOCKET_WRITE        8       /* socket write failed      */
#define ERR_RESPONSE            9       /* response read failed / timeout */
#define ERR_SCRIPT             10       /* script open / read failed       */

#ifdef WIN32
# define READ_SOCK(fd,buff,n)  recv(fd, buff, n, 0)
# define WRITE_SOCK(fd,buff,n)  send(fd, buff, n, 0)
#else
# define READ_SOCK(fd,buff,n)  read(fd, buff, n)
# define WRITE_SOCK(fd,buff,n)  write(fd, buff, n)
#endif

#ifdef DEBUG
errprintf(fmt, arg1, arg2, arg3, arg4)
    char *fmt;
{
    fprintf(stderr, fmt, arg1, arg2, arg3, arg4);
}
#endif

/* *********************************************************************** */
/* This function starts stx as a script server */

static int
start_server()
{
    int retVal;

    if ((retVal = system("stscriptserver & >/dev/null 2>/dev/null")) >= 0) {
	return retVal;
    }
    if ((retVal = system("/opt/smalltalk/bin/stscriptserver & >/dev/null 2>/dev/null")) >= 0) {
	return retVal;
    }
    if ((retVal = system("smalltalk & >/dev/null 2>/dev/null")) >= 0) {
	return retVal;
    }
    if ((retVal = system("/opt/smalltalk/bin/smalltalk & >/dev/null 2>/dev/null")) >= 0) {
	return retVal;
    }
    return -1;
}

/* *********************************************************************** */
/* This function creates and connects a socket. */

static jmp_buf timeoutLabel;

static void
timeoutOccurred() {
    DPRINTF(("alarm timer\n"));
    longjmp(timeoutLabel, 1);
}

static SOCKET
call_socket(char *hostname, int portnum, char *udSocketPath, int timeout)
{
    union {
#ifndef WIN32
	struct sockaddr_un un;
#endif
	struct sockaddr_in in;
    } sa;
    int family = AF_INET, sz, a;
    SOCKET s = -1;
    int connectResult;

    if (udSocketPath != NULL) {
#ifdef WIN32
	return (SOCK_ERROR);
#else
	strcpy(sa.un.sun_path, udSocketPath);
	sa.un.sun_family = family = AF_UNIX;
	sz = strlen(udSocketPath) + sizeof ( sa.un.sun_family );
#endif
    } else {
	struct hostent *hp;
	static WSAInitialized = 0;

#ifdef WIN32
	if (! WSAInitialized) {
	    WSADATA wsaData;

	    WSAInitialized = 1;
	    WSAStartup( 0x202, &wsaData );
	}
#endif

	memset(&sa.in, 0, sizeof(sa.in));

	if ((hp = gethostbyname(hostname)) == NULL) {
	    if (strcmp(hostname, "localhost") == 0) {
		sa.in.sin_addr.s_addr = htonl((127<<24) | (0<<16) | (0<<8) | 1);
		sa.in.sin_family = family = AF_INET;
	    } else {
# ifdef WIN32
		DPRINTF(("gethostbyname('%s') - errno:%d lastErr:%d (%x)\n",
			hostname, errno, GetLastError(), GetLastError()));
# endif
# ifdef ECONNREFUSED
		errno = ECONNREFUSED;
# endif
		return(SOCK_ERROR);
	    }
	} else {
	    memcpy((char *)&sa.in.sin_addr, hp->h_addr, hp->h_length);  /* set address */
	    sa.in.sin_family = family = hp->h_addrtype;
	}
	sa.in.sin_port = htons((u_short)portnum);
	sz = sizeof(struct sockaddr_in);
    }

    if ((s = socket(family, SOCK_STREAM, 0)) < 0) {   /* get socket */
#ifdef WIN32
	DPRINTF(("socket creation failed s=%d (0x%x) errno=%d lastError=%d\n", s, s, errno, GetLastError() ));
#else
	DPRINTF(("socket creation failed s=%d (0x%x) errno=%d\n", s, s, errno ));
#endif
	return(SOCK_ERROR);
    }
#ifdef WIN32
    DPRINTF(("socket created s=%d (0x%x) errno=%d lastError=%d\n", s, s, errno, GetLastError() ));
#else
    DPRINTF(("socket created s=%d (0x%x) errno=%d\n", s, s, errno ));
#endif

    if (timeout) {
	void *oldSignalHandler;

	if (setjmp(timeoutLabel)) {
	    DPRINTF(("connect timeout occurred\n"));
	    if (! quiet) {
		fprintf(stderr, "rdoit: Connect timeout\n");
	    }
	    close(s);
	    return(SOCK_TIMEOUT);
	}

	DPRINTF(("connecting with timeout %d\n", timeout));
#ifdef SIGALRM
	oldSignalHandler = signal(SIGALRM, timeoutOccurred);
	alarm(timeout);
#endif
	connectResult = connect(s, (struct sockaddr *)(&sa), sz);
#ifdef SIGALRM
	alarm(0);
	signal(SIGALRM, oldSignalHandler);
#endif
    } else {
	connectResult = connect(s, (struct sockaddr *)(&sa), sz);
    }

    if (connectResult < 0) {
#ifdef WIN32
	DPRINTF(("connect failed errno=%d lastError=%d\n", errno, GetLastError()));
#else
	DPRINTF(("connect failed errno=%d\n", errno));
#endif
#ifdef WIN32
	if (GetLastError() == WSAECONNREFUSED) {
	    close(s);
	    return(SOCK_CONNREFUSED);
	}
#endif
	close(s);
	return(SOCK_ERROR);
    }
    DPRINTF(("connect s=%d (0x%x)\n", s, s ));

    return(s);        /* Sucess */
}


/* *********************************************************************** */
/* The system write() function may not output all "nbyte" characters
   when talking to a socket.  This wrapper keeps trying until
   all the characters are output, or an error is detected. */

static int
write_data(SOCKET s, char *buf, int nbyte)
     /* int s;                /* connected socket */
     /* char *buf;            /* pointer to the buffer */
     /* int nbyte;            /* number of bytes to write */
{
    register int count, sent;

    sent = 0;
    while(nbyte) {
	count = WRITE_SOCK(s,buf,nbyte);
	if (count <= 0) return(count);
	sent += count;
	nbyte -= count;
	buf = &(buf[count]);
    }
    return(sent);
}

static void
writeWithExitOnError(SOCKET sock, char *buf, int n)
{
    if (WRITE_SOCK(sock, buf, n) < 0) {
	if (! quiet) {
	    DPRINTF(("write error sock=%d (0x%x)\n", sock, sock));
	    perror("rdoit: Error in socket write");
	}
	close(sock);
	exit(ERR_SOCKET_WRITE);
    }
}

static int
readInitialAnswerCharacter(SOCKET fd, int timeout)
{
    char buff[1];
    int readResult;

    if (timeout) {
	void *oldSignalHandler;

	if (setjmp(timeoutLabel)) {
	    DPRINTF(("response timeout occurred\n"));
	    if (! quiet) {
		fprintf(stderr, "rdoit: Response timeout\n");
	    }
	    return(SOCK_TIMEOUT);
	}

	DPRINTF(("reading with timeout %d\n", timeout));
#ifdef SIGALRM
	oldSignalHandler = signal(SIGALRM, timeoutOccurred);
	alarm(timeout);
#endif
	readResult = READ_SOCK(fd, buff, 1);
#ifdef SIGALRM
	alarm(0);
	signal(SIGALRM, oldSignalHandler);
#endif
    } else {
	readResult = READ_SOCK(fd, buff, 1);
    }
    if (readResult != 1) {
	return SOCK_ERROR;
    }
    return buff[0];
}

/*
 * read all available data from the socket;
 * and print it on stdout / stderr.
 * The first character received specifies if the result
 * is an error message ('-') or the printString ('+').
 * Returns 0 for ok, -1 for socket error, +1 for ST-error.
 * This is used to print the value of an rdoit command.
 */
static int
read_and_print_upTo_EOF(SOCKET fd, int doPrint, int timeout)
{
    char buff[128];
    int count;
    int outFd;
    int responseType;

    responseType = readInitialAnswerCharacter(fd, timeout);
    if (responseType < 0) {
	return responseType;
    }

    outFd = (responseType == '-') ? 2 : 1;

    while (1) {
	count = READ_SOCK(fd, buff, sizeof(buff));
	if (count <= 0) break;
	if (doPrint || (outFd==2)) write(outFd, buff, count);
    }
    return (outFd == 1 ? 0 : ERR_ST_EXPRESSION);
}

/*
 * read data from the socket and print it on stdout / stderr.
 * The first character received within each block specifies if the result
 * is to be sent to stderr ('2') or to stdout ('1').
 * This is used to print the stdout/stderr of a script.
 * TODO: also read stdin, and send it to the pipe (also fix RDOIT.st for this).
 */
static int
read_and_printEach(SOCKET fd)
{
    char buff[256];
    char responseType, c;
    int n;
    int outFd;

    while (1) {
	responseType = readInitialAnswerCharacter(fd, 0);
	if (responseType < 0) {
	    return -1;
	}

	n = 0;
	while (1) {
	    if (READ_SOCK(fd, &c, 1) != 1) {
		return -1;
	    }
	    if ((c < '0') || (c > '9')) break;
	    n = n * 10 + (c - '0');
	}
	if (responseType == '#') {
	    return n;   /* exitCode */
	}

	outFd = responseType - '0';
	while (n) {
	    int nr;

	    nr = n > sizeof(buff) ? sizeof(buff) : n;
	    nr = READ_SOCK(fd, buff, nr);
	    if (nr < 0) return -1;

	    write(outFd, buff, nr);
	    n -= nr;
	}
    }
}

void
send_file(char *fileName, SOCKET sock)
{
    int inFd;
    char c;
    long pos1, pos2, n;
    char buffer[512];
    int nr, nremain;

    inFd = open(fileName, 0);
    if (inFd < 0) {
#ifndef WIN32
	DPRINTF(("rdoit: current dir: '%s'\n", realpath(".", buffer)));
#endif
	DPRINTF(("rdoit: cannot open '%s'.\n", fileName));

	perror("rdoit: Cannot open script.");
	close(sock);
	exit(ERR_SCRIPT);
    }
    /* skip over 1st line */
    while ((read(inFd, &c, 1) == 1) && (c != '\n')) ;;

    /* compute size (remaining bytes) */
    pos1 = lseek(inFd, 0L, SEEK_CUR);
    pos2 = lseek(inFd, 0L, SEEK_END);
    n = pos2 - pos1;
    lseek(inFd, pos1, SEEK_SET);

    DPRINTF(("pos1: %d\n", pos1));
    DPRINTF(("pos2: %d\n", pos2));
    DPRINTF(("n: %d\n", n));

    sprintf(buffer, "#%d\r\n", n);
    if (write_data(sock, buffer, strlen(buffer)) < 0) {
	perror("rdoit: Cannot send script size");
	exit(11);
    }

    DPRINTF(("sent: %s\n", buffer));

    nremain = n;
    while (nremain > 0) {
	nr = nremain;
	if (nr > sizeof(buffer)) nr = sizeof(buffer);
	nr = read(inFd, buffer, nr);

	DPRINTF(("nr: %d\n", nr));

	if (nr < 0) {
	    perror("rdoit: Script read error.");
	    close(sock);
	    exit(ERR_SCRIPT);
	}
	write_data(sock, buffer, nr);
#ifdef DEBUG
	printf("sent: "); fflush(stdout);
	write(1, buffer, nr);
#endif
	nremain -= nr;
    }
    close(inFd);
}

void
usage(char *argv[])
{
    char *cmd = argv[0];
#ifdef WIN32
    char *arg = "st-expression";
    cmd = "rdoit";
#else
    char *arg = "\"st-expression\"";
#endif

    if (scriptMode) {
	arg = "st-file";
    }
    fprintf(stderr,"Usage: %s [--host hostname] [--port portnumber] [moreOptions] %s\n", cmd, arg);
    fprintf(stderr,"    --version ................... print rdoit version info\n");
    fprintf(stderr,"    --quiet ..................... don't print error messages\n");
    fprintf(stderr,"    --print ..................... print smalltalk expression value\n");
    fprintf(stderr,"    --wait ...................... wait for evaluation to complete\n");
    fprintf(stderr,"    --display <displayName> ..... redirect views to another display (same as DISPLAY variable)\n");
    fprintf(stderr,"    --connectTimeout <seconds> .. connection timeout\n");
    fprintf(stderr,"    --responseTimeout <seconds> . response timeout\n");
    fprintf(stderr,"    --host <hostname> ........... send request to rdoit server on host \n");
    fprintf(stderr,"    --port <portNr> ............. send request to rdoit server listening on port \n");
    fprintf(stderr,"  Error Codes:\n");
    fprintf(stderr,"    %2d - ST-expression error\n", ERR_ST_EXPRESSION);
    fprintf(stderr,"    %2d - usage / invalid argument\n", ERR_USAGE);
    fprintf(stderr,"    %2d - expression too long\n", ERR_EXPRESSION_TOO_LONG);
    fprintf(stderr,"    %2d - socket open / connect / timeout\n", ERR_CONNECT);
    fprintf(stderr,"    %2d - request send failed\n", ERR_SOCKET_WRITE);
    fprintf(stderr,"    %2d - response failed / timeout\n", ERR_RESPONSE);
    fprintf(stderr,"    %2d - script read failed\n", ERR_SCRIPT);

    if (! scriptMode) {
	fprintf(stderr,"  Examples:\n");
#ifdef WIN32
	fprintf(stderr,"    %s FileBrowser open\n", cmd);
	fprintf(stderr,"    %s Transcript topView raise\n", cmd);
	fprintf(stderr,"    %s Transcript showCR: 'it worked.'\n", cmd);
	fprintf(stderr,"    %s 50 factorial printCR\n", cmd);
	fprintf(stderr,"    %s --print 50 factorial\n", cmd);
	fprintf(stderr,"    %s --host <remoteHost> Transcript showCR: 'it worked.'\n", cmd);
	fprintf(stderr,"    %s --host localhost --port 8004 Transcript showCR: 'it worked.'\n", cmd);
#else
	fprintf(stderr,"    %s \"FileBrowser open\"\n", cmd);
	fprintf(stderr,"    %s \"Transcript topView raise\"\n", cmd);
	fprintf(stderr,"    %s \"Transcript showCR: 'it worked.'\"\n", cmd);
	fprintf(stderr,"    %s \"50 factorial printCR\"\n", cmd);
	fprintf(stderr,"    %s --print \"50 factorial\"\n", cmd);
	fprintf(stderr,"    %s --host <remoteHost> \"Transcript showCR: 'it worked.'\"\n", cmd);
	fprintf(stderr,"    %s --host localhost --port 8004 \"Transcript showCR: 'it worked.'\"\n", cmd);
#endif
    }
    exit(ERR_USAGE);
}

/* *********************************************************************** */
/* Quick and dirty code to take apart the command line. */

static void
parse_args(int argc, char *argv[])
{
    int length, portArg, aa, i;
    char *hostArgString = NULL, *portArgString = NULL,
	 *connectTimeoutArgString = NULL, *responseTimeoutArgString = NULL;
    char *displayNameArg = NULL;
    char *logFilename;

    printResult = 0;
    waitForCompletion = 0;
    scriptMode = 0;
    quiet = 0;
    boUMLMode = 0;

    /*
     * check for logFile arg first, to get the rest been sent there...
     */
    for (i = 1; argv[i]; i++) {
	if (strncmp(argv[i], "--logfile", 9) == 0) {
	    logFilename = argv[i+1];
	    freopen(logFilename, "w+", stderr);
	    break;
	}
    }

    DPRINTF(("cmd: <%s>\n", argv[0]));
    if (argv[0] && (strcmp(argv[0], "stscript") == 0)) {
	DPRINTF(("scriptMode\n"));
	scriptMode = 1;
    }

    while (argv[1]) {
	DPRINTF(("arg: <%s>\n", argv[1]));
	if ((strcmp(argv[1], "--version") == 0)) {
	    fprintf(stderr, "rdoit %s\n", "$Revision: 1.46 $");
	    exit(0);
	}
	if ((strncmp(argv[1], "-p", 2) == 0) || (strncmp(argv[1], "--print", 7) == 0)) {
	    printResult = 1;
	    argv++;
	    argc--;
	} else if ((strncmp(argv[1], "-q", 2) == 0) || (strncmp(argv[1], "--quiet", 7) == 0)) {
	    quiet = 1;
	    argv++;
	    argc--;
	} else if ((strncmp(argv[1], "-w", 2) == 0) || (strncmp(argv[1], "--wait", 6) == 0)) {
	    waitForCompletion = 1;
	    argv++;
	    argc--;
	} else if ((strncmp(argv[1], "-h", 2) == 0) || (strncmp(argv[1], "--host", 6) == 0)) {
	    hostArgString = argv[2];
	    argv += 2;
	    argc -= 2;
	} else if ((strncmp(argv[1], "-n", 2) == 0) || (strncmp(argv[1], "--port", 6) == 0)) {
	    portArgString = argv[2];
	    argv += 2;
	    argc -= 2;
	} else if ((strncmp(argv[1], "-tr", 3) == 0) || (strncmp(argv[1], "--responseTimeout", 17) == 0)) {
	    responseTimeoutArgString = argv[2];
	    argv += 2;
	    argc -= 2;
	} else if ((strncmp(argv[1], "-tc", 3) == 0) || (strncmp(argv[1], "--connectTimeout", 16) == 0)) {
	    connectTimeoutArgString = argv[2];
	    argv += 2;
	    argc -= 2;
	} else if ((strncmp(argv[1], "-display", 8) == 0)  || (strncmp(argv[1], "--display", 9) == 0)) {
	    displayNameArg = argv[2];
	    argv += 2;
	    argc -= 2;
	} else if (strncmp(argv[1], "--help", 6) == 0) {
	    usage(argv);
	} else if (strncmp(argv[1], "--logfile", 9) == 0) {
	    argv += 2;
	    argc -= 2;
	} else {
	    break;
	}
    }


#ifndef WIN32
# if 0  /* no longer allowed: host/port args without leading -p/-n */
    if (! scriptMode) {
	portArgString = argv[2];
	hostArgString = argv[1];

	if (argc < 4) {
	    switch (argc) {
		case 3:
		    portArgString = DEFAULT_PORT;
		    break;

		case 2:
		    portArgString = DEFAULT_PORT;
		    hostArgString = DEFAULT_HOST;
		    break;

		default:
		    fprintf(stderr, "rdoit: Missing Arg(s)\n");
		    usage(argv);
	    }
	}
	if (argc > 4) {
	    fprintf(stderr, "rdoit: Usage\n");
	    fprintf(stderr, "Too many arguments. (You must QUOTE message.)\n");
	    exit(ERR_USAGE);
	}
    }
# endif
#endif

    if (hostArgString == NULL) {
	hostArgString = DEFAULT_HOST;
    }
    if (portArgString == NULL) {
	portArgString = getenv("STX_RDOIT_PORT");
	if (portArgString == NULL) {
	    portArgString = DEFAULT_PORT;
	}
    }
    if (connectTimeoutArgString == NULL) {
	connectTimeoutArgString = getenv("STX_RDOIT_CONNECT_TIMEOUT");
	if (connectTimeoutArgString == NULL) {
	    connectTimeoutArgString = DEFAULT_CONNECT_TIMEOUT;
	}
    }
    if (responseTimeoutArgString == NULL) {
	responseTimeoutArgString = getenv("STX_RDOIT_RESPONSE_TIMEOUT");
	if (responseTimeoutArgString == NULL) {
	    responseTimeoutArgString = DEFAULT_RESPONSE_TIMEOUT;
	}
    }

    DPRINTF(("host = <%s>\n", hostArgString ? hostArgString : "NULL"));
    DPRINTF(("port = <%s>\n", portArgString ? portArgString : "NULL"));
    DPRINTF(("connectTimeout = <%s>\n", connectTimeoutArgString ? connectTimeoutArgString : "NULL"));
    DPRINTF(("responseTimeout = <%s>\n", responseTimeoutArgString ? responseTimeoutArgString : "NULL"));
    DPRINTF(("display = <%s>\n", displayNameArg ? displayNameArg : "NULL"));

#ifdef xxWIN32
    if (strcmp(hostArgString, "localhost") == 0) {
	static char buffer[256];

	if (gethostname(buffer, sizeof(buffer)) == 0) {
	    hostArgString = buffer;
	} else {
	    int buffSize = sizeof(buffer);

	    if (GetComputerName(buffer, &buffSize) == TRUE) {
		hostArgString = buffer;
	    }
	}
    }
#endif

    if (hostArgString) strncpy(Hostname, hostArgString, MAXNAME);
    if (displayNameArg) {
	strncpy(DisplayName, displayNameArg, MAXNAME);
    } else {
	DisplayName[0] = 0;
    }

#ifndef WIN32
    if (gethostbyname(hostArgString) == NULL) {
	if (! quiet) {
	    fprintf(stderr, "rdoit: Bad Host:'%s' (gethostbyname failed))\n", hostArgString);
	    usage(argv);
	}
	exit(ERR_USAGE);
    }
#endif

    if ((connectTimeoutArgString[0] >= '0') && (connectTimeoutArgString[0] <= '9')) {
	if (sscanf(connectTimeoutArgString,"%d",&ConnectTimeout) != 1) {
	    if (! quiet) {
		fprintf(stderr, "rdoit: Invalid ConnectTimeout argument\n");
		usage(argv);
	    }
	    exit(ERR_USAGE);
	}
    }
    if ((responseTimeoutArgString[0] >= '0') && (responseTimeoutArgString[0] <= '9')) {
	if (sscanf(responseTimeoutArgString,"%d",&ResponseTimeout) != 1) {
	    if (! quiet) {
		fprintf(stderr, "rdoit: Invalid ResponseTimeout argument\n");
		usage(argv);
	    }
	    exit(ERR_USAGE);
	}
    }

    while (portArgString[0] && (portArgString[0] == ' ')) portArgString++;
    if ((portArgString[0] >= '0') && (portArgString[0] <= '9')) {
	if (sscanf(portArgString,"%d",&PortNumber) != 1) {
	    if (! quiet) {
		fprintf(stderr, "rdoit: Invalid Port argument\n");
		usage(argv);
	    }
	    exit(ERR_USAGE);
	}
	UDSocketPath = NULL;
    } else {
#ifdef WIN32
	if (! quiet) {
	    fprintf(stderr, "rdoit: Unix domain sockets only available on unix machines.\n");
	}
	exit(ERR_USAGE);
#endif
	PortNumber = -1;
	UDSocketPath = portArgString;
    }

#if 1 /* def WIN32 */
    /*
     * concatenate remaining args as command
     */
    aa = 1;
    length = 0;
    while (argv[aa]) {
	DPRINTF(("word = <%s>\n", argv[aa]));
	if ((length + 1 + strlen(argv[aa])) > (BUFSIZE - 5)) {
	    if (! quiet) {
		fprintf(stderr, "rdoit: Expression string is too long.\n");
	    }
	    exit(ERR_EXPRESSION_TOO_LONG);
	}

	if (length != 0) {
	    Buffer[length++] = ' ';
	    Buffer[length] = '\0';
	}
	strcpy(Buffer+length, argv[aa]);
	length += strlen(argv[aa]);
	aa++;
    }
#else /* unix */
    /* single cmd-arg */
    if ((length = strlen(argv[argc - 1])) > (BUFSIZE - 5)) {
	if (! quiet) {
	    fprintf(stderr, "rdoit: Expression string is too long.\n");
	}
	exit(ERR_EXPRESSION_TOO_LONG);
    }
    strcpy(Buffer, argv[argc - 1]);
#endif

    if (argc <= 1) {
	if (! quiet) {
	    fprintf(stderr, "rdoit: Script filename missing.\n");
	    usage(argv);
	}
	exit(ERR_USAGE);
    }

    strcpy(ScriptFile, argv[argc - 1]);

    Buffer[length] = '\000';
    DPRINTF(("cmd = <%s>\n", Buffer));

    /* The ST process is going to read through: CR */
    Buffer[length++] = '\015';    /* CR */
    Buffer[length++] = '\012';    /* LF */
    Buffer[length++] = '\n';      /* NL */
    Buffer[length++] = '\000';    /* NULL */
}

int
main(int argc, char *argv[])
{
    SOCKET sock;
    int exitCode;

    parse_args(argc, argv);

    sock = call_socket(Hostname, PortNumber, UDSocketPath, ConnectTimeout);
    if (sock < 0) {
	if (scriptMode) {
	    /* try to start the script server ... */
	    if (start_server() < 0) {
		fprintf(stderr, "Cannot start stx script server (stscriptserver or smalltalk)\n");
	    };
	    sock = call_socket(Hostname, PortNumber, UDSocketPath, ConnectTimeout);
	}
    }
    if (sock < 0) {
	if (! quiet) {
	    if (scriptMode) {
		fprintf(stderr, "Error connecting to stx script server (start stx and its rdoitServer)\n");
	    } else {
		if (sock == SOCK_TIMEOUT) {
		    fprintf(stderr, "rdoit: Error opening socket to stx (connect timeout)\n");
		} else {
		    fprintf(stderr, "rdoit: Error opening socket to stx\n");
		}
	    }
	}
	exit(ERR_CONNECT);
    }
    DPRINTF(("printResult      : %d\n", printResult));
    DPRINTF(("waitForCompletion: %d\n", waitForCompletion));
    DPRINTF(("scriptMode       : %d\n", scriptMode));

    if (strlen(DisplayName) > 0) {
	writeWithExitOnError(sock, "(", 1);
	writeWithExitOnError(sock, DisplayName, strlen(DisplayName));
	writeWithExitOnError(sock, ")", 1);
    }

    if (scriptMode) {
	/* send the files contents to st/x */
	send_file(ScriptFile, sock);
	printResult = 1;
	exitCode = read_and_printEach(sock);
    } else {
	if (printResult || waitForCompletion) {
	    DPRINTF(("sending a '?'...\n"));
	    writeWithExitOnError(sock, "?", 1);
	} else {
	    DPRINTF(("sending a ' '...\n"));
	    writeWithExitOnError(sock, " ", 1);
	}
	DPRINTF(("sending command...\n"));
	writeWithExitOnError(sock, Buffer, strlen(Buffer));

	if (printResult || waitForCompletion) {
	    exitCode = read_and_print_upTo_EOF(sock, printResult, ResponseTimeout);
	    if (exitCode < 0) {
	      if (! quiet) {
		  fprintf(stderr, "rdoit: Cannot read result\n");
	      }
	      exit(ERR_RESPONSE);
	    }
	} else {
	    exitCode = 0;
	}
    }

    close(sock);
    exit(exitCode);
    return 0;
}
