"{ Package: 'stx:goodies/refactoryBrowser/helpers' }"

Object subclass:#RBMethod
	instanceVariableNames:'class compiledMethod source selector info'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Model'
!

RBMethod comment:'RBMethod represents a method in an RBAbstractClass.

Instance Variables:
	class	<RBAbstractClass>	where the method is defined
	compiledMethod	<CompiledCode | nil>	the CompiledMethod object (if we have been compiled)
	selector	<Symbol>	our name
	source	<String>	our source'
!


!RBMethod class methodsFor:'instance creation'!

for: aRBClass fromMethod: aCompiledMethod andSelector: aSymbol 
	^(self new)
		modelClass: aRBClass;
		method: aCompiledMethod;
		selector: aSymbol;
		yourself
!

for: aRBClass source: aString selector: aSelector 
	^(self new)
		modelClass: aRBClass;
		selector: aSelector;
		source: aString;
		yourself
! !

!RBMethod methodsFor:'accessing'!

allLiterals
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m allLiterals.
    ].
    ^#()

    "Created: / 17-02-2012 / 00:10:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

annotationsAt:aSymbol do:aBlock
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m annotationsAt:aSymbol do:aBlock
    ].

    "Created: / 17-02-2012 / 00:03:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

category
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m category.
    ].
    ^'as yet unclassified'

    "Created: / 17-02-2012 / 00:06:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

info
    "Returns an 'info object' that can answer various
     questions like 'what selectors method sends',
     names if temporaries etc.

     Actally, 'info object' is instance of Parser"

    info isNil ifTrue:[
        info := Parser parseMethod: source.
    ].
    ^info

    "Created: / 16-02-2012 / 16:33:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

literals
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m literals.
    ].
    ^#()

    "Created: / 17-02-2012 / 00:00:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

method
    compiledMethod isNil ifTrue:[
        compiledMethod := class realClass compiledMethodAt: self selector.
    ].
    ^compiledMethod

    "Modified: / 17-02-2012 / 00:07:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

method: aCompiledMethod 
	compiledMethod := aCompiledMethod
!

modelClass
	^class
!

modelClass: aRBClass
	class := aRBClass
!

package
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m package.
    ].
    ^PackageId noProjectID

    "Created: / 17-02-2012 / 00:41:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseTree
    ^RBParser 
        parseMethod: self source 
        onError: [:str :pos | 
            "/Transcript showCR:str. Transcript showCR:pos.
            ^ nil
        ]

    "Modified: / 17-02-2012 / 11:28:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

selector
    selector isNil ifTrue:[
        selector := (Parser parseMethodSpecification: source) selector.
    ].
    ^selector

    "Modified: / 16-02-2012 / 16:51:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

selector: aSymbol 
	selector := aSymbol
!

source
        ((Smalltalk respondsTo:#isSmalltalkX) and:[Smalltalk isSmalltalkX]) ifTrue:[
            ^source isNil 
                    ifTrue: [source := compiledMethod source]
                    ifFalse: [source]
        ].

        ^source isNil 
                ifTrue: [source := compiledMethod getSource]
                ifFalse: [source]
!

source: aString 
	source := aString
! !

!RBMethod methodsFor:'printing'!

printOn: aStream 
	class printOn: aStream.
	aStream
		nextPutAll: '>>';
		nextPutAll: self selector
! !

!RBMethod methodsFor:'private'!

literal: anObject containsReferenceTo: aSymbol 
	anObject = aSymbol ifTrue: [^true].
	anObject class = Array ifFalse: [^false].
	anObject 
		do: [:each | (self literal: each containsReferenceTo: aSymbol) ifTrue: [^true]].
	^false
! !

!RBMethod methodsFor:'queries'!

refersToLiteral:anAssociation
    | m |

    m := self method.
    m notNil ifTrue:[
        ^m refersToLiteral:anAssociation

    ].
    ^false "Actually - don't know"

    "Created: / 16-02-2012 / 23:57:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sendsSelector:aSymbol
    ^self info messagesSent includes: aSymbol

    "Created: / 16-02-2012 / 16:33:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBMethod methodsFor:'testing'!

refersToClassNamed: aSymbol 
	| searcher |
	searcher := ParseTreeSearcher new.
	searcher matches: aSymbol asString do: [:node :answer | true].
	^(searcher executeTree: self parseTree initialAnswer: false) 
		or: [self refersToSymbol: aSymbol]
!

refersToSymbol: aSymbol 
        | searcher p|
        searcher := ParseTreeSearcher new.
        p := (RBParser isSmalltalkX ifTrue:[aSymbol storeString] ifFalse:[aSymbol printString]).

        searcher
                matches: p do: [:node :answer | true];
                matches: '`#literal'
                        do: [:node :answer | answer or: [self literal: node value containsReferenceTo: aSymbol]].
        (RBScanner isSelector: aSymbol) 
                ifTrue: 
                        [searcher 
                                matches: '`@object ' , (ParseTreeSearcher buildSelectorString: aSymbol)
                                do: [:node :answer | true]].
        ^searcher executeTree: self parseTree initialAnswer: false
!

refersToVariable: aString 
	| searcher tree |
	tree := self parseTree.
	((tree defines: aString) or: [tree body defines: aString]) 
		ifTrue: [^false].
	searcher := ParseTreeSearcher new.
	searcher
		matches: aString do: [:node :answer | true];
		matches: '[:`@vars | | `@temps | `@.Stmts]'
			do: 
				[:node :answer | 
				answer or: 
						[((node defines: aString) or: [node body defines: aString]) not 
							and: [searcher executeTree: node body initialAnswer: false]]].
	^searcher executeTree: self parseTree initialAnswer: false
! !

!RBMethod class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/helpers/RBMethod.st,v 1.12 2012/02/17 14:34:26 vrany Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/helpers/RBMethod.st,v 1.12 2012/02/17 14:34:26 vrany Exp $'
! !
