"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBBlockLintRule subclass:#RBBadMessageRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!


!RBBadMessageRule methodsFor:'accessing'!

group
	^ 'Miscellaneous'
!

name
	^ 'Sends "questionable" message'
!

rationale
    ^ 'Check methods that send messages that perform low level things. 
You might want to limit the number of such messages in your application. 
For example, using <code>become:</code> throughout your application might not be the best thing. 
Also, messages such as <code>isKindOf:</code> can signify a lack of polymorphism.
Finally, <code>halt</code> messages might be due to leftover debug code.
You can change which methods are "questionable" by editing the 
<A INFO="Click to browse this method"
   ACTION="doit: UserPreferences current systemBrowserClass
                    openInClass:%1 selector:#badSelectors">
<CODE>%1>>badSelectors</CODE>
</A> method.'
        bindWith:self class name

    "Modified: / 07-09-2011 / 21:49:47 / cg"
! !

!RBBadMessageRule methodsFor:'private'!

badSelectors
        ^ #( #become: #primBecome: #becomeNil #becomeSameAs:
             #changeClassToThatOf: #changeClassTo:
             #replaceReferencesTo:with:
             #respondsTo: 
             #isMemberOf: #isKindOf:
             #performMethod: 
             #performMethod:arguments: 
             #performMethod:with: 
             #performMethod:with:with: 
             #performMethod:with:with:with: 
             #allOwners #allOwnersWeakly: 
             #firstOwner 
             #ownerAfter: 
             #instVarAt: #instVarAt:put: 
             #allInstances #allInstancesDo: #nextInstance #nextObject 
             #halt #halt:)

    "Modified: / 02-03-2012 / 03:02:11 / cg"
! !

!RBBadMessageRule methodsFor:'running'!

checkClass:aContext 
    |selectors|

    selectors := self badSelectors inject:Set new
                into:[:set :each | 
                    set
                        addAll:(aContext selectedClass whichSelectorsReferTo:each);
                        yourself
                ].
    selectors do:[:each | 
        self 
            addClass:aContext selectedClass
            selector:each
            to:result
    ].
    selectors isEmpty ifFalse:[
        result searchStrings:self badSelectors
    ]
! !

!RBBadMessageRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBBadMessageRule.st,v 1.8 2012/03/02 02:05:24 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBBadMessageRule.st,v 1.8 2012/03/02 02:05:24 cg Exp $'
!

version_SVN
    ^ 'Id: RBBadMessageRule.st 24 2010-08-25 12:47:16Z vranyj1 '
! !
