"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBLintRule subclass:#RBBasicLintRule
	instanceVariableNames:'result'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!


!RBBasicLintRule methodsFor:'accessing'!

filteredResult
	"Be very careful when filtering results not to introduce new items and not to lose the dedicated browser environments. Try the following steps in order:
	
	- If this is a selector environment use the set-operations of the refactoring browser.
	- If this is a class environment, remove the classes that have a filter annotation in any of its methods.
	- If this is a variable environment, remove the classes and all its variables that have a filter annotation in any of its methods.
	- Otherwise return the unfiltered environment."

	| filter |
	result isEmpty
		ifTrue: [ ^ result ].
	filter := PragmaEnvironment
		onEnvironment: BrowserEnvironment new
		keywords: #( lint: lint:rationale: lint:rationale:author: lint:author: ignoreLintRule: ignoreLintRule:rationale: ignoreLintRule:rationale:author: ignoreLintRule:author: ).
	filter condition: [ :pragma |
		pragma arguments first = self name
			or: [ pragma arguments first = self group
			or: [ pragma arguments first = self class name ] ] ].
	result isSelectorEnvironment
		ifTrue: [ ^ result & filter not ].
	result isClassEnvironment
		ifTrue: [ filter classesDo: [ :class | result removeClass: class ] ]
		ifFalse: [
			(result isKindOf: VariableEnvironment) ifTrue: [
				filter classesDo: [ :class |
					class classVarNames do: [ :var | result removeClass: class classVariable: var ].
					class instVarNames do: [ :var | result removeClass: class instanceVariable: var ] ] ] ].
	^ result
!

problemCount
	^ self result problemCount
!

result
	^ result
!

resultClass
	self subclassResponsibility
! !

!RBBasicLintRule methodsFor:'helpers'!

appendVariableNamesFromResultToRationale:rationaleMessage
    "extract instance- and class-variables from the result,
     and append to the given rationaleMessage.
     Common code used by rationale of readNotWritten and unref'd vars rules"

    |msg instVars classVars classInstVars collector appendThem|

    result notNil ifTrue:[
        collector := 
            [:dict |
                |c|
                dict isEmpty ifTrue:[
                    ''
                ] ifFalse:[
                    dict keys size == 1 ifTrue:[
                        dict values first asOrderedCollection sort.
                    ] ifFalse:[
                        dict keysAndValuesCollect:[:cls :names |
                            (names asOrderedCollection sort
                                collect:[:nm | nm,' (',(cls isBehavior ifTrue:[cls name] ifFalse:[cls]),')'])
                                    asStringCollection
                                        asString
                        ].
                    ]
                ]
            ].
        instVars := collector value:result instanceVariables.
        classVars := collector value:result classVariables.
        "/ classInstVars := collector value:result classInstanceVariables.
    ].

    msg := rationaleMessage.

    appendThem := [:vars :what |
        vars notEmptyOrNil ifTrue:[
            msg := msg , '\<P>',what,' variable(s): '.
            msg := msg , '<ul>'.
            vars do:[:each | msg := msg , '<li><code>',each,'</code>'].
            msg := msg , '\</ul>'.
        ].
    ].

    appendThem value:instVars value:'Instance'.
    appendThem value:classVars value:'Class'.
    appendThem value:classInstVars value:'Class inst'.

    ^ msg withCRs

    "Created: / 07-03-2012 / 18:26:09 / cg"
! !

!RBBasicLintRule methodsFor:'initialization'!

initialize
	super initialize.
	self resetResult
	
! !

!RBBasicLintRule methodsFor:'private'!

failedClassesInto:aStream 
    result isNil ifFalse:[
        result classesInto:aStream.
    ].

    "Created: / 10-01-2009 / 15:14:23 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 13-01-2009 / 16:53:28 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

failedMethodsInto:aStream 
    result isNil ifFalse:[
        result methodsInto:aStream.
    ].

    "Created: / 08-01-2009 / 09:46:48 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 12-01-2009 / 23:19:05 / Jiri Bazant <bazanj2@fel.cvut.cz>"
! !

!RBBasicLintRule methodsFor:'running'!

resetResult
	result := self resultClass new.
	result label: self name
! !

!RBBasicLintRule methodsFor:'testing'!

isEmpty
	^ self result isEmpty
! !

!RBBasicLintRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBBasicLintRule.st,v 1.2 2012/03/07 17:29:17 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBBasicLintRule.st,v 1.2 2012/03/07 17:29:17 cg Exp $'
!

version_SVN
    ^ 'Id: RBBasicLintRule.st 20 2010-02-05 14:10:34Z vranyj1 '
! !
