"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBLintRule subclass:#RBCompositeLintRule
	instanceVariableNames:'rules name'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!


!RBCompositeLintRule class methodsFor:'instance creation'!

rules: aCollection
	^ self new
		rules: aCollection;
		yourself
!

rules: aCollection name: aString
	^ self new
		rules: aCollection;
		name: aString;
		yourself
!

rulesFor: aRuleClass
	| rules |
	rules := SortedCollection
		sortBlock: [ :a :b | a name <= b name ].
	aRuleClass withAllSubclassesDo: [ :each |
		each isVisible
			ifTrue: [ rules add: each new ] ].
	^ rules asArray
!

rulesGroupedFor: aRuleClass
        | groups rules |
        groups := Dictionary new.
        (self rulesFor: aRuleClass) do: [ :each |
                (groups at: each group ifAbsentPut: [ OrderedCollection new ])
                        addLast: each ].
        rules := SortedCollection sortBlock: [ :a :b | a name <= b name ].
        groups keysAndValuesDo: [ :group :elements |
                rules add: (RBCompositeLintRule 
                        rules: elements asArray
                        name: group) ].
        ^ rules asArray
! !

!RBCompositeLintRule class methodsFor:'accessing'!

allRules
	^ self rules: (Array with: self lintChecks with: self transformations) name: 'All checks'
!

lintChecks

    ^ self 
            rules: (self rulesGroupedFor: RBBasicLintRule) 
            name: 'Lint checks'

    "
        RBCompositeLintRule allRules
    "

    "Modified: / 25-08-2010 / 14:35:02 / Jan Vrany <enter your email here>"
!

transformations
	^ self rules: (self rulesGroupedFor: RBTransformationRule) name: 'Transformations'
! !

!RBCompositeLintRule methodsFor:'accessing'!

changes
	^ rules gather: [ :each  | each changes ]
!

failedRules
    ^ (rules ? #()) inject:OrderedCollection new
        into:[:oc :each | 
            oc
                addAll:each failedRules;
                yourself
        ]

    "Modified: / 23-09-2011 / 15:25:16 / cg"
!

name
	^ name
!

name: aString
	name := aString
!

problemCount
	^ rules inject: 0 into: [ :count :each | count + each problemCount ]
!

ruleCount
        ^ rules inject: 0 into: [ :count :each | count + each ruleCount ]

    "Modified: / 24-07-2010 / 15:38:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rules
	^ rules
!

rules: aCollection
	rules := aCollection
!

scope

    ^nil
! !

!RBCompositeLintRule methodsFor:'displaying'!

displayString

    ^Text streamContents:
        [:s| | cnt |
        s nextPutAll: self name.
        s space.
        cnt := self ruleCount.
        cnt == 0 ifTrue:[s nextPutAll:(('[ no rules ]') asText colorizeAllWith: Color gray)].
        cnt == 1 ifTrue:[s nextPutAll:(('[ 1 rule ]') asText colorizeAllWith: Color gray)].
        cnt > 1 ifTrue:[s nextPutAll:(('[ ',cnt printString,' rules ]') asText colorizeAllWith: Color gray)].
        ]

    "Created: / 24-07-2010 / 15:40:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBCompositeLintRule methodsFor:'initialization'!

resetResult
	rules do: [ :each | each resetResult ]
! !

!RBCompositeLintRule methodsFor:'private'!

failedClassesInto:aStream 
    self failedRules do:[:failedRule | 
        failedRule failedClassesInto:aStream
    ]

    "Created: / 10-01-2009 / 15:14:42 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 22-07-2009 / 14:41:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

failedMethodsInto:aStream 
    self failedRules do:[:failedRule | 
        failedRule failedMethodsInto:aStream
    ]

    "Created: / 08-01-2009 / 13:01:03 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 08-01-2009 / 23:49:30 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

flattenOn: stream

    (self rules ? #()) do:[:rule|rule flattenOn: stream]

    "Created: / 21-07-2009 / 23:12:05 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 23-09-2011 / 15:25:04 / cg"
! !

!RBCompositeLintRule methodsFor:'running'!

checkClass: aContext 
	rules do: [ :each | each checkClass: aContext ]
!

checkMethod: aContext 
	rules do: [ :each | each checkMethod: aContext ]
! !

!RBCompositeLintRule methodsFor:'testing'!

hasConflicts
	^ rules anySatisfy: [ :each | each hasConflicts ]
!

isComposite
	^ true
!

isEmpty
    ^ rules isEmptyOrNil or:[ rules allSatisfy: [ :each | each isEmpty ]]

    "Modified: / 01-03-2012 / 08:34:41 / cg"
!

isEmptyInTree
    "added due to the semantic confusion of isEmpty 
     (means both: has subitems AND has subitems with result !!)"

    ^ rules isEmptyOrNil or:[ rules allSatisfy: [ :each | each isEmptyInTree ]]

    "Created: / 01-03-2012 / 19:05:53 / cg"
! !

!RBCompositeLintRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBCompositeLintRule.st,v 1.6 2012/03/01 18:44:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBCompositeLintRule.st,v 1.6 2012/03/01 18:44:06 cg Exp $'
!

version_SVN
    ^ 'Id: RBCompositeLintRule.st 24 2010-08-25 12:47:16Z vranyj1 '
! !
