"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBConsistencyCheckRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBConsistencyCheckRule methodsFor:'accessing'!

group
	^ 'Intention revealing'
!

name
        ^ 'Uses "size = 0", "size > 0", "= nil", or "at: 1" instead of "isEmpty", "notEmpty", "isNil", or "first"'

    "Modified: / 06-03-2012 / 18:42:58 / cg"
!

rationale
        ^ 'Checks for equality tests instead of a message sends. 
Since the code "<code>aCollection size = 0</code>" works for all objects, 
it may be more difficult for someone reading such code to determine that "aCollection" is a collection. 
Whereas, if you say "<code>aCollection isEmpty</code>" then aCollection must be a collection since isEmpty is only defined for collections.
<P>
However, be careful: sometimes a lazy initialized variable with nil value is treated like an empty collection.
In this case, better use "<code>aCollection isEmptyOrNil</code>" to make that clear.'

    "Modified: / 06-03-2012 / 18:43:24 / cg"
! !

!RBConsistencyCheckRule methodsFor:'initialization'!

initialize
	super initialize.
	self matcher 
		matchesAnyOf: #(
			'`@object size == 0'
			'`@object size = 0'
			'`@object size > 0'
			'`@object size >= 1'
			'`@object == nil'
			'`@object = nil'
			'`@collection at: 1'
			'`@collection at: `@collection size' )
		do: [ :node :answer | node ]
! !

!RBConsistencyCheckRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBConsistencyCheckRule.st,v 1.4 2012/03/06 17:56:49 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBConsistencyCheckRule.st,v 1.4 2012/03/06 17:56:49 cg Exp $'
!

version_SVN
    ^ 'Id: RBConsistencyCheckRule.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
