"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBEqualsTrueRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBEqualsTrueRule methodsFor:'accessing'!

group
	^ 'Unnecessary code'
!

name
	^ 'Unnecessary "= true"'
!

rationale
        ^ 'Check for a =, ==, ~=, or ~~ message being sent to true/false or with true/false as the argument. 
Many times these can be eliminated since their receivers are already booleans. 
For example, "<code>anObject isFoo == false</code>" could be replaced with 
"<code>anObject isFoo not</code>" if isFoo always returns a boolean. 
<P>
Sometimes variables might refer to true, false, and something else, but this is considered bad style.
<BR>
Sometimes, variables are lazily initialized, treating nil as either true or false. 
This might be ok, but only if there is ONLY one place, which defines the meaning of nil (i.e. in  a getter).
<P>
Notice, that some confirmation dialogs return true, false or nil on cancel.'

    "Modified: / 07-03-2012 / 21:37:43 / cg"
! !

!RBEqualsTrueRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf: #('true' 'false')
                do: [ :node :answer | 
                        answer isNil 
                                ifTrue: [
                                        (node parent isMessage and: [ #(#= #== #~= #~~) includes: node parent selector ]) 
                                                ifTrue: [ node ]
                                                ifFalse: [ nil ] ]
                                ifFalse: [ answer ] ]

    "Modified: / 02-03-2012 / 01:23:41 / cg"
! !

!RBEqualsTrueRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBEqualsTrueRule.st,v 1.4 2012/03/08 01:04:47 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBEqualsTrueRule.st,v 1.4 2012/03/08 01:04:47 cg Exp $'
!

version_SVN
    ^ 'Id: RBEqualsTrueRule.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
