"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBLawOfDemeterRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBLawOfDemeterRule methodsFor:'accessing'!

group
	^ 'Intention revealing'
!

name
	^ 'Law of demeter'
!

rationale
    ^ 'The Law of Demeter is a design guideline for developing software and can be succinctly summarized as 
"Only talk to your immediate friends". 
The fundamental notion is that a given object should assume as little as possible 
about the structure or properties of anything else. 
If long method chains are used a lot of system knowledge is hardcoded into a single method 
and might make reusability difficult.
<P>
Instead of "<code>foo bar baz</code>", better add a forwarding "<code>baz</code>"-method to foo''s class and
replace by "<code>foo baz</code>". So the knowledge that there is a bar-object in between
is now limited to foo''s class. This avoids a need to change all senders,
whenever the substructure of foo (or any along the chain) changes in the future.'

    "Modified: / 01-03-2012 / 12:46:48 / cg"
! !

!RBLawOfDemeterRule methodsFor:'initialization'!

initialize
        "only look for unary-selector chains"

        super initialize.
        self matcher 
                "/ matches: '(((`@reciver `@msg1: `@arg1) `@msg2: `@arg2) `@msg3: `@arg3)  `@msg4: `@arg4'
                matches: '((`@reciver `@msg1: `@arg1) `@msg2: `@arg2) `@msg3: `@arg3 '
                do: [ :node :answer | 
                    node selector isUnarySelector 
                        ifTrue:[ node ]
                        ifFalse:[ nil ]
                ]

    "Modified: / 08-03-2012 / 03:20:25 / cg"
! !

!RBLawOfDemeterRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLawOfDemeterRule.st,v 1.4 2012/03/08 02:41:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLawOfDemeterRule.st,v 1.4 2012/03/08 02:41:27 cg Exp $'
!

version_SVN
    ^ 'Id: RBLawOfDemeterRule.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
