"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

Object subclass:#RBLintRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!


!RBLintRule class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!RBLintRule class methodsFor:'testing'!

isVisible
	"Answer true if the class should be visible in the GUI."

	^ false
! !

!RBLintRule methodsFor:'accessing'!

changes
	^ #()
!

failedClasses
    |s|

    s := OrderedCollection new.
    self failedClassesInto:s.
    ^ s asArray

    "Created: / 22-07-2009 / 14:18:26 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 26-10-2009 / 16:36:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

failedMethods
    |s|

    s := OrderedSet new .
    self failedMethodsInto:s.
    s := s select:[:el | el notNil].
    ^ s asArray

    "Created: / 22-07-2009 / 14:18:40 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 26-10-2009 / 16:36:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-08-2011 / 19:30:10 / cg"
!

failedMethodsInAnyOf: classes meta: meta

    | klasses |
    klasses := meta 
                ifTrue:[classes collect:[:cls|cls theMetaclass]]
                ifFalse:[classes collect:[:cls|cls theNonMetaclass]].

    ^self failedMethods select:
        [:mth|klasses includes: mth mclass].

    "Created: / 02-02-2010 / 21:45:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

flattened

    | s |
    s := (OrderedCollection new: 16) writeStream.
    self flattenOn: s.
    ^s contents.

    "Created: / 21-07-2009 / 23:13:01 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 22-07-2009 / 15:47:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

group
	^ String new
!

name
	self subclassResponsibility
!

problemCount
	self subclassResponsibility
!

rationale
	^ String new
!

rationaleWithAnchor
    self isComposite ifTrue:[^ self rationale].

    ^ '
<b>',self name,'</b>
<p>', 
self rationale,
'<p>
Class: <A ACTION="',self className,' browse">',self className,'</A>'

    "Created: / 01-03-2012 / 20:58:43 / cg"
!

ruleCount

    ^1

    "Created: / 24-07-2010 / 15:38:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

scope

    ^self subclassResponsibility
!

severity

    ^#warning

    "Created: / 14-03-2011 / 14:11:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBLintRule methodsFor:'displaying'!

displayString

    ^Text streamContents:
        [:s|
            | scope group |
            s nextPutAll: self name.

            (group := self group) notEmptyOrNil ifTrue:[
                s nextPutAll:((' (in ', group , ')') asText colorizeAllWith: Color gray)
            ].
            (scope := self scope) notNil ifTrue:[
                s nextPutAll:((' [', scope , ']') asText colorizeAllWith: Color gray)
            ].
        ]

    "Created: / 02-02-2010 / 20:21:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 01-03-2012 / 16:51:50 / cg"
! !

!RBLintRule methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    "/ please change as required (and remove this comment)

    "/ super initialize.   -- commented since inherited method does nothing
! !

!RBLintRule methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	self name isNil 
		ifFalse: [ aStream nextPutAll: ' name: '; print: self name ]
! !

!RBLintRule methodsFor:'private'!

addClass:class selector:selector to:result 

    (self shouldIgnoreClass: class selector: selector) ifFalse:
        [result 
            addClass:class
            selector:selector]

    "Created: / 24-08-2010 / 21:28:19 / Jan Vrany <enter your email here>"
!

failedClassesInto:arg 
    "raise an error: must be redefined in concrete subclass(es)"
    
    ^ self subclassResponsibility

    "Created: / 10-01-2009 / 15:13:54 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

failedMethodsInto:arg 
    "raise an error: must be redefined in concrete subclass(es)"
    
    ^ self subclassResponsibility

    "Created: / 08-01-2009 / 09:46:53 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

failedRules
	^self isEmpty
		ifTrue: [#()]
		ifFalse: [Array with: self]
!

flattenOn: stream

    ^stream nextPut: self.

    "Created: / 21-07-2009 / 23:11:25 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

genericPatternForSelector: aSymbol
	^ String streamContents: [ :stream |
		aSymbol keywords keysAndValuesDo: [ :index :value |
			stream space; nextPutAll: value.
			aSymbol last = $:
				ifTrue: [ stream space; nextPutAll: '`@object'; print: index ] ] ]
! !

!RBLintRule methodsFor:'running'!

checkClass: aContext
!

checkMethod: aContext
!

resetResult
!

run
	^ SmalllintChecker runRule: self
!

runOnEnvironment: anEnvironment 
	^ SmalllintChecker runRule: self onEnvironment: anEnvironment
!

shouldIgnoreClass: class selector: selector

   (class >> selector) annotationsAt: #ignore:rationale:author: do:
        [:ann|
        (ann rule = self name or:[ann rule == self class name]) 
            ifTrue:[^true]].
    ^false.

    "Created: / 24-08-2010 / 21:30:46 / Jan Vrany <enter your email here>"
!

shouldIgnoreMethod: aContext

    ^self shouldIgnoreClass: aContext selectedClass selector: aContext selector

    "Created: / 24-08-2010 / 20:48:14 / Jan Vrany <enter your email here>"
! !

!RBLintRule methodsFor:'testing'!

hasConflicts
	^ false
!

isComposite
	^ false
!

isEmpty
	self subclassResponsibility
!

isEmptyInTree
    "added due to the semantic confusion of isEmpty 
     (means both: has subitems AND has subitems with result !!)"

    ^ false

    "Created: / 01-03-2012 / 19:05:01 / cg"
!

isLintChecksGroupParent
    "still bad, but better than comparing in the browser..."

    ^ self name = 'Lint checks'

    "Created: / 07-03-2012 / 20:11:53 / cg"
!

isSqueakBugsGroupParent
    "still bad, but better than comparing in the browser..."

    ^ self name = 'Bugs - Squeak'

    "Created: / 07-03-2012 / 20:12:29 / cg"
! !

!RBLintRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLintRule.st,v 1.6 2012/03/07 19:14:40 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLintRule.st,v 1.6 2012/03/07 19:14:40 cg Exp $'
!

version_SVN
    ^ 'Id: RBLintRule.st 27 2011-03-14 17:33:50Z vranyj1 '
! !
