"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBBlockLintRule subclass:#RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule
	instanceVariableNames:'matcher'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!


!RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule methodsFor:'accessing'!

group
	^ 'Possible bugs'
!

name
	^ 'Literal array contains a #true, #false, or #nil but the source doesn''t.'
!

rationale
        ^ 'With ANSI changes, "<code>#(true false nil)</code>" now is equal to 
"<code>{true. false. nil}</code>" not "<code>{#true. #false. #nil}</code>" as it used to be in ST80.
<P>Notice: the ST/X behavior has always been the same as the new ANSI behavior,
so this is usually only a potential bug, when porting old VW, V''Age or Squeak code to ST/X.'

    "Modified: / 01-03-2012 / 12:26:01 / cg"
! !

!RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule methodsFor:'as yet unclassified'!

initialize
    super initialize.
    matcher := ParseTreeSearcher new.
    matcher matches:'`{ :n | n isLiteral and: [ n value isKindOf: Array ] }'
        do:[:aNode :answer | 
            answer
                addAll:(self literalTrueFalseOrNilSymbolsIn:aNode value);
                yourself
        ]
! !

!RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule methodsFor:'private'!

literalTrueFalseOrNilSymbolsIn: aLiteral 
	| retval |
	aLiteral class == Array ifFalse: [^#()].
	retval := OrderedCollection withAll: (aLiteral select: [:ea | ea isSymbol and: [#(#true #false #nil ) includes: ea]]).
	aLiteral do: [ :each | retval addAll: (self literalTrueFalseOrNilSymbolsIn: each) ].
	^retval.
! !

!RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule methodsFor:'running'!

checkMethod: aContext 
	| compiledLits parsedLits |
	compiledLits := aContext compiledMethod allLiterals
		inject: OrderedCollection new
		into: [ :collection :literal |
			collection
				addAll: (self literalTrueFalseOrNilSymbolsIn: literal);
				yourself ].
	compiledLits size > 0 ifTrue: [ 
		parsedLits := OrderedCollection new.
		matcher 
			executeTree: aContext parseTree
			initialAnswer: parsedLits.
		compiledLits size ~= parsedLits size ifTrue: [
			result 
				addClass: aContext selectedClass
				selector: aContext selector ] ]
! !

!RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule.st,v 1.3 2012/03/01 12:10:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule.st,v 1.3 2012/03/01 12:10:20 cg Exp $'
!

version_SVN
    ^ 'Id: RBLiteralArrayContainsSuspiciousTrueFalseOrNilRule.st 24 2010-08-25 12:47:16Z vranyj1 '
! !
