"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBBlockLintRule subclass:#RBMissingSuperSendsRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!


!RBMissingSuperSendsRule methodsFor:'accessing'!

group
	^ 'Possible bugs'
!

name
	^ 'Missing super sends in selected methods.'
!

rationale
        ^ 'Checks that some methods contain a "<code>super</code>" message send. 
Some methods should always contain a super message send. 
For example, the "<code>postCopy</code>" method should always contain a 
"<code>super postCopy</code>". 
<p>
The list of methods that should contain super message sends is defined in 
<A ACTION="RBMissingSuperSendsRule browse">RBMissingSuperSendsRule>>superMessages</A>.'

    "Modified: / 01-03-2012 / 12:23:11 / cg"
! !

!RBMissingSuperSendsRule methodsFor:'private'!

superMessages
    ^ #( 
        #release 
        #postCopy 
        #postBuildWith: 
        #preBuildWith: 
        #postOpenWith: 
        #noticeOfWindowClose: 
        #initialize )

    "Modified (format): / 01-03-2012 / 12:16:48 / cg"
! !

!RBMissingSuperSendsRule methodsFor:'running'!

checkMethod: aContext 
        | definer superMethod |
        (aContext selectedClass isMeta not and: [ self superMessages includes: aContext selector ]) ifTrue: 
                [ definer := aContext selectedClass superclass ifNotNilDo: [ :sc | sc whichClassIncludesSelector: aContext selector ].
                definer ifNotNil: 
                        [ "super defines same method"
                        (aContext superMessages includes: aContext selector) ifFalse: 
                                [ "but I don't call it"
                                superMethod := definer 
                                        compiledMethodAt: aContext selector
                                        ifAbsent: [  ].
                                "FIXME: Look into Squeak code how #isReturnSelf is implemented"
                                ( "superMethod isReturnSelf" false 
                                or: [ (superMethod sendsSelector: #subclassResponsibility)
                                or: [ (superMethod sendsSelector: #subclassResponsibility:) ]]) ifFalse: 
                                        [ result 
                                                addClass: aContext selectedClass
                                                selector: aContext selector ] ] ] ]

    "Modified: / 05-08-2011 / 13:46:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 06-03-2012 / 15:48:38 / cg"
! !

!RBMissingSuperSendsRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBMissingSuperSendsRule.st,v 1.5 2012/03/06 14:49:09 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBMissingSuperSendsRule.st,v 1.5 2012/03/06 14:49:09 cg Exp $'
!

version_SVN
    ^ 'Id: RBMissingSuperSendsRule.st 24 2010-08-25 12:47:16Z vranyj1 '
! !
