"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBMissingYourselfRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBMissingYourselfRule methodsFor:'accessing'!

group
	^ 'Possible bugs'
!

name
	^ 'Possible missing "; yourself"'
!

rationale
        ^ 'Checks for missing "<code>; yourself</code>" cascaded message send
 for cascaded messages that are used. 
This helps locate common coding mistakes such as 
<br>&nbsp;&nbsp;<code>anArray := (Array new: 2) at: 1 put: 1; at: 2 put: 2</code>. 
<br>In this example, <i>anArray</i> would be assigned to 2 not the array object,
due to the (possibly stupid) decision to return the argument from the "<code>at:put:</code>" message).
<P>
This check may also bark at code, where the last expression in a block (which is its return value) is such 
an expression. Most of the time, this can be ignored; especially: if the caller is a collection enumerator
or the cascade-message happens to return the receiver.
The checker does not know or check if the block''s value is actually needed/used
and what value is returned by the last cascade message.'

    "Modified: / 06-03-2012 / 18:53:24 / cg"
! !

!RBMissingYourselfRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matches: '`{:node | 
                                node isMessage 
                                    and: [node parent isCascade 
                                    and: [node isDirectlyUsed 
                                    and: [node selector ~~ #yourself]]]}'
                do: [ :node :answer | node ]

    "Modified: / 06-03-2012 / 18:52:16 / cg"
! !

!RBMissingYourselfRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBMissingYourselfRule.st,v 1.3 2012/03/06 17:56:57 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBMissingYourselfRule.st,v 1.3 2012/03/06 17:56:57 cg Exp $'
!

version_SVN
    ^ 'Id: RBMissingYourselfRule.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
