"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBObfuscatedLazyInitializerRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!


!RBObfuscatedLazyInitializerRule methodsFor:'accessing'!

group
	^ 'Intention revealing'
!

name
        ^ 'Obfuscated lazy initialization'

    "Modified: / 13-02-2012 / 11:23:33 / cg"
!

rationale
        ^ 'Checks for lazy initialization, written in an unclear way.
Notice that the definition of "unclear" is very subjective. 
Some proclaim exactly this style as being "elegant",
whereas others believe it takes extra reading to understand it. 
The general distinction is between functional style:
<CODE><PRE>
    ^ foo isNil ifTrue:[ foo := initExpr ] ifFalse:[ foo ]
</PRE></CODE>
and guarding clause style:
<CODE><PRE>
    foo isNil ifTrue:[ foo := initExpr ].
    ^ foo
</PRE></CODE>
The authotr of this rule would argue, that the side effect in the functional style is too easily overlooked and completely non-functional.
And that it is better to guide the reader to this side effect, by giving it an exposed look in the code.
'

    "Modified: / 01-03-2012 / 11:22:57 / cg"
! !

!RBObfuscatedLazyInitializerRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyMethodOf: #(
                        '`@MethodName
                                ^ `@MethodName isNil 
                                    ifTrue: [ `@MethodName := `@initExpression ] 
                                    ifFalse: [ `@MethodName ]' 
                )
                do: [ :node :answer | node ]

    "Modified: / 13-02-2012 / 11:23:09 / cg"
! !

!RBObfuscatedLazyInitializerRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBObfuscatedLazyInitializerRule.st,v 1.3 2012/03/01 12:17:13 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBObfuscatedLazyInitializerRule.st,v 1.3 2012/03/01 12:17:13 cg Exp $'
! !
