"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBBlockLintRule subclass:#RBReturnsBooleanAndOtherRule
	instanceVariableNames:'matcher'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!


!RBReturnsBooleanAndOtherRule methodsFor:'accessing'!

group
	^ 'Possible bugs'
!

name
	^ 'Returns a boolean and non boolean'
!

rationale
        ^ 'Checks for methods that return a boolean value (true or false) 
and return some other value such as (nil or self). 
If the method is suppose to return a boolean, 
then this signifies that there is one path through the method that might return a non-boolean. 
If the method doesn''t need to return a boolean, 
you should probably rewrite it to return some non-boolean value since other programmers reading your method might assume that it returns a boolean.
<P>
Notice that there are situations, in which a three-way result makes sense;
for example, some dialogs return true/false/nil for "yes" / "no" / "cancelled"'

    "Modified: / 01-03-2012 / 12:09:26 / cg"
! !

!RBReturnsBooleanAndOtherRule methodsFor:'initialization'!

initialize
    super initialize.
    matcher := ParseTreeSearcher new.
    matcher matches:'^ ``@object'
        do:[:node :answer | 
            answer
                add:node value;
                yourself
        ]
! !

!RBReturnsBooleanAndOtherRule methodsFor:'running'!

checkMethod: aContext 
	| hasBool hasSelf |
	hasBool := false.
	hasSelf := aContext parseTree lastIsReturn not.
	(matcher executeTree: aContext parseTree initialAnswer: Set new) do: [ :each | 
		hasBool := hasBool 
			or: [ (each isLiteral and: [ #(true false) includes: each value ]) 
			or: [ (each isMessage and: [ #(and: or:) includes: each selector ]) ] ].
		hasSelf := hasSelf
			or: [ (each isVariable and: [ each name = 'self' ]) 
			or: [ (each isLiteral and: [ (#(true false) includes: each value) not ]) ] ] ].
	(hasSelf and: [ hasBool ]) 
		ifTrue: [ result addClass: aContext selectedClass selector: aContext selector ]
! !

!RBReturnsBooleanAndOtherRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBReturnsBooleanAndOtherRule.st,v 1.3 2012/03/01 12:45:48 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBReturnsBooleanAndOtherRule.st,v 1.3 2012/03/01 12:45:48 cg Exp $'
!

version_SVN
    ^ 'Id: RBReturnsBooleanAndOtherRule.st 24 2010-08-25 12:47:16Z vranyj1 '
! !
