"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

RBParseTreeLintRule subclass:#RBSendsUnknownMessageToGlobalRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBSendsUnknownMessageToGlobalRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
	^ 'Sends unknown message to global'
!

rationale
        ^ 'Check for unknown messages sent to a known global.

In some cases, the receiver redefines "<code>doesNotUnderstand:</code>" to emulate the 
protocol of another class or to forward messages to a delegate.
For example, Transcript is doing this, to emulate stream protocol.
However, this lint rule knows about that
special behavior of Transcript and does not complain (but only for Transcript).'

    "Created: / 07-03-2012 / 17:43:35 / cg"
! !

!RBSendsUnknownMessageToGlobalRule methodsFor:'initialization'!

initialize
    super initialize.
    self matcher 
        matches:'`{:node :context | node isVariable and: [ Smalltalk includesKey: node name asSymbol ] } `@message: `@args'
        do:[:node :oldAnswer |
            |newAnswer|

            (newAnswer := oldAnswer) isNil ifTrue:[
                |what|

                what := Smalltalk at:node receiver name asSymbol.
                (what notNil 
                        and:[ what ~~ Preferences 
                        and:[ (what isBehavior not or:[ what isLoaded ]) 
                        and:[ (what respondsTo:node selector) not ]]]) 
                ifTrue:[ 
                    "/ special: check for Transcript, which delegates some stream messages...
                    "/ this is pure heuristics
                    ((what isKindOf:TextCollector) 
                    and:[ (Stream canUnderstand:node selector) ]) ifFalse:[    
                        newAnswer := node 
                    ]
                ]
            ].
            newAnswer
        ]

    "Modified (format): / 07-03-2012 / 17:57:19 / cg"
! !

!RBSendsUnknownMessageToGlobalRule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBSendsUnknownMessageToGlobalRule.st,v 1.3 2012/03/07 16:59:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/RBSendsUnknownMessageToGlobalRule.st,v 1.3 2012/03/07 16:59:21 cg Exp $'
!

version_SVN
    ^ 'Id: RBSendsUnknownMessageToGlobalRule.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
