"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

Object subclass:#SmalllintContext
	instanceVariableNames:'class selector parseTree literals literalSemaphore literalProcess
		selectors compiledMethod selfMessages superMessages messages
		allMessages'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!


!SmalllintContext class methodsFor:'instance creation'!

new
	^(super new)
		initialize;
		yourself
!

newNoCache
	^self basicNew
! !

!SmalllintContext class methodsFor:'others'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintContext.st,v 1.14 2012/04/07 10:59:47 cg Exp $'
! !

!SmalllintContext methodsFor:'accessing'!

allMessages
        allMessages isNil ifTrue: [self computeMessages].
        ^allMessages

    "Created: / 07-04-2012 / 12:56:03 / cg"
!

compiledMethod
	^compiledMethod notNil
		ifTrue: [compiledMethod]
		ifFalse: [compiledMethod := class compiledMethodAt: selector]
!

instVarNames
	^self selectedClass allInstVarNames
!

literals
        |s|

        (s := literalSemaphore) isNil
                ifTrue: 
                        [literals isNil ifTrue: 
                                        [self computeLiterals wait]]
                ifFalse: [s wait].
        ^literals

    "Modified: / 07-03-2012 / 18:09:53 / cg"
!

messages
	messages isNil ifTrue: [self computeMessages].
	^messages
!

parseTree
	^parseTree isNil
		ifTrue: [parseTree := self buildParseTree]
		ifFalse: [parseTree]
!

protocol
	^self selectedClass whichCategoryIncludesSelector: self selector
!

protocols
	^Array with: self protocol
!

selectedClass
	^class
!

selectedClass: anObject 
	class := anObject.
	self selector: nil
!

selector
	^selector
!

selector: anObject 
	selector := anObject.
	parseTree := compiledMethod := selfMessages := superMessages := messages := nil
!

selectors
        |s|

        (s := literalSemaphore) isNil
                ifTrue: 
                        [selectors isNil ifTrue: 
                                        [self computeLiterals wait]]
                ifFalse: [s wait].
        ^selectors

    "Modified: / 07-03-2012 / 18:09:49 / cg"
!

selfMessages
	selfMessages isNil ifTrue: [self computeMessages].
	^selfMessages
!

sourceCode

    ^self compiledMethod source
!

superMessages
	superMessages isNil ifTrue: [self computeMessages].
	^superMessages
! !

!SmalllintContext methodsFor:'initialize-release'!

initialize
	self computeLiterals
!

release
	literalProcess notNil ifTrue: [literalProcess terminate].
	super release
! !

!SmalllintContext methodsFor:'private'!

addLiteralsFor: aCompiledMethod 
	aCompiledMethod withAllBlockMethodsDo: 
			[:each | 
			each literalsDo: [:literal | self checkLiteral: literal]]
!

buildParseTree
        | tree |
        tree := self selectedClass parseTreeFor: self selector.
        tree isNil ifTrue: [^RBParser parseMethod: 'method'].
        ^tree
!

checkLiteral: aLiteral 
	(aLiteral isSymbol or: [aLiteral isVariableBinding])
		ifTrue: [literals add: aLiteral]
		ifFalse: 
			[aLiteral class == Array
				ifTrue: [aLiteral do: [:each | self checkLiteral: each]]]
!

computeLiterals
        |sema|

        sema := literalSemaphore := Semaphore new.
        literalProcess := [self primitiveComputeLiterals] fork.
        ^ sema

    "Modified: / 07-03-2012 / 18:07:48 / cg"
!

computeLiteralsForClass: aClass 
	(selectors addAll: aClass selectors) do: 
			[:sel | 
			self computeLiteralsForSelector: sel in: aClass.
			Processor yield]
!

computeLiteralsForSelector: aSelector in: aClass 
        | method |
        method := aClass compiledMethodAt: aSelector ifAbsent: [nil].
        method notNil ifTrue: [
            self addLiteralsFor: method
        ]

    "Modified: / 01-03-2012 / 14:36:31 / cg"
!

computeMessages
        | searcher |
        selfMessages := Set new.
        superMessages := Set new.
        messages := Set new.
        searcher := ParseTreeSearcher new.
        searcher
                matches: 'self `@message: ``@args'
                        do: [:aNode :answer | selfMessages add: aNode selector];
                matches: 'super `@message: ``@args'
                        do: [:aNode :answer | superMessages add: aNode selector];
                matches: '``@receiver `@message: ``@args'
                        do: [:aNode :answer | messages add: aNode selector].
        searcher executeTree: self parseTree initialAnswer: nil.

        "/ cg: messages should include self and super messages as well;
        "/ I add allMessages, just in case to avoid breaking some code which depends
        "/ on the sets being disjoint.
        "/ late note: yes it would (sentNotImplementedRule thinks them disjoint)
        allMessages := Set new.
        allMessages addAll: messages; addAll: selfMessages; addAll: superMessages.

    "Modified (comment): / 07-04-2012 / 12:57:28 / cg"
!

primitiveComputeLiterals
	| semaphore |
	literals := IdentitySet new: 25000.
	literals addAll: self specialSelectors.
	selectors := IdentitySet new.
	Smalltalk allBehaviorsDo: [:aClass | self computeLiteralsForClass: aClass].
	semaphore := literalSemaphore.
	literalSemaphore := nil.
	self signalProcesses: semaphore.
	^literalProcess := nil
!

signalProcesses: aSemaphore 
	aSemaphore isNil ifTrue: [^self].
	[aSemaphore isEmpty] whileFalse: [aSemaphore signal]
!

specialSelectors
        RBParser isSmalltalkX ifTrue:[
            ^ #()
        ].
        ^(OpcodePool at: #SpecialSelectorDictionary) keys

    "Modified: / 4.2.2000 / 11:11:41 / cg"
! !

!SmalllintContext methodsFor:'testing'!

implements: aSelector 
	^self selectors includes: aSelector
!

includesBehaviorNamed: clsName

    | cls |
    cls := Smalltalk at: clsName asSymbol.
    ^cls notNil and:[self selectedClass includesBehavior: cls].

    "Created: / 05-08-2011 / 13:48:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isAbstract: aClass 
	^(aClass isMeta or: 
			[(self literals includes: aClass name)
				or: [self literals includes: (Smalltalk associationAt: aClass name)]])
		not
!

uses: anObject 
	^self literals includes: anObject
! !

!SmalllintContext class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintContext.st,v 1.14 2012/04/07 10:59:47 cg Exp $'
!

version_SVN
    ^ 'Id: SmalllintContext.st 19 2010-02-02 21:20:19Z vranyj1 '
! !
