"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

TestCase subclass:#SmalllintTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!


!SmalllintTests methodsFor:'testing'!

test01
    |myLint myRules composite myEnvironment output|

    myRules := OrderedCollection new.
    myRules
        addFirst:ParseTreeLintRule equalNotUsed;
        addFirst:BlockLintRule unreferencedVariables;
        addFirst:ParseTreeLintRule fileBlocks.
    composite := CompositeLintRule new.
    composite rules:myRules.
    myEnvironment := ClassEnvironment new.
    myEnvironment
        addClass:SmalllintTestsMock01;
        addClass:SmalllintTestsMock02.
    myLint := Smalllint new.
    output := myLint
                setEnvironment:myEnvironment;
                setRules:composite;
                runChecker;
                getResults.
    self assert:composite isNil = false.
    self assert:composite problemCount = 4.
    self assert:(composite failedRules) size = 2.
    self assert:(composite failedRules) size = output size

    "Created: / 19-11-2008 / 13:59:28 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 26-11-2008 / 14:10:09 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-12-2008 / 18:52:50 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Modified: / 16-12-2008 / 15:19:53 / bazantj <enter your email here>"
!

test02
| myLint myRules composite myEnvironment output|
myRules := OrderedCollection new.
myRules addFirst: ParseTreeLintRule equalNotUsed;
        addFirst: BlockLintRule unreferencedVariables.

composite := CompositeLintRule new.
composite rules: myRules.

myEnvironment := ClassEnvironment new.
myEnvironment addClass: Smalllint.

myLint := Smalllint new. 
output := myLint 
        setEnvironment: myEnvironment;
        setRules: composite;
        runChecker;
        getResults.

self assert: output isNil = false.
self assert: composite problemCount = 0.
self assert: composite failedRules isEmpty

    "Created: / 26-11-2008 / 14:13:25 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-12-2008 / 18:55:20 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Modified: / 15-01-2009 / 08:07:45 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

test03
| myEnvironment composite lint|

myEnvironment := ClassEnvironment new.
myEnvironment addClass: SmalllintTestsMock01.

lint := Smalllint runAllChecksOnEnvironment: myEnvironment.

composite := lint currentChecker.
self assert: composite isNil = false.
self assert: composite problemCount = 14.
self assert: (composite failedRules) size = 8

    "Created: / 26-11-2008 / 14:55:51 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-12-2008 / 19:00:58 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Modified: / 20-12-2008 / 15:19:52 / bazantj <enter your email here>"
    "Modified: / 15-01-2009 / 08:09:17 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

test04
| myRules composite myEnvironment output lint |
myRules := OrderedCollection new.
myRules addFirst: ParseTreeLintRule equalNotUsed;
        addFirst: BlockLintRule unreferencedVariables.

myEnvironment := ClassEnvironment new.
myEnvironment addClass: SmalllintTestsMock01.

lint := Smalllint runRules: myRules onEnvironment: myEnvironment.

output := lint getResults.
composite := lint currentChecker.

self assert: composite isNil = false.
self assert: composite problemCount = 4.
self assert: (composite failedRules) size = 2.
self assert: (composite failedRules) size  = output size

    "Created: / 26-11-2008 / 20:47:48 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-12-2008 / 19:14:46 / Josef Grega <gregaj1@fel.cvut.cz>"
!

test05
| output |

output := (Smalllint 
                runRule: ParseTreeLintRule equalNotUsed
                onEnvironment: (ClassEnvironment new addClass: SmalllintTestsMock01))
        getResults.

self assert: output isNil = false.
self assert: output size = 1.

    "Created: / 26-11-2008 / 20:56:19 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 06-12-2008 / 21:18:06 / bazantj <enter your email here>"
    "Modified: / 15-12-2008 / 19:36:27 / Josef Grega <gregaj1@fel.cvut.cz>"
!

testInfo01
| myEnvironment output lint failedMethods failedClasses|

myEnvironment := ClassEnvironment new.
myEnvironment addClass: SmalllintTestsMock01.

lint := Smalllint runRule: (ParseTreeLintRule equalNotUsed) onEnvironment: myEnvironment.

output := lint getResults.

failedMethods := IdentityDictionary new.
failedClasses := IdentityDictionary new.

output do:[
        :failedRule |
        failedRule failedMethodsInfoOn: failedMethods.
        failedRule failedClassesInfoOn: failedClasses.
].
self assert: failedClasses size = 1.
self assert: failedClasses first size = 1.
self assert: failedMethods size = 2.
self assert: failedMethods first size = 1.

    "Modified: / 15-12-2008 / 19:14:46 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Created: / 12-01-2009 / 12:47:55 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-01-2009 / 08:37:47 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

testInfo02
| myEnvironment output lint methodDict classDict |

myEnvironment := ClassEnvironment new.
myEnvironment addClass: SmalllintTestsMock03.

lint := Smalllint runRule: (BlockLintRule equivalentSuperclassMethods) onEnvironment: myEnvironment.

output := lint getResults.

methodDict := IdentityDictionary new.
classDict := IdentityDictionary new.

output do:[
        :failedRule |
        failedRule failedMethodsInfoOn: methodDict.
        failedRule failedClassesInfoOn: classDict.
].
self assert: classDict size = 1.
self assert: classDict first size = 1.
self assert: methodDict size = 1.
self assert: methodDict first size = 1.

    "Modified: / 15-12-2008 / 19:14:46 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Created: / 12-01-2009 / 22:44:20 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-01-2009 / 08:37:41 / Jiri Bazant <bazanj2@fel.cvut.cz>"
!

testVarEnv01
| myEnvironment output lint methodDict classDict |

myEnvironment := ClassEnvironment new.
myEnvironment addClass: SmalllintTestsMock01.

lint := Smalllint runRule: (BlockLintRule instVarInSubclasses) onEnvironment: myEnvironment.

output := lint getResults.

methodDict := IdentityDictionary new.
classDict := IdentityDictionary new.

output do:[
        :failedRule |
        failedRule failedMethodsInfoOn: methodDict.
        failedRule failedClassesInfoOn: classDict.

].
self assert: classDict size = 1.
self assert: classDict first size = 1.
self assert: methodDict isEmpty.

    "Modified: / 15-12-2008 / 19:14:46 / Josef Grega <gregaj1@fel.cvut.cz>"
    "Created: / 13-01-2009 / 16:50:29 / Jiri Bazant <bazanj2@fel.cvut.cz>"
    "Modified: / 15-01-2009 / 08:38:22 / Jiri Bazant <bazanj2@fel.cvut.cz>"
! !

!SmalllintTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintTests.st,v 1.1 2011/07/09 17:30:30 vrany Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintTests.st,v 1.1 2011/07/09 17:30:30 vrany Exp $'
! !
