"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

TestCase subclass:#SmalllintTests2
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Tests'
!

Object subclass:#TestClass
	instanceVariableNames:'temporaryVariable'
	classVariableNames:''
	poolDictionaries:''
	privateIn:SmalllintTests2
!

SmalllintTests2 comment:''
!


!SmalllintTests2 class methodsFor:'accessing'!

packageNamesUnderTest
	^ #('Refactoring-Critics')
! !

!SmalllintTests2 methodsFor:'private'!

checkRule: aLintRule isEqualTo: anEnvironment 
	| returnedEnvironment |
	returnedEnvironment := aLintRule result.
	self compare: returnedEnvironment to: anEnvironment.
	self compare: anEnvironment to: returnedEnvironment
!

compare:subEnvironment to:superEnvironment 
    subEnvironment 
        classesDo:[:class | 
            (subEnvironment selectorsForClass:class) do:[:selector | 
                self assert:(superEnvironment includesSelector:selector in:class)
            ]
        ]

    "Modified (format): / 23-07-2012 / 12:38:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

currentSelector
	^ testSelector
!

ruleFor: aSelector 
	self ruleFor: aSelector plusSelectors: #()
!

ruleFor: aSelector plusSelectors: symbolCollection 
        | returnedEnvironment rule class selector className |
        selector := aSelector asString 
                copyFrom: 5
                to: aSelector size.
        className := 'RB' , selector , 'Rule'.
        class := Smalltalk classNamed: className.
        class isNil ifTrue: [ self error: className , ' class not found' ].
        selector 
                at: 1
                put: selector first asLowercase.
        selector := selector asSymbol.
        SmalllintChecker 
                runRule: (rule := class new)
                onEnvironment: self smalllintTestEnvironment.
        returnedEnvironment := SelectorEnvironment new.
        returnedEnvironment 
                addClass: TestClass
                selector: selector.
        symbolCollection do: [ :each | 
                returnedEnvironment 
                        addClass: TestClass
                        selector: each ].
        self 
                assert: (rule name isString and: [ rule name notEmpty ])
                description: 'Missing rule name'.
        self 
                assert: (rule group isString and: [ rule group notEmpty ])
                description: 'Missing group name'.
        self 
                assert: (rule rationale isString and: [ rule rationale notEmpty ])
                description: 'Missing rationale'.
        self
                assert: (#(error warning information) includes: rule severity)
                description: 'Invalid severity'.
        self
                checkRule: rule
                isEqualTo: returnedEnvironment

    "Modified: / 23-07-2012 / 13:29:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

smalllintTestEnvironment
        | classEnvironment |
        classEnvironment := ClassEnvironment new.
        classEnvironment addClass: TestClass.
        ^ classEnvironment

    "Modified: / 23-07-2012 / 13:29:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SmalllintTests2 methodsFor:'tests'!

testAsOrderedCollectionNotNeeded
	self ruleFor: self currentSelector
!

testAssignmentInBlock
	self ruleFor: self currentSelector
!

testAssignmentInIfTrue
	self ruleFor: self currentSelector
!

testAssignmentWithoutEffect
	self ruleFor: self currentSelector
!

testAtIfAbsent
	self ruleFor: self currentSelector
!

testBadMessage
	self ruleFor: self currentSelector plusSelectors: #(codeCruftLeftInMethods)
!

testBooleanPrecedence
	self ruleFor: self currentSelector
!

testCodeCruftLeftInMethods
	self ruleFor: self currentSelector
!

testCollectSelectNotUsed
	self ruleFor: self currentSelector
!

testCollectionMessagesToExternalObject
	self ruleFor: self currentSelector
!

testCollectionProtocol
	self ruleFor: self currentSelector
!

testConsistencyCheck
	self 
		ruleFor: self currentSelector
		plusSelectors: #(#noIsNil: )
!

testContains
	self ruleFor: self currentSelector
!

testDetectContains
	self ruleFor: self currentSelector
!

testEmptyExceptionHandler
	self ruleFor: self currentSelector
!

testEndTrueFalse
	self ruleFor: self currentSelector
!

testEqualNotUsed
	self ruleFor: self currentSelector
!

testEqualsTrue
	self ruleFor: self currentSelector
!

testEquivalentSuperclassMethods
        | returnedEnvironment rule |
        SmalllintChecker 
                runRule: (rule := RBEquivalentSuperclassMethodsRule new)
                onEnvironment: self smalllintTestEnvironment.
        returnedEnvironment := SelectorEnvironment new.
        returnedEnvironment 
                addClass: RBSmalllintTestObject
                selector: #isLiteral.
        self 
                checkRule: rule
                isEqualTo: returnedEnvironment

    "Modified: / 23-07-2012 / 13:29:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testExtraBlock
	self 
		ruleFor: self currentSelector
		plusSelectors: #(#testMethod1 )
!

testFileBlocks
	self ruleFor: self currentSelector
!

testFloatEqualityComparison
	self ruleFor: self currentSelector
!

testFullBlocks
	"skip this test in squeak"

"	self blockRuleFor: self currentSelector
		plusSelectors: #(#caller1 #fullBlocks #detectContains #fileBlocks #moveDefinition #caller #assignmentInBlock #equalNotUsed #stringConcatenation #noMoveDefinition #referencesConditionFor: #returnInEnsure)"
!

testGuardingClause
	self ruleFor: self currentSelector
!

testIfTrueBlocks
	self ruleFor: self currentSelector
!

testIfTrueReturns
	self ruleFor: self currentSelector
!

testJustSendsSuper
	self ruleFor: self currentSelector
!

testLiteralArrayCharacters
	self ruleFor: self currentSelector
!

testLiteralArrayContainsComma
	self ruleFor: self currentSelector
!

testLongMethods
	self ruleFor: self currentSelector
!

testMinMax
	self ruleFor: self currentSelector
!

testMissingYourself
	self 
		ruleFor: self currentSelector
		plusSelectors: #(#inlineMethod )
!

testModifiesCollection
	self ruleFor: self currentSelector
!

testOnlyReadOrWrittenTemporary
	self ruleFor: self currentSelector
!

testPrecedence
	self ruleFor: self currentSelector
!

testRefersToClass
	self ruleFor: self currentSelector
!

testReturnInEnsure
	self ruleFor: self currentSelector
!

testReturnsBooleanAndOther
	self ruleFor: self currentSelector
!

testReturnsIfTrue
	self ruleFor: self currentSelector
!

testSearchingLiteral
	self ruleFor: self currentSelector
!

testSendsDifferentSuper
	self ruleFor: self currentSelector
!

testSizeCheck
	self ruleFor: self currentSelector
!

testStringConcatenation
	self ruleFor: self currentSelector
!

testTempVarOverridesInstVar
	self ruleFor: self currentSelector
!

testTemporaryVariableCapitalization
	self ruleFor: self currentSelector
!

testTempsReadBeforeWritten
    self ruleFor:self currentSelector
        plusSelectors:#( #inlineTemporary #noMoveDefinition #tempsReadBeforeWritten #equalNotUsed #fileBlocks #referencesConditionFor: )

    "Modified (format): / 23-07-2012 / 12:38:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testThreeElementPoint
	self ruleFor: self currentSelector
!

testToDo
	self ruleFor: self currentSelector
!

testToDoCollect
	self ruleFor: self currentSelector
!

testToDoWithIncrement
	self ruleFor: self currentSelector
!

testUnclassifiedMethods
	self ruleFor: self currentSelector
!

testUncommonMessageSend
	self ruleFor: self currentSelector
!

testUnoptimizedAndOr
	self ruleFor: self currentSelector
!

testUsesAdd
	self ruleFor: self currentSelector
!

testUsesTrue
	self ruleFor: self currentSelector
!

testWhileTrue
	self ruleFor: self currentSelector
!

testYourselfNotUsed
	self ruleFor: self currentSelector
! !

!SmalllintTests2::TestClass methodsFor:'as yet unclassified'!

unclassifiedMethods
	"intentionally unclassified method"
! !

!SmalllintTests2::TestClass methodsFor:'methods'!

asOrderedCollectionNotNeeded
	self foo addAll: (1 to: 10) asOrderedCollection
!

assignmentInBlock
	[^self printString] ensure: [self close]
!

assignmentInIfTrue
	| variable |
	self isVariable 
		ifTrue: [variable := self]
		ifFalse: [variable := self printString].
	^variable
!

assignmentWithoutEffect
	| a |
	a := 1.
	a := a
!

atIfAbsent
	^ Smalltalk at: #MyTest
		ifAbsent: 
			[| collection |
			collection := #().
			Smalltalk at: #MyTest put: collection]
!

badMessage
	self become: String new
!

booleanPrecedence
	^true & 4 = 45
!

codeCruftLeftInMethods
	self halt
!

collectSelectNotUsed
	(1 to: 10) select: [:each | each = 4]
!

collectionMessagesToExternalObject
	self someObject collection remove: 10
!

collectionProtocol
	| newCollection |
	newCollection := OrderedCollection new.
	(1 to: 10) asOrderedCollection do: 
			[:each | 
			| new |
			new := each * 2.
			newCollection add: new].
	^newCollection
!

consistencyCheck
	^(1 to: 10) at: 1
!

contains
	^((1 to: 10) detect: [:each | each > 2] ifNone: [nil]) isNil
!

detectContains
	^(1 to: 10) do: [:each | each > 2 ifTrue: [^each]]
!

emptyExceptionHandler
	[ ] on: Error do: [ :err | ]
!

endTrueFalse
	self isVariable 
		ifTrue: 
			[self printString.
			self isVariable printString]
		ifFalse: 
			[self printString.
			^4]
!

equalNotUsed
	| string |
	string = '' yourself.
	(1 to: 10) do: [:i | string := i printString].
	^string
!

equalsTrue
	^true == self
!

extraBlock
	^[:arg | arg + 43] value: 45
!

fileBlocks
	| file |
	^
	[file := 'asdf' asFilename readStream.
	file contents] ensure: [file close]
!

floatEqualityComparison
	^ 1.0 = 1
!

fullBlocks
	^[thisContext]
!

guardingClause
	self isSymbol 
		ifFalse: 
			[self printString.
			self isSymbol printString]
!

ifTrueReturns
	self isSymbol ifFalse: [^true].
	^false
!

isLiteral
	^false
!

justSendsSuper
	super justSendsSuper
!

literalArrayCharacters
	^#($a $b $c) includes: $a
!

literalArrayContainsComma
	^ #(#,)
!

longMethods
	self printString.
	self printString.
	self isVariable ifTrue: [self printString].
	self isVariable ifTrue: [self printString].
	self isVariable ifTrue: [self printString].
	self isVariable ifTrue: [self printString]
!

minMax
	"Bug in 3<5 ifTrue: [3]  ifFalse: [5]"

^3<5 ifTrue: [3]  ifFalse: [5]
"	| var |
	var := 4.
	var < 5 ifTrue: [var := 5].
	^var"
!

missingYourself
	^(OrderedCollection new)
		add: 1;
		add: 2;
		removeFirst
!

modifiesCollection
	| collection |
	collection := (1 to: 10) asOrderedCollection.
	collection do: [:each | each > 5 ifTrue: [collection remove: each]].
	^collection
!

noIsNil: arg
	^arg = nil or: [ arg ~= nil ]
!

onlyReadOrWrittenTemporary
	| a |
	a := 1
!

precedence
	^self isArray ifFalse: [self block + 5 * 34] ifTrue: [self printString = 10]
!

refersToClass
    ^ TestClass

    "Modified: / 23-07-2012 / 13:28:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

release
	self printString
!

returnInEnsure
	[self error: 'asdf'] ensure: [^4]
!

returnsBooleanAndOther
	self isVariable ifTrue: [^false].
	self printString
!

returnsIfTrue
	^self isNil ifTrue: [4]
!

searchingLiteral
	^self printString = #a or: [#() = self printString | ( #() == self printString)]
!

sendsDifferentSuper
	super printString
!

sizeCheck
	self isEmpty
		ifFalse: [self do: 
					[:each | 
					]]
!

stringConcatenation
	| string |
	string := '' yourself.
	(1 to: 10) do: [:i | string := string , i printString].
	^string
!

tempVarOverridesInstVar
	| temporaryVariable |
	temporaryVariable := 4.
	^temporaryVariable
!

temporaryVariableCapitalization
	| Capital |
	Capital := 'Bern'.
	^ Capital
!

tempsReadBeforeWritten
	| temp |
	self isVariable ifTrue: [temp := 4].
	^temp
!

threeElementPoint
	^5 @ 5 + 6 @ 6
!

toDo
	1 to: self size do: [:i | (self at: i) printString]
!

toDoCollect
	| array |
	array := Array new: 10.
	1 to: 10 do: [:i | array at: i put: i * i].
	^array
!

toDoWithIncrement
	| counter |
	counter := 0.
	1 to: 10
		by: 2
		do: [:i | counter := counter + 2].
	^counter
!

uncommonMessageSend
	true false
!

unoptimizedAndOr
	^ (true and: [ false ]) and: [ true ]
!

usesAdd
	^(1 to: 10) asOrderedCollection addAll: (11 to: 20)
!

usesTrue
	^ True
!

variableAssignedLiteral
	temporaryVariable := #() 
!

whileTrue
	| i |
	i := 1.
	[i < self size] whileTrue: 
			[(self at: i) printString.
			i := i + 1]
!

yourselfNotUsed
	self
		printString;
		printString;
		yourself
! !

!SmalllintTests2 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintTests2.st,v 1.1 2012/07/23 12:31:52 vrany Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/lint/SmalllintTests2.st,v 1.1 2012/07/23 12:31:52 vrany Exp $'
! !
