'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 05:51:36'                     !

Object subclass:#RBSpellChecker
	instanceVariableNames:''
	classVariableNames:'Default'
	poolDictionaries:''
	category:'Refactory-Lint-Spelling'
!

!RBSpellChecker class methodsFor:'initialization'!

initialize
	self startUp.
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self
!

shutDown
	Default := nil
!

startUp
	Default := RBMacSpellChecker isSupported
		ifTrue: [ RBMacSpellChecker new ]
		ifFalse: [ RBInternalSpellChecker new ]
!

unload
	Smalltalk removeFromStartUpList: self.
	Smalltalk removeFromShutDownList: self
! !

!RBSpellChecker class methodsFor:'accessing'!

default
	^ Default
! !

!RBSpellChecker methodsFor:'private'!

validate: aString
	"Answer whether aString is correctly spelled."
	
	self subclassResponsibility
! !

!RBSpellChecker methodsFor:'public'!

check: anObject
        "Answer a collection of misspelled substrings. 
         anObject is either a string to be normalized and split into individual words, or a collection of already split words."
        
        anObject isString
                ifTrue: [ ^ (self normalize: anObject) reject: [ :each | self validate: each ] ].
        anObject isCollection
                ifTrue: [ ^ anObject gather: [ :each | self check: each ] ].
        ^ #()

    "Modified (comment): / 04-09-2011 / 20:23:42 / cg"
!

normalize: aString
	"Filter out non alphabetical characters, remove prefixes as commonly found in class names, split camel case expressions and filter out one character words."
	
	| result input output |
	result := Set new.
	input := aString readStream.
	output := WriteStream on: (String new: 128).
	[ input atEnd ] whileFalse: [
		[ input atEnd not and: [ input peek isAlphabetic or: [ input peek = $' ] ] ]
			whileTrue: [ output nextPut: input next ].
		output position = 0
			ifTrue: [ input next ]
			ifFalse: [
				| stream |
				stream := output contents readStream.
				[ stream atEnd ] whileFalse: [
					output reset; nextPut: stream next.
					[ stream atEnd not and: [ stream peek isLowercase or: [ stream peek = $' ] ] ]
						whileTrue: [ output nextPut: stream next ].
					output position > 1
						ifTrue: [ result add: output contents ] ] ].
		output reset ].
	^ result
! !


RBSpellChecker initialize!
