"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

ParseTreeRewriter subclass:#ParseTreeSourceRewriter
	instanceVariableNames:'replacements'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-ParseTree Matching'
!

!ParseTreeSourceRewriter class methodsFor:'documentation'!

documentation
"
    This is a parse tree rewriter which does NOT generate new code from scratch
    (i.e. it does not generate its source by recursively walking along the formattedCode methods),
    but instead inserts the new nodes code right into the original source code.
    This has two advantages:
        it does not destroy the user's formatting, indentation etc.
        it does not forget comments.
    I added this, because I constantly got mad at the rewriter reformatting my code for every tiny
    little variable rewrite operation.

    Caveats: 
        needs more intelligence when rewriting non-trivial code, such as keyword message sends.
        For now, it seems to only behave reasonable for very simple stuff, such as replacing a variable
        by another or by a simple unary send.

    [author:]
        Claus Gittinger
    [see also:]
        ParseTreeRewriter
"
! !

!ParseTreeSourceRewriter class methodsFor:'others'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/ParseTreeSourceRewriter.st,v 1.7 2012/05/10 23:02:33 vrany Exp $'
! !

!ParseTreeSourceRewriter methodsFor:'accessing'!

addReplacement:aReplacement
    replacements isNil ifTrue:[
	replacements := OrderedCollection new.
    ].
    replacements add:aReplacement
!

forgetReplacements
    replacements := nil 
! !

!ParseTreeSourceRewriter methodsFor:'queries'!

isRewritingOriginalSource
    ^ true.
! !

!ParseTreeSourceRewriter methodsFor:'replacing'!

executeReplacementsInSource:aSourceString
    "execute all replacements in a sourceString.
     Invoked after we have collected all replacements."

    |atomicReplacements newSource|

    "just in case that a replacement is a multi-word replacement;
     let each replacement add its sub-replacements add to a collection of
     atomic replacements. 
     (just think of a multi-part-keyword message being replaced)"

    atomicReplacements := OrderedCollection new.
    replacements ? #() do:[:eachReplacement |
        eachReplacement addIndividualReplacementsTo:atomicReplacements.
    ].

    "now, atomic replacements can be sorted by their characterPosition"

    atomicReplacements sort:[:a :b | a tokenPosition < b tokenPosition ].

    "there may not be any overlaps ..."

    newSource := aSourceString.
    atomicReplacements reverseDo:[:eachReplacement |
        newSource := eachReplacement executeIn:newSource
    ].

    ^ newSource

    "Modified: / 10-05-2012 / 13:21:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ParseTreeSourceRewriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/ParseTreeSourceRewriter.st,v 1.7 2012/05/10 23:02:33 vrany Exp $'
! !
