"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBAssignmentNode
	instanceVariableNames:'variable assignment value'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBAssignmentNode comment:'BRAssignmentNode is an AST node for assignment statements

Instance Variables:
	assignment	<Integer>	position of the :=
	value	<BRValueNode>	the value that we''re assigning
	variable	<BRVariableNode>	the variable being assigned

'
!


!RBAssignmentNode class methodsFor:'instance creation'!

variable: aVariableNode value: aValueNode 
	^self 
		variable: aVariableNode
		value: aValueNode
		position: nil
!

variable: aVariableNode value: aValueNode position: anInteger 
	^self new
		variable: aVariableNode
		value: aValueNode
		position: anInteger
! !

!RBAssignmentNode methodsFor:'accessing'!

children
	^Array with: value with: variable
!

precedence
	^5
!

startWithoutParentheses
	^variable start
!

stopWithoutParentheses
	^value stop
!

value
	^value
!

value: aValueNode 
	value := aValueNode.
	value parent: self
!

variable
	^variable
!

variable: varNode 
	variable := varNode.
	variable parent: self
! !

!RBAssignmentNode methodsFor:'comparing'!

= anObject 
	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	^self variable = anObject variable and: [self value = anObject value]
!

equalTo: anObject withMapping: aDictionary 
	self class = anObject class ifFalse: [^false].
	^(self variable equalTo: anObject variable withMapping: aDictionary)
		and: [self value equalTo: anObject value withMapping: aDictionary]
!

hash
	^self variable hash bitXor: self value hash
! !

!RBAssignmentNode methodsFor:'copying'!

postCopy
	super postCopy.
	variable := variable postCopy.
	value := value postCopy
! !

!RBAssignmentNode methodsFor:'initialization & release'!

variable: aVariableNode value: aValueNode position: anInteger 
	self variable: aVariableNode.
	self value: aValueNode.
	assignment := anInteger
! !

!RBAssignmentNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new) variable: (variable copyInContext: aDictionary);
		value: (value copyInContext: aDictionary);
		yourself
!

match: aNode inContext: aDictionary 
	aNode class == self class ifFalse: [^false].
	^(variable match: aNode variable inContext: aDictionary)
		and: [value match: aNode value inContext: aDictionary]
! !

!RBAssignmentNode methodsFor:'querying'!

bestNodeFor: anInterval 
	(self intersectsInterval: anInterval) ifFalse: [^nil].
	(self containedBy: anInterval) ifTrue: [^self].
	assignment isNil ifTrue: [^super bestNodeFor: anInterval].
	((anInterval first between: assignment and: assignment + 1) 
		or: [assignment between: anInterval first and: anInterval last]) 
			ifTrue: [^self].
	self children do: 
			[:each | 
			| node |
			node := each bestNodeFor: anInterval.
			node notNil ifTrue: [^node]]
!

firstLineNumber
    ^ variable lineNumber
!

lastLineNumber
    ^ value lastLineNumber
! !

!RBAssignmentNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	value == aNode ifTrue: [self value: anotherNode].
	variable == aNode ifTrue: [self variable: anotherNode]
! !

!RBAssignmentNode methodsFor:'testing'!

assigns: aVariableName 
	^variable name = aVariableName or: [value assigns: aVariableName]
!

directlyUses: aNode 
	^aNode = value ifTrue: [true] ifFalse: [self isDirectlyUsed]
!

isAssignment
	^true
!

uses: aNode 
	^aNode = value ifTrue: [true] ifFalse: [self isUsed]
! !

!RBAssignmentNode methodsFor:'visiting'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptAssignmentNode: self
! !

!RBAssignmentNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBAssignmentNode.st,v 1.11 2006/01/31 16:48:36 cg Exp $'
! !
