"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBBlockNode
	instanceVariableNames:'left right colons body arguments bar firstLineNumber
		lastLineNumber'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBBlockNode comment:'BRBlockNode is an AST node that represents a block "[...]".

Instance Variables:
	arguments	<SequenceableCollection of: BRVariableNode>	the arguments for the block
	bar	<Integer | nil>	position of the | after the arguments
	body	<BRSequenceNode>	the code inside the block
	colons	<SequenceableCollection of: Integer>	positions of each : before each argument
	left	<Integer>	position of [
	right	<Integer>	position of ]

'
!


!RBBlockNode class methodsFor:'instance creation'!

arguments: argNodes body: sequenceNode 
	^(self new)
		arguments: argNodes;
		body: sequenceNode;
		yourself
!

body: sequenceNode 
	^self arguments: #() body: sequenceNode
! !

!RBBlockNode methodsFor:'accessing'!

allArgumentVariables
	^(self argumentNames asOrderedCollection)
		addAll: super allArgumentVariables;
		yourself
!

allDefinedVariables
	^(self argumentNames asOrderedCollection) addAll: super allDefinedVariables;
		yourself
!

allVariablesOnScope
        | vars |
        vars := super allVariablesOnScope asOrderedCollection.
        vars addAll: self argumentNames.
        ^vars
!

argumentNames
	^self arguments collect: [:each | each name]
!

arguments
	^arguments
!

arguments: argCollection 
	arguments := argCollection.
	arguments do: [:each | each parent: self]
!

bar
	^bar
!

bar: anObject
	bar := anObject
!

blockVariables
	| vars |
	vars := super blockVariables asOrderedCollection.
	vars addAll: self argumentNames.
	^vars
!

body
	^body
!

body: stmtsNode 
	body := stmtsNode.
	body parent: self
!

children
	^self arguments copyWith: self body
!

colons: anObject
	colons := anObject
!

firstLineNumber
    ^ firstLineNumber
!

firstLineNumber:something
    firstLineNumber := something.
!

lastLineNumber
    ^ lastLineNumber
!

lastLineNumber:something
    lastLineNumber := something.
!

left
	^left
!

left: anObject
	left := anObject
!

numArgs
    ^ arguments size

    "Created: / 29-11-2011 / 11:13:30 / cg"
!

precedence
	^0
!

right
	^right
!

right: anObject
	right := anObject
!

startWithoutParentheses
	^left
!

stopWithoutParentheses
	^right
! !

!RBBlockNode methodsFor:'comparing'!

= anObject
        |myArgs otherArgs|

        self == anObject ifTrue: [^true].
        self class = anObject class ifFalse: [^false].
        self body = anObject body ifFalse: [^false].
        myArgs := self arguments.
        otherArgs := anObject arguments.
        myArgs size = otherArgs size ifFalse: [^false].
        1 to: myArgs size
                do: [:i | (myArgs at: i) = (otherArgs at: i) ifFalse: [^false]].
        ^true

    "Modified: / 29-11-2011 / 11:16:39 / cg"
!

equalTo: anObject withMapping: aDictionary 
        |myArgs otherArgs numArgs|

        self class = anObject class ifFalse: [^false].
        myArgs := self arguments.
        otherArgs := anObject arguments.
        (numArgs := myArgs size) = otherArgs size ifFalse: [^false].
        1 to: numArgs
                do: 
                        [:i | 
                        ((myArgs at: i) equalTo: (otherArgs at: i)
                                withMapping: aDictionary) ifFalse: [^false]].
        (self body equalTo: anObject body withMapping: aDictionary)
                ifFalse: [^false].
        myArgs do: [:each | aDictionary removeKey: each name ifAbsent:[] ].
        ^true

    "Modified: / 29-11-2011 / 11:16:10 / cg"
!

hash
	^self arguments hash bitXor: self body hash
! !

!RBBlockNode methodsFor:'copying'!

postCopy
	super postCopy.
	arguments := arguments collect: [:each | each copy].
	body := body copy
! !

!RBBlockNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new)
		arguments: (self copyList: arguments inContext: aDictionary);
		body: (body copyInContext: aDictionary);
		yourself
!

match: aNode inContext: aDictionary 
	aNode class == self class ifFalse: [^false].
	^(self matchList: arguments
		against: aNode arguments
		inContext: aDictionary)
			and: [body match: aNode body inContext: aDictionary]
! !

!RBBlockNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	body == aNode ifTrue: [self body: anotherNode].
	self arguments: (arguments 
				collect: [:each | each == aNode ifTrue: [anotherNode] ifFalse: [each]])
! !

!RBBlockNode methodsFor:'testing'!

defines: aName 
	^(arguments detect: [:each | each name = aName] ifNone: [nil]) notNil
!

directlyUses: aNode 
	^false
!

hasArguments
    ^ arguments notEmpty
!

hasStatements
        ^body hasStatements
!

isBlock
	^true
!

isImmediate
	^true
!

isLast: aNode 
	^body isLast: aNode
!

references: aVariableName 
	^body references: aVariableName
!

uses: aNode 
	aNode = body ifFalse: [^false].
	^parent isMessage
		ifTrue: [(#(#ifTrue:ifFalse: #ifTrue: #ifFalse: #ifFalse:ifTrue:) includes: parent selector) not or: [parent isUsed]]
		ifFalse: [self isUsed]
! !

!RBBlockNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptBlockNode: self
! !

!RBBlockNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBBlockNode.st,v 1.13 2011/11/29 10:17:15 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBBlockNode.st,v 1.13 2011/11/29 10:17:15 cg Exp $'
! !
