"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBCascadeNode
	instanceVariableNames:'messages semicolons'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBCascadeNode comment:'BRCascadeNode is an AST node for cascaded messages (e.g., "self print1 ; print2").

Instance Variables:
	messages        <SequenceableCollection of: BRMessageNode>      the messages 
	semicolons      <SequenceableCollection of: Integer>    positions of the ; between messages

'
!


!RBCascadeNode class methodsFor:'instance creation'!

messages: messageNodes 
	^self new messages: messageNodes
!

messages: messageNodes semicolons: integerCollection 
	^self new messages: messageNodes semicolons: integerCollection
! !

!RBCascadeNode methodsFor:'accessing'!

children
	^self messages
!

messages
	^messages
!

messages: messageNodeCollection 
	messages := messageNodeCollection.
	messages do: [:each | each parent: self]
!

precedence
	^4
!

startWithoutParentheses
	^messages first start
!

stopWithoutParentheses
	^messages last stop
! !

!RBCascadeNode methodsFor:'comparing'!

= anObject 
	|m1 m2|

	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	self messages size = anObject messages size ifFalse: [^false].
	1 to: self messages size
		do: [:i | m1 := (self messages at: i).
			  m2 := (anObject messages at: i).
			  m1 = m2 ifFalse: [^false]].
	^true
!

equalTo: anObject withMapping: aDictionary 
	self class = anObject class ifFalse: [^false].
	self messages size == anObject messages size ifFalse: [^false].
	1 to: self messages size
		do: 
			[:i | 
			((self messages at: i) equalTo: (anObject messages at: i)
				withMapping: aDictionary) ifFalse: [^false]].
	^true
!

hash
	^self messages hash
! !

!RBCascadeNode methodsFor:'copying'!

postCopy
	super postCopy.
	messages := messages collect: [:each | each copy]
! !

!RBCascadeNode methodsFor:'initialize-release'!

messages: messageNodes semicolons: integerCollection 
	self messages: messageNodes.
	semicolons := integerCollection
! !

!RBCascadeNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new) messages: (self copyList: messages inContext: aDictionary);
		yourself
!

match: aNode inContext: aDictionary 
	aNode class == self class ifFalse: [^false].
	^self matchList: messages
		against: aNode messages
		inContext: aDictionary
! !

!RBCascadeNode methodsFor:'querying'!

bestNodeFor: anInterval 
	| selectedChildren |
	(self intersectsInterval: anInterval) ifFalse: [^nil].
	(self containedBy: anInterval) ifTrue: [^self].
	messages 
		reverseDo: [:each | (each containedBy: anInterval) ifTrue: [^each]].
	selectedChildren := (messages 
				collect: [:each | each bestNodeFor: anInterval]) 
					reject: [:each | each isNil].
	^selectedChildren detect: [:each | true] ifNone: [nil]
!

firstLineNumber
    ^ messages first firstLineNumber.
!

lastLineNumber
    ^ messages last lastLineNumber.
!

whichNodeIsContainedBy: anInterval 
	| selectedChildren |
	(self intersectsInterval: anInterval) ifFalse: [^nil].
	(self containedBy: anInterval) ifTrue: [^self].
	messages 
		reverseDo: [:each | (each containedBy: anInterval) ifTrue: [^each]].
	selectedChildren := (messages 
				collect: [:each | each whichNodeIsContainedBy: anInterval]) 
					reject: [:each | each isNil].
	^selectedChildren detect: [:each | true] ifNone: [nil]
! !

!RBCascadeNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	self messages: (messages 
				collect: [:each | each == aNode ifTrue: [anotherNode] ifFalse: [each]])
! !

!RBCascadeNode methodsFor:'testing'!

directlyUses: aNode 
	^messages last = aNode and: [self isDirectlyUsed]
!

isCascade
	^true
!

uses: aNode 
	^messages last = aNode and: [self isUsed]
! !

!RBCascadeNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptCascadeNode: self
! !

!RBCascadeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBCascadeNode.st,v 1.10 2006/01/31 16:48:31 cg Exp $'
! !
