"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBLiteralNode
	instanceVariableNames:'token'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBLiteralNode comment:'BRLiteralNode is an AST node that represents literals (e.g., #foo, #(1 2 3), true, etc.).

Instance Variables:
	token   <BRLiteralToken>        the token that contains the literal value as well as its source positions

'
!


!RBLiteralNode class methodsFor:'instance creation'!

literalToken: aLiteralToken 
	^self new literalToken: aLiteralToken
!

value: aValue 
        ^self literalToken: (RBLiteralToken value: aValue)
! !

!RBLiteralNode methodsFor:'accessing'!

generatedCode
    ^ token realValue storeString
!

precedence
	^0
!

startWithoutParentheses
	^token start
!

stopWithoutParentheses
	^token stop
!

token
	^token
!

value
	^token realValue
! !

!RBLiteralNode methodsFor:'comparing'!

= anObject 
        |myValue otherValue|

        self == anObject ifTrue: [^true].
        self class == anObject class ifFalse: [^false].
        myValue := self value.
        otherValue := anObject value.
        myValue class == otherValue class ifFalse: [^false].

        myValue class == Array ifTrue:[
            myValue size == otherValue size ifFalse: [^false].
            myValue with:otherValue do:[:elHere :elThere |
                (elHere isLimitedPrecisionReal and:[elThere isLimitedPrecisionReal]) ifTrue:[
                    " avoid precision bug with floats "
                    (elHere printString = elThere printString) ifFalse:[^ false]
                ] ifFalse:[
                    (elHere = elThere) ifFalse:[^ false]
                ].
            ].
            ^ true.
        ].
        (myValue isLimitedPrecisionReal and:[otherValue isLimitedPrecisionReal]) ifTrue:[
            " avoid precision bug with floats "
            ^ (myValue printString = otherValue printString) 
        ].
        ^myValue = otherValue
!

hash
	^self value hash
! !

!RBLiteralNode methodsFor:'initialization & release'!

literalToken: aLiteralToken 
	token := aLiteralToken
! !

!RBLiteralNode methodsFor:'matching'!

copyInContext: aDictionary
	^self class literalToken: token removePositions
!

match: aNode inContext: aDictionary
        "/ ST/X: patternStringLiteral 
        token isPatternLiteral ifTrue:[
            (super match: aNode inContext: aDictionary) ifTrue:[^ true].

            (aNode isLiteral and:[ aNode value isString and:[aNode value isSymbol not] ]) ifTrue:[
                ^ (aNode value) 
                    perform:#matchesRegex: with:(self value)
                    ifNotUnderstood:[
                        self value = '.*' ifTrue:[^ true].

                        "/ mhm - the regex package seems to be not loaded...
                        "/ Smalltalk loadPackage:'stx:goodies/regex'
                        self error:'the regex package is missing'.
                    ]
            ].
            ^ false.
        ].
        ^ super match: aNode inContext: aDictionary

    "Modified: / 05-10-2011 / 11:59:49 / cg"
! !

!RBLiteralNode methodsFor:'special queries'!

addUsedLiteralsTo:aCollection
    aCollection add:token value

    "Created: / 12-10-2006 / 21:26:02 / cg"
!

addUsedSymbolsTo:aCollection
    |value|

    (value := token value) isSymbol ifTrue:[
        aCollection add:value
    ].
! !

!RBLiteralNode methodsFor:'testing'!

isImmediate
	^true
!

isLiteral
	^true
!

isLiteralArray
        ^ token isLiteralArray
!

isLiteralByteArray
        ^ token isLiteralByteArray
!

isLiteralNumber
        ^ token isLiteralNumber
!

isLiteralString
        ^ token isLiteralString

    "Created: / 28-12-2011 / 16:05:08 / cg"
! !

!RBLiteralNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptLiteralNode: self
! !

!RBLiteralNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBLiteralNode.st,v 1.27 2011/12/28 15:07:37 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBLiteralNode.st,v 1.27 2011/12/28 15:07:37 cg Exp $'
! !
