"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBMessageNode
	instanceVariableNames:'receiver selector selectorParts arguments'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBMessageNode comment:'BRMessageNode is an AST node that represents a message send.

Instance Variables:
	arguments       <SequenceableCollection of: BRValueNode>        our argument nodes
	receiver        <BRValueNode>   the receiver''s node
	selector        <Symbol | nil>  the selector we''re sending (cached)
	selectorParts   <SequenceableCollection of: BRValueToken>       the tokens for each keyword

'
!


!RBMessageNode class methodsFor:'instance creation'!

receiver: aValueNode selector: aSymbol 
	^self 
		receiver: aValueNode
		selector: aSymbol
		arguments: #()
!

receiver: aValueNode selector: aSymbol arguments: valueNodes 
	^(self new)
		receiver: aValueNode;
		arguments: valueNodes;
		selector: aSymbol;
		yourself
!

receiver: aValueNode selectorParts: keywordTokens arguments: valueNodes 
	^((keywordTokens detect: [:each | each isPatternVariable] ifNone: [nil]) 
		notNil ifTrue: [RBPatternMessageNode] ifFalse: [RBMessageNode]) 
		new 
			receiver: aValueNode
			selectorParts: keywordTokens
			arguments: valueNodes
! !

!RBMessageNode class methodsFor:'others'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBMessageNode.st,v 1.17 2011/07/04 12:49:15 cg Exp $'
! !

!RBMessageNode methodsFor:'accessing'!

arguments
	^arguments isNil
		ifTrue: [#()]
		ifFalse: [arguments]
!

arguments: argCollection 
	arguments := argCollection.
	arguments do: [:each | each parent: self]
!

children
        ^(OrderedCollection with: receiver) 
            addAll: arguments;
            yourself
!

leftMostReceiver
    ^ receiver leftMostReceiver
!

precedence
	^self isUnary ifTrue: [1] ifFalse: [self isKeyword ifTrue: [3] ifFalse: [2]]
!

receiver
	^receiver
!

receiver: aValueNode 
	receiver := aValueNode.
	receiver parent: self
!

selector
	^selector isNil
		ifTrue: [selector := self buildSelector]
		ifFalse: [selector]
!

selector: aSelector 
	| keywords numArgs |
	keywords := aSelector keywords.
	numArgs := aSelector numArgs.
	numArgs == arguments size 
		ifFalse: 
			[self 
				error: 'Attempting to assign selector with wrong number of arguments.'].
	selectorParts := numArgs == 0 
				ifTrue: [Array with: (RBIdentifierToken value: keywords first start: nil)]
				ifFalse: 
					[keywords first last == $: 
						ifTrue: [keywords collect: [:each | RBKeywordToken value: each start: nil]]
						ifFalse: [Array with: (RBBinarySelectorToken value: aSelector start: nil)]].
	selector := aSelector
!

sentMessages
	^(super sentMessages)
		add: self selector;
		yourself
!

startWithoutParentheses
	^receiver start
!

stopWithoutParentheses
	^arguments isEmpty 
		ifTrue: [selectorParts first stop]
		ifFalse: [arguments last stop]
! !

!RBMessageNode methodsFor:'comparing'!

= anObject 
        |a1 a2|

        self == anObject ifTrue: [^true].
        self class = anObject class ifFalse: [^false].
        (receiver = anObject receiver 
                and: [self selector = anObject selector]) ifFalse: [^false].
        1 to: arguments size
                do: [:i | 
                            a1 := (arguments at: i).
                            a2 := (anObject arguments at: i).
                            a1 = a2 ifFalse: [^false]].
        ^true
!

equalTo: anObject withMapping: aDictionary 
        self class = anObject class ifFalse: [^false].
        ((receiver equalTo: anObject receiver withMapping: aDictionary)
                and: [self selector = anObject selector]) ifFalse: [^false].
        1 to: arguments size
                do: 
                        [:i | 
                        ((arguments at: i) equalTo: (anObject arguments at: i)
                                withMapping: aDictionary) ifFalse: [^false]].
        ^true
!

hash
        ^(receiver hash bitXor: self selector hash)
                bitXor: (arguments isEmpty ifTrue: [0] ifFalse: [arguments first hash])
! !

!RBMessageNode methodsFor:'copying'!

postCopy
	super postCopy.
	receiver := receiver copy.
	arguments := arguments collect: [:each | each copy]
! !

!RBMessageNode methodsFor:'initialization & release'!

receiver: aValueNode selectorParts: keywordTokens arguments: valueNodes 
	self receiver: aValueNode.
	selectorParts := keywordTokens.
	self arguments: valueNodes
! !

!RBMessageNode methodsFor:'libtool3 support'!

highlightInCodeView: aCodeView

    ^aCodeView highlightMessageNode: self

    "Created: / 18-02-2008 / 17:51:11 / janfrog"
!

leftClickMenuInCodeView: aCodeView 
    ^ aCodeView leftClickMenuForMessageNode: self.

    "Created: / 18-02-2008 / 19:04:45 / janfrog"
!

middleClickMenuInCodeView: aCodeView 
    ^ aCodeView middleClickMenuForMessageNode: self.

    "Created: / 18-02-2008 / 19:04:58 / janfrog"
!

rightClickMenuInCodeView: aCodeView 
    ^ aCodeView rightClickMenuForMessageNode: self.

    "Created: / 18-02-2008 / 19:05:18 / janfrog"
! !

!RBMessageNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new) receiver: (receiver copyInContext: aDictionary);
		selectorParts: (selectorParts collect: [:each | each removePositions]);
		arguments: (arguments collect: [:each | each copyInContext: aDictionary]);
		yourself
!

match: aNode inContext: aDictionary 
	aNode class == self class ifFalse: [^false].
	self selector == aNode selector ifFalse: [^false].
	(receiver match: aNode receiver inContext: aDictionary) ifFalse: [^false].
	1 to: arguments size
		do: 
			[:i | 
			((arguments at: i) match: (aNode arguments at: i) inContext: aDictionary)
				ifFalse: [^false]].
	^true
! !

!RBMessageNode methodsFor:'private'!

buildSelector
	| selectorStream |
	selectorStream := WriteStream on: (String new: 50).
	selectorParts do: [:each | selectorStream nextPutAll: each value].
	^selectorStream contents asSymbol
!

selectorParts
	^selectorParts
!

selectorParts: tokenCollection 
	selectorParts := tokenCollection
! !

!RBMessageNode methodsFor:'querying'!

bestNodeFor: anInterval 
	(self intersectsInterval: anInterval) ifFalse: [^nil].
	(self containedBy: anInterval) ifTrue: [^self].
	selectorParts do: 
			[:each | 
			((anInterval first between: each start and: each stop) 
				or: [each start between: anInterval first and: anInterval last]) 
					ifTrue: [^self]].
	self children do: 
			[:each | 
			| node |
			node := each bestNodeFor: anInterval.
			node notNil ifTrue: [^node]]
!

firstLineNumber
    ^ receiver firstLineNumber
!

lastLineNumber
    arguments notEmptyOrNil ifTrue:[
        ^ arguments last lastLineNumber
    ].
    ^ selectorParts last lineNumber.
"/    ^ lineNumber.
! !

!RBMessageNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	"If we're inside a cascade node and are changing the receiver, change all the receivers"

	receiver == aNode 
		ifTrue: 
			[self receiver: anotherNode.
			(parent notNil and: [parent isCascade]) 
				ifTrue: [parent messages do: [:each | each receiver: anotherNode]]].
	self arguments: (arguments 
				collect: [:each | each == aNode ifTrue: [anotherNode] ifFalse: [each]])
! !

!RBMessageNode methodsFor:'testing'!

isBinary
	^(self isUnary or: [self isKeyword]) not
!

isCascaded
	^parent notNil and: [parent isCascade]
!

isFirstCascaded
	^self isCascaded and: [parent messages first == self]
!

isKeyword
	^selectorParts first value last == $:
!

isMessage
	^true
!

isUnary
	^arguments isEmpty
!

lastIsReturn
	^(#(#ifTrue:ifFalse: #ifFalse:ifTrue:) includes: self selector) and: 
			[arguments first isBlock and: 
					[arguments first body lastIsReturn
						and: [arguments last isBlock and: [arguments last body lastIsReturn]]]]
! !

!RBMessageNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptMessageNode: self
! !

!RBMessageNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBMessageNode.st,v 1.17 2011/07/04 12:49:15 cg Exp $'
! !
