"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBStatementNode subclass:#RBReturnNode
	instanceVariableNames:'return value'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBReturnNode comment:'BRReturnNode is an AST node that represents a return expression.

Instance Variables:
	return	<Integer>	the position of the ^ character
	value	<BRValueNode>	the value that is being returned

'
!


!RBReturnNode class methodsFor:'instance creation'!

return: returnInteger value: aValueNode 
	^self new return: returnInteger value: aValueNode
!

value: aNode
	^self return: nil value: aNode
! !

!RBReturnNode methodsFor:'accessing'!

children
	^Array with: value
!

start
	^return
!

stop
	^value stop
!

value
	^value
!

value: valueNode 
	value := valueNode.
	value parent: self
! !

!RBReturnNode methodsFor:'comparing'!

= anObject 
	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	^self value = anObject value
!

equalTo: anObject withMapping: aDictionary 
	self class = anObject class ifFalse: [^false].
	^self value equalTo: anObject value withMapping: aDictionary
!

hash
	^self value hash
! !

!RBReturnNode methodsFor:'copying'!

postCopy
	super postCopy.
	value := value copy
! !

!RBReturnNode methodsFor:'initialization & release'!

return: returnInteger value: aValueNode 
	return := returnInteger.
	self value: aValueNode
! !

!RBReturnNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new) value: (value copyInContext: aDictionary); yourself
!

match: aNode inContext: aDictionary 
	aNode class == self class ifFalse: [^false].
	^value match: aNode value inContext: aDictionary
! !

!RBReturnNode methodsFor:'queries'!

lastLineNumber
    ^ value lastLineNumber
! !

!RBReturnNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	value == aNode ifTrue: [self value: anotherNode]
! !

!RBReturnNode methodsFor:'testing'!

containsReturn
	^true
!

isReturn
	^true
! !

!RBReturnNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptReturnNode: self
! !

!RBReturnNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBReturnNode.st,v 1.10 2006/01/31 16:48:29 cg Exp $'
! !
