"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBValueNode subclass:#RBVariableNode
	instanceVariableNames:'token'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBVariableNode comment:'BRVariableNode is an AST node that represent a variable (global, inst var, temp, etc.).

Instance Variables:
	token	<BRValueToken>	the token that contains our name and position

'
!


!RBVariableNode class methodsFor:'instance creation'!

identifierToken: anIdentifierToken 
	^(anIdentifierToken isPatternVariable 
		ifTrue: [RBPatternVariableNode]
		ifFalse: [RBVariableNode]) new 
		identifierToken: anIdentifierToken
!

named: aString 
        ^self identifierToken: (RBIdentifierToken value: aString start: 0)
! !

!RBVariableNode class methodsFor:'others'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBVariableNode.st,v 1.19 2012/04/02 10:27:57 vrany Exp $'
! !

!RBVariableNode methodsFor:'accessing'!

addReferencedVariablesTo:aCollection
    aCollection add:self
!

name
	^token value
!

precedence
	^0
!

startWithoutParentheses
	^token start
!

stopWithoutParentheses
	^token stop
!

tokenPosition
        ^token position
! !

!RBVariableNode methodsFor:'comparing'!

= anObject 
	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	^self name = anObject name
!

equalTo: anObject withMapping: aDictionary 
        |otherName|
        self class = anObject class ifFalse: [^false].
        otherName := anObject name.
        ^(aDictionary at: self name ifAbsentPut: [otherName]) = otherName
!

hash
	^self name hash
! !

!RBVariableNode methodsFor:'initialization & release'!

identifierToken: anIdentifierToken 
	token := anIdentifierToken
! !

!RBVariableNode methodsFor:'libtool3 support'!

highlightInCodeView: aCodeView

    ^aCodeView highlightVariableNode: self

    "Created: / 18-02-2008 / 17:51:44 / janfrog"
!

leftClickMenuInCodeView: aCodeView 
    ^ aCodeView leftClickMenuForVariableNode: self.

    "Created: / 01-09-2009 / 08:41:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rightClickMenuInCodeView: aCodeView 
    ^ aCodeView rightClickMenuForVariableNode: self.

    "Created: / 18-02-2008 / 21:05:04 / janfrog"
! !

!RBVariableNode methodsFor:'matching'!

copyInContext: aDictionary 
	^self class identifierToken: token removePositions
! !

!RBVariableNode methodsFor:'testing'!

isImmediate
	^true
!

isSelf
    ^ self name = 'self'

    "Created: / 17-07-2011 / 10:28:56 / cg"
!

isSuper
    ^ self name = 'super'

    "Created: / 17-07-2011 / 10:29:29 / cg"
!

isVariable
	^true
!

isVariableNotInLeftHandSideOfAssignment
    "Returns true iff receiver is variable node but not
     in left-hand side of assignment"    
    ^self parent isAssignment not 
        or:[self parent variable ~~ self]

    "Created: / 28-03-2012 / 17:15:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isWrite
	^ self parent notNil and: [ self parent isAssignment and: [ self parent variable == self ] ]
!

references: aVariableName 
	^self name = aVariableName
! !

!RBVariableNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptVariableNode: self
! !

!RBVariableNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBVariableNode.st,v 1.19 2012/04/02 10:27:57 vrany Exp $'
! !
