"{ Package: 'stx:goodies/refactoryBrowser/refactoring' }"

ChangeMethodNameRefactoring subclass:#RenameMethodRefactoring
	instanceVariableNames:'hasPermutedArguments suppressRewriteOfSenders
		rewriteLocalSendersOnly'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Refactorings'
!


!RenameMethodRefactoring class methodsFor:'instance creation'!

model: aRBSmalltalk renameMethod: aSelector in: aClass to: newSelector permuation: aMap 
	^(self new)
		model: aRBSmalltalk;
		renameMethod: aSelector
			in: aClass
			to: newSelector
			permuation: aMap;
		yourself
! !

!RenameMethodRefactoring methodsFor:'performing'!

renameSymbols
	| replacer builder |
	self hasPermutedArguments ifTrue: [^self].
	builder := RefactoryBuilder new.
	replacer := ParseTreeRewriter replaceLiteral: oldSelector with: newSelector.
	self 
		convertAllClassesSelect: [:aClass | aClass whichSelectorsReferTo: oldSelector]
		using: replacer
		notifying: builder.
	builder doIt: 'Renaming symbols'
! !

!RenameMethodRefactoring methodsFor:'preconditions'!

myConditions
        ^RBCondition 
            withBlock: [oldSelector numArgs = newSelector numArgs]
            errorString: newSelector printString 
                                , ' doesn''t have the correct number (' , oldSelector numArgs printString , ') of arguments.'
!

preconditions
    | newCondition |

    newSelector ~= oldSelector ifTrue:[
        ^ super preconditions
    ].

    newCondition := (RBCondition 
                            withBlock: [ newSelector = oldSelector ]
                            errorString: 'The selectors are <1?:not >equivalent') 
                    & (RBCondition 
                            withBlock: [permutation asArray ~= (1 to: oldSelector numArgs) asArray]
                            errorString: 'The arguments are <1?:not >permuted').
    ^ newCondition | super preconditions
!

rewriteLocalSendersOnly
    ^ rewriteLocalSendersOnly

    "Created: / 28-02-2007 / 21:22:58 / cg"
!

rewriteLocalSendersOnly:aBoolean
    rewriteLocalSendersOnly := aBoolean

    "Created: / 28-02-2007 / 21:22:49 / cg"
!

suppressRewriteOfSenders
    ^ suppressRewriteOfSenders ? false
!

suppressRewriteOfSenders:aBoolean
    suppressRewriteOfSenders := aBoolean
! !

!RenameMethodRefactoring methodsFor:'printing'!

changeString
    ^ 'Rename %1 to %2 in %3' bindWith:oldSelector with:newSelector with:class printString
!

storeOn: aStream 
	aStream nextPut: $(.
	self class storeOn: aStream.
	aStream
		nextPutAll: ' renameMethod: #';
		nextPutAll: oldSelector;
		nextPutAll: ' in: '.
	class storeOn: aStream.
	aStream
		nextPutAll: ' to: #';
		nextPutAll: newSelector;
		nextPutAll: ' permuation: '.
	permutation storeOn: aStream.
	aStream nextPut: $)
! !

!RenameMethodRefactoring methodsFor:'testing'!

hasPermutedArguments
	^hasPermutedArguments isNil 
		ifTrue: [hasPermutedArguments := super hasPermutedArguments]
		ifFalse: [hasPermutedArguments]
!

implementorsCanBePrimitives
	^self hasPermutedArguments not
! !

!RenameMethodRefactoring methodsFor:'transforming'!

parseTreeRewriter
	| rewriteRule oldString newString |
	oldString := self buildSelectorString: oldSelector.
	newString := self buildSelectorString: newSelector
				withPermuteMap: permutation.
	rewriteRule := self hasPermutedArguments 
				ifTrue: [ParseTreeRewriter new]
				ifFalse: [ParseTreeRewriter replaceLiteral: oldSelector with: newSelector].
	rewriteRule replace: '``@object ' , oldString
		with: '``@object ' , newString.
	^rewriteRule
! !

!RenameMethodRefactoring class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/refactoring/RenameMethodRefactoring.st,v 1.14 2007/02/28 20:23:36 cg Exp $'
! !
