"{ Package: 'stx:goodies/roelTyper' }"

"{ NameSpace: RoelTyper }"

AbstractType subclass:#ExtractedType
	instanceVariableNames:'interface assignments elementsInterface extractedTypes
		elementTypes'
	classVariableNames:''
	poolDictionaries:''
	category:'RoelTyper-Core'
!


!ExtractedType class methodsFor:'instance creation'!

forInstvar: iv inClass: aClass interface: selectorCollection assignments: classCollection elementsInterface: aCollection

	^(self forInstvar: iv inClass: aClass) interface: selectorCollection
		assignments: classCollection elementsInterface: aCollection
! !

!ExtractedType class methodsFor:'private'!

mergerClass
	"Still experimenting with different mergers. So for the moment I uncomment things to experiment with."

"       ^ConcreteMerger"
"       ^AbstractMerger"
	^AssignmentsFirstMerger
! !

!ExtractedType methodsFor:'accessing'!

elementTypes

	^elementTypes ifNil: [elementTypes := self mergeElementTypes]
		ifNotNil: [elementTypes]
!

types

	^extractedTypes ifNil: [extractedTypes := self mergeTypes]
		ifNotNil: [extractedTypes]
! !

!ExtractedType methodsFor:'initialize-release'!

initialize

	super initialize.
	self interface: IdentitySet new assignments: OrderedCollection new elementsInterface: IdentitySet new
! !

!ExtractedType methodsFor:'printing'!

printInterfaceAndAssigmentsOn: aStream
	aStream nextPutAll: 'Sends: {'.
	self interface do: [:symbol | aStream print: symbol]
		separatedBy: [aStream space].
	aStream
		nextPutAll: '}';
		cr;
		nextPutAll: 'Assignments: {'.
	self assignments do: [:symbol | aStream print: symbol]
		separatedBy: [aStream space].
	aStream
		nextPutAll: '}';
		cr
!

printOn: aStream

	aStream nextPutAll: 'ExtractedType: '.
	self basicExtractedTypes
		ifNil: [self printInterfaceAndAssigmentsOn: aStream]
		ifNotNil: [self printTypesOn: aStream]
! !

!ExtractedType methodsFor:'private'!

cleanedAssignments

	| assigns |
	assigns := self assignments copy.
	((assigns includes: True) and: [assigns includes: False])
		ifTrue:
			[assigns
				remove: True;
				remove: False;
				add: Boolean].
	^assigns
!

foldInterfaceTypes: interfaceClasses withAssignmentTypes: assignmentClasses

	"       ^interfaceClasses asOrderedCollection"                  "Only interface types"
	"       ^assignmentClasses "                                                    "Only assignment types"

		^self class mergerClass
				interfaceTypes: interfaceClasses assignmentTypes: assignmentClasses
!

mergeElementTypes

	^self
		foldInterfaceTypes: (self rootsUnderstanding: self elementsInterface)
		withAssignmentTypes: Set new
!

mergeTypes
	^self foldInterfaceTypes: (self rootsUnderstanding: self interface)
		withAssignmentTypes: self cleanedAssignments
!

rootsUnderstanding: selectors
	"When the set of selectors is not empty, answer a set of the highest possible classes in the system that implement all the selectors.
	  When the set of selectors is empty, return the empty set."


	| initialClasses nextClasses traverseStack next prototypeSet |
	selectors isEmpty ifTrue: [^IdentitySet new].
	prototypeSet := IdentitySet new: 20.
	nextClasses := prototypeSet copy add: Object; yourself.
	selectors do:
			[:selector |
			initialClasses := nextClasses.
			nextClasses := prototypeSet copy.
			initialClasses do:
					[:initialClass |
					(initialClass canUnderstand: selector)
						ifTrue: [nextClasses add: initialClass]
						ifFalse:
							[traverseStack := OrderedCollection with: initialClass.
							[traverseStack isEmpty] whileFalse:
									[next := traverseStack removeFirst.
									 next nonMetaSubclassesDo: [:subcl |
											(subcl includesSelector: selector)
												ifTrue: [nextClasses add: subcl]
												ifFalse: [traverseStack add: subcl]]]]]].
	^nextClasses
! !

!ExtractedType methodsFor:'private-accessing'!

addAssignment: anObject

	(self assignments includes: anObject) ifFalse: [self assignments add: anObject]
!

addElementSend: selector
	self elementsInterface add: selector
!

addSend: anObject
	self interface add: anObject
!

assignments
	^assignments
!

basicExtractedTypes

	^extractedTypes
!

elementsInterface
	^ elementsInterface
!

interface
	^interface
!

interface: selectorCollection assignments: classCollection elementsInterface: aSelectorCollection
	interface := selectorCollection.
	assignments := classCollection.
	elementsInterface := aSelectorCollection
! !

!ExtractedType methodsFor:'testing'!

hasEmptyAssignments
	^self assignments isEmpty
!

hasEmptyInterface
	^self interface isEmpty
!

isEmpty
	^self hasEmptyAssignments and: [self hasEmptyInterface]
!

isExtractedResult

	^true
! !

!ExtractedType class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/roelTyper/RoelTyper__ExtractedType.st,v 1.2 2009/10/02 14:02:47 cg Exp $'
! !
