"{ Package: 'stx:goodies/roelTyper' }"

"{ NameSpace: RoelTyper }"

InstructionClient subclass:#InstvarInterfaceExtractor
	instanceVariableNames:'stack copied initialStack method saveStacks input collector
		instVarOfElementType'
	classVariableNames:''
	poolDictionaries:''
	category:'RoelTyper-Core'
!

InstvarInterfaceExtractor comment:'I am responsible for extracting the messages sent to an instance variable and the assignments done to an instance variable. I am supposed to work together with a TypeCollector.

Instance Variables:
	collector       <TypeCollector> description of collector
	copied  <Array> description of copied
	initialStack    <Collection | KeyBindingsList | MethodFilterAnd | MethodFilterOr>       description of initialStack
	input   <InstructionStream>     description of input
	method  <CompiledInitializer | CompiledMethod>  description of method
	saveStacks      <Dictionary>    description of saveStacks
	stack   <(OrderedCollection of: Object)>        description of stack'
!


!InstvarInterfaceExtractor methodsFor:'extracting'!

extractInterfacesFrom: m addTo: aTypeCollector requestElementTypeFor: instVar

	method := m.
	saveStacks := Dictionary new.
	stack := OrderedCollection new.
	instVarOfElementType := instVar.
	instVarOfElementType ~~ nil
		ifTrue:
			[method numArgs timesRepeat:[stack add: #arg].
			"aTypeCollector resetCurrentElementType."
			method numTemps - method numArgs timesRepeat: [stack add: #temp]]
		ifFalse: [method numTemps timesRepeat: [stack add: #temp]].
	initialStack := stack copy.
	collector := aTypeCollector.
	input := InstructionStream on: method.
	[input atEnd]
		whileFalse:
			[self reloadStack.
			input interpretNextInstructionFor: self]
! !

!InstvarInterfaceExtractor methodsFor:'opcodes-control'!

send: selector numArgs: na

	| receiver args |
	args := stack removeLast: na.
	receiver := (stack removeLast: 1) first.
	(receiver isInteger) ifTrue: [collector addSend: selector to: receiver].
	((instVarOfElementType ~~ nil) and:[receiver == #arg]) ifTrue:[collector addElementSend: selector to: instVarOfElementType].
	stack add: (collector
				pushSendOf: selector
				to: receiver
				args: args)
! !

!InstvarInterfaceExtractor methodsFor:'opcodes-data movement'!

dupFirst

	stack add: stack last.
!

pop

	stack removeLast.
!

pushConstant: value

	value class == BlockClosure
		ifTrue: [self readBlock: value method copied: 0]
		ifFalse: [stack addLast: value class]
!

pushContext

	stack add: #context.
!

pushInst: index

	stack add: index
!

pushReceiver

	stack addLast: #self.
!

pushStatic: assoc
	"assoc can be an association OR a variable binding. I just push the complete association, since it does not interest me for the moment."

	stack addLast: assoc
! !

!InstvarInterfaceExtractor methodsFor:'private'!

copied: list

	copied := list
!

input

	^input
!

method: aMethod

	method := aMethod.
	copied := #()
!

readBlock: block copied: count

	| newCopied |
	newCopied := stack removeLast: count.
	stack add: #block.
	((stack last: 2) first isInteger and:[(#(#do: #collect: #select: #detect:) includes: input peekForSelector)])
		ifTrue: [
				^(self class new)
				copied: newCopied;
				extractInterfacesFrom: block addTo: collector
				requestElementTypeFor: (stack last: 2) first ]
		ifFalse:
			[^(self class new)
				copied: newCopied;
				extractInterfacesFrom: block addTo: collector
				requestElementTypeFor: nil ].
!

reloadStack

	stack isNil
		ifTrue: [stack :=  self saveStacks at: self input pc ifAbsent: [initialStack copy]. ^self].
	stack := self saveStacks at: self input pc ifAbsent: [stack]
!

saveStacks

	saveStacks ifNil: [saveStacks := Dictionary new].
	^saveStacks
! !

!InstvarInterfaceExtractor class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/roelTyper/RoelTyper__InstvarInterfaceExtractor.st,v 1.3 2009/10/02 14:02:47 cg Exp $'
! !
