"{ Package: 'stx:goodies/roelTyper' }"

"{ NameSpace: RoelTyper }"

Object subclass:#TypeCollector
	instanceVariableNames:'theClass instVars typingResults currentElementType'
	classVariableNames:''
	poolDictionaries:''
	category:'RoelTyper-Core'
!

TypeCollector comment:'I collect and assemble the types sent to instance variables for a certain class (I use the InterfaceExtractor for that), And use them to create and return TypingResults.

Instance Variables:
	assignments     <Object>        description of assignments
	instVarOffset   <Object>        description of instVarOffset
	instVars        <(Palette of: (ExceptionSet | GenericException | SequenceableCollection | Signal)) | (SequenceableCollection of: (ExceptionSet | GenericException | SequenceableCollection | Signal))>  description of instVars
	sends   <Object>        description of sends
	theClass        <ClassDescription>      description of theClass'
!


!TypeCollector class methodsFor:'instance creation'!

new
	"Override new to return either a VWTypeCollector or a SqueakTypeCollector"

	^self newForPlatform
!

newForPlatform
	"Return either a VWTypeCollector or a SqueakTypeCollector, depending on the platform used."

	| versionString |
	versionString := self versionString.
	('*Squeak*' match: versionString)  ifTrue: [^SqueakTypeCollector basicNew].
	('*VisualWorks*' match: versionString)  ifTrue: [^VWTypeCollector basicNew].
	^self error: 'Unsupported Platform!!'
!

onClass: aClass

	^self new onClass: aClass
!

typeInstvar: var ofClass: aClass
	"self typeInstvar: #x ofClass: Point "

	^(self new typeInstvarsOfClass: aClass) at: var
!

typeInstvarsOfClass: aClass
	"self typeInstvarsOfClass: Point"

	^self new typeInstvarsOfClass: aClass
!

versionString
	"Take the removal of the standard version method on Smalltalk into account for Squeak :-( "

	^(Smalltalk at: #SystemVersion ifAbsent: [^Smalltalk version]) current version
! !

!TypeCollector methodsFor:'accessing'!

absoluteIndexForVar: iVar
	"Note: works for Squeak and VisualWorks. If it would be only for VW, you could just use instVarIndexFor: instead."

	^instVars indexOf: iVar
!

assignmentsTo: instVarName

	^(self typingResultsFor: instVarName) assignments
!

currentElementType
	^ currentElementType
!

packagedResults

	| results |
	results := IdentityDictionary new: self typingResults size.

	instVars size - theClass instVarNames size + 1 to: instVars size do: [:index |
		results at: (instVars at: index) put: (self typingResults at: index)].
	^results
!

sendsTo: instVarName
	"       ^self sends at: (self indexForVar: instVarName)"

	^(self typingResultsFor: instVarName) interface
!

typingResults

	^typingResults
! !

!TypeCollector methodsFor:'adding'!

addAssignment: value to: index

	self withTranslatedIndex: index do: [:i | (self typingResults at: i) addAssignment: value]
!

addElementSend: selector to: index
	self withTranslatedIndex: index do: [:i | (self typingResults at: i) addElementSend: selector]
!

addSend: selector to: index
	"Add a range check to filter out sends to instvars defined in superclasses, etc."

	self withTranslatedIndex: index do: [:i | (self typingResults at: i) addSend: selector]
! !

!TypeCollector methodsFor:'heuristics'!

assignmentTypeOf: val

	^self subclassResponsibility
!

handleAssignment: val for: index
	"Cannot use ifNotNil: with argument in Squeak, so use a temporary instead."

	| result |
	result := nil.
	val isInteger
		ifTrue: [self withTranslatedIndex: val do: [:idx | result := self lastAssignmentForIndex: idx]]
		ifFalse: [result := self assignmentTypeOf: val].
	result ifNotNil: [self addAssignment: result to: index]
!

langueSpecificPushSendOf: selector to: rec args: args

	^#computed
!

pushSendOf: selector to: rec args: args
	"Needs to be optimized"

	| recValue index |
	(rec == #self) ifTrue: [
		((index := instVars indexOf: selector) > 0) ifTrue: [^index - 1].
		((selector last == $:) and: [(index := instVars indexOf: (selector copyFrom: 1 to: selector size - 1) asSymbol) > 0])
			ifTrue: [self handleAssignment: args first for: index - 1. ^#computed]].
	(#(#= #== #< #> #<= #>= #~=) includes: selector) ifTrue: [^Boolean].
	('is*' match: selector) ifTrue: [^Boolean].
	((rec isVariableBinding) and: [(recValue := rec value) isBehavior
		and: [(recValue class categoryForSelector: selector) == #'instance creation' ]])
			ifTrue: [^Association key: #instcreation value: recValue].
	^self langueSpecificPushSendOf: selector to: rec args: args
! !

!TypeCollector methodsFor:'initialize-release'!

onClass: aClass

	theClass := aClass.
	instVars := aClass allInstVarNames collect: [:e | e asSymbol].
	typingResults :=  (instVars collect: [:ivar | ExtractedType forInstvar: ivar inClass: aClass]) asArray.
! !

!TypeCollector methodsFor:'private'!

lastAssignmentForIndex: anIndex

	| assignments |
	assignments := (self typingResults at: anIndex) assignments.
	^assignments isEmpty ifTrue: [nil] ifFalse: [assignments last]
!

newExtractor
	"Return a new extractor class. This is typically Smalltalk dialect dependent, so subclasses have to override to choose the one they want."

	^self subclassResponsibility
!

tryUsing: receiver for: selector ifNotUse: aClass
	"If receiver has a type, and the selector is understood by that type, pass that type. If not, use aClass as type."

	| cl |
	cl := self assignmentTypeOf: receiver.
	^(cl notNil and: [cl selectors includes: selector])
		ifTrue: [cl]
		ifFalse: [aClass]
!

typingResultsFor: instVarName
	^self typingResults at: (self absoluteIndexForVar: instVarName)
!

withTranslatedIndex: index do: aBlock

	aBlock value: index + 1
! !

!TypeCollector methodsFor:'public-typing'!

typeInstvarsOfClass: aClass
	| extractor |
	self onClass: aClass.
	extractor := self newExtractor.
	aClass
		selectorsAndMethodsDo: [:sel :method | extractor extractInterfacesFrom: method addTo: self requestElementTypeFor: nil].
	^self packagedResults
! !

!TypeCollector class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/roelTyper/RoelTyper__TypeCollector.st,v 1.2 2009/10/02 14:02:47 cg Exp $'
! !
