"{ Package: 'stx:goodies/roelTyper/squeak' }"

"{ NameSpace: RoelTyper }"

InstvarInterfaceExtractor subclass:#SqueakInstvarInterfaceExtractor
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'RoelTyper-Squeak'
!


!SqueakInstvarInterfaceExtractor methodsFor:'instruction decoding'!

blockReturnTop
	"Return Top Of Stack bytecode."

	^self pop
!

doDup
	"Duplicate Top Of Stack bytecode."

	self dupFirst
!

doPop
	"Remove Top Of Stack bytecode."

	^self pop
!

jump: delta

	| currentPC |
	currentPC := self input pc.
	delta < 1 ifTrue: [^self].
	self saveStacks at: currentPC + delta put: stack copy.
	stack := OrderedCollection new
!

jump: delta if: condition

	self pop.               "receiver of ifTrue or ifFalse, according to condition"
	delta < 1 ifTrue: [^self].
	self saveStacks at: self input pc + delta
		put: ((stack copy)
				add: #computed;
				yourself)
!

methodReturnConstant: value
	"Return Constant bytecode."

	^self pushConstant: value; sqReturnTop
!

methodReturnReceiver
	"Return Self bytecode."

	^self pushReceiver; sqReturnTop
!

methodReturnTop
	"Return Top Of Stack bytecode."

	^self sqReturnTop
!

popIntoLiteralVariable: anAssociation
	"Remove Top Of Stack And Store Into Literal Variable bytecode."

	^self pop
!

popIntoReceiverVariable: offset
	"Remove Top Of Stack And Store Into Instance Variable bytecode."

	collector handleAssignment: stack removeLast for: offset
!

popIntoTemporaryVariable: offset
	"Remove Top Of Stack And Store Into Temporary Variable bytecode."

	"Nothing to do,since I do not treat temporary variables for the moment."
!

pushActiveContext
	"Push Active Context On Top Of Its Own Stack bytecode."

	self pushContext
!

pushLiteralVariable: anAssociation
	"Push Contents Of anAssociation On Top Of Stack bytecode."

	self pushStatic: anAssociation
!

pushReceiverVariable: offset
	"Push Contents Of the Receiver's Instance Variable Whose Index
	is the argument, offset, On Top Of Stack bytecode."

	self pushInst: offset
!

pushTemporaryVariable: offset
	"Push Contents Of Temporary Variable Whose Index Is the
	argument, offset, On Top Of Stack bytecode."

	stack add: #tempVariable
!

send: selector super: supered numArgs: numberArguments
	"Send Message With Selector, selector, bytecode. The argument,
	supered, indicates whether the receiver of the message is specified with
	'super' in the source method. The arguments of the message are found in
	the top numArguments locations on the stack and the receiver just
	below them."

	^supered
		ifTrue: [self sendSuper: selector numArgs: numberArguments]
		ifFalse: [self send: selector numArgs: numberArguments]
!

storeIntoLiteralVariable: anAssociation
	"Store Top Of Stack Into Literal Variable Of Method bytecode."

	"Nothing to do, since I do not do anything with literal variables. Just keep the right-hand side on the stack for further processing"
!

storeIntoReceiverVariable: offset
	"Store Top Of Stack Into Instance Variable Of Method bytecode."

	| rvalue |
	collector handleAssignment: (rvalue := stack removeLast) for: offset.
	stack add: rvalue
!

storeIntoTemporaryVariable: offset
	"Store Top Of Stack Into Temporary Variable Of Method bytecode."

	"Nothing to do,since I do not treat temporary variables for the moment."
! !

!SqueakInstvarInterfaceExtractor methodsFor:'opcodes-control'!

sendSuper: selector numArgs: na

	stack removeLast: na + 1.
	stack add: #computed
! !

!SqueakInstvarInterfaceExtractor methodsFor:'private'!

sqReturnTop
	"In VW, method returnTop is inherited from instructionClient and does nothing."
! !

!SqueakInstvarInterfaceExtractor class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/roelTyper/squeak/RoelTyper__SqueakInstvarInterfaceExtractor.st,v 1.2 2009/10/02 14:02:47 cg Exp $'
! !
