"{ Package: 'stx:goodies/roelTyper/tests' }"

"{ NameSpace: RoelTyper }"

RoelTypingTestRoot subclass:#InstvarInterfaceExtractorTest
	instanceVariableNames:'a b c u v w x y z'
	classVariableNames:''
	poolDictionaries:''
	category:'RoelTyper-Tests'
!


!InstvarInterfaceExtractorTest methodsFor:'assignment tests'!

testArithmetic
	"self run: #testArithmetic"

	| collector |
	collector := self doForSource: 'u := v \\ 3. a := b floor. b := c truncateTo: (x := 4 roundTo: 5)'.
	self assertSends:  #( #(v #(\\)) #(b #(floor)) #(c #(truncateTo:))) in: collector.
	self assertAssignments:  #(#(u #(Number)) #(a #(Number)) #(b #(Number)) #(x #(Number))) in: collector
!

testArrayAssignment

	| collector |
	collector := self doForSource: 'v := #(one two three)'.
	self assertAssignments:  #(#(v #(#Array))) in: collector
!

testComplicated1

	| collector |
	collector := self doForSource: '
		b := Set new: 3 * (c collect: [:s | s])'.
	self assertSends: #( #(c #(collect:)) ) in: collector.
	self assertAssignments:  #(#(b #(#Set))) in: collector
!

testEquality

	| collector |
	collector := self doForSource: 'u := v = 3'.
	self assertSends:  #(#(v #(#=))) in: collector.
	self assertAssignments:  #(#(u #(#Boolean))) in: collector
!

testIdentity

	| collector |
	collector := self doForSource: 'u := v == 3'.
	self assertSends:  #(#(v #(#==))) in: collector.
	self assertAssignments:  #(#(u #(#Boolean))) in: collector
!

testIndirectLocalPop
	"self run: #testIndirectLocalPop"

	| collector |
	collector := self doForSource: 'u ifNil: [u := String new]'.
	self assertAssignments:  #(#(u #(String))) in: collector
!

testInstanceAssignment

	| collector |
	collector := self doForSource: 'w := TypeCollector new'.
	self assertAssignments:  #(#(w #(#TypeCollector))) in: collector
!

testInstanceAssignmentIndirect

	| collector |
	collector := self doForSource: 'self w: TypeCollector new'.
	self assertAssignments:  #(#(w #(#TypeCollector))) in: collector
!

testInstvarAssignment1

	| collector |
	collector := self doForSource: '
		| temp1 temp2 temp3 |
		a := 3.
		b := a'.
	self assertAssignments:  #(#(b #(#SmallInteger)) (a #(#SmallInteger))) in: collector
!

testInstvarAssignment2

	| collector |
	collector := self doForSource: '
		| temp1 temp2 temp3 |
		c := b := 3.'.
	self assertAssignments:  #(#(b #(#SmallInteger)) (c #(#SmallInteger))) in: collector
!

testMetaClassAssignment

	| collector wAssignments wIndex wIndexRelative |
	collector := self doForSource: 'w := TypeCollector'.
	wIndex := collector absoluteIndexForVar: #w.  "Absolute"
	wIndexRelative := wIndex - (self class allInstVarNames size - self class instVarNames size).
	wAssignments := collector assignmentsTo: #w.
	self assert: wAssignments size = 1.
	self assert: (wAssignments includes: TypeCollector class).
	self emptyAssigment: collector typingResults atIndexes: ((1 to: self class instVarNames size) asOrderedCollection remove: wIndexRelative; yourself)
! !

!InstvarInterfaceExtractorTest methodsFor:'auxiliary'!

assertAssignments: description in: collector

	| emptyIndexes ivarName ivarInterface extractedInterface ivarIndex anArray |
	anArray := collector typingResults.
	emptyIndexes := (1 to: self class allInstVarNames size) asOrderedCollection.
	description do: [:desc |
		ivarName := desc first.
		ivarInterface := desc at: 2.
		ivarIndex := collector absoluteIndexForVar: ivarName.
		extractedInterface := collector assignmentsTo: ivarName.
		self assert: extractedInterface size = ivarInterface size.
		self assert: (extractedInterface allSatisfy: [:each | ivarInterface includes: each name]).
		emptyIndexes remove: ivarIndex].
	self emptyAssigment: anArray atIndexes: emptyIndexes.
!

assertSends: description in: collector

	| emptyIndexes ivarName ivarInterface extractedInterface ivarIndex anArray |
	anArray := collector typingResults.
	emptyIndexes := (1 to: self class allInstVarNames size) asOrderedCollection.
	description do: [:desc |
		ivarName := desc first.
		ivarInterface := desc at: 2.
		ivarIndex :=  collector absoluteIndexForVar: ivarName.
		extractedInterface := collector sendsTo: ivarName.
		self assert: extractedInterface size = ivarInterface size.
		self assert: (extractedInterface allSatisfy: [:each | ivarInterface includes: each]).
		emptyIndexes remove: ivarIndex].
	self emptyInterface: anArray atIndexes: emptyIndexes.
!

denyAssignments: description in: collector

	| emptyIndexes ivarName ivarInterface extractedInterface ivarIndex anArray |
	anArray := collector typingResults.
	emptyIndexes := (1 to: self class allInstVarNames size) asOrderedCollection.
	description do: [:desc |
		ivarName := desc first.
		ivarInterface := desc at: 2.
		ivarIndex := collector absoluteIndexForVar: ivarName.
		extractedInterface := collector assignmentsTo: ivarName.
		ivarInterface do: [:each | self deny: (extractedInterface includes: each)].
		emptyIndexes remove: ivarIndex].
	self emptyAssigment: anArray atIndexes: emptyIndexes.
!

empty: anArray atIndexes: indexCollection

	indexCollection do: [:index |
		self assert: (anArray at: index) isEmpty]
!

emptyAssigment: anArray atIndexes: indexCollection

	indexCollection do: [:index |
		self assert: (anArray at: index) hasEmptyAssignments]
!

emptyInterface: anArray atIndexes: indexCollection

	indexCollection do: [:index |
		self assert: (anArray at: index) hasEmptyInterface]
!

processMethod: aCompiledMethod
	"Fail by default. Needs to be overridden by subclasses to trigger the base testing backbone."

	| collector |
	collector := TypeCollector onClass: self class.
	TypeCollector new newExtractor extractInterfacesFrom: aCompiledMethod addTo: collector requestElementType: false.
	^collector
! !

!InstvarInterfaceExtractorTest methodsFor:'combined tests'!

testArithmeticValueSpecialisation

	| collector |
	collector := self doForSource: '
		b := (Point x: 2 y: 4) - (1@1) '.
	self assertSends: #() in: collector.
	self assertAssignments:  #(#(b #(#Point))) in: collector
! !

!InstvarInterfaceExtractorTest methodsFor:'non supported'!

testComplicated2

	| collector |
	collector := self doForSource: '
		| temp1 temp2 |
		temp1 := b := temp2 := 3.'.
	self denyAssignments:  #(#(b #(#SmallInteger))) in: collector
!

testComplicated3
	"self run: #testComplicated3"
	"selfdebug: #testComplicated3"

	| collector |
	collector := self doForSource: '
		| temp1 temp2 temp3 |
		temp2 := b := temp1 := 3.'.
	self denyAssignments:  #(#(b #(#SmallInteger))) in: collector
! !

!InstvarInterfaceExtractorTest methodsFor:'send tests'!

testCascaded1

	| collector |
	collector := self doForSource: 'x printString; size'.
	self assertSends:  #(#(x #(#printString #size))) in: collector
!

testCascaded2

	| collector |
	collector := self doForSource: 'x printString; size; yourself'.
	self assertSends:  #(#(x #(#printString #size #yourself))) in: collector
!

testIftrue

	| vSends collector |
	collector := self doForSource: 'u := v isNil ifTrue: [1] ifFalse: [2]'.
	vSends := collector sendsTo: #v.
	self assert: (vSends size = 1).
	self assert: (vSends includes: #isNil).
!

testSuperivarAssignment

	| collector |
	collector := self doForSource: '^testSelector := testSelector'.
	self assertSends:  #() in: collector
!

testSuperivarSend

	| collector |
	collector := self doForSource: '^testSelector printString'.
	self assertSends:  #(  #(testSelector #(#printString))  ) in: collector
!

testblockindirectxy

	| collector |
	collector := self doForSource: '^self testu ifTrue: [self x] ifFalse: [self y asString]'.
	self assertSends:  #(#(y #(#asString))) in: collector
!

testblockuwx

	| collector |
	collector := self doForSource: '^[u + w] on: Error do: [:exc | x printString]'.
	self assertSends:  #(#(u #(#+)) #(x #(#printString))) in: collector
!

testblockxy

	| collector |
	collector := self doForSource: '^self xyw ifTrue: [x] ifFalse: [y]'.
	self assertSends:  #() in: collector
!

testblockyab
	" #'ifTrue:ifFalse: is sent to a, but is not Captured by the extractor"

	| collector |
	collector := self doForSource: '^self a ifTrue: [b] ifFalse: [y]'.
	self assertSends:  #() in: collector
!

testindirectx

	| collector |
	collector := self doForSource: '^self x printString'.
	self assertSends:  #(#(x #(#printString))) in: collector
!

testu

	| collector |
	collector := self doForSource: '^u'.
	self assertSends:  #() in: collector
!

testuxyw

	| collector |
	collector := self doForSource: '        u := (x asString ~= y printString). ^u = w'.
	self assertSends:  #(#(y #(#printString)) #(x #(#asString)) #(u #(#=))) in: collector
!

testxyw

	| collector |
	collector := self doForSource: '
		| t |
		t := (x = y).
		^t = w'.
	self assertSends:  #(#(x #(#=))) in: collector
!

testzuv

	| collector |
	collector := self doForSource: '^u ~= z or: [u = v]'.
	self assertSends:  #(#(u #(#= #~=))) in: collector
! !

!InstvarInterfaceExtractorTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/roelTyper/tests/RoelTyper__InstvarInterfaceExtractorTest.st,v 1.2 2009/10/02 14:04:04 cg Exp $'
! !
