"{ Package: 'stx:goodies' }"

ScreenSaver subclass:#LightInTheDark
	instanceVariableNames:'xpos ypos bigForm radius timedBlock2'
	classVariableNames:''
	poolDictionaries:''
	category:'demos-ScreenSaver'
!

LightInTheDark comment:'
start with:

    LightInTheDark open
'!

!LightInTheDark class methodsFor:'documentation'!

documentation
"
    a SpotLight screensaver ...

    a simple full-screen view example.
    Also, demonstrates arbitrary shaped views (X displays only).

    [author:]
	Claus Gittinger

    [see also:]
	ScreenSaver LightInTheDark2

    [start with:]
	LightInTheDark open
"
! !

!LightInTheDark methodsFor:'cleanup'!

destroy
    "stop any background timeout activities"

    Processor removeTimedBlock:timedBlock2.
    super destroy

    "Modified: 17.5.1996 / 11:34:05 / cg"
! !

!LightInTheDark methodsFor:'initialization'!

initialize
    "create a big bitmap, to hold the shape
     (size of the screen - from which a circle is stamped out)"

    |cursor|

    radius := (device horizontalPixelPerMillimeter * 20) rounded.

    bigForm := Form width:device width 
		   height:device height 
		    depth:1
		       on:device.
    bigForm fill:(Color colorId:1).

    super initialize.

    timedBlock2 := [self evenCycle].
    timedBlock := [self oddCycle].
    self saveUnder:true.

    "LightInTheDark open"

    "Modified: 17.5.1996 / 11:34:53 / cg"
! !

!LightInTheDark methodsFor:'private'!

evenCycle
    "show darkness (fill my hole & change the view shape)"

    self fillHole.

    self borderShape:bigForm.
    self viewShape:bigForm.

    Processor addTimedBlock:timedBlock afterSeconds:0.4

    "Modified: 17.5.1996 / 11:36:25 / cg"
!

fillHole
    "fill a rectangluar area in the shape bitmap (fill the hole)"

    xpos notNil ifTrue:[
	bigForm paint:(Color colorId:1).
	bigForm fillRectangleX:(xpos-radius-2) rounded y:(ypos-radius-2) rounded 
			 width:radius*2+4 height:radius*2+4.
    ]

    "Modified: 17.5.1996 / 11:40:57 / cg"
!

oddCycle
    "punch a hole at a random place"

    self fillHole.
    xpos := (random next * (device width - radius)) rounded asInteger.
    ypos := (random next * (device height - radius)) rounded asInteger.
    self punchHole.
    Processor addTimedBlock:timedBlock2 afterSeconds:1 + (random nextBetween:1 and:3) 
!

punchHole
    "punch out a circular area from the shape bitmap,
     and change my view shape."

    bigForm paint:(Color colorId:0).
    bigForm fillCircleX:xpos rounded y:ypos rounded radius:radius.

    self borderShape:bigForm.
    self viewShape:bigForm.
    device flush.

    "Modified: 17.5.1996 / 11:37:02 / cg"
! !

!LightInTheDark class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/screenSavers/LightInTheDark.st,v 1.7 2000/03/02 14:14:51 cg Exp $'
! !
