'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:42'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WABuilder
	instanceVariableNames:'fullDocument rootBlock rootClass documentClass
		scriptGeneratorClass rendererClass codec actionUrl resourceUrl'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Rendering'
!

WABuilder comment:'This is a convenience class which provides a result of a rendering operation as a string. It is expected to be used like this:
WARenderCanvas builder render: [ :html |
	html anchor
		url: ''htttp://www.seaside.st'';
		with: ''Seaside Homepage'' ]
See WABuilderCanvasTest for more examples.'
!

!WABuilder class methodsFor:'instance creation'!

on: aRendererClass
	^ self new
		rendererClass: aRendererClass;
		yourself
! !

!WABuilder methodsFor:'accessing'!

actionUrl
	^ actionUrl ifNil: [ actionUrl := WAUrl new ]
!

actionUrl: aUrl
	actionUrl := aUrl
!

codec
	^ codec ifNil: [
		 codec := [ self requestContext codec ] 
			on: WARequestContextNotFound
			do: [ :e | GRNullCodec new ] ]
!

codec: aCodec
	codec := aCodec
!

documentClass
	^ documentClass ifNil: [ documentClass := WADocument ]
!

documentClass: aDocumentClass
	documentClass := aDocumentClass
!

fullDocument
	^ fullDocument
!

fullDocument: aBoolean
	fullDocument := aBoolean
!

rendererClass
	^ rendererClass
!

rendererClass: aRendererClass 
	rendererClass := aRendererClass
!

resourceUrl
	^ resourceUrl
!

resourceUrl: aUrl
	resourceUrl := aUrl
!

rootBlock
	^ rootBlock ifNil: [ rootBlock := [ :root | ] ]
!

rootBlock: anObject
	rootBlock := anObject
!

rootClass
	^ rootClass ifNil: [ rootClass := WARoot ]
!

rootClass: aRootClass
	rootClass := aRootClass
!

scriptGeneratorClass
	^ scriptGeneratorClass ifNil: [ scriptGeneratorClass := WADefaultScriptGenerator  ]
!

scriptGeneratorClass: aScriptGeneratorClass
	scriptGeneratorClass := aScriptGeneratorClass
! !

!WABuilder methodsFor:'deprecated'!

renderResponse: anObject 
	"This implementation will only work when used within a Session or something that has WACurrentRequestContext defined."
	self 
		greaseDeprecatedApi: 'WABuilder>>#renderResponse:'
		details: 'Returning a response is no longer allowed. Use WABuilder>>#render: to obtain the contents, configure the response returned by WARequestContext>>#response, then respond using WARequestContext>>#respond.'.
	self fullDocument: true.
	self requestContext respond: 
		[ :response | 
		response
			contentType: WAMimeType textHtml;
			nextPutAll: (self render: anObject) ]
! !

!WABuilder methodsFor:'initialization'!

initialize
	super initialize.
	self fullDocument: false
! !

!WABuilder methodsFor:'private'!

closeDocument: aDocument
	self fullDocument 
		ifTrue: [ aDocument close ]
		ifFalse: [ 
			"hack to make it work"
			(aDocument isKindOf: WAHtmlDocument)
				ifTrue: [ aDocument scriptGenerator closeOn: aDocument ] ]
!

openDocument: aDocument context: aContext 
	| documentRoot |
	self fullDocument ifFalse: [ ^ self ].
	documentRoot := self rootClass context: aContext.
	self rootBlock value: documentRoot.
	aDocument open: documentRoot
! !

!WABuilder methodsFor:'rendering'!

render: anObject
	"Render anObject and return the contents of the resulting Document as a String.
	anObject must understand #renderOn:. Commonly anObject will be a one-argument
	block, which will be evaluated with the appropriate renderer."
	
	^ String streamContents: [ :stream |
		| context document renderer |
		document := self documentClass on: stream codec: self codec.
		document scriptGenerator: self scriptGeneratorClass new.
		context := WARenderContext new.
		context document: document.
		context actionUrl: self actionUrl; resourceUrl: self resourceUrl.
		renderer := self rendererClass context: context.
		
		self openDocument: document context: context.
		renderer render: anObject; flush.
		self closeDocument: document ]
! !

