'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:42'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WARequestHandler subclass:#WADispatcher
	instanceVariableNames:'defaultName handlers'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-RequestHandling'
!

WADispatcher class instanceVariableNames:'default'

"
 No other class instance variables are inherited by this class.
"
!

WADispatcher comment:''
!

!WADispatcher class methodsFor:'initialization'!

resetDefault
	default := nil.
! !

!WADispatcher class methodsFor:'accessing'!

default
	^ default ifNil: [ default := self new ]
!

description
	^ 'Dispatcher'
! !

!WADispatcher methodsFor:'accessing'!

defaultHandler
	self defaultName isNil ifTrue: [ ^ nil ].
	^ self handlerAt: self defaultName ifAbsent: [ nil ]
!

defaultName
	^ defaultName
!

defaultName: aString
	defaultName := aString
!

handlerAt: aString
	^ self
			handlerAt: aString
			with: [ :dispatcher :token |
				dispatcher handlers at: token ] 
!

handlerAt: aString ifAbsent: aBlock
	^ self
		handlerAt: (aString
			ifNil: [ ^ aBlock value ])
		with: [ :dispatcher :token |
			dispatcher handlers
				at: token
				ifAbsent: [ ^ aBlock value ] ]
!

handlerAt: aString with: aBlock
	^ (aString subStrings: '/') inject: self into: aBlock
!

handlers
	^ handlers
!

nameOfHandler: aRequestHandler 
	"Answer the name of the given request handler. Signal an error the given handler is not registered in the receiver."
	^ handlers
		keyAtValue: aRequestHandler
		ifAbsent: [ WAUnregisteredHandlerError signal ]
!

urlFor: aRequestHandler
	^ self url
		addToPath: (self nameOfHandler: aRequestHandler);
		yourself
! !

!WADispatcher methodsFor:'copying'!

postCopy
	super postCopy.
	handlers := handlers valuesCollect: [ :each | each copy setParent: self; yourself ]
! !

!WADispatcher methodsFor:'deprecated'!

entryPointAt: aString 
	self 
		greaseDeprecatedApi: 'WADispatcher>>#entryPointAt:'
		details: 'Use WADispatcher>>#handlerAt:.'.
	^ self handlerAt: aString
! !

!WADispatcher methodsFor:'handling'!

handleDefault: aRequestContext
	self defaultName isNil
		ifTrue: [ aRequestContext responseGenerator forbidden; respond ]
		ifFalse: [ self handleFiltered: aRequestContext named: self defaultName ]
!

handleFiltered: aRequestContext
	| consumer |
	consumer := aRequestContext consumer.
	consumer atEnd
		ifTrue: [ self handleDefault: aRequestContext ]
		ifFalse: [ self handleFiltered: aRequestContext named: consumer next ]
!

handleFiltered: aRequestContext named: aString
	(self 
		handlerAt: aString 
		ifAbsent: [ aRequestContext responseGenerator notFound; respond ])
			handle: aRequestContext
! !

!WADispatcher methodsFor:'initialization'!

initialize
	super initialize.
	handlers := Dictionary new
! !

!WADispatcher methodsFor:'parent/child'!

childrenDo: aBlock
	handlers do: aBlock
!

register: aRequestHandler at: aString 
	aString ifNil: [ WAError signal: 'Registration key cannot be nil.' ].
	handlers 
		at: aString
		put: (aRequestHandler setParent: self; yourself).
	^ aRequestHandler
!

unregister: aRequestHandler
	handlers 
		removeKey: aRequestHandler name
		ifAbsent: [ ^ nil ].
	^ aRequestHandler unregistered
! !

!WADispatcher methodsFor:'testing'!

isDispatcher
	^ true
! !

