'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:44'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAEncoder
	instanceVariableNames:'stream table'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Document'
!

WAEncoder class instanceVariableNames:'table'

"
 No other class instance variables are inherited by this class.
"
!

WAEncoder comment:''
!

!WAEncoder class methodsFor:'initialization'!

initializeTable
	"Initializes the encoding table."
	| stream characterLimit |
	characterLimit := self maximumCharacterValue.
	"character values at zero so we need to add 1"
	table := Array new: characterLimit + 1.
	stream := WriteStream on: (String new: 6).
	0 to: characterLimit do: [ :index | 
		| character value |
		character := Character codePoint: index.
		self encode: character on: stream reset.
		"Smalltalk indices are one based but character values start at 0"
		value := stream contents = (String with: character)
			ifTrue: [ nil ]
			ifFalse: [ stream contents ].
		table at: index + 1 put: value ]
! !

!WAEncoder class methodsFor:'instance creation'!

new
	^ self on: GRPlatform current readWriteCharacterStream
!

on: aStream
	^ self on: aStream table: table
!

on: aStream codec: aCodec
	"For compatibility with WAUrlEncoder"
	^ self on: aStream
!

on: aStream table: anArray
	^ self basicNew initializeOn: aStream table: anArray; yourself
! !

!WAEncoder class methodsFor:'private'!

encode: aCharacter on: aStream
	self subclassResponsibility
!

maximumCharacterValue
	"find the maximum value of a character that we can instantiate, for Squeak 3.7 this is 255"
	^ [ Character codePoint: 16rFFFF.
		16rFFFF ]
			on: Error
			do: [ :error  | 16rFF ]
! !

!WAEncoder methodsFor:'accessing'!

nextPut: aCharacter 
	| value encoded |
	value := aCharacter greaseInteger.
	value < table size 
		ifFalse: [ stream nextPut: aCharacter ]
		ifTrue: [
			encoded := table at: value + 1.
			"Issue 482: use #notNil because it is faster than #isString because it is not actually sent"
			encoded notNil
				ifTrue: [ stream nextPutAll: encoded ]
				ifFalse: [ stream nextPut: aCharacter ] ]
!

nextPutAll: aString 
	"uses #to:do: for speed reasons (on Pharo)
	this is not premature optimization, this is a hotspot method method
	and #to:do: shows measurable speed improvements for rendering seaside pages"
	1 to: aString size do: [ :index |
		| character value encoded |
		character := aString at: index.
		value := character greaseInteger.
		encoded := table at: value + 1.
		"Issue 482: use #notNil because it is faster than #isString because it is not actually sent"
		encoded notNil 
			ifTrue: [ stream nextPutAll: encoded ]
			ifFalse: [ stream nextPut: character ] ]
! !

!WAEncoder methodsFor:'initialization'!

initializeOn: aStream table: anArray
	self initialize.
	stream := aStream.
	table := anArray
! !

