'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAFile
	instanceVariableNames:'contents contentType fileName'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WAFile comment:'I represent a file that was uploaded by the user via #fileInput.
Instance Variables
	contents:		<ByteArray>
	contentType:	<WAMimeType>
	fileName:		<String>
contents
	- the contents of the file, binary
contentType
	- the content type of the file
	
fileName
	- The name of the file the user agent uploaded. This is locale name on the machine of the client. The instance variable might include the full path on the file system, the accessor will never return the path.'
!

!WAFile methodsFor:'accessing'!

contentType
	^ contentType
!

contentType: aMimeTypeOrString
	contentType := aMimeTypeOrString isNil
		ifFalse: [ aMimeTypeOrString seasideMimeType ]
!

contents
	"Answer a String"
	self
		greaseDeprecatedApi: 'WAFile>>#contents'
		details: 'Use #rawContents, #contentsDecoded or #contentsDecodedUsing: instead.'.
	^ self contentsDecodedUsing: 'ISO-8859-1'
!

contents: aByteArray
	contents := aByteArray
!

contentsDecoded
	"Answer the contents decoded using the character set in the part header. Signal an error if not character set is present in the part header."
	| type charSet |
	type := self contentType.
	charSet := type isNil ifFalse: [ type charSet ].
	^ charSet isNil
		ifTrue: [ WAIllegalStateException signal: 'no character set of file upload can not be determined' ]
		ifFalse: [ self contentsDecodedUsing: charSet ]
!

contentsDecodedUsing: aCharSetName
	"Answer the contents decoded using the given character set name."
	^ (GRCodec forEncoding: aCharSetName) decode: self rawContents
!

fileName
	"compensate for windows explorer behavior"
	^ self isWindowsPath
		ifTrue: [ fileName copyAfterLast: $\ ]
		ifFalse: [ fileName ]
!

fileName: aString
	fileName := aString
!

rawContents
	"Answer the raw, undecoded contents as sent by the user agent as a ByteArray.
	
	Use #contentsDecoded or #contentsDecodeUsing: to access the decoded contents."
	
	^ contents
!

size
	"the file size in bytes"

	^ self rawContents size
! !

!WAFile methodsFor:'private'!

isWindowsPath
	"'#:\*' match: fileName is broken on Squeak for WideString arguments"
	^ fileName notEmpty
		and: [ fileName first isLetter
		and: [ fileName size >= 3
		and: [ (fileName copyFrom: 2 to: 3) = ':\' ] ] ]
! !

