'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:44'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

GRSmallDictionary subclass:#WAHtmlAttributes
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Document'
!

WAHtmlAttributes comment:'I represent the attributes of a (X)(HT)ML tag. Compared to my superclass I ignore requests to add a nil-values. I also don''t throw an exception when accessing a key that doesn''t exist, but instead return nil.'
!

!WAHtmlAttributes methodsFor:'accessing'!

at: aKey append: aValue
	"Append aValue to the attribute aKey. If already present, concatenate it with a space. Ignore the request if aValue is nil."

	^ self at: aKey append: aValue separator: ' '
!

at: aKey append: aValue separator: aString 
	"Append aValue to the attribute aKey. If already present, concatenate it with aString. Ignore the request if aValue is nil."
	
	^ aValue isNil
		ifTrue: [ self at: aKey ]
		ifFalse: [
			self at: aKey put: (String streamContents: [ :stream | 
				self at: aKey ifPresent: [ :value | 
					stream nextPutAll: value greaseString.
					stream nextPutAll: aString ].
				stream nextPutAll: aValue greaseString ]) ]
! !

!WAHtmlAttributes methodsFor:'convenience'!

addClass: aString
	"Add an additional CSS class aString to the receiver."

	self at: 'class' append: aString separator: ' '
!

addMedia: aString
	"Add an additional intended destination medium aString to the receiver."

	self at: 'media' append: aString separator: ', '
!

addStyle: aString
	"Add an additional CSS style definition aString to the receiver."

	self at: 'style' append: aString separator: ';'
! !

!WAHtmlAttributes methodsFor:'encoding'!

encodeOn: aDocument
	"Encode the receivers attribute onto aDocument. Note that this implementation reqires those two checks for true and false exactly the way they are here, to reliable encode boolean attributes in an XHTML compliant way."

	self keysAndValuesDo: [ :key :value |
		value == false ifFalse: [
			aDocument nextPut: $ ; nextPutAll: key; nextPutAll: '="'.
			value == true
				ifTrue: [ aDocument nextPutAll: key ]
				ifFalse: [ aDocument print: value ].
			aDocument nextPut: $" ] ]
! !

!WAHtmlAttributes methodsFor:'private'!

errorKeyNotFound
	^ nil
!

privateAt: aKey put: aValue
	aValue ifNil: [ ^ nil ].
	^ super privateAt: aKey put: aValue
! !

