'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WARoot subclass:#WAHtmlRoot
	instanceVariableNames:'docType htmlAttrs headAttrs bodyAttrs headElements title styles
		scripts'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Document'
!

WAHtmlRoot comment:'A WAHtmlRoot is the root element of an HTML Document (<html>).
Instance Variables
	bodyAttrs:		<WAHtmlAttributes>
	context:			<WARenderContext>
	docType:		<String>
	headAttrs:		<WAHtmlAttributes>
	headElements:	<OrderedCollection<WAHtmlElement>>
	htmlAttrs:		<WAHtmlAttributes>
	scripts:			<Set<String>>
	styles:			<Set<String>>
	title:			<String>
bodyAttrs
	- the attributes of the <body> element
context
	- the context used to render the contents of the <body> element
docType
	- the document type
headAttrs
	- the attributes of the <head> element
headElements
	- the elements inside the <head> section
htmlAttrs
	- the attriebutes of the <html> element
scripts
	- contains the strings returned by WAPresenter >> #script
styles
	- contains the strings returned by WAPresenter >> #style
title
	- the title of the HTML document, the contents of the <title> element
'
!

!WAHtmlRoot methodsFor:'accessing'!

headElements
	^ headElements
!

headElements: anOrderedCollection
	headElements := anOrderedCollection
! !

!WAHtmlRoot methodsFor:'accessing-attributes'!

bodyAttributes
	^ bodyAttrs ifNil: [ bodyAttrs := WAHtmlAttributes new ]
!

headAttributes
	^ headAttrs ifNil: [ headAttrs := WAHtmlAttributes new ]
!

htmlAttributes
	^ htmlAttrs ifNil: [ htmlAttrs := WAHtmlAttributes new ]
! !

!WAHtmlRoot methodsFor:'accessing-properties'!

title
	^ title
!

title: aString
	title := aString
! !

!WAHtmlRoot methodsFor:'adding'!

add: aHtmlElement
	"Add a HTML head element to the receiver."

	^ headElements add: aHtmlElement
!

addScript: aString
	"Include the script aString into the receiver."

	scripts ifNil: [ scripts := Set new ].
	scripts add: aString
!

addStyle: aString
	"Include the style-sheet aString into the receiver."

	styles ifNil: [ styles := Set new ].
	styles add: aString
! !

!WAHtmlRoot methodsFor:'convenience'!

contentBase: urlString
	self base url: urlString
!

redirectTo: aLocationString delay: aNumber
	self meta redirectAfter: aNumber to: aLocationString
! !

!WAHtmlRoot methodsFor:'doctype'!

beXhtml10Strict
	self docType: '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
!

beXhtml10Transitional
	self docType: '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
!

beXhtml11
	self docType: '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'
!

docType
	^ docType
!

docType: aString
	"Set the document-type of the receiver. The table at http://www.w3.org/TR/2002/NOTE-xhtml-media-types-20020430/ summarizes the recommendation to content authors for labeling their XHTML documents:

Media type				XHTML 1.0 (HTML compatible)	XHTML 1.0 (other)	XHTML Basic / 1.1	XHTML+MathML
text/html				MAY							SHOULD NOT			SHOULD NOT			SHOULD NOT
application/xhtml+xml	SHOULD							SHOULD				SHOULD				SHOULD
application/xml			MAY							MAY				MAY				MAY
text/xml					MAY							MAY				MAY				MAY"

	docType := aString
! !

!WAHtmlRoot methodsFor:'elements'!

base
	^ self add: (WABaseElement root: self)
!

if
	^ WAConditionalComment root: self
!

javascript
	^ self script beJavascript; yourself
!

link
	^ self add: (WALinkElement root: self)
!

meta
	^ self add: (WAMetaElement root: self)
!

revealedIf
	^ WARevealedConditionalComment root: self
!

script
	^ self add: (WAScriptElement root: self)
!

stylesheet
	^ self link beStylesheet; beCss; yourself
! !

!WAHtmlRoot methodsFor:'initialization'!

initialize
	super initialize.
	headElements := OrderedCollection new.
	title := docType := String new
! !

!WAHtmlRoot methodsFor:'writing'!

closeOn: aDocument
	aDocument scriptGenerator
		close: self on: aDocument.
	self writeFootOn: aDocument
!

openOn: aDocument
	aDocument scriptGenerator 
		open: self on: aDocument.
	self writeHeadOn: aDocument
!

writeElementsOn: aDocument
	aDocument openTag: 'title'; nextPutAll: title; closeTag: 'title'.
	headElements do: [ :each | aDocument print: each ]
!

writeFootOn: aDocument
	aDocument closeTag: 'body'.
	aDocument closeTag: 'html'
!

writeHeadOn: aDocument
	aDocument nextPutAll: docType.
	aDocument openTag: 'html' attributes: htmlAttrs.
	aDocument openTag: 'head' attributes: headAttrs.
	self writeElementsOn: aDocument.
	self writeStylesOn: aDocument.
	self writeScriptsOn: aDocument.
	aDocument closeTag: 'head'.
	aDocument openTag: 'body' attributes: bodyAttrs
!

writeScriptsOn: aDocument
	scripts ifNil: [ ^ self ].
	scripts do: [ :each |
		aDocument print: ((WAScriptElement root: self)
			beJavascript;
			document: each;
			yourself) ]
!

writeStylesOn: aDocument
	styles ifNil: [ ^ self ].
	styles do: [ :each |
		aDocument print: ((WALinkElement root: self)
			beStylesheet; beCss;
			document: each;
			yourself) ]
! !

