'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAMimeType
	instanceVariableNames:'main sub parameters'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WAMimeType comment:'A WAMimeType abstracts a Internet media type, it is a two-part identifier for file formats on the Internet.
Instance Variables
	main:			<String>
	parameters:	<GRSmallDictionary>
	sub:			<String>
main
	- the main type
parameters
	- a lazily initialized dictionary of optional parameters
sub
	- the subtype
'
!

!WAMimeType class methodsFor:'instance creation'!

fromString: aString 
	| main endOfSub sub parts parameters slashIndex |
	slashIndex := aString indexOf: $/.
	slashIndex = 0
		ifTrue: [
			"work arount for Issue 655
			some user agents just send '*', not '*/*'"
			endOfSub := aString indexOf: $;.
			endOfSub = 0
				ifTrue: [
					main := aString.
					endOfSub := aString size ]
				ifFalse: [
					main := aString copyFrom: 1 to: endOfSub - 1 ].
			sub := main ]
		ifFalse: [
			main := aString copyUpTo: $/.
			endOfSub := aString indexOf: $;.
			endOfSub := endOfSub = 0 
				ifTrue: [ aString size ]
				ifFalse: [ endOfSub - 1 ].
			sub := aString 
				copyFrom: main size + 2
				to: endOfSub ].
	endOfSub = aString size ifTrue: [
		^ self 
			main: main
			sub: sub ].
	parts := (aString 
		copyFrom: endOfSub + 1
		to: aString size) subStrings: ';'.
	parameters := GRSmallDictionary new.
	parts do: [ :each | 
		parameters 
			at: (each copyUpTo: $=) trimBoth
			put: (each copyAfter: $=) trimBoth ].
	^ self 
		main: main
		sub: sub
		parameters: parameters
!

main: aMainString sub: aSubString
	^ self new
		main: aMainString;
		sub: aSubString;
		yourself
!

main: aMainString sub: aSubString parameters: aDictionary
	^ (self main: aMainString sub: aSubString)
		parameters: aDictionary;
		yourself
! !

!WAMimeType class methodsFor:'convenience'!

applicationAtomXml
	^ self main: 'application' sub: 'atom+xml'
!

applicationJson
	^ self main: 'application' sub: 'json'
!

applicationOctetStream
	^ self main: 'application' sub: 'octet-stream'
!

applicationRssXml
	^ self main: 'application' sub: 'rss+xml'
!

formUrlencoded
	^ self main: 'application' sub: 'x-www-form-urlencoded'
!

imageGif
	^ self main: 'image' sub: 'gif'
!

imageJpeg
	^ self main: 'image' sub: 'jpeg'
!

imagePng
	^ self main: 'image' sub: 'png'
!

textCss
	^ self main: 'text' sub: 'css'
!

textHtml
	^ self main: 'text' sub: 'html'
!

textJavascript
	^ self main: 'text' sub: 'javascript'
!

textPlain
	^ self main: 'text' sub: 'plain'
! !

!WAMimeType methodsFor:'accessing'!

main
	^ main
!

main: aString
	main := aString
!

parameters
	parameters isNil ifTrue: [ parameters := GRSmallDictionary new ].
	^ parameters
!

parameters: aDictionary
	parameters := aDictionary
!

sub
	^ sub
!

sub: aString
	sub := aString
! !

!WAMimeType methodsFor:'comparing'!

= other
	^ (other isKindOf: WAMimeType)
		and: [ self main = other main
		and: [ self sub = other sub ] ]
!

hash
	^ self main hash bitXor: self sub hash
! !

!WAMimeType methodsFor:'converting'!

greaseString
	^ String streamContents: 
		[ :stream | 
		stream
			nextPutAll: self main;
			nextPut: $/;
			nextPutAll: self sub.
		parameters isNil ifFalse: 
			[ parameters keysAndValuesDo: 
				[ :key :value | 
				stream
					nextPut: $;;
					nextPutAll: key;
					nextPut: $=;
					nextPutAll: value ] ] ]
!

seasideMimeType
	^ self
! !

!WAMimeType methodsFor:'parameters'!

charSet
	^ self parameters at: 'charset' ifAbsent: [ nil ]
!

charset: aString
	self parameters at: 'charset' put: aString
!

parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue
! !

!WAMimeType methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self greaseString;
		nextPut: $)
! !

!WAMimeType methodsFor:'testing'!

isBinary
	"answers whether the contents of a document of the receiving mime type are binary"
	| subTypes |
	self main = 'text' ifTrue: [ ^ false ].
	self main = 'application' ifFalse: [ ^ true ].
	subTypes := self sub subStrings: '+'.
	^ subTypes noneSatisfy: [ :each | 
		#('x-javascript' 'xml') includes: each ]
!

isNonStandard
	"tests if the receiver is a non-standard mime type that is not registered with IANA"
	^ (self main beginsWithSubCollection: 'x-')
		or: [ (self main beginsWithSubCollection: 'X-')
		or: [ (self sub beginsWithSubCollection: 'x-')
		or: [ self sub beginsWithSubCollection: 'X-' ] ] ]
!

isVendorSpecific
	"tests if the receiver is a vendor specific mimetype"

	^ self sub beginsWithSubCollection: 'vnd.'
!

matches: aMimeType
	"aMimeType is the pattern to match, it is a normal WAMimeType instance where main or sub can be wildcards"
	^ (aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]
! !

