'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:40'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAMutex
	instanceVariableNames:'semaphore owner'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Utilities'
!

WAMutex comment:'WAMutex provides mutual exclusion properties.
Only one process at a time can execute code within its #critical: method. Other processes attempting to call #critical: will block until the first process leaves the critical section. The process that owns the mutex (the one currently in the critical section), however, may call #critical: repeatedly without fear of blocking.
The process currently inside the critical section can be terminated by calling #terminateOwner. This will result in the process being unwound and the critical section being freed.'
!

!WAMutex methodsFor:'accessing'!

owner
	^ owner
! !

!WAMutex methodsFor:'copying'!

copy
	^ self species new
! !

!WAMutex methodsFor:'initialization'!

initialize
	super initialize.
	semaphore := GRPlatform current semaphoreClass forMutualExclusion
! !

!WAMutex methodsFor:'mutual exclusion'!

critical: aBlock
	| process |
	process := Processor activeProcess.
	^ (owner == process)
		ifTrue: [ aBlock value ]
		ifFalse: [
			semaphore critical: [
				owner := process.
				aBlock ensure: [ owner := nil ] ] ]
!

terminateOwner
	owner isNil ifFalse: [ GRPlatform current terminateProcess: owner ]
! !

