'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WARenderContext
	instanceVariableNames:'visitor actionUrl actionBaseUrl resourceUrl callbacks document
		properties'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Rendering'
!

WARenderContext comment:'A RenderContext provides all the state needed by a Renderer to do its work.
It is a decoupling device between whatever is initiating rendering and the Renderer itself.'
!

!WARenderContext methodsFor:'accessing'!

absoluteUrlForResource: aString 
	| url |
	aString isNil ifTrue: [ ^ nil ].
	url := self resourceUrl.
	(url isNil or: [ aString includesSubString: '://' ]) ifTrue: [ ^ WAUrl absolute: aString ].
	^ url copy addToPath: aString; yourself
!

actionBaseUrl
	"Answer the action URL, that was not modified by #updateUrl:."
	
	^ actionBaseUrl
!

actionBaseUrl: anObject
	actionBaseUrl := anObject seasideUrl
!

actionUrl
	"Answer the action URL that was modified by #updateUrl:."

	^ actionUrl
!

actionUrl: anObject
	actionUrl := anObject seasideUrl
!

callbacks
	^ callbacks
!

document
	"Answer the value of document"

	^ document
!

document: anObject
	"Set the value of document"

	document := anObject
!

document: aDocument during: aBlock
	"Set the value of document for the duration of aBlock"
	| previous |
	previous := document.
	[ document := aDocument.
	aBlock value ]
		ensure: [ document := previous ]
!

properties
	"Answer a property dictionary."
	^ properties ifNil: [ properties := GRSmallDictionary new ]
!

resourceUrl
	^ resourceUrl
!

resourceUrl: anObject
	resourceUrl := anObject seasideUrl
!

visitor
	^ visitor ifNil: [ visitor := self defaultVisitor ]
!

visitor: anObject
	visitor := anObject
! !

!WARenderContext methodsFor:'callbacks'!

callbackAt: aString
	^ callbacks at: aString ifAbsent: [ ]
! !

!WARenderContext methodsFor:'copying'!

postCopy
	super postCopy.
	visitor := visitor copyForRenderContext: self.
	actionUrl := actionUrl copy.
	resourceUrl := resourceUrl copy.
	properties := properties copy
! !

!WARenderContext methodsFor:'deprecated'!

advanceKey
	self 
		greaseDeprecatedApi: 'WARenderContext>>#advanceKey'
		details: 'Use WACallbackRegistry>>#advanceKey. The callback registry is available from WARenderContext or WARenderCanvas by sending #callbacks.'.
	^ self callbacks advanceKey
!

callbacksFor: anObject 
	self 
		greaseDeprecatedApi: 'WARenderContext>>#callbacksFor:'
		details: 'Use #callbacks. Callbacks are no longer evaluated in the context of an owning object.'.
	^ self callbacks
!

increaseKey
	self 
		greaseDeprecatedApi: 'WARenderContext>>#increaseKey'
		details: 'Use WACallbackRegistry>>increaseKey. The callback registry is available from WARenderContext or WARenderCanvas by sending #callbacks.'.
	^ self callbacks increaseKey
!

nextKey
	self 
		greaseDeprecatedApi: 'WARenderContext>>#nextKey'
		details: 'Use WACallbackRegistry>>nextKey. The callback registry is available from WARenderContext or WARenderCanvas by sending #callbacks.'.
	^ self callbacks nextKey
!

session
	self 
		greaseDeprecatedApi: 'WARenderContext>>#session'
		details: 'The session is no longer stored in the render context. You can get at the session in the current request context by calling "self session" from any subclass of WAObject.'.
	^ super session
!

session: aSession 
	self 
		greaseDeprecatedApi: 'WARenderContext>>#session:'
		details: 'The session is no longer stored in the render context.'
! !

!WARenderContext methodsFor:'documents'!

urlForDocument: anObject
	^ self urlForDocument: anObject mimeType: nil
!

urlForDocument: anObject mimeType: mimeString
	^ self urlForDocument: anObject mimeType: mimeString fileName: nil
!

urlForDocument: anObject mimeType: mimeType fileName: fileName
	| handler |
	handler := (WADocumentHandler
		document: anObject
		mimeType: mimeType
		fileName: fileName).
	^ self requestContext registry
		register: handler;
		urlFor: handler
! !

!WARenderContext methodsFor:'initialization'!

destroy
	self document destroy.
	document := nil
!

initialize
	super initialize.
	callbacks := WACallbackRegistry new
! !

!WARenderContext methodsFor:'private'!

defaultVisitor
	^ WARenderVisitor context: self
! !

