'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WARequestCookie
	instanceVariableNames:'key path domain ports value version'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WARequestCookie comment:'A WARequestCookie is the cookie the user agent sent to the server.
Instance Variables
	domain:		<String>
	key:		<String>
	path:		<String>
	ports:		<Collection<Integer>>
	value:		<String>
	version:		<Integer>
domain
	- xxxxx
key
	- xxxxx
path
	- xxxxx
ports
	- xxxxx
value
	- xxxxx
version
	- the version of the cookie specification supported, currently only 1 is known
'
!

!WARequestCookie class methodsFor:'instance creation'!

fromString: aString 
	"Creates an array of cookies from a HTTP Cookie: header value."
	| version tokens cookieLine current |
	cookieLine := aString.
	aString isEmptyOrNil
		ifTrue: [ ^ #() ].
	tokens := cookieLine subStrings: ';'.
	current := nil.
	^ Array streamContents: [ :cookies | 
		tokens do: [ :each | 
			| key value action |
			key := self keyFrom: each.
			value := self valueFrom: each.
			key = '$Version' 
				ifTrue: [
					version := value greaseInteger ]
				ifFalse: [
					action := self actions 
						at: key
						ifAbsent: [ nil ].
					action isNil 
						ifTrue: [
							current := self 
								key: key
								value: value.
							current version: version.
							cookies nextPut: current ]
						ifFalse: [
							action 
								value: current
								value: value ] ] ] ]
!

key: keyString value: valueString
	^ self new
		key: keyString;
		value: valueString;
		yourself
! !

!WARequestCookie class methodsFor:'accessing'!

actions
	^ Dictionary new
		at: '$Domain' put: [ :cookie :value | cookie domain: value ];
		at: '$Path' put: [ :cookie :value | cookie path: value ];
		at: '$Port' put: [ :cookie :value | cookie initializePorts: value ];
		yourself
! !

!WARequestCookie class methodsFor:'private'!

keyFrom: aString
	^ (aString copyUpTo: $=) trimBoth
!

trimBlanksAndQuotes: aString
	| trimmed |
	trimmed := aString trimBoth.
	^ (trimmed size > 2
		and: [ trimmed first = $"
		and: [ trimmed last = $" ] ])
			ifTrue: [ trimmed copyFrom: 2 to: trimmed size - 1 ]
			ifFalse: [ trimmed ]
!

valueFrom: aString
	^ (aString includes: $=)
		ifTrue: [ self trimBlanksAndQuotes: (aString copyAfter: $=) ]
! !

!WARequestCookie methodsFor:'accessing'!

domain
	^ domain
!

domain: aString
	domain := aString
!

key
	^ key
!

key: aString
	key := aString
!

path
	^ path ifNil: [ '/' ]
!

path: aString
	path := aString
!

port: anInteger
	self ports: (Array with: anInteger)
!

ports
	^ ports
!

ports: aCollection
	ports := aCollection
!

value
	^ value
!

value: aString
	value := aString
!

version
	^ version
!

version: anInteger
	version := anInteger
! !

!WARequestCookie methodsFor:'comparing'!

= other
	"cookies are identified by the triple name/domain/path"
	^ self species = other species
		and: [ self key = other key
		and: [ self domain = other domain
		and: [ self path = other path ] ] ]
!

hash
	"see #="

	^ (self key hash bitXor: self domain hash) bitXor: self path hash
! !

!WARequestCookie methodsFor:'copying'!

postCopy
	super postCopy.
	ports := ports copy
! !

!WARequestCookie methodsFor:'initialization'!

initializePorts: aString 
	self ports: ((aString subStrings: ',') collect: [ :each | each greaseInteger ])
! !

