'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:45'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAResponse
	instanceVariableNames:'headers cookies status message'
	classVariableNames:'StatusMessages'
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WAResponse comment:'A WAResponse is an abstract HTTP response objects. It is independent of the used server adapter.
Instance Variables
	status:		<Integer> The HTTP status code.
	message:	<String> The HTTP status message.
	headers:	<WAHeaderFields> A dictionary with the header fields.
	cookies:	<OrderedCollection> A collection of cookies.
'
!

!WAResponse class methodsFor:'initialization'!

initialize
	self initializeStatusMessages
!

initializeStatusMessages
	StatusMessages := Dictionary new.
	StatusMessages
		at: self statusContinue put: 'Continue';
		at: self statusSwitchingProtocols put: 'Switching Protocols';
		at: self statusProcessing put: 'Processing';
		at: self statusOk put: 'OK';
		at: self statusCreated put: 'Created';
		at: self statusAccepted put: 'Accepted';
		at: self statusNonAuthoritativeInformation put: 'Non-Authoritative Information';
		at: self statusNoContent put: 'No Content';
		at: self statusResetContent put: 'Reset Content';
		at: self statusPartialContent put: 'Partial Content';
		at: self statusMultiStatus put: 'Multi-Status';
		at: self statusMultipleChoices put: 'Multiple Choices';
		at: self statusMovedPermanently put: 'Moved Permanently';
		at: self statusFound put: 'Found';
		at: self statusSeeOther put: 'See Other';
		at: self statusNotModified put: 'Not Modified';
		at: self statusUseProxy put: 'Use Proxy';
		at: self statusTemporaryRedirect put: 'Temporary Redirect';
		at: self statusBadRequest put: 'Bad Request';
		at: self statusUnauthorized put: 'Unauthorized';
		at: self statusPaymentRequired put: 'Payment Required';
		at: self statusForbidden put: 'Forbidden';
		at: self statusNotFound put: 'Not Found';
		at: self statusMethodNotAllowed put: 'Method Not Allowed';
		at: self statusNotAcceptable put: 'Not Acceptable';
		at: self statusProxyAuthenticationRequired put: 'Proxy Authentication Required';
		at: self statusRequestTimeout put: 'Request Timeout';
		at: self statusConflict put: 'Conflict';
		at: self statusGone put: 'Gone';
		at: self statusLengthRequired put: 'Length Required';
		at: self statusPreconditionFailed put: 'Precondition Failed';
		at: self statusRequestEntityTooLarge put: 'Request Entity Too Large';
		at: self statusRequestUriTooLong put: 'Request-URI Too Long';
		at: self statusUnsupportedMediaType put: 'Unsupported Media Type';
		at: self statusRequestedRangeNotSatisfiable put: 'Requested Range Not Satisfiable';
		at: self statusExpectationFailed put: 'Expectation Failed';
		at: self statusImATeapot put: 'I''m a teapot';
		at: self statusUnprocessableEntity put: 'Unprocessable Entity';
		at: self statusLocked put: 'Locked';
		at: self statusFailedDependency put: 'Failed Dependency';
		at: self statusInternalServerError put: 'Internal Server Error';
		at: self statusNotImplemented put: 'Not Implemented';
		at: self statusBadGateway put: 'Bad Gateway';
		at: self statusServiceUnavailable put: 'Service Unavailable';
		at: self statusGatewayTimeout put: 'Gateway Timeout';
		at: self statusHttpVersionNotSupported put: 'HTTP Version Not Supported';
		at: self statusInsufficientStorage put: 'Insufficient Storage'
! !

!WAResponse class methodsFor:'accessing'!

messageForStatus: anInteger
	^ StatusMessages at: anInteger ifAbsent: [ nil ]
! !

!WAResponse class methodsFor:'status'!

statusAccepted
	^ 202
!

statusBadGateway
	^ 502
!

statusBadRequest
	^ 400
!

statusConflict
	^ 409
!

statusContinue
	^ 100
!

statusCreated
	^ 201
!

statusExpectationFailed
	^ 417
!

statusFailedDependency
	^ 424
!

statusForbidden
	^ 403
!

statusFound
	^ 302
!

statusGatewayTimeout
	^ 504
!

statusGone
	^ 410
!

statusHttpVersionNotSupported
	^ 505
!

statusImATeapot
	^ 418
!

statusInsufficientStorage
	^ 507
!

statusInternalServerError
	^ 500
!

statusLengthRequired
	^ 411
!

statusLocked
	^ 423
!

statusMethodNotAllowed
	^ 405
!

statusMovedPermanently
	^ 301
!

statusMultiStatus
	^ 207
!

statusMultipleChoices
	^ 300
!

statusNoContent
	^ 204
!

statusNonAuthoritativeInformation
	^ 203
!

statusNotAcceptable
	^ 406
!

statusNotFound
	^ 404
!

statusNotImplemented
	^ 501
!

statusNotModified
	^ 304
!

statusOk
	^ 200
!

statusPartialContent
	^ 206
!

statusPaymentRequired
	^ 402
!

statusPreconditionFailed
	^ 412
!

statusProcessing
	^ 102
!

statusProxyAuthenticationRequired
	^ 407
!

statusRequestEntityTooLarge
	^ 413
!

statusRequestTimeout
	^ 408
!

statusRequestUriTooLong
	^ 414
!

statusRequestedRangeNotSatisfiable
	^ 416
!

statusResetContent
	^ 205
!

statusSeeOther
	^ 303
!

statusServiceUnavailable
	^ 503
!

statusSwitchingProtocols
	^ 101
!

statusTemporaryRedirect
	^ 307
!

statusUnauthorized
	^ 401
!

statusUnprocessableEntity
	^ 422
!

statusUnsupportedMediaType
	^ 415
!

statusUseProxy
	^ 305
! !

!WAResponse methodsFor:'accessing'!

cookies
	"Answer the cookies of this response."

	^ cookies
!

headers
	"Answer the headers of this response."
	
	^ headers
!

message
	"Answer the message associated with this response."

	^ message
!

status
	"Answer the status code associated with this response."

	^ status
!

stream
	"Answer a write stream, this is where contents should be written to."
	
	self subclassResponsibility
! !

!WAResponse methodsFor:'accessing-cookies'!

addCookie: aCookie
	"Add the cookie to the receiver."

	^ self cookies add: aCookie
!

deleteCookie: aCookie 
	"Try to get the path from the closest entry point in the context. This path
	is used in case the browser did not provide the path information in the cookie.
	Trouble is, this method may be called in a context where WACurrentRequestContext
	is not defined or where there is no RequestHandler. In this case, we just use nil and
	pray the cookie has path information or was created without any."
	
	| baseUrl |
	baseUrl := [ self requestContext handler url ] 
		on: WARequestContextNotFound
		do: [ :err | nil ].
	self 
		deleteCookie: aCookie
		path: (baseUrl isNil ifFalse: [ baseUrl pathString ])
		domain: (baseUrl isNil ifFalse: [ baseUrl host ])
!

deleteCookie: aCookie path: aPathString domain: aDomainString
	"Delete the cookie in the browser.
	
	The spec sez:
	If a CGI script wishes to delete a cookie, it can do so by returning a cookie with the same name, and an expires time which is in the past. The path and name must match exactly in order for the expiring cookie to replace the valid cookie. This requirement makes it difficult for anyone but the originator of a cookie to delete a cookie.
	
	What is not specified but also has to match is the domain.
	
	The trouble is:
	Browsers that follow the Netscape spec don't send path or domain information"
	| copy |
	copy := (WACookie 
		key: aCookie key
		value: '')
		path: aCookie path;
		version: aCookie version;
		domain: aCookie domain;
		yourself.
	aCookie version isNil ifTrue:  [
		"old Netscape cookie with no path or domain information"
		aPathString isNil ifFalse: [ 
			copy path: aPathString ].
		aDomainString isNil ifFalse: [
			copy domain: aDomainString ] ].
	copy expireInPast.
	self addCookie: copy
!

deleteCookieAt: aKeyString
	"Delete the cookie in the browser. This method is quite unreliable. Consider using #deleteCookie:"
	self deleteCookie: (WARequestCookie key: aKeyString value: '')
! !

!WAResponse methodsFor:'accessing-headers'!

headerAt: aString
	"Answer the header with aString."
	
	^ self headers at: aString
!

headerAt: aString append: aValue 
	"Appends a response header with aString and aValue. This method allows response headers to have multiple values."
	^ self headers 
		at: aString
		add: aValue greaseString
!

headerAt: aString ifAbsent: aBlock
	"Answer the header with aString, if none answer the result of evaluating aBlock."
	
	^ self headers at: aString ifAbsent: aBlock
!

headerAt: aString put: aValue
	"Set a response header with aString and aValue. If the header had already been set, the new value overwrites the previous one."
	
	^ self headers at: aString put: aValue
!

headerAt: aKeyString putDate: aDateAndTime 
	"Convenience method that sets a response header with aKeyString and aDateAndTime. If the header had already been set, the new value overwrites the previous one."
	^ self 
		headerAt: aKeyString
		put: (GRPrinter httpDate print: aDateAndTime)
!

removeHeaderAt: aString
	"Removes all occurences of the header value aString."

	^ self headers removeKey: aString
! !

!WAResponse methodsFor:'accessing-status'!

status: aNumber
	"Set the status to aNumber and associate the default message."

	self status: aNumber message: (self class messageForStatus: aNumber)
!

status: aNumber message: aString
	"Set the status to aNumber and the message to aString."
	
	status := aNumber.
	message := aString
! !

!WAResponse methodsFor:'configuration'!

cacheForeverExpiryDate
	"Answer the expiry date that should be used if the response should be cached forever.
	
	Note that according to HTTP 1.1 says: To mark a response as 'never expires,' an origin server sends an Expires date approximately one year from the time the response is sent. HTTP/1.1 servers SHOULD NOT send Expires dates more than one year in the future."
	
	^ (DateAndTime now) + (Duration days: 300 hours: 0 minutes: 0 seconds: 0)
!

doNotCacheExpiryDate
	"Answer the expiry date that should be used if the response should not be cached at all."
	
	^ DateAndTime year: 2000 month: 1 day: 1 hour: 0 minute: 0 second: 0
!

httpVersion
	"Answer the HTTP version to be used."

	^ 'HTTP/1.1'
! !

!WAResponse methodsFor:'convenience'!

respond
	WAResponseNotification signal
! !

!WAResponse methodsFor:'convenience-accessing'!

contentType
	^ self headerAt: 'Content-Type' ifAbsent: [ WAMimeType textPlain ]
!

contentType: aMimeType
	self headerAt: 'Content-Type' put: aMimeType
! !

!WAResponse methodsFor:'convenience-caching'!

cacheFor: aDuration
	"Configure the receiver to be cached for aDuration."

	self removeHeaderAt: 'Pragma'. "Proxies"
	self headerAt: 'Expires' putDate: DateAndTime now + aDuration. "HTTP 1.0"
	self headerAt: 'Cache-Control' put: 'max-age=' , aDuration asSeconds printString. "HTTP 1.1"
!

cacheForever
	"Configure the response to be cached by user agents."
	
	self removeHeaderAt: 'Pragma'. "Proxies"
	self headerAt: 'Expires' putDate: self cacheForeverExpiryDate. "HTTP 1.0"
	self removeHeaderAt: 'Cache-Control'. "HTTP 1.1"
!

doNotCache
	"Configure the receiver to not be cached at all."
	
	self headerAt: 'Pragma' put: 'no-cache'. "Proxies"
	self headerAt: 'Expires' putDate: self doNotCacheExpiryDate. "HTTP 1.0"
	self headerAt: 'Cache-Control' put: 'no-cache, must-revalidate'. "HTTP 1.1"
	
! !

!WAResponse methodsFor:'convenience-headers'!

attachmentWithFileName: aString 
	aString notNil ifTrue:  [
		self 
			headerAt: 'Content-Disposition'
			put: 'attachment; filename="' , aString greaseString , '"' ]
!

inlineWithFileName: aString 
	aString notNil ifTrue: [
		self 
			headerAt: 'Content-Disposition'
			put: 'inline; filename="' , aString greaseString , '"' ]
!

location: aUrlOrString 
	self 
		headerAt: 'Location'
		put: aUrlOrString greaseString
!

strictTransportSecurity: maxAge includeSubDomains: aBoolean
	"Implements Strict Transport Security (STS).
		http://lists.w3.org/Archives/Public/www-archive/2009Dec/att-0048/draft-hodges-strict-transport-sec-06.plain.html
	Use only over secure connections (HTTPS)."
	
	self
		headerAt: 'Strict-Transport-Security'
		put: 'max-age=', maxAge greaseString,
			(aBoolean ifTrue: [ '; includeSubDomains' ] ifFalse: [ '' ])
! !

!WAResponse methodsFor:'convenience-status'!

basicAuthenticationRealm: aString 
	self 
		headerAt: 'WWW-Authenticate'
		put: 'Basic realm="' , aString greaseString , '"'.
	self authenticationFailed
!

redirectTo: aUrlOrString
	self location: aUrlOrString.
	self found
!

refreshAfter: aDuration location: aUrl 
	self 
		headerAt: 'Refresh'
		put: aDuration asSeconds greaseString , '; ' , aUrl greaseString
! !

!WAResponse methodsFor:'documents'!

document: aDocument
	| document mimeType |
	document := aDocument seasideMimeDocument.
	self attachmentWithFileName: document fileName.
	mimeType := document mimeType.
	self contentType: mimeType.
	mimeType isBinary
		ifTrue: [ self binary ].
	self nextPutAll: document content
!

document: anObject mimeType: aMimeType
	self document: (anObject seasideMimeDocument 
		mimeType: aMimeType;
		yourself)
!

document: anObject mimeType: aMimeType fileName: aString
	self document: (anObject seasideMimeDocument 
		mimeType: aMimeType;
		fileName: aString;
		yourself)
! !

!WAResponse methodsFor:'initialization'!

destroy
	headers := cookies := nil
!

initialize
	super initialize.
	status := 200.
	message := 'OK'.
	headers := WAHeaderFields new.
	cookies := OrderedCollection new
!

initializeOn: aStream 
	self initialize
! !

!WAResponse methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: ' status: '; print: self status.
	aStream nextPutAll: ' message: '; print: self message
! !

!WAResponse methodsFor:'status'!

authenticationFailed
	"Indicate that the request requires HTTP authentication."
	
	self status: WAResponse statusUnauthorized
!

badRequest
	"The request could not be understood by the server due to malformed syntax. The client SHOULD NOT repeat the request without modifications."
	
	self status: WAResponse statusBadRequest
!

created
	"The request has been fulfilled and resulted in a new resource being created."
	
	self status: WAResponse statusCreated
!

forbidden
	"Indicate that the server understood the request but refused to fulfill it."
	
	self status: WAResponse statusForbidden
!

found
	"Indicate that the resource reside temporarily under a different URI."
	
	self status: WAResponse statusFound
!

gone
	"Indicate that the resource is no longer available at the server and no forwarding address is known."
	
	self status: WAResponse statusGone
!

internalError
	"Indicate an error inside the server which prevented it from fulfilling the request."

	self status: WAResponse statusInternalServerError
!

movedPermanently
	"The requested resource has been assigned a new permanent URI and any future references to this resource SHOULD use one of the returned URIs."
	
	self status: WAResponse statusMovedPermanently
!

notFound
	"Indicate that the requested resource is not available."
	
	self status: WAResponse statusNotFound
!

notImplemented
	"Indicate the HTTP server does not support the functionality needed to fulfill the request."
	
	self status: WAResponse statusNotImplemented
!

preconditionFailed
	"The precondition given in one or more of the request-header fields evaluated to false when it was tested on the server. This response code allows the client to place preconditions on the current resource metainformation (header field data) and thus prevent the requested method from being applied to a resource other than the one intended."
	
	self status: WAResponse statusPreconditionFailed
! !

!WAResponse methodsFor:'streaming'!

binary
	"put the response stream into binary mode"
	
	self subclassResponsibility
!

nextPutAll: aStringOrByteArray
	"Append the elements of aStringOrByteArray to the body part of the response."
	
	self stream nextPutAll: aStringOrByteArray
! !

!WAResponse methodsFor:'testing'!

isCommitted
	"Answer a boolean indicating if the response header has been committed."
	
	^ false
! !

!WAResponse methodsFor:'writing'!

writeContentOn: aStream
	self subclassResponsibility
!

writeCookiesOn: aStream
	cookies do: [ :each | 
		aStream nextPutAll: 'Set-Cookie: '.
		each writeOn: aStream.
		aStream crlf ]
!

writeHeadersOn: aStream 
	headers keysAndValuesDo: [ :key :value | 
		aStream
			nextPutAll: key;
			nextPutAll: ': ';
			nextPutAll: value greaseString;
			crlf ]
!

writeOn: aStream
	self writeStatusOn: aStream.
	self writeHeadersOn: aStream.
	self writeCookiesOn: aStream.
	aStream crlf.
	self writeContentOn: aStream
!

writeStatusOn: aStream 
	aStream nextPutAll: self httpVersion.
	aStream
		nextPut: $ ;
		nextPutAll: self status greaseString.
	self message isNil ifFalse:  [
		aStream
			nextPut: $ ;
			nextPutAll: self message ].
	aStream crlf
! !


WAResponse initialize!
