'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:41'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAResponseGenerator
	instanceVariableNames:'requestContext'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-RequestHandling'
!

WAResponseGenerator comment:'The response generator configures the response of the provided request context according to the called generation method. This class can be subclassed to provide custom response templates.'
!

!WAResponseGenerator class methodsFor:'instance creation'!

on: aRequestContext
	^ self basicNew initializeOn: aRequestContext
! !

!WAResponseGenerator methodsFor:'accessing'!

request
	"Answer the current request."

	^ self requestContext request
!

requestContext
	"Answer the current request context."

	^ requestContext
!

response
	"Answer the current response."

	^ self requestContext response
! !

!WAResponseGenerator methodsFor:'actions'!

respond
	"Tell the current request context to return the response."

	self requestContext respond
! !

!WAResponseGenerator methodsFor:'generation'!

authenticate: aString
	"Asks for authentication to the domain aString."

	self response basicAuthenticationRealm: aString
!

expiredRegistryKey
	"The session key has expired, redirect the request to the home directory preserving the path as good as possible."

	| url |
	self request isXmlHttpRequest
		ifTrue: [ ^ self forbidden ].
	url := self requestContext registry url.
	url addAllToPath: self requestContext consumer upToEnd.
	self request fields keysAndValuesDo: [ :key :value |
		(url isSeasideField: key) 
			ifFalse: [ url addField: key value: value ] ].
	self response redirectTo: url
!

forbidden
	"The resource is forbidden to access and the request was not processed."
	self response
		forbidden;
		contentType: WAMimeType textPlain;
		nextPutAll: 'Error: you are forbidden to access "';
		nextPutAll: self request url pathString;
		nextPutAll: '"'
!

internalError: anException
	"An internal error occurred while processing the request."

	self response 
		internalError;
		contentType: WAMimeType textPlain;
		nextPutAll: 'Internal Error: ';
		nextPutAll: (anException messageText ifNil: [ String new ])
!

notFound
	"The resource could not be found and the request processing was aborted."
	self response
		notFound;
		contentType: WAMimeType textPlain;
		nextPutAll: self request url pathString;
		nextPutAll: ' not found'
!

notImplemented
	"The functionality requested is not implemented and the request processing was aborted."

	self response 
		notImplemented;
		contentType: WAMimeType textPlain;
		nextPutAll: 'Not Implemented'
!

pageExpired
	"The page is no longer in the cache and the request could not be processed."

	self response
		doNotCache;
		contentType: WAMimeType textPlain;
		nextPutAll: 'This page has expired, please use the back-button to navigate to the last valid page.'
!

redirectTo: aUrlOrString
	"Redirect to a new location."
	
	self response redirectTo: aUrlOrString
!

refreshAfter: aDuration location: aUrl message: aString 
	"Show aString for aDuration, then redirect to aUrl."
	self response
		contentType: WAMimeType textPlain;
		refreshAfter: aDuration
			location: aUrl
			message: aString , (String with: Character cr) , 'You are being redirected to ' , aUrl greaseString
! !

!WAResponseGenerator methodsFor:'initialization'!

initializeOn: aRequestContext
	self initialize.
	requestContext := aRequestContext
! !

