'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAServerAdaptor
	instanceVariableNames:'manager port requestHandler codec'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Server'
!

WAServerAdaptor comment:'A WAServer is the abstract base class for all servers. Actual servers do not have to subclass it but have to support the protocol:
 - #codec
 - #usesSmalltalkEncoding
Instance Variables
	codec:		<WACodec>
codec
	- the codec used for response conversion from characters to bytes'
!

!WAServerAdaptor class methodsFor:'instance creation'!

manager: aServerManager
	^ aServerManager register: 
		(self basicNew
			initializeWithManager: aServerManager;
			yourself)
!

new
	^ self manager: WAServerManager default
!

port: anInteger
	^ self new
		port: anInteger;
		yourself
! !

!WAServerAdaptor class methodsFor:'testing'!

isAbstract
	^ self name = #WAServerAdaptor
! !

!WAServerAdaptor methodsFor:'accessing'!

codec
	^ codec
!

codec: aCodec
	codec := aCodec
!

port
	^ port
!

port: anObject
	port := anObject
!

requestHandler
	"We don't set the instance variable here, if nil, because the default
	request handler may change over time and we want to keep using
	the current default if the user hasn't specified one."
	^ requestHandler ifNil: [ self defaultRequestHandler ]
!

requestHandler: aRequestHandler
	requestHandler := aRequestHandler
! !

!WAServerAdaptor methodsFor:'accessing-readonly'!

additionalStatusOn: aStream
!

manager
	^ manager
!

statusString
	^ self isRunning
		ifTrue: [ 'running' ]
		ifFalse: [ 'stopped' ]
! !

!WAServerAdaptor methodsFor:'converting'!

contextFor: aNativeRequest
	"Answer a request context for aNativeRequest."

	^ WARequestContext
		request: (self requestFor: aNativeRequest)
		response: (self responseFor: aNativeRequest)
		codec: self codec
!

requestFor: aNativeRequest
	"Answer a request object for aNativeRequest."

	^ (WARequest
		method: (self requestMethodFor: aNativeRequest)
		uri: (self requestUrlFor: aNativeRequest)
		version: (self requestVersionFor: aNativeRequest))
			setHeaders: (self requestHeadersFor: aNativeRequest);
			setBody: (self requestBodyFor: aNativeRequest);
			setPostFields: (self requestFieldsFor: aNativeRequest);
			setCookies: (self requestCookiesFor: aNativeRequest);
			setRemoteAddress: (self requestAddressFor: aNativeRequest);
			yourself
!

responseFor: aNativeRequest
	"Answer a response object for aNativeRequest."

	^ WABufferedResponse on: (self codec encoderFor: (WriteStream on: String new))
!

responseFrom: aRequestContext
	"Answer a native response from aRequestContext."

	self subclassResponsibility
! !

!WAServerAdaptor methodsFor:'converting-request'!

requestAddressFor: aNativeRequest
	self subclassResponsibility
!

requestBodyFor: aNativeRequest
	"Answer the undecoded request body as a String."
	
	self subclassResponsibility
!

requestCookiesFor: aNativeRequest
	self subclassResponsibility
!

requestFieldsFor: aNativeRequest
	self subclassResponsibility
!

requestHeadersFor: aNativeRequest
	self subclassResponsibility
!

requestMethodFor: aNativeRequest
	self subclassResponsibility
!

requestUrlFor: aNativeRequest
	self subclassResponsibility
!

requestVersionFor: aNativeRequest
	self subclassResponsibility
! !

!WAServerAdaptor methodsFor:'defaults'!

defaultCodec
	^ GRNullCodec new
!

defaultPort
	^ 8080
!

defaultRequestHandler
	^ WADispatcher default
! !

!WAServerAdaptor methodsFor:'initialization'!

initialize
	super initialize.
	codec := self defaultCodec.
	port := self defaultPort.
!

initializeWithManager: aServerManager
	self initialize.
	manager := aServerManager
! !

!WAServerAdaptor methodsFor:'private'!

basicStart
	self subclassResponsibility
!

basicStop
	self subclassResponsibility
!

shutDown
!

startUp
! !

!WAServerAdaptor methodsFor:'processing'!

handle: aRequestContext
	"Handle the basic request-response loop."

	self handleRequest: aRequestContext.
	self handlePadding: aRequestContext
!

handlePadding: aRequestContext
	"If you create an error page that's too small, Microsoft Internet Explorer will display its own error page that includes a link to the MSN network search engine rather than your own error page. See <http://www.404-error-page.com/404-error-page-too-short-problem-microsoft-ie.shtml> for details."

	| size |
	size := aRequestContext response stream size.
	^ (size < 512 and: [ #(400 403 404 405 406 408 409 410 500 501 505) includes: aRequestContext response status ])
		ifTrue: [ aRequestContext response nextPutAll: ((String new: 512 - size) atAllPut: $ ) ]
!

handleRequest: aRequestContext
	"Pass the request context to the appropriate request handler."
	
	[ self requestHandler handle: aRequestContext ]
		on: WAResponseNotification
		do: [ :n | "got a response" ]
!

process: aNativeRequest
	"Process aNativeRequest by converting to a request context, handling the request, and converting the request context back to a native response. Make sure to release the context after processing, to help the garbage collector and avoid having large streams and sockets hanging around if people still refer to the obsolete request context."

	| context |
	context := self contextFor: aNativeRequest.
	^ [ self handle: context; responseFrom: context ]
		ensure: [ context destroy ]
! !

!WAServerAdaptor methodsFor:'registration'!

unregister
	self manager unregister: self
!

unregistered
	manager := nil
! !

!WAServerAdaptor methodsFor:'starting/stopping'!

kill
	self basicStop
!

restart
	self manager restart: self
!

start
	self manager start: self
!

stop
	self manager stop: self
! !

!WAServerAdaptor methodsFor:'testing'!

canStart
	^ self manager canStart: self
!

canStop
	^ self manager canStop: self
!

isRunning
	self subclassResponsibility
!

isStarting
	self subclassResponsibility
!

isStopped
	self subclassResponsibility
!

isStopping
	self subclassResponsibility
! !

