'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAResponse subclass:#WAStreamedResponse
	instanceVariableNames:'committed externalStream'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WAStreamedResponse comment:'A WAStreamedResponse is a HTTP response that directly writes to an external write stream. This response class is used to implement efficient HTTP response streaming, as it can directly write do the socket while content is still generated.
Instance Variables
	committed:		<Boolean>	Whether the status and header was written to the stream.
	externalStream:		<WriteStream>	The external stream to write to.'
!

!WAStreamedResponse class methodsFor:'instance creation'!

on: aStream
	^ self basicNew initializeOn: aStream
! !

!WAStreamedResponse methodsFor:'accessing'!

stream
	self flush.
	^ externalStream
! !

!WAStreamedResponse methodsFor:'accessing-cookies'!

addCookie: aCookie
	self committedCheck.
	^ super addCookie: aCookie
! !

!WAStreamedResponse methodsFor:'accessing-headers'!

headerAt: aString append: aValue
	self committedCheck.
	^ super headerAt: aString append: aValue
!

headerAt: aString put: aValue
	self committedCheck.
	^ super headerAt: aString put: aValue
!

removeHeaderAt: aString
	self committedCheck.
	^ super removeHeaderAt: aString
! !

!WAStreamedResponse methodsFor:'accessing-status'!

status: aNumber message: aString
	self committedCheck.
	super status: aNumber message: aString
! !

!WAStreamedResponse methodsFor:'initialization'!

destroy
	super destroy.
	externalStream := nil
!

initializeOn: aStream 
	super initializeOn: aStream.
	committed := false.
	externalStream := aStream
! !

!WAStreamedResponse methodsFor:'private'!

committedCheck
	"Raises a signal if this response is already committed."

	self isCommitted 
		ifTrue: [ WAIllegalStateException signal ]
! !

!WAStreamedResponse methodsFor:'public'!

flush
	"Flush the status code and header fields, if they haven't been written yet."

	self isCommitted
		ifFalse: [ self writeOn: externalStream ].
	externalStream flush
! !

!WAStreamedResponse methodsFor:'streaming'!

binary
	"put the response stream into binary mode"

	externalStream binary
! !

!WAStreamedResponse methodsFor:'testing'!

isCommitted
	^ committed
! !

!WAStreamedResponse methodsFor:'writing'!

writeContentOn: aStream 
	committed := true
! !

