"{ Package: 'stx:goodies/seaside30/core' }"!

!ByteArray methodsFor:'*seaside-core'!

renderOn: aRenderer
	aRenderer text: self
! !

!ByteArray methodsFor:'*seaside-core'!

seasideMimeDocument
	^ WAMimeDocument on: self mimeType: WAMimeType applicationOctetStream
! !

!ByteArray methodsFor:'*seaside-core'!

seasideMimeDocumentType: aMimeType
	^ self seasideMimeDocument
		mimeType: aMimeType;
		yourself
! !

!Collection methodsFor:'*seaside-core'!

renderOn: html
	self do: [ :each | html render: each ]
! !

!Dictionary methodsFor:'*seaside-core'!

seasideRequestFields
	^ WARequestFields withAll: self
! !

!Dictionary methodsFor:'*seaside-core'!

valuesCollect: aBlock
	| result |
	result := self species new: self size.
	self keysAndValuesDo: [ :key :value |
		result at: key put: (aBlock value: value) ].
	^ result
! !

!Exception methodsFor:'*seaside-core'!

seasidePossibleCauses
	"Used by WAWalkback to render a list of what might have caused the receiver to be signaled.
	answers a collection of strings"
	^ #()
! !

!GRInvalidUtf8Error methodsFor:'*seaside-core'!

seasidePossibleCauses
	^ #('The input is not UTF-8 but some other encoding like ISO-8859-1.')
! !

!GRPackage class methodsFor:'*seaside-core'!

seasideCore
	^ self new
		name: 'Seaside-Core';
		description: 'The core of Seaside, required for all packages.';
		addDependency: 'Grease-Core';
		url: #seasideUrl;
		yourself
! !

!GRPlatform methodsFor:'*seaside-core'!

seasideVersion
	"Answer the Seaside version"

	^ (GRVersion major: 3 minor: 0 revision: 6)
		yourself
! !

!GRSmallDictionary methodsFor:'*seaside-core'!

seasideRequestFields
	^ WARequestFields withAll: self
! !

!GRUnsupportedEncodingError methodsFor:'*seaside-core'!

seasidePossibleCauses
	^ #(
		'The given encoding name is not supported on this Smalltalk dialect.'
		'The given encoding name is misspelled.'
	)
! !

!MessageNotUnderstood methodsFor:'*seaside-core'!

seasidePossibleCauses
	| causes |
	self receiver isNil ifFalse: [
		^ #('you sent a message this type of object doesn''t understand') ].
	causes := #(
		'the receiver of the message is nil'
		'a class extension hasn''t been loaded correctly'
		'you sent the wrong message' ) asOrderedCollection.
	self message selector = #contents ifTrue: [
		causes addFirst: 'you forgot to send "super initialize" in a initialize method of a component or task' ].
	^ causes
! !

!Object methodsFor:'*seaside-core'!

encodeOn: aDocument 
	aDocument print: self greaseString
! !

!Object methodsFor:'*seaside-core'!

labelForSelector: aSymbol
	^ aSymbol asCapitalizedPhrase
! !

!Object methodsFor:'*seaside-core'!

renderOn: aRenderer
	"This method is called on all objects passed as an argument to the #render: method of a WARenderer subclass. Override this method to customize how objects render themselves. The default is the return value of #displayString.
	Subclasses of WAPainter (including Components) should override #renderContentOn: and #rendererClass instead."

	aRenderer text: self
! !

!String methodsFor:'*seaside-core'!

asCapitalizedPhrase
	| read words currentWord capitalizedWord |
	(self noneSatisfy: [ :ea | ea isLowercase ])
		ifTrue: [ ^ self ].
	words := WriteStream on: String new.
	read := ReadStream on: self.
	[ read atEnd ] whileFalse: [
		currentWord := WriteStream on: String new.
		currentWord nextPut: read next.
		[ | x | x := read peek. x isNil or: [ x isUppercase ] ]
			whileFalse: [ currentWord nextPut: read next ].
		capitalizedWord := currentWord contents capitalized.
			(#(Of In At A Or To By) includes: capitalizedWord)
				ifTrue: [ capitalizedWord := capitalizedWord asLowercase ].
		words nextPutAll: capitalizedWord.
		words nextPutAll: ' ' ].
	words skip: -1.
	^ words contents
! !

!String methodsFor:'*seaside-core'!

encodeOn: aDocument
	aDocument xmlEncoder nextPutAll: self
! !

!String methodsFor:'*seaside-core'!

renderOn: aRenderer
	aRenderer text: self
! !

!String methodsFor:'*seaside-core'!

seasideMimeDocument
	^ WAMimeDocument on: self mimeType: WAMimeType textPlain
! !

!String methodsFor:'*seaside-core'!

seasideMimeDocumentType: aMimeType
	^ self seasideMimeDocument
		mimeType: aMimeType;
		yourself
! !

!String methodsFor:'*seaside-core'!

seasideMimeType
	^ WAMimeType fromString: self
! !

!String methodsFor:'*seaside-core'!

seasideUrl
	^ WAUrl absolute: self
! !

!UndefinedObject methodsFor:'*seaside-core'!

encodeOn: aDocument
! !

!UndefinedObject methodsFor:'*seaside-core'!

renderOn: html
! !

!UndefinedObject methodsFor:'*seaside-core'!

seasideUrl
	^ self
! !

!WADispatcher methodsFor:'parent/child'!

childrenDo: aBlock
	handlers do: aBlock
! !

!WADispatcher methodsFor:'accessing'!

defaultHandler
	self defaultName isNil ifTrue: [ ^ nil ].
	^ self handlerAt: self defaultName ifAbsent: [ nil ]
! !

!WADispatcher methodsFor:'accessing'!

defaultName
	^ defaultName
! !

!WADispatcher methodsFor:'accessing'!

defaultName: aString
	defaultName := aString
! !

!WADispatcher methodsFor:'deprecated'!

entryPointAt: aString 
	self 
		greaseDeprecatedApi: 'WADispatcher>>#entryPointAt:'
		details: 'Use WADispatcher>>#handlerAt:.'.
	^ self handlerAt: aString
! !

!WADispatcher methodsFor:'handling'!

handleDefault: aRequestContext
	self defaultName isNil
		ifTrue: [ aRequestContext responseGenerator forbidden; respond ]
		ifFalse: [ self handleFiltered: aRequestContext named: self defaultName ]
! !

!WADispatcher methodsFor:'handling'!

handleFiltered: aRequestContext
	| consumer |
	consumer := aRequestContext consumer.
	consumer atEnd
		ifTrue: [ self handleDefault: aRequestContext ]
		ifFalse: [ self handleFiltered: aRequestContext named: consumer next ]
! !

!WADispatcher methodsFor:'handling'!

handleFiltered: aRequestContext named: aString
	(self 
		handlerAt: aString 
		ifAbsent: [ aRequestContext responseGenerator notFound; respond ])
			handle: aRequestContext
! !

!WADispatcher methodsFor:'accessing'!

handlerAt: aString
	^ self
			handlerAt: aString
			with: [ :dispatcher :token |
				dispatcher handlers at: token ] 
! !

!WADispatcher methodsFor:'accessing'!

handlerAt: aString ifAbsent: aBlock
	^ self
		handlerAt: (aString
			ifNil: [ ^ aBlock value ])
		with: [ :dispatcher :token |
			dispatcher handlers
				at: token
				ifAbsent: [ ^ aBlock value ] ]
! !

!WADispatcher methodsFor:'accessing'!

handlerAt: aString with: aBlock
	^ (aString subStrings: '/') inject: self into: aBlock
! !

!WADispatcher methodsFor:'accessing'!

handlers
	^ handlers
! !

!WADispatcher methodsFor:'initialization'!

initialize
	super initialize.
	handlers := Dictionary new
! !

!WADispatcher methodsFor:'testing'!

isDispatcher
	^ true
! !

!WADispatcher methodsFor:'accessing'!

nameOfHandler: aRequestHandler 
	"Answer the name of the given request handler. Signal an error the given handler is not registered in the receiver."
	^ handlers
		keyAtValue: aRequestHandler
		ifAbsent: [ WAUnregisteredHandlerError signal ]
! !

!WADispatcher methodsFor:'copying'!

postCopy
	super postCopy.
	handlers := handlers valuesCollect: [ :each | each copy setParent: self; yourself ]
! !

!WADispatcher methodsFor:'parent/child'!

register: aRequestHandler at: aString 
	aString ifNil: [ WAError signal: 'Registration key cannot be nil.' ].
	handlers 
		at: aString
		put: (aRequestHandler setParent: self; yourself).
	^ aRequestHandler
! !

!WADispatcher methodsFor:'parent/child'!

unregister: aRequestHandler
	handlers 
		removeKey: aRequestHandler name
		ifAbsent: [ ^ nil ].
	^ aRequestHandler unregistered
! !

!WADispatcher methodsFor:'accessing'!

urlFor: aRequestHandler
	^ self url
		addToPath: (self nameOfHandler: aRequestHandler);
		yourself
! !

!WADispatcher class methodsFor:'accessing'!

default
	^ default ifNil: [ default := self new ]
! !

!WADispatcher class methodsFor:'accessing'!

description
	^ 'Dispatcher'
! !

!WADispatcher class methodsFor:'initialization'!

resetDefault
	default := nil.
! !

!WAEncoder methodsFor:'initialization'!

initializeOn: aStream table: anArray
	self initialize.
	stream := aStream.
	table := anArray
! !

!WAEncoder methodsFor:'accessing'!

nextPut: aCharacter 
	| value encoded |
	value := aCharacter greaseInteger.
	value < table size 
		ifFalse: [ stream nextPut: aCharacter ]
		ifTrue: [
			encoded := table at: value + 1.
			"Issue 482: use #notNil because it is faster than #isString because it is not actually sent"
			encoded notNil
				ifTrue: [ stream nextPutAll: encoded ]
				ifFalse: [ stream nextPut: aCharacter ] ]
! !

!WAEncoder methodsFor:'accessing'!

nextPutAll: aString 
	"uses #to:do: for speed reasons (on Pharo)
	this is not premature optimization, this is a hotspot method method
	and #to:do: shows measurable speed improvements for rendering seaside pages"
	1 to: aString size do: [ :index |
		| character value encoded |
		character := aString at: index.
		value := character greaseInteger.
		encoded := table at: value + 1.
		"Issue 482: use #notNil because it is faster than #isString because it is not actually sent"
		encoded notNil 
			ifTrue: [ stream nextPutAll: encoded ]
			ifFalse: [ stream nextPut: character ] ]
! !

!WAEncoder class methodsFor:'private'!

encode: aCharacter on: aStream
	self subclassResponsibility
! !

!WAEncoder class methodsFor:'initialization'!

initializeTable
	"Initializes the encoding table."
	| stream characterLimit |
	characterLimit := self maximumCharacterValue.
	"character values at zero so we need to add 1"
	table := Array new: characterLimit + 1.
	stream := WriteStream on: (String new: 6).
	0 to: characterLimit do: [ :index | 
		| character value |
		character := Character codePoint: index.
		self encode: character on: stream reset.
		"Smalltalk indices are one based but character values start at 0"
		value := stream contents = (String with: character)
			ifTrue: [ nil ]
			ifFalse: [ stream contents ].
		table at: index + 1 put: value ]
! !

!WAEncoder class methodsFor:'private'!

maximumCharacterValue
	"find the maximum value of a character that we can instantiate, for Squeak 3.7 this is 255"
	^ [ Character codePoint: 16rFFFF.
		16rFFFF ]
			on: Error
			do: [ :error  | 16rFF ]
! !

!WAEncoder class methodsFor:'instance creation'!

new
	^ self on: GRPlatform current readWriteCharacterStream
! !

!WAEncoder class methodsFor:'instance creation'!

on: aStream
	^ self on: aStream table: table
! !

!WAEncoder class methodsFor:'instance creation'!

on: aStream codec: aCodec
	"For compatibility with WAUrlEncoder"
	^ self on: aStream
! !

!WAEncoder class methodsFor:'instance creation'!

on: aStream table: anArray
	^ self basicNew initializeOn: aStream table: anArray; yourself
! !

!WAFileHandler methodsFor:'accessing'!

baseUrlForLibrary
	| resourceBaseUrl |
	resourceBaseUrl := self resourceBaseUrl.
	^ resourceBaseUrl isNil
		ifTrue: [ self url ]
		ifFalse: [ resourceBaseUrl copy ]
! !

!WAFileHandler methodsFor:'configuration'!

defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARequestHandlingConfiguration instance;
		addParent: WAFileHandlerConfiguration instance;
		yourself
! !

!WAFileHandler methodsFor:'configuration'!

fileHandlerListingClass
	^ self preferenceAt: #fileHandlerListingClass
! !

!WAFileHandler methodsFor:'handling'!

handleFiltered: aRequestContext
	self responseForContext: aRequestContext
! !

!WAFileHandler methodsFor:'accessing'!

libraries
	^ WAFileLibrary allSubclasses
! !

!WAFileHandler methodsFor:'accessing'!

libraryAt: aSymbol ifAbsent: aBlock
	^ self libraries 
		detect: [ :each | each handlesFolder: aSymbol ]
		ifNone: aBlock
! !

!WAFileHandler methodsFor:'configuration'!

resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl
! !

!WAFileHandler methodsFor:'private'!

responseForContext: aRequestContext
	| libraryClass consumer |
	consumer := aRequestContext consumer.
	consumer atEnd
		ifTrue: [ ^ (self fileHandlerListingClass on: self context: aRequestContext) responseForLibraries ].
	libraryClass := self 
		libraryAt: consumer next asSymbol
		ifAbsent: [ ^ aRequestContext responseGenerator notFound; respond ].
	consumer atEnd
		ifTrue: [
			| listingClass |
			listingClass := (self fileHandlerListingClass on: self context: aRequestContext).
			listingClass responseForLibrary: libraryClass default ]
		ifFalse: [
			libraryClass handle: aRequestContext.
			" won't be executed if a document is found "
			aRequestContext responseGenerator
				notFound;
				respond ]
! !

!WAFileHandler methodsFor:'configuration'!

useXSendfile
	^ self preferenceAt: #useXSendfile.
! !

!WAFileHandler methodsFor:'configuration'!

xSendfileBaseDirectory
	^ self preferenceAt: #xSendfileBaseDirectory.
! !

!WAFileHandler class methodsFor:'accessing'!

default

	default isNil ifTrue: [ default := self new ].
	^ default
! !

!WAFileHandler class methodsFor:'accessing'!

default: aFileHandler
	default := aFileHandler
! !

!WAFileHandler class methodsFor:'accessing'!

description
	^ 'File Library'
! !

!WAFileLibrary methodsFor:'actions'!

addFile: aFile 
	"adds a file to the receiver

	aFile an instance of of WAFile"
	| contents |
	aFile isNil ifTrue: [
		"in case no file was selected for uploading"
		^ self ].
	contents := (self class isBinary: aFile fileName) 
		ifTrue: [ aFile rawContents ]
		ifFalse: [
			"#contents is deprecated, we shouldn't send it"
			GRPlatform current convertToSmalltalkNewlines: aFile contents ].
	self class 
		addFileNamed: aFile fileName
		contents: contents
! !

!WAFileLibrary methodsFor:'private'!

asFilename: aSelector 
	| dotIndex extension |
	dotIndex := self lastUpperCaseIndexIn: aSelector.
	dotIndex = 0 ifTrue: [
		"convert Symbol to String"
		^ aSelector greaseString ].
	extension := (aSelector last: aSelector size - dotIndex + 1) asLowercase.
	^ (aSelector first: dotIndex - 1) , '.' , extension
! !

!WAFileLibrary methodsFor:'private'!

asSelector: aFilename
	^ self class asSelector: aFilename
! !

!WAFileLibrary methodsFor:'accessing'!

cacheDuration
	"Answer the duration the response should be cached by the user agent."

	^ Duration
		days: 0
		hours: 0
		minutes: 30
		seconds: 0
! !

!WAFileLibrary methodsFor:'private'!

compileBinary: aString selector: aSymbol
	^ self class compileBinary: aString selector: aSymbol
! !

!WAFileLibrary methodsFor:'private'!

compileText: aString selector: aSymbol
	self class compileText: aString selector: aSymbol
! !

!WAFileLibrary methodsFor:'actions'!

deployFiles
	"Write to disk the files that the receiver use to serve as methods.
	The files are stored in a subfolder named like the classname of the receiver in a subfolder of Smalltalk image folder."
	GRPlatform current ensureExistenceOfFolder: self name.
	self fileSelectors do: [ :each | 
		GRPlatform current 
			write: (self perform: each)
			toFile: (self asFilename: each)
			inFolder: self name ]
! !

!WAFileLibrary methodsFor:'accessing'!

documentForFile: aFilename
	^ self perform: (self asSelector: aFilename)
! !

!WAFileLibrary methodsFor:'private'!

fileSelectors
	| selectors current |
	selectors := Set new.
	current := self class.
	[ WAFileLibrary = current ] whileFalse: [
		current selectors do: [ :each |
			(self isFileSelector: each) ifTrue: [
				selectors add: each ] ].
		current := current superclass ].
	^ selectors
! !

!WAFileLibrary methodsFor:'accessing'!

fileSizeOf: aSymbol
	^ (self perform: aSymbol) size
! !

!WAFileLibrary methodsFor:'accessing'!

filenames
	^ self fileSelectors collect: [ :each | self asFilename: each ]
! !

!WAFileLibrary methodsFor:'private'!

fullPathForFileName: aString using: aFileHandler
	^ String streamContents: [ :stream |
		| pathSeparator xSendfileBaseDirectory |
		pathSeparator := GRPlatform current pathSeparator.
		xSendfileBaseDirectory := aFileHandler xSendfileBaseDirectory.
		stream nextPutAll: xSendfileBaseDirectory.
		(xSendfileBaseDirectory endsWithSubCollection: pathSeparator) ifFalse: [
			stream nextPutAll: pathSeparator ].
		stream
			nextPutAll: self name;
			nextPutAll: pathSeparator;
			nextPutAll: aString ]
! !

!WAFileLibrary methodsFor:'handling'!

handle: aRequestContext
	| fileName selector fileHandler |
	fileName := aRequestContext consumer next.
	selector := self asSelector: fileName.
	(self fileSelectors includes: selector) 
		ifFalse: [ ^ self ].
	
	fileHandler := aRequestContext handler.
	fileHandler useXSendfile
		ifTrue: [
			aRequestContext respond: [ :response |
				response
					cacheFor: self cacheDuration;
					headerAt: 'X-Sendfile' put: (self fullPathForFileName: fileName using: fileHandler);
					contentType: (self mimetypeForFile: fileName) ] ]
		ifFalse: [
			aRequestContext respond: [ :response |
				response
					cacheFor: self cacheDuration;
					document: (self documentForFile: fileName)
					mimeType: (self mimetypeForFile: fileName) ] ]
! !

!WAFileLibrary methodsFor:'private'!

isFileSelector: aSymbol
	"added condition to prevent overrides of methods defined in this class being picked up"
	^ aSymbol isUnary
		and: [ (self lastUpperCaseIndexIn: aSymbol) > 1
		and: [ (self nonFileSelectors includes: aSymbol) not ] ]
! !

!WAFileLibrary methodsFor:'private'!

lastUpperCaseIndexIn: aString
	^ aString findLast: [ :each | each isUppercase ]
! !

!WAFileLibrary methodsFor:'accessing'!

mimetypeForFile: aFilename
	^ self class mimetypeFor: (aFilename copyAfterLast: $.)
! !

!WAFileLibrary methodsFor:'accessing'!

name
	^ self class name greaseString
! !

!WAFileLibrary methodsFor:'private'!

nonFileSelectors
	"Answers all the unary selectors with an upper clase latter that are not file selector.s
	This method is intended to be overriden by superclasses that add functionality. Don't forget to add the result of the super send."
	^ #(cacheDuration fileSelectors deployFiles configurationComponent selectorsToInclude nonFileSelectors)
! !

!WAFileLibrary methodsFor:'actions'!

removeFile: aFilename 
	GRPlatform current 
		removeSelector: (self asSelector: aFilename)
		from: self class
! !

!WAFileLibrary methodsFor:'actions'!

renameFile: oldName to: newName
	| contents file |
	(self asSelector: oldName) = (self asSelector: newName)
		ifTrue: [ ^ nil ].
	contents := self perform: (self asSelector: oldName).
	file := WAFile new contents: contents.
	file fileName: newName.
	self addFile: file.
	self removeFile: oldName
! !

!WAFileLibrary methodsFor:'accessing'!

selectorsToInclude
	"The files represented by the selectors this method returns will be automatically added to the html <head> if the receiver is added to the respective Seaside application.

	This makes only sense for CSS and JS files"
	^ #()
! !

!WAFileLibrary methodsFor:'processing'!

updateRoot: anHtmlRoot
	"Only override if you want to automatically include new types of resources. The default implementation knows how to reference CSS, JS and FavIcons into anHtmlRoot."

	self selectorsToInclude do: [ :each |
		(each endsWithSubCollection: #Js)
			ifTrue: [ anHtmlRoot javascript url: (self urlOf: each) ].
		(each endsWithSubCollection: #Ico)
			ifTrue: [ anHtmlRoot link beShortcutIcon; url: (self urlOf: each) ].
		(each endsWithSubCollection: #Css)
			ifTrue: [ anHtmlRoot stylesheet url: (self urlOf: each) ] ]
! !

!WAFileLibrary methodsFor:'accessing'!

urlForFile: aFilename
	^ self urlOf: (self asSelector: aFilename)
! !

!WAFileLibrary methodsFor:'accessing'!

urlOf: aSymbol
	^ self urlOf: aSymbol using: WAFileHandler default
! !

!WAFileLibrary methodsFor:'accessing'!

urlOf: aSymbol using: aHandler
	^ aHandler baseUrlForLibrary
		addToPath: self className;
		addToPath: (self asFilename: aSymbol);
		yourself
! !

!WAFileLibrary class methodsFor:'convenience'!

/ aSymbol
	^ self urlOf: aSymbol
! !

!WAFileLibrary class methodsFor:'file addition'!

addAllFilesIn: aPathString 
	"adds all files in the directory specified by aPathString to the current file library"
	(GRPlatform current filesIn: aPathString) do: [ :each | self addFileAt: each ]
! !

!WAFileLibrary class methodsFor:'file addition'!

addFileAt: aPath 
	"Add the file specified by aPath to the current file library."
	self 
		addFileAt: aPath
		contents: (GRPlatform current 
				contentsOfFile: aPath
				binary: (self isBinaryAt: aPath))
! !

!WAFileLibrary class methodsFor:'private'!

addFileAt: aPath contents: aByteArrayOrString 
	self 
		addFileNamed: (GRPlatform current localNameOf: aPath)
		contents: aByteArrayOrString
! !

!WAFileLibrary class methodsFor:'private'!

addFileNamed: aFilename contents: aByteArrayOrString
	| selector |
	selector := self asSelector: aFilename.
	(self isBinary: aFilename)
		ifTrue: [ self compileBinary: aByteArrayOrString selector: selector ]
		ifFalse: [ self compileText: aByteArrayOrString selector: selector ]
! !

!WAFileLibrary class methodsFor:'vw port-squeak'!

adjustForVisualWorks
	| newSource oldSource badCode goodCode |
	badCode := 'xxxCache := #(nil).'.
	goodCode := 'xxxCache := #(nil) beMutable.'.
	self selectors do: [ :selector |
		oldSource := self sourceCodeAt: selector.
		(oldSource indexOfSubCollection: badCode startingAt: 1) > 0 ifTrue: [
			newSource := oldSource copyReplaceAll: badCode with: goodCode.
			self compile: newSource classified: self methodCategory ] ]
! !

!WAFileLibrary class methodsFor:'private'!

asSelector: aFilename
	| mainPart extension |
	mainPart := (aFilename copyUpToLast: $.)
		select: [ :each | each isAlphaNumeric ].
	[ mainPart first isDigit ]
		whileTrue: [ mainPart := mainPart allButFirst ].
	extension := (aFilename copyAfterLast: $.) asLowercase capitalized.
	^ (mainPart , extension) asSymbol
! !

!WAFileLibrary class methodsFor:'private'!

compileBinary: aByteArrayOrString selector: aSymbol 
	"compiles aByteArrayOrString into a method named aSymbol that returns aByteArrayOrString as a byte array"
	| code |
	code := GRPlatform current 
		asMethodReturningByteArray: aByteArrayOrString
		named: aSymbol.
	GRPlatform current 
		compile: code
		into: self
		classified: self methodCategory
! !

!WAFileLibrary class methodsFor:'private'!

compileText: aByteArrayOrString selector: aSymbol 
	"Compiles aByteArrayOrString into a method named aSymbol that returns aByteArrayOrString as a string literal.
	aSymbol
		^ aByteArrayOrString"
	| code |
	code := String streamContents: [ :stream | 
		stream
			nextPutAll: aSymbol;
			nextPut: Character cr.
		stream
			tab;
			nextPutAll: '^ '''.
		aByteArrayOrString greaseString do: [ :each | 
			each = $' ifTrue: [ stream nextPut: $' ].
			stream nextPut: each ].
		stream nextPutAll: '''' ].
	GRPlatform current 
		compile: code
		into: self
		classified: self methodCategory
! !

!WAFileLibrary class methodsFor:'instance creation'!

default
	^ default ifNil: [ default := self new ]
! !

!WAFileLibrary class methodsFor:'accessing-defaults'!

defaultMimeType
	^ 'application/octet-stream'
! !

!WAFileLibrary class methodsFor:'accessing-defaults'!

defaultMimeTypes
	^ #(
		'%' 'application/x-trash'
		'323' 'text/h323'
		'abw' 'application/x-abiword'
		'ai' 'application/postscript'
		'aif' 'audio/x-aiff'
		'aifc' 'audio/x-aiff'
		'aiff' 'audio/x-aiff'
		'alc' 'chemical/x-alchemy'
		'art' 'image/x-jg'
		'asc' 'text/plain'
		'asf' 'video/x-ms-asf'
		'asn' 'chemical/x-ncbi-asn1-spec'
		'aso' 'chemical/x-ncbi-asn1-binary'
		'asx' 'video/x-ms-asf'
		'au' 'audio/basic'
		'avi' 'video/x-msvideo'
		'b' 'chemical/x-molconn-Z'
		'bak' 'application/x-trash'
		'bat' 'application/x-msdos-program'
		'bcpio' 'application/x-bcpio'
		'bib' 'text/x-bibtex'
		'bin' 'application/octet-stream'
		'bmp' 'image/x-ms-bmp'
		'book' 'application/x-maker'
		'bsd' 'chemical/x-crossfire'
		'c' 'text/x-csrc'
		'c++' 'text/x-c++src'
		'c3d' 'chemical/x-chem3d'
		'cac' 'chemical/x-cache'
		'cache' 'chemical/x-cache'
		'cascii' 'chemical/x-cactvs-binary'
		'cat' 'application/vnd.ms-pki.seccat'
		'cbin' 'chemical/x-cactvs-binary'
		'cc' 'text/x-c++src'
		'cdf' 'application/x-cdf'
		'cdr' 'image/x-coreldraw'
		'cdt' 'image/x-coreldrawtemplate'
		'cdx' 'chemical/x-cdx'
		'cdy' 'application/vnd.cinderella'
		'cef' 'chemical/x-cxf'
		'cer' 'chemical/x-cerius'
		'chm' 'chemical/x-chemdraw'
		'chrt' 'application/x-kchart'
		'cif' 'chemical/x-cif'
		'class' 'application/java-vm'
		'cls' 'text/x-tex'
		'cmdf' 'chemical/x-cmdf'
		'cml' 'chemical/x-cml'
		'cod' 'application/vnd.rim.cod'
		'com' 'application/x-msdos-program'
		'cpa' 'chemical/x-compass'
		'cpio' 'application/x-cpio'
		'cpp' 'text/x-c++src'
		'cpt' 'image/x-corelphotopaint'
		'crl' 'application/x-pkcs7-crl'
		'crt' 'application/x-x509-ca-cert'
		'csf' 'chemical/x-cache-csf'
		'csh' 'text/x-csh'
		'csm' 'chemical/x-csml'
		'csml' 'chemical/x-csml'
		'css' 'text/css'
		'csv' 'text/comma-separated-values'
		'ctab' 'chemical/x-cactvs-binary'
		'ctx' 'chemical/x-ctx'
		'cu' 'application/cu-seeme'
		'cub' 'chemical/x-gaussian-cube'
		'cxf' 'chemical/x-cxf'
		'cxx' 'text/x-c++src'
		'dat' 'chemical/x-mopac-input'
		'dcr' 'application/x-director'
		'deb' 'application/x-debian-package'
		'dif' 'video/dv'
		'diff' 'text/plain'
		'dir' 'application/x-director'
		'djv' 'image/vnd.djvu'
		'djvu' 'image/vnd.djvu'
		'dl' 'video/dl'
		'dll' 'application/x-msdos-program'
		'dmg' 'application/x-apple-diskimage'
		'dms' 'application/x-dms'
		'doc' 'application/msword'
		'dot' 'application/msword'
		'dv' 'video/dv'
		'dvi' 'application/x-dvi'
		'dx' 'chemical/x-jcamp-dx'
		'dxr' 'application/x-director'
		'emb' 'chemical/x-embl-dl-nucleotide'
		'embl' 'chemical/x-embl-dl-nucleotide'
		'ent' 'chemical/x-pdb'
		'eps' 'application/postscript'
		'etx' 'text/x-setext'
		'exe' 'application/x-msdos-program'
		'ez' 'application/andrew-inset'
		'fb' 'application/x-maker'
		'fbdoc' 'application/x-maker'
		'fch' 'chemical/x-gaussian-checkpoint'
		'fchk' 'chemical/x-gaussian-checkpoint'
		'fig' 'application/x-xfig'
		'flac' 'application/x-flac'
		'fli' 'video/fli'
		'fm' 'application/x-maker'
		'frame' 'application/x-maker'
		'frm' 'application/x-maker'
		'gal' 'chemical/x-gaussian-log'
		'gam' 'chemical/x-gamess-input'
		'gamin' 'chemical/x-gamess-input'
		'gau' 'chemical/x-gaussian-input'
		'gcd' 'text/x-pcs-gcd'
		'gcf' 'application/x-graphing-calculator'
		'gcg' 'chemical/x-gcg8-sequence'
		'gen' 'chemical/x-genbank'
		'gf' 'application/x-tex-gf'
		'gif' 'image/gif'
		'gjc' 'chemical/x-gaussian-input'
		'gjf' 'chemical/x-gaussian-input'
		'gl' 'video/gl'
		'gnumeric' 'application/x-gnumeric'
		'gpt' 'chemical/x-mopac-graph'
		'gsf' 'application/x-font'
		'gsm' 'audio/x-gsm'
		'gtar' 'application/x-gtar'
		'h' 'text/x-chdr'
		'h++' 'text/x-c++hdr'
		'hdf' 'application/x-hdf'
		'hh' 'text/x-c++hdr'
		'hin' 'chemical/x-hin'
		'hpp' 'text/x-c++hdr'
		'hqx' 'application/mac-binhex40'
		'hs' 'text/x-haskell'
		'hta' 'application/hta'
		'htc' 'text/x-component'
		'htm' 'text/html'
		'html' 'text/html'
		'hxx' 'text/x-c++hdr'
		'ica' 'application/x-ica'
		'ice' 'x-conference/x-cooltalk'
		'ico' 'image/x-icon'
		'ics' 'text/calendar'
		'icz' 'text/calendar'
		'ief' 'image/ief'
		'iges' 'model/iges'
		'igs' 'model/iges'
		'iii' 'application/x-iphone'
		'inp' 'chemical/x-gamess-input'
		'ins' 'application/x-internet-signup'
		'iso' 'application/x-iso9660-image'
		'isp' 'application/x-internet-signup'
		'ist' 'chemical/x-isostar'
		'istr' 'chemical/x-isostar'
		'jad' 'text/vnd.sun.j2me.app-descriptor'
		'jar' 'application/java-archive'
		'java' 'text/x-java'
		'jdx' 'chemical/x-jcamp-dx'
		'jmz' 'application/x-jmol'
		'jng' 'image/x-jng'
		'jnlp' 'application/x-java-jnlp-file'
		'jpe' 'image/jpeg'
		'jpeg' 'image/jpeg'
		'jpg' 'image/jpeg'
		'js' 'application/x-javascript'
		'kar' 'audio/midi'
		'key' 'application/pgp-keys'
		'kil' 'application/x-killustrator'
		'kin' 'chemical/x-kinemage'
		'kpr' 'application/x-kpresenter'
		'kpt' 'application/x-kpresenter'
		'ksp' 'application/x-kspread'
		'kwd' 'application/x-kword'
		'kwt' 'application/x-kword'
		'latex' 'application/x-latex'
		'lha' 'application/x-lha'
		'lhs' 'text/x-literate-haskell'
		'lsf' 'video/x-la-asf'
		'lsx' 'video/x-la-asf'
		'ltx' 'text/x-tex'
		'lzh' 'application/x-lzh'
		'lzx' 'application/x-lzx'
		'm3u' 'audio/x-mpegurl'
		'm4a' 'audio/mpeg'
		'maker' 'application/x-maker'
		'man' 'application/x-troff-man'
		'mcif' 'chemical/x-mmcif'
		'mcm' 'chemical/x-macmolecule'
		'mdb' 'application/msaccess'
		'me' 'application/x-troff-me'
		'mesh' 'model/mesh'
		'mid' 'audio/midi'
		'midi' 'audio/midi'
		'mif' 'application/x-mif'
		'mm' 'application/x-freemind'
		'mmd' 'chemical/x-macromodel-input'
		'mmf' 'application/vnd.smaf'
		'mml' 'text/mathml'
		'mmod' 'chemical/x-macromodel-input'
		'mng' 'video/x-mng'
		'moc' 'text/x-moc'
		'mol' 'chemical/x-mdl-molfile'
		'mol2' 'chemical/x-mol2'
		'moo' 'chemical/x-mopac-out'
		'mop' 'chemical/x-mopac-input'
		'mopcrt' 'chemical/x-mopac-input'
		'mov' 'video/quicktime'
		'movie' 'video/x-sgi-movie'
		'mp2' 'audio/mpeg'
		'mp3' 'audio/mpeg'
		'mp4' 'video/mp4'
		'mpc' 'chemical/x-mopac-input'
		'mpe' 'video/mpeg'
		'mpeg' 'video/mpeg'
		'mpega' 'audio/mpeg'
		'mpg' 'video/mpeg'
		'mpga' 'audio/mpeg'
		'ms' 'application/x-troff-ms'
		'msh' 'model/mesh'
		'msi' 'application/x-msi'
		'mvb' 'chemical/x-mopac-vib'
		'mxu' 'video/vnd.mpegurl'
		'nb' 'application/mathematica'
		'nc' 'application/x-netcdf'
		'nwc' 'application/x-nwc'
		'o' 'application/x-object'
		'oda' 'application/oda'
		'odb' 'application/vnd.oasis.opendocument.database'
		'odc' 'application/vnd.oasis.opendocument.chart'
		'odf' 'application/vnd.oasis.opendocument.formula'
		'odg' 'application/vnd.oasis.opendocument.graphics'
		'odi' 'application/vnd.oasis.opendocument.image'
		'odm' 'application/vnd.oasis.opendocument.text-master'
		'odp' 'application/vnd.oasis.opendocument.presentation'
		'ods' 'application/vnd.oasis.opendocument.spreadsheet'
		'odt' 'application/vnd.oasis.opendocument.text'
		'ogg' 'application/ogg'
		'old' 'application/x-trash'
		'oth' 'application/vnd.oasis.opendocument.text-web'
		'oza' 'application/x-oz-application'
		'p' 'text/x-pascal'
		'p7r' 'application/x-pkcs7-certreqresp'
		'pac' 'application/x-ns-proxy-autoconfig'
		'pas' 'text/x-pascal'
		'pat' 'image/x-coreldrawpattern'
		'pbm' 'image/x-portable-bitmap'
		'pcf' 'application/x-font'
		'pcf.Z' 'application/x-font'
		'pcx' 'image/pcx'
		'pdb' 'chemical/x-pdb'
		'pdf' 'application/pdf'
		'pfa' 'application/x-font'
		'pfb' 'application/x-font'
		'pgm' 'image/x-portable-graymap'
		'pgn' 'application/x-chess-pgn'
		'pgp' 'application/pgp-signature'
		'pk' 'application/x-tex-pk'
		'pl' 'text/x-perl'
		'pls' 'audio/x-scpls'
		'pm' 'text/x-perl'
		'png' 'image/png'
		'pnm' 'image/x-portable-anymap'
		'pot' 'text/plain'
		'ppm' 'image/x-portable-pixmap'
		'pps' 'application/vnd.ms-powerpoint'
		'ppt' 'application/vnd.ms-powerpoint'
		'prf' 'application/pics-rules'
		'prt' 'chemical/x-ncbi-asn1-ascii'
		'ps' 'application/postscript'
		'psd' 'image/x-photoshop'
		'psp' 'text/x-psp'
		'py' 'text/x-python'
		'pyc' 'application/x-python-code'
		'pyo' 'application/x-python-code'
		'qt' 'video/quicktime'
		'qtl' 'application/x-quicktimeplayer'
		'ra' 'audio/x-realaudio'
		'ram' 'audio/x-pn-realaudio'
		'rar' 'application/rar'
		'ras' 'image/x-cmu-raster'
		'rd' 'chemical/x-mdl-rdfile'
		'rdf' 'application/rdf+xml'
		'rgb' 'image/x-rgb'
		'rm' 'audio/x-pn-realaudio'
		'roff' 'application/x-troff'
		'ros' 'chemical/x-rosdal'
		'rpm' 'application/x-redhat-package-manager'
		'rss' 'application/rss+xml'
		'rtf' 'text/rtf'
		'rtx' 'text/richtext'
		'rxn' 'chemical/x-mdl-rxnfile'
		'sct' 'text/scriptlet'
		'sd' 'chemical/x-mdl-sdfile'
		'sd2' 'audio/x-sd2'
		'sda' 'application/vnd.stardivision.draw'
		'sdc' 'application/vnd.stardivision.calc'
		'sdd' 'application/vnd.stardivision.impress'
		'sdf' 'chemical/x-mdl-sdfile'
		'sdp' 'application/vnd.stardivision.impress'
		'sdw' 'application/vnd.stardivision.writer'
		'ser' 'application/java-serialized-object'
		'sgf' 'application/x-go-sgf'
		'sgl' 'application/vnd.stardivision.writer-global'
		'sh' 'text/x-sh'
		'shar' 'application/x-shar'
		'shtml' 'text/html'
		'sid' 'audio/prs.sid'
		'sik' 'application/x-trash'
		'silo' 'model/mesh'
		'sis' 'application/vnd.symbian.install'
		'sit' 'application/x-stuffit'
		'skd' 'application/x-koan'
		'skm' 'application/x-koan'
		'skp' 'application/x-koan'
		'skt' 'application/x-koan'
		'smf' 'application/vnd.stardivision.math'
		'smi' 'application/smil'
		'smil' 'application/smil'
		'snd' 'audio/basic'
		'spc' 'chemical/x-galactic-spc'
		'spl' 'application/x-futuresplash'
		'src' 'application/x-wais-source'
		'stc' 'application/vnd.sun.xml.calc.template'
		'std' 'application/vnd.sun.xml.draw.template'
		'sti' 'application/vnd.sun.xml.impress.template'
		'stl' 'application/vnd.ms-pki.stl'
		'stw' 'application/vnd.sun.xml.writer.template'
		'sty' 'text/x-tex'
		'sv4cpio' 'application/x-sv4cpio'
		'sv4crc' 'application/x-sv4crc'
		'svg' 'image/svg+xml'
		'svgz' 'image/svg+xml'
		'sw' 'chemical/x-swissprot'
		'swf' 'application/x-shockwave-flash'
		'swfl' 'application/x-shockwave-flash'
		'sxc' 'application/vnd.sun.xml.calc'
		'sxd' 'application/vnd.sun.xml.draw'
		'sxg' 'application/vnd.sun.xml.writer.global'
		'sxi' 'application/vnd.sun.xml.impress'
		'sxm' 'application/vnd.sun.xml.math'
		'sxw' 'application/vnd.sun.xml.writer'
		't' 'application/x-troff'
		'tar' 'application/x-tar'
		'taz' 'application/x-gtar'
		'tcl' 'text/x-tcl'
		'tex' 'text/x-tex'
		'texi' 'application/x-texinfo'
		'texinfo' 'application/x-texinfo'
		'text' 'text/plain'
		'tgf' 'chemical/x-mdl-tgf'
		'tgz' 'application/x-gtar'
		'tif' 'image/tiff'
		'tiff' 'image/tiff'
		'tk' 'text/x-tcl'
		'tm' 'text/texmacs'
		'torrent' 'application/x-bittorrent'
		'tr' 'application/x-troff'
		'ts' 'text/texmacs'
		'tsp' 'application/dsptype'
		'tsv' 'text/tab-separated-values'
		'txt' 'text/plain'
		'udeb' 'application/x-debian-package'
		'uls' 'text/iuls'
		'ustar' 'application/x-ustar'
		'val' 'chemical/x-ncbi-asn1-binary'
		'vcd' 'application/x-cdlink'
		'vcf' 'text/x-vcard'
		'vcs' 'text/x-vcalendar'
		'vmd' 'chemical/x-vmd'
		'vms' 'chemical/x-vamas-iso14976'
		'vor' 'application/vnd.stardivision.writer'
		'vrm' 'x-world/x-vrml'
		'vrml' 'x-world/x-vrml'
		'vsd' 'application/vnd.visio'
		'wad' 'application/x-doom'
		'wav' 'audio/x-wav'
		'wax' 'audio/x-ms-wax'
		'wbmp' 'image/vnd.wap.wbmp'
		'wbxml' 'application/vnd.wap.wbxml'
		'wk' 'application/x-123'
		'wm' 'video/x-ms-wm'
		'wma' 'audio/x-ms-wma'
		'wmd' 'application/x-ms-wmd'
		'wml' 'text/vnd.wap.wml'
		'wmlc' 'application/vnd.wap.wmlc'
		'wmls' 'text/vnd.wap.wmlscript'
		'wmlsc' 'application/vnd.wap.wmlscriptc'
		'wmv' 'video/x-ms-wmv'
		'wmx' 'video/x-ms-wmx'
		'wmz' 'application/x-ms-wmz'
		'wp5' 'application/wordperfect5.1'
		'wpd' 'application/wordperfect'
		'wrl' 'x-world/x-vrml'
		'wsc' 'text/scriptlet'
		'wvx' 'video/x-ms-wvx'
		'wz' 'application/x-wingz'
		'xbm' 'image/x-xbitmap'
		'xcf' 'application/x-xcf'
		'xht' 'application/xhtml+xml'
		'xhtml' 'application/xhtml+xml'
		'xlb' 'application/vnd.ms-excel'
		'xls' 'application/vnd.ms-excel'
		'xlt' 'application/vnd.ms-excel'
		'xml' 'application/xml'
		'xpi' 'application/x-xpinstall'
		'xpm' 'image/x-xpixmap'
		'xsl' 'application/xml'
		'xtel' 'chemical/x-xtel'
		'xul' 'application/vnd.mozilla.xul+xml'
		'xwd' 'image/x-xwindowdump'
		'xyz' 'chemical/x-xyz'
		'zip' 'application/zip'
		'zmt' 'chemical/x-mopac-input'
		'~' 'application/x-trash'

	)
! !

!WAFileLibrary class methodsFor:'actions'!

deployFiles
	^ self default deployFiles
! !

!WAFileLibrary class methodsFor:'handling'!

handle: aRequestContext
	^ self default handle: aRequestContext
! !

!WAFileLibrary class methodsFor:'testing'!

handlesFolder: aSymbol
	^ self name = aSymbol
! !

!WAFileLibrary class methodsFor:'class initialization'!

initialize
	MimeTypes := Dictionary new.
	1 to: self defaultMimeTypes size by: 2 do: [ :index |
		MimeTypes
			at: (self defaultMimeTypes at: index)
			put: (self defaultMimeTypes at: index + 1) ]
! !

!WAFileLibrary class methodsFor:'private'!

isBinary: aFilename
	^ (self mimetypeFor: (aFilename copyAfterLast: $.)) isBinary
! !

!WAFileLibrary class methodsFor:'private'!

isBinaryAt: aPath 
	^ self isBinary: (GRPlatform current localNameOf: aPath)
! !

!WAFileLibrary class methodsFor:'private'!

methodCategory
	"the method category for uploaded files"

	^ #uploaded
! !

!WAFileLibrary class methodsFor:'migrating'!

migrateSubclassesSeaside29
	"migrates all subclasses of the receiver to Seaside 2.9
	you should not send this message to WAFileLibrary because that will also migrate the classes in Seaside"
	self allSubclassesDo: [ :each |
		each migrateToSeaside29 ]
! !

!WAFileLibrary class methodsFor:'migrating'!

migrateToSeaside29
	"migrates only the receiver to Seaside 2.9"
	(self default fileSelectors intersection: self selectors) do: [ :each |
		| contents |
		contents := (self default perform: each).
		(contents isKindOf: ByteArray) ifTrue: [
			| file |
			file := WAFile new
				contents: contents;
				fileName: (self default asFilename: each);
				yourself.
			self default addFile: file ] ]
! !

!WAFileLibrary class methodsFor:'accessing'!

mimetypeFor: aString
	^ (self mimetypes at: aString ifAbsent: [ self defaultMimeType ]) seasideMimeType
! !

!WAFileLibrary class methodsFor:'accessing'!

mimetypes
	^ MimeTypes
! !

!WAFileLibrary class methodsFor:'convenience'!

register
	"no longer needed left in so that code can still be loaded since it is sent in class side initialize methods"
! !

!WAFileLibrary class methodsFor:'convenience'!

unregister
	"no longer used"
! !

!WAFileLibrary class methodsFor:'convenience'!

urlOf: aSymbol
	^ self default urlOf: aSymbol
! !

!WAKeyGenerator methodsFor:'accessing'!

generator: aStream
	"A stream of random numbers, basically a random generator."

	generator := aStream
! !

!WAKeyGenerator methodsFor:'initialization'!

initialize
	super initialize.
	self generator: GRPlatform current newRandom.
	self space: 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-'
! !

!WAKeyGenerator methodsFor:'public'!

keyOfLength: anInteger
	"Answer a random key as a string with the length anInteger. This does not use any of the fancy collection methods for speed reasons."

	| key |
	key := String new: anInteger.
	1 to: anInteger do: [ :index | key at: index put: (space at: (generator nextInt: space size)) ].
	^ key
! !

!WAKeyGenerator methodsFor:'accessing'!

space: aCollectionOfCharacters
	"Defines the key generator space."

	space := aCollectionOfCharacters
! !

!WAKeyGenerator class methodsFor:'accessing'!

current
	^ current
! !

!WAKeyGenerator class methodsFor:'initialization'!

initialize
	current := self new
! !

!WALocale methodsFor:'accessing'!

country
	^ country
! !

!WALocale methodsFor:'accessing'!

country: aString
	country := aString
! !

!WALocale methodsFor:'accessing'!

countryName
	"answers the country name in english and all capitals"

	^ self country size = 2
		ifTrue: [ self class iso2Countries at: self country ]
		ifFalse: [ self class iso3Countries at: self country ]
! !

!WALocale methodsFor:'converting'!

greaseString
	^ String streamContents: 
		[ :stream | 
		stream nextPutAll: self language.
		self country isNil ifFalse: 
			[ stream
				nextPut: $-;
				nextPutAll: self country ] ]
! !

!WALocale methodsFor:'accessing'!

language
	^ language
! !

!WALocale methodsFor:'accessing'!

language: aString
	language := aString
! !

!WALocale methodsFor:'accessing'!

languageName
	"answers the language name in english and all capitals"

	^ self language size = 2
		ifTrue: [ self class iso2Languages at: self language ]
		ifFalse: [ self class iso3Languages at: self language ]
! !

!WALocale methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self greaseString;
		nextPut: $)
! !

!WALocale methodsFor:'copying'!

withoutCountry
	"Answer a copy with the language but no country."
	^ WALocale language: self language country: nil
! !

!WALocale class methodsFor:'accessing'!

countryList
	^ #(('AF' 'AFG' 'AFGHANISTAN') ('AL' 'ALB' 'ALBANIA') ('DZ' 'DZA' 'ALGERIA') ('AS' 'ASM' 'AMERICAN SAMOA') ('AD' 'AND' 'ANDORRA') ('AO' 'AGO' 'ANGOLA') ('AI' 'AIA' 'ANGUILLA') ('AQ' 'ATA' 'ANTARCTICA') ('AG' 'ATG' 'ANTIGUA AND BARBUDA') ('AR' 'ARG' 'ARGENTINA') ('AM' 'ARM' 'ARMENIA') ('AW' 'ABW' 'ARUBA') ('AU' 'AUS' 'AUSTRALIA') ('AT' 'AUT' 'AUSTRIA') ('AZ' 'AZE' 'AZERBAIJAN') ('BS' 'BHS' 'BAHAMAS') ('BH' 'BHR' 'BAHRAIN') ('BD' 'BGD' 'BANGLADESH') ('BB' 'BRB' 'BARBADOS') ('BY' 'BLR' 'BELARUS') ('BE' 'BEL' 'BELGIUM') ('BZ' 'BLZ' 'BELIZE') ('BJ' 'BEN' 'BENIN') ('BM' 'BMU' 'BERMUDA') ('BT' 'BTN' 'BHUTAN') ('BO' 'BOL' 'BOLIVIA') ('BA' 'BIH' 'BOSNIA AND HERZEGOWINA') ('BW' 'BWA' 'BOTSWANA') ('BV' 'BVT' 'BOUVET ISLAND') ('BR' 'BRA' 'BRAZIL') ('IO' 'IOT' 'BRITISH INDIAN OCEAN TERRITORY') ('BN' 'BRN' 'BRUNEI DARUSSALAM') ('BG' 'BGR' 'BULGARIA') ('BF' 'BFA' 'BURKINA FASO') ('BI' 'BDI' 'BURUNDI') ('KH' 'KHM' 'CAMBODIA') ('CM' 'CMR' 'CAMEROON') ('CA' 'CAN' 'CANADA') ('CV' 'CPV' 'CAPE VERDE') ('KY' 'CYM' 'CAYMAN ISLANDS') ('CF' 'CAF' 'CENTRAL AFRICAN REPUBLIC') ('TD' 'TCD' 'CHAD') ('CL' 'CHL' 'CHILE') ('CN' 'CHN' 'CHINA') ('CX' 'CXR' 'CHRISTMAS ISLAND') ('CC' 'CCK' 'COCOS (KEELING) ISLANDS') ('CO' 'COL' 'COLOMBIA') ('KM' 'COM' 'COMOROS') ('CG' 'COG' 'CONGO') ('CK' 'COK' 'COOK ISLANDS') ('CR' 'CRI' 'COSTA RICA') ('CI' 'CIV' 'COTE D''IVOIRE') ('HR' 'HRV' 'CROATIA (local name: Hrvatska)') ('CU' 'CUB' 'CUBA') ('CY' 'CYP' 'CYPRUS') ('CZ' 'CZE' 'CZECH REPUBLIC') ('DK' 'DNK' 'DENMARK') ('DJ' 'DJI' 'DJIBOUTI') ('DM' 'DMA' 'DOMINICA') ('DO' 'DOM' 'DOMINICAN REPUBLIC') ('TP' 'TMP' 'EAST TIMOR') ('EC' 'ECU' 'ECUADOR') ('EG' 'EGY' 'EGYPT') ('SV' 'SLV' 'EL SALVADOR') ('GQ' 'GNQ' 'EQUATORIAL GUINEA') ('ER' '   ' 'ERITREA') ('EE' 'EST' 'ESTONIA') ('ET' 'ETH' 'ETHIOPIA') ('FK' 'FLK' 'FALKLAND ISLANDS (MALVINAS)') ('FO' 'FRO' 'FAROE ISLANDS') ('FJ' 'FJI' 'FIJI') ('FI' 'FIN' 'FINLAND') ('FR' 'FRA' 'FRANCE') ('FX' 'FXX' 'FRANCE, METROPOLITAN') ('GF' 'GUF' 'FRENCH GUIANA') ('PF' 'PYF' 'FRENCH POLYNESIA') ('TF' 'ATF' 'FRENCH SOUTHERN TERRITORIES') ('GA' 'GAB' 'GABON') ('GM' 'GMB' 'GAMBIA') ('GE' 'GEO' 'GEORGIA') ('DE' 'DEU' 'GERMANY') ('GH' 'GHA' 'GHANA') ('GI' 'GIB' 'GIBRALTAR') ('GR' 'GRC' 'GREECE') ('GL' 'GRL' 'GREENLAND') ('GD' 'GRD' 'GRENADA') ('GP' 'GLP' 'GUADELOUPE') ('GU' 'GUM' 'GUAM') ('GT' 'GTM' 'GUATEMALA') ('GN' 'GIN' 'GUINEA') ('GW' 'GNB' 'GUINEA-BISSAU') ('GY' 'GUY' 'GUYANA') ('HT' 'HTI' 'HAITI') ('HM' 'HMD' 'HEARD AND MC DONALD ISLANDS') ('VA' 'VAT' 'HOLY SEE (VATICAN CITY STATE)') ('HN' 'HND' 'HONDURAS') ('HK' 'HKG' 'HONG KONG') ('HU' 'HUN' 'HUNGARY') ('IS' 'ISL' 'ICELAND') ('IN' 'IND' 'INDIA') ('ID' 'IDN' 'INDONESIA') ('IR' 'IRN' 'IRAN (ISLAMIC REPUBLIC OF)') ('IQ' 'IRQ' 'IRAQ') ('IE' 'IRL' 'IRELAND') ('IL' 'ISR' 'ISRAEL') ('IT' 'ITA' 'ITALY') ('JM' 'JAM' 'JAMAICA') ('JP' 'JPN' 'JAPAN') ('JO' 'JOR' 'JORDAN') ('KZ' 'KAZ' 'KAZAKHSTAN') ('KE' 'KEN' 'KENYA') ('KI' 'KIR' 'KIRIBATI') ('KP' 'PRK' 'KOREA, DEMOCRATIC PEOPLE''S REPUBLIC OF') ('KR' 'KOR' 'KOREA, REPUBLIC OF') ('KW' 'KWT' 'KUWAIT') ('KG' 'KGZ' 'KYRGYZSTAN') ('LA' 'LAO' 'LAO PEOPLE''S DEMOCRATIC REPUBLIC') ('LV' 'LVA' 'LATVIA') ('LB' 'LBN' 'LEBANON') ('LS' 'LSO' 'LESOTHO') ('LR' 'LBR' 'LIBERIA') ('LY' 'LBY' 'LIBYAN ARAB JAMAHIRIYA') ('LI' 'LIE' 'LIECHTENSTEIN') ('LT' 'LTU' 'LITHUANIA') ('LU' 'LUX' 'LUXEMBOURG') ('MO' 'MAC' 'MACAU') ('MK' 'MKD' 'MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF') ('MG' 'MDG' 'MADAGASCAR') ('MW' 'MWI' 'MALAWI') ('MY' 'MYS' 'MALAYSIA') ('MV' 'MDV' 'MALDIVES') ('ML' 'MLI' 'MALI') ('MT' 'MLT' 'MALTA') ('MH' 'MHL' 'MARSHALL ISLANDS') ('MQ' 'MTQ' 'MARTINIQUE') ('MR' 'MRT' 'MAURITANIA') ('MU' 'MUS' 'MAURITIUS') ('YT' 'MYT' 'MAYOTTE') ('MX' 'MEX' 'MEXICO') ('FM' 'FSM' 'MICRONESIA, FEDERATED STATES OF') ('MD' 'MDA' 'MOLDOVA, REPUBLIC OF') ('MC' 'MCO' 'MONACO') ('MN' 'MNG' 'MONGOLIA') ('MS' 'MSR' 'MONTSERRAT') ('MA' 'MAR' 'MOROCCO') ('MZ' 'MOZ' 'MOZAMBIQUE') ('MM' 'MMR' 'MYANMAR') ('NA' 'NAM' 'NAMIBIA') ('NR' 'NRU' 'NAURU') ('NP' 'NPL' 'NEPAL') ('NL' 'NLD' 'NETHERLANDS') ('AN' 'ANT' 'NETHERLANDS ANTILLES') ('NC' 'NCL' 'NEW CALEDONIA') ('NZ' 'NZL' 'NEW ZEALAND') ('NI' 'NIC' 'NICARAGUA') ('NE' 'NER' 'NIGER') ('NG' 'NGA' 'NIGERIA') ('NU' 'NIU' 'NIUE') ('NF' 'NFK' 'NORFOLK ISLAND') ('MP' 'MNP' 'NORTHERN MARIANA ISLANDS') ('NO' 'NOR' 'NORWAY') ('OM' 'OMN' 'OMAN') ('PK' 'PAK' 'PAKISTAN') ('PW' 'PLW' 'PALAU') ('PA' 'PAN' 'PANAMA') ('PG' 'PNG' 'PAPUA NEW GUINEA') ('PY' 'PRY' 'PARAGUAY') ('PE' 'PER' 'PERU') ('PH' 'PHL' 'PHILIPPINES') ('PN' 'PCN' 'PITCAIRN') ('PL' 'POL' 'POLAND') ('PT' 'PRT' 'PORTUGAL') ('PR' 'PRI' 'PUERTO RICO') ('QA' 'QAT' 'QATAR') ('RE' 'REU' 'REUNION') ('RO' 'ROM' 'ROMANIA') ('RU' 'RUS' 'RUSSIAN FEDERATION') ('RW' 'RWA' 'RWANDA') ('KN' 'KNA' 'SAINT KITTS AND NEVIS') ('LC' 'LCA' 'SAINT LUCIA') ('VC' 'VCT' 'SAINT VINCENT AND THE GRENADINES') ('WS' 'WSM' 'SAMOA') ('SM' 'SMR' 'SAN MARINO') ('ST' 'STP' 'SAO TOME AND PRINCIPE') ('SA' 'SAU' 'SAUDI ARABIA') ('SN' 'SEN' 'SENEGAL') ('SC' 'SYC' 'SEYCHELLES') ('SL' 'SLE' 'SIERRA LEONE') ('SG' 'SGP' 'SINGAPORE') ('SK' 'SVK' 'SLOVAKIA (Slovak Republic)') ('SI' 'SVN' 'SLOVENIA') ('SB' 'SLB' 'SOLOMON ISLANDS') ('SO' 'SOM' 'SOMALIA') ('ZA' 'ZAF' 'SOUTH AFRICA') ('GS' 'SGS' 'SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS') ('ES' 'ESP' 'SPAIN') ('LK' 'LKA' 'SRI LANKA') ('SH' 'SHN' 'ST. HELENA') ('PM' 'SPM' 'ST. PIERRE AND MIQUELON') ('SD' 'SDN' 'SUDAN') ('SR' 'SUR' 'SURINAME') ('SJ' 'SJM' 'SVALBARD AND JAN MAYEN ISLANDS') ('SZ' 'SWZ' 'SWAZILAND') ('SE' 'SWE' 'SWEDEN') ('CH' 'CHE' 'SWITZERLAND') ('SY' 'SYR' 'SYRIAN ARAB REPUBLIC') ('TW' 'TWN' 'TAIWAN, PROVINCE OF CHINA') ('TJ' 'TJK' 'TAJIKISTAN') ('TZ' 'TZA' 'TANZANIA, UNITED REPUBLIC OF') ('TH' 'THA' 'THAILAND') ('TG' 'TGO' 'TOGO') ('TK' 'TKL' 'TOKELAU') ('TO' 'TON' 'TONGA') ('TT' 'TTO' 'TRINIDAD AND TOBAGO') ('TN' 'TUN' 'TUNISIA') ('TR' 'TUR' 'TURKEY') ('TM' 'TKM' 'TURKMENISTAN') ('TC' 'TCA' 'TURKS AND CAICOS ISLANDS') ('TV' 'TUV' 'TUVALU') ('UG' 'UGA' 'UGANDA') ('UA' 'UKR' 'UKRAINE') ('AE' 'ARE' 'UNITED ARAB EMIRATES') ('GB' 'GBR' 'UNITED KINGDOM') ('US' 'USA' 'UNITED STATES') ('UM' 'UMI' 'UNITED STATES MINOR OUTLYING ISLANDS') ('UY' 'URY' 'URUGUAY') ('UZ' 'UZB' 'UZBEKISTAN') ('VU' 'VUT' 'VANUATU') ('VE' 'VEN' 'VENEZUELA') ('VN' 'VNM' 'VIET NAM') ('VG' 'VGB' 'VIRGIN ISLANDS (BRITISH)') ('VI' 'VIR' 'VIRGIN ISLANDS (U.S.)') ('WF' 'WLF' 'WALLIS AND FUTUNA ISLANDS') ('EH' 'ESH' 'WESTERN SAHARA') ('YE' 'YEM' 'YEMEN') ('YU' 'YUG' 'YUGOSLAVIA') ('ZR' 'ZAR' 'ZAIRE') ('ZM' 'ZMB' 'ZAMBIA') ('ZW' 'ZWE' 'ZIMBABWE'))
! !

!WALocale class methodsFor:'instance creation'!

fromString: aString
	| language country delimiter |
	delimiter := (aString includes: $_)
		ifTrue: [ $_ ]
		ifFalse: [ $- ].
	language := aString copyUpTo: delimiter.
	country := aString copyAfter: delimiter.
	country isEmpty ifTrue: [ country := nil ].
	^ self
		language: language
		country: country
! !

!WALocale class methodsFor:'class initialization'!

initialize
	self initializeLangaugeList.
	self initializeCountryList
! !

!WALocale class methodsFor:'class initialization'!

initializeCountryList
	iso2Countries := Dictionary new.
	iso3Countries := Dictionary new.
	self countryList do: [ :each |
		iso2Countries
			at: each first
			put: each third.
		iso3Countries
			at: each second
			put: each third ]
! !

!WALocale class methodsFor:'class initialization'!

initializeLangaugeList
	iso2Languages := Dictionary new.
	iso3Languages := Dictionary new.
	
	self languageList do: [ :each |
		each size > 2
			ifTrue: [
				iso2Languages at: each first put: each third.
				iso3Languages at: each second put: each third ]
			ifFalse: [
				iso3Languages at: each first put: each second ] ]
! !

!WALocale class methodsFor:'accessing'!

iso2Countries
	^ iso2Countries
! !

!WALocale class methodsFor:'accessing'!

iso2Languages
	^ iso2Languages
! !

!WALocale class methodsFor:'accessing'!

iso3Countries
	^ iso3Countries
! !

!WALocale class methodsFor:'accessing'!

iso3Languages
	^ iso3Languages
! !

!WALocale class methodsFor:'instance creation'!

language: aLanguageString
	^ self new
		language: aLanguageString;
		yourself
! !

!WALocale class methodsFor:'instance creation'!

language: aLanguageString country: aCountryString
	^ self new
		language: aLanguageString;
		country: aCountryString;
		yourself
! !

!WALocale class methodsFor:'accessing'!

languageList
	^ #(('aa' 'aar' 'Afar') ('ab' 'abk' 'Abkhazian') ('ace' 'Achinese') ('ach' 'Acoli') ('ada' 'Adangme') ('ady' 'Adyghe; Adygei') ('afa' 'Afro-Asiatic (Other)') ('afh' 'Afrihili') ('af' 'afr' 'Afrikaans') ('ain' 'Ainu') ('ak' 'aka' 'Akan') ('akk' 'Akkadian') ('sq' 'alb' 'Albanian') ('ale' 'Aleut') ('alg' 'Algonquian languages') ('alt' 'Southern Altai') ('am' 'amh' 'Amharic') ('ang' 'English, Old (ca.450-1100)') ('anp' 'Angika') ('apa' 'Apache languages') ('ar' 'ara' 'Arabic') ('arc' 'Official Aramaic (700-300 BCE); Imperial Aramaic (700-300 BCE)') ('an' 'arg' 'Aragonese') ('hy' 'arm' 'Armenian') ('arn' 'Mapudungun; Mapuche') ('arp' 'Arapaho') ('art' 'Artificial (Other)') ('arw' 'Arawak') ('as' 'asm' 'Assamese') ('ast' 'Asturian; Bable; Leonese; Asturleonese') ('ath' 'Athapascan languages') ('aus' 'Australian languages') ('av' 'ava' 'Avaric') ('ae' 'ave' 'Avestan') ('awa' 'Awadhi') ('ay' 'aym' 'Aymara') ('az' 'aze' 'Azerbaijani') ('bad' 'Banda languages') ('bai' 'Bamileke languages') ('ba' 'bak' 'Bashkir') ('bal' 'Baluchi') ('bm' 'bam' 'Bambara') ('ban' 'Balinese') ('eu' 'baq' 'Basque') ('bas' 'Basa') ('bat' 'Baltic (Other)') ('bej' 'Beja; Bedawiyet') ('be' 'bel' 'Belarusian') ('bem' 'Bemba') ('bn' 'ben' 'Bengali') ('ber' 'Berber (Other)') ('bho' 'Bhojpuri') ('bh' 'bih' 'Bihari') ('bik' 'Bikol') ('bin' 'Bini; Edo') ('bi' 'bis' 'Bislama') ('bla' 'Siksika') ('bnt' 'Bantu (Other)') ('bs' 'bos' 'Bosnian') ('bra' 'Braj') ('br' 'bre' 'Breton') ('btk' 'Batak languages') ('bua' 'Buriat') ('bug' 'Buginese') ('bg' 'bul' 'Bulgarian') ('my' 'bur' 'Burmese') ('byn' 'Blin; Bilin') ('cad' 'Caddo') ('cai' 'Central American Indian (Other)') ('car' 'Galibi Carib') ('ca' 'cat' 'Catalan; Valencian') ('cau' 'Caucasian (Other)') ('ceb' 'Cebuano') ('cel' 'Celtic (Other)') ('ch' 'cha' 'Chamorro') ('chb' 'Chibcha') ('ce' 'che' 'Chechen') ('chg' 'Chagatai') ('zh' 'chi' 'Chinese') ('chk' 'Chuukese') ('chm' 'Mari') ('chn' 'Chinook jargon') ('cho' 'Choctaw') ('chp' 'Chipewyan; Dene Suline') ('chr' 'Cherokee') ('cu' 'chu' 'Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic') ('cv' 'chv' 'Chuvash') ('chy' 'Cheyenne') ('cmc' 'Chamic languages') ('cop' 'Coptic') ('kw' 'cor' 'Cornish') ('co' 'cos' 'Corsican') ('cpe' 'Creoles and pidgins, English based (Other)') ('cpf' 'Creoles and pidgins, French-based (Other)') ('cpp' 'Creoles and pidgins, Portuguese-based (Other)') ('cr' 'cre' 'Cree') ('crh' 'Crimean Tatar; Crimean Turkish') ('crp' 'Creoles and pidgins (Other)') ('csb' 'Kashubian') ('cus' 'Cushitic (Other)') ('cs' 'cze' 'Czech') ('dak' 'Dakota') ('da' 'dan' 'Danish') ('dar' 'Dargwa') ('day' 'Land Dayak languages') ('del' 'Delaware') ('den' 'Slave (Athapascan)') ('dgr' 'Dogrib') ('din' 'Dinka') ('dv' 'div' 'Divehi; Dhivehi; Maldivian') ('doi' 'Dogri') ('dra' 'Dravidian (Other)') ('dsb' 'Lower Sorbian') ('dua' 'Duala') ('dum' 'Dutch, Middle (ca.1050-1350)') ('nl' 'dut' 'Dutch; Flemish') ('dyu' 'Dyula') ('dz' 'dzo' 'Dzongkha') ('efi' 'Efik') ('egy' 'Egyptian (Ancient)') ('eka' 'Ekajuk') ('elx' 'Elamite') ('en' 'eng' 'English') ('enm' 'English, Middle (1100-1500)') ('eo' 'epo' 'Esperanto') ('et' 'est' 'Estonian') ('ee' 'ewe' 'Ewe') ('ewo' 'Ewondo') ('fan' 'Fang') ('fo' 'fao' 'Faroese') ('fat' 'Fanti') ('fj' 'fij' 'Fijian') ('fil' 'Filipino; Pilipino') ('fi' 'fin' 'Finnish') ('fiu' 'Finno-Ugrian (Other)') ('fon' 'Fon') ('fr' 'fre' 'French') ('frm' 'French, Middle (ca.1400-1600)') ('fro' 'French, Old (842-ca.1400)') ('frr' 'Northern Frisian') ('frs' 'Eastern Frisian') ('fy' 'fry' 'Western Frisian') ('ff' 'ful' 'Fulah') ('fur' 'Friulian') ('gaa' 'Ga') ('gay' 'Gayo') ('gba' 'Gbaya') ('gem' 'Germanic (Other)') ('ka' 'geo' 'Georgian') ('de' 'ger' 'German') ('gez' 'Geez') ('gil' 'Gilbertese') ('gd' 'gla' 'Gaelic; Scottish Gaelic') ('ga' 'gle' 'Irish') ('gl' 'glg' 'Galician') ('gv' 'glv' 'Manx') ('gmh' 'German, Middle High (ca.1050-1500)') ('goh' 'German, Old High (ca.750-1050)') ('gon' 'Gondi') ('gor' 'Gorontalo') ('got' 'Gothic') ('grb' 'Grebo') ('grc' 'Greek, Ancient (to 1453)') ('el' 'gre' 'Greek, Modern (1453-)') ('gn' 'grn' 'Guarani') ('gsw' 'Swiss German; Alemannic; Alsatian') ('gu' 'guj' 'Gujarati') ('gwi' 'Gwich''in') ('hai' 'Haida') ('ht' 'hat' 'Haitian; Haitian Creole') ('ha' 'hau' 'Hausa') ('haw' 'Hawaiian') ('he' 'heb' 'Hebrew') ('hz' 'her' 'Herero') ('hil' 'Hiligaynon') ('him' 'Himachali') ('hi' 'hin' 'Hindi') ('hit' 'Hittite') ('hmn' 'Hmong') ('ho' 'hmo' 'Hiri Motu') ('hsb' 'Upper Sorbian') ('hu' 'hun' 'Hungarian') ('hup' 'Hupa') ('iba' 'Iban') ('ig' 'ibo' 'Igbo') ('is' 'ice' 'Icelandic') ('io' 'ido' 'Ido') ('ii' 'iii' 'Sichuan Yi; Nuosu') ('ijo' 'Ijo languages') ('iu' 'iku' 'Inuktitut') ('ie' 'ile' 'Interlingue; Occidental') ('ilo' 'Iloko') ('ia' 'ina' 'Interlingua (International Auxiliary Language Association)') ('inc' 'Indic (Other)') ('id' 'ind' 'Indonesian') ('ine' 'Indo-European (Other)') ('inh' 'Ingush') ('ik' 'ipk' 'Inupiaq') ('ira' 'Iranian (Other)') ('iro' 'Iroquoian languages') ('it' 'ita' 'Italian') ('jv' 'jav' 'Javanese') ('jbo' 'Lojban') ('ja' 'jpn' 'Japanese') ('jpr' 'Judeo-Persian') ('jrb' 'Judeo-Arabic') ('kaa' 'Kara-Kalpak') ('kab' 'Kabyle') ('kac' 'Kachin; Jingpho') ('kl' 'kal' 'Kalaallisut; Greenlandic') ('kam' 'Kamba') ('kn' 'kan' 'Kannada') ('kar' 'Karen languages') ('ks' 'kas' 'Kashmiri') ('kr' 'kau' 'Kanuri') ('kaw' 'Kawi') ('kk' 'kaz' 'Kazakh') ('kbd' 'Kabardian') ('kha' 'Khasi') ('khi' 'Khoisan (Other)') ('km' 'khm' 'Central Khmer') ('kho' 'Khotanese') ('ki' 'kik' 'Kikuyu; Gikuyu') ('rw' 'kin' 'Kinyarwanda') ('ky' 'kir' 'Kirghiz; Kyrgyz') ('kmb' 'Kimbundu') ('kok' 'Konkani') ('kv' 'kom' 'Komi') ('kg' 'kon' 'Kongo') ('ko' 'kor' 'Korean') ('kos' 'Kosraean') ('kpe' 'Kpelle') ('krc' 'Karachay-Balkar') ('krl' 'Karelian') ('kro' 'Kru languages') ('kru' 'Kurukh') ('kj' 'kua' 'Kuanyama; Kwanyama') ('kum' 'Kumyk') ('ku' 'kur' 'Kurdish') ('kut' 'Kutenai') ('lad' 'Ladino') ('lah' 'Lahnda') ('lam' 'Lamba') ('lo' 'lao' 'Lao') ('la' 'lat' 'Latin') ('lv' 'lav' 'Latvian') ('lez' 'Lezghian') ('li' 'lim' 'Limburgan; Limburger; Limburgish') ('ln' 'lin' 'Lingala') ('lt' 'lit' 'Lithuanian') ('lol' 'Mongo') ('loz' 'Lozi') ('lb' 'ltz' 'Luxembourgish; Letzeburgesch') ('lua' 'Luba-Lulua') ('lu' 'lub' 'Luba-Katanga') ('lg' 'lug' 'Ganda') ('lui' 'Luiseno') ('lun' 'Lunda') ('luo' 'Luo (Kenya and Tanzania)') ('lus' 'Lushai') ('mk' 'mac' 'Macedonian') ('mad' 'Madurese') ('mag' 'Magahi') ('mh' 'mah' 'Marshallese') ('mai' 'Maithili') ('mak' 'Makasar') ('ml' 'mal' 'Malayalam') ('man' 'Mandingo') ('mi' 'mao' 'Maori') ('map' 'Austronesian (Other)') ('mr' 'mar' 'Marathi') ('mas' 'Masai') ('ms' 'may' 'Malay') ('mdf' 'Moksha') ('mdr' 'Mandar') ('men' 'Mende') ('mga' 'Irish, Middle (900-1200)') ('mic' 'Mi''kmaq; Micmac') ('min' 'Minangkabau') ('mis' 'Uncoded languages') ('mkh' 'Mon-Khmer (Other)') ('mg' 'mlg' 'Malagasy') ('mt' 'mlt' 'Maltese') ('mnc' 'Manchu') ('mni' 'Manipuri') ('mno' 'Manobo languages') ('moh' 'Mohawk') ('mo' 'mol' 'Moldavian') ('mn' 'mon' 'Mongolian') ('mos' 'Mossi') ('mul' 'Multiple languages') ('mun' 'Munda languages') ('mus' 'Creek') ('mwl' 'Mirandese') ('mwr' 'Marwari') ('myn' 'Mayan languages') ('myv' 'Erzya') ('nah' 'Nahuatl languages') ('nai' 'North American Indian') ('nap' 'Neapolitan') ('na' 'nau' 'Nauru') ('nv' 'nav' 'Navajo; Navaho') ('nr' 'nbl' 'Ndebele, South; South Ndebele') ('nd' 'nde' 'Ndebele, North; North Ndebele') ('ng' 'ndo' 'Ndonga') ('nds' 'Low German; Low Saxon; German, Low; Saxon, Low') ('ne' 'nep' 'Nepali') ('new' 'Nepal Bhasa; Newari') ('nia' 'Nias') ('nic' 'Niger-Kordofanian (Other)') ('niu' 'Niuean') ('nn' 'nno' 'Norwegian Nynorsk; Nynorsk, Norwegian') ('nb' 'nob' 'Bokml, Norwegian; Norwegian Bokml') ('nog' 'Nogai') ('non' 'Norse, Old') ('no' 'nor' 'Norwegian') ('nqo' 'N''Ko') ('nso' 'Pedi; Sepedi; Northern Sotho') ('nub' 'Nubian languages') ('nwc' 'Classical Newari; Old Newari; Classical Nepal Bhasa') ('ny' 'nya' 'Chichewa; Chewa; Nyanja') ('nym' 'Nyamwezi') ('nyn' 'Nyankole') ('nyo' 'Nyoro') ('nzi' 'Nzima') ('oc' 'oci' 'Occitan (post 1500); Provenal') ('oj' 'oji' 'Ojibwa') ('or' 'ori' 'Oriya') ('om' 'orm' 'Oromo') ('osa' 'Osage') ('os' 'oss' 'Ossetian; Ossetic') ('ota' 'Turkish, Ottoman (1500-1928)') ('oto' 'Otomian languages') ('paa' 'Papuan (Other)') ('pag' 'Pangasinan') ('pal' 'Pahlavi') ('pam' 'Pampanga; Kapampangan') ('pa' 'pan' 'Panjabi; Punjabi') ('pap' 'Papiamento') ('pau' 'Palauan') ('peo' 'Persian, Old (ca.600-400 B.C.)') ('fa' 'per' 'Persian') ('phi' 'Philippine (Other)') ('phn' 'Phoenician') ('pi' 'pli' 'Pali') ('pl' 'pol' 'Polish') ('pon' 'Pohnpeian') ('pt' 'por' 'Portuguese') ('pra' 'Prakrit languages') ('pro' 'Provenal, Old (to 1500)') ('ps' 'pus' 'Pushto; Pashto') ('qaa-qtz' 'Reserved for local use') ('qu' 'que' 'Quechua') ('raj' 'Rajasthani') ('rap' 'Rapanui') ('rar' 'Rarotongan; Cook Islands Maori') ('roa' 'Romance (Other)') ('rm' 'roh' 'Romansh') ('rom' 'Romany') ('ro' 'rum' 'Romanian') ('rn' 'run' 'Rundi') ('rup' 'Aromanian; Arumanian; Macedo-Romanian') ('ru' 'rus' 'Russian') ('sad' 'Sandawe') ('sg' 'sag' 'Sango') ('sah' 'Yakut') ('sai' 'South American Indian (Other)') ('sal' 'Salishan languages') ('sam' 'Samaritan Aramaic') ('sa' 'san' 'Sanskrit') ('sas' 'Sasak') ('sat' 'Santali') ('sr' 'scc' 'Serbian') ('scn' 'Sicilian') ('sco' 'Scots') ('hr' 'scr' 'Croatian') ('sel' 'Selkup') ('sem' 'Semitic (Other)') ('sga' 'Irish, Old (to 900)') ('sgn' 'Sign Languages') ('shn' 'Shan') ('sid' 'Sidamo') ('si' 'sin' 'Sinhala; Sinhalese') ('sio' 'Siouan languages') ('sit' 'Sino-Tibetan (Other)') ('sla' 'Slavic (Other)') ('sk' 'slo' 'Slovak') ('sl' 'slv' 'Slovenian') ('sma' 'Southern Sami') ('se' 'sme' 'Northern Sami') ('smi' 'Sami languages (Other)') ('smj' 'Lule Sami') ('smn' 'Inari Sami') ('sm' 'smo' 'Samoan') ('sms' 'Skolt Sami') ('sn' 'sna' 'Shona') ('sd' 'snd' 'Sindhi') ('snk' 'Soninke') ('sog' 'Sogdian') ('so' 'som' 'Somali') ('son' 'Songhai languages') ('st' 'sot' 'Sotho, Southern') ('es' 'spa' 'Spanish; Castilian') ('sc' 'srd' 'Sardinian') ('srn' 'Sranan Tongo') ('srr' 'Serer') ('ssa' 'Nilo-Saharan (Other)') ('ss' 'ssw' 'Swati') ('suk' 'Sukuma') ('su' 'sun' 'Sundanese') ('sus' 'Susu') ('sux' 'Sumerian') ('sw' 'swa' 'Swahili') ('sv' 'swe' 'Swedish') ('syc' 'Classical Syriac') ('syr' 'Syriac') ('ty' 'tah' 'Tahitian') ('tai' 'Tai (Other)') ('ta' 'tam' 'Tamil') ('tt' 'tat' 'Tatar') ('te' 'tel' 'Telugu') ('tem' 'Timne') ('ter' 'Tereno') ('tet' 'Tetum') ('tg' 'tgk' 'Tajik') ('tl' 'tgl' 'Tagalog') ('th' 'tha' 'Thai') ('bo' 'tib' 'Tibetan') ('tig' 'Tigre') ('ti' 'tir' 'Tigrinya') ('tiv' 'Tiv') ('tkl' 'Tokelau') ('tlh' 'Klingon; tlhIngan-Hol') ('tli' 'Tlingit') ('tmh' 'Tamashek') ('tog' 'Tonga (Nyasa)') ('to' 'ton' 'Tonga (Tonga Islands)') ('tpi' 'Tok Pisin') ('tsi' 'Tsimshian') ('tn' 'tsn' 'Tswana') ('ts' 'tso' 'Tsonga') ('tk' 'tuk' 'Turkmen') ('tum' 'Tumbuka') ('tup' 'Tupi languages') ('tr' 'tur' 'Turkish') ('tut' 'Altaic (Other)') ('tvl' 'Tuvalu') ('tw' 'twi' 'Twi') ('tyv' 'Tuvinian') ('udm' 'Udmurt') ('uga' 'Ugaritic') ('ug' 'uig' 'Uighur; Uyghur') ('uk' 'ukr' 'Ukrainian') ('umb' 'Umbundu') ('und' 'Undetermined') ('ur' 'urd' 'Urdu') ('uz' 'uzb' 'Uzbek') ('vai' 'Vai') ('ve' 'ven' 'Venda') ('vi' 'vie' 'Vietnamese') ('vo' 'vol' 'Volapk') ('vot' 'Votic') ('wak' 'Wakashan languages') ('wal' 'Walamo') ('war' 'Waray') ('was' 'Washo') ('cy' 'wel' 'Welsh') ('wen' 'Sorbian languages') ('wa' 'wln' 'Walloon') ('wo' 'wol' 'Wolof') ('xal' 'Kalmyk; Oirat') ('xh' 'xho' 'Xhosa') ('yao' 'Yao') ('yap' 'Yapese') ('yi' 'yid' 'Yiddish') ('yo' 'yor' 'Yoruba') ('ypk' 'Yupik languages') ('zap' 'Zapotec') ('zbl' 'Blissymbols; Blissymbolics; Bliss') ('zen' 'Zenaga') ('za' 'zha' 'Zhuang; Chuang') ('znd' 'Zande languages') ('zu' 'zul' 'Zulu') ('zun' 'Zuni') ('zxx' 'No linguistic content') ('zza' 'Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki'))
! !

!WARenderer methodsFor:'public'!

absoluteUrlForResource: aString
	"Answer a resource URL for aString. The URL will always be absolute (ie. it will begin
	with a / or a URL scheme like http://. If aString already looks like a URL, an instance
	of WAUrl will be returned for it. If it looks like a path, it will be appended to the
	resource base URL configured in the RenderContext if there is one."

	^ self context absoluteUrlForResource: aString
! !

!WARenderer methodsFor:'public'!

actionBaseUrl
	^ self context actionBaseUrl
! !

!WARenderer methodsFor:'public'!

actionUrl
	^ self context actionUrl
! !

!WARenderer methodsFor:'accessing'!

callbacks
	"Answer the callback registry."

	^ self context callbacks
! !

!WARenderer methodsFor:'accessing'!

context
	"Answer the rendering context."

	^ context
! !

!WARenderer methodsFor:'accessing'!

document
	^ self context document
! !

!WARenderer methodsFor:'private'!

flush
	"Close any open elements and write any remaining data."
! !

!WARenderer methodsFor:'initialization'!

initializeWithContext: aRenderContext
	self initialize.
	context := aRenderContext
! !

!WARenderer methodsFor:'public'!

lastId
	"Answer the last identifier that was generated with #nextId."
	
	^ lastId
! !

!WARenderer methodsFor:'public'!

nextId
	"Answer an identifier string that is unique within this RenderContext."
	
	^ lastId := 'id', self callbacks advanceKey
! !

!WARenderer methodsFor:'public'!

render: anObject
	"Render anObject using this Renderer."
	
	anObject renderOn: self
! !

!WARenderer methodsFor:'public'!

text: anObject
	"Emit anObject onto the target document."

	self document print: anObject
! !

!WARenderer methodsFor:'public'!

urlForAction: aBlock
	"Answer an action URL that will evaluate aBlock when it is requested."

	^ self actionUrl withField: (self callbacks store: (WAActionCallback on: aBlock))
! !

!WARenderer class methodsFor:'instance creation'!

builder
	^ WABuilder on: self
! !

!WARenderer class methodsFor:'instance creation'!

context: aRenderContext
	^ self basicNew initializeWithContext: aRenderContext; yourself
! !

!WARenderer class methodsFor:'defaults'!

default
	^ default ifNil: [ default := WARenderer ]
! !

!WARenderer class methodsFor:'defaults'!

default: anObject
	default := anObject
! !

!WARenderer class methodsFor:'defaults'!

resetDefault
	WARenderer default: nil
! !

!WARenderer class methodsFor:'defaults'!

selectAsDefault
	WARenderer default: self
! !

!WARenderer class methodsFor:'defaults'!

unselectAsDefault
	(WARenderer default = self) ifTrue: [ WARenderer resetDefault ]
! !

!WAServerManager methodsFor:'accessing'!

adaptors
	^ adaptors copy
! !

!WAServerManager methodsFor:'private'!

canStart: aServerAdaptor
	"Because we can't rely on the socket refusing to listen on a port already in use, we simply check here if we know of another adaptor using this adaptor's port"
	^ aServerAdaptor isStopped 
			and: [ self adaptors noneSatisfy: [ :adaptor | 
				(adaptor = aServerAdaptor) not
					and: [ adaptor port = aServerAdaptor port
					and: [ adaptor isStopped not ] ] ] ]
! !

!WAServerManager methodsFor:'private'!

canStop: aServerAdaptor
	^ aServerAdaptor isStopped not
! !

!WAServerManager methodsFor:'initialization'!

initialize
	super initialize.
	adaptors := OrderedCollection new
! !

!WAServerManager methodsFor:'accessing'!

register: aServerAdaptor
	^ adaptors add: aServerAdaptor
! !

!WAServerManager methodsFor:'starting/stopping'!

restart: aServerAdaptor
	self stop: aServerAdaptor.
	self start: aServerAdaptor
! !

!WAServerManager methodsFor:'starting/stopping'!

restartAll
	adaptors do: [ :each | each restart ]
! !

!WAServerManager methodsFor:'starting/stopping'!

start: aServerAdaptor
	(self canStart: aServerAdaptor) ifTrue: [
		[ aServerAdaptor basicStart ]
			ifCurtailed: [ self stop: aServerAdaptor ] ]
! !

!WAServerManager methodsFor:'starting/stopping'!

startAll
	adaptors do: [ :each | each start ]
! !

!WAServerManager methodsFor:'starting/stopping'!

stop: aServerAdaptor
	(self canStop: aServerAdaptor) ifTrue: [ aServerAdaptor basicStop ]
! !

!WAServerManager methodsFor:'starting/stopping'!

stopAll
	adaptors do: [ :each | each stop ]
! !

!WAServerManager methodsFor:'accessing'!

unregister: aServerAdaptor
	self stop: aServerAdaptor.
	adaptors remove: aServerAdaptor.
	aServerAdaptor unregistered.
	^ aServerAdaptor
! !

!WAServerManager class methodsFor:'accessing'!

default
	^ default ifNil: [ default := self new ]
! !

!WAServerManager class methodsFor:'class initialization'!

initialize
	GRPlatform current
		addToStartUpList: self;
		addToShutDownList: self
! !

!WAServerManager class methodsFor:'system startup'!

shutDown
	self default adaptors do: [ :each | each shutDown ]
! !

!WAServerManager class methodsFor:'system startup'!

startUp
	self default adaptors do: [ :each | each startUp ]
! !

!WAServerManager class methodsFor:'class initialization'!

unload
	GRPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self
! !

!WASharedConfiguration methodsFor:'initialization'!

initializeWithName: aString
	self initialize.
	name := aString
! !

!WASharedConfiguration methodsFor:'testing'!

isShared
	^ true
! !

!WASharedConfiguration methodsFor:'accessing'!

name
	^ name
! !

!WASharedConfiguration methodsFor:'actions'!

unregister
	self class unregister: self
! !

!WASharedConfiguration class methodsFor:'accessing'!

configurations
	^ instances copy
! !

!WASharedConfiguration class methodsFor:'initialization'!

initialize
	instances := Set new
! !

!WASharedConfiguration class methodsFor:'instance creation'!

named: aString
	"Answer a shared configuration with the given name aString or answer a newly created one."
	
	^ instances
		detect: [ :each | each name = aString ]
		ifNone: [
			instances add: (self basicNew
				initializeWithName: aString;
				yourself) ]
! !

!WASharedConfiguration class methodsFor:'instance creation'!

new
	self error: 'Create or retrieve shared configurations by sending the message #named:'
! !

!WASharedConfiguration class methodsFor:'instance creation'!

unregister: aConfiguration
	"Removes aConfiguration from the list of known configurations."
	
	^ instances remove: aConfiguration ifAbsent: [ nil ]
! !

!WASystemConfiguration methodsFor:'private'!

attributes
	^ self description attributes
! !

!WASystemConfiguration methodsFor:'description'!

buildDescription
	| d |
	d := WAConfigurationDescription new.
	self describeOn: d.
	^ description := d
! !

!WASystemConfiguration methodsFor:'description'!

clearDescription
	description := nil.
	self clearSearchContexts.
! !

!WASystemConfiguration methodsFor:'copying'!

copy
	^ self error: 'SystemConfigurations are singletons and should not be copied.'
! !

!WASystemConfiguration methodsFor:'description'!

describeOn: conf
	"Should be implemented by subclasses to add attributes and value overrides to the configuration"
! !

!WASystemConfiguration methodsFor:'private'!

description
	^ description isNil 
		ifFalse: [ description ]
		ifTrue: [ self buildDescription ]
! !

!WASystemConfiguration methodsFor:'values'!

expressionAt: aSymbol ifAbsent: absentBlock
	^ self description expressions at: aSymbol ifAbsent: absentBlock.
! !

!WASystemConfiguration methodsFor:'attributes'!

localAttributeAt: aSymbol ifAbsent: absentBlock
	^ self attributes at: aSymbol ifAbsent: absentBlock
! !

!WASystemConfiguration methodsFor:'attributes'!

localAttributesDo: aBlock
	self attributes do: aBlock
! !

!WASystemConfiguration class methodsFor:'actions'!

clearAllDescriptions
	self allSubclasses do: [ :each | each instance clearDescription ]
! !

!WASystemConfiguration class methodsFor:'class initialization'!

initialize
	"Work around for Issue 643
	http://code.google.com/p/seaside/issues/detail?id=643"
	self clearAllDescriptions
! !

!WASystemConfiguration class methodsFor:'accessing'!

instance
	^ instance ifNil: [ instance := self basicNew initialize ]
! !

!WASystemConfiguration class methodsFor:'instance creation'!

new
	^ self error: 'SystemConfigurations are singletons. Use #instance.'
! !

!stx_goodies_seaside30_core class methodsFor:'documentation'!

extensionsVersion_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/core/extensions.st,v 1.2 2011/09/05 20:50:00 cg Exp $'
! !