'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 22:03:41'                     !

"{ Package: 'stx:goodies/seaside30/core/tests' }"

WAContextTest subclass:#WACacheTest
	instanceVariableNames:'cache key'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Tests-Core-Cache'
!

WACacheTest comment:''
!

!WACacheTest methodsFor:'running'!

setUp
	cache := WACache new.
	key := WAKeyGenerator current keyOfLength: 8
! !

!WACacheTest methodsFor:'tests'!

testAt
	self assert: (cache at: key) isNil.
	self assert: (cache at: (cache store: 1)) = 1
!

testAtIfAbsent
	self assert: (cache at: key ifAbsent: [ 0 ]) = 0.
	self assert: (cache at: (cache store: 1) ifAbsent: [ 0 ]) = 1
!

testClear
	key := cache store: 1.
	cache clear.
	self assert: (cache at: key) isNil.
	self assert: (cache keyAtValue: 1) isNil
!

testCopy
	| one two copy |
	one := cache store: 1.
	two := cache store: 2.
	
	copy := cache copy.
	copy removeKey: one.
	
	self assert: (cache at: one) = 1.
	self assert: (cache at: two) = 2.
	self assert: (cache keyAtValue: 1) = one.
	self assert: (cache keyAtValue: 2) = two.
	self assert: (copy at: two) = 2.
	self assert: (copy keyAtValue: 2) = two.
	self assert: (copy keyAtValue: 1) isNil.
	self assert: (copy at: one) isNil.
	
	self deny: (cache instVarNamed: 'mutex') == (copy instVarNamed: 'mutex').
	self deny: cache expiryPolicy == copy expiryPolicy.
	self deny: cache missStrategy == copy missStrategy.
	self deny: cache removalAction == copy removalAction.
	self deny: cache reapingStrategy == copy reapingStrategy.
!

testKeyAtValue
	key := cache store: 1.
	self assert: (cache keyAtValue: 1) = key.
	self assert: (cache keyAtValue: 2) isNil
!

testRemove
	key := cache store: 1.
	self assert: (cache remove: 1) = 1.
	self assert: (cache at: key) isNil.
	self assert: (cache keyAtValue: 1) isNil
!

testRemoveKey
	key := cache store: 1.
	self assert: (cache removeKey: key) = 1.
	self assert: (cache at: key) isNil.
	self assert: (cache keyAtValue: 1) isNil
!

testRetrieve
	self assert: (cache at: key) isNil.
	key := cache store: 1.
	self assert: (cache at: key) = 1
!

testSize
	self assert: cache isEmpty.
	cache store: #a.
	self assert: cache size = 1.
	cache store: #b.
	self assert: cache size = 2.
	cache remove: #a.
	self assert: cache size = 1.
! !

!WACacheTest methodsFor:'tests-policies'!

testLastAccess
	| keys |
	cache setExpiryPolicy: WALastAccessExpiryPolicy new.
	keys := (1 to: 30) collect: [ :index | cache store: index ].
	cache reap.
	keys do: [ :each | self assert: (cache at: each) notNil ]
!

testLeastRecentlyUsed
	| keys |
	cache setExpiryPolicy: WALeastRecentlyUsedExpiryPolicy new.
	keys := (1 to: 30) collect: [ :index | cache store: index ].
	cache reap.
	1 to: 9 do: [ :index | self assert: (cache at: (keys at: index)) isNil ].
	10 to: 30 do: [ :index | self assert: (cache at: (keys at: index)) = index ]
!

testLeastRecentlyUsedCopy
	| keys1 keys2 cacheCopy |
	cache setExpiryPolicy: WALeastRecentlyUsedExpiryPolicy new.
	keys1 := (1 to: 20) collect: [ :index | cache store: index ].
	keys2 := keys1 copy.
	cacheCopy := cache copy.
	
	"touch different objects in each cache"
	1 to: 9 by: 2 do: [ :index | self assert: (cache at: (keys1 at: index)) notNil ].
	2 to: 8 by: 2 do: [ :index | self assert: (cacheCopy at: (keys2 at: index)) notNil ].

	"now overflow the caches"
	keys1 := keys1 , ((21 to: 30) collect: [ :index | cache store: index ]).
	keys2 := keys2 , ((21 to: 30) collect: [ :index | cacheCopy store: index ]).
	cache reap.
	cacheCopy reap.

	"verify that different caches expired different objects"
	1 to: 9 by: 2 do: [ :index | self assert: (cache at: (keys1 at: index)) notNil ].
	1 to: 9 by: 2 do: [ :index | self assert: (cacheCopy at: (keys2 at: index)) isNil ].

	2 to: 8 by: 2 do: [ :index | self assert: (cache at: (keys1 at: index)) isNil ].
	2 to: 8 by: 2 do: [ :index | self assert: (cacheCopy at: (keys2 at: index)) notNil ].

	10 to: 30 do: [ :index | self assert: (cache at: (keys1 at: index)) notNil ].
	10 to: 30 do: [ :index | self assert: (cacheCopy at: (keys2 at: index)) notNil ].
! !

