'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 22:03:41'                     !

"{ Package: 'stx:goodies/seaside30/core/tests' }"

WARequestHandlerTest subclass:#WARegistryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Tests-Core-RequestHandling'
!

WARegistryTest comment:''
!

!WARegistryTest methodsFor:'private'!

newHandler
	^ WARegistry new
!

requestContextFor: aRequest
	^ WARequestContext request: aRequest response: WABufferedResponse
! !

!WARegistryTest methodsFor:'tests'!

testCopyRegistry
	| stored key copy stored2 key2 |
	stored := WARequestHandler new.
	key := handler register: stored.

	copy := handler copy.
	
	self assert: key = (handler keyFor: stored).
	"Since registry keys are dynamically assigned, we don't expect a copy to have the same
	keys and values."
	
	stored2 := WARequestHandler new.
	key2 := handler register: stored2.
	self assert: key2 = (handler keyFor: stored2).
	self should: [ copy keyFor: stored2 ] raise: Error.
	
	copy clear.
	self assert: key = (handler keyFor: stored).
!

testEmpty
	| context cookie |
	context := self requestContextFor: WARequest new.
	cookie := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: cookie isNil
!

testOneOfMany
	| cookie1 cookie2 cookie3 cookie4 actual request context |
	cookie1 := WARequestCookie key: handler keyCookieName value: '1234'.
	cookie2 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/';
		yourself.
	cookie3 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2';
		yourself.
	cookie4 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2/3';
		yourself.
	request := (WARequest method: 'GET' uri: '/1/2')
		setCookies: (Array
			with: cookie1
			with: cookie2
			with: cookie3
			with: cookie4);
		yourself.
	context := self requestContextFor: request.
	actual := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: actual == cookie3
!

testOneOfManySlashInCookie
	| cookie1 cookie2 cookie3 cookie4 actual request context |
	cookie1 := WARequestCookie key: handler keyCookieName value: '1234'.
	cookie2 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/';
		yourself.
	cookie3 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2/';
		yourself.
	cookie4 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2/3/';
		yourself.
	request := (WARequest method: 'GET' uri: '/1/2')
		setCookies: (Array
			with: cookie1
			with: cookie2
			with: cookie3
			with: cookie4);
		yourself.
	context := self requestContextFor: request.
	actual := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: actual == cookie3
!

testOneOfManySlashInUrl
	| cookie1 cookie2 cookie3 cookie4 actual request context |
	cookie1 := WARequestCookie key: handler keyCookieName value: '1234'.
	cookie2 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/';
		yourself.
	cookie3 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2';
		yourself.
	cookie4 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/1/2/3';
		yourself.
	request := (WARequest method: 'GET' uri: '/1/2/')
		setCookies: (Array
			with: cookie1
			with: cookie2
			with: cookie3
			with: cookie4);
		yourself.
	context := self requestContextFor: request.
	actual := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: actual == cookie3
!

testOnlyOne
	| cookie1 cookie2 request context |
	cookie1 := WARequestCookie key: handler keyCookieName value: '1234'.
	request := (WARequest method: 'GET' uri: '/')
		setCookies: (Array with: cookie1);
		yourself.
	context := self requestContextFor: request.
	cookie2 := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: cookie1 == cookie2
!

testWrongKey
	| cookie request context actual |
	cookie := WARequestCookie key: (handler keyCookieName, '-not') value: '1234'.
	request := (WARequest method: 'GET' uri: '/')
		setCookies: (Array with: cookie);
		yourself.
	context := self requestContextFor: request.
	actual := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: actual isNil
!

testWrongPrefix
	| cookie1 cookie2 actual request context |
	cookie1 := WARequestCookie key: handler keyCookieName value: '1234'.
	cookie2 := (WARequestCookie key: handler keyCookieName value: '1234')
		path: '/one';
		yourself.

	request := (WARequest method: 'GET' uri: '/oneo')
		setCookies: (Array
			with: cookie1
			with: cookie2);
		yourself.
	context := self requestContextFor: request.
	actual := handler cookieFromContext: context ifAbsent: [ nil ].
	self assert: actual == cookie1
! !

