'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 22:03:41'                     !

"{ Package: 'stx:goodies/seaside30/core/tests' }"

TestCase subclass:#WAResponseTest
	instanceVariableNames:'response'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Tests-Core-HTTP'
!

WAResponseTest comment:''
!

!WAResponseTest class methodsFor:'testing'!

isAbstract
	^ self name = #WAResponseTest
! !

!WAResponseTest methodsFor:'accessing'!

contents
	"Answer a string of the complete response as it is sent to the client."
	
	self subclassResponsibility
!

lines
	"Answer the response as an array of lines, to be used within the tests."

	| result stream |
	result := OrderedCollection new.
	stream := self contents readStream.
	[ stream atEnd ] whileFalse: [ result add: (stream greaseUpToAll: (String with: Character cr with: Character lf)) ].
	^ result asArray
!

response
	"Answer the response under test."

	^ response
! !

!WAResponseTest methodsFor:'private'!

assertLines: anArray
	self assert: self lines = anArray
! !

!WAResponseTest methodsFor:'tests'!

testContentType
	self response contentType: WAMimeType textHtml.
	self assert: self response contentType = WAMimeType textHtml.
	
	self response contentType: WAMimeType imagePng.
	self assert: self response contentType = WAMimeType imagePng
!

testIsCommitted
	self deny: self response isCommitted.
	self response contentType: 'text/html'.

	self deny: self response isCommitted.
	self response nextPutAll: 'Visit <a href="http://www.seaside.st">seaside.st</a>.'.

	self assertLines: #('HTTP/1.1 200 OK' 'Content-Type: text/html' 'Content-Length: 53' '' 'Visit <a href="http://www.seaside.st">seaside.st</a>.')
!

testStreaming
	| stream |
	self response notFound.
	
	" This is a normal write stream that supports the standard WriteStream protocol. All Smalltalk dialects should support common messages such as #nextPut:, #nextPutAll:, #print:, #crlf, etc."
	stream := self response stream.
	stream nextPutAll: 'A long long way from home.'.
	stream crlf.
	stream nextPut: $!!.
	stream print: 123.
	
	self assertLines: #('HTTP/1.1 404 Not Found' 'Content-Length: 32' '' 'A long long way from home.' '!!123')
! !

!WAResponseTest methodsFor:'tests-cache'!

testCacheFor
	self response cacheFor: (Duration days: 1 hours: 0 minutes: 0 seconds: 0).
	self assert: (self response headerAt: 'Pragma') isNil.
	self assert: (self response headerAt: 'Expires') notNil.
	self assert: (self response headerAt: 'Cache-Control') = 'max-age=86400'
!

testCacheForever
	self response cacheForever.
	self assert: (self response headerAt: 'Pragma') isNil.
	self assert: (self response headerAt: 'Expires') notNil.
	self assert: (self response headerAt: 'Cache-Control') isNil
!

testCacheForeverExpiryDate
	| expiry delta |
	expiry := self response cacheForeverExpiryDate.
	delta := expiry - DateAndTime now.
	self assert: delta days > 7.
	self assert: delta days < 356
!

testDefaultCache
	self deny: (self response headers includesKey: 'Pragma').
	self deny: (self response headers includesKey: 'Expires').
	self deny: (self response headers includesKey: 'Cache-Control')
!

testDeleteCookie
	| cookie actual |
	cookie := (WARequestCookie key: 'key' value: 'value')
		path: '/highway/to/hell';
		domain: 'hotel'
		yourself.
	self response deleteCookie: cookie.
	self assert: self response cookies size = 1.
	actual := self response cookies first.
	self assert: cookie ~~ actual.
	self assert: actual domain = 'hotel'.
	self assert: actual path = '/highway/to/hell'
!

testDoNotCache
	self response doNotCache.
	self assert: (self response headerAt: 'Pragma') = 'no-cache'.
	self assert: (self response headerAt: 'Expires') notNil.
	self assert: (self response headerAt: 'Cache-Control') = 'no-cache, must-revalidate'
!

testDoNotCacheExpiryDate
	| expiry delta |
	expiry := self response doNotCacheExpiryDate.
	delta := expiry - DateAndTime now.
	self assert: delta days < -7
! !

!WAResponseTest methodsFor:'tests-conveniance'!

testAttachmentWithFileName
	self response attachmentWithFileName: 'foo.txt'.
	self assertLines: #('HTTP/1.1 200 OK' 'Content-Disposition: attachment; filename="foo.txt"' 'Content-Length: 0' '')
!

testBasicAuthentication
	self response basicAuthenticationRealm: 'seaside'.
	self assertLines: #('HTTP/1.1 401 Unauthorized' 'WWW-Authenticate: Basic realm="seaside"' 'Content-Length: 0' '')
!

testInlineWithFileName
	self response inlineWithFileName: 'bar.txt'.
	self assertLines: #('HTTP/1.1 200 OK' 'Content-Disposition: inline; filename="bar.txt"' 'Content-Length: 0' '')
!

testRedirectTo
	self response redirectTo: 'http://www.seaside.st/'.
	self assertLines:  #('HTTP/1.1 302 Found' 'Location: http://www.seaside.st/' 'Content-Length: 0' '')
!

testRefreshAfterLocation
	self response refreshAfter: 3 seconds location: 'http://www.seaside.st/'.
	self assertLines: #('HTTP/1.1 200 OK' 'Refresh: 3; http://www.seaside.st/' 'Content-Length: 0' '')
!

testStrictTransportSecurity
	self response strictTransportSecurity: 500 includeSubDomains: true.
	self assert: (self response headerAt: 'Strict-Transport-Security') = 'max-age=500; includeSubDomains'.
	
	self response strictTransportSecurity: 500 includeSubDomains: false.
	self assert: (self response headerAt: 'Strict-Transport-Security') = 'max-age=500'.
! !

!WAResponseTest methodsFor:'tests-documents'!

testDocumentMimeType
	self response document: 'alert("Seaside")' mimeType: 'text/javascript'.
	self assertLines: #('HTTP/1.1 200 OK' 'Content-Type: text/javascript' 'Content-Length: 16' '' 'alert("Seaside")')
!

testDocumentMimeTypeFileName
	self response document: 'foo;bar;zork' mimeType: 'text/csv' fileName: 'table.csv'.
	self assertLines: #('HTTP/1.1 200 OK' 'Content-Disposition: attachment; filename="table.csv"' 'Content-Type: text/csv' 'Content-Length: 12' '' 'foo;bar;zork')
! !

!WAResponseTest methodsFor:'tests-headers'!

testHeadersAt
	self assert: (self response headerAt: 'foo') isNil.
	self response headerAt: 'foo' put: 'bar'.
	self assert: (self response headerAt: 'foo') = 'bar'
!

testHeadersAtAppend
	self assert: (self response headerAt: 'foo' append: 'bar') = 'bar'.
	self assert: (self response headerAt: 'foo') = 'bar'.
	self assert: (self response headerAt: 'foo' append: 'zork') = 'zork'.
	self assert: (self response headerAt: 'foo') = 'bar'.
	self assert: (self response headers size) = 2
!

testHeadersAtIfAbsent
	self assert: (self response headerAt: 'foo' ifAbsent: [ 'zork' ]) = 'zork'.
	self response headerAt: 'foo' put: 'bar'.
	self assert: (self response headerAt: 'foo' ifAbsent: [ 'zork' ]) = 'bar'
!

testHeadersAtPut
	self assert: (self response headerAt: 'foo' put: 'bar') = 'bar'.
	self assert: (self response headerAt: 'foo') = 'bar'.
	self assert: (self response headerAt: 'foo' put: 'zork') = 'zork'.
	self assert: (self response headerAt: 'foo') = 'zork'.
	self assert: (self response headers size) = 1
!

testHeadersRemoveAt
	self response headerAt: 'foo' put: 'bar'.
	self response headerAt: 'bar' put: 'baz'.
	self assert: (self response removeHeaderAt: 'foo') = #('bar').
	self assert: (self response removeHeaderAt: 'bar') = #('baz').
	self assert: (self response headers isEmpty)
! !

!WAResponseTest methodsFor:'tests-status'!

testCodes
	| names codes |
	names := #( authenticationFailed forbidden found gone internalError notFound notImplemented ).
	codes := #( 401 403 302 410 500 404 501 ).
	names with: codes do: [ :name :code | 
		self response perform: name.
		self assert: self response status = code.
		self assert: self response message = (self response class messageForStatus: code) ]
!

testMessage
	self assert: self response message = 'OK'.
	
	self response status: 404.
	self assert: self response message = 'Not Found'.
	
	self response status: 290.
	self assert: self response message isNil.
	
	self response status: 290 message: 'Seaside Rocks'.
	self assert: self response message = 'Seaside Rocks'.
	
!

testStatus
	self assert: self response status = 200.
	self response status: 404.
	self assert: self response status = 404
! !

