'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 22:03:41'                     !

"{ Package: 'stx:goodies/seaside30/core/tests' }"

WAContextTest subclass:#WAUrlTest
	instanceVariableNames:'url'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Tests-Core-HTTP'
!

WAUrlTest comment:''
!

!WAUrlTest methodsFor:'running'!

setUp
	url := WAUrl new
! !

!WAUrlTest methodsFor:'testing-parsing'!

absoluteExamples
	^ #( 'http://www.seaside.st/'
		 'http://www.seaside.st:8080/'
		 'http://lukas@www.seaside.st/'
		 'http://lukas:renggli@www.seaside.st/'
		 'http://www.seaside.st/resources'
		 'http://www.seaside.st/styles/favicon.png'
		 'http://www.seaside.st/#top'
		 'http://www.seaside.st/styles#top'
		 'http://www.seaside.st/about/screenshots?_c'
		 'http://www.seaside.st/about/screenshots?_s=XrkNBpErXTKlVRhn'
		 'http://www.seaside.st/about/screenshots?_s=XrkNBpErXTKlVRhn&_k=lZWXfESZ'
		 'http://www.seaside.st/about/screenshots?_s=XrkNBpErXTKlVRhn&_k=lZWXfESZ&_n&47'
		 'http://www.seaside.st/about/screenshots?_s=XrkNBpErXTKlVRhn&_k=lZWXfESZ&_n&47#149361951'
		 'http://www.seaside.st/;jsessionid=42?redirectcounter=1'
		 'http://www.seaside.st/seaside/files;old/WAStandardFiles;key1=value1,value3;key2=value2/seaside.jpg;jsessionid=42?redirectcounter=1'
		 'http://lukas:renggli@www.seaside.st/about/screenshots?_s=XrkNBpErXTKlVRhn&_k=lZWXfESZ&_n&47#149361951'
		 'http://lukas:renggli@www.seaside.st:8080/about/screenshots?_s=XrkNBpErXTKlVRhn&_k=lZWXfESZ&_n&47#149361951' )
!

ambigousExamples
	"These examples consist of many edge cases that are sometimes not quite legal. The test assures that they no error is thrown and that they parse to something meaningful anyway."
	
	^ #( "scheme"
		 ( 'www.seaside.st' '/www.seaside.st' )
		 ( 'http://www.seaside.st:443/' 'http://www.seaside.st:443/' )
		 ( 'https://www.seaside.st:80/' 'https://www.seaside.st:80/' )
		
		 "port"
		 ( 'http://www.seaside.st:/' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st:8x/' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st:x8/' 'http://www.seaside.st/' )
		
		 "host"
		 ( 'http://' '/' )
		 ( 'http://@' 'http://%40/' )
		
		 "username, password"
		 ( 'http://@www.seaside.st/' 'http://%40www.seaside.st/' )
		 ( 'http://:lukas@www.seaside.st/' 'http://:lukas@www.seaside.st/' )
		 ( 'http://lukas:@www.seaside.st/' 'http://lukas:@www.seaside.st/' )
		
		 "path"
		 ( 'http://www.seaside.st' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st//' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/.' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/./' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/..' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/../' 'http://www.seaside.st/' )
		
		 "query"
		 ( 'http://www.seaside.st/?' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/??' 'http://www.seaside.st/?%3F' )
		 ( 'http://www.seaside.st/?=2' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/?&' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/?1==' 'http://www.seaside.st/?1=%3D' )
		 ( 'http://www.seaside.st/?1=2&' 'http://www.seaside.st/?1=2' )
		 
		
		 "fragment"
		 ( 'http://www.seaside.st/#' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/##' 'http://www.seaside.st/#%23' )
		 ( 'http://www.seaside.st/#?a=b' 'http://www.seaside.st/#%3Fa%3Db' ) )
!

equalExamples
	"These examples about URI equivalence come from Sam Ruby and are quite hardcore. Java and Perl fail them, C# passes them.
	http://intertwingly.net/blog/2004/07/31/URI-Equivalence"
	
	^ #( 
		 ( 'http://example.com/'				'http://example.com' )
		 ( 'http://example.com/' 				'HTTP://example.com/' )
		 ( 'http://example.com/' 				'http://example.com:/' )
		 ( 'http://example.com/' 				'http://example.com:80/' )
		 ( 'http://example.com/' 				'http://Example.com/' )
		 ( 'http://example.com/~smith/'		'http://example.com/%7Esmith/' )
		 ( 'http://example.com/~smith/'		'http://example.com/%7esmith/' )
		 ( 'http://example.com/%7Esmith/'	'http://example.com/%7esmith/' )
		 ( 'http://example.com/foo/bar/'		'http://example.com/foo//bar/' )
			
		"The following example requires Unicode normalization, something that no Smalltalk implementation is able to perform.
		 ( 'http://example.com/%C3%87'		'http://example.com/C%CC%A7' )" )
!

relativeExamples
	^ #( ( 'http://www.seaside.st/foo' '' 'http://www.seaside.st/foo' )
		 ( 'http://www.seaside.st/foo' '.' 'http://www.seaside.st/foo' )
		 ( 'http://www.seaside.st/foo' './bar' 'http://www.seaside.st/foo/bar' )
		 ( 'http://www.seaside.st/foo' 'bar' 'http://www.seaside.st/foo/bar' )
		 ( 'http://www.seaside.st/foo' 'bar/.' 'http://www.seaside.st/foo/bar' )
		 ( 'http://www.seaside.st/foo' 'bar/..' 'http://www.seaside.st/foo' )
		 ( 'http://www.seaside.st/foo' 'bar/../..' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/foo' 'bar/../../..' 'http://www.seaside.st/' )

		 ( 'http://www.seaside.st/foo' '/' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/foo' '/.' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/foo' '/./bar' 'http://www.seaside.st/bar' )
		 ( 'http://www.seaside.st/foo' '/bar' 'http://www.seaside.st/bar' )
		 ( 'http://www.seaside.st/foo' '/bar/.' 'http://www.seaside.st/bar' )
		 ( 'http://www.seaside.st/foo' '/bar/..' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/foo' '/bar/../..' 'http://www.seaside.st/' )
		 ( 'http://www.seaside.st/foo' '/bar/../../..' 'http://www.seaside.st/' ) )
!

testIsSsl
	url := WAUrl absolute: 'https://www.seaside.st/'.
	self assert: url isSsl.
	url := WAUrl absolute: 'http://www.seaside.st/'.
	self deny: url isSsl.
	url := WAUrl absolute: 'ftps://www.seaside.st/'.
	self assert: url isSsl.
	url := WAUrl absolute: 'ftp://www.seaside.st/'.
	self deny: url isSsl.
	self deny: WAUrl new isSsl
!

testParseAbsolute
	self absoluteExamples do: [ :each | 
		self 
			assert: (WAUrl absolute: each) greaseString = each
			description: each
			resumable: true ]
!

testParseAmbigous
	self ambigousExamples do: [ :each | 
		self 
			assert: (WAUrl absolute: each first) greaseString = each last
			description: each last
			resumable: true ]
!

testParseEqual
	self equalExamples do: [ :each |
		self 
			assert: (WAUrl absolute: each first) = (WAUrl absolute: each second)
		 	description: each second resumable: true ]
!

testParseRelative
	self relativeExamples do: [ :each | 
		self 
			assert: (WAUrl 
				absolute: each first
				relative: each second) greaseString = each last
			description: each last
			resumable: true ]
! !

!WAUrlTest methodsFor:'tests'!

testConversion
	| converted |
	url
		host: 'example.com';
		addToPath: 'foo';
		addToPath: 'bar'.
	
	converted := 'http://example.com/foo/bar' seasideUrl.
	
	self assert: (converted isKindOf: WAUrl).
	self assert: url = converted.
!

testFragment
	url fragment: 'id'.
	self assert: url fragment = 'id'.
	self assert: url greaseString = '/#id'
!

testHost
	url host: 'seaside.st'.
	self assert: url host = 'seaside.st'.
	self assert: url greaseString = 'http://seaside.st/'
!

testPassword
	url
		host: 'seaside.st';
		user: 'foo';
		password: 'bar'.
	self assert: url password = 'bar'.
	self assert: url greaseString = 'http://foo:bar@seaside.st/'
!

testPath
	self assert: url greaseString = '/'.
	url path add: 'aa'.
	self assert: url greaseString = '/aa'.
	url path add: 'bb'.
	self assert: url greaseString = '/aa/bb'.
	self assert: url path asArray = #('aa' 'bb' )
!

testPort
	url
		host: 'seaside.st';
		port: 8080.
	self assert: url port = 8080.
	self assert: url greaseString = 'http://seaside.st:8080/'
!

testQueryFields
	url queryFields 
		at: '1'
		put: nil.
	self assert: url greaseString = '/?1'.
	url queryFields 
		at: '2'
		put: 'foo'.
	self assert: url greaseString = '/?1&2=foo'.
	url queryFields 
		at: '3'
		put: 123.
	self assert: url greaseString = '/?1&2=foo&3=123'.
	url queryFields 
		at: '4'
		put: 'foo bar&zork'.
	self assert: url greaseString = '/?1&2=foo&3=123&4=foo%20bar%26zork'.
	url queryFields 
		at: '1'
		add: 'dupe'.
	self assert: url greaseString = '/?1&2=foo&3=123&4=foo%20bar%26zork&1=dupe'.
!

testScheme
	url
		scheme: 'https';
		host: 'seaside.st'.
	self assert: url scheme = 'https'.
	self assert: url greaseString = 'https://seaside.st/'.
	"Tests from Boris"
	url host: 'seaside.st'.
	url
		port: 80;
		scheme: 'http'.
	self assert: url greaseString = 'http://seaside.st/'.
	url
		port: 443;
		scheme: 'https'.
	self assert: url greaseString = 'https://seaside.st/'.
	url
		port: 80;
		scheme: #http.
	self assert: url greaseString = 'http://seaside.st/'.
	url
		port: 443;
		scheme: #https.
	self assert: url greaseString = 'https://seaside.st/'
!

testSlash
	url host: 'seaside.st'.
	self assert: (url
			slash: true;
			greaseString) = 'http://seaside.st/'.
	self assert: (url
			slash: false;
			greaseString) = 'http://seaside.st/'.
	url addToPath: 'foo'.
	self assert: (url
			slash: true;
			greaseString) = 'http://seaside.st/foo/'.
	self assert: (url
			slash: false;
			greaseString) = 'http://seaside.st/foo'
!

testTakeServerParametersFromRequest
	| request |
	request := WARequest new
		setHeaders: (Dictionary new
			at: 'host' put: 'www.seaside.st:8080';
			yourself);
		yourself.
	url takeServerParametersFromRequest: request.
	self assert: url printString = 'http://www.seaside.st:8080/'
!

testTakeServerParametersFromRequestBase
	| request |
	request := (WARequest 
		method: 'GET'
		uri: (WAUrl new
				scheme: 'https';
				yourself))
		setHeaders: (Dictionary new
				at: 'host' put: 'www.thehost.com:1337';
				yourself);
		yourself.
	url takeServerParametersFromRequest: request.
	self assert: url greaseString = 'https://www.thehost.com:1337/'
!

testTakeServerParametersFromRequestHttps
	| request |
	request := WARequest new
		setHeaders: (Dictionary new
				at: 'host' put: 'www.seaside.st';
				at: 'base' put: 'foo/bar';
				yourself);
		yourself.
	url takeServerParametersFromRequest: request.
	self assert: url greaseString = 'http://www.seaside.st/foo/bar'
!

testUser
	url
		host: 'seaside.st';
		user: 'foo'.
	self assert: url user = 'foo'.
	self assert: url greaseString = 'http://foo@seaside.st/'
!

testUserinfo
	url
		host: 'seaside.st';
		userinfo: 'foo:bar'.
	self assert: url userinfo = 'foo:bar'.
	self assert: url greaseString = 'http://foo%3Abar@seaside.st/'.
	"A colon needs to be escaped unless it is separating a (deprecated) password
	part. This password part must be specified with #password: because we can't
	tell the difference between the two types of colons in #userinfo:."
	self assert: url user = 'foo:bar'.
	self assert: url password isNil.
	url user: 'foo'; password: 'bar'.
	self assert: url userinfo = 'foo:bar'.
	self assert: url user = 'foo'.
	self assert: url password = 'bar'
	
! !

!WAUrlTest methodsFor:'tests-adding'!

testAddAllToPath
	url addAllToPath: #('files' 'WAStandardFiles' 'seaside.jpg' ).
	self assert: url path size = 3.
	self assert: url path first = 'files'.
	self assert: url path last = 'seaside.jpg'.
	self assert: url greaseString = '/files/WAStandardFiles/seaside.jpg'
!

testAddField
	url addField: 'x'.
	self assert: url greaseString = '/?x'.
	url addField: 'y'.
	self assert: url greaseString = '/?x&y'.
	url addField: 'x'.
	self assert: url greaseString = '/?x&y&x'
!

testAddFieldValue
	url 
		addField: 'x'
		value: 1.
	self assert: url greaseString = '/?x=1'.
	url 
		addField: 'y'
		value: 2.
	self assert: url greaseString = '/?x=1&y=2'.
	url 
		addField: 'x'
		value: 3.
	self assert: url greaseString = '/?x=1&y=2&x=3'
!

testAddPathParameter
	url addAllToPath: #('seaside' 'files' 'WAStandardFiles' 'seaside.jpg' ).
	url addField: 'redirectcounter' value: 1.
	url addPathParameterAt: 2 key: 'old'.
	url addPathParameterAt: 3 key: 'key1' value: 'value1'.
	url addPathParameterAt: 3 key: 'key2' value: 'value2'.
	url addPathParameterAt: 3 key: 'key1' value: 'value3'.
	url addPathParameterAt: 4 key: 'jsessionid' value: 42.
	
	self assert: (url pathParameterAt: 1 key: 'jsessionid' ifAbsent: [ 'foo' ]) = 'foo'.
	self assert: (url pathParameterAllAt: 1 key: 'jsessionid' ifAbsent: [ 'foo' ]) = 'foo'.
	
	self assert: (url pathParameterAllAt: 2 key: 'jsessionid' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAt: 2 key: 'jsessionid' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAt: 2 key: 'old' ifAbsent: [ 'bar' ]) isNil.
	self assert: (url pathParameterAllAt: 2 key: 'old' ifAbsent: [ 'bar' ]) = (Array with: nil).
	
	self assert: (url pathParameterAt: 3 key: 'jsessionid' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAllAt: 3 key: 'jsessionid' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAt: 3 key: 'key1' ifAbsent: [ 'bar' ]) = 'value1'.
	self assert: (url pathParameterAllAt: 3 key: 'key1' ifAbsent: [ 'bar' ]) = (Array with: 'value1' with: 'value3').
	self assert: (url pathParameterAt: 3 key: 'key2' ifAbsent: [ 'bar' ]) = 'value2'.
	self assert: (url pathParameterAllAt: 3 key: 'key2' ifAbsent: [ 'bar' ]) = (Array with: 'value2').
	
	self assert: (url pathParameterAllAt: 4 key: 'key1' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAt: 4 key: 'key1' ifAbsent: [ 'bar' ]) = 'bar'.
	self assert: (url pathParameterAt: 4 key: 'jsessionid' ifAbsent: [ 'bar' ]) = 42.
	self assert: (url pathParameterAllAt: 4 key: 'jsessionid' ifAbsent: [ 'bar' ]) = (Array with: 42).
	
	self assert: url greaseString = '/seaside/files;old/WAStandardFiles;key1=value1,value3;key2=value2/seaside.jpg;jsessionid=42?redirectcounter=1'
!

testAddToPathEmptyString
	url addToPath: ''.
	self assert: url path size = 0.
	self assert: url greaseString = '/'
!

testAddToPathWithLeadingAndTrailingSlash
	url addToPath: '/files/WAStandardFiles/'.
	self assert: url path size = 2.
	self assert: url path first = 'files'.
	self assert: url path second = 'WAStandardFiles'.
	self assert: url greaseString = '/files/WAStandardFiles/'
!

testAddToPathWithLeadingSlash
	url addToPath: '/files/WAStandardFiles/seaside.jpg'.
	self assert: url path size = 3.
	self assert: url path first = 'files'.
	self assert: url path second = 'WAStandardFiles'.
	self assert: url path third = 'seaside.jpg'.
	self assert: url greaseString = '/files/WAStandardFiles/seaside.jpg'
!

testAddToPathWithTrailingSlashOnly
	url addToPath: 'files/WAStandardFiles/'.
	self assert: url path size = 2.
	self assert: url path first = 'files'.
	self assert: url path second = 'WAStandardFiles'.
	self assert: url greaseString = '/files/WAStandardFiles/'
!

testAddToPathWithoutLeadingSlash
	url addToPath: 'files/WAStandardFiles/seaside.jpg'.
	self assert: url path size = 3.
	self assert: url path first = 'files'.
	self assert: url path second = 'WAStandardFiles'.
	self assert: url path third = 'seaside.jpg'.
	self assert: url greaseString = '/files/WAStandardFiles/seaside.jpg'
! !

!WAUrlTest methodsFor:'tests-comparing'!

testEqual
	url host: 'seaside.st'; addField: 'foo' value: 'bar'. 
	self assert: url = url.
	self deny: url = WAUrl new.
	self deny: url = url printString.
	self deny: url = (url withoutQuery).
	self deny: url = (url withField: 'zork').
	self deny: url = (url withField: 'zork' value: 'zonk')
!

testHash
	url host: 'seaside.st'.
	self assert: url hash = url hash.
	self assert: url hash = url copy hash
! !

!WAUrlTest methodsFor:'tests-copy'!

testCopy
	| copy |
	url addToPath: 'foo/bar'.
	url addField: 'foo' value: 'bar'.
	copy := url copy.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/foo/bar/zork?foo=bar&zork'.
	self assert: copy printString = '/foo/bar?foo=bar'
	
!

testCopyWithPathParameter
	| copy |
	url addToPath: 'zork'.
	copy := url copy.
	url addPathParameterAt: 1 key: 'foo'.	
	self assert: url printString = '/zork;foo'.
	self assert: copy printString = '/zork'
!

testCopyWithPathParameterValue
	| copy |
	url addToPath: 'zork'.
	copy := url copy.
	url addPathParameterAt: 1 key: 'foo' value: 'bar'.
	self assert: url printString = '/zork;foo=bar'.
	self assert: copy printString = '/zork'
!

testWith
	| copy |
	copy := url with: 'with'.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/zork?zork'.
	self assert: copy printString = '/with'
!

testWithField
	| copy |
	copy := url withField: 'foo'.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/zork?zork'.
	self assert: copy printString = '/?foo'
!

testWithFieldValue
	| copy |
	copy := url withField: 'foo' value: 'bar'.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/zork?zork'.
	self assert: copy printString = '/?foo=bar'
!

testWithoutFragment
	| copy |
	url fragment: 'foo'.
	copy := url withoutFragment.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/zork?zork#foo'.
	self assert: copy printString = '/'
!

testWithoutQuery
	| copy |
	url addField: 'foo' value: 'bar'.
	copy := url withoutQuery.
	url addToPath: 'zork'.
	url addField: 'zork'.
	self assert: url printString = '/zork?foo=bar&zork'.
	self assert: copy printString = '/'
! !

!WAUrlTest methodsFor:'tests-encoding'!

testDecodedWith
	"http://s:er:pa:@localhost:8080/foo/bc/df?=stgetc&#trget"
	| pathArray expectedPathArray |
	url := (WAUrl absolute: 'http://%c3%b9s%3aer:pa%3A%c3%9f@localhost:8080/foo/%C3%A4bc/d%c3%aaf?%c3%b5=st%c3%aegetc&%c3%b4#t%c3%a5rget') decodedWith: (GRCodec forEncoding: 'utf-8').
	self assert: url scheme = 'http'.
	self assert: url userinfo = (self asString: #(249 115 58 101 114 58 112 97 58 223 )).
	self assert: url user = (self asString: #(249 115 58 101 114 )).
	self assert: url password = (self asString: #(112 97 58 223 )).
	self assert: url host = 'localhost'.
	self assert: url port = 8080.
	pathArray := url path asArray.
	self assert: pathArray size = 3.
	"must not use Collection >> #= due to Enterprise Smalltalk (tm)"
	expectedPathArray := Array 
		with: 'foo'
		with: (self asString: #(228 98 99 ))
		with: (self asString: #(100 234 102 )).
	expectedPathArray keysAndValuesDo: [ :index :each | self assert: (pathArray at: index) = each ].
	self assert: url queryFields size = 2.
	self assert: (url queryFields at: (self asString: #(245 ))) = (self asString: #(
					115
					116
					238
					103
					101
					116
					99
				)).
	self assert: (url queryFields includesKey: (self asString: #(244 ))).
	self assert: (url queryFields at: (self asString: #(244 ))) isNil.
	""
	self assert: url fragment = (self asString: #(116 229 114 103 101 116 ))
!

testPathEncoding
	url path: #( 'foo/bar' ).
	self assert: url printString = '/foo%2Fbar'.

	url path: #( 'foo bar' ).
	self assert: url printString = '/foo%20bar'.

	url path: #( 'foo+bar' ).
	self assert: url printString = '/foo%2Bbar'.

	url path: #( 'foo%bar' ).
	self assert: url printString = '/foo%25bar'
!

testPathString
	self assert: url pathString = '/'.

	url path: #( 'foo' 'bar' ).
	self assert: url pathString = '/foo/bar'.
!

testQueryEmpty
	self assert: url queryFields isEmpty.
	self assert: url printString = '/'
!

testQueryEncoding
	self
		assert: (url withField: '/' value: ' ')
			printString = '/?%2F=%20'.
	self
		assert: (url withField: '+' value: '%')
			printString = '/?%2B=%25'.
	self
		assert: (url withField: '?' value: '&')
			printString = '/?%3F=%26'.
	self
		assert: (url withField: '[' value: ']')
			printString = '/?%5B=%5D'.
	self
		assert: (url withField: '=' value: '<')
			printString = '/?%3D=%3C'
!

testQueryString
	self assert: url queryString = ''.

	url addField: 'foo'.
	self assert: url queryString = 'foo'.
	url addField: 'foo' value: 'bar'.
	self assert: url queryString = 'foo&foo=bar'.
	url addField: 'foo#' value: 'bar#'.
	self assert: url queryString = 'foo&foo=bar&foo%23=bar%23'.
!

testRelativeTo
	| first second relativePath |
	first := WAUrl absolute: 'http://www.seaside.st/news'.
	second := WAUrl absolute: 'http://www.seaside.st/news/2008/April'.
	relativePath := first relativeTo: second.
	"we can not use #= here because of the Smalltalk that makes the easy parts more fucking tedious than Java"
	self assert: relativePath size = 2.
	self assert: relativePath first = '2008'.
	self assert: relativePath second = 'April'.
! !

!WAUrlTest methodsFor:'utilities'!

asString: aCollectionOfIntegers
	^ String streamContents: [ :stream |
		aCollectionOfIntegers do: [ :each |
			stream nextPut: (Character codePoint: each) ] ]
! !

