"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRCodec
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core-Text'
!

GRCodec comment:'A codec defines how Seaside communicates without the outside world and how outside data is converted into the image (decoding) and back outside the image (encoding). The codec is essentially a stream factory that provides wrappers around standard streams. All streams do support binary mode for non-converted transfer.'
!


!GRCodec class methodsFor:'instance creation'!

forEncoding: aString
	"Answer a new codec instance for the given encoding name. Raise an WAUnsupportedEncodingError if the encoding name is not supported by this image."

	self allSubclassesDo: [ :each | 
		(each supportsEncoding: aString)
			ifTrue: [ ^ each basicForEncoding: aString ] ].
	^ self unsupportedEncoding: aString
! !

!GRCodec class methodsFor:'accessing'!

allCodecs
	"Answer all codecs supported in this system. This is a collection of codec instances."

	^ self subclasses
		inject: self codecs asArray
		into: [ :result :each | result , each allCodecs ]
!

codecs
	"Answer a collection of possible codecs of the receiver. To be overridden by concrete subclasses."

	^ #()
! !

!GRCodec class methodsFor:'private'!

basicForEncoding: aString
	"Create the actual instance."
	self subclassResponsibility
!

unsupportedEncoding: aString
	"Signal an unsupported encoding."

	^ GRUnsupportedEncodingError signal: 'unsupported encoding: ' , aString
! !

!GRCodec class methodsFor:'testing'!

supportsEncoding: aString
	"Answer whether the the given encoding name is supported by this codec class."
	self subclassResponsibility
! !

!GRCodec methodsFor:'accessing'!

name
	"Answer a human readable string of the receivers encoding policy."

	self subclassResponsibility
!

url
	"Answer a codec that is responsible to encode and decode URLs. In most cases an UTF-8 codec is the only valid choice, but subclasses might decide to do something else."

	self subclassResponsibility
! !

!GRCodec methodsFor:'convenience'!

decode: aString
	| readStream writeStream |
	readStream := self decoderFor: aString readStream.
	writeStream := WriteStream on: (String new: aString size).
	[ readStream atEnd ]
		whileFalse: [ writeStream nextPutAll: (readStream next: 1024) ].
	^ writeStream contents
!

encode: aString
	| writeStream |
	writeStream := self encoderFor: (WriteStream
		on: (String new: aString size)).
	writeStream nextPutAll: aString.
	^ writeStream contents
! !

!GRCodec methodsFor:'conversion'!

decoderFor: aReadStream
	"Wrap aReadStream with an decoder for the codec of the receiver. Answer a read stream that delegates to and shares the state of aReadStream." 
	
	self subclassResponsibility
!

encoderFor: aWriteStream
	"Wrap aWriteStream with an encoder for the codec of the receiver. Answer a write stream that delegates to and shares the state of aWriteStream."
	
	self subclassResponsibility
! !

!GRCodec methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' name: '; print: self name
! !

!GRCodec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRCodec.st,v 1.1 2011/09/05 17:43:05 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRCodec.st,v 1.1 2011/09/05 17:43:05 cg Exp $'
! !
