"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRDelayedSend
	instanceVariableNames:'receiver message'
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core-Utilities'
!

GRDelayedSend comment:'A WADelayedSend is a future message send of a message to an object. Some of the arguments can be predefined. Instances are intended to be interchangeable with blocks.
This class should conform the ANSI valuable protocol.
This is an abstract class. Use the methods in the ''instance-creation'' protocol on the class side to create intances.
Instance Variables
	receiver:		<Object>
	selector:		<Symbol>
receiver
	- the object receiving the message
selector
	- the message selector sent to the receiver
'
!


!GRDelayedSend class methodsFor:'instance creation'!

empty
	^ self receiver: nil selector: #yourself
!

new
	^ self empty
!

receiver: anObject selector: aSymbol 
	^ self basicNew 
		initializeWithReceiver: anObject
			message: (GRUnboundMessage selector: aSymbol);
		yourself
!

receiver: anObject selector: aSymbol argument: aParameter
	^ self receiver: anObject selector: aSymbol arguments: (Array with: aParameter)
!

receiver: anObject selector: aSymbol arguments: anArray 
	^ self basicNew 
		initializeWithReceiver: anObject
			message: (GRBoundMessage selector: aSymbol arguments: anArray);
		yourself
! !

!GRDelayedSend methodsFor:'accessing'!

argumentCount
	"Answer the number of arguments that must be provided to the receiver when sending it."
	
	^ message argumentCount
!

fixCallbackTemps
	"For polymorphism with BlockContext>>#fixCallbackTemps."
! !

!GRDelayedSend methodsFor:'evaluating'!

value
	^ self valueWithArguments: #()
!

value: anObject
	^ self valueWithArguments: (Array with: anObject)
!

value: aFirstObject value: aSecondObject
	^ self valueWithArguments: (Array with: aFirstObject with: aSecondObject)
!

valueWithArguments: anArray
	^ message valueFor: receiver withArguments: anArray
!

valueWithPossibleArguments: anArray
	^ message valueFor: receiver withPossibleArguments: anArray
! !

!GRDelayedSend methodsFor:'initialization'!

initializeWithReceiver: anObject message: aDelayedSendMessage
	self initialize.
	receiver := anObject.
	message := aDelayedSendMessage
! !

!GRDelayedSend methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' receiver: '; print: receiver.
	aStream nextPut: $ .
	message printOn: aStream
! !

!GRDelayedSend class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRDelayedSend.st,v 1.1 2011/09/05 17:28:04 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRDelayedSend.st,v 1.1 2011/09/05 17:28:04 cg Exp $'
! !
