"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRDelayedSendMessage
	instanceVariableNames:'selector'
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core-Utilities'
!

GRDelayedSendMessage comment:''
!


!GRDelayedSendMessage class methodsFor:'instance creation'!

new
	^ self selector: #yourself
!

selector: aSymbol
	^ self basicNew
		initializeWithSelector: aSymbol;
		yourself
! !

!GRDelayedSendMessage methodsFor:'delegation'!

argumentCount
	"Answer the number of arguments that must be provided when sending this message."
	
	self subclassResponsibility
!

valueFor: receiver withArguments: anArray
	"Send the message to receiver with the arguments in anArray. Answer the result.
	Signal an error if the number of arguments does not match the selector."
	
	^ self basicPerformFor: receiver withArguments: (self mergeArguments: anArray)
!

valueFor: receiver withPossibleArguments: anArray
	"Send the message to receiver with as many of the arguments in anArray as possible. Answer the result.
	Error if not enough arguments are available for the selector."

	| allArguments |
	allArguments := self mergeArguments: anArray.
	
	^ allArguments size < selector numArgs
		ifTrue: [ self invalidArgumentCount ]
		ifFalse: [ self basicPerformFor: receiver withArguments: (allArguments first: selector numArgs) ]
! !

!GRDelayedSendMessage methodsFor:'initialization'!

initializeWithSelector: aSymbol
	self initialize.
	selector := aSymbol
! !

!GRDelayedSendMessage methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: 'selector: '; print: selector
! !

!GRDelayedSendMessage methodsFor:'private'!

basicPerformFor: receiver withArguments: anArray
	^ anArray size = selector numArgs
		ifTrue: [ receiver perform: selector withArguments: anArray ]
		ifFalse: [ self invalidArgumentCount ]
!

invalidArgumentCount
	GRInvalidArgumentCount signal
!

mergeArguments: anArray
	self subclassResponsibility
! !

!GRDelayedSendMessage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRDelayedSendMessage.st,v 1.1 2011/09/05 17:22:38 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRDelayedSendMessage.st,v 1.1 2011/09/05 17:22:38 cg Exp $'
! !
