"{ Package: 'stx:goodies/seaside30/grease' }"

GRPrinter subclass:#GRNumberPrinter
	instanceVariableNames:'characters base delimiter digits infinite nan padding accuracy
		precision separator'
	classVariableNames:'NumbersToCharactersLowercase NumbersToCharactersUppercase'
	poolDictionaries:''
	category:'Grease-Core-Text'
!

GRNumberPrinter comment:'A WANumberPrinter is prints numbers (integers and floats) in various formats in a platform independent way.
Instance Variables
	accuracy:		<UndefinedObject|Float>
	base:			<Integer>
	delimiter:		<UndefinedObject|Charater>
	digits:			<UndefinedObject|Integer>
	infinite:		<UndefinedObject|String>
	nan:			<UndefinedObject|String>
	padding:		<UndefinedObject|Charater>
	precision:		<Integer>
	separator:		<UndefinedObject|Charater>'
!


!GRNumberPrinter class methodsFor:'initialization'!

initialize
	NumbersToCharactersLowercase := '0123456789abcdefghijklmnopqrstuvwxyz'.
	NumbersToCharactersUppercase := '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
! !

!GRNumberPrinter methodsFor:'accessing'!

accuracy: aFloat
	"Round towards the nearest number that is a multiple of aFloat."
	
	accuracy := aFloat
!

base: anInteger
	"The numeric base to which the number should be printed."

	base := anInteger
!

characters: aString
	"The characters to be used to convert a number to a string."

	characters := aString
!

delimiter: aCharacter
	"The delimiter to separate the integer and fraction part of the number."

	delimiter := aCharacter
!

digits: anInteger
	"The number of digits to be printed in the integer part."

	digits := anInteger
!

infinite: aString
	"The string that should be displayed if the number is positive or negative infinity."
	
	infinite := aString
!

nan: aString
	"The string that should be displayed if the number is not a number."
	
	nan := aString
!

padding: aCharacter
	"The padding for the integer part."

	padding := aCharacter
!

precision: anInteger
	"The number of digits to be printed in the fraction part."
	
	precision := anInteger
!

separator: aCharacter
	"Separator character to be used to group digits."

	separator := aCharacter
! !

!GRNumberPrinter methodsFor:'actions'!

lowercase
	"Use lowercase characters for numbers of base 10 and higher."

	self characters: NumbersToCharactersLowercase
!

uppercase
	"Use uppercase characters for numbers of base 10 and higher."
	
	self characters: NumbersToCharactersUppercase
! !

!GRNumberPrinter methodsFor:'initialization'!

initialize
	super initialize.
	self lowercase.
	self base: 10.
	self delimiter: $..
	self infinite: 'Infinite'.
	self nan: 'NaN'.
	self padding: $ .
	self precision: 0
! !

!GRNumberPrinter methodsFor:'printing'!

print: aNumber on: aStream
	aNumber isNaN
		ifTrue: [ ^ self printNaN: aNumber on: aStream ].
	aNumber isInfinite 
		ifTrue: [ ^ self printInfinite: aNumber on: aStream ].
	precision = 0
		ifTrue: [ self printInteger: aNumber on: aStream ]
		ifFalse: [ self printFloat: aNumber on: aStream ]
!

printFloat: aNumber on: aStream
	| multiplier rounded |
	multiplier := base asFloat raisedTo: precision.
	rounded := aNumber roundTo: (accuracy ifNil: [ 1.0 / multiplier ]).
	self printInteger: rounded on: aStream.
	delimiter isNil
		ifFalse: [ aStream nextPut: delimiter ].
	self printFraction: rounded fractionPart abs * multiplier on: aStream
!

printFraction: aNumber on: aStream
	| result |
	result := self
		pad: (self digitsOf: aNumber rounded base: base)
		left: $0 to: precision.
	separator isNil
		ifFalse: [ result := self separate: result left: separator ].
	aStream nextPutAll: result
!

printInfinite: aNumber on: aStream
	infinite isNil
		ifFalse: [ aStream nextPutAll: infinite ]
!

printInteger: aNumber on: aStream
	| result |
	result := self digitsOf: aNumber integerPart base: base.
	separator isNil
		ifFalse: [ result := self separate: result right: separator ].
	(digits isNil or: [ padding isNil ])
		ifFalse: [ result := self pad: result left: padding to: digits ].
	aStream nextPutAll: result
!

printNaN: anInteger on: aStream
	nan isNil
		ifFalse: [ aStream nextPutAll: nan ]
! !

!GRNumberPrinter methodsFor:'utilities'!

digitsOf: aNumber base: aBaseInteger
	"Answer the absolute digits of aNumber in the base aBaseInteger."

	| integer stream next |
	integer := aNumber truncated abs.
	integer = 0 ifTrue: [ ^ '0' ].
	stream := WriteStream on: (String new: 10).
	[ integer > 0 ] whileTrue: [
		next := integer quo: aBaseInteger.
		stream nextPut: (characters 
			at: 1 + integer - (next * aBaseInteger)).
		integer := next ].
	^ stream contents reverse
!

separate: aString left: aCharacter
	"Separate from the left side every 3 characters with aCharacter."
		
	| size stream |
	size := aString size.
	stream := WriteStream on: (String new: 2 * size).
	1 to: size do: [ :index |
		(index ~= 1 and: [ index \\ 3 = 1 ])
			ifTrue: [ stream nextPut: aCharacter ].
		stream nextPut: (aString at: index) ].
	^ stream contents
!

separate: aString right: aCharacter
	"Separate from the right side every 3 characters with aCharacter."
		
	| size stream |
	size := aString size.
	stream := WriteStream on: (String new: 2 * size).
	1 to: size do: [ :index |
		(index ~= 1 and: [ size - index \\ 3 = 2 ])
			ifTrue: [ stream nextPut: aCharacter ].
		stream nextPut: (aString at: index) ].
	^ stream contents
! !

!GRNumberPrinter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRNumberPrinter.st,v 1.1 2011/09/05 17:29:36 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRNumberPrinter.st,v 1.1 2011/09/05 17:29:36 cg Exp $'
! !

GRNumberPrinter initialize!
