"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRPackage
	instanceVariableNames:'name description dependencies license url'
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core'
!

GRPackage comment:''
!





!GRPackage class methodsFor:'accessing'!

greaseCore
	^ self new
		name: 'Grease-Core';
		description: 'The main package of the Grease compatibility layer.';
		url: #seasideUrl;
		yourself
! !

!GRPackage class methodsFor:'querying'!

packages
	"Answer a list of all registered packages. A package is registered by adding a class extension to the receiving class answering an instance of the receiving class."

	| packages package |
	packages := Dictionary new.
	self class selectors do: [ :each |
		(each numArgs = 0 and: [ each ~= #packages ]) ifTrue: [ 
			package := self perform: each.
			packages at: package name put: package ] ].
	packages do: [ :each | each resolveWith: packages ].
	^ packages values
! !

!GRPackage methodsFor:'accessing'!

description
	"Answer a short description of the package."

	^ description
!

description: aString
	description := aString
!

license
	"Answer the current license of this package, by default MIT is used."

	^ license
!

license: aSymbol
	license := aSymbol
!

name
	"Answer the name of the package. This string should be useable to identify the platform specific native package object, e.g. the Monticello package name."

	^ name
!

name: aString
	name := aString
!

url
	"Answer the base-URL of the package. This string is only meaningful for platforms that can directly access Monticello repositories."

	^ url isSymbol
		ifTrue: [ self perform: url ]
		ifFalse: [ url ]
!

url: aStringOrSymbol
	"Set the base-URL of the package, or a symbol referring to a method in this class that answers the URL. This setting is only meaningful for platforms that can directly access Monticello repositories."

	url := aStringOrSymbol
! !

!GRPackage methodsFor:'accessing-repositories'!

seasideAddonsUrl
	^ 'http://www.squeaksource.com/Seaside30Addons'
!

seasideLGPLUrl
	^ 'http://www.squeaksource.com/Seaside30LGPL'
!

seasideUrl
	^ 'http://www.squeaksource.com/Seaside30'
! !

!GRPackage methodsFor:'dependencies'!

addDependency: aString
	dependencies add: aString
!

allDependencies
	"Answer all dependencies on which this package depends."

	^ self addDependenciesTo: OrderedCollection new
!

dependencies
	"Return a collection of package names on which this package depends."

	^ dependencies
!

resolveWith: aDictionary
	dependencies := dependencies 
		collect: [ :each | aDictionary at: each ifAbsent: [ self error: self name printString , ' depends on unknown package ' , each printString ] ]
! !

!GRPackage methodsFor:'initialization'!

initialize
	super initialize.
	dependencies := OrderedCollection new.
	license := #MIT
! !

!GRPackage methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)
! !

!GRPackage methodsFor:'private'!

addDependenciesTo: aCollection
	(aCollection includes: self) ifFalse: [
		self dependencies
			do: [ :each | each addDependenciesTo: aCollection ].
		aCollection add: self ].
	^ aCollection
! !

!GRPackage methodsFor:'testing'!

isLGPL
	^ self license = #LGPL
!

isMIT
	^ self license = #MIT
! !

!GRPackage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPackage.st,v 1.3 2011/09/05 18:39:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPackage.st,v 1.3 2011/09/05 18:39:41 cg Exp $'
! !
