"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRPrinter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core-Text'
!

GRPrinter comment:''
!


!GRPrinter class methodsFor:'factory'!

cookieTimestamp
	"Netscape's original proposal defined an Expires header that took a date value in a fixed-length variant format in place of Max-Age: Wdy, DD-Mon-YY HH:MM:SS GMT"
	^ GRMappedPrinter 
		block: [ :timestamp | timestamp asUTC ]
		next: self abbreviatedWeekName , ', ' , self paddedDay , '-' , self abbreviatedMonthName , '-' , self paddedYear , ' ' , self isoTime , ' GMT'
!

httpDate
	"Answers a printer that formats dates for HTTP1.1 (RFC 2616)"
	^ self rfc1123
!

isoDate
	"Ansers a printer that formats dates accoring to ISO(YYYY-MM-DD) E.g.
	2003-12-24"
	^ self paddedYear , $- , self paddedMonth , $- , self paddedDay
!

isoTime
	"Ansers a printer that formats time accoring to ISO(HH:MM:SS) E.g.
	12:23:34"
	^ self paddedHour24 , $: , self paddedMinute , $: , self paddedSecond
!

rfc1123
	"Answers a printer that formats dates for HTTP1.1 (RFC 1123). Eg.
	Sun, 06 Nov 1994 08:49:37 GMT "
	^ GRMappedPrinter 
		block: 
			[ :date | 
			"For the purposes of HTTP, GMT is exactly equal to UTC (Coordinated Universal Time)"
			date asUTC ]
		next: GRSequentialPrinter new , self abbreviatedWeekName , ', ' , self paddedDay , Character space , self abbreviatedMonthName , Character space , self paddedYear , Character space , self isoTime , ' GMT'
!

rfc822
	"Answers a privter that formats dates according to RFC 822 (email). Eg.
	Sun, 31 Aug 2008 19:41:46 +0200"
	^ self abbreviatedWeekName , ', '
		, self paddedDay, Character space, self abbreviatedMonthName, Character space, self paddedYear, Character space,
		self isoTime, Character space, self offsetSign, self absOffsetHoursPadded, self absOffsetMinutesPadded
!

rfc822WithTimeZone: aString
	"Answers a privter that formats dates according to RFC 822 (email) with the given time zone String. Eg.
	Sun, 31 Aug 2008 19:41:46 <aString>"
	^ self abbreviatedWeekName , ', '
		, self paddedDay, Character space, self abbreviatedMonthName, Character space, self paddedYear, Character space,
		self isoTime, Character space, aString
!

swissCurrency
	^ GRSequentialPrinter new , 'CHF ' , GRSignPrinter new , (GRNumberPrinter new
			separator: $';
			precision: 2;
			accuracy: 0.05;
			yourself)
!

usCurrency
	^ GRSignPrinter new , $$ , (GRNumberPrinter new
			separator: $,;
			precision: 2;
			yourself)
! !

!GRPrinter class methodsFor:'parts-date'!

abbreviatedMonthName
	^ self monthName: #('Jan' 'Feb' 'Mar' 'Apr' 'May' 'Jun' 'Jul' 'Aug' 'Sep' 'Oct' 'Nov' 'Dec')
!

abbreviatedWeekName
	^ self weekName: #('Sun' 'Mon' 'Tue' 'Wed' 'Thu' 'Fri' 'Sat')
!

absOffsetHoursPadded
	^ GRMappedPrinter 
		block: [ :date | date offset hours abs ]
		next: (self numberWithAtLeastDigits: 2)
!

absOffsetMinutesPadded
	^ GRMappedPrinter 
		block: [ :date | date offset minutes abs ]
		next: (self numberWithAtLeastDigits: 2)
!

fullMonthName
	^ self monthName: #('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
!

fullWeekName
	^ self weekName: #('Sunday' 'Monday' 'Tuesday' 'Wednesday' 'Thursday' 'Friday' 'Saturday')
!

monthName: anArray 
	^ GRPluggablePrinter on: [ :date | anArray at: date monthIndex ]
!

offsetSign
	^ GRMappedPrinter 
		block: [ :date | date offset ]
		next: (GRSignPrinter new
				positivePrinter: $+;
				negativePrinter: $-;
				yourself)
!

paddedCentury
	^ GRMappedPrinter 
		block: [ :date | date year \\ 100 ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedDay
	^ GRMappedPrinter 
		block: [ :date | date dayOfMonth ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedMonth
	^ GRMappedPrinter 
		block: [ :date | date monthIndex ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedYear
	^ GRMappedPrinter 
		block: [ :date | date year ]
		next: (self numberWithAtLeastDigits: 4)
!

unpaddedCentury
	^ GRMappedPrinter 
		block: [ :date | date year \\ 100 ]
		next: GRNumberPrinter new
!

unpaddedDay
	^ GRMappedPrinter 
		block: [ :date | date dayOfMonth ]
		next: GRNumberPrinter new
!

unpaddedMonth
	^ GRMappedPrinter 
		block: [ :date | date monthIndex ]
		next: GRNumberPrinter new
!

unpaddedYear
	^ GRMappedPrinter 
		block: [ :date | date year ]
		next: GRNumberPrinter new
!

weekName: anArray 
	^ GRPluggablePrinter on: [ :date | anArray at: date dayOfWeek ]
! !

!GRPrinter class methodsFor:'parts-time'!

paddedHour12
	^ GRMappedPrinter 
		block: [ :time | (time hour - 1) \\ 12 + 1 ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedHour24
	^ GRMappedPrinter 
		block: [ :time | time hour ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedMinute
	^ GRMappedPrinter 
		block: [ :time | time minute ]
		next: (self numberWithAtLeastDigits: 2)
!

paddedSecond
	^ GRMappedPrinter 
		block: [ :time | time second ]
		next: (GRNumberPrinter new
				padding: $0;
				digits: 2)
!

unpaddedHour12
	^ GRMappedPrinter 
		block: [ :time | (time hour - 1) \\ 12 + 1 ]
		next: GRNumberPrinter new
!

unpaddedHour24
	^ GRMappedPrinter 
		block: [ :time | time hour ]
		next: GRNumberPrinter new
!

unpaddedMinute
	^ GRMappedPrinter 
		block: [ :time | time minute ]
		next: GRNumberPrinter new
!

unpaddedSecond
	^ GRMappedPrinter 
		block: [ :time | time second ]
		next: GRNumberPrinter new
! !

!GRPrinter class methodsFor:'parts-units'!

binaryFileSize
	^ GRUnitPrinter 
		base: 1024
		units: #(
				'byte'
				'bytes'
				'KiB'
				'MiB'
				'GiB'
				'TiB'
				'PiB'
				'EiB'
				'ZiB'
				'YiB'
			)
!

decimalFileSize
	^ GRUnitPrinter 
		base: 1000
		units: #(
				'byte'
				'bytes'
				'kB'
				'MB'
				'GB'
				'TB'
				'PB'
				'EB'
				'ZB'
				'YB'
			)
!

numberWithAtLeastDigits: anInteger 
	^ GRNumberPrinter new
		padding: $0;
		digits: anInteger;
		yourself
! !

!GRPrinter methodsFor:'operators'!

, aPrinter 
	^ GRSequentialPrinter new , self , aPrinter
! !

!GRPrinter methodsFor:'printing'!

print: anObject
	^ String streamContents: [ :stream | self print: anObject on: stream ]
!

print: anObject on: aStream
	"Subclasses override this method to produce some output."
! !

!GRPrinter methodsFor:'utilities'!

pad: aString center: aCharacter to: anInteger
	"Pad to the center of aString with aCharacter to at least anInteger characters."

	| result index |
	anInteger <= aString size
		ifTrue: [ ^ aString ].
	index := (anInteger - aString size) // 2.
	result := (String new: anInteger) atAllPut: aCharacter.
	result
		replaceFrom: index + 1
		to: index + aString size
		with: aString
		startingAt: 1.
	^ result
!

pad: aString left: aCharacter to: anInteger
	"Pad to the left side of aString with aCharacter to at least anInteger characters."

	| result |
	anInteger <= aString size
		ifTrue: [ ^ aString ].
	result := (String new: anInteger) atAllPut: aCharacter.
	result
		replaceFrom: anInteger - aString size + 1
		to: anInteger
		with: aString
		startingAt: 1.
	^ result
!

pad: aString right: aCharacter to: anInteger
	"Pad to the right side of aString with aCharacter to at least anInteger characters."

	| result |
	anInteger <= aString size
		ifTrue: [ ^ aString ].
	result := (String new: anInteger) atAllPut: aCharacter.
	result
		replaceFrom: 1
		to: aString size
		with: aString
		startingAt: 1.
	^ result
! !

!GRPrinter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPrinter.st,v 1.1 2011/09/05 17:47:18 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPrinter.st,v 1.1 2011/09/05 17:47:18 cg Exp $'
! !
