"{ Package: 'stx:goodies/seaside30/grease/stx' }"

GRSTXConverterCodecStream subclass:#GRSTXUtf8CodecStream
	instanceVariableNames:''
	classVariableNames:'Latin1ToUtf8Encodings Latin1ToUtf8Map'
	poolDictionaries:''
	category:'Grease-STX-Core'
!


!GRSTXUtf8CodecStream class methodsFor:'class initialization'!

initialize
	(Smalltalk hasClassNamed: #UTF8TextConverter) "guard for Squeak 37"
		ifFalse: [ ^ self ].

	Latin1ToUtf8Map := ByteArray new: 256.
	Latin1ToUtf8Encodings := Array new: 256.
	0 to: 255 do:[ :index |
		| latin1 utf8 |
		latin1 := String with: (Character codePoint: index).
		utf8 := latin1 convertToWithConverter: UTF8TextConverter new.
		latin1 = utf8
			ifTrue:[
				Latin1ToUtf8Map at: index + 1 put: 0 ] "no translation needed"
			ifFalse:[
				Latin1ToUtf8Map at: index + 1 put: 1. "no translation needed"
				Latin1ToUtf8Encodings at: index + 1 put: utf8 ] ]
! !

!GRSTXUtf8CodecStream methodsFor:'private'!

encodeDefault: aString
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	1 to: aString size by: 1 do: [ :index |
		converter nextPut: (aString at: index) toStream: stream ]
!

encodeFast: aByteString
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	| lastIndex nextIndex |
	lastIndex := 1.
	nextIndex := ByteString findFirstInString: aByteString inSet: Latin1ToUtf8Map startingAt: lastIndex.
	nextIndex = 0 ifTrue: [ ^ stream nextPutAll: aByteString ].
	[	nextIndex > lastIndex ifTrue: [
			stream greaseNext: nextIndex - lastIndex putAll: aByteString startingAt: lastIndex ].
		stream nextPutAll: (Latin1ToUtf8Encodings at: (aByteString byteAt: nextIndex) + 1).
		lastIndex := nextIndex + 1.
		nextIndex := ByteString findFirstInString: aByteString inSet: Latin1ToUtf8Map startingAt: lastIndex.
		nextIndex = 0 ] whileFalse.
	stream greaseNext: aByteString size - lastIndex + 1 putAll: aByteString startingAt: lastIndex.
!

invalidUtf8
	^ GRInvalidUtf8Error signal: 'Invalid UTF-8 input'
! !

!GRSTXUtf8CodecStream methodsFor:'streaming'!

crlf
	stream nextPut: Character cr.
	stream nextPut: Character lf
!

greaseNext: anInteger putAll: aCollection startingAt: startIndex
	| index |
	aCollection isByteString
		ifTrue: [ 
			index := ByteString findFirstInString: aCollection inSet: Latin1ToUtf8Map startingAt: startIndex.
			(index = 0 or: [ index > (startIndex + anInteger - 1) ])
				ifTrue: [ stream greaseNext: anInteger putAll: aCollection startingAt: startIndex ]
				ifFalse: [ super greaseNext: anInteger putAll: aCollection startingAt: startIndex ] ]
		ifFalse: [ super greaseNext: anInteger putAll: aCollection startingAt: startIndex ]
!

next
	^ (self next: 1) first
!

next: anInteger
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	| outStream byte1 byte2 byte3 byte4 unicode count |
	outStream := WriteStream on: (String new: anInteger).
	count := 0.
	[ count < anInteger and: [ stream atEnd not ] ] whileTrue: [
		byte1 := stream next.
		unicode := byte1.
		(byte1 bitAnd: 16rE0) = 192 ifTrue: [ "two bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 31) bitShift: 6) + (byte2 bitAnd: 63)].
		(byte1 bitAnd: 16rF0) = 224 ifTrue: [ "three bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte3 := stream next.
			(byte3 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 15) bitShift: 12) + ((byte2 bitAnd: 63) bitShift: 6)
				+ (byte3 bitAnd: 63)].
		(byte1 bitAnd: 16rF8) = 240 ifTrue: [ "four bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte3 := stream next.
			(byte3 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte4 := stream next.
			(byte4 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 16r7) bitShift: 18) +
							((byte2 bitAnd: 63) bitShift: 12) + 
							((byte3 bitAnd: 63) bitShift: 6) +
							(byte4 bitAnd: 63)].
		unicode ifNil: [ self invalidUtf8 ].
		unicode = 16rFEFF "ignore BOM" ifFalse: [
			outStream nextPut: (Character codePoint: unicode).
			count := count + 1 ].
		unicode := nil ].
	^ outStream contents
!

nextPut: aCharacter
	| codePoint shouldEncode |
	codePoint := aCharacter codePoint.
	codePoint > 255
		ifTrue: [ ^ self nextPutAll: (String with: aCharacter) ].
	shouldEncode := Latin1ToUtf8Map at: codePoint + 1.
	shouldEncode = 1
		ifTrue: [ stream nextPutAll: (Latin1ToUtf8Encodings at: codePoint + 1) ]
		ifFalse: [ stream nextPut: aCharacter ]
	
!

nextPutAll: aString
	aString isByteString
		ifTrue: [ self encodeFast: aString ]
		ifFalse: [ self encodeDefault: aString ]
! !

!GRSTXUtf8CodecStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/stx/GRSTXUtf8CodecStream.st,v 1.2 2011/09/06 05:54:59 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/stx/GRSTXUtf8CodecStream.st,v 1.2 2011/09/06 05:54:59 cg Exp $'
! !

GRSTXUtf8CodecStream initialize!
