"{ Package: 'stx:goodies/sif' }"

SmalltalkInterchangeFileManager subclass:#SmalltalkInterchangeFileInManager
	instanceVariableNames:'headerStream currentItem nextItem isSmalltalkItemProcessed
		itemInfoByFirstToken itemInfoBySecondToken classToPackageName'
	classVariableNames:''
	poolDictionaries:''
	category:'SIF-Support'
!


!SmalltalkInterchangeFileInManager class methodsFor:'accessing'!

defaultName
	"       ^       <Symbol>
	Return the name to be used when looking up which manager should be the default for filing in code."

	^#fileInManager
! !

!SmalltalkInterchangeFileInManager class methodsFor:'testing'!

isAbstract
	"       ^       <Boolean>
	Return true if I represent an abstract class.
	See concreteClasses for a list of concrete classes."

	^self == SmalltalkInterchangeFileInManager
! !

!SmalltalkInterchangeFileInManager methodsFor:'accessing'!

atGlobalNamed: name ifAbsent: ifAbsent
	"       name    <String>
		ifAbsent        [<Object>]
		^       <Object> | ifAbsent value
	Return the value in global named name.  If its not there return the result of ifAbsent."

	^Smalltalk at: name asSymbol ifAbsent: ifAbsent
!

atGlobalNamed: name put: value
	"       name    <String>
		value   <Object>
		^       void
	Set the value of the global named name to value."

	Smalltalk at: name asSymbol put: value
!

classNameToPackageName
	"       ^       <Dictionary key: <String> to: <String>>
	Return the mapping from class name to package name.
	See fileInClassItem:, fileInMethodItem, fileInClassMethodItem."

	^classToPackageName
!

currentItem
	"       ^       <AnsiInterchangeFileItem> | nil
	"

	^currentItem
!

currentItem: item
	"       item            <AnsiInterchangeFileItem>
		^               self
	"

	currentItem := item
!

evaluateDoIt: doIt in: object
	"       doIt    <String>
		in      <Object>
		^       <Object>
	Return the result of evaluating the code in doIt.
	Evaluate the expression within the context of doIt."

	^Compiler evaluate: doIt for: object logged: true
!

isSmalltalkItemProcessed
	"       ^       <Boolean>
	Return true if the Smalltalk Item has been processed to initialize me for the version identified in it."

	^isSmalltalkItemProcessed
!

uppercaseString: string
	"       string          <String>
		^               <String>
	Return string with all of its alphabetic character converted to upper case."

	^string asUppercase
! !

!SmalltalkInterchangeFileInManager methodsFor:'declaring'!

declarePool: name
	"       name            <String>
		^       void
	Declare a Pool named name"

	self atGlobalNamed: name ifAbsent: [
		self atGlobalNamed: name put: Dictionary new]
!

declarePoolConstant: constantName in: poolName
	"       constantName    <String>
		poolName                <String>
		^       void
	item is a constant definition for a pool."

	(self atGlobalNamed: poolName ifAbsent: [
		self error: 'The pool dictionary ', poolName printString, ' should have been defined first'.])
			at: constantName
			put: nil
!

declarePoolVariable: variableName in: poolName
	"       variableName    <String>
		poolName                <String>
		^       void
	item is a variable definition for a pool."

	(self atGlobalNamed: poolName ifAbsent: [
		self error: 'The pool dictionary ', poolName printString, ' should have been defined first'.])
			at: variableName
			put: nil
! !

!SmalltalkInterchangeFileInManager methodsFor:'filein'!

fileIn

	self managedStream.
	self itemsDo: [:each |
		each fileIn].
	self close.
!

fileInClassItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a class item that needs to be processed.
	Item attributes:
		name                            <String> 'named:'
		superclassName          <String> 'superclass:'
		instVarType             #none | #byte | #object 'indexedInstanceVariables:'
		instVarNames            <String>'instanceVariableNames:'
		classVarNames           <String> 'classVariableNames:'
		sharedPoolNames <String> 'sharedPools:'
		classInstVarNames       <String> 'classInstanceVariableNames:'
	item annotations
		package
	"

	self subclassResponsibility
!

fileInClassMethodItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a classMethod item that needs to be processed.
	NOTE: If item does not have a packahe annotation, I will place the
		method into the same package as the class of item.
	Item attributes:
		firstToken              Class name
		value           Source string
	item annotations
		category
		categories
		package"

	self fileInMethodItem: item intoClass: (self installedMetaclassNamed: item firstToken)
!

fileInCommentItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a class item that needs to be processed.
	Item attributes:
		firstToken              Double quote
		value                   <String>
	item annotations
	"
!

fileInConstantItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a constant definition for a pool.
	If first token is 'Global' then define a global instead of a pool variable.
	Item attributes:
		firstToken              Pool name
		name            Name of variable within pool
	item annotations"

	(self uppercaseString: item firstToken) = 'GLOBAL' ifTrue: [
		self atGlobalNamed: item name ifAbsent: [
			self atGlobalNamed: item name put: nil].
		^self].
	self declarePoolConstant: item name in: item firstToken
!

fileInInitializerForItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is an initializer item that needs to be processed.
	Item attributes:
		type            #initializerFor
		firstToken      Name of the pool dictionary
		name            Name of the pool variable
		value           Code to eval the value
	item annotations
		"
	(self atGlobalNamed: item firstToken ifAbsent: [self error: 'No pool has been declared by the name of ', item firstToken printString])
		at: item name
		put: (self evaluateDoIt: item value in: nil)
!

fileInInitializerItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is an initializer item that needs to be processed.
	Item attributes:
		firstToken              Global name
		value           Code to eval
	item annotations
		Package-PreInstallCode
		Package-PreUnInstallCode
		Package-PostInstallCode
		Package-PosUntInstallCode"


	(self uppercaseString: item firstToken) = 'GLOBAL' ifTrue: [
		self handleAnnotationsOnGlobalInitializerItem: item.
		^self evaluateDoIt: item value in: nil].
	((self
		atGlobalNamed: item firstToken
		ifAbsent: [self error: 'Encountered initializer for ', item firstToken printString, ' before it was defined.'])
			isKindOf: Behavior) ifTrue: [
				^self evaluateDoIt: item value in: nil].
	self atGlobalNamed: item firstToken put: (self evaluateDoIt: item value in: nil)
!

fileInMethodItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a method item that needs to be processed.
	NOTE: If item does not have a packahe annotation, I will place the
		method into the same package as the class of item.
	Item attributes:
		firstToken              Class name
		value           Source string
	item annotations
		category
		categories
		package"


	self fileInMethodItem: item intoClass: (self installedClassNamed: item firstToken)
!

fileInMethodItem: item intoClass: class
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a smalltalk item that needs to be processed.
	Item attributes:
		firstToken              Class name
		value           Source string
	item annotations
		category
		categories
		package"

	self subclassResponsibility
!

fileInPoolItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a declaration for a Pool item that needs to be processed.
	Item attributes:
		firstToken              'Pool'
		name                    Name of the pool
	item annotations"


	self declarePool: item name
!

fileInSmalltalkItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a smalltalk item that needs to be processed.
	Item attributes:
		value   Version string."
!

fileInVariableItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is a variable definition for a pool.
	If first token is 'Global' then define a global instead of a pool variable.
	Item attributes:
		firstToken              Pool name
		name            Name of variable within pool
	item annotations"

	(self uppercaseString: item firstToken) = 'GLOBAL' ifTrue: [
		self atGlobalNamed: item name ifAbsent: [
			self atGlobalNamed: item name put: nil].
		^self].
	self declarePoolVariable: item name in: item firstToken
!

handleAnnotationsOnGlobalInitializerItem: item
	"       item    <SmalltalkInterchangeFileItem>
		^       void
	item is an initializer item that needs to be processed.  I am called by the generic handler
	in order to handle the annotations on the item.
	Item attributes:
		firstToken              Global name
		value           Code to eval
	item annotations
		package-preInstallCode
		package-preUnInstallCode
		package-postInstallCode
		package-posUntInstallCode
		package-preRequisites"

	self subclassResponsibility
! !

!SmalltalkInterchangeFileInManager methodsFor:'get contents'!

getAnnotationItemContents

	| keyword |
	self pushStream: headerStream while: [
		#(
			#('KEY:' #name:)
			#('VALUE:' #value:)
		) do: [:pair |
			keyword := self nextWord.
			(self uppercaseString: keyword) = (pair at: 1) ifFalse: [
				self error: 'Expecting ', (pair at: 1)].
			currentItem perform: (pair at: 2) with: self nextStringOrSymbolToken]].
!

getClassInitializerItemContents

	currentItem value: self nextChunk
!

getClassItemContents

	| keyword |
	self pushStream: headerStream while: [
		#(
			#('named:' #name:)
			#('superclass:' #superclassName:)
			#('indexedInstanceVariables:' #instVarType:)
			#('instanceVariableNames:' #instVarNames:)
			#('classVariableNames:' #classVarNames:)
			#('sharedPools:' #sharedPoolNames:)
			#('classInstanceVariableNames:' #classInstVarNames:)
		) do: [:pair |
			keyword := self nextWord.
			(self uppercaseString: keyword) = (self uppercaseString: (pair at: 1)) ifFalse: [
				self error: 'Expecting ....'].
			currentItem perform: (pair at: 2) with: self nextStringOrSymbolToken]].
!

getClassMethodItemContents

	currentItem value: (self skipWhiteSpace; nextChunk)
!

getCommentItemContents

	currentItem value: (headerStream upTo: $")
!

getConstantItemContents

	self getVariableItemContents
!

getInitializerForItemContents

	self pushStream: headerStream while: [
			"Skip initializeFor:"
		self nextWord.
		currentItem name: self nextStringOrSymbolToken].
	currentItem value: self nextChunk.
!

getInitializerItemContents

	currentItem value: (self skipWhiteSpace; nextChunk)
!

getMethodItemContents

	currentItem value: self skipWhiteSpace nextChunk
!

getPackageItemContents

	self notSupportedYet
!

getPoolItemContents

	self pushStream: headerStream while: [
			"Skip named:"
		self nextWord.
		currentItem name: self nextStringOrSymbolToken].
!

getSmalltalkItemContents

	self pushStream: headerStream while: [
			"Skip version:"
		self nextWord.
		currentItem value: self nextStringOrSymbolToken]
!

getVariableItemContents

	self pushStream: headerStream while: [
			"Skip variable:"
		self nextWord.
		currentItem name: self nextStringOrSymbolToken].
! !

!SmalltalkInterchangeFileInManager methodsFor:'initializing'!

initialize
	"       self
	Initialize myself to be ready to read a file"

	super initialize.
	isSmalltalkItemProcessed := false.
	itemInfoByFirstToken := Dictionary new
		at: 'SMALLTALK' put: #(#smalltalk #getSmalltalkItemContents #fileInSmalltalkItem:);
		yourself.
	itemInfoBySecondToken := Dictionary new
		yourself.
!

initializeForVersion10
	"       self
	Initialize myself to be ready to read a Version 1.0 file"

	itemInfoByFirstToken
		at: '"' put: #(#comment #getCommentItemContents #fileInCommentItem:);
		at: 'ANNOTATION' put: #(#annotation #getAnnotationItemContents #fileInAnnotationItem:);
		at: 'CLASS' put: #(#class #getClassItemContents #fileInClassItem:);
		at: 'POOL' put: #(#pool #getPoolItemContents #fileInPoolItem:).
	itemInfoBySecondToken
		at: 'METHOD' put: #(#method #getMethodItemContents #fileInMethodItem:);
		at: 'CLASSMETHOD' put: #(#classMethod #getClassMethodItemContents #fileInClassMethodItem:);
		at: 'INITIALIZER' put: #(#initializer #getInitializerItemContents #fileInInitializerItem:);
		at: 'INITIALIZERFOR:' put: #(#initializerFor #getInitializerForItemContents #fileInInitializerForItem:);
		at: 'VARIABLE:' put: #(#variable #getVariableItemContents #fileInVariableItem:);
		at: 'CONSTANT:' put: #(#constant #getConstantItemContents #fileInConstantItem:).
!

initializeForVersion10Extended
	"       self
	Initialize myself to be ready to read a Version 1.0 extended file."

	self initializeForVersion10.
	itemInfoByFirstToken at: 'PACKAGE' put: #(#package #getPackageItemContents #fileInPackageItem:).
! !

!SmalltalkInterchangeFileInManager methodsFor:'item'!

checkAndSetupVersion
	"       ^       self
	Read the first item on the file and make sure that it is a version marker and that I support this version.
	I will also set myself up to be able to read in something of whatever version is identified.
	If any of my checks fail, I will cause an error and not return."

	| read write selector next |
	currentItem isSmalltalkItem ifFalse: [
		self error: 'File MUST start with a Smalltalk version: ''1.0'' type of item.'].
	read := ReadStream on: currentItem value.
	write := WriteStream on: (String new: 32).
	write nextPutAll: 'initializeForVersion'.
	[read atEnd] whileFalse: [
		(next := read next) isAlphaNumeric ifTrue: [
			write nextPut: next]].
	selector := write contents asSymbol.
	(self respondsTo: selector) ifFalse: [
		self error: 'I do not support version ', currentItem value printString].
	self perform: selector.
	isSmalltalkItemProcessed := true
!

items
	"       ^       <OrderedCollection withAll: <AnsiInterchangeFileItem>>
	Return a colletion containing all of the items found on my file."

	| result |
	result := OrderedCollection new.
	self itemsDo: [:each |
		result add: each].
	^result
!

itemsDo: do
	"       do      [:item <AnsiInterchangeFileItem> | <void>]
	Assume the first chunk I read is the header chunk.  Then let
	the get its data, then pass on this item to do."

	| item |
	[self atEnd] whileFalse: [
		item := self nextItem.
		item isNil ifTrue: [^self].
		do value: item]
!

newItem
	"       ^       <SmalltalkInterchangeFileItem>
	Return a fresh new instance of an item."

	^SmalltalkInterchangeFileItem new
		manager: self;
		yourself
!

nextItem
	"       ^       <SmalltalkInterchangeFileItem> | nil
	Return the next item, excluding annotations, found on my stream.
	The annotations that should be attached to this item will be part of that
	items annotations."

	| item |
	nextItem isNil
		ifTrue: [self primNextItem]
		ifFalse: [
			currentItem := nextItem.
			nextItem := nil].
	currentItem isNil ifTrue: [^nil].
	self isSmalltalkItemProcessed ifFalse: [
		self checkAndSetupVersion].
	currentItem isAnnotation ifTrue: [
		self error: 'Cannot have an annotation before there is an element to attach annotations to.'].
	item := currentItem.
	[self primNextItem.
	currentItem notNil and: [currentItem isAnnotation]] whileTrue: [
		item addAnnotation: currentItem].
	nextItem := currentItem.
	^currentItem := item
!

primNextItem
	"       void
	Set my currentItem to be the next item, including annotations, I find on my stream.
	I also set my currentItem to be this item."

	| firstToken position secondToken |
	headerStream := self pushStream: (ReadStream on: self nextChunk).
	self skipWhiteSpace.
	(self peekFor: $")
		ifTrue: [
			firstToken := '"']
		ifFalse: [
			firstToken := self nextWord].
	firstToken isNil ifTrue: [
		self popStream.
		^currentItem := nil].
	currentItem := self newItem.
	position := headerStream position.
	secondToken := self nextWord.
	headerStream position: position.
		"We check for the second token, since we could have a global, or class
			by the same name as the special first tokens of the standard."
	secondToken notNil ifTrue: [
		currentItem info: (itemInfoBySecondToken at: (self uppercaseString: secondToken) ifAbsent: [nil])].
	currentItem info isNil ifTrue: [
		currentItem info: (itemInfoByFirstToken at: (self uppercaseString: firstToken) ifAbsent: [nil])].
	currentItem info isNil ifTrue: [
		self error: 'Expecting one of ...'].
	self popStream.
	currentItem firstToken: firstToken;
		getContents
! !

!SmalltalkInterchangeFileInManager methodsFor:'streaming'!

atEnd
	"       ^       <Boolean>
	"

		"If I am the bottom most stream, the original stream being read in,
			then there is a special case for atEnd since we read ahead."
	streamStack size = 1 ifFalse: [
		^self managedStream atEnd].
	^self managedStream atEnd and: [nextItem isNil]
!

collectionOfStringsFrom: string
	"       string          <String>
		^               <OrderedCollection withAll: <String>>
	Return the collection of strings in string that was placed there by nextPutCollectionOfStrings:."

	| result |
	result := OrderedCollection new.
	self pushStream: (ReadStream on: string) while: [
		[self atEnd] whileFalse: [
			result add: self nextChunk]].
	^result
!

nextChunk

	| result |
	result := self upTo: $!!.
	(self peekFor: $!!) ifFalse: [^result].
	result := (WriteStream on: (String new: result size + 128))
		nextPutAll: result;
		nextPut: $!!;
		yourself.
	[self atEnd] whileFalse: [
		result nextPutAll: (self upTo: $!!).
		(self peekFor: $!!) ifFalse: [
			^result contents].
		result nextPut: $!!].
	self skipWhiteSpace.
	^result contents
!

nextWord
	"       ^                       <String> | nil
	Return the next word found on my managed stream.  If I reach the end of the stream before
	finding a word, then return nil.
	I skip any current whitespcae, start collecting, and stop at the first white space."

	self subclassResponsibility
!

peekFor: char

	^self managedStream peekFor: char
!

skipWhiteSpace
	"       stream  <ReadStream on: <String>>
		^       void
	Skip any carriage returns, tabs, spaces anything that is a spacing character."

	self managedStream skipSeparators
!

upTo: char
	"       char            <Character>
		^               <String>
	Return a string containing all characters starting from current position, up to but
	but not including char."

	^self managedStream upTo: char
! !

!SmalltalkInterchangeFileInManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeFileInManager.st,v 1.2 2001/09/04 20:44:41 cg Exp $'
! !
