"{ Package: 'stx:goodies/sif' }"

Object subclass:#SmalltalkInterchangeFileItem
	instanceVariableNames:'manager info firstToken name classItemInfo value annotations'
	classVariableNames:''
	poolDictionaries:''
	category:'SIF-Support'
!


!SmalltalkInterchangeFileItem methodsFor:'accessing'!

firstToken
	"       ^       <String> | nil
	Return the firstToken for me."

	^firstToken
!

firstToken: object
	"       object  <String> | nil
		^               self
	Set the firstToken for me."

	firstToken := object
!

info
	"       ^       <Array withAll: <Symbol>>
	Retur the array that contains the type and selectors to process me."

	^info
!

info: object
	"       object  <Array withAll: <Symbol>>
	Set the array that contains the type and selectors to process me."

	info := object
!

manager
	"       ^       <SmalltalkInterchangeFileManager>
	Return the manager that owns me."

	^manager
!

manager: object
	"       object          <SmalltalkInterchangeFileManager>
		^               self
	Set the manager that owns me."

	manager := object
!

name
	"       ^       <String> | nil
	Return the name for me."

	^name
!

name: object
	"       object  <String> | nil
		^               self
	Set the name for me."

	name := object
!

type
	"       ^       <Symbol>
	Return the type of item I am."

	^self info at: 1
!

value
	"       ^       <String> | nil
	Return the value for me."

	^value
!

value: object
	"       object  <String> | nil
		^               self
	Set the value for me."

	value := object
! !

!SmalltalkInterchangeFileItem methodsFor:'annotation'!

addAnnotation: item
	"       item            <self>
		^               self
	Add item to my list of annotations."

	annotations isNil ifTrue: [
		annotations := OrderedCollection new].
	annotations add: item
!

annotationNamed: lookFor
	"       lookFor <String>
		^               <SmalltalkInterchangeFileItem> | nil
	Return the annotation that has it's name matching lookFor.
	Case does not matter."

	| upper |
	annotations isNil ifTrue: [^nil].
	upper := self manager uppercaseString: lookFor.
	annotations do: [:each |
		(self manager uppercaseString: each name) = upper ifTrue: [
			^each]].
	^nil
!

annotations
	"       ^       <OrderedCollection withAll: <self>>
	Return all of the annotations that are attached to me."

	^annotations isNil
		ifTrue: [#()]
		ifFalse: [annotations]
!

annotationsPrefixedWtih: prefix
	"       prefix  <String>
		^       <Collection withAll: <self>>
	Return all of my anotations that have begin with the prefix prefix.
	Case does not matter."

	| lookFor |
	lookFor := self uppercaseString: prefix.
	^self annotations select: [:each |
		( self uppercaseString: (each name copyFrom: 1 to: lookFor size)) = lookFor]
!

categoriesAnnotation
	"       ^               <SmalltalkInterchangeFileItem> | nil
	Retur the appropriate annotation."

	^self annotationNamed: 'categories'
!

categoryAnnotation
	"       ^               <SmalltalkInterchangeFileItem> | nil
	Retur the appropriate annotation."

	^self annotationNamed: 'category'
!

commentAnnotation
	"       ^               <SmalltalkInterchangeFileItem> | nil
	Retur the appropriate annotation."

	^self annotationNamed: 'comment'
!

packageAnnotation
	"       ^               <SmalltalkInterchangeFileItem> | nil
	Retur the appropriate annotation."

	^self annotationNamed: 'package'
! !

!SmalltalkInterchangeFileItem methodsFor:'class item'!

classInstVarNames
	"       ^       <String> | nil
	Return the classInstVarNames for me."

	^self classItemInfoAt: 6
!

classInstVarNames: object
	"       object  <String> | nil
		^               self
	Set the classInstVarNames for me."

	self classItemInfoAt: 6 put: object
!

classItemInfoAt: i
	"       i       <Integer>
		^       <Object>
	Return whatever is contained in my class item info at item i."

	classItemInfo isNil ifTrue: [
		^nil].
	^classItemInfo at: i
!

classItemInfoAt: i put: object
	"       i       <Integer>
		^       <Object>
	Place object in my class item info at item i."

	classItemInfo isNil ifTrue: [
		classItemInfo := Array new: 6].
	classItemInfo at: i put: object.
	^object
!

classVarNames
	"       ^       <String> | nil
	Return the classVarNames for me."

	^self classItemInfoAt: 4
!

classVarNames: object
	"       object  <String> | nil
		^               self
	Set the classVarNames for me."

	self classItemInfoAt: 4 put: object
!

instVarNames
	"       ^       <String> | nil
	Return the instVarNames for me."

	^self classItemInfoAt: 3
!

instVarNames: object
	"       object  <String> | nil
		^               self
	Set the instVarNames for me."

	self classItemInfoAt: 3 put: object
!

instVarType
	"       ^       <String> | nil
	Return the instVarType for me."

	^self classItemInfoAt: 2
!

instVarType: object
	"       object  <String> | nil
		^               self
	Set the instVarType for me."

	self classItemInfoAt: 2 put: object
!

sharedPoolNames
	"       ^       <String> | nil
	Return the sharedPoolNames for me."

	^self classItemInfoAt: 5
!

sharedPoolNames: object
	"       object  <String> | nil
		^               self
	Set the sharedPoolNames for me."

	self classItemInfoAt: 5 put: object
!

superclassName
	"       ^       <String> | nil
	Return the superclassName for me."

	^self classItemInfoAt: 1
!

superclassName: object
	"       object  <String> | nil
		^               self
	Set the superclassName for me."

	self classItemInfoAt: 1 put: object
! !

!SmalltalkInterchangeFileItem methodsFor:'parsing/file in'!

fileIn
	"       ^       self
	Install whatever item I represent onto the current system."

	self manager perform: (self info at: 3) with: self
!

getContents
	"       ^       self
	Initialize my contents to whatever is found in my manager's stream."

	self manager perform: (self info at: 2)
! !

!SmalltalkInterchangeFileItem methodsFor:'printing'!

attributesToPrint
	"       ^       <Array withAll: <Symbol>>
	Return the attributes to be printed."

	^#(#type #firstToken #name #superclassName #instVarType #instVarNames #classVarNames #sharedPoolNames #classInstVarNames #value)
!

printOn: stream

	super printOn: stream.
	stream nextPutAll: '('.
	self attributesToPrint do: [:each |
		(self perform: each) isNil ifFalse: [
			stream cr; tab; nextPutAll: each; nextPutAll: ': '.
			(self perform: each) printOn: stream]].
	stream nextPut: $)
! !

!SmalltalkInterchangeFileItem methodsFor:'testing'!

isAnnotation
	"       ^       <Boolean>
	Return true if I am an annotation item."

	^self type == #annotation
!

isSmalltalkItem
	"       ^       <Boolean>
	Return true if I am an annotation item."

	^self type == #smalltalk
! !

!SmalltalkInterchangeFileItem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeFileItem.st,v 1.2 2001/09/04 20:44:59 cg Exp $'
! !
