"{ Package: 'stx:goodies/sif' }"

Object subclass:#SmalltalkInterchangeFileManager
	instanceVariableNames:'fileName managedStream streamStack'
	classVariableNames:'Defaults'
	poolDictionaries:''
	category:'SIF-Support'
!


!SmalltalkInterchangeFileManager class methodsFor:'instance creation'!

new

	^super new initialize
!

newForFileIn
	"       ^       <AnsiInterchangeFileInManager>
	Return an object capable of filing in an interchange file with classes, methods, packages."

	^(self defaultAt: #fileInManager) new
!

newForFileOut
	"       ^       <AnsiInterchangeFileOutManager>
	Return an object capable of filing out classes, methods, packages."

	^(self defaultAt: #fileOutManager) new
! !

!SmalltalkInterchangeFileManager class methodsFor:'accessing'!

concreteClasses
	"       ^       <Collection withAll: <Class>>
	Return a collection containing all of my subclasses that are concrete."
	"AnsiInterchangeFileManager concreteClasses"

	^self allSubclasses reject: [:each | each isAbstract]
!

defaultAt: default
	"       default         <Symbol>
		^               <Object> | nil
	Return the value of the default named default.  If there is not a default named default, then return nil."

	^self defaultAt: default ifAbsent: [nil]
!

defaultAt: default ifAbsent: ifAbsent
	"       default         <Symbol>
		ifAbsent                [<Object>]
		^               <Object> | ifAbsent value
	Return the value of the default named default.  If there is not a default named default, then return the result of evaluating ifAbsent."

	^self defaults at: default ifAbsent: ifAbsent
!

defaultAt: default put: value
	"       default         <Symbol>
		value                   <Object>
		^               value
	Set the value of the default named default to value"

	self defaults at: default put: value.
	^value
!

defaultName
	"       ^       <Symbol>
	Return the name to be used when looking up which manager should be the default for filing in code."

		"Because I get called by the loading of packaged and other such things,
			I must make sure that initialize does not cause any errors."
	^#errorInitializedUsingAnAbstractClass
!

defaults
	"       ^       <Dictionary key: <Symbol> value: <Object>>
	Return my mapping of default names to their values.
	NOTE: Being destructive to the result will change the values I keep in defaults."

	Defaults isNil ifFalse: [^Defaults].
	^Defaults := Dictionary new
!

isAbstract
	"       ^       <Boolean>
	Return true if I represent an abstract class.
	See concreteClasses for a list of concrete classes."

	^self == SmalltalkInterchangeFileManager
! !

!SmalltalkInterchangeFileManager class methodsFor:'initializing'!

initialize
	"       ^       self
	Cause the defaults for Dolphin to be placed into my defaults."

	self isAbstract ifFalse: [
		self defaultAt: self defaultName put: self].
! !

!SmalltalkInterchangeFileManager methodsFor:'accessing'!

fileName
	"       ^       <String> | nil
	Return the name that the user has specified for the file I will use for read or write.
	If nil, then user has not specified a fileName.  Should have specified  stream then."

	^fileName
!

fileName: value
	"       value           <String> | nil
	Use value as the name that the user has specified for the file I will use for read or write.
	If nil, then user has not specified a fileName.  Should have specified  stream then."

	fileName := value
!

installedClassNamed: className
	"       className       <String>
		^               <Class> | nil
	Return the class named className."

	^self installedClassNamed: className ifAbsent: [nil]
!

installedClassNamed: className ifAbsent: ifAbsent
	"       className       <String>
		ifAbsent                [<Object>]
		^               <Class> | ifAbsent value
	Return the class named className."

	^Smalltalk at: className asSymbol ifAbsent: ifAbsent
!

installedMetaclassNamed: className
	"       className       <String>
		^               <Metaclass>
	Return the metaclass named className."

	^(self installedClassNamed: className) class
!

newStreamBasedOnFileName

	self subclassResponsibility
! !

!SmalltalkInterchangeFileManager methodsFor:'initializing'!

initialize
	"       ^       void
	Initialize myself to be in a consistent state with regards to my defaults."

	streamStack := OrderedCollection new.
! !

!SmalltalkInterchangeFileManager methodsFor:'opening/closing'!

close

	streamStack isEmpty ifFalse: [
		self popStream close].
!

newStreamOnFileNamed: file
	"       file    <String>
		^       <ReadStream> | <WriteStream>
	Return a stream that is opened on the file named file.
	If I am meant to write them return a write stream, read then a read strea."

	self subclassResponsibility
! !

!SmalltalkInterchangeFileManager methodsFor:'streaming'!

atEnd

	^self managedStream atEnd
!

managedStream
	"       ^       <ReadStream on: <String>>
	Retur the stream to use to read or write text from.  If the user has specified the stream then
	return it.  If the user has not specified the stream, then I assume that a filename has been
	specified and I will return a stream open on this file."

	managedStream isNil ifFalse: [
		^managedStream].
	self fileName isNil ifFalse: [
		^self pushStream: (self newStreamOnFileNamed: self fileName)].
	self error: 'Must specify either a stream or a fileName'
!

nextStringOrSymbolToken
        "       ^               <String> | <Symbol>
        "

        | char result |
        self skipWhiteSpace.
        self managedStream atEnd ifTrue: [^''].
        char := self managedStream next.
        char = $# ifTrue: [  
                "start CG's fix:"
                self managedStream peek == $' ifTrue:[
                    ^ self nextStringOrSymbolToken asSymbol
                ].
                "end CG's fix:"
                ^self nextWord asSymbol
        ].
        char = $' ifFalse: [
                self error: 'Expecting a '' or a #.'].
        result := WriteStream on: (String new: 32).
        [self managedStream atEnd] whileFalse: [
                char := self managedStream next.
                char = $'
                        ifTrue: [(self managedStream peekFor: $')
                                ifTrue: [result nextPut: $']
                                ifFalse: [^result contents]]
                        ifFalse: [result nextPut: char]].
        ^result contents
!

popStream
	"       ^       <Stream>
	Pop the top stream on my stream stack and return it.
	See pushStream: for more info."

	| result |
	result := streamStack removeLast.
	managedStream := streamStack isEmpty
		ifTrue: [nil]
		ifFalse: [streamStack last].
	^result
!

pushStream: stream
	"       stream          <Stream>
		^               stream
	Push stream onto my stream stack.  This means that any streaming operation
	I use will be done on the item on the top of the stream stack."

	managedStream := streamStack addLast: stream.
	^stream
!

pushStream: stream while: while
	"       stream  <Stream>
		while           [<Object>]
		^               stream
	Push stream onto my stream stack for the duration of the evaluation
	of while.  This means that any streaming operation
	I use will be done on the item on the top of the stream stack."

	self pushStream: stream.
	while value.
	self popStream
! !

!SmalltalkInterchangeFileManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeFileManager.st,v 1.3 2001/12/18 16:47:32 cg Exp $'
! !
SmalltalkInterchangeFileManager initialize!
