"{ Package: 'stx:goodies/sif' }"

SmalltalkInterchangeFileManager subclass:#SmalltalkInterchangeFileOutManager
	instanceVariableNames:'versionString fileOutInfoByType addedItems'
	classVariableNames:''
	poolDictionaries:''
	category:'SIF-Support'
!


!SmalltalkInterchangeFileOutManager class methodsFor:'accessing'!

defaultName
	"       ^       <Symbol>
	Return the name to be used when looking up which manager should be the default for filing out code."

	^#fileOutManager
! !

!SmalltalkInterchangeFileOutManager class methodsFor:'fileout'!

fileOutSifSupportInSif
	"       ^       self
	File out the basic SIF support in SIF format."
	"
	SmalltalkInterchangeFileOutManager fileOutSifSupportInSif
	"

	SmalltalkInterchangeFileManager newForFileOut
		fileName: 'sif-support.sif';
		addPackageNamed: 'SIF-Support';
		fileOut;
		close
!

fileOutSifSupportUsingSystemCategories: useSystemCategories
	"       useSystemCategories     <Boolean>
		^       void
	File out to a file named 'sifsupport.st' a simple form of all the classes and methods contained
	to provide the base file-in for the first time port of Smalltalk Interchange File to a platform.
	useSystemCategories is used if I should use the subclass:*category: form of class
	creation.
	Once the the basic groundwork for SIF exists, one can read in a SIF version of this to get
	all of the categories and such to clean up the code."
	"
	For Squeak, VW, use:
		SmalltalkInterchangeFileOutManager fileOutSifSupportUsingSystemCategories: true
	For VA, Dolphin, Digitalk, use:
		SmalltalkInterchangeFileOutManager fileOutSifSupportUsingSystemCategories: false
	"

	| manager package eachClassInfo byClass inits |
	manager := self newForFileOut.
	manager fileName: 'sifsupport.st'.
	package := manager packageItemInfoFor: 'SIF-Support'.
	inits := OrderedCollection new.
	(manager hierarchicallySortClasses: (package at: #classes)) do: [:eachClass |
		eachClassInfo := manager classDefinitionInfoFor: eachClass.
		manager nextPutAll: (eachClassInfo at: #superclassName);
			nextPutAll: ' subclass: #';
			nextPutAll: (eachClassInfo at: #name);
			cr; tab; nextPutAll: 'instanceVariableNames: '''.
		(eachClassInfo at: #instVarNames) do: [:each |
			manager nextPutAll: each; space].
		manager nextPut: $';
			cr; tab; nextPutAll: 'classVariableNames: '''.
		(eachClassInfo at: #classVarNames) do: [:each |
			manager nextPutAll: each; space].
		manager nextPut: $';
			cr; tab; nextPutAll: 'poolDictionaries: '''.
		(eachClassInfo at: #poolVarNames) do: [:each |
			manager nextPutAll: each; space].
		manager nextPut: $'.
		useSystemCategories ifTrue: [
			manager cr; tab; nextPutAll: 'category: ''SIF-Support'''].
		manager nextChunkPut: '';
			cr; cr; nextPut: $!!;
			nextPutAll: (eachClassInfo at: #name); nextPutAll: ' class methods';
			nextChunkPut: ''.
		byClass := Dictionary new.
		(manager methodItemsForClass: eachClass) do: [:each |
			(byClass at: each last ifAbsent: [byClass at: each last put: OrderedCollection new])
				add: each first].
		(byClass at: eachClass class ifAbsent: [#()]) do: [:each |
			each == #initialize ifTrue: [
				inits add: eachClass].
			manager cr;
				nextChunkPut: ((manager methodItemInfoFor: each ofClass: eachClass class) at: #source)].
		manager nextChunkPut: ' '.
		manager cr; cr; nextPut: $!!;
			nextPutAll: (eachClassInfo at: #name); nextPutAll: ' methods';
			nextChunkPut: ''.
		(byClass at: eachClass ifAbsent: [#()]) do: [:each |
			manager cr;
				nextChunkPut: ((manager methodItemInfoFor: each ofClass: eachClass) at: #source)].
		manager nextChunkPut: ' ';
			cr].
	inits do: [:each |
		manager cr;
			nextPutAll: each name;
			nextPutAll:' initialize';
			nextChunkPut: ''].
	manager close.
! !

!SmalltalkInterchangeFileOutManager class methodsFor:'testing'!

isAbstract
	"       ^       <Boolean>
	Return true if I represent an abstract class.
	See concreteClasses for a list of concrete classes."

	^self == SmalltalkInterchangeFileOutManager
! !

!SmalltalkInterchangeFileOutManager methodsFor:'accessing'!

addedItems
	"       ^       <OrderedCollection withAll: <???>>
	Return all of the items that were added to me for file out."

	^addedItems
!

globalNamed: name
	"       name    <String>
		^               <Dictionary>
	Return the global named name."

	^Smalltalk at: name asSymbol
!

hierarchicallySortClasses: classes
	"       classes <Collection withAll: <Class>>
		^               <Array withAll: <Class>>
	Return classes sorted in such a way as to have all super classes be listed first,
	and all subclasses come after.
	NOTE: I do not promise to sort them in an in-depth order, in fact its most
		likely to be all classes at one depth followed by the next depth."

	| depths depth superClass |
	depths := Dictionary new.
	classes do: [:each |
		depth := 0.
		superClass := self classOfBehavior: each.
		[superClass := superClass superclass.
		superClass isNil] whileFalse: [
			depth := depth + 1].
		depths at: each put: depth].
	^(classes asSortedCollection: [:a :b | (depths at: a) <= (depths at: b)])
		asArray
!

nameForClass: class
	"       class           <Behavior>
		^               <String>
	Return the name of class."

	^(self classOfBehavior: class) name
!

poolNamed: name
	"       name    <String>
		^               <Dictionary>
	Return the pool named name."

	^Smalltalk at: name asSymbol
! !

!SmalltalkInterchangeFileOutManager methodsFor:'adding'!

addClass: item
	"       item    <Behavior>
		^               self
	Add item as a class to be filed out."

	self addItem: item ofType: #Class.
!

addClassNamed: newEntry
	"       newEntry        <Symbol>
		^               self
	Add all newEntry to the collection of classes that have been eplicitly specified for file out."

	| class |
	class := self installedClassNamed: newEntry.
	class isNil ifTrue: [
		self error: 'There is no such class installed in your image called: ', newEntry printString].
	self addClass: class.
!

addClasses: items
	"       items           <Collection withAll: <Behavior>>
		^               self
	Add each of the classes in items as a class to be filed out.
	Keep the order of items."

	self addItems: items ofType: #Class.
!

addClassesNamed: items
	"       items   <Collection withAll: <Symbol>>
		^               self
	Add the class corresponding to each name in items to be filed out.."

	items do: [:each | self addClassNamed: each].
!

addComment: item
	"       item    <String>
		^               self
	Add item as a comment to be filed out."

	self addItem: item ofType: #Comment.
!

addDoIt: item
	"       item    <String>
		^               self
	Add item as an expression to be evaled on loading."

	self addItem: item ofType: #DoIt.
!

addGlobalNamed: newEntry
	"       newEntry        <String>
		^               self
	Add all newEntry to the collection of Globals that have been eplicitly specified for file out."

	self addGlobalNamed: newEntry initializer: nil.
!

addGlobalNamed: newEntry initializer: initializer
	"       newEntry        <String>
		initializer     nil | #default | <String>
		^               self
	Add all newEntry to the collection of Globals that have been eplicitly specified for file out.
	initializer is a chunk of code to build the Global, this code will be evaled at file in time.
	If initializer is nil, then I rill just allocate the global.  #default I will place the printString
	of it's current value.  If it's a string then use this as the init expression.
	NOTE: The order of the Globals is kept."

	self addItem: (Array with: newEntry with: initializer) ofType: #Global.
!

addGlobalsNamed: newEntries
	"       newEntries      <Collection withAll: <String>>
		^               self
	Add all newEntries to the collection of Globals that have been eplicitly specified for file out."

	newEntries do: [:each |
		self addGlobalNamed: each]
!

addItem: item ofType: type
	"       item    <Object>
		type    <Symbol>
		^       void
	PRIVATE
	Item is either an object of some kind or a collection.  type is one of
	a set which is defined as being valid for me.  My subclasses may add
	more, but for now I can deal generically with all types.  The ones I
	know of now are #classes, #globals, #globals, #packages, #pools.
	If two items of the same type are added to me one after the other, then
	I will combine then into one item."

	addedItems last last = type ifTrue: [
		addedItems last first add: item.
		^self].
	addedItems addLast: (Array with: (OrderedCollection with: item) with: type).
!

addItems: items ofType: type
	"       items           <Collection withAll: <Object>>
		type            <Symbol>
		^       void
	PRIVATE
	Item is either an object of some kind or a collection.  type is one of
	a set which is defined as being valid for me.  My subclasses may add
	more, but for now I can deal generically with all types.  The ones I
	know of now are #classes, #globals, #globals, #packages, #pools.
	If two items of the same type are added to me one after the other, then
	I will combine then into one item."

	addedItems last last = type ifTrue: [
		addedItems last first addAll: items.
		^self].
	addedItems addLast: (Array
		with: (OrderedCollection new addAll: items;
			yourself)
		with: type).
!

addMethodNamed: entry ofClass: class
	"       entry           <Symbol>
		class           <Behavior>
		^               self
	Add all newEntries to the collection of methods that have been eplicitly specified for file out."

	self addItem: (Array with: entry with: class) ofType: #Method.
!

addPackage: newEntry
	"       newEntry        <Dictionary key: <Symbol> value: <Object>>
	Add newEntry to the collection of packages that have been eplicitly specified for file out.
		#name                   <String>
		#classes                        <Class>
		#methods                        <Collection withAll: <CompiledMethod>>
		#preInstallCode         <String>
		#postInstallCode                <String>
		#preUnInsrallCode       <String>
		#postUninstallCode      <String>
	"

	self addItem: newEntry ofType: #Package.
!

addPackageNamed: newEntry
	"       newEntry        <String>
		^               self
	Add newEntry to the collection of packages that have been eplicitly specified for file out."

	self addPackage: (self packageItemInfoFor: newEntry).
!

addPackages: newEntries
	"       newEntries      <Collection withAll: <Dictionary key: <Symbol> value: <Object>
		^               self
	Add newEntries to the collection of packages that have been eplicitly specified for file out.
	See packageItemInfoForPackageNamed: for info on what what each entry should be."

	self addItems: newEntries ofType: #Package.
!

addPackagesNamed: newEntries
	"       newEntries      <Collection withAll: <String>>
		^               self
	Add all newEntries to the collection of packages that have been eplicitly specified for file out."

	newEntries do: [:each |
		self addPackageNamed: each]
!

addPoolNamed: newEntry
	"       newEntry        <String>
		^               self
	Add all newEntry to the collection of Pools that have been eplicitly specified for file out.
	I will define each of the keys in new entry with no initializer."

	self addPoolNamed: newEntry initializer: nil
!

addPoolNamed: newEntry initializer: initializer
	"       newEntry        <String>
		initializer     nil | #default | <Dictionary key: varName <String> value: valueExpression <String>>
		^               self
	Add all newEntry to the collection of Pools that have been eplicitly specified for file out.
	If initializer is nil, then I will put out a default initializer that will create a pool with the
	same keys as found now, and will leave the values nil.
	If initializer is #default then I will do the same as nil, put I will also put something out
	to initialize.
	If initializer is a dictionary, then I will put out the expression for each var define in initializer.
	NOTE: The order of the pools is kept."

	self addItem: (Array with: newEntry with: initializer) ofType: #Pool.
!

addPoolsNamed: newEntries
	"       newEntries      <Collection withAll: <String>>
		^               self
	Add all newEntries to the collection of pools that have been eplicitly specified for file out."

	newEntries do: [:each |
		self addPoolNamed: each]
! !

!SmalltalkInterchangeFileOutManager methodsFor:'fileout'!

fileOut
	"       ^       self
	File out whatever code the user has specified for me to do."

	self nextPutAll: 'Smalltalk interchangeVersion: ';
		nextQuotedPutString: versionString;
		nextChunkPut: '';
		cr;
		cr.
	self fileOutItems;
		close
!

fileOutAnnotationKey: key value: value
	"       key     <String>
		value   <String> | <Collection withAll: <String>>
	Put out an annotation to my managed stream that has its key beign key and value being value.
	If value is a collection of strings, then put something in that can then be broken out into a
	collection of strings by collectionOfStringsFrom:"

	self cr;
		nextPutAll: 'Annotation key: ';
		nextQuotedPutString: key;
		nextPutAll: ' value: '.
	(value isKindOf: String)
		ifTrue: [self nextQuotedPutString: value]
		ifFalse: [self nextPutCollectionOfStrings: value].
	self nextChunkPut: ''
!

fileOutClassDefinitionsFor: items
	"       items           <OrderedCollection withAll: <Class>>
		^               void
	File out the definitions for the classes in items."

	| info |
	(self hierarchicallySortClasses: items) do: [:eachClass |
		info := self classDefinitionInfoFor: eachClass.
		self nextPutAll: 'Class named: ';
			nextQuotedPutString: (info at: #name);
			cr; tab;
			nextPutAll: 'superclass: ';
			nextQuotedPutString: (info at: #superclassName);
			cr; tab;
			nextPutAll: 'indexedInstanceVariables: ';
			nextSymbolPutString: (info at: #instVarType);
			cr; tab;
			nextPutAll: 'instanceVariableNames: '''.
		(info at: #instVarNames) do: [:each |
			self nextPutAll: each;
				space].
		self nextPut: $';
			cr; tab;
			nextPutAll: 'classVariableNames: '''.
		(info at: #classVarNames) do: [:each |
			self nextPutAll: each;
				space].
		self nextPut: $';
			cr; tab;
			nextPutAll: 'sharedPools: '''.
		(info at: #poolVarNames) do: [:each |
			self nextPutAll: each;
				space].
		self nextPut: $';
			cr; tab;
			nextPutAll: 'classInstanceVariableNames: '''.
		(info at: #classInstVarNames) do: [:each |
			self nextPutAll: each;
				space].
		self nextPut: $';
			nextChunkPut: ''.
		(info at: #annotations ifAbsent: [Dictionary new]) associationsDo: [:assoc |
			self fileOutAnnotationKey: assoc key value: assoc value].
		self cr]
!

fileOutClassItems: items
	"       items   <OrderedCollection withAll: <Class>>
		^               void
	File out the classes in items."

	| selector methods |
	self fileOutClassDefinitionsFor: items.
	methods := OrderedCollection new: 64.
	items do: [:each |
		methods addAll: (self methodItemsForClass: each)].
	methods := self fileOutMethodItems: methods.
	methods do: [:each |
		self fileOutInitializer: (self nameForClass: each last), ' ', each first forName: (self nameForClass: each last)].
!

fileOutCommentItems: items
	"       items           <OrderedCollection withAll: <String>>
		^               void
	File out the comments in items."

	items do: [:each |
		self cr;
			nextPut: $";
			nextChunkablePut: each;
			nextPut: $";
			nextChunkPut: ''.].
!

fileOutDoItItems: items
	"       items           <OrderedCollection withAll: <String>>
		^               void
	File out the doIts in items."

	self cr.
	items do: [:each |
		self fileOutInitializer: each last forName: 'Global'].
!

fileOutDummyItems: dummy
	"       dummy   <Array new: 0>
		^               void
	I'm here to handle the dummy item added to the start of my added items list."
!

fileOutGlobalItems: items
	"       items           <OrderedCollection withAll: <Array with: name <String> with: initializer <String> | nil>
		^               void
	File out the globals in items."

	| initializer |
	items do: [:each |
		self cr;
			nextPutAll: 'Global variable: ';
			nextQuotedPutString: each first;
			nextChunkPut: ''].
	self cr.
	items do: [:each | each last isNil ifFalse: [
		initializer := each last == #default
			ifTrue: [(self globalNamed: each first) printString]
			ifFalse: [each last].
		self fileOutInitializer: initializer forName: each first]].
!

fileOutInitializer: initializer forName: name
	"       initializer     <String>
		name            <String>
		^               void
	File out the an initializer whose code is initializer for a global or class named name."

	self cr;
		nextPutAll: name;
		nextPutAll: ' initializer';
		nextChunkPut: '';
		cr;
		nextChunkPut: initializer.
!

fileOutItems
	"       ^       self
	File out whatever code the user has specified for me to do."

	self addedItems do: [:each |
		self perform: (fileOutInfoByType at: each last) with: each first].
!

fileOutMethodItem: item
	"       item            <Array with: <Symbol> with: <Class>>
	"

	| info |
	info := self methodItemInfoFor: item first ofClass: item last.
	self cr;
		nextPutAll: (info at: #className);
		nextPutAll: ((info at: #isClassMethod) ifTrue: [' classMethod'] ifFalse: [' method']);
		nextChunkPut: '';
		cr;
		nextChunkPut: (info at: #source).
	(info at: #annotations ifAbsent: [Dictionary new]) associationsDo: [:assoc |
		self fileOutAnnotationKey: assoc key value: assoc value].
	self cr
!

fileOutMethodItems: items
	"       items           <OrderedCollection withAll: <Array with: <Symbol> with: <Behavior>>>>
		^               <OrderedCollection withAll: <Array with: <Symbol> with: <Behavior>>>
	File out the classes in items.
	Return a collection of all the classes that had an initialize method I filed out.  This
	collection is sorted in the hierarchichal order of the classes."

	| byClass result methods selector |
	result := OrderedCollection new.
	byClass := Dictionary new.
	items do: [:each |
		(byClass at: each last ifAbsent: [byClass at: each last put: (OrderedCollection new: 16)])
			add: each].
	(self hierarchicallySortClasses: byClass keys) do: [:eachClass |
		methods := byClass at: eachClass.
		(self classOfBehavior: eachClass) ~~ eachClass ifTrue: [
			selector := self initializerSelectorForClass: eachClass.
			selector isNil ifFalse: [
				(methods detect: [:each | each first == selector] ifNone: [nil]) isNil ifFalse: [
					result add: (Array with: selector with: eachClass)]]].
		methods do: [:each |
			self fileOutMethodItem: each]].
	^result
!

fileOutPackageItem: item
	"       item    <Dictionary>    See packageItemInfoForPackageNamed:
		^               void
	File out a declaration and contents of the package named name."

	| inits |
	(item at: #preRequisiteNames ifAbsent: ['']) isEmpty ifFalse: [
		self fileOutInitializer: '' forName: 'Global';
			fileOutAnnotationKey: 'package' value: (item at: #name) asString;
			fileOutAnnotationKey: 'package-preRequisites' value: (item at: #preRequisiteNames);
			cr].
	(item at: #preInstallCode ifAbsent: ['']) isEmpty ifFalse: [
		self fileOutInitializer: (item at: #preInstallCode) forName: 'Global';
			fileOutAnnotationKey: 'package' value: (item at: #name) asString;
			fileOutAnnotationKey: 'package-preInstallCode' value: (item at: #preInstallCode);
			cr].
	(item at: #preUnInstallCode ifAbsent: ['']) isEmpty ifFalse: [
		self fileOutInitializer: '' forName: 'Global';
			fileOutAnnotationKey: 'package' value: (item at: #name) asString;
			fileOutAnnotationKey: 'package-preUnInstallCode' value: (item at: #preUnInstallCode);
			cr].
	(item at: #postUnInstallCode ifAbsent: ['']) isEmpty ifFalse: [
		self fileOutInitializer: '' forName: 'Global';
			fileOutAnnotationKey: 'package' value: (item at: #name) asString;
			fileOutAnnotationKey: 'package-postUnInstallCode' value: (item at: #postUnInstallCode);
			cr].
	self fileOutClassDefinitionsFor: (item at: #classes).
	inits := self fileOutMethodItems: (item at: #methods).
	inits do: [:each |
		self fileOutInitializer: (self nameForClass: each last), ' ', each first forName: (self nameForClass: each last)].
	(item at: #postInstallCode ifAbsent: ['']) isEmpty ifFalse: [
		self fileOutInitializer: (item at: #postInstallCode) forName: 'Global';
			fileOutAnnotationKey: 'package' value: (item at: #name) asString;
			fileOutAnnotationKey: 'package-postInstallCode' value: (item at: #postInstallCode)].
!

fileOutPackageItems: items
	"       items           <OrderedCollection withAll: <Array with: name <String> with: initializer <String> | nil>
		^               void
	File out the globals in items."

	items do: [:each |
		self fileOutPackageItem: each].
	self cr.
!

fileOutPoolDefinitionFor: pool named: name initializer: initializer
	"       poolName        <Dictionary key: <String> value: <Object>>
		name            <String>
		initializer     nil | #default | <Dictionary key: <String> value: <String>>
		^               void
	File out the definition of the pool poolName.
	See addPool* for info on initializer."

	| initializers infos info |
	self cr;
		nextPutAll: 'Pool named: ';
		nextQuotedPutString: name asString;
		nextChunkPut: ''.
	infos := Dictionary new.
	pool associationsDo: [:assoc |
		info := infos at: assoc key put: (self poolVariableItemInfoFor: assoc key in: pool).
		self cr;
			nextPutAll: name asString;
			space;
			nextPutAll: ((info at: #isConstant) ifTrue: ['constant:'] ifFalse: ['variable:']);
			space;
			nextQuotedPutString: assoc key asString;
			nextChunkPut: ''].
	self cr.
	initializer isNil ifTrue: [
			"The default is for all the values to be nil."
		^self].
	initializer == #default
		ifTrue: [
			initializers := Dictionary new.
			pool associationsDo: [:assoc |
				info := infos at: assoc key.
				(info at: #initializer ifAbsent: ['']) isEmpty ifFalse: [
					initializers at: assoc key put: (info at: #initializer)]]]
		ifFalse: [initializers := initializer].
	initializers associationsDo: [:assoc |
		(pool includesKey: assoc key) ifFalse: [
			self error: 'Trying to init a pool variable that is not within this pool.'].
		self cr;
			nextPutAll: name asString;
			nextPutAll: ' initializerFor: ';
			nextQuotedPutString: assoc key;
			nextChunkPut: '';
			cr;
			nextChunkPut: assoc value].
!

fileOutPoolItems: items
	"       items           <OrderedCollection withAll: <Array with: name <String> with: initializer <String> | nil>
		^               void
	File out the globals in items."

	items do: [:each | self
		fileOutPoolDefinitionFor: (self poolNamed: each first)
		named: each first
		initializer: each last].
! !

!SmalltalkInterchangeFileOutManager methodsFor:'fileout info'!

classDefinitionInfoFor: class
	"       class           <Behavior>
		^               <Dictionary key: <Symbol> value: <String>>
	Return info on
		#name                   <String>
		#superclassName <String>
		#instVarType            #byte | #object | #none
		#instVarNames           <Array withAll: <String>>
		#classVarNames          <Array withAll: <String>>
		#poolVarNames           <Array withAll: <String>>
		#classInstVarNames      <Array withAll: <String>>
		#annotations            <Dictionary key: <String> value: <String>>
	"

	self subclassResponsibility
!

initializerSelectorForClass: class
	"       class           <Behavior>
		^               <Symbol> | nil
	Return the method to invoke in order to initialize class on file in.
	Return nil if there is to be none."

	((self classOfBehavior: class) class includesSelector: #initialize) ifTrue: [
		^#initialize].
	^nil
!

methodItemInfoFor: methodName ofClass: class
	"       methodName      <Symbol>
		class                   <Behavior>
		^               <Dictionary key: <Symbol> value: <Object>>
	Return info on method
		#className      <String>
		#isClassMethod  <Boolean>
		#source         <String>
		#annotations    <Dictionary key: <String> value: <String?
	"

	self subclassResponsibility
!

methodItemsForClass: class
	"       class           <Behavior>
		^               <Collection withAll: <Array with: <Symbol> with: <Behavior>>>
	"

	| methods |
	methods := OrderedCollection new: 64.
	class selectors do: [:each |
		methods add: (Array with: each with: class)].
	class class selectors do: [:each |
		methods add: (Array with: each with: class class)].
	^methods
!

packageItemInfoFor: name
	"       name    <String>
		^               <Dictionary key: <Symbol> value: <Object>>
	Return info on the package named name.
		#name                   <String>
		#classes                        <Class>
		#methods                        <Collection withAll: <Array with: <Symbol> with: <Behavior>>>
		#preInstallCode         <String>
		#postInstallCode                <String>
		#preUnInsrallCode       <String>
		#postUninstallCode      <String>
		#preRequisiteNames      <Collection withAll: <String>>
	The list of classes are the classes I file out their definition as part of this package.
	methods, includes all methods from the classes in the classes list.  This is to make
	sure that only the methods belonging to the class and package are included.  It can
	contain other methods that are part of the package but not necessarily part of one
	the package's classes."

	self subclassResponsibility
!

poolVariableItemInfoFor: variableName in: pool
	"       variableName    <String>
		pool                    <Dictionary key: <String> value: <Object>>
		^               <Dictionary key: <Symbol> value: <Object>>
	Return info on method
		#isConstant     <Boolean>
		#initializer            <String>
	"

	self subclassResponsibility
! !

!SmalltalkInterchangeFileOutManager methodsFor:'initializing'!

initialize
	"       ^       self
	Set myself up to be ready to file out under Version 1.0."

	super initialize.
	addedItems := OrderedCollection new: 32.
		"Add at least one item such that I don't have to check for isEmpty on
			my add code."
	addedItems add: #(#() #Dummy).
	self initializeForVersion10
!

initializeForVersion10
	"       ^       self
	Set myself up to be ready to file out under Version 1.0 Extended.
	NOTE: I work from the assumption that I have already been initialized as a Version 1.0 file out."

	versionString := self version10String.
	fileOutInfoByType := Dictionary new.
	#(#Pool #Package #Method #Global #DoIt #Comment #Class #Dummy) do: [:each |
		fileOutInfoByType at: each put: ('fileOut', each, 'Items:') asSymbol].
!

initializeForVersion10Extended
	"       ^       self
	Set myself up to be ready to file out under Version 1.0 Extended.
	NOTE: I work from the assumption that I have already been initialized as a Version 1.0 file out."

	versionString := self version10ExtendedString
!

version10ExtendedString
	"       ^       <String>
	Return the string to use for version 1.0 extended."

	^'1.0 Extended'
!

version10String
	"       ^       <String>
	Return the string to use for version 1.0"

	^'1.0'
! !

!SmalltalkInterchangeFileOutManager methodsFor:'streaming'!

cr

	self managedStream cr
!

newStreamOnFileNamed: file
	"       file    <String>
		^       <WriteStream>
	Return a stream that is opened on the file named file.
	If I am meant to write them return a write stream, read then a read strea."

	^self subclassResponsibility
!

nextChunkPut: chunk
	"       chunk   <String>
	Add the contents of chunk to my stream to use.  Chunking means to end with a single !!.  IF there are any embedded !!s, they will doubled up."

	self nextChunkablePut: chunk;
		nextPut: $!!
!

nextChunkablePut: chunk
	"       chunk   <String>
	Add the contents of chunk to my stream to use.
	Make sure that nextChunk functionality is kept, meaning that
	yf there are any embedded !!s, they will doubled up."

	(chunk includes: $!!)
		ifTrue: [
			chunk do: [ :character |
				self managedStream nextPut: character.
				character == $!! ifTrue: [self managedStream nextPut: $!!]]]
		ifFalse: [
			self managedStream nextPutAll: chunk].
!

nextPut: char
	self managedStream nextPut: char
!

nextPutAll: all

	self managedStream nextPutAll: all
!

nextPutCollectionOfStrings: strings
	"       strings <Collection withAll: <String>>
		^               void
	Place a string which when read back can yield a collection of strings."

	self pushStream: (WriteStream on: (String new: 32)).
	strings do: [:each |
		self nextChunkPut: each].
	self nextChunkablePut: self popStream contents printString
!

nextQuotedPutString: string
	"       string          <String>
	"

	self print: string asString
!

nextSymbolPutString: string
	"       string          <String>
	"

	self print: string asSymbol
!

print: object
	self managedStream print: object
!

space
	self managedStream space
!

tab
	self managedStream tab
! !

!SmalltalkInterchangeFileOutManager methodsFor:'testing'!

is10
	"       ^       <Boolean>
	Return true if I am for version 1.0."

	^versionString == self version10String
!

is10Extended
	"       ^       <Boolean>
	Return true if I am for version 1.0 extended."

	^versionString == self version10ExtendedString
! !

!SmalltalkInterchangeFileOutManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeFileOutManager.st,v 1.2 2001/09/04 20:45:50 cg Exp $'
! !
