"{ Package: 'stx:goodies/sif' }"

SmalltalkInterchangeFileInManager subclass:#SmalltalkInterchangeSTXFileInManager
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIF-ST/X'
!


!SmalltalkInterchangeSTXFileInManager class methodsFor:'initializing'!

bootstrap
	"
	SmalltalkInterchangeFileManager newForFileIn
		fileName: 'C:\Dev\Dolphin Smalltalk 98\sif-support.sif';
		fileIn

	SmalltalkInterchangeFileManager newForFileIn
		fileName: 'C:\Dev\Dolphin Smalltalk 98\SifTestAll.sif';
		fileIn
	"
!

initialize
        "       ^       self
        Initialize everything that is needed for my being used by the Dolphin dialect."
        "
        SmalltalkInterchangeSTXFileInManager initialize
        "

                "This is here only to allow the file in of the code to put the
                        do it to init me."
        super initialize.
! !

!SmalltalkInterchangeSTXFileInManager methodsFor:'file in'!

fileInClassItem: item

        "       item    <SmalltalkInterchangeFileItem>
                ^       void
        item is a class item that needs to be processed.
        Item attributes:
        Item attributes:
                name                            <String> 'named:'
                superclassName          <String> 'superclass:'
                instVarType                     #none | #byte | #object 'indexedInstanceVariables:'
                instVarNames            <String>'instanceVariableNames:'
                classVarNames           <String> 'classVariableNames:'
                sharedPoolNames         <String> 'sharedPools:'
                classInstVarNames       <String> 'classInstanceVariableNames:'
        item annotations
                package
        "

        | result package category definitionMessage |

        definitionMessage := #'subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:'.
        item instVarType == #none ifFalse: [
            item instVarType == #byte ifTrue: [
                definitionMessage := #'variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:'.
            ] ifFalse:[
                item instVarType == #object ifTrue: [
                    definitionMessage := #'variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:'.
                ] ifFalse:[
                    self error: 'Have not added support for variable subclasses yet'
                ].
            ].
        ].
        package := (package := item packageAnnotation) isNil
                ifTrue: ['None Specified-SIF']
                ifFalse: [package value].
        category := (category := item categoryAnnotation ) isNil
                ifTrue: [package]
                ifFalse: [category value].
        result := (Smalltalk at: item superclassName asSymbol)
                    perform:definitionMessage
                with: "subclass:" item name asSymbol
                with: "instanceVariableNames:" item instVarNames
                with: "classVariableNames:" item classVarNames
                with: "poolDictionaries:" item sharedPoolNames
                with: "category:" category.
        result class instanceVariableNames: item classInstVarNames.
!

fileInMethodItem: item intoClass: class
        "       item    <SmalltalkInterchangeFileItem>
                ^       void
        item is a smalltalk item that needs to be processed.
        Item attributes:
                firstToken              Class name
                value           Source string
        item annotations
                category
                categories
                package"

        | method |
        method := item categoryAnnotation isNil
                ifTrue: [class compile: item value notifying: nil]
                ifFalse: [class compile: item value classified: item categoryAnnotation value].
        method isNil ifTrue: [
                self error: 'Compile error.'].
!

handleAnnotationsOnGlobalInitializerItem: item
        "       item    <SmalltalkInterchangeFileItem>
                ^       void
        item is an initializer item that needs to be processed.  I am called by the generic handler
        in order to handle the annotations on the item.
        Item attributes:
                firstToken              Global name
                value           Code to eval
        item annotations
                Package-PreInstallCode
                Package-PreUnInstallCode
                Package-PostInstallCode
                Package-PosUntInstallCode"

        | annotation missing allCategories |
        annotation := item annotationNamed: 'package-preRequisites'.
        annotation isNil ifFalse: [
                allCategories := (Smalltalk allClasses collect:[:each | each category]) asSet.
                missing := Set new.

                (self collectionOfStringsFrom: annotation value) do: [:each |
                        (allCategories includes: each) ifFalse: [
                                missing add: each]].
                missing isEmpty ifFalse: [
                        Transcript cr; show: '### SIF: Missing pre-requisites:'.
                        missing asSortedCollection do: [:each |
                                Transcript cr; tab; show: each]]]
! !

!SmalltalkInterchangeSTXFileInManager methodsFor:'streaming'!

newStreamOnFileNamed: file
        "       file    <String>
                ^       <ReadStream>
        Return a stream that is opened on the file named file.
        If I am meant to write them return a write stream, read then a read strea."

        |s|

        (file asFilename exists) ifFalse: [
                self error: 'Non-existent file named: ', file asFilename pathName].
        s := file asFilename readStream.
        s eolMode:#crlf.
        ^ s
!

nextWord
	"	^			<String> | nil
	Return the next word found on my managed stream.  If I reach the end of the stream before
	finding a word, then return nil.
	I skip any current whitespcae, start collecting, and stop at the first white space."

	"| s m |
	s := ReadStream on: 'hello world'.
	m := SmalltalkInterchangeSqueakFileInManager new.
	m nextWordOn: s.
	m nextWordOn: s"

	| result done next |
	self skipWhiteSpace.
	done := false.
	result := WriteStream on: (String new: 32).
	[self atEnd or: [done]] whileFalse: [
		next := self managedStream next.
		next isSeparator
			ifTrue: [
				done := true.
				self managedStream skip: -1]
			ifFalse: [result nextPut: next]].
	result size = 0 ifTrue: [
		^nil].
	^result contents
! !

!SmalltalkInterchangeSTXFileInManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeSTXFileInManager.st,v 1.4 2003/09/29 08:33:54 cg Exp $'
! !

SmalltalkInterchangeSTXFileInManager initialize!
