"{ Package: 'stx:goodies/sif' }"

SmalltalkInterchangeFileOutManager subclass:#SmalltalkInterchangeSTXFileOutManager
        instanceVariableNames:'mostRecentClasses'
        classVariableNames:''
        poolDictionaries:''
        category:'SIF-ST/X'
!


!SmalltalkInterchangeSTXFileOutManager class methodsFor:'initializing'!

initialize
        "       ^       self
        Initialize everything that is needed for my being used by the Dolphin dialect."
        "
        SmalltalkInterchangeSTXFileOutManager initialize
        "

                "This is here only to allow the file in of the code to put the
                        do it to init me."
        super initialize.
! !

!SmalltalkInterchangeSTXFileOutManager methodsFor:'accessing'!

classOfBehavior: class
        "       class           <Behavior>
                ^               <Class>
        If class is a Class then return it, if not then class is a metaclass,
        return it's sole instance."
        "
        SmalltalkInterchangeDolphinFileOutManager new classOfBehavior: Class
        SmalltalkInterchangeDolphinFileOutManager new classOfBehavior: Class class
        "

        ^class theNonMetaclass 
! !

!SmalltalkInterchangeSTXFileOutManager methodsFor:'fileout info'!

classDefinitionInfoFor: class
        "       class   <Behavior>
                ^               <Dictionary key: <Symbol> value: <String>>
        Return info on
                #name                   <String>
                #superclassName         <String>
                #instVarType            #byte | #object | #none
                #instVarNames           <Array withAll: <String>>
                #classVarNames          <Array withAll: <String>>
                #poolVarNames           <Array withAll: <String>>
                #classInstVarNames      <Array withAll: <String>>
                #annotations            <Dictionary key: <String> value: <String>>
        "

        ^Dictionary new
                at: #name put: class name asString;
                at: #superclassName put: (class superclass isNil ifTrue: [''] ifFalse: [class superclass name asString]);
                at: #instVarType put: (class isVariable
                        ifTrue: [ 
                                class isBytes 
                                        ifTrue: [#byte]
                                        ifFalse: [#object] ]
                        ifFalse: [#none]);
                at: #instVarNames put: class instVarNames;
                at: #classVarNames put: class classVarNames;
                at: #poolVarNames put: class sharedPools;
                at: #classInstVarNames put: class class instVarNames;
                at: #annotations put: (Dictionary new
                        "at: 'comment' put: '';."
                        " at: 'package' put: class category asString; "
                        at: 'category' put: class category asString;
                        yourself);
                yourself
!

methodItemInfoFor: methodName ofClass: class
        "       methodName      <Symbol>
                class                   <Behavior>
                ^               <Dictionary key: <Symbol> value: <Object>>
        Return info on method
                #className      <String>
                #isClassMethod  <Boolean>
                #source                 <String>
                #annotations    <Dictionary key: <String> value: <String?
        "

        | result |
        result := Dictionary new.
        result at: #className put: (self nameForClass: class);
                at: #isClassMethod put: class isMeta.
        (result at: #source put: (class compiledMethodAt: methodName) source) isNil ifTrue: [
                self error: 'There is no source for this method'].
        result at: #source put: (result at: #source) asString.
        result at: #annotations put: (Dictionary new
                at: 'category' put: (class whichCategoryIncludesSelector: methodName) asString;
                yourself).
        ^result
!

packageItemInfoFor: name
        "       name    <String>
                ^               <Dictionary key: <Symbol> value: <Object>>
        Return info on the package named name
                #name                   <String>
                #classes                        <Class>
                #methods                        <Collection withAll: <CompiledMethod>>
                #preInstallCode                 <String>
                #postInstallCode                <String>
                #preUnInsrallCode       <String>
                #postUninstallCode      <String>
        "

        | classes methods eachClass |
        classes := Smalltalk allClassesInCategory:name.
        methods := OrderedCollection new: 64.
        classes do:[:eachClass | 
                eachClass selectors do: [:each |
                        methods add: (Array with: each with: eachClass)].
                eachClass class selectors do: [:each |
                        methods add: (Array with: each with: eachClass class)]].
        ^Dictionary new
                at: #name put: name;
                at: #classes put: classes;
                at: #methods put: methods;
                at: #preInstallCode put: '';
                at: #postInstallCode put: '';
                at: #preUnInstallCode put: '';
                at: #postUnInstallCode put: '';
                yourself
!

poolVariableItemInfoFor: variableName in: pool
        "       variableName    <String>
                pool                    <Dictionary key: <String> value: <Object>>
                ^               <Dictionary key: <Symbol> value: <Object>>
        Return info on method
                #isConstant     <Boolean>
                #initializer            <String>
        "

        ^Dictionary new
                at: #isConstant put: false;
                at: #initializer put: (pool at: variableName) printString;
                yourself
! !

!SmalltalkInterchangeSTXFileOutManager methodsFor:'streaming'!

newStreamOnFileNamed: file
        "       file    <String>
                ^       <WriteStream>
        Return a stream that is opened on the file named file.
        If I am meant to write them return a write stream, read then a read strea."

        ^FileStream newFileNamed: file
!

nextQuotedPutString: string
        "       string          <String>
        "

        self print: string storeString
!

nextSymbolPutString: string
        "       string          <String>
        "

        self nextPut: $#.
        ^super nextSymbolPutString: string
! !

!SmalltalkInterchangeSTXFileOutManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/sif/SmalltalkInterchangeSTXFileOutManager.st,v 1.3 2002/06/21 09:39:59 penk Exp $'
! !
SmalltalkInterchangeSTXFileOutManager initialize!
