"
 COPYRIGHT (c) 2002 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/simpleServices' }"

Object subclass:#AbstractSTXServer
	instanceVariableNames:'inStream outStream socket myPortNumber connectProcess'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-Smalltalk'
!

AbstractSTXServer class instanceVariableNames:'BadHosts OkHosts ServerRunning Password LocalConnectionsOnly Logging
	ErrorLogging ErrorDebugging'

"
 No other class instance variables are inherited by this class.
"
!

!AbstractSTXServer class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2002 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Abstract superclass with common code for my subclasses:
        STXScriptingServer      - allows for scripting of STX applications via a socket
                                  in a JavaScript like language.

        WindowMigrationServer   - allows for X-windows to be pulled to another screen.

    Notice:
        all of my subclasses are singleton classes; there is only one (and only one)
        of these servers running per image.

    [author:]
        Claus Gittinger

    [see also:]
        RDoItServer
"
! !

!AbstractSTXServer class methodsFor:'instance creation'!

hasServerRunning
    "return true, if a server is running"

    ^ ServerRunning notNil
!

new
    ^ self basicNew initialize

    "Created: / 05-12-2011 / 23:49:39 / cg"
!

server
    "return the server singleton if it is running. nil otherwise"

    ^ ServerRunning
!

serverRunning
    "return true, if a server is running"

    ^ ServerRunning notNil
!

setServerRunning:aServer
    ServerRunning := aServer
!

start
    "Start an rdoit server on the default port."

    ^ self startAt:(self defaultPortNumber)

    "
     WindowMigrationServer start.
    "
!

startAt:aPortNumber
    "Start a server on a specified port.
     If the passed portNumber is nil, the class-specific default is used.
     If it is 0, a class specific alternative channel is used (typically: stdin/stdout)"

    |server|

    server := self new.
    ServerRunning isNil ifTrue:[
        aPortNumber == self defaultPortNumber ifTrue:[
            ServerRunning := server
        ].
    ].

    ^ server startServerAt:aPortNumber

    "
     WindowMigrationServer startAt:8004.
     WindowMigrationServer startAt:9104.

     STXScriptingServer startAt:8008.
     STXScriptingServer startAt:nil.
     STXScriptingServer startAt:0.
    "
! !

!AbstractSTXServer class methodsFor:'defaults'!

defaultPortNumber
    "the port we are listening on (by default).
     However, a server may also be started on another port"

    self subclassResponsibility
!

errorDebugging:aBoolean
    ErrorDebugging := aBoolean
!

errorLogging:aBoolean
    ErrorLogging := aBoolean
!

isErrorDebugging
    ^ ErrorDebugging == true
!

isErrorLogging
    ^ ErrorLogging == true
!

isLogging
    ^ Logging == true
!

logging:aBoolean
    Logging := aBoolean
!

password
    "the password; if nil, no authentication is performed"

    ^ Password
!

password:aString
    "set the password; if nil, no authentication is performed"

    Password := aString
! !

!AbstractSTXServer class methodsFor:'host management'!

allowHost:aHostname
    "allow a host"

    OkHosts isNil ifTrue:[ OkHosts := Set new ].
    OkHosts add: aHostname.
    BadHosts notNil ifTrue:[
	BadHosts remove:aHostname ifAbsent:[].
    ].

    "
     WindowMigrationServer allowHost:'localhost'
     WindowMigrationServer allowHost:'193.12.12.0'
     WindowMigrationServer allowHost:'fooHost'
    "

    "Modified: / 20.5.1998 / 18:34:49 / cg"
!

allowedHosts
    "return the list of allowed hosts"

    ^ OkHosts ? #()

    "
     WindowMigrationServer allowedHosts
     STXScriptingServer allowedHosts
    "
!

badHosts
    ^ BadHosts
!

deniedHosts
    "return the list of denied hosts"

    ^ BadHosts ? #()

    "
     WindowMigrationServer deniedHosts
     STXScriptingServer deniedHosts
    "
!

denyHost:aHostname
    "disallow a host"

    BadHosts isNil ifTrue:[ BadHosts := Set new ].
    BadHosts add: aHostname.
    OkHosts notNil ifTrue:[
	OkHosts remove:aHostname ifAbsent:[].
    ].

    "
     WindowMigrationServer denyHost:'untrustedHost'
    "

    "Modified: / 20.5.1998 / 18:34:55 / cg"
!

isHostAllowed:aHostName
    |isInBadHosts isInOkHosts ok realHostName|

    "/ AllowAllHosts == true ifTrue:[^ true].

    aHostName = 'localhost' ifTrue:[
        realHostName := OperatingSystem getHostName.
    ].

    ok := false. 
    isInBadHosts := (BadHosts includes:aHostName) or:[BadHosts includes:realHostName].
    isInBadHosts ifFalse:[ 
        isInOkHosts := (OkHosts includes:aHostName) or:[OkHosts includes:realHostName].
        isInOkHosts ifTrue: [
            ok := true.
        ] ifFalse: [
"/            (self confirmNewHost:aHostName) ifTrue: [
"/                ok := true
"/            ]
        ]
    ].
    ^ ok
!

localConnectionsOnly
    "if true (the default), only local connections are accepted"

    ^ LocalConnectionsOnly ? true
!

localConnectionsOnly:aBoolean
    "if true only local connections are accepted"

    LocalConnectionsOnly := aBoolean
!

okHosts
    ^ OkHosts
!

showHostsOn:aStream
    "Output the current hosts, normally to stdout."

    aStream
        show: ((self class name),' [info]: Authorized Hosts: ');
        showCR: OkHosts;
        show: ((self class name),' [info]: Unauthorized Hosts: ');
        showCR: BadHosts.
! !

!AbstractSTXServer class methodsFor:'server destruction'!

stop
    "shut down the single instance of the server. (overkill)"

    |server|

    (server := ServerRunning) notNil ifTrue:[
        ServerRunning := nil.
        server release.
    ].

    "
     WindowMigrationServer stop.
    "
! !

!AbstractSTXServer methodsFor:'handle client'!

checkConnectionAllowed:aSocket
    |anotherDisplay targetSocketAddress hostName|

    targetSocketAddress := aSocket getPeer.
    targetSocketAddress isLocal ifTrue:[
        anotherDisplay := Screen default.
    ] ifFalse:[
        (self class localConnectionsOnly ? true) ifTrue:[
            aSocket nextPutLine:'Permission denied.'.
            ^ false
        ].
        hostName := targetSocketAddress hostNameOrAddressString.
        (self class isHostAllowed:hostName) ifFalse:[
            aSocket nextPutLine:'Permission denied.'.
            ^ false
        ].
    ].

    (self performAuthenticationOn:aSocket) ifFalse:[
        aSocket nextPutLine:'Permission denied.'.
        ^ false
    ].

    ^ true
!

handleIncomingRequestFrom:aSocket
    self subclassResponsibility
!

loopForever
    "Called when snapshot is started, and at instance creation.
     Create and listen to my socket. When a connection arrives,
     to handle the request (in #handleIncomingRequestFrom:).
     With a portNumber of 0, serve stdIn/stdOut.

     Note: Someone else has already forked me, so I run as a subprocess and can block."

    |myPortNumber thisSocket|

    myPortNumber := self portNumber.
    (self class name,' [info]: serving ',myPortNumber printString) infoPrintCR.

    myPortNumber == 0 ifTrue:[
        "/ serve stdin/stdout
        self copy 
            setupDefaultsFrom:self;
            handleScriptCommandsReadingFrom:Stdin writingTo:Stdout.
        ^ self.
    ].

    "/ serve a TCP port
    Object infoPrinting ifTrue:[
        self class showHostsOn:Stderr.
    ].

    thisSocket := Socket newTCPserverAtPort:myPortNumber.

    thisSocket isNil ifTrue: [
        'WindowMigrationServer [warning]: socket creation failed (port:' errorPrint.
        myPortNumber errorPrint. ')' errorPrintCR.
        ^ nil
    ].

    socket := thisSocket.
    thisSocket listenFor: 5.

    [
        |stayInLoop|

        stayInLoop := true.
        [socket == thisSocket and:[stayInLoop]] whileTrue:[
            "/ don't move newskt out of the block,
            "/ it is local for each thread!!
            |newskt clientProcess|

            newskt := socket accept.
            newskt notNil ifTrue: [
                "/
                "/ 'got a connection ...' printNL.
                "/
                [
                    (self checkConnectionAllowed:newskt) ifTrue:[
                        AbortOperationRequest handle:[:ex |
                        ] do:[
                            self handleIncomingRequestFrom:newskt.
                        ].
                    ].
                ] ensure:[
                    newskt close
                ].
            ] ifFalse:[
                stayInLoop := false.
                'WindowMigrationServer [warning]: socket accept failed' errorPrintCR.
            ].
        ].
    ] ensure:[
        thisSocket notNil ifTrue: [thisSocket shutDown].
        self release.
    ].
    ^ nil

    "Modified: / 15.7.1997 / 12:20:04 / stefan"
    "Modified: / 4.7.1999 / 19:38:28 / cg"
!

performAuthenticationOn:aSocket
    |servicePassword password|

    (servicePassword := self class password) isNil ifTrue:[
        "/ allow always
        ^ true
    ].

    aSocket nextPutAll:'password:'.
    aSocket flush.
    password := aSocket nextLine.
    ^ password = servicePassword
! !

!AbstractSTXServer methodsFor:'initialize-release'!

release
    (socket notNil or:[connectProcess notNil]) ifTrue:[
        (self class name,' [info]: shutting down.') infoPrintCR.

        socket notNil ifTrue: [
            socket shutDown "close". 
            socket := nil
        ].
        connectProcess notNil ifTrue:[
            connectProcess terminateGroup.
            connectProcess := nil
        ].
    ].
    super release.

    "Modified: / 20.5.1998 / 18:10:35 / cg"
! !

!AbstractSTXServer methodsFor:'private'!

portNumber
    "Return my port number."
    "Default is subclassResponsibility."

    myPortNumber isNil ifTrue: [^ self class defaultPortNumber].
    ^ myPortNumber

    "Modified: / 15.7.1998 / 16:36:01 / cg"
! !

!AbstractSTXServer methodsFor:'queries'!

connectProcess
    "return the connectProcess"

    ^ connectProcess
!

isActive
    "return true, if I am active"

    ^ connectProcess notNil
! !

!AbstractSTXServer methodsFor:'restart-update'!

restart
    "Called when snapshot is started, and at instance creation."

    connectProcess := [
                    |removeMe|

                    self class hasServerRunning ifFalse:[
                        self class setServerRunning:self.
                        removeMe := true.
                    ].
                    self loopForever.
                    removeMe == true ifTrue:[
                        self class setServerRunning:nil.
                    ].
                  ] fork.

    connectProcess name:(self class name),' (port: ' , self portNumber printString , ')'.
    (self class name,' [info]: started.') infoPrintCR.
!

startServer
    "This is only run by SERVER instances at startup."

    ObjectMemory addDependent: self.
    self restart.
    ^self

    "
     WindowMigrationServer stop.
     WindowMigrationServer start.
    "

    "Modified: 9.1.1997 / 15:22:53 / cg"
!

startServerAt: aPortNumber
    "This is only run by SERVER instances at startup."

    myPortNumber := aPortNumber.
    ^ self startServer
!

update:aSymbol
    "Dependents of ObjectMemory are sent update:
     #returnFromSnapshot when a snapshot is started."

    aSymbol == #returnFromSnapshot ifTrue: [self restart].
    super update: aSymbol.
    ^self
! !

!AbstractSTXServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/simpleServices/AbstractSTXServer.st,v 1.12 2011/12/06 00:50:07 cg Exp $'
! !
