"
 COPYRIGHT (c) 2008 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/simpleServices' }"

AbstractSTXServer subclass:#STXScriptingServer
	instanceVariableNames:'scriptEnvironment parser promptHolder returnValueDisplayHolder
		showBacktraceHolder scriptingLanguage applicationTitle'
	classVariableNames:'ApplicationTitle'
	poolDictionaries:''
	category:'Net-Communication-Smalltalk'
!

AbortOperationRequest subclass:#EndOfInputRequest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:STXScriptingServer
!

PositionableStream subclass:#LineBufferedExternalStream
	instanceVariableNames:'currentLineStream lineQueue underlyingReadStream lineNumber
		forcedEOF previousLineWasEmpty'
	classVariableNames:''
	poolDictionaries:''
	privateIn:STXScriptingServer
!

Stream subclass:#TelnetStream
	instanceVariableNames:'controller underlyingStream'
	classVariableNames:''
	poolDictionaries:''
	privateIn:STXScriptingServer
!

!STXScriptingServer class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2008 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Allows for smalltalk-script to be added to an application.
    This makes them scriptable like QTScript.

    How to use:
    Smalltalk side:

        STXScriptingServer initialize
                        To reset the authorized/unauthorized host sets.

        STXScriptingServer allowHost:'aHostname'
                        To add a host to the list of allowed hosts.
                        If a request comes in from an unknown host, a Dialog
                        pops up asking if this host is allowed. If it is,
                        future requests are OK without further Dialogs to pop up.

        STXScriptingServer denyHost:'aHostname'
                        To add a host to the list of denied hosts.
                        No requests are allowed from this host.        

        STXScriptingServer start 
                        To start a server on the default port (8008).

        STXScriptingServer startAt:8008
                        To start a server on port.

        STXScriptingServer stop              
                        To stop the servers.


    Talk to the scripting service via the telnet command (on the shell level):
        telnet <hostName> 8008


    [author:]
        Claus Gittinger   


    [start with:]
        STXScriptingServer allowHost:'localhost'
        STXScriptingServer denyHost:'untrustedHost'
        STXScriptingServer start
        STXScriptingServer stop              

    [see also:]
        Socket
"
!

examples
"
    STXScriptingServer new
        initializeParser;
        showBacktrace:true;
        returnValueDisplay:true;
        handleScriptCommandsReadingFrom:'1+2' readStream 
        writingTo:Transcript

    STXScriptingServer new
        initializeParser;
        showBacktrace:true;
        returnValueDisplay:true;
        handleScriptCommandsReadingFrom:'
function f(n) { return n+1; }
' readStream 
        writingTo:Transcript

    STXScriptingServer new
        initializeParser;
        showBacktrace:true;
        returnValueDisplay:true;
        handleScriptCommandsReadingFrom:'
function f(n) { return n+1; }
function g(n) { return n-1; }
' readStream 
        writingTo:Transcript


    STXScriptingServer new
        initializeParser;
        showBacktrace:true;
        returnValueDisplay:true;
        handleScriptCommandsReadingFrom:'
function f(n) { return n+1; }
f(10)

' readStream 
        writingTo:Transcript
"
! !

!STXScriptingServer class methodsFor:'defaults'!

applicationTitle
    ^ ApplicationTitle ? 'ST/X'

    "Created: / 04-08-2010 / 11:18:51 / cg"
!

applicationTitle:aString
    ApplicationTitle := aString

    "Created: / 05-12-2011 / 16:49:23 / cg"
!

defaultPortNumber
    "the port we are listening on (by default).
     However, a server may also be started on another port"

    ^ 8008
! !

!STXScriptingServer methodsFor:'accessing'!

applicationTitle
    ^ applicationTitle ? self class applicationTitle

    "Created: / 05-12-2011 / 20:42:34 / cg"
!

applicationTitle:something
    applicationTitle := something.
!

inStream:aStream
    inStream := aStream.

    "Created: / 05-12-2011 / 16:37:16 / cg"
!

outStream:aStream
    outStream := aStream.

    "Created: / 05-12-2011 / 16:43:24 / cg"
!

prompt:aBoolean
    promptHolder value:aBoolean

    "Created: / 05-12-2011 / 16:35:45 / cg"
!

register:anObject as:name
    "register an object by name - typically done by the application during startup, to publish itself.
     Notice, that the environment of the server is NOT the environment of a session;
     instead, it will be the outer-environment of every session.
     Thus, by changing one of the servers variables or adding a binding, all current and future
     sessions will see this binding (unless they have a local binding, because the inner binding is
     seen first)"

    |nameKey var locals|

    nameKey := name asSymbol.

    locals := self scriptEnvironment _localVariables.
    locals isNil ifTrue:[
        self scriptEnvironment _localVariables:(locals := IdentityDictionary new).
    ].

    var := locals 
        at:nameKey
        ifAbsentPut:[ JavaScriptVariable new name:nameKey ].

    var value:anObject.

    "
     STXScriptingServer server 
        register:'Transcript' as:'app'.
    "
!

returnValueDisplay:aBoolean
    returnValueDisplayHolder value:aBoolean

    "Created: / 05-12-2011 / 16:35:21 / cg"
!

scriptEnvironment
    scriptEnvironment isNil ifTrue:[
        scriptEnvironment := STXScriptingEnvironment new.
    ].
    ^ scriptEnvironment
!

showBacktrace:aBoolean
    showBacktraceHolder value:aBoolean

    "Created: / 06-12-2011 / 00:01:17 / cg"
! !

!STXScriptingServer methodsFor:'connection commands'!

booleanCommand:line for:booleanHolder name:name
    (line startsWith:'+') ifTrue:[
        booleanHolder value:true.
    ].
    (line startsWith:'-') ifTrue:[
        booleanHolder value:false.
    ].
    outStream nextPutLine:name, ' flag is ',(booleanHolder value ifTrue:'on' ifFalse:'off')
!

connectionCommand:line
    "process lines starting with a tilde"

    (line startsWith:'~.') ifTrue:[
        self finishConnection.
        ^ self.
    ].
    self connectionCommandWithoutPrompt:line.
    self showPrompt.

    "Modified: / 05-12-2011 / 17:29:02 / cg"
!

connectionCommandWithoutPrompt:line
    "process lines starting with a tilde"

    |arg lang|

    (line startsWith:'~?') ifTrue:[
        self usage.
        ^ self.
    ].

    (line startsWith:'~~') ifTrue:[
        parser sourceStream forceEOF.
        ^ self.
    ].
    (line startsWith:'~h') ifTrue:[
        self halt.
        ^ self.
    ].

    (line startsWith:'~e') ifTrue:[
        self dumpEnvironment.
        ^ self.
    ].

    (line startsWith:'~p') ifTrue:[
        self booleanCommand:(line copyFrom:3) for:promptHolder name:'prompt'.
        ^ self.
    ].
    (line startsWith:'~v') ifTrue:[
        self booleanCommand:(line copyFrom:3) for:returnValueDisplayHolder name:'show returnValue'.
        ^ self.
    ].
    (line startsWith:'~b') ifTrue:[
        self booleanCommand:(line copyFrom:3) for:showBacktraceHolder name:'show error backtrace'.
        ^ self.
    ].

    (line startsWith:'~l') ifTrue:[
        arg := (line copyFrom:3) withoutSeparators.
        arg notEmpty ifTrue:[
            lang := ProgrammingLanguage named:arg ifNone:nil.
            lang isNil ifTrue:[
                outStream nextPutLine:'Unsupported language.'.
            ] ifFalse:[
                scriptingLanguage := lang.
                outStream nextPutLine:'Language changed to: ',scriptingLanguage name.
                self initializeParser.
                AbortSignal raise.
                ^ self.
            ].
        ] ifFalse:[
            outStream nextPutLine:'Current language is: ',scriptingLanguage name.
        ].
        outStream nextPutLine:'Available scripting languages:'.
        ProgrammingLanguage allDo:[:lang |
            outStream nextPutLine:'  ',lang name.
        ].
        ^ self.
    ].

    outStream nextPutLine:'Unrecognized control-command.'.

    "Created: / 05-12-2011 / 17:28:42 / cg"
!

dumpEnvironment
    |env locals|

    outStream nextPutLine:'Session Environment:'.
    env := scriptEnvironment.
    [env notNil] whileTrue:[
        locals := env _localVariables.
        locals notEmptyOrNil ifTrue:[
            locals keys asSortedCollection do:[:nm |
                |var|

                var := locals at:nm.
                outStream nextPutAll:'  '.
                outStream nextPutAll:nm asString.
                outStream nextPutAll:' -> '.
                outStream nextPutLine:(var value printString).
            ].
        ].
        env := env _outerEnvironment.
        env notNil ifTrue:[
            outStream nextPutLine:'Main Environment:'.
        ].
    ].
!

usage
    outStream nextPutLine:'Connection commands:'.
    outStream nextPutLine:'    ~?          this text'.
    outStream nextPutLine:'    ~.          finish and close connection'.
    outStream nextPutLine:'    ~~          force eval (EOF)'.
    outStream nextPutLine:'    ~p[+|-]     prompt (on/off)'.
    outStream nextPutLine:'    ~v[+|-]     return value display (on/off)'.
    outStream nextPutLine:'    ~b[+|-]     error backtrace (on/off)'.
    outStream nextPutLine:'    ~e          dump environemnt variables'.
    outStream nextPutLine:'    ~l [<lang>] change the scripting language (without <lang> for list)'.
    outStream nextPutLine:''.
    outStream nextPutLine:'Anything else is interpreted as ',scriptingLanguage name,'-expression.'.

    "/ sigh - ought to ask the language for the delimiter..
    scriptingLanguage isSTXJavaScript ifTrue:[
        outStream nextPutLine:'Expressions must be delimited by a semicolon (;).'.
    ] ifFalse:[
        scriptingLanguage isSmalltalk ifTrue:[
            outStream nextPutLine:'Expressions must be delimited by a period (.).'.
        ] ifFalse:[
            outStream nextPutLine:'Expressions must be delimited by a statement terminator.'.
        ]
    ].
    outStream nextPutLine:'(eval enforced when 2 empty lines are encountered in a row).'.

    "Modified: / 05-12-2011 / 18:21:52 / cg"
! !

!STXScriptingServer methodsFor:'handle client'!

evaluateScriptCommand
    |value input|

    inStream atEnd ifTrue:[AbortSignal raise].

    "/ self initializeParser.
    inStream isExternalStream ifTrue:[
        input := LineBufferedExternalStream on:inStream
    ] ifFalse:[
        input := LineNumberReadStream on:inStream
    ].
    parser source:input.

    (AbortSignal, Error) handle:[:ex |
"/ Parser parseErrorSignal handle:[:ex |
        ex signal = EndOfInputRequest ifTrue:[
            outStream nextPutLine:'finish.'.
            AbortSignal raise.
        ].

        ex signal = AbortSignal ifTrue:[
            outStream nextPutLine:'aborted'.
        ] ifFalse:[
            outStream nextPutAll:('Error cought in line %1: ' bindWith:(input lineNumber)).
            outStream nextPutLine:(ex description).
            self class isErrorDebugging == true ifTrue:[
                ex reject.
            ].
            self class isErrorLogging == true ifTrue:[
                "/ ex suspendedContext fullPrintAllOn:outStream.
            ].

            false "(Parser parseErrorSignal accepts:ex signal)" ifFalse:[
                showBacktraceHolder value == true ifTrue:[
                    ex suspendedContext fullPrintAllOn:outStream.
                ].
            ].
        ].
        self initializeParser.
        AbortSignal raise.
    ] do:[
        value := parser evaluateDeclarationFor:self scriptEnvironment.
        returnValueDisplayHolder value ifTrue:[
            outStream nextPutLine:(value printString).
        ].
    ].

    "
     STXScriptingServer start
    "

    "Created: / 04-08-2010 / 11:48:03 / cg"
!

finishConnection
    EndOfInputRequest raise.

    "Modified: / 05-12-2011 / 17:45:28 / cg"
!

handleIncomingRequestFrom:aSocket
    aSocket lineEndCRLF.

    [
        |input|

        input := TelnetStream on:aSocket.
        input controller:self.

        self copy 
            setupDefaultsFrom:self;
            handleScriptCommandsReadingFrom:input writingTo:aSocket.
    ] ensure:[
        aSocket close.
    ].

    "Modified: / 06-12-2011 / 01:59:43 / cg"
!

handleScriptCommandsReadingFrom:input writingTo:output
    inStream := input.
    outStream := output.
    scriptEnvironment _outStream:outStream.

    outStream nextPutLine:'Welcome to ',self applicationTitle,' (Type ~? for help)'.

    self register:(self scriptEnvironment) as:#'__environment__'.
    self initializeParser.

    [input atEnd] whileFalse:[   
        self showPrompt.
        self evaluateScriptCommand
    ].

    "Modified (format): / 06-12-2011 / 00:56:54 / cg"
!

initializeParser
    parser := scriptingLanguage parserClass new.
    parser interactiveMode:true.

    self register:parser as:#'__interpreter__'.
    self register:socket as:#'__connection__'.
    self register:inStream as:#'__stdin__'.
    self register:outStream as:#'__stdout__'.
    self register:outStream as:#'__stderr__'.

    "Created: / 05-12-2011 / 16:46:24 / cg"
!

showPrompt
    promptHolder value ifTrue:[
        outStream nextPutAll:'> '.
    ].

    "Created: / 04-08-2010 / 11:33:10 / cg"
! !

!STXScriptingServer methodsFor:'initialize-release'!

initialize
    self setupDefaultsFrom:nil.

    "Created: / 05-12-2011 / 23:50:06 / cg"
!

setupDefaultsFrom:masterServerOrNil
    showBacktraceHolder := false asValue.
    promptHolder := true asValue.
    returnValueDisplayHolder := false asValue.

    scriptingLanguage := ProgrammingLanguage named: 'JavaScript'.
    scriptingLanguage isNil ifTrue:[
        scriptingLanguage := ProgrammingLanguage named: 'Smalltalk'.
    ].

    scriptEnvironment := STXScriptingEnvironment new.
    masterServerOrNil notNil ifTrue:[
        scriptEnvironment _outerEnvironment:(masterServerOrNil scriptEnvironment).
    ]

    "Modified (format): / 05-12-2011 / 16:31:11 / cg"
! !

!STXScriptingServer::LineBufferedExternalStream class methodsFor:'instance creation'!

on:aStream
    ^ self basicNew on:aStream
! !

!STXScriptingServer::LineBufferedExternalStream methodsFor:'accessing'!

forceEOF
    forcedEOF := true.

    "Created: / 05-12-2011 / 17:42:26 / cg"
!

lineNumber
    ^ lineNumber
! !

!STXScriptingServer::LineBufferedExternalStream methodsFor:'initialization'!

on:aStream
    readLimit := SmallInteger maxVal.
    position := ZeroPosition.
    underlyingReadStream := aStream.
    currentLineStream := nil.
    lineQueue := Queue new.
    lineNumber := 1.
    previousLineWasEmpty := false.

    "Modified: / 05-12-2011 / 18:10:07 / cg"
! !

!STXScriptingServer::LineBufferedExternalStream methodsFor:'misc'!

readStream
    ^ self
! !

!STXScriptingServer::LineBufferedExternalStream methodsFor:'reading'!

atEnd
    forcedEOF == true ifTrue:[^ true].

    currentLineStream notNil ifTrue:[
        currentLineStream atEnd ifFalse:[
            ^ false
        ].
        currentLineStream := nil.
        ^ false
    ].
    lineQueue notEmpty ifTrue:[^ false].

    ^ underlyingReadStream atEnd

    "Modified: / 05-12-2011 / 17:41:49 / cg"
!

fillLineBuffer
    |line lineIsEmpty|

    lineQueue isEmpty ifTrue:[
        line := underlyingReadStream nextLine.
        line isNil ifTrue:[ forcedEOF := true. ^ self ].
        lineQueue addLast:line.
    ].

    line := lineQueue removeFirst.
    (lineIsEmpty := line isEmpty) ifTrue:[
        forcedEOF := previousLineWasEmpty.
    ].
    previousLineWasEmpty := lineIsEmpty.
    currentLineStream := ReadStream on:line.

    "Created: / 05-12-2011 / 18:14:29 / cg"
!

next
    [
        forcedEOF == true ifTrue:[^ nil].

        currentLineStream notNil ifTrue:[
            currentLineStream atEnd ifFalse:[
                ^ currentLineStream next
            ].
            currentLineStream := nil.
            lineNumber := lineNumber + 1.
            ^ Character cr.
        ].
        self fillLineBuffer.
    ] loop.

    "Modified: / 05-12-2011 / 18:15:34 / cg"
!

peek
    [
        forcedEOF == true ifTrue:[^ nil].

        currentLineStream notNil ifTrue:[
            currentLineStream atEnd ifFalse:[
                ^ currentLineStream peek
            ].
            ^ Character cr.
        ].
        self fillLineBuffer.
    ] loop

    "Modified: / 05-12-2011 / 18:15:22 / cg"
! !

!STXScriptingServer::TelnetStream class methodsFor:'instance creation'!

on:aStream
    ^ self basicNew on:aStream
! !

!STXScriptingServer::TelnetStream methodsFor:'accessing'!

controller:something
    controller := something.
!

readStream
    ^ self

    "Created: / 06-12-2011 / 02:09:10 / cg"
! !

!STXScriptingServer::TelnetStream methodsFor:'initialization'!

on:aStream
    underlyingStream := aStream
! !

!STXScriptingServer::TelnetStream methodsFor:'queries'!

isExternalStream
    ^ true

    "Created: / 06-12-2011 / 02:14:59 / cg"
! !

!STXScriptingServer::TelnetStream methodsFor:'reading'!

atEnd
    ^ underlyingStream isOpen not
    or:[ underlyingStream isExternalStream not
         and:[ underlyingStream atEnd ] ]

    "Modified: / 06-12-2011 / 00:25:30 / cg"
!

nextLine
    |line si di ch|

    line := underlyingStream nextLine.
    line notNil ifTrue:[
        (line startsWith:'~') ifTrue:[
            controller connectionCommand:line.
            ^ self nextLine.
        ].
    ].

    "/ handle backspace manually here...
    si := di := 1.
    [si <= line size] whileTrue:[
        ch := line at:si.
        si := si + 1.
        ch == Character backspace ifTrue:[
            di := (di - 1) max:1.
        ] ifFalse:[
            line at:di put:ch.
            di := di + 1.
        ].
    ].        
    
    ^ line
! !

!STXScriptingServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/simpleServices/STXScriptingServer.st,v 1.19 2011/12/06 01:15:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/simpleServices/STXScriptingServer.st,v 1.19 2011/12/06 01:15:27 cg Exp $'
! !
