"
 COPYRIGHT (c) 2002 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/simpleServices' }"

AbstractSTXServer subclass:#WindowMigrationServer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-Smalltalk'
!

!WindowMigrationServer class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2002 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Allows for smalltalk windows to be pulled to another display.
    Waits on a socket for incoming connetions. When a connection arrives,
    all of my windows are moved to the x-server running on the connecting machine.

    Thus, if you forgot your stx-window on the machine at your desk, and you need a window
    in the meeting room, just pull it !!

    WARNING: an experimental (unprotected) version; does not perform any
             authentication and is pretty unsecure (someone can pull away your
             smalltalk session from your screen ...)


    How to use:
    On the Smalltalk side, setup a server:

        WindowMigrationServer initialize
                        To reset the authorized/unauthorized host sets.

        WindowMigrationServer allowHost:'aHostname'
                        To add a host to the list of allowed hosts.
                        If a request comes in from an unknown host, a Dialog
                        pops up asking if this host is allowed. If it is,
                        future requests are OK without further Dialogs to pop up.

        WindowMigrationServer denyHost:'aHostname'
                        To add a host to the list of denied hosts.
                        No requests are allowed from this host.        

        WindowMigrationServer start 
                        To start a server on the default port (8005).

        WindowMigrationServer startAt:8005
                        To start a server on port.

        WindowMigrationServer stop              
                        To stop the servers.


    To pull the windows to another X-display, on the machine which has the X-server running,
    enter the Unix command (on the shell level):
        telnet <hostName> 8005



    [author:]
        Claus Gittinger   


    [start with:]
        WindowMigrationServer allowHost:'localhost'
        WindowMigrationServer denyHost:'untrustedHost'
        WindowMigrationServer start
        WindowMigrationServer stop              

    [see also:]
        Socket
"
! !

!WindowMigrationServer class methodsFor:'defaults'!

defaultPortNumber
    "the port we are listening on (by default).
     However, a server may also be started on another port"

    ^ 8005
! !

!WindowMigrationServer methodsFor:'handle client'!

handleIncomingRequestFrom:aSocket
    |targetHost anotherDisplay targetSocketAddress|

    targetSocketAddress := aSocket getPeer.
    targetHost := targetSocketAddress hostNameOrAddressString.
    ((targetHost = OperatingSystem getHostName)
    or:[targetHost = (OperatingSystem getHostName , '.' , OperatingSystem getDomainName)]) ifTrue:[
        anotherDisplay := Screen default
    ] ifFalse:[
        "/ try to connect ...
        [
            anotherDisplay := XWorkstation newDispatchingFor:(targetHost , ':0').
        ] on:Screen deviceOpenErrorSignal do:[:ex|
            aSocket nextPutLine:'ST/X: Could not connect to your display (' , targetHost , ':0)'.
            aSocket close.
            ^ self
        ]
    ].

    aSocket nextPutLine:'ST/X: here you are ...'.
    aSocket close.

    Transcript topView windowGroup migrateTo:anotherDisplay.
! !

!WindowMigrationServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/simpleServices/WindowMigrationServer.st,v 1.13 2008/11/20 17:10:24 cg Exp $'
! !
