"{ Package: 'stx:goodies/smaCC/runtime' }"

"{ NameSpace: SmaCC }"

PeekableStream subclass:#SmaCCLineNumberStream
	instanceVariableNames:'sourceStream previousWasCR eolPositions lastPosition'
	classVariableNames:'CR LF'
	poolDictionaries:''
	category:'SmaCC-Runtime'
!

SmaCCLineNumberStream comment:'SmaCCLineNumberStream is a wrapper for streams that calculates line numbers.

Instance Variables:
        eolPositions    <OrderedCollection of: Integer>     the positions of each end of line
        lastPosition    <Integer>                           the position of the last character that we have calculated the end of line 
                                                            information for (we know the line number for all characters before this position 
                                                            and don''''t know anything about the characters after this position)
        previousWasCR   <Boolean>                           was the previous character a CR. This is used for CR LF streams. 
                                                            A CR LF combination should only increment the line counter by 1
        sourceStream    <Stream>                            the stream that we are wrapping
'
!


!SmaCCLineNumberStream class methodsFor:'initialization'!

initialize
    CR := Character cr.
    Smalltalk isSmalltalkX ifTrue:[
        CR := Character return.
    ].
    LF := Character nl.

    "
     self initialize
    "
! !

!SmaCCLineNumberStream class methodsFor:'instance creation'!

on: aReadStream 
	^(self basicNew)
		on: aReadStream;
		yourself
! !

!SmaCCLineNumberStream methodsFor:'accessing'!

columnNumber
	^self columnNumberFor: self position
!

columnNumberFor: anInteger 
	^anInteger - (eolPositions at: (self lineNumberFor: anInteger)) + 1
!

contents
	^sourceStream contents
!

flush
	^sourceStream flush
!

lineNumber
	^self lineNumberFor: self position
!

lineNumberFor: anInteger
	| index start stop pos |
	pos := anInteger.
	pos >= eolPositions last ifTrue: [^eolPositions size].
	start := 1.
	stop := eolPositions size.
	[start + 1 < stop] whileTrue: 
			[index := (start + stop) // 2.
			(eolPositions at: index) <= pos 
				ifTrue: [start := index]
				ifFalse: [stop := index]].
	^start
!

next
        | character streamPosition cr|

        cr := Smalltalk isSmalltalkX ifTrue:[Character return] ifFalse:[Character cr].

        character := sourceStream next.
        streamPosition := sourceStream position.
        streamPosition - 1 >= lastPosition 
                ifTrue: 
                        [lastPosition := streamPosition.
                        character == cr 
                                ifTrue: 
                                        [eolPositions add: lastPosition.
                                        previousWasCR := true]
                                ifFalse: 
                                        [character == Character lf 
                                                ifTrue: 
                                                        [previousWasCR 
                                                                ifTrue: [eolPositions at: eolPositions size put: lastPosition]
                                                                ifFalse: [eolPositions add: lastPosition]].
                                        previousWasCR := false]].
        ^character

    "Modified: / 14-07-2006 / 11:07:37 / cg"
!

nextPut: anObject 
	^sourceStream nextPut: anObject
!

peek
	^sourceStream peek
!

position
	^sourceStream position
!

position: anInteger 
	anInteger > lastPosition 
		ifTrue: 
			[sourceStream position: lastPosition.
			[sourceStream position < anInteger and: [sourceStream atEnd not]] 
				whileTrue: [self next]]
		ifFalse: [sourceStream position: anInteger]
!

skip: anInteger
	^self position: self position + anInteger
! !

!SmaCCLineNumberStream methodsFor:'error handling'!

doesNotUnderstand: aMessage 
	^sourceStream perform: aMessage selector withArguments: aMessage arguments
! !

!SmaCCLineNumberStream methodsFor:'initialize-release'!

on: aReadStream 
	sourceStream := aReadStream.
	eolPositions := OrderedCollection with: aReadStream position.
	lastPosition := aReadStream position.
	previousWasCR := false
! !

!SmaCCLineNumberStream methodsFor:'testing'!

atEnd
	^sourceStream atEnd
!

isReadable
	^sourceStream isReadable
!

isWritable
	^sourceStream isWritable
! !

!SmaCCLineNumberStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/smaCC/runtime/SmaCCLineNumberStream.st,v 1.5 2006/07/14 09:59:33 cg Exp $'
! !

SmaCCLineNumberStream initialize!
