"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapLiteServer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-ExampleClients'
!

!SoapLiteServer class methodsFor:'documentation'!

documentation
"
An abstract class.
Derive this class to get a SOAP Server.
Override the 'defineServiceHandlersIn:soapServer secure:secure' method (subclassResponsibility).

"
! !

!SoapLiteServer class methodsFor:'startup'!

defineService:serviceName withImplementor:inst selector:selector in:soapServer secure:secure
    |service|

    secure ifTrue:[
        service := SOAP::SoapSignedService implementor:inst selector:selector.
        service signatureHandler:(SOAP::SignatureHandler new signer:(SOAP::TestSigner new)).
    ] ifFalse:[
        service := SOAP::SoapService implementor:inst selector:selector.
    ].
    service signature: (SOAP::SoapServiceSignature name:serviceName).

    soapServer add:service.
    ^ service
!

defineServiceHandlersIn:soapServer secure:secure

    self subclassResponsibility.



    "
    Override in Subclass
    i.e. :

    |inst service|
    inst := self new.

    service := self defineService:'hello' withImplementor:inst selector:#helloWorld in:soapServer secure:secure.

    service := self defineService:'helloWithArg' withImplementor:inst selector:#helloWorldWithArgument: in:soapServer secure:secure.
    service signature:(SoapServiceSignature name:'helloWithArg' paramNames:#( aString )).
    "
!

soapEnabledHTTPServerOnPort:portNr
    "get an HTTPServer with SOAP enabled"

    |server|

    server := HTTPServer soapEnabledServerOnPort:portNr.
    server isNil ifTrue:[
        self error:'Cannot start HTTP server on port: ' , portNr printString
    ].
    ^ server
!

startOnPort:aPortNr secure:secure
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    |server|

    server := self soapEnabledHTTPServerOnPort:(aPortNr ? SOAP::SoapHttpSetup defaultPort).
    self defineServiceHandlersIn:(server soapServiceHandler) secure:secure.
    ^ server.
! !

!SoapLiteServer class methodsFor:'startup-public'!

start
    "startup the service"

    ^ self startOnPort:nil secure:false

    "
     self start
    "
!

startOnPort:portNr
    "startup the service"

    ^ self startOnPort:portNr secure:false

    "
     self startOnPort:9000
    "
!

startSecure
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    ^ self startOnPort:nil secure:true

    "
     self startSecure
    "
!

startSecureOnPort:aPortNr
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    ^ self startOnPort:aPortNr secure:true

    "
     self startSecureOnPort:9000
    "
! !

!SoapLiteServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SOAP__SoapLiteServer.st,v 1.2 2006/07/25 07:11:04 stefan Exp $'
! !
