"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoXmlAttributesHolder
	instanceVariableNames:'namespaceDict otherDict'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-SoXML-Base'
!

SoXmlAttributesHolder comment:'An utitily class for XML attribute handling.
	
---	
mu 5/3/2001 19:22'
!


!SoXmlAttributesHolder class methodsFor:'instance creation'!

new
        ^self basicNew initialize
! !

!SoXmlAttributesHolder methodsFor:'accessing'!

namespaceDict
	^namespaceDict
!

otherDict
	^otherDict
! !

!SoXmlAttributesHolder methodsFor:'actions'!

asDictionary
	| dict |
	dict := Dictionary new.
	self namespaceDict
		keysAndValuesDo: [:k :v | dict at: k put: v].
	self otherDict
		keysAndValuesDo: [:k :v | dict at: k put: v].
	^ dict
!

declareNamespace: prefix uri: uri
	self namespaceDict at: SoXmlConstants xmlnsPrefixColon, prefix put: uri
!

declaredNamespacePrefixes
	^ self namespaceDict keys
		collect: [:each | each copyFrom: SoXmlConstants xmlnsPrefixColon size + 1 to: each size]
!

declaredNamespaceURIFor: prefix
	^self declaredNamespaceURIFor: prefix ifAbsent: []
!

declaredNamespaceURIFor: prefix ifAbsent: aBlock
	^self namespaceDict at: (SoXmlConstants xmlnsPrefixColon, prefix) ifAbsent: aBlock
!

getAttributeLike: aNameString 
	| key |
	key := (self otherDict keys detect: [:each | aNameString match: each] ifNone:[] ).
	key isNil ifTrue: [key := self namespaceDict keys detect: [:each | aNameString match: each] ifNone:[]].
	^self getAttributeNamed: key
!

getAttributeNamed: aNameString 
	^ self otherDict
		at: aNameString
		ifAbsent: [self namespaceDict
				at: aNameString
				ifAbsent: []]
!

hasAttributeLike: aNameString 
        ^ ((self otherDict keys contains: [:each | aNameString match: each]))
           or: [(self namespaceDict keys contains: [:each | aNameString match: each])]
!

hasAttributeNamed: aNameString 
	^ (self otherDict includesKey: aNameString)
		or: [self namespaceDict includesKey: aNameString]
!

putAttribute: aNameValueArray
	self putAttributeNamed: aNameValueArray first asString value: aNameValueArray last asString
!

putAttributeNamed: attribName value: attribValue 
	| xmlns |
	xmlns := SoXmlConstants xmlnsPrefixColon.
	(attribName size > xmlns size
			and: [(attribName copyFrom: 1 to: xmlns size)
					= xmlns])
		ifTrue: [self namespaceDict at: attribName put: attribValue]
		ifFalse: [self otherDict at: attribName put: attribValue]
!

removeAttributeNamed: aNameString
	^self otherDict removeKey: aNameString
!

trimAttributePrefixNamed: attribName
	| value newKey |
	value := self otherDict at: attribName ifAbsent:[].
	value isNil ifTrue: [^nil].
	newKey := (SoXmlUtil prefixAndLocalNameFrom: attribName) last. 
	self removeAttributeNamed: attribName.
	^ self putAttributeNamed: newKey value: value
		
!

undeclareNamespaces
	namespaceDict := Dictionary new
! !

!SoXmlAttributesHolder methodsFor:'initialization & release'!

initialize
        namespaceDict := Dictionary new.
        otherDict := Dictionary new.
! !

!SoXmlAttributesHolder methodsFor:'printing'!

printOn: aStream
	"super printOn: aStream." "for Debug"
	self printXmlOn: aStream
!

printXmlOn: aStream
        self printXmlOn: aStream indent:nil
!

printXmlOn: aStream indent:levelOrNil
        self namespaceDict keysAndValuesDo:[ :k :v |
"/                levelOrNil notNil ifTrue:[
"/                    aStream cr; spaces:levelOrNil.
"/                ] ifFalse:[
                    aStream nextPut: Character space.
"/                ].
                self writeName: k on: aStream.
                aStream nextPutAll: '="'.
                self writeAttributeValue: v asString on: aStream.
                aStream nextPut: $".
        ].
        self otherDict keysAndValuesDo:[ :k :v |
"/                levelOrNil notNil ifTrue:[
"/                    aStream cr; spaces:levelOrNil.
"/                ] ifFalse:[
                    aStream nextPut: Character space.
"/                ].
                self writeName: k on: aStream.
                aStream nextPutAll: '="'.
                self writeAttributeValue: v asString on: aStream.
                aStream nextPut: $".
        ]
! !

!SoXmlAttributesHolder methodsFor:'private'!

writeAttributeValue: aCharacterArray on: aStream 
        ^SoXmlUtil writeXmlAttributeText: aCharacterArray on: aStream
!

writeName: aCharacterArray on: aStream 
	"TODO: check illegal characters"
	^ aStream nextPutAll: aCharacterArray
!

writeValue: aCharacterArray on: aStream 
	^SoXmlUtil writeXmlText: aCharacterArray on: aStream
! !

!SoXmlAttributesHolder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoXmlAttributesHolder.st,v 1.9 2010/06/14 10:54:09 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoXmlAttributesHolder.st,v 1.9 2010/06/14 10:54:09 cg Exp $'
! !
