"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoXmlUtil
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-SoXML-Base'
!

SoXmlUtil class instanceVariableNames:'convertCharDict convertAttributeCharDict parserAdapter'

"
 No other class instance variables are inherited by this class.
"
!

SoXmlUtil comment:'An utility for XML processing.
It includes:
	-Parsing XML
	-Writing XML text

I assume you have VWXMLParser.
If you would like to use other XML parsers, you should subclass me.


---
mu 5/3/2001 19:23'
!


!SoXmlUtil class methodsFor:'accessing'!

convertAttributeCharDict
        ^convertAttributeCharDict
!

convertCharDict
	^convertCharDict
!

parserAdapter
	parserAdapter ifNil:[self initParserAdapter]. 
	^parserAdapter
!

parserAdapter: xmlParserAdapter
	parserAdapter := xmlParserAdapter
! !

!SoXmlUtil class methodsFor:'actions'!

asXmlText: aString 
	|  wStr |
	wStr := WriteStream
				on: (String new: aString size).
	^(self writeXmlText: aString on: wStr) contents
!

contentsWithXmlDeclaration: aString 
        | wStr |
        wStr := WriteStream on: (String new: 1024).
        wStr nextPutAll: '<?xml version="1.0" encoding="utf-8"?>'.
        2 timesRepeat: [wStr nextPut: Character return.
                        wStr nextPut: Character lf].
        wStr nextPutAll: aString.
        ^wStr contents.
!

localNameFrom: aString 
	^(self prefixAndLocalNameFrom: aString) last
!

prefixAndLocalNameFrom: aString 
	| index readStream prefixName localName |
	index := aString indexOf: $:.
	index = 0
		ifTrue: [prefixName := ''.
			localName := aString]
		ifFalse: [readStream := aString readStream.
			prefixName := readStream next: index - 1.
			readStream skip: 1.
			localName := readStream next: aString size - index].
	^ Array with: prefixName with: localName
!

writeXmlAttributeText: aString on: aWriteStream
        ^ self
            writeXmlText: aString on: aWriteStream convertUsing:self convertAttributeCharDict
!

writeXmlText: aString on: aWriteStream
        ^ self
            writeXmlText: aString on: aWriteStream convertUsing:self convertCharDict
!

writeXmlText:aString on:aWriteStream convertUsing:cDict 
    |wStr|

    wStr := aWriteStream.
    aString do:[:eachChar | 
        |val|

        val := eachChar codePoint.
        val >= 127 ifTrue:[
            wStr
                nextPutAll:'&#';
                nextPutAll:val asString;
                nextPut:$;
        ] ifFalse:[
            |conv|

            val <= 44 ifTrue:[
                wStr
                    nextPutAll:'&#';
                    nextPutAll:val asString;
                    nextPut:$;
            ] ifFalse:[
                (eachChar == $& 
                or:[ eachChar == $< 
                or:[ eachChar == $> 
                or:[ eachChar == $' 
                or:[ eachChar == $" ]]]])
                ifTrue:[
                    wStr
                        nextPutAll:'&#';
                        nextPutAll:val asString;
                        nextPut:$;
                ] ifFalse:[
                    wStr nextPut:eachChar
                ]
            ].

"/            conv := cDict at:val ifAbsent:[].
"/            conv isNil ifTrue:[
"/                wStr nextPut:eachChar
"/            ] ifFalse:[
"/                wStr nextPutAll:conv
"/            ]
        ]
    ].
    ^ wStr
! !

!SoXmlUtil class methodsFor:'actions-parse'!

attribDictFrom: aParsedXMLElement
	^self parserAdapter attribDictFrom: aParsedXMLElement
!

elementNameFrom: aParsedXMLElement
	^self parserAdapter elementNameFrom: aParsedXMLElement
	
!

elementShortNameFrom: aParsedXMLElement
	^self parserAdapter elementShortNameFrom: aParsedXMLElement
	
!

elementValueFrom: aParsedXMLElement
	^self parserAdapter elementValueFrom: aParsedXMLElement
!

elementsFrom: aParsedXMLElement
	^self parserAdapter elementsFrom: aParsedXMLElement
!

elementsWithoutBlankTextsFrom: aParsedXMLElement
	^self parserAdapter elementsWithoutBlankTextsFrom: aParsedXMLElement
!

elementsWithoutTextsFrom: aParsedXMLElement
	^self parserAdapter elementsWithoutTextsFrom: aParsedXMLElement
!

isTextElement: aParsedXMLElement
	^self parserAdapter isTextElement: aParsedXMLElement
!

parseXml: xmlString
	^self parserAdapter parseXml: xmlString
	
! !

!SoXmlUtil class methodsFor:'class initialization'!

initialize
        "SoXmlUtil initialize"
        self initConvertCharDict.
        self initConvertAttributeCharDict.
        parserAdapter := nil
! !

!SoXmlUtil class methodsFor:'private-initialization'!

initConvertAttributeCharDict
        convertAttributeCharDict := Dictionary new.
        0 to: 31 do: [:each | convertAttributeCharDict at: each put: '&#', each asString, ';'].
        convertAttributeCharDict
                at: $& codePoint put: '&amp;';
                at: $< codePoint put: '&lt;';
                at: $' codePoint put: '&apos;';
                at: $" codePoint put: '&quot;'.
        convertAttributeCharDict
                removeKey: Character tab codePoint;
                removeKey: Character cr codePoint;
                removeKey: Character return codePoint.
!

initConvertCharDict
        convertCharDict := Dictionary new.
        0 to: 47 do: [:each | convertCharDict at: each put: '&#', each asString, ';'].
        convertCharDict 
                at: $& codePoint put: '&amp;';
                at: $< codePoint put: '&lt;';
                at: $> codePoint put: '&gt;';
                at: $' codePoint put: '&apos;';
                at: $" codePoint put: '&quot;'.
        convertCharDict 
                removeKey: Character space codePoint;
                removeKey: Character tab codePoint;
                removeKey: Character cr codePoint;
                removeKey: Character return codePoint.
        convertCharDict 
                removeKey: $/ codePoint;
                removeKey: $- codePoint;
                removeKey: $. codePoint.
!

initParserAdapter
        "Currently YAXO and VWXML conscious"

    parserAdapter := SoVWXMLParserAdapter

"
        parserAdapter := self environment
                                at: #SoYaxXMLParserAdapter
                                ifAbsent: [self environment
                                                at: #SoVWXMLParserAdapter
                                                ifAbsent: [Transcript cr; show: '##failed to initParserAdapter##']]

"
! !

!SoXmlUtil class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoXmlUtil.st,v 1.9 2005/12/01 15:03:22 cg Exp $'
! !

SoXmlUtil initialize!
