"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoXmlWrapElement
	instanceVariableNames:'prefixName localName name value attributesHolder children parent
		isEmptyBody isPrintCDATA'
	classVariableNames:'IsPrintCDATA'
	poolDictionaries:''
	category:'SOAP-SoXML-Base'
!

SoXmlWrapElement comment:'Representing an XML element.
Using XmlUtil, I am independent of XML parser implementations.

--
mu 5/3/2001 19:28'
!


!SoXmlWrapElement class methodsFor:'instance creation'!

emptyNamed: anElemName
	^(self named: anElemName) beEmpty
!

named: anElemName
	^self new name: anElemName
!

named: anElemName withPrefix: prefix
	^self named: prefix, ':', anElemName
!

new
        ^self basicNew initialize
! !

!SoXmlWrapElement class methodsFor:'accessing'!

isPrintCDATA
	IsPrintCDATA isNil ifTrue:[ IsPrintCDATA := false ].
	^IsPrintCDATA
!

isPrintCDATA: aBoolean
	IsPrintCDATA := aBoolean
! !

!SoXmlWrapElement class methodsFor:'from xml'!

fromXml: xmlString
	^self fromXmlElement: (SoXmlUtil parseXml: xmlString).
!

fromXmlElement: aParsedXmlElement
	^self new initByDom: aParsedXmlElement
! !

!SoXmlWrapElement methodsFor:'accessing'!

attributesHolder
	attributesHolder isNil ifTrue:[ attributesHolder := SoXmlAttributesHolder new].
	^attributesHolder
!

attributesHolder: aValue

	attributesHolder := aValue
!

children
	children isNil ifTrue:[ children := OrderedCollection new].
	^children
!

children: aValue
	
	children := aValue
!

isEmptyBody

	^isEmptyBody
!

isEmptyBody: aValue

	isEmptyBody := aValue
!

isPrintCDATA
	isPrintCDATA isNil ifTrue:[ isPrintCDATA := self class isPrintCDATA].
	^isPrintCDATA
!

isPrintCDATA: aValue

	isPrintCDATA := aValue
!

localName

	^localName
!

localName: aValue

	localName := aValue
!

name

	^name
!

name: aValue

	name := aValue
!

parent

	^parent
!

parent: aValue

	parent := aValue
!

prefixName

	^prefixName
!

prefixName: aValue

	prefixName := aValue
!

value

	^value
!

value: aValue 
	aValue isNil
		ifTrue: [^ self].
	value := aValue asString
! !

!SoXmlWrapElement methodsFor:'actions'!

addChild: anXMLEleemnt
	anXMLEleemnt isNil ifTrue:[^nil].
	anXMLEleemnt parent: self.
	^self children add: anXMLEleemnt.
	
!

allDeclaredNamespacePrefixes
	^ self attributesHolder declaredNamespacePrefixes isEmpty
		ifTrue: [self parent notNil
				ifTrue: [self parent declaredNamespacePrefixes]]
!

allDeclaredNamespaceURIFor: prefix 
	^ self attributesHolder
		declaredNamespaceURIFor: prefix
		ifAbsent: [self parent notNil
				ifTrue: [self parent declaredNamespaceURIFor: prefix]]
!

attributesDictionary
	^self attributesHolder asDictionary
!

beEmpty
	self isEmptyBody: true
	
!

beShortName
	self name: self shortName
!

createChildNamed: anElemName
	| child |
	child := self class named: anElemName.
	self addChild: child.
	^child
!

createParentNamed: anElemName
	| pa |
	pa := self class named: anElemName.
	pa addChild: self.
	^pa
!

declareNamespace: prefix uri: uri
	self attributesHolder declareNamespace: prefix uri: uri
!

declaredNamespacePrefixes
	^ self attributesHolder declaredNamespacePrefixes 
!

declaredNamespaceURIFor: prefix
    |namespaceURI|

    namespaceURI := self attributesHolder declaredNamespaceURIFor: prefix.
    namespaceURI isNil ifTrue:[
        ^ self parent declaredNamespaceURIFor:prefix.
    ].
    ^ namespaceURI.
!

findChild: aBlock
	^self children detect: aBlock ifNone:[]
!

findChildLocalNamed: anElemName
	^self children detect: [:each | each localName = anElemName] ifNone:[]
!

findChildNamed: anElemName
	^self children detect: [:each | each name = anElemName] ifNone:[]
!

findChildren: aBlock
	^self children select: aBlock
!

findChildrenLocalNamed: anElemName
	^self children select: [:each | each localName = anElemName]
!

findChildrenNamed: anElemName
	^self children select: [:each | each name = anElemName]
!

firstChild
	^self children first
!

getAttributeLike: aNameString
	^self attributesHolder getAttributeLike: aNameString
!

getAttributeNamed: aNameString
	^self attributesHolder getAttributeNamed: aNameString
!

hasAttributeLike: aNameString
	^self attributesHolder hasAttributeLike: aNameString
!

hasAttributeNamed: aNameString
	^self attributesHolder hasAttributeNamed: aNameString
!

putAttribute: aNameValueArray
	
	self putAttributeNamed: aNameValueArray first asString value: aNameValueArray last asString
!

putAttributeNamed: aNameString value: aValueString
	^self attributesHolder putAttributeNamed: aNameString value: aValueString
!

removeAttributeNamed: aNameString
	^self attributesHolder removeAttributeNamed: aNameString
!

removeChild: anXMLEleemnt
	^self children remove: anXMLEleemnt ifAbsent:[nil].
	
!

shortName
	| nm idx sz |
	nm := self name.
	idx := nm indexOf: $: ifAbsent: [^nm].
	sz := nm size.
	idx > sz ifTrue:[^nm].
	^nm copyFrom: idx + 1 to: sz
!

undeclareNamespaces
	self attributesHolder undeclareNamespaces
! !

!SoXmlWrapElement methodsFor:'initialization & release'!

initialize
	isEmptyBody := false.
! !

!SoXmlWrapElement methodsFor:'printing'!

printCanonicalXmlContentsOn: aStream 
        self isEmptyBody
                ifFalse: 
                        [self value isNil ifFalse: [self privPrintCanonicalXmlContentOn: aStream ].
                        aStream cr.
                        children notNil ifTrue:[
                            self privPrintCanonicalChildrenOn: aStream.
                        ]].
!

printCanonicalXmlFooterOn: aStream 
        "no end-tag optimization here"

        self isXmlPrintable ifFalse: [^ self].
        aStream nextPut: $<.
        aStream nextPut: $/.
        self writeName: self name on: aStream.
        aStream nextPut: $>.
!

printCanonicalXmlHeaderOn: aStream 
        "no end-tag optimization here"
        self isXmlPrintable ifFalse: [^ self].

        aStream nextPut: $<.
        self writeName: self name on: aStream.
        self privPrintAttribsOn: aStream indent:nil.
        aStream nextPut: $>.
!

printCanonicalXmlOn: aStream 
        self isXmlPrintable ifFalse: [^ self].

        self printCanonicalXmlHeaderOn: aStream.
        self printCanonicalXmlContentsOn: aStream.
        self printCanonicalXmlFooterOn: aStream.
        aStream cr
!

printOn: aStream
	"super printOn: aStream" "for Debug"
	self printXmlOn: aStream
!

printXmlContentsOn: aStream indent: level 
	self isEmptyBody
		ifFalse: 
			[self value isNil ifFalse: [self privPrintXmlContentOn: aStream ].
			children isNil ifFalse:["ugly - but for performance"
			aStream cr.
			self privPrintChildrenOn: aStream indent: level + 1.
			level timesRepeat: [aStream space]]].
!

printXmlFooterOn: aStream 
	self isXmlPrintable ifFalse: [^ self].
	self isEmptyBody
		ifFalse: 
			[aStream nextPut: $<.
			aStream nextPut: $/.
			self writeName: self name on: aStream.
			aStream nextPut: $>].
!

printXmlHeaderOn: aStream 
        self printXmlHeaderOn: aStream indent:nil
!

printXmlHeaderOn: aStream indent:levelOrNil
        self isXmlPrintable ifFalse: [^ self].
        self isEmptyBody
                ifTrue: 
                        [aStream nextPut: $<.
                        self writeName: self name on: aStream.
                        self privPrintAttribsOn: aStream indent:levelOrNil.
                        aStream nextPut: $/.
                        aStream nextPut: $>]
                ifFalse: 
                        [aStream nextPut: $<.
                        self writeName: self name on: aStream.
                        self privPrintAttribsOn: aStream indent:levelOrNil.
                        aStream nextPut: $>.].
!

printXmlOn: aStream 
	self printXmlOn: aStream indent: 0
!

printXmlOn: aStream indent: level 
        self isXmlPrintable
                ifFalse: [^ self].
        level
                timesRepeat: [aStream space].
        self printXmlHeaderOn: aStream indent: level.
        self printXmlContentsOn: aStream indent: level.
        self printXmlFooterOn: aStream.
        aStream cr
!

xmlString
	| wStr |
	wStr := WriteStream on: (String new: 64).
	self printXmlOn: wStr.
	^wStr contents
! !

!SoXmlWrapElement methodsFor:'private'!

isXmlPrintable
	^self name notNil
!

privElementClassFrom: parentXmlElement
	"override to return apporopriate XmlWrapElement"
	^self class
!

privPrintAttribsOn: aStream
    self privPrintAttribsOn: aStream indent:nil
!

privPrintAttribsOn: aStream indent:levelOrNil
        attributesHolder isNil ifTrue:[^self]. "ugly - but for performance"
        self attributesHolder printXmlOn: aStream indent:levelOrNil
!

privPrintCanonicalChildrenOn: aStream 
        (self privSortChildren: children) do:[ :each |
                each printCanonicalXmlOn: aStream .
        ]
!

privPrintCanonicalXmlContentOn: aStream 
        self privPrintXmlContentOn: aStream
!

privPrintChildrenOn: aStream indent: level
	(self privSortChildren: children) do:[ :each |
		each printXmlOn: aStream indent: level.
	]
!

privPrintXmlContentOn: aStream 
	self isPrintCDATA
		ifTrue: 
			[aStream nextPutAll: '<!![CDATA['.
			aStream nextPutAll: self value.
			aStream nextPutAll: ']]>']
		ifFalse: [self writeValue: self value on: aStream.].
!

writeName: aCharacterArray on: aStream 
	"TODO: check illegal characters"
	^ aStream nextPutAll: aCharacterArray
!

writeValue: aCharacterArray on: aStream 
	^SoXmlUtil writeXmlText: aCharacterArray on: aStream
! !

!SoXmlWrapElement methodsFor:'private-initialization'!

initAttributesBy: aDictionary
	aDictionary keysAndValuesDo:[:k :v |
		self putAttributeNamed: k value: v.
	].
	
!

initByDom: aParsedXmlElement 
	| prefixAndLocalName |
	self
		name: (SoXmlUtil elementNameFrom: aParsedXmlElement).
	prefixAndLocalName := SoXmlUtil prefixAndLocalNameFrom: self name.
	self prefixName: prefixAndLocalName first.
	self localName: prefixAndLocalName last.
	self
		initAttributesBy: (SoXmlUtil attribDictFrom: aParsedXmlElement).
	self
		value: (SoXmlUtil elementValueFrom: aParsedXmlElement).

	(SoXmlUtil elementsWithoutBlankTextsFrom: aParsedXmlElement)
		do: [:each | self initChildElementByDom: each]
!

initChildElementByDom: aParsedXmlElement 
	| parentElem childElmClass child |
	(SoXmlUtil isTextElement: aParsedXmlElement)
		ifTrue: [^ self
				value: (SoXmlUtil elementValueFrom: aParsedXmlElement)].
	parentElem := SoXmlUtil elementNameFrom: aParsedXmlElement.
	childElmClass := self privElementClassFrom: parentElem.
	child := childElmClass fromXmlElement: aParsedXmlElement.
	self addChild: child
! !

!SoXmlWrapElement methodsFor:'sorting'!

privSortChildren: aCollectionOfXmlElement
	"default do nothing"
	^aCollectionOfXmlElement
! !

!SoXmlWrapElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoXmlWrapElement.st,v 1.11 2010/06/14 10:54:23 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoXmlWrapElement.st,v 1.11 2010/06/14 10:54:23 cg Exp $'
! !
