"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapPRCElement subclass:#SoapCall
	instanceVariableNames:'connector locator'
	classVariableNames:'TraceCalls AccessLock InvokeWithAccessLock'
	poolDictionaries:''
	category:'SOAP-RPC'
!

SoapCall comment:'I represent a Remote Procedure Call in SOAP.

---
MU 5/12/2002 00:47'
!


!SoapCall class methodsFor:'instance creation'!

locator: aLocatorObject
        ^self new locator: aLocatorObject
!

methodName:methodName
    ^ self new methodName:methodName.
!

methodName:methodName tcpHost:host port:port
    ^ self new methodName:methodName tcpHost:host port:port
!

newLocal
        "default locator is SoapTcpLocator for localhost"
        ^ self locator: (SoapTcpLocator host: 'localhost' port: 8823)
!

tcpHost:host port:port
    ^ self locator: (SoapTcpLocator host: host port: port)
!

url: anURLString
	
	"host port transport targetObjectURI := self parseURL: anURLString"
! !

!SoapCall class methodsFor:'class initialization'!

initialize
    AccessLock := Semaphore forMutualExclusion.
    InvokeWithAccessLock := false.
! !

!SoapCall class methodsFor:'tracing'!

invokeWithAccessLock
    ^ InvokeWithAccessLock
!

invokeWithAccessLock:aBoolean
    InvokeWithAccessLock := aBoolean
!

traceCalls
    ^ TraceCalls ? false
!

traceCalls:aBoolean
    TraceCalls := aBoolean

    "
     self traceCalls:true
    "
! !

!SoapCall methodsFor:'accessing'!

addParameter: aSoapVariable
	aSoapVariable parent: self.
	^self parameters add: aSoapVariable
!

addParameterNamed: paramName 
	self addParameter: (SoapVariable name: paramName). 
!

addParameterNamed: paramName type: xsdType
	self addParameter: (SoapVariable name: paramName type: xsdType). 
!

addParameterNamed: paramName value: paramValue
	self addParameter: (SoapVariable name: paramName value: paramValue). 
!

addParameterNamed: paramName value: paramValue type: xsdType
	self addParameter: (SoapVariable name: paramName value: paramValue type: xsdType). 
!

addParameterNamed: paramName value: paramValue xsdType: xsdType
	self addParameter: (SoapVariable name: paramName value: paramValue xsdType: xsdType). 
!

addParameterNamed: paramName xsdType: xsdType
	self addParameter: (SoapVariable name: paramName xsdType: xsdType). 
!

connector
	connector isNil ifTrue: [ connector := SoapConnectorFactory createFrom: self transport].
	^connector
!

connector: aValue

	connector := aValue
!

locator
        locator isNil ifTrue: [ locator := SoapCallEntry local locator].
        ^locator
!

locator: aValue

	locator := aValue
!

methodName:mName tcpHost:host port:port
    self methodName:mName.
    self tcpHost:host port:port.
!

tcpHost:host port:port
    self locator:(SoapTcpLocator host:host port:port)
!

transport: aSymbol
	"need to reset connector"
	self connector: nil.
	super transport: aSymbol.
! !

!SoapCall methodsFor:'actions'!

invoke
    "perform the RPC; return a response object containing result or error-information"

    |returnedMsg resp envelope invokeBlock|

    invokeBlock := [
        self optionHeaderProperties 
            keysAndValuesDo:[:k :v | self context at:k put:v ].
        envelope := self envelope.
        TraceCalls == true ifTrue:[
            Transcript showCR:'SOAP: -> ' , self locator printString.
            Transcript showCR:envelope.
        ].
        returnedMsg := self connector 
                    send:envelope
                    to:self locator
                    with:self context.
        TraceCalls == true ifTrue:[
            Transcript showCR:'SOAP: <- ' , self locator printString.
            Transcript showCR:returnedMsg.
        ].
        resp := SoapResponse fromSoapMessage:returnedMsg.
        self clear.
    ].

    "/ Just a temporary hack to invoke SOAP calls with or without access lock
    InvokeWithAccessLock ifTrue: [
        AccessLock critical:invokeBlock.
    ] ifFalse: [
        invokeBlock value
    ].

    ^ resp
!

invokeAndReturn
    "perform the RPC.
     Return the result or raise an exception."
    
    |resp|

    resp := self invoke.
    ^ resp isFault 
        ifTrue:[ resp raiseException ]
        ifFalse:[ resp returnValue ]
!

invokeAndReturnWith:value 
    "rebind the 1st parameter with the argument
         and (re)perform the RPC.
         Return the result or raise an exception.
         Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeAndReturnWithValues:(Array with:value)
!

invokeAndReturnWithValues:values 
    "rebind the parameters with the arguments
         and (re)perform the RPC.
         Return the result or raise an exception.
         Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    |paramSize|

    paramSize := self parameters size.
    1 to:paramSize do:[:idx | 
        (self parameters at:idx) value:(values at:idx ifAbsent:[])
    ].
    ^ self invokeAndReturn
!

invokeWithHeaderProperties:aHeaderProperties 
    "cg: huh - what is this supposed to do ?"
    
    self headerProperties:aHeaderProperties.
    ^ self invoke
!

invokeWithValue:value 
    "rebind the 1st parameter with the argument
     and (re)perform the RPC.
     Return a response object containing result or error-information.
     Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeWithValues:(Array with:value)
!

invokeWithValue:value1 value:value2 
    "rebind the 1st and 2nd parameters with the arguments
     and (re)perform the RPC.
     Return a response object containing result or error-information.
     Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeWithValues:(Array with:value1 with:value2)
!

invokeWithValue:value1 value:value2 value:value3 
    "rebind the 1st three parameters with the arguments
     and (re)perform the RPC.
     Return a response object containing result or error-information.
     Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeWithValues:(Array 
                with:value1
                with:value2
                with:value3)
!

invokeWithValues:values 
    "rebind theparameters with the arguments
     and (re)perform the RPC.
     Return a response object containing result or error-information.
     Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    |paramSize|

    paramSize := self parameters size.
    1 to:paramSize do:[:idx | 
        (self parameters at:idx) value:(values at:idx ifAbsent:[])
    ].
    ^ self invoke
!

perform
    "perform the RPC.
     Return the result or raise an exception."
    
    ^ self invokeAndReturn
!

performWith:value 
    "rebind the 1st parameter with the argument
         and (re)perform the RPC.
         Return the result or raise an exception.
         Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeAndReturnWith:value
!

performWithValues:values 
    "rebind the parameters with the arguments
         and (re)perform the RPC.
         Return the result or raise an exception.
         Useful if multiple calls are to be performed in a sequence with differing arg values."
    
    ^ self invokeAndReturnWithValues:values
! !

!SoapCall methodsFor:'building envelope'!

buildSoapBody: aSoapBody 
	| pref callElem |
	pref := self prefix.
	callElem := self xmlElementClass named: self methodName withPrefix: pref.
	callElem declareNamespace: pref uri: self namespace.
	self parameters
		do: [:each | callElem
				addChild: (each asXmlElement)].
	aSoapBody addBodyEntry: callElem.
	^ aSoapBody
!

buildSoapHeader: aSoapHeader
	"##TODO: support additional header properties (like transaction id)"
	^aSoapHeader
	
	
! !

!SoapCall methodsFor:'private'!

clear
	self optionHeaderProperties: nil.
	self envelope: nil.
	self context: nil.
	
! !

!SoapCall methodsFor:'testing'!

isCall
	^true
! !

!SoapCall class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapCall.st,v 1.14 2011/05/27 14:23:16 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapCall.st,v 1.14 2011/05/27 14:23:16 sr Exp $'
! !

SoapCall initialize!
