"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapAbstractDecoder subclass:#SoapDecoder
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Decoding'
!

SoapDecoder class instanceVariableNames:'useXsiNullAttribute'

"
 The following class instance variables are inherited by this class:

	SOAP::SoapAbstractDecoder - 
	SOAP::SoapDataBinder - typeDict
	Object - 
"
!


!SoapDecoder class methodsFor:'class initialization'!

initTypeDict
    "Apparently this mapping is not so sophisticted. "
    "##TODO: Implement more serious mapping method "
    
    |pref|

    typeDict := super initTypeDict.
    pref := SoapConstants xsdPrefixColon.
    #((double Float)
      (base64Binary ByteArray)) 
            do:[:each | typeDict at:(pref , each first) asSymbol put:each last ].

    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
        #((dateTime Timestamp)) do:[:each | 
            typeDict at:(pref , each first) asSymbol put:each last
        ].
    ].
    ^ typeDict

    "
     self initTypeDict
    "

    "Modified: / 02-08-2007 / 16:26:12 / cg"
!

initialize
	"SoapDecoder initialize"
	super initialize.
	useXsiNullAttribute := false.

	
	
! !

!SoapDecoder class methodsFor:'setting'!

useXsiNullAttribute
	^useXsiNullAttribute
!

useXsiNullAttribute: aBoolean
	useXsiNullAttribute := aBoolean
! !

!SoapDecoder methodsFor:'actions'!

decodeXmlElement: aSoapWrapElement 
	self class useXsiNullAttribute
		ifTrue: [^ self decodeNilXmlElement: aSoapWrapElement].
	aSoapWrapElement isArrayType
		ifTrue: [^ self decodeArrayXmlElement: aSoapWrapElement].
	^ aSoapWrapElement children isEmpty
		ifTrue: [self decodePrimitiveTyped: aSoapWrapElement xsiType value: aSoapWrapElement value]
		ifFalse:[self decodeComplexXmlElement: aSoapWrapElement]
! !

!SoapDecoder methodsFor:'private'!

decodeArrayXmlElement: aSoapWrapElement 
	| sz array |
	sz := aSoapWrapElement size.
	array := Array new: sz.
	1 to: aSoapWrapElement children size
		do: [:idx | 
			| itemElem | 
			itemElem := aSoapWrapElement children	at: idx ifAbsent: [].
			itemElem isNil
				ifFalse: [| itemType decodedItemValue | 
					itemType := aSoapWrapElement arrayItemType.
					decodedItemValue :=  (itemType = SoapConstants xsdAnyType or: [itemElem children isEmpty not])
						ifTrue: [self decodeXmlElement: itemElem]
						ifFalse: [self decodePrimitiveTyped: itemType value: itemElem value].
					array at: idx put: decodedItemValue]].
	^ array
!

decodeComplexXmlElement: aSoapXmlElement 
        | stClass |

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            stClass := Smalltalk
                                    at: aSoapXmlElement xsiType ? ''
                                    ifAbsent: [].
            stClass isNil ifTrue:[
                stClass := self class environment
                                    at: aSoapXmlElement xsiType ? ''
                                    ifAbsent: [].
            ].
        ] ifFalse:[
            stClass := self class environment
                                    at: aSoapXmlElement xsiType ? ''
                                    ifAbsent: [].
        ].

        stClass isNil
                ifTrue: [| structDict | 
                        structDict := Dictionary new.
                        aSoapXmlElement children
                                do: [:eachChild | structDict
                                                at: (SoXmlUtil localNameFrom: eachChild name)
                                                put: (self decodeXmlElement: eachChild)].
                        ^ structDict]
                ifFalse: [| structInst allInstVarNames | 
                        structInst := stClass new.
                        allInstVarNames := stClass allInstVarNames.
                        aSoapXmlElement children
                                do: [:eachChild | structInst
                                                instVarAt: (allInstVarNames
                                                                indexOf: (SoXmlUtil localNameFrom: eachChild name))
                                                put: (self decodeXmlElement: eachChild)].
                        ^ structInst]
!

decodeNilXmlElement: aSoapWrapElement
	self class useXsiNullAttribute ifTrue: [
		| xsiNull |
		xsiNull := aSoapWrapElement getAttributeNamed: (SoapConstants xsiNil).
		(xsiNull = '1' or: [xsiNull = 'true']) ifTrue:[^nil].
	].
	
	
	
	
	
!

decodePrimitiveTyped: xsiTypeName value: value 
        "quick hack - should see both prefix url and type"
        | xsiTypeShortName stClsName stClass val  |
        xsiTypeName isNil ifTrue:  [^value].
        xsiTypeShortName := SoXmlUtil localNameFrom: xsiTypeName.
        
        (xsiTypeShortName = SoapConstants string) ifTrue: [^value isNil ifTrue:[''] ifFalse:[value]].
        (xsiTypeShortName = SoapConstants time) ifTrue: [^value]. "quick hack"
        ((xsiTypeShortName = SoapConstants base64Binary) or: [xsiTypeShortName = SoapConstants base64]) ifTrue:[^ (Base64Coder on:value readStream) upToEnd].
        
        stClsName := self typeDict 
                        at: xsiTypeName asSymbol 
                        ifAbsent:[ SoapDecodingError signal: xsiTypeName, '''s decoding is not supported by ', self class printString].
        stClass := Smalltalk at: stClsName.
        val := stClass readFrom: (ReadStream on: value).
        ^val

    "Modified: / 02-08-2007 / 16:28:08 / cg"
! !

!SoapDecoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapDecoder.st,v 1.8 2007/08/18 14:53:48 cg Exp $'
! !

SoapDecoder initialize!
