"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapElement
	instanceVariableNames:'attributesHolder subElements prefixName localName name value
		parent'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Elements'
!

SoapElement comment:'An abstract class for SOAP element.


---
mu 5/21/2001 22:24'
!


!SoapElement class methodsFor:'instance creation'!

fromXmlElement: anXmlWrapElement 
	"Create a new SoapElement that wraps the passed XML element 
	or one of its children"
	| inst |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	self addSubElementsOf: anXmlWrapElement to: inst.
	^ inst
!

named: anElementName 
	| index readStream prefixName localName |
	index := anElementName indexOf: $:.
	index = 0
		ifTrue: [SoapNoNamespacePrefix signal: 'Namespace prefix must be provided!!']
		ifFalse: [readStream := anElementName readStream.
			prefixName := readStream next: index - 1.
			readStream skip: 1.
			localName := readStream next: anElementName size - index].
	(localName isEmpty
			or: [localName ~= self localName])
		ifTrue: [SoapWrongTagName signal: '"' , localName , '"' , ' must be "' , self localName , '"'].
	^ self new prefixName: prefixName;
		 localName: localName;
		 name: anElementName
!

new
        ^self basicNew initialize
! !

!SoapElement class methodsFor:'constants'!

localName
        |nm|

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            nm := self nameWithoutPrefix copy.
            (nm startsWith:'Soap') ifTrue:[
                nm := nm copyFrom:5.
            ].
            ^ nm
        ].
        ^self name copyReplaceAll: 'Soap' with: ''
! !

!SoapElement class methodsFor:'private'!

addAttributesOf: anXmlWrapElement to: aSoapElement
	anXmlWrapElement attributesDictionary
		keysAndValuesDo: [:k :v | aSoapElement putAttributeNamed: k value: v].
!

addSubElementsOf: anXmlWrapElement to: aSoapElement
	anXmlWrapElement children
		do: [:each | aSoapElement addSubElement: each].
! !

!SoapElement methodsFor:'accessing'!

attributesHolder

	^attributesHolder
!

attributesHolder: aValue

	attributesHolder := aValue
!

localName

	^localName
!

localName: aString
        name := nil.
        localName := aString.
localName = 'SoapFault' ifTrue:[self halt].
!

name
        name isNil
                ifTrue: [prefixName := self prefixName.
                        localName := self class localName.
"/ localName = 'SoapFault' ifTrue:[self breakPoint:#SoapElement].
                        name := prefixName , ':' , localName].
"/ name = 'env:SoapFault' ifTrue:[self breakPoint:#SoapElement].
        ^ name
!

name: aString
        
        name := aString.
name = 'env:SoapFault' ifTrue:[self halt].
!

parent
	^parent
!

parent: aSoapElement
	parent := aSoapElement
!

prefixName
        ^ SoapConstants soapEnvPrefix
!

prefixName: aString
	name := nil.
	prefixName := aString
!

subElements
	subElements isNil ifTrue: [subElements := OrderedCollection new].
	^subElements
!

subElements: aValue

	subElements := aValue
!

value
	^value
!

value: aString
	value := aString
! !

!SoapElement methodsFor:'actions'!

addSubElement: anXmlElement
	"anXmlElement mustUnderstand:
		#printXmlOn:aStream indent: anInteger
		#parent: anXmlElement"
	anXmlElement parent: self.
	^self subElements add: anXmlElement.
	
!

declareDefaultNamespace
	self declareNamespace: SoapConstants soapEnvPrefix  uri: SoapConstants soapEnvURI.
	
!

declareEncodingIn: nameSpacePrefix  uri: uriString
	self putAttributeNamed: nameSpacePrefix, ':', SoapConstants encodingStyle value: uriString.
	
!

declareNamespace: prefix uri: uri
	self attributesHolder declareNamespace: prefix uri: uri
!

declareSoapDsNamespace
        self declareNamespace: SoapConstants soapDsPrefix  uri: SoapConstants soapDsigURI.
!

declareSoapEncEncoding
	self declareEncodingIn: SoapConstants soapEnvPrefix uri: SoapConstants soapEncURI.
!

declareSoapEncNamespace
	self declareNamespace: SoapConstants soapEncPrefix  uri: SoapConstants soapEncURI.
	
!

declareSoapSecNamespace
        self declareNamespace: SoapConstants soapSecPrefix  uri: SoapConstants soapSecURI.
!

declareTypicalNamespaces
	self declareSoapEncNamespace.
	self declareXsdNamespace.
	self declareXsiNamespace
!

declareXsdNamespace
        self declareNamespace: SoapConstants xsdPrefix  uri: SoapConstants xsdURI.
!

declareXsiNamespace
	self declareNamespace: SoapConstants xsiPrefix  uri: SoapConstants xsiURI.
	
!

declaredNamespacePrefixes
	^self attributesHolder declaredNamespacePrefixes
!

declaredNamespaceURIFor: prefix
    |namespaceURI|

    namespaceURI := self attributesHolder declaredNamespaceURIFor: prefix.
    namespaceURI isNil ifTrue:[
        ^ self parent declaredNamespaceURIFor:prefix.
    ].
    ^ namespaceURI.
!

defineSecId:idString
        self putAttributeNamed: SoapConstants soapSecPrefixColon, 'id' value: idString.
!

encodingStyleURIs
        "ToDo: take a more serious approach...Namespace conscious"
        | encs |
        encs := self attributesHolder encodingStyle.
        self parent notNil
                ifTrue: [| parentEncodeingURIs myURIs | 
                        parentEncodeingURIs := self parent encodingStyleURIs.
                        myURIs := SoPortableUtil substringsFrom: encs.
                        ^ (parentEncodeingURIs ~= myURIs and: [encs notEmptyOrNil])
                                ifTrue: [myURIs]
                                ifFalse: [parentEncodeingURIs]].
        ^ SoPortableUtil substringsFrom: encs

    "Modified: / 04-10-2006 / 12:36:49 / cg"
!

evaluate
	"default do nothing"
!

findSubElementLocalNamed: anElemName
	^self subElements detect: [:each | each localName = anElemName] ifNone:[]
!

findSubElementNamed: anElemName
	^self subElements detect: [:each | each name = anElemName] ifNone:[]
!

findSubElementsLocalNamed: anElemName
	^self subElements select: [:each | each localName = anElemName]
!

findSubElementsNamed: anElemName
	^self subElements select: [:each | each name = anElemName]
!

getAttributeNamed: aNameString
	^self attributesHolder getAttributeNamed: aNameString
!

putAttribute: aNameValueArray
	^self attributesHolder putAttribute: aNameValueArray
!

putAttributeNamed: attribName value: attribValue
	^self attributesHolder putAttributeNamed: attribName value: attribValue
!

undeclareNamespaces
	self attributesHolder undeclareNamespaces
! !

!SoapElement methodsFor:'factory'!

parser
	
	^self class parser
! !

!SoapElement methodsFor:'initialization & release'!

initialize
	attributesHolder := SoapAttributesHolder new.
	
	
! !

!SoapElement methodsFor:'printing'!

printCanonicalXmlOn: aStream
        self printCanonicalXmlHeaderOn: aStream.
        self printCanonicalXmlContentsOn: aStream.
        self printCanonicalXmlFooterOn: aStream.
        aStream cr
!

printOn: aStream
	"super printOn: aStream" "for Debug"
	self printXmlOn: aStream
!

printXmlOn: stream
	self printXmlOn: stream indent: 0
!

printXmlString
	| stream |
	stream := WriteStream on: (String new: 1024).
	self printOn: stream.
	^stream contents.
! !

!SoapElement methodsFor:'private'!

writeName: aCharacterArray on: aStream 
	"ToDo: check illegal characters"
	^ aStream nextPutAll: aCharacterArray
!

writeValue: aCharacterArray on: aStream 
	^SoXmlUtil writeXmlText: aCharacterArray on: aStream
!

writeValue: aCharacterArray on: aStream betweenTag: aTagString
	"ToDo: check illegal characters"
	aStream nextPutAll: ('<', aTagString, '>').
	SoXmlUtil writeXmlText: aCharacterArray on: aStream.
	aStream nextPutAll: ('</', aTagString, '>').
! !

!SoapElement methodsFor:'private-printing'!

printCanonicalXmlContentsOn: aStream 
        aStream cr.
        self privPrintCanonicalChildrenOn: aStream.
!

printCanonicalXmlFooterOn: aStream 
        self printXmlFooterOn: aStream.
!

printCanonicalXmlHeaderOn: aStream 
        self printXmlHeaderOn: aStream
!

printXmlContentsOn: aStream indent: level 
	aStream cr.
	self privPrintChildrenOn: aStream indent: level + 1.
	level timesRepeat: [aStream space].
!

printXmlFooterOn: aStream 
	aStream nextPut: $<.
	aStream nextPut: $/.
	self writeName: self name on: aStream.
	aStream nextPut: $>
!

printXmlHeaderOn: aStream 
        self printXmlHeaderOn: aStream indent:nil
!

printXmlHeaderOn: aStream indent:levelOrNil
        |nextLevel|

        aStream nextPut: $<.
        self writeName: self name on: aStream.
        levelOrNil notNil ifTrue:[
            nextLevel := levelOrNil+1.
        ].
        self privPrintAttribsOn: aStream indent:nextLevel.
        aStream nextPut: $>
!

printXmlOn: aStream indent: level 
        level timesRepeat: [aStream space].
        self printXmlHeaderOn: aStream indent:level.
        self printXmlContentsOn: aStream indent: level.
        self printXmlFooterOn: aStream.
        aStream cr
!

privPrintAttribsOn: aStream
        self privPrintAttribsOn: aStream indent:nil
!

privPrintAttribsOn: aStream indent:levelOrNil
        self attributesHolder printXmlOn: aStream indent:levelOrNil
!

privPrintCanonicalChildrenOn: aStream
        "$override$"
        subElements isNil ifFalse: [
            self subElements do:[:each | each printCanonicalXmlOn: aStream].
        ]
!

privPrintChildrenOn: aStream indent: level
	"$override$"
	subElements isNil ifFalse: [
		self subElements do:[:each | each  printXmlOn: aStream indent: level].
	]
! !

!SoapElement methodsFor:'testing'!

isBody
	^false
!

isEnvelope
	^false
!

isFault
	^false
!

isFaultDetail
	^false
!

isHeader
	^false
! !

!SoapElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapElement.st,v 1.13 2010/06/14 10:54:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapElement.st,v 1.13 2010/06/14 10:54:39 cg Exp $'
! !
