"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapAbstractEncoder subclass:#SoapEncoder
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Encoding'
!

SoapEncoder class instanceVariableNames:'useXsiNullAttribute'

"
 The following class instance variables are inherited by this class:

	SOAP::SoapAbstractEncoder - 
	SOAP::SoapDataBinder - typeDict
	Object - 
"
!


!SoapEncoder class methodsFor:'class initialization'!

initTypeDict
        "Apparently this mapping is not mature. "
        "##TODO: Implement more serious mapping"
        | pref |
        typeDict := super initTypeDict.
        pref := SoapConstants xsdPrefixColon.
        #(
            #(#Any #any)
            #(#DateAndTime #dateTime)
            #(#Time #time)
            #(#Date #date)
            #(#Boolean #boolean)
            #(#True #boolean)
            #(#False #boolean)
            #(#Array #arrayType)
            #(#ByteArray #base64Binary)
            #(#Float #float)
            #(#Double #double)
            #(#Integer #integer)
            #(#SmallInteger #integer)
            #(#LargePositiveInteger #positiveInteger)
            #(#LargeNegativeInteger #negativeInteger)
            #(#Byte #byte)
            #(#String #string)
            #(#Symbol #string)
        ) do:[:each | typeDict at: each first put: (pref, each last) asSymbol].

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
        #(
            (AbsoluteTime      dateTime)             "backward compatibility"
            (Timestamp         dateTime)
            (LargeInteger      integer)
            (ShortFloat        float)
            (Unicode16String   string)
        ) do:[:each | typeDict at: each first put: (pref, each last) asSymbol].
        ].
        ^ typeDict

        "
         self initTypeDict
        "
!

initialize
	"SoapEncoder initialize"
	super initialize.
	useXsiNullAttribute := false.

	
	
! !

!SoapEncoder class methodsFor:'setting'!

useXsiNullAttribute
	^useXsiNullAttribute
!

useXsiNullAttribute: aBoolean
	useXsiNullAttribute := aBoolean
! !

!SoapEncoder methodsFor:'actions'!

encodeSoapVariable: aSoapVariable 
        | val type xsiTypeAttVal elem |
        val := aSoapVariable value.
        val isNil ifTrue: [^ self encodeNilSoapVariable: aSoapVariable].
        type := aSoapVariable type.
        type isNil ifTrue: [
            (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                type := val class nameWithoutPrefix asSymbol
            ] ifFalse:[
                type := val class name asSymbol
            ]
        ].
        type == #SoapVariable ifTrue:[
            "this is recursive"
            elem := self xmlElementClass named: aSoapVariable name.
            elem addChild:(self encodeSoapVariable:val).
            ^ elem.
        ].
        (type == #Array or: [type == #SoapEncArray]) ifTrue: [^ self encodeArraySoapVariable: aSoapVariable].
        ((type == #Dictionary   or: [type == #IdentityDictionary or:[type == #OrderedDictionary]])
                        or: [type == #SoapEncStruct])   ifTrue: [^ self encodeDictionarySoapVariable: aSoapVariable].
        xsiTypeAttVal := aSoapVariable xsdType.
        xsiTypeAttVal isNil ifTrue: [xsiTypeAttVal := self class typeDict
                                at: type
                                ifAbsent: [self breakPoint:#'sv'. SoapEncodingError signal: type printString , '''s encoding is not supported by ' , self class printString]].
        elem := self xmlElementClass named: aSoapVariable name.
        xsiTypeAttVal notEmpty ifTrue:[
            elem xsiType: xsiTypeAttVal asString.
        ].
        self
                setXmlElementValue: aSoapVariable
                of: xsiTypeAttVal
                into: elem.
        ^ elem
! !

!SoapEncoder methodsFor:'private'!

encodeArray: anArray named: elemName elementType: anXSDType
    | inst encPrefix lastValidElementIndex|

    encPrefix := SoapConstants soapEncPrefix.

    inst := SoapEncArray named: elemName.
    inst declareNamespace: encPrefix uri: SoapConstants soapEncURI.
    inst declareXsdNamespace.
    inst declareXsiNamespace.
    inst putAttributeNamed: SoapConstants arrayType value: anXSDType , '[' , anArray size printString , ']'.

    #isSmalltalkX.
    lastValidElementIndex := anArray findLast:[:el |el notNil].
    anArray keysAndValuesDo:[:idx :each | 
        | child |
        idx <= lastValidElementIndex ifTrue:[
            "/ CG: this is wrong; nil elements must be transmitted,
            "/ except for the trailing ones...
            "/ (each isNil or: [each == #nil]) ifFalse: [
                child := ((SoapVariable name: 'item' value: each) encoder:self) asXmlElement.
                inst addChild: child.
            "/ ]
        ]
    ].
    ^ inst
!

encodeArraySoapVariable: aSoapVariable 
        | arr allTypes elementType |
        arr := aSoapVariable value.
        arr class == SoapEncArray ifTrue: [^arr trimForNested].
        "/ arr class name == #SoapEncArray ifTrue: [^arr trimForNested].
        allTypes := arr collect: [:each | each class].
        (allTypes asSet size > 1 or:[allTypes isEmpty])
            ifTrue: [ 
                elementType := (SoapConstants xsdAnyType)]
            ifFalse: [ 
                elementType := ( self class typeDict at: allTypes first name ifAbsent:[SoapConstants xsdAnyType])].

        ^ self
            encodeArray: aSoapVariable value
            named: aSoapVariable name
            elementType: elementType
!

encodeDictionarySoapVariable: aSoapVariable 
        | elem dictOrStruct  soapVariable|
        elem := self xmlElementClass named: aSoapVariable name.
        dictOrStruct := aSoapVariable value.
        dictOrStruct class == SoapEncStruct
                ifTrue: [elem parent isNil ifTrue:[elem declareXsiNamespace].
                        elem xsiType: dictOrStruct name].
        dictOrStruct keysAndValuesDo: [:k :v |
            soapVariable := SoapVariable name:k.
            (v class nameWithoutPrefix asSymbol == #SoapVariable and:[v name isNil]) ifTrue:[
                "A SoapVariable without name is used to set the xsdType"
                soapVariable value:v value; type:v type; xsdType:v xsdType.
            ] ifFalse:[
                soapVariable value:v.
            ].
            elem addChild: (self encodeSoapVariable: soapVariable)
        ].
        ^ elem
!

encodeNilSoapVariable: aSoapVariable 
	| elem |
	^self class useXsiNullAttribute
		ifTrue: [elem := self xmlElementClass emptyNamed: aSoapVariable name.
			elem putAttributeNamed: SoapConstants xsiNil value: 1.
			elem]
		ifFalse: [nil]
!

setXmlElementValue:aSoapVariable of:xsiTypeString into:anXmlElement 
    |value|

    value := aSoapVariable value.
"/    type == #SoapVariable ifTrue:[
"/        "recursive"
"/        elem value:(self encodeSoapVariable:value).
"/        ^ self.
"/    ].
    xsiTypeString = SoapConstants xsdBase64BinaryType ifTrue:[
        value := Base64Coder encode:value asByteArray.
    ].
    anXmlElement value:value.

    "otherwise"
! !

!SoapEncoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapEncoder.st,v 1.16 2009/11/22 10:09:59 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapEncoder.st,v 1.16 2009/11/22 10:09:59 cg Exp $'
! !

SoapEncoder initialize!
