"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapElement subclass:#SoapEnvelope
	instanceVariableNames:'header body'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Elements'
!

SoapEnvelope comment:'SOAP Envelope element.


---
mu 5/21/2001 22:24'
!


!SoapEnvelope class methodsFor:'instance creation'!

fromXmlElement: anXmlWrapElement 
	| inst headElem bodyElem others |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	headElem := anXmlWrapElement findChildLocalNamed: SoapHeader localName.
	headElem isNil
		ifFalse: [inst
				header: (SoapHeader fromXmlElement: headElem)].
	bodyElem := anXmlWrapElement findChildLocalNamed: SoapBody localName.
	bodyElem isNil
		ifFalse: [inst
				body: (SoapBody fromXmlElement: bodyElem)].
	others := anXmlWrapElement children
				reject: [:each | each localName = SoapHeader localName
						or: [each localName = SoapBody localName]]. "will be obsolete in SOAP 1.2"
	others
		do: [:otherE | inst addSubElement: otherE].
	^ inst
! !

!SoapEnvelope class methodsFor:'constants'!

localName
	^'Envelope'
! !

!SoapEnvelope methodsFor:'actions'!

addBody
        | bo |
        bo := SoapBody new.
        bo parent: self.
        self body: bo.
        ^bo
!

addHeader
	| he |
	he := SoapHeader new.
	he parent: self.
	self header: he.
	^he
! !

!SoapEnvelope methodsFor:'elements'!

body
	body isNil
		ifTrue: [self addBody].
	^ body
!

body: aSoapBody
	body := aSoapBody.
	aSoapBody parent: self
!

header
	^ header
!

header: aSoapHeader
	header := aSoapHeader.
	aSoapHeader parent: self
! !

!SoapEnvelope methodsFor:'initialization & release'!

initialize
	super initialize.
	
	self declareDefaultNamespace.
	
	
	
! !

!SoapEnvelope methodsFor:'private-printing'!

privPrintCanonicalChildrenOn: aStream
        self header isNil ifFalse:[self header printCanonicalXmlOn: aStream].
        self body printCanonicalXmlOn: aStream. "##body always exists##"
        super privPrintCanonicalChildrenOn: aStream 
!

privPrintChildrenOn: aStream indent: level
	self header isNil ifFalse:[self header printXmlOn: aStream indent: level].
	self body printXmlOn: aStream indent: level. "##body always exists##"
	super privPrintChildrenOn: aStream indent: level
! !

!SoapEnvelope methodsFor:'testing'!

hasFault
	^self body hasFault 
!

hasHeader
	^self header notNil
	
!

isEnvelope
	^true
!

verifySignature:aSignature
    |secPrefix ns|

    secPrefix := aSignature prefixName.
    ns := aSignature declaredNamespaceURIFor:secPrefix.
    (ns = SoapConstants soapSecURI) ifFalse:[
        SoapUnsupportedSignature raise.
        ^ self.
    ].
    SoapSignatureAlgorithm validate:aSignature in:self.
!

verifySignatures
    |header signatures|

    header := self header.
    header isNil ifTrue:[^ self].

    signatures := header signatures.
    signatures isEmptyOrNil ifTrue:[^ self].

    signatures do:[:eachSignature | self verifySignature:eachSignature].
    ^ self 
! !

!SoapEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapEnvelope.st,v 1.8 2006/07/24 17:51:40 stefan Exp $'
! !
