"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapElement subclass:#SoapFault
	instanceVariableNames:'isBody faultcode faultstring faultactor detail'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Elements'
!

SoapFault class instanceVariableNames:'TheSignal'

"
 No other class instance variables are inherited by this class.
"
!

ProceedableError subclass:#SoapFaultException
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:SoapFault
!


!SoapFault class methodsFor:'initialization'!

initialize
    self initializeSignal

   "
     TheSignal := nil.
     self initializeSignal.

     self theSignal
   "
!

initializeSignal
    TheSignal isNil ifTrue:[
	self == SOAP::SoapFault ifTrue:[
	    TheSignal := SoapFaultException.
	] ifFalse:[
	    TheSignal := self superclass theSignal newSignal
			notifierString:self className;
			nameClass:self message:#theSignal.
	]
    ].
! !

!SoapFault class methodsFor:'instance creation'!

beClient

	^self new beClient
!

beMustUnderstand

	^self new beMustUnderstand
!

beServer

	^self new beServer
!

beVersionMismatch

	^self new beVersionMismatch
!

fromXmlElement: anXmlWrapElement
	| inst faultcode faultstring faultactor detail others |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	faultcode := anXmlWrapElement findChildNamed: 'faultcode'.
	faultcode isNil ifFalse: [inst faultcode: faultcode value].
	faultstring := anXmlWrapElement findChildNamed: 'faultstring'.
	faultstring isNil       ifFalse: [inst faultstring: faultstring value].
	faultactor := anXmlWrapElement findChildNamed: 'faultactor'.
	faultactor isNil        ifFalse: [inst faultactor: faultactor value].
	detail := anXmlWrapElement findChildNamed: 'detail'.
	detail isNil    ifFalse: [inst detail: (SoapFaultDetail fromXmlElement: detail)].
	others := anXmlWrapElement children
				reject: [:each | #('faultcode' 'faultstring' 'faultactor' 'detail') includes: each name].
	others  do: [:otherE | inst addSubElement: otherE].
	^ inst
! !

!SoapFault class methodsFor:'exception mimicry'!

accepts:aSignal
    ^ TheSignal accepts:aSignal
!

exceptionHandlerFor:anException in:aContext
    "answer the exceptionHandler for anException from aContext."

    |sel|

    sel := aContext selector.
    (sel == #'handle:from:do:'
     or:[sel == #'handle:do:']) ifTrue:[
	^ aContext receiver
    ].

    ^ nil
!

handle:aSignal do:aBlock
    <exception: #handle>
    <context: #return>

    aBlock value.
!

handlerForSignal:signal context:theContext originator:originator
    "answer the handler block for the signal from originator.
     The block is retrieved from aContext.
     Answer nil if the signal is not handled"

    (theContext selector ~~ #'handle:from:do:'
     or:[(theContext argAt:2) == originator]) ifTrue:[
	(self == signal or:[self accepts:signal]) ifTrue:[
	    ^ theContext argAt:1
	]
    ].

    ^ nil

    "Created: / 25.7.1999 / 19:52:58 / stefan"
!

handlerProtectedBlock:doBlock inContext:context
    "set the handlerProtectedBlock in context"

    |sel|

    sel := context selector.
    sel == #handle:do: ifTrue:[
	context argAt:2 put:doBlock.
    ] ifFalse:[sel == #handle:from:do: ifTrue:[
	context argAt:3 put:doBlock.
    ]].
!

handles:anException
    ^ TheSignal accepts:anException signal
!

isExceptionHandler
    ^ true
! !

!SoapFault class methodsFor:'signal access'!

theSignal
    TheSignal isNil ifTrue:[
	self initializeSignal
    ].
    ^ TheSignal
! !

!SoapFault methodsFor:'accessing'!

beClient
	self faultCode:(SoapConstants soapEnvPrefixColon, SoapConstants client)
	"/ self faultCode:'Client'
!

beMustUnderstand
	self faultCode:(SoapConstants soapEnvPrefixColon, SoapConstants mustUnderstand).
	"/ self faultCode:'MustUnderstand'.
	self beHeader.
!

beServer
	self faultCode:(SoapConstants soapEnvPrefixColon, SoapConstants server).
	"/ self faultCode:'Server'
!

beVersionMismatch
	self faultCode:(SoapConstants soapEnvPrefixColon, SoapConstants versionMismatch).
	"/ self faultCode:'VersionMismatch'
!

faultCode
	faultcode isNil ifTrue: [^''].
	^faultcode

"/        faultCode isNil ifTrue: [^''].
"/        ^faultCode
!

faultCode: aString
	"/ faultCode := aString
	faultcode := aString
!

faultString
	faultstring isNil ifTrue: [^''].
	^faultstring
!

faultactor: aValue

	faultactor := aValue
!

isBody
	isBody isNil ifTrue: [isBody := true].
	^isBody
!

isBody: aBoolean
	isBody := aBoolean
! !

!SoapFault methodsFor:'actions'!

addDetail
	| de |
	de := SoapFaultDetail new.
	de parent: self.
	self detail: de.
	^de
!

attachMinorFaultCodes: aArrayOfString
	| origFaultCode wStr |
	origFaultCode := self faultcode.
	wStr := WriteStream with: origFaultCode.
	aArrayOfString
		do: [:each |
			wStr nextPut: $..
			wStr nextPutAll: each].
	^self faultcode: wStr contents
! !

!SoapFault methodsFor:'actions-faultcode'!

client
	self faultcode: (SoapConstants soapEnvPrefixColon, SoapConstants client)
!

mustUnderstand
	self faultcode: (SoapConstants soapEnvPrefixColon, SoapConstants mustUnderstand)
!

server
	self faultcode: (SoapConstants soapEnvPrefixColon, SoapConstants server)
!

versionMismatch
	self faultcode: (SoapConstants soapEnvPrefixColon, SoapConstants versionMismatch)
! !

!SoapFault methodsFor:'elements'!

detail
	^ detail
!

detail: aValue

	detail := aValue
!

faultDetail:aString

    ^ self detail:aString
!

faultactor

	^faultactor
!

faultcode

	^ faultcode
!

faultcode: aValue

	faultcode := aValue
!

faultstring
	^faultstring
!

faultstring: aValue

	faultstring := aValue
!

messageText:aString

    ^ self faultstring:aString
! !

!SoapFault methodsFor:'initialization & release'!

initialize
	super initialize.
	"faultCode := "faultcode := faultstring := ''.
! !

!SoapFault methodsFor:'private-accessing'!

beBody
	isBody := true.

!

beHeader
	isBody := false.

! !

!SoapFault methodsFor:'private-printing'!

privPrintCanonicalChildrenOn: aStream
	self writeValue: self faultcode on: aStream betweenTag: 'faultcode'.
aStream cr.
	self writeValue: self faultstring on: aStream betweenTag: 'faultstring'.
aStream cr.
	self faultactor isNil ifFalse:[self writeValue: self faultactor on: aStream betweenTag: 'faultactor'. aStream cr].
	self detail isNil ifFalse:[self detail printCanonicalXmlOn: aStream. aStream cr].
	super privPrintCanonicalChildrenOn: aStream
!

privPrintChildrenOn: aStream indent: level
	aStream spaces:level. self writeValue: self faultcode on: aStream betweenTag: 'faultcode'.
aStream cr.
	aStream spaces:level. self writeValue: self faultstring on: aStream betweenTag: 'faultstring'.
aStream cr.
	self faultactor isNil ifFalse:[aStream spaces:level. self writeValue: self faultactor on: aStream betweenTag: 'faultactor'. aStream cr].
	self detail isNil ifFalse:[aStream spaces:level. self detail printXmlOn: aStream. aStream cr].
	super privPrintChildrenOn: aStream indent: level
! !

!SoapFault methodsFor:'raising'!

raiseRequest
    ^ self class theSignal 
        raiseRequestWith:self "from:thisContext sender"
        errorString:(self faultString)
!

signal:aMessage
    ^ self class theSignal raiseSignalWith:self errorString:aMessage "from:thisContext sender".
! !

!SoapFault methodsFor:'testing'!

faultCodeWithoutPrefixName
    |fc prefixWithColon|

    fc := self faultCode.
    prefixWithColon := self prefixName, ':'.
    (fc startsWith:prefixWithColon) ifTrue:[
	^ fc copyFrom:(prefixWithColon size + 1).
    ].
    ^ fc
!

faultSubCode
    |fc|

    fc := self faultCodeWithoutPrefixName.
    (fc includes:$.) ifTrue:[
	^ fc copyFrom:(fc indexOf:$.) + 1
    ].
    ^ ''
!

isClientFault
    |fc|

    fc := self faultCodeWithoutPrefixName.
    ^ fc startsWith:(SoapConstants client)
!

isFault
	^true
!

isServerFault
    |fc|

    fc := self faultCodeWithoutPrefixName.
    ^ fc startsWith:(SoapConstants server)
! !

!SoapFault class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapFault.st,v 1.18 2010/03/15 22:44:16 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapFault.st,v 1.18 2010/03/15 22:44:16 cg Exp $'
! !

SoapFault initialize!
