"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapHttpClient
	instanceVariableNames:'specifiedServer host port serverAddress reqURI socket
		sendXmlString soapAction'
	classVariableNames:'CrLf ProxyPort ProxyServer'
	poolDictionaries:''
	category:'SOAP-Http'
!

SoapHttpClient class instanceVariableNames:'showLog'

"
 No other class instance variables are inherited by this class.
"
!


!SoapHttpClient class methodsFor:'instance creation'!

host: hostnameString page: pageString
	^ self host: hostnameString port: self defaultPort page: pageString
!

host: hostnameString port: portNumberInteger page: pageString 
	| reqURI |
	reqURI := (pageString isNil
					or: [pageString isEmpty])
				ifTrue: ['/']
				ifFalse: [pageString].
	^ self new
		host: hostnameString
		port: portNumberInteger
		page: reqURI
!

url: urlString
	^self new parseURL: urlString
! !

!SoapHttpClient class methodsFor:'class initialization'!

initialize
        "SoapHttpClient initialize"
        showLog := false.

        CrLf := String with: Character return with: Character lf.
        
        ProxyPort := 8080.
        ProxyServer := nil
! !

!SoapHttpClient class methodsFor:'constants'!

defaultPort
	"default port to connect on"
	^80
! !

!SoapHttpClient class methodsFor:'setting'!

showLog
	^showLog
!

showLog: aBoolean
        showLog := aBoolean

        "
         self showLog:true
        "
! !

!SoapHttpClient methodsFor:'accessing'!

host

	^host
!

host: aValue

	host := aValue
!

port

	^port
!

port: aValue

	port := aValue
!

reqURI

	^reqURI
!

reqURI: aValue

	reqURI := aValue
!

sendXmlString

	^sendXmlString
!

sendXmlString: aValue

	sendXmlString := aValue
!

soapAction

	^soapAction
!

soapAction: aValue

	soapAction := aValue
!

socket

	^socket
!

socket: aValue

	socket := aValue
!

specifiedServer

	^specifiedServer
!

specifiedServer: aValue

	specifiedServer := aValue
! !

!SoapHttpClient methodsFor:'actions'!

send: soapEnvelopeXmlString
	^self send: soapEnvelopeXmlString action: SoapAction noIntent
!

send: soapEnvelopeXmlString action: aSoapAction 
        | cont response |
        sendXmlString := soapEnvelopeXmlString.
        soapAction := aSoapAction.
        self initializeNetwork.
        self resolveServerAddress.

        response := self sendBySocket.
        self class showLog
                ifTrue: [Transcript cr; show: 'response: ' , response printString].
        cont := self getResponseContentsFrom:response.
        self class showLog
                ifTrue: [Transcript cr; show: '##' , super printString , ' Client:Return:{'.
                        Transcript show: cont printString , '}'].
        ^ cont
! !

!SoapHttpClient methodsFor:'constants'!

defaultPort
	^self class defaultPort
! !

!SoapHttpClient methodsFor:'private'!

getResponseContentsFrom:responseIfSTX
        | list header firstData length newUrl aStream con |

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            responseIfSTX isNil ifTrue:[^ nil].

            responseIfSTX isMovedResponse ifTrue: [
                newUrl := responseIfSTX location.
                newUrl ifNotNil:[
                    Transcript show: 'redirecting to: ', newUrl; cr.
                    socket destroy.
                    ^ (self class url: newUrl) send: self sendXmlString action: self soapAction
                ] 
            ].
            socket destroy.         "Always OK to destroy!!"

            responseIfSTX isErrorResponse ifTrue: [
                SoapCannotConnectToServer raiseErrorString:('SOAP Service: ' , (responseIfSTX responseText ? responseIfSTX response), ' on ', host , ':', port printString).
            ].

            responseIfSTX mimeType ~= 'text/xml' ifTrue:[
                "/ an http error
                SoapCannotConnectToServer raiseErrorString:('SOAP Service seems to be unavailable / no xml response from ', host , ':', port printString).
            ].

"/            ^ responseIfSTX data
            ^ responseIfSTX dataFromFileOrData
        ].

        list := socket getResponseUpTo: CrLf, CrLf ignoring: (String with: Character cr).       "list = header, CrLf, CrLf, beginningOfData"
        header := list at: 1.
        firstData := list at: 3.
        
        "Transcript cr; show: 'header: ', header, ' data: ', firstData."

        "dig out some headers"
        socket header: header.
        length := socket getHeader: 'content-length'.
        length ifNotNil: [ length := length asNumber ].
        socket responseCode first = $3 ifTrue: [
                newUrl := socket getHeader: 'location'.
                newUrl ifNotNil: [
                        Transcript show: 'redirecting to: ', newUrl; cr.
                        socket destroy.
                        ^(self class url: newUrl) send: self sendXmlString action: self soapAction  ] ].

        aStream := socket getRestOfBuffer: firstData totalLength: length.
        con := aStream contents.
        socket responseCode = '401' ifTrue: [^ header, con].
        socket destroy.         "Always OK to destroy!!"
        ^con
!

host: hostnameString port: portNumberInteger page: pageString 
	host := hostnameString.
	port := portNumberInteger.
	reqURI := pageString.
	specifiedServer := host , ':' , port printString.
	ProxyServer
		ifNotNil: [reqURI := 'http://' , host , ':' , port printString , reqURI.
			"put back together"
			host := ProxyServer.
			port := ProxyPort]
!

initializeNetwork
"/ ST/X: nothing to be done
"/        Socket initializeNetwork.
!

parseURL: anUrlString 
	| url bare frm |
	url := anUrlString.
	bare := (url asLowercase beginsWith: 'http://')
				ifTrue: [url copyFrom: 8 to: url size]
				ifFalse: [url].
	host := bare copyUpTo: $/.
	specifiedServer := host.
	(host includes: $:)
		ifTrue: [port := (host copyFrom: (host indexOf: $:)
							+ 1 to: host size) asNumber.
			host := host copyUpTo: $:]
		ifFalse: [port := self defaultPort].
	(frm := (bare indexOf: $/) ) = 0 ifTrue:[frm := 1].
	reqURI := bare
				copyFrom: frm
				to: bare size.
	(reqURI size = 0 or: [reqURI = specifiedServer])
		ifTrue: [reqURI := '/'].
	ProxyServer
		ifNotNil: [reqURI := 'http://' , host , ':' , port printString , reqURI.
			"put back together"
			host := ProxyServer.
			port := ProxyPort]
!

resolveServerAddress
        serverAddress := host.
"/        serverAddress := NetNameResolver addressForName: host timeout: 20.
"/        serverAddress isNil ifTrue: [SoapCannnotResolveServer signal: 'Could not resolve the server named: ' , host].
!

sendBySocket
        | content header response |

        (Smalltalk dialectName = #SmalltalkX) ifTrue:[
            socket := HTTPInterface new.

            socket transferTimeout:(SoapTransmitTimeoutQuery query). "/ seconds
            socket connectTimeout:(SoapConnectTimeoutQuery query).   "/ seconds

            socket userAgent:('Smalltalk/X ' , Smalltalk versionString).
            socket connectTo:serverAddress port:port.
            socket isConnected ifFalse:[
                SoapCannotConnectToServer signal: 'No connection to server: ', host , ':', port printString
            ].

            socket
                additionalHeaderLines:soapAction printString;
                contentType:'text/xml; charset="utf-8"'.

            content := SoXmlUtil contentsWithXmlDeclaration: sendXmlString.

            HTTPInterface transferTimeoutNotificationSignal handle:[:ex |
                SoapCannotConnectToServer 
                    raiseErrorString:('Server does not respond: ', host , ':', port printString).
                ex proceed.
            ] do:[
                SoapCall traceCalls ifTrue:[
                    Transcript showCR:'HTTP: -> ' , serverAddress, ':' , port printString , ' url:' , reqURI.
                    Transcript showCR:soapAction printString.
                    Transcript showCR:content.
                ].

                response := socket requestPOST:content url:reqURI. 

                SoapCall traceCalls ifTrue:[
                    Transcript showCR:'HTTP: <- ' , reqURI.
                    Transcript showCR:response data.
                ].
            ].
            ^ response
        ].


        "Other Smalltalks...."

        socket := HTTPSocket new. 
        socket waitForConnectionUntil: Socket standardDeadline.
        socket connectTo: serverAddress port: port.
        socket waitForConnectionUntil: Socket standardDeadline.
        socket isConnected ifFalse: [SoapCannotConnectToServer signal: 'Server does not renpond - ''host: ', host , ' port: ', port printString].
        content := SoXmlUtil contentsWithXmlDeclaration: sendXmlString.
        header := String
                                streamContents: [:str | 
                                        str nextPutAll: 'POST ' , reqURI , ' HTTP/1.0' , CrLf.
                                        str nextPutAll: 'Host: ' , specifiedServer , CrLf.
                                        str nextPutAll: 'User-Agent: Squeak 3.2' , CrLf.
                                        str nextPutAll: 'Content-length: ', (content size-1) printString, CrLf.
                                        str nextPutAll: 'Content-type: text/xml; charset="utf-8"' , CrLf.
                                        str nextPutAll: soapAction printString, CrLf.].


        socket sendData: header.
        socket sendData: content.
        ^ self.
! !

!SoapHttpClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapHttpClient.st,v 1.23 2011/05/27 14:22:01 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapHttpClient.st,v 1.23 2011/05/27 14:22:01 sr Exp $'
! !

SoapHttpClient initialize!
