"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapEncodable subclass:#SoapPRCElement
	instanceVariableNames:'targetObjectURI methodName parameters optionSignature
		optionHeaderProperties transport envelope context namespace
		prefix'
	classVariableNames:'DefaultNamespace DefaultPrefix'
	poolDictionaries:''
	category:'SOAP-RPC'
!


!SoapPRCElement class methodsFor:'accessing'!

defaultNamespace
	DefaultNamespace isNil
		ifTrue: [DefaultNamespace := 'http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/rpc/'].
	^ DefaultNamespace
!

defaultNamespace: aString
	DefaultNamespace := aString
!

defaultPrefix
	DefaultPrefix isNil
		ifTrue: [DefaultPrefix := 'm'].
	^ DefaultPrefix
!

defaultPrefix: aString
	DefaultPrefix := aString
! !

!SoapPRCElement class methodsFor:'actions'!

unmarshallFrom: aSoapEnvelope
	"create one of my family"
! !

!SoapPRCElement class methodsFor:'class initialization'!

initialize
	"SoapPRCElement initialize"
	DefaultPrefix := DefaultNamespace := nil
! !

!SoapPRCElement methodsFor:'accessing'!

context
	context isNil ifTrue: [context := self buildContext].
	^context
!

context: aValue

	context := aValue
!

envelope
	envelope isNil ifTrue: [envelope := self buildEnvelope].
	^envelope
!

envelope: aValue

	envelope := aValue
!

methodName
	methodName isNil ifTrue: [methodName := ''].
	^methodName
!

methodName: aValue

	methodName := aValue
!

namespace
	namespace isNil ifTrue: [namespace := self class defaultNamespace].
	^namespace
!

namespace: aValue

	namespace := aValue
!

optionHeaderProperties
	optionHeaderProperties isNil ifTrue: [optionHeaderProperties := Dictionary new].
	^optionHeaderProperties
!

optionHeaderProperties: aValue

	optionHeaderProperties := aValue
!

optionSignature

	^optionSignature
!

optionSignature: aValue

	optionSignature := aValue
!

parameters
	parameters isNil ifTrue: [parameters := OrderedCollection new].
	^parameters
!

parameters: aValue

	parameters := aValue
!

prefix
	prefix isNil ifTrue: [prefix := self class defaultPrefix].
	^prefix
!

prefix: aValue

	prefix := aValue
!

targetObjectURI
	targetObjectURI isNil ifTrue: [targetObjectURI := ''].
	^targetObjectURI
!

targetObjectURI: aString
	targetObjectURI := SoPortableUtil relativeURIFrom: aString.
!

transport
	transport isNil ifTrue: [transport := #http].
	^transport
!

transport: aValue

	transport := aValue
! !

!SoapPRCElement methodsFor:'actions-encoding'!

encoder: aValue 
	parameters notNil
		ifTrue: [self parameters
				do: [:each | each encoder: aValue]].
	super encoder: aValue.
	
!

encoding: aSymbol
	parameters notNil
		ifTrue: [self parameters
				do: [:each | each encoding: aSymbol]].
	super encoding: aSymbol.
!

encodingStyleURI: aURIString
	parameters notNil
		ifTrue: [self parameters
				do: [:each | each encodingStyleURI: aURIString]].
	super encodingStyleURI: aURIString.
!

encodingStyleURIString: anEncodingStyleURI 
	parameters notNil
		ifTrue: [self parameters
				do: [:each | each encodingStyleURIString: anEncodingStyleURI]].
	super encodingStyleURIString: anEncodingStyleURI.
!

encodingStyleURIs: aValue
	parameters notNil
		ifTrue: [self parameters
				do: [:each | each encodingStyleURIs: aValue]].
	super encodingStyleURIs: aValue.
	
! !

!SoapPRCElement methodsFor:'building context'!

buildContext
	| ctx | 
	ctx := SoapContext new.
	ctx targetObjectURI: self targetObjectURI.
	^ctx
! !

!SoapPRCElement methodsFor:'building envelope'!

buildEnvelope
        | env |
        env := SoapEnvelope new.
        env declareEncodingIn: SoapConstants soapEnvPrefix uri: self encodingStyleURIString.
        env declareTypicalNamespaces.
        self optionHeaderProperties isEmpty
                ifFalse: [ self buildSoapHeader: env addHeader.].
        env addBody.
        env body declareEncodingIn: SoapConstants soapEnvPrefix uri:self encodingStyleURIString.
        self buildSoapBody: env body.
        ^env
!

buildSoapBody: aSoapBody
	"Default do nothing"
	^aSoapBody
	
!

buildSoapHeader: aSoapHeader
	"Default do nothing"
	^aSoapHeader
	
	
! !

!SoapPRCElement methodsFor:'factory'!

xmlElementClass
	^SoapWrapElement
! !

!SoapPRCElement methodsFor:'testing'!

isCall
	^false
!

isResponse
	^false
!

isVariable
	^false
! !

!SoapPRCElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapPRCElement.st,v 1.3 2006/07/24 17:52:41 stefan Exp $'
! !

SoapPRCElement initialize!
