"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

SoapPRCElement subclass:#SoapResponse
	instanceVariableNames:'returnValue outparameters fault exceptionBuilder'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-RPC'
!

SoapResponse comment:'I represent a SOAP RPC response.

---
MU 5/12/2002 00:47'
!


!SoapResponse class methodsFor:'instance creation'!

fromSoapMessage: aSoapMessage
	^self new initialize: aSoapMessage
	
! !

!SoapResponse methodsFor:'accessing'!

exceptionBuilder
	exceptionBuilder isNil ifTrue: [exceptionBuilder := SoapExceptionBuilder new].
	^exceptionBuilder
!

exceptionBuilder: aSoapExceptionBuilder
	exceptionBuilder := aSoapExceptionBuilder
!

fault
	fault isNil ifTrue: [fault := self envelope body fault].
	^fault
!

methodName
        (self isFault not
                        and: [methodName isNil])
                ifTrue: [
                        methodName := self envelope body bodyEntries first name.
                        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                            (methodName startsWith:'Response') ifTrue:[
                                methodName := methodName copyFrom: 'Response' size + 1
                            ]
                        ] ifFalse:[
                            methodName := methodName copyReplaceAll: 'Response' with: ''
                        ]
                ].
        ^ methodName
!

targetObjectURI
	targetObjectURI isNil ifTrue: [targetObjectURI := self context targetObjectURI ].
	^targetObjectURI
!

transport
	transport isNil ifTrue: [transport := self context at: #transport ifAbsent:[]].
	^transport
! !

!SoapResponse methodsFor:'actions'!

docReturnValue
        self isFault ifTrue:[^nil].
        returnValue isNil ifTrue: [returnValue := self buildDocReturnValue].
        ^returnValue
!

isFault
	^self fault notNil
!

outparameters
	outparameters isNil ifTrue: [outparameters := self buildOutparameters].
	^outparameters
!

raiseException
        |ex|

        self isFault ifTrue: [
            ex := self buildException.
            (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                ex raiseSignal
            ] ifFalse:[
                ex signal
            ].
        ].
!

returnValue
	self isFault ifTrue:[^nil].
	returnValue isNil ifTrue: [returnValue := self buildReturnValue].
	^returnValue
! !

!SoapResponse methodsFor:'initialization & release'!

initialize: aSoapMessage
	self envelope: aSoapMessage envelope.
	self context: aSoapMessage context
! !

!SoapResponse methodsFor:'printing'!

printOn: aStream
        aStream nextPutAll: 'a ', self class name.
        aStream nextPutAll: ' ['.
        aStream nextPutAll: ' isFault= ', self isFault printString.
        aStream nextPutAll: ' methodName= ', self methodName printString.
        self isFault ifTrue:[
            aStream nextPutAll: ' faultString= ', self fault faultstring.            
        ].
        aStream nextPutAll: ' ]'.
! !

!SoapResponse methodsFor:'private'!

buildDocReturnValue
        | firstBodyEntry |
        firstBodyEntry := self envelope body bodyEntries first.

        ^ self extractDecodedValueFrom: firstBodyEntry
!

buildException
	
	^self exceptionBuilder buildExceptionFrom: self fault
!

buildOutparameters
	| retValues sz outparams |
	retValues := self envelope body bodyEntries.
	sz := retValues size.
	sz <= 1
		ifTrue: [^ #()].
	outparams := OrderedCollection new.
	2
		to: sz
		do: [:idx | outparams
				add: (self
						extractDecodedValueFrom: (retValues at: idx))].
	^outparams
!

buildReturnValue
        | firstBodyEntry retValueElem children |
        firstBodyEntry := self envelope body bodyEntries first.

        retValueElem := firstBodyEntry findChildNamed: (SoapConstants returnValueAccessor).
        retValueElem isNil ifTrue: [
            children := firstBodyEntry children.
            retValueElem := children first. "Soap1.1?"
        ].
        ^self extractDecodedValueFrom: retValueElem
!

extractDecodedValueFrom: aSoapWrapElement
	| decoder |
	aSoapWrapElement isNil ifTrue:[^nil].
	decoder := SoapDecoderFactory createFrom: aSoapWrapElement encodingStyleURIs.
	^decoder decodeXmlElement: aSoapWrapElement
! !

!SoapResponse methodsFor:'testing'!

isResponse
	^true
! !

!SoapResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapResponse.st,v 1.13 2006/07/24 17:54:22 stefan Exp $'
! !
