"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapService
	instanceVariableNames:'signature selector implementor targetObjectURI'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Server'
!


!SoapService class methodsFor:'instance creation'!

implementor: anObject selector: aSymbol 
	^ self new 
		 implementor: anObject;
		 selector: aSymbol
! !

!SoapService methodsFor:'accessing'!

implementor

	^implementor
!

implementor: aValue

	implementor := aValue
!

selector

	^selector
!

selector: aValue

	selector := aValue
!

signature
	
	^signature
!

signature: aValue

	signature := aValue
!

targetObjectURI
	targetObjectURI isNil ifTrue: [targetObjectURI := ''].
	^targetObjectURI
!

targetObjectURI: aString
	targetObjectURI := SoPortableUtil relativeURIFrom: aString.
! !

!SoapService methodsFor:'actions'!

getSignatureNamed: aSignatureName 
	^ (self signature name = aSignatureName)
		ifTrue:[self signature]
		ifFalse: [SoapNoSuchService signal: aSignatureName]
!

invoke: aSoapSignature
        | registeredSig values |
        registeredSig := self getSignatureNamed: aSoapSignature name.
        registeredSig verify: aSoapSignature.
        values := registeredSig paramNames
                                collect: [:each | aSoapSignature valueAt: each ifAbsent:[nil]].
        ^ self implementor perform: self selector withArguments: values
!

name
	"##currently just ignore namespace##"
	^self signature name
!

pathName
	"##currently just ignore namespace##"
	^ self targetObjectURI isEmpty
		ifTrue: [self name]
		ifFalse: [self targetObjectURI , ':' , self name]
!

wrapResult:result serviceHandler:aServiceHandler
    "create a SOAP envelope with result.
     Default is to generate a RPC response.
     Subclasses may redefine this."

    ^ aServiceHandler wrapResult:result
! !

!SoapService methodsFor:'printing'!

printOn: aStream
        aStream nextPutAll:(self classNameWithArticle , '{ ').
        self signature isNil ifFalse: [aStream nextPutAll: 'signature: ', self signature printString].
        self targetObjectURI isEmpty ifFalse: [aStream nextPutAll: ' targetObjectURI: ', self targetObjectURI printString].
        self implementor isNil ifFalse: [aStream nextPutAll: ' implementor: ', self implementor printString].
        self selector isNil ifFalse: [aStream nextPutAll: ' selector: ', self selector printString].
        aStream nextPutAll: '} '.
! !

!SoapService methodsFor:'validation'!

validateSignature:aSoapEnvelope serviceHandler:aServiceHandler
    "nothing checked here"
! !

!SoapService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapService.st,v 1.5 2006/07/24 17:52:53 stefan Exp $'
! !
