"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapServiceHandler
	instanceVariableNames:'accessLock serviceHome envelope context responseEncoder
		faultDetailBuilder inInvocation'
	classVariableNames:'DebuggingErrors DebugSOAP'
	poolDictionaries:''
	category:'SOAP-Server'
!

SoapServiceHandler class instanceVariableNames:'defaultResponseEncoder handlerDict useResponseEncoder'

"
 No other class instance variables are inherited by this class.
"
!


!SoapServiceHandler class methodsFor:'accessing'!

defaultResponseEncoder
	defaultResponseEncoder isNil ifTrue: [defaultResponseEncoder := SoapEncoder new].
	^defaultResponseEncoder
!

defaultResponseEncoder: aSoapAbstractEncoder
	defaultResponseEncoder := SoapAbstractEncoder
!

handlerDict
	handlerDict isNil ifTrue: [handlerDict := IdentityDictionary new].
	^handlerDict
!

handlerDict: aDictionary
	handlerDict := aDictionary
!

useResponseEncoder
	useResponseEncoder isNil ifTrue: [useResponseEncoder := false].
	^useResponseEncoder
!

useResponseEncoder: aBoolean
	useResponseEncoder := aBoolean
! !

!SoapServiceHandler class methodsFor:'actions'!

default
	^self named: #default
!

deleteNamed: aSymbol
	^self handlerDict removeKey: aSymbol ifAbsent: []
	
!

named: aSymbol
	^self handlerDict at: aSymbol ifAbsentPut: [self new]
	
! !

!SoapServiceHandler class methodsFor:'class initialization'!

initialize
	"SoapServiceHandler initialize"
	defaultResponseEncoder := handlerDict := nil.
	
	
!

new

    ^ self basicNew initialize
! !

!SoapServiceHandler methodsFor:'accessing'!

context
	^context
!

context: aValue

	context := aValue
!

envelope
	^envelope
!

envelope: aValue

	envelope := aValue
!

faultDetailBuilder
	faultDetailBuilder isNil ifTrue:[faultDetailBuilder := SoapFaultDetailBuilder new].
	^faultDetailBuilder
!

faultDetailBuilder: aValue

	faultDetailBuilder := aValue
!

responseEncoder
	responseEncoder isNil ifTrue:[responseEncoder := self class defaultResponseEncoder].
	^responseEncoder
!

responseEncoder: aValue

	responseEncoder := aValue
!

serviceHome
	serviceHome isNil ifTrue:[serviceHome := SoapServiceHome new].
	^serviceHome
!

serviceHome: aValue

	serviceHome := aValue
! !

!SoapServiceHandler methodsFor:'actions'!

dispatch:aSoapEnvelope with:aSoapContext 
    |env|

    accessLock 
        critical:[
            inInvocation := false.
            self envelope:aSoapEnvelope.
            self context:aSoapContext.
            env := Error 
                        handle:[:ex | 
                            UserPreferences current soapErrorDebugging ifTrue:[
                                ex reject
                            ].
                            DebugSOAP == true ifTrue:[
                                Debugger enter
                            ].
                            ex messageText:(ex description).
                            self breakPoint:#cg info:('dispatch-exception:' , ex description).
                            SoapErrorLog write:ex.
                            ex return:(self wrapError:ex)
                        ]
                        do:[ self invoke:aSoapEnvelope. ].
        ].
    ^ env

    "Modified: / 12-07-2011 / 17:44:30 / cg"
    "Modified (comment): / 08-08-2011 / 12:08:01 / sr"
!

encoding: aSymbol
	| uris |
	uris := Array with: (SoapEncodingStyleURIMap at: aSymbol).
	^self responseEncoder: (SoapEncoderFactory createFrom: uris).  
! !

!SoapServiceHandler methodsFor:'as yet unclassified'!

wrapResult: aResultObject using: aSoapEnvelope 
        |  methodElem resultTagName prefs nsPref nsURI resultMethodElem encoder respElem |
        methodElem := aSoapEnvelope body bodyEntries first.
        resultTagName := methodElem name , 'Response'.
        prefs := methodElem declaredNamespacePrefixes asArray.
        prefs isEmpty ifTrue:[ self error:'missing declaredNamespacePrefix'].
        nsPref := prefs first.
        nsURI := methodElem declaredNamespaceURIFor: nsPref.
        resultMethodElem := self xmlElementClass named: resultTagName.
        resultMethodElem declareNamespace: nsPref uri: nsURI.
        
        encoder := self selectResponseEncoderFor: methodElem. 
    
        respElem := encoder
                                encodeSoapVariable: (SoapVariable name: (SoapConstants returnValueAccessor) value: aResultObject).
        resultMethodElem addChild: respElem.

        aSoapEnvelope body clearBodyEntries.
        aSoapEnvelope body addBodyEntry: resultMethodElem.
        ^ aSoapEnvelope
! !

!SoapServiceHandler methodsFor:'factory'!

xmlElementClass
	^SoapWrapElement
! !

!SoapServiceHandler methodsFor:'initialize-release'!

initialize
        serviceHome := context := responseEncoder := faultDetailBuilder := nil.
        inInvocation := false.
        accessLock := Semaphore forMutualExclusion.
!

unregister
! !

!SoapServiceHandler methodsFor:'private'!

createSignature: aSoapEnvelope
	| mElem name argNames sig |
	mElem := aSoapEnvelope body bodyEntries first.
	name := mElem shortName asSymbol. 
	argNames := mElem children
				collect: [:each | each shortName asSymbol].
	sig := SoapServiceSignature name: name paramNames: argNames.
	mElem children do: [:each | sig valueAt: each shortName asSymbol put: (self extractDecodedValueFrom: each ) ].
	^sig
!

extractDecodedValueFrom: aSoapWrapElement
	| decoder |
	decoder := SoapDecoderFactory createFrom: aSoapWrapElement encodingStyleURIs.
	^decoder decodeXmlElement: aSoapWrapElement
!

findService: aSoapEnvelope 
	| mElement nm key  serv |
	mElement := aSoapEnvelope body bodyEntries first.
	nm := mElement shortName.
	key := self context targetObjectURI isEmpty
		ifTrue: [nm]
		ifFalse: [self context targetObjectURI , ':' , nm].
	serv := self serviceHome at: key.
	^ serv
!

invoke: aSoapEnvelope
        
        | service signature ret  env|

        service := self findService: aSoapEnvelope.
        signature := self createSignature: aSoapEnvelope.

        SoapServiceHandlerQuery 
            answer:self
            do:[
                service validateSignature:aSoapEnvelope serviceHandler:self. "/ possibly raises a SignatureCheckFailed exception
                inInvocation := true.
                ret := service invoke: signature.
                env := service wrapResult:ret serviceHandler:self.
            ].
        ^ env
!

invoke: aSoapEnvelope with: aSoapContext
	
	"##obsolete##" "for testing"
	self context: aSoapContext.
	^self invoke: aSoapEnvelope 
!

selectResponseEncoderFor: aMethodElement 
	^self class useResponseEncoder
		ifTrue: [self responseEncoder]
		ifFalse: [self context encodingStyleURIs isEmpty
				ifTrue: [SoapEncoderFactory createFrom: aMethodElement encodingStyleURIs]
				ifFalse: [SoapEncoderFactory createFrom: self context encodingStyleURIs]]
!

wrapError: anError
        |body fault errorName errorClass isClientFault |

        body := self envelope body.
        body clearBodyEntries.
        fault := body addFault.
        self faultDetailBuilder buildSoapFaultDetailOn: fault from: anError.

        errorClass := anError class.
        Smalltalk isSmalltalkX ifTrue:[
            isClientFault := errorClass == SoapNoSuchService
                            or:[ errorClass == SoapInvalidParamName 
                            or:[ errorClass == SoapDecodingError
                            or:[ SoapSignatureCheckFailure notNil
                                 and:[ SoapSignatureCheckFailure accepts:errorClass] ]]]
        ] ifFalse:[
            isClientFault := errorClass == SoapNoSuchService
                            or:[ errorClass == SoapInvalidParamName
                            or:[ errorClass == SoapDecodingError
                            or:[ errorClass ==  SoapSignatureCheckFailure
                            or:[ errorClass ==  SoapMissingSignature ]]]]
        ].

        isClientFault ifTrue:[
            fault beClient
        ] ifFalse:[
            fault beServer
        ].
"/        Smalltalk isSmalltalkX ifTrue:[
"/            errorName := anError class nameWithoutPrefix.
"/        ] ifFalse:[
"/            errorName := anError class name.
"/        ].
"/        (#(#SoapNoSuchService #SoapInvalidParamName #SoapDecodingError
"/           #SoapMissingSignature) includes: errorName)
"/                ifTrue: [fault beClient "client" ]
"/                ifFalse: [fault beServer "server" ].

        inInvocation ifFalse:[
            fault faultCode:(fault faultCode , '.Authentication').
        ].

        ^ self envelope

    "Modified: / 24-07-2007 / 22:24:25 / cg"
!

wrapResult: aResultObject
        |  methodElem resultTagName prefs nsPref nsURI resultMethodElem encoder respElem |
        methodElem := self envelope body bodyEntries first.
        resultTagName := methodElem name , 'Response'.
        nsPref := methodElem prefixName.
"/        prefs := methodElem declaredNamespacePrefixes asArray.
"/        prefs isEmpty ifTrue:[ self error:'missing declaredNamespacePrefix'].
"/        nsPref := prefs first.
        nsURI := methodElem declaredNamespaceURIFor: nsPref.
        resultMethodElem := self xmlElementClass named: resultTagName.
        resultMethodElem declareNamespace: nsPref uri: nsURI.
        
        encoder := self context encodingStyleURIs isEmpty ifFalse: [SoapEncoderFactory createFrom: self context encodingStyleURIs]
                                ifTrue: [ SoapEncoderFactory createFrom: methodElem encodingStyleURIs].
    
        respElem := encoder
                                encodeSoapVariable: (SoapVariable name: (SoapConstants returnValueAccessor) value: aResultObject).
        resultMethodElem addChild: respElem.

        self envelope body clearBodyEntries.
        self envelope body addBodyEntry: resultMethodElem.
        ^ self envelope
! !

!SoapServiceHandler methodsFor:'service register/unregister'!

add: aSoapService
	"shortcut for global service"
	^ self at: aSoapService pathName put: aSoapService 
!

at: aServicePathname
	^ self serviceHome at: aServicePathname 
!

at: aServicePathname put: aSoapService
	^ self serviceHome at: aServicePathname put: aSoapService 
!

remove: aSoapService
	"shortcut for global service"
	^ self serviceHome removeKey: aSoapService pathName
!

removeServiceNamed: aSoapServiceName
        "shortcut for global service"
        ^ self serviceHome removeKey: aSoapServiceName
! !

!SoapServiceHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapServiceHandler.st,v 1.22 2011/08/08 11:20:52 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapServiceHandler.st,v 1.22 2011/08/08 11:20:52 sr Exp $'
! !

SoapServiceHandler initialize!
