"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapSqtpHandler
	instanceVariableNames:'status port logging connectionHandler'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Sqtp-Server'
!

SoapSqtpHandler class instanceVariableNames:'defaultInstance showLog'

"
 No other class instance variables are inherited by this class.
"
!


!SoapSqtpHandler class methodsFor:'instance creation'!

default
	"SoapSqtpServer default"
	defaultInstance ifNil:[defaultInstance _ self new].
	^defaultInstance
! !

!SoapSqtpHandler class methodsFor:'setting'!

showLog
	showLog isNil ifTrue:[showLog := false].
	^showLog
!

showLog: aBoolean
	showLog := aBoolean
! !

!SoapSqtpHandler methodsFor:'accessing'!

logging
	logging isNil ifTrue: [logging _ self class showLog].
	^logging
!

logging: aBoolean
	logging _ aBoolean
!

port
	port isNil ifTrue: [port _ 4649].
	^port
!

port: anInteger
	port _ anInteger
! !

!SoapSqtpHandler methodsFor:'actions'!

start
        Smalltalk isSqueak ifTrue:[
            Socket initializeNetwork.
        ].

        self startConnectionHandler.
        status := #running.
        self log: self class name, '##Running: ' , self port printString
!

stop
	self stopConnectionHandler.
	status _ #stoped.
	self log: self class name, '##Stoped!!'
! !

!SoapSqtpHandler methodsFor:'actions-server'!

process: rawRequest
	| soapContext soapEnvelope handlerName returnEnvelope targetObjectURI |
        
	targetObjectURI _ rawRequest first.
	soapEnvelope _ rawRequest last.
        
	self class showLog ifTrue: [Transcript cr; show: '##' , super printString ,' Server:Request:{ ', soapEnvelope printString, '} '. ].
        
	soapContext := SoapContext new.
	soapContext targetObjectURI: targetObjectURI. "maybe method namespace"
        
	handlerName := #default.

	 returnEnvelope := soapEnvelope hasFault 
		ifTrue: [ soapEnvelope ] 
		ifFalse: [ (SoapServiceHandler named: handlerName) dispatch: soapEnvelope with: soapContext.].

	self class showLog ifTrue: [Transcript cr; show: '##' , super printString ,' Server:Return:{ ', returnEnvelope printString, '} '. ].
        
	^returnEnvelope
        
        
        
	
!

processSocket: aSocket 
	"process a new open socket"
	[| request result | 
	request := SoapSqtpProtocol getObjectFrom: aSocket.
	result := self process: request.
	SoapSqtpProtocol sendReturnObject: result by: aSocket]
		ensure: [aSocket closeAndDestroy]
! !

!SoapSqtpHandler methodsFor:'callback'!

value: aSocket 
	"this method is invoked by ConnectioHandler when a new connection is  
	established"
	[self processSocket: aSocket]
		forkAt: Processor lowIOPriority
! !

!SoapSqtpHandler methodsFor:'logging'!

log: aString 
	self logging
		ifTrue: [Transcript cr; show: aString]
! !

!SoapSqtpHandler methodsFor:'private'!

startConnectionHandler
    connectionHandler isNil ifTrue: [
        ConnectionHandler notNil ifTrue:[
            connectionHandler _ ConnectionHandler
                                    onPort: self port
                                    handler: self
                                    priority: Processor highIOPriority.
            connectionHandler start
        ]
    ]

    "Modified: / 10-11-2006 / 11:55:09 / cg"
!

stopConnectionHandler
	connectionHandler isNil
		ifFalse: [connectionHandler stop.
			connectionHandler _ nil]
! !

!SoapSqtpHandler methodsFor:'testing'!

isRunning
	^status == #running
! !

!SoapSqtpHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapSqtpHandler.st,v 1.3 2006/11/10 10:54:58 cg Exp $'
! !
