"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapSqtpProtocol
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Sqtp'
!


!SoapSqtpProtocol class methodsFor:'actions'!

getObjectFrom: socket
	| data object |
	data := self getDataFrom: socket.
	object _ self objectFromBinary: data.
	^ object
	
!

sendRequestObject: object by: socket
	^self sendObject: object by: socket command: #REQUEST
	
!

sendReturnObject: object by: socket
	^self sendObject: object by: socket command: #RETURN
	
! !

!SoapSqtpProtocol class methodsFor:'private'!

binaryFromObject: anObject 
	| file |
	"Currently ReferenceStream only"
	file := RWBinaryOrTextStream
				on: (ByteArray new: 5000).
	file binary.
	(ReferenceStream on: file)
		nextPut: anObject.
	file close.
	^ file contents
!

getDataFrom: socket
	| encoded firstRead headerDivider headerBufferStream header bodySize |
        
	socket isNil ifTrue: [^nil].

	firstRead := socket getData.
	headerDivider := firstRead indexOf: $:.
	headerDivider = 0 ifTrue: [self error: 'invalid header'].
	headerBufferStream := firstRead readStream.
	header := headerBufferStream next: headerDivider.
	bodySize := (header copyFrom: (header indexOf: $*) + 1 to: header size - 1 ) asNumber. 
	encoded := WriteStream on: (String new: bodySize).
	encoded nextPutAll: headerBufferStream upToEnd.
        
	[socket isConnected and:[bodySize > encoded position]]
		whileTrue: [encoded nextPutAll: socket getData].
	^encoded contents
!

objectFromBinary: bytes 
	"Currently ReferenceStream only"
	| object |
	object _ ReferenceStream unStream: bytes.
	^ object
!

sendData: data by: socket in: encoder command: command
	| headerStream header sentSize toSendSize  sendData |
        
	headerStream := WriteStream on: (String new: 80).
	headerStream nextPutAll: 'SQTP01'; space.
	headerStream nextPutAll: command; space.
	headerStream nextPutAll: '#ReferenceStream'; space. "Currently encoder is ReferenceStream only"
	headerStream nextPutAll: '*', data size printString, ':'.
	header := headerStream contents.

	sentSize := 0.
	toSendSize := header size + data size.
	sendData := header, data.
	[sentSize < toSendSize]
		whileTrue: [sentSize := sentSize
						+ (socket sendSomeData: sendData startIndex: sentSize + 1)]
!

sendObject: object by: socket command: command
	| data |
	data := self binaryFromObject: object.
	^self sendData: data by: socket in: #ReferenceStream command: command
! !

!SoapSqtpProtocol class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapSqtpProtocol.st,v 1.1 2002/11/11 09:52:18 cg Exp $'
! !
