"{ Package: 'stx:goodies/soap' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapTransportSetup
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Sqtp-Server'
!

SoapTransportSetup class instanceVariableNames:'defaultPort handler'

"
 No other class instance variables are inherited by this class.
"
!


!SoapTransportSetup class methodsFor:'accessing'!

handler
        handler isNil ifTrue: [ handler := self defaultHandler ].
        ^handler
! !

!SoapTransportSetup class methodsFor:'actions'!

start
    |handler|

    self stop.
    (handler := self handler) isNil ifTrue:[
        ^ nil
    ].
    ^ handler start
!

startOn: port
        self stop.
        self handler port: port.
        ^self handler start
!

stop
        handler isNil
                ifFalse: [self handler stop].
        handler := nil
! !

!SoapTransportSetup class methodsFor:'class initialization'!

initialize
        "SoapTransportSetup initialize"
        "SoapHttpSetup initialize"
        "/ self name == #SoapTransportSetup
        self == SoapTransportSetup
                ifTrue: [
                    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                        ObjectMemory addDependent:self
                    ] ifFalse:[
                        Smalltalk addToStartUpList: self.
                        Smalltalk addToShutDownList: self
                    ]
                ]
                ifFalse: [
                    self registerInTransportMap.    
                    self release
                ]
!

initializeAfterLoad
    self registerInTransportMap
!

initializeAll
        "SoapTransportSetup initializeAll"

        self subclassesDo: [:each | each initialize]
!

obsolete
        "/ self name == #SoapTransportSetup
        self == SoapTransportSetup
                ifTrue: [
                    self stopAll.
                    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                        ObjectMemory removeDependent:self
                    ] ifFalse:[
                        Smalltalk removeFromStartUpList: self.
                        Smalltalk removeFromShutDownList: self
                    ]
                ]
                ifFalse: [SoapTransportMap removeSetupAt: self transportName asSymbol].
        super obsolete
!

registerInTransportMap

        "/ self name == #SoapTransportSetup
        self == SoapTransportSetup
                ifFalse: [SoapTransportMap setupAt: self transportName asSymbol put: self].
! !

!SoapTransportSetup class methodsFor:'constants'!

transportName
        "self subclassResponsibility"
        ^#noName
! !

!SoapTransportSetup class methodsFor:'defaults'!

defaultHandler
        self subclassResponsibility
!

defaultPort
        self subclassResponsibility.
!

defaultPort: anInteger
        defaultPort := anInteger
! !

!SoapTransportSetup class methodsFor:'start up/shut down'!

shutDown
        self stopAll
!

startUp
        self startAll
!

update:something with:aParameter from:changedObject
    something == #returnFromSnapshot ifTrue:[
        self startUp.
        ^ self
    ].
    something == #aboutToQuit ifTrue:[
        self shutDown.
        ^ self
    ].
    super update:something with:aParameter from:changedObject
! !

!SoapTransportSetup class methodsFor:'utility'!

defaultPortAt: transportName 
        "SoapTransportSetup defaultPortAt: #http"
        | setUp |
        setUp := SoapTransportMap setupAt: transportName.
        ^ setUp defaultPort
!

defaultPortAt: transportName put: portNumber
        "SoapTransportSetup defaultPortAt: #http put: 8824"
        | setUp |
        setUp := SoapTransportMap setupAt: transportName.
        setUp isNil
                ifTrue: [SoapTransportMap initialize.
                        setUp := SoapTransportMap setupAt: #http].
        ^ setUp defaultPort: portNumber
!

release
        "SoapTransportSetup release"
        self stop.
        defaultPort _ handler _ nil
!

restartAt: transportName port: portNumber 
        "SoapTransportSetup restartAt: #http port: 8824"
        | setUp |
        setUp _ SoapTransportMap setupAt: transportName.
        setUp isNil
                ifFalse: [setUp startOn: portNumber]
!

runningTransportDescriptions
        "SoapTransportSetup runningTransportDescriptions"
        ^ self subclasses
                select: [:each | each handler isRunning] thenCollect:[:each | (each transportName, ':', each defaultPort printString) asSymbol]
!

runningTransportNames
        "SoapTransportSetup runningTransportNames"
        ^ self subclasses
                select: [:each | each handler isRunning] thenCollect:[:each | each transportName]
!

start: transportNames 
        "SoapTransportSetup start: #(http) "
        transportNames
                do: [:each | 
                        | setUp | 
                        setUp _ SoapTransportMap setupAt: each.
                        setUp isNil
                                ifFalse: [setUp stop; start]]
!

startAll
        "SoapTransportSetup startAll"
        "/ self name == #SoapTransportSetup
        self == SoapTransportSetup
                ifTrue: [self subclassesDo: [:each | each start]]
!

startOnly: transportNames 
        "SoapTransportSetup startOnly: #(http) "
        | stops |
        stops _ SoapTransportMap setupDict keys removeAll: transportNames; yourself.
        self stop: stops.
        self start: transportNames
!

stop: transportNames 
        "SoapTransportSetup stop: #(http)"
        transportNames
                do: [:each | 
                        | setUp | 
                        setUp _ SoapTransportMap setupAt: each.
                        setUp isNil
                                ifFalse: [setUp stop]]
!

stopAll
        "SoapTransportSetup stopAll"
        "/ self name == #SoapTransportSetup
        self == SoapTransportSetup
                ifTrue: [self subclassesDo: [:each | each stop]]
! !

!SoapTransportSetup class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/SoapTransportSetup.st,v 1.9 2009/03/02 17:10:29 stefan Exp $'
! !

SoapTransportSetup initialize!
