"{ Package: 'stx:goodies/soap/examples' }"

"{ NameSpace: SOAP }"

Object subclass:#GoogleClient
	instanceVariableNames:'licenseKey doGetCachedPage doSpellingSuggestion doGoogleSearch'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-ExampleClients'
!

GoogleClient class instanceVariableNames:'licenseKey callEntry targetObjectURI namespace host port'

"
 No other class instance variables are inherited by this class.
"
!

GoogleClient comment:'I am an example that accesses Google Web APIs by SoapOpera.

About Google Web API:
http://www.google.com/apis/index.html

---
MU 5/12/2002 00:51'
!

!GoogleClient class methodsFor:'documentation'!

documentation
"
    I am an example that accesses the Google Web APIs via Soap.
    This service provides spelling correction, google search and cached page access.

    About Google Web API:
        http://www.google.com/apis/index.html


    [author:]
        original: MU 5/12/2002 00:51
        Claus Gittinger (port to ST/X); Jan 2003
"
!

howToUse
        "##GoogleClient is an example that accesses Google Web APIs by SoapOpera.##"
        
        "1: Preparations
             In order to call Google Web APIs, you need a Google Account license key. (It is free).
             The license key is obtained from: http://www.google.com/apis/index.html
             Then, please set the license key by: 
                GoogleClient licenseKey: 'yourkey' 
             or deposit it in the UserPreferences under the key 'SOAP::GoogleClient.licenceKey'
        "
        
        "2: Usages (explore or print it)
             GoogleClient new search: 'Google Web APIs Reference'. 
             GoogleClient new spellingSuggestionOf: 'Smmalltlaak and Soaap'.
             GoogleClient new cachedPageOf: 'http://www.squeak.org'
        "
! !

!GoogleClient class methodsFor:'accessing'!

callEntry
	callEntry isNil ifTrue: [ callEntry := SoapCallEntry tcpHost: self host port: self port].
	^callEntry
!

callEntry: aValue

	callEntry := aValue
!

host
	host isNil ifTrue: [host := 'api.google.com'].
	^host
!

host: aValue

	host := aValue
!

licenseKey
    "get the licence key wither from the user prefs, or from a file ~/.googleLicenceKey"

    |s|

    licenseKey isNil ifTrue:[
        licenseKey := UserPreferences current at:(self class name , '.' , 'licenceKey') ifAbsent:nil.
        licenseKey isNil ifTrue:[
            s := (Filename homeDirectory construct:'.googleLicenceKey') readStreamOrNil.
            s notNil ifTrue:[
                licenseKey := s nextLine.
                s close.
            ] ifFalse:[
                licenseKey := Dialog request:'Please insert the google licence key:\(Save settings in Launcher to make it persistent)\\If you don''t know about that key, read the SOAP::GoogleClient classes documentation.' withCRs.
                licenseKey isEmptyOrNil ifTrue:[
                    licenseKey := nil
                ] ifFalse:[
                    UserPreferences current at:(self class name , '.' , 'licenceKey') put:licenseKey.
                ]
            ]
        ].
    ].
    ^ licenseKey

    " 
     licenseKey := 'gAE2CMNBd1xA+bRPXOmLkmT9+X8Y3bwG'. 'your Google API registration key'
    "
!

licenseKey: aString
	"GoogleClient licenseKey: 'your key'"
	licenseKey := aString
!

namespace
	namespace isNil ifTrue: [namespace := 'urn:GoogleSearch'].
	^namespace
!

namespace: aValue

	namespace := aValue
!

port
	port isNil ifTrue: [port := 80].
	^port
!

port: aValue

	port := aValue
!

targetObjectURI
	targetObjectURI isNil ifTrue: [targetObjectURI := 'http://api.google.com/search/beta2'].
	^targetObjectURI
!

targetObjectURI: aValue

	targetObjectURI := aValue
! !

!GoogleClient class methodsFor:'class initialization'!

initialize
        "GoogleClient initialize"
        "GoogleClient licenseKey: 'your key' (see the 'documentation' category first)"

        licenseKey := nil.
        "/ licenseKey := 'your key here'.
        
        callEntry := targetObjectURI := namespace := host := port := nil.
! !

!GoogleClient methodsFor:'accessing'!

callEntry
	^self class callEntry
!

doGetCachedPage
	doGetCachedPage isNil ifTrue: [doGetCachedPage := self buildDoGetCachedPage].
	^doGetCachedPage
!

doGoogleSearch
	doGoogleSearch isNil ifTrue: [doGoogleSearch := self buildDoGoogleSearch].
	^doGoogleSearch
!

doSpellingSuggestion
	doSpellingSuggestion isNil ifTrue: [doSpellingSuggestion := self buildDoSpellingSuggestion].
	^doSpellingSuggestion
!

licenseKey
	licenseKey isNil ifTrue: [licenseKey := self class licenseKey].
	^licenseKey
!

licenseKey: key
	licenseKey := key
! !

!GoogleClient methodsFor:'actions'!

cachedPageOf:urlString 
    |params resp|

    params := OrderedCollection with:self licenseKey.
    params add:urlString.
    resp := self doGetCachedPage invokeWithValues:params.
    ^ resp isFault 
        ifTrue:[ resp raiseException ]
        ifFalse:[ resp returnValue asString ]

    "
     GoogleClient new cachedPageOf: 'http://www.squeak.org'
    "
!

search:query 
    ^ self 
        search:query
        start:0
        maxResults:10

    "
     GoogleClient new search: 'Google Web APIs Reference'. 
    "
!

search:query start:start maxResults:maxResults 
    ^ self 
        search:query
        start:start
        maxResults:maxResults
        filter:true
        restrict:''
        safeSearch:false
        languageRestrict:''
!

search:query start:start maxResults:maxResults filter:filter restrict:restrict safeSearch:safeSearch languageRestrict:lr 
    ^ self 
        search:query
        start:start
        maxResults:maxResults
        filter:filter
        restrict:restrict
        safeSearch:safeSearch
        languageRestrict:lr
        inputEncoding:'utf8'
        outputEncoding:'utf8'
!

search:query start:start maxResults:maxResults filter:filter restrict:restrict safeSearch:safeSearch languageRestrict:lr inputEncoding:ie outputEncoding:oe 
    |params resp|

    params := OrderedCollection with:self licenseKey.
    params
        add:query;
        add:start;
        add:maxResults;
        add:filter;
        add:restrict;
        add:safeSearch;
        add:lr;
        add:ie;
        add:oe.
    resp := self doGoogleSearch invokeWithValues:params.
    ^ resp isFault 
        ifTrue:[ resp raiseException ]
        ifFalse:[ resp returnValue ]
!

spellingSuggestionOf:phrase 
    |key params resp|

    (key := self licenseKey) isNil ifTrue:[
        ^ nil
    ].
    params := OrderedCollection with:key.
    params add:phrase.
    resp := self doSpellingSuggestion invokeWithValues:params.
    ^ resp isFault 
        ifTrue:[ resp raiseException ]
        ifFalse:[ resp returnValue ]

    "
     GoogleClient new spellingSuggestionOf: 'Smmalltlaak and Soaap'.
    "
! !

!GoogleClient methodsFor:'private'!

buildDoGetCachedPage
	| call |
	call := self newCall.
	call methodName: 'doGetCachedPage'.
	call addParameterNamed: #key.
	call addParameterNamed: #url.
	^call
!

buildDoGoogleSearch
	| call |
	call := self newCall.
	call methodName: 'doGoogleSearch'.
	call addParameterNamed: #key.
	call addParameterNamed: #q. 
	call addParameterNamed: #start xsdType: 'xsd:int'.
	call addParameterNamed: #maxResults xsdType: 'xsd:int'.
	call addParameterNamed: #filter.
	call addParameterNamed: #restrict.
	call addParameterNamed: #safeSearch.
	call addParameterNamed: #lr.
	call addParameterNamed: #ie.
	call addParameterNamed: #oe.
	^call
!

buildDoSpellingSuggestion
	| call |
	call := self newCall.
	call methodName: 'doSpellingSuggestion'.
	call addParameterNamed: #key.
	call addParameterNamed: #phrase.
	^call
!

newCall
	| call |
	call := self callEntry newCall.
	call targetObjectURI: self class targetObjectURI.
	call namespace: self class namespace.
	^call
! !

!GoogleClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/GoogleClient.st,v 1.18 2010/03/08 21:27:05 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/GoogleClient.st,v 1.18 2010/03/08 21:27:05 cg Exp $'
! !

GoogleClient initialize!
