"{ Package: 'stx:goodies/soap/examples' }"

"{ NameSpace: SOAP }"

Object subclass:#HelloWorldServer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-ExampleClients'
!

!HelloWorldServer class methodsFor:'documentation'!

documentation
"
    Output a message on the transcript.
    Both a SOAP client and a server.

    To start the server:
        SOAP::HelloWorldServer start

    to invoke it:
        SOAP::HelloWorldServer invoke


    Also provided is an example for sending signed (secure) messages;
    here (in the example), a TestSigner is used; real world applications
    would use a true RSA_SHA1 Signer (See classes in the Soap-SEC caregory).

    [author:]
        Claus Gittinger; Jan 2003
"
! !

!HelloWorldServer class methodsFor:'examples'!

helloWithArg:aString On:hostName port:portNr
    |call response|

    call := (SoapCallEntry tcpHost:hostName port:portNr) newCall.
    call transport: #http.
    call methodName: 'helloWithArg'.
    call addParameter:(SoapVariable name:#aString value:aString).
    response := call invoke.
    "/^true.
    ^response isFault ifTrue: [response raiseException] ifFalse: [response returnValue] 

    "
     self startOnPort:8080.
     self helloWithArg:'test' On: 'localhost' port:8080
     self invoke.
    "
!

invoke
    ^ self invokeOnPort: SoapHttpSetup defaultPort

    "
     self start.
     self invoke.
    "
!

invokeOn:hostName port:portNr
    |call response|

    call := (SoapCallEntry tcpHost:hostName port:portNr) newCall.
    call transport: #http.
    call methodName: 'hello'.
    response := call invoke.
    ^response isFault ifTrue: [response raiseException] ifFalse: [response returnValue] 

    "
     self start.
     self invoke.
    "
!

invokeOnPort:aPortNr
    ^ self invokeOn:'localhost' port:aPortNr

    "
     self startOnPort:9000.
     self invokeOnPort:9000.
    "
!

invokeSigned
    "invoke the service in secure mode (send a signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    ^ self invokeSignedOn:'localhost' port:SoapHttpSetup defaultPort.

    "
     self start.
     self invoke.
     self invokeSigned.
    "

    "
     self startSecure.
     self invokeSigned.
     self invoke.
    "
!

invokeSignedOn:hostName port:portNr
    "invoke the service in secure mode (send a signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    |call response signatureHandler|

    signatureHandler := SignatureHandler new algorithm:(SoapDigest_DSA_SHA1 new) signer:(TestSigner new).

    call := (SoapCallEntry tcpHost:hostName port:portNr) newSignedCall.
    call signatureHandler:signatureHandler.
    call transport: #http.
    call methodName: 'helloWorld'.
    response := call invoke.
    ^response isFault ifTrue: [response raiseException] ifFalse: [response returnValue] 

    "
     self start.
     self invoke.
     self invokeSigned.
    "

    "
     self startSecure.
     self invokeSigned.
     self invoke.
    "
! !

!HelloWorldServer class methodsFor:'startup'!

defineService:serviceName withImplementor:inst selector:selector in:soapServer secure:secure
    |service|

    secure ifTrue:[
        service := SOAP::SoapSignedService implementor:inst selector:selector.
        service signatureHandler:(SOAP::SignatureHandler new signer:(SOAP::TestSigner new)).
    ] ifFalse:[
        service := SOAP::SoapService implementor:inst selector:selector.
    ].
    service signature: (SOAP::SoapServiceSignature name:serviceName).

    soapServer add:service.
    ^ service
!

defineServiceHandlersIn:soapServer secure:secure
    |inst service|

    inst := self new.

    service := self defineService:'hello' withImplementor:inst selector:#helloWorld in:soapServer secure:secure.

    service := self defineService:'helloWithArg' withImplementor:inst selector:#helloWorldWithArgument: in:soapServer secure:secure.
    service signature:(SoapServiceSignature name:'helloWithArg' paramNames:#( aString )).
!

soapEnabledHTTPServerOnPort:portNr
    "get an HTTPServer with SOAP enabled"

    |server|

    server := HTTPServer soapEnabledServerOnPort:portNr.
    server isNil ifTrue:[
        self error:'Cannot start HTTP server on port: ' , portNr printString
    ].
    ^ server
!

startOnPort:aPortNr secure:secure
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    |server|

    server := self soapEnabledHTTPServerOnPort:(aPortNr ? SOAP::SoapHttpSetup defaultPort).
    self defineServiceHandlersIn:(server soapServiceHandler) secure:secure.

    "
     self startOnPort:nil secure:false
    "
! !

!HelloWorldServer class methodsFor:'startup-public'!

start
    "startup the service"

    ^ self startOnPort:nil secure:false

    "
     self start

     self invoke
    "
!

startOnPort:portNr
    "startup the service"

    ^ self startOnPort:portNr secure:false

    "
     self startOnPort:9000

     self invokeOnPort:9000
    "
!

startSecure
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    ^ self startOnPort:nil secure:true

    "
     self startSecure
    "
!

startSecureOnPort:aPortNr
    "startup the service in secure mode (expects signed messages).
     Here (in the example), a dummy TestSigner is used - in a real world application,
     an RSA_SHA1 capapble signer would be used."

    ^ self startOnPort:aPortNr secure:true

    "
     self startSecure
    "
! !

!HelloWorldServer methodsFor:'soap-calls'!

helloWorld
    |serviceHandler keyInfo|

    Transcript showCR:'Hello world (it is now ' , Time now printString , ')'.

    " 
      the stuff below is not needed normally - it has been added
      for demonstration purposes only (how can a SOAP-method reflect on its caller, envelope etc.)
      (see more in SOAPServiceHandler)
    "

    serviceHandler := SoapServiceHandlerQuery query.
    serviceHandler isNil ifTrue:[
        Transcript showCR:'   Local (direct) call'.
    ] ifFalse:[
        keyInfo := serviceHandler context keyInfo.
        keyInfo notNil ifTrue:[
            Transcript showCR:'   KeyName    was: ' , keyInfo keyName value.
        ].
        Transcript showCR:'   SoapAction was: ' , (serviceHandler context soapAction) printString.
        Transcript showCR:'   URI was: ' , (serviceHandler context targetObjectURI) printString.
    ].
    ^ 123

    "
     SOAP::HelloWorldServer start
     SOAP::HelloWorldServer startSecure

     SOAP::HelloWorldServer new helloWorld.

     SOAP::HelloWorldServer invoke.
     SOAP::HelloWorldServer invokeSigned.
    "
!

helloWorldWithArgument:argument
    Transcript showCR:argument.

    "
     SOAP::HelloWorldServer start
     SOAP::HelloWorldServer startSecure

     SOAP::HelloWorldServer new helloWorld.

     SOAP::HelloWorldServer invoke.
     SOAP::HelloWorldServer invokeSigned.
    "
! !

!HelloWorldServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/HelloWorldServer.st,v 1.27 2005/12/01 15:24:24 cg Exp $'
! !
