"{ Package: 'stx:goodies/soap/examples' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapExampleClient
	instanceVariableNames:'host port targetObjectURI'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Example'
!

SoapExampleClient class instanceVariableNames:'hostAddress port targetObjectURI'

"
 No other class instance variables are inherited by this class.
"
!

SoapExampleClient comment:'Simple SOAP client example.

- Call services implemented in SoapExampleServiceImpl

---
mu 1/4/2002 23:05'
!


!SoapExampleClient class methodsFor:'accessing'!

hostAddress
        "the default host"

        ^hostAddress
!

hostAddress: aString
        "the default host"

        hostAddress := aString
!

port
        "the default port"

        ^port
!

port: anInteger
        "the default port"

        port := anInteger
!

targetObjectURI
        "the default targetObjectURI (URI in the HTTP-request)"

        ^targetObjectURI
!

targetObjectURI: aString
        "the default targetObjectURI (URI in the HTTP-request)"

        targetObjectURI := aString
! !

!SoapExampleClient class methodsFor:'class initialization'!

initialize
        "SoapExampleClient initialize" 

        hostAddress := 'localhost'.                      "/ the SOAP server
        port := SoapHttpSetup defaultPort.               "/ the HTTP-SOAP server
        targetObjectURI := ''                            "/ the URI in the request
! !

!SoapExampleClient methodsFor:'accessing'!

host
    ^ host ? self class hostAddress
!

host:something
    host := something.
!

hostAddress
    ^ self host
!

hostAddress:something
    self host:something.
!

port
    ^ port ? self class port
!

port:something
    port := something.
!

targetObjectURI
    ^ targetObjectURI ? self class targetObjectURI
!

targetObjectURI:something
    targetObjectURI := something.
! !

!SoapExampleClient methodsFor:'actions'!

callBrowseClassDefinition:aString 
    |call|

    call := SoapCall methodName:'browseClassDefinition' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#className value:aString).
    ^ call perform.

    "
     self new callBrowseClassDefinition:'Boolean'   
    "

    "
     self new 
        hostAddress:'nilpferd'; 
        port:(HTTPServer defaultPort);
        callBrowseClassDefinition:'Boolean'   
    "
!

callHelloWorld
    |call|

    call := SoapCall methodName:'helloWorld' tcpHost:(self host) port:(self port).
    ^ call perform.

    "
     self new callHelloWorld
    "

    "
     self new 
        hostAddress:'swikis.ddo.jp'; 
        port:8823;
        callHelloWorld      
    "
!

callHelloWorld_withFullCallSpecification
    "shows that we can set host, port, transport and targetObjectURI.
     If left undefined, these default to:
        host ............. localHost
        port ............. SoapHttpSetup defaultPort -> 8823
        transport ........ #http
        targetObjectURI .. ''
    "

    |call|

    call := SoapCall new
                tcpHost:self host port:self port;
                transport:#http;
                targetObjectURI:self targetObjectURI;
                methodName:'helloWorld'.

    ^ call perform.

    "
     self new callHelloWorld_withFullCallSpecification
    "
    "
     self new 
        hostAddress:'nilpferd'; 
        port:(HTTPServer defaultPort);
        callHelloWorld
    "
!

callReturnComplexDictionary
    |call dict|

    call := SoapCall methodName:'returnComplexDictionary' tcpHost:(self host) port:(self port).

    dict := Dictionary new.
    dict at:#Java put:'Forget it'.
    call addParameter:(SoapVariable name:#aDictionary value:dict).

    ^ call perform.

    "
     self new callReturnComplexDictionary   
    "
!

callReverseArray:arrayOfSimpleType 
    |call|

    call := SoapCall methodName:'reverseArray' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#anArray value:arrayOfSimpleType).
    ^ call perform.

    "
     self new callReverseArray:#(1 2 3 4)   
    "
!

callReverseString:aString 
    |call|

    call := SoapCall methodName:'reverseString' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#aString value:aString).
    ^ call perform.

    "
     self new callReverseString:'abcdefg'  
    "
!

callSecureHelloWorld
    "like helloWorld, but a invokes a secure (signed)service"

    |signatureHandler call|

    signatureHandler := SignatureHandler new algorithm:(SoapDigest_DSA_SHA1 new) signer:(TestSigner new).

    call := SoapSignedCall methodName:'secureHelloWorld' tcpHost:self host port:self port.
    call signatureHandler:signatureHandler.

    ^ call perform.

    "
     self new callSecureHelloWorld

     self new 
        host:'alan.exept.de';
        callSecureHelloWorld

     self new 
        host:'alan.exept.de';
        port:8890;
        callSecureHelloWorld
    "
!

callSecureReverseString:aString 
    "like reverseString, but a invokes a secure (signed)service"

    |call signatureHandler|

    signatureHandler := SignatureHandler new algorithm:(SoapDigest_DSA_SHA1 new) signer:(TestSigner new).

    call := SoapSignedCall methodName:'reverseString' tcpHost:self host port:self port.
    call addParameter:(SoapVariable name:#aString value:aString).
    call signatureHandler:signatureHandler.

    ^ call perform.

    "
     self new callReverseString:'abcdefg'  
    "
!

callTimeNow
    "return the partners time"

    |call|

    call := SoapCall methodName:'timeNow' tcpHost:(self host) port:(self port).
    ^ call perform.

    "
     self new callTimeNow  
    "
    "
     self new 
        hostAddress:'nilpferd'; 
        port:(HTTPServer defaultPort);
        callTimeNow
    "
!

callTranscriptShow2:s1 and:s2 
    "show some string on the partners transcript"

    |call|

    call := SoapCall methodName:'transcriptShow2' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#string2 value:s2).
    call addParameter:(SoapVariable name:#string1 value:s1).
    ^ call perform.

    "
     self new callTranscriptShow2:'hello' and:'world' 
    "
    "
     self new 
        port:(HTTPServer defaultPort);
        callTranscriptShow:'hello World'
    "
    "
     self new 
        hostAddress:'nilpferd'; 
        port:(HTTPServer defaultPort);
        callTranscriptShow:'hello World'
    "
!

callTranscriptShow:aString 
    "show some string on the partners transcript"

    |call|

    call := SoapCall methodName:'transcriptShow' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#aString value:aString).
    ^ call perform.

    "
     self new callTranscriptShow:'hello World'
    "
    "
     self new 
        port:(HTTPServer defaultPort);
        callTranscriptShow:'hello World'
    "
    "
     self new 
        hostAddress:'nilpferd'; 
        port:(HTTPServer defaultPort);
        callTranscriptShow:'hello World'
    "
!

twice_callReverseString:aString 
    |call val1 val2|

    call := SoapCall methodName:'reverseString' tcpHost:(self host) port:(self port).
    call addParameter:(SoapVariable name:#aString value:aString).

    val1 := call perform.
    val2 := call performWith:'123456'.

    ^ val1 , val2

    "
     self new twice_callReverseString:'abcdefg'  
    "
! !

!SoapExampleClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/SoapExampleClient.st,v 1.17 2005/12/01 15:22:11 cg Exp $'
! !

SoapExampleClient initialize!
