"{ Package: 'stx:goodies/soap/examples' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapExampleServiceImpl
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Example'
!

SoapExampleServiceImpl comment:'Simple SOAP server example.

- Implements very trivial services. (like just returning ''HelloWorld'')

- To register services, see class side.

---
mu 1/4/2002 23:05'
!


!SoapExampleServiceImpl class methodsFor:'private'!

registerBrowseClassDefinitionIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#browseClassDefinition:.
    serv signature:(SoapServiceSignature name:'browseClassDefinition' paramNames:#( className )).
    aServiceHandler add:serv
!

registerHelloWorldIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#helloWorld.
    serv signature:(SoapServiceSignature name:'helloWorld').
    aServiceHandler add:serv
!

registerReturnComplexDictionaryIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#returnComplexDictionary:.
    serv signature:(SoapServiceSignature name:'returnComplexDictionary' paramNames:#( aDictionary )).
    aServiceHandler add:serv
!

registerReverseArrayIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#reverseArray:.
    serv signature:(SoapServiceSignature name:'reverseArray' paramNames:#( anArray )).
    aServiceHandler add:serv
!

registerReverseStringIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#reverseString:.
    serv signature:(SoapServiceSignature name:'reverseString' paramNames:#( aString )).
    aServiceHandler add:serv
!

registerSecureHelloWorldIn:aServiceHandler
    | impl service |

    impl := self new.
    service := SoapSignedService implementor: impl selector: #secureHelloWorld.
    service signature: (SoapServiceSignature name: 'secureHelloWorld').
    service signatureHandler:(SignatureHandler new signer:(TestSigner new)).
    aServiceHandler add:service
!

registerSecureReverseStringIn:aServiceHandler 
    |impl service|

    impl := self new.
    service := SoapSignedService implementor:impl selector:#secureReverseString:.
    service signature:(SoapServiceSignature name:'secureReverseString' paramNames:#( aString )).
    service signatureHandler:(SignatureHandler new signer:(TestSigner new)).
    aServiceHandler add:service
!

registerTimeNowIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#timeNow.
    serv signature:(SoapServiceSignature name:'timeNow').
    aServiceHandler add:serv
!

registerTranscriptShow2StringsIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#transcriptShow:and:.
    serv signature:(SoapServiceSignature name:'transcriptShow2' paramNames:#( string1 string2 )).
    aServiceHandler add:serv
!

registerTranscriptShowIn:aServiceHandler 
    |impl serv|

    impl := self new.
    serv := SoapService implementor:impl selector:#transcriptShow:.
    serv signature:(SoapServiceSignature name:'transcriptShow' paramNames:#( aString )).
    aServiceHandler add:serv
! !

!SoapExampleServiceImpl class methodsFor:'registering'!

inspectRegisteredServices
        "SoapExampleServiceImpl inspectRegisteredServices"

        self inspectRegisteredServicesOnPort:(SoapHttpSetup defaultPort).
!

inspectRegisteredServicesOnPort:aPortNr
        "SoapExampleServiceImpl inspectRegisteredServices"

        |serviceHandler|

        serviceHandler := (HTTPServer runningServerOnPort:aPortNr) soapServiceHandler.
        serviceHandler serviceHome inspect
!

registerAllServices
        "SoapExampleServiceImpl registerAllServices"

        self registerAllServicesOnPort:(SoapHttpSetup defaultPort)
!

registerAllServicesOnPort:aPortNr
        "SoapExampleServiceImpl registerAllServicesOnPort:9000"
        "SoapExampleServiceImpl registerAllServicesOnPort:SoapHttpSetup defaultPort"

        |serviceHandler|

        SoapHttpSetup startUpOnPort:aPortNr.
        serviceHandler := (HTTPServer serverOnPort:aPortNr) soapServiceHandler.
        serviceHandler isNil ifTrue:[
            self error:'SOAP is not enabled in HTTPServer port ' , aPortNr printString.
        ].

        self registerHelloWorldIn:serviceHandler.
        self registerTimeNowIn:serviceHandler.
        self registerReverseStringIn:serviceHandler.
        self registerTranscriptShowIn:serviceHandler.
        self registerTranscriptShow2StringsIn:serviceHandler.
        self registerBrowseClassDefinitionIn:serviceHandler.
        self registerReverseArrayIn:serviceHandler.
        self registerReturnComplexDictionaryIn:serviceHandler.

        self registerSecureHelloWorldIn:serviceHandler.
        self registerSecureReverseStringIn:serviceHandler.
! !

!SoapExampleServiceImpl class methodsFor:'server status'!

runningTransportDescriptions
        "SoapExampleServiceImpl runningTransportDescriptions"
        "Show what transport server(s) is running"

        ^SoapTransportSetup runningTransportDescriptions
! !

!SoapExampleServiceImpl methodsFor:'actions'!

browseClassDefinition: aClassName
        "return the source of the passed-in class"

        | stream targetClass |
        stream := WriteStream on: (String new: 1024).
        Smalltalk isSmalltalkX ifTrue:[
            targetClass := Smalltalk at: aClassName asSymbol.
            targetClass isNil ifTrue:[
                targetClass := self class environment at: aClassName asSymbol.
            ].
        ] ifFalse:[
            targetClass := self class environment at: aClassName asSymbol.
        ].
        targetClass fileOutOn: (stream).
        ^stream contents asByteArray
!

helloWorld
        "return a constant string"

        ^'HelloWorld' 
!

returnComplexDictionary: aDictionary
        "add some elements to a passed-in dictionary"

        | dictionary |
        dictionary := aDictionary.
        dictionary at: #Smalltalk put: 'Best'.
        dictionary at: #Ruby put: #('Beautiful' 'Need IDE').
        ^dictionary
!

reverseArray: anArray
        "reverse an array"

        ^anArray reversed
!

reverseString: aString
        "reverse a string"

        ^ aString reversed
!

secureHelloWorld
        "like helloWorld, but a secure service (see registration)"

        ^ 'HelloWorld - but Secure' 
!

secureReverseString: aString
        "like reverseString:, but a secure service (see registration)"

        ^ aString reversed
!

timeNow
        "return the current time"

        ^Time now printString
!

transcriptShow: aString 
        "show some string on the transcript"

        [
            Transcript show: '#soap> '; show:aString; cr.
        ] on: Error
          do: [:ex | 
            self breakPoint:'debugHalt'. 
            Transcript cr; show: '#soap> ' , ex description.
            ^ false
        ].
        ^ true
!

transcriptShow:s1 and:s2
        "show some string on the transcript"

            Transcript show: '#soap> '; show:s1; cr.
            Transcript show: '#soap> '; show:s2; cr.
            ^ true.
! !

!SoapExampleServiceImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/SoapExampleServiceImpl.st,v 1.15 2005/12/01 15:22:15 cg Exp $'
! !
