"{ Package: 'stx:goodies/soap/examples' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapLiteClient
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-ExampleClients'
!

!SoapLiteClient class methodsFor:'documentation'!

documentation
"
    An abstract class for easy SOAP client construction;
    All you need to define in subclasses is the definition of
        host       - hostname (required)
        port       - portNumber (optional)
        nameSpace  - the SOAP namespace defining the method(s) (optional)
        targetURL  - the url used to get to the service (in an HTTP request) (optional)
        soapAction - if non-empty (optional)

    a wrapper method, which constructs the call (aka: does the invoke).
    This is a low level SOAPLite implementation - we will soon go to use wsdl for even more convenience.

    See my subclasses for concrete examples.

    [author:]
        Claus Gittinger; Jan 2003
"
! !

!SoapLiteClient class methodsFor:'accessing'!

callEntry
	^ SoapCallEntry tcpHost: self host port: self port
!

host
	self subclassResponsibility
!

namespace
    ^'/'.
    "/self subclassResponsibility
!

port
        ^ 8823

        "/Override method in subclass to define default port.
!

soapAction
    ^ ''
!

targetObjectURI
        ^ '/' 
        "/ self subclassResponsibility
! !

!SoapLiteClient methodsFor:'private-helpers'!

callForMethod:methodName
        | call |
        call := self newCall.
        call methodName: methodName.
        call context soapAction:(SoapAction intent:(self soapAction bindWith:methodName)).
        ^call
!

callForMethod:methodName parameter:parameterName
	| call |
	call := self callForMethod:methodName.
	call addParameterNamed: parameterName.
	^call
!

callForMethod:methodName parameter:parameterName1 parameter:parameterName2
	| call |
	call := self callForMethod:methodName parameter:parameterName1.
	call addParameterNamed: parameterName2.
	^call
!

callForMethod:methodName parameter:parameterName1 parameter:parameterName2 parameter:parameterName3
        | call |
        call := self callForMethod:methodName parameter:parameterName1 parameter:parameterName2.
        call addParameterNamed: parameterName3.
        ^call
!

callMethod:methodName
        | call resp |

        call := self callForMethod:methodName.
        resp := call invoke.
        ^ resp isFault
                ifTrue: [resp raiseException]
                ifFalse: [resp returnValue]
!

callMethod:methodName withParameter:nameValuePair
        | call resp |

        call := self callForMethod:methodName parameter:nameValuePair key.
        resp := call invokeWithValue:nameValuePair value.
        ^ resp isFault
                ifTrue: [resp raiseException]
                ifFalse: [resp returnValue]
!

callMethod:methodName withParameter:nameValuePair1 withParameter:nameValuePair2
        | call resp |

        call := self callForMethod:methodName parameter:nameValuePair1 key parameter:nameValuePair2 key.
        resp := call invokeWithValues:(Array with:nameValuePair1 value with:nameValuePair2 value).
        ^ resp isFault
                ifTrue: [resp raiseException]
                ifFalse: [resp returnValue]
!

callMethod:methodName withParameter:nameValuePair1 withParameter:nameValuePair2 withParameter:nameValuePair3
        | call resp |

        call := self callForMethod:methodName parameter:(nameValuePair1 key) parameter:(nameValuePair2 key) parameter:(nameValuePair3 key).
        resp := call invokeWithValues:(Array with:(nameValuePair1 value) with:(nameValuePair2 value) with:(nameValuePair3 value)).
        ^ resp isFault
                ifTrue: [resp raiseException]
                ifFalse: [resp returnValue ]
!

newCall
        | call |

        call := self class callEntry newCall.
        call targetObjectURI:(self class targetObjectURI).

        call namespace:(self class namespace).
        call context soapAction:(SoapAction intent:self class soapAction).
        ^ call
!

soapAction
        ^ self class soapAction
! !

!SoapLiteClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/examples/SoapLiteClient.st,v 1.7 2005/12/01 15:21:53 cg Exp $'
! !
