"{ Package: 'stx:goodies/soap/opera' }"

"{ NameSpace: SOAP }"

Object subclass:#OperaExampleDoc
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Opera-Example'
!

!OperaExampleDoc class methodsFor:'documentation'!

aboutSoapOpera
"## SoapOpera ## 

SoapOpera consists of two parts: SoapCore and OperaORB.

SoapCore is a SOAP implementation for Squeak (including SOAP RPC).
OperaORB is a is a tiny ORB wrapper on SOAP (or some network messaging service layer).

-2000/12/28 0.2
-2001/12/02 0.2b
-2002/03/01 0.5beta1
-2002/10/6 0.5beta4

---
[:masashi | ^umezawa] 
"
!

howToRunExample
^'1:
	Register a server object implementation.
	"OperaExampleDoc serverExample1"

2:
	Call remote methods by OperaRemoteObject (do it)
	"OperaExampleDoc clientExample1"
	"OperaExampleDoc clientExample2"
	"OperaExampleDoc clientExample3"
	"OperaExampleDoc clientExample4"
	"OperaExampleDoc clientExample5"
	"OperaExampleDoc clientExample6"

	(If you register object implementation in some remote machine, you have to set the default remote host address).
	"OperaSetting defaultHost: <your server name>."

3:
	If you would like to see message passing log, do it. (Open Transcript.)
	"OperaSetting showClientLog: true" 

4:
	Now OperaORB runs on not only well-known HTTP, but also SQTP, a Squeak specific binary transport protocol.
	SQTP is extremely fast in bulky objects sending, but it loses interoperability with other SOAPs (normally running on text-based HTTP).
	"OperaSetting defaultTransport: #http" 
	"OperaSetting defaultTransport: #sqtp" 	

---
MU 9/14/2002 0:24
'
! !

!OperaExampleDoc class methodsFor:'examples'!

clientExample1
	| john |
	"OperaExampleDoc clientExample1"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	"OperaSetting defaultHost: '<your server name>'."
	"OperaSetting showClientLog: true."
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	john doSomething: 'Smalltalk!!!!'.
	
	
!

clientExample2
	| john ret |
	"OperaExampleDoc clientExample2"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret := john doSomethingAndReturnSimple: 'Smalltalk!!!!'.
	
	" 4: remote message send (causes error)"
	"john add: #fooBarBaz"
	
	Transcript cr; show: ret printString. 
	
	
!

clientExample3
	| john  |
	"OperaExampleDoc clientExample3"
		
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send (causing server side error)"
	[ john doBadly: 'not Java'] on: MessageNotUnderstood do:[:ex | Transcript cr; show: 'an Error raised from server-> ', ex description].
	
!

clientExample4
	| john  ret1 ret2 |
	"OperaExampleDoc clientExample4"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret1 := john doSomethingAndReturnSelf: 'Smalltalk - but sometimes I think we may need void'.
	ret2 := john doSomethingAndReturnSelfCopy: 'Smalltalk'.

	ret1 doSomething: 'Local'. "local copy send"
	ret2 doSomething: 'Remote'. "remote"
	
	" 4: please see the difference"
	ret1 inspect.
	ret2 inspect.
	
!

clientExample5
	| john  ret1 ret2 |
	"OperaExampleDoc clientExample5"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret1 := john doSomethingAndReturnObjRef: 'Eiffel'.
	ret2 := john doSomethingAndReturnObjValue: 'Smalltalk'.
	
	ret1 doSomething: 'Remote'. "remote"
	ret2 doSomething: 'Local'. "local copy send"
	
	" 4: please see the difference"
	ret1 inspect.
	ret2 inspect.
	
!

clientExample6
	| john paul |
	"OperaExampleDoc clientExample6"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := OperaRemoteObject named: #John.
	"john := OperaRemoteObject name: #John host:<your server name>. "
	
	" 3: create object reference"
	paul := OperaObjectReference on: (OperaPerson named: 'Paul').
	
	" 4: remote message send"
	john doSomethingAndArgDoSomething: paul.
	
	
!

serverExample1
	"OperaExampleDoc serverExample1"
	
	"##Server side##"
	" 0: open transcript(to see the results)"

	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: init object adapter"
	OperaORB root initialize.
	
	" 3: register object implementation"
	OperaORB root at: #John put: OperaPerson new.
	
! !

!OperaExampleDoc class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/opera/OperaExampleDoc.st,v 1.3 2002/11/26 18:40:07 cg Exp $'
! !
