"{ Package: 'stx:goodies/soap/opera' }"

"{ NameSpace: SOAP }"

Object subclass:#OperaObjectAdapter
	instanceVariableNames:'namedObjectHome oidGenerator'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Opera-Server'
!

OperaObjectAdapter class instanceVariableNames:'default'

"
 No other class instance variables are inherited by this class.
"
!


!OperaObjectAdapter class methodsFor:'instance creation'!

default
	default isNil ifTrue:[default := self new].
	^default
!

new
	^super new initialize
!

root
	"OperaObjectAdapter root"
	"Returns RootOOA"
	^self default
! !

!OperaObjectAdapter class methodsFor:'class initialization'!

initialize
	"OperaObjectAdapter initialize"
	super initialize.
	OperaPortableUtil installRootOOA
!

obsolete
	OperaPortableUtil uninstallRootOOA.
	super obsolete.
	
! !

!OperaObjectAdapter class methodsFor:'constants'!

dispatchSelector
	^#dispatch:
!

dispatchServiceName
	^'Dispatch'
!

dispatchSoapSignature
	^(SoapServiceSignature name: self dispatchServiceName paramNames: #(aRemoteMessageSend)).
!

serverName
	^'RootOOA'
!

serverURIName
	^'urn:SoapOpera:', self serverName
!

serviceTargetObjectURI
	^'/soapOpera/rootOOA'
! !

!OperaObjectAdapter methodsFor:'accessing'!

namedObjectHome
	namedObjectHome isNil ifTrue:[namedObjectHome := OperaNamedObjectHome new].
	^namedObjectHome
!

namedObjectHome: aNamedObjectHome
	namedObjectHome := aNamedObjectHome
	
!

oidGenerator
	oidGenerator isNil ifTrue:[oidGenerator := OperaOidGenerator new].
	^oidGenerator
!

oidGenerator: aNamedObjectHome
	oidGenerator := aNamedObjectHome
	
!

service
	^service
! !

!OperaObjectAdapter methodsFor:'initialize-release'!

initialize
	oidGenerator := nil.
	self initializeNamedObjectHome
!

initializeNamedObjectHome
	
	self namedObjectHome initialize
! !

!OperaObjectAdapter methodsFor:'private'!

activateObjectReference: anObjectReference named: aRegisteredObjectName 
	| found |
	"assign object name"
	"implicitly create unshared obj ref name and register"
	found := self at: aRegisteredObjectName ifAbsent: [].
	found isNil
		ifTrue: 
			[|  impl nm | 
			impl := anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl. 
			anObjectReference name: nm]
		ifFalse: [anObjectReference name: aRegisteredObjectName]. 
	^ anObjectReference
!

nextTempObjectName
	^('temp:', self oidGenerator nextValue) asSymbol
!

nextTempObjectNameFor: anObject
	| wStr |
	"^('temp:', self oidGenerator nextValue,':', anObject name) asSymbol"
	wStr := WriteStream on: (String new: 32).
	wStr nextPutAll: 'temp:'.
	wStr nextPutAll: self oidGenerator nextValue.
	wStr nextPutAll: ':'.
	wStr nextPutAll: anObject class name.
	^wStr contents asSymbol
! !

!OperaObjectAdapter methodsFor:'receiving'!

dispatch: anOperaRemoteMessage
	
	| message receiver ret |
	message := anOperaRemoteMessage.
	receiver := self at: message receiverName.
	OperaPortableUtil iterate: message arguments
		keysAndValuesDo: [:idx :each | (OperaObjectReference hasInstance: each)
				ifTrue: [message arguments at: idx put: each asRemoteObject]].

	"##need more work##"
	"need error handling for obj ref"
	ret := receiver perform: message selector withArguments: message arguments.

	"##restrict implicit self return-by-value##"
	ret == receiver ifTrue: [^ ret := (self activateObjectReference: (OperaObjectReference on: ret )
				named: message receiverName)].

	"##investigate return-by-value or return-by-ref##"
	(OperaObjectReference hasInstance: ret)
		ifTrue: [^ ret := (self activateObjectReference: ret named: ret name)].

	^ ret
! !

!OperaObjectAdapter methodsFor:'registering'!

registerObjectImplFor: anObjectReference
	| found |
	(OperaObjectReference hasInstance: anObjectReference) ifFalse:[OperaError badParameter; description:'parameter is not object refrence'; signal].
	
	"if found, object has been registered already, so do nothing"
	
	found := self at: anObjectReference name ifAbsent: [].
	found isNil
		ifTrue: 
			[| nm impl |
			impl :=  anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl.
			anObjectReference name: nm].

	^ anObjectReference
! !

!OperaObjectAdapter methodsFor:'tiny naming service'!

at: aSymbol
	^ self namedObjectHome at: aSymbol
!

at: aSymbol ifAbsent: aBlock
	^ self namedObjectHome at: aSymbol ifAbsent: aBlock
!

at: aSymbol ifAbsentPut: aBlock
	^ self namedObjectHome at: aSymbol ifAbsentPut: aBlock
!

at: aSymbol put: anObject 
	^ self namedObjectHome at: aSymbol put: anObject
!

removeKey: aSymbol
	^ self namedObjectHome removeKey: aSymbol 
!

removeKey: aSymbol ifAbsent: aBlock
	^ self namedObjectHome removeKey: aSymbol ifAbsent: aBlock
! !

!OperaObjectAdapter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/opera/OperaObjectAdapter.st,v 1.3 2002/11/26 18:39:57 cg Exp $'
! !

OperaObjectAdapter initialize!
