"{ Package: 'stx:goodies/soap/opera' }"

"{ NameSpace: SOAP }"

Object subclass:#OperaPortableUtil
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Opera-Base'
!

OperaPortableUtil class instanceVariableNames:'service localAddress localHostname defaultHost defaultEncoding
	defaultTransport'

"
 No other class instance variables are inherited by this class.
"
!


!OperaPortableUtil class methodsFor:'adapter for Collection'!

iterate: aCollection keysAndValuesDo: aBlock
	"Squeak withIndexDo: is weird"
	^aCollection keysAndValuesDo: aBlock
! !

!OperaPortableUtil class methodsFor:'adapter for Network'!

isLocal: host port: port 
	"Squeak specific"
	^ port = self defaultPort
		and: [((host = 'localhost'
						or: [host = '127.0.0.1'])
					or: [host = self localAddress])
				or: [host = self localHostname]]
!

localAddress
	"Squeak specific"
	localAddress isNil ifTrue: [localAddress := NetNameResolver localAddressString].
	^localAddress
!

localHostname
        "Squeak specific"
        localHostname isNil
                ifTrue: [[localHostname := NetNameResolver nameForAddress: NetNameResolver localHostAddress timeout: 30]
                                on: Error
                                do: [:ex |  self halt.
                                        localHostname := 'localhost'.
                                        ex return]].
        ^ localHostname
! !

!OperaPortableUtil class methodsFor:'adapter for Soap'!

defaultEncoding
	"Assuming SoapOpera Squeak"
	defaultEncoding isNil ifTrue: [defaultEncoding := #soapSqEncoding].
	^defaultEncoding
!

defaultEncoding: aSymbol
	"Assuming SoapOpera Squeak"
	defaultEncoding := aSymbol
!

defaultHost
	defaultHost isNil ifTrue: [defaultHost := 'localhost'].
	^defaultHost
!

defaultHost: aString
	defaultHost := aString
!

defaultPort
        "Assuming SoapOpera Squeak"
        ^SOAP::SoapSetting defaultPort
!

defaultPort: anInteger
	"Assuming SoapOpera Squeak"
	SoapSetting defaultPort: anInteger
!

defaultTransport
        "Assuming SoapOpera Squeak"
        ^SOAP::SoapSetting defaultTransport
!

defaultTransport: aSymbol
	"Assuming SoapOpera Squeak (#http #sqtp)"
	SoapSetting defaultTransport: aSymbol
!

installRootOOA
	"OperaPortableUtil installRootOOA"
	"TODO: Supports multi SOAP implementaions (Spray or VWSOAP)"
	"Assuming SoapOpera Squeak"
	| rootOOA |
	service notNil
		ifTrue: [self uninstallRootOOA].
	SoapTransportSetup restartAt: self defaultTransport port: self defaultPort.
	rootOOA := OperaORB root.
	service := SoapService implementor: rootOOA selector: rootOOA class dispatchSelector.
	service targetObjectURI: rootOOA class serviceTargetObjectURI.
	service signature: rootOOA class dispatchSoapSignature.
	SoapServiceHandler default add: service
!

soapIsRunning
	"Assuming SoapOpera Squeak"
	^SoapHttpSetup service isRunning
!

soapSend: anOperaRemoteMessage host: host port: port transport: transportSymbol 
	"Assuming SoapOpera Squeak"
	| call resp signature |
	call := (SoapCallEntry tcpHost: host port: port) newCall.
	call transport: transportSymbol.
	call targetObjectURI: OperaObjectAdapter serviceTargetObjectURI.
	call encoding: self defaultEncoding.
	signature := OperaObjectAdapter dispatchSoapSignature.
	call methodName: signature name.
	call
		addParameter: (SoapVariable name: signature paramNames first value: anOperaRemoteMessage).
	resp := call invoke.
	^ resp isFault
		ifTrue: [resp raiseException]
		ifFalse: [resp returnValue]
!

startSoap
	SoapHttpSetup stop.
	SoapHttpSetup start
!

stopSoap
	SoapHttpSetup stop
!

uninstallRootOOA
	"OperaPortableUtil uninstallRootOOA"
	SoapServiceHandler default remove: service.
	service := nil
! !

!OperaPortableUtil class methodsFor:'adapter for System Initialization'!

startUp

	service := localAddress := localHostname := nil.
	self installRootOOA.
!

update:something with:aParameter from:changedObject
    something == #returnFromSnapshot ifTrue:[
        self startUp.
        ^ self
    ].
"/    something == #aboutToExit ifTrue:[
"/        self shutDown.
"/        ^ self
"/    ].
    super update:something with:aParameter from:changedObject
! !

!OperaPortableUtil class methodsFor:'class initialization'!

initialize
        "OperaPortableUtil initialize"

        Smalltalk isSmalltalkX ifTrue:[
            ObjectMemory addDependent:self
        ] ifFalse:[
            Smalltalk addToStartUpList: self.
            Smalltalk addToShutDownList: self.
        ].
!

obsolete
        "OperaPortableUtil obsolete"

        Smalltalk isSmalltalkX ifTrue:[
            ObjectMemory removeDependent:self
        ] ifFalse:[
            Smalltalk removeFromStartUpList: self.
            Smalltalk removeFromShutDownList: self.
        ]
! !

!OperaPortableUtil class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/opera/OperaPortableUtil.st,v 1.8 2003/01/10 16:07:18 cg Exp $'
! !

OperaPortableUtil initialize!
