"{ Package: 'stx:goodies/soap/opera/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#OperaRemoteObjectTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Opera-TestCases'
!


!OperaRemoteObjectTestCase methodsFor:'running'!

setUp
	OperaPortableUtil installRootOOA
! !

!OperaRemoteObjectTestCase methodsFor:'testing'!

testBasicRemoteMessageSend
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testOrderedCollection put: OrderedCollection new.
        
	remote := OperaRemoteObject named: #testOrderedCollection.
	ret := remote add: #newElem1.
	self should: [ret = #newElem1].

	self should: [(OperaORB root at: #testOrderedCollection) size = 1].
	self should: [(OperaORB root at: #testOrderedCollection) first = #newElem1].
	OperaORB root removeKey: #testOrderedCollection.
!

testBasicRemoteMessageSendWithError
        
	| remote |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testOrderedCollection put: OrderedCollection new.
        
	remote := OperaRemoteObject named: #testOrderedCollection.
	self should: [ remote at: #kay put: #newElem1.] raise: MessageNotUnderstood.

	self shouldnt: [(OperaORB root at: #testOrderedCollection) size = 1].
	OperaORB root removeKey: #testOrderedCollection.
!

testCatNameWith
        
	| remote remoteCopy catName catName2 |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].

	self shouldnt:[remote name: 'original server'] raise:MessageNotUnderstood.
        
	self shouldnt:[remoteCopy := remote returnSelfCopy] raise:MessageNotUnderstood.
	self should:[remoteCopy name = 'original server'].
	self should:[remoteCopy isTestRemoteServer].
	self shouldnt:[remoteCopy name: 'copy server'] raise:MessageNotUnderstood.

	self shouldnt:[catName := remote catNameWith: remoteCopy] raise:MessageNotUnderstood.
	self should:[catName = 'original servercopy server'].
	self should:[remoteCopy name = 'copy server'].
        
	self shouldnt:[catName2 := remote catNameWith: (OperaObjectReference on: remoteCopy)] raise:MessageNotUnderstood.
	self should:[catName2 = 'original servercopy server'].
	self should:[remoteCopy name = 'original servercopy server'].

	OperaORB root removeKey: #testServer.
!

testComplexRemoteMessageSend
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testDictionary put: Dictionary new.
        
	remote := OperaRemoteObject named: #testDictionary.
	remote at: #key1 put: #newElem1.
	ret := remote at: #key2 put: #(#newElem2 #newElem3).
	self should: [ret = #(#newElem2 #newElem3)].

	self should: [(OperaORB root at: #testDictionary) values size = 2].
	self should: [((OperaORB root at: #testDictionary) at: #key2) = #(#newElem2 #newElem3)].
	OperaORB root removeKey: #testDictionary.
!

testEchoString
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].
	self shouldnt:[ret := remote echoString: 'HELLO'] raise:MessageNotUnderstood.
	self should:[ret = 'RET:HELLO'].
        
	OperaORB root removeKey: #testServer.
!

testReturnDictionary
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].
        
	self shouldnt:[ret := remote returnDictionary] raise:MessageNotUnderstood.
	self should:[ret = OperaTestRemoteServer new returnDictionary].
        
	OperaORB root removeKey: #testServer.
!

testReturnSelf
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].
        
	self shouldnt:[ret := remote returnSelf] raise:MessageNotUnderstood.
	self should:[ret isRemote].
	self should:[(ret echoString: 'HELLO') = 'RET:HELLO'].
        
	OperaORB root removeKey: #testServer.
!

testReturnSelfCopy
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].
	self shouldnt:[remote name: 'original server'] raise:MessageNotUnderstood.
	self shouldnt:[ret := remote returnSelfCopy] raise:MessageNotUnderstood.
	self should:[ret isTestRemoteServer].
	self shouldnt:[ret name: 'copied server'] raise:MessageNotUnderstood.
	self shouldnt:[remote name = ret name].
        
	OperaORB root removeKey: #testServer.
!

testReturnSelfObjectReference
        
	| remote ret |
	OperaSetting useColocation: false.
        
	OperaORB initialize.
	OperaORB root at: #testServer put: OperaTestRemoteServer new.
        
	remote := OperaRemoteObject named: #testServer.
	self should:[remote isRemote].

	self shouldnt:[remote name: 'original server'] raise:MessageNotUnderstood.
	self shouldnt:[ret := remote returnSelfObjectReference] raise:MessageNotUnderstood.
	self should:[ret isRemote].
	self shouldnt:[ret name: 'changed name'] raise:MessageNotUnderstood.
	self should:[ret name = remote name].
        
	OperaORB root removeKey: #testServer.
! !

!OperaRemoteObjectTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/opera/tests/OperaRemoteObjectTestCase.st,v 1.4 2003/01/20 19:02:15 cg Exp $'
! !
