"{ Package: 'stx:goodies/soap/splash' }"

"{ NameSpace: SOAP }"

Object subclass:#SplashNodeReader
	instanceVariableNames:'readMap topNode currentDecoding textDecoder typeName isLiteralEnc'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash'
!

SplashNodeReader comment:'I am part of Splash. I am used by a SplashResponse to decode the xml contents of a soap response to Objects. I do the best I can. I do not have knowledge of the metadata. Unlike Spray which uses the information in a service''s WSDL, when I can I rely on inline type attributes in the envelope. I can also be told to explicitly convert to a class of object.

I should not be used directly.

Instance Variables:
	readMap		<XeNodeReadMap>
	topNode		<XeNode>
	currentDecoding	<SplashNodeDecoding>	
	textDecoder	<SplashTextDecoder>
	typeName	<XeName>
	isLiteralEnc	<Boolean>'
!


!SplashNodeReader class methodsFor:'instance creation'!

new
	^self basicNew initialize
!

onNode: aXeNode
	^self new initializeOn: aXeNode
!

readNode: aXeNode
	| instance |
	instance := self onNode: aXeNode.
	instance decode.
	^instance
! !

!SplashNodeReader class methodsFor:'operations'!

decodeNode: aXeNode
	^(self readNode: aXeNode) nodeObject
! !

!SplashNodeReader methodsFor:'accessing'!

convertTypeFor: aSplashNodeDecoding
	^(typeName notNil and: [typeName isAnyTypeName not]) 
		ifTrue: [typeName]
		ifFalse: [aSplashNodeDecoding node qValueForXSIType]
! !

!SplashNodeReader methodsFor:'accessing-private'!

currentChildNodes
	^currentDecoding childNodes
!

currentConvertType
	^self convertTypeFor: currentDecoding
!

currentDecoding
	^currentDecoding
!

currentNode
	^currentDecoding node
!

currentNodeText
	^currentDecoding node text
!

makeCurrentNode: aXeNode
	currentDecoding := SplashNodeDecoding node: aXeNode reader: self
!

nodeObject
	^currentDecoding object
!

readMap
	^readMap
!

textDecoder: anObject
	textDecoder := anObject
!

typeName
	^typeName
!

typeName: aTypeQName
	typeName := aTypeQName
! !

!SplashNodeReader methodsFor:'branching-private'!

branchWithCurrentNode: aXeNode typeName: aTypeQName
	^(self copy)
		makeCurrentNode: aXeNode;
		typeName: aTypeQName;
		yourself
! !

!SplashNodeReader methodsFor:'copying-private'!

postCopy
	super postCopy.
	typeName := nil.
! !

!SplashNodeReader methodsFor:'initializing-private'!

initialize
	super initialize.
	textDecoder := SplashTextDecoder new.
	isLiteralEnc := false
!

initializeOn: aXeNode
	topNode := aXeNode.
	readMap := XeNodeReadMap on: aXeNode.
	self makeCurrentNode: aXeNode
! !

!SplashNodeReader methodsFor:'modes-private'!

beLiteralEnc
	isLiteralEnc := true
! !

!SplashNodeReader methodsFor:'operations'!

decode
	"Answer the object decoded from the node in currentDecoding.
	Note: the currentDecoding is responsible for maintaining soap-enc 'id/href' references and answering identical objects for a multireference. If there is no object previously created for the currentNode, the currentDecoding sends the receiver the #doDecoding message"

	^currentDecoding readNode object
!

doDecoding
	currentDecoding object: self readDecoding
! !

!SplashNodeReader methodsFor:'operations-private'!

convert: aString typeName: aXeQName
	^textDecoder isNil 
		ifTrue: [aString]
		ifFalse: [textDecoder convert: aString typeName: aXeQName]
! !

!SplashNodeReader methodsFor:'reading-private'!

readAsArray
	| arrayNode itemType answer index |
	(arrayNode := XeSoapArrayNodeAbstract forNode: self currentNode) isMultiDim 
		ifTrue: [self error: 'MultiDim Arrays are not support by splash'].
	itemType := arrayNode itemTypeQName.
	answer := Array new: arrayNode aSize.
	index := arrayNode offsetAttribute + 1.
	arrayNode childNodes do: 
			[:each | 
			| att |
			att := each getAttribute: 'position' in: XeNamespace soapEnc.
			att notNil ifTrue: [index := (arrayNode extractSize: att) + 1].
			answer at: index put: (self visitChildNode: each usingType: itemType).
			index := index + 1].
	^answer
!

readAsComplex
	^currentDecoding isNodeSoapArray ifTrue: [self readAsArray] ifFalse: [self readAsStruct]
!

readAsLiteralStruct
	| struct |
	struct := XeQStruct new.
	self currentChildNodes 
		do: [:each | struct at: each qName put: (self visitChildNode: each)].
	self currentNode getQAttributes 
		do: [:each | struct attributes at: each key put: each value].
	^struct
!

readAsSimple
	| convertType |
	^(convertType := self currentConvertType) isNil 
		ifTrue: [self currentNodeText]
		ifFalse: [self convert: self currentNodeText typeName: convertType]
!

readAsSoapStruct
	| struct |
	struct := XeQStruct new.
	self currentChildNodes 
		do: [:each | struct at: each localName asSymbol put: (self visitChildNode: each)].
	^struct
!

readAsStruct
	^isLiteralEnc ifTrue: [self readAsLiteralStruct] ifFalse: [self readAsSoapStruct]
!

readDecoding
	^currentDecoding isSimpleContent ifTrue: [self readAsSimple] ifFalse: [self readAsComplex]
! !

!SplashNodeReader methodsFor:'visiting-private'!

visitChildNode: aXeNode
	^self visitChildNode: aXeNode usingType: nil
!

visitChildNode: aXeNode usingType: aTypeQName
	^(self branchWithCurrentNode: aXeNode typeName: aTypeQName) decode
! !

!SplashNodeReader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashNodeReader.st,v 1.2 2002/11/18 19:02:53 cg Exp $'
! !
