"{ Package: 'stx:goodies/soap/splash' }"

"{ NameSpace: SOAP }"

SpwSoapOutEnvelope subclass:#SplashRequest
	instanceVariableNames:'prefixMap xsiNS'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash'
!

SplashRequest comment:'I am a request soap envelope.

I have methods that allow an envelope to be built by asking objects added to the envelope, for their xml representation. I am a qualifing context (See XeName class comment).

See showHelp for examples.

Instance Variables:
	prefixMap		<LookupTable>	Maps prefix strings to symbol namespaces
	xsiNS		<Symbol>	The Schema instance namespace

'
!


!SplashRequest class methodsFor:'instance creation'!

new
	^self basicNew initialize
! !

!SplashRequest class methodsFor:'class initialization'!

initialize
    (ByteArray implements:#'putSplashOn:context:') ifFalse:[
        Smalltalk fileIn:'extensions.st' inPackage:self package.
    ].

    "
     self initialize
    "
! !

!SplashRequest methodsFor:'accessing-private'!

addPrefix: aString namespace: aSymbol 
	prefixMap at: aString put: aSymbol 
!

namespacePrefixed: aString
	"Private - Answer a NSSymbol"

	^prefixMap at: aString
		ifAbsent: [self error: 'The typespace does not contain the prefix: ' , aString displayString]
!

prefixMap
	"Private - Answer the value of the receiver's ''prefixMap'' instance variable."

	^prefixMap
! !

!SplashRequest methodsFor:'constants-private'!

qNameXSIType
	^XeQName 
		prefix: 'xsi'
		local: 'type'
		ns: xsiNS
! !

!SplashRequest methodsFor:'encoding-helpers'!

putXSINilOn: aXeNode
	"Add nil/null to the node.
	-the definition for this depends on the schema version being used"

	aXeNode addNullAttributeFor: xsiNS
!

putXSIType: aName on: aXeNode
	"Private - add the inline type attribute to the node.
	aName can either be a string/XePName/XeQName (see SplashRPCRequest>>parameterAt:put:type:)"

	aXeNode setQAttribute: self qNameXSIType qName: (self qualify: aName)
!

qualify: aName 
	^aName asXeName qualifiedNameIn: self
!

useReferenceOf: anObject on: aXeNode
	^self subclassResponsibility
!

useReferenceOf: anObject on: aXeNode special: aBool
	"Answer true if the writeMap was able to encode object as a multireference."

	^self subclassResponsibility
! !

!SplashRequest methodsFor:'initializing'!

beSchema1999
	"Set the envelope to be for the 1999 schema. (While the SOAP1.1 spec says to use 1999, Common (very common) practice is to ignore 1999 and use 2001.
	-You should never have to use this.
	Note: this should only ever be sent before adding objects to an envelope"
 
	self addXSINamespace: (xsiNS := XeNamespace schema1999i).
	node addNSPrefix: 'xsd' to: XeNamespace schema1999.
	self addPrefix: 'xsd' namespace: XeNamespace schema1999.
!

beSchema2001
	"Set the envelope to be for the 2001 schema. 
	-add the 2001 schema-instance namespace declaration to the envelope
	-add a prefix mapping from 'xsd' to the schema namespace so that XePNames that use 'xsd', qualify into the 2001 namespace
	-add an xsd declaration to the envelope (not strictly necessary as it would be added if required, but as it usually is, I add it anyway)
	Note: this should only ever be sent before adding objects to an envelope"

	self addXSINamespace: (xsiNS := XeNamespace schema2001i).
	node addNSPrefix: 'xsd' to: XeNamespace schema2001.
	self addPrefix: 'xsd' namespace: XeNamespace schema2001.
! !

!SplashRequest methodsFor:'testing'!

isIDHref
	^self subclassResponsibility
!

isSoapEnc
	^self subclassResponsibility
! !

!SplashRequest methodsFor:'transporting'!

exchangeTo: aURLString soapAction: aString2 
	| ex |
	ex := SptRequestingExchange out: self to: aURLString asString.
	ex soapAction: aString2.
	^ex
!

responseFrom: aURLString soapAction: aStringOrNil 
	"Answer a SplashResponse.
	-Send the receiver to the server at aURLString. (with the HTTP Header SOAPAction set to aString2).
	--If the message exchange does not end in the Success state, signal an error"

	| exchange |
	exchange := self exchangeTo: aURLString soapAction: aStringOrNil.
	exchange processUsingDefault.
	exchange assertSuccessState.
	^SplashResponse exchange: exchange.

! !

!SplashRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashRequest.st,v 1.8 2008/09/05 17:48:32 cg Exp $'
! !

SplashRequest initialize!
