"{ Package: 'stx:goodies/soap/splash/tests' }"

"{ NameSpace: SOAP }"

SplashTests subclass:#SplashEncoderTest
	instanceVariableNames:'encodeContext'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash-TestCases'
!


!SplashEncoderTest methodsFor:'accessing'!

setInlineTyping

        
!

setNotInlineTyping
        
! !

!SplashEncoderTest methodsFor:'set up'!

setUp
	super setUp.
	encodeContext := SplashRPCRequest new
! !

!SplashEncoderTest methodsFor:'tests'!

testAnyValue
	| node anyValue |
	anyValue := XeAnyValue text: 'Whoops' typeName: (XeQName xsd2001: 'float').
	node := self newNode: 'blah'.
	anyValue putSplashOn: node context: encodeContext.
		self assertXML: node xml isSameAs: 
'<blah xsi:type="xsd:float"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema">Whoops</blah>'
!

testAnyValueNonXsdNS
	| node anyValue |
	anyValue := XeAnyValue text: 'Whoops' typeName: (self blahQ: 'foo').
	node := self newNode: 'blah'.
	anyValue putSplashOn: node context: encodeContext.
		self assertXML: node xml isSameAs: 
'<blah xsi:type="ns1:foo"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:ns1="urn:blah">Whoops</blah>'
!

testArrayAnyType
	| node |

	node := self newNode: 'blah'.
	#(1 3.14 'abc')  putSplashOn: node context: encodeContext.
	self assertXML: node xml isSameAs: 
'<blah ns1:arrayType="xsd:anyType[3]" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:ns1="http://schemas.xmlsoap.org/soap/encoding/"><item xsi:type="xsd:integer">1</item><item xsi:type="xsd:double">3.14</item><item xsi:type="xsd:string">abc</item></blah>'
!

testLiteralStruct
	| node |
	self setInlineTyping.
	node := self newNode: 'blah'.
	(self getSoapStruct  asElementQualifiedIn: #'urn:blah') putSplashOn: node context: encodeContext.
		self assertXML: node xml isSameAs: 
'<blah xmlns:ns1="urn:blah">
<ns1:varString>abc</ns1:varString>
<ns1:varInt>123</ns1:varInt>
<ns1:varFloat>3.14</ns1:varFloat>
</blah>'
!

testSimple
	#(1 3.14 'abc') with: #('1' '3.14' 'abc') do: [ :value :answer | | node |
		self setNotInlineTyping.
		node := self newNode: 'blah'.
		value putSplashOn: node context: encodeContext.
		self assert: node xml = ('<blah>%1</blah>' formatWith: answer)]
!

testSoapStruct
	| node |
	self setInlineTyping.
	node := self newNode: 'blah'.
	self getSoapStruct  putSplashOn: node context: encodeContext.
		self assertXML: node xml isSameAs: 
'<blah>
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</blah>'
!

testSoapStructTyped
	| node struct |
	self setInlineTyping.
	struct := self getSoapStruct.
	struct typeName: ('SOAPStruct' ns: #'http://soapinterop.org').
	node := self newNode: 'blah'.
	struct putSplashOn: node context: encodeContext.
	self assertXML: node xml
		isSameAs: '<blah
xsi:type="ns1:SOAPStruct"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:ns1="http://soapinterop.org">
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</blah>'
!

testSplashObject
	| node |
	node := self newNode: 'blah'.
	SWSplashObject example1 putSplashOn: node context: encodeContext.
	self assertXML: node xml isSameAs: '<blah><myX>5</myX><myY>hello</myY></blah>'
!

testTypeSimple
	#(1 3.14 'abc') 
		with: #('xsd:integer' 'xsd:double' 'xsd:string') 
		do: [ :value :typeString | | node |
		node := self newNode: 'blah'.
		value putSplashTypeOn: node context: encodeContext.
		self 
			assertXML: node xml 
			isSameAs:
('<blah xsi:type="%1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"/>'  formatWith: typeString)]
! !

!SplashEncoderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/tests/SplashEncoderTest.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
