"{ Package: 'stx:goodies/soap/splash/tests' }"

"{ NameSpace: SOAP }"

SplashTests subclass:#SplashNodeReaderTest
	instanceVariableNames:'node reader'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash-TestCases'
!


!SplashNodeReaderTest methodsFor:'accessing'!

setNodeWith: aString

	node := self rootNodeFor: 
('<root
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema">%1</root>' formatWith: aString)
!

setReaderWithFirstChild
	reader := SplashNodeReader onNode: node firstChild.
	reader decode
! !

!SplashNodeReaderTest methodsFor:'tests'!

testArrayWithArrayType
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[2]">
<item>123</item>
<item>456</item>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(123 456)
!

testArrayWithIDHrefSpecial
	self 
		setNodeWith: 
'<blah SOAP-ENC:arrayType="xsd:string[3]">
    <item id="id1" xsi:type="xsd:string">abc</item>
    <item href="#id1"/>
    <item xsi:type="xsd:string">def</item>
  </blah>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #('abc' 'abc' 'def').
	self assert: reader nodeObject first == reader nodeObject second
!

testArrayWithMissing
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[3]">
<item>123</item>
<item xsi:nil="true"/>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(123 nil nil)
!

testArrayWithNil
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[3]">
<item>123</item>
<item xsi:nil="true"/>
<item>456</item>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(123 nil 456)
!

testArrayWithOffset
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[4]" SOAP-ENC:offset="[2]">
<item>123</item>
<item>456</item>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(nil nil 123 456)
!

testArrayWithOffset0
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[4]" SOAP-ENC:offset="[0]">
<item>123</item>
<item>456</item>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(123 456 nil nil)
!

testArrayWithPosition
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[4]">
<item SOAP-ENC:position="[1]">123</item>
<item SOAP-ENC:position="[3]">456</item>
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(nil 123 nil 456)
!

testArrayWithPositionIrreg
	self 
		setNodeWith: 
'<param1 SOAP-ENC:arrayType="xsd:int[10]">
<number SOAP-ENC:position="[3]">4</number> 
<number>5</number> 
<number SOAP-ENC:position="[8]">9</number> 
</param1>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = #(nil nil nil 4 5 nil nil nil 9 nil)
!

testSimpleIntWithType
	self setNodeWith: '<ans xsi:type="xsd:int">123</ans>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = 123
!

testSimpleString
	self setNodeWith: '<ans>abc</ans>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = 'abc'
!

testSimpleStringWithType
	self setNodeWith: '<ans xsi:type="xsd:string">abc</ans>'.
	self setReaderWithFirstChild.
	self assert: reader nodeObject = 'abc'
!

testStructLiteral
	self 
		setNodeWith: 
'<blah>
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</blah>'.
	reader := SplashNodeReader onNode: node firstChild.
	reader beLiteralEnc.
	reader decode.
	self assert: (reader nodeObject isKindOf: XeQStruct).
	self assert: (reader nodeObject at: (XeQName null: #varString)) = 'abc'.
	self assert: (reader nodeObject at: (XeQName null: #varInt)) = '123'.
	self assert: (reader nodeObject at: (XeQName null: #varFloat)) = '3.14'
!

testStructSoap
	self 
		setNodeWith: 
'<blah>
<varString xsi:type="xsd:string">abc</varString>
<varInt xsi:type="xsd:int">123</varInt>
<varFloat xsi:type="xsd:float">3.14</varFloat>
</blah>'.
	self setReaderWithFirstChild.
	self assert: (reader nodeObject isKindOf: XeQStruct).
	self assert: (reader nodeObject at: #varString) = 'abc'.
	self assert: (reader nodeObject at: #varInt) = 123.
	self assert: (reader nodeObject at: #varFloat) = 3.14
!

testStructSoapNoTypes
	self 
		setNodeWith: 
'<blah>
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</blah>'.
	self setReaderWithFirstChild.
	self assert: (reader nodeObject isKindOf: XeQStruct).
	self assert: (reader nodeObject at: #varString) = 'abc'.
	self assert: (reader nodeObject at: #varInt) = '123'.
	self assert: (reader nodeObject at: #varFloat) = '3.14'
!

testStructSoapWithNil
	self 
		setNodeWith: 
'<blah>
<varString xsi:type="xsd:string">abc</varString>
<varInt xsi:type="xsd:int">123</varInt>
<varFloat xsi:nil="true"/>
</blah>'.
	self setReaderWithFirstChild.
	self assert: (reader nodeObject isKindOf: XeQStruct).
	self assert: (reader nodeObject at: #varString) = 'abc'.
	self assert: (reader nodeObject at: #varInt) = 123.
	self assert: (reader nodeObject at: #varFloat) isNil
!

testStructWithIDHrefSpecial
	self setNodeWith: 
'<blah>
<p1 id="id1">abc</p1>
<p2 href="#id1"/>
</blah>'.
	self setReaderWithFirstChild.
	self assert: (reader nodeObject isKindOf: XeQStruct).
	self assert: (reader nodeObject at: #p1) = 'abc'.
	self assert: (reader nodeObject at: #p1) == (reader nodeObject at: #p2)
! !

!SplashNodeReaderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/tests/SplashNodeReaderTest.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
