"{ Package: 'stx:goodies/soap/splash/tests' }"

"{ NameSpace: SOAP }"

SplashRequestTests subclass:#SplashRPCRequestTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash-TestCases'
!


!SplashRPCRequestTest methodsFor:'accessing'!

addRPCWrapperBlah
	request addRPCWrapperNamed: ('method' ns: #'urn:blah')
!

getSoapStruct
	^(XeQStruct new)
		at: #varString put: 'abc';
		at: #varInt put: 123;
		at: #varFloat put: 3.14;
		yourself
! !

!SplashRPCRequestTest methodsFor:'asserting'!

assertRequestBodyXML: aString 
	self assertXML: request xml isSameAs: (
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Body>%1</SOAP-ENV:Body>
</SOAP-ENV:Envelope>' formatWith: aString)
!

assertRequestXML: aString 
	self assertXML: request xml isSameAs: aString
! !

!SplashRPCRequestTest methodsFor:'set up'!

setUp
	super setUp.
	request := SplashRPCRequest new
! !

!SplashRPCRequestTest methodsFor:'tests'!

testAddRPCWrapperNode
	self should: [request assertHasRPCNode] raise: Error.
	self addRPCWrapperBlah.
	request assertHasRPCNode.
	self assertRequestBodyXML: '<ns1:method xmlns:ns1="urn:blah"/>'
!

testAddRPCWrapperNodeTwice
	self addRPCWrapperBlah.
	self 
		should: [self addRPCWrapperBlah]
		raise: Error.
	 self assertRequestBodyXML: '<ns1:method xmlns:ns1="urn:blah"/>'
!

testBeSchema1999
	request beSchema1999.
	self assert: request qNameXSIType = ('type' ns: XeNamespace schema1999i).
	self 
		assertRequestXML: '<?xml version="1.0"?><SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><SOAP-ENV:Body/></SOAP-ENV:Envelope>'
!

testConstruction
	request := SplashRPCRequest method: 'method' ns: #'urn:blah'.
	self assertRequestBodyXML: 
'<m:method xmlns:m="urn:blah"></m:method>'
!

testHasParameter
	| sameString |
	sameString := 'abc'.
	self addRPCWrapperBlah.
	self deny: (request hasParameter: 'param1').
	self deny: (request hasParameter: 'param2').
	request parameterAt: 'param1' put: sameString.
	self assert: (request hasParameter: 'param1').
	self deny: (request hasParameter: 'param2').
	request parameterAt: 'param2' put: sameString.
	self assert: (request hasParameter: 'param1').
	self assert: (request hasParameter: 'param2').

!

testIsIDHref
	| sameStruct |
	sameStruct := self getSoapStruct.
	self addRPCWrapperBlah.
	request isIDHref: false.
	self assert: (request parameterAt: 'param1' put: sameStruct) localName = 'param1'.
	self assert: (request parameterAt: 'param2' put: sameStruct) localName = 'param2'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1>
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</param1>
<param2>
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</param2>
</ns1:method>'
!

testParamAt
	| answer |
	self addRPCWrapperBlah.
	answer := false.
	self assert: (request parameterAt: 'param1' ifAbsent: [answer := true]).
	self assert: answer .
	answer := false.
	request parameterAt: 'param1' put: 123.
	self assert: (request parameterAt: 'param1' ifAbsent: [answer := true]) localName = 'param1'.
	self deny: answer .

!

testParamAtPut
	self addRPCWrapperBlah.
	self assert: (request parameterAt: 'param1' put: 123) localName = 'param1'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1>123</param1>
</ns1:method>'
!

testParamAtPutAll
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putAll: #(123 456) itemType: (XePName xsd: 'int').
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:int[2]">
<item>123</item>
<item>456</item>
</param1>
</ns1:method>'
!

testParamAtPutAllEmpty
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putAll: #() itemType: (XePName xsd: 'int').
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:int[0]"/>
</ns1:method>'
!

testParamAtPutAllID
	| sameArray |
	sameArray := #(123 456).
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putAll: sameArray  itemType: (XePName xsd: 'int').
	request parameterAt: 'param2' putAll: sameArray  itemType: (XePName xsd: 'int').
	"self assertRequestBodyXML: "
'<ns1:method xmlns:ns1="urn:blah">
<param1 href="#id1"/>
<param2 href="#id1"/>
</ns1:method>
<id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0" SOAP-ENC:arrayType="xsd:int[2]">
<item>123</item>
<item>456</item>
</id1>'
!

testParamAtPutAllXeName
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putAll: #(123 456) itemType: 'xsd:int'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:int[2]">
<item>123</item>
<item>456</item>
</param1>
</ns1:method>'
!

testParamAtPutArray
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: #(123 456).
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:anyType[2]">
<item xsi:type="xsd:integer">123</item>
<item xsi:type="xsd:integer">456</item>
</param1>
</ns1:method>'
!

testParamAtPutArrayEmpty
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: #().
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:anyType[0]"></param1>
</ns1:method>'
!

testParamAtPutArrayWithNil
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: #(123 nil 456 nil).
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 SOAP-ENC:arrayType="xsd:anyType[4]">
<item xsi:type="xsd:integer">123</item>
<item xsi:nil="true"/>
<item xsi:type="xsd:integer">456</item>
<item xsi:nil="true"/>
</param1>
</ns1:method>'
!

testParamAtPutBase64
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: 'abc123' asByteArray.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah"><param1 xsi:type="xsd:base64Binary">YWJjMTIz</param1></ns1:method>'
!

testParamAtPutIDHref
	| sameStruct |
	sameStruct := self getSoapStruct.
	self addRPCWrapperBlah.
	self assert: (request parameterAt: 'param1' put: sameStruct) localName = 'param1'.
	self assert: (request parameterAt: 'param2' put: sameStruct) localName = 'id1'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 href="#id1"/>
<param2 href="#id1"/>
</ns1:method>
<id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0">
<varString>abc</varString>
<varInt>123</varInt>
<varFloat>3.14</varFloat>
</id1>'
!

testParamAtPutIDHrefNot
	| sameString |
	sameString := 'abc'.
	self addRPCWrapperBlah.
	self assert: (request parameterAt: 'param1' put: true) localName = 'param1'.
	self assert: (request parameterAt: 'param2' put: true) localName = 'param2'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1>true</param1>
<param2>true</param2>
</ns1:method>'
!

testParamAtPutIDHrefSpecial
	| sameString |
	sameString := 'abc'.
	self addRPCWrapperBlah.
	self assert: (request parameterAt: 'param1' put: sameString) localName = 'param1'.
	self assert: (request parameterAt: 'param2' put: sameString) localName = 'param1'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 id="id1">abc</param1>
<param2 href="#id1"/>
</ns1:method>'
!

testParamAtPutNil
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: nil.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 xsi:nil="true"/>
</ns1:method>'
!

testParamAtPutTypeP
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: 123 type: (XePName xsd: 'int').
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 xsi:type="xsd:int">123</param1>
</ns1:method>'
!

testParamAtPutTypeQ
	self addRPCWrapperBlah.
	request 
		parameterAt: 'param1'
		put: 123
		type: ('int' ns: XeNamespace schema2001).
	self 
		assertRequestBodyXML: '<ns1:method xmlns:ns1="urn:blah">
<param1 xsi:type="xsd:int">123</param1>
</ns1:method>'
!

testParamAtPutTypeS
	self addRPCWrapperBlah.
	request parameterAt: 'param1' put: 123 type: 'xsd:int'.
	self assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1 xsi:type="xsd:int">123</param1>
</ns1:method>'
!

testParamAtPutXmlString
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putXml: '<test>123</test>'.
	self 
		assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1><test>123</test></param1>
</ns1:method>'
!

testParamAtPutXmlStringSimple
	self addRPCWrapperBlah.
	request parameterAt: 'param1' putXml: '123'.
	self 
		assertRequestBodyXML: 
'<ns1:method xmlns:ns1="urn:blah">
<param1>123</param1>
</ns1:method>'
!

testValueNodeFor
	| sameString node1 node2|
	sameString := 'abc'.
	self addRPCWrapperBlah.
	node1 := request parameterAt: 'param1' put: sameString.
	self assert: (request valueNodeFor: node1) == node1.
	self assert: (request parameterAt: 'param2' put: sameString) == node1.
	self assert: (node2 := request parameterAt: 'param2') localName = 'param2'.
	self assert: (request valueNodeFor: node2) == node1.

        
! !

!SplashRPCRequestTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/tests/SplashRPCRequestTest.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
