"{ Package: 'stx:goodies/soap/splash/tests' }"

"{ NameSpace: SOAP }"

SplashTests subclass:#SplashResponseTest
	instanceVariableNames:'response'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash-TestCases'
!


!SplashResponseTest methodsFor:'accessing'!

sampleXmlWith: aString 
	^
'<?xml version="1.0"?><SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
 xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body>%1</SOAP-ENV:Body></SOAP-ENV:Envelope>' 
		formatWith: aString
!

setResponseWith: aString 
	response := SplashResponse new 
				envelope: (SpwSoapInEnvelope fromXmlString: (self sampleXmlWith: aString))
! !

!SplashResponseTest methodsFor:'set up'!

setUp
	super setUp.
	response := SplashResponse new 
! !

!SplashResponseTest methodsFor:'tests'!

testConvertTextArray
	self assert: (response convertTextArray: #('123' '456') as: 'xsd:int') = #(123 456).


        
!

testConvertTextArrayNil
	self assert: (response convertTextArray: nil as: 'xsd:string') isNil.


        
!

testConvertTextAs
	self assert: (response convertText: 'true' as: 'xsd:boolean').
        
!

testConvertTextStruct
	| struct answer |
	(struct := XeQStruct new)
		at: #varString put: 'abc';
		at: #varInt put: '123';
		at: #varFloat put: '3.14'.
	answer := response convertTextStruct: struct
				asAll: #(#(#varString 'xsd:string') #(#varInt 'xsd:int') #(#varFloat 'xsd:float')).
	self assert: (answer isKindOf: XeQStruct).
	self assert: (answer at: #varString) = 'abc'.
	self assert: (answer at: #varInt) = 123.
	self assert: (answer at: #varFloat) = 3.14
!

testConvertTextStructNil
	self assert: (response convertTextStruct: nil asAll: #((varString 'xsd:string'))) isNil.


        
!

testDontConvertArray
	self setResponseWith: 
'<response>
<return SOAP-ENC:arrayType="xsd:boolean[2]">
<item>true</item>
<item>123</item>
</return>
</response>'.
	response beDontConvertText.
	self assert: (response return first) = 'true'.
	self assert: (response return second) = '123'
        
!

testDontConvertComplex
	self setResponseWith: 
'<response>
<return>
<varString xsi:type="xsd:string">abc</varString>
<varInt xsi:type="xsd:int">123</varInt>
<varFloat xsi:type="xsd:float">3.14</varFloat>
</return>
</response>'.
	response beDontConvertText.
	self assert: (response return at: #varInt) = '123'.
	self assert: (response return at: #varFloat) = '3.14'
        
!

testDontConvertSimple
	self setResponseWith: '<response><return xsi:type="xsd:boolean">true</return></response>'.
	response beDontConvertText.
	self assert: response return = 'true'
        
!

testIXml
	self setResponseWith: '<input><return/></input>'.
	self assert: (response iXml isKindOf: IXMLDOMNode).
	self assertXML: response iXml xml
		isSameAs: (self sampleXmlWith: '<input><return/></input>')


        
!

testMissingReturn
	self setResponseWith: '<inputStringResponse></inputStringResponse>'.
	self assert: response return isNil
        
!

testMissingReturnAs
	self setResponseWith: '<inputStringResponse></inputStringResponse>'.
	self assert: (response returnAs: 'xsd:boolean') isNil
        
!

testModifiedDecoder
	self setResponseWith: 
'<response>
<return>
<varString xsi:type="xsd:string">abc</varString>
<varInt xsi:type="xsd:int">true</varInt>
<varFloat xsi:type="xsd:float">3.14</varFloat>
</return>
</response>'.
	response textDecoder atTypeName: (XeQName xsd2001: 'int') put: XeTextConverter boolean.
	self assert: (response return at: #varInt).
	self assert: (response return at: #varFloat) = 3.14.


        
!

testNilReturn
	self setResponseWith: '<inputStringResponse><return xsi:nil="true" ></return></inputStringResponse>'.
	self assert: response return isNil
        
!

testNilReturnAs
	self setResponseWith: '<inputStringResponse><return xsi:nil="true"></return></inputStringResponse>'.
	self assert: (response returnAs: 'xsd:boolean') isNil
        
!

testOutParameters
	| outs |
	self setResponseWith: 
'<inputStringResponse>
<p1 xsi:type="xsd:string">abc</p1>
<p2 xsi:type="xsd:boolean">true</p2>
</inputStringResponse>'.
	outs := response outParameters.
	self assert: (outs at: #p1) = 'abc'.
	self assert: (outs at: #p2).
	self assert: outs size = 2
        
!

testReturnAs
	self setResponseWith: '<inputStringResponse><return>true</return></inputStringResponse>'.
	self assert: (response returnAs: 'xsd:boolean')
        
!

testReturnAsOverrides
	self setResponseWith: '<inputStringResponse><return xsi:type="xsd:string">true</return></inputStringResponse>'.
	self assert: (response returnAs: 'xsd:boolean').
	response beDontConvertText.
	self assert: (response returnAs: 'xsd:boolean').
        
!

testSimpleTyped
	self setResponseWith: '<inputStringResponse><return xsi:type="xsd:string">abc</return></inputStringResponse>'.
	self assert: response return = 'abc'
        
!

testXml
	self setResponseWith: '<input><return/></input>'.
	self assertXML: response xml
		isSameAs: (self sampleXmlWith: '<input><return/></input>')
        
! !

!SplashResponseTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/tests/SplashResponseTest.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
