"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwBlockGuard subclass:#SpAuthMissingGuard
	instanceVariableNames:'authorizer'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth'
!

SpAuthMissingGuard comment:'I am a guard that is used by Auth services. I check that an exchange has been authorized (if the correct authorization information is in the soap:Header, the header processing places an authenticated property in the user namespace in the exchange.

If not authenticated, I tell the authorizer (which converts the exchange to a Fault, and place a soap:Header in the response that the client needs to re-authenticate
 '
!


!SpAuthMissingGuard methodsFor:'accessing'!

authorizer
	^authorizer
!

authorizer: anObject
	authorizer := anObject
! !

!SpAuthMissingGuard methodsFor:'asserting'!

assertAndPass: anExchange
	"If the header data does not included #authenticated. We must return a fault, with a AuthSCF header.
	-The Authorizer will insert the header, and make the exchange's out envelope a Fault"

	^anExchange atQP: #'user:authenticated' ifAbsent: [
		self missingAuthentication: anExchange.
		^false]
! !

!SpAuthMissingGuard methodsFor:'operations'!

missingAuthentication: anExchange
	"Private - Tell the authorizer that an attempt was made to access a protected operation without a Auth header"

	authorizer missingExchange: anExchange.
	^false
! !

!SpAuthMissingGuard class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SOAP__SpAuthMissingGuard.st,v 1.1 2008/09/07 16:08:45 cg Exp $'
! !
