"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpdMetaDataGenerator
	instanceVariableNames:'global flags port portNamespace selectorItems'
	classVariableNames:'ReplaceExisting'
	poolDictionaries:''
	category:'SOAP-Spray-Development'
!

SpdMetaDataGenerator comment:'I am used to analyze the source of a class and create a <SpmdPort>. A SpmdPort contains a number of <SpmdOperation>s, one for each method the class is exposing as a web service operation

In earlier versions of Spray I was controlled by a UI, but I am now used by a Service (when it #storeOperationPort)  

==========
Instance Variables:
	global		<Class>
	flags		<Integer>
	port		<SpmdPort>
	portNamespace	<Symbol>	
	selectorItems	<Collection> of SpdSelectorItems'
!


!SpdMetaDataGenerator class methodsFor:'initializing'!

initialize
	"
	self initialize
	"

	ReplaceExisting := 2
! !

!SpdMetaDataGenerator methodsFor:'accessing'!

createSelectorItems
	^self global selectors collect: 
			[:each | 
			(SpdSelectorItem new)
				selector: each;
				generator: self;
				generate: false;
				yourself]
!

flags
	flags isNil ifTrue: [flags := self defaultFlags].
	^flags
!

flags: anInteger
	flags := anInteger
!

global
	^global
!

global: aClass
	global := aClass
!

port
	^port
!

portDocumentation
	^(global respondsTo: #sprayPortDocumentation)
		ifTrue: [global sprayPortDocumentation]
		ifFalse: ['']
!

portName
	^self port name
!

portName: aString
	self port name: aString
!

portNamespace
	^portNamespace
!

portNamespace: aSymbol 
	portNamespace := aSymbol
!

replaceExisting
	^self flags anyMask: ReplaceExisting
!

replaceExisting: aBoolean
	flags := self flags mask: ReplaceExisting set: aBoolean
!

selectorItems
	^selectorItems
!

setForGlobal: aGlobal 
        global := aGlobal.
        port := self createNewPortData.
        self selectorItems removeAll.
        self selectorItems addAll: self createSelectorItems.
        self includeMethodsIn: 'spray generate' "(MethodCategory name: 'spray generate')"











!

soapAction: aString
	self port soapAction: aString
! !

!SpdMetaDataGenerator methodsFor:'initializing'!

defaultFlags
	^0
!

initialize
        super initialize.
        selectorItems := List "ListModel" new.
        flags := self defaultFlags.
        global := SprayOperationModel












! !

!SpdMetaDataGenerator methodsFor:'operations'!

createAndAdd: aSelectorItem
	"Create and add a <SpmdOperation> for the selector item, to the <SpmdPort> that we are constructing"

	| creator |
	creator := (SpdSourceAnalysis new setGlobal: global selector: aSelectorItem selector) 
				operationCreator.
	creator namespace: portNamespace.
	creator selector: aSelectorItem selector.
	self port addOperation: creator createOperation
!

createNewPortData
	^(SpmdPort new)
		name: global name asString;
		documentation: self portDocumentation;
		yourself
!

generateOperation: aSelectorItem
	aSelectorItem shouldGenerate ifFalse: [^nil].
	self createAndAdd: aSelectorItem
!

generateOperations
	"Create and answer a collection of <SpmdOperation>s.
	The selectorItems control which methods are analyzed and which are ignored.
		(defaults to analyzing all methods in the 'spray generate' category"

	self selectorItems do: [:each | self generateOperation: each]
!

methodFor: aSymbol
	^self global lookupMethod: aSymbol
!

suggestOperationNameFor: aString
	| count |

	(self port operations anySatisfy: [ :each | each name = aString]) ifFalse: [^aString].
	count := 0.
	[count := count + 1.
	self port operations anySatisfy: [ :each | each name = (aString , count printString)]] whileTrue.
	^aString , count printString
! !

!SpdMetaDataGenerator methodsFor:'private'!

includeMethodsIn: category
        | incSelectors |

        Smalltalk isSmalltalkX ifTrue:[
            incSelectors := self global methodDictionary keys select:[:sel | (self global compiledMethodAt:sel) category = category].
        ] ifFalse:[
            "/ Dolphin
            incSelectors := (category methodsInBehavior: self global) 
                                collect: [:each | each selector].
        ].
        self selectorItems do: [:each | each generate: (incSelectors includes: each selector)]













!

includesOperationFor: aSymbol 
	^self port operations anySatisfy: [ :each | each selectors includes: aSymbol]
! !

!SpdMetaDataGenerator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SOAP__SpdMetaDataGenerator.st,v 1.2 2008/09/07 17:30:47 cg Exp $'
! !

SpdMetaDataGenerator initialize!
