"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpdOperationCreator
	instanceVariableNames:'operation flags sourceAnalysis namespace selector'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Development'
!


!SpdOperationCreator class methodsFor:'instance creation'!

new
	^super new initialize
! !

!SpdOperationCreator methodsFor:'accessing'!

answerArrayFor: aString
	^Array with: self returnPartName with: (self trimQNameLiteral: aString)
!

bodyBindingWithParts: aCollection
	| bodyBinding |
	bodyBinding := SpmdSoapBody soapEncMessage: (SpmdMessage onParts: aCollection)
				ns: self namespace.
	bodyBinding handlerClass: self handlerClass.
	^bodyBinding
!

flags
	^flags
!

flags: anInteger
	flags := anInteger
!

handlerClass
	^SpwRPCHandler
!

isAnswerStringVoid: aString
	^#('void' 'nil') includes: aString
!

namespace
	^namespace
!

namespace: aSymbol
	"Private - Set the operation's namespace to aSymbol.
	Typically this should not be set, but left to default to the classes #sprayPortNamespace."

	namespace := aSymbol.
	(operation inputBody notNil and: [operation inputBody namespace isNil]) 
		ifTrue: [operation inputBody namespace: aSymbol].
	(operation outputBody notNil and: [operation outputBody namespace isNil]) 
		ifTrue: [operation outputBody namespace: aSymbol]
!

operation
	"Private - Answer the <SpmdOperation>"

	^operation
!

returnPartName
	^'Result'


!

selector
	^selector
!

selector: aSymbol
	"Private - Hard to check the validity of the selector ... the handler classes can do a better job of that "

	selector := aSymbol.
	operation inputBody notNil ifTrue: [
		operation inputBody selector: aSymbol].
!

setInputBodyWithParts: aCollection
	| bodyBinding |
	bodyBinding := self bodyBindingWithParts: aCollection.
	selector notNil ifTrue: [bodyBinding selector: selector].
	operation inputBody: bodyBinding
!

setOutputBodyWithParts: aCollection 
	
	| bodyBinding |
	bodyBinding := self bodyBindingWithParts: aCollection.
	operation outputBody: bodyBinding 
!

sourceAnalysis
	^sourceAnalysis
!

sourceAnalysis: aSpdSourceAnalysis
	sourceAnalysis := aSpdSourceAnalysis
! !

!SpdOperationCreator methodsFor:'helpers'!

trimQNameLiteral: aString
	self assert: [aString includes: $:].
	^aString last = $. ifTrue: [aString copyFrom: 1 to: aString size - 1] ifFalse: [aString]
! !

!SpdOperationCreator methodsFor:'initializing'!

initialize
	super initialize.
	operation := SpmdRPCOperation new
! !

!SpdOperationCreator methodsFor:'operations'!

createOperation
	"Answer a <SpmdOperation"

	sourceAnalysis assertNotImpliedSelfReturn.
	operation name isNil ifTrue: [operation name: self operationName].
	operation inputBody isNil ifTrue: [self setInputBodyFromSource].
	operation outputBody isNil ifTrue: [self setOutputBodyFromSource].
	operation renameMessages.
	^operation
!

operationName
	^self sourceAnalysis suggestedOperationName
!

partsFrom: anArray
	^anArray collect: 
			[:each | 
			SpmdTypePart partName: each first asString
				typeName: (self trimQNameLiteral: each second asString)]
!

setInputBodyFromSource
	self setInputBodyWithParts: self sourceAnalysis inputParts
!

setOutputBodyFromSource
	self setOutputBodyWithParts: self sourceAnalysis outputParts
! !

!SpdOperationCreator methodsFor:'setters'!

answer: aString
	"Sets the operation to have one return parameter of a type specified by aString. 
	The <String> argument either: 
		-has the form of a QName literal ie: 'prefix:local'
		- is 'void' or 'nil' which is interpreted as no return part.
	The name of the parameter (which under SOAP section7 rules is insignificant) defaults to 'Result'
	See class comments and #showHelp for more information"

	self assert: [aString isKindOf: String].
	self out: ((self isAnswerStringVoid: aString) 
				ifTrue: [#()]
				ifFalse: [Array with: (self answerArrayFor: aString)])
!

documentation: aString
	"Set the operation's documentation. This is included in a WSDL"

	self assert: [aString isNil or: [aString isKindOf: String]].
	operation documentation: aString
!

in: aCollection
	"Set the in parameters of the operation.
	-This should describe the arguments of the receivers method. 
	For each argument, we need to supply a parameter name and type.
	-aCollection is a collection of two element arrays. ie; #((param1 type1) (param2 type2) ... ) 
		-param has the form of a NCName literal ie: 'local'.
		-type has the form of a QName literal ie: 'prefix:local'.
	-If the operation has no input parameters (ie the method has no arguments), aCollection should be an empty array.
	See class comments and #showHelp for more information"

	self assert: [aCollection isKindOf: Collection].
	self assert: [aCollection allSatisfy: [:each | each size = 2]].
	self setInputBodyWithParts: (self partsFrom: aCollection)
!

name: aString
	"Sets the name of the operation.
	-This will default to the first keyword of the selector"

	operation name: aString
!

out: aCollection
	"Set the out parameters of the operation (alternatively use the #answer: setter)
	-If aCollection has a size greater than 1, the operation is assumed to have multiple out parameters, 
	and the method must answer a <Collection> of the same size as aCollection.
	-aCollection is a collection of two element arrays. ie; #((param1 type1) (param2 type2) ... ) 
		-param has the form of a NCName literal ie: 'local'.
		-type has the form of a QName literal ie: 'prefix:local'.
	-If the operation has no output parameters (ie void), aCollection should be an empty array.
	See class comments and #showHelp for more information"

	self assert: [aCollection isKindOf: Collection].
	self assert: [aCollection allSatisfy: [:each | each size = 2]].
	self setOutputBodyWithParts: (self partsFrom: aCollection)
! !

!SpdOperationCreator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SOAP__SpdOperationCreator.st,v 1.1 2008/09/07 16:35:31 cg Exp $'
! !
