"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpdTypeSuggestor
	instanceVariableNames:'argString xsdMatches'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Development'
!

SpdTypeSuggestor comment:'I am a type suggestor. I will take a guess at a smalltalk argument''s name and answer an xml name.

My use is generally deprecated. Best practice is to explicitly type parameters using SpdOperationCreators'
!


!SpdTypeSuggestor class methodsFor:'instance creation'!

new
	^super new initialize
! !

!SpdTypeSuggestor methodsFor:'accessing'!

suggestForArray
	^self doesXSDMatch ifTrue: [self suggestForXSDArray] ifFalse: [self suggestForOtherArray]
!

suggestForOtherArray
	^'soapE:ArrayOfanyType'
!

suggestForXSD
	^'xsd:' , self xsdMatch
!

suggestForXSDArray
	^'soapE:ArrayOf' , self xsdMatch
!

xsdMatch
	^(self xsdMatches detect: [:each | each key match: argString]) value
!

xsdMatches
	^xsdMatches
! !

!SpdTypeSuggestor methodsFor:'enquiries'!

suggestFor: aString 
	^self suggestFor: aString ifNone: [self error: 'Can not make a suggestion for: ' , aString displayString]
!

suggestFor: aString ifNone: operation
	argString := aString.
	self isArrayType ifTrue: [^self suggestForArray].
	self doesXSDMatch ifTrue: [^self suggestForXSD].
	^operation value
! !

!SpdTypeSuggestor methodsFor:'initializing'!

createXSDMatches
	xsdMatches := OrderedCollection new.
	xsdMatches
		add: '*String*' -> 'string';
		add: '*Float*' -> 'float';
		add: '*Integer*' -> 'integer';
		add: '*Int*' -> 'integer';
		add: '*Decimal*' -> 'decimal';
		add: '*Number*' -> 'number';
		add: '*XMLDateAndTime*' -> 'dateTime';
		add: '*Bool*' -> 'boolean';
		add: '*Boolean*' -> 'boolean';
		add: '*Date*' -> 'date';
		add: '*Time*' -> 'time'.
	^xsdMatches
!

initialize
	super initialize.
	self createXSDMatches
! !

!SpdTypeSuggestor methodsFor:'testing'!

doesXSDMatch
	^self xsdMatches anySatisfy: [:each | each key match: argString]
!

isArrayType
	^'*Array*' match: argString
! !

!SpdTypeSuggestor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SOAP__SpdTypeSuggestor.st,v 1.1 2008/09/07 16:35:46 cg Exp $'
! !
