"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdBlockBinding
	instanceVariableNames:'operationName handlerClass use namespace message partNames
		encStyleUri'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdBlockBinding comment:'I am a web service binding. I am the metadata that describes a soapBlock in a soap message.
Instances model the <soap:body> or <soap:header> nodes from WSDL;

<binding ....>
  <soap:binding ...>
  <operation ...>
    <soap:operation .../>
    <input>
       <soap:body  use="..."   namespace="..."  encodingStyle="..."/>
    </input>
    <output>
         <soap:body  use="..."  namespace="..."  encodingStyle="..."/>
     </output>
  </operation>

I am  used to create a "wire" object that decodes a soapBlock (either in the soap:Body or soap:Header) to an object, and encodes an object to a soapBlock (Note: a soapBlock is a string of xml)

Instance Variables:
	operationName	<String>
	handlerClass	<Class>
	use		<string>
	namespace	<Symbol>
	message		<SpmdMessage>
	partNames	<Collection>
	encStyleUri	<Symbol>
'
!


!SpmdBlockBinding class methodsFor:'instance creation'!

fromWSDLBinding: aWSDLBinding
	^self new setFromWSDLBinding: aWSDLBinding
!

literalMessage: aSpmdMessage 
	"Answer a new literal instance. (literal means it follows xml schema rules vs soap-enc rules)"

	^(self new)
		beLiteralStyle;
		message: aSpmdMessage;
		yourself
!

soapEncMessage: aSpmdMessage ns: aNSSymbol
	"Answer a new soap-enc instance. (soap-enc means it follows the soap section 5 rules)"

	^(self new)
		beSoapEncStyle;
		namespace: aNSSymbol;
		message: aSpmdMessage;
		yourself
! !

!SpmdBlockBinding class methodsFor:'constants'!

icon
	^SpmdPort icon
! !

!SpmdBlockBinding methodsFor:'accessing'!

docSelector: aSymbol
	"Set the selector to use for a document style operation with only one message (typical for DOTNET)"

	self message docSelector: aSymbol
!

docSelectors
	"Answer a collection of selectors.
	Note: theoretically there can be more than one, but typically there is only one"

	^message docSelectors: partNames
!

encStyleUri
	"Answer the uri <Symbol> that describes the encoding style of the receiver"

	^encStyleUri
!

namespace: aNSSymbol
	namespace := aNSSymbol
!

partNames: aCollection 
	self assertValidPartNames: aCollection.
	partNames := aCollection
!

selector: aSymbol 
	^self subclassResponsibility
! !

!SpmdBlockBinding methodsFor:'accessing-private'!

handlerClass: aSpHandlerClass
	"Set the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
	Subclass of <SpwSoapBHandler> differ in how they construct their SmalltalkMessage. 
		See definitions of #messageFor:exchange: "

	handlerClass := aSpHandlerClass
!

message
	"Answer a <SpmdMessage>"

	^message
!

message: aSpmdMessage
	message := aSpmdMessage
!

namespace
	"Answer a NS <Symbol>.
	In an rpc style body this describes the namespace of the SOAP RPC Wrapper element
		 <soap:Body>
			<m:operationName xmlns:m='namespace'/>
		  </soap:Body>
	"

	^namespace
!

operationName
	"Answer a <String>.
	In an rpc style body this describes the name of the SOAP RPC Wrapper element
		 <soap:Body>
			<m:operationName xmlns:m='namespace'/>
		  </soap:Body>
	"

	^operationName
!

operationName: aString
	operationName := aString
!

partNames
	"Answer the collection of partNames that the receiver uses from its message.
	This is (IMO an ugly) implementation detail of WSDL."

	^partNames
!

setEncodingStyleWith: aSymbol
	aSymbol == #'' 
		ifTrue: 
			[self beLiteralStyle.
			^self].
	aSymbol == XeNamespace soapEnc 
		ifTrue: 
			[self beSoapEncStyle.
			^self].
	self error: 'Encoding style not supported: ' , aSymbol displayString
!

setFromWSDLBinding: aWSDLBinding
	namespace := aWSDLBinding namespace.
	self setEncodingStyleWith: aWSDLBinding soapEncodingStyle
!

use
	^use
! !

!SpmdBlockBinding methodsFor:'asserting-private'!

assertValidPartNames: aCollection
	(aCollection notNil and: [aCollection isEmpty]) 
		ifTrue: [self error: 'Can not have an Empty Parts list, should be nil for default']
! !

!SpmdBlockBinding methodsFor:'modes'!

beDOCArgsEnvHandler
	"Set the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
		See SpwDOCArgsEnvHandler>>messageFor:exchange:"

	self handlerClass: SpwDOCArgsEnvHandler
!

beDOCHandler
	"Set the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
		See SpwDOCHandler>>messageFor:exchange:"

	self handlerClass: SpwDOCHandler
!

beLiteralStyle
	use := 'literal'.
	encStyleUri := #''
!

beSoapEncStyle
	use := 'encoded'.
	encStyleUri := XeNamespace soapEnc
! !

!SpmdBlockBinding methodsFor:'operations'!

encodingStyleIn: aXeTypespace
	"Answer the <XeEncodingStyle> that the typespace has registered for the receiver's encStyleUri"

	^aXeTypespace encStyleFor: self encStyleUri
! !

!SpmdBlockBinding methodsFor:'operations-private'!

renameMessageAppending: aString
	self assert: [operationName notNil].
	message name: operationName , aString
!

renameMessageAsInput
	self renameMessageAppending: 'Input'
!

renameMessageAsOutput
	self renameMessageAppending: 'Output'
! !

!SpmdBlockBinding methodsFor:'testing'!

isEncoded
	^use = 'encoded'
!

isSingleDocBody
	"Answer whether the document style body has 0 or 1 parts.
	If so we treat the body as a single unit with a single selector (typical for DOTNET)"

	^message hasNoneOrOneParts
! !

!SpmdBlockBinding methodsFor:'testing-private'!

isBody
	^false
! !

!SpmdBlockBinding methodsFor:'wire creation-coder'!

docCodersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBCoder>s from the receivers description of the soapBlock (either soap:Body or soap:Header)
	Notes:
		-Used by both clients and servers.
		-Typically in document style there is only 1 body <SpwSoapBCoder>.
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	| elements |
	elements := (self encodingStyleIn: aXeTypespace) 
				docElementsSpeced: (message docSpecs: partNames namespace: namespace).
	"The <SpwSoapBCoder> wraps an <XeElement>"
	^elements collect: [:each | SpwSoapBCoder onElement: each]
! !

!SpmdBlockBinding methodsFor:'wire creation-handler'!

docHandlersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBHandler>s
	Notes: 
		-Handlers are used by servers.
		-Typically in document style there is 1 handler for the request and 1 for the response.
		-Handlers are responsible for
			-(input) decoding xml soapBlocks into objects.
			-(input) creating a Smalltalk message from a soapBlock.
			-(output) encoding objects into xml soapBlocks"

	| handlers |
	self assert: [self handlerClass isDOCHandlerClass].
	handlers := OrderedCollection new.
	(self docCodersIn: aXeTypespace) with: self docSelectors
		do: 
			[:each :eachSelector | 
			handlers add: ((self handlerClass new)
						selector: eachSelector;
						inputDecoder: each;
						isBodyHandler: self isBody;
						yourself)
			"operation: operation;"].
	^handlers
!

rpcHandlerIn: aXeTypespace
	"Create and answer a <SpwSoapBHandler>
	Notes: 
		-Handlers are used by servers.
		-In rpc style there is 1 body handler for each operation.
		-Handlers are responsible for
			-(input) decoding xml soapBlocks into objects.
			-(input) creating a Smalltalk message from a soapBlock.
			-(output) encoding objects into xml soapBlocks"

	^self subclassResponsibility
! !

!SpmdBlockBinding methodsFor:'wsdl'!

defineBinding: node
	"Add the receiver to the WSDL binding <XeNode>"

	| soapNode |
	self assert: [#('literal' 'encoded') includes: use].
	soapNode := node addElement: self wsdlElementName.
	soapNode setAttribute: 'use' value: use.
	self isEncoded 
		ifTrue: 
			[self assert: [namespace notNil].	"empty is valid"
			soapNode setAttribute: 'encodingStyle' value: XeNamespace soapEnc.
			soapNode setAttribute: 'namespace' value: namespace].
	^soapNode
! !

!SpmdBlockBinding methodsFor:'wsdl-private'!

wsdlElementName
	^self subclassResponsibility
! !

!SpmdBlockBinding class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdBlockBinding.st,v 1.1 2002/11/18 19:41:26 cg Exp $'
! !
