"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdMetaDataScript
	instanceVariableNames:'global portName soapAction portNamespace'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdMetaDataScript comment:'I am used to generate a <SpmdPort> for an Operation class

I make use of a classes in the "SW Spray Development" package

Instance Variables:
	global		<Class>	The Operation class to be analyzed
	portName		<String>	The name of the SpmdPort (not significant)
	soapAction	<String>	The soapAction that the port''s operations will be use by default
	portNamespace	<Symbol>	The namespace of the port''s operations'
!


!SpmdMetaDataScript class methodsFor:'instance creation'!

newFor: sourceClass
	^(self new)
		global: sourceClass
			portName: sourceClass sprayPortName
			portNamespace: sourceClass sprayPortNamespace;
		yourself
! !

!SpmdMetaDataScript methodsFor:'accessing'!

global: aClass portName: aString portNamespace: aNSSymbol
	global := aClass.
	portName := aString.
	portNamespace := aNSSymbol
!

soapAction: aString
	soapAction := aString
! !

!SpmdMetaDataScript methodsFor:'asserting-private'!

assertValidPortNames
	(portName isNil or: [portNamespace isNil]) 
		ifTrue: [self error: 'Must specify a name and namespace for the generated port'].
	self assert: [portName  notEmpty].
	self assert: [portNamespace isKindOf: Symbol].
! !

!SpmdMetaDataScript methodsFor:'constants-private'!

generatorClass
	^self class environment at: #SpdMetaDataGenerator ifAbsent: [self errorDevelopmentOnly]
! !

!SpmdMetaDataScript methodsFor:'exceptions-private'!

errorDevelopmentOnly
	^self 
		error: 'MetaData can only be constructed using the Spray Development package in a Development image'
! !

!SpmdMetaDataScript methodsFor:'operations'!

createPort
	"Answer a <SpmdPort>"

	| generator port |
	generator := self constructGenerator.
	generator generateOperations.
	port := generator port.
	soapAction notNil ifTrue: [port soapAction: soapAction].
	^port
! !

!SpmdMetaDataScript methodsFor:'operations-private'!

constructGenerator
	"Private - self assertValidSourceNames."
	self assertValidPortNames.
	^(self generatorClass new initialize)
		setForGlobal: global;
		portName: portName;
		portNamespace: portNamespace;
		yourself
! !

!SpmdMetaDataScript methodsFor:'test accessing-private'!

portName: aString
	"Private - Test helper method"

	portName := aString
!

portNamespace: anNSSymbol
	"Private - Test helper method"

	portNamespace := anNSSymbol
! !

!SpmdMetaDataScript class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdMetaDataScript.st,v 1.1 2002/11/18 19:40:09 cg Exp $'
! !
