"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdPortIdentifier
	instanceVariableNames:'service name sourceClass'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdPortIdentifier comment:'I am used to access a SpmdPort which is saved as a binaryStoredObject.

I can also be asked to generate the SpmdPort using sourceAnalysis and save it into a SprayLocalService''s method.

Instance Variables:
	service		<SprayLocalService>
	name		<String>
	sourceClass	<Class>'
!


!SpmdPortIdentifier class methodsFor:'instance creation'!

service: aSprayService sourceClass: aClass name: aString
	^(super new)
		service: aSprayService
			sourceClass: aClass
			name: aString;
		yourself
! !

!SpmdPortIdentifier class methodsFor:'constants'!

storeCategory
	^MethodCategory name: 'spray store'
! !

!SpmdPortIdentifier methodsFor:'accessing'!

port
	"Answer the <SpmdPort> that is 'saved' as a binaryStoredObject in sourceClasses 'name' method (typically  SprayLocalService>>storedOperationPort)"

	^self service perform: self name asSymbol
! !

!SpmdPortIdentifier methodsFor:'accessing-private'!

name
	^name
!

name: aString
	name := aString
!

nameString
	^name isNil
		ifTrue: ['']
		ifFalse: [name asString]
!

service
	^service
!

service: aService sourceClass: aClass name: aString 
	service := aService.
	sourceClass := aClass.
	name := aString
!

sourceClass
	^sourceClass
! !

!SpmdPortIdentifier methodsFor:'operations'!

generateNewPort
	"Answer a <SpmdPort> which is constructed by analyzing the source code of sourceClass"

	^(SpmdMetaDataScript newFor: self sourceClass) createPort
!

store: aPort
	"Store the <SpmdPort> as a binaryStoredObject in the 'name' method of the sourceClass. (typically SprayLocalService>>storedOperationPort)"

	self service class compile: (self generateStoreMethodFor: aPort)
		categories: self storeCategories
! !

!SpmdPortIdentifier methodsFor:'operations-private'!

generateStoreMethodFor: aPortMetaData
	| stream ts |
	ts := TimeStamp current.
	stream := String writeStream.
	stream
		nextPutAll: self name;
		crtab;
		nextPutAll: '"Private- Answer a SpmdPort. ';
		crtab;
		nextPutAll: 'This method was automatically generated at ';
		nextPutAll: ts displayString;
		nextPut: $";
		cr;
		cr;
		tab;
		nextPutAll: '^Object fromBinaryStoreBytes: ##( ByteArray fromHexString: '''.
	aPortMetaData binaryStoreBytes printHexOn: stream.
	stream nextPutAll: ''')'.
	^stream contents
!

storeCategories
	^Array 
		with: self class storeCategory
		with: MethodCategory private
! !

!SpmdPortIdentifier class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdPortIdentifier.st,v 1.1 2002/11/18 19:41:58 cg Exp $'
! !
