"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpmdBlockBinding subclass:#SpmdSoapBody
	instanceVariableNames:'selector'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!


!SpmdSoapBody methodsFor:'accessing'!

selector
	^selector
!

selector: aSymbol 
	"For an RPC Operation, only one selector is needed for the body (In Doc each part has its own selector)"

	selector := aSymbol 
!

sourceFormat
	^self message sourceFormat
! !

!SpmdSoapBody methodsFor:'accessing-private'!

defaultInputQName
	^XeQName local: self operationName ns: self namespace
!

defaultOutputQName
	^XeQName local: self operationName , 'Response' ns: self namespace
!

handlerClass
	"Private - Answer the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
	Subclass of <SpwSoapBHandler> differ in how they construct their SmalltalkMessage. 
		See definitions of #messageFor:exchange: "

	handlerClass isNil ifTrue: [handlerClass := SpwRPCHandler].
	^handlerClass
!

setFromWSDLBinding: aWSDLBinding
	super setFromWSDLBinding: aWSDLBinding.
	partNames := aWSDLBinding partsAttribute.
	message := aWSDLBinding createBodyMessage
! !

!SpmdSoapBody methodsFor:'modes'!

beArgsEnvHandler
	"Set the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
		See SpwRPCArgsEnvHandler>>messageFor:exchange:"

	self handlerClass: SpwRPCArgsEnvHandler
!

beRPCHandler
	"Set the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
		See SpwRPCHandler>>messageFor:exchange:"

	self handlerClass: SpwRPCHandler
! !

!SpmdSoapBody methodsFor:'printing-private'!

printOn: aStream
    super printOn:aStream.
    aStream nextPut:$(.
    operationName printOn:aStream.
    aStream nextPut:$).
!

putSpraySignatureOn: aStream
	"Private - Put a readable description string on the stream for documentation purposes"

	message putSpraySignatureOn: aStream partNames: partNames
! !

!SpmdSoapBody methodsFor:'testing-private'!

isBody
	^true
! !

!SpmdSoapBody methodsFor:'wire creation-coder'!

rpcInputCoderIn: aXeTypespace
	"Create and answer a <SpwSoapBCoder> from the receivers description of the soap:Body
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	| coder |
	coder := SpwSoapBCoder 
				onElement: ((self encodingStyleIn: aXeTypespace) rpcWrapperNamed: self defaultInputQName
						elementSpecs: (message rpcSpecs: partNames)).
	coder elementContent isNoReturnPart: true.
	^coder
!

rpcOutputCoderIn: aXeTypespace
	"Create and answer a <SpwSoapBCoder> from the receivers description of the soap:Body
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	| coder |
	coder := SpwSoapBCoder 
				onElement: ((self encodingStyleIn: aXeTypespace) rpcWrapperNamed: self defaultOutputQName
						elementSpecs: (message rpcSpecs: partNames)).
	coder elementContent isNoReturnPart: message isNoReturnPart.
	^coder
! !

!SpmdSoapBody methodsFor:'wire creation-handler'!

rpcHandlerIn: aXeTypespace
	"Create and answer a <SpwSoapBHandler>
	Notes: 
		-Handlers are used by servers.
		-In rpc style there is 1 body handler for each operation.
		-Handlers are responsible for
			-(input) decoding xml soapBlocks into objects.
			-(input) creating a Smalltalk message from a soapBlock.
			-(output) encoding objects into xml soapBlocks.
		-The handler is constructed from the combined information of an 
		operation's input and output <SpmdSoapBody>"

	^(self handlerClass new)
		selector: selector;
		inputDecoder: (self rpcInputCoderIn: aXeTypespace);
		isBodyHandler: true;
		yourself
! !

!SpmdSoapBody methodsFor:'wsdl-private'!

defineBinding: node
	"Add the receiver to the WSDL binding <XeNode>"

	| soapNode |
	soapNode := super defineBinding: node.
	self assert: [partNames isNil]
!

wsdlElementName
	^'soap:body'
! !

!SpmdSoapBody class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdSoapBody.st,v 1.2 2005/02/09 12:55:19 stefan Exp $'
! !
