"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpmdBlockBinding subclass:#SpmdSoapHeader
	instanceVariableNames:'outputLink isOwned'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!


!SpmdSoapHeader class methodsFor:'instance creation'!

literalDeclared: aXeQName
	| message instance |
	message := SpmdMessage forSingleElement: aXeQName.
	message name: aXeQName localName , 'Header'.
	instance := self onOwnLiteralMessage: message.
	instance selector: (aXeQName localName , ':exchange:') asSymbol.
	^instance
!

literalMessage: aSpmdMessage 
	"Answer a new literal instance. (literal means it follows xml schema rules vs soap-enc rules)"

	^(self new)
		beLiteralStyle;
		message: aSpmdMessage;
		yourself
!

onOwnLiteralMessage: aSpmdMessage 
	"Instance creation helper method. Answer an instance, based on the aSpmdMessage.
	WSDL allows for a header to share a message part, this creates the instance as the sole
	user of the message"

	| instance |
	instance := self literalMessage: aSpmdMessage.
	instance makeOwnMessage: aSpmdMessage.
	^instance
!

onOwnSoapEncMessage: aSpmdMessage ns: aNSSymbol
	"Instance creation helper method. Answer an instance, based on the aSpmdMessage.
	WSDL allows for a header to share a message part, this creates the instance as the sole
	user of the message"

	| instance |
	instance := self soapEncMessage: aSpmdMessage ns: aNSSymbol.
	instance makeOwnMessage: aSpmdMessage.
	^instance
!

soapEncTypeName: aXePName partName: aString selector: aSymbol ns: aNSSymbol
	"Instance creation helper method"

	| message instance |
	message := SpmdMessage forSingleTypeName: aXePName partName: aString.
	message name: aString , 'Header'.
	message parts first selector: aSymbol.
	instance := self onOwnSoapEncMessage: message ns: aNSSymbol.
	^instance
! !

!SpmdSoapHeader methodsFor:'accessing'!

linkOutput: aSpmdSoapHeader
	"See #handlerIn: comment"

	outputLink := aSpmdSoapHeader
!

selector: aSymbol 
	"A soap:header can only contain one document part. We set the selector into that part."

	self docSelector: aSymbol 
! !

!SpmdSoapHeader methodsFor:'accessing-private'!

handlerClass
	"Answer the <SpwSoapBHandler> class to use when creating a handler (for use by servers).
	Subclass of <SpwSoapBHandler> differ in how they construct their SmalltalkMessage. 
		See definitions of #messageFor:exchange: "

	handlerClass isNil ifTrue: [self beDOCArgsEnvHandler].
	^handlerClass
!

handlerClass: aSpHandlerclass
	aSpHandlerclass isDOCHandlerClass 
		ifFalse: [self error: 'Header bindings can only be handled in a document style'].
	handlerClass := aSpHandlerclass
!

isOwned
	"Answer if the containing <SpmdOperation> is free to rename the header
	(the only requirement for names is that they are unique)"

	isOwned isNil ifTrue: [isOwned := true].
	^isOwned
!

outputLink
	"See #handlerIn: comment"

	^outputLink
!

setFromWSDLBinding: aWSDLBinding
	super setFromWSDLBinding: aWSDLBinding.
	partNames := Array with: aWSDLBinding partAttribute.
	message := aWSDLBinding createHeaderMessage
! !

!SpmdSoapHeader methodsFor:'modes'!

beNotOwned
	"Make the receiver independent of its operation. This allows the same header to be shared
		by multiple operations (typically the default)"

	isOwned := false
!

beOwned
	isOwned := true
! !

!SpmdSoapHeader methodsFor:'operations'!

makeOwnMessage: aSpmdMessage 
	(aSpmdMessage parts size ~~ 1) ifTrue: [self error: 'A Header must use a single part. This construction method assumes the message has a single part'].
	self partNames: (Array with: aSpmdMessage firstPartName).
	

! !

!SpmdSoapHeader methodsFor:'operations-private'!

renameMessageAppending: aString
	self isOwned ifTrue: [super renameMessageAppending: aString] ifFalse: []
! !

!SpmdSoapHeader methodsFor:'testing-private'!

hasOutputLink
	"See #handlerIn: comment"

	^outputLink notNil
! !

!SpmdSoapHeader methodsFor:'wire creation-coder'!

docCoderIn: aXeTypespace
	"Create and answer a <SpwSoapBCoder> from the receivers description of the soap:Header.
	Notes:
		-Used by both clients and servers.
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	| coders |
	coders := self docCodersIn: aXeTypespace.
	self assert: [coders size = 1].
	^coders first
! !

!SpmdSoapHeader methodsFor:'wire creation-coder-private'!

addOutputCoderTo: aHandler in: aXeTypespace
	| coders |
	coders := self docCodersIn: aXeTypespace.
	self assert: [coders size = 1].
	aHandler outputEncoder: coders first
! !

!SpmdSoapHeader methodsFor:'wire creation-handler'!

handlerIn: aXeTypespace
	"Answer a <SpwSoapBHandler> (for use in a SpwSoapResponder)
	This can only be document style, and can only have a single handler.
	There is no natural way to associate an input and output header. 
	-We provide a pointer, which if not nil, will become the outputEncoder, and encode an answer into the outEnvelope. 
	-Otherwise the header handler just receives messages; (its answer is not encoded)"

	| handlers |
	handlers := self docHandlersIn: aXeTypespace.
	self assert: [handlers size = 1].
	self hasOutputLink 
		ifTrue: [self outputLink addOutputCoderTo: handlers first in: aXeTypespace].
	^handlers first
!

rpcHandlerIn: aXeTypespace 
	self error: 'Header bindings can only be handled in a document style'
! !

!SpmdSoapHeader methodsFor:'wsdl-private'!

defineBinding: node
	"Add the receiver to the WSDL binding <XeNode>"

	| soapNode |
	soapNode := super defineBinding: node.
	self assert: [partNames size = 1].
	soapNode setAttribute: 'part' value: partNames first.
	self message putWSDLNameOn: soapNode
!

wsdlElementName
	^'soap:header'
! !

!SpmdSoapHeader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdSoapHeader.st,v 1.1 2002/11/18 19:35:58 cg Exp $'
! !
