"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Model subclass:#SprayAsyncModel
	instanceVariableNames:'client exchangeQueue processSize'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Client'
!

SprayAsyncModel comment:'SprayAsyncModel

I am a helper model that is used to provide async messaging for Presenters/Views. The user can queue any number of soap exchanges which are sent in a background process. Before queueing the exchange(s), it is told to send itself (and arguments that can be specified) to a receiver object once it has been processed. 

For a Dolphin presenter, this makes sending and receiving soap messages similar to receiving an event from a model.

I contain a SprayClient and a SptAsyncSendQueue. I provide access to the SprayClient (which is a factory to create exchanges <SpraySoapSend>s) and to the SptAsyncSendQueue which is used to process the exchanges.

I provide status information. When the state of the queue changes, I will trigger the #statusChanged event, and I can be accessed for various status information (#simpleStatus #sendsStatus #isProcessing)

A user of this model should queue the exchanges by accessing the SptAsyncSendQueue. This class provides a number of helper methods which enable both the queueing of the exchange and providing the information it needs to send itself to a receiver object once it has been processed.


=========
Instance Variables:
	client		<SprayClient>
	exchangeQueue	<SptAsyncSendQueue>
	processSize	<Integer>



'
!


!SprayAsyncModel class methodsFor:'instance creation'!

onClient: aSprayClient size: anInteger 
	^(self basicNew initializeFor: anInteger)
		client: aSprayClient;
		yourself
! !

!SprayAsyncModel methodsFor:'accessing'!

sendsStatus
	"Answer a string that can be used for status. 'Waiting' is the number of exchanges queued and waiting"

	^'Waiting: ' , (exchangeQueue isNil
		ifTrue: ['na']
		ifFalse: [ exchangeQueue  totalSize printString])
	
!

simpleStatus
	exchangeQueue isNil ifTrue: [^'Not connected'].
	^exchangeQueue  totalSize > 0
		ifTrue: ['Working']
		ifFalse: ['Ready']
	
!

sprayClient
	^client
! !

!SprayAsyncModel methodsFor:'accessing-private'!

client
	^client
!

client: anObject
	client := anObject
!

exchangeQueue
	"Private - Answer the value of the receiver's ''exchangeQueue'' instance variable."

	^exchangeQueue
!

exchangeQueue: anSptAsyncSendQueue 
	exchangeQueue := anSptAsyncSendQueue 
! !

!SprayAsyncModel methodsFor:'event handling'!

onExchangeQueued: anExchange 
	"Event sent by the queue. The exchange has been added to the queue."

	self notifyStatusChanged
!

onProcessed: anExchange 
	"Event sent by the queue. The exchange has been processed."

	self notifyStatusChanged
! !

!SprayAsyncModel methodsFor:'initializing-private'!

initializeFor: anInteger
	"Private - Initialize with a queue that will send anInteger exchanges concurrently"

	self initialize.
	processSize := anInteger.
	self setNewQueue
!

setNewQueue
	exchangeQueue := SptAsyncSendQueue newProcessing: processSize.
	exchangeQueue 
		when: #processed:
		send: #onProcessed:
		to: self.
	exchangeQueue 
		when: #exchangeQueued:
		send: #onExchangeQueued:
		to: self
! !

!SprayAsyncModel methodsFor:'operations'!

closeQueue
	exchangeQueue close
!

notifyStatusChanged
	self trigger: #statusChanged
!

resetQueue
	self closeQueue.
	self setNewQueue.
	self notifyStatusChanged
! !

!SprayAsyncModel methodsFor:'testing'!

isProcessing
	exchangeQueue isNil ifTrue: [^false].
	^exchangeQueue  totalSize > 0
	
! !

!SprayAsyncModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayAsyncModel.st,v 1.1 2002/11/18 19:36:20 cg Exp $'
! !
