"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SprayDispatchClient
	instanceVariableNames:'sprayClient exchange'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayDispatchClient comment:'I am a proxy like object to a webservice. I can be sent messages, which I convert to soap messages and send them to the webservice. I then extract the soap response return, and answer it.

"Example"
	client := TemperaturesService new createClientTo: ''http://www.dolphinharbor.org/services/temper''.
	dispatchClient := SpsDispatchClient onClient: client.
	dispatchClient c2f: 30.5'
!


!SprayDispatchClient class methodsFor:'instance creation'!

onClient: aSpSprayClient
	"Answer an instance of the receiver.
	This wraps a SpSprayClient and uses DNU to convert Smalltalk messages into SoapMessages"

	^self new sprayClient: aSpSprayClient
! !

!SprayDispatchClient class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'client.ico'
    ]
! !

!SprayDispatchClient class methodsFor:'must strip'!

example
        <resource: #example>

        "Requires the 'SW WS Temperatures' package.
        Spray has a lightweight client that implements #doesNotUnderstand "

        | service client dispatchClient |
        client := (SprayWSDLService 
                                onUrl: 'http://www.dolphinharbor.org/services/temper/service.wsdl') createClient.
        "service := (self environment at: #TemperaturesService) new.
        client := service createClientTo: 'http://www.dolphinharbor.org/services/temper'."
        dispatchClient := SprayDispatchClient onClient: client.
        ^dispatchClient c2f: 30.5
! !

!SprayDispatchClient methodsFor:'accessing-private'!

soapSend
	^exchange
!

sprayClient: aSprayClient
	sprayClient := aSprayClient
! !

!SprayDispatchClient methodsFor:'converting-private'!

convertToSoap: aMessage
	exchange := sprayClient send: (self extractOperationName: aMessage)
				withArguments: aMessage arguments
! !

!SprayDispatchClient methodsFor:'evaluating'!

value
	^exchange value
! !

!SprayDispatchClient methodsFor:'helpers-private'!

extractOperationName: aMessage
	^(aMessage selector subStrings: $:) first asString
! !

!SprayDispatchClient methodsFor:'message dispatching-private'!

doesNotUnderstand: aMessage
	^(self hasOperationFor: aMessage) 
		ifTrue: 
			[self convertToSoap: aMessage.
			self value]
		ifFalse: [super doesNotUnderstand: aMessage]
! !

!SprayDispatchClient methodsFor:'test accessing-private'!

sprayClient
	"Test helper method"

	^sprayClient
! !

!SprayDispatchClient methodsFor:'testing-private'!

hasOperationFor: aMessage
	"Private - Answer whether an operation exists which matches the Smalltalk message.
	The match test is very simple at this stage 
		... it checks for an operation with the same name as the selectors first keyword"

	^sprayClient includesOperationNamed: (self extractOperationName: aMessage)
! !

!SprayDispatchClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayDispatchClient.st,v 1.4 2010/02/26 12:02:54 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayDispatchClient.st,v 1.4 2010/02/26 12:02:54 cg Exp $'
! !
