"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SprayService subclass:#SprayLocalService
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayLocalService comment:'I am the interface of a web service that is implemented in this image

I can construct SprayClients that are used to access operations defined by my interface.
I can construct SprayServers that are used to expose the interface and models that implement it.

==========
Overview

"Operations" are methods exposed by the service (term used in WSDL spec).

"Soap Blocks" are xml contained in soap messages (term used in SOAP1.2 spec). They are the direct children of soap:Header and soap:Body. From the spec draft:
	"An element information item used to delimit data that logically constitutes a single computational unit "

	operationModelClass	The class that will have its methods exposed as operations.
	operationModel 		Defaults to an instance of the operationModelClass. This instance will be wrapped 
				by a SpwSoapBReceiver and messages to the namespace defined by
				operationModelClass>>sprayPortNamespace sent to it.
	operationReceiver 		Answers an instance of a SpwSoapBReceiver that wraps the operationModel.
	soapBReceivers 		Answers a collection of SpwSoapBReceiver that by default includes the operationReceiver .
				Can include additional soapBlockReceivers with other namespaces to process headers.
	soapBHandlers		Answers a collection of SpwSoapBHandlers that are used to decode soapBlocks into objects
				construct Smalltalk Messages (which are sent by the soapBReceiver) and encode the
				answer into an out message soapBlock
	operationPort		A <SpmdPort>

==========
Generating the OperationPort

Two techniques are used:
	-Source Analysis (A models source is analyzed and an interface constructed for it See SprayOperationModel comment)
	-External WSDL Import. A 3rd party WSDL is imported and selectors attached to its operations (See SprayExternalWSDLServiceAbstract)

Source Analysis requires the "Steve Waring\Spray WebServices\Spray\Core\SW Spray Development" package.

==========
Operation Port

The operationPort is generated (send a service instance #storeOperationPort) and saved into the services #storedOperationPort method. This is an instance of SpmdPort saved as a Dolphin STB byteArray. The operationPort needs to be regenerated if new operations are added, or any of the metadata changes (i.e. parameter type information). The generation of the storedOperationPort can only be performed in a development image, but the service can be used in a deployed executable.

==========
SOAPAction

Spray by default uses an operation''s name as its SoapAction. Services should use the event #onPortGenerated: to set a specific soapAction. The method SpmdPort>>formatSoapActionWith: is a helper method which takes a Win32 format string, and formats it with each operations name.

==========
WSDL

Spray automatically generates a WSDL for a service. By implementing the method #hasDefinitionWSDL to answer true, Spray will not generate the WSDL, but instead use a WSDL string supplied by #definitionWSDL.

If the service answers true to #useEFDQSchemas Spray will write schemas with the elementFormDefault=qualified attribute.This is not necessary as Spray will use the element form attribute to correctly write elements whatever this attribute is set to. However if all the elements in a schema are qualified (ie a typical DOC/Literal service), answering true will write a ''neater'' schema (and may help with interop for schema readers that do not take into account the form attribute" 

==========
Example Modifying existing services

("I" == Steve Waring (not an instance!!))

I would suggest the following steps when altering the definition of a
service;

1/ Ensure that Swazoo is not serving by evaluating "SwazooSiteRegistry
flush".

2/ Add your new method to the model and then evaluate "DemoService new
storeOperationPort". (This evaluation does a limited form of validation of
the service and should root out any errors). all the model''s methods in the "spray generate"
category are analyzed. 

3/ For a quick check that the definition is what you expect, evaluate
"DemoService new viewLocalWSDL". This will open a browser on the WSDL. You
should see references to a "newMessage" operation.

4/ Create a server by evaluating "SwazooSiteScript startUpOn:
(SpSwazooServiceResource uriPattern: ''hibye'' service: DemoService new)."
Once you have done this, in a browser go to http://localhost/hibye and you
should see the "newMessage" operation listed.

5/ Create a client by evaluating "client := DemoService new createClientTo:
''http://localhost/hibye''" Once you have done this, send client the message
#operationNames, and again you should see "newMessage" as an operation name.


==========
Why are my Arrays using the http://tempuri.org/schema namespace? How can I change this?

Add the following method:
	MyServiceClass>>onTypespaceCreated
		super onTypespaceCreated
		typespace
			replaceSchemaNamespaceOf: ''soapE'' to: #''urn:myNamespace'';
			

Background:

By default a typespace is configured with the prefix: "soapE" mapped to the namespace: "http://tempuri.org/schemas" which is targeted by an XeSchema instance containing an XeSoapEncFactory. (The XeSoapEncFactory creates Arrays of the base schema dataTypes).

The use of the specific prefix ''soapE'' and the namespace http://tempuri.org/schemas are arbitrary and can be replaced if necessary. An XePName with the prefix ''soapE'', uses the information in the typespace to late bind to the namespace mapped to ''soapE''.

See #exampleQNames




'
!


!SprayLocalService class methodsFor:'helpers-private'!

regenerateAll
	"
	self regenerateAll
	"

	(self allSubclasses select: [:each | ('*Abstract*' match: each name) not]) 
		do: [:each | each new storeOperationPort]
! !

!SprayLocalService class methodsFor:'must strip'!

exampleCreateClient
        <resource: #example>

        "Requires 'SW WS Temperatures' package."

        | service client exchange |
        service := (self environment at: #TemperaturesService) new.
        "A Spray local service interface does not contain an 'addressLocation' as this is not determined until the service is deployed as a server.
        Therefore we need to supply the 'addressLocation' (ie endPoint Url) when creating a client"
        client := service createClientTo: 'http://www.dolphinharbor.org/services/temper'.
        exchange := client send: 'c2f' with: 30.5.
        ^exchange value
!

exampleCreateServer
        <resource: #example>

        "Requires 
                'SW WS Temperatures' package.
                Steve Waring\Spray WebServices\Spray\Transport\SW Spray Swazoo.pac
                Steve Waring\Swazoo\SZ Swazoo Server.pac"

        | serviceClass swazooSiteScriptClass swazooSiteRegistryClass client exchange |
        serviceClass := self environment at: #TemperaturesService.
        swazooSiteScriptClass := self environment at: #SwazooSiteScript.
        swazooSiteRegistryClass := self environment at: #SwazooSiteRegistry.
        "Start Swazoo"
        swazooSiteScriptClass 
                startUpIP: '127.0.0.1'
                service: serviceClass new
                uriPattern: 'temper'.
        "Go to: http://localhost/temper'' in a browser."
        "Open the Tools/Additional Tools/SwazooShell to view the binding"
        "Send a soap message to the server"
        client := serviceClass new createClientTo: 'http://localhost/temper'.
        exchange := client send: 'c2f' with: 30.5.
        exchange value.

        "To stop Swazoo"
        swazooSiteRegistryClass flush
!

exampleGeneratePort
        <resource: #example>

        "Requires 'SW WS Temperatures' package."

        | service |
        service := (self environment at: #TemperaturesService) new.
        service storeOperationPort
!

exampleQNames
        <resource: #example>

        "===Arrays===
        By default a typespace is configured with the prefix: 'soapE' mapped to the namespace: 'http://tempuri.org/schemas' which is targeted by an XeSchema instance containing an XeSoapEncFactory. (The XeSoapEncFactory creates Arrays of the base schema dataTypes).
The use of the specific prefix 'soapE' and the namespace http://tempuri.org/schemas are arbitrary and can be replaced if necessary."

        "To see this evaluate:"

        | ts typeName element |
        XeTypespace defaultSoapEnc2001 inspect.

        "A XePName with the prefix 'soapE', uses the information in the typespace to late bind to the namespace mapped to 'soapE'.
To see this debug:"
        ts := XeTypespace defaultSoapEnc2001.
        'soapE:ArrayOfstring' asXeName qualifiedNameIn: ts.

        "To encode an Array:"
        ts := XeTypespace defaultSoapEnc2001.
        "ts options beXSITypeArrays."
        typeName := 'soapE:ArrayOfstring' asXeName.
        element := (XeElementSpec named: #myArray typeName: typeName) 
                                xeParticleWith: ts soapEncStyle.
        "element beAllTyped."
        (XeSoapXMLWriter write: #('hello' 'goodbye') element: element) xml

        "<root
SOAP-ENC:arrayType='ns1:string[2]'
xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/'
xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
xmlns:ns1='http://www.w3.org/2001/XMLSchema'>
<item>hello</item>
<item>goodbye</item>
</root>"
!

exampleViewWSDL
        <resource: #example>

        "Requires 'SW WS Temperatures' package."

        | service |
        service := (self environment at: #TemperaturesService) new.
        service localWSDL.      "Inspect It "
        service viewLocalWSDL   "Evaluate It - creates a temporary file and opens it in IE"
! !

!SprayLocalService methodsFor:'accessing-namespace'!

name
        "Answer the name of the service, subclasses should override this method if the class name is not appropriate.
        This will be the name of the service in its WSDL"

        "/ Smalltalk/X names may contain colons
        ^ self class name asString replaceAll:$: with:$_ 
        "/ ^self class name asString
!

namespace
	"Answer the namespace of the service, subclasses should override this method if the default is not appropriate.
	This will be the targetNamespace of the service WSDL"

	^('urn:' , self name) asSymbol
! !

!SprayLocalService methodsFor:'accessing-port-private'!

operationPort
	"Answer a <SpmdPort>. This is usually stored in the storedOperationPort method, however it should
	always be accessed through a SpmdPortIdentifier so that alternate methods of persistence can be used in the future"

	^self operationPortID port
!

operationPortID
	"Private - Answer a SpmdPortIdentifier. Unlikely that subclasses will need to override.
	The use of an SpmdPortIdentifier played a larger role in previous versions, and is used to access the <SpmdPort> stored as a binaryStoredObject"

	^SpmdPortIdentifier 
		service: self
		sourceClass: self operationModelClass
		name: 'storedOperationPort'
! !

!SprayLocalService methodsFor:'accessing-receiving objects-private'!

operationModel
	"Private - Answer the instance that implements the methods of the service.
		-This is wrapped by a SpwSoapBReceiver ... see #operationReceiver comment
	Override if necessary for more specific instance creation"

	^self operationModelClass new
!

operationModelClass
	"Private - Answer the operation model class of the service (ie it implements the operations of the service)"

	^self subclassResponsibility
!

operationReceiver
	"Private - Answer a SpwSoapBReceiver . This holds the object (operationModel) that will be 
		the receiver of the Smalltalk messages created from the SoapBlocks in the body of a 
		SoapEnvelope, that have the same namespace as the SpwSoapBReceiver.
	A SpwSoapResponder (ie a Server) can have multiple SpwSoapBReceivers, this methods 
		answers the default.
	In the example below, information in the SpmdOperation named 'someOperation' is used to 
		convert the parameters into Smalltalk objects, and a Smalltalk message will be sent 
		by this SpwSoapBReceiver , to the #operationModel.

	<soap:Body>
	  <m:someOperation xmlns:m='sprayPortNamespace'>
	    <parameter>....</parameter>
	     ....
	  </m:someOperation>"

	^SpwSoapBReceiver receiveFor: self operationModel
		at: self operationModelClass sprayPortNamespace
!

soapBReceivers
	"Answer a collection of <SpwSoapBReceiver>s for the service. By default this contains the operationReceiver, however servers can be configured with multiple <SpwSoapBReceiver>s.
	-Override to add receivers for Header soapBlocks
	-(I dont see why it would be necessary to have multiple receivers for body blocks, but it is possible)"

	^OrderedCollection with: self operationReceiver
! !

!SprayLocalService methodsFor:'accessing-service'!

definitionWSDL
	"Answer a string or nil.
	If the receiver answers true to #hasDefinitionWSDL, a SprayServer will not generate a WSDL for the service, but will respond with the string answered by this method. If this method answers false, the transport binding will respond with a 404"

	^nil
!

destinationUrl
	"Answer a default destinationUrl <String> that will be used by clients created from this service"

	^''
!

documentation
	"Answer the documentation string that will be included in the services WSDL"

	^''
!

endPointEntityClass
	"Answer the class that a SprayServer will use to create the endPointEntity html.
	For an example of the default see: <http://www.dolphinharbor.org/services/hibye>"

	^SprayServerEndPointHTML
!

importOnlySchemaNamespaces
	"Answer a collection of namespaces that should be imported when creating a WSDL schema, 
		-ie we do not provide a schema targeting these namespaces"

	| nsColl |
	nsColl := OrderedCollection new.
	nsColl 
		addAll: XeNamespace schemaNamespaceURIs;
		add: XeNamespace soapEnc.
	^nsColl

!

privateOperations
	"Answer a collection of operation names that should not be added to the WSDL for the service"

	^#()

! !

!SprayLocalService methodsFor:'client server creation'!

createServer
	"Answer a <SprayServer> for this service."

	^self createServer: SptLocalServerBinding new
!

createServer: aSptReceiveBinding
	"Answer a <SprayServer> for this service that will use SptReceiveBinding to receive soap messages"

	^self createServer: aSptReceiveBinding soapBReceivers: self soapBReceivers
!

localWSDL
	"Answer the receivers WSDL as an xml string.
	-Use a default local <SptReceiveBinding> which will supply 'localhost' as the addressLocation of the server"

	^(self createWSDL: SptLocalServerBinding new) xmlFormatted
!

viewLocalWSDL
	"Opens IE and displays the WSDL of the service.
	Creates a temporary file and uses shellOpen.
	Note: the temporary file is not deleted "

	^self localWSDL browserView: 'xml'
! !

!SprayLocalService methodsFor:'client server creation-private'!

createServer: aSptReceiveBinding soapBReceivers: aCollection
	"Answer a <SprayServer> "

	^self 
		createServer: aSptReceiveBinding
		soapBReceivers: aCollection
		port: self operationPort
!

createServer: aSptReceiveBinding soapBReceivers: aCollection port: aSpmdPort
	"Answer a <SprayServer> that 
		-will receive soapMessages via a<SptReceiveBinding> (typically a <SpSwazooBinding>
		-will send SmalltalkMessage to the soapBReceivers
		-has its interface described by aSpmdPort"

	| server |
	server := (SprayServer newIn: self typespace) 
				port: aSpmdPort
				transport: aSptReceiveBinding
				service: self.
	server createOperationHandlers.
	server addSoapBReceivers: aCollection.
	^server
!

createWSDL: aSptReceiveBinding
	"Answer a wsdl XeDoc for the service. 
	The <SptReceiveBinding> is necessary to determine the addressLocation of the service"

	^(self createServer: aSptReceiveBinding) wsdl
! !

!SprayLocalService methodsFor:'must not strip-private'!

sstoredReferences
	#todo.
	"Lagoon Hack ... the following classes exist in the storedOperationPort.
	-A cleaner way would be to write an ImageStripper that could read the storedOperation port"

	SpmdPort.
	SpmdPart.
	SpmdElementPart.
	SpmdTypePart.
	SpmdRPCOperation.
	SpmdDOCOperation.
	SpmdMessage.
	SpmdSoapHeader.
	SpmdSoapBody.
	SpmdBlockBinding.
!

storedOperationPort
	"Answer an SpmdPort for the receiver.
	-In many of the sample services this is automatically generated.
	--To generate send #generateStoredPort to the receiver"

	^self subclassResponsibility
! !

!SprayLocalService methodsFor:'port generation'!

storeOperationPort
	"Create and store an instance of <SpmdPort> into the method #storedOperationPort.
	-By default this generates the <SpmdPort> from the source of the receivers operationModelClass.
	--This can only be evaluated in a development image,and requires the 'SW Spray Development' package to be installed. 
	--This method only need to be (re)sent when the interface of the service changes. 
	See the comments for #generateOperationPort #onPortGenerated: #assertValidPort:"

	| port |
	port := self generateOperationPort.
	self onPortGenerated: port.
	self assertValidPort: port.
	self operationPortID store: port
! !

!SprayLocalService methodsFor:'port generation-private'!

assertValidPort: aSpmdPort
	"Private - This puts a SpmdPort through a variety of functional tests, with the aim of flushing out any errors before it is saved"

	| server wsdl wsdlService |
	aSpmdPort assertValidIn: self typespace.
	"can create a server?"
	server := self 
				createServer: SptLocalServerBinding new
				soapBReceivers: #()
				port: aSpmdPort.
	"can create a wsdl?"
	wsdl := server wsdl.
	"can create a wsdl service?"
	wsdlService := SprayWSDLService definitionNode: wsdl asParserNode portName: nil.
	wsdlService assertValidOperations.
	self assert: [wsdlService hasInvalidOperations not].
	self assert: 
			[wsdlService operationPort operations size + self privateOperations size 
				= aSpmdPort operations size]
!

generateOperationPort
	"Generate and answer a <SpmdPort>. 
	Subclasses can override this method to use alternate methods of generating a <SpmdPort>.
		-By default this will use 'Source Analysis' and analyze the source code of the operationModelClass 
		and create soap operations for all methods in the 'spray generate' category of the operationModelClass.

		-The subclasses of <SprayExternalWSDLServiceAbstract> use 'External WSDL' to generate a <SpmdPort>.
		They parse an External WSDL string and insert selectors into the operations. An operationModel is still 
		required to implement the operations, but the source is not analyzed.

	See SprayOperationModel class comment.
	See the 'Steve Waring\Spray WebServices\Services\Temperatures\' packages for a sample that 
	implements the same service using both techniques"

	^self operationPortID generateNewPort
!

onPortGenerated: port
	"Handle the onPortGenerated: sent after a SpmdPort is generated, but before it is saved. 
	This is the opportunity to; 
		-add headers
		-attach selectors to ports generated from 3rd party metadata
		-add operation from other classes
		-change the SOAPAction from its default"
	
! !

!SprayLocalService methodsFor:'testing'!

hasDefinitionWSDL
	"Answer whether a SprayServer created from the receiver, will not generate a WSDL, but will ask the receiver for it's definitionWSDL (either a String or nil)"

	^false
!

hasVoidBodyOperation
	^false
!

useEFDQSchemas
	"Answer should schemas be written with the elementFormDefault=qualified attribute?
	-This is not necessary as Spray will use the element form attribute to correctly write elements whatever this attribute is set to. 
		--However if all the elements in a schema are qualified, answering true will write a 'neater' schema (and may help with interop for schema readers that do not take into account the form attribute"

	^false
! !

!SprayLocalService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayLocalService.st,v 1.3 2008/09/08 10:35:28 cg Exp $'
! !
