"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SprayOperationModel
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayOperationModel comment:'Subclasses are the classes that implement the operations of a webservice.

A class does not need to inherit from this class to be an operation model.

See the tutorial: 
	#showHelp

==========
Constructing the <SpmdPort>

A <SpmdPort> describes the interface used to access the <SprayOperationModel>. Objects are created from the <SpmdPort> which process soap messages and encode objects to xml, and decode xml to objects. This allows soap messages to be converted into Smalltalk messages and be sent to a SprayOperationModel.

Source analysis is one method of constructing a <SpmdPort>  The other method commonly used in the Spray Samples, is to parse a supplied WSDL and insert selectors into the <SpmdOperation>s.

Source analysis is best used when you are defining both the interface and implementation of the webservice.
ExternalWSDL is best used when you are implementing a pre-defined service and have access to an existing WSDL.

===============
Source Analysis

If you want to use Spray''s Source Analysis you need to have the "SW Spray Development" package installed.

The SprayOperationModel class needs to:

	1/ Define two class methods: #sprayPortName and #sprayPortNamespace. 
	(#sprayPortName is for documentation purposes only). 
	#sprayPortNamespace is the namespace of all the operations created from 
	the classes methods.

	2/ Place methods that are to have generated operations in the "spray generate" category.

	3/ Add a comment to each  method which evaluates to a <SpdOperationCreator>.

Then in the model''s  <SprayLocalService>:

	1/Generate the operationPort by sending an instance of the service 
	the #storeOperationPort message.  This creates  the services #storedOperationPort method. 
	This is an instance of SpmdPort saved as a Dolphin STB byteArray. The operationPort 
	needs to be regenerated if new operations are added, or any of the metadata changes 
	(i.e. parameter type information). The generation of the storedOperationPort can only be 
	performed in a development image, but the service can be used in a deployed executable.


For Example, from the Temperatures service:

	Temperatures is the operationModel class.
	TemperaturesService is the <SprayLocalService> class.

==
	Temperatures>>c2f: aFloat 
		"SpdOperationCreator new
			in: #((param1 xsd:float));
			answer: ''xsd:float''''
		^32 + (aFloat 9 / 5)
==
	Temperatures (class)>>sprayPortNamespace
		^#''http://www.soaplite.com/Temperatures''
==
Evaluating:

	TemperaturesService new storeOperationPort

creates an operation which will process the following soap message:

	<SOAP-ENV:Envelope
	  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
	  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
	  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
	  <SOAP-ENV:Body>
	    <m:c2f 
	      SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
	      xmlns:m="http://www.soaplite.com/Temperatures">
	        <param1 xsi:type="xsd:float">30.5</param1>
	    </m:c2f>
	  </SOAP-ENV:Body>
	</SOAP-ENV:Envelope>


===============
Exceptions

A subclass of Exception(SoapServerFault)  contains instance variables for the faultcode and detail.

The typical way for a model to signal this kind of exception would be:
	SoapServerFault beClient signal: ''Error - can not find that color''

When processing an incoming soap request, Spray sets up two exception
handlers. The first handles SoapServerFault''s and if one is signaled, it
will construct a Fault envelope from the information in the exception
instance. The enclosing handler (Error) will construct a Fault envelope with
faultCode=''Client'' if any error is signaled. The faultstring is the
exceptions description, and detail is an exceptions stackdump)

This gives the model control over Faults, but provides the backup of
returning all errors as Client Faults.

This cover 95% of the cases.

I also provide a mechanism where a model can construct a Fault (rather
than signaling an exception, and letting Spray build the Fault). I found
this necessary for services which process soap:Headers, and which need more
control over the processing of a soap message. The "SW Spray Auth" package
(which implements digest authorization via soap:Headers) is an example of a
service that build its own faults.

Once Spray has processed a soapMessage, the out envelope can be tested to
whether it is a Fault (#isFault), and if so, the transport layer (ie HTTP)
can return it with the appropriate extra information (ie status code 500).

==========


'
!


!SprayOperationModel class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'spOperationClass.ico'
    ]
! !

!SprayOperationModel class methodsFor:'spray namespaces'!

sprayPortDocumentation
	"Answer a String that will be included in the model's WSDL as port documentation"

	^''
!

sprayPortName
	"Answer a String. The port name serves as a form of documentation."

	^self name asString
!

sprayPortNamespace
	"Answer a namespace symbol. 
	-RPC Operations constructed from the class are qualified in this namespace.
	-A server that exposes an instance of the receiver, uses the namespace to locate the instance.
	See the comment in SprayLocalServicesAbstract>>operationReceiver"

	^('urn:' , self sprayPortName) asSymbol
! !

!SprayOperationModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayOperationModel.st,v 1.4 2010/02/26 12:02:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayOperationModel.st,v 1.4 2010/02/26 12:02:41 cg Exp $'
! !
