"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SprayServer
	instanceVariableNames:'soapNode typespace operationPort service binding handlers'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayServer comment:'I am a web service server and I have the knowledge of the interface of my service. I can be asked for a WSDL (which I dynamically generate), and a html description of myself. I forward all incoming soap messages to my soap node <SpwSoapResponder> .

When I am created, I am responsible for constructing my soap node from my knowledge of the interface.

-I am multiple process safe because I do not change state once created. 

"To create a single server using a Swazoo binding"

"To start Swazoo"
SwazooSiteScript startUpIP: ''127.0.0.1'' service: TemperaturesService new new uriPattern: ''temper'' .

"Go to: ''http://localhost/temper'' in a browser."
"Open the Tools/Additional Tools/SwazooShell to view the binding"
"Send a soap message to the server"
client := TemperaturesService new createClientTo: ''http://localhost/temper''.
soapSend := client send: ''c2f'' with: 30.5.
soapSend value.

"To stop Swazoo"
SwazooSiteRegistry flush.

Instance Variables:
	soapNode 	<SpwSoapResponder> processes soap
	typespace		<XeTypespace> typespace for XeEncoding
	operationPort	<SpmdPort>	the interface of the operations of the service
	service		<SprayService> 	the interface of the service
	binding		<SptReceiveBindingAbstract> transport binding
	handlers		<Collection> of SpwSoapBHandlers

'
!


!SprayServer class methodsFor:'instance creation'!

new
	^self shouldNotImplement
!

newIn: aTypespace
	^super new initializeIn: aTypespace
! !

!SprayServer class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'server.ico'
    ]
! !

!SprayServer methodsFor:'accessing'!

addressLocation
	"Answer the url <String>"

	^binding addressLocation
!

allDeclareElements
	"Answer all instances of <XeElement> that may need to be declared in a XMLSchema"

	| coll |
	coll := OrderedCollection new.
	self doXeElements: [:each | coll add: each].
	^coll
!

allDefineContent
	"Answer all instances of <XeContent> used by the server that may need to be defined in a XMLSchema"

	| coll |
	coll := OrderedCollection new.
	self doXeContents: [:each | coll add: each].
	^coll
!

endPointEntity
	"Answer a <String> of html that provides some documentation about the service"

	^(service endPointEntityClass forServer: self) endPointEntity
!

operationNames
	"Answer a collection of <Strings>"

	^operationPort operationNames
!

service
	"Answer a <SprayLocalService"

	^service
!

serviceName
	^service name
!

serviceNamespace
	^service namespace
!

soapNode
	"Answer the <SpwSoapResponder>"

	^soapNode
!

soapNode: aSpwSoapResponder
	soapNode := aSpwSoapResponder
!

xsiNamespace
	^typespace xsiNamespace
! !

!SprayServer methodsFor:'accessing-private'!

binding
	^binding
!

operationPort
	"Answer a <SpmdPort>"

	^operationPort
!

typespace
	"Answer the <XeTypespace> of the server"

	^typespace
! !

!SprayServer methodsFor:'helpers'!

doXeContents: aBlockClosure
	"Evaluate the blockClosure for all the XeContent instances used by the receiver."

	handlers do: [:each | each doXeContents: aBlockClosure].
	service doAdditionalTypes: aBlockClosure
!

doXeElements: aBlockClosure 
	"Evaluate the blockClosure for all the XeElement instances used by the receiver in literal encoding."

	handlers do: [:each | each doXeElements: aBlockClosure]
! !

!SprayServer methodsFor:'initializing'!

addSoapBReceivers: anOrderedCollection
	"Add the <SpwSoapBReceiver>s"

	^soapNode addSoapBReceivers: anOrderedCollection
!

createOperationHandlers
	"Create the <SpwSoapBHandler>s and add them to the <SpwSoapResponder>"

	soapNode addHandlers: (handlers := operationPort allHandlersIn: typespace).
	service hasVoidBodyOperation ifTrue: [soapNode voidProcessor: SpwVoidProcessor new]
!

initializeIn: aXeTypespace 
	typespace := aXeTypespace.
	soapNode := SpwSoapResponder new.
	self initialize
!

port: aSpmdPort transport: aSptReceiveBinding service: aService
	operationPort := aSpmdPort.
	binding := aSptReceiveBinding.
	service := aService.
	binding server: self
! !

!SprayServer methodsFor:'wsdl'!

wsdl
	"Answer a <XeDoc>"

	^self wsdlBuilder wsdl
!

wsdlAddressLocation
	^self addressLocation , (self addressLocation last = $/
		ifTrue: ['service.wsdl']
		ifFalse: ['/service.wsdl'])
!

wsdlBuilder
	^(SpmdWSDLBuilder new)
		service: self service;
		xeContents: self allDefineContent;
		xeElements: self allDeclareElements;
		ports: (Array with: operationPort);
		addressLocation: self addressLocation;
		yourself
!

wsdlXmlString
	^self service hasDefinitionWSDL
		ifTrue: [self service definitionWSDL]
		ifFalse: [self wsdlBuilder wsdlXmlString ]
! !

!SprayServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayServer.st,v 1.3 2010/02/26 12:02:57 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayServer.st,v 1.3 2010/02/26 12:02:57 cg Exp $'
! !
