"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptExchangeQueue subclass:#SptAsyncSendQueue
	instanceVariableNames:'pending'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptAsyncSendQueue comment:'I am a queue of exchanges waiting to be processed. When the exchange has been processed I perform a messageSend to an object that was specified as the receiver.

The selector must have an arg count of at least one. The first argument is always the exchange itself. Other arguments can be included (so that the receiver can identify the exchange etc).

The cookie is used to avoid duplicated identical request in the queue. An exchange will not be queued if another exchange is queued with the same cookie. (using nil disables this).

The message is sent is the main process to minimize process safety issues when used by MPV''s. (If you extend this, please let me know what you do)




'
!


!SptAsyncSendQueue class methodsFor:'constants'!

namespace
	^#'http://www.tempuri.org/spray/async'
!

qNameArguments
	^XeQName local: 'arguments' ns: self namespace
!

qNameCookie
	^XeQName local: 'cookie' ns: self namespace
!

qNameReceiver
	^XeQName local: 'receiver' ns: self namespace
!

qNameSelector
	^XeQName local: 'selector' ns: self namespace
! !

!SptAsyncSendQueue class methodsFor:'must strip'!

example
	<resource: #example>

	"Requires the 'SW Splash' package"

	| splashRequestClass q test request exchange |
	splashRequestClass := self environment at: #SplashRPCRequest.
	request := splashRequestClass method: 'echoBoolean' ns: #'http://soapinterop.org/'.
	request parameterAt: #inputBoolean put: true.
	exchange := request exchangeTo: 'http://www.dolphinharbor.org/services/interop2001'
				soapAction: nil.
	q := SptAsyncSendQueue new.
	test := IdentityDictionary new.
	q
		queueLast: exchange
		sending: #at:put:
		to: test
		with: 'foo'
		cookie: nil.
	test    "inspect it"
! !

!SptAsyncSendQueue methodsFor:'accessing-private'!

pending
	^pending
!

removePending: anExchange
	| id |
	(id := self identifierFor: anExchange) notNil
		ifTrue: [pending remove: id ifAbsent: []]

! !

!SptAsyncSendQueue methodsFor:'event handling-private'!

onProcessedExchange: anExchange
	self removePending: anExchange.
	SessionManager current inputState queueDeferredAction:
			[self triggerExchange: anExchange.
			self trigger: #processed: with: anExchange]
! !

!SptAsyncSendQueue methodsFor:'helpers-private'!

identifierFor: anExchange
	^anExchange atQP: self class qNameCookie ifAbsent: []
!

nextForTests
	"Private - UnitTest helper"
	| ex |
	ex := queue nextNoWait.
	ex notNil ifTrue: [self removePending: ex].
	^ex
!

prepareExchange: anExchange selector: aSelector receiver: aReceiver arguments: anArray cookie: anObject
	aSelector argumentCount = (anArray size + 1) ifFalse: [self error: 'Incorrect number of arguments'].
	anExchange
		atQP: self class qNameReceiver put: aReceiver;
		atQP: self class qNameArguments put: anArray;
		atQP: self class qNameSelector put: aSelector;
		atQP: self class qNameCookie put: anObject
! !

!SptAsyncSendQueue methodsFor:'initializing-private'!

initializeFor: maxWating
	super initializeFor: maxWating.
	pending := Dolphin::SharedSet new.

! !

!SptAsyncSendQueue methodsFor:'operations-private'!

checkPending: anExchange
	| id |
	(id := self identifierFor: anExchange) notNil
		ifTrue:
			[(pending includes: id)
				ifTrue: [^true]
				ifFalse:
					[pending add: id.
					^false]]
		ifFalse: [^false]
!

triggerExchange: anExchange
	| args actualArgs |
	args := anExchange atQP: self class qNameArguments.
	actualArgs := Array new: args size + 1.
	actualArgs at: 1 put: anExchange.
	args notEmpty ifTrue: [
		actualArgs
			replaceFrom: 2
			to: actualArgs size
			with: args].
	(anExchange atQP: self class qNameReceiver)
		perform: (anExchange atQP: self class qNameSelector)
		withArguments: actualArgs
! !

!SptAsyncSendQueue methodsFor:'queueing'!

queueFirst: anExchange
	(self checkPending: anExchange) ifFalse: [super queueFirst: anExchange]
!

queueFirst: anExchange sending: aSelector to: aReceiver with: arg cookie: aCookie
	self
		prepareExchange: anExchange
		selector: aSelector
		receiver: aReceiver
		arguments: (Array with: arg)
		cookie: aCookie.
	self queueFirst: anExchange


!

queueFirst: anExchange sending: aSelector to: aReceiver withAll: args cookie: aCookie
	self
		prepareExchange: anExchange
		selector: aSelector
		receiver: aReceiver
		arguments: args
		cookie: aCookie.
	self queueFirst: anExchange


!

queueLast: anExchange
	(self checkPending: anExchange) ifFalse: [super queueLast: anExchange]
!

queueLast: anExchange sending: aSelector to: aReceiver with: arg cookie: aCookie
	self
		prepareExchange: anExchange
		selector: aSelector
		receiver: aReceiver
		arguments: (Array with: arg)
		cookie: aCookie.
	self queueLast: anExchange


!

queueLast: anExchange sending: aSelector to: aReceiver withAll: args cookie: aCookie
	self
		prepareExchange: anExchange
		selector: aSelector
		receiver: aReceiver
		arguments: args
		cookie: aCookie.
	self queueLast: anExchange


! !

!SptAsyncSendQueue class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptAsyncSendQueue.st,v 1.4 2008/09/06 15:02:27 cg Exp $'
! !
