"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptPropertyScope subclass:#SptEnvironment
	instanceVariableNames:''
	classVariableNames:'Current'
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptEnvironment comment:'I am the root of property scope. When any SptPropertyScope instance is asked for a scoped property (#atScopedQP:) that it does not include, I (the singleton current instance) will be asked for that property.

I include qualifiedProperties for settings that apply to all of the transport framework.

==========
Qualified Properties
	sprayTrans:httpClient <SptHTTPClientAbstract> Used for HTTP GET''s
	sprayTrans:httpRequestSoapBinding <SptSoapBindingAbstract> binding class for POSTing soap messages
	sprayTrans:logErrorLevel <Integer> Logging level
	sprayTrans:logStream <Stream> Logging stream
	sprayHttp:proxySettings <SptHTTPProxySettings>

I would also be the logical place to put username/password information etc
	'
!


!SptEnvironment class methodsFor:'accessing'!

clear
	"self clear"

	Current := nil
!

current
	Current isNil ifTrue: [Current := self new ].
	^Current
! !

!SptEnvironment class methodsFor:'helpers'!

reportString
	| stream |
	stream := String writeStream.
	stream
		display: 'Parsing OK: ';
		display: self canParse;
		cr;
		"display: 'IXMLDOM Class: ';
		display: XeEnvironment docClass name;
		cr;"
		"display: 'HTTP Request Class: ';
		display: self current httpRequestClass;
		cr;"
		display: 'SoapBinding Class: ';
		display: self current httpRequestSoapBinding.
	^stream contents

! !

!SptEnvironment class methodsFor:'initializing-private'!

uninitialize
	self clear
! !

!SptEnvironment class methodsFor:'testing'!

canParse
    [(XeEnvironment rootNodeFor: '<testing>123</testing>') xml] on: Error do: [ :e | self halt. ^ false].
    ^true
! !

!SptEnvironment methodsFor:'accessing'!

httpClient
	"Answer an http client."

	^self atQP: (self class sprayTransQ: 'httpClient')
!

httpProxySettings 
	^self atQP: (self class sprayHttpQ: 'proxySettings') ifAbsentPut: [SptHTTPProxySettings new]  
!

httpProxySettings: aSptHTTPProxySettings 
	self atQP: (self class sprayHttpQ: 'proxySettings') put: aSptHTTPProxySettings 
!

httpRequestSoapBinding
	"Answer a binding. See setHttpRequestSoapBinding for details"

	^self atQP: (self class sprayTransQ: 'httpRequestSoapBinding') 

!

soapBindingForWSDLUri: aString
	#todo.	"Handle alternate transport uris"
	^self httpRequestSoapBinding
! !

!SptEnvironment methodsFor:'accessing-private'!

atScopedQP: aName ifAbsent: aBlockClosure 
	^self atQP: aName ifAbsent: aBlockClosure 

!

gobalAt: aSymbol
    |env|

    env := self class environment.
    env ~~ Smalltalk ifTrue:[
        ^ env at: aSymbol ifAbsent:[ Smalltalk at:aSymbol ]
    ].
    ^ env at: aSymbol
!

gobalAt: aSymbol ifAbsent: operation
    |env|

    env := self class environment.
    env ~~ Smalltalk ifTrue:[
        ^ env at: aSymbol ifAbsent:[ Smalltalk at:aSymbol ifAbsent: operation ]
    ].
    ^ env at: aSymbol ifAbsent: operation
!

hasGlobal: aSymbol
	^self class environment includesKey: aSymbol
! !

!SptEnvironment methodsFor:'initializing-private'!

determineLogStream
	^self gobalAt: #Transcript ifAbsent: [^DeafObject].
!

initialize
	super initialize.
	self beSprayHTTPBinding.
	self beSprayHTTPClient.
! !

!SptEnvironment methodsFor:'logging'!

logErrorLevel
        ^ (self atQP: (self class sprayTransQ: 'logErrorLevel') ifAbsentPut: [0])
        max: (UserPreferences current soapLoggingLevel ? 0)
!

logStream
	^self atQP: (self class sprayTransQ: 'logStream') ifAbsentPut: [
		self determineLogStream]
! !

!SptEnvironment methodsFor:'modes'!

beSprayHTTPBinding
	self httpRequestSoapBinding: SptHTTPBinding


!

beSprayHTTPClient
	self httpClient: SptHTTPClient




!

beSptURLMonClient
	"
	self current beSptURLMonClient
	"

	self atQP: (self class sprayTransQ: 'httpClient')
		put: (self class environment at: #SptURLMonClient)
!

httpClient: aClass
	self atQP: (self class sprayTransQ: 'httpClient') put: aClass




!

httpRequestSoapBinding: aClass
	self atQP: (self class sprayTransQ: 'httpRequestSoapBinding') put: aClass


! !

!SptEnvironment methodsFor:'test accessing-private'!

sprayEnvironment: anSptEnvironment
	self assert: [anSptEnvironment == self].
	super sprayEnvironment: anSptEnvironment
! !

!SptEnvironment class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptEnvironment.st,v 1.5 2010/03/15 12:13:26 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptEnvironment.st,v 1.5 2010/03/15 12:13:26 cg Exp $'
! !
