"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptSoapBindingAbstract subclass:#SptHTTPBindingAbstract
	instanceVariableNames:'request'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptHTTPBindingAbstract comment:'This class is a SOAP HTTP binding. The class is asked to process an exchange which it does by POSTing to the immediateDestination

==========
Scope
-See superclass comment

When asked to process an exchange, this class creates an instance to do the actual processing. The class side #addQPTo: method can be used to add binding scope scopedQP''s to the instance. 

==========
Qualified Properties
	#''action:SOAPActionURI'' <String> SOAPAction header
	#''sprayTrans:httpSoapHeaders'' <Collection> of associations of header name to header value to add to the request
	#''sprayTrans:proxySettings''  <SptHTTPProxySettings> Can be nil
	#''sprayTrans:exception''  <Exception> a copy of the exception
	#''sprayTrans:exceptionStatus''  <String>The HTTP Status code if a transport error occurs
	#''sprayTrans:exceptionStatusText''  <String>The HTTP Status text if a transport error occurs
	#''sprayTrans:exceptionStatusResponse''  <String>The HTTP response text if a transport error occurs
	#''sprayHttp:userName''  <String> For HTTP bindings that support authentication
	#''sprayHttp:password''  <String> For HTTP bindings that support authentication
	#''sprayHttp:timeoutConnect''  <Integer> milliseconds (for bindings supporting timeouts)
	#''sprayHttp:timeoutResolve''  <Integer> milliseconds (for bindings supporting timeouts)
	#''sprayHttp:timeoutSend''  <Integer> milliseconds (for bindings supporting timeouts)
	#''sprayHttp:timeoutReceive''  <Integer> milliseconds (for bindings supporting timeouts)
'
!


!SptHTTPBindingAbstract class methodsFor:'instance creation'!

newBindingFor: aSpwRequestingExchange 
	| instance |
	instance := self new exchange: aSpwRequestingExchange.
	self addQPTo: instance.
	^instance
! !

!SptHTTPBindingAbstract class methodsFor:'operations'!

addQPTo: aBindingInstance
!

processExchange: aSpwRequestingExchange 
	(self newBindingFor: aSpwRequestingExchange) process.
! !

!SptHTTPBindingAbstract methodsFor:'accessing-private'!

hostHeaderValue
	| stream |
	(self immediateDestination beginsWith: 'http://') ifFalse: [ self error: 'Invalid immediateDestination. Expected: url starting with http:// Received: ' , self immediateDestination displayString].
	stream := self immediateDestination readStream.
	stream upTo: $/.
	stream upTo: $/.
	^stream upTo: $/.
!

httpHeaders
	^self subclassResponsibility
!

immediateDestination
	^self atScopedQP: #'transport:ImmediateDestination'
!

immediateDestination: aString
	self atQP: #'transport:ImmediateDestination' put: aString
!

inEnvelope
	^self subclassResponsibility
!

password
	^self atScopedQP: (self class sprayHttpQ: 'password') ifAbsent: [nil]
!

request
	^request
!

responseSafeString
	^[request responseBody] on: Error do: [:e | 'Error']
!

setTimeouts
	^self subclassResponsibility
!

soapAction
	^self ensureQuoted: (self atScopedQP: #'action:SOAPActionURI' ifAbsent: ['""'])
!

statusSafe
        ^[request status] on: Error do: [:e | 500]
!

statusSafeString
	^[request status printString] on: Error do: [:e | 'Error']
!

statusTextSafeString
	^[request statusText displayString] on: Error do: [:e | 'Error']
!

timeoutConnect
	^self atScopedQP: (self class sprayHttpQ: 'timeoutConnect') ifAbsent: [60000]
!

timeoutReceive
	^self atScopedQP: (self class sprayHttpQ: 'timeoutReceive') ifAbsent: [30000]
!

timeoutResolve
	^self atScopedQP: (self class sprayHttpQ: 'timeoutResolve') ifAbsent: [30000]
!

timeoutSend
	^self atScopedQP: (self class sprayHttpQ: 'timeoutSend') ifAbsent: [30000]
!

userName
	^self atScopedQP: (self class sprayHttpQ: 'userName') ifAbsent: [nil]
!

waitErrorDescription
	| stream |
	stream := String writeStream.
	stream
		nextPutAll: 'Soap HTTP Binding error, Status: ';
		nextPutAll: self statusSafeString.
	^stream contents
! !

!SptHTTPBindingAbstract methodsFor:'asserting-private'!

assertStatusOKor500
    |status|

    status := self statusSafe.
    (status = 200 or: [status = 500])  ifFalse: [
            self error: ('HTTP Request not OK, status: %1 %2' 
                    formatWith: self statusSafeString
                    with: self statusTextSafeString)]

    "Modified: / 12-07-2011 / 18:01:04 / cg"
! !

!SptHTTPBindingAbstract methodsFor:'exceptions-private'!

handleException: e 

        super handleException: e.
        exchange atQP: (self class sprayTransQ: 'exceptionStatus') put: self statusSafeString.
        exchange atQP: (self class sprayTransQ: 'exceptionStatusText') put: self statusTextSafeString.
        exchange atQP: (self class sprayTransQ: 'exceptionResponse') put: self responseSafeString
! !

!SptHTTPBindingAbstract methodsFor:'helpers-private'!

ensureQuoted: aString
	(aString isNil or: [aString isEmpty])
		 ifTrue: [^'""'].
	^aString first = $"
		ifTrue: [aString]
		ifFalse: ['"%1"' formatWith: aString]

	
	
! !

!SptHTTPBindingAbstract methodsFor:'operations'!

process
        self forkMainIfNecessary.
        
        [
            self createRequest.
            self setTimeouts.
            self openRequest.
            self setHeaders.
            self sendRequest
        ] on: Error do: [:e |
            UserPreferences current soapErrorDebugging ifTrue:[
                e reject.
            ].
            ^ self handleException: e
        ].
        exchange beWaitingState.
        "self waitOnRequest."
        [
            self waitOnRequest
        ] on: Error do: [:e |  
            UserPreferences current soapErrorDebugging ifTrue:[
                e reject.
            ].
            ^self handleException: e
        ].
        exchange beReceivingState.
        
        [
            self assertStatusOKor500.
            exchange in: self inEnvelope
        ] on: Error do: [:e |  
            UserPreferences current soapErrorDebugging ifTrue:[
                e reject.
            ].
            ^self handleException: e
        ].
        exchange beSuccessState.
        self ifLoggingInfo: ['Finished Request to: ' , self immediateDestination]
! !

!SptHTTPBindingAbstract methodsFor:'operations-private'!

createRequest
	^self subclassResponsibility
!

forkMainIfNecessary
	
!

openRequest
	"self assert: [request readyState = 0]."
	self assert: [self immediateDestination notNil].
	self assert: [self immediateDestination notEmpty].
	request 
		open: 'POST'
		bstrUrl: self immediateDestination
		varAsync: true
		bstrUser: self userName
		bstrPassword: self password.

	self ifLoggingInfo: ['Opening Request to: ' , self immediateDestination]
	
!

sendRequest
	self ifLoggingWire: ['sending: ' , exchange outXml].
	request send: exchange outXml
		
	
	
!

setHeaders
	self shouldAddHostHeader ifTrue: [
		request setRequestHeader: 'Host' bstrValue: self hostHeaderValue].
	self httpHeaders do: [ :each |
		request setRequestHeader: each key bstrValue: each value]

	
!

waitOnRequest
	^self subclassResponsibility
! !

!SptHTTPBindingAbstract methodsFor:'testing-private'!

shouldAddHostHeader
	^self atScopedQP: (self class sprayHttpQ: 'addHostHeader') ifAbsent: [false]
! !

!SptHTTPBindingAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptHTTPBindingAbstract.st,v 1.8 2011/07/12 16:20:09 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptHTTPBindingAbstract.st,v 1.8 2011/07/12 16:20:09 cg Exp $'
! !
