"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptPropertyScope subclass:#SptMEContext
	instanceVariableNames:'in out'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptMEContext comment:'I am a message exchange. I am based on the principles of [spec] http://www.w3.org/TR/soap12-part2/#soaptmep

I have an in and out soap envelope and I go through a series of state changes as I am processed by a Binding. 
==========
Scope
When asked for a scoped qualifiedProperty (#atScopedQP:) I first check myself, and ifAbsent I check the SptEnvironment current

==========
Qualified Properties
	#''single:State''  <Symbol> Current state (depends on subclass, states defined by [spec]


'
!


!SptMEContext class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'soapSend.ico'
    ]
! !

!SptMEContext methodsFor:'accessing'!

addAllQPs: aXeQProperties

"/ Exept/SV: This does not work. ANSI defines #addAll: different than used below!!
"/        qProperties addAll: aXeQProperties associations

        aXeQProperties associations do:[:eachAssociation|
            qProperties at:eachAssociation key put:eachAssociation value.
        ].
!

in
	"Answer a SpwSoapInEnvelope"

	^in
!

out
	"Answer a SpwSoapOutEnvelope"

	^out
!

state
	^self atQP: #'single:State'
! !

!SptMEContext methodsFor:'accessing-private'!

atScopedQP: aName ifAbsent: aBlockClosure 
	^self atQP: aName ifAbsent: [self sprayEnvironment atQP: aName ifAbsent: aBlockClosure]
!

setTransportFailureReason
	^self subclassResponsibility
!

state: aString 
	self atQP: #'single:State' put: aString
! !

!SptMEContext methodsFor:'asserting'!

isFailState
	^self state = 'Fail'
!

isSuccessState
	^self state = 'Success'
! !

!SptMEContext methodsFor:'state'!

beFailState
	self setTransportFailureReason.
	self changeStateTo: 'Fail'
!

beProcessingState
	self changeStateTo: 'Processing'
!

beReceivingState
	self changeStateTo: 'Receiving'
!

beSuccessState
	self changeStateTo: 'Success'.
!

beWaitingState
	self changeStateTo: 'Waiting'
! !

!SptMEContext methodsFor:'state-private'!

changeStateTo: aString 
        self state: aString.
        Smalltalk isSmalltalkX ifTrue:[
            self changed: #stateChanged: with: aString
        ] ifFalse:[
            self trigger: #stateChanged: with: aString
        ].
! !

!SptMEContext class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptMEContext.st,v 1.5 2010/02/26 12:02:52 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptMEContext.st,v 1.5 2010/02/26 12:02:52 cg Exp $'
! !
