"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SptPropertyScope
	instanceVariableNames:'qProperties'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptPropertyScope comment:'I implement a qualified property framework. 

It is based in part on: http://www.w3.org/TR/soap12-part2/#soaptmep

==========
Qualified Properties:

The keys of the qualifiedProperties can be either:
-An Instance of XeQName (ie XeQName local: ''foo'' ns: #''urn:blah''). This is a fully qualified name 
-An Instance of XePName (ie XePName local: ''ns1:foo'' ). This will be late qualified. 
-A String or Symbol that contain a colon (ie ''ns2:foo''). This will be late qualified. 

The prefixes/namespaces in use are defined in SptPropertyScope(class)>>newQProperties. 

(Note: some of the property prefixes are named from the draft SOAP1.2 specification, however since this document is a draft, I am using different namespaces)

If you want to store you own information use the "user" prefix.

==========
Scoping

Scoping is used so that a "setting" can be changed for a specific exchange or for all exchanges for a binding, or for the whole environment. For example the scope of a SptHTTPBinding instance is 
1/ The exchange it is processing
2/ Itself
3/ The SptEnvironment current.

If the ''sprayHttp:timeoutSend'' property is set into the exchange (1) it will be used. If not the scope will expand to the binding itself(2), and lastly to the the environment (3).

==========
Motivation:
Subclasses of this class are involved in the Transportation of envelopes and interfacing to network protocols. This needs to be highly configurable and easily extendable. 

==========
Logging:
-Logging is only partially implemented.
Evaluating
	SptEnvironment current beLoggingWire.
	SptEnvironment current beNotLogging.
Should turn logging on and off for all requests.
'
!


!SptPropertyScope class methodsFor:'instance creation'!

new
	^super basicNew initialize
! !

!SptPropertyScope class methodsFor:'accessing'!

newQProperties
	| qProperties |
	qProperties := XeQProperties new.
	qProperties addPrefix: 'action' namespace: #'http://www.tempuri.org/spray/action'.
	"qProperties addPrefix: 'spray' namespace: #'http://www.tempuri.org/spray/spray'."
	qProperties addPrefix: 'user' namespace: #'http://www.tempuri.org/spray/user'.
	qProperties addPrefix: 'sprayTrans' namespace: self sprayTransNS.
	qProperties addPrefix: 'sprayHttp' namespace: self sprayHttpNS.
	qProperties addPrefix: 'transport' namespace: #'http://www.tempuri.org/spray/transport'.
	qProperties addPrefix: 'single' namespace: #'http://www.tempuri.org/spray/single'.
	qProperties addPrefix: 'fail' namespace: #'http://www.tempuri.org/spray/fail'.
	qProperties addPrefix: 'mep' namespace: #'http://www.tempuri.org/spray/mep'.
	^qProperties
! !

!SptPropertyScope class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'connFolder.ico'
    ]
!

logErrorLevel
	^0
!

logInfoLevel
	^2
!

logOffLevel
	^0
!

logWarnLevel
	^1
!

logWireLevel
	^3
!

sprayHttpNS
	^#'http://www.tempuri.org/spray/sprayHttp'
!

sprayTransNS
	^#'http://www.tempuri.org/spray/sprayTrans'
! !

!SptPropertyScope class methodsFor:'helpers'!

sprayHttpQ: aString
	^XeQName local: aString ns: self sprayHttpNS
!

sprayTransQ: aString
	^XeQName local: aString ns: self sprayTransNS
! !

!SptPropertyScope methodsFor:'accessing'!

atQP: aName 
	^qProperties at: aName
!

atQP: aName ifAbsent: aBlockClosure 
	^qProperties at: aName ifAbsent: aBlockClosure
!

atQP: aName ifAbsentPut: aBlockClosure 
	^qProperties at: aName ifAbsentPut: aBlockClosure 
!

atQP: aName put: anObject 
	"Answer the receiver ... semantics similar  to an accessor, not #at:put:"

	qProperties at: aName put: anObject
!

includesQP: aName 
	self atQP: aName ifAbsent: [^false].
	^true
! !

!SptPropertyScope methodsFor:'accessing-private'!

atScopedQP: aName 
	^self atScopedQP: aName ifAbsent: [self errorNotFound: aName]
!

atScopedQP: aName ifAbsent: aBlockClosure 
	self subclassResponsibility
!

qProperties
	"Private - Answer the value of the receiver's ''qProperties'' instance variable."

	^qProperties
!

qProperties: aXeQProperties
	qProperties := aXeQProperties
!

sprayEnvironment
	^self atQP: (self class sprayTransQ: 'Environment') ifAbsent: [SptEnvironment current]
! !

!SptPropertyScope methodsFor:'initializing-private'!

initialize
	super initialize.
	qProperties := self class newQProperties.
	
! !

!SptPropertyScope methodsFor:'logging'!

beLoggingWire
	"Note: Logging is incomplete"

	"
	SptEnvironment current beLoggingWire
	"

	^self logErrorLevel: self class logWireLevel
!

beNotLogging
	"Note: Logging is incomplete"

	"
	SptEnvironment current beNotLogging
	"

	^self logErrorLevel: self class logOffLevel
!

isLoggingInfo
	"Note: Logging is incomplete"

	^self logErrorLevel >= self class logInfoLevel
!

isLoggingWarn
	"Note: Logging is incomplete"

	^self logErrorLevel >= self class logWarnLevel
!

isLoggingWire
	"Note: Logging is incomplete"

	^self logErrorLevel >= self class logWireLevel
!

logErrorLevel
        ^ (self atScopedQP: (self class sprayTransQ: 'logErrorLevel') ifAbsent: [0])
        max: (UserPreferences current soapLoggingLevel ? 0)
!

logErrorLevel: aNumber
	^self atQP: (self class sprayTransQ: 'logErrorLevel') put: aNumber
!

logStream
	"Note: Logging is incomplete"

	^self atScopedQP: (self class sprayTransQ: 'logStream') ifAbsent: [self sprayEnvironment logStream]
! !

!SptPropertyScope methodsFor:'logging-private'!

ifLoggingInfo: operation
	"Private - Note: Logging is incomplete"

	self isLoggingInfo ifTrue: [
		self logLevel: 'info' log: operation ]
!

ifLoggingWarn: operation
	"Private - Note: Logging is incomplete"

	self isLoggingWarn ifTrue: [
		self logLevel: 'warn' log: operation]
!

ifLoggingWire: operation
	"Private - Note: Logging is incomplete"

	self isLoggingWire ifTrue: [
		self logLevel: 'wire' log: operation]
!

logLevel: aLevelString log: operation
        "Private - Note: Logging is incomplete"

        | stream aDescription |
        #cg.

        aDescription := [operation value] on: Error do: [:e | self halt. 'Logging Error'].
        stream := String writeStream.
        stream
                display: 'SOAP: ';
                display: Timestamp now printString"TimeStamp current rfc1123String" ;
                space;
                display: aLevelString;
                space;
                display: aDescription;
                cr.

        (self logStream ? Transcript) nextPutAll: stream contents

    "Modified: / 20-03-2012 / 21:35:09 / cg"
! !

!SptPropertyScope methodsFor:'test accessing-private'!

sprayEnvironment: anSptEnvironment
	"Private - Change the environment.
	-Intended for unit testing purposes"

	^self atQP: (self class sprayTransQ: 'Environment') put: anSptEnvironment
! !

!SptPropertyScope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptPropertyScope.st,v 1.9 2012/04/21 17:16:08 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptPropertyScope.st,v 1.9 2012/04/21 17:16:08 stefan Exp $'
! !
