"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwSoapEnvelope subclass:#SpwFaultEnvelope
	instanceVariableNames:'xml'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Server'
!

SpwFaultEnvelope comment:'I am a SOAP Fault envelope'
!


!SpwFaultEnvelope class methodsFor:'instance creation'!

code: aString string: aString2 details: aString3 isBody: aBoolean
	^self new 
		code: aString
		string: aString2
		details: aString3
		isBody: aBoolean
!

fromException: anException
	^self 
		code: 'Client'
		string: anException description
		details: (anException stackTrace: 20)
		isBody: true
!

fromFaultException: anException
	^self 
		code: anException faultCode
		string: anException description
		details: (anException stackTrace: 20)
		isBody: anException isBody
!

misunderstand: anArray 
	^self new misunderstand: anArray
! !

!SpwFaultEnvelope class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'fault.ico'
    ]
! !

!SpwFaultEnvelope methodsFor:'accessing'!

xml
	^xml
! !

!SpwFaultEnvelope methodsFor:'accessing-private'!

code: aCodeString string: aFaultString details: aDetailsString isBody: aBoolean
	xml := self 
				faultPayLoadFor: aCodeString
				string: aFaultString
				details: aDetailsString
				isBody: aBoolean
!

misunderstand: anArray
	"Private - Future versions will need to add the mustUnderstands to the header."

	xml := self 
				faultDocFor: 'MustUnderstand'
				string: 'One or more mandatory headers not understood'
				details: ''
				isBody: false
! !

!SpwFaultEnvelope methodsFor:'converting'!

asInEnvelope
	^SpwSoapInEnvelope fromXmlString: xml
! !

!SpwFaultEnvelope methodsFor:'operations-private'!

dump: aString 
	

	
!

faultDocFor: aCodeString string: aFaultString details: aDetailsString isBody: aBoolean
        | faultNode env |
        env := SpwSoapOutEnvelope new createEnvelope.
        faultNode := env bodyNode addElementNamed: ('Fault' ns: XeNamespace soapEnv)
                                rootDeclare: true.
        (faultNode addElement: 'faultcode') addTextNode: 'SOAP-ENV:' , aCodeString.
        (faultNode addElement: 'faultstring') addTextNode: aFaultString printString "displayString".
        aBoolean 
                ifTrue: 
                        [((faultNode addElement: 'detail') addElementNamed: (XeQName 
                                                prefix: 'e'
                                                local: 'stackdump'
                                                ns: #'urn:stackdump')
                                rootDeclare: false) addTextNode: aDetailsString].
        ^env xml
!

faultPayLoadFor: aCodeString string: aFaultString details: aDetailsString isBody: aBoolean
	"Private - Keep this as robust as possible so we can always get back some kind of fault response"

	| xmlString |
	xmlString := self 
				faultDocFor: aCodeString
				string: aFaultString
				details: aDetailsString
				isBody: aBoolean.
	self dump: xmlString.
	^xmlString
! !

!SpwFaultEnvelope methodsFor:'testing'!

isFault
	^true
! !

!SpwFaultEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwFaultEnvelope.st,v 1.4 2010/02/26 12:02:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwFaultEnvelope.st,v 1.4 2010/02/26 12:02:39 cg Exp $'
! !
