"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwEnvelope subclass:#SpwInEnvelope
	instanceVariableNames:'headerObjects bodyObjects headerCoders bodyCoders'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Client'
!

SpwInEnvelope comment:'I am an Envelope of schema components that can be asked asked to decode a SoapEnvelope into my objects

My coders (<SpwSoapBCoder>) 


==========
Instance Variables:
	headerObjects	<Collection> of objects decoded from soap:Headers.
	bodyObjects	<Collection> of objects decoded from soap:Headers.
	headerCoders	<Collection> of <SpwSoapBCoder>
	bodyCoders	<Collection> of <SpwSoapBCoder>'
!


!SpwInEnvelope methodsFor:'accessing-body'!

bodyObjects
	^bodyObjects
!

extractReturn
	^self subclassResponsibility
!

firstBodyObject
	"Answer the first body object that was decoded"

	^bodyObjects first
!

outParameters
	"Answer a keyed collection of all out parameters received. (convention is to answer first as the 'return')"

	^self subclassResponsibility
! !

!SpwInEnvelope methodsFor:'accessing-headers'!

atHeader: anIndex
	^headerObjects at: anIndex
!

firstHeaderObject
	headerObjects isEmpty ifTrue: [self error: 'Envelope has no header object'].
	^headerObjects first
!

headerObjects
	"Answer the objects that were decoded from the header blocks of the soapMessage"

	^headerObjects
! !

!SpwInEnvelope methodsFor:'accessing-private'!

bodyCoders
	^bodyCoders
!

bodyCoders: aCollOfSpCoders1 headerCoders: aCollOfSpCoders2
	"Private - Construct the receiver based on the information in the SpmdOperation, in the aXeTypespace"

	bodyCoders := aCollOfSpCoders1.
	headerCoders := aCollOfSpCoders2.
	self resetObjects
!

headerCoders
	^headerCoders
!

headerCoders: aCollection
	headerCoders := aCollection
!

headerObjects: aCollection
	headerObjects := aCollection
! !

!SpwInEnvelope methodsFor:'operations'!

decodeHeaders: aSpSoapInEnvelope 
	headerObjects := Array new: headerCoders size.
	self processHeaders: aSpSoapInEnvelope
!

decodeSoap: aSoapInEnvelope 
	self
		resetObjects;
		processBodies: aSoapInEnvelope;
		processHeaders: aSoapInEnvelope
!

forceInSoap: aSoapInEnvelope
	| guessTypespace |
	guessTypespace := XeTypespace defaultSoapEnc2001.
	headerObjects := aSoapInEnvelope headerBlocks 
				collect: [:each | XeSoapXMLReader guess: each in: guessTypespace].
	bodyObjects := aSoapInEnvelope bodyBlocks 
				collect: [:each | XeSoapXMLReader guess: each in: guessTypespace]
!

resetObjects
	headerObjects := Array new: headerCoders size.
	bodyObjects := Array new: bodyCoders size
!

resumeInSoap: soapInEnvelope 
        [self decodeSoap: soapInEnvelope] 
                on: XeTypeMismatch , XeSchemaNotFound , SoapBlockNameMismatch
                do: [:e | 
                    UserPreferences current soapErrorDebugging ifTrue:[e reject].
                    self breakPoint:#cg. 
                    e resume
                ].

    "Modified: / 12-07-2011 / 17:46:37 / cg"
! !

!SpwInEnvelope methodsFor:'operations-private'!

processBodies: aSoapInEnvelope
	^self subclassResponsibility
!

processHeaderBlock: aXeNode
	"Private - Convert the incoming header soapBlock <XeNode> into an object.
	-The WSDL spec allows for loose defining of headers, therefore we can not be sure of the order of
		headers received, nor if we will know to receive them.
	-If we cant process them, the application can still access the raw xml of the header"

	| blockName coder |
	blockName := aXeNode qName.
	coder := self headerCoders detect: [:each | each elementQName = blockName]
				ifNone: [^aXeNode].
	^coder decodeSoapB: aXeNode
!

processHeaders: aSoapInEnvelope 
	headerObjects := aSoapInEnvelope headerBlocks collect: [:each | self processHeaderBlock: each]
! !

!SpwInEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwInEnvelope.st,v 1.4 2011/07/12 16:20:55 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwInEnvelope.st,v 1.4 2011/07/12 16:20:55 cg Exp $'
! !
