"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwEnvelope subclass:#SpwOutEnvelope
	instanceVariableNames:'headers bodies'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Client'
!

SpwOutEnvelope comment:'I am an Envelope of objects and schema components that can be asked to create a SpwOutSoapEnvelope.

I am "semi-private", typically a user will not need to interact directly with me, unless soap:Headers need to be sent. My implementation may change in future versions.

In typical client use, you should access the SpraySoapSend instance that contains me

==========
Instance Variables:
	headers	<Collection> of SpCodedObject
	bodies	<Collection> of SpCodedObject

==========
Implementation:
-I am not happy with these classes. The problem they solve is that they collect the objects (a soap envelope can have separate sections ... soap:Headers and a soap:Body) that need to be encoded, and then perform the encoding in a single pass. 
-Alternatives
--I have thought about splitting these classes and creating a third envelope hierarchy, (so there would be soapEnvelopes, objectEnvelopes and interfaceEnvelopes) eeck. 
--Do not collect objects (which is not necessary for normal use) and provide a helper class that would add soap:Headers to an envelope'
!


!SpwOutEnvelope methodsFor:'accessing'!

soapOutEnvelope
	| env |
	env := self createSoapOutEnvelope.
	self encodeHeadersInto: env.
	self encodeBodyInto: env.
	^env
	
!

xsiNamespace
	^typespace xsiNamespace 
! !

!SpwOutEnvelope methodsFor:'accessing-body'!

arguments: anArray 
	"Sets the arguments for the soapSend. Arguments are matched to parameter names (which are stored in the operation).
	-For RPC the arguments become the parameters of the RPC.
	-For DOC the arguments are encoded directly below the soap:Body node "

	^self subclassResponsibility
!

firstBodyObject: anObject
	"Set the object to be encoded into the first soap:Body child"

	bodies first object: anObject
! !

!SpwOutEnvelope methodsFor:'accessing-headers'!

firstHeader
	"Answer a <SpwCodedObject> which is the first soap:Header child"

	^headers first
!

firstHeaderObject: anObject
	"Set the object to be encoded into the first soap:Header child"

	headers first object: anObject
!

headerNamed: aQName
	^headers 
		detect: [ :each | each isNamed: aQName]
		ifNone: [self error: 'Can not locate the header coder for: ' , aQName displayString]
!

secondHeaderObject: anObject
	"Set the object to be encoded into the first soap:Header child"

	headers second object: anObject
! !

!SpwOutEnvelope methodsFor:'accessing-private'!

bodies
	"Private - Answer the value of the receiver's ''bodies'' instance variable."

	^bodies
!

bodies: bodyCodedObjects headers: headerCodedObjects
	"Private - Construct the receiver based on the information in the SpmdOperation, in the aXeTypespace"

	bodies := bodyCodedObjects asOrderedCollection.
	headers := headerCodedObjects asOrderedCollection
!

bodyCoders: aCollOfSpCoders1 headerCoders: aCollOfSpCoders2
	"Private - Construct the receiver based on the information in the SpmdOperation, in the aXeTypespace.
	The receiver holds collections of <SpwCodedObject>s. A  <SpwCodedObject> contains an <Object> to be encoded and a <SpwSoapBCoder>  which is the template to use for the encoding"

	self bodies: (aCollOfSpCoders1 collect: [:each | SpwCodedObject onCoder: each])
		headers: (aCollOfSpCoders2 collect: [:each | SpwCodedObject onCoder: each])
!

bodyElements
	^bodies collect: [:each | each element]
!

bodyObjects
	"Private - Answer the collection of <Objects> that will encode into the soap:Body"

	^bodies collect: [ :each | each object]
!

headers
	^headers
! !

!SpwOutEnvelope methodsFor:'headers'!

addAnyHeaderNamed: aXeQName with: anEncodingStyle
	"Add a soap:Header to the envelope that will encode any <Object> using the encodingStyle.
	Answer the <SpwCodedObject> (set the object to be encoded into the codedObject using #object:)"

	| docElements |
	docElements := anEncodingStyle 
				docElementsSpeced: (Array with: (XeElementSpec anyNamed: aXeQName)).
	^self addHeader: (SpwCodedObject onElement: docElements first)
!

addAnyLiteralHeaderNamed: aXeQName
	"Add a soap:Header to the envelope that will encode any <Object> using literal style.
	Answer the <SpwCodedObject> (set the object to be encoded into the codedObject using #object:)"

	^self addAnyHeaderNamed: aXeQName with: typespace literalStyle
!

addAnySoapEncHeaderNamed: aXeQName 
	"Add a soap:Header to the envelope that will encode any <Object> using soap-enc style.
	Answer the <SpwCodedObject> (set the object to be encoded into the codedObject using #object:)"

	^self addAnyHeaderNamed: aXeQName with: typespace soapEncStyle
! !

!SpwOutEnvelope methodsFor:'headers-private'!

addHeader: aSpwOutBlock 
	self assert: [aSpwOutBlock isKindOf: SpwCodedObject].
	^headers add: aSpwOutBlock
! !

!SpwOutEnvelope methodsFor:'modes'!

beAllTyped
	"Turn on inline typing for the receiver and all it's children."

	self bodyElements do: [:each | each beAllTyped]
! !

!SpwOutEnvelope methodsFor:'operations'!

resetObjects
	
! !

!SpwOutEnvelope methodsFor:'operations-private'!

createSoapOutEnvelope
	^(SpwSoapOutEnvelope new createEnvelope)
		addXSINamespace: self xsiNamespace;
		yourself
!

encodeBodyInto: aSpwSoapOutEnvelope
	bodies do: [:each | each encodeInto: aSpwSoapOutEnvelope bodyNode]
!

encodeHeadersInto: aSpwSoapOutEnvelope
	headers isEmpty ifTrue: [^nil].
	headers 
		do: [:each | each isObjectNil ifFalse: [each encodeInto: aSpwSoapOutEnvelope headerNode]]
!

replaceBodyObjectsWith: anArray 
	self assert: [anArray size = bodies size].
	bodies with: anArray do: [ :codedObject :object | codedObject object: object] 
! !

!SpwOutEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwOutEnvelope.st,v 1.1 2002/11/18 19:33:48 cg Exp $'
! !
