"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpwSoapBHandler
	instanceVariableNames:'selector inputDecoder outputEncoder isBodyHandler soapAction'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Server'
!

SpwSoapBHandler comment:'I handle the conversion from xml to objects and back to xml for a soapBlock. I am located by matching the qualifiedName of the soapBlock to my identificationQName. I am used by servers.

-I am multiple process safe because I do not change state once created. 

This soapBlock:
<m:GetLastTradePrice xmlns:m="http://example.org/quotes">
<symbol>DIS</symbol>
</m:GetLastTradePrice>

Would match me if my identificationQName is XeQName local: ''GetLastTradePrice'' ns: #''http://example.org/quotes''

The various subclasses differ in the way the Smalltalk Message arguments are extracted from the soapBlock. In the typical RPC case, each parameter converts to an argument. See the #messageFor:exchange method comments.

If an operation needs more access to the raw soap message, it can use a *ArgsEnvHandler, which maps to a two argument Smalltalk Message, the first argument is an Array of the converted soap parameters, the second argument is the EnvelopeExchange itself.

SpwBlockHandlers contain instances of SpwSoapBCoders which are the interface between Spray and the XeEncoding package.

Instance Variables:
	selector		<Symbol>
	inputDecoder	<SpwSoapBCoder> decodes xml parameters to object arguments
	outputEncoder	<SpwSoapBCoder> encodes object answer(s) to xml return
	isBodyHandler	<Boolean> true if soapBlock is child of soap:Body, false if child of soap:Header
	soapAction	<String> the value of the HTTP SOAPAction


'
!


!SpwSoapBHandler class methodsFor:'testing'!

isDOCHandlerClass
	^false
! !

!SpwSoapBHandler methodsFor:'accessing'!

identificationQName
	"Answer a <XeQName>. An incoming xml soapBlock is matched to a handler by it's QName"

	^inputDecoder elementQName
!

inputDecoder: aSpwSoapBCoder
	inputDecoder := aSpwSoapBCoder.
	 self assertValidSelector
!

outputContent
	^outputEncoder element content
!

outputEncoder
	"Answer a <SpwSoapBCoder>"

	^outputEncoder
!

outputEncoder: aSpwSoapBCoder
	outputEncoder := aSpwSoapBCoder
!

selector: aSymbol 
	selector := aSymbol 
!

soapAction
	"SOAPAction is likely to be deprecated in the SOAP1.2 spec.
	Spray servers do not check soapAction."

	^soapAction
!

soapAction: aString
	"SOAPAction is likely to be deprecated in the SOAP1.2 spec.
	Spray servers do not check soapAction."

	soapAction := aString
! !

!SpwSoapBHandler methodsFor:'accessing-private'!

inputDecoder
	^inputDecoder
!

isBodyHandler: anObject
	isBodyHandler := anObject
!

selector
	^selector
! !

!SpwSoapBHandler methodsFor:'asserting-private'!

assertValidSelector
	
! !

!SpwSoapBHandler methodsFor:'modes'!

beBody
	self isBodyHandler: true
! !

!SpwSoapBHandler methodsFor:'operations'!

doXeContents: aBlockClosure
	"Evaluate the blockClosure for all the XeContent instances used by the receiver."

	inputDecoder doXeContents: aBlockClosure.
	self isDecodeOnly ifFalse: [outputEncoder doXeContents: aBlockClosure]
!

doXeElements: aBlockClosure
	"Evaluate the blockClosure for all the XeElement instances used by the receiver."

	inputDecoder doXeElements: aBlockClosure.
	self isDecodeOnly ifFalse: [outputEncoder doXeElements: aBlockClosure]
!

messageFor: aXeNode exchange: aSptRespondingExchange
	"Answer a Smalltalk <Message>"

	^self subclassResponsibility
! !

!SpwSoapBHandler methodsFor:'testing'!

isDecodeOnly 
	"While RPC Body blocks are never decode only, it is valid for headers (by default) to be decode only"

	^self outputEncoder isNil
! !

!SpwSoapBHandler methodsFor:'testing-private'!

isBodyHandler
	^isBodyHandler
! !

!SpwSoapBHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapBHandler.st,v 1.3 2008/09/05 17:48:05 cg Exp $'
! !
