"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwSoapEnvelope subclass:#SpwSoapInEnvelope
	instanceVariableNames:'node firstBodyBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SpwSoapInEnvelope comment:'I am a soap envelope created by parsing xml. 

If the xml is a Fault, #faultException extracts the details into a SoapSendFault instance (which can then be signalled)

Instance Variables:
	node		<XeNode>
	firstBodyBlock	<XeNode>




'
!


!SpwSoapInEnvelope class methodsFor:'instance creation'!

fromRequest: aByteObject
        | envNode |
"/        [
            envNode := XeEnvironment rootNodeFor: aByteObject.
"/        ] on: Error do: [:e | 
"/            self error: 'Envelope parsing error: ' , e description
"/        ].
        ^ self onEnvelopeNode: envNode

    "Modified (format): / 12-07-2011 / 17:48:43 / cg"
!

fromXmlString:xmlString 
    |envNode|

    [
        envNode := XeEnvironment rootNodeFor:xmlString
    ] on:Error do:[:e | 
        UserPreferences current soapErrorDebugging ifTrue:[e reject].
        (SoapServerFault new)
            beClient;
            signal:e description
    ].
    ^ self onEnvelopeNode:envNode

    "Modified: / 12-07-2011 / 17:49:08 / cg"
!

onEnvelopeNode: envNode
        envNode localName = 'Envelope' 
                ifFalse: 
                        [SoapServerFault new beClient 
                                signal: 'Expecting Envelope element received: ' , envNode localName "envNode firstXMLLine"].
        "envNode namespace == XeNamespace soapEnv 
                ifFalse: [SoapServerFault new beVersionMismatch signal: 'Spray only supports Soap1.1']."
        ^self new setEnvelopeNode: envNode



! !

!SpwSoapInEnvelope class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'in.ico'
    ]
! !

!SpwSoapInEnvelope class methodsFor:'must strip'!

example
        <resource: #example>

        ^SpwSoapInEnvelope 
                fromXmlString: '<SOAP-ENV:Envelope 
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Body>
    <m:getQuote 
      SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
      xmlns:m="urn:xmethods-delayed-quotes">
      <symbol xsi:type="xsd:string">YHOO</symbol>
    </m:getQuote>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>'
! !

!SpwSoapInEnvelope methodsFor:'accessing'!

bodyBlocks
	"Answer a collection of XeNodes"

	^self bodyNode childNodes
!

bodyNode
	^self bodyNodeIfNone: 
			[SoapServerFault new beClient signal: 'Can not find the Body Element in the namespace ' 
						, XeNamespace soapEnv displayString]
!

firstBodyBlock
	"Answer a XeNode. The firstBodyBlock is the first child which is not marked as 'not a serialization root'"

	^firstBodyBlock
!

headerBlocks
	"Answer a collection of XeNodes; the childNodes of soap:Header"

	| headerNode |
	(headerNode := self headerNode) isNil ifTrue: [^#()].
	^headerNode childNodes
!

mustUnderstandNodes
	"Answer the header soapBlocks that are marked mustUnderstand.
	-Note: At this point we can not determine whether it is us that mustUnderstand them"

	| headerNode |
	(headerNode := self headerNode) isNil ifTrue: [^#()].
	^headerNode childNodes select: 
			[:each | 
			(each 
				getAttribute: 'mustUnderstand'
				in: XeNamespace soapEnv
				ifNone: ['0']) = '1']
!

xml
	"Answer a <String>"

	^node ownerDocument xml
! !

!SpwSoapInEnvelope methodsFor:'accessing-faults'!

faultException
	^(SoapSendFault new)
		faultCode: self faultcode;
		messageText: self faultstring;
		faultDetail: self faultDetail;
		yourself
! !

!SpwSoapInEnvelope methodsFor:'accessing-faults-private'!

faultChild: aString
	^(self bodyNode firstChild 
		detectChildNode: aString
		in: nil
		ifNone: [^'']) text
!

faultDetail
	^(self bodyNode firstChild 
		detectChildNode: 'detail'
		in: nil
		ifNone: [^'']) xml
!

faultcode
	^self faultChild: 'faultcode'
!

faultstring
	^self faultChild: 'faultstring'
! !

!SpwSoapInEnvelope methodsFor:'accessing-private'!

bodyNodeIfNone: operation
	"Private -  Answer a XeNode"

	^self node 
		detectChildNode: 'Body'
		in: XeNamespace soapEnv
		ifNone: operation
!

headerNode
	"Private - Answer a XeNode"

	^self node 
		detectChildNode: 'Header'
		in: XeNamespace soapEnv
		ifNone: []
!

node
	^node
!

node: aXeNode
	node := aXeNode
!

setEnvelopeNode: aXeNode
	node := aXeNode.
	firstBodyBlock := self findFirstBodyBlock
! !

!SpwSoapInEnvelope methodsFor:'converting'!

asInEnvelope
	^self
! !

!SpwSoapInEnvelope methodsFor:'exceptions-private'!

errorCantDetermineRoot
	^SoapServerFault new beClient 
		signal: 'Multiple message nodes received. Expected one to not have the attribute root="0"'
! !

!SpwSoapInEnvelope methodsFor:'operations-private'!

findFirstBodyBlock
	"Private - Answer a XeNode
	-Soap Builders Andrew Layman 23Apr2001;
		Simply search for the first element that
		 does not have a SOAP-ENC:root='0' attribute and value. "

	| nodes |
	(nodes := self bodyNode childNodes) isEmpty ifTrue: [^nil].	"This might be valid"
	nodes size = 1 ifTrue: [^nodes first].	"Short cut for the common simple case"
	^nodes detect: 
			[:each | 
			| rootAtt |
			rootAtt := each getAttribute: 'root' in: XeNamespace soapEnc.
			rootAtt isNil or: [rootAtt ~= '0']]
		ifNone: [self errorCantDetermineRoot]
! !

!SpwSoapInEnvelope methodsFor:'testing'!

hasBody
	self bodyNodeIfNone: [ ^false].
	^true
!

hasHeader
	^self headerNode notNil
!

isFault
	"While the examples show that the fault element should be in the Soap-Env namespace,
	the spec does not mention it. To be on the safe side, any element named Fault will be interpreted as
	a fault"

	^self firstBodyBlock notNil and: [
		self firstBodyBlock localName = 'Fault']
! !

!SpwSoapInEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapInEnvelope.st,v 1.6 2011/07/12 16:19:32 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapInEnvelope.st,v 1.6 2011/07/12 16:19:32 cg Exp $'
! !
