"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SpwSoapEnvelope subclass:#SpwSoapOutEnvelope
	instanceVariableNames:'node bodyNode headerNode isFault'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SpwSoapOutEnvelope comment:'I am a soap envelope created by building the xml nodes. 

Instance Variables:
	node		<XeNode> the soap:Envelope node
	bodyNode	<XeNode> the soap:Body node
	headerNode	<XeNode> the soap:Header node (lazily created)
	isFault		<Boolean>'
!


!SpwSoapOutEnvelope class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'outEnv.ico'
    ]
! !

!SpwSoapOutEnvelope methodsFor:'accessing'!

bodyNode
	"Answer the XeNode which is the soap:Body"

	^bodyNode
!

headerNode
	"Answer the XeNode which is the soap:Header"

	headerNode isNil ifTrue: [headerNode := self addHeader].
	^headerNode
!

xml
	^self node ownerDocXML
!

xmlFormatted
	^self node ownerDocument xmlFormatted
! !

!SpwSoapOutEnvelope methodsFor:'accessing-private'!

node
	^node
!

node: aXeNode
	node := aXeNode
! !

!SpwSoapOutEnvelope methodsFor:'constants-private'!

headerNodeName
	^XeQName 
		prefix: 'SOAP-ENV'
		local: 'Header'
		ns: XeNamespace soapEnv
! !

!SpwSoapOutEnvelope methodsFor:'converting'!

asInEnvelope
	^SpwSoapInEnvelope fromXmlString: self xml
! !

!SpwSoapOutEnvelope methodsFor:'initializing'!

createEncodedEnvelope
	"Answer the receiver, creating the normal envelope and adding soap-enc encodingStyle"

	self createEnvelope.
	node setAttribute: 'SOAP-ENV:encodingStyle' value: XeNamespace soapEnc
!

createEnvelope
        "Answer the receiver creating the required envelope nodes, and adding default declarations"

        | doc |
        doc := YAXO::XeDoc new.
        (node := doc addElement: 'SOAP-ENV:Envelope')
                addNSPrefix: 'SOAP-ENV' to: XeNamespace soapEnv;
                addNSPrefix: 'SOAP-ENC' to: XeNamespace soapEnc.
        bodyNode := node addElement: 'SOAP-ENV:Body'
! !

!SpwSoapOutEnvelope methodsFor:'modes-private'!

beFaultCode: aString string: aString2 
	"Private - Use with care. Mechanism to return fault from header processing"
	| faultNode |
	isFault := true.
	faultNode := self bodyNode addElement: 'SOAP-ENV:Fault'.
	(faultNode addElement: 'faultcode') addTextNode: 'SOAP-ENV:' , aString. 
	(faultNode addElement: 'faultstring') addTextNode: aString2 

	
	
! !

!SpwSoapOutEnvelope methodsFor:'operations'!

removeXMLInstructions
	node ownerDocument removeXMLInstructions
! !

!SpwSoapOutEnvelope methodsFor:'operations-private'!

addHeader
	"Private - add the soap:Header node as a child of soap:Envelope"

	| header |
	header := node addElementNamed: self headerNodeName rootDeclare: false.
	header beFirstChild.
	^header
!

addXSINamespace: aNSSymbol
	node addNSPrefix: 'xsi' to: aNSSymbol
! !

!SpwSoapOutEnvelope methodsFor:'testing'!

isFault
	isFault isNil ifTrue: [isFault := false].
	^isFault
! !

!SpwSoapOutEnvelope class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapOutEnvelope.st,v 1.4 2010/02/26 12:02:45 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapOutEnvelope.st,v 1.4 2010/02/26 12:02:45 cg Exp $'
! !
