"{ Package: 'stx:goodies/soap/spray/auth' }"

"{ NameSpace: SOAP }"

Object subclass:#SpAuthDigestAuthorizer
	instanceVariableNames:'realm users'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth'
!


!SpAuthDigestAuthorizer class methodsFor:'instance creation'!

new
        ^self basicNew initialize
! !

!SpAuthDigestAuthorizer class methodsFor:'constants'!

icon
	^SWEnvironment spIconNamed: 'lock16.ico'
! !

!SpAuthDigestAuthorizer methodsFor:'accessing'!

clientAuthFor: aString nonce: aByteArray 
	^SpAuthClientAuth onAuthorizer: self  userID: aString nonce: aByteArray

		
		 
!

numberOfUsers
	^users size
!

realm
	^realm
!

realm: aString 
	realm := aString 
! !

!SpAuthDigestAuthorizer methodsFor:'accessing-private'!

etag
	"Private - ?"

	^'*'
!

secretOf: aUIDString 
	"Private - Answer the secret for the user aUIDString.
	See #secret:pwd:"

	^self secretOf: aUIDString ifAbsent: [self errorNotFound: aUIDString]
!

secretOf: aUIDString ifAbsent: aBlockClosure 
	^users at: aUIDString ifAbsent: aBlockClosure 
! !

!SpAuthDigestAuthorizer methodsFor:'adding'!

addUid: aUIDString pwd: aPassString
	self assertValidUID: aUIDString .
	self assertValidPWD: aPassString.
	users at: aUIDString put: (self secret: aUIDString pwd: aPassString)
! !

!SpAuthDigestAuthorizer methodsFor:'asserting-private'!

assertValidPWD: aPassString 
	self assert: [aPassString notNil and: [aPassString notEmpty]]
!

assertValidUID: aUIDString 
	self assert: [aUIDString notNil and: [aUIDString notEmpty]]
! !

!SpAuthDigestAuthorizer methodsFor:'initializing'!

loadSecretsFromFile: aString
	"Hide the error if the file doesnt exist??"

	| fs fn|
	fn := FileLocator default localFileSpecFor: aString.
	(File exists: fn) ifFalse: [^nil].
	fs := FileStream read: fn text: true.
	[fs contents lines do: [ :each | | stream un|
		stream := each readStream.
		un := (stream upTo: $,) trimBlanks.
		users at: un put: stream upToEnd trimBlanks]] ensure: [fs close]
! !

!SpAuthDigestAuthorizer methodsFor:'initializing-private'!

initialize
	super initialize.
	users := SharedLookupTable new
! !

!SpAuthDigestAuthorizer methodsFor:'operations'!

authOf: aUIDString sn: aServerNonceString cn: aClientNonceOrNil
	^self authOf: aUIDString sn: aServerNonceString cn: aClientNonceOrNil ifAbsent: [self errorNotFound: aUIDString ]
!

authOf: aUIDString sn: aServerNonceString cn: aClientNonceOrNil ifAbsent: aBlockClosure 
	"Answer the auth of the user, using the server nonce.
	-If the client nonce is notNil, it indicates mutual authentication is required.
	See http://www.whitemesa.com/digauthmod.html Sec4.2"

	| rawAuth secret |
	secret := self secretOf: aUIDString ifAbsent: aBlockClosure.
	rawAuth := secret , ':' , aServerNonceString.
	aClientNonceOrNil notNil 
		ifTrue: [rawAuth := rawAuth , ':' , aClientNonceOrNil ].
	^MD5 hMessage: rawAuth
!

generateNonce
	^self subclassResponsibility
!

secret: aUIDString pwd: aPassString
	"Answer the secret for the username/password.
	From: http://www.whitemesa.com/digauthmod.html
	secret (string) represents the hex encoding of the digest MD5( concat(uid, ':', realm, ':', pwd) ) where: 
		-uid (string) represents the User ID. 
		-realm (string) represents the Realm name. 
		-pwd (string) represents the password.  "

	self assert: [realm notNil and: [realm notEmpty]].
	^MD5 hexBinaryHashMessage: aUIDString , ':' , realm , ':' , aPassString
! !

!SpAuthDigestAuthorizer methodsFor:'removing'!

removeAllSecrets
	self initialize
! !

!SpAuthDigestAuthorizer methodsFor:'testing'!

isExpiredNonce: aString 
	^self subclassResponsibility
! !

!SpAuthDigestAuthorizer class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/auth/SOAP__SpAuthDigestAuthorizer.st,v 1.3 2010/06/14 10:55:15 cg Exp $'
! !
