"{ Package: 'stx:goodies/soap/spray/auth' }"

"{ NameSpace: SOAP }"

SprayLocalService subclass:#SpAuthServiceAbstract
	instanceVariableNames:'authorizer'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth'
!

SpAuthServiceAbstract comment:'Subclasses are services that use the authentication specification defined by Robert Cunnings at:
	<http://www.whitemesa.com/digauthmod.html>

"To use an authenticated service"

See the SoapInteropAuthService example in the "Steve Waring\Spray WebServices\Interop\ServerInterop\Auth\SW Interop Auth.pac''" package'
!


!SpAuthServiceAbstract class methodsFor:'constants'!

icon
	^SWEnvironment spIconNamed: 'lock16.ico'
! !

!SpAuthServiceAbstract methodsFor:'accessing-auth'!

digestAuthorizer
	^self subclassResponsibility
!

realm
	^self subclassResponsibility
! !

!SpAuthServiceAbstract methodsFor:'accessing-auth-private'!

authorizer
	authorizer isNil ifTrue: [self createAuthorizer].
	^authorizer
! !

!SpAuthServiceAbstract methodsFor:'accessing-private'!

challengeHeader
	^self authHeaderTyped: 'AuthSC_Fault_t' named: 'AuthSCF'

		

	
!

clientAuthHeader
	^(self authHeaderTyped: 'AuthCS_t' named: 'AuthCS') selector: #clientAuth:exchange:

	

	
!

missingGuard
	"Answer the guard used in the operationModelClasses <SpwSoapBReceiver>."

	^(SpAuthMissingGuard new)
		authorizer: self authorizer;
		yourself
!

nextChallengeHeader
	^self authHeaderTyped: 'AuthSC_t'  named:  'AuthSC'

 
	

	
! !

!SpAuthServiceAbstract methodsFor:'accessing-receiving objects'!

guardedBlockReceiver
	^(SpwSoapBReceiver receiveFor: self operationModel
		at: self operationModelClass sprayPortNamespace)
		addGuard: self missingGuard;
		yourself
! !

!SpAuthServiceAbstract methodsFor:'accessing-receiving objects-private'!

authorizerBlockReceiver
	^SpwSoapBReceiver receiveFor: self authorizer at: self authNamespace
!

soapBReceivers
	^OrderedCollection with: self guardedBlockReceiver with: self authorizerBlockReceiver
! !

!SpAuthServiceAbstract methodsFor:'client server creation-private'!

createAuthorizer
	authorizer := SpAuthOpAuthorizer usingAuthorizer: self digestAuthorizer.
	authorizer realm: self realm.
	authorizer
		challengeCoder: (self challengeHeader docCoderIn: self typespace);
		nextChallengeCoder: (self nextChallengeHeader docCoderIn: self typespace).
	self onAuthorizerCreated
! !

!SpAuthServiceAbstract methodsFor:'constants-private'!

authNamespace
	^#'http://soap-authentication.org/digest/2001/08/'

	
!

schemaImport
	^'<?xml version="1.0"?>
<schema targetNamespace="http://soap-authentication.org/digest/2001/08/"
xmlns:tns="http://soap-authentication.org/digest/2001/08/"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns="http://www.w3.org/2001/XMLSchema">
<import namespace="http://schemas.xmlsoap.org/soap/envelope/"/>
<complexType name="AuthCS_t">
<sequence>
<element name="Nonce" type="xsd:hexBinary"/>
<element name="Auth" type="xsd:hexBinary"/>
<element name="UserID" type="xsd:string"/>
<element name="Realm" type="xsd:string"/>
<element name="CNonce" type="xsd:hexBinary" minOccurs="0"/>
</sequence>
</complexType>
<complexType name="AuthSC_t">
<sequence>
<element name="Status" type="xsd:string"/>
<element name="Nonce" type="xsd:hexBinary"/>
<element name="CNonce" type="xsd:hexBinary" minOccurs="0"/>
<element name="RAuth" type="xsd:hexBinary" minOccurs="0"/>
</sequence>
</complexType>
<complexType name="AuthSC_Fault_t">
<sequence>
<element name="Status" type="xsd:string"/>
<element name="Nonce" type="xsd:hexBinary"/>
<element name="Realm" type="xsd:string"/>
</sequence>
</complexType>
<element name="AuthCS" type="tns:AuthCS_t">
<attribute ref="SOAP-ENV:actor" use="optional"/>
<attribute ref="SOAP-ENV:mustUnderstand" use="optional"/>
</element>
<element name="AuthSC" type="tns:AuthSC_t">
<attribute ref="SOAP-ENV:actor" use="optional"/>
<attribute ref="SOAP-ENV:mustUnderstand" use="optional"/>
</element>
<element name="AuthSCF" type="tns:AuthSC_Fault_t">
<attribute ref="SOAP-ENV:actor" use="optional"/>
<attribute ref="SOAP-ENV:mustUnderstand" use="optional"/>
</element>
</schema>'
! !

!SpAuthServiceAbstract methodsFor:'event handling-private'!

onAuthorizerCreated	

!

onTypespaceCreated
	"Private - Modify the default typespace"

	super onTypespaceCreated.
	"This modifies the typespace by including the XMLSchema created from the xml string, at it's targetNamespace in the typespace. 
	-The definitions and declarations can then be used in the service (ie in the operationModelClasses methods, or as part of other definitions)"
	typespace addSchemaString: self schemaImport.
	"This modifies the typespace by telling the <XeSchema> targeting the namespace, to use a <XeSprayClassFactory> for  the listed localNames only.
	-This is useful for selectively choosing types in the schema to encode/decode as instances of pre-prepared classes"
	typespace overrideAll: #('AuthCS_t') toSprayClassAt: self authNamespace.
	"This modifies the typespace by mapping the complexType typeNames to alternate classNames. 
	( by default the class name is assumed to be the same as the local part of the typeName)"
	typespace mapTypeName: (self authQ: 'AuthCS_t') to: SpAuthClientAuth
! !

!SpAuthServiceAbstract methodsFor:'helpers'!

beDemo
	self authorizer removeAllSecrets.
	self authorizer
		addUid: 'user' pwd: 'dolphin';
		addUid: 'demo' pwd: 'dolphin'.
!

doAdditionalTypes: operation 
	self authorizer challengeCoder doXeContents: operation.
	self authorizer nextChallengeCoder doXeContents: operation
! !

!SpAuthServiceAbstract methodsFor:'helpers-private'!

authHeaderTyped: aString named: aString2
	| header |
	header := SpmdSoapHeader 
		soapEncTypeName: (self authQ: aString) 
		partName: aString2
		selector: nil
		ns: self authNamespace.
	header beNotOwned.
	^header

	
!

authQ: aString
	^XeQName local: aString ns: self authNamespace 
	
! !

!SpAuthServiceAbstract methodsFor:'port generation-private'!

onPortGenerated: port
	"Handle the onPortGenerated event. See superclass #onPortGenerated: comment."

	| clientAuth nextChallenge challenge |
	super onPortGenerated: port.
	clientAuth := self clientAuthHeader beNotOwned.
	nextChallenge := self nextChallengeHeader beNotOwned.
	challenge := self challengeHeader beNotOwned.
	port operations do: 
			[:each | 
			each addInputHeader: clientAuth.
			each addOutputHeader: nextChallenge.
			each addOutputHeader: challenge]
! !

!SpAuthServiceAbstract class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/auth/SOAP__SpAuthServiceAbstract.st,v 1.2 2009/10/04 17:38:06 cg Exp $'
! !
