"{ Package: 'stx:goodies/soap/spray/services' }"

"{ NameSpace: SOAP }"

SoapInteropAbstract subclass:#SoapInteropPort
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Interop-R2'
!

SoapInteropPort comment:'My service (can/does) use SourceAnalysis (ie #storeOperationPort) to generate an interface for me. '
!


!SoapInteropPort class methodsFor:'spray namespaces'!

sprayPortName
	^'soapinterop'
! !

!SoapInteropPort methodsFor:'operations'!

echoBase64: aByteArray 
	"SpdOperationCreator new
		in: #((inputBase64 xsd:base64Binary));
		out: #((return xsd:base64Binary))"

	self assert: aByteArray kindOf: ByteArray.
	^aByteArray asByteArray
!

echoBoolean: aBool 
	"SpdOperationCreator new
		in: #((inputBoolean xsd:boolean));
		out: #((return xsd:boolean))"

	self assert: aBool kindOf: Boolean.
	^aBool
!

echoDate: aDateTime 
	"SpdOperationCreator new
		in: #((inputDate xsd:dateTime));
		out: #((return xsd:dateTime))"

	self assert: aDateTime kindOf: XeXsdDateTime.
	^aDateTime
!

echoDecimal: aScaledDecimal 
	"SpdOperationCreator new
		in: #((inputDecimal xsd:decimal));
		out: #((return xsd:decimal))"

	self assert: aScaledDecimal kindOf: ScaledDecimal.
	^aScaledDecimal
!

echoFloat: aFloat 
	"SpdOperationCreator new
		in: #((inputFloat xsd:float));	
		out: #((return xsd:float))"

	"If the port is resuming type conversion errors, aFloat may be an instance of XMLAnyValue. If so check it, and return it"

	^self assertIsFloatOrNil: aFloat.
!

echoFloatArray: anArray 
	"SpdOperationCreator new
		in: #((inputFloatArray nsi:ArrayOffloat));
		out: #((return nsi:ArrayOffloat))"

	anArray isNil ifTrue: [^#()].
	self assertIsCollection: anArray.
	^anArray collect: 
			[:each | 
			self assertIsFloatOrNil: each.
			each]
!

echoHexBinary: aByteArray 
	"SpdOperationCreator new
		in: #((inputHexBinary xsd:hexBinary));
		out: #((return xsd:hexBinary))"

	self assert: aByteArray kindOf: ByteArray.
	^aByteArray asByteArray
!

echoInteger: aSmallInteger 
	"SpdOperationCreator new
		in: #((inputInteger xsd:int));
		out: #((return xsd:int))"

	self assert: aSmallInteger kindOf: Integer.
	^aSmallInteger
!

echoIntegerArray: anArray 
	"SpdOperationCreator new
		in: #((inputIntegerArray nsi:ArrayOfint));
		out: #((return nsi:ArrayOfint))"

	anArray isNil ifTrue: [^#()].
	self assertIsCollection: anArray.
	^anArray collect: 
			[:each | 
			self assert: each kindOfOrNil: Integer.
			each]
!

echoMap: aMap 
	"SpdOperationCreator new
		in: #((inputMap nsa:Map));
		out: #((return nsa:Map))"

	self assertIsCollection: aMap .
	^aMap 
!

echoMapArray: anArray 
	"SpdOperationCreator new
		in: #((inputMapArray nsa:ArrayOfMap));
		out: #((return nsa:ArrayOfMap))"

	anArray isNil ifTrue: [^#()].
	self assertIsCollection: anArray.
	^anArray collect: 
			[:each | 
			self assertIsCollection:  each.
			each]
!

echoString: aString 
	"SpdOperationCreator new
		in: #((inputString xsd:string));	
		out: #((return xsd:string))"

	self assert: aString kindOfOrNil: String.
	^aString
!

echoStringArray: anArray 
	"SpdOperationCreator new
		in: #((inputStringArray nsi:ArrayOfstring));
		out: #((return nsi:ArrayOfstring))"

	anArray isNil ifTrue: [^#()].
	self assertIsCollection: anArray.
	^anArray collect: 
			[:each | 
			self assert: each kindOfOrNil: String.
			each]
!

echoStruct: aStruct 
	"SpdOperationCreator new
		in: #((inputStruct nsi:SOAPStruct));
		out: #((return nsi:SOAPStruct))"

	self assert: aStruct kindOfOrNil: XeQStruct.
	^aStruct
!

echoStructArray: anArray 
	"SpdOperationCreator new
		in: #((inputStructArray nsi:ArrayOfSOAPStruct));
		out: #((return nsi:ArrayOfSOAPStruct))"

	anArray isNil ifTrue: [^#()].
	self assertIsCollection: anArray.
	^anArray collect: 
			[:each | 
			self assert: each kindOfOrNil: XeQStruct.
			each]
!

echoVoid
	"SpdOperationCreator new
		in: #();
		out: #()"

	^nil
! !

!SoapInteropPort class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/services/SOAP__SoapInteropPort.st,v 1.1 2008/09/07 18:46:29 cg Exp $'
! !
