"{ Package: 'stx:goodies/soap/spray/services/tests' }"

"{ NameSpace: SOAP }"

SoapInteropFTs subclass:#R1FTs
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Interop-R2-TestCases'
!


!R1FTs methodsFor:'accessing'!

getService
	^SoapInteropRound1Service new
! !

!R1FTs methodsFor:'tests'!

testEchoBase64
	soapSend := client send: 'echoBase64' with: 'abc' asByteArray.
	answer := soapSend value.
	self assert: answer = 'abc' asByteArray
!

testEchoBoolean
	soapSend := client send: 'echoBoolean' with: true.
	answer := soapSend value.
	self assert: answer.
	soapSend := client send: 'echoBoolean' with: false.
	answer := soapSend value.
	self deny: answer
!

testEchoDate
	| date |
	date := XeXsdDateTime current.
	soapSend := client send: 'echoDate' with: date.
	answer := soapSend value.
	self assert: answer = date
!

testEchoDecimal
	soapSend := client send: 'echoDecimal' with: '0.0001s' asNumber.
	answer := soapSend value.
	self assert: answer = '0.0001s' asNumber.
	self assert: (answer isKindOf: ScaledDecimal).
	soapSend := client send: 'echoDecimal' with: '123456789.987654s' asNumber.
	answer := soapSend value.
	self assert: answer = '123456789.987654s' asNumber.
	self assert: (answer isKindOf: ScaledDecimal)
!

testEchoFloat
	soapSend := client send: 'echoFloat' with: 3.14.
	answer := soapSend value.
	self assert: answer = 3.14
!

testEchoInteger
	soapSend := client send: 'echoInteger' with: 1.
	answer := soapSend value.
	self assert: answer = 1.
	soapSend := client send: 'echoInteger' with: -1.
	answer := soapSend value.
	self assert: answer = -1
!

testEchoIntegerArray
	soapSend := client send: 'echoIntegerArray' with: #(1 2 3 4 5 6 7).
	answer := soapSend value.
	self assert: answer = #(1 2 3 4 5 6 7)
!

testEchoIntegerException
	soapSend := client send: 'echoInteger' with: (XeAnyValue new text: '2147483648').	"force the client to send it"
	self should: [soapSend value] raise: SoapSendFault
!

testEchoString
	soapSend := client send: 'echoString' with: 'abc'.
	answer := soapSend value.
	self assert: answer = 'abc'
!

testEchoStringArray
	soapSend := client send: 'echoStringArray' with: #('abc' 'def').
	answer := soapSend value.
	self assert: answer = #('abc' 'def')
!

testEchoStringArrayIdentical
	| string |
	string := 'abc'.
	soapSend := client send: 'echoStringArray'
				with: (Array 
						with: string
						with: string
						with: 'def').
	answer := soapSend value.
	self assert: answer first = 'abc'.
	self assert: answer third = 'def'.
	self assert: answer first == answer second
!

testEchoStruct
	soapSend := client send: 'echoStruct' with: self getStruct.
	answer := soapSend value.
	self assert: answer = self getStruct
!

testEchoStructArray
	soapSend := client send: 'echoStructArray' with: (Array with: self getStruct with: self getStruct).
	answer := soapSend value.
	self assert: answer = (Array with: self getStruct with: self getStruct)
!

testEchoStructArrayIdentical
	| struct |
	struct := self getStruct.
	soapSend := client send: 'echoStructArray'
				with: (Array 
						with: struct
						with: struct
						with: self getStruct).
	answer := soapSend value.
	self 
		assert: answer = (Array 
						with: self getStruct
						with: self getStruct
						with: self getStruct).
	self assert: answer first == answer second.
	self deny: answer first == answer third
!

testEchoStructReordered
	soapSend := client send: 'echoStruct' with: self getStructReordered.
	answer := soapSend value.
	self assert: (answer isSoapEncSameAs: self getStruct)
!

testEchoVoid
	soapSend := client send: 'echoVoid' withArguments: #().
	answer := soapSend value.
	self assert: answer isNil
! !

!R1FTs class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/services/tests/SOAP__R1FTs.st,v 1.1 2008/09/07 18:51:29 cg Exp $'
! !
