"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayAuthTests subclass:#SpAuthClientAuthTest
	instanceVariableNames:'service clientAuth'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth-TestCases'
!


!SpAuthClientAuthTest methodsFor:'accessing'!

addUidPwd1
        self authorizer addUid: 'user' pwd: 'dolphin'
!

authorizer
        ^service authorizer authorizer
!

beIncorrect
        clientAuth := self clientAuthFor: #getIncorrectXmlString
!

beInvalidID
        clientAuth := self clientAuthFor: #getInvalidXmlString
!

beInvalidRealm
        clientAuth := self clientAuthFor: #getInvalidRealmXmlString
!

beSuccessNoClientChallenge
        clientAuth := self clientAuthFor: #getSuccessNoChallengeXmlString
!

beSuccessWithClientChallenge
        clientAuth := self clientAuthFor: #getSuccessWithChallengeXmlString
!

clientAuthFor: aSelector
        | instance |
        instance := SpSprayTestAuthObjects new clientAuthIn: service typespace for: aSelector.
        instance authorizer: self authorizer.
        ^instance
! !

!SpAuthClientAuthTest methodsFor:'asserting'!

assertAnswerFault
        self assert: clientAuth answerStruct typeName 
                                = ('AuthSC_Fault_t' ns: #'http://soap-authentication.org/digest/2001/08/').
        self assert: clientAuth isFail
!

assertAnswerSuccess
        self assert: clientAuth answerStruct typeName 
                                = ('AuthSC_t' ns: #'http://soap-authentication.org/digest/2001/08/').
        self deny: clientAuth isFail
! !

!SpAuthClientAuthTest methodsFor:'set up'!

setUp
        super setUp.
        service := SpAuthTestService new.
! !

!SpAuthClientAuthTest methodsFor:'tests'!

testIncorrect
        self beIncorrect.
        clientAuth authorize.
        self assertAnswerFault.
        self assert: (clientAuth answerStruct at: #Status) = 'E02'.
        self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
        self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.
!

testInvalidID
        self beInvalidID.
        clientAuth authorize.
        self assertAnswerFault.
        self assert: (clientAuth answerStruct at: #Status) = 'E04'.
        self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
        self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.
!

testInvalidRealm
        self beInvalidRealm.
        clientAuth authorize.
        self assertAnswerFault.
        self assert: (clientAuth answerStruct at: #Status) = 'E04'.
        self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
        self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.
!

testSuccessNoClientChallenge
        self beSuccessNoClientChallenge.
        clientAuth authorize.
        self assertAnswerSuccess.
        self assert: (clientAuth answerStruct at: #Status) = 'S01'.
        self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
        self assert: (clientAuth answerStruct at: #CNonce) isNil.
        self assert: (clientAuth answerStruct at: #RAuth) isNil.
!

testSuccessWithClientChallenge
        self beSuccessWithClientChallenge.
        clientAuth authorize.
        self assertAnswerSuccess.
        self assert: (clientAuth answerStruct at: #Status) = 'S01'.
        self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
        self assert: (clientAuth answerStruct at: #CNonce) printSchemaHexString = 'CEA8A3DB3C06C7970A61B92AE9560A08'.
        self assert: (clientAuth answerStruct at: #RAuth) = #[70 186 66 205 157 40 241 34 16 136 209 240 4 116 191 236].
! !

!SpAuthClientAuthTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthClientAuthTest.st,v 1.3 2009/10/04 17:40:31 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthClientAuthTest.st,v 1.3 2009/10/04 17:40:31 cg Exp $'
! !
