"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayAuthTests subclass:#SpAuthOpAuthorizerTest
	instanceVariableNames:'service opAuthorizer exchange'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth-TestCases'
!


!SpAuthOpAuthorizerTest methodsFor:'accessing'!

clientAuthorize: aSelector 
        opAuthorizer clientAuth: (SpSprayTestAuthObjects new clientAuthIn: self typespace for: aSelector)
                exchange: (exchange := self getSampleExchange)
!

getFaultXmlStringWithStatus: aString

^'<?xml version="1.0"?>
<SOAP-ENV:Envelope
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Header>
    <ns1:AuthSCF xsi:type="ns1:AuthSC_Fault_t"
      SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
      xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
      <Status xsi:type="xsd:string">%1</Status>
      <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>
      <Realm xsi:type="xsd:string">TestRealm</Realm>
    </ns1:AuthSCF>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
  <SOAP-ENV:Fault>
    <faultcode>SOAP-ENV:Client</faultcode>
    <faultstring>Authentication failed: missing, malformed, or invalid credentials</faultstring>
  </SOAP-ENV:Fault>
</SOAP-ENV:Body></SOAP-ENV:Envelope>' formatWith: aString
!

getSampleExchange
        "^SpwRespondingExchange received: SpSprayTestEnvelopes new getEnvelopeSpec1 in: self typespace"

        ^SptRespondingExchange onXmlString: SpSprayTestEnvelopes new getEnvelopeSpec1 xml
!

isAuthenticated
        ^exchange atQP: #'user:authenticated'
!

typespace
        ^service typespace
! !

!SpAuthOpAuthorizerTest methodsFor:'asserting'!

assertExchangeOutXML: aString 
        | xml |
        self assertXML: (xml := exchange out xml) isSameAs: aString
! !

!SpAuthOpAuthorizerTest methodsFor:'set up'!

setUp

        service := SpAuthTestService new.
        opAuthorizer := service authorizer.
! !

!SpAuthOpAuthorizerTest methodsFor:'tests'!

testContruction
        self assert: (opAuthorizer authorizer isKindOf: SpAuthDigestAuthorizer).
        self assert: opAuthorizer authorizer realm = service realm
!

testIncorrect
        self clientAuthorize: #getIncorrectXmlString.
        self deny: self isAuthenticated.
        self deny: exchange isContinueState.
        self assertExchangeOutXML: (self getFaultXmlStringWithStatus: 'E02')
!

testInvalidID
        self clientAuthorize: #getInvalidXmlString.
        self deny: self isAuthenticated.
        self deny: exchange isContinueState.
        self assertExchangeOutXML: (self getFaultXmlStringWithStatus: 'E04')
!

testInvalidRealm
        self clientAuthorize: #getInvalidRealmXmlString.
        self deny: self isAuthenticated.
        self deny: exchange isContinueState.
        self assertExchangeOutXML: (self getFaultXmlStringWithStatus: 'E04')
!

testMissingExchange
        opAuthorizer missingExchange: (exchange := self getSampleExchange).
        self deny: self isAuthenticated.
        self deny: exchange isContinueState.
        self assertExchangeOutXML: (self getFaultXmlStringWithStatus: 'E01')
!

testSuccessNoChallenge
        self clientAuthorize: #getSuccessNoChallengeXmlString.
        self assert: self isAuthenticated.
        self 
                assertExchangeOutXML: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
  <ns1:AuthSC 
    SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
    xsi:type="ns1:AuthSC_t"
    xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
    <Status xsi:type="xsd:string">S01</Status>
    <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>
  </ns1:AuthSC>
</SOAP-ENV:Header>
<SOAP-ENV:Body/></SOAP-ENV:Envelope>'
!

testSuccessWithChallenge
        self clientAuthorize: #getSuccessWithChallengeXmlString.
        self assert: self isAuthenticated.
        self 
                assertExchangeOutXML: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
  <ns1:AuthSC
    SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
    xsi:type="ns1:AuthSC_t" xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
    <Status xsi:type="xsd:string">S01</Status>
    <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>
    <CNonce xsi:type="xsd:hexBinary">CEA8A3DB3C06C7970A61B92AE9560A08</CNonce>
    <RAuth xsi:type="xsd:hexBinary">46BA42CD9D28F1221088D1F00474BFEC</RAuth>
  </ns1:AuthSC>
</SOAP-ENV:Header><SOAP-ENV:Body/></SOAP-ENV:Envelope>'
! !

!SpAuthOpAuthorizerTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthOpAuthorizerTest.st,v 1.3 2009/10/04 17:40:18 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthOpAuthorizerTest.st,v 1.3 2009/10/04 17:40:18 cg Exp $'
! !
