"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayFunctionalTests subclass:#SpAuthTestFT
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth-TestCases'
!


!SpAuthTestFT methodsFor:'accessing'!

getFaultXmlStringWith: aString

^'<SOAP-ENV:Envelope 
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Header>
    <ns1:AuthSCF
     SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
     xsi:type="ns1:AuthSC_Fault_t"
     xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
     <Status xsi:type="xsd:string">%1</Status>
     <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>
     <Realm xsi:type="xsd:string">TestRealm</Realm>
    </ns1:AuthSCF>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
<SOAP-ENV:Fault>
<faultcode>SOAP-ENV:Client</faultcode>
<faultstring>Authentication failed: missing, malformed, or invalid credentials</faultstring>
</SOAP-ENV:Fault>
</SOAP-ENV:Body></SOAP-ENV:Envelope>' formatWith: aString
!

getService
        ^SpAuthTestService new 
! !

!SpAuthTestFT methodsFor:'tests'!

testIncorrect
        | xml |
        soapSend := client send: 'testMethod1' with: 'foo'.
        soapSend outEnvelope firstHeaderObject: (SpSprayTestAuthObjects new clientAuthIn: client typespace
                                for: #getIncorrectXmlString).
        self should: [soapSend value] raise: SoapSendFault.
        self assertXML: (xml := soapSend inXmlOrEmpty) isSameAs: (self getFaultXmlStringWith: 'E02')
!

testInvalid
        | xml |
        soapSend := client send: 'testMethod1' with: 'foo'.
        soapSend outEnvelope firstHeaderObject: (SpSprayTestAuthObjects new clientAuthIn: client typespace
                                for: #getInvalidXmlString).
        self should: [soapSend value] raise: SoapSendFault.
        self assertXML: (xml := soapSend inXmlOrEmpty) isSameAs: (self getFaultXmlStringWith: 'E04')
!

testMissing
        | xml |
        soapSend := client send: 'echoVoid'.
        self should: [soapSend value] raise: SoapSendFault.
        self assertXML: (xml := soapSend inXmlOrEmpty) isSameAs: (self getFaultXmlStringWith: 'E01')
!

testSuccessNoChallenge
        | xml |
        soapSend := client send: 'testMethod1' with: 'foo'.
        soapSend outEnvelope firstHeaderObject: (SpSprayTestAuthObjects new clientAuthIn: client typespace
                                for: #getSuccessNoChallengeXmlString).
        self assert: soapSend value.
        self assertXML: (xml := soapSend inXmlOrEmpty)
                isSameAs: 
'<SOAP-ENV:Envelope
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Header>
     <ns1:AuthSC 
       SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
       xsi:type="ns1:AuthSC_t"
       xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
       <Status xsi:type="xsd:string">S01</Status>
       <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>

     </ns1:AuthSC>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
<m:testMethod1Response 
  SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:m="urn:SprayTesting123">
   <Result xsi:type="xsd:boolean">true</Result>
</m:testMethod1Response>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testSuccessWithChallenge
        | xml |
        soapSend := client send: 'testMethod1' with: 'foo'.
        soapSend outEnvelope firstHeaderObject: (SpSprayTestAuthObjects new clientAuthIn: client typespace
                                for: #getSuccessWithChallengeXmlString).
        self assert: soapSend value.
        self assertXML: (xml := soapSend inXmlOrEmpty)
                isSameAs: 
'<SOAP-ENV:Envelope 
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Header>
    <ns1:AuthSC
      SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
      xsi:type="ns1:AuthSC_t"
      xmlns:ns1="http://soap-authentication.org/digest/2001/08/">
        <Status xsi:type="xsd:string">S01</Status>
        <Nonce xsi:type="xsd:hexBinary">436F6E7374616E74466F7254657374696E67</Nonce>
        <CNonce xsi:type="xsd:hexBinary">CEA8A3DB3C06C7970A61B92AE9560A08</CNonce>
        <RAuth xsi:type="xsd:hexBinary">46BA42CD9D28F1221088D1F00474BFEC</RAuth>
      </ns1:AuthSC>
  </SOAP-ENV:Header>
<SOAP-ENV:Body>
<m:testMethod1Response SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:m="urn:SprayTesting123">
<Result xsi:type="xsd:boolean">true</Result>
</m:testMethod1Response></SOAP-ENV:Body></SOAP-ENV:Envelope>'
! !

!SpAuthTestFT class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthTestFT.st,v 1.3 2009/10/04 17:40:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpAuthTestFT.st,v 1.3 2009/10/04 17:40:06 cg Exp $'
! !
