"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayDevelopmentTests subclass:#SpdOperationCreatorTest
	instanceVariableNames:'operation creator'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpdOperationCreatorTest methodsFor:'accessing'!

createOperation
	operation := creator createOperation
!

firstInputPart
	^operation inputBody message parts first
!

firstOutputPart
	^operation outputBody message parts first
!

secondInputPart
	^operation inputBody message parts second
!

setCreatorFor: aSelector 
	| sa |
	sa := SpdSourceAnalysis new setGlobal: SoapTestMetaGenerator selector: aSelector.
	creator := sa operationCreator.
	creator namespace: #'urn:test'.
	operation := creator createOperation
!

setEmptyCreator
	creator := SpdOperationCreator new.
	creator sourceAnalysis: SpdSourceAnalysis new
! !

!SpdOperationCreatorTest methodsFor:'asserting'!

assertOperationNS
	self assert: operation inputBody namespace = #'urn:test'.
	self assert: operation outputBody namespace = #'urn:test'
!

assertOperationUsesRPC
	self assert: operation inputBody handlerClass == SpwRPCHandler.
	self assert: operation outputBody handlerClass == SpwRPCHandler
! !

!SpdOperationCreatorTest methodsFor:'tests'!

testAnswer
	self setEmptyCreator.
	creator answer: 'xsd:string'.
	self assert: creator operation outputBody message parts size = 1.
	self assert: creator operation outputBody sourceFormat = #(#('Result' 'xsd:string'))
!

testAnswerNil
	self setEmptyCreator.
	creator answer: 'nil'.
	self assert: creator operation outputBody message parts isEmpty.
	self assert: creator operation outputBody sourceFormat = #()
!

testAnswerVoid
	self setEmptyCreator.
	creator answer: 'void'.
	self assert: creator operation outputBody message parts isEmpty.
	self assert: creator operation outputBody sourceFormat = #()
!

testInvalidIn
        self setEmptyCreator.
        self should: [creator in: 'xsd:string'] raise: (Error , AssertionFailedError).
        self should: [creator in: #'xsd:string'] raise: (Error , AssertionFailedError).
        self should: [creator in: #(())] raise: (Error , AssertionFailedError).
        self should: [creator in: #(('abc'))] raise: (Error , AssertionFailedError).
        self should: [creator in: #(('abc' 'nocolon'))] raise: (Error , AssertionFailedError).



!

testSampleCreatorDocumentation
	self setCreatorFor: #sampleCreatorDocumentation:.
	self createOperation.
	self assertOperationUsesRPC.
	self assert: operation name = 'sampleCreatorDocumentation'.
	self assert: operation inputBody sourceFormat = #(#('sampleCreatorDocumentation' 'xsd:string')).
	self assert: operation outputBody sourceFormat = #(#('return' 'xsd:boolean')).
	self assert: operation documentation = 'foo'
!

testSampleCreatorMultipleOuts
	self setCreatorFor: #sampleCreatorMultipleOuts.
	self createOperation.
	self assertOperationUsesRPC.
	self assertOperationNS.
	self assert: operation name = 'sampleCreatorMultipleOuts'.
	self assert: operation documentation isEmpty.
	self assert: operation inputBody message parts isEmpty.
	self assert: operation outputBody sourceFormat = #(#('return' 'xsd:boolean') #('foo' 'xsd:string'))
!

testSampleCreatorNamed
	self setCreatorFor: #sampleCreatorNamed:.
	self createOperation.
	self assertOperationUsesRPC.
	self assert: operation name = 'MyName'
!

testSampleCreatorOneIn
	self setCreatorFor: #sampleCreatorOneIn:.
	self createOperation.
	self assertOperationUsesRPC.
	self assertOperationNS.
	self assert: operation name = 'sampleCreatorOneIn'.
	self assert: operation inputBody sourceFormat = #(#('foo' 'xsd:boolean')).
	self assert: operation outputBody sourceFormat = #(#('return' 'xsd:boolean'))
!

testSampleCreatorOutOnly
	self setCreatorFor: #sampleCreatorOutOnly.
	self createOperation.
	self assertOperationUsesRPC.
	self assertOperationNS.
	self assert: operation name = 'sampleCreatorOutOnly'.
	self assert: operation documentation isEmpty.
	self assert: operation inputBody message parts isEmpty.
	self assert: self firstOutputPart partName = 'return'.
	self assert: self firstOutputPart typeName qString = 'xsd:boolean'
!

testSampleCreatorTwoIn
	self setCreatorFor: #sampleCreatorTwoIn:and:.
	self createOperation.
	self assertOperationUsesRPC.
	self assertOperationNS.
	self assert: operation name = 'sampleCreatorTwoIn'.
	self assert: self firstInputPart partName = 'foo'.
	self assert: self firstInputPart typeName qString = 'xsd:boolean'.
	self assert: self secondInputPart partName = 'bar'.
	self assert: self secondInputPart typeName qString = 'xsd:string'.
	self assert: self firstOutputPart partName = 'return'.
	self assert: self firstOutputPart typeName qString = 'xsd:boolean'
! !

!SpdOperationCreatorTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpdOperationCreatorTest.st,v 1.2 2008/09/07 17:27:04 cg Exp $'
! !
