"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayDevelopmentTests subclass:#SpdSourceAnalysisTest
	instanceVariableNames:'sourceAnalysis'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpdSourceAnalysisTest methodsFor:'accessing'!

testAnswerCommentFrom
	self assert: (sourceAnalysis answerTypeStringFrom: 'answer an xsd:string') = 'xsd:string'.
	self assert: (sourceAnalysis answerTypeStringFrom: 'Answer an xsd:string') = 'xsd:string'.
	self assert: (sourceAnalysis answerTypeStringFrom: 'answer a xsd:string.') = 'xsd:string.'.
	self assert: (sourceAnalysis answerTypeStringFrom: 'answer  xsd:string') = 'xsd:string'.
	self assert: (sourceAnalysis answerTypeStringFrom: 'answer String') = 'String'
!

testCommentArray
	#(('abc""nil' ('')) ('abc""""nil' ('' '')) ('abc $"""""nil' ('' '')) ('abc""1.""nil' ('' '')) ('abc""1.$"""nil' ('' ''))
	('abc"a"1."b"nil' ('a' 'b')) ('abc"a"$"1."b"nil' ('a' 'b')) ('abc"ac"1."bd"nil' ('ac' 'bd'))
('abc 
"ac"
"bd"
nil' ('ac' 'bd'))) 
	do: [ :each |
		sourceAnalysis methodSource: each first.
		self assert: sourceAnalysis commentArray = each second]




!

testCommentArrayUnclosed
	sourceAnalysis methodSource: 'abc""1."nil'.
	sourceAnalysis commentArray
!

testCommentStrippedSource
	#(('abc""nil' 'abc nil') ('abc""""nil' 'abc  nil') ('abc""1.""nil' 'abc 1. nil') ('abc $"' 'abc $"') ('abc $"."c"nil' 'abc $". nil'))
		do: [ :each |
			sourceAnalysis methodSource: each first.
			self assert: sourceAnalysis commentStrippedSource = each second].

	

!

testCommentStrippedUnclosed
	sourceAnalysis methodSource: 'abc""1."nil'.
	sourceAnalysis commentStrippedSource
!

testExtractCommentAnswerEx
	sourceAnalysis methodSource: 'blah 
"answer"'.
	self should: [sourceAnalysis answerTypeString] raise: Error.
	sourceAnalysis methodSource: 'blah 
"answer a"'.
	self should: [sourceAnalysis answerTypeString] raise: Error
!

testExtractCommentAnswerOnly
#(('blah 
"Answer string"' 'string')
('blah 
"answer string"' 'string')
('blah 
"answer a string"' 'string'))
	do: [ :each |
		sourceAnalysis methodSource: each first.
		self assert: sourceAnalysis answerTypeString = each second]

!

testInputParts
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleUnaryC.
	self assert: sourceAnalysis inputParts isEmpty.

	"one input ... argument cant be converted to typeString"
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleArg1Empty:.
	self should: [sourceAnalysis inputParts] raise: Error.

	"one input ... argument can be converted to typeString"
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleArg1Type:.
	self assert: sourceAnalysis inputParts size = 1.
	self assert: (sourceAnalysis inputParts first isKindOf: SpmdTypePart).

	"multiple"
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleArg2C:and:.
	self assert: sourceAnalysis inputParts size = 2.
	self assert: (sourceAnalysis inputParts first isKindOf: SpmdTypePart).
	self assert: (sourceAnalysis inputParts second isKindOf: SpmdTypePart)
!

testIsImpliedSelfReturn
	sourceAnalysis global: SoapTestMetaGenerator.
	sourceAnalysis selector: #dummy.
	sourceAnalysis methodSource: 'abc "^something"'.
	self assert: sourceAnalysis isImpliedSelfReturn.
	#('abc ^something' 'abc "^something" ^something' 'abc "^something""again" ^something')
		do: [ :each |
			sourceAnalysis methodSource: each.
			self deny: sourceAnalysis isImpliedSelfReturn].
!

testIsUnary
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleUnaryC.
	self assert: sourceAnalysis isUnaryMethod.
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis setGlobal: SoapTestMetaGenerator selector: #sampleArg1Empty:.
	self deny: sourceAnalysis isUnaryMethod.
	
!

testOutputPartsGoodComment
	| ans |
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis methodSource: 'blah "answer a string" ^nil'.
	ans := sourceAnalysis outputParts.
	self assert: ans notEmpty.
	self assert: (ans first isKindOf: SpmdTypePart).
	self assert: ans first partName = 'Result'.
	self assert: ans first typeName qString = 'xsd:string'
!

testOutputPartsNil
	#('blah "answer  nil "' 'blah "answer  null "' 'blah "answer  void "')
		do: [ :each |
			sourceAnalysis := SpdSourceAnalysis new.
			sourceAnalysis methodSource: each.
			self assert: sourceAnalysis outputParts isEmpty].
!

testOutputPartsNoComment
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis methodSource: 'blah ^nil'.
	self should: [sourceAnalysis outputParts] raise: Error
!

testOutputPartsPrefixedComment
	| ans |
	sourceAnalysis := SpdSourceAnalysis new.
	sourceAnalysis methodSource: 'blah "answer xsd:string" ^nil'.
	ans := sourceAnalysis outputParts.
	self assert: ans notEmpty.
	self assert: (ans first isKindOf: SpmdTypePart).
	self assert: ans first partName = 'Result'.
	self assert: ans first typeName qString = 'xsd:string'
!

testSetPairsFromSourceMultiple
	sourceAnalysis := SpdSourceAnalysis new.
	#( 'aMethod: arg next: arg2' 'aMethod:arg next: arg2' 'aMethod:arg next:arg2' 'aMethod: arg next: arg2 somethingElse')
		do: [ :each | | pairs |
			pairs := sourceAnalysis setPairsFromSource: each.
			self assert: pairs first = ('aMethod' -> 'arg').
			self assert: pairs second = ('next' -> 'arg2')]

!

testSetPairsFromSourceSingle
	sourceAnalysis := SpdSourceAnalysis new.
	#('aMethod:arg' 'aMethod: arg' 'aMethod: arg somethingElse')
		do: [ :each |
			self assert: (sourceAnalysis setPairsFromSource: each) first = ('aMethod' -> 'arg')]
!

testSetPairsFromSourceUnary
	sourceAnalysis := SpdSourceAnalysis new.
	#('aMethod' 'aMethod somethingElse')
		do: [ :each |
			self assert: (sourceAnalysis setPairsFromSource: each) first = ('aMethod' -> nil)]
! !

!SpdSourceAnalysisTest methodsFor:'set up'!

setUp
	super setUp.
	sourceAnalysis := SpdSourceAnalysis new
! !

!SpdSourceAnalysisTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpdSourceAnalysisTest.st,v 1.1 2008/09/07 15:49:23 cg Exp $'
! !
