"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SpmdTests subclass:#SpmdMessageTest
	instanceVariableNames:'message specs'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpmdMessageTest methodsFor:'accessing'!

assertValidDocSpecs
	self assert: specs size = 2.
	specs do: [:each | each isKindOf: XeQName].
	self assert: specs first = ('string' ns: XeNamespace soapEnc).
	self assert: specs second = ('integer' ns: XeNamespace soapEnc)
!

be1ElementPart
	message := SpmdMessage 
				forSingleElement: ('TradePriceRequest' ns: #'http://example.com/stockquote.xsd').
	message name: 'GetLastTradePrice'
!

be1TypedPart
	message := SpmdMessage onPart: (SpmdTypePart partName: 'param1' typeName: 'xsd:string').
	message name: 'testOpInput'
!

be2ElementParts
	message := SpmdMessage name: 'fooInput'
				parts: (Array with: (SpmdElementPart partName: 'part1'
								elementName: (XeQName 
										prefix: 'soap'
										local: 'string'
										ns: XeNamespace soapEnc))
						with: (SpmdElementPart partName: 'part2'
								elementName: (XeQName 
										prefix: 'soap'
										local: 'integer'
										ns: XeNamespace soapEnc)))
!

be2TypedParts
	message := SpmdMessage 
				onParts: (Array with: (SpmdTypePart partName: 'param1' typeName: 'xsd:string')
						with: (SpmdTypePart partName: 'param2' typeName: 'xsd:int'))
!

beVoid
	message := SpmdMessage onParts: (Array new: 0)




	
! !

!SpmdMessageTest methodsFor:'asserting'!

assertValidRPCSpecs
	self assert: specs size = 2.
	specs do: [:each | each isKindOf: XeElementSpec].
	self assert: specs first qName = ('param1' ns: #'').
	self assert: specs second qName = ('param2' ns: #'').
	self assert: specs first typeName = (XePName xsd: 'string').
	self assert: specs second typeName = (XePName xsd: 'int')
! !

!SpmdMessageTest methodsFor:'tests'!

testAddDefinitionElementsTo
	| node typespace|
	self be2ElementParts.
	typespace := self get2001Typespace.
	node := self newNode: 'blah'.
	message defineMessage: node typespace: typespace.
	self assertXML: node xml
		isSameAs: '<blah xmlns:soap="http://schemas.xmlsoap.org/soap/encoding/"><message name="fooInput"><part name="part1" element="soap:string"/><part name="part2" element="soap:integer"/></message></blah>'
!

testAddDefinitionTypedTo
	| node xml domNode typespace  |
	self be1TypedPart.
	typespace := self get2001Typespace.
	node := self newNode: 'blah'.
	message defineMessage: node typespace: typespace.
	self assertXML: (xml := node xml)
		isSameAs: '<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema"><message name="testOpInput"><part name="param1" type="xsd:string"/></message></blah>'.

	"test namespace normalizes correctly"
	node := self newNode: 'blah'.
	node addNSPrefix: 'xsd' to: 'urn:blah'.
	message defineMessage: node typespace: typespace.
	domNode := node asParserNode.
	self deny: (domNode firstChild firstChild getAttribute: 'type') xmlPrefix = 'xsd'
!

testDocSelectors
	self be2TypedParts.
	message parts first selector: #myDocSelector1.
	message parts second selector: #myDocSelector2.
	self assert: (message docSelectors: nil) size = 2.
	self assert: (message docSelectors: #('param1')) size = 1.
	self should: [message docSelectors: #('na')] raise: Error.
!

testDocSpecs
	self be2ElementParts.
	specs := message docSpecs: nil namespace: #'urn:blah'.
	self assertValidDocSpecs.
	specs := message docSpecs: #('part1' 'part2') namespace: #'urn:blah'.
	self assertValidDocSpecs.
!

testDocSpecsWithPartNames
	self be2ElementParts.
	specs := message docSpecs: #('part2') namespace: #'urn:blah'.
	self assert: specs size = 1.
	self assert: specs first = ('integer' ns: XeNamespace soapEnc)
!

testRpcSpecs
	self be2TypedParts.
	specs := message rpcSpecs: nil.
	self assertValidRPCSpecs.
	specs := message rpcSpecs: #('param1' 'param2').
	self assertValidRPCSpecs
!

testRpcSpecsVoid
	self beVoid.
	self assert: (message rpcSpecs: nil) isEmpty.
!

testRpcSpecsWithPartNames
	self be2TypedParts.
	specs := message rpcSpecs: #('param2').
	self assert: specs first qName = ('param2' ns: #'').
	self assert: specs first typeName = (XePName xsd: 'int')
! !

!SpmdMessageTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpmdMessageTest.st,v 1.1 2008/09/07 15:49:44 cg Exp $'
! !
