"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayServiceTests subclass:#SprayClientTest
	instanceVariableNames:'service client'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SprayClientTest methodsFor:'accessing'!

setClientDoc
	service := SprayTestDocService new.
	client := service createClientTo: 'http://foo'
!

setClientRpc
	service := SprayTestRpcService new.
	client := service createClientTo: 'http://foo'.
! !

!SprayClientTest methodsFor:'set up'!

setUp
	super setUp.
	self setClientRpc
! !

!SprayClientTest methodsFor:'tests'!

testDestination
	| soapSend|
	client immediateDestination: 'http://foo'.

	soapSend := client send: 'testMethod1'.
	self assert: (soapSend immediateDestination = 'http://foo').
	soapSend immediateDestination: 'http://bar'.
	self assert: (soapSend immediateDestination = 'http://bar').

	soapSend := client send: 'testMethod1'.
	self assert: (soapSend immediateDestination = 'http://foo').

!

testIncludesOperationNamed
	self assert: (client includesOperationNamed: 'testMethod1').
	self deny: (client includesOperationNamed: 'NottestMethod1').
	
!

testNewSoapSendFor
	| soapSend operation |
	operation := client operationNamed: 'testMethod1'.
	soapSend := client newSoapSendFor: operation.
	self assert: (soapSend isKindOf: SpraySoapSend).
	self assert: soapSend soapAction = operation soapAction.
	self 
		assert: (soapSend atQP: 'transport:ImmediateDestination') = client immediateDestination
	"self assert: soapSend binding timeouts == client binding timeouts ."
!

testOperationNamed
	| op|
	op := client operationNamed: 'testMethod1'.
	self assert: (op isKindOf: SpmdOperation).
	self should: [client operationNamed: 'notAOperationName'] raise: Error
!

testSendDoc
	| soapSend |
	self setClientDoc.
	soapSend := client sendDoc: 'echoString'
				with: (XeQStruct param: #echoString value: (XeQStruct param: #param0 value: 'abc')).
	self assert: (soapSend isKindOf: SpraySoapSend).
	self assert: (soapSend outEnvelope isKindOf: SpwOutDocument).
	self assert: (soapSend inEnvelope isKindOf: SpwInDocument).
	self assert: soapSend inEnvelope typespace == client typespace.
	self assert: soapSend outEnvelope typespace == client typespace
!

testSendRpc
	| soapSend |
	soapSend := client send: 'testMethod1'.
	self assert: (soapSend isKindOf: SpraySoapSend).
	self assert: (soapSend outEnvelope isKindOf: SpwRPC).
	self assert: (soapSend inEnvelope isKindOf: SpwRPCReturn).
	self assert: soapSend inEnvelope typespace == client typespace.
	self assert: soapSend outEnvelope typespace == client typespace
!

testSendRpcDocOp
	| soapSend |
	self setClientDoc.
	soapSend := client send: 'echoString' with: 'abc'.
	self assert: (soapSend outEnvelope isKindOf: SpwRPC).
	self assert: (soapSend inEnvelope isKindOf: SpwRPCReturn ).
	self assert: soapSend outEnvelope bodies first object first = 'abc'.
	soapSend soapOutEnvelope
	
!

testSendRpcWith
	| soapSend |
	soapSend := client send: 'testMethod1' with: 'abc'.
	self assert: soapSend outEnvelope bodies first object first = 'abc'.
	soapSend := client send: 'testMethod1' with: nil.
	self assert: soapSend outEnvelope bodies first object first isNil

	
!

testSoapAction
	| soapSend|
	soapSend := client send: 'testMethod1'.
	self assert: (soapSend soapAction = 'testMethod1').
	client soapAction: 'override'.
	soapSend := client send: 'testMethod1'.
	self assert: (soapSend soapAction = 'override').
!

testTimeouts
	| soapSend recTimeout|
	client beLongTimeouts.
	soapSend := client send: 'testMethod1'.
	self assert: (recTimeout := soapSend atScopedQP: #'sprayHttp:timeoutReceive') == (client atMessageQP: #'sprayHttp:timeoutReceive') .
	client beQuickTimeouts.
	soapSend := client send: 'testMethod1'.
	self assert: (soapSend atScopedQP: #'sprayHttp:timeoutReceive') == (client atMessageQP: #'sprayHttp:timeoutReceive' ).
	self deny: recTimeout = (soapSend atScopedQP: #'sprayHttp:timeoutReceive').
	
! !

!SprayClientTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SprayClientTest.st,v 1.1 2008/09/07 15:50:47 cg Exp $'
! !
