"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayServerTests subclass:#SptRespondingExchangeTest
	instanceVariableNames:'testEnvelopes exchange soapNode'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SptRespondingExchangeTest methodsFor:'accessing'!

setExchange: aSymbol 
	"exchange := SpwRespondingExchange received: (testEnvelopes perform: aSymbol) in: typespace"

	exchange := SptRespondingExchange onXmlString: (testEnvelopes perform: aSymbol) xml
! !

!SptRespondingExchangeTest methodsFor:'set up'!

setUp
	super setUp.
	testEnvelopes := SpSprayTestEnvelopes new.
	soapNode := SpwSoapResponder new
! !

!SptRespondingExchangeTest methodsFor:'tests'!

testBeOutFaultMisunderstand
	self setExchange: #getEnvelopeSpec1.
	exchange failMisunderstanding: #().
	self assert: exchange out isFault
!

testConstructionOut
	self setExchange: #getEnvelopeSpec1.
	self assert: exchange outBodyNode localName = 'Body'.
	self assert: exchange outHeaderNode localName = 'Header'.
!

testFailMisunderstanding
	exchange := SptRespondingExchange new.
	exchange failMisunderstanding: #().
	self deny: exchange isContinueState.
	self assert: exchange state = 'Fail'.
	self assert: exchange out isFault
!

testFailWithException
	exchange := SptRespondingExchange new.
	[self error: 'oops'] on: Error do: [:e | exchange failWithException: e].
	self deny: exchange isContinueState.
	self assert: exchange state = 'Fail'.
	self assert: exchange out isFault
!

testFailWithFaultException
	exchange := SptRespondingExchange new.
	[SoapServerFault new beClient signal: 'oops'] on: SoapServerFault
		do: [:e | exchange failWithException: e].
	self deny: exchange isContinueState.
	self assert: exchange state = 'Fail'.
	self assert: exchange out isFault
!

testFirstInBodyBlock
	self setExchange: #getEnvelopeSpec1.
	self assert: exchange firstInBodyBlock qName 
				= ('GetLastTradePrice' ns: #'http://example.org/quotes')
!

testHeaderData
	self setExchange: #getEnvelopeSpec1.
	exchange atQP: #'user:foo' put: #bar.
	self assert: (exchange includesQP: #'user:foo').
	self deny: (exchange includesQP: #'user:bar').
	self assert: (exchange atQP: #'user:foo') = #bar.
	self assert: (exchange atQP: #'user:foo' ifAbsent: []) = #bar.
	self assert: (exchange atQP: #'user:bar' ifAbsent: [#foo]) = #foo.
!

testInHeaderBlocksActorNext
	self setExchange: #getEnvelopeWithActorNext.
	self assert: (exchange inHeaderBlocksFor: soapNode) size = 1.
	self assert: (exchange inHeaderBlocksFor: soapNode) first qName 
				= ('Transaction' ns: #'some-URI')
!

testInHeaderBlocksEmpty
	self setExchange: #getEnvelopeSpec1.
	self assert: (exchange inHeaderBlocksFor: soapNode) isEmpty.
	
!

testInitialize
	exchange := SptRespondingExchange new.
	self assert: (exchange atQP: 'single:Role') = 'RespondingSOAPNode'.
	self assert: (exchange atQP: 'single:State') = 'Receiving'.
	self assert: (exchange atQP: 'transport:FailureReason') = 'None'
!

testIsContinueState
	exchange := SptRespondingExchange new.
	self assert: exchange isContinueState.
	exchange beReceivingState.
	self assert: exchange isContinueState.
	exchange beProcessingState.
	self assert: exchange isContinueState.
	exchange beSuccessState.
	self assert: exchange isContinueState
!

testIsContinueStateNot
	exchange := SptRespondingExchange new.
	exchange beProcessingState.
	exchange beFailState.
	self deny: exchange isContinueState.
	exchange beFailState.
	self deny: exchange isContinueState
!

testIsHeaderTarget
	| node |
	exchange := SptRespondingExchange new.
	node := self 
				rootNodeFor: 
'<t:Transaction xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
 xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1"/>'.
	self assert: (exchange isHeaderBlock: node targetedAt: soapNode).
	node := self 
				rootNodeFor: 
'<t:Transaction xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
 xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1"
SOAP-ENV:actor="http://schemas.xmlsoap.org/soap/actor/next"/>'.
	self assert: (exchange isHeaderBlock: node targetedAt: soapNode).
	node := self 
				rootNodeFor: 
'<t:Transaction xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
 xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1"
SOAP-ENV:actor="http://notme"/>'.
	self deny: (exchange isHeaderBlock: node targetedAt: soapNode).
	soapNode actors add: #'http://notme'.
	node := self 
				rootNodeFor: 
'<t:Transaction xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
 xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1"
SOAP-ENV:actor="http://notme"/>'.
	self assert: (exchange isHeaderBlock: node targetedAt: soapNode)
!

testMUs1
	self setExchange: #getEnvelopeWithMustHeader.
	self assert: (exchange mustUnderstandsFor: soapNode) size = 1.
	self 
		assert: (exchange mustUnderstandsFor: soapNode) first = ('Transaction' ns: #'some-URI')
!

testMUs1of2
	self setExchange: #getEnvelopeWithMustHeader2.
	self assert: (exchange mustUnderstandsFor: soapNode) size = 1.
	self 
		assert: (exchange mustUnderstandsFor: soapNode) first = ('Transaction' ns: #'some-URI')
!

testMUs2
	self setExchange: #getEnvelopeWith2MustHeaders.
	self assert: (exchange mustUnderstandsFor: soapNode) size = 2.
	self 
		assert: (exchange mustUnderstandsFor: soapNode) second = ('Transaction' ns: #'some-URI')
!

testMUsEmpty
	self setExchange: #getEnvelopeSpec1.
	self assert: (exchange mustUnderstandsFor: soapNode) isEmpty.
	
!

testMUsFalse
	self setExchange: #getEnvelopeWithMustHeaderFalse.
	self assert: (exchange mustUnderstandsFor: soapNode) isEmpty
! !

!SptRespondingExchangeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SptRespondingExchangeTest.st,v 1.1 2008/09/07 15:52:38 cg Exp $'
! !
