"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayServerTests subclass:#SpwBlockReceiverTest
	instanceVariableNames:'receiver instance'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpwBlockReceiverTest methodsFor:'Dolphin methods'!

setUp
	super setUp.
	receiver := SpwSoapBReceiver receiveFor: (instance := SprayTestObject new) at: #'urn:blah'
! !

!SpwBlockReceiverTest methodsFor:'accessing'!

addErrorGuard
	receiver addGuard: (SpwPlugableGuard new guardBlock: [:ex | self error: 'error'])
!

addFailingGuard
	receiver addGuard: (SpwPlugableGuard new guardBlock: [:ex | false])
!

addPassingGuard
	receiver addGuard: (SpwPlugableGuard new guardBlock: [:ex | true])
!

getTestMessage
	^Message selector: #testMethod1: arguments: #('test')
!

sendTestMessageWith: args
	^receiver sendMessage: (Message selector: #testMethod1: arguments: args) exchange: nil
	
! !

!SpwBlockReceiverTest methodsFor:'tests'!

testAllowNilArguments
	receiver beAllowNilArguments.
	self assert: (self sendTestMessageWith: #('test')).
	self assert: (self sendTestMessageWith: #(nil)).
	receiver beDisallowNilArguments.
	self assert: (self sendTestMessageWith: #('test')).
	self should: [self sendTestMessageWith: #(nil)] raise: SoapServerFault .
	receiver beAllowNilArguments.
	self assert: (self sendTestMessageWith: #('test')).
	self assert: (self sendTestMessageWith: #(nil)).
!

testConstruction
	self assert: receiver object == instance 
	
!

testErrorGuard
	self addErrorGuard.
	self 
		should: [self sendTestMessageWith: #('test')]
		raise: Error
	
	
!

testGuards
	| ex |
	ex := SptRespondingExchange new.
	self addPassingGuard.
	self assert: (receiver sendMessage: self getTestMessage exchange: ex).
	self assert: ex isContinueState.
	self addFailingGuard.
	self assert: (receiver sendMessage: self getTestMessage exchange: ex) isNil.
	self deny: ex isContinueState.
	ex := SptRespondingExchange new.
	self addPassingGuard.
	self assert: (receiver sendMessage: self getTestMessage exchange: ex) isNil.
	self deny: ex isContinueState
!

testSendMessage
	self assert: (receiver sendMessage: self getTestMessage ).
	
! !

!SpwBlockReceiverTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpwBlockReceiverTest.st,v 1.1 2008/09/07 15:52:45 cg Exp $'
! !
