"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SpwOutEnvelopeTests subclass:#SpwRPCTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpwRPCTest methodsFor:'accessing'!

getEnvClass
	^SpwRPC
!

getSoapStruct
	^(XeQStruct new)
		at: #username put: 'user';
		at: #password put: 'dolphin';
		yourself
!

setEmptyBody
	super setUp.
	envelope := SpwRPC newIn: typespace.
	envelope 
		bodyCoders: #()
		headerCoders: #()
! !

!SpwRPCTest methodsFor:'set up'!

setUp
	super setUp.
	envelope := SpwRPC newIn: typespace.
	envelope 
		bodyCoders: (Array with: self wireObjects rpcEnc2)
		headerCoders: #()
! !

!SpwRPCTest methodsFor:'tests'!

testRPCArguments
	| rpcObject |
	envelope arguments: #('abc' 123).
	self assert: ((rpcObject := envelope bodyObjects first) isKindOf: XeQStruct).
	self assert: rpcObject size = 2.
	self assert: rpcObject first = 'abc'.
	self assert: rpcObject second = 123
!

testRPCArgumentsNil
	| rpcObject |
	envelope arguments: #(nil nil).
	self assert: ((rpcObject := envelope bodyObjects first) isKindOf: XeQStruct).
	self assert: rpcObject size = 2.
	self assert: rpcObject first isNil.
	self assert: rpcObject second isNil
!

testSoapOutEnvelopeRPC
	envelope arguments: #('abc' 123).
	self encodeSoapOut.
	self assertXML: soapOutEnv xml
		isSameAs: '<?xml version="1.0"?>
<SOAP-ENV:Envelope 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<ns1:myOperation xmlns:ns1="urn:blah"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<param1 xsi:type="xsd:string">abc</param1>
<param2 xsi:type="xsd:int">123</param2>
</ns1:myOperation>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testSoapOutWithHeader
	self addDocEncSHeader.
	envelope firstHeaderObject: 'def'.
	envelope arguments: #('abc' 123).
	self encodeSoapOut.
	self assertXML: soapOutEnv xml
		isSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
<ns1:param1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xsi:type="xsd:string" xmlns:ns1="urn:blah">def</ns1:param1>
</SOAP-ENV:Header>
<SOAP-ENV:Body><m:myOperation SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:m="urn:blah"><param1 xsi:type="xsd:string">abc</param1><param2 xsi:type="xsd:int">123</param2></m:myOperation></SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testSoapOutWithHeaderNil
	self addDocEncSHeader.
	envelope arguments: #('abc' 123).
	self encodeSoapOut.
	self assertXML: soapOutEnv xml
		isSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body><m:myOperation SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:m="urn:blah"><param1 xsi:type="xsd:string">abc</param1><param2 xsi:type="xsd:int">123</param2></m:myOperation></SOAP-ENV:Body></SOAP-ENV:Envelope>'

! !

!SpwRPCTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpwRPCTest.st,v 1.1 2008/09/07 15:54:49 cg Exp $'
! !
