"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayServerTests subclass:#SpwSoapResponderTest
	instanceVariableNames:'soapNode'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpwSoapResponderTest methodsFor:'set up'!

setUp
	super setUp.
	soapNode := SpwSoapResponder new.
	soapNode addHandlers: (Array with: self wireObjects handlerRPC).
	soapNode addSoapBReceiver: (SpwSoapBReceiver receiveFor: SprayTestObject new at: #'urn:blah')
! !

!SpwSoapResponderTest methodsFor:'tests'!

testAddReceiver
	| o1 o2 |
	soapNode 
		addSoapBReceiver: (SpwSoapBReceiver receiveFor: (o1 := SprayTestObject new) at: #'urn:blah1').
	soapNode 
		addSoapBReceiver: (SpwSoapBReceiver receiveFor: (o2 := SprayTestObject new) at: #'urn:blah2').
	self assert: (soapNode receiverAt: #'urn:blah1') object == o1.
	self assert: (soapNode receiverAt: #'urn:blah2') object == o2.
	self should: [soapNode receiverAt: #'urn:blah3'] raise: SoapServerFault
!

testAddReceiverNullNS
	| o1 o2 |
	soapNode 
		addSoapBReceiver: (SpwSoapBReceiver receiveFor: (o1 := SprayTestObject new) at: #'urn:blah1').
	soapNode addSoapBReceiver: (SpwSoapBReceiver receiveFor: (o2 := SprayTestObject new) at: #'').
	self assert: (soapNode receiverAt: #'urn:blah1') object == o1.
	self assert: (soapNode receiverAt: #'') object == o2.
	self should: [soapNode receiverAt: #'urn:blah3'] raise: SoapServerFault
!

testBlockHandlers
	self assert: soapNode numberOfHandlers = 1
!

testHTTPRequestOK
	| exchange |
	exchange := SptRespondingExchange onXmlString: self wireObjects envXmlRpcEnc2.
	soapNode process: exchange.
	self deny: exchange out isFault
!

testHandlerFor
	| handler |
	handler := soapNode handlerFor: self qMyOperation ifAbsent: [].
	self assert: (handler isKindOf: SpwSoapBHandler).
	handler := soapNode handlerFor: ('myOperation' ns: #'urn:NOTblah') ifAbsent: [].
	self assert: handler isNil
!

testSelectCantHandle
	| qNames |
	soapNode receiverTable at: #'urn:blah' put: nil.
	qNames := #().
	self assert: (soapNode selectCantHandle: qNames) isEmpty.
	qNames := Array with: self qMyOperation.
	self assert: (soapNode selectCantHandle: qNames) isEmpty.
	qNames := Array with: self qMyOperation with: ('param1' ns: #'urn:NOTblah').
	self assert: (soapNode selectCantHandle: qNames) size = 1
! !

!SpwSoapResponderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpwSoapResponderTest.st,v 1.1 2008/09/07 15:55:02 cg Exp $'
! !
