"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayClientTests subclass:#SpwSoapSendTest
	instanceVariableNames:'service client transport soapSend'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SpwSoapSendTest methodsFor:'accessing'!

beTestFaultClient
	transport 
		envResponse: 
'<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
<soap:Body>
<soap:Fault>
<faultcode>soap:Client</faultcode>
<faultstring>Server did not recognize the value of HTTP Header SOAPAction: ''''.</faultstring>
<faultactor>http://131.107.72.13/test/simple.asmx</faultactor>
<detail/>
</soap:Fault>
</soap:Body>
</soap:Envelope>'.
	
!

beTestMethod1Client
	transport 
		envResponse: 
'<SOAP-ENV:Envelope 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:n1="http://soapinterop.org/">
<SOAP-ENV:Body>
<m:testMethod1Response xmlns:m="urn:SprayTesting123">
<theAnswer xsi:type="xsd:boolean">true</theAnswer>
</m:testMethod1Response>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'.
	
!

beTestMethod1ClientTypeMismatch
	transport 
		envResponse: 
'<SOAP-ENV:Envelope 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:n1="http://soapinterop.org/">
<SOAP-ENV:Body>
<m:testMethod1Response xmlns:m="urn:SprayTesting123">
<theAnswer xsi:type="xsd:int">0</theAnswer>
</m:testMethod1Response>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'.
!

beTestMethod1ClientWrongName
	transport 
		envResponse: 
'<SOAP-ENV:Envelope 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:n1="http://soapinterop.org/">
<SOAP-ENV:Body>
<m:NOTtestMethod1Response xmlns:m="urn:SprayTesting123">
<theAnswer xsi:type="xsd:boolean">false</theAnswer>
</m:NOTtestMethod1Response>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'.
! !

!SpwSoapSendTest methodsFor:'set up'!

setUp
	super setUp.
	service := SprayTestRpcService new.
	client := service createClientTo: 'http://www.blah.org'.
	client httpRequestSoapBinding: (transport := SptHTTPLocalTransport new).
! !

!SpwSoapSendTest methodsFor:'tests'!

testReceiveTypeMismatch
	self beTestMethod1ClientTypeMismatch.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self should: [soapSend value] raise: XeTypeMismatch.
	self assert: soapSend outXmlOrEmpty notEmpty.
	self assert: soapSend inXmlOrEmpty notEmpty.
	self assert: soapSend extractResuming = false
!

testReceiveWrongName
	self beTestMethod1ClientWrongName.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self deny: soapSend value.
	self assert: soapSend outXmlOrEmpty notEmpty.
	self assert: soapSend inXmlOrEmpty notEmpty
!

testSendFaultResponse
	self beTestFaultClient.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self should: [soapSend value] raise: SoapSendFault.
	self assert: soapSend outXmlOrEmpty notEmpty.
	self assert: soapSend inXmlOrEmpty notEmpty
!

testSendRequest
	self beTestMethod1Client.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self assert: soapSend value.
	self assert: soapSend outXmlOrEmpty notEmpty.
	self assert: soapSend inXmlOrEmpty notEmpty
!

testSendRequestReset
	self beTestMethod1Client.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self assert: soapSend value.
	"soapSend reset.
	self assert: soapSend inXmlOrEmpty isEmpty.
	self assert: soapSend inEnvelope bodyObjects first isNil"
!

testValueForXml
	self beTestMethod1Client.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self assert: (soapSend valueForXml: '<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><m:testMethod1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:m="urn:SprayTesting123"><someString xsi:type="xsd:string">abc</someString></m:testMethod1></SOAP-ENV:Body></SOAP-ENV:Envelope>').
	self assert: soapSend outXmlOrEmpty notEmpty.
	self assert: soapSend inXmlOrEmpty notEmpty
!

testValueForceTypeMismatch
	self beTestMethod1ClientTypeMismatch.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self should: [soapSend value] raise: XeTypeMismatch.
	self assert: soapSend extractForcing = 0
!

testValueResumeTypeMismatch
	self beTestMethod1ClientTypeMismatch.
	soapSend := client send: 'testMethod1' with: 'abc'.
	self should: [soapSend value] raise: XeTypeMismatch.
	self deny: soapSend extractResuming
! !

!SpwSoapSendTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SOAP__SpwSoapSendTest.st,v 1.1 2008/09/07 15:55:15 cg Exp $'
! !
