"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SpwSoapEnvelopeTests subclass:#SpOutEnvelopeTest
	instanceVariableNames:'envelope'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore-TestCases'
!


!SpOutEnvelopeTest methodsFor:'asserting'!

assertEnvelopeXML: xmlString
	| xml |
	self assertXML: (xml := envelope xml) isSameAs: xmlString
! !

!SpOutEnvelopeTest methodsFor:'set up'!

setUp
	super setUp.
	envelope := SpwSoapOutEnvelope new.
	envelope createEnvelope.
! !

!SpOutEnvelopeTest methodsFor:'tests'!

testAddHeader
	envelope addHeader addElement: 'hi'.
	self assertEnvelopeXML: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header><hi/></SOAP-ENV:Header>
<SOAP-ENV:Body/></SOAP-ENV:Envelope>'
!

testAddXSINamespace
	envelope addXSINamespace: #'urn:blah'.
	self assertEnvelopeXML: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsi="urn:blah" 
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body/>
</SOAP-ENV:Envelope>'
!

testAsInEnvelope
	self assert: (envelope asInEnvelope isKindOf: SpwSoapInEnvelope).
	self assertXML: envelope asInEnvelope xml isSameAs: envelope xml
!

testBeFaultCode
	self deny: envelope isFault.
	envelope beFaultCode: 'blah' string: 'foo' .
	self assert: envelope isFault.
	self assertEnvelopeXML:
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
  xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
  <SOAP-ENV:Body>
    <SOAP-ENV:Fault>
      <faultcode>SOAP-ENV:blah</faultcode>
      <faultstring>foo</faultstring>
</SOAP-ENV:Fault></SOAP-ENV:Body></SOAP-ENV:Envelope>'.
!

testEnvelopeCreation
	self assertEnvelopeXML:
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body/>
</SOAP-ENV:Envelope>'
!

testHeaderNodeTwice
	envelope headerNode addElement: 'hi'.
	envelope headerNode addElement: 'hiagain'.
	self assertEnvelopeXML:
'<?xml version="1.0"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header><hi/><hiagain/></SOAP-ENV:Header>
<SOAP-ENV:Body/></SOAP-ENV:Envelope>'
!

testRemoveEnvelopeInstructions
	self assert: (envelope xml beginsWith: '<?xml version="1.0"?>').
	envelope removeXMLInstructions.
	self deny: (envelope xml beginsWith: '<?xml version="1.0"?>').
! !

!SpOutEnvelopeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SpOutEnvelopeTest.st,v 1.2 2003/01/20 19:02:15 cg Exp $'
! !
