"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SpwSoapEnvelopeTests subclass:#SpSoapInEnvelopeTest
	instanceVariableNames:'envelope'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore-TestCases'
!


!SpSoapInEnvelopeTest methodsFor:'tests'!

testEnvelopeWithRootAttribute
	envelope := testEnvelopes getEnvelopeWithRootAttribute.
	self assert: envelope hasBody.
	self assert: envelope firstBodyBlock localName = 'PurchaseOrder'.
	self deny: envelope isFault.
	self assert: envelope firstBodyBlock qName = (self blahQ: 'PurchaseOrder')
!

testEnvelopeWithoutBody
	self should: [testEnvelopes getEnvelopeWithoutBody] raise: SoapServerFault
!

testEnvelopeWithoutEnvelope
	self should: [SpwSoapInEnvelope fromXmlString: ''] raise: SoapServerFault
!

testEnvelopeWrongNS
	self should: [testEnvelopes getEnvelopeWrongNS] raise: SoapServerFault
!

testExtractSoapAction
        
	#(('SOAPAction:""' '') ('SOAPAction: ""' '') ('SOAPAction:"normal"' 'normal') ('SOAPAction: "normal"' 'normal'))
	"       do: [ :each |
			envelope := SpwSoapInEnvelope new.
			envelope httpHeaders: each first.
			envelope extractSoapAction.
			self assert: envelope soapAction = each second]."
!

testExtractSoapActionEmpty
	#(nil '' ' '
'
' 'SOAPAction:')
	"do: [ :each |
		envelope := SpwSoapInEnvelope new.
		envelope extractSoapAction.
		self assert: (envelope soapAction isNil or: [envelope soapAction = ''])]"

!

testFaultEnvelope
	| e |
	envelope := testEnvelopes getFaultEnvelope.
	self assert: envelope hasBody.
	self assert: envelope isFault.
	self assert: envelope faultcode = 'soap:Client'.
	self 
		assert: envelope faultstring = 'Server did not recognize the value of HTTP Header SOAPAction: ''''.'.
	e := envelope faultException.
	self assert: (e isKindOf: SoapSendFault).
	self assert: e faultCode = 'soap:Client'.
	self assert: e faultString = 'Server did not recognize the value of HTTP Header SOAPAction: ''''.'
!

testFaultEnvelopeSpec10
	| e |
	envelope := testEnvelopes getFaultEnvelopeSpec10.
	self assert: envelope isFault.
	self assert: envelope faultcode = 'SOAP-ENV:Server'.
	self assert: envelope faultstring = 'Server Error'.
	self assert: envelope faultDetail notEmpty.
	e := envelope faultException.
	self assert: (e isKindOf: SoapSendFault).
	self assert: e faultCode = 'SOAP-ENV:Server'.
	self assert: e faultString = 'Server Error'
!

testFaultEnvelopeSpec9
	| e |
	envelope := testEnvelopes getFaultEnvelopeSpec9.
	self assert: envelope isFault.
	self assert: envelope faultcode = 'SOAP-ENV:MustUnderstand'.
	self assert: envelope faultstring = 'SOAP Must Understand Error'.
	self assert: envelope faultDetail isEmpty.
	e := envelope faultException.
	self assert: (e isKindOf: SoapSendFault).
	self assert: e faultCode = 'SOAP-ENV:MustUnderstand'.
	self assert: e faultString = 'SOAP Must Understand Error'
!

testHeaderSpecEnvelopes
	#(#getEnvelopeWith2MustHeaders #getEnvelopeWithActorNext #getEnvelopeWithActorNotNext #getEnvelopeWithHeader #getEnvelopeWithMustHeader #getEnvelopeWithMustHeader2 #getEnvelopeWithMustHeaderFalse #getEnvelopeWithMustHeaderNQ) 
		do: 
			[:each | 
			envelope := testEnvelopes perform: each.
			self assert: envelope hasBody.
			self deny: envelope isFault.
			self assert: envelope headerNode notNil.
			self assert: envelope bodyNode notNil.
			self assert: envelope firstBodyBlock qName 
						= ('deleteItem' ns: #'http://schemas.pge.com/ItemService')]
!

testQFaultEnvelope
	envelope := testEnvelopes getQFaultEnvelope.
	self assert: envelope isFault.
	self assert: envelope faultcode = 'SOAP:Client.NoMethod'.
	self assert: envelope faultstring = 'The message did not contain any recognized methods.'
!

testSoapLiteEnvelope
	envelope := testEnvelopes getSoapLiteEnvelope.
	self assert: envelope hasBody.
	self deny: envelope isFault.
	self assert: envelope firstBodyBlock qName = ('echoStruct' ns: #'urn:xmethodsInterop')
! !

!SpSoapInEnvelopeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SpSoapInEnvelopeTest.st,v 1.2 2003/01/20 19:02:15 cg Exp $'
! !
