"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SoapCoreTests subclass:#SptHTTPBindingTests
	instanceVariableNames:'binding'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore-TestCases'
!


!SptHTTPBindingTests methodsFor:'accessing'!

getTestEnv
	| testEnv |
	testEnv := SptEnvironment new.
	testEnv sprayEnvironment: testEnv.
	^testEnv
!

setTestEnv
	binding sprayEnvironment: self getTestEnv
! !

!SptHTTPBindingTests methodsFor:'set up'!

setUp
	super setUp.
	binding := self bindingClass new.
	binding exchange: SptRequestingExchange new.
	self bindingClass addQPTo: binding 
! !

!SptHTTPBindingTests methodsFor:'tests'!

testDefaultTimouts2
	self assert: (binding timeoutSend isKindOf: Integer).
	self assert: (binding timeoutReceive isKindOf: Integer).
	self assert: (binding timeoutConnect isKindOf: Integer).
	self assert: (binding timeoutResolve isKindOf: Integer)
!

testEnsureQuoted
	self assert: (binding ensureQuoted: nil) = '""'.
	self assert: (binding ensureQuoted: '') = '""'.
	self assert: (binding ensureQuoted: 'abc') = '"abc"'.
	self assert: (binding ensureQuoted: '"abc"') = '"abc"'.
!

testHTTPHeaders
	self assert: (binding httpHeaders isKindOf: Collection)
!

testHandleException
	[self error: 'this is an error']
		on: Error
		do: [ :e | binding handleException: e ].
        
	self assert: ('*this is an error*' match: binding exchange failedString).
	self assert: binding exchange isFailState.
	self deny: binding exchange isSuccessState.
        
!

testHostHeaderValue
	#('http://www.foo.com' 'http://www.foo.com/' 'http://www.foo.com/a' 'http://www.foo.com/a/') do: [ :each |
		self assert: (binding immediateDestination: each) hostHeaderValue= 'www.foo.com'].
	#('http://www.foo.com:1234' 'http://www.foo.com:1234/' 
'http://www.foo.com:1234/a' 'http://www.foo.com:1234/a/') do: [ :each |
		self assert: (binding immediateDestination: each) hostHeaderValue= 'www.foo.com:1234'].

!

testSoapAction
	self assert: binding soapAction = '""'.
	binding atQP: 'action:SOAPActionURI' put: 'MySoapAction'.
	self assert: binding soapAction = '"MySoapAction"'.
	binding atQP: 'action:SOAPActionURI' put: ''.
	self assert: binding soapAction = '""'.
	binding atQP: 'action:SOAPActionURI' put: nil.
	self assert: binding soapAction = '""'
!

testUNPW
	self assert: binding userName isNil.
	self assert: binding password isNil.
!

testWaitErrorDescription
	self assert: (binding waitErrorDescription isKindOf: String).
! !

!SptHTTPBindingTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SptHTTPBindingTests.st,v 1.2 2003/01/20 19:02:15 cg Exp $'
! !
