"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SptHTTPRequestTest
	instanceVariableNames:'request sentClose'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore-TestCases'
!


!SptHTTPRequestTest methodsFor:'accessing'!

cancel
	sentClose := true
!

close
	sentClose := true
!

openGetFoo1234Bar
	request openGetTo: 'http://www.foo.org:1234/bar'
!

openGetFooBar
	request openGetTo: 'http://www.foo.org/bar'
!

setNewRequest
	request := SptHTTPRequest new
!

setProxy
	request proxySettings: (SptHTTPProxySettings new
		useProxy: true;
		proxyHost: 'aHost';
		proxyPort: 4321;
		yourself)
! !

!SptHTTPRequestTest methodsFor:'asserting'!

assertHeader: aHeaderLine isKey: aKeyString value: aValueString 
	self setNewRequest.
	request addResponseHeaderLine: aHeaderLine.
	self assert: (request responseHeaders at: aKeyString) = aValueString
!

assertIsFoo1234
	self assert: request host = 'www.foo.org'.
	self assert: request port = 1234
!

assertIsFoo80
	self assert: request host = 'www.foo.org'.
	self assert: request port = 80
! !

!SptHTTPRequestTest methodsFor:'set up'!

setUp
	super setUp.
	self setNewRequest.
	sentClose := false
! !

!SptHTTPRequestTest methodsFor:'tests'!

testAddResponseHeaderLine
	self assert: request responseHeaders isEmpty.
	self assertHeader: 'Content-Length: 456' isKey: 'Content-Length' value: '456'.
	self assertHeader: 'Content-Length:  456' isKey: 'Content-Length' value: '456'.
	self assertHeader: 'Content-Length: 456 ' isKey: 'Content-Length' value: '456'.
	self assertHeader: 'CONTENT-LENGTH: 456' isKey: 'Content-Length' value: '456'.





!

testHasResponseContentLength
	self deny: request hasResponseContentLength.
	request addResponseHeaderLine: 'Content-Length: 456'.
	self assert: request hasResponseContentLength.
	self setNewRequest.
	self deny: request hasResponseContentLength.
	request addResponseHeaderLine: 'CONTENT-LENGTH: 456'.
	self assert: request hasResponseContentLength.



!

testHost
	request url: 'http://www.foo.org'.
	self assertIsFoo80.
	self assert: request urlPath = '/'.
	request url: 'http://www.foo.org/'.
	self assertIsFoo80.
	self assert: request urlPath = '/'.
	request url: 'http://www.foo.org/bar'.
	self assertIsFoo80.
	self assert: request urlPath = '/bar'.
	request url: 'www.foo.org/bar'.
	self should: [request host] raise: Error
!

testIsRedirect
	request statusLine:  'HTTP/1.1 301 Moved Permanently'.
	self deny: request isRedirect.
	request addResponseHeaderLine: 'Location: http://www.foo.org/baz'.
	self assert: request isRedirect.
	request statusLine:  'HTTP/1.1 200 OK'.
	self deny: request isRedirect.
!

testIsRedirectCase
	request statusLine:  'HTTP/1.1 301 Moved Permanently'.
	request addResponseHeaderLine: 'loCatIon: http://www.foo.org/baz'.
	self assert: request isRedirect.
!

testNextLineFrom
	self assert: (request nextLineFrom: ('abc' , String lineDelimiter) asByteArray readStream) = 'abc'.
	self assert: (request nextLineFrom: (String lineDelimiter) asByteArray readStream) isEmpty.




!

testPort
	request url: 'http://www.foo.org:1234'.
	self assertIsFoo1234.
	self assert: request urlPath = '/'.
	request url: 'http://www.foo.org:1234/'.
	self assertIsFoo1234.
	self assert: request urlPath = '/'.
	request url: 'http://www.foo.org:1234/bar'.
	self assertIsFoo1234.
	self assert: request urlPath = '/bar'.
	request url: 'http://www.foo.org:1234/bar?baz'.
	self assertIsFoo1234.
	self assert: request urlPath = '/bar?baz'.
	request url: 'www.foo.org:1234/bar'.
	self should: [request port] raise: Error
!

testPortWithProxy
	self setProxy.
	request url: 'http://www.foo.org:1234'.
	self assert: request host = 'aHost'.
	self assert: request port = 4321.
!

testPrepareRedirectLoop
	| coll |
	coll :=OrderedCollection new.
	self openGetFooBar.
	coll add: request url.
	request statusLine:  'HTTP/1.1 301'.
	request addResponseHeaderLine: 'Location: http://www.foo.org/bar'.
	request responseBody: 'Hi Wrong'.
	self should: [request prepareRedirect: coll] raise: Error.
	
!

testPrepareRedirectNone
	| coll |
	coll :=OrderedCollection new.
	self openGetFooBar.
	request statusLine:  'HTTP/1.1 200'.
	request responseBody: 'Hi'.
	self deny: (request prepareRedirect: coll).
	self assert: request statusLine =  'HTTP/1.1 200'.
	self assert: request responseBody = 'Hi'.

	
!

testPrepareRedirectOne
	| coll |
	coll :=OrderedCollection new.
	self openGetFooBar.
	coll add: request url.
	request statusLine:  'HTTP/1.1 301'.
	request addResponseHeaderLine: 'Location: http://www.foo.org/baz'.
	request responseBody: 'Hi Wrong'.
	self assert: (request prepareRedirect: coll).
	self assert: request statusLine = 'HTTP/1.1 301'.
	self assert: request responseBody isNil.
	self assert: request responseHeaders isEmpty.

	
!

testPutHeaderOn
	| stream |
	stream := String writeStream.
	request putHeader: 'abc' -> 'def' on: stream.
	self assert: stream contents =
'abc: def
'.
	stream := String writeStream.
	request putHeader: 'abc' -> 123 on: stream.
	self assert: stream contents =
'abc: 123
'.





!

testRequestStringWith
	self openGetFooBar.
	self assert: (request requestStringWith: 'abc') 
				= 
'GET /bar HTTP/1.0
Host: www.foo.org
User-Agent: Spray/0.55
Content-Length: 3

'
!

testRequestStringWith1234
	self openGetFoo1234Bar.
	self assert: (request requestStringWith: 'abc') 
				= 
'GET /bar HTTP/1.0
Host: www.foo.org:1234
User-Agent: Spray/0.55
Content-Length: 3

'
!

testRequestStringWithProxy
	self setProxy.
	self openGetFooBar.
	self assert: (request requestStringWith: 'abc') 
				= 
'GET http://www.foo.org/bar HTTP/1.0
Host: www.foo.org
User-Agent: Spray/0.55
Content-Length: 3

'
!

testResponseContentLength
	request addResponseHeaderLine: 'Content-Length: 456'.
	self assert: request responseContentLength = 456.



!

testSendGetRequest
	| stream contents |
	stream := String writeStream.
	self openGetFooBar.
	request sendRequestOn: stream with: ''.
	contents := stream contents.
	self assert: (contents beginsWith: 'GET /bar HTTP/1.0').
	self deny: ('*Content-Length:*' match: contents).
!

testSendGetRequestEntity
	| stream contents |
	stream := String writeStream.
	self openGetFooBar.
	request sendRequestOn: stream with: 'abc123'.
	contents := stream contents.
	self assert: (contents beginsWith: 'GET /bar HTTP/1.0').
	self assert: ('*abc123*' match: contents).
	self assert: ('*Content-Length: 6*' match: contents).
!

testSendPostRequestEntity
	| stream contents |
	stream := String writeStream.
	request openPostTo: 'http://www.foo.org/bar'.
	request sendRequestOn: stream with: 'abc123'.
	contents := stream contents.
	self assert: (contents beginsWith: 'POST /bar HTTP/1.0').
	self assert: ('*abc123*' match: contents).
	self assert: ('*Content-Length: 6*' match: contents).
!

testStatus
	request statusLine: 'HTTP/1.1 500 NotGood'.
	self assert: request status = 500.
	self assert: request statusText = 'NotGood'.



!

testTimeoutState
	self assert: request timeoutState = #off.
	request checkTimeout.
	self assert: request timeoutState = #off.
	request timeoutState: #on.
	self assert: request timeoutState = #on.
	request checkTimeout.
	self assert: request timeoutState = #wait.
!

testUseProxy
	self deny: request useProxy.
	request proxySettings: SptHTTPProxySettings new.
	self deny: request useProxy.
	request proxySettings: (SptHTTPProxySettings new useProxy: true;yourself).
	self assert: request useProxy.
!

testWaitNoTiming
	request timeout: nil.
	request deferredSend: [100 factorial] deferredValue.
	self assert: request waitOnReady = 100 factorial.
	self deny: sentClose.
!

testWaitTiming
	request socket: self.
	request timeout: 200.
	request timeoutState: #off.
	request deferredSend: [(Delay forMilliseconds: 600) wait. 1] deferredValue.
	self assert: request waitOnReady = 1.
	self deny: sentClose.

	request timeoutState: #on.
	request deferredSend: [(Delay forMilliseconds: 600) wait. 1] deferredValue.
	request waitOnReady.
	self assert: sentClose.


!

testWaitTimingEarly
	request socket: self.
	request timeout: 200.
	request timeoutState: #on.
	request deferredSend: [1] deferredValue.
	self assert: request waitOnReady = 1.
	self deny: sentClose.


! !

!SptHTTPRequestTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SptHTTPRequestTest.st,v 1.1 2003/02/02 16:35:49 cg Exp $'
! !
