"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SoapCoreTests subclass:#SptRequestingExchangeTest
	instanceVariableNames:'exchange'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore-TestCases'
!


!SptRequestingExchangeTest methodsFor:'accessing'!

getSampleExchange
	^SptRequestingExchange out: self getSampleOut to: 'http://www.dolphinharbor.org/services/interop'
!

getSampleOut
	^SpSprayTestEnvelopes new getEnvelopeSpec1
! !

!SptRequestingExchangeTest methodsFor:'asserting'!

assertDHDestination
	self assert: exchange immediateDestination = 'http://www.dolphinharbor.org/services/interop'.
	self assert: (exchange atQP: 'transport:ImmediateDestination') 
				= 'http://www.dolphinharbor.org/services/interop'
!

assertInitialState
	self assert: (exchange atQP: 'single:Role') = 'RequestingSOAPNode'.
	self assert: (exchange atQP: 'single:State') = 'Requesting'.
	self assert: (exchange atQP: 'transport:FailureReason') = 'None'
! !

!SptRequestingExchangeTest methodsFor:'tests'!

testAssertSuccessState
	exchange := self getSampleExchange.
	exchange beSuccessState.
	exchange assertSuccessState.
	exchange beFailState.
	self should: [exchange assertSuccessState ] raise: Error.
!

testAssertSuccessState2
	exchange := self getSampleExchange.
	self should: [exchange assertSuccessState ] raise: Error.
!

testConstruction
	exchange := self getSampleExchange.
	self assertInitialState.
	self assertDHDestination
!

testFailedString
	exchange := self getSampleExchange.
	self assert: (exchange failedString isKindOf: String).
	exchange beFailState.
	self assert: (exchange failedString isKindOf: String).
!

testImmediateDestination
	exchange := SptRequestingExchange new.
	exchange immediateDestination: 'http://www.dolphinharbor.org/services/interop'.
	self assertDHDestination
!

testInXmlOrEmpty
	exchange := SptRequestingExchange new.
	self assert: exchange inXmlOrEmpty = ''.
	exchange := self getSampleExchange.
	self assert: exchange inXmlOrEmpty = ''.
	exchange in: self getSampleOut asInEnvelope.
	self assert: exchange inXmlOrEmpty notEmpty
!

testInitialize
	exchange := SptRequestingExchange new.
	self assertInitialState
!

testOutXmlOrEmpty
	exchange := SptRequestingExchange new.
	self assert: exchange outXmlOrEmpty = ''.
	exchange := self getSampleExchange.
	self assert: exchange outXmlOrEmpty notEmpty
!

testSignalIfFault
	exchange := self getSampleExchange.
	exchange in: self getSampleOut asInEnvelope.
	exchange assertNotFault.
	exchange in: (SpSprayTestEnvelopes new getFaultEnvelopeSpec10).
	self should: [exchange assertNotFault] raise: SoapSendFault .
!

testSoapAction
	exchange := self getSampleExchange.
	self assert: exchange soapAction = '""'.
	exchange soapAction: 'MySoapAction'.
	self assert: exchange soapAction = 'MySoapAction'.
! !

!SptRequestingExchangeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/tests/SptRequestingExchangeTest.st,v 1.2 2003/01/20 19:02:15 cg Exp $'
! !
