"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapInteropXMethods
	instanceVariableNames:'client test idSelector exchange answer'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!

SoapInteropXMethods comment:'This class is used to test send operations to a number of the XMethods listed services.

The methods are named after the XMethods assigned "id". See http://www.XMethods.net.

See #example

Instance Variables:
	client		<SprayClient>
	test		<SoapInteropXMethodResult>
	idSelector		<Symbol>
	soapSend	<SpraySoapSend>
	answer		<Object>

'
!


!SoapInteropXMethods class methodsFor:'public'!

getAllSOAPServices
        | client soapSend |
        client := (SprayWSDLService onUrl: 'http://www.xmethods.net/wsdl/query.wsdl') 
                                createClient.
        client beLongTimeouts.
        soapSend := client send: 'getAllServiceSummaries'.
        [soapSend value] on: XeTypeMismatch do: [:e | ].
        ^soapSend extractResuming
!

icon
	^SWEnvironment spIconNamed: 'pass.ico'
! !

!SoapInteropXMethods class methodsFor:'spray examples'!

example
	"This class has two uses. The main use is as part of a batch testing of all listed services. The second is to be an example of using a webservice."

	"Details about the service id2 can be found at: http://www.xmethods.net/ve2/ViewListing.po?serviceid=2"

	| xMethods |
	xMethods := SoapInteropXMethods new.
	"The following method is designed to be used in batch testing. To see what is happening, 'debugIt'"
	xMethods id2	"Debug it"
! !

!SoapInteropXMethods methodsFor:'private'!

assertExampleValid: aUrl
	test wsdlURL trimBlanks = aUrl trimBlanks ifFalse: [self halt].
	self assert: [test wsdlURL trimBlanks = aUrl trimBlanks]
!

performSendTest
	"Private - used by batch tests"

	^self perform: idSelector
!

runSendTest
	"Private - used by batch tests"

	[self performSendTest] 
		on: SoapTransportError
		do: [:e | test sendTransportError: e]
		on: Error
		do: [:e | test sendError: e]
!

setPass: aBool
	"Private - used by batch tests"

	test notNil ifTrue: [test sendResult: aBool]
!

test: aSoapInteropXMethodTest service: aSpSprayWSDLService
	"Private - used by batch tests"

	(self respondsTo: aSoapInteropXMethodTest idSelector) ifFalse: [^nil].
	client := aSpSprayWSDLService createClient.
	test := aSoapInteropXMethodTest.
	idSelector := aSoapInteropXMethodTest idSelector.
	self runSendTest
! !

!SoapInteropXMethods methodsFor:'public'!

client
	^client
!

createClientOn: aUrl
        "If we are part of batch testing, the client will already have been set into me after doing some WSDL tests"

        client notNil ifTrue: [^self assertExampleValid: aUrl].
        client := (SprayWSDLService onUrl: aUrl) createClient
!

id130
        self createClientOn: 'http://www.xmlme.com/WSCustNews.asmx?WSDL'.
        exchange := self client send: 'GetCustomNews' with: 'today'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id130
        "
!

id131
        self createClientOn: 'http://www.xmlme.com/WSDailyNet.asmx?WSDL'.
        exchange := self client send: 'GetDotnetDailyFact' withArguments: #().
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id131
        "
!

id132
        self createClientOn: 'http://www.xmlme.com/WSDailyXml.asmx?WSDL'.
        exchange := self client send: 'GetXmlDailyFact' withArguments: #().
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id132
        "
!

id133
        self createClientOn: 'http://www.xmlme.com/WSSportingGoods.asmx?WSDL'.
        exchange := self client send: 'FindSportingGoods' with: 'ski'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id133
        "
!

id134
        self createClientOn: 'http://www.xmlme.com/WSVideoGames.asmx?WSDL'.
        exchange := self client send: 'FindVideoGames' with: 'wing'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id134
        "
!

id135
        self createClientOn: 'http://www.xmlme.com/WSElectronics.asmx?WSDL'.
        exchange := self client send: 'FindElectronicProducts' with: 'tv'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id134
        "
! !

!SoapInteropXMethods methodsFor:'public-dead services'!

id1004
        self 
                createClientOn: 'http://www.xml-webservices.net/services/conversions/euro_convert/euro_conver.asmx?WSDL'.
        exchange := self client send: 'CurrenciesToDataSet' withArguments: #().
        answer := exchange value.
        exchange := self client send: 'FromEurosToPortugalEscudos' with: 10.
        answer := exchange value.
        self setPass: (answer isKindOf: ScaledDecimal).
        ^answer

        "
         SoapInteropXMethods new id1004
        "
!

id1006
        self 
                createClientOn: 'http://www.richsolutions.com/RichPayments/RichCardValidator.asmx?WSDL'.
        exchange := self client send: 'GetCardType' with: '1234123412341234'.
        answer := exchange value.
        self setPass: answer = 'Unknown'.
        ^answer

        "
         SoapInteropXMethods new id1006
        "
!

id1011
        self 
                createClientOn: 'http://www.xml-webservices.net/services/directory/profile/user_account.asmx?WSDL'.
        exchange := self client 
                                send: 'Login'
                                with: 'test'
                                with: 'test'.
        answer := exchange value.
        self setPass: answer = 'False'.
        ^answer

        "
         SoapInteropXMethods new id1011
        "
!

id1013
	self 
		createClientOn: 'http://www.xml-webservices.net/services/conversions/unit_convert.asmx?WSDL'.
	exchange := self client 
				send: 'ConvetDistances'
				with: 10
				with: 'm'
				with: 'feet'.
	answer := exchange value.
	self setPass: answer = 32.8084.
	^answer
!

id1014
	self createClientOn: 'http://www.xml-webservices.net/services/maths/calculator.asmx?WSDL'.
	exchange := self client 
				send: 'Multiply'
				with: 2
				with: 3.
	answer := exchange value.
	self setPass: answer = 6.
	^answer
!

id109
        | c1 c2 c3 |
        self createClientOn: 'http://live.capescience.com/wsdl/PolyCalcApp.wsdl'.
        c1 := (XeQStruct new)
                                at: #x put: 0.0;
                                at: #y put: 0.0;
                                yourself.
        c2 := (XeQStruct new)
                                at: #x put: 0.0;
                                at: #y put: 2.0;
                                yourself.
        c3 := (XeQStruct new)
                                at: #x put: 2.0;
                                at: #y put: 2.0;
                                yourself.
        exchange := self client send: 'calculate'
                                with: (Array 
                                                with: c1
                                                with: c2
                                                with: c3).
        answer := exchange value.
        self setPass: answer = 2.0.
        ^answer

        "
         SoapInteropXMethods new id109
        "
!

id128
        self createClientOn: 'http://services.pagedownweb.com/ZipCodes.asmx?WSDL'.
        exchange := self client 
                                send: 'ZipToState'
                                with: '98052'
                                with: 'WA'.
        answer := exchange value.
        exchange := self client 
                                send: 'rtnZipDS'
                                with: '98052'
                                with: 'WA'.
        answer := exchange value.
        self 
                setPass: ((answer isKindOf: YAXO::XeNode) and: [answer qName = (XeQName xsd2001: 'schema')]).
        ^answer

        "
         SoapInteropXMethods new id128
        "
!

id173
        self 
                createClientOn: 'http://beta2.eraserver.net/webservices/mxchecker/mxchecker.asmx?WSDL'.
        exchange := self client 
                                send: 'CheckEmail'
                                with: 0
                                with: 'swaring@ozemail.com.au'.
        answer := exchange value.
        self setPass: answer = 'OK'.
        ^answer

        "
         SoapInteropXMethods new id173
        "
!

id185
        self 
                createClientOn: 'http://webservices.eraserver.net/zipcoderesolver/zipcoderesolver.asmx?WSDL'.
        exchange := self client send: 'FullZipCode'
                                withArguments: #('9999' '123 test' 'foo' 'good').
        answer := exchange value.
        self setPass: answer = '00000-0000'.
        ^answer

        "
         SoapInteropXMethods new id185
        "
!

id192
        self 
                createClientOn: 'http://www.PerfectXML.NET/WebServices/SalesRankNPrice/BookService.asmx?WSDL'.
        exchange := self client send: 'GetAmazonPrice' with: '0201737930'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id192
        "
!

id19611
        self 
                createClientOn: 'http://www.aspxpressway.com/maincontent/webservices/piglatin.asmx?wsdl'.
        exchange := self client send: 'toPigLatin' with: 'Dolphin Smalltalk'.
        answer := exchange value.
        self setPass: answer trimBlanks = 'Olphinday Alltalksmay'.
        ^answer

        "
         SoapInteropXMethods new id19611
        "
!

id19675
        self createClientOn: 'http://services.develop.co.za/GetLocalTime.asmx?WSDL'.
        exchange := self client send: 'GetTime' withArguments: #().
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id19675
        "
!

id198
        self 
                createClientOn: 'http://erwin.dstc.edu.au/GUIDDictionaryService/GUIDDictionary.asmx?wsdl'.
        exchange := self client send: 'GetGUID' with: 'testing'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id198
        "
!

id218
        self createClientOn: 'http://upload.eraserver.net/circle24/autoloan.asmx?wsdl'.
        exchange := self client 
                                send: 'Calculate'
                                with: 12
                                with: 5
                                with: 1000.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id218
        "
!

id220
        "self halt."

        | args |
        self 
                createClientOn: 'http://www.PerfectXML.net/WebServices/MusicTeachers/MusicTeachers.asmx?wsdl'.
        args := Array new: 6.
        args
                at: 1 put: '38603';
                at: 2 put: '';
                at: 3 put: '';
                at: 4 put: '';
                at: 5 put: 0;
                at: 6 put: 0.
        exchange := self client send: 'FindMusicTeachers' withArguments: args.
        answer := exchange value.
        self setPass: true.
        ^answer

        "
         SoapInteropXMethods new id220
        "
!

id221
        self createClientOn: 'http://www.vbws.com/services/weatherretriever.asmx?WSDL'.
        exchange := self client send: 'GetWeather' with: '38603'.
        answer := exchange value.
        self setPass: true.
        ^answer

        "
         SoapInteropXMethods new id221
        "
!

id223
        self createClientOn: 'http://soap.fmui.de/chemistry.wsdl'.
        exchange := self client send: 'getElementByNumber' with: 7.
        answer := exchange value.
        self setPass: ((answer at: #name) = 'Nitrogen' and: [(answer at: #meltingPoint) = 63.29]).
        ^answer

        "
         SoapInteropXMethods new id223
        "
!

id226
        self createClientOn: 'http://www.forthlink.gr/TakisLib/TakisActiveLib.WSDL'.
        exchange := self client send: 'GetAutoHellasRecordSetinXml'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id226
        "
!

id229
        self 
                createClientOn: 'http://www.jusufdarmawan.com/wsprimegenerator.exe/wsdl/IPrimeGenerator'.
        exchange := self client 
                                send: 'primegenerator'
                                with: '1'
                                with: '10'.
        answer := exchange value.
        self setPass: answer = '2,3,5,7'.
        ^answer

        "
         SoapInteropXMethods new id229
        "
!

id242
        self createClientOn: 'http://www.soapengine.com/lucin/soapenginex/smsx.asmx?wsdl'.
        exchange := self client 
                                send: 'GetServerTime'
                                with: 'test'
                                with: 'test'.
        answer := exchange value.
        self setPass: true.
        ^answer

        "
         SoapInteropXMethods new id242
        "
!

id24680
        self 
                createClientOn: ' http://www.xml-webservices.net/services/time_server/clock.asmx?WSDL'.
        exchange := self client send: 'GetUTC_Time' withArguments: #().
        answer := exchange value.
        self setPass: (answer asNative isKindOf: TimeStamp).
        ^answer

        "
         SoapInteropXMethods new id24680
        "
!

id24682
        self 
                createClientOn: 'http://www.xml-webservices.net/services/entretainment/joke_server.asmx?WSDL'.
        exchange := self client send: 'JokeToString' with: 'ES'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id24682
        "
!

id251
        self createClientOn: 'http://aspnet.lamaan.com/webservices/QuranVerse.asmx?WSDL'.
        exchange := self client 
                                send: 'GetQuranVerseEnglish'
                                with: 10
                                with: 1.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id251
        "
!

id254
        self 
                createClientOn: 'http://upload.eraserver.net/MOUNALISA/adictionary/arabicdictionary.asmx?WSDL'.
        exchange := self client send: 'GetEAMeaning' with: 'dolphin'.
        answer := exchange value.
        self setPass: (answer atLocalName: #Trans) first = 'dolphin'.
        ^answer

        "
         SoapInteropXMethods new id254
        "
!

id259
        | pass |
        self createClientOn: 'http://hosting.msugs.ch/aravindc/BookPriceComparison.asmx?wsdl'.
        exchange := self client send: 'CompareBookPrices' with: '0201737930'.
        answer := exchange value.
        pass := true.
        pass := pass and: [(answer atLocalName: 'Book') size > 1].
        pass := pass and: [(answer atLocalName: 'Book') first attributes notEmpty].
        self setPass: pass.
        ^answer

        "
         SoapInteropXMethods new id259
        "
!

id265
        self createClientOn: 'http://HOSTING.MSUGS.CH/cheeso9/zips/ZipService.asmx?WSDL'.
        exchange := self client send: 'ZipToCityAndState' with: '12345'.
        answer := exchange value.
        self setPass: answer = #('SCHENECTADY NY').
        ^answer

        "
         SoapInteropXMethods new id265
        "
!

id34923
        self createClientOn: 'http://www.alethea.net/webservices/LocalTime.asmx?WSDL'.
        exchange := self client send: 'LocalTimeByZipCode' with: '38603'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id34923
        "
!

id34945
        self createClientOn: 'http://www.dev1.eraserver.net/REFLECTIONIT/Html2xml.asmx?WSDL'.
        exchange := self client send: 'HtmlString2XmlString' with: '<b>hi<br></b>'.
        answer := exchange value.
        self setPass: ('*<b>hi<br /></b>*' match: answer).
        ^answer

        "
         SoapInteropXMethods new id34945
        "
!

id42115
        self createClientOn: 'http://www.esynaps.com/WebServices/eSynapsSearch.asmx?WSDL'.
        exchange := self client 
                                send: 'SearchESynaps'
                                with: 'SOAP'
                                with: 10.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id42115
        "
!

id47216
        | pass |
        self 
                createClientOn: 'http://hosting001.vs.k2unisys.net/Weather/PDCWebService/WeatherServices.asmx?WSDL'.
        exchange := self client send: 'GetWeather' with: '38603'.
        answer := exchange value.
        pass := true.
        pass := pass & ((answer atLocalName: 'ZipCode') = '38603').
        pass := pass & ((answer atLocalName: 'DayForecast') isKindOf: Array).
        pass := pass & (answer atLocalName: 'DayForecast') notEmpty.
        self setPass: pass.
        ^answer

        "
         SoapInteropXMethods new id47216
        "
!

id47238
        self createClientOn: 'http://www.esynaps.com/WebServices/MsProxy.asmx?WSDL'.
        exchange := self client send: 'MsProxySays'.
        answer := exchange value.
        self setPass: (answer isKindOf: ByteArray).
        ^answer

        "
         SoapInteropXMethods new id47238
        "
!

id7534
        self createClientOn: 'http://www.alethea.net/webservices/ZipCode.asmx?WSDL'.
        exchange := self client 
                                send: 'CityStateToZipCode'
                                with: 'Denver'
                                with: 'Colorado'.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id7534
        "
!

id9348
        self createClientOn: 'http://www.esynaps.com/WebServices/DailyDiblert.asmx?WSDL'.
        exchange := self client send: 'DailyDilbertImage' withArguments: #().
        answer := exchange value.
        self setPass: ((answer isKindOf: ByteArray) and: [answer notEmpty]).
        ^answer

        "
         SoapInteropXMethods new id9348
        "
!

id9349
        self createClientOn: 'http://www.esynaps.com/WebServices/WhoIsService.asmx?WSDL'.
        exchange := self client send: 'WhoIs' with: 'www.dolphinharbor.org'.
        answer := exchange value.
        self setPass: answer notEmpty.
        ^answer

        "
         SoapInteropXMethods new id9349
        "
!

id94
        | struct |
        self createClientOn: 'http://services.xmltoday.com/vx_engine/wsdl.vep/translate.wsdl'.
        (struct := XeQStruct new)
                at: #text put: 'Go Smalltalk, Go Dolphin';
                at: #language put: 'Spanish'.
        exchange := self client send: 'getTranslation' with: struct.
        answer := exchange value.
        self setPass: (answer at: #text) trimBlanks = 'Va El Palique, Va Delfín'.
        ^answer

        "
         SoapInteropXMethods new id94
        "
!

id95
        self createClientOn: 'http://services.xmltoday.com/vx_engine/wsdl.vep/realtime.wsdl'.
        exchange := self client send: 'getRTQuote' with: (XeQStruct param: #symbol value: 'YHOO').
        answer := exchange value.
        self setPass: ((answer at: #stock_quote) at: #symbol) = 'YHOO'.
        ^answer

        "
         SoapInteropXMethods new id95
        "
! !

!SoapInteropXMethods methodsFor:'rpc encoded'!

id110
        self createClientOn: 'http://www.compkarori.com/wsdl/discordian.wsdl'.
        exchange := self client 
                                send: 'Discordian'
                                with: 1000
                                with: 10
                                with: 10.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id110
        "
!

id140
        self createClientOn: 'http://www.ebob42.com/cgi-bin/Romulan.exe/wsdl/IRoman'.
        exchange := self client send: 'IntToRoman' with: 42.
        answer := exchange value.
        self setPass: answer = 'XLII'.
        ^answer

        "
         SoapInteropXMethods new id140
        "
!

id141
        self createClientOn: 'http://www.ebob42.com/cgi-bin/NumberToWordsInDutch.exe/wsdl/IDutch'.
        exchange := self client send: 'NumToStr' with: 1234.
        answer := exchange value.
        self setPass: answer = 'eenduizend tweehonderdvierendertig'.
        ^answer

        "
         SoapInteropXMethods new id141
        "
!

id159
        self createClientOn: 'http://ws.xara.com/graphicrender/render3d.wsdl'.
        exchange := self client send: 'GetFonts'.
        answer := exchange value.
        self setPass: (answer includes: 'Arial').
        ^answer

        "
         SoapInteropXMethods new id159
        "
!

id236
        self createClientOn: 'http://ws.xara.com/navbar/navbar.wsdl'.
        exchange := self client send: 'GetFonts'.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id236
        "
!

id32
        self createClientOn: 'http://www.SoapClient.com/xml/SQLDataSoap.WSDL'.
        exchange := self client 
                                send: 'ProcessSRL'
                                with: 'WEB.SRI'
                                with: 'Yahoo'
                                with: 'Smalltalk'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id32
        "
!

id34
        self createClientOn: 'http://www.SoapClient.com/xml/SQLDataSoap.WSDL'.
        exchange := self client 
                                send: 'ProcessSRL'
                                with: 'WHOIS.SRI'
                                with: 'whois'
                                with: 'dolphinharbor.org'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id34
        "
!

id37
        self createClientOn: 'http://www.SoapClient.com/xml/SQLDataSoap.wsdl'.
        exchange := self client 
                                send: 'ProcessSRL'
                                with: 'EDGAR.SRI'
                                with: 'EDGAR'
                                with: 'foo bar'.
        answer := exchange value.
        self setPass: (answer isNil or: [answer isKindOf: String]).
        ^answer

        "
         SoapInteropXMethods new id37
        "
!

id45
        self createClientOn: 'http://www.SoapClient.com/xml/SQLDataSoap.WSDL'.
        exchange := self client 
                                send: 'ProcessSRL'
                                with: 'NEWS.SRI'
                                with: 'yahoo'
                                with: 'today'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id45
        "
!

id45b
        self createClientOn: 'http://www.soapclient.com/interop/SQLDataInterop.wsdl'.
        exchange := self client 
                                send: 'echoInteger'
                                with: 12345.
        answer := exchange value.
        self setPass: (answer isKindOf: Integer).
        ^answer

        "
         SoapInteropXMethods new id45b   
        "
!

id60
        self createClientOn: 'http://www.alanbushtrust.org.uk/soap/compositions.wsdl'.
        exchange := self client send: 'GetCategories'.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id60
        "
! !

!SoapInteropXMethods methodsFor:'rpc encoded - dead services'!

id1
        self createClientOn: 'http://www.xmethods.net/sd/2001/EBayWatcherService.wsdl'.
        exchange := self client send: 'getCurrentPrice' with: '414173888'.
        answer := exchange value.
        self setPass: answer = -1.0.
        ^answer

        "
         SoapInteropXMethods new id1
        "
!

id1001
        self createClientOn: 'http://213.23.125.181:8080/christmas.wsdl'.
        exchange := self client 
                                send: 'ChristmasTree'
                                with: 223.4
                                with: 112.56
                                with: 12.5.
        answer := exchange value.
        self setPass: answer = 223.
        ^answer

        "
         SoapInteropXMethods new id1001
        "
!

id1007
        "self halt."

        self createClientOn: 'http://www.webappcabaret.com/sk/pub/sinosoap.wsdl'.
        exchange := self client send: 'jfortune' with: 'hello'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).        "?"
        ^answer

        "
         SoapInteropXMethods new id1007
        "
!

id1016
        self createClientOn: 'http://www.mybubble.com:8080/mybubbleEntServer/MBSoapServices.wsdl'.
        exchange := self client 
                                send: 'getServiceResponsePublic'
                                with: 'Synonym'
                                with: 'permeate'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id1016
        "
!

id111
        self createClientOn: 'http://www.shinkatech.com/interop/NSLookup/NSLookup.wsdl'.
        exchange := self client send: 'translate' with: 'dolphinharbor.org'.
        answer := exchange value.
        answer at: #IP.
        self setPass: (answer at: #Hostname) = 'dolphinharbor.org'.
        ^answer

        "
         SoapInteropXMethods new id111
        "
!

id129
        self createClientOn: 'http://live.capescience.com/wsdl/AirportWeather.wsdl'.
        exchange := self client send: 'getSummary' with: 'EGLL'.
        answer := exchange value.
        self setPass: (answer at: #location) = 'London / Heathrow Airport, United Kingdom'.
        ^answer

        "
         SoapInteropXMethods new id129
        "
!

id13437
        self createClientOn: 'http://www.geographynetwork.com/services/v1/PlaceFinder.wsdl'.
        exchange := self client send: 'findPlace' with: 'Sydney'.
        answer := exchange value.
        self setPass: ((answer at: #candidates) isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id13437
        "
!

id14
        self createClientOn: 'http://www.xmethods.net/sd/2001/BabelFishService.wsdl'.
        exchange := self client 
                                send: 'BabelFish'
                                with: 'en_it'
                                with: 'Go Smalltalk, Go Dolphin'.
        answer := exchange value.
        self setPass: answer trimBlanks = 'Vanno Le Chiacchiere, Vanno Delfino'.
        ^answer

        "
         SoapInteropXMethods new id14
        "
!

id147
        self createClientOn: 'http://www.ebob42.com/cgi-bin/DrBobsClinic.exe/wsdl/IHeadline'.
        exchange := self client send: 'SOAPNews' with: 5.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id147
        "
!

id15487
        self createClientOn: 'http://www.mybubble.com:8080/mybubbleEntServer/MBSoapServices.wsdl'.
        exchange := self client 
                                send: 'getServiceResponsePublic'
                                with: 'StockQuote'
                                with: 'IBM'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id15487
        "
!

id15492
        self createClientOn: 'http://www.mybubble.com:8080/mybubbleEntServer/MBSoapServices.wsdl'.
        exchange := self client 
                                send: 'getServiceResponsePublic'
                                with: 'StockProfile'
                                with: 'IBM'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id15492
        "
!

id156
        self 
                createClientOn: 'http://www.allesta.net:51110/webservices/wsdl/YahooUserPingService.xml'.
        exchange := self client send: 'isuseronline' with: 'notauser'.
        answer := exchange value.
        self setPass: answer not.
        ^answer

        "
         SoapInteropXMethods new id156
        "
!

id186
        self createClientOn: 'http://www.iemfamily.com/iemfamily/CreditCard.asmx?WSDL'.
        exchange := self client send: 'VerifyCreditCard' with: '4111111111111111'.
        answer := exchange value.
        self setPass: answer = 'Valid Visa'.    "!!?"
        ^answer

        "
         SoapInteropXMethods new id186
        "
!

id197
        self createClientOn: 'http://xmlrad.com/WSFindMP3Bin/WSFindMP3.dll/WSDL'.
        exchange := self client 
                                send: 'FindMP3'
                                with: 'Cartoon'
                                with: 10.
        answer := exchange value.
        self setPass: (answer isKindOf: Collection).
        ^answer

        "
         SoapInteropXMethods new id197
        "
!

id2
        self createClientOn: 'http://services.xmethods.net/soap/urn:xmethods-delayed-quotes.wsdl'.
        exchange := self client send: 'getQuote' with: 'YHOO'.
        answer := exchange value.
        self setPass: (answer isKindOf: Float).
        ^answer

        "
         SoapInteropXMethods new id2
        "
!

id201
        self createClientOn: 'http://www.ebi.ac.uk/xembl/XEMBL.wsdl'.
        exchange := self client 
                                send: 'getNucSeq'
                                with: 'Bsml'
                                with: 'AC000057'.
        answer := exchange value.
        self setPass: ((answer isKindOf: String) and: [answer notEmpty]).
        ^answer

        "
         SoapInteropXMethods new id201
        "
!

id207
        self createClientOn: 'http://213.23.125.181:8080/statistics.wsdl'.
        exchange := self client send: 'isPrimeNumber' with: 13.
        self setPass: exchange value.
        ^answer

        "
         SoapInteropXMethods new id207
        "
!

id235
        self 
                createClientOn: 'http://www.abecas.com/webservices/wordservice.dll/wsdl/IWordHandler'.
        exchange := self client send: 'DefineWord2' with: 'good'.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id235
        "
!

id24707
        self createClientOn: 'http://www.durrios.com/Finance.wsdl'.
        exchange := self client send: 'stockquote_open' with: 'IBM'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id24707
        "
!

id255
        self createClientOn: 'http://www.foxcentral.net/foxcentral.wsdl'.
        exchange := self client send: 'GetProviders'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id255
        "
!

id25703
        self createClientOn: 'http://www.xmethods.net/wsdl/query.wsdl'.
        exchange := self client send: 'getAllServiceSummaries'.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id25703
        "
!

id268
        self createClientOn: 'http://www.drbob42.co.uk/cgi-bin/Euro42/wsdl/IEuro'.
        exchange := self client 
                                send: 'FromEuro'
                                with: 'DEM'
                                with: 123.45.
        answer := exchange value.
        self setPass: (answer isKindOf: Float).
        ^answer

        "
         SoapInteropXMethods new id268
        "
!

id35
        self createClientOn: 'http://www.itfinity.net:8008/soap/Calculator/Calculator.wsdl'.
        exchange := self client send: 'Evaluate' with: '2+2'.
        answer := exchange value.
        self setPass: answer = '4'.
        ^answer

        "
         SoapInteropXMethods new id35
        "
!

id4
        self createClientOn: 'http://www.xmethods.net/sd/2001/CATrafficService.wsdl'.
        exchange := self client send: 'getTraffic' with: '101'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id4
        "
!

id48231
        self createClientOn: 'http://www.tankebolaget.se/scripts/Haddock.exe/wsdl/IHaddock'.
        exchange := self client send: 'GetCurseLanguages'.
        [answer := exchange value] on: XeStrictError do: [:e | e resume].
        exchange := self client send: 'Curse' with: answer first first.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id48231
        "
!

id48232
        self createClientOn: 'http://www.tankebolaget.se/scripts/TBWS.exe/wsdl/IModulusCheck'.
        exchange := self client send: 'CheckModulus' with: '1234'.
        answer := exchange value.
        self setPass: answer not.
        ^answer

        "
         SoapInteropXMethods new id48232
        "
!

id5
        self createClientOn: 'http://www.xmethods.net/sd/2001/CurrencyExchangeService.wsdl'.
        exchange := self client 
                                send: 'getRate'
                                with: 'australia'
                                with: 'usa'.
        answer := exchange value.
        self setPass: (answer isKindOf: Float).
        ^answer

        "
         SoapInteropXMethods new id5
        "
!

id6
        self createClientOn: 'http://services.xmethods.net/soap/urn:xmethods-DomainChecker.wsdl'.
        exchange := self client send: 'checkDomain' with: 'dolphinharbor.org'.
        answer := exchange value.
        self setPass: answer = 'unavailable'.
        ^answer

        "
         SoapInteropXMethods new id6
        "
!

id7
        self createClientOn: 'http://www.xmethods.net/sd/2001/BNQuoteService.wsdl'.
        exchange := self client send: 'getPrice' with: '0201737930'.
        answer := exchange value.
        self setPass: (answer isKindOf: Number).
        ^answer

        "
         SoapInteropXMethods new id7
        "
!

id7275
        self createClientOn: 'http://www.durrios.com/Bills.wsdl'.
        exchange := self client 
                                send: 'restaurant'
                                with: 100
                                with: 10
                                with: 0.
        answer := exchange value.
        self setPass: answer = 10.
        ^answer

        "
         SoapInteropXMethods new id7275
        "
!

id7328
        self createClientOn: 'http://www.mybubble.com:8080/mybubbleEntServer/MBSoapServices.wsdl'.
        exchange := self client 
                                send: 'getServiceResponsePublic'
                                with: 'Meaning'
                                with: 'permeate'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id7328
        "
!

id7346
        self createClientOn: 'http://www.techmetrix.net:8080/wsdl/techservice.wsdl'.
        exchange := self client 
                                send: 'getLatestArticles'
                                with: 'US'
                                with: 3.
        answer := exchange value.
        self setPass: (answer isKindOf: Array).
        ^answer

        "
         SoapInteropXMethods new id7346
        "
!

id7396
        | geoPoint |
        self 
                createClientOn: 'http://inovagis.dcea.fct.unl.pt/services/GIServerWS.exe/wsdl/IGIServer'.
        (geoPoint := XeQStruct new)
                at: #X put: -7;
                at: #Y put: 38;
                at: #SRS put: 'EPSG:4326'.
        exchange := self client 
                                send: 'ConvertCoordEx'
                                with: geoPoint
                                with: 'SHGM'.
        answer := exchange value.
        self setPass: ((answer at: #SRS) = 'SHGM' and: [(answer at: #X) isKindOf: Float]).
        ^answer

        "
         SoapInteropXMethods new id7396
        "
!

id7420
        self createClientOn: 'http://www.pocketsoap.com/services/whois.wsdl'.
        exchange := self client send: 'whois' with: 'www.dolphinharbor.org'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id7420
        "
!

id7508
        self 
                createClientOn: 'http://developerdays.com/cgi-bin/tempconverter.exe/wsdl/ITempConverter'.
        exchange := self client send: 'FtoC' with: 100.
        answer := exchange value.
        self setPass: answer = 37.
        ^answer

        "
         SoapInteropXMethods new id7508
        "
!

id8
        self createClientOn: 'http://www.xmethods.net/sd/2001/TemperatureService.wsdl'.
        exchange := self client send: 'getTemp' with: '94041'.
        answer := exchange value.
        self setPass: (answer isKindOf: Float).
        ^answer

        "
         SoapInteropXMethods new id8
        "
!

id80
        self 
                createClientOn: 'http://www.xmlbus.com:9010/xmlbus/container/Electricity/ElectricityService/ElectricityPort'.
        exchange := self client 
                                send: 'computeWatts'
                                with: 10
                                with: 5.
        answer := exchange value.
        self setPass: (answer isKindOf: Number).
        ^answer

        "
         SoapInteropXMethods new id80
        "
!

id81
        self 
                createClientOn: 'http://www.xmlbus.com:9010/xmlbus/container/Converter/ConverterService/ConverterPort'.
        exchange := self client send: 'mmToInch' with: 300.
        answer := exchange value.
        self setPass: (answer isKindOf: Number).
        ^answer

        "
         SoapInteropXMethods new id81
        "
!

id84
        self createClientOn: 'http://www.agnisoft.com/soap/mssoapmp3search.xml'.
        exchange := self client 
                                send: 'SearchMP3'
                                with: 'any'
                                with: 10.
        answer := exchange value.
        self setPass: (answer isKindOf: Number).
        ^answer

        "
         SoapInteropXMethods new id84
        "
!

id93
        self createClientOn: 'http://www.nickhodges.com/bin/ShakespeareWS.exe/wsdl/IShakespeare'.
        exchange := self client send: 'GetShakespeareInsult'.
        answer := exchange value.
        self setPass: (answer isKindOf: String).
        ^answer

        "
         SoapInteropXMethods new id93
        "
! !

!SoapInteropXMethods class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SOAP__SoapInteropXMethods.st,v 1.2 2010/03/08 23:07:50 cg Exp $'
! !
