"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoXmlUtilTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoXmlUtilTestCase methodsFor:'fixtures'!

compString
	^self normalString, self convString
!

convString
	^'<>''"&'
!

nameapacedSimpleXmlString
	^'<m:Bar a="1" b="2" xmlns:m="http://soapinterop.org/">
		<bar>cont</bar>
	</m:Bar>'
        
!

namespacedSimpleXmlElement
	^(SoXmlUtil parseXml: self nameapacedSimpleXmlString)
!

normalString
	^'SoapOpera is a Squeak SOAP implementaion written by Masashi Umezawa'
!

simpleXmlElement
	^(SoXmlUtil parseXml: self simpleXmlString)
!

simpleXmlElementWithText
	^(SoXmlUtil parseXml: self simpleXmlStringWithText)
!

simpleXmlString
	^'<Foo a="1" b="2">
		<bar>cont</bar>
	</Foo>'
        
!

simpleXmlStringWithText
	^'<Foo a="1" b="2">
		some text...
		<bar>cont</bar>
	</Foo>'
        
! !

!SoXmlUtilTestCase methodsFor:'testing'!

testAttribDict
	| elem dic1 dic2 |
	elem :=self simpleXmlElement.
	dic1 := SoXmlUtil attribDictFrom: elem.
	dic2 := Dictionary new.
	dic2 at: 'a' put: '1'.
	dic2 at: 'b' put: '2'.
	self should:[dic1 = dic2]
        
!

testComplexStringWrite
	| str1 str2 str3 |
	str1 := self compString.
	str2 := SoXmlUtil asXmlText: str1.
	str3 := self normalString, '&lt;&gt;&apos;&quot;&amp;'.
	self shouldnt: [str1 = str2].
	self should:[str2 = str3]
!

testConvStringWrite
	| str1 str2 str3 |
	str1 := self convString.
	str2 := SoXmlUtil asXmlText: str1.
	str3 := '&lt;&gt;&apos;&quot;&amp;'.
	self shouldnt: [str1 = str2].
	self should:[str2 = str3]
        
!

testElementName
	| elem nm |
	elem :=self simpleXmlElement.
	nm := SoXmlUtil elementNameFrom: elem.
	self should:[nm = 'Foo'].
        
        
!

testElementShortName
        
	| elem2 nm2 |
	elem2 := self namespacedSimpleXmlElement.
	nm2 := SoXmlUtil elementShortNameFrom: elem2.
	self should:[nm2 = 'Bar'].
        
!

testElementsWithoutTexts
        
	| elem1 elems1 elem2 elems2 |
	elem1 :=self simpleXmlElement.
	elems1 := SoXmlUtil elementsWithoutTextsFrom: elem1.
	elem2 :=self simpleXmlElementWithText.
	elems2 := SoXmlUtil elementsWithoutTextsFrom: elem2.
	self should: [elems1 size = elems2 size].
	self should: [elems1 printString  = elems2 printString]
        
!

testNormalStringWrite
	| str1 str2 |
	str1 := self normalString.
	str2 := SoXmlUtil asXmlText: str1.
	self should: [str1 = str2]
        
! !

!SoXmlUtilTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoXmlUtilTestCase.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
