"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapCallTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapCallTestCase methodsFor:'defaults'!

soapPort
    ^ 8823
! !

!SoapCallTestCase methodsFor:'fixtures'!

simpleAddService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	^serv
! !

!SoapCallTestCase methodsFor:'testing'!

testAnyArrayHttpCall
	| handlerName call resp |
	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1.0 'two' 3)).

	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1.0 'two' 3)].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testCannnotConnectHttpCall
	| call resp |

	call := (SoapCallEntry tcpHost: 'foobarbazboo' port: self soapPort) newCall.
	call transport: #http.
	call methodName: 'helloWorld'.
        
	resp := call invoke.
        
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SOAP::SoapCannnotConnectToServer
!

testCannnotResolveHttpCall
	| notUsedPort call resp |
	notUsedPort := 9876.
	call := (SoapCallEntry tcpHost: 'localhost' port: notUsedPort) newCall.
	call transport: #http.
	call methodName: 'helloWorld'.
        
	resp := call invoke.
        
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SOAP::SoapCannnotConnectToServer
!

testComplexSTXEncoding
	| call pa1 pa2 env methodElem arg1 arg2 |

	Smalltalk isSmalltalkX ifFalse:[^ self].

	call := SoapCall new.
	call transport: #localRpc.
	call targetObjectURI: 'urn:auauObject'.
	call encoding: #soapSTXEncoding.
	call methodName: 'auau'.
	pa1 := (SoapVariable name: #arg1 value: 1).
	pa2 := (SoapVariable name: #arg2 value: #two) encoding: #soapSTXEncoding.
	call addParameter: pa1.
	call addParameter: pa2.
	env := call buildEnvelope.
        
	self should: [env name = 'env:Envelope'].
	self should: [env body name = 'env:Body'].
	self should: [(env getAttributeNamed: 'env:encodingStyle') = SoapConstants soapSTXEncURI]. 
	methodElem := env body bodyEntries first.
	self should: [ methodElem name = 'm:auau'].
	arg1 := methodElem children at: 1.
	self should:[ arg1 name = 'arg1']. 
	self should:[ (arg1 getAttributeNamed: 'xsi:type') = 'xsd:integer']. 
	self should:[ (arg1 encodingStyleURIs first) = SoapConstants soapSTXEncURI]. 
	self should:[ arg1 value = '1']. 
	arg2 := methodElem children at: 2.
	self should:[ arg2 name = 'arg2']. 
	self should:[ (arg2 getAttributeNamed: 'xsi:type') =  'SmalltalkX-ENC:BinaryObjectStorage']. 
!

testComplexSTXHttpCall
	| handlerName call resp |

	Smalltalk isSmalltalkX ifFalse:[^ self].

	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call encoding: #soapSTXEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem).
        
	resp := call invokeWithValue: #symbol.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #symbol].
        
	resp := call invokeWithValue: Date today.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = Date today].
        
	resp := call invokeWithValue: (OrderedCollection with: #(1 2 #('three' 3) with: Date today)).
	self shouldnt:[resp isFault].
	self should:[resp returnValue = (OrderedCollection with: #(1 2 #('three' 3) with: Date today))].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
!

testComplexSqEncoding
	| call pa1 pa2 env methodElem arg1 arg2 |

	Smalltalk isSqueak ifFalse:[^ self].

	call := SoapCall new.
	call transport: #localRpc.
	call targetObjectURI: 'urn:auauObject'.
	call encoding: #soapSqEncoding.
	call methodName: 'auau'.
	pa1 := (SoapVariable name: #arg1 value: 1).
	pa2 := (SoapVariable name: #arg2 value: #two) encoding: #soapSqEncoding.
	call addParameter: pa1.
	call addParameter: pa2.
	env := call buildEnvelope.
        
	self should: [env name = 'env:Envelope'].
	self should: [env body name = 'env:Body'].
	self should: [(env getAttributeNamed: 'env:encodingStyle') = SoapConstants soapSqEncURI]. 
	methodElem := env body bodyEntries first.
	self should: [ methodElem name = 'm:auau'].
	arg1 := methodElem children at: 1.
	self should:[ arg1 name = 'arg1']. 
	self should:[ (arg1 getAttributeNamed: 'xsi:type') = 'xsd:integer']. 
	self should:[ (arg1 encodingStyleURIs first) = SoapConstants soapSqEncURI]. 
	self should:[ arg1 value = '1']. 
	arg2 := methodElem children at: 2.
	self should:[ arg2 name = 'arg2']. 
	self should:[ (arg2 getAttributeNamed: 'xsi:type') =  'Squeak-ENC:ReferenceStream']. 
!

testComplexSqHttpCall
	| handlerName call resp |

	Smalltalk isSqueak ifFalse:[^ self].

	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem).
        
	resp := call invokeWithValue: #symbol.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #symbol].
        
	resp := call invokeWithValue: Date today.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = Date today].
        
	resp := call invokeWithValue: (OrderedCollection with: #(1 2 #('three' 3) with: Date today)).
	self shouldnt:[resp isFault].
	self should:[resp returnValue = (OrderedCollection with: #(1 2 #('three' 3) with: Date today))].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
!

testDefaultEncodingCreation
	| call |
	call := SoapCall new.
	call targetObjectURI: 'urn:target:1'.
	call encoding: #soapEncoding.
	self should: [call encoder class = SoapEncoder].
        
!

testIntegerArrayHttpCall
	| handlerName call resp |
	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1 2 3)).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1 2 3)].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testSTXEncodingCreation
	| call |

	Smalltalk isSmalltalkX ifFalse:[^ self].

	call := SoapCall new.
	call targetObjectURI: 'urn:target:1'.
	call encoding: #soapSTXEncoding.
	self should: [call encoder class = SoapSTXEncoder].
!

testSimpleEnvelopeCreatein
	| call env methodElem arg1 arg2 |
	call := SoapCall new.
	call methodName: 'auau'.
	call addParameter: (SoapVariable name: #arg1 value: 1).
	call addParameter: (SoapVariable name: #arg2 value: 'two').
	env := call buildEnvelope.
        
	self should: [env name = 'env:Envelope'].
	self should: [env body name = 'env:Body'].
	methodElem := env body bodyEntries first.
	self should: [ methodElem name = 'm:auau'].
	arg1 := methodElem children at: 1.
	self should:[ arg1 name = 'arg1']. 
	self should:[ (arg1 xsiType) = 'xsd:integer']. 
	self should:[ arg1 value = '1']. 
	arg2 := methodElem children at: 2.
	self should:[ arg2 name = 'arg2']. 
	self should:[ (arg2 xsiType) = 'xsd:string']. 
	self should:[ arg2 value = 'two']. 
        
        
!

testSimpleHttpCall
	| handlerName call resp |
	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testSimpleHttpCallWithError
	| handlerName call resp |
	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #badArg1 value: 1).

	resp := call invoke.
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should:[resp raiseException] raise: SOAP::SoapInvalidParamName.
        
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testSimpleLocalCall
	| call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #localRpc.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	SoapServiceHandler default remove: self simpleAddService.
        
        
        
        
!

testSimpleLocalCallWithError
	| call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #localRpc.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #badArg1 value: 1).
	resp := call invoke.
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should:[resp raiseException] raise: SOAP::SoapInvalidParamName. "/ SoapInvalidParamName.
                
	SoapServiceHandler default remove: self simpleAddService.
        
        
        
        
!

testSimpleSTXCall

	| call resp |

	Smalltalk isSmalltalkX ifFalse:[^ self].

	SoapServiceHandler default  add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #localRpc.
	call encoding: #soapSTXEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	SoapServiceHandler default remove: self simpleAddService.
!

testSimpleSTXHttpCall
	| handlerName call resp |

	Smalltalk isSmalltalkX ifFalse:[^ self].

	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call encoding: #soapSTXEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
!

testSimpleSqCall

	| call resp |

	Smalltalk isSqueak ifFalse:[^ self].

	SoapServiceHandler default  add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #localRpc.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	SoapServiceHandler default remove: self simpleAddService.
!

testSimpleSqHttpCall
	| handlerName call resp |

	Smalltalk isSqueak ifFalse:[^ self].

	SoapHttpSetup startUp.
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #http.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
!

testSqEncodingCreation
	| call |

	Smalltalk isSqueak ifFalse:[^ self].

	call := SoapCall new.
	call targetObjectURI: 'urn:target:1'.
	call encoding: #soapSqEncoding.
	self should: [call encoder class = SoapSqEncoder].
! !

!SoapCallTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapCallTestCase.st,v 1.6 2003/01/20 19:02:15 cg Exp $'
! !
