"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapDecoderTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapDecoderTestCase methodsFor:'fixtures'!

arrayWithEmptyStringXML
	^ ' <m:reverseArray
xmlns:m="http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/rpc/"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<anArray SOAP-ENC:arrayType="xsd:anyType[3]">
<item xsi:type="xsd:string">Hello</item>
<item xsi:type="xsd:string"></item>
<item xsi:type="xsd:string">Dolphin</item>
</anArray>
</m:reverseArray>
'
!

complexDataXML
	"Google API like complex XML"
	^' <return xsi:type="ns1:GoogleSearchResult" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema">
   <documentFiltering xsi:type="xsd:boolean">false</documentFiltering>
   <estimatedTotalResultsCount xsi:type="xsd:int">1</estimatedTotalResultsCount>
   <directoryCategories xmlns:ns2="http://schemas.xmlsoap.org/soap/encoding/" xsi:type="ns2:Array" ns2:arrayType="ns1:DirectoryCategory[0]"></directoryCategories>
   <searchTime xsi:type="xsd:double">0.194871</searchTime>
   <resultElements xmlns:ns3="http://schemas.xmlsoap.org/soap/encoding/" xsi:type="ns3:Array" ns3:arrayType="ns1:ResultElement[1]">
     <item xsi:type="ns1:ResultElement">
  <cachedSize xsi:type="xsd:string">2k</cachedSize>
  <hostName xsi:type="xsd:string">somehost</hostName>
  <snippet xsi:type="xsd:string">snippet...</snippet>
  <directoryCategory xsi:type="ns1:DirectoryCategory">
    <specialEncoding xsi:type="xsd:string"></specialEncoding>
    <fullViewableName xsi:type="xsd:string"></fullViewableName>
  </directoryCategory>
  <relatedInformationPresent xsi:type="xsd:boolean">true</relatedInformationPresent>
  <directoryTitle xsi:type="xsd:string"></directoryTitle>
  <summary xsi:type="xsd:string"></summary>
  <URL xsi:type="xsd:string">http://www.squeak.org</URL>
  <title xsi:type="xsd:string">Squeak home page</title>
     </item>
   </resultElements>
   <endIndex xsi:type="xsd:int">1</endIndex>
   <searchTips xsi:type="xsd:string"></searchTips>
   <searchComments xsi:type="xsd:string"></searchComments>
   <startIndex xsi:type="xsd:int">1</startIndex>
   <estimateIsExact xsi:type="xsd:boolean">true</estimateIsExact>
   <searchQuery xsi:type="xsd:string">squeak</searchQuery>
 </return>'
! !

!SoapDecoderTestCase methodsFor:'testing'!

testDecodeAnyArray
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'anyArr1' value: #(1 '2' 3.0)) asXmlElement.
	self should: [ ret = #(1 '2' 3.0)].
        
!

testDecodeArrayWithEmptyString
	| dec arrayElem ret |
	dec := SoapDecoder new.
	arrayElem := SoapWrapElement fromXml: self arrayWithEmptyStringXML.
	ret := dec decodeXmlElement: arrayElem.
	self should: [(ret at: #anArray)
				isKindOf: Array].
	self should: [((ret at: #anArray)
				at: 2)
				= '']
!

testDecodeBase64Binary
        
	| dec ret bytes |
	dec := SoapDecoder new.
	bytes := ByteArray with: 1 with: 2 with: 3 with: 4.
	ret := dec decodeXmlElement: (SoapVariable name: 'binbin' value: bytes type: #ByteArray) asXmlElement.
	self should: [ ret = bytes].
        
!

testDecodeBoolean
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'arg1' value: 'true' type: #Boolean) asXmlElement.
	self should: [ ret = true].
        
!

testDecodeComplexData
        
	| dec complexElem ret resultElements resultElement directoryCategory |
	dec := SoapDecoder new.
	complexElem := SoapWrapElement fromXml: self complexDataXML.
	ret := dec decodeXmlElement: complexElem.
	self should: [ ret isKindOf: Dictionary].
	self should: [ (ret at: #startIndex) = 1].
	self should: [ (ret at: #searchTime) = 0.194871].
	self should: [ (ret at: #searchQuery) = 'squeak'].
	resultElements := ret at: #resultElements.
	self should: [resultElements isKindOf: Array].
	self should: [resultElements size = 1].
	resultElement := resultElements first.
	self should: [resultElement isKindOf: Dictionary].
	self should: [(resultElement at: #URL) = 'http://www.squeak.org'].
	self should: [(resultElement at: #title) = 'Squeak home page'].
	directoryCategory := resultElement at: #directoryCategory.
	self should: [directoryCategory isKindOf: Dictionary].
	self should: [(directoryCategory at: #fullViewableName) = ''].
        
                




        
        
!

testDecodeEmptyString
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'arg1' value: '' type: #String) asXmlElement.
	self should: [ ret = ''].
        
!

testDecodeFloatArray
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'floatArr1' value: #(1.1 2.2 3.3)) asXmlElement.
	self should: [ ret = #(1.1 2.2 3.3)].
        
!

testDecodeInteger
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'arg1' value: '1234567' type: #Integer) asXmlElement.
	self should: [ ret = 1234567].
        
!

testDecodeIntegerArray
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'intArr1' value: #(1 2 3) type: #Array) asXmlElement.
	self should: [ ret = #(1 2 3)].
        
!

testDecodeString
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'arg1' value: 'Smalltalk' type: #String) asXmlElement.
	self should: [ ret = 'Smalltalk'].
        
!

testDecodeStringArray
        
	| dec ret |
	dec := SoapDecoder new.
	ret := dec decodeXmlElement: (SoapVariable name: 'stringArr1' value: #('a' 'b' '<c>') type: #Array) asXmlElement.
	self should: [ ret = #('a' 'b' '<c>')].
        
! !

!SoapDecoderTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapDecoderTestCase.st,v 1.3 2003/01/20 19:02:15 cg Exp $'
! !
