"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapEncoderTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapEncoderTestCase methodsFor:'testing'!

testEncodeBase64Binary
	| enc ret bytes |
	enc := SoapEncoder new.

	bytes := ByteArray with: 1 with: 2 with: 3 with: 4.
	ret := enc encodeSoapVariable: (SoapVariable name: 'binbin' value: bytes).
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:base64Binary'].
	self should: [ ret value = 'AQIDBA=='].
        
        
        
        
        
        
!

testEncodeBoolean
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 'true' type: #Boolean).
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:boolean'].
	self should: [ ret value = 'true'].
        
        
        
        
        
        
!

testEncodeFloatArray
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'floatArr' value: #(1.0 2.3 3.4)).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:float[3]'].
	self should: [ (ret children at: 1) value = '1.0'].
	self should: [ (ret children at: 3) value = '3.4'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(4.5 5.6) type: #Array).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:float[2]'].
	self should: [ (ret children at: 1) value = '4.5'].
	self should: [ (ret children at: 2) value = '5.6'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #()).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:anyType[0]'].
	self should: [ ret children isEmpty].
        
        
        
        
        
!

testEncodeInteger
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 1).
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:integer'].
	self should: [ ret value = '1'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 10000000000 type: #Integer).
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:integer'].
	self should: [ ret value = '10000000000'].
        
        
        
        
!

testEncodeIntegerArray
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(1 2 3)).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:integer[3]'].
	self should: [ (ret children at: 1) value = '1'].
	self should: [ (ret children at: 3) value = '3'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(4 5) type: #Array).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:integer[2]'].
	self should: [ (ret children at: 1) value = '4'].
	self should: [ (ret children at: 2) value = '5'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #()).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:anyType[0]'].
	self should: [ ret children isEmpty].
        
        
        
        
        
!

testEncodeNil
	| enc ret |
	enc := SoapEncoder new.
        
	SoapEncoder useXsiNullAttribute: false.
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: nil).
	self should: [ ret = nil].
        
	SoapEncoder useXsiNullAttribute: true.
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: nil).
	self should: [ ret name = 'auau'].
	self should: [ (ret getAttributeNamed: SoapConstants xsiNil) = 1].
        
        
        
        
        
!

testEncodeSimpleNamedStruct
	| enc ret dic struct |
	enc := SoapEncoder new.
	dic := Dictionary new.
	dic at: #varString put: 'hello'.
	dic at: #varInt put: 42.
	dic at: #varFloat put: 0.5.
	struct := SoapEncStruct fromDictionary: dic named: 'SearchResult'.

	ret := enc encodeSoapVariable: (SoapVariable name: 'inputStruct' value: struct).
	self should:[ret name = 'inputStruct'].
"/        self should:[ret xsiType = 'SearchResult'].
	self should:[ret children size = 3].
	self should:[(ret findChildNamed: 'varString') xsiType = SoapConstants xsdStringType].
	self should:[(ret findChildNamed: 'varInt') xsiType = SoapConstants xsdIntegerType].
	self should:[(ret findChildNamed: 'varFloat') xsiType = SoapConstants xsdFloatType].
	self should:[(ret findChildNamed: 'varString') value = 'hello'].
	self should:[(ret findChildNamed: 'varInt') value = '42'].
	self should:[(ret findChildNamed: 'varFloat') value = '0.5'].
        
        
        
        
        
        
        
!

testEncodeSimpleStruct
	| enc ret dic |
	enc := SoapEncoder new.
	dic := Dictionary new.
	dic at: #varString put: 'hello'.
	dic at: #varInt put: 42.
	dic at: #varFloat put: 0.5.

	ret := enc encodeSoapVariable: (SoapVariable name: 'inputStruct' value: dic).
	self should:[ret name = 'inputStruct'].
	self should:[ret children size = 3].
	self should:[(ret findChildNamed: 'varString') xsiType = SoapConstants xsdStringType].
	self should:[(ret findChildNamed: 'varInt') xsiType = SoapConstants xsdIntegerType].
	self should:[(ret findChildNamed: 'varFloat') xsiType = SoapConstants xsdFloatType].
	self should:[(ret findChildNamed: 'varString') value = 'hello'].
	self should:[(ret findChildNamed: 'varInt') value = '42'].
	self should:[(ret findChildNamed: 'varFloat') value = '0.5'].
        
        
        
        
        
        
        
!

testEncodeString
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 'smalltalk').
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:string'].
	self should: [ ret value = 'smalltalk'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 'ruby' type: #String).
	self should: [ (ret getAttributeNamed: 'xsi:type') = 'xsd:string'].
	self should: [ ret value = 'ruby'].
        
        
        
        
!

testEncodeStringArray
	| enc ret |
	enc := SoapEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'stringArr' value: #('a' 'b' 'c')).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:string[3]'].
	self should: [ (ret children at: 1) value = 'a'].
	self should: [ (ret children at: 3) value = 'c'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #('' 'eee') type: #Array).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:string[2]'].
	self should: [ (ret children at: 1) value = ''].
	self should: [ (ret children at: 2) value = 'eee'].
        
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #()).
	self should: [ (ret getAttributeNamed: (SoapConstants soapEncPrefixColon, 'arrayType')) =  'xsd:anyType[0]'].
	self should: [ ret children isEmpty].
        
        
        
        
        
! !

!SoapEncoderTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapEncoderTestCase.st,v 1.4 2003/01/20 19:02:15 cg Exp $'
! !
