"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapHttpClientTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapHttpClientTestCase methodsFor:'fixtures'!

errorEnvelope
        
	^'

<env:Envelope

	xmlns:env="http://schemas.xmlsoap.org/soap/envelope/"

	env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">

	<env:Body>

		<m:DoSomeOtherThing xmlns:m="urn:stSoap:Person1234">

			<symbol>DIS</symbol>

		</m:DoSomeOtherThing>

	</env:Body>

</env:Envelope>

' copy.
!

simpleEnvelope
        
	^'

<env:Envelope

	xmlns:env="http://schemas.xmlsoap.org/soap/envelope/"

	env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">

	<env:Body>

		<m:add xmlns:m="http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/">

			<newElem>newElement</newElem>

		</m:add>

	</env:Body>

</env:Envelope>

' copy.
!

simpleService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	^serv
! !

!SoapHttpClientTestCase methodsFor:'testing'!

testErrorEnvelopeSend
        
	| curPort retXML envelope body prefix fault |
	SoapHttpSetup startUp.
	SoapServiceHandler default add: self simpleService.
	curPort := SoapSetting defaultPort.
        
	retXML := (SoapHttpClient url: 'http://localhost:', curPort printString, '/' ) send: self errorEnvelope.
	envelope := SoapEnvelopeBuilder new buildSoapEnvelopeFromXmlString: retXML.
        
	self should:[envelope isKindOf: SoapEnvelope].
	body := envelope body.
	self should:[ body hasFault].
	fault := body fault.
	prefix := SoapConstants soapEnvPrefixColon.
	self should:[ fault name = (prefix, 'Fault')].
	self should:[ fault faultstring  =  'SoapNoSuchService: DoSomeOtherThing'].
	self should:[ fault faultcode =  (prefix,'Client')].
	SoapServiceHandler default remove: self simpleService.
        
!

testSimpleEnvelopeSend
        
	| curPort retXML envelope body method |
	SoapHttpSetup startUp.
	SoapServiceHandler default add: self simpleService.
	curPort := SoapSetting defaultPort.
        
	retXML := (SoapHttpClient url: 'http://localhost:', curPort printString, '/' ) send: self simpleEnvelope.
	envelope := SoapEnvelopeBuilder new buildSoapEnvelopeFromXmlString: retXML.
        
	self should:[envelope isKindOf: SoapEnvelope].
	body := envelope body.
	self shouldnt:[ body hasFault].
	method := body bodyEntries first.
	self should:[ method name = 'm:addResponse'].
	self should:[ (method findChildNamed: 'result' ) value = 'newElement'].
        
	SoapServiceHandler default remove: self simpleService.
        
! !

!SoapHttpClientTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapHttpClientTestCase.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
