"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapServiceHandlerTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapServiceHandlerTestCase methodsFor:'fixtures'!

defaultTargetObjectURI
        
	^'http://www.fooServer.com/OrderedCollection'
        
!

errorEnvelope
	| xmlStr env |
	xmlStr := self errorEnvelopeXml.
	env := SoapEnvelopeBuilder new buildSoapEnvelopeFromXmlString: xmlStr.
	^env
!

errorEnvelopeXml
	^'<?xml version="1.0" encoding="UTF-8"?>

<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Body>
<m:adddd xmlns:m="http://soapinterop.org/">
<newElem>1</newElem>
</m:adddd>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'
!

simpleContext
        
	^SoapContext new targetObjectURI: self defaultTargetObjectURI
        
!

simpleEnvelope
	| xmlStr env |
	xmlStr := self simpleEnvelopeXml.
	env := SoapEnvelopeBuilder new buildSoapEnvelopeFromXmlString: xmlStr.
	^env
!

simpleEnvelopeWithXsdType
	| xmlStr env |
	xmlStr := self simpleEnvelopeXmlWithXsdType.
	env := SoapEnvelopeBuilder new buildSoapEnvelopeFromXmlString: xmlStr.
	^env
!

simpleEnvelopeXml
	^'<?xml version="1.0" encoding="UTF-8"?>

<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Body>
<m:add xmlns:m="http://soapinterop.org/">
<newElem>1</newElem>
</m:add>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'
!

simpleEnvelopeXmlWithXsdType
	^'<?xml version="1.0" encoding="UTF-8"?>

<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Body>
<m:add xmlns:m="http://soapinterop.org/">
<newElem xsi:type="xsd:float">1.02</newElem>
</m:add>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'
!

simpleService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	serv targetObjectURI: self defaultTargetObjectURI.
	^serv
! !

!SoapServiceHandlerTestCase methodsFor:'testing'!

testDispatchError

	| handler env fault details |
	handler := SoapServiceHandler new. 
	handler add: self simpleService.
	env := self errorEnvelope.
	"self shouldnt:[env := handler dispatch: env with: self simpleContext.] raise: Error."
	env := handler dispatch: env with: self simpleContext.
	self assert:( env body hasFault).
        
	fault := env body fault.
        
	self should: [fault name = 'env:Fault'].
	self should: [fault faultstring =  'SoapNoSuchService: /OrderedCollection:adddd'].
	"/ self should: [fault faultcode =  'env:Client'].
        
	details := fault detail subElements first.
	self should: [details shortName = 'details'].
	self should: [(details declaredNamespaceURIFor: SoapFaultDetailBuilder defaultPrefix)
				= SoapFaultDetailBuilder defaultNamespace].
        
	self should:[ (details findChildNamed: 'ErrorClass') value =  'SoapNoSuchService'].
	self should: [fault faultcode =  'env:Client'].

        
!

testDispatchSimple
	| handler env ret |
	handler := SoapServiceHandler new.
	handler add: self simpleService.
	env := handler dispatch: self simpleEnvelope with: self simpleContext.
	self shouldnt:[env body hasFault].

	ret := env body bodyEntries first.
        
	self should:[ ret name = 'm:addResponse'].
	self should: [ ret encodingStyleURIs = #('http://schemas.xmlsoap.org/soap/encoding/')].
	self should:[ret firstChild xsiType = 'xsd:string'].
	self should:[ ret firstChild value = '1'].
        
        

        
!

testDispatchSimpleWithXsdType
	| handler env ret |
	handler := SoapServiceHandler new.
	handler add: self simpleService.
	env := handler dispatch: self simpleEnvelopeWithXsdType with: self simpleContext.
	self shouldnt:[env body hasFault].
	ret := env body bodyEntries first.

	self should:[ ret name = 'm:addResponse'].
	self should: [ ret encodingStyleURIs = #('http://schemas.xmlsoap.org/soap/encoding/')].
	self should:[ret firstChild xsiType = 'xsd:float'].
	self should:[ ret firstChild value = '1.02'].
        
        
!

testDispatchSoapNoSuchServiceError

	| handler env fault |
	handler := SoapServiceHandler new.
	handler add: self simpleService.
	env := self errorEnvelope.
	self shouldnt:[env := handler dispatch: env with: self simpleContext.] raise: Error.
	self assert:( env body hasFault).
        
	fault := env body fault.
        
        

        
!

testInvokeSimple
	| handler ret |
	handler := SoapServiceHandler new.
	handler add: self simpleService.
	ret := handler invoke: self simpleEnvelope with: self simpleContext.
        
	self should:[ret = '1'].
        
!

testInvokeSimpleWithXsdType
	| handler ret |
	handler := SoapServiceHandler new.
	handler add: self simpleService.
	ret := handler invoke: self simpleEnvelopeWithXsdType with: self simpleContext.
        
	self should:[ret = 1.02].
        
        
! !

!SoapServiceHandlerTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapServiceHandlerTestCase.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
