"{ Package: 'stx:goodies/soap/tests' }"

"{ NameSpace: SOAP }"

TestCase subclass:#SoapSqtpCallTestCase
	instanceVariableNames:'originalTransports'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-TestCases'
!


!SoapSqtpCallTestCase methodsFor:'fixtures'!

simpleAddService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	^serv
! !

!SoapSqtpCallTestCase methodsFor:'running'!

setUp
	originalTransports  := SoapTransportSetup runningTransportNames.
	SoapTransportSetup startOnly: #(sqtp) 
!

tearDown
        
	SoapTransportSetup startOnly: originalTransports 
! !

!SoapSqtpCallTestCase methodsFor:'testing'!

testAnyArraySqtpCall
	| handlerName call resp |
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1.0 'two' 3)).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1.0 'two' 3)].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testBulkDataSqtpCall
	| longString call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.

	longString := (String new: 1024 * 1024). "1000K"

	call addParameter: (SoapVariable name: #newElem value: longString).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = longString].
        
	SoapServiceHandler default remove: self simpleAddService.
        
        
        
        
!

testCannnotConnectSqtpCall
	| call resp |

	call := (SoapCallEntry tcpHost: 'foobarbazboo' port: 8823) newCall.
	call transport: #sqtp.
	call methodName: 'helloWorld'.
        
	resp := call invoke.
        
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SoapCannnotResolveServer
        
        
        
        
        
        
        
!

testCannnotResolveSqtpCall
	| notUsedPort call resp |
	notUsedPort := 9876.
	call := (SoapCallEntry tcpHost: 'localhost' port: notUsedPort) newCall.
	call transport: #sqtp.
	call methodName: 'helloWorld'.
        
	resp := call invoke.
        
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SoapCannnotConnectToServer
        
        
        
        
        
        
        
!

testComplexSqSqtpCall
	| handlerName call resp |

	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem).
        
	resp := call invokeWithValue: #symbol.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #symbol].
        
	resp := call invokeWithValue: Date today.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = Date today].
        
	resp := call invokeWithValue: (OrderedCollection with: #(1 2 #('three' 3) with: Date today)).
	self shouldnt:[resp isFault].
	self should:[resp returnValue = (OrderedCollection with: #(1 2 #('three' 3) with: Date today))].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testIntegerArraySqtpCall
	| handlerName call resp |
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1 2 3)).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1 2 3)].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testSimpleSqSqtpCall
	| handlerName call resp |
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
!

testSimpleSqtpCall
	| call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
        
	resp := call invoke.
        
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
        
	SoapServiceHandler default remove: self simpleAddService.
        
        
        
        
!

testSimpleSqtpCallWithError
	| handlerName call resp |
        
	handlerName := SoapHttpModule useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort )newCall.
	call methodName: 'add'.
	call transport: #sqtp.
	call addParameter: (SoapVariable name: #badArg1 value: 1).

	resp := call invoke.
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should:[resp raiseException] raise: SoapInvalidParamName.
        
        
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
        
        
        
        
        
! !

!SoapSqtpCallTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/tests/SoapSqtpCallTestCase.st,v 1.2 2003/01/20 19:02:16 cg Exp $'
! !
