"{ Package: 'stx:goodies/soap/wsdl' }"

"{ NameSpace: SOAP }"

Object subclass:#WSDLDefinitions
	instanceVariableNames:'rootUrl documents transport messages portTypes bindings services
		schemas'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-WSDL'
!

WSDLDefinitions comment:'I am all the definition elements in a WSDL document and the documents that are imported.

'
!


!WSDLDefinitions class methodsFor:'instance creation'!

onUrl: aURL transport: aTransport
	^self new rootUrl: aURL transport: aTransport
! !

!WSDLDefinitions class methodsFor:'instance creation-private'!

new
	^super new initialize

!

onDocument: aXeDoc transport: aTransport
	^self new rootDocument: aXeDoc asParserNode ownerDocument transport: aTransport
! !

!WSDLDefinitions methodsFor:'accessing'!

allPorts
	| ports |
	ports := OrderedCollection new.
	self services 
		do: [:each | ports addAll: (each value childNodes select: [:eachC | eachC localName = 'port'])].
	^ports
!

definitionsNode
	| node |
	node := self rootDocument firstChild.
	self assert: [node localName = 'definitions'].
	^node
!

extractSchemas
	"extract the schema nodes."

	schemas do: 
			[:each | 
			| targetNamespace |
			(targetNamespace := each getAttribute: 'targetNamespace') isNil 
				ifTrue: [self error: 'Schema does not have a targetNamespace'].
			(XeNamespace isNonSchemaNS: targetNamespace asSymbol) 
				ifFalse: [self error: 'Can not import schema targeting: ' , targetNamespace]].
	^schemas
!

qualifiedPortName: aString
	^self supportedPortQNames detect: [:each | each localName = aString]
!

singlePortBinding
	(bindings size = 1 and: [self allPorts size = 1]) 
		ifFalse: [self error: 'Can not resolve binding'].
	^bindings first
!

singleSupportedPortQName
    (bindings size = 1) ifTrue:[ 
        (self isBindingSupported: bindings first value) ifTrue:[
            (self allPorts size = 1) ifTrue:[
                    ^ self allPorts first tQValueForName
            ].
            ^ self allPorts first tQValueForName
        ]
    ].
    ^ self error: 'Can not resolve qName to binding'

    "Modified: / 20-03-2012 / 18:38:30 / cg"
! !

!WSDLDefinitions methodsFor:'accessing-private'!

bindings
	^bindings

!

documents
	^documents
!

messages
	^messages
!

portTypes
	^portTypes
!

rootDocument
	^documents at: self rootUrl
!

rootUrl
	^rootUrl
!

schemas
	^schemas 
!

services
	^services
!

transport
	"Answer the object used for Transport"

	^transport
! !

!WSDLDefinitions methodsFor:'accessing-wsdl definitions'!

bindingNamed: aXeQName
    |binding|

    binding := bindings detect: [:each | each key = aXeQName] ifNone:nil.
"/    binding isNil ifTrue:[
"/        binding := bindings detect: [:each | each key localName = aXeQName localName] ifNone:nil.
"/    ].
    binding isNil ifTrue:[
        self errorLocating: 'binding' named: aXeQName.
        ^ nil
    ].
    ^ binding value
!

messageNamed: aXeQName
	^(messages detect: [:each | each key = aXeQName]
		ifNone: [self errorLocating: 'message' named: aXeQName]) value
!

portNamed: aXeQName
	^self allPorts detect: [:each | each tQValueForName = aXeQName]
		ifNone: [self errorLocating: 'port' named: aXeQName]
!

portTypeNamed: aXeQName
    |portType|

    portType := portTypes detect: [:each | each key = aXeQName] ifNone:nil.
"/    portType isNil ifTrue:[
"/        portType := portTypes detect: [:each | each key localName = aXeQName localName] ifNone:nil.
"/    ].
    portType isNil ifTrue:[
        self errorLocating: 'portType' named: aXeQName.
        ^ nil.
    ].
    ^ portType value
! !

!WSDLDefinitions methodsFor:'accessing-wsdl definitions-private'!

errorLocating: aDefString named: aXeQName
	^self error: 
		('Can not locate the %1 named: %2' formatWith: aDefString with: aXeQName displayString)  

! !

!WSDLDefinitions methodsFor:'accessing-wsdl names'!

allPortNames
	^self allPorts collect: [:each | each getAttribute: 'name']
!

serviceQName
	self services isEmpty ifTrue: [^nil].
	^self services first key
!

supportedPortQNames
        "Answer the QNames of the supported ports.
        Only ports with soap-binding are supported at the moment. (not http get/post bindings)"

        ^self allPorts 
            select:[:each | 
                self isBindingSupported: 
                    (self bindingNamed: 
                        (each qValueFor: 'binding' ifNone: [self error: 'Could not locate binding attribute']))]
            thenCollect: [:each | each tQValueForName]

    "Modified (comment): / 20-03-2012 / 18:35:02 / cg"
! !

!WSDLDefinitions methodsFor:'accessing-wsdl names-private'!

definitionsName
	^self definitionsNode getAttribute: 'name'
! !

!WSDLDefinitions methodsFor:'asserting-private'!

assertTypesSupport: schemaNodes
	schemaNodes do: 
			[:each | 
			each localName = 'schema' 
				ifFalse: 
					[self error: 'Unsupported type system, Spray only supports the XML Schema type system']]
! !

!WSDLDefinitions methodsFor:'enquiries-private'!

urlOf: aXeDoc
	^documents keyAtValue: aXeDoc
! !

!WSDLDefinitions methodsFor:'importing'!

importUrl: aString namespace: aString2 relativeTo: aString3
	| document targetNamespace absoluteUrl |
	#todo.	"the import process can be improved by selectively importing namespaces.
	See SoapBuilders discussion ~ 20011018"
	absoluteUrl := self makeUrl: aString absoluteFrom: aString3.
	(self hasImported: absoluteUrl) 
		ifFalse: 
			[document := self transport getXeDocAt: absoluteUrl.
			self addDocument: document at: absoluteUrl.
			(targetNamespace := document documentElement getAttribute: 'targetNamespace') isNil 
				ifTrue: [self error: 'The documentElement does not have a targetNamespace'].
			"(targetNamespace = aString2) not ifTrue: [self error: 'The targetNamespace does not match the import namespace']."
			self importDocument: document namespace: targetNamespace]
!

rootDocument: aXeDoc transport: aTransport
	self 
		rootDocument: aXeDoc
		rootUrl: ''
		transport: aTransport
!

rootUrl: aString transport: aTransport 
	"Import the root wsdl at the url; aString, using the transport aTransport.
	Any import nodes in this document will also be imported"

	self 
		rootDocument: (aTransport getXeDocAt: aString)
		rootUrl: aString
		transport: aTransport
! !

!WSDLDefinitions methodsFor:'importing-private'!

addAllFrom: aXeDoc at: aUrl
	| targetNamespace |
	self addDocument: aXeDoc at: aUrl.
	targetNamespace := aXeDoc documentElement getAttribute: 'targetNamespace'.
	targetNamespace isNil 
		ifTrue: [self error: 'The documentElement does not have a targetNamespace'].
	self importDocument: aXeDoc namespace: targetNamespace
!

importDefinitions: aXeDoc namespace: aString
	| children imports |
	children := aXeDoc documentElement childNodes.
	self selectChildNodesFrom: children in: aXeDoc documentElement targetNamespace.
	imports := children select: [:each | each localName = 'import'].
	imports do: 
			[:each | 
			self 
				importUrl: (each getAttribute: 'location')
				namespace: (each getAttribute: 'namespace')
				relativeTo: (self urlOf: aXeDoc)]
!

importDocument: aXeDoc namespace: aString
	| root |
	root := aXeDoc documentElement.
	root localName = 'schema' 
		ifTrue: [self importSchema: aXeDoc namespace: aString]
		ifFalse: [self importDefinitions: aXeDoc namespace: aString]
!

importSchema: aXeDoc namespace: aString
	schemas add: aXeDoc documentElement
!

rootDocument: aXeDoc rootUrl: aUrl transport: aTransport
        "Private - Import the root document  which was from the Url.
        If any imports are found, import the node using aTransport  "

        rootUrl := aUrl.
        transport := aTransport.
        "/ cg: changed to Dolphin::LookupTable
        "/ documents := LookupTable new.
        documents := Dolphin::LookupTable new.
        self addAllFrom: aXeDoc at: self rootUrl
! !

!WSDLDefinitions methodsFor:'initializing-private'!

initialize
        super initialize.
        "/ cg: changed to Dolphin::LookupTable
        "/ documents := LookupTable new.
        documents := Dolphin::LookupTable new.
        bindings:= OrderedCollection new.
        messages := OrderedCollection new.
        portTypes := OrderedCollection new.
        services := OrderedCollection new.
        schemas := OrderedCollection new
! !

!WSDLDefinitions methodsFor:'operations-private'!

addDocument: aXeDoc at: aString
	"Private - Add the document, checking for circular references"

	(documents includesKey: aString) 
		ifTrue: [self error: 'the document has already been imported at: ' , aString displayString].
	documents at: aString put: aXeDoc
!

addTypes: anArray
	| schemaNodes |
	anArray isEmpty ifTrue: [^nil].
	self assert: [anArray size = 1].
	self assertTypesSupport: (schemaNodes := anArray first childNodes).
	^schemas addAll: schemaNodes
!

extractQNodes: aString from: children in: aTargetNS
	^children select: [:each | each localName = aString]
		thenCollect: [:each | (XeQName local: (each getAttribute: 'name') ns: aTargetNS) -> each]
!

makeUrl: aString absoluteFrom: aString2 
	^(WSDLUrl url: aString base: aString2) absoluteUrl
!

selectChildNodesFrom: children in: aTargetNS
	"Extract the definition nodes by type, and add associations between the nodes QName and the node to collections"

	services addAll: (self 
				extractQNodes: 'service'
				from: children
				in: aTargetNS).
	bindings addAll: (self 
				extractQNodes: 'binding'
				from: children
				in: aTargetNS).
	portTypes addAll: (self 
				extractQNodes: 'portType'
				from: children
				in: aTargetNS).
	messages addAll: (self 
				extractQNodes: 'message'
				from: children
				in: aTargetNS).
	self addTypes: (children select: [:each | each localName = 'types'])
! !

!WSDLDefinitions methodsFor:'testing'!

isBindingSupported: aBindingNode
	| binding |
	binding := aBindingNode childNodes detect: [:each | each localName = 'binding']
				ifNone: [^true	"probably old, and we can probably read it"].
	^binding namespace ~~ XeNamespace wsdlHTTP
! !

!WSDLDefinitions methodsFor:'testing-private'!

hasImported: aString 
	^documents keys includes: aString
! !

!WSDLDefinitions class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLDefinitions.st,v 1.5 2012/03/20 21:04:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLDefinitions.st,v 1.5 2012/03/20 21:04:28 cg Exp $'
! !
