"{ Package: 'stx:goodies/soap/wsdl' }"

"{ NameSpace: SOAP }"

Object subclass:#WSDLOperation
	instanceVariableNames:'wsdlPort portTypeOpNode bindingOpNode name inputBody outputBody
		inputHeaders outputHeaders style'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-WSDL'
!

WSDLOperation comment:'Parses the information from; 
http://www.w3.org/TR/wsdl.html#_how-s

<definitions .... >
    <binding .... >
        <soap:binding style="rpc|document" transport="uri">
        <operation .... >
           <soap:operation soapAction="uri"? style="rpc|document"?>?
           <input>
               <soap:body parts="nmtokens"? use="literal|encoded"
                          encodingStyle="uri-list"? namespace="uri"?>
               <soap:header message="qname" part="nmtoken" use="literal|encoded"
                            encodingStyle="uri-list"? namespace="uri"?>*
                 <soap:headerfault message="qname" part="nmtoken" use="literal|encoded"
                                   encodingStyle="uri-list"? namespace="uri"?/>*
               <soap:header>                                
           </input>
           <output>
               <soap:body parts="nmtokens"? use="literal|encoded"
                          encodingStyle="uri-list"? namespace="uri"?>
               <soap:header message="qname" part="nmtoken" use="literal|encoded"
                            encodingStyle="uri-list"? namespace="uri"?>*
                 <soap:headerfault message="qname" part="nmtoken" use="literal|encoded"
                                   encodingStyle="uri-list"? namespace="uri"?/>*
               <soap:header>                                
           </output>
           <fault>*
               <soap:fault name="nmtoken" use="literal|encoded"
                           encodingStyle="uri-list"? namespace="uri"?>
            </fault>
        </operation>
    </binding>


'
!


!WSDLOperation methodsFor:'accessing-binding-private'!

bindingOpNode
	^bindingOpNode
!

findStyle
	^self wsdlPort styleFor: bindingOpNode
!

guessSoapOperationNode
	"Private - these guess methods were necessary because a major WSDL toolkit was deployed which used incorrect namespaces"

	| answer |
	answer := bindingOpNode 
				detectChildNode: 'operation'
				in: #'http://schemas.xmlsoap.org/wsdl/soap'
				ifNone: [].
	answer notNil ifTrue: [self wsdlPort warnSoapNS].
	^answer
!

soapAction
	| soapOpNode |
	(soapOpNode := self soapOperationNode) isNil ifTrue: [^nil].
	^soapOpNode getAttribute: 'soapAction' 
!

soapOperationNode
	^self bindingOpNode 
		detectChildNode: 'operation'
		in: XeNamespace soapWSDL
		ifNone: [self guessSoapOperationNode]
!

style
	style isNil ifTrue: [style := self findStyle].
	^style
! !

!WSDLOperation methodsFor:'accessing-private'!

definitions
	^self wsdlPort definitions
!

input
	^inputBody
!

inputHeaders
	^inputHeaders
!

inputHeaders: aCollection
	inputHeaders := aCollection
!

name
	^name
!

output
	^outputBody
!

outputHeaders
	^outputHeaders
!

outputHeaders: aCollection
	outputHeaders := aCollection
!

portTypeOpNode
	^portTypeOpNode
!

spmdOperationClass
        ^self isRPC ifTrue: [SOAP::SpmdRPCOperation] ifFalse: [SOAP::SpmdDOCOperation]
!

wsdlPort
	^wsdlPort
! !

!WSDLOperation methodsFor:'initializing'!

setName: aString wsdlPort: aWsdlPort portTypeOpNode: ptNode bindingOpNode: bNode
	portTypeOpNode := ptNode.
	bindingOpNode := bNode.
	name := aString.
	wsdlPort := aWsdlPort.
	inputBody := (WSDLOperationBinding new operation: self) setInputBody.
	outputBody := (WSDLOperationBinding new operation: self) setOutputBody.
	inputHeaders := inputBody headerBindings.
	outputHeaders := outputBody headerBindings
! !

!WSDLOperation methodsFor:'operations-private'!

addHandlerTo: aBodyBinding
	self isRPC ifTrue: [aBodyBinding beRPCHandler] ifFalse: [aBodyBinding beDOCHandler].
	^aBodyBinding
! !

!WSDLOperation methodsFor:'spray creation'!

createInputBody
	| bodyBinding |
	bodyBinding := inputBody createBody.
	self addHandlerTo: bodyBinding.
	^bodyBinding
! !

!WSDLOperation methodsFor:'spray creation-private'!

createInputHeaders
	^inputHeaders collect: [ :each | each createHeader]
!

createOutputBody
	| bodyBinding |
	bodyBinding := outputBody createBody.
	self addHandlerTo: bodyBinding.
	^bodyBinding
!

createOutputHeaders
	^outputHeaders collect: [ :each | each createHeader]
!

createSpmdOperation
	^self spmdOperationClass fromWSDLOperation: self
			
! !

!WSDLOperation methodsFor:'testing-private'!

isRPC 
	^self style = 'rpc'
		
! !

!WSDLOperation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLOperation.st,v 1.3 2009/11/18 16:10:44 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLOperation.st,v 1.3 2009/11/18 16:10:44 cg Exp $'
! !
