"{ Package: 'stx:goodies/soap/wsdl' }"

"{ NameSpace: SOAP }"

Object subclass:#WSDLOperationBinding
	instanceVariableNames:'operation messageType bindingNode soapNode wsdlMessage'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-WSDL'
!

WSDLOperationBinding comment:'Parses the information in;

<definitions .... >
    <binding .... >
        <soap:binding ...>
        <operation .... >
           <soap:operation ... >?
           <input>
               <soap:body parts="nmtokens"? use="literal|encoded"
                          encodingStyle="uri-list"? namespace="uri"?>

OR:     <soap:header message="qname" part="nmtoken" use="literal|encoded"
                            encodingStyle="uri-list"? namespace="uri"?>*
                            '
!


!WSDLOperationBinding methodsFor:'accessing-binding-private'!

bindingNodeName
	" <operation ...>
            <soap:operation ....>
            <input name='echoString'>
	Would answer echoString"

	^bindingNode getAttribute: 'name'
!

guessSoapBodyNode
	"these guess methods were necessary because a major WSDL toolkit (Delphi)
	was deployed which used incorrect namespaces"
	
	| answer |
	answer := bindingNode 
		detectChildNode: 'body'
		in: #'http://schemas.xmlsoap.org/wsdl/soap'
		ifNone: [self error: 'Unable to find the soap:body element'].
	self wsdlPort warnSoapNS.
	^answer	

!

namespace
	"Optional"

	| ns |
	ns := soapNode getAttribute: 'namespace'.
	^ns isNil ifTrue: [] ifFalse: [ns asSymbol]
!

soapBodyNode
	^bindingNode 
				detectChildNode: 'body'
				in: XeNamespace soapWSDL
				ifNone: [self guessSoapBodyNode ]
!

soapEncodingStyle
	"Optional."
	| att |
	^(att := soapNode getAttribute: 'encodingStyle') isNil 
		ifTrue: [#'']
		ifFalse: [att asSymbol]
!

soapUse
	^soapNode getAttribute: 'use'
!

use
	"In one section the soap:body 'use' attribute is shown as required (3.2), and in another it is optional(3.5)?????"

	| use |
	use := self soapUse.
	(#('literal' 'encoded') includes: use) 
		ifFalse: [self error: 'Use attribute must be either literal or encoded'].
	^use
! !

!WSDLOperationBinding methodsFor:'accessing-port type'!

partAttribute
	| part |
	self assert: [(part := soapNode getAttribute: 'part') notNil].
	^part
! !

!WSDLOperationBinding methodsFor:'accessing-port type-private'!

partsAttribute
	| bodyParts |
	(bodyParts := soapNode getAttribute: 'parts') isNil ifTrue: [^nil].
	^bodyParts subStrings
!

wsdlMessage
	^wsdlMessage
! !

!WSDLOperationBinding methodsFor:'accessing-private'!

bindingNode: aXeNode
	bindingNode := aXeNode
!

definitions 
	^self operation definitions 
!

messageType
	^messageType
!

messageType: aString 
	messageType := aString 
!

operation
	^operation
!

operation: aWSDLOperation
	operation := aWSDLOperation
!

soapNamespace
	^XeNamespace soapWSDL
!

soapNode: aXeNode
	soapNode := aXeNode
!

wsdlPort
	^operation wsdlPort
! !

!WSDLOperationBinding methodsFor:'exceptions-private'!

errorLocatingMessage
	^self error: 'Could not locate message attribute'
! !

!WSDLOperationBinding methodsFor:'initializing'!

setInputBody
	"beBody;"

	self
		beInput;
		setNodes.
	soapNode := self soapBodyNode
!

setOutputBody
	"beBody;"

	self
		beOutput;
		setNodes.
	soapNode := self soapBodyNode
!

setWSDLBodyMessage
	"node := self operation portTypeOpNode selectSingleNode: './' , self messageType"

	| node |
	node := self operation portTypeOpNode 
				detectChildNode: self messageType
				in: nil
				ifNone: 
					[self 
						error: ('Spray currently only supports Request-Response operations; Expected %1 node.' 
								formatWith: self messageType displayString)].
	wsdlMessage := WSDLMessage new setDefinitions: self definitions
				name: (node qValueFor: 'message' ifNone: [self errorLocatingMessage])

	"WSDL Spec1.1 sec3.5 If the operation style is rpc each part is a parameter or a return value and appears inside a wrapper element within the body (following Section 7.1 of the SOAP specification). The wrapper element is named identically to the operation name and its namespace is the value of the namespace attribute. Each message part (parameter) appears under the wrapper, represented by an accessor named identically to the corresponding parameter of the call. Parts are arranged in the same order as the parameters of the call. "

	"This defines the case for the input message, but what about the output message? ... by section7 rules, we MUST ignore it"
!

setWSDLHeaderMessage
	wsdlMessage := WSDLMessage new setDefinitions: self definitions
				name: (soapNode qValueFor: 'message' ifNone: [self errorLocatingMessage])
! !

!WSDLOperationBinding methodsFor:'initializing-private'!

setNodes
	"Private - selectSingleNode: './' , self messageType "

	bindingNode := self operation bindingOpNode 
				detectChildNode: self messageType
				in: nil
				ifNone: 
					[self error: ('Expected %1 node, Spray only supports Request-Response operations' 
								formatWith: self messageType displayString)].
	self setWSDLBodyMessage
! !

!WSDLOperationBinding methodsFor:'modes-private'!

beInput
	messageType := 'input'
!

beOutput
	messageType := 'output'
! !

!WSDLOperationBinding methodsFor:'spray creation'!

headerBindings
	^(bindingNode selectChildNodes: 'header' in: XeNamespace soapWSDL) collect: 
			[:each | 
			(self class new)
				operation: operation;
				messageType: messageType;
				bindingNode: bindingNode;
				soapNode: each;
				setWSDLHeaderMessage;
				yourself
			"beHeader;"]
! !

!WSDLOperationBinding methodsFor:'spray creation-private'!

createBody
        ^SOAP::SpmdSoapBody fromWSDLBinding: self 
!

createBodyMessage
	| message |
	message := wsdlMessage createMessage.
	self isOutput ifTrue: [message isNoReturnPart: self hasNoReturnPart].
	^message
!

createHeader
        ^SOAP::SpmdSoapHeader fromWSDLBinding: self 
!

createHeaderMessage
	^wsdlMessage createMessage.
! !

!WSDLOperationBinding methodsFor:'testing'!

hasNoReturnPart
	"See WSDL1.1 Sec 2.4.6.
	This is an optional attribute, but if we can use it to determine that there is no return part, we can use this information to extract response accessors more accurately. (ie we dont have to ignore the first accessor name (See SoapRPCStructType))"

	| paramOrder paramList |
	(paramOrder := self operation portTypeOpNode getAttribute: 'parameterOrder') isNil 
		ifTrue: [^false].
	paramList := paramOrder trimBlanks subStrings.
	^self wsdlMessage hasNoReturnPartWith: paramList
! !

!WSDLOperationBinding methodsFor:'testing-private'!

isInput
	^messageType = 'input'
!

isOutput
	^self isInput not
! !

!WSDLOperationBinding class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLOperationBinding.st,v 1.3 2009/11/18 16:10:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLOperationBinding.st,v 1.3 2009/11/18 16:10:39 cg Exp $'
! !
