"{ Package: 'stx:goodies/soap/wsdl' }"

"{ NameSpace: SOAP }"

Object subclass:#WSDLPortData
	instanceVariableNames:'definitions portQName portNode bindingNode soapBindingNode
		portTypeNode wsdlOperations report'
	classVariableNames:'WarnQName WarnSoapNS'
	poolDictionaries:''
	category:'SOAP-WSDL'
!

WSDLPortData comment:'I am a single port in a WSDL. I provide access to the information of the port




'
!

!WSDLPortData class methodsFor:'documentation'!

examples
"
    |defs|

    defs := SOAP::WSDLDefinitions 
        onUrl:'http://webservices.daehosting.com/services/isbnservice.wso?WSDL' 
        transport:(SOAP::SptHTTPClient).

    WSDLPortData
        onDefinitions:defs 
        portName: 'abc'.
"
! !

!WSDLPortData class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	WarnQName := 1.
	WarnSoapNS := 2
! !

!WSDLPortData class methodsFor:'instance creation'!

onDefinitions: aWSDLDefinitions portName: aString
	^self onDefinitions: aWSDLDefinitions
		portQName: (aString isNil ifTrue: [] ifFalse: [aWSDLDefinitions qualifiedPortName: aString])
!

onDefinitions: aWSDLDefinitions portQName: aXeQName
	| instance |
	instance := self new definitions: aWSDLDefinitions.
	instance setPortQName: aXeQName.
	^instance
! !

!WSDLPortData methodsFor:'accessing'!

namespace
	^#''
!

wsdlOperationNamed: aString
	| wsdlOps |
	wsdlOps := wsdlOperations select: [:each | each name = aString].
	wsdlOps isEmpty ifTrue: [self errorNotFound: aString].
	wsdlOps size > 1 ifTrue: [self error: 'Multiple operations are named: ' , aString].
	^wsdlOps first
! !

!WSDLPortData methodsFor:'accessing-binding'!

styleFor: bindingOpNode
	"Answer aString or nil. Can be either rpc, document or nil. First check the operation node, if not present check for the
	default in the binding node"

	| soapOpNode style |
	soapOpNode := bindingOpNode 
				detectChildNode: 'operation'
				in: XeNamespace soapWSDL
				ifNone: [].
	soapOpNode notNil ifTrue: [style := soapOpNode getAttribute: 'style'].
	^style notNil ifTrue: [style] ifFalse: [self bindingStyle]
! !

!WSDLPortData methodsFor:'accessing-binding-private'!

bindingNode
	^bindingNode
!

bindingOperationNodes
	^self bindingNode selectChildNodes: 'operation' in: nil
!

bindingStyle
	^self soapBindingNode getAttribute: 'style'
!

bindingTransport
	^self soapBindingNode getAttribute: 'transport'
!

findSoapBindingNode
	^self bindingNode 
		detectChildNode: 'binding'
		in: XeNamespace soapWSDL
		ifNone: [self guessSoapBindingElement]
!

guessSoapBindingElement
	"Private - these guess methods were necessary because a major WSDL toolkit was deployed which used incorrect namespaces"

	| answer |
	answer := self bindingNode 
				detectChildNode: 'binding'
				in: #'http://schemas.xmlsoap.org/wsdl/soap'
				ifNone: [self error: 'Unable to find the soap:binding element'].
	self warnSoapNS.
	^answer
!

soapBindingNode
	soapBindingNode isNil ifTrue: [soapBindingNode := self findSoapBindingNode].
	^soapBindingNode
! !

!WSDLPortData methodsFor:'accessing-port'!

portDocumentation
	| docNode |
	docNode := self portNode 
				detectChildNode: 'documentation'
				in: nil
				ifNone: [^''].
	^docNode text
!

portNode
	^portNode
! !

!WSDLPortData methodsFor:'accessing-port type'!

portTypeNode
	^portTypeNode
! !

!WSDLPortData methodsFor:'accessing-port type-private'!

portTypeOperationNodes
	^self portTypeNode selectChildNodes: 'operation' in: nil
! !

!WSDLPortData methodsFor:'accessing-port-private'!

addressLocation
	"Answer the url of the ports endpoint."

	^self soapAddressNode getAttribute: 'location'
!

guessSoapAddressNode
	"Private - these guess methods were necessary because a major WSDL toolkit was deployed which used incorrect namespaces"

	| answer |
	answer := self portNode 
				detectChildNode: 'address'
				in: #'http://schemas.xmlsoap.org/wsdl/soap'
				ifNone: [self error: 'Unable to find the soap:address element'].
	self warnSoapNS.
	^answer
!

portQName
	^portQName
!

portQName: aXeName
	portQName := aXeName
!

soapAddressNode
	^self portNode 
		detectChildNode: 'address'
		in: XeNamespace soapWSDL
		ifNone: [self guessSoapAddressNode]
! !

!WSDLPortData methodsFor:'accessing-private'!

report
	self addReportElements.
	^report
!

singlePortBinding
	self warnQName.
	^self definitions singlePortBinding
!

warnQName
	 report warnQName
!

warnSoapNS
	report warnSoapNS
!

wsdlOperationNames
	^wsdlOperations collect: [:each | each name]
!

wsdlOperations
	^wsdlOperations
! !

!WSDLPortData methodsFor:'accessing-service'!

defaultPortQName
    ^ [
        self definitions supportedPortQNames first
    ] on:Error do:[:e | 
        UserPreferences current soapErrorDebugging ifTrue:[ e reject ].
        self warnQName.
        self definitions singleSupportedPortQName
    ]

    "Modified (format): / 12-07-2011 / 17:49:32 / cg"
!

serviceDocumentation
	| docNode |
	docNode := self portNode parentNode 
				detectChildNode: 'documentation'
				in: nil
				ifNone: [^''].
	^docNode text
!

serviceQName
	^self portNode parentNode tQValueForName
! !

!WSDLPortData methodsFor:'accessing-service-private'!

definitions
	^definitions
!

definitions: aWSDLDefinitions
	definitions := aWSDLDefinitions
! !

!WSDLPortData methodsFor:'enumerating'!

operationNodesDo: aThreeArgBlock
        "Enumerate over the operation nodes. 
        Each operation has two nodes, one as a binding child, the other as a portType child.
        The args to the block are; name, portType/operation node binding/operation node"

        | portOpNodes bindingOpNodes |
        portOpNodes := self portTypeOperationNodes.
        "/ cg: changed to Dolphin::LookupTable
        "/ bindingOpNodes := LookupTable new.
        bindingOpNodes := Dolphin::LookupTable new.
        self bindingOperationNodes 
                do: [:each | bindingOpNodes at: (each getAttribute: 'name') put: each].
        portOpNodes do: 
                        [:p | 
                        | pName bNode |
                        pName := p getAttribute: 'name'.
                        bNode := bindingOpNodes at: pName ifAbsent: [].
                        bNode notNil 
                                ifTrue: 
                                        [aThreeArgBlock 
                                                value: pName
                                                value: p
                                                value: bNode]]
! !

!WSDLPortData methodsFor:'enumerating-private'!

parseOperationNodes
	| coll |
	coll := OrderedCollection new.
	self operationNodesDo: 
			[:opName :po :bo | 
			coll add: (WSDLOperation new 
						setName: opName
						wsdlPort: self
						portTypeOpNode: po
						bindingOpNode: bo)].
	^coll
! !

!WSDLPortData methodsFor:'initializing'!

setDefinitionNodes
	portNode := self definitions portNamed: self portQName.
	bindingNode := self definitions bindingNamed: (portNode qValueFor: 'binding'
						ifNone: [self error: 'Could not locate binding attribute']).
	"ifAbsent: [self singlePortBinding]."
	portTypeNode := self definitions portTypeNamed: (bindingNode qValueFor: 'type'
						ifNone: [self error: 'Could not locate type attribute'])
!

setPortQName: aXeQNameOrNil
	report := WSDLParseReport new.
	portQName := aXeQNameOrNil isNil ifTrue: [self defaultPortQName] ifFalse: [aXeQNameOrNil].
	self setDefinitionNodes.
	wsdlOperations := self parseOperationNodes
! !

!WSDLPortData methodsFor:'reporting-private'!

addReportElements
        report
                atInfo: 'Service Name' put: self serviceQName;
                atInfo: 'Service Documentation' put: self serviceDocumentation printString "displayString";
                atInfo: 'All Ports' put: self definitions allPortNames asArray;
                atInfo: 'Address Location' put: self addressLocation printString "displayString";
                atInfo: 'Port Name' put: self portQName;
                atInfo: 'Port Documentation' put: self portDocumentation printString "displayString";
                atInfo: 'Operations' put: self wsdlOperationNames asArray;
                atInfo: 'OperationNodes' put: self wsdlOperations asArray
! !

!WSDLPortData methodsFor:'spray creation'!

createPort
        "answer a SpmdPort"

        | md |
        md := SOAP::SpmdPort new.
        md
                name: self portQName localName;
                documentation: self portDocumentation;
                transportUri: self bindingTransport;
                operations: (wsdlOperations collect: [:each | each createSpmdOperation]).
        ^md
! !

!WSDLPortData class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLPortData.st,v 1.12 2012/03/16 18:26:34 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/WSDLPortData.st,v 1.12 2012/03/16 18:26:34 cg Exp $'
! !

WSDLPortData initialize!
