"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SprayAuthTests subclass:#SpAuthClientAuthTest
	instanceVariableNames:'service clientAuth'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpAuthClientAuthTest methodsFor:'public'!

addUidPwd1
	self authorizer addUid: 'user' pwd: 'dolphin'
!

assertAnswerFault
	self assert: clientAuth answerStruct typeName 
				= ('AuthSC_Fault_t' ns: #'http://soap-authentication.org/digest/2001/08/').
	self assert: clientAuth isFail
!

assertAnswerSuccess
	self assert: clientAuth answerStruct typeName 
				= ('AuthSC_t' ns: #'http://soap-authentication.org/digest/2001/08/').
	self deny: clientAuth isFail
!

authorizer
	^service authorizer authorizer
!

beIncorrect
	clientAuth := self clientAuthFor: #getIncorrectXmlString
!

beInvalidID
	clientAuth := self clientAuthFor: #getInvalidXmlString
!

beInvalidRealm
	clientAuth := self clientAuthFor: #getInvalidRealmXmlString
!

beSuccessNoClientChallenge
	clientAuth := self clientAuthFor: #getSuccessNoChallengeXmlString
!

beSuccessWithClientChallenge
	clientAuth := self clientAuthFor: #getSuccessWithChallengeXmlString
!

clientAuthFor: aSelector
	| instance |
	instance := SpSprayTestAuthObjects new clientAuthIn: service typespace for: aSelector.
	instance authorizer: self authorizer.
	^instance
	 
! !

!SpAuthClientAuthTest methodsFor:'set up'!

setUp
	super setUp.
	service := SpAuthTestService new.
	
! !

!SpAuthClientAuthTest methodsFor:'tests'!

testIncorrect
	self beIncorrect.
	clientAuth authorize.
	self assertAnswerFault.
	self assert: (clientAuth answerStruct at: #Status) = 'E02'.
	self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
	self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.



!

testInvalidID
	self beInvalidID.
	clientAuth authorize.
	self assertAnswerFault.
	self assert: (clientAuth answerStruct at: #Status) = 'E04'.
	self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
	self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.



!

testInvalidRealm
	self beInvalidRealm.
	clientAuth authorize.
	self assertAnswerFault.
	self assert: (clientAuth answerStruct at: #Status) = 'E04'.
	self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
	self assert: (clientAuth answerStruct at: #Realm) ='TestRealm'.



!

testSuccessNoClientChallenge
	self beSuccessNoClientChallenge.
	clientAuth authorize.
	self assertAnswerSuccess.
	self assert: (clientAuth answerStruct at: #Status) = 'S01'.
	self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
	self assert: (clientAuth answerStruct at: #CNonce) isNil.
	self assert: (clientAuth answerStruct at: #RAuth) isNil.


!

testSuccessWithClientChallenge
	self beSuccessWithClientChallenge.
	clientAuth authorize.
	self assertAnswerSuccess.
	self assert: (clientAuth answerStruct at: #Status) = 'S01'.
	self assert: (clientAuth answerStruct at: #Nonce) printSchemaHexString = self authorizer generateNonce.
	self assert: (clientAuth answerStruct at: #CNonce) printSchemaHexString = 'CEA8A3DB3C06C7970A61B92AE9560A08'.
	self assert: (clientAuth answerStruct at: #RAuth) = #[70 186 66 205 157 40 241 34 16 136 209 240 4 116 191 236].


! !

!SpAuthClientAuthTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpAuthClientAuthTest.st,v 1.1 2009/10/02 19:40:12 cg Exp $'
! !
