"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SpmdTests subclass:#SpmdOperationTest
	instanceVariableNames:'operation typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpmdOperationTest methodsFor:'public'!

assertIsDocEncoded: aXeNode
	| opWNode |
	opWNode := aXeNode firstChild firstChild.
	self assert: (opWNode getAttribute: 'style') = 'document'.
	self assert: opWNode nextSibling localName = 'input'.
	self assert: (opWNode nextSibling firstChild getAttribute: 'use') = 'encoded'.
	self assert: opWNode nextSibling nextSibling localName = 'output'.
	self assert: (opWNode nextSibling nextSibling firstChild getAttribute: 'use') = 'encoded'
!

assertIsDocLiteral: domNode 
	| opWNode |
	opWNode := domNode firstChild firstChild.
	self assert: (opWNode getAttribute: 'style') = 'document'.
	self assert: opWNode nextSibling localName = 'input'.
	self assert: (opWNode nextSibling firstChild getAttribute: 'use') = 'literal'.
	self assert: opWNode nextSibling nextSibling localName = 'output'.
	self assert: (opWNode nextSibling nextSibling firstChild getAttribute: 'use') = 'literal'
!

assertIsRpcEncoded: domNode 
	| opWNode |
	opWNode := domNode firstChild firstChild.
	self assert: (opWNode getAttribute: 'style') = 'rpc'.
	self assert: opWNode nextSibling localName = 'input'.
	self assert: (opWNode nextSibling firstChild getAttribute: 'use') = 'encoded'.
	self assert: opWNode nextSibling nextSibling localName = 'output'.
	self assert: (opWNode nextSibling nextSibling firstChild getAttribute: 'use') = 'encoded'
!

assertIsRpcLiteral: domNode 
	| opWNode |
	opWNode := domNode firstChild firstChild.
	self assert: (opWNode getAttribute: 'style') = 'rpc'.
	self assert: opWNode nextSibling localName = 'input'.
	self assert: (opWNode nextSibling firstChild getAttribute: 'use') = 'literal'.
	self assert: opWNode nextSibling nextSibling localName = 'output'.
	self assert: (opWNode nextSibling nextSibling firstChild getAttribute: 'use') = 'literal'
!

beOperationDOCTypedXSD11
	operation := SpmdDOCOperation 
				createEnc: 'docOp'
				namespace: #'needed?'
				selector: #docOp:
				inputParts: (Array with: (SpmdTypePart partName: 'partIn1' typeName: 'xsd:string'))
				outputParts: (Array with: (SpmdTypePart partName: 'partOut1' typeName: 'xsd:int')
						with: (SpmdTypePart partName: 'partOut2' typeName: 'xsd:string')).
	operation soapAction: 'docTestSoapAction'
!

beOperationEncRPCTypedXSD11
	operation := SpmdRPCOperation 
				createEnc: 'TestOp4'
				namespace: #'urn:testOpNS'
				selector: #abc
				inputParts: (Array with: (SpmdTypePart partName: 'aPart' typeName: 'xsd:integer'))
				outputParts: (Array with: (SpmdTypePart partName: 'aPart' typeName: 'xsd:integer')).
	operation soapAction: 'rpcTestSoapAction'
!

beOperationLiteralDOCElement2
	operation := SpmdDOCOperation 
				createLit: 'docOp'
				selector: #docOp:
				inputPart: (SpmdElementPart partName: 'part1'
								elementName: (XeQName soapEnc: 'string'))
				outputPart: (SpmdElementPart partName: 'part1'
								elementName: (XeQName soapEnc: 'string')).
	operation soapAction: 'docTestSoapAction'.
	
!

beOperationLiteralRPCXSD
	operation := SpmdRPCOperation 
				createLit: 'TestOp4'
				selector: #abc
				inputPart: (SpmdTypePart partName: 'aPart' typeName: 'xsd:integer')
				outputPart: (SpmdTypePart partName: 'aPart' typeName: 'xsd:integer').
	operation soapAction: 'this'
!

newSoapedNode
	| node |
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	^node
! !

!SpmdOperationTest methodsFor:'set up'!

setUp
	super setUp.
	typespace := self get2001Typespace
! !

!SpmdOperationTest methodsFor:'tests'!

testAddInputHeaderNamed
	| qName inputHeader |
	operation := testObjects getOperationEncRPCTypedXSD.
	self assert: operation inputHeaders isEmpty.
	operation addInputHeaderDeclared: (qName := self qFoo).
	self assert: operation inputHeaders size = 1.
	inputHeader := operation inputHeaders first.
	self assert: (inputHeader isKindOf: SpmdSoapHeader).
	self assert: inputHeader partNames size = 1.
	self assert: inputHeader message name = 'myOperationHeader1'.
	self assert: inputHeader message parts size = 1.
	self assert: (inputHeader message parts first isKindOf: SpmdElementPart).
	self assert: inputHeader message parts first elementName = qName.
	self assert: inputHeader message parts first partName = operation inputHeaders first partNames first
!

testAddMessageDefinitions
	| node |
	operation := testObjects getOperationEncDOCTypedXSD.
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	self assert: operation allMessages size = 2
!

testAddMessageDefinitionsWithHeaders
	| node  |
	operation := testObjects getOperationWithInputHeaders.
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	operation allMessages do: [:each | each defineMessage: node typespace: typespace].
	self assert: node asParserNode childNodes size = 3.
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="urn:abc" xmlns:ns1="http://www.dolphinharbor.org/spray/class">
<message name="myOperationInput">
<part name="param1" type="xsd:string"/>
<part name="param2" type="xsd:int"/>
</message>
<message name="myOperationOutput">
<part name="param1" type="xsd:string"/>
<part name="param2" type="xsd:int"/>
</message>
<message name="myOperationHeader1">
<part name="part1" element="ns1:SWXMLTestObject"/>
</message></blah>'
!

testAddPortType
	|  node |
	operation := testObjects getOperationEncRPCTypedXSD.
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	operation definePortType: node.
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<operation name="myOperation">
<input message="tns:myOperationInput"/>
<output message="tns:myOperationOutput"/>
</operation>
</blah>'
!

testAllMessages
	operation := testObjects getOperationEncDOCTypedXSD.
	self assert: operation allMessages size = 2.
	self assert: (operation allMessages allSatisfy: [:each | each isKindOf: SpmdMessage]).
	operation := testObjects getOperationWithInputHeaders.
	self assert: operation allMessages size = 3.
	self assert: (operation allMessages allSatisfy: [:each | each isKindOf: SpmdMessage]).
	operation := testObjects getOperationWithMultipleHeaders.
	self assert: operation allMessages size = 5.
	self assert: (operation allMessages allSatisfy: [:each | each isKindOf: SpmdMessage])
!

testBodyCoders
	| coders |
	operation := testObjects getOperationEncRPCTypedXSD.
	coders := operation bodyOutputCodersIn: typespace.
	self assert: coders size = 1.
	self assert: (coders first isKindOf: SpwSoapBCoder).
	coders := operation bodyInputCodersIn: typespace.
	self assert: coders size = 1.
	self assert: (coders first isKindOf: SpwSoapBCoder)
!

testConstructionWithHeader
	operation := testObjects getOperationWithInputHeaders.
	self assert: (operation inputHeaders first isKindOf: SpmdSoapHeader).
	self deny: operation inputHeaders first isEncoded.
	self assert: operation inputHeaders first message name = 'myOperationHeader1'.
	self assert: operation allMessages size = 3
!

testCreateHandlersDOCWrongNumArgs
	operation := testObjects getOperationEncDOCTypedXSD.
	operation inputBody message parts first selector: #myDocSelector1.
	self should: [operation allHandlersIn: typespace] raise: Error
!

testCreateHandlersEncDOCTypedXSD
	| ans handler |
	operation := testObjects getOperationEncDOCTypedXSD.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 2.
	ans do: 
			[:each | 
			self assert: (each isKindOf: SpwDOCHandler).
			self assert: each soapAction = 'myAction'.
			self assert: (each inputDecoder isKindOf: SpwSoapBCoder).
			self assert: (each outputEncoder isKindOf: SpwSoapBCoder)].
	handler := ans first.
	self assert: handler inputDecoder element localName = 'param1'.
	self assert: handler inputDecoder element namespace = #'urn:blah'.
	self assert: handler outputEncoder element localName = 'param1'.
	self assert: handler outputEncoder element namespace = #'urn:blah'.
	self assert: handler selector = #myDocSelector1:.
	handler := ans second.
	self assert: handler inputDecoder element localName = 'param2'.
	self assert: handler inputDecoder element namespace = #'urn:blah'.
	self assert: handler outputEncoder element localName = 'param2'.
	self assert: handler outputEncoder element namespace = #'urn:blah'.
	self assert: handler selector = #myDocSelector2:
!

testCreateHandlersEncRPCTypedXSD
	| ans handler |
	operation := testObjects getOperationEncRPCTypedXSD.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 1.
	handler := ans first.
	self assert: (handler isKindOf: SpwRPCHandler).
	self assert: handler soapAction = 'myAction'.
	self assert: (handler inputDecoder isKindOf: SpwSoapBCoder).
	self assert: (handler outputEncoder isKindOf: SpwSoapBCoder).
	self assert: handler inputDecoder element localName = 'myOperation'.
	self assert: handler inputDecoder element namespace = #'urn:blah'.
	self assert: handler outputEncoder element localName = 'myOperationResponse'.
	self assert: handler outputEncoder element namespace = #'urn:blah'.
	self assert: handler selector = #mySelector:with:
!

testCreateHandlersLiteralDOCElement
	| ans handler |
	typespace := testObjects getWSDLSpecTypespace.
	operation := testObjects getOperationLiteralDOCElement.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 1.
	handler := ans first.
	self assert: (handler isKindOf: SpwDOCHandler).
	handler soapAction.	"??????"
	self assert: (handler inputDecoder isKindOf: SpwSoapBCoder).
	self assert: (handler outputEncoder isKindOf: SpwSoapBCoder).
	self assert: handler inputDecoder element localName = 'TradePriceRequest'.
	self assert: handler inputDecoder element namespace = #'http://example.com/stockquote.xsd'.
	self assert: handler outputEncoder element localName = 'TradePrice'.
	self assert: handler outputEncoder element namespace = #'http://example.com/stockquote.xsd'.
	self assert: handler selector = #myLiteralDocSelector1:
!

testCreateHandlersWithHeaders
	| ans handler |
	operation := testObjects getOperationWithInputHeaders.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 2.
	handler := ans last.
	self assert: handler inputDecoder element localName = 'SWXMLTestObject'.
	self assert: handler inputDecoder element namespace = XeNamespace sprayClass.
	self assert: handler outputEncoder isNil.
	self assert: handler selector = #myHeaderSelector1:ex:
!

testCreateHandlersWithLinkedHeaders
	| ans handler |
	operation := testObjects getOperationWithLinkedHeaders.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 2.
	handler := ans second.
	self assert: handler inputDecoder element localName = 'SWXMLTestObject'.
	self assert: handler inputDecoder element namespace = XeNamespace sprayClass.
	self assert: handler outputEncoder element localName = 'SWXMLTestObject'.
	self assert: handler outputEncoder element namespace = XeNamespace sprayClass.
	self assert: handler selector = #myHeaderSelector1:ex:
!

testCreateHandlersWithMultipleHeaders
	| ans |
	operation := testObjects getOperationWithMultipleHeaders.
	ans := operation allHandlersIn: typespace.
	self assert: ans size = 3.
	
!

testDefineBindingEncDOC
	"?"
	| node xml domNode |
	self beOperationDOCTypedXSD11.
	node := self newSoapedNode.
	operation defineBinding: node.
	domNode := node asParserNode.
	self assertIsDocEncoded: domNode.
	self assertXML: (xml := node ownerDocXML)
		isSameAs: 
'<?xml version="1.0"?><blah xmlns:soap="urn:abc"><operation name="docOp"><soap:operation style="document" soapAction="docTestSoapAction"/>
<input><soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="needed?"/></input>
<output><soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="needed?"/></output></operation></blah>'
!

testDefineBindingEncRPC
	| node xml domNode |
	self beOperationEncRPCTypedXSD11.
	node := self newSoapedNode.
	operation defineBinding: node.
	domNode := node asParserNode.
	self assertIsRpcEncoded: domNode.
	self assertXML: (xml := node ownerDocXML)
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="TestOp4">
<soap:operation style="rpc" soapAction="rpcTestSoapAction"/>
<input><soap:body use="encoded" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" namespace="urn:testOpNS"/></input>
<output><soap:body use="encoded" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" namespace="urn:testOpNS"/></output>
</operation></blah>'
!

testDefineBindingEncRPCNullNamespace
	| node xml domNode |
	operation := testObjects getOperationEncRPCTypedXSD.
	operation inputBody namespace: #''.
	operation outputBody namespace: #''.
	operation soapAction: 'rpcTestSoapAction'.
	node := self newSoapedNode.
	operation defineBinding: node.
	domNode := node asParserNode.
	self assertIsRpcEncoded: domNode.
	self assert: (domNode firstChild firstChild nextSibling firstChild getAttribute: 'namespace') = ''.
	self assertXML: (xml := node ownerDocXML)
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="myOperation">
<soap:operation style="rpc" soapAction="rpcTestSoapAction"/>
<input><soap:body use="encoded" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" namespace=""/></input>
<output><soap:body use="encoded" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" namespace=""/></output>
</operation></blah>'
!

testDefineBindingLiteralDOC
	| node xml domNode |
	self beOperationLiteralDOCElement2.
	node := self newSoapedNode.
	operation defineBinding: node.
	domNode := node asParserNode.
	self assertIsDocLiteral: domNode.
	self assertXML: (xml := node ownerDocXML)
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="docOp">
<soap:operation style="document" soapAction="docTestSoapAction"/>
<input><soap:body use="literal"/></input>
<output><soap:body use="literal"/>
</output></operation></blah>'
!

testDefineBindingLiteralRPC
	| node xml domNode |
	self beOperationLiteralRPCXSD.
	node := self newSoapedNode.
	operation defineBinding: node.
	domNode := node asParserNode.
	self assertIsRpcLiteral: domNode.
	self assertXML: (xml := node ownerDocXML)
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="TestOp4">
<soap:operation style="rpc" soapAction="this"/>
<input><soap:body use="literal" /></input>
<output><soap:body use="literal" /></output>
</operation></blah>'
!

testDefineBindingWithHeaders
	| node |
	operation := testObjects getOperationWithInputHeaders.
	operation soapAction: 'this'.
	node := self newSoapedNode.
	operation defineBinding: node.
	self assertXML: node ownerDocXML
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="myOperation"><soap:operation style="rpc" soapAction="this"/>
<input>
<soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="urn:blah"/>
<soap:header message="tns:myOperationHeader1" part="part1" use="literal"/>
</input>
<output><soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="urn:blah"/></output></operation></blah>'
!

testDefineBindingWithMultipleHeaders
	| node |
	operation := testObjects getOperationWithMultipleHeaders.
	operation soapAction: 'this'.
	node := self newSoapedNode.
	operation defineBinding: node.
	self assertXML: node ownerDocXML
		isSameAs: 
'<?xml version="1.0"?>
<blah xmlns:soap="urn:abc">
<operation name="myOperation"><soap:operation style="rpc" soapAction="this"/>
<input>
<soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="urn:blah"/>
<soap:header message="tns:myOperationHeader1" part="part1" use="literal"/>
<soap:header message="tns:myOperationHeader2" part="part1" use="literal"/>
</input>
<output>
<soap:body encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" use="encoded" namespace="urn:blah"/>
<soap:header message="tns:myOperationHeader3" part="part1" use="literal"/>

</output></operation></blah>'
!

testNames
	operation := testObjects getOperationEncRPCTypedXSD.
	self assert: operation name = 'myOperation'.
	"self assert: operation defaultInputName = 'myOperation'.
	self assert: operation defaultOutputName = 'myOperationResponse'."
	"self assert: op inputName = 'myOperation'.
	self assert: op outputName = 'myOperationResponse'."


	"test the message names (used for WSDL internals)"
	self assert: operation inputBody message messageName = 'myOperationInput'.
	self assert: operation outputBody message messageName = 'myOperationOutput'
!

testRenameMessages
	operation := testObjects getOperationEncRPCTypedXSD.
	self assert: operation inputBody message messageName = 'myOperationInput'.
	self assert: operation outputBody message messageName = 'myOperationOutput'.
	operation renameMessages.
	self assert: operation inputBody message messageName = 'myOperationInput'.
	self assert: operation outputBody message messageName = 'myOperationOutput'.
	operation name: 'newName'.
	operation renameMessages.
	self assert: operation inputBody message messageName = 'newNameInput'.
	self assert: operation outputBody message messageName = 'newNameOutput'
!

testSignatureEncRPCTypedXSD
	self assert: testObjects getOperationEncRPCTypedXSD spraySignature = 'myOperation ((param1 xsd:string) (param2 xsd:int))'
!

testSignatureEncRPCVoid
	operation := SpmdRPCOperation
		createEnc: 'myOperation'
		namespace: '#urn:blah'
		selector: #void 
		inputParts: #()
		outputParts: #(). 
	self assert: operation  spraySignature = 'myOperation ()'
!

testSignatureLiteralDOC
	typespace := testObjects getWSDLSpecTypespace.
	operation := testObjects getOperationLiteralDOCElement.
	self assert: operation spraySignature = 'GetLastTradePrice ((part1 TradePriceRequest))'
! !

!SpmdOperationTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpmdOperationTest.st,v 1.1 2009/10/02 19:43:19 cg Exp $'
! !
