"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SpmdTests subclass:#SpmdPortTest
	instanceVariableNames:'port typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpmdPortTest methodsFor:'public'!

newSoapedNode
	| node |
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	^node
! !

!SpmdPortTest methodsFor:'set up'!

setUp
	super setUp.
	port := testObjects getMixedPort.
	typespace := self get2001Typespace
! !

!SpmdPortTest methodsFor:'tests'!

testAddDuplicate
	self should: [port addOperation: testObjects getOperationEncDOCTypedXSD] raise: Error
!

testAllHandlersIn
	self assert: (port allHandlersIn: typespace) size = 3.
	(port allHandlersIn: typespace) do: [:each | self assert: (each isKindOf: SpwSoapBHandler)]
!

testAllMessages
	(port allMessages) do: [ :each |
		self assert: (each isKindOf: SpmdMessage)]
!

testDefineBinding
	| node |
	node := self newSoapedNode.
	port operations: OrderedCollection new.
	port defineBinding: node private: #().
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<binding
  name="samplePortBinding"
  type="tns:samplePortPortType">
  <soap:binding 
    transport="http://schemas.xmlsoap.org/soap/http"/>
</binding></blah>'
!

testDefineBindingTransportUri
	| node |
	node := self newSoapedNode.
	port operations: OrderedCollection new.
	port transportUri: 'http://myTransport'.
	port defineBinding: node private: #().
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<binding
  name="samplePortBinding"
  type="tns:samplePortPortType">
  <soap:binding 
    transport="http://myTransport"/>
</binding></blah>'
!

testDefinePort
	| node |
	node := self newSoapedNode.
	port operations: OrderedCollection new.
	port definePort: node at: 'http://sample.org'.
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<port
  name="samplePort"
  binding="tns:samplePortBinding">
  <soap:address location="http://sample.org"/>
</port></blah>'
!

testDefinePortType
	| node |
	node := self newSoapedNode.
	port operations: OrderedCollection new.
	port definePortType: node private: #().
	self assertXML: node xml
		isSameAs: '<blah xmlns:soap="urn:abc"><portType name="samplePortPortType"/></blah>'
!

testFormatSoapActionWith
	#(( '/%1' '/myOperation') ('urn:samplePort/%1' 'urn:samplePort/myOperation')
	('urn:samplePort' 'urn:samplePort') ('%1' 'myOperation'))
		do: [ :each |
			port formatSoapActionWith: each first.
			self assert: (port operationNamed: 'myOperation') soapAction = each second].



!

testOperationNamed
	self should: [port operationNamed: 'notAnOperation'] raise: Error
!

testOperationNames
	self assert: (port operationNames size = 2)
!

testRemoveOperation
	self assert: (port operations size = 2).
	port removeOperation: port operations first.
	self assert: (port operations size = 1).
!

testSoapAction
	self deny: (port operations allSatisfy: [ :each | each soapAction = 'newAction']).
	port soapAction: 'newAction'.
	self assert: (port operations allSatisfy: [ :each | each soapAction = 'newAction']).
! !

!SpmdPortTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpmdPortTest.st,v 1.1 2009/10/02 19:40:50 cg Exp $'
! !
