"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SpmdTests subclass:#SpmdSoapBodyTest
	instanceVariableNames:'bodyBinding typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpmdSoapBodyTest methodsFor:'public'!

assertBodyBindingPartNames
	bodyBinding partNames: #('param1').
	self assert: (bodyBinding docHandlersIn: typespace) size = 1.
	bodyBinding partNames: #('param1' 'param2').
	self assert: (bodyBinding docHandlersIn: typespace) size = 2.
	bodyBinding partNames: nil.
	self assert: (bodyBinding docHandlersIn: typespace) size = 2.
	self should: [bodyBinding partNames: #()] raise: Error
!

beEncDOC
	bodyBinding := SpmdSoapBody soapEncMessage: testObjects getMessage2TypedXSD ns: #'urn:blah'.
	bodyBinding message name: 'myOperationInput'.
	bodyBinding message parts first selector: #myDocSelector1:.
	bodyBinding message parts second selector: #myDocSelector2:.
	bodyBinding beDOCHandler
!

beEncRPC
	bodyBinding := SpmdSoapBody soapEncMessage: testObjects getMessage2TypedXSD ns: #'urn:blah'.
	bodyBinding operationName: 'myOperation'.
	bodyBinding message name: 'myOperationInput'.
	bodyBinding selector: #abc:def:
!

beLiteralDOC
	bodyBinding := SpmdSoapBody literalMessage: testObjects getMessageElementComplex2Children.
	bodyBinding message name: 'myMessage2'.
	bodyBinding beDOCHandler.
	bodyBinding message parts first selector: #myBodySelector:.
	bodyBinding message parts second selector: #myOtherSelector:.
	bodyBinding partNames: #('body')
! !

!SpmdSoapBodyTest methodsFor:'set up'!

setUp
	super setUp.
	typespace := self get2001Typespace
! !

!SpmdSoapBodyTest methodsFor:'tests'!

testBeLiteral
	self beEncRPC.
	bodyBinding beLiteralStyle.
	self deny: bodyBinding isEncoded.
	self assert: bodyBinding encStyleUri = #''
!

testBeSoapEnc
	self beEncRPC.
	bodyBinding beSoapEncStyle.
	self assert: bodyBinding isEncoded.
	self assert: bodyBinding encStyleUri = XeNamespace soapEnc
!

testEncDOC
	| ans |
	self beEncDOC.
	bodyBinding partNames: nil.
	ans := bodyBinding docHandlersIn: typespace.
	self assert: ans size = 2.
	self assert: ans first inputDecoder element localName = 'param1'.
	self assert: ans second inputDecoder element localName = 'param2'
!

testEncDOCPartNames
	self beEncDOC.
	bodyBinding partNames: #('param1').
	self assert: (bodyBinding docHandlersIn: typespace) size = 1.
	bodyBinding partNames: #('param1' 'param2').
	self assert: (bodyBinding docHandlersIn: typespace) size = 2.
	bodyBinding partNames: nil.
	self assert: (bodyBinding docHandlersIn: typespace) size = 2.
	self should: [bodyBinding partNames: #()] raise: Error
!

testEncRPC
	| ans |
	self beEncRPC.
	bodyBinding partNames: nil.
	bodyBinding selector: #abc:def:.
	ans := bodyBinding rpcHandlerIn: typespace.
	self assert: ans inputDecoder element childElements size = 2.
	self assert: ans inputDecoder element childElements first localName = 'param1'.
	self assert: ans inputDecoder element childElements second localName = 'param2'
!

testEncRPCPartNames
	self beEncRPC.
	bodyBinding partNames: #('param1').
	bodyBinding selector: #abc:.
	self assert: (bodyBinding rpcHandlerIn: typespace) inputDecoder element childElements size = 1.
	bodyBinding selector: #abc:def:.
	bodyBinding partNames: #('param1' 'param2').
	self assert: (bodyBinding rpcHandlerIn: typespace) inputDecoder element childElements size = 2.
	bodyBinding partNames: nil.
	self assert: (bodyBinding rpcHandlerIn: typespace) inputDecoder element childElements size = 2.
	self should: [bodyBinding partNames: #()] raise: Error
!

testEncRPCPartNamesWrongArgs
	self beEncRPC.
	bodyBinding partNames: #('param1').
	self should: [bodyBinding rpcHandlerIn: typespace] raise: Error
!

testLiteralDOCDifNS
	| ans |
	self beLiteralDOC.
	bodyBinding namespace: #'urn:different'.
	ans := bodyBinding docHandlersIn: typespace.
	self assert: ans size = 1.
	self assert: ans first inputDecoder element localName = 'SWXMLTestObject'.

	"Note: specified namespace should not replace"
	self deny: ans first inputDecoder element namespace = #'urn:different'.
	self assert: ans first inputDecoder element namespace = #'http://www.dolphinharbor.org/spray/class'.

	
!

testLiteralDOCPartNames
	| ans |
	self beLiteralDOC.
	ans := bodyBinding docHandlersIn: typespace.
	self assert: ans size = 1.
	self assert: ans first inputDecoder element localName = 'SWXMLTestObject'.
	bodyBinding partNames: #('body' 'other').
	ans := bodyBinding docHandlersIn: typespace.
	self assert: ans size = 2.
	self assert: ans first inputDecoder element localName = 'SWXMLTestObject'.
	self assert: ans second inputDecoder element localName = 'SWXMLTestComposite'.
	bodyBinding partNames: nil.
	ans := bodyBinding docHandlersIn: typespace.
	self assert: ans size = 2.
	self assert: ans first inputDecoder element localName = 'SWXMLTestObject'.
	self assert: ans second inputDecoder element localName = 'SWXMLTestComposite'
!

testRPCHandlerInWrongArgs
	self beEncRPC.
	bodyBinding partNames: #('param1').
	bodyBinding selector: #abc:def:hij:.
	self should: [bodyBinding rpcHandlerIn: typespace] raise: Error
!

testRPCHandlers
	| ans |
	self beEncRPC.
	ans := bodyBinding rpcHandlerIn: typespace.
	self assert: (ans isKindOf: SpwSoapBHandler).
	self assert: (ans inputDecoder isKindOf: SpwSoapBCoder)
!

testRPCHandlersLiteral
	self beEncRPC.
	bodyBinding beLiteralStyle.
	"self should: [bodyBinding rpcHandlerIn: typespace] raise: Error"
!

testRPCInputCoder
	| ans |
	self beEncRPC.
	ans := bodyBinding rpcInputCoderIn: typespace.
	self assert: (ans isKindOf: SpwSoapBCoder).
	self assert: ans elementQName = (self blahQ: 'myOperation')
!

testRPCOutputCoder
	| ans |
	self beEncRPC.
	ans := bodyBinding rpcOutputCoderIn: typespace.
	self assert: (ans isKindOf: SpwSoapBCoder).
	self assert: ans elementQName = (self blahQ: 'myOperationResponse')
!

testSourceFormat
	self beEncDOC.
	self assert: bodyBinding sourceFormat = #(#('param1' 'xsd:string') #('param2' 'xsd:int'))
! !

!SpmdSoapBodyTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpmdSoapBodyTest.st,v 1.1 2009/10/02 19:44:51 cg Exp $'
! !
