"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SpmdTests subclass:#SpmdSoapHeaderTest
	instanceVariableNames:'headerBinding typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpmdSoapHeaderTest methodsFor:'public'!

beEncoded
	headerBinding := SpmdSoapHeader 
				soapEncTypeName: (XePName xsd: 'string')
				partName: 'echoMeStringRequest'
				selector: #echoMeString:exchange:
				ns: #'http://soapinterop.org/echoheader/'
!

beLiteral
	headerBinding := SpmdSoapHeader literalMessage: testObjects getMessageElementComplex2Children.
	headerBinding message name: 'myMessage2'.
	headerBinding message parts first selector: #myDocHSelector1:ex:.
	headerBinding message parts second selector: #myDocHSelector2:ex:.
	headerBinding partNames: #('other')
! !

!SpmdSoapHeaderTest methodsFor:'set up'!

setUp
	super setUp.
	typespace := self get2001Typespace
! !

!SpmdSoapHeaderTest methodsFor:'tests'!

testConstruction
	| ans |
	self beLiteral.
	ans := headerBinding docHandlersIn: typespace.
	self assert: ans size = 1.
	self assert: (ans first isKindOf: SpwSoapBHandler).
	self assert: (ans first inputDecoder isKindOf: SpwSoapBCoder).
	self assert: ans first selector = #myDocHSelector2:ex:
!

testDefineBindingEncoded
	| node |
	self beEncoded.
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	headerBinding defineBinding: node.
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<soap:header
  encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
  use="encoded"
  part="echoMeStringRequest"
  namespace="http://soapinterop.org/echoheader/"
  message="tns:echoMeStringRequestHeader"/>
</blah>'
!

testDefineBindingLiteral
	| node |
	self beLiteral.
	node := self newNode: 'blah'.
	node addNSPrefix: 'soap' to: #'urn:abc'.
	headerBinding defineBinding: node.
	self assertXML: node xml
		isSameAs: 
'<blah xmlns:soap="urn:abc">
<soap:header
  use="literal"
  part="other"
  message="tns:myMessage2"/>
</blah>'
!

testIsOwned
	self beLiteral.
	headerBinding operationName: 'myDOCOperation'.
	headerBinding renameMessageAppending: 'Header'.
	self assert: headerBinding message name = 'myDOCOperationHeader'.
	headerBinding beNotOwned.
	headerBinding renameMessageAppending: 'Header1'.
	self assert: headerBinding message name = 'myDOCOperationHeader'.
	headerBinding beOwned.
	headerBinding renameMessageAppending: 'Header1'.
	self assert: headerBinding message name = 'myDOCOperationHeader1'
!

testLiteralDeclaredConstruction
	headerBinding := SpmdSoapHeader literalDeclared: self qFoo.
	self deny: headerBinding isEncoded.
	self assert: headerBinding partNames = #('part1').
	self assert: headerBinding message name = 'fooHeader'.
	self assert: headerBinding message parts size = 1.
	self assert: headerBinding message parts first selector = #foo:exchange:
!

testSoapEncConstruction
	self beEncoded.
	self assert: headerBinding isEncoded.
	self assert: headerBinding partNames = #('echoMeStringRequest').
	self assert: headerBinding message name = 'echoMeStringRequestHeader'.
	self assert: headerBinding message parts size = 1.
	self assert: headerBinding message parts first selector = #echoMeString:exchange:
! !

!SpmdSoapHeaderTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpmdSoapHeaderTest.st,v 1.1 2009/10/02 19:43:04 cg Exp $'
! !
