"{ Package: 'stx:goodies/soap/spray/tests' }"

"{ NameSpace: SOAP }"

SprayTests subclass:#SprayFunctionalTests
	instanceVariableNames:'service transport server client responseXML responseEnvelope
		answer soapSend'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-TestCases'
!


!SprayFunctionalTests methodsFor:'accessing'!

getLocalSprayClient
	| localClient |
	localClient := self getService createClientTo: 'http://blah'.
	localClient httpRequestSoapBinding: self getLocalTransport.
	"localClient immediateDestination: 'http://blah'."
	^localClient


!

getLocalSprayServer
	^service createServer
!

getLocalTransport
	^SptHTTPLocalTransport new server: server
!

getLocalWSDLClient
	| wsdlService wsdlClient |
	wsdlService := self getLocalWSDLService.
	wsdlClient := wsdlService createClient.
	wsdlClient httpRequestSoapBinding: self getLocalTransport.
	^wsdlClient
!

getLocalWSDLService
	| wsdlDefs wsdlService |
	wsdlDefs := WSDLDefinitions onDocument: server wsdl asParserNode transport: nil.
	wsdlService := (SprayWSDLService onDefinitions: wsdlDefs) setPortNamed: nil.
	^wsdlService
!

getRPCWrapperNode
	^self responseEnvelope bodyNode firstChild 
!

getReturnNode
	^self responseEnvelope bodyNode firstChild firstChild
!

getService
	^self subclassResponsibility
!

makeBodyRequest: aString
	self makeRequest: (self getEnvStringBody: aString)
!

makeRequest: aString 
	| soapOutEnv exchange |
	exchange := SptRespondingExchange onXmlString: aString xsiNS: server xsiNamespace.
	server soapNode process: exchange.
	soapOutEnv := exchange out.
	responseXML := soapOutEnv xml
!

responseEnvelope
	responseEnvelope isNil ifTrue: [responseEnvelope := SpwSoapInEnvelope fromXmlString: responseXML].
	^responseEnvelope
!

responseNode
	^self rootNodeFor: responseXML
!

responseNodesByTagName: aString 
	^self responseNode selectAllNodes: aString in: nil
!

soapNode
	^server soapNode
! !

!SprayFunctionalTests methodsFor:'asserting'!

assertAnswerXMLSameAs: aString 
	self assertXML: responseXML isSameAs: aString
!

assertFaultResponse
	self assert: self responseEnvelope isFault
!

assertSoapSendXML: xmlString
	self assertXML: soapSend soapOutEnvelope xml isSameAs: xmlString
!

denyFaultResponse
	self deny: self responseEnvelope isFault
! !

!SprayFunctionalTests methodsFor:'set up'!

setUp
	super setUp.
	service := self getService.
	server := self getLocalSprayServer.
	client := self getLocalSprayClient.
! !

!SprayFunctionalTests methodsFor:'tests'!

testWSDLServiceValid
	| wsdlService |
	wsdlService := self getLocalWSDLService.
	wsdlService checkForInvalidOperations.
	self deny: wsdlService hasInvalidOperations
	
! !

!SprayFunctionalTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SprayFunctionalTests.st,v 1.2 2010/03/08 21:20:24 cg Exp $'
! !
