"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SprayClientTests subclass:#SpwCoderTest
	instanceVariableNames:'coder node answer'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpwCoderTest methodsFor:'public'!

assertLiteralEncode: aString
	self assertNodeXMLSameAs: 
('<blah 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" >
%1
</blah>' formatWith: aString)
!

assertNodeXMLSameAs: aString
	| xml |
	self assertXML: (xml := node xml)
		isSameAs: aString
!

assertSoapEncEncode: aString
	self assertNodeXMLSameAs: 
('<blah 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
%1
</blah>' formatWith: aString)
!

formatBodyXmlWith: aString
	
^'<SOAP-ENV:Body 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:d="http://www.dolphinharbor.org/spray/class"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
%1
</SOAP-ENV:Body>' formatWith: aString
!

setNodeFormatting: aString
	self setNodeFrom: (self formatBodyXmlWith: aString)
!

setNodeFrom: aString 
	node := self rootNodeFor: aString
! !

!SpwCoderTest methodsFor:'set up'!

setUp
	super setUp.
	node := self new2001Node: 'blah'.


! !

!SpwCoderTest methodsFor:'tests'!

testConstructionDocEncS
	coder := self wireObjects docEncS.
	self assert: (coder isKindOf: SpwSoapBCoder).
	self assert: coder element qName = self qParam1
!

testConstructionDocLitC1
	coder := self wireObjects docLitC1.
	self assert: (coder isKindOf: SpwSoapBCoder).
	self assert: coder element qName = (XeQName sprayClass: SWXMLTestObject)
!

testConstructionRpcEnc1
	coder := self wireObjects rpcEnc1.
	self assert: (coder isKindOf: SpwSoapBCoder).
	self assert: coder element qName = self qMyOperation.
	self assert: coder element qName prefix = 'm'.
	self deny: coder isMultipleRPCParameters
!

testConstructionRpcEncV
	coder := self wireObjects rpcEncV.
	self deny: coder isMultipleRPCParameters
!

testDecodeDOC
	coder := self wireObjects docEncS.
	self setNodeFormatting: '<param1 xmlns="urn:blah">abc</param1>'.
	answer := coder decodeSoapB: node firstChild.
	self assert: answer = 'abc'
!

testDecodeDOCLit
	coder := self wireObjects docLitC1.
	self 
		setNodeFormatting: '<d:SWXMLTestObject xsi:type="d:SWXMLTestObject">
<x xsi:type="xsd:integer">5</x>
<y xsi:type="xsd:string">hello</y>
</d:SWXMLTestObject>'.
	answer := coder decodeSoapB: node firstChild.
	self assert: (answer isKindOf: SWXMLTestObject).
	self assert: answer x = 5
!

testDecodeDOCLitEFDQ
	typespace := self get2001FQTypespace.
	coder := self wireObjects docLitC1.
	self 
		setNodeFormatting: '<ns1:SWXMLTestObject xsi:type="ns1:SWXMLTestObject"
xmlns:ns1="http://www.dolphinharbor.org/spray/class">
<ns1:x xsi:type="xsd:integer">5</ns1:x>
<ns1:y xsi:type="xsd:string">hello</ns1:y>
</ns1:SWXMLTestObject>'.
	answer := coder decodeSoapB: node firstChild.
	self assert: (answer isKindOf: SWXMLTestObject).
	self assert: answer x = 5
!

testDecodeDOCNoNS
	coder := self wireObjects docEncS.
	self setNodeFormatting: '<param1 >abc</param1>'.
	self should: [coder decodeSoapB: node firstChild] raise: SoapBlockNameMismatch
!

testDecodeDOCWrongName
	coder := self wireObjects docEncS.
	self setNodeFormatting: '<param2 xmlns="urn:blah">abc</param2>'.
	self should: [coder decodeSoapB: node firstChild] raise: SoapBlockNameMismatch
!

testDecodeRPC
	coder := self wireObjects rpcEnc2.
	self 
		setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
<param1>abc</param1>
<param2>123</param2>
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	self assert: answer = #('abc' 123)
!

testDecodeRPCMissing
	coder := self wireObjects rpcEnc2.
	self 
		setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
<param2>123</param2>
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	"Impossible to get right"
	self class isSkipAckFailures ifTrue: [^nil].
	self assert: answer = #(nil 123)
!

testDecodeRPCMissingNoRP
	coder := self wireObjects rpcEnc2.
	coder element content isNoReturnPart: true.
	self 
		setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
<param2>123</param2>
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	"Impossible to get right"
	self assert: answer = #(nil 123)
!

testDecodeRPCNoNS
	coder := self wireObjects rpcEnc2.
	self 
		setNodeFormatting: 
'<myOperation>
<param1>abc</param1>
<param2>123</param2>
</myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	self assert: answer = #('abc' 123)
!

testDecodeRPCReOrdered
	coder := self wireObjects rpcEnc2.
	self 
		setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
<param2>123</param2>
<param1>abc</param1>
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	"Impossible to get right"
	self class isSkipAckFailures ifTrue: [^nil].
	self assert: answer = #('abc' 123)
!

testDecodeRPCReOrderedNoRP
	coder := self wireObjects rpcEnc2.
	coder element content isNoReturnPart: true.
	self 
		setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
<param2>123</param2>
<param1>abc</param1>
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	self assert: answer = #('abc' 123)
!

testDecodeRPCVoid
	coder := self wireObjects rpcEncV.
	self setNodeFormatting: 
'<m:myOperation xmlns:m="urn:blah">
</m:myOperation>'.
	answer := coder decodeRPCSoapB: node firstChild.
	self assert: answer isEmpty
!

testDecodeRPCWrongName
	coder := self wireObjects rpcEnc2.
	self 
		setNodeFormatting: 
'<m:NOTmyOperation xmlns:m="urn:blah">
<param1>abc</param1>
<param2>123</param2>
</m:NOTmyOperation>'.

	"must ignore name"
	answer := coder decodeRPCSoapB: node firstChild.
	self assert: answer = #('abc' 123)
!

testEncodeDOCAnswer
	coder := self wireObjects docEncS.
	coder encodeObject: 'abc' into: node.
	self 
		assertSoapEncEncode: '<ns1:param1 xsi:type="xsd:string" xmlns:ns1="urn:blah" 
  SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">abc</ns1:param1>'
!

testEncodeDOCLit
	coder := self wireObjects docLitC1.
	coder encodeObject: SWXMLTestObject example1 into: node.
	self 
		assertLiteralEncode: '<ns1:SWXMLTestObject 
xmlns:ns1="http://www.dolphinharbor.org/spray/class">
<x>5</x>
<y>hello</y>
</ns1:SWXMLTestObject>'
!

testEncodeDOCLitEFDQ
	typespace := self get2001FQTypespace.
	coder := self wireObjects docLitC1.
	coder encodeObject: SWXMLTestObject example1 into: node.
	self 
		assertLiteralEncode: '<ns1:SWXMLTestObject 
xmlns:ns1="http://www.dolphinharbor.org/spray/class">
<ns1:x>5</ns1:x>
<ns1:y>hello</ns1:y>
</ns1:SWXMLTestObject>'
!

testEncodeDOCLitQ
	coder := self wireObjects docLitC1.
	coder element childElements 
		do: [:each | each beFormQualifiedNS: #'http://www.dolphinharbor.org/spray/class'].
	coder encodeObject: SWXMLTestObject example1 into: node.
	self 
		assertLiteralEncode: '<ns1:SWXMLTestObject
xmlns:ns1="http://www.dolphinharbor.org/spray/class">
<ns1:x>5</ns1:x>
<ns1:y>hello</ns1:y>
</ns1:SWXMLTestObject>'
!

testEncodeRPC
	coder := self wireObjects rpcEnc1.
	coder encodeRPCAnswer: 'abc' into: node.
	self 
		assertSoapEncEncode: 
'<m:myOperation xmlns:m="urn:blah"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<param1 xsi:type="xsd:string">abc</param1>
</m:myOperation>'
!

testEncodeRPCMultipleOuts
	coder := self wireObjects rpcEnc2.
	coder element content isNoReturnPart: true.
	coder encodeRPCAnswer: #('abc' 123) into: node.
	self 
		assertSoapEncEncode: 
'<m:myOperation xmlns:m="urn:blah"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<param1 xsi:type="xsd:string">abc</param1>
<param2 xsi:type="xsd:int">123</param2>
</m:myOperation>'
!

testEncodeRPCVoid
	coder := self wireObjects rpcEncV.
	coder encodeRPCAnswer: nil into: node.
	self 
		assertSoapEncEncode: 
'<m:myOperation xmlns:m="urn:blah"
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"></m:myOperation>'
!

testParametersFromRpcEnc1
	coder := self wireObjects rpcEnc1.
	self assert: coder numberOfRPCArgs = 1.
	self assert: (coder parametersFrom: #('abc')) size = 1.
	self assert: ((coder parametersFrom: #('abc')) at: #param1) = 'abc'.
	self should: [coder parametersFrom: #('abc' 'def')] raise: Error.
	self should: [coder parametersFrom: #()] raise: Error.

!

testParametersFromRpcEncV
	coder := self wireObjects rpcEncV.
	self assert: coder numberOfRPCArgs = 0.
	self assert: (coder parametersFrom: #()) isEmpty.
	self should: [coder parametersFrom: #('abc' )] raise: Error.
	self should: [coder parametersFrom: #('abc' 'def')] raise: Error.

! !

!SpwCoderTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpwCoderTest.st,v 1.1 2009/10/02 19:42:59 cg Exp $'
! !
