"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SpwInEnvelopeTests subclass:#SpwRPCReturnTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!SpwRPCReturnTest methodsFor:'public'!

beRpcEnc1
	envelope := SpwRPCReturn newIn: typespace.
	envelope 
		bodyCoders: (Array with: self wireObjects rpcEnc1)
		headerCoders: #()

!

beRpcEnc2
	envelope := SpwRPCReturn newIn: typespace.
	envelope 
		bodyCoders: (Array with: self wireObjects rpcEnc2)
		headerCoders: #()

!

beRpcEncV
	envelope := SpwRPCReturn newIn: typespace.
	envelope 
		bodyCoders: (Array with: self wireObjects rpcEncV)
		headerCoders: #()

!

setSoapInFormating: aString 
	soapInEnv := SpwSoapInEnvelope 
				fromXmlString: (
'<SOAP-ENV:Envelope 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<m:MustIgnore xmlns:m="urn:MustIgnore">
%1
</m:MustIgnore>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>' 
						formatWith: aString)
! !

!SpwRPCReturnTest methodsFor:'set up'!

setUp
	super setUp.
	self beRpcEnc1

! !

!SpwRPCReturnTest methodsFor:'tests'!

testExtractNilReturn
	self setSoapInFormating: ''.
	self beRpcEnc1.
	self decodeSoapIn.
	self assert: envelope extractReturn isNil.
	self beRpcEnc2.
	self decodeSoapIn.
	self assert: envelope extractReturn isNil
!

testExtractReturn
	self setSoapInFormating: '<mustIgnore>abc</mustIgnore>'.
	self beRpcEnc1.
	self decodeSoapIn.
	self assert: envelope extractReturn = 'abc'.
	self beRpcEnc2.
	self decodeSoapIn.
	self assert: envelope extractReturn = 'abc'
!

testExtractReturn2
	self setSoapInFormating: '<mustIgnore>abc</mustIgnore><param2>123</param2>'.
	self beRpcEnc2.
	self decodeSoapIn.
	self assert: envelope extractReturn = 'abc'
!

testExtractVoidReturn
	self setSoapInFormating: ''.
	self beRpcEncV.
	self decodeSoapIn.
	self assert: envelope extractReturn isNil.
!

testForceIn
	self setSoapInFormating: '<mustIgnore xsi:type="xsd:integer">123</mustIgnore>'.
	self beRpcEnc1.
	self should: [self decodeSoapIn] raise: XeTypeMismatch .
	envelope forceInSoap: soapInEnv.
	self assert: envelope extractReturn = 123.

!

testMultipleOuts
	self setSoapInFormating: '<mustIgnore>abc</mustIgnore><param2>123</param2>'.
	self beRpcEnc2.
	self decodeSoapIn.
	self assert: envelope firstBodyObject first = 'abc'.
	self assert: envelope firstBodyObject second = 123.
	self assert: (envelope outParameters at: #param1) = 'abc'.
	self assert: (envelope outParameters at: #param2) = 123


!

testResumeIn
	self setSoapInFormating: '<mustIgnore xsi:type="xsd:integer">abc</mustIgnore>'.
	self beRpcEnc1.
	self should: [self decodeSoapIn] raise: XeTypeMismatch .
	envelope resumeInSoap: soapInEnv.
	self assert: envelope extractReturn = 'abc'.

! !

!SpwRPCReturnTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__SpwRPCReturnTest.st,v 1.1 2009/10/02 19:40:02 cg Exp $'
! !
