"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

WSDLTests subclass:#WSDLDefinitionsTest
	instanceVariableNames:'definitions'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!WSDLDefinitionsTest methodsFor:'public'!

setApacheDefs
	definitions := self getApacheDefs
!

setStockQuoteDefs
	definitions := self getStockQuoteDefs
! !

!WSDLDefinitionsTest methodsFor:'tests'!

testDOCConstruction
	self setApacheDefs.
	self assert: (definitions isKindOf: WSDLDefinitions).
	self assert: definitions rootUrl = ''.
	self assert: definitions documents size = 1.
	self assert: definitions services size = 1.
	self assert: definitions bindings size = 3
!

testDefinitionsName
	self setStockQuoteDefs.
	self assert: definitions definitionsName = 'StockQuote'
!

testExtractNoneSchemaTypes
	self should: [WSDLDefinitions onDocument: (self rootNodeFor: self getSampleWSDLNonSchemaTypes) transport: nil]
		raise: Error
!

testExtractSchemaImports
	| imports |
	imports := self getApacheDefs extractSchemas.
	self assert: imports size = 1
!

testExtractSchemaImportsNoTypes
	| imports |
	definitions := WSDLDefinitions onDocument: (self rootNodeFor: self getSampleWSDLNoTypes) transport: nil.
	imports := definitions extractSchemas.
	self assert: imports isEmpty
!

testNodesNamed
	| ans name |
	self setStockQuoteDefs.
	name := 'GetLastTradePriceInput' ns: #'http://example.com/stockquote/definitions'.
	ans := definitions messageNamed: name.
	self assert: ans localName = 'message'.
	name := 'StockQuoteSoapBinding' ns: #'http://example.com/stockquote/service'.
	ans := definitions bindingNamed: name.
	self assert: ans localName = 'binding'.
	name := 'StockQuotePortType' ns: #'http://example.com/stockquote/definitions'.
	ans := definitions portTypeNamed: name.
	self assert: ans localName = 'portType'.
	name := 'StockQuotePort' ns: #'http://example.com/stockquote/service'.
	ans := definitions portNamed: name.
	self assert: ans localName = 'port'
!

testSchemaImport
	self setStockQuoteDefs.
	self assert: definitions schemas size = 1.
	self assert: definitions schemas first localName = 'schema'
!

testServiceQName
	self setStockQuoteDefs.
	self assert: definitions serviceQName 
				= ('StockQuoteService' ns: #'http://example.com/stockquote/service')
!

testSupportedPortQNames
	| ans |
	self setStockQuoteDefs.
	ans := definitions supportedPortQNames.
	self assert: ans size = 1.
	self assert: ans first = ('StockQuotePort' ns: #'http://example.com/stockquote/service').

	"has unsupported"
	self setApacheDefs.
	ans := definitions supportedPortQNames.
	self assert: ans size = 1.
	self assert: ans first = ('BidBuySoap' ns: #'http://www.soapinterop.org/Bid')
!

testURLConstruction
	self setStockQuoteDefs.
	self assert: (definitions isKindOf: WSDLDefinitions).
	self assert: definitions rootUrl = 'http://example.com/stockquote/stockquoteservice.wsdl'.
	self assert: definitions documents size = 3.
	self assert: definitions messages size = 2.
	self assert: definitions portTypes size = 1.
	self assert: definitions services size = 1.
	self assert: definitions bindings size = 1
!

testUrlOf
	| doc |
	definitions := WSDLDefinitions new.
	doc := XeEnvironment rootNodeFor: '<abc/>'.
	definitions addDocument: doc at: 'http://www.foo.org/wsdl'.
	self assert: (definitions urlOf: doc) = 'http://www.foo.org/wsdl'
! !

!WSDLDefinitionsTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__WSDLDefinitionsTest.st,v 1.2 2009/10/02 19:44:37 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__WSDLDefinitionsTest.st,v 1.2 2009/10/02 19:44:37 cg Exp $'
! !
