"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

WSDLTests subclass:#WSDLOperationBindingTest
	instanceVariableNames:'wsdlPort wsdlOperation'
	classVariableNames:''
	poolDictionaries:''
	category:'Unclassified'
!


!WSDLOperationBindingTest methodsFor:'public'!

setPortWithNRP
	| xmlString defs |
	xmlString := 
'<?xml version="1.0"?>
<definitions name="InteropTestB" targetNamespace="http://soapinterop.org/" xmlns="http://schemas.xmlsoap.org/wsdl/" xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:tns="http://soapinterop.org/" xmlns:s="http://soapinterop.org/xsd" xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/">
	<types>
		<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://soapinterop.org/xsd">
			<complexType name="SOAPStruct">
				<all>
					<element name="varString" type="string"/>
					<element name="varInt" type="int"/>
					<element name="varFloat" type="float"/>
				</all>
			</complexType>
		</schema>
	</types>

	<message name="echoStructAsSimpleTypesRequest">
		<part name="inputStruct" type="s:SOAPStruct"/>
	</message>
	<message name="echoStructAsSimpleTypesResponse">
		<part name="outputString" type="xsd:string"/>
		<part name="outputInteger" type="xsd:int"/>
		<part name="outputFloat" type="xsd:float"/>
	</message>
	<message name="echoSimpleTypesAsStructRequest">
		<part name="inputString" type="xsd:string"/>
		<part name="inputInteger" type="xsd:int"/>
		<part name="inputFloat" type="xsd:float"/>
	</message>
	<message name="echoSimpleTypesAsStructResponse">
		<part name="return" type="s:SOAPStruct"/>
	</message>
	<portType name="InteropTestPortTypeB">
		<operation name="echoStructAsSimpleTypes" parameterOrder="inputStruct outputString outputInteger outputFloat">
			<input message="tns:echoStructAsSimpleTypesRequest" name="echoStructAsSimpleTypes"/>
			<output message="tns:echoStructAsSimpleTypesResponse" name="echoStructAsSimpleTypesResponse"/>
		</operation>
		<operation name="echoSimpleTypesAsStruct" parameterOrder="inputString inputInteger inputFloat">
			<input message="tns:echoSimpleTypesAsStructRequest" name="echoSimpleTypesAsStruct"/>
			<output message="tns:echoSimpleTypesAsStructResponse" name="echoSimpleTypesAsStructResponse"/>
		</operation>
	</portType>

	<binding name="InteropTestSoapBindingB" type="tns:InteropTestPortTypeB">
		<soap:binding style="rpc" transport="http://schemas.xmlsoap.org/soap/http"/>
		<operation name="echoStructAsSimpleTypes">
			<soap:operation soapAction="http://soapinterop.org/"/>
			<input>
				<soap:body use="encoded" namespace="http://soapinterop.org/" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
			</input>
			<output>
				<soap:body use="encoded" namespace="http://soapinterop.org/" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
			</output>
		</operation>
		<operation name="echoSimpleTypesAsStruct">
			<soap:operation soapAction="http://soapinterop.org/"/>
			<input>
				<soap:body use="encoded" namespace="http://soapinterop.org/" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
			</input>
			<output>
				<soap:body use="encoded" namespace="http://soapinterop.org/" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
			</output>
		</operation>
	</binding>
<service name="interopB2001">
<port name="soapinteropB" binding="tns:InteropTestSoapBindingB">
<soap:address location="http://localhost/services/interopB2001"/></port></service>

</definitions>'.
	defs := WSDLDefinitions onDocument: (self rootNodeFor: xmlString) transport: nil.
	wsdlPort := WSDLPortData onDefinitions: defs portName: nil
! !

!WSDLOperationBindingTest methodsFor:'tests'!

testCreateBody
	| body |
	wsdlPort := WSDLPortData onDefinitions: self getApacheDefs portName: nil.
	wsdlOperation := wsdlPort wsdlOperationNamed: 'SimpleBuy'.
	"<operation name='SimpleBuy'>
      <soap:operation soapAction='http://www.soapinterop.org/SimpleBuy' style='rpc' />
      <input>
        <soap:body use='encoded' namespace='http://www.soapinterop.org/Bid' encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' />
      </input>
       </operation>"
	body := wsdlOperation input createBody.
	self assert: body use = 'encoded'.
	self assert: body namespace = #'http://www.soapinterop.org/Bid'.
	self assert: body encStyleUri = #'http://schemas.xmlsoap.org/soap/encoding/'.
	self assert: body partNames isNil
!

testCreateBodyPartNames
        | body |
        wsdlPort := WSDLPortData onDefinitions: self getSample1Defs portName: nil.
        wsdlOperation := wsdlPort wsdlOperationNamed: 'myPartNamesOperation'.
        body := wsdlOperation input createBody.
        self assert: body partNames asArray = #('part1' 'part2').
        body := wsdlOperation output createBody.
        self assert: body partNames asArray = #('part1' 'part2' 'part3' 'part4')
!

testCreateHeader
	| soapHeader definitions|
	definitions := WSDLDefinitions onDocument: (self rootNodeFor: self getSampleWSDLHeaders) ownerDocument transport: nil.
	wsdlPort := WSDLPortData onDefinitions: definitions portName: nil.
	wsdlOperation := wsdlPort wsdlOperationNamed: 'echoVoid'.
	soapHeader := wsdlOperation createInputHeaders first.
	self assert: soapHeader isEncoded.
	self assert: soapHeader namespace = #'http://soapinterop.org/echoheader/'.
	self assert: soapHeader partNames = #('echoMeStringRequest').
	self assert: soapHeader message parts size = 1.
	self assert: soapHeader message parts first partName = 'echoMeStringRequest'
!

testCreateMessage
	| message |
	wsdlPort := WSDLPortData onDefinitions: self getApacheDefs portName: nil.
	wsdlOperation := wsdlPort wsdlOperationNamed: 'SimpleBuy'.
	message := wsdlOperation input createBodyMessage.
	self assert: message name = 'SimpleBuySoapIn'.
	"self deny: message isOutput."
	self assert: message parts size = 3.
	message := wsdlOperation output createBodyMessage.
	self assert: message name = 'SimpleBuySoapOut'.
	"self assert: message isOutput."
	self assert: message parts size = 1.
	self deny: message isNoReturnPart
!

testHasNoReturnPart
	self setPortWithNRP.
	self assert: wsdlPort wsdlOperations first name = 'echoStructAsSimpleTypes'.
	self assert: wsdlPort wsdlOperations first output hasNoReturnPart.
	self deny: wsdlPort wsdlOperations second output hasNoReturnPart
!

testNamedBindingOpNodes
	| definitions |
	definitions := WSDLDefinitions onDocument: (self rootNodeFor: self getSampleWSDLNamedOpNodes) ownerDocument transport: nil.
	wsdlPort := WSDLPortData onDefinitions: definitions portName: nil.
	wsdlOperation := wsdlPort wsdlOperationNamed: 'echoString'.
	self assert: wsdlOperation input bindingNodeName = 'echoString'.
	self assert: wsdlOperation output bindingNodeName = 'echoStringResult'
!

testRPCEncoded
	wsdlPort := WSDLPortData onDefinitions: self getApacheDefs portName: nil.
	wsdlOperation := wsdlPort wsdlOperationNamed: 'SimpleBuy'.
	"<operation name='SimpleBuy'>
      <soap:operation soapAction='http://www.soapinterop.org/SimpleBuy' style='rpc' />
      <input>
        <soap:body use='encoded' namespace='http://www.soapinterop.org/Bid' encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' />
      </input>
      <output>
        <soap:body use='encoded' namespace='http://www.soapinterop.org/Bid' encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' />
      </output>
    </operation>"
	self assert: wsdlOperation isRPC.
	self assert: wsdlOperation soapAction = 'http://www.soapinterop.org/SimpleBuy'.
	self assert: wsdlOperation name = 'SimpleBuy'
! !

!WSDLOperationBindingTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__WSDLOperationBindingTest.st,v 1.4 2009/11/19 11:52:28 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/SOAP__WSDLOperationBindingTest.st,v 1.4 2009/11/19 11:52:28 cg Exp $'
! !
