"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

SprayTests subclass:#WSDLGenerateFuncTests
	instanceVariableNames:'wsdlNode'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-WSDL-TestCases'
!


!WSDLGenerateFuncTests methodsFor:'accessing'!

getLocalSprayServer
	^self getService createServer
!

getServerWSDLNode
	^(self getService createWSDL: SptLocalServerBinding new) asParserNode
!

targetNamespace
	^self getService namespace
! !

!WSDLGenerateFuncTests methodsFor:'asserting'!

assertNodesNamedUniquely: aCollection 
	| names |
	names := aCollection collect: 
					[:eachN | 
					| theName |
					theName := eachN getAttribute: 'name'.
					self assert: theName notNil.
					theName].
	self assert: names asSet size = names size
!

assertValidBinding: bindingNode
	| typeQName portTypeNodes opNodes portTypeOpNodes |
	typeQName := bindingNode qValueForType.
	"WSDL Spec1.1 2.5
A binding references the portType that it binds using the type attribute. This QName value follows the linking rules defined by WSDL "
	"bindingNode ownerDocument browserView"


	"Since I only produce single document wsdls, this needs to be true if qualified;"
	self assert: typeQName namespace == bindingNode targetNamespace.
	portTypeNodes := (bindingNode ownerDocument documentElement selectChildNodes: 'portType'
				in: nil) select: [:each | (each getAttribute: 'name') = typeQName localName].
	self assert: portTypeNodes size = 1.
	"WSDL Spec1.1 2.5
An operation element within a binding specifies binding information for the operation with the same name within the binding's portType. Since operation names are not required to be unique (for example, in the case of overloading of method names), the name attribute in the operation binding element might not be enough to uniquely identify an operation. In that case, the correct operation should be identified by providing the name attributes of the corresponding wsdl:input and wsdl:output elements."
	portTypeOpNodes := portTypeNodes first selectChildNodes: 'operation' in: nil.
	opNodes := bindingNode selectChildNodes: 'operation' in: nil.
	opNodes do: 
			[:each | 
			| opName |
			opName := each getAttribute: 'name'.
			self assert: opName notNil.
			self 
				assert: (portTypeOpNodes select: [:eachN | (eachN getAttribute: 'name') = opName]) size 
						>= 1.

			"CapeClear: Always providing a SOAPAction attribute, when the HTTP protocol for SOAP is used."
			self assertValidBindingOpNode: each]
!

assertValidBindingOpNode: aXeNode
	| soapOperationNode |
	soapOperationNode := aXeNode detectChildNode: 'operation' in: XeNamespace soapWSDL.
	self assertValidBindingSoapOperation: soapOperationNode.
	aXeNode detectChildNode: 'input' in: nil.
	aXeNode detectChildNode: 'output' in: nil
!

assertValidBindingSoapOperation: soapOperationNode 
	self assert: soapOperationNode notNil.
	self assert: (soapOperationNode getAttribute: 'soapAction') notEmpty
!

assertValidDefinitions: defs 
	defs messages do: [:each | self assert: each key namespace = self targetNamespace].
	defs portTypes do: [:each | self assert: each key namespace = self targetNamespace].
	defs bindings do: [:each | self assert: each key namespace = self targetNamespace].
	defs services do: [:each | self assert: each key namespace = self targetNamespace]
!

assertValidLocalService: service 
	self deny: service hasInvalidOperations.
	self deny: service hasWarnings
!

assertValidMessages: messageNodes 
!

assertValidNameScope: node 
	"WSDL Spec1.1 Sec2.1.1
WSDL definitions: service, port, message, bindings, and portType 
.....
Each WSDL definition type listed above has its own name scope (i.e. port names and message names never conflict). Names within a name scope MUST be unique within the WSDL document. "

	#('message' 'binding' 'service' 'portType') do: 
			[:each | 
			| nodes |
			nodes := node childNodes select: [:eachN | eachN localName = each].
			self assert: nodes size > 0.
			self assertNodesNamedUniquely: nodes]
!

assertValidParts: messageNodes 
!

assertValidPortTypeNode: portNode 
	| opNodes elements |
	opNodes := portNode selectChildNodes: 'operation' in: nil.
	opNodes do: 
			[:each | 
			| childNames |
			childNames := each childNodes collect: [:eachN | eachN localName].
			self assert: (childNames select: [:eachN | eachN = 'input']) size = 1.
			self assert: (childNames select: [:eachN | eachN = 'output']) size = 1].

	"WSDL Spec1.1
2.4.5 Names of Elements within an Operation
The name attribute of the input and output elements provides a unique name among all input and output elements within the enclosing port type."
	elements := OrderedCollection new.
	elements addAll: (portNode selectAllNodes: 'input' in: nil).
	elements addAll: (portNode selectAllNodes: 'output' in: nil)
	"I am now not naming these"
	"self checkNodesNamedUniquely: elements"
!

assertValidSchema: schemaNode 
	self assert: (schemaNode namespaceFor: 'xsi') = self xsiNamespace.
	schemaNode childNodes do: [ :each |
		self assert: each namespace = self xsdNamespace]
!

assertValidTopNodes: node 
        self assert: node localName = 'definitions'.
        self assert: (node getAttribute: 'targetNamespace') = self targetNamespace asString.
        Smalltalk isSmalltalkX ifTrue:[
            self assert: ((node childNodes collect: [:each | each localName]) asSet 
                                = #('types' 'binding' 'portType' 'service' 'message') asSet)
        ] ifFalse:[
            self assert: ((node childNodes collect: [:each | each localName]) asSet 
                                equals: #('types' 'binding' 'portType' 'service' 'message') asSet)
        ].
!

assertWSDLPortValid: portNode 
	| bindingQName bindingNodes |
	bindingQName := portNode qValueFor: 'binding'.
	"WSDL Spec1.1 2.6
The binding attribute (of type QName) refers to the binding using the linking rules defined by WSDL  "

	"Since I only produce single document wsdls, this is true;"
	self assert: bindingQName namespace == portNode targetNamespace.
	bindingNodes := (portNode ownerDocument documentElement selectChildNodes: 'binding' in: nil) 
				select: [:each | (each getAttribute: 'name') = bindingQName localName].
	self assert: bindingNodes size = 1
! !

!WSDLGenerateFuncTests methodsFor:'set up'!

setUp
	super setUp.
	wsdlNode := self getServerWSDLNode.
! !

!WSDLGenerateFuncTests methodsFor:'tests'!

testBindings
	| bindingNodes |
	bindingNodes := wsdlNode selectChildNodes: 'binding' in: nil.
	self assert: bindingNodes notEmpty.

	"wsdl spec1.1 Sec2.5
The name attribute provides a unique name among all bindings defined within in the enclosing WSDL document."
	self assertNodesNamedUniquely: bindingNodes.
	bindingNodes do: [:each | self assertValidBinding: each]
!

testDefinitions
	"CapeClear common errors
Rec: the default namespace for the root element to 'http://schemas.xmlsoap.org/wsdl/'"

	| defs |
	self assert: wsdlNode namespace = #'http://schemas.xmlsoap.org/wsdl/'.
	defs := WSDLDefinitions onDocument: wsdlNode ownerDocument transport: nil.
	self assert: defs schemas size = self numberOfSchemaNodes.
	self assert: defs allPortNames notEmpty.
	self assert: defs supportedPortQNames notEmpty.
	self assert: defs messages notEmpty.
	self assert: defs portTypes notEmpty.
	self assert: defs bindings notEmpty.
	self assert: defs services notEmpty.
	defs supportedPortQNames do: [:each | self assert: (each isKindOf: XeQName)].
	self assertValidDefinitions: defs
!

testLocalSprayService
	| service |
	service := SprayWSDLService definitionNode: wsdlNode portName: nil.
	self assertValidLocalService: service
!

testMessageParts
	"WSDL Spec1.1 Sec2.3
<message name=nmtoken> * <part name=nmtoken element=qname? type=qname?/> * </message> "

	| allPartNodes messageNodes allTypeNames |
	allPartNodes := wsdlNode selectAllNodes: 'part' in: nil.
	allPartNodes do: 
			[:each | 
			self assert: (each getAttribute: 'name') notNil.
			self 
				assert: ((each getAttribute: 'type') notNil or: [(each getAttribute: 'element') notNil])].

	"The part name attribute provides a unique name among all the parts of the enclosing message."
	messageNodes := wsdlNode selectChildNodes: 'message' in: nil.
	messageNodes do: 
			[:each | 
			| partNodes |
			partNodes := each selectChildNodes: 'part' in: nil.
			self assertNodesNamedUniquely: partNodes].

	"I always prefix my types, make sure they are prefixed, and no double prefixes slip through"
	allTypeNames := OrderedCollection new.
	allPartNodes do: 
			[:each | 
			| tn |
			tn := each getAttribute: 'type'.
			tn notNil 
				ifTrue: 
					[self assert: (tn select: [:eachC | eachC = $:]) size = 1.
					allTypeNames add: each qValueForType]].

	"Make sure each type has a namespace"
	allTypeNames do: [:each | self assert: each namespace notNil].
	self assertValidMessages: messageNodes.
	self assertValidParts: allPartNodes
!

testPortType
	| portTypeNodes |
	portTypeNodes := wsdlNode selectChildNodes: 'portType' in: nil.
	self assert: portTypeNodes notEmpty.
	portTypeNodes do: [:each | self assertValidPortTypeNode: each]
!

testPorts
	| portNodes |
	portNodes := (wsdlNode detectChildNode: 'service' in: nil) selectChildNodes: 'port' in: nil.

	"wsdl spec1.1 Sec2.6
The name attribute provides a unique name among all ports defined within in the enclosing WSDL document."
	self assertNodesNamedUniquely: portNodes.
	portNodes do: [:each | self assertWSDLPortValid: each]
!

testSchemaNodes
	| schemaNodes targets |
	schemaNodes := (wsdlNode detectChildNode: 'types' in: nil) selectChildNodes: 'schema' in: nil.
	self assert: schemaNodes size = self numberOfSchemaNodes.
	targets := schemaNodes collect: [:each | each getAttribute: 'targetNamespace'].
	self assert: (targets allSatisfy: [:each | each notNil]).
	self assert: (targets allSatisfy: [:each | each first ~~ $/]).
	self 
		assert: (self requiredSchemaTargetNamespaces allSatisfy: [:each | targets includes: each asString]).
	schemaNodes do: [:each | self assertValidSchema: each]
!

testTopNodes
	self assertValidTopNodes: wsdlNode
!

testXSINamespace
	self assert: (wsdlNode namespaceFor: 'xsi') = self xsiNamespace.

! !

!WSDLGenerateFuncTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/WSDLGenerateFuncTests.st,v 1.7 2010/03/08 23:28:33 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/WSDLGenerateFuncTests.st,v 1.7 2010/03/08 23:28:33 cg Exp $'
! !
