"{ Package: 'stx:goodies/soap/wsdl/tests' }"

"{ NameSpace: SOAP }"

WSDLTests subclass:#WSDLMSExplainedTest
	instanceVariableNames:'service client soapSend'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-WSDL-TestCases'
!

WSDLMSExplainedTest comment:'http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09'
!


!WSDLMSExplainedTest methodsFor:'accessing'!

commonWSDLString
^'<definitions
  xmlns:stns="(SchemaTNS)"
  xmlns:wtns="(WsdlTNS)"
  targetNamespace="(WsdlTNS)"
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/" 
>
<types>
<schema targetNamespace="(SchemaTNS)"
    elementFormDefault="qualified">
  <element name="SimpleElement" type="xsd:int"/>
  <element name="CompositeElement" type="stns:CompositeType"/>
  <complexType name="CompositeType">
    <all>
      <element name="a" type="xsd:int"/>
      <element name="b" type="xsd:string"/>
    </all>
  </complexType>
</schema>
</types>

<message name="theMessage">
  <part name="p1" type="stns:CompositeType"/>
  <part name="p2" type="xsd:int"/>
  <part name="p3" element="stns:SimpleElement"/>
  <part name="p4" element="stns:CompositeElement"/>
</message>

<message name="voidMessage">
</message>

<portType name="fooSamplePortType">
     <operation name="foo"  >
	  <input message="wtns:theMessage"/>
	  <output message="wtns:voidMessage"/>
     </operation>
</portType>

  <service name="FOOService">
	  <port name="fooSamplePort" binding="wtns:fooSampleBinding">
	       <soap:address 
		  location="http://carlos:8080/fooService/foo.asp"/>
	  </port>
 </service>

'
!

sampleCompositeSoapType
	^(XeQStruct new)
		at: #a put: 123;
		at: #b put: 'hello';
		yourself
!

setServiceFor: aSelector 
	service := SprayWSDLService definitionNode: (self rootNodeFor: (self perform: aSelector))
				portName: nil.
	client := service createClient
!

wsdlStringDOCEncoded

	^self commonWSDLString ,
'<binding name="fooSampleBinding" type="wtns:fooSamplePortType">
  <soap:binding style="document" transport="http://schemas.xmlsoap.org/soap/http"/> 
  <operation name="foo">
     <soap:operation soapAction="http://tempuri.org/action/foo2"/>
	  <input >
	      <soap:body parts="p1 p2" use="encoded" 
encodingStyle= "http://schemas.xmlsoap.org/soap/encoding/"
namespace="(MessageNS)"/>
	  </input>
	   <output>
	       <soap:body use="encoded" 
	       namespace="http://tempuri.org/message/"  
	       encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" />
	  </output>
</operation>
</binding>
</definitions>'

        
!

wsdlStringDOCLiteralElement

	^self commonWSDLString ,
'<binding name="fooSampleBinding" type="wtns:fooSamplePortType">
  <soap:binding style="document" transport="http://schemas.xmlsoap.org/soap/http"/> 
  <operation name="foo">
     <soap:operation soapAction="http://tempuri.org/action/foo2"/>
	  <input >
	      <soap:body parts="p3 p4" use="literal" />
	  </input>
	   <output>
		<soap:body use="literal" />
	  </output>
</operation>
</binding>
</definitions>'

        
!

wsdlStringDOCLiteralType

	^self commonWSDLString ,
'<binding name="fooSampleBinding" type="wtns:fooSamplePortType">
  <soap:binding style="document" transport="http://schemas.xmlsoap.org/soap/http"/> 
  <operation name="foo">
     <soap:operation soapAction="http://tempuri.org/action/foo2"/>
	  <input >
	      <soap:body parts="p1" use="literal" />
	  </input>
	   <output>
		<soap:body use="literal" />
	  </output>
</operation>
</binding>
</definitions>'

        
!

wsdlStringRPCEncoded

	^self commonWSDLString ,
'<binding name="fooSampleBinding" type="wtns:fooSamplePortType">
  <soap:binding style="rpc" transport="http://schemas.xmlsoap.org/soap/http"/> 
  <operation name="foo">
     <soap:operation soapAction="http://tempuri.org/action/foo2"/>
	  <input >
	      <soap:body parts="p1 p2" use="encoded" 
encodingStyle= "http://schemas.xmlsoap.org/soap/encoding/"
namespace="(MessageNS)"/>
	  </input>
	   <output>
	       <soap:body use="encoded" 
	       namespace="http://tempuri.org/message/"  
	       encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" />
	  </output>
</operation>
</binding>
</definitions>'

        
!

wsdlStringRPCLiteral

	^self commonWSDLString ,
'<binding name="fooSampleBinding" type="wtns:fooSamplePortType">
  <soap:binding style="rpc" transport="http://schemas.xmlsoap.org/soap/http"/> 
  <operation name="foo">
     <soap:operation soapAction="http://tempuri.org/action/foo2"/>
	  <input >
	      <soap:body parts="p1 p2 p3 p4" use="literal" 
	       namespace="(MessageNS)"/>
	  </input>
	   <output>
	       <soap:body use="literal" 
	      namespace="(MessageNS)"/>
	  </output>
</operation>
</binding>
</definitions>'

        
! !

!WSDLMSExplainedTest methodsFor:'asserting'!

assertSoapOutXMLSameAs: aString
	self assertXML: soapSend soapOutEnvelope xml isSameAs: aString
! !

!WSDLMSExplainedTest methodsFor:'tests'!

testDOCEncoded
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09
	Note: this differs from the MS example ... I do not see their interpretation in the WSDL Spec"

	self setServiceFor: #wsdlStringDOCEncoded.
	self deny: service hasInvalidOperations.
	soapSend := client 
				sendDoc: 'foo'
				with: self sampleCompositeSoapType
				with: 123.
	self assertSoapOutXMLSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
    xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
    xmlns:ns2="(SchemaTNS)">
<SOAP-ENV:Body>
<ns1:p1 
    SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
    xsi:type="ns2:CompositeType"
    xmlns:ns1="(MessageNS)">
<a xsi:type="xsd:int">123</a>
<b xsi:type="xsd:string">hello</b>
</ns1:p1>
<ns1:p2 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xsi:type="xsd:int" xmlns:ns1="(MessageNS)">123</ns1:p2>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testDOCLiteralElement
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09"

	self setServiceFor: #wsdlStringDOCLiteralElement.
	self deny: service hasInvalidOperations.
	soapSend := client 
				sendDoc: 'foo'
				with: 123
				with: self sampleCompositeSoapType.
	self assertSoapOutXMLSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
   xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
   xmlns:xsd="http://www.w3.org/2001/XMLSchema"
   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
   xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<ns1:SimpleElement xmlns:ns1="(SchemaTNS)">123</ns1:SimpleElement>
<ns1:CompositeElement xmlns:ns1="(SchemaTNS)">
<ns1:a>123</ns1:a>
<ns1:b>hello</ns1:b>
</ns1:CompositeElement>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testDOCLiteralType
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09"

	self setServiceFor: #wsdlStringDOCLiteralType.
	self deny: service hasInvalidOperations.
	soapSend := client sendDoc: 'foo' with: self sampleCompositeSoapType.
	self class isSkipAckFailures ifTrue: [^nil].
	self assertSoapOutXMLSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
   xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
   xmlns:ns1="(SchemaTNS)"
   xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<ns1:a>123</ns1:a>
<ns1:b>hello</ns1:b>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'
!

testRPCEncoded
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09"

	self setServiceFor: #wsdlStringRPCEncoded.
	self deny: service hasInvalidOperations.
	soapSend := client 
				send: 'foo'
				with: self sampleCompositeSoapType
				with: 123.
	self assertSoapOutXMLSameAs: 
'<?xml version="1.0"?>
<SOAP-ENV:Envelope 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:ns1="(SchemaTNS)"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<m:foo 
    SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
    xmlns:m="(MessageNS)">
<p1 xsi:type="ns1:CompositeType">
<a xsi:type="xsd:int">123</a>
<b xsi:type="xsd:string">hello</b>
</p1>
<p2 xsi:type="xsd:int">123</p2>
</m:foo></SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

testRPCLiteral
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic09"

	self setServiceFor: #wsdlStringRPCLiteral.
	self class isSkipAckFailures ifTrue: [^nil].
	self deny: service hasInvalidOperations.
	soapSend := client 
				send: 'foo'
				with: self sampleCompositeSoapType
				with: 123
				with: 123
				with: self sampleCompositeSoapType.
	self assertSoapOutXMLSameAs:  ''

	"not supported"
! !

!WSDLMSExplainedTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/tests/WSDLMSExplainedTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
