"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeElementAbstract subclass:#XeAnyTypeElement
	instanceVariableNames:''
	classVariableNames:'IsWild'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeAnyTypeElement comment:'I am an element component with type: xsd:anyType.

-If decoding, I look for an inline type attribute, and if not present I
resort to a guess (which discriminates between Structs Arrays and simple
content)
-If encoding, I use reflection.

Background:
The design of Spray is that a schema component is modeled as a tree of
XeParticles and XeContent. This tree can then by visited to either 1/ decode
xml to an object 2/encode an object to xml 3/ write a schema describing
itself.

The XeParticles and XeContent are created by asking a factory (located by
targetNamespace) to create the content for a particular (qualified)
typeName. This is efficient for WebService/Soap use because the same
XeParticles/XeContent are used to encode/decode many instances through the
life of the service.

The XeParticle subclass that models xsd:anyType (ie this class!!) is not as efficient, as every
time it is asked to encode/decode an instance (either object instance or xml
instance) it asks for a new XeParticle/XeContent to be created, which
does the encode/decode, and is then discarded.

So back to the "simple answer" above:
-If decoding I grab the inline type attribute typeName and ask the typespace
(... effectively a root schema containing the factories I mentioned above.
Multiple typespaces can exist in the same image so that if required,
different content can be created for different targetNamespaces ...) to
create the content.
--If I have to guess, I check if the xml instance node is complex. If it is,
I check for an arrayType attribute. I will also keep checking for inline
type attributes on the childNodes so that I dont have to resort to answering
"text"

-To support encoding, I can ask any object for its #sprayTypeName which is
then used to ask the typespace to create the content. "Value" objects (like
instances of Float,String etc) mostly map to the XMLSchema dataType
typeNames. Object(class) by default answers a typeName that is the class
name.
--Spray has a flexible XML naming framework that I can "late-qualify" names.
For example a float instance would answer its #sprayTypeName as ''xsd:float'',
which is then qualified in the typespace by matching ''xsd'' to a namespace.
This is how I support both 1999 and 2001 schema. See XeName comment.  Similarly
Object(class) actutally answers ''d:ClassName'' and ''d'' is typically mapped to
a namespace that is targeted by a factory that knows how to extract a
definition from a class.

In Summary:

To support xsd:anyType I do exactly the same thing I would if I know the
type beforehand, except that I have to create and discard what is created
for each instance (xml or object) handled.


==========
dolphinharbor has a good service for xsd:anyType testing at:
http://www.dolphinharbor.org/services/interopPM/service.wsdl

'
!


!XeAnyTypeElement class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	IsWild := 256
! !

!XeAnyTypeElement methodsFor:'accessing'!

childElements
	"Answer a collection of <XeElements>
	This is empty because we create XeComponents when needed "

	^#()
!

elementForNode: aXeNode
	"Answer a <XeElement> created from the <XeNode> instance"

	^self encodingStyle elementForNode: aXeNode
!

elementForObject: anObject
	"Answer a <XeElement> created from the anObject instance"

	| newElement |
	newElement := self encodingStyle elementForObject: anObject.
	self isTyped ifFalse: [newElement beAllNotTyped].
	^newElement
!

isWild: aBoolean 
	flags := self flags mask: IsWild set: aBoolean
! !

!XeAnyTypeElement methodsFor:'copying-private'!

postCopy
	super postCopy.
	objectAccessor := objectAccessor copy component: self
! !

!XeAnyTypeElement methodsFor:'modes'!

beWild
	self isWild: true

! !

!XeAnyTypeElement methodsFor:'testing'!

isAnonymous
	^false
!

isWild
	"Answer if the receiver matches all child nodes"

	^self flags allMask: IsWild
! !

!XeAnyTypeElement methodsFor:'visitor'!

decode: aXeSoapXMLReader
	"Answer the <Object> decoded from the aXeSoapXMLReader "

	^aXeSoapXMLReader decodeAnyTypeElement: self
!

encode: anObject writer: aXeSoapXMLWriter
	"Encode anObject into the aXeSoapXMLWriter"

	^aXeSoapXMLWriter encode: anObject inAnyTypeElement: self
! !

!XeAnyTypeElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeAnyTypeElement.st,v 1.5 2008/09/03 08:52:34 cg Exp $'
! !

XeAnyTypeElement initialize!
