"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeNamedParticle subclass:#XeAttributeAbstract
	instanceVariableNames:'use'
	classVariableNames:'IsDeclare'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeAttributeAbstract comment:'

==========
Definition
From Section 2.2 at http://www.w3.org/TR/xmlschema-1/ 

"2.2.2.3 An attribute declaration is an association between a name and a simple type definition, together with occurrence information and (optionally) a default value" 
'
!


!XeAttributeAbstract class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	IsDeclare := 16
! !

!XeAttributeAbstract methodsFor:'accessing'!

use
	"Private - Answer the value of the receiver's ''use'' instance variable."

	^use
!

use: anObject
	"Private - Set the value of the receiver's ''use'' instance variable to the argument, anObject."

	use := anObject
! !

!XeAttributeAbstract methodsFor:'defaults-private'!

defaultFlags
        ^super defaultFlags bitOr: IsDeclare
        "/ ^super defaultFlags | IsDeclare
! !

!XeAttributeAbstract methodsFor:'modes'!

beDontDeclare
	self isDeclare: false
! !

!XeAttributeAbstract methodsFor:'operations'!

convertValue: anObject 
	^self subclassResponsibility
!

createAttribute: aXeNode value: anObject
	| convertValue |
	(anObject isNil and: [self isOptional]) ifTrue: [^aXeNode].
	convertValue := self convertValue: anObject.
	self isFormQualified 
		ifTrue: [aXeNode setQAttribute: self qName value: convertValue]
		ifFalse: [aXeNode setAttribute: self localName value: convertValue].
	^aXeNode
! !

!XeAttributeAbstract methodsFor:'testing'!

isDeclare
	^self flags allMask: IsDeclare
!

isDeclare: aBool
	flags := self flags mask: IsDeclare set: aBool
 
!

isDeclaredIn: schemaNode
	"Answer if the receiver has been declared in the <XeNode> schema"

	^self isDeclaredIn: schemaNode space: 'attribute'
!

isFixed
	^false
!

isOptional
	^use isNil or: [use='optional']
! !

!XeAttributeAbstract methodsFor:'visitor'!

declareLocallyUsing: aXeSchemaWriter
	"Tell the <XeSchemaWriter> to add a local attribute declaration for the receiver"

	self isDeclare ifTrue: [aXeSchemaWriter localDeclareAttribute: self]
!

encode: anObject writer: aXeSoapXMLWriter
	"Encode anObject into the aXeSoapXMLWriter"

	^aXeSoapXMLWriter encode: anObject inAttribute: self
! !

!XeAttributeAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeAttributeAbstract.st,v 1.6 2008/09/03 08:52:31 cg Exp $'
! !

XeAttributeAbstract initialize!
