"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeCompositeParticle subclass:#XeChoiceElement
	instanceVariableNames:'choices'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeChoiceElement comment:'This class is the template for an xml choice node'
!


!XeChoiceElement class methodsFor:'instance creation'!

onElements: aCollection 
	^self new onElements: aCollection 
! !

!XeChoiceElement methodsFor:'accessing'!

choices
	^choices
!

choices: aCollection
	choices := aCollection
! !

!XeChoiceElement methodsFor:'accessing-private'!

accessContent
	^self xeChildren first content
!

onElements: aCollection
	choices := aCollection
!

xeChildren
	^choices
! !

!XeChoiceElement methodsFor:'matching'!

selectChoiceForObject: anObject
	"This is limited in its ability to detect the correct choice, and it this stage will signal an error rather than
	trying to guess"

	| typeName |
	typeName := (anObject sprayTypeName) qualifiedNameIn: self typespace.
	^self xeChildren
		detect: [ :each | each content isMatchFor: typeName ]
		ifNone: [self error: 'Could not detect the correct choice for: ' , anObject displayString].
        
	
!

selectChoiceForParentObject: aXeQStruct
	^self xeChildren detect: [:each | each objectAccessor canAccessParent: aXeQStruct]
		ifNone: [self error: 'Could not detect the correct choice']
!

selectElementForNode: aXeWrappedNode 
	| elementName |
	elementName := aXeWrappedNode qName.
	^self xeChildren detect: [:each | each qName = elementName]
		ifNone: [self error: 'Could not detect the correct choice for: ' , elementName displayString]
!

selectMatchingNodeFrom: aComplexNodeTable
	| choiceAccessors match|
	choiceAccessors  := self allQNames.
	match := nil.
	aComplexNodeTable nodeTable keysAndValuesDo: [ :k :v |
		(choiceAccessors includes: k) ifTrue: [
			match notNil ifTrue: [self error: 'Could not resolve choice'].
			match := v]].
	^match
! !

!XeChoiceElement methodsFor:'testing'!

isAnonymous
	^false
! !

!XeChoiceElement methodsFor:'visitor'!

declareLocallyUsing: aXeSchemaWriter
	"Tell the <XeSchemaWriter> to add a local element declaration for the receiver"

	aXeSchemaWriter localDeclareChoice: self
!

decode: aXeSoapXMLReader
	"Answer the <Object> decoded from the aXeSoapXMLReader "

	^aXeSoapXMLReader decodeChoiceElement: self
!

encode: anObject writer: aXeSoapXMLWriter
	"Encode anObject into the aXeSoapXMLWriter"

	^aXeSoapXMLWriter encode: anObject inChoiceElement: self
!

encodeChildValueFrom: anObject writer: aXeSoapXMLWriter 
	"Select the appropriate choice for anObject, then extract my value from anObject, then encode it"

	^(self selectChoiceForParentObject: anObject)
		encodeChildValueFrom: anObject writer: aXeSoapXMLWriter 
! !

!XeChoiceElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeChoiceElement.st,v 1.4 2003/10/08 17:46:14 cg Exp $'
! !
