"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComplexFactoryAbstract subclass:#XeComplexClassAbstract
	instanceVariableNames:'mutex defined'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeComplexClassAbstract comment:'I am a factory that create <XeComponent>s that can encode/decode objects that have children (ie XMLSchema complexTypes). I do this by asking the object''s class for its #sprayElements

See XeSprayClassFactory class comments and example

==========
Instance Variables:
	mutex   <Mutex>
	defined <Collection>

The defined collection is necessary to detect circular references. The mutex is necessary because this needs to be multiple process safe.'
!


!XeComplexClassAbstract methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer a <XeComplexContent>."

	^anEncodingStyle complexSpray: aTypeQName factory: self
! !

!XeComplexClassAbstract methodsFor:'helpers-private'!

elementsDefinedFor: actualClass
	"Answer the collection of <XeElementSpecs> defined by the class"

	^self subclassResponsibility
! !

!XeComplexClassAbstract methodsFor:'initializing-private'!

initialize
        super initialize.
        "/ cg: changed to RecursionLock
        mutex := RecursionLock new.
        defined := Set new
! !

!XeComplexClassAbstract methodsFor:'operations-private'!

replaceIfCircular: elementSpec in: anEncodingStyle
	^(elementSpec doesTypeMatchAny: defined in: anEncodingStyle typespace) 
		ifTrue: [elementSpec asAnyTypedIn: anEncodingStyle typespace]
		ifFalse: [elementSpec]
! !

!XeComplexClassAbstract methodsFor:'particle creation'!

elementsFor: aTypeQName encStyle: anEncodingStyle
	"Answer the childElements <XeElement>s for aTypeQName (used to construct the XeComplexContent )
	We must handle circular definitions, and since this can be accessed by multiple processes, we use a mutex to protect the state"

	| actualClass elements |
	self assert: [aTypeQName isKindOf: XeQName].
	actualClass := self classFor: aTypeQName.
	mutex critical: 
			[| elementSpecs |
                        
			[defined add: aTypeQName.
			elementSpecs := (self elementsDefinedFor: actualClass) 
						collect: [:each | self replaceIfCircular: each in: anEncodingStyle].
			self configureElementSpecs: elementSpecs.
			elements := elementSpecs collect: [:each | each xeParticleWith: anEncodingStyle]] 
					ensure: [defined remove: aTypeQName ifAbsent: []]].
	^elements
! !

!XeComplexClassAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexClassAbstract.st,v 1.7 2010/02/26 12:04:22 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexClassAbstract.st,v 1.7 2010/02/26 12:04:22 cg Exp $'
! !
