"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentAbstract subclass:#XeComplexContent
	instanceVariableNames:'childElements convertClass'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeComplexContent comment:'I am a content component that can encode/decode complex types 

Instance Variables:
	childElements   <Collection> of <XeElement> components
	convertClass    <Class> The class of instances I will decode to.

convertClass defaults to XeQStruct, and the childElements use #at:put: to set its values. convertClass can be any other class, and the childElements can be configured to use getters and setters. The ''configuration'' of instances of this class is done by an encodingStyle object.

==========
To view an instance diagram image evaluate  showExampleImage
See example


'
!


!XeComplexContent class methodsFor:'must strip'!

example
        <resource: #example>

        "In this example we use content created from an imported schema definition. "

        | typespace element |
        typespace := XeTypespace defaultSoapEnc2001.
        typespace 
                addSchemaString: '<schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
targetNamespace="urn:myNamespace">
<complexType name="SOAPStruct">
<all>
<element name="varString" type="xsd:string"/>
<element name="varInt" type="xsd:int"/>
<element name="varFloat" type="xsd:float"/>
</all>
</complexType>
</schema>'.
        element := (XeElementSpec named: #foo typeName: ('SOAPStruct' ns: #'urn:myNamespace')) 
                                xeParticleWith: typespace soapEncStyle
! !

!XeComplexContent methodsFor:'accessing'!

accessorKeys
	"Answer a collection of <XeQName>s"

	^self childElements collect: [:each | each objectAccessor key]
!

attributes: anOrderedCollection 
	self assertAttributesValid: anOrderedCollection.
	attributes := anOrderedCollection asOrderedCollection
!

childElements
	"Answer a collection of <XeElements>"

	^childElements
!

childElements: aCollOfXeElements
	self assert: [aCollOfXeElements allSatisfy: [ :each | each isMultipleOccurs not]].
	childElements := aCollOfXeElements
!

childLocallyNamed: aString
	"Answer a child <XeElement>"

	^self childElements detect: [:each | each localName = aString]
!

childObjectAccessors
	"Answer a collection of the childElement's <XeObjectAccessor>s"

	| coll |
	coll := OrderedCollection new.
	self childElements do: [:each | coll addAll: each allObjectAccessors].
	^coll
!

convertClass: aClass
	"Set the class which the receiver will create instances of when decoding"

	convertClass := aClass
!

readerClass
	^XeComplexNodeTable
! !

!XeComplexContent methodsFor:'accessing-private'!

convertClass
	"Answer the class which the receiver will create instances of when decoding"

	^convertClass
! !

!XeComplexContent methodsFor:'adding'!

addAttributes: anOrderedCollection 
	self assertAttributesValid: anOrderedCollection.
	attributes addAll: anOrderedCollection
!

addChildElements: aCollection
	self assert: [aCollection allSatisfy: [ :each | each isMultipleOccurs not]].
	childElements addAll: aCollection
! !

!XeComplexContent methodsFor:'asserting-private'!

assertAttributesValid: anOrderedCollection
	"It is not valid in soap-enc to encode information into attributes"

	(self isSoapEnc and: [anOrderedCollection notEmpty]) 
		ifTrue: [self error: 'Attributes not valid in soapEnc']
! !

!XeComplexContent methodsFor:'initializing-private'!

initialize
	super initialize.
	childElements := OrderedCollection new.
	attributes := OrderedCollection new
! !

!XeComplexContent methodsFor:'operations'!

complexExtensionNamed: aXeQName with: collOfElements with: collOfAttributes
	"Create a new instance which is an extension of the receiver.
		The receiver is copied and the <XeElement>s and <XeAttribute>s are added"

	| newContent |
	newContent := self copyContent.
	newContent
		baseTypeName: self typeName;
		typeName: aXeQName.
	newContent addChildElements: collOfElements.
	newContent addAttributes: collOfAttributes.
	^newContent
!

copyContent
	| copy |
	copy := self copy.
	copy childElements: self childElements copy.
	^copy
!

newContentObject
	"Answer a new instance of the convertClass"

	^self convertClass sprayInstanceWith: self
! !

!XeComplexContent methodsFor:'visitor'!

acceptVisitor: aContentVisitor
	^aContentVisitor acceptComplex: self
! !

!XeComplexContent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexContent.st,v 1.6 2008/09/05 17:45:23 cg Exp $'
! !
