"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeLocalFactoryAbstract subclass:#XeComplexFactoryAbstract
	instanceVariableNames:'isElementFormQualified isAttributeFormQualified classMaps'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeComplexFactoryAbstract comment:'This class is the superclass for factories that create XeComponents that can encode/decode objects that are mutable 

==========
Instance Variables:
	isElementFormQualified  <Boolean>       Are children''s names qualified?
	isAttributeFormQualified        <Boolean>       Are attribute''s names qualified?
	classMaps               <Dictionary>    Maps typeName to class name'
!


!XeComplexFactoryAbstract class methodsFor:'instance creation'!

new
        "Answer a new initialized instance of the receiver."

        ^self basicNew initialize
! !

!XeComplexFactoryAbstract methodsFor:'configuration'!

isAttributeFormQualified: aBool
	isAttributeFormQualified := aBool
!

isElementFormQualified: aBool
	"-this is equivalent to the XMLSchema elementFormDefault switch (which controls whether local elements are qualified).
	Typically:
		-soap-enc complexTypes are 'false' (which is the default).
		-dotnet are 'true' "

	isElementFormQualified := aBool
!

mapTypeName: aTypeQName to: aClass
	"Map the typeName to the class (Without a mapping the local part of typeName is used)"

	classMaps at: aTypeQName localName put: aClass
! !

!XeComplexFactoryAbstract methodsFor:'configuration-private'!

configureAttributeSpecs: aCollection
	"configure the <XeAttributeSpecs>.
	-this is equivalent to the XMLSchema attributeFormDefault switch"

	self isAttributeFormQualified 
		ifTrue: [aCollection do: [:each | each beFormQualifiedNS: self targetNS]]
!

configureElementSpecs: aCollection
	"configure the <XeAttributeSpecs>.
	-this is equivalent to the XMLSchema elementFormDefault switch (which controls whether local elements are qualified)"

	self isElementFormQualified 
		ifTrue: [aCollection do: [:each | each beFormQualifiedNS: self targetNS]]
! !

!XeComplexFactoryAbstract methodsFor:'helpers-private'!

classFor: aTypeQName
        "Private - Answer the class that aTypeQName is mapped to. 
        This defaults to the local part of aTypeQName unless a classMap has been added to the receiver"

        | local |
        local := aTypeQName localName.
        ^classMaps at: local
            ifAbsent: 
                [self class environment at: local asSymbol
                    ifAbsent: [
                        Smalltalk at: local asSymbol
                            ifAbsent: [
                                self error: 'Can not find the class mapped to: ' , aTypeQName displayString]]]
! !

!XeComplexFactoryAbstract methodsFor:'initialize-private'!

initialize
        super initialize.
        "/ cg: changed from LookupTable
        classMaps := Dictionary new
! !

!XeComplexFactoryAbstract methodsFor:'operations-private'!

attributesDefinedFor: actualClass
	"Answer the collection of <XeAttributeSpecs> defined by the class"

	^self subclassResponsibility
! !

!XeComplexFactoryAbstract methodsFor:'particle creation'!

attributesFor: aTypeQName encStyle: anEncodingStyle
	"Answer the collection of <XeAttribute>s that the class defines"

	| attributeSpecs |
	self assert: [aTypeQName isKindOf: XeQName].
	attributeSpecs := self attributesDefinedFor: (self classFor: aTypeQName).
	self configureAttributeSpecs: attributeSpecs.
	^attributeSpecs collect: [:each | each attributeWith: anEncodingStyle]
! !

!XeComplexFactoryAbstract methodsFor:'testing-private'!

isAttributeFormQualified
	isAttributeFormQualified isNil ifTrue: [isAttributeFormQualified := false].
	^isAttributeFormQualified
!

isElementFormQualified 
	isElementFormQualified isNil ifTrue: [isElementFormQualified := false].
	^isElementFormQualified 
! !

!XeComplexFactoryAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexFactoryAbstract.st,v 1.8 2010/06/14 10:57:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexFactoryAbstract.st,v 1.8 2010/06/14 10:57:27 cg Exp $'
! !
