"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeComplexNodeTable
	instanceVariableNames:'nodeTable objectAccessors childNodes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!


!XeComplexNodeTable class methodsFor:'instance creation'!

new
        ^self basicNew initialize
!

onComplexContent: aXeComplexContent node: node
	^(self new)
		content: aXeComplexContent node: node;
		createNodeTable;
		yourself
! !

!XeComplexNodeTable methodsFor:'accessing-private'!

childNodes
	^childNodes
!

content: aXeComplexContent node: aXeNode
	objectAccessors := aXeComplexContent childObjectAccessors.
	childNodes := aXeNode childNodes
!

firstNode
	^self nodeTable values first
!

nodeTable
	^nodeTable
!

selectMatchingAccessorFor: aXeElement node: aXeNode
	"Answer an <XeObjectAccessor>
	If the XeElement isUnnamed (ie choice element) it selects the 'actual' XeElement to use"

	^aXeElement isUnnamed 
		ifTrue: [(aXeElement selectElementForNode: aXeNode) objectAccessor]
		ifFalse: [aXeElement objectAccessor]
!

selectMatchingNodeFor: aChildXeElement
	"Answer a <XeNode>
	If the XeElement isUnnamed (ie choice element) it selects the <XeNode> from the receiver's nodeTable"

	^aChildXeElement isUnnamed 
		ifTrue: [aChildXeElement selectMatchingNodeFrom: self]
		ifFalse: [nodeTable at: aChildXeElement qName ifAbsent: []]
! !

!XeComplexNodeTable methodsFor:'asserting-private'!

assertExpectedAccessor: name
	(objectAccessors anySatisfy: [:each | each isFor: name]) 
		ifFalse: [self errorUnexpectedAccessor: name]
! !

!XeComplexNodeTable methodsFor:'enumerating'!

with: aComplexContent doContent: operation
	"Enumerate over the childNodes evaluating operation with 
		-the XeElement
		-the matching XeNode
		-the object accessor
	A wild child XeElement complicates matters needs post-processing"

	(aComplexContent childElements anySatisfy: [:each | each isWild]) 
		ifTrue: [self withAny: aComplexContent doContent: operation]
		ifFalse: [self withElements: aComplexContent childElements doContent: operation]
! !

!XeComplexNodeTable methodsFor:'enumerating-private'!

withAny: aComplexContent doContent: operation
	"One of the child XeElements is wild.
		Enumerate of the children as per usual, and then evaluate operation for all childNodes 
		that were not directly handled by the named elements"

	| wildElements namedElements handledNodes |
	wildElements := OrderedCollection new.
	namedElements := OrderedCollection new.
	aComplexContent childElements 
		do: [:each | each isWild ifTrue: [wildElements add: each] ifFalse: [namedElements add: each]].
	handledNodes := self withElements: namedElements doContent: operation.
	wildElements size > 1 
		ifTrue: [self error: 'Spray does not support multiple wild elements'].
	(childNodes select: [:each | (handledNodes includes: each) not]) do: 
			[:childNode | 
			| accessorObj |
			accessorObj := self selectMatchingAccessorFor: wildElements first node: childNode.
			operation 
				value: wildElements first
				value: childNode
				value: accessorObj]
!

withElements: childElements doContent: operation
	"Enumerate over the childNodes evaluating operation with 
		-the XeElement
		-the matching XeNode
		-the object accessor"

	| handledNodes |
	handledNodes := OrderedCollection new.
	childElements do: 
			[:each | 
			| childNode accessorObj |
			handledNodes add: (childNode := self selectMatchingNodeFor: each).
			accessorObj := self selectMatchingAccessorFor: each node: childNode.
			operation 
				value: each
				value: childNode
				value: accessorObj].
	^handledNodes
! !

!XeComplexNodeTable methodsFor:'exceptions-private'!

errorUnexpectedAccessor: anAccessorName
        "Private - A Structure was received that contained a childNode we were not expecting.
        -This can be ignored, (but the source of the problem should be identified (is the server Borland Delphi?))
        Spray users can use #extractResuming to ignore this error"

        Smalltalk isSmalltalkX ifTrue:[
            XeStrictError 
                raiseRequestErrorString: 'Structure includes unexpected accessor: ' , anAccessorName displayString
        ] ifFalse:[
            XeStrictError 
                signal: 'Structure includes unexpected accessor: ' , anAccessorName displayString
        ]
! !

!XeComplexNodeTable methodsFor:'initializing-private'!

addToNodeTable: aXeNode
	"Assert that we expected a node with aXeNode's name, and then add it to the table"

	| name |
	self assertExpectedAccessor: (name := aXeNode qName).
	^nodeTable at: name put: aXeNode
!

createNodeTable
	"Create a dictionary associating the node's childElement names with its childElementNodes."

	self createNodeTableFrom: 1
!

createNodeTableFrom: index
	childNodes 
		from: index
		to: childNodes size
		do: [:each | self addToNodeTable: each]
!

initialize
        super initialize.
        "/ cg: changed from LookupTable
        nodeTable := Dictionary new
! !

!XeComplexNodeTable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexNodeTable.st,v 1.7 2010/06/14 10:57:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComplexNodeTable.st,v 1.7 2010/06/14 10:57:39 cg Exp $'
! !
