"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeComponent
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeComponent comment:'I am a component. I model information from an XMLSchema, and I am used (by visitors) to encode objects, decode xml, and write a schema description of myself.

-I am multiple process safe because I do not change state once created. I can be visited by multiple XeComponentVisitors concurrently.

==========
Definition
From Section 2.2 at http://www.w3.org/TR/xmlschema-1/ 

"2.2[Definition:] Schema component is the generic term for the building blocks that comprise the abstract data model of the schema" 

==========
Notes

Using a XeComponent (ie encoding/decoding/schema writing) is multiple process safe, and a XeComponent is typically used multiple times. 

A rule of thumb: For <foo xsi:type="ns:fooType">....</foo> 
-an XeParticle(XeElement) is responsible for everything inside the "<>" 
-an XeContent is responsible for "..." 

==========
Creation

There are multiple ways to create components:

From a typeName (most commonly used in WSDL/Soap parts). 
From an object (most commonly used when encoding an object into an xsd:anyType part) . 
From xml (most commonly used when decoding xml from an xsd:anyType part) . 

See examples



'
!


!XeComponent class methodsFor:'instance creation'!

new 
        ^self basicNew initialize
! !

!XeComponent class methodsFor:'must strip'!

examples
        <resource: #example>

        "Some examples of creating instances"

        | typespace struct |
        typespace := XeTypespace defaultSoapEnc2001.

        "Content from Base Factory"
        (XeQName xsd2001: 'string') xeContentWith: typespace soapEncStyle.

        "Same as"
        'xsd:string' asXeName xeContentWith: typespace soapEncStyle.


        "Content from Class Factory ... requires spray sample BigDog package"
        (XeQName sprayClass: (self environment at: #PurchaseOrders)) 
                xeContentWith: typespace soapEncStyle.

        "Content from Imported Schema Factory"
        "(XeQName local: 'SOAPStruct' ns: #'urn:myNamespace') xmlContentWith: typespace soapEncStyle."

        "Content from instance - simple"
        typespace soapEncStyle elementForObject: 'abc'.

        "Content from instance - complex"
        typespace soapEncStyle elementForObject: (self environment at: #PurchaseOrders) example1.

        "Content from instance - struct"
        (struct := XeQStruct new)
                at: #param1 put: 'abc';
                at: #param2 put: 123.
        typespace soapEncStyle elementForObject: struct.

        "Content from xml -simple"
        typespace soapEncStyle elementForNode: (XeEnvironment 
                                rootNodeFor: '<root xsi:type="xsd:string" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">abc</root>').

        "Content from xml -complex"
        typespace soapEncStyle elementForNode: (XeEnvironment 
                                rootNodeFor: '<root xsi:type="d:PurchaseOrders" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:d="http://www.dolphinharbor.org/spray/class" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><ordersCollection SOAP-ENC:arrayType="d:PurchaseOrder[1]"><item xsi:type="d:PurchaseOrder"><poID xsi:type="xsd:string">anID</poID><createDate xsi:type="xsd:dateTime">2001-11-24T00:44:36Z</createDate><shipTo xsi:type="d:Address"><name xsi:type="xsd:string">John Smith</name><address xsi:type="xsd:string">1 Dolphin str</address><city xsi:type="xsd:string">Object Arts</city><state xsi:type="xsd:string">Great</state><zipCode xsi:type="xsd:string">99</zipCode></shipTo><billTo xsi:type="d:Address"><name xsi:type="xsd:string">John Smith</name><address xsi:type="xsd:string">1 Dolphin str</address><city xsi:type="xsd:string">Object Arts</city><state xsi:type="xsd:string">Great</state><zipCode xsi:type="xsd:string">99</zipCode></billTo><items SOAP-ENC:arrayType="d:LineItem[2]"><item xsi:type="d:LineItem"><name xsi:type="xsd:string">Widget</name><quantity xsi:type="xsd:int">1</quantity><price xsi:type="xsd:decimal">9.99</price></item><item xsi:type="d:LineItem"><name xsi:type="xsd:string">Widget</name><quantity xsi:type="xsd:int">2</quantity><price xsi:type="xsd:decimal">15</price></item></items></item></ordersCollection></root>').

        "Content from xml - struct"
        typespace soapEncStyle elementForNode: (XeEnvironment 
                                rootNodeFor: '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><param1 xsi:type="xsd:string">abc</param1><param2 xsi:type="xsd:integer">123</param2></root>')
! !

!XeComponent class methodsFor:'spray examples'!

showExampleImage
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        Dolphin::SWEnvironment xeImageNamed: 'XeEncCompUseCases.gif'
    ]
! !

!XeComponent methodsFor:'accessing'!

childElements
	"Answer a collection of <XeElements>"

	self subclassResponsibility
!

encodingStyle
	^self subclassResponsibility
!

typespace
	"Answer the <XeTypespace> the component was created in"

	^self encodingStyle typespace
! !

!XeComponent methodsFor:'test accessing-private'!

firstChildContent
	self subclassResponsibility
! !

!XeComponent methodsFor:'testing'!

isArrayContent
        ^false
!

isSoapEnc
	^self encodingStyle isSoapEnc
! !

!XeComponent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComponent.st,v 1.9 2010/06/14 10:56:36 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeComponent.st,v 1.9 2010/06/14 10:56:36 cg Exp $'
! !
