"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: YAXO }"

XeXMLNodeAbstract subclass:#XeCompositeNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Nodes'
!


!XeCompositeNode class methodsFor:'instance creation'!

new
	^super new initialize
! !

!XeCompositeNode methodsFor:'accessing'!

childNodes
	self subclassResponsibility
!

firstChild
	^self childNodes isEmpty 
		ifTrue: []
		ifFalse: [self childNodes first]
	
!

lastChild
	^self childNodes isEmpty 
		ifTrue: []
		ifFalse: [self childNodes last]
	
! !

!XeCompositeNode methodsFor:'accessing-namespaces'!

defaultNamespace
	self subclassResponsibility
! !

!XeCompositeNode methodsFor:'enumerating'!

preOrderElementDo: aBlockClosure 
	self subclassResponsibility
! !

!XeCompositeNode methodsFor:'searching'!

detectAllNodes: aLocalName in: aNSSymbol
	^self 
		detectAllNodes: aLocalName
		in: aNSSymbol
		ifNone: [self errorNotFound: aLocalName in: aNSSymbol]
!

detectAllNodes: aLocalName in: aNSSymbol ifNone: operation
	"Answer the first node, travelling across the whole tree (preOrder), to be named a localName 
	in aNamespace. ie detect <abc:aLocalName xmlns:abc='aNSSymbol'>
	-If aNSSymbol is nil, answer the first node to have the specified localName."

	self 
		preOrderElementDo: [:each | (each matchLocal: aLocalName ns: aNSSymbol) ifTrue: [^each]].
	^operation value
!

detectChildNode: aLocalName in: aNSSymbol
	^self 
		detectChildNode: aLocalName
		in: aNSSymbol
		ifNone: [self errorNotFound: aLocalName in: aNSSymbol]
!

detectChildNode: aLocalName in: aNSSymbol ifNone: operation
	"Answer the first node, of this nodes childNodes, to have a localName
	equal to aLocalName in the aNSSymbol namespace . If aNSSymbol is nil, 
	answer the first node to have the specified localName.
	This ignores prefixes"

	^self childNodes detect: [:each | each matchLocal: aLocalName ns: aNSSymbol]
		ifNone: operation
!

detectFirstChildNode: aLocalName in: aNamespace ifNone: operation 
	| cn |
	cn := self firstChild.
	cn isNil ifTrue: [^operation value].
	^(cn matchLocal: aLocalName ns: aNamespace) 
		ifTrue: [cn]
		ifFalse: 
			[cn 
				detectFirstChildNode: aLocalName 
				in: aNamespace
				ifNone: operation]
!

selectAllNodes: aLocalName in: aNSSymbol
	"Answer the all descendant nodes, having a localName 
	equal to aLocalName in the aNSSymbol namespace . If aNSSymbol is nil, 
	answer the first node to have the specified localName.
	This ignores prefixes"

	| coll |
	coll := OrderedCollection new.
	self 
		preOrderElementDo: [:each | (each matchLocal: aLocalName ns: aNSSymbol) ifTrue: [coll add: each]].
	^coll
!

selectChildNodes: aLocalName in: aNSSymbol
	"Answer the immediate child nodes, having a localName 
	equal to aLocalName in the aNSSymbol namespace . If aNSSymbol is nil, 
	answer the first node to have the specified localName.
	This ignores prefixes"

	^self childNodes select: [:each | each matchLocal: aLocalName ns: aNSSymbol]
! !

!XeCompositeNode methodsFor:'searching-private'!

errorNotFound: aLocalName in: aNSSymbol
	^self errorNotFound: aLocalName displayString , ' in ' , aNSSymbol displayString
!

matchLocal: aLocalName ns: aNSSymbol
	"Private - Answer whether the node has the localName and aNSSymbol"

	^self localName = aLocalName and: [aNSSymbol isNil or: [self namespace == aNSSymbol]]
!

nextSiblingOf: aNode
	| index |
	index := self childNodes indexOf: aNode.
	^index = self childNodes size
		ifTrue: [] 
		ifFalse: [self childNodes at: index + 1]
!

previousSiblingOf: aNode
	| index |
	index := self childNodes indexOf: aNode.
	^index <= 1 
		ifTrue: [] 
		ifFalse: [self childNodes at: index - 1]
! !

!XeCompositeNode methodsFor:'testing-private'!

isInDefaultNSFor: aXeQName
	^aXeQName isNullNamespace or: [self defaultNamespace = aXeQName namespace]
! !

!XeCompositeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeCompositeNode.st,v 1.4 2002/11/19 17:12:11 cg Exp $'
! !
