"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComponent subclass:#XeContentAbstract
	instanceVariableNames:'typeName baseTypeName encodingStyle attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeContentAbstract comment:'
==========
Definition
From Section 2.2 at http://www.w3.org/TR/xmlschema-1/ 

"2.2.1.3 [Definition:] A particle can be used in a complex type definition to constrain the validation of the [children] of an element information item; such a particle is called a content model." 
'
!


!XeContentAbstract methodsFor:'accessing'!

attributes
	^attributes
!

baseTypeName
	"Answer nil or a <XeQName> if the receiver has a baseType"

	^baseTypeName
!

baseTypeName: aXeName
	baseTypeName := aXeName
!

childElements
	"Answer a collection of <XeElements>"

	^#()
!

elementClass
	^XeElement
!

encodingStyle
	"Answer the XeEncodingStyle the receiver was created in"

	^encodingStyle
!

encodingStyle: aXeEncodingStyle
	encodingStyle := aXeEncodingStyle
!

localName
	^self typeName localName
!

namespace
	^self typeName namespace
!

typeDisplayString
        ^self typeName printString "displayString"
!

typeName
	^typeName
!

typeName: aXeQName 
	typeName := aXeQName
! !

!XeContentAbstract methodsFor:'accessing-private'!

typespaceOptions
	^self typespace options
! !

!XeContentAbstract methodsFor:'adding'!

addFixedAttribute: aXeFixedAttribute
	"Add a <XeFixedAttribute> which encodes the same each encoding"

	| removes |
	removes := self attributes select: [:each | each qName = aXeFixedAttribute qName].
	removes do: [:each | self attributes remove: each].
	self attributes add: aXeFixedAttribute
! !

!XeContentAbstract methodsFor:'comparing'!

= aXeContent
	^self species = aXeContent species 
		and: [self isNamed and: [aXeContent isNamed and: [self typeName = aXeContent typeName]]]
!

hash
	^self typeName hash
! !

!XeContentAbstract methodsFor:'initializing-private'!

initialize
	super initialize.
	attributes := OrderedCollection new
! !

!XeContentAbstract methodsFor:'modes'!

beAllNotTyped
	"Turn off inline typing for the receiver and all it's children."

	self childElements do: [:each | each beAllNotTyped]
!

beAllTyped
	"Turn on inline typing for the receiver and all it's children."

	self childElements do: [:each | each beAllTyped]
! !

!XeContentAbstract methodsFor:'operations'!

checkTypeMismatch: aNode
	"Note for Spray user, if this signals an error decoding a response, try sending the message #extractResuming to your SpwSoapSend"

	(self isTypeMismatch: aNode) 
		ifTrue: 
			[XeTypeMismatch 
				signal: ('Expected type: %1 received: %2' formatWith: self typeDisplayString
						with: aNode qValueForXSIType displayString)]
!

complexExtensionNamed: aXeQName with: aColl1 with: aColl2
	self error: 'Complex extension of this type not supported'
!

convertToObject: text
	self error: 'Conversion error, content must be simple'
!

convertValue: anObject
	self error: 'Conversion error, content must be simple'
!

putXSITypeAttributeOn: aXeNode
	"Add the type attribute in the xsi namespace, which is used in instance documents."

	self isNamed ifTrue: [aXeNode setAttribute: 'xsi:type' qName: self typeName]
! !

!XeContentAbstract methodsFor:'operations-private'!

copyContent
	^self copy
! !

!XeContentAbstract methodsFor:'test accessing-private'!

firstChildContent
	^self firstChildElement content
!

firstChildElement
	^self childElements first
! !

!XeContentAbstract methodsFor:'testing'!

isAnyContent
	^false
!

isDefinedIn: aSchema
	"Answer if the receiver has been defined in the <XeNode> schema"

	^aSchema childNodes anySatisfy: 
			[:each | 
			(#('complexType' 'simpleType') includes: each localName) 
				and: [each nameAttribute = self typeName localName]]
!

isMatchFor: aXeQName 
	^self isNamed ifTrue: [self mismatchPolicy isContent: self matched: aXeQName] ifFalse: [false]
!

isNamed
	^self typeName notNil
!

isSimpleContent 
	^false
!

isSimpleRestrictedContent
        ^false

    "Created: / 20-03-2012 / 15:51:00 / cg"
!

isSoapSpecialMultiRef
	"Answer if the receiver's type is one of the SOAP1.1 spec's 'special' types that allows inline multireferences"

	^false
!

isTargetNS: aNSSymbol
	^self isNamed and: [self namespace = aNSSymbol]
!

isTypeMismatch: aXeNode
	"We can only answer true if we have a typeName, and the node is inline typed"

	| nodeTypeQName |
	^(self isNamed and: [(nodeTypeQName := aXeNode qValueForXSIType) notNil]) 
		ifTrue: [(self mismatchPolicy isContent: self matched: nodeTypeQName) not]
		ifFalse: [false]
! !

!XeContentAbstract methodsFor:'testing-private'!

mismatchPolicy
	"Private - Answer a <XeTypeMismatchPolicy> to use to check inline type attributes against the receivers type"

	^self encodingStyle mismatchPolicy
! !

!XeContentAbstract methodsFor:'visitor'!

acceptVisitor: aContentVisitor
	^self subclassResponsibility
! !

!XeContentAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeContentAbstract.st,v 1.6 2012/03/20 14:52:09 cg Exp $'
! !
