"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: YAXO }"

XeCompositeNode subclass:#XeDoc
	instanceVariableNames:'documentElement instructions comments prologAttributes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Nodes'
!

XeDoc comment:'I am an xml document'
!


!XeDoc methodsFor:'accessing'!

childNodes
        #cg. "/ ifNotNil passes no argument in ST/X - sigh .

        ^documentElement 
                ifNil: [Array new]
                ifNotNil: [":t | "Array with: documentElement]
!

documentElement
	^documentElement
!

nextSibling
	^nil
!

ownerDocument
	^self
!

previousSibling
	^nil
!

prologAttributes
	^prologAttributes
! !

!XeDoc methodsFor:'accessing-namespaces'!

defaultNamespace
	^#''
!

targetNamespace
	self error: 'Could not locate the targetNamespace'
! !

!XeDoc methodsFor:'accessing-namespaces-private'!

namespaceFor: aPrefixString
	^#''
! !

!XeDoc methodsFor:'accessing-private'!

childIndexOf: aXeNode
	aXeNode == documentElement ifFalse: [self errorNotFound: aXeNode displayString].
	^1
!

comments
	comments isNil ifTrue: [comments := OrderedCollection new].
	^comments
!

instructions
	instructions isNil ifTrue: [instructions := OrderedCollection new].
	^instructions
!

prefixFor: aNSSymbol
	^''
!

prologAttributes: anObject
	anObject at: 'encoding' ifPresent: [ :each |
		each asUppercase = 'UTF-8' ifFalse: [Notification signal: 'Received non utf-8 encoding: ' , each displayString]].
	prologAttributes := anObject
! !

!XeDoc methodsFor:'adding'!

addComment: aString 
	^self comments add: aString
!

addElement: aString
	documentElement notNil ifTrue: [self error: 'A Document can only have one child'].
	^documentElement := (XeNode nodeName: aString) parentNode: self
!

addElementNamed: aXeQName rootDeclare: aBool
	"Add an element. If the QName is qualified, check the namespace has been declared, and if not declare it. Since we are adding the documentElement rootDeclare has no effect"

	| node |
	(self isInDefaultNSFor: aXeQName) ifTrue: [^self addElement: aXeQName localName].
	node := self addElement: aXeQName qString.
	node addNSAttribute: aXeQName rootDeclare: true.
	^node
!

addInstruction: aString 
	"Add a Processing instruction node."

	^self instructions add: aString 
!

addNode: aXeNode
	documentElement := aXeNode parentNode: self
! !

!XeDoc methodsFor:'adding-private'!

removeXMLInstructions
	instructions := OrderedCollection new
! !

!XeDoc methodsFor:'converting'!

asParserNode
	| env|
	env := self class environment at: #XeEnvironment ifAbsent: [].
	^(env isNil or: [env isYAXOParser])
		ifTrue: [self]
		ifFalse: [env docNodeFor: self xml]

! !

!XeDoc methodsFor:'enumerating'!

preOrderElementDo: aBlockClosure 
!

upToRootDo: aBlockClosure 
! !

!XeDoc methodsFor:'helpers'!

browserView
	"Open the xml of the envelope in IE
	-Creates a temporary file and uses shellOpen.
	Note: the temporary file is not deleted "

	self xml browserView: 'xml'
! !

!XeDoc methodsFor:'initializing-private'!

initialize
	super initialize.
	self addInstruction: '?xml version="1.0"?'
! !

!XeDoc methodsFor:'operations'!

replaceChild: aXeNode oldChild: aXeNode2
	aXeNode2 == self documentElement 
		ifFalse: [self error: 'old child is not documentElement'].
	documentElement := aXeNode
! !

!XeDoc methodsFor:'printing-private'!

printCommentsXMLOn: aStream
	self comments do: 
			[:each | 
			aStream
				nextPutAll: '<!!--';
				nextPutAll: each;
				nextPutAll: '-->']
!

printInstructionXMLOn: aStream
	self instructions do: 
			[:each | 
			aStream
				nextPut: $<;
				nextPutAll: each;
				nextPut: $>]
		separatedBy: [aStream cr]
!

printXMLOn: aWriteStream cr: aBool
	instructions notNil ifTrue: [self printInstructionXMLOn: aWriteStream].
	comments notNil ifTrue: [self printCommentsXMLOn: aWriteStream].
	documentElement notNil ifTrue: [documentElement printXMLOn: aWriteStream cr: aBool]
! !

!XeDoc methodsFor:'searching-private'!

locateInParserDoc: aXeDoc
	^aXeDoc
! !

!XeDoc methodsFor:'testing'!

hasDocumentElement
	^documentElement notNil
!

isDocument
	^true 
! !

!XeDoc methodsFor:'testing-private'!

isPrefixDeclared: aString
	^false
! !

!XeDoc class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeDoc.st,v 1.5 2002/11/20 09:54:24 cg Exp $'
! !
