"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeSpecification subclass:#XeElementSpec
	instanceVariableNames:'minOccurs maxOccurs nillable'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeElementSpec comment:'I model the information of a XMLSchema element declaration. 

I describe and can create a <XeElement>. I use late-qualifying typeNames. This allows the same instance to create different <XeElements> in different typespaces. A practical example of this would be:

		XeElementSpec named: #foo typeName: ''xsd:string''
        
	A typespace by default is configured to map ''xsd'' to the 2001 XMLSchema namespace, 
	and therefore this elementSpec will create an <XeElement> with the 2001 Schema string type. 
	If the typespace is re-configured to map ''xsd'' to the 1999 XMLSchema namespace, 
	the <XeElement> will be created with the old namespace.

==========
Instance Variables:
	minOccurs       <String> (nonNegativeInteger )
	maxOccurs       <String> (nonNegativeInteger | unbounded)  
	nillable                <Boolean>





'
!


!XeElementSpec class methodsFor:'instance creation'!

anon: aName
	^self new elementName: aName asQName
!

named: aName arrayOfClass: aClass
        "Answer an instance with the specified name and type.
        The type is constructed as a late-qualifying XePName from the class name.
                It uses the Spray convention of prefixing classes with 'd'.
                In a default typespace, the 'd' prefix is associated with a namespace that is targeted by a <XeSprayClassFactory>.
                A <XeSprayClassFactory> can create definitions for classes (by accessing #sprayElements) and can also create definitions for Array's of classes"

        Smalltalk isSmalltalkX ifTrue:[
            ^self named: aName typeName: 'd:ArrayOf' , aClass nameWithoutPrefix
        ].
        ^self named: aName typeName: 'd:ArrayOf' , aClass name asString
!

named: aName arrayOfXSD: aString2
        "Answer an instance with the specified name and type.
        The type is constructed as a late-qualifying XePName using the WSDL convention of 'ArrayOf???'"

(aString2 includes:$:) ifTrue:[self halt:'debugHalt'].
        ^self named: aName typeName: 'soapE:ArrayOf' , aString2
!

named: aName class: aClass
        "Answer an instance with the specified name and type.
        The type is constructed as a late-qualifying XePName from the class name.
                It uses the Spray convention of prefixing classes with 'd'.
                In a default typespace, the 'd' prefix is associated with a namespace that is targeted by a <XeSprayClassFactory>.
                A <XeSprayClassFactory> can create definitions for classes (by accessing #sprayElements) and can also create definitions for Array's of classes"

        Smalltalk isSmalltalkX ifTrue:[
            ^self named: aName typeName: 'd:' , aClass nameWithoutPrefix asString
        ].

        ^self named: aName typeName: 'd:' , aClass name asString
! !

!XeElementSpec methodsFor:'accessing'!

elementName
	^qName
!

elementName: aXeQName 
	qName := aXeQName
!

maxOccurs
	^maxOccurs
!

maxOccurs: aString
	maxOccurs := aString
!

minOccurs
	^minOccurs
!

minOccurs: aString
	minOccurs := aString
!

nillable
	^nillable
!

nillable: aBoolean
	nillable := aBoolean
! !

!XeElementSpec methodsFor:'converting'!

asAnyTypedIn: aTypespace
	^self copyWithTypeName: (aTypespace anyTypeName)
!

copyWithTypeName: aXeQName
	^self class new named: self elementName typeName: aXeQName
! !

!XeElementSpec methodsFor:'initializing'!

initialize
	"nillable=false is ignored in soap-enc (by section 5 rules). This sets up the same default as xsd:schema for literal encoding"

	super initialize.
	nillable := false
! !

!XeElementSpec methodsFor:'modes'!

beNotNillable
	"Set the receiver to be nillable='false', and disallow nil instances    (should always be true in soap-enc)"

	nillable := false
! !

!XeElementSpec methodsFor:'particle creation'!

xeParticleWith: anEncodingStyle
	"Answer a <XeElement> 
	The receiver is functioning as a XMLSchema element declaration"

	^anEncodingStyle elementSpeced: self
! !

!XeElementSpec methodsFor:'testing'!

doesTypeMatchAny: aSet in: aTypespace
	"Sent by a <XeSprayClassFactory> to detect circular class references."

	^aSet includes: (self qualifiedTypeNameIn: aTypespace)
!

isNillable
	^nillable notNil and: [nillable]
!

isOmitable
	^self minOccurs = 0 or: [self minOccurs = '0']
!

isUnique
	^maxOccurs isNil or: [maxOccurs = '1']
! !

!XeElementSpec class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeElementSpec.st,v 1.6 2003/10/08 17:45:26 cg Exp $'
! !
