"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeEncodingStyle
	instanceVariableNames:'typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeEncodingStyle comment:'I am responsible for the creation of XeParticles and XeContent (components). All ContentFactorys dispatch back to me and ask me to create their components. This allows the same typeName to create different components depending on the encodingStyle. This is necessary because XeSoapEncodingStyle (which implements SOAP section 5) takes precedence over the XMLSchema specification.

-I am multiple process safe because I do not change state once created.

I can also be asked to provide XeComponents for an object (#elementForObject:) or an xml instance (#elementForNode:)

==========
Background

From the Soap1.2 Specification:
	"The SOAP encoding rules define a serialization mechanism that can be used to exchange instances of application-defined datatypes"

In a Soap envelope the use of these rules is indicated by the attribute:
	SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"

Again from the Soap specification: 
	"A value of the zero-length URI ("") explicitly indicates that no claims are made for the encoding style of contained elements. This can be used to turn off any claims from containing elements."

XeEncoding uses instances of the following classes:
	XeSoapEncodingStyle for soap-enc "http://schemas.xmlsoap.org/soap/encoding/". 
	XeSoapEncodingNoRefStyle for soap-enc with id/href encoding. 
	XeLiteralStyle for literal. 

When a typespace or schema is asked to create content or particles, an encoding style is always supplied.
See the following method in XeSchema:
	#contentOf:aTypeQName encStyle: anEncodingStyle 
	#elementDeclared: aXeQName encStyle: anEncodingStyle 
	#attributeDeclared: aXeQName encStyle: anEncodingStyle 

==========
Why do you not read the schema definitions directly?

A schema definition can not be used directly because in soap-enc style, section 5 rules take precedence over the schema definition. This is discussed in the WSDL specification at http://www.w3.org/TR/wsdl#_types. My understanding is that the ambiguities in this section will be addressed in future WSDL version, but the general consensus is that for soap-enc, schema definitions serve as a guide as to the form of a Struct or Array, and they can not be used to validate their xml instances.

==========
Design
I believe this design follows the principle of "once only". Whether a definition is sourced from a ClassFactory  or an imported schema factory or a BaseFactory, they all dispatch back to the encodingStyle to create the actual XeComponents.







'
!


!XeEncodingStyle methodsFor:'accessing'!

anyTypeName
	^self typespace anyTypeName
!

mismatchPolicy
	^self subclassResponsibility
!

typespace
	^typespace
!

typespace: aXeTypespace 
	typespace := aXeTypespace 
! !

!XeEncodingStyle methodsFor:'accessing-private'!

complexContentClass
	^self subclassResponsibility
!

simpleContentClass
	^XeSimpleContent
!

simpleRestrictionContentClass
	^XeSimpleRestrictedContent
! !

!XeEncodingStyle methodsFor:'attribute creation'!

attributeSpeced: aXeAttributeSpec 
	| typeName attribute content |

	typeName := aXeAttributeSpec qualifiedTypeNameIn: self typespace.
	content := self contentOf: typeName.
	(content isSimpleContent or: [content isAnyContent])ifFalse: [self error: 'Attribute content must be simple'].
	attribute := aXeAttributeSpec attributeClass name: aXeAttributeSpec attributeName content: content.
	aXeAttributeSpec isFixed ifTrue: [attribute fixedValue: aXeAttributeSpec fixed].
	attribute isFormQualified: aXeAttributeSpec isFormQualified.
	attribute use: aXeAttributeSpec use.
	^attribute
! !

!XeEncodingStyle methodsFor:'configuration-private'!

configureComplexAttributes: attributes
	^self subclassResponsibility
!

configureComplexChildElements: attributes
	^self subclassResponsibility
!

configureSprayAttributes: attributes
	^self subclassResponsibility
!

configureSprayChildElements: attributes
	^self subclassResponsibility
! !

!XeEncodingStyle methodsFor:'content creation-complex'!

complex: aXeQName extending: aXeQName2 xsdContent: aXeXSDExtensionContent
	"Answer a <XeComplexContent> (used by a <XeXSDSchema> factory)"

	| baseContent elements attributes |
	baseContent := aXeQName2 xeContentWith: self.
	elements := aXeXSDExtensionContent elementsWith: self.
	attributes := aXeXSDExtensionContent attributesWith: self.
	self configureComplexChildElements: elements.
	self configureComplexAttributes: attributes.
	^baseContent 
		complexExtensionNamed: aXeQName
		with: elements
		with: attributes
!

complex: aTypeQName restricting: aBaseQName xsdContent: anXSDContentModel
	"Answer a <XeComplexContent> (used by a <XeXSDSchema> factory)"

	self subclassResponsibility
!

complexSpray: aTypeQName factory: aXeSprayClassFactory
	"Answer a <XeComplexContent> (used by a <XeSprayClassFactory>)"

	^self 
		complexSpray: aTypeQName
		class: (aXeSprayClassFactory classFor: aTypeQName)
		elements: (aXeSprayClassFactory elementsFor: aTypeQName encStyle: self)
		attributes: (aXeSprayClassFactory attributesFor: aTypeQName encStyle: self)
		factory: aXeSprayClassFactory
!

complexSpray: aTypeQName restricting: aBaseName instance: aKeyedCollection
	"Create a complex restriction. The content is local, so we just do what we are asking."

	^self complex: aTypeQName
		elements: (aKeyedCollection associations 
				collect: [:each | self elementSpeced: (XeElementSpec anon: each key) content: self anyContent])
! !

!XeEncodingStyle methodsFor:'content creation-complex-private'!

complex: aTypeQName elements: elements
	"Answer a <XeComplexContent>"

	^self 
		complex: aTypeQName
		elements: elements
		attributes: OrderedCollection new
!

complex: aTypeQName elements: elements attributes: attributes
	"Answer a <XeComplexContent>"

	^(self areComplexUniqueElements: elements attributes: attributes) 
		ifTrue: 
			[self 
				complexUnique: aTypeQName
				elements: elements
				attributes: attributes]
		ifFalse: 
			[self 
				complexMultiples: aTypeQName
				elements: elements
				attributes: attributes]
!

complexMultiples: aTypeQName elements: elements attributes: attributes
	"Answer a <XeArrayContent >"

	elements size ~= 1 
		ifTrue: [self error: 'Spray can only create complex types with a single unbounded element'].
	attributes notEmpty 
		ifTrue: 
			[self 
				error: 'Spray can not create a complex type with both an unbounded element and attributes'].
	self configureComplexChildElements: elements.
	^(XeArrayContent new)
		encodingStyle: self;
		typeName: aTypeQName;
		childElement: elements first;
		yourself
!

complexRPC: aTypeQName elements: aCollection
	"Private - Answer a <XeSoapRPCContent>"

	self subclassResponsibility
!

complexRPC: aTypeQName specs: aCollection
	"Private - Answer a <XeSoapRPCContent>"

	^self complexRPC: aTypeQName
		elements: (aCollection collect: [:each | each xeParticleWith: self])
!

complexSpray: aTypeQName class: aClass elements: elements attributes: attributes factory: anObject
	"Private - Answer a <XeComplexContent> (used by a <XeSprayClassFactory>)"

	self configureSprayChildElements: elements.
	self configureSprayAttributes: attributes.
	^(XeComplexContent new)
		encodingStyle: self;
		typeName: aTypeQName;
		childElements: elements;
		attributes: attributes;
		convertClass: aClass;
		yourself
!

complexUnique: aTypeQName elements: elements attributes: attributes
	"Answer a <XeComplexContent>"

	self configureComplexChildElements: elements.
	self configureComplexAttributes: attributes.
	^(XeComplexContent new)
		encodingStyle: self;
		typeName: aTypeQName;
		childElements: elements;
		attributes: attributes;
		convertClass: self complexContentClass;
		yourself
! !

!XeEncodingStyle methodsFor:'content creation-general'!

anyContent
	^(XeAnyContent new)
		typeName: self anyTypeName;
		encodingStyle: self;
		yourself
!

contentOf: aTypeQName
	^self typespace xeContentOf: aTypeQName encStyle: self
!

nodeContent
	^(XeNodeContent new)
		typeName: nil;
		encodingStyle: self;
		yourself
! !

!XeEncodingStyle methodsFor:'content creation-simple'!

simple: aTypeQName base: aBaseTypeQName converter: aConverter
	"Answer a <XeSimpleContent>"

	^self simpleContentClass 
		onConverter: aConverter
		typeName: aTypeQName
		base: aBaseTypeQName
		encodingStyle: self
!

simple: aTypeQName converter: aConverter
	"Answer a <XeSimpleContent>"

	^self 
		simple: aTypeQName
		base: self anyTypeName
		converter: aConverter
!

simple: aTypeQName extending: aBaseTypeQName xsdContent: anXSDContent
	"Answer a <XeSimpleExtendedContent> (used by a <XeXSDSchema> factory)"

	self subclassResponsibility
!

simple: aTypeQName restricting: aTypeQName2 with: aCollOfRestrictions
	"Answer a <XeSimpleRestrictedContent>"

	^(self simpleRestrictionContentClass new)
		typeName: aTypeQName;
		baseContent: (aTypeQName2 xeContentWith: self);
		restrictions: aCollOfRestrictions;
		encodingStyle: self;
		yourself
!

simpleSpray: aTypeQName class: aClass baseContent: aXeContent attributes: attributes factory: aFactory
	"Answer a <XeSimpleExtendedContent >"

	self configureSprayAttributes: attributes.
	^(XeSimpleExtendedContent new)
		encodingStyle: self;
		typeName: aTypeQName;
		baseContent: aXeContent;
		attributes: attributes;
		convertClass: aClass;
		yourself
!

simpleSpray: aTypeQName factory: aFactory
	"Answer a <XeSimpleExtendedContent >"

	^self 
		simpleSpray: aTypeQName
		class: (aFactory classFor: aTypeQName)
		baseContent: (aFactory baseContentFor: aTypeQName encStyle: self)
		attributes: (aFactory attributesFor: aTypeQName encStyle: self)
		factory: aFactory
! !

!XeEncodingStyle methodsFor:'helpers'!

newNode: aString
	^self typespace newNode: aString
!

schemaTargeting: aNamespace 
	^self typespace schemaTargeting: aNamespace 
! !

!XeEncodingStyle methodsFor:'particle creation'!

anyElementNamed: aString
	^(XeElementSpec anyNamed: aString) xeParticleWith: self
!

elementForObject: anObject
	"Answer an XeElement for anObject. 
	Query anObject for its xmlContent in the receivers style."

	| element |
	element := self elementSpeced: (self elementSpecForObject: anObject)
				content: (anObject xeContentWith: self).
	self shouldInlineType ifTrue: [element beAllTyped].
	element beEncodeBelow.
	^element
!

elementSpeced: aXeElementSpec 
	| typeName|
	typeName := aXeElementSpec qualifiedTypeNameIn: self typespace.
	^self elementSpeced: aXeElementSpec content: (self contentOf: typeName) 
	
!

elementSpeced: aXeElementSpec anon: aContent 
	^self elementSpeced: aXeElementSpec content: aContent 
!

elementSpeced: aXeElementSpec content: content
	| element |

	element := content elementClass name: aXeElementSpec elementName content: content.
	element isNillable: aXeElementSpec isNillable.
	element isOmitable: aXeElementSpec isOmitable.
	element isFormQualified: aXeElementSpec isFormQualified.
	element isMultipleOccurs: aXeElementSpec isUnique not.
	^element
	
! !

!XeEncodingStyle methodsFor:'particle creation-private'!

elementSpecForObject: anObject
        Smalltalk isSmalltalkX ifTrue:[
            ^XeElementSpec anon: anObject class nameWithoutPrefix asQName
        ].
        ^XeElementSpec anon: anObject class name asQName
! !

!XeEncodingStyle methodsFor:'testing'!

isIDHref
	"Answer if the receiver allows identical objects to be encoded using soap-enc multireferences"

	^self subclassResponsibility
!

isLiteralStyle
	^false
!

isSoapEnc
	^false
!

shouldInlineType
	^self subclassResponsibility
! !

!XeEncodingStyle methodsFor:'testing-private'!

areComplexUniqueElements: elements attributes: attributes
	^(elements anySatisfy: [:each | each isMultipleOccurs]) not
!

isArrayTypeName: aTypeQName
	^aTypeQName localName beginsWith: 'ArrayOf'
! !

!XeEncodingStyle methodsFor:'wsdl'!

docElementsSpeced: elementSpecs
	"Answer a collection of <XeElement>s"

	^self subclassResponsibility
!

rpcWrapperNamed: aQName elementSpecs: elementSpecs
	"Answer a <XeElement> with <XeSoapRPCContent>"

	^self subclassResponsibility
! !

!XeEncodingStyle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeEncodingStyle.st,v 1.5 2003/10/08 17:44:04 cg Exp $'
! !
