"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeEncodingStyle subclass:#XeLiteralStyle
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeLiteralStyle comment:'I am the literal encoding style.'
!


!XeLiteralStyle methodsFor:'accessing'!

mismatchPolicy
	^XeSoapTypeMismatchPolicy new

!

namespace
	^#''
! !

!XeLiteralStyle methodsFor:'accessing-private'!

complexContentClass
	^XeQStruct
! !

!XeLiteralStyle methodsFor:'configuration-private'!

configureComplexAttributes: attributes
	attributes do: [:each | each beAttributesKeyedAccessor]
!

configureComplexChildElements: elements
	elements do: 
			[:each | 
			each
				beEncodeBelow;
				beKeyedAccessor;
				beRootDeclare]
!

configureSprayAttributes: attributes
	attributes do: [:each | each beSelectorAccessor]
!

configureSprayChildElements: elements
	elements do: 
			[:each | 
			each
				beEncodeBelow;
				beSelectorAccessor;
				beRootDeclare
			"beNotFormQualified;"
			"beNillable"]
! !

!XeLiteralStyle methodsFor:'content creation-complex'!

complex: aTypeQName restricting: aBaseQName xsdContent: anXSDContentModel
	"Answer a <XeComplexContent> (used by a <XeXSDSchema> factory)"

	| elements attributes |
	self assert: [aBaseQName localName = 'anyType'].
	elements := anXSDContentModel elementsWith: self.
	attributes := anXSDContentModel attributesWith: self.
	^self 
		complex: aTypeQName
		elements: elements
		attributes: attributes
!

complexSpray: aTypeQName factory: aFactory
	"Answer a <XeComplexContent> (used by a <XeSprayClassFactory>)"

	(self isArrayTypeName: aTypeQName) 
		ifTrue: [self error: 'Arrays can only be used in SoapEncStyle'].
	^super complexSpray: aTypeQName factory: aFactory
! !

!XeLiteralStyle methodsFor:'content creation-complex-private'!

complexRPC: aTypeQName elements: elements
	"Private - Answer a <XeSoapRPCContent>"

	self configureComplexChildElements: elements.
	^(XeSoapRPCContent new)
		typeName: aTypeQName;
		childElements: elements;
		encodingStyle: self;
		convertClass: XeQStruct;
		yourself
! !

!XeLiteralStyle methodsFor:'content creation-simple'!

simple: aTypeQName extending: aBaseTypeQName xsdContent: anXSDContent
	"Answer a <XeSimpleExtendedContent> (used by a <XeXSDSchema> factory)"

	| attributes |
	attributes := anXSDContent attributesWith: self.
	self configureComplexAttributes: attributes.
	^(XeSimpleExtendedContent new)
		typeName: aTypeQName;
		baseContent: (aBaseTypeQName xeContentWith: self);
		attributes: attributes;
		encodingStyle: self;
		yourself
! !

!XeLiteralStyle methodsFor:'particle creation'!

elementForNode: aNode 
	^XeGuessContent guessElementNamed: aNode qName encodingStyle: self
! !

!XeLiteralStyle methodsFor:'testing'!

isIDHref
	"Answer if the receiver allows identical objects to be encoded using soap-enc multireferences"

	^false
!

isLiteralStyle
	^true
!

shouldInlineType
	^false
! !

!XeLiteralStyle methodsFor:'wsdl'!

docElementsSpeced: elementSpecs
	"Public - Answer a collection of XeElements created from a Collection of XeElementSpecs and configured to encode and decode  Document/Literal soapBlocks.
	Note: Configured to not add inline type attributes"

	^elementSpecs collect: 
			[:each | 
			| element |
			element := each xeParticleWith: self.
			element
				beEncodeBelow;
				beAllNotTyped;
				beFormQualified;
				yourself]
!

rpcWrapperNamed: aQName elementSpecs: elementSpecs
	"self error: 'Spray does not support RPC/Literal'"
	| element |
	element := self elementSpeced: (XeElementSpec anon: aQName)
		content: (self complexRPC: nil specs: elementSpecs).
	element
		beEncodeBelow;
		beFormQualified.

	element qName prefix: 'm'.
	^element

! !

!XeLiteralStyle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeLiteralStyle.st,v 1.4 2003/10/08 17:42:59 cg Exp $'
! !
