"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeName
	instanceVariableNames:'prefix localName'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Names'
!

!XeName class methodsFor:'documentation'!

documentation
"
    Spray and Splash share a naming system that allows late qualification of a prefix,
    and the convenience of using Strings and Symbols.

    A Qualifying Context contains a dictionary which maps prefixes to namespaces.

    A class that has this protocol in Spray is: XeTypespace
    Classes that have this protocol in Splash are: SplashRequest and SplashResponse
    The Transport framework class: XeQProperty also has this protocol.

    The protocol is:
      -addPrefix:namespace
      -qualify:

    A 'Name' can either be:
      - An Instance of XeQName (ie XeQName local: 'foo' ns: #'urn:blah'). This is a fully qualified name
      - An Instance of XePName (ie XeQName local: 'ns1:foo' ). This will be late qualified.
      - A String or Symbol that contain a colon (ie 'ns2:foo'). This will be late qualified.
      - A String or Symbol that does not contain a colon (ie 'foo'). This will be late qualified into the null namespace.

    ==========
    Example (Splash)

        |request|
        request := SplashDOCRequest new.
        request addPrefix: 'b' namespace: #'urn:blah'.
        request addPrefix: 'c' namespace: #'urn:clah'.
        request addDocElement: 'foo' object: 'bar1'.
        request addDocElement: 'b:foo' object: 'bar2'.
        request addDocElement: (XePName prefix: 'c' local: 'foo') object: 'bar3'.
        request addDocElement: (XeQName local: 'foo' ns: #'urn:notBlah') object: 'bar4'.
        request xmlFormatted

    generates:
        <?xml version="1.0"?>
        <SOAP-ENV:Envelope xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/'>
        <SOAP-ENV:Body>
        <foo>bar1</foo>
        <b:foo xmlns:b='urn:blah'>bar2</b:foo>
        <c:foo xmlns:c='urn:clah'>bar3</c:foo>
        <ns1:foo xmlns:ns1='urn:notBlah'>bar4</ns1:foo>
        </SOAP-ENV:Body>
        </SOAP-ENV:Envelope>
"
! !

!XeName class methodsFor:'public'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
	^ nil
    ] ifFalse:[
	^Dolphin::SWEnvironment xeIconNamed: 'qName.ico'
    ]
! !

!XeName methodsFor:'accessing'!

getters
	^Array
		with: self localName asSymbol
		with: self localName uncapitalized asSymbol
!

localName
	^localName
!

localName: aString
	localName := aString.
(localName includesString:'SOAP::') ifTrue:[self halt:'debugHalt'].
!

prefix
	^prefix
!

prefix: aString
	prefix := aString
!

qString
	"Answer a <String> of the form prefix:localName
		ie a XMLSchema QName literal"

	| stream |
	stream := String writeStream.
	self printQStringOn: stream.
	^stream contents
!

qualifiedNameIn: aXeTypespace
	self subclassResponsibility
! !

!XeName methodsFor:'content creation'!

xeContentWith: anEncodingStyle
	"Answer a <XeContent> for the receiver"

	self subclassResponsibility
! !

!XeName methodsFor:'converting'!

asXeName
	^self
! !

!XeName methodsFor:'copying'!

copyWithLocalName: aString
	^self copy
		localName: aString;
		yourself
! !

!XeName methodsFor:'particle creation'!

xeParticleWith: anEncodingStyle
	"Answer a <XeElement> "

	self subclassResponsibility
! !

!XeName methodsFor:'printing'!

printQStringOn: aWriteStream
	self hasPrefix
		ifTrue:
			[aWriteStream
				display: self prefix;
				nextPut: $:].
	aWriteStream display: self localName
! !

!XeName methodsFor:'testing'!

hasPrefix
	^self subclassResponsibility
! !

!XeName class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeName.st,v 1.14 2010/12/22 09:46:23 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeName.st,v 1.14 2010/12/22 09:46:23 stefan Exp $'
! !
