"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeParticleAbstract subclass:#XeNamedParticle
	instanceVariableNames:'content qName flags objectAccessor'
	classVariableNames:'FormQualified Nillable Omitable'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeNamedParticle comment:'I am an named component, I model the information in a XMLSchema declarations

Instance Variables:
	content <XeContentAbstract>     My content
	qName   <XeQName>               My name
	flags   <Integer>
	objectAccessor  <XeObjectAccessor>      An object that I use to extract (or set) my ''value'' from a parent object. Can either be via selectors (ie I send a getter and setter based on my name) or via a keys (ie I send #at: #at:put: with a key based on my name)

Flags:
	FormQualified   Is my name qualified or unqualified?
	Nillable                Can I encode as nil?
	Omitable                Can I be omitted?'
!


!XeNamedParticle class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	FormQualified := 1.
	Nillable := 2.
	Omitable := 4
! !

!XeNamedParticle class methodsFor:'instance creation'!

name: aXeQName content: aXeContent
	^self new setName: aXeQName asQName content: aXeContent
! !

!XeNamedParticle methodsFor:'accessing'!

allObjectAccessors
	^Array with: self objectAccessor
!

content
	"Answer the receivers <XeContent>"

	^content
!

form
	^self isFormQualified
		ifTrue: ['qualified']
		ifFalse: ['unqualified']
!

isNillable: aBoolean
	"See #beNillable"

	flags := self flags mask: Nillable set: aBoolean
!

isOmitable: aBoolean
	"See #beOmitable"

	flags := self flags mask: Omitable set: aBoolean
!

localName
	"Answer the particles local name"

	^self qName localName
!

namespace
	^self qName namespace
!

objectAccessor
	"Answer the <XeObjectAccessor> used to access values from a parent object"

	^objectAccessor
!

qName
	"Answer a <XeQName>"

	^qName
! !

!XeNamedParticle methodsFor:'accessing-private'!

accessContent
	"Answer a XeContent which can be used to access the <XeEncodingStyle> the receiver was created in"

	^content
!

flags
	^flags
!

initialize
	super initialize.
	flags := self defaultFlags
!

qName: aXeName
	"Set the particles name "

	qName := aXeName
!

setName: aXeQName content: aXeContent
	qName := aXeQName.
	content := aXeContent
! !

!XeNamedParticle methodsFor:'defaults'!

defaultFlags
	^FormQualified
! !

!XeNamedParticle methodsFor:'modes'!

beAttributesKeyedAccessor
	"Set the receiver to use a <XeAttributesKeyedAccessor> to access values from a parent object"

	objectAccessor := XeAttributesKeyedAccessor new component: self
!

beFormQualified
	"Set the receiver to be form='qualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified."

	self isFormQualified: true
!

beFormQualifiedNS: aSymbol
	"Set the receiver to be form='qualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified.
	-override any existing namespace with aSymbol"

	self assert: [aSymbol isKindOf: Symbol].
	self beFormQualified.
	qName namespace: aSymbol
!

beFormUnqualified
	"Set the receiver to be form='unqualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified."

	self isFormQualified: false.
	qName := self qName copy beNullNamespace
!

beKeyedAccessor
	"Set the receiver to use a <XeKeyedAccessor> to access  values from a parent object"

	objectAccessor := XeKeyedAccessor new component: self
!

beNillable
	"Set the receiver to be nillable='true', and allow nil instances        (should always be true in soap-enc).
		nil will encode into a <XeNode> using the xsi:nil attribute. In soap-enc this is equivalent to omitting the node"

	self isNillable: true
!

beNotNillable
	"Set the receiver to be nillable='false', and disallow nil instances    (should always be true in soap-enc)"

	self isNillable: false
!

beNotOmitable
	self isOmitable: false
!

beOmitable
	"Set the receiver to be omitable ie  minOccurs='0'.
		In soap-enc this is equivalent to encoding the xsi:nil attribute into the node"

	self isOmitable: true
!

beSelectorAccessor
	"Set the receiver to use a <XeSelectorAccessor > to access  values from a parent object"

	objectAccessor := XeSelectorAccessor new component: self
! !

!XeNamedParticle methodsFor:'modes-private'!

isFormQualified: aBoolean
	"Private - Set the receiver's to be form='qualified' | 'unqualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified."

	flags := self flags mask: FormQualified set: aBoolean
! !

!XeNamedParticle methodsFor:'testing'!

isDeclaredIn: schemaNode
	"Answer if the receiver has been declared in the <XeNode> schema"

	^self subclassResponsibility
!

isDeclaredIn: schemaNode space: aSymbolSpaceString
	"Has the receiver been globally declared in the schemaNode in the 'symbolSpace'?
		(XMLSchemas have separate name symbolSpaces for the kinds of components 
			(except complexType and simpleType which share one space))"

	^schemaNode childNodes 
		anySatisfy: [:each | each localName = aSymbolSpaceString and: [each nameAttribute = self localName]]
!

isFormQualified
	"Answer if the receiver's is form='qualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified."

	^self flags allMask: FormQualified
!

isNillable
	"See #beNillable"

	^self flags allMask: Nillable
!

isOmitable
	"See #beOmitable"

	^self flags allMask: Omitable
!

isUnnamed
	"Answer if the receiver does not have a name"

	^false
! !

!XeNamedParticle methodsFor:'visitor'!

encodeChildValueFrom: anObject writer: aXeSoapXMLWriter
	"Extract my value from a parent object and encode it into the aXeSoapXMLWriter"

	^self encode: (self objectAccessor getValueFrom: anObject) writer: aXeSoapXMLWriter
! !

!XeNamedParticle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeNamedParticle.st,v 1.5 2008/09/03 08:53:10 cg Exp $'
! !

XeNamedParticle initialize!
