"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: YAXO }"

XeCompositeNode subclass:#XeNode
	instanceVariableNames:'parentNode prefix localName attributes nsMap defaultNS content
		childNodes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Nodes'
!

XeNode comment:'I am an xml element node'
!


!XeNode class methodsFor:'instance creation'!

named: elementName attributes: attributeList
	^self new named: elementName attributes: attributeList
!

nodeName: aString 
	^self new nodeName: aString
! !

!XeNode methodsFor:'accessing'!

attributes
	^attributes
!

attributes: aXeBAttributes
	attributes := aXeBAttributes
!

childNodes
	"Answer a collection of <XeNode>s.
	(The content collection may contain both XeNodes and XeText if the xml is mixed)
	This is cached"

	childNodes isNil 
		ifTrue: [childNodes := (content select: [:each | each isElementNode]) asArray].
	^childNodes
!

content
	^content
!

defaultNS: aNSSymbol
	self assert: [aNSSymbol isNil or: [aNSSymbol isKindOf: Symbol]].
	defaultNS := aNSSymbol
!

defaultNamespace 
	^defaultNS isNil 
		ifTrue: [self parentNode defaultNamespace]
		ifFalse: [defaultNS]
!

documentElement
	^self parentNode documentElement
!

firstXMLLine
        | xmlLines |

        Smalltalk isSmalltalkX ifTrue:[
            xmlLines := self xml asCollectionOfLines first
        ] ifFalse:[
            xmlLines := self xml lines
        ].
        ^xmlLines isEmpty 
                ifTrue: ['']
                ifFalse: [xmlLines first]
!

getQAttributes
	"Answer a collection of attributes. 
	-NCName attributes (ie no colon) are *not* in the default namespace"

	^attributes asQualifiedCollection
!

localName
	^localName
!

named: elementName attributes: attributeList
	self assert: [elementName class = String].
	self nodeName: elementName.
	attributeList keysAndValuesDo: [ :k :v | 
		self assert: [k class = String].
		self assert: [v class = String].
		k = 'xmlns' 
			ifTrue: [defaultNS := v asSymbol]
			ifFalse: [(k beginsWith: 'xmlns:')
				ifTrue: [self addNSPrefix: (k copyFrom: "##" ('xmlns:' size + 1) to: k size) to: v]
				ifFalse: [self setAttribute: k value: v]]]
!

namespace
	"Answer a Symbol"

	^self hasPrefix 
		ifTrue: [self namespaceFor: self prefix]
		ifFalse: [self defaultNamespace]
!

nextSibling
	^self parentNode nextSiblingOf: self
!

nodeAtId: idString 
	self preOrderElementDo: [ :each | (each getAttribute: 'id')  = idString ifTrue: [^each]].
	^nil
!

nodeName
	^self qName qString
!

ownerDocXML
	^self ownerDocument xml
!

ownerDocument
	^self parentNode ownerDocument
!

parentNode
	^parentNode
!

parentNode: aXeNode
	parentNode := aXeNode
!

prefix
	"Answer a String"

	prefix isNil ifTrue: [^''].
	^prefix
!

prefix: aPrefixString localName: aLocalName
	prefix := aPrefixString.
	localName := aLocalName.
!

previousSibling
	^self parentNode previousSiblingOf: self
!

qString
	| stream |
	stream := String writeStream.
	self printQStringOn: stream.
	^stream contents 
!

setDefaultNS: aNSSymbol
	Notification deprecated.
	defaultNS := aNSSymbol asSymbol
!

text
	^self childTextNode text

! !

!XeNode methodsFor:'accessing-namespaces'!

namespaceFor: aPrefixString 
	"Answer the namespace of the prefix.
	This is a major profiling bottleneck.
	Necessary when qualifying attribute values"
	self assert: [aPrefixString class == String].
	nsMap isNil ifTrue: [^self parentNode namespaceFor: aPrefixString].
	^nsMap at: aPrefixString ifAbsent: [self parentNode namespaceFor: aPrefixString]
        
!

targetNamespace
	| att |
	(att := self getAttribute: 'targetNamespace') notNil ifTrue: [^att asSymbol].
	^self parentNode targetNamespace
! !

!XeNode methodsFor:'accessing-private'!

addAllNSDeclarationsTo: aLookupTable
	nsMap isNil ifTrue: [^nil].
	nsMap keysAndValuesDo: [:k :v | aLookupTable at: k ifAbsentPut: [v]]
!

childIndexOf: aXeNode
	^content indexOf: aXeNode
!

childTextNode
	content isEmpty ifTrue: [self addTextNode: String new].
	self assert: [content size = 1].
	self assert: [content first isTextNode].
	^content first

!

content: aCollection
	content := aCollection
!

extractChildASCIIText
	"Extract the nodes text as ASCII, which uses dolphins standard Unicode to Ascii function,
	and possibly depends on the locale of the machine. Whether this is used, or the UTF Extraction
	depends on a mode of the typespace.
	Note: Avoid use of empty literal string, as in soap-enc, identical string will encode as multireferences"
	#todo. "This still needs to be implemented for Yaxo"

	^self childTextNode asASCII
!

extractChildUTF8Text
	"Extract the nodes text as UTF8 (ie ASCII chars>127 will be rubbish),
	Whether this is used, or the ASCII Extraction depends on a mode of the typespace.
	Note: Avoid use of empty literal string, as in soap-enc, identical string will encode as multireferences"

	#todo. "This still needs to be implemented for Yaxo"

	^self childTextNode asUTF8

!

nodeName: aString
	prefix := aString xmlPrefix.
	localName := aString xmlBase
!

nsMap
        nsMap isNil ifTrue: [
            "/ cg: changed from LookupTable
            "/ nsMap := LookupTable new
            nsMap := Dictionary new
        ].
        ^nsMap
!

nsMap: aLookupTable
	nsMap := aLookupTable
! !

!XeNode methodsFor:'accessing-qualified'!

qName
	"Answer the XeQName for the nodes element"

	^SOAP::XeQName prefix: self prefix
		local: self localName
		ns: self namespace
!

qValueFor: anAttributeName 
	""

	^self makeQName: (self getAttribute: anAttributeName ifNone: [^nil])
!

qValueFor: anAttributeName ifNone: operation 
	""

	^self makeQName: (self getAttribute: anAttributeName ifNone: [^operation value])
!

qValueForQ: aXeQName
	"Answer the <XeQName> for the attribute named aXeQName "

	| att |
	att := self getAttributeQ: aXeQName ifNone: [^nil].
	^self makeQName: att
!

qValueForType
	"Answer a <XeQName> for the type attribute of the node or nil. "

	^self qValueFor: 'type'
!

qValueForXSIType
	| att |
	(att := self xsiTypeAttribute) isNil ifTrue: [^nil].
	^self makeQName: att
!

tQValueForName
	"Answer the <XeQName> of the 'name' attribute qualified in the targetNamespace"

	| name ns |
	(name := self getAttribute: 'name') isNil 
		ifTrue: [^self error: 'Expecting name attribute'].
	(name includes: $:) ifTrue: [^self error: 'Expecting unqualified name attribute'].
	ns := self targetNamespace.
	ns isEmpty ifTrue: [^self error: 'Expecting targetNamespace to qualify name attribute'].
	^SOAP::XeQName local: name ns: ns
! !

!XeNode methodsFor:'accessing-unqualified'!

getAttribute: aString 
	self assert: [aString class == String].
	^attributes getAttribute: aString 
        
!

getAttribute: aString ifNone: operation
	| att |
	^(att := self getAttribute: aString) isNil 
		ifTrue: [operation value]
		ifFalse: [att]
        


        
!

getAttribute: aLocalName in: aNSSymbol
	^self 
		getAttribute: aLocalName
		in: aNSSymbol
		ifNone: []
!

getAttribute: aLocalName in: aNSSymbol ifNone: operation
	(aNSSymbol isNil or: [aNSSymbol isEmpty]) 
		ifTrue: [^self getAttribute: aLocalName ifNone: operation].
	^attributes 
		getAttribute: aLocalName
		in: aNSSymbol
		ifNone: operation
	"^attributes 
		ifNil: [operation value]
		ifNotNil: [:t | attributes getAttribute: aBaseString in: aNamespace ifNone: operation]"
!

getAttributeQ: aXeQName
	^self getAttribute: aXeQName localName in: aXeQName namespace
!

getAttributeQ: aXeQName ifNone: operation
	^self 
		getAttribute: aXeQName localName
		in: aXeQName namespace
		ifNone: operation
!

nameAttribute
	"Answer the value of the attribute with the NCName 'name' (or nil)"

	^self attributes getAttribute: 'name' 
!

xsiTypeAttribute
	^self 
		getAttribute: 'type'
		in: SOAP::XeNamespace schema2001i
		ifNone: [
			self 
				getAttribute: 'type'
				in: SOAP::XeNamespace schema1999i
				ifNone: [nil]]
 


! !

!XeNode methodsFor:'adding'!

addContent: aXeNode
	self addNode: aXeNode
!

addElement: aString 
	^self addNode: (self class nodeName: aString)
!

addElementFirst: aString 
	childNodes := nil.
	^content addFirst: ((self class nodeName: aString) parentNode: self)
!

addElementNamed: aXeQName rootDeclare: aBool
	"Add an element. If the QName is qualified, check the namespace has been declared, and if not declare it. If aBool is true, add the declaration to the root node, otherwise add it locally"

	| node needsDeclare |
	(self isInDefaultNSFor: aXeQName) ifTrue: [^self addElement: aXeQName localName].
	needsDeclare := self fixupQNamePrefix: aXeQName.
	node := self addElement: aXeQName qString.
	needsDeclare ifTrue: [node addNSAttribute: aXeQName rootDeclare: aBool].
	^node
!

addInsert: aString 
	"Add the string as an insert ie dont escape the string"

	^self addNode: (XeBText insert: aString) 
!

addNSAttribute: aXeQName 
	"add the prefix/namespace as a xmlns declaration."

	self addNSPrefix: aXeQName prefix to: aXeQName namespace
        
!

addNSAttribute: aXeQName rootDeclare: aBool
	"add the prefix/namespace as a xmlns declaration. If aBool add it to the root node otherwise locally"

	(aBool ifTrue: [self ownerDocument documentElement] ifFalse: [self]) 
		addNSAttribute: aXeQName
!

addNSPrefix: aPrefixString to: aNSSymbol
	"Declare the prefix to the namespace "

	self assert: [aPrefixString class == String].
	self nsMap at: aPrefixString asString put: aNSSymbol asSymbol
!

addNode: aXeNode
	childNodes := nil.
	^content add: (aXeNode parentNode: self)
!

addNullAttributeFor: xsiNS 

	(xsiNS isNil or: [xsiNS isEmpty or: [xsiNS asSymbol == SOAP::XeNamespace schema2001i]]) 
		ifTrue: [self setQAttribute: SOAP::XeQName nilAttribute value: 'true']
		ifFalse: 
			[xsiNS asSymbol == SOAP::XeNamespace schema1999i 
				ifFalse: [self error: 'Error declaring nil/null attribute'].
			self setQAttribute: SOAP::XeQName nullAttribute value: '1']
!

addTextNode: aString 
	"make aString the simple content of this node"

	^self addNode:(XeBText text: aString)
!

appendChild: aXeNode
	Notification deprecated.
	^self addNode: aXeNode
!

replaceChild: aXeNode oldChild: aXeNode2
	childNodes := nil.
	content at: (content indexOf: aXeNode2) put: (aXeNode parentNode: self)
!

setAttribute: aString qName: aXeQName 
	aXeQName isNullNamespace 
		ifTrue: [self setAttribute: aString value: aXeQName localName]
		ifFalse: 
			[ self fixupPrefixGloballyFor: aXeQName .
			self setAttribute: aString value: aXeQName qString]
!

setAttribute: nameString value: valueString
	self attributes setAttribute: nameString value: valueString
!

setQAttribute: aKeyQName qName: aValueQName 
	self fixupPrefixGloballyFor: aKeyQName.
	self setAttribute: aKeyQName qString qName: aValueQName
!

setQAttribute: aKeyQName value: aString 
	self fixupPrefixGloballyFor: aKeyQName.
	self setAttribute: aKeyQName qString value: aString
! !

!XeNode methodsFor:'converting'!

asParserNode
	| doc env|
	env := self class environment at: #XeEnvironment ifAbsent: [].
	^(env isNil or: [env isYAXOParser])
		ifTrue: [self]
		ifFalse: [
			doc := env docNodeFor: self ownerDocument xml.
			self locateInParserDoc: doc]
! !

!XeNode methodsFor:'copying'!

cloneAllWithPrefixes
        "Answer a deep clone of this node, adding namespace declarations to the new documentElement"

        | nsAtts clone |
        "/ cg: changed from LookupTable
        nsAtts := Dictionary new.
        self upToRootDo: [ :each |
                each addAllNSDeclarationsTo: nsAtts].
        clone := self cloneNode: true.
        nsAtts keysAndValuesDo: [ :k :v | 
                clone addNSPrefix: k to: v ].
        clone defaultNS: self defaultNamespace.
        ^clone
!

cloneNode: aBool 
	^aBool 
		ifTrue: [self cloneNodeDeep]
		ifFalse: [self cloneNodeShallow]
!

cloneNodeDeep
	| p clone |
	p := parentNode.
	parentNode := nil.
	clone := self deepCopy.
	parentNode := p.
	self ownerDocument shallowCopy addNode: clone.
	self assert: [clone parentNode documentElement == clone].
	^clone
!

cloneNodeShallow 
	| copy |
	copy :=self class new.
	^copy
		prefix: prefix localName: localName;
		nsMap: nsMap copy;
		defaultNS: defaultNS;
		attributes: (attributes copy setNode: copy);
		yourself
!

cloneShallowWithPrefixes

	| nsAtts clone |
	nsAtts := LookupTable new.
	self upToRootDo: [ :each |
		each addAllNSDeclarationsTo: nsAtts].
	clone := self cloneNode: false.
	nsAtts keysAndValuesDo: [ :k :v | 
		clone addNSPrefix: k to: v ].
	clone defaultNS: self defaultNamespace.
	clone content: content.
	^clone
! !

!XeNode methodsFor:'enumerating'!

preOrderElementDo: aBlockClosure 
	"Evaluate monadic value argument, operation, for each of the elements of the 
	receiver. Answers the receiver. The elements are enumerated using a 'pre-order'
	or 'depth-first' traversal where the parent is visited before the children."

	aBlockClosure value: self.
	self childNodes do: [:each | each preOrderElementDo: aBlockClosure]
!

upToRootDo: aBlockClosure 
	"Travel up the parent pointers doing the block"

	aBlockClosure value: self.
	self parentNode upToRootDo: aBlockClosure 
! !

!XeNode methodsFor:'helpers'!

browserView
	"Open the xml of the envelope in IE
	-Creates a temp file and uses shellOpen "

	^self ownerDocument browserView
!

makeQName: aString
	"Answer an XeQName
	-If aString is a 'NCName' (ie no colon) it is qualified in the default namespace
	-If aString is a 'QName' (ie abc:def) the prefix is used to locate its namespace"

	| split ns |
	split := aString xmlSplit.
	split first notNil 
		ifTrue: 
			[ns := (self namespaceFor: split first) asSymbol.
			ns isEmpty ifTrue: [self error: 'Can not find the namespace for ' , split first displayString]]
		ifFalse: [ns := self defaultNamespace].
	^SOAP::XeQName prefix: split first
		local: split second
		ns: ns
! !

!XeNode methodsFor:'helpers-private'!

fixupPrefixGloballyFor: aXeQName
	aXeQName isNullNamespace ifTrue: [^nil].
	(self fixupQNamePrefix: aXeQName) ifTrue: [self documentElement addNSAttribute: aXeQName]
!

fixupPrefixLocallyFor: aXeQName
	"Private - Ensure the prefix/namespace pair are valid. 
	  -If not, change the prefix if a clash exists.
	  -If not, declare the namespace.
	(This method stops multi prefixes being declared for the same namespace. While this
	  is not illegal, it is messy"

	aXeQName isNullNamespace 
		ifFalse: [(self fixupQNamePrefix: aXeQName) ifTrue: [self addNSAttribute: aXeQName]]
!

fixupQNamePrefix: aXeQName 
	"Private - Answer if the prefix needs declaring. (we dont declare it, as we are not aware of whether to do it locally or globally)
	Fixup a prefix/namespace, 
	-to avoid clashes
	-to avoid multiple prefixes to the same namespace"

	| p hasClash notDeclared|
	hasClash := self hasClash: aXeQName ifMatch: [^false].
	p := self prefixFor: aXeQName namespace.
	(notDeclared := (p = '')) 
		ifTrue: [(aXeQName hasPrefix not or: [hasClash]) ifTrue: [aXeQName prefix: self nextEmptyPrefix]]
		ifFalse: [aXeQName prefix: p].
	^notDeclared 
!

hasClash: aXeQName ifMatch: operation
	"Private - Answer if the prefix/namespace of XeQName clashes with the receivers prefix declarations.
	Strange method, but this is profiled bottleneck"

	| ns |
	aXeQName hasPrefix ifFalse: [^false].
	(ns := self namespaceFor: aXeQName prefix) notEmpty
		ifTrue: [ns asSymbol = aXeQName namespace 
				ifTrue: [^operation value] 
				ifFalse: [^true]]
		ifFalse: [^false]
        
        
!

nextEmptyPrefix
	| basePrefix count |
	basePrefix := 'ns'.
	count := 1.
	[(self namespaceFor: basePrefix , count displayString) isEmpty] 
		whileFalse: [count := count + 1].
	^basePrefix , count displayString
! !

!XeNode methodsFor:'initializing'!

initialize
	super initialize.
	content := OrderedCollection new.
	attributes := XeBAttributes new setNode: self
! !

!XeNode methodsFor:'moving'!

beFirstChild
	self parentNode makeFirstChild: self
!

makeFirstChild: aXeNode
	self assert: [aXeNode parentNode == self].
	childNodes := nil.
	content remove: aXeNode.
	content addFirst: aXeNode
! !

!XeNode methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream 
		display: ' - ';
		display: self localName
! !

!XeNode methodsFor:'printing-private'!

contentXML
	| stream |
	stream := String writeStream.
	content do: [:each | each printXMLOn: stream cr: false].
	^stream contents
!

printEndTagOn: aStream
	aStream nextPutAll: '</'.
	self printQStringOn: aStream.
	aStream nextPut: $>
!

printNamespaceXMLOn: aStream
	defaultNS notNil 
		ifTrue: 
			[aStream
				display: ' xmlns="';
				display: defaultNS;
				nextPut: $"].
	nsMap notNil 
		ifTrue: 
			[nsMap keysAndValuesDo: 
					[:k :v | 
					aStream
						display: ' xmlns:';
						display: k;
						display: '="';
						display: v;
						nextPut: $"]]
!

printQStringOn: aStream
	(prefix notNil and: [prefix notEmpty]) 
		ifTrue: 
			[aStream
				display: prefix;
				nextPut: $:].
	aStream display: localName
!

printXMLOn: aStream cr: aBool
	aBool ifTrue: [aStream cr].
	aStream nextPut: $<.
	self printQStringOn: aStream.
	attributes printXMLOn: aStream cr: aBool.
	self printNamespaceXMLOn: aStream.
	content isEmpty 
		ifTrue: [aStream nextPutAll: '/>']
		ifFalse: 
			[aStream nextPut: $>.
			content do: [:each | each printXMLOn: aStream cr: aBool].
			(aBool and: [self isSimpleContent not]) ifTrue: [aStream cr].
			self printEndTagOn: aStream]
! !

!XeNode methodsFor:'removing'!

removeAttribute: aString 
	self assert: [(aString beginsWith: 'xmlns') not].
	self attributes removeAttribute: aString 
!

removeChild: aXeNode
	childNodes := nil.
	content remove: aXeNode
! !

!XeNode methodsFor:'searching'!

prefixFor: aNSSymbol
	"Answer the aNSSymbol the prefix is associated with."

	^self nsMap keyAtValue: aNSSymbol ifAbsent: [self parentNode prefixFor: aNSSymbol]
! !

!XeNode methodsFor:'searching-private'!

locateInParserDoc: aXeDoc
	^(self parentNode locateInParserDoc: aXeDoc) childNodes 
		at: (self parentNode childIndexOf: self)
! !

!XeNode methodsFor:'testing'!

hasSameAttributeInformation: aXeNode
        "Not possible to be 100% accurate, as we do not know whether attribute values are QNames"

        self getQAttributes do: 
                        [:each | 
                        | otherValue |
                        otherValue := aXeNode getAttributeQ: each key ifNone: [self breakPoint:#sv. ^false].
                        "If the value contains a colon, and a ns has been declared, assume the value is qualified and compare XeQNames. Otherwise do a straight compare on the values"
                        (((each value includes: $:) and: [(self namespaceFor: each value xmlPrefix) notEmpty]) 
                                ifTrue: [(self qValueForQ: each key) = (aXeNode qValueForQ: each key)]
                                ifFalse: [each value = otherValue]) ifFalse: [self breakPoint:#sv. ^false]].
        ^true
!

isElementNode
	^true
!

isNullNamespace
	^self namespace isEmpty
!

isSimpleContent
	^content isEmpty or: [content first isTextNode]
!

isSimpleElement
	content isEmpty ifTrue: [^true].
	^(content size = 1 and: [content first isTextNode])
!

isXSINil
	| ans |
	ans := self getAttribute: 'nil' in:  #'http://www.w3.org/2001/XMLSchema-instance'.
	ans notNil ifTrue: [ ^#('1' 'true') includes: ans].
	ans := self getAttribute: 'null' in:  #'http://www.w3.org/1999/XMLSchema-instance'.
	ans notNil ifTrue: [^#('1' 'true') includes: ans].
	^false

        
! !

!XeNode methodsFor:'testing-private'!

hasPrefix
	^prefix notNil and: [prefix notEmpty]
!

isPrefixDeclared: aString
	^(self nsMap includesKey: aString) 
		ifTrue: [true]
		ifFalse: [self parentNode isPrefixDeclared: aString]
! !

!XeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeNode.st,v 1.12 2010/02/26 10:58:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeNode.st,v 1.12 2010/02/26 10:58:20 cg Exp $'
! !
