"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeNodeDecodingAbstract
	instanceVariableNames:'node reader object'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeNodeDecodingAbstract comment:'I am a node in an xml tree during a decoding. I am responsible for creating an identical object if multireferences are used, and for decoding a nil node.

-I am not multiple process safe. I must be used in a single process

Instance Variables:
	node	<XeNode>
	reader	<XeSoapXMLReader> or <SplashNodeReader>
	object	<Object>'
!


!XeNodeDecodingAbstract class methodsFor:'instance creation'!

node: aXeNode reader: aXMLReader
	"Create an instance to decode the aXeNode for the aXMLReader (either a <XeSoapXMLReader> or <SplashNodeReader>)"

	^self new node: aXeNode reader: aXMLReader
! !

!XeNodeDecodingAbstract methodsFor:'accessing'!

object
	^object
!

object: anObject
	object := anObject
! !

!XeNodeDecodingAbstract methodsFor:'accessing-private'!

hRefAttribute
	^node getAttribute: 'href'
!

idAttribute
	^node getAttribute: 'id'
!

map
	^reader readMap
!

node
	^node
!

node: aXeNode reader: aXMLReader
	"Set the instance to decode the aXeNode for the aXMLReader (either a <XeSoapXMLReader> or <SplashNodeReader>)"

	node := aXeNode.
	reader := aXMLReader
! !

!XeNodeDecodingAbstract methodsFor:'asserting-private'!

assertNotNilNode
	^self subclassResponsibility
! !

!XeNodeDecodingAbstract methodsFor:'operations'!

readNode
	"Answer the receiver.
	Read the node setting the decoded value into object.
	This method will decode multireferences to a node as identical objects"

	| href |
	self assertNotNilNode.
	self isIDHref not 
		ifTrue: [self readUsingContent]
		ifFalse: 
			[(href := self hRefAttribute) isNil 
				ifTrue: [self readKeepingReference xmlDecodeFixupWith: self]
				ifFalse: [self readReference: href]]
! !

!XeNodeDecodingAbstract methodsFor:'operations-private'!

readComplexKeepingID: id
        "Private - Read the node,  First store a dummy object to the map to handle circular encodings, 
        when the read returns switch the pointers, so both the map, and any circular references now point to the value.
        -This feels more dangerous than a oneWayBecome, but is much faster.
        -object is an object that we have just created, and by definition it is an error for it to be nil,true,false,"

        | proxy |
        proxy := Object new.
        self map atHRef: '#' , id put: proxy.
        self readUsingContent.
        self assert: [object notNil and: [object ~~ true and: [object ~~ false]]].
        Smalltalk isSmalltalkX ifTrue:[
            proxy become: object.
        ] ifFalse:[
            proxy swappingBecome: object.
        ].
        object := proxy
!

readHRef: href
	"Locate the node with an id attribute for href and read it
	Answer the decoded object"

	node := self map nodeAt: (href copyFrom: 2 to: href size).
	node isNil ifTrue: [self error: 'Could not resolve the href ' , href displayString].
	self readNode.
	^object
!

readKeepingID: id
	"Private - Read the node,  If content is complex, we need to handle circular references."

	self isSimpleContent
		ifTrue: [self readSimpleKeepingID: id]
		ifFalse: [self readComplexKeepingID: id]
!

readKeepingReference
	"Private - Read the node, if it has an id attribute we need to store the object in case it is referenced by another node"

	| id |
	(id := self idAttribute) notNil 
		ifTrue: [self readKeepingID: id]
		ifFalse: [self readUsingContent]
!

readReference: href
	"Set the object by locating the object with the corresponding id.
		-either we have already read that object
		-or we need to find and read it now"

	object := self map atHRef: href ifAbsent: [self readHRef: href]
!

readSimpleKeepingID: id
	"Private - Read the node, no circular reference problems possible.
	Put the object in the map at it's id"

	self readUsingContent.
	self map atHRef: '#' , id put: object
!

readUsingContent
	"Private - Answer the receiver.
	-If the node is a nil node, set object to nil.
	-Otherwise, ask our reader <SplashNodeReader> <XeSoapXMLReader> to do the decoding"

	self isNullNode ifTrue: [object := nil] ifFalse: [reader doDecoding]
! !

!XeNodeDecodingAbstract methodsFor:'testing-private'!

isIDHref
	self subclassResponsibility
!

isNodeSoapArray
	^(node getAttribute: 'arrayType' in: XeNamespace soapEnc) notNil
!

isNullNode
	^node isXSINil
!

isSimpleContent
	^self subclassResponsibility
! !

!XeNodeDecodingAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeNodeDecodingAbstract.st,v 1.5 2003/10/08 17:31:53 cg Exp $'
! !
