"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeName subclass:#XePName
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Names'
!

!XePName class methodsFor:'documentation'!

documentation
"
    A name can be one of:
       - An Instance of XeQName (ie XeQName local: 'foo' ns: #'urn:blah'). This is a fully qualified name
       - An Instance of XePName (ie XeQName local: 'ns1:foo' ). This will be late qualified.
       - A String or Symbol that contain a colon (ie 'ns2:foo'). This will be late qualified.
       - A String or Symbol that does not contain a colon (ie 'foo'). This will be late qualified into the null namespace.
"
! !

!XePName class methodsFor:'instance creation'!

fromString: aString
	| sIndex local |
	(sIndex := aString indexOf: $:) = 0 
		ifTrue: [self error: 'Invalid prefixed string, must contain one and only one colon'].
	local := aString copyFrom: sIndex + 1 to: aString size.
	(local includes: $:) 
		ifTrue: [self error: 'Invalid prefixed string, must contain one and only one colon'].
	^self prefix: (aString copyFrom: 1 to: sIndex - 1) local: local
!

prefix: aPrefixString local: aLocalName
	^self new prefix: aPrefixString local: aLocalName
!

qTyped: aString
        self obsoleteMethodWarning. " Notification deprecated. "
        ^self fromString: aString
!

xsd: aString
	^self prefix: 'xsd' local: aString
! !

!XePName methodsFor:'accessing'!

hasPrefix
	^true
!

prefix: aString local: aString2 
	prefix := aString.
	localName := aString2
!

qualifiedNameIn: aXeTypespace
	"Answer a XeQName, use the typespace to locate the namespace associated with the prefix"

	^XeQName 
		prefix: self prefix
		local: self localName
		ns: (aXeTypespace namespacePrefixed: self prefix)
! !

!XePName methodsFor:'comparing'!

= aXeName
	^self species == aXeName species 
		and: [self localName = aXeName localName and: [self prefix = aXeName prefix]]
!

hash
	^self localName hash bitXor: self prefix hash
! !

!XePName methodsFor:'content creation'!

xeContentWith: anEncodingStyle
	"Answer a <XeContent> for the receiver"

	^(self qualifiedNameIn: anEncodingStyle typespace) xeContentWith: anEncodingStyle
! !

!XePName methodsFor:'converting'!

asQName
	self error: 'Can not qualify in the absence of a typespace'
! !

!XePName methodsFor:'particle creation'!

xeParticleWith: anEncodingStyle
	"Answer a <XeElement> 
	-Answer the XeElement declared with the receivers name."

	^(self qualifiedNameIn: anEncodingStyle typespace) xeParticleWith: anEncodingStyle
! !

!XePName class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XePName.st,v 1.6 2010/03/15 17:53:19 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XePName.st,v 1.6 2010/03/15 17:53:19 cg Exp $'
! !
