"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeName subclass:#XeQName
	instanceVariableNames:'namespace'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Names'
!

!XeQName class methodsFor:'documentation'!

documentation
"
    A name can be one of:
       - An Instance of XeQName (ie XeQName local: 'foo' ns: #'urn:blah'). This is a fully qualified name
       - An Instance of XePName (ie XeQName local: 'ns1:foo' ). This will be late qualified.
       - A String or Symbol that contain a colon (ie 'ns2:foo'). This will be late qualified.
       - A String or Symbol that does not contain a colon (ie 'foo'). This will be late qualified into the null namespace.
"
! !

!XeQName class methodsFor:'instance creation'!

arrayOfSprayClass: aClass 
        Smalltalk isSmalltalkX ifTrue:[
            ^self prefix: 'd' local: 'ArrayOf' , aClass nameWithoutPrefix ns: XeNamespace sprayClass 
        ].
        ^self prefix: 'd' local: 'ArrayOf' , aClass name asString ns: XeNamespace sprayClass 
!

fromString: aString 
	self assert: [(aString includes: $:) not].
	^self 
		prefix: ''
		local: aString
		ns: nil
!

inSprayClass: aString 
	^self prefix: 'd' local: aString  ns: XeNamespace sprayClass 
!

local: aString2 ns: aSymbol 
	^self 
		prefix: ''
		local: aString2 asString
		ns: aSymbol
!

nilAttribute
	^self prefix: 'xsi'
		local: 'nil'
		ns: XeNamespace schema2001i
!

null: aString 
	^self prefix: '' local: aString ns: #'' 
!

nullAttribute
	^self prefix: 'xsi'
		local: 'null'
		ns: XeNamespace schema1999i
!

prefix: aString local: aString2 ns: aSymbol 
	self assert: [aSymbol isNil or: [aSymbol isKindOf: Symbol]].
	^self new 
		prefix: aString
		local: aString2
		ns: aSymbol
!

soapEnc: aString 
	^self prefix: '' local: aString ns: XeNamespace soapEnc 
!

sprayClass: aClass 
        Smalltalk isSmalltalkX ifTrue:[
            ^self inSprayClass: aClass nameWithoutPrefix
        ].
        ^self inSprayClass: aClass name asString
!

xsd1999: aString 
	^self prefix: 'xsd' local: aString ns: XeNamespace schema1999
!

xsd2001: aString 
	^self prefix: 'xsd' local: aString ns: XeNamespace schema2001 
! !

!XeQName methodsFor:'accessing'!

beNullNamespace
	namespace := #''
!

namespace
	namespace isNil ifTrue: [namespace := #''].
	^namespace
!

namespace: aSymbol 
	
	namespace := aSymbol
!

prefix: aString local: aString2 ns: aSymbol 
        self assert: [aString2 isKindOf: String].
        prefix := aString.
        localName := aString2 asString.
        namespace := aSymbol.

        (localName includesString:'::') ifTrue:[
            self halt:'should not happen'.
        ].
!

qualifiedNameIn: aXeTypespace 
	"Answer self, we are qualified."

	^self
! !

!XeQName methodsFor:'comparing'!

= aXeName
        ^self species == aXeName species 
                and: [self localName = aXeName localName 
                and: [self namespace == aXeName namespace]]
!

hash
	^self localName hash bitXor: self namespace hash
! !

!XeQName methodsFor:'content creation'!

xeContentWith: anEncodingStyle
	"Answer a <XeContent> for the receiver"

	^anEncodingStyle contentOf: self
! !

!XeQName methodsFor:'converting'!

asQName
	^self
! !

!XeQName methodsFor:'particle creation'!

xeParticleWith: anEncodingStyle
	"Answer a <XeElement> 
	-Answer the XeElement declared with the receivers name."

	^anEncodingStyle typespace xeElementDeclared: self encStyle: anEncodingStyle
! !

!XeQName methodsFor:'printing'!

displayOn: aStream
	self printDetailsOn: aStream.
!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: ${.
	self printDetailsOn: aStream.
	aStream nextPut: $}.
! !

!XeQName methodsFor:'printing-private'!

printDetailsOn: aStream
    |ns|

    aStream nextPutAll: self qString.
    ns := self namespace printString.
    ns notEmpty ifTrue:[
        aStream space; nextPutAll: ns
    ].

    "Modified: / 19-03-2012 / 16:57:18 / cg"
! !

!XeQName methodsFor:'testing'!

hasPrefix 
	^prefix notNil and: [prefix notEmpty]
!

isAnyTypeName
	^((namespace == XeNamespace schema1999) and: [localName = 'ur-type']) 
		or: [(namespace == XeNamespace schema2001) and: [localName = 'anyType']]
!

isNullNamespace
	^self namespace isEmpty
! !

!XeQName class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQName.st,v 1.9 2012/03/19 16:41:11 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQName.st,v 1.9 2012/03/19 16:41:11 cg Exp $'
! !
