"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Dictionary variableSubclass:#XeQProperties
	instanceVariableNames:'prefixMap'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Names'
!

!XeQProperties class methodsFor:'documentation'!

documentation
"
    I am a QualifiedProperties dictionary.

    I qualify my keys using prefixes and namespace that are added to me.

    Instance Variables:
            prefixMap       <Table>
"
! !

!XeQProperties class methodsFor:'instance creation'!

new
    "here, we WANT super new - what a kludge"
    ^ super new initialize
!

new:anInteger
    "here, we WANT super new - what a kludge"
    ^ (super new:anInteger) initialize
! !

!XeQProperties class methodsFor:'must strip'!

example
        <resource: #example>

        | qProperties |
        qProperties := XeQProperties new.
        qProperties addPrefix: 'me' namespace: #'urn:myNamespace'.
        qProperties addPrefix: 'you' namespace: #'urn:yourNamespace'.
        qProperties at: 'me:local1' put: 123.
        "added in null namespace"
        qProperties at: 'local2' put: 345.
        qProperties at: ('local3' ns: #'urn:yourNamespace') put: 678.
        "Accessing"
        qProperties at: 'me:local1'.    "Inspect It"
        qProperties at: 'local2'.       "Inspect It"
        qProperties at: 'you:local3'.   "Inspect It"
        ^ qProperties

        "
         self example
        "
! !

!XeQProperties methodsFor:'accessing'!

addPrefix: aString namespace: aSymbol 
	prefixMap at: aString put: aSymbol 
!

addSafelyPrefixOf: aQName
        (aQName hasPrefix and: [(prefixMap includesKey: aQName prefix) not]) 
                ifTrue: [prefixMap at: aQName prefix put: aQName namespace]
!

at: key ifAbsent: exceptionHandler
	^super at: (self qualifyKey: key) ifAbsent: exceptionHandler
!

at: key put: newElement
	^super at: (self qualifyKey: key) put: newElement
!

at: key putOrAdd: newElement
        | qKey |
        qKey := self qualifyKey: key.
        ^(self includesKey: qKey) 
                ifTrue: 
                        [| existing |
                        existing := self at: qKey.
                        (existing isKindOf: OrderedCollection) 
                                ifTrue: [existing add: newElement]
                                ifFalse: [super at: qKey put: (OrderedCollection with: existing with: newElement)]]
                ifFalse: [super at: qKey put: newElement]
!

includesKey: key
        "Answer whether the receiver has a key equal to the argument, key.
        Override superclass to permit symbolic/string keys."

        ^super includesKey: (self qualifyKey: key)
! !

!XeQProperties methodsFor:'accessing-private'!

namespacePrefixed: aString 
	"Answer a NSSymbol"

	^prefixMap at: aString ifAbsent: [self error: 'The typespace does not contain the prefix: ' , aString displayString]
!

prefixMap
	^prefixMap
!

prefixMap: anObject
	prefixMap := anObject
! !

!XeQProperties methodsFor:'adding-private'!

preResize: newMe
	super preResize: newMe.
	newMe prefixMap: prefixMap
! !

!XeQProperties methodsFor:'copying'!

postCopy
	super postCopy.
	prefixMap := prefixMap copy
! !

!XeQProperties methodsFor:'helpers'!

qualify: aName 
	^aName asXeName qualifiedNameIn: self
!

qualifyKey: key
	^self qualify: key
! !

!XeQProperties methodsFor:'initializing-private'!

initialize
        "Private - Initialize the receiver."

        super initialize.
        prefixMap := Dictionary new.
        "/ prefixMap := Dolphin::LookupTable new.
! !

!XeQProperties methodsFor:'removing'!

removeKey: key ifAbsent: exceptionHandler
	^super removeKey: (self qualifyKey: key) ifAbsent: exceptionHandler
! !

!XeQProperties class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQProperties.st,v 1.10 2010/06/14 10:58:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQProperties.st,v 1.10 2010/06/14 10:58:41 cg Exp $'
! !
