"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Collection subclass:#XeQStruct
	instanceVariableNames:'typeName elements attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeQStruct comment:'I have a protocol similar to a dictionary, however I retain the order that elements are put into me.

My keys are either XeQNames or strings/symbols which are firstly converted to XeQNames with a null namespace.

Instance Variables:
	typeName	<XeQName>
	elements		<OrderedCollection>
	attributes		<Table>'
!


!XeQStruct class methodsFor:'instance creation'!

new
	^super new initialize
!

param: aSymbolOrName value: anObject 
	"Answer an instance. Helper construction method"

	^self new
		at: aSymbolOrName put: anObject;
		yourself
!

params: aCollOfNames values: aCollOfValues
	| struct |
	aCollOfNames size = aCollOfValues size 
		ifFalse: [self error: 'Expected ' , aCollOfNames size displayString , ' values'].
	struct := self new.
	aCollOfNames with: aCollOfValues do: [:k :v | struct at: k put: v].
	^struct
! !

!XeQStruct class methodsFor:'must strip'!

example
        <resource: #example>

        | struct qName |
        struct := XeQStruct new.
        "Add in null namespace"
        struct at: #local1 put: 'abc'.
        "Add qualifed"
        struct at: (qName := 'local2' ns: #'urn:myNamespace') put: 123.
        struct at: #local1.     "Inspect it"
        struct at: (XeQName null: 'local1').    "Inspect it"
        struct at: qName        "Inspect it"
! !

!XeQStruct class methodsFor:'spray'!

sprayInstanceWith: aXeContent
	^self new 
		typeName: aXeContent typeName;
		yourself
! !

!XeQStruct methodsFor:'accessing'!

at: key
	"Answer the first value with an equal key.
	key can either be a XeQName, or a string/symbol (which is converted to a XeQName with a null namespace"
 
	^self at: key asQName ifAbsent: [self errorNotFound: key]
!

at: key ifAbsent: operation 
	"Answer the first value with an equal key.
	key can either be a XeQName, or a string/symbol (which is converted to a XeQName with a null namespace"

	| index qKey|
	qKey := key asQName.
	index := elements findFirst: [:each | each key = qKey ].
	^index = 0 ifTrue: [operation value] ifFalse: [(elements at: index) value]
!

at: aKey put: aValue
	"Store aValue.
	aKey can either be a XeQName, or a string/symbol (which is converted to a XeQName with a null namespace.
	Note: This will *not* replace an existing key->value (See atUnique:put:)"

	^elements add: (aKey asQName -> aValue)
	
!

atLocalName: aString
	"Answer the value at the first key that has the localName aString"

	^self atLocalName: aString ifAbsent: [self errorNotFound: aString]
!

atLocalName: aString ifAbsent: operation
	"Answer the value at the first key that has the localName aString"

	^(elements detect: [:each | each key localName = aString asString] ifNone: [^operation value]) value.
!

atUnique: aKey put: aValue 
	"Store aValue.
	aKey can either be a XeQName, or a string/symbol (which is converted to a XeQName with a null namespace.
	Find the first key which equals aKey and replace its value, otherwise append the key->value"


	| replaceIndex  qKey |
	qKey := aKey asQName.
	replaceIndex := elements findFirst: [:each | each key = qKey ].
	replaceIndex = 0 
		ifTrue: [elements add: qKey -> aValue]
		ifFalse: [elements at: replaceIndex put: qKey -> aValue].
	^aValue
!

attributes
	"Answer the LookupTable of attributes"

	^attributes
!

attributes: aLookupTable 
	attributes := aLookupTable	
!

first
	"Answer the first value"

	^elements first value
!

firstKey
	"Answer the first key <XeQName>"

	^elements first key
!

keys
	"Answer a collection of XeQNames"

	^elements collect: [ :each | each key]
!

second
	^elements second value
!

size
	"Answer the number of elements in the receiver."

	^elements size
!

third
	^elements third value
!

typeName
	"Answer the typeName <XeQName> (will be added as an inline type attribute to an xml encoding)"

	^typeName
!

typeName: aXeQName
	"Set the typeName <XeQName> (set when creating an instance from decoding xml)"

	typeName := aXeQName
!

values
	^elements collect: [:each | each value]
! !

!XeQStruct methodsFor:'accessing-private'!

associations
	^elements
!

elements
	^elements
! !

!XeQStruct methodsFor:'comparing'!

= comparand
	"-This is stricter than soap-enc equals, which treats nil the same as missing elements, 
	  and does not impose any order on the elements
	-See #isSameAs: for a soap-enc valid ="

	self == comparand ifTrue: [^true].
	(self species == comparand species) ifFalse: [^false].
	(attributes equals: comparand attributes ) ifFalse: [^false].
	^elements = comparand elements 
!

hash
	^attributes hash bitXor: elements hash
!

isOneWaySoapEncSameAs: aComplexValue
        "testing purposes only. Is the receiver the same as the KeyedObject under soap-enc rules?
        -nil and missing are equal
        -order of accessors not significant"

        [self keysAndValuesDo: [ :k :v |
                (v isNil 
                        ifTrue: [ (aComplexValue includesKey: k) not or: [ (aComplexValue at: k) isNil]]
                        ifFalse: [  (aComplexValue at: k) isSoapEncSameAs: v]) ifFalse: [^false]]
        ]
                on: Error
                do: [:e | 
                    UserPreferences current soapErrorDebugging ifTrue:[ e reject ].
                    self breakPoint:#sv. ^false
                ].

        ^true

    "Modified: / 12-07-2011 / 17:51:01 / cg"
!

isSoapEncSameAs: aComplexValue
	"testing purposes only. Is the receiver the same as the KeyedObject under soap-enc rules?
	-nil and missing are equal
	-order of accessors not significant"

	^(self isOneWaySoapEncSameAs: aComplexValue) and: [aComplexValue isOneWaySoapEncSameAs: self]
		
	
! !

!XeQStruct methodsFor:'content creation'!

xeContentWith: anEncodingStyle
	"Answer a <XeContent> for the receiver"

	| complexContent |
	complexContent := anEncodingStyle 
				complexSpray: typeName
				restricting: anEncodingStyle anyTypeName
				instance: self.
	complexContent childElements do: [:each | each beFormQualified].
	^complexContent
! !

!XeQStruct methodsFor:'converting'!

asElementQualifiedIn: aNSSymbol
	"Answer the receiver.  Put all elements into the namespace: aNSSymbol"

	self qualifyAllElements: aNSSymbol
! !

!XeQStruct methodsFor:'copying'!

postCopy
	super postCopy.
	elements := elements copy.
	typeName := typeName copy
! !

!XeQStruct methodsFor:'enumerating'!

do: operation
	"Evaluate the <monadicValuable> argument, operation, for each of the 
	receiver's element values. "

	self keysAndValuesDo: [:i :elem | operation value: elem]
!

from: start to: stop keysAndValuesDo: operation 
	elements 
		from: start
		to: stop
		keysAndValuesDo: [:i :each | operation value: each key value: each value]
!

keysAndValuesDo: operation
	"Evaluate the <dyadicValuable>, operation, for each of the elements."

	self from: 1 to: self size keysAndValuesDo: operation
! !

!XeQStruct methodsFor:'helpers'!

qualifyAllElements: aNSSymbol 
	"Put all elements into the namespace: aNSSymbol"

	self elements do: [ :each | each key namespace: aNSSymbol]
! !

!XeQStruct methodsFor:'initializing-private'!

initialize
        super initialize.
        elements := OrderedCollection new.
        "/ cg: changed from LookupTable
        attributes := Dictionary new.
! !

!XeQStruct methodsFor:'printing-private'!

attributeValues
		| values |
	values := OrderedCollection new.
	self attributes keysAndValuesDo: [ :k :v |
		values add: v].
	^values asArray
!

printPrefixOn: aStream
	"Private - Print a prefix string for the debug representation of the receiver on aStream.
	-Rare to have attributes"

	aStream basicPrint: self.
	(attributes notNil and: [attributes notEmpty]) ifTrue: [
		aStream print: self attributeValues].
	aStream nextPut: $(
! !

!XeQStruct methodsFor:'removing'!

remove: anObject ifAbsent: aBlock
	"Elements  must be removed by key."

	^self shouldNotImplement

!

removeKey: key 
	| removal qKey |
	qKey := key asQName.
	removal := elements detect: [:each | each key = qKey ] ifNone: [self errorNotFound: key].
	elements remove: removal.
	^removal value
! !

!XeQStruct methodsFor:'searching'!

includesKey: aSymbol
	| qKey |
	qKey := aSymbol asQName.
	^elements anySatisfy: [ :each | each key = qKey ]
! !

!XeQStruct methodsFor:'splash'!

putSplashContentOn: aXeNode context: aSplashContext
	self keysAndValuesDo: [ :k :v | | accessorNode |
		v notNil ifTrue: [
			accessorNode := aXeNode addElementNamed: k rootDeclare: true.
			v putSplashOn: accessorNode context: aSplashContext]]
	
!

putSplashOn: aXeNode context: aSplashContext

	(aSplashContext useReferenceOf: self on: aXeNode) ifFalse: [
		self putSplashTypeOn: aXeNode context: aSplashContext.
		self putSplashContentOn: aXeNode context: aSplashContext]

!

putSplashTypeOn: aXeNode context: aSplashContext
	typeName notNil ifTrue: [
		aSplashContext putXSIType: typeName  on: aXeNode]
! !

!XeQStruct class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQStruct.st,v 1.11 2011/07/12 16:20:14 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeQStruct.st,v 1.11 2011/07/12 16:20:14 cg Exp $'
! !
