"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeSchema
	instanceVariableNames:'typespace contentCache factory overrideMaps targetNS'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSchema comment:'I am a schema. I can be asked to create content (for definitions) or particles (for declarations) . I can use either imported XMLSchemas or local factories. I am part of a typespace

I create components (ie content or particles) for a particular namespace (my targetNS)

When I am asked to create a component  an encodingStyle must be supplied. 

==========
Instance Variables:
	typespace               <XeTypespace>
	contentCache    <Dictionary>
	factory         <XeContentFactory>
	overrideMaps    <Dictionary>
	targetNS                <Symbol>

==========
Implementation notes:
-I cache XeContent instances and only create one instance for a typeName/encodingStyle.
-The overrideMaps allow me to divert a typeName to an alternate factory'
!


!XeSchema class methodsFor:'instance creation'!

forNode: aXeNode
	^self new setFromNode: aXeNode
!

guessSchema
	^self onFactory: XeGuessFactory
!

new
        ^self basicNew initialize
!

onFactory: aClass 
	^self onFactory: aClass targetNS: aClass defaultTargetNamespace
!

onFactory: aClass targetNS: aSymbol 
	^(self new)
		targetNS: aSymbol;
		createContentFactory: aClass;
		yourself
! !

!XeSchema methodsFor:'accessing'!

namespace: aSymbol 
	targetNS := aSymbol
!

preferedPrefix
	^factory class preferedPrefix
!

typespace
	^typespace
!

typespace: aXeTypespace 
	typespace := aXeTypespace 
!

typespaceOptions
	^typespace options 
! !

!XeSchema methodsFor:'accessing-private'!

factory
	^factory
!

factoryFor: aTypeQName
	"Private - Typically all the types from the same namespace will be created from the same factory,
	  however an override can be used (using the 'configuration' methods in XeTypespace) to divert a type to an alternate factory.
	This is mainly useful if you want to map only certain complexTypes from a schema to specified classes. "

	^self overrideMaps at: aTypeQName localName ifAbsent: [factory]
!

overrideMaps
	^overrideMaps
!

targetNS
	"Private - Answer the value of the receiver's ''targetNS'' instance variable."

	^targetNS
!

targetNS: anObject
	targetNS := anObject
! !

!XeSchema methodsFor:'configuration'!

declare: aLocalElementName ownType: aLocalTypename
	"This modifies the typespace by adding an element declaration for the typeName.
	(by default an element will be declared with the same name as the typeName ... this is useful when implementing dotnet defined services)"

	^self factory declare: aLocalElementName ownType: aLocalTypename
!

isElementFormQualified: aBoolean
	"This modifies the classFactory to be elementFormDefault='qualified' (which is equivalent to the XMLSchema attribute switch). 
	-This tells the factory whether the  local elements should be qualified. 
	-Not used in soap-enc (the default is false), but it is common for DOTNET services to need to set this to true"

	self factory isElementFormQualified: aBoolean
!

mapTypeName: aTypeQName to: aClass
	(self factoryFor: aTypeQName) mapTypeName: aTypeQName to: aClass
!

overrideAll: aCollOfLocalNames to: aFactory
	"Should only be used (by the metaData classes) during creation of the typespace."

	| overFactory |
	overFactory := aFactory schema: self.
	aCollOfLocalNames do: [ :each |
		self overrideMaps at: each put: overFactory].
! !

!XeSchema methodsFor:'content creation'!

defaultEncContent: aString
	"Helper method; Answer a XeContent
	  Qualify the aString in this targetNamespace, and create the type with typespace's default encodingStyle"

	^self xeContentOf: (self qualifyLocal: aString)
		encStyle: self typespace defaultEncodingStyle
!

xeContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer a <XeContent>"

	^self typespaceOptions isCacheContent 
		ifTrue: [self cachedContentOf: aTypeQName encStyle: anEncodingStyle]
		ifFalse: [self basicContentOf: aTypeQName encStyle: anEncodingStyle]
! !

!XeSchema methodsFor:'content creation-private'!

basicContentOf: aTypeQName encStyle: anEncodingStyle
	"Private - Answer the XeContent identified by the typeName in the encodingStyle"

	^(self factoryFor: aTypeQName) xeContentOf: aTypeQName encStyle: anEncodingStyle
!

cachedContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer the cached <XeContent> of the aTypeQName/anEncodingStyle combination.
	If not cached, create it and add it to the cache"

	| contentSpec newContent |
	contentSpec := XeContentSpec typeName: aTypeQName encodingStyle: anEncodingStyle.
	contentCache at: contentSpec ifPresent: [:each | ^each].
	"First place an <XeAnyContent> in the cache in case of a circular definition"
	contentCache at: contentSpec put: anEncodingStyle anyContent.
	newContent := self basicContentOf: aTypeQName encStyle: anEncodingStyle.
	contentCache at: contentSpec put: newContent.
	^newContent
! !

!XeSchema methodsFor:'helpers'!

qualifyLocal: aString 
        ^SOAP::XeQName 
                prefix: self preferedPrefix
                local: aString
                ns: targetNS
! !

!XeSchema methodsFor:'initializing-private'!

createContentFactory: aClass
	factory := aClass new schema: self
!

initialize
        super initialize.
        "/ cg: changed from LookupTable
        overrideMaps := Dictionary new.
        contentCache := Dictionary new
!

setFromNode: aXeNode
	aXeNode localName = 'schema' ifFalse: [self error: 'Expecting a schema'].
	self assert: [(targetNS := aXeNode targetNamespace) notEmpty].
	factory := (XeXSDSchema schemaNode: aXeNode) schema: self
! !

!XeSchema methodsFor:'particle creation'!

xeAttributeDeclared: aXeQName encStyle: anEncodingStyle
	"Answer an <XeAttribute>."

	^self factory xeAttributeDeclared: aXeQName encStyle: anEncodingStyle
!

xeElementDeclared: aXeQName encStyle: anEncodingStyle
	"Answer a <XeElement>.  forward this request to the factory"

	^self factory xeElementDeclared: aXeQName encStyle: anEncodingStyle
! !

!XeSchema class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSchema.st,v 1.8 2010/06/14 10:59:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSchema.st,v 1.8 2010/06/14 10:59:21 cg Exp $'
! !
