"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeSchemaRestriction subclass:#XeSchemaFacet
	instanceVariableNames:'monadicBlock component value'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!


!XeSchemaFacet class methodsFor:'instance creation'!

fractionDigits: aString
	"http://www.w3.org/TR/xmlschema-2
	4.3.12 fractionDigits
[Definition:] fractionDigits is the maximum number of digits in the fractional part of values of datatypes derived from decimal. The value of fractionDigits must be a nonNegativeInteger . "

	#todo.  "needs to work with floats and integers"
	^(self new)
		component: 'fractionDigits';
		value: aString;
		block: [:num | num scale <= (Integer fromString: aString)];
		yourself
!

fromNode: aXeNode
	^self perform: (aXeNode localName , ':') asSymbol with: (aXeNode getAttribute: 'value')
!

length: aString
	^(self new)
		component: 'length';
		value: aString;
		block: [:string | string size = (Integer fromString: aString)];
		yourself
!

maxExclusive: aString
	^(self new)
		component: 'maxExclusive';
		value: aString;
		block: (Message selector: #< argument: (Number fromString: aString));
		yourself
!

maxInclusive: aString 
	^self new 
		component: 'maxInclusive';
		value: aString;
		block: (Message 
			selector: #<= 
			argument: (Number fromString: aString));
		yourself 
!

maxLength: aString
	^(self new)
		component: 'maxLength';
		value: aString;
		block: [:string | string size <= (Integer fromString: aString)];
		yourself
!

minExclusive: aString
	^(self new)
		component: 'minExclusive';
		value: aString;
		block: (Message selector: #> argument: (Number fromString: aString));
		yourself
!

minInclusive: aString
	^(self new)
		component: 'minInclusive';
		value: aString;
		block: (Message selector: #>= argument: (Number fromString: aString));
		yourself
!

minLength: aString
	^(self new)
		component: 'minLength';
		value: aString;
		block: [:string | string size >= (Integer fromString: aString)];
		yourself
!

nameString
        Smalltalk isSmalltalkX ifTrue:[
            ^self new block: [ :each | each size > 0
                                       and:[ each first isLetter or: [ #( $_ $:) includes: each first] ]]
        ].
        ^self new block: [ :each | each first isLetter or: [ #( $_ $:) includes: each first]]
!

ncNameString
	^self new block: [ :string | (string includes: $:) not]
!

nmToken
	"This is not accurate, and will have false failures  ... I do not know how to handle unicode issues with this"

	^self new block: [ :string | string allSatisfy: [ :c | c isAlphaNumeric or: ['._ -:' includes: c]]]
!

normalizedString
	^self new block: [ :each | (each anySatisfy: [ :c | self nonNormalCharacters includes: c]) not]
!

pattern: aString
        |regex|

        regex := aString asRegex.    

        ^(self new)
                component: 'pattern';
                value: aString;
                block: [:string | regex matches:string];
                yourself
!

token
	^self new block: [ :string | string isEmpty 
		or: [ (string first ~= Character space and: [string last ~= Character space ])
			and: [ (string indexOfSubCollection: '  ') = 0 ]]]
!

totalDigits: aString
	"http://www.w3.org/TR/xmlschema-2
4.3.11 totalDigits
[Definition:] totalDigits is the maximum number of digits in values of datatypes derived from decimal. The value of totalDigits must be a positiveInteger. "

	"eeck ... but it has to work for ScaledDecimals,Floats and Integers"

	^(self new)
		component: 'totalDigits';
		value: aString;
		block: [:num | (num schemaDisplayString copyWithout: $.) size <= (Integer fromString: aString)];
		yourself
!

whiteSpace: aString
	^Error notYetImplemented
! !

!XeSchemaFacet class methodsFor:'instance creation-private'!

nonNormalCharacters
	^Array with: Character lf with: Character tab with: Character cr                
! !

!XeSchemaFacet methodsFor:'accessing'!

block: aMessage 
	monadicBlock := aMessage 
!

component
	^component
!

component: aString
	component := aString
!

value
	^value
!

value: anObject
	value := anObject
! !

!XeSchemaFacet methodsFor:'operations'!

addToNode: aXMLBNode 
	"Note: a bit of a hack in that we assume that a parent node declares the xsd prefix"

	^(aXMLBNode addElement: 'xsd:' , self component) 
		setAttribute: 'value' value: self value
!

validate: anObject 
	^monadicBlock value: anObject
! !

!XeSchemaFacet class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSchemaFacet.st,v 1.6 2009/10/04 10:07:34 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSchemaFacet.st,v 1.6 2009/10/04 10:07:34 cg Exp $'
! !
