"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComponentVisitor subclass:#XeSchemaWriter
	instanceVariableNames:'targetNamespace isNaming currentNode schemaNode
		isElementFormQualified isAttributeFormQualified imported'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSchemaWriter comment:'I am a schema writer. I am initialized on a targetNamespace and can then be asked to define (XeContent) and declare (XeParticles) components. I will search the components for child definitions/declarations in my targetNamespace.

While I pretend to be able to write 1999 schemas, I actually write them with 2001 form. 

Instance Variables:
	targetNamespace <Symbol> 
	isNaming                <Boolean>
	currentNode     <XeNode>
	schemaNode      <XeNode>
	isElementFormQualified  <Boolean> does the schema have elementFormDefault="qualified"
	isAttributeFormQualified        <Boolean> does the schema have attributeFormDefault="qualified"
	imported                <Collection> of namespaces that need importing.

Note: isElementFormQualified and isAttributeFormQualified        should not be changed once writing has started.

'
!


!XeSchemaWriter class methodsFor:'instance creation'!

targeting: aTargetNSSymbol 
	^self targeting: aTargetNSSymbol schemaNS: (XeNamespace schema2001)
!

targeting: aTargetNSSymbol node: aXeNode schemaNS: aSchemaNSSymbol
	^self new 
		initializeTargeting: aTargetNSSymbol
		node: aXeNode
		schemaNS: aSchemaNSSymbol
!

targeting: aTargetNSSymbol schemaNS: aSchemaNSSymbol
        ^self 
                targeting: aTargetNSSymbol
                node: YAXO::XeDoc new
                schemaNS: aSchemaNSSymbol
! !

!XeSchemaWriter class methodsFor:'must strip'!

example
        <resource: #example>

        | typespace elementSpec element writer |
        typespace := XeTypespace defaultSoapEnc2001.
        elementSpec := XeElementSpec named: #myOrder class: (self environment at: #PurchaseOrder).
        element := elementSpec xeParticleWith: typespace defaultEncodingStyle.
        writer := XeSchemaWriter targeting: XeNamespace sprayClass.
        writer define: element content.
        writer xml

        "<schema
  targetNamespace='http://www.dolphinharbor.org/spray/class'
  xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/'
  xmlns:d='http://www.dolphinharbor.org/spray/class'
  xmlns:xsd='http://www.w3.org/2001/XMLSchema'
  xmlns:wsdl='http://schemas.xmlsoap.org/wsdl/'>
  
  <element name='myOrder' type='d:PurchaseOrder'/>
  
  <complexType name='PurchaseOrder'>
    <sequence>
      <element name='poID' type='xsd:string'/>
      <element name='createDate' type='xsd:dateTime'/>
      <element name='shipTo' type='d:Address'/>
      <element name='billTo' type='d:Address'/>
      <element name='items' type='d:ArrayOfLineItem'/>
    </sequence>
  </complexType>
  
  <complexType name='Address'>
    <sequence>
      <element name='name' type='xsd:string'/>
      <element name='address' type='xsd:string'/>
      <element name='city' type='xsd:string'/>
      <element name='state' type='xsd:string'/>
      <element name='zipCode' type='xsd:string'/>
    </sequence>
  </complexType>
  
  <complexType name='ArrayOfLineItem'>
    <complexContent>
      <restriction base='SOAP-ENC:Array'>
        <attribute wsdl:arrayType='d:LineItem[]' ref='SOAP-ENC:arrayType'/>
      </restriction>
    </complexContent>
  </complexType>
  
  <complexType name='LineItem'>
    <sequence>
      <element name='name' type='xsd:string'/>
      <element name='quantity' type='xsd:int'/>
      <element name='price' type='xsd:decimal'/>
    </sequence>
  </complexType>
</schema>"
! !

!XeSchemaWriter methodsFor:'accessing'!

currentNode
	^currentNode
!

currentNode: aXeNode
	currentNode := aXeNode
!

defaultNS: aNSSymbol
	schemaNode defaultNS: aNSSymbol
!

targetNamespace
	^targetNamespace
!

targetNamespace: aNSSymbol
	targetNamespace := aNSSymbol
!

xml
	^schemaNode xml
! !

!XeSchemaWriter methodsFor:'accessing-private'!

addNamespace: aNSSymbol
	(self shouldAddImport: aNSSymbol) 
		ifTrue: 
			[imported add: aNSSymbol.
			(schemaNode addElementFirst: 'xsd:import') setAttribute: 'namespace' value: aNSSymbol]
!

schemaNode
	^schemaNode
! !

!XeSchemaWriter methodsFor:'branching-private'!

postCopy
	super postCopy.
	isNaming := true
! !

!XeSchemaWriter methodsFor:'initializing-private'!

initializeTargeting: aSymbol node: aXeNode schemaNS: aSchemaNSSymbol
	targetNamespace := aSymbol.
	schemaNode := aXeNode addElement: 'xsd:schema'.
	isElementFormQualified := false.
	isAttributeFormQualified := false.
	imported := OrderedCollection new.
	schemaNode setAttribute: 'targetNamespace' value: targetNamespace asString.
	schemaNode addNSPrefix: 'xsd' to: aSchemaNSSymbol.
	self reset
! !

!XeSchemaWriter methodsFor:'modes'!

beAttributeFormQualified 
	isAttributeFormQualified := true.
	schemaNode setAttribute: 'attributeFormDefault' value: 'qualified'
!

beAttributeFormUnqualified 
	isAttributeFormQualified := false.
	schemaNode setAttribute: 'attributeFormDefault' value: 'unqualified'
!

beElementFormQualified 
	isElementFormQualified := true.
	schemaNode setAttribute: 'elementFormDefault' value: 'qualified'
!

beElementFormUnqualified 
	isElementFormQualified := false.
	schemaNode setAttribute: 'elementFormDefault' value: 'unqualified'
!

beInline
	isNaming := false
!

beNaming
	isNaming := true
! !

!XeSchemaWriter methodsFor:'operations-private'!

reset
	currentNode := schemaNode.
	self beNaming
! !

!XeSchemaWriter methodsFor:'schema writing-declaration'!

globalDeclareInline: aXeElement 
	| branch |
	branch := self branch.
	branch 
		addElement: 'xsd:element';
		addNameAttribute: aXeElement;
		beInline.
	branch visitContent: aXeElement content 
!

globalDeclareTyped: aXeElement 
	self branch
		addElement: 'xsd:element';
		addNameAttribute: aXeElement;
		addTypeAttribute: aXeElement content.
	self define: aXeElement content.
!

localDeclareAttribute: aXeAttribute 
	|branch|
	branch := self branch.
	branch 
		addElement: 'xsd:attribute';
		addNameAttribute: aXeAttribute ;
		addAttributeFormAttribute: aXeAttribute;
		addTypeAttribute: aXeAttribute content.
        
	aXeAttribute isFixed ifTrue: [branch currentNode setAttribute: 'fixed' value: aXeAttribute fixedValue]

        
!

localDeclareChoice: aXeChoiceElement 
	| branch |
	branch := self branch.
	branch addElement: 'xsd:choice'.
	aXeChoiceElement choices do: [ :each | branch declareLocally: each]
        
	
!

localDeclareInline: aXeElement 
	| branch |
	branch := self branch.
	branch 
		addElement: 'xsd:element';
		addNameAttribute: aXeElement;
		addNillableAttribute: aXeElement;
		addElementFormAttribute: aXeElement;
		addElementOccurenceAttributes: aXeElement;
		beInline.
	branch visitContent: aXeElement content 
!

localDeclareTyped: aXeElement 
	self branch
		addElement: 'xsd:element';
		addNameAttribute: aXeElement;
		addNillableAttribute: aXeElement;
		addElementFormAttribute: aXeElement;
		addElementOccurenceAttributes: aXeElement;
		addTypeAttribute: aXeElement content
	"aXeElement content putTypeAttributeOn: branch currentNode"
! !

!XeSchemaWriter methodsFor:'schema writing-definition-private'!

writeArray: anArrayContent 
	"Private - This is used to define a complexType that will encode Arrays in literal style.
	-This is NOT used to define soap:Arrays"

	self
		addElement: 'xsd:complexType';
		addNameAttribute: anArrayContent.               
	self branch
		writeSequence: anArrayContent.

        
!

writeComplex: aXeComplexContent
	self
		addElement: 'xsd:complexType';
		addNameAttribute: aXeComplexContent.
	self branch writeSequence: aXeComplexContent.
	aXeComplexContent attributes do: [:each | self declareLocally: each]
!

writeExtended: aXeSimpleExtendedContent
	self
		addElement: 'xsd:complexType';
		addNameAttribute: aXeSimpleExtendedContent;
		addElement: 'xsd:simpleContent';
		addElement: 'xsd:extension';
		addBaseAttribute: aXeSimpleExtendedContent.
	"self currentNode setAttribute: 'base' qName: aXeSimpleExtendedContent baseTypeName."
	aXeSimpleExtendedContent attributes do: [:each | self declareLocally: each]
!

writeRestricted: aXeRestrictedContent
	self
		addElement: 'xsd:simpleType';
		addNameAttribute: aXeRestrictedContent;
		addElement: 'xsd:restriction';
		addBaseAttribute: aXeRestrictedContent.
	"self currentNode setAttribute: 'base' qName: aRestrictedContent baseTypeName."
	aXeRestrictedContent restrictions do: [:each | each addToNode: currentNode]
!

writeSequence: aXeComplexContent
	self addElement: 'xsd:sequence'.
	aXeComplexContent childElements do: [:each | self declareLocally: each]
!

writeSimple: aXeSimpleContent
	self
		addElement: 'xsd:simpleType';
		addNameAttribute: aXeSimpleContent;
		addElement: 'xsd:restriction';
		addBaseAttribute: aXeSimpleContent
!

writeSoapArray: anArrayContent 
        "Private - Note: The 'correct' interpretation of how to define arrays in the WSDL spec changes each month"

        |b|

        self
                addElement: 'xsd:complexType';
                addNameAttribute: anArrayContent;
                addElement: 'xsd:complexContent';
                addElement: 'xsd:restriction';
                addBaseAttribute: anArrayContent.
        "self branch
                writeSequence: anArrayContent."
"/        self branch
"/                addElement: 'xsd:attribute';
"/                currentNode setAttribute: 'ref' qName: anArrayContent arrayTypeAttribute;
"/                currentNode setQAttribute: anArrayContent wsdlArrayTypeAttribute qName: anArrayContent  wsdlArrayType.

        b := self branch.
        b addElement: 'xsd:attribute'.
        b currentNode setAttribute: 'ref' qName: anArrayContent arrayTypeAttribute.
        b currentNode setQAttribute: anArrayContent wsdlArrayTypeAttribute qName: anArrayContent  wsdlArrayType.

"/        ((self branch
"/                addElement: 'xsd:attribute';
"/                currentNode) setAttribute: 'ref' qName: anArrayContent arrayTypeAttribute;
"/                currentNode) setQAttribute: anArrayContent wsdlArrayTypeAttribute qName: anArrayContent  wsdlArrayType.
        "self branch
                addElement: 'xsd:attribute';
                currentNode setAttribute: 'ref' qName: anArrayContent offsetAttribute.
        self branch
                addElement: 'xsd:attributeGroup';
                currentNode setAttribute: 'ref' qName: anArrayContent commonAttributeAttribute."

        self addNamespace: XeNamespace soapEnc.
        self addNamespace: XeNamespace wsdl.
        anArrayContent childTypeName notNil ifTrue: [
                self addNamespace: anArrayContent childTypeName namespace]              
        "anArrayContent putWSDLArrayTypeAttributeOn: self currentNode"
! !

!XeSchemaWriter methodsFor:'schema writing-helpers-private'!

addAttributeFormAttribute: aXeAttribute
	(self shouldWriteAttributeForm: aXeAttribute) 
		ifTrue: [currentNode setAttribute: 'form' value: aXeAttribute form]
!

addBaseAttribute: aXeContent
	self assert: [aXeContent baseTypeName notNil].
	currentNode setAttribute: 'base' qName: aXeContent baseTypeName.
	self addNamespace: aXeContent baseTypeName namespace
!

addElement: aString
	self currentNode: (self currentNode addElement: aString)
!

addElementFormAttribute: aXeElement
	(self shouldWriteElementForm: aXeElement) 
		ifTrue: [currentNode setAttribute: 'form' value: aXeElement form]
!

addElementOccurenceAttributes: aXeElement
	aXeElement isMultipleOccurs 
		ifTrue: [currentNode setAttribute: 'maxOccurs' value: 'unbounded'].
	(aXeElement isNillable or: [aXeElement isOmitable]) 
		ifTrue: [currentNode setAttribute: 'minOccurs' value: '0']
!

addNameAttribute: aXeElement
	self isNaming ifTrue: [currentNode setAttribute: 'name' value: aXeElement localName]
!

addNillableAttribute: aXeElement
	aXeElement isNillable ifTrue: [currentNode setAttribute: 'nillable' value: 'true']
!

addTypeAttribute: aXeContent
	currentNode setAttribute: 'type' qName: aXeContent typeName.
	self addNamespace: aXeContent namespace
! !

!XeSchemaWriter methodsFor:'testing-private'!

isNaming
	isNaming isNil ifTrue: [isNaming := true].
	^isNaming
!

shouldAddImport: aNSSymbol
	^((XeNamespace isSchemaNS: aNSSymbol) 
		or: [targetNamespace == aNSSymbol or: [imported includes: aNSSymbol]]) not
!

shouldDeclare: aParticle
	^aParticle namespace == targetNamespace and: [(aParticle isDeclaredIn: schemaNode) not]
!

shouldDefine: aXeContent
	^self isNaming not 
		or: [aXeContent namespace == targetNamespace and: [(aXeContent isDefinedIn: schemaNode) not]]
!

shouldWriteAttributeForm: aXeAttribute
	^aXeAttribute isFormQualified ~= isAttributeFormQualified
!

shouldWriteElementForm: aXeElement
	^aXeElement isFormQualified ~= isElementFormQualified
! !

!XeSchemaWriter methodsFor:'visiting'!

declare: aXeElementOrAttribute
	self declareGlobally: aXeElementOrAttribute
!

declareAnyAttribute: aXeAnyAttribute
	self branch addElement: 'xsd:anyAttribute'
!

declareGlobally: aXeElementOrAttribute
	self reset.
	(self shouldDeclare: aXeElementOrAttribute)
		ifTrue: [aXeElementOrAttribute declareGloballyUsing: self]
!

declareLocally: aXeElementOrAttribute
	aXeElementOrAttribute declareLocallyUsing: self
!

define: aContent 
	self reset.
	aContent acceptVisitor: self
! !

!XeSchemaWriter methodsFor:'visitor'!

acceptAny: anAnyContent
	
!

acceptArray: anArrayContent 
	(self shouldDefine: anArrayContent) ifTrue: [self writeArray: anArrayContent].
	self define: anArrayContent childElement content
!

acceptComplex: aComplexContent 
	"add type definition to the root schema node"

	(self shouldDefine: aComplexContent) ifTrue: [self writeComplex: aComplexContent].
	aComplexContent childElements 
		do: [:each | each isAnonymous ifFalse: [each allContent do: [:c | self define: c]]]
!

acceptSimple: aSimpleContent 
	(self shouldDefine: aSimpleContent) ifTrue: [self writeSimple: aSimpleContent]
!

acceptSimpleExtended: aXeSimpleExtendedContent 
	(self shouldDefine: aXeSimpleExtendedContent) 
		ifTrue: 
			[self assert: [aXeSimpleExtendedContent baseContent isNamed].
			self writeExtended: aXeSimpleExtendedContent ].
	self define: aXeSimpleExtendedContent baseContent
!

acceptSimpleRestricted: aRestrictedContent 
	(self shouldDefine: aRestrictedContent) 
		ifTrue: 
			[self assert: [aRestrictedContent baseContent isNamed].
			self writeRestricted: aRestrictedContent].
	self define: aRestrictedContent baseContent
!

acceptSoapArray: anArrayContent 
	(self shouldDefine: anArrayContent) ifTrue: [self writeSoapArray: anArrayContent].
	self define: anArrayContent childElement content
! !

!XeSchemaWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSchemaWriter.st,v 1.6 2008/09/05 17:45:17 cg Exp $'
! !
