"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentAbstract subclass:#XeSimpleContent
	instanceVariableNames:'converter'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSimpleContent comment:'I am a content component that can encode/decode simple types 

==========
To view an instance diagram image evaluate showExampleImage
See example
        
===========
Instance Variables:
	converter               <XeTextConverter>

'
!


!XeSimpleContent class methodsFor:'instance creation'!

onConverter: aXeTextConverter typeName: aXeQName base: aXeQName2 encodingStyle: anEncStyle
	^(self new)
		converter: aXeTextConverter;
		typeName: aXeQName;
		baseTypeName: aXeQName2;
		encodingStyle: anEncStyle;
		yourself
! !

!XeSimpleContent class methodsFor:'must strip'!

example
        <resource: #example>

        "Create an element with simpleContent using an <XeElementSpec>"

        | typespace element writer |
        typespace := XeTypespace defaultSoapEnc2001.
        element := (XeElementSpec named: #foo xsd: 'string') 
                                xeParticleWith: typespace soapEncStyle.

        "use it to encode something"
        element beTyped.
        element beEncodeBelow.
        writer := XeSoapXMLWriter write: 'Hello' element: element.
        ^writer xml

        "<root
        xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
        xmlns:xsd='http://www.w3.org/2001/XMLSchema'>
        <foo xsi:type='xsd:string'>Hello</foo>
        </root>"
! !

!XeSimpleContent methodsFor:'accessing'!

converter
	"Answer the <XeTextConverter> the receiver uses to convert to/from text"

	^converter
! !

!XeSimpleContent methodsFor:'accessing-private'!

converter: aXeTextConverter
	converter := aXeTextConverter
! !

!XeSimpleContent methodsFor:'operations'!

convertNode:aNode 
    "Answer an <Object> converted from the aNode's text.
            A value range exception is signaled when a valid value is received that can not
            be mapped to a dolphin value (ie float NaN etc)"
    
    ^ [
        converter fromString:(self extractTextFrom:aNode)
    ] on:XeValueRangeError do:[:e | 
        UserPreferences current soapErrorDebugging ifTrue:[ e reject ].
        self typespaceOptions isConvertAllValues ifTrue:[
            self newAnyValueFrom:aNode
        ] ifFalse:[ 
            e reject 
        ]
    ]

    "Modified: / 12-07-2011 / 17:52:32 / cg"
!

convertToObject: text
	"Answer an <Object> converted from  text"

	^converter fromString: text
!

convertValue: aValue
	"Convert aValue into text"

	^self converter asString: aValue
!

newAnyValueFrom: aXeNode
        | av |
        av := XeAnyValue new text: aXeNode text.
        av typeName: ([aXeNode qValueForXSIType] on: Error do: [:e | self halt:'debughalt'.]).
        ^av
! !

!XeSimpleContent methodsFor:'operations-private'!

extractTextFrom: aNode
	^self isReadUTF8 
		ifTrue: [aNode extractChildUTF8Text]
		ifFalse: [aNode extractChildASCIIText]
! !

!XeSimpleContent methodsFor:'testing'!

isSimpleContent 
	^true
!

isSoapSpecialMultiRef
	"Answer if the receiver's type is one of the SOAP1.1 spec's 'special' types that allows inline multireferences"

	self typeName isNil ifTrue: [^false].
	^#('string' 'base64' 'base64Binary') includes: self localName
! !

!XeSimpleContent methodsFor:'testing-private'!

isReadUTF8
	^self typespaceOptions isReadUTF8
! !

!XeSimpleContent methodsFor:'visitor'!

acceptVisitor: aContentVisitor
	^aContentVisitor acceptSimple: self
! !

!XeSimpleContent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSimpleContent.st,v 1.9 2011/07/12 16:20:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSimpleContent.st,v 1.9 2011/07/12 16:20:27 cg Exp $'
! !
