"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentAbstract subclass:#XeSimpleRestrictedContent
	instanceVariableNames:'restrictions baseContent'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSimpleRestrictedContent comment:'I am a content component that can encode/decode simple restricted types 

I ask my baseContent to do the conversions to/from text, and I validate the values using my restrictions

Instance Variables:
	restrictions    <Collection> of <XeSchemaRestriction>
	baseContent     <XeSimpleContent>'
!


!XeSimpleRestrictedContent class methodsFor:'must strip'!

example
        <resource: #example>

        | typespace content |
        typespace := XeTypespace defaultSoapEnc2001.
        content := 'xsd:int' asXeName xeContentWith: typespace soapEncStyle.
        content convertToObject: '2147483647'.  "Evaluate It"
        content convertToObject: '2147483649'   "Will signal error"
! !

!XeSimpleRestrictedContent methodsFor:'accessing'!

baseContent
	"Answer a <XeSimpleContent>"

	^baseContent
!

baseContent: aXeContent
	baseContent := aXeContent
!

baseTypeName
	"Answer the type <XeQName> of my baseContent"

	baseTypeName isNil ifTrue: [baseTypeName := self baseContent typeName].
	^baseTypeName
!

restrictions
	"Answer a collection of <XeSchemaRestriction>s"

	^restrictions
!

restrictions: aCollOfRestrictions
	restrictions := aCollOfRestrictions
! !

!XeSimpleRestrictedContent methodsFor:'exceptions-private'!

errorValidation: aRestriction failed: aValue
        "Signal an error that the validation failed.
        This can be safely resumed"

        XeValueMismatch signal: ('The value: %1 does not match the type: %2 because of: %3' 
                                formatWith: aValue printString
                                with: self typeName displayString
                                with: aRestriction "component" displayString)
! !

!XeSimpleRestrictedContent methodsFor:'operations'!

convertToObject: text
	"Convert the text to an <Object> and then assert that the object  is a valid value for the receiver"

	| value |
	value := self baseContent convertToObject: text.
	(self shouldntValidate: value) ifFalse: [self validate: value].
	^value
!

convertValue: anObject
	"Assert that anObject is a valid value for the receiver and then convert anObject into text"

	(self shouldntValidate: anObject) ifFalse: [self validate: anObject].
	^self baseContent convertValue: anObject
!

validate: aValue 
        self restrictions 
                do:[:each | 
                    ([each validate: aValue] 
                        on: Error 
                        do: [:e | 
                            UserPreferences current soapErrorDebugging ifTrue:[ e reject ].
                            self breakPoint:#cg. false
                        ]
                    ) ifFalse: [
                        self errorValidation: each failed: aValue
                    ]
                ]

    "Modified: / 12-07-2011 / 17:54:09 / cg"
! !

!XeSimpleRestrictedContent methodsFor:'testing'!

isSimpleContent 
	^true
!

isSimpleRestrictedContent 
        ^true

    "Created: / 20-03-2012 / 15:50:52 / cg"
! !

!XeSimpleRestrictedContent methodsFor:'testing-private'!

hasRestrictions
	^restrictions notNil and: [restrictions notEmpty]
!

shouldntValidate: value
	^value isNil or: [(value isKindOf: XeAnyValue) or: [self hasRestrictions not]]
! !

!XeSimpleRestrictedContent methodsFor:'visitor'!

acceptVisitor: aContentVisitor 
	^aContentVisitor acceptSimpleRestricted: self
! !

!XeSimpleRestrictedContent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSimpleRestrictedContent.st,v 1.11 2012/03/20 18:18:48 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSimpleRestrictedContent.st,v 1.11 2012/03/20 18:18:48 cg Exp $'
! !
