"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeSoapArrayContentAbstract subclass:#XeSoapArrayContent
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapArrayContent comment:'I am a content component that can encode/decode soap-enc arrays 

I encode single dimension arrays.

=========
Encoding
	An Array will encode into a standard soap-enc array
	A KeyedCollection will encode into a sparse array, using the keys as the position attribute.

=========
Decoding
	A soap-enc:Array will normally decode into an Array, except if;
		-the array is sparse (includes position attributes), it will decode into a keyedCollection
		-the array is partial (includes offset attribute (except if zero))

	The reason for this is to stop dolphin from constructing massive arrays in response to 
	large positions/offsets
'
!


!XeSoapArrayContent class methodsFor:'must strip'!

example
        <resource: #example>

        | element ts typeName |
        ts := XeTypespace defaultSoapEnc2001.
        typeName := 'soapE:ArrayOfstring' asXeName.
        element := (XeElementSpec named: #myArray typeName: typeName) 
                                xeParticleWith: ts soapEncStyle.
        ^(XeSoapXMLWriter write: #('hello' 'goodbye') element: element) xml
! !

!XeSoapArrayContent methodsFor:'accessing'!

readerClass
	^XeSoapArrayNode
! !

!XeSoapArrayContent methodsFor:'constants'!

oneKey
	^1
!

zeroKey
	^0
! !

!XeSoapArrayContent methodsFor:'constants-private'!

aSizeFormString
	^'[]'
! !

!XeSoapArrayContent methodsFor:'helpers'!

aSizeStringFor: aNumber
	"This allows a 2D array to be encoded into a single dimension array ... not sure if this is a good idea"

	| actualNumber |
	actualNumber := (aNumber isKindOf: Point) 
		ifTrue: [ aNumber x * aNumber y]
		ifFalse: [aNumber].
	^'[' , actualNumber displayString , ']'
! !

!XeSoapArrayContent methodsFor:'visitor'!

acceptVisitor: aContentVisitor
	^aContentVisitor acceptSoapArray: self
! !

!XeSoapArrayContent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapArrayContent.st,v 1.5 2008/09/05 17:45:32 cg Exp $'
! !
