"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeSoapArrayNodeAbstract
	instanceVariableNames:'node arrayType childNodes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeSoapArrayNodeAbstract comment:'I am a soap:Array node.

I wrap the XeNode and provide some helper access methods and cache some of the calculated information.

Instance Variables:
	node		<XeBNode>
	arrayType		<XeQName>
	childNodes	<Collection> of <XeNode>s'
!


!XeSoapArrayNodeAbstract class methodsFor:'instance creation'!

forNode: aNode 
	| instance |
	instance := XeSoapArrayNode on: aNode.
	^(instance aSizeString includes: $,) ifTrue: [XeSoapMultiDimArrayNode on: aNode] ifFalse: [instance]
!

on: aXeNode
	^self new node: aXeNode
! !

!XeSoapArrayNodeAbstract methodsFor:'accessing'!

arrayType
	arrayType isNil ifTrue: [ arrayType := node getAttribute: 'arrayType' in: XeNamespace soapEnc].
	^arrayType
!

childNodes
	childNodes isNil ifTrue: [childNodes := node childNodes].
	^childNodes
!

itemTypeQName
	^node makeQName: self itemTypeQString
!

itemTypeQString
	self arrayType isNil ifTrue: [self error: 'Expecting arrayType attribute'].
	^self arrayType readStream upTo: $[
!

offsetAttribute
	| offset |
	offset := node 
				getAttribute: 'offset'
				in: XeNamespace soapEnc
				ifNone: [^self zeroKey].
	^self extractSize: offset
! !

!XeSoapArrayNodeAbstract methodsFor:'accessing-private'!

node
	"Private - Answer the value of the receiver's ''node'' instance variable."

	^node
!

node: aXeNode
	node := aXeNode
! !

!XeSoapArrayNodeAbstract methodsFor:'asserting'!

assertValidArrayNode
	(self hasArrayType) ifFalse: [self error: 'Expecting Array with arrayType attribute']
! !

!XeSoapArrayNodeAbstract methodsFor:'exceptions-private'!

errorAttributeForm
	^self subclassResponsibility
! !

!XeSoapArrayNodeAbstract methodsFor:'helpers'!

aSize
	"See the soap spec."

	^self extractSize: self aSizeString
!

aSizeString
	| splitIndex |
	self arrayType isNil ifTrue: [self error: 'Expecting arrayType attribute'].
	(splitIndex := self arrayType indexOf: $[) = 0 ifTrue: [self errorAttributeForm].
	^self arrayType copyFrom: splitIndex to: self arrayType size
!

extractSize: aString 
	^self subclassResponsibility
! !

!XeSoapArrayNodeAbstract methodsFor:'testing'!

hasArrayType
	^self arrayType notNil
!

hasPositionNode
	^self childNodes anySatisfy: [ :each |
		(each 
			getAttribute: 'position'
			in: XeNamespace soapEnc
			ifNone: [false]) ~~ false]

!

isKeyedNode
	"There are security issues with certain soap-enc arrays.
	Answer whether to decode the soap:Array as a KeyedCollection"
	
	^self isOffsetNode or: [ 
		self hasPositionNode]
!

isMultiDim
	^false
!

isOffsetNode
	^(self extractSize: (node 
				getAttribute: 'offset'
				in: XeNamespace soapEnc
				ifNone: [^false])) 
		isZero not
! !

!XeSoapArrayNodeAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapArrayNodeAbstract.st,v 1.4 2003/10/08 17:31:40 cg Exp $'
! !
