"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentFactory subclass:#XeSoapEncFactory
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapEncFactory comment:'I create XeComponents for the soap-enc namespace'
!


!XeSoapEncFactory class methodsFor:'constants'!

defaultTargetNamespace
	^XeNamespace soapEnc
!

preferedPrefix

	^'SOAP-ENC'
! !

!XeSoapEncFactory class methodsFor:'must strip'!

example
        <resource: #example>

        | typespace |
        typespace := XeTypespace defaultSoapEnc2001.

        "Content "
        (XeQName soapEnc: 'base64') xeContentWith: typespace soapEncStyle.

        "Particle "
        (XeQName soapEnc: 'float') xeParticleWith: typespace soapEncStyle
! !

!XeSoapEncFactory methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer a <XeContent>
	This factory answers either <XeArrayContent>, or a base64 <XeSimpleContent>. "

	aTypeQName localName = 'Array' ifTrue: [^anEncodingStyle arraySoapEnc: aTypeQName].
	aTypeQName localName = 'base64' 
		ifTrue: [^anEncodingStyle simple: aTypeQName converter: XeTextConverter base64Binary].
	self errorNotSupported: aTypeQName
! !

!XeSoapEncFactory methodsFor:'particle creation'!

xeElementDeclared: aQName encStyle: anEncodingStyle
	"Answer a <XeElement>
	SoapSpec sec5.2
If a simple value is encoded as an independent element or member of a heterogenous array it is convenient to have an element declaration corresponding to the datatype. Because the XML Schema Part 2: Datatypes Specification [11] includes type definitions but does not include corresponding element declarations, the SOAP-ENC schema and namespace declares an element for every simple datatype. These MAY be used.
<SOAP-ENC:int id='int1'>45</SOAP-ENC:int>"

	"Check for the declarations with definitions in soap-enc, otherwise assume an xsd type"

	| spec |
	spec := (self isSoapEncType: aQName) 
				ifTrue: [XeElementSpec named: aQName typeName: aQName]
				ifFalse: [XeElementSpec named: aQName typeName: (self xsdQualify: aQName)].
	^spec xeParticleWith: anEncodingStyle
! !

!XeSoapEncFactory methodsFor:'testing-private'!

isSoapEncType: aQName 
	^#('Array' 'Struct' 'base64') includes: aQName localName
! !

!XeSoapEncFactory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapEncFactory.st,v 1.5 2008/09/05 17:45:45 cg Exp $'
! !
