"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeEncodingStyle subclass:#XeSoapEncodingStyle
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapEncodingStyle comment:'I am the soap-enc encoding style.

Most of my additional methods are soap:Array related

See XeContentFactory subclass examples'
!


!XeSoapEncodingStyle class methodsFor:'must strip'!

example
        <resource: #example>

        "Examples of creating content from objects and xml instances"

        | typespace |
        typespace := XeTypespace defaultSoapEnc2001.

        "Content from xml (the inline type attribute is used"
        typespace soapEncStyle elementForNode: (XeEnvironment 
                                rootNodeFor: '<root xsi:type="xsd:string" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">abc</root>').

        "Content from instance - simple"
        typespace soapEncStyle elementForObject: 'abc'.
        "Content from instance - complex"
        typespace soapEncStyle elementForObject: (self environment at: #PurchaseOrders) example1.
! !

!XeSoapEncodingStyle methodsFor:'accessing'!

encodingNamespace
	^XeNamespace soapEnc
!

mismatchPolicy
	^XeSoapTypeMismatchPolicy new

!

namespace
	^XeNamespace soapEnc
! !

!XeSoapEncodingStyle methodsFor:'accessing-private'!

complexContentClass
	^XeQStruct
! !

!XeSoapEncodingStyle methodsFor:'configuration-private'!

configureComplexAttributes: attributes
	attributes 
		do: [:each | self error: 'Use of attributes to encode values not allowed in soapEnc']
!

configureComplexChildElements: elements
	elements do: 
			[:each | 
			each
				beEncodeBelow;
				beFormUnqualified;
				beKeyedAccessor]
!

configureSoapArrayChildElement: element
	element
		beEncodeBelow;
		beFormUnqualified;
		beKeyedAccessor;
		beMultipleOccurs;
		beNillable
!

configureSprayAttributes: attributes
	attributes 
		do: [:each | self error: 'Use of attributes to encode values not allowed in soapEnc']
!

configureSprayChildElements: elements
	elements do: 
			[:each | 
			each
				beEncodeBelow;
				beFormUnqualified;
				beSelectorAccessor]
! !

!XeSoapEncodingStyle methodsFor:'content creation-complex'!

complex: aTypeQName restricting: aBaseQName xsdContent: anXSDContentModel
	"Answer a <XeComplexContent> (used by a <XeXSDSchema> factory)"

	^(aBaseQName localName = 'Array' and: [aBaseQName namespace == self encodingNamespace]) 
		ifTrue: 
			[self 
				array: aTypeQName
				restricting: aBaseQName
				xsdContent: anXSDContentModel]
		ifFalse: 
			[anXSDContentModel assertValidSoapEncStruct.
			self complex: aTypeQName elements: (anXSDContentModel elementsWith: self)]
!

complexSpray: aTypeQName factory: aXeSprayClassFactory
	"Answer a <XeComplexContent> (used by a <XeSprayClassFactory>)"

	^(self isArrayTypeName: aTypeQName) 
		ifTrue: [self arrayWsdl: aTypeQName elementNamespace: aTypeQName namespace]
		ifFalse: [super complexSpray: aTypeQName factory: aXeSprayClassFactory]
! !

!XeSoapEncodingStyle methodsFor:'content creation-complex-private'!

complexRPC: aTypeQName elements: elements
	"Private - Answer a <XeSoapRPCContent>"

	self configureComplexChildElements: elements.
	^(XeSoapRPCContent new)
		typeName: aTypeQName;
		childElements: elements;
		encodingStyle: self;
		convertClass: XeQStruct;
		yourself
! !

!XeSoapEncodingStyle methodsFor:'content creation-simple'!

simple: aTypeQName extending: aBaseTypeQName xsdContent: anXSDContent
	self error: 'Simple content extension not supported in this encodingStyle'
!

simpleSpray: aTypeQName factory: aFactory
	self error: 'Simple content extension not supported in this encodingStyle'
! !

!XeSoapEncodingStyle methodsFor:'content creation-soap'!

array: aXeQName restricting: aXeQName2 xsdContent: aXeXSDContent
	"Answer a <XeArrayContent>"

	| attribute |
	^(attribute := aXeXSDContent wsdlArrayAttributeNode) notNil 
		ifTrue: [self array: aXeQName wsdlAttribute: attribute]
		ifFalse: [self array: aXeQName xsdElementContent: aXeXSDContent]
!

array: aTypeQName wsdlAttribute: attribute
	"Answer a <XeArrayContent>"

	| splitArrayType arrayElement |
	splitArrayType := self 
				splitArrayTypeName: (attribute qValueForQ: ('arrayType' ns: XeNamespace wsdl)).
	arrayElement := (XeElementSpec named: 'item' typeName: splitArrayType first) 
				xeParticleWith: self.
	^self 
		array: aTypeQName
		extending: arrayElement
		aSize: splitArrayType second
!

arraySoapEnc: aTypeQName
	"Answer a <XeArrayContent>"

	^self 
		array: aTypeQName
		class: XeSoapArrayContent
		element: ((XeElementSpec named: 'item' xsd: 'anyType') xeParticleWith: self)
!

arrayWsdl: aTypeQName
	"Answer a <XeArrayContent>
	aTypeQName is of the form suggested in the WSDL1.1 spec; ArrayOfXXX.
	Spray uses this *only* to create arrays of base types."

	^self arrayWsdl: aTypeQName elementNamespace: self typespace xsdNamespace
! !

!XeSoapEncodingStyle methodsFor:'content creation-soap-private'!

array: aTypeQName class: aClass element: anElement
	"Answer a <XeArrayContent>"

	self configureSoapArrayChildElement: anElement.
	^(aClass new)
		encodingStyle: self;
		typeName: aTypeQName;
		childElement: anElement;
		yourself
!

array: aTypeQName class: aClass elementSpeced: anElementSpec
	"Answer a <XeArrayContent>"

	^self 
		array: aTypeQName
		class: aClass
		element: (anElementSpec xeParticleWith: self)
!

array: aTypeQName extending: anElement aSize: aString
	"Answer a <XeArrayContent>"

	^self 
		array: aTypeQName
		class: (self arrayContentClassFor: aString)
		element: anElement
!

array: aTypeQName xsdElementContent: aXMLXSDContent
	"Answer a <XeArrayContent>"

	| elements |
	elements := aXMLXSDContent elementsWith: self.
	elements size = 1 
		ifFalse: 
			[self 
				error: 'Error extracting array definition. Expected either a WSDL1.1 definition (arrayType attribute ref) or a single element'].
	^self 
		array: aTypeQName
		extending: elements first
		aSize: nil
!

arrayExtending: aXeElement aSize: aString
	"Answer a <XeArrayContent>"

	^self 
		array: (XeQName 
				prefix: 'SOAP-ENC'
				local: 'Array'
				ns: self encodingNamespace)
		extending: aXeElement
		aSize: aString
!

arrayWsdl: aTypeQName elementNamespace: aNamespace
	"Answer a <XeArrayContent>
	Private - aTypeQName is of the form suggested in the WSDL1.1 spec; ArrayOfXXX."

	^self 
		array: (self fixupArrayTypeName: aTypeQName)
		class: (self arrayContentClassFor: aTypeQName localName)
		element: (self arrayExtensionElement: aTypeQName in: aNamespace)
! !

!XeSoapEncodingStyle methodsFor:'helpers'!

decodeInstanceNode: aXeNode
	| element |
	element := self elementForNode: aXeNode.
	^element decode: (XeSoapXMLReader new initializeOn: aXeNode)
! !

!XeSoapEncodingStyle methodsFor:'helpers-private'!

arrayContentClassFor: anASizeString
	anASizeString isNil ifTrue: [^XeSoapArrayContent].      "probably 1999 schema"
	^(anASizeString includes: $,) 
		ifTrue: [XeSoapMultiDimContent]
		ifFalse: [XeSoapArrayContent]
!

arrayExtensionElement: aXeQName in: aNamespace
        "Private - Soap-enc specific, WSDL1.1 recommended method of Array naming  
        The item type type is appended to the string 'ArrayOf'. We parse out the item type and use the schema targeting aNamespace to create it.
        This allows us to use this method to create arrays of class definitions .. ie ArrayOfSomething, where Something is a class"

        | localName extensionName |
        localName := aXeQName localName readStream upTo: $[.
        (localName beginsWith: 'ArrayOf') 
                ifFalse: [self error: 'Invalid localName for specifying an Array'].
        extensionName := localName copyFrom: "##" ('ArrayOf' size + 1) to: localName size.
        self assert: [(extensionName includes: $:) not].
        ^(XeElementSpec named: 'item' typeName: (XeQName local: extensionName ns: aNamespace)) 
                xeParticleWith: self
!

fixupArrayTypeName: aTypeQName
	| local |
	local := aTypeQName localName.
	(local includes: $[) ifTrue: [local := local readStream upTo: $[].
	self assert: [(aTypeQName localName includes: $:) not].
	^XeQName 
		prefix: aTypeQName prefix
		local: local
		ns: aTypeQName namespace
!

splitArrayTypeName: aXeQName
	"Private - This splits the 'item' type from the aSize. for example;
	SOAP-ENC:arrayType='xsd:string[2]'  .... xsd:string is the 'item' type, and [2] is the aSize"

	| baseName rest stream |
	stream := aXeQName localName readStream.
	baseName := stream upTo: $[.
	rest := stream upToEnd.
	(rest select: [:each | each = $,]) size > 1 
		ifTrue: [self error: 'Only one or two dimension arrays supported'].
	(rest isEmpty or: [rest last ~= $]]) ifTrue: [self error: 'Invalid arrayType'].
	^Array with: (XeQName 
				prefix: aXeQName prefix
				local: baseName
				ns: aXeQName namespace)
		with: (rest copyFrom: 1 to: rest size - 1)
! !

!XeSoapEncodingStyle methodsFor:'particle creation'!

attributeSpeced: aXeAttributeSpec
	self error: 'Use of attributes to encode values not allowed in soapEnc'
!

elementForNode: aNode 
	"If the node is a reference to another node, firstly dereference it (but keep its accessor name)"

	| href derefNode |
	^(href := aNode getAttribute: 'href') notNil 
		ifTrue: 
			[derefNode := (XeNodeReadMap on: aNode) dereferenceNode: aNode.
			(self basicElementForNode: derefNode) qName: aNode qName]
		ifFalse: [self basicElementForNode: aNode]
! !

!XeSoapEncodingStyle methodsFor:'particle creation-private'!

basicElementForArrayNode: aXeNode
	"Private - Extract the information needed to create an XeArrayContent from the node. This is typically of the form;
<inputStringArray SOAP-ENC:arrayType='xsd:string[2]' xsi:type='SOAP-ENC:Array'>"

	| arrayType typeName splitArrayType arrayElement content |
	typeName := aXeNode qValueForXSIType.
	splitArrayType := self 
				splitArrayTypeName: (aXeNode qValueForQ: (XeQName soapEnc: 'arrayType')).
	arrayType := splitArrayType first.
	arrayElement := (XeElementSpec named: 'item' typeName: arrayType) xeParticleWith: self.
	content := typeName isNil 
				ifTrue: [self arrayExtending: arrayElement aSize: splitArrayType second]
				ifFalse: 
					[self 
						array: typeName
						extending: arrayElement
						aSize: splitArrayType second].
	^self elementSpeced: (XeElementSpec anon: aXeNode qName) content: content
!

basicElementForNode: aXeNode
	"Private - This is not commonly used in Spray, because Spray uses a WSDL to prebuild the element and types.
	This methods builds an element from an xml instance. Unfortunately to decode legal soap-enc, we must
	resort to a guess if there is no inline typing or namespace declared.
	The cases are;
		-Is it an array?
		-Is it inline typed?
		-Is the element in a namespace?
		-If none of the above ... guess"

	| typeName |
	(self isArrayNode: aXeNode) ifTrue: [^self basicElementForArrayNode: aXeNode].
	((typeName := aXeNode qValueForXSIType) notNil and: [typeName isAnyTypeName not]) 
		ifTrue: [^(XeElementSpec named: aXeNode qName typeName: typeName) xeParticleWith: self].
	^aXeNode isNullNamespace not 
		ifTrue: [self typespace xeElementDeclared: aXeNode qName encStyle: self]
		ifFalse: [XeGuessContent guessElementNamed: aXeNode qName encodingStyle: self]
! !

!XeSoapEncodingStyle methodsFor:'testing'!

isIDHref
	"Answer if the receiver allows identical objects to be encoded using soap-enc multireferences"

	^true
!

isSoapEnc
	^true
! !

!XeSoapEncodingStyle methodsFor:'testing-private'!

isArrayNode: aXeNode
	^(aXeNode getAttribute: 'arrayType' in: self encodingNamespace) notNil
!

shouldInlineType
	^true
! !

!XeSoapEncodingStyle methodsFor:'wsdl'!

docElementsSpeced: elementSpecs
	"Answer a collection of <XeElement>s"

	^elementSpecs collect: 
			[:each | 
			| element |
			element := each xeParticleWith: self.
			element beSoapRoot.
			element]
!

rpcWrapperNamed: aQName elementSpecs: elementSpecs
	"Answer a <XeElement> with <XeSoapRPCContent>"

	| element |
	element := self elementSpeced: (XeElementSpec anon: aQName)
				content: (self complexRPC: nil specs: elementSpecs).
	element beSoapRoot.
	element qName prefix: 'm'.
	^element
! !

!XeSoapEncodingStyle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapEncodingStyle.st,v 1.5 2008/09/05 17:45:38 cg Exp $'
! !
