"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeTypeMismatchPolicy subclass:#XeSoapTypeMismatchPolicy
	instanceVariableNames:'content typeName'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapTypeMismatchPolicy comment:'I am a policy that answers whether a typeName (received in an incoming soap message as an inline type attribute) matches an XeContent instance.

I need to deal with complications that arise from the SOAP1.1 spec mandating the use of schema1999, but common practice being to use schema2001. '
!


!XeSoapTypeMismatchPolicy methodsFor:'accessing'!

isContent: aContent matched: aXeQName 
	aContent typeName isNil ifTrue: [^true].
	content := aContent.
	typeName := aXeQName.
	^self isSameAs or: [self isSimilarTo] 
! !

!XeSoapTypeMismatchPolicy methodsFor:'testing'!

isBase64Match
	"This is a loose implementation until the SOAP1.2 namespace is fixed"
        
	| base64Names |
	base64Names := #('base64' 'base64Binary').
	^(base64Names includes: typeName localName) and: [base64Names includes: content localName]
!

isXSDSimilar
	"Private - -Check if the type and typeName are both schema
	-Check if the typeName is SOAP:ENCbase64, which is similar to xsd:base64Binary"

	^self isTypeNameXSD ifTrue: [self isXSDSimilarToXSD] ifFalse: [self isBase64Match]
! !

!XeSoapTypeMismatchPolicy methodsFor:'testing-private'!

isContentBase64
	^content localName = 'base64'
!

isContentSoapEnc
	^content namespace = XeNamespace soapEnc
!

isContentXSD
	^XeNamespace schemaNamespaceURIs includes: content namespace
!

isOtherSimilar
	"Private - I dont agree with this, and I can not see how it is correct, but common practice suggests
	that if I expect a restriction of SOAP-ENC:Array, I should accept SOAP-ENC:Array"

	^content baseTypeName notNil 
		and: [content baseTypeName localName = 'Array' and: [content baseTypeName = typeName]]
!

isSameAs
	^content typeName = typeName
!

isSimilarTo
	self isContentXSD ifTrue: [^self isXSDSimilar].
	self isContentSoapEnc ifTrue: [^self isSoapSimilar].
	^self isOtherSimilar
!

isSoapSimilar
	^self isContentBase64 and: [self isTypeNameBase64Binary]
!

isTypeNameBase64Binary
	^typeName = (XeQName xsd2001: 'base64Binary')
!

isTypeNameXSD
	^XeNamespace schemaNamespaceURIs includes: typeName namespace
!

isXSDSimilarToXSD
	"Private - Both type, and typeName are from the schema namespaces. 
		-Accept as a match if the baseNames are the same, or an equivalent
		-Accept if the types baseType is same (restrictions will catch value errors)"

	| names |
	names := OrderedCollection with: content localName.
	content baseTypeName notNil ifTrue: [names add: content baseTypeName localName].
	^names anySatisfy: 
			[:each | 
			each = typeName localName or: 
					[(#('dateTime' 'timeInstant') includes: each) 
						and: [#('dateTime' 'timeInstant') includes: typeName localName]]]
! !

!XeSoapTypeMismatchPolicy class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapTypeMismatchPolicy.st,v 1.4 2003/10/08 17:46:10 cg Exp $'
! !
