"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeSpecification
	instanceVariableNames:'typeName form qName'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSpecification comment:'
Instance Variables:
	qName           <XeQName> The name of the element or attribute
	typeName        <XeName> 
	form            <String> either qualified or unqualified'
!


!XeSpecification class methodsFor:'instance creation'!

anyNamed: aString
	"Answer an instance with the specified name and type.
	The type is xsd:anyType which Spray models as an instance of XeAnyElement"

	^self named: aString typeName: 'xsd:anyType'
!

named: aName typeName: aTypeName
	"Answer an instance with the specified name and type.
	aName can be either:
		a string or symbol (will be qualified in the null namespace).
		a XeQName
	aTypeName can be either:
		a QName literal <String>  ie 'prefix:local'
		a XePName
		a XeQName"

	^self new named: aName asQName typeName: aTypeName asXeName
!

named:aName typed:aTypeName ns:ns
    ^ self named: aName typeName: (XePName prefix: 'd' local:aTypeName)       
!

named: aStringOrXeQName xsd: aString
	"Answer an instance with the specified name and type.
	 -The xsd type argument should be a XMLSchema NCName literal and be one of the XMLSchema dataTypes .
	 -The instance does not qualify the type name, but creates an XePName which will late-qualify 
		into the specific XMLSchema namespace of a typespace "

	^self named: aStringOrXeQName typeName: 'xsd:' , aString
!

named: aStringOrXeQName xsdTyped: aString
        "Answer an instance with the specified name and type.
         -The xsd type argument should be a XMLSchema NCName literal and be one of the XMLSchema dataTypes .
         -The instance does not qualify the type name, but creates an XePName which will late-qualify 
                into the specific XMLSchema namespace of a typespace "

        ^self named: aStringOrXeQName typeName: 'xsd:' , aString
!

new
        ^self basicNew initialize
! !

!XeSpecification methodsFor:'accessing'!

form
	^form
!

form: aString
	form := aString
!

named: aXeQName typeName: aXeName
	qName := aXeQName.
	typeName := aXeName
!

typeName
	^typeName
! !

!XeSpecification methodsFor:'accessing-private'!

qName
	^qName
! !

!XeSpecification methodsFor:'modes'!

beFormQualified
	"Set the receiver to be form='qualified' (default is 'unqualified')
	-this controls whether local (to type definitions) elements/attributes should be qualified."

	form := 'qualified'
!

beFormQualifiedNS: aNSSymbol 
	self assert: [aNSSymbol notNil].
	self beFormQualified.
	qName namespace: aNSSymbol
! !

!XeSpecification methodsFor:'operations'!

qualifiedTypeNameIn: aTypespace
	^aTypespace qualify: self typeName
! !

!XeSpecification methodsFor:'testing'!

isFormQualified
	^form notNil and: [form = 'qualified']
! !

!XeSpecification class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSpecification.st,v 1.6 2010/06/14 10:59:52 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSpecification.st,v 1.6 2010/06/14 10:59:52 cg Exp $'
! !
