"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComplexClassAbstract subclass:#XeSprayClassFactory
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSprayClassFactory comment:'XeSprayClassFactory

==========
Background:

In a default typespace:, 
	-1/ Spray maps the prefix ''d'' to the namespace #''http://www.dolphinharbor.org/spray/class'', 
	-2/ and the namespace #''http://www.dolphinharbor.org/spray/class'' namespace to an XeSchema
	 containing an XeSprayClassFactory. 

The typespace can be configured to use any namespace:
	-1/ Change ''d'' to the new namespace, 
	-2/ and target the new namespace with a XeSprayClassFactory . 
	(See XeTypespace>>setSprayClassPrefixTo: and XeTypespace>>setSprayClassTo: for two 
	helper methods that do this)

Throughout Spray, the prefix ''d'' is used to "late-qualify" into the namespace mapped to ''d''. Similarly the prefix ''xsd'' is used to late-qualify to either the 1999schema or 2001schema. This allows literals to be used and to change their meaning by changing the typespace configuration.

Many of the SpraySamples will replace the namespace of the ''d'' prefix in order to encode/decode into specified classes instead of the default XeQStruct.

==========
Implementation

The LocalFactories ask the encodingStyle to create the content. The encodingStyle may ask the factory for child element/attribute definitions, but the actual creation of the XeContent is the responsibility of the encodingStyle. 

The XeSprayClassFactory accesses the definition of a class by sending the class:
	-#sprayElements .(answers a collection of <XeElementSpecs>) for the classes element declarations. 
	-#sprayAttributes .(answers a collection of <XeAttributeSpecs>) for the classes attribute declarations. 


==========
Example
An example follows which can be found in the "SW SpraySample BigDog" package;

CHB-Value Class - PurchaseOrders
	PurchaseOrders>>sprayElements

	^(OrderedCollection new)
		add: (XeElementSpec named: #poID xsd: ''string'' );
		add: (XeElementSpec named: #createDate xsd: ''dateTime'');
		add: (XeElementSpec named: #shipTo class: Address);
		add: (XeElementSpec named: #billTo class: Address);
		add: (XeElementSpec named: #items arrayOfClass: LineItem);
		yourself

XeElementSpec has a number of helper class construction methods. In the example above, the first XeElementSpec instance is created with a late binding XePName. When this XeElementSpec is asked to create an XeElement in a typespace , it will qualify the typeName in the namespace mapped to ''xsd'' in that typespace. Similarly the third XeElementSpec is created with a late binding XePName with the ''d'' prefix.

'
!

!XeSprayClassFactory class methodsFor:'documentation'!

documentation
"
XeSprayClassFactory

==========
Background:

In a default typespace:, 
        -1/ Spray maps the prefix 'd' to the namespace #'http://www.dolphinharbor.org/spray/class', 
        -2/ and the namespace #'http://www.dolphinharbor.org/spray/class' namespace to an XeSchema
         containing an XeSprayClassFactory. 

The typespace can be configured to use any namespace:
        -1/ Change 'd' to the new namespace, 
        -2/ and target the new namespace with a XeSprayClassFactory . 
        (See XeTypespace>>setSprayClassPrefixTo: and XeTypespace>>setSprayClassTo: for two 
        helper methods that do this)

Throughout Spray, the prefix 'd' is used to 'late-qualify' into the namespace mapped to 'd'. 
Similarly the prefix 'xsd' is used to late-qualify to either the 1999schema or 2001schema. 
This allows literals to be used and to change their meaning by changing the typespace configuration.

Many of the SpraySamples will replace the namespace of the 'd' prefix in order to encode/decode into 
specified classes instead of the default XeQStruct.

==========
Implementation

The LocalFactories ask the encodingStyle to create the content. The encodingStyle may ask the factory 
for child element/attribute definitions, but the actual creation of the XeContent is the responsibility 
of the encodingStyle. 

The XeSprayClassFactory accesses the definition of a class by sending the class:
        -#sprayElements .(answers a collection of <XeElementSpecs>) for the classes element declarations. 
        -#sprayAttributes .(answers a collection of <XeAttributeSpecs>) for the classes attribute declarations. 


==========
Example
An example follows which can be found in the 'SW SpraySample BigDog' package;

CHB-Value Class - PurchaseOrders
        PurchaseOrders>>sprayElements

        ^(OrderedCollection new)
                add: (XeElementSpec named: #poID xsd: 'string' );
                add: (XeElementSpec named: #createDate xsd: 'dateTime');
                add: (XeElementSpec named: #shipTo class: Address);
                add: (XeElementSpec named: #billTo class: Address);
                add: (XeElementSpec named: #items arrayOfClass: LineItem);
                yourself

XeElementSpec has a number of helper class construction methods. In the example above, the first 
XeElementSpec instance is created with a late binding XePName. When this XeElementSpec is asked to 
create an XeElement in a typespace , it will qualify the typeName in the namespace mapped to 'xsd' 
in that typespace. Similarly the third XeElementSpec is created with a late binding XePName with the 
'd' prefix.
"
! !

!XeSprayClassFactory class methodsFor:'constants'!

defaultTargetNamespace
	^XeNamespace sprayClass
!

preferedPrefix

	^'d'
! !

!XeSprayClassFactory class methodsFor:'must strip'!

example
        <resource: #example>

        | typespace |
        typespace := XeTypespace defaultSoapEnc2001.

        "Content from Class Factory ... requires spray sample BigDog package"
        ^(XeQName sprayClass: (self environment at: #PurchaseOrder)) 
                xeContentWith: typespace soapEncStyle
! !

!XeSprayClassFactory methodsFor:'helpers-private'!

attributesDefinedFor: actualClass
	"Answer the collection of <XeAttributeSpecs> defined by the class"

	^actualClass sprayAttributes
!

elementsDefinedFor: actualClass
	"Answer the collection of <XeElementSpecs> defined by the class"

	^actualClass sprayElements
! !

!XeSprayClassFactory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSprayClassFactory.st,v 1.6 2010/03/15 17:53:12 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSprayClassFactory.st,v 1.6 2010/03/15 17:53:12 cg Exp $'
! !
