"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeTextConverter
	instanceVariableNames:'asStringBlock fromStringBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeTextConverter comment:'I am a pluggable converter. I am based on the CS2 Soap TextConverter class.

I perform no validation, I only convert to and from text.

Instance Variables:
	asStringBlock 	<MonadicBlock>
	fromStringBlock	<MonadicBlock>




'
!


!XeTextConverter class methodsFor:'instance creation'!

base64Binary
        Smalltalk isSmalltalkX ifTrue:[
            ^self  new
                    asStringBlock: [ :byteArray | Base64Coder encodingOf: byteArray];
                    fromStringBlock: [ :text | Base64Coder decodingOf:text ];
                    yourself.
        ] ifFalse:[
            Base64Encoder. "force a prerequiste outside the block. (problem on 4.00/Win95)"
            ^self  new
                    asStringBlock: [ :byteArray | Base64Encoder convert: byteArray];
                    fromStringBlock: [ :text | Base64Decoder convert: text ];
                    yourself.
        ]
!

boolean
	^self new
		fromStringBlock: [:text | Boolean fromSchemaString: text ];
		yourself.
!

date
	^(self new)
		fromStringBlock: [:text | XeXsdDate fromString: text ];
		yourself
!

dateTime
	^(self new)
		fromStringBlock: [:text | XeXsdDateTime fromString: text ];
		yourself
!

decimal
        #cg.

        Smalltalk isSmalltalkX ifTrue:[
            ^self new
                    fromStringBlock: [:text | FixedPoint fromSchemaString: text ];
                    asStringBlock: [ :s | s = s truncated ifTrue:[
                                            s truncated printString
                                          ] ifFalse:[
                                            s printString 
                                          ]
                                   ];
                    yourself.
        ].

        ^self new
                fromStringBlock: [:text | ScaledDecimal fromSchemaString: text ];
                asStringBlock: [ :s | s displayString];
                yourself.
!

double
	^self new
		fromStringBlock: [:text | Float fromSchemaString: text ];
		asStringBlock: [ :float | float schemaDisplayString];
		yourself.

!

hexBinary
	^self  new
		asStringBlock: [ :byteArray | byteArray printSchemaHexString];
		fromStringBlock: [ :text | ByteArray fromHexString: text ];
		yourself.

!

integer
	^self new
		fromStringBlock: [:text | Integer fromSchemaString: text];
		yourself.
!

new
        ^self basicNew initialize
!

number
	^self  new
		fromStringBlock: [:text | Number fromString: text trimBlanks];
		yourself.
!

stb
        Smalltalk isSmalltalkX ifTrue:[
            ^self  new
                    asStringBlock: [ :object | Base64Coder encodingOf:(object binaryStoreBytes)];
                    fromStringBlock: [ :text | Object fromBinaryStoreBytes:(Base64Coder decodingOf:text) ];
                    yourself.
        ].
        ^self  new
                asStringBlock: [ :object | Base64Encoder convert: object binaryStoreBytes];
                fromStringBlock: [ :text | Object fromBinaryStoreBytes: (Base64Decoder convert: text )];
                yourself.
!

string
	^self new
		fromStringBlock: [:text | text];
		yourself

!

time
	^(self new)
		fromStringBlock: [:text | XeXsdTime fromString: text ];
		yourself
! !

!XeTextConverter class methodsFor:'must strip'!

example
        <resource: #example>

        | converter byteArray base64 |
        converter := XeTextConverter base64Binary.
        byteArray := '123456abc,.<>$' asByteArray.
        "convert to text"
        base64 := converter asString: byteArray.
        "convert from text"
        ^converter fromString: base64

        "
         self example
        "
! !

!XeTextConverter methodsFor:'accessing'!

asStringBlock: aBlock
	asStringBlock := aBlock
!

fromStringBlock: aBlock
	fromStringBlock := aBlock
! !

!XeTextConverter methodsFor:'constants-private'!

defaultAsStringBlock
        #cg.

        "/ ^Message selector: #schemaDisplayString
        ^ [:arg | arg schemaDisplayString ]
! !

!XeTextConverter methodsFor:'initializing-private'!

initialize
	super initialize.
	asStringBlock := self defaultAsStringBlock
! !

!XeTextConverter methodsFor:'operations'!

asString: anObject
        ^asStringBlock value: anObject
!

fromString: aString 
	^fromStringBlock value: aString
! !

!XeTextConverter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeTextConverter.st,v 1.8 2010/06/14 11:00:07 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeTextConverter.st,v 1.8 2010/06/14 11:00:07 cg Exp $'
! !
