"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeTypespace
	instanceVariableNames:'schemaDict prefixMap xsiNamespace options encStyles
		defaultEncStyle'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

!XeTypespace class methodsFor:'documentation'!

documentation
"
I am the context in which encoding and decoding takes place. 

==========
To view a class diagram image evaluate showExampleImage

==========
-I am multiple process safe because I do not (should not!!) change state once created and configured    

==========

I am similar to a top level schema. I contain;
-a SchemaDictionary which associates namespaces to the XeSchemas targeting that namespace. 
-a PrefixMap which associates prefix strings with namespace, 
and is used by instances of XePName to 'late-qualify' their localNames. 
-An encodingStyleMap which associates namespaces to instances of XeEncodingStyle which are used to 
locate the correct encodingStyle instance for a WSDL encodingStyle uri. 


An XeElement or XeContent instance is created from a typeName by using the typespace to locate the 
XeSchema targeting that namespace. The XeElement or XeContent is always created with an XeEncodingStyle. 
The same typeName will necessarily create different content in different XeEncodingStyles 
(in effect the XeSoapEncodingStyle class will ignore or change xsd:schema definitions to be 
compatible with soap-enc section 5 rules).

The default XeTypespaces include a number of default XeSchemas and prefixes. 
These include XeSchemas for the xsd namespace, soap-enc and a couple of default namespaces for 
defining classes. A XeTypespaceSchema instance is used to create a XeTypespace, 
and the XeTypespaceSchema can be reconfigured with alternate namespaces/prefixes 
and it can import external schemas.
"
! !

!XeTypespace class methodsFor:'instance creation'!

defaultLiteral2001
	| typespace |
	typespace := self defaultSoapEnc2001.
	"This modifies the classFactory to be elementFormDefault='qualified' (which is equivelent to the XMLSchema attribute switch). 
	-This tells the factory that local elements should be qualified. 
	-Not used in soap-enc, but common for DOTNET services"
	(typespace schemaTargeting: XeNamespace sprayClass) isElementFormQualified: true.
	(typespace schemaTargeting: XeNamespace spraySimpleClass) isElementFormQualified: true.
	^typespace
!

defaultSoapEnc1999
	| typespace |
	typespace := self new.
	self addDefaultSchemasTo: typespace.
	self addCommonPrefixesTo: typespace.
	self add1999PrefixesTo: typespace.
	typespace xsiNamespace: XeNamespace schema1999i.
	^typespace
!

defaultSoapEnc2001
	| typespace |
	typespace := self new.
	self addDefaultSchemasTo: typespace.
	self addCommonPrefixesTo: typespace.
	self add2001PrefixesTo: typespace.
	typespace xsiNamespace: XeNamespace schema2001i.
	^typespace

!

new
        ^self basicNew initialize
! !

!XeTypespace class methodsFor:'helpers-private'!

add1999PrefixesTo: aTypespace
	aTypespace
		addPrefix: 'xsd'  namespace: XeNamespace schema1999

	
!

add2001PrefixesTo: aTypespace
	aTypespace
		addPrefix: 'xsd'  namespace: XeNamespace schema2001

	
!

addCommonPrefixesTo: aTypespace
	aTypespace
		addPrefix: 'd' namespace: XeNamespace sprayClass;
		addPrefix: 'dolstb' namespace: XeNamespace spraySimple;
		addPrefix: 'soap' namespace: XeNamespace soapEnc;
		addPrefix: 'soapE' namespace: #'http://tempuri.org/schemas'
!

addDefaultSchemasTo: aTypespace
	aTypespace
		addSchemaFactory: XeSprayClassFactory at: XeNamespace sprayClass;
		addSchemaFactory: XeSimpleClassFactory at: XeNamespace spraySimpleClass;
		addSchemaFactory: XeSpraySimpleFactory at: XeNamespace spraySimple;
		addSchemaFactory: XeXSDBase1999 at: XeNamespace schema1999;
		addSchemaFactory: XeXSDBase1999 at: #'http://www.w3.org/2000/10/XMLSchema';
		addSchemaFactory: XeXSDBase2001 at: XeNamespace schema2001;
		addSchemaFactory: XeSoapEncFactory at: XeNamespace soapEnc;
		addSchemaFactory: XeSoapEncFactory at: #'http://schemas.xmlsoap.org/wsdl/soap/';
		addSchemaFactory: XeSoapEncExtFactory at: #'http://tempuri.org/schemas'
! !

!XeTypespace methodsFor:'accessing'!

anyTypeName
	^self xsdSchema factory anyTypeName
!

defaultEncodingStyle
	"Note: the need for a default encoding style will soon be deprecated"

	defaultEncStyle isNil ifTrue: [defaultEncStyle := self soapEncStyle].
	^defaultEncStyle
!

encStyleFor: aUriSymbol
	"Answer an instance of XeEncodingStyle. The uri to encodingStyle is defined by the WSDL spec"

	^encStyles at: aUriSymbol
		ifAbsent: 
			[self error: 'EncodingStyle not supported in this typespace: ' , aUriSymbol displayString]
!

literalStyle
	"Answer an instance of XeEncodingStyle (by default an instance of XeLiteralStyle). Alternate subclasses of XeEncodingStyle can be used to customize encoding. All content that is created in Spray in the literal style will access the encodingStyle through this method (Note: the choice of #'' as the uri for literalStyle is the choice of the WSDL spec writers.)"

	^self encStyleFor: #''
!

namespacePrefixed: aPrefixString
	"Answer a NSSymbol"

	^self prefixMap at: aPrefixString
		ifAbsent: 
			[self error: 'The typespace does not contain the prefix: ' , aPrefixString displayString]
!

options
	"Answer a <XeTypespaceOptions>"

	^options
!

schemaPrefixed: aPrefixString
	"Answer the XeSchema with a targetNamespace mapped to the prefix"

	^self schemaTargeting: (self prefixMap at: aPrefixString)
!

schemaTargeting: aNSSymbol
        "Answer the XeSchema with the targetNamespace aSymbol.
        Generate a resumable exception if it is not found, 
                -If resumed a schema will be answered which creates XeGuessContent"

        ^self schemaTargeting: aNSSymbol
                ifAbsent:[   
self breakPoint:#cg.
                    XeSchemaNotFound signal: 'Can not find the schema targeting: ' , aNSSymbol displayString.
                    XeSchema guessSchema typespace: self
                ]
!

schemaTargeting: aNSSymbol ifAbsent: operation
	^self schemaDict at: aNSSymbol ifAbsent: operation
!

soapEncStyle
	"Answer an instance of XeEncodingStyle (by default an instance of XeSoapEncodingStyle). Alternate subclasses of XeEncodingStyle can be used to customize encoding. All content that is created in Spray in the soap-enc style will access the encodingStyle through this method"

	^self encStyleFor: XeNamespace soapEnc

!

xsdNamespace
	^XeNamespace xsdForXSI: self xsiNamespace
!

xsdSchema
	#todo. "probably should cache this ... would need to watch changes to the prefixMap"

	^self schemaPrefixed: 'xsd'
!

xsiNamespace
	^xsiNamespace
!

xsiNamespace: aNSSymbol
	xsiNamespace := aNSSymbol
! !

!XeTypespace methodsFor:'accessing-private'!

options: aXeTypespaceOptions
	options := aXeTypespaceOptions
!

prefixMap
	^prefixMap
!

schemaDict
	^schemaDict
! !

!XeTypespace methodsFor:'configuration'!

addEncodingStyle: anEncodingStyleClass for: aUriSymbol
	self assert: [aUriSymbol isKindOf: Symbol].
	encStyles at: aUriSymbol put: (anEncodingStyleClass new typespace: self)
!

addPrefix: aString namespace: aNSSymbol
	self assert: [(self prefixMap includesKey: aString) not].
	self prefixMap at: aString put: aNSSymbol
!

addSchema: aSchema
	"Add the <XeSchema> at it's targetNamespace. Answer the XeSchema"

	aSchema typespace: self.
	^self schemaDict at: aSchema targetNS put: aSchema
!

addSchemaFactory: aFactoryClass at: aNSSymbol
	"Create a <XeSchema> for the factory, and add it at it's targetNamespace"

	^self addSchema: (XeSchema onFactory: aFactoryClass targetNS: aNSSymbol)
!

addSchemaNode: aXeNode
	"Create a <XeSchema> for the <XeXSDSchema> which is created by parsing the node. 
	Add it to the receiver at  it's targetNamespace"

	| schema |
	schema := XeSchema forNode: aXeNode.
	self addSchema: schema.
	^schema
!

addSchemaString: xmlString
	"Create a <XeSchema> for the <XeXSDSchema> which is created by parsing the xml string. 
	Add it to the receiver at  it's targetNamespace"

	^self addSchemaNode: (XeEnvironment rootNodeFor: xmlString)
!

addSchemaString: xmlString prefix: aPrefixString
	"Create a <XeSchema> for the <XeXSDSchema> which is created by parsing the xml string. 
	-Add it to the receiver at  it's targetNamespace.
	-Add a prefix for the targetNamespace"

	| schema |
	schema := self addSchemaString: xmlString.
	self addPrefix: aPrefixString namespace: schema targetNS.
	^schema
!

addSprayClassFactoryAt: aNSSymbol
	"Create a <XeSchema> for a sprayClass factory, and add it to the receiver at the targetNamespace"

	^self addSchema: (XeSchema onFactory: XeSprayClassFactory targetNS: aNSSymbol)
!

mapTypeName: aTypeQName to: aClass
	"Configure the typespace so that instances of the class will encode/decode to the type named  aTypeQName.  
	The schema targeting the type's namespace must use a <XeLocalFactoryAbstract>"

	self assert: [aClass isKindOf: Class].
	(self schemaTargeting: aTypeQName namespace) mapTypeName: aTypeQName to: aClass
!

overrideAll: aCollOfLocalNames to: aContentFactoryClass at: aNSSymbol
	"Divert the local typeNames in the aNSSymbol namespace to create content using aContentFactoryClass 
	Answer the ClassFactory "

	| schema classFactory |
	self assert: [aContentFactoryClass isKindOf: Class].
	schema := self schemaTargeting: aNSSymbol.
	classFactory := aContentFactoryClass new schema: schema.
	schema overrideAll: aCollOfLocalNames to: classFactory.
	^classFactory
!

overrideAll: aCollOfLocalNames toSprayClassAt: aNSSymbol
	"Divert the local typeNames in the namespace to create content using a XeSprayClassFactory .
	Answer the XeSprayClassFactory "

	^self 
		overrideAll: aCollOfLocalNames
		to: XeSprayClassFactory
		at: aNSSymbol
!

replacePrefix: aString namespace: aNSSymbol
	self prefixMap at: aString put: aNSSymbol
!

replaceSchemaNamespaceOf: aPrefixString to: aNSSymbol
	| replaceNS replaceS |
	self assert: [aNSSymbol isKindOf: Symbol].
	replaceNS := prefixMap at: aPrefixString
				ifAbsent: [self error: 'Could not find the prefix to replace'].
	replaceS := schemaDict removeKey: replaceNS
				ifAbsent: [self error: 'Could not find the schema to change its targetNamespace'].
	replaceS targetNS: aNSSymbol.
	prefixMap at: aPrefixString put: aNSSymbol.
	schemaDict at: aNSSymbol put: replaceS
!

setSprayClassPrefixTo: aNSSymbol
	"Modify a default typespace so that the the XePNames that follow the convention of using 'd' to identify classes, late-qualify into the aNSSymbol namespace.
	-Use this method when you have an existing schema targeting aNSSymbol (typically a parsed XMLSchema from a WSDL) and you want to override certain types to specific classes using the #overrideAll:toSprayClassAt: method.
	-This is the best strategy for DOTNET clients"

	self replacePrefix: 'd' namespace: aNSSymbol
!

setSprayClassTo: aNSSymbol
	"Modify a default typespace so that the default <XeSprayClassFactory> has its targetNamespace changed to aNSSymbol.
	-This method will change both the namespace of the prefix, and ensure that there is a  <XeSprayClassFactory> targeting that namespace. (this will replace any existing schema targeting aNSSymbol).
	-This is *not* suitable for a typical DOTNET client  (because the schema contains alot of junk that you would need to implement). For a better DOTNET approach see #setSprayClassPrefixTo:
	-This is suitable when you are implementing your own service and want to use a namespace other than the default dolphinharbor namespace"

	self replaceSchemaNamespaceOf: 'd' to: aNSSymbol
! !

!XeTypespace methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
    "Answer a <XeContent>.
    -If the XeSchemaNotFound exception is resumed, a <XeGuessContent> instance will be answered"

    ^(self schemaTargeting: aTypeQName namespace
        ifAbsent:[
            XeSchemaNotFound signal: ('Can not locate schema to create content of type: "',aTypeQName displayString,'"').
            XeSchema guessSchema typespace: self
        ]
     ) 
        xeContentOf: aTypeQName encStyle: anEncodingStyle

    "Modified: / 19-03-2012 / 16:57:25 / cg"
    "Modified (format): / 20-03-2012 / 13:42:51 / cg"
! !

!XeTypespace methodsFor:'helpers'!

newNode: aString
        "Answer a XeNode, with the correct xsi namespace declaration added."

        | doc root |
        doc := YAXO::XeDoc new.
        root := doc addElement: aString.
        self putXSINamespaceOn: root.
        ^root
!

putXSDDefaultNSOn: aXeNode
	"Make the schema namespace the default namespace for the node. The typespace stores a
	  default for the xsi namespace (which is impossible to determine otherwise), and uses the
	  matching xsd namespace"

	aXeNode defaultNS: self xsdNamespace.
	^aXeNode
!

putXSINamespaceOn: aXeNode
	self assert: [self xsiNamespace notNil].
	aXeNode addNSPrefix: 'xsi' to: self xsiNamespace.
	^aXeNode
! !

!XeTypespace methodsFor:'initializing-private'!

initialize
        super initialize.
        schemaDict := IdentityDictionary new.
        "/ cg: changed from LookupTable
        prefixMap := Dictionary new.
        encStyles := IdentityDictionary new.
        self setDefaultEncodingStyles.
        options := XeTypespaceOptions defaultSoapEnc
!

setDefaultEncodingStyles
	self addEncodingStyle: XeSoapEncodingStyle for: XeNamespace soapEnc.
	self addEncodingStyle: XeLiteralStyle for: #''
! !

!XeTypespace methodsFor:'modes'!

makeLiteralDefaultStyle
	"Note: the need for a default encoding style will soon be deprecated"

	defaultEncStyle := self literalStyle
!

makeSoapEncDefaultStyle
	"Note: the need for a default encoding style will soon be deprecated"

	defaultEncStyle := self soapEncStyle
! !

!XeTypespace methodsFor:'operations'!

qualify: aName 
	^aName asXeName qualifiedNameIn: self
! !

!XeTypespace methodsFor:'particle creation'!

xeAttributeDeclared:aXeQName encStyle:anEncodingStyle 
    "Answer an XeAttribute (not legal for use in soap-enc)"
    
    ^(self schemaTargeting:aXeQName namespace) 
        xeAttributeDeclared:aXeQName encStyle:anEncodingStyle

    "Modified (format): / 20-03-2012 / 13:41:00 / cg"
!

xeElementDeclared: aXeQName encStyle: anEncodingStyle
        "Answer a <XeElement>. See XeSchema>>elementDeclared:."

        ^(self schemaTargeting: aXeQName namespace) 
            xeElementDeclared: aXeQName encStyle: anEncodingStyle

    "Modified (format): / 20-03-2012 / 13:41:08 / cg"
! !

!XeTypespace methodsFor:'testing'!

hasSchemaTargeting: aNSSymbol
	^self schemaDict includesKey: aNSSymbol
! !

!XeTypespace class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeTypespace.st,v 1.14 2012/03/20 13:05:08 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeTypespace.st,v 1.14 2012/03/20 13:05:08 cg Exp $'
! !
