"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeTypespaceOptions
	instanceVariableNames:'flags'
	classVariableNames:'AllowKeyedArrays IsConvertAllValues ReadUTF8 XSITypeArrays'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!


!XeTypespaceOptions class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	AllowKeyedArrays := 1.
	ReadUTF8 := 2.
	XSITypeArrays := 4.
	IsConvertAllValues := 8
! !

!XeTypespaceOptions class methodsFor:'instance creation'!

defaultSoapEnc
	^self new
		beReadUTF8;
		beAllowKeyedArrays;
		yourself

		
! !

!XeTypespaceOptions methodsFor:'accessing-private'!

flags
	flags isNil ifTrue: [flags := self defaultFlags].
	^flags
!

flags: anInteger
	flags := anInteger
! !

!XeTypespaceOptions methodsFor:'modes'!

beAllowKeyedArrays
	flags := self flags mask: AllowKeyedArrays set: true
!

beConvertAllValues
	"This is only needed for the correct decoding of the float values NaN INF and -INF.
	-This mode will decode those values as instances of XeAnyValue"

	flags := self flags mask: IsConvertAllValues set: true
!

beDisallowKeyedArrays
	flags := self flags mask: AllowKeyedArrays set: false
!

beDontXSITypeArrays
	flags := self flags mask: XSITypeArrays  set: false
!

beNotConvertAllValues
	"This is only needed for the correct decoding of the float values NaN INF and -INF.
	-This mode will fault those values"

	flags := self flags mask: IsConvertAllValues set: false
!

beNotReadUTF8
	flags := self flags mask: ReadUTF8 set: false
!

beReadUTF8
        (OperatingSystem isMSWINDOWSlike
        and:[OperatingSystem isMSWINDOWSNTlike not ])
        "/ OSVERSIONINFO getCurrent isWin95 
                ifFalse: [flags := self flags mask: ReadUTF8 set: true]
!

beXSITypeArrays
	flags := self flags mask: XSITypeArrays  set: true
!

defaultFlags
	^0
! !

!XeTypespaceOptions methodsFor:'testing'!

allowKeyedArrays
	"Answer a Boolean.
	Sparse/partial Soap-enc Arrays decode into keyed collections. 
	The application can use this option to disable reading of Sparse/partial. 
	 -if this answers false, a fault will be generated on receiving a Sparse/partial array"

	^self flags anyMask: AllowKeyedArrays
!

isCacheContent
	"Answer a Boolean. By default all instances of XeContent are cached and reused based on their typeName and encoding style. For typical use in a typespace that has consistent encodingStyle policy, this is safe. (If you are experimenting with alternate encodingStyles you may want to answer false"

	^true
!

isConvertAllValues
	"Answer a Boolean. If a XeValueRangeError is signaled, should the exception be resumed and an instance of XeAnyValue answered?"

	^self flags anyMask: IsConvertAllValues
!

isReadUTF8
	"Answer a Boolean. Should the text of a node be extracted as UTF-8?"

	^self flags anyMask: ReadUTF8
!

isXSITypeArrays
	"Answer a Boolean. Should an array be inline typed? This can cause interop problems if the type is derived from soap:Array (vs being soap:Array)"

	^self flags anyMask: XSITypeArrays
! !

!XeTypespaceOptions class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeTypespaceOptions.st,v 1.7 2008/09/03 08:53:43 cg Exp $'
! !

XeTypespaceOptions initialize!
