"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: YAXO }"

Object subclass:#XeXMLNodeAbstract
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Nodes'
!

XeXMLNodeAbstract comment:'I am part of an xml tree.

-I am not multiple process safe. I must be used in a single process'
!


!XeXMLNodeAbstract class methodsFor:'initialization'!

postAutoload
    self projectDefinitionClass loadExtensions
! !

!XeXMLNodeAbstract class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment xeIconNamed: 'xml.ico'
    ]
! !

!XeXMLNodeAbstract class methodsFor:'must strip'!

exampleAccessing
        <resource: #example>

        | rootNode paramNode|
        rootNode := XeXMLNodeAbstract exampleDocument documentElement.
        "Accessing-qualifed"
        rootNode qName.
        "Accessing-namespaces"
        rootNode namespaceFor: 'xsd' .
        paramNode := rootNode firstChild firstChild firstChild.
        "Accessing-qualifed attribute values"
        paramNode qValueForXSIType.
        "View it (note creates temporary file)"
        rootNode browserView
!

exampleBuilding
        <resource: #example>

        | doc root child |
        doc := XeDoc new.
        root := doc addElement: 'my:rootNode'.
        root addNSPrefix: 'my' to: #'urn:myNamespace'.
        doc xmlFormatted.       "Inspect It"
        child := root addElement: 'local'.
        child addTextNode: 'Hi!!'.
        doc xmlFormatted        "Inspect It"
!

exampleDocument
        <resource: #example>

        ^(self environment at: #XeYAXOParser) 
                docNodeFor: '<SOAP-ENV:Envelope
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<ns1:echoString xmlns:ns1="http://soapinterop.org/xsd">
<param0 xsi:type="xsd:string">abc</param0>
</ns1:echoString>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
!

exampleQualifiedBuilding
        <resource: #example>

        | doc root child |
        doc := XeDoc new.
        root := doc addElementNamed: (XeQName 
                                                prefix: 'my'
                                                local: 'rootNode'
                                                ns: #'urn:myNamespace')
                                rootDeclare: false.
        doc xmlFormatted.       "Inspect It"
        "declaration is reused, prefix is ignored"
        child := root addElementNamed: (XeQName 
                                                prefix: 'meAgain'
                                                local: 'childNode'
                                                ns: #'urn:myNamespace')
                                rootDeclare: false.
        child addTextNode: 'Hi!!'.
        doc xmlFormatted        "Inspect It"
! !

!XeXMLNodeAbstract methodsFor:'helpers-private'!

newStream
	^String writeStream
! !

!XeXMLNodeAbstract methodsFor:'inspecting'!

inspectorExtraAttributes
    "extra (pseudo instvar) entries to be shown in an inspector."

    |d|

    d := Dictionary new
        declareAllNewFrom:(super inspectorExtraAttributes ? #());
        add:'-xml' -> [ self xmlFormatted ];
        yourself.
    ^ d
! !

!XeXMLNodeAbstract methodsFor:'printing'!

xml
	"Answer aString, the xmlString"

	^self xmlFormatting: false
!

xmlFormatted
	"Answer aString, the xmlString"

	^self xmlFormatting: true
! !

!XeXMLNodeAbstract methodsFor:'printing-private'!

printXMLOn: aStream cr: aBool
	^self subclassResponsibility
!

xmlFormatting: aBool
	"Private - Answer aString, the xmlString"

	| stream |
	stream := self newStream.
	self printXMLOn: stream cr: aBool.
	^stream contents
! !

!XeXMLNodeAbstract methodsFor:'testing'!

isDocument
	^false
!

isElementNode
	^false
!

isTextNode
	^false
! !

!XeXMLNodeAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXMLNodeAbstract.st,v 1.13 2010/12/22 13:16:05 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXMLNodeAbstract.st,v 1.13 2010/12/22 13:16:05 stefan Exp $'
! !
