"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentFactory subclass:#XeXSDBaseFactory
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeXSDBaseFactory class instanceVariableNames:'textConverters derivationData'

"
 No other class instance variables are inherited by this class.
"
!

XeXSDBaseFactory comment:'I create XeComponents for the schema of schemas namespace #''http://www.w3.org/2001/XMLSchema''

The schema spec at http://www.w3.org/TR/xmlschema-2/ defines the dataTypes created by this factory. While SOAP1.1 specifies the use of the 1999 schema, common practice is to ignore this and use the 2001 schema

A number of the types are "primitive" to Spray, and are implemented using XeSimpleContent and a XeTextConverter instance to convert to/from text. The remaining types are "derived" and are implemented using XeRestrictedContent, which contains a base XeSimpleContent and a number of XeSchemaRestriction.




'
!


!XeXSDBaseFactory class methodsFor:'accessing-private'!

derivationData
	^derivationData
!

derivationDataFor: aString ifAbsent: operation
	^self derivationData
		at: aString 
		ifAbsent: operation
!

preferedPrefix
	^'xsd'
!

textConverterFor: aString ifAbsent: operation
	^self textConverters 
		at: aString 
		ifAbsent: operation
!

textConverters
	^textConverters
! !

!XeXSDBaseFactory class methodsFor:'helpers'!

derivationByte
	^Array with: (self qualifyLocal: 'short')
		with: (Array with: (XeSchemaFacet maxInclusive: '127') with: (XeSchemaFacet minInclusive: '-128'))
!

derivationFloat
        "As I read the spec 1.4E-45, should not be a valid float. However it appears to be valid, and therefore
        I have modified my restrictions"

        ^Array with: (self qualifyLocal: 'double')
                with: (Array 
                                with: (XeSchemaFacet maxInclusive: ((2 raisedTo: 24) * (2 raisedTo: 104)) printString)
                                with: (XeSchemaFacet minInclusive: ((2 raisedTo: 24) * (2 raisedTo: 104) * -1) printString)
                                with: (XeSchemaFacet new block: [:each | each <= 0.0 or: [each > "##"(2.0 raisedTo: -150)]])
                                with: (XeSchemaFacet new block: [:each | each >= 0.0 or: [each < "##"((2.0 raisedTo: -150) * -1)]]))
!

derivationID
	^Array with: (self qualifyLocal: 'NCName') with: #()
!

derivationIDREF
	^Array with: (self qualifyLocal: 'NCName') with: #()
!

derivationInt
	^Array with: (self qualifyLocal: 'integer')
		with: (Array with: (XeSchemaFacet maxInclusive: '2147483647')
				with: (XeSchemaFacet minInclusive: '-2147483648'))
!

derivationLong
	^Array with: (self qualifyLocal: 'integer')
		with: (Array with: (XeSchemaFacet maxInclusive: '9223372036854775807')
				with: (XeSchemaFacet minInclusive: '-9223372036854775808'))
!

derivationNCName
	^Array with: (self qualifyLocal: 'name') with: (Array with: XeSchemaFacet ncNameString)
!

derivationNMToken
	"This is not accurate due to unicode issues, it will have false failures"

	^Array with: (self qualifyLocal: 'string')
		with: (Array 
				with: XeSchemaFacet normalizedString
				with: XeSchemaFacet token
				with: XeSchemaFacet nmToken)
!

derivationName
	^Array with: (self qualifyLocal: 'token') with: (Array with: XeSchemaFacet nameString)
!

derivationNegativeInteger
	^Array with: (self qualifyLocal: 'nonPositiveInteger')
		with: (Array with: (XeSchemaFacet maxInclusive: '-1'))
!

derivationNonNegativeInteger
	^Array with: (self qualifyLocal: 'integer') with: (Array with: (XeSchemaFacet minInclusive: '0'))
!

derivationNonPositiveInteger
	^Array with: (self qualifyLocal: 'integer') with: (Array with: (XeSchemaFacet maxInclusive: '0'))
!

derivationNormalizedString
	^Array with: (self qualifyLocal: 'string') with: (Array with: XeSchemaFacet normalizedString)
!

derivationPositiveInteger
	^Array with: (self qualifyLocal: 'nonNegativeInteger')
		with: (Array with: (XeSchemaFacet minInclusive: '1'))
!

derivationShort
	^Array with: (self qualifyLocal: 'int')
		with: (Array with: (XeSchemaFacet maxInclusive: '32767') with: (XeSchemaFacet minInclusive: '-32768'))
!

derivationToken
	^Array with: (self qualifyLocal: 'string')
		with: (Array with: XeSchemaFacet normalizedString with: XeSchemaFacet token)
!

derivationUnsignedByte
	^Array with: (self qualifyLocal: 'unsignedShort')
		with: (Array with: (XeSchemaFacet maxInclusive: '255') with: (XeSchemaFacet minInclusive: '0'))
!

derivationUnsignedInt
	^Array with: (self qualifyLocal: 'unsignedLong')
		with: (Array with: (XeSchemaFacet maxInclusive: '4294967295') with: (XeSchemaFacet minInclusive: '0'))
!

derivationUnsignedLong
	^Array with: (self qualifyLocal: 'nonNegativeInteger')
		with: (Array with: (XeSchemaFacet maxInclusive: '18446744073709551615')
				with: (XeSchemaFacet minInclusive: '0'))
!

derivationUnsignedShort
	^Array with: (self qualifyLocal: 'unsignedInt')
		with: (Array with: (XeSchemaFacet maxInclusive: '65535') with: (XeSchemaFacet minInclusive: '0'))
! !

!XeXSDBaseFactory class methodsFor:'helpers-private'!

commonDerivationData
        | table |
        "/ cg: changed from LookupTable
        table := Dictionary new.
        table
                at: 'float' put: self derivationFloat;
                at: 'int' put: self derivationInt;
                at: 'long' put: self derivationLong;
                at: 'short' put: self derivationShort;
                at: 'nonPositiveInteger' put: self derivationNonPositiveInteger;
                at: 'nonNegativeInteger' put: self derivationNonNegativeInteger;
                at: 'positiveInteger' put: self derivationPositiveInteger;
                at: 'negativeInteger' put: self derivationNegativeInteger;
                at: 'byte' put: self derivationByte;
                at: 'unsignedByte' put: self derivationUnsignedByte;
                at: 'unsignedInt' put: self derivationUnsignedInt;
                at: 'unsignedLong' put: self derivationUnsignedLong;
                at: 'unsignedShort' put: self derivationUnsignedShort;
                at: 'normalizedString' put: self derivationNormalizedString;
                at: 'token' put: self derivationToken;
                at: 'NMTOKEN' put: self derivationNMToken;
                at: 'name' put: self derivationName;
                at: 'NCName' put: self derivationNCName;
                at: 'ID' put: self derivationID;
                at: 'IDREF' put: self derivationIDREF;
                yourself.
        ^table
!

commonTextConverters
        | table |
        "/ cg: changed from LookupTable
        table := Dictionary new.
        table
                at: 'string' put: XeTextConverter string;
                at: 'number' put: XeTextConverter number;
                at: 'integer' put: XeTextConverter integer;
                at: 'double' put: XeTextConverter double;
                at: 'boolean' put: XeTextConverter boolean;
                at: 'anyURI' put: XeTextConverter string;
                at: 'decimal' put: XeTextConverter decimal;
                at: 'date' put: XeTextConverter date;
                at: 'time' put: XeTextConverter time;
                at: 'hexBinary' put: XeTextConverter hexBinary;
                yourself.
        ^table
! !

!XeXSDBaseFactory class methodsFor:'initializing-private'!

clear
	textConverters := nil.
	derivationData := nil
! !

!XeXSDBaseFactory methodsFor:'accessing-private'!

derivedTypeFor: aTypeQName encStyle: anEncodingStyle ifAbsent: operation
	| data |
	data := self class derivationDataFor: aTypeQName localName ifAbsent: [^operation value].
	^anEncodingStyle 
		simple: aTypeQName
		restricting: data first
		with: data second
!

primativeTypeFor: aTypeQName encStyle: anEncodingStyle ifAbsent: operation
	^anEncodingStyle simple: aTypeQName
		converter: (self class textConverterFor: aTypeQName localName ifAbsent: [^operation value])
! !

!XeXSDBaseFactory methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer a <XeContent>"

	(self isAnyTypeName: aTypeQName) ifTrue: [^anEncodingStyle anyContent].
	(self isSchemaName: aTypeQName) ifTrue: [^anEncodingStyle nodeContent].
	^self 
		primativeTypeFor: aTypeQName
		encStyle: anEncodingStyle
		ifAbsent: 
			[self 
				derivedTypeFor: aTypeQName
				encStyle: anEncodingStyle
				ifAbsent: [self error: 'Unsupported data type: ' , aTypeQName displayString]]
! !

!XeXSDBaseFactory methodsFor:'particle creation'!

xeElementDeclared: aQName encStyle: anEncodingStyle
	"Answer a <XeElement>"

	(self isSchemaName: aQName) 
		ifTrue: [^XeNodeContent nodeElementNamed: aQName encodingStyle: anEncodingStyle].
	^super xeElementDeclared: aQName encStyle: anEncodingStyle
! !

!XeXSDBaseFactory methodsFor:'testing-private'!

isAnyTypeName: aTypeQName
	^self subclassResponsibility
!

isSchemaName: aTypeQName
	^aTypeQName localName = 'schema'
! !

!XeXSDBaseFactory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDBaseFactory.st,v 1.6 2010/02/26 10:57:55 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDBaseFactory.st,v 1.6 2010/02/26 10:57:55 cg Exp $'
! !
