"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeXSDContentModel
	instanceVariableNames:'baseTypeName xsdDefinition'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeXSDContentModel comment:'I am a content model in an XMLSchema document. I provide access to the information in the XMLSchema.

(SW Note: These classes are not a full XMLSchema implementation, but only what is necessary for soap-enc)

Future versions of XeEncoding/Spray may parse XMLSchemas directly into the XeComponent classes. 

See XeXSDNode class comment'
!


!XeXSDContentModel class methodsFor:'instance creation'!

fromXSDDefinition: anXSDComplexType
	| contentModelName |
	contentModelName := anXSDComplexType contentModelName.
	contentModelName = 'complexContent'
		ifTrue: [^(XeXSDComplexContent classFor: anXSDComplexType) new setXSDDefinition: anXSDComplexType].
	contentModelName = 'simpleContent'
		ifTrue: [^(XeXSDSimpleContent classFor: anXSDComplexType) new setXSDDefinition: anXSDComplexType].

	^XeXSDComplexContent emptyContentFrom: anXSDComplexType
! !

!XeXSDContentModel methodsFor:'accessing-private'!

anyTypeName
	^self typespace anyTypeName
!

baseTypeName
	^baseTypeName
!

contentNodes
	^self subclassResponsibility
!

schema
	^self xsdSchema schema
!

setAnyBaseTypeName
	baseTypeName := self anyTypeName
!

setXSDDefinition: aXMLXSDComplexType 
	xsdDefinition := aXMLXSDComplexType.
	(baseTypeName := self extractBaseTypeName) isNil ifTrue: [self setAnyBaseTypeName].
!

typespace
	^self xsdDefinition typespace
!

xsdAttributes
	| xsdAttributes |
	xsdAttributes := OrderedCollection new.
	self xsdContent do: [ :each | xsdAttributes addAll: each xsdAttributes ].
	^xsdAttributes 
!

xsdContent
	^self contentNodes collect: [:each | self xsdContentFor: each]
!

xsdDefinition
	^xsdDefinition
!

xsdDefinition: aXMLXSDComplexType 
	xsdDefinition := aXMLXSDComplexType
!

xsdSchema
	^self xsdDefinition xsdSchema
! !

!XeXSDContentModel methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
	"Answer a <XeContent>"

	^self subclassResponsibility
! !

!XeXSDContentModel methodsFor:'exceptions-private'!

errorNotImplemented: aString
	self error: aString
! !

!XeXSDContentModel methodsFor:'helpers-private'!

extractBaseTypeName
	| node |
	node := self xsdDefinition contentNode firstChild.
	^node qValueFor: 'base'
! !

!XeXSDContentModel methodsFor:'operations'!

attributesWith: anEncodingStyle
	^self xsdAttributes collect: [ :each | each attributeWith: anEncodingStyle]
! !

!XeXSDContentModel methodsFor:'operations-private'!

xsdContentFor: aXeNode
	^XeXSDNode onNode: aXeNode xsdSchema: self xsdSchema
! !

!XeXSDContentModel methodsFor:'testing'!

hasAttributeContent
	^self xsdAttributes notEmpty
! !

!XeXSDContentModel methodsFor:'testing-private'!

isOldXSDSchema
	^self xsdSchema isOldXSDSchema
! !

!XeXSDContentModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDContentModel.st,v 1.4 2003/10/08 17:42:47 cg Exp $'
! !
