"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeXSDDeclaration subclass:#XeXSDElement
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!


!XeXSDElement class methodsFor:'accessing'!

nodeName
	^'element'
! !

!XeXSDElement methodsFor:'accessing'!

xsdAttributes

	^#()
!

xsdParticle
	^self
!

xsdParticles
	"Spec: A particle is a term in the grammar for element content, consisting of either an element declaration, a wildcard or a model group"

	^Array with: self
! !

!XeXSDElement methodsFor:'accessing-private'!

elementName
	self isGlobal ifTrue: [^self nameTQAttribute].
	^self isFormQualified ifTrue: [self nameTQAttribute] ifFalse: [XeQName null: self nameAttribute]
!

form
	^node getAttribute: 'form' ifNone: [self xsdSchema elementFormDefault]
! !

!XeXSDElement methodsFor:'operations-private'!

defaultElementWith: anEncodingStyle 
	^self elementTyped: self typespace anyTypeName with: anEncodingStyle 
!

elementTyped: aTypeQName with: anEncodingStyle
	| spec |
	spec := XeElementSpec named: self elementName typeName: aTypeQName.
	self setOccurrenceConstraintsInto: spec.
	self setNillableInto: spec.
	self setFormInto: spec.
	^spec xeParticleWith: anEncodingStyle
!

inlineElementWith: anEncodingStyle
	| content childNode spec |
	childNode := node firstChild.
	self assert: [#('complexType' 'simpleType') includes: childNode localName].
	content := (XeXSDDefinitionAbstract onNode: childNode xsdSchema: self xsdSchema) 
				xeContentWith: anEncodingStyle.
	spec := XeElementSpec anon: self elementName.
	self setOccurrenceConstraintsInto: spec.
	self setNillableInto: spec.
	self setFormInto: spec.
	^anEncodingStyle elementSpeced: spec anon: content
	"^(type elementNamed: name)
		isNillable: self isNillable;
		yourself"
!

refElementWith: anEncodingStyle
	"Spec: If ref is present, then all of <complexType>, <simpleType>, <key>, <keyref>, <unique>, nillable, default, fixed, form, block and type must be absent, i.e. only minOccurs, maxOccurs, id are allowed in addition to ref"

	^(self typespace xeElementDeclared: self refQAttribute encStyle: anEncodingStyle)
		beFormQualified;
		yourself
!

setNillableInto: aXeElementSpec 
	aXeElementSpec nillable: (self node getAttribute: 'nillable') = 'true'.

!

setOccurrenceConstraintsInto: aXeElementSpec 
	aXeElementSpec minOccurs: (self node getAttribute: 'minOccurs').
	aXeElementSpec maxOccurs: (self node getAttribute: 'maxOccurs').
        

!

typedElementWith: anEncodingStyle 
	^self elementTyped: self node qValueForType with: anEncodingStyle
! !

!XeXSDElement methodsFor:'particle creation'!

xeParticleWith: anEncodingStyle
	"Answer a <XeElement> "

	self isTyped ifTrue: [^self typedElementWith: anEncodingStyle].
	self isRef ifTrue: [^self refElementWith: anEncodingStyle].
	self isInline ifTrue: [^self inlineElementWith: anEncodingStyle].
	^self defaultElementWith: anEncodingStyle
! !

!XeXSDElement methodsFor:'testing'!

doesTypeMatchAny: aSet in: aTypespace
	"Sent by a <XeSprayClassFactory> to detect circular class references."

	^false
! !

!XeXSDElement methodsFor:'testing-private'!

isInline
	^self node childNodes notEmpty
!

isNillable
	"Private - default is false ... (this is ignored in soap-enc)"

	^(node getAttribute: 'nillable') = 'true'
! !

!XeXSDElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDElement.st,v 1.4 2003/10/08 17:45:46 cg Exp $'
! !
