"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeXSDNode
	instanceVariableNames:'node xsdSchema'
	classVariableNames:'ClassMap'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeXSDNode class instanceVariableNames:'cachedNodeNames'

"
 No other class instance variables are inherited by this class.
"
!

XeXSDNode comment:'I am a node in an XMLSchema document. I provide access to the information in the XMLSchema.

(SW Note: These classes are not a full XMLSchema implementation, but only what is necessary for soap-enc)

Future versions of XeEncoding/Spray may parse XMLSchemas directly into the XeComponent classes. 

==========
Instance Variables:
	node            <XeNode>
	xsdSchema       <XeXSDSchema>

==========
Use

These classes are used by an XeXSDSchema instance to create XeComponent instances. See XeXSDSchema class>>example'
!


!XeXSDNode class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	Smalltalk isSmalltalkX ifTrue:[
	    self == XeXSDNode ifTrue:[
		Smalltalk addDependent:self.  "/ to get class changes.
	    ]
	].
	self createClassMap
! !

!XeXSDNode class methodsFor:'instance creation'!

ifNode: aXeNode xsdSchema: aXeXSDSchema
	^self 
		onNode: aXeNode
		xsdSchema: aXeXSDSchema
		ifNone: []
!

onNode: aXeNode xsdSchema: aXeXSDSchema
	^self 
		onNode: aXeNode
		xsdSchema: aXeXSDSchema
		ifNone: [self halt. self error: 'Unexpected Schema node']
!

onNode: aXeNode xsdSchema: aXeXSDSchema ifNone: operation
	^(self nodeNames includes: aXeNode localName) 
		ifTrue: 
			[(self classFor: aXeNode) isNil ifTrue:[self halt:'debugHalt'].
			((self classFor: aXeNode) new)
				node: aXeNode;
				xsdSchema: aXeXSDSchema;
				yourself]
		ifFalse: [operation value]
! !

!XeXSDNode class methodsFor:'ST/X special'!

autoloadMyClasses
    "autoload workaround"

    Smalltalk allClassesDo:[:eachClass |
	(eachClass category = self category) ifTrue:[
	    eachClass isLoaded ifFalse:[
		eachClass autoload
	    ]
	]
    ].

    "
     self autoloadMyClasses
    "
!

flushCachedNodeNames
    cachedNodeNames := nil.
!

update:something with:someArgument from:changedObject
    changedObject == Smalltalk ifTrue:[
	something == #methodInClass ifTrue:[^ self].
	something == #lastTestRunResult ifTrue:[^ self].

	self withAllSubclassesDo:[:each | each flushCachedNodeNames].
	^ self.
    ].
    super update:something with:someArgument from:changedObject
! !

!XeXSDNode class methodsFor:'accessing-private'!

nodeName
	^nil
!

nodeNames
	|names|

	Smalltalk isSmalltalkX ifTrue:[
	    (names := cachedNodeNames) isNil ifTrue:[
		self autoloadMyClasses.
		names := cachedNodeNames := self withAllSubclasses collect:[:each | each nodeName] thenSelect:[:each | each notNil].
	    ].
	    ^ names
	].

	^ (self withAllSubclasses collect:[:each | each nodeName]) select:[:each | each notNil].
! !

!XeXSDNode class methodsFor:'helpers-private'!

classFor: aXeNode
        ^ClassMap 
            at: aXeNode localName
            ifAbsent: [self error: 'Unsupported schema language element: ' , aXeNode localName]
! !

!XeXSDNode class methodsFor:'initializing-private'!

uninitialize
	ClassMap := nil.
! !

!XeXSDNode class methodsFor:'operations-private'!

createClassMap
        "/ cg: changed from LookupTable
        ClassMap := Dictionary new.
        ClassMap 
                at: 'complexType' put: XeXSDComplexType;
                at: 'simpleType' put: XeXSDSimpleType;
                at: 'element' put: XeXSDElement;
                at: 'attribute' put: XeXSDAttribute;
                at: 'all' put: XeXSDAll;
                at: 'any' put: XeXSDAny;
                at: 'attributeGroup' put: XeXSDAttributeGroup;
                at: 'anyAttribute' put: XeXSDAnyAttribute;
                at: 'sequence' put: XeXSDSequence;
                at: 'choice' put: XeXSDChoice;
                at: 'group' put: XeXSDGroup.

        (ClassMap includes:nil) ifTrue:[
            self error:'missing class' mayProceed:true
        ].
! !

!XeXSDNode methodsFor:'accessing-private'!

nameAttribute
        

	^self node getAttribute: 'name'
!

nameTQAttribute
	"Private - answer the name attribute (which is qualified in the targetNamespace)"

	| local |
	(local := self nameAttribute) isNil ifTrue: [^nil].
	^XeQName local: local ns: self targetNamespace
!

node
	^node
!

node: aXeNode
	node := aXeNode
!

refQAttribute
	^self node qValueFor: 'ref'
!

schema
	^xsdSchema schema
!

schemaNode
	^xsdSchema schemaNode
!

targetNamespace
	^xsdSchema targetNamespace
!

typespace
	^xsdSchema typespace
!

xsdSchema
	"Private - Answer a <XeXSDSchema>"

	^xsdSchema
!

xsdSchema: aXeXSDSchema
	xsdSchema := aXeXSDSchema
! !

!XeXSDNode methodsFor:'exceptions-private'!

errorNotImplemented: aString
	self error: aString
! !

!XeXSDNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDNode.st,v 1.14 2010/02/26 10:58:04 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDNode.st,v 1.14 2010/02/26 10:58:04 cg Exp $'
! !

XeXSDNode initialize!
