"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeContentFactory subclass:#XeXSDSchema
	instanceVariableNames:'schemaNode targetNamespace namedNodes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeXSDSchema comment:'I am an xsd:schema. I am used to imported an xsd:schema document into a typespace. Once the schema is imported, XeContent can be created from my definitions, and XeParticles created from my declarations.

I am often used when creating a SprayWSDLService. A WSDL file can contain one or more schemas in its types section.



'
!


!XeXSDSchema class methodsFor:'instance creation'!

new
        ^self basicNew initialize
!

schemaNode: aXeNode
	^self new setSchemaNode: aXeNode
! !

!XeXSDSchema class methodsFor:'constants'!

preferedPrefix

	^''
! !

!XeXSDSchema class methodsFor:'must strip'!

example
        <resource: #example>

        "To import a schema"

        | typespace |
        typespace := XeTypespace defaultSoapEnc2001.
        typespace 
                addSchemaString: '<schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
targetNamespace="urn:myNamespace">
<complexType name="SOAPStruct">
<all>
  <element name="varString" type="xsd:string"/>
  <element name="varInt" type="xsd:int"/>
  <element name="varFloat" type="xsd:float"/>
</all>
</complexType>
</schema>'.
        ^('SOAPStruct' ns: #'urn:myNamespace') xeContentWith: typespace soapEncStyle
! !

!XeXSDSchema methodsFor:'accessing'!

attributeFormDefault
	^schemaNode getAttribute: 'attributeFormDefault' ifNone: ['unqualified']
!

elementFormDefault
	^schemaNode getAttribute: 'elementFormDefault' ifNone: ['unqualified']
!

schemaNode
	^schemaNode
!

schemaNode: aXeNode
	schemaNode := aXeNode
!

targetNamespace
	targetNamespace isNil ifTrue: [targetNamespace := schemaNode targetNamespace].
	^targetNamespace
! !

!XeXSDSchema methodsFor:'accessing-private'!

definitionFor: aTypeQName
        |typeNode|

        typeNode := self namedNodeFor: aTypeQName space: 'Type' ifAbsent:[nil].
        typeNode isNil ifTrue:[
            SOAP::XeSchemaNotFound raiseRequestErrorString: 'Unable to locate component for ' , aTypeQName displayString.
            ^ nil.
        ].
        ^XeXSDDefinitionAbstract onNode: typeNode xsdSchema: self

    "Modified: / 20-03-2012 / 13:38:16 / cg"
!

namedNodeFor: aXeQName space: aString
        ^ self namedNodeFor: aXeQName space: aString 
                ifAbsent: [
                    self error: 'Unable to locate component for ' , aXeQName displayString.
                ]

    "Modified: / 16-03-2012 / 20:50:02 / cg"
!

namedNodeFor: aXeQName space: aString ifAbsent: exceptionBlock
        ^namedNodes at: (Array with: aString with: aXeQName localName)
                ifAbsent: exceptionBlock

    "Created: / 16-03-2012 / 20:49:48 / cg"
!

namedNodeLocal: aLocalName space: aString
	"Private - UnitTesting only"

	^namedNodes at: (Array with: aString with: aLocalName )
!

setSchemaNode: aXeNode
	schemaNode := aXeNode.
	aXeNode childNodes do: [:each | self addSchemaChildNode: each]
!

xsdAttributeFor: aXeQName
	^(XeXSDAttribute onNode: (self namedNodeFor: aXeQName space: 'attribute') xsdSchema: self)
		beGlobal;
		yourself
!

xsdElementFor: aXeQName
	^(XeXSDElement onNode: (self namedNodeFor: aXeQName space: 'element') xsdSchema: self)
		beGlobal;
		yourself
! !

!XeXSDSchema methodsFor:'adding-private'!

addNamedNode: aXeNode name: symbolSpaceArray
	^namedNodes at: symbolSpaceArray put: aXeNode
!

addSchemaChildNode: aXeNode
        "Private - Spec 2.5  ... term symbol space to denote a collection of names, each of which is unique with respect to the others ... except that within a target namespace, simple type definitions and complex type definitions share a symbol space"

        | name symbolSpace typeElementNames firstChild|

        typeElementNames := #('complexType' 'simpleType').

        name := aXeNode getAttribute: 'name'.
        name notNil ifTrue: [ 
            symbolSpace := aXeNode localName.

            "/ 2012/03/16-cg: kludge: an element consisting of a type only is also added as Type
            "/ I do not understand how and why this is needed, but at least we can read the hermes WSDL.
            (symbolSpace = 'element' 
            and:[ aXeNode childNodes size == 1 
            and:[ firstChild := aXeNode childNodes first.
                  (typeElementNames includes: firstChild localName) ]]) ifTrue:[
                self breakPoint:#ab.
                firstChild setAttribute: 'name' value: name.
                self addNamedNode: firstChild name: (Array with: 'Type' with: name)
            ] ifFalse:[
                (typeElementNames includes: symbolSpace) ifTrue: [symbolSpace := 'Type'].
            ].

            self addNamedNode: aXeNode name: (Array with: symbolSpace with: name)
        ]

    "Modified: / 19-03-2012 / 16:07:36 / cg"
! !

!XeXSDSchema methodsFor:'content creation'!

xeContentOf: aTypeQName encStyle: anEncodingStyle
        "Answer a <XeContent>"

        | dfn content |

        dfn := self definitionFor: aTypeQName.
        dfn isNil ifTrue:[^ nil].

        content := dfn xeContentWith: anEncodingStyle.
        self assert: [content typeName = aTypeQName].
        ^content

    "Modified: / 16-03-2012 / 20:54:41 / cg"
! !

!XeXSDSchema methodsFor:'initializing-private'!

initialize
        super initialize.   
        "/ cg: changed from LookupTable
        namedNodes := Dictionary new
! !

!XeXSDSchema methodsFor:'particle creation'!

xeAttributeDeclared: aXeQName encStyle: anEncodingStyle
	"Answer an <XeAttribute>."

	^(self xsdAttributeFor: aXeQName) attributeWith: anEncodingStyle
!

xeElementDeclared: aXeQName encStyle: anEncodingStyle
	"Answer a <XeElement>.
	-Locate the element declaration, and use it to create the element. "

	^(self xsdElementFor: aXeQName) xeParticleWith: anEncodingStyle
! !

!XeXSDSchema methodsFor:'testing'!

isOldXSDSchema
	^(Array with: XeNamespace schema1999 with: XeNamespace schema200010) 
		includes: schemaNode namespace
! !

!XeXSDSchema class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDSchema.st,v 1.12 2012/03/20 21:04:44 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXSDSchema.st,v 1.12 2012/03/20 21:04:44 cg Exp $'
! !
