"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeXsdDateTimeAbstract subclass:#XeXsdDateTime
	instanceVariableNames:'date time'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!


!XeXsdDateTime class methodsFor:'instance creation'!

current
	"Answer a new instance of the receiver representing the current
	date and time *********in utc********
	drop the precision to what it prints"

	| instance |
	instance := self fromSYSTEMTIME: SYSTEMTIME nowUTC.
	instance time: (Time fromSeconds: (instance time asSeconds)).
	^instance
!

date: aDate time: aTime
	"Answer a new instance of the receiver based on aDate and aTime"

	^self 	date: aDate time: aTime offset: (Time fromSeconds: 0)
!

date: aDate time: aTime offset: aTime2 
	^self new
		date: aDate;
		time: aTime;
		offset: aTime2; 
		yourself
!

fromSYSTEMTIME: aSYSTEMTIME
	"Answer a new instance of the receiver representing the current
	date and time"

	^self 
		date: (Date fromSYSTEMTIME: aSYSTEMTIME) 
		time: (Time fromSYSTEMTIME: aSYSTEMTIME)
!

fromString: aString
        | stream date offset time |
        stream := aString readStream.
        date := self readDateSchemaFrom:stream. "/ Date readSchemaFrom: stream.
        self assert: [stream next = $T].
        time := self readTimeSchemaFrom:stream. "/ Time readSchemaFrom: stream.
        offset := self offsetArrayFrom: stream.
        ^self   new
                date: date;
                time: time;
                offset: offset second;
                isPositiveOffset: offset first;
                yourself
! !

!XeXsdDateTime class methodsFor:'must strip'!

example
        <resource: #example>

        ^XeXsdDateTime fromString: '1999-05-31T13:20:00-05:00'
! !

!XeXsdDateTime class methodsFor:'spray'!

sprayTypeName
        ^SOAP::XePName xsd: 'dateTime'
! !

!XeXsdDateTime methodsFor:'accessing-private'!

date
	^date
!

date: aDate
	date := aDate
!

normalized
        | normalTime normalDate millis t |
        self isNormal ifTrue: [^self].
        normalDate := self date.
        Smalltalk isSmalltalkX ifTrue:[
            t := Timestamp fromDate:date andTime:time.
            self isPositiveOffset ifTrue:[
                t := t subtractTime:offset asSeconds
            ] ifFalse:[
                t := t addSeconds:offset asSeconds
            ].
            normalDate := t asDate.
            normalTime := t asTime.
        ] ifFalse:[
            normalTime := self normalizedTime.
            millis := normalTime asMilliseconds.
            millis < 0 
                    ifTrue: 
                            [normalDate := normalDate subtractDays: 1.
                            normalTime := self time24Hours addTime: normalTime].
            millis >= 86400000 "##(24 * 60 * 60 * 1000)" 
                    ifTrue: 
                            [normalDate := normalDate addDays: 1.
                            normalTime := normalTime subtractTime: self time24Hours].
        ].
        ^self class 
                date: normalDate
                time: normalTime
                offset: (Time fromSeconds: 0)
!

normalizedTime
	^self isPositiveOffset 
		ifTrue: [self time subtractTime: self offset]
		ifFalse: [self time addTime: self offset]
!

time
	^time
!

time24Hours
        ^Time fromSeconds: 86400 "##(24 * 60 * 60)"
!

time: aTime
	time := aTime
! !

!XeXsdDateTime methodsFor:'comparing'!

< operand
	"This is not as simple as it first looks"

	^Error notYetImplemented
!

= aDateTime
	^(self species == aDateTime species)
		and: [self isSameUTCAs: aDateTime ]
!

hash
	| n |
	n := self normalized.
	^n date hash bitXor: n time hash
! !

!XeXsdDateTime methodsFor:'converting'!

asNative
	"Note: these classes are a stub. I am hoping that Dolphin5 will contain date/time/timestamp classes that would retain timezone info"

	^self timeStamp
!

timeStamp
        | normalized |
        #cg.

        normalized := self normalized.
        Smalltalk isSmalltalkX ifTrue:[
            ^ Timestamp 
                fromDate: normalized date
                andTime: normalized time
        ].
        ^ TimeStamp
                date: normalized date
                time: normalized time
! !

!XeXsdDateTime methodsFor:'printing-private'!

displayOffsetOn: stream 
        "This assumes that the timestamp is UTC"

        #cg.

        self date printUnboundedSchemaStringOn: stream. 
        stream nextPutAll: 'T'.
        Smalltalk isSmalltalkX ifTrue:[
            self time printOn: stream format: '%h:%m:%s'.
        ] ifFalse:[
            self time printOn: stream format: 'HH:mm:ss'.
        ].
        self isPositiveOffset
                ifTrue: [stream nextPutAll: '+']
                ifFalse: [stream nextPutAll: '-'].
        Smalltalk isSmalltalkX ifTrue:[
            self offset printOn: stream format: '%h:%m'.
        ] ifFalse:[
            self offset printOn: stream format: 'HH:mm'.
        ].
!

displayUTCOn: stream 
        "This assumes that the timestamp is UTC"

        #cg.

        self date printUnboundedSchemaStringOn: stream. 
        stream nextPutAll: 'T'.
        Smalltalk isSmalltalkX ifTrue:[
            self time printOn: stream format: '%h:%m:%s'.
        ] ifFalse:[
            self time printOn: stream format: 'HH:mm:ss'.
        ].
        stream nextPutAll: 'Z'.
! !

!XeXsdDateTime methodsFor:'testing-private'!

isSameNormalizedAs: aXMLDateAndTime 
	^self date = aXMLDateAndTime date and: [
		self time asSeconds = aXMLDateAndTime time asSeconds]
!

isSameUTCAs: aXMLDateAndTime 
	^self normalized isSameNormalizedAs: aXMLDateAndTime normalized
! !

!XeXsdDateTime class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXsdDateTime.st,v 1.8 2012/04/21 17:15:51 stefan Exp $'
! !
