"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Magnitude subclass:#XeXsdDateTimeAbstract
	instanceVariableNames:'offset isPositiveOffset'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeXsdDateTimeAbstract comment:'These classes are used to hold all the information in the XMLSchema data and time types.

They are a stub, and should be integrated into the base classes.

To access an instance from a base class (ie Date/Time/TimeStamp) send the conversion method #asNative 

Note: Spray will encode instance of Date/Time/TimeStamp as being in UTC'
!


!XeXsdDateTimeAbstract class methodsFor:'helpers-private'!

offsetArrayFrom: stream
        | isPositive |
        #cg.

        ^(stream atEnd or: [stream peek = $Z])
                ifTrue: [Array with: nil with: (TimeDuration "Time" fromSeconds: 0)]
                ifFalse: [   
                        isPositive := stream next = $+.
                        Array with: isPositive with: (TimeDuration "Time" readFrom: "fromString:" stream upToEnd)].
!

readDateSchemaFrom: stream 
        "Read the stream assuming the format as described in the 2001 XML Schema datatypes spec."

        | isNegative y m d |
        #swAdded.
        [(isNegative := stream peek = $-) ifTrue: [stream next]. 
        y := (stream upTo: $-) asNumber.
        m := (stream next: 2) asNumber.
        self assert: [stream next = $-].
        d :=  (stream next: 2) asNumber] 
            on: Error 
            do: [:e | 
                UserPreferences current soapErrorDebugging ifTrue:[ e reject ].
                self errorInvalidFormat
            ].
        "note: the day must be two digits, and the stream must be left in this position"

        ((1 to: 31) includes: d) ifFalse: [self errorInvalidFormat].
        ((1 to: 12) includes: m) ifFalse: [self errorInvalidFormat].
        ^ Date 
                newDay: d 
                monthIndex: m 
                year: (isNegative ifTrue: [y negated] ifFalse: [y])

    "Modified: / 12-07-2011 / 17:54:55 / cg"
!

readTimeSchemaFrom: stream
        | timeString |

        #todo.  "I need to get milliseconds from this"
        #swAdded.
        "can not starting skipping whitespace/looking for am/pm"
        Smalltalk isSmalltalkX ifTrue:[
            timeString := stream upToAny:#($Z $+ $-).
            stream atEnd ifFalse:[ stream backStep ].
        ] ifFalse:[
            timeString := stream swUpToWhile: [:each | (#($Z $+ $-) includes: each) not].
        ].
        ^Time fromString: timeString
! !

!XeXsdDateTimeAbstract methodsFor:'accessing-private'!

isPositiveOffset
	^isPositiveOffset
!

isPositiveOffset: aBoolean
	isPositiveOffset := aBoolean
!

offset
	^offset
!

offset: aTime
        offset := aTime
! !

!XeXsdDateTimeAbstract methodsFor:'comparing'!

= anXsdDateTime
	"This is not accurate, and only for testing purposes"

	^(self species == anXsdDateTime species)
		and: [self asNative = anXsdDateTime asNative ]
!

hash
	^self asNative hash
! !

!XeXsdDateTimeAbstract methodsFor:'converting'!

asNative
	"Note: these classes are a stub. I am hoping that Dolphin5 will contain date/time/timestamp classes that would retain timezone info"

	self subclassResponsibility
! !

!XeXsdDateTimeAbstract methodsFor:'printing'!

displayOn: aStream
	"This assumes that the timestamp is UTC"

	self isNormal 
		ifTrue: [self displayUTCOn: aStream]
		ifFalse: [self displayOffsetOn: aStream]
!

displayString
    ^ String streamContents:[:s | self displayOn: s]
!

printOn: aStream
    #cg.

    self displayOn:aStream
! !

!XeXsdDateTimeAbstract methodsFor:'printing-private'!

displayOffsetOn: stream 
	self subclassResponsibility
!

displayUTCOn: stream 
	self subclassResponsibility
! !

!XeXsdDateTimeAbstract methodsFor:'testing'!

isNormal 
        ^self offset isNil or: [self offset getMilliseconds "asMilliseconds" = 0]
! !

!XeXsdDateTimeAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXsdDateTimeAbstract.st,v 1.8 2011/07/12 16:20:23 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeXsdDateTimeAbstract.st,v 1.8 2011/07/12 16:20:23 cg Exp $'
! !
