"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: YAXO }"

SAXHandler subclass:#XeYAXOParser
	instanceVariableNames:'stack'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!

XeYAXOParser comment:'I am a SAXHandler that builds an XML Tree modeled by instances of XeXMLNodes (from the "SW XE XMLNodes" package)

'
!


!XeYAXOParser class methodsFor:'operations'!

docNodeFor: aByteObject
        "Answer a XeDoc"

        |inStream|

        inStream := LineNumberReadStream readingFrom:(aByteObject asString readStream).
        ^(super parseDocumentFrom: inStream) document
! !

!XeYAXOParser class methodsFor:'parsing'!

rootNodeFor: aByteObject
	"Answer a XeNode"

	^(self docNodeFor: aByteObject) documentElement
! !

!XeYAXOParser class methodsFor:'public'!

example1
	"self example1"

	^self 
		docNodeFor: '<SOAP-ENV:Envelope
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body>
<ns1:echoString xmlns:ns1="http://soapinterop.org/xsd">
<param0>abc</param0>
</ns1:echoString>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'
! !

!XeYAXOParser methodsFor:'accessing-private'!

push: anObject
	self stack add: anObject
	
!

stack
	^stack
!

top
	^self stack isEmpty
		ifTrue: [nil]
		ifFalse: [self stack last]
! !

!XeYAXOParser methodsFor:'content'!

characters: aString
        | newElement |
        newElement := XeBText string: aString.
        self top value addContent: newElement.
!

documentAttributes: attributeList
	self document prologAttributes: attributeList 
!

endDocument
        #swYAXO.
        self pop value isDocument ifFalse: [self driver malformedError: 'document element must be complete'].
        super endDocument
!

endElement: elementName
        | currentPop |
        #swYAXO.
        currentPop := self pop.
        "sw added #asString"
        currentPop value isDocument
                ifTrue: [self driver malformedError: 'Invalid end tag'].
        currentPop key = elementName
                ifFalse: [self driver malformedError: 'End tag "', elementName , '" doesn''t match "' , currentPop key , '".']
!

processingInstruction: piName data: dataString
	"| newElement |
	newElement := XMLPI target: piName data: dataString.
	self top addEntity: newElement"
!

startDocument
        self document: XeDoc new.
        self push: 'document' -> self document
!

startElement: elementName attributeList: attributeList
        | newElement |
        #swYAXO.
        newElement := XeNode named: elementName attributes: attributeList.
        self stack isEmpty 
                ifFalse: 
                        ["sw added check for only one document element"

                        | top |
                        ((top := self top value) isDocument and: [top hasDocumentElement]) 
                                ifTrue: [self driver malformedError: 'only one document element allowed'].
                        top addNode: newElement].
        self push: elementName -> newElement
! !

!XeYAXOParser methodsFor:'initialize'!

initialize
	super initialize.
	stack := OrderedCollection new.
! !

!XeYAXOParser methodsFor:'parsing'!

nextEntity
	"| currentTop |
	Notification deprecated.
	self halt.
	currentTop := self top.
	[self driver nextEntity isNil
		or: [self top ~~ currentTop]] whileTrue.
	^entity"
! !

!XeYAXOParser methodsFor:'positioning-private'!

pop
	^self stack removeLast
! !

!XeYAXOParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeYAXOParser.st,v 1.3 2010/03/10 11:55:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeYAXOParser.st,v 1.3 2010/03/10 11:55:27 cg Exp $'
! !
