"{ Package: 'stx:goodies/soap/xe' }"!

!Boolean class methodsFor:'spray'!

fromSchemaString: aString
	#swAdded.
	(aString = '0' or: [aString = 'false']) ifTrue: [^false].
	(aString = '1' or: [aString = 'true']) ifTrue: [^true].
	^self error: 'Can not convert to boolean: ' , aString displayString
! !

!Boolean class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'boolean'
! !

!ByteArray methodsFor:'spray'!

printSchemaHexString
	| stream |
	#swAdded.

	Smalltalk isSmalltalkX ifTrue:[
	    stream := WriteStream on:(String new:self size * 2).
	    self do:[:eachByte |
		stream nextPutAll:((eachByte printStringRadix:16) leftPaddedTo:2 with:$0)
	    ].
	] ifFalse:[
	    stream := String writeStream: self size * 2.
	    self printHexOn: stream.
	].
	^stream contents
! !

!ByteArray class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'base64Binary'
! !

!Date methodsFor:'converting'!

asNative
	#swAdded.
	^self
! !

!Date methodsFor:'printing'!

printUnboundedSchemaStringOn: stream
	"If a Date is prior to 01 January 1601, or post 31 December 65535, then it cannot be
	formatted by Windows. ... and the Dolphin printUnboundedDate method does not respect the format string"

	#swAdded.
	stream
		display: self year;
		nextPut: $-;
		display: (self monthIndex printStringRadix: 10 padTo: 2);
		nextPut: $-;
		display: (self dayOfMonth printStringRadix: 10 padTo: 2).
! !

!Date methodsFor:'printing'!

schemaDisplayString
	"Answer a valid XMLSchema  xsd:date lexical representation"
	"Assume UTC"

	| stream |
	#swAdded.
	stream := String writeStream.
	self printUnboundedSchemaStringOn: stream.
	stream nextPut: $Z.
	^stream contents
! !

!Date class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'date'
! !

!FixedPoint methodsFor:'spray'!

schemaDisplayString
    |int|

    int := self asInteger.
    self = int ifTrue:[
	^ int printString
    ].
    ^ self printString
! !

!FixedPoint class methodsFor:'instance creation'!

fromSchemaString: aString
	"Answer an instance. (keeping the scale)"

	#swAdded.

	Smalltalk isSmalltalkX ifTrue:[
	    ^super fromString: aString
	].
	^super fromString: aString , 's'
! !

!FixedPoint class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'decimal'
! !

!Float methodsFor:'spray'!

isSoapEncSameAs: anObject
	"This method is thanks to Tamara Kogan and Steve Dahl
	Note: changed 1.0e-6 to 1.0e-5 to 'be more liberal'"

	^(anObject isKindOf: Number)
		and: [
			self = 0.0
				ifTrue: [anObject = 0.0]
				ifFalse: [self sign = anObject sign
					and: [(self exponent - anObject exponent) abs <= 1
					and: [(self - anObject / self) abs < 1.0e-5]]]]


	"^(anObject isKindOf: Number) and: [
		 (self - anObject) abs < 0.0001]"

! !

!Float methodsFor:'printing'!

schemaDisplayString
	"Answer a valid XMLSchema lexical representation of the float"

	| basicString basicStream real|
	#swAdded.
	basicString := self displayString.
	basicString := (basicString = '0.0' and: [self sign = -1])
		ifTrue: ['-0.0']
		ifFalse: [basicString].

	basicStream  := basicString readStream.
	real := basicStream upTo: $..
	^(basicStream peek = $e)
		ifTrue: [real , '.0' , basicStream upToEnd]
		ifFalse: [basicString ]
! !

!Float class methodsFor:'instance creation'!

fromSchemaString: aString
	"Answer a float. Convert the schema string into a format dolphin can work with.
	Throw an error if the string is a lexical representation that I cant convert to a float, (so I can catch INF,-INF,NaN)"

	| lcString |
	#swAdded.
	(aString isNil or: [aString isEmpty]) ifTrue: [self error: 'Invalid Float'].
	(#('INF' '-INF' 'NaN') includes: aString)
		ifTrue: [SOAP::XeValueRangeError signal: 'Can not convert into a native value: ' , aString displayString]
		ifFalse: [(aString allSatisfy: [ :each | each isDigit or: [ 'eE]+-.' includes: each]]) ifFalse: [
			self error: 'Can not convert to integer: ' , aString displayString]].

	lcString := aString asLowercase copyWithout: $+.
	(lcString first isLetter or: [lcString first = $- and: [lcString second isLetter]])
		ifTrue: [self error: 'Can not convert ' , aString displayString , ' into a Float'].
	^self fromString: lcString
! !

!Float class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'double'
! !

!IdentityDictionary methodsFor:'spray-private'!

xeContentWith: anEncodingStyle
	"Answer a <XeContent> for the receiver"

	#swAdded.
	^anEncodingStyle
		complexSpray: nil
		restricting: anEncodingStyle anyTypeName
		instance: self
! !

!Integer class methodsFor:'spray'!

fromSchemaString: aString
	"(aString allSatisfy: [ :each | each isDigit or: [ '+-.' includes: each]]) ifFalse: [
		self error: 'Can not convert to integer: ' , aString displayString]."

	#swAdded.
	^self fromString: aString
! !

!Integer class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'integer'
! !

!Number class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'number'
! !

!Object methodsFor:'spray'!

isSoapEncSameAs: anObject
	#swAdded.
	^self = anObject
! !

!Object methodsFor:'spray'!

schemaDisplayString
    "Answer a <readableString> which is a valid XMLSchema lexical representation of the receiver"

    #swAdded.
    Smalltalk isSmalltalkX ifTrue:[
	^ self printString
    ].
    ^self displayString
! !

!Object methodsFor:'spray'!

sprayTypeName
    "Answer the <XeName> of the receiver's XMLSchema type.
    -Should only be implemented as an instance method if different instances have different type"

    #swAdded.
    ^self class sprayTypeName
! !

!Object methodsFor:'spray'!

xeContentWith: anEncodingStyle
    "Answer a <XeContent> for the receiver.
    --Only override this method if different instances have different type (See XeQStruct , and KeyedCollections,)."

    #swAdded.
    ^self sprayTypeName xeContentWith: anEncodingStyle
! !

!Object methodsFor:'spray'!

xmlDecodeFixupWith: aNodeDecoding
    "Answer the true object that must be used to represent the receiver when decoded from xml."

    #swAdded.
    ^self
! !

!Object methodsFor:'spray'!

xmlEncodeUsing: aXeSoapXMLWriter
    "Answer if the receiver has perfomed the encoding itself.
     Sent by  the <XeSoapXMLWriter>, to give the receiver the opportunity to override standard encoding."

    #swAdded.
    ^false
! !

!Object class methodsFor:'spray factory'!

sprayAttributes
	"Answer a collection of <XeAttributeSpec>s.
	-The answer is used by a <XeSprayClassFactory> to build a schema definition for the class."

	#swAdded.
	^#()
! !

!Object class methodsFor:'spray factory'!

sprayElements
	"Answer a collection of <XeElementSpec>s.
	-The answer is used by a <XeSprayClassFactory> to build a schema definition for the class.
	-This has a similar function to an XMLSchema complexType definition. For example;

<schema>
<complexType name='Something'>
  <element name='foo' type='xsd:string'/>
  <element name='bar' type='xsd:integer'/>
</complexType>
</schema>

Would be equivelent to a method

	Something(class)>>sprayElement;

	^OrderedCollection new
		add: (XeElementSpec named: #foo xsdTyped: 'string');
		add: (XeElementSpec named: #bar xsdTyped: 'integer');
		yourself
"

	#swAdded.
	^Error signal: 'This class does not define any child xml elements'
! !

!Object class methodsFor:'spray'!

sprayInstanceWith: aXeContent
	#swAdded.
	^self basicNew
! !

!Object class methodsFor:'spray'!

sprayTypeName
	"Answer the XMLSchema type <XeName> for the receiver's instances.
	-Default to answering a late-qualifying XePName with the 'd' prefix and the class name.
		-In a default typespace, the prefix 'd' late qualifies to a namespace targeted by a <XeSprayClassFactory>"

	#swAdded.
	Smalltalk isSmalltalkX ifTrue:[
	    ^SOAP::XePName prefix: 'd' local: self nameWithoutPrefix asString
	].
	^SOAP::XePName prefix: 'd' local: self name asString
! !

!Object class methodsFor:'spray'!

sprayValueTypeName
	"Answer the XMLSchema type <XeName> that the receivers instances extend.
	-Used by a <XeSimpleClassFactory> to create XMLSchema simple content definitions.
	-If the receiver implements this method, it also needs to define #sprayAttributes"

	#swAdded.
	^self error: 'Not simple content'
! !

!SequenceableCollection methodsFor:'comparing'!

isSoapEncSameAs: comparand
	"Answer whether the receiver and the <Object> argument, comparand,
	are considered equivalent in terms of the soap-enc rules."

	| size |
	#swAdded.
	self == comparand ifTrue: [^true].
	self species == comparand species ifFalse: [^false].
	size := self size.
	size = comparand size ifFalse: [^false].
	1 to: size do: [:i | ((self at: i) isSoapEncSameAs: (comparand at: i)) ifFalse: [^false]].
	^true
! !

!SequenceableCollection class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName prefix: 'soap' local: 'Array'
! !

!String methodsFor:'spray'!

asQName
	#swAdded.
	^SOAP::XeQName fromString: self asString
! !

!String methodsFor:'spray'!

asXeName
	#swAdded.
	^((self includes: $:) ifTrue: [SOAP::XePName] ifFalse: [SOAP::XeQName]) fromString: self asString
! !

!String methodsFor:'spray'!

ns: aSymbol
	#swAdded.
	^SOAP::XeQName local: self asString ns: aSymbol
! !

!String methodsFor:'converting'!

uncapitalized
        "Answer a <readableString> which is a copy of the receiver but with
        the first character converted to its lowercase equivalent."

        ^ self asLowercaseFirst
"/        | answer |
"/        #swAdded.
"/        (answer := self copy) isEmpty ifFalse: [
"/                answer at: 1 put: (self first asLowercase)].
"/        ^answer
! !

!String methodsFor:'spray'!

xmlBase
	#swAdded.
	^(self includes: $:)
		ifTrue: [self copyFrom: (self indexOf: $:) + 1 to: self size]
		ifFalse: [self]
! !

!String methodsFor:'spray'!

xmlPrefix
	#swAdded.
	^(self includes: $:)
		ifTrue: [self copyFrom: 1 to: (self indexOf: $:) - 1]
		ifFalse: []
! !

!String methodsFor:'spray'!

xmlSplit
	"Split a QName into prefix and local parts."

	| sIndex sArray |
	#swAdded.
	sArray := Array new: 2.
	(self includes: $:)
		ifTrue: [
			sIndex := (self indexOf: $:).
			sArray at: 1 put: (self copyFrom: 1 to: sIndex  - 1).
			sArray at: 2 put: (self copyFrom: sIndex + 1 to: self size)]
		ifFalse: [sArray at: 2 put: self].
	^sArray

! !

!String class methodsFor:'spray'!

sprayTypeName
    #swAdded.
    ^SOAP::XePName xsd: 'string'
! !

!Time methodsFor:'converting'!

asNative
	#swAdded.
	^self
! !

!Time methodsFor:'spray'!

schemaDisplayString
	"Answer a valid XMLSchema xsd:time lexical representation"

	"Assume UTC"

	| stream |
	#swAdded.
	stream := String writeStream.
	Smalltalk isSmalltalkX ifTrue:[
	    self printOn: stream format: '%h:%m:%s'.
	] ifFalse:[
	    self printOn: stream format: 'HH:mm:ss'.
	].
	stream nextPut: $Z.
	^stream contents
! !

!Time class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'time'
! !

!Timestamp methodsFor:'converting'!

asNative
	#swAdded.
	^self
! !

!Timestamp methodsFor:'spray'!

schemaDisplayString
	"Answer a valid XMLSchema xsd:dateTime lexical representation"

	"Assume UTC"

	| stream |
	#swAdded.

	stream := String writeStream.
	Smalltalk isSmalltalkX ifTrue:[
	    self asDate printUnboundedSchemaStringOn: stream.
	    stream nextPut: $T.
	    self asTime printOn: stream format: '%h:%m:%s'.
	] ifFalse:[
	    self date printUnboundedSchemaStringOn: stream.
	    stream nextPut: $T.
	    self time printOn: stream format: 'HH:mm:ss'.
	].
	stream nextPut: $Z.
	^stream contents
! !

!Timestamp class methodsFor:'spray'!

sprayTypeName
	#swAdded.
	^SOAP::XePName xsd: 'dateTime'
! !

!Unicode16String methodsFor:'uncategorized'!

asUTF8String
	"Answer a byte string representation of the receiver.
	-Not supported in Win95, but should work in Win98
	-If this fails,
	--edit the method XeTypespaceOptions>>beReadUTF8 to set the ReadUTF8 to false.
	--please contact me at swaring@ozemail.com.au with your system information. Thanks!!"

	| buf size bytes |
	#swAdded.
	size := self size.
	buf := String new: size+size+size.
	size == 0 ifTrue: [^buf].       "Avoid 'The Parameter is Incorrect' error"
	bytes := KernelLibrary default
		wideCharToMultiByte: 65001 "CP_UTF8"
		dwFlags: 0
		lpWideCharStr: self
		cchWideChar: size
		lpMultiByteStr: buf
		cchMultiByte: buf size
		lpDefaultChar: nil
		lpUsedDefaultChar: nil.
	bytes == 0 ifTrue: [^KernelLibrary default systemError].
	buf resize: bytes.
	^buf
! !

!stx_goodies_soap_xe class methodsFor:'documentation'!

extensionsVersion_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/extensions.st,v 1.13 2010/12/22 09:52:51 stefan Exp $'
! !