"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeXMLBuilderTests subclass:#XeBNodeTest
	instanceVariableNames:'node node1 node2 node3'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeBNodeTest methodsFor:'tests'!

testActualNamespace
	node1 := doc addElement: 'first'.
	node2 := node1 addElement: 'second'.
	node3 := node2 addElement: 'third'.
	self assert: node1 prefix = ''.
	node1 defaultNS: #'urn:blah'.
	self assert: node1 namespace asString = 'urn:blah'.
	self assert: node2 namespace asString = 'urn:blah'.
	self assert: node3 namespace asString = 'urn:blah'.
	node1 defaultNS: #'urn:alah'.
	self assert: node1 namespace asString = 'urn:alah'.
	self assert: node2 namespace asString = 'urn:alah'.
	self assert: node3 namespace asString = 'urn:alah'
!

testActualNamespaceIsDefault
	node := doc addElement: 'first'.
	node defaultNS: #'urn:blah'.
	node1 := node addElement: 'second'.
	self assert: node namespace asString = 'urn:blah'.
	self assert: node1 namespace asString = 'urn:blah'
!

testAddElement
	node := doc addElement: 'blah1'.
	self assert: node parentNode == doc.
	node1 := node addElement: 'blah2'.
	self assert: node1 parentNode == node.
	self assertDocXML: 
'<blah1><blah2/></blah1>'
!

testAddElementNamed
	| name  |
	node := doc addElement: 'abc'.
	node1 := node addElement: 'def'.
	name := XeQName 
				prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.

	node1 addElementNamed: name rootDeclare: true.
	self assertDocXML: 
'<?xml version="1.0"?>
<abc xmlns:a="urn:myspace"><def><a:aNode/></def></abc>'.

!

testAddElementNamedNoNamespace
	| name |
	node := doc addElement: 'abc'.
	node1 := node addElement: 'def'.
	name := 'aNode' asQName.
	node1 addElementNamed: name rootDeclare: true.
	self assertDocXML: 
'<?xml version="1.0"?>
<abc><def><aNode/></def></abc>'
!

testAddElementNamedNotRootDeclare
	| name |
	node := doc addElement: 'abc'.
	node1 := node addElement: 'def'.
	name := XeQName 
				prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.
	node1 addElementNamed: name rootDeclare: false.
	self assertDocXML: 
'<?xml version="1.0"?>
<abc><def><a:aNode xmlns:a="urn:myspace"/></def></abc>'
!

testAddElementPrefixed
	node := doc addElement: 'a:blah1'.
	self assert: node parentNode == doc.
	node1 := node addElement: 'b:blah2'.
	self assert: node1 parentNode == node.
	node addNSPrefix: 'a' to: 'urn:a'.
	node addNSPrefix: 'b' to: 'urn:b'.
	self assertDocXML: 
'<?xml version="1.0"?><a:blah1 xmlns:a="urn:a" xmlns:b="urn:b"><b:blah2/></a:blah1>'
!

testAddNullAttribute
	| root |
	root := doc addElement: 'blah'.
	root addNullAttributeFor: XeNamespace schema2001i.
	self assert: (root isPrefixDeclared: 'xsi').
	self assert: (root attributes includesKey: 'xsi:nil')

!

testAddNullAttribute1999
	| root |
	root := doc addElement: 'blah'.
	root addNullAttributeFor: XeNamespace schema1999i.
	self assert: (root isPrefixDeclared: 'xsi').
	self assert: (root attributes includesKey: 'xsi:null')

!

testAddNullAttribute2001
	| root |
	root := doc addElement: 'blah'.
	root addNullAttributeFor: XeNamespace schema2001i.
	self assert: (root isPrefixDeclared: 'xsi').
	self assert: (root attributes includesKey: 'xsi:nil')

!

testAddQName
	| root |
	root := doc addElement: 'blah1'.
	node := root addElementNamed: (XeQName 
						prefix: 'a'
						local: 'blah2'
						ns: #'urn:blah3')
				rootDeclare: false.
	self assertDocXML: '<blah1><a:blah2 xmlns:a="urn:blah3"/></blah1>'.
	node := node addElementNamed: (XeQName 
						prefix: 'a'
						local: 'foo'
						ns: #'urn:blah3')
				rootDeclare: false.
	self assertDocXML: '<blah1><a:blah2 xmlns:a="urn:blah3"><a:foo/></a:blah2></blah1>'.
	node := node addElementNamed: (XeQName 
						prefix: 'a'
						local: 'foo'
						ns: #'urn:blah3')
				rootDeclare: true.
	self assertDocXML: '<blah1><a:blah2 xmlns:a="urn:blah3"><a:foo><a:foo/></a:foo></a:blah2></blah1>'.
	node := node addElementNamed: (XeQName 
						prefix: 'b'
						local: 'bar'
						ns: #'urn:blah3')
				rootDeclare: false.
	self assertDocXML: '<blah1><a:blah2 xmlns:a="urn:blah3"><a:foo><a:foo><a:bar/></a:foo></a:foo></a:blah2></blah1>'
!

testAddText
	node := doc addElement: 'blah1'.
	node1 := node addTextNode: 'blah2'.
	self assertDocXML: '<blah1>blah2</blah1>'
!

testAppendChild
	#cg.
	node := doc addElement: 'root'.
	node1 := YAXO::XeNode nodeName: 'b1'.
	node addNode: node1.
	self assert: node1 parentNode == node
!

testBeFirstChild
	node := doc addElement: 'blah1'.
	node1 := node addElement: 'c1'.
	node2 := node addElement: 'c2'.
	node3 := node addElement: 'c3'.
	self assertDocXML: '<blah1><c1/><c2/><c3/></blah1>'.
	node1 beFirstChild.
	self assertDocXML: '<blah1><c1/><c2/><c3/></blah1>'.
	node2 beFirstChild.
	self assertDocXML: '<blah1><c2/><c1/><c3/></blah1>'.
	node2 beFirstChild.
	self assertDocXML: '<blah1><c2/><c1/><c3/></blah1>'.
	node3 beFirstChild.
	self assertDocXML: '<blah1><c3/><c2/><c1/></blah1>'.
!

testDefaultAndPrefixNamespace
	|node4 node5  |
	node1 := doc addElement: 'first'.
	node2 := node1 addElement: 'second'.
	node3 := node2 addElement: 'third'.
	self assert: node1 prefix = ''.
	node1 defaultNS: #'urn:blah'.
	node1 addNSPrefix: 'c' to: 'urn:clah'.
	self assert: node1 namespace asString = 'urn:blah'.
	self assert: node2 namespace asString = 'urn:blah'.
	self assert: node3 namespace asString = 'urn:blah'.
	node4 := node1 addElement: 'c:fourth'.
	node5 := node4 addElement: 'fifth'.
	self assert: node4 namespace asString = 'urn:clah'.
	self assert: node5 namespace asString = 'urn:blah'
!

testFixupPrefixExisting
	| name |
	node1 := doc addElement: 'abc'.
	node2 := node1 addElement: 'def'.
	name := XeQName 
				prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.
	node1 fixupPrefixGloballyFor: name.
	self assert: (node1 isPrefixDeclared: 'a').
	node1 fixupPrefixLocallyFor: name.
	self assert: (node1 isPrefixDeclared: 'a').
	node2 fixupPrefixLocallyFor: name.
	self assert: (node1 isPrefixDeclared: 'a').
	self assertDocXML: '<abc xmlns:a="urn:myspace"><def/></abc>'
!

testFixupPrefixNoPrefix
	| name1 name2 |
	node1 := doc addElement: 'abc'.
	node2 := node1 addElement: 'def'.
	name1 := XeQName prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.
	name2 := XeQName local: 'aNode' ns: #'urn:myspace'.
	node2 fixupPrefixGloballyFor: name1.
	self assert: (node1 isPrefixDeclared: 'a').
	node2 fixupPrefixLocallyFor: name2.
	self assert: name2 prefix = 'a'.
	self assert: (node1 isPrefixDeclared: 'a').
	self assert: (node2 isPrefixDeclared: 'a')
!

testFixupPrefixPrefix
	|  name1 name2 |
	node1 := doc addElement: 'abc'.
	node2 := node1 addElement: 'def'.
	name1 := XeQName prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.
	name2 := XeQName prefix: 'b'
				local: 'aNode'
				ns: #'urn:myspace'.
	node2 fixupPrefixGloballyFor: name1.
	self assert: (node1 isPrefixDeclared: 'a').
	node2 fixupPrefixLocallyFor: name2.
	self assert: name2 prefix = 'a'.
	self assert: (node1 isPrefixDeclared: 'a').
	self assert: (node2 isPrefixDeclared: 'a')
!

testFixupPrefixPrefixClash
	| name1 name2 |
	node1 := doc addElement: 'abc'.
	node2 := node1 addElement: 'def'.
	name1 := XeQName prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace'.
	name2 := XeQName prefix: 'a'
				local: 'aNode'
				ns: #'urn:myspace2'.
	node2 fixupPrefixGloballyFor: name1.
	self assert: (node1 isPrefixDeclared: 'a').
	node2 fixupPrefixLocallyFor: name2.
	self deny: (name2 prefix = 'a').
	self assert: (node1 isPrefixDeclared: 'a')
!

testIsPrefixDeclared
	node := doc addElement: 'first'.
	node2 := node addElement: 'second'.
	self deny: (node isPrefixDeclared: 'a').
	self deny: (node2 isPrefixDeclared: 'a').
	node addNSPrefix: 'a' to: 'urn:foo'.
	self assert: (node isPrefixDeclared: 'a') .
	self assert: (node2 isPrefixDeclared: 'a').
	self deny: (node isPrefixDeclared: 'b').
	self deny: (node2 isPrefixDeclared: 'b').
!

testIsSimpleContent
	#cg.
	node := doc addElement: 'blah1'.
	self assert: node isSimpleContent.
	node addTextNode: 'abc'.
	self assert: node isSimpleContent.
	doc := YAXO::XeDoc new.
	node := doc addElement: 'blah1'.
	node addElement: 'c1'.
	self deny: node isSimpleContent.
	node addElement: 'c2'.
	self deny: node isSimpleContent
!

testNamespaceFor
	node1 := doc addElement: 'first'.
	node2 := node1 addElement: 'second'.
	node3 := node2 addElement: 'third'.
	self assert: node1 prefix = ''.
	node1 addNSPrefix: 'a' to: 'urn:foo'.
	node1 defaultNS: #'urn:blah'.
	self assert: node1 namespace asString = 'urn:blah'.
	self assert: node2 namespace asString = 'urn:blah'.
	self assert: (node1 namespaceFor: 'a') asString = 'urn:foo'.
	self assert: (node2 namespaceFor: 'a') asString = 'urn:foo'.
	self assert: (node3 namespaceFor: 'a') asString = 'urn:foo'.
	self assert: (node1 namespaceFor: 'z') asString = ''.
	self assert: (node2 namespaceFor: 'z') asString = ''
!

testNextEmptyPrefix
	node := doc addElement: 'blah1'.
	self assert: node nextEmptyPrefix = 'ns1'.
	node addNSPrefix: 'ns1' to: #'urn:blah'.
	self assert: node nextEmptyPrefix = 'ns2'.

!

testRemoveAttribute
	node := doc addElement: 'blah1'.
	node setAttribute: 'id1' value: 'this'.
	node setAttribute: 'id2' value: 'that'.
	self assertDocXML: '<blah1 id1="this" id2="that"/>'.
	self should: [node removeAttribute: 'xmlns:id1'] raise: Error.
	node removeAttribute: 'id1'.
	self assertDocXML: '<blah1 id2="that"/>'.
	self should: [node removeAttribute: 'id1'] raise: Error.
	node removeAttribute: 'id2'.
	self assertDocXML: '<blah1/>'.
!

testReplaceChild
	| newNode |

	#cg.
	node := doc addElement: 'root'.
	node1 := node addElement: 'a'.
	node1 addElement: 'a1'.
	node2 := node1 addElement: 'a2'.
	node1 addElement: 'a3'.
	newNode := YAXO::XeNode nodeName: 'b1'.
	node1 replaceChild: newNode oldChild: node2.
	self assert: newNode parentNode == node1.
	self assertDocXML: '<?xml version="1.0"?><root><a><a1/><b1/><a3/></a></root>'
!

testSetAttribute
	node := doc addElement: 'blah1'.
	node setAttribute: 'id' value: 'this'.
	self assertDocXML: 
'<blah1 id="this"/>'
!

testSetAttributeQName
	node := doc addElement: 'blah1'.
	node addNSPrefix: 'b' to: #'urn:blah'.
	node setAttribute: 'id' qName: (self blahQ: 'this').
	self assertDocXML: 
'<blah1 id="b:this" xmlns:b="urn:blah"/>'
!

testSetQAttributeQName
	node := doc addElement: 'blah1'.
	node addNSPrefix: 'b' to: #'urn:blah'.
	node setQAttribute: (self blahQ: 'id') qName: (self blahQ: 'this').
	self assertDocXML: 
'<blah1 b:id="b:this" xmlns:b="urn:blah"/>'
!

testSetQAttributeValue
	node := doc addElement: 'blah1'.
	node addNSPrefix: 'b' to: #'urn:blah'.
	node setQAttribute: (self blahQ: 'id') value: 'this'.
	self assertDocXML: 
'<blah1 b:id="this" xmlns:b="urn:blah"/>'
! !

!XeBNodeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeBNodeTest.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
