"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeValueTests subclass:#XeBaseValuesTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeBaseValuesTest methodsFor:'tests'!

testByte
	self be2001Typespace.
	self setForXSDTyped: 'byte'.
	self assertValuesEncodeAndDecode: #(0 127 -128).
	self assertValuesAndDisplayStrings: #(128 -129) raise: XeValueMismatch.
        
!

testEnumeration
	self be2001Typespace.
	self importEnumerationSchema.
	element := (XeElementSpec named: #root typeName: 'enumns:EyeColor') 
				xeParticleWith: self encodingStyle.
	self assertValuesEncodeAndDecode: #('Green' 'Blue').
"/        self assertValuesAndDisplayStrings: #('' ' ' 'Black' ' Green' 'Green ')
"/                raise: XeValueMismatch.
	self assertValuesAndDisplayStrings: #('')
		raise: XeValueMismatch.
	self assertValuesAndDisplayStrings: #(' ')
		raise: XeValueMismatch.
	self assertValuesAndDisplayStrings: #('Black')
		raise: XeValueMismatch.
	self assertValuesAndDisplayStrings: #(' Green')
		raise: XeValueMismatch.
	self assertValuesAndDisplayStrings: #('Green ')
		raise: XeValueMismatch.
!

testFacet
	self be2001Typespace.
	self importFacetSchema.
	element := (XeElementSpec named: #root typeName: 'f:myInteger') 
				xeParticleWith: self encodingStyle.
	self assertValuesEncodeAndDecode: #(10000 99999 50000).
	self assertValuesAndDisplayStrings: #(0 9999 100000 100000000000000000000)
		raise: XeValueMismatch
!

testFloat
	self be2001Typespace.
	self setForXSDTyped: 'float'.
	self assertValuesEncodeAndDecode: #(3.14 -3.14 0).
	self assertValuesAndDisplayStrings: 
		(Array
			with: 1.0e+50
			with: -1.0e50
			with: 1.0e-50
			with: -1.0e-50)
	 raise: XeValueMismatch.        
        
!

testFloatNAN
	self be2001Typespace.
	self typespaceOptions beConvertAllValues.
	self setForXSDTyped: 'float'.
	self assertValuesEncodeAndDecode: (Array 
		with: (XeAnyValue new text: 'NaN')
		with: (XeAnyValue new text: '-INF')
		with: (XeAnyValue new text: 'INF')).
        
!

testFloatNANNotConvert
	self be2001Typespace.
	self typespaceOptions beNotConvertAllValues.
	self setForXSDTyped: 'float'.
	self assertValues: (Array 
			with: (XeAnyValue new text: 'NaN')
			with: (XeAnyValue new text: '-INF')
			with: (XeAnyValue new text: 'INF'))
		raise: XeValueRangeError 
        
!

testHexBinary
	self be2001Typespace.
	self setForXSDTyped: 'hexBinary'.
	self assertValuesEncodeAndDecode: (Array with: #[0] with: #[]).
!

testID
	self be2001Typespace.
	self setForXSDTyped: 'ID'.
	self assertValuesEncodeAndDecode: #('abc'  'a b' '_a').
	self assertValuesAndDisplayStrings: #(' ' '' ' a' 'a ' 'a  b' '1' '1a' ':a' 'a:a') raise: XeValueMismatch.      
        



!

testIDREF
	self be2001Typespace.
	self setForXSDTyped: 'IDREF'.
	self assertValuesEncodeAndDecode: #('abc' 'a b' '_a').
	self assertValuesAndDisplayStrings: #(' ' '' ' a' 'a ' 'a  b' '1' '1a' ':a' 'a:a') raise: XeValueMismatch.      
        



!

testInt
	self be2001Typespace.
	self setForXSDTyped: 'int'.
	self assertValuesEncodeAndDecode: #(0 2147483647).
	self assertValuesAndDisplayStrings: #(2147483648) raise: XeValueMismatch.       

        
!

testLong
	self be2001Typespace.
	self setForXSDTyped: 'long'.
	self assertValuesEncodeAndDecode: #(0 9223372036854775807 -9223372036854775808).
	self assertValuesAndDisplayStrings: #(9223372036854775808 -9223372036854775809) raise: XeValueMismatch. 
        
!

testNCName
	self be2001Typespace.
	self setForXSDTyped: 'NCName'.
	self assertValuesEncodeAndDecode: #('abc'  'a b' '_a').
	self assertValuesAndDisplayStrings: #(' ' '' ' a' 'a ' 'a  b' '1' '1a' ':a' 'a:a') raise: XeValueMismatch.      
        



!

testNMTOKEN
	self be2001Typespace.
	self setForXSDTyped: 'NMTOKEN'.
	self assertValuesEncodeAndDecode: #('abc').
        
!

testNegativeInteger
	self be2001Typespace.
	self setForXSDTyped: 'negativeInteger'.
	self assertValuesEncodeAndDecode: #(-1 -34124124).
	self assertValuesAndDisplayStrings: #(0 1) raise: XeValueMismatch.      
!

testNonNegativeInteger
	self be2001Typespace.
	self setForXSDTyped: 'nonNegativeInteger' .
	self assertValuesEncodeAndDecode: #(0 34124124 ).
	self assertValuesAndDisplayStrings: #(-1) raise: XeValueMismatch.       
!

testNonPositiveInteger
	self be2001Typespace.
	self setForXSDTyped: 'nonPositiveInteger'.
	self assertValuesEncodeAndDecode: #(0 -34124124 ).
	self assertValuesAndDisplayStrings: #(1) raise: XeValueMismatch.        
!

testNormalizedString
	self be2001Typespace.
	self setForXSDTyped: 'normalizedString'.
	self assertValuesEncodeAndDecode: #('abc' '' ).
	self assertValuesAndDisplayStrings: (Array 
		with: (String with: (Character tab)) 
		with: (String with: (Character cr)))
		raise: XeValueMismatch.

        
!

testPositiveInteger
	self be2001Typespace.
	self setForXSDTyped: 'positiveInteger'.
	self assertValuesEncodeAndDecode: #(1 34124124  ).
	self assertValuesAndDisplayStrings: #(-1 0) raise: XeValueMismatch.     

        
!

testShort
	self be2001Typespace.
	self setForXSDTyped: 'short'.
	self assertValuesEncodeAndDecode: #(0 32767 -32768).
	self assertValuesAndDisplayStrings: #(32768 -32769) raise: XeValueMismatch.     

        
!

testString
	self be2001Typespace.
	self setForXSDTyped: 'string'.
	self assertValuesEncodeAndDecode: #(' ' '' '  ' ).
	self assertValuesEncodeAndDecode: (Array with: (String with: (Character lf))).
        

        
!

testToken
	self be2001Typespace.
	self setForXSDTyped: 'token'.
	self assertValuesEncodeAndDecode: #('abc' '' 'a b').
	self assertValuesAndDisplayStrings: #(' ' ' a' 'a ' 'a  b') raise: XeValueMismatch.     
	self assertValuesAndDisplayStrings: (Array 
		with: (String with: (Character tab)) 
		with: (String with: (Character cr)))
		raise: XeValueMismatch.



!

testUnsignedByte
	self be2001Typespace.
	self setForXSDTyped: 'unsignedByte'.
	self assertValuesEncodeAndDecode: #(0 255).
	self assertValuesAndDisplayStrings: #(-1 256) raise: XeValueMismatch.   
        
!

testUnsignedInt
	self be2001Typespace.
	self setForXSDTyped:  'unsignedInt'.
	self assertValuesEncodeAndDecode: #(0 4294967295 ).
	self assertValuesAndDisplayStrings: #(-1 4294967296 ) raise: XeValueMismatch.   
        

        
!

testUnsignedLong
	self be2001Typespace.
	self setForXSDTyped:  'unsignedLong'.
	self assertValuesEncodeAndDecode: #(0 18446744073709551615 ).
	self assertValuesAndDisplayStrings: #(-1 18446744073709551616 ) raise: XeValueMismatch. 
        
!

testUnsignedShort
	self be2001Typespace.
	self setForXSDTyped:   'unsignedShort'.
	self assertValuesEncodeAndDecode: #(0 65535 ).
	self assertValuesAndDisplayStrings: #(-1 65536 ) raise: XeValueMismatch.        
        
!

test_Name
    "/ sorry, but testName is already used for SUnit itself... (sigh)"

        self be2001Typespace.
        self setForXSDTyped: 'name'.
        self assertValuesEncodeAndDecode: #('abc'  'a b' ':a' '_a').
        self assertValuesAndDisplayStrings: #(' ' '' ' a' 'a ' 'a  b' '1' '1a') raise: XeValueMismatch. 
! !

!XeBaseValuesTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeBaseValuesTest.st,v 1.5 2010/03/12 14:01:08 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeBaseValuesTest.st,v 1.5 2010/03/12 14:01:08 cg Exp $'
! !
