"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingCoreTests subclass:#XeNodeWriteMapTest
	instanceVariableNames:'map topNode node'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeNodeWriteMapTest methodsFor:'set up'!

setUp
	super setUp.
	topNode := self newNode: 'top'.
	node := topNode addElement: 'blah'.
	map := XeNodeWriteMap new initializeFor: nil node: topNode.
! !

!XeNodeWriteMapTest methodsFor:'tests'!

testMakeMultiRefComplex
	(node addElement: 'c1') addTextNode: '1'.
	(node addElement: 'c2') addTextNode: '2'.
	map makeNormalMultiRef: node.
	self assertXML: (topNode ownerDocXML)
		isSameAs: 
'<top xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<blah href="#id1"/>
<id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0">
<c1>1</c1>
<c2>2</c2>
</id1></top>'
!

testMakeMultiRefLeavesPosition
	topNode addNSPrefix: 'blah' to: #'urn:blah'.
	node addTextNode: '3'.
	node setAttribute: 'blah:position' value: '3'.
	map makeNormalMultiRef: node.
	self assertXML: topNode ownerDocXML
		isSameAs: 
'<top xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:blah="urn:blah" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<blah blah:position="3" href="#id1"/>
<id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0">3</id1></top>'
!

testMakeMultiRefMovesType
	topNode addNSPrefix: 'xsd' to: XeNamespace schema2001.
	topNode addNSPrefix: 'xsi' to: XeNamespace schema2001i.
	node addTextNode: '3'.
	node setAttribute: 'xsi:type' value: 'xsd:int'.
	map makeNormalMultiRef: node.
	self assertXML: topNode ownerDocXML
		isSameAs:
'<top xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<blah href="#id1"/><id1 xsi:type="xsd:int" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0">3</id1></top>'


!

testMakeMultiRefSimple
	node addTextNode: '3'.
	map makeNormalMultiRef: node.
	self assertXML: topNode ownerDocXML
		isSameAs: 
'<top xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<blah href="#id1"/><id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0">3</id1></top>'


!

testMakeMultiRefSimpleEmpty
	map makeNormalMultiRef: node.
	self assertXML: topNode ownerDocXML
		isSameAs: 
'<top xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<blah href="#id1"/><id1 SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" id="id1" SOAP-ENC:root="0"/></top>'


!

testMakeMultiRefSpecial
	node addTextNode: '3'.
	map topNode: topNode.
	map makeSpecialMultiRef: node.
	self assertXML: topNode ownerDocXML isSameAs: '<top><blah id="id1">3</blah></top>'
!

testNextID
	self assert: map nextID = 'id1'.
	self assert: map nextID = 'id2'
! !

!XeNodeWriteMapTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeNodeWriteMapTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
