"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingCoreTests subclass:#XeQStructTest
	instanceVariableNames:'struct'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeQStructTest methodsFor:'accessing'!

getBlahQStruct
	^(XeQStruct new)
		at: (self blahQ: #varString) put: 'abc';
		at: (self blahQ: #varInt) put: 123;
		at: (self blahQ: #varFloat) put: 3.14;
		yourself
!

getSoapStruct
	^(XeQStruct new)
		at: #varString put: 'abc';
		at: #varInt put: 123;
		at: #varFloat put: 3.14;
		yourself
!

setBlahQStruct
	struct := self getBlahQStruct
! !

!XeQStructTest methodsFor:'set up'!

setUp
	super setUp.
	struct := self getSoapStruct
        
! !

!XeQStructTest methodsFor:'tests'!

testAccessing
	self assert: struct first = 'abc'.
	self assert: struct second = 123.
	self assert: (struct at: #varInt) = 123.
	self assert: (struct at: #varNotInt ifAbsent: [456] ) = 456.
!

testAccessingExceptions
	self should: [struct at: #varNotInt] raise: Error.
	struct := XeQStruct new.
	self should: [struct at: #varNotInt] raise: Error.
	self should: [struct first] raise: Error
!

testAccessingLocalName
	self assert: (struct atLocalName: #varInt) = 123.
	self assert: (struct atLocalName: 'varInt') = 123.
	self assert: (struct atLocalName: 'varInt' ifAbsent: [456]) = 123.
	self assert: (struct atLocalName: 'notvarInt' ifAbsent: [456]) = 456.

!

testAccessingLocalNameQ
	self setBlahQStruct.
	self assert: (struct atLocalName: #varInt) = 123.
	self assert: (struct atLocalName: 'varInt') = 123.
	self assert: (struct atLocalName: 'varInt' ifAbsent: [456]) = 123.
	self assert: (struct atLocalName: 'notvarInt' ifAbsent: [456]) = 456.

!

testAccessingQ
	self setBlahQStruct.
	self assert: struct first = 'abc'.
	self assert: struct second = 123.
	self assert: (struct at: (self blahQ: #varInt)) = 123.
	self assert: (struct at: #varInt ifAbsent: [456]) = 456.
	self assert: (struct at: ('varInt' ns: #notblah) ifAbsent: [456]) = 456
!

testAccessorsNotUnique
	self setBlahQStruct.
	struct at: (self blahQ: #varString) put: 'def'.
	self assert: struct size = 4.
	self assert: (struct at: (self blahQ: #varString)) = 'abc'
!

testAsQualified
	| literalStruct |
	literalStruct := struct asElementQualifiedIn: #'http://soapinterop.org/xsd'.
	self 
		assert: literalStruct elements first key = ('varString' ns: #'http://soapinterop.org/xsd').
	self assert: literalStruct first = 'abc'
!

testAsQualifiedQ
	| struct2 |
	self setBlahQStruct.
	struct2 := struct asElementQualifiedIn: #'http://soapinterop.org/xsd'.
	self assert: struct2 == struct.
	self assert: struct elements first key = ('varString' ns: #'http://soapinterop.org/xsd').
	self assert: struct first = 'abc'
!

testAtPut
	struct at: #somethingElse put: 'def'.
	self assert: (struct at: #somethingElse) = 'def'.
	struct at: #somethingElse put: 'hij'.
	self assert: (struct at: #somethingElse) = 'def'.
!

testAtUniquePut
	struct at: #somethingElse put: 'def'.
	self assert: (struct at: #somethingElse) = 'def'.
	struct atUnique: #somethingElse put: 'hij'.
	self assert: (struct at: #somethingElse) = 'hij'.
!

testIncludesKey
	self assert: (struct includesKey: #varInt).
	self deny: (struct includesKey: #varNotInt).
!

testIsSameAs
	self assert: (struct isSoapEncSameAs: struct copy).
	self assert: (struct isSoapEncSameAs: struct deepCopy).
	self deny: (struct isSoapEncSameAs: XeQStruct new).
	self 
		deny: (struct isSoapEncSameAs: ((self getSoapStruct)
						at: #varInt put: 345;
						yourself)).
	self 
		deny: (struct isSoapEncSameAs: ((self getSoapStruct)
						at: #varnotInt put: 345;
						yourself)).
	self 
		deny: (struct isSoapEncSameAs: ((self getSoapStruct)
						removeKey: #varInt;
						yourself))
!

testIsSameAsNull
	struct atUnique: #varInt put: nil.
	self 
		assert: (struct isSoapEncSameAs: ((self getSoapStruct)
						removeKey: #varInt;
						yourself)).
	self 
		assert: (((self getSoapStruct)
				removeKey: #varInt;
				yourself) isSoapEncSameAs: struct)
!

testIsSameAsReOrder
	(struct := XeQStruct new)
		at: #varFloat put: 3.14;
		at: #varString put: 'abc';
		at: #varInt put: 123.
	self assert: (struct isSoapEncSameAs: self getSoapStruct)
!

testKeys
	self assert: (struct elements collect: [:each | each key localName asSymbol]) asArray 
				= #(#varString #varInt #varFloat)
!

testRemoveKey
	struct removeKey: #varInt.
	self assert: struct first = 'abc'.
	self assert: struct second = 3.14.
!

testRemoveKeyQ
	self setBlahQStruct.
	struct removeKey: (self blahQ: #varInt).
	self assert: struct first = 'abc'.
	self assert: struct second = 3.14.
!

testValues
	self assert: struct values asArray = #('abc' 123 3.14).


! !

!XeQStructTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeQStructTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
