"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeFuncTests subclass:#XeSchemaDecReadTests
	instanceVariableNames:'schema schemaString element'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSchemaDecReadTests methodsFor:'accessing'!

accessorKeysInlineComplexNQ
	^#(#currency #value) collect: [ :each | XeQName null: each]
!

assertElement: operation
	operation value.
	self recreateElement.
	operation value.
!

content
	^element content
!

elementContent
	^element content 
!

importSchemaString: xmlString
	self setTypespace.
	schemaString := xmlString.
	schema := super importSchemaString: xmlString
!

recreateElement
	| schemaWriter elementName |
	elementName := element elementName.
	schemaWriter := XeSchemaWriter targeting: elementName namespace schemaNS: typespace xsdNamespace.
	schemaWriter declareGlobally: element.
	self importSchemaString: schemaWriter xml.
	self setElementFromSchema: elementName localName
!

setElementFromSchema: aLocalName
	element := schema xeElementDeclared: (schema qualifyLocal: aLocalName)
				encStyle: self encodingStyle
! !

!XeSchemaDecReadTests methodsFor:'tests'!

testAnyType
	self importSchemaString: XeTestSchemaImports new decAnyType.
	self setElementFromSchema: 'anything'.
	self assertElement: 
			[self assert: element localName = 'anything'.
			self assert: element content typeName = (XeQName xsd2001: 'anyType').
			self assert: (element isKindOf: XeAnyTypeElement)]
!

testAnyTypeImplied
	self importSchemaString: XeTestSchemaImports new decAnyTypeImplied.
	self setElementFromSchema: 'anything'.
	self assertElement: 
			[self assert: element localName = 'anything'.
			self assert: element content typeName = (XeQName xsd2001: 'anyType').
			self assert: (element isKindOf: XeAnyTypeElement)]
!

testInlineComplex
	self importSchemaString: XeTestSchemaImports new decComplexInline.
	self setElementFromSchema: 'price'.
	self assertElement: 
			[| childContent |
			self assert: element localName = 'price'.
			self deny: element content isNamed.
			self assert: (self content isKindOf: XeComplexContent).
			self assert: (self symmetricDifferenceOf:self content accessorKeys and: self accessorKeysInlineComplexNQ) isEmpty.
			childContent := (self content childLocallyNamed: 'value') content.
			self assert: childContent typeName = (XeQName xsd2001: 'decimal')]
!

testInlineComplexEFDQ
	self importSchemaString: XeTestSchemaImports new decComplexInlineEFDQ.
	self setElementFromSchema: 'price'.
	self assertElement: 
			[| childContent |
			self assert: element localName = 'price'.
			self deny: element content isNamed.
			self assert: (self content isKindOf: XeComplexContent).
			self assert: (self symmetricDifferenceOf:self content accessorKeys and: self accessorKeysInlineComplexEFDQ) isEmpty.
			childContent := (self content childLocallyNamed: 'value') content.
			self assert: childContent typeName = (XeQName xsd2001: 'decimal')]
!

testInlinePrefixed
	self importSchemaString: XeTestSchemaImports new decComplexInlinePrefixed.
	self setElementFromSchema: 'addIt'.
	self assertElement: 
			[self assert: element qName localName = 'addIt'.
			self assert: (element content isKindOf: XeComplexContent).
			self assert: element firstChildElement localName = 'a'.
			"self assert:element firstChildElement namespace = #'http://soap.develop.com/wsdl/calculator.wsdl'."
			self assert: element firstChildContent localName = 'double'.
			self assert: element firstChildContent namespace = #'http://www.w3.org/2001/XMLSchema'.
			self assert: element content typespace notNil]
!

testInlineSimple
	self importSchemaString: XeTestSchemaImports new decSimpleInline.
	self setElementFromSchema: 'quantity'.
	self assertElement: 
			[self assert: element localName = 'quantity'.
			self deny: element content isNamed.
			self assert: (self content isKindOf: XeSimpleRestrictedContent).
			self assert: self content restrictions size = 1.
			self assert: (self content baseContent isKindOf: XeSimpleRestrictedContent).
			self assert: self content baseContent localName = 'int']
!

testJoke
	self importSchemaString: XeTestSchemaImports new decJoke.
	self setElementFromSchema: 'JokeOfTheDay'.
	self assertElement: [self assert: (self content isKindOf: XeComplexContent)]
!

testJokeNested
	self importSchemaString: XeTestSchemaImports new decJoke.
	self setElementFromSchema: 'JokeOfTheDayResult'.
	self assertElement: 
			[self assert: (self content isKindOf: XeComplexContent).
			self assert: self content childElements size = 1.
			self assert: (self content firstChildContent isKindOf: XeComplexContent)]
!

testMSDeclaredEnumeration
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic05"

	self importSchemaString: XeTestSchemaImports new decSimpleEnum.
	self setElementFromSchema: 'Gender'.
	self assertElement: 
			[self assert: element localName = 'Gender'.
			self assert: element namespace = #someNamespace.
			self assert: (self content isKindOf: XeSimpleRestrictedContent).
			self assert: self content baseContent typeName = (XeQName xsd2001: 'string').
			self content validate: 'Male'.
			self content validate: 'Female'.
			self should: [self content validate: 'NOT'] raise: XeValueMismatch]
!

testTypedSameNamespace
	self importSchemaString: XeTestSchemaImports new decComplexGlobalSameNS.
	self setElementFromSchema: 'addIt'.
	self assertElement: 
			[self assert: element qName localName = 'addIt'.
			self assert: (self content isKindOf: XeComplexContent).
			self assert: self content firstChildContent localName = 'double'.
			self assert: self content typespace notNil]
!

testTypedXSDNamespace
	self importSchemaString: XeTestSchemaImports new decComplexGlobalBase.
	self setElementFromSchema: 'inputString'.
	self assertElement: [
		self assert: self content localName = 'string'.
		self assert: self content namespace = #'http://www.w3.org/2001/XMLSchema'.
		self assert: self content typespace notNil]
! !

!XeSchemaDecReadTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSchemaDecReadTests.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
