"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeFuncTests subclass:#XeSchemaDefReadTests
	instanceVariableNames:'schema schemaString content'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSchemaDefReadTests methodsFor:'accessing'!

acessorKeys2001ComplexType
	^(Array 
		with: #varInt
		with: #varFloat
		with: #varString) collect: [ :each | XeQName null: each]
!

acessorKeysRefElement
	^(Array 
		with: #'street'
		with: #'comment'
		with: #'zip') collect: [ :each | XeQName null: each]
!

importSchemaString: xmlString
	self setTypespace.
	schemaString := xmlString.
	schema := super importSchemaString: xmlString
!

localName
	^content typeName localName
!

recreateContent
	| schemaWriter typeName|
	typeName := content typeName.
	schemaWriter := XeSchemaWriter targeting: typeName namespace schemaNS: typespace xsdNamespace.
	"schemaWriter initializeTargeting: typeName namespace."
	schemaWriter define: content.
	self importSchemaString: schemaWriter xml.
	self setContentFromSchema: typeName localName 

        
        
        
!

setContentFromSchema: aLocalName 
	content := schema defaultEncContent: aLocalName.
	self assert: content localName = aLocalName.
	self assert: content namespace = schema targetNS.
	self assert: (content isKindOf: XeContentAbstract)
! !

!XeSchemaDefReadTests methodsFor:'asserting'!

assertContent: operation
	operation value.
	self recreateContent.
	operation value.
        
        
! !

!XeSchemaDefReadTests methodsFor:'tests'!

test1999SimpleEnumeration
	self importSchemaString: XeTestSchemaImports new simpleDef1999Enum.
	self setContentFromSchema: 'EyeColor'.
	self assertContent: 
			[self assert: (content isKindOf: XeSimpleRestrictedContent).
			self assert: content baseContent localName = 'string'.
			self assert: content baseContent namespace = XeNamespace schema1999.
			self assert: content restrictions size = 1.
			self assert: (content restrictions first isKindOf: XeSchemaEnumeration).
			self assert: (content restrictions first validate: 'Green').
			self deny: (content restrictions first validate: 'Red')]
!

test1999TypedElements
	self importSchemaString: XeTestSchemaImports new complexDef1999Comp.
	self setContentFromSchema: 'PurchaseOrderType'.
	self assertContent: 
			[self assert: content childElements size = 2.
			self assert: content firstChildElement qName localName = 'shipTo'.
			self assert: content firstChildContent typeName localName = 'USAddress'.
			self assert: content firstChildContent typeName namespace = #'urn:someTarget'.
			self assert: content firstChildContent childElements size = 3.
			self assert: content firstChildContent firstChildElement  qName localName = 'name'.
			self assert: content firstChildContent firstChildContent typeName localName = 'string']
!

test2001ComplexExtension
	"http://msdn.microsoft.com/library/en-us/dnwebsrv/html/wsdlexplained.asp?frame=true#wsdlexplained_topic06"

	self importSchemaString: XeTestSchemaImports new complexDefExtension2.
	self setContentFromSchema: 'femalePerson'.
	self assertContent: 
			["self assert: content baseTypeName localName = 'PERSON'.
			self assert: content baseTypeName namespace = #someNamespace."
			self assert: content childElements size = 6.
			content childLocallyNamed: 'firstName'.
			content childLocallyNamed: 'favoriteLipstick']
!

test2001ComplexType
	self importSchemaString: XeTestSchemaImports new complexDefSS.
	self setContentFromSchema: 'MySoapStruct'.
	self assertContent: 
			[| childContent |
			self assert: (content isKindOf: XeComplexContent).
			self assert: (self symmetricDifferenceOf:content accessorKeys and: self acessorKeys2001ComplexType) isEmpty.
			childContent := (content childLocallyNamed: 'varFloat') content.
			self assert: childContent typeName = (XeQName xsd2001: 'float')]
!

test2001ComplexTypeChoice
	self importSchemaString: XeTestSchemaImports new complexDefChoice.
	self setContentFromSchema: 'MyChoiceStruct'.
	self assertContent: 
			[self assert: content childElements size = 1.
			self assert: (content firstChildElement  isKindOf: XeChoiceElement).
			self assert: content firstChildElement choices size = 3.
			self assert: content firstChildElement choices first localName = 'varInt'.
			self assert: content firstChildElement choices last localName = 'varString']
!

test2001ComplexTypeNestedChoice
	self importSchemaString: XeTestSchemaImports new complexDefChoiceNested.
	self setContentFromSchema: 'MyChoiceStruct'.
	self assertContent: 
			[self assert: content childElements size = 2.
			self assert: (content firstChildElement  isKindOf: XeChoiceElement).
			self assert: content firstChildElement  choices size = 2.
			self assert: content firstChildElement  choices first localName = 'varInt'.
			self assert: content firstChildElement  choices last localName = 'varFloat']
!

test2001ComplexTypeWithAnyAttribute
	self importSchemaString: XeTestSchemaImports new complexDefAttributeAny.
	self setContentFromSchema: 'PurchaseOrderType'.
	self assertContent: [self assert: (content isKindOf: XeComplexContent)]
!

test2001EmptyComplexType
	self importSchemaString: XeTestSchemaImports new complexDefEmpty.
	self setContentFromSchema: 'MySoapStruct'.
	self assertContent: 
			[self assert: (content isKindOf: XeComplexContent).
			self assert: content childElements isEmpty]
!

test2001EmptyComplexTypeShorthand
	self importSchemaString: XeTestSchemaImports new complexDefEmptyShorthand.
	self setContentFromSchema: 'MySoapStruct'.
	self assertContent: 
			[self assert: (content isKindOf: XeComplexContent).
			self assert: content childElements isEmpty]
!

test2001ExtensionOfXSDBase
	self importSchemaString: XeTestSchemaImports new complexDefExtBase.


	"Not yet implemented"
	self should: [schema defaultEncContent: 'USAddress'] raise: Error
!

test2001SimpleEnumeration
	self importSchemaString: XeTestSchemaImports new simpleDefEnum.
	self setContentFromSchema: 'EyeColor'.
	self assertContent: 
			[self assert: (content isKindOf: XeSimpleRestrictedContent).
			self assert: content baseContent typeName = (XeQName xsd2001: 'string').
			self assert: content restrictions size = 1.
			self assert: (content restrictions first isKindOf: XeSchemaEnumeration).
			self assert: (content restrictions first validate: 'Green').
			self deny: (content restrictions first validate: 'Red')]
!

test2001SimpleFacets
	self importSchemaString:  XeTestSchemaImports new simpleDefEnum2.
	self setContentFromSchema: 'myInteger'.
	self assertContent: 
			[self assert: (content isKindOf: XeSimpleRestrictedContent).
			self assert: content baseContent typeName = (XeQName xsd2001: 'integer').
			self assert: content restrictions size = 2.
			content restrictions do: 
					[:each | 
					self assert: (each isKindOf: XeSchemaFacet).
					self assert: (each validate: 20000).
					self assert: (each validate: 10000).
					self assert: (each validate: 99999)].
			self deny: (content restrictions allSatisfy: [:each | each validate: 300]).
			self deny: (content restrictions allSatisfy: [:each | each validate: 100000])]
!

test2001SprayFacets
	self importSchemaString: XeTestSchemaImports new simpleDefEnum3.
	self setContentFromSchema: 'foo'.
	self assertContent: 
			[self assert: (content isKindOf: XeSimpleRestrictedContent).
			self assert: content baseContent typeName = (XeQName xsd2001: 'string').
			self assert: content restrictions size = 2.
			content restrictions do: 
					[:each | 
					self assert: (each isKindOf: XeSchemaFacet).
					self assert: (each validate: '12345')].
			self deny: (content restrictions allSatisfy: [:each | each validate: '1234']).
			self deny: (content restrictions allSatisfy: [:each | each validate: '1234567'])]
! !

!XeSchemaDefReadTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSchemaDefReadTests.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
