"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingTests subclass:#XeSchemaFacetTest
	instanceVariableNames:'facet'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSchemaFacetTest methodsFor:'accessing'!

setFacetFrom: xmlString 
	facet := XeSchemaFacet fromNode: (self rootNodeFor: xmlString) firstChild
! !

!XeSchemaFacetTest methodsFor:'asserting'!

assertFacetXML: aString
	| node xml|
	node := self new2001Node: 'foo'.
	node addNSPrefix: 'xsd' to: XeNamespace schema2001.
	facet addToNode: node.
	self assertXML: (xml := node xml)
		isSameAs: 
('<foo
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema">%1</foo>' formatWith: aString)
! !

!XeSchemaFacetTest methodsFor:'tests'!

testFractionDigits
	self class isSkipAckFailures ifTrue: [^nil].
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <fractionDigits value="1"/>
</blah>'.

	self assert: (facet validate: '1s' asNumber).
	self assert: (facet validate: '1.1s' asNumber).
	self deny: (facet validate: '1.11s' asNumber).
	self assert: (facet validate: 1).
	self assert: (facet validate: 1.1).
	self deny: (facet validate: 1.11)
!

testLength
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<xsd:length value="2"/>
</blah>'.
	self assert: (facet validate: 'ab').
	self deny: (facet validate: 'a').
	self deny: (facet validate: 'abc')
!

testMaxExclusive
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<maxExclusive value="101"/>
</blah>'.
	self assert: (facet validate: 99).
	self assert: (facet validate: 100).
	self deny: (facet validate: 101)
!

testMaxInclusive
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <maxInclusive value="100"/>
</blah>'.
	self assert: (facet validate: 99).
	self assert: (facet validate: 100).
	self deny: (facet validate: 101)
!

testMaxLength
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<xsd:maxLength value="32"/>
</blah>'.
	self assert: (facet validate: 'abc').
	self deny: (facet validate: 'abcdddddddddddddddddddddddddddddddddddddddddddddddddddddddddd')
!

testMinExclusive
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<minExclusive value="99"/>
</blah>'.
	self deny: (facet validate: 99).
	self assert: (facet validate: 100).
	self assert: (facet validate: 101)
!

testMinInclusive
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <minInclusive value="100"/>
</blah>'.
	self deny: (facet validate: 99).
	self assert: (facet validate: 100).
	self assert: (facet validate: 101)
!

testMinLength
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<minLength value="1"/>
</blah>'.
	self assert: (facet validate: 'c').
	self deny: (facet validate: '')
!

testPattern
        self 
                setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<xsd:pattern value="\d*"/>
</blah>'.

        self assert: (facet validate: '1234').
        self assert: (facet validate: '123').
        self deny: (facet validate: 'a12').
!

testTotalDigits
	self 
		setFacetFrom: 
'<blah xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<totalDigits value="8"/>
</blah>'.

	self assert: (facet validate: '1234567s' asNumber).
	self assert: (facet validate: '12345678s' asNumber).
	self deny: (facet validate: '123456789s' asNumber).
	self assert: (facet validate: '123456.7s' asNumber).
	self assert: (facet validate: '123456.78s' asNumber).
	self deny: (facet validate: '123456.789s' asNumber).
	self assert: (facet validate: 1234567).
	self assert: (facet validate: 12345678).
	self deny: (facet validate: 123456789)
!

testWritingFractionDigits
	facet := XeSchemaFacet fractionDigits: '10'.
	self assertFacetXML: 
'<xsd:fractionDigits value="10"/>'
!

testWritingMaxExclusive
	facet := XeSchemaFacet maxExclusive: '100'.
	self assertFacetXML: 
'<xsd:maxExclusive value="100"/>'
!

testWritingMaxInclusive
	facet := XeSchemaFacet maxInclusive: '100'.
	self assertFacetXML: 
'<xsd:maxInclusive value="100"/>'
!

testWritingMaxLength
	facet := XeSchemaFacet maxLength: '10'.
	self assertFacetXML: 
'<xsd:maxLength value="10"/>'
!

testWritingMinExclusive
	facet := XeSchemaFacet minExclusive: '1'.
	self assertFacetXML: 
'<xsd:minExclusive value="1"/>'
!

testWritingMinInclusive
	facet := XeSchemaFacet minInclusive: '100'.
	self assertFacetXML: 
'<xsd:minInclusive value="100"/>'
!

testWritingMinLength
	facet := XeSchemaFacet minLength: '5'.
	self assertFacetXML: 
'<xsd:minLength value="5"/>'
!

testWritingPattern
        facet := XeSchemaFacet pattern: '\d*'.
        self assertFacetXML: 
'<xsd:pattern value="\d*"/>'
!

testWritingTotalDigits
	facet := XeSchemaFacet totalDigits: '15'.
	self assertFacetXML: 
'<xsd:totalDigits value="15"/>'
! !

!XeSchemaFacetTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSchemaFacetTest.st,v 1.5 2009/10/04 10:11:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSchemaFacetTest.st,v 1.5 2009/10/04 10:11:39 cg Exp $'
! !
