"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeContentTests subclass:#XeSimpleContentTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSimpleContentTest methodsFor:'accessing'!

setDoubleContent
	self be2001Typespace.
	content := (XePName xsd: 'double') xeContentWith: self encodingStyle
!

setStringContent
	self be2001Typespace.
	content := (XePName xsd: 'string') xeContentWith: self encodingStyle
! !

!XeSimpleContentTest methodsFor:'tests'!

testChildElements
	self setStringContent.
	self assert: content childElements isEmpty
!

testConvertAllValuesInlineType
	| node ans |
	self setDoubleContent.
	node := self 
				rootNodeFor: '<float xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xsi:type="xsd:double">NaN</float>'.
	self should: [content convertNode: node] raise: XeValueRangeError.
	self typespaceOptions beConvertAllValues.
	ans := content convertNode: node.
	self assert: (ans isKindOf: XeAnyValue).
	self assert: ans text = 'NaN'.
	self assert: ans typeName = (XeQName xsd1999: 'double')
!

testConvertAllValuesNoInlineType
	| node ans |
	self setDoubleContent.
	node := self rootNodeFor: '<float>NaN</float>'.
	self should: [content convertNode: node] raise: XeValueRangeError.
	self typespaceOptions beConvertAllValues.
	ans := content convertNode: node.
	self assert: (ans isKindOf: XeAnyValue).
	self assert: ans text = 'NaN'.
	self assert: ans typeName isNil
!

testReadISO8859
	| node ans |
	self setStringContent.
	self typespaceOptions beNotReadUTF8.
	node := self rootNodeFor: '<?xml version="1.0" encoding="iso-8859-1"?><test>ਢ</test>'.
	ans := content convertNode: node.
	self assert: ans = 'ਢ'.    "If this fails ... please contact swaring@ozemail.com.au ... what codepage/ locale are you in?"
	self typespaceOptions beReadUTF8.
	ans := content convertNode: node.
	self class isSkipAckFailures ifTrue: [^nil].
	self assert: (ans asOrderedCollection collect: [:each | each codePoint]) asArray 
				= #(194 175 195 160 194 168 194 162 194 165 195 162)
!

testReadUTF8
	| node ans hiString |
	self setStringContent.
	"normal"
	self typespaceOptions beNotReadUTF8.
	node := self rootNodeFor: '<test> a </test>'.
	ans := content convertNode: node.
	self assert: ans = ' a '.
	self typespaceOptions beReadUTF8.
	ans := content convertNode: node.
	self assert: ans = ' a '.
	"space"
	self typespaceOptions beNotReadUTF8.
	node := self rootNodeFor: '<test> </test>'.
	ans := content convertNode: node.
	self assert: ans = ' '.
	self typespaceOptions beReadUTF8.
	ans := content convertNode: node.
	self assert: ans = ' '.
	self class isSkipAckFailures ifTrue: [^nil].

	"utf8"
	self typespaceOptions beNotReadUTF8.
	node := self rootNodeFor: '<test>ЁЂЃЄ</test>'.
	ans := content convertNode: node.
	self assert: ans = '????'.
	self typespaceOptions beReadUTF8.
	ans := content convertNode: node.
	self assert: ans = 'ЁЂЃЄ'.
	"hiString"
	hiString := String 
				with: (Character codePoint: 130)
				with: (Character codePoint: 131)
				with: (Character codePoint: 132).
	"hiString"
	self typespaceOptions beNotReadUTF8.
	node := self rootNodeFor: '<test>' , hiString , '</test>'.
	ans := content convertNode: node.
	self assert: ans = '???'.
	self typespaceOptions beReadUTF8.
	ans := content convertNode: node.
	self assert: ans = ''
!

testSoapSpecial
	self setStringContent.
	self assert: content isSoapSpecialMultiRef.
	self setDoubleContent.
	self deny: content isSoapSpecialMultiRef.
	content := (XePName xsd: 'base64Binary') xeContentWith: self encodingStyle.
	self assert: content isSoapSpecialMultiRef
! !

!XeSimpleContentTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSimpleContentTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
