"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingCoreTests subclass:#XeSoapArrayReaderTest
	instanceVariableNames:'content'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeSoapArrayReaderTest methodsFor:'accessing'!

rootWrap: aString 
	content := XeSoapArrayNode on: (
		self rootNodeFor: (
'<root xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">%1</root>'  formatWith: aString) )firstChild
! !

!XeSoapArrayReaderTest methodsFor:'set up'!

setUp
	super setUp.
	content := XeSoapArrayNode new
! !

!XeSoapArrayReaderTest methodsFor:'tests'!

testExtractSize
	self assert: (content extractSize: '[3]') = 3.
	self assert: (content extractSize: '[][3]') = 3.
	self assert: (content extractSize: '[123]') = 123.
	self assert: (content extractSize: '[][123]') = 123.
	self should: [content extractSize: '[]'] raise: Error.
	self should: [content extractSize: '1]'] raise: Error.
	self should: [content extractSize: '[1'] raise: Error.
!

testHasArrayType
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"></array>'.
	self assert: content hasArrayType.
	self rootWrap: '<array SOAP-ENC:array="xsd:int[2]"></array>'.
	self deny: content hasArrayType.
	self rootWrap: '<array arrayType="xsd:int[2]"></array>'.
	self deny: content hasArrayType.
        
!

testHasArrayTypeWrongNS
	| node |
	node := self 
				rootNodeFor: 
'<root xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://NOTschemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
<array></array></root>'.
	self deny: (XeSoapArrayNode on: node firstChild) hasArrayType
!

testHasPositionNode
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"><i SOAP-ENC:position="[1]"/></array>'.
	self assert: content hasPositionNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"></array>'.
	self deny: content hasPositionNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"><i/></array>'.
	self deny: content hasPositionNode.
	self rootWrap:'<array SOAP-ENC:arrayType="xsd:int[2]"><i position="[1]"/></array>'.
	self deny: content hasPositionNode.
!

testIsKeyedNode
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"><i>3</i></array>'.
	self deny: content isKeyedNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="[2]"><i>3</i></array>'.
	self assert: content isKeyedNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" ><i SOAP-ENC:position="[2]">3</i></array>'.
	self assert: content  isKeyedNode
!

testIsOffsetNode
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="[1]"></array>'.
	self assert: content isOffsetNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="[0]"></array>'.
	self deny: content isOffsetNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" offset="[1]"></array>'.
	self deny: content isOffsetNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"></array>'.
	self deny: content isOffsetNode.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="[1"></array>'.
	self should: [content isOffsetNode] raise: Error.
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="123"></array>'.
	self should: [content isOffsetNode] raise: Error.
!

testIsOffsetNodeWrongNS
	| node |
	node := self 
				rootNodeFor: 
'<root xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://NOTschemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
<array SOAP-ENC:arrayType="xsd:int[2]" SOAP-ENC:offset="[1]"></array></root>'.

	self deny: (XeSoapArrayNode on: node firstChild) isOffsetNode
!

testItemTypeQName
	self rootWrap: '<array SOAP-ENC:arrayType="xsd:int[2]"></array>'.
	self assert: content itemTypeQName = ('int' ns: XeNamespace schema2001)
! !

!XeSoapArrayReaderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSoapArrayReaderTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
