"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeReadTests subclass:#XeSoapEncReadTest
	instanceVariableNames:'elements'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSoapEncReadTest methodsFor:'accessing'!

allElementsDo: operation 
	elements do: 
			[:each | 
			element := each.
			self decodeFirstChild.
			operation value]
!

allElementsRaise: anException 
	elements do: 
			[:each | 
			element := each.
			self should: [self decodeFirstChild] raise: anException ]
!

setFromSpecs: aCollection
	elements := (aCollection collect: [:each | each xeParticleWith: self encodingStyle]) 
				asOrderedCollection
! !

!XeSoapEncReadTest methodsFor:'tests'!

testAnyInlineTyped
	self be2001Typespace.
	elements := Array 
				with: ((XeElementSpec anyNamed: #blah) xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah  xsi:type="xsd:anyType">abc</blah>
</testStruct>'.
	self allElementsDo: [self assert: instance = 'abc']
!

testArrayConvertAllValues
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOffloat')
				with: (XeElementSpec named: 'blah' typeName: 'soap:Array')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<blah xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<param SOAP-ENC:arrayType="xsd:float[3]" xsi:type="SOAP-ENC:Array">
   <item xsi:type="xsd:float">1.1</item>
   <item xsi:type="xsd:float">INF</item>
   <item xsi:type="xsd:float">3.4</item>
</param>
</blah>'.
	self typespaceOptions beConvertAllValues.
	self allElementsDo: [
		self assert: instance first = 1.1.
		self assert: (instance second isKindOf: XeAnyValue).
		self assert: instance second text = 'INF'.
		self assert: instance second typeName localName = 'float'.
		self assert: instance second typeName namespace = #'http://www.w3.org/1999/XMLSchema']
!

testArrayConvertAllValuesNot
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOffloat')
				with: (XeElementSpec named: 'blah' typeName: 'soap:Array')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<blah xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<param SOAP-ENC:arrayType="xsd:float[3]" xsi:type="SOAP-ENC:Array">
   <item xsi:type="xsd:float">1.1</item>
   <item xsi:type="xsd:float">INF</item>
   <item xsi:type="xsd:float">3.4</item>
</param>
</blah>'.
	self typespaceOptions beNotConvertAllValues.
	self allElementsRaise: XeValueRangeError
!

testArrayNoArrayTable
	self be2001Typespace.
	element := (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring') 
				xeParticleWith: self encodingStyle.
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah type="SOAP-ENC:Array">
<item>3</item> 
</blah>
</testStruct>'.
	self should: [self decodeFirstChild] raise: Error
!

testArrayOfClass
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'd:ArrayOfSWXMLTestObject')
				with: (XeElementSpec named: 'blah' typeName: 'soap:Array')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<root>
<outputStructArray 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:d="http://www.dolphinharbor.org/spray/class"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
SOAP-ENC:arrayType="d:SWXMLTestObject[2]" >
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">5</x><y xsi:type="xsd:string">hello</y></item>
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">6</x><y xsi:type="xsd:string">goodbye</y></item>
</outputStructArray></root>'.
	self allElementsDo: [
		self assert: instance size =2.
		self assert: (instance first isKindOf: SWXMLTestObject).
		self assert: instance first x = 5.
		self assert: instance second y = 'goodbye']
!

testArrayOfClassTypedArray
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'd:ArrayOfSWXMLTestObject')
				with: (XeElementSpec named: 'blah' typeName: 'soap:Array')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<root>
<outputStructArray 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:d="http://www.dolphinharbor.org/spray/class"
xsi:type="SOAP-ENC:Array" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
SOAP-ENC:arrayType="d:SWXMLTestObject[2]" >
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">5</x><y xsi:type="xsd:string">hello</y></item>
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">6</x><y xsi:type="xsd:string">goodbye</y></item>
</outputStructArray></root>'.
	self allElementsDo: [
		self assert: instance size =2.
		self assert: (instance first isKindOf: SWXMLTestObject).
		self assert: instance first x = 5.
		self assert: instance second y = 'goodbye']
!

testArrayOfClassTypedArrayOf
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'd:ArrayOfSWXMLTestObject')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<root>
<outputStructArray 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:d="http://www.dolphinharbor.org/spray/class"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xsi:type="d:ArrayOfSWXMLTestObject" 
SOAP-ENC:arrayType="d:SWXMLTestObject[2]" >
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">5</x><y xsi:type="xsd:string">hello</y></item>
<item xsi:type="d:SWXMLTestObject"><x xsi:type="xsd:integer">6</x><y xsi:type="xsd:string">goodbye</y></item>
</outputStructArray></root>'.
	self allElementsDo: [
		self assert: instance size =2.
		self assert: (instance first isKindOf: SWXMLTestObject).
		self assert: instance first x = 5.
		self assert: instance second y = 'goodbye']
!

testArrayOffset
	self be2001Typespace.
	element := (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring') 
				xeParticleWith: self encodingStyle.
	xmlString := '<testStruct xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<SOAP-ENC:Array  SOAP-ENC:arrayType="xsd:string[5]" SOAP-ENC:offset="[2]">
<item>The third element</item>
<item>The fourth element</item>
</SOAP-ENC:Array>
</testStruct>'.
	self typespaceOptions beAllowKeyedArrays.
	self decodeFirstChild.
	self assert: (instance at: 3) = 'The third element'.
	self assert: (instance at: 4) = 'The fourth element'.
	self typespaceOptions beDisallowKeyedArrays.
	self should: [self decodeFirstChild] raise: Error
!

testArrayOffsetZero
	self be2001Typespace.
	element := (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring') 
				xeParticleWith: self encodingStyle.
	xmlString := '<testStruct xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"  xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<SOAP-ENC:Array  SOAP-ENC:arrayType="xsd:string[5]" SOAP-ENC:offset="[0]">
<item>The first element</item>
</SOAP-ENC:Array>
</testStruct>'.
	self typespaceOptions beAllowKeyedArrays.
	self decodeFirstChild.
	self assert: instance first = 'The first element'.
	self typespaceOptions beDisallowKeyedArrays.
	self decodeFirstChild.
	self assert: instance first = 'The first element'
!

testArrayPosition
	self be2001Typespace.
	self 
		setFromSpecs: (Array with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := '<testStruct xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"  xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<myFavoriteNumbers SOAP-ENC:arrayType="xsd:int[10]" type="SOAP-ENC:Array">
<number SOAP-ENC:position="[3]">3</number> 
</myFavoriteNumbers>
</testStruct>'.
	self typespaceOptions beAllowKeyedArrays.
	self allElementsDo: [self assert: (instance at: 4) = 3]
!

testArrayPositionIrregularFirst
	self be2001Typespace.
	self 
		setFromSpecs: (Array with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := '<testStruct xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"  xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<myFavoriteNumbers SOAP-ENC:arrayType="xsd:int[10]" type="SOAP-ENC:Array">
<number SOAP-ENC:position="[3]">4</number> 
<number>5</number> 
<number SOAP-ENC:position="[8]">9</number> 
</myFavoriteNumbers>
</testStruct>'.
	self typespaceOptions beAllowKeyedArrays.
	self allElementsDo: 
			[self assert: (instance at: 10) isNil.
			self assert: (instance at: 4) = 4.
			self assert: (instance at: 5) = 5.
			self assert: (instance at: 9) = 9]
!

testArrayPositionIrregularNotFirst
	self be2001Typespace.
	self 
		setFromSpecs: (Array with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := '<testStruct xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"  xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<myFavoriteNumbers SOAP-ENC:arrayType="xsd:int[10]" type="SOAP-ENC:Array">
<number>1</number> 
<number SOAP-ENC:position="[3]">4</number> 
<number SOAP-ENC:position="[8]">9</number> 
</myFavoriteNumbers>
</testStruct>'.
	self typespaceOptions beAllowKeyedArrays.
	self allElementsDo: 
			[self assert: (instance at: 10) isNil.
			self assert: (instance at: 4) = 4.
			self assert: (instance at: 9) = 9]
!

testArrayStringIDIndependent
	self be2001Typespace.
	elements := Array with: ((XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring') 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:string[3]">
 <item href="#ind"/>
</blah>
<ind id="ind">abc</ind>
</testStruct>'.
	self allElementsDo: [self assert: instance = #('abc')]
!

testArrayTypedArray
	self be2001Typespace.
	self setFromSpecs: (Array 
				with: (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring')
				with: (XeElementSpec named: 'blah' typeName: 'soap:Array')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := 
'<namesp1:echoStringArray 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"
xmlns:namesp1="http://soapinterop.org/">
<inputStringArray SOAP-ENC:arrayType="xsd:string[2]" xsi:type="SOAP-ENC:Array">
<c-gensym5 xsi:type="xsd:string">hello</c-gensym5>
<c-gensym5 xsi:type="xsd:string">goodbye</c-gensym5>
</inputStringArray>
</namesp1:echoStringArray>'.
	self typespaceOptions beAllowKeyedArrays.
	self allElementsDo: 
			[self assert: instance first = 'hello'.
			self assert: instance last = 'goodbye']
!

testArrayTypedMismatch
	self be2001Typespace.
	element := (XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfstring') 
				xeParticleWith: self encodingStyle.
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:string[1]" xsi:type="SOAP-ENC:NotArray">
<item>3</item> 
</blah>
</testStruct>'.
	self should: [self decodeFirstChild] raise: Error
!

testArrayWithNil
	self be2001Typespace.
	elements := Array with: ((XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint') 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:int[3]" xsi:type="SOAP-ENC:Array">
 <item xsi:type="xsd:int">1</item>
	<nil xsi:nil="true"/>
   <item xsi:type="xsd:int">2</item>

</blah>
</testStruct>'.
	self allElementsDo: [self assert: instance = #(1 nil 2)]
!

testArrayWithNull
	self be1999Typespace.
	elements := Array with: ((XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint') 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:int[3]" xsi:type="SOAP-ENC:Array">
 <item xsi:type="xsd:int">1</item>
	<nil xsi:null="1"/>
   <item xsi:type="xsd:int">2</item>

</blah>
</testStruct>'.
	self allElementsDo: [self assert: instance = #(1 nil 2)]
!

testComplexWildAny
	self be2001Typespace.
	self importSchemaString: XeTestSchemaImports new complexDefAny1.
	elements := Array with: ((XeElementSpec named: #blah typeName: (self blahQ: 'MyType')) 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema">
<blah>
 <varInt xsi:type="xsd:int">123</varInt>
</blah>
</testStruct>'.
	self allElementsDo: [self assert: (instance at: #varInt) = 123]
!

testComplexWildAnySchema
	self be2001LiteralTypespace.
	self importSchemaString: XeTestSchemaImports new complexDefAnySchema.
	elements := Array with: ((XeElementSpec named: #blah typeName: (self blahQ: 'MyType')) 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<blah>
  <xsd:schema targetNamespace="urn:foo"><xsd:complexType name="foo"/></xsd:schema>
 <varInt xsi:type="xsd:int">123</varInt>
</blah>
</testStruct>'.
	self allElementsDo: 
			[self assert: (instance at: #varInt) = '123'.
			self assert: ((instance at: (XeQName xsd2001: 'schema')) isKindOf: YAXO::XeNode)]
!

testIDFloat
	self be2001Typespace.
	elements := Array with: ((XeElementSpec named: 'blah' typeName: 'soap:Array') 
						xeParticleWith: self encodingStyle)
				with: ((XeElementSpec named: 'blah' typeName: 'soapE:ArrayOffloat') 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:float[1]">
 <item href="#ind"/>
</blah>
<ind id="ind" xsi:type="xsd:float">3.14</ind>
</testStruct>'.
	self allElementsDo: [self assert: instance = #(3.14)]
!

testIDInteger
	self be2001Typespace.
	elements := Array with: ((XeElementSpec named: 'blah' typeName: 'soap:Array') 
						xeParticleWith: self encodingStyle)
				with: ((XeElementSpec named: 'blah' typeName: 'soapE:ArrayOfint') 
						xeParticleWith: self encodingStyle).
	xmlString := '<testStruct
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" >
<blah SOAP-ENC:arrayType="xsd:int[1]">
 <item href="#ind"/>
</blah>
<ind id="ind" xsi:type="xsd:int">123</ind>
</testStruct>'.
	self allElementsDo: [self assert: instance = #(123)]
!

testMultiDim
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent ).
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]">
<item>r1c1</item> 
<item>r1c2</item> 
<item>r1c3</item> 
<item>r2c1</item> 
<item>r2c2</item> 
<item>r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (1@1)) = 'r1c1'.
		self assert: (instance at: (1@2)) = 'r1c2'.
		self assert: (instance at: (1@3)) = 'r1c3'.
		self assert: (instance at: (2@1)) = 'r2c1'.
		self assert: (instance at: (2@2)) = 'r2c2'.
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testMultiDimOffsetOneOne
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent) .
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]" SOAP-ENC:offset="[1,1]">
<item>r2c2</item> 
<item>r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (2@2)) = 'r2c2'.
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testMultiDimOffsetOneOnePos
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent) .
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]" SOAP-ENC:offset="[1,1]">
<item SOAP-ENC:position="[1,2]">r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testMultiDimOffsetOneZero
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent) .
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]" SOAP-ENC:offset="[1,0]">
<item>r2c1</item> 
<item>r2c2</item> 
<item>r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (2@1)) = 'r2c1'.
		self assert: (instance at: (2@2)) = 'r2c2'.
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testMultiDimOffsetZeroZero
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent ).
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]" SOAP-ENC:offset="[0,0]">
<item>r1c1</item> 
<item>r1c2</item> 
<item>r1c3</item> 
<item>r2c1</item> 
<item>r2c2</item> 
<item>r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (1@1)) = 'r1c1'.
		self assert: (instance at: (1@2)) = 'r1c2'.
		self assert: (instance at: (1@3)) = 'r1c3'.
		self assert: (instance at: (2@1)) = 'r2c1'.
		self assert: (instance at: (2@2)) = 'r2c2'.
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testMultiDimPosition
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapMDArrayContent ).
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:string[2,3]">
<item>r1c1</item> 
<item SOAP-ENC:position="[0,2]" >r1c3</item> 
<item>r2c1</item> 
<item>r2c2</item> 
<item>r2c3</item> 
</SOAP-ENC:Array>
</blah>'.
	self allElementsDo: [
		self assert: (instance at: (1@1)) = 'r1c1'.
		self deny: (instance includesKey: (1@2)).
		self assert: (instance at: (1@3)) = 'r1c3'.
		self assert: (instance at: (2@1)) = 'r2c1'.
		self assert: (instance at: (2@2)) = 'r2c2'.
		self assert: (instance at: (2@3)) = 'r2c3'.]


!

testReferenceMissing
	self be2001Typespace.
	self setFromSpecs: (Array with: (XeElementSpec named: 'blah' xsd: 'string')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := '<Body xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<Return href="#1" xmlns=""/>
</Body>'.
	self allElementsRaise: Error
!

testRestriction
	self be2001Typespace.
	self setFromSpecs: (Array with: (XeElementSpec named: 'blah' xsd: 'float')
				with: (XeElementSpec anyNamed: 'blah')).
	elements add: self getSimpleChoiceElement.
	xmlString := '<root
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<blah xsi:type="xsd:float">3.14</blah>
</root>'.
	self allElementsDo: [self assert: instance = 3.14]
!

testSimple
	self be2001Typespace.
	self setFromSpecs: (Array with: (XeElementSpec named: 'blah' xsd: 'string')
				with: (XeElementSpec anyNamed: 'blah')).
	xmlString := '<root
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<x xsi:type="xsd:string">abc</x>
</root>'.
	self allElementsDo: [self assert: instance = 'abc']
!

testSimple2
	self be2001Typespace.
	self setFromSpecs: (Array with: (XeElementSpec named: 'blah' xsd: 'integer')
				with: (XeElementSpec anyNamed: 'blah')).
	elements add: self getSimpleChoiceElement.
	xmlString := '<root
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<x xsi:type="xsd:integer">123</x>
</root>'.
	self allElementsDo: [self assert: instance = 123]
!

testSoapStruct
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent).
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:ns1="urn:mySample">
<echo xsi:type="ns1:SOAPStruct">
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
 <varFloat xsi:type="xsd:float">3.14</varFloat>
</echo>
</testStruct>'.
	self allElementsDo: [self assert: instance = self getSoapStruct]
!

testSoapStructConvertAllValues
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent).
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:ns1="urn:mySample">
<echo xsi:type="ns1:SOAPStruct">
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
 <varFloat xsi:type="xsd:float">INF</varFloat>
</echo>
</testStruct>'.
	self typespaceOptions beConvertAllValues.
	self allElementsDo: 
			[self assert: (instance at: #varInt) = 123.
			self assert: (instance at: #varString) = 'abc'.
			self assert: ((instance at: #varFloat) isKindOf: XeAnyValue).
			self assert: (instance at: #varFloat) text = 'INF'.
			self assert: (instance at: #varFloat) typeName localName = 'float'.
			self assert: (instance at: #varFloat) typeName namespace = #'http://www.w3.org/1999/XMLSchema']
!

testSoapStructConvertAllValuesNot
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent).
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:ns1="urn:mySample">
<echo xsi:type="ns1:SOAPStruct">
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
 <varFloat xsi:type="xsd:float">INF</varFloat>
</echo>
</testStruct>'.
	self typespaceOptions beNotConvertAllValues.
	self allElementsRaise: XeValueRangeError
!

testSoapStructGeneric
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent)
				with: (self encodingStyle anyElementNamed: 'blah').
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema">
<echo>
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
 <varFloat xsi:type="xsd:float">3.14</varFloat>
</echo>
</testStruct>'.
	self allElementsDo: [self assert: instance = self getSoapStruct]
!

testSoapStructMissing
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent).
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:ns1="urn:mySample">
<echo xsi:type="ns1:SOAPStruct">
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
</echo>
</testStruct>'.
	self allElementsDo: 
			[self assert: (instance at: #varString) = 'abc'.
			self assert: (instance at: #varFloat) isNil]
!

testSoapStructNotTyped
	self be2001Typespace.
	elements := Array with: (self getAnonElement: #blah content: self getSoapStructContent).
	xmlString := 
'<testStruct>
<echo>
 <varString>abc</varString>
 <varInt>123</varInt>
 <varFloat>3.14</varFloat>
</echo>
</testStruct>'.
	self allElementsDo: [self assert: instance = self getSoapStruct]
!

testSoapStructUnexpected
	self be2001Typespace.
	element := self getAnonElement: #blah content: self getSoapStructContent.
	xmlString := 
'<testStruct
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:ns1="urn:mySample">
<echo xsi:type="ns1:SOAPStruct">
 <varString xsi:type="xsd:string">abc</varString>
 <varInt xsi:type="xsd:int">123</varInt>
 <varFloat xsi:type="xsd:float">3.14</varFloat>
 <varOops xsi:type="xsd:float">3.14</varOops >
</echo>
</testStruct>'.
	self should: [self decodeFirstChild] raise: XeStrictError.
	[self decodeFirstChild] on: XeStrictError do: [ :e | e resume].
	self assert: instance = self getSoapStruct
! !

!XeSoapEncReadTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSoapEncReadTest.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
