"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingStyleTests subclass:#XeSoapEncodingStyleTest
	instanceVariableNames:'content'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSoapEncodingStyleTest methodsFor:'accessing'!

beSoapEnc1999
	| typespace |
	typespace := self get1999Typespace.
	typespace makeSoapEncDefaultStyle.
	encodingStyle := typespace soapEncStyle
!

beSoapEnc2001
	| typespace |
	typespace := self get2001Typespace.
	typespace makeSoapEncDefaultStyle.
	encodingStyle := typespace soapEncStyle
!

getPMCircStruct
	| cStruct |
	cStruct := XeQStruct new.
	cStruct
		at: #varA put: 123;
		at: #varB put: 'abc';
		at: #varC put: cStruct.
	^cStruct
!

setSprayContent: sprayContent
	| typeName baseTypeName |
	content := encodingStyle 
				complexSpray: (typeName := self blahQ: 'abc')
				restricting: (baseTypeName := self blahQ: 'def')
				instance: sprayContent.
	self assert: (content isKindOf: XeComplexContent).
	self assert: content typeName = typeName
	"self assert: content baseTypeName  = baseTypeName "
! !

!XeSoapEncodingStyleTest methodsFor:'set up'!

setUp
	super setUp.
	self beSoapEnc2001
! !

!XeSoapEncodingStyleTest methodsFor:'tests'!

testAnyElementNamed
	| element |
	element := encodingStyle anyElementNamed: 'blah'.
	self assert: (element qName localName = 'blah').
	self assert: (element content isKindOf: XeAnyContent).
!

testAnyTypeName
	self assert: encodingStyle anyTypeName = (XeQName xsd2001: 'anyType').
	self beSoapEnc1999.
	self assert: encodingStyle anyTypeName = (XeQName xsd1999: 'ur-type').
!

testArrayContentClass
	self assert: (encodingStyle arrayContentClassFor: '[]') = XeSoapArrayContent.
	self assert: (encodingStyle arrayContentClassFor: '[3]') = XeSoapArrayContent.
	self assert: (encodingStyle arrayContentClassFor: '[][2]') = XeSoapArrayContent.
	self assert: (encodingStyle arrayContentClassFor: '[2,3]') = XeSoapMultiDimContent
!

testArraySoapEnc
	| typeName|
	content := encodingStyle 
		arraySoapEnc: (typeName := (self blahQ: 'abc')).
	self assert: (content isKindOf: XeSoapArrayContent).
	self assert: content typeName = typeName.
	self assert: content baseTypeName = (XeQName soapEnc: 'Array').
                 
!

testBasicElementForNode
	| xmlString ns doc s |
	ns := self get1999Typespace.
	"===declaration"
	xmlString := 
'<blah 
xmlns:soap="http://schemas.xmlsoap.org/soap/encoding/"
xmlns="urn:notMe">
<soap:int> 5</soap:int>
</blah>'.
	doc := self docNodeFor: xmlString.
	s := ns defaultEncodingStyle elementForNode: doc documentElement firstChild.
	self assert: s content typeName localName = 'int'.


	"===declaration"
	xmlString := 
'<blah 
xmlns="http://schemas.xmlsoap.org/soap/encoding/">
<int> 5</int>
</blah>'.
	doc := self docNodeFor: xmlString.
	s := ns defaultEncodingStyle elementForNode: doc documentElement firstChild.
	self assert: s content typeName localName = 'int'.

	"===definition"
	xmlString := 
'<blah 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema">
<foo xsi:type="xsd:int"> 5</foo>
</blah>'.
	doc := self docNodeFor: xmlString.
	s := ns defaultEncodingStyle elementForNode: doc documentElement firstChild.
	self assert: s content typeName localName = 'int'.


	"=========Guess"
	xmlString := 
'<blah> 
<foo> 5</foo>
</blah>'.
	doc := self docNodeFor: xmlString.
	s := ns defaultEncodingStyle elementForNode: doc documentElement firstChild.
	self assert: (s content isKindOf: XeGuessContent)
!

testComplexClass
	| typeName |
	content := encodingStyle complexSpray: (typeName := XeQName sprayClass: SWXMLTestObject)
				factory: (encodingStyle schemaTargeting: XeNamespace sprayClass) factory.
	self assert: (content isKindOf: XeComplexContent).
	self assert: content typeName = typeName
!

testComplexClassArray
	| typeName |
	content := encodingStyle 
				complexSpray: (typeName := XeQName arrayOfSprayClass: SWXMLTestObject)
				factory: (encodingStyle schemaTargeting: XeNamespace sprayClass) factory.
	self assert: (content isKindOf: XeSoapArrayContent).
	self assert: content typeName = typeName
!

testComplexElements
	| typeName |
	content := encodingStyle complex: (typeName := self qAbc) elements: #().
	self assert: (content isKindOf: XeComplexContent).
	self assert: content typeName = typeName.
	self assert: content baseTypeName isNil
!

testComplexRPC
	| typeName |
	content := encodingStyle complexRPC: (typeName := self qAbc) elements: #().
	self assert: (content isKindOf: XeSoapRPCContent).
	self assert: content typeName = typeName
!

testComplexRestrictionSpray
	| sprayContent |
	sprayContent := XeQStruct new.
	sprayContent
		at: #x put: 5;
		at: #y put: 'hello'.
	self setSprayContent: sprayContent.
	self assert: content childElements size = 2.
	self assert: content childElements second localName = 'y'
!

testComplexRestrictionSprayNil
	| sprayContent |
	sprayContent := XeQStruct new.
	sprayContent
		at: #x put: 5;
		at: #y put: nil.
	self setSprayContent: sprayContent.
	self assert: content childElements size = 2.
	self assert: content childElements second localName = 'y'.
	self assert: (content childElements second isKindOf: XeAnyTypeElement)
!

testConfigureComplexElement
	self schemaImport: (XeTestSchemaImport for: #complexSOAPStruct).
	content := self qSOAPStruct xeContentWith: encodingStyle.
	content childElements do: 
			[:each | 
			self deny: each isFormQualified.
			self assert: each qName isNullNamespace].
	self beSoapEnc2001.
	self schemaImport: (XeTestSchemaImport for: #complexSOAPStruct) beEFDUq.
	content := self qSOAPStruct xeContentWith: encodingStyle.
	content childElements do: 
			[:each | 
			self deny: each isFormQualified.
			self assert: each qName isNullNamespace].
	self beSoapEnc2001.
	self schemaImport: (XeTestSchemaImport for: #complexSOAPStruct) beEFDQ.
	content := self qSOAPStruct xeContentWith: encodingStyle.
	content childElements do: 
			[:each | 
			self deny: each isFormQualified.
			self assert: each qName isNullNamespace]
!

testMismatchPolicy
	self assert: (encodingStyle mismatchPolicy isKindOf: XeSoapTypeMismatchPolicy).
	self assert: (encodingStyle mismatchPolicy ~~ encodingStyle mismatchPolicy).
!

testNamespace
	self assert: encodingStyle encodingNamespace = XeNamespace soapEnc
!

testNewNode
	self beSoapEnc2001.
	self assertXML: (encodingStyle newNode: 'blah') xml isSameAs: 
'<blah xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>'.
	self beSoapEnc1999.
	self assertXML: (encodingStyle newNode: 'blah') xml isSameAs: 
'<blah xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"/>'.
!

testSimple
	| converter typeName |
	content := encodingStyle simple: (typeName := self qAbc)
				converter: (converter := XeTextConverter string).
	self assert: (content isKindOf: XeSimpleContent).
	self assert: content typeName = typeName.
	self assert: content baseTypeName = (XeQName xsd2001: 'anyType')
!

testSimpleBaseConverter
	| converter typeName baseTypeName |
	content := encodingStyle 
				simple: (typeName := self qAbc)
				base: (baseTypeName := self blahQ: 'def')
				converter: (converter := XeTextConverter string).
	self assert: (content isKindOf: XeSimpleContent).
	self assert: content typeName = typeName.
	self assert: content baseTypeName = baseTypeName
!

testSimpleRestricting
	| typeName baseTypeName |
	content := encodingStyle 
				simple: (typeName := self qAbc)
				restricting: (baseTypeName := XeQName xsd1999: 'string')
				with: #().
	self assert: (content isKindOf: XeSimpleRestrictedContent).
	self assert: content typeName = typeName.
	self assert: content baseTypeName = baseTypeName
!

testSplitArrayTypeName
	| data |
	data := OrderedCollection new.
	data
		add: (Array
				with: (self blahQ: 'string[2]')
				with: (self blahQ: 'string')
				with: '2');
		add: (Array
				with: (self blahQ: 'string[123]')
				with: (self blahQ: 'string')
				with: '123');
		add: (Array
				with: (self blahQ: 'string[1,3]')
				with: (self blahQ: 'string' )
				with: '1,3').

	data do: [ :each |
		self assert: (encodingStyle splitArrayTypeName: each first) first = each second.
		self assert: (encodingStyle splitArrayTypeName: each first) second = each third].

        
                

!

testSplitArrayTypeNameEx
	| data |
	data := OrderedCollection new.
	data
		add: (self blahQ: 'string[1,2,3]').
	data do: [ :each |
		self should: [encodingStyle splitArrayTypeName: each] raise: Error ].

        
                

!

testWsdlArray
	| typeName |
	content := encodingStyle arrayWsdl: (typeName := (self blahQ: 'ArrayOfstring')).
	self assert: (content isKindOf: XeSoapArrayContent).
	self assert: content typeName = typeName
!

testXmlContentWithCirc
	| xeContent struct |
	struct := self getPMCircStruct.
	xeContent := struct xeContentWith: encodingStyle.
	self assert: (xeContent isKindOf: XeComplexContent).
	self assert: xeContent childElements size = 3
! !

!XeSoapEncodingStyleTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSoapEncodingStyleTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
