"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeFuncTests subclass:#XeSoapInstanceReadTest
	instanceVariableNames:'xmlString instance isFirstChild'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSoapInstanceReadTest methodsFor:'accessing'!

beFirstChild
	isFirstChild := true.
!

beSpecTypespace
	self be1999Typespace.
	self importSpecSchemaString.
	typespace addPrefix: 'e' namespace: #'urn:testSpace'.

!

decodeInstanceNode
	| decodeNode |
	decodeNode := self rootNodeFor: xmlString.
	isFirstChild ifTrue: [decodeNode := decodeNode firstChild].
	instance := self encodingStyle decodeInstanceNode: decodeNode
!

importSpecSchemaString
	self importSchemaString:
'<schema 
xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:tns="urn:testSpace"
targetNamespace="urn:testSpace">

<element name="Book" type="tns:Book"/>
<complexType name="Book">
<!!-- Either the following group must occur or else the
href attribute must appear, but not both. -->
<sequence minOccurs="0" maxOccurs="1">
<element name="title" type="xsd:string"/>
<element name="firstauthor" type="tns:Person"/>
<element name="secondauthor" type="tns:Person"/>
</sequence>
<attribute name="href" type="uriReference"/>
<attribute name="id" type="ID"/>
<anyAttribute namespace="##other"/>
</complexType>

<element name="Person" base="tns:Person"/>
<complexType name="Person">
<!!-- Either the following group must occur or else the
href attribute must appear, but not both. -->
<sequence minOccurs="0" maxOccurs="1">
<element name="name" type="xsd:string"/>
<element name="address" type="tns:Address"/>
</sequence>
<attribute name="href" type="uriReference"/>
<attribute name="id" type="ID"/>
<anyAttribute namespace="##other"/>
</complexType>

<element name="Address" base="tns:Address"/>
<complexType name="Address">
<!!-- Either the following group must occur or else the
href attribute must appear, but not both. -->
<sequence minOccurs="0" maxOccurs="1">
<element name="street" type="xsd:string"/>
<element name="city" type="xsd:string"/>
<element name="state" type="xsd:string"/>
</sequence>
<attribute name="href" type="uriReference"/>
<attribute name="id" type="ID"/>
<anyAttribute namespace="##other"/>
</complexType>


</schema>'


! !

!XeSoapInstanceReadTest methodsFor:'set up'!

setUp
	super setUp.
	isFirstChild := false.
! !

!XeSoapInstanceReadTest methodsFor:'tests'!

testArrayDeclaredWithAnyArrayType
	self be2001Typespace.
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:anyType[2]">
<SOAP-ENC:int>3</SOAP-ENC:int>
<SOAP-ENC:int>4</SOAP-ENC:int>
</SOAP-ENC:Array>
</blah>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: instance  = #(3 4).
!

testArrayDeclaredWithArrayType
	self be2001Typespace.
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/1999/XMLSchema" 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:int[2]">
<SOAP-ENC:int>3</SOAP-ENC:int>
<SOAP-ENC:int>4</SOAP-ENC:int>
</SOAP-ENC:Array>
</blah>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: instance  = #(3 4).
!

testArrayEmpty
	self be2001Typespace.
	xmlString := 
'<blah xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
<param SOAP-ENC:arrayType="xsd:int[0]" xsi:type="SOAP-ENC:Array"/>
</blah>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: instance  = #().
!

testArrayOfArrays
	self be2001Typespace.
	xmlString := 
'<allDataResponse xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" >
<returnValue SOAP-ENC:arrayType="xsd:anyType[3]">
  <item SOAP-ENC:arrayType="xsd:anyType[2]">
    <item xsi:type="xsd:integer">10</item><item xsi:type="xsd:integer">20</item>
  </item>
  <item SOAP-ENC:arrayType="xsd:anyType[2]">
    <item xsi:type="xsd:integer">50</item><item xsi:type="xsd:integer">50</item>
  </item>
  <item SOAP-ENC:arrayType="xsd:anyType[2]">
    <item xsi:type="xsd:integer">100</item><item xsi:type="xsd:integer">50</item>
  </item>
</returnValue></allDataResponse>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: instance  = #(#(10 20) #(50 50) #(100 50)).
!

testArrayReferenced
	self be2001Typespace.
	xmlString := 
'<Body xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema"
xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/">

<SOAP-ENC:Array SOAP-ENC:arrayType="xsd:ur-type[3]">
<item href="#array-1"/>
<item href="#array-1"/>
<item href="#array-2"/>
</SOAP-ENC:Array>
<SOAP-ENC:Array id="array-1" SOAP-ENC:arrayType="xsd:string[3]">
<item>r1c1</item>
<item>r1c2</item>
<item>r1c3</item>
</SOAP-ENC:Array>
<SOAP-ENC:Array id="array-2" SOAP-ENC:arrayType="xsd:string[2]">
<item>r2c1</item>
<item>r2c2</item>
</SOAP-ENC:Array>
</Body>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: instance size = 3.
	self assert: instance first == instance second.
	self assert: instance first first = 'r1c1'.
	self assert: instance third first = 'r2c1'

!

testArrayReferenced2
	self be2001Typespace.
	xmlString := 
'<soap:Body 
xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
		<echoIntegerArrayResponse>
			<Return href="#1" xmlns=""/>
		</echoIntegerArrayResponse>
		<soapenc:Array id="1" soapenc:arrayType="xsd:int[3]">
			<Item>1</Item>
			<Item>2</Item>
			<Item>3</Item>
		</soapenc:Array>
	</soap:Body>'.

	self beFirstChild.
	self decodeInstanceNode.
	self assert: (instance at: #Return) = #(1 2 3).
!

testCircular
	self be2001Typespace.
	xmlString := 
'<?xml version="1.0"?>
<root xmlns:soap="http://schemas.xmlsoap.org/soap/encoding/" 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
<ssw href="#id1"/>
<id1 id="id1">
<owningObject href="#id2"/>
</id1>
<id2 id="id2" >
<collection soap:arrayType="xsd:anyType[2]">
<item href="#id1"/>
<item >
<owningObject href="#id2"/>
<integer xsi:type="xsd:integer">10</integer>
</item>
</collection>
</id2>
</root>'.
	self decodeInstanceNode.
	self assert: ((instance at: #ssw) at: #owningObject) 
				== ((((instance at: #ssw) at: #owningObject) at: #collection) second at: #owningObject).
	self assert: (instance at: #ssw) == (((instance at: #ssw) at: #owningObject) at: #collection) first

!

testComplex
	self be2001Typespace.
	xmlString := 
'<someStruct>
<varInt> 3 </varInt>
</someStruct>'.
	self decodeInstanceNode.
	self assert: (instance at: #varInt) = ' 3 '
!

testComplexComplex
	self be2001Typespace.
	xmlString := 
'<someStruct>
<innerStruct>
<varInt> 3 </varInt>
</innerStruct>
<varString> blah </varString>
</someStruct>'.
	self decodeInstanceNode.
	self assert: (instance at: #varString) = ' blah '.
	self assert: ((instance at: #innerStruct) at: #varInt) = ' 3 '

!

testComplexComplexTyped
	self be2001Typespace.
	xmlString := 
'<SOAP-ENV:Body xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
		<easyStructTest>
			<param1>
				<curly xsi:type="xsd:int">-5</curly>
				<larry xsi:type="xsd:int">35</larry>
				<moe xsi:type="xsd:int">14</moe>
				</param1>
			</easyStructTest>
		</SOAP-ENV:Body>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: ((instance at: #param1) at: #curly) = -5


!

testGuessFromArrayType
	self be2001Typespace.
	xmlString := 
'<myArray xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/encoding/" soap:arrayType="xsd:int[2]">
<i>1</i>
<i>2</i>
</myArray>'.
	self decodeInstanceNode.
	self assert: (instance = #('1' '2') or: [instance = #(1 2)])
!

testSimple
	self be2001Typespace.
	xmlString := 
'<varInt> 3 </varInt>'.
	self decodeInstanceNode.
	self assert: instance = ' 3 '
!

testSimpleSoapDeclared
	self be2001Typespace.
	xmlString := 
'<SOAP-ENC:integer xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/">3</SOAP-ENC:integer>'.
	self decodeInstanceNode.
	self assert: instance = 3
!

testSimpleTyped
	self be2001Typespace.
	xmlString := 
'<varInt xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xsi:type="xsd:integer">3</varInt>'.
	self decodeInstanceNode.
	self assert: instance = 3
!

testStringReferenced
	self be2001Typespace.
	xmlString := 
'<Body>
<Return>
  <greeting id="String-0">Hello</greeting>
  <salutation href="#String-0"/>
</Return>
</Body>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: (instance at: #greeting) == (instance at: #salutation).
	self assert: (instance at: #greeting) = 'Hello'


!

testStructReferenced
	self be2001Typespace.
	xmlString := 
'<Body xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<Return>
	<struct1 href="#1"/>
	<struct2 href="#1"/>
</Return>
<SOAPStruct id="1" >
	<varInt >123</varInt>
	<varString >abc</varString>
	<varFloat >3.14</varFloat>
</SOAPStruct>
</Body>'.
	self beFirstChild.
	self decodeInstanceNode.
	self assert: (instance at: #struct1) == (instance at: #struct2)

!

testStructsSpec
	self beSpecTypespace.
	xmlString := 
'<blah 
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:e="urn:testSpace"
xmlns:n="urn:testSpace"
xmlns:m="urn:testSpace">
<e:Book>
<title>My Life and Work</title>
<firstauthor href="#Person-1"/>
<secondauthor href="#Person-2"/>
</e:Book>
<e:Person id="Person-1">
<name>Henry Ford</name>
<address xsi:type="e:Address">
<street>my Rd</street>
<city>my city</city>
<state>Sorth Carolina</state>
</address>
</e:Person>
<e:Person id="Person-2">
<name>Samuel Crowther</name>
<address xsi:type="e:Address">
<street>Martin Luther King Rd</street>
<city>Raleigh</city>
<state>North Carolina</state>
</address>
</e:Person>

</blah>'.

	self beFirstChild.
	self decodeInstanceNode.
	self assert: (instance at: #title) = 'My Life and Work'.
	self assert: ((instance at: #firstauthor) at: #name) = 'Henry Ford'.
	self assert: (((instance at: #firstauthor) at: #address) at: #street) = 'my Rd'.
	self assert: ((instance at: #secondauthor) at: #name) = 'Samuel Crowther'.
	self assert: (((instance at: #secondauthor) at: #address) at: #street) = 'Martin Luther King Rd'


! !

!XeSoapInstanceReadTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSoapInstanceReadTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
