"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeReadTests subclass:#XeSoapRPCStructTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSoapRPCStructTest methodsFor:'accessing'!

setEmptyRPC
	| content |
	self be2001Typespace.
	content := (XeSoapRPCContent new)
				typeName: (XeQName fromString: 'blah');
				childElements: OrderedCollection new;
				encodingStyle: self encodingStyle;
				convertClass: XeQStruct;
				yourself.
	element := self getAnonElement: #blah content: content
!

setMultipleRPC
	| content specs |
	self be2001Typespace.
	specs := OrderedCollection new.
	specs
		add: (XeElementSpec named: 'x' xsd: 'integer');
		add: (XeElementSpec named: 'y' xsd: 'string').
	content := self encodingStyle complexRPC: (XeQName fromString: 'blah') specs: specs.
	"content := XeSoapRPCContent 
				typeName: (XeQName fromString: 'blah')
				childSpecs: specs
				encodingStyle: self encodingStyle."
	element := self getAnonElement: #blah content: content
!

setSingleRPC
	| content |
	self be2001Typespace.
	"content := XeSoapRPCContent 
				typeName: (XeQName fromString: 'blah')
				childSpecs: (Array with: (XeElementSpec named: 'x' xsdTyped: 'integer'))
				encodingStyle: self encodingStyle."
	content := self encodingStyle complexRPC: (XeQName fromString: 'blah')
				specs: (Array with: (XeElementSpec named: 'x' xsd: 'integer')).
	element := self getAnonElement: #blah content: content
! !

!XeSoapRPCStructTest methodsFor:'tests'!

testEmpty
	self setEmptyRPC.
	xmlString :=
'<doesntMatter>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: instance isEmpty.

	element content isNoReturnPart: true.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: instance isEmpty
!

testMultiple
	self setMultipleRPC.
	xmlString :=
'<doesntMatter>
<x>1</x>
<y> goodbye </y>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.
	self assert: (instance at: #y) = ' goodbye '.

        
!

testMultipleIgnoreFirst
	self setMultipleRPC.
	xmlString :=
'<doesntMatter>
<notX>1</notX>
<y> goodbye </y>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.
	self assert: (instance at: #y) = ' goodbye '.

        
!

testMultipleNoRetPartDifferent
	self setMultipleRPC.
	element content isNoReturnPart: true.
	xmlString :=
'<doesntMatter>
<notX>1</notX>
<y> goodbye </y>
</doesntMatter>'.
	self should: [self decodeRoot] raise: Error
        

!

testMultipleNoRetPartSame
	self setMultipleRPC.
	element content isNoReturnPart: true.
	xmlString :=
'<doesntMatter>
<x>1</x>
<y> goodbye </y>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.
	self assert: (instance at: #y) = ' goodbye '.
!

testSingle
	self setSingleRPC.
	xmlString :=    
'<doesntMatter>
<x>1</x>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.


!

testSingleIgnoreFirst
	self setSingleRPC.
	xmlString :=    
'<doesntMatter>
<notX>1</notX>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.


!

testSingleNoRetPartDifferent
	self setSingleRPC.
	element content isNoReturnPart: true.
	xmlString :=
'<doesntMatter>
<notX>1</notX>
</doesntMatter>'.
	self should: [self decodeRoot] raise: Error.


!

testSingleNoRetPartSame
	self setSingleRPC.
	element content isNoReturnPart: true.
	xmlString :=
'<doesntMatter>
<x>1</x>
</doesntMatter>'.
	self decodeRoot.
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.


        
!

testUnexpected
	self setMultipleRPC.
	xmlString :=
'<doesntMatter>
<x>1</x>
<y>goodbye</y>
<z>oops</z>
</doesntMatter>'.
	self should: [self decodeRoot] raise: XeStrictError.
        
        
!

testUnexpectedResume
	self setMultipleRPC.
	xmlString :=
'<doesntMatter>
<x>1</x>
<y>goodbye</y>
<z>oops</z>
</doesntMatter>'.
	[self decodeRoot] on: XeStrictError do: [ :e | e resume].
	self assert: (instance isKindOf: Collection).
	self assert: (instance at: #x) = 1.
	self assert: (instance at: #y) = 'goodbye'.


        
        
! !

!XeSoapRPCStructTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSoapRPCStructTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
