"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeSchemaFactoriesTests subclass:#XeSprayClassFactoryTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeSprayClassFactoryTest methodsFor:'accessing'!

setFactory
	factory := (typespace schemaTargeting: XeNamespace sprayClass) factory
! !

!XeSprayClassFactoryTest methodsFor:'asserting'!

assertTestExtContent
	self assert: (content isKindOf: XeComplexContent).
	self assert: content firstChildElement qName localName = 'x'.
	self deny: content firstChildElement isFormQualified.
	self assert: content firstChildContent typeName = (XeQName xsd2001: 'integer').
	self assert: content attributes size = 2.
	content attributes do: 
			[:each | 
			self assert: (each isKindOf: XeAttribute).
			self deny: each isFormQualified].
	self assert: content attributes first localName = 'a'.
	self assert: content attributes first content typeName = (XeQName xsd2001: 'float').
	self assert: content attributes second localName = 'b'.
	self assert: content attributes second content typeName = (XeQName xsd2001: 'string')
! !

!XeSprayClassFactoryTest methodsFor:'set up'!

setUp
	super setUp.
	self be2001Typespace.
	self setFactory
! !

!XeSprayClassFactoryTest methodsFor:'tests'!

testAddDeclaration
	| element |
	factory declare: 'myNewName' ownType: 'SWXMLTestObject'.
	element := self schema xeElementDeclared: (XeQName inSprayClass: 'myNewName')
				encStyle: self encodingStyle.
	content := element content.
	self assert: element qName = (XeQName inSprayClass: 'myNewName').
	self assertTestObjectContent
!

testAttributes
	self be2001LiteralTypespace.
	self setFactory.
	content := self schema defaultEncContent: 'SWXMLTestExtObject'.
	self assertTestExtContent
!

testAttributesMapped
	self be2001LiteralTypespace.
	self setFactory.
	typespace mapTypeName: ('mySimple' ns: XeNamespace sprayClass) to: SWXMLTestExtObject .
	"(XeClassMap map: 'mySimple' to: SWXMLTestExtObject in: XeNamespace sprayClass) createIn: typespace."
	content := self schema defaultEncContent: #SWXMLTestExtObject asString.
	self assertTestExtContent
!

testCircular
	content := self schema defaultEncContent: 'SWXMLTestCircC'.
	self assert: (content isKindOf: XeComplexContent).
	self assert: (content firstChildContent isKindOf: XeComplexContent).
	self assert: (content firstChildContent firstChildContent isKindOf: XeComplexContent).
	self 
		assert: (content firstChildContent firstChildContent firstChildElement isKindOf: XeAnyTypeElement).
	content := self schema defaultEncContent: 'SWXMLTestCircA'.
	self assert: (content isKindOf: XeComplexContent).
	self assert: (content firstChildContent isKindOf: XeAnyContent ).
        
!

testCircularCirc
	content := self schema defaultEncContent: 'SWXMLTestCircCirc'.
	self assert: (content isKindOf: XeComplexContent).
	self assert: (content firstChildContent isKindOf: XeSimpleContent)
!

testCircularSelf
	content := self schema defaultEncContent: 'SWXMLTestCircSelf'.
	self assert: (content isKindOf: XeComplexContent).
	self assert: (content firstChildContent isKindOf: XeSimpleContent).
	self assert: (content childElements second isKindOf: XeAnyTypeElement)
!

testClassName
	| element |
	element := self schema xeElementDeclared: (XeQName sprayClass: SWXMLTestObject)
				encStyle: self encodingStyle.
	content := element content.
	self assert: element qName = (XeQName sprayClass: SWXMLTestObject).
	self assertTestObjectContent
!

testComplex
	content := self schema defaultEncContent: 'SWXMLTestObject'.
	self assertTestObjectContent
!

testException
	self should: [self schema defaultEncContent: 'NoClassNamedThis'] raise: Error
!

testIsFormQualifiedLiteral
	self be2001LiteralTypespace.
	factory := (typespace schemaTargeting: XeNamespace sprayClass) factory.
	content := self schema defaultEncContent: 'SWXMLTestObject'.
	self assert: (content childElements allSatisfy: [:each | each isFormQualified not]).
	self be2001LiteralTypespace.
	factory := (typespace schemaTargeting: XeNamespace sprayClass) factory.
	factory isElementFormQualified: true.
	content := self schema defaultEncContent: 'SWXMLTestObject'.
	self assert: (content childElements allSatisfy: [:each | each isFormQualified ]).
!

testIsFormQualifiedSoapEnc
	"MUST ingnore"
	factory isElementFormQualified: true.
	content := self schema defaultEncContent: 'SWXMLTestObject'.
	self assert: (content childElements allSatisfy: [:each | each isFormQualified not]).
!

testMapped
	| element |
	typespace mapTypeName: ('myNewName' ns: XeNamespace sprayClass) to: SWXMLTestObject.
	"(XeClassMap map: 'myNewName' to: SWXMLTestObject in: XeNamespace sprayClass) createIn: typespace."
	element := self schema xeElementDeclared: (XeQName inSprayClass: 'myNewName')
				encStyle: self encodingStyle.
	content := element content.
	self assert: element qName = (XeQName inSprayClass: 'myNewName').
	self assertTestObjectContent
! !

!XeSprayClassFactoryTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeSprayClassFactoryTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
