"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingCoreTests subclass:#XeTextConvertersTest
	instanceVariableNames:'dataType'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeTextConvertersTest methodsFor:'tests'!

testBase64
	| byteArray enc ans |
	dataType := XeTextConverter base64Binary.
	byteArray := '123456abc,.<>$' asByteArray.
	enc := dataType asString: byteArray.
	self assert: enc = 'MTIzNDU2YWJjLC48PiQ='.
	ans := dataType fromString: enc.
	self assert: ans = byteArray.
	ans := dataType fromString: 'eW91IGNhbid0IHJlYWQgdGhpcyE='.
	self assert: ans asString = 'you can''t read this!!'
!

testBoolean
	dataType := XeTextConverter boolean.
	self assert: (dataType fromString: 'true').
	self deny: (dataType fromString: 'false').
	self assert: (dataType fromString: '1').
	self deny: (dataType fromString: '0').
	self assert: (dataType asString: true) = 'true'.
	self assert: (dataType asString: false) = 'false'
!

testBooleanEx
	dataType := XeTextConverter boolean.
	self should: [dataType fromString: 'oops'] raise: Error.
!

testDate
	| dt |
	dataType := XeTextConverter date.
	dt := XeXsdDate fromString: '1999-05-31Z'.
	self assert: ((dataType fromString: '1999-05-31Z') isKindOf: XeXsdDate).
	self assert: (dataType asString: dt) = '1999-05-31Z'
!

testDateTime
	| dt ts |
	#cg.

	dataType := XeTextConverter dateTime.
	dt := XeXsdDateTime fromString: '1999-05-31T13:20:00Z'.
	Smalltalk isSmalltalkX ifTrue:[
	    ts := AbsoluteTime fromDate: dt date andTime: dt time.
	] ifFalse:[
	    ts := TimeStamp date: dt date time: dt time.
	].
	self assert: (dataType fromString: '1999-05-31T13:20:00Z') = dt.
	self assert: (dataType asString: dt) = '1999-05-31T13:20:00Z'.
	self assert: (dataType asString: ts) = '1999-05-31T13:20:00Z'
!

testDecimal
	dataType := XeTextConverter decimal.
	self assert: (dataType fromString: '0') = 0.
	self assert: (dataType fromString: '1') = 1.
	self assert: (dataType fromString: '-1') = -1.
	self assert: (dataType fromString: '0.0001') = 1.0e-004.

	self assert: (dataType fromString: '123456789.987654321') = '123456789.987654321s' asNumber.

	#('0' '1' '-1' '0.0001' '123456789.987654321') do: 
			[:each | 
			| s |
			s := dataType fromString: each.
			self assert: (s isKindOf:(Smalltalk isSmalltalkX ifTrue:[FixedPoint] ifFalse:[ScaledDecimal])).
			self assert: (dataType asString: s) = each]
!

testFloatAs
	|t|

	dataType := XeTextConverter double.
	self assert: (t := dataType asString: 3.14) = '3.14'.
	self assert: (t := dataType asString: -0.0) = '-0.0'.
	self assert: (t := dataType asString: 0.0) = '0.0'.
	Smalltalk isSmalltalkX ifTrue:[
	    self assert: (t := dataType asString: 5.0e-003) = '0.005'.
	] ifFalse:[
	    self assert: (t := dataType asString: 5.0e-003) = '5.0e-003'.
	]
!

testFloatFrom
	dataType := XeTextConverter double.
	self assert: (dataType fromString: '3.14') = 3.14.
	self assert: (dataType fromString: '1e10') = 10000000000.
	self assert: (dataType fromString: '1e+10') = 10000000000.
	self assert: (dataType fromString: '1e+010') = 10000000000.
	self assert: (dataType fromString: '1.0e+010') = 10000000000.
	self assert: (dataType fromString: '1.5e+010') = 15000000000.
	self assert: (dataType fromString: '1E10') = 10000000000.
	self assert: (dataType fromString: '1E+10') = 10000000000.
	self assert: (dataType fromString: '1E+010') = 10000000000.
	self assert: (dataType fromString: '1.0E+010') = 10000000000.
	self assert: (dataType fromString: '1.5E+010') = 15000000000.
	self assert: (dataType fromString: '-1e10') = -10000000000.
	self assert: (dataType fromString: '-1e+10') = -10000000000.
	self assert: (dataType fromString: '-1e+010') = -10000000000.
	self assert: (dataType fromString: '-1.0e+010') = -10000000000.
	self assert: (dataType fromString: '-1.5e+010') = -15000000000.
	self assert: (dataType fromString: '-1E10') = -10000000000.
	self assert: (dataType fromString: '-1E+10') = -10000000000.
	self assert: (dataType fromString: '-1E+010') = -10000000000.
	self assert: (dataType fromString: '-1.0E+010') = -10000000000.
	self assert: (dataType fromString: '-1.5E+010') = -15000000000.
	self assert: (dataType fromString: '1e-3') = 1e-003.
	self assert: (dataType fromString: '1e-03') = 1e-003.
	self assert: (dataType fromString: '1e-003') = 1e-003.
	self assert: (dataType fromString: '1.0e-03') = 1e-003.
	self assert: (dataType fromString: '1.5e-3') = 1.5e-003.
	self assert: (dataType fromString: '1E-3') = 1e-003.
	self assert: (dataType fromString: '1E-03') = 1e-003.
	self assert: (dataType fromString: '1E-003') = 1e-003.
	self assert: (dataType fromString: '1.0E-03') = 1e-003.
	self assert: (dataType fromString: '1.5E-3') = 1.5e-003.
	self assert: (dataType fromString: '-1e-3') = -1e-003.
	self assert: (dataType fromString: '-1e-03') = -1e-003.
	self assert: (dataType fromString: '-1e-003') = -1e-003.
	self assert: (dataType fromString: '-1.0e-03') = -1e-003.
	self assert: (dataType fromString: '-1.5e-3') = -1.5e-003.
	self assert: (dataType fromString: '-1E-3') = -1e-003.
	self assert: (dataType fromString: '-1E-03') = -1e-003.
	self assert: (dataType fromString: '-1E-003') = -1e-003.
	self assert: (dataType fromString: '-1.0E-03') = -1e-003.
	self assert: (dataType fromString: '-1.5E-3') = -1.5e-003.
	self assert: (dataType fromString: '-0.0') = -0.0.
	self assert: (dataType fromString: '0.0') = 0.0.
	self assert: (dataType fromString: '6.762E+28') > 10.
	self assert: (dataType fromString: '6.762E-28') < 10.


!

testFloatFromEx
	dataType := XeTextConverter double.
	self should: [dataType fromString: '3.1a4'] raise: Error.
	self should: [dataType fromString: 'abc'] raise: Error.
!

testFloatStrings
	dataType := XeTextConverter double.
	"As long as these signal an error, they can be picked up as conversion errors"
	self should: [dataType fromString: 'NaN'] raise: XeValueRangeError.
	self should: [dataType fromString: '-INF'] raise: XeValueRangeError.
	self should: [dataType fromString: 'INF'] raise: XeValueRangeError
!

testInteger
	dataType := XeTextConverter integer.
	self assert: (dataType fromString: '0') = 0.
	self assert: (dataType fromString: '1') = 1.
	self assert: (dataType fromString: '-0') = 0
!

testIntegerEx
	dataType := XeTextConverter integer.
	self should: [dataType fromString: 'abc'] raise: Error.
	self should: [dataType fromString: '1abc'] raise: Error.
	self should: [dataType fromString: '1.1'] raise: Error.
	self should: [dataType fromString: '.11'] raise: Error.
	self should: [dataType fromString: '1junk1'] raise: Error.
	"cg: I dont think that this is much of a problem"
	Smalltalk isSmalltalkX ifFalse:[
	    self should: [dataType fromString: ' 1'] raise: Error.
	].
	self should: [dataType fromString: '1 '] raise: Error.
	self class isSkipAckFailures ifTrue: [^nil].
	"I dont think that this is much of a problem"
	self should: [dataType fromString: '16r2'] raise: Error
!

testSTB
	| instance enc ans |
	dataType := XeTextConverter stb.
	instance := #(1 #('a') #a).
	enc := dataType asString: instance.
	Smalltalk isSmalltalkX ifTrue:[
	] ifFalse:[
	    self assert: enc 
				= 'IVNUQiAxIGIAAAADAAAAAwAAAGIAAAABAAAAUgAAAAEAAABhugAAAAAAAABSAAAAAQAAAG
E='.
	].

	ans := dataType fromString: enc.
	self assert: ans = instance
!

testStringsWhitespace
	| ws |
	dataType := XeTextConverter string.
	ws := '
'.
	self assert: (dataType fromString: ws) = ws
!

testTime
	| dt  |
	dataType := XeTextConverter time.
	dt := XeXsdTime fromString: '12:34:56Z'.
	self assert: ((dataType fromString: '12:34:56Z') isKindOf: XeXsdTime).
	self assert: (dataType asString: dt) = '12:34:56Z'
! !

!XeTextConvertersTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeTextConvertersTest.st,v 1.5 2003/01/31 22:07:47 cg Exp $'
! !
