"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeEncodingTests subclass:#XeTypespaceTest
	instanceVariableNames:'typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeTypespaceTest methodsFor:'accessing'!

addSchema
	typespace addSchema:  (XeSchema new targetNS: #'urn:blah').
!

be1999Typespace
	typespace := self get1999Typespace
!

be2001Typespace
	typespace := self get2001Typespace
!

getSampleSchemaString

^'<schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
targetNamespace = "http://soapinterop.org/xsd">
<complexType name="SOAPStruct">
<all>
<element name="varString" type="xsd:string" />
<element name="varInt" type="xsd:int" /> 
<element name="varFloat" type="xsd:float" /> 
</all>
</complexType>
</schema>'.
! !

!XeTypespaceTest methodsFor:'asserting'!

assertTypespaceValidDefault
	| schema |
	self assert: (typespace isKindOf: XeTypespace).
	self assert: ((typespace schemaTargeting: XeNamespace soapEnc) isKindOf: XeSchema).
	schema := typespace schemaTargeting: #'http://www.dolphinharbor.org/spray/class'.
	self assert: (schema factory isKindOf: XeSprayClassFactory).
	schema := typespace schemaTargeting: #'http://schemas.xmlsoap.org/soap/encoding/'.
	self assert: (schema factory isKindOf: XeSoapEncFactory).
	schema := typespace schemaTargeting: #'http://www.w3.org/2001/XMLSchema'.
	self assert: (schema factory isKindOf: XeXSDBase2001).
	schema := typespace schemaTargeting: #'http://www.w3.org/1999/XMLSchema'.
	self assert: (schema factory isKindOf: XeXSDBase1999).
	^schema
! !

!XeTypespaceTest methodsFor:'set up'!

setUp
	super setUp.
	typespace := XeTypespace new
! !

!XeTypespaceTest methodsFor:'tests'!

test1999QualifiedNameIn
	| pName |
	self be1999Typespace.
	pName := XePName prefix: 'xsd' local: 'string'.
	self assert: (pName qualifiedNameIn: typespace) = (XeQName prefix: 'xsd'
						local: 'string'
						ns: XeNamespace schema1999).
!

test2001QualifiedNameIn
	| pName |
	self be2001Typespace.
	pName := XePName prefix: 'xsd' local: 'string'.
	self assert: (pName qualifiedNameIn: typespace) = (XeQName prefix: 'xsd'
						local: 'string'
						ns: XeNamespace schema2001).
!

testAddPrefix
	typespace addPrefix: 'abc' namespace: #'urn:blah'.
	self assert: (typespace prefixMap at: 'abc') == #'urn:blah'.
	self should: [typespace addPrefix: 'abc' namespace: #'urn:blah'] raise: Error.
	self should: [typespace addPrefix: 'abc' namespace: #'urn:else'] raise: Error.
!

testAddPrefixSchema
	typespace addSchemaString: '<schema targetNamespace="urn:myBlah"/>' prefix: 'abc'.
	self assert: (typespace schemaTargeting: #'urn:myBlah') targetNS == #'urn:myBlah'.
	self 
		assert: (typespace schemaTargeting: #'urn:myBlah') == (typespace schemaPrefixed: 'abc')
!

testAddSchemaString
	| schema |
	schema := typespace addSchemaString: self getSampleSchemaString.
	self assert: schema targetNS = #'http://soapinterop.org/xsd'.
	self assert: (schema factory isKindOf: XeXSDSchema).
!

testAnyTypeName
	self be1999Typespace.
	self assert: typespace anyTypeName = (XeQName xsd1999: 'ur-type').
	self be2001Typespace.
	self assert: typespace anyTypeName = (XeQName xsd2001: 'anyType').
!

testDefault1999PrefixMap
	| schema |
	self be1999Typespace.
	self assertTypespaceValidDefault.
	self assert: typespace xsdSchema targetNS = XeNamespace schema1999.
	schema := typespace schemaPrefixed: 'xsd'.
	self assert: (schema factory isKindOf: XeXSDBase1999).
	self assert: typespace xsiNamespace == XeNamespace schema1999i
!

testDefault2001PrefixMap
	| schema |
	self be2001Typespace.
	self assertTypespaceValidDefault.
	self assert: typespace xsdSchema targetNS = XeNamespace schema2001.
	schema := typespace schemaPrefixed: 'xsd'.
	self assert: (schema factory isKindOf: XeXSDBase2001).
	self assert: typespace xsiNamespace == XeNamespace schema2001i.

        

!

testDefaultEncodingStyle
	self be2001Typespace.
	self assert: typespace defaultEncodingStyle ==  typespace defaultEncodingStyle.
	self assert: typespace defaultEncodingStyle typespace ==  typespace
!

testDefaultLiteral
	| factory |
	typespace := XeTypespace defaultLiteral2001.
	factory := (typespace schemaTargeting: XeNamespace sprayClass) factory.
	self assert: factory isElementFormQualified.
	self deny: factory isAttributeFormQualified
!

testHasSchemaTargeting
	self deny: (typespace hasSchemaTargeting: #'urn:blah').
	self deny: (typespace hasSchemaTargeting: #'urn:Notblah').
	self addSchema.
	self assert: (typespace hasSchemaTargeting: #'urn:blah').
	self deny: (typespace hasSchemaTargeting: #'urn:Notblah').
        

!

testMakeNamespacesAbsolute
	"| s1 s2 |
	typespace addSchema: (s1 := XeSchema new namespace: #'/schemaBlah').
	typespace addSchema: (s2 := XeSchema new namespace: #'urn:schemaBlah').
	typespace addPrefix: 'a' namespace: #'/schemaBlah'.
	typespace addPrefix: 'b' namespace: #'urn:schemaBlah'.
	typespace makeNamespacesAbsolute: #'http://www.dolphinharbor.org/schemas'.
	self assert: (typespace schemaTargeting: #'urn:schemaBlah') == s2.
	self assert: (typespace schemaTargeting: #'http://www.dolphinharbor.org/schemas/schemaBlah') == s1.
	self assert: (typespace schemaPrefixed: 'a') == s1.
	self assert: (typespace schemaPrefixed: 'b') == s2.
	self assert: s1 namespace == #'http://www.dolphinharbor.org/schemas/schemaBlah'.
	self assert: s2 namespace == #'urn:schemaBlah'"
!

testNamespacePrefixed
	typespace addPrefix: 'abc' namespace: #'urn:blah'.
	self assert: (typespace namespacePrefixed: 'abc') == #'urn:blah'.
	self should: [typespace namespacePrefixed: 'def'] raise: Error.

!

testNewNode
	self be2001Typespace.
	self assertXML: (typespace newNode: 'blah') xml isSameAs: 
'<blah xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>'.
	self be1999Typespace.
	self assertXML: (typespace newNode: 'blah') xml isSameAs: 
'<blah xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"/>'.
!

testOptions
	self be2001Typespace.
	self assert: (typespace options isKindOf: XeTypespaceOptions)
!

testOverride
	| tn type |
	self be2001Typespace.
	typespace 
		addSchema: (XeSchema onFactory: XeSprayClassFactory targetNS: #'http://tempuri.org/').
	typespace 
		overrideAll: #('string')
		to: XeXSDBase2001
		at: #'http://tempuri.org/'.
	"(XeTypeOverride 
		namespace: #'http://tempuri.org/'
		localName: 'string'
		factory: XeXSDBase2001) createIn: typespace."
	tn := 'string' ns: #'http://tempuri.org/'.
	type := (typespace schemaTargeting: tn namespace) xeContentOf: tn
				encStyle: typespace defaultEncodingStyle.
	self assert: (type isKindOf: XeSimpleContent).
	self assert: type typeName == tn
!

testPutXSDDefaultNSOn
	self be1999Typespace.
	self assertXML: (typespace putXSDDefaultNSOn: (typespace newNode: 'blah')) xml isSameAs: 
'<blah xmlns="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"/>'.

	self be2001Typespace.
	self assertXML: (typespace putXSDDefaultNSOn: (typespace newNode: 'blah')) xml isSameAs: 
'<blah xmlns="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>'.

        
!

testQualifiedNameIn
	| qName |
	qName := XeQName prefix: 'xsd'
				local: 'string'
				ns: XeNamespace schema2001. 

	self assert: (qName qualifiedNameIn: typespace) == qName
!

testReplaceSchemaNamespaceOf
	| schemas |
	self be2001Typespace.
	typespace replaceSchemaNamespaceOf: 'd' to: #'http://www.soapinterop.org/Bid'.
	self assert: (typespace namespacePrefixed: 'd') == #'http://www.soapinterop.org/Bid'.
	schemas := typespace schemaTargeting: #'http://www.soapinterop.org/Bid'
!

testSchemaPrefixed
	typespace addPrefix: 'abc' namespace: #'urn:blah'.
	self addSchema.
	self assert: (typespace schemaPrefixed: 'abc') targetNS == #'urn:blah'.
	self should: [typespace schemaPrefixed: 'def' ] raise: Error.

!

testSchemaTargeting
	self addSchema.
	self assert: ((typespace schemaTargeting: #'urn:blah') isKindOf: XeSchema).
	self should: [typespace schemaTargeting: #'urn:notblah'] raise: XeSchemaNotFound.

        
!

testSchemaTargetingResume
	| schema |
	self addSchema.
	schema := [typespace schemaTargeting: #'urn:notBlah']
		on: XeSchemaNotFound do: [ :e | e resume].
	self assert: (schema isKindOf: XeSchema).


        
!

testSprayClassFormQualified
	| factory |
	self be2001Typespace.
	factory := (typespace schemaTargeting: XeNamespace sprayClass) factory.
	self deny: factory isElementFormQualified.
	self deny: factory  isAttributeFormQualified.
	factory isAttributeFormQualified: true.
	self assert: factory isAttributeFormQualified.
	self deny: factory isElementFormQualified.
!

testTypeWith
	| pName |
	self be2001Typespace.
	pName := XePName prefix: 'xsd' local: 'string'.
	(Array with: pName with: (XePName prefix: 'd' local: #SWXMLTestObject asString)) 
		with: (Array with: (XeQName xsd2001: 'string')
				with: ('SWXMLTestObject' ns: XeNamespace sprayClass))
		do: 
			[:p :q | 
			| content |
			content := p xeContentWith: typespace defaultEncodingStyle.
			self assert: content typeName = q.
			self assert: (content isKindOf: XeContentAbstract)]
! !

!XeTypespaceTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeTypespaceTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
