"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeXMLTests subclass:#XeXMLParserTests
	instanceVariableNames:'xmlString node'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeXMLParserTests methodsFor:'accessing'!

loadSample1
	self 
		loadXmlString: 
'<first 
  xmlns="urn:blah"
  xmlns:c="urn:clah">
  <second/>
  <third/>
  <c:fourth/>
  <fourth/>
</first>'
!

loadSample2
	self 
		loadXmlString: 
'<first 
  xmlns="urn:blah"
  xmlns:c="urn:clah">
  <aParent>
    <second/>
    <third/>
    <c:fourth/>
    <fourth/>
  </aParent>
</first>'
!

loadXmlString: aString 
	xmlString := aString.
	node := self rootNodeFor: xmlString
! !

!XeXMLParserTests methodsFor:'tests'!

testBaseFromNode
	| baseTypeName |
	self 
		loadXmlString: 
'<schema 
targetNamespace="http://www.xmethods.com/services" 
xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/" 
xmlns="http://www.w3.org/1999/XMLSchema">
 <complexType name="ArrayOfstring" base="soap:Array">
 <sequence>
  <element name="item" type="xsd:string" minOccurs="0" maxOccurs="*" /> 
  </sequence>
  </complexType>
</schema>'.
	baseTypeName := node firstChild qValueFor: 'base'.
	self assert: baseTypeName localName = 'Array'.
	self assert: baseTypeName namespace = #'http://schemas.xmlsoap.org/wsdl/soap/'
!

testClone
	| clone |
	self loadXmlString:
'<root
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <next xmlns:mine="urn:mine"> 
    <schema>
	<element name="blah" type="blah" />
    </schema>
  </next>
</root>'.
	clone := node firstChild firstChild cloneAllWithPrefixes.
	self assertXML: clone xml
		isSameAs: 
'<schema xmlns:mine="urn:mine" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<element name="blah" type="blah"/>
</schema>'
!

testCloneAttributeDuplicated
	| clone |
	self loadXmlString:
'<root
xmlns:xsd="NOThttp://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <next xmlns:mine="urn:mine"> 
    <schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" >
	<element name="blah" type="blah" />
    </schema>
  </next>
</root>'.
	clone := node firstChild firstChild cloneAllWithPrefixes.
	self assertXML: clone xml
		isSameAs: 
'<schema xmlns:mine="urn:mine" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<element name="blah" type="blah"/>
</schema>'
!

testCloneWithDefaultNS
	| clone |
	self loadXmlString:
'<root
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <next xmlns:mine="urn:mine"> 
    <schema>
	<element name="blah" type="blah" />
    </schema>
  </next>
</root>'.
	clone := node firstChild firstChild cloneAllWithPrefixes.
	self assertXML: clone xml
		isSameAs: 
'<schema xmlns="http://www.w3.org/2001/XMLSchema" xmlns:mine="urn:mine" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<element name="blah" type="blah"/>
</schema>'
!

testCloneWithDefaultNSAllPrefixed
	| clone |
	self loadXmlString:
'<root
xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns="http://www.w3.org/2001/XMLSchema" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <next xmlns:mine="urn:mine"> 
    <xsd:schema>
	<xsd:element name="blah" type="blah" />
    </xsd:schema>
  </next>
</root>'.
	clone := node firstChild firstChild cloneAllWithPrefixes.
	self assertXML: clone xml
		isSameAs: 
'<xsd:schema xmlns="http://www.w3.org/2001/XMLSchema" xmlns:mine="urn:mine" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<xsd:element name="blah" type="blah"/>
</xsd:schema>'
!

testDetectAllNodeIn
	self loadSample2.
	self assert: (node detectAllNodes: 'second' in: #'urn:blah') localName = 'second'.
	self assert: (node detectAllNodes: 'second' in: nil) localName = 'second'.
	self should: [node detectAllNodes: 'second' in: #'urn:zlah'] raise: NotFoundError.
	self should: [node detectAllNodes: 'z' in: #'urn:blah'] raise: NotFoundError.
	self assert: (node detectAllNodes: 'fourth' in: #'urn:clah') previousSibling localName 
				= 'third'.
	self assert: (node detectAllNodes: 'fourth' in: nil) previousSibling localName = 'third'.
	self assert: (node detectAllNodes: 'fourth' in: #'urn:blah') previousSibling localName 
				= 'fourth'.
	self should: [node detectChildNode: 'second' in: #'urn:blah'] raise: NotFoundError
!

testDetectChildNodeIn
	self loadSample1.
	self assert: (node detectChildNode: 'second' in: #'urn:blah') localName = 'second'.
	self assert: (node detectChildNode: 'second' in: nil) localName = 'second'.
	self should: [node detectChildNode: 'second' in: #'urn:zlah'] raise: NotFoundError.
	self should: [node detectChildNode: 'z' in: #'urn:blah'] raise: NotFoundError.
	self assert: (node detectChildNode: 'fourth' in: #'urn:clah') previousSibling localName = 'third'.
	self assert: (node detectChildNode: 'fourth' in: nil) previousSibling localName = 'third'.
	self assert: (node detectChildNode: 'fourth' in: #'urn:blah') previousSibling localName = 'fourth'
!

testElementNodes
	self loadXmlString:
'<root>
  <child1/> 
  <child2/>
</root>'.
	self assert: node childNodes size = 2.
	self assert: node firstChild localName = 'child1'.
	self assert: node lastChild localName = 'child2'.
	self assert: node firstChild nextSibling localName = 'child2'.
	self assert: node lastChild previousSibling localName= 'child1'.
	self assert: node firstChild nextSibling nextSibling isNil.
	self assert: node lastChild previousSibling previousSibling isNil.
	self loadXmlString:
'<root>
</root>'.
	self assert: node childNodes isEmpty.
	self assert: node firstChild isNil.
	self assert: node lastChild isNil
!

testEmptyTextContent
	self loadXmlString: '<a/>'.
	self assert: node text = ''
!

testEncodingUTF8
	self loadXmlString:
'<?xml version="1.0" encoding="UTF-8"?>
<hello/>'.

!

testGetAttributeNamespace
	self loadXmlString:
'<first a:xyz="stw"
  xmlns="urn:blah"
  xmlns:a="urn:foo">
    <second a:abc="def"/>
</first>'.
	self assert: (node firstChild getAttribute: 'a:abc') = 'def'.
	self assert: (node firstChild getAttribute: 'abc' in: #'urn:foo') = 'def'.
	self assert: (node firstChild getAttribute: 'abc' in: #'urn:blah') isNil.
	self assert: (node getAttribute: 'a:xyz') = 'stw'.
	self assert: (node getAttribute: 'xyz' in: #'urn:foo') = 'stw'.
	self assert: (node getAttribute: 'xyz' in: #'urn:blah') isNil
!

testGetQAttributes
	| table |
	self 
		loadXmlString: '<first xmlns="urn:blah" xmlns:a="urn:foo" a1="a1V" a:a2="a2V">
  <second/>
</first>'.
	self assert: node getQAttributes size = 2.
	table := Dolphin::LookupTable new.
	node getQAttributes do: [:each | table at: each key put: each value].
	self assert: (table at: ('a1' ns: #'')) = 'a1V'.
	self assert: (table at: ('a2' ns: #'urn:foo')) = 'a2V'
!

testHasPrefix
	node := self rootNodeFor: '<first/>'.
	self deny: node hasPrefix.
	node := self rootNodeFor: '<a:first xmlns:a="urn:b"/>'.
	self assert: node hasPrefix
!

testIsSimpleElement
	node := self rootNodeFor: '<first/>'.
	self assert: node isSimpleElement.
	node := self rootNodeFor: '<first>abc</first>'.
	self assert: node isSimpleElement.
	node := self rootNodeFor: '<first><second>abc</second></first>'.
	self deny: node isSimpleElement.
	self assert: node firstChild isSimpleElement
!

testIsXSINil
	#(
(true '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><node xsi:nil="1"/></root>')
(true  '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><node xsi:nil="true"/></root>')
(false '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><node xsi:null="true"/></root>')
(true '<root xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"><node xsi:null="1"/></root>')
(true '<root xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"><node xsi:null="true"/></root>')
(false '<root xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"><node xsi:nil="true"/></root>'))
	do: [ :each |
		self loadXmlString: each second.
		self assert: node firstChild isXSINil == each first]
        
!

testLeadingSpace
	self loadXmlString: '<a> b</a>'.
	self assert: node text = ' b'
!

testLoadInvalidXDR

self loadXmlString:
'<?xml version=''1.0''?>
<SOAP:Envelope SOAP:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:dt="urn:schemas-microsoft-com:datatypes" xmlns:wsdl=''http://soapinterop.org/xsd'' ><SOAP:Body><cli:echoStructAsSimpleTypesResponse xmlns:cli=''http://soapinterop.org/''><outputString xsi:type="xsd:string" dt:dt="string">abc</outputString><outputInteger xsi:type="xsd:int" dt:dt="int">123</outputInteger><outputFloat xsi:type="xsd:double" dt:dt="double">3.140000</outputFloat></cli:echoStructAsSimpleTypesResponse></SOAP:Body></SOAP:Envelope>'
!

testNamespaces
	self loadXmlString:
'<first xmlns:c="urn:clah">
  <second xmlns="urn:blah">
    <a:third xmlns:a="urn:alah"/>
    <fourth/>
  </second>
</first>'.
	self assert: node defaultNamespace = #''.
	self assert: node firstChild defaultNamespace = #'urn:blah'.
	self assert: node firstChild firstChild defaultNamespace = #'urn:blah'.
	self assert: node firstChild firstChild nextSibling defaultNamespace = #'urn:blah'.
	self assert: node namespace = #''.
	self assert: node firstChild namespace = #'urn:blah'.
	self assert: node firstChild firstChild namespace = #'urn:alah'.
	self assert: node firstChild firstChild nextSibling namespace = #'urn:blah'
!

testOwnerDocument
	node := self rootNodeFor: '<first/>'.
	self assert: (node ownerDocument isKindOf: self docKindOfClass).
	self assert: (node ownerDocument ownerDocument isKindOf: self docKindOfClass).
!

testQName
	| xmlName |
	self loadXmlString:
 '<blah 
  xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
  xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns="urn:blahspace"/>'.
	xmlName := node qName.
	self assert: xmlName localName = 'blah'.
	self assert: xmlName prefix = ''.
	self assert: xmlName namespace = #'urn:blahspace'.
	self loadXmlString:
'<abc:blah 
  xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
  xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:abc="urn:blahspace"/>'.
	xmlName := node qName.
	self assert: xmlName localName = 'blah'.
	self assert: xmlName prefix = 'abc'.
	self assert: xmlName namespace = #'urn:blahspace'
!

testQNameEquality
	| name |
	node := self rootNodeFor: '<first/>'.
	name := node qName.
	self assert: name localName = 'first'.
	self assert: name = node qName.
	node := self rootNodeFor: '<first xmlns="urn:blah"/>'.
	self deny: name = node qName.
	name := node qName.
	self assert: name = node qName.
	node := self rootNodeFor: '<first xmlns="urn:clah"/>'.
	self deny: name = node qName.
	name := node qName.
	self assert: name = node qName
!

testSelectAllNodesIn
	self loadSample2.
	self assert: (node selectAllNodes: 'second' in: #'urn:blah') size = 1.
	self assert: (node selectAllNodes: 'second' in: nil) size = 1.
	self assert: (node selectAllNodes: 'second' in: #'urn:clah') isEmpty.
	self assert: (node selectAllNodes: 'fourth' in: #'urn:clah') size = 1.
	self assert: (node selectAllNodes: 'fourth' in: nil) size = 2.
	self assert: (node selectAllNodes: 'fourth' in: #'urn:blah') size = 1.
	self assert: (node selectAllNodes: 'notExist' in: #'urn:blah') isEmpty.
	self assert: (node selectAllNodes: 'notExist' in: nil) isEmpty
!

testSelectChildNodeIn
	self loadSample1.
	self assert: (node selectChildNodes: 'second' in: #'urn:blah') size = 1.
	self assert: (node selectChildNodes: 'second' in: nil) size = 1.
	self assert: (node selectChildNodes: 'second' in: #'urn:clah') isEmpty.
	self assert: (node selectChildNodes: 'fourth' in: #'urn:clah') size = 1.
	self assert: (node selectChildNodes: 'fourth' in: nil) size = 2.
	self assert: (node selectChildNodes: 'fourth' in: #'urn:blah') size = 1.
	self assert: (node selectChildNodes: 'notExist' in: #'urn:blah') isEmpty.
	self assert: (node selectChildNodes: 'notExist' in: nil) isEmpty
!

testSingleSpace
	self loadXmlString: '<a> </a>'.
	self assert: node text = ' '
!

testTargetNamespace
	self loadXmlString:
'<blah xmlns:target="urn:that"  targetNamespace="urn:this">
<abc/>
</blah>'.
	self assert: node firstChild targetNamespace = #'urn:this'.
	self assert: node targetNamespace = #'urn:this'.

        
!

testTrailingSpace
	self loadXmlString: '<a>b </a>'.
	self assert: node text = 'b '
!

testTypeQName
	| name |
	node := self rootNodeFor: '<first/>'.
	self assert: node qValueForType isNil.
	node := self rootNodeFor: '<first xmlns:a="urn:blah" type="a:blah"/>'.
	name := node qValueForType.
	self assert: name prefix = 'a'.
	self assert: name localName = 'blah'.
	self assert: name namespace = #'urn:blah'.
	node := self rootNodeFor: '<first xmlns:a="urn:blah"><second type="a:blah"/></first>'.
	self assert: node qValueForType isNil.
	name := node firstChild qValueForType.
	self assert: name prefix = 'a'.
	self assert: name localName = 'blah'.
	self assert: name namespace = #'urn:blah'
!

testXSITypeQName
	| name |
	node := self rootNodeFor: '<first/>'.
	self assert: node qValueForXSIType isNil.
	node := self 
				rootNodeFor: '<first xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:a="urn:blah" xsi:type="a:blah"/>'.
	name := node qValueForXSIType.
	self assert: name prefix = 'a'.
	self assert: name localName = 'blah'.
	self assert: name namespace = #'urn:blah'.
	'<first xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:a="urn:blah" xsi:type="a:blah"/>'.
	name := node qValueForXSIType.
	self assert: name prefix = 'a'.
	self assert: name localName = 'blah'.
	self assert: name namespace = #'urn:blah'.
	node := self 
				rootNodeFor: '<first xmlns:xsi="Nothttp://www.w3.org/2001/XMLSchema-instance" xmlns:a="urn:blah" xsi:type="a:blah"/>'.
	self assert: node qValueForXSIType isNil
! !

!XeXMLParserTests class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeXMLParserTests.st,v 1.4 2003/01/20 19:02:16 cg Exp $'
! !
