"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeXSDTests subclass:#XeXSDContentModelTest
	instanceVariableNames:'xsdContent'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Schema'
!


!XeXSDContentModelTest methodsFor:'accessing'!

setXSDContentFor: aQName
	schema := typespace schemaTargeting: aQName namespace.
	xsdContent := (schema factory definitionFor: aQName) 
				xsdContentModel.
        
! !

!XeXSDContentModelTest methodsFor:'tests'!

testAttributeRef
	self be2001Typespace.
	self importSchemaString: XeTestSchemaImports new complexDefAttributeRef.
	self setXSDContentFor: ('PurchaseOrderType' ns: #'urn:blah').
	self assert: (xsdContent isKindOf: XeXSDEmptyComplexContent).
	self assert: xsdContent xsdContent size = 2.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDSequence).
	self assert: (xsdContent xsdContent last isKindOf: XeXSDAttribute)
!

testExtendingComplexType
	self be2001Typespace.
	self importSchemaString: XeTestSchemaImports new complexDefExtension2.
	self setXSDContentFor: ('femalePerson' ns: #someNamespace).
	self assert: (xsdContent isKindOf: XeXSDExtensionComplexContent).
	self assert: xsdContent baseTypeName localName = 'PERSON'.
	self assert: xsdContent baseTypeName namespace = #someNamespace.
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDElement).
	self assert: xsdContent xsdContent first node localName = 'element'.
	self deny: xsdContent isOldXSDSchema
!

testExtensionContent
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefExtension.
	self setXSDContentFor: (self blahQ: 'USAddress').
	self assert: (xsdContent isKindOf: XeXSDExtensionComplexContent).
	self assert: xsdContent baseTypeName localName = 'Address'.
	self assert: xsdContent baseTypeName namespace = #'urn:cantFindThis'.
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDSequence).
	self assert: xsdContent xsdContent first node localName = 'sequence'.
	self deny: xsdContent isOldXSDSchema.
	"Not yet implemented"
	self should: [schema defaultEncContent: 'USAddress'] raise: XeSchemaNotFound.
        
!

testInlineElement
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefInline.
	self setXSDContentFor: (self blahQ: 'PurchaseOrderType').
	self assert: (xsdContent isKindOf: XeXSDEmptyComplexContent).
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'anyType').
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDSequence).
	self assert: xsdContent xsdParticles size = 2.
	self deny: xsdContent isOldXSDSchema.
!

testNCBaseAttribute
	self be2001Typespace.
	self 
		importSchemaString: 
'<xsd:schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns="http://www.w3.org/2001/XMLSchema"

targetNamespace="urn:blah" >
  <xsd:complexType name="InternationalPrice">
   <xsd:simpleContent>
    <xsd:extension base="decimal">
     <xsd:attribute name="currency" type="xsd:string"/>
    </xsd:extension>
   </xsd:simpleContent>
  </xsd:complexType>
</xsd:schema>'.

	self setXSDContentFor: (self blahQ: 'InternationalPrice').
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'decimal').
	self deny: xsdContent isOldXSDSchema.
!

testNoContent
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefNoContentAll.
	self setXSDContentFor: (self blahQ: 'PurchaseOrderType').
	self assert: (xsdContent isKindOf: XeXSDEmptyComplexContent).
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'anyType').
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDAll).
	self assert: xsdContent xsdContent first node localName = 'all'.
	self deny: xsdContent hasAttributeContent.
	self deny: xsdContent isOldXSDSchema.
!

testNoContentSequence
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefNoContent.
	self setXSDContentFor: (self blahQ: 'USAddress').
	self assert: (xsdContent isKindOf: XeXSDEmptyComplexContent).
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'anyType').
	self assert: xsdContent xsdContent size = 2.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDSequence).
	self assert: (xsdContent xsdContent second isKindOf: XeXSDAttribute).
	self assert: xsdContent hasAttributeContent.
	self deny: xsdContent isOldXSDSchema.

	"We cant read this in soap-enc because of the attribute"
	self should: [schema defaultEncContent: 'USAddress'] raise: Error
!

testRestrictionAnyContent
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefRestrictionOfAny.
	self setXSDContentFor: (self blahQ: 'Price').
	self assert: (xsdContent isKindOf: XeXSDRestrictionComplexContent).
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'anyType').
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDAll).
	self assert: xsdContent xsdContent first node localName = 'all'.
	self deny: xsdContent hasAttributeContent.
	self deny: xsdContent isOldXSDSchema.
!

testRestrictionContent
	self be2001Typespace.
	self 
		importSchemaString: XeTestSchemaImports new complexDefRestriction.
	self setXSDContentFor: (self blahQ: 'Price' ).
	self assert: (xsdContent isKindOf: XeXSDRestrictionComplexContent).
	self assert: xsdContent baseTypeName localName = 'something'.
	self assert: xsdContent baseTypeName namespace = #'urn:ipo'.
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDAttribute).
	self assert: xsdContent hasAttributeContent.
	self deny: xsdContent isOldXSDSchema.
!

testSimpleExtensionContent
	self be2001Typespace.
	self importSchemaString: XeTestSchemaImports new complexDefSimple.
	self setXSDContentFor: (self blahQ: 'InternationalPrice').
	self assert: (xsdContent isKindOf: XeXSDExtensionSimpleContent).
	self assert: xsdContent baseTypeName = (XeQName xsd2001: 'decimal').
	self assert: xsdContent xsdContent size = 1.
	self assert: (xsdContent xsdContent first isKindOf: XeXSDAttribute).
	self assert: xsdContent hasAttributeContent.
	self deny: xsdContent isOldXSDSchema.
!

testValidSoapEnc
	self be2001Typespace.
	self 
		importSchemaString: 
'<xsd:schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"

targetNamespace="urn:blah" >
  <xsd:complexType name="MyType">
   <xsd:sequence>
    <xsd:element name="myElement" type="xsd:string"/>
   </xsd:sequence>
    <xsd:attribute ref="SOAP-ENV:actor" use="optional"/>
    <xsd:attribute ref="SOAP-ENV:mustUnderstand" use="optional"/>
  </xsd:complexType>
</xsd:schema>'.

	self setXSDContentFor: (self blahQ: 'MyType').
	self deny: xsdContent isOldXSDSchema.
	xsdContent assertValidSoapEncStruct
!

testValidSoapEncNot
	self be2001Typespace.
	self 
		importSchemaString: 
'<xsd:schema 
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns="http://www.w3.org/2001/XMLSchema"
xmlns:SOAP-ENV="NOThttp://schemas.xmlsoap.org/soap/envelope/"

targetNamespace="urn:blah" >
  <xsd:complexType name="MyType">
   <xsd:sequence>
    <xsd:element name="myElement" type="xsd:string"/>
   </xsd:sequence>
    <xsd:attribute ref="SOAP-ENV:actor" use="optional"/>
    <xsd:attribute ref="SOAP-ENV:mustUnderstand" use="optional"/>
  </xsd:complexType>
</xsd:schema>'.

	self setXSDContentFor: (self blahQ: 'MyType').
	self deny: xsdContent isOldXSDSchema.
	self should: [xsdContent assertValidSoapEncStruct] raise: Error.
! !

!XeXSDContentModelTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeXSDContentModelTest.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
