"{ Package: 'stx:goodies/soap/xe/tests' }"

"{ NameSpace: SOAP }"

XeXSDDateTimeTests subclass:#XeXSDDateTimeTest
	instanceVariableNames:'xsdDateTime'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-XeEncoding-TestCases-Core'
!


!XeXSDDateTimeTest methodsFor:'accessing'!

date
	^xsdDateTime date
!

time
	^xsdDateTime time
! !

!XeXSDDateTimeTest methodsFor:'tests'!

testBefore1900String
	xsdDateTime := XeXsdDateTime fromString: '1899-01-01T00:00:00Z'.
	self assert: xsdDateTime displayString = '1899-01-01T00:00:00Z'.

	xsdDateTime := XeXsdDateTime fromString: '1-01-01T00:00:00Z'.
	self assert: xsdDateTime displayString = '1-01-01T00:00:00Z'.
!

testEqual
	| dt1 dt2 |
	dt1 := XeXsdDateTime fromString: '1999-05-31T06:00:00-05:00'.
	dt2 := XeXsdDateTime fromString: '1999-05-31T06:00:00-05:00'.
	self assert: dt1 = dt2.
	self assert: dt2 = dt1.
	dt2 := XeXsdDateTime fromString: '1999-05-31T11:00:00Z'.
	self assert: dt1 = dt2.
	self assert: dt2 = dt1.
	dt1 := XeXsdDateTime fromString: '1999-05-31T11:00:00Z'.
	self assert: dt1 = dt2.
	self assert: dt2 = dt1
!

testFromOffsetString
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T13:20:00-05:00'.
	self assert: self is19990531.
	self assert: self is1320.
	self assert: xsdDateTime offset hours = 5.
	self assert: xsdDateTime offset minutes = 0.
	self deny: xsdDateTime isPositiveOffset.
	self deny: xsdDateTime isNormal.
	self assert: xsdDateTime displayString = '1999-05-31T13:20:00-05:00'
!

testFromUTCString
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T13:20:00Z'.
	self assert: self is19990531.
	self assert: self is1320.
	self assert: xsdDateTime isNormal.
	self assert: xsdDateTime displayString = '1999-05-31T13:20:00Z'
!

testNegativeString
	xsdDateTime := XeXsdDateTime fromString: '-10-01-01T00:00:00Z'.
	self assert: xsdDateTime displayString = '-10-01-01T00:00:00Z'
!

testNoZ
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T13:20:00'.
	self assert: self is19990531.
	self assert: self is1320.
	self assert: xsdDateTime isNormal.
	self assert: xsdDateTime displayString = '1999-05-31T13:20:00Z'
!

testNormalized
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T06:00:00+05:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-31T01:00:00Z'.
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T06:00:00+06:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-31T00:00:00Z'.
	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T06:00:00+07:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-30T23:00:00Z'.
	xsdDateTime := XeXsdDateTime fromString: '1999-05-30T22:00:00-01:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-30T23:00:00Z'.
	xsdDateTime := XeXsdDateTime fromString: '1999-05-30T22:00:00-02:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-31T00:00:00Z'.
	xsdDateTime := XeXsdDateTime fromString: '1999-05-30T22:00:00-03:00'.
	self assert: xsdDateTime normalized displayString = '1999-05-31T01:00:00Z'
!

testPrecisionMSecs
	xsdDateTime := XeXsdDateTime fromString: '2001-06-16T18:13:40.012'.
	self class isSkipAckFailures ifTrue: [^nil].
	self assert: xsdDateTime displayString = '2001-06-16T18:13:40.012'
!

testTimeStamp
	| ts |
	#cg.

	xsdDateTime := XeXsdDateTime fromString: '1999-05-31T13:20:00Z'.
	Smalltalk isSmalltalkX ifTrue:[
	    ts := AbsoluteTime fromDate: xsdDateTime date andTime: xsdDateTime time.
	] ifFalse:[
	    ts := TimeStamp date: xsdDateTime date time: xsdDateTime time.
	].
	self assert: ts schemaDisplayString = '1999-05-31T13:20:00Z'
! !

!XeXSDDateTimeTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/tests/XeXSDDateTimeTest.st,v 1.5 2003/01/20 19:02:16 cg Exp $'
! !
