"{ Package: 'stx:goodies/soap/xmlsig' }"

"{ NameSpace: SOAP }"

Object subclass:#SignatureHandler
	instanceVariableNames:'algorithm signer'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-SEC-Algorithms'
!


!SignatureHandler class methodsFor:'helpers'!

extractOptionalSingleChildNamed:childName from:anElement
    |coll|

    coll := anElement children select:[:each | each name = childName].
    coll size ~~ 1 ifTrue:[
	coll size == 0 ifTrue:[
	    ^ nil.
	].
	SoapMalformedSignature raiseErrorString:('more than one element: ' , childName).
	^ nil.
    ].
    ^ coll first.
!

extractSingleChildNamed:childName from:anElement
    |coll|

    coll := anElement children select:[:each | each name = childName].
    coll size ~~ 1 ifTrue:[
	coll size == 0 ifTrue:[
	    SoapMalformedSignature raiseErrorString:('missing element: ' , childName).
	    ^ nil.
	].
	SoapMalformedSignature raiseErrorString:('more than one element: ' , childName).
	^ nil.
    ].
    ^ coll first.
!

extractSingleChildWithLocalName:childsLocalName from:anElement
    |coll|

    coll := anElement subElements select:[:each | each localName = childsLocalName].
    coll size ~~ 1 ifTrue:[
	SoapMalformedSignature raiseErrorString:('missing element: ' , childsLocalName).
	^ nil.
    ].
    ^ coll first.
! !

!SignatureHandler methodsFor:'accessing'!

algorithm:algorithmArg signer:signerArg 
    "set instance variables (automatically generated)"

    algorithm := algorithmArg.
    signer := signerArg.
!

signer
    "return the value of the instance variable 'signer' (automatically generated)"

    ^ signer
!

signer:something
    "set the value of the instance variable 'signer' (automatically generated)"

    signer := something.
! !

!SignatureHandler methodsFor:'signature generation/validation'!

generateSignatureInEnvelope:aSoapEnvelope
    (algorithm signer:signer) generateSignatureInEnvelope:aSoapEnvelope
!

validateSignature:signatureElement inEnvelope:aSoapEnvelope serviceHandler:aServiceHandler
    |secPrefix ns dsPrefix dsNs coll dsSignatureElement dsSignedInfoElement dsSignatureValueElement
     dsCanonicalizationMethodElement dsSignatureMethodElement dsReferenceElement
     algorithmName algorithmClass algorithmInstance|

    secPrefix := signatureElement prefixName.
    ns := signatureElement declaredNamespaceURIFor:secPrefix.

    "/ TODO: more flexibility here ?
    "/ for now: fixed to 'http://schemas.xmlsoap.org/soap/security/2000-12'
    (ns = SoapConstants soapSecURI) ifFalse:[
	SoapUnsupportedSignature raiseErrorString:('expected "' , SoapConstants soapSecURI , '" signature').
	^ self.
    ].

    "/ single ds:Signature element
    coll := signatureElement children select:[:each | each localName = 'Signature'].
    coll size ~~ 1 ifTrue:[
	SoapMalformedSignature raiseErrorString:('sec:Signature must have a single ds:Signature element').
	^ self.
    ].
    dsSignatureElement := coll first.

    dsPrefix := dsSignatureElement prefixName.
    "/ TODO: more flexibility here ?
    "/ for now: fixed to 'http://www.w3.org/2000/09/xmldsig#'
    dsNs := dsSignatureElement declaredNamespaceURIFor:dsPrefix.
    (dsNs = SoapConstants soapDsigURI) ifFalse:[
	SoapUnsupportedSignature raiseErrorString:('expected "' , SoapConstants soapDsigURI , '" signature').
	^ self.
    ].

    dsSignedInfoElement := self class extractSingleChildNamed:(dsPrefix , ':' , 'SignedInfo') from:dsSignatureElement.
    dsSignatureValueElement := self class extractSingleChildNamed:(dsPrefix , ':' , 'SignatureValue') from:dsSignatureElement.


    dsCanonicalizationMethodElement := self class extractSingleChildNamed:(dsPrefix , ':' , 'CanonicalizationMethod') from:dsSignedInfoElement.
    dsSignatureMethodElement := self class extractSingleChildNamed:(dsPrefix , ':' , 'SignatureMethod') from:dsSignedInfoElement.
    dsReferenceElement := self class extractSingleChildNamed:(dsPrefix , ':' , 'Reference') from:dsSignedInfoElement.


    "/ CanonicalizationMethod supported ?
    "/ TODO: move into instance
    algorithmName := dsCanonicalizationMethodElement getAttributeNamed: 'Algorithm'.
    (algorithmName isNil) ifTrue:[
	SoapMalformedSignature raiseErrorString:('missing CanonicalizationMethod-Attribute').
	^ self.
    ].
    (algorithmName = SoapConstants canonicalizationMethodURI) ifFalse:[
	SoapUnsupportedSignature raiseErrorString:('unsupported CanonicalizationMethod').
	^ self.
    ].

    "/ SignatureMethod supported ?
    algorithmName := dsSignatureMethodElement getAttributeNamed: 'Algorithm'.
    algorithmName isString ifFalse:[
	SoapMalformedSignature raiseErrorString:('missing SignatureMethod-Attribute').
	^ self.
    ].
    (algorithmName startsWith:(SoapConstants soapDsigURI)) ifFalse:[
	SoapMalformedSignature raiseErrorString:('bad SignatureMethod').
	^ self.
    ].
    algorithmName := algorithmName copyFrom:(SoapConstants soapDsigURI size + 1).
    algorithmClass := SoapSignatureAlgorithm allSubclasses detect:[:cls | cls algorithmName = algorithmName] ifNone:nil.
    algorithmClass isNil ifTrue:[
	SoapUnsupportedSignature raiseErrorString:('unsupported SignatureMethod').
	^ self.
    ].

    algorithm notNil ifTrue:[
	"/ if the algorithm has already been instanciated, 
	"/ validate it against the one given in the message...
	algorithm class == algorithmClass ifFalse:[
	    SoapUnsupportedSignature raiseErrorString:('SignatureMethod must be ' , algorithmName).
	    ^ self.
	].
	algorithmInstance := algorithm.
    ] ifFalse:[
	"/ otherwise, create one as specified in the message
	algorithmInstance := algorithmClass new.
    ].

    algorithmInstance serviceHandler:aServiceHandler.

    algorithmInstance signer:signer.
    algorithmInstance header:aSoapEnvelope header.
    algorithmInstance body:aSoapEnvelope body.
    algorithmInstance dsPrefix:dsPrefix.

    algorithmInstance dsSignatureElement:dsSignatureElement.
    algorithmInstance dsSignedInfoElement:dsSignedInfoElement.
    algorithmInstance dsSignatureValueElement:dsSignatureValueElement.

    algorithmInstance dsCanonicalizationMethodElement:dsCanonicalizationMethodElement.
    algorithmInstance dsSignatureMethodElement:dsSignatureMethodElement.
    algorithmInstance dsReferenceElement:dsReferenceElement.

    algorithmInstance validateSignatureIn:aSoapEnvelope.
! !

!SignatureHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xmlsig/SignatureHandler.st,v 1.3 2003/01/20 19:02:16 cg Exp $'
! !
