"{ Package: 'stx:goodies/soap/xmlsig' }"

"{ NameSpace: SOAP }"

Object subclass:#SoapSignatureAlgorithm
	instanceVariableNames:'serviceHandler signer header body dsPrefix signatureElement
		dsSignatureElement dsSignedInfoElement dsSignatureValueElement
		dsCanonicalizationMethodElement dsSignatureMethodElement
		dsReferenceElement bodyReferenceName dsTransformsElement
		dsDigestMethodElement dsDigestValueElement dsKeyInfoElement'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-SEC-Algorithms'
!


!SoapSignatureAlgorithm class methodsFor:'queries'!

algorithmName
    "the name as in a SignatureMethod - Algorithm attribute"

    self subclassResponsibility
!

digestHashStreamClass
    "the hashStream class for the digest"

    self subclassResponsibility
!

digestName
    "the name as in a DigestMethod - Algorithm attribute"

    self subclassResponsibility
! !

!SoapSignatureAlgorithm class methodsFor:'signature generation'!

generateSignatureInEnvelope:aSoapEnvelope
    ^ self new generateSignatureInEnvelope:aSoapEnvelope
! !

!SoapSignatureAlgorithm class methodsFor:'validation'!

extractSingleChildNamed:childName from:anElement
    |coll|

    coll := anElement children select:[:each | each name = childName].
    coll size ~~ 1 ifTrue:[
	SoapMalformedSignature raise.
	^ self.
    ].
    ^ coll first.
!

validate:aSignature in:anEnvelope
    |dsSignature dsPrefix dsNs
     dsSignedInfo dsSignatureValue coll|

    coll := aSignature children select:[:each | each localName = 'Signature'].
    coll size ~~ 1 ifTrue:[
	SoapMalformedSignature raise.
	^ self.
    ].
    dsSignature := coll first.

    dsPrefix := dsSignature prefixName.
    dsNs := dsSignature declaredNamespaceURIFor:dsPrefix.
    dsNs = SoapConstants soapDsigURI ifFalse:[
	SoapUnsupportedSignature raise.
	^ self.
    ].

    dsSignedInfo := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'SignedInfo') from:dsSignature.
    dsSignatureValue := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'SignatureValue') from:dsSignature.

    self validateSignedInfo:dsSignedInfo against:dsSignatureValue in:anEnvelope prefix:dsPrefix
!

validateSignedInfo:aSignedInfo against:aSignatureValueElement in:anEnvelope prefix:dsPrefix
    |dsCanonicalizationMethod dsSignatureMethod dsReference 
     algorithmName algorithmClass algorithm|

    dsCanonicalizationMethod := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'CanonicalizationMethod') from:aSignedInfo.
    dsSignatureMethod := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'SignatureMethod') from:aSignedInfo.
    dsReference := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'Reference') from:aSignedInfo.

    "/ CanonicalizationMethod supported ?
    "/ TODO: move into instance
    algorithmName := dsCanonicalizationMethod getAttributeNamed: 'Algorithm'.
    (algorithmName = SoapConstants canonicalizationMethodURI) ifFalse:[
	SoapUnsupportedSignature raise.
	^ self.
    ].

    "/ SignatureMethod supported ?
    algorithmName := dsSignatureMethod getAttributeNamed: 'Algorithm'.
    (algorithmName startsWith:(SoapConstants soapDsigURI)) ifFalse:[
	SoapUnsupportedSignature raise.
	^ self.
    ].
    algorithmName := algorithmName copyFrom:(SoapConstants soapDsigURI size + 1).
    algorithmClass := self allSubclasses detect:[:cls | cls algorithmName = algorithmName] ifNone:nil.
    algorithmClass isNil ifTrue:[
	SoapUnsupportedSignature raise.
	^ self.
    ].

    algorithm := algorithmClass new.
    algorithm dsCanonicalizationMethodElement:dsCanonicalizationMethod.
    algorithm dsSignatureMethodElement:dsSignatureMethod.
    algorithm dsSignatureValueElement:aSignatureValueElement.
    algorithm dsReferenceElement:dsReference.
    algorithm dsSignedInfoElement:aSignedInfo.
    algorithm header:anEnvelope header.
    algorithm body:anEnvelope body.
    algorithm dsPrefix:dsPrefix.

    algorithm validateSignature.
! !

!SoapSignatureAlgorithm methodsFor:'accessing'!

body:something
    "set the value of the instance variable 'body' (automatically generated)"

    body := something.
!

dsCanonicalizationMethodElement:something
    "set the value of the instance variable 'dsCanonicalizationMethodElement' (automatically generated)"

    dsCanonicalizationMethodElement := something.
!

dsPrefix:something
    "set the value of the instance variable 'dsPrefix' (automatically generated)"

    dsPrefix := something.
!

dsReferenceElement:something
    "set the value of the instance variable 'dsReferenceElement' (automatically generated)"

    dsReferenceElement := something.
!

dsSignatureElement:something
    dsSignatureElement := something.
!

dsSignatureMethodElement:something
    "set the value of the instance variable 'dsSignatureMethodElement' (automatically generated)"

    dsSignatureMethodElement := something.
!

dsSignatureValueElement:something
    "set the value of the instance variable 'dsSignatureValueElement' (automatically generated)"

    dsSignatureValueElement := something.
!

dsSignedInfoElement:something
    "set the value of the instance variable 'dsSignedInfoElement' (automatically generated)"

    dsSignedInfoElement := something.
!

header:something
    "set the value of the instance variable 'header' (automatically generated)"

    header := something.
!

serviceHandler:something
    serviceHandler := something.
!

signer:something
    "set the value of the instance variable 'signer' (automatically generated)"

    signer := something.
! !

!SoapSignatureAlgorithm methodsFor:'element tree construction'!

addDsCanonicalizationMethodElement
    dsCanonicalizationMethodElement := dsSignedInfoElement createChildNamed:(dsPrefix , ':' , 'CanonicalizationMethod').
    dsCanonicalizationMethodElement putAttributeNamed:'Algorithm' value:SoapConstants canonicalizationMethodURI.
!

addDsDigestMethodElement
    dsDigestMethodElement := dsReferenceElement createChildNamed:(dsPrefix , ':' , 'DigestMethod').
    dsDigestMethodElement putAttributeNamed:'Algorithm' value:SoapConstants soapDsigURI , self digestName.
!

addDsDigestValueElement
    dsDigestValueElement := dsReferenceElement createChildNamed:(dsPrefix , ':' , 'DigestValue').
!

addDsKeyInfoElement
    dsKeyInfoElement := SoapWrapElement new.
    dsKeyInfoElement localName:'KeyInfo'.
    dsKeyInfoElement name:(dsPrefix , ':' , 'KeyInfo').

    dsSignatureElement addSubElement:dsKeyInfoElement.
!

addDsReferenceElement
    dsReferenceElement := dsSignedInfoElement createChildNamed:(dsPrefix , ':' , 'Reference').
    dsReferenceElement putAttributeNamed:'URI' value:('#' , bodyReferenceName).
!

addDsSignatureElement
    dsSignatureElement := SoapElement new.
    dsSignatureElement localName:'Signature'.
    dsSignatureElement name:(dsPrefix , ':' , 'Signature').
    dsSignatureElement declareSoapDsNamespace.

    signatureElement addSubElement:dsSignatureElement.
!

addDsSignatureMethodElement
    dsSignatureMethodElement := dsSignedInfoElement createChildNamed:(dsPrefix , ':' , 'SignatureMethod').
    dsSignatureMethodElement putAttributeNamed:'Algorithm' value:SoapConstants soapDsigURI , self algorithmName.
!

addDsSignatureValueElement
    dsSignatureValueElement := SoapWrapElement new.
    dsSignatureValueElement localName:'SignatureValue'.
    dsSignatureValueElement name:(dsPrefix , ':' , 'SignatureValue').

    dsSignatureElement addSubElement:dsSignatureValueElement.
!

addDsSignedInfoElement
    dsSignedInfoElement := SoapWrapElement "SoapElement"  new.
    dsSignedInfoElement localName:'SignedInfo'.
    dsSignedInfoElement name:(dsPrefix , ':' , 'SignedInfo').

    dsSignatureElement addSubElement:dsSignedInfoElement.
!

addDsTransformsElement
    dsTransformsElement := dsReferenceElement createChildNamed:(dsPrefix , ':' , 'Transforms').
    dsTransformsElement putAttributeNamed:'Algorithm' value:SoapConstants canonicalizationMethodURI.
!

addSignatureElement
    signatureElement := SoapSignature new.
    signatureElement declareSoapSecNamespace.
    signatureElement enableMustUnderstand.

    header addSubElement:signatureElement.
! !

!SoapSignatureAlgorithm methodsFor:'error reporting'!

signatureError:aString
    "report an error that the signature check failed"

    SoapSignatureCheckFailure raiseErrorString:aString
! !

!SoapSignatureAlgorithm methodsFor:'queries'!

algorithmName
    ^ self class algorithmName
!

digestHashStreamClass
    ^ self class digestHashStreamClass
!

digestName
    ^ self class digestName
! !

!SoapSignatureAlgorithm methodsFor:'signature generation'!

digestHashValueFor:anElement
    |hashStream|

    hashStream := self digestHashStreamClass new.
    anElement printCanonicalXmlOn:hashStream.
    ^ hashStream hashValue.

    "/ String streamContents:[:s | anElement printCanonicalXmlOn:s].
    "/ anElement printCanonicalXmlOn:Transcript.
!

generateDigestFor:anElement into:aDigestValueElement
    aDigestValueElement value: (Base64Coder encode:(self digestHashValueFor:anElement)).
!

generateSignatureFor:aSignedInfoElement into:aSignatureValueElement
    |digest sig keyInfo|

    digest := self digestHashValueFor:aSignedInfoElement.

    sig := signer 
            generateSignatureFor:digest 
            keyInfo:(keyInfo := SoapDsKeyInfo new).

    keyInfo isEmpty ifFalse:[
        keyInfo name:(dsPrefix , ':' , 'KeyInfo').
        keyInfo prefixName:dsPrefix.
        keyInfo localName:'KeyInfo'.

        dsSignatureElement addSubElement:keyInfo.
    ].

    aSignatureValueElement value: (Base64Coder encode:sig).
!

generateSignatureInEnvelope:aSoapEnvelope
    dsPrefix := SoapConstants soapDsPrefix.

    body := aSoapEnvelope body.
    header := aSoapEnvelope header.

    bodyReferenceName := 'Body'.
    body defineSecId:bodyReferenceName.

    self addSignatureElement.
    self addDsSignatureElement.
    self addDsSignedInfoElement.
    self addDsCanonicalizationMethodElement.
    self addDsSignatureMethodElement.
    self addDsReferenceElement.
    self addDsTransformsElement.
    self addDsDigestMethodElement.
    self addDsDigestValueElement.
    self generateDigestFor:body into:dsDigestValueElement.

    self addDsSignatureValueElement.

    self generateSignatureFor:dsSignedInfoElement into:dsSignatureValueElement.
! !

!SoapSignatureAlgorithm methodsFor:'validation'!

validateSignatureIn:anEnvelope
    |secNs digestAlgorithmName ref signedObject 
     digestComputed encodedDigestValue digestGiven signatureValue signedInfoDigest
    |

    signatureElement := SignatureHandler extractSingleChildWithLocalName:'Signature' from:header.
    secNs := signatureElement prefixName.

    dsTransformsElement := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'Transforms') from:dsReferenceElement.
    dsDigestMethodElement := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'DigestMethod') from:dsReferenceElement.
    dsDigestValueElement := SignatureHandler extractSingleChildNamed:(dsPrefix , ':' , 'DigestValue') from:dsReferenceElement.

    digestAlgorithmName := dsDigestMethodElement getAttributeNamed:'Algorithm'.
    (digestAlgorithmName isString) ifFalse:[
	SoapMalformedSignature raiseErrorString:'missing Algorithm in DigestMethod'.
	^ self.
    ].
    (digestAlgorithmName startsWith:(SoapConstants soapDsigURI)) ifFalse:[
	SoapUnsupportedSignature raiseErrorString:'wrong Algorithm in DigestMethod'.
	^ self.
    ].
    digestAlgorithmName := digestAlgorithmName copyFrom:(SoapConstants soapDsigURI size + 1).
    digestAlgorithmName = self digestName ifFalse:[
	SoapUnsupportedSignature raiseErrorString:'unsupported Algorithm in DigestMethod'.
	^ self.
    ].

    encodedDigestValue := dsDigestValueElement value.
    encodedDigestValue isString ifFalse:[
	SoapMalformedSignature raiseErrorString:'bad digestValue'.
	^ self.
    ].
    digestGiven := Base64Coder decodingOf:encodedDigestValue.

    ref := dsReferenceElement getAttributeNamed:'URI'.
    ref isEmptyOrNil ifTrue:[
	SoapMalformedSignature raiseErrorString:'missing URI attribute in Reference'.
	^ self.
    ].
    (ref startsWith:'#') ifFalse:[
	"/ must be a local ref.
	SoapMalformedSignature raiseErrorString:'URI must be a localRef'.
	^ self.
    ].
    ref := ref copyFrom:2.

    "/ it must be the id of the body (cannot validate subElements yet)
    ((body getAttributeNamed:(secNs , ':id')) = ref) ifFalse:[
	SoapMalformedSignature raiseErrorString:'Reference URI does not refer to body'.
	^ self.
    ].

    "/ digest OK ?
    signedObject := body.
    digestComputed := self digestHashValueFor:signedObject.
    digestGiven ~= digestComputed ifTrue:[
	SoapSignatureCheckFailure raiseErrorString:'wrong digest'.
	^ self.
    ].

    dsKeyInfoElement := SignatureHandler extractOptionalSingleChildNamed:(dsPrefix , ':' , 'KeyInfo') from:dsSignatureElement.
    dsKeyInfoElement notNil ifTrue:[
	dsKeyInfoElement := SoapDsKeyInfo fromXmlElement:dsKeyInfoElement.
    ].
    serviceHandler context at:#keyInfo put:dsKeyInfoElement.

    "/ signature OK ?
    signatureValue := Base64Coder decodingOf:(dsSignatureValueElement value).
    signedInfoDigest := self digestHashValueFor:dsSignedInfoElement.
    (signer verifySignature:signatureValue of:signedInfoDigest algorithm:self keyInfo:dsKeyInfoElement) ifFalse:[
	SoapSignatureCheckFailure raiseErrorString:'wrong signature'.        
    ].
! !

!SoapSignatureAlgorithm class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xmlsig/SoapSignatureAlgorithm.st,v 1.10 2005/11/01 11:33:41 stefan Exp $'
! !
