static char revision[] = "$Header: /cvs/stx/stx/goodies/stxInExternalWindow/main.c,v 1.12 2005/12/15 19:03:26 cg Exp $";

/*
 * clockInExternalWindow
 *
 * opens an X-window;
 * passes the windowID to STX (via rdoit)
 * lets ST/X open some application in that window.
 *
 * Usage:
 *      make certain that rdoit is found along the PATH
 *      enable rdoit in ST/X
 *      execute: clockInExternalWindow in a shell.
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#ifdef WORKSPACE
# define RDOIT_CMD "Demos::PlayInAlienWindow startWorkspaceInWindowID:"
# define INITIAL_WIDTH      500
# define INITIAL_HEIGHT     300
#else
# ifdef WORKSPACEAPP
#  define RDOIT_CMD "Demos::PlayInAlienWindow startWorkspaceApplicationInWindowID:"
#  define INITIAL_WIDTH      500
#  define INITIAL_HEIGHT     300
# else
#  ifdef CLOCK
#   define RDOIT_CMD "Demos::PlayInAlienWindow startClockInWindowID:"
#   define INITIAL_WIDTH      300
#   define INITIAL_HEIGHT     300
#  else
#   ERROR - dont know which startup-message to use
#  endif
# endif
#endif

#define xTRACE_EVENTS
#define xTRACE_ACTIONS

#ifdef TRACE_EVENTS
# define EVENT_PRINTF(x)        printf x
#else
# define EVENT_PRINTF(x)        /* nothing */
#endif
#ifdef TRACE_ACTIONS
# define ACTION_PRINTF(x)        printf x
#else
# define ACTION_PRINTF(x)        /* nothing */
#endif

#define MAX_HOSTNAME_LEN 128

static char stxHost[MAX_HOSTNAME_LEN];
static int remoteSTX = 0;
static int currWidth = -1, currHeight = -1;

main() {
    Display *dpy;
    int screen;
    int depth;
    Visual visual;
    XSetWindowAttributes xswa;
    Window win, parentWindow;
    XEvent ev;
    int rdoitCode;
    int flags;
    int myWidth=INITIAL_WIDTH, myHeight=INITIAL_HEIGHT;
    int haveSTXChild = 0;
    char *dpyName;

    /*
     * get a display connection
     */
    dpy = XOpenDisplay(NULL);
    if (! dpy) {
	fprintf(stderr, "Cannnot open display\n");
	exit(1);
    }

    stxHost[0] = '\0';
#if 0
    /*
     * stx-host is same as DISPLAY (or localHost, if no DISPLAY is set)
     * extract hostname from DISPLAY shell-variable.
     */
    dpyName = getenv("DISPLAY");
    if (dpyName && (strlen(dpyName) > 0)) {
	char *cp;

	strncpy(stxHost, dpyName, sizeof(stxHost));
	cp = index(stxHost, ':');
	if (cp) {
	    *cp = '\0';
	    remoteSTX = 1;
	}
    }
#endif

    /*
     * see if rdoit is available and allowed
     */
    if (! checkIfRDoitIsPossible()) {
	exit(1);
    }

    /*
     * create a window ...
     */
    screen = DefaultScreen(dpy);
    parentWindow = RootWindow(dpy, screen);
    depth = DefaultDepth(dpy,screen);
    visual.visualid = CopyFromParent;

    flags = 0;
    xswa.override_redirect = 0;
    xswa.background_pixel = BlackPixel(dpy, screen);
    xswa.event_mask = 0
		      /* | KeyPressMask | KeyReleaseMask */
		      /* | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask */
		      /* | PointerMotionMask | ExposureMask | FocusChangeMask */
		      /* | EnterWindowMask | LeaveWindowMask | KeymapStateMask */
		      /* | VisibilityChangeMask */
		      | StructureNotifyMask /* | ResizeRedirectMask */
		      /* | PropertyChangeMask | ColormapChangeMask */
		      ;

    flags = CWEventMask | CWBackPixel;

    win = XCreateWindow(dpy, parentWindow,
		       10, 10,
		       myWidth, myHeight,
		       1, depth, InputOutput, &visual,
		       flags, &xswa);
    if (! win) {
	fprintf(stderr, "Cannnot create new window\n");
	exit(1);
    }


    XMapWindow(dpy, win);
    XStoreName(dpy, win, "This is not an ST/X Window !");

    XSync(dpy, 0);

    while(1) {
	XNextEvent(dpy, &ev);
	switch (ev.type) {
	    /*
	     * whenever my view is resized,
	     * also resize the ST/X child window.
	     */
	    case ConfigureNotify:
		EVENT_PRINTF(("ConfigureNotify\n"));

		if (((XConfigureEvent *)&ev)->window == win) {
		    myWidth = ((XConfigureEvent *)&ev)->width;
		    myHeight = ((XConfigureEvent *)&ev)->height;
		    if (haveSTXChild
		     && ((currWidth != myWidth) || (currHeight != myHeight))) {
			resizeChild(dpy, win, myWidth, myHeight);
			currWidth = myWidth;
			currHeight = myHeight;
		    }
		} else {
		    fprintf(stderr, "oops\n");
		}
		break;

	    /*
	     * when my view is finally mapped (i.e. created and visible),
	     * tell ST/X to create its child window in it.
	     */
	    case MapNotify:
		EVENT_PRINTF(("MapNotify\n"));

		if (! haveSTXChild) {
		    startSTXChildIn(win);
		    haveSTXChild = 1;
		    resizeChild(dpy, win, myWidth, myHeight);
		}
		break;

#ifdef TRACE_EVENTS
	    case DestroyNotify:
		EVENT_PRINTF(("DestroyNotify\n"));
		break;

	    case UnmapNotify:
		EVENT_PRINTF(("UnmapNotify\n"));
		break;

	    case CreateNotify:
		EVENT_PRINTF(("CreateNotify\n"));
		break;

	    case MapRequest:
		EVENT_PRINTF(("MapRequest\n"));
		break;

	    case MappingNotify:
		EVENT_PRINTF(("MappingNotify\n"));
		break;

	    case EnterNotify:
		EVENT_PRINTF(("EnterNotify\n"));
		break;

	    case LeaveNotify:
		EVENT_PRINTF(("LeaveNotify\n"));
		break;

	    case KeymapNotify:
		EVENT_PRINTF(("KeymapNotify\n"));
		break;

	    case FocusIn:
		EVENT_PRINTF(("FocusIn\n"));
		break;

	    case FocusOut:
		EVENT_PRINTF(("FocusOut\n"));
		break;

	    case MotionNotify:
		EVENT_PRINTF(("MotionNotify\n"));
		break;

	    case VisibilityNotify:
		EVENT_PRINTF(("VisibilityNotify\n"));
		break;

	    case Expose:
		EVENT_PRINTF(("Expose\n"));
		break;

	    case PropertyNotify:
		EVENT_PRINTF(("PropertyNotify\n"));
		break;

	    case ReparentNotify:
		EVENT_PRINTF(("ReparentNotify win=%x\n", ((XReparentEvent *)&ev)->window));
		break;

	    case ResizeRequest:
		EVENT_PRINTF(("ResizeRequest win=%x %d/%d\n",
			((XResizeRequestEvent *)&ev)->window,
			((XResizeRequestEvent *)&ev)->width,
			((XResizeRequestEvent *)&ev)->height));

		resizeChild(dpy, win,
				((XResizeRequestEvent *)&ev)->width,
				((XResizeRequestEvent *)&ev)->height);
# if 0
		XResizeWindow(dpy, win,
				((XResizeRequestEvent *)&ev)->width,
				((XResizeRequestEvent *)&ev)->height);
# endif
		break;

	    default:
		EVENT_PRINTF(("other: %d 0x%x\n", ev.type, ev.type));
		break;
#endif
	}
    }
}


/*
 * see if rdoit is available and allowed.
 * return 0 if not.
 */
checkIfRDoitIsPossible() {
    int rdoitCode;
    char rdoitCmd[MAX_HOSTNAME_LEN + 100];

    ACTION_PRINTF(("checking if rdoit (stxHost='%s') is possible ...\n", stxHost));

    sprintf(rdoitCmd, "rdoit -w %s true", stxHost);

    rdoitCode = system(rdoitCmd);
    if (rdoitCode != 0) {
	if ((rdoitCode>>8) == 127) {
	    fprintf(stderr, "Cannnot communicate with ST/X (rdoit command not available)\n");
	} else {
	    fprintf(stderr, "Cannnot communicate with ST/X (rdoit error code %d)\n", rdoitCode>>8);
	}
	ACTION_PRINTF(("... no\n"));
	return 0;
    }
    ACTION_PRINTF(("... yes\n"));
    return 1;
}

/*
 * tell ST/X to open a clock in my view.
 * needs RDoit, to send
 *      Demos::PlayInAlienWindow startClockInWindowID:16rxxxxx
 * or
 *      Demos::PlayInAlienWindow startWorkspaceInWindowID:16rxxxxx
 * to ST/X
 */
startSTXChildIn(myWin)
    Window myWin;
{
    char rdoitCmd[MAX_HOSTNAME_LEN + 200];
    int rdoitCode;

    sprintf(rdoitCmd, "rdoit -w %s '%s 16r%x'",
			stxHost, RDOIT_CMD, myWin);

    ACTION_PRINTF(("Sending WindowId=%x to ST/X\n", myWin));
    ACTION_PRINTF(("--> %s\n", rdoitCmd));

    rdoitCode = system(rdoitCmd);

    if (rdoitCode != 0) {
	fprintf(stderr, "rdoit failed.\n");
	exit(1);
    }

    ACTION_PRINTF(("done.\n"));
}

/*
 * must find the top-child (i.e. the ST/X view)
 * and resize it ...
 * (since ST/X does not get events for my view)
 */
resizeChild(dpy, myWindow, w, h)
    Display *dpy;
    Window myWindow;
{
    Window rootReturn, parentReturn;
    Window* children = (Window *)0;
    unsigned int numChildren;
    int i;

    if (XQueryTree(dpy, myWindow, &rootReturn, &parentReturn, &children, &numChildren)) {
	for (i=0; i < numChildren; i++) {
	    if (children[i]) {
		XResizeWindow(dpy, children[i], w, h);
		ACTION_PRINTF(("resizing child %x to %d/%d\n", children[i], w, h));
		break;
	    }
	}
	if (children) XFree( children );
    }
}
