"{ Package: 'stx:goodies/stxInInvisibleVNCWindow' }"

Object subclass:#InvisibleViewDemo
	instanceVariableNames:'vncLock vncServerPID vncPortNr vncServerIsTerminated
		vncConnection viewerPID connected shownApplication image'
	classVariableNames:''
	poolDictionaries:''
	category:'demos-InvisibleView'
!

!InvisibleViewDemo class methodsFor:'documentation'!

documentation
"
    an example of how to create a hardcopy of an application without making the
    application visible (and without explicit programming for the output).

    The application is opened on an invisible (VNC-) X display, then a hardcopy is
    taken from this.

    to try, evaluate:
        InvisibleViewDemo new demo        
"
! !

!InvisibleViewDemo class methodsFor:'defaults'!

maxNumberOfVNCRestarts

    ^ 10
!

unixVNCCommand

    ^ 'Xvnc'
! !

!InvisibleViewDemo methodsFor:'actions VNC'!

startVNCserver
    |cmd blocked connected numberOfVncStarts|

    connected := false.
    numberOfVncStarts isNil ifTrue:[
        numberOfVncStarts := 1.
    ] ifFalse:[
        numberOfVncStarts := numberOfVncStarts + 1.
    ].

    vncLock isNil ifTrue:[vncLock := Semaphore forMutualExclusion].

    vncServerPID isNil ifTrue:[
        cmd := ('/usr/X11R6/bin/', self class unixVNCCommand).
        cmd asFilename exists ifFalse:[
            cmd := self class unixVNCCommand.
        ].
        vncPortNr isNil ifTrue:[
            vncPortNr := 29
        ].
        cmd := cmd , ' :' , vncPortNr printString , ' nologo -nevershared -localhost -depth 16'
                          , ' -geometry ' , self serverExtent x printString , 'x' , 
                                           self serverExtent y printString.

        blocked := OperatingSystem blockInterrupts.

        vncServerPID := Processor
                   monitor:[
                      vncServerIsTerminated := false.
                      OperatingSystem
                          exec:'/bin/sh'
                          withArguments:(Array with:'sh' with:'-c' with:cmd)
                          environment:nil
                          fileDescriptors:#()
                          fork:true
                          newPgrp:true
                          inDirectory:nil.
                   ]
                   action:[:status |
                      status stillAlive ifFalse:[
                          OperatingSystem closePid:vncServerPID.
                          vncLock critical:[  
                             vncServerPID := nil.
                             self vncServerTerminated.
                          ].
                      ].
                  ].

        blocked ifFalse:[
            OperatingSystem unblockInterrupts
        ].
    ].

    self waitForTerminationOfServer ifTrue:[
"/        Transcript showCR:'server has terminated really - try with next port'.
        (numberOfVncStarts < self class maxNumberOfVNCRestarts) ifTrue:[
            vncPortNr := vncPortNr + 1.
"/            Transcript showCR:'try next port ', vncPortNr asString.
            self startVNCserver.
        ] ifFalse:[
            Dialog warn:'have started servers from port '
                      , (vncPortNr - self class maxNumberOfVNCRestarts) asString
                      , ' to '
                      , vncPortNr
                      , ' - cant connect anyway'.
"/            Transcript showCR:'have started servers from port ', vncPortNr - self class maxNumberOfVNCRestarts, ' to ', vncPortNr.
        ].
    ].
!

stopVNCServer
    vncServerPID notNil ifTrue:[
        OperatingSystem terminateProcess:vncServerPID.
        Delay waitForSeconds:0.2.
        vncServerPID notNil ifTrue:[
            OperatingSystem killProcess:vncServerPID.
            vncServerPID := nil.
        ].
    ].
    self vncServerTerminated.
!

vncServerTerminated
"/    Transcript showCR:'vnc server has terminated'.
"/    'vnc server has terminated' errorPrintCR.
    "/ Dialog information:'vnc server has terminated'.

    vncConnection notNil ifTrue:[
        vncConnection close.
        vncConnection := nil
    ].
    vncServerIsTerminated := true.
! !

!InvisibleViewDemo methodsFor:'demo'!

applicationExtent
    ^ 100 @ 200.
    "/ ^ 1000 @ 2000
!

demo
    [
        Transcript show:self class name; showCR:': starting VNC server...'.
        self startVNCserver.

        Transcript show:self class name; showCR:': opening view on VNC display...'.
        self openViewOnVNCServer.

        Transcript show:self class name; showCR:': grabbing screen image...'.
        self grabScreenImageFromVNCServer.

        Transcript show:self class name; showCR:': printing hardCopy...'.
        self printImage.
    ] ensure:[
        shownApplication notNil ifTrue:[
            shownApplication closeRequest.
            shownApplication windowGroup process waitUntilTerminatedWithTimeout:10.
        ].
        self stopVNCServer.        
    ].

    "
     InvisibleViewDemo new demo
    "
!

grabScreenImageFromVNCServer
    |device|

    device := shownApplication window device.
    image := Image 
                fromScreen:(shownApplication window bounds)
                on:device
                grab:true.

    "
     self new demo
    "
!

openViewOnVNCServer
    |vncXDevice app|

    vncXDevice := XWorkstation newDispatchingFor:(':' , vncPortNr printString).

    app := InvisibleApplication new.
    Screen currentScreenQuerySignal answer:vncXDevice
    do:[
        app allButOpen.
    ].
    app window origin:0@0.
    app window borderWidth:1.
    app window borderColor:(Color black).
    app window extent:(app window preferredExtent).
    app openWindow.
    app window waitUntilVisible.

    shownApplication := app.

    "
     InvisibleViewDemo new demo
    "
!

printImage
    |stream psgc|

    image isNil ifTrue:[^ self].

    Printer supportsPostscript ifFalse:[
        ^ self warn:'No postscript printer configured !!'
    ].

    stream := Printer newNative.
    stream isNil ifTrue:[
        ^ self warn:'Cannot open printer stream !!'
    ].

    psgc := PSGraphicsContext on:stream.  
    psgc displayForm:image x:0 y:0.
    psgc close
!

serverExtent
    ^ 1024 @ 768
! !

!InvisibleViewDemo methodsFor:'private-setup'!

waitForTerminationOfServer
    | cycles |

    cycles := 1.
    [vncServerIsTerminated] whileFalse:[
        (cycles > 3) ifTrue:[^ false].
        cycles := cycles + 1.
        Delay waitForSeconds:0.5.
    ].
    ^ true
! !

!InvisibleViewDemo class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/stxInInvisibleVNCWindow/InvisibleViewDemo.st,v 1.3 2003/08/28 13:30:28 cg Exp $'
! !
