Object subclass:#SocketGateway
	instanceVariableNames:'socket connection stream readQueue writeQueue readWaitTimeoutMs'
	classVariableNames:''
	poolDictionaries:''
	category:'TCPIP'
!


!SocketGateway class methodsFor:'instance creation'!

socket:aSocket 

    |gateway|

    gateway := SocketGateway new.
    gateway socket:aSocket.
    gateway connection:(self socket).
    gateway stream: (gateway connection readAppendStream).
    gateway stream eolMode:#crlf.
  ^ gateway
! !

!SocketGateway methodsFor:'accessing'!

connection
    ^ connection
!

connection:aConnection
    connection := aConnection
!

readQueue

    readQueue isNil ifTrue:[
        readQueue := SharedQueue new
    ].
    ^ readQueue
!

readWaitTimeoutMs

    readWaitTimeoutMs isNil ifTrue:[
        readWaitTimeoutMs := self defaultReadWaitTimeoutMs
    ].
    ^ readWaitTimeoutMs
!

readWaitTimeoutMs:ms
    self stream readWaitTimeoutMs:(readWaitTimeoutMs := ms)
!

socket
    ^ socket
!

socket:aSocket
    socket := aSocket
!

stream
    ^ stream
!

stream:aStream
    stream := aStream
!

writeQueue

    writeQueue isNil ifTrue:[
        writeQueue := SharedQueue new
    ]. 
    ^ writeQueue
! !

!SocketGateway methodsFor:'control'!

eStop

    self readQueue nextPut:nil.

    socket notNil ifTrue:[
        socket shutdown:2.
        socket := nil.
    ].
    stream := nil.
    connection := nil.
!

externalStop
    self readQueue nextPut:nil
!

init
    self gatewayAction.
! !

!SocketGateway methodsFor:'default'!

defaultReadWaitTimeoutMs
    ^50         "milliseconds"
! !

!SocketGateway methodsFor:'process'!

gatewayAction
    [ self gatewayLoop ] repeat
!

gatewayLoop

    |stream wQueue msg|

    stream := self stream.
    wQueue := self writeQueue.

    [(msg := wQueue peek) notNil] whileTrue:[
        stream nextPut:wQueue next.
        stream commit.
    ].

    [stream canReadWithoutBlocking] whileTrue:[
        self readQueue nextPut:stream next
    ].

    Delay waitForMilliseconds:100.      "/ wait 100 milliseconds before next poll

! !

!SocketGateway methodsFor:'receive'!

next
    ^ self readQueue next
! !

!SocketGateway methodsFor:'send'!

nextPut:aByte
    self writeQueue nextPut:aByte
! !

!SocketGateway methodsFor:'synchronization'!

drainReadQueue
    self readQueue nextPut:nil
! !

!SocketGateway class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tcpip/SocketGateway.st,v 1.1 1998/01/16 15:48:22 ca Exp $'
! !
