TCPIPProcess subclass:#TCPIPClient
	instanceVariableNames:'serverPort serverHost serverAddress clientBlock failOpenBlock
		userName password authenticatedFlag lastCommand serverResponse'
	classVariableNames:''
	poolDictionaries:''
	category:'TCPIP'
!


!TCPIPClient class methodsFor:'instance creation'!

address:anAddress port:aServerPort block:aClientBlock 
    ^ self byteAddress:(self stringToBytes:anAddress) port:aServerPort block:aClientBlock
!

byteAddress:aServerAddress port:aServerPort block:aClientBlock 

    |client|

    client := self new.
    client serverPort:aServerPort.
    client serverAddress:aServerAddress.
    client clientBlock:aClientBlock.
  ^ client
!

host:aServerHost port:aServerPort block:aClientBlock 

    |client|

    client := self new.
    client serverPort:aServerPort.
    client serverHost:aServerHost.
    client clientBlock:aClientBlock.
  ^ client
! !

!TCPIPClient class methodsFor:'converting'!

stringToBytes:aString
    |address stream|

    stream  := aString readStream.
    address := ByteArray with:((stream upTo: $.) asNumber)
                         with:((stream upTo: $.) asNumber)
                         with:((stream upTo: $.) asNumber)
                         with:((stream upToEnd ) asNumber).
    stream close.
  ^ address



! !

!TCPIPClient class methodsFor:'examples'!

daytimeExample1

"doit" "TCPIPClient daytimeExample1"

| block serverSaidString tcpipClient |

block :=

[ :client |

	serverSaidString := client getLine.

	Dialog warn: ('The Server Said:', serverSaidString).

	client stop.

].

tcpipClient := TCPIPClient host: 'pepper.eznet.com' port: 13 block: block.

tcpipClient failOpenBlock: tcpipClient dialogFailOpenBlock.

tcpipClient start.

^tcpipClient!

helloClientExample1

"first start the example:" "TCPIPServer helloClientExample"

"doit" "TCPIPClient helloClientExample1"

| block serverSaidString tcpipClient |

block :=

[ :client |

	serverSaidString := client getLine.

	Dialog warn: ('The Server Said:', serverSaidString).

	client stop.

].

tcpipClient := TCPIPClient host: 'localhost' port: 1286 block: block.

tcpipClient failOpenBlock: tcpipClient dialogFailOpenBlock.

tcpipClient start.

^tcpipClient!

helloClientExample2

"first start the example:" "TCPIPServer helloClientExample"

"doit" "TCPIPClient helloClientExample2"

| block serverSaidString tcpipClient |

block :=

[ :client |

	serverSaidString := client getLine.

	Dialog warn: ('The Server Said:', serverSaidString).

	client stop.

].

tcpipClient := TCPIPClient address: '127.0.0.1' port: 1286 block: block.

tcpipClient failOpenBlock: tcpipClient dialogFailOpenBlock.

tcpipClient start.

^tcpipClient! !

!TCPIPClient methodsFor:'accessing'!

authenticatedFlag
    ^ authenticatedFlag
!

clientBlock
    ^ clientBlock
!

clientBlock:aBlock
    clientBlock := aBlock ? [ :aClient| ]
!

clientProcess

    process isNil ifTrue:[
        self socketGateway isNil ifTrue:[
            process := [ self clientBlock value:self ] newProcess
        ] ifFalse:[
            process := [ self socketGateway init ] newProcess
        ].
        process resume.
    ].
    ^ process
!

failOpenBlock
    ^ failOpenBlock
!

failOpenBlock:aBlock
    failOpenBlock := aBlock ? self transcriptFailOpenBlock
!

lastCommand
    ^ lastCommand
!

password
    ^ password
!

password:aPassword
    password := aPassword
!

serverAddress
    ^ serverAddress
!

serverAddress:anAddress
    serverAddress := anAddress
!

serverHost
    ^ serverHost
!

serverHost:aHostName

    aHostName first isDigit ifTrue:[
        self serverAddress:(self class stringToBytes:aHostName)
    ] ifFalse:[
        serverHost := aHostName
    ]
!

serverPort
    ^ serverPort
!

serverPort:aPort
    serverPort := aPort
!

socket

    socket isNil ifTrue:[
        (socket := self serverAddress) isNil ifTrue:[self socketByName]
                                            ifFalse:[self socketByAddress]
    ].
    ^ socket
!

socketByAddress

    ^ self newTCPclientToHost:(self serverAddress)
                         port:(self serverPort)
!

socketByName

    ^ self newTCPclientToHost:(self serverHost)
                         port:(self serverPort)
"/    |port host|
"/
"/    port := self serverPort.
"/    host := self serverHost.
"/
"/    (port notNil and:[host notNil]) ifTrue:[
"/        Object errorSignal handle:[:ex|
"/            self failOpenBlock value.
"/            self stop.
"/            ex return
"/        ] do:[
"/            socket := Socket newTCPclientToHost:host port:port.
"/        ]
"/    ].
"/    ^ socket
"/
!

socketPromise

    [
        Object errorSignal handle:[:ex|
            Transcript show: 'Unable to Open Socket For TCPIPClient'.
            self stop.
            ex return
        ] do:[
            socket := Socket newTCPclientToHost:self serverHost
                                           port:self serverPort.
        ]               
    ] promise
!

userName
    ^ userName
!

userName:aName
    userName := aName
! !

!TCPIPClient methodsFor:'blocks'!

dialogFailOpenBlock

    ^ [ self dialogFailed ]                              
!

dialogFailed

       Dialog warn:

                (       'Unable to Open Socket For TCPIPClient',

                        (String with: Character cr),

                        'Host: ', self serverHost,

                        (String with: Character cr),

                        'Port: ', self serverPort displayString

                )
!

transcriptFailOpenBlock

^                               

[       Transcript show: 'Unable to Open Socket For TCPIPClient'.

]! !

!TCPIPClient methodsFor:'control'!

start

    |newProcess|

    newProcess := [
        self stream notNil ifTrue:[
            self clientProcess
        ].
    ] newProcess.

    newProcess resume.
! !

!TCPIPClient methodsFor:'initialization'!

initialize
    super initialize.

    authenticatedFlag := false.
    self clientBlock:nil.
    self failOpenBlock:nil.
! !

!TCPIPClient methodsFor:'private'!

newTCPclientToHost:aHostOrAddr port:aPortOrService
    "creates a socket
    "
    (aPortOrService notNil and:[aHostOrAddr notNil]) ifTrue:[
        Object errorSignal handle:[:ex|
            self failOpenBlock value.
            self stop.
            ex return
        ] do:[
            socket := Socket newTCPclientToHost:aHostOrAddr
                                           port:aPortOrService
                                    withTimeout:10000.

            socket isNil ifTrue:[
                self failOpenBlock value.
                self stop.
            ]       
        ]
    ].
    ^ socket
! !

!TCPIPClient methodsFor:'status'!

isConnected

^self socket notNil! !

!TCPIPClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tcpip/TCPIPClient.st,v 1.2 1999/04/15 09:06:06 cg Exp $'
! !
