"{ Package: 'stx:goodies/tcpip' }"

Object subclass:#TCPIPProcess
	instanceVariableNames:'socket stream process socketGateway'
	classVariableNames:''
	poolDictionaries:''
	category:'TCPIP'
!


!TCPIPProcess class methodsFor:'instance creation'!

new
    ^ super basicNew initialize
! !

!TCPIPProcess methodsFor:'accessing'!

connection
    "returns the connection, the socket
    "
    ^ self socket

!

process
    ^ process
!

socket
    "get the socket
    "
    ^ socket
!

socketGateway
    "get the gateway
    "
    ^ socketGateway


!

stream
    "get the straem on the socket
    "
    stream isNil ifTrue:[
	(stream := self socket) notNil ifTrue:[
	    stream eolMode:#crlf
	]
    ].
    ^ stream
! !

!TCPIPProcess methodsFor:'control'!

eStop
    "shutdown connection and stop all processes
    "
    self socketGateway notNil ifTrue:[
	self socketGateway eStop
    ].

    process notNil ifTrue:[
	process terminate.
	process := nil.
    ].


!

externalStop
    "stop required from extern; stop process and close connection"

    process notNil ifTrue:[
	process terminate.
	process := nil.
    ].
    self stop.
!

restart
    "restart
    "
    self stop.
    self start.

!

start
    "start process on connection
    "
    self subclassResponsibility


!

stop
    "close connection; terminate process
    "
    |s|

    (s := stream ? socket) notNil ifTrue:[
	s close.
	socket := nil.
    ].
    stream := nil.

    process notNil ifTrue:[
	process terminate.
	process := nil.
    ].

! !

!TCPIPProcess methodsFor:'ioTypeSelection'!

usePollingSocketGateway
    "useing polling gateway
    "
    socketGateway := SocketGateway new.

    socketGateway stream:(self stream).
    socketGateway connection:(self connection).
    socketGateway socket:(self socket).



! !

!TCPIPProcess methodsFor:'retrieving'!

getLine
    "get a line; upto character cr
    "
    ^ self stream upTo:Character cr


!

getLineWithLF
    "read from stream upto lineFeed or carriage return
    "
    |string stream peek|

    string := nil.

    (stream := self stream) notNil ifFalse:[
	(peek := stream peek) notNil ifTrue:[
	    peek = Character lf ifFalse:[
		string := stream upTo:Character cr
	    ] ifTrue:[
		string := ''
	    ].
	    stream next
	]
    ].
    ^ string

!

getWord
    "read from stream upto next space
    "
    ^ self stream upTo:Character space

!

next
    "read next character from gateway or stream
    "
    |stream|

    stream := self socketGateway ? self stream.
  ^ stream next


!

upTo:aChar
    "read upto a character
    "
    ^ self stream upTo:aChar

! !

!TCPIPProcess methodsFor:'sending'!

binary
    "set binary mode
    "
    |stream|

    stream := self socketGateway ? self stream.
    self stream binary.


!

commit
    "commit send buffer
    "
    |stream|

    stream := self socketGateway ? self stream.
    stream commit.

!

nextPut:aByte
    "put a byte on stream
    "
    |stream|

    stream := self socketGateway ? self stream.
    stream nextPut:aByte.

!

nextPutAll:aByteArray
    "put bytes on stream
    "
    |stream|

    stream := self socketGateway ? self stream.
    stream nextPutAll:aByteArray.
!

sendLine:aLine
    "send a line
    "
    |stream|

    stream := self socketGateway ? self stream.
    stream nextPutAll:aLine.
    stream cr.

! !

!TCPIPProcess methodsFor:'synchronisation'!

drainReadQueue
    "drain the read queue
    "
    |gateway|

    (gateway := socketGateway) notNil ifTrue:[
	gateway drainReadQueue
    ].



!

readWaitWithTimeoutMs:ms
    "read wait for a timeout
    "
    ^ self socket readWaitWithTimeoutMs:ms



! !

!TCPIPProcess class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tcpip/TCPIPProcess.st,v 1.3 2006/08/24 08:36:59 cg Exp $'
! !
