TCPIPProcess subclass:#TCPIPServerProcess
	instanceVariableNames:'tcpipServer stopBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'TCPIP'
!


!TCPIPServerProcess class methodsFor:'instance creation'!

socket:aSocket tcpipServer:aTcpipServer
    ^ self socket:aSocket tcpipServer:aTcpipServer ioType:nil
!

socket:aSocket tcpipServer:aTcpipServer ioType:ioType

    |aTCPIPServerProcess|

    aTCPIPServerProcess := self new.
    aTCPIPServerProcess socket: aSocket.
    aTCPIPServerProcess tcpipServer: aTcpipServer.

    ioType == #polling ifTrue:[
        aTCPIPServerProcess usePollingSocketGateway
    ].
    aTCPIPServerProcess start.
  ^ aTCPIPServerProcess
! !

!TCPIPServerProcess methodsFor:'accessing'!

process
    "get the process; in case of not existing yet, the process
     is instanceated
    "
    process isNil ifTrue:[
        process := [
            self prepExceptionsFor:[self tcpipServer serverBlock value:self].

            self socketGateway notNil ifTrue:[
                self prepExceptionsFor:[self socketGateway init]
            ].
            self stop.

        ] newProcess.

        process resume.
    ].
    ^ process
!

socket:aSocket
    "set the socket
    "
    socket := aSocket.

!

stopBlock
    "get the stop block; a block without any arguments
    "
    ^ stopBlock
!

stopBlock:aNoneArgBlock
    "set the stop block; a block without any arguments
    "
    stopBlock := aNoneArgBlock
!

tcpipServer
    "get the tcpip server
    "
    ^ tcpipServer
!

tcpipServer:aTcpipServer
    "set the tcpip server
    "
    tcpipServer := aTcpipServer
! !

!TCPIPServerProcess methodsFor:'control'!

eStop
    "shutdown connection and stop all processes
    "
    socket notNil ifTrue:[
        socket shutdown:2.              "/ no more io-access
        socket := nil.
    ].
    stream := nil.

    tcpipServer notNil ifTrue:[
        tcpipServer closeClientProcessFor:self
    ].
    super eStop.
!

start
    "start process on connection
    "
    self process
!

stop
    "close connection; terminate process
    "
    super stop.

    tcpipServer notNil ifTrue:[
        tcpipServer closeClientProcessFor:self
    ].


! !

!TCPIPServerProcess methodsFor:'exception handling'!

handleException:ex
    "handle an exception; close the connection
    "
    |block|

    tcpipServer notNil ifTrue:[
        tcpipServer logException:ex
    ].

    (block := self stopBlock) notNil ifTrue:[
        self stopBlock value
    ].
    self eStop.
    ex return
!

prepExceptionsFor:aNoneArgBlock
    "setup exception
    "
    Object errorSignal handle:[:ex| self handleException: ex ]
                           do:[ aNoneArgBlock value ]

! !

!TCPIPServerProcess class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tcpip/TCPIPServerProcess.st,v 1.1 1998/01/16 15:48:05 ca Exp $'
! !
