"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#AbstractParser
	instanceVariableNames:'scanner prevToken requestor failBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

AbstractParser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

This class represents abstract parsing behavior.

Instance Variables:

	scanner <a scanner class>       - this parser''s scanner
	prevToken <String + Symbol>     - the last token scanned
	requestor <Controller + Object> - the object invoking the parser, errors are reported to this object
	failBlock <Block>               - this block is evaluated before the parse is aborted'
!


!AbstractParser class methodsFor:'instance creation'!

new

	^super new init
! !

!AbstractParser class methodsFor:'Soul'!

recursiveCopyReplace: anObject with: anotherObject in: aCollection 
        "Hack method to work around literal arrays that contain &lt;nil&gt;."

        ^aCollection collect: [:each | (each isKindOf: Array)
                        ifTrue: [self
                                        recursiveCopyReplace: anObject
                                        with: anotherObject
                                        in: each]
                        ifFalse: [each = anObject
                                        ifTrue: [anotherObject]
                                        ifFalse: [each]]]
! !

!AbstractParser methodsFor:'initialization'!

init

	self scanner: self scannerClass new
! !

!AbstractParser methodsFor:'private'!

scannerClass
	"Answer the preferred class of scanners for this kind of parser."

	self subclassResponsibility
! !

!AbstractParser methodsFor:'scanning'!

endOfInput
	"Some parsers may use the eof token while others may use the eof token type."

	self subclassResponsibility
!

endOfInputToken
	"Answer the token used by my scanner to represent the end of the input."

	^self scanner endOfInputToken
!

endOfInputTokenType
	"Answer the token type used by my scanner to represent the end of the input."

	^self scanner endOfInputTokenType
!

initScannerSource: aString 
	"The scanner is responsible for scanning the first token (i.e. for priming the token 
	buffers)."

	self scanner scanSource: aString
!

nextToken

	^self scanner tokenType
!

nextTokenValue

	^self scanner token
!

scanToken
	"Subclasses may not always want the previous token value and may override this 
	method for efficiency."

	self prevToken: self nextTokenValue.
	self scanner scanToken
! !

!AbstractParser methodsFor:'state accessing'!

failBlock

	^failBlock
!

failBlock: argument 

	failBlock := argument
!

prevToken

	^prevToken
!

prevToken: argument 

	prevToken := argument
!

requestor

	^requestor
!

requestor: argument 

	requestor := argument
!

scanner

	^scanner
!

scanner: argument 

	scanner := argument
! !

!AbstractParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/AbstractParser.st,v 1.7 2006/03/31 17:52:31 cg Exp $'
! !
