"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#AbstractScanner
	instanceVariableNames:'source nextChar token tokenType buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Scanners'
!

AbstractScanner comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I scan a source string and break it up into tokens using mechanisms provided in concrete subclasses.

Instance Variables:
	source                  <ReadStream> - character input stream.
	nextChar                <Character + UndefinedObject> - one-character lookahead buffer for source, nil if no input left.
	token                   <String> - current token buffer.
	tokenType       <String + Symbol> - current token type buffer.
	buffer                  <WriteStream> - character accumulation buffer for tokens.
'
!


!AbstractScanner class methodsFor:'instance creation'!

new

	^super new init
!

scanFrom: aString 

	| newScanner |
	newScanner := self new.
	newScanner scanSource: aString.
	^newScanner
! !

!AbstractScanner class methodsFor:'Soul'!

recursiveCopyReplace: anObject with: anotherObject in: aCollection 
        "Hack method to work around literal arrays that contain &lt;nil&gt;."

        ^aCollection collect: [:each | (each isKindOf: Array)
                        ifTrue: [self
                                        recursiveCopyReplace: anObject
                                        with: anotherObject
                                        in: each]
                        ifFalse: [each = anObject
                                        ifTrue: [anotherObject]
                                        ifFalse: [each]]]
! !

!AbstractScanner methodsFor:'accessing'!

contents

	^self source contents
!

endOfInputToken
	"Answer a token representing the end of the input."

	self subclassResponsibility
!

endOfInputTokenType
	"Answer the token type representing the end of the input."

	self subclassResponsibility
!

errorPosition
	"Answer the source position of the last acceptable character."

	^source position + (self atEnd
			ifTrue: [1]
			ifFalse: [0]) max: 1
!

position

	^self source position
! !

!AbstractScanner methodsFor:'initialization'!

init

	self buffer: (RetractableWriteStream on: (String new: 32))
!

reset
	"Reset the initial state of the scanner before scanning a new source."

	self buffer reset.
	self token: nil.
	self tokenType: nil.
	self nextChar: nil
!

scanSource: aString 
	"Convert the input string to a read stream and scan the first token."

	self reset.
	self source: ((RetractableReadStream on: aString) signalAtEnd:true).
	self nextChar: self source next.
	self scanToken

    "Created: 5.2.1996 / 21:36:43 / stefan"
    "Modified: 5.2.1996 / 21:44:50 / stefan"
! !

!AbstractScanner methodsFor:'scanning'!

backspaceSource
	"When the source is at the end, 'source current' is the last character."

	self atEnd ifFalse: [self source backspace].
	self nextChar: self source current
!

getNextChar
	"Source will answer an empty string when no more input is available. 
	Subclasses may override this to avoid unnecessary buffering."

	self buffer nextPut: self nextChar.
	self nextChar: self source next
!

putBackChar
	"Remove the last character in the buffer and backspace the source. 
	Subclasses may override this to avoid unnecessary buffering."

	self buffer backspace.
	self backspaceSource
!

scanToken
	"Subclasses must compute values for token and tokenType here."

	self subclassResponsibility
!

signalEndOfInput
	"Set scanner to the end-of-input state."

	self tokenType: self endOfInputTokenType.
	self token: self endOfInputToken
! !

!AbstractScanner methodsFor:'state accessing'!

buffer

	^buffer
!

buffer: argument 

	buffer := argument
!

nextChar

	^nextChar
!

nextChar: argument 

	nextChar := argument
!

source

	^source
!

source: argument 

	source := argument
!

token

	^token
!

token: argument 

	token := argument
!

tokenType

	^tokenType
!

tokenType: argument 

	tokenType := argument
! !

!AbstractScanner methodsFor:'testing'!

atEnd

	^self nextChar = self endOfInputToken
! !

!AbstractScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/AbstractScanner.st,v 1.7 2006/03/31 09:15:54 cg Exp $'
! !
