"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

ParseTreeBuilder subclass:#AbstractSyntaxTreeBuilder
	instanceVariableNames:'shamMode'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Parse Trees'
!

AbstractSyntaxTreeBuilder comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I build parse trees by creating specific objects for each kind of node as indicated by the parser directives in grammar productions.  Parser directives currently fall into one of three groups: node (class) names, special directives, and arbitrary message selectors. For a node name, a new instance of the specified node is created and given the values associated with the right-hand side nonterminals, if any, as its children. The special directive ''nil'' simply returns nil. The directive liftRightChild adds any nodes preceeding the right-most node as children to the right-most node, and returns the right-most node. The directive liftLeftChild works in an analogous fashion. Arbitrary message selectors must take the same number of arguments as there are right-hand-side nodes and are invoked as a builder message, thus allowing users to define their own tree-building messages.

Productions of the form ''A -> <tc> => symbol'' are treated specially. The symbol can be either a node name or a one-argument message selector. If it is a node name then create a new instance of that node with the specified attribute value. If it is a message selector then invoke the corresponding operation on the builder with the specified value.

Instance Variables:
	shamMode        <Boolean> - If true DerivationTreeNode-based ASTs are built, otherwise specific ParseTreeNode-based ASTs are built.'
!


!AbstractSyntaxTreeBuilder methodsFor:'accessing'!

setNormalMode

	self shamMode: false
!

setShamMode

	self shamMode: true
! !

!AbstractSyntaxTreeBuilder methodsFor:'initialization'!

init

	super init.
	self setNormalMode
!

reset
	"Empty the node stack and set to normal mode."

	self init
! !

!AbstractSyntaxTreeBuilder methodsFor:'state accessing'!

shamMode

	^shamMode
!

shamMode: argument 

	shamMode := argument
! !

!AbstractSyntaxTreeBuilder methodsFor:'tree building'!

makeNewNode: stringOrSymbol 
        "The argument represents the name of a node class. If in sham mode answer a 
        new derivation tree node for the argument, otherwise answer a new instance of 
        that class."

        ^self shamMode
                ifTrue: [DerivationTreeNode symbol: stringOrSymbol]
                ifFalse: [(self class nameSpace at: stringOrSymbol asSymbol ifAbsent: [self error: 'no class named ' , stringOrSymbol]) new]
! !

!AbstractSyntaxTreeBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/AbstractSyntaxTreeBuilder.st,v 1.5 2002/11/20 13:48:31 cg Exp $'
! !
