"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

EnnaryRegExprNode subclass:#AlternationNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

AlternationNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent the alternation of two or more regular expressions, i.e. child1 | child2 | ... | childN.'
!


!AlternationNode methodsFor:'building parse trees'!

alternateWith: node

	^node alternateWithAltNode: self
!

alternateWithAltNode: node

	node children addAllLast:  children.
	^node
!

alternateWithNonAltNode: node

	self children addFirst: node.
	^self
! !

!AlternationNode methodsFor:'converting'!

asFSAStartingAt: startState endingAt: finalState 

	self children do: [:child | child asFSAStartingAt: startState endingAt: finalState]
! !

!AlternationNode methodsFor:'printing'!

printOn: aStream 

	aStream nextPut: $(.
	self children do: 
		[:child | 
		child printOn: aStream.
		child == self children last ifFalse: [aStream nextPutAll: ' | ']].
	aStream nextPut: $)
! !

!AlternationNode methodsFor:'private'!

processTransformation: lhs 
	| newNode prods |
	prods := OrderedCollection new.
	children do: 
		[:node | 
		newNode := RRPGProductionNode new.
		newNode leftHandSide: lhs.
		newNode rightHandSides: (RRPGRightHandSideNode new symbols: node).
		prods add: newNode].
	^prods
! !

!AlternationNode methodsFor:'testing'!

isAlternationNode

	^true
! !

!AlternationNode methodsFor:'transforming'!

transformUsing: lhsNames withLHS: lhsNode 
	"I am a top-level node (e.g. A : B | C | D). Break-up the 
	right-hand-side into separate productions and answer a collection 
	of these productions."

	| newProds |
	newProds := OrderedCollection new.
	self childrenDo: [:child | newProds add: (self productionNodeClass leftHandSide: lhsNode rightHandSides: child)].
	^newProds
!

transformUsing: lhsNames withLHS: lhsNode alpha: alphaNodes gamma: gammaNodes 
	"Transform productions of the form 'A -> alpha ( beta1 | beta2 ) gamma' 
	into 'A -> alpha beta1 gamma' and 'A -> alpha beta2 gamma'."

	| newProds |
	newProds := OrderedCollection new.
	self childrenDo: [:beta | newProds add: (self createNewProductionWithLHS: lhsNode andRHS: ((OrderedCollection new)
					addAllLast: alphaNodes copy;
					addLast: beta;
					addAllLast: gammaNodes copy;
					yourself))].
	^newProds
! !

!AlternationNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/AlternationNode.st,v 1.5 2002/11/20 13:47:42 cg Exp $'
! !
