"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

EnnaryRegExprNode subclass:#AlternationRangeNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

AlternationRangeNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent the alternation of two or more atomic regular expressions.  My specification encorporates the use of character ranges, e.g. [a-z] (= a | b | c | ... | z).  My children are either simple characters or character ranges, e.g. [a-z0-9!!@#] or [aeiou] (vowels).'
!


!AlternationRangeNode methodsFor:'accessing'!

characters
	"Answer the Set of Characters I represent."

	| chars |
	chars := Set new.
	self childrenDo: [:child | child addCharsTo: chars].
	^chars
! !

!AlternationRangeNode methodsFor:'building parse trees'!

addChildrenInitial: anOrderedCollection 

	anOrderedCollection size = 1
		ifTrue: [self children: anOrderedCollection removeFirst]
		ifFalse: [self error: 'wrong number of children']
! !

!AlternationRangeNode methodsFor:'converting'!

asPureRegExpr
	"Answer a new version of the receiver consisting of only characters, 
	concatenations, alternations, and (star) closures. Also, eliminate single 
	child alternations and concatenations."

	| kids |
	kids := OrderedChildren new.
	self characters do: [:char | kids add: (self makeCharNodeFor: char)].
	^self alternationNodeClass children: kids
! !

!AlternationRangeNode methodsFor:'printing'!

printOn: aStream 

	aStream nextPut: $[.
	self children do: [:child | child printOn: aStream].
	aStream nextPut: $]
! !

!AlternationRangeNode methodsFor:'private'!

makeCharNodeFor: aChar 
	"Answer a new CharacterNode for aChar."

	^self characterNodeClass charSpec: (String with: aChar)
! !

!AlternationRangeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/AlternationRangeNode.st,v 1.5 2002/11/20 13:47:47 cg Exp $'
! !
