"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

EnnaryRegExprNode subclass:#ConcatenationNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

ConcatenationNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent the concatenation of two or more regular expressions, i.e. child1 child2 ... childN.'
!


!ConcatenationNode methodsFor:'building parse trees'!

concatenateWith: node

	^node concatenateWithCatNode: self
!

concatenateWithCatNode: node

	node children addAllLast:  children.
	^node
!

concatenateWithNonCatNode: node

	self children addFirst: node.
	^self
! !

!ConcatenationNode methodsFor:'collecting'!

collectSymbol

	^self children collect: [:sym | sym asGrammarSymbol]
! !

!ConcatenationNode methodsFor:'converting'!

asFSAStartingAt: startState endingAt: finalState 

	| prevState newState |
	prevState := startState.
	self children do: [:child | child == self children last
			ifTrue: [child asFSAStartingAt: prevState endingAt: finalState]
			ifFalse: 
				[newState := self fsaStateClass new.
				child asFSAStartingAt: prevState endingAt: newState.
				prevState := newState]]
! !

!ConcatenationNode methodsFor:'printing'!

printOn: aStream 

	self children do: 
		[:child | 
		child printOn: aStream.
		child == self children last ifFalse: [aStream space]]
! !

!ConcatenationNode methodsFor:'testing'!

isConcatenationNode

	^true
! !

!ConcatenationNode methodsFor:'transforming'!

transformUsing: lhsNames withLHS: lhsNode alpha: alphaNodes gamma: gammaNodes 
	"Transform productions of the form 'A -> alpha (beta)op gamma'. 
	Transform AlternationNodes at the last possible moment to minimize 
	the number of new nonterminals created. Transform all consequitive 
	operator nodes right to left to avoid unnecessary 'tail-splitting', which 
	increases the grammar size."

	| node |
	node := self children reverseDetect: [:child | child needsTransforming]
				ifNone: [self children reverseDetect: [:child | child isAlternationNode]
						ifNone: [^OrderedCollection new]].
	alphaNodes addAllLast: self children.
	self children reverseDo: [:child | child == node
			ifTrue: 
				[alphaNodes removeLast.
				^node
					transformUsing: lhsNames
					withLHS: lhsNode
					alpha: alphaNodes
					gamma: gammaNodes]
			ifFalse: [gammaNodes addFirst: alphaNodes removeLast]].
	self error: 'Control should never reach this point.'
! !

!ConcatenationNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/ConcatenationNode.st,v 1.5 2002/11/20 13:49:38 cg Exp $'
! !
