"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

ParseTreeNode subclass:#DerivationTreeNode
	instanceVariableNames:'symbol children'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Parse Trees'
!

DerivationTreeNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent an arbitrary node in a derivation or abstract tree.  (It would be nice to expand this concept so that heterogeneous parse trees could be built.)

Instance Variables:
	symbol  <String> - node attribute.
	children        <OrderedCollection of: DerivationTreeNode>'
!


!DerivationTreeNode class methodsFor:'instance creation'!

symbol: aSymbol 

	| newNode |
	newNode := self new init.
	newNode symbol: aSymbol.
	^newNode
! !

!DerivationTreeNode methodsFor:'building parse trees'!

addChildrenFirst: anOrderedCollection 

	anOrderedCollection reverseDo: [:child | self addFirstChild: child]
!

addChildrenInitial: anOrderedCollection 

	self children: anOrderedCollection copy
!

addChildrenLast: anOrderedCollection 

	anOrderedCollection reverseDo: [:child | self addLastChild: child]
!

setAttribute: value 

	self symbol: value
! !

!DerivationTreeNode methodsFor:'initialization'!

init

	self children: OrderedCollection new
! !

!DerivationTreeNode methodsFor:'manipulating children'!

addChild: aNode 

	self addLastChild: aNode
!

addFirstChild: aNode 

	self children addFirst: aNode
!

addLastChild: aNode 

	self children addLast: aNode
! !

!DerivationTreeNode methodsFor:'printing'!

printOn: aStream 

	self printOn: aStream level: 0
!

printOn: aStream dots: anInteger 

	anInteger timesRepeat: [aStream nextPutAll: ' . ']
!

printOn: aStream level: level 

	self printOn: aStream dots: level.
	self symbol storeOn: aStream.
	aStream cr.
	self childrenDo: [:child | child printOn: aStream level: level + 1]

    "Created: 5.2.1996 / 21:37:09 / stefan"
    "Modified: 6.2.1996 / 11:19:58 / stefan"
! !

!DerivationTreeNode methodsFor:'state accessing'!

children

	^children
!

children: argument 

	children := argument
!

symbol

	^symbol
!

symbol: argument 

	symbol := argument
! !

!DerivationTreeNode methodsFor:'testing'!

isNonterminal

	^self symbol isNonterminal
!

isTerminal

	^self symbol isTerminal
! !

!DerivationTreeNode methodsFor:'traversing'!

childrenDo: aBlock 

	self children do: aBlock
!

updateChildrenUsing: aBlock 
	"Replace my children according to the value of aBlock."

	self children: (self children collect: [:child | aBlock value: child])
! !

!DerivationTreeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/DerivationTreeNode.st,v 1.5 2002/11/20 13:51:09 cg Exp $'
! !
