"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

GraphNode subclass:#EdgeLabeledDigraphNode
	instanceVariableNames:'edgeLabelMap'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Collections-Graph Nodes'
!

EdgeLabeledDigraphNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent a node in an edge-labeled digraph.

Instance Variables:

	edgeLabelMap            <SetDictionary from: labels to: successors>'
!


!EdgeLabeledDigraphNode class methodsFor:'instance creation'!

new

	^super new init
! !

!EdgeLabeledDigraphNode methodsFor:'accessing'!

successors

	^self edgeLabelMap elements
! !

!EdgeLabeledDigraphNode methodsFor:'converting'!

spaceOptimizeMap
	"Assumes self edgeLabelMap isDeterministic. 
	Note: doing this will dissable the messages #successors, 
	#addSuccessor:withEdgeLabeled:, and any senders of them, 
	since they assume a SetDictionary."

	self edgeLabelMap: self edgeLabelMap asDictionary
! !

!EdgeLabeledDigraphNode methodsFor:'enumerating'!

successorsDo: aBlock 

	self successors do: aBlock
!

successorsExceptSelfDo: aBlock 

	(self successors reject: [:succ | succ = self])
		do: aBlock
! !

!EdgeLabeledDigraphNode methodsFor:'initialization'!

init

	self edgeLabelMap: SetDictionary new
! !

!EdgeLabeledDigraphNode methodsFor:'modifying'!

addSuccessor: node withEdgeLabeled: label 

	self edgeLabelMap at: label add: node
! !

!EdgeLabeledDigraphNode methodsFor:'printing'!

printOn: aStream 

	self hash printOn: aStream.
	aStream nextPutAll: ': '; crtab.
	self edgeLabelMap
		associationsDo: 
			[:assoc | 
			assoc key printOn: aStream.
			aStream nextPutAll: ' ==> '.
			assoc value hash printOn: aStream.
			aStream crtab]
! !

!EdgeLabeledDigraphNode methodsFor:'state accessing'!

edgeLabelMap

	^edgeLabelMap
!

edgeLabelMap: argument 

	edgeLabelMap := argument
! !

!EdgeLabeledDigraphNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/EdgeLabeledDigraphNode.st,v 1.6 2002/11/20 13:49:19 cg Exp $'
! !
