"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

CharacterNode subclass:#EscapedCharNode
	instanceVariableNames:''
	classVariableNames:'SpecialCharMap'
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

EscapedCharNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am a atomic character of a regular expression.  I am specified by a String of the form ''\c'' where each c is some character.  Normally, the character I represent is just c, however, some specifications have special meanings (see comment in class initialization method).'
!


!EscapedCharNode class methodsFor:'initialization'!

initialize
	"The following characters are special: 
        
	spec    ascii   character 
	----    ----    -------- 
	\0              0               null 
	\b              8               backspace 
	\t              9               horizontal tab 
	\n              10              linefeed (UNIX newline \n) 
	\f              12              form feed 
	\r              13              carriage return (Smalltalk cr) 
	\e              27              escape 
	\s              32              space 
	\d              127     delete"
	"EscapedCharNode initialize"

	| dict |
	dict := Dictionary new.
	dict
		 at: $0 put: (Character value: 0);
		 at: $b put: (Character value: 8);
		 at: $t put: (Character value: 9);
		 at: $n put: (Character value: 10);
		 at: $f put: (Character value: 12);
		 at: $r put: (Character value: 13);
		 at: $e put: (Character value: 27);
		 at: $s put: (Character value: 32);
		 at: $d put: (Character value: 127).
	SpecialCharMap := dict
! !

!EscapedCharNode methodsFor:'accessing'!

myChar
	"Answer the Character represented by the receiver. 
	The spec is of the form '\c' where some c's are special (see class comment)."

	| spec char |
	spec := self charSpec.
	(spec size = 2 and: [spec first = $\])
		ifTrue: 
			[char := spec last.
			^self specialChars at: char ifAbsent: [char]]
		ifFalse: [self error: 'Escaped character specifications must be of the form ''\c''.']
!

specialChars

	^SpecialCharMap
! !

!EscapedCharNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/EscapedCharNode.st,v 1.5 2002/11/20 13:50:38 cg Exp $'
! !

EscapedCharNode initialize!
