"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

FSAState subclass:#FSAFinalState
	instanceVariableNames:'literalTokens tokenClasses'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Scanning/Parsing'
!

FSAFinalState comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am a final state of a finite state automata.  If I''m part of a minimal deterministic fsa then it is possible that I represent several final states of some original non-deterministic fsa.  My instance variables are used to distinguish between these various different final states.  Final states for literal tokens (keywords) are represented by name in literalTokens.  Final states for larger token classes are represented by TokenClassifications.  When a token is recognized by this final state, it is first checked against the list of literal tokens.  If not found, it is then classified as belonging to the one token class of which it is a member.  The current implementation does not support overlapping token classes, hence, there can only really be one element in the OrderedCollection.  However, in the future we hope to be able to support overlapping token classes.

Instance Variables:
	literalTokens           <Set of: String> - the literal tokens I recognize.
	tokenClasses    <OrderedCollection of: TokenClassification> - the token classes I recognize.'
!


!FSAFinalState methodsFor:'initialization'!

init

	super init.
	self literalTokens: Set new.
	self tokenClasses: OrderedCollection new
! !

!FSAFinalState methodsFor:'private'!

typeActionHolderClass

	^TokenTypeActionHolder
! !

!FSAFinalState methodsFor:'state accessing'!

literalTokens

	^literalTokens
!

literalTokens: argument 

	literalTokens := argument
!

tokenClasses

	^tokenClasses
!

tokenClasses: argument 

	tokenClasses := argument
! !

!FSAFinalState methodsFor:'state transitions'!

transitionFor: aSymbol 
	"The default for final states is to not raise an exception 
	if no transitions are possible, rather, they answer nil."

	^self transitionFor: aSymbol ifNone: [nil]
! !

!FSAFinalState methodsFor:'testing'!

isFSAFinalState

	^true
! !

!FSAFinalState methodsFor:'token classifying'!

addLiteralToken: literal 

	self literalTokens add: literal
!

addTokenClass: tokenClass 
	"Don't add the same tokenClass twice."

	self tokenClasses detect: [:tc | tc tokenType = tokenClass tokenType]
		ifNone: [self tokenClasses size ~~ 0
				ifTrue: [self error: 'Current implementation only handles non-overlapping token classes.']
				ifFalse: [self tokenClasses add: tokenClass]]
!

tokenTypeAndActionFor: aString 
	"The current implementation does not handle overlapping token classes. Hence, a final state 
	can only represent a literal or a single token class. Therefore, if not a literal then it must be 
	the token class."

	| tc |
	((self literalTokens includes: aString)
		or: [aString size = 0])
		ifTrue: [^self typeActionHolderClass type: aString action: nil].
	tc := self tokenClasses first.
	^self typeActionHolderClass type: tc tokenType action: tc action
! !

!FSAFinalState class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/FSAFinalState.st,v 1.5 2002/11/20 13:47:50 cg Exp $'
! !
