"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

GrammarParseTreeNode subclass:#GrammarNode
	instanceVariableNames:'productions'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Grammar Nodes'
!

GrammarNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am the root of the grammar tree.

Instance Variables:

	productions     <OrderedCollection>     - the productions of the grammar.'
!


!GrammarNode class methodsFor:'instance creation'!

new

	^super new init
! !

!GrammarNode methodsFor:'building parse trees'!

addChildrenFirst: anOrderedCollection 

	self productions addAllFirst: anOrderedCollection
!

addChildrenInitial: anOrderedCollection 

	self productions addAll: anOrderedCollection
! !

!GrammarNode methodsFor:'converting'!

asContextFreeGrammar
	"Answer the CFG representation of this RRPG grammar."

	| transProds lhsNames baseProds |
	transProds := OrderedCollection new.
	lhsNames := Set new.
	"Remove all RRPG grammar operators."
	self childrenDo: [:prod | transProds addAll: (prod convertToCFGUsing: lhsNames)].
	"Convert from parsetree format to grammar format."
	baseProds := transProds
				collect: 
					[:prod | 
					| lhs |
					lhs := prod leftHandSide asGrammarSymbol.
					prod rightHandSides asProductionWithLeftHandSide: lhs].
	^Grammar buildGrammarWithProductions: baseProds
! !

!GrammarNode methodsFor:'initialization'!

init

	self productions: OrderedCollection new
! !

!GrammarNode methodsFor:'printing'!

printOn: aStream 

	self productions do: 
		[:prod | 
		prod printOn: aStream.
		aStream cr]
! !

!GrammarNode methodsFor:'state accessing'!

productions

	^productions
!

productions: argument 

	productions := argument
! !

!GrammarNode methodsFor:'traversing'!

childrenDo: aBlock 
	"Evaluate aBlock for each of my children."

	^self productions do: aBlock
!

updateChildrenUsing: aBlock 
	"Replace my children according to the value of aBlock."

	^self productions: (self productions collect: aBlock)
! !

!GrammarNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/GrammarNode.st,v 1.5 2002/11/20 13:48:05 cg Exp $'
! !
