"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

AbstractScanner subclass:#HandCodedScanner
	instanceVariableNames:'charTypeTable'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Scanners'
!

HandCodedScanner class instanceVariableNames:'charTypeTable'

"
 No other class instance variables are inherited by this class.
"
!

HandCodedScanner comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an abstract class of scanner that scans a source string and breaks it up into tokens using a character type table and hand-coded scanner methods.  Specific type tables are stored in class instance variables of my concrete subclasses.

Instance Variables:
	charTypeTable   <Array of: Symbol> - a local reference to the type table for this class of scanner; the ascii value of each character is mapped to a symbol token type.
'
!


!HandCodedScanner class methodsFor:'class initialization'!

initialize
	"Concrete subclasses must provide a character type table."
	"HandCodedScanner initialize"

	| newTable |
	newTable := Array new: 256 withAll: #xDefault.          "default"
	self charTypeTable: newTable
! !

!HandCodedScanner class methodsFor:'state accessing'!

charTypeTable

	^charTypeTable
!

charTypeTable: argument 

	charTypeTable := argument
! !

!HandCodedScanner methodsFor:'accessing'!

endOfInputToken
	"Answer a token representing the end of the input."

	^nil
!

endOfInputTokenType
	"Answer the token type representing the end of the input."

	^#doIt
!

myTypeTable

	^self class charTypeTable
! !

!HandCodedScanner methodsFor:'initialization'!

init

	super init.
	self charTypeTable: self myTypeTable
! !

!HandCodedScanner methodsFor:'scanning'!

scanToken
	"Scan the next token and compute its token type.  This may be 
	overridden in subclasses for efficiency and customization."

        
	[self atEnd ifTrue: [^self signalEndOfInput].
	self tokenType: (self charTypeTable at: self nextChar asInteger).
	self tokenType == #xDelimiter]
		whileTrue: 
			["Skip delimiters fast, there almost always is one."
			self getNextChar].
	self atStartOfComplexToken
		ifTrue: 
			["perform to compute token & type"
			self perform: tokenType]
		ifFalse: 
			["else just the character"
			self token: self nextChar.
			self getNextChar]
! !

!HandCodedScanner methodsFor:'state accessing'!

charTypeTable

	^charTypeTable
!

charTypeTable: argument 

	charTypeTable := argument
! !

!HandCodedScanner methodsFor:'testing'!

atStartOfComplexToken
	"Answer true if the first character of the tokenType is an $x and false otherwise."

	^(self tokenType at: 1)
		= $x
! !

!HandCodedScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/HandCodedScanner.st,v 1.6 2002/11/20 13:49:14 cg Exp $'
! !

HandCodedScanner initialize!
