"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

TableDrivenParser subclass:#LL1Parser
	instanceVariableNames:'startSymbol'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

LL1Parser class instanceVariableNames:'startSymbol'

"
 The following class instance variables are inherited by this class:

	Tgen::TableDrivenParser - parseTable
	Tgen::AbstractParser - 
	Object - 
"
!

LL1Parser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an LL(1) parser.

Instance Variables:
	parseTable*     <LL1ParserTable> - basic parsing mechanism.
	startSymbol  <Symbol> - my grammars start symbol.

* inherited from AbstractParser'
!


!LL1Parser class methodsFor:'instance creation'!

parseTable: table startSymbol: sym 

	| newParser |
	newParser := self new.
	newParser parseTable: table.
	newParser startSymbol: sym.
	^newParser
! !

!LL1Parser class methodsFor:'state accessing'!

startSymbol

	^startSymbol
!

startSymbol: argument 

	startSymbol := argument
! !

!LL1Parser methodsFor:'converting'!

fastParser

	^OptimizedLL1Parser buildFrom: self
! !

!LL1Parser methodsFor:'exception handling'!

raiseExceptionExpectedToken: aString 

	self raiseNoTransitionExceptionErrorString: 'expecting ' , aString
!

raiseExceptionUnparsedTokens

	self raiseNoTransitionExceptionErrorString: 'unparsed tokens remaining in input'
!

raiseNoTransitionExceptionErrorString: aString 

	self parserErrorSignal raiseErrorString: aString
! !

!LL1Parser methodsFor:'initialization'!

init

	super init.
	self startSymbol: self myStartSymbol
! !

!LL1Parser methodsFor:'parsing'!

parse

	| stack prod |
	stack := Stack new.
	stack push: self startSymbol.
	[stack isEmpty]
		whileFalse: [stack top isTerminal
				ifTrue: [stack top = self nextToken
						ifTrue: 
							[stack pop.
							self scanToken]
						ifFalse: [self raiseExceptionExpectedToken: stack top symbol]]
				ifFalse: 
					[prod := self productionAtNonterminal: stack pop andTerminal: self nextToken.
					prod rightHandSide reverseDo: [:sym | stack push: sym]]].
	self nextToken = self endOfInput ifFalse: [self raiseExceptionUnparsedTokens]
!

parseForDerivationTreeAlternative
	"Derivation trees can be build efficiently during a top-down parse. 
	This method implements this option (see parseForDerivationTree)."

	| stack prod root parent node |
	stack := Stack new.
	root := DerivationTreeNode symbol: self startSymbol.
	stack push: root.
	[stack isEmpty]
		whileFalse: [stack top isTerminal
				ifTrue: [stack top symbol = self nextToken
						ifTrue: 
							[stack pop.
							self scanToken]
						ifFalse: [self raiseExceptionExpectedToken: stack top symbol]]
				ifFalse: 
					[prod := self productionAtNonterminal: stack top symbol andTerminal: self nextToken.
					parent := stack pop.
					prod rightHandSide isEmpty
						ifTrue: 
							[node := DerivationTreeNode symbol: self epsilon.
							parent addChild: node]
						ifFalse: [prod rightHandSide
								reverseDo: 
									[:sym | 
									node := DerivationTreeNode symbol: sym.
									parent addFirstChild: node.
									stack push: node]]]].
	self nextToken = self endOfInput ifFalse: [self raiseExceptionUnparsedTokens].
	^root
!

parseWithTreeBuilder: parseTreeBuilder 
	"Rather than building the tree top-down during the parse, it's easier to save 
	the productions on a stack and build the tree bottom-up after parsing."

	| stack productionStack |
	productionStack := Stack new.
	stack := Stack new.
	stack push: self startSymbol.
	[stack isEmpty]
		whileFalse: [stack top isTerminal
				ifTrue: 
					["cancel matching tokens"
					stack top = self nextToken
						ifTrue: 
							[stack pop.
							self scanToken]
						ifFalse: [self raiseExceptionExpectedToken: stack top]]
				ifFalse: 
					["expand nonterminal"
					productionStack push: (self productionAtNonterminal: stack pop andTerminal: self nextToken)
							@ self nextTokenValue.
					productionStack top x rightHandSide reverseDo: [:sym | stack push: sym]]].
	self nextToken = self endOfInput ifFalse: [self raiseExceptionUnparsedTokens].
	productionStack do: 
		[:prod | 
		self prevToken: prod y.
		parseTreeBuilder processProduction: prod x forParser: self].
	^parseTreeBuilder result
!

productionAtNonterminal: nont andTerminal: term
	^self parseTable productionAtNonterminal: nont andTerminal: term
!

traceParse

	| stack prod |
	self
		 cr;
		 cr;
		 showCR: 'LL Parser trace of:  ' , self scanner contents;
		 cr.
	stack := OrderedCollection new.
	stack addFirst: self startSymbol.
	[stack isEmpty]
		whileFalse: [stack first isTerminal
				ifTrue: [stack first = self nextToken
						ifTrue: 
							[self showCR: 'cancel ''' , stack first asString, ''' from input'.
							stack removeFirst.
							self scanToken]
						ifFalse: [self error: 'raise exception:  top of stack = ''' , stack first asString , ''' next token = ''' , self nextToken asString, '''']]
				ifFalse: 
					[prod := self productionAtNonterminal: stack first andTerminal: self nextToken.
					self showCR: 'apply production ' , prod printString.
					stack removeFirst.
					prod rightHandSide reverseDo: [:sym | stack addFirst: sym]]].
	self nextToken = self endOfInput ifFalse: [self raiseExceptionUnparsedTokens]
! !

!LL1Parser methodsFor:'private'!

epsilon
	"Answer an object used to represent the empty string (epsilon)."

	^'<epsilon>'
!

myStartSymbol

	^self class startSymbol
!

parserErrorSignal

	^LLParserTable noTransitionSignal
! !

!LL1Parser methodsFor:'scanner/parser generation'!

classInitializationMethodTextForClassNamed: name spec: grammarSpec

 | ws |
 ws := self newStreamForMethodRendering.
 ws nextPutAll: 'initialize "' , name , ' initialize  " '.
 ws cr.
 ws nextPutAll: ' "  ' , grammarSpec , ' " '.
 ws nextPut: $".
 grammarSpec do:
  [:ch |
  "double embedded double-quote characters"
  ws nextPut: ch.
  ch = $" ifTrue: [ws nextPut: $"]].
 ws nextPut: $".
 ws nextPutAll: ' |  llParserTable table gp | '.
 ws nextPutAll: self parseTable buildParseTable.
 ws nextPutAll: ' self parseTable:  llParserTable  . '.
 ws nextPutAll: ' self startSymbol:   '.
 self startSymbol printOn: ws.
 ^ws contents
! !

!LL1Parser methodsFor:'state accessing'!

startSymbol

	^startSymbol
!

startSymbol: argument 

	startSymbol := argument
! !

!LL1Parser methodsFor:'testing'!

performsLeftmostDerivation

	^true
! !

!LL1Parser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/LL1Parser.st,v 1.7 2006/03/31 09:18:33 cg Exp $'
! !
