"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Dictionary subclass:#LLParserTable
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Scanning/Parsing'
!

LLParserTable class instanceVariableNames:'noTransitionSignal'

"
 No other class instance variables are inherited by this class.
"
!

LLParserTable comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I implement a two dimensional LL(1) parser table with rows indexed by nonterminals, columns indexed by terminals, and with production table entries.  At the top level I''m a Dictionary from nonterminals to rows; each row is a SetDictionary from terminals to productions.  In deterministic tables (tables without multiple entries) the SetDictionaries can be (and are) converted into simple Dictionaries.'
!


!LLParserTable class methodsFor:'class initialization'!

initialize
	"LLParserTable initialize"

	self noTransitionSignal: (Signal new nameClass: self message: #noTransitionSymbol)
! !

!LLParserTable class methodsFor:'state accessing'!

noTransitionSignal

	^noTransitionSignal
!

noTransitionSignal: argument 

	noTransitionSignal := argument
! !

!LLParserTable methodsFor:'accessing'!

atNonterminal: nont andTerminal: term addProduction: prod 

	| row |
	row := self at: nont ifAbsent: [self at: nont put: self newRow].
	^row at: term add: prod
!

productionAtNonterminal: nont andTerminal: term 

	| row |
	row := self at: nont ifAbsent: [self raiseNoTransitionExceptionErrorString: 'illegal nonterminal symbol encountered:  ' , nont].
	^row at: term ifAbsent: [self raiseNoTransitionExceptionErrorString: 'expecting one of ' , row keys printString , ' but encountered:  ''' , term , '''']
! !

!LLParserTable methodsFor:'converting'!

spaceOptimize
	"Assumes self isDeterministic."

	self associationsDo: [:assoc | self at: assoc key put: assoc value asDictionary]
! !

!LLParserTable methodsFor:'exception handling'!

raiseNoTransitionExceptionErrorString: aString 

	self class noTransitionSignal raiseErrorString: aString
! !

!LLParserTable methodsFor:'private'!

newRow

	^self rowClass new
!

rowClass

	^SetDictionary
! !

!LLParserTable methodsFor:'testing'!

isDeterministic

	self detect: [:row | row isDeterministic not]
		ifNone: [^true].
	^false
! !

!LLParserTable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/LLParserTable.st,v 1.5 2002/11/20 13:51:20 cg Exp $'
! !

LLParserTable initialize!
