"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#LR0Item
	instanceVariableNames:'leftHandSide preDotSymbols postDotSymbols translationSymbol'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Support'
!

LR0Item comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent a context-free grammar production with a ''dot'' somewhere in the right hand side.  I''m used in the construction of LR(0), SLR(1), and LALR(1) parsers.

	leftHandSide -> <preDotSymbols> . <postDotSymbols> => <translationSymbol>


Instance Variables:
	leftHandSide                    <Symbol>
	preDotSymbols           <OrderedCollection of: (String + Symbol)>
	postDotSymbols          <OrderedCollection of: (String + Symbol)>
	translationSymbol       <String + Symbol> - used for generating abstract syntax trees.'
!


!LR0Item class methodsFor:'instance creation'!

initialItemForGrammar: grammar 
	"After conceptually augment the grammar with the production 
	'@@ -> S <endOfInput>', answer the initial item for construction 
	of the LR CFSM."

	^self
		leftHandSide: self augmentedGrammarStartSymbol
		preDotSymbols: OrderedCollection new
		postDotSymbols: (OrderedCollection with: grammar startSymbol with: self endOfInputSymbol)
!

leftHandSide: arg1 preDotSymbols: arg2 postDotSymbols: arg3 

	| newMe |
	newMe := self new.
	newMe leftHandSide: arg1.
	newMe preDotSymbols: arg2.
	newMe postDotSymbols: arg3.
	^newMe
!

leftHandSide: arg1 preDotSymbols: arg2 postDotSymbols: arg3 translationSymbol: arg4 

	| newMe |
	newMe := self new.
	newMe leftHandSide: arg1.
	newMe preDotSymbols: arg2.
	newMe postDotSymbols: arg3.
	newMe translationSymbol: arg4.
	^newMe
! !

!LR0Item class methodsFor:'constants'!

augmentedGrammarStartSymbol

	^#@@
!

endOfInputSymbol

	^Character endOfInput
! !

!LR0Item methodsFor:'accessing'!

augmentedGrammarStartSymbol

	^self class augmentedGrammarStartSymbol
!

endOfInputSymbol

	^self class endOfInputSymbol
! !

!LR0Item methodsFor:'comparing'!

= anItem 

	^anItem isLR0Item
		ifTrue: [self leftHandSide = anItem leftHandSide and: [self preDotSymbols = anItem preDotSymbols and: [self postDotSymbols = anItem postDotSymbols]]]
		ifFalse: [false]
!

hash
	"This is redefined because = is redefined."

	^self leftHandSide hash bitXor: (self preDotSymbols hash bitXor: self postDotSymbols hash)
! !

!LR0Item methodsFor:'conversion'!

asGrammarProduction

	| rhs |
	rhs := OrderedCollection new.
	rhs addAll: self preDotSymbols.
	rhs addAll: self postDotSymbols.
	^self makeGrammarProductionWithLeftHandSide: self leftHandSide rightHandSide: rhs
! !

!LR0Item methodsFor:'copying'!

postCopy

	super postCopy.
	self preDotSymbols: self preDotSymbols copy.
	self postDotSymbols: self postDotSymbols copy.
	^self
! !

!LR0Item methodsFor:'operations'!

nextSymbol
	"Answer the symbol immediately to the right of the dot and nil if none."

	^self postDotSymbols isEmpty
		ifTrue: [nil]
		ifFalse: [self postDotSymbols first]
!

shift
	"Conceptually move the dot past the next symbol."

	self atEnd ifFalse: [self preDotSymbols addLast: self postDotSymbols removeFirst]
! !

!LR0Item methodsFor:'printing'!

printOn: aStream 

	self leftHandSide printOn: aStream.
	aStream nextPutAll: ' -> '.
	self preDotSymbols do: 
		[:sym | 
		sym printOn: aStream.
		aStream space].
	aStream nextPutAll: '. '.
	self postDotSymbols do: 
		[:sym | 
		sym printOn: aStream.
		aStream space]
! !

!LR0Item methodsFor:'private'!

makeGrammarProductionWithLeftHandSide: lhs rightHandSide: rhs 

	^self translationSymbol isNil
		ifTrue: [GrammarProduction leftHandSide: lhs rightHandSide: rhs]
		ifFalse: [TransductionGrammarProduction
				leftHandSide: lhs
				rightHandSide: rhs
				translationSymbol: self translationSymbol]
! !

!LR0Item methodsFor:'state accessing'!

leftHandSide

	^leftHandSide
!

leftHandSide: argument 

	leftHandSide := argument
!

postDotSymbols

	^postDotSymbols
!

postDotSymbols: argument 

	postDotSymbols := argument
!

preDotSymbols

	^preDotSymbols
!

preDotSymbols: argument 

	preDotSymbols := argument
!

translationSymbol

	^translationSymbol
!

translationSymbol: argument 

	translationSymbol := argument
! !

!LR0Item methodsFor:'testing'!

atEnd

	^self postDotSymbols isEmpty
!

isFinalStateItem
	"A final state item is of the form '@@ -> S <endOfInput> . '."

	^self atEnd and: [self leftHandSide = self augmentedGrammarStartSymbol]
!

isLR0Item

	^true
! !

!LR0Item class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/LR0Item.st,v 1.5 2002/11/20 13:45:56 cg Exp $'
! !
