"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

LR0Item subclass:#LR1Item
	instanceVariableNames:'lookahead'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Support'
!

LR1Item comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I associate a lookahead symbol with a context-free grammar production with a ''dot'' somewhere in the right hand side.  I''m used in the construction of LR(1) parsers.

	leftHandSide -> <preDotSymbols> . <postDotSymbols> : <lookahead> => <translationSymbol>


Instance Variables:
	lookahead                       <Symbol>'
!


!LR1Item class methodsFor:'instance creation'!

initialItemForGrammar: grammar 
	"After conceptually augment the grammar with the production 
	'@@ -> S <endOfInput>', answer the initial item for construction 
	of the LR CFSM."

	^self
		leftHandSide: self augmentedGrammarStartSymbol
		preDotSymbols: OrderedCollection new
		postDotSymbols: (OrderedCollection with: grammar startSymbol with: self endOfInputSymbol)
		lookahead: self endOfInputSymbol
!

leftHandSide: arg1 preDotSymbols: arg2 postDotSymbols: arg3 lookahead: arg4 

	| newMe |
	newMe := self new.
	newMe leftHandSide: arg1.
	newMe preDotSymbols: arg2.
	newMe postDotSymbols: arg3.
	newMe lookahead: arg4.
	^newMe
!

leftHandSide: arg1 preDotSymbols: arg2 postDotSymbols: arg3 lookahead: arg4 translationSymbol: arg5 

	| newMe |
	newMe := self new.
	newMe leftHandSide: arg1.
	newMe preDotSymbols: arg2.
	newMe postDotSymbols: arg3.
	newMe lookahead: arg4.
	newMe translationSymbol: arg5.
	^newMe
! !

!LR1Item methodsFor:'accessing'!

lookaheadTail
	"For an lr1 item 'B -> <alpha> . A <beta> : lookahead' answer 
	the string '<beta> lookahead'."

	| string |
	string := self postDotSymbols copy.
	string removeFirst.
	string addLast: self lookahead.
	^string
! !

!LR1Item methodsFor:'comparing'!

= anItem 

	^anItem isLR1Item
		ifTrue: [self lookahead = anItem lookahead and: [super = anItem]]
		ifFalse: [false]
!

hash
	"This is redefined because = is redefined."

	^(self leftHandSide hash bitXor: self lookahead hash)
		bitXor: (self preDotSymbols hash bitXor: self postDotSymbols hash)
! !

!LR1Item methodsFor:'printing'!

printOn: aStream 

	super printOn: aStream.
	aStream nextPutAll: ': '.
	self lookahead printOn: aStream
! !

!LR1Item methodsFor:'state accessing'!

lookahead

	^lookahead
!

lookahead: argument 

	lookahead := argument
! !

!LR1Item methodsFor:'testing'!

isLR1Item

	^true
! !

!LR1Item class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/LR1Item.st,v 1.5 2002/11/20 13:46:06 cg Exp $'
! !
