"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

TableDrivenParser subclass:#LR1Parser
	instanceVariableNames:'finalState'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

LR1Parser class instanceVariableNames:'finalState'

"
 The following class instance variables are inherited by this class:

	Tgen::TableDrivenParser - parseTable
	Tgen::AbstractParser - 
	Object - 
"
!

LR1Parser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an LR parser.

Instance Variables:
	parseTable*     <LRParserState> - basic parsing mechanism, a CFSM.
	finalState              <LRParserState> - final state of my CFSM.

* inherited from AbstractParser'
!


!LR1Parser class methodsFor:'instance creation'!

parseTable: table finalState: state 

	| newParser |
	newParser := self new.
	newParser parseTable: table.
	newParser finalState: state.
	^newParser
! !

!LR1Parser class methodsFor:'state accessing'!

finalState

	^finalState
!

finalState: argument 

	finalState := argument
! !

!LR1Parser methodsFor:'converting'!

fastParser

	^OptimizedLR1Parser buildFrom: self
! !

!LR1Parser methodsFor:'initialization'!

init

	super init.
	self finalState: self myFinalState
! !

!LR1Parser methodsFor:'lalr analysis'!

lalr1AnalyzeConflicts: stateSet originalGrammar: aGrammar 

	^self parseTable lalr1AnalyzeConflicts: stateSet originalGrammar: aGrammar
! !

!LR1Parser methodsFor:'parsing'!

acceptSymbol

	^self lrParserStateClass acceptSymbol
!

actionAt: currState 

	^currState actionFor: self nextToken
!

at: state transitionFor: symbol 

	^state transitionFor: symbol
!

lrParserStateClass

	^LRParserState
!

parse

	| stack action currState |
	stack := Stack new.
	currState := self startState.
	stack push: currState.
	[currState = self finalState or: [(action := self actionAt: currState) = self acceptSymbol]]
		whileFalse: 
			[currState := action isGrammarProduction
						ifTrue: 
							["reduce"
							stack pop: action rightHandSide size.
							self at: stack top transitionFor: action leftHandSide]
						ifFalse: 
							["shift"
							self scanToken.
							action].
			stack push: currState]
!

parseWithTreeBuilder: parseTreeBuilder 

	| stack currState action |
	stack := Stack new.
	currState := self startState.
	stack push: currState.
	[currState = self finalState or: [(action := self actionAt: currState) = self acceptSymbol]]
		whileFalse: 
			[currState := action isGrammarProduction
						ifTrue: 
							["reduce"
							stack pop: action rightHandSide size.
							parseTreeBuilder processProduction: action forParser: self.
							self at: stack top transitionFor: action leftHandSide]
						ifFalse: 
							["shift"
							self scanToken.
							action].
			stack push: currState].
	^parseTreeBuilder result
!

startState

	^self parseTable
!

traceParse

	| stack action currState nextState |
	self
		 cr;
		 cr;
		 showCR: 'LR Parser trace of:  ' , self scanner contents;
		 cr.
	stack := Stack new.
	currState := self startState.
	stack push: currState.
	[currState = self finalState or: [(action := self actionAt: currState) = self acceptSymbol]]
		whileFalse: 
			[currState := action isGrammarProduction
						ifTrue: 
							["reduce"
							stack pop: action rightHandSide size.
							nextState := self at: stack top transitionFor: action leftHandSide.
							self showCR: 'reduce by ' , action printString , ' then goto state ' , nextState hash printString.
							nextState]
						ifFalse: 
							["shift"
							self showCR: 'shift on ''' , self nextToken asString, ''' to state ' , action hash printString.
							self scanToken.
							action].
			stack push: currState]
! !

!LR1Parser methodsFor:'private'!

myFinalState

	^self class finalState
!

parserErrorSignal

	^LRParserState noTransitionSignal
! !

!LR1Parser methodsFor:'state accessing'!

finalState

	^finalState
!

finalState: argument 

	finalState := argument
! !

!LR1Parser methodsFor:'testing'!

performsRightmostDerivation

	^true
! !

!LR1Parser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/LR1Parser.st,v 1.6 2002/11/20 13:46:11 cg Exp $'
! !
