"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

LL1Parser subclass:#OptimizedLL1Parser
	instanceVariableNames:'nonterminals terminals'
	classVariableNames:'NoTransitionSignal'
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

OptimizedLL1Parser class instanceVariableNames:'nonterminals terminals'

"
 The following class instance variables are inherited by this class:

	Tgen::LL1Parser - startSymbol
	Tgen::TableDrivenParser - parseTable
	Tgen::AbstractParser - 
	Object - 
"
!

OptimizedLL1Parser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an LL(1) parser represented efficiently in Array table format.

Instance variables:
	tokenTypeTable <Array of: String>       - the integer mapping for terminals and nonterminals'
!


!OptimizedLL1Parser class methodsFor:'instance creation'!

buildFrom: ll1Parser

	^self new convertToTable: ll1Parser
! !

!OptimizedLL1Parser class methodsFor:'class initialization'!

initialize
	"OptimizedLL1Parser initialize"

	self noTransitionSignal: (Signal new nameClass: self message: #noTransitionSymbol)
! !

!OptimizedLL1Parser class methodsFor:'reconstructing'!

constructGrammarProduction: arg with: prodTable 

	| rhs |
	(arg at: 2) isEmpty
		ifTrue: [rhs := OrderedCollection new]
		ifFalse: 
			[rhs := OrderedCollection new.
			(arg at: 2)
				do: [:ea | rhs addLast: (prodTable at: ea)]].
	^GrammarProduction
		leftHandSide: (prodTable at: (arg at: 1))
		rightHandSide: rhs
!

constructParseTable: table with: prodTable 

	| ea row |
	parseTable := Array new: table size.
	1 to: table size do: 
		[:index | 
		row := Array new: (table at: index) size.
		parseTable at: index put: row.
		1 to: (table at: index) size do: 
			[:i | 
			ea := (table at: index)
						at: i.
			ea isNil ifFalse: [ea isInteger
					ifTrue: [row at: i put: ea]
					ifFalse: [ea size == 2
							ifTrue: [row at: i put: (self constructGrammarProduction: ea with: prodTable)]
							ifFalse: [row at: i put: (self constructTransductionGrammarProduction: ea with: prodTable)]]]]]
!

constructTransductionGrammarProduction: arg with: prodTable 

	| rhs |
	(arg at: 2) isEmpty
		ifTrue: [rhs := OrderedCollection new]
		ifFalse: 
			[rhs := OrderedCollection new.
			(arg at: 2)
				do: [:ea | rhs addLast: (prodTable at: ea)]].
	^TransductionGrammarProduction
		leftHandSide: (prodTable at: (arg at: 1))
		rightHandSide: rhs
		translationSymbol: (prodTable at: (arg at: 3))
! !

!OptimizedLL1Parser class methodsFor:'state accessing'!

noTransitionSignal

	^NoTransitionSignal
!

noTransitionSignal: arg

	NoTransitionSignal := arg
!

nonterminals 
	^nonterminals
!

nonterminals: arg
	nonterminals := arg
!

terminals
	^terminals
!

terminals: arg
	terminals := arg
! !

!OptimizedLL1Parser methodsFor:'accessing'!

myNonterminals

	^self class nonterminals
!

myTerminals

	^self class terminals
!

myTokenTypeTable

	^self class tokenTypeTable
! !

!OptimizedLL1Parser methodsFor:'converting'!

changeToObjectTable: llParseTable 

	| terms objectTable |
	self nonterminals: llParseTable keys asOrderedCollection asArray.
	terms := Set new.
	llParseTable do: [:row | row
			associationsDo: 
				[:assoc | 
				terms add: assoc key.
				assoc value rightHandSide do: [:sym | sym isTerminal ifTrue: [terms add: sym]]]].
	self terminals: terms asOrderedCollection asArray.
	objectTable := Array new: self nonterminals size.
	^self convert: llParseTable to: objectTable
!

convert: llParseTable to: objectTable 
	| nonterms terms row |
	nonterms := self nonterminals.
	terms := self terminals.
	llParseTable
		associationsDo: 
			[:assoc1 | 
			row := Array new: terms size.
			objectTable at: (nonterms indexOf: assoc1 key)
				put: row.
			assoc1 value associationsDo: [:assoc2 | row at: (terms indexOf: assoc2 key)
					put: assoc2 value]].
	^objectTable
!

convertToTable: ll1Parser 

	self scanner: ll1Parser scanner fastScanner.
	self parseTable: (self changeToObjectTable: ll1Parser parseTable).
	self treeBuilder:  ll1Parser treeBuilder.
	self startSymbol: ll1Parser startSymbol
! !

!OptimizedLL1Parser methodsFor:'exception handling'!

endOfInputErrorString

	^'end of input encountered'
!

parserErrorSignal

	^self class noTransitionSignal
!

raiseNoTransitionExceptionErrorString: aString 

	self parserErrorSignal raiseErrorString: aString
!

scannerErrorSignal

	^OptimizedScanner noTransitionSignal
!

standardErrorString

	^'unexpected token encountered:  '
! !

!OptimizedLL1Parser methodsFor:'initialization'!

init

	super init.
	self nonterminals: self myNonterminals.
	self terminals: self myTerminals
! !

!OptimizedLL1Parser methodsFor:'parsing'!

productionAtNonterminal: nont andTerminal: term 
	| nontIndex termIndex prod |
	nontIndex := self nonterminals indexOf: nont.
	termIndex := self terminals indexOf: term.
	^(prod := (self parseTable at: nontIndex)
				at: termIndex) isNil
		ifTrue: [self raiseNoTransitionExceptionErrorString: (term = self endOfInputToken
					ifTrue: [self endOfInputErrorString]
					ifFalse: [self standardErrorString , '''' , term printString , ''''])]
		ifFalse: [prod]
! !

!OptimizedLL1Parser methodsFor:'private'!

parseError

	self raiseNoTransitionExceptionErrorString: (scanner tokenType == self endOfInputToken
			ifTrue: [self endOfInputErrorString]
			ifFalse: [self standardErrorString , '''' , scanner tokenType printString , ''''])
! !

!OptimizedLL1Parser methodsFor:'reconstructing'!

mapProductionToInteger
	"Answer an Array of all grammar symbols - nonterminals, terminals, 
	and translation symbols."

	| transSyms |
	transSyms := Set new.
	parseTable do: [:row | row do: [:ea | ea isGrammarProduction ifTrue: [ea hasTranslation ifTrue: [transSyms add: ea translationSymbol]]]].
	^self nonterminals , self terminals , transSyms asOrderedCollection asArray
!

reconstructOn: aStream 

	| prodTable n |
	prodTable := self mapProductionToInteger.
	aStream nextPutAll: 'prodTable := '.
	prodTable reconstructOn: aStream.
	aStream
		period;
		crtab;
		nextPutAll: 'self nonterminals:  (prodTable copyFrom: 1 to:  ';
		nextPutAll: (n := self nonterminals size) printString;
		nextPutAll: ').';
		crtab;
		nextPutAll: 'self terminals:  (prodTable copyFrom: ';
		nextPutAll: (n + 1) printString;
		nextPutAll: ' to: ';
		nextPutAll: (self terminals size + n) printString;
		nextPutAll: ').';
		crtab;
		nextPutAll: 'table := '.
	self parseTable reconstructOn: aStream using: prodTable.
	aStream
		period;
		crtab;
		nextPutAll: 'self constructParseTable: table  with: prodTable.';
		crtab;
		nextPutAll: 'self startSymbol: '.
	self startSymbol printOn: aStream
! !

!OptimizedLL1Parser methodsFor:'scanner/parser generation'!

classInitializationMethodTextForClassNamed: name spec: grammarSpec
 | ws |
 ws := self newStreamForMethodRendering.
 ws
  nextPutAll: 'initialize';
  crtab;
  nextPut: $";
  nextPutAll: name;
  nextPutAll: ' initialize"';
  crtab;
  nextPut: $".
 grammarSpec do:
  [:ch |
  "double embedded double-quote characters"
  ws nextPut: ch.
  ch = $" ifTrue: [ws nextPut: $"]].
 ws
  nextPut: $";
  cr;
  crtab;
  nextPutAll: '| table prodTable |';
  crtab.
 self reconstructOn: ws.
 ^ws contents
! !

!OptimizedLL1Parser methodsFor:'state accessing'!

nonterminals
	^nonterminals
!

nonterminals: arg
	nonterminals := arg
!

terminals
	^terminals
!

terminals: arg
	terminals := arg
! !

!OptimizedLL1Parser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/OptimizedLL1Parser.st,v 1.7 2006/03/31 17:52:13 cg Exp $'
! !

OptimizedLL1Parser initialize!
