"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

LR1Parser subclass:#OptimizedLR1Parser
	instanceVariableNames:'tokenTypeTable'
	classVariableNames:'NoTransitionSignal'
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

OptimizedLR1Parser class instanceVariableNames:'tokenTypeTable'

"
 The following class instance variables are inherited by this class:

	Tgen::LR1Parser - finalState
	Tgen::TableDrivenParser - parseTable
	Tgen::AbstractParser - 
	Object - 
"
!

OptimizedLR1Parser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an LR parser represented efficietly in Array table form.

Instance variables:
	tokenTypeTable <Array of: String>       - the integer mapping of terminals and nonterminals'
!


!OptimizedLR1Parser class methodsFor:'instance creation'!

buildFrom: fsaParser

	^self new convertToTable: fsaParser
! !

!OptimizedLR1Parser class methodsFor:'class initialization'!

initialize
	"OptimizedLR1Parser initialize"

	self noTransitionSignal: (Signal new nameClass: self message: #noTransitionSymbol)
! !

!OptimizedLR1Parser class methodsFor:'reconstructing'!

constructGrammarProduction: arg with: prodTable 

	| rhs |
	(arg at: 2) isEmpty
		ifTrue: [rhs := OrderedCollection new]
		ifFalse: 
			[rhs := OrderedCollection new.
			(arg at: 2)
				do: [:ea | rhs addLast: (prodTable at: ea)]].
	^GrammarProduction leftHandSide: (prodTable at: (arg at: 1))
		rightHandSide: rhs
!

constructParseTable: table with: prodTable 

	| ea row |
	parseTable := Array new: table size.
	1 to: table size do: 
		[:index | 
		row := Array new: (table at: index) size.
		parseTable at: index put: row.
		1 to: (table at: index) size do: 
			[:i | 
			ea := (table at: index)
						at: i.
			ea isNil ifFalse: [ea isInteger
					ifTrue: [row at: i put: ea]
					ifFalse: [ea size == 2
							ifTrue: [row at: i put: (self constructGrammarProduction: ea with: prodTable)]
							ifFalse: [row at: i put: (self constructTransductionGrammarProduction: ea with: prodTable)]]]]]
!

constructTransductionGrammarProduction: arg with: prodTable 

	| rhs |
	(arg at: 2) isEmpty
		ifTrue: [rhs := OrderedCollection new]
		ifFalse: 
			[rhs := OrderedCollection new.
			(arg at: 2)
				do: [:ea | rhs addLast: (prodTable at: ea)]].
	^TransductionGrammarProduction
		leftHandSide: (prodTable at: (arg at: 1))
		rightHandSide: rhs
		translationSymbol: (prodTable at: (arg at: 3))
! !

!OptimizedLR1Parser class methodsFor:'state accessing'!

noTransitionSignal

	^NoTransitionSignal
!

noTransitionSignal: argument 

	NoTransitionSignal := argument
!

tokenTypeTable

	^tokenTypeTable
!

tokenTypeTable: arg 

	tokenTypeTable := arg
! !

!OptimizedLR1Parser methodsFor:'accessing'!

myTokenTypeTable

	^self class tokenTypeTable
!

startState

	^1
! !

!OptimizedLR1Parser methodsFor:'converting'!

assignNextIDAfter: id toSuccessorOf: state 

	| nextID nextState |
	nextID := id + 1.
	state edgeLabelMap
		associationsDo: 
			[:assoc | 
			tokenTypeTable add: assoc key.
			nextState := assoc value.
			nextState stateID isNil
				ifTrue: 
					[nextState stateID: nextID.
					nextID := self assignNextIDAfter: nextID toSuccessorOf: nextState]].
	state reduceMap associationsDo: [:assoc | tokenTypeTable add: assoc key].
	^nextID
!

changeToObjectTable: lrParserState 

	| sizePlusOne objectTable |
	lrParserState stateID notNil ifTrue: [lrParserState nilOutStateIDs].
	lrParserState stateID: self startState.
	self tokenTypeTable: Set new.
	sizePlusOne := self assignNextIDAfter: self startState toSuccessorOf: lrParserState.
	self tokenTypeTable: tokenTypeTable asOrderedCollection asArray.
	objectTable := Array new: sizePlusOne - 1.
	^self convert: lrParserState to: objectTable
!

convert: state to: objectTable 
	"I try to create a table that maps state ( represented by integer ) to state or state to 
	production"

	| arr nextState |
	arr := Array new: self tokenTypeTable size.
	objectTable at: state stateID put: arr.
	state edgeLabelMap
		associationsDo: 
			[:assoc | 
			nextState := assoc value.
			(objectTable at: nextState stateID) isNil ifTrue: [self convert: nextState to: objectTable].
			arr at: (tokenTypeTable indexOf: assoc key)
				put: nextState stateID].
	state reduceMap associationsDo: [:assoc | arr at: (tokenTypeTable indexOf: assoc key)
			put: assoc value first].
	^objectTable
!

convertToTable: lr1Parser 

	self scanner: lr1Parser scanner fastScanner.
	self parseTable: (self changeToObjectTable: lr1Parser parseTable).
	self treeBuilder:  lr1Parser treeBuilder.
	self finalState: lr1Parser finalState stateID
! !

!OptimizedLR1Parser methodsFor:'exception handling'!

endOfInputErrorString

	^'end of input encountered'
!

parserErrorSignal

	^self class noTransitionSignal
!

raiseNoTransitionExceptionErrorString: aString 

	self parserErrorSignal raiseErrorString: aString
!

scannerErrorSignal

	^OptimizedScanner noTransitionSignal
!

standardErrorString

	^'unexpected token encountered:  '
! !

!OptimizedLR1Parser methodsFor:'initialization'!

init

	super init.
	self tokenTypeTable: self myTokenTypeTable
! !

!OptimizedLR1Parser methodsFor:'parsing'!

actionAt: currState 

        | action tok |
        (action := (parseTable at: currState)
                                at: (tokenTypeTable indexOf: (tok := self nextToken))) isNil ifTrue: [(scanner finalStateTable includes: currState)
                        ifTrue: [^#accept]
                        ifFalse: [self parseError]].
        ^action
!

at: currState transitionFor: symbol 

	| value |
	(value := (parseTable at: currState)
				at: (tokenTypeTable indexOf: symbol)) isNil ifTrue: [self raiseNoTransitionExceptionErrorString: (symbol = self endOfInputToken
				ifTrue: [self endOfInputErrorString]
				ifFalse: [self standardErrorString , '''' , symbol printString , ''''])].
	^value
! !

!OptimizedLR1Parser methodsFor:'private'!

parseError

	self raiseNoTransitionExceptionErrorString: (scanner tokenType == self endOfInputToken
			ifTrue: [self endOfInputErrorString]
			ifFalse: [self standardErrorString , '''' , scanner tokenType printString , ''''])
! !

!OptimizedLR1Parser methodsFor:'reconstructing'!

mapProductionToInteger
	"Answer an Array of all grammar symbols - nonterminals, terminals, 
	and translation symbols."

	| transSyms |
	transSyms := Set new.
	parseTable do: [:row | row do: [:ea | ea isGrammarProduction ifTrue: [ea hasTranslation ifTrue: [transSyms add: ea translationSymbol]]]].
	^self tokenTypeTable , transSyms asOrderedCollection asArray
!

reconstructOn: aStream 
	"Recreate a parse table and a token type table"

	| prodTable |
	prodTable := self mapProductionToInteger.
	aStream nextPutAll: 'prodTable := '.
	prodTable reconstructOn: aStream.
	aStream
		period;
		crtab;
		nextPutAll: 'self tokenTypeTable:  (prodTable copyFrom: 1 to:  ';
		nextPutAll: tokenTypeTable size printString;
		nextPutAll: ').';
		crtab;
		nextPutAll: 'table := '.
	self parseTable reconstructOn: aStream using: prodTable.
	aStream
		period;
		crtab;
		nextPutAll: 'self constructParseTable: table  with: prodTable.';
		crtab;
		nextPutAll: 'self finalState: '.
	self finalState printOn: aStream
! !

!OptimizedLR1Parser methodsFor:'scanner/parser generation'!

classInitializationMethodTextForClassNamed: name spec: grammarSpec
 | ws |
 ws := self newStreamForMethodRendering.
 ws
  nextPutAll: 'initialize';
  crtab;
  nextPut: $";
  nextPutAll: name;
  nextPutAll: ' initialize"';
  crtab;
  nextPut: $".
 grammarSpec do:
  [:ch |
  "double embedded double-quote characters"
  ws nextPut: ch.
  ch = $" ifTrue: [ws nextPut: $"]].
 ws
  nextPut: $";
  cr;
  crtab;
  nextPutAll: '| table prodTable |';
  crtab.
 self reconstructOn: ws.
 ^ws contents
! !

!OptimizedLR1Parser methodsFor:'state accessing'!

tokenTypeTable

	^tokenTypeTable
!

tokenTypeTable: arg 

	tokenTypeTable := arg
! !

!OptimizedLR1Parser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/OptimizedLR1Parser.st,v 1.7 2006/03/31 17:51:57 cg Exp $'
! !

OptimizedLR1Parser initialize!
