"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

FSABasedScanner subclass:#OptimizedScanner
	instanceVariableNames:'finalStateTable'
	classVariableNames:'NoTransitionSignal'
	poolDictionaries:''
	category:'Tgen-Compilers-Scanners'
!

OptimizedScanner class instanceVariableNames:'finalStateTable tokenTable'

"
 The following class instance variables are inherited by this class:

	Tgen::FSABasedScanner - fsa
	Tgen::AbstractScanner - 
	Object - 
"
!

OptimizedScanner comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an abstract class of scanner that scans a source string and breaks it up into tokens
using a table created by converting FSA to integer.

instance Variables:
	finalStateTable         - a table that maps integer ( represented as final state ) to 
						   literal tokens and token classes.
'
!


!OptimizedScanner class methodsFor:'instance creation'!

buildFrom: fsaScanner

	^self new convertToTable: fsaScanner
! !

!OptimizedScanner class methodsFor:'class initialization'!

initialize
	"OptimizedScanner initialize"

	self noTransitionSignal: (Signal new nameClass: self message: #noTransitionSymbol).
! !

!OptimizedScanner class methodsFor:'reconstructing'!

constructFinalStateTable: arg 

	finalStateTable := Array new: arg size.
	1 to: arg size do: [:index | finalStateTable at: index put: ((arg at: index) isNil
				ifTrue: [nil]
				ifFalse: [Array with: ((arg at: index)
							at: 1)
						with: (self constructTokenClassification: ((arg at: index)
									at: 2))])]
!

constructTokenClassification: aCollection 

	| tc ea arr |
	aCollection size == 1
		ifTrue: 
			[tc := aCollection first.
			^Array with: (TokenClassification
					tokenType: (tc at: 1)
					action: (tc at: 2))]
		ifFalse: 
			[arr := Array new: aCollection size.
			1 to: aCollection size do: 
				[:index | 
				ea := aCollection at: index.
				arr at: index put: (TokenClassification
						tokenType: (ea at: 1)
						action: (ea at: 2))].
			^arr]
! !

!OptimizedScanner class methodsFor:'state accessing'!

finalStateTable

	^finalStateTable
!

finalStateTable: arg 

	finalStateTable := arg
!

noTransitionSignal

	^NoTransitionSignal
!

noTransitionSignal: arg 

	NoTransitionSignal := arg
!

tokenTable

	^tokenTable
!

tokenTable: arg 

	tokenTable := arg
! !

!OptimizedScanner methodsFor:'accessing'!

myFinalStateTable

	^self class finalStateTable
!

startState

	^1
! !

!OptimizedScanner methodsFor:'converting'!

assignNextIDAfter: id toSuccessorOf: state 
	"I try to assing a number to fsa in order to create a fsa table."

	| nextID nextState |
	nextID := id + 1.
	state edgeLabelMap
		associationsDo: 
			[:assoc | 
			nextState := assoc value.
			nextState stateID isNil
				ifTrue: 
					[nextState stateID: nextID.
					nextState isFSAFinalState ifTrue: [(finalStateTable includes: nextState)
							ifFalse: [finalStateTable at: nextID put: nextState]].
					nextID := self assignNextIDAfter: nextID toSuccessorOf: nextState]].
	^nextID
!

changeFSAToObjectTable: fsaState 

	| sizePlusOne objectTable  |
	fsaState stateID notNil ifTrue: [fsaState nilOutStateIDs].
	fsaState stateID:  self startState.
	self finalStateTable: Dictionary new.
	sizePlusOne := self assignNextIDAfter: self startState toSuccessorOf: fsaState.
	objectTable := Array new: sizePlusOne - 1.
	self convert: fsaState to: objectTable.
	self modifyFSAFinalStates: sizePlusOne - 1.             "convert Dictionary to Array for speed"
	^objectTable
!

convert: state to: objectTable 
	"I try to create a table that maps state ( represented by integer ) to state"

	| arr nextState |
	arr := Array new: 127.
	objectTable at: state stateID put: arr.
	state edgeLabelMap
		associationsDo: 
			[:assoc | 
			nextState := assoc value.
			(objectTable at: nextState stateID) isNil ifTrue: [self convert: nextState to: objectTable].
			arr at: assoc key asInteger put: nextState stateID].
	^objectTable
!

convertToTable: fsaScanner 

	self fsa: (self changeFSAToObjectTable: fsaScanner fsa)
!

modifyFSAFinalStates: index 
	"Convert Dictionary and its values to Array of Array"

	| tokenSet table |
	table := Array new: index.
	finalStateTable do: 
		[:st | 
		tokenSet := Array new: 2.
		tokenSet at: 1 put: st literalTokens asOrderedCollection asArray; at: 2 put: st tokenClasses asArray.
		table at: st stateID put: tokenSet].
	self finalStateTable: table
! !

!OptimizedScanner methodsFor:'exception handling'!

endOfInputErrorString

	^'end of input encountered'
!

raiseNoTransitionExceptionErrorString: aString 

	self class noTransitionSignal raiseErrorString: aString
!

standardErrorString

	^'illegal character encountered:  '
! !

!OptimizedScanner methodsFor:'initialization'!

init

	super init.
	self finalStateTable: self myFinalStateTable
! !

!OptimizedScanner methodsFor:'private'!

at: state transitionFor: char 

	| value |
	(value := (fsa at: state)
				at: char asInteger) isNil ifTrue: [(finalStateTable at: state) isNil ifTrue: [self raiseNoTransitionExceptionErrorString: (char == self endOfInputToken
					ifTrue: [self endOfInputErrorString]
					ifFalse: [self standardErrorString , '''' , char printString , ''''])]].
	^value
! !

!OptimizedScanner methodsFor:'reconstructing'!

reconstructFSAOn: aStream 

	aStream nextPutAll: 'self fsa: '.
	fsa reconstructOn: aStream.
	aStream period; crtab
!

reconstructFinalStateTableOn: aStream 

	aStream nextPutAll: 'table := '.
	finalStateTable reconstructOn: aStream.
	aStream
		period;
		crtab;
		nextPutAll: 'self constructFinalStateTable: table'
!

reconstructOn: aStream 
	"Recreate fsa and final state tables"

	self reconstructFSAOn: aStream.
	self reconstructFinalStateTableOn: aStream
! !

!OptimizedScanner methodsFor:'scanner generation'!

classInitializationMethodTextForClassNamed: name spec: tokenSpec
 | ws |
 ws := self newStreamForMethodRendering.
 ws
  nextPutAll: 'initialize';
  crtab;
  nextPut: $";
  nextPutAll: name;
  nextPutAll: ' initialize"';
  crtab;
  nextPut: $".
 tokenSpec do:
  [:ch |
  "double embedded double-quote characters"
  ws nextPut: ch.
  ch = $" ifTrue: [ws nextPut: $"]].
 ws
  nextPut: $";
  cr;
  crtab;
  nextPutAll: '| table |';
  crtab.
 self reconstructOn: ws.
 ^ws contents
! !

!OptimizedScanner methodsFor:'scanning'!

at: state tokenTypeAndActionFor: aString 
	"The current implementation does not handle overlapping token classes. Hence, a final state 
	can only represent a literal or a single token class. Therefore, if not a literal then it must be 
	the token class."

	| tc |
	(((finalStateTable at: state)
		at: 1)
		includes: aString)
		ifTrue: [^TokenTypeActionHolder type: aString action: nil].
	tc := ((finalStateTable at: state)
				at: 2) first .
	^TokenTypeActionHolder type: tc tokenType action: tc action
!

getNextChar
	"Source will answer an eof char when no more input is available. 
	Subclasses may override this to avoid unnecessary buffering."

	buffer nextPut: nextChar.
	nextChar := source next
!

signalEndOfInput
	"Set scanner to the end-of-input state."

	tokenType := token := self endOfInputToken
! !

!OptimizedScanner methodsFor:'state accessing'!

finalStateTable

	^finalStateTable
!

finalStateTable: arg 

	finalStateTable := arg
! !

!OptimizedScanner methodsFor:'testing'!

atEnd

	^nextChar == self endOfInputToken               "end-of-file character"
! !

!OptimizedScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/OptimizedScanner.st,v 1.7 2006/03/31 09:16:10 cg Exp $'
! !

OptimizedScanner initialize!
