"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#OrderedPair
	instanceVariableNames:'x y'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Support'
!

OrderedPair comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

An OrderedPair extends the concept of a Point from Numbers to Objects. It is often
convenient to associate two objects together or to return a pair of objects from a
method.  OrderedPair provides the mechanism to do this without the inconvenience
of verbose syntax (as would be required if an Array or OrderedCollection were used).
The main instance creation method for OrderedPairs is the binary operator @.  This
operator is defined in Object and (now) overridden in Number so that numerical
points are treated and created in the traditional manner.

instance variables:
	x       <Object>        the first component of the pair
	y       <Object>        the second component of the pair
'
!


!OrderedPair class methodsFor:'instance creation'!

x: anObject y: anotherObject
	"Answer a new OrderedPair whose x element is anObject and whose y element is anotherObject."

	^self new x: anObject y: anotherObject
! !

!OrderedPair methodsFor:'accessing'!

x
	"answer the first element of the pair"

	^x
!

y
	"answer the second element of the pair"

	^y
! !

!OrderedPair methodsFor:'comparing'!

= anOrderedPair
	"answers whether two OrderedPairs are equal"

	^self species = anOrderedPair species
		and: [(x = anOrderedPair x) & (y = anOrderedPair y)]
!

hash
	"answer the receiver's hash value"

	^(x hash bitShift: -1) + (y hash bitShift: -2)
! !

!OrderedPair methodsFor:'initialization'!

x: anObject y: anotherObject
	"initializes an OrderedPair"

	x := anObject.
	y := anotherObject.
! !

!OrderedPair methodsFor:'printing'!

printOn: aStream 
	"Append to the argument aStream a sequence of characters that identifies the receiver."

	x printOn: aStream.
	aStream nextPutAll: ' @ '.
	y printString printOn: aStream.
! !

!OrderedPair class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/OrderedPair.st,v 1.6 2002/11/20 13:45:44 cg Exp $'
! !
