"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#PartitionTransitionMap
	instanceVariableNames:'partition transitionMap'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Support'
!

PartitionTransitionMap comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am a partition-based transition map for an ordinary fsa state.  I am used as an intermediate computational object in the dfsa minimization algorithm (see #asMinimalDFSA in class FiniteStateAutomata).  My responsibilities include adding new partition-based transitions and testing for transition map equivalence.

Instance Variables:
	partition                       <Partition> - the partition for the super state that I represent.
	transitionMap   <Dictionary from: (transition symbol) to: Partition>

where <Partition> = <Set of: (FSAState + FSAFinalState)>'
!


!PartitionTransitionMap class methodsFor:'instance creation'!

forPartition: pt 

	^self new initWithPartition: pt
! !

!PartitionTransitionMap methodsFor:'comparing'!

hasSameTransitionMapAs: aPTMap 
	"Two partition transition maps are equivalent if they have equivalent transition maps."

	| map1 map2 keys1 keys2 values1 values2 |
	^aPTMap isPartitionTransitionMap
		ifTrue: 
			["First, check map sizes."
			map1 := self transitionMap.
			map2 := aPTMap transitionMap.
			map1 size ~= map2 size ifTrue: [^false].
			"Next, check map domain sizes."
			keys1 := map1 keys.
			keys2 := map2 keys.
			keys1 size ~= (keys1 union: keys2) size ifTrue: [^false].
			"Last, check map range sizes."
			values1 := map1 valuesAsSet.
			values2 := map2 valuesAsSet.
			values1 size ~= (values1 union: values2) size ifTrue: [^false].
			^true]
		ifFalse: [false]
! !

!PartitionTransitionMap methodsFor:'initialization'!

initWithPartition: pt 

	self transitionMap: Dictionary new.
	self partition: pt
! !

!PartitionTransitionMap methodsFor:'printing'!

printOn: aStream 

	aStream nextPutAll: self partition hash printString; cr.
	self transitionMap printOn: aStream
! !

!PartitionTransitionMap methodsFor:'state accessing'!

partition

	^partition
!

partition: argument 

	partition := argument
!

transitionMap

	^transitionMap
!

transitionMap: argument 

	transitionMap := argument
! !

!PartitionTransitionMap methodsFor:'state transitions'!

goto: aPartition on: aSymbol 

	(self transitionMap includesKey: aSymbol)
		ifTrue: [self error: 'these are supposed to be deterministic, what gives?'].
	self transitionMap at: aSymbol put: aPartition
! !

!PartitionTransitionMap methodsFor:'testing'!

isPartitionTransitionMap

	^true
! !

!PartitionTransitionMap class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/PartitionTransitionMap.st,v 1.5 2002/11/20 13:50:11 cg Exp $'
! !
