"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#ProductionPartition
	instanceVariableNames:'leftHandSide problemProductions otherProductions'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Support'
!

ProductionPartition comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent a partition of productions for a given nonterminal into two sets, potentially problematic productions and others.  I am currently used for detecting left-recursive productions and productions with common prefixes.

Instance Variables:
	leftHandSide                    <Symbol> - the left hand side nonterminal of my productions.
	problemProductions      <Set of: GrammarProduction> - when used for left-recursion
							<Set of: (Set of: GrammarProduction)> - when used for left-factoring.
	otherProductions                <Set of: GrammarProduction>.'
!


!ProductionPartition class methodsFor:'instance creation'!

partitionProdSetForLeftFactoring: prodSet 
	"ProdSet contains all productions for a given nonterminal. Partition these 
	productions into two sets: a set of sets of problem productions with common 
	prefixes and all the rest. Answer the partitioned productions."

	| prodPrefix newPP |
	prodSet isEmpty ifTrue: [self error: 'cannot partition an empty prodSet'].
	prodPrefix := SetDictionary new.
	prodSet do: [:prod | prodPrefix at: (prod rightHandSide isEmpty
				ifTrue: [self epsilon]
				ifFalse: [prod rightHandSide first])
			add: prod].
	newPP := self new initWithLeftHandSide: prodSet first leftHandSide.
	prodPrefix do: [:set | set size > 1
			ifTrue: [newPP addProblemProduction: set]
			ifFalse: [newPP addOtherProduction: set first]].
	^newPP
!

partitionProdSetForLeftRecursion: prodSet 
	"ProdSet contains all productions for a given nonterminal. Partition these 
	productions into two sets: left-recursive problem productions and other 
	non-left-recursive productions. Answer the partitioned productions."

	| newPP |
	prodSet isEmpty ifTrue: [self error: 'cannot partition an empty prodSet'].
	newPP := self new initWithLeftHandSide: prodSet first leftHandSide.
	prodSet do: [:prod | (prod rightHandSide isEmpty or: [prod leftHandSide ~= prod rightHandSide first])
			ifTrue: [newPP addOtherProduction: prod]
			ifFalse: [newPP addProblemProduction: prod]].
	^newPP
! !

!ProductionPartition class methodsFor:'constants'!

epsilon
	"Answer an object used to represent the empty string (epsilon)."

	^EpsilonNode epsilon
! !

!ProductionPartition methodsFor:'adding'!

addOtherProduction: prod 

	self otherProductions add: prod
!

addProblemProduction: prod 

	self problemProductions add: prod
! !

!ProductionPartition methodsFor:'initialization'!

initWithLeftHandSide: lhs 

	self leftHandSide: lhs.
	self problemProductions: Set new.
	self otherProductions: Set new
! !

!ProductionPartition methodsFor:'state accessing'!

leftHandSide

	^leftHandSide
!

leftHandSide: argument 

	leftHandSide := argument
!

otherProductions

	^otherProductions
!

otherProductions: argument 

	otherProductions := argument
!

problemProductions

	^problemProductions
!

problemProductions: argument 

	problemProductions := argument
! !

!ProductionPartition methodsFor:'testing'!

anyProblems

	^self problemProductions isEmpty not
! !

!ProductionPartition class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/ProductionPartition.st,v 1.5 2002/11/20 13:49:48 cg Exp $'
! !
