"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

ParseTreeNode subclass:#RegularExpressionNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

RegularExpressionNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am the root of a regular expression tree.'
!


!RegularExpressionNode methodsFor:'building parse trees'!

alternateWith: node

	^node alternateWithNonAltNode: self
!

alternateWithAltNode: node

	node children addLast: self.
	^node
!

alternateWithNonAltNode: node

	^self alternationNodeClass children: (OrderedCollection with: node with: self)
!

concatenateWith: node

	^node concatenateWithNonCatNode: self
!

concatenateWithCatNode: node

	node children addLast: self.
	^node
!

concatenateWithNonCatNode: node

	^self concatenationNodeClass children: (OrderedCollection with: node with: self)
! !

!RegularExpressionNode methodsFor:'converting'!

asDFSA

	| fsa |
	fsa := self asFSA.
	^fsa asDeterministicFSA
!

asFSA

	| startState finalState |
	startState := self fsaStateClass new.
	finalState := self fsaFinalStateClass new.
	self asFSAStartingAt: startState endingAt: finalState.
	^startState
!

asFSAStartingAt: startState endingAt: finalState 

	self subclassResponsibility
!

asFSAWithLiteral: literal startingAt: startState 

	| finalState |
	"First, build main fsa."
	finalState := self fsaFinalStateClass new.
	self asFSAStartingAt: startState endingAt: finalState.
	finalState addLiteralToken: literal.
	^startState
!

asFSAWithType: type andAction: action startingAt: startState 

	| finalState |
	"First, build main fsa."
	finalState := self fsaFinalStateClass new.
	self asFSAStartingAt: startState endingAt: finalState.
	finalState addTokenClass: (self tokenClassificationClass tokenType: type action: action).
	^startState
!

asPureRegExpr
	"Answer a new version of the receiver consisting of only characters, 
	concatenations, alternations, and (star) closures. Also, eliminate single 
	child alternations and concatenations."

	^self           "default"
!

minimize: recognizer 

	^recognizer asMinimalDFSA
! !

!RegularExpressionNode methodsFor:'private'!

alternationNodeClass

	^AlternationNode
!

characterNodeClass

	^CharacterNode
!

concatenationNodeClass

	^ConcatenationNode
!

createNewProductionWithLHS: lhsNode andRHS: anOrderedCollection 
	"Create a new RRPGProductionNode given its lhs and a collection of regular 
	expression nodes whose concatenation forms the rhs. Note: anOrderedCollection 
	may contain ConcatenationNodes which need to be 'expanded' to avoid nesting 
	within the ConcatenationNode created by this method. Since each transformation 
	is done individually (i.e. this method gets invoked each time a new production is 
	created), the CatenationNodes will never be nested more then one deep."

	| rhs rhsNode childNodes |
	rhs := anOrderedCollection isEmpty
				ifTrue: [self epsilonNodeClass new]
				ifFalse: [anOrderedCollection size = 1
						ifTrue: [anOrderedCollection first]
						ifFalse: 
							[childNodes := OrderedCollection new.
							anOrderedCollection do: [:node | node isConcatenationNode
									ifTrue: [childNodes addAllLast: node children]
									ifFalse: [childNodes addLast: node]].
							self concatenationNodeClass children: childNodes]].
	rhsNode := self rhsNodeClass symbols: rhs translationSymbol: nil.
	^self productionNodeClass leftHandSide: lhsNode rightHandSides: rhsNode
!

epsilon
	"Answer an object used to represent the empty string (epsilon)."

	^self epsilonNodeClass epsilon
!

epsilonNodeClass

	^EpsilonNode
!

fsaFinalStateClass

	^FSAFinalState
!

fsaStateClass

	^FSAState
!

nameExtensionString
	"Answer a string appropriate for creating new nonterminals by suffixing."

	^'0'
!

newNonterminal: lhsNode with: lhsNames 

	| lhs |
	lhs := lhsNode.
	lhsNames isEmpty
		ifTrue: [lhsNames add: lhs]
		ifFalse: 
			[[lhsNames includes: lhs]
				whileTrue: [lhs := lhs , '`'].
			lhsNames add: lhs].
	^NonterminalNode new setAttribute: lhs
!

newNonterminalFrom: lhsNode excluding: lhsNames 
	"Answer a new nonterminal based on lhsNode that is
	not already in lhsNames."

	| newName |
	newName := lhsNode symbol.
	[newName := newName , self nameExtensionString.
	lhsNames includes: newName] whileTrue.
	lhsNames add: newName.
	^self nonterminalNodeClass symbol: newName
!

nonterminalNodeClass

	^NonterminalNode
!

productionNodeClass

	^RRPGProductionNode
!

rhsNodeClass

	^RRPGRightHandSideNode
!

starClosureNodeClass

	^StarClosureNode
!

tokenClassificationClass

	^TokenClassification
! !

!RegularExpressionNode methodsFor:'transforming'!

needsTransforming

	^false
! !

!RegularExpressionNode methodsFor:'traversing'!

addNonemptyLeavesTo: aSet 

	^self           "default is do nothing"
!

collectNonemptyLeavesIn: aSet 

	self postorderDo: [:child | child addNonemptyLeavesTo: aSet]
! !

!RegularExpressionNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/RegularExpressionNode.st,v 1.5 2002/11/20 13:50:54 cg Exp $'
! !
