"{ Package: 'stx:goodies/tgen' }"

ReadStream subclass:#RetractableReadStream
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Streams-Misc'
!

RetractableReadStream comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

This class adds a ''backspace'' method and overrides several methods to correctly support this behavior.'
!


!RetractableReadStream methodsFor:'accessing'!

current
        "Answer the element at the current position or nil if at the beginning. This is useful for
        rereading the stream after backspacing."

        ^self atBeginning
                ifTrue: [nil]
                ifFalse: [collection at: position-ZeroPosition]
! !

!RetractableReadStream methodsFor:'positioning'!

backspace
	"Backup one position, if possible. It may be best to signal an error when attempting to backup
	past the beginning of the stream, but for now just do nothing."

	self atBeginning ifFalse: [self skip: -1]
! !

!RetractableReadStream methodsFor:'private'!

pastEndRead
	"The receiver has attempted to read past the end, answer an EOF indicator."
	"NOTE: currently, this class is used only by T-gen so it is acceptable to use the end-of-input character
	rather than nil to denote the end of the stream. However, in a more general context, it may
	be desirable to change this back to nil. If this is done then either the transitionFor:ifNone:
	method in class FSAState must be changed to check for nil as a transition symbol
	(Dictionaries do not allow nil keys), or scanners must be changed to translate a nil character
	to the end-of-input character. These changes affect what happens when a scanner runs out of
      input in the middle of a token."
^ Character endOfInput.
	^Signal noHandlerSignal handle: [:ex | ex parameter proceedWith: (Character endOfInput)]
		do: [self class endOfStreamSignal raiseRequestFrom: self]
! !

!RetractableReadStream methodsFor:'testing'!

atBeginning

        ^position = ZeroPosition
! !

!RetractableReadStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/RetractableReadStream.st,v 1.7 2006/04/06 10:54:39 stefan Exp $'
! !
