"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

GrammarParseTreeNode subclass:#RightHandSideNode
	instanceVariableNames:'symbols translationSymbol'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Grammar Nodes'
!

RightHandSideNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

This class represents the right-hand side of a grammar rule.

Instance Variables

	symbols         <OrderedCollection>     - rhs of production rule.
	translationSymbol       <Symbol>        - symbol for transduction.'
!


!RightHandSideNode class methodsFor:'instance creation'!

symbols: arg1 translationSymbol: arg2

	| newNode |
	newNode := self new.
	newNode symbols: arg1.
	newNode translationSymbol: arg2.
	^newNode
! !

!RightHandSideNode methodsFor:'building parse trees'!

addChildrenInitial: anOrderedCollection 

	self symbols: anOrderedCollection removeFirst.
	self translationSymbol: anOrderedCollection removeFirst
! !

!RightHandSideNode methodsFor:'converting'!

asProductionWithLeftHandSide: lhs 
	"Answer the GrammarProduction I represent."

	| rhs |
	rhs := self convertSymbols.
	^self translationSymbol isNil
		ifTrue: [GrammarProduction leftHandSide: lhs rightHandSide: rhs]
		ifFalse: [TransductionGrammarProduction
				leftHandSide: lhs
				rightHandSide: rhs
				translationSymbol: self translationSymbol symbol]
!

convertSymbols 

	^self symbols collect: [:sym | sym asGrammarSymbol]
! !

!RightHandSideNode methodsFor:'printing'!

printOn: aStream 

	self  printSymbolsOn: aStream.
	self translationSymbol notNil
		ifTrue: 
			[aStream nextPut: ${.
			self translationSymbol printOn: aStream.
			aStream nextPutAll: '}  ;']
!

printSymbolsOn: aStream 

	self symbols do: 
		[:sym | 
		sym printOn: aStream.
		aStream space]
! !

!RightHandSideNode methodsFor:'state accessing'!

symbols

	^symbols
!

symbols: argument 

	symbols := argument
!

translationSymbol

	^translationSymbol
!

translationSymbol: argument 

	translationSymbol := argument
! !

!RightHandSideNode methodsFor:'traversing'!

childrenDo: aBlock 
	"Evaluate aBlock for each of my children."

	self symbols do: aBlock.
	aBlock value: self translationSymbol
!

updateChildrenUsing: aBlock 
	"Replace my children according to the value of aBlock."

	self symbols: (self symbols collect: aBlock).
	self translationSymbol: (aBlock value: self translationSymbol)
! !

!RightHandSideNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/RightHandSideNode.st,v 1.5 2002/11/20 13:50:28 cg Exp $'
! !
