"
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate 'Tgen::ReadMe tgenCopyright.'
"

"{ Package: 'stx:goodies/tgen' }"

Dictionary subclass:#SetDictionary
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Unordered'
!

!SetDictionary class methodsFor:'documentation'!

copyright
"
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate 'Tgen::ReadMe tgenCopyright.'
"
!

documentation
"
    This class is part of the tgen package.

    This class represents a Dictionary of Sets.
"
!

examples
"
    |d|

    d := SetDictionary new.

    d at:'hans' add: 60.
    d at:'hans' add: 100.
    d at:'hans' add: 100.
    d at:'anton' add: 1234.
    d at:'berta' add: 1234.
    d at:'berta' add: 3333.
    d at:'caesar' addAll: #(1 2 3 4).
    d do:[:el |
        Transcript showCR:el.
    ].
    d keysAndValuesDo:[:k :el |
        Transcript show:'k: '; show:k; show:' val: '; showCR:el.
    ].
    Transcript show:'at berta: '; showCR:(d at:'berta').

"
! !

!SetDictionary methodsFor:'accessing'!

at: key ifAbsent: absentBlock ifNotUnique: notUniqueBlock 

	| elementSet |
	elementSet := self at: key ifAbsent: [^absentBlock value].
	^elementSet size > 1
		ifTrue: [notUniqueBlock value]
		ifFalse: [elementSet first]
!

at: key ifNotUnique: aBlock 

	| elementSet |
	elementSet := self at: key.
	^elementSet size > 1
		ifTrue: [aBlock value]
		ifFalse: [elementSet first]
!

elements

	| elements |
	elements := Set new.
	self do: [:set | elements addAll: set].
	^elements
! !

!SetDictionary methodsFor:'adding'!

at: key add: anObject 

	(self at: key ifAbsent: [self at: key put: Set new])
		add: anObject
!

at: key addAll: aSet 

	(self at: key ifAbsent: [self at: key put: Set new])
		addAll: aSet
! !

!SetDictionary methodsFor:'converting'!

asDictionary

	| newDict |
	self isDeterministic
		ifTrue: 
			[newDict := Dictionary new: self size.
			self associationsDo: [:assoc | newDict at: assoc key put: assoc value first].
			^newDict]
		ifFalse: [self error: 'SetDictionary cannot be converted to a Dictionary']
! !

!SetDictionary methodsFor:'dictionary enumerating'!

elementsDo: aBlock 
	"Evaluate aBlock with each element of each of the receiver's set elements as the 
	argument."

	self elements do: [:element | aBlock value: element]
! !

!SetDictionary methodsFor:'removing'!

at: key remove: anObject 

	^(self at: key)
		remove: anObject
!

at: key remove: anObject ifAbsent: aBlock 

	^(self at: key)
		remove: anObject ifAbsent: aBlock
! !

!SetDictionary methodsFor:'testing'!

isDeterministic

	self associationsDo: [:assoc | assoc value size > 1 ifTrue: [^false]].
	^true
! !

!SetDictionary class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/SetDictionary.st,v 1.5 2002/03/15 09:27:18 martin Exp $'
! !
