"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

UnaryRegExprNode subclass:#StarClosureNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Regular Expression Nodes'
!

StarClosureNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I represent the regular expression ''(expr)*'' which denotes zero or more repetitions of ''expr''.'
!


!StarClosureNode methodsFor:'converting'!

asFSAStartingAt: startState endingAt: finalState 

	| middleState |
	middleState := self fsaStateClass new.
	startState goto: middleState on: self epsilon.
	middleState goto: finalState on: self epsilon.
	self onlyChild asFSAStartingAt: middleState endingAt: middleState
!

asPureRegExpr
	"Answer a new version of the receiver consisting of only characters, 
	concatenations, alternations, and (star) closures."

	^self starClosureNodeClass onlyChild: self onlyChild asPureRegExpr
! !

!StarClosureNode methodsFor:'printing'!

printOn: aStream 

	aStream nextPut: $(.
	self onlyChild printOn: aStream.
	aStream nextPutAll: ')*'
! !

!StarClosureNode methodsFor:'transforming'!

transformUsing: lhsNames withLHS: lhsNode alpha: alphaNodes gamma: gammaNodes 
	"Transform productions of the form 'A -> alpha beta* gamma' 
	into 'A -> alpha B', 'B -> beta B', and 'B -> gamma'."

	| newProds newLHS |
	newProds := OrderedCollection new.
	newLHS := self newNonterminalFrom: lhsNode excluding: lhsNames.
	newProds add: (self createNewProductionWithLHS: lhsNode andRHS: (alphaNodes addLast: newLHS; yourself)).
	newProds add: (self createNewProductionWithLHS: newLHS andRHS: (OrderedCollection
					with: self onlyChild
					with: newLHS)).
	newProds add: (self createNewProductionWithLHS: newLHS andRHS: gammaNodes).
	^newProds
! !

!StarClosureNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/StarClosureNode.st,v 1.5 2002/11/20 13:48:47 cg Exp $'
! !
