"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

TokenSpecParseNode subclass:#TokenSpecRuleNode
	instanceVariableNames:'tokenClass regExpr directive'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Token Specification Nodes'
!

TokenSpecRuleNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================


Instance Variables:
	tokenClass      <TokenClassNode> - the name of the token class.
	regExpr         <RegularExpressionNode> (actually, any one of its concrete subclasses) - the regular expression tree representing the specification of the token class.
	directive               <DirectiveNode + UndefinedObject> - the optional scanner directive.'
!


!TokenSpecRuleNode class methodsFor:'instance creation'!

tokenClass: arg1 regExpr: arg2 directive: arg3 

	| newMe |
	newMe := self new.
	newMe tokenClass: arg1.
	newMe regExpr: arg2.
	newMe directive: arg3.
	^newMe
! !

!TokenSpecRuleNode methodsFor:'building parse trees'!

addChildrenInitial: anOrderedCollection 

	anOrderedCollection size = 3
		ifTrue: 
			[self tokenClass: anOrderedCollection removeFirst.
			self regExpr: anOrderedCollection removeFirst.
			self directive: anOrderedCollection removeFirst]
		ifFalse: [self error: 'wrong number of children']
! !

!TokenSpecRuleNode methodsFor:'converting'!

asSpecRule
	"Answer the specification rule I represent."

	^self specRuleClass
		tokenClass: self tokenClass symbol
		regExpr: self regExpr asPureRegExpr
		directive: (self directive notNil
				ifTrue: [self directive asMessageSelector]
				ifFalse: [nil])
! !

!TokenSpecRuleNode methodsFor:'printing'!

printOn: aStream 

	self tokenClass printOn: aStream.
	aStream tab; nextPutAll: ': '.
	self regExpr printOn: aStream.
	aStream tab; tab.
	self directive notNil 
		ifTrue:
			[self directive printOn: aStream.
			aStream space].
	aStream nextPut: $;
! !

!TokenSpecRuleNode methodsFor:'private'!

specRuleClass

	^TokenSpecificationRule
! !

!TokenSpecRuleNode methodsFor:'state accessing'!

directive

	^directive
!

directive: argument 

	directive := argument
!

regExpr

	^regExpr
!

regExpr: argument 

	regExpr := argument
!

tokenClass

	^tokenClass
!

tokenClass: argument 

	tokenClass := argument
! !

!TokenSpecRuleNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TokenSpecRuleNode.st,v 1.5 2002/11/20 13:49:07 cg Exp $'
! !
