"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#TranslatorGenerator
	instanceVariableNames:'grammar transformedGrammar isLL1 isSLR1 isLALR1 isLR1 parser
		statusTextStream tokenSpecification grammarSpecification
		inputText grammarMode parserMode parserResult tokenSpecController
		scannerClass treeBuilderClass oldScannerSpec oldScannerLits
		oldScanner specsFileName conflictStrategy'
	classVariableNames:'StatusTextBufferMenu TextMenu TokenSpecMetaChars'
	poolDictionaries:''
	category:'Tgen-Interface'
!

TranslatorGenerator class instanceVariableNames:'grammarSpecParser tokenSpecParser'

"
 No other class instance variables are inherited by this class.
"
!

TranslatorGenerator comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Object tgenCopyright."
=================================================

I''m an application for generating scanners and parsers for arbitrary context-free languages.  My view contains four panes and several status buttons, as described below:

View Panes:
	top-left                - token class specifications are entered and ''accepted'' here.
	bottom-left     - grammar specifications are entered and ''accepted'' here.  Status buttons control what kind of parser is generated, either LL or LR varieties.
	top-right               - status transcript, status messages and other information is sent here.
	bottom-right    - test strings in the specified language are entered and ''accepted'' here.  Status buttons control the resulting parser actions.  To view the result of a test input parse choose the middle button menu item ''result'' in the transcript pane.

I attempt to generate a parser for a given a context-free grammar.  Basically, I try LL(1) first and LR variations second.  (See the parser generation protocol for more details.)

Instance Variables:
	grammar                                 <Grammar> - original context-free grammar for which I''m trying to build a parser.
	transformedGrammar      <Grammar | UndefinedObject> - an LL(1) transformed version of my grammar, if needed, requested, and possible.
	isLL1,
	isSLR1,
	isLALR1,
	isLR1                                           <Boolean> - grammar status flags.
	parser                                          <LL1Parser | LR1Parser | UndefinedObject> - a parser for my grammar, if one could be built.
	statusTextBuffer                        <Text> - status transcript pane text buffer.
	tokenSpecification              <Text>
	grammarSpecification    <Text>
	inputText                                       <Text>
	grammarMode                             <Symbol> - represents state of grammar pane status button.
	parserMode                              <Symbol> - represents state of test input pane status button.
	parserResult                            <DerivationTreeNode | ParseTreeNode> - result of test input parse.
	tokenSpecController             <Controller> - ''accepting'' a grammar specification actually processes both grammar and token class specifications.  Direct access to the token spec Controller is needed for reporting errors in the token class specification.
	scannerClass                            <FSABasedScanner | ?> - class used to build scanners.  Some languages may require more sophisticated scanners with multiple-token lookahead (see FSABasedScannerWithOneTokenLookahead and FSABasedScannerWithTwoTokenLookahead).
	treeBuilderClass                        <AbstractSyntaxTreeBuilder | ?> - class used to build AST builders for parsers.  Users may define specilizations of this class.
	oldScannerSpec                  <String> - previous scanner spec.
	oldScannerLits                          <Set of String> - literals from previous grammar spec.
	oldScanner                                      <FSABasedScanner | ?> - previously generated scanner.
	specsFileName                           <String> - last used base file name for saving or loading specs.

Class Variables:
	StatusTextBufferMenu            <ActionMenu> - menu cache
	TextMenu                                        <ActionMenu> - menu cache
	TokenSpecMetaChars              <Set of: Character> - These characters are special to the token specification parser and must be escaped.'
!


!TranslatorGenerator class methodsFor:'instance creation'!

new

	^super new init
! !

!TranslatorGenerator class methodsFor:'class initialization'!

flushMenus 
	"Cause all menus to be newly created (so changes appear)." 
	"TranslatorGenerator flushMenus."

	StatusTextBufferMenu := nil.
	TextMenu := nil
!

initialize
	"TranslatorGenerator initialize"

	GrammerSpecScanner initialize.
	GrammerSpecParser initialize.
	TokenSpecScanner initialize.
	TokenSpecParser initialize.

	self grammarSpecParser: GrammarSpecParser new.
	self tokenSpecParser: TokenSpecParser new.
	self initializeTokenSpecMetaChars.
	self flushMenus
!

initializeTokenSpecMetaChars
	"These characters are special to the TokenSpecParser and must be escaped."

	| chars |
	chars := Set new.
	chars addAll: #($: $; $| $* $+ $? $( $) $[ $] $~ $- $< $> ${ $} $\ ).
	chars
		 add: Character space;
		 add: Character tab;
		 add: Character cr.
	TokenSpecMetaChars := chars
! !

!TranslatorGenerator class methodsFor:'examples'!

example1
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	(none) 
        
	grammar spec: 
	S       : 'a' S 'b'     {Pair} ;
	S       :                       {Core} ;
        
	status: LL(1) and SLR(1) 
        
	test input: 
	aaabbb 
	"
!

example10
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	<var>   : [a-z] ; 
	<num>   : [0-9]+ ; 
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
        
	grammar spec: 
	Prog                    : LetExpr ; 
	Prog                    : Expr                                          {LetNode} ; 
	LetExpr         : 'let' Defs 'in' Expr          {LetNode} ; 
	Defs                    : Def Defs0                             {liftRightChild} ; 
	Defs0                   : ',' Defs ; 
	Defs0                   :                                                       {OrderedChildren} ; 
	Def                     : VarName '=' Number    {DefinitionNode} ; 
	Expr                    : Term Expr0                            {liftRightChild}; 
	Expr0                   : '+' Expr                                      {PlusNode} ; 
	Expr0                   : '-' Expr                                      {MinusNode} ; 
	Expr0                   :                                                       {nil} ; 
	Term                    : Factor Term0                  {liftRightChild}; 
	Term0                   : '*' Term                                      {TimesNode} ; 
	Term0                   : '/' Term                                      {DivideNode} ; 
	Term0           :                                                       {nil}; 
	Factor          : VarName ; 
	Factor          : Number ; 
	Factor          : '(' Expr ')' ; 
	VarName         : <var>                                         {VarNode} ; 
	Number          : <num>                                         {NumberNode} ; 
        
	status: LL(1) and SLR(1) 
        
	test input: 
	let 
	x = 3, 
	y = 7, 
	z = 2 
	in 
	x * (y + z / 4) "
!

example11
	"This grammar demonstrates the ability to fix nondeterministic parsers by hand. 
	Evaluate the following expression and then test the grammar. Try to build an LL(1) parser 
	and when asked to perform transformations, answer no. From the middle button menu in 
	the transcript pane inspect the parser. Inspect the parseTable instance variable. The 'E' row 
	and 'else' column has a double entry. Using the inspectors, remove the production 'E : ;' from 
	the set and evaluate 'self spaceOptimize' in the parseTable inspector. The resulting parser 
	will successfully parse the input and can be installed. For more information see the T-gen 
	User's Guide."
	"TranslatorGeneratorView open"
	"token spec: 
	<space> : [\s\t\r]+     {ignoreDelimiter} ; 
        
	grammar spec: 
	G : S ';' ; 
	S : 'if' S E | 'other' ; 
	E : 'else' S | ; 
        
	status: not LL(1) and not LR(1) 
        
	test input: 
	if if other else other; 
	"
!

example12
	"Smalltalk-80 grammar"
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec:    (YOU MUST USE FSABasedScannerWithOneTokenLookahead)      
	<identifier>    :       [A-Za-z][A-Za-z0-9]* ; 
	<number>        : [0-9]+(.[0-9]+)?((e|d)(\-)?[0-9]+)? | 
	([0-9]+r)(\-)?[0-9A-Z]+(.[0-9A-Z]+)?((e|d)(\-)?[0-9]+)? ; 
	<characterConstant>     :       $[!!-\~\s\t\r\n] ; 
	<keyword>       :       ([A-Za-z][A-Za-z0-9]*\:)+ ; 
	<string>                 :      '(~['] | '' | [\s\t\r\n]+)*' ; 
	<binarySelector> : [,\+/\\\*\~\<\>=@%\|&\?!!] ([,\+/\\\*\~\<\>=@%\|&\?!!])? ; 
	<comment>        :      \""( ~[\""] | \""\""| [\s\t\r\n] )* \""         {ignoreComment} ; (REMEMBER TO COMPRESS CONSEQUITIVE DOUBLE-QUOTES) 
	<space>          :       [\s\t\r\n]+                            {ignoreDelimiter}       ; 


	grammar spec:           (YOU MUST USE SmalltalkAbstractSyntaxTreeBuilder)
	method                                          : messagePattern temporaries primitive statements       {selector:primitive:temporaries:statements:} 
								; 
	messagePattern                  : unarySelector                                                         {OrderedChildren}        
								| binarySelector argument                                       {OrderedChildren}        
								| keywordVariableGroup                                           
								; 
	keywordVariableGroup            : keywordVariableGroup keyword argument {liftLeftChild} 
								| keyword argument                                              {OrderedChildren} 
								; 

	temporaries                             : '|' argumentList '|' 
								|                                                                                       {nil} 
								; 
	argumentList                            : argument argumentList                                         {liftRightChild} 
								|                                                                                       {OrderedChildren} 
								; 
	primitive                                       : '<' 'primitive:' number '>'                                   {primitive:} 
								|                                                                                       {nil} 
								; 
	block                                           : '[' blockArguments temporaries statements ']' {arguments:temporaries:statements:} 
								; 

	blockArguments                  : blockArgumentList '|'                          
								|                                                                                       {nil} 
								; 

	blockArgumentList                       : blockArgumentList ':' argument                        {liftLeftChild} 
								| ':'argument                                                           {OrderedChildren} 
								; 

	statements                                      : expression '.' statements                                     {liftRightChild} 
								| '^'expression                                                         {return:} 
								| '^' expression '.'                                                    {return:} 
								| expression                                                            {OrderedChildren} 
								|                                                                                       {OrderedChildren} 
								;        

	expression                                      : variableName '_' expression                           {variable:leftArrowAndValue:} 
								| variableName ':=' expression                          {variable:value:} 
								| mainExpression                                                 
								; 

	mainExpression                  : primary 
								| messageExpression      
								| cascadeMessageExpression 
								; 
	cascadeMessageExpression : messageExpression cascadePart                        {receiver:messages:} 
								; 
	cascadePart                             : cascadePart ';' cascade                                               {liftLeftChild} 
								| ';' cascade                                                                   {OrderedChildren} 
								; 
	cascade                                         : unarySelector                                                         {selector:} 
								| binarySelector unaryObjectDescription {selector:argument:} 
								| keywordBinObjGroup                                            {selectorAndArgument:} 
								; 

	messageExpression                       : unaryExpression                
								| binaryExpression       
								| keywordExpression      
								; 

	keywordBinObjGroup              : keywordBinObjGroup keyword binaryObjectDescription    {liftLeftChild} 
								| keyword binaryObjectDescription                       {OrderedChildren} 
								; 

	unaryExpression                 : unaryObjectDescription unarySelector  {receiver:selector:} 
								; 

	binaryExpression                        : binaryObjectDescription binarySelector unaryObjectDescription 
	{receiver:selector:argument:} 
								; 

	keywordExpression               : binaryObjectDescription keywordBinObjGroup    {receiver:selectorAndArgument:}          
								; 

	unaryObjectDescription  : primary 
								| unaryExpression                
								; 
	binaryObjectDescription : unaryObjectDescription 
								| binaryExpression                                                       
								; 
	primary                                 : variableName           
								| literal                        
								| block 
								| '(' expression ')'     
								; 

	keyword                                 : <keyword>                                                             {asSymbol:} 
								; 

	binarySelector                          : <binarySelector>                                                      {asSymbol:} 
								| '-'                                                                                   {minusSymbol} 
								| '<'                                                                                   {lessThanSymbol} 
								| '>'                                                                                   {greaterThanSymbol}
								| '|'                                                                                   {vbarSymbol} 
								| 'primitive:'                                                          {primitiveSymbol}
								; 

	unarySelector                           : <identifier>                                                          {asSymbol:} 
								; 

	variableName                            : <identifier>                                                          {VariableNode} 
								; 

	argument                                        : <identifier>                                                          {ParameterNode}  
								; 

	number                                  : <number>                                                                      {asNumber:} 
								| '-' constant                                                          {negate:} 
								; 
	constant                                        : <number>                                                                      {asNumber:} 
								; 
	literal                                         : number                                                                        {LiteralNode} 
								| symbolConstant                                                        {LiteralNode} 
								| <characterConstant>                                           {characterLiteral:} 
								| <string>                                                                      {stringLiteral:} 
								| arrayConstant                                                         {LiteralNode}                            
								; 
	arrayConstant                           : '#' array                              
								; 
	array                                           : '(' arrayList ')'                                                             {asArray:} 
								; 

	arrayList                                       : number arrayList                                                      {liftRightChild} 
								| symbol arrayList                                                      {liftRightChild} 
								| symbolConstant arrayList                                      {liftRightChild} 
								| arrayConstant arrayList                                       {liftRightChild} 
								| characterConstant arrayList                           {liftRightChild} 
								| array arrayList                                                       {liftRightChild} 
								|                                                                                       {OrderedChildren} 
								; 

	characterConstant                       : <characterConstant>                                           {asCharacter:} 
								; 
	symbolConstant                  : '#' symbol                     
								; 
	symbol                                          : <identifier>                                                          {asSymbol:} 
								| <keyword>                                                             {asSymbol:} 
								| <string>                                                                      {asString:} 
								| <binarySelector>                                                      {asSymbol:}      
								| '<'                                                                                   {lessThanSymbol} 
								| '>'                                                                                   {greaterThanSymbol}
								| '|'                                                                                   {vbarSymbol} 
								| 'primitive:'                                                          {primitiveSymbol}
								; 

	status: LALR(1) 
	test input: select a method from browser 

	comments: the new faster scanner does not support nested token classes. Accordingly, the 
	specifications and productions for <keyword> and <keywordList> were combined in the 
	above grammar. This accepts all correct Smalltalk-80 programs, but also allows keywords in 
	message sends to have multiple colons, e.g. 'receiver msg1:msg2: argument'. If overlapping 
	token classes are ever implemented, the grammar can be changed back to: 
		<keyword>       :       [A-Za-z][A-Za-z0-9]*\: ; 
		<keywordList>   :       ([A-Za-z][A-Za-z0-9]*\:)+ ; 
		symbol : <keywordList> ... 
	The T-gen scanners are also not yet context-sensitive.  Thus, white space is required
	before the assignment token (:=).  For example, 'var := 0' scans as <id> ':=' <num>
	whereas 'var:=0' or 'var:= 0' scan as <keyword> <binop> <num>.

	"
!

example2
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	<id> : [a-z]+ ; 
	<number> : [0-9]+ ; 
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
        
        
        
	grammar spec: 
	Z               : 'program' Decls Stmts         {Program} ; 
	Decls   : 'var' IdList ':' 'integer'            {Decls} ; 
	IdList  : Name IdList                           {liftRightChild} ; 
	IdList  : Name                                          {IdList} ; 
	Stmts   : 'begin' SL 'end'                      {Stmts} ; 
	SL              : S SL                                          {liftRightChild} ; 
	SL              : Stmts SL                                      {liftRightChild} ; 
	SL              : S                                                     {StmtList} ; 
	SL              : Stmts                                         {StmtList} ; 
	S               : Name ':=' E ';'                               {Assign} ; 
	E               : E '+' T                                               {Plus} ; 
	E               : T ; 
	T               : P '*' T                                               {Times} ; 
	T               : P; 
	P               : '(' E ')' ; 
	P               : Name ; 
	P               : <number>                                      {Number} ; 
	Name    : <id>                                          {Id} ; 
	status: SLR(1) and LL(1), with transformations 
        
	test input: 
	program 
	var a b c : integer 
	begin 
		a := 3; 
		b := a * 4 * 5; 
		begin 
			c := a + b * 19;
			b := b * c + 1;
			a := (a + 1) * a; 
		end
		c := b * (c + 5);
	end 
	"
!

example3
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
        
	grammar spec:    
	E : T Ec                        {liftRightChild} ; 
	Ec : '+' E              {Plus} ; 
	Ec :                            {nil} ; 
	T : P Tc                        {liftRightChild} ; 
	Tc : '*' T              {Times} ; 
	Tc :                            {nil} ; 
	P : 'a'                         {A} ; 
	P : 'b'                         {B} ; 
	P : 'c'                         {C} ; 
        
	status: LL(1) and SLR(1) 
        
	test input: 
	a + b * c + a 
	"
!

example4
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
        
        
	grammar spec: 
	E : P ; 
	P : 'a' ;
	P : '(' A ';' A ')' ;
	P : '(' V ',' V ')' ; 
	V : 'a' ; 
	A : 'a' ; 
        
	status: LALR(1) 
        
	test input: 
	(a ; a) 
	"
!

example5
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	(none)
        
        
	grammar spec: 
	S : 'a' B 'b' ;
	S : 'a' D 'a' ;
	S : 'b' B 'a' ;
	S : 'b' D 'b' ; 
	B : A ; 
	A : 'a' ; 
	D : 'a' ; 
        
	status: LR(1) 
        
	test input: 
	aaa 
	"
!

example6
	"Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
        
        
	grammar spec: 
	S : E ;
	E : E '+' E | E '*' E | 'a' ; 
        
	status: not LL(1) and not LR(1) 
        
	test input: 
	a + a * a 
	"
!

example7
	"This grammar demonstrates the need of simple scanner backup. 
	Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	(none) 
        
	grammar spec: 
	S : E S | ; 
	E : 'a' | 'abc' | 'bd' ; 
        
	status: LL(1) and SLR(1) 
        
	test input: 
	abd 
	"
!

example8
	"This grammar demonstrates the need of two-token scanner lookahead. 
	Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	(none) 
        
	grammar spec: 
	S : E S | ; 
	E : 'a' | 'ab' | 'abc' | 'bd' ; 
        
	status: LL(1) and SLR(1) 
        
	test input: 
	abd 
	"
!

example9
	"This demonstrates how to build flat non-empty lists. 
	Evaluate the following expression and then test the grammar."
	"TranslatorGeneratorView open"
	"token spec: 
	(none) 
        
	grammar spec: 
	E : E P         {liftLeftChild} 
	| P                     {OrderedChildren} ; 
	P : 'a'         {A} 
	| 'b'                   {B} 
	| 'c'                   {C} ; 
        
	status: LL(1) with transformations (translations are still not handled) and SLR(1) 
        
	test input: 
	abc 
	"
! !

!TranslatorGenerator class methodsFor:'state accessing'!

grammarSpecParser

	^grammarSpecParser
!

grammarSpecParser: argument 

	grammarSpecParser := argument
!

tokenSpecParser

	^tokenSpecParser
!

tokenSpecParser: argument 

	tokenSpecParser := argument
! !

!TranslatorGenerator class methodsFor:'testing'!

basicLLParserGenerationTest1
	"TranslatorGenerator basicLLParserGenerationTest1"

	| tgen failBlock testName |
	testName := 'T-gen LL Parser Generation Test 1'.
	tgen := self new setGrammarModeToLL.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			''
			'aabb'
			)
		compareDTTo: (Array
			with: '#S\ . ''<epsilon>''\' withCRs
			with: '#S\ . ''a''\ . #S\ .  . ''a''\ .  . #S\ .  .  . ''<epsilon>''\ .  . ''b''\ . ''b''\' withCRs
			)
		compareSASTTo: (Array
			with: '#Core\' withCRs
			with: '#Pair\ . #Pair\ .  . #Core\' withCRs
			)
		tokenSpec: ''
		grammarSpec: '
	S       : ''a'' S ''b''         {Pair} ;
	S       :                       {Core} ;
'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLLParserGenerationTest2
	"TranslatorGenerator basicLLParserGenerationTest2"

	| tgen failBlock testName |
	testName := 'T-gen LL Parser Generation Test 2'.
	tgen := self new setGrammarModeToLL.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: (Array with:
			'       program\        var a b c : integer\    begin\          a := 3;\                b := a * 4 * 5;\                begin\                  c := a + b * 19;\                       b := b * c + 1;\                        a := (a + 1) * a;\              end\            c := b * (c + 5);\      end\' withCRs
			)
		compareDTToAny: (Array with: (Array
			with: '#Z\ . ''program''\ . #Decls\ .  . ''var''\ .  . #IdList\ .  .  . #Name\ .  .  .  . ''<id>''\ .  .  . #IdList1\ .  .  .  . #IdList\ .  .  .  .  . #Name\ .  .  .  .  .  . ''<id>''\ .  .  .  .  . #IdList1\ .  .  .  .  .  . #IdList\ .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  . #IdList1\ .  .  .  .  .  .  .  . ''<epsilon>''\ .  . '':''\ .  . ''integer''\ . #Stmts\ .  . ''begin''\ .  . #SL\ .  .  . #S\ .  .  .  . #Name\ .  .  .  .  . ''<id>''\ .  .  .  . '':=''\ .  .  .  . #E\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  . #T1\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  . #E0\ .  .  .  .  .  . ''<epsilon>''\ .  .  .  . '';''\ .  .  . #SL2\ .  .  .  . #SL\ .  .  .  .  . #S\ .  .  .  .  .  . #Name\ .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  . '':=''\ .  .  .  .  .  . #E\ .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  . '';''\ .  .  .  .  . #SL2\ .  .  .  .  .  . #SL\ .  .  .  .  .  .  . #Stmts\ .  .  .  .  .  .  .  . ''begin''\ .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  . #SL2\ .  .  .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  .  .  . #SL2\ .  .  .  .  .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  .  .  .  .  . #SL2\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  . ''end''\ .  .  .  .  .  .  . #SL1\ .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  . #SL2\ .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  . ''end''\' withCRs
			with: '#Z\ . ''program''\ . #Decls\ .  . ''var''\ .  . #IdList\ .  .  . #Name\ .  .  .  . ''<id>''\ .  .  . #IdList1\ .  .  .  . #IdList\ .  .  .  .  . #Name\ .  .  .  .  .  . ''<id>''\ .  .  .  .  . #IdList1\ .  .  .  .  .  . #IdList\ .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  . #IdList1\ .  .  .  .  .  .  .  . ''<epsilon>''\ .  . '':''\ .  . ''integer''\ . #Stmts\ .  . ''begin''\ .  . #SL\ .  .  . #S\ .  .  .  . #Name\ .  .  .  .  . ''<id>''\ .  .  .  . '':=''\ .  .  .  . #E\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  . #T1\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  . #E0\ .  .  .  .  .  . ''<epsilon>''\ .  .  .  . '';''\ .  .  . #SL1\ .  .  .  . #SL\ .  .  .  .  . #S\ .  .  .  .  .  . #Name\ .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  . '':=''\ .  .  .  .  .  . #E\ .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  . '';''\ .  .  .  .  . #SL1\ .  .  .  .  .  . #SL\ .  .  .  .  .  .  . #Stmts\ .  .  .  .  .  .  .  . ''begin''\ .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  . #SL1\ .  .  .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  .  .  . #SL1\ .  .  .  .  .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  .  .  .  .  . #SL1\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  . ''end''\ .  .  .  .  .  .  . #SL2\ .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T1\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  .  . #E0\ .  .  .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  .  . #SL1\ .  .  .  .  .  .  .  .  .  . ''<epsilon>''\ .  . ''end''\' withCRs
			))
		tokenSpec: '
	<id> : [a-z]+ ; 
	<number> : [0-9]+ ; 
	<space> : [\s\t\r\n]+                     {ignoreDelimiter} ;
			'
		grammarSpec: '
	Z               : ''program'' Decls Stmts               {Program} ; 
	Decls   : ''var'' IdList '':'' ''integer''              {Decls} ; 
	IdList  : Name IdList                           {liftRightChild} ; 
	IdList  : Name                                          {IdList} ; 
	Stmts   : ''begin'' SL ''end''                  {Stmts} ; 
	SL              : S SL                                          {liftRightChild} ; 
	SL              : Stmts SL                                      {liftRightChild} ; 
	SL              : S                                                     {StmtList} ; 
	SL              : Stmts                                         {StmtList} ; 
	S               : Name '':='' E '';''                           {Assign} ; 
	E               : E ''+'' T                                             {Plus} ; 
	E               : T ; 
	T               : P ''*'' T                                             {Times} ; 
	T               : P; 
	P               : ''('' E '')'' ; 
	P               : Name ; 
	P               : <number>                                      {Number} ; 
	Name    : <id>                                          {Id} ; 
			'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLLParserGenerationTest3
	"TranslatorGenerator basicLLParserGenerationTest3"

	| tgen failBlock testName |
	testName := 'T-gen LL Parser Generation Test 3'.
	tgen := self new setGrammarModeToLL.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'a'
			'a + b * c + a'
			)
		compareDTTo: (Array
			with: '#E\ . #T\ .  . #P\ .  .  . ''a''\ .  . #Tc\ .  .  . ''<epsilon>''\ . #Ec\ .  . ''<epsilon>''\' withCRs
			with: '#E\ . #T\ .  . #P\ .  .  . ''a''\ .  . #Tc\ .  .  . ''<epsilon>''\ . #Ec\ .  . ''+''\ .  . #E\ .  .  . #T\ .  .  .  . #P\ .  .  .  .  . ''b''\ .  .  .  . #Tc\ .  .  .  .  . ''*''\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''c''\ .  .  .  .  .  . #Tc\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  . #Ec\ .  .  .  . ''+''\ .  .  .  . #E\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''a''\ .  .  .  .  .  . #Tc\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  . #Ec\ .  .  .  .  .  . ''<epsilon>''\' withCRs
			)
		compareSASTTo: (Array
			with: '#A\' withCRs
			with: '#Plus\ . #A\ . #Plus\ .  . #Times\ .  .  . #B\ .  .  . #C\ .  . #A\' withCRs
			)
		tokenSpec: '
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
'
		grammarSpec: '
	E : T Ec                        {liftRightChild} ; 
	Ec : ''+'' E            {Plus} ; 
	Ec :                            {nil} ; 
	T : P Tc                        {liftRightChild} ; 
	Tc : ''*'' T            {Times} ; 
	Tc :                            {nil} ; 
	P : ''a''                       {A} ; 
	P : ''b''                       {B} ; 
	P : ''c''                       {C} ; 
'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest1
	"TranslatorGenerator basicLRParserGenerationTest1"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 1'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			''
			'aabb'
			)
		compareDTTo: (Array
			with: '#S\ . ''<epsilon>''\' withCRs
			with: '#S\ . ''a''\ . #S\ .  . ''a''\ .  . #S\ .  .  . ''<epsilon>''\ .  . ''b''\ . ''b''\' withCRs
			)
		compareSASTTo: (Array
			with: '#Core\' withCRs
			with: '#Pair\ . #Pair\ .  . #Core\' withCRs
			)
		tokenSpec: ''
		grammarSpec: '
	S       : ''a'' S ''b''         {Pair} ;
	S       :                       {Core} ;
		'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest2
	"TranslatorGenerator basicLRParserGenerationTest2"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 2'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: (Array
			with: ' program\        var a b c : integer\    begin\          a := 3;\                b := a * 4 * 5;\                begin\                  c := a + b * 19;\                       b := b * c + 1;\                        a := (a + 1) * a;\              end\            c := b * (c + 5);\      end\' withCRs
			)
		compareDTTo: (Array
			with: '#Z\ . ''program''\ . #Decls\ .  . ''var''\ .  . #IdList\ .  .  . #Name\ .  .  .  . ''<id>''\ .  .  . #IdList\ .  .  .  . #Name\ .  .  .  .  . ''<id>''\ .  .  .  . #IdList\ .  .  .  .  . #Name\ .  .  .  .  .  . ''<id>''\ .  . '':''\ .  . ''integer''\ . #Stmts\ .  . ''begin''\ .  . #SL\ .  .  . #S\ .  .  .  . #Name\ .  .  .  .  . ''<id>''\ .  .  .  . '':=''\ .  .  .  . #E\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''<number>''\ .  .  .  . '';''\ .  .  . #SL\ .  .  .  . #S\ .  .  .  .  . #Name\ .  .  .  .  .  . ''<id>''\ .  .  .  .  . '':=''\ .  .  .  .  . #E\ .  .  .  .  .  . #T\ .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  . '';''\ .  .  .  . #SL\ .  .  .  .  . #Stmts\ .  .  .  .  .  . ''begin''\ .  .  .  .  .  . #SL\ .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  .  .  . #SL\ .  .  .  .  .  .  .  .  . #S\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  . '';''\ .  .  .  .  .  . ''end''\ .  .  .  .  . #SL\ .  .  .  .  .  . #S\ .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  . '':=''\ .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  . ''*''\ .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  . ''(''\ .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  . #E\ .  .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  . #Name\ .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<id>''\ .  .  .  .  .  .  .  .  .  .  .  . ''+''\ .  .  .  .  .  .  .  .  .  .  .  . #T\ .  .  .  .  .  .  .  .  .  .  .  .  . #P\ .  .  .  .  .  .  .  .  .  .  .  .  .  . ''<number>''\ .  .  .  .  .  .  .  .  .  .  . '')''\ .  .  .  .  .  .  . '';''\ .  . ''end''\' withCRs
			)
		compareSASTTo: (Array
			with: '#Program\ . #Decls\ .  . #IdList\ .  .  . ''a''\ .  .  . ''b''\ .  .  . ''c''\ . #Stmts\ .  . #StmtList\ .  .  . #Assign\ .  .  .  . ''a''\ .  .  .  . ''3''\ .  .  . #Assign\ .  .  .  . ''b''\ .  .  .  . #Times\ .  .  .  .  . ''a''\ .  .  .  .  . #Times\ .  .  .  .  .  . ''4''\ .  .  .  .  .  . ''5''\ .  .  . #Stmts\ .  .  .  . #StmtList\ .  .  .  .  . #Assign\ .  .  .  .  .  . ''c''\ .  .  .  .  .  . #Plus\ .  .  .  .  .  .  . ''a''\ .  .  .  .  .  .  . #Times\ .  .  .  .  .  .  .  . ''b''\ .  .  .  .  .  .  .  . ''19''\ .  .  .  .  . #Assign\ .  .  .  .  .  . ''b''\ .  .  .  .  .  . #Plus\ .  .  .  .  .  .  . #Times\ .  .  .  .  .  .  .  . ''b''\ .  .  .  .  .  .  .  . ''c''\ .  .  .  .  .  .  . ''1''\ .  .  .  .  . #Assign\ .  .  .  .  .  . ''a''\ .  .  .  .  .  . #Times\ .  .  .  .  .  .  . #Plus\ .  .  .  .  .  .  .  . ''a''\ .  .  .  .  .  .  .  . ''1''\ .  .  .  .  .  .  . ''a''\ .  .  . #Assign\ .  .  .  . ''c''\ .  .  .  . #Times\ .  .  .  .  . ''b''\ .  .  .  .  . #Plus\ .  .  .  .  .  . ''c''\ .  .  .  .  .  . ''5''\' withCRs
			)
		tokenSpec: '
	<id> : [a-z]+ ; 
	<number> : [0-9]+ ; 
	<space> : [\s\t\r\n]+                     {ignoreDelimiter} ;
			'
		grammarSpec: '
	Z               : ''program'' Decls Stmts               {Program} ; 
	Decls   : ''var'' IdList '':'' ''integer''              {Decls} ; 
	IdList  : Name IdList                           {liftRightChild} ; 
	IdList  : Name                                          {IdList} ; 
	Stmts   : ''begin'' SL ''end''                  {Stmts} ; 
	SL              : S SL                                          {liftRightChild} ; 
	SL              : Stmts SL                                      {liftRightChild} ; 
	SL              : S                                                     {StmtList} ; 
	SL              : Stmts                                         {StmtList} ; 
	S               : Name '':='' E '';''                           {Assign} ; 
	E               : E ''+'' T                                             {Plus} ; 
	E               : T ; 
	T               : P ''*'' T                                             {Times} ; 
	T               : P; 
	P               : ''('' E '')'' ; 
	P               : Name ; 
	P               : <number>                                      {Number} ; 
	Name    : <id>                                          {Id} ; 
			'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest3
	"TranslatorGenerator basicLRParserGenerationTest3"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 3'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'a'
			'a + b * c + a'
			)
		compareDTTo: (Array
			with: '#E\ . #T\ .  . #P\ .  .  . ''a''\ .  . #Tc\ .  .  . ''<epsilon>''\ . #Ec\ .  . ''<epsilon>''\' withCRs
			with: '#E\ . #T\ .  . #P\ .  .  . ''a''\ .  . #Tc\ .  .  . ''<epsilon>''\ . #Ec\ .  . ''+''\ .  . #E\ .  .  . #T\ .  .  .  . #P\ .  .  .  .  . ''b''\ .  .  .  . #Tc\ .  .  .  .  . ''*''\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''c''\ .  .  .  .  .  . #Tc\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  . #Ec\ .  .  .  . ''+''\ .  .  .  . #E\ .  .  .  .  . #T\ .  .  .  .  .  . #P\ .  .  .  .  .  .  . ''a''\ .  .  .  .  .  . #Tc\ .  .  .  .  .  .  . ''<epsilon>''\ .  .  .  .  . #Ec\ .  .  .  .  .  . ''<epsilon>''\' withCRs
			)
		compareSASTTo: (Array
			with: '#A\' withCRs
			with: '#Plus\ . #A\ . #Plus\ .  . #Times\ .  .  . #B\ .  .  . #C\ .  . #A\' withCRs
			)
		tokenSpec: '
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
'
		grammarSpec: '
	E : T Ec                        {liftRightChild} ; 
	Ec : ''+'' E            {Plus} ; 
	Ec :                            {nil} ; 
	T : P Tc                        {liftRightChild} ; 
	Tc : ''*'' T            {Times} ; 
	Tc :                            {nil} ; 
	P : ''a''                       {A} ; 
	P : ''b''                       {B} ; 
	P : ''c''                       {C} ; 
'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest4
	"TranslatorGenerator basicLRParserGenerationTest4"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 4'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'(a ; a)'
			'(a , a)'
			)
		compareDTTo: (Array
			with: '#E\ . #P\ .  . ''(''\ .  . #A\ .  .  . ''a''\ .  . '';''\ .  . #A\ .  .  . ''a''\ .  . '')''\' withCRs
			with: '#E\ . #P\ .  . ''(''\ .  . #V\ .  .  . ''a''\ .  . '',''\ .  . #V\ .  .  . ''a''\ .  . '')''\' withCRs
			)
		tokenSpec: '
	<space> : [\s\t\r]+                     {ignoreDelimiter} ; 
'
		grammarSpec: '
	E : P ; 
	P : ''a'' ;
	P : ''('' A '';'' A '')'' ;
	P : ''('' V '','' V '')'' ; 
	V : ''a'' ; 
	A : ''a'' ; 
'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest5
	"TranslatorGenerator basicLRParserGenerationTest5"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 5'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'aab'
			'aaa'
			'baa'
			'bab'
			)
		compareDTTo: (Array
			with: '#S\ . ''a''\ . #B\ .  . #A\ .  .  . ''a''\ . ''b''\' withCRs
			with: '#S\ . ''a''\ . #D\ .  . ''a''\ . ''a''\' withCRs
			with: '#S\ . ''b''\ . #B\ .  . #A\ .  .  . ''a''\ . ''a''\' withCRs
			with: '#S\ . ''b''\ . #D\ .  . ''a''\ . ''b''\' withCRs
			)
		tokenSpec: ''
		grammarSpec: '
	S : ''a'' B ''b'' ;
	S : ''a'' D ''a'' ;
	S : ''b'' B ''a'' ;
	S : ''b'' D ''b'' ; 
	B : A ; 
	A : ''a'' ; 
	D : ''a'' ; 
'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicLRParserGenerationTest6
	"TranslatorGenerator basicLRParserGenerationTest6"

	| tgen failBlock testName |
	testName := 'T-gen LR Parser Generation Test 6'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #()
		compareDTTo: #()
		tokenSpec: ''
		grammarSpec: '  S : E ; E : E ''+'' E | E ''*'' E | ''a'' ;'
		ifFail: failBlock.
	(tgen isLL1 | tgen isSLR1 | tgen isLALR1 | tgen isLR1)
		ifTrue: failBlock
		ifFalse: [Transcript show: '*'].
	Transcript cr; show: 'STOP: ' , testName
!

basicScannerTest1
	"TranslatorGenerator basicScannerTest1"

	| tgen failBlock testName |
	testName := 'T-gen Scanner Test 1'.
	tgen := self new setGrammarModeToLR.
	tgen scannerClass: FSABasedScannerWithOneTokenLookahead.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'abc'
			'abd'
			)
		compareDTTo: (Array
			with: '#S\ . #E\ .  . ''abc''\ . #S\ .  . ''<epsilon>''\' withCRs
			with: '#S\ . #E\ .  . ''a''\ . #S\ .  . #E\ .  .  . ''bd''\ .  . #S\ .  .  . ''<epsilon>''\' withCRs
			)
		tokenSpec: ''
		grammarSpec: 'S : E S | ; E : ''a'' | ''abc'' | ''bd'' ;'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

basicScannerTest2
	"TranslatorGenerator basicScannerTest2"

	| tgen failBlock testName |
	testName := 'T-gen Scanner Test 2'.
	tgen := self new setGrammarModeToLR.
	tgen scannerClass: FSABasedScannerWithTwoTokenLookahead.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen
		parseInputs: #(
			'abc'
			'abd'
			'abbd'
			'a'
			)
		compareDTTo: (Array
			with: '#S\ . #E\ .  . ''abc''\ . #S\ .  . ''<epsilon>''\' withCRs
			with: '#S\ . #E\ .  . ''a''\ . #S\ .  . #E\ .  .  . ''bd''\ .  . #S\ .  .  . ''<epsilon>''\' withCRs
			with: '#S\ . #E\ .  . ''ab''\ . #S\ .  . #E\ .  .  . ''bd''\ .  . #S\ .  .  . ''<epsilon>''\' withCRs
			with: '#S\ . #E\ .  . ''a''\ . #S\ .  . ''<epsilon>''\' withCRs
			)
		tokenSpec: ''
		grammarSpec: 'S : E S | ; E : ''a'' | ''ab'' | ''abc'' | ''bd'' ;'
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

rrpgToCfgTransformationsTest1
	"TranslatorGenerator rrpgToCfgTransformationsTest1"

	| tgen failBlock testName |
	testName := 'T-gen RRPG-to-CFG Transformation Test 1'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen tokenSpecification: ''.
	tgen
		grammarSpec: 'S : ''b''+ ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' S0 ;\S0 : ''b'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''+ ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' S0 ;\S0 : ''b'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b''+ ''c'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' S0 ;\S0 : ''b'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''+ ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' S0 ;\S0 : ''b'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b''* ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' S0 ;\S0 : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''* ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' S0 ;\S0 : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b''* ''c'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' S0 ;\S0 : ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''* ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' S0 ;\S0 : ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b''? ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' ;\S0    : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''? ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' ;\S0    : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b''? ''c'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' ''c'' ;\S0      : ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b''? ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' ''c'' ;\S0      : ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b'' ^ ''d'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' ;\S0    : ''b'' ''d'' S0 ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b'' ^ ''d'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''a'' ''b'' ;\S0      : ''a'' ''b'' ''d'' S0 ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b'' ^ ''d'' ''c'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''b'' ;\S0    : ''b'' ''d'' ''c'' S0 ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b'' ^ ''d'' ''c'' ;\' withCRs
		targetGrammar: 'S       : S0 ;\S0       : ''a'' ''b'' ;\S0      : ''a'' ''b'' ''d'' ''c'' S0 ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b'' | ''d'' ;\' withCRs
		targetGrammar: 'S       : ''b'' ;\S     : ''d'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b'' | ''d'' ;\' withCRs
		targetGrammar: 'S       : ''a'' ''b'' ;\S       : ''d'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''b'' | ''d'' ''c'' ;\' withCRs
		targetGrammar: 'S       : ''b'' ;\S     : ''d'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b'' | ''d'' ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' ''b'' ;\S       : ''d'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : (''b'' | ''d'') ;\' withCRs
		targetGrammar: 'S       : ''b'' ;\S     : ''d'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' (''b'' | ''d'') ;\' withCRs
		targetGrammar: 'S       : ''a'' ''b'' ;\S       : ''a'' ''d'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : (''b'' | ''d'') ''c'' ;\' withCRs
		targetGrammar: 'S       : ''b'' ''c'' ;\S       : ''d'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' (''b'' | ''d'') ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' ''b'' ''c'' ;\S : ''a'' ''d'' ''c'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : A B+ ;\A : ''a'' ;\B : ''b'' ;\' withCRs
		targetGrammar: 'S       : A S0 ;\S0     : B S0 ;\S0     : B ;\A : ''a'' ;\B     : ''b'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : A B* ;\A : ''a'' ;\B : ''b'' ;\' withCRs
		targetGrammar: 'S       : A S0 ;\S0     : B S0 ;\S0     : ;\A   : ''a'' ;\B     : ''b'' ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : A B? ;\A : ''a'' ;\B : ''b'' ;\' withCRs
		targetGrammar: 'S       : A S0 ;\S0     : B ;\S0        : ;\A   : ''a'' ;\B     : ''b'' ;\' withCRs
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

rrpgToCfgTransformationsTest2
	"TranslatorGenerator rrpgToCfgTransformationsTest2"

	| tgen failBlock testName |
	testName := 'T-gen RRPG-to-CFG Transformation Test 2'.
	tgen := self new setGrammarModeToLR.
	failBlock := [self error: testName, ' failed.'].
	Transcript cr; show: 'START: ' , testName; cr.
	tgen tokenSpecification: ''.
	tgen
		grammarSpec: 'S : ''a'' (''b'' | ''d'')? ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : ''b'' ;\S0    : ''d'' ;\S0    : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' ''b'' | ''d''? ;\' withCRs
		targetGrammar: 'S       : ''a'' ''b'' ;\S       : S0 ;\S0       : ''d'' ;\S0    : ;\' withCRs
		ifFail: failBlock.
	tgen
		grammarSpec: 'S : ''a'' (B ^ C)? ;\B : ''b'' ;\C : ''c'' ;\' withCRs
		targetGrammar: 'S       : ''a'' S0 ;\S0 : S00 ;\S00     : B ;\S00       : B C S00 ;\S0  : ;\B   : ''b'' ;\C     : ''c'' ;\' withCRs
		ifFail: failBlock.
	Transcript cr; show: 'STOP: ' , testName
!

runAllTests
	"TranslatorGenerator runAllTests"

	Transcript cr; show: 'T-gen TEST SUITE (answer ''yes'' to all prompters)'.
	self basicLLParserGenerationTest1.
	self basicLLParserGenerationTest2.
	self basicLLParserGenerationTest3.
	self basicLRParserGenerationTest1.
	self basicLRParserGenerationTest2.
	self basicLRParserGenerationTest3.
	self basicLRParserGenerationTest4.
	self basicLRParserGenerationTest5.
	self basicLRParserGenerationTest6.
	self basicScannerTest1.
	self basicScannerTest2.
	self rrpgToCfgTransformationsTest1.
	self rrpgToCfgTransformationsTest2.
	Transcript cr; show: 'ALL TESTS PASSED'; cr.
! !

!TranslatorGenerator class methodsFor:'version information'!

versionName

	^'T-gen, Version 2.1'
! !

!TranslatorGenerator methodsFor:'LL parser generation'!

ll1Parser
	"Attempt to construct an LL(1) parser for my grammar. 
	Answer true if I can and false otherwise."

	| table prod |
	table := self llParserTableClass new.
	self grammar selectSets
		associationsDo: 
			[:assoc | 
			prod := assoc key.
			assoc value do: [:term | table
					atNonterminal: prod leftHandSide
					andTerminal: term
					addProduction: prod]].
	table isDeterministic
		ifTrue: 
			[self isLL1: true.
			self parser: (self ll1ParserClass parseTable: table spaceOptimize startSymbol: self grammar startSymbol).
			^true]
		ifFalse: 
			[self parser: (self ll1ParserClass parseTable: table startSymbol: self grammar startSymbol).
			^false]
!

ll1ParserWithTransformations
	"Assuming my original grammar is not LL(1), attempt to derive an equivalent LL(1) 
	grammar and parser for my original grammar through various grammar 
	manipulations. Answer true if I can and false otherwise."

	| oldGrammar |
	oldGrammar := self grammar.
	self grammar: oldGrammar copy.
	self grammar makeLL1Transformations.
	self ll1Parser
		ifTrue: 
			[self isLL1: true.
			self transformedGrammar: self grammar.
			self grammar: oldGrammar.
			^true]
		ifFalse: 
			[self grammar: oldGrammar.
			^false]
! !

!TranslatorGenerator methodsFor:'LR parser generation'!

computeLR0ClosureOf: itemSet 
	"For all items in itemSet of the form 'B -> <alpha> . A <beta>' 
	recursively add new items of the form 'A -> . <gamma>'."

	^self computeLR0ClosureOf: itemSet withProdMap: self computeProductionMap
!

computeLR0ClosureOf: itemSet withProdMap: prodMap
	"For all items in itemSet of the form 'B -> <alpha> . A <beta>' 
	recursively add new items of the form 'A -> . <gamma>'."

	| nonterms sym newItemSet |
	nonterms := Set new.
	itemSet do: [:item | (sym := item nextSymbol) isNonterminal ifTrue: [nonterms add: sym]].
	newItemSet := ItemSet new.
	newItemSet addAll: itemSet.
	nonterms do: [:nont | newItemSet addAll: ((prodMap at: nont)
				collect: [:prod | prod asInitialLR0Item])].
	^itemSet size = newItemSet size
		ifTrue: [newItemSet]
		ifFalse: [self computeLR0ClosureOf: newItemSet withProdMap: prodMap]
!

computeLR1ClosureOf: itemSet 
	"For all items in itemSet of the form 'B -> <alpha> . A <beta> : lookahead' 
	recursively add new items of the form 'A -> . <gamma> : First(<beta> lookahead)'."

	^self computeLR1ClosureOf: itemSet withProdMap: self computeProductionMap
!

computeLR1ClosureOf: itemSet withProdMap: prodMap
	"For all items in itemSet of the form 'B -> <alpha> . A <beta> : lookahead' 
	recursively add new items of the form 'A -> . <gamma> : First(<beta> lookahead)'."

	| nontermItems newItemSet |
	nontermItems := Set new.
	itemSet do: [:item | item nextSymbol isNonterminal ifTrue: [nontermItems add: item]].
	newItemSet := ItemSet new.
	newItemSet addAll: itemSet.
	nontermItems do: [:item | (self lr1LookaheadSetFor: item)
			do: [:la | (prodMap at: item nextSymbol)
					do: [:prod | newItemSet add: (prod asInitialLR1ItemWithLookahead: la)]]].
	^itemSet size = newItemSet size
		ifTrue: [newItemSet]
		ifFalse: [self computeLR1ClosureOf: newItemSet withProdMap: prodMap]
!

extantStateFor: itemSet in: stateItemMap 
	"If stateItemMap includes itemSet as one of its values answer the corresponding 
	state, otherwise answer nil. (This is the same as Dictionary>>keyAtValue:ifAbsent: 
	[^nil] except that it uses = rather than ==.)"

	stateItemMap associationsDo: [:assoc | assoc value = itemSet ifTrue: [^assoc key]].
	^nil
!

findTransitionItemSet: itemSet 
	"Arbitrarily choose a transition symbol (immediately after a dot) 
	and answer the set of items that have transitions on that symbol. 
	All 'atEnd' items have already been removed."

	| transitSym |
	transitSym := itemSet first postDotSymbols first.
	^itemSet select: [:item | self is: item postDotSymbols first theSameAs: transitSym]
!

is: symOne theSameAs: symTwo 
	"Compare the symbols for equality. This method is required because T-gen uses
	Strings to represent terminals and Symbols for non-terminals, and a String and
	Symbol with the same characters are considered equal. Thus, 'prod' = #prod is true
	in Smalltalk but not in T-gen."

	^symOne isTerminal = symTwo isTerminal
		ifTrue: [symOne = symTwo]
		ifFalse: [false]
!

lr1Parser
	"Attempt to build an LR(1) parser for my grammar. 
	Answer true if successful and false otherwise. 
        
	Based on Algorithm 6.2 from 'Principles of Compiler Design', 
	by Aho and Ullman, 1977."

	| stateItemMap startState unprocessedStates currState closure transitItems transitSym newItemSet nextState finalState conflictStates |
	stateItemMap := Dictionary new.
	conflictStates := Set new.
	startState := self lrParserStateClass new.
	unprocessedStates := Set with: startState.
	stateItemMap at: startState put: self initialLR1ItemSet.
	[unprocessedStates isEmpty]
		whileFalse: 
			[currState := unprocessedStates removeFirst.
			closure := self computeLR1ClosureOf: (stateItemMap at: currState).
			"Process all reduce items."
			(closure select: [:item | item atEnd])
				do: 
					[:item | 
					item isFinalStateItem
						ifTrue: [finalState := currState]
						ifFalse: 
							[currState reduceMap at: item lookahead add: item asGrammarProduction.
							currState reduceMap at: item lookahead ifNotUnique: 
								["reduce/reduce conflict"
								conflictStates add: currState]].
					closure remove: item].
			[closure isEmpty]
				whileFalse: 
					[transitItems := self findTransitionItemSet: closure.
					transitSym := transitItems first postDotSymbols first.
					newItemSet := ItemSet new.
					transitItems do: [:item | newItemSet add: item copy shift].
					(nextState := self extantStateFor: newItemSet in: stateItemMap) isNil
						ifTrue: 
							[nextState := self lrParserStateClass new.
							unprocessedStates add: nextState.
							stateItemMap at: nextState put: newItemSet].
					currState goto: nextState on: transitSym.
					(currState reduceMap includesKey: transitSym)
						ifTrue: 
							["shift/reduce conflict"
							conflictStates add: currState].
					closure removeAll: transitItems]].
	conflictStates isEmpty
		ifTrue: 
			[self isLR1: true.
			self parser: (self lrParserClass parseTable: startState finalState: finalState) spaceOptimize.
			^true]
		ifFalse: 
			[self parser: (self lrParserClass parseTable: startState finalState: finalState).
			^false]
!

slr1Parser
	"Attempt to build an SLR(1) parser for my grammar. 
	Answer true if successful and false otherwise. 
	
	Based on Algorithm 6.1 from 'Principles of Compiler Design', 
	by Aho and Ullman, 1977."

	| stateItemMap startState unprocessedStates currState closure transitItems transitSym newItemSet nextState finalState conflictStates slr1Parser |
	stateItemMap := Dictionary new.
	conflictStates := Set new.
	startState := self lrParserStateClass new.
	unprocessedStates := Set new.
	stateItemMap at: startState put: self initialLR0ItemSet.
	unprocessedStates add: startState.
	[unprocessedStates isEmpty]
		whileFalse: 
			[currState := unprocessedStates removeFirst.
			closure := self computeLR0ClosureOf: (stateItemMap at: currState).
			"Process all reduce items."
			(closure select: [:item | item atEnd])
				do: 
					[:item | 
					item isFinalStateItem
						ifTrue: [finalState := currState]
						ifFalse: 
							["SLR(1) lookahead symbols"
							(self grammar followSetOf: item leftHandSide)
								do: 
									[:term | 
									currState reduceMap at: term add: item asGrammarProduction.
									currState reduceMap at: term ifNotUnique: 
										["reduce/reduce conflict"
										conflictStates addAll: (conflictStrategy resolveReduceReduceFor: term atState: currState)]]].
					closure remove: item].
			[closure isEmpty]
				whileFalse: 
					[transitItems := self findTransitionItemSet: closure.
					transitSym := transitItems first postDotSymbols first.
					newItemSet := ItemSet new.
					transitItems do: [:item | newItemSet add: item copy shift].
					(nextState := self extantStateFor: newItemSet in: stateItemMap) isNil
						ifTrue: 
							[nextState := self lrParserStateClass new.
							unprocessedStates add: nextState.
							stateItemMap at: nextState put: newItemSet].
					currState goto: nextState on: transitSym.
					(currState reduceMap includesKey: transitSym)
						ifTrue: 
							["shift/reduce conflict"
							conflictStates addAll: (conflictStrategy resolveShiftReduceFor: transitSym atState: currState)].
					closure removeAll: transitItems]].
	slr1Parser := self lrParserClass parseTable: startState finalState: finalState.
	conflictStates isEmpty
		ifTrue: 
			[self isSLR1: true.
			self parser: slr1Parser spaceOptimize.
			^true]
		ifFalse: 
			["try LALR(1) analysis"
			(slr1Parser lalr1AnalyzeConflicts: conflictStates originalGrammar: self grammar)
				ifTrue: 
					[self isLALR1: true.
					self parser: slr1Parser spaceOptimize.
					^true]
				ifFalse: 
					[self parser: slr1Parser.
					^false]]
! !

!TranslatorGenerator methodsFor:'accessing'!

grammarSpecParser

	^self class grammarSpecParser
!

tokenSpecMetaChars

	^TokenSpecMetaChars
!

tokenSpecParser

	^self class tokenSpecParser
! !

!TranslatorGenerator methodsFor:'grammar mode'!

isGrammarModeDefault

	^self grammarMode = self defaultModeMarker
!

isGrammarModeLL

	^self grammarMode = self llModeMarker
!

isGrammarModeLR

	^self grammarMode = self lrModeMarker
!

setGrammarMode: modeMarker

	self grammarMode: modeMarker.
	self changed: #value            "broadcast change so radio buttons may change"
!

setGrammarModeToDefault

	self grammarMode: self defaultModeMarker
!

setGrammarModeToLL

	self grammarMode: self llModeMarker
!

setGrammarModeToLR

	self grammarMode: self lrModeMarker
! !

!TranslatorGenerator methodsFor:'grammar text'!

acceptGrammarSpec: aText from: aController 

	self grammarSpecification: aText.
	self grammarSpecParser requestor: aController.
	self tokenSpecParser requestor: self tokenSpecController.
	(self tokenSpecification isNil or: [self tokenSpecification isEmpty])
		ifTrue: [self cr; showCR: 'No token classes have been specified (did you forget to ''accept'' them?).'].
	self buildParserIfFail: [^false].
	^true
!

text

	^'hello' asText
!

textMenu
	"Answer an pop-up menu of operations on the source code that is to be 
	displayed when the operate menu button is pressed."
	"TranslatorGenerator flushMenus"

	^TextMenu isNil
		ifTrue: [TextMenu := PopUpMenu
						labelList: #(('again' 'undo') ('copy' 'cut' 'paste') ('accept' 'cancel'))
						values: #(#again #undo #copySelection #cut #paste #accept #cancel )]
		ifFalse: [TextMenu]
! !

!TranslatorGenerator methodsFor:'initialization'!

init
        "set status defaults"

        self resetParserAndFlags.
        self tokenSpecification: Text new.
        self setGrammarModeToDefault.
        self setParserModeToDefault.
        self scannerClass: self defaultScannerClass.
        self treeBuilderClass: self defaultTreeBuilderClass.
        self oldScannerSpec: nil.
        self oldScannerLits: nil.
        self clear.
        conflictStrategy := StandardConflictStrategy new.
!

resetParserAndFlags
	"Reset my state for parser generation."

	self isLL1: false.
	self isSLR1: false.
	self isLALR1: false.
	self isLR1: false.
	self parser: nil.
	self grammar: nil.
	self transformedGrammar: nil
! !

!TranslatorGenerator methodsFor:'input text'!

acceptInputText: aText from: aController 

	self inputText: aText.
	self parser isNil
		ifTrue: 
			[self cr; showCR: 'No parser has been generated.'.
			^true].
	isLL1 | isSLR1 | isLALR1 | isLR1 ifFalse: [(Dialog confirm: 'The current parser was nondeterministic when built.
Are you sure you want to continue?')
			ifFalse: [^true]].
	self parser requestor: aController.
	self isParserModeDefault
		ifTrue: 
			[self parserResult: (self parser parseForDerivationTree: aText asString ifFail: [^false]).
			self postInspectResultMessage.
			^true].
	self isParserModeTrace
		ifTrue: 
			[self parserResult: (self parser
					parseAndTrace: aText asString
					on: self
					ifFail: [^false]).
			self postSuccessMessage.
			^true].
	self isParserModeShamAST
		ifTrue: 
			[self parserResult: (self parser parseForShamAST: aText asString ifFail: [^false]).
			self postInspectResultMessage.
			^true].
	self isParserModeAST
		ifTrue: 
			[self parserResult: (self parser parseForAST: aText asString ifFail: [^false]).
			self postInspectResultMessage.
			^true].
	^false
!

postInspectResultMessage

	self cr; showCR: 'Choose ''result'' from the middle button menu in this pane to inspect the results of the successful parse.'
!

postSuccessMessage

	self cr; showCR: 'The parse of the test input was successful.'
! !

!TranslatorGenerator methodsFor:'parser generation'!

generateLLParser
	"Try to build an LL parser for this language."

	self ll1Parser ifTrue: [^true].
	(Dialog confirm: 'Grammar is not LL(1) but a transformation may be.
Do you wish to try transforming the grammar?')
		ifTrue: [self ll1ParserWithTransformations ifTrue: [^true]].
	^false
!

generateLRParser
	"Try to build an LR parser for this language."

	self slr1Parser ifTrue: [^true].
	(Dialog confirm: 'Grammar is not LALR(1).
Do you wish to try LR(1) analysis
(this may take a while)?')
		ifTrue: [self lr1Parser ifTrue: [^true]].
	^false
!

generateParser
	"Try to build a parser for this language."

	^self generateLLParser
		ifTrue: [true]
		ifFalse: [self generateLRParser]
!

grammarClassification
	"Answer a string describing my classification."

	self isLL1 ifTrue: [self transformedGrammar isNil
			ifTrue: [^'Grammar is LL(1).']
			ifFalse: [^'Grammar is LL(1), but required transformations.']].
	self isSLR1 ifTrue: [^'Grammar is SLR(1).'].
	self isLALR1 ifTrue: [^'Grammar is LALR(1).'].
	self isLR1
		ifTrue: [^'Grammar is LR(1).']
		ifFalse: [^'Parser generation failed; nondeterministic parser is available for inspection.']
! !

!TranslatorGenerator methodsFor:'parser mode'!

isParserModeAST

	^self parserMode = self astModeMarker
!

isParserModeDefault

	^self parserMode = self defaultModeMarker
!

isParserModeShamAST

	^self parserMode = self shamAstModeMarker
!

isParserModeTrace

	^self parserMode = self traceModeMarker
!

setParserMode: modeMarker

	self parserMode: modeMarker.
	self changed: #value            "broadcast change so radio buttons may change"
!

setParserModeToAST

	self parserMode: self astModeMarker
!

setParserModeToDefault

	self parserMode: self defaultModeMarker
!

setParserModeToShamAST

	self parserMode: self shamAstModeMarker
!

setParserModeToTrace

	self parserMode: self traceModeMarker
! !

!TranslatorGenerator methodsFor:'private'!

astModeMarker

	^#AST
!

computeProductionMap
	"Delegate to my grammar."

	^self grammar computeProductionMap
!

defaultModeMarker

	^#default
!

defaultScannerClass
	"This is the default scanner class, a more specialized scanner class may be designated using a 
	middle button menu item in the transcript pane."

	^FSABasedScanner
!

defaultTreeBuilderClass
	"This is the default tree builder class, a more specialized tree builder class may be designated 
	using a middle button menu item in the transcript pane."

	^AbstractSyntaxTreeBuilder
!

fsaStateClass

	^FSAState
!

grammarClass

	^Grammar
!

initialLR0ItemSet

	^ItemSet with: (self lr0ItemClass initialItemForGrammar: self grammar)
!

initialLR1ItemSet

	^ItemSet with: (self lr1ItemClass initialItemForGrammar: self grammar)
!

ll1ParserClass

	^LL1Parser
!

llModeMarker

	^#LL
!

llParserTableClass

	^LLParserTable
!

lr0ItemClass

	^LR0Item
!

lr1ItemClass

	^LR1Item
!

lr1LookaheadSetFor: item 
	"Delegate to my grammar."

	^self grammar lr1LookaheadSetFor: item
!

lrModeMarker

	^#LR
!

lrParserClass

	^LR1Parser
!

lrParserFinalStateClass

	^FSAFinalState
!

lrParserStateClass

	^LRParserState
!

shamAstModeMarker

	^#shamAST
!

traceModeMarker

	^#Trace
! !

!TranslatorGenerator methodsFor:'public access'!

buildParserIfFail: aBlock 

	| tokenSpecNode grammarNode rules scanner newParser litDict |
	self resetParserAndFlags.
	tokenSpecNode := self tokenSpecParser parseForAST: self tokenSpecification ifFail: aBlock.
	grammarNode := self grammarSpecParser parseForAST: self grammarSpecification ifFail: aBlock.
	rules := tokenSpecNode asSpecRuleList.
	self cr; show: 'Checking grammar... '.
	self grammarClass notReducedSignal
		handle: [:ex | self showCR: ex errorString.  ^aBlock value]
		do: [self grammar: grammarNode asContextFreeGrammar].
	self showCR: 'done.'.
	litDict := self buildLiteralDictionaryIfFail: aBlock.
	self show: 'Building scanner... starting at: '.
	self showCR: Time now printString; tab.
	(self sameScannerSpec: self tokenSpecification usingLits: litDict)
		ifTrue: 
			[self showCR: 'Scanner has not changed - no build required.'.
			scanner := self oldScanner]
		ifFalse: 
			[self showCR: 'Scanner new or has changed - build required.'.
			Cursor execute showWhile: [scanner := self buildScannerFromRules: rules withLiteralDict: litDict]].
	self tab; showCR: '...done.'.
	self oldScannerSpec: self tokenSpecification asString.          "Must force a copy"
	self oldScannerLits: litDict keys.
	self oldScanner: scanner.
	self show: 'Building parser...starting at: '.
	self showCR: Time now printString.
	Cursor execute showWhile: [self isGrammarModeDefault
			ifTrue: [self generateParser]
			ifFalse: [self isGrammarModeLL
					ifTrue: [self generateLLParser]
					ifFalse: [self isGrammarModeLR
							ifTrue: [self generateLRParser]
							ifFalse: [self error: 'What mode am I in then?!!?!!']]]].
	self tab; show: '...done at:'.
	self showCR: Time now printString.
	self showCR: self grammarClassification.
	newParser := self parser.
	newParser scanner: scanner.
	newParser treeBuilder: self treeBuilderClass new.
	^newParser
!

loadSpecFrom: fileName

	| file |
	file := Filename named: fileName , '.tok'.
	file isReadable
		ifTrue: 
			[(Dialog confirm: 'This load will overwrite the existing specs.
Are you sure you want to do this?') ifFalse: [^self].
			tokenSpecification := file contentsOfEntireFile.
			self changed: #tokenSpecification]
		ifFalse: 
			[self showCR: 'Could not find file ''' , fileName , '.tok' , '''  - skipping load.'.
			^self].
	file := Filename named: fileName , '.grm'.
	file isReadable
		ifTrue: 
			[grammarSpecification := file contentsOfEntireFile.
			self changed: #grammarSpecification].
	file := Filename named: fileName , '.eg'.
	file isReadable
		ifTrue: 
			[inputText := file contentsOfEntireFile.
			self changed: #inputText]
!

saveSpecTo: fileName 
	tokenSpecification notNil ifTrue: [((Filename named: fileName , '.tok') writeStream) nextPutAll: tokenSpecification; close].
	grammarSpecification notNil ifTrue: [((Filename named: fileName , '.grm') writeStream) nextPutAll: grammarSpecification; close].
	^inputText notNil ifTrue: [((Filename named: fileName , '.eg') writeStream) nextPutAll: inputText; close]
! !

!TranslatorGenerator methodsFor:'scanner generation'!

buildFSAFrom: rules and: literalDict 
	"Answer a minimal deterministic FSA that will recognize rules and literals."

	| startState fsa stream ch state nextStateSet nextState |
	"First, build the basic DFSA for the token classes (rules)."
	startState := self fsaStateClass new.
	rules do: [:rule | rule regExpr
			asFSAWithType: rule tokenClass
			andAction: rule directive
			startingAt: startState].
	fsa := startState asNearMinimalDFSAWithUniqueTokenClasses.
	"Then, see if any of the literal tokens (keywords) can be placed in an existing final state."
	literalDict copy associationsDo: [:assoc |
		stream := ReadStream on: assoc key.
		ch := stream next.
		state := fsa.
		"Remember, we have not yet space optimized so we are dealing with SetDictionaries."
		[nextStateSet := state transitionFor: ch ifNone: [nil].
		nextState := nextStateSet isNil ifTrue: [nil] ifFalse: [nextStateSet first].
		nextState isNil or: [stream atEnd]]
			whileFalse:
				[state := nextState.
				ch := stream next].
		nextState isFSAFinalState
			ifTrue:
				[nextState addLiteralToken: assoc key.
				literalDict removeKey: assoc key]].
	"Attach remaining literal tokens to start state, re-DFSA-ify, and space optimize."
	literalDict associationsDo: [:assoc | assoc value asFSAWithLiteral: assoc key startingAt: fsa].
	^fsa asNearMinimalDFSAWithUniqueTokenClasses spaceOptimize
!

buildLiteralDictionaryIfFail: aBlock 
	"Answer a Dictionary from (de-escaped) literal tokens to their corresponding 
	regular expressions."

	| litDict |
	litDict := Dictionary new.
	self grammar literalTerminals do: [:lit | litDict at: lit put: (self convertToRegExpr: lit ifFail: aBlock)].
	^litDict
!

buildScannerFromRules: rules withLiteralDict: literalDict 
	"Answer a scanner that scans according to the rules, also recognizing 
	literalTokens."

	| fsa |
	fsa := self buildFSAFrom: rules and: literalDict.
	^self scannerClass new fsa: fsa
!

convertToRegExpr: aString ifFail: aBlock 
	"Answer a regular expression object by interpreting aString as a token class 
	specification. Token specification meta-characters must be escaped in aString."

	| rules tokenSpecNode |
	tokenSpecNode := self tokenSpecParser parseForAST: (self escapedTokenSpecDeclFor: aString)
				ifFail: aBlock.
	rules := tokenSpecNode asSpecRuleList.
	^rules first regExpr
!

escapedTokenSpecDeclFor: aString 
	"Answer a string of the form '<x> : aString ;' where all metacharacters in aString 
	have been escaped (proceeded by a backslash)."

	| aStream metaChars |
	aStream := WriteStream on: (String new: 20).
	aStream nextPutAll: '<x> : '.
	metaChars := self tokenSpecMetaChars.
	aString do: 
		[:ch | 
		(metaChars includes: ch)
			ifTrue: [aStream nextPut: $\].
		aStream nextPut: ch].
	aStream nextPut: $;.
	^aStream contents
!

oldBuildFSAFrom: rules and: literalDict 
	"Answer a minimal deterministic FSA that will recognize rules and literals."

	| startState |
	startState := self fsaStateClass new.
	rules do: [:rule | rule regExpr
			asFSAWithType: rule tokenClass
			andAction: rule directive
			startingAt: startState].
	literalDict associationsDo: [:assoc | assoc value asFSAWithLiteral: assoc key startingAt: startState].
	^startState asNearMinimalDFSAWithUniqueTokenClasses spaceOptimize
!

sameScannerSpec: newSpec usingLits: newLits 
	"Has the Specification for the scanner changed since it was last generated?"

	(self oldScannerSpec = nil or: [self oldScannerLits = nil])
		ifTrue: [^false].
	^self oldScannerLits size = (newLits keys union: self oldScannerLits) size and: [self oldScannerSpec = newSpec asString]
! !

!TranslatorGenerator methodsFor:'state accessing'!

grammar

	^grammar
!

grammar: argument 

	grammar := argument
!

grammarMode

	^grammarMode
!

grammarMode: argument 

	grammarMode := argument
!

grammarSpecification

	^grammarSpecification
!

grammarSpecification: argument 

	grammarSpecification := argument
!

inputText

	^inputText
!

inputText: argument 

	inputText := argument
!

isLALR1

	^isLALR1
!

isLALR1: argument 

	isLALR1 := argument
!

isLL1

	^isLL1
!

isLL1: argument 

	isLL1 := argument
!

isLR1

	^isLR1
!

isLR1: argument 

	isLR1 := argument
!

isSLR1

	^isSLR1
!

isSLR1: argument 

	isSLR1 := argument
!

oldScanner

	^oldScanner
!

oldScanner: argument 

	oldScanner := argument
!

oldScannerLits

	^oldScannerLits
!

oldScannerLits: argument 

	oldScannerLits := argument
!

oldScannerSpec

	^oldScannerSpec
!

oldScannerSpec: argument 

	oldScannerSpec := argument
!

parser

	^parser
!

parser: argument 

	parser := argument
!

parserMode

	^parserMode
!

parserMode: argument 

	parserMode := argument
!

parserResult

	^parserResult
!

parserResult: argument 

	parserResult := argument
!

scannerClass

	^scannerClass
!

scannerClass: argument 

	scannerClass := argument
!

specsFileName

	^specsFileName
!

specsFileName: argument 

	specsFileName := argument
!

statusTextBuffer

        ^ statusTextStream contents

    "Modified: 18.2.1997 / 15:59:17 / stefan"
!

statusTextBuffer: argument 

        statusTextStream := TextCollectorStream on:(StringCollection new)

    "Modified: 18.2.1997 / 16:00:13 / stefan"
!

tokenSpecController

	^tokenSpecController
!

tokenSpecController: argument 

	tokenSpecController := argument
!

tokenSpecification

	^tokenSpecification
!

tokenSpecification: argument 

	tokenSpecification := argument
!

transformedGrammar

	^transformedGrammar
!

transformedGrammar: argument 

	transformedGrammar := argument
!

treeBuilderClass

	^treeBuilderClass
!

treeBuilderClass: argument 

	treeBuilderClass := argument
! !

!TranslatorGenerator methodsFor:'status text'!

clear
        "Reset the transcriptView for the next accept."

        self statusTextBuffer:StringCollection new.
        self changed: #clear

    "Modified: 18.2.1997 / 16:00:45 / stefan"
!

cr
	"Perform a carriage return."

	self show: (String with: Character cr)
!

defineScannerClass
        "Prompt the user for the name of a new scanner class. An instance of this class will 
        be installed in each new parser built."

        | scannerName newScannerClass |
        scannerName := DialogView 
                           request: 'Type the name of the preferred scanner class.' 
                           initialAnswer: scannerClass name
                           onCancel:[^ self].
        newScannerClass := Smalltalk at: scannerName asSymbol ifAbsent: [^self showCR: 'No class named ' , scannerName , '.'].
        newScannerClass isBehavior ifFalse: [^self showCR: scannerName , ' is not a class name.'].
        self scannerClass: newScannerClass.
        self showCR: 'Rebuild scanner and parser.'.
        self resetParserAndFlags.
        "Force generation of new scanner, even is spec has not changed."
        self oldScannerSpec: nil

    "Modified: 18.2.1997 / 12:34:21 / stefan"
!

defineTreeBuilderClass
        "Prompt the user for the name of a new parse tree builder class. An instance of this class will 
        be installed in each new parser built."

        | builderName builderClass |

        builderName := DialogView request: 'Type the name of the new parse tree builder class.' 
                                  initialAnswer: self treeBuilderClass name
                                  onCancel:[^ self].
        builderClass := Smalltalk at: builderName asSymbol ifAbsent: [^self showCR: 'No class named ' , builderName , '.'].
        builderClass isBehavior ifFalse: [^self showCR: builderName , ' is not a class name.'].
        self treeBuilderClass: builderClass.
        self parser notNil
                ifTrue: 
                        [self parser treeBuilder: builderClass new.
                        self showCR: 'New parse tree builder installed in current parser.']

    "Modified: 18.2.1997 / 12:35:45 / stefan"
!

inspectGrammar

	self transformedGrammar isNil
		ifTrue: [self grammar inspect]
		ifFalse: [self transformedGrammar inspect]
!

inspectParser

	self parser isNil
		ifTrue: [self showCR: 'No parser to inspect.']
		ifFalse: [self parser inspect]
!

inspectResult

	self parserResult inspect
!

installScannerAndParserClasses
	| baseName category |
	(baseName := DialogView request: 'Type in a base name for the new scanner
and parser classes, e.g. if you typed ''Gork''
the classes created would be named
GorkScanner and GorkParser' initialAnswer: 'Gork') isEmpty ifFalse: [(category := DialogView request: 'Type in the category name for the new classes.' initialAnswer: 'category name') isEmpty
			ifFalse: 
				[self
					cr;
					show: 'Installing classes ';
					show: baseName;
					show: 'Scanner and ';
					show: baseName;
					show: 'Parser... '.
				self parser fastParser
					createScannerParserClassesNamed: baseName
					category: category
					tokenSpec: self tokenSpecification
					grammarSpec: self grammarSpecification.
				self showCR: 'done.']]
!

loadSpecs
        "load a token specification, a grammar specification, and a sample 
        input. Use a single file name with three different extensions: 
        *.tok, *.grm, and  *.eg."

        | file fileName |
        fileName := Dialog requestFileName: 'Enter base file name\\(standard extensions will be added):' withCRs 
                               default: (
                                   self specsFileName isNil
                                        ifTrue: ['Tgen']
                                        ifFalse: [self specsFileName]
                               )
                               version:#old
                               ifFail:[nil]
                               pattern:'*.tok'
                               fromDirectory:nil.

        fileName isNil ifTrue:[^ self].

        #('.eg' '.tok' '.grm') do:[:suffix|
            (fileName endsWith:suffix) ifTrue:[
                fileName := fileName copyWithoutLast:(suffix size).
            ].
        ].
        
        fileName isEmpty
                ifTrue: [^self]
                ifFalse: ["cache" self specsFileName: fileName].
        file := Filename named: fileName , '.tok'.
        file isReadable ifTrue: 
                [         
                    (grammarSpecification notNil 
                     or:[tokenSpecification notNil
                     or:[inputText notNil]]) ifTrue:[
(DialogView confirm: 'This load will overwrite the existing specs.
Are you sure you want to do this?') ifFalse: [^self].
                    ].
                    tokenSpecification := file contentsOfEntireFile.
                    self changed: #tokenSpecification
                ]
                ifFalse: 
                        [self showCR: 'Could not find file ''' , fileName , '.tok' , '''  - skipping load.'.
                        ^self].
        file := Filename named: fileName , '.grm'.
        file isReadable
                ifTrue: 
                        [grammarSpecification := file contentsOfEntireFile.
                        self changed: #grammarSpecification].
        file := Filename named: fileName , '.eg'.
        file isReadable
                ifTrue: 
                        [inputText := file contentsOfEntireFile.
                        self changed: #inputText]

    "Modified: 21.2.1997 / 14:40:31 / stefan"
!

saveSpecs
	"Save the token specification, the grammar specification, and the 
	sample input.  Use a single file name with three different extensions:
	*.tok, *.grm, and *.eg."

	| fileName |
	fileName := DialogView request: 'Enter base file name (standard extensions will be added):' initialAnswer: (self specsFileName isNil
					ifTrue: ['Tgen']
					ifFalse: [self specsFileName]).
	fileName isEmpty
		ifTrue: [^self]
		ifFalse: ["cache" self specsFileName: fileName].
	Cursor write
		showWhile: 
			[tokenSpecification notNil ifTrue: [((Filename named: fileName , '.tok') writeStream) nextPutAll: tokenSpecification; close].
			grammarSpecification notNil ifTrue: [((Filename named: fileName , '.grm') writeStream) nextPutAll: grammarSpecification; close].
			inputText notNil ifTrue: [((Filename named: fileName , '.eg') writeStream) nextPutAll: inputText; close]]
!

show: aString 
        "Append aString to the status transcript text."

        statusTextStream show:aString.
        self changed: #statusTextBuffer

    "Modified: 18.2.1997 / 16:00:25 / stefan"
!

showCR: aString 
	"Append aString to the status transcript text and perform a carriage return."

	self show: aString; cr
!

statusTextBufferMenu
	"TranslatorGenerator flushMenus"

	^StatusTextBufferMenu isNil
		ifTrue: [StatusTextBufferMenu := PopUpMenu
						labelList: #(('again' 'undo') ('copy' 'cut' 'paste') ('accept' 'cancel') ('result' 'builder' 'grammar' 'parser' 'scanner') ('install') ('save specs' 'load specs'))
						values: #(#again #undo #copySelection #cut #paste #accept #cancel #inspectResult #defineTreeBuilderClass #inspectGrammar #inspectParser #defineScannerClass #installScannerAndParserClasses #saveSpecs #loadSpecs)]
		ifFalse: [StatusTextBufferMenu]
!

tab
	"Perform a tab over."

	self show: (String with: Character tab)
! !

!TranslatorGenerator methodsFor:'testing'!

buildAndInstallParserNamed: name tokenSpec: tSpec grammarSpec: gSpec ifFail: failBlock

	self tokenSpecification: tSpec.
	self grammarSpecification: gSpec.
	self buildParserIfFail: failBlock.
	self parser fastParser
		createScannerParserClassesNamed: name
		category: self testingCategoryName
		tokenSpec: self tokenSpecification
		grammarSpec: self grammarSpecification
!

grammarSpec: gSpec targetGrammar: gram ifFail: failBlock

	self grammarSpecification: gSpec.
	self buildParserIfFail: failBlock.
	self grammar printString = gram
		ifTrue: [Transcript show: '*']
		ifFalse: failBlock.
!

parseInputs: strings compareDTTo: dtTargets compareSASTTo: sastTargets tokenSpec: tSpec grammarSpec: gSpec ifFail: failBlock 

	| scannerName parserName dt |
	scannerName := (self testingClassBaseName , 'Scanner') asSymbol.
	parserName := (self testingClassBaseName , 'Parser') asSymbol.
	(Smalltalk includesKey: scannerName)
		ifTrue: [self error: 'A class named ' , scannerName , ' already exists.'].
	(Smalltalk includesKey: parserName)
		ifTrue: [self error: 'A class named ' , parserName , ' already exists.'].
	self
		buildAndInstallParserNamed: self testingClassBaseName
		tokenSpec: tSpec
		grammarSpec: gSpec
		ifFail: failBlock.
	strings with: dtTargets do:
		[:string :target |
		dt := (Smalltalk at: parserName) new parseForDerivationTree: string ifFail: failBlock.
		dt printString = target
			ifTrue: [Transcript show: '*']
			ifFalse: failBlock].
	strings with: sastTargets do:
		[:string :target |
		dt := (Smalltalk at: parserName) new parseForShamAST: string ifFail: failBlock.
		dt printString = target
			ifTrue: [Transcript show: '*']
			ifFalse: failBlock].
	(Smalltalk includesKey: scannerName)
		ifTrue: [(Smalltalk at: scannerName) removeFromSystem].
	(Smalltalk includesKey: parserName)
		ifTrue: [(Smalltalk at: parserName) removeFromSystem]
!

parseInputs: strings compareDTTo: targets tokenSpec: tSpec grammarSpec: gSpec ifFail: failBlock 

	| scannerName parserName dt |
	scannerName := (self testingClassBaseName , 'Scanner') asSymbol.
	parserName := (self testingClassBaseName , 'Parser') asSymbol.
	(Smalltalk includesKey: scannerName)
		ifTrue: [self error: 'A class named ' , scannerName , ' already exists.'].
	(Smalltalk includesKey: parserName)
		ifTrue: [self error: 'A class named ' , parserName , ' already exists.'].
	self
		buildAndInstallParserNamed: self testingClassBaseName
		tokenSpec: tSpec
		grammarSpec: gSpec
		ifFail: failBlock.
	strings with: targets do:
		[:string :target |
		dt := (Smalltalk at: parserName) new parseForDerivationTree: string ifFail: failBlock.
		dt printString = target
			ifTrue: [Transcript show: '*']
			ifFalse: failBlock].
	(Smalltalk includesKey: scannerName)
		ifTrue: [(Smalltalk at: scannerName) removeFromSystem].
	(Smalltalk includesKey: parserName)
		ifTrue: [(Smalltalk at: parserName) removeFromSystem]
!

parseInputs: strings compareDTToAny: targets tokenSpec: tSpec grammarSpec: gSpec ifFail: failBlock 
	"The argument 'strings' is an Array of strings.  The argument 'targets'
	is an Array of Array of strings.  The test for a given element of 'strings'
	will succeed if it matches any of the strings in the corresponding
	'targets' Array.  Due to the nondeterminism of hashing, productions
	are sometimes processed in different orders and new nonterminals
	get numbered differently.  As an example, see TranslatorGenerator
	class>>basicLLParserGenerationTest2."

	| scannerName parserName dt dtps |
	scannerName := (self testingClassBaseName , 'Scanner') asSymbol.
	parserName := (self testingClassBaseName , 'Parser') asSymbol.
	(Smalltalk includesKey: scannerName)
		ifTrue: [self error: 'A class named ' , scannerName , ' already exists.'].
	(Smalltalk includesKey: parserName)
		ifTrue: [self error: 'A class named ' , parserName , ' already exists.'].
	self
		buildAndInstallParserNamed: self testingClassBaseName
		tokenSpec: tSpec
		grammarSpec: gSpec
		ifFail: failBlock.
	strings with: targets do:
		[:string :target |
		dt := (Smalltalk at: parserName) new parseForDerivationTree: string ifFail: failBlock.
		dtps := dt printString.
		(target inject: false into: [:result :targetStr | result or: [dtps = targetStr]])
			ifTrue: [Transcript show: '*']
			ifFalse: failBlock].
	(Smalltalk includesKey: scannerName)
		ifTrue: [(Smalltalk at: scannerName) removeFromSystem].
	(Smalltalk includesKey: parserName)
		ifTrue: [(Smalltalk at: parserName) removeFromSystem]
!

testingCategoryName

	^'T-gen-Interface'
!

testingClassBaseName

	^'TgenTemporaryTest'
! !

!TranslatorGenerator methodsFor:'token text'!

acceptTokenSpec: aText from: aController 

	self tokenSpecification: aText.
	^true
! !

!TranslatorGenerator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TranslatorGenerator.st,v 1.9 2006/04/06 10:02:44 stefan Exp $'
! !

TranslatorGenerator initialize!
