"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

StandardSystemView subclass:#TranslatorGeneratorView
	instanceVariableNames:'grammarModeButtonViews parserModeButtonViews'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Interface'
!

!TranslatorGeneratorView class methodsFor:'documentation'!

documentation
"
    I am the front end to T-gen.
    Start me with:

        Tgen::TranslatorGeneratorView open

"
!

examples
"
                                                [exBegin]
    Tgen::TranslatorGeneratorView open
                                                [exEnd]
"
! !

!TranslatorGeneratorView class methodsFor:'initialization'!

fileInChanges
    "look for necessary base class additions, and file them in if
     not present"

    |fileName|

    (Object respondsTo:#isFSAFinalState) ifFalse:[
        [
            fileName := Dialog requestFileName: 'Need additions to base classes. Enter file name:' 
                               "/ default:'../../goodies/tgen/tgen.ch'
                               default:'../../goodies/tgen/extensions.st'
                               version:#mustBeOld
                               ifFail:[^self].
            fileName = '' ifTrue:[
                Dialog warn:'File does not exist. Try again'.
                true
            ].
        ] whileTrue. 
        
        Smalltalk fileIn:fileName logged:false.
    ].

    "self fileInChanges"

    "Created: 21.2.1997 / 10:50:54 / stefan"
    "Modified: 21.2.1997 / 11:05:49 / stefan"
! !

!TranslatorGeneratorView class methodsFor:'queries'!

isVisualStartable
    ^ true
! !

!TranslatorGeneratorView methodsFor:'private'!

label: label model: aModel getSelector: gSel putSelector: pSel selectorValue: aSymbol 
	"Create and return a radio button (view) to be used in aModel."

	| baseModel valueModel button |
	baseModel := (PluggableAdaptor on: aModel)
				getSelector: gSel putSelector: pSel.
	valueModel := (PluggableAdaptor on: baseModel)
				selectValue: aSymbol.
"/        button := LabeledBooleanView model: valueModel.
	button := Toggle model: valueModel.
"/        button beToggle.
"/        button controller beTriggerOnUp.
	button label: label.
	^button

    "Created: 7.2.1996 / 10:25:27 / stefan"
    "Modified: 7.2.1996 / 12:55:58 / stefan"
! !

!TranslatorGeneratorView methodsFor:'private-subview creation'!

addGrammarModeView: area on: aModel in:aView

        |panel|
        grammarModeButtonViews := OrderedCollection new.
        grammarModeButtonViews
                 add: (self
                                        label: 'LL(1)'
                                        model: aModel
                                        getSelector: #grammarMode
                                        putSelector: #setGrammarMode:
                                        selectorValue: aModel llModeMarker);
                 add: (self
                                        label: 'SLR(1), LALR(1), LR(1)'
                                        model: aModel
                                        getSelector: #grammarMode
                                        putSelector: #setGrammarMode:
                                        selectorValue: aModel lrModeMarker);
                 add: (self
                                        label: 'don''t care'
                                        model: aModel
                                        getSelector: #grammarMode
                                        putSelector: #setGrammarMode:
                                        selectorValue: aModel defaultModeMarker).

        panel := HorizontalPanelView new.
        grammarModeButtonViews do:[:each|
           panel addSubView:each
        ].
        aView add:panel in:area.

    "Modified: 7.2.1996 / 12:38:05 / stefan"
    "Created: 21.2.1997 / 12:10:19 / stefan"
!

addGrammarView: area on: aParserGen in:aView
        |view|

        view := UpdateRequestTextView
                                on: aParserGen
                                aspect: #grammarSpecification
                                change: #acceptGrammarSpec:from:
                                menu: #textMenu
                                updatePrompt: 'Grammar specification text has changed;
ignore these changes?'.
        aView add: ((HVScrollableView miniScroller:true) scrolledView:view) in: area

    "Modified: 7.2.1996 / 14:49:35 / stefan"
    "Created: 21.2.1997 / 12:01:41 / stefan"
!

addParserModeView: area on: aModel in:aView

        |panel|
        parserModeButtonViews := OrderedCollection new.
        parserModeButtonViews
                 add: (self
                                        label: 'Derivation'
                                        model: aModel
                                        getSelector: #parserMode
                                        putSelector: #setParserMode:
                                        selectorValue: aModel defaultModeMarker);
                 add: (self
                                        label: 'Trace'
                                        model: aModel
                                        getSelector: #parserMode
                                        putSelector: #setParserMode:
                                        selectorValue: aModel traceModeMarker);
                 add: (self
                                        label: 'sham AST'
                                        model: aModel
                                        getSelector: #parserMode
                                        putSelector: #setParserMode:
                                        selectorValue: aModel shamAstModeMarker);
                 add: (self
                                        label: 'AST'
                                        model: aModel
                                        getSelector: #parserMode
                                        putSelector: #setParserMode:
                                        selectorValue: aModel astModeMarker).
        panel := HorizontalPanelView new.
        parserModeButtonViews do:[:each|
           panel addSubView:each
        ].
        aView add:panel in:area.

    "Created: 21.2.1997 / 12:10:37 / stefan"
    "Modified: 21.2.1997 / 12:36:20 / stefan"
!

addStatusView: area on: aParserGen in:aView
        |view|

"/        view := TranscriptView
        view := TextCollector
                                on: aParserGen
                                aspect: #statusTextBuffer
                                change: #show:
                                menu: #statusTextBufferMenu.

        aView add: ((HVScrollableView miniScroller:true) scrolledView:view)
                in: area

    "Modified: 12.2.1996 / 12:14:21 / stefan"
    "Created: 21.2.1997 / 12:01:13 / stefan"
!

addTestView: area on: aParserGen in:aView
        |view|

        view := NonrelianceTextView
                                on: aParserGen
                                aspect: #inputText
                                change: #acceptInputText:from:
                                menu: #textMenu.

        aView add: ((HVScrollableView miniScroller:true) scrolledView:view)
                in: area

    "Modified: 7.2.1996 / 14:43:41 / stefan"
    "Created: 21.2.1997 / 12:01:55 / stefan"
!

addTokenView: area on: aParserGen in:aView

        | newSubview |
        newSubview := UpdateRequestTextView
                                on: aParserGen
                                aspect: #tokenSpecification
                                change: #acceptTokenSpec:from:
                                menu: #textMenu
                                updatePrompt: 'Token specification text has changed;
ignore these changes?'.
        "Direct access is needed for reporting token spec errors back to the proper view."
        aParserGen tokenSpecController: newSubview controller.
        aView add: ((HVScrollableView miniScroller:true) scrolledView:newSubview) in: area

    "Modified: 7.2.1996 / 14:48:20 / stefan"
    "Created: 21.2.1997 / 12:02:11 / stefan"
! !

!TranslatorGeneratorView methodsFor:'startup'!

open
        "TranslatorGeneratorView new open"

        | aParserGen topY midY spc vpanel buttonPanel button|

        self class fileInChanges.

        spc := ViewSpacing.

        aParserGen := TranslatorGenerator new.
        topY := 0.3.            "change this to re-proportion view"
        midY := 0.70.           "and this"
        self model: aParserGen;
             label: 'T-gen';
             minimumSize: 400 @ 250.

        vpanel := View 
                        origin:(0.0 @ 0.0) corner:(1.0 @ 1.0)
                        in:self.
        buttonPanel := View origin:(0.0 @ 1.0) corner:(1.0 @ 1.0)
                        in:self.

        self
                addGrammarModeView: (0.0 @ 0.0 extent: 0.5 @ 1.0)
                                on: aParserGen in:buttonPanel;
                 addParserModeView: (0.5 @ 0.0 extent: 0.5 @ 1.0)  
                                on: aParserGen in:buttonPanel.

        button := parserModeButtonViews first.
        vpanel bottomInset:(button height + spc + button borderWidth).
        buttonPanel topInset:(button height + (spc//2) + button borderWidth) negated.

        self
                 addTokenView: (0.0 @ 0.0 extent: 0.5 @ topY)
                                on: aParserGen in:vpanel;
                 addStatusView: (0.5 @ 0.0 extent: 0.5 @ topY)
                                on: aParserGen in:vpanel;
                 addGrammarView: (0.0 @ topY extent: 0.5 @ midY)
                                on: aParserGen in:vpanel;
                 addTestView: (0.5 @ topY extent: 0.5 @ midY)
                                on: aParserGen in:vpanel.

        self icon: (Icon constantNamed: #default).
        super open

    "Created: 7.2.1996 / 10:25:02 / stefan"
    "Modified: 21.2.1997 / 12:49:50 / stefan"
! !

!TranslatorGeneratorView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TranslatorGeneratorView.st,v 1.7 2006/04/05 09:22:34 cg Exp $'
! !
