'From Smalltalk/X, Version:2.10.8 on 12-feb-1996 at 16:19:48'                   !

AbstractSyntaxTreeBuilder subclass:#SmalltalkSyntaxTreeBuilder
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Compilers-Smalltalk'
!

SmalltalkSyntaxTreeBuilder comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Object tgenCopyright."
=================================================

I am an AST builder for a Smalltalk parser.'!

!SmalltalkSyntaxTreeBuilder methodsFor:'obsolete'!

argument: arg1 temporaries: arg2 statements: arg3 

	| arg |
	(arg := arg1) size = 0 ifTrue: [arg := #()].
	^BlockNode new arguments: arg asArray body: (self temporaries: arg2 statements: arg3)!

selector: arg1 temporaries: arg2 primitive: arg3 statements: arg4 

	| block methodNode |
	block := self
				arguments: arg1 extractEvenElements asArray
				temporaries: arg2
				statements: arg4.
	methodNode := MethodNode new
				selector: (self coalesceIntoSymbol: arg1 extractOddElements)
				primitive: arg3
				block: block.
	^MethodNodeHolder new node: methodNode! !

!SmalltalkSyntaxTreeBuilder methodsFor:'private'!

coalesceIntoSymbol: anOrderedCollection 

	| aStream |
	aStream := WriteStream on: (String new: 32).
	anOrderedCollection do: [:sym | aStream nextPutAll: sym].
	^aStream contents asSymbol! !

!SmalltalkSyntaxTreeBuilder methodsFor:'tree building'!

arguments: arg1 temporaries: arg2 statements: arg3 

	|args temp block|
	(args := arg1) size = 0 ifTrue: [args := #()].
	(temp := arg2) size = 0 ifTrue: [temp := #()].
	block := BlockNode arguments:args asArray home:nil variables:temp.
	block statements:arg3.
	^ block
!

asArray: collection 
	^ collection asArray
!

asCharacter: arg 

	^arg at: 2!

asInteger: num 

	^num asNumber!

asNumber: aString 
	"Create a number from a string of the form 
	(([0-9]+r)(\-)?)?[0-9A-Z]+(.[0-9A-Z]+)?((e|d)(\-)?[0-9]+)? 
	ParcPlace's Smalltalk-80, Release 4, allows the exponent character and base > 10 digist to be 
	either upper or lower case. This means that exponents are not allowed in base 15 or higher 
	numbers since the 'e' is a digit and not an exponent character. (Doubles in base 14 or higher 
	cannot have exponents due to a similar problem with 'd'). The above syntax for numbers 
	allows exponents on any base, allows quick differentiation between digits (0-9 and A-Z are 
	all full-height characters) and metacharacters ('r', 'e', and 'd'), and has a much simpler 
	specification and recognizer. Note it is easy to automatically convert Smalltalk-80 constants 
	in existing source to this new form. The method digitValue for Characters should be 
	reimplemented so it ignores a-z."

	| aStream radix value digit neg exp num den negExp |
	aStream := ReadStream on: aString.
	neg := false.
	radix := 10.            "default"
	value := 0.
	[aStream atEnd
		or: 
			[digit := aStream next newDigitValue.
			digit < 0 or: [digit >= radix]]]
		whileFalse: [value := value * radix + digit].
	aStream atEnd
		ifTrue: 
			["<unsigned integer>"
			^value]
		ifFalse: [aStream skip: -1].
	(aStream peekFor: $r)
		ifTrue: 
			[((radix := value) between: 2 and: 36)
				ifFalse: [^self error: 'INVALID RADIX:  ' , radix printString].
			neg := aStream peekFor: $-.
			value := 0.
			[aStream atEnd
				or: 
					[digit := aStream next newDigitValue.
					digit < 0 or: [digit >= radix]]]
				whileFalse: [value := value * radix + digit].
			digit >= radix
				ifTrue: 
					[aStream skip: -1.
					self error: aStream next printString , ' is not a vaild base ' , radix printString , ' digit.'].
			aStream atEnd
				ifTrue: 
					["<unsigned integer>r<signed integer>"
					neg
						ifTrue: [^value negated]
						ifFalse: [^value]]
				ifFalse: [aStream skip: -1]].
	(aStream peekFor: $.)
		ifTrue: 
			[den := 1.
			num := 0.
			[aStream atEnd
				or: 
					[digit := aStream next newDigitValue.
					digit < 0 or: [digit >= radix]]]
				whileFalse: 
					[num := num * radix + digit.
					den := den * radix].
			digit >= radix
				ifTrue: 
					[aStream skip: -1.
					self error: aStream next printString , ' is not a vaild base ' , radix printString , ' digit.'].
			value := value + (num / den).
			aStream atEnd
				ifTrue: 
					["<unsigned integer>.<unsigned integer>"
					"<unsigned integer>r<signed integer>.<unsigned integer>"
					neg
						ifTrue: [^value asFloat negated]
						ifFalse: [^value asFloat]]
				ifFalse: [aStream skip: -1]].
	(aStream peekFor: $d)
		ifTrue: [value := value asDouble]
		ifFalse: [(aStream peekFor: $e)
				ifFalse: [self error: 'Invalid number token:  ' , aString]].
	negExp := aStream peekFor: $-.
	exp := 0.
	[aStream atEnd or: [(digit := aStream next newDigitValue) < 0]]
		whileFalse: [exp := exp * 10 + digit].
	aStream atEnd
		ifTrue: 
			["<unsigned integer><exponent>"
			"<unsigned integer>.<unsigned integer><exponent>"
			"<unsigned integer>r<signed integer><exponent>"
			"<unsigned integer>r<signed integer>.<unsigned integer><exponent>"
			negExp ifTrue: [exp := exp negated].
			value := value * (radix raisedTo: exp).
			neg
				ifTrue: [^value negated]
				ifFalse: [^value]]
		ifFalse: [self error: 'The scanner should never allow this to happen.']!

asString: arg 

	^(arg copyFrom: 2 to: arg size - 1) asString!

asSymbol: arg 
	^ arg asSymbol
!

characterLiteral: arg 
	^ ConstantNode value: (self asCharacter: arg)
!

greaterThanSymbol
	^ #'>'
!

intLiteral: arg 

	^ ConstantNode value: (self asInteger: arg)
!

lessThanSymbol
	^ #'<'
!

literal: arg 
	^ ConstantNode value:arg
!

minusSymbol
	^ #'-'
!

negate: num 
	^ num negated
!

primitive: prim 

	^prim!

primitiveSymbol
	^ #primitive:
!

receiver: msgNode messages: orderedCollection 

	^ orderedCollection do: [:eachMessage | eachMessage receiver: msgNode receiver]; addFirst: msgNode.
"        ^CascadeNode new receiver: msgNode receiver messages: orderedCollection asArray "
!

receiver: rcvr selector: sel 

	^ UnaryNode 
		receiver: rcvr
		selector: sel
!

receiver: rcvr selector: sel argument: arg 
	"A type of arguments , an instance variable of MessageNode."

	^ BinaryNode
		receiver: rcvr
		selector: sel
		arg: arg
!

receiver: rcvr selectorAndArgument: keywordBinObjGroup 

	^ MessageNode 
		receiver: rcvr
		selector: (self coalesceIntoSymbol: keywordBinObjGroup extractOddElements)
		args: keywordBinObjGroup extractEvenElements asArray
!

resource: aSymbol 

	^ aSymbol
!

resourceSymbol
	^ #resource:
!

return: value 

	| oc |
	oc := OrderedChildren new.
	oc add: (ReturnNode expression: value).
	^oc
!

selector: sel 
	"A name of *receiver* is not its real name"

	^ UnaryNode receiver: (VariableNode new name: 'receiver')
		    selector: sel fold:false
!

selector: sel argument: arg 
	"A name of *receiver is not its real name"

	^ BinaryNode
		receiver: (VariableNode new name: 'receiver')
		selector: sel
		arg: arg
		fold: false
!

selector: arg1 primitive: arg2 temporaries: arg3 statements: arg4 

	| block methodNode |
	block := self arguments: arg1 extractEvenElements asArray
				temporaries: arg3
				statements: arg4.
"        methodNode := MethodNode new
				selector: (self coalesceIntoSymbol: arg1 extractOddElements)
				primitive: arg3
				block: block.
	^MethodNodeHolder new node: methodNode
"

	^ block
!

selectorAndArgument: keywordBinObjGroup 
	"This is part of a cascaded message so let the receiver be nil."

	^ MessageNode
		receiver: nil
		selector: (self coalesceIntoSymbol: keywordBinObjGroup extractOddElements)
		args: keywordBinObjGroup extractEvenElements asArray
		fold: false
!

skipAssign:aString
    ^ aString copyFrom:1 to:(aString size - 2)
!

stringLiteral: arg 

	^ ConstantNode value:(self asString: arg)
!

temporaries: arg1 statements: arg2 

	| temp |
	(temp := arg1) isNil ifTrue: [temp := #()].
	^SequenceNode new temporaries: temp statements: arg2!

variable:name
    ^ Variable name:name
!

variable: aVariable leftArrowAndValue: anExpression 

	^AssignmentNode 
		variable: aVariable
		expression: anExpression
		"leftArrow: true"
!

variable: aVariable value: anExpression 

	^AssignmentNode variable: aVariable expression: anExpression
!

vbarSymbol
	^ #'|'
! !

