"{ Package: 'stx:goodies/tgen' }"
!

!Array methodsFor:'reconstructing'!

reconstructOn: aStream
	"Emit #( elements) on aStream ."

	aStream
		 poundSign;
		 leftParenthesis;
		 space.
	1 to: self size do:
		[:index |
		(self at: index)
			reconstructOn: aStream.
		aStream space].
	aStream rightParenthesis
! !

!Array methodsFor:'reconstructing'!

reconstructOn: aStream using: tokenTable

	aStream
		 poundSign;
		 leftParenthesis;
		 space.
	1 to: self size do:
		[:index |
		(self at: index)
			reconstructOn: aStream using: tokenTable.
		aStream space].
	aStream rightParenthesis
! !

!Character methodsFor:'copying'!

copyUpToLast: char

	^self
! !

!Character methodsFor:'accessing'!

newDigitValue
	"Answer 0-9 if the receiver is $0-$9, 10-35 if it is $A-$Z, and < 0 otherwise.
	This is used to parse literal numbers of radix 2-36."

	| code value |
	code := self asInteger.
	value := code - $0 asInteger.
	value <= 9 ifTrue: [^value].
	value := code - $A asInteger.
	(value >= 0 and: [value < 26])
		ifTrue: [^value + 10].
	^-1
! !

!Character methodsFor:'printing & storing'!

reconstructOn:aStream
    aStream nextPut:$$; nextPut:self

    "Created: 6.2.1996 / 09:46:12 / stefan"
    "Modified: 6.2.1996 / 10:57:35 / stefan"
! !

!Character class methodsFor:'accessing untypeable characters'!

endOfInput
	"Answer the Character representing ctrl-d ."

	^self value: 4
! !

!Character class methodsFor:'accessing untypeable characters'!

leftParenthesis
	"Answer the Character representing a left parenthesis."

	^self value: 40
! !

!Character class methodsFor:'accessing untypeable characters'!

period
	"Answer the Character representing a carriage period."

	^self value: 46
! !

!Character class methodsFor:'accessing untypeable characters'!

poundSign
	"Answer the Character representing a pound sign."

	^self value: 35
! !

!Character class methodsFor:'accessing untypeable characters'!

rightParenthesis
	"Answer the Character representing a right parenthesis."

	^self value: 41
! !

!CharacterArray methodsFor:'copying'!

copyUpToLast: aCharacter
	"Answer a copy of the receiver from index 1 to the last occurrence of
	aCharacter, non-inclusive."

	| index |
	(index := self lastIndexOf: aCharacter) == 0 ifTrue: [^self].
	^self copyFrom: 1 to: index - 1

    "Modified: 6.2.1996 / 16:31:02 / stefan"
! !

!Class methodsFor:'adding/removing'!

removeFromSystem
    "ST-80 compatibility
     remove myself from the system"

    Smalltalk removeClass:self.
    Smalltalk removeKey:name.

    "Created: 6.2.1996 / 11:32:58 / stefan"
! !

!Collection methodsFor:'reconstructing'!

reconstructOn: aStream
	"Emit #( elements ) on aStream "

	aStream poundSign; leftParenthesis.
	self do:
		[:ea |
		ea reconstructOn: aStream.
		aStream space].
	aStream rightParenthesis
! !

!Dictionary methodsFor:'converting'!

asDictionary

	^self
! !

!Dictionary methodsFor:'accessing'!

elements

	^self values
! !

!Dictionary methodsFor:'reconstructing'!

reconstructOn: aStream
	"Emit #( keys ) and #( values ) on aSteam"

	aStream
		 poundSign;
		 leftParenthesis;
		 space.
	self
		associationsDo:
			[:assoc |
			assoc key reconstructOn: aStream.
			aStream space].
	aStream
		 rightParenthesis;
		 space;
		 poundSign;
		 leftParenthesis.
	self
		associationsDo:
			[:assoc |
			assoc value reconstructOn: aStream.
			aStream space].
	aStream rightParenthesis; space
! !

!Dictionary methodsFor:'accessing'!

valuesAsSet
	"Answer a set containing the receiver's values."

	| aSet |
	aSet := Set new: self size.
	self do: [:each | aSet add: each].
	^aSet
! !

!MessageNode methodsFor:'initialize-release'!

receiver: rcvr

	receiver := rcvr
! !

!Number methodsFor:'converting'!

reversePairWith: x
	"Answer a new Point whose x value is the argument and whose y value is the receiver."

	^Point x: x y: self
! !

!Object methodsFor:'converting'!

@ anObject
	"Answer an OrderedPair with the receiver as the x element and anObject as the y element."

	^Tgen::OrderedPair x: self y: anObject
! !

!Object methodsFor:'testing'!

isAlternationNode

	^false
! !

!Object methodsFor:'testing'!

isConcatenationNode

	^false
! !

!Object methodsFor:'testing'!

isEpsilonNode

	^false
! !

!Object methodsFor:'testing'!

isFSAFinalState

	^false
! !

!Object methodsFor:'testing'!

isGrammarProduction

	^false
! !

!Object methodsFor:'testing'!

isItemSet

	^false
! !

!Object methodsFor:'testing'!

isLR0Item

	^false
! !

!Object methodsFor:'testing'!

isLR1Item

	^false
! !

!Object methodsFor:'testing'!

isNonterminal

	^false
! !

!Object methodsFor:'testing'!

isPartitionTransitionMap

	^false
! !

!Object methodsFor:'testing'!

isTerminal

	^false
! !

!Object methodsFor:'testing'!

isTerminalNode

	^false
! !

!Object methodsFor:'testing'!

isTokenClassification

	^false
! !

!Object methodsFor:'reconstructing'!

reconstructOn: aStream

	self printOn: aStream
! !

!Object methodsFor:'reconstructing'!

reconstructOn: aStream using: dummy

	self printOn: aStream
! !

!Object methodsFor:'converting'!

reversePairWith: x
	"Answer a new OrderedPair whose x value is the argument and whose y value is the receiver."

	^OrderedPair x: x y: self
! !

!OrderedCollection methodsFor:'accessing'!

extractEvenElements
	"Answer a collection like the receiver containing only the even indexed elements of the
	receiver."

	| newCollection inx newSize |
	newSize := (self size / 2) floor.
	newCollection := self species new:newSize.
	inx := 1.
	newSize
		timesRepeat:
			[newCollection add: (self at: 2 * inx).
			inx := inx + 1].
	^newCollection
! !

!OrderedCollection methodsFor:'accessing'!

extractOddElements
	"Answer a collection like the receiver containing only the odd indexed elements of the
	receiver."

	| newCollection inx newSize |
	newSize := (self size / 2) ceiling.
	newCollection := self species new:newSize.
	inx := 1.
	newSize
		timesRepeat:
			[newCollection add: (self at: 2 * inx - 1).
			inx := inx + 1].
	^newCollection
! !

!SequenceableCollection methodsFor:'accessing'!

extractEvenElements
	"Answer a collection like the receiver containing only the even indexed elements of the
	receiver."

	| newCollection inx newSize |
	newSize := (self size / 2) floor.
	newCollection := self species new: newSize.
	inx := 1.
	newSize
		timesRepeat:
			[newCollection at: inx put: (self at: 2 * inx).
			inx := inx + 1].
	^newCollection
! !

!SequenceableCollection methodsFor:'accessing'!

extractOddElements
	"Answer a collection like the receiver containing only the odd indexed elements of the
	receiver."

	| newCollection inx newSize |
	newSize := (self size / 2) ceiling.
	newCollection := self species new: newSize.
	inx := 1.
	newSize
		timesRepeat:
			[newCollection at: inx put: (self at: 2 * inx - 1).
			inx := inx + 1].
	^newCollection
! !

!SequenceableCollection methodsFor:'enumerating'!

reverseDetect: aBlock ifNone: exceptionBlock
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the last element for which aBlock evaluates to true."

	self reverseDo: [:each | (aBlock value: each) ifTrue: [^each]].
	^exceptionBlock value
! !

!Stream methodsFor:'character writing'!

leftParenthesis
	"Append a left parenthesis character to the receiver."

	self nextPut: Character leftParenthesis
! !

!Stream methodsFor:'character writing'!

period
	"Append a period character to the receiver."

	self nextPut: Character period
! !

!Stream methodsFor:'character writing'!

poundSign
	"Append a # character to the receiver."

	self nextPut: Character poundSign
! !

!Stream methodsFor:'character writing'!

rightParenthesis
	"Append a right parenthesis character to the receiver."

	self nextPut: Character rightParenthesis
! !

!String methodsFor:'converting'!

asNonterminal

	^self asSymbol
! !

!String methodsFor:'testing'!

isTerminal

	^true
! !

!String methodsFor:'testing'!

isTokenClassTerminal

	^'<*>' match: self
! !

!String methodsFor:'reconstructing'!

reconstructOn: aStream

	self storeOn: aStream
! !

!Symbol methodsFor:'testing'!

isNonterminal

	^true
! !

!Symbol methodsFor:'testing'!

isTerminal

	^false
! !

!Symbol methodsFor:'testing'!

isTokenClassTerminal

	^false
! !
