"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailInboxFile subclass:#MHMailInboxFile
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

MHMailInboxFile comment:'I am a parser for mail files in the format output by the Unix ''mh'' program.
'
!


!MHMailInboxFile methodsFor:'scanning'!

mailMessagesDo: aBlock
	"Invoke the given block for each message in the mail inbox. The block argument is the text of a new message."

	| fileStream stream textStart textSize msgText |
	fileStream _ FileStream fileNamed: filename.
	Smalltalk garbageCollect.
	(fileStream size < (Smalltalk bytesLeft - 200000))
		ifTrue:
			["if possible, buffer the entire file in memory for speed"
			 stream _ ReadStream on: (fileStream contentsOfEntireFile).
			 fileStream _ nil]
		ifFalse:
			["otherwise, use the actual file stream, reading from disk"
			 stream _ fileStream].
	[self scanToNextMessageIn: stream] whileTrue:
		["skip the three-line message delimiter"
		 3 timesRepeat: [MailDB skipRestOfLine: stream].
		 textStart _ stream position.
		 self scanToNextMessageIn: stream.
		 textSize _ stream position - textStart.
		 stream position: textStart.
		 msgText _ stream next: textSize.
		 aBlock value: msgText].
	fileStream = nil ifFalse: [fileStream close].
!

scanToNextMessageIn: aStream
	"Scan to the start of the next message in the given stream. Answer true if we find a message delimiter, false if we hit the end of the stream first. The stream is left positioned at the start of the next message delimiter (if there is one) or at the end of the stream."

	| msgStart |
	[true] whileTrue:
		[(aStream skipTo: $:) ifFalse: [^false].	"end of stream"
		 msgStart _ aStream position - 1.
		 ((MailDB readStringLineFrom: aStream) = ':::::::::::::') ifTrue:
			["looking good..."
			 MailDB skipRestOfLine: aStream.	"skip message number"
			 ((MailDB readStringLineFrom: aStream) = '::::::::::::::') ifTrue:
				["found a message!!"
				 aStream position: msgStart. ^true]].
		 "false alarm, keep scanning"
		 aStream position: msgStart + 1].
! !

!MHMailInboxFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/MHMailInboxFile.st,v 1.1 2003/05/09 21:40:12 cg Exp $'
! !
