"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

Object subclass:#MailDBFile
	instanceVariableNames:'filename'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

MailDBFile comment:'I am an abstract class that collects some of the operations common to all mail database files.
'
!


!MailDBFile class methodsFor:'instance creation'!

openOn: fileName
	"Answer a new instance of me, backed by the file with the given name."

	^(super new) openOn: fileName
! !

!MailDBFile methodsFor:'error reporting'!

reportInconsistency
	"Report that the index file is not consistent with the messages file."

	self inform:
'The .index file is not consistent with the .messages file.
Choosing ''salvage & compact'' will rebuild the index from scratch.'
! !

!MailDBFile methodsFor:'file operations'!

delete
	"Delete this file."

	FileDirectory splitName: filename to: [:dirPath :name |
		(FileDirectory forFileName: filename) 
			deleteFileNamed: name ifAbsent: []].
!

openOn: aFileName
	"Initialize myself from the file with the given name."

	| fileStream |
	filename _ aFileName.
	fileStream _ FileStream fileNamed: aFileName.
	self readFrom: fileStream.
	fileStream setToEnd; close; release.
		"close and release the file stream"
	fileStream _ nil.
!

rename: newFileName
	"Rename this file."

	FileDirectory splitName: filename to: [:dirPath :oldFileName |
		(FileDirectory forFileName: filename) 
			rename: oldFileName toBe: newFileName].
	filename _ newFileName.
!

save
	"Atomically save a representation of this object to its file. The old file is 
	renamed to '<oldname>.bak' before the new file is written. If the write 
	operation fails, the old file may be restored by renaming it. If it 
	succeeds, the .bak file is deleted."
	"create the file if it doesn't already exist"
	| f dir shortName |

	(StandardFileStream fileNamed: filename) close.	"ensure it exists"
	shortName _ FileDirectory localNameFor: filename.
	dir _ FileDirectory forFileName: filename.
	dir rename: shortName toBe: shortName , '.bak'.
	Cursor write showWhile: [
		f _ FileStream fileNamed: filename.
		self writeOn: f.
		f setToEnd; close
		].
	dir deleteFileNamed: shortName , '.bak' ifAbsent: []
! !

!MailDBFile methodsFor:'read-write'!

readFrom: aStream
	"Subclasses must override this method to provide a means of reading themselves into memory."

	self subclassResponsibility
!

writeOn: aStream
	"Subclasses must override this method to provide a means of writing themseves out on their files."

	self subclassResponsibility
! !

!MailDBFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/MailDBFile.st,v 1.1 2003/05/09 21:40:07 cg Exp $'
! !
