"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

ReadNewsInboxFile subclass:#RNInboxFile
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

RNInboxFile comment:'I am a parser for news files in the format output by the Unix ''rn'' program.
'
!


!RNInboxFile methodsFor:'scanning'!

nextStringOf: aStream equals: aString

	aString do:
		[: c | (c == (aStream next) ) ifFalse: [^false]].
	^true
!

parse: aStream do: aBlock
	"Parse the given stream into newsgroup articles, invoking the given block once for each article in the stream. The stream is divided into articles by delimiters that includes the newsgroup name. Ignore text before the first article delimiter."

	| done line groupName |
	currentNewsgroup _ nil.  "have not found start of article"
	msgBuffer _ WriteStream on: (String new: 5000).
	done _ false.
	[done] whileFalse:
		[(aStream atEnd) ifTrue:
			["end of stream"
			 self endOfArticleDo: aBlock.
			 done _ true].
		 line _ MailDB readStringLineFrom: aStream.
		 groupName _ self startOfArticle: line.
		 (groupName notNil) ifTrue:
			[self endOfArticleDo: aBlock.
			 currentNewsgroup _ groupName.
			 line _ MailDB readStringLineFrom: aStream].
		 (currentNewsgroup notNil) ifTrue: [self appendLine: line]].
!

startOfArticle: aString
	"Answer the newsgroup name if the given string is the start of a news article, for example:
		Article 2958 of comp.lang.smalltalk:
	Otherwise, answer nil."

	| s name |
	s _ ReadStream on: aString.
	(self nextStringOf: s equals: 'Article ') ifFalse: [^nil].
	[s next isDigit] whileTrue.	"consumes digits plus the following space"
	(self nextStringOf: s equals: 'of ') ifFalse: [^nil].
	name _ s upTo: $:.
	((name size > 1) & (s atEnd)) ifFalse: [^nil].
	^name
! !

!RNInboxFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/RNInboxFile.st,v 1.1 2003/05/09 21:40:19 cg Exp $'
! !
