"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfLinkedContext subclass:#TfBoxContext
	instanceVariableNames:'position bounds protectedBoxes boxMargin measuredExtent
		localTranslation rightMargin pendingPositionDelta'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfBoxContext methodsFor:'accessing'!

addProtectedBox: aTfElementLayout

	self protectedBoxes add: aTfElementLayout
!

alignToLocalPosition: aPoint

	self position: self position - aPoint.
	self bounds: (self bounds translatedBy: aPoint negated).
	localTranslation := aPoint.
	^ localTranslation
!

alignToZero

	^ self alignToLocalPosition: self bounds origin
!

applyPendingPositionDelta

	| newPos |
	pendingPositionDelta notNil
		ifTrue: [
			pendingPositionDelta ~= (0@0)
				ifTrue: [
					newPos := self applyProtectedBoxesTo: self position + self pendingPositionDelta flowRight: true.
					self position: newPos.
				].
			self pendingPositionDelta: nil.
		].
!

applyPositionToParent

	self parentContext notNil
		ifTrue: [ self parentContext globalPosition: self globalPosition ]
!

applyProtectedBoxesTo: aPoint flowRight: flowRight
	" make sure that the point is not within a protected box "

	| i  p matchingLayout gb |
	" all protected boxes are in global coords "
	p := aPoint + self globalTranslation.
	flowRight
		ifFalse: [ 
			p x: (p x min: self globalBounds right - rightMargin)
		].
	self protectedBoxes isEmpty
		ifFalse: [
			i := 0.
			[
				matchingLayout := self protectedBoxes
						detect: [ : layout | layout globalProtectedBox containsPoint: p ]
						ifNone: [ nil ].
				matchingLayout notNil and: [ i <= self protectedBoxes size ]
			] whileTrue: [
				| box |
				box := matchingLayout globalProtectedBox.
				gb := self globalBounds.
				p := flowRight
					ifTrue: [ 
						box right >= (gb right - self uselessWidth)
							ifTrue: [ gb left@(box bottom + 1) ]
							ifFalse: [ (box right + 1)@p y ].
					] ifFalse: [
						box left <= (gb left + self uselessWidth)
							ifTrue: [ gb left@(box bottom + 1) ]
							ifFalse: [ (box left - 1)@p y ].
					].
				i := i + 1.
			].
		].
	^ p - self globalTranslation
!

bounds
	" bounding rectangle within the document's bounds "
	" ^ Rectangle "

	^ bounds
!

bounds: aRectangle
	" bounding rectangle within the document's bounds "

	bounds := aRectangle.
!

extent

	^ self bounds extent
!

extent: aPoint

	self bounds: (self bounds origin extent: aPoint).
!

globalBounds

	^ self bounds translatedBy: self globalTranslation
!

globalPosition

	^ self position + self globalTranslation
!

globalPosition: aPoint

	self position: aPoint - self globalTranslation
!

height

	^ self bounds height
!

insetBy: inset

	bounds := bounds insetBy: inset.
	self translatePositionBy: inset tfAsTranslation.
!

newLine: currentLineHeight
	" start a new line. "

	| newPos |
	newPos := self bounds left@(self position y + currentLineHeight).
	newPos := self applyProtectedBoxesTo: newPos flowRight: true.
	self position: newPos.
!

position
	" local position within the bounds "
	" ^ Point "

	^ position
!

position: aPoint
	" local position within the bounds "

	self basicPosition: aPoint.
!

protectedBoxMargin

	boxMargin isNil
		ifTrue: [ boxMargin := 3@3 ].
	^ boxMargin
!

remainingLineWidth

	^ self bounds right - self position x
!

remainingWidth

	^ self bounds right - self position x
!

reset

	self basicPosition: self bounds origin.
!

resetLine
	" reset the current line "

	self basicPosition: self bounds left@self position y.
	rightMargin := 0.
!

resetPendingPositionDelta

	pendingPositionDelta := nil
!

rightIndentBy: anInteger

	rightMargin := rightMargin + anInteger.
	^ self bounds right - anInteger
!

width

	^ self bounds width
!

width: anInteger

	self bounds width: anInteger
! !

!TfBoxContext methodsFor:'genetic stuff'!

beClone

	super beClone.
	protectedBoxes := nil.
! !

!TfBoxContext methodsFor:'initialize-release'!

initialize

	super initialize.
	self bounds: (0@0 extent: 0@0).
	self position: 0@0.
	localTranslation := 0@0.
	rightMargin := 0.
! !

!TfBoxContext methodsFor:'measuring'!

isForMeasuring

	^ measuredExtent notNil
!

measureExtent: aPoint

	self basicPosition: (position x + aPoint x)@(position y max: aPoint y).
!

measureExtentAdd: aPoint

	self basicPosition: position + aPoint.
!

measureExtentMax: aPoint

	self basicPosition: (position max: aPoint).
!

measureNextLine

	measuredExtent := measuredExtent max: position.
	position := 0@0.
!

measuredExtent

	^ measuredExtent max: position
!

prepareForMeasuring

	measuredExtent := 0@0.
	position := 0@0.
	bounds := 0@0 extent: SmallInteger maxVal asPoint
! !

!TfBoxContext methodsFor:'private'!

basicPosition: aPoint

	position := aPoint.
!

pendingPositionDelta

	pendingPositionDelta isNil
		ifTrue: [ pendingPositionDelta := 0@0 ].
	^ pendingPositionDelta
!

pendingPositionDelta: aPoint

	pendingPositionDelta := aPoint
!

pendingPositionDeltaMax: aPoint

	pendingPositionDelta := pendingPositionDelta notNil
		ifTrue: [ pendingPositionDelta max: aPoint ]
		ifFalse: [ aPoint ]
!

protectedBoxes

	protectedBoxes isNil
		ifTrue: [ protectedBoxes := OrderedCollection new ].
	^ protectedBoxes
!

uselessWidth
	" if there is less space than this, it is useless to do any layouts "

	^ 4
! !

!TfBoxContext methodsFor:'translation'!

globalTranslation

	| tr |
	tr := localTranslation.
	self parentContext notNil 
		ifTrue: [ tr := tr + self parentContext globalTranslation  ].
	^ tr
!

moveBy: aPoint

	aPoint = (0@0)
		ifFalse: [ self position: self position + aPoint ].
!

translateBy: aPoint

	localTranslation := localTranslation + aPoint.
!

translatePositionBy: aPoint

	aPoint = (0@0)
		ifFalse: [ self position: self position + aPoint ].
!

translation

	^ localTranslation
! !

!TfBoxContext class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfBoxContext.st,v 1.1 2009/10/01 18:26:35 cg Exp $'
! !
