"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfBrowserModel
	instanceVariableNames:'resourceManager configuration browserWindowModels dialogLock
		activeBrowserWindowModel styleSheet settings'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfBrowserModel class methodsFor:'instance creation'!

new

	^ super new initialize
! !

!TfBrowserModel methodsFor:'accessing'!

activeBrowserWindowModel

	^ activeBrowserWindowModel
!

activeBrowserWindowModel: aTfBrowserWindowModel

	activeBrowserWindowModel := aTfBrowserWindowModel.
	self triggerEvent: #activeBrowserWindowModel with: aTfBrowserWindowModel with: self.
!

addWindowModel: aTfBrowserWindowModel

	| i |
	aTfBrowserWindowModel browserModel: self.
	i := self browserWindowModels indexOf: aTfBrowserWindowModel ifAbsent: [ nil ].
	i isNil
		ifTrue: [ " new window "
			self browserWindowModels addLast: aTfBrowserWindowModel.
			self triggerEvent: #addedWindow with: aTfBrowserWindowModel with: self.
		]
		ifFalse: [ " changed window "
			self browserWindowModels at: i put: aTfBrowserWindowModel.
			self triggerEvent: #changedWindow with: aTfBrowserWindowModel with: self.
		].
!

bookmarks

	^ self settings bookmarks
!

bookmarks: aTfBookmarkContainer 

	self settings bookmarks: aTfBookmarkContainer
!

browserWindowModels

        browserWindowModels isNil
                ifTrue: [ browserWindowModels := List new ].
        ^ browserWindowModels
!

dialogLockDo: aBlock
	" avoid popping up several modal dialogs from different processes "

	^ dialogLock critical: aBlock
!

indexOfWindow: aTfBrowserWindowModel

	^ self browserWindowModels indexOf: aTfBrowserWindowModel ifAbsent: [ nil ]
!

removeWindowModel: aTfBrowserWindowModel

	| wm |
	wm := self browserWindowModels remove: aTfBrowserWindowModel ifAbsent: [ nil ].
	wm notNil
		ifTrue: [ 
			self activeBrowserWindowModel == aTfBrowserWindowModel
				ifTrue: [ self activeBrowserWindowModel: nil ].
			self triggerEvent: #removedWindow with: wm with: self
		].
!

resourceManager

	resourceManager isNil
		ifTrue: [ resourceManager := TfResourceManager browserModel: self ].
	^ resourceManager
!

resourceManager: aTfResourceManager

	resourceManager := aTfResourceManager
!

settings

	settings isNil
		ifTrue: [ settings := TfSettings loadOrNew ].
	^ settings
!

settings: aTfSettings

	settings := aTfSettings
!

styleSheet

	| sh |
	styleSheet isNil
		ifTrue: [ 
			sh := TfDefaultStyleSheet fromSettings: self settings.
			self styleSheet: sh 
		].
	^ styleSheet
!

styleSheet: aTfStyleSheet

	styleSheet notNil ifTrue: [ styleSheet retractAllInterestsFor: self ].
	styleSheet := aTfStyleSheet.
	aTfStyleSheet onStyleChangeSend: #changedStyle to: self.
!

windowModelOfClass: modelClass

	^ self browserWindowModels detect: [ : wm | wm class = modelClass ] ifNone: [ nil ]
! !

!TfBrowserModel methodsFor:'initialize-release'!

initialize

	dialogLock := RecursionLock new.
	configuration := Dictionary new.
!

shutDown

	self settings save.
	self browserWindowModels copy do: [ : bwm | bwm shutDown ].
	self resourceManager shutDown.
! !

!TfBrowserModel methodsFor:'notification'!

onAddedWindowSend: methodSelector to: anObject

	self when: #addedWindow send: methodSelector to: anObject
!

onChangedWindowSend: methodSelector to: anObject

	self when: #changedWindow send: methodSelector to: anObject
!

onRemovedWindowSend: methodSelector to: anObject

	self when: #removedWindow send: methodSelector to: anObject
! !

!TfBrowserModel methodsFor:'private'!

changedStyle

	self browserWindowModels do: [ : bwm | bwm defaultStyleChanged ]
! !

!TfBrowserModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfBrowserModel.st,v 1.2 2009/10/01 23:36:41 cg Exp $'
! !
