"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfDocumentWrapper subclass:#TfBrowserWrapper
	instanceVariableNames:'boundsProcess boundsQueue invalidateSemaphore scrollToEntry'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfBrowserWrapper methodsFor:'accessing'!

component: aView

	component notNil
		ifTrue: [ component release; container: nil ].
	self setComponent: aView.
	self isOpen
		ifTrue: [
			invalidateSemaphore critical: [
				self basicBounds: self bounds.
				self scrollOffsetHolder notNil
					ifTrue: [ self scrollOffsetHolder valueWithoutDisplaying: 0@0 ].
				self invalidateRectangle: self bounds repairNow: true.
			].
		].
!

invalidateRectangle: aRectangle repairNow: aBoolean forComponent: aVisualComponent

	invalidateSemaphore critical: [].
	super invalidateRectangle: aRectangle repairNow: aBoolean forComponent: aVisualComponent.
!

resourceWindowModel: aTfResourceWindowModel

	super resourceWindowModel: aTfResourceWindowModel.
	scrollToEntry := aTfResourceWindowModel urlDescriptor.
	aTfResourceWindowModel navigationModel 
		when: #navigationInPage send: #scrollToEntry: to: self;
		when: #onNavigation send: #onNavigation: to: self.
! !

!TfBrowserWrapper methodsFor:'bounds accessing'!

bounds: newBounds
	" the windows VM has the habit of flooding the system with window resize events.
	this is bad for Twoflower because changing bounds causes re--rendering.
	protect against this by using a shared queue as a buffer. "

	boundsQueue nextPut: newBounds.
! !

!TfBrowserWrapper methodsFor:'displaying'!

displayOn: aGraphicsContext

	boundsQueue isEmpty
		ifTrue: [ super displayOn: aGraphicsContext ]
		ifFalse: [ aGraphicsContext paint: ColorValue gray ]
! !

!TfBrowserWrapper methodsFor:'initialize-release'!

initialize

	super initialize.
	invalidateSemaphore := RecursionLock new.
	boundsQueue := SharedQueue new.
	boundsProcess := [ self boundsProcessLoop ] forkAt: Processor userBackgroundPriority.
!

release

	boundsProcess terminate.
	super release.
! !

!TfBrowserWrapper methodsFor:'notification'!

onNavigation: navigationEntry

	scrollToEntry := navigationEntry
! !

!TfBrowserWrapper methodsFor:'private'!

basicBounds: newBounds

	super bounds: newBounds.
	"  scroll to anchor if needed "
	scrollToEntry notNil
		ifTrue: [ 
			self scrollToEntry: scrollToEntry.
			scrollToEntry := nil
		].
	self isOpen
		ifTrue: [
			invalidateSemaphore critical: [
				self scrollOffsetHolder value: self scrollOffsetHolder value.
			].
		].
!

boundsProcessLoop

	[	| newBounds |
		newBounds := boundsQueue next.
		(Delay forMilliseconds: 25) wait.
		[ boundsQueue isEmpty ] whileFalse: [
			newBounds := boundsQueue next 
		].
		self isOpen
			ifTrue: [
				self basicBounds: newBounds.
				self invalidateRectangle: newBounds repairNow: true.
			].
	] repeat.
! !

!TfBrowserWrapper class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfBrowserWrapper.st,v 1.1 2009/10/01 19:35:38 cg Exp $'
! !
