"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Object subclass:#TfCSSStyleReader
	instanceVariableNames:'cssBlocks'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!

TfCSSStyleReader class instanceVariableNames:'singleton'

"
 No other class instance variables are inherited by this class.
"
!


!TfCSSStyleReader class methodsFor:'instance creation'!

new

	^ super new initialize
! !

!TfCSSStyleReader class methodsFor:'singleton'!

flushSingleton

	singleton := nil
!

obsolete

	self flushSingleton.
	super obsolete.
!

singleton

	singleton isNil
		ifTrue: [ singleton := self new ].
	^ singleton
! !

!TfCSSStyleReader methodsFor:'CSS styles'!

style: tfStyle backgroundColor: cssValue
	" cssValue: CSS color  (e.g. #DA5E03) "

	tfStyle backgroundStyle color: (self valueReader parseColor: cssValue)
!

style: tfStyle color: cssValue
	" cssValue: CSS color (e.g. #DA5E03) "

	tfStyle fontStyle color: (self valueReader parseColor: cssValue)
!

style: tfStyle fontFamily: cssValue
	" cssValue: comma separated font names "

	| fontNames |
	fontNames := self valueReader parseFontNames: cssValue.
	tfStyle fontStyle family: fontNames.
!

style: tfStyle fontSize: cssValue
	" cssValue: font size (currently only pt.) "

	| pixelSizeLayout |
	pixelSizeLayout := self valueReader parseCSSFontSize: cssValue.
	pixelSizeLayout notNil
		ifTrue: [ tfStyle fontStyle pixelSizeLayout: pixelSizeLayout ].
!

style: tfStyle fontWeight: cssValue
	" cssValue: one of #(bold bolder lighter), or a numerical value between 100 and 900 "

	((#('bold' 'bolder') includes: cssValue asLowercase) or: [
			(Integer readFrom: cssValue readStream) >= 700 ])
		ifTrue: [ tfStyle fontStyle isBold: true ].
! !

!TfCSSStyleReader methodsFor:'initialize-release'!

initialize

	self initializeCSSBlocks
!

initializeCSSBlocks

	cssBlocks := Dictionary new.
	cssBlocks at: 'background-color' put: [ :style :cssValue |
		self style: style backgroundColor: cssValue ].
	cssBlocks at: 'color' put: [ :style :cssValue |
		self style: style color: cssValue ].
	cssBlocks at: 'font-family' put: [ :style :cssValue |
		self style: style fontFamily: cssValue ].
	cssBlocks at: 'font-weight' put: [ :style :cssValue |
		self style: style fontWeight: cssValue ].
	cssBlocks at: 'font-size' put: [ :style :cssValue |
		self style: style fontSize: cssValue ].
! !

!TfCSSStyleReader methodsFor:'parsing'!

parseCSSValues: cssValuesString for: tfStyle
	" parse all supported CSS styles of the CSS style string
	and apply them to the TfStyle "

	| assocs |
	assocs := self valueReader parseValuePairs: cssValuesString separatedBy: $:.
	assocs do: [ : cssAssociation | | block |
		block := self cssBlocks at: cssAssociation key ifAbsent: [ nil ].
		block notNil
			ifTrue: [ block value: tfStyle value: cssAssociation value ]
			ifFalse: [ Log logDebug: ['CSS style not supported: ', cssAssociation key] level: 2 ].
	].
! !

!TfCSSStyleReader methodsFor:'private'!

cssBlocks

	^ cssBlocks
!

valueReader

	^ TfHTMLValueReader singleton
! !

!TfCSSStyleReader class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfCSSStyleReader.st,v 1.1 2009/10/01 18:27:45 cg Exp $'
! !
