"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfTestCase subclass:#TfCSSTests
	instanceVariableNames:'reader valueReader style'
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerTesting'
!


!TfCSSTests methodsFor:'private'!

fontBaseSize

	^ TfHTMLValueReader singleton fontBaseSize
! !

!TfCSSTests methodsFor:'setup'!

setUp

	reader := TfCSSStyleReader new.
	valueReader := TfHTMLValueReader new.
! !

!TfCSSTests methodsFor:'testing'!

testBasicStyles

	style := TfStyle new.
	reader
		parseCSSValues:
			'background-color:#0000FF; color:#FFFF00; font-family:Fixedsys,Courier;'
		for: style.
	self should: [ style backgroundStyle color = ColorValue blue ].
	self should: [ style fontStyle color = ColorValue yellow ].
	self should: [ style fontStyle family asArray = #('Fixedsys' 'Courier')].
!

testColorConversion

	| color  |
	color := valueReader parseColor: '#000000'.
	self should: [ color = ColorValue black ].
	color := valueReader parseColor: '  #FffFFf  '.
	self should: [ color = ColorValue white ].
	color := valueReader parseColor: '#FffFFe'.
	self shouldnt: [ color = ColorValue white ].
	color := valueReader parseColor: '#fFff00'.
	self should: [ color = ColorValue yellow ].
	color := valueReader parseColor: '#00fFff'.
	self should: [ color = ColorValue cyan ].
	color := valueReader parseColor: 'Red'.
	self should: [ color = ColorValue red ].
	color := valueReader parseColor: 'yellow'.
	self should: [ color = ColorValue yellow ].
	color := valueReader parseColor: 'kotzgrün'.
	self should: [ color isNil ].
!

testFontConversions

	| sizeLayout fs |
	fs := self fontBaseSize.
	sizeLayout := valueReader parseFontSize: '7'.
	self should: [ (sizeLayout sizeRelativeTo: fs) > fs ].
	sizeLayout := valueReader parseFontSize: '1'.
	self should: [ (sizeLayout sizeRelativeTo: fs) < fs ].
	sizeLayout := valueReader parseFontSize: '+3'.
	self should: [ (sizeLayout sizeRelativeTo: fs) > fs ].
	sizeLayout := valueReader parseFontSize: '-3'.
	self should: [ (sizeLayout sizeRelativeTo: fs) < fs ].
	sizeLayout := valueReader parseCSSFontSize: '24pt'.
	self should: [ (sizeLayout sizeRelativeTo: fs) = 24 ].
	sizeLayout := valueReader parseCSSFontSize: 'small'.
	self should: [ (sizeLayout sizeRelativeTo: fs) < fs ].
	sizeLayout := valueReader parseCSSFontSize: 'medium'.
	self should: [ (sizeLayout sizeRelativeTo: fs) = fs ].
!

testFontNamesConversion

	| names |
	names := valueReader parseFontNames: 'Fixedsys'.
	self should: [ names asArray = #('Fixedsys') ].
	names := valueReader parseFontNames: 'Fixedsys,Courier'.
	self should: [ names asArray = #('Fixedsys' 'Courier') ].
	names := valueReader parseFontNames: ' Fixedsys , Courier '.
	self should: [ names asArray = #('Fixedsys' 'Courier') ].

	" bad HTML "
	names := valueReader parseFontNames: 'Fixedsys,'.
	self should: [ names asArray = #('Fixedsys') ].
!

testNotSupportedStyles
	" CSS styles not supported by Twoflower should be ignored "

	style := TfStyle new.
	reader
		parseCSSValues:
			'not-supported:value; background-color:#0000FF'
		for: style.
	self should: [ style backgroundStyle color = ColorValue blue ].
!

testReadInteger

	| int |
	int := valueReader parseInteger: '42'.
	self should: [ int = 42 ].
	int := valueReader parseInteger: '  -  42  '.
	self should: [ int = -42 ].
	int := valueReader parseInteger: '  +42'.
	self should: [ int = 42 ].
!

testReadTableWidth

	| widthLayout |
	widthLayout:= valueReader parseTableWidth: '42'.
	self should: [ (widthLayout sizeRelativeTo: 1013) = 42 ].
	widthLayout:= valueReader parseTableWidth: '50%'.
	self should: [ (widthLayout sizeRelativeTo: 200) = 100 ].
! !

!TfCSSTests class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfCSSTests.st,v 1.1 2009/10/01 23:39:25 cg Exp $'
! !
