"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfWrapper subclass:#TfCanvasWrapper
	instanceVariableNames:'subcanvas canvasApp spec isCanvasBuilt preferredExtent
		preferredBounds xmlElement'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfCanvasWrapper class methodsFor:'accessing'!

classForFormType: formTypeSymbol

	^ self withAllSubclasses detect: [ : cl | cl formTypes includes: formTypeSymbol ] ifNone: [ nil ]
!

formTypes

	^ #()
! !

!TfCanvasWrapper methodsFor:'bounds accessing'!

measureExtentFor: renderContext

	renderContext layoutContext measureExtent: (self preferredExtentFor: renderContext).
!

minimalExtentFor: renderContext

	^ 1@1
!

preferredExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	| ext |
	ext := self canvasApp preferredExtentFor: renderContext.
	ext := ext min: (renderContext boxContext extent).
	^ ext
!

tfLocalTranslateBy: aPoint

	self tfLocalTranslation: self tfLocalTranslation + aPoint
!

tfLocalTranslation

	^ self tfTranslatingWrapper translation
!

tfLocalTranslation: aPoint

	self tfTranslatingWrapper translation: aPoint
! !

!TfCanvasWrapper methodsFor:'private'!

buildSubcanvasWith: anUIBuilder

	| specSelector |
	isCanvasBuilt := true.
	specSelector := self canvasSpecSelector.
	(self canvasClass respondsTo: specSelector)
		ifFalse: [ ^ self ].
	self subcanvas 
		client: self canvasApp 
		spec: specSelector 
		builder: anUIBuilder.
!

canvasApp

	canvasApp isNil
		ifTrue: [ self createCanvasApp ].
	^ canvasApp
!

canvasClass

	^ self model canvasClass
!

canvasSpecSelector

	^ #windowSpec
!

createCanvasApp

	self subclassResponsibility
!

initializeSubcanvas

	| widgetWrapper |
	subcanvas := TfSubCanvas new.
	widgetWrapper := WidgetStateWrapper on: subcanvas.
	self setComponent: (TfBoundingWrapper on: widgetWrapper)
!

isCanvasBuilt

	^ isCanvasBuilt == true
!

spec
	" returns the spec of the canvas "

	| canvasClass specSelector |
	spec isNil
		ifTrue: [
			canvasClass := self canvasClass.
			specSelector := self canvasSpecSelector.
			(canvasClass notNil and: [ canvasClass  respondsTo: specSelector ])
				ifTrue: [ spec := (canvasClass perform: specSelector) decodeAsLiteralArray ].
		].
	^ spec
!

subcanvas

	subcanvas isNil
		ifTrue: [ self initializeSubcanvas ].
	^ subcanvas
! !

!TfCanvasWrapper methodsFor:'rendering'!

finishLayout: layoutContext

	| lineLayout |
	lineLayout := TfFormLayout 
					for: self 
					bounds: (preferredBounds translatedBy: self tfLocalTranslation).
	lineLayout addToLayoutContext: layoutContext.
!

setupRendering: renderContext
	" prepare the view for rendering "

	| lc formContext |
	formContext := renderContext formContext.
	renderContext layoutContext applyPendingLayouts.
	self canvasApp form: formContext form.
	self isCanvasBuilt 
		ifFalse: [ self buildSubcanvasWith: formContext formBuilder newSubBuilder ].
	lc := renderContext layoutContext.
	preferredBounds := 0@0 extent: (self preferredExtentFor: renderContext).
	self tfLocalTranslation: lc position.
	self bounds: preferredBounds.
	self finishLayout: renderContext layoutContext.
! !

!TfCanvasWrapper class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfCanvasWrapper.st,v 1.1 2009/10/01 19:36:10 cg Exp $'
! !
