"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfCompositeView subclass:#TfCompositeElementView
	instanceVariableNames:'layoutBox styleContext stripped'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfCompositeElementView methodsFor:'accessing'!

addComponent: aVisualComponent

	super addComponent: aVisualComponent.
	sortedComponents := nil.
!

alignment

	^ self style paragraphStyle alignment
!

stripped

	^ stripped == true
!

stripped: aBoolean

	stripped := aBoolean
! !

!TfCompositeElementView methodsFor:'bounds accessing'!

computePreferredBounds

	| cb |
	preferredBounds := nil.
	self components do: [ :component |
		(cb := component bounds) extent = (0@0)
			ifFalse: [
				preferredBounds := preferredBounds isNil
					ifTrue: [ cb copy ]
					ifFalse: [ preferredBounds merge: cb ].
			].
	].
	preferredBounds isNil
		ifTrue: [ preferredBounds := 0@0 extent: 0@0 ].
"	preferredBounds  = Screen default bounds
		ifTrue: [ Log logWarning: 'Document bounds seem to be wrong' ]."
	^ preferredBounds
!

layoutComponentsForBounds: newBounds
	"The receiver has been sized to the given parameters.  
	Re-layout all of the receiver's components."

	| maxRight delta |
	maxRight := self components 
					inject: 0
					into: [ : m : c | m max: (c preferredBounds right) ].
	(delta := maxRight - newBounds right) > 0
		ifTrue: [
			self components do: [ : c |
				c tfWrappedComponent alignment = #right
					ifTrue: [ c tfLocalTranslateBy: delta negated@0 ].
			].
		].
!

preferredBounds: aRectangle

	preferredBounds := aRectangle
!

preferredComponentBounds

	| componentBounds |
	componentBounds := nil.
	self components do: [ :component | 
		componentBounds isNil
			ifTrue: [ componentBounds := component preferredBounds copy ]
			ifFalse: [ componentBounds := componentBounds merge: component preferredBounds ].
	].
	componentBounds isNil
		ifTrue: [ componentBounds := 0@0 extent: 0@0 ].
	^ componentBounds
! !

!TfCompositeElementView methodsFor:'private'!

computeComponents

	components := OrderedCollection new.
	self createComponentsFor: self model.
!

createComponentsFor: aTfElement

	| elementView |
	aTfElement tfComponents do: [ :componentModel |
		elementView := self createViewForElement: componentModel.
		elementView notNil
			ifTrue: [ 	components add: elementView ].
		(elementView isNil or: [ elementView tfComposesComponents not ])
			ifTrue: [ self createComponentsFor: componentModel ].
	].
!

createViewForElement: aTfElement

	^ self createViewForElement: aTfElement class: aTfElement tfViewClass
!

createViewForElement: aTfElement class: viewClass

	| view wrapper |
	viewClass notNil
		ifTrue: [
			view := viewClass model: aTfElement.
			wrapper := self createWrapperForElementView: view.
			wrapper container: self
		].
	^ wrapper
!

createWrapperForElementView: aVisualPart

	^ aVisualPart tfWrapped
!

defaultDecorator

	^ self style backgroundStyle createDecorator
!

rectangleRelativeTo: containingBoundingBox

	^ containingBoundingBox
! !

!TfCompositeElementView methodsFor:'rendering'!

displayOn: aGraphicsContext

	aGraphicsContext intersectClip: self bounds.
	super displayOn: aGraphicsContext.
!

finishOwnRendering: renderContext
!

finishParagraph: renderContext

	| paragraph |
	paragraph := self style paragraphStyle.
	paragraph finishLayoutContext: renderContext layoutContext.
!

measureExtentFor: renderContext

	| pstyle layoutContext |
	pstyle := self style paragraphStyle.
	layoutContext := renderContext layoutContext.
	pstyle startMeasureContext: layoutContext.
	super measureExtentFor: renderContext.
	pstyle finishMeasureContext: layoutContext.
	preferredExtent := layoutContext measuredExtent.
!

removeDiscardedComponents

	self stripped
		ifFalse: [
			self components: (self components reject: [ :c | c canBeDiscarded ]).
			self stripped: true
		].
!

setupRendering: renderContext
	" prepare the view for rendering "

	self flushOwnRenderingCaches: false.
	self startOwnRendering: renderContext.
	self startParagraph: renderContext.
	self setupComponentRendering: renderContext.
	self finishParagraph: renderContext.
	self finishOwnRendering: renderContext.
	self removeDiscardedComponents.
!

setupStyleContext

	self styleContext setupStyleFor: self model.
!

startOwnRendering: renderContext
!

startParagraph: renderContext

	| paragraph |
	paragraph := self style paragraphStyle.
	paragraph startLayoutContext: renderContext layoutContext.
!

style

	^ self styleContext style
!

styleContext

	styleContext isNil
		ifTrue: [
			styleContext := self container styleContext createChildContext.
			self setupStyleContext.
		].
	^ styleContext
! !

!TfCompositeElementView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfCompositeElementView.st,v 1.1 2009/10/01 19:36:37 cg Exp $'
! !
