"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

CompositeView subclass:#TfCompositeView
	instanceVariableNames:'decorator preferredExtent debug helpText sortedComponents
		minimalExtent'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfCompositeView class methodsFor:'instance creation'!

components: someComponents

	^ self new
		components: someComponents;
		yourself.
! !

!TfCompositeView methodsFor:'accessing'!

addComponent: aVisualComponent

	components addLast: aVisualComponent.
	aVisualComponent container: self.
	sortedComponents := nil.
	preferredBounds := nil.
!

alignment

	^ nil
!

components: someComponents

	components := someComponents.
	sortedComponents := nil.
!

container

	^ super container
!

documentOffset
	" offset of the view relative to the document view "

	^ (self localPointToGlobal: 0@0) - (self tfTopDocumentView localPointToGlobal: 0@0)
!

graphicsContextFor: aVisualComponent
	" bug workaround "

	| pixmap |
	^ self container notNil
		ifTrue: [ super graphicsContextFor: aVisualComponent ]
		ifFalse: [
			" dummy context "
			Log log: 'No graphics context'.
			pixmap := Pixmap extent: 1@1.
			^ pixmap graphicsContext
		].
!

helpText

	^ helpText
!

helpText: aStringOrText

	helpText := aStringOrText
!

layer
	" layer of this view. Twoflower views sometimes overlap other views, so layers are required
	for correctly dispatching events "

	^ 1
!

listContainer

	^ self container notNil
		ifTrue: [ self container listContainer ]
		ifFalse: [ nil ]
!

removeComponent: aVisualComponent

	components remove: aVisualComponent ifAbsent: [ ^ self ].
	sortedComponents := nil.
	self upcastEvent: #tfNeedsRendering with: nil from: self.
!

sortedComponents
	" returns components sorted by layer "

	sortedComponents isNil
		ifTrue: [
			sortedComponents := components asSortedCollection: [ :c1 :c2 | c1 layer <= c2 layer ].
		].
	^ sortedComponents
!

tfWrapped

	^ TfTranslatingWrapper on: self
!

xmlNode

	^ self model
! !

!TfCompositeView methodsFor:'bounds accessing'!

changedBoundsOf: aSubComponent

Transcript show: '.'.
	preferredBounds := nil
!

componentBounds

	^ self bounds
!

height

	^ self bounds height
!

tfLocalTranslateBy: aPoint

	self tfLocalTranslation: self tfLocalTranslation + aPoint.
!

tfLocalTranslation

	^ self tfTranslatingWrapper translation
!

tfLocalTranslation: aPoint

	self tfTranslatingWrapper translation: aPoint
!

width

	^ self bounds width
! !

!TfCompositeView methodsFor:'controller accessing'!

defaultControllerClass

	^TfController
! !

!TfCompositeView methodsFor:'debug'!

debug

	debug isNil
		ifTrue: [ debug := false ].
	^ debug
!

debug: aBoolean

	debug := aBoolean.
!

displayDebugOn: aGraphicsContext

	(self debug and: [ Log isDebug ])
		ifTrue: [ self displayFlashOn: aGraphicsContext ].
!

displayFlashOn: aGraphicsContext

	| gc |
	gc := aGraphicsContext copy.
	gc paint: ColorValue red.
	(self bounds insetBy: (0@0 corner: 1@1)) displayStrokedOn: gc.
!

flash

	| gc |
	gc := self graphicsContext.
	(1 to: 3) 
		do: [ : i |
			self displayFlashOn: gc.
			gc flush.
			(Delay forMilliseconds: 100) wait.
			self invalidateRectangle: self bounds repairNow: true.
		] separatedBy: [
			(Delay forMilliseconds: 100) wait.
		].
!

tfAllViewsDo: aBlock

	aBlock value: self.
	self components do: [ :c | c tfAllViewsDo: aBlock ]
! !

!TfCompositeView methodsFor:'event driven'!

basicHandlerForMouseEvent: aMouseEvent 

	| ctrl |
	ctrl := self getController.
	ctrl isNil ifTrue: [^nil].
	^[ctrl handlerForMouseEvent: aMouseEvent]
		on: Object errorSignal 
		do: 
			[:ex | "Trap errors occurring while searching for the object wanting control."
			Controller badControllerSignal 
			raiseErrorString: 'Bad controller in handlerForMouseEvent:']
!

componentHandlerForMouseEvent: aMouseEvent 

	| handler |
	self sortedComponents reverseDo: [ :component |
		(handler := component handlerForMouseEvent: aMouseEvent) notNil
			ifTrue: [ ^ handler ]
	].
	^ nil
!

downcastEvent: aKey with: aParameter from: anInitiator 

	| result |
	^ aKey = #tfUpdateTextSelection
		ifTrue: [ 
			self updateTextSelectionStart: aParameter for: anInitiator.
			result := super downcastEvent: aKey with: aParameter from: anInitiator.
			self updateTextSelectionEnd: aParameter for: anInitiator.
			result
		] ifFalse: [
			super downcastEvent: aKey with: aParameter from: anInitiator.
		].
!

handlerForMouseEvent: aMouseEvent 

	| handler |
	^ [
		handler := self componentHandlerForMouseEvent: aMouseEvent.
		handler notNil
			ifTrue: [ handler ]
			ifFalse: [ self basicHandlerForMouseEvent: aMouseEvent ]
	] on: Error do: [ : ex |
		nil
	].
!

updateSpot: type

	[
		super updateSpot: type
	] on: Error do: [  : ex |
	].
! !

!TfCompositeView methodsFor:'notification'!

update: anAspect with: param from: sender

	anAspect = #flash
		ifTrue: [ ^ self flash ].
	anAspect = #removed
		ifTrue: [ ^ self container removeComponent: self ].
	^ super update: anAspect with: param from: sender
! !

!TfCompositeView methodsFor:'printing'!

printOn: aStream

	super printOn: aStream.
	aStream nextPut: $[.
	self bounds printOn: aStream.
	aStream nextPut: $].
! !

!TfCompositeView methodsFor:'private'!

computeComponents

	components := OrderedCollection new.
!

decorator

	^ decorator
!

decorator: aTfDecorator

	decorator := aTfDecorator
!

ownAdditionalExtent

	^ 0@0
!

registerAsLinkTargetNamed: anchorName
	" register this view as a named link target within the document "

	| docView pos |
	docView := self tfTopDocumentView.
	pos := (self localPointToGlobal: self bounds origin) - (docView localPointToGlobal: 0@0).
	docView 
		linkTargetPositionsAt: anchorName
		put: pos.
!

updateTextSelectionEnd: textSelection for: aDocumentView

	textSelection textStreamPendingDo: [ : str | 
		self styleContext style paragraphStyle breakAfter
			ifTrue: [ str cr ].
	].
!

updateTextSelectionStart: textSelection for: aDocumentView

	textSelection textStreamPendingDo: [ : str | 
		self styleContext style paragraphStyle breakBefore
			ifTrue: [ str cr ].
	].
! !

!TfCompositeView methodsFor:'rendering'!

computeExtentFor: renderContext extentBlock: extentBlock

	| prefExt |
	prefExt := 0@0.
	self components do: [ :component | | ext |
		ext := extentBlock value: component value: renderContext.
		prefExt := prefExt max: ext.
	].
	^ prefExt + self ownAdditionalExtent
!

computeMinimalExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	^ self
		computeExtentFor: renderContext
		extentBlock: [ :component :rc | component minimalExtentFor: rc ]
!

computePreferredExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	| layoutContext |
	layoutContext := renderContext pushLayoutContext.
	layoutContext prepareForMeasuring.
	self measureExtentFor: renderContext.
	renderContext popLayoutContext.
	^ preferredExtent
!

displayDecoratorOn: aGraphicsContext

	self decorator notNil
		ifTrue: [ self decorator displayOn: aGraphicsContext copy in: self bounds for: self ].
!

displayOn: aGraphicsContext

	self displayDecoratorOn: aGraphicsContext.
	super displayOn: aGraphicsContext.
	self displayDebugOn: aGraphicsContext.
!

flushOwnRenderingCaches: needsMeasuring
	" flush all caches associated with rendering "

	needsMeasuring
		ifTrue: [
" todo "
			minimalExtent := nil.		
"			preferredExtent := nil."
		].
	preferredBounds := nil.
!

flushRenderingCaches: needsMeasuring
	" flush all caches associated with rendering "

	self flushOwnRenderingCaches: needsMeasuring.
	self components do: [ : c | c flushRenderingCaches: needsMeasuring ]
!

measureExtentFor: renderContext

	self measureOwnExtentFor: renderContext.
	self components do: [ : c | c measureExtentFor: renderContext ].
!

measureOwnExtentFor: renderContext

	renderContext layoutContext measureExtentAdd: self ownAdditionalExtent
!

minimalExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	minimalExtent isNil
		ifTrue: [ minimalExtent := self computeMinimalExtentFor: renderContext ].
	^ minimalExtent
!

preferredExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	preferredExtent isNil
		ifTrue: [ self computePreferredExtentFor: renderContext ].
	^ preferredExtent
!

setupComponentRendering: renderContext
	" prepare the components for rendering "

	self components do: [ :component |
		self setupComponentRendering: component for: renderContext
	].
!

setupComponentRendering: component for: renderContext

	component setupRendering: renderContext.
!

setupRendering: renderContext
	" prepare the view for rendering "

	self flushOwnRenderingCaches: false.
	self setupComponentRendering: renderContext.
!

styleContext

	^ self container styleContext
!

tfRenderingFinished

	self components do: [ : c | c tfRenderingFinished ]
!

tfSetupComponents

	self components do: [ :c | c release ].
	self computeComponents.
	self components do: [ :c | c tfSetupComponents ].
	sortedComponents := nil.
! !

!TfCompositeView methodsFor:'testing'!

canBeDiscarded

	^ false
!

intersects: aRectangle

	^ ((self preferredBounds intersects: aRectangle) and: [ self decorator notNil ])
		ifTrue: [ true ]
		ifFalse: [ super intersects: aRectangle ]
!

isTableCell

	^ false
!

isTableRow

	^ false
!

tfComposesComponents

	^ true
! !

!TfCompositeView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfCompositeView.st,v 1.1 2009/10/01 19:37:00 cg Exp $'
! !
