"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

ControllerWithMenu subclass:#TfController
	instanceVariableNames:'keyboardProcessor'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-UIApplications-New'
!


!TfController class methodsFor:'resources'!

debugMenu
        "MenuEditor new openOnClass: self andSelector: #debugMenu"

        <resource: #menu>
        ^#(#Menu #(
                        #(#MenuItem 
                                #rawLabel: 'Inspect model' 
                                #value: #inspect 
                                #labelImage: #(#ResourceRetriever #ToolbarIconLibrary #inspectIt ) ) 
                        #(#MenuItem 
                                #rawLabel: 'Remove (experimental)' 
                                #value: #remove 
                                #labelImage: #(#ResourceRetriever #ToolbarIconLibrary #delete ) ) ) #(2 ) nil ) decodeAsLiteralArray
! !

!TfController methodsFor:'accessing'!

browserWindowModel

	^ self view xmlNode document documentModel browserWindowModel
!

documentMousePointFor: aMouseEvent
	" location of the event relative to the document "

	^ (self sensor mousePointFor: aMouseEvent) + self view documentOffset
!

keyboardProcessor

	^ keyboardProcessor
!

keyboardProcessor: aKeyboardProcessor

	keyboardProcessor := aKeyboardProcessor
!

menu
	
	| menuElement m |
	menuElement := self xmlMenuElement.
	m := menuElement notNil
		ifTrue: [ self browserWindowModel menuFor: menuElement ]
		ifFalse: [ super menu ].
	Log isDebug
		ifTrue: [ m addItemGroup: self class debugMenu menuItems ].
	^ m
!

selectionTracker

	^ TfTextSelectionTracker on: self
!

takeKeyboardFocus

	| kp |
	(kp := self keyboardProcessor) notNil
		ifTrue: [ 
			kp requestActivationFor: self.
			self class triggerEvent: #gettingFocus]
!

takeKeyboardFocusWith: aKeyboardProcessor

	self keyboardProcessor: aKeyboardProcessor.
	self takeKeyboardFocus.
! !

!TfController methodsFor:'events'!

enterEvent: event
	" subclasses should override tfEnterEvent:, not this one "

	self showDebugView
		ifTrue: [ 
			self view debug: true.
			self view invalidate
		].
	self tfEnterEvent: event.
	^ super enterEvent: event
!

exitEvent: event
	" subclasses should override tfExitEvent:, not this one "

	self showDebugView
		ifTrue: [ 
			self view debug: false.
			self view invalidate
		].
	self tfExitEvent: event.
	^ super exitEvent: event
!

handleEvent: anEvent 
	" override brain dead implementation of ControllerWithMenu "

	^anEvent dispatchTo: self
!

redButtonPressedEvent: event
	" subclasses should override tfRedButtonPressedEvent:, not this one "

	Log isDebug
		ifTrue: [ 
			InputState default ctrlDown 
				ifTrue: [ Log log: self model. ^ nil ].
			InputState default altDown 
				ifTrue: [ self view inspect. ^ nil ].
		].
	^ self tfRedButtonPressedEvent: event
!

tfEnterEvent: event

	^ nil
!

tfExitEvent: event

	^ nil
!

tfRedButtonPressedEvent: event

	| docController |
	docController := self view tfTopDocumentView controller.
	docController takeKeyboardFocus.
	^ self selectEvent: event
! !

!TfController methodsFor:'menu messages'!

copyTextToClipboard

	^ self xmlMenuElement copyTextToClipboard
!

exportTable

	self xmlMenuElement exportTable
!

isInTable

	^ self xmlMenuElement isInTable
! !

!TfController methodsFor:'menu processing'!

dispatchMenuSelection: menuSelection

	(menuSelection isSymbol and: [ (self menuMessageReceiver respondsTo: menuSelection) not ])
		ifTrue: [ self xmlMenuElement perform: menuSelection ]
		ifFalse: [ super dispatchMenuSelection: menuSelection ]
! !

!TfController methodsFor:'private'!

showDebugView

	^ ((Log isDebugLevel: 2) and: [ Log debugViewClass isNil or: [ Log debugViewClass == self view class ]])
!

xmlMenuElement

	^ self xmlNode
!

xmlNode

	^ self view xmlNode
! !

!TfController class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfController.st,v 1.1 2009/10/01 23:41:43 cg Exp $'
! !
