"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

XML::DOM_SAXDriver subclass:#TfDOMDriver
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-XML-SAX'
!


!TfDOMDriver methodsFor:'content handler'!

characters: aString

	stack last addNode: (TfText text: aString)
!

elementClass: aTfElementClass

	self elementClass: aTfElementClass attributes: nil
!

elementClass: aTfElementClass attributes: attributes

	self startElementClass: aTfElementClass attributes: attributes.
	self endElementClass: aTfElementClass.
!

elementClass: aTfElementClass attributesArray: attributesArray

	self elementClass: aTfElementClass attributes: (self attributesFromArray: attributesArray).
!

endElement: name

	^ self endElementClass: (self elementClassFor: name) name: name
!

endElementClass: aTfElementClass

	self endElementClass: aTfElementClass name: aTfElementClass defaultTagName
!

endElementClass: aTfElementClass name: name

	| tag |
	tag := self tagFor: name.
	(Notifier isDevelopment and: [ (self currentElement tag isLike: tag) not ])
		ifTrue: [ self halt: 'HTML mismatch' ].
	stack removeLast condenseList
!

startDocument

	document := TfDocument new.
	document dtd: DocumentType new.
	stack := OrderedCollection with: document
!

startElement: name

	^ self startElement: name attributes: nil
!

startElement: name attributes: attributes

	^ self startElementClass: nil name: name attributes: attributes
!

startElement: namespaceURI localName: localName qName: name attributes: attributes 

	| element tag |
	document == nil ifTrue: [ self startDocument ].
	tag := TfNodeTag new qualifier: (( name includes: $: ) 
					ifTrue: [ name copyUpTo: $: ]
					ifFalse: [ '' ])
			ns: namespaceURI
			type: localName.
	element := ( self elementClassFor: name ) tag: tag
			attributes: ( self attributesFromXMLAttributes: attributes )
			elements: OrderedCollection new.
	element namespaces: newNamespaces.
	newNamespaces := nil.
	stack size = 1 ifTrue: [ document dtd declaredRoot: name ].
	stack last addNode: element.
	stack addLast: element
!

startElementClass: aTfElementClass 

	^ self startElementClass: aTfElementClass attributes: nil
!

startElementClass: aTfElementClass attributes: attributes

	| name |
	name := aTfElementClass defaultTagName.
	^ self startElementClass: aTfElementClass name: name attributes: attributes
!

startElementClass: aTfElementClass attributesArray: attributesArray

	^ self startElementClass: aTfElementClass attributes: (self attributesFromArray: attributesArray)
!

startElementClass: aTfElementClassOrNil name: name attributes: attributes 

	| element tag elementClass |
	document == nil ifTrue: [ self startDocument ].
	tag := self tagFor: name.
	elementClass := aTfElementClassOrNil notNil 
			ifTrue: [ aTfElementClassOrNil ]
			ifFalse: [ self elementClassFor: tag asString ].
	element := elementClass tag: tag
			attributes: ( self attributesFromXMLAttributes: attributes )
			elements: OrderedCollection new.
	element namespaces: newNamespaces.
	newNamespaces := nil.
	stack size = 1 ifTrue: [ document dtd declaredRoot: name ].
	stack last addNode: element.
	stack addLast: element.
	^ element
! !

!TfDOMDriver methodsFor:'content handler - common elements'!

colWith: attributesArray

	self startElementClass: TfColElement attributesArray: attributesArray.
	self endElementClass: TfColElement.
!

endBold

	self endElement: 'b'
!

endCell

	self endElementClass: TfTableCellElement
!

endCol

	self endElementClass: TfColElement
!

endColGroup

	self endElementClass: TfColGroupElement
!

endFont

	self endElementClass: TfFontElement
!

endH1

	self endElementClass: TfHeadingElement name: 'h1'
!

endH2

	self endElementClass: TfHeadingElement name: 'h2'
!

endH3

	self endElementClass: TfHeadingElement name: 'h3'
!

endItalic

	self endElement: 'i'
!

endLink

	self endElementClass: TfAnchorElement
!

endRow

	self endElementClass: TfTableRowElement
!

endTable

	self endElementClass: TfTableElement
!

endUnderlined

	self endElementClass: TfUnderlineElement
!

lineBreak

	self elementClass: TfBrElement
!

nbsp

	self characters: (String with: (Character value: 160))
!

paragraph

	self elementClass: TfParagraphElement
!

startBold

	self startElement: 'b'
!

startCell

	self startElementClass: TfTableCellElement
!

startCellWith: attributesArray

	self startElementClass: TfTableCellElement attributesArray: attributesArray
!

startColGroup

	self startElementClass: TfColGroupElement
!

startColWith: attributesArray

	self startElementClass: TfColElement attributesArray: attributesArray
!

startFont

	self startElementClass: TfFontElement
!

startFontWith: attributesArray

	self startElementClass: TfFontElement attributesArray: attributesArray
!

startH1

	^ self startElementClass: TfHeadingElement name: 'h1' attributes: nil
!

startH2

	^ self startElementClass: TfHeadingElement name: 'h2' attributes: nil
!

startH3

	^ self startElementClass: TfHeadingElement name: 'h3' attributes: nil
!

startItalic

	self startElement: 'i'
!

startLinkWith: attributesArray

	self startElementClass: TfAnchorElement attributesArray: attributesArray
!

startRow

	self startElementClass: TfTableRowElement
!

startRowWith: attributesArray

	self startElementClass: TfTableRowElement attributesArray: attributesArray
!

startTable

	self startElementClass: TfTableElement
!

startTableWith: attributesArray

	self startElementClass: TfTableElement attributesArray: attributesArray
!

startUnderlined

	self startElementClass: TfUnderlineElement
! !

!TfDOMDriver methodsFor:'private'!

attributesFromArray: attributesArray 

	^ attributesArray collect: 
		[: tuple | | tag |
			( tuple first isString and: [ tuple last isString ]) ifFalse: [ self error: 'Illegal attribute array' ].
			tag := TfNodeTag new qualifier: '' ns: '' type: tuple first.
			XML.Attribute name: tag value: tuple last
		]
!

attributesFromXMLAttributes: someAttributes 

	someAttributes isNil ifTrue: [ ^ nil ].
	^ someAttributes collect: 
		[: attr | | tag |
			tag := TfNodeTag new qualifier: '' ns: '' type: attr tag type.
			XML.Attribute name: tag value: attr value
		]
!

currentElement

	^ stack last
!

defaultElementClass

	^ TfUnknownElement
!

defaultQualifier

	^ ''
!

elementClassFor: tag

	^ TfElement elementClassFor: tag ifAbsent: [ self defaultElementClass ]
!

nameSpaceForQualifier: aString

	^ ''
!

tagFor: aString 

	| nm1 str qualifier type ns |
	str := aString readStream.
	nm1 := str upTo: $:.
	str atEnd 
		ifTrue: 
		[	qualifier := self defaultQualifier.
			type := nm1
		]
		ifFalse: 
		[	qualifier := nm1.
			type := str upToEnd
		].
	ns := self nameSpaceForQualifier: qualifier.
	^ TfNodeTag nameSpace: ns qualifier: qualifier type: type
! !

!TfDOMDriver class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDOMDriver.st,v 1.1 2009/10/02 00:50:21 cg Exp $'
! !
