"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfStyleSheet subclass:#TfDefaultStyleSheet
	instanceVariableNames:'defaultCodeFontStyle defaultFontStyle settings'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Testing'
!


!TfDefaultStyleSheet class methodsFor:'instance creation'!

fontStyle: fontStyle codeFontStyle: codeFontStyle 

	^ self new
		defaultFontStyle: fontStyle;
		defaultCodeFontStyle: codeFontStyle;
		initializeStyles;
		yourself
!

fromSettings: aTfSettings
	"TfDefaultStyleSheet fromSettings: TfSettings load"
	"TfDefaultStyleSheet fromSettings: TfSettings new;save;yourself"

	| sh |
	sh := self new.
	sh 
		settings: aTfSettings;
		initializeStyles.
	^ sh
! !

!TfDefaultStyleSheet methodsFor:'initialize-release'!

initializeStyles

	styles := Dictionary new.
	self addDefaultStyle.
	self addBoldStyle.
	self addItalicStyle.
	self addLinkStyle.
	self addStrikeoutStyle.
	self addUnderlinedStyle.
	self addHeader1Style.
	self addHeader2Style.
	self addHeader3Style.
	self addHeader4Style.
	self addHeader5Style.
	self addHeader6Style.
	self addParagraphStyle.
	self addBrStyle.
	self addHrStyle.
	self addPreStyle.
	self addBodyStyle.
	self addTdStyle.
	self addThStyle.
	self addTrStyle.
	self addTableStyle.
	self addCodeStyle.
	self addListStyle.
	self addListItemStyle.
	self addDivStyle.
	self addFormStyle.
	self addTextAreaStyle.
	self addBlockquoteStyle.
	self addCenterStyle.
! !

!TfDefaultStyleSheet methodsFor:'predefined styles'!

addBlockquoteStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'blockquote'.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addBodyStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'body'.
	self addStyle: style.
!

addBoldStyle

	| style |
	#('b' 'strong') do: [ :tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		style fontStyle isBold: true.
		self addStyle: style.
	].
!

addBrStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'br'.
	style paragraphStyle breakBefore: true.
	self addStyle: style.
!

addCenterStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'center'.
	style paragraphStyle 
		alignment: #center;
		breakAfter: true;
		breakBefore: true.
	self addStyle: style.
!

addCodeStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'code'.
	self defaultCodeFontStyle modifyStylePart: style fontStyle isInherited: false.
	self addStyle: style.
!

addDefaultStyle

	defaultStyle := TfStyle new.
	self defaultFontStyle modifyStylePart: defaultStyle fontStyle isInherited: false.
	defaultStyle listStyle
		propertyAt: #indent putDefaultValue: self defaultParagraphSpacing * 2.
	defaultStyle styleSheet: self.
!

addDivStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'div'.
	style paragraphStyle
		breakBefore: true;
		breakAfter: true.
	self addStyle: style.
!

addFormStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'form'.
	style paragraphStyle applyPendingLayouts: true.
	self addStyle: style.
!

addHeader1Style

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'h1'.
	style fontStyle
		pixelSize: self defaultPixelSize * 2;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHeader2Style

	| style |
	style := TfStyle new.
	style tagName: 'h2'.
	style fontStyle
		pixelSize: (self defaultPixelSize * (8/6)) rounded;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHeader3Style

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'h3'.
	style fontStyle
		pixelSize: (self defaultPixelSize * (7/6)) rounded;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHeader4Style

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'h4'.
	style fontStyle
		pixelSize: self defaultPixelSize;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHeader5Style

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'h5'.
	style fontStyle
		pixelSize: (self defaultPixelSize * (5/6)) rounded;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHeader6Style

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'h6'.
	style fontStyle
		pixelSize: (self defaultPixelSize * (4/6)) rounded;
		isBold: true.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addHrStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'hr'.
	style paragraphStyle breakBefore: true.
	self addStyle: style.
!

addItalicStyle

	| style |
	#('i' 'address' 'em' 'var' 'cite' 'dfn') do: [ :tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		style fontStyle isItalic: true.
		self addStyle: style.
	].
!

addLinkStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'a'.
	style fontStyle
		isUnderlined: true;
		color: (ColorValue tfFromHTMLHex: '#0000AA').
	self addStyle: style.
!

addListItemStyle

	| style |
	#('li' 'dt' 'dd') do: [ :tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		style paragraphStyle
				breakAfter: true;
				breakBefore: true;
				applyPendingLayouts: true.
		self addStyle: style.
	].
!

addListStyle

	| style |
	#('ul' 'ol' 'blockquote' 'menu' 'dir' 'dl') do: [:tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		self modifyParagaphStyle: style.
		self addStyle: style.
	].
!

addParagraphStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'p'.
	self modifyParagaphStyle: style.
	self addStyle: style.
!

addPreStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'pre'.
	style paragraphStyle
		breakBefore: true;
		convertWhitespaces: false;
		isPreformatted: true;
		wrapText: false.
	self defaultCodeFontStyle modifyStylePart: style fontStyle isInherited: false.
	self addStyle: style.
!

addStrikeoutStyle

	| style |
	#('del') do: [ :tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		style fontStyle isStrikedOut: true.
		self addStyle: style.
	].
!

addTableStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'table'.
	style tableStyle cellSpacing: 1.
	style backgroundStyle borderColor: ColorValue gray.
	style paragraphStyle 
		breakBefore: true;
		isBox: true.
	self addStyle: style.
!

addTdStyle

	| style alignmentOrigin |
	style := self newStyleWithDefaults.
	style tagName: 'td'.
	style paragraphStyle isBox: true.
	alignmentOrigin := AlignmentOrigin new.
	alignmentOrigin
		topFraction: 0.5;
		topAlignmentFraction: 0.5.
	style tableCellStyle
		propertyAt: #alignmentOrigin putDefaultValue: alignmentOrigin.
	self addStyle: style.
!

addTextAreaStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'textArea'.
	style paragraphStyle 
		breakAfter: true;
		breakBefore: true.
	self addStyle: style.
!

addThStyle

	| style alignmentOrigin |
	style := self newStyleWithDefaults.
	style tagName: 'th'.
	style fontStyle isBold: true.
	style paragraphStyle isBox: true.
	alignmentOrigin := AlignmentOrigin new.
	alignmentOrigin
		leftFraction: 0.5;
		topFraction: 0.5;
		leftAlignmentFraction: 0.5;
		topAlignmentFraction: 0.5.
	style tableCellStyle
		propertyAt: #alignmentOrigin putDefaultValue: alignmentOrigin.
	self addStyle: style.
!

addTrStyle

	| style |
	style := self newStyleWithDefaults.
	style tagName: 'tr'.
	style paragraphStyle 
		breakAfter: true;
		isBox: true.
	self addStyle: style.
!

addUnderlinedStyle

	| style |
	#('u' 'ins') do: [ :tagName |
		style := self newStyleWithDefaults.
		style tagName: tagName.
		style fontStyle isUnderlined: true.
		self addStyle: style.
	].
! !

!TfDefaultStyleSheet methodsFor:'private'!

changedStyle

	self initializeStyles.
	super changedStyle.
!

changedStyleSettings

	self initializeStyles.
	self triggerEvent: #styleChanged.
!

defaultCodeFontStyle

	^ defaultCodeFontStyle isNil
		ifTrue: [ settings codeFontStyle ]
		ifFalse: [ defaultCodeFontStyle ]
!

defaultCodeFontStyle: aTfFontStyle

	defaultCodeFontStyle := aTfFontStyle
!

defaultFontStyle

	^ defaultFontStyle isNil
		ifTrue: [ settings defaultFontStyle ]
		ifFalse: [ defaultFontStyle ]
!

defaultFontStyle: aTfFontStyle

	defaultFontStyle := aTfFontStyle
!

defaultParagraphSpacing

	^ (self defaultPixelSize * (5/6)) rounded
!

defaultPixelSize

	^ self defaultStyle fontStyle pixelSize
!

initializeInternationalStyle

        | style cStyle encodings description success |
        encodings := OrderedCollection new.
"/        encodings add: Locale current defaultStreamEncoder name.
"/        encodings addAll: Screen default preferredFontClass tfEncodings.
        encodings add: #unicode.
        cStyle := CharacterAttributes newWithDefaultAttributes.
        description := self defaultStyle fontStyle createFontDescription.
        style := TextAttributes new.
        style setCharacterAttributes: cStyle.
        success := false.
        " check which encodings are available .... the hard way "
        [ success ] whileFalse: [
                [ 
                        description encodings: encodings.
                        cStyle setDefaultQuery: description.
                        "/ Screen default defaultFontPolicy findFont: description.
                        success := true.
                ] on: NoMatchingFontError do: [ : ex | | enc |
                        enc := ex parameter encoding.
                        enc notNil
                                ifTrue: [       encodings remove: enc ].
                ].
        ].
        TextAttributes styleNamed: #tfInternational put: style
!

modifyParagaphStyle: style

	style paragraphStyle
		breakAfter: true;
		spacingBefore: self defaultParagraphSpacing;
		breakBefore: true;
		spacingAfter: self defaultParagraphSpacing.
!

newStyleWithDefaults

	| style |
	style := TfStyle new.
	self defaultFontStyle modifyStylePart: style fontStyle isInherited: false.
	^ style
!

settings: aTfSettings
	" settings used to build the style sheet "

	settings := aTfSettings.
	aTfSettings onStyleChangeSend: #changedStyle to: self.
! !

!TfDefaultStyleSheet class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDefaultStyleSheet.st,v 1.2 2009/10/01 23:39:05 cg Exp $'
! !
