"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfController subclass:#TfDocumentController
	instanceVariableNames:'dispatchTable'
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerViews'
!


!TfDocumentController methodsFor:'events'!

processKeyboardEvent: anEvent 

	self dispatchTable
		add: anEvent 
		do:
			[:charEvent :selector |
			selector == #ignoreInputKey: ifFalse: [self perform: selector with: charEvent]].
! !

!TfDocumentController methodsFor:'keyboard'!

cursorDownKey: keyEvent

	self scrollByLines: 1
!

cursorLeftKey: keyEvent
!

cursorRightKey: keyEvent
!

cursorUpKey: keyEvent

	self scrollByLines: -1
!

endKey: keyEvent

	self scrollTo: 0@self view preferredBounds bottom
!

homeKey: keyEvent

	self scrollTo: 0@0
!

pageDownKey: keyEvent

	(self scrollByPages: 1)
		ifFalse: [self class triggerEvent: #scrolledToBottom]
!

pageKey: keyEvent
	keyEvent hasShift
		ifTrue: [self pageUpKey: keyEvent]
		ifFalse: [self pageDownKey: keyEvent]
!

pageUpKey: keyEvent

	self scrollByPages: -1
!

tabKey: keyEvent

	| isBackTab |
	isBackTab := keyboardProcessor isBackTab: keyEvent.
	isBackTab
		ifTrue: [self moveToPrevField]
		ifFalse: [self moveToNextField]
! !

!TfDocumentController methodsFor:'private'!

dispatchTable

	^ dispatchTable ifNil: [self initializeDispatchTable ].
!

documentWrapper

	^ self view tfDocumentWrapper
!

hasControl
	InputManager default focusView: view
!

initializeDispatchTable

	dispatchTable := DispatchTable new.
	dispatchTable defaultForCharacters: #ignoreInputKey:.
	dispatchTable defaultForNonCharacters: #ignoreInputKey:.
	dispatchTable bindValue: #cursorUpKey: to: #Up.
	dispatchTable bindValue: #cursorDownKey: to: #Down.
	dispatchTable bindValue: #cursorLeftKey: to: #Left.
	dispatchTable bindValue: #cursorRightKey: to: #Right.
	dispatchTable bindValue: #homeKey: to: #Home.
	dispatchTable bindValue: #endKey: to: #End.
	dispatchTable bindValue: #pageUpKey: to: #PageUp. 
	dispatchTable bindValue: #pageDownKey: to: #PageDown.
	dispatchTable bindValue: #pageKey: to: Character space.
	dispatchTable bindValue: #tabKey: to: Character tab.
	^dispatchTable
!

lineGrid

	^ self view styleContext textStyle lineGrid
!

moveToNextField

	keyboardProcessor == nil ifFalse: [keyboardProcessor nextFieldFrom: self]
!

moveToPrevField

	keyboardProcessor == nil ifFalse: [keyboardProcessor prevFieldFrom: self]
! !

!TfDocumentController methodsFor:'scrolling'!

scrollBy: aPoint

	^ self documentWrapper scrollBy: aPoint negated
!

scrollByLines: numLines

	^ self scrollBy: 0@(numLines * self lineGrid)
!

scrollByPages: numPages

	| amount |
	amount := self view layoutBox height - self lineGrid.
	^ self scrollBy: 0@(numPages * amount)
!

scrollTo: aPoint

	^ self documentWrapper scrollTo: aPoint negated
! !

!TfDocumentController class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDocumentController.st,v 1.1 2009/10/01 23:40:22 cg Exp $'
! !
