"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfResourceModel subclass:#TfDocumentModel
	instanceVariableNames:'htmlAspect documentAspect selectedText'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfDocumentModel class methodsFor:'instance creation'!

changeHTMLString: aString

	^ self new
		changeHTMLString: aString;
		yourself
!

htmlString: aString

	^ self new
		htmlString: aString;
		yourself
! !

!TfDocumentModel class methodsFor:'accessing'!

handledMimeTypes
	" patterns of all mime types that are handled by this class "

	^ #('text/html' 'text/htm' 'text/xhtml')
! !

!TfDocumentModel methodsFor:'accessing'!

changeHTMLString: aString

	| parser doc |
	self htmlAspect value: aString.
	parser := TfHTMLParser new.
	parser validate: false.
	doc := parser parse: self htmlString readStream.
	self document: doc
!

copyTextToClipboard

	| text |
	(text := self selectedText) notNil
		ifTrue: [ ParagraphEditor currentSelection: text ]
!

defaultTitle

	| t |
	^ (t := self document title) notNil
		ifTrue: [ t tfWithoutWhitespaces ]
		ifFalse: [ 'HTML' ].
!

description
	" description of the model. uses for bookmarks, history etc. "

	^ self document title
!

document

	^ self documentAspect value
!

document: aTfDocument

	self documentAspect value: aTfDocument.
	aTfDocument documentModel: self.
	aTfDocument parsingFinished.
	self resourceModelChanged.
!

documentAspect

	documentAspect isNil
		ifTrue: [ documentAspect := nil asValue ].
	^ documentAspect
!

documentModel

	^ self
!

htmlAspect

	htmlAspect isNil
		ifTrue: [ htmlAspect := String new asValue ].
	^ htmlAspect
!

htmlString

	^ self htmlAspect value
!

htmlString: aString

	self htmlAspect value: aString.
	self createDocument.
!

htmlStringNoUpdate: aString

	self htmlAspect value: aString.
!

inspectResource

	self document inspect
!

loadResourceFrom: aMimeEntity
	" convert the entity into a resource "

	| s |
	s := aMimeEntity tfUnencodedValue.
	self htmlString: s asByteString
!

loadResourceFromStream: aStream
	" convert the stream into a resource "

	self htmlString: aStream upToEnd.
!

redirection
	" returns an URL descriptor if the resource includes a redirection "

	| refresh newUrl resolvedUrl |
	refresh := self document metaInfos detect: [ : mi | mi httpEquiv = 'refresh' ] ifNone: [ nil ].
	^ (refresh notNil and: [ (newUrl := refresh urlString ) notNil ])
		ifTrue: [ 
			resolvedUrl := self url tfResolveURL: newUrl.
			TfURLDescriptor url: resolvedUrl delay: refresh seconds
		]
		ifFalse: [ nil ].
!

resource

	^ self document
!

selectedText
	" return currently selected text, or nil "

	^ selectedText
!

selectedText: aString
	" currently selected text, or nil "

	selectedText := aString
!

sidebarInspector

	^ (TfHTMLInspector on: self resource) -> #sidebarSpec
!

sourceText

	^ self htmlString
! !

!TfDocumentModel methodsFor:'private'!

createDocument

	| doc |
	doc := TfHTMLParser processDocumentBytes: self htmlString asString asByteArray.
	self document: doc
! !

!TfDocumentModel methodsFor:'views'!

createEmbeddedApp

	^ TfEmbeddedHTMLApp resourceModel: self.
!

createViewWithKeyboard: aKeyboardProcessorOrNil

	| docView |
	^self document notNil 
		ifTrue: [ 
			docView := self tfViewClass model: self.
			aKeyboardProcessorOrNil notNil
				ifTrue: [ docView controller keyboardProcessor: aKeyboardProcessorOrNil ].
			docView 
		]
		ifFalse: [ nil ]
!

tfComponents

	^ self document tfComponents
!

tfViewClass

	^ self document tfViewClass
! !

!TfDocumentModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDocumentModel.st,v 1.1 2009/10/01 18:04:40 cg Exp $'
! !
