"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfCompositeElementView subclass:#TfDocumentView
	instanceVariableNames:'graphicsMedium isRendered textSelection linkTargetPositions'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfDocumentView methodsFor:'accessing'!

linkTargetPositionsAt: anchorName 
	" positions of link targets <a name='target'/> "

	^ linkTargetPositions at: anchorName asLowercase ifAbsent: [ nil ].
!

linkTargetPositionsAt: anchorName put: position
	" positions of link targets <a name='target'/> "

	linkTargetPositions at: anchorName asLowercase put: position
!

listContainer

	^ nil
!

margin

	^ self tfDocumentWrapper margin
!

tfDocumentWrapper

	^ self container tfDocumentWrapper
!

tfTopDocumentView

	^ self
!

xmlNode

	^ self model document
! !

!TfDocumentView methodsFor:'bounds accessing'!

bounds: newBounds

	| oldBounds |
	oldBounds := self layoutBox.
	self layoutBox: (0@0 extent: newBounds extent).
	(oldBounds width ~= newBounds width and: [ newBounds width > 16])
		ifTrue: [ self updateDocument ]
		ifFalse: [ self flushCoordinateCaches ].
	super bounds: newBounds.
!

computePreferredBounds
	" updating the preferred bounds before rendering is finished results in wrong bounds "

	^ self isRendered 
		ifTrue: [ super computePreferredBounds ]
		ifFalse: [ Screen default bounds ]
!

layoutBox

	layoutBox isNil
		ifTrue: [ layoutBox := 0@0 extent: 0@0 ].
	^ layoutBox
!

layoutBox: aRectangle

	layoutBox := aRectangle
!

preferredBounds
	"Answer the merged bounds of the receiver's components"

	^ (self isRendered not and: [ preferredBounds == nil ])
		ifTrue: [ Screen default bounds ]
		ifFalse: [ super preferredBounds ]
!

updateDocument

	| alignedBounds renderContext maxCorner |
	self model isNil ifTrue: [ ^ self ].
	alignedBounds := 0@0 extent: self bounds extent.
	renderContext := self createRenderContext.
	renderContext layoutContext bounds: alignedBounds.
	self setupRendering: renderContext.
	maxCorner := self components inject: 0@0 into: [ : ext : comp | ext max: (comp bounds corner) ].
	preferredBounds := 0@0	corner: maxCorner.
	self flushCoordinateCaches.
	isRendered := true.
! !

!TfDocumentView methodsFor:'controller accessing'!

defaultControllerClass

	^TfDocumentController
! !

!TfDocumentView methodsFor:'displaying'!

displayOn: aGraphicsContext

	| p |
	self isRendered
		ifFalse: [ ^ self ].
	aGraphicsContext tfClientDataAt: #tfTextSelection put: textSelection.
	p := aGraphicsContext paint.
	aGraphicsContext paint: ColorValue white. "self backgroundColor."
	self bounds displayFilledOn: aGraphicsContext.
	aGraphicsContext paint: p.
	super displayOn: aGraphicsContext.
! !

!TfDocumentView methodsFor:'events'!

handlerForMouseEvent: event

	^ self isRendered
		ifTrue: [ super handlerForMouseEvent: event ]
		ifFalse: [ nil ]
! !

!TfDocumentView methodsFor:'initialize-release'!

initialize

	super initialize.
	isRendered := false.
	linkTargetPositions := Dictionary new.
! !

!TfDocumentView methodsFor:'private'!

isRendered

 	^ isRendered == true
! !

!TfDocumentView methodsFor:'rendering'!

createRenderContext

	| renderContext docModel |
	renderContext := self renderContextClass new.
	docModel := self model.
	renderContext documentModel: docModel.
	renderContext formContext formBuilder: self model browserWindowModel formBuilder.
	renderContext graphicsDevice: self graphicsDevice.
	renderContext styleContext styleSheet: docModel browserWindowModel browserModel styleSheet.
	^ renderContext
!

finishOwnRendering: renderContext

	renderContext layoutContext forceNextLine.
!

graphicsMedium

	^ graphicsMedium notNil
		ifTrue: [ graphicsMedium ]
		ifFalse: [ Screen default ]
!

graphicsMedium: aGraphicsMedium

	graphicsMedium := aGraphicsMedium
!

renderContextClass

	^ TfRenderContext
!

setupComponentRendering: renderContext
	" prepare the view for rendering "

	self components isEmpty
		ifTrue: [ 	self tfSetupComponents ].
	super setupComponentRendering: renderContext.
!

setupRendering: renderContext

	super setupRendering: renderContext.
	self tfRenderingFinished.
!

startOwnRendering: renderContext

	| box |
	styleContext := renderContext styleContext.
	box := self layoutBox insetBy: self margin.
	renderContext layoutContext
		bounds: box;
		position: box origin.
!

styleContext

	^ styleContext
! !

!TfDocumentView methodsFor:'selection'!

clearTextSelection

	self updateTextSelectionAt: textSelection corner.
	self textSelection: nil.
	self model documentModel selectedText: nil.
!

finalizeTextSelection

	| selectedText |
	textSelection startTextStream.
	self updateTextSelectionAt: textSelection corner.
	selectedText := textSelection text.
	textSelection releaseTextStream.
	self model documentModel selectedText: selectedText.
!

selectAll

	" ugly "

	textSelection := TfTextSelection origin: self bounds origin corner: self bounds corner.
	self finalizeTextSelection.
	self finalizeTextSelection.
!

selectWordAt: aPoint for: aTfView

	textSelection := TfTextSelection view: aTfView at: aPoint extent: 1@1.
	textSelection selectionMode: #word.
	self finalizeTextSelection.
!

textSelection

	^ textSelection
!

textSelection: aTfTextSelection

	textSelection := aTfTextSelection.
!

updateTextSelectionAt: aPoint

	| mergedSelection oldSelection docBounds |
	oldSelection := textSelection tfNormalized.
	self textSelection: textSelection.
	self 
		downcastLocalEvent: #tfTrackTextSelection
		with: textSelection
		at: aPoint
		from: self controller.
	self 
		downcastEvent: #tfUpdateTextSelection
		with: textSelection
		from: self.
	" update view "
	mergedSelection := oldSelection merge: textSelection tfNormalized.
	docBounds := self bounds.
	mergedSelection 
		left: docBounds left;
		right: docBounds right.
	self invalidateRectangle: docBounds repairNow: false.
	self tfDocumentWrapper scrollIfNeededTo: aPoint negated
! !

!TfDocumentView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDocumentView.st,v 1.1 2009/10/01 19:38:03 cg Exp $'
! !
