"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Wrapper subclass:#TfDocumentWrapper
	instanceVariableNames:'scrollOffsetHolder resourceWindowModel'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfDocumentWrapper class methodsFor:'instance creation'!

on: documentViewView resourceWindowModel: aTfResourceWindowModel

	^ (self resourceWindowModel: aTfResourceWindowModel)
		setComponent: documentViewView;
		yourself
!

resourceWindowModel: aTfResourceWindowModel

	^ self new
		resourceWindowModel: aTfResourceWindowModel;
		yourself
! !

!TfDocumentWrapper methodsFor:'accessing'!

flushRenderingCaches: needsMeasuring

	self component flushRenderingCaches: needsMeasuring
!

resourceWindowModel

	^ resourceWindowModel
!

resourceWindowModel: aTfResourceWindowModel

	resourceWindowModel := aTfResourceWindowModel
!

scrollOffsetHolder

	^ scrollOffsetHolder
!

scrollOffsetHolder: aValueHolder

	scrollOffsetHolder := aValueHolder.
!

tfDocumentWrapper

	^ self
! !

!TfDocumentWrapper methodsFor:'broadcast'!

upcastEvent: aKey with: aParameter from: anInitiator path: aPathCollection

	aKey == #mouseWheel
		ifTrue: [ ^ self mouseWheelScroll: aParameter ].
	aKey = #tfNeedsRendering
		ifTrue: [ ^ self recompose ].
	super upcastEvent: aKey with: aParameter from: anInitiator path: aPathCollection.
! !

!TfDocumentWrapper methodsFor:'private'!

composeAndInvalidate

	self isOpen
		ifTrue: [
			self bounds: self bounds.
			self scrollOffsetHolder notNil
				ifTrue: [ self scrollOffsetHolder valueWithoutDisplaying: 0@0 ].
			self invalidate
		]
!

documentView

	^ self component
!

margin

	^ 3
!

mouseWheelScroll: scrollUp

	| amount |
	amount := scrollUp ifTrue: [ 1 ] ifFalse: [ -1 ].
	amount := amount * self component styleContext textStyle lineGrid * 3.
	self scrollBy: (0@amount).
!

recompose

	self isOpen
		ifTrue: [
			self flushRenderingCaches: true.
			self component updateDocument.
			self invalidateRectangle: self bounds.
			self topComponent sensor noteEvent.
		]
! !

!TfDocumentWrapper methodsFor:'scrolling'!

changePositionBy: aPoint
	"Translate the view without affecting the display."

	self positionTo: self scrollOffsetHolder value + aPoint.
!

positionTo: aPoint
	"Position the view to a particular point without affecting the display."

	self scrollOffsetHolder valueWithoutDisplaying: (self validateScrollPosition: aPoint)
!

scrollBy: aPoint
	"Translate the view and update the display."

	^self scrollTo: self scrollOffsetHolder value + aPoint.
!

scrollIfNeededTo: aPoint
	" only scroll if outside visible part "

	| box |
	box := self scrollOffsetHolder value negated extent: self visibleExtent.
	(box containsPoint: aPoint negated)
		ifFalse: [ 
			aPoint y negated < box top
				ifTrue: [ self scrollTo: aPoint ]
				ifFalse: [ self scrollTo: aPoint + (0@box height) ]
		]
!

scrollPositionForTarget: targetIDString

	| pos |
	pos := self documentView linkTargetPositionsAt: targetIDString.
	^ pos notNil
		ifTrue: [ pos negated ]
		ifFalse: [ 0@0 ].
!

scrollTo: aPoint
	"Position the view to a particular place, updating the display.
	 Make sure pending invalidation is updated before scrolling."

	| p1 p comp |
	p := self scrollOffsetHolder value.
	(p x  = aPoint x and: [p y = aPoint y]) ifTrue: [^self].
	p1 := self validateScrollPosition: aPoint.
	(p x  = p1 x and: [p y = p1 y]) ifFalse:
		[(comp := self topComponent) notNil ifTrue:
			[comp displayPendingInvalidation].
		self scrollOffsetHolder value: p1.
		^true].
	^false
!

scrollToEntry: aTfEntry

	self scrollToTargetID: aTfEntry url tfFragment
!

scrollToTargetID: targetIDString

	targetIDString isNil ifTrue: [^ self ].
	self scrollTo: (self scrollPositionForTarget: targetIDString)
! !

!TfDocumentWrapper methodsFor:'scrolling - private'!

scrollableExtent
	"Answer the extent of the receiver's display object bounding box."

	^(self preferredBounds expandedBy: self scrollOffsetHolder extraSpace) extent max: (1@1)
!

validateScrollPosition: aPoint
	"Make sure that this is a legal scroll offset for us to occupy"

	| p |
	p := aPoint negated min: self scrollableExtent - self visibleExtent.
	p := p max: self preferredBounds origin - self scrollOffsetHolder extraSpace origin.
	p := p grid: self scrollOffsetHolder grid.
	^ p negated
!

visibleExtent
	"Answer the extent of the receiver's clipping box."

	^(self bounds extent truncateTo: self scrollOffsetHolder grid) max: self scrollOffsetHolder grid
! !

!TfDocumentWrapper class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDocumentWrapper.st,v 1.1 2009/10/01 19:38:22 cg Exp $'
! !
