"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

XML::Element subclass:#TfElement
	instanceVariableNames:'dependents'
	classVariableNames:'RegisteredElements'
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfElement class methodsFor:'instance creation'!

bestMatchTag: tag attributes: attributes elements: elements

	^ (self elementClassFor: tag) new setTag: tag attributes: attributes elements: elements
! !

!TfElement class methodsFor:'accessing'!

allowedChildElements
	" poor man's DTD. return all allowed child elements, or nil if any element is allowed "

	^ nil
!

allowsPCData
	" poor man's DTD. return if PCData is allowed as child node "

	^ true
!

defaultTagName

	^ self tagNames detect: [ : x | true ] ifNone: [ nil ]
!

elementClassFor: aTag
	"TfElement elementClassFor: 'b'"
	"TfElement elementClassFor: 'undefined'"

	^ self elementClassFor: aTag ifAbsent: [ TfUnknownElement ].
!

elementClassFor: aTag ifAbsent: absentBlock
	"TfElement elementClassFor: 'b'"
	"TfElement elementClassFor: 'undefined'"

	^ self registeredElements at: aTag asString asLowercase ifAbsent: absentBlock
!

elementsToCloseClasses
	" returns the classes of all elements that must be closed before
	opening elements of me "

	^ #()
!

elementsToCloseStopClasses

	^ #()
!

flushRegisteredElements

	RegisteredElements := nil
!

implicitCloseStopClasses
	" quite often, tags are not properly closed (even if the DTD does not allow that), e.g. <tr><td>....</tr> (missing </td>.
	by default the parser will implicitly close all opened tags. however, this is sometimes not desired.
	Return all tag classes that should not be implicitly closed when closing an element of this cass "

	^ #()
!

registeredElements
	"RegisteredElements := nil"

	RegisteredElements isNil
		ifTrue: [ self registerAllElements ].
	^ RegisteredElements
!

remarks

	^ ''
!

supportedAttributes

	^ #()
!

tagNames
	" returns the full tag names (including namespace) of all tags that
	are mapped to this element "

	^ #()
! !

!TfElement class methodsFor:'private'!

blockNames

	| blockNames |
	blockNames :=  'P | DL | DIV | CENTER |
      NOSCRIPT | NOFRAMES | BLOCKQUOTE | FORM | ISINDEX | HR |
      TABLE | FIELDSET | ADDRESS | PRE' tfAsStringsSeperatedByAll: '| \' withCRs.
	blockNames addAll: self listNames.
	blockNames addAll: self headingNames.
	^ blockNames
!

flowNames

	| flowNames |
	flowNames :=  Set new.
	flowNames addAll: self blockNames.
	flowNames addAll: self inlineNames.
	^ flowNames
!

fontstyleNames

	^ 'TT | I | B | U | S | STRIKE | BIG | SMALL' tfAsStringsSeperatedByAll: '| \' withCRs
!

formctrlNames

	^ 'INPUT | SELECT | TEXTAREA | LABEL | BUTTON' tfAsStringsSeperatedByAll: '| \' withCRs
!

headingNames

	^ 'H1|H2|H3|H4|H5|H6' tfAsStringsSeperatedByAll: '| \' withCRs
!

headmiscNames

	^ 'SCRIPT|STYLE|META|LINK|OBJECT' tfAsStringsSeperatedByAll: '| \' withCRs
!

inlineNames

	| inlineNames |
	inlineNames := Set new.
	inlineNames addAll: self fontstyleNames.
	inlineNames addAll: self phraseNames.
	inlineNames addAll: self specialNames.
	inlineNames addAll: self formctrlNames.
	^ inlineNames
!

listNames

	^ 'UL | OL |  DIR | MENU' tfAsStringsSeperatedByAll: '| \' withCRs
!

phraseNames

	^ 'EM | STRONG | DFN | CODE | SAMP | KBD | VAR | CITE | ABBR | ACRONYM' tfAsStringsSeperatedByAll: '| \' withCRs
!

registerAllElements
	"TfElement registerAllElements"

	| ocl |
	RegisteredElements := Dictionary new.
	self allSubclasses do: [:cl |
		cl tagNames do: [ :nm |
			(ocl := RegisteredElements at: nm asLowercase ifAbsent: [nil]) notNil
				ifTrue: [ self error: 'Tag ', nm, ' already mapped to ', ocl fullName ].
			RegisteredElements at: nm asLowercase put: cl
		]
	].
	TfDefaultElement registerDefaultElements.
	^ RegisteredElements
!

specialNames

	^ 'A | IMG | APPLET | OBJECT | FONT | BASEFONT | BR | SCRIPT | MAP | Q | SUB | SUP | SPAN | BDO | IFRAME' tfAsStringsSeperatedByAll: '| \' withCRs
! !

!TfElement class methodsFor:'resources'!

defaultBrowserMenu
	"Tools.MenuEditor new openOnClass: self andSelector: #defaultBrowserMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem
				#rawLabel: 'Copy text'
				#value: #copyTextToClipboard
				#labelImage: #(#ResourceRetriever #ToolbarIconLibrary #copy ) )
			#(#MenuItem
				#rawLabel: 'Export table'
				#value: #exportTable
				#enablementSelector: #isInTable
				#labelImage: #(#ResourceRetriever #ToolbarIconLibrary #tableViewWidget ) ) ) #(2 ) nil ) decodeAsLiteralArray
!

widgetBrowserMenu
	"Tools.MenuEditor new openOnClass: self andSelector: #widgetBrowserMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem
				#rawLabel: 'Copy text'
				#value: #copyTextToClipboard
				#labelImage: #(#ResourceRetriever #ToolbarIconLibrary #copy ) )
			#(#MenuItem
				#rawLabel: 'Export table'
				#value: #exportTable
				#enablementSelector: #isInTable
				#labelImage: #(#ResourceRetriever #ToolbarIconLibrary #tableViewWidget ) ) ) #(2 ) nil ) decodeAsLiteralArray
! !

!TfElement methodsFor:'DOM fixing'!

ensureChildren: childCondition default: defaultChildTag

	| newParent |
	self elements copy do: [ : node |
		node isElement
			ifTrue: [
				(childCondition value: node)
					ifFalse: [
						newParent := (TfElement elementClassFor: defaultChildTag)
										tag: defaultChildTag
										attributes: Array new
										elements: nil.
						node insertParent: newParent.
					].
			].
	].
!

ensureParent: parentCondition default: defaultParentTag

	| newParent |
	(parentCondition value: self parent)
		ifFalse: [
			newParent := TfElement
							bestMatchTag: defaultParentTag
							attributes: Array new
							elements: nil.
			self insertParent: newParent.
			newParent tfFixNodes.
		].
!

tfFixNodes

	super tfFixNodes.
	self elements copy do: [ : t | t tfFixNodes ].
! !

!TfElement methodsFor:'accessing'!

addedToDocument: aTfDocument
	" this element has been added to the document "
!

attributeNamed: attributeName

	^ self attributeNamed: attributeName ifAbsent: [nil]
!

attributeNamed: attributeName ifAbsent: aBlock

	^ self attributes
		detect: [:a | a tag isLike: attributeName]
		ifNone: aBlock
!

booleanValueOfAttribute: attributeName
	" returns the boolean value of the given attribute, or absentBlock "

	^ self booleanValueOfAttribute: attributeName ifAbsent: nil
!

booleanValueOfAttribute: attributeName ifAbsent: absentBlock
	" returns the boolean value of the given attribute, or absentBlock "

	| attr |
	^ (attr := self attributeNamed: attributeName) notNil
		ifTrue: [ attr value value asLowercase = 'true' ]
		ifFalse: [ absentBlock value ].
!

documentModel

	^ self document documentModel
!

insertParent: newParentElement

	| oldParent |
	oldParent := self parent.
	newParentElement parent: oldParent.
	oldParent elements replaceAll: self with: newParentElement.
	newParentElement elements: (Array with: self).
!

integerValueOfAttribute: attributeName
	" returns the integer value of the given attribute, or nil "

	^ self integerValueOfAttribute: attributeName ifAbsent: [ nil ]
!

integerValueOfAttribute: attributeName ifAbsent: absentBlock
	" returns the integer value of the given attribute, or absentBlock "

	| attr |
	^ (attr := self attributeNamed: attributeName) notNil
		ifTrue: [ self valueReader parseInteger: attr value ]
		ifFalse: [ absentBlock value ].
!

isAttributeSet: attributeName

	^ (self valueOfAttribute: attributeName) = attributeName
!

isInTable

	^ self tfTableElement notNil
!

listElement

	^ self parent listElement
!

parsingFinished
	" this element has been parsed "

	self parseValues.
!

remove

	self parent removeNode: self.
	self changed: #removed
!

replaceElement: childElement withElements: newChildrenElements

	self elements: (self elements copyReplaceAll: (Array with: childElement) with: newChildrenElements).
!

replaceWithElements

	self parent replaceElement: self withElements: self elements
!

symbolValueOfAttribute: attributeName

	^ self symbolValueOfAttribute: attributeName ifAbsent: [ nil ]
!

symbolValueOfAttribute: attributeName ifAbsent: absentBlock

	| val |
	val := self valueOfAttribute: attributeName ifAbsent: [ nil ].
	^val notNil
		ifTrue: [ val asLowercase asSymbol ]
		ifFalse: [ absentBlock value ].
!

tfAdaptToTable: aTfTableEnumerator
	" table elemenets have been enumerated, now make sure that stored information is valid"

	super tfAdaptToTable: aTfTableEnumerator.
	self elements do: [ : node |
		node tfAdaptToTable: aTfTableEnumerator
	]
!

tfAddNodesMatching: matchBlock upTo: stopBlock to: aCollection

	(matchBlock value: self)
		ifTrue: [ aCollection add: self ].
	self elements do: [ : node |
		(stopBlock value: node )
			ifFalse: [ node tfAddNodesMatching: matchBlock upTo: stopBlock to: aCollection ].
	].
!

tfEnumerateTable: aTfTableEnumerator
	" compute row/column indexes of cells/rows "

	super tfEnumerateTable: aTfTableEnumerator.
	self elements do: [ : node |
		node tfEnumerateTable: aTfTableEnumerator
	].
!

tfSelfOrPreviousElementNamed: elementName ifAbsent: aBlock

	^  (self tag isLike: elementName)
		ifTrue: [ self ]
		ifFalse: [ super tfSelfOrPreviousElementNamed: elementName ifAbsent: aBlock ]
!

valueOfAttribute: attributeName

	^ self valueOfAttribute: attributeName ifAbsent: [ nil ]
!

valueOfAttribute: attributeName do: aBlock
	" performs the block with the value of the given attribute "

	| val |
	val := self valueOfAttribute: attributeName.
	val notNil
		ifTrue: [ aBlock value: val ].
! !

!TfElement methodsFor:'alignment'!

alignCenter: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: 0.5 alignVertical: nil paragraphAlignment: #center
!

alignJustify: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: 0.5 alignVertical: nil paragraphAlignment: #justify
!

alignLeft: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: 0 alignVertical: nil paragraphAlignment: #left
!

alignRight: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: 1 alignVertical: nil paragraphAlignment: #right
!

applyAlignmentStyleTo: aTfStyle alignHorizontal: alignHorizontalNum alignVertical: alignVerticalNum paragraphAlignment: paragraphAlignmentSymbol

	| layout |
	layout := aTfStyle tableCellStyle alignmentOrigin.
	alignHorizontalNum notNil
		ifTrue: [ layout leftFraction: alignHorizontalNum; leftAlignmentFraction: alignHorizontalNum ].
	alignVerticalNum notNil
		ifTrue: [ layout topFraction: alignVerticalNum; topAlignmentFraction: alignVerticalNum ].
	(alignHorizontalNum notNil or: [ alignVerticalNum notNil ])
		ifTrue: [ aTfStyle tableCellStyle alignmentOrigin: layout ].
	paragraphAlignmentSymbol notNil
		ifTrue: [ aTfStyle paragraphStyle alignment: paragraphAlignmentSymbol ].
!

applyHAlignmentTo: aTfStyle

	| align |
	self valueOfAttribute: 'align' do: [ :val |
		align := val asLowercase.
		align = 'left'
			ifTrue: [ ^ self alignLeft: aTfStyle ].
		align = 'right'
			ifTrue: [ ^ self alignRight: aTfStyle ].
		align = 'center'
			ifTrue: [ ^ self alignCenter: aTfStyle ].
		align = 'justify'
			ifTrue: [ ^ self alignJustify: aTfStyle ].
	].
!

applyVAlignmentTo: aTfStyle

	| align |
	self valueOfAttribute: 'valign' do: [ :val |
		align := val asLowercase.
		align = 'top'
			ifTrue: [ ^ self vAlignTop: aTfStyle ].
		align = 'bottom'
			ifTrue: [ ^ self vAlignBottom: aTfStyle ].
		align = 'middle'
			ifTrue: [ ^ self vAlignMiddle: aTfStyle ].
	].
!

vAlignBottom: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: nil alignVertical: 1 paragraphAlignment: nil
!

vAlignMiddle: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: nil alignVertical: 0.5 paragraphAlignment: nil
!

vAlignTop: aTfStyle

	self applyAlignmentStyleTo: aTfStyle alignHorizontal: nil alignVertical: 0 paragraphAlignment: nil
! !

!TfElement methodsFor:'bookmarks'!

createBookmarks: description
	" this DOM represents Netscape bookmarks "

	| bookmarkContainer |
	bookmarkContainer := TfBookmarkContainer description: description.
	self elements
		do: [:node | node tfAddToBookmarks: bookmarkContainer emit: false].
	^bookmarkContainer
!

tfAddToBookmarks: aTfBookmarkContainer emit: aBoolean

	| bookmarkContainer |
	bookmarkContainer := aTfBookmarkContainer.
	self elements do: [:node |
		bookmarkContainer := node tfAddToBookmarks: bookmarkContainer emit: aBoolean
	].
	^ aTfBookmarkContainer
! !

!TfElement methodsFor:'copying'!

postCopy
	"Do not copy the dependents list."

	super postCopy.
	self breakDependents
! !

!TfElement methodsFor:'inspecting'!

inspectorClasses

	^ super inspectorClasses
		copyWith: TfHTMLInspector
! !

!TfElement methodsFor:'menu actions'!

copyTextToClipboard

	ParagraphEditor currentSelection: self characterData.
	self documentModel copyTextToClipboard
!

exportTable

	| te |
	(te := self tfTableElement) notNil
		ifTrue: [ te exportTable ].
!

exportTableOn: aStream columns: numberOfColumns emit: aBoolean

	self elements do: [ : el |
		el exportTableOn: aStream columns: numberOfColumns  emit: aBoolean
	].
! !

!TfElement methodsFor:'menus'!

defaultBrowserMenu

	^ self class defaultBrowserMenu
!

widgetBrowserMenu

	^self class widgetBrowserMenu
! !

!TfElement methodsFor:'parsing'!

parseValues
! !

!TfElement methodsFor:'printing'!

printOn: aStream

	self tfPrintOn: aStream emphasicDictionary: Dictionary new
!

shortPrintString

	| s |
	s := self printString.
	s replaceAll: Character cr with: Character space.
	s replaceAll: Character tab with: Character space.
	s size > 100
		ifTrue: [ s := s copyFrom: 1 to: 100 ].
	^ s
!

tfPrintOn: aTextStream depth: indent emphasicDictionary: emphasicDictionary

    | elem emph |
    emph := aTextStream emphasis.
    aTextStream emphasis: #color -> ColorValue blue.
    self saxDo: (TfSAXWriter new output: aTextStream)
        forBodyDo:[
            aTextStream emphasis: emph.
            elements == nil
                ifFalse: [
                    elem := elements reject: [:str | str isText and: [str isStripped]].
                    (elem size <= 1 and: [(elem contains: [:n | n isText not]) not])
                        ifTrue: [elem do: [:e |
                                    e tfPrintOn: aTextStream depth: indent+1 emphasicDictionary: emphasicDictionary]]
                        ifFalse:
                            [1 to: elem size do: [:i | | e |
                                e := elem at: i.
                                "aTextStream tab: indent+1."
                                e isString 
                                    ifTrue: [aTextStream nextPutAll: e]
                                    ifFalse: [e tfPrintOn: aTextStream depth: indent+1 emphasicDictionary: emphasicDictionary]].
                            "aTextStream crtab: indent" ]
                ].
                aTextStream emphasis: #color -> ColorValue blue.
            ].
    aTextStream emphasis: emph.
!

tfPrintOn: aTextStream emphasicDictionary: emphasicDictionary

	self tfPrintOn: aTextStream depth: 0 emphasicDictionary: emphasicDictionary
!

tfTreeDisplayString

	^ super tfTreeDisplayString asText emphasizeAllWith: #color -> ColorValue blue
! !

!TfElement methodsFor:'private'!

myDependents
	"Answer the receiver's dependents or nil."

	^dependents
!

myDependents: dependentsOrNil
	"Set the receivers dependents."

	dependents := dependentsOrNil
!

tagName
	" Returns the tagname used for lookups "

	^ self tag asString asLowercase
!

valueReader

	^ TfHTMLValueReader singleton
! !

!TfElement methodsFor:'styles'!

applyAlignmentStyleTo: aTfStyle

	self applyHAlignmentTo: aTfStyle.
	self applyVAlignmentTo: aTfStyle.
!

applyCSSStyleTo: aTfStyle
	" parse the CSS style attribute "

	| styleValues |
	(styleValues := self cssStyleValues) notNil
		ifTrue: [
			self styleReader parseCSSValues: styleValues for: aTfStyle
		].
!

applyStyleSheet: aTfStyleSheet to: aTfStyle

	| elementStyle |
	elementStyle := aTfStyleSheet styleAt: self tagName.
	elementStyle notNil
		ifTrue: [ aTfStyle setupStyleFrom: elementStyle inherited: false ].
!

canHaveCSSStyle
	" true if this element may contain a CSS style attribute
	(that has to be parsed) "

	^ true
!

cssStyleValues
	" unparsed values of the CSS style attribute "
	" ^ String "

	^ self valueOfAttribute: 'style'
!

modifyStyle: aTfStyle
	" modify the style according to the local style definitions "

	self canHaveCSSStyle
		ifTrue: [ self applyCSSStyleTo: aTfStyle ]
!

modifyStyle: aTfStyle using: aTfStyleSheet
	" modify the style according to the local style definitions "

	self applyStyleSheet: aTfStyleSheet to: aTfStyle.
	self modifyStyle: aTfStyle.
!

styleReader

	^ TfCSSStyleReader singleton
! !

!TfElement methodsFor:'universal attributes'!

cssClass

	^ self valueOfAttribute: 'class'.
!

id

	^ self valueOfAttribute: 'id'.
!

title

	^ self valueOfAttribute: 'title'.
! !

!TfElement methodsFor:'views'!

tfComponents

	^ self elements
!

tfViewClass
	" returns the default composite view class for this element "

	^ TfCompositeElementView
! !

!TfElement class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfElement.st,v 1.2 2009/11/07 03:21:08 cg Exp $'
! !
