"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfStylePart subclass:#TfFontStyle
	instanceVariableNames:'enabledStyles disabledStyles'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfFontStyle class methodsFor:'instance creation'!

new

	^ super new initialize
! !

!TfFontStyle methodsFor:'accessing'!

applyPixelSizeLayout

	| ps |
	(self pixelSizeLayout notNil and: [ (ps := self pixelSize) notNil ])
		ifTrue: [ 
			ps := self pixelSizeLayout sizeRelativeTo: ps.
			ps := (ps rounded) max: 12.
			self propertyAt: #pixelSize putValue: ps.
		].
!

color

	^ self propertyValueAt: #color
!

color: aColorValue

	self propertyAt: #color putValue: aColorValue
!

emphases

	| emph |
	emph := OrderedCollection new.
	self isBold
		ifTrue: [ emph add: #bold ].
	self isItalic
		ifTrue: [ emph add: #italic ].
	self color notNil
		ifTrue: [ emph add: #color -> self color ].
	self isUnderlined
		ifTrue: [ emph add: #underline ].
	^ emph asArray
!

encoding

	^ self propertyValueAt: #encoding ifAbsent: [ nil ]
!

encoding: aSymbol

	self propertyAt: #encoding putValue: aSymbol
!

family

	^ self propertyValueAt: #family
!

family: anArray

	self propertyAt: #family putValue: anArray
!

isBold

	^ self propertyValueAt: #isBold ifAbsent: [ false ]
!

isBold: aBoolean

	self propertyAt: #isBold putValue: aBoolean
!

isItalic

	^ self propertyValueAt: #isItalic ifAbsent: [ false ]
!

isItalic: aBoolean

	self propertyAt: #isItalic putValue: aBoolean
!

isStrikedOut

	^ self propertyValueAt: #isStrikedOut ifAbsent: [ false ]
!

isStrikedOut: aBoolean

	self propertyAt: #isStrikedOut putValue: aBoolean
!

isUnderlined

	^ self propertyValueAt: #isUnderlined ifAbsent: [ false ]
!

isUnderlined: aBoolean

	self propertyAt: #isUnderlined putValue: aBoolean
!

manufacturer

	^ self propertyValueAt: #manufacturer
!

manufacturer: anArray

	self propertyAt: #manufacturer putValue: anArray
!

modifyStylePart: aTfFontStyle isInherited: isInherited

	super modifyStylePart: aTfFontStyle isInherited: isInherited.
	self disabledStyles do: [ :styleSymbol |
		aTfFontStyle enabledStyles remove: styleSymbol ifAbsent: [].
		aTfFontStyle disabledStyles add: styleSymbol.
	].
	self enabledStyles do: [ :styleSymbol |
		" disabled styles cannot be enabled "
		(aTfFontStyle disabledStyles includes: styleSymbol)
			ifFalse: [ aTfFontStyle enabledStyles add: styleSymbol ].
	].
!

pixelSize

	^ self propertyValueAt: #pixelSize.
!

pixelSize: anInteger

	self propertyAt: #pixelSize putValue: anInteger.
	self applyPixelSizeLayout
!

pixelSizeLayout

	^ self propertyValueAt: #pixelSizeLayout
!

pixelSizeLayout: aTfSizeLayout

	self propertyAt: #pixelSizeLayout putLocalValue: aTfSizeLayout.
	self applyPixelSizeLayout
! !

!TfFontStyle methodsFor:'copying'!

beClone

	super beClone.
	enabledStyles notNil ifTrue: [ enabledStyles := enabledStyles copy ].
	disabledStyles notNil ifTrue: [ disabledStyles := disabledStyles copy ].
! !

!TfFontStyle methodsFor:'instance creation'!

createFontDescription

        | fontDescription encoding encoder |
        fontDescription := FontDescription new.
        self color notNil ifTrue: [ fontDescription color: self color ].
        self family notNil ifTrue: [ fontDescription family: self family ].
        self manufacturer notNil ifTrue: [ fontDescription manufacturer: self manufacturer ].
        self pixelSize notNil 
                ifTrue: [ 
                        fontDescription pixelSize: self pixelSize rounded
                ].
        self isBold ifTrue: [ fontDescription boldness: 0.7 ].
        self isItalic ifTrue: [ fontDescription italic: true ].
        self isUnderlined ifTrue: [ fontDescription underline: true ].
        self isStrikedOut ifTrue: [ fontDescription strikeout: true ].
"/        encoding := self encoding.
"/        encoding := (encoding notNil and: [ (encoder := StreamEncoder encoderDirectory at: encoding asLowercase asSymbol ifAbsent: [ nil ]) notNil ])
"/                ifTrue: [ encoder name ]
"/                ifFalse: [ Locale current defaultStreamEncoder name ].
"/        fontDescription encoding: encoding.
        ^ fontDescription
!

createTextStyleFor: aFontPolicy
	" create a text style based on my information "

	| cStyle textAttributes deviceFont fontDescription |
	fontDescription := self createFontDescription.
	cStyle := CharacterAttributes new.
	cStyle setDefaultQuery: fontDescription.
	cStyle tfSetDefaultAttributes.
	textAttributes := TextAttributes new.
	textAttributes
		setCharacterAttributes: cStyle;
		useFourSpaceTabWidth.
	deviceFont := aFontPolicy findFont: fontDescription.
	textAttributes
		lineGrid: deviceFont height;
		baseline: deviceFont ascent.
	^ textAttributes
! !

!TfFontStyle methodsFor:'private'!

disabledStyles

	disabledStyles isNil
		ifTrue: [disabledStyles := Set new ].
	^ disabledStyles
!

enabledStyles

	enabledStyles isNil
		ifTrue: [enabledStyles := Set new ].
	^ enabledStyles
!

style: styleSymbol isEnabled: aBooleanOrNil
	" if aBooleanOrNil is nil, then the sender does not care about the style "

	aBooleanOrNil notNil
		ifTrue: [
			aBooleanOrNil
				ifTrue: [
					self enabledStyles add: styleSymbol.
					self disabledStyles remove: styleSymbol ifAbsent: [].
				] ifFalse: [
					self enabledStyles remove: styleSymbol ifAbsent: [].
					self disabledStyles add: styleSymbol.
				].
		] ifFalse:[
			self enabledStyles remove: styleSymbol ifAbsent: [].
			self disabledStyles remove: styleSymbol ifAbsent: [].
		]
! !

!TfFontStyle class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfFontStyle.st,v 1.2 2009/10/01 23:37:23 cg Exp $'
! !
