"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfHTMLReviser subclass:#TfHTMLCloseTagReviser
	instanceVariableNames:'tagDictionary stopDictionary childDictionary'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower'
!


!TfHTMLCloseTagReviser methodsFor:'api'!

reviseOpenTag: aTfNodeTag parser: htmlParser 
	 " make sure that elements are properly closed before opening a new tag "

	| tagsToClose stopTags el allowedChildren |
	(el := htmlParser currentElement) notNil 
		ifTrue: [
			allowedChildren := self childDictionary at: (TfElement elementClassFor: el tag) ifAbsent: [ nil ].
			(allowedChildren notNil and: [ (allowedChildren includes: aTfNodeTag) not ])
				ifTrue: [ htmlParser endTag: htmlParser currentElement tag].
		].
	tagsToClose := self tagDictionary at: aTfNodeTag ifAbsent: [ ^ self ].
	stopTags := self stopDictionary at: aTfNodeTag ifAbsent: [ #() ].
	tagsToClose do: [ :tagToClose |
		htmlParser openedTagsUpTo: tagToClose stopAt: stopTags do: [ :tag |
			htmlParser endTag: tag
		].
	].
! !

!TfHTMLCloseTagReviser methodsFor:'initialize-release'!

initialize

	| cs |
	super initialize.
	TfElement withAllSubclasses do: [ : elementClass | | closeElementClasses |
		closeElementClasses := elementClass elementsToCloseClasses.
		closeElementClasses isEmpty
			ifFalse:[
				self
					beforeOpening: elementClass
					closeElements: closeElementClasses
					stopAt: elementClass elementsToCloseStopClasses.
			].
		(cs := elementClass allowedChildElements) notNil
			ifTrue: [ 
				cs := cs asSet collect: [ : childTag | TfNodeTag tagName: childTag ].
				self childDictionary at: elementClass put: cs 
			].
	].
! !

!TfHTMLCloseTagReviser methodsFor:'private'!

beforeOpening: openElementClass closeElements: closeElementClasses stopAt: stopElementClasses
	" the reviser will make sure that elements of all closeElementClasses
	will be closed before starting elements of openElementClass "

	openElementClass tagNames do: [ : openName | | openTag closeTags allCloseTags stopTags allStopTags  |
		openTag := TfNodeTag tagName: openName.
		allCloseTags := self tagDictionary at: openTag ifAbsentPut: [ Set new ].
		closeElementClasses do: [:closeElementClass |
			closeTags := closeElementClass tagNames
						collect: [ :closeName | TfNodeTag tagName: closeName ].
			allCloseTags addAll: closeTags.
		].
		allStopTags := self stopDictionary at: openTag ifAbsentPut: [ Set new ].
		stopElementClasses do: [ :stopElementClass |
			stopTags := stopElementClass tagNames
							collect: [ :stopName | TfNodeTag tagName: stopName ].
			allStopTags addAll: stopTags.
		].
	].
!

childDictionary

	childDictionary isNil
		ifTrue: [ childDictionary := Dictionary new ].
	^ childDictionary
!

stopDictionary

	stopDictionary isNil
		ifTrue: [ stopDictionary := Dictionary new ].
	^ stopDictionary
!

tagDictionary

	tagDictionary isNil
		ifTrue: [ tagDictionary := Dictionary new ].
	^ tagDictionary
! !

!TfHTMLCloseTagReviser class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfHTMLCloseTagReviser.st,v 1.1 2009/10/01 18:00:05 cg Exp $'
! !
