"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

XML::XMLParser subclass:#TfHTMLParser
	instanceVariableNames:'revisers encoder implicitStopTags buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfHTMLParser class methodsFor:'instance creation'!

on:aStream 
    |parser saxDriver saxBuilder|

    Smalltalk isSmalltalkX ifTrue:[
        parser := self new on: aStream.
        saxDriver := TfSAXDriver new.
        saxBuilder := XML::SAXBuilder driver: saxDriver source: aStream.
        parser builder:saxBuilder.
        ^ parser
    ].
    ^super on:aStream
!

processDTDString: aString

	| p |
	p := self on: aString readStream.
	^p scanDTD
!

processDocumentBytes: aByteArray

	^ self processDocumentBytes: aByteArray beforeScanDo: [ : p | ]
!

processDocumentBytes: aByteArray beforeScanDo: aBlock

	^ [
		self processDocumentBytes: aByteArray encoding: self defaultEncoding beforeScanDo: aBlock
	] on: TfEncodingWarning do: [ : w |
		[
			self processDocumentBytes: aByteArray encoding: w encoding beforeScanDo: aBlock
		] on: TfEncodingWarning do: [ : x | x resume ].
	].
!

processDocumentBytes: aByteArray encoding: encodingSymbol beforeScanDo: aBlock

	| encoder p |
	^ (StreamEncoder encoderDirectory includesKey: encodingSymbol)
		ifTrue: [
			encoder := StreamEncoder new: encodingSymbol.
			p := self on: (EncodedStream on: aByteArray readStream encodedBy: encoder).
			p encoder: encoder.
			aBlock value: p.
			p scanDocument
		]
	ifFalse: [
		self processDocumentString: aByteArray asByteString beforeScanDo: aBlock
	]
!

processDocumentString: aString beforeScanDo: aBlock

	| encoder |
	^ [
		super processDocumentString: aString beforeScanDo: aBlock
	] on: TfEncodingWarning do: [ : w |
		| p |
		(aString size == aString sizeInBytes and: [ StreamEncoder encoderDirectory includesKey: w encoding ])
			ifTrue: [
				[
					encoder := StreamEncoder new: w encoding.
					p := self on: (EncodedStream on: aString asByteArray readStream encodedBy: encoder).
					aBlock value: p.
					w returnWith: p scanDocument
				] on: TfEncodingWarning do: [ : x | x resume ].
			]
			ifFalse: [ w resume ]
	].
! !

!TfHTMLParser class methodsFor:'DTDs'!

looseDTD
	"self looseDTD"

^ '<!!--
    This is the HTML 4.01 Transitional DTD, which includes
    presentation attributes and elements that W3C expects to phase out
    as support for style sheets matures. Authors should use the Strict
    DTD when possible, but may use the Transitional DTD when support
    for presentation attribute and elements is required.

    HTML 4 includes mechanisms for style sheets, scripting,
    embedding objects, improved support for right to left and mixed
    direction text, and enhancements to forms for improved
    accessibility for people with disabilities.

	  Draft: $Date: 2009/11/07 03:21:45 $

	  Authors:
	      Dave Raggett <dsr@w3.org>
	      Arnaud Le Hors <lehors@w3.org>
	      Ian Jacobs <ij@w3.org>

    Further information about HTML 4.01 is available at:

	http://www.w3.org/TR/1999/REC-html401-19991224


    The HTML 4.01 specification includes additional
    syntactic constraints that cannot be expressed within
    the DTDs.

-->
<!!ENTITY % HTML.Version "-//W3C//DTD HTML 4.01 Transitional//EN"
  -- Typical usage:

    <!!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
	    "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    ...
    </head>
    <body>
    ...
    </body>
    </html>

    The URI used as a system identifier with the public identifier allows
    the user agent to download the DTD and entity sets as needed.

    The FPI for the Strict HTML 4.01 DTD is:

	"-//W3C//DTD HTML 4.01//EN"

    This version of the strict DTD is:

	http://www.w3.org/TR/1999/REC-html401-19991224/strict.dtd

    Authors should use the Strict DTD unless they need the
    presentation control for user agents that don''t (adequately)
    support style sheets.

    If you are writing a document that includes frames, use
    the following FPI:

	"-//W3C//DTD HTML 4.01 Frameset//EN"

    This version of the frameset DTD is:

	http://www.w3.org/TR/1999/REC-html401-19991224/frameset.dtd

    Use the following (relative) URIs to refer to
    the DTDs and entity definitions of this specification:

    "strict.dtd"
    "loose.dtd"
    "frameset.dtd"
    "HTMLlat1.ent"
    "HTMLsymbol.ent"
    "HTMLspecial.ent"

-->

<!!--================== Imported Names ====================================-->
<!!-- Feature Switch for frameset documents -->
<!!ENTITY % HTML.Frameset "IGNORE">

<!!ENTITY % ContentType "CDATA"
    -- media type, as per [RFC2045]
    -->

<!!ENTITY % ContentTypes "CDATA"
    -- comma-separated list of media types, as per [RFC2045]
    -->

<!!ENTITY % Charset "CDATA"
    -- a character encoding, as per [RFC2045]
    -->

<!!ENTITY % Charsets "CDATA"
    -- a space-separated list of character encodings, as per [RFC2045]
    -->

<!!ENTITY % LanguageCode "NAME"
    -- a language code, as per [RFC1766]
    -->

<!!ENTITY % Character "CDATA"
    -- a single character from [ISO10646]
    -->

<!!ENTITY % LinkTypes "CDATA"
    -- space-separated list of link types
    -->

<!!ENTITY % MediaDesc "CDATA"
    -- single or comma-separated list of media descriptors
    -->

<!!ENTITY % URI "CDATA"
    -- a Uniform Resource Identifier,
       see [URI]
    -->

<!!ENTITY % Datetime "CDATA" -- date and time information. ISO date format -->


<!!ENTITY % Script "CDATA" -- script expression -->

<!!ENTITY % StyleSheet "CDATA" -- style sheet data -->

<!!ENTITY % FrameTarget "CDATA" -- render in this frame -->


<!!ENTITY % Text "CDATA">


<!!-- Parameter Entities -->

<!!ENTITY % head.misc "SCRIPT|STYLE|META|LINK|OBJECT" -- repeatable head elements -->

<!!ENTITY % heading "H1|H2|H3|H4|H5|H6">

<!!ENTITY % list "UL | OL |  DIR | MENU">

<!!ENTITY % preformatted "PRE">

<!!ENTITY % Color "CDATA" -- a color using sRGB: #RRGGBB as Hex values -->

<!!-- There are also 16 widely known color names with their sRGB values:

    Black  = #000000    Green  = #008000
    Silver = #C0C0C0    Lime   = #00FF00
    Gray   = #808080    Olive  = #808000
    White  = #FFFFFF    Yellow = #FFFF00
    Maroon = #800000    Navy   = #000080
    Red    = #FF0000    Blue   = #0000FF
    Purple = #800080    Teal   = #008080
    Fuchsia= #FF00FF    Aqua   = #00FFFF
 -->

<!!ENTITY % bodycolors "
  bgcolor     %Color;        #IMPLIED  -- document background color --
  text        %Color;        #IMPLIED  -- document text color --
  link        %Color;        #IMPLIED  -- color of links --
  vlink       %Color;        #IMPLIED  -- color of visited links --
  alink       %Color;        #IMPLIED  -- color of selected links --
  ">

<!!--================ Character mnemonic entities =========================-->

<!!ENTITY % HTMLlat1 PUBLIC
   "-//W3C//ENTITIES Latin1//EN//HTML"
   "HTMLlat1.ent">
%HTMLlat1;

<!!ENTITY % HTMLsymbol PUBLIC
   "-//W3C//ENTITIES Symbols//EN//HTML"
   "HTMLsymbol.ent">
%HTMLsymbol;

<!!ENTITY % HTMLspecial PUBLIC
   "-//W3C//ENTITIES Special//EN//HTML"
   "HTMLspecial.ent">
%HTMLspecial;
<!!--=================== Generic Attributes ===============================-->

<!!ENTITY % coreattrs
 "id          ID             #IMPLIED  -- document-wide unique id --
  class       CDATA          #IMPLIED  -- space-separated list of classes --
  style       %StyleSheet;   #IMPLIED  -- associated style info --
  title       %Text;         #IMPLIED  -- advisory title --"
  >

<!!ENTITY % i18n
 "lang        %LanguageCode; #IMPLIED  -- language code --
  dir         (ltr|rtl)      #IMPLIED  -- direction for weak/neutral text --"
  >

<!!ENTITY % events
 "onclick     %Script;       #IMPLIED  -- a pointer button was clicked --
  ondblclick  %Script;       #IMPLIED  -- a pointer button was double clicked--
  onmousedown %Script;       #IMPLIED  -- a pointer button was pressed down --
  onmouseup   %Script;       #IMPLIED  -- a pointer button was released --
  onmouseover %Script;       #IMPLIED  -- a pointer was moved onto --
  onmousemove %Script;       #IMPLIED  -- a pointer was moved within --
  onmouseout  %Script;       #IMPLIED  -- a pointer was moved away --
  onkeypress  %Script;       #IMPLIED  -- a key was pressed and released --
  onkeydown   %Script;       #IMPLIED  -- a key was pressed down --
  onkeyup     %Script;       #IMPLIED  -- a key was released --"
  >

<!!-- Reserved Feature Switch -->
<!!ENTITY % HTML.Reserved "IGNORE">

<!!-- The following attributes are reserved for possible future use -->
<!![ %HTML.Reserved; [
<!!ENTITY % reserved
 "datasrc     %URI;          #IMPLIED  -- a single or tabular Data Source --
  datafld     CDATA          #IMPLIED  -- the property or column name --
  dataformatas (plaintext|html) plaintext -- text or html --"
  >
]]>

<!!ENTITY % reserved "">

<!!ENTITY % attrs "%coreattrs; %i18n; %events;">

<!!ENTITY % align "align (left|center|right|justify)  #IMPLIED"
		   -- default is left for ltr paragraphs, right for rtl --
  >

<!!--=================== Text Markup ======================================-->

<!!ENTITY % fontstyle
 "TT | I | B | U | S | STRIKE | BIG | SMALL">

<!!ENTITY % phrase "EM | STRONG | DFN | CODE |
		   SAMP | KBD | VAR | CITE | ABBR | ACRONYM" >

<!!ENTITY % special
   "A | IMG | APPLET | OBJECT | FONT | BASEFONT | BR | SCRIPT |
    MAP | Q | SUB | SUP | SPAN | BDO | IFRAME">

<!!ENTITY % formctrl "INPUT | SELECT | TEXTAREA | LABEL | BUTTON">

<!!-- %inline; covers inline or "text-level" elements -->
<!!ENTITY % inline "#PCDATA | %fontstyle; | %phrase; | %special; | %formctrl;">

<!!ELEMENT (%fontstyle;|%phrase;) - - (%inline;)*>
<!!ATTLIST (%fontstyle;|%phrase;)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT (SUB|SUP) - - (%inline;)*    -- subscript, superscript -->
<!!ATTLIST (SUB|SUP)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT SPAN - - (%inline;)*         -- generic language/style container -->
<!!ATTLIST SPAN
  %attrs;                              -- %coreattrs, %i18n, %events --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT BDO - - (%inline;)*          -- I18N BiDi over-ride -->
<!!ATTLIST BDO
  %coreattrs;                          -- id, class, style, title --
  lang        %LanguageCode; #IMPLIED  -- language code --
  dir         (ltr|rtl)      #REQUIRED -- directionality --
  >

<!!ELEMENT BASEFONT - O EMPTY           -- base font size -->
<!!ATTLIST BASEFONT
  id          ID             #IMPLIED  -- document-wide unique id --
  size        CDATA          #REQUIRED -- base font size for FONT elements --
  color       %Color;        #IMPLIED  -- text color --
  face        CDATA          #IMPLIED  -- comma-separated list of font names --
  >

<!!ELEMENT FONT - - (%inline;)*         -- local change to font -->
<!!ATTLIST FONT
  %coreattrs;                          -- id, class, style, title --
  %i18n;                               -- lang, dir --
  size        CDATA          #IMPLIED  -- [+|-]nn e.g. size="+1", size="4" --
  color       %Color;        #IMPLIED  -- text color --
  face        CDATA          #IMPLIED  -- comma-separated list of font names --
  >

<!!ELEMENT BR - O EMPTY                 -- forced line break -->
<!!ATTLIST BR
  %coreattrs;                          -- id, class, style, title --
  clear       (left|all|right|none) none -- control of text flow --
  >

<!!--================== HTML content models ===============================-->

<!!--
    HTML has two basic content models:

	%inline;     character level elements and text strings
	%block;      block-like elements e.g. paragraphs and lists
-->

<!!ENTITY % block
     "P | %heading; | %list; | %preformatted; | DL | DIV | CENTER |
      NOSCRIPT | NOFRAMES | BLOCKQUOTE | FORM | ISINDEX | HR |
      TABLE | FIELDSET | ADDRESS">

<!!ENTITY % flow "%block; | %inline;">

<!!--=================== Document Body ====================================-->

<!!ELEMENT BODY O O (%flow;)* +(INS|DEL) -- document body -->
<!!ATTLIST BODY
  %attrs;                              -- %coreattrs, %i18n, %events --
  onload          %Script;   #IMPLIED  -- the document has been loaded --
  onunload        %Script;   #IMPLIED  -- the document has been removed --
  background      %URI;      #IMPLIED  -- texture tile for document
					  background --
  %bodycolors;                         -- bgcolor, text, link, vlink, alink --
  >

<!!ELEMENT ADDRESS - - ((%inline;)|P)*  -- information on author -->
<!!ATTLIST ADDRESS
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT DIV - - (%flow;)*            -- generic language/style container -->
<!!ATTLIST DIV
  %attrs;                              -- %coreattrs, %i18n, %events --
  %align;                              -- align, text alignment --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT CENTER - - (%flow;)*         -- shorthand for DIV align=center -->
<!!ATTLIST CENTER
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--================== The Anchor Element ================================-->

<!!ENTITY % Shape "(rect|circle|poly|default)">
<!!ENTITY % Coords "CDATA" -- comma-separated list of lengths -->

<!!ELEMENT A - - (%inline;)* -(A)       -- anchor -->
<!!ATTLIST A
  %attrs;                              -- %coreattrs, %i18n, %events --
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  type        %ContentType;  #IMPLIED  -- advisory content type --
  name        CDATA          #IMPLIED  -- named link end --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  hreflang    %LanguageCode; #IMPLIED  -- language code --
  target      %FrameTarget;  #IMPLIED  -- render in this frame --
  rel         %LinkTypes;    #IMPLIED  -- forward link types --
  rev         %LinkTypes;    #IMPLIED  -- reverse link types --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  shape       %Shape;        rect      -- for use with client-side image maps --
  coords      %Coords;       #IMPLIED  -- for use with client-side image maps --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!--================== Client-side image maps ============================-->

<!!-- These can be placed in the same document or grouped in a
     separate document although this isn''t yet widely supported -->

<!!ELEMENT MAP - - ((%block;) | AREA)+ -- client-side image map -->
<!!ATTLIST MAP
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #REQUIRED -- for reference by usemap --
  >

<!!ELEMENT AREA - O EMPTY               -- client-side image map area -->
<!!ATTLIST AREA
  %attrs;                              -- %coreattrs, %i18n, %events --
  shape       %Shape;        rect      -- controls interpretation of coords --
  coords      %Coords;       #IMPLIED  -- comma-separated list of lengths --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  target      %FrameTarget;  #IMPLIED  -- render in this frame --
  nohref      (nohref)       #IMPLIED  -- this region has no action --
  alt         %Text;         #REQUIRED -- short description --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!--================== The LINK Element ==================================-->

<!!--
  Relationship values can be used in principle:

   a) for document specific toolbars/menus when used
      with the LINK element in document head e.g.
	start, contents, previous, next, index, end, help
   b) to link to a separate style sheet (rel=stylesheet)
   c) to make a link to a script (rel=script)
   d) by stylesheets to control how collections of
      html nodes are rendered into printed documents
   e) to make a link to a printable version of this document
      e.g. a postscript or pdf version (rel=alternate media=print)
-->

<!!ELEMENT LINK - O EMPTY               -- a media-independent link -->
<!!ATTLIST LINK
  %attrs;                              -- %coreattrs, %i18n, %events --
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  hreflang    %LanguageCode; #IMPLIED  -- language code --
  type        %ContentType;  #IMPLIED  -- advisory content type --
  rel         %LinkTypes;    #IMPLIED  -- forward link types --
  rev         %LinkTypes;    #IMPLIED  -- reverse link types --
  media       %MediaDesc;    #IMPLIED  -- for rendering on these media --
  target      %FrameTarget;  #IMPLIED  -- render in this frame --
  >

<!!--=================== Images ===========================================-->

<!!-- Length defined in strict DTD for cellpadding/cellspacing -->
<!!ENTITY % Length "CDATA" -- nn for pixels or nn% for percentage length -->
<!!ENTITY % MultiLength "CDATA" -- pixel, percentage, or relative -->

<!![ %HTML.Frameset; [
<!!ENTITY % MultiLengths "CDATA" -- comma-separated list of MultiLength -->
]]>

<!!ENTITY % Pixels "CDATA" -- integer representing length in pixels -->

<!!ENTITY % IAlign "(top|middle|bottom|left|right)" -- center? -->

<!!-- To avoid problems with text-only UAs as well as
   to make image content understandable and navigable
   to users of non-visual UAs, you need to provide
   a description with ALT, and avoid server-side image maps -->
<!!ELEMENT IMG - O EMPTY                -- Embedded image -->
<!!ATTLIST IMG
  %attrs;                              -- %coreattrs, %i18n, %events --
  src         %URI;          #REQUIRED -- URI of image to embed --
  alt         %Text;         #REQUIRED -- short description --
  longdesc    %URI;          #IMPLIED  -- link to long description
					  (complements alt) --
  name        CDATA          #IMPLIED  -- name of image for scripting --
  height      %Length;       #IMPLIED  -- override height --
  width       %Length;       #IMPLIED  -- override width --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  ismap       (ismap)        #IMPLIED  -- use server-side image map --
  align       %IAlign;       #IMPLIED  -- vertical or horizontal alignment --
  border      %Pixels;       #IMPLIED  -- link border width --
  hspace      %Pixels;       #IMPLIED  -- horizontal gutter --
  vspace      %Pixels;       #IMPLIED  -- vertical gutter --
  >

<!!-- USEMAP points to a MAP element which may be in this document
  or an external document, although the latter is not widely supported -->

<!!--==================== OBJECT ======================================-->
<!!--
  OBJECT is used to embed objects as part of HTML pages
  PARAM elements should precede other content. SGML mixed content
  model technicality precludes specifying this formally ...
-->

<!!ELEMENT OBJECT - - (PARAM | %flow;)*
 -- generic embedded object -->
<!!ATTLIST OBJECT
  %attrs;                              -- %coreattrs, %i18n, %events --
  declare     (declare)      #IMPLIED  -- declare but don''t instantiate flag --
  classid     %URI;          #IMPLIED  -- identifies an implementation --
  codebase    %URI;          #IMPLIED  -- base URI for classid, data, archive--
  data        %URI;          #IMPLIED  -- reference to object''s data --
  type        %ContentType;  #IMPLIED  -- content type for data --
  codetype    %ContentType;  #IMPLIED  -- content type for code --
  archive     CDATA          #IMPLIED  -- space-separated list of URIs --
  standby     %Text;         #IMPLIED  -- message to show while loading --
  height      %Length;       #IMPLIED  -- override height --
  width       %Length;       #IMPLIED  -- override width --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  name        CDATA          #IMPLIED  -- submit as part of form --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  align       %IAlign;       #IMPLIED  -- vertical or horizontal alignment --
  border      %Pixels;       #IMPLIED  -- link border width --
  hspace      %Pixels;       #IMPLIED  -- horizontal gutter --
  vspace      %Pixels;       #IMPLIED  -- vertical gutter --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT PARAM - O EMPTY              -- named property value -->
<!!ATTLIST PARAM
  id          ID             #IMPLIED  -- document-wide unique id --
  name        CDATA          #REQUIRED -- property name --
  value       CDATA          #IMPLIED  -- property value --
  valuetype   (DATA|REF|OBJECT) DATA   -- How to interpret value --
  type        %ContentType;  #IMPLIED  -- content type for value
					  when valuetype=ref --
  >

<!!--=================== Java APPLET ==================================-->
<!!--
  One of code or object attributes must be present.
  Place PARAM elements before other content.
-->
<!!ELEMENT APPLET - - (PARAM | %flow;)* -- Java applet -->
<!!ATTLIST APPLET
  %coreattrs;                          -- id, class, style, title --
  codebase    %URI;          #IMPLIED  -- optional base URI for applet --
  archive     CDATA          #IMPLIED  -- comma-separated archive list --
  code        CDATA          #IMPLIED  -- applet class file --
  object      CDATA          #IMPLIED  -- serialized applet file --
  alt         %Text;         #IMPLIED  -- short description --
  name        CDATA          #IMPLIED  -- allows applets to find each other --
  width       %Length;       #REQUIRED -- initial width --
  height      %Length;       #REQUIRED -- initial height --
  align       %IAlign;       #IMPLIED  -- vertical or horizontal alignment --
  hspace      %Pixels;       #IMPLIED  -- horizontal gutter --
  vspace      %Pixels;       #IMPLIED  -- vertical gutter --
  >

<!!--=================== Horizontal Rule ==================================-->

<!!ELEMENT HR - O EMPTY -- horizontal rule -->
<!!ATTLIST HR
  %attrs;                              -- %coreattrs, %i18n, %events --
  align       (left|center|right) #IMPLIED
  noshade     (noshade)      #IMPLIED
  size        %Pixels;       #IMPLIED
  width       %Length;       #IMPLIED
  >

<!!--=================== Paragraphs =======================================-->

<!!ELEMENT P - O (%inline;)*            -- paragraph -->
<!!ATTLIST P
  %attrs;                              -- %coreattrs, %i18n, %events --
  %align;                              -- align, text alignment --
  >

<!!--=================== Headings =========================================-->

<!!--
  There are six levels of headings from H1 (the most important)
  to H6 (the least important).
-->

<!!ELEMENT (%heading;)  - - (%inline;)* -- heading -->
<!!ATTLIST (%heading;)
  %attrs;                              -- %coreattrs, %i18n, %events --
  %align;                              -- align, text alignment --
  >

<!!--=================== Preformatted Text ================================-->

<!!-- excludes markup for images and changes in font size -->
<!!ENTITY % pre.exclusion "IMG|OBJECT|APPLET|BIG|SMALL|SUB|SUP|FONT|BASEFONT">

<!!ELEMENT PRE - - (%inline;)* -(%pre.exclusion;) -- preformatted text -->
<!!ATTLIST PRE
  %attrs;                              -- %coreattrs, %i18n, %events --
  width       NUMBER         #IMPLIED
  >

<!!--===================== Inline Quotes ==================================-->

<!!ELEMENT Q - - (%inline;)*            -- short inline quotation -->
<!!ATTLIST Q
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- URI for source document or msg --
  >

<!!--=================== Block-like Quotes ================================-->

<!!ELEMENT BLOCKQUOTE - - (%flow;)*     -- long quotation -->
<!!ATTLIST BLOCKQUOTE
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- URI for source document or msg --
  >

<!!--=================== Inserted/Deleted Text ============================-->


<!!-- INS/DEL are handled by inclusion on BODY -->
<!!ELEMENT (INS|DEL) - - (%flow;)*      -- inserted text, deleted text -->
<!!ATTLIST (INS|DEL)
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- info on reason for change --
  datetime    %Datetime;     #IMPLIED  -- date and time of change --
  >

<!!--=================== Lists ============================================-->

<!!-- definition lists - DT for term, DD for its definition -->

<!!ELEMENT DL - - (DT|DD)+              -- definition list -->
<!!ATTLIST DL
  %attrs;                              -- %coreattrs, %i18n, %events --
  compact     (compact)      #IMPLIED  -- reduced interitem spacing --
  >

<!!ELEMENT DT - O (%inline;)*           -- definition term -->
<!!ELEMENT DD - O (%flow;)*             -- definition description -->
<!!ATTLIST (DT|DD)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!-- Ordered lists (OL) Numbering style

    1   arablic numbers     1, 2, 3, ...
    a   lower alpha         a, b, c, ...
    A   upper alpha         A, B, C, ...
    i   lower roman         i, ii, iii, ...
    I   upper roman         I, II, III, ...

    The style is applied to the sequence number which by default
    is reset to 1 for the first list item in an ordered list.

    This can''t be expressed directly in SGML due to case folding.
-->

<!!ENTITY % OLStyle "CDATA"      -- constrained to: "(1|a|A|i|I)" -->

<!!ELEMENT OL - - (LI)+                 -- ordered list -->
<!!ATTLIST OL
  %attrs;                              -- %coreattrs, %i18n, %events --
  type        %OLStyle;      #IMPLIED  -- numbering style --
  compact     (compact)      #IMPLIED  -- reduced interitem spacing --
  start       NUMBER         #IMPLIED  -- starting sequence number --
  >

<!!-- Unordered Lists (UL) bullet styles -->
<!!ENTITY % ULStyle "(disc|square|circle)">

<!!ELEMENT UL - - (LI)+                 -- unordered list -->
<!!ATTLIST UL
  %attrs;                              -- %coreattrs, %i18n, %events --
  type        %ULStyle;      #IMPLIED  -- bullet style --
  compact     (compact)      #IMPLIED  -- reduced interitem spacing --
  >

<!!ELEMENT (DIR|MENU) - - (LI)+ -(%block;) -- directory list, menu list -->
<!!ATTLIST DIR
  %attrs;                              -- %coreattrs, %i18n, %events --
  compact     (compact)      #IMPLIED -- reduced interitem spacing --
  >
<!!ATTLIST MENU
  %attrs;                              -- %coreattrs, %i18n, %events --
  compact     (compact)      #IMPLIED -- reduced interitem spacing --
  >

<!!ENTITY % LIStyle "CDATA" -- constrained to: "(%ULStyle;|%OLStyle;)" -->

<!!ELEMENT LI - O (%flow;)*             -- list item -->
<!!ATTLIST LI
  %attrs;                              -- %coreattrs, %i18n, %events --
  type        %LIStyle;      #IMPLIED  -- list item style --
  value       NUMBER         #IMPLIED  -- reset sequence number --
  >

<!!--================ Forms ===============================================-->
<!!ELEMENT FORM - - (%flow;)* -(FORM)   -- interactive form -->
<!!ATTLIST FORM
  %attrs;                              -- %coreattrs, %i18n, %events --
  action      %URI;          #REQUIRED -- server-side form handler --
  method      (GET|POST)     GET       -- HTTP method used to submit the form--
  enctype     %ContentType;  "application/x-www-form-urlencoded"
  accept      %ContentTypes; #IMPLIED  -- list of MIME types for file upload --
  name        CDATA          #IMPLIED  -- name of form for scripting --
  onsubmit    %Script;       #IMPLIED  -- the form was submitted --
  onreset     %Script;       #IMPLIED  -- the form was reset --
  target      %FrameTarget;  #IMPLIED  -- render in this frame --
  accept-charset %Charsets;  #IMPLIED  -- list of supported charsets --
  >

<!!-- Each label must not contain more than ONE field -->
<!!ELEMENT LABEL - - (%inline;)* -(LABEL) -- form field label text -->
<!!ATTLIST LABEL
  %attrs;                              -- %coreattrs, %i18n, %events --
  for         IDREF          #IMPLIED  -- matches field ID value --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!ENTITY % InputType
  "(TEXT | PASSWORD | CHECKBOX |
    RADIO | SUBMIT | RESET |
    FILE | HIDDEN | IMAGE | BUTTON)"
   >

<!!-- attribute name required for all but submit and reset -->
<!!ELEMENT INPUT - O EMPTY              -- form control -->
<!!ATTLIST INPUT
  %attrs;                              -- %coreattrs, %i18n, %events --
  type        %InputType;    TEXT      -- what kind of widget is needed --
  name        CDATA          #IMPLIED  -- submit as part of form --
  value       CDATA          #IMPLIED  -- Specify for radio buttons and checkboxes --
  checked     (checked)      #IMPLIED  -- for radio buttons and check boxes --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  readonly    (readonly)     #IMPLIED  -- for text and passwd --
  size        CDATA          #IMPLIED  -- specific to each type of field --
  maxlength   NUMBER         #IMPLIED  -- max chars for text fields --
  src         %URI;          #IMPLIED  -- for fields with images --
  alt         CDATA          #IMPLIED  -- short description --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  ismap       (ismap)        #IMPLIED  -- use server-side image map --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onselect    %Script;       #IMPLIED  -- some text was selected --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  accept      %ContentTypes; #IMPLIED  -- list of MIME types for file upload --
  align       %IAlign;       #IMPLIED  -- vertical or horizontal alignment --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT SELECT - - (OPTGROUP|OPTION)+ -- option selector -->
<!!ATTLIST SELECT
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED  -- field name --
  size        NUMBER         #IMPLIED  -- rows visible --
  multiple    (multiple)     #IMPLIED  -- default is single selection --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT OPTGROUP - - (OPTION)+ -- option group -->
<!!ATTLIST OPTGROUP
  %attrs;                              -- %coreattrs, %i18n, %events --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  label       %Text;         #REQUIRED -- for use in hierarchical menus --
  >

<!!ELEMENT OPTION - O (#PCDATA)         -- selectable choice -->
<!!ATTLIST OPTION
  %attrs;                              -- %coreattrs, %i18n, %events --
  selected    (selected)     #IMPLIED
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  label       %Text;         #IMPLIED  -- for use in hierarchical menus --
  value       CDATA          #IMPLIED  -- defaults to element content --
  >

<!!ELEMENT TEXTAREA - - (#PCDATA)       -- multi-line text field -->
<!!ATTLIST TEXTAREA
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED
  rows        NUMBER         #REQUIRED
  cols        NUMBER         #REQUIRED
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  readonly    (readonly)     #IMPLIED
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onselect    %Script;       #IMPLIED  -- some text was selected --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  %reserved;                           -- reserved for possible future use --
  >

<!!--
  #PCDATA is to solve the mixed content problem,
  per specification only whitespace is allowed there!!
 -->
<!!ELEMENT FIELDSET - - (#PCDATA,LEGEND,(%flow;)*) -- form control group -->
<!!ATTLIST FIELDSET
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT LEGEND - - (%inline;)*       -- fieldset legend -->
<!!ENTITY % LAlign "(top|bottom|left|right)">

<!!ATTLIST LEGEND
  %attrs;                              -- %coreattrs, %i18n, %events --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  align       %LAlign;       #IMPLIED  -- relative to fieldset --
  >

<!!ELEMENT BUTTON - -
     (%flow;)* -(A|%formctrl;|FORM|ISINDEX|FIELDSET|IFRAME)
     -- push button -->
<!!ATTLIST BUTTON
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED
  value       CDATA          #IMPLIED  -- sent to server when submitted --
  type        (button|submit|reset) submit -- for use as form button --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  %reserved;                           -- reserved for possible future use --
  >

<!!--======================= Tables =======================================-->

<!!-- IETF HTML table standard, see [RFC1942] -->

<!!--
 The BORDER attribute sets the thickness of the frame around the
 table. The default units are screen pixels.

 The FRAME attribute specifies which parts of the frame around
 the table should be rendered. The values are not the same as
 CALS to avoid a name clash with the VALIGN attribute.

 The value "border" is included for backwards compatibility with
 <TABLE BORDER> which yields frame=border and border=implied
 For <TABLE BORDER=1> you get border=1 and frame=implied. In this
 case, it is appropriate to treat this as frame=border for backwards
 compatibility with deployed browsers.
-->
<!!ENTITY % TFrame "(void|above|below|hsides|lhs|rhs|vsides|box|border)">

<!!--
 The RULES attribute defines which rules to draw between cells:

 If RULES is absent then assume:
     "none" if BORDER is absent or BORDER=0 otherwise "all"
-->

<!!ENTITY % TRules "(none | groups | rows | cols | all)">

<!!-- horizontal placement of table relative to document -->
<!!ENTITY % TAlign "(left|center|right)">

<!!-- horizontal alignment attributes for cell contents -->
<!!ENTITY % cellhalign
  "align      (left|center|right|justify|char) #IMPLIED
   char       %Character;    #IMPLIED  -- alignment char, e.g. char='':'' --
   charoff    %Length;       #IMPLIED  -- offset for alignment char --"
  >

<!!-- vertical alignment attributes for cell contents -->
<!!ENTITY % cellvalign
  "valign     (top|middle|bottom|baseline) #IMPLIED"
  >

<!!ELEMENT TABLE - -
     (CAPTION?, (COL*|COLGROUP*), THEAD?, TFOOT?, TBODY+)>
<!!ELEMENT CAPTION  - - (%inline;)*     -- table caption -->
<!!ELEMENT THEAD    - O (TR)+           -- table header -->
<!!ELEMENT TFOOT    - O (TR)+           -- table footer -->
<!!ELEMENT TBODY    O O (TR)+           -- table body -->
<!!ELEMENT COLGROUP - O (COL)*          -- table column group -->
<!!ELEMENT COL      - O EMPTY           -- table column -->
<!!ELEMENT TR       - O (TH|TD)+        -- table row -->
<!!ELEMENT (TH|TD)  - O (%flow;)*       -- table header cell, table data cell-->

<!!ATTLIST TABLE                        -- table element --
  %attrs;                              -- %coreattrs, %i18n, %events --
  summary     %Text;         #IMPLIED  -- purpose/structure for speech output--
  width       %Length;       #IMPLIED  -- table width --
  border      %Pixels;       #IMPLIED  -- controls frame width around table --
  frame       %TFrame;       #IMPLIED  -- which parts of frame to render --
  rules       %TRules;       #IMPLIED  -- rulings between rows and cols --
  cellspacing %Length;       #IMPLIED  -- spacing between cells --
  cellpadding %Length;       #IMPLIED  -- spacing within cells --
  align       %TAlign;       #IMPLIED  -- table position relative to window --
  bgcolor     %Color;        #IMPLIED  -- background color for cells --
  %reserved;                           -- reserved for possible future use --
  datapagesize CDATA         #IMPLIED  -- reserved for possible future use --
  >

<!!ENTITY % CAlign "(top|bottom|left|right)">

<!!ATTLIST CAPTION
  %attrs;                              -- %coreattrs, %i18n, %events --
  align       %CAlign;       #IMPLIED  -- relative to table --
  >

<!!--
COLGROUP groups a set of COL elements. It allows you to group
several semantically related columns together.
-->
<!!ATTLIST COLGROUP
  %attrs;                              -- %coreattrs, %i18n, %events --
  span        NUMBER         1         -- default number of columns in group --
  width       %MultiLength;  #IMPLIED  -- default width for enclosed COLs --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!--
 COL elements define the alignment properties for cells in
 one or more columns.

 The WIDTH attribute specifies the width of the columns, e.g.

     width=64        width in screen pixels
     width=0.5*      relative width of 0.5

 The SPAN attribute causes the attributes of one
 COL element to apply to more than one column.
-->
<!!ATTLIST COL                          -- column groups and properties --
  %attrs;                              -- %coreattrs, %i18n, %events --
  span        NUMBER         1         -- COL attributes affect N columns --
  width       %MultiLength;  #IMPLIED  -- column width specification --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!--
    Use THEAD to duplicate headers when breaking table
    across page boundaries, or for static headers when
    TBODY sections are rendered in scrolling panel.

    Use TFOOT to duplicate footers when breaking table
    across page boundaries, or for static footers when
    TBODY sections are rendered in scrolling panel.

    Use multiple TBODY sections when rules are needed
    between groups of table rows.
-->
<!!ATTLIST (THEAD|TBODY|TFOOT)          -- table section --
  %attrs;                              -- %coreattrs, %i18n, %events --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!ATTLIST TR                           -- table row --
  %attrs;                              -- %coreattrs, %i18n, %events --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  bgcolor     %Color;        #IMPLIED  -- background color for row --
  >


<!!-- Scope is simpler than headers attribute for common tables -->
<!!ENTITY % Scope "(row|col|rowgroup|colgroup)">

<!!-- TH is for headers, TD for data, but for cells acting as both use TD -->
<!!ATTLIST (TH|TD)                      -- header or data cell --
  %attrs;                              -- %coreattrs, %i18n, %events --
  abbr        %Text;         #IMPLIED  -- abbreviation for header cell --
  axis        CDATA          #IMPLIED  -- comma-separated list of related headers--
  headers     IDREFS         #IMPLIED  -- list of id''s for header cells --
  scope       %Scope;        #IMPLIED  -- scope covered by header cells --
  rowspan     NUMBER         1         -- number of rows spanned by cell --
  colspan     NUMBER         1         -- number of cols spanned by cell --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  nowrap      (nowrap)       #IMPLIED  -- suppress word wrap --
  bgcolor     %Color;        #IMPLIED  -- cell background color --
  width       %Length;       #IMPLIED  -- width for cell --
  height      %Length;       #IMPLIED  -- height for cell --
  >

<!!--================== Document Frames ===================================-->

<!!--
  The content model for HTML documents depends on whether the HEAD is
  followed by a FRAMESET or BODY element. The widespread omission of
  the BODY start tag makes it impractical to define the content model
  without the use of a marked section.
-->

<!![ %HTML.Frameset; [
<!!ELEMENT FRAMESET - - ((FRAMESET|FRAME)+ & NOFRAMES?) -- window subdivision-->
<!!ATTLIST FRAMESET
  %coreattrs;                          -- id, class, style, title --
  rows        %MultiLengths; #IMPLIED  -- list of lengths,
					  default: 100% (1 row) --
  cols        %MultiLengths; #IMPLIED  -- list of lengths,
					  default: 100% (1 col) --
  onload      %Script;       #IMPLIED  -- all the frames have been loaded  --
  onunload    %Script;       #IMPLIED  -- all the frames have been removed --
  >
]]>

<!![ %HTML.Frameset; [
<!!-- reserved frame names start with "_" otherwise starts with letter -->
<!!ELEMENT FRAME - O EMPTY              -- subwindow -->
<!!ATTLIST FRAME
  %coreattrs;                          -- id, class, style, title --
  longdesc    %URI;          #IMPLIED  -- link to long description
					  (complements title) --
  name        CDATA          #IMPLIED  -- name of frame for targetting --
  src         %URI;          #IMPLIED  -- source of frame content --
  frameborder (1|0)          1         -- request frame borders? --
  marginwidth %Pixels;       #IMPLIED  -- margin widths in pixels --
  marginheight %Pixels;      #IMPLIED  -- margin height in pixels --
  noresize    (noresize)     #IMPLIED  -- allow users to resize frames? --
  scrolling   (yes|no|auto)  auto      -- scrollbar or none --
  >
]]>

<!!ELEMENT IFRAME - - (%flow;)*         -- inline subwindow -->
<!!ATTLIST IFRAME
  %coreattrs;                          -- id, class, style, title --
  longdesc    %URI;          #IMPLIED  -- link to long description
					  (complements title) --
  name        CDATA          #IMPLIED  -- name of frame for targetting --
  src         %URI;          #IMPLIED  -- source of frame content --
  frameborder (1|0)          1         -- request frame borders? --
  marginwidth %Pixels;       #IMPLIED  -- margin widths in pixels --
  marginheight %Pixels;      #IMPLIED  -- margin height in pixels --
  scrolling   (yes|no|auto)  auto      -- scrollbar or none --
  align       %IAlign;       #IMPLIED  -- vertical or horizontal alignment --
  height      %Length;       #IMPLIED  -- frame height --
  width       %Length;       #IMPLIED  -- frame width --
  >

<!![ %HTML.Frameset; [
<!!ENTITY % noframes.content "(BODY) -(NOFRAMES)">
]]>

<!!ENTITY % noframes.content "(%flow;)*">

<!!ELEMENT NOFRAMES - - %noframes.content;
 -- alternate content container for non frame-based rendering -->
<!!ATTLIST NOFRAMES
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--================ Document Head =======================================-->
<!!-- %head.misc; defined earlier on as "SCRIPT|STYLE|META|LINK|OBJECT" -->
<!!ENTITY % head.content "TITLE & ISINDEX? & BASE?">

<!!ELEMENT HEAD O O (%head.content;) +(%head.misc;) -- document head -->
<!!ATTLIST HEAD
  %i18n;                               -- lang, dir --
  profile     %URI;          #IMPLIED  -- named dictionary of meta info --
  >

<!!-- The TITLE element is not considered part of the flow of text.
       It should be displayed, for example as the page header or
       window title. Exactly one title is required per document.
    -->
<!!ELEMENT TITLE - - (#PCDATA) -(%head.misc;) -- document title -->
<!!ATTLIST TITLE %i18n>

<!!ELEMENT ISINDEX - O EMPTY            -- single line prompt -->
<!!ATTLIST ISINDEX
  %coreattrs;                          -- id, class, style, title --
  %i18n;                               -- lang, dir --
  prompt      %Text;         #IMPLIED  -- prompt message -->

<!!ELEMENT BASE - O EMPTY               -- document base URI -->
<!!ATTLIST BASE
  href        %URI;          #IMPLIED  -- URI that acts as base URI --
  target      %FrameTarget;  #IMPLIED  -- render in this frame --
  >

<!!ELEMENT META - O EMPTY               -- generic metainformation -->
<!!ATTLIST META
  %i18n;                               -- lang, dir, for use with content --
  http-equiv  NAME           #IMPLIED  -- HTTP response header name  --
  name        NAME           #IMPLIED  -- metainformation name --
  content     CDATA          #REQUIRED -- associated information --
  scheme      CDATA          #IMPLIED  -- select form of content --
  >

<!!ELEMENT STYLE - - %StyleSheet        -- style info -->
<!!ATTLIST STYLE
  %i18n;                               -- lang, dir, for use with title --
  type        %ContentType;  #REQUIRED -- content type of style language --
  media       %MediaDesc;    #IMPLIED  -- designed for use with these media --
  title       %Text;         #IMPLIED  -- advisory title --
  >

<!!ELEMENT SCRIPT - - %Script;          -- script statements -->
<!!ATTLIST SCRIPT
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  type        %ContentType;  #REQUIRED -- content type of script language --
  language    CDATA          #IMPLIED  -- predefined script language name --
  src         %URI;          #IMPLIED  -- URI for an external script --
  defer       (defer)        #IMPLIED  -- UA may defer execution of script --
  event       CDATA          #IMPLIED  -- reserved for possible future use --
  for         %URI;          #IMPLIED  -- reserved for possible future use --
  >

<!!ELEMENT NOSCRIPT - - (%flow;)*
  -- alternate content container for non script-based rendering -->
<!!ATTLIST NOSCRIPT
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--================ Document Structure ==================================-->
<!!ENTITY % version "version CDATA #FIXED ''%HTML.Version;''">

<!![ %HTML.Frameset; [
<!!ENTITY % html.content "HEAD, FRAMESET">
]]>

<!!ENTITY % html.content "HEAD, BODY">

<!!ELEMENT HTML O O (%html.content;)    -- document root element -->
<!!ATTLIST HTML
  %i18n;                               -- lang, dir --
  %version;
  >'
!

strictDTD
	"self processDTDString: self strictDTD"

^ '<!!--
    This is HTML 4.01 Strict DTD, which excludes the presentation
    attributes and elements that W3C expects to phase out as
    support for style sheets matures. Authors should use the Strict
    DTD when possible, but may use the Transitional DTD when support
    for presentation attribute and elements is required.

    HTML 4 includes mechanisms for style sheets, scripting,
    embedding objects, improved support for right to left and mixed
    direction text, and enhancements to forms for improved
    accessibility for people with disabilities.

	  Draft: $Date: 2009/11/07 03:21:45 $

	  Authors:
	      Dave Raggett <dsr@w3.org>
	      Arnaud Le Hors <lehors@w3.org>
	      Ian Jacobs <ij@w3.org>

    Further information about HTML 4.01 is available at:

	http://www.w3.org/TR/1999/REC-html401-19991224


    The HTML 4.01 specification includes additional
    syntactic constraints that cannot be expressed within
    the DTDs.

-->
<!!--
    Typical usage:

    <!!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	    "http://www.w3.org/TR/html4/strict.dtd">
    <html>
    <head>
    ...
    </head>
    <body>
    ...
    </body>
    </html>

    The URI used as a system identifier with the public identifier allows
    the user agent to download the DTD and entity sets as needed.

    The FPI for the Transitional HTML 4.01 DTD is:

	"-//W3C//DTD HTML 4.01 Transitional//EN"

    This version of the transitional DTD is:

	http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd

    If you are writing a document that includes frames, use
    the following FPI:

	"-//W3C//DTD HTML 4.01 Frameset//EN"

    This version of the frameset DTD is:

	http://www.w3.org/TR/1999/REC-html401-19991224/frameset.dtd

    Use the following (relative) URIs to refer to
    the DTDs and entity definitions of this specification:

    "strict.dtd"
    "loose.dtd"
    "frameset.dtd"
    "HTMLlat1.ent"
    "HTMLsymbol.ent"
    "HTMLspecial.ent"

-->

<!!--================== Imported Names ====================================-->
<!!-- Feature Switch for frameset documents -->
<!!ENTITY % HTML.Frameset "IGNORE">

<!!ENTITY % ContentType "CDATA"
    -- media type, as per [RFC2045]
    -->

<!!ENTITY % ContentTypes "CDATA"
    -- comma-separated list of media types, as per [RFC2045]
    -->

<!!ENTITY % Charset "CDATA"
    -- a character encoding, as per [RFC2045]
    -->

<!!ENTITY % Charsets "CDATA"
    -- a space-separated list of character encodings, as per [RFC2045]
    -->

<!!ENTITY % LanguageCode "NAME"
    -- a language code, as per [RFC1766]
    -->

<!!ENTITY % Character "CDATA"
    -- a single character from [ISO10646]
    -->

<!!ENTITY % LinkTypes "CDATA"
    -- space-separated list of link types
    -->

<!!ENTITY % MediaDesc "CDATA"
    -- single or comma-separated list of media descriptors
    -->

<!!ENTITY % URI "CDATA"
    -- a Uniform Resource Identifier,
       see [URI]
    -->

<!!ENTITY % Datetime "CDATA" -- date and time information. ISO date format -->


<!!ENTITY % Script "CDATA" -- script expression -->

<!!ENTITY % StyleSheet "CDATA" -- style sheet data -->



<!!ENTITY % Text "CDATA">


<!!-- Parameter Entities -->

<!!ENTITY % head.misc "SCRIPT|STYLE|META|LINK|OBJECT" -- repeatable head elements -->

<!!ENTITY % heading "H1|H2|H3|H4|H5|H6">

<!!ENTITY % list "UL | OL">

<!!ENTITY % preformatted "PRE">


<!!--================ Character mnemonic entities =========================-->

<!!ENTITY % HTMLlat1 PUBLIC
   "-//W3C//ENTITIES Latin1//EN//HTML"
   "HTMLlat1.ent">
%HTMLlat1;

<!!ENTITY % HTMLsymbol PUBLIC
   "-//W3C//ENTITIES Symbols//EN//HTML"
   "HTMLsymbol.ent">
%HTMLsymbol;

<!!ENTITY % HTMLspecial PUBLIC
   "-//W3C//ENTITIES Special//EN//HTML"
   "HTMLspecial.ent">
%HTMLspecial;
<!!--=================== Generic Attributes ===============================-->

<!!ENTITY % coreattrs
 "id          ID             #IMPLIED  -- document-wide unique id --
  class       CDATA          #IMPLIED  -- space-separated list of classes --
  style       %StyleSheet;   #IMPLIED  -- associated style info --
  title       %Text;         #IMPLIED  -- advisory title --"
  >

<!!ENTITY % i18n
 "lang        %LanguageCode; #IMPLIED  -- language code --
  dir         (ltr|rtl)      #IMPLIED  -- direction for weak/neutral text --"
  >

<!!ENTITY % events
 "onclick     %Script;       #IMPLIED  -- a pointer button was clicked --
  ondblclick  %Script;       #IMPLIED  -- a pointer button was double clicked--
  onmousedown %Script;       #IMPLIED  -- a pointer button was pressed down --
  onmouseup   %Script;       #IMPLIED  -- a pointer button was released --
  onmouseover %Script;       #IMPLIED  -- a pointer was moved onto --
  onmousemove %Script;       #IMPLIED  -- a pointer was moved within --
  onmouseout  %Script;       #IMPLIED  -- a pointer was moved away --
  onkeypress  %Script;       #IMPLIED  -- a key was pressed and released --
  onkeydown   %Script;       #IMPLIED  -- a key was pressed down --
  onkeyup     %Script;       #IMPLIED  -- a key was released --"
  >

<!!-- Reserved Feature Switch -->
<!!ENTITY % HTML.Reserved "IGNORE">

<!!-- The following attributes are reserved for possible future use -->
<!![ %HTML.Reserved; [
<!!ENTITY % reserved
 "datasrc     %URI;          #IMPLIED  -- a single or tabular Data Source --
  datafld     CDATA          #IMPLIED  -- the property or column name --
  dataformatas (plaintext|html) plaintext -- text or html --"
  >
]]>

<!!ENTITY % reserved "">

<!!ENTITY % attrs "%coreattrs; %i18n; %events;">


<!!--=================== Text Markup ======================================-->

<!!ENTITY % fontstyle
 "TT | I | B | BIG | SMALL">

<!!ENTITY % phrase "EM | STRONG | DFN | CODE |
		   SAMP | KBD | VAR | CITE | ABBR | ACRONYM" >

<!!ENTITY % special
   "A | IMG | OBJECT | BR | SCRIPT | MAP | Q | SUB | SUP | SPAN | BDO">

<!!ENTITY % formctrl "INPUT | SELECT | TEXTAREA | LABEL | BUTTON">

<!!-- %inline; covers inline or "text-level" elements -->
<!!ENTITY % inline "#PCDATA | %fontstyle; | %phrase; | %special; | %formctrl;">

<!!ELEMENT (%fontstyle;|%phrase;) - - (%inline;)*>
<!!ATTLIST (%fontstyle;|%phrase;)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT (SUB|SUP) - - (%inline;)*    -- subscript, superscript -->
<!!ATTLIST (SUB|SUP)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT SPAN - - (%inline;)*         -- generic language/style container -->
<!!ATTLIST SPAN
  %attrs;                              -- %coreattrs, %i18n, %events --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT BDO - - (%inline;)*          -- I18N BiDi over-ride -->
<!!ATTLIST BDO
  %coreattrs;                          -- id, class, style, title --
  lang        %LanguageCode; #IMPLIED  -- language code --
  dir         (ltr|rtl)      #REQUIRED -- directionality --
  >


<!!ELEMENT BR - O EMPTY                 -- forced line break -->
<!!ATTLIST BR
  %coreattrs;                          -- id, class, style, title --
  >

<!!--================== HTML content models ===============================-->

<!!--
    HTML has two basic content models:

	%inline;     character level elements and text strings
	%block;      block-like elements e.g. paragraphs and lists
-->

<!!ENTITY % block
     "P | %heading; | %list; | %preformatted; | DL | DIV | NOSCRIPT |
      BLOCKQUOTE | FORM | HR | TABLE | FIELDSET | ADDRESS">

<!!ENTITY % flow "%block; | %inline;">

<!!--=================== Document Body ====================================-->

<!!ELEMENT BODY O O (%block;|SCRIPT)+ +(INS|DEL) -- document body -->
<!!ATTLIST BODY
  %attrs;                              -- %coreattrs, %i18n, %events --
  onload          %Script;   #IMPLIED  -- the document has been loaded --
  onunload        %Script;   #IMPLIED  -- the document has been removed --
  >

<!!ELEMENT ADDRESS - - (%inline;)* -- information on author -->
<!!ATTLIST ADDRESS
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT DIV - - (%flow;)*            -- generic language/style container -->
<!!ATTLIST DIV
  %attrs;                              -- %coreattrs, %i18n, %events --
  %reserved;                           -- reserved for possible future use --
  >


<!!--================== The Anchor Element ================================-->

<!!ENTITY % Shape "(rect|circle|poly|default)">
<!!ENTITY % Coords "CDATA" -- comma-separated list of lengths -->

<!!ELEMENT A - - (%inline;)* -(A)       -- anchor -->
<!!ATTLIST A
  %attrs;                              -- %coreattrs, %i18n, %events --
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  type        %ContentType;  #IMPLIED  -- advisory content type --
  name        CDATA          #IMPLIED  -- named link end --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  hreflang    %LanguageCode; #IMPLIED  -- language code --
  rel         %LinkTypes;    #IMPLIED  -- forward link types --
  rev         %LinkTypes;    #IMPLIED  -- reverse link types --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  shape       %Shape;        rect      -- for use with client-side image maps --
  coords      %Coords;       #IMPLIED  -- for use with client-side image maps --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!--================== Client-side image maps ============================-->

<!!-- These can be placed in the same document or grouped in a
     separate document although this isn''t yet widely supported -->

<!!ELEMENT MAP - - ((%block;) | AREA)+ -- client-side image map -->
<!!ATTLIST MAP
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #REQUIRED -- for reference by usemap --
  >

<!!ELEMENT AREA - O EMPTY               -- client-side image map area -->
<!!ATTLIST AREA
  %attrs;                              -- %coreattrs, %i18n, %events --
  shape       %Shape;        rect      -- controls interpretation of coords --
  coords      %Coords;       #IMPLIED  -- comma-separated list of lengths --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  nohref      (nohref)       #IMPLIED  -- this region has no action --
  alt         %Text;         #REQUIRED -- short description --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!--================== The LINK Element ==================================-->

<!!--
  Relationship values can be used in principle:

   a) for document specific toolbars/menus when used
      with the LINK element in document head e.g.
	start, contents, previous, next, index, end, help
   b) to link to a separate style sheet (rel=stylesheet)
   c) to make a link to a script (rel=script)
   d) by stylesheets to control how collections of
      html nodes are rendered into printed documents
   e) to make a link to a printable version of this document
      e.g. a postscript or pdf version (rel=alternate media=print)
-->

<!!ELEMENT LINK - O EMPTY               -- a media-independent link -->
<!!ATTLIST LINK
  %attrs;                              -- %coreattrs, %i18n, %events --
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  href        %URI;          #IMPLIED  -- URI for linked resource --
  hreflang    %LanguageCode; #IMPLIED  -- language code --
  type        %ContentType;  #IMPLIED  -- advisory content type --
  rel         %LinkTypes;    #IMPLIED  -- forward link types --
  rev         %LinkTypes;    #IMPLIED  -- reverse link types --
  media       %MediaDesc;    #IMPLIED  -- for rendering on these media --
  >

<!!--=================== Images ===========================================-->

<!!-- Length defined in strict DTD for cellpadding/cellspacing -->
<!!ENTITY % Length "CDATA" -- nn for pixels or nn% for percentage length -->
<!!ENTITY % MultiLength "CDATA" -- pixel, percentage, or relative -->

<!![ %HTML.Frameset; [
<!!ENTITY % MultiLengths "CDATA" -- comma-separated list of MultiLength -->
]]>

<!!ENTITY % Pixels "CDATA" -- integer representing length in pixels -->


<!!-- To avoid problems with text-only UAs as well as
   to make image content understandable and navigable
   to users of non-visual UAs, you need to provide
   a description with ALT, and avoid server-side image maps -->
<!!ELEMENT IMG - O EMPTY                -- Embedded image -->
<!!ATTLIST IMG
  %attrs;                              -- %coreattrs, %i18n, %events --
  src         %URI;          #REQUIRED -- URI of image to embed --
  alt         %Text;         #REQUIRED -- short description --
  longdesc    %URI;          #IMPLIED  -- link to long description
					  (complements alt) --
  name        CDATA          #IMPLIED  -- name of image for scripting --
  height      %Length;       #IMPLIED  -- override height --
  width       %Length;       #IMPLIED  -- override width --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  ismap       (ismap)        #IMPLIED  -- use server-side image map --
  >

<!!-- USEMAP points to a MAP element which may be in this document
  or an external document, although the latter is not widely supported -->

<!!--==================== OBJECT ======================================-->
<!!--
  OBJECT is used to embed objects as part of HTML pages
  PARAM elements should precede other content. SGML mixed content
  model technicality precludes specifying this formally ...
-->

<!!ELEMENT OBJECT - - (PARAM | %flow;)*
 -- generic embedded object -->
<!!ATTLIST OBJECT
  %attrs;                              -- %coreattrs, %i18n, %events --
  declare     (declare)      #IMPLIED  -- declare but don''t instantiate flag --
  classid     %URI;          #IMPLIED  -- identifies an implementation --
  codebase    %URI;          #IMPLIED  -- base URI for classid, data, archive--
  data        %URI;          #IMPLIED  -- reference to object''s data --
  type        %ContentType;  #IMPLIED  -- content type for data --
  codetype    %ContentType;  #IMPLIED  -- content type for code --
  archive     CDATA          #IMPLIED  -- space-separated list of URIs --
  standby     %Text;         #IMPLIED  -- message to show while loading --
  height      %Length;       #IMPLIED  -- override height --
  width       %Length;       #IMPLIED  -- override width --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  name        CDATA          #IMPLIED  -- submit as part of form --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT PARAM - O EMPTY              -- named property value -->
<!!ATTLIST PARAM
  id          ID             #IMPLIED  -- document-wide unique id --
  name        CDATA          #REQUIRED -- property name --
  value       CDATA          #IMPLIED  -- property value --
  valuetype   (DATA|REF|OBJECT) DATA   -- How to interpret value --
  type        %ContentType;  #IMPLIED  -- content type for value
					  when valuetype=ref --
  >


<!!--=================== Horizontal Rule ==================================-->

<!!ELEMENT HR - O EMPTY -- horizontal rule -->
<!!ATTLIST HR
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--=================== Paragraphs =======================================-->

<!!ELEMENT P - O (%inline;)*            -- paragraph -->
<!!ATTLIST P
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--=================== Headings =========================================-->

<!!--
  There are six levels of headings from H1 (the most important)
  to H6 (the least important).
-->

<!!ELEMENT (%heading;)  - - (%inline;)* -- heading -->
<!!ATTLIST (%heading;)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--=================== Preformatted Text ================================-->

<!!-- excludes markup for images and changes in font size -->
<!!ENTITY % pre.exclusion "IMG|OBJECT|BIG|SMALL|SUB|SUP">

<!!ELEMENT PRE - - (%inline;)* -(%pre.exclusion;) -- preformatted text -->
<!!ATTLIST PRE
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--===================== Inline Quotes ==================================-->

<!!ELEMENT Q - - (%inline;)*            -- short inline quotation -->
<!!ATTLIST Q
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- URI for source document or msg --
  >

<!!--=================== Block-like Quotes ================================-->

<!!ELEMENT BLOCKQUOTE - - (%block;|SCRIPT)+ -- long quotation -->
<!!ATTLIST BLOCKQUOTE
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- URI for source document or msg --
  >

<!!--=================== Inserted/Deleted Text ============================-->


<!!-- INS/DEL are handled by inclusion on BODY -->
<!!ELEMENT (INS|DEL) - - (%flow;)*      -- inserted text, deleted text -->
<!!ATTLIST (INS|DEL)
  %attrs;                              -- %coreattrs, %i18n, %events --
  cite        %URI;          #IMPLIED  -- info on reason for change --
  datetime    %Datetime;     #IMPLIED  -- date and time of change --
  >

<!!--=================== Lists ============================================-->

<!!-- definition lists - DT for term, DD for its definition -->

<!!ELEMENT DL - - (DT|DD)+              -- definition list -->
<!!ATTLIST DL
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT DT - O (%inline;)*           -- definition term -->
<!!ELEMENT DD - O (%flow;)*             -- definition description -->
<!!ATTLIST (DT|DD)
  %attrs;                              -- %coreattrs, %i18n, %events --
  >


<!!ELEMENT OL - - (LI)+                 -- ordered list -->
<!!ATTLIST OL
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!-- Unordered Lists (UL) bullet styles -->
<!!ELEMENT UL - - (LI)+                 -- unordered list -->
<!!ATTLIST UL
  %attrs;                              -- %coreattrs, %i18n, %events --
  >



<!!ELEMENT LI - O (%flow;)*             -- list item -->
<!!ATTLIST LI
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--================ Forms ===============================================-->
<!!ELEMENT FORM - - (%block;|SCRIPT)+ -(FORM) -- interactive form -->
<!!ATTLIST FORM
  %attrs;                              -- %coreattrs, %i18n, %events --
  action      %URI;          #REQUIRED -- server-side form handler --
  method      (GET|POST)     GET       -- HTTP method used to submit the form--
  enctype     %ContentType;  "application/x-www-form-urlencoded"
  accept      %ContentTypes; #IMPLIED  -- list of MIME types for file upload --
  name        CDATA          #IMPLIED  -- name of form for scripting --
  onsubmit    %Script;       #IMPLIED  -- the form was submitted --
  onreset     %Script;       #IMPLIED  -- the form was reset --
  accept-charset %Charsets;  #IMPLIED  -- list of supported charsets --
  >

<!!-- Each label must not contain more than ONE field -->
<!!ELEMENT LABEL - - (%inline;)* -(LABEL) -- form field label text -->
<!!ATTLIST LABEL
  %attrs;                              -- %coreattrs, %i18n, %events --
  for         IDREF          #IMPLIED  -- matches field ID value --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  >

<!!ENTITY % InputType
  "(TEXT | PASSWORD | CHECKBOX |
    RADIO | SUBMIT | RESET |
    FILE | HIDDEN | IMAGE | BUTTON)"
   >

<!!-- attribute name required for all but submit and reset -->
<!!ELEMENT INPUT - O EMPTY              -- form control -->
<!!ATTLIST INPUT
  %attrs;                              -- %coreattrs, %i18n, %events --
  type        %InputType;    TEXT      -- what kind of widget is needed --
  name        CDATA          #IMPLIED  -- submit as part of form --
  value       CDATA          #IMPLIED  -- Specify for radio buttons and checkboxes --
  checked     (checked)      #IMPLIED  -- for radio buttons and check boxes --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  readonly    (readonly)     #IMPLIED  -- for text and passwd --
  size        CDATA          #IMPLIED  -- specific to each type of field --
  maxlength   NUMBER         #IMPLIED  -- max chars for text fields --
  src         %URI;          #IMPLIED  -- for fields with images --
  alt         CDATA          #IMPLIED  -- short description --
  usemap      %URI;          #IMPLIED  -- use client-side image map --
  ismap       (ismap)        #IMPLIED  -- use server-side image map --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onselect    %Script;       #IMPLIED  -- some text was selected --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  accept      %ContentTypes; #IMPLIED  -- list of MIME types for file upload --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT SELECT - - (OPTGROUP|OPTION)+ -- option selector -->
<!!ATTLIST SELECT
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED  -- field name --
  size        NUMBER         #IMPLIED  -- rows visible --
  multiple    (multiple)     #IMPLIED  -- default is single selection --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  %reserved;                           -- reserved for possible future use --
  >

<!!ELEMENT OPTGROUP - - (OPTION)+ -- option group -->
<!!ATTLIST OPTGROUP
  %attrs;                              -- %coreattrs, %i18n, %events --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  label       %Text;         #REQUIRED -- for use in hierarchical menus --
  >

<!!ELEMENT OPTION - O (#PCDATA)         -- selectable choice -->
<!!ATTLIST OPTION
  %attrs;                              -- %coreattrs, %i18n, %events --
  selected    (selected)     #IMPLIED
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  label       %Text;         #IMPLIED  -- for use in hierarchical menus --
  value       CDATA          #IMPLIED  -- defaults to element content --
  >

<!!ELEMENT TEXTAREA - - (#PCDATA)       -- multi-line text field -->
<!!ATTLIST TEXTAREA
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED
  rows        NUMBER         #REQUIRED
  cols        NUMBER         #REQUIRED
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  readonly    (readonly)     #IMPLIED
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  onselect    %Script;       #IMPLIED  -- some text was selected --
  onchange    %Script;       #IMPLIED  -- the element value was changed --
  %reserved;                           -- reserved for possible future use --
  >

<!!--
  #PCDATA is to solve the mixed content problem,
  per specification only whitespace is allowed there!!
 -->
<!!ELEMENT FIELDSET - - (#PCDATA,LEGEND,(%flow;)*) -- form control group -->
<!!ATTLIST FIELDSET
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!ELEMENT LEGEND - - (%inline;)*       -- fieldset legend -->

<!!ATTLIST LEGEND
  %attrs;                              -- %coreattrs, %i18n, %events --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  >

<!!ELEMENT BUTTON - -
     (%flow;)* -(A|%formctrl;|FORM|FIELDSET)
     -- push button -->
<!!ATTLIST BUTTON
  %attrs;                              -- %coreattrs, %i18n, %events --
  name        CDATA          #IMPLIED
  value       CDATA          #IMPLIED  -- sent to server when submitted --
  type        (button|submit|reset) submit -- for use as form button --
  disabled    (disabled)     #IMPLIED  -- unavailable in this context --
  tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
  accesskey   %Character;    #IMPLIED  -- accessibility key character --
  onfocus     %Script;       #IMPLIED  -- the element got the focus --
  onblur      %Script;       #IMPLIED  -- the element lost the focus --
  %reserved;                           -- reserved for possible future use --
  >

<!!--======================= Tables =======================================-->

<!!-- IETF HTML table standard, see [RFC1942] -->

<!!--
 The BORDER attribute sets the thickness of the frame around the
 table. The default units are screen pixels.

 The FRAME attribute specifies which parts of the frame around
 the table should be rendered. The values are not the same as
 CALS to avoid a name clash with the VALIGN attribute.

 The value "border" is included for backwards compatibility with
 <TABLE BORDER> which yields frame=border and border=implied
 For <TABLE BORDER=1> you get border=1 and frame=implied. In this
 case, it is appropriate to treat this as frame=border for backwards
 compatibility with deployed browsers.
-->
<!!ENTITY % TFrame "(void|above|below|hsides|lhs|rhs|vsides|box|border)">

<!!--
 The RULES attribute defines which rules to draw between cells:

 If RULES is absent then assume:
     "none" if BORDER is absent or BORDER=0 otherwise "all"
-->

<!!ENTITY % TRules "(none | groups | rows | cols | all)">

<!!-- horizontal placement of table relative to document -->
<!!ENTITY % TAlign "(left|center|right)">

<!!-- horizontal alignment attributes for cell contents -->
<!!ENTITY % cellhalign
  "align      (left|center|right|justify|char) #IMPLIED
   char       %Character;    #IMPLIED  -- alignment char, e.g. char='':'' --
   charoff    %Length;       #IMPLIED  -- offset for alignment char --"
  >

<!!-- vertical alignment attributes for cell contents -->
<!!ENTITY % cellvalign
  "valign     (top|middle|bottom|baseline) #IMPLIED"
  >

<!!ELEMENT TABLE - -
     (CAPTION?, (COL*|COLGROUP*), THEAD?, TFOOT?, TBODY+)>
<!!ELEMENT CAPTION  - - (%inline;)*     -- table caption -->
<!!ELEMENT THEAD    - O (TR)+           -- table header -->
<!!ELEMENT TFOOT    - O (TR)+           -- table footer -->
<!!ELEMENT TBODY    O O (TR)+           -- table body -->
<!!ELEMENT COLGROUP - O (COL)*          -- table column group -->
<!!ELEMENT COL      - O EMPTY           -- table column -->
<!!ELEMENT TR       - O (TH|TD)+        -- table row -->
<!!ELEMENT (TH|TD)  - O (%flow;)*       -- table header cell, table data cell-->

<!!ATTLIST TABLE                        -- table element --
  %attrs;                              -- %coreattrs, %i18n, %events --
  summary     %Text;         #IMPLIED  -- purpose/structure for speech output--
  width       %Length;       #IMPLIED  -- table width --
  border      %Pixels;       #IMPLIED  -- controls frame width around table --
  frame       %TFrame;       #IMPLIED  -- which parts of frame to render --
  rules       %TRules;       #IMPLIED  -- rulings between rows and cols --
  cellspacing %Length;       #IMPLIED  -- spacing between cells --
  cellpadding %Length;       #IMPLIED  -- spacing within cells --
  %reserved;                           -- reserved for possible future use --
  datapagesize CDATA         #IMPLIED  -- reserved for possible future use --
  >


<!!ATTLIST CAPTION
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--
COLGROUP groups a set of COL elements. It allows you to group
several semantically related columns together.
-->
<!!ATTLIST COLGROUP
  %attrs;                              -- %coreattrs, %i18n, %events --
  span        NUMBER         1         -- default number of columns in group --
  width       %MultiLength;  #IMPLIED  -- default width for enclosed COLs --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!--
 COL elements define the alignment properties for cells in
 one or more columns.

 The WIDTH attribute specifies the width of the columns, e.g.

     width=64        width in screen pixels
     width=0.5*      relative width of 0.5

 The SPAN attribute causes the attributes of one
 COL element to apply to more than one column.
-->
<!!ATTLIST COL                          -- column groups and properties --
  %attrs;                              -- %coreattrs, %i18n, %events --
  span        NUMBER         1         -- COL attributes affect N columns --
  width       %MultiLength;  #IMPLIED  -- column width specification --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!--
    Use THEAD to duplicate headers when breaking table
    across page boundaries, or for static headers when
    TBODY sections are rendered in scrolling panel.

    Use TFOOT to duplicate footers when breaking table
    across page boundaries, or for static footers when
    TBODY sections are rendered in scrolling panel.

    Use multiple TBODY sections when rules are needed
    between groups of table rows.
-->
<!!ATTLIST (THEAD|TBODY|TFOOT)          -- table section --
  %attrs;                              -- %coreattrs, %i18n, %events --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >

<!!ATTLIST TR                           -- table row --
  %attrs;                              -- %coreattrs, %i18n, %events --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >


<!!-- Scope is simpler than headers attribute for common tables -->
<!!ENTITY % Scope "(row|col|rowgroup|colgroup)">

<!!-- TH is for headers, TD for data, but for cells acting as both use TD -->
<!!ATTLIST (TH|TD)                      -- header or data cell --
  %attrs;                              -- %coreattrs, %i18n, %events --
  abbr        %Text;         #IMPLIED  -- abbreviation for header cell --
  axis        CDATA          #IMPLIED  -- comma-separated list of related headers--
  headers     IDREFS         #IMPLIED  -- list of id''s for header cells --
  scope       %Scope;        #IMPLIED  -- scope covered by header cells --
  rowspan     NUMBER         1         -- number of rows spanned by cell --
  colspan     NUMBER         1         -- number of cols spanned by cell --
  %cellhalign;                         -- horizontal alignment in cells --
  %cellvalign;                         -- vertical alignment in cells --
  >


<!!--================ Document Head =======================================-->
<!!-- %head.misc; defined earlier on as "SCRIPT|STYLE|META|LINK|OBJECT" -->
<!!ENTITY % head.content "TITLE & BASE?">

<!!ELEMENT HEAD O O (%head.content;) +(%head.misc;) -- document head -->
<!!ATTLIST HEAD
  %i18n;                               -- lang, dir --
  profile     %URI;          #IMPLIED  -- named dictionary of meta info --
  >

<!!-- The TITLE element is not considered part of the flow of text.
       It should be displayed, for example as the page header or
       window title. Exactly one title is required per document.
    -->
<!!ELEMENT TITLE - - (#PCDATA) -(%head.misc;) -- document title -->
<!!ATTLIST TITLE %i18n>


<!!ELEMENT BASE - O EMPTY               -- document base URI -->
<!!ATTLIST BASE
  href        %URI;          #REQUIRED -- URI that acts as base URI --
  >

<!!ELEMENT META - O EMPTY               -- generic metainformation -->
<!!ATTLIST META
  %i18n;                               -- lang, dir, for use with content --
  http-equiv  NAME           #IMPLIED  -- HTTP response header name  --
  name        NAME           #IMPLIED  -- metainformation name --
  content     CDATA          #REQUIRED -- associated information --
  scheme      CDATA          #IMPLIED  -- select form of content --
  >

<!!ELEMENT STYLE - - %StyleSheet        -- style info -->
<!!ATTLIST STYLE
  %i18n;                               -- lang, dir, for use with title --
  type        %ContentType;  #REQUIRED -- content type of style language --
  media       %MediaDesc;    #IMPLIED  -- designed for use with these media --
  title       %Text;         #IMPLIED  -- advisory title --
  >

<!!ELEMENT SCRIPT - - %Script;          -- script statements -->
<!!ATTLIST SCRIPT
  charset     %Charset;      #IMPLIED  -- char encoding of linked resource --
  type        %ContentType;  #REQUIRED -- content type of script language --
  src         %URI;          #IMPLIED  -- URI for an external script --
  defer       (defer)        #IMPLIED  -- UA may defer execution of script --
  event       CDATA          #IMPLIED  -- reserved for possible future use --
  for         %URI;          #IMPLIED  -- reserved for possible future use --
  >

<!!ELEMENT NOSCRIPT - - (%block;)+
  -- alternate content container for non script-based rendering -->
<!!ATTLIST NOSCRIPT
  %attrs;                              -- %coreattrs, %i18n, %events --
  >

<!!--================ Document Structure ==================================-->
<!!ENTITY % html.content "HEAD, BODY">

<!!ELEMENT HTML O O (%html.content;)    -- document root element -->
<!!ATTLIST HTML
  %i18n;                               -- lang, dir --
  >'
! !

!TfHTMLParser class methodsFor:'defaults'!

defaultEncoding

	^ Locale current defaultStreamEncoder name
! !

!TfHTMLParser methodsFor:'DTD processing'!

dtdFile: newURI
	" ignored for the moment "
!

externalID: usage
	"Usage may be #docType, #entity, or #notation.
	DocType is treated specially, since PE references are not allowed.
	Notation is treated specially since the system identifier of the
		PUBLIC form is optional."

	| lit2 lit1 forceSpace skipSpace |
	forceSpace := [usage == #docType
					ifTrue: [self forceSpace]
					ifFalse: [self forceSpaceInDTD]].
	skipSpace := [usage == #docType
					ifTrue: [self skipSpace]
					ifFalse: [self skipSpaceInDTD]].
	^(self skipIf: 'SYSTEM')
		ifTrue:
			[forceSpace value.
			lit2 := self systemLiteral.
			Array with: nil with: lit2]
		ifFalse: [(self skipIf: 'PUBLIC')
			ifTrue:
				[forceSpace value.
				lit1 := self pubIdLiteral.
				usage == #notation
					ifTrue:
						[(skipSpace value and:
								[hereChar = $' or: [hereChar = $"]])
							ifTrue: [lit2 := self systemLiteral]
							ifFalse: [lit2 := nil]]
					ifFalse:
						[ self skipSpace.
						hereChar = $>
							ifFalse: [ lit2 := self systemLiteral ]].
				Array with: lit1 with: lit2]
			ifFalse:
				[nil]]
!

scanDTD

	sourceStack usedAsExternal: true.
	self getNextChar.
	[self atEnd ] whileFalse: [ self dtdEntry ]
! !

!TfHTMLParser methodsFor:'HTML'!

addHTML4Entities

    | entName |
    self html4Entities do: [ :entNameValueArray | 
        entName := entNameValueArray first.
        entName := entName copyFrom: 2 to: entName size -1.
        self dtd 
            generalEntityAt: entName
            put: (XML::GeneralEntity new name: entName; text: entNameValueArray last)
            from: self.
    ].
!

html4Entities
	^ #(
#('&quot;'      '&#34;')
#('&amp;'       '&#38;')
#('&lt;'        '&#60;')
#('&gt;'        '&#62;')
#('&nbsp;'      '&#160;')
#('&iexcl;'     '&#161;')
#('&curren;'    '&#164;')
#('&cent;'      '&#162;')
#('&pound;'     '&#163;')
#('&yen;'       '&#165;')
#('&brvbar;'    '&#166;')
#('&sect;'      '&#167;')
#('&uml;'       '&#168;')
#('&copy;'      '&#169;')
#('&ordf;'      '&#170;')
#('&laquo;'     '&#171;')
#('&not;'       '&#172;')
#('&shy;'       '&#173;')
#('&reg;'       '&#174;')
#('&macr;'      '&#175;')
#('&deg;'       '&#176;')
#('&plusmn;'    '&#177;')
#('&sup2;'      '&#178;')
#('&sup3;'      '&#179;')
#('&acute;'     '&#180;')
#('&micro;'     '&#181;')
#('&para;'      '&#182;')
#('&middot;'    '&#183;')
#('&cedil;'     '&#184;')
#('&sup1;'      '&#185;')
#('&ordm;'      '&#186;')
#('&raquo;'     '&#187;')
#('&frac14;'    '&#188;')
#('&frac12;'    '&#189;')
#('&frac34;'    '&#190;')
#('&iquest;'    '&#191;')
#('&times;'     '&#215;')
#('&divide;'    '&#247;')
#('&Agrave;'    '&#192;')
#('&Aacute;'    '&#193;')
#('&Acirc;'     '&#194;')
#('&Atilde;'    '&#195;')
#('&Auml;'      '&#196;')
#('&Aring;'     '&#197;')
#('&AElig;'     '&#198;')
#('&Ccedil;'    '&#199;')
#('&Egrave;'    '&#200;')
#('&Eacute;'    '&#201;')
#('&Ecirc;'     '&#202;')
#('&Euml;'      '&#203;')
#('&Igrave;'    '&#204;')
#('&Iacute;'    '&#205;')
#('&Icirc;'     '&#206;')
#('&Iuml;'      '&#207;')
#('&ETH;'       '&#208;')
#('&Ntilde;'    '&#209;')
#('&Ograve;'    '&#210;')
#('&Oacute;'    '&#211;')
#('&Ocirc;'     '&#212;')
#('&Otilde;'    '&#213;')
#('&Ouml;'      '&#214;')
#('&Oslash;'    '&#216;')
#('&Ugrave;'    '&#217;')
#('&Uacute;'    '&#218;')
#('&Ucirc;'     '&#219;')
#('&Uuml;'      '&#220;')
#('&Yacute;'    '&#221;')
#('&THORN;'     '&#222;')
#('&szlig;'     '&#223;')
#('&agrave;'    '&#224;')
#('&aacute;'    '&#225;')
#('&acirc;'     '&#226;')
#('&atilde;'    '&#227;')
#('&auml;'      '&#228;')
#('&aring;'     '&#229;')
#('&aelig;'     '&#230;')
#('&ccedil;'    '&#231;')
#('&egrave;'    '&#232;')
#('&eacute;'    '&#233;')
#('&ecirc;'     '&#234;')
#('&euml;'      '&#235;')
#('&igrave;'    '&#236;')
#('&iacute;'    '&#237;')
#('&icirc;'     '&#238;')
#('&iuml;'      '&#239;')
#('&eth;'       '&#240;')
#('&ntilde;'    '&#241;')
#('&ograve;'    '&#242;')
#('&oacute;'    '&#243;')
#('&ocirc;'     '&#244;')
#('&otilde;'    '&#245;')
#('&ouml;'      '&#246;')
#('&oslash;'    '&#248;')
#('&ugrave;'    '&#249;')
#('&uacute;'    '&#250;')
#('&ucirc;'     '&#251;')
#('&uuml;'      '&#252;')
#('&yacute;'    '&#253;')
#('&thorn;'     '&#254;')
#('&yuml;'      '&#255;')
#('&OElig;'     '&#338;')
#('&oelig;'     '&#339;')
#('&Scaron;'    '&#352;')
#('&scaron;'    '&#353;')
#('&Yuml;'      '&#376;')
#('&circ;'      '&#710;')
#('&tilde;'     '&#732;')
#('&ensp;'      '&#8194;')
#('&emsp;'      '&#8195;')
#('&thinsp;'    '&#8201;')
#('&zwnj;'      '&#8204;')
#('&zwj;'       '&#8205;')
#('&lrm;'       '&#8206;')
#('&rlm;'       '&#8207;')
#('&ndash;'     '&#8211;')
#('&mdash;'     '&#8212;')
#('&lsquo;'     '&#8216;')
#('&rsquo;'     '&#8217;')
#('&sbquo;'     '&#8218;')
#('&ldquo;'     '&#8220;')
#('&rdquo;'     '&#8221;')
#('&bdquo;'     '&#8222;')
#('&dagger;'    '&#8224;')
#('&Dagger;'    '&#8225;')
#('&permil;'    '&#8240;')
#('&lsaquo;'    '&#8249;')
#('&rsaquo;'    '&#8250;')
#('&euro;'      '&#8364;')
)
! !

!TfHTMLParser methodsFor:'api'!

scanDocument
        " modification: don't complain if the document stream is not at end (this usually means
        that the parser is out of sync) "

        ^ [
                builder driver "sax" startDocument.
                self prolog.
                self pushDummyRoot.
                [
                        self getElement.
                        self misc
                ] whileTrue.
                self popDummyRoot.
                self endDocument.
                builder driver "sax" endDocument.
                self document
        ] ensure: [
                self closeAllFiles
        ]
! !

!TfHTMLParser methodsFor:'attribute def processing'!

attListDecl
	" modification: multiple element names: <attlist (a|b|c)>"

	| str1 attr names multiple |
	str1 := currentSource.
	^(self skipIf: '<!!ATTLIST')
		ifTrue:
			[self forceSpaceInDTD.
			names := Set new.
			multiple := self skipIf: '('.
			[       hereChar = $% ifTrue: [ self PERef: #data ].
				names add: self getQualifiedName.
				self skipIf: ';'.
				multiple
					ifTrue: [
						self skipSpaceInDTD.
						self skipIf: '|' ]
					ifFalse: [ false ].
			] whileTrue: [
				self skipSpaceInDTD.
			].
			multiple ifTrue: [ self skipIf: ')' ].
			[self skipSpaceInDTD.
			self skipIf: '>']
				whileFalse:
					[self skipSpaceInDTD.
					self PERef: #data.
					attr := AttributeDef new name: self getQualifiedName.
					self forceSpaceInDTD.
					attr type: self attType.
					attr type isExternal: self inInternalSubset not.
					self forceSpaceInDTD.
					attr default: (self defaultDeclType: attr type).
					self isValidating ifTrue: [attr selfValidateFor: self].
					self checkReservedAttributes: attr name asString
						type: attr type
						value: attr default.
					self ignore
						ifFalse: [
							names do: [ : nm | self dtd attributeFor: nm subKey: attr name put: attr from: self ].
						]].
			str1 == lastSource
				ifFalse: [self invalid: (#declarationsNestingInPEntity << #xml >> 'Improper nesting of declarations within a parameter entity')].
			true]
		ifFalse: [false]
! !

!TfHTMLParser methodsFor:'attribute processing'!

attValue: attType inDTD: isInDTD

	| aQuote s str1 sawSpace needsSpace isCDATA count |
	isCDATA := attType class == CDATA_AT.
" modification: allow unquoted attributes >> "
	(hereChar = $' or: [hereChar = $"])
		ifTrue: [
			aQuote := hereChar.
			self getNextChar.
		].
	buffer reset.
	s := currentSource.
" << modification "
	count := 0.
	sawSpace := true.
	needsSpace := false.
	[ ((aQuote notNil and: [ hereChar = aQuote])
		or: [ aQuote isNil and: [ self isAttributeValueDelimiter: hereChar ]])
			and: [s = currentSource] ]
		whileFalse:
			[hereChar == nil
				ifTrue: [self malformed: #attrCloseQuoteMissing << #xml >> 'No close quote found for attribute value'].
" modification: allow '>' in attributes >> "
"                       hereChar = $<
				ifTrue: [self malformed: (#lessThanNotPermittedInAttributes << #xml >> '< not permitted in attribute values; use &lt;')]."
" <<< modification "
			hereChar = $&
				ifTrue:
					[str1 := currentSource.
					(self skipIf: '&#')
						ifTrue:
							[needsSpace ifTrue: [buffer space].
							needsSpace := sawSpace := false.
							count := count+1.
							self charEntity: buffer startedIn: str1]
						ifFalse: [self getNextChar; generalEntityInText: buffer canBeExternal: false]]
				ifFalse: [(isInDTD and: [self PERef: #data])
					ifFalse:
						[hereChar asInteger <= 16r20
							ifTrue: [isCDATA
								ifTrue: [buffer space]
								ifFalse: [sawSpace ifFalse: [sawSpace := needsSpace := true]]]
							ifFalse:
								[needsSpace ifTrue: [buffer space].
								needsSpace := sawSpace := false.
								buffer nextPut: hereChar].
						count := count+1.
						self getNextChar]]].
	(self isValidating and: [self isDeclaredStandalone
			and: [count ~= buffer position and: [attType isExternal]]])
		ifTrue: [self invalid: #NotStandalone << #xml >> 'This document is not standalone'].
" modification: allow unquoted attributes >> "
	aQuote isNil
		ifFalse: [ self getNextChar ].
" << modification "
	^buffer contents
!

attributeFor: elementTag

	| nm value |
	nm := self getQualifiedName.
	self skipSpace.
" modification: allow attributes without values. expand to name=name, as proposed by XHTML >> "
	((self skipIf: '=') or: [ hereChar = $' or: [hereChar = $" ]])
		ifTrue: [
			self skipSpace.
			value := self attValue: (self dtd attributeTypeFor: elementTag subKey: nm from: self)
					inDTD: false.
			value == nil ifTrue: [self malformed: (#expectedQuotedAttributeValue << #xml >> 'A quoted value for the attribute was expected, but not found')].
		] ifFalse: [
			value := nm asString.
			(hereChar = $> or: [ hereChar = $< ])
				ifFalse: [
					"currentSource skip: -2.
					self getNextChar."
				].
		].
" << modification "
	self checkReservedAttributes: nm asString
		type: nil
		value: value.
	^Attribute name: nm value: value
!

processAttributes: nm

        | attributes nextAttribute |
        attributes := nil.
        [self skipSpace.
        self isValidNameStart: hereChar]
                whileTrue: [
" Modification: don't complain about missing space or non-unique attributes >> "
                        attributes == nil ifTrue: [attributes := OrderedCollection new].
                        nextAttribute := self attributeFor: nm.
                        (attributes contains: [ :att | att key = nextAttribute key ])
                                ifFalse: [ attributes add: nextAttribute ].
" << Modification "
                ].
        self isValidating
                ifTrue: [attributes := self validateAttributes: attributes for: nm].
        attributes := builder "self" resolveNamespaces: attributes.
        ^attributes
! !

!TfHTMLParser methodsFor:'element processing'!

charEntity: data startedIn: str1

	| base digit n d fakedSemicolon |
	hereChar = $x
		ifTrue:
			[base := 16.
			digit := (#expectedHexDigit << #xml >> 'Expected to find a hex digit').
			self getNextChar]
		ifFalse:
			[base := 10.
			digit := (#expectedDigit << #xml >> 'Expected to find a digit')].
	n := 0.
" modification: support for missing semicolon >>> "
	fakedSemicolon := false.
	[fakedSemicolon or: [ hereChar = $; ]]
		whileFalse:
			[d := hereChar digitValue.
			(d >= 0 and: [d < base])
				ifTrue: [
					n := n * base + d.
					self getNextChar
				] ifFalse: [ " fake semicolon "
					fakedSemicolon := true.
				]
		].
" << modification "
	str1 = currentSource
		ifFalse: [self malformed: (#nestingCharacterEntitiesInEntities << #xml >> 'Character entities must nest properly inside other entities')].
	"Originally we tested ch to make sure it was less than 16r110000,
	but now CharacterClasses' implementation of #at: answers 0 for
	large values of ch. If primitive failure code can not be trusted to do
	this, then the bounds check would have to be added back."
	(CharacterClasses at: n+1) = 0
		ifTrue: [sax fatalError: (BadCharacterSignal new messageText:((#illegalUnicodeCharacter << #xml >> 'A character with Unicode value <1p> is not legal') expandMacrosWith: n))].
	data nextPut: (self convertToCharacter: n).
" modification: support for missing semicolon >>> "
	fakedSemicolon ifFalse: [ self getNextChar ]
" << modification "
!

closeTag: tag
	" modifications: allow
		- standalone tags
		- closing tags that were never opened "

	| nm stopTags |
	nm := self getQualifiedName.
	nm := self correctTag: nm.
	self skipSpace.
	self skipIf: '>'.
	nm = tag
		ifTrue: [       self endTag: nm ]
		ifFalse: [
			stopTags := implicitStopTags at: nm ifAbsent: [ #() ].
			(self isTagOpen: nm)
				ifTrue: [
					self openedTagsUpTo: nm stopAt: stopTags do: [ :openTag |
						self endTag: openTag
					].
				].
		].
!

completeComment: str1

	| str comment size |
	buffer reset.
	[str := self upToAll: '->'.
	str last = $-]
		whileFalse:
			[buffer nextPutAll: str; nextPutAll: '->'].
	buffer nextPutAll: (str copyFrom: 1 to: str size-1).
	comment := buffer collection.
	size := buffer position.
" modification: don't care about hyphens "
	lastSource = str1
		ifFalse: [self malformed: (#nestingCommentsInEntities << #xml >> 'Comments must nest properly in entities')].
	self ignore ifFalse:
		[sax comment: comment from: 1 to: size]
!

completePI: str1
	"this is a complete hack in order to get by some MS office formatted docs that have xml decls
	smack in the middle of the html"

	| nm pi |
	nm := self getSimpleName.
	nm asLowercase = 'xml'
		ifTrue: [pi := ''.
				self upTo: $>]
		ifFalse: [      self skipSpace
			ifTrue:
				[pi := self upToAll: '?>']
			ifFalse:
				[pi := ''.
				self mustFind: '?>']].
	lastSource = str1
		ifFalse: [self malformed: (#nestingPIsInEntities << #xml >> 'Pprogramming instructions must nest properly in entities')].
	self ignore ifFalse: [sax processingInstruction: nm data: pi].
!

currentElement
        ^ builder currentElement
"/        ^ elementStack isEmpty
"/                ifTrue: [ nil ]
"/                ifFalse: [ elementStack last ]
!

elementAtPosition: startPosition

        | attributes nm str1 |
        str1 := currentSource.
        self mustFind: '<'.
        nm := self getQualifiedName.
" modfication: revise tag >> "
        self revisers do: [ :reviser | reviser reviseOpenTag: nm parser: self ].
" << modfication: revise tag "
        ('script' match: nm type)
                ifTrue: [ ^ self skipScript ].
        builder pushTag:nm.
        "/ self pushNewTag: nm.
        "/ latestID := nil.
        attributes := self processAttributes: nm.
        nm := self correctTag: nm.
        builder currentElement "elementStack last" definesNamespaces
                ifTrue: [builder currentElement "elementStack last" namespaces
                        keysAndValuesDo: [:qualifier :uri |
                                builder driver "sax" startPrefixMapping: qualifier uri: uri]].
        builder driver "sax" startElement: nm namespace
                localName: nm type
                qName: nm asString
                attributes: (attributes == nil ifTrue: [#()] ifFalse: [attributes]).
        attributes notNil
                ifTrue: [ self checkCharsetIn: nm type attributes: attributes ].
" modfication: revise tag >> "
        self revisers do: [ :reviser | reviser reviseOpenedTag: nm parser: self ].
" << modfication: revise tag "
        builder driver "sax" sourcePosition: startPosition inStream: str1.
        "/ latestID notNil ifTrue: [sax idOfElement: latestID].
        (self skipIf: '/>')
                ifTrue:
                        [str1 = lastSource ifFalse: [self expected: (#elementNestingInEntities << #xml >> 'elements properly nested within entities')].
" modfication >>"
                        self popTag: nm.
" << modfication "
                        builder driver "sax" endElement: nm namespace
                                localName: nm type
                                qName: nm asString]
                ifFalse: [(self skipIf: '>')
                        ifTrue:
                                [str1 = lastSource ifFalse: [self expected: (#elementNestingInEntities << #xml >> 'elements properly nested within entities')].
                                self elementContent: nm
                                        openedIn: str1]
                        ifFalse: [
" modfication: allow '<tag<nextTag>' >>"

                                hereChar = $<
                                        ifFalse: [
                                                self upTo: $<.
                                                hereChar := $<.
                                        ].
                                self elementContent: nm openedIn: str1
                        ]].
" << modfication "
" modfication: pop moved "
!

elementContent: tag openedIn: str
        " support standalone tags "

        | str1 braceCount |
        braceCount := 0.
        buffer := CharacterWriteStream on:String new.
        [
" modification: allow standalone tags >> "
        hereChar == nil
                ifTrue: [
                        self emitBuffer.
                        ^ self endTag: tag ].
" << modification "
        hereChar = $<
                ifTrue:
                        [braceCount := 0.
" modification >> "
                        self emitBuffer.
" << modification "
                        str1 := currentSource.
                        (self skipIf: '</')
                                ifTrue:
                                        [self closeTag: tag.
                                        str == lastSource
                                                ifFalse: [self malformed: (#elementNestingInEntities << #xml >> 'Elements must nest properly within entities')].
                                        ^self]
                                ifFalse: [(self skipIf: '<?')
                                        ifTrue: [self completePI: str1]
                                        ifFalse: [(self skipIf: '<!![CDATA[')
                                                ifTrue: [self completeCDATA: str1]
                                                ifFalse: [(self skipIf: '<!!--')
                                                        ifTrue: [self completeComment: str1]
                                                        ifFalse: [self getElement]]]].
                        buffer reset]
                ifFalse: [hereChar = $&
                        ifTrue:
                                [braceCount := 0.
                                str1 := currentSource.
                                (self skipIf: '&#')
                                        ifTrue: [self charEntity: buffer startedIn: str1]
                                        ifFalse: [self getNextChar; generalEntityInText: buffer canBeExternal: true]]
                        ifFalse:
                                [hereChar = $]
                                        ifTrue: [braceCount := braceCount+1]
                                        ifFalse:
                                                [(hereChar = $> and: [braceCount >= 2])
                                                        ifTrue: [self malformed: #embeddedCdataClose << #xml >> ']]> is not permitted in element content'].
                                                braceCount := 0].
                                buffer nextPut: hereChar.
                                self getNextChar]]] repeat
!

emitBuffer

        | data size textType |
        size := buffer position.
        size = 0 ifTrue: [ ^ self ].
        data := buffer collection.      "Not necessarily portable, but faster than #contents"
        textType := self
                validateText: data
                from: 1
                to: size
                testBlanks: true.
        textType == #whitespace
                ifTrue:
                        [(self isValidating and: [self isDeclaredStandalone and: [elementStack last isDefinedExternal]])
                                ifTrue: [self invalid: (#NotStandalone << #xml >> 'This document is not standalone')].
                        builder driver "sax" ignorableWhitespace: data from: 1 to: size]
                ifFalse: [builder driver "sax" characters: data from: 1 to: size].
        buffer reset.
!

endTag: aTag

        (builder elementStack isEmpty not and: [ builder elementStack last tag = aTag ])
                ifTrue: [
                        self emitBuffer.
                        builder elementStack last definesNamespaces
                                ifTrue: [builder elementStack last namespaces
                                        keysDo: [:qualifier |
                                                builder driver "sax" endPrefixMapping: qualifier]].
                        self revisers do: [ :reviser | reviser reviseCloseTag: aTag parser: self ].
                        builder driver "sax" endElement: aTag namespace localName: aTag type qName: aTag asString.
                        "/ self popTag.
                        self revisers do: [ :reviser | reviser reviseClosedTag: aTag parser: self ].
                ].
!

generalEntityInText: str canBeExternal: external

" modification: & is sometimes just a &, not the beginning of an entity >> "
	| str1 nm exp msg |
	(self isValidNameStart: hereChar)
		ifFalse: [
			str nextPut: $&.
			^ self.
		].
	str1 := lastSource.
	nm := self getSimpleName.
	hereChar = $;
		ifFalse: [
			str nextPut: $&; nextPutAll: nm.
			^ self.
		].
" << modification end "
	currentSource = str1
		ifFalse: [self malformed: (#nestingEntityRefsInEntities << #xml >> 'Entity references must nest properly within other entity references')].
	exp := self dtd generalEntityAt: nm.
	exp == nil
		ifTrue:
			[msg := (#undefinedGeneralEntity1 << #xml >> 'The general entity "<1s>" has not been defined')
							expandMacrosWith: nm.
			self isValidating
				ifTrue: [self invalid: msg]
				ifFalse: [self warn: msg].
			self shouldTestWFCEntityDeclared
				ifTrue: [self malformed: (#undefinedGeneralEntity2 << #xml >> 'General entity used but not defined')].
			"str nextPut: $&; nextPutAll: nm; nextPut: $;."
			self getNextChar]
		ifFalse:
			[(external or: [exp isExternal not])
				ifFalse: [self malformed: (#invalidExternalEntityUse << #xml >> 'External entity references are not permitted in attribute values')].
			(self isValidating and: [self isDeclaredStandalone
					and: [exp isDefinedExternally]])
				ifTrue: [self invalid: #NotStandalone << #xml >> 'This document is not standalone'].
			exp isParsed
				ifFalse: [self malformed: (#invalidUnparsedEntityUse << #xml >> 'References to unparsed entities other than in an attribute of type ENTITY are not permitted')].
			exp streamFor: self].
!

popTag: aTag

	elementStack last tag = aTag
		ifTrue: [ self popTag ]
!

validateText: characters from: start to: stop testBlanks: testBlanks

        | textType cl |
        textType := super validateText: characters from: start to: stop testBlanks: testBlanks.
        (textType = #characters
                        and: [ builder elementStack isEmpty not
                        and: [ (cl := TfElement elementClassFor: builder elementStack last tag) notNil
                        and: [ cl allowsPCData not ]]])
                ifTrue: [ textType = #whitespace ].
        ^ textType
! !

!TfHTMLParser methodsFor:'entity processing'!

entityDecl

	| nm def str |
	str := currentSource.
	^(self skipIf: '<!!ENTITY')
		ifTrue:
			[self forceSpace.
			hereChar = $%
				ifTrue:
					[self getNextChar; forceSpaceInDTD.
					nm := self getSimpleName.
					self forceSpaceInDTD.
					def := self peDef: nm.
					self ignore ifFalse: [self dtd parameterEntityAt: nm put: def from: self]]
				ifFalse:
					[self skipSpaceInDTD.
					nm := self getSimpleName.
					self forceSpaceInDTD.
					def := self entityDef: nm.
					self ignore ifFalse: [self dtd generalEntityAt: nm put: def from: self]].
			self skipSpaceInDTD.
" modification: ignore comments and other garbage >> "
			self getNextCharBehind: '>'.
" << modification "
			str == lastSource
				ifFalse: [self invalid: (#declarationsNestingInPEntity << #xml >> 'Improper nesting of declarations within a parameter entity')].
			true]
		ifFalse: [false]
! !

!TfHTMLParser methodsFor:'initialize'!

builder: anXMLNodeBuilder

" modified: HTML driver "
        Smalltalk isSmalltalkX ifTrue:[
            super builder: anXMLNodeBuilder
        ] ifFalse:[
            self saxDriver: (TfSAXDriver new builder: anXMLNodeBuilder)
        ].
!

encoder: aByteStreamEncoder

	encoder := aByteStreamEncoder
!

initialize

        super initialize.
        Smalltalk isSmalltalkX ifFalse:[ self saxDriver: TfSAXDriver new ].
        self initializeReviser.
        self registerStopClasses.
!

initializeReviser
	" initialize the revisers. revisers correct structural errors "

	revisers := OrderedCollection new.
	revisers add: TfHTMLCloseTagReviser new.
	revisers add: (TfHTMLFixDOMReviser new).
!

on: inputStream

	super on: inputStream.
	validating := false.
	self addHTML4Entities.
!

registerStopClasses

	implicitStopTags := Dictionary new.
	TfElement implicitCloseStopClasses do: [ : cl |
		cl tagNames do: [ : nm |
			| stopNames |
			stopNames := Set new.
			cl elementsToCloseStopClasses do: [ : clStop |
				stopNames addAll: clStop tagNames
			].
			implicitStopTags at: cl put: stopNames
		].
	].
! !

!TfHTMLParser methodsFor:'inspecting'!

inspectorClasses

	^ super inspectorClasses copyWith: TfHTMLParserInspector
! !

!TfHTMLParser methodsFor:'namespaces'!

correctAttributeTag: attribute

	| ns tag qual type |
	qual := attribute tag qualifier.
	qual isEmpty
		ifTrue: [^self].
	type := attribute tag type.
	ns := self findNamespace: qual.
" modification: html tag >> "
	tag := self tagClass  new qualifier: qual ns: ns type: type.
" << modification >> "
	attribute tag: tag
!

correctTag: tag

	| ns type qualifier |
	qualifier := tag qualifier.
	type := tag type.
	ns := self findNamespace: qualifier.
" modification: html tag >> "
	^ self tagClass  new qualifier: qualifier ns: ns type: type
" << modification >> "
!

findNamespace: ns

	| nsURI |
	ns = 'xml' ifTrue: [^XML_URI].
	ns = 'xmlns' ifTrue: [^'<!!-- xml namespace -->'].
	ns = 'xlink' ifTrue: [^ 'http://www.w3.org/1999/xlink' ].
	elementStack size to: 1 by: -1 do: [:i |
		nsURI := (elementStack at: i) findNamespace: ns.
		nsURI = nil ifFalse: [^nsURI]].
" modification: don't complain about missing namespaces >> "
	^ ns
" << modification "
! !

!TfHTMLParser methodsFor:'private'!

checkCharsetIn: localTagName attributes: someAttributes
	" evil hack to detect the charset definition "

	| ct cs assocs encAssoc |
	((localTagName asLowercase = 'meta')
			and: [ (ct := someAttributes detect: [ : a | a tag isLike: 'http-equiv' ] ifNone: [ nil ]) notNil
			and: [ ct value asLowercase = 'content-type'
			and: [ (cs := someAttributes detect: [ : a | a tag isLike: 'content' ] ifNone: [ nil ]) notNil ]]])
		ifTrue: [
			assocs := TfHTMLValueReader singleton parseValuePairs: cs value separatedBy: $=.
			encAssoc := assocs detect: [ : a | a key asLowercase = 'charset' ] ifNone: [ nil ].
			encAssoc notNil
				ifTrue: [ self forceEncoding: encAssoc value ]
		].
!

convertToCharacter: n

	| charEncoder |
	^ (n < 256 and: [ encoder notNil and: [ (charEncoder := encoder encoder) notNil ]])
		ifTrue: [ charEncoder decode: n ]
		ifFalse: [ Character value: n ]
!

emptyElementTags

	^ #('br' 'p')
!

endDocument

        " make sure that all remaing tags are closed "
        builder elementStack reverseDo: [ :context | self endTag: context tag ].
        " fix DOM "
        self revisers do: [ :r | r postProcess: self document ].
!

expected: str
	"With so much truly awful HTML out there, we cannot just bail on expected tags"

	Transcript show: '<- ', str , '> expected '; cr
!

getQualifiedName

	| nm |
	nm := self getSimpleName.
	^hereChar = $:
		ifTrue:
			[self getNextChar.
" modification: html tag >> "
			self tagClass new qualifier: nm ns: '' type: self getSimpleName]
		ifFalse:
			[self tagClass  new qualifier: '' ns: '' type: nm]
" << modification: html tag "
!

getSimpleName

" modification: ignore all illegal characters >>>> "
	[ self atEnd not and: [ self isValidNameStart: hereChar ]] whileFalse: [ self getNextChar ].
" <<<< modification "
	^ super getSimpleName
!

invalid: aMessage

	self showDebugInfo.
	super invalid: aMessage
!

isAttributeValueDelimiter: aChar

	^ (self isWhitespace: aChar) or: [ aChar = $> ]
!

isTagOpen: aTag

	^ elementStack contains: [ :context | context tag isLike: aTag ]
!

isWhitespace: aChar

	^ self whitespaceValues includes: aChar asInteger
!

malformed: aMessage

	self showDebugInfo.
	super malformed: aMessage
!

openedTagsUpTo: aTag stopAt: stopTags do: aBlock

	elementStack size to: 1 by: -1 do: [ :i | | tag |
		tag := (elementStack at: i) tag.
		(stopTags includes: tag)
			ifTrue: [ ^ self ].
		aTag = tag
			ifTrue: [
				elementStack size to: i by: -1 do: [ :j |
					aBlock value: (elementStack at: j) tag
				].
				^ self
			].
	].
!

popDummyRoot

	self endTag: (TfNodeTag tagName: 'html').
!

pushDummyRoot
    Smalltalk isSmalltalkX ifTrue:[
        builder driver startElement: '' localName: 'html' qName: 'html' attributes: #().
        builder pushTag: (TfNodeTag tagName: 'html').
    ] ifFalse:[
        sax startElement: '' localName: 'html' qName: 'html' attributes: #().
        self pushNewTag: (TfNodeTag tagName: 'html').
    ]
!

revisers
	" returns all revisers. these dudes correct structural errors "

	^ revisers
!

showDebugInfo

	| str i j |
	str := currentSource stream.
	(str isKindOf: PositionableStream)
		ifTrue: [
			i := (str position - 20) max: 1.
			j := (str position + 20) min: str collection size.
			#{TfLog} ifDefinedDo: [ :log | log singleton logWarning: 'HTML parse error near ', (str collection copyFrom: i to: j) printString ].
		].
!

skipScript
	" skip all contents until we find /script "

	self upToAll: '</script'.
	currentSource skip: -1.
	self upTo: $>.
	self getNextChar.
!

tagClass

	^ TfNodeTag
!

whitespaceValues

	^ #(9 10 13 32)
! !

!TfHTMLParser methodsFor:'streaming'!

forceEncoding: encodingName

	| sym |
	sym := (self class mapEncoding: encodingName) asSymbol.
	encoder := StreamEncoder new: sym.
	encoder notNil
		ifTrue: [
			encoder initializeForFiles.
			currentSource tfEncoder: encoder.
		].
!

getNextCharBehind: aString

	self skipBehind: aString.
	^ hereChar := currentSource stream peek
!

skipBehind: aString

	(self skipIf: aString)
		ifFalse: [ currentSource stream skipThroughAll: aString ].
!

skipIf: str
        " modification: ignore case "

        | p oc l c |
        hereChar asLowercase = str asLowercase first ifFalse: [^false].
        p := self sourceWrapper stream position.
        l := self sourceWrapper line.
        c := self sourceWrapper column.
        oc := hereChar.
        1 to: str size do: [:i |
                hereChar asLowercase = (str at: i) asLowercase
                        ifFalse:
                                [self sourceWrapper stream position: p.
                                self sourceWrapper line: l; column: c.
                                hereChar := oc.
                                ^false].
                lastSource := currentSource.
                currentSource := self sourceWrapper.
                hereChar := self sourceWrapper "next" nextFor:self].
        hereChar == nil
                ifTrue: [self getNextChar].
        ^true
!

upTo: aCharacter
	" modification: ignore case "

	| newStream element ch |
	ch := aCharacter asLowercase.
	newStream := (String new: 64) writeStream.
	[self atEnd]
		whileFalse:
			[element := self nextChar.
			element asLowercase = ch
				ifTrue: [^newStream contents].
			newStream nextPut: element.].
	self expected: (String with: aCharacter).
	^newStream contents
!

upToAll: target
	" modification: ignore case "

	| str i lowerTarget |
	(target occurrencesOf: target first) = 1
		ifFalse: [self error: (#badSearchStringTarget << #xml >> 'The target collection is ambiguous.')].
	lowerTarget := target asLowercase.
	self sourceWrapper skip: -1.
	str := (String new: 32) writeStream.
	[str nextPutAll: (self upTo: target first).
	i := 2.
	[i <= target size and:
			[self nextChar asLowercase = (lowerTarget at: i) ]]
		whileTrue:
			[i := i+1].
	i <= target size]
		whileTrue:
			[str nextPutAll: (target copyFrom: 1 to: i-1).
			self sourceWrapper skip: -1].
	self getNextChar.
	^str contents
! !

!TfHTMLParser methodsFor:'testing'!

shouldTestWFCEntityDeclared

	^ false
! !

!TfHTMLParser class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfHTMLParser.st,v 1.4 2009/11/07 03:21:45 cg Exp $'
! !
