"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfTestCase subclass:#TfHTMLTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerTesting'
!


!TfHTMLTests methodsFor:'documents'!

attributesHTML

	^ '
<html>
	<a alt="1"href="2"id"3" >
</html>
'
!

attributesResultHTML

	^ '<html><a alt="1" href="2" id="3"></a></html>'
!

colGroupTableHTML

	^ '
<table>
<colgroup>
	<col with="1%">
	<col with="2%">
	<tr><td>1</td><td>2</td></tr>
</table>
'
!

colGroupTableResultHTML

	^ '<table><colgroup><col with="1%"></col><col with="2%"></col></colgroup><tr><td>1</td><td>2</td></tr></table>'
!

scriptHTML

	^ '
<html>
1
<script>contents of >script< does >not< matter</SCRIPT>
2
</html>
'
!

scriptResultHTML

	^ '<html>
1

2
</html>'
!

tableCellsHTML

	^ '
<table>
	<tr><td>1<td>2</tr>
	<tr><td>3<td>4</tr>
</table>
'
!

tableCellsResultHTML

	^ '<table><tr><td>1</td><td>2</td></tr><tr><td>3</td><td>4</td></tr></table>'
! !

!TfHTMLTests methodsFor:'testing - HTML processing'!

testPreviousElement

	| doc test2 sub abc def test3 |
	doc := TfHTMLParser processDocumentString:
'<html>
	<abc>ABC</abc>
	<sub>
		<test1/><test2/><test3/>
	</sub>
	<def>DEF</DEF>
</html>'.
	self should: [ doc notNil ].
	test2 := (doc root elementNamed: 'sub') elementNamed: 'test2'.
	sub := test2 tfPreviousElementNamed: 'sub'.
	self should: [ sub = (doc root elementNamed: 'sub') ].
	abc := test2 tfPreviousElementNamed: 'abc'.
	self should: [ abc = (doc root elementNamed: 'abc') ].
	def := test2 tfPreviousElementNamed: 'def'.
	self should: [ def = nil ].
	test3 := test2 tfPreviousElementNamed: 'test3'.
	self should: [ test3 = nil ].
! !

!TfHTMLTests methodsFor:'testing - HTML revisers'!

testColGroupTable

	| doc |
	doc := TfHTMLParser processDocumentString: self colGroupTableHTML.
	self should: [ doc tfDebugString =  self colGroupTableResultHTML ].
!

testTableCells

	| doc |
	doc := TfHTMLParser processDocumentString: self tableCellsHTML.
	self should: [ doc tfDebugString =  self tableCellsResultHTML ].
! !

!TfHTMLTests methodsFor:'testing - bad HTML'!

testAmpersand
	" sometimes, an ampersand is an ampersand, not the beginning of an entity "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html>Thunder & Consolation</html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html>Thunder &amp; Consolation</html>' ].
	doc := TfHTMLParser processDocumentString: '<html><a href="/cgi?action1&action2"></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><a href="/cgi?action1&amp;action2"></a></html>' ].
!

testDamagedAttributes

	| doc |
	doc := self createDocument: self attributesHTML.
	self should: [ doc tfDebugString = self attributesResultHTML].
!

testDamagedEntities

	| doc |
	doc := TfHTMLParser processDocumentString: '<html>&#84woflower</html>'.
	self should: [ doc tfDebugString = '<html>Twoflower</html>' ].
!

testInvalidTags
	" typos make parsing hard "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html><p>42</p<hl></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><p>42</p><hl></hl></html>' ].
!

testMissingTable
	"ensure that there are no <tr> elements without enclosing table "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html><tr><td>Cell</td></tr></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><table><tr><td>Cell</td></tr></table></html>' ].
!

testUnexpectedEnd
	" sometimes, the document ends without closing the root tag "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html><b>no HTML end tag'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><b>no HTML end tag</b></html>' ].
! !

!TfHTMLTests methodsFor:'testing - valid HTML'!

testAttributesWithoutValue
	" HTML allows attributes without value. these are expanded to 'name=name' "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html name1 name2></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString ='<html name1="name1" name2="name2"></html>' ].
!

testComments
	" HTML comments are more relaxed "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html><!!-- double -- hypens --></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><!!-- double -- hypens --></html>' ].
	doc := TfHTMLParser processDocumentString: '<html><!!-- hypen at end ---></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><!!-- hypen at end ---></html>' ].
!

testHelloWorld
	" if THIS fails something seriously went wrong ;-) "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html>Hello world</html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString =  '<html>Hello world</html>'	].
!

testIgnoreCase
	" HTML is case insensitive "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html>case <b>does not</B> matter</html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html>case <b>does not</b> matter</html>' ].
!

testIgnoreScript
	" ignore contents of script tags "

	" note: currently fails "
	| doc |
	doc := TfHTMLParser processDocumentString: self scriptHTML.
	self should: [ doc tfDebugString = self scriptResultHTML ].
!

testStandaloneTags
	" HTML allows standalone tags that are never closed "
	" note: standalone tags are closed by the parser at the position where they are detected,
	so they are usually not empty. this is necessary for tags which do formatting (e.g. <font>).
	other tags (e.g. <p> or <br>) don't care where they are closed, so this behavior should not be
	a problem "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html><b>here<p>are some<p>paragraphs</b></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html><b>here<p>are some<p>paragraphs</p></p></b></html>' ].
!

testUnqoutedAttributes
	" HTML allows unqouted attribute values  "

	| doc |
	doc := TfHTMLParser processDocumentString: '<html name1=value1 name2=value2></html>'.
	self should: [ doc notNil ].
	self should: [ doc tfDebugString = '<html name1="value1" name2="value2"></html>' ].
!

testWhitespaces
	" all whitespace sequences are converted to a single space"

	| doc |
	" 2 spaces "
	doc := TfHTMLParser processDocumentString: '<tag>Hello  world</tag>'.
	self should: [ doc tfDebugString tfWithoutWhitespaces = '<tag>Hello world</tag>' ].
	" tab space tab "
	doc := TfHTMLParser processDocumentString: '<tag>Hello	 	world</tag>'.
	self should: [ doc tfDebugString tfWithoutWhitespaces = '<tag>Hello world</tag>' ].
	" return "
	doc := TfHTMLParser processDocumentString: 
'<tag>Hello
world</tag>'.
	self should: [ doc tfDebugString tfWithoutWhitespaces = '<tag>Hello world</tag>' ].
	" spaces and returns "
	doc := TfHTMLParser processDocumentString: 
'<tag>Hello  
  world</tag>'.
	self should: [ doc tfDebugString tfWithoutWhitespaces = '<tag>Hello world</tag>' ].
! !

!TfHTMLTests class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfHTMLTests.st,v 1.1 2009/10/01 23:39:41 cg Exp $'
! !
