"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfResourceModel subclass:#TfImageModel
	instanceVariableNames:'image'
	classVariableNames:'ImageReaders'
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfImageModel class methodsFor:'instance creation'!

image: anImage

	^ self new
		image: anImage;
		yourself
! !

!TfImageModel class methodsFor:'accessing'!

flushReaders

	ObjectMemory removeDependent: self.
	ImageReaders := nil.
!

handledMimeTypes
	" patterns of all mime types that are handled by this class "

	^ #('image/*')
!

obsolete
	"This class is being removed from the system."

	self flushReaders.
	^ super obsolete
! !

!TfImageModel class methodsFor:'image readers'!

imageReaders

	ImageReaders isNil
		ifTrue: [ 
			ImageReaders := OrderedCollection new.
			TfImageReader allSubclassesDo: [ : cl |
				cl isAbstract ifFalse: [ ImageReaders add: cl new ].
			].
			ImageReaders := ImageReaders asSortedCollection: [ : reader1 : reader2 | reader1 class priority > reader2 class priority ].
		].
	^ ImageReaders
!

update: anAspect with: arguments from: anObject

	(anObject == ObjectMemory and: [ #(aboutToSnapshot #aboutToQuit) includes: anAspect ])
		ifTrue: [ self flushReaders.].
	^ super update: anAspect with: arguments from: anObject
! !

!TfImageModel methodsFor:'accessing'!

defaultTitle

	^ 'Image'
!

image

	^ image
!

image: anImage

	image := anImage tfAsCachedImage.
	self resourceModelChanged.
!

inspectResource

	self image inspect
!

loadResourceFrom: aMimeEntity
	" convert the entity into a resource "

	| str im imageReader |
	imageReader := self imageReaderFor: aMimeEntity contentType.
	imageReader notNil
		ifTrue: [
			str := aMimeEntity contentsStream binary.
			[	
				 im := imageReader readImageFrom: str  mimeType: aMimeEntity contentType
			] ensure: [ 
				str close
			].
		].
	im notNil
		ifTrue: [ self image: im tfAsCachedImage ].
!

resource

	^ self image
! !

!TfImageModel methodsFor:'image readers'!

imageReaderFor: contentType 
	" answer a block that reads an image from a stream, or nil "

	self class imageReaders do:  [ : readerClass | 
		readerClass handledMimeTypes do: [ : mimePattern | 
			(mimePattern match: contentType) ifTrue: [ ^readerClass ]
		]
	].
	^ nil
! !

!TfImageModel methodsFor:'views'!

createEmbeddedApp

	^ TfEmbeddedImageApp resourceModel: self.
! !

!TfImageModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfImageModel.st,v 1.1 2009/10/01 18:05:08 cg Exp $'
! !
