"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfView subclass:#TfImageView
	instanceVariableNames:'image loadImage lastScale graphicsDevice adjustedImage'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfImageView class methodsFor:'resources'!

unavailableImage
	"UIMaskEditor new openOnClass: self andSelector: #unavailableImage"

	<resource: #image>
	^CachedImage on: (Image extent: 11@11 depth: 2 bitsPerPixel: 2 palette: (Graphics.MappedPalette withColors: ((Core.Array new: 3) at: 1 put: Graphics.ColorValue red; at: 2 put: Graphics.ColorValue lightGray; at: 3 put: Graphics.ColorValue darkGray; yourself)) usingBits: #[170 170 168 0 133 85 72 0 145 85 24 0 148 84 88 0 149 17 88 0 149 69 88 0 149 17 88 0 148 84 88 0 145 85 24 0 133 85 72 0 170 170 168 0])
! !

!TfImageView methodsFor:'accessing'!

image

	^ image
!

image: anImage

	anImage notNil
		ifTrue: [ 
			image := anImage tfAsCachedImage.
			self adjustImage
		]
		ifFalse: [ 
			image := nil
		].
! !

!TfImageView methodsFor:'bounds accessing'!

computePreferredExtentFor: renderContext

	| ext |
	self setupImageFor: renderContext.
	(ext := self specifiedExtent) isNil
		ifTrue: [
			ext := image notNil
				ifTrue: [ image extent ]
				ifFalse: [ 1@1 ]
		].
	^ ext + (2*((self model hspace@self model vspace) + self model border))
! !

!TfImageView methodsFor:'displaying'!

displayDebugImageOn: aGraphicsContext

	| gc box |
	Log isDebug
		ifTrue: [
			gc := aGraphicsContext copy.
			gc paint: ColorValue veryLightGray.
			box := self bounds insetBy: (0@0 corner: 1@1).
			gc 
				displayLineFrom: box origin to: (self bounds corner - (1@1));
				displayLineFrom: box topRight to: self bounds bottomLeft;
				displayRectangularBorder: box.
		].
!

displayElementOn: aGraphicsContext

	adjustedImage notNil
		ifTrue: [ 
			adjustedImage 
				displayOn: aGraphicsContext 
				at: self preferredBounds origin + (self model hspace@self model vspace)
		] ifFalse: [
			self displayDebugImageOn: aGraphicsContext.
		].
! !

!TfImageView methodsFor:'private'!

adjustImage
	" scale image, make it compatible with the output device "

	| scale ext |
	(image isNil or: [ preferredBounds isNil ])
		ifTrue: [ ^ self ].
	((ext := image extent) notNil and: [ preferredBounds extent < ext])
		ifTrue: [ 
			scale := ext / preferredBounds extent.
			scale ~= lastScale
				ifTrue: [
					lastScale := scale.
					adjustedImage := ((image tfConvertForGraphicsDevice: graphicsDevice) tfShrunkenBy: scale) tfAsCachedImage
				].
		]
		ifFalse: [ adjustedImage := (image tfConvertForGraphicsDevice: graphicsDevice) tfAsCachedImage ].
!

changedImage
	" image has changed. update the view "

	(self specifiedExtent isNil or: [ self specifiedExtent < self image extent ])
		ifTrue: [ 
			preferredExtent := nil.
			minimalExtent := nil.
			self upcastEvent: #tfNeedsRendering with: nil from: self
		]
		ifFalse: [ 
			self invalidateRectangle: self bounds.
			self topComponent isNil
				ifFalse: [self topComponent sensor metaInput isNil
					ifFalse: [self topComponent sensor noteEvent]].
		].
!

loadImage

	loadImage isNil
		ifTrue: [ loadImage := self settings loadImages ].
	^ loadImage
!

loadImageFrom: url browserWindowModel: aTfBrowserWindowModel

	aTfBrowserWindowModel resourceManager
		resourceNeeded: url tfAsURLDescriptor
		resourceDo: [ : resourceModel | 
			self image: resourceModel image.
			self image notNil ifTrue: [ self changedImage ].
		].
!

specifiedExtent

	| w h |
	w := self model width.
	h := self model height.
	^ (w notNil and: [ h notNil ])
		ifTrue: [ w@h ]
		ifFalse: [ nil ]
! !

!TfImageView methodsFor:'rendering'!

computeMinimalExtentFor: renderContext

	^ self preferredExtentFor: renderContext
!

finishLayout: layoutContext

	| lineLayout |
	lineLayout := TfBoxElementLayout 
					for: self 
					bounds: (preferredBounds translatedBy: self tfLocalTranslation).
	lineLayout addToLayoutContext: layoutContext.
!

setupImageFor: renderContext

	| docModel url imageURL |
	self loadImage
		ifTrue: [
			loadImage := false.	 " only try once "
			(imageURL := self model sourceURL) notNil
				ifTrue: [
					docModel := renderContext documentModel.
					url := docModel resolveURL: imageURL.
					url notNil
						ifTrue: [ self loadImageFrom: url browserWindowModel: docModel browserWindowModel ].
				].
		].
!

setupRendering: renderContext

	| layoutContext pref alignOffset pos |
	super setupRendering: renderContext.
	graphicsDevice := renderContext graphicsDevice.
	layoutContext := renderContext layoutContext.
	layoutContext applyPendingLayouts.
	self setupImageFor: renderContext.
	layoutContext remainingLineWidth < (self preferredExtentFor: renderContext) x
		ifTrue: [ renderContext layoutContext nextLine ].
	pref := self preferredExtentFor: renderContext.
	pos := renderContext layoutContext position.
	alignOffset := self style paragraphStyle alignFactor * (renderContext boxContext bounds right - pos x - pref x).
	self tfLocalTranslation: pos + (alignOffset@0).
	preferredBounds := 0@0 extent: pref.
	self adjustImage.
	self finishLayout: renderContext layoutContext.
! !

!TfImageView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfImageView.st,v 1.1 2009/10/01 19:42:22 cg Exp $'
! !
