"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfLinkedContext subclass:#TfLayoutContext
	instanceVariableNames:'currentElementLayouts currentLineHeight lineHeights
		isParagraphStart documentLayout isFirstParagraph listLevel
		listWidth rowSpanLayouts rightAlignOffset columnIndexHolder
		tableLayout cellSpacing columnWidthLayouts useMaxSpace
		isForMeasuring'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfLayoutContext methodsFor:'accessing'!

boxContext

	^ self renderContext boxContext
!

documentLayout

	^ documentLayout
!

doesNotUnderstand: aMessage

	^ self boxContext 
		perform: aMessage selector
		withArguments: aMessage arguments
!

isForMeasuring

	^ isForMeasuring
!

prepareForMeasuring

	isForMeasuring := true.
	self boxContext prepareForMeasuring.
!

reset

	self boxContext reset.
	self resetLayout.
!

resetLayout

	self resetLine.
	self isFirstParagraph: nil.
	self isParagraphStart: nil.
	self boxContext pendingPositionDelta: nil.
! !

!TfLayoutContext methodsFor:'genetic stuff'!

beClone

	super beClone.
	currentElementLayouts := OrderedCollection new.
! !

!TfLayoutContext methodsFor:'initialize-release'!

initialize

	super initialize.
	rightAlignOffset := 0.
	documentLayout := TfDocumentLayout new.
	lineHeights := OrderedCollection new.
	currentElementLayouts := OrderedCollection new.
	isForMeasuring := false.
! !

!TfLayoutContext methodsFor:'line layout'!

addElementLayout: aTfElementLayout

	aTfElementLayout globalTranslation: self boxContext globalTranslation.
	self currentElementLayouts add: aTfElementLayout
!

currentElementLayouts

	^ currentElementLayouts
!

currentLineHeight

	currentLineHeight isNil
		ifTrue: [ currentLineHeight := 0 ].
	^ currentLineHeight
!

currentLineHeight: anInteger

	currentLineHeight := anInteger
!

currentLineHeightMax: anInteger

	currentLineHeight := self currentLineHeight max: anInteger
!

currentLineIndex

	^ lineHeights size + 1
!

finishLineLayout
	" all elements of the current line have been prepared.
	now give them a chance to adapt to their neighbors "

	| maxAscent maxDescent maxHeight remainingWidth |
	(self isForMeasuring not and: [ currentElementLayouts notNil ]) 
		ifTrue: [
			remainingWidth := self boxContext remainingWidth.
			maxAscent := 0.
			maxDescent := 0.
			maxHeight := 0.
			" compute max. values "
			currentElementLayouts do:  [ : layout | 
				maxAscent := maxAscent max: layout ascent.
				maxDescent := maxDescent max: layout descent.
				maxHeight := maxHeight max: layout lineHeight
			].
			currentElementLayouts do: [ : layout | 
				layout 
					adaptToAscent: maxAscent
					descent: maxDescent
					height: maxHeight
					remainingWidth: remainingWidth 
					for: nil
			].
			currentElementLayouts := nil
		].
	lineHeights add: self currentLineHeight
!

forceNextLine

	self finishLineLayout.
	self boxContext newLine: currentLineHeight.
	self resetLineLayout.
!

lineHeightAt: index

	^ lineHeights size >= index
		ifTrue: [ lineHeights at: index ]
		ifFalse: [ nil ]
!

nextLine

	(self currentElementLayouts allSatisfy: [ :layout | layout isInvisibleElement ])
		ifTrue: [ 	self resetLine ]
		ifFalse: [ self forceNextLine ].
!

resetLine

	self resetLineLayout.
	self boxContext resetLine.
! !

!TfLayoutContext methodsFor:'list layout'!

decreaseListLevel

	self listLevel: self listLevel -1.
!

increaseListLevel

	self listLevel: self listLevel + 1.
!

listLevel

	listLevel isNil
		ifTrue: [ listLevel := 0 ].
	^ listLevel
!

listLevel: anInteger

	listLevel := anInteger
!

listWidth

	listWidth isNil
		ifTrue: [ listWidth := 16 ].
	^ listWidth
!

listWidth: anInteger

	listWidth := anInteger
! !

!TfLayoutContext methodsFor:'paragraph'!

addParagraph: spacing flowAroundBoxes: flowAroundBoxes

	(self currentElementLayouts allSatisfy: [ :layout | layout ignoreParagraphStart ])
		ifFalse: [ self forceNextLine ].
	self isParagraphStart: true.
	self isFirstParagraph
		ifTrue: [ self isFirstParagraph: false ]
		ifFalse: [ self boxContext pendingPositionDeltaMax: 0@spacing ].
	flowAroundBoxes
		ifFalse: [ self applyProtectedBoxesToParagraph ].
!

applyPendingLayouts

	self isFirstParagraph
		ifTrue: [ self boxContext resetPendingPositionDelta ]
		ifFalse: [ self boxContext applyPendingPositionDelta ]
!

isFirstParagraph

	isFirstParagraph isNil
		ifTrue: [ isFirstParagraph := true ].
	^ isFirstParagraph
!

isFirstParagraph: aBoolean

	isFirstParagraph := aBoolean
!

isParagraphStart

	isParagraphStart isNil
		ifTrue: [ isParagraphStart := true ].
	^ isParagraphStart
!

isParagraphStart: aBoolean

	isParagraphStart := aBoolean
! !

!TfLayoutContext methodsFor:'private'!

applyProtectedBoxesToParagraph

	| p boxContext |
	boxContext := self boxContext.
	boxContext protectedBoxes isEmpty
		ifFalse: [ 
			self applyPendingLayouts.
			p := boxContext position copy.
			boxContext protectedBoxes do: [ : layout |
				p := layout applyToParagraphPosition: p.
			].
			boxContext pendingPositionDeltaMax: p - boxContext position.
			self applyPendingLayouts.
			self resetLine.
		].
!

resetLineLayout

	currentElementLayouts := OrderedCollection new.
	currentLineHeight := nil.
! !

!TfLayoutContext methodsFor:'table layout'!

addRowSpanLayout: cellLayout

	self rowSpanLayouts add: cellLayout
!

cellSpacing

	^ cellSpacing
!

columnIndex

	^ columnIndexHolder value
!

columnIndex: newColumnIndex

	columnIndexHolder value: newColumnIndex
!

columnWidthLayouts

	^ columnWidthLayouts
!

finishRow

	self nextLine.
	rowSpanLayouts copy do: [ : layout | 
		(layout finishRowFor: self) 
			ifFalse: [rowSpanLayouts remove: layout ]
	].
!

isInTable

	^ self tableLayout notNil
!

rowSpanLayouts

	rowSpanLayouts isNil
		ifTrue: [ rowSpanLayouts := OrderedCollection new ].
	^ rowSpanLayouts
!

startColumnIndex: newColumnIndex
	" apply bounds of cells spanning multiple rows to the position "

	self rowSpanLayouts do: [ : layout |
		layout applyRowSpan: (self columnIndex to: newColumnIndex - 1) to: self cellSpacing: self cellSpacing.
	].
	self columnIndex: newColumnIndex
!

startRow

	self columnIndex: 1.
!

startTable: aTfTableView

	rowSpanLayouts := OrderedCollection new.
	columnIndexHolder := ValueHolder with: 1.
	cellSpacing := aTfTableView cellSpacing.
	columnWidthLayouts := aTfTableView columnWidthLayouts.
!

tableLayout

	^ tableLayout
!

tableLayout: aTfColumnsLayout

	tableLayout := aTfColumnsLayout
! !

!TfLayoutContext class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfLayoutContext.st,v 1.1 2009/10/01 18:43:46 cg Exp $'
! !
