"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfLog
	instanceVariableNames:'transcript isDebug debugLevel debugViewClass'
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerCore'
!

TfLog class instanceVariableNames:'singleton'

"
 No other class instance variables are inherited by this class.
"
!


!TfLog class methodsFor:'initialize-release'!

new

	^ super new initialize
! !

!TfLog class methodsFor:'singleton'!

flushSingleton

	singleton := nil.
	Log := nil.
!

obsolete

	self flushSingleton.
	super obsolete.
!

singleton

	singleton isNil
		ifTrue: [ singleton := self new ].
	^ singleton
!

singleton: aTfLog

	singleton := aTfLog
! !

!TfLog methodsFor:'accessing'!

cr

	self transcript cr.
!

debugDo: aBlock

	self isDebug
		ifTrue: [ aBlock value ].
!

debugLevel

	debugLevel isNil
		ifTrue: [ debugLevel := 1 ].
	^ debugLevel
!

debugLevel: anInteger

	debugLevel := anInteger
!

debugLevel: level do: aBlock

	(self isDebug and: [ level <= self debugLevel ])
		ifTrue: [ aBlock value ].
!

debugViewClass

	^ debugViewClass
!

debugViewClass: aClass

	debugViewClass := aClass
!

flush

	self transcript flush
!

isDebugLevel: level

	^ level <= self debugLevel and: [ self isDebug ]
!

log: anObject

	self triggerEvent: #log with: anObject displayString.
	self
		basicLogHeader;
		basicShow: ': ';
		basicShow: anObject;
		cr.
!

log: aString milliseconds: milliseconds

	| ts |
	ts := (PrintConverter for: #fixedpoint) printStringFor: milliseconds/1000.
	self logDebug: ['<1s> (<2s> sec.)' expandMacrosWith: aString with: ts]
!

logDebug: debugStringBlock

	self isDebug
		ifTrue: [ self basicShow: debugStringBlock value; cr ].
!

logDebug: debugStringBlock level: level

	level <= self debugLevel
		ifTrue: [ self logDebug: debugStringBlock ]
!

logError: anObject

	self triggerEvent: #error with: anObject displayString.
	(self isDebug and: [ self debugLevel >= 2 ]) ifTrue: [ self halt ].
	self
		basicLogHeader;
		basicShow: ': ERROR: ';
		basicShow: anObject;
		cr.
	self flush.
!

logSimple: anObject

	self
		basicShow: anObject;
		cr.
!

logTimestamp

	self basicLogTimestamp
!

logWarning: anObject

	self triggerEvent: #warning with: anObject.
	self
		basicLogHeader;
		basicShow: ': WARNING: ';
		basicShow: anObject;
		cr.
	self flush.
!

show: aStringOrObject

	self basicShow: aStringOrObject
!

toggleDebug

	^ self isDebug: self isDebug not
! !

!TfLog methodsFor:'initialize-release'!

initialize

	transcript := Transcript.
	isDebug := false.
	Log := self.
! !

!TfLog methodsFor:'private'!

basicLogHeader

	self basicShow: '[Twoflower]'.
!

basicLogTimestamp

	| str |
	str := String new writeStream.
	Locale current timePolicy printShort: Timestamp now on: str.
	self basicShow: str contents.
!

basicShow: aStringOrObject

	self transcript show: aStringOrObject displayString
! !

!TfLog methodsFor:'setup'!

isDebug

	^ isDebug == true
!

isDebug: aBoolean

	isDebug := aBoolean
!

transcript

	^ transcript notNil
		ifTrue: [ transcript ]
		ifFalse: [ Transcript ]
!

transcript: aTextCollector

	transcript := aTextCollector
! !

!TfLog class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfLog.st,v 1.1 2009/10/01 20:04:29 cg Exp $'
! !
