"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfNavigationModel
	instanceVariableNames:'navigationHistory gotoNewEntry currentNavigationEntryAspect
		currentNavigationIndexAspect canGoForwardAspect canGoBackAspect
		navigationLock'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfNavigationModel class methodsFor:'instance creation'!

new

	^ super new initialize
! !

!TfNavigationModel methodsFor:'accessing'!

beforeNavigationSend: methodSymbol to: anObject

	self when: #beforeNavigation send: methodSymbol to: anObject
!

canGoBackAspect

	canGoBackAspect isNil
		ifTrue: [ 
			canGoBackAspect := BlockValue with: [ self currentNavigationIndex > 1 ].
			canGoBackAspect dependOn: currentNavigationIndexAspect.
		].
	^ canGoBackAspect
!

canGoForwardAspect

	canGoForwardAspect isNil
		ifTrue: [ 
			canGoForwardAspect := BlockValue with: [ self currentNavigationIndex < self navigationHistory size ].
			canGoForwardAspect dependOn: currentNavigationIndexAspect.
		].
	^ canGoForwardAspect
!

cancelGotoNewEntry
	" could not navigate to an entry "

	navigationLock critical: [ gotoNewEntry := nil ].
!

currentNavigationEntry

	^ self currentNavigationEntryAspect value
!

currentNavigationEntryAspect

	currentNavigationEntryAspect isNil
		ifTrue: [ 
			currentNavigationEntryAspect := self currentNavigationIndex asValue.
		].
	^ currentNavigationEntryAspect
!

goBack
	" go back in history "

	self currentNavigationIndex > 1
		ifTrue: [ self navigateToIndex: self currentNavigationIndex - 1 ].
!

goForward
	" go forward in history "

	self currentNavigationIndex < self navigationHistory size
		ifTrue: [ self navigateToIndex: self currentNavigationIndex + 1 ].
!

goto: url
	" go to the specified URL (may be a string, URL or URLDescriptor "

	| entry |
	entry := url tfAsURLDescriptor.
	entry lastVisit: Timestamp now.
	self gotoNewEntry: entry
!

gotoNewEntry: aTfNavigationEntry
	" navigate to the new entry. navigation may fail, so do not add it to the history yet "

"	| isNewEntry |
	isNewEntry := navigationLock critical: [
		(gotoNewEntry isNil or: [ gotoNewEntry ~= aTfNavigationEntry ])
			ifTrue: [
				aTfNavigationEntry rememberVisit
					ifTrue: [ gotoNewEntry := aTfNavigationEntry ].
				true
			] ifFalse: [ false ]
	].
	isNewEntry
		ifTrue: [ "
			self gotoEntry: aTfNavigationEntry
		"]."
!

onNavigationInPageSend: methodSelector to: anObject

	self when: #navigationInPage send: methodSelector to: anObject
!

onNavigationSend: methodSymbol to: anObject

	self when: #onNavigation send: methodSymbol to: anObject
!

submit: aTfForm
	" submit the form "

	| entry |
	entry := TfFormDescriptor form: aTfForm.
	entry lastVisit: Timestamp now.
	self gotoNewEntry: entry
!

visitedEntry: aTfNavigationEntry
	" navigation has succeeded. remember the visit if desired "

	| currentEntry |
	aTfNavigationEntry rememberVisit
		ifTrue: [
			navigationLock critical: [
				self cropHistory.
				Log  logDebug: ['Navigation: ', aTfNavigationEntry printString] level: 1.
				((currentEntry := self navigationHistory topOrNil) notNil
					and: [ aTfNavigationEntry sameNavigationAs: currentEntry ])
						ifTrue: [ Log  logDebug: ['Same naviagtion target '] level: 1 ]
						ifFalse: [ self navigationHistory push: aTfNavigationEntry ].
				self currentNavigationIndexAspect value: self navigationHistory size.
				gotoNewEntry := nil.
			].
		].
! !

!TfNavigationModel methodsFor:'initialize-release'!

initialize

	navigationLock := RecursionLock new.
! !

!TfNavigationModel methodsFor:'private'!

cropHistory
	" remove all URLs behind the current index "

	self navigationHistory size > self currentNavigationIndex
		ifTrue: [	self navigationHistory: (self navigationHistory copyFrom: 1 to: self currentNavigationIndex) ].
!

currentNavigationIndex

	^ self currentNavigationIndexAspect value
!

currentNavigationIndexAspect

	currentNavigationIndexAspect isNil
		ifTrue: [ currentNavigationIndexAspect := 0 asValue  ].
	^ currentNavigationIndexAspect
!

gotoEntry: aTfEntry

	| inPage |
	inPage := aTfEntry url tfFragment notNil 
		and: [ (aTfEntry samePageAs: self currentNavigationEntry) 
		and: [ aTfEntry openMode = #sameWindow ]].
	self triggerEvent: #beforeNavigation with: aTfEntry.
	self currentNavigationEntryAspect value: aTfEntry.
	inPage
		ifTrue: [ self triggerEvent: #navigationInPage with: aTfEntry ]
		ifFalse: [ self triggerEvent: #onNavigation with: aTfEntry ].
!

navigateToIndex: anInteger

	| entry |
	self currentNavigationIndexAspect value: anInteger.
	entry := (anInteger >= 1  and: [ anInteger <= self navigationHistory size ])
				ifTrue: [ self navigationHistory at: anInteger ]
				ifFalse: [ nil ].
	entry notNil
		ifTrue: [ 
			entry rememberVisit: false.
			self gotoEntry: entry
		]
!

navigationHistory

	navigationHistory isNil
		ifTrue: [ navigationHistory := TfStack new ].
	^ navigationHistory
!

navigationHistory: aTfStack

	navigationHistory := aTfStack
! !

!TfNavigationModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfNavigationModel.st,v 1.1 2009/10/01 18:05:41 cg Exp $'
! !
