"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfResourceManager
	instanceVariableNames:'baseURL resourceCache downloads browserModel requestsLock
		requests maxRequests'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfResourceManager class methodsFor:'instance creation'!

browserModel: aTfBrowserModel

	^ self new 
		browserModel: aTfBrowserModel;
		yourself
! !

!TfResourceManager class methodsFor:'initialize-release'!

new

	^ super new initialize
! !

!TfResourceManager methodsFor:'accessing'!

browserModel

	^ browserModel
!

browserModel: aTfBrowserModel

	browserModel := aTfBrowserModel
!

modelClassForMimeType: mimeType descriptor: aTfURLDescriptor

	| mc |
	aTfURLDescriptor forceDownload
		ifFalse: [ mc := TfResourceModel modelClassForMimeType: mimeType ].
	^ mc notNil
		ifTrue: [ mc ]
		ifFalse: [ TfDownloadModel ]
!

resourceFromDescriptor: urlDescriptor
	" get a (possibly cached) resource model from the url "

	| url resourceModel |
	url := urlDescriptor url tfAsURL.
	resourceModel := url tfDirectResource.
	resourceModel isNil
		ifTrue: [
			urlDescriptor useCache
				ifTrue: [ resourceModel := self cachedResourceAt: urlDescriptor ].
			resourceModel isNil
				ifTrue: [ resourceModel := self loadResourceFromDescriptor: urlDescriptor ].
		].
	resourceModel notNil
		ifTrue: [ resourceModel urlDescriptor: urlDescriptor ]
		ifFalse: [ urlDescriptor rememberVisit: false ].
	^ resourceModel
! !

!TfResourceManager methodsFor:'cache'!

cacheEntryFor: resourceModel mimeEntity: mimeEntity

	^ resourceModel notNil
		ifTrue: [
			resourceModel useCache
				ifTrue: [ TfCacheEntry resourceModelClass: resourceModel class mimeEntity: mimeEntity ]
				ifFalse: [ nil ]
		] ifFalse: [
			 TfCacheEntry resourceModelClass: nil mimeEntity: nil
		].
!

cachedResourceAt: aTfURLDescriptor 

	| cacheEntry |
	cacheEntry := self resourceCacheAt: aTfURLDescriptor.
	^ cacheEntry notNil
		ifTrue: [ cacheEntry createResourceModel ]
		ifFalse: [ nil ].
!

hasResourceCacheAt: aTfURLDescriptor

	^ self resourceCache includesKey: aTfURLDescriptor
!

invalidateCacheAt: aTfURLDescriptor

	^ self resourceCache removeKey: aTfURLDescriptor ifAbsent: [].
!

resourceCache

	resourceCache isNil
		ifTrue: [ resourceCache := Dictionary new ].
	^ resourceCache
!

resourceCacheAt: aTfURLDescriptor

	^ self resourceCache at: aTfURLDescriptor ifAbsent: [ nil ]
!

resourceCacheAt: aTfURLDescriptor put: aTfCacheEntry

	self resourceCache at: aTfURLDescriptor put: aTfCacheEntry.
	aTfCacheEntry urlDescriptor: aTfURLDescriptor.
	^ aTfCacheEntry
! !

!TfResourceManager methodsFor:'downloads'!

addDownload: aTfDownload

	self downloads add: aTfDownload.
	self triggerEvent: #downloadBegin with: aTfDownload.
!

askDownloadFilenameFor: aTfURLDescriptor

	^ self browserModel dialogLockDo: [
		Dialog 
			requestFileName: 'Save file as:' 
			default: aTfURLDescriptor url tfDownloadFilename
			version: #new
			ifFail: [ nil ]
	].
!

download: aTfURLDescriptor
	" initiate a download  "

	| downloadModel filename |
	filename := self askDownloadFilenameFor: aTfURLDescriptor.
	filename isNil ifTrue: [ TfUserCanceledError raiseSignal ].
	downloadModel := TfDownloadModel resourceManager: self.
	downloadModel
		download: aTfURLDescriptor 
		to: filename asFilename.
	self startDownload: downloadModel of: aTfURLDescriptor.
	^ downloadModel
!

downloadEntityIfNeeded: aMimeEntity from: aTfURLDescriptor
	" initiate a download if the entity is not handled internally "

	| modelClass downloadModel filename |
	modelClass := self modelClassForMimeType: aMimeEntity contentType descriptor: aTfURLDescriptor.
	(modelClass handleDownload and: [ aTfURLDescriptor allowDownload ])
		ifFalse: [ ^ nil ].
	filename := self askDownloadFilenameFor: aTfURLDescriptor.
	filename isNil ifTrue: [ TfUserCanceledError raiseSignal ].
	downloadModel := modelClass resourceManager: self.
	downloadModel
		redirectEntity: aMimeEntity 
		from: aTfURLDescriptor 
		to: filename asFilename.
	self startDownload: downloadModel of: aTfURLDescriptor.
	^ downloadModel
!

downloads
	" all downloads (including completed ones)  "

	downloads isNil
		ifTrue: [ downloads := Core.List new ].
	^ downloads
!

removeDownload: aTfDownload

	self downloads remove: aTfDownload
!

startDownload: downloadModel of: urlDescriptor

	| request |
	(request := self requestAt: urlDescriptor) notNil
		ifTrue: [ 
			" I'm responsible for the download now "
			request finishRequest
		].
	self addDownload: downloadModel.
	downloadModel startExecuteProcess.
! !

!TfResourceManager methodsFor:'initialize-release'!

initialize

	requestsLock := RecursionLock new.
	requests := Set new.
	maxRequests := 0.
!

shutDown
	" sent before the browser is closed "

	self cancelAllRequests.
	self downloads do: [ :dm | dm shutDown ].
! !

!TfResourceManager methodsFor:'notification'!

changedRequests

	| numRequests |
	numRequests := self numberOfRequests.
	maxRequests := maxRequests max: numRequests.
	self triggerEvent: #requests with: (Array with: numRequests with: maxRequests).
!

onDownloadSend: methodSelector to: anObject

	self when: #downloadBegin send: methodSelector to: anObject
!

onRequestsChangeSend: methodSelector to: anObject

	self when: #requests send: methodSelector to: anObject
! !

!TfResourceManager methodsFor:'private'!

cachedResource: aTfURLDescriptor do: resourceBlock
	" process the cached resource "

	| cacheEntry resModel |
	cacheEntry := self resourceCacheAt: aTfURLDescriptor.
	resModel := cacheEntry createResourceModel.
	resModel notNil
		ifTrue: [ 	resourceBlock value: resModel ].
!

errors

	| errors |
	errors := Error.
	#{Net.HttpClientError} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpServerError} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpTimeout} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpObjectNotFound} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpEntityError} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpEntityLengthError} ifDefinedDo: [ : cl | errors := errors , cl ].
	#{Net.HttpStatusLineError} ifDefinedDo: [ : cl | errors := errors , cl ].
	^ errors
!

forkRequestFor: aTfURLDescriptor resourceDo: resourceBlock
	" fork a process that will fetch the resource located by the URL "

	requestsLock critical: [
		| request |
		request := requests 
					detect: [ : req | req urlDescriptor = aTfURLDescriptor ] 
					ifNone: [ | newRequest |
						newRequest := TfResourceRequest urlDescriptor: aTfURLDescriptor resourceManager: self.
						requests add: newRequest.
						self changedRequests.
						newRequest
					].
		request addResourceBlock: resourceBlock.
		requests add: request.
		request startProcess.
	].
!

loadResourceFromDescriptor: urlDescriptor
	" loads a resource model from the url "

	^ self 
		url: urlDescriptor url
		safeDo: [ urlDescriptor loadResourceFor: self ].
!

loadResourceFromMimeEntity: mimeEntity descriptor: urlDescriptor
	" loads a resource model from the url "

	| resourceModel cacheEntry finalURL |
	mimeEntity notNil
		ifTrue: [ 	
			resourceModel := self loadResourceModelFrom: mimeEntity descriptor: urlDescriptor.
			(resourceModel notNil and: [ (finalURL := mimeEntity tfURL) notNil and: [ finalURL ~= urlDescriptor url ]])
				ifTrue: [ resourceModel redirectedURLDescriptor: finalURL tfAsURLDescriptor ]]
		ifFalse: [ 
			resourceModel := nil ].
	urlDescriptor useCache
		ifTrue: [ 
			cacheEntry := self cacheEntryFor: resourceModel mimeEntity: mimeEntity.
			cacheEntry notNil
				ifTrue: [ self resourceCacheAt: urlDescriptor put: cacheEntry ]
		].
	^ resourceModel
!

loadResourceModelFrom: mimeEntity descriptor: urlDescriptor

	| modelClass |
	modelClass := self modelClassForMimeType: mimeEntity contentType descriptor: urlDescriptor.
	^ modelClass notNil
		ifTrue: [
			[	| resourceModel |
				resourceModel := modelClass new.
				resourceModel loadResourceFrom: mimeEntity.
				resourceModel
			] on: Error do: [ :ex |
				Twoflower.Log logError: 'Could not load entity from ', urlDescriptor printString, ': ', ex description.
				nil
			].
		]
		ifFalse: [ nil ]
!

url: anURLOrString safeDo: aBlock

	^ [[ 
		aBlock value
	] on: self warnings do: [ : ex |
		Log logWarning: ex description, ' [', anURLOrString asString, ']'.
		ex resume
	]] on: self errors do: [ :ex |
		Log logError: ex description, ' [', anURLOrString asString, ']'.
		nil
	].
!

warnings

	| warnings |
	warnings := Warning.
	#{Net.SSLWarning} ifDefinedDo: [ : cl | warnings := warnings, cl ].
	^ warnings
! !

!TfResourceManager methodsFor:'requests'!

cancelAllRequests

	requestsLock critical: [
		requests copy do: [ : request |
			request stopProcess.
		].
		requests := Set new.
	].
	maxRequests := 0.
	self changedRequests.
!

finishedRequest: aTfResourceRequest

	requestsLock critical: [ 
		requests remove: aTfResourceRequest ifAbsent: [].
	].
	self changedRequests.
!

numberOfRequests

	^ requestsLock critical: [
		requests size
	].
!

open: aTfURLDescriptor resourceDo: resourceBlock

	| handlerBlock |
	handlerBlock := TfApplicationRegistry singleton actionBlockFor: aTfURLDescriptor.
	handlerBlock notNil
		ifTrue: [ handlerBlock value: aTfURLDescriptor ]
		ifFalse: [ self resourceNeeded: aTfURLDescriptor resourceDo: resourceBlock ]
!

requestAt: aTfURLDescriptor

	^ requestsLock critical:[
		 requests
			detect: [ : req | req urlDescriptor = aTfURLDescriptor ] 
			ifNone: [ nil ].
	].
!

resourceNeeded: aTfURLDescriptor resourceDo: resourceBlock

	(self hasResourceCacheAt: aTfURLDescriptor)
		ifTrue: [ self cachedResource: aTfURLDescriptor do: resourceBlock ]
		ifFalse: [ self forkRequestFor: aTfURLDescriptor resourceDo: resourceBlock ].
! !

!TfResourceManager class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfResourceManager.st,v 1.1 2009/10/01 18:05:52 cg Exp $'
! !
