"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfResourceModel
	instanceVariableNames:'urlDescriptor redirectedURLDescriptor browserWindowModel
		resourceManager title'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfResourceModel class methodsFor:'instance creation'!

loadFrom: aMimeEntity

	^ self new
		loadResourceFrom: aMimeEntity;
		yourself
!

new

	^ super new initialize
!

resourceManager: aTfResourceManager

	^ self new
		resourceManager: aTfResourceManager;
		yourself
! !

!TfResourceModel class methodsFor:'accessing'!

handleDownload

	^ false
!

handledMimeTypes
	" patterns of all mime types that are handled by this class "

	^ #()
!

modelClassForMimeType: mimeType 

	^ self allSubclasses 
		detect: [ :cl | cl handledMimeTypes contains: [ :mtPattern | mtPattern match: mimeType ]]
		ifNone: [ nil ].
! !

!TfResourceModel methodsFor:'accessing'!

baseURL

	| d |
	^ (d := self redirectedURLDescriptor) notNil
		ifTrue: [ d url ]
		ifFalse: [ 
			(d := self urlDescriptor) notNil
				ifTrue: [d url ]
				ifFalse: [ nil ].
		].
!

basicBrowserWindowModel

	^ browserWindowModel
!

browserWindowModel

	browserWindowModel isNil
		ifTrue: [ browserWindowModel := TfResourceWindowModel new ].
	^ browserWindowModel
!

browserWindowModel: aTfBrowserWindowModel

	browserWindowModel := aTfBrowserWindowModel.
	browserWindowModel resourceModel ~~ self
		ifTrue: [ browserWindowModel resourceModel: self ].
	self resourceManager: aTfBrowserWindowModel resourceManager.
!

copyTextToClipboard
!

defaultTitle

	^ 'Unknown'
!

description
	" description of the model. uses for bookmarks, history etc. "

	^ self url asString
!

htmlString
	" HTML representation of the resource, or nil if not available "

	^ nil
!

image
	" image representation of the resource, or nil if not available "

	^ nil
!

inspectResource
!

loadResourceFrom: aMimeEntity
	" convert the entity into a resource "
!

loadResourceFromStream: aStream
	" convert the stream into a resource "
!

navigateTo: anURLOrString openMode: openModeOrNil

	| url descriptor |
	url := self resolveURL: anURLOrString.
	url notNil
		ifTrue: [ 
			descriptor := url tfAsURLDescriptor.
			descriptor originalUrlString: anURLOrString.
			openModeOrNil notNil	
				ifTrue: [ descriptor openMode: openModeOrNil ].
			self browserWindowModel navigationModel goto: descriptor ].
!

redirectedURLDescriptor
	" returns the URL where the resource came from in case of a redirected URL "

	^ redirectedURLDescriptor
!

redirectedURLDescriptor: aTfURLDescriptor
	" returns the URL where the resource came from in case of a redirected URL "

	redirectedURLDescriptor := aTfURLDescriptor
!

redirection
	" returns an URL descriptor if the resource includes a redirection "

	^ nil
!

resolveURL: anURLOrString
	" create an absolute URL from the given absolute or relative URL "

	| resolvedURL baseURL |
	[
		resolvedURL := (baseURL := self baseURL) notNil
			ifTrue: [ baseURL tfResolveURL: anURLOrString ]
			ifFalse: [ Net.URL tfFromString: anURLOrString asString ]
	] on: Error do: [ :ex |
		Log logDebug: ['Could not resolve URL: ', anURLOrString asString] level: 3
	].
	^ resolvedURL
!

resource

	self subclassResponsibility
!

resourceManager

	^ resourceManager
!

resourceManager: aTfResourceManager

	resourceManager := aTfResourceManager
!

sidebarInspector

	^ nil
!

sourceText

	^ nil
!

title

	^ title notNil
		ifTrue: [ title ]
		ifFalse: [ self defaultTitle ]
!

title: aString

	title := aString
!

url

	| d |
	^ (d := self urlDescriptor) notNil
		ifTrue: [ d url ]
		ifFalse: [ nil ]
!

urlDescriptor
	" URL descriptor that was used to fetch the resource "

	^ urlDescriptor
!

urlDescriptor: aTfURLDescriptor
	" URL descriptor that was used to fetch the resource "

	urlDescriptor := aTfURLDescriptor.
	urlDescriptor description: self description.
!

useCache
	" true if this model should be cached "

	^ true
! !

!TfResourceModel methodsFor:'comparing'!

= anotherResourceModel

	^ self class = anotherResourceModel class
		and: [ self urlDescriptor = anotherResourceModel urlDescriptor ]
! !

!TfResourceModel methodsFor:'initialize-release'!

initialize
! !

!TfResourceModel methodsFor:'private'!

resourceModelChanged

	| m |
	(m := self basicBrowserWindowModel)
		 notNil ifTrue: [ m resourceModelChanged ]
! !

!TfResourceModel methodsFor:'views'!

createEmbeddedApp

	self subclassResponsibility
!

createViewWithKeyboard: aKeyboardProcessorOrNil

	^ nil
! !

!TfResourceModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfResourceModel.st,v 1.1 2009/10/01 18:05:58 cg Exp $'
! !
