"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfBrowserWindowModel subclass:#TfResourceWindowModel
	instanceVariableNames:'resourceModel navigationModel formBuilder requestsLock requests
		maxRequests'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!

TfResourceWindowModel comment:'Model of a window that displays a resource model

Instance Variables:
	formBuilder	&lt;UIBuilder&gt;	UIBuilder used to create forms (widgets)
	maxRequests	&lt;Magnitude&gt;	max. number of requests while loading the resource model and its content
	navigationModel	&lt;TfNavigationModel&gt;	Navigation model
	requests	&lt;TfResourceRequest&gt;	All outstanding requests for a resource model
	requestsLock	&lt;RecursionLock&gt;	Lock to synchronize the requests
	resourceModel	&lt;TfResourceModel&gt;	Resource model'
!


!TfResourceWindowModel methodsFor:'accessing'!

canNavigate

	^ true
!

formBuilder

	formBuilder isNil
		ifTrue: [ formBuilder := UIBuilder new ].
	^ formBuilder
!

formBuilder: aTfFormBuilder

	formBuilder := aTfFormBuilder
!

isEmbedded

	^ true
!

isResourceWindowModel

	^ true
!

menuFor: aTfElement

	^ aTfElement defaultBrowserMenu
!

navigationModel

	navigationModel isNil
		ifTrue: [ self createNavigationModel ].
	^ navigationModel
!

resourceModel

	resourceModel isNil
		ifTrue: [ self createEmptyDocument ].
	^ resourceModel
!

resourceModel: aTfResourceModel

	resourceModel := aTfResourceModel.
	(resourceModel notNil and: [ resourceModel basicBrowserWindowModel  ~~ self ])
		ifTrue: [ resourceModel browserWindowModel: self ].
	self resourceModelChanged.
!

title

	^ self resourceModel title
!

urlDescriptor

	^ self resourceModel notNil
		ifTrue: [ self resourceModel urlDescriptor ]
		ifFalse: [ nil ]
! !

!TfResourceWindowModel methodsFor:'comparing'!

= anotherWindowModel

	^ self class = anotherWindowModel class
		and: [ self resourceModel = anotherWindowModel resourceModel ]
! !

!TfResourceWindowModel methodsFor:'loading'!

openURL: url

	Cursor wait showWhile: [
		| descriptor |
		descriptor := url tfAsURLDescriptor.
		self browserModel resourceManager 
			open: descriptor
			resourceDo: [ : resModel |
				self navigationModel visitedEntry: descriptor.
				self resourceModel: resModel
			].
	].
!

reload

	self urlDescriptor notNil
		ifTrue: [ 
			self resourceManager invalidateCacheAt: self urlDescriptor.
			self openURL: self urlDescriptor ]
! !

!TfResourceWindowModel methodsFor:'notification'!

defaultStyleChanged

	self resourceModelChanged
!

onNavigationInPageSend: methodSelector to: anObject

	self when: #navigationInPage send: methodSelector to: anObject
!

onNavigationSend: methodSelector to: anObject

	self when: #onNavigation send: methodSelector to: anObject
!

onResourceChangeSend: methodSelector to: anObject

	self when: #resourceModelChanged send: methodSelector to: anObject
!

resourceModelChanged

	self triggerEvent: #resourceModelChanged with: resourceModel with: self.
! !

!TfResourceWindowModel methodsFor:'private'!

changedNavigationEntry

	| entry |
	entry := self navigationModel currentNavigationEntry.
	entry isNil ifTrue: [ ^ self ].
	self triggerEvent: #onNavigation with: entry with: self.
!

changedNavigationInPage

	| entry |
	entry := self navigationModel currentNavigationEntry.
	entry isNil ifTrue: [ ^ self ].
	self triggerEvent: #navigationInPage with: entry with: self.
!

createEmptyDocument

	| dm |
	dm := TfDocumentModel new.
	dm htmlString: '<html/>'.
	self resourceModel: dm.
!

createNavigationModel

	navigationModel := TfNavigationModel new.
	navigationModel 
		onNavigationSend: #changedNavigationEntry to: self;
		onNavigationInPageSend: #changedNavigationInPage to: self;
		beforeNavigationSend: #cancelAllRequests to: self resourceManager.
!

privateRemoveRequest: aTfResourceRequest

	requests remove: aTfResourceRequest ifAbsent: []
! !

!TfResourceWindowModel methodsFor:'views'!

createApplication

	^ self resourceModel createEmbeddedApp
! !

!TfResourceWindowModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfResourceWindowModel.st,v 1.1 2009/10/01 18:06:09 cg Exp $'
! !
