"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Model subclass:#TfSettings
	instanceVariableNames:'properties'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!

TfSettings class instanceVariableNames:'current'

"
 No other class instance variables are inherited by this class.
"
!


!TfSettings class methodsFor:'instance creation'!

new

	^ super new initialize
! !

!TfSettings class methodsFor:'defaults'!

defaultBookmarks

	^ #(
		#('http://wiki.cs.uiuc.edu/VisualWorks'	'VisualWorks Wiki')
		#('http://www.cincomsmalltalk.com:8080/CincomSmalltalkWiki'	'Cincom Wiki')
	)
!

defaultCodeFontStyle

	| style |
	style := TfFontStyle new.
	style family: #('Courier New' 'Courier' 'fixed').
	style pixelSize: self defaultFontStyle pixelSize.
	^ style
!

defaultFontStyle

	^ TextAttributes default characterAttributes tfAsScreenFontStyle
!

defaultVersion

	^ 1
! !

!TfSettings class methodsFor:'load/save'!

filename

	^ 'twoflower.xml' asFilename
!

load
	"Twoflower.TfSettings load"
	"Twoflower.TfSettings new save"

	| settingsCollection |
	self filename asFilename exists ifFalse: [ ^ nil ].
	settingsCollection := Tools.XMLConfigFileSupport.XMLConfigFile loadFrom: self filename.
	^ settingsCollection size = 1
		ifTrue: [ settingsCollection first postLoad; yourself ]
		ifFalse: [ nil ].
!

loadOrNew
	
	| settings |
	^ (settings := self load) notNil
		ifTrue: [ settings ]
		ifFalse: [ self new ].
! !

!TfSettings methodsFor:'accessing'!

bookmarks

	^ self propertyAt: #bookmarks
!

bookmarks: aTfBookmarkContainer

	self propertyAt: #bookmarks put: aTfBookmarkContainer.
	self triggerEvent: #bookmarksChanged with: aTfBookmarkContainer.
!

codeFontStyle

	^ self propertyAt: #codeFontStyle ifAbsent: [ self class defaultCodeFontStyle clone ]
!

codeFontStyle: aTfFontStyle

	self propertyAt: #codeFontStyle put: aTfFontStyle.
	self triggerEvent: #styleChanged.
!

defaultFontStyle

	^ self propertyAt: #defaultFontStyle ifAbsent: [ self class defaultFontStyle clone ]
!

defaultFontStyle: aTfFontStyle

	self propertyAt: #defaultFontStyle put: aTfFontStyle.
	self triggerEvent: #styleChanged.
!

defaultPixelSize: anInteger
	" comfy method "

	| fs cfs |
	fs := self class defaultFontStyle clone .
	fs pixelSize: anInteger.
	self propertyAt: #defaultFontStyle put: fs.
	cfs := self class defaultCodeFontStyle clone.
	cfs pixelSize: anInteger.
	self propertyAt: #codeFontStyle put: cfs.
	self triggerEvent: #styleChanged.
!

loadImages

	^ self propertyAt: #loadImages ifAbsent: [ true ]
!

loadImages: aBoolean

	self propertyAt: #loadImages put: aBoolean.
	self changed: #loadImages.
!

showSidebar

	^ self propertyAt: #showSidebar ifAbsent: [ true ]
!

showSidebar: aBoolean

	self propertyAt: #showSidebar put: aBoolean.
	self changed: #showSidebar.
! !

!TfSettings methodsFor:'initialize-release'!

initialize

	self initializeDefaults.
!

initializeBookmarks

	| bc |
	bc := TfBookmarkContainer description: 'Bookmarks'.
	self class defaultBookmarks do: [ : array | 
		bc addBookmark: (TfURLDescriptor url: (array at: 1) description: (array at: 2))
	].
	self bookmarks: bc.
!

initializeDefaults

	self version: self class defaultVersion.
	self initializeBookmarks.
!

postLoad

	properties isNil 
		ifTrue: [ self initializeDefaults ].
! !

!TfSettings methodsFor:'load/save'!

save

	| config |
	config := Tools.XMLConfigFileSupport.XMLConfigFile filename: self class filename.
	config add: self.
	config saveConfiguration.
! !

!TfSettings methodsFor:'notification'!

onBookmarksChangeSend: methodSelector to: anObject

	self when: #bookmarksChanged send: methodSelector to: anObject
!

onStyleChangeSend: methodSelector to: anObject

	self when: #styleChanged send: methodSelector to: anObject
!

update: anAspect with: param from: sender

	(sender == ObjectMemory and: [ (#(aboutToQuit aboutToSnapshot) includes: anAspect) ])
		ifTrue: [
			self save.
		].
! !

!TfSettings methodsFor:'private'!

properties

	properties isNil ifTrue: [ properties := Dictionary new ].
	^ properties
!

propertyAt: aKey 

	^ self propertyAt: aKey ifAbsent: [ nil ]
!

propertyAt: aKey ifAbsent: absentBlock

	^ self properties at: aKey ifAbsent: absentBlock
!

propertyAt: aKey put: aValue

	self properties at: aKey put: aValue
!

version

	^ self propertyAt: #version
!

version: aNumber

	self propertyAt: #version put: aNumber
! !

!TfSettings class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfSettings.st,v 1.1 2009/10/01 18:06:14 cg Exp $'
! !
