"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfAbstractTableElement subclass:#TfTableCellElement
	instanceVariableNames:'widthLayout columnIndex rowElement rowHeight noWrap
		spannedColumns spannedRows spannedRowElements'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfTableCellElement class methodsFor:'accessing'!

elementsToCloseClasses
	
	^ Array
		with: self
!

elementsToCloseStopClasses
	
	^ Array
		with: TfTableElement
		with: TfTableRowElement
!

remarks

	^ '<i>rowspan</i> causes wrong cell bounds. <i>align</i> does not affect text alignment'
!

supportedAttributes

	^ super supportedAttributes, #('width' 'height' 'colspan' 'rowspan')
!

tagNames

	^ #('td')
! !

!TfTableCellElement methodsFor:'DOM fixing'!

tfFixNodes

	self ensureParent: [ : el | el tfIsTableRow ] default: 'tr'.
	super tfFixNodes.
! !

!TfTableCellElement methodsFor:'accessing'!

addSpannedRowElement: aTfRowElement

	self spannedRowElements add: aTfRowElement
!

columnIndex
	" index of the first column of this cell "

	columnIndex isNil
		ifTrue: [ 
			Log logDebug: ['Encountered a table cell without column index'].
			columnIndex := 1.
		].
	^ columnIndex
!

columnIndex: anInteger

	columnIndex := anInteger.
!

columnInterval

	^ self columnIndex to: self columnIndex + self spannedColumns - 1
!

rowElement
	" should be my parent, but you'll never know... "

	^ rowElement
!

rowElement: aTfRowElement
	" should be my parent, but you'll never know... "

	rowElement := aTfRowElement
!

rowIndex

	^ self rowElement notNil
		ifTrue: [ self rowElement rowIndex ]
		ifFalse: [ 1 ]
!

rowInterval

	^ self rowIndex to: self rowIndex + self spannedRows - 1
!

spannedColumns
	" number of columns spanned by this cell "

	spannedColumns isNil
		ifTrue: [ spannedColumns := self integerValueOfAttribute: 'colspan' ifAbsent: [ 1 ] ].
	^ spannedColumns
!

spannedRowElements
	" row elements spanned by me "

	spannedRowElements isNil
		ifTrue: [ spannedRowElements := OrderedCollection with: self rowElement ].
	^ spannedRowElements
!

spannedRows
	" number of columns spanned by this cell "

	spannedRows isNil
		ifTrue: [ spannedRows :=  self integerValueOfAttribute: 'rowspan' ifAbsent: [ 1 ] ].
	^ spannedRows
!

tfAdaptToTable: aTfTableEnumerator 

	spannedRows := self spannedRows min: aTfTableEnumerator numberOfRows - self rowIndex + 1.
	spannedColumns := aTfTableEnumerator spannedColumnsOf: self.
	" stop enumeration here "
!

tfEnumerateTable: aTfTableEnumerator
	" compute row/column indexes of cells/rows "

	aTfTableEnumerator enumCell: self.
	" stop enumeration here "
!

tfIsTableCell

	^ true
! !

!TfTableCellElement methodsFor:'menu actions'!

exportTableOn: aStream columns: numberOfColumns emit: aBoolean

	aStream nextPut: $".
	self elements do: [ : el |
		el exportTableOn: aStream columns: numberOfColumns emit: true.
	].
	aStream nextPut: $".
	(self spannedColumns - 1) timesRepeat: [ aStream nextPut: $; ].
! !

!TfTableCellElement methodsFor:'parsing'!

parseValues

	super parseValues.
	self valueOfAttribute: 'width' do: [ :val |
		widthLayout := self valueReader parseTableWidth: val ].
	self valueOfAttribute: 'height' do: [ :val |
		rowHeight := self valueReader parseInteger: val ].
	noWrap := self isAttributeSet: 'nowrap'
! !

!TfTableCellElement methodsFor:'styles'!

applyCellStyleTo: aTfStyle

	rowHeight notNil
		ifTrue: [ aTfStyle tableStyle rowHeight: rowHeight ].
	widthLayout notNil
		ifTrue: [ aTfStyle tableCellStyle widthLayout: widthLayout ].
	noWrap notNil
		ifTrue: [ aTfStyle tableCellStyle noWrap: noWrap ].
!

modifyStyle: aTfStyle
	" modify the style according to the local style definitions "

	super modifyStyle: aTfStyle.
	self applyCellStyleTo: aTfStyle.
! !

!TfTableCellElement methodsFor:'views'!

tfViewClass

	^ TfTableCellView
! !

!TfTableCellElement class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableCellElement.st,v 1.1 2009/10/01 18:50:49 cg Exp $'
! !
