"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfAbstractTableView subclass:#TfTableCellView
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfTableCellView methodsFor:'accessing'!

addMinimalWidthTo: columnWidths for: renderContext 

	| minimalWidth |
	self style tableCellStyle noWrap
		ifTrue: [
			self addPreferredWidthTo: columnWidths for: renderContext.
		]
		ifFalse:  [
			minimalWidth := (self minimalExtentFor: renderContext) x.
			self addWidth: minimalWidth to: columnWidths for: renderContext
		].
!

addPreferredWidthTo: columnWidths for: renderContext 

	| ext |
	ext := self preferredExtentFor: renderContext.
	self addWidth: ext x to: columnWidths for: renderContext.
!

addWidthLayoutsTo: columnWidthLayouts

	| cellWidthLayout i numColumns layout |
	cellWidthLayout := self style tableCellStyle widthLayout.
	cellWidthLayout notNil
		ifTrue: [
			i := self model columnIndex.
			numColumns := self model spannedColumns.
			layout := cellWidthLayout dividedBy: numColumns.
			i to: i + numColumns - 1 do: [ :j |
				(columnWidthLayouts at: j) isNil
					ifTrue: [ columnWidthLayouts at: j put: layout ].
			]
		].
!

componentBounds

	^ self bounds insetBy: self inset
!

computeComponents

	| contentsView |
	contentsView := (TfTableCellContentsView model: self model) tfWrapped.
	contentsView container: self.
	components := OrderedCollection with: contentsView.
! !

!TfTableCellView methodsFor:'private'!

addWidth: cellWidth to: columnWidths for: renderContext 

	| numColumns width |
	numColumns := self model spannedColumns.
	width := numColumns > 0
		ifTrue: [ cellWidth // numColumns ]
		ifFalse: [ cellWidth ].
	(self model columnInterval) do: [ :j | | w |
		((w := columnWidths at: j) isNil or: [ w < width ])
			ifTrue: [ columnWidths at: j put: width ].
	]
!

columnsWidthFor: renderContext
	" combined width of all columns spanned by me, 
	including extra pixels for space between columns "

	| w i numColumns tableLayout |
	tableLayout := renderContext layoutContext tableLayout.
	w := 0.
	i := self model columnIndex.
	i isNil ifTrue: [ ^ 0 ].
	numColumns := self model spannedColumns.
	i to: i + numColumns - 1 do: [ :j |
		w := w + (tableLayout columnWidthAt: j).
	].
	(w > 0 and: [ numColumns > 1 ])
		ifTrue: [
			w := (numColumns - 1) * self style tableStyle cellSpacing + w.
		].
	^ w
!

height: anInteger

	preferredBounds height: (anInteger max: preferredBounds height).
!

inset

	^ self cellPadding + self borderWidth
!

ownAdditionalExtent

	^ 2 * self inset
! !

!TfTableCellView methodsFor:'rendering'!

finishOwnRendering: renderContext

	| lineLayout layoutBounds |
	super finishOwnRendering: renderContext.
	layoutBounds := preferredBounds tfDeepCopy.
	self model spannedRows > 1
		ifTrue: [ layoutBounds height: 1 ].
	renderContext layoutContext tableLayout 
			preferredCellHeight: preferredBounds height 
			for: self xmlNode.
	lineLayout := TfTableCellLayout 
					for: self 
					bounds: layoutBounds
					columnInterval: self model columnInterval
					spannedRows: self model spannedRows.
	lineLayout addToLayoutContext: renderContext layoutContext.
!

preferredRowHeight

	^ preferredBounds height // self model spannedRows
!

setupComponentRendering: renderContext

	| layoutContext height width |
	layoutContext := renderContext pushLayoutContext.
	width := self columnsWidthFor: renderContext.
	height := layoutContext bounds height.
	layoutContext 
		bounds: (0@0 extent: width@height);
		insetBy: self inset;
		reset.
	height := 0.
	self components do: [ :componentWrapper |
		self setupComponentRendering: componentWrapper for: renderContext.
		height := height max: componentWrapper component preferredBounds height.
	].
	renderContext popLayoutContext.
	preferredBounds := 0@0 extent: width@height.
!

startOwnRendering: renderContext

	| i |
	super startOwnRendering: renderContext.
	i := self model columnIndex.
	renderContext layoutContext startColumnIndex: i.
	self tfLocalTranslation: renderContext layoutContext position.
!

updateBoundsFor: renderContext in: aRectangle
	" table rendering is finished. now update the bounds of me "

	| height box |
	height := renderContext layoutContext tableLayout cellHeightFor: self xmlNode.
	height := height + (self xmlNode spannedRows * (2 * self inset)).
	self height: height.
	box := 0@0 extent: self preferredBounds extent - (2 * self inset).
	self components do: [ : c | c tfWrappedComponent updateBoundsFor: renderContext in: box ].
! !

!TfTableCellView methodsFor:'testing'!

isTableCell

	^ true
! !

!TfTableCellView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableCellView.st,v 1.1 2009/10/01 19:44:42 cg Exp $'
! !
