"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfAbstractTableElement subclass:#TfTableElement
	instanceVariableNames:'cellPadding cellSpacing widthLayout columnElements
		numberOfColumns numberOfRows rowElements'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfTableElement class methodsFor:'accessing'!

allowsPCData
	
	^ false
!

supportedAttributes

	^ super supportedAttributes, #('cellspacing' 'cellpadding' 'width' 'border' 'bordercolor')
!

tagNames

	^ #('table')
! !

!TfTableElement methodsFor:'DOM fixing'!

tfFixNodes

	self ensureChildren: [ : el | el tfIsTableElement or: [ el tfIsColElement ]] default: 'tr'.
	super tfFixNodes.
! !

!TfTableElement methodsFor:'accessing'!

addColumnElement: columnElement
	" adds a column. returns it's column index "

	| colIndex |
	colIndex := self columnElements isEmpty
		ifTrue: [ 1 ]
		ifFalse: [ self columnElements last lastColumnIndex + 1 ].
	self columnElements add: columnElement.
	^ colIndex
!

addRowElement: aTfRowElement

	self rowElements add: aTfRowElement
!

addedToDocument: aTfDocument

	| enumerator |
	super addedToDocument: aTfDocument.
	enumerator := TfTableEnumerator for: self.
	self elements do: [ : node | node tfEnumerateTable: enumerator ].
	numberOfColumns := enumerator numberOfColumns.
	numberOfRows := enumerator numberOfRows.
	self elements do: [ : node | node tfAdaptToTable: enumerator ].
!

cellPadding

	^ cellPadding
!

cellSpacing

	^ cellSpacing
!

columnElements

	columnElements isNil
		ifTrue: [ columnElements := OrderedCollection new ].
	^ columnElements
!

hspace

	^ self integerValueOfAttribute: 'hspace' ifAbsent: [ 0 ]
!

numberOfColumns

	^ numberOfColumns
!

numberOfRows

	^ numberOfRows
!

rowElements

	rowElements isNil
		ifTrue: [ rowElements := OrderedCollection new ].
	^ rowElements
!

tableColumnElements

	^ self 
		tfNodesMatching: [ : n | n tfIsColElement ]
		upTo: [ : n | n tfIsTableCell or: [ n tfIsTableRow ]]
!

tableRowElements

	^ self 
		tfNodesMatching: [ : n | n tfIsTableRow ]
		upTo: [ : n | n tfIsTableCell ]
!

tfAdaptToTable: aTfTableEnumerator

	" stop enumeration "
!

tfEnumerateTable: aTfTableEnumerator
	" compute row/column indexes of cells/rows "

	" stop enumeration "
!

tfIsTable

	^ true
!

tfTableElement

	^ self
!

vspace

	^ self integerValueOfAttribute: 'vspace' ifAbsent: [ 0 ]
!

widthLayout

	^ widthLayout
! !

!TfTableElement methodsFor:'initialize'!

initialize

	super initialize.
	numberOfColumns := 1.
	numberOfRows := 1
! !

!TfTableElement methodsFor:'menu actions'!

exportTable

	| fn file str |
	fn := Dialog requestNewFileName:  'Export table:' default: 'table.csv'.
	(fn isNil or: [ fn isEmpty ]) ifTrue: [ ^ self ].
	file := fn asFilename.
	[
		str := file asFilename writeStream text.
		[
			self elements do: [ : el |
				el exportTableOn: str columns: self numberOfColumns emit: false.
			].
		] ensure: [
			str close
		].
	] on: Error do: [ : ex |
		Dialog warn: 'Unable to save file: ', ex description.
	].
!

exportTableOn: aStream columns: numColumns emit: aBoolean

	aStream nextPutAll: self characterData.
! !

!TfTableElement methodsFor:'parsing'!

parseValues

	super parseValues.
	cellSpacing := self integerValueOfAttribute: 'cellspacing'.
	cellPadding := self integerValueOfAttribute: 'cellpadding'.
	self valueOfAttribute: 'width' do: [ :val |
		widthLayout := self valueReader parseTableWidth: val ].
	borderWidth := self integerValueOfAttribute: 'border'.
! !

!TfTableElement methodsFor:'styles'!

applyAlignmentStyleTo: aTfStyle
	" no "
!

applyColumnsStyleTo: aTfStyle

	columnElements notNil
		ifTrue: [ columnElements do: [ :colElement | colElement modifyStyle: aTfStyle ]].
!

applyTableStyleTo: aTfStyle

	| tableStyle |
	tableStyle := aTfStyle tableStyle.
	cellSpacing notNil ifTrue: [ tableStyle cellSpacing: cellSpacing ].
	cellPadding notNil ifTrue: [ tableStyle cellPadding: cellPadding ].
	widthLayout notNil ifTrue: [ tableStyle widthLayout: widthLayout ].
	borderWidth notNil ifTrue: [ aTfStyle backgroundStyle borderWidth: borderWidth ].
!

modifyStyle: aTfStyle
	" modify the style according to the local style definitions "

	super modifyStyle: aTfStyle.
	self applyTableStyleTo: aTfStyle.
	self applyColumnsStyleTo: aTfStyle.
! !

!TfTableElement methodsFor:'views'!

tfComponents

	^ super tfComponents "select: [ :c | c tfIsTableElement  ]."
!

tfViewClass

	^ TfTableView
! !

!TfTableElement class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableElement.st,v 1.1 2009/10/01 18:51:26 cg Exp $'
! !
