"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

Object subclass:#TfTableEnumerator
	instanceVariableNames:'columnIndex rowIndex spannedRowsDictionary numberOfColumns
		spannedColumnsDictionary tableElement rowElement
		rowSpanStartIndex'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfTableEnumerator class methodsFor:'instance creation'!

for: aTfTableElement

	^ self new 
		initializeFor: aTfTableElement;
		yourself
! !

!TfTableEnumerator methodsFor:'accessing'!

numberOfColumns

	^ numberOfColumns max: columnIndex
!

numberOfRows

	^ rowIndex
!

spannedColumnsOf: aTableCellElement
	" some rows might have less or more columns than the table "

	| spannedColumns minColumns assoc |
	spannedColumns := aTableCellElement spannedColumns.
	assoc := spannedColumnsDictionary at: aTableCellElement rowIndex ifAbsent: [ nil ].
	(assoc notNil and: [ assoc key = aTableCellElement ])
		ifTrue: [ 	spannedColumns := spannedColumns + self numberOfColumns - assoc value ].
	" remaining number of columns "
	minColumns := self numberOfColumns - aTableCellElement columnIndex + 1.
	^ spannedColumns min: minColumns
!

tableElement

	^ tableElement
! !

!TfTableEnumerator methodsFor:'enumeration'!

enumCell: aTableCellElement 
	" record columnspan/rowspan information of a cell "

	| stopColumnIndex cell |
	" apply rowspan of previous cells to start index "
	columnIndex := columnIndex + 1.
	[ (cell := spannedRowsDictionary at: columnIndex ifAbsent: [ nil ]) notNil ] whileTrue: [
		columnIndex := columnIndex + cell spannedColumns.
	].
	aTableCellElement 
		columnIndex: columnIndex;
		rowElement: rowElement.
	stopColumnIndex := columnIndex + aTableCellElement spannedColumns - 1.
	" apply rowspan "
	rowElement notNil 
		ifTrue: [ rowElement spannedCellRowsMax: aTableCellElement spannedRows ].
	columnIndex to: stopColumnIndex do: [ : colIndex |
		spannedRowsDictionary at: colIndex put: aTableCellElement.
	].
	columnIndex := stopColumnIndex.
	(spannedRowsDictionary at: columnIndex ifAbsent: [ nil ]) isNil
		ifTrue: [ spannedColumnsDictionary at: rowIndex put: aTableCellElement -> columnIndex ].
!

enumRow: aTableRowElement 
	" record information of a row "

	rowIndex := rowIndex + 1.
	rowElement := aTableRowElement.
	aTableRowElement rowIndex: rowIndex.
	tableElement addRowElement: aTableRowElement.
	" update rowspan "
	spannedRowsDictionary copy keysAndValuesDo: [ : colIndex : cellElement |
		cellElement rowInterval last < rowIndex
			ifTrue: [ spannedRowsDictionary removeKey: colIndex ]
			ifFalse: [
				aTableRowElement addRowSpanCellsAbove: cellElement.
				cellElement addSpannedRowElement: aTableRowElement.
			].
	].
	(spannedRowsDictionary allSatisfy: [ : c | c == 0 ])
		ifTrue: [ rowSpanStartIndex := rowIndex ].
	" apply column information "
	numberOfColumns := numberOfColumns max: columnIndex.
	columnIndex := 0.
! !

!TfTableEnumerator methodsFor:'initialize-release'!

initializeFor: aTfTableElement

	tableElement := aTfTableElement.
	columnIndex := 0.
	rowIndex := 0.
	numberOfColumns := 0.
	rowSpanStartIndex := 0.
	spannedRowsDictionary := Dictionary new.
	spannedColumnsDictionary := Dictionary new.
! !

!TfTableEnumerator class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableEnumerator.st,v 1.1 2009/10/01 18:51:42 cg Exp $'
! !
