"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfBoxElementLayout subclass:#TfTableLayout
	instanceVariableNames:'columnWidths columnWidthLayouts preferredColumnWidths
		minimalColumnWidths proportionSum availableTableWidth
		preferredTableWidth reservedTableWidth targetTableWidth
		preferredSingleSpanCellHeights singleSpanRowHeights cellHeights
		cellSpacing cellPadding borderWidth'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfTableLayout methodsFor:'accessing'!

availableWidth: anInteger

	availableTableWidth := anInteger
!

columnOffsetAt: columnIndex

	| w |
	w := 0.
	1 to: columnIndex - 1 do: [ : i | 
		w := w + (columnWidths at: i)
	].
	columnIndex > 1
		ifTrue: [ w := (columnIndex - 1) * self cellSpacing + w ].
	^ w
!

columnWidths

	^ columnWidths
!

preferredCellHeight: cellHeight for: aTfCellElement
	" record the preferred height of the cell "

	| rowSpan rowIndex |
	aTfCellElement columnInterval do: [ : i |
		(self cellHeights at: aTfCellElement rowIndex) at: i put: aTfCellElement spannedRows@cellHeight
	].

	rowSpan := aTfCellElement spannedRows.
	rowIndex := aTfCellElement rowIndex.
	rowSpan = 1
		ifTrue: [ 
			self preferredSingleSpanCellHeights 
				at: rowIndex 
				put: ((self preferredSingleSpanCellHeights at: rowIndex) max: cellHeight)
		].
!

tableElement

	^ self parentView xmlNode
!

tableView

	^ self parentView
! !

!TfTableLayout methodsFor:'debug'!

logLayoutFor: renderContext

	| resultingTableWidth s |
	s := self tableElement printString.
	s replaceAll: Character cr with: Character space.
	s := s copyFrom: 1 to: (100 min: s size).
	Log cr; log: 	'Layout for: ', s , ': '.
	Log show:
				'Target Width: ', targetTableWidth printString, 
				' Available: ', availableTableWidth printString, 
				' Actual: ', self tableWidth printString.
	resultingTableWidth := (columnWidths inject: 0 into: [ : i : j | i + j ]) + self reservedTableWidth.
	Log show: 'Resulting table width: ', resultingTableWidth printString;cr.
	Log show: 'Actual columns: '.
	columnWidths do: [ :w | Transcript show: w printString, '  ' ].
	Log cr.
	Log show: 'Minimal columns: '.
	minimalColumnWidths do: [ :w | Transcript show: w printString, '  ' ].
	Log cr.
	Log show: 'Preferred columns: '.
	minimalColumnWidths do: [ :w | Transcript show: w printString, '  ' ].
	Log cr.
	Log show: 'Layouts:'.
	columnWidthLayouts do: [ :l | Transcript show: l printString, '  ' ].
	Log cr.
! !

!TfTableLayout methodsFor:'layout'!

cellHeightFor: aTfCellElement

	| cellHeight interval |
	interval := aTfCellElement rowInterval.
	cellHeight := interval
		inject: 0
		into: [ : h : j | h + (self singleSpanRowHeightAt: j) ].
	^ cellHeight + ((interval size - 1) * (self cellSpacing))
!

computeColumnLayoutFor: renderContext 

	self prepareLayoutFor: renderContext.
	targetTableWidth := (self tableLayoutFor: renderContext) notNil 
				ifTrue: [self targetTableWidthFor: renderContext]
				ifFalse: [self proportionalTableWidthFor: renderContext].
	targetTableWidth > 0 
		ifTrue: [
			self computeUnscaledColumnWidthsFor: renderContext.
			self scaleColumnWidthsFor: renderContext.
			self shiftColumnSpace.
		].
	width := self columnWidths inject: 0 into: [ : s : w | s + w].
	width := width +  ((self tableView numberOfColumns - 1) * self cellSpacing).
	Log debugLevel: 3 do: [self logLayoutFor: renderContext]
!

singleSpanRowHeightAt: rowIndex

	^ self singleSpanRowHeights 
		at: rowIndex
		ifAbsentPut: [ self preferredSingleSpanCellHeights at: rowIndex ]
! !

!TfTableLayout methodsFor:'layout - private'!

applyAlignTo: aTfLayoutContext

	align isNil
		ifTrue: [ aTfLayoutContext nextLine ]
		ifFalse: [ super applyAlignTo: aTfLayoutContext ]
!

applyPositionTo: aTfLayoutContext

	self align = #left
		ifTrue: [ super applyPositionTo: aTfLayoutContext ]
!

borderWidth

	borderWidth isNil
		ifTrue: [ borderWidth :=  self tableView style backgroundStyle borderWidth ].
	^ borderWidth
!

cellPadding

	cellPadding isNil
		ifTrue: [ cellPadding :=  self tableView style tableStyle cellSpacing ].
	^ cellPadding
!

cellSpacing

	cellSpacing isNil
		ifTrue: [ cellSpacing :=  self tableView style tableStyle cellSpacing ].
	^ cellSpacing
! !

!TfTableLayout methodsFor:'private - column layout'!

cellLayoutsDo: aBlock

	1 to: self tableView numberOfColumns do: [ :i | | cellWidthLayout |
		cellWidthLayout := self columnWidthLayouts at: i.
		aBlock value: cellWidthLayout  value: i.
	].
!

cellWidthsDo: aBlock

	1 to: self tableView numberOfColumns do: [ :i | | cellWidth |
		cellWidth := self columnWidths at: i.
		aBlock value: cellWidth  value: i.
	].
!

columnLayoutsDo: aBlock

	1 to: self tableView numberOfColumns do: [ :i | | cellWidthLayout |
		cellWidthLayout := self columnWidthLayouts at: i.
		aBlock value: cellWidthLayout  value: i.
	].
!

columnWidthAt: i 

	^ columnWidths at: i
!

columnWidthAt: i put: cellWidth

	^ columnWidths at: i put: (cellWidth max: (minimalColumnWidths at: i)) truncated.
!

columnWidthLayouts

	^ columnWidthLayouts
!

completeLayouts
	" create layouts for columns that have not specified a layout yet "

	self columnLayoutsDo: [ :layout :i | | newLayout |
		layout isNil
			ifTrue: [ 
				newLayout := TfPreferredColumnWidthLayout pixelSize: (preferredColumnWidths at: i).
				self columnWidthLayouts at: i put: newLayout
			].
	].
!

computeMinimalColumnWidthsFor: renderContext
	" compute minimal width of all columns. this is a hard constraint, 
	columns will never ever be smaller than the minimal width "

	minimalColumnWidths := Array new: self tableView numberOfColumns.
	minimalColumnWidths atAllPut: 0.
	self tableView rows do: [ :rowWrapper |
		rowWrapper tfWrappedComponent cells do: [ :cell | 
			cell tfWrappedComponent addMinimalWidthTo: minimalColumnWidths for: renderContext.
		].
	].
!

computePreferredColumnWidthsFor: renderContext
	" compute preferred column widths for all columns "

	preferredColumnWidths := minimalColumnWidths copy.
	self tableView rows do: [ :rowWrapper |
		rowWrapper tfWrappedComponent cells do: [ :cell | 
			cell tfWrappedComponent addPreferredWidthTo: preferredColumnWidths for: renderContext.
		].
	].
!

computeProportionSumFor: renderContext
	" compute the sum of all proportions. this is necessary for column layout
	where the sum is not known in advance (<colgroup width=4*>) "

	proportionSum := 0.
	self columnLayoutsDo: [ :cellWidthLayout :i |
		cellWidthLayout notNil
			ifTrue: [ 	proportionSum := proportionSum + cellWidthLayout sizeFactor ].
	].
!

computeUnscaledColumnWidthsFor: renderContext

	self cellLayoutsDo: [ :layout :i | | minFactor cellWidth |
		minFactor := (minimalColumnWidths at: i) / targetTableWidth.
		cellWidth := layout
						sizeRelativeTo: targetTableWidth
						minFactor: minFactor
						factorSum: proportionSum.
		self columnWidthAt: i put: cellWidth.
	].
!

fixedColumnsWidthFor: renderContext
	" sum of all columns with fixed width "

	^ self columnWidthLayouts
		inject: 0
		into: [ :sum :layout | sum + layout pixelSize ].
!

preferredTableWidthFor: renderContext
	" preferred width of the table "

	preferredTableWidth isNil
		ifTrue: [ 
			preferredTableWidth := preferredColumnWidths inject: 0 into: [ :x :y | x + y ].
		].
	^ preferredTableWidth
!

prepareLayoutFor: renderContext

	columnWidths := Array new: self tableView numberOfColumns.
	columnWidths atAllPut: 0.
	columnWidthLayouts := renderContext layoutContext columnWidthLayouts.
	self computeMinimalColumnWidthsFor: renderContext.
	self computePreferredColumnWidthsFor: renderContext.
	self completeLayouts.
	self computeProportionSumFor: renderContext.
!

proportionalTableWidthFor: renderContext
	" target width of the table computing by using proportional colum constraints "

	| ptw fixedFactor fixedColumnsWidth tw |
	fixedFactor := 1.
	fixedColumnsWidth := self fixedColumnsWidthFor: renderContext.
	tw := self targetTableWidthFor: renderContext.
	tw = 0
		ifTrue: [ ^  tw ].
	ptw := tw.
	" compute max. table width required by the columns "
	self columnLayoutsDo: [ :layout :i | | minFactor requiredWidth|
		minFactor := (minimalColumnWidths at: i) / tw.
		requiredWidth := layout
			requiredSizeFor: fixedColumnsWidth
			minFactor: minFactor
			factorSum: proportionSum.
		ptw := ptw max: requiredWidth.
		fixedFactor := fixedFactor - (layout factorForMinFactor: minFactor factorSum: proportionSum)
	].
	" compute max. table width required by the fixed part of the table"
	fixedFactor > 0
		ifTrue: [ ptw := ptw max: (fixedColumnsWidth / fixedFactor) ].
	^ ptw min: availableTableWidth - self reservedTableWidth
!

reservedTableWidth
	" combined width of all regions that are reserved (e.g cell spacing) "

	reservedTableWidth isNil
		ifTrue: [ reservedTableWidth := self tableView ownAdditionalExtent x ].
	^ reservedTableWidth
!

scaleColumnWidthsFor: renderContext
	" scale columns if needed "
	" only scale fixed columns (proportional columns are already properly scaled) "

	| scale fixedWidth scaledWidth |
	fixedWidth := 0.
	scaledWidth := 0.
	self cellLayoutsDo: [ :layout :i |
		layout isFixedLayout
			ifTrue: [ layout isSoftLayout ifTrue: [ fixedWidth := fixedWidth + (columnWidths at: i) ]]
			ifFalse: [ scaledWidth := scaledWidth + (columnWidths at: i) ]
	].
	(fixedWidth > 0 and: [ fixedWidth ~= targetTableWidth ])
		ifTrue: [
			scale := (targetTableWidth - scaledWidth) / fixedWidth.
			self columnLayoutsDo: [:layout :i | | cellWidth |
				(layout isFixedLayout and: [ layout isSoftLayout ])
					ifTrue: [
						cellWidth := columnWidths at: i.
						self columnWidthAt: i put: (cellWidth * scale).
					].
			]
		].
!

shiftColumnSpace
	" if the proportional columns need too much space, then shift some space "

	| requiredSpace availableSpace shiftedSpace shiftFactor remainder |
	requiredSpace := 0.
	availableSpace := 0.

	" detect available/required space "
	self cellWidthsDo: [ :cellWidth :i | | minCellWidth delta |
		minCellWidth := minimalColumnWidths at: i.
		(delta := cellWidth - minCellWidth) > 0
			ifTrue: [
				availableSpace := availableSpace + delta.
			] ifFalse: [
				requiredSpace := requiredSpace - delta
			].
	].
	requiredSpace := requiredSpace max: (self tableWidth - availableTableWidth + self reservedTableWidth).
	shiftedSpace := availableSpace min: requiredSpace.
	shiftFactor := availableSpace > 0
			ifTrue: [ shiftedSpace  / availableSpace ]
			ifFalse: [ 0 ].
	remainder := targetTableWidth \\ self tableView numberOfColumns.
	" shift space"
	self cellWidthsDo: [ :cellWidth :i | | minCellWidth delta newCellWidth |
		minCellWidth := minimalColumnWidths at: i.
		newCellWidth := (delta := cellWidth - minCellWidth) > 0
			ifTrue: [ cellWidth - (shiftFactor * delta) ]
			ifFalse: [ minCellWidth ].
		i < remainder ifTrue: [ newCellWidth := newCellWidth + 1 ].
		self columnWidthAt: i put: newCellWidth.
	].
!

tableLayout

	^ self tableView style tableStyle widthLayout
!

tableLayoutFor: renderContext

	^ self tableView style tableStyle widthLayout
!

tableWidth

	^ columnWidths inject: 0 into: [ :x :y | x + y ]
!

targetTableWidthFor: renderContext
	" target width of the table "

	| tableLayout tw |
	tableLayout := (self tableLayoutFor: renderContext).
	tw := tableLayout notNil
		ifTrue: [ (tableLayout sizeRelativeTo: availableTableWidth) - self reservedTableWidth ]
		ifFalse: [ self preferredTableWidthFor: renderContext ].
	^ tw
! !

!TfTableLayout methodsFor:'private - row layout'!

cellHeights

	cellHeights isNil
		ifTrue: [ 	
			cellHeights := Array new: self tableElement numberOfRows.
			1 to: cellHeights size do: [ : i | | row |
				row := Array new: self tableElement numberOfColumns.
				row atAllPut: 0@0.
				cellHeights at: i put: row.
			]
		].
	^ cellHeights
!

computeRowHeights

	self tableElement rowElements do: [ : rowElement | 	
		| interval neededHeight currentHeight delta |
		interval := rowElement spannedCellRowInterval.
		neededHeight := self preferredHeightOfRowInterval: interval.
		currentHeight := self singleSpanRowHeightOfInterval: interval.
		(((neededHeight - currentHeight) // interval size) max: 0).
		delta := (((neededHeight - currentHeight) // interval size) max: 0).
		interval do: [ : j |
			self singleSpanRowHeights 
					at: j 
					put: (self singleSpanRowHeightAt: j) + delta
		].
	].
!

preferredHeightOfRowInterval: rowInterval

	| ph |
	ph := 0.
	1 to: self tableElement numberOfColumns do: [ : i | | ch |
		ch := 0.
		rowInterval do: [ : j | | p |
			p := ((self cellHeights at: j) at: i).
			(p x + j - 1) <= rowInterval last
				ifTrue: [ ch := ch + p y ].
			ch := ch max: (self preferredSingleSpanCellHeights at: j).
		].
		ph := ph max: ch.				
	].
	^ ph
!

preferredSingleSpanCellHeights
	" preferred max. height of cells spanning a single row "

	preferredSingleSpanCellHeights isNil
		ifTrue: [ 
			preferredSingleSpanCellHeights := Array new: self tableElement numberOfRows.
			preferredSingleSpanCellHeights atAllPut: 0.
		].
	^ preferredSingleSpanCellHeights
!

singleSpanRowHeightOfInterval: rowInterval

	^ rowInterval 
		inject: 0
		into: [ : h : j | h + (self singleSpanRowHeightAt: j) ]
!

singleSpanRowHeights
	" preferred height of rows, not counting multispan cells "

	singleSpanRowHeights isNil
		ifTrue: [ 	
			singleSpanRowHeights := Dictionary new.
			self computeRowHeights.
		].
	^ singleSpanRowHeights
! !

!TfTableLayout class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableLayout.st,v 1.1 2009/10/01 18:52:53 cg Exp $'
! !
