"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfAbstractTableElement subclass:#TfTableRowElement
	instanceVariableNames:'rowHeight rowIndex spannedCellRows rowSpanCellsAbove'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-HTML'
!


!TfTableRowElement class methodsFor:'accessing'!

allowedChildElements
	
	^ #('th' 'td')
!

allowsPCData
	
	^ false
!

elementsToCloseClasses
	
	^ Array
		with: self
		with: TfColGroupElement
		with: TfColElement
!

elementsToCloseStopClasses
	
	^ Array
		with: TfTableElement
!

supportedAttributes

	^ super supportedAttributes, #('height')
!

tagNames

	^ #('tr')
! !

!TfTableRowElement methodsFor:'DOM fixing'!

tfFixNodes

	self ensureParent: [ : el | el tfIsTable] default: 'table'.
	self ensureChildren: [ : el | el tfIsTableCell ] default: 'td'.
	super tfFixNodes.
! !

!TfTableRowElement methodsFor:'accessing'!

addRowSpanCellsAbove: aTfTableCelLElement
	" cells spanning this row that are not my elements "

	rowSpanCellsAbove isNil ifTrue: [ rowSpanCellsAbove := OrderedCollection new ].
	rowSpanCellsAbove add: aTfTableCelLElement
!

cellRowInterval
	" combined interval of all rows spanned by all of my cells or cells above "

	| start stop |
	start := self rowIndex.
	stop := self rowIndex + self spannedCellRows - 1.
	self rowSpanCellsAbove do: [ : cell | | interval |
		interval := cell rowElement cellRowInterval.
		start := start min: interval first.
		stop := stop max: interval last.
	].
	^ start to: stop
!

rowIndex

	^ rowIndex
!

rowIndex: anInteger

	rowIndex := anInteger
!

rowSpanCellsAbove
	" cells spanning this row that are not my elements "

	^ rowSpanCellsAbove notNil
		ifTrue: [ rowSpanCellsAbove ]
		ifFalse: [ #() ]
!

spannedCellRowInterval

	^ self rowIndex to: self rowIndex + self spannedCellRows - 1
!

spannedCellRows

	spannedCellRows isNil ifTrue: [ spannedCellRows := 1 ].
	^ spannedCellRows
!

spannedCellRowsMax: anInteger

	spannedCellRows := self spannedCellRows max: anInteger
!

tfAdaptToTable: aTfTableEnumerator 

	super tfAdaptToTable: aTfTableEnumerator.
	spannedCellRows := self spannedCellRows min: aTfTableEnumerator numberOfRows - self rowIndex + 1.
!

tfEnumerateTable: aTfTableEnumerator
	" compute row/column indexes of cells/rows "

	aTfTableEnumerator enumRow: self.
	super tfEnumerateTable: aTfTableEnumerator.
!

tfIsTableRow

	^ true
! !

!TfTableRowElement methodsFor:'initialize'!

initialize

	super initialize.
	rowIndex := 1
! !

!TfTableRowElement methodsFor:'menu actions'!

exportTableOn: aStream columns: numberOfColumns emit: aBoolean

	(self elements select: [ : n | n isElement and: [ n tfIsTableElement ]])
		do: [ : el | el exportTableOn: aStream columns: numberOfColumns  emit: true ]
		separatedBy: [ aStream nextPut: $; ].
	aStream cr.
! !

!TfTableRowElement methodsFor:'parsing'!

parseValues

	super parseValues.
	self valueOfAttribute: 'height' do: [ :val |
		rowHeight := self valueReader parseInteger: val ].
! !

!TfTableRowElement methodsFor:'styles'!

applyRowStyleTo: aTfStyle

	| tableStyle |
	tableStyle := aTfStyle tableStyle.
	rowHeight notNil
		ifTrue: [ tableStyle rowHeight: rowHeight ].
!

modifyStyle: aTfStyle
	" modify the style according to the local style definitions "

	super modifyStyle: aTfStyle.
	self applyRowStyleTo: aTfStyle.
! !

!TfTableRowElement methodsFor:'views'!

tfComponents

	^ super tfComponents select: [ :c | c tfIsTableElement ].
!

tfViewClass

	^ TfTableRowView
! !

!TfTableRowElement class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableRowElement.st,v 1.1 2009/10/01 18:53:25 cg Exp $'
! !
