"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfAbstractTableView subclass:#TfTableView
	instanceVariableNames:'actualColumnWidths rows numberOfColumns layout'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfTableView methodsFor:'accessing'!

canBeDiscarded

	^ self model numberOfColumns = 0 or: [ self model numberOfRows = 0 ]
!

columnWidthLayouts

	| columnWidthLayouts |
	columnWidthLayouts := self style tableStyle columnWidthLayouts.
	columnWidthLayouts growToAtLeast: self numberOfColumns.
	columnWidthLayouts size > self numberOfColumns
		ifTrue: [ columnWidthLayouts := columnWidthLayouts copyFrom: 1 to: self numberOfColumns ].
	(columnWidthLayouts includes: nil) 
		ifTrue: [
			self rows detect: [ :row |
					row tfWrappedComponent cells do: [ :cell | 
						cell tfWrappedComponent addWidthLayoutsTo: columnWidthLayouts.
					].
					(columnWidthLayouts includes: nil) not
				] ifNone: [ ].
		].
	^ columnWidthLayouts
!

componentBounds

	^ self bounds insetBy: self inset
!

numberOfColumns

	numberOfColumns isNil
		ifTrue: [	numberOfColumns := self model numberOfColumns ].
	^ numberOfColumns
!

rows

	rows isNil
		ifTrue: [ rows := self components select: [ :component | component isTableRow ]].
	^ rows
!

setupStyleContext

	super setupStyleContext
! !

!TfTableView methodsFor:'private'!

actualColumnWidths

	^ actualColumnWidths
!

computeTableLayoutFor: renderContext

	layout := TfTableLayout for: self.
	layout availableWidth: renderContext layoutContext bounds width.
	renderContext layoutContext tableLayout: layout.
	self numberOfColumns > 0
		ifTrue: [
			layout computeColumnLayoutFor: renderContext.
			actualColumnWidths := layout columnWidths.
		].
!

inset

	^ (self model hspace@self model vspace) + self borderWidth + self style tableStyle cellSpacing
!

ownAdditionalExtent

	^ (2 * self inset) + (self cellSpacing * (self numberOfColumns - 1))
! !

!TfTableView methodsFor:'rendering'!

computeExtentFor: renderContext extentBlock: extentBlock

	| ext |
	ext := 0@0.
	self rows do: [ :component | | componentExt |
		componentExt := extentBlock value: component value: renderContext.
		ext x: (ext x max: componentExt x).
		ext y: ext y + componentExt y.
	].
	^ ext + self ownAdditionalExtent.
!

finishOwnRendering: renderContext

	| layoutContext |
	self numberOfColumns = 0 ifTrue: [ ^ self ].
	super finishOwnRendering: renderContext.
	self updateBoundsFor: renderContext.
	renderContext popLayoutContext.
	" add table to the layout "
	layoutContext := renderContext layoutContext.
	layout bounds: preferredBounds tfDeepCopy.
	layout addToLayoutContext: layoutContext.
!

measureExtentFor: renderContext

	| widthLayout w |
	((widthLayout := self model widthLayout) notNil and: [ (w := widthLayout preferredWidth) notNil ])
		ifTrue: [ 
			renderContext layoutContext measureExtentMax: (w@0)
		]
		ifFalse: [
			renderContext layoutContext isInTable
				ifTrue: [
					renderContext layoutContext measureExtentMax: renderContext boxContext extent.
				]
				ifFalse: [
					super measureExtentFor: renderContext
				].
		].
	self alignment isNil
		ifTrue: [ renderContext layoutContext measureNextLine ].
!

setupComponentRendering: renderContext

	| cellSpacing width layoutContext extent |
	self numberOfColumns = 0 ifTrue: [ ^ self ].
	layoutContext := renderContext pushLayoutContext.
	cellSpacing := self style tableStyle cellSpacing.
	layoutContext
		insetBy: self inset;
		alignToZero;
		reset.
	width := layout width.
	layoutContext boxContext width: width.
	self rows 
		do: [ : wrappedRow | self setupComponentRendering: wrappedRow for: renderContext ]
		separatedBy: [ layoutContext moveBy: 0@cellSpacing ].
	extent := (width@layoutContext position y) + (2 * self inset).
	preferredBounds := 0@0 corner: extent.
	renderContext popLayoutContext.
!

startOwnRendering: renderContext

	| lc |
	super startOwnRendering: renderContext.
	(renderContext layoutContext isInTable and: [ self style tableStyle widthLayout isNil ])
		ifTrue: [ self style tableStyle widthLayout: (TfScaledSizeLayout sizeFactor: 1) ].
	renderContext layoutContext 
		nextLine;
		applyPendingLayouts;
		resetLayout.
	lc := renderContext pushLayoutContext.
	self tfLocalTranslation: renderContext layoutContext position.
	lc startTable: self.
	self computeTableLayoutFor: renderContext.
!

updateBoundsFor: renderContext
	" table rendering is finished. now update the bounds of my components"

	| layoutContext cellSpacing |
	layoutContext := renderContext pushLayoutContext.
	layoutContext 
		bounds: preferredBounds; 
		reset; 
		position: self inset.
	cellSpacing := self cellSpacing.
	self rows 
		do: [ : wrappedRow | 
			wrappedRow tfWrappedComponent updateBoundsFor: renderContext in: preferredBounds.
		] separatedBy: [
			renderContext layoutContext moveBy: 0@(cellSpacing).
		].
	preferredBounds bottom: (layoutContext position + self inset) y.
	renderContext popLayoutContext.
! !

!TfTableView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTableView.st,v 1.1 2009/10/01 19:45:06 cg Exp $'
! !
