"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfView subclass:#TfTextView
	instanceVariableNames:'lines deviceFont text decorator decoratorRectangles fontPolicy
		selectionTranslation isSelected displayScanner selectionScanner
		displayText localTranslation layoutBox'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfTextView methodsFor:'accessing'!

canBeDiscarded

	^ text isNil or: [ text isEmpty or: [ super canBeDiscarded ]]
!

decorator

	^ decorator
!

decorator: aTfDecorator

	decorator := aTfDecorator
!

hasPoint: aPoint

	^ self lines notNil and: [ self lines contains: [ :line | line localBounds containsPoint: aPoint ]]
!

layer

	^ -1
!

preferredBounds

	preferredBounds isNil
		ifTrue: [ 
			self lines do: [ : line | | lineBounds |
				lineBounds := line bounds translatedBy: localTranslation negated.
				preferredBounds := preferredBounds notNil
					ifTrue: [ preferredBounds merge: lineBounds ]
					ifFalse: [ lineBounds ].
			].
			preferredBounds isNil 
				ifTrue: [ preferredBounds := 0@0 extent: 0@0 ]
		].
	^ preferredBounds
!

styleContext

	^ self container styleContext
!

translateBy: aPoint

	preferredBounds := preferredBounds translatedBy: aPoint.
	lines do: [ :line |
		line bounds: (line bounds translatedBy: aPoint)
	].
! !

!TfTextView methodsFor:'controller accessing'!

defaultControllerClass

	^TfTextController
! !

!TfTextView methodsFor:'displaying'!

displayDecoratorOn: aGraphicsContext

	self decorator notNil
		ifTrue: [
			self decoratorRectangles do: [ :rect |
				self decorator displayOn: aGraphicsContext copy in: rect for: self
			].
		].
!

displayElementOn: aGraphicsContext

" TODO: think about it. collides with selection of overlapping texts "
"	self displayDecoratorOn: aGraphicsContext."	
	text isNil ifTrue: [ ^ self ].
	self displaySelectionOn: aGraphicsContext.
	self displayTextOn: aGraphicsContext.
!

displaySelectionOn: aGraphicsContext
	" display the background of the selected parts of the text "

	| p |
	self isSelected
		ifFalse: [ ^ self ].	
	" display all selected parts "
	p := aGraphicsContext paint.
	aGraphicsContext paint: self selectionBackgroundColor.
	lines do: [ : textLine | | box |
		box := textLine selectionBounds.
		box isNil
			ifFalse: [ box displayFilledOn: aGraphicsContext ].
	].
	aGraphicsContext paint: p.
!

displayTextOn: aGraphicsContext

	| scanner b |
	(displayText isNil or: [ displayText isEmpty]) ifTrue: [ ^ self ].
	scanner := self displayScanner.
	self lines do: [ : textViewLine |
		b := textViewLine localBounds.
		scanner
			displayLine: textViewLine indexInterval
			inText: displayText
			left: b left
			right: b right
			top: b top
			on: aGraphicsContext.
	].
! !

!TfTextView methodsFor:'event driven'!

downcastEvent: aKey with: aParameter from: anInitiator 

	aKey = #tfUpdateTextSelection
		ifTrue: [ 
			self updateTextSelection: aParameter for: anInitiator.
			^ self
		].
	aKey = #tfClearTextSelection
		ifTrue: [ 
			self clearTextSelection.
			^ self
		].
	^ super downcastEvent: aKey with: aParameter from: anInitiator
!

downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator 

	aKey = #tfTrackTextSelection 
		ifFalse: [
			^super 
				downcastLocalEvent: aKey
				with: aParameter
				at: aPoint
				from: anInitiator
		].
	^((self bounds containsPoint: aPoint) 
		and: [lines notNil and: [ lines contains: [:textLine | textLine bounds containsPoint: aPoint]]]) 
			ifTrue: 
				[" aParameter is a TfTextSelection "
				aParameter stopView: self.
				true]
			ifFalse: [false]
!

handlerForMouseEvent: event

	| handler |
	^ ((handler := super handlerForMouseEvent: event) notNil
				and: [ self hasPoint: (self controller sensor mousePointForEvent: event) ])
		ifTrue: [ handler ]
		ifFalse: [ nil ].
! !

!TfTextView methodsFor:'private'!

computeDecoratorRectanglesFor: renderContext
	" compute rectangles for regions covered by my decorator.
	up to three rectangles: first line, body, last line "

	| origin corner rect1 |
	decoratorRectangles := OrderedCollection new.
"	self style paragraphStyle isPreformatted
		ifTrue: [" " single line " "
			rect1 := self bounds copy.
			rect1 right: renderContext layoutContext bounds right.	
			decoratorRectangles add: rect1
		] ifFalse: [
"
			self lines size > 0
				ifTrue: [
					rect1 := (self lines at: 1) bounds.
					self lines size > 1
						ifTrue: [
							rect1 right: self bounds right.
							" last line "
							decoratorRectangles add: (self lines at: self lines size) bounds .
							self lines size > 2
								ifTrue: [ " body "
									origin := (self lines at: 2) bounds origin.
									corner := self bounds right@(self lines at: self lines size -1) bounds bottom.
									decoratorRectangles add: (origin corner: corner).
								].
						].
					" first line "
					decoratorRectangles add: rect1.
				].
"		]."
!

decoratorRectangles

	^ decoratorRectangles
!

deviceFont
	" there might be quite a difference between what we ask for (FontDescription)
	and what we get (DeviceFont), so for proper layouts it's necessary to query
	the device font "

	deviceFont isNil
		ifTrue: [ deviceFont := fontPolicy findFont: self styleContext textStyle defaultFont ].
	^ deviceFont
!

displayScanner

	displayScanner isNil
		ifTrue: [ 
			displayScanner := DisplayScanner
				fontPolicy: fontPolicy
				text: text
				textAttributes: self styleContext textStyle
		].
	^ displayScanner
!

isSelected

	^ isSelected == true
!

lines
	" ^ Collection(TfTextViewLine) "

	lines isNil
		ifTrue: [ lines := OrderedCollection new ].
	^ lines
!

rememberLineFrom: startIndex to: stopIndex lineIndex: lineIndex bounds: lineBounds for: renderContext
	" done computing one line. now remember all necessary information about it "

	| df lineLayout |
	df := self deviceFont.
	preferredBounds := preferredBounds notNil
			ifTrue: [ preferredBounds merge: lineBounds ]
			ifFalse: [ lineBounds ].
	renderContext layoutContext isForMeasuring
		ifTrue: [ ^ self ].
	lineLayout := TfTextViewLine new.
	lineLayout
		parentView: self;
		lineIndex: lineIndex;
		bounds: (lineBounds translatedBy: localTranslation);
		translation: localTranslation;
		startIndex: startIndex;
		stopIndex: stopIndex;
		ascent: df ascent;
		descent: df descent;
		isWhitespaceOnly: (text copyFrom: startIndex to: stopIndex) tfIsWhitespaceOnly.
	lines add: lineLayout copy.
	lineLayout addToLayoutContext: renderContext layoutContext.
!

scannerFor: renderContext
	" text scanner used for measuring the text "

	^ TfCompositionScanner
		fontPolicy: fontPolicy
		text: text
		textAttributes: self styleContext textStyle.
!

setupTextFor: renderContext
	" properly convert the model's text by applying paragraph rules "

	| paragraphStyle |
	text notNil ifTrue: [ ^ self ].
	paragraphStyle := self style paragraphStyle.
	paragraphStyle isPreformatted
		ifTrue: [
			text := self model text.
			text isEmpty not
				ifTrue: [ 
					(text first = Character cr and: [ renderContext layoutContext isParagraphStart ])
						ifTrue: [ text := text copyFrom: 2 to: text size ].
					renderContext layoutContext isParagraphStart: false.
				].
		] ifFalse: [
			paragraphStyle convertWhitespaces
				ifTrue: [
					text := self model textWithoutWhitespace.
					(text isEmpty not and: [ renderContext layoutContext isParagraphStart ])
						ifTrue: [
							" trim spaces when beginning a new paragraph "
							text := text tfTrimLeadingSpace.
							text isEmpty
								ifFalse: [ renderContext layoutContext isParagraphStart: false ].
						].
				]
		].
	" argh "
	displayText := text copy.
	displayText replaceAll: (Character value: 160) with: Character space.
!

text

	^ text
!

translateLineAt: lineIndex by: aPoint

	| textLine |
	lines size < lineIndex ifTrue: [ ^ self ].	" todo ??? "
	textLine := lines at: lineIndex.
	" move line "
	textLine tfLocalTranslateBy: aPoint.
	" reset bounds "
	preferredBounds := nil
! !

!TfTextView methodsFor:'rendering'!

computeMinimalExtentFor: renderContext

	self measureFor: renderContext.
	^ minimalExtent
!

computePreferredExtentFor: renderContext

	self measureFor: renderContext.
	^ self preferredBounds extent
!

flushRenderingCaches: needsMeasuring

	super flushRenderingCaches: needsMeasuring.
	displayScanner := nil.
!

measureFor: renderContext

	(renderContext pushLayoutContext) prepareForMeasuring.
	self setupTextFor: renderContext.
	self canBeDiscarded
		ifTrue: [ 
			preferredBounds := 0@0 extent: 0@0.
			minimalExtent := 0@0.
		]
		ifFalse: [
			localTranslation := 0@0.
			self setupTextRendering: renderContext.
		].
	renderContext popLayoutContext.
	text := nil.
!

selectionScanner

	selectionScanner isNil
		ifTrue: [ 
			selectionScanner := TfSelectionScanner
				fontPolicy: fontPolicy
				text: text
				textAttributes: self styleContext textStyle
		].
	^ selectionScanner
!

setupDecorator: renderContext

	self decorator: self style backgroundStyle createDecorator.
	self decorator notNil
		ifTrue: [ 	self computeDecoratorRectanglesFor: renderContext ].
!

setupRendering: renderContext
	" prepare the view for rendering "

	self setupTextFor: renderContext.
	self canBeDiscarded
		ifTrue: [ ^ self ].
	renderContext layoutContext 
		applyPendingLayouts;
		isFirstParagraph: false.
	localTranslation := (0@renderContext layoutContext position y).
	self tfLocalTranslation: localTranslation.
	self setupTextRendering: renderContext.
	self setupDecorator: renderContext.
	preferredBounds := nil.
!

setupTextRendering: renderContext
	" scan each visual text line "
	
	| scanner origin lineHeight startIndex lineIndex stopIndex lineBounds wrapText layoutContext clippedRight isForMeasuring |
	layoutContext := renderContext layoutContext.
	layoutBox := layoutContext bounds.
	fontPolicy := renderContext graphicsDevice defaultFontPolicy.
	lines := OrderedCollection new.
	scanner := self scannerFor: renderContext.
	lineHeight := self deviceFont height.
	isForMeasuring := layoutContext isForMeasuring.
	wrapText := isForMeasuring not and: [ self style paragraphStyle wrapText ].
	preferredBounds := nil.
	startIndex := 1.
	stopIndex := 0.
	lineIndex := 1.
	origin := layoutContext position - localTranslation.
	[	
		clippedRight := wrapText 
			ifTrue: [ ((layoutContext applyProtectedBoxesTo: (layoutBox right@origin y) + localTranslation flowRight: false) - localTranslation) x ]
			ifFalse: [ layoutBox right ].
		origin x <= clippedRight
			ifTrue: [
				" measure a single line "
				scanner
					composeLine: lineIndex
					fromCharacterIndex: startIndex
					wrap: wrapText
					leftMargin: origin x
					rightMargin: clippedRight.
				stopIndex := scanner lineLast.
				stopIndex >= startIndex
					ifTrue: [
						" remember information about the line "
						lineBounds := origin extent: scanner width@lineHeight.
						self
							rememberLineFrom: startIndex
							to: stopIndex
							lineIndex: lineIndex
							bounds: lineBounds
							for: renderContext.
					].
			].
		stopIndex < text size and: [ lineIndex <= 2 or: [ stopIndex >= startIndex ]]
	] whileTrue: [
		" next line "
		renderContext layoutContext
			nextLine;
			currentLineHeight: lineHeight.
		origin := layoutContext position - localTranslation.
		startIndex := stopIndex + 1.
		lineIndex := lineIndex + 1.
	].
	isForMeasuring 
		ifTrue: [ minimalExtent := scanner maxWordWidth@lineHeight ].
! !

!TfTextView methodsFor:'selection - private'!

basicSelectedBoundsFor: aRectangle textSelection: textSelection
	" compute selected part of a text line "

	| normSel newLeft newRight rect |
	normSel := textSelection tfNormalized.
	(normSel top >= aRectangle bottom or: [ normSel bottom <= aRectangle top ])
		ifTrue: [ ^ nil ].
	(normSel top >= aRectangle top and: [ normSel bottom <= aRectangle bottom ])
		ifTrue: [ ^ ((normSel left@aRectangle top) corner: (normSel right@aRectangle bottom)) ].
	(normSel top < aRectangle top and: [ normSel bottom > aRectangle bottom ])
		ifTrue: [ ^ aRectangle ].
	rect := textSelection height > 0
		ifTrue: [ textSelection ]
		ifFalse: [ textSelection corner corner: textSelection origin ].
	normSel top < aRectangle top
		ifTrue: [ newLeft := aRectangle left. newRight := rect right ]
		ifFalse: [ newRight := aRectangle right. newLeft := rect left ].
	^ (newLeft@aRectangle top) corner: (newRight@aRectangle bottom)
!

clearTextSelection

	self isSelected
		ifFalse: [ ^ self ].	
	isSelected := false.
	lines do: [ : textLine |
		textLine deselect
	].
!

selectFrom: startIndex to: stopIndex line: textLine stream: textStreamOrNil

	| i start stop |
	i := textLine startIndex.
	start := (startIndex + i -1) max: 1.
	stop := (stopIndex + i-1) min: text size.
	textLine selectionStartIndex: start.
	textLine selectionStopIndex: stop.
	textStreamOrNil notNil 
		ifTrue: [ textStreamOrNil nextPutAll:  (text copyFrom: start to: stop) ].
	^ start to: stop
!

selectTextFor: aTextSelection
	" select all text within the selection bounds "

	lines do: [ : textLine | | box |
		box := self selectedBoundsFor: (textLine localBounds translatedBy: selectionTranslation) textSelection: aTextSelection.
		box isNil
			ifFalse: [ 
				box:= box translatedBy: selectionTranslation negated.
				box := self updateSelectionIndexOf: textLine selectedBounds: box stream: aTextSelection textStream.
			].
		textLine selectionBounds: box.
	].
!

selectWordFor: aTextSelection 
	" select the word at the selection point "

	| textLine b selText wordBlock point sc tr interval startBlock stopBlock |
	point := aTextSelection origin - selectionTranslation.
	textLine := self lines detect: [ : line | line localBounds containsPoint: point ] ifNone: [ nil ].
	textLine isNil ifTrue: [ ^ self ].
	b := 0@0 extent: textLine bounds extent.
	selText := text copyFrom: textLine startIndex to: textLine stopIndex.
	sc := self selectionScanner .
	tr := textLine localBounds origin.
	wordBlock := sc characterBlockAtPoint: point - tr text: selText in: b.
	interval := self wordIntervalAt: wordBlock stringIndex in: selText.
	interval = textLine selectionInterval
		ifTrue: [ " word already selected, select the entire line "
			interval := 1 to: selText size.
		].
	self selectFrom: interval first to: interval last line: textLine stream: aTextSelection textStream.
	startBlock := sc characterBlockForIndex: interval first text: selText in: b.
	stopBlock := sc characterBlockForIndex: interval last text: selText in: b.
	textLine selectionBounds: ((startBlock origin corner: stopBlock corner) translatedBy: tr).
!

selectedBoundsFor: aRectangle textSelection: textSelection

	| box |
	box := self basicSelectedBoundsFor: aRectangle textSelection: textSelection.
	box := box notNil
		ifTrue: [ 
			box := box intersect: aRectangle.
			(box width > 0 and: [ box height > 0 ])
				ifTrue: [ box ]
				ifFalse: [ nil ]
		]
		ifFalse: [ nil ].
	^ box
!

updateSelectionIndexOf: textLine selectedBounds: box stream: textStreamOrNil
	" compute the characters contained in the box "

	| sc selText startBlock stopBlock textBox b tr |
	sc := self selectionScanner.
	b := 0@0 extent: textLine bounds extent.
	tr := textLine localBounds origin.
	selText := text copyFrom: textLine startIndex to: textLine stopIndex.
	startBlock := sc characterBlockAtPoint: box origin - tr text: selText in: b.
	stopBlock := sc characterBlockAtPoint: box corner - tr text: selText in: b.
	self selectFrom: startBlock stringIndex to: stopBlock stringIndex -1 line: textLine stream: textStreamOrNil.
	textBox := startBlock origin corner: stopBlock bottomLeft.
	^ textBox translatedBy: tr.
!

updateTextSelection: aTextSelection for: aDocumentView

	| normalizedTextSelection boundsForTextSelection textStream |
	aTextSelection isNil ifTrue: [ ^ isSelected := false ].
	selectionTranslation := self documentOffset.
	boundsForTextSelection := self bounds translatedBy: selectionTranslation.
	normalizedTextSelection  := aTextSelection tfNormalized.
	isSelected := (normalizedTextSelection bottom >= boundsForTextSelection top 
				and: [ normalizedTextSelection top <= boundsForTextSelection bottom ]).
	self isSelected
		ifFalse: [ ^ self ].	
	" compute all selected parts "
	aTextSelection textStreamAppendDo: [ : str | 
		textStream := str.
		str emphasis: self styleContext style fontStyle emphases ].
	aTextSelection selectionMode = #word
		ifTrue: [ self selectWordFor: aTextSelection ]
		ifFalse: [ self selectTextFor: aTextSelection ]
!

wordIntervalAt: index in: aString
	" select the word at index "

	| start stop stopCharacters n |
	start := index.
	stop := index.
	stopCharacters := aString tfWhitespaceCharacters, ',;.:-_#"*+~!!§$%&/()=?\}][{^''°'.
	[ start >= 1 and: [ (stopCharacters includes: (aString at: start)) not ]]
		whileTrue: [ start := start - 1 ].
	n := aString size.
	[ stop <= n and: [ (stopCharacters includes: (aString at: stop)) not ]]
		whileTrue: [ stop := stop + 1 ].
	^ start + 1 to: stop - 1
! !

!TfTextView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfTextView.st,v 1.1 2009/10/01 19:45:33 cg Exp $'
! !
