"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfURLHolder subclass:#TfURLDescriptor
	instanceVariableNames:'url lastVisit delay rememberVisit openMode allowDownload
		originalUrlString'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfURLDescriptor class methodsFor:'instance creation'!

url: anURLOrString

	^ self new
		url: anURLOrString;
		yourself
!

url: anURLOrString delay: anInteger

	^ self new
		url: anURLOrString;
		delay: anInteger;
		yourself
!

url: anURLOrString description: aString

	^ self new
		url: anURLOrString;
		description: aString;
		yourself
! !

!TfURLDescriptor methodsFor:'accessing'!

addToMenu: aMenu for: anObject

	| item |
	(item := self createMenuItemFor: anObject) notNil
		ifTrue: [ aMenu addItem: item ].
!

allowDownload

	^ allowDownload == true
!

allowDownload: aBoolean

	allowDownload := aBoolean
!

allowsDownload

	^ true
!

canAddBookmarks

	^ true
!

createMenuItemFor: anObject

	| menuItem |
	menuItem := TfURLMenuItem labeled: self description.
	menuItem value: [ anObject gotoURLDescriptor: self ].
	menuItem helpText: self url asString.
	^ menuItem
!

delay
	" delay (in seconds) before visiting this URL, or nil "

	^ delay
!

delay: anInteger
	" delay (in seconds) before visiting this URL, or nil "

	delay := anInteger
!

forceDownload

	^ self allowsDownload and: [ self openMode == #download ]
!

lastVisit

	^ lastVisit
!

lastVisit: aTimestamp

	lastVisit := aTimestamp
!

openBookmarkEditor

	^ TfEditBookmarkDialog openOnBookmark: self
!

openMode
	" one of: #(#sameWindow #embeddedWindow #newWindow) "

	^ openMode notNil
		ifTrue: [ openMode ]
		ifFalse: [ #sameWindow ]
!

openMode: aSymbol
	" one of: #(#sameWindow #embeddedWindow #newWindow #download) "

	openMode := aSymbol
!

originalUrlString

	^ originalUrlString notNil
		ifTrue: [ originalUrlString ]
		ifFalse: [ self url asString ]
!

originalUrlString: aString
	" unmodified, possibly relative URL string "
	" optional "

	originalUrlString := aString
!

rememberVisit
	" true if the visit should be added to the navigation history "

	^ rememberVisit notNil
		ifTrue: [ rememberVisit ]
		ifFalse: [ self url tfRememberVisit ]
!

rememberVisit: aBoolean
	" true if the visit should be added to the navigation history "

	rememberVisit := aBoolean
!

tfAsURL

	^ self url
!

tfAsURLDescriptor

	^ self
!

url

	^ url
!

url: anURLOrString

	url := anURLOrString tfAsURL
!

urlWithoutFragment

	^ url tfURLWithoutFragment
!

useCache

	^ true
! !

!TfURLDescriptor methodsFor:'comparing'!

= anotherDescriptor

	^ self class = anotherDescriptor class
		and: [ self url = anotherDescriptor url ]
!

hash

	^ self url hash
!

sameNavigationAs: anotherDescriptor

	^ self class = anotherDescriptor class
		and: [ self url = anotherDescriptor url ]
!

samePageAs: anotherDescriptor

	^ self class = anotherDescriptor class
		and: [ self urlWithoutFragment = anotherDescriptor urlWithoutFragment ]
! !

!TfURLDescriptor methodsFor:'import/export'!

exportNetscapeBookmarksTo: aStream

	self depth timesRepeat: [ aStream tab ].
	aStream 
		nextPutAll: '<DT><A HREF="';
		nextPutAll: self url asString;
		nextPutAll: '">';
		nextPutAll: self description;
		nextPutAll: '</A>';cr.
!

exportOperaBookmarksTo: aStream

	aStream nextPutAll: '#URL';cr.
	aStream 
		tab; nextPutAll: 'NAME='; nextPutAll: self description;cr;
		tab; nextPutAll: 'URL='; nextPutAll: self url asString;cr.
	aStream nextPutAll: '	CREATED=0
	VISITED=0
	PERSONALBAR_POS=-1
	PANEL_POS=-1
	DESCRIPTION=
	SHORT NAME=

'.
!

importOperaKey: key value: value

	key = 'URL'
		ifTrue: [ ^ self url: value ].
	super importOperaKey: key value: value
!

isURLBookmark

	^ true
! !

!TfURLDescriptor methodsFor:'printing'!

printOn: aStream

	self url printOn: aStream.
!

treeDisplayString

	^ self description notNil
		ifTrue: [ self description ]
		ifFalse: [ self url asString ]
! !

!TfURLDescriptor methodsFor:'resources'!

loadResourceFor: aTfResourceManager

	^ self url loadResourceFor: aTfResourceManager descriptor: self
!

tfMimeEntityFor: aTfResourceManager

	^self url tfMimeEntityFor: aTfResourceManager descriptor: self
! !

!TfURLDescriptor class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfURLDescriptor.st,v 1.1 2009/10/01 18:06:38 cg Exp $'
! !
