"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfTestCase subclass:#TfURLTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerTesting'
!


!TfURLTests methodsFor:'testing'!

testDirectoryURL

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.smalltalk.org/'.
	resolvedURL := url tfResolveURL: 'test/'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/test/' ].
!

testDirectoryUpURL

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.smalltalk.org/test'.
	resolvedURL := url tfResolveURL: '../faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].
!

testDomainURL

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.java.org'.
	resolvedURL := url tfResolveURL: '//www.smalltalk.org/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org/test'.
	resolvedURL := url tfResolveURL: '//www.smalltalk.org/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org:8080/test?id=42'.
	resolvedURL := url tfResolveURL: '//www.smalltalk.org/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].
!

testFragment

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.smalltalk.org'.
	resolvedURL := url tfResolveURL: '#TOP'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/#TOP' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org/page#TOP'.
	resolvedURL := url tfResolveURL: '#HEADLINE'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/page#HEADLINE' ].
!

testHelloWorld
	" if THIS fails something seriously went wrong ;-) "

	| urlString |
	urlString := (Net.URL tfFromString: 'http://www.smalltalk.org') asString.
	self should: [ urlString = 'http://www.smalltalk.org/' ].

	" default protocol is HTTP "
	urlString := (Net.URL tfFromString: 'www.smalltalk.org') asString.
	self should: [ urlString = 'http://www.smalltalk.org/' ].
!

testRelativeURL

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.smalltalk.org'.
	resolvedURL := url tfResolveURL: 'faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org/index.html'.
	resolvedURL := url tfResolveURL: 'faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org/test'.
	resolvedURL := url tfResolveURL: 'faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/test/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org:8080/test?id=42'.
	resolvedURL := url tfResolveURL: '/'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org:8080/' ].
!

testRootURL

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.smalltalk.org'.
	resolvedURL := url tfResolveURL: '/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org/test'.
	resolvedURL := url tfResolveURL: '/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org/faq.html' ].

	url := Net.URL tfFromString: 'http://www.smalltalk.org:8080/test?id=42'.
	resolvedURL := url tfResolveURL: '/faq.html'.
	self should: [ resolvedURL asString = 'http://www.smalltalk.org:8080/faq.html' ].
!

testURLParsing

	| urlString |
	urlString := (Net.URL tfFromString: 'http://www.smalltalk.org') asString.
	self should: [ urlString = 'http://www.smalltalk.org/' ].

	" default protocol is HTTP "
	urlString := (Net.URL tfFromString: 'www.smalltalk.org') asString.
	self should: [ urlString = 'http://www.smalltalk.org/' ].
!

testURLWithParameters

	| url resolvedURL |
	url := Net.URL tfFromString: 'http://www.test.org/servlet/TestServlet?hello=world'.
	resolvedURL := url tfResolveURL: 'AnotherServlet?success=true'.
	self should: [ resolvedURL asString = 'http://www.test.org/servlet/AnotherServlet?success=true' ].
	self should: [ resolvedURL query = 'success=true' ].
! !

!TfURLTests class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfURLTests.st,v 1.1 2009/10/01 23:39:55 cg Exp $'
! !
