"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

View subclass:#TfView
	instanceVariableNames:'preferredBounds debug minimalExtent preferredExtent styleContext'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Views'
!


!TfView methodsFor:'accessing'!

alignment

	^ self style paragraphStyle alignment
!

browserModel

	^ self model documentModel browserWindowModel browserModel
!

documentOffset
	" offset of the view relative to the document view "

	^ (self localPointToGlobal: 0@0) - (self tfTopDocumentView localPointToGlobal: 0@0)
!

layer
	" layer of this view. Twoflower views sometimes overlap other views, so layers are required
	for correctly dispatching events "

	^ 0
!

settings

	^ self browserModel settings
!

setupStyleContext

	self styleContext setupStyleFor: self model.
!

styleContext

	styleContext isNil
		ifTrue: [
			styleContext := self container styleContext createChildContext.
			self setupStyleContext.
		].
	^ styleContext
!

tfNonWrappingContainer

	^ self container tfNonWrappingContainer
!

tfWrapped

	^ TfTranslatingWrapper on: self
!

xmlNode

	^ self model
! !

!TfView methodsFor:'bounds accessing'!

height

	^ self bounds height
!

preferredBounds

	^ preferredBounds notNil
		ifTrue: [ preferredBounds ]
		ifFalse: [ super preferredBounds ]
!

tfLocalTranslateBy: aPoint

	self tfLocalTranslation: self tfLocalTranslation + aPoint
!

tfLocalTranslation

	^ self tfTranslatingWrapper translation
!

tfLocalTranslation: aPoint

	self tfTranslatingWrapper translation: aPoint
!

width

	^ self bounds width
! !

!TfView methodsFor:'controller accessing'!

defaultControllerClass

	^TfController
! !

!TfView methodsFor:'debug'!

debug

	debug isNil
		ifTrue: [ debug := false ].
	^ debug
!

debug: aBoolean

	debug := aBoolean.
!

displayDebugOn: aGraphicsContext

	(self debug and: [ Log isDebug ])
		ifTrue: [ self displayFlashOn: aGraphicsContext ].
!

displayFlashOn: aGraphicsContext

	| gc |
	gc := aGraphicsContext copy.
	gc paint: ColorValue red.
	(self bounds insetBy: (0@0 corner: 1@1)) displayStrokedOn: gc.
!

flash

	| gc |
	gc := self graphicsContext.
	(1 to: 3) 
		do: [ : i |
			self displayFlashOn: gc.
			gc flush.
			(Delay forMilliseconds: 100) wait.
			self invalidateRectangle: self bounds repairNow: true.
		] separatedBy: [
			(Delay forMilliseconds: 100) wait.
		].
!

tfAllViewsDo: aBlock

	aBlock value: self.
!

update: anAspect with: param from: sender

	anAspect = #flash
		ifTrue: [ ^ self flash ].
	anAspect = #removed
		ifTrue: [ ^ self container removeComponent: self ].
	^ super update: anAspect with: param from: sender
! !

!TfView methodsFor:'displaying'!

displayElementOn: aGraphicsContext
!

displayOn: aGraphicsContext

	self displayElementOn: aGraphicsContext.
	self displayDebugOn: aGraphicsContext.
! !

!TfView methodsFor:'printing'!

printOn: aStream

	super printOn: aStream.
	aStream nextPut: $[.
	self bounds printOn: aStream.
	aStream nextPut: $].
! !

!TfView methodsFor:'rendering'!

computeMinimalExtentFor: renderContext

	^ 0@0
!

computePreferredExtentFor: renderContext

	^ self preferredBounds extent
!

flushRenderingCaches: needsMeasuring
	" flush all caches associated with rendering "

	needsMeasuring
		ifTrue: [
			minimalExtent := nil.
			preferredExtent := nil.
		].
!

measureExtentFor: renderContext

	| pstyle |
	pstyle := self style paragraphStyle.
	pstyle startMeasureContext: renderContext layoutContext.
	renderContext layoutContext measureExtent: (self preferredExtentFor: renderContext).
	pstyle finishMeasureContext: renderContext layoutContext.
!

minimalExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	minimalExtent isNil
		ifTrue: [ minimalExtent := self computeMinimalExtentFor: renderContext ].
	^ minimalExtent
!

preferredExtentFor: renderContext
	" extent of the view assuming that there are no layout restrictions "
	" ^ Point "

	preferredExtent isNil
		ifTrue: [ preferredExtent := self computePreferredExtentFor: renderContext ].
	^ preferredExtent
!

setupRendering: renderContext
	" prepare the view for rendering "

	self flushRenderingCaches: false.
!

style

	^ self styleContext style
!

tfRenderingFinished
! !

!TfView methodsFor:'testing'!

canBeDiscarded

	^ false
!

isTableCell

	^ false
!

isTableRow

	^ false
! !

!TfView class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfView.st,v 1.1 2009/10/01 19:46:25 cg Exp $'
! !
