"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfTestCase subclass:#TfViewTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'TwoflowerTesting'
!


!TfViewTests methodsFor:'private'!

createViewFor: documentString

	^ self createViewFor: documentString width: 10000
!

createViewFor: documentString width: width

	| doc view docModel |
	doc := self createDocument: documentString.
	docModel := TfDocumentModel new.
	docModel document: doc.
	view := docModel createViewWithKeyboard: nil.
	view bounds: (0@0 corner: width@1000).
	view setupRendering: self renderContext.
	^ view
!

renderContext

	| styleSheet renderContext |
	styleSheet := TfDefaultStyleSheet fromSettings: TfSettings new.
	renderContext := TfRenderContext new.
	renderContext styleContext styleSheet: styleSheet.
	renderContext layoutContext bounds: Screen default bounds.
	^ renderContext
!

tableCellsIn: elementView do: aBlock

	| i  |
	i := 0.
	elementView tfAllViewsDo: [ :view |
		(view isKindOf: TfTableCellView)
			ifTrue: [
				i := i + 1.
				aBlock
					value: view
					value: view components first component components first
					value: view model columnIndex
			].
	].
	^ i
!

tablesIn: elementView do: aBlock

	| i |
	i := 0.
	elementView tfAllViewsDo: [ :v |
		(v isKindOf: TfTableView)
			ifTrue: [
				aBlock value: v.
				i := i + 1.
			].
	].
	^ i
!

textViewForID: anID in: elementView

	^ (self viewForID: anID in: elementView) components first
!

viewForID: anID in: elementView

	| node |
	elementView tfAllViewsDo: [ :v |
		((node := v xmlNode) notNil and: [ node isElement and: [ node id = anID ]])
			ifTrue: [ ^ v ].
	].
	^ nil
!

viewForText: aString in: elementView

	| node |
	elementView tfAllViewsDo: [ :v |
		((node := v xmlNode) notNil and: [ node isText and: [ node text = aString ]])
			ifTrue: [ ^ v ].
	].
	^ nil
! !

!TfViewTests methodsFor:'testing'!

testHelloWorld
	" if THIS fails something seriously went wrong ;-) "

	| view tagView textView |
	view := self createViewFor: '<html><body>Hello world</body></html>'.
	" check that the views have been propertly created and initialized "
	self should: [ view components size = 1 ].
	tagView := view components first components first.
	textView := tagView components first.
	self should: [ textView container = tagView ].
	self should: [ textView class = TfTextView  ].
	self should: [ textView lines size = 1 ].
!

testParagraphSpacing

	| view offsets spacings |
	view := self createViewFor: TfResources paragraphHTML.
	offsets := (1 to: 4) collect: [ :i | (self textViewForID: i printString in: view) preferredBounds top ].
	" paragraphs at the beginning of a page don't matter "
	self should: [ (offsets at: 1) = 0 ].
	" multiple paragraphs are reduced to a single paragraphs "
	spacings := ((1 to: 3) collect: [ :i | (offsets at: i + 1) - (offsets at: i) ]) asSet.
	self should: [ spacings size = 1 ].
! !

!TfViewTests methodsFor:'testing - fonts'!

testFontSizes
	" check that the font styles work "

	| view normalTextView largerTextView smallerTextView h largeTextView smallTextView largeCssTextView smallCssTextView |
	view := self createViewFor: TfResources fontSizesHTML.
	normalTextView := self textViewForID: '1' in: view.
	h := normalTextView bounds height.

	largerTextView := self textViewForID: '2' in: view.
	smallerTextView := self textViewForID: '3' in: view.
	self should: [ h < largerTextView bounds height ].
	self should: [ h > smallerTextView bounds height ].

	largeTextView := self textViewForID: '4' in: view.
	smallTextView := self textViewForID: '5' in: view.
	self should: [ h < largeTextView bounds height ].
	self should: [ h > smallTextView bounds height ].

	largeCssTextView := self textViewForID: '6' in: view.
	smallCssTextView := self textViewForID: '7' in: view.
	self should: [ h < largeCssTextView bounds height ].
	self should: [ h > smallCssTextView bounds height ].
!

testMixedFontSizes
	" when mixing text with different font sizes, then all text views should be aligned to the baseline "

	| view smallTextView largeTextView st lt |
	view := self createViewFor: TfResources mixedFontSizesHTML.
	smallTextView := self textViewForID: '1' in: view.
	largeTextView := self textViewForID: '2' in: view.
	self should: [ (st := smallTextView bounds top) > (lt := largeTextView bounds top) ].
	self should: [ (st + smallTextView deviceFont ascent) = (lt + largeTextView deviceFont ascent) ]
! !

!TfViewTests methodsFor:'testing - tables'!

testColGroupTable1HTML
	" 3x3 table with fixed column widths, defined by col elements "

	| view i w colWidths cw |
	view := self createViewFor: TfResources colGroupTable1HTML.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		self should: [ (550 - w) abs <= 3 ].
	].
	colWidths := #(200 200 150).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		cw := colWidths at: colIndex.
		w := cellView width.
		self should: [ (cw - w) abs <= 1 ].
	].
	self should: [ i = 9 ].
!

testColGroupTable2HTML
	" 3x3 table with fixed column widths, defined by a colgroup "

	| view i w colWidths cw |
	view := self createViewFor: TfResources colGroupTable2HTML.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		self should: [ (450 - w) abs <= 3 ].
	].
	colWidths := #(150 150 150).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		cw := colWidths at: colIndex.
		w := cellView width.
		self should: [ (cw - w) abs <= 1 ].
	].
	self should: [ i = 9 ].
!

testFixedColumnWidthsTable
	" 3x3 table width fixed column widths "

	| view i w colWidths cw |
	view := self createViewFor: TfResources fixedColumnWidthsTableHTML width: 1000.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		" check width. truncation may rip some pixels "
		self should: [ w >= 447 and: [ w <= 450 ]].
	].
	colWidths := #(100 200 150).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
		cw := colWidths at: colIndex.
		w := cellView width.
		self should: [ (cw - w) abs <= 1 ]
	].
	self should: [ i = 9 ].
!

testFixedWidthTable
	" 3x3 table width fixed width "

	| view i w |
	view := self createViewFor: TfResources fixedWidthTableHTML width: 1000.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		" check width. truncation may rip some pixels "
		self should: [ w >= 397 and: [ w <= 400 ]].
	].
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
	].
	self should: [ i = 9 ].
!

testMixedColumnWidthsTable
	" 3x3 table with mixed column widths "

	| view i w colWidths cw col1width col3width col2with tw |
	view := self createViewFor: TfResources mixedColumnWidthsTableHTML width: 1000.
	col1width := 100.
	col3width := (self viewForText: 'Fleischpflanzerl' in: view) bounds width.
	col2with := col1width + col3width.
	colWidths := Array with: col1width with: col2with with: col3width.
	self tablesIn: view do: [ :tableView |
		tw := tableView bounds width.
		w := col1width + col2with + col3width.
		self should: [  (w-tw) abs <= 3 ].
	].
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
		cw := colWidths at: colIndex.
		w := cellView width.
		self should: [ (w-cw) abs <= 1].
	].
	self should: [ i = 9 ].
!

testRelativeColumnWidths2Table
	" 3x3 table with relative column widths "

	| view i w cw tw colFactor f |
	view := self createViewFor: TfResources relativeColumnWidthsTable2HTML width: 1000.
	self tablesIn: view do: [ :tableView |
		tw := tableView bounds width.
	].
	colFactor := #(0.35 0.3 0.35).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
		f := colFactor at: colIndex.
		cw := tw * f.
		w := cellView width.
		self should: [  (w-cw) abs <= 2].
	].
	self should: [ i = 9 ].
!

testRelativeColumnWidthsTable
	" 3x3 table with relative column widths "

	| view i w cw tw colFactor f |
	view := self createViewFor: TfResources relativeColumnWidthsTableHTML width: 1000.
	self tablesIn: view do: [ :tableView |
		tw := tableView bounds width.
	].
	colFactor := #(0.2 0.3 0.5).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
		f := colFactor at: colIndex.
		cw := tw * f.
		w := cellView width.
		self should: [  (w-cw) abs <= 2].
	].
	self should: [ i = 9 ].
!

testRowspanTable
	" table with one cell spanning 3 rows "

	| view rowRef row1 row2 |
	view := self createViewFor: TfResources rowSpanHTML width: 1000.
	rowRef := self viewForID: 'ref' in: view.
	self should: [ view height > (15 * rowRef height) ].
	row1 := self viewForID: '1' in: view.
	row2 := self viewForID: '2' in: view.
	self should: [ row1 height > rowRef height ].
	self should: [ row2 height > rowRef height ].
!

testTweakedFixedColumnWidthsTable
	" 3x3 table with fixed column widths. Give the table less space than
	it asks for, but more than the mininmal required space  "

	| view i w colWidths cw |
	view := self createViewFor: TfResources fixedColumnWidthsTableHTML width: 300.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		self should: [ w <= 450 ].
	].
	colWidths := #(100 200 150).
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
		cw := colWidths at: colIndex.
		w := cellView width.
		self should: [ w <= (cw + 1) ].
	].
	self should: [ i = 9 ].
!

testUnconstrainedTable
	" simple 3x3 table "

	| view i |
	view := self createViewFor: TfResources unconstrainedTableHTML.
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
	].
	" ... and 3 * 3 = 9 :-) "
	self should: [ i = 9 ].
!

testUnconstrainedTableWrapping
	" 3x3 table with fixed column widths. Give the table less space than required "

	| view i w |
	view := self createViewFor: TfResources fixedColumnWidthsTableHTML width: 50.
	i := self tablesIn: view do: [ :tableView |
		w := tableView actualColumnWidths inject: 0 into: [ :x :y | x + y ].
		self should: [ w > 50 ].
		self should: [ (tableView width - w) abs <= 3 ].
	].
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" should not have wrapped text "
		self should: [ textView lines size = 1 ].
	].
	self should: [ i = 9 ].
!

testVariableWidthTable
	" 3x3 table with variable width (50%)"

	| view i w |
	view := self createViewFor: TfResources variableWidthTableHTML width: 1000.
	i := self tablesIn: view do: [ :tableView |
		w := tableView bounds width.
		" check width. truncation may rip some pixels "
		self should: [ w >= 497 and: [ w <= 500 ]].
	].
	i := self tableCellsIn: view do: [ :cellView :textView :colIndex |
		" should have created text for each cell "
		self should: [ textView model text isEmpty not ].
		" no wrapping "
		self should: [ textView lines size = 1 ].
	].
	self should: [ i = 9 ].
! !

!TfViewTests class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfViewTests.st,v 1.1 2009/10/01 23:40:03 cg Exp $'
! !
