"{ Package: 'stx:goodies/twoflower/compat' }"

"{ NameSpace: XML }"

SAXDriver subclass:#SAXBuilderDriver
	instanceVariableNames:'builder document elementStack newNamespaces'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-SAX'
!

SAXBuilderDriver comment:'This class converts SAX events into XMLNodeBuilder events, allowing old builders to still be used with the new parser.

This is essentially a private class for XMLParser to allow the parser to pretend to still support the old NodeBuilder API.

Instance Variables:
	builder				&lt;XML.XMLNodeBuilder&gt;		The client''s NodeBuilder, which creates XML Nodes.
	document			&lt;XML.Document&gt;				The Document which models the entire XML document.
	elementStack		&lt;OrderedCollection&gt;			A stack of proxies for the various elements that are in scope at the current stage of parsing.
	newNamespaces	&lt;Dictionary&gt;					maps qualifiers to namespaces for the next element'
!


!SAXBuilderDriver methodsFor:'DTD handler'!

notationDecl: name publicID: publicID systemID: systemID

	| notation |
	notation := builder notation: name value: (Array with: publicID with: systemID).
	document dtd notationAt: name put: notation from: self
! !

!SAXBuilderDriver methodsFor:'accessing'!

builder: aNodeBuilder

	builder := aNodeBuilder
!

document

	^document
! !

!SAXBuilderDriver methodsFor:'content handler'!

characters: aString

	| text |
	text := builder makeText: aString.
	text isDiscarded
		ifFalse: [elementStack last nodes add: text].
!

endDocument
!

endDocumentFragment

	^self endDocument
!

endElement: namespaceURI localName: localName qName: name
	"indicates the end of an element. See startElement"

	| elm element |
	elm := elementStack last.
	element := builder
			tag: elm tag
			attributes: elm attributes
			elements: (elm nodes isEmpty
					ifTrue: [nil]
					ifFalse: [elm nodes asArray])
			position: elm startPosition
			stream: elm stream.
	element namespaces: elm namespaces.
	elementStack removeLast.
	elementStack isEmpty
		ifTrue:
			[document addNode: element.
			document dtd declaredRoot: element tag asString]
		ifFalse: [element isDiscarded
			ifFalse: [elementStack last nodes add: element]].
	(element isDiscarded not and: [elm id notNil])
		ifTrue: [document atID: elm id put: element].
	builder popTag
!

ignorableWhitespace: aString

	| text |
	text := builder makeText: aString.
	text isDiscarded
		ifFalse: [elementStack last nodes add: text].
!

processingInstruction: targetString data: dataString

	| pi |
	document == nil ifTrue: [self startDocument].
	pi := builder pi: targetString text: dataString.
	elementStack isEmpty
		ifTrue: [document addNode: pi]
		ifFalse: [elementStack last nodes add: pi]
!

startDocument

	document := Document new.
	document dtd: DocumentType new.
	elementStack := OrderedCollection new.
!

startDocumentFragment

	document := DocumentFragment new.
	document dtd: DocumentType new.
	elementStack := OrderedCollection new.
!

startElement: namespaceURI localName: localName qName: name attributes: attributes

	| nm |
	document == nil ifTrue: [self startDocument].
	nm := NodeTag new
		qualifier: ((name includes: $:)
				ifTrue: [name copyUpTo: $:]
				ifFalse: [''])
		ns: namespaceURI
		type: localName.
	elementStack addLast: (SAXElementContext new tag: nm).
	elementStack last
		attributes: (attributes collect: [:att | att copy]);
		nodes: OrderedCollection new;
		namespaces: newNamespaces.
	newNamespaces := nil.
	builder pushTag: nm.
!

startPrefixMapping: prefix uri: uri 

	newNamespaces == nil
		ifTrue: [newNamespaces := Dictionary new].
	newNamespaces at: prefix put: uri.
! !

!SAXBuilderDriver methodsFor:'other'!

comment: data from: start to: stop

	| comment |
	document == nil ifTrue: [self startDocument].
	comment := builder comment: (data copyFrom: start to: stop).
	comment isDiscarded
		ifFalse: [elementStack isEmpty
			ifTrue: [document addNode: comment]
			ifFalse: [elementStack last nodes add: comment]].
!

idOfElement: elementID
	"Notify the client what was the ID of the latest startElement"

	elementStack last id: elementID
!

sourcePosition: position inStream: streamWrapper
	"Non-standard API to ease transition from
	builders to SAX."

	elementStack last
		startPosition: position;
		stream: streamWrapper
! !

!SAXBuilderDriver class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/compat/XML__SAXBuilderDriver.st,v 1.1 2009/10/02 00:56:58 cg Exp $'
! !
