"{ Package: 'stx:goodies/twoflower' }"!

!ApplicationModel class methodsFor:'Twoflower'!

tfForObject: contextObject
	"open the app using the context object"

	^ self new
! !

!CachedImage methodsFor:'converting'!

tfAsCachedImage

	^ self
! !

!CachedImage methodsFor:'accessing'!

tfConvertForGraphicsDevice: aGraphicsDevice

	^ (self image tfConvertForGraphicsDevice: aGraphicsDevice) tfAsCachedImage
! !

!CachedImage methodsFor:'accessing'!

tfShrunkenBy: scale

	^ (self image tfShrunkenBy: scale) tfAsCachedImage
! !

!CharacterArray methodsFor:'Twoflower'!

tfAsAssociation: separatorChar
	" utility method "
	" for 'key=value' strings, return an association, for all other nil
	(replace = with the separatorChar) "

	| i |
	i := self indexOf: separatorChar ifAbsent: [nil].
	^ (i notNil and: [ i>1 and: [ i < self size ]])
		ifTrue: [ (self copyFrom: 1 to: i - 1) -> (self copyFrom: i + 1 to: self size) ]
		ifFalse: [ nil ].
! !

!CharacterArray methodsFor:'converting'!

tfAsResourceModel

	^ Twoflower::TfDocumentModel htmlString: self
! !

!CharacterArray methodsFor:'accessing'!

tfAsStringsSeperatedByAll: arrayOfCharacters

	| start skip o |
	o := OrderedCollection new.
	self isEmpty ifTrue: [ ^ o].
	start := 1.
	skip := (arrayOfCharacters includes: (self at: 1)).
	1 to: self size do: [ :i |
		(arrayOfCharacters includes: (self at: i))
			ifTrue: [ " found a seperator "
				skip
					ifFalse: [
						o add: (self copyFrom: start to: i - 1).
						skip := true.].
				start := i + 1
			]
			ifFalse: [ skip := false ].
	].
	start <= self size
		ifTrue: [ o add: (self copyFrom: start to: self size) ].
	^ o
! !

!CharacterArray methodsFor:'converting'!

tfAsURL

	^ URL tfFromString: self
! !

!CharacterArray methodsFor:'converting'!

tfAsURLDescriptor

	^ Twoflower::TfURLDescriptor url: self tfAsURL
! !

!CharacterArray methodsFor:'accessing'!

tfIsWhitespaceOnly

	| whitespaces |
	whitespaces := self tfWhitespaceValues.
	^ self allSatisfy: [ :ch | (whitespaces includes: ch asInteger) ]
! !

!CharacterArray methodsFor:'accessing'!

tfTrimLeadingSpace

	| whitespaces |
	whitespaces := self tfWhitespaceValues.
	1 to: self size do: [ :i |
		(whitespaces includes: (self at: i) asInteger)
			ifFalse: [
				^ i = 1 ifTrue: [ self ] ifFalse: [ self copyFrom: i to: self size ]
			].
	].
	^ self species new
! !

!CharacterArray methodsFor:'accessing'!

tfTrimSpace

	^ (self tfTrimLeadingSpace) tfTrimTrailingSpace
! !

!CharacterArray methodsFor:'accessing'!

tfTrimTrailingSpace

	| whitespaces |
	whitespaces := self tfWhitespaceValues.
	self size to: 1 by: -1 do: [ :i |
		(whitespaces includes: (self at: i) asInteger)
			ifFalse: [
				^ i = self size ifTrue: [ self ] ifFalse: [ self copyFrom: 1 to: i ]
			].
	].
	^ self species new
! !

!CharacterArray methodsFor:'accessing'!

tfTrimWhitespaces
	"Return a copy of the receiver that has leading and trailing whitespaces removed."

	| size index whitespaces |
	whitespaces := self tfWhitespaceCharacters.
	(size := self size) == 0 ifTrue: [^self].
	index := 1.
	[ whitespaces includes: (self at: index) ] whileTrue: [
		index = size ifTrue: [^''].
		index := index + 1].
	[ whitespaces includes: (self at: size) ] whileTrue: [
		index = size ifTrue: [^''].
		size := size - 1].
	^self copyFrom: index to: size
! !

!CharacterArray methodsFor:'Twoflower'!

tfValueAtKey: key
	" utility method "
	" for 'key=value' strings, returns 'value', otherwise nil "

	^ self tfValueAtKey: key separator: $=
! !

!CharacterArray methodsFor:'Twoflower'!

tfValueAtKey: key separator: separatorChar
	" utility method "
	" for 'key=value' strings, returns 'value', otherwise nil
	(replace = with the separatorChar) "

	| assoc |
	assoc:= self tfAsAssociation: separatorChar.
	^ (assoc notNil and: [ assoc key asLowercase = key asLowercase ])
		ifTrue: [ assoc value ]
		ifFalse: [ nil ].
! !

!CharacterArray methodsFor:'accessing'!

tfWhitespaceCharacters

	^ (ByteArray withAll: self tfWhitespaceValues) asByteString
! !

!CharacterArray methodsFor:'accessing'!

tfWhitespaceValues

	^ #(9 10 13 32)
! !

!CharacterArray methodsFor:'accessing'!

tfWithoutWhitespaces

	| whitespaces spc str ch hasPendingWhitespace |
	hasPendingWhitespace := false.
	whitespaces := self tfWhitespaceValues.
	spc := Character space.
	str := (String new: self size) writeStream.
	1 to: self size do: [ :i |
		ch := self at: i.
		(whitespaces includes: ch asInteger)
			ifTrue: [ hasPendingWhitespace := true ]
			ifFalse: [
				hasPendingWhitespace
					ifTrue: [
						str nextPut: spc.
						hasPendingWhitespace := false
					].
				str nextPut: ch
			].
	].
	hasPendingWhitespace
		ifTrue: [
			str nextPut: spc.
			hasPendingWhitespace := false
		].
	^ str contents
! !

!CharacterAttributes methodsFor:'accessing'!

tfAsScreenFontStyle

	^ (self defaultQueryFor: Screen default defaultFontPolicy) tfAsFontStyle
! !

!CharacterAttributes methodsFor:'accessing'!

tfSetDefaultAttributes
	" modified version of CharacterAttributes class>>newWithDefaultAttributes as defined in ExtraEmphases
	(hope you don't mind, guys ;) "

	self at: #small
		put:
		[: query : value : fontPolicy | | size |
			size := ( self defaultFontFor: fontPolicy ) pixelSize.
			query pixelSize: (( size * 5 / 6 ) rounded min: size - 1 )
		].
	self at: #large
		put:
		[: query : value : fontPolicy | | size |
			size := ( self defaultFontFor: fontPolicy ) pixelSize.
			query pixelSize: (( size * 6 / 5 ) rounded max: size + 1 )
		].
	self at: #serif put: [: query | query serif: true ].
	self at: #sansSerif put: [: query | query serif: false ].
	self at: #normal
		put:
		[: query |
			query
				boldness: 0.5;
				italic: false
		].
	self at: #bold put: [: query | query boldness: 0.7 ].
	self at: #italic put: [: query | query italic: true ].
	self at: #underline put: [: query | query underline: true ].
	self at: #strikeout put: [: query | query strikeout: true ].
	self at: #color put: [: query : color | query color: color ].
	self at: #family put: [: query : family | query family: family ]
! !

!Collection methodsFor:'accessing'!

tfSortedBy: aBlock

	| associations sortedAssociations |
	associations := self collect: [ :x | (aBlock value: x) -> x ].
	sortedAssociations := associations asSortedCollection: [ :x :y | x key <= y key ].
	^ sortedAssociations asOrderedCollection collect: [ :assoc | assoc value ]
! !

!ColorValue class methodsFor:'access rights utilities (clara)'!

tfFromHTMLHex: htmlColorValue

	| str colorInt redByte greenByte blueByte |
	str := htmlColorValue readStream.
	str peek = $#
		ifTrue: [ str next ].
	colorInt := Integer readFrom: str radix: 16.
	redByte := colorInt bitShift: -16.
	greenByte := (colorInt bitShift: -8) bitAnd: 255.
	blueByte := colorInt bitAnd: 255.
	^ self
		scaledRed: (redByte*8191)//255
		scaledGreen: (greenByte*8191)//255
		scaledBlue: (blueByte*8191)//255
! !

!CompositePart methodsFor:'Twoflower'!

tfSetupComponents

	self components do: [ :c | c tfSetupComponents ].
! !

!Cursor class methodsFor:'constants'!

tfFingerPointer
	" 7.0 base image compatibility "

	^ (self respondsTo: #fingerPointer)
		ifTrue: [ self fingerPointer ]
		ifFalse: [ self hand ]
! !

!Delay methodsFor:'process delay'!

tfValueWithTimeout: aBlock
	" Ripped from COAST (www.opencoast.org) ;-) "

	"Try to evaluate aBlock before the delay time is over."
	"Will safely catch infinite loops for example."
	"(Delay forSeconds: 2) valueWithTimeout: [ [true] whileTrue ]"

	| timeout p oldPri |

	self startup.
	"fork block evaluation, but call Processor yield first, so this
	method can continue first. This is very important!!"
	p := [ Processor yield. aBlock value. delaySemaphore signal ] fork.
	oldPri := Processor activePriority.
	"Raise the priority of this process, otherwise it will never wake up
	in case the forked evaluation will run forever."
	Processor activeProcess priority: oldPri + 1.
	"Now it is safe to wait for the timeout."
	delaySemaphore wait.
	"Do not forget to switch back to the old priority."
	Processor activeProcess priority: oldPri.
	timeout := self inProgress not. "check if it was a timeout"
	self disable.
	timeout ifTrue: [ p terminate ]. "kill the forked evaluation"
	"return true if the forked action finished correctly (before timeout)"
	^timeout not
! !

!EncodedStream methodsFor:'accessing'!

tfCoreStream

	^ stream
! !

!FontDescription methodsFor:'accessing'!

tfAsFontStyle

	| style |
	style := Twoflower::TfFontStyle new.
	style family: self family.
	style pixelSize: self pixelSize.
	style manufacturer: self manufacturer.
	^ style
! !

!GraphicsContext methodsFor:'accessing'!

tfClientDataAt: key put: aValue

	| d |
	d := self clientData.
	d at: key put: aValue.
	clientData := d.
! !

!HostGraphicsDevice methodsFor:'Compatibility-VW'!

defaultFontPolicy
    ^ FontPolicy new
! !

!Image methodsFor:'converting'!

tfAsCachedImage

	^ CachedImage on: self
! !

!Image methodsFor:'converting'!

tfAsResourceModel

	^ Twoflower::TfImageModel image: self
! !

!Image methodsFor:'image processing'!

tfConvertForGraphicsDevice: aGraphicsDevice

	^ self convertForGraphicsDevice: aGraphicsDevice
! !

!Image methodsFor:'image processing'!

tfShrunkenBy: scale

	^ self shrunkenBy: scale
! !

!Integer methodsFor:'converting'!

tfAsTranslation

	^ self asPoint
! !

!Net::HttpURL methodsFor:'Twoflower'!

tfMimeEntityFor: aTfResourceManager descriptor: urlDescriptor

	^ self safelyExecuteBlock: [
		(self tfNewClientFor: aTfResourceManager) getDescriptor: urlDescriptor.
	]
! !

!Net::HttpURL methodsFor:'Twoflower'!

tfNewClientFor: aTfResourceManager

	| client |
	client := Twoflower::TfHttpClient new.
	self user notNil
		ifTrue: [ client user: self user ].
	client resourceManager: aTfResourceManager.
	^client
! !

!Net::MessageElement methodsFor:'accessing'!

tfURL

	^ self parent notNil
		ifTrue: [ self parent tfURL ]
		ifFalse: [ nil ]
! !

!Net::MessageElement methodsFor:'accessing'!

tfUnencodedValue

	^ self value
! !

!Net::MimeEntity methodsFor:'accessing'!

tfUnencodedValue

	^ self body tfUnencodedValue
! !

!Net::MimeEntity class methodsFor:'instance creation'!

tfFileName: fileNameOrString

	| stream mimeType |
	stream := fileNameOrString asFilename readStream.
	mimeType :=  self mimeTypeForFile: fileNameOrString.
	('text/*' match: mimeType)
		ifTrue: [ stream text ]
		ifFalse: [ stream binary ].
	^[self source: stream.] ensure: [ stream close ]
! !

!Net::SimpleBody methodsFor:'accessing'!

tfUnencodedValue

	^ source isNil
		ifTrue: [nil]
		ifFalse: [
			[
				self source reset.
				self source tfCoreStream contents
			] ensure: [self source close]
		]
! !

!Net::SimpleBody class methodsFor:'lint'!

fromLine: aString

	self error: 'Not implemented'
! !

!Number methodsFor:'printing'!

tfSizePrintString

	self >= 1000000
		ifTrue: [ ^ ((self / 10000) rounded / 100) asFloat printString, ' MB' ].
	self >= 1000
		ifTrue: [ ^ ((self / 10) rounded / 100) asFloat printString, ' KB' ].
	^ self printString, ' Bytes'.
! !

!OpaqueImage methodsFor:'converting'!

tfAsCachedImage

	^ self class
		figure: figure tfAsCachedImage
		shape:  shape tfAsCachedImage
! !

!OpaqueImage methodsFor:'converting'!

tfAsResourceModel

	^ Twoflower::TfImageModel image: self
! !

!OpaqueImage methodsFor:'accessing'!

tfConvertForGraphicsDevice: aGraphicsDevice

	^ self class
		figure: (self figure tfConvertForGraphicsDevice: aGraphicsDevice)
		shape: (self shape tfConvertForGraphicsDevice: aGraphicsDevice)
! !

!OpaqueImage methodsFor:'accessing'!

tfShrunkenBy: scale

	^ self class
		figure: (self figure tfShrunkenBy: scale)
		shape: (self shape tfShrunkenBy: scale)
! !

!Point methodsFor:'converting'!

tfAsTranslation

	^ self
! !

!PositionableStream methodsFor:'accessing'!

tfDebugString

	^ self tfDebugStringOfSize: 40
! !

!PositionableStream methodsFor:'accessing'!

tfDebugStringOfSize: n

	| str i |
	str := TextStream on: (String new: 2*n+3).
	self position > 1
		ifTrue: [
			i := (self position - n) max: 1.
			str next: (self position - i) putAll: self collection startingAt: i
		].
	str emphasis: (Array with: #bold with: #color ->ColorValue red).
	str nextPut: $>.
	str emphasis: (Array with: #bold with: #color ->ColorValue black).
	str nextPut: (self collection at: self position).
	str emphasis: (Array with: #bold with: #color ->ColorValue red).
	str nextPut: $<.
	str emphasis: nil.
	self position < self collection size
		ifTrue: [
			i := n min: (self collection size - self position).
			str next: i putAll: self collection startingAt: self position + 1.
		].
	^ str contents.
! !

!Rectangle methodsFor:'accessing'!

tfAsTranslation

	^ self origin
! !

!Rectangle methodsFor:'copying'!

tfDeepCopy

	^ self copy origin: origin copy; corner: corner copy; yourself
! !

!Rectangle methodsFor:'transforming'!

tfNormalize

	| newOrigin newCorner |
	newOrigin := (origin x min: corner x)@(origin y min: corner y).
	newCorner := (origin x max: corner x)@(origin y max: corner y).
	origin := newOrigin.
	corner := newCorner.
! !

!Rectangle methodsFor:'transforming'!

tfNormalized

	^ self copy tfNormalize
! !

!ScreenFont class methodsFor:'accessing'!

tfEncodings
	"Screen default preferredFontClass tfEncodings"

	^ (FontEncodings at: self platformName)  collect: [ : enc | enc name ]
! !

!Stream methodsFor:'accessing'!

tfCoreStream

	^ self
! !

!Stream methodsFor:'accessing'!

tfLineEndCollection

	^ String with: Character cr
! !

!Stream methodsFor:'accessing'!

tfNextLine

	^ self upToAndSkipThroughAll: self tfLineEndCollection
! !

!SubCanvas methodsFor:'accessing'!

tfApplication

	^ client
! !

!SubCanvas methodsFor:'accessing'!

tfApplication: anApp

	client := anApp
! !

!TabControlComposite methodsFor:'accessing'!

tfApplication

	^ subCanvas tfApplication
! !

!TabControlComposite methodsFor:'accessing'!

tfApplication: anApp

	^ subCanvas tfApplication: anApp
! !

!Twoflower class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/extensions.st,v 1.4 2009/11/07 04:06:01 cg Exp $'
! !

!URL methodsFor:'accessing'!

loadResourceFor: aTfResourceManager descriptor: urlDescriptor

	urlDescriptor allowsDownload
		ifTrue: [ aTfResourceManager download: urlDescriptor ].
	^ nil
! !

!URL methodsFor:'accessing'!

tfAsURL

	^ self
! !

!URL methodsFor:'accessing'!

tfAsURLDescriptor

	^ Twoflower::TfURLDescriptor url: self
! !

!URL methodsFor:'accessing'!

tfDirectResource

	^ nil
! !

!URL methodsFor:'accessing'!

tfDownloadFilename

	^ 'file.dat'
! !

!URL methodsFor:'accessing'!

tfEncode
! !

!URL methodsFor:'accessing'!

tfFragment

	^ nil
! !

!URL methodsFor:'accessing'!

tfMimeEntityFor: aTfResourceManager descriptor: urlDescriptor

	^ nil
! !

!URL methodsFor:'accessing'!

tfRememberVisit

	^ true
! !

!URL methodsFor:'accessing'!

tfResolveRelativeURL: anURIOrString
	" create an absolute URL from the given relative URL "

	^ self error: 'cannot resolve URL: ', anURIOrString
! !

!URL methodsFor:'accessing'!

tfResolveSymbols
	" resolve symbolic components like '..' "
! !

!URL methodsFor:'accessing'!

tfResolveURL: anURIOrString
	" create an absolute URL from the given absolute or relative URL "

	| s |
	s := anURIOrString asString.
	^ ('*://*' match: s)
		ifTrue: [ Net.URL tfFromString: s ]
		ifFalse: [
			s isEmpty
				ifTrue: [ self copy ]
				ifFalse: [ self tfResolveRelativeURL: anURIOrString ]
		].
! !

!URL methodsFor:'accessing'!

tfURLWithoutFragment

	^ self copy
! !

!URL class methodsFor:'utility'!

tfEncode: aString on: aStream
	" copied from URLEncoder (which is in the Wave package) "

	aString do: [ : ch |
		ch = Character space
			ifTrue: [aStream nextPut: $+]
			ifFalse: [
				((ch isAlphaNumeric or: [ch == $_]) not)
					ifTrue: [
						aStream nextPut: $%.
						ch asInteger printOn: aStream base: 16 digitsToPad: 2 highDigit: 2.
					]
				ifFalse: [ aStream nextPut: ch ]
		].
	].
! !

!URL class methodsFor:'instance creation'!

tfFromString: aString

	| s |
	s := "self decode:" aString.
	('*://*' match: s)
		ifFalse: [ s := 'http://', s ].
	^ (self fromString: s)
		tfResolveSymbols;
		tfEncode;
		yourself
! !

!URL class methodsFor:'utility'!

tfLooksLikeURL: aString

	^ (aString includes: $.) or: [ (aString indexOfSubCollection: '://' startingAt: 1) > 0 ]
! !

!VisualComponent methodsFor:'Twoflower'!

tfComposesComponents

	^ false
! !

!VisualComponent methodsFor:'Twoflower'!

tfLocalTranslateBy: aPoint
! !

!VisualComponent methodsFor:'Twoflower'!

tfWrapped
	" wrap me with additional wrappers if needed "

	^ self
! !

!VisualPart methodsFor:'accessing'!

tfNonWrappingContainer

	^ self
! !

!VisualPart methodsFor:'Twoflower'!

tfSetupComponents
! !

!VisualPart methodsFor:'accessing'!

tfTopDocumentView

	^ self container tfTopDocumentView
! !

!VisualPart methodsFor:'accessing'!

tfTranslatingWrapper

	^ self container tfTranslatingWrapper
! !

!VisualPart methodsFor:'accessing'!

tfWrappedComponent

	^ self
! !

!Wrapper methodsFor:'Twoflower'!

tfAllViewsDo: aBlock

	self component tfAllViewsDo: aBlock
! !

!Wrapper methodsFor:'Twoflower'!

tfComposesComponents

	^ self component notNil
		ifTrue: [ self component tfComposesComponents ]
		ifFalse: [ false ]
! !

!Wrapper methodsFor:'accessing'!

tfNonWrappingContainer

	^ self container tfNonWrappingContainer
! !

!Wrapper methodsFor:'Twoflower'!

tfRenderingFinished

	self component notNil
		ifTrue: [ self component tfRenderingFinished ].
! !

!Wrapper methodsFor:'Twoflower'!

tfSetupComponents

	self component notNil
		ifTrue: [ self component tfSetupComponents ].
! !

!Wrapper methodsFor:'accessing'!

tfWrappedComponent

	^ self component tfWrappedComponent
! !

!XML::Comment methodsFor:'printing'!

tfPrintOn: aTextStream depth: depth emphasicDictionary: emphasicDictionary

	(Twoflower::TfSAXWriter new output: aTextStream)
		comment: text
		from: 1
		to: text size
! !

!XML::Document methodsFor:'Twoflower'!

tfPrintOn: aTextStream emphasicDictionary: emphasicDictionary

	nodes do: [:n | n tfPrintOn: aTextStream emphasicDictionary: emphasicDictionary. aTextStream cr]
! !

!XML::Node methodsFor:'Twoflower'!

tfAdaptToTable: aTfTableEnumerator
	" table elemenets have been enumerated, now make sure that stored information is valid"
! !

!XML::Node methodsFor:'Twoflower'!

tfAddNodesMatching: matchBlock upTo: stopBlock to: aCollection

	(matchBlock value: self)
		ifTrue: [ aCollection add: self ].
! !

!XML::Node methodsFor:'Twoflower'!

tfAddToBookmarks: aTfBookmarkContainer emit: aBoolean

	^ aTfBookmarkContainer
! !

!XML::Node methodsFor:'Twoflower'!

tfComponents

	^ Array new
! !

!XML::Node methodsFor:'printing'!

tfDebugString

	| str |
	str := String new writeStream.
	 self
		printNoIndentOn: str
		endSpacing: [ :e :n | ]
		spacing: [ :e :n | ].
	^ str contents
! !

!XML::Node methodsFor:'Twoflower'!

tfDocument

	^ self parent tfDocument
! !

!XML::Node methodsFor:'Twoflower'!

tfEnumerateListItem: index of: listElement

	^ index
! !

!XML::Node methodsFor:'Twoflower'!

tfEnumerateTable: aTfTableEnumerator
	" compute row/column indexes of cells/rows "
! !

!XML::Node methodsFor:'Twoflower'!

tfFixNodes
! !

!XML::Node methodsFor:'Twoflower'!

tfFormElement

	^ self parent tfFormElement
! !

!XML::Node methodsFor:'Twoflower'!

tfIsColElement

	^ false
! !

!XML::Node methodsFor:'Twoflower'!

tfIsFormPart

	^ false
! !

!XML::Node methodsFor:'testing'!

tfIsListOption

	^ false
! !

!XML::Node methodsFor:'Twoflower'!

tfIsTable

	^ false
! !

!XML::Node methodsFor:'Twoflower'!

tfIsTableCell

	^ false
! !

!XML::Node methodsFor:'testing'!

tfIsTableElement

	^ false
! !

!XML::Node methodsFor:'Twoflower'!

tfIsTableRow

	^ false
! !

!XML::Node methodsFor:'Twoflower'!

tfIsVisibleInBody

	^ true
! !

!XML::Node methodsFor:'Twoflower'!

tfNodesMatching: matchBlock upTo: stopBlock

	| nodes |
	nodes := OrderedCollection new.
	self tfAddNodesMatching: matchBlock upTo: stopBlock to: nodes.
	^ nodes
! !

!XML::Node methodsFor:'Twoflower'!

tfPreviousElementNamed: elementName
	" find the previous element (left neighbor/parent) with the given name "

	^ self tfPreviousElementNamed: elementName ifAbsent: [ nil ]
! !

!XML::Node methodsFor:'Twoflower'!

tfPreviousElementNamed: elementName ifAbsent: aBlock
	" find the previous element (left neighbor/parent) with the given name "

	| i neighborElements prev |
	neighborElements := self parent elements.
	i := neighborElements indexOf: self.
	i > 1
		ifTrue: [ prev := neighborElements at: i - 1]
		ifFalse: [ prev := self parent ].
	^ prev tfSelfOrPreviousElementNamed: elementName ifAbsent: aBlock
! !

!XML::Node methodsFor:'Twoflower'!

tfPrintOn: aTextStream depth: depth emphasicDictionary: emphasicDictionary

	self printOn: aTextStream depth: depth
! !

!XML::Node methodsFor:'Twoflower'!

tfPrintOn: aTextStream emphasicDictionary: emphasicDictionary

	self printOn: aTextStream
! !

!XML::Node methodsFor:'Twoflower'!

tfPrintString

	| str |
	str := TextStream on: (String new: 256).
	self tfPrintOn: str emphasicDictionary: Dictionary new.
	^ str contents
! !

!XML::Node methodsFor:'Twoflower'!

tfSelfOrPreviousElementNamed: elementName ifAbsent: aBlock

	^ self tfPreviousElementNamed: elementName ifAbsent: aBlock
! !

!XML::Node methodsFor:'Twoflower'!

tfTableElement

	^ self parent tfTableElement
! !

!XML::Node methodsFor:'printing'!

tfTreeDisplayString

	^ self simpleDescription
! !

!XML::Node methodsFor:'Twoflower'!

tfViewClass

	^ nil
! !

!XML::StreamWrapper methodsFor:'accessing'!

tfEncoder: encoder

	(stream respondsTo: #encoder:)
		ifTrue: [ stream encoder: encoder ].
! !
