"
 COPYRIGHT (c) 2005 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/webServer' }"

HTTPResponse subclass:#FCGIResponse
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-FCGI-Server'
!

HTTPResponseHeader subclass:#ResponseHeader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:FCGIResponse
!

!FCGIResponse class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2005 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!FCGIResponse methodsFor:'initialization'!

initialize

    replyPresenterDataStream := WriteStream on:''.
    self replyPresenter:(request server serverReplyPresenterFor:request).
    header := ResponseHeader response:self.
    responseAlreadySent := false.
! !

!FCGIResponse methodsFor:'private sending'!

endRecord
    request connection writeRecord:(FCGI::FCGIRecordEndRequest newRequestId:request requestId).
!

endRecordWithStatus:anInteger
    request connection writeRecord:(FCGI::FCGIRecordEndRequest newRequestId:request requestId status:anInteger).
!

send:aStream
    |connection requestId data|

    requestId := request requestId.
    connection := request connection.

    [data := aStream nextAvailable:65535. data notEmptyOrNil] whileTrue:[
        connection writeRecord:
            (FCGI::FCGIRecordStdout newRequestId:requestId data:data)
    ].
! !

!FCGIResponse methodsFor:'sending'!

send
    "copy over from some stream and send it as a response.
     Any already collected replyData is ignored.
     Typically used with big files, which we do not want to fill up our memory
     (i.e. the document is not loaded as a whole)"

    |buffer responseStream headerLength contentLength|

    responseAlreadySent ifTrue:[^ self].
    request connection isOpen ifFalse:[^ self].

    dataStream isNil ifTrue:[
        "/ if nobody is setting explicit data or dataStream 
        "/ the data is coming from the replyPresenter
        "/ default is the server presenter if not overwritten by service
        dataStream := ReadWriteStream on:(String new:1024).
        replyPresenter present:self on:dataStream.
        dataStream reset.
    ].
    (contentLength := header contentLength) isNil ifTrue:[
        "/ set header content length if not done before
        contentLength := dataStream size.
        header contentLength:contentLength.
    ].

    buffer := String uninitializedNew:(8*1024).
    responseStream := WriteStream on:buffer.
    header generateResponseOn:responseStream.

    request server verbose ifTrue:[
        Transcript 
            showCR:'>-- Reply (connection=', request connection printString, ') -- to ', request url;
            show:(responseStream contents copyReplaceAll:Character return with:Character space).

        request server verboseData ifTrue:[
            Transcript showCR:'.....................'.
            dataStream isExternalStream ifTrue:[
                Transcript showCR:'... reading from stream...'.
            ] ifFalse:[
                Transcript showCR:(dataStream contents printString)
            ]
        ].
        Transcript showCR:'>--------------------'.
    ].

    headerLength := responseStream size.

    request isHEAD ifTrue:[
        self send:responseStream contents readStream.
    ] ifFalse:[
        "/ try hard to send it as one block
        (headerLength + contentLength) <= (8*1024) ifTrue:[
            responseStream nextPutAll:dataStream contents.
            "dataStream copyToEndInto:responseStream."
            self send:responseStream contents readStream .
        ] ifFalse:[
            self send:responseStream contents readStream.
            self send:dataStream.
        ].
    ].
    self endRecord.
    request logAccess.
    responseAlreadySent := true.
! !

!FCGIResponse::ResponseHeader methodsFor:'sending'!

externalHeaderString:aCompleteHeaderString
    "some services (e.g. comanche services) generates his own response header 
     send this header if set from service"

    |rest stream statusLine reply|

    "Fetch the HTTP status line and extract the result code"

    (aCompleteHeaderString startsWith:'HTTP') ifTrue:[
        stream := aCompleteHeaderString readStream.
        statusLine := stream nextLine.
        rest := stream upToEnd.
        reply := statusLine asCollectionOfWords.
        reply size > 2 ifTrue:[ 
            self responseCode:(Integer readFrom:(reply at:2)).
        ].
    ] ifFalse:[
        rest := aCompleteHeaderString.
    ].

    super externalHeaderString:rest.
!

generateResponseStatusOn:headerStream
    |reasonPhrase|

    responseCode notNil ifTrue:[
        headerStream nextPutAll:'Status: '.
        responseCode printOn:headerStream.
        reasonPhrase := self class reasonPhrasesDictionary at:responseCode ifAbsent:nil.
        reasonPhrase notNil ifTrue:[
            headerStream space; nextPutAll:reasonPhrase.
        ].
        headerStream crlf.
    ] ifFalse:[
        "no Status: means success"
        responseCode := 200
    ].
! !

!FCGIResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/FCGIResponse.st,v 1.6 2005/12/28 12:34:10 cg Exp $'
! !
