"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPFileAccessLogger subclass:#HTTPApacheFormatAccessLogger
	instanceVariableNames:'formatString'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!HTTPApacheFormatAccessLogger class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Generates Apache compatible logs

    Apache Format directives:

        %...a:          Remote IP-address
        %...A:          Local IP-address
        %...B:          Bytes sent, excluding HTTP headers.
        %...b:          Bytes sent, excluding HTTP headers. In CLF format
                i.e. a '-' rather than a 0 when no bytes are sent.
        %...c:          Connection status when response was completed.
                        'X' = connection aborted before the response completed.
                        '+' = connection may be kept alive after the response is sent.
                        '-' = connection will be closed after the response is sent.
        %...{FOOBAR}e:  The contents of the environment variable FOOBAR
        %...f:          Filename
        %...h:          Remote host
        %...H           The request protocol
        %...{Foobar}i:  The contents of Foobar: header line(s) in the request
                        sent to the server.
        %...l:          Remote logname (from identd, if supplied)
        %...m           The request method
        %...{Foobar}n:  The contents of note Foobar from another module.
        %...{Foobar}o:  The contents of Foobar: header line(s) in the reply.
        %...p:          The canonical Port of the server serving the request
        %...P:          The process ID of the child that serviced the request.
        %...q           The query string (prepended with a ? if a query string exists,
                        otherwise an empty string)
        %...r:          First line of request
        %...s:          Status.  For requests that got internally redirected, this is
                        the status of the *original* request --- %...>s for the last.
        %...t:          Time, in common log format time format (standard english format)
        %...{format}t:  The time, in the form given by format, which should
                        be in strftime(3) format. (potentially localized)
        %...T:          The time taken to serve the request, in seconds.
        %...u:          Remote user (from auth; may be bogus if return status (%s) is 401)
        %...U:          The URL path requested, not including any query string.
        %...v:          The canonical ServerName of the server serving the request.
        %...V:          The server name according to the UseCanonicalName setting.

    [author:]
        Christian Penk
"
! !

!HTTPApacheFormatAccessLogger class methodsFor:'defaults'!

clfFormatString
" Common Log Format (CLF)"

    ^ '%h %l %u %t \"%r\" %>s %b'
!

defaultFormatString
" default is the NCSA Common Log Format"

    ^ self ncsaFormatString
!

ncsaFormatString
" NCSA extended/combined log format"

    ^ '%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-agent}i\"'
!

settingsApplication

    ^ ApacheFormatAccessLoggerSettingsApp
! !

!HTTPApacheFormatAccessLogger methodsFor:'accessing'!

formatString
    formatString isNil ifTrue:[
        formatString := self class defaultFormatString.
    ].
    ^ formatString
!

formatString:something
    formatString := something.
! !

!HTTPApacheFormatAccessLogger methodsFor:'actions'!

logAccessFrom:aRequest to:aStreamOrFilename 
    "aStreamOrFilename notNil logging to aStreamOrFilename otherwise
     log to instVar logFile "

    aStreamOrFilename isNil ifTrue:[
        logFile isNil ifTrue:[
            self error:'can not log without a log file'.
            ^ self
        ] 
    ] ifFalse:[
        aStreamOrFilename ~~ logStream ifTrue:[
            aStreamOrFilename isStream ifTrue:[
                self logStream:aStreamOrFilename.
            ] ifFalse:[
                self logFile:aStreamOrFilename asFilename.
            ].
        ].
    ].

    self log:aRequest on:logStream.
! !

!HTTPApacheFormatAccessLogger methodsFor:'html presentation'!

changeFromRequest:aRequest 
    "Superclass says that I am responsible to implement this method"
    | fieldsFormatString |

    fieldsFormatString := aRequest postDataFieldAt:'formatString'.
    fieldsFormatString notNil ifTrue:[
        self formatString:fieldsFormatString.
    ].
!

inputFieldsForRequest:aRequest 
    "Superclass says that I am responsible to implement this method"

    |stream|

    stream := WriteStream on:''.
    stream nextPutLine:'<H3>Apache Format Request Logger Settings</H3>'.
    stream nextPutLine:(aRequest inputFieldStringFor:'formatString' 
                                        type:nil 
                                        size:60 
                                        label:'Format String:' 
                                        initialValue:self formatString 
                                        append:nil).
    ^ stream contents.
! !

!HTTPApacheFormatAccessLogger methodsFor:'parsing'!

log:aRequest on:outputStream
    |formatStream nextCharacter|

    formatStream := self formatString readStream.

    [formatStream atEnd] whileFalse:[
        nextCharacter := formatStream next.
        (nextCharacter == $\) ifTrue:[
            self performBackSlashFor:formatStream outStream:outputStream request:aRequest.    
        ] ifFalse:[
            (nextCharacter == $%) ifTrue:[
                self performPercentDirectiveFor:formatStream outStream:outputStream request:aRequest.
            ] ifFalse:[
                nextCharacter == Character space ifTrue:[
                    outputStream nextPut:nextCharacter.
                ]
            ]
        ].
    ].
    outputStream cr
!

performBackSlashFor:formatStream outStream:outStream request:aRequest
    | next |
    next := formatStream next.
    next == $r ifTrue:[
        outStream nextPut:Character return.
        ^ self
    ].
    next == $n ifTrue:[
        outStream nextPut:Character nl.
        ^ self
    ].
    next == $b ifTrue:[
        outStream nextPut:Character backspace.
        ^ self
    ].
    next == $f ifTrue:[
        outStream nextPut:Character newPage.
        ^ self
    ].
    next == $e ifTrue:[
        outStream nextPut:Character esc.
        ^ self
    ].
    outStream nextPut:next.
!

performPercentDirectiveFor:formatStream outStream:outStream request:aRequest
    |contentLength connectionStatus fooBarVar headerLineContents headerLines 
     time offset user nextCharacter|

    nextCharacter := formatStream peek.
    nextCharacter isLetter ifFalse:[
        nextCharacter == ${ ifTrue:[
            formatStream next.
            fooBarVar := formatStream nextUpTo:$}.
            formatStream next.

            nextCharacter := formatStream peek.
            nextCharacter == $e ifTrue:[
                "The contents of the environment variable FOOBAR"
                outStream nextPutAll:(OperatingSystem getEnvironment:fooBarVar) ? ''.
                ^ self
            ].
            (nextCharacter == $i or:[nextCharacter == $o]) ifTrue:[
                "The contents of Foobar: header line(s) in the request"
                headerLines := fooBarVar asCollectionOfWords.
                headerLineContents := StringCollection new.
                headerLines do:[:headerLine|
                    headerLineContents add:((aRequest fieldValueFor:(headerLine asLowercase)) ? '-').
                ].
                outStream nextPutAll:(headerLineContents asStringWith:(Character space)).
                ^ self
            ].
            nextCharacter == $n ifTrue:[
                "The contents of note Foobar from another module."
                outStream nextPut:$-.
                ^ self
            ].
            nextCharacter == $t ifTrue:[
                "The time, in the form given by format, which should
                 be in strftime(3) format. (potentially localized)"
                self shouldImplement.
            ].
            ^ self.
        ].

        "/ skip next non letter character. 
        "/ Can't find documentation what to do with ('!!' '>' '<' etc.)
        nextCharacter := formatStream nextPeek
    ].

    nextCharacter := formatStream next.
    (nextCharacter == $a) ifTrue:[
        "Remote IP-address"
        outStream nextPutAll:aRequest peer.
        ^ self
    ].
    (nextCharacter == $A) ifTrue:[
        "Local IP-address"
        aRequest connection getFullSocketAddress printOn:outStream.
        ^ self
    ].
    (nextCharacter == $B) ifTrue:[
        "Bytes sent, excluding HTTP headers"
        contentLength := (aRequest contentLength - aRequest replyHeader size).
        contentLength printOn:outStream.
        ^ self
    ].
    (nextCharacter == $b) ifTrue:[
        "Bytes sent, excluding HTTP headers. In CLF format
         i.e. a '-' rather than a 0 when no bytes are sent "
        contentLength := (aRequest response contentLength).
        contentLength notNil ifTrue:[
            contentLength printOn:outStream.
        ] ifFalse:[
            outStream nextPut:$-.
        ].
        ^ self
    ].
    (nextCharacter == $c) ifTrue:[
        "Connection status when response was completed.
         'X' = connection aborted before the response completed.
         '+' = connection may be kept alive after the response is sent.
         '-' = connection will be closed after the response is sent."
        self shouldImplement.
        aRequest response responseCode ifTrue:[
            connectionStatus := 'X'
        ] ifFalse:[
            aRequest connection isKeepAlive ifTrue:[
                connectionStatus := '+'
            ] ifFalse:[
                connectionStatus := '-'
            ].
        ].
        outStream nextPutAll:connectionStatus.
        ^ self
    ].
    (nextCharacter == $f) ifTrue:[
        "Filename"
        self shouldImplement.
        outStream nextPut:$-.
        ^ self
    ].
    (nextCharacter == $H) ifTrue:[
        "The request protocol"
        outStream nextPut:$-.
        ^ self
    ].
    (nextCharacter == $h) ifTrue:[
        "Remote host"
        "/ has been commented, because it could take  along time if the DNS is slow or misconfigured
        "/ aStream nextPutAll:(aRequest peerName ? '-').
        aRequest peer printAddressOn:outStream.
        ^ self
    ].
    (nextCharacter == $l) ifTrue:[
        "Remote logname (from identd, if supplied)"
        outStream nextPut:$-.
        ^ self
    ].
    (nextCharacter == $m) ifTrue:[
        "The request method"
        outStream nextPutAll:aRequest method ? '-'.
        ^ self
    ].
    (nextCharacter == $p) ifTrue:[
        "The canonical Port of the server serving the request"
        aRequest port printOn:outStream.
        ^ self
    ].
    (nextCharacter == $P) ifTrue:[
        "The process ID of the child that serviced the request."
        aRequest connection serveProcess id printOn:outStream.
        ^ self
    ].
    (nextCharacter == $q) ifTrue:[
        "The query string (prepended with a ? if a query string exists,
         otherwise an empty string)"
        outStream nextPutAll:aRequest queryString ? ''.
        ^ self
    ].
    (nextCharacter == $r) ifTrue:[
        "First line of request"
        | startLine |
        startLine := aRequest headerParser startLine.
        startLine notEmptyOrNil ifTrue:[
            outStream nextPutAll:startLine.
        ].
        ^ self
    ].
    (nextCharacter == $s) ifTrue:[
        "Status.  For requests that got internally redirected, this is
         the status of the *original* request --- %...>s for the last."
        aRequest response responseCode printOn:outStream.
        ^ self
    ].
    (nextCharacter == $t) ifTrue:[
        "Time, in common log format time format (standard english format)"
        time := aRequest connection startReadRequestTime.

        "could do:
          time printOn:Transcript format:'[%d/%(ShortMonthName)/%y:%h:%m:%s '.
        - but speed up everything"

        outStream nextPut:$[.   
        time day printOn:outStream base:10 size:2 fill:$0.
        outStream nextPut:$/.
        outStream nextPutAll:(time asDate abbreviatedMonthNameForLanguage:#en) asUppercaseFirst.
        outStream nextPut:$/.
        time year printOn:outStream base:10 size:4 fill:$0.
        outStream nextPut:$:.
        time hours printOn:outStream base:10 size:2 fill:$0.
        outStream nextPut:$:.
        time minutes printOn:outStream base:10 size:2 fill:$0.
        outStream nextPut:$:.
        time seconds printOn:outStream base:10 size:2 fill:$0.
        outStream space.

        offset := time utcOffset.
        outStream nextPut:(offset <= 0 ifTrue:[ $+ ] ifFalse:[ $- ]).
        offset := offset abs.
        (offset // 3600) printOn:outStream base:10 size:2 fill:$0.
        (offset \\ 3600) printOn:outStream base:10 size:2 fill:$0.

        outStream nextPut:$].
        ^ self
    ].
    (nextCharacter == $T) ifTrue:[
        "The time taken to serve the request, in seconds."
        (aRequest timeToPerform \ 1000) printOn:outStream.
        ^ self
    ].
    (nextCharacter == $u) ifTrue:[
        "Remote user (from auth; may be bogus if return status (%s) is 401)"
        user := aRequest authenticatedUsername.
        user notNil ifTrue:[
            user printOn:outStream.
        ] ifFalse:[
            outStream nextPut:$-.
        ].
        ^ self
    ].
    (nextCharacter == $U) ifTrue:[
        "The URL path requested, not including any query string."
        self shouldImplement.
        outStream nextPut:$-.
        ^ self
    ].
    (nextCharacter == $v) ifTrue:[
        "The canonical ServerName of the server serving the request."
        self shouldImplement.
        outStream nextPut:$-.
        ^ self
    ].
    (nextCharacter == $V) ifTrue:[
        "The server name according to the UseCanonicalName setting."
        self shouldImplement.
        outStream nextPut:$-.
        ^ self
    ].
! !

!HTTPApacheFormatAccessLogger class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPApacheFormatAccessLogger.st,v 1.16 2012/04/16 08:04:57 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPApacheFormatAccessLogger.st,v 1.16 2012/04/16 08:04:57 stefan Exp $'
! !
