"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPFileService subclass:#HTTPCGIService
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

!HTTPCGIService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This service provides a callout mechanism for cgi-scripts.
    It inherits from fileService, so both cgi-scripts and 
    regular files can be located in the same directory.
    (this will no longer be needed, when we can have multiple
     services linked to the same linkName).

    [author:]
        Claus Gittinger
"
! !

!HTTPCGIService class methodsFor:'defaults'!

linkName
    "return the default path (with slash)"

    ^ '/cgi-bin'
! !

!HTTPCGIService methodsFor:'private'!

cgiEnvironmentFor:aRequest cgiFile:scriptFile
    |env t authenticator|

    env := Dictionary new.

    (t := aRequest contentLength) notNil ifTrue:[
        env at:'CONTENT_LENGTH' put:(t printString).
    ].
    (t := aRequest contentType) notNil ifTrue:[
        env at:'CONTENT_TYPE' put:(t printString).
    ].
    env at:'GATEWAY_INTERFACE' put:'CGI/1.2'.
    "/ env at:'HTTP_*' put:''.
    env at:'PATH_INFO' put:(aRequest relativePath).
    env at:'PATH_TRANSLATED' put:(scriptFile pathName).
    env at:'QUERY_STRING' put:(aRequest queryString).
    t := String streamContents:[:s | aRequest peer printOn:s withPort:false].
    env at:'REMOTE_ADDR' put:t.
    env at:'REMOTE_HOST' put:aRequest peerName.
    (t := aRequest userAgent) notNil ifTrue:[
        env at:'REMOTE_IDENT' put:t.
    ].
    (authenticator := aRequest authenticator "authorization") notNil ifTrue:[
        env at:'AUTH_TYPE' put:authenticator mechanismName "nameInHTTPHeader".
        env at:'REMOTE_USER' put:authenticator username.
    ].
    env at:'REQUEST_METHOD' put:aRequest method.
    env at:'SCRIPT_NAME' put:aRequest relativePath.
    "/ env at:'SERVER_NAME' put:aRequest server virtualHostName.
    env at:'SERVER_NAME' put:OperatingSystem getHostName.
    env at:'SERVER_PORT' put:aRequest server port printString.
    env at:'SERVER_PROTOCOL' put:aRequest server class httpVersion.
    env at:'SERVER_SOFTWARE' put:'ST/X WebServer'.

    ^ env
!

executeCGIScript:scriptFile for:aRequest 
    |scriptFilename command execDir env connection cmdOutReader 
     executeFailed perlCmd|

    scriptFilename := scriptFile asFilename.
    command := scriptFilename baseName.
    execDir := scriptFilename directoryName.

    env := self cgiEnvironmentFor:aRequest cgiFile:scriptFilename.

    connection := aRequest connection.
    connection keepConnection:false.

    cmdOutReader := self nonProcessingCommandOutputReaderFor:aRequest outputTo:connection socket.

    executeFailed := false.
    OperatingSystem
        executeCommand:('./' , command)
        inputFrom:nil
        outputTo:cmdOutReader
        errorTo:nil
        auxFrom:nil 
        environment:env
        inDirectory:execDir
        lineWise:true 
        onError:[:status | executeFailed := true].

    executeFailed ifTrue:[
        "/ CG: a temporary hack to support bugzilla (which has cgi-files for perl)
        "/ need a better way to do this

        (scriptFilename contentsAsString startsWith:'#!!/usr/bin/perl') ifTrue:[
            executeFailed := false.
            OperatingSystem isMSWINDOWSlike ifTrue:[
                perlCmd := 'c:\perl\bin\perl.exe'.    
            ] ifFalse:[
                perlCmd := 'perl'.    
            ].
            OperatingSystem
                executeCommand:(perlCmd,' -T ' , command)
                inputFrom:nil
                outputTo:cmdOutReader
                errorTo:Transcript
                auxFrom:nil 
                environment:env
                inDirectory:execDir
                lineWise:true 
                onError:[:status | executeFailed := true].
        ].
        executeFailed ifTrue:[
            aRequest reportServerError:'Error in CGI-Script.' moreInfo:nil.
            "/ not reached
            ^ self.
        ]
    ].

    aRequest response responseAlreadySent:true.
!

nonProcessingCommandOutputReaderFor:aRequest outputTo:outputStream
    "return a stream which
     simply prepends an HTML-ok line; perform nl->crnl processing in the header lines
     and pass the rest of the cgi-scripts output unchanged."
    
    |inHeader firstLine cmdOutReader|

    inHeader := firstLine := true.
    cmdOutReader := ActorStream new.
    cmdOutReader
        nextPutLineBlock:
            [:line | 

                inHeader ifTrue:[
                    firstLine ifTrue:[
                        outputStream nextPutAll:'HTTP/'; nextPutAll:(aRequest httpVersion).
                        outputStream nextPutAll:' 200 OK'; nextPutAll:(String crlf).
                        firstLine := false.
                    ].
                    outputStream nextPutAll:line.
                    outputStream nextPutAll:(String crlf).
                    line isEmpty ifTrue:[
                        inHeader := false.
                    ]
                ] ifFalse:[
                    outputStream nextPutLine:line.
                ].
            ].

    ^ cmdOutReader.
! !

!HTTPCGIService methodsFor:'queries-service'!

isCGIService
    ^ true
! !

!HTTPCGIService methodsFor:'requests-reply'!

process:aRequest 
    |suffix relativePath fn|

    relativePath := aRequest relativePath copyFrom:2.
    relativePath isEmptyOrNil ifTrue:[ 
        ^ super process:aRequest 
    ].
    suffix := relativePath asFilename suffix asLowercase.
    suffix = 'cgi' ifFalse:[ 
        ^ super process:aRequest 
    ].

    fn := self fileRoot asFilename construct:relativePath.
    fn exists ifFalse:[
        aRequest reportNotFound:'Not found: ' , (aRequest fullPath).
        "/ not reached
    ].

    "/ mhmh - the following is only useful with unix's execute permission;
    "/ under windows, we cannot check on this.
    OperatingSystem isUNIXlike ifTrue:[
        fn isExecutableProgram ifFalse:[
            aRequest reportServerError:'CGI-Script not Executable.' moreInfo:nil
            "/ not reached
        ].
    ].

    self executeCGIScript:fn for:aRequest.
! !

!HTTPCGIService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPCGIService.st,v 1.8 2008/12/09 14:02:55 cg Exp $'
! !
