"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPSelectorService subclass:#HTTPClassDocService
	instanceVariableNames:'rootDirectory exportedModules classListExpirationTime
		classDocExpirationTime'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

!HTTPClassDocService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Generates class documentation

    [author:]
        Claus Gittinger
"
!

examples
"
    HTTPClassDocService registerServiceIn:(HTTPServer runningServers first) as:'XX'
"
! !

!HTTPClassDocService class methodsFor:'defaults'!

defaultClassDocExpirationTime
    ^ 1
!

defaultClassListExpirationTime
    ^ 7
!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
!

defaultRootDirectory
    ^ '../../doc'
!

isThreadSafe
    ^ true
!

linkName
    "return the default linkName path (with slash)"

    ^ '/ClassDoc'
! !

!HTTPClassDocService class methodsFor:'queries-service'!

settingsApplicationClass
    "my application in the settings dialog"

    ^ HTTPClassDocServiceSettingsApp
! !

!HTTPClassDocService methodsFor:'accessing'!

classDocExpirationTime
    classDocExpirationTime isNil ifTrue:[
        ^ self class defaultClassDocExpirationTime
    ].
    ^ classDocExpirationTime
!

classDocExpirationTime:something
    classDocExpirationTime := something.
!

classListExpirationTime
    classListExpirationTime isNil ifTrue:[
        ^ self class defaultClassListExpirationTime
    ].
    ^ classListExpirationTime
!

classListExpirationTime:something
    classListExpirationTime := something.
!

rootDirectory
    rootDirectory isNil ifTrue:[
        rootDirectory := self class defaultRootDirectory.
    ].
    ^ rootDirectory
!

rootDirectory:something
    self removeRequiredForeignServicesFrom:self httpServer.
    rootDirectory := something.
    self addRequiredForeignServicesTo:self httpServer
! !

!HTTPClassDocService methodsFor:'maintenance html presentation'!

specificChangeFromRequest:aRequest 
    "extracts parameter from an html-maintenance post-request"

    |tmp|

    tmp := aRequest urlParameterAt:'rootDirectory'.
    tmp notNil ifTrue:[
        self rootDirectory:(tmp asFilename).
    ].
    tmp := aRequest urlParameterAt:'classListExpirationTime'.
    tmp notNil ifTrue:[
        self classListExpirationTime:(tmp asNumber).
    ].
    tmp := aRequest urlParameterAt:'classDocExpirationTime'.
    tmp notNil ifTrue:[
        self classDocExpirationTime:(tmp asNumber).
    ].
!

specificInputFieldsForRequest:aRequest
    "generates the html-fragment for the maintenance form"

    aRequest 
        inputFieldFor:'rootDirectory' 
            type:nil 
            size:'50' 
            label:('Documentation Root Directory:<BR><BLOCKQUOTE>') 
            initialValue:self rootDirectory asString;
        nextPutLine:'</BLOCKQUOTE><BR>'.
    aRequest 
        inputFieldFor:'classListExpirationTime' 
            type:nil 
            size:'50' 
            label:('Class List Expiration Time (Seconds):<BR><BLOCKQUOTE>') 
            initialValue:self classListExpirationTime asString;
        nextPutLine:'</BLOCKQUOTE><BR>'.
    aRequest 
        inputFieldFor:'classDocExpirationTime' 
            type:nil 
            size:'50' 
            label:('Class Doc Expiration Time (Seconds):<BR><BLOCKQUOTE>') 
            initialValue:self classDocExpirationTime asString;
        nextPutLine:'</BLOCKQUOTE><BR>'.
! !

!HTTPClassDocService methodsFor:'queries-access'!

exportedModules
    "list the modules we make accessible.
     Everything else is not shown (i.e. classes from other modules are invisible).
     If never set, only the public classes from stx are shown."

    ^ exportedModules ? #('stx')
!

exportedModules:aCollection
    "set the list of modules we make accessible.
     Everything else is not shown (i.e. classes from other modules are invisible).
     If never set, only the public classes from stx are shown."

    exportedModules := aCollection
! !

!HTTPClassDocService methodsFor:'queries-service'!

allowedMessages
    ^ #(
        classDocIndex: 
        classDocOf: 
        classListForPrefix:
        classListForPackage:
        packageIndex:
    ).
!

defaultMessage
    ^ #classDocIndex
!

packageIsShown:aPackage
    ^ (self exportedModules includes:aPackage)
    or:[ self exportedModules contains:[:m | aPackage startsWith:(m,':')]].
! !

!HTTPClassDocService methodsFor:'registration'!

addRequiredForeignServicesTo:aServer 
    "I need a few file services for the bitmaps, icons and doc files"

    |rootDirectory|

    aServer isNil ifTrue:[
        "no server yet, do it later..."
        ^ self.
    ].
    rootDirectory := self rootDirectory asFilename.
    (rootDirectory exists) ifFalse:[ 
        self error:'rootDirectory for classDocService does not exist' mayProceed:true.
        ^ self 
    ].

    (aServer serviceForLink:'/pictures' ifAbsent:nil) isNil ifTrue:[
        aServer 
            addFileServiceWithRootLinkName:'/pictures'
            rootDirectory:(rootDirectory constructString:'online/pictures')
            defaultFileName:nil.
    ].
    (aServer serviceForLink:'/doc' ifAbsent:nil) isNil ifTrue:[
        aServer 
            addFileServiceWithRootLinkName:'/doc'
            rootDirectory:rootDirectory asString
            defaultFileName:'online/english/TOP.html'.
    ].

    self addStandardFileServicesTo:aServer.

    "Modified: / 01-02-2007 / 23:21:37 / cg"
!

removeRequiredForeignServicesFrom:aServer
    |rootDirectory pictureFileService docFileService|

    rootDirectory := self rootDirectory asFilename.
    (rootDirectory exists and:[aServer notNil]) ifTrue:[
        pictureFileService := aServer serviceForLink:'/pictures' ifAbsent:nil.
        pictureFileService notNil ifTrue:[
            pictureFileService unRegisterServiceOn:aServer.
        ].
        docFileService := docFileService := aServer serviceForLink:'/doc' ifAbsent:nil.
        docFileService notNil ifTrue:[
            docFileService unRegisterServiceOn:aServer.
        ].
    ].
! !

!HTTPClassDocService methodsFor:'requests'!

classDocIndex: aRequest
    "'server:classDocIndex' generates the class documentation index page."

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Class Documentation Index'.
    replyPresenter secondHeadLine:'Class Documentation Index'.
    aRequest
        reply:'
&nbsp;<p>
Please notice that this documentation is generated dynamically 
by the (<code>',self class name,'</code>) class.
<br>It is extracted from the running webBrowser''s source code. (Thats what <b>we</b> call reflection !! )
<br>Any change in the code is instantly visible in the generated documentation. 
<br>(Hint: try adding a class in the browser...)
<p>

<h3>Documentation by Class Name:</h3>

<p>
'.
    $A to:$Z do:[:ch |
        aRequest
            reply:
(aRequest webLinkForText:ch asString service:self selector:#'classListForPrefix:' argument:ch) , ' '
    ].

    aRequest
        reply:'
<p>

<H3>By Package</H3>
<P>
' , (aRequest webLinkForText:'packages' service:self selector:#'packageIndex:') , '
<P>
'.

    "/ not yet implemented ...

false ifTrue:[
    aRequest
        reply:'
<p>

<H3>By class category</H3>
<P>
' , (aRequest webLinkForText:'class categories' service:self selector:#'classCategoryList:') , '
<P>

<h3>By selector</h3>
<P>
' , (aRequest webLinkForText:'all selectors' service:self selector:#'selectorList:') , '
<BR>
'.

    $a to: $z do:[:ch |
        aRequest
            reply:
(aRequest webLinkForText:(ch asString) service:self selector:#'selectorListForRequest:prefix:' argument:ch asString) , ' '
    ].

    aRequest
        reply:'
<BR>
' , (aRequest webLinkForText:'others' service:self selector:#'selectorListForRequest:matching:' argument:'[!!-@]*') , '
<P>
'.
].

    false ifTrue:[
        aRequest reply:'<p>'.
        aRequest reply:'<p>'.
        aRequest reply:'<H4>Statistics</H4>'.
        aRequest reply:'<TABLE>'.
        aRequest reply:'<TR>'.
        aRequest reply:'<TD>Classes:</TD>'.
        aRequest reply:'<TD>' , (Metaclass allSubInstances size printString) , '</TD>'.
        aRequest reply:'</TR>'.
        aRequest reply:'<TR>'.
        aRequest reply:'<TD>Methods:</TD>'.
        aRequest reply:'<TD>' , (Method allSubInstances size printString) , '</TD>'.
        aRequest reply:'</TR>'.
        aRequest reply:'</TABLE>'.
    ].

    aRequest response maxAge:7 days.

    "Modified: / 02-02-2007 / 12:20:40 / cg"
!

classDocOf:aRequest
    "The class-by prefix character list page.
     generate an HTML document string which contains HREFS for a list
     of classes which start with some prefix (typically, the first
     character is given)"

    |className class title text generator replyPresenter|

    className := aRequest arguments asStringCollection asStringWith:'::'.
    className isNil ifTrue:[
        aRequest reportNotFound:'Missing Parameter'.
        "/ not reached
        ^ self
    ].

    class := Smalltalk classNamed:className.
    class isNil ifTrue:[
        aRequest reportNotFound:'No such class'.
        "/ not reached
        ^ self
    ].

    (self packageIsShown:class package) ifFalse:[
        "do not show documentation of non-public packages"
        aRequest reportNotFound:'No access to class'.
        "/ not reached
        ^ self
    ].
    
    generator := HTMLDocGenerator new.
    generator httpRequest:aRequest.
    generator generateBodyOnly:true.

    text := generator 
                htmlDocOf:class 
                back:nil 
                backRef:(aRequest serviceLinkName , '/classListForPrefix:,', class name first asUppercase) 
                imagePath:'/doc/online/icons'.

    title := 'Documentation of class ''' , className , ''''.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:title.
    replyPresenter secondHeadLine:(title , ':').
    aRequest reply:text.

    aRequest response maxAge:1 days.

    "Modified: / 02-02-2007 / 12:20:46 / cg"
!

classListForPackage:aRequest
    "The class-by package list page.
     generate an HTML document string which contains HREFS for a list
     of classes which are part of a package."

    |classesInpackageOrSubPackage classes packages subPackages packagePrefix classNames s replyPresenter|

    packagePrefix := aRequest arguments asStringWith:$/.

    classesInpackageOrSubPackage := Smalltalk allClasses
                select:[:cls | 
                                cls isPrivate not
                                and:[(cls isRealNameSpace not)
                                and:[(cls package = packagePrefix 
                                      or:[ (cls package startsWith:packagePrefix,'/') ])
                                and:[self exportedModules includes:(cls package upToAny:':/')]]]
                       ].

    classes := classesInpackageOrSubPackage select:[:cls | cls package = packagePrefix ].
    classNames := (classes collect:[:cls | cls name]) asSortedCollection.

    packages := (classesInpackageOrSubPackage collect:[:cls | cls package]) asSortedCollection.
    subPackages := packages select:[:p | p ~= packagePrefix].
    subPackages sort.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:('Classes in package ''' , packagePrefix asString , ''':').
    replyPresenter secondHeadLine:('Classes in package ''' , packagePrefix asString , ''':').
    aRequest
        reply:''.

    s := '' writeStream.


    s nextPutAll:(aRequest webLinkForImage:'/doc/online/icons/DocsUpArrow.gif' service:self selector:#'classDocIndex:').
    s nextPutAll:'<HR><UL>'.

    classNames do:[:nm |
        s nextPutLine:(aRequest webLinkForText:nm service:self selector:#'classDocOf:' argument:nm).
        s nextPutLine:'<BR>'.
    ].

    subPackages notEmpty ifTrue:[
        s nextPutAll:'<h3>'.
        s nextPutAll:'Subpackages:'.
        s nextPutAll:'</h3>'.
        subPackages do:[:p |
            s nextPutLine:(aRequest webLinkForText:p service:self selector:#'classListForPackage:' argument:(HTMLUtilities escape:p)).
            s nextPutLine:'<BR>'.
        ].
    ].

    s nextPutAll:'</UL>'.

    aRequest reply:s contents.

    aRequest response maxAge:7 days.   
!

classListForPrefix:aRequest
    "The class-by prefix character list page.
     generate an HTML document string which contains HREFS for a list
     of classes which start with some prefix (typically, the first
     character is given)"

    |classes prefix classNames s replyPresenter|

    prefix := aRequest arguments first.

    classes := Smalltalk allClasses
                select:[:cls | 
                                cls isPrivate not
                                and:[(cls isRealNameSpace not)
                                and:[(cls name startsWith:prefix)
                                and:[self exportedModules includes:(cls package upToAny:':/')]]]
                       ].

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:('Classes starting with ''' , prefix asString , ''':').
    replyPresenter secondHeadLine:('Classes starting with ''' , prefix asString , ''':').
    aRequest
        reply:''.

    classNames := (classes collect:[:cls | cls name]) asSortedCollection.

    s := '' writeStream.


    s nextPutAll:(aRequest webLinkForImage:'/doc/online/icons/DocsUpArrow.gif' service:self selector:#'classDocIndex:').
    s nextPutAll:'<HR><UL>'.

    classNames do:[:nm |
        s nextPutLine:(aRequest webLinkForText:nm service:self selector:#'classDocOf:' argument:nm).
        s nextPutLine:'<BR>'.
    ].

    s nextPutAll:'</UL>'.

    aRequest reply:s contents.

    aRequest response maxAge:7 days.

    "Created: / 23-04-1996 / 15:31:55 / cg"
    "Modified: / 02-02-2007 / 12:20:51 / cg"
!

packageIndex: aRequest
    "generates the package index page."

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Package Index'.
    replyPresenter secondHeadLine:'Package Index'.

    aRequest
        reply:'<h3>Packages:</h3>'.

    Smalltalk allProjectIDs
        select:[:p | self packageIsShown:p]
        thenDo:[:p |
            aRequest
                reply:
(aRequest webLinkForText:p service:self selector:#'classListForPackage:' argument:p).
            aRequest
                reply:'<br>'.
        ].

    aRequest response maxAge:7 days.
! !

!HTTPClassDocService methodsFor:'settings persistency'!

settingSelectors
    "answer the selectors that are used to configure a service - i.e. those which are
     saved in the webServer-settings file."

    "/ cg: changed to use a positive list, instead of an exclusion algorithm.
    "/ because it makes always trouble when an instvar is added.

    ^ super settingSelectors ,
      #(rootDirectory exportedModules classListExpirationTime classDocExpirationTime).

    "
     HTTPService new settingSelectors
     self new settingSelectors
    "
! !

!HTTPClassDocService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPClassDocService.st,v 1.62 2009/07/10 11:17:40 cg Exp $'
! !
