"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPConnection
	instanceVariableNames:'socket lastRequestTime httpServer keepConnection logStream
		serveProcess closeConnection nextRequestData startReadRequestTime
		startWaitRequestTime endReplyTime isRequestPending requestCount
		overallNumberOfRequests'
	classVariableNames:'DefaultConnectionProcessStackSize'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!HTTPConnection class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!HTTPConnection class methodsFor:'accessing'!

defaultConnectionProcessStackSize
    ^ DefaultConnectionProcessStackSize
!

defaultConnectionProcessStackSize:something
    DefaultConnectionProcessStackSize := something.

    "
     HTTPConnection defaultConnectionProcessStackSize:2*1024*1024
    "
! !

!HTTPConnection class methodsFor:'defaults'!

idleTimeoutToCloseConnection
    "the number of idle seconds, after which a kept connection is closed"

    ^ 20

    "Created: / 19-03-2007 / 10:07:45 / cg"
!

maxRequestHeaderSize
    "return the maximum size of a requests header"

    ^ 1024 * 32

    "Created: / 19-03-2007 / 10:06:27 / cg"
!

requestTimeOut
    "return the maximum time in secs until a request times out"

    ^ 10 * 60 "/ ten minutes
!

requestTransmitTimeOut
    "return the maximum time in secs until a request must be transmitted"

    ^ 60 "/ seconds
! !

!HTTPConnection methodsFor:'accessing'!

httpServer
    ^ httpServer
!

httpServer:something
    httpServer := something.
!

keepConnection:something
    keepConnection := something.
!

lastRequestTime
    ^ lastRequestTime
!

lastRequestTime:something
    lastRequestTime := something.
!

serveProcess
    ^ serveProcess
!

socket
    ^ socket
!

socket:something
    socket := something.
!

startReadRequestTime
    ^ startReadRequestTime
!

verbose
    "return the verbose messages flag from the server"

    ^ httpServer verbose
!

verboseData
    "return the verbose-data flag from the server"

    ^ httpServer verboseData
!

verboseErrors
    "return the verbose errors flag from the server"

    ^ httpServer verboseErrors
! !

!HTTPConnection methodsFor:'closing'!

close
    "force an immediate close and terminate the process"

    self closeSocket.
    serveProcess notNil ifTrue:[
        serveProcess terminate.
    ].
!

closeSocket
    "close down the connection"

    |aSocket|

    aSocket := socket.
    aSocket notNil ifTrue:[
        socket := nil.
        nextRequestData := ''.
"/        aSocket flush. -- not needed; not buffered
        aSocket shutdown:1.  "/ ensure, sent data is transmitted
        aSocket close.
    ].
!

scheduleForClose
    "remember to close the connection after processing"

    (httpServer verbose or:[httpServer verboseErrors]) ifTrue:[
        socket notNil ifTrue:[
            Error handle:[:ex |
                Error handle:[:ex | ]
                do:[
                    Transcript showCR:socket address asString .
                ]
            ] do:[
                Transcript show:Timestamp now printString, 'scheduleForClose: '.
                Transcript showCR:socket filePointer address asString .
            ]
        ].
    ].

    keepConnection := false.
    closeConnection := true.
! !

!HTTPConnection methodsFor:'logging'!

debugError:msg
    "debug-report some error condition to the Transcript"

    (httpServer verbose or:[httpServer verboseErrors]) ifTrue:[
        Transcript showCR:(self printString, '[error]: ' , msg).
    ].
!

debugInfo:msg
    "debug-report some trace information to the Transcript"

    httpServer verbose ifTrue:[
        Transcript showCR:(self printString, '[info]: ' , msg).
    ].
!

flushLog
    "flush our buffered log"

    logStream notEmptyOrNil ifTrue:[
        httpServer log:logStream contents.
        logStream reset.
    ].
!

logAccess:request
    |logger|

    logger := httpServer accessLogger.
    (logger notNil and:[logStream notNil]) ifTrue:[
"/        MessageTally spyDetailedOn:[
"/            100 timesRepeat:[
                logger logAccessFrom:request to:logStream
"/            ].
"/        ].
    ].
!

logError: aString
    "Add to the log."

    self logString:aString
!

logReply: aString
    "Add to the log."

    self logString:aString
!

logStream
    ^ logStream
!

logStream: aStream
    "set the log stream"

    logStream := aStream
!

logString: aString
    "Add to the log."

"/    (aString includes:(Character value:0)) ifTrue:[self halt].

    logStream notNil ifTrue:[
        logStream nextPut:$#;
            space;
            nextPutAll:aString;
            cr.
    ]
!

logWarning:aString
    "Add to the log."

    self logString:aString
! !

!HTTPConnection methodsFor:'printing'!

printOn:aStream
    self className printOn:aStream.
    socket notNil ifTrue:[
        aStream nextPutAll:' ('.
        socket getPeer printOn:aStream.
        aStream nextPutAll:' fd='.
        socket fileDescriptor printOn:aStream.
"/        aStream nextPutAll:' n='.
"/        requestCount printOn:aStream.
        aStream nextPut:$).
    ].

    "Modified: / 31-08-2007 / 15:38:14 / cg"
! !

!HTTPConnection methodsFor:'queries'!

overallNumberOfRequests
    "the number of served requests in this connection"
    
    ^ overallNumberOfRequests
!

requestTimeToPerform
    <resource: #obsolete>

    "/ what a stupid name
    self obsoleteMethodWarning:'use timeToPerformRequest'.
    ^ self timeToPerformRequest
!

timeInRequest
    "the time we have spent in the current request
     Or the last one, if finished.
     In milliseconds or nil"
    
    (startReadRequestTime isNil) ifTrue:[
        ^ nil
    ].
    ^ (endReplyTime ? Timestamp now) millisecondDeltaFrom:startReadRequestTime.
!

timeToPerformRequest
    "the time it took to perform the previous request.
     In milliseconds or nil"
    
    (startReadRequestTime isNil or:[endReplyTime isNil]) ifTrue:[
        ^ nil
    ].
    ^ endReplyTime millisecondDeltaFrom:startReadRequestTime.
! !

!HTTPConnection methodsFor:'serving-processing'!

getContentLengthFrom:requestString
    |contentLengthString contentLengthStartIndex contentLengthEndIndex contentLength s|

    contentLengthString  := 'content-length:'.
    contentLengthStartIndex := requestString asLowercase findString:contentLengthString startingAt: 1. 
    contentLengthStartIndex ~~ 0 ifTrue:[
        contentLengthEndIndex := requestString indexOf:(Character cr) startingAt:contentLengthStartIndex.
        contentLengthEndIndex ~~ 0 ifTrue:[
            s := requestString copyFrom:(contentLengthStartIndex+(contentLengthString size)) to:contentLengthEndIndex.
            contentLength := s withoutSeparators asNumber.
        ]
    ].
    ^ contentLength


!

process
    "serve the connection (as a new process)"

    |maxPrio serveBlock priorityRange oldMax|

    overallNumberOfRequests := 0.

    socket sendBufferSize:16r3FFFF.  "256 Kb"
    socket setTCPNoDelay:true.  

    serveBlock := 
        [
            [
                self serve.
            ] ensure:[
                self flushLog.
                socket notNil ifTrue:[
                    self debugInfo:'regular close'.
                    self closeSocket.
                ].
                httpServer removeConnection:self.
            ]
        ].

    DefaultConnectionProcessStackSize notNil ifTrue:[
        oldMax := Process defaultMaximumStackSize:DefaultConnectionProcessStackSize.
        [
            serveProcess := serveBlock newProcess
        ] ensure:[
            Process defaultMaximumStackSize:oldMax.
        ].
    ] ifFalse:[
        serveProcess := serveBlock newProcess.
    ].

"/ cg: it should be possible, to change the stackSize until the process has really started...
"/    DefaultConnectionProcessStackSize notNil ifTrue:[
"/        serveProcess setMaximumStackSize:DefaultConnectionProcessStackSize
"/    ].
"/
    serveProcess name:self printString.
    serveProcess priority:Processor activePriority - 1.
    maxPrio := (priorityRange := httpServer serverProcess priorityRange) notNil 
                    ifTrue:[ priorityRange max ] 
                    ifFalse:[ httpServer serverProcess priority ].
    serveProcess priorityRange:(Processor systemBackgroundPriority to:(maxPrio - 1)).
    self debugInfo:'new connection'.

    serveProcess resume.

    "Modified: / 31-08-2007 / 15:34:36 / cg"
!

readQueryDataForPostMessageFrom:header

    |contentLength queryData remainingString|

    contentLength := self getContentLengthFrom:header.
    contentLength isNil ifTrue:[
        "some browsers apparently don't send content-length when user hits back button"
"/        self error: 'missing content-length in header' 
        ^ ''
    ].

    nextRequestData size < contentLength ifTrue:[
        "/ need to read more data
        remainingString := self readUpToEndOrSize:contentLength - nextRequestData size.
        remainingString isNil ifTrue:[
            nextRequestData := ''.
            ^ nil
        ].
        queryData := nextRequestData , remainingString.
        nextRequestData := ''.
    ] ifFalse:[
        "/ the whole request has already been read
        queryData := nextRequestData copyTo:contentLength.
        "/ if client does pipelining, parts of the next request may have already been read
        nextRequestData := nextRequestData copyFrom:contentLength+1.
    ].
    ^ queryData.
!

readUpToEndOrSize:nMax
    |buffer nRemain input nBytesRead bufferSize|

    bufferSize := nMax min:4096.
    buffer := String uninitializedNew:bufferSize.
    nRemain := nMax.
    input := WriteStream on:''.

    socket class readErrorSignal handle:[:ex |
        self debugError:'shutdown connection due to readError'.
        ^ nil.
    ] do:[
        [nRemain ~~ 0] whileTrue:[
            (socket readWaitWithTimeout:10) ifTrue:[
                "got timeout"
                nBytesRead := 0.
                self shouldTimeout ifTrue:[
                    "bad guy was lazy feeding me with data"
                    httpServer temporaryForbid:socket getPeer.
                    self debugError:'shutdown connection because bad guy was lazy feeding me with data'.
                    ^ nil.
                ].
            ] ifFalse:[
                nBytesRead := socket nextAvailableBytes:(nRemain min:bufferSize) into:buffer startingAt:1.
                nBytesRead == 0 ifTrue:[
                    "socket was closed by client (readWait returned ok && nAvail==0)"
                    self debugError:'EOF - connection closed by other side without valid request'.
                    ^ nil.
                ].
                nBytesRead = bufferSize ifTrue:[
                    input nextPutAll:buffer
                ] ifFalse:[
                    input nextPutAll:(buffer copyTo:nBytesRead)
                ].
                nRemain := nRemain - nBytesRead.
            ].
        ].
        ^ input contents
    ].
!

serve
    "main loop on a connection:
        - read request
        - generate reply

    when leaving the loop, the connection will be closed by our sender"

    |request|

    keepConnection := true.
    [
        |endReadRequestTime verboseTiming|

        [closeConnection not 
          and:[keepConnection == true
          and:[httpServer isServing
"/          and:[socket notNil and:[socket isOpen and:[socket atEnd not]]]
        ]]] whileTrue:[
            verboseTiming := httpServer verboseTiming.

            keepConnection := nil.
            startWaitRequestTime := Timestamp now.

            request := self waitForAndFetchRequest.
            request isNil ifTrue:[
                "error reading request - terminate loop"
                ^ self
            ].
            httpServer countRequest.  "/ statistics only
            overallNumberOfRequests := overallNumberOfRequests + 1.

            verboseTiming ifTrue:[
                endReadRequestTime := Timestamp now
            ].

            self setKeepConnectionForRequest:request.
            request generateReply.

            verboseTiming ifTrue:[
                endReplyTime := Timestamp now.
            ].
            self flushLog.

            verboseTiming ifTrue:[
                Transcript showCR:('wait time: %1 ms' 
                            bindWith:(startReadRequestTime millisecondDeltaFrom:startWaitRequestTime)).
                Transcript showCR:('read request for %1 took %2 ms' bindWith:request url
                            with:(endReadRequestTime millisecondDeltaFrom:startReadRequestTime)).
                Transcript showCR:('reply for %1 took %2 ms' bindWith:request url
                            with:(endReplyTime millisecondDeltaFrom:endReadRequestTime)).
                Transcript showCR:'-----------------' 
            ].
            endReadRequestTime := nil.
            startWaitRequestTime := nil.
            startReadRequestTime := nil.
            endReplyTime := nil.
            request := nil.
        ].
    ] on:(Error, HaltInterrupt, AbortAllOperationRequest, Warning) do:[:ex | 
        |signal|

        signal := ex signal.           
        (signal == HTTPServer abortRequestProcessing) ifTrue:[
            "if this exception is raised after the request has been completed 
             (e.g. after error reporting), the socket should be kept open.
             But AbortOperationRequest should close the socket"
            (keepConnection == true 
             and:[socket notNil and:[socket isOpen and:[socket gotErrorOrEOF not and:[httpServer isServing]]]]) ifTrue:[
                ex restart.
            ].
        ] ifFalse:[(Socket writeErrorSignal accepts:signal) ifTrue:[
            self debugError:'client closed connection while sending reply'.
        ] ifFalse:[(Socket readErrorSignal accepts:signal) ifTrue:[
            self debugError:'shutdown connection due to readError'.
        ] ifFalse:[(Socket streamIOErrorSignal accepts:signal) ifTrue:[
            self debugError:('socket error ' , ex description).
        ] ifFalse:[
            (HaltInterrupt accepts:signal) ifTrue:[
                httpServer standaloneMode ifTrue:[
                    "ignore halts in standalone apps"
                    ex proceed.
                ] ifFalse:[
                    ex reject.
                ].
            ].
            httpServer debugErrors ifTrue:[
                "user wants to get a debugger on error/warning"
                self openDebuggerOnException:ex.
            ].
            (Warning accepts:signal) ifTrue:[
                self logWarning:ex description.
                ex proceed.
            ].


            "be careful when presenting the error to the connected browser;
             especially be smart w.r.t. more errors."

            request notNil ifTrue:[
                [
                    "report exception. Use the possibly active replyPresenter"
                    request reportException:ex.
                    "not reached"
                ] on:Error do:[:ex | 
                    httpServer debugErrors ifTrue:[
                        "debugger requested on error"
                        self openDebuggerOnException:ex.
                    ].
                    request response replyPresenter notNil ifTrue:[
                        "retry without replyPresenter"
                        request response replyPresenter:nil.
                        ex restart.
                    ].
                    "could not report error, so close the connection..."
                ].
            ]
        ]]]].
        self debugError:'shutDown connection due to error'.
    ].
!

setKeepConnectionForRequest:aRequest 
    "handle keepConnection as specified in aRequest.
     Notice: this is invoked BEFORE the request is actually handled - if a service
     detects a bad guy or intruder, it may still choose to invoke keepConnection:false"
    
    |vsn attrs|

    keepConnection notNil ifTrue:[
        "already set - maybe due to a request-read timeout"
        ^ self
    ].
    httpServer maxNumberOfKeepAliveConnections == 0 ifTrue:[
        keepConnection := false.
        ^ self.
    ]. 

    "HTTP1.1 clients assume the connection remains open - if not specified otherwise.
     older versions assume it is closed by default."

    attrs := aRequest connectionAttributes.
    attrs notNil ifTrue:[
        attrs := attrs asCollectionOfWords.
        (attrs includes:'keep-alive') ifTrue:[
            keepConnection := true
        ] ifFalse:[
            (attrs includes:'close') ifTrue:[
                keepConnection := false
            ].
        ]
    ].
    keepConnection isNil ifTrue:[
        vsn := aRequest httpVersion.
        vsn isEmptyOrNil ifTrue:[
            "no valid header - do not keep the connection"
            self debugError:'bad/missing HTTP version'.
            keepConnection := false.
        ] ifFalse:[
            keepConnection := (vsn >= '1.1').
        ]
    ].
!

waitForAndFetchRequest
    "Read the request and return an HTTPRequest instance, 
     or nil on any error."

    |headerEndIndex header queryData bufferSize buffer nBytesRead rewriteUrl
     httpRequest requestString crlfcrlf rawRequestString lastReqEndIndex
     startTimeForIdleTimeout|

    "/ Transcript showCR:'------------------------------------'.
    startTimeForIdleTimeout := lastRequestTime.
    startTimeForIdleTimeout isNil ifTrue:[ startTimeForIdleTimeout := Timestamp now].

    crlfcrlf := HTTPServer crlfcrlf.
    bufferSize := 1024*4.
    buffer := String uninitializedNew:bufferSize.
    requestString := ''.
    lastReqEndIndex := 1.

    [
        nBytesRead := nextRequestData size.   "/ already read with last request ?
        nBytesRead == 0 ifTrue:[
            (socket readWaitWithTimeout:30) ifTrue:[
                "timeout"
                self debugInfo:'socket read timeout'.
                (Timestamp now getSeconds - startTimeForIdleTimeout getSeconds) > self class idleTimeoutToCloseConnection ifTrue:[
                    closeConnection := true.
                ].
                closeConnection ifTrue:[
                    "the server wants this cached connection to be released.
                     Notice, that the closeConnection flag could also be set from elsewhere"
                    ^ nil
                ].
                (requestString notEmpty and:[self shouldTimeout]) ifTrue:[
                    "bad guy was lazy feeding me with data"
                    closeConnection := true.
                    httpServer temporaryForbid:socket getPeer.
                    self debugError:'shutdown connection because bad guy was lazy feeding me with data'.
                    ^ nil.
                ].
            ] ifFalse:[                
                nBytesRead := socket nextAvailableBytes:bufferSize into:buffer startingAt:1.
                nBytesRead == 0 ifTrue:[
                    closeConnection := true.
                    "/ socket was closed by client (readWait returned ok && nAvail==0)
                    self debugError:('EOF - connection closed by peer without valid request after <1p>s idle time'
                                     expandMacrosWith:(Timestamp now - startWaitRequestTime)).
                    ^ nil.
                ].
                startTimeForIdleTimeout := startReadRequestTime := Timestamp now.
            ].
        ].

        httpServer isServing ifFalse:[
            self debugInfo:'shutdown connection because server went offline'.
            ^ nil.
        ].

        "/ Transcript showCR:'nRead: ',nBytesRead printString.
        nBytesRead ~~ 0 ifTrue:[
            startReadRequestTime isNil ifTrue:[
                startReadRequestTime := Timestamp now.
            ].

            nextRequestData isEmptyOrNil ifTrue:[
                requestString := requestString , (buffer copyTo:nBytesRead).
            ] ifFalse:[
                self assert:(requestString size == 0).
                requestString := nextRequestData.
                nextRequestData := ''.
            ].
            headerEndIndex := requestString findString:crlfcrlf startingAt:lastReqEndIndex.
            "/ Transcript showCR:'found CRLFCRLF at: ',headerEndIndex printString.
            lastReqEndIndex := requestString size - 3.
            lastReqEndIndex < 1 ifTrue:[lastReqEndIndex := 1].
            (lastReqEndIndex > self class maxRequestHeaderSize) ifTrue:[
                self debugError:'shutdown connection because request is longer than maxHeaderSize (32k)'.
                ^ nil
            ].
            headerEndIndex == 0 ifTrue:[    
                ((Timestamp now - startReadRequestTime) > self class requestTransmitTimeOut) ifTrue:[
                    "bad guy was lazy feeding me with data"
                    closeConnection := true.
                    httpServer temporaryForbid:socket getPeer.
                    self debugError:'shutdown connection because bad guy was lazy feeding me with data'.
                    ^ nil.
                ].
            ].
        ] ifFalse:[
            headerEndIndex := 0
        ].
        headerEndIndex == 0 
    ] whileTrue.

    header := requestString copyFrom:1 to:headerEndIndex + crlfcrlf size - 1.
    nextRequestData := requestString copyFrom:headerEndIndex + crlfcrlf size.
    rawRequestString := header.

    (header startsWith:String crlf) ifTrue:[
        header := header copyFrom:3.
    ].

    "/ Transcript showCR:'Request: ',header.
    self verbose ifTrue:[
        Transcript showCR:'<-- HTTP Request (fd=', socket fileDescriptor printString, ') --'.
        Transcript show:(header copyReplaceAll:Character return with:Character space).
    ].

    "/ read query data
    (header startsWith: 'POST') ifTrue: [
        queryData := self readQueryDataForPostMessageFrom:header.
        queryData isNil ifTrue:[
            ^ nil.
        ].
        rawRequestString := rawRequestString,  queryData.

        httpServer verboseData ifTrue:[
            Transcript showCR:'.....................'.
            Transcript showCR:(queryData printString)
        ].
    ].

    self verbose ifTrue:[
        Transcript showCR:'<------------------'.
    ].

    lastRequestTime := Timestamp now.

    httpRequest := HTTPRequest server:httpServer connection:self.
    httpRequest 
        rawRequestString:rawRequestString
        rawHeader:header
        data:queryData.

    "use alias list from server to change url "
    rewriteUrl := httpServer rewriteURLFor:httpRequest url.
    rewriteUrl notNil ifTrue:[
        httpRequest url:rewriteUrl
    ].
    requestCount := (requestCount ? 0) + 1.
    ^ httpRequest.

    "Modified: / 19-03-2007 / 10:29:05 / cg"
! !

!HTTPConnection methodsFor:'startup & release'!

initializeFor:aHTTPServer socket:aSocket
    "initialize a connection served by aHTTPServer on aSocket"

    "let the system check if a socket is still connected"
    aSocket setOption:#SO_KEEPALIVE value:true.

    nextRequestData := ''.
    socket := aSocket.
    httpServer := aHTTPServer.
    closeConnection := false.
    requestCount := 0.

    "Modified: / 19-03-2007 / 10:14:04 / cg"
! !

!HTTPConnection methodsFor:'test support'!

quickSendTestResponse
    "used only for tests"

    socket 
        nextPutAll:('HTTP/1.1 200 OK' , String crlf , 'Content-Type: text/plain' 
                , String crlf , 'Content-Length: 10' 
                , String crlf , 'Connection: Keep-Alive' 
                , String crlf , 'Date: Tue, 28 Oct 2003 13:03:43 GMT' 
                , String crlf , 'Cache-Control: public' 
                , String crlf , 'Expires: Tue, 28 Oct 2003 14:03:40 GMT' 
                , String crlf , String crlf 
                , 'aaaaaaaaaa').
! !

!HTTPConnection methodsFor:'testing'!

isKeepAlive
    ^ keepConnection == true
!

isOpen
    ^ socket notNil and:[socket isOpen]
!

shouldTimeout
    "answer true, if the currently pending request should timeout"

    ^ (Timestamp now - startWaitRequestTime) > self class requestTimeOut
! !

!HTTPConnection class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPConnection.st,v 1.28 2010/09/01 16:51:11 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPConnection.st,v 1.28 2010/09/01 16:51:11 stefan Exp $'
! !
