"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPSelectorService subclass:#HTTPEmbeddedApplicationService
	instanceVariableNames:'registeredApplicationClasses embeddedSmalltalkSupportDirectory
		registerSema useWeirdX'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

!HTTPEmbeddedApplicationService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
   embedded applications setup:
   ----------------------------

        Applications must be registered (with an optional userName)
        in order for embedding to be allowed.
        Standard embedding is supported via the URL Server.embed.<appName>,
        however, an application may generate its own private embed page
        (have a look at self embed: and/or startApplication: for details)

        Registration is done by:
            self registerApplication:appClass
        or:
            self registerApplication:appClass forUser:userName

        The first allows any user to open the embedded application,
        the later requires that a user mapping (name+password) has been
        setup earlier and will lead to an authorization dialog to open
        in the www-broeser.

        For example, to only allow a user named 'TestUser' with password 'foo'
        to invoke embedded SystemBrowsers, you need the following procedure:

            aHTTPServer registerApplication:SystemBrowser forUser:'BrowserUser'.
            aHTTPServer mapName:'TestUser' password:'foo' to:'BrowserUser'.

        Then, the SystemBrowser can be opened in a page, by entering the URL
        Server.embed.SystemBrowser (usually, this is included in a generated
        page).

    [author:]
        Claus Gittinger
"
! !

!HTTPEmbeddedApplicationService class methodsFor:'configuring - setup'!

embeddedSmalltalkPackageFilename
    ^ (Smalltalk getPackageFileName:(self embeddedSmalltalkPackageId copyReplaceAll:$: with:$/))

    "
     self embeddedSmalltalkPackageFilename
    "
!

embeddedSmalltalkPackageId
    ^ 'exept:www/embeddedSmalltalk'.
! !

!HTTPEmbeddedApplicationService class methodsFor:'defaults'!

defaultEbeddedAppClasses

    ^ #(
        #MemoryMonitor
        #Workspace
        #'Demos::PenDemo'
        #'Demos::FractalPlantsDemo'
        #'Demos::FractalPatternsDemo'
        #'Demos::LSystemsDemo'
        #'CodingExamples_GUI::GUIDemo'
        #SystemBrowser
        #FileBrowser
        #MemoryMonitor
        #ChangesBrowser
    )
!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
!

linkName
    "return the default linkName path (with slash)"

    ^ '/Embed'
!

noRealmBehavior
    "allow access ONLY from localHost, if no realm was specified for this service"

    ^ HTTPService::NoRealmBehavior localHostOnly  
! !

!HTTPEmbeddedApplicationService class methodsFor:'queries-service'!

settingsApplicationClass
    "a SettingsApplication class - or nil (used in the settings dialog if non-nil)"

    ^ nil
! !

!HTTPEmbeddedApplicationService methodsFor:'accessing'!

embeddedSmalltalkSupportDirectory
    |supportDir|

    embeddedSmalltalkSupportDirectory isNil ifTrue:[
        supportDir := self class embeddedSmalltalkPackageFilename asFilename.
        supportDir notNil ifTrue:[
            (supportDir construct:'classFiles') exists ifTrue:[
                supportDir := supportDir construct:'classFiles'.
            ].
            embeddedSmalltalkSupportDirectory := supportDir pathName.
        ]
    ].

    ^ embeddedSmalltalkSupportDirectory

    "
     self new embeddedSmalltalkSupportDirectory
    "
!

embeddedSmalltalkSupportDirectory:something
    embeddedSmalltalkSupportDirectory := something.
!

registeredApplicationClasses
    registeredApplicationClasses isNil ifTrue:[
        registeredApplicationClasses := IdentityDictionary new.
    ].
    ^ registeredApplicationClasses
!

registeredApplicationClasses:something
    registeredApplicationClasses := something.
! !

!HTTPEmbeddedApplicationService methodsFor:'configuring - setup'!

setupEmbeddedSmalltalkSupportClasses
    "ensure, RemoteWorkstation and required classFiles are present for java applet"

    |supportDir|

    supportDir := self class embeddedSmalltalkPackageFilename asFilename.
    supportDir notNil ifTrue:[
        RemoteWorkstation isNil ifTrue:[
            Error handle:[:ex |
                ^ self.
            ] do:[
                Class withoutUpdatingChangesDo:[
                    (supportDir construct:'RemoteWorkstation.st') fileIn.
                ].
            ].
        ].
        (supportDir construct:'classFiles') exists ifTrue:[
            supportDir := supportDir construct:'classFiles'.
        ].
        embeddedSmalltalkSupportDirectory := supportDir pathName.
        self class defaultEbeddedAppClasses do:[:aClassName|
            |class|
            class := Smalltalk at:aClassName.
            class notNil ifTrue:[
                self registerApplication:class forRealm:nil
            ].
        ].
    ].
! !

!HTTPEmbeddedApplicationService methodsFor:'links-registration'!

webLinkForApplication:anApplicationClass name:linkName request:aRequest
    |link|

    link := self webLinkForApplication:anApplicationClass request:aRequest.
    link isNil ifTrue:[
        ^  linkName,' [not available]'
    ].

    ^ '<a href="', link , '">',linkName,'</a>'
!

webLinkForApplication:applicationClassOrSymbol request:aRequest
    "Return a link which starts up an embedded smalltalk application."

    | applicationClass |
    applicationClass := applicationClassOrSymbol.
    applicationClass isClass ifFalse:[
        applicationClass := Smalltalk at:applicationClass.    
    ].
    applicationClass isNil ifTrue:[
        ^ nil
    ].
    (self registeredApplicationClasses keys includes:applicationClass name) ifFalse:[
        ^ nil
    ].

    ^ aRequest linkIdFor:self selector:#'getApplication:' argument:applicationClass name
! !

!HTTPEmbeddedApplicationService methodsFor:'queries'!

useWeirdX
    ^ useWeirdX ? true
!

useWeirdX:aBoolean
    useWeirdX := aBoolean
! !

!HTTPEmbeddedApplicationService methodsFor:'queries-service'!

allowedMessages
    ^ #(
        maintenanceRegisterdApplications: 
        getApplication: 
        registeredApplicationList:
        )
!

defaultMessage
    ^ #registeredApplicationList:
! !

!HTTPEmbeddedApplicationService methodsFor:'registration-apps'!

registerApplication:applicationClass
    "remember that applicationClass may be opened as application
     by anyone.
     (i.e. it is allowed to invoke an embed page)"

    self registerApplication:applicationClass forRealm:'Execute Embedded Smalltalk Applications'
!

registerApplication:applicationClass forRealm:aRealm
    "remember that applicationClass may be opened as application
     by aUserName. (i.e. it is only allowed to authorized users)"

    |realm appName|

    appName := applicationClass name.
    realm := self registeredApplicationClasses at:appName ifAbsent:nil.
    registerSema critical:[
        self registeredApplicationClasses at:appName put:aRealm.
    ].
!

unregisterApplication:applicationClass
    "forget that applicationClass may be opened as application
     (i.e. it is no longer allowed to invoke an embed page)"

    registerSema critical:[
        self registeredApplicationClasses removeKey:applicationClass name ifAbsent:nil
    ].
! !

!HTTPEmbeddedApplicationService methodsFor:'registration-service'!

addRequiredForeignServicesTo:aServer 
    "I need a file service for the bitmaps and icons"

    self addStandardFileServicesTo:aServer.
    (aServer serviceForLink:'/classFiles' ifAbsent:nil) isNil ifTrue:[
        aServer addFileServiceWithRootLinkName:'/classFiles' 
                    rootDirectory:(self embeddedSmalltalkSupportDirectory) 
                    defaultFileName:nil.
    ].
!

registerServiceOn:aServer 

    registerSema := Semaphore forMutualExclusion.
    self setupEmbeddedSmalltalkSupportClasses.
    super registerServiceOn:aServer.
!

removeRequiredForeignServicesFrom:aServer
    |fileServices|

    fileServices := aServer fileServices.
    fileServices := fileServices select:[:aFileService| (aFileService linkName = '/classFiles')].
    fileServices do:[:aFileService| aFileService unRegisterServiceOn:aServer].
! !

!HTTPEmbeddedApplicationService methodsFor:'requests-helpers'!

registerClass:registerClassName request:aRequest
    "register aClass and response if not successful"

    |registerClass answerTitle answerText replyPresenter|

    registerClassName notNil ifTrue:[
        registerClass := Smalltalk at:registerClassName asSymbol.
        registerClass isBehavior ifTrue:[
            (registerClass perform:#isVisualStartable ifNotUnderstood:false) ifTrue:[
                self registerApplication:registerClass.
                ^ true
            ] ifFalse:[
                answerTitle := 'Application is not visual'.
                answerText := '<H2>Application named ''' , registerClass className , ''' is not visual startable - NOT registered.</H2>'
            ]
        ] ifFalse:[
            answerTitle := 'No Such Application'.
            answerText := '<H2>Application named ''' , registerClass className, ''' found in the server - NOT registered.</H2>'
        ].
        answerTitle notNil ifTrue:[
            replyPresenter := aRequest replyPresenter.
            replyPresenter parametersAt:#beforeTitle put:(aRequest refreshStringFor:self selector:#'maintenanceRegisterdApplications:' time:2).
            replyPresenter title:answerTitle.
            replyPresenter parametersAt:#secondHeadLine put:'Register Applications'.
            aRequest 
                reply: answerText, '<BR>';
                reply:(aRequest webLinkForText:'[back]' service:self selector:#'registeredApplicationList:').
        ].        
    ].
    ^ false
!

startApplication:anApplicationClassOrName title:aTitle extent:extentOrNil for:aRequest
    "reply with an embedded stxApplet application;
     this is neat, the returned page will lead to an embedded applet
     to be loaded, which plays the application inside the browser page."

    |appClass|

    anApplicationClassOrName isBehavior ifFalse:[
        appClass := Smalltalk at:anApplicationClassOrName asSymbol ifAbsent:nil.    
    ] ifTrue:[
        appClass := anApplicationClassOrName
    ].

    appClass isNil ifTrue:[
        aRequest reportNotFound:'No class: ' , anApplicationClassOrName.
        "/ not reached
        ^ self
    ].
    (appClass isBehavior not or:[appClass isNameSpace]) ifTrue:[
        aRequest reportNotFound:'Bad class: ' , anApplicationClassOrName.
        "/ not reached
        ^ self
    ].

    useWeirdX ifFalse:[
        ^ self startRemoteWorkstationApplicationClass:appClass title:aTitle extent:extentOrNil for:aRequest
    ].

    ^ self startWeirdXApplicationClass:appClass title:aTitle extent:extentOrNil for:aRequest
!

startRemoteWorkstationApplicationClass:anApplicationClass title:aTitle extent:extentOrNil for:aRequest
    "reply with an embedded stxApplet application;
     this is neat, the returned page will lead to an embedded applet
     to be loaded, which plays the application inside the browser page."

    |portNr w h userAgent userAgentVersion useJAR|

    RemoteWorkstation isNil ifTrue:[
        aRequest reportForbidden:'Server is not configured to support embedded smalltalk applications<BR>(missing RemoteWorkstation class)'.
        "/ not reached
        ^ self
    ].

    portNr := RemoteWorkstation serveAtAnyPortIn:(9000 to:9099) applicationTimeout:30.
    portNr isNil ifTrue:[
        aRequest reportServerError:'Could not start the application server process' moreInfo:nil.
        "/ not reached
        ^ self
    ].
    w := h := 300.
    extentOrNil notNil ifTrue:[
        w := extentOrNil x.
        h := extentOrNil y.
    ].

    "/ does that client handle jar-files ?
    "/ thats heuristic for now, and should be changed.
    useJAR := false.
    ((userAgent := aRequest userAgent) startsWith:'Mozilla/') ifTrue:[
        userAgentVersion := userAgent copyFrom:'Mozilla/' size + 1.
        userAgentVersion := userAgentVersion copyTo:3.
        userAgentVersion := Number readFrom:userAgentVersion onError:nil.
        userAgentVersion notNil ifTrue:[
            userAgentVersion < 4.0 ifTrue:[
                aRequest reply:'
<CENTER><H1>Unsupported Browser version</H1></CENTER>
<BR>Sorry, you need a newer browser version to be able to embed Smalltalk
applications (the support applet needs at least Java 1.1).
<P>
Please <A HREF="http://www.netscape.com">upgrade</A> to a version 4.x browser.
'.
                ^ self.
            ].
            userAgentVersion >= 4.6 ifTrue:[
                useJAR := true.
            ]
        ]
    ].

    aRequest replyPresenter title:aTitle.
    aRequest reply:'
<CENTER><H1>' , aTitle , '</H1></CENTER>
<APPLET CODE="STXApplet.class" '.
    useJAR ifTrue:[
        aRequest reply:'archive="STXApplet.jar" '.
    ].
    aRequest reply:'CODEBASE="/classFiles" width=' , w printString , ' height=' , h printString , '>
<PARAM NAME="application" value="' , anApplicationClass name , '">
<PARAM NAME="port" value="' , portNr printString , '">
The applet requires Java.
<BR>You must have a Java enabled browser such as <A HREF="http://www.netscape.com">
Netscape Navigator</A> to be able to see this applet.
<BR>
If your browser does support Java, please make certain that it is enabled
in the browsers option settings.
</APPLET>
'.

    "Modified: / 25.9.1999 / 16:26:14 / cg"
!

startWeirdXApplicationClass:anApplicationClass title:aTitle extent:extentOrNil for:aRequest
    "reply with an embedded stxApplet application;
     this is neat, the returned page will lead to an embedded applet
     to be loaded, which plays the application inside the browser page."

    |portNr w h userAgent userAgentVersion useJAR|

    portNr := 2.

    w := h := 300.
    extentOrNil notNil ifTrue:[
        w := extentOrNil x.
        h := extentOrNil y.
    ].

    "/ does that client handle jar-files ?
    "/ thats heuristic for now, and should be changed.
    useJAR := false.
    ((userAgent := aRequest userAgent) startsWith:'Mozilla/') ifTrue:[
        userAgentVersion := userAgent copyFrom:'Mozilla/' size + 1.
        userAgentVersion := userAgentVersion copyTo:3.
        userAgentVersion := Number readFrom:userAgentVersion onError:nil.
        userAgentVersion notNil ifTrue:[
            userAgentVersion < 5.0 ifTrue:[
                aRequest reply:'
<center><h1>Unsupported Browser version</h1></center>
<br>Sorry, you need a newer browser version to be able to embed Smalltalk
applications (the support applet needs at least Java 1.3).
<p>
Please <a href="http://www.netscape.com">upgrade</a> to a version 4.x browser.
'.
                ^ self.
            ].
            userAgentVersion >= 4.6 ifTrue:[
                useJAR := true.
            ]
        ]
    ].

    aRequest replyPresenter title:aTitle.
    aRequest reply:'
<center><h1>' , aTitle , '</h1></center>
<applet code="com.jcraft.weirdx.WeirdX.class"
  archive="weirdx.jar" 
  codebase="/classFiles"
  width=' , w printString , ' height=' , h printString , 'align="baseline">
  <param name="application" value="' , anApplicationClass name , '">
  <PARAM NAME="cabbase" VALUE="weirdx-1.0.26-rsa.cab">
  <PARAM NAME="weirdx.display.width" VALUE="',w printString,'">
  <PARAM NAME="weirdx.display.height" VALUE="',h printString,'">
  <PARAM NAME="weirdx.displaynum" VALUE="',portNr printString,'">
  <PARAM NAME="weirdx.display.visual" VALUE="TrueColor16">
  <PARAM NAME="weirdx.display.acl" VALUE="+">
  <PARAM NAME="weirdx.display.threebutton" VALUE="yes">
  <PARAM NAME="weirdx.display.copypaste" VALUE="yes">

  <PARAM NAME="weirdx.windowmode" VALUE="InBrowser">
  <!!--
  <PARAM NAME="weirdx.windowmode" VALUE="Rootless">
  -->

  <!!-- 
  <PARAM NAME="weirdx.xdmcp.mode" VALUE="query">
  <PARAM NAME="weirdx.xdmcp.address" VALUE="192.168.0.1">
  -->
This applet requires Java.
<br>You must have a Java enabled browser such as <a href="http://www.netscape.com">
Netscape Navigator/Mozilla</a> to be able to see this applet.
<br>
If your browser does support Java, please make certain that it is enabled
in the browsers option settings.
</applet>
'.

    "Modified: / 25.9.1999 / 16:26:14 / cg"
! !

!HTTPEmbeddedApplicationService methodsFor:'requests-server'!

getApplication:aRequest 
    "embed something in the browsers view - wow !!"

    |embedWhat appClassName appClass realm|

    aRequest userAgent = 'Mozilla/3.01'ifTrue:[
        "/ that one does not support jar files ...
    ].
    aRequest arguments isEmptyOrNil ifTrue:[
        ^ self
    ].
    embedWhat := aRequest arguments first.
    embedWhat knownAsSymbol ifTrue:[
        appClassName := embedWhat asSymbol.
        appClass := Smalltalk at:appClassName ifAbsent:nil.
        appClass isNil ifTrue:[
                aRequest reportNotFound:'No such application class ', embedWhat.
        ].
"/        (appClass notNil and:[appClass isBehavior and:[appClass isNameSpace]]) ifTrue:[
"/            | ns |
"/            ns := embedWhat.
"/            self halt.
"/            embedWhat := aRequest halt "message" at:4 ifAbsent:nil.
"/            embedWhat isNil ifTrue:[
"/                "/ not reached
"/                ^ self
"/            ].
"/            embedWhat := ns , '::' , embedWhat
"/        ].
    ].

    embedWhat knownAsSymbol ifFalse:[
        aRequest reportNotFound:'No class: ' , embedWhat.
        "/ not reached
        ^ self
    ].
    appClassName := embedWhat asSymbol.

    ((realm := self registeredApplicationClasses at:appClassName ifAbsent:nil) notNil) ifTrue:[
        (aRequest authenticateForRealm:realm) ifFalse:[
            "/ not reached
            ^ self
        ].
    ].

    appClass := Smalltalk at:appClassName ifAbsent:nil.
    self
        startApplication:embedWhat 
        title:('Embedded ' , embedWhat)
        extent:640@500
        for:aRequest.
!

maintenanceRegisterdApplications:aRequest
    "add and remove applications from registeredClasses"

    | registeredClasses registerClass unRegisterClass replyPresenter|

    registerClass := aRequest postDataFieldAt:'RegisterClass'.
    unRegisterClass := aRequest postDataFieldAt:'UnRegisterClass'.
    registerClass size == 0 ifTrue:[ registerClass := nil].
    unRegisterClass size == 0 ifTrue:[ unRegisterClass := nil].
    unRegisterClass notNil ifTrue:[
        unRegisterClass := Smalltalk at:unRegisterClass asCollectionOfWords first asSymbol.
        self unregisterApplication:unRegisterClass.
    ].
    registerClass notNil ifTrue:[
        (self registerClass:registerClass request:aRequest) ifFalse:[
            ^ self.
        ].
    ].

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Add and Remove Registered Applications'.
    replyPresenter parametersAt:#secondHeadLine put:'Registered Applications'.

    registeredClasses := self registeredApplicationClasses keys.
    registeredClasses notEmpty ifTrue:[
        aRequest
            reply:'<BR><H2><span style="font-weight: bold ;">Registered Applications</span></H2>'.
        aRequest
            reply: '
<FORM action="', (aRequest linkIdFor:self selector:#'maintenanceRegisterdApplications:'),'" method=post>'.
        aRequest reply:'
    <SELECT NAME=UnRegisterClass SIZE=11>'.
        registeredClasses asSortedCollection do:[:appName |
            aRequest
                reply:'
        <Option>', appName.
        ].
        aRequest reply:'
</SELECT>';
            reply:'&nbsp;&nbsp;&nbsp;';
            formSubmitButtonForString:'Unregister';
            formEnd.
    ] ifFalse:[
        aRequest
            reply:'
<BR><BR><H2><span style="font-weight: bold ;">No registered Applications</span></H2>'.
    ].

    aRequest 
        reply: '<BR><BR><span style="font-weight: bold ;">Register an Application</span><BR><BR>';
        reply: '<FORM action="', (aRequest linkIdFor:self selector:#'maintenanceRegisterdApplications:'),'" method=post>';
        inputFieldFor:'RegisterClass' size:40 label:'Application' initialValue:'<className>';
        reply:'&nbsp;&nbsp;&nbsp;';
        formSubmitButtonForString:'Register';
        formEnd.
!

registeredApplicationList: aRequest
    "send a list of registered applications"

    | registeredClasses replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Registered Applications'.
    replyPresenter parametersAt:#secondHeadLine put:'Registered Applications'.

    registeredClasses := self registeredApplicationClasses keys.
    registeredClasses notEmpty ifTrue:[
        aRequest
            reply:'<BR><H2><span style="font-weight: bold ;">Registered Applications</span></H2>'.
        aRequest
            reply:'
<Table cellpadding="2" cellspacing="2" border="1">'.
        registeredClasses asSortedCollection do:[:appName |
            aRequest
                reply:'
        <TR><TD>', (self webLinkForApplication:appName name:appName request:aRequest), '</TR></TD>'.
        ].
        aRequest reply:'
</Table>'.
    ] ifFalse:[
        aRequest
            reply:'
<BR><BR><H2><span style="font-weight: bold ;">No registered Applications</span></H2>'.
    ].
    aRequest reply:'<BR><BR>', (aRequest webLinkForText:'Add and Remove Applications' service:self selector:#'maintenanceRegisterdApplications:').
! !

!HTTPEmbeddedApplicationService methodsFor:'settings persistency'!

settingSelectors
    "answer the selectors that are used to configure a service - i.e. those which are
     saved in the webServer-settings file."

    "/ cg: changed to use a positive list, instead of an exclusion algorithm.
    "/ because it makes always trouble when an instvar is added.

    ^ super settingSelectors ,
      #(registerApplicationClasses embeddedSmalltalkSupportDirectory useWeirdX).   

    "
     HTTPService new settingSelectors
     self new settingSelectors
    "
! !

!HTTPEmbeddedApplicationService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPEmbeddedApplicationService.st,v 1.44 2009/07/10 11:17:21 cg Exp $'
! !
