"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPRequest
	instanceVariableNames:'server connection peer peerName rawRequestString rawHeader method
		httpVersion rawUrl url postDataFields urlParameters contentLength
		contentType contentEncoding connectionAttributes authenticator
		acceptedTypes acceptedEncodings acceptedCharsets
		acceptedLanguages ifModifiedSince data postData cookies service
		serviceLinkName pathRelativeToService arguments selector
		multipartBoundary multiPartChunks queryString response
		requestLine headerParser serviceRequestContext'
	classVariableNames:'EscapeControlCharacters'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

Object subclass:#MultipartChunk
	instanceVariableNames:'request contentDisposition contentType data
		contentDispositionElements contentTypeElements'
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPRequest
!

Notification subclass:#ReprocessRequest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPRequest
!

!HTTPRequest class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    represents an HTTP request as received by the HTTPServer.


    [instance variables:]
        server              the server from which this request was received
        connection          the socket conection from which this request was received
        peer                the sender of this request
        peerName

        rawRequestString:   the whole request incl. any POST-data (required for com-swiki)
        rawHeader           the raw http-header (without data)
        headerDictionary    extracted fields
        headerLines         header lines
        method              ?
        httpVersion         httpVersion from the header
        url                 the requests original url
        postDataFields      POST request arguments
        urlParameters       &-parameters as extracted from the URL
        contentLength       contentLength from the header
        contentType         contentType from the header
        contentEncoding     contentEncoding from the header
        connectionAttributes     connectionAttributes from the header (Keep/close etc.)
        userAgent           userAgent from the header (IE, mozilla etc.)
        authorization       authorization from the header
        acceptedTypes       acceptedTypes from the header
        acceptedEncodings   acceptedEncodings from the header
        acceptedCharsets    acceptedCharsets from the header
        acceptedLanguages   acceptedLanguages from the header
        referer             referer from the header
        host                host from the header
        soapAction          soapAction from the header
        cookies             cookies from the header

        data                the POST-data alone
        postData            ?
        userId              ?
        serviceLinkName     ?
        service             ?
        relativePath        URL relative to service-linkName
        message             the original message; split and preparsed
        arguments           additional arguments for object-requests (preparsed from the url)
        selector            ?
        multipartBoundary   ?
        multiPartChunks     ?
        queryString         url parameters - rest after any in { & ; ? }
        response            response object

    [author:]
        Christian Penk

    [see also:]
        http://www.w3.org/Protocols/rfc2616/rfc2616.html
        HTTPResponse
        HTTPServer
        HTTPInterface
"
! !

!HTTPRequest class methodsFor:'instance creation'!

server:httpServer connection:connection
    ^ self basicNew initializeForServer:httpServer connection:connection
! !

!HTTPRequest class methodsFor:'defaults'!

allowedRequestMethods
    ^ #( 'GET' 'HEAD' 'POST').
! !

!HTTPRequest class methodsFor:'helpers'!

extractAcceptedFromLine:acceptLine
    |acceptedStrings accepted|

    acceptedStrings := acceptLine withoutSeparators asCollectionOfSubstringsSeparatedByAny:',; '.
    accepted := OrderedCollection new.
    acceptedStrings do:[:aString|
        (aString startsWith:'q=') ifTrue:[
            |weight|

            weight := (aString copyFrom:3) asNumber.
            accepted notEmpty ifTrue:[
                accepted last value:weight
            ].        
        ] ifFalse:[
            accepted add:(aString -> 1.0)
        ]
    ].

    ^ accepted.

    "
     self extractAccepted:'accept:' fromLine:'image/gif, image/x-xbitmap;q=0.2, image/jpeg, image/pjpeg, image/png, */*;q=0'


     self extractAccepted:'accept:' fromLine:'text/*;q=0.3, text/html;q=0.7, text/html;level=1, text/html;level=2;q=0.4, */*;q=0.5'
       should cause the following values to be associated: 

       text/html;level=1         = 1
       text/html                 = 0.7
       text/plain                = 0.3

       image/jpeg                = 0.5
       text/html;level=2         = 0.4
       text/html;level=3         = 0.7

    "
!

extractMultipartBoundaryFromContentType:contentTypeLine
    |idx  rest mbound|

    idx := contentTypeLine findString:'boundary='.
    idx == 0 ifTrue:[
        ^ nil
    ].
    rest := (contentTypeLine copyFrom:idx+'boundary=' size) withoutSeparators.
    idx := rest indexOf:$;.
    idx == 0 ifTrue:[
        mbound := rest
    ] ifFalse:[
        mbound := rest copyTo:idx-1.
    ].
    ^ mbound.
! !

!HTTPRequest class methodsFor:'serving-helpers'!

controlCharacters
    <resource: #obsolete>

    self obsoleteMethodWarning:'use HTMLUtilities controlCharacters'.
    ^ HTMLUtilities controlCharacters
!

escape:arg
    <resource: #obsolete>

    self obsoleteMethodWarning:'use HTMLUtilities escapeCharacterEntities:'.
    ^ HTMLUtilities escape:arg
!

escapeCharacterEntities:arg
    <resource: #obsolete>

    self obsoleteMethodWarning:'use HTMLUtilities escapeCharacterEntities:'.
    ^ HTMLUtilities escapeCharacterEntities:arg
!

unEscape:arg
    <resource: #obsolete>

    self obsoleteMethodWarning:'use HTMLUtilities unEscape:'.
    ^ HTMLUtilities unEscape:arg
!

urlEncoded:arg
    <resource: #obsolete>

    self obsoleteMethodWarning:'use HTMLUtilities urlEncoded:'.
    ^ HTMLUtilities urlEncoded:arg
! !

!HTTPRequest methodsFor:'accessing'!

arguments
    "the arguments of a SelectorService"

    ^ arguments ? #()
!

authenticator
    ^ authenticator
!

authenticator:something
    authenticator := something.
!

connection
    ^ connection
!

multiPartChunks
    ^ multiPartChunks
!

pathRelativeToService
    "set the path relative to the services root path.
     - without a leading /"

    ^ pathRelativeToService
!

pathRelativeToService:aString
    "set the path relative to the services root path.
     - without a leading /"

    pathRelativeToService := aString
!

queryString
    ^ queryString
!

rawUrl
    ^ rawUrl
!

rawUrl:something
    rawUrl := something.
!

relativePath
    "Obsolete: use #pathRelativeToService.
     get the path relative to the services root path.
     - with a leading /"

    <resource: #obsolete>

    ^ '/', pathRelativeToService
!

replyPresenter
    ^ response replyPresenter
!

response
    ^ response
!

selector
    "the selector of a SelectorService"

    ^ selector
!

service
    ^ service
!

service:serviceArg serviceLinkName:serviceLinkNameArg 
    service := serviceArg.
    serviceLinkName := serviceLinkNameArg.
!

serviceRequestContext
    serviceRequestContext isNil ifTrue:[
        serviceRequestContext := service requestContextForRequest:self.
    ].
    ^ serviceRequestContext
!

serviceRequestContext:something
    serviceRequestContext := something.
! !

!HTTPRequest methodsFor:'accessing-http request data'!

cookiesAt:aString 
    "Selects all the cookies with the name aString.
     <return: Collection>"

    |cookies|

    cookies := self cookies.
    cookies isNil ifTrue:[
        ^ #().
    ].
    ^ cookies select:[:eachCookie| eachCookie name = aString].
!

data
    ^ data
!

getURLQueryString
    |urlString questionMarkIndex |

    self isGET ifTrue:[
        urlString := self rawUrl.   
        urlString isString ifTrue: [
            questionMarkIndex := urlString indexOf:$?.
            questionMarkIndex ~= 0  ifTrue:[
                ^ urlString copyFrom: questionMarkIndex to: urlString size
            ].
        ].
    ].
    ^ ''
!

hasUrlParameter:key
    "true, if key is present in the url-param (after the urls path, separated by &'s)"

    ^ self urlParameters includesKey:key

    "Created: / 21-07-2007 / 19:57:03 / cg"
!

multiPartChunkDataAt:key
    ^ self multiPartChunkDataAt:key ifAbsent:nil.
!

multiPartChunkDataAt:key ifAbsent:replacement 
    multiPartChunks isNil ifTrue:[
        ^ replacement value
    ].
    ^ (multiPartChunks at:key ifAbsent:[^ nil]) data.
!

multiPartChunkFilenameAt:key
    ^ self multiPartChunkFilenameAt:key ifAbsent:nil.
!

multiPartChunkFilenameAt:key ifAbsent:aBlock 
    |multiPartChunk|

    multiPartChunks isNil ifTrue:[
        ^ aBlock value
    ].
    multiPartChunk := multiPartChunks at:key ifAbsent:[nil].
    multiPartChunk isNil ifTrue:[^nil].
    ^multiPartChunk filenameBaseName.
!

peer
    |socket|

    peer isNil ifTrue:[
        connection isNil ifTrue:[^ nil].
        (socket := connection socket) isNil ifTrue:[^ nil].
        peer := socket getPeer
    ].
    ^ peer
!

peer:something
    "set the value of the instance variable 'peer' (automatically generated)"

    peer := something.
!

peerName
    |peer|

    peerName isNil ifTrue:[
        peer := self peer.

        peer notNil ifTrue:[
            peerName := peer hostName
        ].
    ].
    ^ peerName
!

peerName:something
    "set the value of the instance variable 'peerName' (automatically generated)"

    peerName := something.
!

postData
    ^ postData
!

postDataFieldAt:key
    "a field (typically of a POST request)"

    ^ self postDataFieldAt:key ifAbsent:nil
!

postDataFieldAt:key ifAbsent:replacement
    "a field (typically of a POST request)"

    |vector|

    postDataFields isNil ifTrue:[^ replacement value].
    vector := postDataFields at:key ifAbsent:nil.
    vector isEmptyOrNil ifTrue:[^ replacement value].
    vector size ~~ 1 ifTrue:[
        self error:'duplicate key:', key, ' in postData fields' mayProceed:true.
    ].
    ^ vector first

    "Modified: / 10-07-2010 / 13:10:59 / cg"
!

postDataFields
    "the fields (typically of a POST request).
     Nil if there are none."

    ^ postDataFields
!

postDataVectorAt:key
    "all url parameters for key (after the urls path, separated by &'s)"

    ^ self postDataVectorAt:key ifAbsent:#()
!

postDataVectorAt:key ifAbsent:replacement
    "all url parameters for key (after the urls path, separated by &'s)"

    postDataFields isNil ifTrue:[^ replacement value].
    ^ postDataFields at:key ifAbsent:replacement.
!

rawRequestString
    ^ rawRequestString
!

selector:selectorArg arguments:argumentsArg
    "remember selector and arguments of a SelectorService (for the service)"

    selector := selectorArg.
    arguments := argumentsArg
!

server
    ^ server
!

serverHostName
    ^ server serverHostName
!

serviceLinkName
    ^ serviceLinkName
!

serviceLinkName:something
    serviceLinkName := something.
!

socketAddress
    "answer the address that received the request.
     This is one of the addresses assigned to the server"

    |sock|

    connection isNil ifTrue:[^ nil].
    sock := connection socket.
    sock isNil ifTrue:[^ nil].
    ^ sock getFullSocketAddress
!

urlParameterAt:key
    "a scalar url-param (after the urls path, separated by &'s).
     For example, if the url is 'foo?bar=1?baz=2', then 
        urlParameterAt:'bar' 
     would return '1' (a string)"

    ^ self urlParameterAt:key ifAbsent:nil

    "Modified: / 23-08-2007 / 09:42:00 / cg"
!

urlParameterAt:key ifAbsent:replacement
    "a scalar url-param (after the urls path, separated by &'s)"

    |vector|

    urlParameters isNil ifTrue:[^ replacement value].
    vector := urlParameters at:key ifAbsent:nil.
    vector isEmptyOrNil ifTrue:[^ replacement value].
    vector size ~~ 1 ifTrue:[
        self error:'duplicate key:', key, ' in url parameters' mayProceed:true.
        ^ vector last.
    ].
    ^ vector first

    "Modified: / 10-07-2010 / 13:11:18 / cg"
!

urlParameterAt:key put:aString
    urlParameters isNil ifTrue:[
        urlParameters := Dictionary new.
    ].
    urlParameters at:key put:(Array with:aString)
!

urlParameterVectorAt:key
    "all url parameters for key (after the urls path, separated by &'s)"

    ^ self urlParameterVectorAt:key ifAbsent:#()
!

urlParameterVectorAt:key ifAbsent:replacement
    "all url parameters for key (after the urls path, separated by &'s)"

    urlParameters isNil ifTrue:[^ replacement value].
    ^ urlParameters at:key ifAbsent:replacement.
!

urlParameters
    "the url-params (after the urls path, separated by &'s).
     Nil if there are none."

    ^ urlParameters 
!

valueOfNumericPostDataField:fieldName
    |field|

    field := self postDataFieldAt:fieldName ifAbsent:nil.
    field isEmptyOrNil ifTrue:[^ nil].

    ^ Number readFrom:field onError:nil.
! !

!HTTPRequest methodsFor:'accessing-http request header'!

acceptedCharsets
    "as extracted from the http-header, 
     returns key-value associations "

    |value|

    acceptedCharsets isNil ifTrue:[
        value := headerParser fieldValueFor:'accept-charset'.
        value notNil ifTrue:[
            acceptedCharsets := (HTTPRequest extractAcceptedFromLine:value asLowercase)
        ].
    ].
    ^ acceptedCharsets
!

acceptedEncodings
    "as extracted from the http-header"
    
    |value|

    acceptedEncodings isNil ifTrue:[
        value := headerParser fieldValueFor:'accept-encoding'.
        value notNil ifTrue:[
            acceptedEncodings := (HTTPRequest extractAcceptedFromLine:value asLowercase)
        ].
    ].
    ^ acceptedEncodings
!

acceptedLanguages
    "return the value of the instance variable 'acceptedLanguages' (automatically generated)"

    |value|

    acceptedLanguages isNil ifTrue:[
        value := headerParser fieldValueFor:'accept-language'.
        value notNil ifTrue:[
            acceptedLanguages := (HTTPRequest extractAcceptedFromLine:value asLowercase)
        ].
    ].
    ^ acceptedLanguages
!

acceptedTypes
    "as extracted from the http-header, if no Accept: field is present, 
     then it is assumed that text/plain and text/html are accepted."

    |value|

    acceptedTypes isNil ifTrue:[
        value := headerParser fieldValueFor:'accept'.
        value notNil ifTrue:[
            acceptedTypes := (HTTPRequest extractAcceptedFromLine:value "asLowercase")
        ]. 
"/        acceptedTypes isNil ifTrue:[
"/            acceptedTypes := #('text/plain' 'text/html').
"/        ].
    ].
    ^ acceptedTypes
!

authorizationHeader
    "as extracted from the http-header"
    
    ^ headerParser fieldValueFor:'authorization'.
!

connectionAttributes
    |value|

    connectionAttributes isNil ifTrue:[
        value := headerParser fieldValueFor:'connection'.
        value notNil ifTrue:[
            connectionAttributes := value asLowercase.
        ].
    ].
    ^ connectionAttributes
!

contentEncoding
    |value|

    contentEncoding isNil ifTrue:[
        value := headerParser fieldValueFor:'content-type'.
        value notNil ifTrue:[
            contentEncoding := HTMLUtilities extractCharSetEncodingFromContentType:value asLowercase.
        ].
    ].
    ^ contentEncoding
!

contentLength
    "return the value of the instance variable 'contentLength' (automatically generated)"
    
    |value|

    contentLength isNil ifTrue:[
        value := headerParser fieldValueFor:'content-length'.
        value notNil ifTrue:[
            contentLength := Integer readFromString:value
        ]
    ].
    ^ contentLength
!

contentType
    |value|

    contentType isNil ifTrue:[
        value := headerParser fieldValueFor:'content-type'.
        value notNil ifTrue:[
            contentType := HTMLUtilities extractMimeTypeFromContentType:(value asLowercase).
        ].
    ].
    ^ contentType
!

cookies
    |value|

    cookies isNil ifTrue:[
        value := headerParser fieldValueFor:'cookie'.
        cookies := self extractCookiesFrom:value.
    ].
    ^ cookies
!

fieldValueFor:aFieldKey
    ^ headerParser fieldValueFor:aFieldKey.
!

fieldValues
    "answer all the header fields as a Dictionary"

    ^ headerParser fieldValues.
!

header
    "the header as a dictionary"

    ^ headerParser fieldValues 
!

host
    ^ headerParser fieldValueFor:'host'.
!

httpVersion
    ^ httpVersion
!

ifModifiedSince
    "the file should be sent if modified after this date.
     Otherwise reponseCode 304 Not Modified may be send."

    ^ headerParser fieldValueFor:'if-modified-since'.
!

method
    ^ method
!

method:aString
    method := aString
!

multipartBoundary
    |value|

    self contentType = 'multipart/form-data' ifTrue:[
        value := headerParser fieldValueFor:'content-type'.
        multipartBoundary := HTTPRequest extractMultipartBoundaryFromContentType:value.
    ].
    ^ multipartBoundary
!

rawHeader
    ^ rawHeader 
!

referer
    "yes, I know that 'referer' is a misspelling of 'referrer'.
     However, somehow it made it into the HTTP standard, and is called 'referer' there - sigh."

    ^ headerParser fieldValueFor:'referer'.
!

referrer
    "the referring page - if known.
     Notice the mispelling below - due to the fact that the misspelling made it into 
     the HTTP standard"

    ^ headerParser fieldValueFor:'referer'.
!

soapAction
    ^ headerParser fieldValueFor:'soapaction'.
!

url
    ^ url
!

userAgent
    ^ headerParser fieldValueFor:'user-agent'.
! !

!HTTPRequest methodsFor:'accessing-http response data'!

binary:aBoolean
    response binary:aBoolean
!

expirationTime:aTimeStamp
    "the responses expirationTimestamp; consider to use response maxAge:"

    response expirationTime:aTimeStamp.
!

replyData:aStringOrBytes
    response data:aStringOrBytes
!

replyHeader
    "return the value of the instance variable 'replyHeader' (automatically generated)"

    ^ response header
!

resetResponseData
    "start anew collecting response data 
     (i.e. throw away what has been already collected)"

    response resetResponseData
!

responseContentType:something
    "set the value of the instance variable 'contentType' (automatically generated)"

    response contentType:something.
! !

!HTTPRequest methodsFor:'accessing-sessions'!

getSession
    ^ self getSession:true
!

getSession:shouldCreateIfNone
    "Get the current valid session associated with the request. 
     If shouldCreateIfNone is true then either a new session will be created 
     or the current one will be returned if one exists. 
     Answer nil, if shouldCreateIfNone is false and there is no session.

     The session's scope is a single service (over all of its possible linkNames).

     <return: HTTPSession | nil>"

    |cookie knownCookies sessionId linkNames makeCookies cookiesMatchingLinkNames|

    makeCookies := 
        [
            sessionId := UUID genRandomUUID printString.
            (server resolveServiceForPath:server homeURL) first == service ifTrue:[
                "this service is reached via the home url. Set the cookie for
                 the whole server"
                linkNames := #('/').
            ] ifFalse:[
                "create a session cookie for each link"
                "PROBLEM if one link is below another link:
                    first cookie    for /service/subservice,
                    second cookie   for /service
                    client then requests /service/subservice -> two cookies are received"
                linkNames := service linkNames.
            ].

            linkNames do:[:eachLinkName|
                cookie := HTTPCookie new 
                            name:#stxSessionId 
                            value:(Array with:sessionId with:eachLinkName)
                            path:eachLinkName 
                            expiration:nil. 

                cookie secure:self isSecure.
                self addCookie:cookie.
            ].
        ].

    knownCookies := self cookiesAt:#stxSessionId.
    knownCookies size == 1 ifTrue:[
        cookie := knownCookies first.
    ] ifFalse:[knownCookies size == 0 ifTrue:[
        shouldCreateIfNone ifFalse:[
            ^ nil.
        ].
        makeCookies value.
    ] ifFalse:["knownCookies size > 1" 
        "/ this happens if:
        "/    there is a service under "/foo" and another one under "/"
        "/ and both services have defined a cookie.
        "/ In this case the request contains 2 cookies.

        cookiesMatchingLinkNames := knownCookies select:[:cookie | service linkNames includes:cookie path] ifNone:nil. 
        cookiesMatchingLinkNames isEmpty ifTrue:[
            shouldCreateIfNone ifFalse:[
                ^ nil.
            ].
            makeCookies value.
        ] ifFalse:[
            cookiesMatchingLinkNames size > 1 ifTrue:[
                cookiesMatchingLinkNames sort:[:a :b | a path aize > b path size].
            ].
            cookie := cookiesMatchingLinkNames first.
        ].
    ]].

    ^ HTTPSession cookie:cookie server:server.
! !

!HTTPRequest methodsFor:'authentication'!

authenticateForRealm:aRealm
    "authenticate myself for a realm.
     Raise an error, if authentication fails"

    |usersCodes authorizationHeader|

    response expectedAuthRealm:aRealm.

    authorizationHeader := self authorizationHeader.
    authorizationHeader isNil ifTrue:[
        "By sending 'unauthorized', the client will open an username/password dialog
         and resend the request with an authorization header field"
        Authentication::BadCredentialsError raiseErrorString:'Please authenticate yourself!!'.
        "not reached"
    ].

    usersCodes := (service ? server) usersForRealm:aRealm.
    usersCodes isEmptyOrNil ifTrue:[
        "no users have been set up for this realm, so you can't authenticate against this realm.
         Setup users in the launcher's settings dialog

         If your webServer is running standalone, you have to do following steps:
            1. setup the server in the settings dialog 
            2. save the settings to a file and start your server 
               using the settings file. 

            As an alternative you can start a browser on the server machine (localhost)
            and setup authorization through the HTTP maintenance service. 

            This is for security reasons: avoid servers running with 
            default passwords by accident."

        "no retry here"

        HTTPAccessDeniedError raiseErrorString:'Security Error: No users have been set up for the "', aRealm printString, '" realm'.
    ].

    authenticator := Authentication::Authenticator initializeFromResponse:authorizationHeader.
"/    (authenticator isValidRealm:aRealm) ifTrue:[
"/        "authentication data from another realm - request new data"
"/        Authentication::BadCredentialsError raiseErrorString:'Please authenticate yourself!!'.
"/    ].

    authenticator requestMethod:method.
    authenticator bodyData:data.
    authenticator authenticateWithUserResolver:[:userName| usersCodes at:userName ifAbsent:[]].
    "not reached if authentication failed"

    (authenticator isValidUri:rawUrl) ifFalse:[
        "some authenticators (Digest) authenticate the url, too"
        HTTPAccessDeniedError new 
            resultCode:500; 
            errorString:'Security Error: bad url in authentication data: ', rawUrl printString;
            raise.
    ].

    ((service ? server) canAuthenticate:self with:authenticator) ifFalse:[
        "this should normally happen only if the allowed mechanisms have been changed
         - we supplied a challenge for supported mechanisms only"
        Authentication::BadCredentialsError raiseErrorString:'Unsupported authentication method: ', 
                                                    authenticator mechanismName.
    ].

    "Modified: / 14-07-2007 / 21:29:15 / cg"
    "Modified: / 26-04-2010 / 13:09:12 / sr"
!

authenticatedUsername
    "get the user, that has been authenticated
     or nil, if there is none"

    |authenticator|

    authenticator := self authenticator.
    authenticator notNil ifTrue:[
        ^ authenticator username.
    ].
    ^ nil.
!

isForbiddenUserAgent:aUserAgentString
    "there are certain user agents which we definitely do not want in our house:"

    ^ server forbiddenUserAgents contains:[:pattern | pattern match:aUserAgentString]
! !

!HTTPRequest methodsFor:'error reporting'!

reportBadRequest:msg
    "bad request."

    |body|

"/    connection logError:('Report BadRequest: ''', msg printString , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 400 Bad Request: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'Bad Request') , '</h1>
<p>The request was not understood by the server due to malformed syntax.</p>'.

    self sendErrorResponseForResponseCode:400 body:body
    "/ not reached

    "Created: / 30.7.1999 / 16:02:44 / cg"
!

reportException:ex
    "Write an error report."

    |s con|

    s := WriteStream on:(String new:500).
    
    s nextPutAll:'********** Request: '. rawHeader printOn:s. s cr.
    s nextPutAll:'********** Exception: '; nextPutAll:ex description; cr.
    con := ex suspendedContext.
    [con ~~ ex handlerContext] whileTrue:[
        con printOn:s.
        s cr.
        con := con sender.
    ].
    s cr.
    s := s contents.

"/    connection logError:s.
    server verboseErrors ifTrue:[
        Transcript show:s.
    ].

    self reportServerError:'Internal Server Error' moreInfo:s.
    "not reached"
!

reportForbidden:msg
    "access forbidden"

    self 
        reportForbidden:msg
        info:'Access to requested object was denied.'

    "not reached"

    "Modified: / 25-03-2011 / 09:53:42 / cg"
!

reportForbidden:msg info:infoText
    "access forbidden"

    |body|

"/    connection logError:('Report Forbidden access: ''', msg printString , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 403 Forbidden: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'Forbidden') , '</h1>
<p>',infoText,'</p>'.

    self sendErrorResponseForResponseCode:403 body:body
    "not reached"

    "Created: / 25-03-2011 / 09:53:09 / cg"
!

reportHTTPVersionNotSupported:msg
    | body |

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 505 HTTP Version not Supported: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'HTTP Version not Supported') , '</h1>
<p>This server does not support the requests HTTP protocol version.</p>'.

    self sendErrorResponseForResponseCode:505 body:body
    "/ not reached
!

reportMethodNotAllowed:msg method:badMethod url:badURL
    "method (such as POST, PUT etc.) is not allowed."

    | body |

"/    connection logError:('Report Forbidden method: ''', (msg ? '') , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 405 Method: ' , badMethod , ' in URL "' , (badURL ? '') , '" not Allowed: ' , (msg ? '') ).
    ].

    body := '<h1>' , (msg ? 'Method Not Allowed') , '</h1>
<p>The requested method ' , badMethod , ' is not allowed for the URL ' , badURL , '</p>'.

    response addLineToHeader:'ALLOW: ', (String fromStringCollection:self class allowedRequestMethods separatedBy:', ').

    self sendErrorResponseForResponseCode:405 body:body
    "not reached"
!

reportMovedPermanently:newUrl
    "page has been permanently moved to another location, newUrl."

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 301 Moved Permanently: ' , newUrl).
    ].
    response location:newUrl.
    self sendErrorResponseForResponseCode:301.
    "not reached"
!

reportNotAcceptable:typeAvailable
    "not acceptable - content-type not in accept list."

    |body|

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 406 Not Acceptable: ' , typeAvailable).
    ].
    body := '<h1>Accepted Types</h1>
<p>The server only accepts the following types:', typeAvailable asString, '<p>'.
    self sendErrorResponseForResponseCode:406 body:body
    "not reached"
!

reportNotFound:msg
    "document/object was not found. 
     Take care: IE6 will not show this message, but its own message,
     if the whole repsonse is shorter than 512 bytes and 
     'Show Short HTTP Error Messages' is on in Internet Settings"

    |body|

"/    connection logError:('Report Not found: ''', msg printString , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 404 Not Found: ' , (msg ? '')).
    ].

    body := '<http><body><h1>', (msg ? 'Object not found') , '</h1>
<!!--Dummy comment for stupid Internet Explorer, which needs more than 512 bytes to show a custom error page-->
<!!--Dummy comment for stupid Internet Explorer, which needs more than 512 bytes to show a custom error page-->
<!!--Dummy comment for stupid Internet Explorer, which needs more than 512 bytes to show a custom error page-->
<!!--Dummy comment for stupid Internet Explorer, which needs more than 512 bytes to show a custom error page-->
<p>The requested URL was not found on this server.</p></body></http>'.

    self sendErrorResponseForResponseCode:404 body:body
    "/ not reached
!

reportRequestTimeout:message
    "request timeout - the request was transmitted too slowly."

    |body|

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 408 Request Timeout: ' , (message ? '')).
    ].
    body := '<h1>Request Timeout</h1>
<p>' , (message ? '') asString, '</p>'.
    self sendErrorResponseForResponseCode:408 body:body
    "not reached"
!

reportServerError:msg moreInfo:aStringOrNil
    "server error."

    | body |

"/    connection logError:('Report Server Error: ''', msg printString , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 500 Internal Server Error: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'Internal Server Error') , '</h1>
<p>An internal server error occurred - your request could not be served.
<br>If this error persists, please notify the system administrator of this site.</p>'.
    aStringOrNil notNil ifTrue:[
        body := body, '<hr><p>Detail:<code><pre>', aStringOrNil, '</pre></code></p>'.
    ].

    self sendErrorResponseForResponseCode:500 body:body
    "/ not reached
!

reportServiceUnavailable:msg
    "report temporary down - will not return (i.e. aborts)."

    | body |

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 503 Service Unavailable: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'Service unavailable') , '</h1>
<p>This service is temporarily unavailable - please retry later.</p>'.
    self sendErrorResponseForResponseCode:503 body:body
    "/ not reached
!

reportUnauthorized:messageText
    "no authorization - ask for authorization."

    |realm|

    realm := response expectedAuthRealm.

    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 401 Unauthorized: ' , realm).
    ].
    server allowedAuthenticationClasses do:[:eachClass | 
        response addLineToHeader:'WWW-Authenticate: ',  (eachClass generateChallengeForRealm:realm).
    ].
    self sendErrorResponseForResponseCode:401 message:messageText
    "not reached"
!

reportUnsupportedFeature:msg
    "bad request."

    |body|

"/    connection logError:('Report BadRequest: ''', msg printString , '''').
    server verboseErrors ifTrue:[
        Transcript showCR:(server serverName, '[info]: 400 Unsupported: ' , (msg ? '')).
    ].

    body := '<h1>' , (msg ? 'Bad Request') , '</h1>
<p>The request was not processed due to an unsupported or missing feature.</p>'.

    self sendErrorResponseForResponseCode:400 body:body
    "/ not reached

    "Created: / 30.7.1999 / 16:02:44 / cg"
!

sendErrorResponseForResponseCode:aStatusCode
    self sendErrorResponseForResponseCode:aStatusCode 
         message:nil.
!

sendErrorResponseForResponseCode:aStatusCode body:bodyText 
    self sendErrorResponseForResponseCode:aStatusCode 
            title:(aStatusCode asString, ' ', (HTTPResponse reasonPhrasesDictionary at:aStatusCode ifAbsent:'')) 
            body:bodyText.
    "not reached"
!

sendErrorResponseForResponseCode:aStatusCode message:messageString
    "Send an error for aStatusCode (e.g. 440 - Not found).
     messageString may contain additional information"

    |body reason|

    reason := aStatusCode asString, ' ', (HTTPResponse reasonPhrasesDictionary at:aStatusCode ifAbsent:'').

    messageString isNil ifTrue:[
        body := '<h1>' , reason, '</h1>'.
    ] ifFalse:[
        body := '<h1>', messageString, '</h1>'.
    ].

    self sendErrorResponseForResponseCode:aStatusCode title:reason body:body.
!

sendErrorResponseForResponseCode:aStatusCode title:titleText body:bodyText 
    |presenter|

    response responseCode:aStatusCode.
    self resetResponseData.

    presenter := self replyPresenter.
    presenter isNil ifTrue:[
        response data:bodyText
    ] ifFalse:[
        presenter title:titleText.
        presenter firstHeadLine:titleText.
        self reply:bodyText.
    ].

    response send.

    "abort processing request - do not close socket in exception handler"
    HTTPServer::AbortRequestProcessing raise.

    "Modified: / 02-02-2007 / 12:21:09 / cg"
!

sendErrorResponseForStatusCode:aStatusCode
    <resource:#obsolete>
    self obsoleteMethodWarning:'use sendErrorResponseForResponseCode:'.
    self sendErrorResponseForResponseCode:aStatusCode message:nil.
!

sendErrorResponseForStatusCode:aStatusCode body:bodyText
    <resource:#obsolete>
    self obsoleteMethodWarning:'use sendErrorResponseForResponseCode:body:'.
    self sendErrorResponseForResponseCode:aStatusCode body:bodyText 
! !

!HTTPRequest methodsFor:'header parsing'!

extractCookiesFrom:line
    |extractedCookies s name cookie encodedValue|

    extractedCookies := OrderedCollection new.
    line notEmptyOrNil ifTrue:[
        s := line readStream.
        [s atEnd] whileFalse:[
            s skipSeparators.
            name := s upTo:$=.
            encodedValue := s upTo:$;.

            cookie := HTTPCookie new.
            cookie name:name.
            cookie rawValue:encodedValue.
            "/ now done lazy, when the value is asked for;
            "/ this is done because GOOGLE analytics adds its own cookies, which we cannot
            "/ parse. Therefore, only parse those cookies which we know are our own ones.
            "/ cookie extractValueAndPathFromRawValue.
            extractedCookies add:cookie.
        ].
    ].
    ^ extractedCookies

    "Created: / 08-07-2009 / 11:36:52 / Administrator"
!

headerParser
    ^ headerParser
!

parseFirstLine
    |firstLine tokens httpVersionsString|

    firstLine := headerParser startLine.
    tokens := firstLine asCollectionOfWords.
    tokens size < 3 ifTrue:[
        self reportBadRequest:'Bad Header Format'.
    ].
    method := tokens at:1.
    url := rawUrl := tokens at:2.
    httpVersionsString := tokens at:3.

    (httpVersionsString startsWith:'HTTP/') ifTrue:[
        httpVersion := (httpVersionsString copyFrom:'HTTP/' size + 1) withoutSeparators.
        httpVersion first digitValue <= 1 ifFalse:[
            self reportHTTPVersionNotSupported:('HTTP version is not supported by server: ' , httpVersion).
        ]
    ] ifFalse:[
        self reportBadRequest:'Missing HTTP version in request'.
        httpVersion := '1.0'.
    ].

    "Modified: / 29-09-2011 / 18:48:36 / cg"
!

parseHeader
    headerParser := HTTPHeaderParser headerString:rawHeader.
    "parse the first header line now - the rest is parsed on demand"
    self parseFirstLine.
!

rawRequestString:rawRequestStringArg rawHeader:rawHeaderArg data:dataArg
    rawRequestString := rawRequestStringArg.
    data := dataArg.
    rawHeader := rawHeaderArg.
"/    MessageTally spyDetailedOn:[
"/        500 timesRepeat:[
            self parseHeader.
"/        ].
"/    ].
! !

!HTTPRequest methodsFor:'html reply composition-stream protocol'!

nextPutAll:aString
    "append aString to the collected reply."

    response nextPutAll:aString
!

nextPutAllEscaped:aString
    "append aString to the collected reply.
     The string is processed, by replacing html-special characters by there html-equivalent (such as '>' into &lt;)"

    response nextPutAllEscaped:aString
!

nextPutLine:aString
    "append aString to the collected reply and append a lineBreak."

    response nextPutLine:aString.
!

nextPutLineEscaped:aString
    "append aString to the collected reply and append a cr.
     The string is processed, by replacing html-special characters by there html-equivalent (such as '>' into &lt;)"

    response nextPutLineEscaped:aString.
! !

!HTTPRequest methodsFor:'logging'!

logAccess
    connection logAccess:self.
!

logError:aString
    |serviceInfo|

    service notNil ifTrue:[
        serviceInfo := ' [',service printString,'] ' 
    ] ifFalse:[
        serviceInfo := ' '
    ].
    connection logError:(Timestamp now printString,serviceInfo,aString,Character cr).
! !

!HTTPRequest methodsFor:'misc'!

environmentValueFor:aFieldKey
    "return the value of an environment value.
     We have none, so return nil here. This is defined for FastCGI compatibility"

    ^ nil.
! !

!HTTPRequest methodsFor:'obsolete'!

escape: aString
    "use class escape:"

    <resource: #obsolete>

    ^ self class escape: aString
!

unEscape: aString
    "use class unEscape:"

    <resource: #obsolete>

    ^ self class unEscape: aString
! !

!HTTPRequest methodsFor:'old html reply composition'!

bodyEnd
    "Write </BODY>.
     OLD stuff - please use a TreeBuilder"

    response bodyEnd

    "Modified: / 02-06-2011 / 12:57:03 / cg"
!

bodyStart
    "Write <BODY>.
     OLD stuff - please use a TreeBuilder"

    response bodyStart

    "Modified: / 02-06-2011 / 12:57:08 / cg"
!

formEnd
    "append the standard footer for a form.
     OLD stuff - please use a TreeBuilder"

    response formEnd

    "Modified: / 02-06-2011 / 12:57:14 / cg"
!

formFooter
    "Write the standard footer for a form.
     OLD stuff - please use a TreeBuilder"

    response formFooter

    "Modified: / 02-06-2011 / 12:57:20 / cg"
!

formResetButton
    "generate a reset button for a form.
     OLD stuff - please use a TreeBuilder"

    response formResetButton

    "Modified: / 02-06-2011 / 12:57:26 / cg"
!

formSubmitButton
    "OLD stuff - please use a TreeBuilder"

    response formSubmitButton

    "Modified: / 02-06-2011 / 12:57:33 / cg"
!

formSubmitButtonForString:aString
    "OLD stuff - please use a TreeBuilder"

    response formSubmitButtonForString:aString

    "Modified: / 02-06-2011 / 12:57:44 / cg"
!

formSubmitButtonForString:aString name:aName
    "OLD stuff - please use a TreeBuilder"

    response formSubmitButtonForString:aString name:aName

    "Modified: / 02-06-2011 / 12:57:51 / cg"
!

formSubmitButtonStringForString:aString name:aName
    "OLD stuff - please use a TreeBuilder"

    ^ response formSubmitButtonStringForString:aString name:aName

    "Modified: / 02-06-2011 / 12:57:54 / cg"
!

htmlEnd 
    "OLD stuff - please use a TreeBuilder"

    response htmlEnd

    "Modified: / 02-06-2011 / 12:57:57 / cg"
!

htmlStart 
    "OLD stuff - please use a TreeBuilder"

    response htmlStart

    "Modified: / 02-06-2011 / 12:58:01 / cg"
!

imageFor:anImageOrImageFilename
    "generate an IMG tag for a real image (or an image file name).
     OLD stuff - please use a TreeBuilder"

    response imageFor:anImageOrImageFilename

    "Modified: / 02-06-2011 / 12:58:13 / cg"
!

imageFor:anImageOrImageFilename alt:alt
    "an IMG tag for a real image (or an image file name).
     OLD stuff - please use a TreeBuilder"

    response imageFor:anImageOrImageFilename alt:alt

    "Modified: / 02-06-2011 / 12:58:24 / cg"
!

imageLinkTextFor:anImageOrImageFilename alt:alt
    "return an IMG tag for a real image (or an image file name).
     OLD stuff - please use a TreeBuilder"

    ^ response imageLinkTextFor:anImageOrImageFilename alt:alt

    "Modified: / 02-06-2011 / 12:58:32 / cg"
!

inputFieldFor:varName label:label initialValue:value
    "an input field - must be within a form.
     OLD stuff - please use a TreeBuilder"

    response inputFieldFor:varName label:label initialValue:value

    "Modified: / 02-06-2011 / 12:58:39 / cg"
!

inputFieldFor:varName size:size label:label initialValue:value
    "an input field - must be within a form.
     OLD stuff - please use a TreeBuilder"

    response inputFieldFor:varName size:size label:label initialValue:value

    "Modified: / 02-06-2011 / 12:58:48 / cg"
!

inputFieldFor:varName type:aType size:size label:label initialValue:value
    "an input field - must be within a form.
     OLD stuff - please use a TreeBuilder"

    response inputFieldFor:varName type:aType size:size label:label initialValue:value

    "Modified: / 02-06-2011 / 12:58:57 / cg"
!

inputFieldFor:varName type:aType size:size label:label initialValue:value append:aString
    "an input field - must be within a form.
     type: 'password' | 'checkbox' | 'submit' | 'reset' | nil"

    "OLD stuff - please use a TreeBuilder"

    response inputFieldFor:varName type:aType size:size label:label initialValue:value append:aString

    "Modified: / 02-06-2011 / 12:59:04 / cg"
!

inputFieldStringFor:varName type:aType size:size label:label initialValue:value append:aString
    "an input field - must be within a form.
     type: 'password' | 'checkbox' | 'submit' | 'reset' | nil
    "

    "OLD stuff - please use a TreeBuilder"

    ^ response inputFieldStringFor:varName type:aType size:size label:label initialValue:value append:aString

    "Modified: / 02-06-2011 / 12:59:09 / cg"
!

label: labelString id: idString value: anObject
    "Add a table row for the given data to the reply."

    "OLD stuff - please use a TreeBuilder"

    response label: labelString id: idString value: anObject

    "Modified: / 02-06-2011 / 12:59:14 / cg"
!

label: labelString text: aString id: idString rows: rows cols: cols
    "Build a textarea form field."

    "OLD stuff - please use a TreeBuilder"

    response label: labelString text: aString id: idString rows: rows cols: cols

    "Modified: / 02-06-2011 / 12:59:17 / cg"
!

refreshStringFor:aService selector:aSelector arguments:arguments time:seconds 
    "OLD stuff - please use a TreeBuilder"

    ^ response 
        refreshStringFor:aService
        selector:aSelector
        arguments:arguments
        time:seconds

    "Modified: / 02-06-2011 / 12:59:25 / cg"
!

refreshStringFor:aService selector:aSelector time:seconds 
    "OLD stuff - please use a TreeBuilder"

    ^ response 
        refreshStringFor:aService
        selector:aSelector
        time:seconds

    "Modified: / 02-06-2011 / 12:59:29 / cg"
!

table:aBlock
    "OLD stuff - please use a TreeBuilder"

    response table:aBlock

    "Modified: / 02-06-2011 / 12:59:33 / cg"
!

table:aBlock attributes:attrString
    "OLD stuff - please use a TreeBuilder"

    response table:aBlock attributes:attrString.

    "Modified: / 02-06-2011 / 12:59:40 / cg"
!

tableEnd
    "OLD stuff - please use a TreeBuilder"

    response tableEnd

    "Modified: / 02-06-2011 / 12:59:37 / cg"
!

tableFormFooter
    "Close the table and write the standard footer for a form."

    "OLD stuff - please use a TreeBuilder"

    response tableFormFooter

    "Modified: / 02-06-2011 / 12:59:45 / cg"
!

tableRow:aBlock
    "OLD stuff - please use a TreeBuilder"

    response tableRow:aBlock

    "Modified: / 02-06-2011 / 12:59:49 / cg"
!

tableRow:aBlock attributes:attrString
    "OLD stuff - please use a TreeBuilder"

    response tableRow:aBlock attributes:attrString.

    "Modified: / 02-06-2011 / 12:59:53 / cg"
!

tableRowEnd
    "OLD stuff - please use a TreeBuilder"

    response tableRowEnd

    "Modified: / 02-06-2011 / 12:59:59 / cg"
!

tableRowStart:attrString
    "OLD stuff - please use a TreeBuilder"

    response tableRowStart:attrString

    "Modified: / 02-06-2011 / 13:00:03 / cg"
!

tableStart:attrString
    "OLD stuff - please use a TreeBuilder"

    response tableStart:attrString

    "Modified: / 02-06-2011 / 13:00:07 / cg"
!

title:aString
    "Write a standard header and set the document title."

    <resource: #obsolete>

    response title:aString with:''
!

title:aString with:someMoreHeaderStuff
    "Write a standard header and set the document title. Also used for logging purposes."

    <resource: #obsolete>

    response title:aString with:someMoreHeaderStuff.
!

titleAndHeader:aString
    "Write a standard header and set the document title."

    "OLD stuff - please use a TreeBuilder"

    <resource: #obsolete>

    response titleAndHeader:aString

    "Modified: / 02-06-2011 / 13:00:13 / cg"
!

unformattedCodeTextEnd
    "end some plainText."

    "OLD stuff - please use a TreeBuilder"

    response unformattedCodeTextEnd

    "Modified: / 02-06-2011 / 13:00:17 / cg"
!

unformattedCodeTextStart
    "start some plainText."

    "OLD stuff - please use a TreeBuilder"

    response unformattedCodeTextStart

    "Modified: / 02-06-2011 / 13:00:20 / cg"
!

unformattedText:aString
    "add some plainText."

    "OLD stuff - please use a TreeBuilder"

    response unformattedText:aString

    "Modified: / 02-06-2011 / 13:00:24 / cg"
!

unformattedTextEnd
    "end some plainText."

    "OLD stuff - please use a TreeBuilder"

    response unformattedTextEnd

    "Modified: / 02-06-2011 / 13:00:28 / cg"
!

unformattedTextStart
    "start some plainText."

    "OLD stuff - please use a TreeBuilder"

    response unformattedTextStart

    "Modified: / 02-06-2011 / 13:00:31 / cg"
! !

!HTTPRequest methodsFor:'old html reply composition stx'!

pageFinish
    "finish the page."
    response pageFinish.
!

pageSTXFooter
    "Write the standard page footer."
    response pageSTXFooter
!

stxBodyEnd 
    response stxBodyEnd
!

stxBodyStart:aString 
    "Write a standard stx body with a headline #f6feff"
    response stxBodyStart:aString
!

stxBodyStart:aString homeLink:aHomeLink
    "Write a standard stx body with a headline #f6feff"
    response stxBodyStart:aString homeLink:aHomeLink
!

stxBodyStart:aString homeLink:aHomeLink afterButtons:afterButtonsString
    "Write a standard stx body with a headline #f6feff"
    response stxBodyStart:aString homeLink:aHomeLink afterButtons:afterButtonsString
!

stxPageFinish
    "finish the page."
    <resource: #obsolete>
    self obsoleteMethodWarning:'use a reply presenter see ExeptReplyPresenter'.
    response stxPageFinish.
!

stxPageParameters
    ^ response stxPageParameters
!

stxPoweredByString
    ^ response stxPoweredByStringForService:service
!

stxReply:aString 
    response stxReply:aString
! !

!HTTPRequest methodsFor:'old links-html generation'!

homeWebLinkForText:nameString 
    "Return HTML code for a link to anObject using the given name and message selector."

    ^ self
        webLinkForText:nameString
        link:'/'
!

linkIdFor:aService 
    "Return the linkId to send aMessage to anObject. 
     if the request is for another service, answer the first link for the service"

    aService == service ifTrue:[
        ^ self serviceLinkName.   
    ].
    ^ aService linkName
!

linkIdFor:aService argument:argumentString
    "Return the linkId to send aMessage to anObject. 
     Create a new linkId if necessary."

    | string |
    string := self linkIdFor:aService.
    argumentString notEmptyOrNil ifTrue:[
        string := string, '/' , argumentString
    ].
    ^ string
!

linkIdFor:aService selector:aSelector
    "Return the linkId to send aMessage to anObject. 
     Create a new linkId if necessary."

    |sel|
    sel := aSelector.
    (sel endsWith:':') ifFalse:[
        sel := sel , ':'
    ].
    ^ ((self linkIdFor:aService) , '/' , sel)  
!

linkIdFor:aService selector:aSelector argument:argumentString
    "Return the linkId to send aMessage to anObject. 
     Create a new linkId if necessary."

    | string |
    string := (self linkIdFor:aService selector:aSelector).
    argumentString notEmptyOrNil ifTrue:[
        string := string, ',' , argumentString
    ].
    ^ string
!

webLinkForImage:imageOrImageFilename service:aService argument:aSelector
    "Return HTML code for a link to anObject using the given name and message selector."

    |imgRef|

    imgRef := '<img SRC="' , (server graphicLinkIdFor:imageOrImageFilename) , '">'.
    ^ self webLinkForText:imgRef service:aService argument:aSelector
!

webLinkForImage:imageOrImageFilename service:aService selector:aSelector
    "Return HTML code for a link to anObject using the given name and message selector."

    |imgRef|

    imageOrImageFilename isImageOrForm ifTrue:[
        imgRef := '<img SRC="' , (server graphicLinkIdFor:imageOrImageFilename) , '">'.
    ] ifFalse:[
        imgRef := '<img SRC="' , imageOrImageFilename , '">'.
    ].
    ^ self webLinkForText:imgRef service:aService selector:aSelector
!

webLinkForText:nameString link:aLinkId 
    "Return HTML code for a linkId using the given name 
     and message selector. When the link is followed,
     aSelector plus argument will be sent to anObject."

    |stream|

    stream := WriteStream on: ''.
    stream
        nextPutAll: '<a href="';
        nextPutAll: aLinkId;
        nextPutAll: '">';
        nextPutAll: nameString;
        nextPutAll: '</a>'.
    ^ stream contents
!

webLinkForText:nameString service:aService argument:anArgument
    "Return HTML code for a link to aService using the given nameString 
     and aSelector. When the link is followed,
     aSelector plus argument will be sent to aService."

    ^ self
        webLinkForText:nameString
        link:(self linkIdFor:aService argument:anArgument) 
!

webLinkForText:nameString service:aService selector:aSelector
    "Return HTML code for a link to anObject using the given name and message selector."

    ^ self
        webLinkForText:nameString
        link:(self linkIdFor:aService selector:aSelector)
!

webLinkForText:nameString service:aService selector:aSelector argument:anArgument
    "Return HTML code for a link to aService using the given nameString 
     and aSelector. When the link is followed,
     aSelector plus argument will be sent to aService."

    ^ self
        webLinkForText:nameString
        link:(self linkIdFor:aService selector:aSelector argument:anArgument) 
!

webLinkText:nameString target:targetString service:aService selector:aSelector
    "Return HTML code for a link to aService using the given name and message selector."

    |stream|

    stream := WriteStream on: ''.
    stream 
        nextPutAll: '<a href="';
        nextPutAll: (self linkIdFor:aService selector:aSelector);
        nextPutAll: '" target="';
        nextPutAll: targetString;
        nextPutAll: '">';
        nextPutAll: nameString;
        nextPutAll: '</a>'.
    ^stream contents

    "Created: / 27.9.1999 / 11:09:19 / cg"
! !

!HTTPRequest methodsFor:'printing'!

printOn:aStream
    aStream nextPutAll:'HTTPRequest('.
    method printOn:aStream.
    aStream nextPutAll:' '.
    url printOn:aStream.
    aStream nextPut:$).

    "Modified: / 07-07-2009 / 12:38:19 / sr"
! !

!HTTPRequest methodsFor:'queries'!

fullPath
    ^ serviceLinkName, '/', pathRelativeToService
!

isGET
    "return true, if the request was a HEAD request"

    ^ method = #GET
!

isHEAD
    "return true, if the request was a HEAD request"

    ^ method = #HEAD
!

isPOST
    "return true, if the request was a POST request"

    ^ method = #POST
!

isSecure
    "answer true, if this request was issued over an secure channel (SSL/TLS)"

    ^ false
!

startReadRequestTime
    ^ connection startReadRequestTime
!

timeToPerform
    "in milliseconds"

    ^ self connection timeToPerformRequest
! !

!HTTPRequest methodsFor:'response collecting'!

addCookie:cookie
    "send a cokkie with the response.
     Remember the cookie in the request, so that session cookies 
     will not be created twice"

    self cookies add:cookie.
    response addCookie:cookie.
!

addToReplyHeader:aString 
    "collect part of a header (i.e. append aString to the reply header)."
    
    response addLineToHeader:aString
!

reply:aString
    "collect part of a reply (i.e. append aString to the reply buffer)."

    response reply:aString.
!

replyCR
    "Append a cr to the reply buffer"

    response replyCR
!

replyEscaped:aStringOrCharacter
    "collect part of a reply (i.e. append aString to the reply buffer).
     Special character (lt, gt) are escaped."

    response replyEscaped:aStringOrCharacter

    "Modified: / 19-03-2007 / 14:05:03 / cg"
!

replyHeader:aString
    response replyHeader:aString
!

replyURLEscaped:aStringOrCharacter
    "collect part of a reply (i.e. append aString to the reply buffer).
     URL-Special characters (&, space etc.) are escaped."

    response reply:(HTMLUtilities escape:aStringOrCharacter)
! !

!HTTPRequest methodsFor:'response sending'!

redirectTo:link
    "redirect both per META and per 302-location-redirect.
     (because content-location redirection doesn't work for most of the browsers)"

    response
        nextPutLine:'<HTML>';
        nextPutLine:'<HEAD>';
        nextPutAll:'<META http-equiv=refresh content="0; url=',link,'">';
        nextPutLine:'</HEAD>';
        nextPutLine:'<BODY>';
        nextPutLine:'You are being redirected to <A HREF="',link, '">',link,'</A>';
        nextPutLine:'</BODY>';
        nextPutLine:'</HTML>'.

    response
        responseCode:302;
        location:link;
        contentLocation:link;
        send.
!

sendBinary:aStreamOrBytes type:contentType 
    "send some binary data as a response. 
     Typically used with images and non-text documents"

    self sendBinaryStream:aStreamOrBytes type:contentType.
!

sendBinaryData:binaryData type:contentType 
    "send some binary data as a response. 
     Typically used with images and non-text documents"

    response sendBinaryData:binaryData type:contentType
!

sendBinaryStream:aStreamOrBytes type:contentType 
    "send some binary data as a response. 
     Typically used with images and non-text documents"

    response 
        binaryDataStream:aStreamOrBytes contentType:contentType;
        send.
!

sendImage:inImage
    "reply with a bitmap image.
     For now, must send it as gif ... (could do jpg as well),
     or even better, remember what types are supported by the browser"

    |imgData|

    imgData := HTTPServer gifImageDataForImage:inImage.
    imgData isNil ifTrue:[ 
        "/ mhm - what should we do if the image cannot be represented as gif ?
        self reportNotFound:'Image cannot be represented as GIF'.
        "/ not reached
        ^ self 
    ].

    response sendBinaryData:imgData type:'image/gif'.
!

sendResponse 
    "send the response"

    response send.
!

sendResponseWithType:aContentType
    "send the response"

    self responseContentType:aContentType.
    response send.
! !

!HTTPRequest methodsFor:'serving-helpers'!

checkAcceptableType:type 
    |locAcceptedTypes|

    locAcceptedTypes := self acceptedTypes.
    locAcceptedTypes isEmptyOrNil ifTrue:[^ true].

    locAcceptedTypes do:[:eachAcceptedTypeAndQPair | 
        |eachAcceptedType correspondingQ|

        eachAcceptedType := eachAcceptedTypeAndQPair key.
        "/ correspondingQ := eachAcceptedTypeAndQPair value.
        (eachAcceptedType = '*/*') ifTrue:[
            ^ true
        ].
        (eachAcceptedType match:type) ifTrue:[
            ^ true
        ].
    ].

    server verbose ifTrue:[
        Transcript showCR:server serverName, ': unacceptable: ' , type printString
    ].
    ^ false
!

decodeFields: aString
    "Convert the form fields in Collection of strings into a dictionary.
     collection is needed because e.g. (on <SELECT name=FOO MULTIPLE> by client selects more than one
     FOO comes with same key and different values)
     Each field should be of the form: &nameKey=collection of values"

    | query dict i key value  oldValue|

    query := aString asCollectionOfSubstringsSeparatedByAny:'&;?'.
    query := query collect:[:each | each withoutSeparators].

    dict := OrderedDictionary new.

    query do: [ :tag |
        i := tag indexOf: $=.
        key := tag copyFrom: 1 to: i - 1.
        value := (i < tag size) 
                     ifTrue: [ 
                         HTMLUtilities unEscape: (tag copyFrom: i + 1 to: tag size)
                     ] ifFalse: [
                         ''
                     ].

        (dict includesKey:key) ifFalse: [ 
            dict at:key put:(OrderedCollection with:value) 
        ] ifTrue:[
            oldValue := (dict at:key) add:value.
        ]
    ].
    ^ dict

    "Modified: / 25-06-2009 / 11:11:44 / sr"
!

extractPostDataFields
    |requestData crlfcrlf startIndex multipartBoundarySize endIndex splitPoint fields chunkData chunk|

    requestData := self data.
    requestData notNil ifTrue:[
        "/ split at CRLFCRLF (left part are fields; right part is data)
        crlfcrlf := HTTPServer crlfcrlf.
        self contentType = 'multipart/form-data' ifTrue:[
            multiPartChunks := Dictionary new.
            startIndex := 1.
            multipartBoundarySize := self multipartBoundary size.
            [
                startIndex := requestData indexOfSubCollection:(self multipartBoundary) startingAt:startIndex.
                endIndex := requestData indexOfSubCollection:(self multipartBoundary) startingAt:(startIndex + multipartBoundarySize).
                startIndex ~= 0 and:[ endIndex ~= 0 ]
            ] whileTrue:[
                splitPoint := requestData indexOfSubCollection:crlfcrlf startingAt:startIndex.
                splitPoint == 0 ifTrue:[
                    fields := requestData
                ] ifFalse:[
                    fields := requestData copyTo:splitPoint - 1.
                    chunkData := requestData copyFrom:splitPoint + crlfcrlf size.
                ].
                chunk := MultipartChunk 
                            fields:fields
                            postData:chunkData
                            request:self.
                multiPartChunks at:chunk name put:chunk.
                startIndex := endIndex.
            ].
        ] ifFalse:[
            splitPoint := requestData indexOfSubCollection:crlfcrlf.
            splitPoint == 0 ifTrue:[
                fields := requestData
            ] ifFalse:[
                fields := requestData copyTo:splitPoint - 1.
                postData := requestData copyFrom:splitPoint + crlfcrlf size.
                self postData:postData.
            ].
            postDataFields := self decodeFields:fields
        ].
    ].
!

extractURLParametersFromPath:path
    "answer the local part of the path without parameters.
     As a side effect the parameters are stored in the instVar urlParameters"

    |idx localPath|

    localPath := path.
    idx := localPath indexOfAny:'&;?'.
    idx ~~ 0 ifTrue:[
        queryString := path copyFrom:idx+1.
        localPath := path copyTo:idx-1.
        urlParameters := self decodeFields:queryString.
    ].
    ^ localPath
! !

!HTTPRequest methodsFor:'serving-processing'!

handleSOAPAction
    |handler answer|

    (handler := self server soapActionHandler) isNil ifTrue:[
        "/ reply with permisssion denied
        self reportServiceUnavailable:'SOAP Service not available'.
        "/ not reached
        ^ self.
    ].

    Error 
        handle:[:ex |
            self reportException:ex.
            "/ not reached
        ] 
        do:[
            answer := handler process:self.
        ].

    answer isNil ifTrue:[
        self reportServiceUnavailable:'SOAP Service not available'.
        "/ not reached
        ^ self.
    ].
    self reply:answer data.
    self sendResponseWithType:answer contentType.
    ^ self.
!

reprocess
    method := 'GET'.
    "/ reproces the current request (see handler in generateReply)
    ReprocessRequest raiseRequest
!

reprocessWithInternalURL:aNewInternalURL
    method := 'GET'.
    "/ reproces the current request with another URL (see handler in generateReply)
    url := aNewInternalURL.
    ReprocessRequest raiseRequest
    "/ not reached
!

reprocessWithServiceURL:aNewInternalServiceURL
    self reprocessWithInternalURL:(serviceLinkName , aNewInternalServiceURL).
!

robotPolicy
    "Return a text specifying our robots policy.
     This is the answer to be given to webCrawlers and tells them,
     which URLs (subtrees) should not be visited.
     See 
        http://info.webcrawler.com/mak/projects/robots/norobots.html
     for details.
     The default here is to disallow everything."

    |s|

    connection logReply: 'Robot Policy requested'.

    s := '' writeStream.

    "/ do not allow anything:
    s nextPutLine:'# Sorry, robot.'.
    s nextPutLine:'User-agent: *'.
    server serviceList do:[:aService|
        aService forbiddenRobotsPaths do:[:aPath |
            s nextPutLine:'Disallow: ', aPath.
        ].
    ].
    ^ s contents
!

url:aNewURL
    url := aNewURL.
! !

!HTTPRequest methodsFor:'serving-requests'!

generateReply
    "handle a request; generate the reply."

    |handledSignals|

    handledSignals := ReprocessRequest, HTTPAccessDeniedError.
    Authentication::AuthenticationError notNil ifTrue:[
        handledSignals := handledSignals , Authentication::AuthenticationError
    ].
    
    handledSignals handle:[:ex|
        |signal|

        signal := ex signal.
        
        signal == ReprocessRequest ifTrue:[
            ex restart.
        ].
        (Authentication::AuthenticationError notNil
        and:[Authentication::AuthenticationError accepts:signal]) ifTrue:[
            (Authentication::ExpiredCredentialsError == signal) ifTrue:[
                response addLineToHeader:'WWW-Authenticate: ',  authenticator generateStaleChallenge.
                self sendErrorResponseForResponseCode:401 message:'stale nonce'.
                "not reached"
            ].
            (Authentication::BadCredentialsError accepts:signal) ifTrue:[
                self reportUnauthorized:ex description.
                "not reached"
            ].
            self reportBadRequest:ex description.
            "not reached"
        ].

        self sendErrorResponseForResponseCode:(ex resultCode ? 403) message:ex description.
        "not reached"
    ] do:[
        method isNil ifTrue:[  "/ POST, GET , HEAD etc.
            self reportBadRequest:'Bad request (missing method)'.
            ^ self.
        ].
        url isNil ifTrue:[
            self reportBadRequest:'Bad request (missing url)'.
            ^ self.
        ].

        method = #GET ifTrue:[
            self generateReplyForGetRequest.
            ^ self.
        ].

        method = #POST ifTrue: [ 
            self generateReplyForPostRequest.
            ^ self
        ].

        method = #HEAD ifTrue:[
            self generateReplyForHeadRequest.
            ^ self.
        ].

        self reportMethodNotAllowed:nil method:method url:self url.
        "not reached"
    ]

    "Modified: / 09-05-2007 / 16:33:30 / cg"
!

generateReplyForGetRequest
    "Generate the reply to a GET request."
    
    |path notFoundURL homeURL serviceReplyPresenter|

    (self isForbiddenUserAgent:(self userAgent)) ifTrue:[
        self reportNotFound:('No service.').
        ^ self.
    ].

    path := url.

    "sr: url parameter have to be generated from the raw url (not unescaped),
     only the parameter values have to be unescaped,
     this happens within #extractURLParametersFromPath:path"
    "/ cleanup from url parameters
    path := self extractURLParametersFromPath:path.

    "/ cg: unescape IS required (for 8-bit characters)
    path := HTMLUtilities unEscape:path.
    "/ cg: and even utf-8 decoding is
    path := path utf8Decoded.
    "/ remember
    url := path.

    "/ translate empty-path to homeURL
    path = '/' ifTrue:[
        homeURL := server homeURL.
        homeURL notEmptyOrNil ifTrue:[
            path := homeURL.
        ].
    ].

    (path = '/robots.txt') ifTrue:[
        self reply:self robotPolicy.
        self sendResponseWithType:'text/plain'.
        ^ self.
    ].

    server serviceAndLinkNameFromPath:path into:self.
    service isNil ifTrue:[
        "unknown link, try the notFoundURL"
        notFoundURL := server notFoundURL.
        notFoundURL notEmptyOrNil ifTrue:[
            server serviceAndLinkNameFromPath:notFoundURL into:self.
            service notNil ifTrue:[
                self reprocessWithInternalURL:notFoundURL.
                 "not reached"
            ].
        ].
        self reportNotFound:('No service found for: ' , path).
         "not reached"
        ^ self.
    ].
    server inMaintenanceMode ifTrue:[
        service isAllowedInMaintenanceMode ifFalse:[
            self checkMaintenanceMode.
             "not reached if not authorized for maintenance mode"
        ]
    ].
    server verbose ifTrue:[
        Transcript showCR:'Serving service ' , service className.
    ].
    serviceReplyPresenter := service replyPresenterForRequest:self.
    serviceReplyPresenter notNil ifTrue:[
        response replyPresenter:serviceReplyPresenter.
    ].
    service authorizeRequest:self.
     "only reached if authenticated/authorized"
    [
        service _process:self.
        response send.
    ] ensure:[
        serviceRequestContext notNil ifTrue:[
            serviceRequestContext release.
        ].
        serviceReplyPresenter notNil ifTrue:[
            serviceReplyPresenter release.
        ].
    ].

    "Modified: / 29-09-2011 / 19:02:15 / cg"
    "Modified: / 04-10-2011 / 15:57:29 / sr"
!

generateReplyForHeadRequest
    "Generate the reply to a HEAD request (same as GET, but no data is sent at the end)."

    self generateReplyForGetRequest
!

generateReplyForPostRequest
    "handle a post-request; generate the reply."
    
    self soapAction notNil ifTrue:[
        self handleSOAPAction.
        ^ self
    ].

    "/ some other POST reply is handled like a GET.
    "/ Thats probably wrong, since we have to care for the
    "/ post action type (should eventually do CGIs etc.).
    self extractPostDataFields.
    self generateReplyForGetRequest.
! !

!HTTPRequest methodsFor:'serving-serverState'!

checkMaintenanceMode
    "if server is in mainenance mode, no files are served to normal users.
     Access to a server in maintenance mode is only possibly to a user 
     that has authenticated for maintenance service"

    |serverMaintenanceServices realm|

    self server inMaintenanceMode ifTrue:[
        serverMaintenanceServices := server servicesForServiceClass:ServerMaintenanceService.
        serverMaintenanceServices isEmpty ifTrue:[
            self reportServiceUnavailable:'Maintenance mode unavailable'.
            ^ self
        ].
        realm := serverMaintenanceServices first realm.

        [
            self authenticateForRealm:realm.
        ] on:Authentication::BadCredentialsError do:[:ex|
            "change the error string in the exception"
            ex errorString:('Authentication for maintenance (realm=<1s>) failed' expandMacrosWith:realm).
            ex reject.
        ].
    ].
! !

!HTTPRequest methodsFor:'setup from connection'!

initializeForServer:serverArg connection:connectionArg
    server := serverArg.
    response := HTTPResponse request:self.
    connection := connectionArg.
!

postData:something
    postData := something.
! !

!HTTPRequest::MultipartChunk class methodsFor:'instance creation'!

fields:fields postData:postData request:aRequest

    ^ self new fields:fields postData:postData request:aRequest
! !

!HTTPRequest::MultipartChunk methodsFor:'accessing'!

contentDisposition
    ^ contentDisposition
!

contentDispositionElements
    ^ contentDispositionElements
!

contentType
    ^ contentType
!

contentTypeElements
    ^ contentTypeElements
!

data
    ^ data
!

name

    ^ contentDispositionElements at:#'name' ifAbsent:#unknown
!

request
    ^ request
! !

!HTTPRequest::MultipartChunk methodsFor:'actions'!

writeFile:fileName intoDirectory:aDirectory 
    |fullFileName stream|

    self isFilenameChunk ifTrue:[
        fullFileName := aDirectory construct:fileName.
        stream := fullFileName writeStream.
        contentType := self contentTypeElements at:#contentType ifAbsent:nil.
        (contentType startsWith:'text/') ifTrue:[ 
            stream text.
        ] ifFalse:[
            stream binary.
        ].
        stream nextPutAll:data.
        stream close.
    ].
!

writeFileIntoDirectory:aDirectory
    |fileName|

    self isFilenameChunk ifTrue:[
        fileName := self filenameBaseName.
        fileName notEmptyOrNil ifTrue:[
            self writeFile:fileName intoDirectory:aDirectory
        ]
    ].
! !

!HTTPRequest::MultipartChunk methodsFor:'initialization'!

fields:fields postData:postData request:aRequest
    |lines crlf|

    request := aRequest.
    crlf := String crlf.
    lines := fields asCollectionOfSubstringsSeparatedByAll:crlf.
    lines copy do:[:aLine|
        (aLine asLowercase startsWith:'content-disposition') ifTrue:[
            self readContentDispositionFrom:aLine.
        ] ifFalse:[
            (aLine asLowercase startsWith:'content-type') ifTrue:[
                self readContentTypeFrom:aLine.
            ]
        ]
    ].
    data := postData copyTo:((postData indexOfSubCollection:(aRequest multipartBoundary)) - 3 - crlf size).
!

readContentDispositionFrom:aLine

    |elements|

    contentDisposition := aLine copyFrom:(('content-disposition:') size + 1).
    elements := contentDisposition asCollectionOfSubstringsSeparatedBy:$;.
    elements := elements collect:[:aElement| aElement withoutSeparators].
    contentDispositionElements := Dictionary new.
    contentDispositionElements at:#'dispositionType' put:elements first.
    self readElementsFrom:(elements copyFrom:2) to:contentDispositionElements.
!

readContentTypeFrom:aLine

    |elements multipartBoundary msg|

    contentType := (aLine copyFrom:(('content-type:') size + 1)) withoutSeparators.
    contentTypeElements := Dictionary new.
    elements := contentType asCollectionOfSubstringsSeparatedBy:$;.
    elements := elements collect:[:aElement| aElement withoutSeparators].
    contentTypeElements at:#'contentType' put:elements first.
    self readElementsFrom:(elements copyFrom:2) to:contentTypeElements.
    ((contentTypeElements at:#contentType ifAbsent:'') = 'multipart/mixed') ifTrue:[
        "/ embedded multipart/mixed content found
        "/ not supported by now because cant find browser who supports this
        msg := 'Upload of Multiple files not yet supported'.
        request connection logError:msg, ' request comes from:', request userAgent, ' fix this in:', thisContext printString.
        request reportBadRequest:msg.
        multipartBoundary := HTTPRequest extractMultipartBoundaryFromContentType:aLine.
    ].
!

readElementsFrom:aStringCollection to:aDictionary

    |index key value|

    aStringCollection do:[:aElement|
        index := aElement indexOf:$=.
        key := (aElement copyTo:(index - 1)) asSymbol.
        value := aElement copyFrom:(index + 1).
        (value startsWith:$") ifTrue:[
            value := value copyFrom:2.
        ].
        (value endsWith:$") ifTrue:[
            value := value copyTo:(value size - 1).
        ].
        aDictionary at:key put:value.
    ].
! !

!HTTPRequest::MultipartChunk methodsFor:'queries'!

filenameBaseName

    |fileName seperatorIndex|

    self isFilenameChunk ifFalse:[^ nil].
    fileName := self contentDispositionElements at:#'filename' ifAbsent:nil.
    fileName notNil ifTrue:[
        "/ IE sends his full pathname - have to extract basename
        seperatorIndex := fileName lastIndexOf:$\.
        seperatorIndex = 0 ifTrue:[
            "/ check unix fileseperator too
            seperatorIndex := fileName lastIndexOf:$/.
        ].
        seperatorIndex ~= 0 ifTrue:[
            fileName := fileName copyFrom:(seperatorIndex + 1).
        ].
    ].
    ^ fileName
!

isFilenameChunk

    ^ (contentDispositionElements at:#'filename' ifAbsent:nil) notNil
! !

!HTTPRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPRequest.st,v 1.223 2011/11/14 18:00:16 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPRequest.st,v 1.223 2011/11/14 18:00:16 stefan Exp $'
! !
