"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPResponse
	instanceVariableNames:'header request cookies binary responseAlreadySent replyPresenter
		dataStream replyPresenterDataStream expectedAuthRealm'
	classVariableNames:'DefaultSTXPageParameters'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!HTTPResponse class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    represents a response to an HTTP request as constructed (filled in)
    by HTTP-Services.


    [instance variables:]
        header              collects all header data
        data                a chunk of data to send (if data is available as String or ByteArray)
        dataStream          a stream collecting (writeStream) or providing (readStream) the replyData
                            (if data is nil)
        request             the request for which this is a reply
        cookies             cokies to set in the client
        binary              ?
        responseAlreadySent flag set by services which send something on their own behalf
                            (prevent the response from beeing sent by the server at the end)

    [author:]
        Christian Penk

    [see also:]
        http://www.w3.org/Protocols/rfc2616/rfc2616.html
        HTTPRequest
        HTTPServer
        HTTPInterface
"
! !

!HTTPResponse class methodsFor:'instance creation'!

request:aRequest 
    "returns an initialized instance for the request"

    ^ (self new request:aRequest) initialize.
! !

!HTTPResponse class methodsFor:'constants'!

REPLYCODE_BADREQUEST
    ^ 400
!

REPLYCODE_FORBIDDEN
    ^ 403
!

REPLYCODE_GONE
    ^ 410
!

REPLYCODE_NOTFOUND
    ^ 404
!

reasonPhrasesDictionary
    "returns a dictionary which maps HTTP-codes to a descriptive message text"

    ^ HTTPResponseHeader reasonPhrasesDictionary
! !

!HTTPResponse methodsFor:'accessing'!

binary
    "get the binary mode flag (data is bytes as opposed to text)"

    ^ binary ? (header isBinaryContent)
!

binary:aBoolean
    "set/clear binary mode (data is bytes as opposed to text)"

    binary := aBoolean
!

binaryData:aStreamOrBytes contentType:contentType
    "setup to return the contents of a binary stream
     (typically: image data)"

    binary := true.
    header contentType:contentType.
    self dataStream:aStreamOrBytes readStream.
!

binaryDataStream:aStreamOrBytes contentType:contentType
    "setup to return the contents of a binary stream
     (typically: image data)"

    binary := true.
    header contentType:contentType.
    self dataStream:aStreamOrBytes.
!

cookies
    "a collection of cookies (to be filled by the service)"

    cookies isNil ifTrue:[
        cookies := OrderedCollection new.
    ].
    ^ cookies
!

data
    ^ dataStream contents
!

data:bytesOrCharacters
    "set the response data en-bloque. 
     Useful if the data is already available in a service (for example, from a cache)"

    (bytesOrCharacters isStream) ifTrue:[
        Transcript showCR:'HTTPResponse: please use #dataStream: to set stream data'.
        self dataStream:bytesOrCharacters.
    ] ifFalse:[
        dataStream := ReadStream on:bytesOrCharacters.
    ]
!

data:bytesOrCharacters contentType:aMimeType
    "set the response data en-bloque and the type. 
     Useful if the data is already available in a service (for example, from a cache)"

    self data:bytesOrCharacters.
    self contentType:aMimeType.
!

dataFile:aFilename
    "set the response data to be read from a file. 
     Useful if the data is big and should not be read in for transmission"

    self dataStream:(aFilename asFilename readStream).
!

dataFile:aFilename contentType:aMimeType
    "set the response data to be read from a file and the content type. 
     Useful if the data is big and should not be read in for transmission"

    self dataStream:(aFilename asFilename readStream).
    self contentType:aMimeType.
!

dataStream
    ^ dataStream
!

dataStream:aStream
    "set the response dataStream. 
     Useful if the data is to be read from a file and should not be read in for transmission"

    (aStream isStream) ifFalse:[
        Transcript showCR:'HTTPResponse: please use #data: to set non-stream data'.
    ].
    dataStream := aStream.
!

dataStream:aStream contentType:aMimeType
    "set the response dataStream and the content type. 
     Useful if the data is to be read from a file and should not be read in for transmission"

    self dataStream:aStream.
    self contentType:aMimeType.
!

expectedAuthRealm
    ^ expectedAuthRealm
!

expectedAuthRealm:aRealm
    expectedAuthRealm := aRealm
!

header
    ^ header
!

header:something
    header := something.
!

replyPresenter
    ^ replyPresenter
!

replyPresenter:something
    replyPresenter := something.
!

replyPresenterDataStream
    ^ replyPresenterDataStream
!

request
    ^ request
!

request:something
    request := something.
!

responseAlreadySent:something
    responseAlreadySent := something.
! !

!HTTPResponse methodsFor:'actions'!

addCookie:aCookie
    self cookies add:aCookie
! !

!HTTPResponse methodsFor:'header access'!

addLineToHeader:aString 
    header addLineToHeader:aString
!

cacheControl
    ^ header cacheControl.
!

contentDisposition:aContentDispositionString filename:aFileNameString
    header contentDisposition:aContentDispositionString filename:aFileNameString
!

contentLength
    ^ header contentLength
!

contentLength:aNumber
    header contentLength:aNumber
!

contentLocation
    ^ header contentLocation
!

contentLocation:aUrl
    header contentLocation:aUrl
!

contentType
    ^ header contentType
!

contentType:aContentType
    header contentType:aContentType
!

contentType:aContentType contentLength:aNumber expirationTime:aTimeStamp
    header contentType:aContentType contentLength:aNumber expirationTime:aTimeStamp
!

expirationTime:aTimeStamp
    "the responses expirationTimestamp; consider to use maxAge:"

    header expirationTime:aTimeStamp
!

lastModified:aNumber
    header lastModified:aNumber
!

location:aUrl
    header location:aUrl
!

maxAge:aSecondsIntegerOrTimeDuration
    "how many seconds may this response be cached"

    header maxAge:aSecondsIntegerOrTimeDuration.
!

replyHeader:aCompleteHeaderString
    "some services (e.g. comanche services) generates his own response header 
     send this header if set from service"

    header externalHeaderString:aCompleteHeaderString.
!

responseCode
    ^ header responseCode
!

responseCode:aNumber
    header responseCode:aNumber
! !

!HTTPResponse methodsFor:'html reply composition'!

nextPutAll:aString
    replyPresenterDataStream nextPutAll:aString
!

nextPutAllEscaped:aString
    "append aString to the collected reply.
     The string is processed, by replacing html-special characters by there html-equivalent (such as '>' into &lt;)"

    self replyEscaped:aString
!

nextPutLine:aString
    replyPresenterDataStream nextPutLine:aString
!

nextPutLineEscaped:aString
    "append aString to the collected reply and append a cr.
     The string is processed, by replacing html-special characters by there html-equivalent (such as '>' into &lt;)"

    self replyEscaped:aString; replyCR
!

reply:aString
    replyPresenterDataStream nextPutAll:aString
!

replyCR
    replyPresenterDataStream cr
!

replyEscaped:aStringOrCharacter
    "collect part of a reply (i.e. append aString to the reply buffer).
     Special character (lt, gt) are escaped."

    self reply:(HTMLUtilities escapeCharacterEntities:aStringOrCharacter)

    "Modified: / 13-04-2011 / 23:10:54 / cg"
! !

!HTTPResponse methodsFor:'initialization'!

initialize
    replyPresenterDataStream := WriteStream on:''.
    self replyPresenter:(request server serverReplyPresenterFor:request).
    header := HTTPResponseHeader response:self.
    responseAlreadySent := false.
!

resetResponseData
    "start anew collecting response data 
     (i.e. throw away what has been already collected)"

    dataStream notNil ifTrue:[
        dataStream reset
    ].
    replyPresenterDataStream notNil ifTrue:[
        replyPresenterDataStream reset.
    ].
! !

!HTTPResponse methodsFor:'old html reply composition'!

bodyEnd
    "Write </BODY>"

    self reply:'</body>'
!

bodyStart
    "Write <BODY>"

    "OLD stuff - please use a TreeBuilder"

    self reply:'<body>'

    "Modified: / 02-06-2011 / 13:01:52 / cg"
!

cr
    "OLD stuff - please use a TreeBuilder"

    replyPresenterDataStream cr.

    "Modified: / 02-06-2011 / 13:01:57 / cg"
!

formEnd
    "append the standard footer for a form."

    "OLD stuff - please use a TreeBuilder"

    self reply:'</form>'

    "Modified: / 02-06-2011 / 13:02:00 / cg"
!

formFooter
    "Write the standard footer for a form."

    "OLD stuff - please use a TreeBuilder"

    self 
        formResetButton;
        reply:'<br>';
        formSubmitButton;
        formEnd

    "Modified: / 02-06-2011 / 13:02:03 / cg"
!

formResetButton
    "generate a reset button for a form."

    "OLD stuff - please use a TreeBuilder"

    self reply:'<input type="reset" value="Reset">'

    "Modified: / 02-06-2011 / 13:02:06 / cg"
!

formSubmitButton
    "OLD stuff - please use a TreeBuilder"

    self formSubmitButtonForString:'OK'

    "Modified: / 02-06-2011 / 13:02:10 / cg"
!

formSubmitButtonForString:aString
    "OLD stuff - please use a TreeBuilder"

    self reply:'<INPUT type="submit" value="', aString, '">'

    "Modified: / 02-06-2011 / 13:02:13 / cg"
!

formSubmitButtonForString:aString name:aName
    "OLD stuff - please use a TreeBuilder"

    self reply:(self formSubmitButtonStringForString:aString name:aName).

    "Modified: / 02-06-2011 / 13:02:16 / cg"
!

formSubmitButtonStringForString:aString name:aName
    "OLD stuff - please use a TreeBuilder"

    ^ '<INPUT type="submit" value="', aString, '" name="', aName,'">'

    "Modified: / 02-06-2011 / 13:02:20 / cg"
!

htmlEnd 
    "OLD stuff - please use a TreeBuilder"

    self reply:'</html>'

    "Modified: / 02-06-2011 / 13:02:23 / cg"
!

htmlStart 
    "OLD stuff - please use a TreeBuilder"

    self reply:'<html>'

    "Modified: / 02-06-2011 / 13:02:27 / cg"
!

imageFor:anImageOrImageFilename
    "generate an IMG tag for a real image (or an image file name)"

    "OLD stuff - please use a TreeBuilder"

    self imageFor:anImageOrImageFilename alt:nil

    "Modified: / 02-06-2011 / 13:02:31 / cg"
!

imageFor:anImageOrImageFilename alt:alt
    "an IMG tag for a real image (or an image file name)"

    "OLD stuff - please use a TreeBuilder"

    self nextPutLine:(self imageLinkTextFor:anImageOrImageFilename alt:alt).

    "Modified: / 02-06-2011 / 13:02:35 / cg"
!

imageLinkTextFor:anImageOrImageFilename alt:alt
    "an IMG tag for a real image (or an image file name)"

    "OLD stuff - please use a TreeBuilder"

    |h link|

    link := request server graphicLinkIdFor:anImageOrImageFilename.
    h := '<img src="' , link , '"'.
    alt notNil ifTrue:[
         h := h , ' alt="' , alt , '">'
    ].
    ^ h

    "Modified: / 02-06-2011 / 13:02:38 / cg"
!

inputFieldFor:varName label:label initialValue:value
    "an input field - must be within a form"

    "OLD stuff - please use a TreeBuilder"

    self inputFieldFor:varName size:nil label:label initialValue:value

    "Modified: / 02-06-2011 / 13:02:43 / cg"
!

inputFieldFor:varName size:size label:label initialValue:value
    "an input field - must be within a form"

    "OLD stuff - please use a TreeBuilder"

    self inputFieldFor:varName type:nil size:size label:label initialValue:value

    "Modified: / 02-06-2011 / 13:02:47 / cg"
!

inputFieldFor:varName type:aType size:size label:label initialValue:value
    "an input field - must be within a form"

    "OLD stuff - please use a TreeBuilder"

    self inputFieldFor:varName type:aType size:size label:label initialValue:value append:nil

    "Modified: / 02-06-2011 / 13:02:50 / cg"
!

inputFieldFor:varName type:aType size:size label:label initialValue:value append:aString
    "an input field - must be within a form.
     type: 'password' | 'checkbox' | 'submit' | 'reset' | nil
    "

    "OLD stuff - please use a TreeBuilder"

    self reply:(self inputFieldStringFor:varName type:aType size:size label:label initialValue:value append:aString).

    "Modified: / 02-06-2011 / 13:02:57 / cg"
!

inputFieldStringFor:varName type:aType size:size label:label initialValue:value append:aString
    "an input field - must be within a form.
     type: 'password' | 'checkbox' | 'submit' | 'reset' | nil
    "
    "OLD stuff - please use a TreeBuilder"

    |stream initialValue|

    stream := WriteStream on:''.
    stream
        nextPutAll:(label ? '');
        nextPutAll:'<input name="';
        nextPutAll:varName;
        nextPutAll:'" '.
        aType notNil ifTrue:[
            stream nextPutAll:'TYPE=';
                 nextPutAll:aType.
        ].       
        stream nextPutAll:(size notNil ifTrue:[' size="' , size printString, '"'] ifFalse:['']);
            nextPutAll:' value="'.
            value isNil ifTrue:[
                initialValue := ''.
            ] ifFalse:[
                initialValue := (HTMLUtilities escapeCharacterEntities:value).
            ].
            stream nextPutAll:initialValue;
            nextPutAll:'"'.
        aString notNil ifTrue:[
            stream nextPutAll:(' ', aString, ' ').
        ].
        stream nextPutAll:'>'.
        stream nextPutAll:'</input>'.
    ^ stream contents

    "Modified: / 02-06-2011 / 13:03:00 / cg"
!

label: labelString id: idString value: anObject
    "Add a table row for the given data to the reply."

    "OLD stuff - please use a TreeBuilder"

    self reply: '<tr><td><b>', labelString, '</b></td><td><input name="', idString, '"'.
    (anObject isKindOf: Boolean) ifTrue:[
        self reply: ' type="checkbox" value="true"'.
        anObject ifTrue: [ self reply: ' checked' ]
    ] ifFalse: [
        self reply: ' size="30"'.
        anObject isNil ifFalse: [ self reply: ' value="', anObject asString, '"' ]].
    self reply: '>'

    "Modified: / 02-06-2011 / 13:03:04 / cg"
!

label: labelString text: aString id: idString rows: rows cols: cols
    "Build a textarea form field."

    "OLD stuff - please use a TreeBuilder"

    self reply:
            '<tr><td colspan="2"><b>', labelString, '</b><br>'.
    self reply:
            '<textarea wrap="soft" name="', idString,
            '" rows="', rows asString, '" cols="', cols asString, '">'.
    aString notNil ifTrue: [ 
        self reply: aString 
    ].
    self reply: '</textarea>'

    "Modified: / 02-06-2011 / 13:03:07 / cg"
!

pageFinish
    "finish the page."

    "OLD stuff - please use a TreeBuilder"

    self bodyEnd.
    self htmlEnd.

    "Modified: / 02-06-2011 / 13:03:36 / cg"
!

refreshStringFor:aService selector:aSelector arguments:arguments time:seconds
    "OLD stuff - please use a TreeBuilder"

    ^ '<META http-equiv=refresh content="',
        seconds asString,
        '; url=', 
        (request linkIdFor:aService selector:aSelector argument:arguments),
        '">'

    "Modified: / 02-06-2011 / 13:03:41 / cg"
!

refreshStringFor:aService selector:aSelector time:seconds
    "OLD stuff - please use a TreeBuilder"
    ^ self refreshStringFor:aService selector:aSelector arguments:nil time:seconds

    "Modified: / 02-06-2011 / 13:03:45 / cg"
!

stxPoweredByString
    "OLD stuff - please use a TreeBuilder"

    ^ self stxPoweredByStringForService:nil

    "Modified: / 02-06-2011 / 13:03:58 / cg"
!

stxPoweredByStringForService:serviceOrNil
    "OLD stuff - please use a TreeBuilder"

    |host dom|

    host := OperatingSystem getHostName.
    (host includes:$.) ifFalse:[
        dom := OperatingSystem getDomainName.
        (dom notEmptyOrNil and:[dom ~= '(none)']) ifTrue:[
            host := host , '.' , dom
        ].
    ].

    ^ String streamContents:[:s |    
        s 
            nextPutAll:'ST/X '; nextPutAll:(Smalltalk versionString); 
            nextPutAll: '; WebServer '; nextPutAll:(HTTPServer revision printString).
        serviceOrNil notNil ifTrue:[
            s 
                nextPutAll: '; ', serviceOrNil class nameWithoutPrefix , ' '; nextPutAll:(serviceOrNil class revision printString).
        ].
        s 
            nextPutAll:' at '; nextPutAll: host; nextPutAll: ':'; nextPutAll: (request connection socket port printString);
            nextPutAll:'; '; 
            nextPutAll:(Timestamp now printStringRFC1123Format).
    ]

    "Modified: / 02-06-2011 / 13:04:04 / cg"
!

table:aBlock
    "OLD stuff - please use a TreeBuilder"

    self table:aBlock attributes:nil

    "Modified: / 02-06-2011 / 13:04:08 / cg"
!

table:aBlock attributes:attrString
    "OLD stuff - please use a TreeBuilder"

    self tableStart:attrString.
    aBlock value.
    self tableEnd.

    "Modified: / 02-06-2011 / 13:04:14 / cg"
!

tableEnd
    "OLD stuff - please use a TreeBuilder"

    self nextPutAll:'</table>'.

    "Modified: / 02-06-2011 / 13:04:18 / cg"
!

tableFormFooter
    "Close the table and write the standard footer for a form."

    "OLD stuff - please use a TreeBuilder"

    self reply: '
</table>
<hr>
<input type=reset value="Reset"> 
<input type=submit value="OK">
</form>
'

    "Modified: / 02-06-2011 / 13:04:21 / cg"
!

tableRow:aBlock
    "OLD stuff - please use a TreeBuilder"

    self tableRow:aBlock attributes:nil

    "Modified: / 02-06-2011 / 13:04:25 / cg"
!

tableRow:aBlock attributes:attrString
    "OLD stuff - please use a TreeBuilder"

    self tableRowStart:attrString.
    aBlock value.
    self tableRowEnd.

    "Modified: / 02-06-2011 / 13:04:29 / cg"
!

tableRowEnd
    "OLD stuff - please use a TreeBuilder"

    self nextPutAll:'</tr>'.

    "Modified: / 02-06-2011 / 13:04:33 / cg"
!

tableRowStart:attrString
    "OLD stuff - please use a TreeBuilder"

    self nextPutAll:'<tr'.
    attrString notNil ifTrue:[
        self nextPutAll:' '; nextPutAll:attrString.
    ].
    self nextPutLine:'>'.

    "Modified: / 02-06-2011 / 13:04:36 / cg"
!

tableStart:attrString
    "OLD stuff - please use a TreeBuilder"

    self nextPutAll:'<table'.
    attrString notNil ifTrue:[
        self nextPutAll:' '; nextPutAll:attrString.
    ].
    self nextPutLine:'>'.

    "Modified: / 02-06-2011 / 13:04:39 / cg"
!

unformattedCodeTextEnd
    "end some plainText."

    "OLD stuff - please use a TreeBuilder"

    self reply: '</code></pre>'

    "Modified: / 02-06-2011 / 13:04:45 / cg"
!

unformattedCodeTextStart
    "start some plainText."

    "OLD stuff - please use a TreeBuilder"

    self reply: '<pre><code>'

    "Modified: / 02-06-2011 / 13:04:48 / cg"
!

unformattedText:aString
    "add some plainText."

    "OLD stuff - please use a TreeBuilder"

    self 
        unformattedTextStart;
        reply: aString;
        unformattedTextEnd

    "Modified: / 02-06-2011 / 13:04:51 / cg"
!

unformattedTextEnd
    "end some plainText."

    "OLD stuff - please use a TreeBuilder"

    self reply: '</pre>'

    "Modified: / 02-06-2011 / 13:04:56 / cg"
!

unformattedTextStart
    "start some plainText."

    "OLD stuff - please use a TreeBuilder"

    self reply: '<pre>'

    "Modified: / 02-06-2011 / 13:04:59 / cg"
! !

!HTTPResponse methodsFor:'old html-reply-composition-stx-obsolete'!

favIconLinkForSTX
    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    ^ '<link rel="SHORTCUT ICON" href="/files/favicon.ico">'
!

pageSTXFooter
    "Write the standard page footer."

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self 
        stxReply:(
    '<p>
    </blockquote>
  </td>
  </tr> <!!-- end of body table row -->
  <tr>
    <td width="150" bgcolor=%(leftBarColor) valign="top"><br>
    </td>
    <td width="100%" valign="top" bgcolor=%(bodyColor)>
    </td>
  </tr>
  <tr>
    <td width="150" bgcolor="white" xxbgcolor=%(leftBarColor) valign="top"><br>
    </td>
    <td width="100%" valign="top" bgcolor=%(bottomRightColor)>
&nbsp;&nbsp;&nbsp;
    <center><small><font color=%(bottomTextColor)><ADDRESS>'
, self stxPoweredByString 
, '
    </ADDRESS></font></small></center>
    <br>
    </td>
  </tr>
</table>
')

    "Modified: / 30.7.1999 / 13:59:28 / cg"
!

stxBodyEnd 
    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self pageSTXFooter.   
    self stxReply:
'</body>
'
!

stxBodyStart:aString 
    "Write a standard stx body with a headline #f6feff"

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self stxBodyStart:aString homeLink:'/'
!

stxBodyStart:aString homeLink:aHomeLink
    "Write a standard stx body with a headline #f6feff"

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self stxBodyStart:aString homeLink:aHomeLink afterButtons:nil
!

stxBodyStart:aString homeLink:aHomeLink afterButtons:afterButtonsString
    "Write a standard stx body with a headline #f6feff"

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self 
        stxReply:(
'<body bgcolor=%(bodyColor) text=%(textColor) link=%(linkColor) alink=%(activeLinkColor) vlink=%(visitedLinkColor) leftmargin=0 topmargin=0>
<table border="0" bgcolor=%(topLeftColor) cellspacing="0" cellpadding="0" width="100%" height="100%">
<tr>
    <td width="150" valign="top" bgcolor=%(topLeftColor)><br><img src="/files/exeptLogo.gif" alt="Exept Logo"></td>
    <td width="100%" bgcolor=%(topRightColor) valign=bottom>
    <blockquote><h2><font color=%(topTextColor)>' 
, 'Smalltalk/X Webserver' , 
'</font></h2>
<p><h4><font color=%(topTextColor)>' , 
aString , 
'</font></h4></BLOCKQUOTE>
    </td>
</tr>
<tr>
    <td height="100%" width="150" bgcolor=%(leftBarColor) valign="top"><br>
').

    (request url ~= aHomeLink) ifTrue:[
        self 
            reply:(
' <center><a HREF="', aHomeLink, '" target="_top" ><img SRC="/files/top_mask.gif" ALT="Home" border="0" width="130" height="80"></A></center>').
    ] ifFalse:[
        self 
            reply:(
' <center><img SRC="/files/topie_mask.gif" ALT="Home" border="0" width="130" height="80"></center>').
    ].
    afterButtonsString notNil ifTrue:[
        self stxReply:afterButtonsString.
    ].
    self 
        stxReply:('
</td>
<td height="100%" width="100%" valign="top" bgcolor=%(bodyColor)>
<blockquote>
<p>')
!

stxPageFinish
    "finish the page."

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self stxBodyEnd.
    self htmlEnd.
!

stxPageParameters
    <resource: #obsolete>
    |params|

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    DefaultSTXPageParameters isNil ifTrue:[
        params := Dictionary new.
        params at:#bodyColor put:'"#f6feff"'.
        params at:#topLeftColor put:'"#00464A"'.
        params at:#topTextColor put:'"#00464A"'.
        params at:#bottomTextColor put:'"#00464A"'.
        params at:#textColor put:'"#000000"'.
        params at:#topRightColor put:'"#e4f5fe"'.
        params at:#bottomRightColor put:'"#e4f5fe"'.
        params at:#leftBarColor put:'"#b6d2da"'.
        params at:#linkColor put:'"#0000cc"'.
        params at:#visitedLinkColor put:'"#551a8b"'.
        params at:#activeLinkColor put:'"#ff0000"'.
        DefaultSTXPageParameters := params.
    ].
    ^ DefaultSTXPageParameters
!

stxReply:aString 
    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self reply:(aString expandPlaceholdersWith:self stxPageParameters)
!

title:aString
    "Write a standard header and set the document title. Also used for logging purposes."

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    ^ self title:aString with:''
!

title:aString with:someMoreHeaderStuff
    "Write a standard header and set the document title. Also used for logging purposes."

    <resource: #obsolete>

"/    self obsoleteMethodWarning:'use a reply presenter - see ExeptReplyPresenter'.
    self 
        nextPutAll: 
'<!!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 3.0//EN">
<html>
<head>
';
        nextPutLine:(self favIconLinkForSTX).

    someMoreHeaderStuff notEmptyOrNil ifTrue:[
        self nextPutLine:someMoreHeaderStuff.
    ].

    self 
        nextPutAll:'<title>'; nextPutAll:aString; 
        nextPutAll:
'</title>
</head>'.
!

titleAndHeader:aString
    "Write a standard header and set the document title. Also used for logging purposes."

    <resource: #obsolete>

    self
        title:aString;
        nextPutAll: '<h1>';
        nextPutAll: aString;
        nextPutLine: '</h1>'.
! !

!HTTPResponse methodsFor:'queries'!

isErrorResponse
    ^ self header hasErrorStatusCode
!

startReadRequestTime
    ^ request startReadRequestTime
! !

!HTTPResponse methodsFor:'sending'!

send
    "copy over from some stream and send it as a response.
     Any already collected replyData is ignored.
     Typically used with big files, which we do not want to fill up our memory
     (i.e. the document is not loaded as a whole)"

    |chunkSize connection socket buffer responseStream headerLength contentLength n data|

    chunkSize := 8*1024.
    responseAlreadySent ifTrue:[^ self].

    connection := request connection.
    socket := connection socket.
    socket isNil ifTrue: [^ self].

    header hasErrorStatusCode ifTrue:[
        "set keepConnection early, so that we can report this to the client"
        connection keepConnection:(request server closeConnectionAfterError not).
    ].

    dataStream isNil ifTrue:[
        "/ if nobody is setting explicit data 
        "/ the data is coming from the replyPresenter
        "/ default is the server presenter if not overwritten by service
        dataStream := ReadWriteStream on:(String new: 1024).
        replyPresenter present:self on:dataStream.
        dataStream reset.
    ].
    (contentLength := header contentLength) isNil ifTrue:[
        "/ set header content length if not done before
        contentLength := dataStream size.
        header contentLength:contentLength.
    ].

    buffer := String uninitializedNew:(8*1024).
    responseStream := WriteStream on:buffer.
"/    MessageTally spyDetailedOn:[
"/        100 timesRepeat:[
            header generateResponseOn:responseStream.
"/            Transcript showCR:request url.
"/            Transcript showCR:responseStream contents.
"/            Transcript showCR:'--------------------'.
"/        ]
"/    ].
"/ (buffer copyReplaceAll:Character return with:Character space)

    request server verbose ifTrue:[
        Transcript 
            showCR:'>-- Reply (fd=', socket fileDescriptor printString, ') -- to ', (request url printString);
            show:(responseStream contents copyReplaceAll:Character return with:Character space).

        request server verboseData ifTrue:[
            Transcript showCR:'.....................'.
            dataStream isExternalStream ifTrue:[
                Transcript showCR:'... reading from stream...'.
            ] ifFalse:[
                Transcript showCR:(dataStream contents printString)
            ]
        ].
        Transcript showCR:'>--------------------'.
    ].

    headerLength := responseStream size.

    request isHEAD ifTrue:[
        n := socket nextPutBytes:headerLength from:(responseStream contents) startingAt:1.
        n ~~ headerLength ifTrue:[
            socket writeError.
        ].
    ] ifFalse:[
        "try hard to send it as one block"
        (headerLength + contentLength) <= chunkSize ifTrue:[
            dataStream copyToEndInto:responseStream bufferSize:chunkSize.
            "/ data := dataStream contents.
            "/ data notNil ifTrue:[ responseStream nextPutAll:data ].
            n := socket nextPutBytes:(headerLength + contentLength) from:(responseStream contents) startingAt:1.
            n ~~ (headerLength + contentLength) ifTrue:[
                socket writeError.
            ].
        ] ifFalse:[
            n := socket nextPutBytes:headerLength from:(responseStream contents) startingAt:1.
            n ~~ headerLength ifTrue:[
                socket writeError.
            ].
            socket binary.
            dataStream copyToEndInto:socket bufferSize:chunkSize.
            socket text.
        ].
    ].
    dataStream close.

"/    MessageTally spyDetailedOn:[
"/        100 timesRepeat:[
            request logAccess.
"/        ]
"/    ].
    responseAlreadySent := true.

    "Modified: / 29-08-2007 / 00:48:22 / cg"
!

sendBinaryData:binaryData type:contentType 
    "send some binary data as a response. 
     Typically used with images and non-text documents"

    self
        binaryData:binaryData contentType:contentType;
        send.
! !

!HTTPResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPResponse.st,v 1.95 2011/11/14 17:59:55 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPResponse.st,v 1.95 2011/11/14 17:59:55 stefan Exp $'
! !
