"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPResponseHeader
	instanceVariableNames:'response responseCode contentType contentLength expirationTime
		lastModified cacheControl additionalHeaderLines
		externalHeaderString contentLocation location headerParser
		httpVersion expectedAuthRealm contentDisposition filename'
	classVariableNames:'ReasonPhrasesDictionary'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

Object subclass:#CacheControl
	instanceVariableNames:'cachable store transform maxAge sMaxAge mustRevalidate
		proxyRevalidate cacheDirective'
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPResponseHeader
!

!HTTPResponseHeader class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    collects the fields of an HTTP response header
"
! !

!HTTPResponseHeader class methodsFor:'instance creation'!

httpOKHeaderForResponse:aResponse 
    ^ self responseCode:200 response:aResponse
!

response:aResponse 
    ^ (self new)
        response:aResponse.
!

responseCode:aCode response:aResponse 
    ^ (self new)
        responseCode:aCode;
        response:aResponse.
!

statusCode:aCode response:aResponse
    <resource: #obsolete>
 
    self obsoleteMethodWarning:'use #responseCode:response:'.
    ^ self responseCode:aCode response:aResponse
! !

!HTTPResponseHeader class methodsFor:'constants'!

reasonPhrasesDictionary
    "returns a dictionary which maps HTTP-codes to a descriptive message text"

    ReasonPhrasesDictionary isNil ifTrue:[
        ReasonPhrasesDictionary := Dictionary new.
        #(
            100   'Continue'
            101   'Switching Protocols'
            200   'OK'
            201   'Created'
            202   'Accepted'
            203   'Non-Authoritative Information'
            204   'No Content'
            205   'Reset Content'
            206   'Partial Content'
            300   'Multiple Choices'
            301   'Moved Permanently'
            302   'Found'
            303   'See Other'
            304   'Not Modified'
            305   'Use Proxy'
            307   'Temporary Redirect'
            400   'Bad Request'
            401   'Unauthorized'
            402   'Payment Required'
            403   'Forbidden'
            404   'Not Found'
            405   'Method Not Allowed'
            406   'Not Acceptable'
            407   'Proxy Authentication Required'
            408   'Request Time-out'
            409   'Conflict'
            410   'Gone'
            411   'Length Required'
            412   'Precondition Failed'
            413   'Request Entity Too Large'
            414   'Request-URI Too Large'
            415   'Unsupported Media Type'
            416   'Requested range not satisfiable'
            417   'Expectation Failed'
            500   'Internal Server Error'
            501   'Not Implemented'
            502   'Bad Gateway'
            503   'Service Unavailable'
            504   'Gateway Time-out'
            505   'HTTP Version not supported'
            902   'Unknown host'
        ) pairWiseDo:[:aStatusCode :aReasonPhrase|
            ReasonPhrasesDictionary at:aStatusCode put:aReasonPhrase
        ]
    ].  
    ^ ReasonPhrasesDictionary 

"
    self reasonPhrasesDictionary
"
! !

!HTTPResponseHeader methodsFor:'* uncommented (%1) *'!

maxAge:aSecondsIntegerOrTimeDuration
    "how many seconds may this response be cached"

    self cacheControl maxAge:aSecondsIntegerOrTimeDuration.

    "keep the following for compatibility with older (HTTP 1.0) clients.
     However, or newer clients, cacheControl takes precedence.

     SV 2011-11 nobody cares about HTTP 1.0 nowadays"

"/    header expirationTime:(Timestamp now addSeconds:seconds).
! !

!HTTPResponseHeader methodsFor:'accessing'!

additionalHeaderLines
    additionalHeaderLines isNil ifTrue:[
        additionalHeaderLines := OrderedCollection new.
    ].
    ^ additionalHeaderLines
!

cacheControl
    cacheControl isNil ifTrue:[
        cacheControl := CacheControl new.
    ].
    ^ cacheControl
!

contentDisposition:aContentDispositionString filename:aFileNameString
    contentDisposition := aContentDispositionString.
    filename := aFileNameString
!

contentLength
    "contentLength from header as a number, if the header is set by a service as a string
     the contentLength field value will be parsed from the header"
    |value|

    (contentLength isNil and:[ externalHeaderString notEmptyOrNil ]) ifTrue:[
        value := headerParser fieldValueFor:'content-length'.
        value notNil ifTrue:[
            contentLength := value asNumber
        ].
    ].
    ^ contentLength
!

contentLength:something
    contentLength := something.
!

contentLocation
    ^ contentLocation
!

contentLocation:something
    contentLocation := something.
!

contentType
    "answer the request's content-type.
     Since we currently use 1-byte strings, our default character set is ISO8859-1.
     This may change to UTF-8 some day"

    contentType isNil ifTrue:[
        externalHeaderString notEmptyOrNil ifTrue:[
            contentType := headerParser fieldValueFor:'content-type'.
        ].
        contentType isNil ifTrue:[
            contentType := 'text/html; charset=ISO-8859-1'.
        ].
    ].
    ^ contentType
!

contentType:something
    contentType := something.
!

contentType:aContentTypeArg contentLength:contentLengthArg expirationTime:expirationTimestampArg
    contentType := aContentTypeArg.
    contentLength := contentLengthArg.
    expirationTime := expirationTimestampArg
!

expectedAuthRealm:something
    expectedAuthRealm := something.
!

expirationTime
    (expirationTime isNil and:[ externalHeaderString notEmptyOrNil ]) ifTrue:[
        expirationTime := headerParser fieldValueFor:'expires'.
    ].
    ^ expirationTime
!

expirationTime:expirationTimestamp
    "the responses expirationTimestamp; consider to use maxAge:"

    expirationTime := expirationTimestamp.
!

externalHeaderString:aCompleteHeaderString
    externalHeaderString := aCompleteHeaderString.
    (externalHeaderString endsWith:String crlf) ifFalse:[
        externalHeaderString := externalHeaderString , HTTPServer crlf.
    ].
    headerParser := HTTPHeaderParser headerString:externalHeaderString.
    "parse the first header line anytime -  but the following fields lazy"
    self parseFirstLine.
!

httpVersion
    ^ httpVersion
!

httpVersion:something
    httpVersion := something.
!

lastModified
    "the last modified time of a page - typically a file"

    ^ lastModified
!

lastModified:lastModifiedTimestamp
    "the last modified time of a page - typically a file"

    lastModified := lastModifiedTimestamp.
!

location
    ^ location
!

location:something
    "Used with a 3xx moved response."

    location := something.
!

parseFirstLine
    "parse the first line of a HTML response string"

    |firstLine tokens|

    firstLine := headerParser startLine.
    tokens := firstLine asCollectionOfWords.
    tokens size < 3 ifTrue:[
        self error:'Bad Response Header Format'.
    ].
    self httpVersion:(tokens at:1).
    self responseCode:(tokens at:2) asInteger.
!

response
    ^ response
!

response:something
    response := something.
!

responseCode
    ^ responseCode
!

responseCode:something
    responseCode := something.
! !

!HTTPResponseHeader methodsFor:'actions'!

addLineToHeader:aString 
    " add a crlf to the string if absent "
    |headerLine crlf|

    headerLine := aString.
    crlf := String crlf.
    (headerLine endsWith:crlf) ifFalse:[
        headerLine := headerLine , crlf.
    ].
    self additionalHeaderLines add:headerLine
!

redirectTo:targetUrl 
    self responseCode:301.
    self location:targetUrl
! !

!HTTPResponseHeader methodsFor:'queries'!

hasErrorStatusCode
    "answer true, if this is an error response"

    ^ responseCode notNil and:[responseCode >= 400]
!

isBinaryContent
    ^ (self contentType startsWith:'text/') not
! !

!HTTPResponseHeader methodsFor:'sending'!

generateResponseOn:headerStream 
    |request responseCookies|

    response isNil ifTrue:[
        self error:'need a response to generate my sendString'.
    ].
    externalHeaderString notEmptyOrNil ifTrue:[
        (headerParser fieldValueFor:'content-length') isNil ifTrue:[
            "must add content-length because external header string does not have one added 
            (com swiki in some cases)"
            headerStream
                nextPutAll:(externalHeaderString copyTo:(externalHeaderString size - 2));
                nextPutAll:'Content-Length: '.
            contentLength printOn:headerStream.
            headerStream
                crlf;
                crlf.
        ] ifFalse:[
            headerStream nextPutAll:externalHeaderString.
        ].
        ^ self.
    ].
    self generateResponseStatusOn:headerStream.
    request := response request.
    location notNil ifTrue:[
        headerStream nextPutAll:'Location: '.
        location printOn:headerStream.
        headerStream crlf.
    ].
    headerStream
        nextPutAll:'Content-Type: '; nextPutAll:self contentType;
        crlf.

    contentDisposition notNil ifTrue:[
        headerStream
            nextPutAll:'Content-disposition: '; nextPutAll:contentDisposition;
            nextPutAll:'; filename= '; nextPutAll:filename;
            crlf.
    ].

    self contentLength notNil ifTrue:[
        headerStream nextPutAll:'Content-Length: '.
        contentLength printOn:headerStream.
        headerStream crlf.
    ].
    (request connection isKeepAlive) ifTrue:[
        headerStream nextPutAll:'Connection: Keep-Alive'
    ] ifFalse:[
        headerStream nextPutAll:'Connection: Close'
    ].
    headerStream crlf.
    
    cacheControl notNil ifTrue:[
        "if no explicit cache control requested - trust the default behavior of the browser"
        cacheControl printOn:headerStream.
        headerStream crlf.
    ].
    expirationTime notNil ifTrue:[
        "date only necessary for Expires: header"
        headerStream nextPutAll:'Date: '.
        response startReadRequestTime printRFC1123FormatOn:headerStream.
        headerStream crlf.
        headerStream nextPutAll:'Expires: '.
        expirationTime printRFC1123FormatOn:headerStream.
        headerStream crlf.
    ].
    lastModified notNil ifTrue:[
        headerStream nextPutAll:'Last-modified: '.
        lastModified printRFC1123FormatOn:headerStream.
        headerStream crlf.
    ].
    contentLocation notNil ifTrue:[
        headerStream
            nextPutAll:'Content-Location: '; nextPutAll:contentLocation;
            crlf.
    ].
    (responseCookies := response cookies) notEmptyOrNil ifTrue:[
        responseCookies do:[:eachCookie | 
            headerStream
                nextPutAll:'Set-Cookie: ';
                nextPutAll:eachCookie htmlString;
                crlf.
        ].
    ].
    self additionalHeaderLines do:[:aLine | 
        headerStream nextPutAll:aLine.
    ].
    headerStream crlf.
!

generateResponseStatusOn:headerStream
    |request reasonPhrase httpVersionSentBack|

    responseCode isNil ifTrue:[responseCode := 200].
    request := response request.
    httpVersionSentBack := httpVersion.
    httpVersionSentBack isNil ifTrue:[httpVersionSentBack := request httpVersion ].
    httpVersionSentBack isNil ifTrue:[
        httpVersionSentBack := request server class httpVersion 
    ].

    reasonPhrase := self class reasonPhrasesDictionary at:responseCode ifAbsent:nil.

    headerStream 
        nextPutAll:'HTTP/'; nextPutAll:httpVersionSentBack; nextPutAll:' '.
    responseCode printOn:headerStream.

    reasonPhrase notNil ifTrue:[
        headerStream space; nextPutAll:reasonPhrase; crlf.
    ].

    responseCode printString = '301' ifTrue:[
        headerStream 
            nextPutAll:'Status:'; nextPutAll:' '.
        responseCode printOn:headerStream.

        reasonPhrase notNil ifTrue:[
            headerStream space; nextPutAll:reasonPhrase; crlf.
        ].
    ].
! !

!HTTPResponseHeader::CacheControl class methodsFor:'constants'!

documentation
"
    cache-request-directive =
          no-cache
          no-store
          max-age = delta-seconds
          max-stale [ = delta-seconds ]
          min-fresh = delta-seconds
          no-transform
          only-if-cached
          cache-extension                   
    cache-response-directive
          public                               
          private [ = <'> 1#field-name <'> ] 
          no-cache [ = <'> 1#field-name <'> ]
          no-store                             
          no-transform
          must-revalidate
          proxy-revalidate
          max-age = delta-seconds           
          s-maxage = delta-seconds          
          cache-extension
"
! !

!HTTPResponseHeader::CacheControl methodsFor:'accessing'!

cacheDirective
    ^ cacheDirective
!

cacheDirective:something
    cacheDirective := something.
!

maxAge
    ^ maxAge
!

maxAge:aSecondsIntegerOrTimeDuration
    aSecondsIntegerOrTimeDuration isInteger ifTrue:[
        maxAge := aSecondsIntegerOrTimeDuration.
    ] ifFalse:[
        maxAge := aSecondsIntegerOrTimeDuration getSeconds.
    ].
!

mustRevalidate
    ^ mustRevalidate
!

mustRevalidate:aBoolean
    mustRevalidate := aBoolean.
!

proxyRevalidate
    ^ proxyRevalidate
!

proxyRevalidate:aBoolean
    proxyRevalidate := aBoolean.
!

sMaxAge
    ^ sMaxAge
!

sMaxAge:anInteger
    sMaxAge := anInteger.
!

setNoCache
    "set the no-cache flag.
     The file will not be cached by the client or intermediate caches.
     Be careful: IE6 denies opening of attachments, if either no-cache or no-store is set"

    cachable := 'no-cache'.
!

setPrivate
    cachable := 'private'.
!

setPublic
    cachable := 'public'.
!

store
    ^ store
!

store:aBoolean
    "set the no-store flag.
     The file will not be stored by the client or intermediate caches.
     Be careful: IE6 denies opening of attachments, if either no-cache or no-store is set"

    store := aBoolean.
!

transform
    ^ transform
!

transform:aBoolean
    transform := aBoolean.
! !

!HTTPResponseHeader::CacheControl methodsFor:'printing'!

printOn:aStream
    "print a cache-control header.
     cacheDirective takes precedence over field values"

    |comma|

    aStream nextPutAll:'Cache-Control: '.
    cacheDirective notEmptyOrNil ifTrue:[
        aStream nextPutAll:cacheDirective.
    ] ifFalse:[
        comma := ''.
        cachable notEmptyOrNil ifTrue:[
            aStream nextPutAll:cachable.
            comma := ', '.
        ].
        store == false ifTrue:[
            aStream
                nextPutAll:comma;
                nextPutAll:'no-store'.
            comma := ', '.
        ].
        transform == false ifTrue:[
            aStream
                nextPutAll:comma;
                nextPutAll:'no-transform'.
            comma := ', '.
        ].
        mustRevalidate == true ifTrue:[
            aStream 
                nextPutAll:comma;
                nextPutAll:'must-revalidate'.
            comma := ', '.
        ].
        proxyRevalidate == true ifTrue:[
            aStream
                nextPutAll:comma;
                nextPutAll:'proxy-revalidate'.
            comma := ', '.
       ].
        maxAge notNil ifTrue:[
            aStream
                nextPutAll:comma;
                nextPutAll:'max-age='.
            maxAge printOn:aStream.
            comma := ', '.
        ].
        sMaxAge notNil ifTrue:[
            aStream
                nextPutAll:comma;
                nextPutAll:'s-maxage='.
            sMaxAge printOn:aStream.
            comma := ', '.
        ].
    ].
! !

!HTTPResponseHeader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPResponseHeader.st,v 1.20 2011/11/14 17:59:02 stefan Exp $'
! !
