"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPService subclass:#HTTPSelectorService
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

!HTTPSelectorService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    A service which takes the next URL component after the linkName as selector
    and sends a corresponding message to itself.

    For security, the set of messages must be enumerated by allowedMessages, and only
    message are send, which are listed there.

    For example, if a service registered itself as 'fooService' on a server which serves port 8088,
    then a request for the URL 'http://hostName:8088/fooService/barMessage' will lead to
    the message #barMessage be sent to the service (if it is listed in #allowedMessages).

    See information on how to start a service in my superclasses.

    Origin:
        this class was added to support easy migration of PWS (pluggable web services) applications.

    [author:]
        Claus Gittinger
"
! !

!HTTPSelectorService class methodsFor:'queries'!

isAbstract
    ^ self == HTTPSelectorService 
! !

!HTTPSelectorService methodsFor:'actions'!

process:aRequest
    |selectorAndArguments selectorSymbol selector arguments|

    selectorAndArguments := (aRequest pathRelativeToService asCollectionOfSubstringsSeparatedByAny:'/\,').
    selectorAndArguments isEmpty ifTrue:[
        selector := self defaultMessage.
        selector isNil ifTrue:[
            aRequest reportNotFound:'Illegal (empty) selector for service ', self className.
            "/ not reached
            ^ self.
        ].
        arguments := #().
    ] ifFalse:[
        selector := selectorAndArguments at:1.
        arguments := selectorAndArguments copyFrom:2.
    ].

    (selector endsWith:$:) ifFalse:[
        selector := selector , ':'.
    ].

    selectorSymbol := selector asSymbolIfInterned.
    (selectorSymbol isNil or:[(self isMessageAllowed:selectorSymbol) not] ) ifTrue:[
        aRequest reportNotFound:'Service ', self className, ' received illegal message ' , selector asString.
        "/ not reached
        ^ self.
    ].

"/    (self respondsTo:selectorSymbol) not ifTrue:[
"/        aRequest reportNotFound:'Service ', self className, ' does not respond to ' , selector asString.
"/        "/ not reached
"/        ^ self.
"/    ].

    "/ remember selector and arguments (for the service)
    aRequest selector:selector arguments:arguments.

    self perform:selectorSymbol with:aRequest
! !

!HTTPSelectorService methodsFor:'queries-service'!

allowedMessages
    "return a collection of messages, that this service processes,
     such as
        #(foo bar baz)"

    ^ self subclassResponsibility
!

defaultMessage
    "return the name of the default message; such as
        #bla or 'index.html'.
     However, the returned string must be in allowedMessages"

    ^ nil
!

isMessageAllowed:aSelector
    "return true, if aSelector is allowed to be sent to me as a web-request.
     The default behavior is, to look in allowedMessages.
     May be redefined in subclasses to do something more intelligent, e.g. pattern matching"

    ^ self allowedMessages includes:aSelector
! !

!HTTPSelectorService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPSelectorService.st,v 1.14 2008/05/27 14:44:47 cg Exp $'
! !
