"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPServer
	instanceVariableNames:'listeningAddresses serverHostName serverSocket serverStatusHolder
		serverProcess serviceList servicesSema linkResolver linkHooks
		homeURL notFoundURL maintenanceMode backupJobs
		maxNumberOfKeepAliveConnections maxNumberOfConnections
		keptConnectionsSema keptConnections verbose verboseData
		verboseErrors verboseTimeConsumtion debugErrors logFile logStream
		loggingSema soapEnabled soapActionHandler realmUsersMap
		allowedAuthMethods settingsFilename allowedHosts forbiddenHosts
		temporaryForbiddenHosts accessLogger startupTime
		inactiveConnectionTimeout serverReplyPresenter httpSessions
		httpSessionsSema urlAliasDictionary logEnabled
		tentativeObjectService overallNumberOfRequests
		overallNumberOfServedConnections'
	classVariableNames:'CrLf CrLfCrLf RunningServers CommonBackupJobs ForbiddenUserAgents
		ForbiddenPeers'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

AbortOperationRequest subclass:#AbortRequestProcessing
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPServer
!

Notification subclass:#TerminationRequest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPServer
!

!HTTPServer class methodsFor:'documentation'!

benchmarking
"
  Please validate these numbers on your machine - the numbers below might be
  outdated by the time you read this ...


  Throughput testing - see how many URL-not found requests are handled per second ...
  try port 80 (default apache1.3 server)
  vs. port 8080 (a Zope HTTPServer).
  vs. port 8081 (ST/X HTTPServer).
                                                                        [exBegin]
    |startTime deltaTime n host port|

    port := Dialog request:'port' initialAnswer:'80'.
    port size==0 ifTrue:[^ self].
    port := Integer readFrom:port.
    host := Dialog request:'host' initialAnswer:(OperatingSystem getHostName).
    host size==0 ifTrue:[^ self].
    5 timesRepeat:[
        startTime := Timestamp now.
        n := 0.
        [(deltaTime := Timestamp now - startTime) < 10] whileTrue:[
            HTTPInterface get:'/foo' fromHost:host port:port.
            n := n + 1
        ].
        Transcript showCR:((n / deltaTime) asFixedPoint:1) printString , ' requests per second'.
    ]
                                                                        [exEnd]

    On our linux box:
        Operating system: [ SuSE Linux 7.1 (i386) ]
        Host: [ Kernel: 2.2.18 (i686) ]

        Apache/1.3.14   166..170
        Zope             50.. 59
        ST/X HTTPServer  57.. 62


  throughput testing - see how many real requests (a small html-file) are
  handled per second ... (here, the apache default index.html was used)
  try port 80 (default apache1.3 server)
  vs. port 8080 (our ST/X HTTPServer).
                                                                        [exBegin]
    |startTime deltaTime n host port|

    port := Dialog request:'port' initialAnswer:'80'.
    port size==0 ifTrue:[^ self].
    port := Integer readFrom:port.
    host := Dialog request:'host' initialAnswer:(OperatingSystem getHostName).
    host size==0 ifTrue:[^ self].
    5 timesRepeat:[
        startTime := Timestamp now.
        n := 0.
        [(deltaTime := Timestamp now - startTime) < 10] whileTrue:[
            HTTPInterface get:'/' fromHost:host port:port.
            n := n + 1
        ].
        Transcript showCR:((n / deltaTime) asFixedPoint:1) printString , ' requests per second'.
    ]
                                                                        [exEnd]

    On our linux box:
        Apache 1.3      85..107  / 87..131 (keepAlive)
        ST/X HTTPServer 71.. 80  / 104..112 (keepAlive)


  throughput testing - see how many generated page requests are
  handled per second ... this can only be done with the ST/X server.
                                                                        [exBegin]
    |startTime deltaTime n host port|

    port := Dialog request:'port' initialAnswer:'8080'.
    port size==0 ifTrue:[^ self].
    port := Integer readFrom:port.
    host := Dialog request:'host' initialAnswer:(OperatingSystem getHostName).
    host size==0 ifTrue:[^ self].
    5 timesRepeat:[
        startTime := Timestamp now.
        n := 0.
        [(deltaTime := Timestamp now - startTime) < 10] whileTrue:[
            HTTPInterface get:'/' fromHost:host port:port.
            n := n + 1
        ].
        Transcript showCR:((n / deltaTime) asFixedPoint:1) printString , ' requests per second'.
    ]
                                                                        [exEnd]

    On our linux box:
        ST/X HTTPServer 69 / 115 (keepAlive)
"
!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Communicate with me via your favourite webBrowser ...
    ... see examples, howToStart and howToConfigure for setup information.

    This class combines the functionalities of our own HTTPServer
    (which itself was a merge of the WebServer and our existing file-access
     HTTPServer), and the new PWS (pluggable web server).

    By setting up the ActionTable, FileRoot and HomeURL as required,
    all of the above Web services (i.e. plugged, dynamic objects and file access)
    are implemented here in a backward compatible fashion.

  Features:
    HTTPServer does file caching - you may want to configure the number of
    cached documents and the maxSize for caching by changing the maxCacheSize
    and maxCacheDocumentSize variables.
    The server also cares for open connections, timeouts, authentication etc.
    Can also operate as slave under a master-apache via FCGI

  Operation Overview:
    For each incoming request, a HTTPConnection is created (or reused) and
    this is asked to handle the request (see #serve). 
    HTTPConnection creates a HTTPRequest object, which is asked to generate a reply (#generateReply).
    For reply, the requests asks me (the httpServer) back, to translate the requests URL
    into an instance of HTTPService and a relative path (see #serviceAndLinkNameFromPath:into:).
    The mapping of relative path to services is done via the linkResolver dictionary, into which
    services can be registered/unregistered either programmatically or via the settings dialog
    (#resolveServiceForPath: and #serviceForLink:ifAbsent:).

    Finally, the service gets the request (#process:) and has to fill out the HTTPResponse object
    which is passed along with the request (either directly or indirectly).
    Eventually, when all intermediate methods return to the #generateReply of the HTTPRequest,
    any collected html code is sent back to the original socket.

    [History:]
        our own HTTPServer              - did file access only.
                                        - supports caching, keep-Alive, connections.
        merged WebServer                - added LinkedObjects
        improvements                    - added temporary links
        merged PWS                      - added an actionTable

    [InstanceVariables:]
        port                            the port, on which this server is listening
        serverSocket                    the socket, on which this server is listening
        serverStatusHolder              a valueHolder holding this servers status (for monitoring)
        serverProcess                   process serving the connections
        serviceList                     list of registered services
        servicesSema                    accessLock
        linkResolver                    maps URLS
        linkHooks                       mappings for individual URLS
        temporaryLinks                  maps objects to temporary names
                                        These links are removed after being
                                        used - i.e. they are one-shot links
        temporaryObjectNames            maps temporary names to objects
                                        These links are removed after being
                                        used - i.e. they are one-shot links
        temporaryObjectNamesSema        accessLock
        temporaryLinksSema              accessLock
        temporaryLinkExpirationTime     the default expiration time for a temporary image
                                        sending with the response so the client can cache this links
                                        server removes that temporary links after that time
        homeURL                         the default URL (default is /Home)
        notFoundURL                     the url invoked if the service for the requested url is not found (default is nil)
        maintenanceMode                 boolean; if true, only local requests or requests includes the authorization for
                                        the ServerMaintenanceService are served
        backupJobs                      blocks to evaluate hourly
        maxNumberOfConnections          the max. number of simultaneous open connections.
        maxNumberOfKeepAliveConnections the max. number of kept connections.
                                        If more, the longest unused connection is closed.
                                        If 0, no connections are kept alive.
                                        (read about the WAIT_FIN TCP problem on
                                         why you may want to do this)
        keptConnectionsSema             accessLock
        keptConnections                 kept, still alive connections
        defaultFile                     obsolete
        verbose                         enables some debug messages (e.g. write header information to the Transcript) (default false)
        verboseData                     enables debug dump of incoming/outgoing data (default false)
        verboseErrors                   enables debug messages for error handling (default false)
        verboseTimeConsumtion           enables request time consumtion printouts to Transcript (default false)
        debugErrors                     enables debugging; if false, errors are cought and no debugger
                                        is opened (instead, the error is reported to the HTTP-requestor).
                                        If true, a regular debugger is opened.(default false)
        logFile                         a log file (will be in apache format soon)
        logToFile                       enables logging ti file
        logStream                       log stream to logFile
        loggingSema                     accessLock for logFile
        soapEnabled                     soap hook
        soapActionHandler
        realmUsersMap                   stores the user:password codes for all the authorization methods for each realm
        allowedAuthMethods              by now DIGEST and BASIC are supported
        settingsFilename                default filename were the server settings can be written to
        allowedHosts                    if non-nil, only those hosts are allowed (list of friends)
        forbiddenHosts                  if non-nil, those hosts are forbidden from connecting (list of intruders)

    [ClassVariables:]
        CrLf                            common constant strings
        CrLfCrLf
        RunningServers  OrderedCollection
                                        list of all running http server instances

    [author:]
        Claus Gittinger and many merged-in pieces from others

    [see also:]
        http://www.w3.org/Protocols/rfc2616/rfc2616.html
        HTTPInterface HTMLBrowser
        HTTPServer::Connection
        HTTPServer::Authorization
        HTTPRequest
        HTTPService
"
!

examples
"
    Notice: st/x must run as setuid or rott in order to allow
    serving port 80 ...
    default setup & go:
                                                                [exBegin]
     HTTPServer startServerOnPort:8080
                                                                [exEnd]

  one more than one port:
                                                                [exBegin]
     HTTPServer startServerOnPort:8081
     HTTPServer startServerOnPort:9090
                                                                [exEnd]
  stop:
                                                                [exBegin]
     HTTPServer stopAllServer
                                                                [exEnd]

  go:
                                                                [exBegin]
     HTTPServer startServerOnPort:8080.
                                                                [exEnd]


                                                                [exBegin]
     HTTPServer startServerOnPort:80.
                                                                [exEnd]
"
!

howToConfigure
"
    Open the new Settings Dialog from Launcher System Settings and switch to the communication/HTTPServer section.

    If you want to have a standalone webServer application type make in projects/webServer.
    Type webServer --help to see the optins for startup.
    The best way is to setup the servers in the Settings Dialog from a running smalltalk and save the settings
    to a file. Give this files name to the standALone server app
    (using the --useFile argument option).
    In projects/webServer also a stxWebServer file as an example system startup file is included
    copy this file to the /usr/sbin directory of your linux distribution, change the parameters
    and the server is started with system startup


    Passwords:
        services may be (individually) protected by realms (userName/password pairs).
        If no password is given, the service is free for everyone; except for the
        maintenance service, which requires a password (except for connections from the local host).

    allowedHosts / forbiddenHosts:
        server-instance specific; these allow fine-grain control over which hosts may access the server.
        If allowedHosts is non-nil, only hosts listed there may connect (for example, for local services on a separate port).
        If forbiddenHosts is non-nil, hosts listed there may never connect (for example, to lock out known intruders)
        temporarilyForbiddenHosts lists all hosts, which have sent a bad request recently; these are given another chance
        after a while (attackers will end up there).
"
!

howToStart
"
    Unix users:
      Notice, although the standard http port is 80, this is a system port and
      requires that ST/X is running either as privileged user or setuid.

    Since we cannot guarantee against any security holes, we do NOT
    recommend doing this; instead, use some non-privileged port (say 8080)
    and add a link from your regular homePage to 'http://yourHost:8080/...'.
"
!

todo
"
    TODO:
    - change Demos::WebHomePageForSTX homePage: from <TABLE> to <DIV>
    - allow and disallow peerNames per service
    - support sessions, using cookies, unique urls and parameters.
    - separate users from realms to allow the assignment of same user to multiple realms
    - if service url not found take the servers not found url use own redirection dictionary

    IDEAS:
    -  provide a global contents-cache (in Server); services should be able to
       specify that their generated contents should be cached there (in reply-send-code);
       for incoming requests, contentsCache is checked first;
       needs per-content-URL expiration time.
"
! !

!HTTPServer class methodsFor:'instance creation'!

new
    self loadRequiredClasses.

    ^ self basicNew initialize
!

newServerOnPort:aPortNr
    "Creates a server on aPortNr and return the server instance.
     This does NOT serve requests (i.e. it is not started)"

    |newServer|

    newServer := self new.
    newServer port:aPortNr.
    ^ newServer
!

newServerWithDefaultServiceListOn:port
    ^ self newServerWithServiceList:(self defaultServiceList) onPort:port

    "Modified: / 01-02-2007 / 20:23:17 / cg"
!

newServerWithServiceList:aServiceList onPort:port
    |newServer|

    newServer := self newServerOnPort:port.
    newServer serviceList:aServiceList.
    ^ newServer

    "Created: / 01-02-2007 / 20:23:02 / cg"
!

runningServerOnPort:aPort
    "return the server which is serving aPort or nil, if there is none."

    ^ self runningServers detect:[:serverInstance | serverInstance port = aPort] ifNone:nil.

    "
     HTTPServer runningServerOnPort:8080
     HTTPServer runningServerOnPort:8081
    "
!

runningServers
    "return a collection of running servers"

    RunningServers isNil ifTrue:[
        RunningServers := IdentitySet new.
    ].
    ^ RunningServers

    "
     RunningServers := IdentitySet new
    "
!

serverOnPort:aPortNr
    "if already running, return an existing server which serves aPortNr.
     If not, start one and return it."

    |server|

    server := self runningServerOnPort:aPortNr.
    server isNil ifTrue:[
        server := self startServerOnPort:aPortNr
    ].
    ^ server

    "
     self serverOnPort:8081
     self serverOnPort:8080
     self serverOnPort:9000
     self serverOnPort:9090
    "
!

soapEnabledServerOnPort:portNr
    "get an HTTPServer with SOAP enabled;
     However, its serviceHandler does not yet have any services defined
     (see Example services for more)."

    |server|

    server := self serverOnPort:portNr.
    server setupSoap.
    ^ server
!

startServer
    "Start a server on the default port (8080) and return the server instance.
     This starts a new process, which executes autonomously"

    ^ self startServerOnPort:(self defaultPort)

    "
     self startServer
    "
!

startServerOnPort:aPortNr
    "Start a server on aPortNr and return the server instance.
     This starts a new process, which executes autonomously.
     See also #serverOnPort:aPortNr, which first looks, if there is already a server
     running for that port."

    |newServer|

    newServer := self newServerOnPort:aPortNr.
    newServer start.
    ^ newServer

    "
     self startServerOnPort:8081
     self startServerOnPort:8080
     self startServerOnPort:9000
     self startServerOnPort:9090
    "

    "Modified: / 23-11-2006 / 17:34:18 / cg"
! !

!HTTPServer class methodsFor:'accessing'!

forbiddenPeers
    "return the list of peers from known fraud sites.
     cuge is here by default - they generate random ratings..."

    ^ ForbiddenPeers
!

forbiddenPeers:aSequenceableCollection
    "set the list of peers from known fraud sites.
     These are blocked from the server"

    ^ ForbiddenPeers
!

forbiddenUserAgents
    "return the list of agents from known fraud sites.
     cuge is here by default - they generate random ratings..."

    ^ ForbiddenUserAgents
!

forbiddenUserAgents:aSequenceableCollection
    "set the list of agents from known fraud sites.
     These are blocked from access"

    ^ ForbiddenUserAgents
!

terminationRequest

    ^ TerminationRequest
! !

!HTTPServer class methodsFor:'class initialization'!

initialize
    "/ self loadRequiredClasses - cg: now done when first instance is created

    "/ well known fraud sites - blocked from our house.
    "/ cuge is blocked: it is generating random ratings.

    "/ you may add more here.

    ForbiddenUserAgents := #(
                                '*cuge.org*'          "/ cugeCrawler 1.2 cuge.org
                            ).

    ForbiddenPeers :=      #(
                                '89.200.169.221'     "/ 'cuge.org'
                                '89.200.169.223'     "/ 'cugerank.com'
                            ) collect:[:b | (IPSocketAddress addressString:b) hostAddress ].
!

installInLauncher
    NewLauncher notNil ifTrue:[
        NewLauncher addSettingsApplicationByClass:#'AbstractSettingsApplication::HTTPStartServerSettingsApplication'
                    withName:'Communication/HTTP Server'
                    icon:nil.
    ].
    "
     self installInLauncher
    "
!

loadRequiredClasses
    HTTPApacheFormatAccessLogger autoload.
    HTTPFullAccessLogger autoload.
    HTTPRequest autoload.
    HTTPResponse autoload.
    ReplyPresenter autoload.
!

removeFromLauncher
    NewLauncher notNil ifTrue:[
        NewLauncher removeSettingsApplicationByClass:#'AbstractSettingsApplication::HTTPStartServerSettingsApplication'
    ].

    "
     self removeFromLauncher
    "
! !

!HTTPServer class methodsFor:'configuring - setup'!

addToBackupJob:aJob
    "add a job (block) to be evaluated hourly by any server"

    CommonBackupJobs isNil ifTrue:[
        CommonBackupJobs := OrderedCollection new.
    ].
    CommonBackupJobs add:aJob
! !

!HTTPServer class methodsFor:'constants'!

crlf
    CrLf isNil ifTrue:[
        CrLf := String with: Character return with: Character linefeed.
    ].
    ^ CrLf
!

crlfcrlf
    CrLfCrLf isNil ifTrue:[
        CrLfCrLf := self crlf , self crlf.
    ].
    ^ CrLfCrLf
!

httpVersion

    ^ '1.1'
! !

!HTTPServer class methodsFor:'defaults'!

availableAuthenticationClasses
    ^ Authentication::Authenticator knownAuthenticators.
!

availableAuthenticationMethods
    ^ self availableAuthenticationClasses
        collect:[:eachClass | eachClass mechanismName].
!

defaultAccessLogger

"/    ^ FullAccessLogger new.
    ^ HTTPApacheFormatAccessLogger new.
!

defaultAllowedAuthenticationMethods
    "answer the alloed authenication methods.
     We exclude Basic here, because it is insecure"

    ^ self availableAuthenticationMethods copyWithout:#Basic.
!

defaultFile
    ^ nil
!

defaultInactiveConnectionTimeout
    ^ 10*60 "/ seconds
!

defaultLogEnabled
    ^ true.
!

defaultMaxNumberOfConnections
    OperatingSystem isUNIXlike ifTrue:[
        ^ 100.
    ].
    "/ windows is not really useful as a server system
    ^ 15.
!

defaultMaxNumberOfKeepAliveConnections
    OperatingSystem isUNIXlike ifTrue:[
        ^ 80.
    ].
    "/ windows is not really useful as a server system
    ^ 10.
!

defaultPort
    "return my default port number.
     Use some high number here, since low ones (i.e. 80) are
     reserved to super users. To use one below 1024, run ST/X as
     setuid root."

    ^ 8080.

    "Modified: / 26.7.1999 / 19:34:54 / cg"
!

defaultServerReplyPresenterClass
    ^ PlainReplyPresenter
!

defaultServiceList
    | services |

    services := IdentitySet new.
    #(
        #'Demos::WebHomePageForSTX'                #('/Home')
        #'ServerMaintenanceService'                #('/Maintenance')
        #'HTTPClassDocService'                     #('/ClassDoc')
    ) pairWiseDo:[:aServiceClassName :aLinkCol|
        | service serviceClass |

        serviceClass := Smalltalk at:aServiceClassName.
        service := serviceClass new.
        service linkNames:(aLinkCol asOrderedCollection).
        services add:service.
    ].
    ^ services

    "
     self defaultServiceList
    "
!

homeURL
    "return the defaultHomeURL - thats the URL used when an empty URL is entered
     in the browser."

    ^ '/Home'
!

inactiveConnectionTimeout
    <resource: #obsolete>
    self obsoleteMethodWarning.
    ^ self defaultInactiveConnectionTimeout
!

logFile
    "default log file name server instance add '_portNumber' before suffix point "

    ^ Filename tempDirectory constructString:'web.log'
!

notFoundURL
    "return the default notFoundURL - thats the URL used when no service for the request is available."

    ^ nil
!

port
    "for protocol compatibility with instances (to allow for common settings dialog)"

    ^ self defaultPort
!

realmUsersMap
    ^ Dictionary new
!

settingsFilename

    ^ './webServerSettings.xml'.
!

temporaryLockoutTime
    ^ 10 "/ seconds
! !

!HTTPServer class methodsFor:'defaults-caching'!

maxCacheSize
    ^ 30 "/ files to cache
! !

!HTTPServer class methodsFor:'defaults-debugging'!

debugErrors
    ^ false.
!

verbose
    ^ false.
!

verboseData
    ^ false.
!

verboseErrors
    ^ false.
! !

!HTTPServer class methodsFor:'defaults-soap'!

defaultSoapEnabled
    ^ false
!

loadSoapPackage
    |soapPackage|

    ((SOAP isNil)
    or:[SOAP::SoapAction isNil
    or:[SOAP::SoapAction isLoaded not]]) ifTrue:[
        soapPackage := 'stx:goodies/soap'.
        (Smalltalk loadPackage:soapPackage) ifFalse:[
            ^ false.
        ].
    ].
    ^ true

"
    self loadSoapPackage
"
!

soapEnabled
    <resource: #obsolete>
    self obsoleteMethodWarning.
    ^ self defaultSoapEnabled
! !

!HTTPServer class methodsFor:'html composition'!

withSpecialHTMLCharactersEscaped:aStringOrCharacter
    "replace ampersand, less and greater by html-character escapes"

    ^ HTMLUtilities withSpecialHTMLCharactersEscaped:aStringOrCharacter

    "
     self withSpecialHTMLCharactersEscaped:'<>#&'
     self withSpecialHTMLCharactersEscaped:$<
     self withSpecialHTMLCharactersEscaped:$#
    "

    "Modified: / 05-12-2006 / 13:48:59 / cg"
! !

!HTTPServer class methodsFor:'obsolete'!

checkSwikiVersion
    "This is Squeak-Swiki 2.2.
     Make sure that the Server:swiki folder is the version for 2.2."

    | fName |

    fName := PWS::ServerAction serverDirectory, 'swiki', (PWS::ServerAction pathSeparator), 'page.html'.
    (fName asFilename exists) ifFalse: [
        Display isNil ifTrue:[
            self error:'The path to the Server folder is wrong (check PWS::ServerAction>>serverDirectory).'.
        ].

        self information: 'The path to the Server folder is wrong.
Please modify the following method ...'.
        Browser
            browseClass: PWS::ServerAction class
            selector: #serverDirectory.
        ^ false
    ].

    fName := PWS::ServerAction serverDirectory, 'swiki', (PWS::ServerAction pathSeparator), 'render.html'.
    (fName asFilename exists) ifFalse: [
        self information: 'Initialization failed.  ''swiki'' folder is out of date.
Get one from:
http://guzdial.cc.gatech.edu/st/Swiki-GuzPWS2.tar or
http://guzdial.cc.gatech.edu/st/Swiki-GuzPWS2.sea'.
        ^ false
    ].
    ^ true

    "
     self checkSwikiVersion
    "
!

disableMySwiki
    | defaultServer |

    defaultServer := self defaultServer.
    defaultServer notNil ifTrue:[
        defaultServer disableMySwiki.
    ].
!

rollForwardFrom: fileName
    "Roll the image forward using the given file."

"/    self halt:'unmaintained, old behavior'.

"/    | log |
"/
"/    log := FileStream oldFileNamed: fileName.
"/    [ log atEnd ] whileFalse: [
"/        log peek isDigit ifTrue: [
"/            [
"/                self new fromLog: (log upTo: CR); getReply
"/            ] ifError: [ :msg :rec | "just ignore" ]
"/        ] ifFalse: [
"/            log skipTo: CR
"/        ]
"/    ].
"/    Time bias: 0.
"/    log close
!

setupMySwiki
    | defaultServer |

    defaultServer := self defaultServer.
    defaultServer notNil ifTrue:[
        defaultServer setupMySwiki.
    ].
! !

!HTTPServer class methodsFor:'private'!

addServer:aServer
    | servers |

    servers := self runningServers.
    servers add:aServer.
    servers changed.
!

removeServer:aServer
    | servers |

    servers := self runningServers.
    servers remove:aServer ifAbsent:nil.
    servers changed.
! !

!HTTPServer class methodsFor:'queries'!

defaultServer
    ^ self runningServers detect:[:aHttpServer | aHttpServer port = self defaultPort] ifNone:nil.
!

isRunning
    "return true, if a server is running on defaultPort"

    ^ self isRunningOnPort:self defaultPort.
!

isRunningOnPort:aPort
    "return true, if a server is running on aPort"

    ^ self runningServers contains:[:serverInstance | serverInstance port = aPort].
! !

!HTTPServer class methodsFor:'queries-service'!

makeValidLinkName:eachLinkName 
    |validName|

    validName := eachLinkName.
    [ validName startsWith:'//'] whileTrue:[
        validName := validName copyFrom:2.
    ].
    validName := (validName ~= '/' and:[ validName endsWith:$/ ]) 
        ifTrue:[ validName copyButLast:1 ]
        ifFalse:[ validName ].
    (validName startsWith:'/') ifFalse:[
        validName := '/',validName
    ].
    ^ validName.

    "
     self makeValidLinkName:'//'     
     self makeValidLinkName:'//foo'  
     self makeValidLinkName:'/foo/'  
     self makeValidLinkName:'foo/'   
    "
! !

!HTTPServer class methodsFor:'read/write settings'!

instanceSettingsList
    "answer a list of instance variables to be saved as server settings"

    ^ #(
        realmUsersMap
        port
        logFile
        homeURL
        notFoundURL
        allowedAuthMethods
        debugErrors
        verbose
        verboseData
        verboseErrors
        verboseTiming
        soapEnabled
        maxNumberOfKeepAliveConnections
        maxNumberOfConnections
        temporaryLinkExpirationTime
        logEnabled
        serviceList
        settingsFilename
        accessLogger
        inactiveConnectionTimeout
        serverReplyPresenter
    )
!

serverFromSettingsFile:aFilename
    "create a new HTTP server from the setting stored in aFilename"
    |stream aHTTPServer|

    [
        stream := aFilename readStream.
        [
            aHTTPServer := self serverFromSettingsStream:stream.
        ] ensure:[
            stream close.
        ].
    ] on:OpenError do:[:ex| ].
    ^ aHTTPServer
!

serverFromSettingsStream:aStream
    "create a new HTTP server from the setting stored in aStream"

    |server|

    server := (XMLStandardDecoder
                    on:aStream
                    beforeScanDo:[:decoder| decoder useSend:true]) next.
    ^ server
! !

!HTTPServer class methodsFor:'serving'!

stopAllServers
    "Stop all running http servers.
     This is done kind of softly, allowing the current request to be finished."

    | servers |

    servers := self runningServers.
    servers do:[:serverInstance |
        self stopServerOnPort:serverInstance port.
    ].
"/    servers removeAll.
"/    servers changed.

    "
     HTTPServer stopAllServers
    "
!

stopServer
    "Stop the default server.
     This is done kind of softly, allowing the current request to be finished."

    self stopServerOnPort:self defaultPort.

    "
     HTTPServer stopServer
    "
!

stopServerOnPort:aPort
    "Stop the server for aPort.
     This is done kind of softly, allowing the current request to be finished."

    |serverInstance|

    serverInstance := self runningServerOnPort:aPort.
    serverInstance isNil ifTrue:[
        ('%1 [warning]: no server to shutdown on port %2.' bindWith:self name with:aPort) infoPrintCR.
        ^ self.
    ].

    serverInstance isRunning ifFalse:[
        ('%1 [warning]: found dead server.' bindWith:serverInstance serverName) errorPrintCR.
        self removeServer:serverInstance.
        ^ self.
    ].

    (serverInstance isRunning and:[serverInstance isInQuitStatus not]) ifTrue:[
        ('%1 [info]: shutdown initiated.' bindWith:serverInstance serverName) infoPrintCR.
        serverInstance changeServerStatusToQuit.
        [ Delay waitForSeconds:10. serverInstance stop] fork.
    ].

    "
     HTTPServer stopServerOnPort:(self defaultPort)
    "
!

terminateAllServers
    "Stop all running http servers.
     This is done kind of softly, allowing the current request to be finished."

    | servers |

    servers := self runningServers copy.
    servers do:[:serverInstance |
        serverInstance terminateServer.
    ].

    "
     HTTPServer terminateAllServers
    "
! !

!HTTPServer class methodsFor:'serving-helpers'!

gifImageDataForImage:img
    "given an image, generate GIF-data"

    |writeStream imageToSave|

    "/ must send it as gif ... (could do jpg as well),
    "/ or even better, remember what types are supported by the browser

    imageToSave := img.
    imageToSave depth ~~ 8 ifTrue:[
        "/ must convert - GIF only supports Depth8.
        imageToSave := Depth8Image fromImage:imageToSave.
    ].

    writeStream := WriteStream on:(ByteArray new:1024).
    GIFReader save:imageToSave onStream:writeStream.

    ^ writeStream contents.
! !

!HTTPServer class methodsFor:'signal access'!

abortRequestProcessing
    ^ AbortRequestProcessing
! !

!HTTPServer methodsFor:'accessing'!

accessLogger
    accessLogger isNil ifTrue:[
        accessLogger := self class defaultAccessLogger
    ].
    ^ accessLogger
!

accessLogger:something
    accessLogger := something.
!

addListeningAddress:aSocketAddress
    "add aSocketAddress to the list of addresses, that this server is listening.
     You have to restart the server, if it is already running."

    listeningAddresses isNil ifTrue:[
        listeningAddresses := Set new.
    ].
    listeningAddresses add:aSocketAddress.
    listeningAddresses size > 1 ifTrue:[
        self error:'HTTPServer: currently only one listener is supported'.
    ].


    "
      self new addListeningAddress:(IPSocketAddress anyHost port:6666).
    "
!

allowedHosts
    ^ allowedHosts
!

closeConnectionAfterError
    "to avoid getting hoged down by a bad guy, limit his access speed,
     by closing the connection"

    ^ true
!

inactiveConnectionTimeout
    "seconds"

    inactiveConnectionTimeout isNil ifTrue:[
        ^ self class defaultInactiveConnectionTimeout.
    ].
    ^ inactiveConnectionTimeout
!

inactiveConnectionTimeout:something
    "seconds"

    inactiveConnectionTimeout := something.
!

listeningAddresses
    ^ listeningAddresses
!

listeningAddresses:something
    listeningAddresses := something.
!

maxNumberOfConnections
    maxNumberOfConnections isNil ifTrue:[
        ^ self class defaultMaxNumberOfConnections
    ].
    ^ maxNumberOfConnections
!

maxNumberOfConnections:aNumber
    maxNumberOfConnections := aNumber.
!

maxNumberOfKeepAliveConnections
    maxNumberOfKeepAliveConnections isNil ifTrue:[
        ^ self class defaultMaxNumberOfKeepAliveConnections.
    ].
    ^ maxNumberOfKeepAliveConnections
!

maxNumberOfKeepAliveConnections:aNumber
    maxNumberOfKeepAliveConnections := aNumber.
!

port
    "answer the (IPV4-)port, that we are or will be listening on.
     This is not very well defined and kept only for backward compatibility"

    serverSocket notNil ifTrue:[
        ^ serverSocket port
    ].

    listeningAddresses isEmptyOrNil ifTrue:[
        "backward compatibility"
        ^ self class defaultPort.
    ].

    ^ listeningAddresses anElement port
!

port:aPort
    "backward compatibility.
     Set the IPV4-port of the server"

    listeningAddresses size > 1 ifTrue:[
        self error:'HTTPServer: only one listener supported in backward compatibility mode'.
    ].
    listeningAddresses := nil.
    self addListeningAddress:(IPSocketAddress anyHost port:aPort).
!

realmUsersMap
    realmUsersMap isNil ifTrue:[
        realmUsersMap := self class realmUsersMap.
    ].
    ^ realmUsersMap
!

realmUsersMap:aDict
    realmUsersMap := aDict.
!

serverHostName
    "Something pretty printed information about where we are running"

    serverHostName isNil ifTrue:[
        serverHostName := OperatingSystem getHostName.
    ].
    ^ serverHostName , ':' , self port printString
!

serverHostName:aString
    "Set the name of the server - only for informational purposes.
     If mothing is set, the host name is used."

    serverHostName := aString.
!

serverProcess
    ^ serverProcess
!

serverProcess:something
    serverProcess := something.
!

serverReplyPresenter
    "the default server reply presenter
     an instance for this class is created for every new request
     this valid until the service is changing the presenter
     if the service replyPrsenterClass is nil this replyPresenter is kept
     during running through the service"

    ^ serverReplyPresenter
!

serverReplyPresenter:something
    serverReplyPresenter := something.
!

serverStatusHolder
    "return a valueHolder holding the servers status (as a symbol)"

    serverStatusHolder isNil ifTrue:[
        serverStatusHolder := #notRunning asValue.
    ].
    ^ serverStatusHolder
!

serviceList
    serviceList isNil ifTrue:[
        serviceList := IdentitySet new.
    ].
    ^ serviceList
!

serviceList:aServiceSet
    self setupServices:aServiceSet.

    "Modified: / 01-02-2007 / 20:40:23 / cg"
!

servicesSema
    ^ servicesSema
!

settingsFilename
    |index|

    settingsFilename isNil ifTrue:[
        settingsFilename := self class settingsFilename.
        index := settingsFilename lastIndexOf:$..
        index ~= 0 ifTrue:[
            settingsFilename := (settingsFilename copyTo:(index - 1)), '_', self port asString, (settingsFilename copyFrom:(index)).
        ].
    ].
    ^ settingsFilename
!

settingsFilename:something
    settingsFilename := something.
!

soapEnabled
    soapEnabled isNil ifTrue:[
        ^ self class defaultSoapEnabled
    ].
    ^ soapEnabled
!

soapEnabled:aBoolean
    soapEnabled := aBoolean.
!

startupTime
    ^ startupTime
!

tentativeObjectService
    "answer an tentative object service.
     For backward compatibility: if there is none, create it on the fly"

    tentativeObjectService isNil ifTrue:[
        tentativeObjectService := HTTPTentativeObjectService new.
        tentativeObjectService registerServiceOn:self.
    ].

    ^ tentativeObjectService
! !

!HTTPServer methodsFor:'accessing-debugging & logging'!

addAlias:anAlias for:anUrl
    urlAliasDictionary at:anUrl put:anAlias.
!

debugErrors
    ^ debugErrors
!

debugErrors:aBoolean
    "if true, errors will trigger a debugger"

    debugErrors := aBoolean.
!

logFile
    "return the name of the log file"

    |index f|

    logFile isNil ifTrue:[
        logFile := self class logFile.
        index := logFile lastIndexOf:$..
        index ~= 0 ifTrue:[
            f := (logFile copyTo:(index - 1)), '_', self port asString, (logFile copyFrom:(index)).
        ].
        self logFile:f.
    ].
    ^ logFile
!

logFile:something
    |newFilename|

    (something isString and:[something size == 0]) ifTrue:[
        newFilename := nil
    ] ifFalse:[
        newFilename := something asFilename.
    ].

    logFile ~= newFilename ifTrue:[
        logFile := newFilename.
        self openNewLogFile.
    ].
!

removeAliasFor:anUrl
    urlAliasDictionary remove:anUrl ifAbsent:nil.
!

standaloneMode
    "answer true, if we are running in standalone mode.
     In this mode, even breakpoint exceptions are ignored"

"/    ^ Smalltalk isStandAloneApp or:[Smalltalk isHeadless].
    ^ Screen default isNil.
!

verbose
    "return the verbose flag - if set, trace messages are sent to the transcript"

    ^ verbose
!

verbose:something
    "set/clear the verbose flag - if set, trace messages are sent to the transcript"

    verbose := something.
!

verboseData
    "return the verboseData flag - if set, request data is dumped to the transcript"

    ^ verboseData
!

verboseData:something
    "set/clear the verboseData flag - if set, request data is dumped to the transcript"

    verboseData := something.
!

verboseErrors
    "return the verboseErrors flag - if set, errornous requests are logged to the transcript"

    ^ verboseErrors
!

verboseErrors:something
    "set/clear the verboseErrors flag - if set, errornous requests are logged to the transcript"

    verboseErrors := something.
!

verboseTimeConsumtion
    "this flag enables some timing statistics"

    "/ obsolete - backward compatibility
    ^ self verboseTiming
!

verboseTiming
    "this flag enables some timing statistics"

    verboseTimeConsumtion isNil ifTrue:[
        verboseTimeConsumtion := false.
    ].
    ^ verboseTimeConsumtion
!

verboseTiming:something
    "this flag enables some timing statistics - the default is false"

    verboseTimeConsumtion := something.
! !

!HTTPServer methodsFor:'accessing-http session'!

httpSessionAt:anId
    httpSessionsSema critical:[
        ^ httpSessions at:anId.
    ]
!

httpSessionAt:anId ifAbsent:aBlock
    httpSessionsSema critical:[
        ^ httpSessions at:anId ifAbsent:aBlock
    ]
!

httpSessionAt:anId put:aHTTPSession
    httpSessionsSema critical:[
        ^ httpSessions at:anId put:aHTTPSession
    ]
!

httpSessionRemove:aString
    httpSessionsSema critical:[
        httpSessions removeKey:aString
    ].
!

removeTimedOutSessions
"
    Associated with the minute job. Removes sessions that have become invalid

<return: self>
"

    httpSessionsSema critical:[
        httpSessions values do:[:aHttpSession |
            aHttpSession isValid ifFalse:[
                aHttpSession invalidate
            ].
        ].
    ]
! !

!HTTPServer methodsFor:'authentication-users'!

addRealm:aRealm
    "add a realm to the list of known realms"

    |map|

    map := self realmUsersMap.
    (map at:aRealm ifAbsent:nil) isNil ifTrue:[
        map at:aRealm put:(Dictionary new).
    ].
!

addUser:aUserName password:aPassword toRealm:aRealm overWrite:overWrite
    "this message is sent if a user is added to a realm or changed.
     If overWrite is true, the data of an already existing user is overwritten."

    |map users user|

    map := self realmUsersMap.

    users := map at:aRealm ifAbsentPut:[Dictionary new].
    (overWrite or:[(users includesKey:aUserName) not]) ifTrue:[
        user := Authentication::User realm:aRealm name:aUserName secret:aPassword.
        users at:aUserName put:user.
        serviceList do:[:eachService|
             "inform services about new user"
             eachService addUser:aUserName password:aPassword toRealm:aRealm.
        ].
    ].
!

allRealms
    "answer a collection of realm names known"

    ^ self realmUsersMap keys.
!

allowedAuthMethods
    "answer a collection of allowd authenticatation method names"

    allowedAuthMethods isNil ifTrue:[
        ^ self class defaultAllowedAuthenticationMethods
    ].
    ^ allowedAuthMethods
!

allowedAuthMethods:something
    allowedAuthMethods := something.
!

allowedAuthenticationClasses
    "answer a collection of authenticators that may be used to authenticate"

    ^ self allowedAuthMethods
        collect:[:eachAuthenticatorName | Authentication::Authenticator authenticatorClassNamed:eachAuthenticatorName]
!

canAuthenticate:aRequest with:authenticator
    "is it allowed to authenticate using authenticator?"

    ^ self allowedAuthMethods includes:(authenticator mechanismName)
!

canAuthenticateWith:authenticator
    <resource: #obsolete>
    "is it allowed to authenticate using authenticator?"

    self obsoleteMethodWarning.
    ^ self canAuthenticate:nil with:authenticator
!

forbiddenUserAgents
    ^ ForbiddenUserAgents
!

hasUser:aUserName forRealm:aRealm
    "is aUserName known in aRealm?"

    |users|

    users := self realmUsersMap at:aRealm ifAbsent:nil.
    ^ users notNil and:[(users at:aUserName ifAbsent:nil) notNil]
!

removeRealm:aRealm
    "remove aRealm completely"

    self realmUsersMap removeKey:aRealm ifAbsent:nil
!

removeUser:aUser fromRealm:aRealm
    "remove a user from a realm"

    |users|

    users := self realmUsersMap at:aRealm ifAbsent:nil.
    users notNil ifTrue:[
        users removeKey:aUser ifAbsent:nil.
    ].
!

usersForRealm:aRealm
    "answer the users belonging to aRealm"

    ^ self realmUsersMap at:aRealm ifAbsent:nil
! !

!HTTPServer methodsFor:'authorization-hosts'!

addToAllowedHosts:aHostAddr
    "add aHostAddr to the list of addresses, that are allowed to connect to the server.
     If there is a single entry in allowedHosts, only connections are permitted from peers that are listed
     in allowedHosts.
     Must be a hostAddress (i.e. 4-byte ByteArrays)"

    servicesSema critical:[
        allowedHosts isNil ifTrue:[allowedHosts := Set new].
        allowedHosts add:aHostAddr.
        self removeFromForbiddenHosts:aHostAddr.
    ].
!

addToForbiddenHosts:aHostAddr
    "add aHostAddr to the list of addresses, that are never allowed to connect to the server.
     Must be a hostAddress (i.e. 4-byte ByteArrays)"

    servicesSema critical:[
        self forbiddenHosts add:aHostAddr.
        self removeFromAllowedHosts:aHostAddr.
    ].
!

allowAllHosts
    allowedHosts := forbiddenHosts := nil
!

forbiddenHosts
    "return the list of addresses, that are never allowed to connect to the server.
     Contains hostAddresses (i.e. 4-byte ByteArrays)"

    forbiddenHosts isNil ifTrue:[
        forbiddenHosts := Set withAll:(self class forbiddenPeers).
    ].
    ^ forbiddenHosts
!

forbiddenHosts:aCollection
    "set the list of addresses, that are never allowed to connect to the server.
     Must contain hostAddresses (i.e. 4-byte ByteArrays)"

    forbiddenHosts := aCollection.
!

isConnectionAllowed:socket
    "return true, if the connection given by socket is allowed.
     Check:
        1. if allowedHosts contains entries, it MUST contain the peers address
        2. if forbiddenHosts MUST NOT contain the peers address
        3. if temporaryForbiddenHosts MUST NOT contain the peers adress.
           Entries in this collection will be removed by the server after some time (temporary not ok)."

    |forbidden peer hostAddr time|

    forbidden := self forbiddenHosts.

    (allowedHosts notNil
     or:[forbidden notEmptyOrNil
     or:[temporaryForbiddenHosts notNil]]) ifTrue:[
        peer := socket getPeer.
        peer isNil ifTrue:[Transcript showCR:'no peer'. ^ false].

        hostAddr := peer hostAddress.

        servicesSema critical:[
            allowedHosts notEmptyOrNil ifTrue:[
                (allowedHosts includes:(IPSocketAddress localHost address)) ifTrue:[
                    hostAddr = (IPSocketAddress hostName:OperatingSystem getHostName) address ifTrue:[
                        ^ true
                    ].
                ].
                ^ (allowedHosts includes:hostAddr)
            ].
            forbidden notEmptyOrNil ifTrue:[
                (forbidden includes:hostAddr) ifTrue:[Transcript showCR:'in forbidden:'. ^ false].
            ].
            temporaryForbiddenHosts notEmptyOrNil ifTrue:[
                time := temporaryForbiddenHosts at:hostAddr ifAbsent:nil.
                time notNil ifTrue:[
                    time > Timestamp now ifTrue:[Transcript showCR:'in temp forbidden:'.  ^ false ]. "/ still locked out
                    "/ ok, he gets another chance
                    temporaryForbiddenHosts removeKey:hostAddr.
                    temporaryForbiddenHosts isEmpty ifTrue:[
                        temporaryForbiddenHosts := nil.
                    ].
                ].
            ].
        ].
    ].
    ^ true
!

removeFromAllowedHosts:aHostAddr
    servicesSema critical:[
        allowedHosts notNil ifTrue:[
            allowedHosts remove:aHostAddr ifAbsent:[].
            allowedHosts isEmpty ifTrue:[allowedHosts := nil].
        ].
    ].
!

removeFromForbiddenHosts:aHostAddr
    servicesSema critical:[
        forbiddenHosts notNil ifTrue:[
            forbiddenHosts remove:aHostAddr ifAbsent:[].
            forbiddenHosts isEmpty ifTrue:[forbiddenHosts := nil].
        ].
    ].
!

temporaryForbid:aHostAddr
    "add aHostAddr to the list of addresses,
     that are not allowed to connect to the server for some time.
     This is used to protect against DOS attacks.
     Must be a hostAddress (i.e. 4-byte ByteArrays)"

    servicesSema critical:[
        temporaryForbiddenHosts isNil ifTrue:[temporaryForbiddenHosts := Dictionary new].
        temporaryForbiddenHosts at:aHostAddr put:(Timestamp now addSeconds:self class temporaryLockoutTime).
    ].
! !

!HTTPServer methodsFor:'change & update'!

update:something with:aParameter from:changedObject
    (changedObject == ObjectMemory and:[something == #returnFromSnapshot]) ifTrue:[
        serverHostName := nil.
        serverSocket := nil.
        (self printString, ' [info]: restart server') infoPrintCR.
        self start.
        ^ self.
    ].
    super update:something with:aParameter from:changedObject
! !

!HTTPServer methodsFor:'configuring'!

addToBackupJob:aJob
    "add a job (block) to be evaluated hourly by this server"

    backupJobs isNil ifTrue:[
        backupJobs := OrderedCollection new.
    ].
    backupJobs add:aJob
!

clearBackupJobs
    "Clear the backupJobs list."

    backupJobs := nil.
!

setupServer
    "this is called immediately before the server is started"

    self clearTemporaryLinkIds.
! !

!HTTPServer methodsFor:'default-urls'!

homeURL
    "return the homeURL - thats the URL used when an empty URL is entered
     in the browser."

    ^ homeURL ? self class homeURL
!

homeURL:anURL
    "set the homeURL - thats the URL used when an empty URL is entered
     in the browser."

    homeURL := anURL.
!

notFoundURL
    "return the notFoundURL - thats the URL used when no service for the request was found"

    ^ notFoundURL ? self class notFoundURL
!

notFoundURL:something
    notFoundURL := something.
!

rootUri
    "return the uri, that represents our root"

    ^ '/'
! !

!HTTPServer methodsFor:'initialization'!

initialize
    debugErrors := self class debugErrors.
    verboseErrors := self class verboseErrors.
    verboseData := self class verboseData.
    verbose := self class verbose.
    loggingSema := RecursionLock forMutualExclusion.
    keptConnectionsSema := RecursionLock forMutualExclusion.
    servicesSema := RecursionLock forMutualExclusion.
    httpSessionsSema := RecursionLock forMutualExclusion.
    httpSessions := Dictionary new.
    linkResolver := Dictionary new.
    overallNumberOfRequests := overallNumberOfServedConnections := 0.

    maxNumberOfKeepAliveConnections := self class defaultMaxNumberOfKeepAliveConnections.
    maxNumberOfConnections := self class defaultMaxNumberOfConnections.
    inactiveConnectionTimeout := self class defaultInactiveConnectionTimeout.

    self soapEnabled ifTrue:[
        self setupSoap.
    ].
    serverReplyPresenter isNil ifTrue:[
        serverReplyPresenter := self class defaultServerReplyPresenterClass new.
    ].
! !

!HTTPServer methodsFor:'links-action-soap'!

disableSoap

    self soapEnabled:false.
    soapActionHandler := nil.
!

setupSoap
    self setupSoapWithHandler:(SOAP::SoapServiceHandler new).
!

setupSoapWithHandler:aSoapServiceHandler
    self soapEnabled:true.
    soapActionHandler isNil ifTrue:[
        soapActionHandler := SOAP::SoapHttpModule new.
    ].
    soapActionHandler serviceHandler:aSoapServiceHandler.

    verbose ifTrue:[
        self verboseLog:'SOAP enabled.'.
    ]
!

soapActionHandler
    ^ soapActionHandler
!

soapServiceHandler
    soapActionHandler notNil ifTrue:[
        ^ soapActionHandler serviceHandler
    ].
    ^ nil
! !

!HTTPServer methodsFor:'logging'!

closeLogFile
    logStream notNil ifTrue:[
        loggingSema critical:[
            logStream close.
            logStream := nil.
        ].
    ].
!

createLogFileFor:fileNameOrStream
    |newLogStream fileName isOldLogfile|

    fileNameOrStream isNil ifTrue:[
        ^ nil.
    ].
    fileNameOrStream isStream ifTrue:[
        newLogStream := fileNameOrStream.
        ^ newLogStream.
    ].
    OpenError handle:[
        self changeServerStatusToError.
        self error:(self serverHostName, ': cannot create log file:'
                    , fileNameOrStream asFilename pathName)
            mayProceed:true.
        ^ nil
    ] do:[
        fileName := self newLogFileNameFor:fileNameOrStream.
        isOldLogfile := fileName exists.
        fileName directory recursiveMakeDirectory.
        newLogStream := fileName appendingWriteStream.
        isOldLogfile ifTrue:[
            newLogStream
                nextPutAll:'# Used by ';
                nextPutAll:self serverName;
                nextPutAll:' on port '.
            self port printOn:newLogStream.
            newLogStream cr.
        ].
        fileName makeWritableForAll
    ].
    ^ newLogStream
!

log:aString
    "write aString to our logfile"

    logStream notNil ifTrue:[
        loggingSema critical:[
            logStream notNil ifTrue:[
                logStream nextPutAll:aString.
"/            logStream flush
            ].
        ].
    ].
!

logCommented:aString
    "write a comment line to our log"

    logStream notNil ifTrue:[
        loggingSema critical:[
            logStream
                nextPutAll:'# ';
                print:( Timestamp now);
                nextPutAll:' ';
                nextPutAll:aString;
                cr.
        ].
    ].
!

logEnabled
    "answer true, if logging is enabled"

    logEnabled isNil ifTrue:[
        logEnabled := self class defaultLogEnabled.
    ].
    ^ logEnabled
!

logEnabled:aBoolean
    "if aBoolean == true, enable logging;
     if false, disable logging"

    logEnabled ~= aBoolean ifTrue:[
        self closeLogFile.
        logEnabled := aBoolean.
        self openNewLogFile.
    ].
!

newLogFileNameFor:fileName
    "creates a backup filename string from a given filename"

    ^ fileName asFilename withSuffix:(Timestamp now printStringFormat:'%(year)%(mon)%(day)%h')

"
    HTTPServer new newLogFileNameFor:'/tmp/web_8080.log'
"
!

openNewLogFile
    "backup existing logfiles and open a filestream on the logfile name"

    self logEnabled ifTrue:[
        logStream notNil ifTrue:[
            self closeLogFile
        ].
        logStream := self createLogFileFor:self logFile.
    ].
!

verboseLog:aString
    "do verbose logging of aString (to Transcript)"

    verbose ifTrue:[
        Transcript show:self serverName , ': '.
        Transcript showCR:aString.
    ].
! !

!HTTPServer methodsFor:'obsolete'!

logToFile:something
    <resource: #obsolete>
    " for backward compatibility "
    self logEnabled:something.
!

startServer

    <resource: #obsolete>

    self obsoleteMethodWarning:'use #start for comanche compatibility'.
    self start.
!

startServerOnPort:aPortNr
    <resource: #obsolete>

    self obsoleteMethodWarning:'use #startOnPort:'.
    self startOnPort:aPortNr.
!

stopServer
    <resource: #obsolete>
    self obsoleteMethodWarning:'use #stop for comanche compatibility'.
    self stop
! !

!HTTPServer methodsFor:'obsolete-pwsswiki'!

setupAuthorizedSwiki
    "setup the authorized swiki service:
        authorizedSwiki
     In this swiki, you must be authorized to modify pages,
     but anyone can read them.
     In addition, to the secureSwiki, pages may include
     embedded smalltalk (so be careful, to not let bad guys crash your system)
     The initial user/password is 'test'/'secure'
    "

    self setupAuthorizedSwikiNamed:'authorizedSwiki'

    "
     self setupAuthorizedSwiki
    "

    "Created: / 30.7.1999 / 20:31:23 / cg"
    "Modified: / 30.7.1999 / 20:35:28 / cg"
!

setupAuthorizedSwikiNamed:swikiName
    "setup the authorized swiki service:
        authorizedSwiki
     In this swiki, you must be authorized to modify pages,
     but anyone can read them.
     In addition, to the secureSwiki, pages may include
     embedded smalltalk (so be careful, to not let bad guys crash your system)
     The initial user/password is 'secure'/'secure'
    "

    |s anAuthorizer|

    (self class checkSwikiVersion) ifFalse: [^ self].

    (PWS::ServerAction serverDirectory asFilename construct:swikiName) exists
    ifTrue:[
        s := PWS::AuthorizedWriteEmbeddedActionSwiki new restore:swikiName fromServer:self.
    ] ifFalse:[
        s := PWS::AuthorizedWriteEmbeddedActionSwiki setUp:swikiName fromServer:self.
    ].

    s notNil ifTrue:[
        s authorizer isNil ifTrue:[
            anAuthorizer := PWS::Authorizer new.
            anAuthorizer realm:'Administrator'.
            anAuthorizer mapName:'secure' password:'secure' to:'Administrator'.
            s authorizer:anAuthorizer.
        ]
    ].
!

setupMySwiki
    "setup the unauthorized swiki service:
        mySwiki
     In this swiki, anyone can read and modify pages.
    "

    self setupSwikiNamed:'mySwiki'.

    "
     self setupMySwiki
    "

    "Modified: / 30.7.1999 / 20:30:25 / cg"
    "Created: / 30.7.1999 / 20:35:44 / cg"
!

setupSecureSwiki
    "setup the secure swiki service:
        secureSwiki
     In this swiki, you must be authorized to modify pages,
     but anyone can read them.
     The initial user/password is 'test'/'test'
    "

    self setupSecureSwikiNamed:'secureSwiki'
!

setupSecureSwikiNamed:aSecureSwikiName
    "setup the secure swiki service:
        secureSwiki
     In this swiki, you must be authorized to modify pages,
     but anyone can read them.
     The initial user/password is 'test'/'test'
    "

    |s anAuthorizer|

    (self class checkSwikiVersion) ifFalse: [^ false].

    (PWS::ServerAction serverDirectory asFilename construct:aSecureSwikiName) exists
    ifTrue:[
        s := PWS::AuthorizedWriteSwiki new restore:aSecureSwikiName fromServer:self.
    ] ifFalse:[
        s := PWS::AuthorizedWriteSwiki setUp:aSecureSwikiName fromServer:self.
    ].

    s notNil ifTrue:[
        s authorizer isNil ifTrue:[
            anAuthorizer := PWS::Authorizer new.
            anAuthorizer realm:'Administrator'.
            anAuthorizer mapName:'test' password:'test' to:'Administrator'.
            s authorizer:anAuthorizer.
        ]
    ].
    ^ true

    "
     self setupSecureSwiki
    "

    "Modified: / 30.7.1999 / 20:31:05 / cg"
    "Created: / 30.7.1999 / 20:35:36 / cg"
!

setupSwikiNamed:swikiName
    "setup or restore a swiki service named swikiName.
     In this swiki, anyone can read and modify pages.
    "

    (self class checkSwikiVersion) ifFalse: [^ self].

    (PWS::ServerAction serverDirectory asFilename construct:swikiName) exists
    ifTrue:[
        PWS::SwikiAction new restore:swikiName fromServer:self.
    ] ifFalse:[
        PWS::SwikiAction setUp:swikiName fromServer:self.
    ].
! !

!HTTPServer methodsFor:'printing'!

printOn:aStream
    self className printOn:aStream.
    aStream nextPutAll:' (port:'.
    self port printOn:aStream.
    aStream nextPut:$).

    "Modified: / 31-08-2007 / 15:37:41 / cg"
! !

!HTTPServer methodsFor:'queries'!

connectionClass
    ^ HTTPConnection
!

doNightlySnapShot
    ^ false
!

hasSoapEnabled
    ^ soapActionHandler notNil
!

overallNumberOfRequests
    "returns the overall number of served requests (both failed and successful)"

    ^ overallNumberOfRequests
!

overallNumberOfServedConnections
    "returns the overall number of served incoming connections"

    ^ overallNumberOfServedConnections
!

parametersPrintStrings
    |value strings|

    strings := StringCollection new.
    self class instanceSettingsList do:[:parameter |
        value := self perform:parameter.
        (parameter == #allowedAuthMethods) ifTrue:[
            strings add:('    ' , parameter asString , ':').
            value do:[:aMethod |
                strings add:('        ' , aMethod asString , ' ').
            ].
        ] ifFalse:[
            (parameter == #serviceList) ifTrue:[
                strings add:('    ' , parameter asString , ':').
                value do:[:aService |
                    strings add:('        ' , aService className , ' ').
                ].
            ] ifFalse:[
                (parameter ~= #realmUsersMap) ifTrue:[
                    strings add:('    ' , parameter asString , ':' , value asString).
                ].
            ].
        ].
    ].
    ^ strings
!

serverName
    "answer the server's name (for messages)"

    ^ self printString
!

serverReplyPresenterFor:aRequest
    ^ self serverReplyPresenter class new.
! !

!HTTPServer methodsFor:'regular jobs'!

doBackupJobs
    "This method is invoked every hour and is supposed to perform
     required backup jobs. Applications may register for a backupaction to
     be invoked hourly via #addToBackupJob:aJobBlock.
     This is just a wrapper so we don't have to restart the server loop,
     when we add/remove jobs."

    self logCommented:'doing backup jobs...'.

    self doBackupJobs:backupJobs.

    self == self class runningServers first ifTrue:[
        "/ only the first server performs the common actions.
        self doBackupJobs:CommonBackupJobs
    ].

    self logCommented:'Backup completed (nothing to do)'.
    ^ self

    "Modified: / 30.7.1999 / 12:04:09 / cg"
!

doBackupJobs:jobCollection
    "This method is invoked every hour and is supposed to perform
     required backup jobs. Applications may register for a backupaction to
     be invoked hourly via #addToBackupJob:aJobBlock.
     This is just a wrapper so we don't have to restart the server loop,
     when we add/remove jobs."

    jobCollection notEmptyOrNil ifTrue:[
        jobCollection do:[:eachBlock |
            eachBlock on:Error do:[:ex |
                self serverName errorPrint. '[warning]: error doing Backup jobs: ' errorPrint.
                ex signal errorPrint. ': 'errorPrint. ex description errorPrintCR
            ].
        ].
        self logCommented:'Backup jobs completed'.
    ].

    "Modified: / 20-08-2007 / 16:04:18 / cg"
!

doHourlyJobs
    "This method is invoked every hour and is supposed to perform cleanup actions.
     Currently, it starts a garbage collect.
     This is just a wrapper so we don't have to restart the server loop,
     when we add/remove jobs."

    self doBackupJobs.

    self logCommented:'doing hourly jobs...'.

    Error handle:[:ex |
        self serverName errorPrint. '[warning]: error in doHourlyJobs: ' errorPrint.
        ex signal errorPrint. ': ' errorPrint. ex description errorPrintCR.
    ] do:[
        Processor activeProcess 
            withPriority:Processor userBackgroundPriority
            do:[
                ObjectMemory garbageCollect 
            ]. 

    ].
    self logCommented:'Hourly jobs done.'.

    "Modified: / 20-08-2007 / 16:04:25 / cg"
!

doNightlyJobs
    "This method is invoked every midnight and is supposed to perform cleanup actions.
     Currently, it purges old graphic and temporary links and performs a
     compressing garbage collect.
     This is just a wrapper so we don't have to restart the server loop,
     when we add/remove jobs."

    self logCommented:'doing nightly jobs...'.

    Error handle:[:ex |
        self serverName errorPrint. '[warning]: error in doNightlyJobs: ' errorPrint.
        ex signal errorPrint. ': ' errorPrint. ex description errorPrintCR.
    ] do:[
        self clearTemporaryLinkIds.

        ObjectMemory garbageCollect.
        "put other nightly jobs here"
    ].
    self logCommented:'Nightly jobs done.'.

    self doNightlySnapShot ifTrue:[
        self logCommented:'Trigger automatic snapshot.'.
        self changeServerStatusToSnapshot.
    ].
    self openNewLogFile.

    "Modified: / 30.7.1999 / 12:04:33 / cg"
! !

!HTTPServer methodsFor:'service-queries'!

allServiceLinkNames
    ^ linkResolver keys
!

fileServices
    ^ (self serviceList select:[:aService| aService isFileService]) asOrderedCollection.
!

hasServiceForLink:aLinkName
    "Return true, if there is a service registered for aLinkName"

    ^ (self serviceForLink:aLinkName ifAbsent:nil) notNil.
!

hasServiceForServiceClass:aServiceClass
    "return true, if there is a link for a aServiceClass"

    ^ self serviceList contains:[:eachService| eachService class == aServiceClass].
!

linkResolver
    ^ linkResolver
!

resolveServiceForPath:aPath
    "get serviceLinkName and service instance for aPath"

    |serviceLinkNameSize service serviceLinkName|

    serviceLinkNameSize := 0.

    servicesSema critical:[
        linkResolver keysAndValuesDo:[:eachServicesLinkName :eachService|
            |thisSize|

            ((aPath startsWith:eachServicesLinkName)
             and:[(thisSize := eachServicesLinkName size) > serviceLinkNameSize
             and:[(eachServicesLinkName = aPath
                   or:[thisSize == 1
                   or:[(eachServicesLinkName at:thisSize) == $/
                   or:[(aPath at:(thisSize + 1)) == $/ ]
                   ]]
             )]]
            ) ifTrue:[
                serviceLinkNameSize := thisSize.
                serviceLinkName := eachServicesLinkName.
                service := eachService.
            ].
        ].
    ].

    ^ Array with:service with:serviceLinkName.
!

serviceAndLinkNameFromPath:aPath into:aRequest
    "get serviceLinkName and service instance for aPath,
     and set them in aRequest.
     Also set the path relative to the service link name.
     Called for incoming requests to determine, which service is to handle it."

    |serviceLinkNameSize service serviceLinkName relativePath result|

    result := self resolveServiceForPath:aPath.

    service := result at:1.
    serviceLinkName := result at:2.

    aRequest service:service serviceLinkName:serviceLinkName.

    service notNil ifTrue:[
        "now compute the path relative to the services link name.
         it has to start with a $/"

        serviceLinkNameSize := serviceLinkName size.

        (serviceLinkNameSize = aPath size) ifTrue:[ 
            relativePath := ''.
        ] ifFalse: [   
            (serviceLinkNameSize == 1 and:[aPath size = 1]) ifTrue:[
                relativePath := ''.
            ] ifFalse:[
                serviceLinkName last == $/ ifTrue:[
                    relativePath := aPath copyFrom:(serviceLinkNameSize + 1).
                ] ifFalse:[
                    relativePath := aPath copyFrom:(serviceLinkNameSize + 2).
                ].
            ].
        ].

        aRequest pathRelativeToService:relativePath.
    ].
!

serviceForLink:aLinkName
    "Return the service registered for aLinkName, or nil if there is none."

    ^ self serviceForLink:aLinkName ifAbsent:nil.
!

serviceForLink:aLinkName ifAbsent:exceptionBlock
    "Return the service registered for aString, or the value from exceptionBlock if there is none."

    |key|

    linkResolver isNil ifTrue:[
        ^ exceptionBlock value
    ].

    key := aLinkName first isDigit
                ifTrue:[aLinkName asNumber]
                ifFalse:[aLinkName].

    ^ linkResolver at:key ifAbsent:exceptionBlock
!

servicesForServiceClass:aServiceClass
    "return the collection of instances, for which the class is aServiceClass.
     empty collection if there is none."

    ^ self serviceList select:[:eachService| eachService class == aServiceClass].
! !

!HTTPServer methodsFor:'service-registration'!

addFileServiceWithRootLinkName:aLinkName class:aFileServiceClass rootDirectory:aRootDirectory defaultFileName:aDefaultFilename
    | fileService |

    fileService := aFileServiceClass new.
    fileService linkNames:(OrderedCollection with:aLinkName).
    fileService fileRoot:aRootDirectory asFilename.
    fileService defaultFileName:aDefaultFilename.
    fileService registerServiceOn:self.
    ^ fileService

    "Modified: / 01-02-2007 / 23:21:23 / cg"
!

addFileServiceWithRootLinkName:aLinkName rootDirectory:aRootDirectory defaultFileName:aDefaultFilename
    |alreadyRegisteredService|

    alreadyRegisteredService := self serviceForLink:aLinkName ifAbsent:nil.
    alreadyRegisteredService notNil ifTrue:[
        ^ alreadyRegisteredService.
    ].

    ^ self
        addFileServiceWithRootLinkName:aLinkName
        class:HTTPFileService
        rootDirectory:aRootDirectory
        defaultFileName:aDefaultFilename

    "Modified: / 01-02-2007 / 21:02:27 / cg"
!

addLink:aLinkName toService:aService
    "add a link for aService"

    |linkNames|

    aService isNil ifTrue:[
        self error:'cannot register nil service'.
        ^ self
    ].
    servicesSema critical:[
        linkNames := aService linkNames.
        aLinkName isNil ifTrue:[
            self error:'cannot register ', aService className, ' without link names'.
            ^ self
        ].
        linkNames add:aLinkName.
        self serviceList add:aService.
        linkResolver at:aLinkName put:aService.
    ].
!

addServiceToServiceList:aService
    "add aService to the list of services.
     If aService redefines link names,
     unregister the old service using the same link names"

    |myServiceList linkNames |

    linkNames := aService linkNames.
    myServiceList := self serviceList.

    myServiceList copy select:[:oldService | oldService linkNames = linkNames] thenDo:[:oldService |
        oldService unRegisterServiceOn:self
    ].

    myServiceList add:aService.

    "Modified: / 01-02-2007 / 20:33:11 / cg"
!

changeLinksForService:aService
    "aService may have changed link names.
     Update them"

    |linkNames|

    aService isNil ifTrue:[
        self error:'cannot change links for nil service'.
    ].

    linkNames := aService linkNames.
    servicesSema critical:[
        linkResolver keysAndValuesDo:[:eachLinkName :eachService|
            (eachService == aService and:[(linkNames includes:eachLinkName) not]) ifTrue:[
                linkResolver saveRemoveKey:eachLinkName.
            ]
        ].

        linkNames do:[:eachLinkName|
            linkResolver at:eachLinkName put:aService.
        ].
    ].
!

linkService:aService
    "register the links from aService"

    |linkNames|

    aService isNil ifTrue:[
        self error:'cannot register nil service'.
    ].

    servicesSema critical:[
        (serviceList isNil
        or:[(serviceList includes:aService) not]) ifTrue:[
            self addServiceToServiceList:aService.
        ].

        linkNames := aService linkNames.
        linkNames isEmptyOrNil ifTrue:[
            ^ self
        ].

        linkResolver isNil ifTrue: [
            linkResolver := Dictionary new.
        ].
        linkNames do:[:aLinkName|
            |linkNameToRegister|

            linkNameToRegister := aLinkName.
            (aLinkName startsWith:'/') ifFalse:[
                linkNameToRegister := '/',aLinkName.
            ].
            (linkResolver includesKey:linkNameToRegister) ifFalse:[
                linkResolver at:linkNameToRegister put:aService.
            ].
        ].
    ].

    "Modified: / 01-02-2007 / 21:00:23 / cg"
!

registerTentativeObjectService:aService
    "if no tentativeObjectService has been registered as default until now,
     register aService"

    tentativeObjectService isNil ifTrue:[
        tentativeObjectService := aService
    ].
!

rewriteURLFor:anURL
    urlAliasDictionary isEmptyOrNil ifTrue:[^ nil].
    ^ urlAliasDictionary at:anURL ifAbsent:nil.
"/    anURL = '/foo' ifTrue:[
"/        ^ '/bar'.
"/    ].
"/    ^ nil
!

setupServices:serviceList
    "setup defaults service links"

    |serviceListCopy|

    serviceListCopy := serviceList copy.

    "/ a bit of a hack here: first create all links
    "/ if the current service list already includes a required foreign service
    "/ and another service is calling addRequiredForeinServices: the link is resolved from the existing service.

    serviceListCopy do:[:aService|
        self linkService:aService.
        aService httpServer:self.
    ].

    serviceListCopy do:[:aService|
        aService registerServiceOn:self.
    ].

    "Created: / 01-02-2007 / 20:36:29 / cg"
!

unRegisterAllServices
    self serviceList copy do:[:aService |
        (self serviceForLink:aService linkName ifAbsent:nil) notNil ifTrue:[
            aService unRegisterServiceOn:self.
        ].
    ].
!

unRegisterTentativeObjectService:aService
    "unregister aService, if is has been registered as default tentativeObjectService"

    tentativeObjectService == aService ifTrue:[
        tentativeObjectService := nil.
    ].
!

unlinkService:aService
    "unlink aService."

    | linkNames|

    aService isNil ifTrue:[
        self error:'cannot unregister nil service'.
        ^ self
    ].
    servicesSema critical:[
        linkNames := aService linkNames.

        linkResolver isNil ifTrue: [
            linkResolver := Dictionary new.
            ^ self.
        ].
        self serviceList remove:aService ifAbsent:nil.
        linkNames do:[:aLinkName|
            linkResolver removeKey:aLinkName ifAbsent:[].
        ].
    ].
! !

!HTTPServer methodsFor:'serving'!

closeServerSocket
    serverSocket notNil ifTrue:[
        verbose ifTrue:[
            self verboseLog:'Server socket closed'.
        ].
        serverSocket close.
        serverSocket := nil.
    ].
!

createServerProcess
    "Initialize the server to be run on the set port.
     This creates and returns a new process, which can be started with #resume"

    self isRunning ifTrue:[ ^ self ].

    ObjectMemory addDependent:self.
    self setupServer.

    self openNewLogFile.
    [
        self openServerSocket.
    ] on:OpenError do:[:ex|
        self changeServerStatusToNotRunning.
        self closeLogFile.
        ex reject.
    ].

    self class addServer:self.

    serverProcess := [
        [
            "inform my services that i start serving from here"
            self serviceList do:[:aService|
                aService preStartServing.
            ].

            self serve.

            verbose ifTrue:[
                self verboseLog:'shutdown finished.'.
            ]
        ] ensure:[
            "inform my services that I stopped serving"
            self serviceList do:[:aService|
                aService postStopServing.
            ].
            self changeServerStatusToNotRunning.
            ObjectMemory removeDependent:self.
            self class removeServer:self.
            self closeLogFile.
        ]
    ] newProcess.

    serverProcess priority:(Processor systemBackgroundPriority).
    serverProcess priorityRange:(Processor systemBackgroundPriority to:Processor activePriority).
    serverProcess name:self serverName.
"/    serverProcess restartable:true.
    serverProcess beGroupLeader.
    startupTime := Timestamp now.

    ^ serverProcess

    "
     self startServerOnPort:8080
     self startServerOnPort:9000
     self startServerOnPort:9090
    "

    "Modified: / 19-03-2007 / 21:41:26 / cg"
!

loop
    "This is the inner HTTP server loop.
     The logging argument may be a stream or a fileName or nil.
     Periodically flush the logfile.
     Invoke hourly and nightly jobs."

    |connection lastDay lastHour lastMinute now socket gracefulClose|

    keptConnections := Set new.

    serverSocket listenFor:5.

    now := Timestamp now.
    lastDay := now asDate.
    lastHour := now hours.
    lastMinute := now minutes.

    self changeServerStatusToServing.

    gracefulClose := false.
    [
        [self serverStatus == #serving] whileTrue:[
            TerminationRequest handle:[:ex |
                "catch a private TerminationRequest-signal.
                 It is only handled here, while waiting for connections (otherwise, it is ignored).
                 It has the effect, that the server terminates quickly while waiting,
                 but finishes any request currently being served"
            ] do:[
                socket := serverSocket waitForNewConnectionWithTimeout:30.
                now := Timestamp now.
                socket notNil ifTrue: [
                    (self isConnectionAllowed:socket) ifFalse:[
                        Transcript showCR:('Not allowed: connection from ' , socket peerName printString).
                        self logCommented:('Not allowed: connection from ' , socket peerName printString).
                        socket shutDown.
                    ] ifTrue:[
                        connection := self connectionClass new initializeFor:self socket:socket.
                        logStream notNil ifTrue:[
                            connection logStream:(WriteStream on:(String uninitializedNew:100)).
                        ].
                        self addConnection:connection.
                        connection process.
                    ].
                ].
            ].
            "/
            "/ must I close old connections ?
            "/
            keptConnections size > maxNumberOfKeepAliveConnections ifTrue:[
                self closeOldestConnection.

                (maxNumberOfConnections notNil and:[keptConnections size >= maxNumberOfConnections]) ifTrue:[
                    "/ if there are already too many connections, what should be done here
                    "/ (only local, accept every other, shutdown old ones ????)
                    self shutdownOldConnections.
                ].
            ].

            "/
            "/ minutely action
            "/
            lastMinute ~= (now minutes) ifTrue: [
                lastMinute := now minutes.
                self doMinuteByMinuteActionAt:now.
            ].
            "/
            "/ 10-minute action
            "/
            (lastMinute // 10) ~= (now minutes // 10) ifTrue: [
                "/ shut down kept connections ...
                "/ ... just to make certain that they are freed eventually
                "/ and not persistently locked up in case of malfunction.
                verbose ifTrue:[
                    self verboseLog:'10min - closing all unused connections ...'.
                ].
                self removeUnusedConnections.
            ].
            "/
            "/ hour action
            "/
            lastHour ~= now hours ifTrue: [
                lastHour := now hours.
                self doHourlyJobs.
            ].
            "/
            "/ midnight action
            "/
            lastDay < now asDate ifTrue: [
                lastDay := now asDate.
                self doNightlyJobs.
            ].
        ].
        self logCommented:'Server socket closed.'.
        gracefulClose := true.
    ] ensure:[
        gracefulClose ifFalse:[
            self logCommented:'Server hard shutdown.'.
        ].
        verbose ifTrue:[
             self verboseLog:'Close all connections'.
        ].
        keptConnections copy do:[:eachConnection |
            eachConnection close.
        ].
        keptConnections := Set new.
    ].

    "Modified: / 4.2.2002 / 16:26:05 / cg"
!

openServerSocket
    |listeningAddress|

    serverSocket isNil ifTrue:[
        listeningAddresses isEmptyOrNil ifTrue:[
            "backward compatibility - use default port"
            self port:self class defaultPort.
        ].
        listeningAddress := listeningAddresses first.
        [
            serverSocket := Socket newTCP.
            serverSocket bindTo:listeningAddress.
            self logCommented:'Server socket created'.
        ] on:OpenError do:[:ex|
            self logCommented:('Cannot create server socket (', listeningAddress printString , ')').
            (self serverName, '[error]: Cannot create server socket (', listeningAddress printString , ')') errorPrintCR.
            (self serverName, '[info]: Socket is probably used by another program') errorPrintCR.
            self changeServerStatusToQuit.
            ex reject.
        ].
    ]
!

restartServer
    "restart the server process"

    self stop.
    "give the server a chance to terminate"
    serverProcess waitUntilTerminated.
    Delay waitForSeconds:0.1.
    self start.
!

serve
    "This is the outer HTTP server loop."

    [
        (Error , HaltInterrupt) handle:[:ex |
            |theSignal|

            Transcript showCR:('Cought error in HTTPServer>>serve: ' , ex description).
            theSignal := ex signal.
            (HaltInterrupt accepts:theSignal) ifTrue:[
                "ignore halts in standalone apps"
                self standaloneMode ifTrue:[
                    ex proceed.
                ].
                ex reject.
            ].

            (self debugErrors or:[theSignal == NoHandlerError]) ifTrue:[
                ex reject.
            ].
        ] do:[
            serverSocket isNil ifTrue:[ self openServerSocket ].
            serverSocket notNil ifTrue:[
                [
                    self loop.
                ] ensure:[
                    self closeServerSocket.
                ]
            ]
        ].

        self isInSnapshotStatus ifTrue: [
            OperatingSystem accessDeniedErrorSignal handle:[:ex |
                Transcript showCR:(self serverName, '[warning]: could not create snapshot: ' , ex description).
            ] do:[
                ObjectMemory snapShot.
            ]
        ].

        self isInCollectGarbageStatus ifTrue: [
            ObjectMemory garbageCollect.
        ].

"/        self isInRestartServerStatus ifTrue: [
"/            self restartServer.
"/        ].
        self isInQuitStatus not.
    ] whileTrue.
!

start
    "Start the server on the default or before already set port.
     This starts a new process, which executes autonomously"

    self isRunning ifTrue:[ ^ self ].

    self createServerProcess.
    startupTime := Timestamp now.
    serverProcess resume.

    "
     self startServerOnPort:8080
     self startServerOnPort:9000
     self startServerOnPort:9090
    "
!

startOnPort:aPortNr
    "Start the server on aPortNr.
     This starts a new process (if not already running), which executes autonomously"

    self port:aPortNr.
    self start.
!

stop
    "tell the server to stop; notice that there is a chance, that the server
     continues operation for some time.
     It will finish any ongoing requests, however not accepting any new ones.
     If the request(s) take too long to execute, they will be aborted
     (i.e. sooner or later the server will be definitely stopped)"

    self isRunning ifTrue:[
        self changeServerStatusToQuit.

        "/ raise a private signal - only handled while waiting for connections;
        "/ otherwise, it is ignored.
        "/ Has the effect, that server terminates quickly while waiting,
        "/ but finishes any request being served
        serverProcess interruptWith:[self class terminationRequest raiseRequest].
    ].
!

terminateServer
    "hard terminate the server - pending requests will not be finished"

    serverProcess notNil ifTrue:[
        serverProcess terminate.
    ].
    self changeServerStatusToQuit.
    self class removeServer:self.
! !

!HTTPServer methodsFor:'serving-helpers'!

addConnection:aConnection
    "remember aConnection"

    verbose ifTrue:[
        Transcript showCR:self serverName, ': add aConnection'.
    ].
    keptConnectionsSema critical:[
        keptConnections add:aConnection.
    ].
    overallNumberOfServedConnections := overallNumberOfServedConnections + 1.     "/ statistics only
!

closeOldestConnection
    "close the oldest connection we currently hold"

    verbose ifTrue:[
        self verboseLog:'remove oldest connection'.
    ].

    keptConnectionsSema critical:[
        |connectionToClose oldestLastRequestTime|

        "/ find the oldest ...
        keptConnections do:[:eachConnection |
            |thisLastRequestTime|

            thisLastRequestTime := eachConnection lastRequestTime.
            thisLastRequestTime notNil ifTrue:[
                (oldestLastRequestTime isNil or:[thisLastRequestTime < oldestLastRequestTime]) ifTrue:[
                    oldestLastRequestTime := thisLastRequestTime.
                    connectionToClose := eachConnection.
                ].
            ]
        ].
        connectionToClose notNil ifTrue:[
            verbose ifTrue:[
                self verboseLog:'closing oldest connection'.
            ].
            "connection removes itself from keptConnections"
            connectionToClose scheduleForClose.
        ].
    ].
!

countRequest
    overallNumberOfRequests := overallNumberOfRequests + 1.     "/ statistics only
!

doMinuteByMinuteActionAt:now
    "minutely action"

    logStream notNil ifTrue:[
        logStream flush
    ].

    self expireOldTemporaryLinksFor:now.

    self removeTimedOutSessions
!

removeConnection:aConnection
    "remove aConnection from the list of remebered connections"

    keptConnectionsSema critical:[
        keptConnections remove:aConnection ifAbsent:[].
    ].
!

removeUnusedConnections
    "remove all connections that are inactive for some time (inactiveConnectionTimeout)"

    |now|

    now := Timestamp now.

    keptConnectionsSema critical:[
        keptConnections do:[:eachConnection| |lastRequestTime|
            lastRequestTime := eachConnection lastRequestTime.
            (lastRequestTime notNil
             and:[(lastRequestTime addSeconds:inactiveConnectionTimeout) < now]) ifTrue:[
                eachConnection scheduleForClose.
            ].
        ].
    ].
!

shutdownOldConnections
    "shut down (immediately) old connections, as we have too many already.
     Here, shutDown the one with the longest already processing request
     (could be a blocking one...)"

    verbose ifTrue:[
        self verboseLog:'remove old connection'.
    ].
    keptConnectionsSema critical:[
        |connectionToClose longestRequestTime|
    
        "/ find the one with the longest processing time...
        keptConnections do:[:eachConnection |
            |thisRequestTime|

            thisRequestTime := eachConnection timeInRequest.
            thisRequestTime notNil ifTrue:[
                (longestRequestTime isNil or:[thisRequestTime > longestRequestTime]) ifTrue:[
                    longestRequestTime := thisRequestTime.
                    connectionToClose := eachConnection.
                ].
            ]
        ].
        connectionToClose notNil ifTrue:[
            verbose ifTrue:[
                self verboseLog:'closing connection'.
            ].
            "connection removes itself from keptConnections"
            connectionToClose close.
        ]
    ].
! !

!HTTPServer methodsFor:'serving-serverState'!

inMaintenanceMode
    "return true, iff in mainenance mode.
     In this mode, no files are served (only Server.control-requests are served)"

    ^ maintenanceMode == true
!

isInCollectGarbageStatus
    ^ self serverStatus == #collectGarbage
!

isInQuitStatus
    ^ self serverStatus == #quit
!

isInRestartServerStatus
    ^ self serverStatus == #restartServer
!

isInSnapshotStatus
    ^ self serverStatus == #snapshot
!

isRunning
    "true if serverProcess is up.
     (but could still be in startUp or shutDown procedure"

    ^ serverProcess notNil and:[serverProcess isDead not]
!

isServing
    "true if serverProcess is up and socket-connections are served"

    |serverStatus|

    serverStatus := serverStatusHolder value.

    ^ self isRunning
            and:[serverStatus ~~ #quit
            and:[serverStatus ~~ #notRunning]]
!

maintenanceMode:aBoolean
    maintenanceMode := aBoolean
!

serverStatus
    "return the serverStatus as a symbol"

    ^ self serverStatusHolder value
!

waitForServerStatus:conditionBlock withTimeout:secondsOrNil
    "wait until server status changed, so that conditionBlock evaluates to true.
     Raise Error on timeout, self otherwise"

    |sema block|

    conditionBlock value ifFalse:[
        [
            sema := Semaphore new.
            block := [sema signal].
            serverStatusHolder onChangeEvaluate:block.
            conditionBlock whileFalse:[
                ((sema waitWithTimeout:secondsOrNil) isNil and:[conditionBlock value not]) ifTrue:[
                    "status didn't change in secondsOrNil time.
                     Proceed to wait for anothert period of time"
                    TimeoutError raiseRequestErrorString:'HTTPServer did not change state'.
                ].
                sema clear.
            ].
        ] ensure:[
            serverStatusHolder retractInterestsFor:block.
        ].
    ].
!

waitUntilServing
    "wait until I am serving requests"

    self waitForServerStatus:[self isServing] withTimeout:nil.
!

waitUntilStopped
    "wait until I am no longer serving requests"

    self 
        waitUntilStoppedWithTimeout:nil.
!

waitUntilStoppedWithTimeout:secondsOrNil
    "wait until I am no longer serving requests"

    self 
        waitForServerStatus:[self isServing not] 
        withTimeout:secondsOrNil.
! !

!HTTPServer methodsFor:'serving-serverState-private'!

changeServerStatusTo:newStatus
    verbose ifTrue:[
        self verboseLog:'serverStatus: ', newStatus.
    ].

    self serverStatusHolder value:newStatus.
!

changeServerStatusToBackingUpLogFile
    self changeServerStatusTo:#backingUpLogFile.
!

changeServerStatusToCollectGarbage
    self changeServerStatusTo:#collectGarbage.
!

changeServerStatusToError
    self changeServerStatusTo:#error.
!

changeServerStatusToNotRunning
    self changeServerStatusTo:#notRunning.
!

changeServerStatusToQuit
    self changeServerStatusTo:#quit.
!

changeServerStatusToRestartServer
    self changeServerStatusTo:#restartServer.
!

changeServerStatusToServing
    self changeServerStatusTo:#serving.
!

changeServerStatusToSnapshot
    self changeServerStatusTo:#snapshot.
! !

!HTTPServer methodsFor:'settings persistency'!

elementDescriptorFor:anAspect
    "support for persistency:
     answer the elements to be made persistent with an ObjectCoder"

    |slots|

    anAspect ~~ #settings ifTrue:[
        ^ super elementDescriptorFor:anAspect
    ].
    slots := self class instanceSettingsList.

    ^ slots collect:[:eachVarName | eachVarName -> (self perform:eachVarName asSymbol)]
            thenSelect:[:pairs | pairs value notNil]
!

fetchServerSettingsFrom:fromHttpServer
    "fetch all settings as in fromHttpServer"

    self class instanceSettingsList do:[:getter |
        |setter|

        setter := (getter, ':') asSymbol.
        (self respondsTo:setter) ifTrue:[
            (self perform:setter with:(fromHttpServer perform:getter))
        ].
    ].
!

hasSameSettingsLike:aHTTPServer

    aHTTPServer == self ifTrue:[
        ^ true
    ].
    self class instanceSettingsList do:[:eachSelector |
        eachSelector = #serviceList ifTrue:[
            |localServiceList foreignServiceList|

            localServiceList := self perform:eachSelector.
            foreignServiceList := aHTTPServer perform:eachSelector.
            localServiceList size ~= foreignServiceList size ifTrue:[
                ^ false
            ].
            localServiceList do:[:eachLocalService|
                foreignServiceList detect:[:eachForeignService|
                    eachForeignService hasSameSettingsLike:eachLocalService
                ] ifNone:[
                    ^ false
                ].
            ].
        ] ifFalse:[
            (aHTTPServer perform:eachSelector) = (self perform:eachSelector) ifFalse:[
                ^ false
            ].
        ]
    ].
    ^ true
!

hasSettingsFile
    ^ self settingsFilename asFilename isReadable.
!

putSettingsOn:aStream
    |coder|

    coder := XMLStandardCoder on:aStream.
    coder aspect:#settings.
    coder nextPut:self.
!

readSettingsFromFile:aFilename
    |aTemporaryHTTPServer|

    aTemporaryHTTPServer := self class serverFromSettingsFile:aFilename.
    aTemporaryHTTPServer notNil ifTrue:[
        self fetchServerSettingsFrom:aTemporaryHTTPServer.
        ^ true
    ].
    ^ false
!

saveSettingsToFile:aFilename
    |stream|

    stream := aFilename asFilename writeStream.
    [
        self putSettingsOn:stream.
    ] ensure:[
        stream close.
    ].
    settingsFilename := aFilename asString.
!

settingsString
    ^ String streamContents:[:aStream|
        self putSettingsOn:aStream.
    ].

"
    HTTPServer new settingsString
"
! !

!HTTPServer methodsFor:'temporary link-add/remove'!

clearTemporaryLinkIds
    "Clear all temp linkIds"

    tentativeObjectService notNil ifTrue:[
        tentativeObjectService clearTemporaryLinkIds
    ].
!

expireOldTemporaryLinksFor:now
    "expire old temporary links"

    tentativeObjectService notNil ifTrue:[
        tentativeObjectService expireOldTemporaryLinksFor:now
    ].
!

graphicLinkIdFor:anImage
    "Return the linkId for a temporary graphic link."

    ^ self tentativeObjectService graphicLinkIdFor:anImage
!

graphicLinkIdFor:anImage expirationTimeDelta:anExpirationTimeDelta
    "Return the linkId for a temporary graphic link."

    ^ self tentativeObjectService graphicLinkIdFor:anImage expirationTimeDelta:anExpirationTimeDelta
!

removeTemporaryLinkInfoForUrlPath:urlPath
    "remove the temporary link if it is available"

    tentativeObjectService notNil ifTrue:[
        tentativeObjectService removeTemporaryLinkInfoForUrlPath:urlPath
    ].
!

temporaryLinkPathFor:anObject type:aType
    "Return the linkId for a temporary object."

    ^ self temporaryLinkPathFor:anObject type:aType expirationTimeDelta:nil
!

temporaryLinkPathFor:aValue type:aType expirationTimeDelta:anExpirationTimeDelta
    "Return the linkId for a temporary object with an lifeTime of anExpirationTimeDelta
     of anExpirationTimeDelta in seconds. aValue may be a block (or other #value-responding) 
     thing; its value is computed when a client asks for the link"

    ^ self tentativeObjectService
        temporaryLinkPathFor:aValue type:aType expirationTimeDelta:anExpirationTimeDelta
!

temporaryLinkPathForProcessBlock:aProcessBlock
    "returns a linkId, which ends up in the request processing block (the argument).
     see example below.

     this will be performed somehow later, not directly after a full request,
     enhance the timeout.

     the link will be removed after perform"
    |link|


    link := self 
        temporaryLinkPathFor:[[:request | "/ nested caused by the getter (which performs #value)
            aProcessBlock value:request.
            self removeTemporaryLinkInfoForUrlPath:link.
        ]] 
        type:#processBlock 
        expirationTimeDelta:#never.

    ^ link

    "
        |actionUrl|

        actionUrl := httpServer tentativeObjectService 
            temporaryLinkPathForProcessBlock:[:request | 
                request postDataFieldsAt:'uploadedFile'
            ].

        builder
            form; target:'iframe'; method:'post'; action:actionUrl; enctype:'multipart/form-data'; 
                input; name:'uploadedFile'; type:'file';
            formEnd.
    "

    "Created: / 19-03-2012 / 13:36:06 / sr"
! !

!HTTPServer methodsFor:'temporary link-queries'!

hasExpiredForLinkObject:aLinkObject time:aTime
    "return false if aLinkObject is valid (exists and has not expired). Otherwise return true"

    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService hasExpiredForLinkObject:aLinkObject time:aTime
    ].
    ^ true
!

hasExpiredForUrlPath:urlPath time:aTime
    "return false if urlPath is valid (exists and has not expired). Otherwise return true"

    <resource: #obsolete>

    self obsoleteMethodWarning.
    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService hasExpiredForUrlPath:urlPath time:aTime
    ].
    ^ true
!

hasTemporaryLinkForLinkObject:aLinkObject
    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService hasTemporaryLinkForLinkObject:aLinkObject
    ].
    ^ false
!

hasTemporaryLinkForUrlPath:aPath
    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService hasTemporaryLinkForUrlPath:aPath
    ].
    ^ false
!

temporaryLinkExpirationTime
    "return the time (in seconds) which controls how long temporary links should be kept"

    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService temporaryLinkExpirationTime
    ].
    ^ HTTPTentativeObjectService defaultTemporaryLinkExpirationTime
!

temporaryLinkExpirationTime:something
    "set the time (in seconds) which controls how long temporary links should be kept"

    something = self temporaryLinkExpirationTime ifTrue:[
        ^ self
    ].
    self tentativeObjectService temporaryLinkExpirationTime:something
!

temporaryLinkInfoForLinkObject:aLinkObject
    "retrieve a temporary objects linkInfo - or nil"

    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService temporaryLinkInfoForLinkObject:aLinkObject
    ].
    ^ nil
!

temporaryLinkInfoForUrlPath:urlPath
    "retrieve a temporary objects linkInfo - or nil"

    tentativeObjectService notNil ifTrue:[
        ^ tentativeObjectService temporaryLinkInfoForUrlPath:urlPath
    ].
    ^ nil
! !

!HTTPServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPServer.st,v 1.624 2012/03/23 09:43:23 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPServer.st,v 1.624 2012/03/23 09:43:23 sr Exp $'
! !

HTTPServer initialize!
