"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPService
	instanceVariableNames:'linkNames realm allowRobots httpServer visibleInHomePage
		noRealmBehavior replyPresenterClass overallNumberOfRequests'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

HTTPService class instanceVariableNames:'ClassResources'

"
 No other class instance variables are inherited by this class.
"
!

Object subclass:#NoRealmBehavior
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPService
!

HTTPService::NoRealmBehavior subclass:#All
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPService::NoRealmBehavior
!

HTTPService::NoRealmBehavior subclass:#LocalHostOnly
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPService::NoRealmBehavior
!

HTTPService::NoRealmBehavior subclass:#LocalNetOnly
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPService::NoRealmBehavior
!

HTTPService::NoRealmBehavior subclass:#Nobody
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPService::NoRealmBehavior
!

!HTTPService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This is an abstract class, providing common helpers for all http-Services.

    An HTTPService registers itself towards an HTTPServer (under a linkName),
    and then gets requests forwarded from that HTTPServer.
    Registration is by the top linkName - all requests for URLS below that name
    are sent as messages to the service 
    (a service may also be registered via multiple linkNames).

    I have two subclasses (which are also abstract):
        HTTPActionService   - which gets the full request as argument to its #process: method.
    and
        HTTPSelectorService - which dispatches to individual processing methods, depending on
                              the URLs next name component.

    To aid in programming threadsafe services there is a class named HTTPServiceRequestContext 
    that provides an interface to be inherited from. This allows you to store variables on a 
    per request basis. For every request an instance of your subclass is created where the 
    instance variable request is set. Before passing through the service, initialize is called 
    where you can do your own initialization of your variables. This enables you to have request 
    dependent state. At the end of the request release is called.

    Redefine the class method requestContextClass: for your service that returns your subclass 
    of HTTPServiceRequestContext.

    See more info in these subclasses and their concrete example subclasses.

    [see also:]
        HTTPServer
        HTTPActionService
        HTTPSelectorService
        HTTPInterface
"
! !

!HTTPService class methodsFor:'instance creation'!

new
    ^ self basicNew initialize
! !

!HTTPService class methodsFor:'browser interface'!

iconInBrowserSymbol
    "can be redefined for a private icon in the browser (for me and my subclasses).
     The returned symbol must be a selector of the ToolbarIconLibrary."

    ^ #webServiceClassBrowserIcon

    "Created: / 20-07-2007 / 09:59:45 / cg"
! !

!HTTPService class methodsFor:'class access'!

defaultReplyPresenterClass
    ^ nil
!

requestContextClass
    "returns a class which will be create a new instance for every new request
     the class have to be inherits from HTTPServiceRequestContext and contains the 
     request as an instance variable"

    ^ nil

    "Modified: / 02-02-2007 / 13:11:08 / cg"
! !

!HTTPService class methodsFor:'class initialization'!

initialize
    Smalltalk addDependent:self

    "
     self initialize
    "
! !

!HTTPService class methodsFor:'defaults'!

canChangeReplyPresenter
    "answer true, if the service can change its reply presenter.
     Subclasses may redefine this"

    ^ true
!

defaultAllowRobots
    ^ true
!

defaultPassword
    ^nil
!

defaultRealm
    ^nil
!

defaultUser
    ^nil
!

linkName
    "return the default path (with slash).
     Either redefine this method in a concrete subclass, 
     or redefine the sender, #linkNames (for multiple default linkNames)."

    ^ self subclassResponsibility
!

noRealmBehavior
    "specify how to behave if the service was registered without a realm.
     Possible return values:
        all                -    allow access for all
        nobody             -    deny access for all
        localHostOnly      -    only allow for local host
        localNetOnly       -    only allow for local network

     default is all here and localHostOnly in MaintenanceService"

    ^ HTTPService::NoRealmBehavior all  
! !

!HTTPService class methodsFor:'queries'!

isAbstract
    "if true, its not listed in wellknownServices and cannot be instantiated"

    ^ self == HTTPService
!

isBrowserStartable
    ^ self isAbstract not
!

isStartableWithStart
    ^ self isAbstract not
! !

!HTTPService class methodsFor:'queries-service'!

addToWellKnownClasses
    ^ true
!

compiledIntoSettingsDialog
    ^ true
!

isThreadSafe
    "return true, if this service is threadSafe.
     If not, only a single request is passed to the service at any time (by the HTTPServer);
     if it is, multiple requests may be served by multiple threads in parallel."

    ^ false
!

linkNames
    "return a collection of default pathes (with initial slashes).
     Either redefine this in a concrete service, or #linkName (for a single default linkName)."

    |nm|

    nm := self linkName.
    nm isNil ifTrue:[ ^ #() ].
    ^ Array with:nm
!

settingsApplicationClass
    "a SettingsApplication class - or nil (used in the settings dialog if non-nil)"

    "/ ^ self subclassResponsibility
    ^ nil
!

shownInSettingsDialog
    ^ true
! !

!HTTPService class methodsFor:'registration'!

availableServiceClassNames
    |loadedClassNames knownClassNames loadedButNotWellKnown code|

    loadedClassNames := self loadedServiceClassNamesWithFilter:[:aClass| aClass shownInSettingsDialog].
    knownClassNames := self wellknownServiceClassNames.
    knownClassNames := knownClassNames select:[:each | |cls|
                                                    cls := (Smalltalk at:each).
                                                    cls notNil
                                                    and:[ (cls isLoaded not)
                                                          or:[ cls shownInSettingsDialog]]
                                              ].

    loadedButNotWellKnown := loadedClassNames asSet copy.
    loadedButNotWellKnown removeAllFoundIn:knownClassNames.
    loadedButNotWellKnown := loadedButNotWellKnown 
                                select:[:aClassName |
                                    | cls |
                                    cls := Smalltalk at:aClassName.
                                    cls notNil and:[cls addToWellKnownClasses and:[cls compiledIntoSettingsDialog]]
                                ].
    knownClassNames := knownClassNames asSet.
    knownClassNames addAll:loadedClassNames.
    loadedButNotWellKnown size > 0 ifTrue:[
        "/ must update known classNames
        knownClassNames := knownClassNames asArray.
        knownClassNames sort.

        code := '' writeStream.
        code nextPutLine:'wellknownServiceClassNames'.
        code nextPutLine:'    "automagically generated - see #availableServiceClassNames."'.
        code nextPutLine:'    "Subclasses of HTTPService which respond to #addToWellKnownClasses with true"'.
        code nextPutLine:'    "are added here. Classes listed here will be shown in the settings dialog."'.
        code nextPutLine:''.
        code nextPutLine:'    ^ #('.

        knownClassNames do:[:eachName |
            code nextPutLine:'          ' , eachName storeString.
        ].
        code nextPutLine:'    )'.
        self class compile:code contents classified:'registration' logged:false.
    ].
    
    ^ knownClassNames asOrderedCollection sort

    "
     self availableServiceClassNames
    "
!

availableServiceClasses
    ^ self availableServiceClassNames collect:[:aClassName | Smalltalk at:aClassName asSymbol].

    "
     self availableServiceClasses
    "
!

loadedServiceClassNames
    ^ self loadedServiceClassNamesWithFilter:nil.

    "
     self loadedServiceClassNames
    "
!

loadedServiceClassNamesWithFilter:aBlock
    |classes|

    classes := HTTPService allSubclasses select:[:eachClass | eachClass isAbstract not].
    aBlock notNil ifTrue:[
        classes := classes select:[:aClass| aBlock value:aClass].
    ].
    classes := classes asOrderedCollection sort:[:a :b | a nameWithoutPrefix < b nameWithoutPrefix].
    ^ classes collect:[:each | each name].

    "
     self loadedServiceClassNames
    "
!

registerServiceIn:anHTTPServer 
    "create a new service and register it as service reachable via the default servicePath on anHTTPServer"

    <resource: #obsolete>
    self obsoleteMethodWarning:'use #registerServiceOn:'.
    ^ self registerServiceOn:anHTTPServer
!

registerServiceOn:anHTTPServer 
    "create a new service and register it as service reachable via the default servicePath on anHTTPServer"
    
    ^ self new registerServiceOn:anHTTPServer
!

unRegisterServiceIn:anHTTPServer 
    <resource: #obsolete>
    self obsoleteMethodWarning:'use #unRegisterServiceOn:'.
    ^ self unRegisterServiceOn:anHTTPServer
!

unRegisterServiceOn:anHTTPServer 
    "unregister a service class on anHTTPServer"

    |services|

    services := anHTTPServer servicesForServiceClass:self.
    services do:[:aServices|
        aServices unRegisterServiceOn:anHTTPServer
    ].
!

wellknownServiceClassNames
    "automagically generated - see #availableServiceClassNames."
    "Subclasses of HTTPService which respond to #addToWellKnownClasses with true"
    "are added here. Classes listed here will be shown in the settings dialog."

    ^ #(
          #ComancheSTTService
          #ComancheSwikiService
          #'Demos::HelloWorldService'
          #'Demos::WebDemoApp1'
          #'Demos::WebDemoShop'
          #'Demos::WebDemoShopSessions'
          #'Demos::WebHomePageForSTX'
          #'Demos::WebProcessList'
          #'Demos::WebScreenImageService'
          #'Demos::WebSlideShow'
          #'Demos::WebSmalltalkBrowser'
          #'Expecco::ExpeccoSOAPService'
          #'Expecco::HTTPAbstractService'
          #'Expecco::HTTPExecutionService'
          #'Expecco::HTTPTestplanService'
          #'Expecco::SimpleInfoService'
          #'ExpeccoNET::ExpeccoNETService'
          #HTTPCGIService
          #HTTPClassDocService
          #HTTPEmbeddedApplicationService
          #HTTPFileService
          #HTTPPluggableActionService
          #HTTPPortalService
          #HTTPSoapSprayService
          #HTTPSourceCodeAccessService
          #HTTPTentativeObjectService
          #HTTPUtilityService
          #MacroExpandingCSSFileService
          #MyWeb
          #SeasideService
          #ServerMaintenanceService
    )
! !

!HTTPService class methodsFor:'resources'!

classResources
    "if not already loaded, get the classes resourcePack and return it"

    ClassResources isNil ifTrue:[
        ClassResources := ResourcePack for:self.
    ].
    ^ ClassResources

    "
     Procware::ProcWareService classResources
    "
!

flushAllClassResources
    ResourcePack flushCachedResourcePacks.
    HTTPService withAllSubclassesDo:[:aClass |
        aClass flushClassResources.
    ]
!

flushClassResources
    ClassResources := nil
!

update:something with:aParameter from:changedObject
    "flush resources on language changes"

    something == #Language ifTrue:[
        self flushAllClassResources
    ]

    "Created: 15.6.1996 / 15:13:29 / cg"
! !

!HTTPService class methodsFor:'startup'!

start
    "start the service on the default web server (8080) if there is one.
     If there is none, ask if it should be started"

    |answer httpServer portNrString portNr|

    (httpServer := HTTPServer defaultServer) isNil ifTrue:[
        answer := Dialog confirmWithCancel:('No default webServer is running.\\Start a server on the default port %1 ?' bindWith:HTTPServer defaultPort) withCRs.
        answer isNil ifTrue:[^ nil].

        answer ifTrue:[
            (httpServer := HTTPServer startServer) isNil ifTrue:[
                Dialog warn:('Could not start a server on port %1 ?' bindWith:HTTPServer defaultPort).
                ^ nil.
            ].
        ] ifFalse:[
            portNrString := Dialog request:'Start or use a server on port:' initialAnswer:'8081'.
            portNrString isEmptyOrNil ifTrue:[^ nil].

            portNr := Integer readFrom:portNrString onError:[ Dialog information:'Invalid port'. ^ nil].
            httpServer := HTTPServer serverOnPort:portNr.

"/                HTTPServer runningServers isEmptyOrNil ifTrue:[
"/                    ^ nil
"/                ].
"/                httpServer := HTTPServer runningServers first.
"/                (Dialog confirm:('Use the webServer on port %1 ?' bindWith:httpServer port) withCRs)
"/                ifFalse:[
"/                    ^ nil
"/                ]
        ].
    ].
    self information:('Start as "%1" on port "%2"' bindWith:self linkName with:httpServer port).
    ^ self startOn:httpServer.

    "Modified: / 02-06-2011 / 12:00:25 / cg"
!

startOn:httpServer
    "start(restart) the service on the given server."

    self unRegisterServiceOn:httpServer.
    ^ self registerServiceOn:httpServer.
! !

!HTTPService methodsFor:'accessing'!

allowRobots
    "true, if robots are allowed to enter this service"

    allowRobots isNil ifTrue:[
        ^ self class defaultAllowRobots
    ].
    ^ allowRobots
!

allowRobots:something
    "true, if robots are allowed to enter this service"

    allowRobots := something.
!

contextForRequest:aRequest
    ^ aRequest serviceRequestContext
!

httpServer
    ^ httpServer
!

httpServer:something
    httpServer := something.
!

noRealmBehavior:something
    noRealmBehavior := something.
!

realm:something
    "set the realm - if non-nil, the request must be authorized"

    realm := something.

    "Modified: / 02-02-2007 / 12:46:06 / cg"
!

replyPresenter
    <resource: #obsolete>

    "return the class name of the replyPresenter,
     the setter does switch between class name and instance (from old settings files)"

    replyPresenterClass isNil ifTrue:[^ nil].

    ^ replyPresenterClass name

    "Modified: / 10-07-2009 / 16:06:58 / sr"
!

replyPresenter:something
    <resource: #obsolete>
    "this has become obsolete, use #replyPresenterClass"

    something isSymbol ifTrue:[
        self replyPresenterClass:(Smalltalk at:something).    
    ] ifFalse:[
        self halt:'fix your settings file, provide a class name'.
    ].

    "Modified: / 10-07-2009 / 13:19:51 / sr"
!

replyPresenterClass
    ^ replyPresenterClass

    "Modified: / 02-02-2007 / 13:03:27 / cg"
!

replyPresenterClass:aClassOrNil
    replyPresenterClass := aClassOrNil.

    "Created: / 02-02-2007 / 12:59:33 / cg"
!

replyPresenterForRequest:aRequest
    replyPresenterClass notNil ifTrue:[
        ^ replyPresenterClass new.
    ].
    ^ nil

    "Modified: / 02-02-2007 / 13:11:30 / cg"
!

requestContextForRequest:aRequest
    | requestContextClass |

    (requestContextClass := self class requestContextClass) notNil ifTrue:[
        ^ requestContextClass new.
    ].
    ^ nil
!

visibleInHomePage
    ^ visibleInHomePage ? true
!

visibleInHomePage:something
    visibleInHomePage := something.
! !

!HTTPService methodsFor:'authorization'!

addUser:aUser password:aPassword toRealm:aRealm 
    "this message is sent if a user is added to a realm or changed.
     Subclasses may redefine this"
!

authorizeRequest:aRequest 
    "ensure, that a request ist authorized to use this service.
     This is the case, if it is authenticated for a user in the services realm"
    
    |serviceRealm|

    serviceRealm := self realm.
    serviceRealm isNil ifTrue:[
        (self noRealmBehavior isAuthorizedFor:aRequest) ifFalse:[
            HTTPAccessDeniedError raiseErrorString:'Access denied'.
        ].
        "a request may contain an authenticator for another realm - clear"
        aRequest authenticator:nil.
    ] ifFalse:[
        aRequest authenticateForRealm:serviceRealm.
        "not reached if authentication failed"
    ].
!

canAuthenticate:aRequest with:authenticator
    "is it allowed to authenticate using authenticator?
     by default, forward to my server. However, individual services might
     redefine this to be more or less strict."

    ^ httpServer canAuthenticate:aRequest with:authenticator
!

noRealmBehavior
    "specify how to behave if the service was registered without a realm"

    noRealmBehavior isNil ifTrue:[
        noRealmBehavior := self class noRealmBehavior.  
    ].
    ^ noRealmBehavior
!

usersForRealm:aRealm
    "by default, forward to my server. However, individual services might
     redefine this to define their own, private user-groups"

    ^ httpServer usersForRealm:aRealm
! !

!HTTPService methodsFor:'initialize & release'!

initialize
    replyPresenterClass := self class defaultReplyPresenterClass.
    overallNumberOfRequests := 0.

    "Created: / 02-02-2007 / 13:10:48 / cg"
!

postStopServing
    "this is sent after the HTTPServer stops to accept connections
     Can be redefined in subclasses for doing any actions "
!

preStartServing
    "this is sent before the HTTPServer starts to accept connections
     Can be redefined in subclasses for doing any actions "
!

shutdown
    "invoked when deregistered, after the last request can be served.
     Can be redefined in subclasses to perform service specific cleanup actions."
!

startup
    "invoked when registered, before the first request can be served.
     Can be redefined in subclasses to perform service specific startup actions."
! !

!HTTPService methodsFor:'maintenance html presentation'!

commonChangeFromRequest:aRequest
    "extracts parameter from an html-maintenance post-request
     (common parameters, valid for all services)"

    | linkNamesValue realmValue visibleInHomePageValue allowRobotsValue replyPresenterClassValue|

    linkNamesValue := aRequest urlParameterAt:'LinkNames'.
    linkNamesValue notNil ifTrue:[
        linkNamesValue := linkNamesValue asCollectionOfSubstringsSeparatedBy:$,.
        self linkNames:linkNamesValue.
    ].
    realmValue := aRequest urlParameterAt:'Realm'.
    realmValue = 'None' ifTrue:[ realmValue := nil].
    self realm:realmValue.

    replyPresenterClassValue := aRequest urlParameterAt:'replyPresenterClass'.
    replyPresenterClassValue = 'None' ifTrue:[ replyPresenterClassValue := nil].
    replyPresenterClassValue notNil ifTrue:[
        replyPresenterClassValue := Smalltalk classNamed:replyPresenterClassValue asSymbol.
        self replyPresenterClass:replyPresenterClassValue.
    ] ifFalse:[
        self replyPresenterClass:nil.
    ].

    allowRobotsValue := aRequest urlParameterAt:'AllowRobots'.
    self allowRobots:(allowRobotsValue = 'ON').

    visibleInHomePageValue := aRequest urlParameterAt:'VisibleInHomePage'.
    self visibleInHomePage:(visibleInHomePageValue = 'ON').
!

commonInputFieldsForRequest:aRequest change:change
    "generates the html-fragment for maintenance form
     (common fields, valid for all services)"

    |realms selected presenterClasses|

"/ Hidden index in service List
    change ifTrue:[
        aRequest 
            nextPutLine:'<H2>';
            inputFieldFor:'IndexInServiceList' 
                type:'hidden' 
                size:nil 
                label:'' 
                initialValue:((aRequest server serviceList asOrderedCollection indexOf:self) asString)
                append:'READONLY';
                nextPutLine:'<BR></H2>'.
    ].    
"/ ServiceClass
    aRequest 
        nextPutLine:'<H2>';
        inputFieldFor:'ServiceClass' 
            type:nil 
            size:'40' 
            label:('Add / Change Service:&nbsp;&nbsp;&nbsp;') 
            initialValue:self className
            append:'READONLY';
            nextPutLine:'<BR></H2>'.
"/ LinkNames
    aRequest 
        nextPutLine:'<TABLE WIDTH="100%">';
        nextPutLine:'<TR><TD>'.

    aRequest 
        inputFieldFor:'LinkNames' 
        type:nil 
        size:'50' 
        label:'Link Names:</TD><TD>' 
        initialValue:(self linkNames asStringWith:',');
        nextPutLine:'<BR>';
        nextPutLine:' (for more than one seperate by ",")';
        nextPutLine:'</TD></TR>'.

   aRequest
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

"/ Realm
    aRequest 
        nextPutLine:'<TR><TD>';
        nextPutLine: 'Service Realm:</TD><TD>';
        nextPutLine: '<SELECT NAME="Realm" SIZE="1"> '.
    realms := aRequest server realmUsersMap keys asOrderedCollection.
    realms addFirst:'None'.
    realms do:[:aKey |
         | serviceRealm |
        serviceRealm := self realm ? 'None'.
        aKey = self realm ifTrue:[
            selected := ' SELECTED'.
        ] ifFalse:[
            selected := ''.
        ].
        aRequest
            nextPutLine: '<OPTION ', selected,'>', aKey asString,'</OPTION>'.
    ].
    aRequest
        nextPutLine: '</SELECT>';
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

"/ ReplyPresenter
    self class canChangeReplyPresenter ifTrue:[
        |servicePresenterClass|

        aRequest 
            nextPutLine:'<TR><TD>';
            nextPutLine: 'Reply Presenter:</TD><TD>';
            nextPutLine: '<SELECT NAME="replyPresenterClass" SIZE="1"> '.

        self replyPresenterClass isNil ifTrue:[
            servicePresenterClass := 'None'.
        ] ifFalse:[
            servicePresenterClass := self replyPresenterClass name.
        ].
        presenterClasses := ReplyPresenter presenterClasses collect:[:each| each name].
        presenterClasses addFirst:'None'.
        presenterClasses do:[:eachPresenterClassName |
            eachPresenterClassName = servicePresenterClass ifTrue:[
                selected := ' SELECTED'.
            ] ifFalse:[
                selected := ''.
            ].
            aRequest
                nextPutLine: '<OPTION ', selected,'>', eachPresenterClassName,'</OPTION>'.
        ].
        aRequest
            nextPutLine: '</SELECT>';
            nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.
    ].

"/ Allow Robots
    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:('AllowRobots') 
            type:'checkbox' 
            size:nil 
            label:('Allow Robots:</TD><TD>') 
            initialValue:'ON'
            append:(self allowRobots ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

"/ VisibleInHomePage
    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:('VisibleInHomePage') 
            type:'checkbox' 
            size:nil 
            label:('Visible in Home Page:</TD><TD>') 
            initialValue:'ON'
            append:(self visibleInHomePage ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'</TABLE>'.
!

specificChangeFromRequest:aRequest
    "extracts parameter from an html-maintenance post-request;
     service specific - to be redefined by services."

    "/ intentionally left blank
!

specificInputFieldsForRequest:aRequest
    "generates the html-fragment for the maintenance form;
     service specific - to be redefined by services."

    "/ intentionally left blank
! !

!HTTPService methodsFor:'printing'!

nameInSettingsTable
    "redefined in ComancheService, which adds the module-name"

    ^ self className
!

printOn:aStream
    aStream nextPutAll:self className.
    aStream nextPut:$[.

    self linkNames do:[:aLinkName |
        aStream nextPutAll:aLinkName.
    ] separatedBy:[
        aStream nextPutAll:', '.
    ].
    aStream nextPut:$].
! !

!HTTPService methodsFor:'queries'!

allowRobotsPaths
    <resource: #obsolete>
    "cg: who was it to name this method just the opposite of what it really does ?"

    self obsoleteMethodWarning.
    ^ self forbiddenRobotsPaths
!

debugErrors
    "answer true, if we run in debug mode,
     i.e., a debugger should be opened on error"

    ^ httpServer debugErrors
!

forbiddenRobotsPaths
    "return the disallowed paths for robots"
    
    |forbiddenRobotsPaths|

    self allowRobots ifTrue:[^ #() ].

    forbiddenRobotsPaths := OrderedCollection new.
    self linkNames do:[:aLinkName|
        forbiddenRobotsPaths add:aLinkName.
    ].
    ^ forbiddenRobotsPaths
!

isRunning
    "answer true if service is running"

    ^ httpServer notNil and:[ httpServer isRunning ].
!

overallNumberOfRequests
    "the overall number of incoming requests"

    ^ overallNumberOfRequests.
! !

!HTTPService methodsFor:'queries-service'!

isAllowedInMaintenanceMode
    "return true, if this service is also allowed while the server is in maintenance mode"

    ^ false
!

isFileService
    "anwser true, if this services serves files"

    ^ false
!

linkName
    "return the pathName of this service (incl. slash); 
     i.e. if '/Bla' is returned, the service is reached as http://hostname:port/Bla.
     If this service is registered under multiple names, only the first is returned here."

    |linkNames|

    linkNames := self linkNames.
    linkNames isEmpty ifTrue:[^ nil].
    ^ linkNames first.
!

linkName:aString
    "set the pathName of this service (incl. slash);
     i.e. if '/Bla' is passed as argument, the service will be reached as http://hostname:port/Bla."

    self linkNames:(Array with:aString)
!

linkNames
    "return the relative pathName of this service; i.e. if
     'Bla' is returned, the service is reached as http://hostname:port/Bla"

    linkNames isNil ifTrue:[
        linkNames := self class linkNames
    ].
    ^ linkNames
!

linkNames:aLinkNameCollection 
    "set the names, under this service may be reaches; 
     e.g. if you set linkNames to #('/xxx' '/yyy') this service may be reached as both
     'http://hostname:port/xxx' and 'http://hostname:port/yyy'"
    
    |canonizedLinkNames|

    "strip trailing /"
    canonizedLinkNames := aLinkNameCollection 
                collect:[:eachLinkName | HTTPServer makeValidLinkName:eachLinkName. ].
    linkNames ~= canonizedLinkNames ifTrue:[
        linkNames := canonizedLinkNames.
        httpServer notNil ifTrue:[
            httpServer changeLinksForService:self.
        ].
    ].
!

nameInSettingsDialog
    |className|

    className := self class nameWithoutPrefix.
    (className startsWith:'HTTP') ifTrue:[
        ^ className copyFrom:5.
    ].
    ^ className
!

realm
    "return the realm - if non nil, the request must be authorized"

    ^ realm
!

settingsApplicationClass
    "answer a class that represents an Application used to configure
     the service's settings - or nil (used in the settings dialog if non-nil)"

    ^ self class settingsApplicationClass.
!

skipInstvarIndexInDeepCopy:anIndex
    "answer true, if the instance variable at: anIndex 
     should not be copied on #deepCopy"

    "we copy only our settings"
    ^ (self settingSelectors includes:(self class allInstVarNames at:anIndex)) not.
!

suppressInHomePage
    "answer true, if the service should not be visible in the server's home page"

    ^ false
! !

!HTTPService methodsFor:'registration'!

addAuthentificationTo:aServer 
    |defaultRealm user password|

    defaultRealm := self class defaultRealm.
    defaultRealm isNil ifTrue:[
        ^ self
    ].
    aServer addRealm:defaultRealm.
    user := self class defaultUser.
    password := self class defaultPassword.
    (user notNil and:[ password notNil ]) ifTrue:[
        aServer 
            addUser:user
            password:password
            toRealm:defaultRealm
            overWrite:false
    ].
!

addRequiredForeignServicesTo:aServer
    "hook to allow for a service to register other required services.
     Typically, these are required file services for bitmaps or icons.
     Redefined in subclasses."

    "/ intentionally left blank
!

addStandardFileServicesTo:aServer
    "many services need the files in data/sampleServerDir/files.
     This method registers a corresponding file service"

    (aServer serviceForLink:'/files' ifAbsent:nil) isNil ifTrue:[
        aServer 
            addFileServiceWithRootLinkName:'/files'
            rootDirectory:'../../goodies/webServer/data/sampleServerDir/files'
            defaultFileName:nil.
    ].
!

isRegistered
    "answer true, if this service is registered with a server"

    ^ httpServer notNil
!

registerServiceIn:aServer 
    "register myself as service on a single HTTPServer"

    <resource: #obsolete>
    
    self obsoleteMethodWarning:'use #registerServiceOn:'.
    ^ self registerServiceOn:aServer
!

registerServiceOn:aServer 
    "register myself as service on a single HTTPServer"
    
    aServer isNil ifTrue:[
        'HTTPService [warning]: trying to register service on non-existing server' errorPrintCR.
        self error:'trying to register service on non-existing server'.
    ].

    httpServer notNil ifTrue:[
        httpServer ~~ aServer ifTrue:[
            'HTTPService [warning]: trying to register service on multiple servers' errorPrintCR.
            self error:'trying to register service on multiple servers'.
        ].
    ].

    self addRequiredForeignServicesTo:aServer.
    self addAuthentificationTo:aServer.

    aServer linkService:self.
    httpServer := aServer.
    self startup.

    "Modified: / 01-02-2007 / 21:01:04 / cg"
!

release
    httpServer notNil ifTrue:[
        self unRegisterServiceOn:httpServer.
    ].
!

removeRequiredForeignServicesFrom:aServer 
    ^ self
!

unRegister
    "unregister myself from my HTTPServer"

    httpServer notNil ifTrue:[
        self unRegisterServiceOn:httpServer
    ]
!

unRegisterServiceIn:anHTTPServer
    <resource: #obsolete>
    self obsoleteMethodWarning:'use #unRegisterServiceOn:'.
    ^ self unRegisterServiceOn:anHTTPServer
!

unRegisterServiceOn:aServer
    "unregister myself from an HTTPServer"

    (httpServer notNil and:[httpServer ~~ aServer]) ifTrue:[
        'HTTPService [warning]: unregister service from wrong server' errorPrintCR.
"/        self error:'trying to unregister service from wrong server'.
    ].

    self removeRequiredForeignServicesFrom:aServer.
    aServer unlinkService:self.
    self shutdown.
    httpServer := nil.
! !

!HTTPService methodsFor:'request handling'!

_process:aRequest
    "internal processing method: care for synchronization"

    overallNumberOfRequests := (overallNumberOfRequests ? 0) + 1.
    self class isThreadSafe ifTrue:[
        self process:aRequest.    
    ] ifFalse:[
        self synchronized:[self process:aRequest].
    ].
!

process:aRequest
    "all the work is done in this method.
     Any service has to implement this method"

    self subclassResponsibility
! !

!HTTPService methodsFor:'settings persistency'!

elementDescriptorFor:anAspect
    "support for persistency:
     answer the elements to be made persistent with an ObjectCoder"

    anAspect ~~ #settings ifTrue:[
        ^ super elementDescriptorFor:anAspect
    ].
    ^ self settingSelectors collect:[:eachVarName | eachVarName -> (self perform:eachVarName asSymbol)]
            thenSelect:[:pairs | pairs value notNil]
!

settingSelectors
    "answer the selectors that are used to configure a service - i.e. those which are
     saved in the webServer-settings file."

    "/ cg: changed to use a positive list, instead of an exclusion algorithm.
    "/ because it makes always trouble when an instvar is added.
    "/ slots := self class allInstVarNames copyWithoutAll:(HTTPService allInstVarNames).
    "/ slots addAll:#(linkNames realm httpServer replyPresenter).

    ^ #(linkNames realm httpServer replyPresenter).

    "
       HTTPService new settingSelectors
    "
! !

!HTTPService::NoRealmBehavior class methodsFor:'authentication'!

isAuthorizedFor:aRequest 
    "answer true, if policy authorizes aRequest"
    
    ^ self subclassResponsibility
! !

!HTTPService::NoRealmBehavior class methodsFor:'class access'!

all
    ^ All
!

localDomainOnly
    "bad name, use localNetOnly"
    <resource:#obsolete>

    ^ LocalNetOnly
!

localHostOnly
    ^ LocalHostOnly
!

localNetOnly
    ^ LocalNetOnly
!

nobody
    ^ Nobody
! !

!HTTPService::NoRealmBehavior::All class methodsFor:'authentication'!

isAuthorizedFor:aRequest 
    "allow all requests"
    
    ^ true.
! !

!HTTPService::NoRealmBehavior::LocalHostOnly class methodsFor:'authentication'!

isAuthorizedFor:aRequest 
    "allow request from localhost only"

    |peer|

    peer := aRequest peer.
    ^ peer isLocal or:[peer hostAddress = aRequest socketAddress hostAddress]
! !

!HTTPService::NoRealmBehavior::LocalNetOnly class methodsFor:'authentication'!

isAuthorizedFor:aRequest 
    "allow requests from one of our own networks"

    ^ aRequest peer networkAddress = aRequest socketAddress networkAddress
! !

!HTTPService::NoRealmBehavior::Nobody class methodsFor:'authentication'!

isAuthorizedFor:aRequest 
    "deny all requests"
    
    ^ false.
! !

!HTTPService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPService.st,v 1.185 2011/06/02 11:05:56 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPService.st,v 1.185 2011/06/02 11:05:56 cg Exp $'
! !

HTTPService initialize!
