"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HTTPSession
	instanceVariableNames:'cookie server stateDictionary lastAccessedTime
		maxInactiveInterval isNew'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!HTTPSession class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Summary:
    
    This class provides a way to store state across multiple page requests. It is achieved through
    the use of a 'session cookie' that is transported between the client and the server. The cookie's
    value uniquely identifes a session (this value is generated from via the UUID class) and provides 
    a Dictionary like interface to access the state associated with the session.

    The state itself is stored in a Dictionary in the HTTPServer. Every hour a job
    is run that removes all the sessions that have timed-out. This happens when there has
    been no closure to a session. Sessions can also be removed explicitly.

    Limitations:

    As yet there is no support for URL rewrite functionality.

    [author:]
        James (james@kokxnix)

    [instance variables:]

    [class variables:]

    [see also:]

"
!

history
    "Created: / 05-10-2004 / 13:35:44 / james"
! !

!HTTPSession class methodsFor:'instance creation'!

cookie:aHTTPCookie server:aHTTPServer
    "Checks the HTTP server if a HTTP session has been created with the same
     id as the cookie sent. If so return this session with the cookie in hand 
     (the expiration could have changed)

     <return: HTTPSession>"

    |aSession|

    aSession := aHTTPServer httpSessionAt:aHTTPCookie value ifAbsent:[]. 
    aSession isNil ifTrue:[
        aSession := self new cookie:aHTTPCookie server:aHTTPServer.
        aHTTPServer httpSessionAt:aHTTPCookie value put:aSession. 
    ] ifFalse:[
        aSession cookie:aHTTPCookie.
        aSession lastAccessedTime:Timestamp now.
        aSession makeActive.
    ].

    ^ aSession
!

new
    ^ self basicNew initialize.
! !

!HTTPSession class methodsFor:'constants'!

defaultMaxInactiveInterval
    "how long may a session be inactive until it is removed"

    ^ 60 * 60  "1 hour"
! !

!HTTPSession methodsFor:'accessing'!

at:aKey
    ^ stateDictionary at:aKey 
!

at:aSymbol ifAbsent:anObject 
    ^ stateDictionary at:aSymbol ifAbsent:anObject 
!

at:aSymbol ifAbsentPut:aBlock 
    ^ stateDictionary at:aSymbol ifAbsentPut:aBlock 
!

at:aSymbol ifPresent:aOneArgBlock 
    ^ stateDictionary at:aSymbol ifPresent:aOneArgBlock 
!

at:aKey put:aSymbol

    ^ stateDictionary at:aKey put:aSymbol
!

cookie
    ^ cookie
!

cookie:something
    cookie := something.
!

cookie:cookieArg server:serverArg 
    cookie := cookieArg.
    server := serverArg.
!

expiration
    ^ cookie expiration 
!

isNew
    ^ isNew
!

lastAccessedTime
    ^ lastAccessedTime
!

lastAccessedTime:something
    lastAccessedTime := something.
!

removeKey:aKey

    ^ stateDictionary removeKey:aKey ifAbsent:nil
! !

!HTTPSession methodsFor:'actions'!

invalidate
    "Invalidates the session and any object bound to it"

    server httpSessionRemove:self getId.
    cookie := nil.
    server := nil.
    stateDictionary := nil.
    lastAccessedTime := nil.
    maxInactiveInterval := nil.
!

makeActive
    "The session has been accepted by the client"

    isNew := false.
! !

!HTTPSession methodsFor:'initialization'!

initialize              
    lastAccessedTime := Timestamp now.
    maxInactiveInterval := self class defaultMaxInactiveInterval. "seconds"
    stateDictionary := Dictionary new.
    isNew := true.
! !

!HTTPSession methodsFor:'jsp-specific api'!

getAttributeNames
"
    <return: OrderedCollection>
"
    ^  stateDictionary keys
!

getId
"
    Returns the id of the session which is the value of the cookie

    <return: String>
"
    ^  cookie value
!

getLastAccessedTime
"

    <return: Timestamp>
"
    ^  lastAccessedTime
!

getMaxInactiveInterval
"
    Returns the maximum time interval that the this session will be alive for

    <return: Time>
"
    ^  maxInactiveInterval
!

removeAttribute:aString 
    ^ self removeKey:aString 
!

setAttribute:aString to:anObject
    ^ self at:aString put:anObject
!

setMaxInactiveInterval:aValue
"
    Sets the maximum time interval that the this session will be alive for in SECONDS
"
    maxInactiveInterval := aValue
! !

!HTTPSession methodsFor:'queries'!

isValid
    "Return true if the session is a valid session
     <return: Boolean>"

    |now|

    maxInactiveInterval isNil ifTrue:[
        ^ true
    ].

    now := Timestamp now.
    (now subtractSeconds:maxInactiveInterval) > lastAccessedTime ifTrue:[
        ^ false
    ].

    ^ self expiration isNil or:[now < self expiration]
! !

!HTTPSession class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPSession.st,v 1.12 2010/02/02 11:12:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPSession.st,v 1.12 2010/02/02 11:12:41 cg Exp $'
! !
