"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/webServer' }"

HTTPSelectorService subclass:#HTTPSourceCodeAccessService
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

!HTTPSourceCodeAccessService class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    a service which provides access to the running smalltalks sourcecode

    [usage:]
        HTTPSourceCodeAccessService registerServiceIn:(HTTPServer runningServers first) as:'Bla'

    [author:]
        Claus Gittinger

    [access with:]
        http:<host>:<port>/SourceCode/get/<className>
"
! !

!HTTPSourceCodeAccessService class methodsFor:'defaults'!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
!

linkName
    "return the default linkName path (with slash)"

    ^ '/SourceCode'
! !

!HTTPSourceCodeAccessService class methodsFor:'queries-service'!

isThreadSafe
    ^ true
!

settingsApplicationClass
    "a SettingsApplication class - or nil (used in the settings dialog if non-nil)"

    ^ nil
! !

!HTTPSourceCodeAccessService methodsFor:'queries-access'!

exportedModules
    "list the modules we make accessible"

    ^ #('stx')
! !

!HTTPSourceCodeAccessService methodsFor:'queries-service'!

allowedMessages
    ^ #(get: getText: howTo:).
!

defaultMessage
    ^ #howTo
! !

!HTTPSourceCodeAccessService methodsFor:'registration'!

addRequiredForeignServicesTo:aServer 
    "I need a file service for the bitmaps and icons"

    self addStandardFileServicesTo:aServer.
! !

!HTTPSourceCodeAccessService methodsFor:'requests-server-classDoc'!

get:aRequest
    self getSource:aRequest mimeType:'application/smalltalk-source'
!

getSource:aRequest mimeType:mimeType
    |className class source|

    aRequest arguments notEmptyOrNil ifTrue:[
        className := aRequest arguments asStringCollection asStringWith:'::'.
    ] ifFalse:[aRequest postDataFields notEmptyOrNil ifTrue:[
        className := aRequest postDataFields first value.      
    ]].

    className isEmptyOrNil ifTrue:[
        aRequest reportBadRequest:'Classname argument missing.'.
    ].

    class := Smalltalk classNamed:className.
    (class notNil and:[class isLoaded]) ifFalse:[
        aRequest reportNotFound:'Class named "' , className , '" does not exist in this Smalltalk image.'.
    ].
    (self exportedModules includes:(class package upToAny:':/')) ifFalse:[
        "do not show non-public packages"
        aRequest reportNotFound:'No access to class named "' , className , '".'.
        "/ not reached
        ^ self
    ].
    source := class source asString.
    source isNil ifTrue:[
        aRequest reportServerError:'Empty source for class "' , className, '"' moreInfo:nil.
    ].

    aRequest responseContentType:mimeType.
    aRequest response data:source.
!

getText:aRequest
    self getSource:aRequest mimeType:'text/plain'
!

howTo:aRequest
    |myURLBase replyPresenter|

    myURLBase := aRequest serviceLinkName.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Smalltalk SourceCode Access'.
    replyPresenter parametersAt:#secondHeadLine put:'Smalltalk SourceCode Access'.
    aRequest
        reply:'

This application (<code>',self class name,'</code>) provides upToDate access to the running smalltalks source code.
<br>For now, this is a simple demo - but might be extended to an automatic software update service in the future.
<br>
To get a classes source code, type the following into your browsers URL-entry field:
<br>
<blockquote>
' , myURLBase , '/get/ClassName
</blockquote>
<p>
As an example, the following link gives you the source of the Boolan class:
<BR>
<BLOCKQUOTE>
' , (aRequest webLinkForText:(myURLBase , '/get/Boolean') service:self selector:#'get:' argument:'Boolean') , '
</BLOCKQUOTE>
<P>
The above returns a classes sourceCode with a mimeType of
"application/smalltalk-source", which most browsers will save into a file 
(ready to be filedIn to your running smalltalk).
<p>
If you want to see the source code (as opposed to saving it somewhere), 
use "getText" instead of "get";
<BR>i.e.
<BLOCKQUOTE>
' , (aRequest webLinkForText:(myURLBase , '/getText/Boolean') service:self selector:#'getText:' argument:'Boolean') , '
</BLOCKQUOTE>
this returns the source code with a mimeType of "text/plain", which will be displayed by your browser.'.
! !

!HTTPSourceCodeAccessService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPSourceCodeAccessService.st,v 1.33 2006/07/28 20:13:32 stefan Exp $'
! !
