"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

HTTPService subclass:#HTTPTentativeObjectService
	instanceVariableNames:'temporaryLinks temporaryObjectNames lock
		temporaryLinkExpirationTime'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Services'
!

Object subclass:#TemporaryLinkInfo
	instanceVariableNames:'linkExpirationTime browserCacheExpirationTime linkName
		temporaryObject type'
	classVariableNames:''
	poolDictionaries:''
	privateIn:HTTPTentativeObjectService
!

!HTTPTentativeObjectService class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Serves temporary objects. Typically images, which are generated on the fly.
    The objects are flushed after a while.
"
! !

!HTTPTentativeObjectService class methodsFor:'defaults'!

defaultTemporaryLinkExpirationTime
    "default temporary link expiration time in seconds"

    ^ 60*2  "2 minutes"
!

linkName
    "return the default path (with slash)"

    ^ '/tentativeObjects'
!

longTemporaryLinkBrowerCacheExpirationTime
    "temporary browerCache expiration time in seconds for things which expire never in the service"

    ^ 60*60*24  "1 day"
!

temporaryLinkExpirationTime
    <resource: #obsolete>
    "default temporary link expiration time in seconds"

    self obsoleteMethodWarning.
    ^ self defaultTemporaryLinkExpirationTime
! !

!HTTPTentativeObjectService class methodsFor:'queries'!

isThreadSafe
    ^ true
! !

!HTTPTentativeObjectService methodsFor:'accessing'!

temporaryLinkExpirationTime
    "return the time (in seconds) which controls how long temporary links should be kept"

    temporaryLinkExpirationTime isNil ifTrue:[
        temporaryLinkExpirationTime := self class defaultTemporaryLinkExpirationTime.
    ].
    ^ temporaryLinkExpirationTime
!

temporaryLinkExpirationTime:something
    "set the time (in seconds) which controls how long temporary links should be kept"

    temporaryLinkExpirationTime := something.
! !

!HTTPTentativeObjectService methodsFor:'initialization'!

initialize
    super initialize.
    lock := RecursionLock forMutualExclusion.
    temporaryLinks := IdentityDictionary new.
    temporaryObjectNames := Dictionary new.
! !

!HTTPTentativeObjectService methodsFor:'maintenance html presentation'!

specificChangeFromRequest:aRequest 
    "extracts parameter from an html-maintenance post-request"

    |tmp|

    tmp := aRequest urlParameterAt:'tentativeObjectExpirationTime'.
    tmp notNil ifTrue:[
        self temporaryLinkExpirationTime:(tmp asNumber).
    ].
!

specificInputFieldsForRequest:aRequest
    aRequest 
        inputFieldFor:'tentativeObjectExpirationTime' 
            type:nil 
            size:'10' 
            label:('Tentative Object Expiration Time (Seconds): ') 
            initialValue:self temporaryLinkExpirationTime asString;
        nextPutLine:'<br />'.
! !

!HTTPTentativeObjectService methodsFor:'persistency'!

settingSelectors
    "answer the selectors that are used to configure a service - i.e. those which are
     saved in the webServer-settings file."

    "/ cg: changed to use a positive list, instead of an exclusion algorithm.
    "/ because it makes always trouble when an instvar is added.

    ^ super settingSelectors ,
      #(temporaryLinkExpirationTime).                       

    "
     HTTPService new settingSelectors
     self new settingSelectors
    "
! !

!HTTPTentativeObjectService methodsFor:'registration'!

shutdown
    "unregister myself from an HTTPServer (if I am registered as default)"

    httpServer unRegisterTentativeObjectService:self.
!

startup
    "when registering with aServer, and no 
     tentativeObjectService has been registered as default until now,
     register myself"
    
    httpServer registerTentativeObjectService:self.
! !

!HTTPTentativeObjectService methodsFor:'response generation'!

process:aRequest 
    "return the temporary object's value"

    |response info responseType temporaryObjectValue|

    Transcript showCR:'Tentative request: ',aRequest url printString.
    aRequest server verboseLog:'Tentative request: ',aRequest url printString.

    response := aRequest response.

    info := self temporaryLinkInfoForUrlPath:aRequest pathRelativeToService.
    info isNil ifTrue:[ 
        aRequest reportNotFound:'Tentative Object not found - possibly expired'.
    ].

    "/ response expirationTime:info browserCacheExpirationTime.
    response maxAge:(info browserCacheExpirationTime - Timestamp now).

    responseType := info type.
    temporaryObjectValue := info temporaryObjectValue.  

    ((responseType startsWith:'image') and:[temporaryObjectValue isImage]) ifTrue:[
        aRequest sendImage:temporaryObjectValue.
    ] ifFalse:[
        response contentType:responseType.
        temporaryObjectValue isFilename ifTrue:[
            response dataStream:temporaryObjectValue readStream.
        ] ifFalse:[
            temporaryObjectValue isStream ifTrue:[
                response dataStream:temporaryObjectValue.
            ] ifFalse:[
                temporaryObjectValue isBlock ifTrue:[
                    temporaryObjectValue value:aRequest.
                ] ifFalse:[
                    response data:temporaryObjectValue.
                ]
            ]
        ].
        response send.
    ].

    "Modified: / 19-03-2012 / 13:29:16 / sr"
! !

!HTTPTentativeObjectService methodsFor:'serving-helpers'!

imageInfoForFileName:aPath
    "get an image from temporary links or hard disc"

    |info|

    lock critical:[
        info := temporaryObjectNames at:aPath ifAbsent:[].
        info notNil ifTrue:[ ^ info].

        (aPath startsWith:'/') ifTrue:[
            ^ temporaryObjectNames at:(aPath copyFrom:2) ifAbsent:[].
        ]
    ].
    ^ nil.
! !

!HTTPTentativeObjectService methodsFor:'temporary link-add/remove'!

clearTemporaryLinkIds
    "Clear all temp linkIds."

    lock critical:[
        temporaryLinks removeAll.
        temporaryObjectNames removeAll.
    ].
!

expireOldTemporaryLinksFor:now
    "expire old temporary links"

    lock critical:[
        temporaryLinks notEmptyOrNil ifTrue:[
            |linksToExpire|

            linksToExpire := temporaryLinks select:[:eachInfo | 
                                                        |timeWhenLinkExpires|

                                                        timeWhenLinkExpires := eachInfo linkExpirationTime.
                                                        timeWhenLinkExpires notNil and:[now > timeWhenLinkExpires]].
            linksToExpire do:[:linkInfo |
                temporaryLinks removeKey:linkInfo temporaryObject.
                temporaryObjectNames removeKey:linkInfo linkName ifAbsent:[].
            ].
        ].
    ].
!

graphicLinkIdFor:anImage
    "Return the linkId for a temporary graphic link."

    ^ self graphicLinkIdFor:anImage expirationTimeDelta:nil.
!

graphicLinkIdFor:anImage expirationTimeDelta:anExpirationTimeDelta
    "Return the linkId for a temporary graphic link."

    anImage isImageOrForm ifFalse:[
        self error:'expected an image'.
    ].
    ^ self 
        temporaryLinkPathFor:anImage 
        type:'image/gif' 
        expirationTimeDelta:anExpirationTimeDelta.
!

nextSequenceNumber
    "generate and return a unique sequence number
     (used for temporary link-generation)"

    ^ UUID genRandomUUID printString
!

removeTemporaryLinkInfoForUrlPath:urlPath
    "remove the temporary link if it is available"
    |info|

    lock critical:[
        info := self temporaryLinkInfoForUrlPath:urlPath.
        info notNil ifTrue:[
            temporaryObjectNames removeKey:info linkName ifAbsent:nil.
            temporaryLinks removeKey:info temporaryObject ifAbsent:nil.
        ].
    ]
!

temporaryLinkPathFor:anObject type:aType
    "Return the linkId for a temporary object."

    ^ self 
        temporaryLinkPathFor:anObject 
        type:aType 
        expirationTimeDelta:nil
!

temporaryLinkPathFor:anObject type:aType expirationTimeDelta:anExpirationTimeDelta
    "Return the linkId for a temporary object with a lifeTime of anExpirationTimeDelta
     in seconds. Additional feature: if the passed anExpirationTimeDelta is the symbol #never,
     that link will be kept for ever in the service. (but with a longBrowserCacheExpirationDelta in the browser).
     aValue may be a block (or other #value-responding) thing; 
     its value is computed when a client asks for the link"

    |now existingLink relativePath linkExpirationTime browserCacheExpirationTime fileSuffix linkInfo|

    now := Timestamp now.
    anExpirationTimeDelta isNil ifTrue:[
        linkExpirationTime := browserCacheExpirationTime := now addSeconds:self temporaryLinkExpirationTime.
    ] ifFalse:[
        anExpirationTimeDelta == #never ifTrue:[
            browserCacheExpirationTime := now addSeconds:self class longTemporaryLinkBrowerCacheExpirationTime.
            linkExpirationTime := nil.
        ] ifFalse:[
            browserCacheExpirationTime := linkExpirationTime := 
                now addSeconds:((anExpirationTimeDelta isInteger) 
                                    ifTrue:[ anExpirationTimeDelta ]
                                    ifFalse:[ anExpirationTimeDelta getSeconds ]).
        ]
    ].

    lock critical:[
        existingLink := temporaryLinks at:anObject ifAbsent:[].
        existingLink notNil ifTrue:[
            existingLink linkExpirationTime:linkExpirationTime.
            existingLink browserCacheExpirationTime:browserCacheExpirationTime.
            relativePath := existingLink linkName.
        ] ifFalse:[
            "not there - add to tempLinks" 

            aType = 'processBlock' ifTrue:[
                fileSuffix := 'html'
            ] ifFalse:[
                fileSuffix := MIMETypes suffixForMimeTypeString:aType.
"/ cg: no suffix for unknown type; you MUST use 'text/html' if you WANT a html suffix.
"/ sr: i dont have a 'text/html' type, but i need the *.html suffix,
"/ if not the browsers would open an save file dialog...
"/                fileSuffix isNil ifTrue:[
"/                    fileSuffix := 'html'
"/                ].
            ].

            fileSuffix notNil ifTrue:[
                relativePath := self nextSequenceNumber printString , '.' , fileSuffix.
            ] ifFalse:[
                relativePath := self nextSequenceNumber printString.
            ].
            temporaryLinks at:anObject put:(linkInfo := TemporaryLinkInfo new
                                           linkName:relativePath;
                                           type:aType;
                                           temporaryObject:anObject;
                                           linkExpirationTime:linkExpirationTime;
                                           browserCacheExpirationTime:browserCacheExpirationTime).
            temporaryObjectNames at:relativePath put:linkInfo.
            httpServer verboseLog:'added tentative link ' , relativePath , ' for ', anObject className.
        ].
    ].
    ^ self linkName, '/', relativePath

    "Modified: / 07-05-2012 / 13:22:26 / sr"
! !

!HTTPTentativeObjectService methodsFor:'temporary link-queries'!

hasExpiredForLinkObject:aLinkObject time:aTime 
    "return true if exists and have a link for this path. Otherwise return false"
    "FIXME: cg: is this really what we want to return here - see hasExpiredForUrlPath's return value.
     also, the name of this method is completely confusing"

    |linkInfo|

    linkInfo := self temporaryLinkInfoForLinkObject:aLinkObject.
    linkInfo isNil ifTrue:[
        ^ true
    ].
    ^ linkInfo linkExpirationTime < aTime
!

hasExpiredForUrlPath:urlPath time:aTime 
    "return false if urlPath is valid (exists and has not expired). Otherwise return true"
    "FIXME: cg: is this really what we want to return here - see hasExpiredForLinkObject's return value.
     also, the name of this method is completely confusing"

    |linkInfo|

    linkInfo := self temporaryLinkInfoForUrlPath:urlPath.
    linkInfo isNil ifTrue:[
        ^ true
    ].
    ^ linkInfo linkExpirationTime > aTime
!

hasTemporaryLinkForLinkObject:anLinkObject
   ^ (self temporaryLinkInfoForLinkObject:anLinkObject) notNil
!

hasTemporaryLinkForUrlPath:aPath
   ^ (self temporaryLinkInfoForUrlPath:aPath) notNil
!

temporaryLinkInfoForLinkObject:aLinkObject
    "retrieve a temporary objects linkInfo - or nil"

    |info|

    lock critical:[
        info := temporaryLinks at:aLinkObject ifAbsent:[].
    ].
    ^ info.
!

temporaryLinkInfoForUrlPath:relativeOrAbsoluteUrlPath
    "retrieve a temporary objects linkInfo - or nil"

    |info urlPath linkNameInUrlPath relativeUrlPath|

    urlPath := relativeOrAbsoluteUrlPath.
    linkNameInUrlPath := self linkNames 
        detect:[:eachLink | urlPath startsWith:eachLink] 
        ifNone:nil.

    linkNameInUrlPath notNil ifTrue:[
        relativeUrlPath := urlPath copyFrom:linkNameInUrlPath size + 2 
    ] ifFalse:[
        relativeUrlPath := urlPath.
    ].

"/    SR: replaced by an easy to read code above...
"/    self linkNames detect:[:eachLink|
"/        (urlPath startsWith:eachLink) ifTrue:[
"/             urlPath := urlPath copyFrom:eachLink size + 2.
"/             true.
"/        ] ifFalse:[
"/             false.
"/        ].
"/    ] ifNone:[].

    lock critical:[
        info := temporaryObjectNames at:relativeUrlPath ifAbsent:[].
    ].
    ^ info.
! !

!HTTPTentativeObjectService::TemporaryLinkInfo methodsFor:'accessing'!

browserCacheExpirationTime
    ^ browserCacheExpirationTime
!

browserCacheExpirationTime:something
    browserCacheExpirationTime := something.
!

linkExpirationTime
    ^ linkExpirationTime
!

linkExpirationTime:something
    linkExpirationTime := something.
!

linkName
    ^ linkName
!

linkName:something
    "set the value of the instance variable 'linkName' (automatically generated)"

    linkName := something.
!

temporaryObject
    ^ temporaryObject 
!

temporaryObject:something
    temporaryObject := something.
!

temporaryObjectValue
    "send value to the object to deal with block values"

    ^ temporaryObject value
!

type
    ^ type
!

type:something
    type := something.
! !

!HTTPTentativeObjectService::TemporaryLinkInfo methodsFor:'printing & storing'!

printOn:aStream
    linkName printOn:aStream.
    aStream nextPutAll:' '.
    linkExpirationTime printOn:aStream.
    aStream nextPutAll:' '.
    browserCacheExpirationTime printOn:aStream.
! !

!HTTPTentativeObjectService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPTentativeObjectService.st,v 1.18 2012/05/07 11:22:51 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HTTPTentativeObjectService.st,v 1.18 2012/05/07 11:22:51 sr Exp $'
! !
