"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HtmlSanitizer
	instanceVariableNames:''
	classVariableNames:'NerfedKeywords KeywordRegex'
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!HtmlSanitizer class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    A port of http://quickwired.com/kallahar/smallprojects/php_xss_filter_function.php.

    The function should take a string and sanitize so it is safe from cross side scripting attacks (XSS).
    It adds <x> to dangerous keywords and removes non viewable characters.

    The main function of the class is #sanitizeHtml:aString.

    [author:]
        james (james@kokxnix)

    [instance variables:]

    [class variables:]

    [see also:]
"
! !

!HtmlSanitizer class methodsFor:'initialization'!

emptyOrTNRWhitespaceAttacksRegex
    "a Regex that matches an emptyString or the hexadecimal or 
     decimal encodings of cr lf and tab"

    ^ '((&#(([xX]0*[9abAB];?)|(0*(9|10|13);?)))*)'.
!

initializeSanitizedKeywordsNerfedKeywordsAndPatterns
    |spaceRegex|

    spaceRegex := self emptyOrTNRWhitespaceAttacksRegex.
    NerfedKeywords := Array new:self sanitizeKeywords size.
    KeywordRegex := Array new:self sanitizeKeywords size.

    self sanitizeKeywords keysAndValuesDo:[:idx :keyword |
        |regexWs |

        regexWs := WriteStream on:''.
        keyword do:[:eachLetter |
            regexWs nextPut:eachLetter
        ] separatedBy:[
            regexWs nextPutAll:spaceRegex.
        ].
        NerfedKeywords at:idx put:(keyword copyTo:2), '<x>', (keyword copyFrom:3).
        KeywordRegex at:idx put:regexWs contents asRegexIgnoringCase.
    ].
!

keywordRegex
    KeywordRegex isNil ifTrue:[
        self initializeSanitizedKeywordsNerfedKeywordsAndPatterns
    ].
    ^ KeywordRegex
!

nerfedKeywords
   "keywords, that are changed to avoid a possible XSS security thread."

    NerfedKeywords isNil ifTrue:[
        self initializeSanitizedKeywordsNerfedKeywordsAndPatterns
    ].
    ^ NerfedKeywords
!

sanitizeKeywords
    "keywords that are considered a possible XSS attack"

    ^ #(
        'javascript' 'vbscript' 'expression' 'applet' 'meta' 'xml' 'blink' 
        'link' 'style' 'script' 'embed' 'object' 'iframe' 'frame' 'frameset' 
        'ilayer' 'layer' 'bgsound' 'title' 'base'

        'onabort' 'onactivate' 'onafterprint' 'onafterupdate' 'onbeforeactivate' 
        'onbeforecopy' 'onbeforecut' 'onbeforedeactivate' 
        'onbeforeeditfocus' 'onbeforepaste' 'onbeforeprint' 'onbeforeunload' 
        'onbeforeupdate' 'onblur' 'onbounce' 'oncellchange' 'onchange' 'onclick' 
        'oncontextmenu' 'oncontrolselect' 'oncopy' 'oncut' 'ondataavailable' 
        'ondatasetchanged' 'ondatasetcomplete' 'ondblclick' 'ondeactivate' 'ondrag' 
        'ondragend' 'ondragenter' 'ondragleave' 'ondragover' 'ondragstart' 'ondrop' 
        'onerror' 'onerrorupdate' 'onfilterchange' 'onfinish' 'onfocus' 'onfocusin' 
        'onfocusout' 'onhelp' 'onkeydown' 'onkeypress' 'onkeyup' 'onlayoutcomplete' 
        'onload' 'onlosecapture' 'onmousedown' 'onmouseenter' 'onmouseleave' 'onmousemove' 
        'onmouseout' 'onmouseover' 'onmouseup' 'onmousewheel' 'onmove' 'onmoveend' 
        'onmovestart' 'onpaste' 'onpropertychange' 'onreadystatechange' 'onreset' 'onresize' 
        'onresizeend' 'onresizestart' 'onrowenter' 'onrowexit' 'onrowsdelete' 
        'onrowsinserted' 'onscroll' 'onselect' 'onselectionchange' 'onselectstart' 
        'onstart' 'onstop' 'onsubmit' 'onunload').
! !

!HtmlSanitizer class methodsFor:'api'!

sanitizeHtml:aString
    "return a html string that has all potential crossscripting attacks disabled. 
     This version does not take Unicode into account"

    |html|

    html := self recursivelyDecodeHtml:aString do:[:aChar :aRepresentation |     
        ('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!!@#$%^&*()~`";:?+/={}[]-_|\''\\' 
         includes:aChar) ifTrue:[
            aChar
        ] ifFalse:[
            aRepresentation
        ]. 
    ].
    html := self sanitizeKeywordsAndRemoveIllegalSpaces:html.
    ^ html

    "
      self sanitizeHtml:(self encodeToHtmlHex:'abcdef').  
      self sanitizeHtml:'<IMG SRC=jav&#x09;asc&#x09;ript:alert(''XSS'');>'.  
      self sanitizeHtml:'<IMG SRC=JAV&#x09;&#9;ASC&#x09;RIPT:alert(''XSS'');>'.  
    "
! !

!HtmlSanitizer class methodsFor:'encoding'!

encodeToHtmlDec:aStringOrStream
    "encoding aStringOrStream to HTML decimal encoding"

    |rs ws|

    rs := aStringOrStream readStream.
    ws := '' writeStream.

    [rs atEnd] whileFalse:[
        |next|
        next := rs next.
        ws nextPutAll:'&#'.
        next codePoint printOn:ws.
    ].
    ^ ws contents  

    "
      self encodeToHtmlDec:'123'  
    "
!

encodeToHtmlHex:aStringOrStream
    "encoding a aStringOrStream to HTML hex encoding"

    |rs ws|

    rs := aStringOrStream readStream.
    ws := '' writeStream.

    [rs atEnd ] whileFalse:[
        |next|
        next := rs next.
        ws nextPutAll:'&#x'.
        next codePoint printOn:ws base:16.
        ws nextPut:$;.
    ].
    ^ ws contents  

    "
      self encodeToHtmlHex:'123'
    "
!

encodeToUrlHex:aString
    "encoding a String to URL hex encoding"

    |rs ws|

    rs := aString readStream.
    ws := '' writeStream.

    [rs atEnd ] whileFalse:[
        |next|
        next := rs next.
        ws nextPut:$%.
        next codePoint printOn:ws base:16.
    ].
    ^ ws contents  

    "
      self encodeToUrlHex:'123'
    "
! !

!HtmlSanitizer class methodsFor:'helpers'!

recursivelyDecodeHtml:aStringOrStream
    "recursively decode all characters"

    ^ self recursivelyDecodeHtml:aStringOrStream do:[:eachChar :eachEncoding | eachChar].
!

recursivelyDecodeHtml:aStringOrStream do:aTwoArgCallbackBlock
    "Recursively decode aString to decode hexadecimal and decimal encodings in 
     html strings. Decide how and what to decode by returning a string from the 
     aTwoArgCallbackBlock. The returned string is then set as the replaced character.

     The aTwoArgCallbackBlock receives a Char and the encoding as arguments.

     return: <String>"

    |rs ws match char changedSomething lastChange peek currentConversion|

    rs := aStringOrStream readStream.

    [
        changedSomething := false.

        lastChange := #start.
        ws := CharacterWriteStream on:''.

        match := rs nextAvailable:2.   

        [rs atEnd ] whileFalse:[
            lastChange := #startOfIteration.

            match = '&#' ifTrue:[
                "this could be an encoded character"
                char := nil.
                peek := rs peek.    
                peek asLowercase = $x ifTrue:[        
                    "hex"     
                    |hexNumber hexString hexStringWs hexToken hexHtmlEncoding|      
                    hexToken := peek.
                    lastChange := #hex.
                    hexStringWs := WriteStream on:''.
                    peek := rs nextPeek. "x"    
                    [(peek notNil and:[peek isDigitRadix:16])] whileTrue:[
                        hexStringWs nextPut:peek.
                        peek := rs nextPeek.
                    ].
                    hexString := hexStringWs contents.   
                    (hexString size between:1 and:5) ifTrue:[          
                        char := (Integer readFrom:hexString radix:16) asCharacter.
                    ].

                    hexHtmlEncoding := '&#', hexToken, hexString.
                    "look if there is a ; as these are optional"
                    rs peek = $; ifTrue:[
                        peek := rs nextPeek. 
                        hexHtmlEncoding := hexHtmlEncoding , ';'
                    ].
                    char isNil ifTrue:[
                        char := hexHtmlEncoding.
                    ] ifFalse:[
                        "the character has been matched decide to use either the encoding or the char"
                         char := aTwoArgCallbackBlock value:char value:hexHtmlEncoding. 
                         char ~= hexHtmlEncoding ifTrue:[        
                            changedSomething := true.
                         ]
                    ].

                    ws nextPutAll:char asString.  
                    match := rs nextAvailable:2.
                ].
                (lastChange == #startOfIteration and:[peek isDigit]) ifTrue:[
                    "decimal"
                    |decimalInteger integerString integerStringWs decHtmlEncoding|   
                    lastChange := #dec.
                    "get nextInteger. nextDecimalInteger does not work for largeIntegers"
                    integerStringWs := WriteStream on:''.
                    [(peek notNil and:[peek isDigit])] whileTrue:[
                        integerStringWs nextPut:peek.
                        peek := rs nextPeek.
                    ].
                    integerString := integerStringWs contents.
                    (integerString size between:1 and:5) ifTrue:[
                        char := (Integer readFrom:integerString)asCharacter.
                    ].

                    decHtmlEncoding := '&#', integerString.
                    rs peek = $; ifTrue:[
                        peek := rs nextPeek. 
                        decHtmlEncoding := decHtmlEncoding , ';'
                    ].

                    char isNil ifTrue:[
                        char := decHtmlEncoding.
                    ] ifFalse:[
                        "the character has been matched"
                         char := (aTwoArgCallbackBlock value:char value:decHtmlEncoding).
                         char = decHtmlEncoding ifFalse:[        
                            changedSomething := true.
                         ]
                    ].

                    ws nextPutAll:char asString.  
                    match := rs nextAvailable:2.
                ].

                "if no change was made so far"
                (lastChange == #startOfIteration ) ifTrue:[
                    "no match found"                                                                          
                    ws nextPut:match first.
                    match size = 1 ifTrue:[
                        "is at end"
                        match := ''.
                    ] ifFalse:[
                        match := (match copyFrom:2), rs next.
                    ].
                ].
            ] ifFalse:[
                ws nextPut:match first.
                match size = 1 ifTrue:[
                     "is at end"
                    match := ''.
                ] ifFalse:[
                    match := (match copyFrom:2), rs next.
                ].
            ].
        ].       
        ws nextPutAll:match. "put the last to characters on the stream"
        currentConversion := ws contents.
        rs := ReadStream on:currentConversion. "ready for next iteration"
    ] doWhile:[changedSomething].

    ^ currentConversion     
!

sanitizeKeywordsAndRemoveIllegalSpaces:aString
    "add <x> to dangerous keywords and remove the characters with ascii values: 9 10 13"

    |newString nerfedKeywords keywordRegex|

    nerfedKeywords := self nerfedKeywords.
    keywordRegex := self keywordRegex.

    newString := aString.
    self sanitizeKeywords keysAndValuesDo:[:idx :keyword |
        newString := (keywordRegex at:idx) 
                        copy:newString replacingMatchesWith:(nerfedKeywords at:idx).
    ].

    ^ newString      

    "
     self sanitizeKeywordsAndRemoveIllegalSpaces:'<IMG SRC=jav&#x09;asc&#x09;ript:alert(''XSS'');>'.     
    "
! !

!HtmlSanitizer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HtmlSanitizer.st,v 1.7 2011/02/09 14:15:37 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HtmlSanitizer.st,v 1.7 2011/02/09 14:15:37 stefan Exp $'
! !
