"{ Package: 'stx:goodies/webServer' }"

HTTPService subclass:#SeasideService
	instanceVariableNames:'entryPoint'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Platform'
!

!SeasideService class methodsFor:'documentation'!

documentation
"
    This is for ST/X's webServer, what WACom is for the Squeak server:
    an adaptor to plug in the seaside framework into the webServer framework.

    Its main purpose is to translate webServer requests into seasides WARequests
    and seasides WAResponses back to webServer responses.

    [start with:]
         SeasideService startOn:9090

    [see also:]
        Launchers settings application
"
! !

!SeasideService class methodsFor:'defaults'!

defaultEntryPoint
    ^ WADispatcher default
!

linkName
    "return the default path (with slash)"

    ^ '/seaside'
! !

!SeasideService class methodsFor:'startup'!

startOn:aPortNr
    "start a new seaside service on the given port"

    |server service|

    server := HTTPServer serverOnPort:aPortNr.
    service := self new entryPoint:(self defaultEntryPoint).
    service registerServiceOn:server.

    "
     self startOn:9090.
    "
! !

!SeasideService methodsFor:'accessing'!

entryPoint
    ^ entryPoint ? self class defaultEntryPoint
!

entryPoint:something
    entryPoint := something.
! !

!SeasideService methodsFor:'processing'!

convertRequest:anHttpRequest 
    |request url rawFields fields cookies|

    self processMultipartFields:anHttpRequest.

    Smalltalk isSmalltalkX ifTrue:[
        url := HTMLUtilities unEscape:anHttpRequest url.
        rawFields := (anHttpRequest method = 'POST' 
                        ifTrue:[ anHttpRequest postDataFields ]
                        ifFalse:[ anHttpRequest urlParameters ]).

        fields := Dictionary new.
        rawFields notNil ifTrue:[
            rawFields keysAndValuesDo:[:k :v |
                k size == 0 ifFalse:[
                    fields at:k put:(v first)
                ].
            ].
        ].
        cookies := anHttpRequest cookies.
        cookies isNil ifTrue:[
            cookies := Dictionary new
        ].
    ] ifFalse:[
        url := anHttpRequest url unescapePercents.
        fields := (anHttpRequest method = 'POST' 
                        ifTrue:[ anHttpRequest postFields ]
                        ifFalse:[ anHttpRequest getFields ]).
        cookies := anHttpRequest cookies.
    ].

    request := WARequest 
                method:anHttpRequest method
                url:url
                headers:anHttpRequest header
                fields:fields
                cookies:cookies
                nativeRequest:anHttpRequest.

    anHttpRequest method = 'PUT' ifTrue:[
        request fields at:'PUTData'
            put:(anHttpRequest stream next:anHttpRequest contentLength)
    ].
    ^ request
!

convertResponse:aResponse for:aRequest
    |httpResponse dataOrStream|

    httpResponse := aRequest response.

    aResponse ifNil:[ 
        httpResponse nextPutAll:'Request handling aborted; reload to retry'.
        ^ self
    ].

    aResponse cookies do:[:assoc | 
        httpResponse
            setCookieName:assoc key
            value:assoc valueWithExpiry
            path:assoc path
    ].
"/    aResponse headers 
"/        associationsDo:[:assoc | httpResponse fieldAt:assoc key put:assoc value ].

"/    (HttpResponse classPool at:#StatusCodes) 
"/        associationsDo:[:assoc | 
"/            assoc value key = aResponse status ifTrue:[
"/                komResponse status:assoc key
"/            ]
"/        ].
    dataOrStream := aResponse contents.
    httpResponse data:dataOrStream.
    httpResponse contentType:aResponse contentType.
!

handleRequest:aRequest 
    ^ (self entryPoint handlerForRequest:aRequest) handleRequest:aRequest
!

log:aRequest 
    Transcript showCR:aRequest url
!

process:httpRequest 
    |request response|

    self log:httpRequest.
    request := self convertRequest:httpRequest.
    response := self handleRequest:request.
    self convertResponse:response for:httpRequest.
    response release.
!

processMultipartFields:aRequest 
"/    aRequest 
"/        multipartFormFieldsDo:[:chunk | 
"/            chunk fileName isEmptyOrNil ifFalse:[
"/                |stream file|
"/
"/                stream := WriteStream on:String new.
"/                chunk saveToStream:stream.
"/                file := (WAFile new)
"/                            fileName:chunk fileName;
"/                            contents:stream contents;
"/                            contentType:chunk contentType.
"/                aRequest postFields at:chunk fieldName put:file
"/            ]
"/        ].
! !

!SeasideService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/SeasideService.st,v 1.9 2009/08/14 22:07:12 cg Exp $'
! !
