"{ Package: 'stx:goodies/webServer' }"

TestCase subclass:#TestHtmlSanitizer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Server'
!

!TestHtmlSanitizer class methodsFor:'documentation'!

documentation
"
    documentation to be added.

    [author:]
        james (james@kokxnix)

    [instance variables:]

    [class variables:]

    [see also:]

"
!

history
    "Created: / 20-12-2006 / 12:39:53 / james"
! !

!TestHtmlSanitizer methodsFor:'initialize / release'!

setUp
    "common setup - invoked before testing."

    super setUp
!

tearDown
    "common cleanup - invoked after testing."

    super tearDown
! !

!TestHtmlSanitizer methodsFor:'testRegex'!

test_emptyOrTNRWhitespaceAttacksRegex
    |regex|
    "/regex := HtmlSanitizer emptyOrTNRWhitespaceAttacksRegex.
    regex := '(&#[xX]0*[9abAB]?;?)?(&#0*(9|10|13)?;?)?'.

    self assert:( '' matchesRegex:regex).

    "test decimal"

    self assert:( '&#000009;' matchesRegex:regex).
    self assert:( '&#0000010;' matchesRegex:regex).
    self assert:( '&#0000013;' matchesRegex:regex).

    self assert:( '&#000009' matchesRegex:regex).
    self assert:( '&#0000010' matchesRegex:regex).
    self assert:( '&#0000013' matchesRegex:regex).

    "test hexidecimal"

    self assert:( '&#x000009;' matchesRegex:regex).
    self assert:( '&#x00000a;' matchesRegex:regex).
    self assert:( '&#x00000b;' matchesRegex:regex).

    self assert:( '&#x000009' matchesRegex:regex).
    self assert:( '&#x00000a' matchesRegex:regex).
    self assert:( '&#x00000b' matchesRegex:regex).

    self assert:( '&#X000009;' matchesRegex:regex).
    self assert:( '&#X00000a;' matchesRegex:regex).
    self assert:( '&#X00000b;' matchesRegex:regex).

    self assert:( '&#X000009' matchesRegex:regex).
    self assert:( '&#X00000a' matchesRegex:regex).
    self assert:( '&#X00000b' matchesRegex:regex).

    "with capitalized hex chars"

    self assert:( '&#x00000A;' matchesRegex:regex).
    self assert:( '&#x00000B;' matchesRegex:regex).

    self assert:( '&#x00000A' matchesRegex:regex).
    self assert:( '&#x00000B' matchesRegex:regex).

    self assert:( '&#X00000A;' matchesRegex:regex).
    self assert:( '&#X00000B;' matchesRegex:regex).

    self assert:( '&#X00000A' matchesRegex:regex).
    self assert:( '&#X00000B' matchesRegex:regex).

    "
     self run:#test_emptyOrTNRWhitespaceAttacksRegex 
    "
!

test_emptyOrTNRWhitespaceAttacksRegex_correct
    |regex|
    regex := HtmlSanitizer emptyOrTNRWhitespaceAttacksRegex.

    "empty"
    self assert:( '' matchesRegex:regex).

    "no zeros"
    self assert:( '&#9;' matchesRegex:regex).
    self assert:( '&#x9;' matchesRegex:regex).

    self assert:( '&#10;' matchesRegex:regex).
    self assert:( '&#xa;' matchesRegex:regex).

    self assert:( '&#13' matchesRegex:regex).
    self assert:( '&#xb;' matchesRegex:regex).

    "test decimal"

    self assert:( '&#000009;' matchesRegex:regex).
    self assert:( '&#0000010;' matchesRegex:regex).
    self assert:( '&#0000013;' matchesRegex:regex).

    self assert:( '&#000009' matchesRegex:regex).
    self assert:( '&#0000010' matchesRegex:regex).
    self assert:( '&#0000013' matchesRegex:regex).

    "test hexidecimal"

    self assert:( '&#x000009;' matchesRegex:regex).
    self assert:( '&#x00000a;' matchesRegex:regex).
    self assert:( '&#x00000b;' matchesRegex:regex).

    self assert:( '&#x000009' matchesRegex:regex).
    self assert:( '&#x00000a' matchesRegex:regex).
    self assert:( '&#x00000b' matchesRegex:regex).

    self assert:( '&#X000009;' matchesRegex:regex).
    self assert:( '&#X00000a;' matchesRegex:regex).
    self assert:( '&#X00000b;' matchesRegex:regex).

    self assert:( '&#X000009' matchesRegex:regex).
    self assert:( '&#X00000a' matchesRegex:regex).
    self assert:( '&#X00000b' matchesRegex:regex).

    "with capitalized hex chars"

    self assert:( '&#x00000A;' matchesRegex:regex).
    self assert:( '&#x00000B;' matchesRegex:regex).

    self assert:( '&#x00000A' matchesRegex:regex).
    self assert:( '&#x00000B' matchesRegex:regex).

    self assert:( '&#X00000A;' matchesRegex:regex).
    self assert:( '&#X00000B;' matchesRegex:regex).

    self assert:( '&#X00000A' matchesRegex:regex).
    self assert:( '&#X00000B' matchesRegex:regex).

    "
     self run:#test_emptyOrTNRWhitespaceAttacksRegex 
    "
!

test_emptyOrTNRWhitespaceAttacksRegex_errors
    |regex|
    regex := HtmlSanitizer emptyOrTNRWhitespaceAttacksRegex.

    "no numbers"
    self shouldnt:( '&#x;' matchesRegex:regex).
    self shouldnt:( '&#;' matchesRegex:regex).

    "test double"

    self shouldnt:( '&#x000009;&#000009;' matchesRegex:regex).
    self shouldnt:( '&#x000009;&#x000009;' matchesRegex:regex).

    "test wrong number"

    self shouldnt:( '&#x000009a;' matchesRegex:regex).
    self shouldnt:( '&#x000009b;' matchesRegex:regex).
    "
     self run:#test_emptyOrTNRWhitespaceAttacksRegex_errors  
    "
! !

!TestHtmlSanitizer methodsFor:'tests'!

test_encodeToHtmlDec
    | helloWorld encodedVersion|


    helloWorld := 'hello world'.
    encodedVersion := HtmlSanitizer encodeToHtmlDec:helloWorld.     
    self assert: (encodedVersion = '&#104&#101&#108&#108&#111&#32&#119&#111&#114&#108&#100').


    "
     self run:#test_removeAllNonPrintableCharacters 
    "
!

test_encodeToHtmlHex
    | helloWorld encodedVersion|


    helloWorld := 'hello world'.
    encodedVersion := HtmlSanitizer encodeToHtmlHex:helloWorld.     
    self assert: (encodedVersion = '&#x68;&#x65;&#x6C;&#x6C;&#x6F;&#x20;&#x77;&#x6F;&#x72;&#x6C;&#x64;').


    "
     self run:#test_removeAllNonPrintableCharacters 
    "
!

test_encodeToUrlHex
    | helloWorld encodedVersion|


    helloWorld := 'hello world'.
    encodedVersion := HtmlSanitizer encodeToUrlHex:helloWorld.     
    self assert: (encodedVersion = '%68%65%6C%6C%6F%20%77%6F%72%6C%64').


    "
     self run:#test_removeAllNonPrintableCharacters 
    "
!

test_recursivelyDecodeHtml_do

    | testText encodedVersion|

    "Ensure that the encoded version remains consistant"
    testText := HtmlSanitizer encodeToHtmlHex:''.    
    encodedVersion := HtmlSanitizer recursivelyDecodeHtml:testText do:[:aChar :anEncoding | self assert:(aChar = $). anEncoding ].
    self assert:encodedVersion = '&#xE4;'.

    "                         
     self run:#test_recursivelyDecodeHtml_do    
    "
!

test_recusivelyDecodeHtmlWhere_StraightHexEncoding
"
    Test that simple conversion from hex works
"
    | helloWorld decodedVersion|
    "with ;"
    helloWorld := '&#x68;&#x65;&#x6C;&#x6C;&#x6F;&#x20;&#x77;&#x6F;&#x72;&#x6C;&#x64;'.

    self assert:[
        decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld do:[:char : encoding | (#($h) includes:char) ifTrue:[char] ifFalse:[encoding]].
    ] completesInSeconds:5.

    self assert: (decodedVersion = 'h&#x65;&#x6C;&#x6C;&#x6F;&#x20;&#x77;&#x6F;&#x72;&#x6C;&#x64;').


    "
     self run:#test_recusivelyDecodeHtmlWhere_StraightHexEncoding    
    "
!

test_recusivelyDecodeHtml_DecEncodingRubbish
"
    Assessing what happens when encoding is wrong.
"
    | helloWorld decodedVersion|
    "with ;"
    helloWorld := '&#1041231231231231213'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = '&#1041231231231231213').

    helloWorld := '&#!!10'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = '&#!!10').

    "
     self run:#test_recusivelyDecodeHtml_DecEncodingRubbish
    "
!

test_recusivelyDecodeHtml_HexEncodingRubbish
"
    Assessing what happens when encoding is wrong.
"
    | helloWorld decodedVersion|
    "test that an invalid hex coding stays the same"
    helloWorld := '&#xJ'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = '&#xJ').


    " keep the case of the X"
    helloWorld := '&#XJ'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = '&#XJ').

    "test that a longer hex value does change when text comes in at the end"
    helloWorld := '&#x10JamesWazEre'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = (16r10 asCharacter asString, 'JamesWazEre' )).   

    "test that a longer hex value does not change"
    helloWorld := '&#x1bcad'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 16r1bcad asCharacter asString).


    "
     self run:#test_recusivelyDecodeHtml_HexEncodingRubbish       
    "
!

test_recusivelyDecodeHtml_RecursiveDecEncoding
"
    Test that simple conversion from dec works
"
    | helloWorld decodedVersion|


    "without ; - one recursion"
    helloWorld := '&#38&#35&#49&#48&#52&#38&#35&#49&#48&#49&#38&#35&#49&#48&#56&#38&#35&#49&#48&#56&#38&#35&#49&#49&#49&#38&#35&#51&#50&#38&#35&#49&#49&#57&#38&#35&#49&#49&#49&#38&#35&#49&#49&#52&#38&#35&#49&#48&#56&#38&#35&#49&#48&#48'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "without ; - two recursions"
    helloWorld := '&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#53&#50&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#52&#57&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#53&#54&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#53&#54&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#53&#49&#38&#35&#53&#48&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#53&#55&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#57&#38&#35&#53&#50&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#53&#54&#38&#35&#51&#56&#38&#35&#51&#53&#38&#35&#52&#57&#38&#35&#52&#56&#38&#35&#52&#56'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').

    "without ; - 5 (n) recursions"
    helloWorld := 'hello world'.
    5 timesRepeat:[helloWorld := HtmlSanitizer encodeToHtmlDec:helloWorld].
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "
     self run:#test_recusivelyDecodeHtml_RecursiveDecEncoding     
    "
!

test_recusivelyDecodeHtml_RecursiveHexEncoding
"
    Test that simple conversion from hex works
"
    | helloWorld decodedVersion|
    "with ; - recusive once"
    helloWorld := '&#x26;&#x23;&#x78;&#x36;&#x38;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x35;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x43;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x43;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x46;&#x3B;&#x26;&#x23;&#x78;&#x32;&#x30;&#x3B;&#x26;&#x23;&#x78;&#x37;&#x37;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x46;&#x3B;&#x26;&#x23;&#x78;&#x37;&#x32;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x43;&#x3B;&#x26;&#x23;&#x78;&#x36;&#x34;&#x3B;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "without ; - recusive once"
    helloWorld := '&#x26&#x23&#x78&#x36&#x38&#x3B&#x26&#x23&#x78&#x36&#x35&#x3B&#x26&#x23&#x78&#x36&#x43&#x3B&#x26&#x23&#x78&#x36&#x43&#x3B&#x26&#x23&#x78&#x36&#x46&#x3B&#x26&#x23&#x78&#x32&#x30&#x3B&#x26&#x23&#x78&#x37&#x37&#x3B&#x26&#x23&#x78&#x36&#x46&#x3B&#x26&#x23&#x78&#x37&#x32&#x3B&#x26&#x23&#x78&#x36&#x43&#x3B&#x26&#x23&#x78&#x36&#x34&#x3B'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').



    "
     self run:#test_recusivelyDecodeHtml_StraightHexEncoding
    "
!

test_recusivelyDecodeHtml_RecursiveMixedEncoding
"
    Test that simple conversion from dec works
"
    | helloWorld decodedVersion|


    "first Dec the Hex"
    helloWorld := '&#x26;&#x23;&#x31;&#x30;&#x34;&#x26;&#x23;&#x31;&#x30;&#x31;&#x26;&#x23;&#x31;&#x30;&#x38;&#x26;&#x23;&#x31;&#x30;&#x38;&#x26;&#x23;&#x31;&#x31;&#x31;&#x26;&#x23;&#x33;&#x32;&#x26;&#x23;&#x31;&#x31;&#x39;&#x26;&#x23;&#x31;&#x31;&#x31;&#x26;&#x23;&#x31;&#x31;&#x34;&#x26;&#x23;&#x31;&#x30;&#x38;&#x26;&#x23;&#x31;&#x30;&#x30;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "first Hex then Dec"
    helloWorld := '&#38&#35&#120&#54&#56&#59&#38&#35&#120&#54&#53&#59&#38&#35&#120&#54&#67&#59&#38&#35&#120&#54&#67&#59&#38&#35&#120&#54&#70&#59&#38&#35&#120&#50&#48&#59&#38&#35&#120&#55&#55&#59&#38&#35&#120&#54&#70&#59&#38&#35&#120&#55&#50&#59&#38&#35&#120&#54&#67&#59&#38&#35&#120&#54&#52&#59'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "
     self run:#test_recusivelyDecodeHtml_StraightDecEncoding
    "
!

test_recusivelyDecodeHtml_StraightDecEncoding
"
    Test that simple conversion from dec works
"
    | helloWorld decodedVersion|
    "with ;"
    helloWorld := '&#104;&#101;&#108;&#108;&#111;&#32;&#119;&#111;&#114;&#108;&#100;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "without ;"
    helloWorld := '&#104&#101&#108&#108&#111&#32&#119&#111&#114&#108&#100'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').
    "
     self run:#test_recusivelyDecodeHtml_StraightDecEncoding
    "
!

test_recusivelyDecodeHtml_StraightHexEncoding
"
    Test that simple conversion from hex works
"
    | helloWorld decodedVersion|
    "with ;"
    helloWorld := '&#x68;&#x65;&#x6C;&#x6C;&#x6F;&#x20;&#x77;&#x6F;&#x72;&#x6C;&#x64;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').

"
     I dont think that you can have hexidecimals without ';' in html but here I test it anyway
"

    "without ;"
    helloWorld := '&#x68&#x65&#x6C&#x6C&#x6F&#x20&#x77&#x6F&#x72&#x6C&#x64;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "without ; + capital X"
    helloWorld := '&#X68;&#X65;&#X6C;&#X6C;&#X6F;&#X20;&#X77;&#X6F;&#X72;&#X6C;&#X64;'.
    decodedVersion := HtmlSanitizer recursivelyDecodeHtml:helloWorld.     
    self assert: (decodedVersion = 'hello world').

    "
     self run:#test_recusivelyDecodeHtml_StraightHexEncoding
    "
!

test_recusivelyDecodeHtml_Timing
"
    Ensure that the speed does not decrease
"
    | helloWorld|
    helloWorld := 'hello world'.
    2 timesRepeat:[helloWorld := HtmlSanitizer encodeToHtmlHex:helloWorld].

    self assert:[10000 timesRepeat:[ HtmlSanitizer recursivelyDecodeHtml:helloWorld]] completesInSeconds:15
    "
     self run:#test_recusivelyDecodeHtml_Timing      
    "
!

test_removeAllNonPrintableCharacters
    | helloWorld decodedVersion|


    helloWorld := '&#00hello&#00 &#00w&#00o&#00rld'.
    decodedVersion := HtmlSanitizer removeAllNonPrintableCharacters:helloWorld.     
    self assert: (decodedVersion = 'hello world').


    "
     self run:#test_removeAllNonPrintableCharacters 
    "
!

test_sanitizeHtml

    | testText encodedVersion|

    "exchange all the non-special characters"
    testText := HtmlSanitizer encodeToHtmlHex:'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!!@#$%^&*()~`";:?+/={}[]-_|\''\\' .    
    encodedVersion := HtmlSanitizer sanitizeHtml:testText. 
    self assert:encodedVersion = '&#xE4;abcdefghijklmnopqrstuvwxyzA&#xC4;BCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!!@#$%^&*()~`";:?+/={}[]-_|\''\\&#xE4;'.    

    "                         
     self run:#test_sanitizeHtml 

    "
!

test_sanitizeKeywordsAndRemoveIllegalSpaces
    |sanitized|
    sanitized := HtmlSanitizer sanitizeKeywordsAndRemoveIllegalSpaces:'<IMG SRC=jav&#x09;asc&#x09;ript:alert(''XSS'');>'.
    self assert:sanitized = '<IMG SRC=ja<x>vasc<x>ript:alert(''XSS'');>'.


    "                         
    50 timesRepeat:[ HtmlSanitizer sanitizeKeywordsAndRemoveIllegalSpaces:'<IMG SRC=jav&#x09;asc&#x09;ript:alert(''XSS'');>']   

    "
! !

!TestHtmlSanitizer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/TestHtmlSanitizer.st,v 1.2 2007/06/01 10:02:36 stefan Exp $'
! !
