"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

"{ NameSpace: Demos }"

HTTPSelectorService subclass:#WebDemoApp1
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'demos-WebApplications'
!

!WebDemoApp1 class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Demonstrates some webApplication features.
    To start:

      start the HTTPServer at some port
      aServer := HTTPServer startServerOnPort:8080

      register the class (towards the HTTPServer) with:
        Demos::WebDemoApp1 registerServiceIn:(HTTPServer runningServers first)

    Open a netscape or internet explorer (somewhere);
    enter the URL:
        httP://host:port/WebDemoApp1/test1
    or (test2, test3, ...)

    For each request, a corresponding method in this class is
    invoked, which returns an html document.


    [author:]
        Claus Gittinger (cg@exept)

    [see also:]
        HTTPServer
"
!

examples

"
  start the HTTPServer at some port and register 
  the class (towards the HTTPServer)

  Open a netscape or internet explorer (somewhere);
  enter the URL:
        httP://host:port/WebDemoApp1:test1
  or (test2, test3, ...)
                                                                [exBegin]
    |aServer|

    aServer := HTTPServer serverOnPort:8081.
    self registerServiceIn:aServer.
                                                                [exEnd]
"
! !

!WebDemoApp1 class methodsFor:'class access'!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
! !

!WebDemoApp1 class methodsFor:'defaults'!

linkName
    "return the default linkName path (with slash)"

    ^ '/webDemoApp'
! !

!WebDemoApp1 class methodsFor:'queries-service'!

isThreadSafe
    ^ true
!

settingsApplicationClass
    "a SettingsApplication class - or nil (used in the settings dialog if non-nil)"

    ^ nil
! !

!WebDemoApp1 methodsFor:'queries-service'!

addRequiredForeignServicesTo:aServer 
    "I need a file service for the bitmaps and icons"

    self addStandardFileServicesTo:aServer.
!

allowedMessages
    ^ #( 
         openAsWebPage:

         showSource:
         editSource:
         acceptSource:

         test1: 
         test2: 
         test3:
         test3Action:

         test4: 
         test4Action:
         test5: 
         test6:
         test7:
         test8:

         testStauSuche:
         testGoogle:
         timeNow:
        ).
!

defaultMessage
    ^ #openAsWebPage
!

offeredItems
    ^ (self allowedMessages asSet 
        removeAllFoundIn:
            #( 
             openAsWebPage:

             showSource:
             editSource:
             acceptSource:

             test3Action:
             test4Action:
            );
        asOrderedCollection) sort
! !

!WebDemoApp1 methodsFor:'serving'!

acceptSource:aRequest
    |selector oldSource newSource result replyPresenter|

    aRequest authenticateForRealm:'SysAdmin'.

    selector := aRequest arguments first.
    oldSource := self sourceForSelector:selector.

    newSource := aRequest postDataFieldAt:'sourceCode'.

    replyPresenter := aRequest replyPresenter.
    oldSource = newSource ifTrue:[
        replyPresenter title:('Unchanged: ' , self class nameWithoutPrefix , '-' , selector).
        replyPresenter parametersAt:#secondHeadLine put:('Unchanged: ' , self class nameWithoutPrefix , '-' , selector).
        aRequest
            nextPutLine:'No change'.
    ] ifFalse:[
        result := self class compile:newSource classified:'serving'.

        result == #Error ifTrue:[
            replyPresenter title:('Unchanged: ' , self class nameWithoutPrefix , '-' , selector).
            replyPresenter parametersAt:#secondHeadLine put:('Unchanged: ' , self class nameWithoutPrefix , '-' , selector).
            aRequest
                nextPutLine:'There was a compilation error'.
        ] ifFalse:[
            replyPresenter title:('Accepted: ' , self class nameWithoutPrefix , '-' , selector).
            replyPresenter parametersAt:#secondHeadLine put:('Accepted: ' , self class nameWithoutPrefix , '-' , selector).
            aRequest
                nextPutLine:'Your new code has been installed'.
        ]
    ].
    
    aRequest
        nextPutLine:'<P>';
        nextPutLine: (aRequest webLinkForText:('Reload ' , selector) service:self selector:selector);
        nextPutLine:'<BR>';
        nextPutLine:(aRequest webLinkForText:'Index' service:self selector:'#openAsWebPage:').
!

editSource:aRequest
    |selector source replyPresenter|

    selector := aRequest arguments first.
    source := self sourceForSelector:selector.
    replyPresenter := aRequest replyPresenter.
    replyPresenter title:('Editing ' , self class nameWithoutPrefix , '-' , selector).
    replyPresenter parametersAt:#secondHeadLine put:('Editing ' , self class nameWithoutPrefix , '-' , selector).
    aRequest
        nextPutLine:'<CODE><PRE>';
        nextPutLine:'<FORM action="', (aRequest linkIdFor:self selector:#'acceptSource:' argument:selector), '" method=POST>';
        nextPutLine:'<TEXTAREA name="sourceCode" COLS=80 ROWS=30>';
        replyEscaped:source;
        nextPutLine:'</TEXTAREA>';
        nextPutLine:'<INPUT type=submit value="Accept"></INPUT>';
        nextPutLine:'</FORM>';
        nextPutLine:'</PRE></CODE>';
        nextPutLine:'<P>';
        nextPutLine:(aRequest webLinkForText:'Index' service:self selector:#'openAsWebPage:').
!

openAsWebPage:aRequest
    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:('Smalltalk Dynamic Web Page Examples - ' , self class nameWithoutPrefix ).
    replyPresenter parametersAt:#secondHeadLine put:('Smalltalk Dynamic Web Page Examples - ' , self class nameWithoutPrefix ).
    aRequest
        nextPutLine:'<br>These pages are generated dynamically by methods of the <code>' ,self class name,'</code> class.<BR>';
        nextPutLine:'You can easily create your own programed webServices, by defining it as a subclass of
<code>HTTPService</code> (find the required protocol by looking at <code>subclassResponsible</code>-methods there).
<BR>Click on "source" to see the corresponding source code.
<UL>
'.

    self offeredItems do:[:aMessageSelector|
        |s|

        s := '<LI>' ,  (aRequest webLinkForText:(aMessageSelector allButLast) service:self selector:aMessageSelector).
        s := s , ' (' , (aRequest webLinkForText:'source' service:self selector:#'showSource:' argument:aMessageSelector) , ')'.
        s := s , String cr.
        aRequest nextPutLine:s.
    ].
    aRequest nextPutLine:'</UL>'.
!

showSource:aRequest
    |selector source replyPresenter|

    selector := aRequest arguments first.
    source := self sourceForSelector:selector.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:('Smalltalk Dynamic Web Pages - Source of ' , self class nameWithoutPrefix , '-' , selector).
    replyPresenter parametersAt:#secondHeadLine put:('Smalltalk Dynamic Web Pages - Source of ' , self class nameWithoutPrefix , '-' , selector).
    aRequest
        nextPutLine:'<PRE>';
        replyEscaped:source;
        nextPutLine:'</PRE>';
        nextPutLine:'<P>';
        nextPutLine:(aRequest webLinkForText:'Edit' service:self selector:#'editSource:' argument:selector);
        nextPutLine:'<BR>';
        nextPutLine:(aRequest webLinkForText:'Index' service:self selector:#'openAsWebPage:').
!

sourceForSelector:anArgument
    |argument|

    argument := anArgument.
    (argument endsWith:':') ifFalse:[
        argument := argument, ':'.
    ].
    ^ (self class compiledMethodAt:(argument asSymbol)) source.
!

test1:aRequest
    "a simple reply, with an URL refering to another method inside myself.

     Invoke via the browser as URL:
        host:port/<N>,test1:
     where <N> is the registered name of this service.
    "

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Hello world Example'.
    replyPresenter parametersAt:#secondHeadLine put:'Test1: Hello world Example'.
    aRequest
        nextPutLine:'<h1>Hello World</h1>'.
!

test2:aRequest
    "another simple reply, with an URL refering to another method inside myself

     Invoke via the browser as URL:
        host:port/<N>,test2:
     where <N> is the registered name of this service.
    "

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Hello world Example with References'.
    replyPresenter parametersAt:#secondHeadLine put:'Test2: Hello world Example with References'.
    aRequest
        nextPutLine:'<h1>Hello World</h1>

This is test2.

'.
    self addLinksForSource:#'test2:' previous:#'test1:' next:#'test3:' to:aRequest.
!

test3:aRequest
    "a dialogForm; when submitted, test3Action is invoked.

     Invoke via the browser as URL:
        host:port/<N>,test3:
     where <N> is the registered name of this service.
    "
    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Simple Dialog Example'.
    replyPresenter parametersAt:#secondHeadLine put:'Test3: Simple Dialog Example'.
    aRequest
        nextPutLine:'<h1>Form example</h1>

This is test3 showing a dialog.

<FORM action="',(aRequest linkIdFor:self selector:#'test3Action:'), '" method=POST>
  Name:     <INPUT               name="username" size=15 value="defaultName"></INPUT>
  Password: <INPUT type=password name="password" size=15>                    </INPUT>
<BR>
            <INPUT type=submit value="Submit"></INPUT>


</form>
'.
    self addLinksForSource:#'test3:' previous:#'test2:' next:#'test4:' to:aRequest.
!

test3Action:aRequest
    "a dialogs action; invoked when the dialog from test3 is submitted.
     This one simply shows the entered values"

    |userNameValue passwordValue replyPresenter|

    aRequest postDataFields isNil ifTrue:[
        aRequest server reportBadRequest:nil.
        "/ not reached
    ].

    userNameValue := (aRequest postDataFieldAt:'username' ifAbsent:'(empty)').
    passwordValue := (aRequest postDataFieldAt:'password' ifAbsent:'(empty)').

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Test3''s form action'.
    replyPresenter parametersAt:#secondHeadLine put:'Test3''s form action'.
    aRequest
        nextPutLine:'<h1>Dialog3''s action</h1>';
        nextPutLine:'<p>';
        nextPutLine:'You entered:';
        nextPutLine:'<p>';
        nextPutLine:'Username = ' , (userNameValue?'') , '<br>';
        nextPutLine:'Password = ' , (passwordValue?'') , '<br>';
        nextPutLine:'<p>';
        nextPutLine:'Thank you';
        nextPutLine:'<p>'.

    self addLinksForSource:#'test3Action:' previous:#'test3:' next:#'test4:' to:aRequest.
!

test4:aRequest
    "another dialog form

     Invoke via the browser as URL:
        host:port/<N>,test4:
     where <N> is the registered name of this service.
    "
    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Another Dialog form'.
    replyPresenter parametersAt:#secondHeadLine put:'Test4: Another Dialog form'.
    aRequest
        nextPutLine:'<h1>Enter values</h1>';
        nextPutLine:'This is a dialog test';
        nextPutAll:'The follwoing forms contents is executed as unix command,';
        nextPutLine:'and the result presented in another page.<p>';
        nextPutLine:'For security reasons, only a very limited set of commands is allowed (such as "ls", "pwd" or "date").<p>';
        nextPutLine:'<FORM action="', (aRequest linkIdFor:self selector:#'test4Action:'), '" method=POST>';
        nextPutLine:'Script: <input name="scriptName" size=15 value="ls"></input><p>';
        nextPutLine:'Arg1: <input name="arg1" size=15 value="-l"></input><br>';
        nextPutLine:'Arg2: <input name="arg2" size=15 value=""></input><br>';
        nextPutLine:'Arg3: <input name="arg3" size=15 value=""></input><p>';
        nextPutLine:'Press <input type=submit value="Execute"></input>  to execute the above as unix command ...<br><br>';
        nextPutLine:'</form>'.

    self addLinksForSource:#'test4:' previous:#'test3:' next:#'test5:' to:aRequest.
!

test4Action:aRequest
    "a dialog4's action; invoked when the dialog from test4 is submitted.
     This one executes a unix command based on the input, and
     shows the output in a textArea"

    |geheim commandString scriptName argName argI
     output p replyPresenter|

    aRequest postDataFields isNil ifTrue:[
        aRequest reportBadRequest:nil
        "/ not reached
    ].
    geheim := aRequest postDataFieldAt:'woher'.

    scriptName := aRequest postDataFieldAt:'scriptName'.
    scriptName isNil ifTrue:[
        aRequest reportBadRequest:'No command was given.'.
        "/ not reached
    ].
    ( #('ls' 'pwd' 'who' 'date') includes:scriptName ) ifFalse:[
        aRequest reportBadRequest:'Unauthorized command was given (trying to trick me ?).'.
        "/ not reached
    ].

    commandString := scriptName.
    1 to:20 do:[:idx |
        argName := 'arg' , idx printString.
        argI := aRequest postDataFieldAt:argName.
        argI notNil ifTrue:[
            commandString := commandString , ' ' , argI
        ].
    ].
    commandString := commandString upTo:$;.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Test4''s form action output'.
    replyPresenter parametersAt:#secondHeadLine put:'Test4''s form action output'.
    aRequest
        nextPutLine:'The output of:"<b>' , commandString , '</b>" is:<br>';
        nextPutLine:'<code><pre>';
        nextPutLine:'<form>';
        nextPutLine:'<textarea COLS="80" ROWS="30">'.

    p := PipeStream readingFrom:commandString.
    p isNil ifTrue:[
        output := 'No such command'
    ] ifFalse:[
        output := p contents asString.
    ].
    aRequest nextPutLine:output.

    aRequest nextPutLine:'</textarea></form></pre></code>'.
    self addLinksForSource:#'test4Action:' previous:#'test3:' next:#'test5:' to:aRequest.
!

test5:aRequest
    "send a file 
     - requires a test4.html file 
       to be found under the htmldialogs directory."

    |message fileName replyPresenter|

    fileName := '/files/test4.html' asFilename. 
    fileName exists ifTrue:[
        message := fileName contents asString
    ] ifFalse:[
        message := 'file not available'
    ].

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Embedding a File'.
    replyPresenter parametersAt:#secondHeadLine put:'Test5: Embedding a File'.
    aRequest
        nextPutLine:message.

    self addLinksForSource:#'test5:' previous:#'test4:' next:#'test6:' to:aRequest.
!

test6:aRequest
    "send a file
     - requires a test6.html file 
       to be found under the htmldialogs directory."

    "/aRequest nextPutLine:('htmldialogs/test6.html' asFilename contents asString)

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Test6'.
    replyPresenter parametersAt:#secondHeadLine put:'Test6'.
    aRequest
        nextPutLine:'<h1>This test is not available</h1>'.

    self addLinksForSource:#'test6:' previous:#'test5:' next:#'test7:' to:aRequest.
!

test7:aRequest
    "a dynamic html page, showing a factorial table.
     Invoke via browser as URL:
        host:port/<N>,test7:
     where <N> is the registered name of this service.
    "

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Generated Factorial table'.
    replyPresenter parametersAt:#secondHeadLine put:'Test7: Generated Factorial table'.
    aRequest
        nextPutLine:'<h1>Factorial table</h1>';
        nextPutLine:'<table border>';
        nextPutLine:'<th WIDTH=50>x</th>';
        nextPutLine:'<th>fac(x)</th>'.
    1 to:40 do:[:i |
        aRequest nextPutLine:'<tr>'.
        aRequest nextPutLine:'<td ALIGN="center">' , i printString , '</td>'.
        aRequest nextPutLine:'<td ALIGN="right">'  , i factorial printString , '</td>'.
        aRequest nextPutLine:'</tr>'.
    ].

    aRequest nextPutLine:'</table>'.
    self addLinksForSource:#'test7:' previous:#'test6:' next:#'test8:' to:aRequest.
!

test8:aRequest
    "a dynamic html page, upload a file.
     Invoke via browser as URL:
        host:port/<N>,test8:
     where <N> is the registered name of this service.
    "
    |fileNameChunk multiPartChunks server fileService uploadDirectory uploadDirectoryBase linkFile removeFilesChunk uploadChunk
     uploadedFilenames replyPresenter|

    server := aRequest server.
    uploadDirectoryBase := 'upLoadTest'.
    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Upload example'.
    replyPresenter parametersAt:#secondHeadLine put:'Test8: Upload example'.
    fileService := server serviceForLink:'/files' ifAbsent:nil.
    fileService notNil ifTrue:[
        uploadDirectory := fileService fileRoot construct:uploadDirectoryBase.
        uploadDirectory exists ifFalse:[
            Error handle:[:err| self error:err description] do:[
                uploadDirectory makeDirectory.
            ].
        ].
        multiPartChunks := aRequest multiPartChunks.
        multiPartChunks notNil ifTrue:[
            aRequest authenticateForRealm:'SysAdmin'.
            removeFilesChunk := multiPartChunks at:#'RemoveFiles' ifAbsent:nil.
            uploadChunk := multiPartChunks at:#'Upload' ifAbsent:nil.
            removeFilesChunk notNil ifTrue:[
                uploadDirectory directoryContentsAsFilenames do:[:aFilename|
                    Error handle:[:err| self error:err description] do:[
                        aFilename remove.
                    ].
                ].
            ].
            uploadChunk notNil ifTrue:[
                fileNameChunk := multiPartChunks at:#'fileName' ifAbsent:nil.
                (fileNameChunk notNil and:[fileNameChunk isFilenameChunk]) ifTrue:[
                    fileNameChunk writeFileIntoDirectory:uploadDirectory.
                ].
            ].
        ].
    ].
    aRequest
        nextPutLine: '<FORM action="', (aRequest linkIdFor:self selector:#'test8:'),'" method="post" enctype="multipart/form-data">';
        nextPutLine:'<input type="file" name="fileName">&nbsp;&nbsp;'.
    aRequest formSubmitButtonForString:'upload' name:'Upload'.
    aRequest nextPutLine:'<br><P><P>'.
    aRequest formSubmitButtonForString:'Remove all uploaded files' name:'RemoveFiles'.
    aRequest formEnd.
    uploadedFilenames := uploadDirectory directoryContentsAsFilenames.
    uploadedFilenames notEmpty ifTrue:[
        aRequest
            nextPutLine: '<P><H2>Uploaded Files:</H2>'.
        uploadedFilenames do:[:aFilename|
            aFilename isDirectory ifFalse:[
                aRequest nextPutLine: '<BR>'.
                linkFile := uploadDirectoryBase, '/', aFilename baseName.
                aRequest nextPutLine: (aRequest webLinkForText:(aFilename baseName) 
                                            service:fileService 
                                            argument:(linkFile)).
            ].
        ].
    ].
    self addLinksForSource:#'test8:' previous:#'test7:' next:#'testGoogle:' to:aRequest.

    "Modified: / 24-07-2011 / 06:55:12 / cg"
!

testGoogle:aRequest
    "a dynamic html page, showing a factorial table.
     Invoke via browser as URL:
        host:port/<N>,testGoogle:
     where <N> is the registered name of this service.
    "

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Google Search Example'.
    replyPresenter parametersAt:#secondHeadLine put:'TestGoogle: Google Search Example'.
    aRequest
        nextPutLine:'<form method="GET" action="http://www.google.com/custom">';
        nextPutLine:'<a HREF="http://www.google.com/search">';
        nextPutLine:'<IMG SRC="http://www.google.com/logos/Logo_40wht.gif" border="0" ALT="Google" align="middle">';
        nextPutLine:'</a><br>';
        nextPutLine:'<INPUT TYPE="text" name="q" size="15" maxlength="255" value="">';
        nextPutLine:'<INPUT type="submit" name="sa" VALUE="Google Search">';
        nextPutLine:'<input type="hidden" name="domains" value="www.exept.de;"><br>';
        nextPutLine:'<input type="radio" name="sitesearch" value=""> WWW';
        nextPutLine:'<input type="radio" name="sitesearch" value="www.exept.de" checked>eXept';
        nextPutLine:'</form>'.
    self addLinksForSource:#'testGoogle:' previous:#'test7:' next:#'timeNow:' to:aRequest.
!

testStauSuche:aRequest
    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Stau Suche'.
    replyPresenter parametersAt:#secondHeadLine put:'Test Stau Suche'.
    aRequest
        reply:
'<!!-- Beginn Stausuche //-->
<table border=1 cellpadding=0 cellspacing=0 bordercolor=#999999>
<tr>
<td>
<table border=0 cellpadding=2 cellspacing=0>
<tr>
<td width=120 height=18 background="" bgcolor=#999999 align=center>
<font face=arial size=2 color=white>
<a style="color=white; text-decoration=none;" href=http://www.verkehrsinformation.de target=_blank><b>STAUSUCHE</b></a>
</td>
</tr>
<tr>
<td width=120 height=35 align=left bgcolor=#e3e4e3>
<form action="http://www.verkehrsinformation.de/index.php" method=get>
<input type=hidden name=tmp value=search>
<input type=hidden name=pid value="f6eb31b9">
<font face=verdana,arial size=1 color=black><b>Autobahn:</b><br>
<input type=text size=4 name=road value="alle" style="font-size: 12px;">z.B.: A3
</td>
</tr>
<tr>
<td align=left bgcolor=#e3e4e3>
<font face=verdana,arial size=1 color=black><b>Bundesland:</b><br>
<select name=region style="font-size: 12px;">
<option value="%">alle
<option value="%">-------
<option value=1>Baden-Wrttenb.
<option value=2>Bayern
<option value=3>Berlin
<option value=4>Brandenburg
<option value=5>Bremen
<option value=6>Hamburg
<option value=7>Hessen
<option value=8>Mecklenburg-V.
<option value=9>Niedersachsen
<option value=10>Nordrhein-Westf.
<option value=11>Rheinland-Pfalz
<option value=12>Saarland
<option value=13>Sachsen-Anhalt
<option value=14>Schleswig-Holst.
<option value=15>Thringen
<option value=16>Sachsen
</select>
</td>
</tr>
<tr>
<td align=center bgcolor=#e3e4e3 height=40 valign=center>
<input type=submit value="Stau suchen" style="font-size:12px; color:black; background-color:#ffcc00; border-color:white">
</td>
</tr></form>
</table>
</td>
</tr>
</table>
<!!-- Ende Stausuche //-->'.
!

timeNow:aRequest
    "a very simple reply: shows the current time.

     Invoke via the browser as URL:
        host:port/<N>,timeNow:
     where <N> is the registered name of this service.
    "

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Server Time Example'.
    replyPresenter parametersAt:#secondHeadLine put:'TimeNow: Server Time Example'.
    aRequest
        nextPutLine:'<h1>Server Time Example</h1>';
        nextPutLine:'The servers time is ' , Time now printString.

    self addLinksForSource:#'timeNow:' previous:#'testGoogle:' next:nil to:aRequest.
! !

!WebDemoApp1 methodsFor:'serving-helpers'!

addLinksForSource:selector previous:previousOrNil next:nextOrNil to:aRequest

    aRequest nextPutLine:'<P>'.
    aRequest nextPutLine:'<HR>'.
    aRequest nextPutLine:(aRequest webLinkForText:'Source' service:self selector:#showSource: argument:selector).
    aRequest nextPutLine:'<P>'.
    previousOrNil notNil ifTrue:[
        aRequest webLinkForText:'Goto ',previousOrNil service:self selector:previousOrNil.
    ].
    nextOrNil notNil ifTrue:[
        aRequest nextPutLine:'<BR>',(aRequest webLinkForText:'Goto ',nextOrNil service:self selector:nextOrNil).
    ].
    aRequest nextPutLine:'<BR>',(aRequest webLinkForText:'Demo Index' service:self selector:#'openAsWebPage:').
! !

!WebDemoApp1 class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebDemoApp1.st,v 1.58 2011/07/24 04:58:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebDemoApp1.st,v 1.58 2011/07/24 04:58:27 cg Exp $'
! !
