"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

"{ NameSpace: Demos }"

HTTPSelectorService subclass:#WebProcessList
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'demos-WebApplications'
!

!WebProcessList class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!WebProcessList class methodsFor:'defaults'!

defaultAllowRobots
    ^ false
!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
!

isThreadSafe
    ^ true
!

linkName
    "return the default linkName path (with slash)"

    ^ '/Processes'
! !

!WebProcessList class methodsFor:'helpers'!

getWhereStringFor:con running:isRunning
    |c found skipping rs rc r sel|

    con notNil ifTrue:[
        c := con.
        found := false.
        isRunning ifFalse:[
            "/ search for a semaphore-wait in the top 10 contexts
            1 to:10 do:[:n |
                found ifFalse:[
                    c notNil ifTrue:[
                        (c receiver class == Semaphore) ifTrue:[
                            c selector == #wait ifTrue:[
                                found := true.
                            ]
                        ].
                        c := c sender.
                    ]
                ]
            ].
        ].
        found ifFalse:[
            "/ search for a non-processor, non-process
            "/ receiver in the top 10 contexts
            c := con.
            1 to:10 do:[:n |
                found ifFalse:[
                    c notNil ifTrue:[
                        ((r := c receiver) ~~ Processor and:[ r class ~~ Process ]) ifTrue:[
                            found := true.
                        ] ifFalse:[
                            c := c sender.
                        ]
                    ]
                ]
            ]
        ].

        "/ skip, until an interesting context is
        "/ found.
        "/ this skips intermediate contexts, which lead
        "/ to the sema-wait (for example, unwind blocks,
        "/ delay-stuff etc.)
        found ifFalse:[
            c := con
        ].
        skipping := true.
        [ skipping ] whileTrue:[
            skipping := false.
            (c notNil and:[ c receiver == Delay or:[ c receiver class == Delay ] ]) ifTrue:[
                c := c sender.
                skipping := true.
            ].
            [
                c notNil and:[ c receiver isBlock and:[ c selector startsWith:'value' ] ]
            ] whileTrue:[
                c := c sender.
                skipping := true.
            ].
            [
                c notNil and:[ c receiver isBlock and:[ c selector = 'ensure:' ] ]
            ] whileTrue:[
                c := c sender.
                skipping := true.
            ].
            [
                c notNil
                    and:[ c receiver == OperatingSystem and:[ c selector == #unblockInterrupts ] ]
            ] whileTrue:[
                c := c sender.
                skipping := true.
            ].
            [
                c notNil and:[ c isBlockContext ]
            ] whileTrue:[
                c := c home.
                skipping := true.
            ].
        ].
        c notNil ifTrue:[
            sel := c selector.
            sel isNil ifTrue:[
                sel := '* unknown *'
            ].
            r := c receiver.
            rc := r class.
            rs := rc name.
            (rc == SharedQueue
            or:[rc == RecursionLock]) ifTrue:[
                rs := rs , ' (', (r identityHash bitShift:-12) hexPrintString,') '.
            ].
            ^ (rs , '>>' , sel).
        ]
    ].
    ^ ''

    "Modified: / 07-06-2007 / 22:53:40 / cg"
!

processList
    "answer the process list as a string"

    |list writeStream|

    list := ProcessorScheduler knownProcesses asOrderedCollection 
                select:[:p | p notNil and:[p id notNil]].

    list sort:[:p1 :p2 | p1 id < p2 id].

    writeStream := '' writeStream.

    list do:[:p |
        |nm con where|

        nm := (p name printStringPaddedTo:50) contractTo:50.
        con := p suspendedContext.
        con isNil ifTrue:[
            where := ''
        ] ifFalse:[
            where := self 
                        getWhereStringFor:con 
                        running:(p == Processor activeProcess).
        ].

        writeStream 
            nextPutAll:(p id printStringLeftPaddedTo:5); space;
            nextPutAll:nm; space;
            nextPutAll:(p priority printStringLeftPaddedTo:2); space;
            nextPutAll:(p state printStringPaddedTo:9); space;
            nextPutAll:where; cr.
    ].

    ^ writeStream contents.

    "
      self processList
    "
! !

!WebProcessList methodsFor:'defaults'!

allowedMessages
    ^ #(
        processList: 
        processListWithFrames:
        processListWithFramesHeader:   
        processListWithFramesList:
        )
! !

!WebProcessList methodsFor:'queries-service'!

defaultMessage

    ^ #processList
! !

!WebProcessList methodsFor:'registration'!

addRequiredForeignServicesTo:aServer 
    "I need a file service for the bitmaps and icons"

    self addStandardFileServicesTo:aServer.
! !

!WebProcessList methodsFor:'requests-server'!

processList: aRequest
    "send a list of running processes"

    |response replyPresenter|

    response := aRequest response.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Processes'.
    replyPresenter parametersAt:#beforeTitle put:(response refreshStringFor:self selector:#'processList:' time:10).
    replyPresenter parametersAt:#secondHeadLine put:'Processes (autoupdate every 10 seconds)'.

    response reply:'
Currently executing threads on host <b>' , OperatingSystem getHostName ,'</b>:
<p>
<pre><code>
', self class processList, '</code></pre>
<p>
' , (aRequest webLinkForText:'[update]' service:self selector:#'processList:') , '
<a HREF="/">[home]</a>
<p>
'.
!

processListWithFrames: aRequest
    "send a list of running processes using frames"

    aRequest
        reply:'<html>
<head><title>Processes</title>
</head>

<frameset ROWS="200,100%">
  <frame NAME="ProcessListHeader" SRC="' , (aRequest linkIdFor:self selector:'processListWithFramesHeader') , '" NORESIZE SCROLLING="no">
  <frame NAME="ProcessList"       SRC="' , (aRequest linkIdFor:self selector:'processListWithFramesList') , '" SCROLLING="yes">
</frameset>

<body>
</body>
</html>
'.

    "Created: / 27.9.1999 / 11:02:35 / cg"
    "Modified: / 27.9.1999 / 11:15:46 / cg"
!

processListWithFramesHeader: aRequest
    "the header of the frame based process list."

    |replyPresenter|

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Processes'.
    replyPresenter parametersAt:#secondHeadLine put:'Processes (autoupdate every 10 seconds)'.
    aRequest
        reply: '
<p>
' , (aRequest webLinkText:'[update]' target:'ProcessList' service:self selector:#'processListWithFramesList:') , '
<a HREF="/" TARGET="_top">[home]</A>
<p>
'.

    "Created: / 27.9.1999 / 11:10:44 / cg"
    "Modified: / 27.9.1999 / 11:18:01 / cg"
!

processListWithFramesList: aRequest
    "the list-part of the frame based process list."

    |response replyPresenter|

    response := aRequest response.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:'Processes'.
    replyPresenter parametersAt:#secondHeadLine put:'Processes'.
    replyPresenter parametersAt:#beforeTitle put:(response refreshStringFor:self selector:#'processListWithFramesList:' time:10).

    response reply:'<body><pre><code>', self class processList, '</code></pre>'.
! !

!WebProcessList class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebProcessList.st,v 1.27 2010/04/01 17:03:57 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebProcessList.st,v 1.27 2010/04/01 17:03:57 stefan Exp $'
! !
