"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer' }"

"{ NameSpace: Demos }"

HTTPService subclass:#WebSlideShow
	instanceVariableNames:'slideDirectory numberOfColumns numberOfSlidesPerPage thumbSize
		thumbBorders thumbStyle showFileNames showFileSizes
		allowSlideShow withImageWidthOption allowUploads uploadRealm
		cachedDirectoryContents cachedImageFiles'
	classVariableNames:''
	poolDictionaries:''
	category:'demos-WebApplications'
!

Object subclass:#CachedDirectoryEntry
	instanceVariableNames:'modificationTime files'
	classVariableNames:''
	poolDictionaries:''
	privateIn:WebSlideShow
!

!WebSlideShow class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Provides a thumbnailed slides presentation;
    for the given slideDirectory, referring pages are synthesized,
    and thumbnails automatically generated, as required.
    To add more slides, simply create a sub-folder of the slidesDirectory,
    put the slides there, and open a browser; thats it - all done automatically
    and on the fly.

    To start:

      start the HTTPServer at some port:
          aServer := HTTPServer startServerOnPort:8080

      register the class (towards the HTTPServer) with:
          service := Demos::WebSlideShow new.
          service slideDirectory:'someDirectory'.
          service registerServiceIn:(HTTPServer runningServers first) as:'slides'.

    Open a netscape or internet explorer (somewhere);
    enter the URL:
        httP://host:port/slides

    [author:]
        Claus Gittinger (cg@exept)

    [see also:]
        HTTPServer
"
!

examples

"
    start the HTTPServer at some port and register 
    the class (towards the HTTPServer)

    Open a netscape or internet explorer (somewhere);
    enter the URL:
        http://host:port/WebSlideShow
                                                                [exBegin]
    |aServer service|

    aServer := HTTPServer serverOnPort:8081.

    service := Demos::WebSlideShow new.
    service slideDirectory:'/phys/exept/unsaved3/cg/camp smalltalk bilder'.
    service numberOfColumns:5.
    service numberOfSlidesPerPage:30.
    service registerServiceIn:aServer as:'campSmalltalk'.
                                                                [exEnd]
"
! !

!WebSlideShow class methodsFor:'defaults'!

defaultReplyPresenterClass
    ^ ExeptReplyPresenter
!

isThreadSafe
    ^ true
!

linkName
    "return the default path (with slash)"

    ^ '/slides'
!

newSetIndicationFile

    ^ 'newSet'
!

slideDirectory
    "my default slides directory"

    ^ Filename homeDirectory construct:'slides'
! !

!WebSlideShow class methodsFor:'queries-service'!

settingsApplicationClass
    "my application in the settings dialog"

    ^ WebSlideShowSettingsApplication
! !

!WebSlideShow methodsFor:'accessing'!

allowSlideShow
    ^ allowSlideShow ? false
!

allowSlideShow:aBoolean
    allowSlideShow := aBoolean.
!

allowUploads
    ^ allowUploads ? false
!

allowUploads:something
    allowUploads := something.
!

numberOfColumns
    "the number of columns to generate (default: 5)"

    ^ numberOfColumns ? 5
!

numberOfColumns:something
    "set the number of columns to generate"

    numberOfColumns := something.
!

numberOfSlidesPerPage
    "the number of thumbs per page (default: 25)"

    ^ numberOfSlidesPerPage ? 25
!

numberOfSlidesPerPage:something
    "set the number of thumbs per page"

    numberOfSlidesPerPage := something.
!

showExeptFrame
    "old instance var keep selector for loading old settings files"

    ^ self
!

showExeptFrame:aBoolean
    "old instance var keep selector for loading old settings files"

    ^ self
!

showFileNames
    ^ showFileNames ? true
!

showFileNames:aBoolean
    showFileNames := aBoolean
!

showFileSizes
    ^ showFileSizes ? false
!

showFileSizes:aBoolean
    showFileSizes := aBoolean
!

slideDirectory
    "where the slides are"

    ^ slideDirectory ? self class slideDirectory
!

slideDirectory:something
    "set the value of the instance variable 'slideDirectory' (automatically generated)"

    slideDirectory := something.
!

slideShowDelay
    ^ 6
!

thumbBorders
    ^ thumbBorders ? true
!

thumbBorders:aBoolean
    thumbBorders := aBoolean
!

thumbSize
    "the size of the generated thumbs (default: 128@128)"

    ^ thumbSize ? (128 @ 128)
!

thumbSize:anExtent
    "the size of the generated thumbs (default: 128@128)"

    thumbSize := anExtent
!

thumbStyle
    ^ thumbStyle ? #normal
!

thumbStyle:aSymbol
    (thumbStyle notNil and:[thumbStyle ~= aSymbol]) ifTrue:[
        self flushCachedThumbImages
    ].
    thumbStyle := aSymbol
!

uploadRealm
    ^ uploadRealm
!

uploadRealm:something
    uploadRealm := something.
!

withImageWidthOption
    ^ withImageWidthOption ? false
!

withImageWidthOption:something
    withImageWidthOption := something.
! !

!WebSlideShow methodsFor:'helpers'!

areAnyImagesIn:aDirectory
    "true if there are any files with an image mimeType in aDirectory"

    aDirectory directoryContentsAsFilenamesDo:[:each |
        (each isDirectory not and:[each mimeTypeFromName isImage]) ifTrue:[^ true].
    ].
    ^ false.
!

criticalDirectoryAccessIn:anAbsolutePath do:aBlock
    "synchronized critical update of the thumb directory"

    "/ for now - use per-class synchronization
    "/ (should do per directory synchronization)

    self class synchronized:[
        aBlock value.
    ].
!

filesInDirectory:path
    |entry files pathName|

    pathName := path pathName.
    cachedDirectoryContents isNil ifTrue:[
        cachedDirectoryContents := Dictionary new.
    ].
    entry := cachedDirectoryContents at:pathName ifAbsent:nil.
    entry notNil ifTrue:[
        path modificationTime > entry modificationTime ifTrue:[
            cachedDirectoryContents removeKey:pathName    
        ] ifFalse:[
            files := entry files.
        ].
    ].
    files isNil ifTrue:[
        files := path directoryContentsAsFilenames.
        files := files sort:[:a :b | a name < b name].
        entry := CachedDirectoryEntry new 
                        modificationTime:(path modificationTime) 
                        files:files.
        cachedDirectoryContents at:pathName put:entry.
    ].
    ^ files.
!

imageFilesInDirectory:path
    |files pathName entry|

    pathName := path pathName.
    cachedImageFiles isNil ifTrue:[
        cachedImageFiles := Dictionary new.
    ].

    entry := cachedImageFiles at:pathName ifAbsent:nil.
    entry notNil ifTrue:[
        path modificationTime > (entry modificationTime + 10 minutes) ifTrue:[
            cachedImageFiles removeKey:pathName    
        ] ifFalse:[
            files := entry files.
        ].
    ].
    files isNil ifTrue:[
        files := self filesInDirectory:path.
        files := files select:[:each | each isDirectory not and:[each mimeTypeFromName isImage]].
        entry := CachedDirectoryEntry new 
                        modificationTime:(path modificationTime) 
                        files:files.
        cachedImageFiles at:pathName put:entry.
    ].
    ^ files.

    "Modified: / 08-05-2012 / 18:01:04 / cg"
!

newSetIn:aDirectory
    "true if there are any files with an image mimeType in aDirectory"

    ^ aDirectory directoryContents includes:(self class newSetIndicationFile).
! !

!WebSlideShow methodsFor:'maintenance html presentation'!

specificChangeFromRequest:aRequest
    "extracts parameter from an html-maintenance post-request"

    |tmp allowSlideShow thumbBorders showFileSizes point 
    thumbStyle thumbStyleIndex withImageWidthOption allowUploads realm|

    self flushCachedThumbImages.

    tmp := aRequest urlParameterAt:'slideDirectory'.
    tmp notNil ifTrue:[
        self slideDirectory:(tmp asFilename).
    ].
"/    showExeptFrame := aRequest urlParameterAt:'showExeptFrame'.
"/    self showExeptFrame:(showExeptFrame = 'ON').
    allowSlideShow := aRequest urlParameterAt:'allowSlideShow'.
    self allowSlideShow:(allowSlideShow = 'ON').


    tmp := aRequest urlParameterAt:'numberOfColumns'.
    tmp notNil ifTrue:[
        self numberOfColumns:(tmp asNumber).
    ].
    tmp := aRequest urlParameterAt:'numberOfSlidesPerPage'.
    tmp notNil ifTrue:[
        self numberOfSlidesPerPage:(tmp asNumber).
    ].
    tmp := aRequest urlParameterAt:'thumbSize'.
    tmp notNil ifTrue:[
        point := Point readFrom:tmp onError:[
            aRequest responseError:'Can not Change Service - thumbSize:', tmp asString, ' is not a point'
                    title:'Can not Change Service' 
                    request:aRequest 
                    backSelector:'addRemoveChangeServices:'.
            ^ self
        ].
        self thumbSize:point.
    ].
    tmp := aRequest urlParameterAt:'thumbStyle'.
    tmp notNil ifTrue:[
        thumbStyleIndex := (self styleDescriptionsAndSymbols collect:[:s | s first]) indexOf:tmp ifAbsent:nil.
        thumbStyleIndex isNil ifTrue:[
            aRequest responseError:('Unexpected thumbStyle ', tmp asString)
                    title:'Can not Change Service' 
                    request:aRequest 
                    backSelector:'addRemoveChangeServices:'.
            ^ self
        ].
        thumbStyle := (self styleDescriptionsAndSymbols collect:[:s | s second]) at:thumbStyleIndex.
        self thumbStyle:thumbStyle.
    ].
    thumbBorders := aRequest urlParameterAt:'thumbBorders'.
    self thumbBorders:(thumbBorders = 'ON').
    showFileNames := aRequest urlParameterAt:'showFileNames'.
    self showFileNames:(showFileNames = 'ON').
    showFileSizes := aRequest urlParameterAt:'showFileSizes'.
    self showFileSizes:(showFileSizes = 'ON').
    withImageWidthOption := aRequest urlParameterAt:'withImageWidthOption'.
    self withImageWidthOption:(withImageWidthOption = 'ON').
    allowUploads := aRequest urlParameterAt:'allowUploads'.
    self allowUploads:(allowUploads = 'ON').
    realm := aRequest urlParameterAt:'uploadRealm'.
    realm = 'None' ifTrue:[ realm := nil].
    self uploadRealm:realm.
!

specificInputFieldsForRequest:aRequest
    "generates the html-fragment for the maintenance form"

    |styleList styleDescriptionsAndSymbols currentStyle styleSymbolList realms selected|

    styleDescriptionsAndSymbols := self styleDescriptionsAndSymbols.
    styleList := styleDescriptionsAndSymbols collect:[:s | s first].
    styleSymbolList := styleDescriptionsAndSymbols collect:[:s | s second].
    currentStyle := self thumbStyle.

    aRequest 
        nextPutLine:'<TABLE WIDTH="100%">';
        nextPutLine:'<TR>';
        nextPutLine:'<TD>';
        inputFieldFor:'slideDirectory' 
            type:nil 
            size:'50' 
            label:('Image Directory:</TD><TD WIDTH="75%">') 
            initialValue:self slideDirectory asString;
        nextPutLine:'</TD></TR>'.

   aRequest
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

"/   aRequest
"/        nextPutLine: '<TR><TD>';
"/        inputFieldFor:'showExeptFrame' 
"/            type:'checkbox' 
"/            size:nil 
"/            label:('eXept Page Style:</TD><TD>') 
"/            initialValue:'ON'
"/            append:(self showExeptFrame ifTrue:['checked'] ifFalse:['']);
"/        nextPutLine: '</TD></TR>'.
    aRequest
        nextPutLine: '<TR><TD>';
        inputFieldFor:'allowSlideShow' 
            type:'checkbox' 
            size:nil 
            label:('Allow Slide Show:</TD><TD>') 
            initialValue:'ON'
            append:(self allowSlideShow ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'<TR> <TD>&nbsp;</TD><TD>&nbsp;</TD> </TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'numberOfColumns' 
            type:nil 
            size:'50' 
            label:'Columns:</TD><TD WIDTH="75%">'
            initialValue:self numberOfColumns asString;
        nextPutLine:'</TD></TR>'.
    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'numberOfSlidesPerPage' 
            type:nil 
            size:'50' 
            label:('Slides per Page:</TD><TD>') 
            initialValue:self numberOfSlidesPerPage asString;
        nextPutLine:'</TD></TR>'.
    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'thumbSize' 
            type:nil 
            size:'50' 
            label:('ThumbSize:</TD><TD WIDTH="75%">') 
            initialValue:self thumbSize asString;
        nextPutLine:'</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        nextPutAll:'Style:';
        nextPutLine: '</TD><TD>';
        nextPutLine: '<SELECT NAME=thumbStyle SIZE=1>'.

    styleList doWithIndex:[:aStyle :index|  
        | selected |
        ((styleSymbolList at:index) = currentStyle) ifTrue:[
            aRequest
                nextPutLine: '<OPTION SELECTED>', aStyle asString,'</OPTION>'.
        ] ifFalse:[
            aRequest
                nextPutLine: '<OPTION>', aStyle asString,'</OPTION>'.
        ].
    ].
    aRequest
        nextPutLine: '</SELECT> ';
        nextPutLine:'</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'thumbBorders' 
            type:'checkbox' 
            size:nil 
            label:('Borders:</TD><TD>') 
            initialValue:'ON'
            append:(self thumbBorders ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'showFileNames' 
            type:'checkbox' 
            size:nil 
            label:('Show File Names:</TD><TD>') 
            initialValue:'ON'
            append:(self showFileNames ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'showFileSizes' 
            type:'checkbox' 
            size:nil 
            label:('Show File Sizes:</TD><TD>') 
            initialValue:'ON'
            append:(self showFileSizes ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:('allowUploads') 
            type:'checkbox' 
            size:nil 
            label:('Allow Uploads:</TD><TD>') 
            initialValue:'ON'
            append:((self allowUploads ? false) ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        nextPutAll:'Upload Realm:';
        nextPutLine: '</TD><TD>';
        nextPutLine: '<SELECT NAME=uploadRealm SIZE=1> '.

    realms := aRequest server realmUsersMap keys asOrderedCollection.
    realms addFirst:'None'.
    realms do:[:aKey |
         | serviceRealm |
        serviceRealm := self uploadRealm ? 'None'.
        aKey = self uploadRealm ifTrue:[
            selected := ' SELECTED'.
        ] ifFalse:[
            selected := ''.
        ].
        aRequest
            nextPutLine: '<OPTION', selected,'>', aKey asString,'</OPTION>'.
    ].
    aRequest
        nextPutLine: '</SELECT>';
        nextPutLine:'</TD></TR>'.

    aRequest
        nextPutLine:'<TR><TD>';
        inputFieldFor:'withImageWidthOption'
            type:'checkbox' 
            size:nil 
            label:('Field for Image Size:</TD><TD>') 
            initialValue:'ON'
            append:(self withImageWidthOption ifTrue:['checked'] ifFalse:['']);
        nextPutLine: '</TD></TR>'.

    aRequest
        nextPutLine: '</TABLE>'.
! !

!WebSlideShow methodsFor:'misc'!

flushCachedThumbImages
    |slideDir|

    slideDir := self slideDirectory asFilename.
    ExternalStream openErrorSignal handle:[:ex |
        ex proceed
    ] do:[
        slideDir recursiveDirectoryContentsDo:[:relPath |
            |f|

            f := slideDir construct:relPath.
            f baseName = 'thumbs' ifTrue:[
                f isDirectory ifTrue:[
                    OperatingSystem accessDeniedErrorSignal 
                        handle:[:ex | ex proceed ]
                        do:[
                            f recursiveRemove
                        ]
                ].
            ].
        ].
    ].
!

styleDescriptionsAndSymbols
    ^ #(
        ('Normal'       #normal)
        ('Grayscale'    #grayScale)
        ('Button'       #button)
        ('Shadow'       #shadow)
        ('Slide'        #slide)
"/        ('Cartoon'      #cartoon)     "/ does not work yet
    )
! !

!WebSlideShow methodsFor:'registration'!

addRequiredForeignServicesTo:aServer 
    "I need a file service for the bitmaps and icons"

    self addStandardFileServicesTo:aServer.
! !

!WebSlideShow methodsFor:'serving'!

process:aRequest
    "process an http request"

    |relPath|

    relPath := aRequest pathRelativeToService.

    self doFormActions:aRequest forRelativePath:relPath.
    self slidePage:aRequest     forRelativePath:relPath
! !

!WebSlideShow methodsFor:'serving-private'!

authenticateUploadFor:aRequest
    | upLoadRealm|

    upLoadRealm := self uploadRealm.
    upLoadRealm notNil ifTrue:[
        aRequest authenticateForRealm:upLoadRealm.
    ].
!

createNewImageSet:aRequest forRelativePath:relPath
    | imageSetName slideDir newDirectory newSetFile errorMessage replyPresenter|

    self authenticateUploadFor:aRequest.

    imageSetName := aRequest urlParameterAt:'imageSetName'.
    imageSetName notEmptyOrNil ifTrue:[
        slideDir := self slideDirectory asFilename.
        slideDir notNil ifTrue:[
            newDirectory := slideDir construct:relPath.
            newDirectory := newDirectory construct:imageSetName.
            newDirectory exists ifFalse:[
                Error handle:[:ex| 
                    errorMessage := ex description.
                    replyPresenter := aRequest replyPresenter.
                    replyPresenter title:'Can not create new image set'.
                    replyPresenter secondHeadLine:'Can not create new image set'.
"/                    self showExeptFrame ifTrue:[
"/                        aRequest stxBodyStart:'Can not create new image set'
"/                    ] ifFalse:[
"/                        aRequest bodyStart
"/                    ].
                    aRequest nextPutAll:errorMessage.
"/                    self pageFinish:aRequest exeptStyle:self showExeptFrame
                    self pageFinish:aRequest 
                ] do:[
                    newDirectory makeDirectory.
                    newSetFile := newDirectory construct:(self class newSetIndicationFile).
                    newSetFile writeStream close.
                ].
            ].
        ]
    ].

    "Modified: / 02-02-2007 / 12:19:54 / cg"
!

doFormActions:aRequest forRelativePath:relPath
    "process all the form actions"

    |createNewImageSet multiPartChunks uploadChunk|


    multiPartChunks := aRequest multiPartChunks.
    multiPartChunks notNil ifTrue:[
        uploadChunk := multiPartChunks at:#'Upload' ifAbsent:nil.
        (uploadChunk notNil) ifTrue:[
            self upLoadImage:aRequest forRelativePath:relPath.
        ].
    ].
    createNewImageSet := aRequest urlParameterAt:'CreateNewImageSet'.
    createNewImageSet notNil ifTrue:[
        self createNewImageSet:aRequest forRelativePath:relPath.
    ].
!

file:absolutePathToFile for:aRequest
    "return a files complete contents"

    |mimeType|

    absolutePathToFile exists ifFalse:[
        ^ aRequest reportNotFound:'No such file: ' , absolutePathToFile baseName. 
    ].

    mimeType := absolutePathToFile mimeTypeFromName.

    aRequest response maxAge:60*60*24.
    aRequest sendBinaryStream:(absolutePathToFile readStream binary) type:mimeType.
!

generateTableForSlidesIn:path relative:relativePath for:aRequest 
    "generate an html table containing the thumbs"

    |colNr numSlides firstSlideIndex prevSlidesIndex nextSlidesIndex imageFiles
     genLinkForPrev genLinkForNext genLinkForFirst genLinkForLast genShowLink genLinks endPage 
     imgWidth slideIndex numberOfSlides|

    aRequest nextPutLine:'<!!-- generateTableForSlides -->'.

    colNr := numSlides := 0.
    firstSlideIndex := aRequest urlParameterAt:'first' ifAbsent:'1'.
    firstSlideIndex := Number readFrom:firstSlideIndex onError:1.
    firstSlideIndex := firstSlideIndex max:1.

    imgWidth := aRequest urlParameterAt:'imageWidth' ifAbsent:nil.
    imgWidth notNil ifTrue:[
        imgWidth := Number readFrom:imgWidth onError:nil.
    ].

    firstSlideIndex > 1 ifTrue:[
        prevSlidesIndex := (firstSlideIndex - self numberOfSlidesPerPage) max:1.
    ].
    nextSlidesIndex := firstSlideIndex + self numberOfSlidesPerPage.

    imageFiles := self imageFilesInDirectory:path.
    numberOfSlides := imageFiles size.

    genLinkForFirst := 
        [
            firstSlideIndex > 1 ifTrue:[
                aRequest reply:'<a href="'. 
                aRequest replyURLEscaped:(aRequest fullPath).
                aRequest reply:'&first=1'.
                imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
                aRequest reply:'"> First </a>'.
            ] ifFalse:[
                aRequest reply:'<br>'.
            ].
        ].

    genLinkForPrev := 
        [
            (prevSlidesIndex notNil 
            and:[ prevSlidesIndex ~~ 1 ]) ifTrue:[
                aRequest reply:'<a href="'.
                aRequest replyURLEscaped:(aRequest fullPath).
                aRequest reply:'&first=',prevSlidesIndex printString.
                imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
                aRequest reply:'"> Previous </a>'.
            ] ifFalse:[
                aRequest reply:'<br>'.
            ].
        ].

    genLinkForNext := 
        [
            |idxOfFirstInlast|

            idxOfFirstInlast := numberOfSlides - self numberOfSlidesPerPage + 1.
            (firstSlideIndex < idxOfFirstInlast 
            and:[ nextSlidesIndex ~= idxOfFirstInlast ]) ifTrue:[
                aRequest reply:'<a href="'.
                aRequest replyURLEscaped:(aRequest fullPath).
                aRequest reply:'&first=',nextSlidesIndex printString.
                imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
                aRequest reply:'"> Next </a>'.
            ] ifFalse:[
                aRequest reply:'<br>'.
            ].
        ].

    genLinkForLast :=
        [
            |idxOfFirstInlast|

            idxOfFirstInlast := numberOfSlides - self numberOfSlidesPerPage + 1.
            firstSlideIndex < idxOfFirstInlast ifTrue:[
                aRequest reply:'<a href="'.
                aRequest replyURLEscaped:(aRequest fullPath).
                aRequest reply:'&first=',idxOfFirstInlast printString.
                imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
                aRequest reply:'"> Last </a>'.
            ] ifFalse:[
                aRequest reply:'<br>'.
            ].
        ].

    genShowLink := 
        [
            aRequest reply:'<a href="'.
            aRequest replyURLEscaped:(aRequest fullPath).
            aRequest reply:'/show.html&index=1&delay=',self slideShowDelay printString.
            imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
            aRequest reply:'">Start Show</a>'.
        ].

    genLinks :=
        [
            numberOfSlides > 0 ifTrue:[
                (numberOfSlides > self numberOfSlidesPerPage
                or:[firstSlideIndex > 1
                or:[self allowSlideShow]])
                ifTrue:[
                    aRequest nextPutLine:'<table border="1"><tr>'.
                    ((numberOfSlides > self numberOfSlidesPerPage)
                    or:[firstSlideIndex > 1]) ifTrue:[
                        aRequest nextPutLine:'<td align="center" width="75px">'.
                        genLinkForFirst value.
                        aRequest nextPutLine:'</td>'.
                        aRequest nextPutLine:'<td align="center" width="75px">'.
                        genLinkForPrev value.
                        aRequest nextPutLine:'</td>'.
                        aRequest nextPutLine:'<td align="center" width="75px">'.
                        genLinkForNext value.
                        aRequest nextPutLine:'</td>'.
                        aRequest nextPutLine:'<td align="center" width="75px">'.
                        genLinkForLast value.
                        aRequest nextPutLine:'</td>'.
                    ].
                    self allowSlideShow ifTrue:[
                        aRequest nextPutLine:'<td align="center" width="75px">'.
                        genShowLink value.
                        aRequest nextPutLine:'</td>'.
                    ].
                    aRequest nextPutLine:'</tr></table>'.
                ].
            ].
        ].

    endPage := [
        aRequest nextPutLine:'<p>'.
        genLinks value.
        aRequest nextPutLine:'<p>'.
    ].

    genLinks value.
    aRequest nextPutLine:'<p>'.

    aRequest nextPutLine:'<table width="80%" border="0">'.

    (self filesInDirectory:path)
    do:[:eachFn |
        (eachFn baseName asLowercase = 'readme') ifTrue:[
            aRequest nextPutLine:'<!!-- start of included readMe file -->'.
            aRequest nextPutAll:(eachFn contentsOfEntireFile). 
            aRequest nextPutLine:'<!!-- end of included readMe file -->'.
        ].
    ].

    slideIndex := firstSlideIndex.
    [ 
        (slideIndex <= numberOfSlides)
         and:[numSlides < self numberOfSlidesPerPage]
    ] whileTrue:[
        |each thumbName imgName imgPageName servicePath|

        each := imageFiles at:slideIndex.

        servicePath := aRequest serviceLinkName , '/'.
        relativePath ~= '' ifTrue:[
            servicePath := servicePath , relativePath.
            (relativePath endsWith:'/') ifFalse:[
                servicePath := servicePath , '/'.    
            ].
        ].

        imgName := servicePath , each baseName.
        imgPageName := imgName , '.html'.
        thumbName := servicePath , 'thumbs/tn_' , each baseName.

        colNr == 0 ifTrue:[
            aRequest nextPutLine:'<tr>'.
        ].
        aRequest nextPutLine:'<td>'.

        self thumbBorders ifTrue:[
            aRequest nextPutLine:'  <table border="1">'.
        ] ifFalse:[
            aRequest nextPutLine:'  <table border="0">'.
        ].
        aRequest nextPutLine:'  <tr>'.
        (self showFileNames not
        and:[self showFileSizes not]) ifTrue:[
            aRequest nextPutLine:('<td width="%1" align="center">' bindWith:(self thumbSize x)).
        ] ifFalse:[
            aRequest nextPutLine:'<td>'.
        ].
        aRequest reply:'<a href="'.
        aRequest replyURLEscaped:imgPageName "imgName".
        aRequest reply:'&index=', slideIndex printString.
        imgWidth notNil ifTrue:[ aRequest reply:'&imageWidth=',imgWidth printString].    
        aRequest reply:'">'.
        aRequest reply:'<img align="bottom" '.
        self thumbStyle == #shadow ifTrue:[
            aRequest reply:'border="0" '.
        ].
        aRequest reply:'src="'.
        aRequest replyURLEscaped:thumbName.
        aRequest reply:'" alt="[',each baseName,']"></a>'.

        aRequest nextPutAll:'<center>'.
        self showFileNames ifTrue:[
            aRequest nextPutLine:each baseName.
            aRequest nextPutAll:'<br>'.
        ].
        self showFileSizes ifTrue:[
            aRequest nextPutLine:((UnitConverter fileSizeStringFor:each fileSize),'b').
            aRequest nextPutAll:'<br>'.
        ].
        aRequest nextPutAll:'</center>'.

        aRequest nextPutLine:'</td>'.
        aRequest nextPutLine:'</tr>'.
        aRequest nextPutLine:'</table>'.

        aRequest nextPutLine:'</td>'.
        colNr := colNr + 1.

        colNr == self numberOfColumns ifTrue:[
            aRequest nextPutLine:'</tr>'.
            colNr := 0.
        ].

        numSlides := numSlides + 1.
        slideIndex := slideIndex + 1.
    ].

    colNr ~= 0 ifTrue:[
        aRequest nextPutLine:'</tr>'.
    ].
    aRequest nextPutLine:'</table>'.
    endPage value.

    numberOfSlides > 0 ifTrue:[
        (firstSlideIndex > 1
        or:[ (firstSlideIndex+numSlides-1) < numberOfSlides ])
        ifTrue:[
            aRequest replyPresenter secondHeadLine:(aRequest fullPath 
                                           , (' (%1..%2 of %3)' 
                                                bindWith:firstSlideIndex 
                                                with:firstSlideIndex+numSlides-1
                                                with:numberOfSlides)).
        ]
    ].

    "Modified: / 02-02-2007 / 13:42:41 / cg"
!

generateTableForSubdirectoriesIn:path relative:relativePath for:aRequest 
    "generate an html table containing links to subdirectories"

    |any imgWidth imagesInDirectory|

    aRequest nextPutLine:'<!!-- generateTableForSubdirectories -->'.

    imgWidth := aRequest urlParameterAt:'imageWidth'.
    imgWidth notNil ifTrue:[
        imgWidth := Number readFrom:imgWidth onError:nil.
    ].

    any := false.
    imagesInDirectory := ((self imageFilesInDirectory:path) collect:[:fn | fn name]) asSet.
    (self filesInDirectory:path) do:[:eachFn |
        |target|

        (self isDirectoryVisible:eachFn) ifTrue:[
            "isDirectory is slow - do a quick check for image files first to speed it up"
            
            ((imagesInDirectory includes:eachFn name) not
                and:[ eachFn isDirectory 
                and:[ (self areAnyImagesIn:eachFn) or:[self newSetIn:eachFn]]]
            )
            ifTrue:[
                any ifFalse:[
                    aRequest nextPutLine:'<table width="80%" border="1">'.
                    aRequest nextPutLine:'<tr>'.
                    any := true.
                ].

                aRequest nextPutLine:'<td>'.
                relativePath notEmpty ifTrue:[
                    target := relativePath , '/', (eachFn baseName).
                ] ifFalse:[
                    target := relativePath , (eachFn baseName).
                ].
                target := HTMLUtilities escape:target.
                imgWidth notNil ifTrue:[ target := target ,'&imageWidth=',imgWidth printString ].    

                aRequest nextPutAll:(aRequest webLinkForText:(eachFn baseName) service:self argument:target).
                aRequest nextPutLine:'</td>'.
                aRequest nextPutLine:'</tr>'.
                aRequest nextPutLine:'<tr>'.
            ].
        ].
    ].

    any ifTrue:[
        aRequest nextPutLine:'</tr>'.
        aRequest nextPutLine:'</table>'.
    ].

    "Modified: / 08-05-2012 / 18:18:24 / cg"
!

generateZipLinksForFilesIn:path relative:relativePath for:aRequest 
    "generate a link for zip/tgz files"

    |zipArchive tgzArchive hasZipArchive hasTGZArchive|

    aRequest nextPutLine:'<!!-- generateZipLinksForFiles -->'.

    zipArchive := path construct:'images.zip'.
    tgzArchive := path construct:'images.tgz'.
    hasZipArchive := zipArchive exists and:[zipArchive isReadable].
    hasTGZArchive := tgzArchive exists and:[tgzArchive isReadable].

    (hasZipArchive or:[hasTGZArchive]) ifTrue:[
        aRequest nextPutLine:'<p>'.
        aRequest nextPutLine:'All ' , (self imageFilesInDirectory:path) size printString , ' Images in "' , path baseName , '" as:'.
        aRequest nextPutLine:'<br>'.
        hasZipArchive ifTrue:[
            aRequest nextPutAll:'<a href="'.
            aRequest replyURLEscaped:(aRequest fullPath).
            aRequest reply:('/', zipArchive baseName , '">zip-Archive</a>').
            aRequest reply:' (' , (UnitConverter fileSizeStringFor:zipArchive fileSize) , 'b)'.
            aRequest nextPutLine:'<br>'.
        ].
        hasZipArchive ifTrue:[
            aRequest reply:'<a href="'.
            aRequest replyURLEscaped:(aRequest fullPath).
            aRequest reply:('/', tgzArchive baseName , '">tgz-Archive</a>').
            aRequest nextPutLine:' (' , (UnitConverter fileSizeStringFor:tgzArchive fileSize) , 'b)'.
        ].
    ].
!

image:absolutePathToImage for:aRequest
    "return a full image"

    ^ self file:absolutePathToImage for:aRequest
!

imagePageBody:aRequest for:path relative:relativePath
    "the body of an image page"

    |imgName files numberOfSlides imgLink slideIndex prev next link firstIndex reqPathDirectory 
     asShow delayTime nextIndex showDir imgWidth response replyPresenter|

    response := aRequest response.

    slideIndex := aRequest urlParameterAt:'index'.
    slideIndex notNil ifTrue:[
        slideIndex := Number readFrom:slideIndex onError:nil.
    ].
    imgWidth := aRequest urlParameterAt:'imageWidth'.
    imgWidth notNil ifTrue:[
        imgWidth := Number readFrom:imgWidth onError:nil.
    ].

    files := self imageFilesInDirectory:(path directory).
    numberOfSlides := files size.
    asShow := false.

    imgName := path withoutSuffix.
    imgName exists ifFalse:[
        asShow := (imgName baseName = 'show').
        asShow ifFalse:[ ^ self ].
    ].
    reqPathDirectory := (UnixFilename named:aRequest fullPath) directory.

    replyPresenter := aRequest replyPresenter.
    asShow ifTrue:[                      
        delayTime := aRequest urlParameterAt:'delay'.
        delayTime := Number readFrom:delayTime onError:[self slideShowDelay].

        nextIndex := slideIndex+1.
        nextIndex > numberOfSlides ifTrue:[ nextIndex := 1 ].

        showDir := (UnixFilename named:('/', aRequest pathRelativeToService)) directoryName.
        showDir = '/' ifTrue:[
            showDir := ''
        ].

        link := (aRequest serviceLinkName,showDir,'/show.html').
        link := HTMLUtilities escape:link.
        link := link , '&index=',nextIndex printString,'&delay=', delayTime printString.
        imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    

        replyPresenter beforeTitle:('<meta http-equiv="refresh" content="',delayTime printString,'; url=',link, '">').
        imgName := files at:slideIndex.
        imgLink := '/' , imgName baseName.
    ] ifFalse:[
        response title:imgName baseName.
        imgLink := (UnixFilename named:relativePath) withoutSuffix baseName.
    ].
    imgLink := reqPathDirectory constructString:imgLink.

    replyPresenter title:imgName baseName.
    replyPresenter secondHeadLine:imgName baseName.
"/    (self showExeptFrame and:[asShow not]) ifTrue:[
"/        aRequest stxBodyStart: imgName baseName
"/    ] ifFalse:[
"/        aRequest bodyStart
"/    ].

    response nextPutLine:'<!!-- imagePageBody -->'.

    slideIndex notNil ifTrue:[
        asShow ifFalse:[
            response nextPutLine:'<table border>'.
            response nextPutLine:'<tr>'.

            response nextPutLine:'<td align="left" width="200px">'.
            slideIndex > 1 ifTrue:[
                prev := (files at:1) baseName.
                link := (HTMLUtilities escape:(reqPathDirectory constructString:prev)) , '.html&index=1'.
                imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    
                response nextPutLine:'First: <a href="',link,'">',prev,'</a>'.
            ] ifFalse:[
                response nextPutLine:'<br>'.
            ].
            response nextPutLine:'</td>'.

            response nextPutLine:'<td align="left" width="200px">'.
            slideIndex > 2 ifTrue:[
                prev := (files at:slideIndex - 1) baseName.
                link := (HTMLUtilities escape:(reqPathDirectory constructString:prev)) , '.html&index=', (slideIndex - 1) printString.
                imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    
                response nextPutLine:'Previous: <a href="',link,'">',prev,'</a>'.
            ] ifFalse:[
                response nextPutLine:'<br>'.
            ].
            response nextPutLine:'</td>'.

            response nextPutLine:'<td align="center" width="100px">'.
            firstIndex := (slideIndex min:(numberOfSlides - self numberOfSlidesPerPage + 1)) max:1.
            link := (HTMLUtilities escape:reqPathDirectory name) , '&first=', slideIndex "firstIndex" printString .
            imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    
            response nextPutLine:'<a href="',link,'">Index</a>'.
            response nextPutLine:'</td>'.

            response nextPutLine:'<td align="right" width="200px">'.
            slideIndex < (numberOfSlides-1) ifTrue:[
                next := (files at:slideIndex + 1) baseName.
                link := (HTMLUtilities escape:(reqPathDirectory constructString:next)) , '.html&index=', (slideIndex + 1) printString.
                imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    

                response nextPutLine:'Next: <a href="',link,'">',next,'</a>'.
            ] ifFalse:[
                response nextPutLine:'<br>'.
            ].
            response nextPutLine:'</td>'.

            response nextPutLine:'<td align="right" width="200px">'.
            slideIndex < numberOfSlides ifTrue:[
                next := (files at:numberOfSlides) baseName.
                link := (HTMLUtilities escape:(reqPathDirectory constructString:next)) , '.html&index=', numberOfSlides printString.
                imgWidth notNil ifTrue:[ link := link , '&imageWidth=',imgWidth printString].    

                response nextPutLine:'Last: <a href="',link,'">',next,'</a>'.
            ] ifFalse:[
                response nextPutLine:'<br>'.
            ].
            response nextPutLine:'</td>'.

            response nextPutLine:'</tr>'.
            response nextPutLine:'</table>'.
            response nextPutLine:'<p>'.
        ].
    ].

    asShow ifTrue:[
        response nextPutAll:'<a href="',(HTMLUtilities escape:(aRequest serviceLinkName, showDir)) "aRequest fullPath",'&first=', slideIndex printString.
        imgWidth notNil ifTrue:[ 
            response nextPutAll:'&imageWidth='; nextPutAll:imgWidth printString
        ].    
        response nextPutAll:'">'.
        response nextPutAll:'<img src="',(HTMLUtilities escape:imgLink),'" alt="Stop Show" width="100%">'.
        response nextPutLine:'</a">'.
    ] ifFalse:[
        response nextPutAll:'<img src="',(HTMLUtilities escape:imgLink),'"'.
        imgWidth notNil ifTrue:[
            response nextPutAll:' width="' , imgWidth printString , '"'.
        ].
        response nextPutLine:'>'.
    ].
"/    self pageFinish:aRequest exeptStyle:(asShow not and:[self showExeptFrame]).
    response maxAge:5*60.

    numberOfSlides > 1 ifTrue:[
        aRequest replyPresenter secondHeadLine:(aRequest fullPath 
                                       , (' (%1 of %2)' 
                                            bindWith:slideIndex 
                                            with:numberOfSlides)).
    ].

    "Modified: / 02-02-2007 / 13:47:24 / cg"
!

isDirectoryVisible:aDirectory
    (#('thumbs' '_hidden') includes:aDirectory baseName) ifTrue:[ ^ false ].
    (aDirectory baseName startsWith:'.') ifTrue:[ ^ false ].
    ^ true.
!

linkToThumbFor:aRealativeImageFilename
    "generate a hyperlink to a thumb"

    |baseName|

    baseName := aRealativeImageFilename baseName.
    ^ (aRealativeImageFilename directory construct:'thumbs') constructString:baseName
!

pageFinish:aRequest 
    aRequest reply:
'<p>
<font size=1>
Page generated by ',self class nameWithoutPrefix,' Vsn ',self class revision printString,'
</font>
<p>'.

"/    showExeptFrame ifTrue:[
"/        aRequest stxPageFinish.
"/    ] ifFalse:[
"/        aRequest pageFinish
"/    ].
!

slidePage:aRequest forRelativePath:relativePathOfSlideDirectory
    "after validating the request, generate a page containing the tables"

    |slideDirectory slidesRoot path headLine mimeType replyPresenter|

    slideDirectory := self slideDirectory.
    slideDirectory isNil ifTrue:[
        aRequest reportServiceUnavailable:'Slide-Directory not configured'. 
        ^ self.
    ].
    slideDirectory := slideDirectory asFilename.
    slideDirectory exists ifFalse:[
        OsError catch:[
            slideDirectory recursiveMakeDirectory.
        ].
        slideDirectory exists ifFalse:[
            aRequest reportServiceUnavailable:'Slide-Directory does not exist and could not be created'. 
            ^ self.
        ].
    ].
    slideDirectory isDirectory ifFalse:[
        aRequest reportServiceUnavailable:'Slide-Directory is not a directory'. 
        ^ self.
    ].

    slidesRoot := slideDirectory asAbsoluteFilename.
    path := (slidesRoot construct:relativePathOfSlideDirectory) asAbsoluteFilename.
    (slidesRoot = path
    or:[path directory = slidesRoot
    or:[slidesRoot isParentDirectoryOf:path]]) ifFalse:[
        aRequest reportForbidden:'Unauthorized access'. 
        ^ self.
    ].

    path directory baseName = 'thumbs' ifTrue:[
        self thumb:path for:aRequest.
        ^ self
    ].
    path suffix = 'html' ifTrue:[
        mimeType := path withoutSuffix mimeTypeFromName.
        (mimeType notNil and:[mimeType isImage]) ifTrue:[
            self imagePageBody:aRequest for:path relative:relativePathOfSlideDirectory.
            ^ self
        ].
        path baseName = 'show.html' ifTrue:[
            self imagePageBody:aRequest for:path relative:relativePathOfSlideDirectory.
            ^ self
        ].
    ].

    mimeType := path mimeTypeFromName.
    (mimeType notNil and:[mimeType isImage]) ifTrue:[
        self image:path for:aRequest.
        ^ self
    ].
    (mimeType notNil and:[mimeType isArchive]) ifTrue:[
        self file:path for:aRequest.
        ^ self
    ].

    headLine := aRequest fullPath.

    replyPresenter := aRequest replyPresenter.
    replyPresenter title:headLine.
    replyPresenter secondHeadLine:headLine.

    aRequest nextPutLine: '<!!-- generated ' , Timestamp now printString , ' by WebSlideShow -->'.
    self slidePageBody:aRequest for:path asFilename relative:relativePathOfSlideDirectory.

    "Modified: / 02-02-2007 / 13:39:49 / cg"
!

slidePageBody:aRequest for:path relative:relativePath
    "the body of a slide page"

    |backRef parent imgWidth response|

    response := aRequest response.

    (path exists and:[path isDirectory]) ifTrue:[
        path pathName ~= self slideDirectory asFilename pathName ifTrue:[
            relativePath = '' ifFalse:[
                backRef := aRequest serviceLinkName.
                (UnixFilename named:relativePath) components size == 1 ifTrue:[
                    parent := ''
                ] ifFalse:[
                    parent := (UnixFilename named:relativePath) directoryName.
                ].
                parent notEmpty ifTrue:[
                    (backRef endsWith:'/') ifFalse:[
                        backRef := backRef , '/'  
                    ].
                    backRef := backRef , parent.
                ].
                response nextPutLine:'<a href="' , backRef , '">Back to Parent Directory</a><p>'.
            ]
        ].

        self slidePageTables:aRequest for:path relative:relativePath.

        self withImageWidthOption ifTrue:[
            imgWidth := aRequest urlParameterAt:'imageWidth'.
            imgWidth notNil ifTrue:[
                imgWidth := Number readFrom:imgWidth onError:nil.
            ].

            relativePath = '' ifTrue:[ 
                response nextPutLine:'<form action="',aRequest serviceLinkName,'">'.
                response nextPutLine:'Individual Images'' width (empty for original size): <input name="imageWidth" value="',(imgWidth ? 640)printString,'">'.
                response nextPutLine:'             <input type="submit" value="OK">'.
                response nextPutLine:'</form>'.
            ].
        ].
        self allowUploads ifTrue:[
            self upLoadInputFor:aRequest relative:relativePath.
        ].
    ].

"/    self pageFinish:aRequest exeptStyle:(self showExeptFrame).
    self pageFinish:aRequest.
    response maxAge:5*60.
!

slidePageTables:aRequest for:path relative:relativePath
    "generates the two tables"

    self generateTableForSlidesIn:path relative:relativePath for:aRequest.
    self generateTableForSubdirectoriesIn:path relative:relativePath for:aRequest.
    self generateZipLinksForFilesIn:path relative:relativePath for:aRequest.
!

thumb:absolutePathToThumb for:aRequest
    "look for or create and cache a thumb image.
     When invoked for the first time, generate a dithered 8-bit version."

    |thumbDir imgName imgPath image thumbImage thumbStream|

    absolutePathToThumb exists ifFalse:[
        self 
            criticalDirectoryAccessIn:absolutePathToThumb
            do:[
                absolutePathToThumb exists ifFalse:[
                    thumbDir := absolutePathToThumb directory.
                    thumbDir exists ifFalse:[
                        thumbDir makeDirectory.
                        thumbDir exists ifFalse:[
                            ^ aRequest reportNotFound:'Missing thumb directory (cannot create).'. 
                        ].
                    ].

                    imgName := absolutePathToThumb baseName copyFrom:'tn_' size+1.
                    imgPath := thumbDir directory construct:imgName.
                    imgPath exists ifFalse:[
                        ^ aRequest reportNotFound:'No such image (recently removed).'. 
                    ].
                    image := Image fromFile:imgPath.
                    image isNil ifTrue:[
                        ^ aRequest reportServiceUnavailable:'Cannot read original image.'. 
                    ].
                    thumbImage := self generateThumbFor:image.
                    "/ Transcript show:absolutePathToThumb asFilename pathName; showCR:' created.'.
                    Image fileCreationErrorSignal handle:[:ex |
                        aRequest logError:'Cannot save thumb (check permissions of thumb-directory)'.
                        thumbStream := WriteStream on:ByteArray new.
                        PNGReader "GIFReader" save:thumbImage onStream:thumbStream.
                        thumbStream := thumbStream contents readStream.
                        ex return.
                    ] do:[
                        PNGReader "GIFReader" save:thumbImage onFile:absolutePathToThumb.
                    ]
                ].
            ].
    ].

    aRequest response maxAge:60*60*24.
    thumbStream isNil ifTrue:[
        thumbStream := absolutePathToThumb readStream binary.
    ].
    aRequest sendBinaryStream:thumbStream type:'image/gif'.

    "Modified: / 08-05-2012 / 17:44:29 / cg"
!

upLoadImage:aRequest forRelativePath:relPath   
    |fileNameChunk uploadDirectory filename|

    self authenticateUploadFor:aRequest.

    fileNameChunk := aRequest multiPartChunks at:#'fileName' ifAbsent:nil.
    (fileNameChunk notNil and:[fileNameChunk isFilenameChunk]) ifTrue:[
        filename := fileNameChunk filenameBaseName.
        filename notNil ifTrue:[
            uploadDirectory := self slideDirectory asFilename construct:relPath.
            fileNameChunk writeFileIntoDirectory:uploadDirectory.
        ].
    ].
!

upLoadInputFor:aRequest relative:relativePath

    aRequest nextPutAll:'<p>
<table>
<tr>'.
    aRequest nextPutAll:'<form action="',aRequest fullPath,'">
<td>
    Create new Image Set:
</td>
<td>'.
    aRequest nextPutAll:(aRequest inputFieldFor:'imageSetName' size:50 label:'' initialValue:'').
    aRequest nextPutAll:'
</td>
<td>'.
    aRequest formSubmitButtonForString:'Create' name:'CreateNewImageSet'.
    aRequest nextPutAll:'
</td>
</form></tr>
<tr>
<form action="', aRequest fullPath,'" method="post" enctype="multipart/form-data">
<td>
    Upload new Image:
</td>
<td>'.
    aRequest nextPutLine:(aRequest inputFieldFor:'fileName' type:'"file"' size:50 label:'' initialValue:'').
aRequest nextPutLine:'</td><td>'.
    aRequest formSubmitButtonForString:'Upload' name:'Upload';
        nextPutAll:'
</td>
</form></tr>
</table>'.

    "Modified: / 02-02-2007 / 00:59:50 / cg"
! !

!WebSlideShow methodsFor:'serving-thumbs'!

generateButtonImageFor:anImage buttonLevel:bw
    |brighter darker buttonImage w h smallImage|

    smallImage := anImage magnifiedPreservingRatioTo:(self thumbSize).

    w := smallImage width.
    h := smallImage height.
    buttonImage := Depth24Image fromImage:smallImage.

    brighter := [:img :pix :x :y | 
                    |r g b|

                    r := (pix bitShift:-16) bitAnd:16rFF.
                    g := (pix bitShift:-8) bitAnd:16rFF.
                    b := pix bitAnd:16rFF.
                    r := r + ((255 - r) // 2).
                    g := g + ((255 - g) // 2).
                    b := b + ((255 - b) // 2).
                    (((r bitShift:8) bitOr:g) bitShift:8) bitOr:b
                ].
    darker := [:img :pix :x :y | 
                    |r g b|

                    r := (pix bitShift:-16) bitAnd:16rFF.
                    g := (pix bitShift:-8) bitAnd:16rFF.
                    b := pix bitAnd:16rFF.
                    r := r // 2.
                    g := g // 2.
                    b := b // 2.
                    (((r bitShift:8) bitOr:g) bitShift:8) bitOr:b
                ].

    0 to:bw do:[:d|
        buttonImage 
            applyPixelValuesTo:brighter
            in:((0+d)@d corner:(w-d)@(d+1)) 
            into:buttonImage.

        buttonImage 
            applyPixelValuesTo:brighter
            in:(d@(0+d) corner:(d+1)@(h-d)) 
            into:buttonImage.

        buttonImage 
            applyPixelValuesTo:darker
            in:((w-d-1)@d corner:(w-d)@(h-d)) 
            into:buttonImage.

        buttonImage 
            applyPixelValuesTo:darker
            in:(d@(h-d-1) corner:(w-d)@(h-d)) 
            into:buttonImage.
    ].

    ^ buttonImage.
!

generateCartoonImageFor:anImage
    |smallImage cartoonImage ditherColors cartoonColors|

    smallImage := anImage magnifiedPreservingRatioTo:(self thumbSize).
    smallImage depth > 8 ifTrue:[
        "/ must convert - GIF only supports Depth8.
        ditherColors := Color standardDitherColorsForDepth8.
        smallImage := smallImage asDitheredImageUsing:ditherColors depth:8.
    ].

    cartoonColors := Color colorCubeWithRed:4 green:4 blue:2.
    cartoonImage := smallImage asNearestPaintImageDepth:8 colors:cartoonColors.
    ^ cartoonImage
!

generateGrayImageFor:anImage
    |thumbImage|

    thumbImage := anImage magnifiedPreservingRatioTo:(self thumbSize).
    ^ thumbImage asGrayImageDepth:8
!

generateShadowImageFor:anImage shadowWidth:sw
    |img gray w h smallImage shadowImage|

    smallImage := anImage magnifiedPreservingRatioTo:(self thumbSize - sw).

    img := Depth24Image fromImage:smallImage.
    w := smallImage width.
    h := smallImage height.
    shadowImage := Depth24Image extent:((w+sw) @ (h+sw)).
    shadowImage createPixelStore.
    shadowImage bits atAllPut:16rFF.
    shadowImage copyFrom:img x:0 y:0 toX:0 y:0 width:w height:h.

    w := shadowImage width.
    h := shadowImage height.

    gray := [:img :pix :x :y | 
                    |r g b|

                    r := g := b := 16r8F.
                    (((r bitShift:8) bitOr:g) bitShift:8) bitOr:b
                ].

    shadowImage
        applyPixelValuesTo:gray
        in:((w-sw)@sw corner:(w)@(h)) 
        into:shadowImage.

    shadowImage
        applyPixelValuesTo:gray
        in:(sw@(h-sw) corner:(w)@(h)) 
        into:shadowImage.

    ^ shadowImage.
!

generateSlideImageFor:anImage
    |w h slw slh smallImage slideImage ditherColors x yT yB xL xR y blackPixel whitePixel|

    smallImage := anImage magnifiedPreservingRatioTo:(self thumbSize - 16).
    w := smallImage width.
    h := smallImage height.

    "/ must convert - GIF only supports Depth8.
    ditherColors := Color standardDitherColorsForDepth8.
    smallImage := smallImage asDitheredImageUsing:ditherColors depth:8.

    slw := slh := (self thumbSize x) max:(self thumbSize y).

    slideImage := Depth8Image extent:(slw @ slh).
    slideImage colorMap:(smallImage colorMap).
    slideImage createPixelStore.
    slideImage bits atAllPut:(smallImage colorMap indexOf:(Color rgbValue:16rBFBFBF))-1.
    slideImage copyFrom:smallImage x:0 y:0 toX:(slw - w // 2) y:(slh - h // 2) width:w height:h.

    whitePixel := (smallImage colorMap indexOf:(Color white)) - 1.
    blackPixel := (smallImage colorMap indexOf:(Color black)) - 1.

    "/ hLine
    x := (slw - w // 2).
    yT := (slh - h // 2).
    yB := (slh - h // 2) + h.
    0 to:w-1 do:[:o |
        slideImage pixelAtX:x+o y:yT put:blackPixel.
        slideImage pixelAtX:x+o y:yT+1 put:blackPixel.
        slideImage pixelAtX:x+o y:yB put:whitePixel.
        slideImage pixelAtX:x+o y:yB-1 put:whitePixel.
    ].
    "/ vLine
    xL := (slw - w // 2).
    xR := (slw - w // 2) + w.
    y := (slh - h // 2).
    0 to:h-1 do:[:o |
        slideImage pixelAtX:xL y:y+o put:blackPixel.
        slideImage pixelAtX:xL+1 y:y+o put:blackPixel.
        slideImage pixelAtX:xR y:y+o put:whitePixel.
        slideImage pixelAtX:xR-1 y:y+o put:whitePixel.
    ].

    0 to:slw-1 do:[:x |
        slideImage pixelAtX:x y:0 put:whitePixel.
        slideImage pixelAtX:x y:slh-1 put:blackPixel.
    ].
    0 to:slh-1 do:[:y |
        slideImage pixelAtX:0 y:y put:whitePixel.
        slideImage pixelAtX:slw-1 y:y put:blackPixel.
    ].

    ^ slideImage.
!

generateThumbFor:anImage
    |thumbImage ditherColors|

    anImage isNil ifTrue:[^ nil].

    thumbStyle == #shadow ifTrue:[
        thumbImage := self generateShadowImageFor:anImage shadowWidth:5.
    ] ifFalse:[ thumbStyle == #grayScale ifTrue:[
        thumbImage := self generateGrayImageFor:anImage.
    ] ifFalse:[ thumbStyle == #button ifTrue:[
        thumbImage := self generateButtonImageFor:anImage buttonLevel:4.
    ] ifFalse:[ thumbStyle == #cartoon ifTrue:[
        thumbImage := self generateCartoonImageFor:anImage.
    ] ifFalse:[ thumbStyle == #slide ifTrue:[
        thumbImage := self generateSlideImageFor:anImage.
    ] ifFalse:[
        thumbImage := anImage magnifiedPreservingRatioTo:(self thumbSize).
    ]]]]].

    thumbImage depth > 8 ifTrue:[                           
        "/ must convert - GIF only supports Depth8.
        ditherColors := Color standardDitherColorsForDepth8.
        thumbImage := thumbImage asDitheredImageUsing:ditherColors depth:8.
    ].
    ^ thumbImage
! !

!WebSlideShow methodsFor:'settings persistency'!

settingSelectors
    "answer the selectors that are used to configure a service - i.e. those which are
     saved in the webServer-settings file."

    "/ cg: changed to use a positive list, instead of an exclusion algorithm.
    "/ because it makes always trouble when an instvar is added.

    ^ super settingSelectors ,
      #(slideDirectory numberOfColumns numberOfSlidesPerPage thumbSize thumbBorders thumbStyle
        showFileNames showFileSizes allowSlideShow withImageWidthOption allowUploads
        uploadRealm).

    "
     HTTPService new settingSelectors
     self new settingSelectors
    "
! !

!WebSlideShow::CachedDirectoryEntry methodsFor:'accessing'!

files
    ^ files
!

modificationTime
    ^ modificationTime
!

modificationTime:modificationTimeArg files:filesArg 
    modificationTime := modificationTimeArg.
    files := filesArg.
! !

!WebSlideShow class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebSlideShow.st,v 1.103 2012/05/08 16:24:32 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/WebSlideShow.st,v 1.103 2012/05/08 16:24:32 cg Exp $'
! !
