"{ Package: 'stx:goodies/webServer/comanche' }"

ComancheService subclass:#ComancheNetService
	instanceVariableNames:'protocolAdaptor'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Mimicry'
!

!ComancheNetService class methodsFor:'documentation'!

documentation
"
    Kom48 (Comanche beta 4.8)
    bolot 4/2/2001 18:44

    Comment from kom46:
        I'm a service that sets up a ComancheServer to listen on a given port.  
        I use a protocol adaptor to adapts TCP/IP based protocol conversations into message based conversations.


    protocolAdaptor     an object that supports the NetworkProtocolAdaptor class interface 
                        (specifically, I send it #processSocket:target:).

    comancheServer      an instance of a ComancheServer that handles all the details of setting up to listen for sockets 
                        on a given port.
"
! !

!ComancheNetService class methodsFor:'instance creation'!

named: aString onPort: portNumber
	^self named: aString onPort: portNumber priority: self defaultPriority
!

named: aString onPort: portNumber priority: priorityNum
	^self 
		named: aString 
		onPort: portNumber 
		priority: priorityNum 
		protocolAdaptor: self defaultProtocolAdaptor
!

named: aString onPort: portNumber priority: priorityNum protocolAdaptor: aPA
	^(super named: aString) 
		initializeOnPort: portNumber priority: priorityNum protocolAdaptor: aPA
! !

!ComancheNetService class methodsFor:'defaults'!

defaultProtocolAdaptor
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^ HttpAdaptor
    ]
!

defaultServerClass
    Smalltalk isSmalltalkX ifTrue:[
        ^ HTTPServer
    ] ifFalse:[
        ^ ComancheServer
    ]
! !

!ComancheNetService class methodsFor:'queries'!

isAbstract
    self == ComancheNetService ifTrue:[^ true].
    ^ super isAbstract
! !

!ComancheNetService methodsFor:'accessing'!

comancheServer
    ^ httpServer
!

module
	"Answer a ComancheModule that will process network
	requests for this ComancheNetService"

	^module
!

module:aKOMModule
    module := aKOMModule
!

portNumber
        httpServer notNil ifTrue: [^httpServer port].
        ^nil
!

priority

	^self comancheServer priority
!

protocolAdaptor
	"Answer the NetworkProtocolAdaptor that is being used to
	translate Socket based conversations into message based ones."
	"HttpAdaptor is the default protocol adaptor."

	^protocolAdaptor
! !

!ComancheNetService methodsFor:'conversation'!

serve: aSocket
	"Provided for TCP/IP conversations.  Works in conjunction with 
	a ComancheServer.  Services that do not serve TCP/IP connections
	do not need to implement this method."

	self protocolAdaptor 
		readAndWriteTo: (SocketStream on: aSocket)
		target: self
! !

!ComancheNetService methodsFor:'defaults'!

defaultProtocolAdaptor

	^self class defaultProtocolAdaptor
!

defaultServerClass
	^self class defaultServerClass
! !

!ComancheNetService methodsFor:'initialization'!

initializeOnPort:portNum priority:priorityLevel protocolAdaptor:aPA 
    ^ self
        initializeOnPort:portNum priority:priorityLevel protocolAdaptor:aPA serverClass:self defaultServerClass
!

initializeOnPort:portNum priority:priorityLevel protocolAdaptor:aPA serverClass:serverClass 
    Smalltalk isSmalltalkX ifTrue:[
        httpServer := serverClass serverOnPort:portNum.
    ] ifFalse:[
        httpServer := serverClass 
                onPort:portNum
                priority:priorityLevel
                serviceProvider:self.
    ].
    protocolAdaptor := aPA
! !

!ComancheNetService methodsFor:'mimicry'!

setupModule
    ^ self
! !

!ComancheNetService methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' port: ', self portNumber asString
! !

!ComancheNetService methodsFor:'starting and stopping'!

start
    |comanche|

    (comanche := self comancheServer) notNil ifTrue:[ 
        comanche start 
    ].
    Smalltalk isSmalltalkX ifTrue:[
        httpServer notNil ifTrue:[
            self registerServiceOn:httpServer.
        ].
    ].
!

stop
    self comancheServer notNil ifTrue:[ 
        self comancheServer stop 
    ].
    httpServer := nil.
    module := nil.
    protocolAdaptor := nil
! !

!ComancheNetService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/ComancheNetService.st,v 1.8 2010/03/18 16:36:35 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/ComancheNetService.st,v 1.8 2010/03/18 16:36:35 cg Exp $'
! !
