"{ Package: 'stx:goodies/webServer/comanche' }"

"{ NameSpace: Comanche }"

Object subclass:#Compiler
	instanceVariableNames:'realCompiler smalltalkDialect'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-STX-Support'
!

!Compiler class methodsFor:'documentation'!

documentation
"
    some mimicry to allow for Squeak syntax code to be loaded transparently
    (as present in Wiki pages - and we do not want to edit/maintain them all)
"
! !

!Compiler class methodsFor:'initialization'!

initialize
    (Object implements:#caseError) ifFalse:[
        Smalltalk fileIn:'extension.st' inPackage:self package
    ]
! !

!Compiler class methodsFor:'evaluation'!

evaluate:aString
    ^ self 
        evaluate:aString 
        notifying:nil
        logged:false
!

evaluate:aString notifying:requestor logged:logged
    |parser rslt|

    Class nameSpaceQuerySignal answer:self nameSpace
    do:[
        parser := (Smalltalk at:#Compiler) new.
        "/ Squeak syntax
        parser allowSqueakExtensions:true.
        parser allowUnderscoreInIdentifier:false.
        parser allowOldStyleAssignment:true.

        rslt := parser 
            evaluate:aString 
            in:nil 
            receiver:nil 
            notifying:requestor 
            logged:logged
            ifFail:nil 
            compile:true.
    ].

    rslt == #Error ifTrue:[^ nil].

    rslt isBlock ifTrue:[
        rslt := SimulatedSqueakBlock new realBlock:rslt.
        rslt source:(aString replaceAll:(Character return) with:Character cr)
    ].
    ^ rslt
! !

!Compiler methodsFor:'accessing'!

smalltalkDialect:something
    smalltalkDialect := something.
! !

!Compiler methodsFor:'compilation'!

compileNoPattern:smalltalkCode in:aClass context:context notifying:requestor ifFail:failBlock
    |compiler mthd|

    Class withoutUpdatingChangesDo:[
        compiler := (Smalltalk at:#Compiler) new.

        smalltalkDialect == #stx ifTrue:[
            "/ ST/X syntax
            compiler allowSqueakExtensions:true.
            compiler allowUnderscoreInIdentifier:true.
        ].
        smalltalkDialect == #squeak ifTrue:[
            "/ Squeak syntax
            compiler allowSqueakExtensions:true.
            compiler allowUnderscoreInIdentifier:false.
            compiler allowOldStyleAssignment:true.
        ].

        mthd := compiler
                    compile:('xXxXxXxX ' , smalltalkCode) 
                    forClass:aClass 
                    inCategory:'others'
                    notifying:requestor 
                    install:false
                    skipIfSame:false
                    silent:false
                    foldConstants:true
                    ifFail:failBlock.
        aClass removeSelector:#xXxXxXxX.
    ].
    mthd isNil ifTrue:[
        ^ mthd
    ].

    ^ SimulatedSqueakMethod new realMethod:mthd
! !

!Compiler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/Compiler.st,v 1.23 2006/10/11 15:23:46 cg Exp $'
! !

Compiler initialize!
