"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#ChieftainModule
	instanceVariableNames:'urlMap urlPrefixMap'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-Kishi'
!

ChieftainModule comment:'ChieftainModule
bolot 6/21/2001 03:08

- decides which app/module/obj handles a given URL
- urlMap is for mapping a single URL to a single object
-- e.g., cached objects
- urlPrefixMap is for mapping a URL prefix to an application

----
- todo: cached objects
-- timeout
-- duplication (e.g., what if registering same object again?)
- todo: urlPrefix mapping
-- fast determination which module needs to run
- todo: session management
- todo: form processing'
!


!ChieftainModule class methodsFor:'examples'!

basicExample
	"ChieftainModule basicExample"

	| service chief |
	chief _ self new initialize.
	chief
		mapPrefix: '/displaycam' to: DisplayCamModule new;
		mapPrefix: '/helloworld' to: HelloWorldModule new;
		mapPrefix: '/turtlegraphics' to:
				(TurtleGraphicsModule new initialize urlPrefix: '/turtlegraphics');
		mapPrefix: '/turtleframe' to:
				(TurtleFrameGraphicsModule new initialize urlPrefix: '/turtleframe').

	service := ComancheNetService named: 'basicExample' onPort: 8017.
	service module: chief.
	service start
! !

!ChieftainModule methodsFor:'accessing'!

cacheRootUrl
	^'/kom/cache/'
!

clearObjectCache
	urlMap _ Dictionary new
!

generateUniqueUrlForObject: anObject
	"prepend a special root URL"
	^self cacheRootUrl, anObject hash asString, '/', Time totalSeconds asString
!

initialize
	urlMap _ Dictionary new.
	urlPrefixMap _ Dictionary new
!

mapObject: anObject
	"generate a random, unique URL, add the object there"
	urlMap at: (self generateUniqueUrlForObject: anObject) put: anObject
!

mapPrefix: aString to: aModule
	"make a given prefix map to the module"
	urlPrefixMap at: aString put: aModule
!

process: request
	"first, check if any cached objects are referenced"
	(urlMap includesKey: request url)
		ifTrue: [^urlMap at: request url].

	"then check if any application's prefix matches"
	urlPrefixMap keysAndValuesDo: [:urlPrefix :module |
		(request url beginsWith: urlPrefix) ifTrue: [^module process: request]].

	"otherwise, the default response"
	^nil
! !

!ChieftainModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/ChieftainModule.st,v 1.1 2003/05/19 19:09:12 cg Exp $'
! !
