"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

ComancheService subclass:#ComancheNetService
	instanceVariableNames:'comancheServer protocolAdaptor module'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-kernel'
!

ComancheNetService comment:'Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:44

ComancheNetService

Comment from kom46:
I''m a service that sets up a ComancheServer to listen on a given port.  I use a protocol adaptor to adapts TCP/IP based protocol conversations into message based conversations.

protocolAdaptor - an object that supports the NetworkProtocolAdaptor class interface (specifically, I send it #processSocket:target:).
comancheServer - an instance of a ComancheServer that handles all the details of setting up to listen for sockets on a given port.
'
!


!ComancheNetService class methodsFor:'instance creation'!

named: aString onPort: portNumber
	^self named: aString onPort: portNumber priority: self defaultPriority
!

named: aString onPort: portNumber priority: priorityNum
	^self 
		named: aString 
		onPort: portNumber 
		priority: priorityNum 
		protocolAdaptor: self defaultProtocolAdaptor
!

named: aString onPort: portNumber priority: priorityNum protocolAdaptor: aPA
	^(super named: aString) 
		initializeOnPort: portNumber priority: priorityNum protocolAdaptor: aPA
! !

!ComancheNetService class methodsFor:'defaults'!

defaultProtocolAdaptor

	^HttpAdaptor
!

defaultServerClass
	^ComancheServer
! !

!ComancheNetService methodsFor:'accessing'!

comancheServer
	^comancheServer
!

module
	"Answer a ComancheModule that will process network
	requests for this ComancheNetService"

	^module
!

module: aComancheModule

	module _ aComancheModule
!

portNumber
	self comancheServer
		ifNotNil: [^self comancheServer portNumber]
		ifNil: [^nil]
!

priority

	^self comancheServer priority
!

protocolAdaptor
	"Answer the NetworkProtocolAdaptor that is being used to
	translate Socket based conversations into message based ones."
	"HttpAdaptor is the default protocol adaptor."

	^protocolAdaptor
! !

!ComancheNetService methodsFor:'conversation'!

processNetworkRequest: aNetworkRequest
	^self module process: aNetworkRequest
!

serve: aSocket
	"Provided for TCP/IP conversations.  Works in conjunction with 
	a ComancheServer.  Services that do not serve TCP/IP connections
	do not need to implement this method."

	self protocolAdaptor 
		readAndWriteTo: (SocketStream on: aSocket)
		target: self
! !

!ComancheNetService methodsFor:'defaults'!

defaultProtocolAdaptor

	^self class defaultProtocolAdaptor
!

defaultServerClass
	^self class defaultServerClass
! !

!ComancheNetService methodsFor:'initialization'!

initializeOnPort: portNum priority: priorityLevel protocolAdaptor: aPA
	comancheServer _ self defaultServerClass 
		onPort: portNum
		priority: priorityLevel
		serviceProvider: self.

	protocolAdaptor _ aPA
!

initializeOnPort: portNum priority: priorityLevel protocolAdaptor: aPA serverClass: serverClass
	comancheServer _ serverClass 
		onPort: portNum
		priority: priorityLevel
		serviceProvider: self.

	protocolAdaptor _ aPA
! !

!ComancheNetService methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' port: ', self portNumber asString
! !

!ComancheNetService methodsFor:'queries'!

isRunning
	self comancheServer ifNotNil: [ 
		^self comancheServer isRunning
	].
	^false
! !

!ComancheNetService methodsFor:'starting and stopping'!

start
	self comancheServer ifNotNil: [
		self comancheServer start
	]
!

stop
	self comancheServer ifNotNil: [
		self comancheServer stop
	].
	comancheServer _ nil.
	module _ nil.
	protocolAdaptor _ nil
! !

!ComancheNetService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/ComancheNetService.st,v 1.1 2003/05/19 19:24:45 cg Exp $'
! !
