"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#ComancheService
	instanceVariableNames:'autoStart log'
	classVariableNames:'Services'
	poolDictionaries:''
	category:'Kom-kernel'
!

ComancheService comment:'Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:43

ComancheService

Comment from kom46:
I am an abstract superclass.  My subclasses are services that can be established on any port.
'
!


!ComancheService class methodsFor:'initialization'!

initialize
	" ComancheService initialize  "

	Services _ Dictionary new.
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self
! !

!ComancheService class methodsFor:'instance creation'!

named: aString
	^self new registerAs: aString
!

new
	^super new initialize
! !

!ComancheService class methodsFor:'accessing'!

defaultPriority
	^Processor highIOPriority
!

defaultSearchPriority
	^self defaultPriority - 1
! !

!ComancheService class methodsFor:'registration'!

addService: aComancheService named: aString
	| tmp |
	tmp _ self services.
	(tmp includesKey: aString)
		ifTrue: [ ^self error: 'A service by the same name already exists' ].
	tmp at: aString put: aComancheService
!

nameForService: aComancheService
	^self services 
		keyAtIdentityValue: aComancheService 
		ifAbsent: [ '(unregisterd service)' ]
!

removeService: aComancheService
	| tmp |
	tmp _ self services.
	tmp removeKey: 
		(tmp keyAtIdentityValue: aComancheService ifAbsent: [ ^self ])
!

removeServiceNamed: aString
	self services removeKey: aString ifAbsent: []
!

replaceServiceNamed: aString with: aComancheService
	self services at: aString put: aComancheService
!

serviceNamed: aString
	^self services at: aString ifAbsent: [nil]
!

services
	^Services
! !

!ComancheService class methodsFor:'snapshots'!

shutDown
	self services do: [ :ea |
		ea prepareForShutdown
	]
!

startUp
	self services do: [ :ea |
		ea prepareForStartup
	].
	self services do: [ :ea |
		ea autoStart ifTrue: [ ea start ]
	]
! !

!ComancheService methodsFor:'accessing'!

autoStart
	^autoStart == true
!

autoStart: aBoolean
	autoStart _ aBoolean
!

log
	^log
!

log: aLog
	log _ aLog
!

name
	^self class nameForService: self
! !

!ComancheService methodsFor:'initialize-release'!

initialize
	autoStart _ false
! !

!ComancheService methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: ('Service: ', self name).
	self isRunning ifTrue: [ 
		aStream nextPutAll: '[running]'
	] ifFalse: [
		aStream nextPutAll: '[stopped]'
	]
! !

!ComancheService methodsFor:'queries'!

isRunning
	self subclassResponsibility
! !

!ComancheService methodsFor:'registration'!

registerAs: aString
	self unregister.
	self class addService: self named: aString
!

unregister
	self stop.
	self class removeService: self
! !

!ComancheService methodsFor:'starting and stopping'!

start
	"Subclasses should override this method to take whatever
	is necessary to start this service"

	"self subclassResponsibility"
!

stop
	"Subclasses should override this method to take whatever
	is necessary to start this service"

	"self subclassResponsibility"
! !

!ComancheService methodsFor:'startup and shutdown'!

prepareForShutdown
	"The squeak system is about to shutdown"
!

prepareForStartup
	"The squeak system just started up"
! !

!ComancheService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/ComancheService.st,v 1.1 2003/05/19 19:24:29 cg Exp $'
! !

ComancheService initialize!
