"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#ConnectionHandler
	instanceVariableNames:'portNumber socketClass process priority handlerBlock lastError
		socketsToDestroy'
	classVariableNames:'Registry'
	poolDictionaries:''
	category:'Kom-kernel'
!

ConnectionHandler comment:'Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:44

ConnectionHandler

Comment from kom46:
- also see ConnectionQueue
- does not keep the queue of open sockets
- launches the applications/modules immediately
  upon accepting a connection'
!


!ConnectionHandler class methodsFor:'instance creation'!

onPort: aNumber handler: aBlock priority: pNum
	^self new
		initializeOnPort: aNumber
		handler: aBlock
		priority: pNum
! !

!ConnectionHandler class methodsFor:'registry'!

register: anObject

	self registry add: anObject
!

registry

	Registry == nil ifTrue: [
		Registry := WeakRegistry new
	].
	^Registry
!

unregister: anObject

	self registry remove: anObject ifAbsent: [ ]
! !

!ConnectionHandler methodsFor:'accessing'!

portNumber
	^portNumber
!

priority
	^priority
!

process
	process == nil ifTrue: [
		self initializeProcessAtPriority: self priority.
	].
	^process
!

socketsToDestroy

	socketsToDestroy == nil ifTrue: [
		socketsToDestroy _ OrderedCollection new.
	].
	^socketsToDestroy
! !

!ConnectionHandler methodsFor:'initialize-release'!

destroySockets
	socketsToDestroy ~~ nil ifTrue: [ 
		socketsToDestroy do: [ :ea | ea destroy ].
	].
	socketsToDestroy _ OrderedCollection new
!

finalize

	"Like #stop, but without the #unregister"

	process ~~ nil ifTrue: [
		process terminate.
	].
	process _ nil.
	self destroySockets.


!

initializeOnPort: aNumber handler: aBlock priority: pNumber
	portNumber _ aNumber.
	handlerBlock _ aBlock.
	priority _ pNumber.
	socketsToDestroy _ OrderedCollection new
!

initializeProcessAtPriority: pNumber
	process _ [ self pvtListenLoop ] newProcess.
	process priority: pNumber
! !

!ConnectionHandler methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: (self class name asString, ' port: ', self portNumber asString).
	self isRunning ifTrue: [
		aStream nextPutAll: ' [running]'
	] ifFalse:[
		aStream nextPutAll: ' [stopped]'
	]
! !

!ConnectionHandler methodsFor:'private'!

newListener
	| listener |

	self flag: #todo. "listener _ self socketClass newTCP."
	listener _ Socket newTCP.

	"backlog is how many simultaneous connections to accept at the same time"
	listener listenOn: portNumber backlogSize: Kom5Preferences backlogSize.
	^listener
!

pvtListenLoop
	[
		self pvtNewListenLoop
	] on: Error do: [ :ex |
		process _ nil.
		self destroySockets.
	]
!

pvtNewListenLoop
	| socket listener |

	listener _ self newListener.
	self socketsToDestroy add: listener.	

	"If the listener is not valid then the we cannot use the
	BSD style accept() mechanism."
	listener isValid ifFalse: [^self pvtOldListenLoop].
	[ true ] whileTrue: [
		socket _ listener waitForAcceptUntil: Kom5Preferences acceptTimeoutDeadline.
		(socket notNil and: [ socket isConnected ])
			ifTrue: [
				handlerBlock value: socket.
				socket _ nil]
			ifFalse: [
				socket notNil ifTrue: [socket destroy].
				listener isValid
					ifFalse: [(Delay forMilliseconds: Kom5Preferences listenerDestroyDelay) wait.
						listener destroy.
						listener _ self newListener].
			].
	]
!

pvtOldListenLoop
	"see comment on ConnectionQueue>>oldStyleListenLoop"
	| socket |
	socket _ nil.

	[ true ] whileTrue: [
		(socket == nil) ifTrue: [
			"try to create a new socket for listening"
			socket _ Socket createIfFail: [ nil ]].

		(socket == nil)
		ifTrue: [(Delay forMilliseconds: 100) wait]
		ifFalse: [
			socket isUnconnected ifTrue: [ socket listenOn: portNumber ].
			socket waitForConnectionUntil: (Socket deadlineSecs: 10).
			socket isConnected
				ifTrue: ["connection established"
					"mutex critical: [" handlerBlock value: socket "]".
					socket _ nil]
				ifFalse: [
					socket isWaitingForConnection
						ifFalse: [socket destroy. socket _ nil]].
		].
	]
! !

!ConnectionHandler methodsFor:'queries'!

isRunning

	^process ~~ nil
! !

!ConnectionHandler methodsFor:'registry'!

register

	^self class register: self
!

unregister

	^self class unregister: self
! !

!ConnectionHandler methodsFor:'starting and stopping'!

start
	self stop.
	self process resume.
	self register
!

stop
	self unregister.
	process ~~ nil ifTrue: [
		process terminate.
	].
	process _ nil.

	"We do the following in a timely manner instead of relying on
	finalization because we may want to turn right around and start
	listening again.  If an old socket is still around an listening, it may
	interfere with a new one's ability to hear."

	self destroySockets
! !

!ConnectionHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/ConnectionHandler.st,v 1.1 2003/05/19 19:25:01 cg Exp $'
! !
