"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

NetworkProtocolAdaptor subclass:#HttpAdaptor
	instanceVariableNames:'request response startTime'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-protocol HTTP'
!

HttpAdaptor comment:'kom/4.12 (Comanche/4.12)

HTTPAdaptor (kom/4.8; bolot 4/2/2001 18:48)
- logging is removed from kom46 core
- so is Process>>contextAt:ifAbsent:
- kom48 adds rudimentary persistent connections
-- see keepAliveConnection senders (also in Preferences)

Comment from kom46:
I encapsulate the HTTP protocol from my clients (who must support the HttpPlug interface).  I use an HttpRequest to pick apart a request and I use an HttpResponse to formulate a response.  Mainly, I broker the conversation, turning a request into a message send (to an HttpPlug) and converting the answer to an HttpResponse.  Thus, my clients must simply implement a message based protocol (eliminating the need for them to be concerned with HTTP syntax).'
!


!HttpAdaptor methodsFor:'processing'!

beginConversation
	startTime _ Time millisecondClockValue.
	[
		[(self pvtReadRequest) ifNotNil:
			[self pvtGetResponseAndDo:
				[self pvtWriteResponse.
				self pvtDestroyGently.
				"self pvtWriteLogAndDestroy"]
			].
		self isPersistentConnection and: [Preferences keepAliveConnections]] whileTrue.
		self pvtDestroy
	] on: Error do: [ :ex |
		(Preferences standaloneServer) ifTrue:
			[self pvtDestroy.
			self pvtHandleError: ex]
		ifFalse: "re-raise the exception"
			[ex signal].
	]
!

isPersistentConnection
	^request ifNotNil: [request isPersistentConnection] ifNil: [false]
!

pvtDestroy
	[
		self readStream close.
		self writeStream close.
		response ~~ nil ifTrue: [ response destroy ].
		readStream _ writeStream _ target _ response _ nil.
	]
	ifError: [:err :rcvr | "ignore"]
!

pvtDestroyGently
	"for persistent connections, only destroy response"
	[
		response ~~ nil ifTrue: [ response destroy ].
		response _ nil.
	]
	ifError: [:err :rcvr | "ignore"]
!

pvtGetResponseAndDo: blk
	[
		response _ ((self dispatchRequest: request) asHttpResponseTo: request)
			request: request;
			yourself.
	] on: Error do: [ :ex |
self flag: #todo. "figure out how to handle this error"
"ex signal."
"'error in HttpAdaptor>>pvtGetResponseAndDo:' displayAt: 10@160."

		response _ (ex asHttpResponseTo: request)
			request: request;
			yourself.
		blk value.
		self pvtHandleError: ex.
		^self
	].

	blk value
!

pvtHandleError: ex
	"place holder"
!

pvtLogActivity
	"place holder"
!

pvtReadRequest

	request _ HttpRequest readFromStream: self readStream.
	"Processor activeProcess contextAt: #request put: request"
	^request
!

pvtWriteLogAndDestroy
	self pvtWriteResponse.
	self pvtLogActivity.
	self pvtDestroy
!

pvtWriteResponse
	[
		(self isPersistentConnection and: [Preferences keepAliveConnections])
			ifTrue: [response fieldAt: 'Connection' put: 'close']
			ifFalse: [response fieldAt: 'Connection' put: 'Keep-Alive'].
		response writeOn: self writeStream
	]
	on: Error do: [ :ex | 
		"If we're here, it's likely because something on the other
		end of the socket it hosed.  Do nothing, a log entry will 
		be written and the socket and response will be destroyed"
	]
! !

!HttpAdaptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/HttpAdaptor.st,v 1.1 2003/05/19 19:11:09 cg Exp $'
! !
