"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#HttpSessionModule
	instanceVariableNames:'module sessions mutex lastCleanup secondsToLive'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-kernel'
!

HttpSessionModule comment:'I''m module decorator that add Session support to an existing module

Usage:
Comanche module: (HttpSessionModule module: XXXXModule new)
'
!


!HttpSessionModule class methodsFor:'instance creation'!

module: aModule 
	^ self new initializeModule: aModule
!

new
^ super new initialize
! !

!HttpSessionModule methodsFor:'accessing'!

module
	^module
!

secondsToLive:anInteger
secondsToLive _ anInteger
! !

!HttpSessionModule methodsFor:'comanche'!

process: aRequest 
	"process the request for session handling, then delegate #process: to the 
	decorated module, and send the new cookie to the browser"
	| response newSessionID |
	newSessionID _ self processRequestForSession: aRequest.
	response _ (module process: aRequest)
				asHttpResponseTo: aRequest.
	newSessionID notNil
		ifTrue: [response
				setCookieName: #SessionID
				value: newSessionID
				path: '/'].
	^ response
! !

!HttpSessionModule methodsFor:'initialization'!

initialize
	sessions _ Dictionary new.
	mutex _ Semaphore forMutualExclusion.

	lastCleanup _ Time totalSeconds.
	"30 minutes"
	secondsToLive _ 30 * 60
!

initializeModule:aModule
module _ aModule
! !

!HttpSessionModule methodsFor:'private'!

checkForCleanup
"PRIVATE: run the cleanup between time and time"
	lastCleanup + 60 < Time totalSeconds
		ifTrue: [self cleanup]
!

cleanup
"PRIVATE: iterate over all sessions and removes old ones"
	| totalSeconds deadLine session |
	totalSeconds _ Time totalSeconds.
	lastCleanup _ totalSeconds.
	deadLine _ totalSeconds - secondsToLive.
	sessions keys
		do: [:key | 
			session _ sessions at: key.
			(session usedSince: deadLine)
				ifTrue: [
					sessions removeKey: key.
					(module respondsTo: #sessionTimedOut:) ifTrue:[
						module sessionTimedOut: session ]]]
!

currentSession: aRequest 
	"PRIVATE: answer the current session based on the 'SessionID' cookie"
	| sessionID |
	sessionID _ aRequest cookies
				at: #SessionID
				ifAbsent: [^ nil].
	^ sessions
		at: sessionID
		ifAbsent: []
!

newSession
	"PRIVATE: creates a new Session and stores it"
	| session |
	session _ HttpSession new.
	sessions at: session id put: session.
	^ session
!

processRequestForSession: aRequest 
	"PRIVATE: process aRequest and put a Session (new or old) in aRequest"
	| session newSessionID |
	mutex
		critical: [
			self checkForCleanup.
			session _ self currentSession: aRequest.
			newSessionID _ session isNil
						ifTrue: [
							session _ self newSession.
							session id].
			aRequest session: session.
			session timestamp: Time totalSeconds]
		ifError: [:msg :rcvr | rcvr error: msg].
	^ newSessionID
! !

!HttpSessionModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/HttpSessionModule.st,v 1.1 2003/05/19 19:12:47 cg Exp $'
! !
