"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#NetworkProtocolAdaptor
	instanceVariableNames:'readStream writeStream target log'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-protocol Abstract'
!

NetworkProtocolAdaptor comment:'Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:53

NetworkProtocolAdaptor

Comment from Kom46:
I represent an adaptor that adapts a network protocol into a message based dialog between me and my target.  Thus, I must have knowledge of the underlying network protocol.  My target must implement a protocol designed to work with a specific subclass.

*NOTE* At the moment, this class and subclasses only work as servers.  A small amount of additional work is needed to get them functioning as clients.

My subclasses should only be concerned with specializing for a particular network protocol.

Instance variables:

readStream - The stream to read requests or responses from.
writeStream - The stream to write requests or responses to.
target - The target object with which I will converse.
'
!


!NetworkProtocolAdaptor class methodsFor:'instance creation'!

readAndWriteTo: aStream target: aTarget
	^self new
		initializeReadFrom: aStream writeTo: aStream target: aTarget;
		beginConversation;
		yourself
! !

!NetworkProtocolAdaptor methodsFor:'accessing'!

log

	^log
!

log: aLog

	log _ aLog
!

readStream

	^readStream
!

target

	^target
!

writeStream

	^writeStream
! !

!NetworkProtocolAdaptor methodsFor:'dispatching'!

dispatchRequest: aNetworkRequest
	"<Stream> Dispatches a NetworkRequest as a method to the target and
	answers a stream to be transferred to the client in the response"

	^self target processNetworkRequest: aNetworkRequest
! !

!NetworkProtocolAdaptor methodsFor:'initialize-release'!

initializeReadFrom: rStream writeTo: wStream target: aTarget
	readStream _ rStream.
	writeStream _ wStream.
	target _ aTarget
! !

!NetworkProtocolAdaptor methodsFor:'processing'!

beginConversation
	"Process the stream, converting protocol requests into messages, and
		answers into protocol responses"

	self subclassResponsibility
! !

!NetworkProtocolAdaptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/NetworkProtocolAdaptor.st,v 1.1 2003/05/19 19:08:29 cg Exp $'
! !
