"{ Package: 'stx:goodies/webServer/comanche/chieftain' }"

"{ NameSpace: Comanche }"

Object subclass:#SinglePlugModule
	instanceVariableNames:'processBlock urlPattern'
	classVariableNames:''
	poolDictionaries:''
	category:'Kom-demo'
!

SinglePlugModule comment:'Kom49 (Comanche beta 4.9)
bolot 5/1/2001 02:50

SinglePlugModule

This is very similar to SinglePlugServerAction of PWS. Its processBlock is what fulfills the request. But the important difference arises because of the architectural differences between Comanche and PWS. SinglePlugModule''s processBlock needs to return a value which will get served to the web browser (whereas PWS SinglePlugServerAction''s processBlock needs to write, by way of #reply:, directly to the socket).

To make SinglePlugModule only respond to certain URLs, its urlPattern instance variable should be set appropriately, for example, through using #urlPrefix:, #urlSuffix:, or #urlPattern.

Here''s an example (select all and do it):
| service module |
service := ComancheNetService named: ''SinglePlugExample'' onPort: 8080.
module _ SinglePlugModule newBlock: [:request | request url].
module urlPrefix: ''/singlePlugExample''.
service module: module.
service start

To stop the above server:
| server |
(server _ ComancheServer serverOnPort: 8080 ifAbsent: [])
	ifNotNil: [server stop].
ComancheNetService removeServiceNamed: ''SinglePlugExample''.
'
!


!SinglePlugModule class methodsFor:'instance creation'!

newBlock: aBlock
	^self new processBlock: aBlock
!

newPlug: aBlock
	^self newBlock: aBlock
! !

!SinglePlugModule class methodsFor:'examples'!

displayCam
	"SinglePlugModule displayCam"
	"this creates an equivalent of DisplayCamModule"
	| module |
	module _ self newBlock: [:request | Display magnifyBy: 1/8].
	module urlPrefix: '/desktopcam'.
	^module
!

helloWorld
	"SinglePlugModule helloWorld"
	"this creates an equivalent of HelloWorldModule"
	| module |
	module _ self newBlock: [:request | 'Hello World!!'].
	module urlPrefix: '/helloworld'.
	^module
!

redirectToDisplayCam
	"SinglePlugModule redirectToDisplayCam"
	| module |
	module _ self newBlock: [:request | request redirectTo: '/desktopcam'].
	module urlPrefix: '/desktopcamera'.
	^module
! !

!SinglePlugModule methodsFor:'accessing'!

processBlock: aBlock
	processBlock := aBlock
!

urlPattern
	^urlPattern ifNil: ['*']
!

urlPattern: aString
	urlPattern _ aString
!

urlPrefix: aString
	urlPattern _ aString, '*'
!

urlSuffix: aString
	urlPattern _ '*', aString
! !

!SinglePlugModule methodsFor:'comanche'!

matchesUrl: aUrlString
	^self urlPattern match: aUrlString
!

process: request
	^(self matchesUrl: request url)
		ifTrue: [processBlock value: request]
		ifFalse: []
! !

!SinglePlugModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/chieftain/SinglePlugModule.st,v 1.1 2003/05/19 19:11:29 cg Exp $'
! !
