"{ Package: 'stx:goodies/webServer/comanche' }"!

!Date methodsFor:'converting'!

asTimeStamp
        "Answer the receiver as a TimeStamp at midnight."

        Smalltalk isSmalltalkX ifTrue:[
            ^ self asTimestamp.
        ].
        ^TimeStamp date: self time: (Time fromSeconds: 0)
! !

!Date methodsFor:'printing & storing'!

yyyymmdd
	"no delimiters"
	"Date today yyyymmdd"

	^ self printFormat: #(3 2 1 $- 1 1 2)
! !

!Dictionary methodsFor:'Squeak Compatibility'!

hasKeyStartingWith: aString
	self keysDo: [:key | (key beginsWith: aString) ifTrue: [^ true]].
	^ false
! !

!Form methodsFor:'comanche processing'!

asHtmlElementIn: htmlContainer
	"answer my HTML representation (String),
	as I would look like inside htmlContainer"

	^'<IMG SRC="', self comancheUrl, '">'
! !

!Form methodsFor:'comanche processing'!

asHttpResponseTo: request
	^HttpResponse fromMIMEDocument: self asWebImage
! !

!Form methodsFor:'comanche processing'!

asWebImage
	"return a MIMEDocument"
	| aStream |
	aStream _ (RWBinaryOrTextStream on: '').
	GIFReadWriter putForm: (self asFormOfDepth: 8) onStream: aStream.
	aStream reset.
	^MIMEDocument contentType: MIMEDocument contentTypeGif content: aStream
! !

!Object methodsFor:'comanche'!

asHtmlDocumentForRequest: aNetworkRequest

	self error:
		('The requested object (',
		self asString,
		'), could not be converted into HTML for your browser.')
! !

!Object methodsFor:'comanche'!

asHtmlElement
	"answer my HTML representation (String),
	as I would look like inside an HTML document"

	^self asHtmlElementIn: nil
! !

!Object methodsFor:'comanche'!

asHtmlElementIn: htmlContainer
	"answer my HTML representation (String),
	as I would look like inside htmlContainer"

	^self printString
! !

!Object methodsFor:'comanche'!

asHttpResponseTo: anHttpRequest

	^(self asHtmlDocumentForRequest: anHttpRequest)
		asHttpResponseTo: anHttpRequest
! !

!Object methodsFor:'comanche'!

comancheUrl
	^ObjectUrlMap urlForObject: self ifAbsent: [ObjectUrlMap autoMap: self]
! !

!Object methodsFor:'comanche'!

makeAvailableInComanche
	"force putting the object on the ObjUrlMap list"
	self comancheUrl
! !

!Object methodsFor:'comanche'!

mimeType

	^ Comanche::MIMEDocument defaultContentType
! !

!String methodsFor:'converting'!

asHttpResponseTo: request

	^ Comanche::HttpResponse fromString: self
! !

!String methodsFor:'Compatibility - Squeak'!

asSwikiLink
	| return |
	return _ self trimNullAndStar.
	^(return isEmpty) ifTrue: [return] ifFalse: ['*' , return , '*']
! !

!String methodsFor:'Compatibility - Squeak'!

skipDelimiters: delimiters startingAt: start
	"Answer the index of the character within the receiver, starting at start, that does NOT match one of the delimiters. If the receiver does not contain any of the delimiters, answer size + 1.  Assumes the delimiters to be a non-empty string."

	start to: self size do: [:i |
		delimiters detect: [:delim | delim = (self at: i)]
				ifNone: [^ i]].
	^ self size + 1
! !

!String methodsFor:'Compatibility - Squeak'!

squeakAsInteger
	"Answer the Integer created by interpreting the receiver as the string representation of an integer.  Answer nil if no digits, else find the first digit and then all consecutive digits after that"

	| startPosition tail endPosition |
	startPosition _ self findFirst: [:ch | ch isDigit].
	startPosition == 0 ifTrue: [^ nil].
	tail _ self copyFrom: startPosition to: self size.
	endPosition _ tail findFirst: [:ch | ch isDigit not].
	endPosition == 0 ifTrue: [endPosition _ tail size + 1].
	^ Number readFromString: (tail copyFrom: 1 to: endPosition - 1)
! !

!String methodsFor:'Compatibility - Squeak'!

translateWith:aTable

	1 to: self size do: [ :i |
	    self at: i put: (aTable at: ((self at: i) asciiValue + 1))
	].


"
'Hallo' translateWith:(String withAll: (Character allCharacters collect: [:c | c asLowercase]))
"
! !

!String methodsFor:'Compatibility - Squeak'!

trimNullAndStar
" '   *   string   ***   ' ------->  'string'  "

	| return |
	return _ self.
	[(return endsWith: ' ') | (return endsWith: '*')] whileTrue: [return _ return allButLast].
	[(return beginsWith: ' ') | (return beginsWith: '*')] whileTrue: [return _ return allButFirst].
	^ return

! !

!String methodsFor:'Compatibility - Squeak'!

unescapePercents
	"change each %XY substring to the character with ASCII value XY in hex.  This is the opposite of #encodeForHTTP"
	| ans c asciiVal pos oldPos specialChars |
	ans _ WriteStream on: String new.
	oldPos _ 1.
	specialChars _ '+%'.

	[pos _ self indexOfAnyOf: specialChars startingAt: oldPos. pos > 0]
	whileTrue: [
		ans nextPutAll: (self copyFrom: oldPos to: pos - 1).
		c _ self at: pos.
		c = $+ ifTrue: [ans nextPut: $ ] ifFalse: [
			(c = $% and: [pos + 2 <= self size]) ifTrue: [
				asciiVal _ (self at: pos+1) asUppercase digitValue * 16 +
					(self at: pos+2) asUppercase digitValue.
				pos _ pos + 2.
				asciiVal > 255 ifTrue: [^self]. "not really an escaped string"
				ans nextPut: (Character value: asciiVal)]
			ifFalse: [ans nextPut: c]].
		oldPos _ pos+1].
	ans nextPutAll: (self copyFrom: oldPos to: self size).
	^ ans contents
! !

!Text methodsFor:'converting'!

asHtmlDocumentForRequest: request
	| readStream targetStream |

	readStream _ ReadStream on: (self string).
	targetStream _ WriteStream on: ''.
	targetStream nextPutAll: (Comanche::HTMLformatter startPage: 'A page generated by Comanche').
	targetStream nextPutAll: '<PRE>'.
	runs runsDo:
	"/ (self runs values) with: (self runs runs) do:
		[:aValue :aRun |
			aValue do: [:attr | attr printHtmlOpenTagOn: targetStream].
			targetStream nextPutAll: (readStream next: aRun).
			aValue reverseDo: [:attr | attr printHtmlCloseTagOn: targetStream].
		].

	targetStream nextPutAll: '</PRE>'.
	targetStream nextPutAll: (Comanche::HTMLformatter endPage).
	^targetStream contents
! !

!UndefinedObject methodsFor:'Compatibility - Squeak'!

asSwikiLink
	^ 'none'
! !

!stx_goodies_webServer_comanche class methodsFor:'documentation'!

extensionsVersion_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/extensions.st,v 1.10 2011/09/13 12:14:23 stefan Exp $'
! !
