"{ Package: 'stx:goodies/webServer/comanche/htmlview' }"

"{ NameSpace: HV }"

Object subclass:#HVHttpView
	instanceVariableNames:'prefix parent title model req'
	classVariableNames:'HtmlEntities SafeChars UltraSafeChars'
	poolDictionaries:''
	category:'HV-base'
!

HVHttpView comment:'A VHHttpView is a user interface ''application model'' for a web interface (forms etc).

A view has a domain object as a model, and thus the VHHttpView hierarchy more or less typically mirrors the domain class hierarchy. It also has a reference to it''s parent view which can be a HVModule if the view is the "top view".

A view is instantiated for every request which means that it does not have to care about multiple
Processes - but since the model can be shared with other views either the model needs to care or
the views needs to serialize their access to the model.

A view holds a reference to the request - ''req'' - which it is given upon instantiation. Most of the interaction with the request is done easier through the use of an HVHtmlBuilder which has a lot of support for handling HTML Forms.

Based on the url the framework will call the method with the same name. These methods are kept by convention in the ''urls'' category. If no method name is given in the url the default method #default is called. The methods should return something that Comanche can make HTML from, typically an object implementing #asHttpResponseTo:. String implements #asHttpResponseTo: so the simplest would be to just return a String with HTML in it. A better approach would be to use an HVHtmlBuilder. Look at HVTodoListView for an example.

Finally HVHttpView includes some simple facilities for keeping track of the user and the current user account.'
!


!HVHttpView class methodsFor:'instance creation'!

on: aModel
    "Create a new view on a model.
    We set the model before initializeing so that it
    can use the model."

    ^super basicNew model: aModel; initialize; yourself

    "Created: / 23-11-2006 / 12:26:10 / cg"
!

on: aModel parent: aParent
	^(self new model: aModel) initializeWith: aParent; yourself
! !

!HVHttpView class methodsFor:'* As yet uncategorized *'!

defaultViewClass
    "This is called when a HVRootView is created
    and no viewClass has been specified.
    Default alias, override in subclass."

    ^self

    "Created: / 23-11-2006 / 12:24:55 / cg"
! !

!HVHttpView class methodsFor:'class initialization'!

initialize
	"HVHttpView initialize"

HtmlEntities _ (Dictionary new: 128)
		at: 'amp'	put: $&;
		at: 'lt'		put: $<;
		at: 'gt'		put: $>;
		at: 'quot'	put: $";
		at: 'euro'	put: Character euro;
		yourself.

SafeChars _ Set new.
('1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ/-_()".,''@?!!:; ', '﻿åäöÅÄÖ
éüèáêôâ') do: [:c | SafeChars add: c].
UltraSafeChars _ SafeChars copy.
UltraSafeChars remove: $/; remove: $.



	"#('nbsp' 'iexcl' 'cent' 'pound' 'curren' 'yen' 'brvbar' 'sect' 'uml' 'copy' 'ordf' 'laquo' 'not' 'shy' 'reg' 'hibar' 'deg' 'plusmn' 'sup2' 'sup3' 'acute' 'micro' 'para' 'middot' 'cedil' 'sup1' 'ordm' 'raquo' 'frac14' 'frac12' 'frac34' 'iquest' 'Agrave' 'Aacute' 'Acirc' 'Atilde' 'Auml' 'Aring' 'AElig' 'Ccedil' 'Egrave' 'Eacute' 'Ecirc' 'Euml' 'Igrave' 'Iacute' 'Icirc' 'Iuml' 'ETH' 'Ntilde' 'Ograve' 'Oacute' 'Ocirc' 'Otilde' 'Ouml' 'times' 'Oslash' 'Ugrave' 'Uacute' 'Ucirc' 'Uuml' 'Yacute' 'THORN' 'szlig' 'agrave' 'aacute' 'acirc' 'atilde' 'auml' 'aring' 'aelig' 'ccedil' 'egrave' 'eacute' 'ecirc' 'euml' 'igrave' 'iacute' 'icirc' 'iuml' 'eth' 'ntilde' 'ograve' 'oacute' 'ocirc' 'otilde' 'ouml' 'divide' 'oslash' 'ugrave' 'uacute' 'ucirc' 'uuml' 'yacute' 'thorn' 'yuml' ) withIndexDo: [:each :index | HtmlEntities at: each put: (index + 159) asCharacter isoToSqueak]"
! !

!HVHttpView class methodsFor:'defaults'!

createModel
    "This is called when a HVRootView is created
    and no model has been specified.
    Default model is nil, override in subclass."

    ^nil

    "Created: / 23-11-2006 / 12:24:03 / cg"
!

defaultAlias
    "This is called when a HVRootView is created
    and no alias has been specified.
    Default alias, override in subclass."

    ^''

    "Created: / 23-11-2006 / 12:24:31 / cg"
!

defaultPort
    "This is called when a HVRootView is created
    and no port number has been specified.
    Default alias, override in subclass."

    ^8080

    "Created: / 23-11-2006 / 11:34:48 / cg"
!

rootView
    "Return class to use as root view. Override to provide your own.
    Default is the generic HVRootView."

    ^HVRootView

    "Created: / 23-11-2006 / 11:35:31 / cg"
!

useSessions
    "By default we don't."

    ^false

    "Created: / 23-11-2006 / 17:39:10 / cg"
! !

!HVHttpView class methodsFor:'start & stop'!

start
    "Start service on default port in deployment mode."

    ^self startOn: self defaultPort mode: #deployment

    "Created: / 23-11-2006 / 11:34:04 / cg"
!

startOn: portNum mode: mode
    "Start service on port <portNum> in mode <mode>.
    We first create a root view that use us as default view class
    and then we start the root view."

    ^(self rootView
        model: self createModel
        alias: self defaultAlias
        viewClass: self defaultViewClass)
        startOn: portNum mode: mode; yourself

    "Created: / 23-11-2006 / 11:35:18 / cg"
! !

!HVHttpView methodsFor:'accessing'!

initialize
	title _ 'No title set'.
!

isTopView
	"Am I a topview?"

	^false
!

model
	^model
!

model: aModel
	model _ aModel
!

parent
	^parent
!

postInitialize
!

request
	^req
!

request: aRequest
	req _ aRequest
!

rootView
	"Return the view at the root."

	^parent parent isNil ifTrue: [self] ifFalse: [parent rootView]
!

title
	^title
!

topView
	"Return the view that thinks it is a top view."

	^self isTopView ifTrue: [self] ifFalse: [
		parent parent ifNil: [self] ifNotNil: [parent topView]]
! !

!HVHttpView methodsFor:'comanche'!

currentAccount
	"Get the account for the request."

	^req session attributes at: #account ifAbsent: [nil]
!

currentToc
	"Get the TOC for the request."

	^req session attributes at: self tocSymbol ifAbsent: [nil]
!

dispatch
	"Dispatch to correct method in this view."

	^self dispatchOn: (self methodAfter: self prefix)


!

dispatchOn: method 
	"Dispatch to correct method:
	- If method is empty we call #default:
	- If self responds then we just call it, otherwise:
		- If method is prefixed with 'tms' we strip that (timing) and try again.
	- If method is prefixed with 'src' we return a HTML page with the source for that method."


	| m |
	method isEmpty ifTrue:[ ^self default ].
	m _ method asSymbol.
	[(self respondsTo: m) ifTrue:[^self perform: m].
	(m beginsWith: 'tms')] whileTrue: [m _ (m last: (m size - 3)) asSymbol].
		
	"Is it a request for the source?"
	((m beginsWith: 'src') and: [self respondsTo: (m asString last: (m size - 3)) asSymbol ])
		ifTrue:[
			m _ (m asString last: (m size - 3)) asSymbol.
			(m beginsWith: 'tms') ifTrue: [m _ (m last: (m size - 3)) asSymbol].
			^self showSourceFor: m]
		ifFalse:[^self class name, ' does not have a method called #', m]

!

isLoggedIn
	"Check if the user is logged in."

	^req session attributes includesKey: #account
!

logout
	"Tell the account that it is being logged out.
	Clear out the account attribute in the session.
	Return true if all succeded."

	self currentAccount ifNil: [^true].
	self currentAccount logout.
	req session attributes removeKey: #account ifAbsent: [^false].
	^true
!

methodAfter: aPrefix
	"Extracts the method from the url in the request.
	The methodname is after the prefix upto the first $? $/ or end of url like this:
	'/prefix/method/knirk?habbadubba' => 'method'
	'/prefix/method?habbadubba' => 'method'
	'/prefix/method' => 'method'
	'/prefix/' => ''
	'/prefix' => ''"

	| method endq ends url |
	url _ req rawUrl.
	(url = aPrefix) ifTrue:[ ^'' ].
	method _ url copyFrom: aPrefix size + 1 to: url size. "cut away prefix"
	(method = '/' or: [method isEmpty]) ifTrue:[ ^'' ].
	method _ method allButFirst.
	endq _ method indexOf: $?.
	ends _ method indexOf: $/.
	(endq = 0) ifTrue:[ endq _ method size + 1].
	(ends = 0) ifTrue:[ ends _ method size + 1].
	^method copyFrom: 1 to: (ends min: endq) - 1
!

methodInUrl: url after: aPrefix
	"Extracts the method from the url.
	The methodname is after the prefix upto the first $? $/ or end of url like this:
	'/prefix/method/knirk?habbadubba' => 'method'
	'/prefix/method?habbadubba' => 'method'
	'/prefix/method' => 'method'
	'/prefix/' => ''
	'/prefix' => ''"

	| method endq ends |
	(url = aPrefix) ifTrue:[ ^'' ].
	method _ url copyFrom: aPrefix size + 1 to: url size. "cut away prefix"
	(method = '/') ifTrue:[ ^'' ].
	method _ method allButFirst.
	endq _ method indexOf: $?.
	ends _ method indexOf: $/.
	(endq = 0) ifTrue:[ endq _ method size + 1].
	(ends = 0) ifTrue:[ ends _ method size + 1].
	^method copyFrom: 1 to: (ends min: endq) - 1
!

prefix
	"Return the full prefix which is composed by concatenating up the parent chain."

	^String streamContents: [:stream | stream nextPutAll: parent prefix; nextPutAll: prefix ]
!

redirectTo: page
	"Make a redirection to another url."

	^req redirectTo: (page isEmpty ifTrue:['/'] ifFalse: [page])
!

redirectToDefault
	"Make a redirection to the default url of this view."

	| px |
	px _ self prefix.
"	(px notEmpty and: [px last ~= $/]) ifTrue:[px _ px, '/']."
	^req redirectTo: (px isEmpty ifTrue:['/default'] ifFalse: [px, '/default'])
!

redirectToLocal: page
	"Make a redirection to another page in this view."

	^req redirectTo: self prefix, '/', page
!

setToc: aToc
	"Set the TOC for the request."

	req session attributes at: self tocSymbol put: aToc
! !

!HVHttpView methodsFor:'html creation'!

builder
	"Create a standard builder with our own request."

	^HVHtmlBuilder request: req view: self
!

endhtml
	"Standard end."

^'</body></html>'
!

endhtml: request
	"Standard end. Attach a link to the source of the page."

	| url slash |
	slash _ request url findLast: [:c | c = $/ ].
	url _ (request url copyFrom: 1 to: slash), 'src', (request url copyFrom: slash + 1 to: request url size).
^'<hr><br><font size=-1>', (self link: url text: 'See source for this page'), '</font></body></html>'
!

fixForHtml: string
	"Replace a lot of characters with HTML friendly stuff."

	| temp |
	(string isEmpty or: [string isNil]) ifTrue:[^string].
	temp _ string copyReplaceAll: '&' with: '&amp;'.
	HtmlEntities keysAndValuesDo:
		[:entity :char |
		char = $& ifFalse:
			[temp _ temp copyReplaceAll: char asString with: '&' , entity , ';']].
	^'<pre>', temp, '</pre>'
!

initializeWith: aParent
	"If we have no request set yet, we pick it from our parent."

	parent _ aParent.
	req ifNil: [req _ parent request]. "This has no effect if the parent is a HVModule but then request is set later in #process: instead"
	self initialize; postInitialize
!

link: url text: text
	"Return a HTML link."

	^String streamContents: [:stream |
		stream
			nextPutAll: '<a href="';
			nextPutAll: url;
			nextPutAll: '">'; nextPutAll: text; nextPutAll: '</a>' ]
!

linkBase: server local: relativeUrl text: text
	"Return a local HTML link to the view."

	^self link: server, self prefix, relativeUrl text: text
!

linkback: text
	"Return a local backlink."

	^self link: self prefix text: text
!

linkbackparent: text
	"Return a parent backlink."

	^self link: parent prefix text: text
!

linklocal: relativeUrl text: text
	"Return a local HTML link to the view."

	^self link: self prefix, relativeUrl text: text
!

message: message
	"Return a single page with a message and a back link to base page of this view."

	^self builder start; h1: message; linkBack: 'Back'; end; yourself
!

message: message text: text
	"Return a single page with a message and a back link to base page of this view."

	^self builder start; h1: message; linkBack: text; end; yourself
!

showSourceFor: selector
	"Show the source for given selector."

^self starthtml, '
<h2>Source for ', self class name, '>>', selector, '</h2>
<pre>
', (self fixForHtml: (self class sourceMethodAt: selector) asString), '
</pre>', self endhtml
!

starthtml
	"Standard start."

^'<html><head><title>', self title, '</title></head><body>'
! !

!HVHttpView methodsFor:'oldmethods'!

createSelectWidget: values name: name 
	"Create a select widget for forms."

	^self createSelectWidget: values selected: 1 name: name
!

createSelectWidget: values selected: selected name: name 
	"Create a select widget for forms."

	^self createSelectWidget: values selected: selected name: name withIds: true
!

createSelectWidget: objects selected: selected name: name withIds: withIds
	"Create a select widget for forms."

	| obj id |
	^String streamContents: [:stream |
		stream nextPutAll: '<select name="', name, '" size="1">'.
		1 to: objects size do: [:n |
			obj _ objects at: n.
			withIds ifTrue:[ id _ obj id] ifFalse:[ id _ n].
			(id = selected)
				ifTrue: [ stream nextPutAll: '<option selected']
				ifFalse: [ stream nextPutAll: '<option' ].
			withIds
				ifTrue: [stream nextPutAll: ' value="', id fullPrintString, '">']
				ifFalse: [stream nextPut: $> ].
			stream nextPutAll: obj name; nextPutAll: '</option>'.
		].
		stream nextPutAll: '</select>']
! !

!HVHttpView methodsFor:'simple messages'!

message: message backlink: link
	"Return a single page with a message and a hard link back."

	^self starthtml, '<h1>', message, '</h1>', (self linklocal: link text: 'Back'), self endhtml
!

message: message link: link
	"Return a single page with a message and a back link relative to this view."

	^self starthtml, '<h1>', message, '</h1>', (self link: link text: 'Back'), self endhtml
!

message: message linklocal: link
	"Return a single page with a message and a back link relative to this view."

	^self message: message linklocal: link text: 'Back'
!

message: message linklocal: link text: text
	"Return a single page with a message and a back link relative to this view."

	^self starthtml, '<h1>', message, '</h1>', (self linklocal: '/', link text: text), self endhtml
!

serverError: aString
	"Return an error to the user."

	^'Error returned by server: ', aString
! !

!HVHttpView methodsFor:'urls'!

collapsetoc
	"Collapse given entry by updating the session info on which entries to show."

	| entryName toc tocString tocSym |
	tocSym _ self tocSymbol.
	entryName _ req getFields at: #entry.
	toc _ req session attributes at: tocSym ifAbsent:[^self redirectToDefault].
	toc _ ((toc findTokens: $,) collect: [:s | s asSymbol]) asSet.
	toc remove: entryName asSymbol.
	tocString _ String streamContents: [:s |
		toc do: [:sym | s nextPutAll: sym, ',']].
	req session attributes at: tocSym put: tocString.
	^self redirectToDefault
!

default
	"This is the default method called on a HttpView if no other
	method is named in the url. It is the base page for the view.
	It should always be implemented. This is a simple test implementation."

	| b |
	b _ self builder.
	b start; h1: 'View for model:'; p: model printString; end.
	^b
!

expandtoc
	""

	| entryName toc tocString tocSym |
	tocSym _ self tocSymbol.
	entryName _ req getFields at: #entry.
	toc _ req session attributes at: tocSym ifAbsent: [nil].
	toc ifNotNil: [
		toc _ ((toc findTokens: $,) collect: [:s | s asSymbol]) asSet.
		toc add: entryName asSymbol.
		tocString _ String streamContents: [:s |
			toc do: [:sym | s nextPutAll: sym, ',']]]
		ifNil: [toc _ ''].
	req session attributes at: tocSym put: tocString.
	^self redirectToDefault
! !

!HVHttpView methodsFor:'utilities'!

validateSafeString: aString

	aString do: [:c | (SafeChars includes: c) ifFalse:[^false]].
	^true
!

validateUltraSafeString: aString

	aString do: [:c | (UltraSafeChars includes: c) ifFalse:[^false]].
	^true
! !

!HVHttpView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/htmlview/HVHttpView.st,v 1.2 2006/11/24 11:05:07 cg Exp $'
! !

HVHttpView initialize!
