"{ Package: 'stx:goodies/webServer/comanche/htmlview' }"

"{ NameSpace: HV }"

HVHttpView subclass:#HVRootView
	instanceVariableNames:'viewClass coreMod sessionMod alias selectorFilter'
	classVariableNames:''
	poolDictionaries:''
	category:'HV-base'
!


!HVRootView class methodsFor:'instance creation'!

model: aModel alias: anAlias viewClass: aViewClass
    "Create a persistent root view that can be plugged into Komanche."

    ^(super on: aModel) alias: anAlias viewClass: aViewClass

    "Created: / 23-11-2006 / 12:25:52 / cg"
! !

!HVRootView methodsFor:'accessing'!

rootView
    "Return the view at the root, that's me."

    ^self

    "Created: / 24-11-2006 / 00:58:35 / cg"
!

urlStream
    "I am the root view and can not hold a urlStream
    because I need to be Process safe."

    ^nil

    "Created: / 24-11-2006 / 00:13:04 / cg"
! !

!HVRootView methodsFor:'comanche'!

getTopView: request
    "Default way of instantiating a top view for the model.
    If the viewClass is set we use that, otherwise we try
    a double dispatch on the model using #viewFor:."

    ^viewClass
        ifNotNil: [viewClass on: model parent: self]
        ifNil: [(model respondsTo: #viewFor:)
                ifTrue: [model viewFor: self]
                ifFalse: [self error: 'Do not know which view to instantiate!!']]

    "Created: / 23-11-2006 / 23:50:39 / cg"
! !

!HVRootView methodsFor:'defaults'!

serviceName
    "By default we use class name of the HVRootView subclass,
    otherwise the class name of viewClass if we have one."

    |cls nm|

    cls := (viewClass notNil and: [self class = HVRootView])
        ifTrue: [viewClass]
        ifFalse: [self class].

    Smalltalk isSmalltalkX ifTrue:[
        nm := cls nameWithoutPrefix.
        (nm startsWith:'HV') ifTrue:[
            nm := nm copyFrom:3.
        ].
    ] ifFalse:[
        nm := cls name.
    ].

    ^ nm

    "Created: / 23-11-2006 / 17:37:38 / cg"
    "Modified: / 23-11-2006 / 22:31:21 / cg"
!

useSessions
    "By default we check with viewClass if we have one,
    otherwise false."

    ^viewClass isNil ifTrue: [false] ifFalse: [viewClass useSessions]

    "Created: / 23-11-2006 / 17:37:56 / cg"
! !

!HVRootView methodsFor:'dispatch'!

allowedSelector: aSelector forView: aView
    "Answer true if <aSelector> is ok to call from a URL
    to the view <aView>. Default implementation is to
    use the pluggable filter block."

    ^selectorFilter copy value: aSelector value: aView

    "Created: / 24-11-2006 / 00:57:40 / cg"
! !

!HVRootView methodsFor:'initialize / release'!

alias: anAlias viewClass: aClass
    "Set the default alias and viewClass if not already set."

    alias ifNil: [alias := anAlias].
    viewClass ifNil: [viewClass := aClass]

    "Created: / 23-11-2006 / 17:30:14 / cg"
!

categoryOfElement: selector inClassOrSuperclass: class
    "Find first category of selector up the superclass chain."

    ^class ifNotNil: [
        ^(class organization categoryOfElement: selector)
            ifNil: [self categoryOfElement: selector inClassOrSuperclass: class superclass]]

    "Created: / 24-11-2006 / 01:00:34 / cg"
!

defaultSelectorFilter
    "Override this method to supply your own selectorFilter.
    Or plug it in using #selectorFilter:"

    ^[:selector :view |
        (view respondsTo: selector) and: [
            (self categoryOfElement: selector inClassOrSuperclass: view class) = #urls]]

    "Created: / 24-11-2006 / 01:00:41 / cg"
!

initialize
    "Default selector filter is to verify that
    the view responds to the selector and that
    it is in the method category 'urls'."

    alias := ''.
    selectorFilter := self defaultSelectorFilter

    "Created: / 24-11-2006 / 01:00:08 / cg"
! !

!HVRootView methodsFor:'request handling'!

process: request
    "Process the request. This is the method that is called by Comanche.
    Given the view we stick the request into it and let it take over.
    When the result is returned we turn it into a proper HttpResponse.
    A special notification can be signaled to return a specific response and
    bypassing the call chain."

    | view |

    (request url endsWith: '/favicon.ico') ifTrue: [
self halt.
        ^HttpResponse fromString: 'favicon.ico not supported'
    ].

    view := self getTopView: request.
    view request: request. "normally not needed when using #viewFor: but here it is"
    [
        ^(view dispatch) asHttpResponseTo: request
    ] on: HVResponseNotification do: [:ex | 
        ^ex response
    ]

    "Created: / 23-11-2006 / 23:47:53 / cg"
! !

!HVRootView methodsFor:'start/stop'!

startOn: portNum mode: mode
    "Creates a new standalone http service with this root view plugged in."

    |  name useSessions ma server service |

    name := self serviceName.
    useSessions := self useSessions.

    Smalltalk isSmalltalkX ifTrue:[
        server := HTTPServer serverOnPort:portNum.
        ma := HTTPPluggableService new.
        ma linkNames:(Array with:name).
        ma registerServiceOn:server.
        ma addPlug: [:request | self process: request].
        ^ self.
    ].

    ma := ModuleAssembly core.
    coreMod := ma currentModule.
"   ma logTo: (KomLogger on: Transcript)."
    useSessions ifTrue: [
        ma trackSessions.
        sessionMod := ma currentModule.
        ma sessionTimeout: 30].

    alias = ''
        ifTrue: [ma addPlug: [:request | self process: request]]
        ifFalse: [ma alias: alias
                    to: [ma addPlug: [:request | self process: request]]].
    ^(HttpService startOn: portNum named: name)
        plug: ma rootModule; mode: mode

    "Created: / 23-11-2006 / 17:36:29 / cg"
    "Modified: / 23-11-2006 / 22:34:53 / cg"
! !

!HVRootView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/htmlview/HV__HVRootView.st,v 1.1 2006/11/24 11:04:19 cg Exp $'
! !
