"{ Package: 'stx:goodies/webServer/comanche/htmlview' }"

"{ NameSpace: HV }"

HVHttpView subclass:#HVTransientView
	instanceVariableNames:'parent title req urlStream urlOriginalPosition'
	classVariableNames:''
	poolDictionaries:''
	category:'HV-base'
!


!HVTransientView methodsFor:'accessing'!

request: aRequest
    "This method is only used from #process: when creating the first view."

    req := aRequest.
    req ifNotNil: [
        urlStream := HVUrlStream onString: req rawUrl.
        urlStream ifNotNil: [urlOriginalPosition := urlStream position]]

    "Created: / 24-11-2006 / 00:12:19 / cg"
!

rootView
    "Return the view at the root, the root view will return self."

    ^parent rootView

    "Created: / 24-11-2006 / 00:58:16 / cg"
!

urlStream
    ^ urlStream
! !

!HVTransientView methodsFor:'dispatching'!

allowedSelector: aSelector
    "Answer true if <aSelector> is ok to call from a URL.
    Default implementation is to delegate to the rootView
    which represents this web application."

    ^self rootView allowedSelector: aSelector forView: self

    "Created: / 24-11-2006 / 00:57:13 / cg"
!

dispatch
    "Dispatch to correct method in this view.
    If dispatchOverride returns something
    different from nil, consider it handled."

    | result |
    result := self dispatchOverride.
    result ifNotNil: [^result].
    ^self dispatchOn: self nextMethod

    "Created: / 24-11-2006 / 00:05:28 / cg"
!

dispatchOn: method 
    "Dispatch to correct method:
    - If method is empty we call #default
    - If the selector is allowed to be executed then we just call it
    - If method is prefixed with 'src' and viewing the source is allowed,
        then we return a HTML page with the source for that method."

    | m |
    method isEmptyOrNil ifTrue:[ ^self default ].
    m := method asSymbol.
    (self allowedSelector: m)
        ifTrue:[^self perform: m].

    "Is it a request for the source?"
    (m beginsWith: 'src')
        ifTrue: [
            m := (m asString last: (m size - 3)) asSymbol.
            (self allowedSrcSelector: m)
                ifTrue: [^self showSourceFor: m]
                ifFalse: [^self serverError: 'Not allowed to view source']]
        ifFalse: [^self serverError: 'Specified URL: #', method, ' not allowed']

    "Created: / 24-11-2006 / 00:08:04 / cg"
!

dispatchOverride
    "Handle special urls. Subclass implementors
    should call super first and see if it was handled."

    ^nil

    "Created: / 24-11-2006 / 00:07:47 / cg"
!

nextMethod
    "Pick out the next method (path part = directory) from the urlStream."

    ^urlStream next

    "Created: / 24-11-2006 / 00:09:11 / cg"
! !

!HVTransientView methodsFor:'initialization'!

initializeWith: aParent
    "If we have no request set yet, we pick it from our parent.
    If aParent is nil then request is set later in #process: instead."

    aParent ifNotNil: [
        parent := aParent.
        req ifNil: [
            req := parent request.
            urlStream := parent urlStream.
            urlStream ifNotNil: [urlOriginalPosition := urlStream position]]].
    self postInitialize

    "Created: / 24-11-2006 / 00:10:33 / cg"
! !

!HVTransientView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/htmlview/HV__HVTransientView.st,v 1.1 2006/11/24 11:04:41 cg Exp $'
! !
