"{ Package: 'stx:goodies/webServer/comanche/seaside2_5b4' }"

WAPresenter subclass:#WAComponent
	instanceVariableNames:'decoration'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Component'
!


!WAComponent class methodsFor:'as yet unclassified'!

applicationWithPath: aString
	| app |
	app _ WAApplication path: aString.
	app configuration addAncestor: WARenderLoopConfiguration localConfiguration.
	app preferenceAt: #rootComponent put: self.
	^ app
!

canBeRoot
	^ false
!

registerAsApplication: appName
	|app|
	app _ self applicationWithPath: appName.
	WADispatcher default registerEntryPoint: app at: appName.
	^ app
!

registerAsAuthenticatedApplication: appName
	| app user password |
	app _ self registerAsApplication: appName.
	app configuration addAncestor: WAAuthConfiguration localConfiguration.
	
	[user _ FillInTheBlank
		request: ('Please choose an administrator\username for the application ''', appName asString , '''') withCRs.
	user = ''] whileTrue.
		
	[password _ FillInTheBlank
		request: 'Please enter a password for ', user printString.
	password = ''] whileTrue.
		
	app preferenceAt: #login put: user.
	app preferenceAt: #password put: password.
	
	^ app
	
! !

!WAComponent methodsFor:'as yet unclassified'!

addDecoration: newDecoration
	| prev dec |
	prev _ nil.
	dec _ self decoration.
	[dec ~~ self and: [self decoration: dec shouldWrap: newDecoration]] whileTrue:
		[prev _ dec.
		dec _ dec owner].
	newDecoration owner: dec.
	prev
		ifNil: [self decoration: newDecoration]
		ifNotNil: [prev owner: newDecoration].
	^ newDecoration
!

authenticateWith: anAuthenticator during: aBlock
	^ self decorateWith: (WABasicAuthentication new authenticator: anAuthenticator) during: aBlock
!

children
	^ #()
!

childrenDo: aBlock
	self children do: [:ea | ea ifNotNil: [aBlock value: ea]]
!

decorateWith: aDecoration during: aBlock
	| val |
	self addDecoration: aDecoration.
	val _ aBlock value.
	self removeDecoration: aDecoration.
	^ val
!

decoration
	decoration ifNil: [decoration _ WAStateHolder new contents: self].
	^ decoration contents
!

decoration: aDecoration
	decoration ifNil: [decoration _ WAStateHolder new].
	decoration contents: aDecoration
!

decoration: oldDecoration shouldWrap: newDecoration
	^ (oldDecoration isGlobal and: [newDecoration isGlobal not])
		or: [oldDecoration isDelegation and: [newDecoration isLocal]]
!

decorationChainDo: aBlock
	aBlock value: self decoration
!

nextPresentersDo: aBlock
	self childrenDo: [:ea | ea decorationChainDo: aBlock]
!

removeDecoration: aDecoration
	| dec |
	dec _ self decoration.
	dec = aDecoration ifTrue:
		[self decoration: dec owner.
		^ self].
	
	[dec notNil] whileTrue:
		[dec owner = aDecoration
			ifTrue: [dec owner: aDecoration owner.  ^ self]
			ifFalse: [dec _ dec owner]]
!

renderOn: aRenderer
	self decorationChainDo: [:ea | ea renderWithContext: aRenderer context]
!

visiblePresentersDo: aBlock
	self decorationChainDo: [:ea | ea withNextPresentersDo: aBlock]
! !

!WAComponent methodsFor:'call/answer'!

answer
	self answer: self
!

answer: anObject
	self decorationChainDo: [:ea | ea handleAnswer: anObject]
!

call: aComponent
	^ Continuation currentDo: [:cc | self show: aComponent onAnswer: cc]
!

handleAnswer: anObject
	^ false
!

onAnswer: aBlock
	^ self addDecoration: (WAAnswerHandler new block: aBlock)
!

show: aComponent
	self show: aComponent onAnswer: [:v]
!

show: aComponent onAnswer: aBlock
	| delegation event |
	delegation _ WADelegation new delegate: aComponent.
	event _ nil.
	event _ aComponent onAnswer: [:v | delegation remove. event remove. aBlock value: v].
	self addDecoration: delegation.
	WARenderNotification raiseSignal
! !

!WAComponent methodsFor:'convenience'!

activeComponent
	self visiblePresentersDo: [:ea | ea isDecoration ifFalse: [^ ea]]
!

addMessage: aString
	self addDecoration: (WAMessageDecoration new message: aString)
!

confirm: aString
	^ self call: (WAYesOrNoDialog new addMessage: aString)
!

inform: aString
	self call: (WAFormDialog new addMessage: aString)
!

isolate: aBlock
	|txn val |
	txn _ WATransaction new.
	val _ self decorateWith: txn during: aBlock.
	txn close.
	^ val
!

request: aString
	^ self request: aString default: ''
!

request: aString default: initialString
	^ self request: aString label: nil default: initialString
!

request: requestString label: labelString
	^ self request: requestString label: labelString default: ''
!

request: requestString label: labelString default: initialString
	^ self call:
		((WAInputDialog new
			label: labelString;
			default: initialString)
				addMessage: requestString)
!

validateWith: aBlock
	self addDecoration: (WAValidationDecoration new validateWith: aBlock)
! !

!WAComponent methodsFor:'rendering'!

script
	^ nil
!

style
	^ nil
! !

!WAComponent class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/seaside2_5b4/WAComponent.st,v 1.2 2004/09/21 23:32:23 cg Exp $'
! !
