"{ Package: 'stx:goodies/webServer/comanche/seaside2_5b4' }"

Object subclass:#WAResponse
	instanceVariableNames:'contentType headers cookies status stream'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-HTTP'
!


!WAResponse class methodsFor:'as yet unclassified'!

basicAuthWithRealm: aString
	^ self new basicAuthenticationRealm: aString
!

document: anObject mimeType: mimeType
	|doc|
	doc _ anObject asMIMEDocument.
	^ self new
		contentType: (mimeType ifNil: [doc contentType]);
		headerAt: 'Expires' put: 'Thu, 01 Jan 2095 12:00:00 GMT';
		nextPutAll: doc content
!

internalError: anError
        |err|

        err := self new.
        err
                internalError;
                nextPutAll: '<h1>Internal Error</h1>';
                nextPutAll: '<h2>';
                nextPutAll: anError asString;
                nextPutAll: '</h2>'.

        Smalltalk isSmalltalkX ifTrue:[
            err nextPutAll: '<p>'.
            err nextPutAll: '<code><pre>'.
            anError suspendedContext fullPrintAllOn:err.
            err nextPutAll: '</pre></code>'.
        ].

        ^ err
!

new
	^ self basicNew initialize
!

notFound: locationString
        ^ self new
                notFound;
                nextPutAll: '<h1>Seaside Error: "', locationString, '" not found. </h1>';
                yourself
!

redirectTo: locationString
	^ self new redirectTo: locationString
!

refreshWithMessage: aString location: locationString delay: aNumber
	^ self new nextPutAll:
		 ('<html>
			<head>
			<meta http-equiv="refresh" content="',aNumber asString,';URL=',locationString,'">
			<title>',aString,'</title>
			</head>
			<body>
			<h1>',aString,'</h1>
			You are being redirected to <a href="',locationString,'">',locationString,'</a>
			</body>
		</html>');
		yourself
	
! !

!WAResponse methodsFor:'accessing'!

addCookie: aCookie
	cookies ifNil: [cookies _ OrderedCollection new].
	cookies add: aCookie
!

contentType
	^ contentType
!

contentType: mimeTypeString
	contentType _ mimeTypeString
!

contents
        Smalltalk isSmalltalkX ifTrue:[
            ^ stream contents
        ].
        ^ stream reset
!

contents: aStream
	stream _ aStream
!

cookieAt: key put: value
	self addCookie: (WACookie key: key value: value)
!

cookies
	^ cookies ifNil: [#()]
!

cr
        stream cr
!

headerAt: key put: value
	headers ifNil: [headers _ Dictionary new].
	headers at: key put: value
!

headers
	^ headers ifNil: [#()]
!

initialize
	stream _ SeasidePlatformSupport readWriteStream.
	status _ '200'.
	contentType _ 'text/html; charset=utf-8'.
!

nextPut: aCharacter
	stream nextPut: aCharacter asCharacter
!

nextPutAll: aString
	aString do: [:ea | self nextPut: ea]
!

release
	stream _ cookies _ headers _ nil
!

status
	^ status
!

status: statusString
	status _ statusString asString
!

stream
	^ stream
!

stream: aStream
	stream _ aStream
! !

!WAResponse methodsFor:'convenience'!

basicAuthenticationRealm: aString
	self headerAt: 'WWW-Authenticate' put: ('Basic realm="',aString,'"').
	self authenticationFailed
!

redirectTo: aString
	self headerAt: 'Location' put: aString.
	self redirect.
! !

!WAResponse methodsFor:'status'!

authenticationFailed
	self status: 401
!

internalError
	self status: 500
!

notFound
	self status: 404
!

redirect
	self status: 302
! !

!WAResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/seaside2_5b4/WAResponse.st,v 1.4 2009/08/14 22:07:04 cg Exp $'
! !
