"{ Package: 'stx:goodies/webServer/comanche/seaside2_5b4' }"

WAExpiringHandler subclass:#WASession
	instanceVariableNames:'continuations application escapeContinuation monitor state
		currentRequest'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Session'
!


!WASession class methodsFor:'as yet unclassified'!

application: anApplication
	^ self basicNew initializeWithApplication: anApplication
! !

!WASession methodsFor:'accessing'!

application
	^ application
!

mainClass
	^ application preferenceAt: #mainClass
! !

!WASession methodsFor:'cookies'!

checkForCookies
	| check |
	check _ WAExternalID new asString.
	self redirectWithCookie: 
		(WACookie new
			key: check;
			value: 'yes';
			expireIn: (Duration seconds: 2);
			yourself).
	^ currentRequest cookies includesKey: check
!

redirectWithCookie: aCookie
	self redirectWithCookies: (Array with: aCookie)
!

redirectWithCookies: aCollection
	| response |
	self respond:
		[:url |
		response _ WAResponse redirectTo: url asString.
		aCollection do: [:ea | response addCookie: ea].
		response]
! !

!WASession methodsFor:'expiring'!

defaultTimeoutSeconds
	^(self application preferenceAt: #sessionExpirySeconds)
		ifNil: [super defaultTimeoutSeconds]
!

pageExpired
	state restoreLastSnapshot.
	self redirectWithMessage: 'That page has expired.' delay: 3.
	WAPageExpired raiseSignal.
! !

!WASession methodsFor:'initializing'!

initialize
!

initializeWithApplication: anApplication
	application _ anApplication.
	continuations _ WALRUCache new.
	monitor _ WAProcessMonitor new.
	state _ WAStateRegistry new.
	self initialize.
! !

!WASession methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', (application urlForRequestHandler: self) asString, ')'
! !

!WASession methodsFor:'request handling'!

actionField
	^ '_k'
!

currentRequest
	^ currentRequest
!

errorHandler
	^ application preferenceAt: #errorHandler
!

incomingRequest: aRequest
        ^ monitor
                critical: [self responseForRequest: aRequest]
                ifError: [:e | self halt. WAResponse internalError: e]
!

performRequest: aRequest
        | key continuation |

        key _ aRequest fields at: self actionField ifAbsent: [^ self start: aRequest].
        continuation _ continuations at: key ifAbsent: [^ self unknownRequest: aRequest].
        continuation value: aRequest
!

responseForRequest: aRequest
	currentRequest _ aRequest.
	^ self withEscapeContinuation:
		[WACurrentSession use: self during: [self withErrorHandler: [self performRequest: aRequest]]]
!

start: aRequest
	self mainClass new start: aRequest
!

unknownRequest: aRequest
	self pageExpired
!

withErrorHandler: aBlock
	^ [aBlock
		on: Error
		do: [:e | self errorHandler handleError: e. WAPageExpired raiseSignal]]
		on: Warning
		do: [:w | self errorHandler handleWarning: w. WAPageExpired raiseSignal]
!

withEscapeContinuation: aBlock
        Continuation supported ifFalse:[
            escapeContinuation := [:val | ^ val].
            aBlock value.
            ^ self pageIntentionallyLeftBlank.
        ].

        ^ Continuation currentDo:
                        [:cc |
                        escapeContinuation := cc.
                        aBlock value.
                        self pageIntentionallyLeftBlank]
! !

!WASession methodsFor:'responding'!

actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: aContinuation)
!

actionUrlForKey: aString
	^ (application urlForRequestHandler: self)
		addParameter: self actionField value: aString;
		yourself
!

closePopup
	^ self respond:
		[:url |
		WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: 'self.opener.location=',url asString printString;
			nextPutAll:'</script></html>';
			yourself]
!

pageIntentionallyLeftBlank
	self returnResponse: WAResponse new
!

redirect
	^ self respond: [:url | WAResponse redirectTo: url asString]
!

redirectTo: urlString
	self returnResponse: (WAResponse redirectTo: urlString)
!

redirectWithMessage: aString delay: aNumber
	self respond:
		[:url |
		WAResponse
			refreshWithMessage: aString
			location: url asString
			delay: aNumber].
!

respond: responseBlock
        |request snapshotHolder response oldEscape |
        snapshotHolder _ ValueHolder new.
        oldEscape _ escapeContinuation.

        escapeContinuation _
                [:v |
                snapshotHolder contents: state snapshot.
                oldEscape value: v].

        Continuation supported ifFalse:[
               |cc url|
                cc := [:val | 
                              state restoreSnapshot: snapshotHolder contents.
                              self halt. 
                              ^ val].
                url _ self actionUrlForContinuation: cc.
                response _ responseBlock value: url.
                request _ self returnResponse: response
        ] ifTrue:[
            request _ 
            Continuation currentDo:
                [:cc ||url|
                url _ self actionUrlForContinuation: cc.
                response _ responseBlock value: url.
                self returnResponse: response].
        ].

        state restoreSnapshot: snapshotHolder contents.
        ^ request 
!

returnResponse: aResponse
	escapeContinuation value: aResponse
!

script: aString
	self redirectWithMessage: '<script>', aString, '</script>' delay: 0
! !

!WASession methodsFor:'state'!

registerObjectForBacktracking: anObject
	state registerObject: anObject
! !

!WASession class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/seaside2_5b4/WASession.st,v 1.4 2004/09/22 11:45:12 cg Exp $'
! !
