"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/webServer/comanche/stt' }"

ComancheService subclass:#ComancheSTTService
	instanceVariableNames:'comancheSTTRoot defaultFile smalltalkDialect'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-STT'
!

!ComancheSTTService class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    I am the glue between the HTTPServers services (on the calling side)
    and and STTModule on the other side.

    Instances of me are registered in the HTTPServer (under the link-name)
    and I will be invoked via the #process: method for incoming
    requests.

    [Author:]
        Claus Gittinger
"
! !

!ComancheSTTService class methodsFor:'configuring - setup'!

loadPackage
    |comSTTPackage comSwikiPackage|

    (Comanche isNil or:[Comanche::STTModule isNil]) ifTrue:[
        comSTTPackage := 'stx:goodies/webServer/comancheSTT'.
        (Smalltalk loadPackage:comSTTPackage) ifFalse:[
            ^ false.
        ].
        Class withoutUpdatingChangesDo:[
            "sigh - we need the Comanche exensions as well (String translateWith:, for example)"
            comSwikiPackage := 'stx:goodies/webServer/comancheSwiki'.
            (Smalltalk loadExtensionsForPackage:comSwikiPackage) ifFalse:[
                ^ false.
            ].
        ]
    ].
    ^ true

"
    self loadPackage
"
! !

!ComancheSTTService class methodsFor:'defaults'!

defaultFile
    ^ 'default.stt'
!

defaultSmalltalkDialect
    ^ #squeak
!

linkName
    "return the default linkName path (with slash)"

    ^ '/stt'
! !

!ComancheSTTService class methodsFor:'queries-service'!

comancheSTTRoot
    #( './webServer/comancheSTT'
     '~/webServer/comancheSTT'
     './examples/webServer/comancheSTT'
     '../../goodies/webServer/data/comancheSTT'
     '/pub/webServer/comancheSTT' ) 
            do:[:aFilename | 
                (aFilename asFilename exists) ifTrue:[
                    ^ aFilename
                ]
            ].
    ^ (Filename currentDirectory) 

    "
     self comancheSTTRoot        
    "
!

isThreadSafe
    ^ true
!

settingsApplicationClass
    "my application in the settings dialog"

    ^ ComancheSTTSettingsApplication
! !

!ComancheSTTService methodsFor:'accessing'!

comancheSTTRoot

    ^ comancheSTTRoot ? self class comancheSTTRoot
!

comancheSTTRoot:aDirectory
    self comancheSTTRoot ~= aDirectory ifTrue:[
        comancheSTTRoot := aDirectory.
        self setupModule.
    ].
!

defaultFile

    ^ defaultFile ? self class defaultFile
!

defaultFile:aFilename
    self defaultFile ~= aFilename ifTrue:[
        defaultFile := aFilename.
        self setupModule.
    ].
!

smalltalkDialect
    ^ smalltalkDialect ? (self class defaultSmalltalkDialect)
!

smalltalkDialect:aDialectSymbol
    self smalltalkDialect ~= aDialectSymbol ifTrue:[
        smalltalkDialect := aDialectSymbol.
        self setupModule.
    ].
! !

!ComancheSTTService methodsFor:'actions'!

process:aSTXHttpRequest
    module isNil ifTrue:[
        self error:'no comanche-module'.
    ].
    (self isDebugRequest:aSTXHttpRequest) ifTrue:[
        self processDebugRequest:aSTXHttpRequest.
        ^ self
    ]. 

    (self isDebugAccept:aSTXHttpRequest) ifTrue:[
        self processDebugAccept:aSTXHttpRequest.
        ^ self
    ] ifFalse:[
        (self debugMode:aSTXHttpRequest) ifTrue:[
            self nextPutDebugLinkFor:aSTXHttpRequest
        ].
    ].
    module process:aSTXHttpRequest.
!

separateHeaderFromData:aStream

    |buffer index|

    buffer := aStream contents.
    index := buffer indexOfSubCollection:(Comanche::HttpRequest endOfRequestHeaderMarker).
    ^ Array with:(buffer copyTo:index) with:(buffer copyFrom:index + 4)
! !

!ComancheSTTService methodsFor:'initialization'!

initialize
    super initialize.
    autoStart := false.
    smalltalkDialect := self class defaultSmalltalkDialect.
!

setupModule
    Comanche::STTModule isNil ifTrue:[ 
        Transcript showCR:'STT classes not loaded - try to load ...'.
        self class loadPackage ifFalse:[
            Transcript showCR:'load of STT classes failed'.
            ^ self
        ].
    ].
    module := Comanche::STTModule on:nil "anExecutionContext" 
                                  withDocumentRoot:self comancheSTTRoot.
    module isNil ifTrue:[
        Transcript showCR:'STT cannot create STT Module'.
        ^ self.
    ].
    module defaultFile:self defaultFile.
    module service:self.
! !

!ComancheSTTService methodsFor:'maintenance html presentation'!

specificChangeFromRequest:aRequest 
    "extracts parameter from an html-maintenance post-request"

    |tmp|

    tmp := aRequest urlParameterAt:'ServiceRootDirectory'.
    tmp notNil ifTrue:[
        self comancheSTTRoot:tmp.
    ].
    tmp := aRequest urlParameterAt:'DefaultPage'.
    tmp notNil ifTrue:[
        self defaultFile:tmp.
    ].
!

specificInputFieldsForRequest:aRequest 
    "generates the html-fragment for the maintenance form"

    aRequest 
        inputFieldFor:'ServiceRootDirectory' 
            type:nil 
            size:'50' 
            label:('Service Root Directory:<BR><BLOCKQUOTE>') 
            initialValue:self comancheSTTRoot asString;
        nextPutLine:'</BLOCKQUOTE><BR>';
        inputFieldFor:'DefaultPage' 
            type:nil 
            size:'50' 
            label:('Default Filename:<BR><BLOCKQUOTE>') 
            initialValue:self defaultFile;
        nextPutLine:'</BLOCKQUOTE><BR>'.
! !

!ComancheSTTService methodsFor:'registration'!

registerServiceOn:aServer
    "register myself as comanche swiki service on a single HTTPServer."

    module isNil ifTrue:[
        self setupModule.
    ].
    super registerServiceOn:aServer.
! !

!ComancheSTTService methodsFor:'stt-debugging'!

debugMode:aHTTPRequest 
    ^ aHTTPRequest getSession at:#'stt-debugMode' ifAbsent:[false]
!

isDebugAccept:aHTTPRequest 
    ^ (aHTTPRequest relativePath endsWith:':debugAccept')
!

isDebugRequest:aHTTPRequest 
    ^ (aHTTPRequest relativePath endsWith:':debug')
!

nextPutDebugLinkFor:aHTTPRequest 
    | debugReference |                                  
    debugReference := self linkName,'/:debug?sttFilename=', aHTTPRequest url.
    aHTTPRequest nextPutAll:'<a href="',debugReference , '">View source</a>'
!

processDebugAccept:aHTTPRequest 
    | filenameToDebug filename filecontent fileStream |


    filenameToDebug := aHTTPRequest urlParameterAt:'sttFilename' ifAbsent:[self error:'file not found'].  
    filename := module getTempateOrNilFor:filenameToDebug.

    filecontent := aHTTPRequest urlParameterAt:'stt-source' ifAbsent:[self error:'file not found'].  
    fileStream := filename writeStream.
    fileStream nextPutAll: filecontent.
    fileStream close.
    aHTTPRequest nextPutAll:'<meta http-equiv=refresh content="0; url=',aHTTPRequest serviceLinkName, filenameToDebug, '">'.
    aHTTPRequest sendResponse.
!

processDebugRequest:aHTTPRequest 
    | filenameToDebug source filename filecontent|
    filenameToDebug := aHTTPRequest urlParameterAt:'sttFilename' ifAbsent:[self error:'file not found'].
    filenameToDebug := filenameToDebug copyFrom:self linkName size + 1.
    filename := module getTempateOrNilFor:filenameToDebug.

    filecontent := filename contents asString.
    source := aHTTPRequest class escapeCharacterEntities:filecontent.

    aHTTPRequest nextPutAll:'<form action="', self linkName,'/:debugAccept?', filenameToDebug,'" >'.
    aHTTPRequest nextPutAll:'<textarea name="stt-source" rows=30 cols=100>',source, '</textarea>'.
    aHTTPRequest nextPutAll:'<input type="hidden" name="sttFilename" value="',filenameToDebug ,'"'.
    aHTTPRequest nextPutAll:'<input type="submit" name="accept" value="Accept">'.
    aHTTPRequest nextPutAll:'</form>'.
    aHTTPRequest sendResponse.
!

startDebug:aRequest
"
    Allow the user to edit the stt pages
"
    | aSession |
    aSession :=  aRequest getSession.
    aSession at:#'stt-debugMode' put:true.
! !

!ComancheSTTService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/stt/ComancheSTTService.st,v 1.52 2005/12/01 16:09:24 stefan Exp $'
! !
