"{ Package: 'stx:goodies/webServer/comanche/stt' }"

"{ NameSpace: Comanche }"

Object subclass:#STTModule
	instanceVariableNames:'documentRoot defaultFile templates service'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-STT'
!

!STTModule class methodsFor:'documentation'!

documentation
"
    STT (Smalltalk Templates) support. 
    Enables embedding Smalltalk code in HTML and other documents too.

    I'm a Comanche module that uses STT to serve dynamic content

    [Date:]
        7 July 2002

    [Author:]
        Federico Gregorio Stilman
"
! !

!STTModule class methodsFor:'instance creation'!

on: anExecutionContext withDocumentRoot: aFilePath
	"Creates an instance of the receiver on anExecutionContext that
	will search for STT template files starting at aFilePath"

	^ self new initializeOn: anExecutionContext withDocumentRoot: aFilePath
! !

!STTModule class methodsFor:'class initialization'!

initialize
"/    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
"/        self loadClassesAndExtensions.
"/    ]
!

loadClassesAndExtensions
    (Object implements:#caseError) ifFalse:[
        Smalltalk fileIn:'extensions.st' inPackage:(SwikiModule package).
    ].

    "
     self loadClassesAndExtensions
    "
! !

!STTModule methodsFor:'accessing'!

defaultFile
        ^ defaultFile
!

defaultFile:aFileName
        defaultFile := aFileName
!

documentRoot
	"Return the receiver's root for searching for STT template files"

	^ documentRoot
!

service
    ^ service
!

service:something
    service := something.
!

templates
	"Returns the receiver's dictionary of templates (actually aCache)"
	^ templates
! !

!STTModule methodsFor:'initializing'!

initializeOn: anExecutionContext withDocumentRoot: aFilePath
        "Initialize the receiver on aContext and with STT templates root
        at aFilePath"

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[   
            documentRoot := aFilePath asFilename.
            templates := CacheDictionaryWithFactory
                        new: 500
                        factory: [:filePath | self templateFor:filePath]. 
        ] ifFalse:[
            documentRoot := FileDirectory default directoryNamed: aFilePath.
            templates := Cache
                        size: 500
                        factory: [:filePath | self templateFor:filePath]. 
        ]
!

templateFor:filePath
    |file|

    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
        file := self documentRoot construct: filePath.
        [
            ^ STTTemplate on:(file contentsOfEntireFile)
        ] on:FileStream openErrorSignal do:[:ex| ^ nil].
    ] ifFalse:[
        file := self documentRoot readOnlyFileNamed: filePath.
        ^ STTTemplate on:(file contents asString)
    ].
! !

!STTModule methodsFor:'kom processing'!

getTempateOrNilFor: aFilePath
        "Process request, expected to point to an STT template resource"

        |path filePath|

        filePath := aFilePath.
        (filePath startsWith:'/') ifTrue:[
            filePath := filePath copyFrom:2
        ].

        path := documentRoot construct:filePath.

        (path isReadable not or:[path isDirectory]) ifTrue:[
            ^ nil
        ].

        ^ path asFilename       
!

process: request
        "Process request, expected to point to an STT template resource"

        |path filePath template relativePath comancheModel templates|

        relativePath := request relativePath.
        relativePath isEmpty ifTrue:[
            filePath := self defaultFile.
        ] ifFalse:[
            filePath := (request relativePath asCollectionOfSubstringsSeparatedBy:$,) first.
        ].
        (filePath startsWith:'/') ifTrue:[
            filePath := filePath copyFrom:2
        ].

        path := documentRoot construct:filePath.

        (path isReadable not or:[path isDirectory]) ifTrue:[
            filePath := self defaultFile.
            path := documentRoot construct:filePath.
            path isReadable ifFalse:[
                request reportNotFound:(request service linkName , relativePath).
                ^ self
            ].
            request nextPutAll:'<META http-equiv=refresh content="0; url=';
                nextPutAll:((request linkIdFor:request service), '/', filePath);
                nextPutAll:'" />'.
            request response send.
            ^ self

"/            request addToReplyHeader:('Content-Location: http://' , request serverHostName , (request linkIdFor:comancheModel service)  , '/' , filePath).
        ].
        templates := self templates.
        template := templates at:filePath.
        (template notNil and:[template isOlderThanFile:path]) ifTrue:[
            templates removeKey:filePath.
            template := templates at:filePath.
        ].
        template isNil ifTrue:[
            request reportNotFound:filePath.
            ^ self.
        ].
        
        "/ use new comancheModel for each request, to make service thread safe

        comancheModel := STTComancheModel newOn:nil "executionContext".
        comancheModel request: request;
                      sttFilePath: path;
                      service:service.

        ^ template evaluateFor:comancheModel.
! !

!STTModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/stt/STTModule.st,v 1.30 2007/02/09 13:28:04 sr Exp $'
! !

STTModule initialize!
