"{ Package: 'stx:goodies/webServer/comanche/stt' }"

"{ NameSpace: Comanche }"

Object subclass:#STTTemplate
	instanceVariableNames:'sttCode cache cacheTimestamp compiledTemplate asStringSelector'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-STT'
!

!STTTemplate class methodsFor:'documentation'!

documentation
"
    STT (Smalltalk Templates) support. 
    Enables embedding Smalltalk code in HTML and other documents too.

    I'm a Smalltalk Template. 
    My escaping sequence for embedding Smalltalk expressions is: {% %} or {%= %}.

    CG:
        I changed these to <?stt ... ?>
        and                <?stt= ... ?>

    [Date:]
        7 July 2002

    [Author:]
        Federico Gregorio Stilman
"
!

examples
"
  Try these expressions:

    (STTTemplate on: 'Repeating word...<?stt 10 timesRepeat: [ ?> WORD <?stt ] ?>   end!!') evaluateFor: nil.

    (STTTemplate on: 'Counting: 
                    <?stt self to: 10 do: [ :each | ?>
                            [<?stt = each ?>]
                    <?stt ] ?>'
    ) evaluateFor: 1.


  obsolete (non xml-compliant):
    (STTTemplate on: 'Repeating word...{% 10 timesRepeat: [ %} WORD {% ] %}   end!!') evaluateFor: nil.

    (STTTemplate on: 'Counting: 
                    {% self to: 10 do: [ :each | %}
                            [{%= each %}]
                    {% ] %}'
    ) evaluateFor: 1.



  checking documents:
    XML::XMLParser
        processDocumentInFilename:'../../goodies/webServer/data/comancheSTT/test1.stt'
        beforeScanDo:[:p | p validate:false]
    XML::XMLParser
        processDocumentInFilename:'../../goodies/webServer/data/comancheSTT/test4.stt'
        beforeScanDo:[:p | p validate:false]

"
! !

!STTTemplate class methodsFor:'instance creation'!

on: aString
	"Creates an instance of the receiver on aString"

	^ self on: aString asStringSelector: self defaultAsStringSelector
!

on: aString asStringSelector: aSymbol
	"Creates an instance of the receiver on aString"

	^ self new initializeOn: aString asStringSelector: aSymbol
! !

!STTTemplate class methodsFor:'defaults'!

defaultAsStringSelector
	^ #asString
! !

!STTTemplate methodsFor:'accessing'!

asStringSelector
	"Returns the selector used to show objects as Strings on the receiver"

	^asStringSelector
!

asStringSelector: aSymbol
	"Sets the selector used to show objects as Strings on the receiver"

	asStringSelector _ aSymbol.
!

sttCode
	"Returns the receiver's Smalltalk Template code"

	^ sttCode
! !

!STTTemplate methodsFor:'caching'!

cache
	"Returns the receiver's cached object"

	^ cache
!

cache: anObject
    "Save anObject in the receiver's cache"

    cache := anObject.
    cacheTimestamp := Timestamp now.
!

flushIfOlderThanFile:aFile
    cacheTimestamp isNil ifTrue:[^ self].

    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
        cacheTimestamp < aFile modificationTime ifTrue:[
            self initializeCache
        ].
    ] ifFalse:[
        self shouldImplement
    ].
!

initializeCache
        "Initialize the receiver's cache"
        cache := nil.
        cacheTimestamp := nil.
!

isCached
        "Tell if the receiver is cached or not. In the future,
        this will consider the fact that a cached object may
        become old after some time, and that means that the
        object is NOT cached anymore."

        ^ self cache notNil
!

isOlderThanFile:aFile
    cacheTimestamp isNil ifTrue:[^ true].
    cache isNil ifTrue:[^ true].

    (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
        aFile isReadable ifFalse:[^ true].
        ^ cacheTimestamp < aFile modificationTime 
    ] ifFalse:[
        self shouldImplement.
    ].
    ^ true.
! !

!STTTemplate methodsFor:'code generation'!

asSmalltalkCode:sttCode model:commancheModel
        "Returns the equivalent version of the receiver as Smalltalk code"

        |sttOpen sttClose sttOpenIndex sttCloseIndex 
         exprIsString lastIndex sttCodeIndex stream smalltalkExpression
         nextChar exprIsInclude|

        stream := String new writeStream.
        lastIndex := 1.

        "/ old - NOT xml compliant
"/        sttOpen := '{%'.    sttClose := '%}'.

        "/ changed to:
        sttOpen := '<?stt'. sttClose := '?>'.

        [ 
            sttOpenIndex := sttCode indexOfSubCollection:sttOpen startingAt: lastIndex.
            sttOpenIndex > 0
        ] whileTrue: [       
            self writeOutputCodeFor: (sttCode copyFrom: lastIndex to: sttOpenIndex-1) on: stream.

            sttCloseIndex := sttCode
                                    indexOfSubCollection:sttClose
                                    startingAt: sttOpenIndex
                                    ifAbsent: [ ^ self error: 'Missing closing tag' ].

            sttCodeIndex := sttOpenIndex + sttOpen size.

            [ (sttCode at:sttCodeIndex) isSeparator ] whileTrue:[ sttCodeIndex := sttCodeIndex + 1 ].

            nextChar := sttCode at:sttCodeIndex.
            exprIsString := (nextChar == $=).
            exprIsString ifTrue:[
                stream nextPutAll: 'request nextPutAll: ('.                     
                sttCodeIndex := sttCodeIndex + 1.
            ].                                                             

            "<stt include extention 29:10:2003 - james>"
            "
                <?stt # ('file', 'nameToInclude') ?>
            "
            exprIsInclude := (nextChar == $# and:[(sttCode at:sttCodeIndex+1) == Character space]).
            exprIsInclude ifTrue:[| sttString smalltalkGeneratedCode|     
                sttCodeIndex := sttCodeIndex + 1.
                smalltalkExpression := (sttCode copyFrom: sttCodeIndex to: sttCloseIndex - 1) withBlanksTrimmed.
                sttString :=(Parser evaluate:smalltalkExpression in:nil receiver:commancheModel notifying:nil ifFail:[])asString.
                smalltalkGeneratedCode := (STTTemplate on:'<?stt [?>', sttString, '<?stt ] value?>')asSmalltalkCodeWithModel:commancheModel.
                stream nextPutAll: smalltalkGeneratedCode.
            "</stt include extention 29:10:2003 - james>"
            ] ifFalse:[
                smalltalkExpression := (sttCode copyFrom: sttCodeIndex to: sttCloseIndex - 1) withBlanksTrimmed.
                stream nextPutAll: smalltalkExpression.
            ].

            exprIsString
                ifTrue: [
                    stream nextPutAll: ') ', self asStringSelector asString.
                    sttCodeIndex := sttCodeIndex + 1.
                ].                                              

            (smalltalkExpression withoutTrailingSeparators endsWithAnyOf: #('|', '[', '(', '{', '.'))
                ifFalse: [ 
                    stream nextPut: $.
                ].

            stream cr.
            lastIndex := sttCloseIndex + sttClose size.                  
        ].

        self writeOutputCodeFor: (sttCode copyFrom: lastIndex to: sttCode size) on: stream.

        ^ stream contents
!

asSmalltalkCodeWithModel:aModel
        "Returns the equivalent version of the receiver as Smalltalk code"

        ^  self asSmalltalkCode:(self sttCode) model:aModel
!

writeOutputCodeFor:aString on:aStream
    "Writes on aStream the required Smalltalk code for sending aString as reply"

    aString size == 0 ifTrue:[^ self].

    aStream 
        nextPutAll: 'request nextPutAll: ';
        nextPutAll: aString storeString;
        nextPutAll: '.'; cr.
! !

!STTTemplate methodsFor:'compiling'!

compileFor: aComancheModel
        "Compiles the receiver's template in aClass and returns aCompiledMethod"

        | compiler methodNode |

        compiler := Compiler new.
        compiler smalltalkDialect:(aComancheModel service smalltalkDialect).
        methodNode := compiler
                            compileNoPattern: (self asSmalltalkCodeWithModel:aComancheModel)
                            in: aComancheModel class
                            context: nil
                            notifying: nil
                            ifFail: [ self error:'STT: cannot compile' ].

        ^ methodNode generate: #(0 0 0 0)
! !

!STTTemplate methodsFor:'evaluating'!

evaluateFor: aComancheModel
        "Evaluates the receiver within for anObject"

        self isCached ifFalse: [ 
            self cache: (self compileFor: aComancheModel).
        ].
        ^ self cache valueWithReceiver: aComancheModel arguments: #().
! !

!STTTemplate methodsFor:'initializing'!

initializeOn: aString asStringSelector: aSymbol
	sttCode _ aString.
	asStringSelector _ aSymbol.

	self initializeCache.
! !

!STTTemplate class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/stt/STTTemplate.st,v 1.27 2004/09/16 13:07:47 ca Exp $'
! !
