"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

StringHolder subclass:#ComSwikiLauncher
	instanceVariableNames:'package port ports isShutdown startStopButton portButton'
	classVariableNames:'ValidPorts'
	poolDictionaries:''
	category:'Comanche-Swiki-Comanche'
!

ComSwikiLauncher comment:'ComSwikiLauncher

This class provides a morphic interface to launch Swiki on Comanche.
In Morphic, doIt: (ComSwikiLauncher openAsMorph)
In MVC, doIt: (ComSwikiLauncher openMVCView)

BUTTON EXPLANATION
start/stop server button
	press on this button to start or stop the Swiki server on the given port.
save & exit button
	once everything is set up, this button will save and quit and turn
	the swiki on at the next reboot. This is not to be confused with ''save
	and quit'' which does not restart the server.
port button
	press on this when the server is inactive to change the port number.
	Note: 80 is the default port for a web server, but many systems will
		not allow a user to use it (Windows NT, Unix)

TO EDIT SWIKI
start the server
bring up ComSwiki Launcher''s menu
	press on the button next to X
chose ''open swiki browser...''
'
!


!ComSwikiLauncher class methodsFor:'instance creation'!

openAsMorph
	^self openAsMorphPackage: #swikiWebServer
!

openAsMorphPackage: package
	| instance window aColor startStopButton shutdownButton portButton |

	"Instance of ComSwiki Launcher"
	instance _ self new.
	instance package: package.
	"Display Morph"
	window _ (SystemWindow labelled: 'ComSwiki Launcher') model: instance.
	aColor _ Color colorFrom: instance defaultBackgroundColor.
	"Start/Stop Button"
	startStopButton _ PluggableButtonMorph on: instance getState: #isOn action: #toggleState.
	instance startStopButton: startStopButton.
	startStopButton askBeforeChanging: true.
	startStopButton color: aColor; onColor: (Color red) offColor: (Color green).
	window addMorph: startStopButton frame: (0@0 corner: 0.35@1).
	"Shutdown Button"
	shutdownButton _ PluggableButtonMorph on: instance getState: #isShutdown action: #shutdown.
	shutdownButton label: 'save & exit'; askBeforeChanging: true.
	shutdownButton color: aColor; onColor: aColor darker offColor: aColor.
	window addMorph: shutdownButton frame: (0.35@0 corner: 0.7@1).
	"Port Button"
	portButton _ PluggableButtonMorph on: instance getState: #isPort action: #changePort.
	instance portButton: portButton.
	portButton askBeforeChanging: true.
	portButton color: aColor; onColor: aColor offColor: aColor.
	window addMorph: portButton frame: (0.7@0 corner: 1@1).
	"Add to World"
	window openInWorld.
!

openMVCView
	^self openMVCViewPackage: #swikiWebServer
!

openMVCViewPackage: package
	"ComSwikiLauncher openMVCViewPackage: #swikiWebServer."
	| instance window aColor startStopButton shutdownButton portButton openSwikiBrowserButton |

	"Instance of ComSwiki Launcher"
	instance _ self new.
	instance package: package.
	"Display View"
	window _ StandardSystemView new model: instance.
	window label: 'ComSwiki Launcher'.

	aColor _ Color colorFrom: instance defaultBackgroundColor.
	"Start/Stop Button"
	startStopButton _ PluggableButtonView on: instance getState: #isOn action: #toggleState label: #startStopLabel.
	instance startStopButton: startStopButton.
	startStopButton askBeforeChanging: true.
	startStopButton
		backgroundColor: aColor;
		borderWidth: 1.
		"; onColor: (Color red) offColor: (Color green)."
	window addSubView: startStopButton.

	"Shutdown Button"
	shutdownButton _ PluggableButtonView on: instance getState: #isShutdown action: #shutdown.
	shutdownButton label: 'save & exit'; askBeforeChanging: true.
	shutdownButton
		backgroundColor: aColor;
		borderWidth: 1.
		"; onColor: aColor darker offColor: aColor."
	window addSubView: shutdownButton toRightOf: startStopButton.

	"Port Button"
	portButton _ PluggableButtonView on: instance getState: #isPort action: #changePort label: #portText.
	instance portButton: portButton.
	portButton askBeforeChanging: true.
	portButton
		backgroundColor: aColor;
		borderWidth: 1.
		"; onColor: aColor offColor: aColor."
	window addSubView: portButton toRightOf: shutdownButton.

	"OpenSwikiBrowserButton"
	openSwikiBrowserButton _ PluggableButtonView on: instance getState: nil action: #openSwikiBrowser.
	openSwikiBrowserButton label: 'SwikiBrowser'.
	openSwikiBrowserButton
		backgroundColor: aColor;
		borderWidth: 1.
	window addSubView: openSwikiBrowserButton toRightOf: portButton.


	"open window"
	window controller open
! !

!ComSwikiLauncher class methodsFor:'initialize-release'!

initialize
	ValidPorts _ OrderedCollection new.
	ValidPorts add: 80.
	ValidPorts add: 8000.
	ValidPorts add: 8080.
	ValidPorts add: 8888.
! !

!ComSwikiLauncher class methodsFor:'private'!

shutdown
	self allInstances do: [:launcher |
		launcher isShutdown: true.
		launcher isOn ifTrue: [launcher stopServer]].
	Smalltalk snapshot: true andQuit: true.
	self allInstances do: [:launcher |
		launcher startServer.
		launcher isShutdown: false]
! !

!ComSwikiLauncher class methodsFor:'utility'!

convertOldATA
	"Convert Addresses, Templates, and Actions from Beta to Release"
	| dir bookOrShelfDir |
	dir _ FileDirectory default directoryNamed: 'swiki'.
	dir directoryNames do: [:dName |
		bookOrShelfDir _ dir directoryNamed: dName.
		(bookOrShelfDir directoryExists: 'addresses') ifTrue: [
			self convertOldATADirectory: (bookOrShelfDir directoryNamed: 'addresses')].
		(bookOrShelfDir directoryExists: 'templates') ifTrue: [
			self convertOldATADirectory: (bookOrShelfDir directoryNamed: 'templates')].
		(bookOrShelfDir directoryExists: 'actions') ifTrue: [
			self convertOldATADirectory: (bookOrShelfDir directoryNamed: 'actions')]]
!

convertOldATADirectory: dir
	| id xmlMeta subDir |
	id _ 1.
	xmlMeta _ WriteStream on: String new.
	xmlMeta
		nextPutAll: '<?xml version="1.0"?>';
		nextPut: Character cr;
		nextPutAll: '<meta>';
		nextPut: Character cr.
	dir fileNames do: [:fName |
		((fName endsWith: '.book') or: [fName endsWith: '.page']) ifTrue: [
			xmlMeta
				nextPutAll: '<file id="';
				nextPutAll: id asString;
				nextPutAll: '" type="';
				nextPutAll: (fName copyAfterLast: $.);
				nextPutAll: '" name="';
				nextPutAll: (fName copyUpTo: $.);
				nextPutAll: '" />';
				nextPut: Character cr.
			dir rename: fName toBe: id asString.
			id _ id + 1]].
	dir directoryNames do: [:dName |
		subDir _ dir directoryNamed: dName.
		subDir fileNames do: [:fName |
			((fName endsWith: '.book') or: [fName endsWith: '.page']) ifTrue: [
				xmlMeta
					nextPutAll: '<file id="';
					nextPutAll: id asString;
					nextPutAll: '" type="';
					nextPutAll: (fName copyAfterLast: $.);
					nextPutAll: '" name="';
					nextPutAll: (fName copyUpTo: $.);
					nextPutAll: '" category="';
					nextPutAll: dName;
					nextPutAll: '" />';
					nextPut: Character cr.
				dir rename: (subDir fullNameFor: fName) toBe: id asString.
				id _ id + 1]].
		dir deleteDirectory: dName].
	xmlMeta
		nextPutAll: '</meta>'.
	(dir newFileNamed: 'meta.xml')
		nextPutAll: xmlMeta contents;
		close
!

convertOldPages
	"Conversion method used to convert old Swikis (pre-Beta 11) to the new file format.
	This is to be run after the new server is up and running."
	| oldAlerts newAlerts |
	NuSwikiPage allInstances do: [:page |
		(page settingsIncludes: 'alerts')	ifTrue: [
			oldAlerts _ page settingsAt: 'alerts'.
			newAlerts _ ''.
			(oldAlerts isKindOf: String) ifFalse: [
				oldAlerts do: [:address | newAlerts _ newAlerts, ', ', address].
				newAlerts _ newAlerts copyFrom: 3 to: (newAlerts size).
				page settingsAt: 'alerts' put: newAlerts]].
		page text: (page text).
		page write]
! !

!ComSwikiLauncher methodsFor:'accessing'!

isShutdown
	^isShutdown ifNil: [false] ifNotNil: [isShutdown]
!

isShutdown: boolean
	(self isShutdown = boolean) ifFalse: [
		isShutdown _ boolean.
		self changed: #isShutdown]
!

package: aSymbol
	package _ aSymbol
!

port: anInteger
	port _ anInteger.
	portButton label: self portText.
	self changed: #portText
!

ports: col
	ports _ col.
	port _ col at: 1.
! !

!ComSwikiLauncher methodsFor:'action'!

changePort
	| index |

	self isOn ifFalse: [index _ 1.
		1 to: (ports size) do: [:i | (ports at: i) = port ifTrue: [index _ i + 1]].
		(ports size < index) ifTrue: [index _ 1].
		self port: (ports at: index)]
!

isOn
	^(ComancheServer serverOnPort: port ifAbsent: []) notNil
!

isPort
	^false
!

openSwikiBrowser
	| shelf label |
	label := port = 80
				ifTrue: ['Swiki Browser']
				ifFalse: ['Swiki Browser: ' , port asString].
	shelf := (ComancheServer
				serverOnPort: port
				ifAbsent: [Smalltalk isMorphic
						ifFalse: [PopUpMenu notify: 'Start the server'].
					^ self]) serviceProvider module shelf.
	Smalltalk isMorphic
		ifTrue: [(SwikiBrowser asMorphOnShelf: shelf label: label) openInWorld]
		ifFalse: [SwikiBrowser asMVCOnShelf: shelf label: label]
!

shutdown
	self class shutdown
!

startServer
	"To Start Swiki on Comanche:"
	| module |
	Socket initializeNetwork.
	module _ SwikiModule perform: package.
	module ifNil: [self error: 'Did not start!!'].
	(ComancheNetService named: ((package asString), '-', (port asString)) onPort: port)
		module: "(HttpSessionModule module: "module")";
		start.
	startStopButton label: self startStopLabel.
	self changed: #startStopLabel.
	self changed: #isOn
!

stopServer
	"To End Swiki on Comanche:"
	| che shelf |
	Smalltalk garbageCollect.
	shelf _ (ComancheServer serverOnPort: port ifAbsent: [nil]) serviceProvider "module" module shelf.
	((SwikiBrowser allInstances select: [:browser | browser shelf = shelf]) size > 0)
		ifTrue: [PopUpMenu notify: 'Before stopping the server, you
need to close the Swiki Brower'.
			^self].
	ComancheNetService removeServiceNamed: ((package asString), '-', (port asString)).
	(che _ ComancheServer serverOnPort: port ifAbsent: [])
		ifNotNil: [che stop].
	Smalltalk garbageCollect.
	startStopButton label: self startStopLabel.
	self changed: #startStopLabel.
	self changed: #isOn
!

toggleState
	(self isOn)
		ifTrue: [self stopServer]
		ifFalse: [self startServer]
! !

!ComSwikiLauncher methodsFor:'buttons'!

portButton: aMorph
	portButton _ aMorph.
	portButton label: self portText
!

portText
	^'port: ', (port asString)
!

startStopButton: aMorph
	startStopButton _ aMorph.
	startStopButton label: self startStopLabel
!

startStopLabel
	^(self isOn)
		ifTrue: ['stop server']
		ifFalse: ['start server']
! !

!ComSwikiLauncher methodsFor:'initialize-release'!

initialize
	self ports: ValidPorts.
	super initialize.
! !

!ComSwikiLauncher methodsFor:'looks'!

defaultBackgroundColor
	^Color yellow
!

initialExtent
	^300@50
! !

!ComSwikiLauncher methodsFor:'window menu'!

addModelItemsToWindowMenu: aMenu
	(self isOn) ifTrue: [
		aMenu addLine.
		aMenu add: 'open swiki browser...' target: self action: #openSwikiBrowser].
	^super addModelItemsToWindowMenu: aMenu
! !

!ComSwikiLauncher class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/ComSwikiLauncher.st,v 1.3 2003/08/29 17:05:00 cg Exp $'
! !

ComSwikiLauncher initialize!
