"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Dictionary subclass:#HttpFormDictionary
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Kom-Protocol-HTTP'
!

!HttpFormDictionary class methodsFor:'documentation'!

documentation
"
    kom/4.12 (Comanche/4.12)

    HttpFormDictionary (kom/4.10; bolot 6/22/2001 12:09)
        - use this instead of plain Dictionary for GET and POST forms
        - maintains multiple values for the same field
        -- but is backwards compatible, #at: returns the first value (?)
        -- so do #booleanAt:, #numberAt:
        - to access the actual value at key, use #rawAt:
        -- returns an OrderedCollection

    TODO:
        - file upload support
        -- idea: first value is file name, second is FileStream?
"
! !

!HttpFormDictionary methodsFor:'accessing'!

at: key
	^ self at: key ifAbsent: [self errorKeyNotFound]
!

at: key ifAbsent: aBlock
        | value valueSize |

        value := super at: key ifAbsent: [^aBlock value].
        valueSize := value size.
        0 = valueSize ifTrue:["is this not absent?"
                value := aBlock value]. 
        1 = valueSize 
                ifTrue:[value := value first]
                ifFalse:[value := value copy].
        ^value
!

at: key put: value
        (self includesKey: key)
                ifFalse: [super at: key put: (OrderedCollection new)].

        (self rawAt: key) add: value.

        ^value
!

booleanAt: key
	^(self rawAt: key) first = 'true'
!

booleansAt: key
        true "(Smalltalk dialectName = 'SmalltalkX')" ifTrue:[          "this is a BUG fix - not ST/X specific"
            ^(self rawAt: key) collect: [:value | value = 'true']
        ].
"        ^(self rawAt: key) collect: [:value | value == 'true'] "
!

numberAt: key
	^[(self rawAt: key) first asNumber]
		ifError: [:err :rcvr | Float nan]
!

numbersAt: key
	^(self rawAt: key) collect: [:value |
		[value asNumber] ifError: [:err :rcvr | Float nan]]
!

rawAt: key
	^super at: key ifAbsent: [self errorKeyNotFound]
!

rawAt: key ifAbsent: aBlock
	^super at: key ifAbsent: aBlock
!

stringAt: key
	^(self rawAt: key) first
!

stringsAt: key
	^(self rawAt: key) copy
! !

!HttpFormDictionary methodsFor:'printing'!

printElementsOn: aStream
	aStream nextPut: $(.
	self keysSortedSafely do:
		[:key | aStream print: key; nextPutAll: '->'; print: (self rawAt: key); space].
	aStream nextPut: $)
! !

!HttpFormDictionary class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/HttpFormDictionary.st,v 1.14 2003/09/11 12:47:54 cg Exp $'
! !
