"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

NetworkRequest subclass:#HttpRequest
	instanceVariableNames:'url stream header rawUrl stxHttpRequest'
	classVariableNames:'EndOfRequestHeaderMarker HttpStatusCodeDict'
	poolDictionaries:''
	category:'Comanche-Kom-Protocol-HTTP'
!

HttpRequest comment:'kom/4.12 (Comanche/4.12)
bolot 2/20/2002 13:53
- rawUrl = the entire request string
- url = rawUrl up to ?
- queryString = rawUrl after the first ?
- rawUrl == url?queryString

HttpRequest (bolot 4/2/2001 18:51)
- HTTP request object wrapper
- handles details of HTTP
-- headers, formats, etc.
- as of kom47, handles multipart posts
- in kom49 (or kom50) a minor refactoring will happen'
!


!HttpRequest class methodsFor:'instance creation'!

readFromStream: aStream

	^self new initializeFromStream: aStream
! !

!HttpRequest class methodsFor:'accessing'!

endOfRequestHeaderMarker
	"This marks the end of an HTTP request (two consecutive CR-LFs)"

	^EndOfRequestHeaderMarker
! !

!HttpRequest class methodsFor:'authentication'!

hashPassword: aPassword
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            |secureHashAlgorithm|

            secureHashAlgorithm := SHA1Stream ? MD5Stream.
            ^ secureHashAlgorithm hashValueOf:(aPassword, ':', self secretWord)
        ].
        ^SecureHashAlgorithm new hashMessage:
                (aPassword, ':', self secretWord)
!

secretWord
	"a string used in hashing passwords"
	^'ab74uihagAS'
! !

!HttpRequest class methodsFor:'defaults'!

defaultContentType

	^MIMEDocument defaultContentType
!

defaultMethod

	^'GET'
!

defaultProtocol

	^'HTTP/1.0'
!

defaultUrl

	^''
! !

!HttpRequest class methodsFor:'initialize-release'!

initialize
	EndOfRequestHeaderMarker _ String crlf, String crlf
! !

!HttpRequest methodsFor:'accessing'!

contentLength

	^self propertyAt: #contentLength ifAbsentPut: [
		(header at: 'content-length' ifAbsent: [0]) asNumber
	]
!

contentType
	| i contentType |
	^self propertyAt: #contentType ifAbsentPut: [
		contentType _ self header at: 'content-type' ifAbsent: [self defaultContentType].
		contentType ifNotNil: [i _ contentType indexOf: $;. "get rid of charset field"
			i > 0 ifTrue: [contentType _ contentType copyFrom: 1 to: i-1]].
		contentType asLowercase]
!

cookies
	"answer a dictionary with the cookies in the request"
	^ self
		propertyAt: #cookies
		ifAbsentPut: [self
				parseCookies: (self header
						at: 'cookie'
						ifAbsent: [''])]
!

endOfRequestHeaderMarker
	^self class endOfRequestHeaderMarker
!

getFields
	"retrieve fields encoded in the URL: http://aha/ha?a=1&b=2"
	| in |
	^self propertyAt: #getFields ifAbsentPut: [
		((in _ rawUrl indexOf: $?) > 0) ifTrue: [
			HttpParser decodeUrlEncodedForm: 
				(rawUrl copyFrom: (in + 1) to: rawUrl size)
				multipleValues: self multiValueFormFields
		] ifFalse: [
			self multiValueFormFields
				ifTrue: [HttpFormDictionary new]
				ifFalse: [Dictionary new].
		].
	]
!

header
	^header
!

header: aDict
	header _ aDict
!

host
	^self propertyAt: #host ifAbsentPut: [self header at: 'host' ifAbsent: []]
!

localAddress
	"Answer the address of the request originator"
	^self propertyAt: #localAddress ifAbsentPut: [
		(stream respondsTo: #localAddress) ifTrue: [
			stream localAddress
		] ifFalse: [
			nil
		]
	]
!

localPort
	"Answer the address of the request originator"
	^self propertyAt: #localPort ifAbsentPut: [
		(stream respondsTo: #localPort) ifTrue: [
			stream localPort
		] ifFalse: [
			nil
		]
	]
!

method
	^self propertyAt: #method ifAbsent: ['GET']
!

method: aString
	self propertyAt: #method put: aString
!

multiValueFormFields
	^self propertyAt: #multiValueFormFields ifAbsent: [false]
!

multiValueFormFieldsDisable
	"disable use of multi-value form fields"
	"thus, if the status changed, need to expunge GET/POST fields if already parsed"
	self multiValueFormFields
		ifTrue: [self propertyAt: #multiValueFormFields put: false.
			self removePropertyAt: #getFields ifAbsent: [].
			self removePropertyAt: #postFields ifAbsent: [].			
			]
!

multiValueFormFieldsEnable
	"enable use of multi-value form fields"
	"thus, if the status changed, need to expunge GET/POST fields if already parsed"
	self multiValueFormFields
		ifFalse: [self propertyAt: #multiValueFormFields put: true.
			self removePropertyAt: #getFields ifAbsent: [].
			self removePropertyAt: #postFields ifAbsent: [].			
			]
!

postFields
	"Answer the multipart fields for a post request (if in fact this is a
	POST request"

	^self propertyAt: #postFields ifAbsentPut: [
		" parse POST fields in the body of request, if it's standard form data "

		(self isPostRequest and: [ 
			self contentType = MIMEDocument contentTypeFormData 
		]) ifTrue: [
			HttpParser
				decodeUrlEncodedForm: self rawPostFields
				multipleValues: true.
		] ifFalse: [
			Dictionary new
		].
	]
!

protocol
	^self propertyAt: #protocol ifAbsent: [self defaultProtocol]
!

queryString

	^self propertyAt: #queryString ifAbsent: [ nil ]
!

queryString: aString

	^self propertyAt: #queryString put: aString
!

rawPostFields
        "save the POST form fields as is, for future processing, see #postFields"
        "reasons: 1) lazy form processing;
                2) allow the app to specify whether it wants multiple values for fields"

        ^self propertyAt: #rawPostFields ifAbsentPut: [

                (self isPostRequest and: [ 
                        self contentType = MIMEDocument contentTypeFormData 
                ]) ifTrue: [
                        stream next: self contentLength
                ] ifFalse: [
                        ''
                ].
        ]
!

rawUrl
	^rawUrl
!

referer
	^self propertyAt: #referer ifAbsentPut: [
		header at: 'referer' ifAbsent: [ nil ]
	]
!

remoteAddress
	^self propertyAt: #remoteAddress ifAbsent: [nil]
!

remoteAddress: aRemoteAddress
	self propertyAt: #remoteAddress put: aRemoteAddress
!

stream
	^stream
!

stream:something
    stream := something.
!

stxHttpRequest
    ^ stxHttpRequest
!

stxHttpRequest:something
    stxHttpRequest := something.
!

timestamp

	^self propertyAt: #timestamp ifAbsent: [ nil ]
!

timestamp: aTimestamp

	self propertyAt: #timestamp put: aTimestamp
!

url
	^url
!

url: aUrl
	url _ aUrl
!

user
	"currently, only basic authentication is supported"
	| userID i |
	userID _ self header at: 'authorization' ifAbsent: [^nil].
	(userID notNil and: [(i _ userID findString: 'Basic ') > 0])
		ifTrue: [userID _ userID copyFrom: i+6 to: userID size]
		ifFalse: [userID _ nil].
	^userID
! !

!HttpRequest methodsFor:'connection mgt'!

isPersistentConnection
	| connectionHeader |
	^self propertyAt: #persistentConnection ifAbsentPut:
		[connectionHeader _ self header at: 'connection' ifAbsent: [nil].
		connectionHeader
			ifNil: [false]
			ifNotNil: [connectionHeader asLowercase = 'keep-alive']
		]
! !

!HttpRequest methodsFor:'defaults'!

defaultContentType

	^self class defaultContentType
!

defaultMethod

	^self class defaultMethod
!

defaultProtocol

	^self class defaultProtocol
!

defaultUrl

	^self class defaultUrl
! !

!HttpRequest methodsFor:'initialize-release'!

destroy
	stream close.
	stream _ nil
!

initRemoteAddress
	(stream respondsTo: #remoteAddress) ifTrue: [
		self remoteAddress: stream remoteAddress.
	]
!

initStatusString: aString
        | statusLine |
        statusLine := aString findTokens: ' '.
        (statusLine size = 3 or: [statusLine size = 2])
                ifTrue: [self method: statusLine first.
                        rawUrl := statusLine second.
                        url := rawUrl copyUpTo: $?.
                        url size + 1 < rawUrl size
                                ifTrue: [self queryString: (rawUrl copyFrom: (url size + 2) to: rawUrl size)].
                        "self protocol: statusLine third"]
                ifFalse: [^nil]
!

initUrl: aString
    rawUrl := aString.
    url := rawUrl copyUpTo: $?.
    url size + 1 < rawUrl size
            ifTrue: [self queryString: (rawUrl copyFrom: (url size + 2) to: rawUrl size)].
!

initializeFromStream: aStream

        stream _ aStream.
        self initRemoteAddress.
        self timestamp: Time totalSeconds.

        (self readRequestHeaderFrom: stream)
                ifNil: [^nil].
        ^ self

"/        [(self readRequestHeaderFrom: stream)
"/                ifNil: [^nil]
"/        ] ifError: [:err :rcvr | ^nil]
!

readRequestHeaderFrom: aStream
        | reqHeader |

        (reqHeader _ [aStream upToAll: self endOfRequestHeaderMarker]
                        ifError: [:err :rcvr | ''])
                isEmpty ifTrue: [^nil].
        aStream match: self endOfRequestHeaderMarker.

        reqHeader _ BufferStream on: reqHeader.
        (self initStatusString: (reqHeader upToAll: String crlf))
                ifNil: [^nil].
        reqHeader match: String crlf.

        self header: (HttpParser parseHttpHeader: reqHeader upToEnd).
        self propertyAt: #headerBytes put: stream inBytes.

        self rawPostFields. "force reading/parsing post fields"
! !

!HttpRequest methodsFor:'multipart forms'!

multipartBoundary
	| str idx |
	"boundary specification in the HTTP header looks like:
	Content-Type: multipart/form-data; boundary=BOUNDARY "

	^self propertyAt: #multipartBoundary ifAbsentPut: [
		(self contentType beginsWith: MIMEDocument contentTypeMultipart)
			ifTrue: ["contentType gets lowercased, so access the raw header"
				str _ header at: 'content-type' ifAbsent: [ '' ].
				idx _ str indexOf: $=.
				'--', (str copyFrom: idx + 1 to: str size)]
			ifFalse: [ nil ]]
!

multipartFormFieldsDo: aBlock
        "USAGE:
        request multipartFormFieldsDo: [:chunk |
                chunk saveToStream: aStream]."
        "NOTE: if the chunk is not saved, save it after aBlock"

        | mChunk aFieldValue |
        (self method = 'POST' and: [self contentType = MIMEDocument contentTypeMultipart])
                ifFalse: [^nil].

        [stream match: self multipartBoundary.
        stream match: '--']
        whileFalse:
        [
        stream match: String crlf.

        mChunk _ self nextChunkHeader.
        aBlock value: mChunk.
"        mChunk flag: #todo. review chunk saving logic "
        mChunk isSaved
                ifFalse: [
                        mChunk fileName isEmptyOrNil
                                ifFalse: [mChunk saveToStream: (DummyStream on: String new)]
                                ifTrue: [
                                        aFieldValue _ String streamContents: [:aStream |
                                                mChunk saveToStream: aStream].
                                        self postFields at: mChunk fieldName put: aFieldValue
                                ]
                ].

        ]
!

nextChunkHeader
        "Read the next multipart data chunk's header"

        | string dict mChunk |
        string _ stream upToAll: self endOfRequestHeaderMarker.
        stream match: self endOfRequestHeaderMarker.

        dict _ HttpParser parseHttpHeader: string.
        mChunk _ MultipartChunk from: dict.
        mChunk request: self.
        mChunk multipartBoundary: self multipartBoundary.
        ^mChunk
! !

!HttpRequest methodsFor:'printing'!

printOn: printStream
        printStream nextPutAll: self class name;
                nextPutAll: ' (URL=', self url asString;
                nextPutAll: '; protocol=', self protocol asString.
                (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                    printStream nextPutAll: '; header=', self header printString;
                            nextPutAll: '; getFields=', self getFields printString;
                            nextPutAll: '; postFields=', self postFields printString.
                ] ifFalse:[
                    printStream nextPutAll: '; header=', self header asString;
                            nextPutAll: '; getFields=', self getFields asString;
                            nextPutAll: '; postFields=', self postFields asString.
                ].
                printStream nextPutAll: ')'
!

writeOn: aStream
	"this can be used to form a client-side request and send it over network"
	aStream nextPutAll: self method, ' ', self url, ' ', self protocol, String crlf.
	self header keysAndValuesDo: [:key :value |
		aStream nextPutAll: key, ': ', value, String crlf].
	aStream nextPutAll: String crlf.
	aStream nextPutAll: self body asString
! !

!HttpRequest methodsFor:'private'!

parseCookies: aString 
	"PRIVATE: Parse a string in the format:  
	Cookie: NAME1=OPAQUE_STRING1; NAME2=OPAQUE_STRING2 ..."
	| dict |
	dict _ Dictionary new.
	(aString findTokens: ';')
		collect: [:pairStr | 
			| pair | 
			pair _ pairStr findTokens: '='.
			pair size = 2
				ifTrue: [| key value | 
					key _ pair first withBlanksTrimmed asSymbol.
					value _ pair second withBlanksTrimmed.
					dict at: key put: value]
				ifFalse: ["self log: 'Cookie invalid pair format: ''' , pairStr , ''''"]].
	^ dict
! !

!HttpRequest methodsFor:'response'!

badRequest
	^HttpBadRequest signal
!

redirectTo: aUrl
	^HttpRedirect new signalLocation: aUrl
!

unauthorizedFor: realm
	^HttpUnauthorized new signalRealm: realm
! !

!HttpRequest methodsFor:'response cookies'!

responseCookies
	"cookies that need to be set"
	^self propertyAt: #responseCookies ifAbsentPut: [OrderedCollection new]
!

setCookieName: nameString value: valueString expiresDate: expiresDate expiresTime: expiresTime path: pathString domain: domainString secure: secureBoolean 
	"set a cookie in the Response"
	| temp |
	temp _ String new writeStream.
	"NAME=VALUE"
	temp nextPutAll: nameString;
		 nextPut: $=;
		 nextPutAll: valueString.
	"; expires=Wdy, DD-Mon-YYYY HH:MM:SS GMT"
	(expiresDate notNil and:[ expiresTime notNil])
		ifTrue: [temp nextPutAll: '; expires=';
				 nextPutAll: expiresDate weekday;
				 nextPutAll: ', '.
			expiresDate printOn: temp format: #(1 2 3 $- 2 2 2 ).
			temp nextPutAll: ' ';
				 nextPutAll: expiresTime print24;
				 nextPutAll: ' GMT'].
	"; path=PATH"
	pathString notNil
		ifTrue: [temp nextPutAll: '; path=';
				 nextPutAll: pathString].
	"; domain=DOMAIN"
	domainString notNil
		ifTrue: [temp nextPutAll: '; domain=';
				 nextPutAll: domainString].
	"; secure"
	(secureBoolean notNil
			and: [secureBoolean])
		ifTrue: [temp nextPutAll: '; secure'].
	""
	self responseCookies add: temp contents
!

setCookieName: nameString value: valueString path: pathString 
	"set a cookie in the Response"
	self
		setCookieName: nameString
		value: valueString
		expiresDate: nil
		expiresTime: nil
		path: pathString
		domain: nil
		secure: nil
! !

!HttpRequest methodsFor:'testing'!

isGetRequest

	^self method = 'GET'
!

isHeaderRequest

	^self method = 'HEAD'
!

isPostRequest

	^self method = 'POST'
! !

!HttpRequest methodsFor:'user authentication'!

clearPassword
	self setCookieName: #ComanchePassword
		value: 'x'
		path: '/'
!

getUsername
	^self cookies at: #ComancheUsername ifAbsent: []
!

hashPassword: aPassword
	^HttpRequest hashPassword: aPassword
!

isUsername: aUsername password: aPassword
	| pwHash cookieUsername cookiePassword |
	cookieUsername _ self cookies at: #ComancheUsername ifAbsent: [^false].
	cookieUsername = aUsername ifFalse: [^false].

	cookiePassword _ self cookies at: #ComanchePassword ifAbsent: [^false].
	pwHash _ self hashPassword: aPassword.
	^cookiePassword = pwHash asString
!

secretWord
	^HttpRequest secretWord
!

setUsername: aUsername
	"save the username in a cookie"
	"make it expire very far into the future"
	self
		setCookieName: #ComancheUsername
		value: aUsername
		expiresDate: (Date today addDays: 1000)
		expiresTime: Time now
		path: '/'
		domain: nil
		secure: nil.

	self flag: #todo.
	"allow for multiple usernames"
!

setUsername: aUsername password: aPassword
	"save the username/password in cookies"
	| pwHash |
	self setUsername: aUsername.
	pwHash _ self hashPassword: aPassword.
	self
		setCookieName: #ComanchePassword
		value: pwHash asString
		expiresDate: (Date today addDays: 1000)
		expiresTime: Time now
		path: '/'
		domain: nil
		secure: nil.

	self flag: #todo.
	"allow for multiple usernames"
!

userObject
	^self session ifNotNil: [self session userObject]
!

userObject: anObject
	^self session ifNotNil: [self session userObject: anObject]
! !

!HttpRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/HttpRequest.st,v 1.26 2005/12/01 16:09:33 stefan Exp $'
! !

HttpRequest initialize!
