"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#HttpResponse
	instanceVariableNames:'contents contentType status request fields cookies'
	classVariableNames:'StatusCodes'
	poolDictionaries:''
	category:'Comanche-Kom-Protocol-HTTP'
!

HttpResponse comment:'kom/4.12 (Comanche/4.12)
bolot 2/20/2002 13:55
- cookies support
- defaultChunkSize delegates to Kom5Preferences

HttpResponse (bolot 4/2/2001 18:52)

Comment from kom46:
I am a response to an HttpRequest.  I can formulate an HTTP response and send it out over a socket.  An HttpAdapter will accept an HttpRequest, dispatch a method call to an HttpPlug (which will result in a stream or an error), and then formulat an instance of me to deliver the response to the client.'
!


!HttpResponse class methodsFor:'initialization'!

initialize
	"  HttpResponse initialize  "

	self initializeStatusCodes.
!

initializeStatusCodes
	"  HttpResponse initializeStatusCodes  "

	StatusCodes _ IdentityDictionary new
		at: #ok				put: ('200' -> 'OK');
		at: #created			put: ('201' -> 'Created');
		at: #accepted		put: ('202' -> 'Accepted');
		at: #noContent		put: ('204' -> 'No Content');
		at: #permMoved		put: ('301' -> 'Moved Permanently');
		at: #tempMoved		put: ('302' -> 'Moved Temporarily');
		at: #notModified		put: ('304' -> 'Not Modified');
		at: #badRequest		put: ('400' -> 'Bad Request');
		at: #unauthorized	put: ('401' -> 'Unauthorized');
		at: #forbidden		put: ('403' -> 'Forbidden');
		at: #notFound		put: ('404' -> 'Not Found');
		at: #serverError		put: ('500' -> 'Internal Server Error');
		at: #notImplemented	put: ('501' -> 'Not Implemented');
		at: #badGateway	put: ('502' -> 'Bad Gateway');
		at: #serviceUnavailable	put: ('503' -> 'Service Unavailable');
 		yourself.
! !

!HttpResponse class methodsFor:'instance creation'!

fromFileStream: aStream

	^self new
		status: #ok;
		"fieldAt: 'Cache-Control' put: 'max-age=86400';"
		contentType: aStream mimeType;
		contents: (AsyncFile from: aStream);
		yourself

	
!

fromMIMEDocument: aMIMEDoc
        | content |

        content _ aMIMEDoc content.
        (content isKindOf: String) ifTrue: [
            content _ ReadStream on: content.
        ].
        ^self new
                status: #ok;
                "
                 fieldAt: 'Expires' put: (String streamContents: [:s |
                        (TimeStamp fromSeconds: (Time primSecondsClock + 86400)) printOnHTTP: s]);
                "
                "
                 fieldAt: 'Cache-Control' put: 'max-age=86400';
                "
                contentType: aMIMEDoc mimeType;
                contents: content;
                yourself
!

fromRedirect: redirect
	| tmp |
	^self new
		status: #tempMoved;
		fieldAt: 'Location' put: (tmp _ redirect fullHttpUrl);
		fieldAt: 'Pragma' put: 'no-cache';
		fieldAt: 'Cache-Control' put: 'private';
		contentType: MIMEDocument contentTypeHtml;
		contents: (ReadStream on: '<head><title>Object moved</title></head>
<body><h1>Object Moved</h1>This object may be found <a HREF="', tmp,'">here</a>.</body>');
		yourself
!

fromStream: aStream

        ^self new
                status: #ok;
                "fieldAt: 'Expires' put: 'Thu, 15 Apr 2010 20:00:00 GMT';"
                "
                 fieldAt: 'Expires' put: (String streamContents: [:s |
                        (TimeStamp fromSeconds: (Time primSecondsClock + 86400)) printOnHTTP: s]);
                "
                "fieldAt: 'Cache-Control' put: 'max-age=86400';"
                contentType: aStream mimeType;
                contents: aStream;
                yourself
!

fromString: aString

	^self new
		status: #ok;
		"fieldAt: 'Cache-Control' put: 'max-age=86400';"
		contentType: MIMEDocument contentTypeHtml;
		contents: (ReadStream on: aString);
		yourself
!

fromUrl: aUrl
	^self new
		status: #tempMoved;
		fieldAt: 'Location' put: aUrl asString;
		contentType: MIMEDocument contentTypeHtml;
		contents: (ReadStream on: '<head><title>Object moved</title></head>
<body><h1>Object Moved</h1>This object may be found <a HREF="', aUrl asString,'">here</a>.</body>');
		yourself
!

status: aSymbol description: aString

	^self new
		status: aSymbol;
		contentType: MIMEDocument contentTypeHtml;
		contents: (ReadStream on: aString);
		yourself
!

status: aSymbol mimeDescription: aMIMEDocument
        | content |

        content _ aMIMEDocument content.
        (content isKindOf: String) ifTrue: [
            content _ ReadStream on: content.
        ].
        ^self new
                status: aSymbol;
                contentType: aMIMEDocument mimeType;
                contents: content;
                yourself
!

statusCodes
    ^ StatusCodes
! !

!HttpResponse methodsFor:'accessing'!

contentType

	^contentType
!

contentType: aMimeType

	contentType _ aMimeType
!

contents

	^contents
!

contents: aStream

	contents _ aStream
!

fieldAt: aString

	^self fields at: aString
!

fieldAt: aString ifAbsent: absBlock

	^self fields at: aString ifAbsent: absBlock
!

fieldAt: aString ifAbsentPut: absBlock

	^self fields at: aString ifAbsentPut: absBlock
!

fieldAt: aString put: aValue
        self fields at: aString put: aValue
!

fields

	fields ifNil: [
		fields _ Dictionary new.
	].

	^fields
!

httpVersion

	^'HTTP/1.1'
!

request

	^request
!

request: anHttpRequest

	request _ anHttpRequest.
!

status

	^status
!

status: aSymbol

	status _ aSymbol
!

statusCode

	^(StatusCodes at: self status) key.
!

statusCodeAndReason
	| tmp |

	tmp _ StatusCodes at: self status.
	^tmp key, ' ', tmp value.
! !

!HttpResponse methodsFor:'accessing-cookies'!

cookies
^ cookies ifNil:[cookies _ OrderedCollection new]
!

setCookieName: nameString value: valueString expiresDate: expiresDate expiresTime: expiresTime path: pathString domain: domainString secure: secureBoolean 
	"set a cookie in the Response"
	| temp |
	temp _ String new writeStream.
	"NAME=VALUE"
	temp nextPutAll: nameString;
		 nextPut: $=;
		 nextPutAll: valueString.
	"; expires=Wdy, DD-Mon-YYYY HH:MM:SS GMT"
	(expiresDate notNil and:[ expiresTime notNil])
		ifTrue: [temp nextPutAll: '; expires=';
				 nextPutAll: expiresDate weekday;
				 nextPutAll: ', '.
			expiresDate printOn: temp format: #(1 2 3 $- 2 2 2 ).
			temp nextPutAll: ' ';
				 nextPutAll: expiresTime print24;
				 nextPutAll: ' GMT'].
	"; path=PATH"
	pathString notNil
		ifTrue: [temp nextPutAll: '; path=';
				 nextPutAll: pathString].
	"; domain=DOMAIN"
	domainString notNil
		ifTrue: [temp nextPutAll: '; domain=';
				 nextPutAll: domainString].
	"; secure"
	(secureBoolean notNil
			and: [secureBoolean])
		ifTrue: [temp nextPutAll: '; secure'].
	""
	self cookies add: temp contents
!

setCookieName: nameString value: valueString path: pathString 
	"set a cookie in the Response"
	self
		setCookieName: nameString
		value: valueString
		expiresDate: nil
		expiresTime: nil
		path: pathString
		domain: nil
		secure: nil
! !

!HttpResponse methodsFor:'authentication'!

hashPassword: aPassword
	^HttpRequest hashPassword: aPassword
!

secretWord
	^HttpRequest secretWord
!

setUsername: aUsername
	"save the username in a cookie"
	self
		setCookieName: #ComancheUsername
		value: aUsername
		path: '/'.

	self flag: #todo.
	"allow for multiple usernames"
!

setUsername: aUsername password: aPassword
	"save the username/password in cookies"
	| pwHash |
	self setUsername: aUsername.
	pwHash _ self hashPassword: aPassword.
	self
		setCookieName: #ComanchePassword
		value: pwHash asString
		path: '/'.

	self flag: #todo.
	"allow for multiple usernames"
! !

!HttpResponse methodsFor:'comanche processing'!

asHttpPartialResponseBlock: aBlock
        ^(self as: HttpPartialResponse)
                producerBlock: aBlock;
                yourself
!

asHttpResponseTo: aRequest
	^self
! !

!HttpResponse methodsFor:'defaults'!

defaultChunkSize

	^ Kom5Preferences defaultResponseChunkSize
!

defaultContentType

	^MIMEDocument contentTypeHtml
! !

!HttpResponse methodsFor:'initialize-release'!

destroy

	self contents close.
! !

!HttpResponse methodsFor:'responding'!

pvtWriteContentLengthOn: aStream
	aStream
		nextPutAll: 'Content-length: ';
		nextPutAll: self contents size asString;
		nextPutAll: String crlf
!

pvtWriteContentTypeOn: aStream
	aStream
		nextPutAll: 'Content-type: ';
		nextPutAll: self contentType;
		nextPutAll: String crlf
!

pvtWriteContentsOn: aStream
        | chunkSize cnts |

        cnts _ self contents.
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            chunkSize _ cnts size.
        ] ifFalse:[
            chunkSize _ self defaultChunkSize.
        ].

        [ cnts atEnd ] whileFalse: [
                aStream nextPutAll: (cnts next: chunkSize); flush.
                "Processor yield."
        ]
!

pvtWriteCookiesOn: aStream 
	cookies
		ifNil: [cookies _ request responseCookies]
		ifNotNil: [cookies addAll: request responseCookies].
	cookies isEmptyOrNil
		ifTrue: [^ self].
	cookies
		do: [:cookie | aStream nextPutAll: 'Set-Cookie: ';
				 nextPutAll: cookie;
				 nextPutAll: String crlf]
!

pvtWriteFieldsOn: aStream
	fields isEmptyOrNil ifTrue: [ ^self ].

	fields keysAndValuesDo: [ :k :v |
		aStream
			nextPutAll: k;
			nextPutAll: ': ';
			nextPutAll: v;
			nextPutAll: String crlf
	]
!

pvtWriteServerOn: aStream
        aStream
                nextPutAll: 'Server: Smalltalk/X HTTPServer';
                nextPutAll: String crlf
!

pvtWriteStatusOn: aStream
	aStream
		nextPutAll: self httpVersion;
		nextPut: $ ;
		nextPutAll: self statusCodeAndReason;
		nextPutAll: String crlf
!

writeOn: aStream

	| tmp |
	"Header"

	tmp _ WriteStream on: ''.
	self pvtWriteStatusOn: tmp.
	self pvtWriteServerOn: tmp.
	self pvtWriteFieldsOn: tmp.
	self pvtWriteCookiesOn: tmp.
	self pvtWriteContentTypeOn: tmp.
	self pvtWriteContentLengthOn: tmp.
	tmp nextPutAll: String crlf.
	aStream nextPutAll: tmp contents; flush.

	"Body"
	self pvtWriteContentsOn: aStream
! !

!HttpResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/HttpResponse.st,v 1.12 2003/09/10 15:30:41 cg Exp $'
! !

HttpResponse initialize!
