"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiRequest subclass:#HttpSwikiRequest
	instanceVariableNames:'raw'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Comanche'
!


!HttpSwikiRequest class methodsFor:'instance creation'!

fromHttpRequest: rawRequest
    ^self new initialize; fromHttpRequest: rawRequest
! !

!HttpSwikiRequest methodsFor:'accessing'!

raw
	^raw
! !

!HttpSwikiRequest methodsFor:'fields'!

fieldsAsBooleanKey: aKey
	^self fieldsAsBooleanKey: aKey ifAbsent: [false]
!

fieldsAsBooleanKey: aKey ifAbsent: block
	(raw postFields)
		ifNil: [(raw getFields)
			ifNil: [^block value]
			ifNotNil: [^(raw getFields at: aKey ifAbsent: [^block value]) = 'true']]
		ifNotNil: [^(raw postFields at: aKey ifAbsent: [
			(raw getFields)
				ifNil: [^block value]
				ifNotNil: [^(raw getFields at: aKey ifAbsent: [^block value]) = 'true']]) = 'true']
!

fieldsAsNumberKey: aKey
	^self fieldsAsNumberKey: aKey ifAbsent: [0]
!

fieldsAsNumberKey: aKey ifAbsent: block
	(raw postFields)
		ifNil: [(raw getFields)
			ifNil: [^block value]
			ifNotNil: [^(raw getFields at: aKey ifAbsent: [^block value]) asNumber]]
		ifNotNil: [^(raw postFields at: aKey ifAbsent: [
			(raw getFields)
				ifNil: [^block value]
				ifNotNil: [^(raw getFields at: aKey ifAbsent: [^block value]) asNumber]]) asNumber]
!

fieldsHasKey: aKey
        ^(raw postFields)
                ifNil: [(raw getFields)
                        ifNil: [false]
                        ifNotNil: [raw getFields includesKey: aKey]]
                ifNotNil: [(raw postFields includesKey: aKey)
                        ifTrue: [true]
                        ifFalse: [(raw getFields)
                                ifNil: [false]
                                ifNotNil: [raw getFields includesKey: aKey]]]
!

fieldsKey: aKey
	^self fieldsKey: aKey ifAbsent: ['']
!

fieldsKey: aKey ifAbsent: block
	^(raw postFields)
		ifNil: [(raw getFields)
			ifNil: [block value]
			ifNotNil: [raw getFields at: aKey ifAbsent: block]]
		ifNotNil: [raw postFields at: aKey ifAbsent: [
			(raw getFields)
				ifNil: [block value]
				ifNotNil: [raw getFields at: aKey ifAbsent: block]]]
! !

!HttpSwikiRequest methodsFor:'initializing'!

fromHttpRequest: rawRequest
        | tokens relativePath|

        raw _ rawRequest.
        relativePath _ raw stxHttpRequest relativePath.
        tokens _ (relativePath copyUpTo: $?) findTokens: '/.'.
        (tokens size = 0) ifTrue: [^self].
        book _ tokens at: 1.
        (tokens size = 1) ifTrue: [address _ 'default'.
                ^self].
        (tokens at: 2) isAllDigits
                ifTrue: [page _ (tokens at: 2) asNumber.
                        address _ (tokens size > 2) ifTrue: [tokens at: 3] ifFalse: ['default']]
                ifFalse: [address _ tokens at: 2]
"/        tokens _ (raw url copyUpTo: $?) findTokens: '/.'.
"/        (tokens size = 0) ifTrue: [^self].
"/        relativePath _ raw stxHttpRequest relativePath copyFrom:2.
"/        idx := relativePath lastIndexOf:$/.
"/        idx == 0 ifTrue:[
"/            book := relativePath.
"/            address _ 'default'.
"/            ^self
"/        ].
"/        self halt.
"/        book := relativePath copyTo:idx - 1.
"/        pageAndAdress := relativePath copyFrom:idx + 1.
"/        tokens := pageAndAdress findTokens: '.'.
"/        (tokens at: 1) isAllDigits
"/                ifTrue: [page _ (tokens at: 1) asNumber.
"/                        address _ (tokens size > 1) ifTrue: [tokens at: 2] ifFalse: ['default']]
"/                ifFalse: [address _ tokens at: 1]
! !

!HttpSwikiRequest methodsFor:'references'!

fileReferenceShelf: shelf book: theBook address: theAddress
        ^'/', theBook name, '/', theAddress
!

referenceShelf: shelf address: theAddress
        ^(shelf settingsAt: 'bookPrefixPath' ifAbsent: ['/']), theAddress
!

referenceShelf: shelf book: theBook
        ^(shelf settingsAt: 'bookPrefixPath' ifAbsent: ['/']), theBook name
!

referenceShelf: shelf book: theBook address: theAddress
        ^(shelf settingsAt: 'bookPrefixPath' ifAbsent: ['/']), theBook name, '/', theAddress
!

referenceShelf: shelf book: theBook page: thePage
        ^(shelf settingsAt: 'bookPrefixPath' ifAbsent: ['/']) , theBook name, '/', (thePage id asString)
!

referenceShelf: shelf book: theBook page: thePage address: theAddress
        ^ (shelf settingsAt: 'bookPrefixPath' ifAbsent: ['/']) , theBook name, '/', (thePage id asString), '.', theAddress
! !

!HttpSwikiRequest methodsFor:'utility'!

authorization
        ^raw user
!

authorizationUser
    | authorization authorizationUser authorizationUserAndPassword|
    authorization := self authorization.
    authorization notNil ifTrue:[
        authorizationUserAndPassword := (RFC1421Coder decode:authorization).
        authorizationUser := authorizationUserAndPassword copyTo:((authorizationUserAndPassword indexOf:$:) - 1).
    ].
    ^ authorizationUser
!

formText
	| texts index |
	(self fieldsHasKey: 'text1') ifFalse: [^''].
	texts _ Dictionary new.
	index _ 1.
	[self fieldsHasKey: ('text', index asString)]
		whileTrue: [texts at: (index asString) put: (self fieldsKey: ('text', index asString)).
			index _ index + 1].
	^texts
!

getUsername
	^raw getUsername
!

hasUserObject
	^raw userObject notNil
!

isFramesCapable
	^((raw header at: 'user-agent' ifAbsent: [^false]) asLowercase beginsWith: 'lynx') not
!

isUsername: aUsername password: aPassword
	^raw isUsername: aUsername password: aPassword
!

setUsername: aUsername password: aPassword
	^raw setUsername: aUsername password: aPassword
!

text
	^self fieldsKey: 'text' ifAbsent: [self formText]
!

url
	^raw url
!

user
	"Give IP address since this is the closest that an HttpRequest without login, password."
	^raw remoteAddress
!

userID
	"Should be refactored"
	| userID i |
	userID _ raw header at: 'authorization' ifAbsent: [^nil].
	(userID notNil and: [(i _ userID findString: 'Basic ') > 0])
		ifTrue: [userID _ userID copyFrom: i+6 to: userID size]
		ifFalse: [userID _ nil].
	^userID
!

userObject
	^raw userObject
!

userObject: anObject
	raw userObject: anObject
! !

!HttpSwikiRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/HttpSwikiRequest.st,v 1.12 2003/11/26 12:59:14 penk Exp $'
! !
