"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#LineFormatter
	instanceVariableNames:'exceptions formatters storeID convertToCrlf'
	classVariableNames:'CR LF'
	poolDictionaries:''
	category:'Comanche-Swiki-Formatting'
!


!LineFormatter class methodsFor:'instance creation'!

appendFormatter
	| instance |

	instance _ self new.
	instance storeID: #appendFormatter.
	instance convertToCrlf: false.
	instance addExceptionFrom: '<html>' to: '</html>' using: [:text :request :response :shelf :book :page | '<html>', (TextFormatter crlfToCr: text), '</html>'].
	instance addExceptionFrom: '<HTML>' to: '</HTML>' using: [:text :request :response :shelf :book :page | '<HTML>', (TextFormatter crlfToCr: text), '</HTML>'].
	instance addExceptionFrom: '<code>' to: '</code>' using: [:text :request :response :shelf :book :page | '<code>', (TextFormatter crlfToCr: text), '</code>'].
	instance addExceptionFrom: '<CODE>' to: '</CODE>' using: [:text :request :response :shelf :book :page | '<CODE>', (TextFormatter crlfToCr: text), '</CODE>'].
	instance addFormatter: (IdFormatter saveAppend) at: $+.
	instance addDefaultFormatter: (IdFormatter leaveAlone).
	^instance
!

renderFormatter
	| instance textFormatter listFormatter |

	instance _ self new.
	instance storeID: #renderFormatter.
	instance convertToCrlf: true.
	textFormatter _ PageFormatter new.
	textFormatter
		initialize;
		addRenderInternalLinks;
		addTagIntegrity;
		addShowSpecialCharacters;
		addRenderUploadLinks.
	instance addExceptionFrom: '<html>' to: '</html>' using: [:text :request :response :shelf :book :page | TextFormatter crToCrlf: text].
	instance addExceptionFrom: '<HTML>' to: '</HTML>' using: [:text :request :response :shelf :book :page | TextFormatter crToCrlf: text].
	instance addExceptionFrom: '<code>' to: '</code>' using: [:text :request :response :shelf :book :page | '<pre>', (TextFormatter encodeToXmlCrlf: text), '</pre>'].
	instance addExceptionFrom: '<CODE>' to: '</CODE>' using: [:text :request :response :shelf :book :page | '<pre>', (TextFormatter encodeToXmlCrlf: text), '</pre>'].
	instance addFormatter: (IdFormatter break) at: $_.
	instance addFormatter: (IdFormatter anchor) at: $@.
	instance addFormatter: (IdFormatter appendRender) at: $+.
	instance addFormatter: (IdFormatter heading: textFormatter) at: $!!.
	listFormatter _ IdFormatter list: textFormatter.
	instance addFormatter: (listFormatter) at: $-.
	instance addFormatter: (listFormatter) at: $#.
	instance addFormatter: (IdFormatter table: textFormatter) at: $|.
	instance addFormatter: (IdFormatter preformatted: textFormatter) at: $=.
	instance addDefaultFormatter: (IdFormatter text: textFormatter).
	^instance
!

showFormatter
	| instance textFormatter listFormatter |

	instance _ self new.
	instance storeID: #showFormatter.
	instance convertToCrlf: true.
	textFormatter _ PageFormatter new.
	textFormatter
		initialize;
		addInternalLinks;
		addTagIntegrity;
		addShowSpecialCharacters;
		addUploadLinks.
	instance addExceptionFrom: '<html>' to: '</html>' using: [:text :request :response :shelf :book :page | TextFormatter crToCrlf: text].
	instance addExceptionFrom: '<HTML>' to: '</HTML>' using: [:text :request :response :shelf :book :page | TextFormatter crToCrlf: text].
	instance addExceptionFrom: '<code>' to: '</code>' using: [:text :request :response :shelf :book :page | '<pre>', (TextFormatter encodeToXmlCrlf: text), '</pre>'].
	instance addExceptionFrom: '<CODE>' to: '</CODE>' using: [:text :request :response :shelf :book :page | '<pre>', (TextFormatter encodeToXmlCrlf: text), '</pre>'].
	instance addFormatter: (IdFormatter break) at: $_.
	instance addFormatter: (IdFormatter anchor) at: $@.
	instance addFormatter: (IdFormatter append) at: $+.
	instance addFormatter: (IdFormatter heading: textFormatter) at: $!!.
	listFormatter _ IdFormatter list: textFormatter.
	instance addFormatter: (listFormatter) at: $-.
	instance addFormatter: (listFormatter) at: $#.
	instance addFormatter: (IdFormatter table: textFormatter) at: $|.
	instance addFormatter: (IdFormatter preformatted: textFormatter) at: $=.
	instance addDefaultFormatter: (IdFormatter text: textFormatter).
	^instance
! !

!LineFormatter class methodsFor:'initialize-release'!

initialize
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            CR := Character return.
        ] ifFalse:[
            CR _ Character cr.
        ].
        LF _ Character lf
! !

!LineFormatter methodsFor:'accessing'!

addDefaultFormatter: anIdFormatter
	(formatters = nil) ifTrue: [formatters _ Dictionary new].
	formatters at: LF put: anIdFormatter
!

addExceptionFrom: startsWith to: endsWith using: formatter
        | element |

        (exceptions = nil) ifTrue: [exceptions _ OrderedCollection new].
        element _ Array new: 3.
        element
                at: 1 put: startsWith;
                at: 2 put: endsWith.
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            element at: 3 put: (SimulatedSqueakBlock wrapIfBlock:formatter).
        ] ifFalse:[
            element at: 3 put: formatter.
        ].
        exceptions add: element.
!

addFormatter: anIdFormatter at: char
	(formatters = nil) ifTrue: [formatters _ Dictionary new].
	formatters at: char put: anIdFormatter
!

convertToCrlf: aBoolean
	convertToCrlf _ aBoolean
!

exceptions
	^exceptions
!

formatters
	^formatters
!

storeID
	^storeID
!

storeID: id
	storeID _ id
! !

!LineFormatter methodsFor:'private'!

combine: formattedText with: exceptionCollection
	| i return pos1 pos2 |

	i _ 0.
	return _ WriteStream on: (String new: (formattedText size * 2)).
	pos1 _ 1.
	[(pos2 _ formattedText indexOf: LF startingAt: pos1 ifAbsent: [0]) == 0] whileFalse: [
		(pos1 = pos2) ifFalse: [return nextPutAll: (formattedText copyFrom: pos1 to: (pos2 - 1))].
		return nextPutAll: (exceptionCollection at: (i _ i + 1) ifAbsent: ['']).
		pos1 _ pos2 + 1].
	(pos1 > formattedText size) ifFalse: [return nextPutAll: (formattedText copyFrom: pos1 to: formattedText size)].
	^return contents
!

formattedTextFrom: text request: request response: response shelf: shelf book: book page: page
	| return start end coll types id |

	return _ ReadWriteStream on: (String new: text size).
	coll _ OrderedCollection new.
	types _ OrderedCollection new.
	start _ 1.
	[start <= (text size)] whileTrue: [
		end _ text indexOf: CR startingAt: start ifAbsent: [end _ text size + 1].
		coll add: ((start = end) ifTrue: [''] ifFalse: [text copyFrom: start to: end-1]).
		start _ end + 1].
	(coll size = 0) ifTrue: [^return contents].
	coll do: [:element | types add: (self typeFrom: element)].
	(coll size = 1) ifTrue: [(types at: 1) singleOn: return text: (coll at: 1) request: request response: response shelf: shelf book: book page: page.
		^return contents].
	((types at: 2) = (types at: 1))
		ifTrue: [id _ (types at: 1) startOn: return text: (coll at: 1) request: request response: response shelf: shelf book: book page: page]
		ifFalse: [id _ (types at: 1) singleOn: return text: (coll at: 1) request: request response: response shelf: shelf book: book page: page].
	(coll size > 2) ifTrue: [
		2 to: (coll size - 1) do: [:i | ((types at: i) = (types at: i+1))
			ifTrue: [((types at: i) = (types at: i-1))
				ifTrue: [id _ (types at: i) transitionAfter: id on: return text: (coll at: i) request: request response: response shelf: shelf book: book page: page]
				ifFalse: [id _ (types at: i) startOn: return text: (coll at: i) request: request response: response shelf: shelf book: book page: page]]
			ifFalse: [((types at: i) = (types at: i-1))
				ifTrue: [id _ (types at: i) endAfter: id on: return text: (coll at: i) request: request response: response shelf: shelf book: book page: page]
				ifFalse: [id _ (types at: i) singleOn: return text: (coll at: i) request: request response: response shelf: shelf book: book page: page]]]].
	((types last) = (types at: (types size - 1)))
		ifTrue: [id _ (types last) endAfter: id on: return text: (coll last) request: request response: response shelf: shelf book: book page: page]
		ifFalse: [id _ (types last) singleOn: return text: (coll last) request: request response: response shelf: shelf book: book page: page].
	^return contents
!

splitTextAndExceptionsFrom: text request: request response: response shelf: shelf book: book page: page
	| col formatterText start tempStart matchStart matchException matchEnd matchText |

	col _ OrderedCollection new.
	formatterText _ ReadWriteStream on: (String new: text size).
	start _ 1.
	[start <= (text size)] whileTrue: [
		matchStart _ text size.
		matchException _ nil.
		exceptions do: [:exception | tempStart _ text findString: (exception at: 1) startingAt: start.
			((tempStart > 0) and: [tempStart < matchStart]) ifTrue: [matchStart _ tempStart.
				matchException _ exception]].
		matchException
			ifNil: ["No more found"
				formatterText nextPutAll: (text copyFrom: start to: text size).
				start _ text size + 1]
			ifNotNil: [
				(start = matchStart) ifFalse: [formatterText nextPutAll: (text copyFrom: start to: matchStart-1)].
				formatterText nextPut: LF.
				matchEnd _ text findString: (matchException at: 2) startingAt: matchStart.
				(matchEnd = 0)
					ifTrue: ["End of text"
						matchText _ text copyFrom: (matchStart + (matchException at: 1) size) to: text size.
						start _ text size + 1]
					ifFalse: [
						matchText _ text copyFrom: (matchStart + (matchException at: 1) size) to: (matchEnd - 1).
						start _ matchEnd + (matchException at: 2) size].
				col add: (((matchException at: 3) copy fixTemps) valueWithArguments: (Array with: matchText with: request with: response with: shelf with: book with: page))]].
	^Array with: formatterText contents with: col
!

storeString
	"Overwrite of Object method to save space for storing"
	(storeID) ifNotNil: ["Check to make sure this storeID is valid"
		(self class respondsTo: storeID)
			ifTrue: [^'(', self class asString, ' ', storeID asString, ')']].
	^super storeString
!

typeFrom: text
        ^(text size = 0)
                ifTrue: [formatters at: LF]
                ifFalse: [formatters at: (text at: 1) ifAbsent: [formatters at: LF]]
! !

!LineFormatter methodsFor:'processing'!

format: text request: request response: response shelf: shelf book: book page: page
	| textAndException crText |

	"textAndException will be a 2 element array with the readToFormatText and the exceptionCollection"
	textAndException _ self splitTextAndExceptionsFrom: text request: request response: response shelf: shelf book: book page: page.
	crText _ self combine: (self formattedTextFrom: textAndException first request: request response: response shelf: shelf book: book page: page) with: textAndException last.
	^convertToCrlf
		ifTrue: [TextFormatter crToCrlf: crText]
		ifFalse: [crText]
! !

!LineFormatter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/LineFormatter.st,v 1.7 2003/06/13 14:12:58 cg Exp $'
! !

LineFormatter initialize!
