"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiStructure subclass:#NuSwikiPage
	instanceVariableNames:'id versionId date time user text book userName'
	classVariableNames:'BadCharTable'
	poolDictionaries:''
	category:'Comanche-Swiki-Structure'
!


!NuSwikiPage class methodsFor:'instance creation'!

new
	| instance |
	instance _ super new.
	instance initialize.
	^instance
! !

!NuSwikiPage class methodsFor:'initialize-release'!

initialize
	BadCharTable _ String newFrom: ((0 to: 255) collect: [:i | i asCharacter]).
	BadCharTable at: 1 put: Character cr.
! !

!NuSwikiPage class methodsFor:'utility'!

hasAProtocol: text 
	(text includes: $:) ifFalse: [^ false].
	^ #('http' 'https' 'ftp' 'gopher' 'mailto' 'mailto' 'news' 'telnet' 'rlogin' 'tn3270' 'wais' 'file' ) includes: (text copyUpTo: $:) asLowercase
!

isFormText: aText
	^(aText isKindOf: Dictionary)
!

isTextAnEmailAddress: text
	(text includes: $@) ifFalse: [^false].
	(text includes: Character space) ifTrue: [^false].
	((text copyAfterLast: $@) includes: $.) ifFalse: [^false].
	^true
!

printDate: aDate
	^(aDate dayOfMonth asString), ' ', (aDate monthName), ' ', (aDate year asString)
!

urlForProtocolText: text 
        "URLs such as http:myfile.txt should be myfile.txt &
        URLs sucha s http:/myfile.txt should be /myfile.txt"
        | lowerText |
        lowerText _ text asLowercase.
        (lowerText beginsWith: 'http:') ifFalse: [^text].
        ^(lowerText beginsWith: 'http://')
                ifTrue: [text]
                ifFalse: [
                    text copyAfter: $:
                ]
! !

!NuSwikiPage methodsFor:'accessing'!

book
    ^ book
!

date
	^date
!

date: aDate
	date _ aDate
!

id
	^id
!

id: anInteger
	id _ anInteger
!

printDate
	^self class printDate: date
!

printTime
	| stream |
	stream _ ReadWriteStream on: String new.
	time printOn: stream.
	^stream contents
!

rawText
	^text
!

showName
	^self showName: name
!

showName: theText
	^(PageFormatter toSafeAlias: theText)
!

storage
	^storage
!

storage: aSwikiStorage
	storage _ aSwikiStorage
!

text
	^storage textFrom: self
!

text: anObject
	text _ anObject
!

textKey: key
	^storage textKey: key from: self
!

time
	^time
!

time: aTime
	time _ aTime
!

user
	^user
!

user: aString
	user _ aString
!

userName
    ^ userName
!

userName:something
    userName := something.
!

versionId
	^versionId
!

versionId: anInteger
	"0 means current version"
	versionId _ anInteger
! !

!NuSwikiPage methodsFor:'formatters'!

appendFormat: oldText request: request response: response shelf: shelf book: book
	| formatter newText |

	"is a form"
	(self isAForm) ifTrue: [^self formAppendFormat: oldText request: request response: response shelf: shelf book: book].
	"is not a form"
	formatter _ self settingsAt: 'appendFormatter' ifAbsent: [book settingsAt: 'appendFormatter' ifAbsent: [^nil]].
	newText _ (formatter format: oldText request: request response: response shelf: shelf book: book page: self).
	"delete extra cr formed from appendFormatter"
	(newText endsWith: String cr) ifTrue: [(newText size > 1) ifTrue: [
		newText _ newText copyFrom: 1 to: (newText size - 1)]].
	^self text: newText
!

appendFormatRequest: request response: response shelf: shelf book: book
	^self appendFormat: (self text) request: request response: response shelf: shelf book: book
!

editFormat: aString request: request response: response shelf: shelf book: book
	| formatter |

	formatter _ self settingsAt: 'editFormatter' ifAbsent: [book settingsAt: 'editFormatter' ifAbsent: [^aString]].
	^formatter format: aString request: request response: response shelf: shelf book: book page: self
!

editFormatRequest: request response: response shelf: shelf book: book
	^self editFormat: (self text) request: request response: response shelf: shelf book: book
!

formAppendFormat: texts request: request response: response shelf: shelf book: book
	| formatter newText element |

	formatter _ self settingsAt: 'appendFormatter' ifAbsent: [book settingsAt: 'appendFormatter' ifAbsent: [^nil]].
	newText _ Dictionary new.
	texts keysAndValuesDo: [:key :value |
		request settingsRemove: 'appendId'.
		request settingsAt: 'formId' put: key.
		element _ (formatter format: value request: request response: response shelf: shelf book: book page: self).
		"delete extra cr formed from appendFormatter"
		(element endsWith: String cr) ifTrue: [(element size > 1) ifTrue: [
			element _ element copyFrom: 1 to: (element size - 1)]].
		newText at: key put: element].
	^self text: newText
!

formSaveFormat: texts request: request response: response shelf: shelf book: book
	| formatter newText |

	formatter _ self settingsAt: 'saveFormatter' ifAbsent: [book settingsAt: 'saveFormatter' ifAbsent: [^nil]].
	newText _ Dictionary new.
	texts keysAndValuesDo: [:key :value | value translateWith: BadCharTable. "Fix for Lynx"
		newText at: key put: (formatter format: value request: request response: response shelf: shelf book: book page: self)].
	^self text: newText
!

formUpdateFormat: texts request: request response: response shelf: shelf book: book
        | formatter newText |

        formatter _ self settingsAt: 'updateFormatter' ifAbsent: [book settingsAt: 'updateFormatter' ifAbsent: [^nil]].
        newText _ Dictionary new.
        texts keysAndValuesDo: [:key :value | newText at: key put: (formatter format: value request: request response: response shelf: shelf book: book page: self)].
        self text: newText.
"/        [
            self write
"/        ] fork

    "Modified: / 27-07-2006 / 10:23:38 / cg"
!

renderFormat: aString request: request response: response shelf: shelf book: book
	| formatter |

	formatter _ self settingsAt: 'renderFormatter' ifAbsent: [book settingsAt: 'renderFormatter' ifAbsent: [^aString]].
	^formatter format: aString request: request response: response shelf: shelf book: book page: self
!

renderFormatRequest: request response: response shelf: shelf book: book
	^self renderFormat: (self text) request: request response: response shelf: shelf book: book
!

saveFormat: newText request: request response: response shelf: shelf book: book 
	| formatter |
	(self class isFormText: newText)
		ifTrue: [^ self
				formSaveFormat: newText
				request: request
				response: response
				shelf: shelf
				book: book].
	formatter _ self
				settingsAt: 'saveFormatter'
				ifAbsent: [book
						settingsAt: 'saveFormatter'
						ifAbsent: [^ nil]].
	newText translateWith: BadCharTable.
	"Fix for Lynx"
	^ self
		text: (formatter
				format: newText
				request: request
				response: response
				shelf: shelf
				book: book
				page: self)
!

showFormat: aString request: request response: response shelf: shelf book: book
        | formatter |
        formatter _ self settingsAt: 'showFormatter' ifAbsent: [book settingsAt: 'showFormatter' ifAbsent: [^aString]].
        ^formatter format: aString request: request response: response shelf: shelf book: book page: self
!

showFormatRequest: request response: response shelf: shelf book: book
        ^self showFormat: self text request: request response: response shelf: shelf book: book
!

updateFormat: updateText request: request response: response shelf: shelf book: book
	| formatter |

	"is a form"
	(self isAForm) ifTrue: [^self formUpdateFormat: updateText request: request response: response shelf: shelf book: book].
	"is not a form"
	formatter _ self settingsAt: 'updateFormatter' ifAbsent: [book settingsAt: 'updateFormatter' ifAbsent: [^nil]].
	self text: (formatter format: updateText request: request response: response shelf: shelf book: book page: self).
	self write
!

updateFormatRequest: request response: response shelf: shelf book: book
	^self updateFormat: self text request: request response: response shelf: shelf book: book
! !

!NuSwikiPage methodsFor:'storage'!

backup
	storage backupPage: self
!

version: vid
	"Returns the XmlSwikiPage version.
	If the version does not exist, return nil."
	| version |
	(vid > 0) ifFalse: [^nil].
	version _ self class new id: id; versionId: vid; storage: storage.
	(storage loadPage: version) ifNil: [^nil].
	^version
!

versions
	^storage loadVersionsFrom: self
!

write
        "sorry for this hack - prevent swiki from shutting down server"

        Error 
            handle:[:ex | ]
            do:[
                storage writePage: self
            ]

    "Modified: / 27-07-2006 / 19:45:30 / cg"
! !

!NuSwikiPage methodsFor:'testing'!

isAForm
	^storage isAComplexPage: self
!

textContains: aString caseSensitive: caseSensitive
	| tempText |

	tempText _ self text.
	(tempText isKindOf: Dictionary)
		ifTrue: [tempText valuesDo: [:value | (value includesSubstring: aString caseSensitive: caseSensitive) ifTrue: [^true]].
			^false]
		ifFalse: [^tempText  includesSubstring: aString caseSensitive: caseSensitive]
! !

!NuSwikiPage methodsFor:'utility'!

alertsFrom: request
        | tokens alerts |

        tokens _ (request fieldsKey: 'alerts') findTokens: String crlf, ' ;,'.

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            (tokens size > 0) ifTrue:[
                alerts _ tokens at: 1.
                (tokens size > 1) ifTrue: [
                    2 to: (tokens size) do: [:i | alerts _ alerts, ', ', (tokens at: i)].
                ].
                self settingsAt: 'alerts' put:alerts.
            ] ifFalse: [
                self settingsRemove:'alerts'
            ]
        ] ifFalse:[
            (tokens size > 0)
                    ifTrue: [self settingsAt: 'alerts' put: ((tokens size = 1)
                            ifTrue: [tokens at: 1]
                            ifFalse: [alerts _ tokens at: 1.
                                    2 to: (tokens size) do: [:i | alerts _ alerts, ', ', (tokens at: i)].
                                    alerts])]
                    ifFalse: [self settingsRemove: 'alerts'].
        ]
!

clearRefsCache
	settings removeKey: 'referenceCache' ifAbsent: []
!

defaultLabelForInspector
	^ super defaultLabelForInspector, ': ', id asString
!

isOlderThan: request
        | editTime |

        editTime _ request fieldsAsNumberKey: 'edittime' ifAbsent: [^true].
        editTime _ Time dateAndTimeFromSeconds: editTime.
        ((editTime at: 1) < date) ifTrue: [^false].
        ((editTime at: 1) = date) ifTrue: [((editTime at: 2) < time) ifTrue: [^false]].
        ^true
!

nameUnique: aString book: book stripping: badElementString
	"First, strip aString of anything that would be bad."
	"Then, name the page uniquely."
	| stripped copy |

	stripped _ WriteStream on: (String new: aString size).
	aString do: [:char | (badElementString includes: char) ifFalse: [stripped nextPut: char]].
	stripped _ stripped contents.
	(stripped = name) ifFalse: ["It only needs to be checked if name is different"
		(book hasPageNamed: stripped)
			ifTrue: ["Check for names of aString, ' #X'."
				copy _ 2.
				[book hasPageNamed: (stripped, ' #', copy asString)] whileTrue: [copy _ copy + 1].
				self name: (stripped, ' #', copy asString)]
			ifFalse: [self name: stripped]]
!

passwordFrom: request response: response shelf: shelf book: book
	| lock password check bookpw |

	lock _ request fieldsAsBooleanKey: 'lock'.
	password _ request fieldsKey: 'password' ifAbsent: [nil].
	check _ request fieldsKey: 'passwordcheck' ifAbsent: [nil].
	bookpw _ book modulesAt: 'lockPassword' ifAbsent: [nil].
	password ifNotNil: [check
		ifNil: [lock
			ifTrue: [(password = bookpw) ifFalse: [self settingsAt: 'lockPassword' put: password].
				self settingsAt: 'lock' put: true]
			ifFalse: [self settingsRemove: 'lockPassword'.
				self settingsAt: 'lock' put: false]]
		ifNotNil: [(password = check) ifTrue: [lock
			ifTrue: [(password = bookpw) ifFalse: [self settingsAt: 'lockPassword' put: password].
				self settingsAt: 'lock' put: true]
			ifFalse: [self settingsRemove: 'lockPassword'.
				self settingsAt: 'lock' put: false]]]]
!

passwordMatches: request response: response shelf: shelf book: book
	| password |

	password _ request fieldsKey: 'password' ifAbsent: [^false].
	(self settingsIncludes: 'lockPassword')
		ifTrue: [((self settingsAt: 'lockPassword') = password) ifTrue: [^true]].
	((book modulesAt: 'lockPassword') = password) ifTrue: [^true].
	^false
!

reverseVersionsUsing: block
	| versions return size |

	versions _ self versions.
	size _ versions size.
	(size = 0) ifTrue: [^''].
	return _ WriteStream on: String new.
	1 to: size do: [:i | return nextPutAll: (block value: (versions at: (size + 1 - i)))].
	^return contents
!

sendAlerts: request response: response shelf: shelf book: book
        | alertSender alertText alertServer |

        (settings includesKey: 'alerts')
                ifTrue: ["Send Messages"
                        alertSender _ self settingsAt: 'alertSender' ifAbsent: [book settingsAt: 'alertSender' ifAbsent: [^self]].
                        alertServer _ self settingsAt: 'alertServer' ifAbsent: [book settingsAt: 'alertServer' ifAbsent: [^self]].
                        alertText _  (book formatPageTemplate: 'alert' request: request response: response shelf: shelf page: self).
                        (Smalltalk dialectName = 'SmalltalkX') ifFalse:[
                            alertText _ TextFormatter crlfToCr:alertText.
                        ].
                        [
                            (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                                |smtp subject idx subjectString|
                                (alertServer notEmpty or:[alertSender isEmpty]) ifTrue:[
                                    subject := ''.
                                    subjectString := 'Subject:'.
                                    (alertText startsWith:subjectString) ifTrue:[alertText := alertText copyFrom:(subjectString size  + 2)].
                                    idx := alertText indexOf:(Character return).
                                    idx ~= 0 ifTrue:[ 
                                        subject := alertText copyTo:idx - 1.
                                        alertText := alertText copyFrom:idx + 1.
                                    ].
                                    smtp := SMTPClient new.                                    
                                    smtp connectTo:alertServer.
                                    smtp userName:alertSender.
                                    smtp
                                       sendMessageString:alertText 
                                       withSubject:subject
                                       toAll:((settings at: 'alerts') findTokens: ', ').
                                    smtp close
                                ].                                
                            ] ifFalse:[
                                SMTPSocket deliverMailFrom: alertSender to: ((settings at: 'alerts') findTokens: ', ') text: alertText usingServer: alertServer] 
                        ] ifError: [:a :b | "No error for bad e-mail addresses"]]
! !

!NuSwikiPage methodsFor:'xpswiki'!

addToCatalog
	^ self
!

book: aSwikiBook
	book _ aSwikiBook
!

removeFromCatalog
	^ self
! !

!NuSwikiPage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/NuSwikiPage.st,v 1.20 2006/07/27 17:45:46 cg Exp $'
! !

NuSwikiPage initialize!
