"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

TextFormatter subclass:#PageFormatter
	instanceVariableNames:'storeID'
	classVariableNames:'ToSafeAlias ToSafeLocation'
	poolDictionaries:''
	category:'Comanche-Swiki-Formatting'
!


!PageFormatter class methodsFor:'instance creation'!

editFormatter
	| return |
	return _ self new.
	return initialize;
		addCrToCrlf;
		addEditHtml;
		addEditCode;
		addChangeLinks;
		addEditSpecialCharacters;
		storeID: #editFormatter.
	^return
!

saveFormatter
	| return |
	return _ self new.
	return
		initialize;
		addCrlfToCr;
		addReverseLinks;
		addSaveHtml;
		addSaveCode;
		addSaveSpecialCharacters;
		storeID: #saveFormatter.
	^return
!

updateFormatter
	| return |
	return _ self new.
	return initialize;
		addUpdateLinks;
		addUpdateHtml;
		addUpdateCode;
		storeID: #updateFormatter.
	^return
! !

!PageFormatter class methodsFor:'formatting'!

toSafeAlias
	^ToSafeAlias
!

toSafeAlias: aString
	^ToSafeAlias format: aString
!

toSafeLocation
	^ToSafeLocation
!

toSafeLocation: aString
	^ToSafeLocation format: aString
! !

!PageFormatter class methodsFor:'initialize-release'!

initialize
	ToSafeAlias _ TextFormatter new.
	ToSafeAlias initialize;
		addMappingFrom: '>' to: '';
		addMappingFrom: '<' to: '';
		addMappingFrom: '@' to: '';
		addMappingFrom: '&star;' to: '*';
		addMappingFrom: '&at;' to: '@';
		addMappingFrom: '&lt;' to: '&lt;';
		addMappingFrom: '&gt;' to: '&gt;';
		addMappingFrom: '&amp;' to: '&amp;';
		addMappingFrom: '&' to: '&amp;'.
	ToSafeLocation _ TextFormatter new.
	ToSafeLocation initialize;
		addMappingFrom: '>' to: '&gt;';
		addMappingFrom: '<' to: '&lt;';
		addMappingFrom: '"' to: '&quot;';
		addMappingFrom: '&' to: '&amp;';
		addMappingFrom: '#' to: ''.
! !

!PageFormatter class methodsFor:'utility'!

copyTreeFromDir: fromDir toDir: toDir
	| fromFile toFile |
	fromDir directoryNames do: [:dName |
		(toDir directoryExists: dName) ifFalse: [toDir createDirectory: dName].
		self copyTreeFromDir: (fromDir directoryNamed: dName) toDir: (toDir directoryNamed: dName)].
	fromDir fileNames do: [:fName |
		(toDir fileExists: fName) ifTrue: [toDir deleteFileNamed: fName].
		fromFile _ fromDir readOnlyFileNamed: fName.
		toFile _ toDir newFileNamed: fName.
		toDir copyFile: fromFile toFile: toFile.
		fromFile close.
		toFile close]
!

deleteTreeForDir: dir
	dir directoryNames do: [:dName |
		self deleteTreeForDir: (dir directoryNamed: dName).
		dir deleteDirectory: dName].
	dir fileNames do: [:fName |
		dir deleteFileNamed: fName]
!

isAnImage: aFileName
	^#('gif' 'jpeg' 'jpg' 'png') includes: ((aFileName copyAfterLast: $.) asLowercase)
! !

!PageFormatter methodsFor:'accessing'!

storeID
	^storeID
!

storeID: id
	storeID _ id
! !

!PageFormatter methodsFor:'functionality'!

addChangeLinks
	self addAction: [:text :request :response :shelf :book :page | self changeLinks: text request: request response: response shelf: shelf book: book page: page] from: '*' to: '*'
!

addCrToCrlf
        |crlf|

        crlf _ String crlf.
        self
                addMappingFrom: String cr to: crlf;
                addMappingFrom: crlf to: crlf;
                addMappingFrom: (Character lf asString) to: crlf
!

addCrlfToCr

        |crlf|

        crlf _ String crlf.
        self
                addMappingFrom: crlf to: String cr;
                addMappingFrom: (Character lf asString) to: String cr
!

addEditCode
	self addAction: [:text :request :response :shelf :book :page | '&lt;code&gt;', (TextFormatter encodeToXmlCrlf: text), '&lt;/code&gt;'] from: '<code>' to: '</code>'.
	self addAction: [:text :request :response :shelf :book :page | '&lt;CODE&gt;', (TextFormatter encodeToXmlCrlf: text), '&lt;/CODE&gt;'] from: '<CODE>' to: '</CODE>'
!

addEditHtml
	self addAction: [:text :request :response :shelf :book :page | '&lt;html&gt;', (TextFormatter encodeToXmlCrlf: text), '&lt;/html&gt;'] from: '<html>' to: '</html>'.
	self addAction: [:text :request :response :shelf :book :page | '&lt;HTML&gt;', (TextFormatter encodeToXmlCrlf: text), '&lt;/HTML&gt;'] from: '<HTML>' to: '</HTML>'
!

addEditSpecialCharacters
	self addMappingFrom: '&amp;' to: '&amp;amp;'.
	self addMappingFrom: '&lt;' to: '&amp;lt;'.
	self addMappingFrom: '&gt;' to: '&amp;gt;'.
	self addMappingFrom: '&*' to: '&amp;star;'.
	self addMappingFrom: '&' to: '&amp;'.
	self addMappingFrom: '"' to: '&quot;'.
	self addMappingFrom: '<' to: '&lt;'.
	self addMappingFrom: '>' to: '&gt;'
!

addInternalLinks
	self addAction: [:text :request :response :shelf :book :page | self internalLinks: text request: request response: response shelf: shelf book: book page: page] from: '*' to: '*'.
	self addMappingFrom: '&*' to: '*'
!

addRenderInternalLinks
	self addAction: [:text :request :response :shelf :book :page | self renderInternalLinks: text request: request response: response shelf: shelf book: book page: page] from: '*' to: '*'.
	self addMappingFrom: '&*' to: '*'
!

addRenderUploadLinks
	self addAction: [:text :request :response :shelf :book :page | self renderUploadLinks: text request: request response: response shelf: shelf book: book page: page] from: '*+' to: '+*'
!

addReverseLinks
	self addAction: [:text :request :response :shelf :book :page | self reverseLinks: text request: request response: response shelf: shelf book: book page: page] from: '*' to: '*'.
	self addAction: [:text :request :response :shelf :book :page | '*+', text, '+*'] from: '*+' to: '+*'.
	self addMappingFrom: '&*' to: '&*'
!

addSaveCode
	self addAction: [:text :request :response :shelf :book :page | '<code>', (TextFormatter crlfToCr: text), '</code>']from: '<code>' to: '</code>'.
	self addAction: [:text :request :response :shelf :book :page | '<CODE>', (TextFormatter crlfToCr: text), '</CODE>']from: '<CODE>' to: '</CODE>'
!

addSaveHtml
	self addAction: [:text :request :response :shelf :book :page | '<html>', (TextFormatter crlfToCr: text), '</html>']from: '<html>' to: '</html>'.
	self addAction: [:text :request :response :shelf :book :page | '<HTML>', (TextFormatter crlfToCr: text), '</HTML>']from: '<HTML>' to: '</HTML>'
!

addSaveSpecialCharacters
	self addMappingFrom: '&&' to: '&amp;'.
	self addMappingFrom: '&<' to: '&lt;'.
	self addMappingFrom: '&>' to: '&gt;'
!

addShowSpecialCharacters
	self addMappingFrom: '&amp;' to: '&amp;'.
	self addMappingFrom: '&star;' to: '*'.
	self addMappingFrom: '&at;' to: '@'
!

addTagIntegrity
	self addAction: [:text :request :response :shelf :book :page | '<', text, '>' ] from: '<' to: '>'
!

addUpdateCode
	self addAction: [:text :request :response :shelf :book :page | '<code>', text, '</code>'] from: '<code>' to: '</code>'.
	self addAction: [:text :request :response :shelf :book :page | '<CODE>', text, '</CODE>'] from: '<CODE>' to: '</CODE>'
!

addUpdateHtml
	self addAction: [:text :request :response :shelf :book :page | '<html>', text, '</html>'] from: '<html>' to: '</html>'.
	self addAction: [:text :request :response :shelf :book :page | '<HTML>', text, '</HTML>'] from: '<HTML>' to: '</HTML>'
!

addUpdateLinks
	self addAction: [:text :request :response :shelf :book :page | self updateLinks: text request: request response: response shelf: shelf book: book page: page] from: '*' to: '*'.
	self addAction: [:text :request :response :shelf :book :page | '*+', text, '+*'] from: '*+' to: '+*'.
	self addMappingFrom: '&*' to: '&*'.
!

addUploadLinks
	self addAction: [:text :request :response :shelf :book :page | self uploadLinks: text request: request response: response shelf: shelf book: book page: page] from: '*+' to: '+*'
! !

!PageFormatter methodsFor:'private'!

action: action value: value text: text
        value at: 1 put: text.
        ^action copy fixTemps valueWithArguments: value
!

storeString
	"Overwrite of Object method to save space for storing"
	(storeID) ifNotNil: ["Check to make sure this storeID is valid"
		(self class respondsTo: storeID)
			ifTrue: [^'(', self class asString, ' ', storeID asString, ')']].
	^super storeString
! !

!PageFormatter methodsFor:'processing'!

format: text request: request response: response shelf: shelf book: book page: page
	|value stream|

	stream _ WriteStream on: (String new: text size).
	value _ Array with: nil with: request with: response with: shelf with: book with: page.
	self format: text actionValue: value onTo: stream.
	^stream contents
! !

!PageFormatter methodsFor:'utility'!

changeLinks: text request: request response: response shelf: shelf book: book page: page
	"edit formatter"
	| alias remaining pageEntry |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>), '>'.
			((alias size) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 1) to: (remaining size)]]
		ifFalse: [alias _ ''].
	"e-mail address"
	(NuSwikiPage isTextAnEmailAddress: remaining)
		ifTrue: [^'*', (TextFormatter encodeToXmlCrlf: (alias, remaining)), '*'].
	"http address"
	(NuSwikiPage hasAProtocol: remaining)
		ifTrue: [^'*', (TextFormatter encodeToXmlCrlf: (alias, remaining)), '*'].
	"Get pageEntry"
	(remaining includes: $@)
		ifTrue: [pageEntry _ remaining copyUpTo: $@.
			remaining _ remaining copyFrom: (pageEntry size + 1) to: (remaining size)]
		ifFalse: [pageEntry _ remaining.
			remaining _ ''].
	"Transform digit to title"
	((pageEntry isAllDigits) and: [pageEntry ~= ''])
		ifTrue: [(book pages size) >= (pageEntry asNumber)
			ifTrue: [^'*', (TextFormatter encodeToXmlCrlf: (alias, (book pages at: pageEntry asNumber) name, remaining)), '*']].
	^'*', (TextFormatter encodeToXmlCrlf: (alias, pageEntry, remaining)), '*'
!

internalLinks: text request: request response: response shelf: shelf book: book page: page
	"show formatter"
	| alias remaining pageEntry location lPage url |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>).
			((alias size + 1) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 2) to: (remaining size)].
			alias _ PageFormatter toSafeAlias: alias]
		ifFalse: [alias _ ''].
	"e-mail address"
	(NuSwikiPage isTextAnEmailAddress: remaining)
		ifTrue: [(alias = '')
			ifTrue: [^'<a href="mailto:', remaining, '">', remaining, '</a>']
			ifFalse: [^'<a href="mailto:', remaining, '">', alias, '</a>']].
	"URL address"
	(NuSwikiPage hasAProtocol: remaining)
		ifTrue: [url _ NuSwikiPage urlForProtocolText: remaining.
			(self class isAnImage: remaining)
				ifTrue: [(alias = '')
					ifTrue: [^'<img alt="External Image" src="', url, '">']
					ifFalse: [^'<img alt="', alias, '" src="', url, '">']]
				ifFalse: [(alias = '')
					ifTrue: [^'<a href="', url, '">', remaining, '</a>']
					ifFalse: [^'<a href="', url, '">', alias, '</a>']]].
	"Get pageEntry and location"
	(remaining includes: $@)
		ifTrue: [pageEntry _ remaining copyUpTo: $@.
			((pageEntry size + 1) = (remaining size))
				ifTrue: [location _ '']
				ifFalse: [location _ remaining copyFrom: (pageEntry size + 2) to: (remaining size)]]
		ifFalse: [pageEntry _ remaining.
			location _ ''].
	"No page entry"
	(pageEntry = '') ifTrue: [(location = '')
		ifTrue: [^'Error: this should not happen']
		ifFalse: [(alias = '')
			ifTrue: [^'<a href="#', (PageFormatter toSafeLocation: location), '">', (PageFormatter toSafeAlias: location), '</a>']
			ifFalse: [^'<a href="#', (PageFormatter toSafeLocation: location), '">', alias, '</a>']]].
	"Page Link"
	(alias = '')
		ifTrue: [request settingsRemove: 'alias']
		ifFalse: [request settingsAt: 'alias' put: alias].
	(location = '')
		ifTrue: [request settingsRemove: 'location']
		ifFalse: [request settingsAt: 'location' put: location].
	(pageEntry isAllDigits)
		ifTrue: [(book pages size) >= (pageEntry asNumber)
			ifTrue: [lPage _ book pages at: pageEntry asNumber]
			ifFalse: [^'Error: this should not happen']]
		ifFalse: [(book hasPageNamed: pageEntry)
			ifTrue: [lPage _ book pageNamed: pageEntry]
			ifFalse: ["New Page: uses newlink.page template"
				request settingsAt: 'pageEntry' put: pageEntry.
				^book formatPageTemplate: 'newLink' request: request response: response shelf: shelf page: page]].
	"Existing Page: uses link.page template"
	^book formatPageTemplate: 'link' request: request response: response shelf: shelf page: lPage.
!

renderInternalLinks: text request: request response: response shelf: shelf book: book page: page
	"render formatter"
	| alias remaining pageEntry rawLocation location lPage url |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>).
			((alias size + 1) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 2) to: (remaining size)].
			alias _ PageFormatter toSafeAlias: alias]
		ifFalse: [alias _ ''].
	"e-mail address"
	(NuSwikiPage isTextAnEmailAddress: remaining)
		ifTrue: [(alias = '')
			ifTrue: [^'<a href="mailto:', remaining, '">', remaining, '</a>']
			ifFalse: [^'<a href="mailto:', remaining, '">', alias, '</a>']].
	"URL address"
	(NuSwikiPage hasAProtocol: remaining)
		ifTrue: [url _ NuSwikiPage urlForProtocolText: remaining.
			(self class isAnImage: remaining)
				ifTrue: [(alias = '')
					ifTrue: [^'<img alt="External Image" src="', url, '">']
					ifFalse: [^'<img alt="', alias, '" src="', url, '">']]
				ifFalse: [(alias = '')
					ifTrue: [^'<a href="', url, '">', remaining, '</a>']
					ifFalse: [^'<a href="', url, '">', alias, '</a>']]].
	"Get pageEntry and location"
	(remaining includes: $@)
		ifTrue: [pageEntry _ remaining copyUpTo: $@.
			((pageEntry size + 1) = (remaining size))
				ifTrue: [location _ '']
				ifFalse: [location _ remaining copyFrom: (pageEntry size + 2) to: (remaining size)]]
		ifFalse: [pageEntry _ remaining.
			location _ ''].
	(location = '') ifFalse: [rawLocation _ (PageFormatter toSafeAlias: location).
		location _ '#', (PageFormatter toSafeLocation: location)].
	"No page entry"
	(pageEntry = '') ifTrue: [(location = '')
		ifTrue: [^'Error: this should not happen']
		ifFalse: [(alias = '')
			ifTrue: [^'<a href="', location, '">', rawLocation, '</a>']
			ifFalse: [^'<a href="', location, '">', alias, '</a>']]].
	"Existing page"
	(pageEntry isAllDigits)
		ifTrue: [
			lPage _ book pages at: (pageEntry asNumber) ifAbsent: [^'Error: this should not happen'].
			(alias = '')
				ifTrue: [(location = '')
					ifTrue: [^'<a href="', pageEntry, '.html">', lPage showName, '</a>']
					ifFalse: [^'<a href="', pageEntry, '.html', location, '">', (lPage showName), ' (', rawLocation, '(</a>']]
				ifFalse: [^'<a href="', pageEntry, '.html', location, '">', alias, '</a>']].
	"Page named this"
	(book hasPageNamed: pageEntry)
		ifTrue: [lPage _ book pageNamed: pageEntry.
			(alias = '')
				ifTrue: [(location = '')
					ifTrue: [^'<a href="', (lPage id asString), '.html">', (lPage showNameRequest: request book: book), '</a>']
					ifFalse: [^'<a href="', (lPage id asString), '.html', location, '">', (lPage showNameRequest: request book: book), ' (', rawLocation, ')</a>']]
				ifFalse: [^'<a href="', (lPage id asString), '.html', location, '">', alias, '</a>']].
	"New page (kind of stupid to render a link that isn't existing)"
	^'*', (TextFormatter encodeToXmlCrlf: text), '*'
!

renderUploadLinks: text request: request response: response shelf: shelf book: book page: page
	"render formatter"
	| alias remaining uploads pageUploads upload |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>).
			((alias size + 1) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 2) to: (remaining size)].
			alias _ PageFormatter toSafeAlias: alias]
		ifFalse: [alias _ ''].
	"use page upload server if possible"
	uploads _ book modulesAt: 'uploads'.
	(pageUploads _ uploads directoryNamed: (page id asString) ifAbsent: [nil])
		ifNil: [upload _ uploads fileRefsCacheAt: remaining ifAbsent: [nil]]
		ifNotNil: [(upload _ pageUploads fileRefsCacheAt: remaining ifAbsent: [nil])
			ifNil: [upload _ uploads fileRefsCacheAt: remaining ifAbsent: [nil]]
			ifNotNil: [uploads _ pageUploads]].
	upload
		ifNil: [^'Missing File (', (TextFormatter encodeToXmlCrlf: remaining), ')'].
	^upload last isAnImage
		ifTrue: ['<img src="uploads/', uploads httpPath, upload last httpName, '" width=', upload last width asString, ' height=', upload last height asString, ' alt="', ((alias = '') ifTrue: ['Uploaded Image: ', upload strictXmlName] ifFalse: [TextFormatter encodeToStrictXmlCrlf: alias]), '">']
		ifFalse: ['<a href="uploads/', uploads httpPath, upload last httpName, '">', ((alias = '') ifTrue: [upload xmlName] ifFalse: [TextFormatter encodeToStrictXmlCrlf: alias]), '</a>']
!

reverseLinks: text request: request response: response shelf: shelf book: book page: page
	"save formatter"
	| alias remaining pageEntry refId |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>), '>'.
			((alias size) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 1) to: (remaining size)]]
		ifFalse: [alias _ ''].
	"e-mail address"
	(NuSwikiPage isTextAnEmailAddress: remaining)
		ifTrue: [^'*', text, '*'].
	"URL address"
	(NuSwikiPage hasAProtocol: remaining)
		ifTrue: [^'*', text, '*'].
	"Get pageEntry"
	(remaining includes: $@)
		ifTrue: [pageEntry _ remaining copyUpTo: $@.
			remaining _ remaining copyFrom: (pageEntry size + 1) to: (remaining size)]
		ifFalse: [pageEntry _ remaining.
			remaining _ ''].
	"No page entry"
	(pageEntry = '')
		ifTrue: [^'*', text, '*'].
	"Check for digit"
	(pageEntry isAllDigits) ifTrue: [
		refId _ pageEntry asNumber.
		(book pages size >= refId) ifTrue: [(page settings at: 'referenceCache' ifAbsentPut: [OrderedCollection new]) addIfNotPresent: refId].
		^'*', alias, pageEntry, remaining, '*'].
	"Transform title to digit"
	(book hasPageNamed: pageEntry) ifTrue: [
		refId _ (book pageNamed: pageEntry) id.
		(page settings at: 'referenceCache' ifAbsentPut: [OrderedCollection new]) 
			addIfNotPresent: refId.
		^'*', alias, (refId asString), remaining, '*'].
	"Not an identified page"
	^'*', text, '*'
!

updateLinks: text request: request response: response shelf: shelf book: book page: page
	"update formatter"
	| alias remaining pageEntry refId |
	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: 
			[alias _ (remaining copyUpTo: $>)
						, '>'.
			alias size = remaining size
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: alias size + 1 to: remaining size]]
		ifFalse: [alias _ ''].
	"e-mail address"
	(NuSwikiPage isTextAnEmailAddress: remaining)
		ifTrue: [^ '*' , text , '*'].
	"URL address"
	(NuSwikiPage hasAProtocol: remaining)
		ifTrue: [^ '*' , text , '*'].
	"Get pageEntry"
	(remaining includes: $@)
		ifTrue: 
			[pageEntry _ remaining copyUpTo: $@.
			remaining _ remaining copyFrom: pageEntry size + 1 to: remaining size]
		ifFalse: 
			[pageEntry _ remaining.
			remaining _ ''].
	"Digit already"
	pageEntry isAllDigits ifTrue: [^ '*' , text , '*'].
	"Transform title to digit"
	(book hasPageNamed: pageEntry)
		ifTrue: 
			[refId _ (book pageNamed: pageEntry) id.
			(page settings at: 'referenceCache' ifAbsentPut: [OrderedCollection new])
				addIfNotPresent: refId.
			^ '*' , alias , refId asString , remaining , '*'].
	"Not an identified page"
	^ '*' , text , '*'
!

uploadLinks: text request: request response: response shelf: shelf book: book page: page
	"show formatter"
	| alias remaining |

	remaining _ text copy.
	"alias"
	(remaining includes: $>)
		ifTrue: [alias _ (remaining copyUpTo: $>).
			((alias size + 1) = (remaining size))
				ifTrue: [remaining _ '']
				ifFalse: [remaining _ remaining copyFrom: (alias size + 2) to: (remaining size)].
			alias _ PageFormatter toSafeAlias: alias]
		ifFalse: [alias _ ''].
	"Use upload.page action"
	(alias = '')
		ifTrue: [request settingsRemove: 'alias']
		ifFalse: [request settingsAt: 'alias' put: alias].
	request settingsAt: 'file' put: remaining.
	^book formatPageAction: 'upload' request: request response: response shelf: shelf page: page
! !

!PageFormatter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/PageFormatter.st,v 1.9 2003/08/29 17:05:27 cg Exp $'
! !

PageFormatter initialize!
