"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiStructure subclass:#SwikiBook
	instanceVariableNames:'setup children pages pageAddresses bookAddresses privAddresses
		pageTemplates bookTemplates pageActions bookActions'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Structure'
!


!SwikiBook methodsFor:'accessing'!

bookActions
	^bookActions
!

bookAddresses
	^bookAddresses
!

bookTemplates
	^bookTemplates
!

pageActions
	^pageActions
!

pageAddresses
	^pageAddresses
!

pageTemplates
	^pageTemplates
!

pages
	^pages
!

pages: anObject
	pages _ anObject
!

privAddresses
	^privAddresses
!

setup
	^setup
!

setup: dict
	setup _ dict
! !

!SwikiBook methodsFor:'inheritance'!

addChild: aBook
	children ifNil: [children _ OrderedCollection new].
	children add: aBook
!

allChildren
	| return |
	return _ OrderedCollection new.
	self hasChildren ifFalse: [^return].
	children do: [:child |
		return add: child.
		return addAll: child allChildren].
	^return
!

children
	^children
!

hasChildren
	^(children = nil) not
!

hasParent
	^false
!

hasSuper: aBook
	^false
!

loadSelfAndChildrenFrom: shelf
	self loadFrom: shelf.
	children ifNotNil: [children do: [:child | child loadSelfAndChildrenFrom: shelf]]
!

removeChild: aBook
	children remove: aBook ifAbsent: [].
	(children size = 0) ifTrue: [children _ nil].
! !

!SwikiBook methodsFor:'initialization'!

initialize
	super initialize.
	pageAddresses _ Dictionary new.
	bookAddresses _ Dictionary new.
	privAddresses _ Dictionary new.
	pageTemplates _ Dictionary new.
	bookTemplates _ Dictionary new.
	pageActions _ Dictionary new.
	bookActions _ Dictionary new
! !

!SwikiBook methodsFor:'modules'!

modules
	^modules copy
!

rawModules
	^modules
!

rawModules: dict
	modules _ dict
!

rawModulesIncludes: key
	^modules includesKey: key
!

rawModulesRemove: key
	^modules removeKey: key ifAbsent: []
! !

!SwikiBook methodsFor:'private'!

bookActionNamed: actionName
	^bookActions at: actionName ifAbsent: [nil]
!

bookAddressNamed: addressName
	^bookAddresses at: addressName ifAbsent: [nil]
!

bookTemplateNamed: templateName
	^bookTemplates at: templateName ifAbsent: [self error: ('book template ', templateName, ' does not exist')]
!

formatBookAction: actionName with: args
	| action |
	^(action _ self bookActionNamed: actionName)
		ifNil: ['<?', actionName, '?>']
		ifNotNil: [action copy fixTemps valueWithArguments: args]
!

formatPageAction: actionName with: args
        "Format book action if no page action is found"
        | action |
        ^(action _ self pageActionNamed: actionName)
                ifNil: [self formatBookAction: actionName with: (args copyFrom: 1 to: 4)]
                ifNotNil: [action copy fixTemps valueWithArguments: args]
!

pageActionNamed: actionName
	^pageActions at: actionName ifAbsent: [nil]
!

pageAddressNamed: addressName
	^pageAddresses at: addressName ifAbsent: [nil]
!

pageTemplateNamed: templateName
	^pageTemplates at: templateName ifAbsent: [self error: ('page template ', templateName, ' does not exist')]
!

privAddressNamed: addressName
	^privAddresses at: addressName ifAbsent: [nil]
! !

!SwikiBook methodsFor:'processing'!

formatBookAction: actionName request: request response: response shelf: shelf
	^self formatBookAction: actionName with: (Array with: request with: response with: shelf with: self)
!

formatBookAddress: addressName request: request response: response shelf: shelf
        | address |
        (address _ self bookAddressNamed: addressName) ifNil: [
                address _ self bookAddressNamed: 'default'].
        ^address copy fixTemps value: request value: response value: shelf value: self
!

formatBookTemplate: templateName request: request response: response shelf: shelf
        ^(BookTemplateFormatter swikiFormatter) format: (self bookTemplateNamed: templateName) request: request response: response shelf: shelf book: self
!

formatPageAction: actionName request: request response: response shelf: shelf page: page
	^self formatPageAction: actionName with: (Array with: request with: response with: shelf with: self with: page)
!

formatPageAddress: addressName request: request response: response shelf: shelf page: page
        | address |
        (address _ self pageAddressNamed: addressName) ifNil: [
                address _ self pageAddressNamed: 'default' ].
        ^address copy fixTemps valueWithArguments: (Array with: request with: response with: shelf with: self with: page)
!

formatPageTemplate: templateName request: request response: response shelf: shelf page: page
        ^(BookTemplateFormatter swikiFormatter) format: (self pageTemplateNamed: templateName) request: request response: response shelf: shelf book: self page: page
!

formatPrivAddress: addressName request: request response: response shelf: shelf
        | address |
        (address _ self privAddressNamed: addressName) ifNil: [^nil].
        ^address copy fixTemps value: request value: response value: shelf value: self
!

process: request response: response shelf: shelf
        "Process the swiki request to return the stream of the text."
        | filter page |
       "Filter Before Processing"
        (filter _ self formatPrivAddress: 'filter' request: request response: response shelf: shelf)
                ifNotNil: ["Return filter after logging"
                        response at: 'content' put: filter.
                        ^self formatPrivAddress: 'log' request: request response: response shelf: shelf].
        "Process Normal Book or Page Request"
        pages ifNotNil: [request page ifNotNil: [
                page _ pages at: (request page) ifAbsent: [nil]]].
        response at: 'content' put: (page
                ifNil: [self formatBookAddress: (request address) request: request response: response shelf: shelf]
                ifNotNil: [self formatPageAddress: (request address) request: request response: response shelf: shelf page: page]).
        "Log After Processing"
        self formatPrivAddress: 'log' request: request response: response shelf: shelf
! !

!SwikiBook methodsFor:'settings'!

rawSettings
	^settings
!

rawSettings: dict
	settings _ dict
!

rawSettingsIncludes: key
	^settings includesKey: key
!

rawSettingsRemove: key
	^settings removeKey: key ifAbsent: []
!

settings
	^settings copy
! !

!SwikiBook methodsFor:'storage'!

loadFrom: shelf
	self loadStorageFrom: shelf.
	storage loadBook: self.
	self loadPages
!

loadPages
	| kind |
	kind _ setup at: 'pageStorage' ifAbsent: [XmlSwikiStorage].
	pages _ (storage dir directoryExists: 'pages')
		ifTrue: [(kind fromDir: (storage dir directoryNamed: 'pages')) loadPages]
		ifFalse: [nil]
!

loadStorageFrom: shelf
	| kind |
	kind _ setup at: 'bookStorage' ifAbsent: [XmlSwikiStorage].
	storage _  kind fromDir: (shelf storage dir directoryNamed: name)
!

writeSettings
	storage writeSettingsForBook: self
! !

!SwikiBook methodsFor:'testing'!

hasBookAddress: addressName
	^bookAddresses includesKey: addressName
!

hasBookTemplate: templateName
	^bookTemplates includesKey: templateName
!

hasPageAddress: addressName
	^pageAddresses includesKey: addressName
!

hasPageNamed: aString
	pages do: [:page | (page name = aString) ifTrue: [^true]].
	^false
!

hasPageTemplate: templateName
	^pageTemplates includesKey: templateName
!

hasPrivAddress: addressName
	^privAddresses includesKey: addressName
! !

!SwikiBook methodsFor:'utility'!

addNewPage
	| test page |
	test _ pages at: 1.
	page _ test class new.
	page
		id: (pages size + 1);
		versionId: 0;
		storage: test storage.
	pages add: page.
	^page
!

pageAlmostNamed: addr ifThere: thereBlock ifNot: notBlock
	| dividers tokens isValid |

	dividers _ OrderedCollection new.
	dividers add: 1.
	(addr size > 1) ifTrue: [
		2 to: (addr size) do: [:i | (addr at: i) isUppercase ifTrue: [dividers add: i]]].
	tokens _ OrderedCollection new.
	(dividers size > 1) ifTrue: [
		1 to: (dividers size - 1) do: [:i | tokens add: (addr copyFrom: (dividers at: i) to: ((dividers at: i + 1) - 1)) asLowercase]].
	tokens add: (addr copyFrom: (dividers at: dividers size) to: addr size) asLowercase.
	pages do: [:page | isValid _ true.
		tokens do: [:token | (((page name asLowercase) findString: token) = 0) ifTrue: [isValid _ false]].
		isValid ifTrue: [^thereBlock value: page]].
	^notBlock value
!

pageNamed: aString
	pages do: [:page | (page name = aString) ifTrue: [^page]].
	^nil
!

recentchanges: beforeBlock during: duringBlock after: afterBlock
	| return sortedPages page date |

	sortedPages _ pages asSortedCollection: [:x :y | (x date = y date) ifTrue: [x time > y time] ifFalse: [x date > y date]].
	page _ sortedPages at: 1.
	date _ page date.
	"First page"
	return _ WriteStream on: String new.
	return nextPutAll: (beforeBlock value: page).
	return nextPutAll: (duringBlock value: page).
	"Other pages"
	(sortedPages size = 1) ifFalse: [
		2 to: (sortedPages size) do: [:i | page _ sortedPages at: i.
			(page date = date) ifFalse: ["New Date"
				date _ page date.
				return nextPutAll: (afterBlock value).
				return nextPutAll: (beforeBlock value: page)].
			return nextPutAll: (duringBlock value: page)]].
	return nextPutAll: (afterBlock value).
	^return contents

!

references: page before: beforeBlock during: duringBlock after: afterBlock ifAbsent: absentBlock
        | results return |

        results _ OrderedCollection new.
        pages do: [:element | ((element settingsAt: 'referenceCache' ifAbsent: [OrderedCollection new]) includes: page id) ifTrue: [results add: element]].
"/        (results includes: (pages at: 1)) ifTrue: [results remove: (pages at: 1)].
        (results size > 0) ifTrue: [
                return _ beforeBlock value: results.
                results do: [:element | return _ return, (duringBlock value: element)].
                ^return, (afterBlock value: results)].
        ^absentBlock value
!

searchFor: text caseSensitive: caseSensitive and: performAnd matchBlock: matchBlock
        | tokens results isMatch remaining pos1 |

        results _ OrderedCollection new.
        "Smart tokenize text with quotes"
        tokens _ OrderedCollection new.
        remaining _ text.
        [(remaining size > 0) and: [(pos1 _ remaining findString: '"' startingAt: 1) > 0]]
                whileTrue: [(pos1 = 1)
                        ifTrue: [(remaining size > 1)
                                ifTrue: [remaining _ remaining copyFrom: 2 to: remaining size.
                                        (pos1 _ remaining findString: '"' startingAt: 1) = 0
                                                ifTrue: [tokens add: remaining.
                                                        remaining _ '']
                                                ifFalse: [pos1 = 1 ifFalse: [tokens add: (remaining copyFrom: 1 to: (pos1 - 1))].
                                                        (pos1 = remaining size)
                                                                ifTrue: [remaining _ '']
                                                                ifFalse: [remaining _ remaining copyFrom: (pos1 + 1) to: remaining size]]]
                                ifFalse: [remaining _ '']]
                        ifFalse: [tokens addAll: ((remaining copyFrom: 1 to: (pos1 - 1)) findTokens: ' ').
                                remaining _ remaining copyFrom: pos1 to: remaining size]].
        tokens addAll: (remaining findTokens: ' ').
"xx"        "Return Results"
"xx        results _ OrderedCollection new.
 xx"
        (tokens size = 0) ifFalse: [
                performAnd
                        ifTrue: [pages do: [:page | "Test Pages in an AND search manner"
                                isMatch _ true.
                                tokens do: [:token |
                                        ((page textContains: token caseSensitive: caseSensitive) or:
                                                [page name includesSubstring: token caseSensitive: caseSensitive])
                                                        ifFalse: [isMatch _ false]].
                                isMatch ifTrue: [matchBlock value: page]]]
                        ifFalse: [pages do: [:page | "Test Pages in an OR search manner"
                                isMatch _ false.
                                tokens do: [:token |
                                        ((page textContains: token caseSensitive: caseSensitive) or:
                                                [page name includesSubstring: token caseSensitive: caseSensitive])
                                                        ifTrue: [isMatch _ true]].
                                isMatch ifTrue: [matchBlock value: page]]]].
        ^results
!

showName
	^(TextFormatter encodeToXmlCrlf: name)
! !

!SwikiBook class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiBook.st,v 1.11 2003/11/17 10:47:59 penk Exp $'
! !
