"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

StringHolder subclass:#SwikiBrowser
	instanceVariableNames:'shelf bookList bookListIndex functionList functionListIndex
		categoryList categoryListIndex elementList elementListIndex
		topView booksAsHierarchy'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Comanche'
!

SwikiBrowser comment:'SwikiBrowser

this is a browser to edit the swikis.

To open, use the ComSwikiLauncher:
1. Start the server.
2. Chose "open swiki browser..." from the window menu (next to the X)'
!

!SwikiBrowser class methodsFor:'documentation'!

examples
"
     ComancheSwikiService allInstances first browseWithSwikiBrowser
"
! !

!SwikiBrowser class methodsFor:'instance creation'!

asMVCOnShelf: aSwikiShelf
        ^self asMVCOnShelf: aSwikiShelf label: 'Swiki Browser'
!

asMVCOnShelf: aSwikiShelf label: label 
        Smalltalk isSmalltalkX ifTrue:[
            ^ self stx_asMVCOnShelf: aSwikiShelf label: label
        ].
        ^ self squeak_asMVCOnShelf: aSwikiShelf label: label
!

asMorphOnShelf: aSwikiShelf
        ^self asMorphOnShelf: aSwikiShelf label: 'Swiki Browser'
!

asMorphOnShelf: aSwikiShelf label: label
        | aSwikiBrowser window |

        Smalltalk isSmalltalkX ifTrue:[
            self error:'unimplemented'.
        ].

        aSwikiBrowser _ self new shelf: aSwikiShelf.
        window _ (SystemWindow labelled: label) model: aSwikiBrowser.
        "book list"
        window addMorph: (PluggableListMorph on: aSwikiBrowser list: #bookList selected: #bookListIndex changeSelected: #bookListIndex: menu: #bookListMenu:) frame: (0@0 corner: 0.25@0.3).
        "function list"
        window addMorph: (PluggableListMorph on: aSwikiBrowser list: #functionList selected: #functionListIndex changeSelected: #functionListIndex: menu: nil) frame: (0.25@0 corner: 0.5@0.3).
        "category list"
        window addMorph: (PluggableListMorph on: aSwikiBrowser list: #categoryList selected: #categoryListIndex changeSelected: #categoryListIndex: menu: #categoryListMenu:) frame: (0.5@0 corner: 0.75@0.3).
        "element list"
        window addMorph: (PluggableListMorph on: aSwikiBrowser list: #elementList selected: #elementListIndex changeSelected: #elementListIndex: menu: #elementListMenu:) frame: (0.75@0 corner: 1@0.3).
        "content area"
        window addMorph: (PluggableTextMorph on: aSwikiBrowser text: #contents accept: #contents:notifying: readSelection: #contentsSelection menu: #contentsMenu:shifted:) frame: (0@0.3 corner: 1@1).
        ^window
!

open

    self openAllRunningShelfs ifFalse:[
        (Dialog request:'There is no Swiki running - load from directory ?') ifTrue:[
            self openFromDirectory.
        ].
    ].
    "
     self open
    "
!

openAllRunningShelfs
    | allInstances |
    allInstances := ComancheSwikiService allInstances.
    allInstances do:[:aSwikiService|
        aSwikiService module notNil ifTrue:[
            self openOn: (aSwikiService module shelf)
                    label:'Swiki Browser for ', aSwikiService comancheSwikiRoot asString.
        ].
    ].
    allInstances isEmpty ifTrue:[
        ^ false
    ].
    ^ true
    "
     self openAllRunningShelfs
    "
!

openFromDirectory

    |module defaultDirectory directory|

    defaultDirectory := ComancheSwikiService comancheSwikiRoot.
    directory := Dialog requestDirectoryName:'Select a Comanche Swiki Data Directory' default:defaultDirectory.
    directory isEmpty ifTrue:[^ self].
    module := SwikiModule swikiWebServerFromDir:directory.
    module ifNil: [Dialog warn: 'Did not start!!'. ^ self].
    ^ self openOn:module shelf
            label:'Swiki Browser for ', directory asString.

"
self openFromDirectory
"
!

openOn: aSwikiShelf
        ^ self asMVCOnShelf: aSwikiShelf
!

openOn: aSwikiShelf  label:aLabel
        ^ self asMVCOnShelf: aSwikiShelf label:aLabel
!

squeak_asMVCOnShelf: aSwikiShelf label: label 
        | aSwikiBrowser topView bookListView categoryListView functionListView elementListView contentsArea topViewSize subViewSize backgroundColor |

        backgroundColor := Color paleBlue.
        topViewSize := 0 @ 0 extent: 800 @ 600.
        subViewSize := 0 @ 0 extent: topViewSize width / 4 @ (topViewSize height / 3).

        aSwikiBrowser := self new shelf: aSwikiShelf.
        topView := (StandardSystemView new label: label)
                                model: aSwikiShelf.
        topView window: topViewSize.
        topView backgroundColor: backgroundColor.

        "book list"
        bookListView := PluggableListView
                                on: aSwikiBrowser
                                list: #bookList
                                selected: #bookListIndex
                                changeSelected: #bookListIndex:
                                menu: #bookListMenu:.
        bookListView window: subViewSize.
        topView addSubView: bookListView.

        "function list"
        functionListView := PluggableListView
                                on: aSwikiBrowser
                                list: #functionList
                                selected: #functionListIndex
                                changeSelected: #functionListIndex:
                                menu: nil.

        functionListView window: subViewSize.
        topView addSubView: functionListView toRightOf: bookListView.

        "category list"
        categoryListView := PluggableListView
                                on: aSwikiBrowser
                                list: #categoryList
                                selected: #categoryListIndex
                                changeSelected: #categoryListIndex:
                                menu: #categoryListMenu:.
        categoryListView window: subViewSize.
        topView addSubView: categoryListView toRightOf: functionListView.

        "element list"
        elementListView := PluggableListView
                                on: aSwikiBrowser
                                list: #elementList
                                selected: #elementListIndex
                                changeSelected: #elementListIndex:
                                menu: #elementListMenu:.
        elementListView window: subViewSize.
        topView addSubView: elementListView toRightOf: categoryListView.

        "content area"
        contentsArea := PluggableTextView
                                on: aSwikiBrowser
                                text: #contents
                                accept: #contents:notifying:
                                readSelection: #contentsSelection
                                menu: #contentsMenu:shifted:.
        topView addSubView: contentsArea below: bookListView.

        topView controller open
!

stx_asMVCOnShelf: aSwikiShelf label: label 
        | aSwikiBrowser topView bookListView categoryListView functionListView elementListView contentsArea topViewSize subViewSize frame lbl panel topPart|

        "/ backgroundColor := Color paleBlue.
        topViewSize := 0 @ 0 extent: 800 @ 600.
        subViewSize := 0 @ 0 extent: topViewSize width / 4 @ (topViewSize height / 3).

        aSwikiBrowser := self new shelf: aSwikiShelf.
        topView := (StandardSystemView new label: label)
                                model: aSwikiShelf.

        topView extent: 800 @ 600.
        "/ topView backgroundColor: backgroundColor.

        panel := VariableVerticalPanel new.
        panel origin:0.0@0.0 corner:1.0@1.0.
        topView addSubView:panel.

        topPart := View new.

        "book list"
        bookListView := PluggableListView
                                on: aSwikiBrowser
                                list: #bookList
                                selected: #bookListIndex
                                changeSelected: #bookListIndex:
                                menu: #bookListMenu:.

        frame := View new.
        lbl := Label label:'Book'.

        frame addSubView: lbl.
        frame addSubView: bookListView.
        topPart addSubView: frame.

        lbl layout:(((Rectangle origin: 0.0@0.0 corner:1.0@0.0) asLayout) bottomOffset:25).
        bookListView layout:(((Rectangle origin: 0.0@0.0 corner:1.0@1.0) asLayout) topOffset:25).
        frame layout:((Rectangle origin: 0.0@0.0 corner:0.25@1.0) asLayout).


        "function list"
        functionListView := PluggableListView
                                on: aSwikiBrowser
                                list: #functionList
                                selected: #functionListIndex
                                changeSelected: #functionListIndex:
                                menu: nil.

        frame := View new.
        lbl := Label label:'Function'.

        frame addSubView: lbl.
        frame addSubView: functionListView.
        topPart addSubView: frame.

        lbl layout:(((Rectangle origin: 0.0@0.0 corner:1.0@0.0) asLayout) bottomOffset:25).
        functionListView layout:(((Rectangle origin: 0.0@0.0 corner:1.0@1.0) asLayout) topOffset:25).
        frame layout:((Rectangle origin: 0.25@0.0 corner:0.5@1.0) asLayout).


        "category list"
        categoryListView := PluggableListView
                                on: aSwikiBrowser
                                list: #categoryList
                                selected: #categoryListIndex
                                changeSelected: #categoryListIndex:
                                menu: #categoryListMenu:.


        frame := View new.
        lbl := Label label:'Category'.

        frame addSubView: lbl.
        frame addSubView: categoryListView.
        topPart addSubView: frame.

        lbl layout:(((Rectangle origin: 0.0@0.0 corner:1.0@0.0) asLayout) bottomOffset:25).
        categoryListView layout:(((Rectangle origin: 0.0@0.0 corner:1.0@1.0) asLayout) topOffset:25).
        frame layout:((Rectangle origin: 0.5@0.0 corner:0.75@1.0) asLayout).


        "element list"
        elementListView := PluggableListView
                                on: aSwikiBrowser
                                list: #elementList
                                selected: #elementListIndex
                                changeSelected: #elementListIndex:
                                menu: #elementListMenu:.

        frame := View new.
        lbl := Label label:'Element'.

        frame addSubView: lbl.
        frame addSubView: elementListView.
        topPart addSubView: frame.

        lbl layout:(((Rectangle origin: 0.0@0.0 corner:1.0@0.0) asLayout) bottomOffset:25).
        elementListView layout:(((Rectangle origin: 0.0@0.0 corner:1.0@1.0) asLayout) topOffset:25).
        frame layout:((Rectangle origin: 0.75@0.0 corner:1.0@1.0) asLayout).


        "content area"
        contentsArea := PluggableTextView
                                on: aSwikiBrowser
                                text: #contents
                                accept: #acceptContents:
                                readSelection: #contentsSelection
                                menu: nil.

        panel addSubView:topPart.
        panel addSubView: contentsArea.

        topPart origin:0.0@0.0 corner:1.0@0.33.
        contentsArea origin: 0.0@0.33 corner:1.0@1.0.

        aSwikiBrowser topView:topView.
        topView controller open.
        "
         self open
        "
! !

!SwikiBrowser class methodsFor:'testing'!

isVisualStartable
    ^ true
! !

!SwikiBrowser methodsFor:'accessing'!

shelf
        ^shelf
!

shelf: aSwikiShelf
        shelf _ aSwikiShelf
!

topView
    ^ topView
!

topView:something
    topView := something.
! !

!SwikiBrowser methodsFor:'book list'!

book
        ^(shelf books select: [:book | book name = self bookName]) first
!

bookList
        bookList ifNil: [self createBookList].
        ^bookList
!

bookListIndex
        ^bookListIndex ifNil: [0] ifNotNil: [bookListIndex]
!

bookListIndex: anInteger
        bookListIndex _ anInteger ? 0.
        self changed: #bookListIndex.
        self updateFunctionList
!

bookName
        ^(bookList at: bookListIndex) withBlanksTrimmed
!

createBookList
        booksAsHierarchy
                ifTrue: [
                        bookList _ OrderedCollection new.
                        ((shelf books select: [:book | book hasParent not]) asSortedCollection: [:a :b | a name asLowercase < b name asLowercase]) do: [:book |
                                bookList addAll: (self hierarchyForBook: book indent: '')]]
                ifFalse: [bookList _ ((shelf books collect: [:book | book name]) asSortedCollection: [:a :b | a asLowercase < b asLowercase]) asOrderedCollection].
        bookList addFirst: '[shelf]'
!

isSelectingABook
        ^bookListIndex > 1
!

isSelectingAShelf
        ^bookListIndex = 1
! !

!SwikiBrowser methodsFor:'book list menu'!

bookListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "List as Hierarchy"
        labels add: (booksAsHierarchy
                ifTrue: ['sort alphabetically']
                ifFalse: ['sort as hierarchy']).
        selections add: #toggleBooksAsHierarchy.
        self isSelectingABook ifTrue: [
                "inspect & explore"
                lines add: labels size.
                labels add: 'inspect'.
                selections add: #inspectBook.
                labels add: 'explore'.
                selections add: #exploreBook.
                "reload"
                lines add: labels size.
                labels add: 'reload'.
                selections add: #reloadBook.
                (self book hasChildren) ifFalse: [
                        "remove"
                        lines add: labels size.
                        labels add: 'remove'.
                        selections add: #removeBook]].
        self isSelectingAShelf ifTrue: [
                "inspect & explore"
                lines add: labels size.
                labels add: 'inspect'.
                selections add: #inspectShelf.
                labels add: 'explore'.
                selections add: #exploreShelf.
                "reload"
                lines add: labels size.
                labels add: 'reload'.
                selections add: #reloadShelf].
        ^aMenu labels: labels lines: lines selections: selections
!

exploreBook
        self isSelectingABook ifTrue: [self book explore]
!

exploreShelf
        shelf explore
!

inspectBook
        self isSelectingABook ifTrue: [self book inspect]
!

inspectShelf
        shelf inspect
!

reloadBook
        | book |
        Cursor execute showWhile: [
                book _ self book.
                book formatPrivAddress: 'stop' request: nil response: nil shelf: shelf.
                book loadFrom: shelf.
                book formatPrivAddress: 'initialize' request: nil response: nil shelf: shelf.
                "Update list"
                bookList _ nil.
                self changed: #bookList.
                2 to: bookList size do: [:i | ((bookList at: i) withBlanksTrimmed = book name)
                        ifTrue: [self bookListIndex: i]]]
!

reloadShelf
        Cursor execute showWhile: [
                shelf reload.
                self changed: #bookList.
                self bookListIndex: 1]
!

removeBook
        | book |
        book _ self book.
        shelf books remove: book.
        book formatPrivAddress: 'stop' request: nil response: nil shelf: shelf.
        book hasParent ifTrue: [book parent removeChild: book].
        bookList _ nil.
        self changed: #bookList.
        self bookListIndex: 0.
        PopUpMenu notify: ('It is now safe to (re)move the ', book name, String cr, 'directory to permanently remove that Swiki.')
!

toggleBooksAsHierarchy
        | bookName |
        booksAsHierarchy _ booksAsHierarchy not.
        (self bookListIndex > 1) ifTrue: [
                bookName _ self bookName].
        bookList _ nil.
        self changed: #bookList.
        (bookListIndex < 2) ifTrue: [^self].
        "Select same book"
        2 to: (bookList size) do: [:i |
                (bookName = (bookList at: i) withBlanksTrimmed) ifTrue: [
                        ^self bookListIndex: i]]
! !

!SwikiBrowser methodsFor:'category list'!

category
        (self categoryListIndex = 0) ifTrue: [^nil].
        ^categoryList at: categoryListIndex
!

categoryList
        ^categoryList ifNil: [^OrderedCollection new] ifNotNil: [categoryList]
!

categoryListIndex
        ^categoryListIndex ifNil: [0] ifNotNil: [categoryListIndex]
!

categoryListIndex: anInteger
        categoryListIndex _ anInteger ? 0.
        self changed: #categoryListIndex.
        self updateElementList
!

updateCategoryList
        | function |
        function _ self function.
        function ifNil: [
                categoryList _ OrderedCollection new.
                categoryListIndex _ 0.
                self changed: #categoryList.
                self updateElementList.
                ^self].
        (self isSelectingABook) ifTrue: [function caseOf: {
                ['addresses (book)']->["Book Address Categories"
                        categoryList _ self book storage categoriesBookFunction: 'addresses' type: 'book'.
                        categoryListIndex _ 1].
                ['addresses (page)']->["Page Address Categories"
                        categoryList _ self book storage categoriesBookFunction: 'addresses' type: 'page'.
                        categoryListIndex _ 1].
                ['addresses (priv)']->["Private Address Categories"
                        categoryList _ self book storage categoriesBookFunction: 'addresses' type: 'priv'.
                        categoryListIndex _ 1].
                ['templates (book)']->["Book Templates Categories"
                        categoryList _ self book storage categoriesBookFunction: 'templates' type: 'book'.
                                categoryListIndex _ 1].
                ['templates (page)']->["Page Templates Categories"
                        categoryList _ self book storage categoriesBookFunction: 'templates' type: 'page'.
                                categoryListIndex _ 1].
                ['actions (book)']->["Book Actions Categories"
                        categoryList _ self book storage categoriesBookFunction: 'actions' type: 'book'.
                                categoryListIndex _ 1].
                ['actions (page)']->["Page Actions Categories"
                        categoryList _ self book storage categoriesBookFunction: 'actions' type: 'page'.
                                categoryListIndex _ 1].
                ['settings']->["Settings"
                        categoryList _ OrderedCollection with: '-- all --'.
                        categoryListIndex _ 1].
                ['setup']->["Setup"
                        categoryList _ OrderedCollection with: '-- all --'.
                        categoryListIndex _ 1]}
                otherwise: [self error: (function, ' is not a valid function.')]].
        (self isSelectingAShelf) ifTrue: [function caseOf: {
                ['addresses (shelf)']->["Shelf Address Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'addresses' type: 'shelf'.
                        categoryListIndex _ 1].
                ['addresses (priv)']->["Private Address Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'addresses' type: 'priv'.
                        categoryListIndex _ 1].
                ['templates (shelf)']->["Shelf Templates Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'templates' type: 'shelf'.
                                categoryListIndex _ 1].
                ['templates (book)']->["Book Templates Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'templates' type: 'book'.
                                categoryListIndex _ 1].
                ['actions (shelf)']->["Shelf Actions Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'actions' type: 'shelf'.
                                categoryListIndex _ 1].
                ['actions (book)']->["Book Actions Categories"
                        categoryList _ shelf storage categoriesShelfFunction: 'actions' type: 'book'.
                                categoryListIndex _ 1].
                ['settings']->["Settings"
                        categoryList _ OrderedCollection with: '-- all --'.
                        categoryListIndex _ 1]}
                otherwise: [self error: (function, ' is not a valid function.')]].
        self changed: #categoryList.
        self updateElementList
! !

!SwikiBrowser methodsFor:'category list menu'!

addBookATACategory
        | response |
        (response _ FillInTheBlank request: 'Please type new category name' initialAnswer: 'category name') isEmpty ifTrue: [^self].
        self book storage addBookFunction: self ata type: self type category: response.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: response)
!

addShelfATACategory
        | response |
        (response _ FillInTheBlank request: 'Please type new category name' initialAnswer: 'category name') isEmpty ifTrue: [^self].
        shelf storage addShelfFunction: self ata type: self type category: response.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: response)
!

bookATACategoryListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new category"
        labels add: 'new category...'.
        selections add: #addBookATACategory.
        "Selected item"
        self category caseOf:
                {[nil]->["No selection"].
                ['-- all --']->["All is selected"].
                ['as yet unclassified']->[
                        lines add: (labels size).
                        "Rename category"
                        labels add: 'rename...'.
                        selections add: #renameBookATACategory]}
                otherwise: [
                        lines add: (labels size).
                        "Rename category"
                        labels add: 'rename...'.
                        selections add: #renameBookATACategory.
                        "Remove category"
                        labels add: 'remove'.
                        selections add: #removeBookATACategory].
        ^aMenu labels: labels lines: lines selections: selections
!

bookCategoryListMenu: aMenu
        self function caseOf:
                {['addresses (book)']->["Book Address"
                        ^self bookATACategoryListMenu: aMenu].
                ['addresses (page)']->["Page Address"
                        ^self bookATACategoryListMenu: aMenu].
                ['addresses (priv)']->["Private Address"
                        ^self bookATACategoryListMenu: aMenu].
                ['templates (book)']->["Book Template"
                        ^self bookATACategoryListMenu: aMenu].
                ['templates (page)']->["Page Template"
                        ^self bookATACategoryListMenu: aMenu].
                ['actions (book)']->["Book Action"
                        ^self bookATACategoryListMenu: aMenu].
                ['actions (page)']->["Page Action"
                        ^self bookATACategoryListMenu: aMenu].
                ['settings']->["Settings"
                        ^nil].
                ['setup']->["Setup"
                        ^nil].
                [nil]->["No Function"
                        ^nil]}
                otherwise: [self error: (self function, ' is not a valid function.')]
!

categoryListMenu: aMenu
        self isSelectingABook ifTrue: [^self bookCategoryListMenu: aMenu].
        self isSelectingAShelf ifTrue: [^self shelfCategoryListMenu: aMenu].
        ^nil
!

removeBookATACategory
        "Remove category"
        self book storage deleteBookFunction: self ata type: self type category: self category.
        self updateCategoryList.
        self categoryListIndex: 0
!

removeShelfATACategory
        "Remove category"
        shelf storage deleteShelfFunction: self ata type: self type category: self category.
        self updateCategoryList.
        self categoryListIndex: 0
!

renameBookATACategory
        | category response |
        category _ self category.
        (response _ FillInTheBlank request: 'Please type new category name' initialAnswer: category) isEmpty ifTrue: [^self].
        (category = response) ifTrue: [^self].
        (category = 'as yet unclassified') ifTrue: [category _ nil].
        self book storage renameBookFunction: self ata type: self type category: category to: response.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: response)
!

renameShelfATACategory
        | category response |
        category _ self category.
        (response _ FillInTheBlank request: 'Please type new category name' initialAnswer: category) isEmpty ifTrue: [^self].
        (category = response) ifTrue: [^self].
        (category = 'as yet unclassified') ifTrue: [category _ nil].
        shelf storage renameShelfFunction: self ata type: self type category: category to: response.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: response)
!

shelfATACategoryListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new category"
        labels add: 'new category...'.
        selections add: #addShelfATACategory.
        "Selected item"
        self category caseOf:
                {[nil]->["No selection"].
                ['-- all --']->["All is selected"].
                ['as yet unclassified']->[
                        lines add: (labels size).
                        "Rename category"
                        labels add: 'rename...'.
                        selections add: #renameShelfATACategory]}
                otherwise: [
                        lines add: (labels size).
                        "Rename category"
                        labels add: 'rename...'.
                        selections add: #renameShelfATACategory.
                        "Remove category"
                        labels add: 'remove'.
                        selections add: #removeShelfATACategory].
        ^aMenu labels: labels lines: lines selections: selections
!

shelfCategoryListMenu: aMenu
        self function caseOf:
                {['addresses (shelf)']->["Shelf Address"
                        ^self shelfATACategoryListMenu: aMenu].
                ['addresses (priv)']->["Private Address"
                        ^self shelfATACategoryListMenu: aMenu].
                ['templates (shelf)']->["Shelf Template"
                        ^self shelfATACategoryListMenu: aMenu].
                ['templates (book)']->["Book Template"
                        ^self shelfATACategoryListMenu: aMenu].
                ['actions (shelf)']->["Book Action"
                        ^self shelfATACategoryListMenu: aMenu].
                ['actions (book)']->["Page Action"
                        ^self shelfATACategoryListMenu: aMenu].
                ['settings']->["Settings"
                        ^nil].
                [nil]->["No Function"
                        ^nil]}
                otherwise: [self error: (self function, ' is not a valid function.')]
! !

!SwikiBrowser methodsFor:'contents'!

acceptContents:contents

    |codeView|

    codeView := (self topView subViews first subViews second subViews first).
    self contents:contents notifying:codeView
!

bookContents: input notifying: aController
        | inputString inputCode book |
        inputString _ input asString.
        book _ self book.
        self function caseOf:
                {['addresses (book)']->["Book Address"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book' notifying: aController.
                        inputCode ifNil: [^false].
                        book bookAddresses at: self element put: inputCode.
                        book storage bookFunction: 'addresses' type: 'book' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['addresses (page)']->["Page Address"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book :page' notifying: aController.
                        inputCode ifNil: [^false].
                        book pageAddresses at: self element put: inputCode.
                        book storage bookFunction: 'addresses' type: 'page' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['addresses (priv)']->["Private Address"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book' notifying: aController.
                        inputCode ifNil: [^false].
                        book privAddresses at: self element put: inputCode.
                        book storage bookFunction: 'addresses' type: 'priv' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['templates (book)']->["Book Template"
                        book bookTemplates at: self element put: (TextFormatter crToCrlf: inputString).
                        book storage bookFunction: 'templates' type: 'book' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['templates (page)']->["Page Template"
                        book pageTemplates at: self element put: (TextFormatter crToCrlf: inputString).
                        book storage bookFunction: 'templates' type: 'page' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['actions (book)']->["Book Action"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book' notifying: aController.
                        inputCode ifNil: [^false].
                        book bookActions at: self element put: inputCode.
                        book storage bookFunction: 'actions' type: 'book' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['actions (page)']->["Page Action"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book :page' notifying: aController.
                        inputCode ifNil: [^false].
                        book pageActions at: self element put: inputCode.
                        book storage bookFunction: 'actions' type: 'page' name: self element content: inputString.
                        contents _ inputString.
                        ^true].
                ['settings']->["Settings"
                        inputCode _ Compiler evaluate: inputString notifying: aController logged: false.
                        inputCode ifNil: [^false].
                        book rawSettings at: self element put: inputCode.
                        book writeSettings.
                        contents _ aController text asString.
                        ^true].
                ['setup']->["Setup"
                        inputCode _ Compiler evaluate: inputString notifying: aController logged: false.
                        inputCode ifNil: [^false].
                        book setup at: self element put: inputCode.
                        shelf storage writeSetupForBook: book.
                        contents _ aController text asString.
                        ^true]}
                otherwise: [self error: (self function, ' is not a valid function.')].
                ^ false
!

compileCode:inputString blockArgs:blockArgsCode notifying: aController
    |offset offsetShiftingErrorHandler code|

    Smalltalk isSmalltalkX ifTrue:[
        offset := 1 + blockArgsCode size + 2.

        offsetShiftingErrorHandler := Plug new.
        offsetShiftingErrorHandler
            respondTo:#'correctableError:position:to:from:'
            with:[:aString :relPos :relEndPos :aCompiler |
                aController correctableError:aString position:(relPos-offset) to:(relEndPos-offset) from:aCompiler.    
            ].
        offsetShiftingErrorHandler
            respondTo:#'warning:position:to:from:'
            with:[:aString :relPos :relEndPos :aCompiler |
                aController warning:aString position:(relPos-offset) to:(relEndPos-offset) from:aCompiler.    
            ].
        offsetShiftingErrorHandler
            respondTo:#'error:position:to:from:'
            with:[:aString :relPos :relEndPos :aCompiler |
                aController error:aString position:(relPos-offset) to:(relEndPos-offset) from:aCompiler.    
            ].

        code := ('[' , blockArgsCode , ' |', inputString, ']').

        (self checkForReturnInBlockCode:code) ifTrue:[
            (Dialog confirm:'Return in block code not allowed here - apply anyway?') ifFalse:[
                ^ self
            ].
        ].

        ^ Compiler 
            evaluate: code 
            notifying: offsetShiftingErrorHandler 
            logged: false.
    ].

    Parser new 
        parse: (ReadStream on: inputString) 
        class: (nil class) 
        noPattern: true 
        context: (Compiler evaluate: ('[' , blockArgsCode , ' | ]')) 
        notifying: aController 
        ifFail: [^ nil].

    ^ Compiler 
        evaluate: ('[' , blockArgsCode , ' |', inputString, ']') 
        notifying: aController 
        logged: false.
!

contents: input notifying: aController
        |action|

        (self elementListIndex = 0) ifTrue: ["Nothing is selected to update"
                PopUpMenu notify: 'This text cannot be accepted in this part of the browser'.
                ^false].
        action := [
            self isSelectingABook ifTrue: [^self bookContents: input notifying: aController].
            self isSelectingAShelf ifTrue: [^self shelfContents: input notifying: aController]].
        OutOfScopeNotification notNil ifTrue:[
            action on: OutOfScopeNotification do: [:ex | ex resume: true].
        ] ifFalse:[
            action value.
        ].

        ^true
!

contentsMenu: aMenu shifted: shifted
| shiftMenu |

^ shifted 
        ifFalse: [aMenu 
                labels: 
'find...(f)
find again (g)
set search string (h)
do again (j)
undo (z)
copy (c)
cut (x)
paste (v)
paste...
do it (d)
print it (p)
inspect it (i)
accept (s)
cancel (l)
more...' 
                lines: #(3 5 9 12 14)
                selections: #(find findAgain setSearchString
again undo
copySelection cut paste pasteRecent
doIt printIt inspectIt
accept cancel
shiftedYellowButtonActivity)]

        ifTrue: [shiftMenu _ ParagraphEditor shiftedYellowButtonMenu.
                aMenu 
                        labels: shiftMenu labelString 
                        lines: shiftMenu lineArray
                        selections: shiftMenu selections]
!

shelfContents: input notifying: aController
        | inputString inputCode |
        inputString := input asString.
        self function caseOf:
                {['addresses (shelf)']->["Shelf Address"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf' notifying: aController.
                        inputCode ifNil: [^false].
                        shelf shelfAddresses at: self element put: inputCode.
                        shelf storage shelfFunction: 'addresses' type: 'shelf' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['addresses (priv)']->["Private Address"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf' notifying: aController.
                        inputCode ifNil: [^false].
                        shelf privAddresses at: self element put: inputCode.
                        shelf storage shelfFunction: 'addresses' type: 'priv' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['templates (shelf)']->["Shelf Template"
                        shelf shelfTemplates at: self element put: (TextFormatter crToCrlf: inputString).
                        shelf storage shelfFunction: 'templates' type: 'shelf' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['templates (book)']->["Book Template"
                        shelf bookTemplates at: self element put: (TextFormatter crToCrlf: inputString).
                        shelf storage shelfFunction: 'templates' type: 'book' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['actions (shelf)']->["Shelf Action"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf' notifying: aController.
                        inputCode ifNil: [^false].
                        shelf shelfActions at: self element put: inputCode.
                        shelf storage shelfFunction: 'actions' type: 'shelf' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['actions (book)']->["Book Action"
                        inputCode := self compileCode:inputString blockArgs:':request :response :shelf :book' notifying: aController.
                        inputCode ifNil: [^false].
                        shelf bookActions at: self element put: inputCode.
                        shelf storage shelfFunction: 'actions' type: 'book' name: self element content: inputString.
                        contents := inputString.
                        ^true].
                ['settings']->["Settings"
                        inputCode := Compiler evaluate: inputString notifying: aController logged: false.
                        inputCode ifNil: [^false].
                        shelf settingsAt: self element put: inputCode.
                        shelf writeSettings.
                        contents := aController text asString.
                        ^true]}
                otherwise: [self error: (self function, ' is not a valid function.')].
                ^ false
! !

!SwikiBrowser methodsFor:'element list'!

contentFromBookElement
        | book element function|

        function := self function.
        book _ self book.
        element _ elementList at: elementListIndex.
        (Switch new) 
        if:[
            function = 'addresses (book)'
        ] then:[
            "Book Address Categories"
            ^ book storage bookFunction: 'addresses' type: 'book' name: element.
        ];
        if:[
            function = 'addresses (page)'
        ] then:[
            "Page Address Categories"
            ^ book storage bookFunction: 'addresses' type: 'page' name: element.
        ];
        if:[
            function = 'addresses (priv)'
        ] then:[
            "Private Address Categories"
            ^ book storage bookFunction: 'addresses' type: 'priv' name: element.
        ];
        if:[
            function = 'templates (book)'
        ] then:[
            "Book Templates Categories"
            ^ book storage bookFunction: 'templates' type: 'book' name: element.
        ];
        if:[
            function = 'templates (page)'
        ] then:[
            "Page Templates Categories"
            ^ book storage bookFunction: 'templates' type: 'page' name: element.
        ];
        if:[
            function = 'actions (book)'
        ] then:[
            "Book Actions Categories"
            ^ book storage bookFunction: 'actions' type: 'book' name: element.
        ];
        if:[
            function = 'actions (page)'
        ] then:[
            "Page Actions Categories"
            ^ book storage bookFunction: 'actions' type: 'page' name: element.
        ];
        if:[
            function = 'settings'
        ] then:[
            "Settings"
            ^ (book settingsAt: element) storeString.
        ];
        if:[
            function = 'setup'
        ] then:[
            "Setup"
            ^ (book setup at: element) storeString.
        ];
        otherwise:[self error: (function, ' is not a valid function.')];
        value.
!

contentFromShelfElement
        | element |
        element _ elementList at: elementListIndex.
        ^self function caseOf:
                {['addresses (shelf)']->["Shelf Address"
                        shelf storage shelfFunction: 'addresses' type: 'shelf' name: element].
                ['addresses (priv)']->["Private Address"
                        shelf storage shelfFunction: 'addresses' type: 'priv' name: element].
                ['templates (shelf)']->["Shelf Template"
                        shelf storage shelfFunction: 'templates' type: 'shelf' name: element].
                ['templates (book)']->["Book Template"
                        shelf storage shelfFunction: 'templates' type: 'book' name: element].
                ['actions (shelf)']->["Shelf Action"
                        shelf storage shelfFunction: 'actions' type: 'shelf' name: element].
                ['actions (book)']->["Book Action"
                        shelf storage shelfFunction: 'actions' type: 'book' name: element].
                ['settings']->["Settings"
                        (shelf settingsAt: element) storeString]}
                otherwise: [self error: (self function, ' is not a valid function.')]
!

element
        ^elementList at: elementListIndex
!

elementList
        ^elementList ifNil: [^OrderedCollection new] ifNotNil: [elementList]
!

elementListForBook
        | book category function|

        function := self function.
        book _ self book.
        category _ self category.
        (Switch new) 
        if:[
            function = 'addresses (book)'
        ] then:[
            "Book Address Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'addresses' type: 'book'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'addresses' type: 'book'].
            ^book storage bookFunction: 'addresses' type: 'book' category: category.
        ];
        if:[
            function = 'addresses (page)'
        ] then:[
            "Page Address Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'addresses' type: 'page'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'addresses' type: 'page'].
            ^book storage bookFunction: 'addresses' type: 'page' category: category.
        ];
        if:[
            function = 'addresses (priv)'
        ] then:[
            "Private Address Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'addresses' type: 'priv'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'addresses' type: 'priv'].
            ^book storage bookFunction: 'addresses' type: 'priv' category: category.
        ];
        if:[
            function = 'templates (book)'
        ] then:[
            "Book Templates Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'templates' type: 'book'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'templates' type: 'book'].
            ^book storage bookFunction: 'templates' type: 'book' category: category.
        ];
        if:[
            function = 'templates (page)'
        ] then:[
            "Page Templates Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'templates' type: 'page'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'templates' type: 'page'].
            ^book storage bookFunction: 'templates' type: 'page' category: category.
        ];
        if:[
            function = 'actions (book)'
        ] then:[
            "Book Actions Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'actions' type: 'book'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'actions' type: 'book'].
            ^book storage bookFunction: 'actions' type: 'book' category: category.
        ];
        if:[
            function = 'actions (page)'
        ] then:[
            "Page Actions Categories"
            (category = '-- all --') ifTrue: [
                    ^book storage allBookFunction: 'actions' type: 'page'].
            (category = 'as yet unclassified') ifTrue: [
                    ^book storage ayuBookFunction: 'actions' type: 'page'].
            ^book storage bookFunction: 'actions' type: 'page' category: category.
        ];
        if:[
            function = 'settings'
        ] then:[
            "Settings"
            ^book rawSettings keys asSortedCollection: [:a :b | a asLowercase < b asLowercase].
        ];
        if:[
            function = 'setup'
        ] then:[
            "Setup"
            ^book setup keys asSortedCollection: [:a :b | a asLowercase < b asLowercase]
        ];
        otherwise:[self error: (function, ' is not a valid function.')];
        value.
!

elementListForShelf
        | category |
        category _ self category.
        self function caseOf: {
                ['addresses (shelf)']->["Shelf Address Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'addresses' type: 'shelf'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'addresses' type: 'shelf'].
                        ^shelf storage shelfFunction: 'addresses' type: 'shelf' category: category].
                ['addresses (priv)']->["Private Address Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'addresses' type: 'priv'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'addresses' type: 'priv'].
                        ^shelf storage shelfFunction: 'addresses' type: 'priv' category: category].
                ['templates (shelf)']->["Shelf Templates Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'templates' type: 'shelf'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'templates' type: 'shelf'].
                        ^shelf storage shelfFunction: 'templates' type: 'shelf' category: category].
                ['templates (book)']->["Book Templates Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'templates' type: 'book'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'templates' type: 'book'].
                        ^shelf storage shelfFunction: 'templates' type: 'book' category: category].
                ['actions (shelf)']->["Shelf Actions Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'actions' type: 'shelf'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'actions' type: 'shelf'].
                        ^shelf storage shelfFunction: 'actions' type: 'shelf' category: category].
                ['actions (book)']->["Book Actions Categories"
                        (category = '-- all --') ifTrue: [
                                ^shelf storage allShelfFunction: 'actions' type: 'book'].
                        (category = 'as yet unclassified') ifTrue: [
                                ^shelf storage ayuShelfFunction: 'actions' type: 'book'].
                        ^shelf storage shelfFunction: 'actions' type: 'book' category: category].
                ['settings']->["Settings"
                        ^shelf settings keys asSortedCollection: [:a :b | a asLowercase < b asLowercase]]}
                otherwise: [self error: (self function, ' is not a valid function.')]
!

elementListIndex
        ^elementListIndex ifNil: [0] ifNotNil: [elementListIndex]
!

elementListIndex: anInteger
        elementListIndex _ anInteger ? 0.
        self changed: #elementListIndex.
        (elementListIndex isNil or:[elementListIndex = 0]) ifTrue: [
                contents _ ''.
                self changed: #contents.
                ^self].
        (self isSelectingABook) ifTrue: [
                contents _ self contentFromBookElement.
                self changed: #contents.
                ^self].
        (self isSelectingAShelf) ifTrue: [
                contents _ self contentFromShelfElement.
                self changed: #contents.
                ^self]
!

updateElementList
        (self categoryListIndex = 0)
                ifTrue: [elementList _ OrderedCollection new]
                ifFalse: [
                        self isSelectingABook ifTrue: [elementList _ self elementListForBook].
                        self isSelectingAShelf ifTrue: [elementList _ self elementListForShelf]].
        elementListIndex _ 0.
        self changed: #elementListIndex.
        self changed: #elementList.
        contents _ ''.
        self changed: #contents
! !

!SwikiBrowser methodsFor:'element list menu'!

bookATAElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new element"
        labels add: ('new ', self ataSingle, '...').
        selections add: #newBookATAElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Move element"
                labels add: 'move to...'.
                selections add: #moveBookATAElement.
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameBookATAElement.
                "Rename element"
                labels add: 'remove'.
                selections add: #removeBookATAElement].
        ^aMenu labels: labels lines: lines selections: selections
!

bookActionElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Senders of"
        labels add: 'senders of'.
        selections add: #sendersBookAction.
        lines add: (labels size).
        "Add a new element"
        labels add: ('new ', self ataSingle, '...').
        selections add: #newBookATAElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Move element"
                labels add: 'move to...'.
                selections add: #moveBookATAElement.
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameBookATAElement.
                "Remove element"
                labels add: 'remove'.
                selections add: #removeBookATAElement].
        ^aMenu labels: labels lines: lines selections: selections
!

bookElementListMenu: aMenu
        self function caseOf:
                {['addresses (book)']->["Book Address"
                        ^self bookATAElementListMenu: aMenu].
                ['addresses (page)']->["Page Address"
                        ^self bookATAElementListMenu: aMenu].
                ['addresses (priv)']->["Private Address"
                        ^self bookATAElementListMenu: aMenu].
                ['templates (book)']->["Book Template"
                        ^self bookATAElementListMenu: aMenu].
                ['templates (page)']->["Page Template"
                        ^self bookATAElementListMenu: aMenu].
                ['actions (book)']->["Book Action"
                        ^self bookActionElementListMenu: aMenu].
                ['actions (page)']->["Page Action"
                        ^self bookActionElementListMenu: aMenu].
                ['settings']->["Settings"
                        ^self bookSettingsElementListMenu: aMenu].
                ['setup']->["Setup"
                        ^self bookSetupElementListMenu: aMenu].
                [nil]->["No Function"
                        ^nil]}
                otherwise: [self error: (self function, ' is not a valid function.')]
!

bookSettingsElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new element"
        labels add: ('new setting...').
        selections add: #newBookSettingElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameBookSettingElement.
                "Remove element"
                labels add: 'remove'.
                selections add: #removeBookSettingElement].
        ^aMenu labels: labels lines: lines selections: selections
!

bookSetupElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new element"
        labels add: ('new setup...').
        selections add: #newBookSetupElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameBookSetupElement.
                "Remove element"
                labels add: 'remove'.
                selections add: #removeBookSetupElement].
        ^aMenu labels: labels lines: lines selections: selections
!

elementListMenu: aMenu
        self isSelectingABook ifTrue: [^self bookElementListMenu: aMenu].
        self isSelectingAShelf ifTrue: [^self shelfElementListMenu: aMenu].
        ^nil
!

moveBookATAElement
        | categories lines select category |
        categories _ categoryList copy.
        lines _ OrderedCollection new.
        categories remove: '-- all --'.
        (categories includes: 'as yet unclassified') ifTrue: [
                categories remove: 'as yet unclassified'].
        (self category = 'as yet unclassified') ifFalse: [
                (self category = '-- all --') ifFalse: [
                        categories remove: self category]].
        lines add: categories size.
        categories add: 'as yet unclassified'.
        select _ (Dialog 
                     request:'move to category' 
                     initialAnswer:(categories at:4)  
                     list:categories).  




"/        select _ (PopUpMenu labelArray: categories lines: lines)
"/                                startUpWithCaption: 'move to category'.
"/        (select = 0) ifTrue: [^self].
        (select isEmptyOrNil) ifTrue: [^self].
        (select = 'as yet unclassified')
                ifTrue: [self book storage moveBookFunction: self ata type: self type name: self element]
                ifFalse: [self book storage moveBookFunction: self ata type: self type name: self element toCategory: select].
        category _ self category.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: category)
!

moveShelfATAElement
        | categories lines select category |
        categories _ categoryList copy.
        lines _ OrderedCollection new.
        categories remove: '-- all --'.
        (categories includes: 'as yet unclassified') ifTrue: [
                categories remove: 'as yet unclassified'].
        (self category = 'as yet unclassified') ifFalse: [
                (self category = '-- all --') ifFalse: [
                        categories remove: self category]].
        lines add: categories size.
        categories add: 'as yet unclassified'.
        select _ (PopUpMenu labelArray: categories lines: lines)
                                startUpWithCaption: 'move to category'.
        (select = 0) ifTrue: [^self].
        select _ categories at: select.
        (select = 'as yet unclassified')
                ifTrue: [shelf storage moveShelfFunction: self ata type: self type name: self element]
                ifFalse: [shelf storage moveShelfFunction: self ata type: self type name: self element toCategory: select].
        category _ self category.
        self updateCategoryList.
        self categoryListIndex: (categoryList indexOf: category)
!

newBookATAElement
        | response book inACategory |
        (response _ FillInTheBlank request: ('Please name new ', self ataSingle) initialAnswer: '') isEmpty ifTrue: [^self].
        book _ self book.
        inACategory _ self category caseOf: {
                ['as yet unclassified']->[false].
                ['-- all --']->[false]}
                otherwise: [true].
        "Add to book function"
        self function caseOf: {
                ['addresses (book)']->[(book bookAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookAddresses at: response put: (Compiler evaluate: '[:request :response :shelf :book |]')]].
                ['addresses (page)']->[(book pageAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageAddresses at: response put: (Compiler evaluate: '[:request :response :shelf :book :page |]')]].
                ['addresses (priv)']->[(book privAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book privAddresses at: response put: (Compiler evaluate: '[:request :response :shelf :book |]')]].
                ['templates (book)']->[(book bookTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookTemplates at: response put: '']].
                ['templates (page)']->[(book pageTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageTemplates at: response put: '']].
                ['actions (book)']->[(book bookActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookActions at: response put: (Compiler evaluate: '[:request :response :shelf :book |]')]].
                ['actions (page)']->[(book pageActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageActions at: response put: (Compiler evaluate: '[:request :response :shelf :book :page |]')]]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "Add to storage"
        inACategory
                ifTrue: [book storage addBookFunction: self ata type: self type name: response category: self category]
                ifFalse: [book storage addBookFunction: self ata type: self type name: response].
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

newBookSettingElement
        | response book |
        (response _ FillInTheBlank request: ('Please name new setting') initialAnswer: '') isEmpty ifTrue: [^self].
        book _ self book.
        (book rawSettingsIncludes: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        book settingsAt: response put: ''.
        book writeSettings.
        "Update List"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

newBookSetupElement
        | response book |
        (response _ FillInTheBlank request: ('Please name new setup') initialAnswer: '') isEmpty ifTrue: [^self].
        book _ self book.
        (book setup includesKey: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        book setup at: response put: ''.
        shelf storage writeSetupForBook: book.
        "Update List"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

newShelfATAElement
        | response inACategory |
        (response _ FillInTheBlank request: ('Please name new ', self ataSingle) initialAnswer: '') isEmpty ifTrue: [^self].
        inACategory _ self category caseOf: {
                ['as yet unclassified']->[false].
                ['-- all --']->[false]}
                otherwise: [true].
        "Add to shelf function"
        self function caseOf: {
                ['addresses (shelf)']->[(shelf shelfAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfAddresses at: response put: (Compiler evaluate: '[:request :response :shelf |]')]].
                ['addresses (priv)']->[(shelf privAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf privAddresses at: response put: (Compiler evaluate: '[:request :response :shelf |]')]].
                ['templates (shelf)']->[(shelf shelfTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfTemplates at: response put: '']].
                ['templates (book)']->[(shelf bookTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf bookTemplates at: response put: '']].
                ['actions (shelf)']->[(shelf shelfActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfActions at: response put: (Compiler evaluate: '[:request :response :shelf |]')]].
                ['actions (book)']->[(shelf bookActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf bookActions at: response put: (Compiler evaluate: '[:request :response :shelf :book |]')]]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "Add to storage"
        inACategory
                ifTrue: [shelf storage addShelfFunction: self ata type: self type name: response category: self category]
                ifFalse: [shelf storage addShelfFunction: self ata type: self type name: response].
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

newShelfSettingElement
        | response |
        (response _ FillInTheBlank request: ('Please name new setting') initialAnswer: '') isEmpty ifTrue: [^self].
        (shelf settingsIncludes: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        shelf settingsAt: response put: ''.
        shelf writeSettings.
        "Update List"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

removeBookATAElement
        | book element |
        book _ self book.
        element _ self element.
        self function caseOf: {
                ['addresses (book)']->["Remove book address"
                        book bookAddresses removeKey: element.
                        book storage deleteBookFunction: 'addresses' type: 'book' name: element].
                ['addresses (page)']->["Remove page address"
                        book pageAddresses removeKey: element.
                        book storage deleteBookFunction: 'addresses' type: 'page' name: element].
                ['addresses (priv)']->["Remove priv address"
                        book privAddresses removeKey: element.
                        book storage deleteBookFunction: 'addresses' type: 'priv' name: element].
                ['templates (book)']->["Remove book template"
                        book bookTemplates removeKey: element.
                        book storage deleteBookFunction: 'templates' type: 'book' name: element].
                ['templates (page)']->["Remove page template"
                        book pageTemplates removeKey: element.
                        book storage deleteBookFunction: 'templates' type: 'page' name: element].
                ['actions (book)']->["Remove book action"
                        book bookActions removeKey: element.
                        book storage deleteBookFunction: 'actions' type: 'book' name: element].
                ['actions (page)']->["Remove page action"
                        book pageActions removeKey: element.
                        book storage deleteBookFunction: 'actions' type: 'page' name: element]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "Update element list"
        self updateElementList.
        self elementListIndex: 0
!

removeBookSettingElement
        (self book)
                rawSettingsRemove: (self element);
                writeSettings.
        "Update element list"
        self updateElementList.
        self elementListIndex: 0
!

removeBookSetupElement
        | book |
        book _ self book.
        book setup removeKey: self element.
        shelf storage writeSetupForBook: book.
        "Update element list"
        self updateElementList.
        self elementListIndex: 0
!

removeShelfATAElement
        | element |
        element _ self element.
        self function caseOf: {
                ['addresses (shelf)']->["Remove shelf address"
                        shelf shelfAddresses removeKey: element.
                        shelf storage deleteShelfFunction: 'addresses' type: 'shelf' name: element].
                ['addresses (priv)']->["Remove priv address"
                        shelf privAddresses removeKey: element.
                        shelf storage deleteShelfFunction: 'addresses' type: 'priv' name: element].
                ['templates (shelf)']->["Remove shelf template"
                        shelf shelfTemplates removeKey: element.
                        shelf storage deleteShelfFunction: 'templates' type: 'shelf' name: element].
                ['templates (book)']->["Remove book template"
                        shelf bookTemplates removeKey: element.
                        shelf storage deleteShelfFunction: 'templates' type: 'book' name: element].
                ['actions (shelf)']->["Remove shelf action"
                        shelf shelfActions removeKey: element.
                        shelf storage deleteShelfFunction: 'actions' type: 'shelf' name: element].
                ['actions (book)']->["Remove book action"
                        shelf bookActions removeKey: element.
                        shelf storage deleteShelfFunction: 'actions' type: 'book' name: element]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "Update element list"
        self updateElementList.
        self elementListIndex: 0
!

removeShelfSettingElement
        shelf
                settingsRemove: (self element);
                writeSettings.
        "Update element list"
        self updateElementList.
        self elementListIndex: 0
!

renameBookATAElement
        | response element book |
        (response _ FillInTheBlank request: ('Please name new ', self ataSingle) initialAnswer: self element) isEmpty ifTrue: [^self].
        element _ self element.
        (response = element) ifTrue: [^self].
        book _ self book.
        self function caseOf: {
                ['addresses (book)']->[(book bookAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookAddresses at: response put: (book bookAddresses removeKey: element)]].
                ['addresses (page)']->[(book pageAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageAddresses at: response put: (book pageAddresses removeKey: element)]].
                ['addresses (priv)']->[(book privAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book privAddresses at: response put: (book privAddresses removeKey: element)]].
                ['templates (book)']->[(book bookTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookTemplates at: response put: (book bookTemplates removeKey: element)]].
                ['templates (page)']->[(book pageTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageTemplates at: response put: (book pageTemplates removeKey: element)]].
                ['actions (book)']->[(book bookActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book bookActions at: response put: (book bookActions removeKey: element)]].
                ['actions (page)']->[(book pageActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [book pageActions at: response put: (book pageActions removeKey: element)]]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "move in storage"
        book storage renameBookFunction: self ata type: self type name: element to: response.
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

renameBookSettingElement
        | response element book |
        element _ self element. 
        (response _ FillInTheBlank request: ('Please name new setting') initialAnswer: element) isEmpty ifTrue: [^self].
        (response = element) ifTrue: [^self].
        book _ self book.
        (book rawSettingsIncludes: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        book settingsAt: response put: (book settingsAt: element).
        book rawSettingsRemove: element.
        book writeSettings.
        "Update element list"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

renameBookSetupElement
        | response element book |
        element _ self element. 
        (response _ FillInTheBlank request: ('Please name new setup') initialAnswer: element) isEmpty ifTrue: [^self].
        (response = element) ifTrue: [^self].
        book _ self book.
        (book setup includesKey: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        book setup
                at: response put: (book setup at: element);
                removeKey: element.
        shelf storage writeSetupForBook: book.
        "Update element list"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

renameShelfATAElement
        | response element |
        (response _ FillInTheBlank request: ('Please name new ', self ataSingle) initialAnswer: self element) isEmpty ifTrue: [^self].
        element _ self element.
        (response = element) ifTrue: [^self].
        self function caseOf: {
                ['addresses (shelf)']->[(shelf shelfAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfAddresses at: response put: (shelf shelfAddresses removeKey: element)]].
                ['addresses (priv)']->[(shelf privAddresses includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf privAddresses at: response put: (shelf privAddresses removeKey: element)]].
                ['templates (shelf)']->[(shelf shelfTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfTemplates at: response put: (shelf shelfTemplates removeKey: element)]].
                ['templates (book)']->[(shelf bookTemplates includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf bookTemplates at: response put: (shelf bookTemplates removeKey: element)]].
                ['actions (shelf)']->[(shelf shelfActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf shelfActions at: response put: (shelf shelfActions removeKey: element)]].
                ['actions (book)']->[(shelf bookActions includesKey: response)
                        ifTrue: [^PopUpMenu notify: 'That name is already taken']
                        ifFalse: [shelf bookActions at: response put: (shelf bookActions removeKey: element)]]}
                otherwise: [self error: (self function, ' is not a valid function.')].
        "move in storage"
        shelf storage renameShelfFunction: self ata type: self type name: element to: response.
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

renameShelfSettingElement
        | response element |
        element _ self element. 
        (response _ FillInTheBlank request: ('Please name new setting') initialAnswer: element) isEmpty ifTrue: [^self].
        (response = element) ifTrue: [^self].
        (shelf settingsIncludes: response) ifTrue: [
                ^PopUpMenu notify: 'That name is already taken'].
        shelf settingsAt: response put: (shelf settingsAt: element).
        shelf settingsRemove: element.
        shelf writeSettings.
        "Update element list"
        self updateElementList.
        self elementListIndex: (elementList indexOf: response)
!

sendersBookAction
        | myBook books action result return |
        action _ '<?', self element, '?>'.
        myBook _ self book.
        books _ myBook allChildren.
        books add: myBook.
        [myBook hasParent] whileTrue: [
                myBook _ myBook parent.
                books add: myBook].
        result _ OrderedCollection new.
        (self function = 'actions (book)') ifTrue: [
                books do: [:book | book bookTemplates keysAndValuesDo: [:key :template |
                        ((template findString: action) = 0) ifFalse: [
                                result add: (Array with: book with: 'book' with: key)]]]].
        books do: [:book | book pageTemplates keysAndValuesDo: [:key :template |
                ((template findString: action) = 0) ifFalse: [
                        result add: (Array with: book with: 'page' with: key)]]].
        return _ WriteStream on: String new.
        return
                nextPutAll: (result size asString);
                nextPutAll: ' MATCHES'.
        result do: [:arr | return
                nextPut: Character cr;
                nextPutAll: (arr at: 1) name;
                nextPutAll: ': ';
                nextPutAll: (arr at: 2);
                nextPutAll: ' template named ';
                nextPutAll: (arr at: 3)].
        (Workspace new)
                open "Label: ('Senders of ', action)";
                contents: return contents;
                contentsChanged
                
!

sendersShelfAction
        | action result return |
        action _ '<?', self element, '?>'.
        result _ OrderedCollection new.
        (self function = 'actions (shelf)') ifTrue: [
                shelf shelfTemplates keysAndValuesDo: [:key :template |
                        ((template findString: action) = 0) ifFalse: [
                                result add: (Array with: 'shelf' with: key)]]].
        shelf bookTemplates keysAndValuesDo: [:key :template |
                ((template findString: action) = 0) ifFalse: [
                        result add: (Array with: 'book' with: key)]].
        return _ WriteStream on: String new.
        return
                nextPutAll: (result size asString);
                nextPutAll: ' MATCHES'.
        result do: [:arr | return
                nextPut: Character cr;
                nextPutAll: (arr at: 1);
                nextPutAll: ' template named ';
                nextPutAll: (arr at: 2)].
        (Workspace new)
                open"Label: ('Senders of ', action)";
                contents: return contents;
                contentsChanged
!

shelfATAElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new element"
        labels add: ('new ', self ataSingle, '...').
        selections add: #newShelfATAElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Move element"
                labels add: 'move to...'.
                selections add: #moveShelfATAElement.
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameShelfATAElement.
                "Rename element"
                labels add: 'remove'.
                selections add: #removeShelfATAElement].
        ^aMenu labels: labels lines: lines selections: selections
!

shelfActionElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Senders of"
        labels add: 'senders of'.
        selections add: #sendersShelfAction.
        lines add: (labels size).
        "Add a new element"
        labels add: ('new ', self ataSingle, '...').
        selections add: #newShelfATAElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Move element"
                labels add: 'move to...'.
                selections add: #moveShelfATAElement.
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameShelfATAElement.
                "Rename element"
                labels add: 'remove'.
                selections add: #removeShelfATAElement].
        ^aMenu labels: labels lines: lines selections: selections
!

shelfElementListMenu: aMenu
        self function caseOf:
                {['addresses (shelf)']->["Shelf Address"
                        ^self shelfATAElementListMenu: aMenu].
                ['addresses (priv)']->["Private Address"
                        ^self shelfATAElementListMenu: aMenu].
                ['templates (shelf)']->["Shelf Template"
                        ^self shelfATAElementListMenu: aMenu].
                ['templates (book)']->["Book Template"
                        ^self shelfATAElementListMenu: aMenu].
                ['actions (shelf)']->["Book Action"
                        ^self shelfActionElementListMenu: aMenu].
                ['actions (book)']->["Page Action"
                        ^self shelfActionElementListMenu: aMenu].
                ['settings']->["Settings"
                        ^self shelfSettingsElementListMenu: aMenu].
                [nil]->["No Function"
                        ^nil]}
                otherwise: [self error: (self function, ' is not a valid function.')]
!

shelfSettingsElementListMenu: aMenu
        | labels lines selections |
        labels _ OrderedCollection new.
        lines _ OrderedCollection new.
        selections _ OrderedCollection new.
        "Add a new element"
        labels add: ('new setting...').
        selections add: #newShelfSettingElement.
        "Selected item"
        (self elementListIndex = 0) ifFalse: [
                lines add: (labels size).
                "Rename element"
                labels add: 'rename...'.
                selections add: #renameShelfSettingElement.
                "Remove element"
                labels add: 'remove'.
                selections add: #removeShelfSettingElement].
        ^aMenu labels: labels lines: lines selections: selections
! !

!SwikiBrowser methodsFor:'function list'!

ata
        "addresses, templates, or actions"
        ^self function copyUpTo: $ 
!

ataSingle
        "address, template, or action"
        ^self ata caseOf: {
                ['addresses']->['address'].
                ['templates']->['template'].
                ['actions']->['action']}
                otherwise: [self error: 'ataSingle is only valid for addresses, templates, and actions']
!

function
        (self functionListIndex = 0) ifTrue: [^nil].
        ^self functionList at: functionListIndex
!

functionList
        ^functionList ifNil: [^OrderedCollection new] ifNotNil: [functionList]
!

functionListIndex
        ^functionListIndex ifNil: [0] ifNotNil: [functionListIndex]
!

functionListIndex: anInteger
        functionListIndex _ anInteger ? 0.
        self changed: #functionListIndex.
        self updateCategoryList
!

type
        ^self function caseOf: {
                ['addresses (shelf)']->['shelf'].
                ['addresses (book)']->['book'].
                ['addresses (page)']->['page'].
                ['addresses (priv)']->['priv'].
                ['templates (shelf)']->['shelf'].
                ['templates (book)']->['book'].
                ['templates (page)']->['page'].
                ['actions (shelf)']->['shelf'].
                ['actions (book)']->['book'].
                ['actions (page)']->['page']}
                otherwise: ['']
!

updateFunctionList

    | index |
    index := self bookListIndex.
    index = 0 ifTrue:[
        functionList := OrderedCollection new.
    ] ifFalse:[
        index = 1 ifTrue:[
            "Shelf Functions"
            functionList := #('addresses (shelf)'
                              'addresses (priv)'
                              'templates (shelf)'
                              'templates (book)'
                              'actions (shelf)'
                              'actions (book)'
                              'settings').
        ] ifFalse:[ 
            "Book Functions"
            functionList := #('addresses (book)'
                              'addresses (page)'
                              'addresses (priv)'
                              'templates (book)'
                              'templates (page)'
                              'actions (book)'
                              'actions (page)'
                              'settings'
                              'setup').
        ].
    ].

    functionListIndex := 0.
    self changed: #functionList.
    self updateCategoryList
! !

!SwikiBrowser methodsFor:'initialization'!

initialize
        super initialize.
        bookListIndex _ 0.
        functionListIndex _ 0.
        categoryListIndex _ 0.
        elementListIndex _ 0.
        booksAsHierarchy _ false
! !

!SwikiBrowser methodsFor:'private'!

checkForReturnInBlockCode:aString
    |searcher codeTree anyReturn|

    searcher := ParseTreeSearcher new.
    searcher 
        matches: '^ `@expr'
        do:[:aNode :answer | anyReturn := true. "answer add:aNode. self halt. answer" ].

    codeTree := RBParser 
                    parseSearchMethod:'foo ' , aString
                    onError: [:str :pos | Transcript showCR:str. Transcript showCR:pos. nil].

    codeTree notNil ifTrue:[
        anyReturn := false.
        searcher executeTree:codeTree initialAnswer:false.
        ^ anyReturn 
    ].

    ^ false.
!

defaultBackgroundColor
        ^Color r: 0.6 g: 1.0 b: 0.8
!

hierarchyForBook: book indent: indent
        | return |
        return _ OrderedCollection new.
        return add: (indent, book name).
        book hasChildren ifFalse: [^return].
        (book children asSortedCollection: [:a :b | a name asLowercase < b name asLowercase])
                do: [:child | return addAll: (self hierarchyForBook: child indent: (indent, '  '))].
        ^return
!

put: aText
        aText asString inspect.
        ^true "accepted"
! !

!SwikiBrowser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiBrowser.st,v 1.19 2005/02/28 15:44:20 stefan Exp $'
! !
