"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiColorScheme
	instanceVariableNames:'directory fileServer templates altTags buttonEnding activeButton
		emoteEnding mimeEnding'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiColorScheme comment:'SwikiColorScheme

are used by Swikis to reference buttons and graphics.

INSTANCE VARIABLES:
directory = the corresponding SwikiDirectory
fileServer = the corresponding SwikiFileServer
templates = a dictionary mapping names of templates to the templates (such as beforeButtons)
altTags = a dictionary mapping image names to their ALTernate tag (from tags.xml)
buttonEnding = the suffix of button files (such as .gif)
activeButton = a boolean that is true if the button should have a rollover
emoteEnding = the suffix of emote (happy, sad, etc.) files (such as .gif)
mimeEnding = the suffix of mime types (image, video, audio, etc.)'
!


!SwikiColorScheme class methodsFor:'instance creation'!

on: aSwikiDirectory fileServer: fs
	| instance |
	instance _ self new.
	instance
		directory: aSwikiDirectory;
		fileServer: fs;
		initialize.
	^instance
! !

!SwikiColorScheme methodsFor:'accessing'!

directory
	^directory
!

directory: aSwikiDirectory
	directory _ aSwikiDirectory
!

fileServer
	^fileServer
!

fileServer: fs
	fileServer _ fs
! !

!SwikiColorScheme methodsFor:'buttons'!

altFor: button ifAbsent: block
	^altTags at: button ifAbsent: block
!

button: button
        ^self button: button alt: (self altFor: button ifAbsent: ['Missing Alternate Tag'])
!

button: button alt: altText
        ^directory referenceFileNamed: (button, buttonEnding)
                fs: fileServer
                options: (Dictionary new
                        at: 'border' put: 0;
                        at: 'alt' put: altText;
                        yourself)
!

button: button link: link
	^self button: button link: link alt: (self altFor: button ifAbsent: ['Missing Alternate Tag'])
!

button: button link: link alt: altText
	| imgref |
	activeButton
		ifTrue: ["With rollovers"
			imgref _ directory referenceFileNamed: (button, buttonEnding)
				fs: fileServer
				options: (Dictionary new
					at: 'border' put: 0;
					at: 'alt' put: altText;
					at: 'name' put: button;
					yourself).
			^'<a href="', link, '" onMouseOver="rollOver(''', button, ''', true)" onMouseOut="rollOver(''', button, ''', false)">', imgref, '</a>']
		ifFalse: ["Without rollovers"
			imgref _ directory referenceFileNamed: (button, buttonEnding)
				fs: fileServer
				options: (Dictionary new
					at: 'border' put: 0;
					at: 'alt' put: altText;
					yourself).
			^'<a href="', link, '">', imgref, '</a>']
!

emote: emote
	^directory referenceFileNamed: (emote, emoteEnding)
		fs: fileServer
		options: (Dictionary new
			at: 'border' put: 0;
			at: 'alt' put: (self altFor: emote ifAbsent: ['Missing Alternate Tag']);
			yourself)
!

mime: mime alt: altTag
	^directory referenceFileNamed: (mime, mimeEnding)
		fs: fileServer
		options: (Dictionary new
			at: 'border' put: 0;
			at: 'alt' put: altTag;
			yourself)
! !

!SwikiColorScheme methodsFor:'initialization'!

initialize
	self initializeTemplates.
	self initializeEndings.
	self initializeAltTags
!

initializeAltTags
        | file pos1 pos2 lineAction |
        altTags _ Dictionary new.
        (file _ directory fileRefsCacheAt: 'tags.xml' ifAbsent: [nil]) ifNotNil: [
                lineAction := [:line |
                        (line beginsWith: '<button name="') ifTrue: [
                                pos1 _ line findString: '" alt="' startingAt: 15.
                                pos2 _ line findString: '" />' startingAt: (pos1 + 7).
                                altTags
                                        at: (line copyFrom: 15 to: (pos1 - 1))
                                        put: (line copyFrom: (pos1 + 7) to: (pos2 - 1))]].
                (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                    (directory readOnlyFile: file last) contents do:lineAction
                ] ifFalse:[
                    (directory readOnlyFile: file last) contentsOfEntireFile linesDo:lineAction 
                ]].
!

initializeEndings
	"default ending is .gif"
	buttonEnding _ (directory fileRefsExists: 'top.gif')
		ifTrue: ['.gif']
		ifFalse: [(directory fileRefsExists: 'top.png')
			ifTrue: ['.png']
			ifFalse: [(directory fileRefsExists: 'top.jpg')
				ifTrue: ['.jpg']
				ifFalse: ['.gif']]].
	activeButton _ directory fileRefsExists: ('topact', buttonEnding).
	emoteEnding _ (directory fileRefsExists: 'happy.gif')
		ifTrue: ['.gif']
		ifFalse: [(directory fileRefsExists: 'happy.png')
			ifTrue: ['.png']
			ifFalse: [(directory fileRefsExists: 'happy.jpg')
				ifTrue: ['.jpg']
				ifFalse: ['.gif']]].
	mimeEnding _ (directory fileRefsExists: 'folder.gif')
		ifTrue: ['.gif']
		ifFalse: [(directory fileRefsExists: 'folder.png')
			ifTrue: ['.png']
			ifFalse: [(directory fileRefsExists: 'folder.jpg')
				ifTrue: ['.jpg']
				ifFalse: ['.gif']]]
!

initializeTemplates
	| name template |
	templates _ Dictionary new.
	directory filesAlphabetically do: [:versions |
		name _ versions name.
		(name endsWith: '.sm') ifTrue: ["Swiki Markup"
			name _ name copyUpToLast: $..
			template _ (directory readOnlyFile: versions last) contentsOfEntireFile.
			template _ TextFormatter crToCrlf: template.
			templates at: name put: template]]
! !

!SwikiColorScheme methodsFor:'templates'!

formatTemplate: templateName request: request response: response shelf: shelf
	^(ShelfTemplateFormatter swikiFormatter) format: (templates at: templateName ifAbsent: [^'']) request: request response: response shelf: shelf
!

formatTemplate: templateName request: request response: response shelf: shelf book: book
	^(BookTemplateFormatter swikiFormatter) format: (templates at: templateName ifAbsent: [^'']) request: request response: response shelf: shelf book: book
!

formatTemplate: templateName request: request response: response shelf: shelf book: book page: page
	^(BookTemplateFormatter swikiFormatter) format: (templates at: templateName ifAbsent: [^'']) request: request response: response shelf: shelf book: book page: page
! !

!SwikiColorScheme class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiColorScheme.st,v 1.9 2003/08/29 17:04:04 cg Exp $'
! !
