"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiEntry
	instanceVariableNames:'name modificationTime creationTime'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiEntry comment:'SwikiEntry

is similar to FileEntry, but more lightweight. This contains the information that is associated with a specific file or directory. This is an abstract class; it holds the joint features of SwikiFile and SwikiDirectory.

INSTANCE VARIABLES:
name = name of the file (a String)
modificationTime = when the file was last modified (a number)
creationTime = when the file was created (a number)'
!


!SwikiEntry class methodsFor:'instance creation'!

fromArray: arr
	^self new fromArray: arr
! !

!SwikiEntry class methodsFor:'private'!

rootNameFor: aString
	| tokens return ignore |
	tokens _ aString findTokens: '.'.
	(tokens size = 1) ifTrue: [^tokens at: 1].
	return _ WriteStream on: String new.
	return nextPutAll: (tokens at: 1).
	tokens _ tokens copyFrom: 2 to: tokens size.
	ignore _ true.
	tokens do: [:token | (ignore and: [token isAllDigits])
		ifTrue: [ignore _ false]
		ifFalse: [return
			nextPut: $.;
			nextPutAll: token]].
	^return contents
! !

!SwikiEntry class methodsFor:'utility'!

validFileNameFromFileLocation: aString
	^((TextFormatter decodeFromHttp: aString) findTokens: ':/\') last reject: [:i | 
		"No HTML/HTTP characters allowed"
		'&<>"?*' includes: i]
! !

!SwikiEntry methodsFor:'accessing'!

creationDate
	^Date fromSeconds: creationTime
!

creationTime
	^Time fromSeconds: (creationTime \\ 86400)
!

httpName
	^TextFormatter encodeToHttp: name
!

modificationDate
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            ^ modificationTime asDate
        ].
        ^Date fromSeconds:modificationTime
!

modificationTime
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            ^ modificationTime
        ].
        ^Time fromSeconds: (modificationTime \\ 86400)
!

name
	^name
!

strictXmlName
	^TextFormatter encodeToStrictXmlCrlf: name
!

xmlName
	^TextFormatter encodeToXmlCrlf: name
! !

!SwikiEntry methodsFor:'initialization'!

fromArray: arr
	name _ arr at: 1.
	creationTime _ arr at: 2.
	modificationTime _ arr at: 3.
! !

!SwikiEntry methodsFor:'testing'!

isAnImage
	^false
!

isValid
	^true
! !

!SwikiEntry class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiEntry.st,v 1.8 2003/08/29 17:05:31 cg Exp $'
! !
