"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiEntry subclass:#SwikiFile
	instanceVariableNames:'fileSize versions'
	classVariableNames:'MimeToClass'
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiFile comment:'SwikiFile

contains the information that is associated with a specific file. During instance creation, the correct subclass of SwikiFile is chosen according to MIME type.

INSTANCE VARIABLES:
fileSize = size of the file in bytes
versions = the SwikiFileVersions that this file belongs to

CLASS VARIABLES:
MimeToClass = a dictionary of MIMETypes to the subclasses of SwikiFile that handle them'
!


!SwikiFile class methodsFor:'instance creation'!

fromArray: arr onDir: dir
        | mime instance |
        mime _ MIMEDocument guessTypeFromName: (arr at: 1).
        instance _ (MimeToClass at: mime ifAbsent: [SwikiFile]) fromArray: arr.
        instance dir: dir mimeType: mime.
        instance isValid ifFalse: [
                instance _ SwikiFile fromArray: arr.
                instance dir: dir mimeType: mime].
        ^instance
! !

!SwikiFile class methodsFor:'initialize-release'!

handlesMime: aMimeType
	MimeToClass at: aMimeType put: self
!

initialize
	MimeToClass _ Dictionary new
! !

!SwikiFile methodsFor:'accessing'!

fileSize
	^fileSize
!

fileSizeInWords
	(fileSize < 1000) ifTrue: [^fileSize asString, ' b'].
	(fileSize < 10000) ifTrue: [^((fileSize // 100) / 10) asFloat asString, ' kb'].
	(fileSize < 1000000) ifTrue: [^(fileSize // 1000) asString, ' kb'].
	(fileSize < 10000000) ifTrue: [^((fileSize // 100000) / 10) asFloat asString, ' Mb'].
	^(fileSize // 1000000) asString, ' Mb'
!

mimeType
	^versions mimeType
!

rootName
	^self class rootNameFor: name
!

version
	| tokens |
	"Files have versions of the form myfile.3.ext, where 3 is the version.
	A file without a numbered extension has version of 0"
	tokens _ name findTokens: '.'.
	(tokens size = 1) ifTrue: [^0].
	(tokens at: 2) isAllDigits ifFalse: [^0].
	^(tokens at: 2) asNumber
!

versions
	^versions
!

versions: aSwikiFileVersions
	versions _ aSwikiFileVersions
! !

!SwikiFile methodsFor:'initialization'!

dir: dir mimeType: mimeType
	"Sub-classes can overwrite this"
!

fromArray: arr
	super fromArray: arr.
	fileSize _ arr at: 5.
! !

!SwikiFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiFile.st,v 1.7 2003/08/29 17:04:31 cg Exp $'
! !

SwikiFile initialize!
