"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiFileServer
	instanceVariableNames:'refFileDict serveFileDict refFileBlock refDirectoryBlock
		refMissingFileBlock serveFileBlock serveDirectoryBlock
		serveMissingFileBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiFileServer comment:'SwikiFileServer

is a pluggable functionality module. It is used by the Swiki to reference or serve files and directories in different ways.

INSTANCE VARIABLES:
refFileDict = a Dictionary of classes to the block used to reference them
serveFileDict = a Dictionary of classes to the block used to serve them
refFileBlock = a block that determines how to reference a SwikiFile
refDirectoryBlock = a block that determines how to reference a SwikiDirectory
refMissingFileBlock = a block that determines how to reference a missing file
serveFileBlock = a block that determines how to serve a SwikiFile or SwikiImage
serveDirectoryBlock = a block that determines how to serve a SwikiDirectory
serveMissingFileBlock = a block that determines how to serve a missing file'
!


!SwikiFileServer class methodsFor:'instance creation'!

new
	^super new initialize
! !

!SwikiFileServer methodsFor:'accessing'!

refDirectoryBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            refDirectoryBlock := (SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        refDirectoryBlock _ aBlock
!

refFileBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            refFileBlock := SimulatedSqueakBlock wrapIfBlock:aBlock.
            ^ self.
        ].
        refFileBlock _ aBlock
!

refFileDictAt: aClass put: aBlock
	refFileDict at: aClass put: aBlock
!

refImageBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            self refFileDictAt: SwikiImage put:(SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        self refFileDictAt: SwikiImage put: aBlock
!

refMissingFileBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            refMissingFileBlock := (SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        refMissingFileBlock _ aBlock
!

serveDirectoryBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            serveDirectoryBlock := (SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        serveDirectoryBlock _ aBlock
!

serveFileBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            serveFileBlock := (SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        serveFileBlock _ aBlock
!

serveFileDictAt: aClass put: aBlock
	serveFileDict at: aClass put: aBlock
!

serveMissingFileBlock: aBlock
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            serveMissingFileBlock := (SimulatedSqueakBlock wrapIfBlock:aBlock).
            ^ self.
        ].
        serveMissingFileBlock _ aBlock
! !

!SwikiFileServer methodsFor:'initialization'!

initialize
	refFileDict _ Dictionary new.
	serveFileDict _ Dictionary new
! !

!SwikiFileServer methodsFor:'referencing'!

referenceDirectory: aSwikiDir options: dict
	^refDirectoryBlock copy fixTemps value: aSwikiDir value: dict
!

referenceFile: aSwikiFile onDir: aSwikiDir options: dict
	^(refFileDict at: (aSwikiFile last class) ifAbsent: [refFileBlock]) copy fixTemps value: aSwikiFile value: aSwikiDir value: dict
!

referenceMissingFileNamed: aString onDir: aSwikiDir options: dict
	^refMissingFileBlock copy fixTemps value: aString value: aSwikiDir value: dict
! !

!SwikiFileServer methodsFor:'serving'!

serveDirectory: aSwikiDir options: dict
	^serveDirectoryBlock copy fixTemps value: aSwikiDir value: dict
!

serveFile: aSwikiFile onDir: aSwikiDir options: dict
    ^ (serveFileDict at: aSwikiFile class ifAbsent: [serveFileBlock]) copy fixTemps value: aSwikiFile value: aSwikiDir value: dict
!

serveMissingFilePath: path onDir: aSwikiDir options: dict
	^serveMissingFileBlock copy fixTemps value: path value: aSwikiDir value: dict
! !

!SwikiFileServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiFileServer.st,v 1.30 2003/08/29 17:05:08 cg Exp $'
! !
