"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiFile subclass:#SwikiImage
	instanceVariableNames:'width height'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiImage comment:'SwikiImage

is a file entry for an image (usually, a file that ends with gif, jpeg, jpg, or png). It includes width and height as further instance variables.

INSTANCE VARIABLES:
width = the width of the image (as a number)
height = the height of the image (as a number)'
!


!SwikiImage class methodsFor:'initialize-release'!

initialize
	self handlesMime: 'image/png'.
	self handlesMime: 'image/jpeg'.
	self handlesMime: 'image/gif'
! !

!SwikiImage methodsFor:'accessing'!

height
	^height
!

width
	^width
! !

!SwikiImage methodsFor:'initialization'!

dir: dir mimeType: mimeType
        | fileStream |
        "Get Width and Height information from file stream.
        On error, width and height become nil"
        fileStream _ dir readOnlyFileNamed: name.
        (mimeType = 'image/gif') ifTrue: [
                [self getWHFromGifStream: fileStream]
                        ifError: [:a :b | width _ nil. height _ nil]].
        (mimeType = 'image/jpeg') ifTrue: [
                [self getWHFromJpegStream: fileStream]
                        ifError: [:a :b | width _ nil. height _ nil]].
        (mimeType = 'image/png') ifTrue: [
                [self getWHFromPngStream: fileStream]
                        ifError: [:a :b | width _ nil. height _ nil]].
        fileStream close
!

getWHFromGifStream: stream
        "Get Width and Height from a GIF image"
        | reader |
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            reader := GIFReader fromStream:stream.
            width := reader width.
            height := reader height.
        ] ifFalse:[
            reader _ GIFReadWriter new on: stream.
            (reader hasMagicNumber: 'GIF87a' asByteArray) ifFalse: [
                    (reader hasMagicNumber: 'GIF89a' asByteArray) ifFalse: [
                            "Not a valid gif"
                            ^self]].
            width _ reader readWord.
            height _ reader readWord
        ]
!

getWHFromJpegStream: stream
	"Get Width and Height from a JPEG image"
	stream binary.
	stream next: 4.
	stream next: ((stream next bitShift: 8) + (stream next) - 2).
	[#(16rFFC0 16rFFC1 16rFFC2 16rFFC3 16rFFC5 16rFFC6 16rFFC7 16rFFC9 16rFFCA 16rFFCB 16rFFCD 16rFFCE 16rFFCF) includes: ((stream next bitShift: 8) + (stream next))] whileFalse: [
		stream next: ((stream next bitShift: 8) + (stream next) - 2)].
	stream next: 3.
	height _ (stream next bitShift: 8) + (stream next).
	width _ (stream next bitShift: 8) + (stream next)
!

getWHFromPngStream: stream
        "Get Width and Height from a PNG image"
        stream binary.
        (stream size > 24) ifTrue: ["Check to make sure it is a PNG"
                ((stream next: 8) = (ByteArray withAll: #(137 80 78 71 13 10 26 10))) ifTrue: [
                        stream next: 8.
                        width _ (stream next bitShift: 24) + (stream next bitShift: 16) + (stream next bitShift: 8) + (stream next).
                        height _ (stream next bitShift: 24) + (stream next bitShift: 16) + (stream next bitShift: 8) + (stream next)]]
! !

!SwikiImage methodsFor:'testing'!

isAnImage
	^true
!

isValid
	(width = nil) ifTrue: [^false].
	(height = nil) ifTrue: [^false].
	^super isValid
! !

!SwikiImage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiImage.st,v 1.8 2003/08/29 17:04:35 cg Exp $'
! !

SwikiImage initialize!
