"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiModule
	instanceVariableNames:'shelf'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Comanche'
!

!SwikiModule class methodsFor:'documentation'!

examples

"
        To Start Swiki on Comanche:

        | module |
        module := SwikiModule swikiWebServer.
        module ifNil: [self error: 'Did not start!!'].
"
!

testModule

"
        To Start Swiki on Comanche:
"
        | module |
        module := ComancheSwikiAction setUp:'refs'.
        module ifNil: [self error: 'Did not start!!'].
        module inspect.
"
    self testModule
"
! !

!SwikiModule class methodsFor:'instance creation'!

loadClassesAndExtensions
    (Object respondsTo:#caseOf:) ifFalse:[
        Smalltalk fileIn:'extensions.st' inPackage:#'stx:goodies/webServer/comanche'
    ].

    "If Standalone, link to the XPSwiki dll if you want it"
    (Smalltalk isStandAloneApp not and:[Comanche::XPSwikiModule notNil]) ifTrue:[
        Comanche::XPSwikiModule autoload
    ].

    "
     self loadClassesAndExtensions
    "
!

swikiWebServerFromDir:aDirectoryName
        | instance dir|

        instance := self new.

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            self loadClassesAndExtensions.
            (aDirectoryName isNil or:[aDirectoryName asFilename exists not]) ifTrue:[
                self error:'swiki directory:', aDirectoryName , ' does not exist'.
            ].
            dir := SimulatedSqueakFileDirectory new fileName:aDirectoryName asFilename.
        ] ifFalse:[
            dir := (FileDirectory default directoryNamed: 'swiki')
        ].

        instance shelf: (SwikiShelf new
                name: 'swiki';
                storage: (XmlSwikiStorage fromDir: dir);
                load;
                yourself).
        ^instance
! !

!SwikiModule methodsFor:'accessing'!

shelf
        ^shelf
!

shelf: aSwikiShelf
        shelf _ aSwikiShelf
! !

!SwikiModule methodsFor:'processing'!

filter: rawRequest
    ^ HttpSwikiRequest fromHttpRequest: rawRequest
!

process: rawRequest
        | request response |

        request _ self filter: rawRequest.
        response _ Dictionary new.
        shelf process: request response: response.
        ^ self responseFrom: response
!

responseFrom: dict
        | response content |
        response _ HttpResponse new.
        "header status"
        response status: (dict at: 'headerStatus' ifAbsent: [#ok]).
        "cacheing"
        (dict at: 'cacheing' ifAbsent: [false]) ifFalse: [
                response fieldAt: 'Pragma' put: 'no-cache';
"/                        fieldAt: 'Expires' put: '0';
                        fieldAt: 'Cache-Control' put: 'no-cache'].
        "location (in case of redirect)"
        (dict includesKey: 'location') ifTrue: [
                response fieldAt: 'Location' put: (dict at: 'location')].
        "authenticate (in case of unauthorized)"
        (dict includesKey: 'authenticate') ifTrue: [
                response fieldAt: 'WWW-Authenticate' put: (dict at: 'authenticate')].
        "content type"
        response contentType: (dict at: 'contentType' ifAbsent: ['text/html; charset=iso-8859-1']).
        "contents"
        content _ dict at: 'content'.
        (content isKindOf:String)
                ifTrue: [response contents: (ReadStream on: content)]
                ifFalse: [(content isKindOf: ((Smalltalk dialectName = 'SmalltalkX') ifTrue:[SimulatedSqueakBlock] ifFalse:[BlockContext]))
                        ifTrue: [response _ response asHttpPartialResponseBlock: content]
                        ifFalse: [response contents: content]].
        "return the response to be served"
        ^response
!

stop
        ^shelf stopProcessing
! !

!SwikiModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiModule.st,v 1.23 2010/04/23 12:57:51 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiModule.st,v 1.23 2010/04/23 12:57:51 stefan Exp $'
! !
