"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiSecurityMember
	instanceVariableNames:'settings privileges'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Modules'
!

SwikiSecurityMember comment:'SwikiSecurityMember

This class provides the members that will belong to the SwikiSecurityModule. These are passed along with the request for identification and security reasons.

INSTANCE VARIABLES:
settings = a Dictionary (user, password, and ip are the settings that are used by the security module)
privileges = a SwikiSecurityPrivileges (this determines what capabilities this user has)
'
!


!SwikiSecurityMember class methodsFor:'instance creation'!

fromXml: xml
	| instance pos1 pos2 type |

	instance _ self new.
	instance initialize.
	"Get Member Settings"
	pos1 _ (xml findString: '<member>') + 8.
	pos1 _ xml findString: '<m' startingAt: pos1.
	[pos1 > 0] whileTrue: [
		pos1 _ (xml findString: 'type="' startingAt: pos1) + 6.
		pos2 _ (xml findString: '"' startingAt: pos1) - 1.
		type _ TextFormatter decodeFromXmlCrlf: ((xml copyFrom: pos1 to: pos2) asLowercase).
		pos1 _ (xml findString: '>' startingAt: pos2) + 1.
		pos2 _ (xml findString: '<' startingAt: pos1) - 1.
		instance settingsAt: type put: (TextFormatter decodeFromXmlCrlf: (xml copyFrom: pos1 to: pos2)).
		pos1 _ xml findString: '<m' startingAt: pos2].
	^instance
!

new
	^super new initialize
! !

!SwikiSecurityMember methodsFor:'accessing'!

ip
	^(self settingsAt: 'ip' ifAbsent: [^nil]) asNumber
!

mask
	^(self settingsAt: 'mask' ifAbsent: [^16rFFFFFFFF]) asNumber
!

password
	^self settingsAt: 'password' ifAbsent: [nil]
!

privileges
	^privileges
!

privileges: privs
	privileges _ privs
!

settingsAt: key ifAbsent: block
	^settings at: key ifAbsent: block
!

settingsAt: key put: setting
	settings at: key put: setting
!

user
	^self settingsAt: 'user' ifAbsent: [nil]
! !

!SwikiSecurityMember methodsFor:'initializing'!

initialize
	settings _ Dictionary new.
! !

!SwikiSecurityMember methodsFor:'storage'!

asXml
	| return |
	return _ '<member>', String cr.
	settings keysAndValuesDo: [:key :value | return _ return, '<m type="', (TextFormatter encodeToStrictXmlCr: key), '">', (TextFormatter encodeToXmlCr: value), '</m>', String cr].
	^return, '</member>'
! !

!SwikiSecurityMember methodsFor:'testing'!

handlesBookAddress: test
	^privileges handlesBookAddress: test
!

handlesOther: test
	^privileges handlesOther: test
!

handlesPageAddress: test
	^privileges handlesPageAddress: test
!

handlesShelfAddress: test
	^privileges handlesShelfAddress: test
! !

!SwikiSecurityMember class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiSecurityMember.st,v 1.6 2003/08/29 17:04:06 cg Exp $'
! !
