"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiSecurityModule
	instanceVariableNames:'aclDict ipColl default'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Modules'
!

SwikiSecurityModule comment:'SwikiSecurityModule

This is what a ''security.xml'' file is supposed to look like:

**********************************************************
<?xml version="1.0"?>
<security>
<default priv="defaultPrivilege" />
<group priv="groupPrivilege">
<member>
<m type="user">userName</m>
<m type="password">passWord</m>
<m type="ip">IPNumber</m>
...
</member>
...
</group>
...
</security>
**********************************************************

''...''						means that there can be several of the type proceeding it.
defaultPrivilege			the privilege of the default group (for example: ''Read Only'')
groupPrivilege			the privilege of the specific group (for example: ''Write'')
userName				the user name that the user must enter
passWord				the password the user must enter
IPNumber				the IPNumber of the user
'
!


!SwikiSecurityModule class methodsFor:'instance creation'!

fromXml: xml withDict: dict
	| instance |
	instance _ self new.
	instance fromXml: xml withDict: dict.
	^instance
!

new
	^super new initialize
! !

!SwikiSecurityModule class methodsFor:'utility'!

encode: nameString password: pwdString
	"Encode per RFC1421 of the username:password combination."

	| clear code clearSize idx map |
	clear := (nameString, ':', pwdString) asByteArray.
	clearSize := clear size.
	[ clear size \\ 3 ~= 0 ] whileTrue: [ clear := clear, #(0) ].
	idx := 1.
	map := 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.
	code := WriteStream on: ''.
	[ idx < clear size ] whileTrue: [ code 
		nextPut: (map at: (clear at: idx) // 4 + 1);
		nextPut: (map at: (clear at: idx) \\ 4 * 16 + ((clear at: idx + 1) // 16) + 1);
   		nextPut: (map at: (clear at: idx + 1) \\ 16 * 4 + ((clear at: idx + 2) // 64) + 1);
   		nextPut: (map at: (clear at: idx + 2) \\ 64 + 1).
		idx := idx + 3 ].
	code := code contents.
	idx := code size.
	clear size - clearSize timesRepeat: [ code at: idx put: $=. idx := idx - 1].
	^code
!

isIpString: aString
	| tokens asNumber |
	tokens _ aString findTokens: '.'.
	(tokens size = 4) ifFalse: [^false].
	tokens do: [:token | "Check for 0..255"
		token isAllDigits ifFalse: [^false].
		asNumber _ token asNumber.
		(asNumber < 256) ifFalse: [^false].
		(asNumber >= 0) ifFalse: [^false]].
	^true
!

toIpNumber: string
	| return |
	return _ 0.
	(string findTokens: '.') do: [:i | (i isAllDigits)
		ifTrue: [return _ (256 * return) + (i asNumber)]].
	^return
!

toIpString: int
	| one two three four |
	two _ int // 256.
	one _ int - (256 * two).
	three _ two // 256.
	two _ two - (256 * three).
	four _ three // 256.
	three _ three - (256 * four).
	^four asString, '.', three asString, '.', two asString, '.', one asString
! !

!SwikiSecurityModule methodsFor:'accessing'!

aclDict
	^aclDict
!

addACLMember: member
	aclDict at: (self class encode: (member user) password: (member password)) put: member
!

addIPMember: member
	ipColl add: (Array with: (member mask) with: (member ip) with: member)
!

addMember: member
	(member user) ifNotNil: [self addACLMember: member].
	(member ip) ifNotNil: [self addIPMember: member]
!

default
	^default
!

default: defaultMember
	default _ defaultMember
!

ipColl
	^ipColl
!

members
	| members |

	members _ OrderedCollection new.
	aclDict valuesDo: [:member | members add: member].
	ipColl do: [:element | (members includes: (element at: 3)) ifFalse: [
		members add: (element at: 3)]].
	^members
! !

!SwikiSecurityModule methodsFor:'initializing'!

initialize
	aclDict _ Dictionary new.
	ipColl _ OrderedCollection new
! !

!SwikiSecurityModule methodsFor:'processing'!

filterIP: request
	| ipInt |

	(ipColl size = 0) ifTrue: [^request security: default].
	ipInt _ 0.
	request user do: [:i | ipInt _ (ipInt*256) + i].
	ipColl do: [:ipArr | ((ipInt bitAnd: (ipArr at: 1)) = (ipArr at: 2))
		ifTrue: [^request security: (ipArr at: 3)]].
	^request security: default
!

process: request response: response shelf: shelf
	| address userID |	

	(address _ request address) ifNil: [address _ 'default'].
	(userID _ request userID) ifNotNil: ["Process by UserID"
		request security: (aclDict at: userID ifAbsent: [default]).
		((request fieldsHasKey: 'book')
			ifTrue: [request security handlesBookAddress: address]
			ifFalse: [request security handlesShelfAddress: address])
			ifTrue: [^true]].
	"Process by IP"
	self filterIP: request.
	^(request fieldsHasKey: 'book')
		ifTrue: [request security handlesBookAddress: address]
		ifFalse: [request security handlesShelfAddress: address]
!

process: request response: response shelf: shelf book: book
	| address userID |

	"Get Address"
	address _ request address.
	address _ (request page)
		ifNil: [(book hasBookAddress: address)
			ifTrue: [address]
			ifFalse: ['default']]
		ifNotNil: [(book hasPageAddress: address)
			ifTrue: [address]
			ifFalse: ['default']].
	(userID _ request userID) ifNotNil: ["Process by UserID"
		request security: (aclDict at: userID ifAbsent: [default]).
		((request page)
			ifNil: [request security handlesBookAddress: address]
			ifNotNil: [request security handlesPageAddress: address])
			ifTrue: [^true]].
	"Process by IP"
	self filterIP: request.
	^(request page)
		ifNil: [request security handlesBookAddress: address]
		ifNotNil: [request security handlesPageAddress: address]
! !

!SwikiSecurityModule methodsFor:'storage'!

asXml
	| return privDict |
	return _ '<?xml version="1.0"?>
<security>
<default priv="', (default privileges name), '" />
'.
	privDict _ Dictionary new.
	self members do: [:member | (privDict includesKey: (member privileges))
		ifTrue: [(privDict at: (member privileges)) add: member]
		ifFalse: [privDict at: (member privileges) put: (OrderedCollection with: member)]].
	privDict keysAndValuesDo: [:priv :members |
		return _ return, '<group priv="', (priv name), '">', String cr.
		members do: [:member | return _ return, member asXml, String cr].
		return _ return, '</group>', String cr].
	^return, '</security>'
!

fromXml: xml withDict: dict
	| pos1 pos2 pos3 priv members member |
	"Get Default Swiki Security Member"
	pos1 _ xml findString: '<default priv="'.
	pos2 _ xml findString: '" />' startingAt: pos1.
	default _ SwikiSecurityMember new.
	priv _ dict at: (xml copyFrom: (pos1 + 15) to: (pos2 - 1)).
	default privileges: priv.
	[(pos1 _ xml findString: '<group priv="' startingAt: pos1) = 0] whileFalse: [
		pos2 _ xml findString: '">' startingAt: pos1.
		priv _ dict at: (xml copyFrom: (pos1 + 13) to: (pos2 - 1)).
		pos1 _ xml findString: '</group>' startingAt: pos2.
		members _ xml copyFrom: (pos2 + 2) to: (pos1 - 1).
		pos3 _ 1.
		[(pos2 _ members findString: '<member>' startingAt: pos3) = 0] whileFalse: [
			pos3 _ members findString: '</member>' startingAt: pos2.
			member _ SwikiSecurityMember fromXml: (members copyFrom: pos2 to: (pos3 + 8)).
			member privileges: priv.
			self addMember: member]].
! !

!SwikiSecurityModule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiSecurityModule.st,v 1.6 2003/08/29 17:04:49 cg Exp $'
! !
