"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiSecurityPrivileges
	instanceVariableNames:'name default pageAddresses bookAddresses shelfAddresses other'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Modules'
!

SwikiSecurityPrivileges comment:'SwikiSecurityPrivileges

This class provides privileges for the various servers and modules to inspect. Try the category testing to figure out which one is right for the purpose.

If a privilege of the type does exist, the instance returns (default not). If the privilege of the type does not exist, the instance returns (default).

INSTANCE VARIABLES:
name = the name of the privilege. This identifies the privilege.
default = the default value as to whether something is allowed
pageAddresses = the list of exceptions to the default value having to do with page addresses
bookAddresses = the list of exceptions to the default value having to do with book addresses
other = the list of exceptions to the default value not having to do with addresses '
!


!SwikiSecurityPrivileges class methodsFor:'instance creation'!

named: aName default: aBoolean
	^(self new)
		initialize;
		name: aName;
		default: aBoolean;
		yourself
! !

!SwikiSecurityPrivileges methodsFor:'accessing'!

addAddress: priv
	| tokens |

	tokens _ priv findTokens: '.'.
	(tokens size = 2) ifFalse: [^false].
	((tokens at: 2) = 'page') ifTrue: [
		pageAddresses add: (tokens at: 1).
		^true].
	((tokens at: 2) = 'book') ifTrue: [
		bookAddresses add: (tokens at: 1).
		^true].
	((tokens at: 2) = 'shelf') ifTrue: [
		shelfAddresses add: (tokens at: 1).
		^true].
	^false
!

addOther: priv
	other add: priv.
	^true
!

default
	^default
!

default: aBoolean
	default _ aBoolean
!

name
	^name
!

name: aString
	name _ aString
! !

!SwikiSecurityPrivileges methodsFor:'initializing'!

initialize
	default _ true.
	other _ OrderedCollection new.
	pageAddresses _ OrderedCollection new.
	bookAddresses _ OrderedCollection new.
	shelfAddresses _ OrderedCollection new
! !

!SwikiSecurityPrivileges methodsFor:'testing'!

handlesBookAddress: test
	(bookAddresses includes: test)
		ifTrue: [^default not]
		ifFalse: [^default]
!

handlesOther: test
	(other includes: test)
		ifTrue: [^default not]
		ifFalse: [^default]
!

handlesPageAddress: test
	(pageAddresses includes: test)
		ifTrue: [^default not]
		ifFalse: [^default]
!

handlesShelfAddress: test
	(shelfAddresses includes: test)
		ifTrue: [^default not]
		ifFalse: [^default]
! !

!SwikiSecurityPrivileges class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiSecurityPrivileges.st,v 1.6 2003/08/29 17:05:16 cg Exp $'
! !
