"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiStructure subclass:#SwikiShelf
	instanceVariableNames:'books privAddresses shelfAddresses bookTemplates shelfTemplates
		bookActions shelfActions'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Structure'
!


!SwikiShelf methodsFor:'accessing'!

bookActions
	^bookActions
!

bookTemplates
	^bookTemplates
!

books
	^books
!

books: anObject
	books _ anObject
!

privAddresses
	^privAddresses
!

shelfActions
	^shelfActions
!

shelfAddresses
	^shelfAddresses
!

shelfTemplates
	^shelfTemplates
! !

!SwikiShelf methodsFor:'initialization'!

initialize
	super initialize.
	privAddresses _ Dictionary new.
	shelfAddresses _ Dictionary new.
	bookTemplates _ Dictionary new.
	shelfTemplates _ Dictionary new.
	bookActions _ Dictionary new.
	shelfActions _ Dictionary new
! !

!SwikiShelf methodsFor:'processing'!

formatBookAction: actionName request: request response: response book: book
	^self formatBookAction: actionName with: (Array with: request with: response with: self with: book)
!

formatBookAction: actionName with: args
	^(bookActions at: actionName ifAbsent: [^self formatShelfAction: actionName with: (args copyFrom: 1 to: 3)]) copy fixTemps valueWithArguments: args
!

formatBookTemplate: templateName request: request response: response book: book
	^(ShelfTemplateFormatter swikiFormatter) format: (bookTemplates at: templateName) request: request response: response shelf: self book: book
!

formatPrivAddress: addressName request: request response: response

        ^(privAddresses at: addressName ifAbsent: [^nil]) copy fixTemps value: request value: response value: self
!

formatShelfAction: actionName request: request response: response
	^self formatShelfAction: actionName with: (Array with: request with: response with: self)
!

formatShelfAction: actionName with: args
	^(shelfActions at: actionName ifAbsent: [^'<?', actionName, '?>']) copy fixTemps valueWithArguments: args
!

formatShelfAddress: addressName request: request response: response
	^(shelfAddresses at: addressName ifAbsent: [^nil]) copy fixTemps value: request value: response value: self
!

formatShelfTemplate: templateName request: request response: response
	^(ShelfTemplateFormatter swikiFormatter) format: (shelfTemplates at: templateName) request: request response: response shelf: self
!

process: request response: response
        | book filter |
        "Filter Before Processing"
        filter _ self formatPrivAddress: 'filter' request: request response: response.
        filter
                ifNil: [(book _ self bookNamed: request book)
                        ifNil: [response at: 'content' put: ((shelfAddresses at: (request book) ifAbsent: [shelfAddresses at: 'default']) copy fixTemps value: request value: response value: self)]
                        ifNotNil: [(book setup at: 'accessible' ifAbsent: [true])
                                ifTrue: [book process: request response: response shelf: self]
                                ifFalse: [response at: 'content' put: (self formatShelfAddress: 'default' request: request response: response)]]]
                ifNotNil: [response at: 'content' put: filter].
        "Log After Processing"
        self formatPrivAddress: 'log' request: request response: response
!

stopProcessing
	"Stop Books"
	books do: [:book | book formatPrivAddress: 'stop' request: nil response: nil shelf: self].
	"Stop Self"
	self formatPrivAddress: 'stop' request: nil response: nil
! !

!SwikiShelf methodsFor:'storage'!

load
	storage loadShelf: self.
	books _ storage setupBooks.
	self loadInheritance.
	self loadBooks.
	self loadModules
!

loadBooks
	books do: [:book | (book hasParent) ifFalse: [book loadSelfAndChildrenFrom: self]]
!

loadInheritance
	| parent |
	books do: [:book | (book hasParent) ifTrue: [
		parent _ self bookNamed: (book setup at: 'parent').
		parent ifNil: [self error: ('parent book (', (book setup at: 'parent'), ') does not exits')].
		(parent setup at: 'inheritable' ifAbsent: [true])
			ifTrue: [parent addChild: book.
				book parent: parent]
			ifFalse: [self error: ('parent book (', (parent name), ') is not inheritable')]]]
!

loadModules
	self formatPrivAddress: 'initialize' request: nil response: nil.
	books do: [:book |
		book formatPrivAddress: 'initialize' request: nil response: nil shelf: self]
!

reload
	"Stop First"
	self formatPrivAddress: 'stop' request: nil response: nil.
	"Reinitialize ATA"
	self initialize.
	storage loadShelf: self.
	"Reinitailize Modules"
	self loadModules
!

writeSettings
	storage writeSettingsForShelf: self
!

writeSetupForBook: book
	storage writeSetupForBook: book
! !

!SwikiShelf methodsFor:'testing'!

hasPrivAddress: addressName
	^privAddresses includesKey: addressName
! !

!SwikiShelf methodsFor:'utility'!

addNewBookNamed: aString
	| test book |
	test _ books at: 1.
	storage dir createDirectory: aString.
	book _ test class new.
	book name: aString.
	^book
!

bookNamed: aString
	| lower |
	aString ifNil: [^nil].
	lower _ aString asLowercase.
	books do: [:book | (book name asLowercase = lower) ifTrue: [^book]].
	^nil
! !

!SwikiShelf class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiShelf.st,v 1.8 2003/08/29 17:05:37 cg Exp $'
! !
