"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#SwikiStorage
	instanceVariableNames:'dir dict'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-Storage'
!


!SwikiStorage class methodsFor:'instance creation'!

fromDir: aFileDirectory
	^self new dir: aFileDirectory
! !

!SwikiStorage class methodsFor:'testing class hierarchy'!

handlesBookStorage
	^false
!

handlesPageStorage
	^false
! !

!SwikiStorage methodsFor:'accessing'!

dir
	^dir
!

dir: aFileDirectory
	dir _ aFileDirectory.
	dict _ Dictionary new
! !

!SwikiStorage methodsFor:'book (browser)'!

addBookFunction: function type: type category: category
	(dict includesKey: (function, type))
		ifTrue: [(dict at: (function, type)) add: category]
		ifFalse: [dict at: (function, type) put: (OrderedCollection with: category)]
!

addBookFunction: function type: type name: name
	^self addBookFunction: function type: type name: name category: nil
!

addBookFunction: function type: type name: name category: category
	| fDir fileNames id coll |
	(dir directoryExists: function) ifFalse: [dir createDirectory: function].	
	fDir _ dir directoryNamed: function.
	fileNames _ fDir fileNames.
	id _ 1.
	[fileNames includes: id asString] whileTrue: [id _ id + 1].
	coll _ dict at: function ifAbsent: [dict at: function put: OrderedCollection new].
	coll add: (Array with: id asString with: type with: name with: category).
	(fDir newFileNamed: id asString) close.
	"Remove category from empty categories"
	(category notNil and: [dict includesKey: (function, type)]) ifTrue: [
		(dict at: (function, type)) remove: category ifAbsent: []].
	"Update meta and browser log files"
	self putMetaColl: coll forDir: fDir.
	self logChangeBookFunction: function type: type name: name
!

allBookFunction: function type: type
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return select: [:arr | (arr at: 2) = type] thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

ayuBookFunction: function type: type
	"As Yet Unclassified"
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return
		select: [:arr | ((arr at: 2) = type) and: [(arr at: 4) isNil]]
		thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

bookFunction: function type: type category: category
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return
		select: [:arr | ((arr at: 2) = type) and: [(arr at: 4) = category]]
		thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

bookFunction: function type: type name: name
	| coll |
	coll _ dict at: function ifAbsent: [
		self error: (function, ' does not contain ', name, ' of type ', type)].
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name]) ifTrue: [
		^((dir directoryNamed: function) readOnlyFileNamed: (arr at: 1))
			contentsOfEntireFile]]
!

bookFunction: function type: type name: name content: content
	| coll fDir file |
	coll _ dict at: function ifAbsent: [
		self error: (function, ' does not contain ', name, ' of type ', type)].
	fDir _ dir directoryNamed: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name]) ifTrue: [
		fDir deleteFileNamed: (arr at: 1).
		file _ fDir newFileNamed: (arr at: 1).
		file nextPutAll: content.
		file close.
		"Update browser log"
		self logChangeBookFunction: function type: type name: name]]
!

categoriesBookFunction: function type: type
        | metaColl return ayu |
        metaColl _ dict at: function ifAbsent: [OrderedCollection new].
        ayu _ false.
        return _ SortedCollection sortBlock: [:a :b | a asLowercase < b asLowercase].
        metaColl do: [:arr | ((arr at: 2) = type) ifTrue: [(arr at: 4)
                ifNil: [ayu _ true]
                ifNotNil: [(return includes: (arr at: 4)) ifFalse: [
                        return add: (arr at: 4)]]]].
        "Add Empty Categories"
        (dict includesKey: (function, type)) ifTrue: [
                (dict at: (function, type)) do: [:cat |
                        (return includes: cat) ifFalse: [return add: cat]]].
        return _ return asOrderedCollection.
        return addFirst: '-- all --'.
        ayu ifTrue: [return add: 'as yet unclassified'].
        ^return
!

deleteBookFunction: function type: type category: category
	| coll |
	coll _ dict at: function.
	coll do: [:arr | ((arr at: 4) = category) ifTrue: [
		arr at: 4 put: nil]].
	self putMetaColl: coll forDir: (dir directoryNamed: function).
!

deleteBookFunction: function type: type name: name
	| coll fDir |
	coll _ OrderedCollection new.
	fDir _ dir directoryNamed: function.
	(dict at: function) do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [fDir deleteFileNamed: (arr at: 1)]
		ifFalse: [coll add: arr]].
	dict at: function put: coll.
	"Update meta and browser log files"
	self putMetaColl: coll forDir: fDir.
	self logDeleteBookFunction: function type: type name: name
!

moveBookFunction: function type: type name: name
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [arr at: 4 put: nil]].
	self putMetaColl: coll forDir: (dir directoryNamed: function)
!

moveBookFunction: function type: type name: name toCategory: category
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [arr at: 4 put: category]].
	self putMetaColl: coll forDir: (dir directoryNamed: function)
!

renameBookFunction: function type: type category: fromName to: toName
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 4) = fromName])
		ifTrue: [arr at: 4 put: toName]].
	self putMetaColl: coll forDir: (dir directoryNamed: function)
!

renameBookFunction: function type: type name: fromName to: toName
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = fromName])
		ifTrue: [arr at: 3 put: toName]].
	"Update meta and browser log files"
	self putMetaColl: coll forDir: (dir directoryNamed: function).
	self logDeleteBookFunction: function type: type name: fromName.
	self logChangeBookFunction: function type: type name: toName
! !

!SwikiStorage methodsFor:'book (load)'!

loadBook: book
	self
		loadSettingsForBook: book;
		loadAddressesForBook: book;
		loadTemplatesForBook: book;
		loadActionsForBook: book
!

loadSettingsForBook: book
	book rawSettings: Dictionary new
!

writeSettingsForBook: book
! !

!SwikiStorage methodsFor:'book (logging)'!

logChangeBookFunction: function type: type name: name
	(dir fileNamed: 'browser.log')
		setToEnd;
		nextPutAll: (Date today asString);
		nextPut: Character tab;
		nextPutAll: (Time now asString);
		nextPut: Character tab;
		nextPutAll: 'changeBookFunction';
		nextPut: Character tab;
		nextPutAll: function;
		nextPut: Character tab;
		nextPutAll: type;
		nextPut: Character tab;
		nextPutAll: name;
		nextPut: Character cr;
		close
!

logDeleteBookFunction: function type: type name: name
	(dir fileNamed: 'browser.log')
		setToEnd;
		nextPutAll: (Date today asString);
		nextPut: Character tab;
		nextPutAll: (Time now asString);
		nextPut: Character tab;
		nextPutAll: 'deleteBookFunction';
		nextPut: Character tab;
		nextPutAll: function;
		nextPut: Character tab;
		nextPutAll: type;
		nextPut: Character tab;
		nextPutAll: name;
		nextPut: Character cr;
		close
! !

!SwikiStorage methodsFor:'pages'!

multipleTextFrom: aPage
	^self subclassResponsibility
!

simpleTextFrom: aPage
	^self subclassResponsibility
!

textFrom: aPage
        | text |
        text _ aPage rawText.
        (text isKindOf: String) ifTrue: [^text].
        (text isKindOf: Array) ifTrue: [^self simpleTextFrom: aPage].
        (text isKindOf: Dictionary) ifTrue: [((text values at: 1) isKindOf: String)
                ifTrue: [^text]
                ifFalse: [^self multipleTextFrom: aPage]].
        ^''
! !

!SwikiStorage methodsFor:'private'!

getMetaCollForDir: aDir
	self subclassResponsibility
!

putMetaColl: coll forDir: aDir
	self subclassResponsibility
! !

!SwikiStorage methodsFor:'shelf (browser)'!

addShelfFunction: function type: type category: category
	(dict includesKey: (function, type))
		ifTrue: [(dict at: (function, type)) add: category]
		ifFalse: [dict at: (function, type) put: (OrderedCollection with: category)]
!

addShelfFunction: function type: type name: name
	^self addShelfFunction: function type: type name: name category: nil
!

addShelfFunction: function type: type name: name category: category
	| dDir fDir fileNames id coll |
	dDir _ dir directoryNamed: 'default'.
	(dDir directoryExists: function) ifFalse: [dDir createDirectory: function].	
	fDir _ dDir directoryNamed: function.
	fileNames _ fDir fileNames.
	id _ 1.
	[fileNames includes: id asString] whileTrue: [id _ id + 1].
	coll _ dict at: function ifAbsent: [dict at: function put: OrderedCollection new].
	coll add: (Array with: id asString with: type with: name with: category).
	(fDir newFileNamed: id asString) close.
	"Remove category from empty categories"
	(category notNil and: [dict includesKey: (function, type)]) ifTrue: [
		(dict at: (function, type)) remove: category ifAbsent: []].
	"Update meta and browser log files"
	self putMetaColl: coll forDir: fDir.
	self logChangeShelfFunction: function type: type name: name
!

allShelfFunction: function type: type
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return select: [:arr | (arr at: 2) = type] thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

ayuShelfFunction: function type: type
	"As Yet Unclassified"
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return
		select: [:arr | ((arr at: 2) = type) and: [(arr at: 4) isNil]]
		thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

categoriesShelfFunction: function type: type
	| metaColl return ayu |
	metaColl _ dict at: function ifAbsent: [^(OrderedCollection with: '-- all --')].
	ayu _ false.
	return _ SortedCollection sortBlock: [:a :b | a asLowercase < b asLowercase].
	metaColl do: [:arr | ((arr at: 2) = type) ifTrue: [(arr at: 4)
		ifNil: [ayu _ true]
		ifNotNil: [(return includes: (arr at: 4)) ifFalse: [
			return add: (arr at: 4)]]]].
	"Add Empty Categories"
	(dict includesKey: (function, type)) ifTrue: [
		(dict at: (function, type)) do: [:cat |
			(return includes: cat) ifFalse: [return add: cat]]].
	return _ return asOrderedCollection.
	return addFirst: '-- all --'.
	ayu ifTrue: [return add: 'as yet unclassified'].
	^return
!

deleteShelfFunction: function type: type category: category
	| coll |
	coll _ dict at: function.
	coll do: [:arr | ((arr at: 4) = category) ifTrue: [
		arr at: 4 put: nil]].
	self putMetaColl: coll forDir: ((dir directoryNamed: 'default') directoryNamed: function).
!

deleteShelfFunction: function type: type name: name
	| coll fDir |
	coll _ OrderedCollection new.
	fDir _ (dir directoryNamed: 'default') directoryNamed: function.
	(dict at: function) do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [fDir deleteFileNamed: (arr at: 1)]
		ifFalse: [coll add: arr]].
	dict at: function put: coll.
	"Update meta and browser log files"
	self putMetaColl: coll forDir: fDir.
	self logDeleteShelfFunction: function type: type name: name
!

moveShelfFunction: function type: type name: name
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [arr at: 4 put: nil]].
	self putMetaColl: coll forDir: ((dir directoryNamed: 'default') directoryNamed: function)
!

moveShelfFunction: function type: type name: name toCategory: category
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name])
		ifTrue: [arr at: 4 put: category]].
	self putMetaColl: coll forDir: ((dir directoryNamed: 'default') directoryNamed: function)
!

renameShelfFunction: function type: type category: fromName to: toName
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 4) = fromName])
		ifTrue: [arr at: 4 put: toName]].
	self putMetaColl: coll forDir: ((dir directoryNamed: 'default') directoryNamed: function)
!

renameShelfFunction: function type: type name: fromName to: toName
	| coll |
	coll _ dict at: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = fromName])
		ifTrue: [arr at: 3 put: toName]].
	"Update meta and browser log files"
	self putMetaColl: coll forDir: ((dir directoryNamed: 'default') directoryNamed: function).
	self logDeleteShelfFunction: function type: type name: fromName.
	self logChangeShelfFunction: function type: type name: toName
!

shelfFunction: function type: type category: category
	| return |
	return _ dict at: function ifAbsent: [^OrderedCollection new].
	return _ return
		select: [:arr | ((arr at: 2) = type) and: [(arr at: 4) = category]]
		thenCollect: [:arr | arr at: 3].
	^return asSortedCollection: [:a :b | a asLowercase < b asLowercase]
!

shelfFunction: function type: type name: name
	| coll |
	coll _ dict at: function ifAbsent: [
		self error: (function, ' does not contain ', name, ' of type ', type)].
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name]) ifTrue: [
		^(((dir directoryNamed: 'default') directoryNamed: function) readOnlyFileNamed: (arr at: 1))
			contentsOfEntireFile]]
!

shelfFunction: function type: type name: name content: content
	| coll fDir file |
	coll _ dict at: function ifAbsent: [
		self error: (function, ' does not contain ', name, ' of type ', type)].
	fDir _ (dir directoryNamed: 'default') directoryNamed: function.
	coll do: [:arr | (((arr at: 2) = type) and: [(arr at: 3) = name]) ifTrue: [
		fDir deleteFileNamed: (arr at: 1).
		file _ fDir newFileNamed: (arr at: 1).
		file nextPutAll: content.
		file close.
		"Update browser log"
		self logChangeShelfFunction: function type: type name: name]]
! !

!SwikiStorage methodsFor:'shelf (load)'!

createBook: book
	dir createDirectory: book name.
	self writeSetupForBook: book
!

loadActionsForBook: book 
        | actionsDir metaColl content bookOrPage|
        (dir directoryExists: 'actions') ifFalse: [^self].
        actionsDir := dir directoryNamed: 'actions'.
        metaColl := self getMetaCollForDir: actionsDir.
        dict at: 'actions' put: metaColl.
        metaColl do: [:arr |
                content := (actionsDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
                bookOrPage := arr at:2.
                bookOrPage = 'book' ifTrue:[
                    book bookActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']').
                ] ifFalse:[
                    bookOrPage = 'page' ifTrue:[
                        book pageActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book :page | ', content , ']').
                    ] ifFalse:[
                        "ignore"
                    ].
                ].
        ].

"
                (arr at: 2) caseOf: {
                        ['book']->[book bookActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']')].
                        ['page']->[book pageActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book :page | ', content , ']')]}
                        otherwise: [ignore]].
"
!

loadActionsForShelf: shelf
        | actionsDir metaColl content  bookOrShelf|
        actionsDir := dir directoryNamed: 'default'.
        (actionsDir directoryExists: 'actions') ifFalse: [^self].
        actionsDir := actionsDir directoryNamed: 'actions'.
        metaColl := self getMetaCollForDir: actionsDir.
        dict at: 'actions' put: metaColl.
        metaColl do: [:arr |
            content := (actionsDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
            bookOrShelf := arr at:2.
            bookOrShelf = 'book' ifTrue:[
                shelf bookActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']').
            ] ifFalse:[
                bookOrShelf = 'shelf' ifTrue:[
                    shelf shelfActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']').
                ] ifFalse:[
                    "ignore"
                ].
            ].
        ]

"
                (arr at: 2) caseOf: {
                        ['book']->[shelf bookActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']')].
                        ['shelf']->[shelf shelfActions at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']')]}
                        otherwise: []].
"
!

loadAddressesForBook: book 
        | addressesDir metaColl content privOrBookOrPage|

        (dir directoryExists: 'addresses') ifFalse: [^self].
        addressesDir := dir directoryNamed: 'addresses'.
        metaColl := self getMetaCollForDir: addressesDir.
        dict at: 'addresses' put: metaColl.
        metaColl do: [:arr |
                content := (addressesDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
                privOrBookOrPage := arr at:2.
                privOrBookOrPage = 'priv' ifTrue:[
                    book privAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']').
                ] ifFalse:[
                    privOrBookOrPage = 'book' ifTrue:[
                        book bookAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']').
                    ] ifFalse:[
                        privOrBookOrPage = 'page' ifTrue:[
                            book pageAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book :page | ', content , ']')
                        ] ifFalse:[
                            "ignore"
                        ]
                    ].
                ].
        ].

"
                (arr at: 2) caseOf: {
                        ['priv']->[book privAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']')].
                        ['book']->[book bookAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book | ', content , ']')].
                        ['page']->[book pageAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf :book :page | ', content , ']')]}
                        otherwise: []].
"
!

loadAddressesForShelf: shelf
        | addressesDir metaColl content  privOrShelf|
        addressesDir := dir directoryNamed: 'default'.
        (addressesDir directoryExists: 'addresses') ifFalse: [^self].
        addressesDir := addressesDir directoryNamed: 'addresses'.
        metaColl := self getMetaCollForDir: addressesDir.
        dict at: 'addresses' put: metaColl.
        metaColl do: [:arr |
                content := (addressesDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
                privOrShelf := arr at:2.
                privOrShelf = 'priv' ifTrue:[
                    shelf privAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']').
                ] ifFalse:[
                    privOrShelf = 'shelf' ifTrue:[
                        shelf shelfAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']').
                    ] ifFalse:[
                        "ignore"
                    ].
                ].
        ].


"
                (arr at: 2) caseOf: {
                        ['priv']->[shelf privAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']')].
                        ['shelf']->[shelf shelfAddresses at: (arr at: 3) put: (Compiler evaluate: '[:request :response :shelf | ', content , ']')]}
                        otherwise: []].
"
!

loadSettingsForShelf: shelf
	shelf settings: Dictionary new
!

loadShelf: shelf
	self
		loadSettingsForShelf: shelf;
		loadAddressesForShelf: shelf;
		loadTemplatesForShelf: shelf;
		loadActionsForShelf: shelf
!

loadTemplatesForBook: book 
        | templatesDir metaColl content bookOrPage|
        (dir directoryExists: 'templates') ifFalse: [^self].
        templatesDir := dir directoryNamed: 'templates'.
        metaColl := self getMetaCollForDir: templatesDir.
        dict at: 'templates' put: metaColl.
        metaColl do: [:arr |
                content := (templatesDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
                "Use Internet Line Endings"
                content := TextFormatter crToCrlf: content.
                bookOrPage := arr at:2.
                bookOrPage = 'book' ifTrue:[
                    book bookTemplates at: (arr at: 3) put: content.
                ] ifFalse:[
                    bookOrPage = 'page' ifTrue:[
                        book pageTemplates at: (arr at: 3) put: content.
                    ] ifFalse:[
                        "ignore"
                    ].
                ]
        ].

"
                (arr at: 2) caseOf: {
                        ['book']->[book bookTemplates at: (arr at: 3) put: content].
                        ['page']->[book pageTemplates at: (arr at: 3) put: content]}
                        otherwise: []].
"
!

loadTemplatesForShelf: shelf
        | templatesDir metaColl content bookOrShelf|
        templatesDir := dir directoryNamed: 'default'.
        (templatesDir directoryExists: 'templates') ifFalse: [^self].
        templatesDir := templatesDir directoryNamed: 'templates'.
        metaColl := self getMetaCollForDir: templatesDir.
        dict at: 'templates' put: metaColl.
        metaColl do: [:arr |
                content := (templatesDir readOnlyFileNamed: (arr at: 1)) contentsOfEntireFile.
                "Use Internet Line Endings"
                content := TextFormatter crToCrlf: content.
                bookOrShelf := arr at:2.
                bookOrShelf = 'book' ifTrue:[
                    shelf bookTemplates at: (arr at: 3) put: content.
                ] ifFalse:[
                    bookOrShelf = 'shelf' ifTrue:[
                        shelf shelfTemplates at: (arr at: 3) put: content.
                    ] ifFalse:[
                        "ignore"
                    ].
                ]
        ]

"
                (arr at: 2) caseOf: {
                        ['book']->[shelf bookTemplates at: (arr at: 3) put: content].
                        ['shelf']->[shelf shelfTemplates at: (arr at: 3) put: content]}
                        otherwise: []].
"
! !

!SwikiStorage methodsFor:'shelf (logging)'!

logChangeShelfFunction: function type: type name: name
	((dir directoryNamed: 'default') fileNamed: 'browser.log')
		setToEnd;
		nextPutAll: (Date today asString);
		nextPut: Character tab;
		nextPutAll: (Time now asString);
		nextPut: Character tab;
		nextPutAll: 'changeShelfFunction';
		nextPut: Character tab;
		nextPutAll: function;
		nextPut: Character tab;
		nextPutAll: type;
		nextPut: Character tab;
		nextPutAll: name;
		nextPut: Character cr;
		close
!

logDeleteShelfFunction: function type: type name: name
	((dir directoryNamed: 'default') fileNamed: 'browser.log')
		setToEnd;
		nextPutAll: (Date today asString);
		nextPut: Character tab;
		nextPutAll: (Time now asString);
		nextPut: Character tab;
		nextPutAll: 'deleteShelfFunction';
		nextPut: Character tab;
		nextPutAll: function;
		nextPut: Character tab;
		nextPutAll: type;
		nextPut: Character tab;
		nextPutAll: name;
		nextPut: Character cr;
		close
! !

!SwikiStorage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiStorage.st,v 1.13 2003/09/30 15:37:07 penk Exp $'
! !
