"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

Object subclass:#TextFormatter
	instanceVariableNames:'match formattingArray'
	classVariableNames:'CrToCrlf CrlfToCr DecodeFromHttp DecodeFromXmlCr
		DecodeFromXmlCrlf EncodeToHttp EncodeToStrictXmlCr
		EncodeToStrictXmlCrlf EncodeToXmlCr EncodeToXmlCrlf'
	poolDictionaries:''
	category:'Comanche-Swiki-Formatting'
!


!TextFormatter class methodsFor:'instance creation'!

new
	^super new initialize
! !

!TextFormatter class methodsFor:'formatting'!

crToCrlf: aCrDelimitedString
	^CrToCrlf format: aCrDelimitedString
!

crlfToCr: aCrlfDelimitedString
	^CrlfToCr format: aCrlfDelimitedString
!

decodeFromHttp: aHttpString
	^DecodeFromHttp format: aHttpString
!

decodeFromXmlCr
	^DecodeFromXmlCr
!

decodeFromXmlCr: xml
	^DecodeFromXmlCr format: xml
!

decodeFromXmlCrlf
	^DecodeFromXmlCrlf
!

decodeFromXmlCrlf: xml
	^DecodeFromXmlCrlf format: xml
!

encodeToHttp: aString
	^EncodeToHttp format: aString
!

encodeToStrictXmlCr
	^EncodeToStrictXmlCr
!

encodeToStrictXmlCr: aString
	^EncodeToStrictXmlCr format: aString
!

encodeToStrictXmlCrlf
	^EncodeToStrictXmlCrlf
!

encodeToStrictXmlCrlf: aString
	^EncodeToStrictXmlCrlf format: aString
!

encodeToXmlCr
	^EncodeToXmlCr
!

encodeToXmlCr: aString
	^EncodeToXmlCr format: aString
!

encodeToXmlCrlf
	^EncodeToXmlCrlf
!

encodeToXmlCrlf: aString
	^EncodeToXmlCrlf format: aString
! !

!TextFormatter class methodsFor:'initialize-release'!

initialize
        | translation crlf|

        "Carriage Return Conversions"
        crlf _ String crlf.
        CrToCrlf _ self new 
                addMappingFrom: String cr to: crlf;
                addMappingFrom: crlf to: crlf;
                addMappingFrom: (Character lf asString) to: crlf;
                yourself.
        CrlfToCr _ self new
                addMappingFrom: crlf to: String cr;
                addMappingFrom: (Character lf asString) to: String cr;
                yourself.

        "Http Conversions"
        EncodeToHttp _ self new.
        DecodeFromHttp _ self new.
        '&<>"?*%#_' do: [:char |
                (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                    translation _ '%', (char asciiValue hexPrintString:2).
                ] ifFalse:[
                    translation _ '%', char hex.
                ].
                EncodeToHttp addMappingFrom: char asString to: translation.
                DecodeFromHttp addMappingFrom: translation to: char asString].
        EncodeToHttp addMappingFrom: ' ' to: '_'.
        DecodeFromHttp addMappingFrom: '_' to: ' '.
        "Xml Conversions"
        EncodeToStrictXmlCr _ self new
                addMappingFrom: '<' to: '&lt;';
                addMappingFrom: '>' to: '&gt;';
                addMappingFrom: '&' to: '&amp;';
                addMappingFrom: '"' to: '&quot;';
                addMappingFrom: crlf to: String cr;
                addMappingFrom: (Character lf asString) to: String cr;
                yourself.
        EncodeToStrictXmlCrlf _ self new
                addMappingFrom: '<' to: '&lt;';
                addMappingFrom: '>' to: '&gt;';
                addMappingFrom: '&' to: '&amp;';
                addMappingFrom: '"' to: '&quot;';
                addMappingFrom: String cr to: crlf;
                addMappingFrom: crlf to: crlf;
                addMappingFrom: (Character lf asString) to: crlf;
                yourself.
        EncodeToXmlCr _ self new
                addMappingFrom: '<' to: '&lt;';
                addMappingFrom: '>' to: '&gt;';
                addMappingFrom: '&' to: '&amp;';
                addMappingFrom: crlf to: String cr;
                addMappingFrom: (Character lf asString) to: String cr;
                yourself.
        EncodeToXmlCrlf _ self new
                addMappingFrom: '<' to: '&lt;';
                addMappingFrom: '>' to: '&gt;';
                addMappingFrom: '&' to: '&amp;';
                addMappingFrom: String cr to: crlf;
                addMappingFrom: crlf to: crlf;
                addMappingFrom: (Character lf asString) to: crlf;
                yourself.
        DecodeFromXmlCr _ self new
                addMappingFrom: '&lt;' to: '<';
                addMappingFrom: '&gt;' to: '>';
                addMappingFrom: '&amp;' to: '&';
                addMappingFrom: '&quot;' to: '"';
                addMappingFrom: crlf to: String cr;
                addMappingFrom: (Character lf asString) to: String cr;
                yourself.
        DecodeFromXmlCrlf _ self new
                addMappingFrom: '&lt;' to: '<';
                addMappingFrom: '&gt;' to: '>';
                addMappingFrom: '&amp;' to: '&';
                addMappingFrom: '&quot;' to: '"';
                addMappingFrom: String cr to: crlf;
                addMappingFrom: crlf to: crlf;
                addMappingFrom: (Character lf asString) to: crlf;
                yourself.

" 
    self initialize
"
! !

!TextFormatter methodsFor:'accessing'!

addAction: block from: beginning to: end
        "When the Action is performed, the text is scanned for things that start with beginning
        and stop with end. Whatever is between these two is then put into the block and what
        gets returned is fed to the formatted document."
        "The format for an Action is a 3 element array of (beginning end block)."
        | starts formatter |

        starts _ beginning at: 1.
        match add: starts.
        formatter _ Array new: 3.
        beginning size = 1
                ifTrue: [formatter at: 1 put: '']
                ifFalse: [formatter at: 1 put: (beginning copyFrom: 2 to: beginning size)].
        formatter at: 2 put: end.
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            formatter at: 3 put: (SimulatedSqueakBlock wrapIfBlock:block).
        ] ifFalse:[
            formatter at: 3 put: block.
        ].
        self addFormatter: formatter at: (starts asInteger).
!

addMappingFrom: original to: replacement
	"A Mapping is when the text is scanned and all instances of original are mapped to
	their replacements."
	"The format for a Mapping is an 3 element array of (original nil replacement)."
	| starts formatter |

	starts _ original at: 1.
	match add: starts.
	formatter _ Array new: 3.
	original size = 1
		ifTrue: [formatter at: 1 put: '']
		ifFalse: [formatter at: 1 put: (original copyFrom: 2 to: original size)].
	formatter at: 3 put: replacement.
	self addFormatter: formatter at: (starts asInteger).
!

remove: key
	"Removes a formatter (action, mapping) from the TextFormatter."
	"It remove it if it matches the orginal for a mapping or the beginning for an action."
	| starts formatters ends |

	starts _ key at: 1. "First character in the key."
	formatters _ formattingArray at: (starts asInteger).
	formatters ifNil: ["There is no matching formatter, so stop" ^nil]
		ifNotNil: [((key size) = 1) ifTrue: [ends _ ''] ifFalse: [ends _ key copyFrom: 2 to: (key size)].
			formatters do: [:element | ((element at: 1) = ends) ifTrue: [formatters remove: element]].
			((formatters size) = 0) ifTrue: [
				formattingArray at: (starts asInteger) put: nil. "Removes formatter"
				match remove: starts. "Removes from match"
			]
		].
	^nil
! !

!TextFormatter methodsFor:'initializing'!

initialize
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            match _ Set new.
        ] ifFalse:[
            match _ CharacterSet new.
        ].
        formattingArray _ Array new: 255.
! !

!TextFormatter methodsFor:'private'!

action: action value: value text: text
	^(action copy fixTemps) value: text
!

addFormatter: formatter at: position
	"Adds a formatter to the position in the formattingArray."
	| formatters |

	formatters _ formattingArray at: position.
	formatters isNil
		ifTrue: [formatters _ SortedCollection new sortBlock: [:x :y | (x at: 1) size >= (y at: 1) size].
			formattingArray at: position put: formatters].
	formatters add: formatter.
!

format: text actionValue: value onTo: stream
        "Apply all the formatting on text and put it onto stream."
        | pos i maxSize formatters index formatter what|

        "i is the position of a match"
        pos _ 1. "Position in text, marking the seperation of processed and yet to be processed"
        [(i _ text indexOfAnyOf: match startingAt: pos) > 0]
                whileTrue: [stream nextPutAll: (text copyFrom: pos to: i-1). "No formatting in this space"
                        maxSize _ text size - i.
                        formatters _ formattingArray at: (text at: i) asInteger.
                        index _ 1.
                        [formatters size >= index] "Index of Formatters"
                                whileTrue: [ "see if formatters match"
                                        formatter _ formatters at: index.
                                        "Check that size of formatter is allowed."
                                        (maxSize >= (formatter at: 1) size) ifTrue: [
                                                "Check if the formatter matches"
                                                ((text copyFrom: (i + 1) to: (i + (formatter at: 1) size)) = (formatter at: 1))
                                                        ifTrue: [ index _ formatters size + 1. "Match => exit loop"
                                                                "Set i to beyond beginning marker"
                                                                pos _ i + 1 + (formatter at: 1) size.
                                                                (formatter at: 2)
                                                                        ifNil: [ "Mapping"
                                                                                        stream nextPutAll: (formatter at: 3)
                                                                                ]
                                                                        ifNotNil: [ "Action"
                                                                                        | textPart |
                                                                                        "This does not allow nesting, but that's probably good."
                                                                                        i _ text findString: (formatter at: 2) startingAt: pos.
                                                                                        "Check for no closing"
                                                                                        textPart := (text copyFrom: pos to: i-1).
                                                                                        i > 0 ifTrue: [
                                                                                            what := (self action: (formatter at: 3) value: value text: textPart).
                                                                                            stream nextPutAll:what.
                                                                                                        pos _ i + (formatter at: 2) size
                                                                                                ]
                                                                                ]
                                                                ]
                                                ].
                                        index _ index + 1.
                                ].
                                "Check for no formatter matched"
                                ((formatters size) = (index - 1)) ifTrue: [stream nextPut: (text at: i). pos _ i + 1]
                ].
                "Add last section if there is one."
                (pos <= text size) ifTrue: [stream nextPutAll: (text copyFrom: pos to: (text size))].
         ^stream
! !

!TextFormatter methodsFor:'processing'!

format: text
        "Apply all the formatting to the text and return it as a stream"
        | return |

        "Create a new stream to write it on."
        return _ WriteStream on: (String new: text size).
        self format: text onTo: return.
        ^return contents
!

format: text onTo: stream
	^self format: text actionValue: nil onTo: stream
! !

!TextFormatter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/TextFormatter.st,v 1.10 2003/08/29 17:05:02 cg Exp $'
! !

TextFormatter initialize!
