"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiStorage subclass:#XmlSwikiStorage
	instanceVariableNames:''
	classVariableNames:'AssocDict TypesDict'
	poolDictionaries:''
	category:'Comanche-Swiki-Storage'
!


!XmlSwikiStorage class methodsFor:'instance creation'!

loadPagesFromDir: dir
	^(self fromDir: dir) loadPages
! !

!XmlSwikiStorage class methodsFor:'conversion'!

booleanSettingFrom: xml
	^xml = 'true'
!

booleanSettingNamed: aString value: value
	^'<s name="', (TextFormatter encodeToStrictXmlCr: aString), '" type="boolean">', (value ifTrue: ['true'] ifFalse: ['false']), '</s>'
!

codeSettingFrom: xml
        |codeString value|

        codeString := TextFormatter decodeFromXmlCr: xml.
        value := Compiler evaluate: codeString.
        value isNil ifTrue:[

            "in order to make the setup.xml-file compatible with the original Swiki,
             global classNames are saved without a nameSpace prefix, and we care for this
             here, when reloading the setup."

            (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                Class nameSpaceQuerySignal answer:self nameSpace
                do:[
                    value := Compiler evaluate: codeString.
                ].
            ].
        ].
        ^value
!

codeSettingNamed: aString value: value
        |storeString|

        "in order to make the setup.xml-file compatible with the original Swiki,
         we MUST save global classNames without a nameSpace prefix, and care for this
         when reloading the setup. Here, we only strip off the Comanche nameSpace."

        ((Smalltalk dialectName = 'SmalltalkX') 
        and:[ value isBehavior 
        and:[ value nameSpace == self nameSpace ]]) ifTrue:[
            storeString := value nameWithoutPrefix.
        ] ifFalse:[
            storeString := self storeStringFor: value.
        ].
        ^'<s name="', (TextFormatter encodeToStrictXmlCr: aString)
          , '" type="code">', (TextFormatter encodeToXmlCr: storeString)
          , '</s>'
!

collSettingFrom: xml
	| return |
	return _ OrderedCollection new.
	(xml findTokens: ' ') do: [:element | return add: element asNumber].
	^return
!

collSettingNamed: aString value: value
	| return |
	return _ '<s name="', (TextFormatter encodeToStrictXmlCr: aString), '" type="coll">'.
	value do: [:element | return _ return, (element asString), ' '].
	^(return copyFrom: 1 to: (return size - 1)), '</s>'
!

numberSettingFrom: xml
	^xml asNumber
!

numberSettingNamed: aString value: value
	^'<s name="', (TextFormatter encodeToStrictXmlCr: aString), '" type="number">', (value asString), '</s>'
!

performerFor: value
	TypesDict keysAndValuesDo: [:class :selector |
		(value isKindOf: class) ifTrue: [^selector]].
	^#codeSettingNamed:value:
!

settingAssocFromXml: xml
	| pos1 pos2 pos3 key type value |
	pos1 _ xml findString: 'name="' startingAt: 1.
	pos2 _ xml findString: '"' startingAt: (pos1 + 6).
	key _ TextFormatter decodeFromXmlCr: (xml copyFrom: (pos1 + 6) to: (pos2 - 1)).
	pos3 _ xml findString: '>' startingAt: pos2.
	pos1 _ xml findString: 'type="' startingAt: 1.
	((pos1 < pos3) and: [pos1 > 0])
		ifTrue: [pos2 _ xml findString: '"' startingAt: (pos1 + 6).
			type _ xml copyFrom: (pos1 + 6) to: (pos2 - 1)]
		ifFalse: [type _ 'code'].
	pos1 _ xml findString: '</s>' startingAt: pos3.
	value _ xml copyFrom: (pos3 + 1) to: (pos1 - 1).
	value _ self perform: (AssocDict at: type) with: value.
	^Association key: key value: value
!

settingStringNamed: aString value: value
	^self perform: (self performerFor: value) with: aString with: value
!

settingsDictFrom: xml
	| pos1 pos2 dict |

	dict _ Dictionary new.
	pos1 _ 1.
	[0 = (pos1 _ xml findString: '<s ' startingAt: pos1)] whileFalse: [
		pos2 _ xml findString: '</s>' startingAt: pos1.
		dict add: (self settingAssocFromXml: (xml copyFrom: pos1 to: (pos2 + 3))).
		pos1 _ pos1 + 1].
	^dict
!

settingsStringFrom: dict
	"Converts a dictionary to a xml string."
	| xml |

	xml _ ''.
	dict keysAndValuesDo: [:key :value | xml _ xml, (self settingStringNamed: key value: value), String cr].
	^xml
!

storeStringFor: object
	| return |

	(object isKindOf: OrderedCollection) ifTrue: [
		((object at: 1 ifAbsent: ['do not do']) isKindOf: Integer)
			ifTrue: [return _ '#('.
				object do: [:element | return _ return, (element storeString), ' '].
				^return, ') asOrderedCollection']].
	^object storeString
!

textDictionaryFrom: xml start: start end: end
	| dict pos1 pos2 key value |

	dict _ Dictionary new.
	pos2 _ start - 1.
	[pos1 _ xml findString: '<t name="' startingAt: pos2. (pos1 = 0) or: [pos1 > end]] whileFalse: ["Keep adding elements"
		pos2 _ xml findString: '">' startingAt: pos1.
		key _ TextFormatter decodeFromXmlCr: (xml copyFrom: (pos1 + 9) to: (pos2 - 1)).
		pos1 _ xml findString: '</t>' startingAt: pos2.
		value _ Array new: 2.
		value at: 1 put: (pos2 + 2).
		value at: 2 put: (pos1 - 1).
		dict at: key put: value.
		pos2 _ pos1 + 2].
	^dict
!

textFrom: xml start: start end: end
	| arr test |

	(start < end) ifTrue: [test _ xml findString: '<' startingAt: start.
		(test < end) ifTrue: [^self textDictionaryFrom: xml start: start end: end]].
	arr _ Array new: 2.
	arr at: 1 put: start.
	arr at: 2 put: end.
	^arr
!

textSettingFrom: xml
	^TextFormatter decodeFromXmlCr: xml
!

textSettingNamed: aString value: value
	^'<s name="', (TextFormatter encodeToStrictXmlCr: aString), '" type="text">', (TextFormatter encodeToXmlCr: value), '</s>'
! !

!XmlSwikiStorage class methodsFor:'initialize-release'!

initialize
	"Dictionaries"
	TypesDict _ Dictionary new.
	TypesDict
		at: String put: #textSettingNamed:value:;
		at: Number put: #numberSettingNamed:value:;
		at: Boolean put: #booleanSettingNamed:value:;
		at: OrderedCollection put: #collSettingNamed:value:.
	AssocDict _ Dictionary new.
	AssocDict
		at: 'text' put: #textSettingFrom:;
		at: 'number' put: #numberSettingFrom:;
		at: 'boolean' put: #booleanSettingFrom:;
		at: 'coll' put: #collSettingFrom:;
		at: 'code' put: #codeSettingFrom:.
! !

!XmlSwikiStorage class methodsFor:'testing class hierarchy'!

handlesBookStorage
	^true
!

handlesPageStorage
	^true
! !

!XmlSwikiStorage methodsFor:'book (load)'!

loadSettingsForBook: book
        | raw pos1 pos2 |

        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            | retDict |
            (dir fileExists: 'settings.xml') ifTrue:[
                raw _ (dir readOnlyFileNamed: 'settings.xml') contentsOfEntireFile.
                pos1 _ raw findString: '<settings>' startingAt: 1.
                pos2 _ raw findString: '</settings>' startingAt: pos1.
                retDict := XmlSwikiStorage settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 10)                
            ] ifFalse:[
                retDict := Dictionary new                
            ].
            book rawSettings:retDict.
        ] ifFalse:[
            book rawSettings: ((dir fileExists: 'settings.xml')
                    ifTrue: [raw _ (dir readOnlyFileNamed: 'settings.xml') contentsOfEntireFile.
                            pos1 _ raw findString: '<settings>' startingAt: 1.
                            pos2 _ raw findString: '</settings>' startingAt: pos1.
                            XmlSwikiStorage settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 10)]
                    ifFalse: [Dictionary new])
        ]
!

writeSettingsForBook: book
	| file |
	"Delete Old Settings"
	dir deleteFileNamed: 'settings.xml'.
	"Add New Settings"
	file _ dir newFileNamed: 'settings.xml'.
	file nextPutAll: '<?xml version="1.0"?>', String cr.
	file nextPutAll: '<settings>', String cr.
	file nextPutAll: (self class settingsStringFrom: book rawSettings).
	file nextPutAll: '</settings>'.
	file close.
! !

!XmlSwikiStorage methodsFor:'pages'!

backupPage: aPage
	"Back-up a .xml page to put it's contents into a .old page."
	| oldPath oldStream file pos |

	"Get the path for the .old file."
	oldPath _ (aPage id asString), '.old'.
	"Get the .old stream to write on"
	(dir fileExists: oldPath)
		ifTrue: [oldStream _ dir oldFileNamed: oldPath.
				oldStream setToEnd]
		ifFalse: [oldStream _ dir newFileNamed: oldPath].
	file _ (dir readOnlyFileNamed: (aPage id asString, '.xml')) contentsOfEntireFile.
	pos _ file findString: '<page>' startingAt: 1.
	"Add current page."
	oldStream nextPutAll: (file copyFrom: pos to: file size).
	oldStream nextPutAll: String cr.
	oldStream close
!

isAComplexPage: aPage
        | text |
        text _ aPage rawText.
        (text isKindOf: String) ifTrue: [^false].
        (text isKindOf: Array) ifTrue: [^false].
        ^true
!

loadPage: aPage
        | xml pos0 |
        xml _ (aPage versionId = 0)
                ifTrue: [(dir readOnlyFileNamed: ((aPage id asString), '.xml')) contentsOfEntireFile]
                ifFalse: [dir readOnlyFileNamed: ((aPage id asString), '.old')].
        (aPage versionId > 0)
                ifTrue: [pos0 _ 0.
                        1 to: (aPage versionId) do: [:i | pos0 _ xml findString: '<page>' startingAt: (pos0 + 1).
                                (pos0 = 0) ifTrue: [^nil]]]
                ifFalse: [pos0 _ 1].
        self loadPage: aPage from: xml startingAt: pos0.
        ^ aPage
!

loadPage: aPage from: xml startingAt: pos0
        | pos1 pos2 date userAndUserNameString userAndUserName user userName|

        "Version"
        pos1 _ xml findString: '<version date="' startingAt: pos0.
        pos2 _ xml findString: '" time="' startingAt: pos1.
            "Date"
        date _ (xml copyFrom: (pos1 + 15) to: (pos2 - 1)) findTokens: './'.
        aPage date: (Date newDay: (date at: 1) asNumber month: (date at: 2) asNumber year: (date at: 3) asNumber).
            "Time"
        pos1 _ xml findString: '" user="' startingAt: pos2.
        aPage time: (Time readFrom: (ReadStream on: (xml copyFrom: (pos2 + 8) to: (pos1 - 1)))).
        pos2 _ xml findString: '" /' startingAt: pos1.
        userAndUserNameString := (xml copyFrom: pos1 + 2 to:pos2 ).
        userAndUserName := userAndUserNameString asCollectionOfSubstringsSeparatedBy:(Character space).
        user := userAndUserName first.
            "User (peer)"
        aPage user: (user copyFrom:7 to:(user size - 1) ).
        userAndUserName size = 2 ifTrue:[
            "User name (authorization)"
            userName := userAndUserName second.
            aPage userName:(userName copyFrom:11 to:(userName size - 1)).
        ].
        "Settings"
        pos1 _ xml findString: '<settings>' startingAt: pos2.
        pos2 _ xml findString: '</settings>' startingAt: pos1.
        aPage settings: (self class settingsDictFrom: (xml copyFrom: (pos1) to: (pos2 + 10))).
        "Name"
        pos1 _ xml findString: '<name>' startingAt: pos2.
        pos2 _ xml findString: '</name>' startingAt: pos1.
        aPage name: (TextFormatter decodeFromXmlCr: (xml copyFrom: (pos1 + 6) to: (pos2 - 1))).
        "Text"
        pos1 _ xml findString: '<text>' startingAt: pos2.
        pos2 _ xml findString: '</text>' startingAt: pos1.
        aPage text: (self class textFrom: xml start: (pos1 + 6) end: (pos2 - 1))
!

loadPages
        | id pages page files |
        pages _ OrderedCollection new.
        id _ 1.
        files _ dir fileNames.
        [files includes: (id asString, '.xml')] whileTrue: [
                page _ NuSwikiPage new id: id; versionId: 0; storage: self.
                page _ self loadPage: page.
                pages add: page.
                id _ id + 1].
        ^pages
!

loadVersionsFrom: aPage
	| versions xml pos0 version vid |
	versions _ OrderedCollection new.
	(dir fileExists: (aPage id asString, '.old'))
		ifTrue: [xml _ dir readOnlyFileNamed: (aPage id asString, '.old').
			vid _ 1.
			pos0 _ 0.
			[0 = (pos0 _ xml findString: '<page>' startingAt: (pos0 + 1))] whileFalse: [
				version _ aPage class new id: (aPage id); versionId: vid; storage: self.
				self loadPage: version from: xml startingAt: pos0.
				versions add: version.
				vid _ vid + 1]].
	^versions
!

multipleTextFrom: aPage
	| file return |
	file _ dir readOnlyFileNamed: (self pathFrom: aPage).
	return _ Dictionary new.
	aPage rawText keysAndValuesDo: [:key :value |
		(value isKindOf: Array)
			ifTrue: [file position: ((value at: 1) - 1).
				return at: key put: (TextFormatter decodeFromXmlCr: (file next: (1 + (value at: 2) - (value at: 1))))]
			ifFalse: [return at: key put: value]].
	file close.
	^return
!

pathFrom: aPage
        |idString|

        idString := aPage id asString.
        ^(aPage versionId = 0)
                ifTrue: [idString, '.xml']
                ifFalse: [idString, '.old']

    "Modified: / 27-07-2006 / 10:14:32 / cg"
!

simpleTextFrom: aPage
	| file return |
	file _ dir readOnlyFileNamed: (self pathFrom: aPage).
	file position: ((aPage rawText at: 1) - 1).
	return _ TextFormatter decodeFromXmlCr: (file next: (1 + (aPage rawText at: 2) - (aPage rawText at: 1))).
	file close.
	^return
!

textFrom: aPage
        | text |
        text _ aPage rawText.

        (text isKindOf: String) ifTrue: [^text].
        (text isKindOf: Array) ifTrue: [^self simpleTextFrom: aPage].
        (text isKindOf: Dictionary) ifTrue: [((text values at: 1) isKindOf: String)
                ifTrue: [^text]
                ifFalse: [^self multipleTextFrom: aPage]].
        ^''
!

textKey: key from: aPage
        | text arr return file |
        text _ aPage rawText.
        (text isKindOf: String) ifTrue: [^''].
        (text isKindOf: Array) ifTrue: [^''].
        arr _ text at: key ifAbsent: [^''].
        (arr isKindOf: String) ifTrue: [^arr].
        file _ dir readOnlyFileNamed: (self pathFrom: aPage).
        file position: ((arr at: 1) - 1).
        return _ TextFormatter decodeFromXmlCr: (file next: (1 + (arr at: 2) - (arr at: 1))).
        file close.
        ^return
!

writePage: aPage
        | pageFilePath file |

        pageFilePath := self pathFrom: aPage.

        "Delete Old Version"
        dir deleteFileNamed: pageFilePath.
        "Header"
        file _ dir newFileNamed: pageFilePath.
        file nextPutAll: '<?xml version="1.0"?>', String cr.
        file nextPutAll: '<page>', String cr.
        "Version"
            "Date"
        file nextPutAll: '<version date="'.
        file nextPutAll: (aPage date dayOfMonth asString, '/', aPage date monthIndex asString, '/', aPage date year asString).
            "Time"
        file nextPutAll: '" time="'.
        aPage time printOn: file.
            "User (peer)"
        file nextPutAll: '" user="', aPage user, '" '.
            "User name (authorization)"
        file nextPutAll: 'userName="', (aPage userName asString ? ''), '" />', String cr.
        "Settings"
        file nextPutAll: '<settings>', String cr.
        file nextPutAll: (self class settingsStringFrom: aPage settings).
        file nextPutAll: '</settings>', String cr.
        "Name"
        file nextPutAll: '<name>'.
        TextFormatter encodeToXmlCr format: aPage name onTo: file.
        file nextPutAll: '</name>', String cr.
        "Text"
        file nextPutAll: '<text>'.
        self addTextFrom: aPage to: file.
        file nextPutAll: '</text>', String cr.
        file nextPutAll: '</page>'.
        file close.

    "Modified: / 27-07-2006 / 10:15:29 / cg"
! !

!XmlSwikiStorage methodsFor:'private'!

addTextFrom: aPage to: file
	| saveText new arr|
	saveText _ aPage text.
	(saveText isKindOf: Dictionary)
		ifTrue: [new _ Dictionary new.
			file nextPutAll: String cr.
			saveText keysAndValuesDo: [:key :value |
				arr _ Array new: 2.
				file nextPutAll: '<t name="'.
				TextFormatter encodeToStrictXmlCr format: key onTo: file.
				file nextPutAll: '">'.
				arr at: 1 put: (1 + file position).
				TextFormatter encodeToXmlCr format: value onTo: file.
				arr at: 2 put: (file position).
				new at: key put: arr.
				file nextPutAll: '</t>', String cr]]
		ifFalse: [new _ Array new: 2.
			new at: 1 put: (1 + file position).
			TextFormatter encodeToXmlCr format: saveText onTo: file.
			new at: 2 put: file position].
	aPage text: new
!

getMetaCollForDir: aDir
        | tokens return id type name category pos1 pos2 crlf|
        return _ OrderedCollection new.
        (aDir fileExists: 'meta.xml') ifFalse: [^return].
        crlf _ String crlf.

        tokens _ (aDir readOnlyFileNamed: 'meta.xml') contentsOfEntireFile findTokens: crlf.
        tokens do: [:token | (token beginsWith: '<file') ifTrue: [
                pos1 _ token findString: 'id="'.
                pos2 _ token findString: '"' startingAt: (pos1 + 4).
                id _ token copyFrom: (pos1+4) to: (pos2 - 1).
                pos1 _ token findString: 'type="'.
                pos2 _ token findString: '"' startingAt: (pos1 + 6).
                type _ token copyFrom: (pos1+6) to: (pos2 - 1).
                pos1 _ token findString: 'name="'.
                pos2 _ token findString: '"' startingAt: (pos1 + 6).
                name _ token copyFrom: (pos1+6) to: (pos2 - 1).
                (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                    ((pos1 _ token findString: 'category="') = 0) ifTrue: [
                        category := nil
                    ] ifFalse: [
                        pos2 _ token findString: '"' startingAt: (pos1 + 10).
                        category := token copyFrom: (pos1 + 10) to: (pos2 - 1).
                    ]
                ] ifFalse:[
                    category _ ((pos1 _ token findString: 'category="') = 0)
                            ifTrue: [nil]
                            ifFalse: [pos2 _ token findString: '"' startingAt: (pos1 + 10).
                                    token copyFrom: (pos1 + 10) to: (pos2 - 1)].
                ].
                return add: (Array with: id with: type with: name with: category)]].
        ^return
!

putMetaColl: coll forDir: aDir
	| file |
	(aDir fileExists: 'meta.xml') ifTrue: [aDir deleteFileNamed: 'meta.xml'].
	file _ aDir newFileNamed: 'meta.xml'.
	file
		nextPutAll: '<?xml version="1.0"?>';
		nextPut: Character cr;
		nextPutAll: '<meta>';
		nextPut: Character cr.
	coll do: [:arr |
		file
			nextPutAll: '<file id="';
			nextPutAll: (arr at: 1);
			nextPutAll: '" type="';
			nextPutAll: (arr at: 2);
			nextPutAll: '" name="';
			nextPutAll: (arr at: 3).
		(arr at: 4) ifNotNil: [file
			nextPutAll: '" category="';
			nextPutAll: (arr at: 4)].
		file
			nextPutAll: '" />';
			nextPut: Character cr].
	file
		nextPutAll: '</meta>';
		close
! !

!XmlSwikiStorage methodsFor:'shelf'!

loadSettingsForShelf: shelf
        | ddir raw pos1 pos2 |
        ddir _ dir directoryNamed: 'default'.
        (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
            (ddir fileExists: 'settings.xml') ifTrue:[
                raw _ (ddir readOnlyFileNamed: 'settings.xml') contentsOfEntireFile.
                pos1 _ raw findString: '<settings>' startingAt: 1.
                pos2 _ raw findString: '</settings>' startingAt: pos1.
                shelf settings:(self class settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 10))
            ] ifFalse:[
                shelf settings:(Dictionary new).
            ].
        ] ifFalse:[
            shelf settings: ((ddir fileExists: 'settings.xml')
                    ifTrue: [raw _ (ddir readOnlyFileNamed: 'settings.xml') contentsOfEntireFile.
                            pos1 _ raw findString: '<settings>' startingAt: 1.
                            pos2 _ raw findString: '</settings>' startingAt: pos1.
                            self class settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 10)]
                    ifFalse: [Dictionary new])        
        ]
!

setupBooks
	| books book bdir raw pos1 pos2 setup |
	books _ OrderedCollection new.
	dir directoryNames do: [:dname | (dname = 'default') ifFalse: [
		bdir _ dir directoryNamed: dname.
		(bdir fileExists: 'setup.xml') ifTrue: [
			raw _ (bdir readOnlyFileNamed: 'setup.xml') contentsOfEntireFile.
			pos1 _ raw findString: '<setup>' startingAt: 1.
			pos2 _ raw findString: '</setup>' startingAt: pos1.
			setup _ XmlSwikiStorage settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 7).
			book _ (setup includesKey: 'parent')
				ifTrue: [SwikiSubBook new]
				ifFalse: [SwikiBook new].
			book
				name: dname;
				setup: setup.
			books add: book]]].
	^books
!

writeSettingsForShelf: shelf
	| ddir file |
	"Delete Old Settings"
	(ddir _ dir directoryNamed: 'default') deleteFileNamed: 'settings.xml'.
	"Add New Settings"
	file _ ddir newFileNamed: 'settings.xml'.
	file nextPutAll: '<?xml version="1.0"?>', String cr.
	file nextPutAll: '<settings>', String cr.
	file nextPutAll: (self class settingsStringFrom: shelf settings).
	file nextPutAll: '</settings>'.
	file close.
!

writeSetupForBook: book
	| bdir file |
	bdir _ dir directoryNamed: book name.
	"Delete Old Setup"
	bdir deleteFileNamed: 'setup.xml'.
	"Add New Setup"
	file _ bdir newFileNamed: 'setup.xml'.
	file nextPutAll: '<?xml version="1.0"?>', String cr.
	file nextPutAll: '<setup>', String cr.
	file nextPutAll: (self class settingsStringFrom: book setup).
	file nextPutAll: '</setup>'.
	file close.
! !

!XmlSwikiStorage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/XmlSwikiStorage.st,v 1.19 2006/07/27 14:54:47 cg Exp $'
! !

XmlSwikiStorage initialize!
