"{ Package: 'stx:goodies/webServer/comanche/swiki/xpSwiki' }"

"{ NameSpace: Comanche }"

TestCase subclass:#XPSwikiShelfTest
	instanceVariableNames:'launcher module shelf request response'
	classVariableNames:''
	poolDictionaries:''
	category:'XPSwiki-Tests'
!


!XPSwikiShelfTest methodsFor:'tests'!

setUp
	launcher _ XPSwikiLauncher package: #swikiWebServer.
	launcher port: 8080.
	module _ launcher startServer.
	shelf _ module shelf.
	request _ MockSwikiRequest new.
	request address: 'createNew'.
	response _ Dictionary new.
	request fieldsAt: 'name' put: 'test'.
	request fieldsAt: 'bookStorage' put: 'MockSwikiStorage'.
	request fieldsAt: 'pageStorage' put: 'MockSwikiStorage'.
!

tearDown
	| dir |
	dir _ shelf storage dir directoryNamed: 'test'.
	launcher stopServer.
	dir exists ifTrue: [dir recursiveDelete]
!

testCreateNew
	| parent |
	parent _ MockXPSwikiBook new.
	parent setup at: 'inheritable' put: true.
	parent result: true.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (shelf bookNamed: 'test') notNil.
	self assert: (response at: 'headerStatus')
			= #tempMoved.
	self assert: (response at: 'location')
			= '/admin/setup?book=test'
!

testCreateNewBookExplicitlyNotInheritable
	|parent |
	parent _ MockXPSwikiBook new.
	parent setup at: 'inheritable' put: false.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'This book cannot be inherited from.'
!

testCreateNewBookNotInheritable
	| parent |
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'This book cannot be inherited from.'
!

testCreateNewExistingName
	| parent |
	request fieldsAt: 'name' put: 'refs'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That name is already taken.'
!

testCreateNewFailing
	| parent |
	parent _ MockXPSwikiBook new.
	parent setup at: 'inheritable' put: true.
	parent result: nil.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'Create.priv failed.'
!

testCreateNewFromASwikiSubBook
	| parent |

	parent _ SwikiSubBook new.
	parent setup: Dictionary new.
	parent name: 'testParent'.
	parent setup at: 'inheritable' put: true.
	parent
		parent: (shelf bookNamed: 'refs').
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self
		assert: ((shelf bookNamed: 'test')
				isKindOf: SwikiSubBook)
!

testCreateNewFromRefsBook
	| parent |

	parent _ shelf bookNamed: 'refs'.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self
		assert: ((shelf bookNamed: 'test')
				isKindOf: SwikiSubBook)
!

testCreateNewInvalidBookStorage
	| parent |
	request fieldsAt: 'bookStorage' put: 'SwikiStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That book storage is invalid.'
!

testCreateNewInvalidName
	| parent |
	request fieldsAt: 'name' put: 'test.'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That name is not allowed (the name must contain only letters, numbers, and/or { ~ - _ }.'
!

testCreateNewInvalidPageStorage
	|parent |
	request fieldsAt: 'pageStorage' put: 'SwikiStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That page storage is invalid.'
!

testCreateNewNoBookStorage
	| parent |
	request fieldsRemoveKey: 'bookStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'The parameter <b>book storage</b> was not transmitted.'
!

testCreateNewNoName
	| parent |
	request fieldsRemoveKey: 'name'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'The parameter <b>name</b> was not transmitted.'
!

testCreateNewNoPageStorage
	| parent |
	request fieldsRemoveKey: 'pageStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'The parameter <b>page storage</b> was not transmitted.'
!

testCreateNewUnexistingBookStorage
	| parent |
	request fieldsAt: 'bookStorage' put: 'UnexistingBookStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That book storage does not exist.'
!

testCreateNewUnexistingPageStorage
	| parent |
	request fieldsAt: 'pageStorage' put: 'UnexistingPageStorage'.
	parent _ MockXPSwikiBook new.
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self assert: (request settingsAt: 'error')
			= 'That page storage does not exist.'
!

testCreateNewXPSwikiSubBook
	| parent |

	parent _ SwikiSubBook new.
	parent setup: Dictionary new.
	parent name: 'testParent'.
	parent setup at: 'inheritable' put: true.
	parent
		parent: (shelf bookNamed: 'refs').
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	self
		assert: ((shelf bookNamed: 'test')
				isKindOf: XPSwikiBook)
!

testLoadANewBook
	| parent |
	parent _ (shelf bookNamed: 'template').
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	module _ launcher stopServer; startServer.
	shelf _ module shelf.
	self
		assert: ((shelf bookNamed: 'test')
				isKindOf: XPSwikiBook)
!

testLoadStorageFromParent
	| parent book |

	parent _ SwikiSubBook new.
	parent setup: Dictionary new.
	parent name: 'template'.
	parent setup at: 'inheritable' put: true.
	parent setup at: 'bookStorage' put: XPSwikiStorage.
	parent setup at: 'pageStorage' put: XPSwikiStorage.
	parent
		parent: (shelf bookNamed: 'refs').
	shelf
		formatBookAction: request address
		request: request
		response: response
		book: parent.
	book _ shelf bookNamed: 'test'.
	self
		assert: (book storage isKindOf: XPSwikiStorage).
	self assert: (book setup at: 'bookStorage')
			= XPSwikiStorage.
	self assert: (book setup at: 'pageStorage')
			= XPSwikiStorage
! !

!XPSwikiShelfTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/xpSwiki/Comanche__XPSwikiShelfTest.st,v 1.1 2004/03/16 17:52:45 cg Exp $'
! !
