"{ Package: 'stx:goodies/webServer/comanche/swiki/xpSwiki' }"

"{ NameSpace: Comanche }"

XmlSwikiStorage subclass:#XPSwikiStorage
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'XPSwiki-Storage'
!


!XPSwikiStorage methodsFor:'append / remove'!

deletePage: oldPage 
	"Delete a page. See XPSwikiPage > append / remove > delete"
	| file newPage newDict |
	
	newPage _ XPSwikiPage new.
	newPage name: oldPage name, 'deleted-', Date today asString,'-', Time now asString.
	newPage id: oldPage id.
	newPage time: (Time now).
	newPage date: (Date today).
	newPage user: 'unknown'.
	newDict := Dictionary new.
	newDict at: '1' put: 'description'.
	newPage text: newDict.
	newPage storage: oldPage storage.
	newPage book: oldPage book.
	
	dir
		deleteFileNamed: (oldPage id asString, '.xml').
	dir
		deleteFileNamed: (oldPage id asString, '.old').
	"Header"
	file _ dir
				newFileNamed: (newPage id asString, '.xml').
	file nextPutAll: '<?xml version="1.0"?>' , String cr.
	file nextPutAll: '<page>' , String cr.
	"Class"
	file nextPutAll: '<class>'.
	file nextPutAll: newPage class name.
	file nextPutAll: '</class>' , String cr.
	"Version"
	file nextPutAll: '<version date="'.
	file nextPutAll: newPage date dayOfMonth asString , '/' , newPage date monthIndex asString , '/' , newPage date year asString.
	file nextPutAll: '" time="'.
	newPage time printOn: file.
	file nextPutAll: '" user="' , newPage user , '" />' , String cr.
	"Settings"
	file nextPutAll: '<settings>' , String cr.
	file
		nextPutAll: (self class settingsStringFrom: newPage settings).
	file nextPutAll: '</settings>' , String cr.
	"Name"
	file nextPutAll: '<name>'.
	TextFormatter encodeToXmlCr format: newPage name onTo: file.
	file nextPutAll: '</name>' , String cr.
	"Text"
	file nextPutAll: '<text>'.
	newPage storage addTextFrom: newPage to: file.
	file nextPutAll: '</text>' , String cr.
	file nextPutAll: '</page>'.
	file close.

	(newPage book pages) at: newPage id put: newPage.
! !

!XPSwikiStorage methodsFor:'as yet unclassified'!

textFrom: aPage
	| text |
	text _ aPage rawText.
	(text isKindOf: String) ifTrue: [^text].
	(text isKindOf: Array) ifTrue: [^self simpleTextFrom: aPage].
	(text isKindOf: Dictionary) ifTrue: [
		(text values size = 0) ifTrue:[^ text].
		((text values at: 1) isKindOf: String)
			ifTrue: [^text]
			ifFalse: [^self multipleTextFrom: aPage]].
	^''
! !

!XPSwikiStorage methodsFor:'history'!

loadPageSuper: aPage 
	| xml pos0 |
	"the only difference between this method and the loadPage: vid method of superclass is in the double call to contentsOfEntireFile.
	For this reason i can't call super loadPage: aPage"
	xml _ aPage versionId = 0
				ifTrue: [(dir readOnlyFileNamed: aPage id asString , '.xml') contentsOfEntireFile]
				ifFalse: [(dir readOnlyFileNamed: aPage id asString , '.old') contentsOfEntireFile].
	aPage versionId > 0
		ifTrue: [pos0 _ 0.
			1
				to: aPage versionId
				do: [:i | 
					pos0 _ xml findString: '<page>' startingAt: pos0 + 1.
					pos0 = 0
						ifTrue: [^ nil]]]
		ifFalse: [pos0 _ 1].
	"loadPage: from: startingAt: defined in XPSwikiStorage doesn't work here. I need to call the same method defined in the superclass
	SwikiStorage"
	^super
		loadPage: aPage
		from: xml
		startingAt: pos0
!

loadVersionsFrom: aPage 
	| versions xml pos0 version vid |
	versions _ OrderedCollection new.
	(dir fileExists: aPage id asString , '.old')
		ifTrue: [xml _ (dir readOnlyFileNamed: aPage id asString , '.old') contentsOfEntireFile.
			vid _ 1.
			pos0 _ 0.
			[0 = (pos0 _ xml findString: '<page>' startingAt: pos0 + 1)]
				whileFalse: [version _ aPage class new id: aPage id;
								 versionId: vid;
								 storage: self.
					version book: aPage book.
					versions add: (self
						loadPage: version
						from: xml
						startingAt: pos0).
					vid _ vid + 1]].
	^ versions
! !

!XPSwikiStorage methodsFor:'pages'!

loadPage: aPage 
	| xml pos0 |
	xml _ aPage versionId = 0
				ifTrue: [(dir readOnlyFileNamed: aPage id asString , '.xml') contentsOfEntireFile]
				ifFalse: [(dir readOnlyFileNamed: aPage id asString , '.old') contentsOfEntireFile].
	aPage versionId > 0
		ifTrue: [pos0 _ 0.
			1
				to: aPage versionId
				do: [:i | 
					pos0 _ xml findString: '<page>' startingAt: pos0 + 1.
					pos0 = 0
						ifTrue: [^ nil]]]
		ifFalse: [pos0 _ 1].
	^self
		loadPage: aPage
		from: xml
		startingAt: pos0
!

loadPage: aPage from: xml startingAt: pos0 
        "Version"
        | pos1 pos2 date newPage |
        pos1 _ xml findString: '<class>' startingAt: pos0.
        pos2 _ xml findString: '</class>' startingAt: pos1.
        Smalltalk isSmalltalkX ifTrue:[
            | className |
            className := (self class nameSpace
                                classNamed: (xml copyFrom: pos1 + 7 to: pos2 - 1)).
            className isNil ifTrue:[
                className := Smalltalk classNamed:(xml copyFrom: pos1 + 7 to: pos2 - 1).
            ].
            newPage _ className new.
        ] ifFalse:[
            newPage _ (Smalltalk
                                classNamed: (xml copyFrom: pos1 + 7 to: pos2 - 1)) new.
        ].
        newPage id: aPage id;
                 versionId: aPage versionId;
                 storage: aPage storage; book: aPage book.
        pos1 _ xml findString: '<version date="' startingAt: pos2.
        pos2 _ xml findString: '" time="' startingAt: pos1.
        date _ (xml copyFrom: pos1 + 15 to: pos2 - 1)
                                findTokens: './'.
        newPage
                date: (Date
                                newDay: (date at: 1) asNumber
                                month: (date at: 2) asNumber
                                year: (date at: 3) asNumber).
        pos1 _ xml findString: '" user="' startingAt: pos2.
        newPage
                time: (Time
                                readFrom: (ReadStream
                                                on: (xml copyFrom: pos2 + 8 to: pos1 - 1))).
        pos2 _ xml findString: '" />' startingAt: pos1.
        newPage
                user: (xml copyFrom: pos1 + 8 to: pos2 - 1).
        "Settings"
        pos1 _ xml findString: '<settings>' startingAt: pos2.
        pos2 _ xml findString: '</settings>' startingAt: pos1.
        newPage
                settings: (self class
                                settingsDictFrom: (xml copyFrom: pos1 to: pos2 + 10)).
        "Name"
        pos1 _ xml findString: '<name>' startingAt: pos2.
        pos2 _ xml findString: '</name>' startingAt: pos1.
        newPage
                name: (TextFormatter
                                decodeFromXmlCr: (xml copyFrom: pos1 + 6 to: pos2 - 1)).
        "Text"
        pos1 _ xml findString: '<text>' startingAt: pos2.
        pos2 _ xml findString: '</text>' startingAt: pos1.
        newPage
                text: (self class
                                textFrom: xml
                                start: pos1 + 6
                                end: pos2 - 1).
        ^ newPage
!

loadPagesForBook: aBook 
        | id pages page files |
        pages _ OrderedCollection new.
        id _ 1.
        files _ dir fileNames.
        [files includes: id asString , '.xml']
                whileTrue: [page _ XPSwikiPage new id: id;
                                                 versionId: 0;
                                                 storage: self; book: aBook.
                        pages
                                add: (self loadPage: page).
                        id _ id + 1].
        ^ pages
!

writePage: aPage 
	"Delete Old Version"
	| file |
	dir
		deleteFileNamed: (self pathFrom: aPage).
	"Header"
	file _ dir
				newFileNamed: (self pathFrom: aPage).
	file nextPutAll: '<?xml version="1.0"?>' , String cr.
	file nextPutAll: '<page>' , String cr.
	"Class"
	file nextPutAll: '<class>'.
	file nextPutAll: aPage class name.
	file nextPutAll: '</class>' , String cr.
	"Version"
	file nextPutAll: '<version date="'.
	file nextPutAll: aPage date dayOfMonth asString , '/' , aPage date monthIndex asString , '/' , aPage date year asString.
	file nextPutAll: '" time="'.
	aPage time printOn: file.
	file nextPutAll: '" user="' , aPage user , '" />' , String cr.
	"Settings"
	file nextPutAll: '<settings>' , String cr.
	file
		nextPutAll: (self class settingsStringFrom: aPage settings).
	file nextPutAll: '</settings>' , String cr.
	"Name"
	file nextPutAll: '<name>'.
	TextFormatter encodeToXmlCr format: aPage name onTo: file.
	file nextPutAll: '</name>' , String cr.
	"Text"
	file nextPutAll: '<text>'.
	self addTextFrom: aPage to: file.
	file nextPutAll: '</text>' , String cr.
	file nextPutAll: '</page>'.
	file close
! !

!XPSwikiStorage methodsFor:'shelf'!

setupBooks
	| books book bdir raw pos1 pos2 setup |
	books _ OrderedCollection new.
	dir directoryNames
		do: [:dname | dname = 'default'
				ifFalse: [bdir _ dir directoryNamed: dname.
					(bdir fileExists: 'setup.xml')
						ifTrue: [raw _ (bdir readOnlyFileNamed: 'setup.xml') contentsOfEntireFile.
							pos1 _ raw findString: '<setup>' startingAt: 1.
							pos2 _ raw findString: '</setup>' startingAt: pos1.
							setup _ XmlSwikiStorage
										settingsDictFrom: (raw copyFrom: pos1 to: pos2 + 7).
							book _ (setup includesKey: 'parent')
										ifTrue: [XPSwikiBook new]
										ifFalse: [SwikiBook new].
							book name: dname;
								 setup: setup.
							books add: book]]].
	^ books
! !

!XPSwikiStorage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/xpSwiki/Comanche__XPSwikiStorage.st,v 1.2 2004/03/17 18:31:09 penk Exp $'
! !
