"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

Object subclass:#AbstractElement
	instanceVariableNames:'parent children attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-ModelTree'
!

!AbstractElement class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!AbstractElement class methodsFor:'instance creation'!

newTag:aString
    "Build a tag for a tag name. If the tag name is fitting to an
     existing tag use this tag. If not build a generic tag holding
     this new tag name."

    |tagClass element|

    tagClass := self tagToClassMapping at:aString asLowercase ifAbsent:nil.
    tagClass notNil ifTrue:[
        ^ tagClass new
    ].

    element := Element new.
    element tagName:aString.

    ^ element

    "
     Element newTag:'a'
     Element newTag:'span'
    "

    "Modified: / 27-07-2006 / 09:29:09 / cg"
!

tagToClassMapping
    "return a mapping for all existing tag names.
     The key is the tag name, the value is a
     concrete element class."

    self subclassResponsibility
!

with:aRawObject
    "Every HTML element can be created upon a collection of children or 
     simply on a single element to be the child(ren) of this html-element.

     It is allowed to create empty html-objects with '#new' too."

    |instance children|

    aRawObject class == self ifTrue:[^ aRawObject].

    instance := self new.

    aRawObject isNil ifTrue:[^ instance].

    children := aRawObject isString ifTrue:[
                    OrderedCollection with:aRawObject 
                ] ifFalse:[
                    aRawObject isCollection
                        ifTrue: [aRawObject]
                        ifFalse:[OrderedCollection with:aRawObject].
                ].

    instance setChildren:children.

    ^ instance.
!

with:anObject class:aHTMLClass
    "Create an element with the information from an object(See '#with:').
     Set a html stylesheet class to use for this element and automatically
     all included elements."

    |instance|

    instance := self with:anObject.
    instance class:aHTMLClass.

    ^ instance.
!

withNew:aOneArgBlock
    | instance |

    instance := self new.
    aOneArgBlock value:instance.
    ^ instance
! !

!AbstractElement class methodsFor:'accessing'!

allConcreteSubclasses
    "Return all concrete subclasses."

    self auloadAllSubclasses.
    ^ self allSubclasses reject:[:class|class isAbstract].

    "
     self allConcreteSubclasses.
    "
!

allElementImplementingClasses
    "Return all classes that implement a concrete tag"

    self auloadAllSubclasses.
    ^ self allSubclasses 
        select:[:class|
            class isElementImplementing 
        ].
        
    "
     self allElementImplementingClasses
    "
!

auloadAllSubclasses
    Smalltalk allClassesInPackage:self package do:[:cls | cls autoload].
! !

!AbstractElement class methodsFor:'queries'!

isAbstract
    "Return if this class is an abstract class.
     Return true for this class but false for all subclasses.
     To have an abstract subclass implement this method there again."

    ^ self == AbstractElement.

    "
     AbstractElement isAbstract.
     Table isAbstract.
    "
! !

!AbstractElement class methodsFor:'required protocol'!

tagName
    "Return a tag name for the html object.
     Each html object needs to define a tag name."

    self subclassResponsibility
! !

!AbstractElement methodsFor:'accessing'!

add:aChildOrClass
    "Add a new element to the children.
     Use this method as interface to add children.
     Do not add children directly into the children collection."

    ^ self addElement:aChildOrClass
!

addElement:aChildOrClass
    "Add a new element to the children.
     Use this method as interface to add children.
     Do not add children directly into the children collection."

    |child|

    child := self buildElement:aChildOrClass.
    self addChild:child.
    ^ child.

    "Modified: / 06-02-2007 / 19:47:33 / cg"
!

addElement:aChildOrClass at:anIndex
    "Add an element of type a class or a child directly.
     Fill up the indices upd to an index with empty elements
     of the same type."

    |child lastIndex|

    child := self buildElement:aChildOrClass.

    lastIndex := children size.

    lastIndex < anIndex ifFalse:[
        self childAt:anIndex put:child.
        ^ self.
    ].

    "Fill with empty elements of same type."
    (anIndex - lastIndex - 1) timesRepeat:[
        self addElement:child class.
    ].

    ^ self addChild:child.
!

addElement:aChildOrClass before:anIndex
    "Add an element of type a class or a child directly before anIndex. If 
     index greater then children size, the element will be added at the end"

    |child lastIndex|

    child := self buildElement:aChildOrClass.

    child isString ifFalse:[
        child parent:self.
    ].

    lastIndex := children size.

    lastIndex < anIndex ifFalse:[
        self children add:child beforeIndex:anIndex.
        ^ child.
    ].

    self children addLast:child.

    ^ child.
!

addElements:aCollection
    "Add new elements to the children.
     Use this method as interface to add children.
     Do not add children directly into the children collection."

    aCollection do:[:aChildOrClass|
        self addElement:aChildOrClass
    ].
!

addFirst:aChildOrClass
    "Add a new element to the children.
     Use this method as interface to add children.
     Do not add children directly into the children collection."

    |child|

    child := self buildElement:aChildOrClass.

    ^ self addFirstChild:child
!

attributes
    "Return a dictionary with html-attributes
     or nil if there are no attributes set.

     The attributes are included in the opening tag.
     An attribute is a key value pair.
    "

    ^ attributes.
!

parent
    "Return the parent of this element.
     Can be nil for top level elements.
     On correct html a page will be the
     top level item."

    ^ parent.
!

removeAllElements
    ^ self children removeAll.
!

removeFirst
    ^ self children removeFirst
!

tagName
    "Return the tag name of this element.
     Each element must define its own tag 
     name in class method."

    ^ self class tagName.
!

tagName:aStringOrSymbol
    "Here, assert that the given tag conforms to my
     class-given default tagName - i.e. fail, if there is a mismatch."

    self assert:(self tagName = aStringOrSymbol)
! !

!AbstractElement methodsFor:'enumeration'!

allSubElementsDo:aOneArgumentBlock
    "Evaluate a block recursively for all children"

    self subElementsDo:[:child |
        aOneArgumentBlock value:child.
        child isString ifFalse:[ child allSubElementsDo:aOneArgumentBlock ]
    ]

    "Created: / 08-11-2007 / 14:24:16 / cg"
!

subElementsDo:aOneArgumentBlock
    "Evaluate a block for my direct children.
     Wrapper for the bad naming..."

    ^ self subelementsDo:aOneArgumentBlock

    "Modified: / 08-11-2007 / 14:22:22 / cg"
!

subelementsDo:aOneArgumentBlock
    "Evaluate a block for all components wrapped by this element.
     Default is to evaluate this block for all children.
     Subclasses may add special components to be evaluated here.
     Bad name; should be named subElementsDo:"

    children notNil ifTrue:[
        children do:aOneArgumentBlock.
    ]

    "Modified: / 08-11-2007 / 14:22:34 / cg"
! !

!AbstractElement methodsFor:'interface-attributes'!

id
    "Return the id of the element."

    ^ self attributeAt:#id.
!

id:anID
    "Set the id of the element."

    self attributeAt:#id put:anID.
! !

!AbstractElement methodsFor:'private-accessing'!

addChild:aChild
    "Add a new child to the children.

     Use this method as interface to add children.
     Do not add children directly into the children collection."

    (aChild notNil and:[aChild isString not]) ifTrue:[
        aChild parent:self.
    ].

    self children add:(aChild ? '').
    ^ aChild.
!

addElementOfClass:anElementOrStringOrCollection with:aClass 
    "Add an elment. this can be a ready build elment
     or a string or a collection of elements."
    
    |element|

    element := aClass with:anElementOrStringOrCollection.
    ^ self addChild:element.

    "Modified: / 13-01-2008 / 09:51:18 / cg"
!

addFirstChild:aChild
    "Add a new child to the children.

     Use this method as interface to add children.
     Do not add children directly into the children collection."

    aChild isString ifFalse:[
        aChild parent:self.
    ].

    ^ self children addFirst:aChild.
!

childAt:anIndex ifAbsent:anExceptionBlock
    "Return the child at an index or nil if not exists."

    ^ self children at:anIndex ifAbsent:anExceptionBlock.
!

childAt:anIndex put:aNewChildOrClass
    "Set a child to be at an index."

    |child|

    child := self buildElement:aNewChildOrClass.

    child isString ifFalse:[
        child parent:self.
    ].

    self children at:anIndex put:child.
!

children
    "Return the children of this element."

    children isNil ifTrue:[
        children := OrderedCollection new.
    ].

    ^ children.
!

getElement:aClass
    "Return the first element of a class that can
     be found, or build a new element if none is
     present.
     Self children can build a children collection, 
     this collection will be needed anyhow to hold 
     the searched element if not present and build."

    ^ self children
        detect:[:item|item class == aClass] 
        ifNone:[self addElement:aClass].
!

parent:anItem
    "Set an item to be the parent of this item.
     Private - Use only to set child elements.
               Do not call from other classes."

    parent := anItem.
!

replaceChild:anUnlovedChild with:adoptedChild
    (adoptedChild notNil and:[adoptedChild isString not]) ifTrue:[
        adoptedChild parent:self.
    ].
    self children replace:anUnlovedChild with:adoptedChild.
!

setChildren:aCollectionOfElementsOrStrings
    "Set a collection of elements or strings 
     to be the children of this element.
     Private - Use only for instance creation."

    children := aCollectionOfElementsOrStrings.

    children do:[:eachChild|
        eachChild isString ifFalse:[
            eachChild parent:self.
        ].
    ].

    "Modified: / 17-04-2007 / 13:31:52 / cg"
!

style
    self subclassResponsibility

    "Created: / 17-04-2007 / 13:32:52 / cg"
!

style:aStyle
    self subclassResponsibility

    "Created: / 17-04-2007 / 13:33:32 / cg"
! !

!AbstractElement methodsFor:'private-accessing-attributes'!

addAttribute:aSymbol
    "Add an attribute without value"

    self getAttributes at:aSymbol put:nil.
!

addStyle:aStyleElement
    "Add a css-style-element to the attributes.
     Private interface used for accessing style methods."

    |style|

    aStyleElement isNil ifTrue:[^ self].

    (aStyleElement isOrderedCollection and:[aStyleElement isString not]) ifTrue:[
        aStyleElement do:[:substyle|
            self addStyle:substyle.
        ].
        ^ self.
    ].

    style := self style.
    style isNil ifTrue:[
        self style:aStyleElement.
    ] ifFalse:[
        (style isCollection and:[style isString not]) ifTrue:[
            style add:aStyleElement.
        ] ifFalse:[
            self style:(OrderedCollection with:style with:aStyleElement).
        ].
    ].

    "Modified: / 17-04-2007 / 13:34:35 / cg"
!

attributeAt:aSymbol
    "Return the attribute at a symbol
     or nil if the attribute is not set."

    |attrDict attrVal lcSymbol ucSymbol|

    attrDict := self attributes.
    attrDict isNil ifTrue:[^ nil].

    attrVal := attrDict at:aSymbol ifAbsent:nil.
    attrVal isNil ifTrue:[
        lcSymbol := aSymbol asLowercase asSymbolIfInterned.
        (lcSymbol notNil and:[lcSymbol ~~ aSymbol]) ifTrue:[
            attrVal := attrDict at:lcSymbol ifAbsent:nil.
        ].
    ].
    attrVal isNil ifTrue:[
        ucSymbol := aSymbol asUppercase asSymbolIfInterned.
        (ucSymbol notNil and:[ucSymbol ~~ aSymbol]) ifTrue:[
            attrVal := attrDict at:ucSymbol ifAbsent:nil.
        ].
    ].
    ^ attrVal
!

attributeAt:aSymbol add:more
    "Set or add an attribute value to the attribute."

    |attrs oldValue newValue sep|

    more notEmptyOrNil ifTrue:[
        attrs := self getAttributes.
        oldValue := attrs at:aSymbol ifAbsent:''.
        oldValue isEmpty ifTrue:[
            sep := ''.
        ] ifFalse:[
            sep := ' '.
            (oldValue withoutTrailingSeparators endsWith:$;) ifFalse:[
                oldValue := oldValue , ';'
            ].
        ].
        newValue := oldValue,sep,more.
        (more withoutTrailingSeparators endsWith:$;) ifFalse:[
            newValue := newValue,';'.
        ].
        attrs at:aSymbol put:newValue.
    ].
!

attributeAt:aSymbol put:aValue
    "Set an attribute value to the attribute.
     If a value is nil the attribute is removed."

    aValue isNil ifTrue:[self removeAttribute:aSymbol].
    self getAttributes at:aSymbol put:aValue.
!

elementAtId:anId
    "Return the element at an ID
     or nil if the attribute is not set."

    |attribute|

    attribute := self attributeAt:#id.
    attribute notNil ifTrue:[
        (attribute = anId) ifTrue:[
            ^ self
        ].
    ].

    self children do:[:aChild |        
        (aChild isKindOf: HTML::AbstractElement) ifTrue:[ |element|
            element := aChild elementAtId:anId.
            element notNil ifTrue:[
                ^ element
            ].
        ].
    ].

    ^ nil
!

getAttributes
    "Return a dictionary to put in attributes.
     Despite normal access method: '#attributes'
     this method garanties to deliver a 
     dictionary for the attributes.

     Call only if you want to set attributes.
     Do not call for checking for attributes.
     Use method: '#attributes' for this instead."

     attributes isNil ifTrue:[
         attributes := Dictionary new.
     ].

     ^ attributes.
!

removeAttribute:aSymbol
    "Remove a symbol from the attribute list
     and check if we need the attributes dictionary
     any more after removing."

    |attributes|

    attributes := self attributes.
    attributes isNil ifTrue:[^ self].

    attributes removeKey:aSymbol ifAbsent:[].

    attributes isEmpty ifTrue:[
        attributes := nil.
    ].
! !

!AbstractElement methodsFor:'private-building'!

buildElement:aNewChildOrClass
    "Take the item or instanciate the class."

    ^ aNewChildOrClass isClass 
        ifTrue: [aNewChildOrClass new]
        ifFalse:[aNewChildOrClass].
!

buildElement:aRawData ofType:aClass
    "Build a new element created from raw data.
     Raw data can be either a ready built subelement of the
     fitting class, or it can be a simple string that will be
     wrapped by a new element of the given class. Or it can be nil
     to construct an empty element of the given class."

    ^ self buildElement:aRawData ofType:aClass onCollectionDo:nil "/ [:a :b|]
!

buildElement:rawData ofType:aClass onCollectionDo:aTwoArgumentBLock
    "Build a new element created from raw data.
     Raw data can be either a ready built subelement of the
     fitting class, or it can be a simple string that will be
     wrapped by a new element of the given class. Or it can be nil
     to construct an empty element of the given class. It is
     also possible to give a collection of subelements the
     added element shall consist of, then a block is executed
     for each of these subelements to construct it properly
     into the element.

     See method: '#addRow' in class Table for usage."

    "Raw data is nil"
    rawData isEmptyOrNil ifTrue:[^ aClass new].     


    "Raw data is of fitting type."
    rawData class == aClass ifTrue:[^ rawData].  

    "Raw data is foreign element, useable!!"
    (rawData askFor:#isElement) ifTrue:[^ aClass with:rawData].


    "Raw data is a real collection."
    (rawData isCollection and:[rawData isString not])  
        ifTrue:[
            |element|

            aTwoArgumentBLock isNil ifTrue:[^ rawData].

            element := aClass new.

            rawData do:[
                :rawData|
                aTwoArgumentBLock value:element value:rawData. 
            ].

            ^ element.
        ].


    "Raw data is a string, integer, ..."
    ^ aClass with:rawData printString.
! !

!AbstractElement methodsFor:'queries'!

canHaveChild:anotherElement
    "return true, if anotherElement is allowed as my child"

    Smalltalk isStandAloneApp ifFalse:[
        self subclassResponsibility.
    ].
    ^ true

    "Created: / 06-02-2007 / 13:52:38 / cg"
    "Modified: / 02-11-2007 / 16:00:23 / cg"
!

canHaveText
    "return true, if I allow text as child (i.f. PCDATA)"

    ^ true

    "Created: / 06-02-2007 / 19:54:18 / cg"
!

hasChildren
"
     summary:
        This may look to you like a terrible idea. You may be also be right.
        However, there is a reason for it. There are cases in the HTML tree where
        children are not in the children instance variable. For example, In <Head> 
        there is a child that is not in the collecion children called title.

        This makes:

                children size ~= 0

        not work. So you may rightly argue that we should define hasChildren in each subclass
        that has children that are not in the instance variable children. Then we have two methods
        that define what an element's children are (a possible reciepe for disaster). So
        with this in mind, the following implementation was born.
"

    self subelementsDo:[:anElement |
        ^ true
    ].
    ^ false
!

mustBeClosed
    "return true, if I need a closing tag (as opposed to br or img, which do not)"

    ^ true

    "Created: / 06-02-2007 / 19:54:18 / cg"
! !

!AbstractElement methodsFor:'required-visiting'!

acceptHTMLVisitor:aVisitor
    "Visit this element.
     To be implemented by each element(subclass)."

    ^ self subclassResponsibility.
! !

!AbstractElement methodsFor:'testing'!

isAnchor
    ^ false.
!

isComment
    ^ false

    "Created: / 17-04-2007 / 13:40:44 / cg"
!

isDummyContainer
    ^ false

    "Created: / 03-07-2009 / 15:16:11 / sr"
!

isProcessingInstruction
    ^ false

    "Created: / 17-04-2007 / 13:40:49 / cg"
! !

!AbstractElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__AbstractElement.st,v 1.97 2011/03/18 16:50:58 az Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__AbstractElement.st,v 1.97 2011/03/18 16:50:58 az Exp $'
! !
