"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

AbstractElement subclass:#AbstractHTMLElement
	instanceVariableNames:''
	classVariableNames:'TagToClassMapping'
	poolDictionaries:''
	category:'Net-Documents-ModelTree'
!

!AbstractHTMLElement class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!AbstractHTMLElement class methodsFor:'instance creation'!

tagToClassMapping
    "return a mapping for all existing tag names.
     The key is the tag name, the value is a
     concrete element class."

    |mapping|

    TagToClassMapping isNil ifTrue:[
        mapping := Dictionary new.

        self
            allElementImplementingClasses do:[:eachClass|
                mapping
                    at: eachClass tagName
                    put:eachClass.
            ].
        TagToClassMapping := mapping.
    ].
    ^ TagToClassMapping

    "
     TagToClassMapping := nil.

     Element newTag:'a'
     Element newTag:'span'
    "

    "Created: / 27-07-2006 / 09:28:59 / cg"
! !

!AbstractHTMLElement class methodsFor:'queries'!

isAbstract
    "Return if this class is an abstract class.
     Return true for this class but false for all subclasses.
     To have an abstract subclass implement this method there again."

    ^ self == AbstractHTMLElement.

    "
     AbstractElement isAbstract.
     Table isAbstract.
    "
!

isElement
    "Say that you are an element.
     Attention this method is not 
     implemented in Object, like 
     on should think!!
     Use method:'#askFor:#isElement' 
     and forget about this return 
     value here."

    ^ true.
!

isElementImplementing
    "Return true if the class is implementing a concrete tag."

    self isAbstract ifTrue:[^ false].
    self isGeneric  ifTrue:[^ false].

    ^ true.

    "
     AbstractElement isElementImplementing
     Element         isElementImplementing
     Table           isElementImplementing
    "
!

isGeneric
    "Return false here.
     The Element will return true."

    ^ false.
! !

!AbstractHTMLElement methodsFor:'accessing-style-background-color'!

backgroundColor:aRawColor
    "Set background-color using style-sheets(CSS)."

    |rgbColorString color|

    rgbColorString := aRawColor isColor 
                          ifTrue: [aRawColor rgbValue hexPrintString]
                          ifFalse:[aRawColor].

    color := CSS_BackgroundColor with:rgbColorString.

    self addStyle:color.
! !

!AbstractHTMLElement methodsFor:'accessing-style-border'!

borderStyle:aStyle
    "Set border-width using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle with:aStyle.

    self addStyle:style.
!

borderStyleDashed               
    "Set border-width to dashed using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newDashed.

    self addStyle:style.
!

borderStyleDotted               
    "Set border-style to dotted using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newDotted.

    self addStyle:style.
!

borderStyleDouble               
    "Set border-width to double using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newDouble.

    self addStyle:style.
!

borderStyleGroove
    "Set border-style to groove using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newGroove.

    self addStyle:style.
!

borderStyleInset
    "Set border-style to inset using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newInset.

    self addStyle:style.
!

borderStyleNone
    "Set border-style to none using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newNone.

    self addStyle:style.
!

borderStyleOutset
    "Set border-style to outset using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newOutset.

    self addStyle:style.
!

borderStyleRidge
    "Set border-style to ridge using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newRidge.

    self addStyle:style.
!

borderStyleSolid
    "Set border-style to solid using style-sheets(CSS)."

    |style|

    style := CSS_BorderStyle newSolid.

    self addStyle:style.
!

borderWidth:aRawWidth
    "Set border-width using style-sheets(CSS)."

    |widthString width|

    widthString := aRawWidth isString 
                       ifTrue: [aRawWidth]
                       ifFalse:[
                           aRawWidth isNumber 
                               ifTrue: [aRawWidth printString, 'px']
                               ifFalse:[aRawWidth printString]
                       ].

    width := CSS_BorderWidth with:widthString.

    self addStyle:width.
!

margin:aDistance
    "Set margin for all four sides using style-sheets(CSS)."

    |margin|

    margin := CSS_Margin with:aDistance.

    self addStyle:margin.
!

padding:aDistance
    "Set padding for all four sides using style-sheets(CSS)."

    |padding|

    padding := CSS_Padding with:aDistance.

    self addStyle:padding.
! !

!AbstractHTMLElement methodsFor:'accessing-style-border-color'!

borderColor:aRawColor
    "Set color using style-sheets(CSS)."

    |rgbColorString color|

    rgbColorString := aRawColor isColor 
                          ifTrue: [aRawColor rgbValue hexPrintString]
                          ifFalse:[aRawColor].

    color := CSS_BorderColor with:rgbColorString.

    self addStyle:color.
!

borderColorAqua
    "Set border-color to aqua using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newAqua.

    self addStyle:style.
!

borderColorBlack
    "Set border-color to black using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newAqua.

    self addStyle:style.
!

borderColorBlue
    "Set border-color to blue using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newBlue.

    self addStyle:style.
!

borderColorFuchsia
    "Set border-color to fuchsia using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newFuchsia.

    self addStyle:style.
!

borderColorGray
    "Set border-color to gray using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newGray.

    self addStyle:style.
!

borderColorGreen
    "Set border-color to green using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newGreen.

    self addStyle:style.
!

borderColorLime
    "Set border-color to lime using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newLime.

    self addStyle:style.
!

borderColorMaroon
    "Set border-color to maroon using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newMaroon.

    self addStyle:style.
!

borderColorNavy
    "Set border-color to navy using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newNavy.

    self addStyle:style.
!

borderColorOlive
    "Set border-color to olive using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newOlive.

    self addStyle:style.
!

borderColorPurple
    "Set border-color to purple using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newPurple.

    self addStyle:style.
!

borderColorRed
    "Set border-color to red using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newRed.

    self addStyle:style.
!

borderColorSilver
    "Set border-color to silver using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newSilver.

    self addStyle:style.
!

borderColorTeal
    "Set border-color to teal using style-sheets(CSS)."

    |style|

    style := CSS_BorderColor newTeal.

    self addStyle:style.
! !

!AbstractHTMLElement methodsFor:'accessing-style-position'!

atPosition:aPosition
    "Set position using style-sheets(CSS)."

    |position|

    position := OrderedCollection
                    with:CSS_Position newAbsolute
                    with:(CSS_Left with:aPosition x printString)
                    with:(CSS_Top  with:aPosition y printString).

    self addStyle:position.
!

bottom:aBottom
    "Set bottom using style-sheets(CSS)."

    |bottom|

    bottom := CSS_Bottom with:aBottom.

    self addStyle:bottom.
!

bounds:aRectangle
    "Set bounds using style-sheets(CSS)."

    |position|

    position := OrderedCollection
                    with:CSS_Position newAbsolute
                    with:(CSS_Top    with:aRectangle top    printString)
                    with:(CSS_Left   with:aRectangle left   printString)
                    with:(CSS_Width  with:aRectangle width  printString)
                    with:(CSS_Height with:aRectangle height printString).

    self addStyle:position.
!

document
    self class == HTML::Document ifFalse:[
        ^ parent document
    ].

    ^ self
   .
!

extent:aPosition
    "Set extent using style-sheets(CSS)."

    |extent|

    extent := OrderedCollection
                  with:CSS_Position newAbsolute
                  with:(CSS_Width  with:aPosition width  printString)
                  with:(CSS_Height with:aPosition height printString).

    self addStyle:extent.
!

height:aNumberOrPercentString
    "Set height using style-sheets(CSS)."

    |height|

    height := CSS_Height with:aNumberOrPercentString.

    self addStyle:height.
!

left:aLeft
    "Set left using style-sheets(CSS)."

    |left|

    left := CSS_Left with:aLeft.

    self addStyle:left.
!

right:aRight
    "Set right using style-sheets(CSS)."

    |right|

    right := CSS_Right with:aRight.

    self addStyle:right.
!

top:aTop
    "Set top using style-sheets(CSS)."

    |top|

    top := CSS_Top with:aTop.

    self addStyle:top.
!

width:aNumberOrPercentString
    "Set width using style-sheets(CSS)."

    |width|

    width := CSS_Width with:aNumberOrPercentString.

    self addStyle:width.
! !

!AbstractHTMLElement methodsFor:'interface-accessing-elements'!

addAnchor
    "Add an anchor."

    ^ self addAnchorHRef:nil.
!

addAnchorHRef:anURLString
    "Add an anchor."

    ^ self addAnchorHRef:anURLString with:nil
!

addAnchorHRef:anURLString with:aElementOrString
    "Add an anchor."

    |anchor|

    anchor := Anchor href:anURLString with:aElementOrString.

    ^ self addChild:anchor.
!

addBig:anELementOrStringOtCollection 
    "Add a big."
    
    ^ self addElementOfClass:anELementOrStringOtCollection with:Big
!

addBold:anELementOrStringOtCollection 
    "Add a bold."
    
    ^ self addElementOfClass:anELementOrStringOtCollection with:Bold
!

addBreak
    "Add a line break to the children."

    ^ self addElement:Break.
!

addDiv
    "Add an empty div."
    
    ^ self addDiv:nil.
!

addDiv:anElementOrStringOrCollection 
    "Add a div."
    
    ^ self addElementOfClass:anElementOrStringOrCollection with:Div

    "Modified: / 13-01-2008 / 09:50:52 / cg"
!

addItalic:anELementOrStringOtCollection 
    "Add an italic."
    
    ^ self addElementOfClass:anELementOrStringOtCollection with:Italic
!

addLineBreak
    "Add a line break to the children."

    ^ self addElement:Break.
!

addScriptType:language code:theCode 
    |script|

    script := Script new type:language.
    script add:theCode.

    ^ self addElement:script.

    "Modified: / 13-01-2008 / 09:52:12 / cg"
!

addSmall:anELementOrStringOtCollection 
    "Add an small."
    
    ^ self addElementOfClass:anELementOrStringOtCollection with:Small
!

addSpan:anELementOrStringOtCollection 
    "Add a span."
    
    ^ self addElementOfClass:anELementOrStringOtCollection with:Span
!

addUnderlined:anElementOrStringOrCollection 
    "Add an underlined."
    
    ^ self addElementOfClass:anElementOrStringOrCollection with:Underlined
! !

!AbstractHTMLElement methodsFor:'interface-attributes'!

bgColor
    "Return the background color of the element."

    ^ self attributeAt:#bgColor.
!

bgColor:aColorOrRGBColorString
    "Set the background color of the element.
     This can be either an rgb-color string or a smalltalk color."

    |rgbColorString|

    rgbColorString := aColorOrRGBColorString isColor 
                          ifTrue: [aColorOrRGBColorString rgbValue hexPrintString]
                          ifFalse:[aColorOrRGBColorString].

    self attributeAt:#bgColor put:rgbColorString.
!

border
    "Return the border of the element."

    ^ self attributeAt:#border.
!

border:aWidth
    "Set the border of the element."

    self attributeAt:#border put:aWidth.
!

class
    "OUCH: class is a bad name!!
     Return the css-class of the element."

self halt.
    ^ self attributeAt:#class.

    "Modified: / 12-04-2011 / 19:30:20 / cg"
!

class:classString
    "Set the css-class of the element."

    self assert:(classString isString).
    self attributeAt:#class put:classString.
!

cssClass
    "Return the css-class of the element."

    ^ self attributeAt:#class.

    "Created: / 12-04-2011 / 19:29:58 / cg"
!

name
    "Return the name of the frame."

    ^ self attributeAt:#name.
!

name:aName
    "Set the name of the frame."

    self attributeAt:#name put:aName.
!

on:what script:aScript     
    "common helper for onXXX: messages;
     looks if there is already a script defined for the what-message, and concatenates the scripts,
     if required"

    |oldScript|

    (oldScript := self attributeAt:what) notEmptyOrNil ifTrue:[
        (oldScript endsWith:';') ifFalse:[
            oldScript := oldScript , '; '
        ].
        self attributeAt:what put:(oldScript,aScript).
    ] ifFalse:[
        self attributeAt:what put:aScript.
    ].

    "Created: / 19-07-2010 / 12:17:08 / cg"
!

onChange:aScript
    "Set the on-change-attribute of this element."

    self on:#onchange script:aScript

    "Modified: / 19-07-2010 / 12:17:20 / cg"
!

onClick
    ^ self attributeAt:#onclick.
!

onClick:aScript     
    self on:#onclick script:aScript

    "Modified: / 19-07-2010 / 12:17:32 / cg"
!

onDoubleClick:aScript
    self on:#ondblclick script:aScript

    "Modified: / 19-07-2010 / 12:17:37 / cg"
!

onKeyPress:aScript 
    self on:#onkeypress script:aScript

    "Modified: / 19-07-2010 / 12:17:48 / cg"
!

onKeyUp:aScript 
    self on:#onkeyup script:aScript

    "Modified: / 19-07-2010 / 12:17:57 / cg"
!

onMouseDown
    ^ self attributeAt:#onMouseDown.
!

onMouseDown:aScript
    self on:#onMouseDown script:aScript

    "Modified: / 19-07-2010 / 12:18:08 / cg"
!

onMouseMove:aScript
    self on:#onmousemove script:aScript

    "Modified: / 19-07-2010 / 12:18:14 / cg"
!

onMouseOut:aScript
    self on:#onmouseout script:aScript

    "Modified: / 19-07-2010 / 12:18:24 / cg"
!

onMouseOver
    ^ self attributeAt:#onmouseover.
!

onMouseOver:aScript
    self on:#onmouseover script:aScript

    "Modified: / 19-07-2010 / 12:18:32 / cg"
!

onMouseUp
    self attributeAt:#onmouseup
!

onMouseUp:aScript
    self on:#onmouseup script:aScript

    "Modified: / 19-07-2010 / 12:18:38 / cg"
!

style
    "Return the style of the element."

    ^ self attributeAt:#style.
!

style:aStyle
    "Set the style of the element."

    self attributeAt:#style put:aStyle.
!

text
    "Return the text of the element."

    ^ self attributeAt:#text.
!

text:aText
    "Set the text of the element."

    self attributeAt:#text put:aText.
!

title:aText
    "Set the title of the element."

    self attributeAt:#title put:aText.

    "Modified: / 05-12-2006 / 13:40:40 / cg"
!

valign
    "Return the valign of the table."

    ^ self attributeAt:#valign.
!

valign:align
    "Set the vertical align of the table/cell (top|middle|bottom|baseline)."

    self attributeAt:#valign put:align.
! !

!AbstractHTMLElement methodsFor:'interface-attributes-alignment'!

align:alignmentString
    "Set the alignment of the element.
     The alignment can be left, center, right."

    self attributeAt:#align put:alignmentString.
!

alignCenter
    "Set alignment to be center."

    self align:#center.
!

alignLeft
    "Set alignment to be left."

    self align:#left.
!

alignRight
    "Set alignment to be right."

    self align:#right.
!

alignment
    "Return the alignment of the table.
     The alignment can be left, center, right
     or nil  in case no alignment is set so far."

    ^ attributes attributeAt:#align.
!

alignment:alignmentString
    "Set the alignment of the table.
     The alignment can be left, center, right."

    self align:alignmentString.
! !

!AbstractHTMLElement methodsFor:'printing'!

htmlString
    "Print yourself as html string.
     See class Encoder and Visitor."

    ^ self htmlStringUsingEncoder:Encoder new
!

htmlStringOfSubElements
    "Print my children as html string.
     See class Encoder and Visitor."

    |encoder |

    encoder := Encoder new. 
    encoder visitSubelementsOf:self indent:false.
    ^ encoder htmlString.

    "Created: / 02-07-2009 / 19:39:28 / sr"
!

htmlStringUsingEncoder:anEncoder
    "Print yourself as html string.
     See class Encoder and Visitor."

    anEncoder visit:self.
    ^ anEncoder htmlString.
!

printHtmlOn:aStream
    "Print yourself as html string.
     See class Encoder and Visitor."

    |encoder|

    encoder := Encoder new. 
    encoder stream:aStream.    
    encoder visit:self.
!

printHtmlString
    "Print yourself as html string.
     See class Encoder and Visitor."

    <resource: #obsolete>
    self obsoleteMethodWarning:'use #htmlString'.
    ^ self htmlString.
!

printOn:aStream
    self printHtmlOn:aStream
! !

!AbstractHTMLElement methodsFor:'queries'!

mustBeClosed
    "return true, if I need a closing tag (as opposed to br or img, which do not)"

    ^ (self tagName sameAs:'area') not.

    "Created: / 06-02-2007 / 19:54:18 / cg"
! !

!AbstractHTMLElement methodsFor:'testing'!

canBeReformatted
   "answer true, if the text may be reformatted by the browser,
    and added white space does not matter.
    Redefined in subclasses."

   ^ true     
!

isArea
    ^ false.
!

isBlockElement
    ^ false

    "Created: / 06-02-2007 / 15:39:27 / cg"
!

isBody
    ^ false.
!

isBold
    ^ false.
!

isButton
    ^ false.
!

isCenter
    ^ false.
!

isCode
    ^ false.
!

isColgroup
    ^ false
!

isDefinitionDescription
    ^ false
!

isDefinitionTerm
    ^ false
!

isDiv
    ^ false.
!

isElement
    "See class-method:'#isElement'"

    ^ self class isElement.
!

isForm
    ^ false.
!

isFormWidgetElement
    ^ false.
!

isFrame
    ^ false
!

isHead
    ^ false.
!

isHeading
    ^ false.
!

isHeading1
    ^ false.
!

isHorizontalRule
    ^ false.
!

isImg
    ^ false.
!

isInlineElement
    (
        #(
            'abbr'
            'acronym'
        ) includes:self tagName asLowercase) ifTrue:[^ true].

    ^ false

    "Created: / 06-02-2007 / 17:33:33 / cg"
!

isInput
    ^ false.
!

isLabel
    ^ false.
!

isLink
    ^ false.
!

isListItem
    ^ false.

    "Created: / 06-02-2007 / 13:55:21 / cg"
!

isMeta
    ^ false.
!

isNoBr
    ^ false

    "Created: / 29-11-2011 / 12:17:57 / sr"
!

isOption
    ^ false
!

isOrderedList
    ^ false.
!

isParagraph
    ^ false.
!

isPre
    ^ false.
!

isScript
    ^ false

    "Created: / 06-02-2007 / 19:00:57 / cg"
!

isSelect
    ^ false
!

isSpan
    ^ false.
!

isStrong
    ^ false.
!

isTable
    ^ false.
!

isTableBody
    ^ false.
!

isTableCell
    ^ false.
!

isTableDataCell
    ^ false.
!

isTableFoot
    ^ false.
!

isTableHead
    ^ false.
!

isTableHeaderCell
    ^ false.
!

isTableRow
    ^ false.
!

isTextArea
    ^ false
!

isTitle
    ^ false.
!

isUnorderedList
    ^ false.
!

printInlineWithSimpleElements
    "helper for printing; controls if begin- and end-tags are generated into extra lines"

    ^ self isInlineElement
! !

!AbstractHTMLElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__AbstractHTMLElement.st,v 1.24 2011/11/29 11:27:01 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__AbstractHTMLElement.st,v 1.24 2011/11/29 11:27:01 sr Exp $'
! !
