"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

AbstractHTMLElement subclass:#Document
	instanceVariableNames:'frameset docType htmlVersion webAddress languageVariant
		docTypeString url'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-ModelTree-TopLevel'
!

!Document class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

examples
"
    Example document with some headings to show something:
    (A document itself would not show something. All 
    necessary top-level parts get built automatically.)
                                                                                        [exBegin]
        |document|

        document := Document new.

        document title:        'This is a generated title'.

        document addHeading1:  'This is a generated level one heading'.
        document addHeading2:  'This is a generated level two heading'.
        document addHeading3:  'This is a generated level three heading'.
        document addHeading4:  'This is a generated level four heading'.
        document addHeading5:  'This is a generated level five heading'.
        document addHeading6:  'This is a generated level six heading'.

        document addParagraph: 'This is a generated paragraph
                            It contains text and text and text.'.

        document addParagraph: 'This is a another generated paragraph.'.

        document addHorizontalRule.
        document htmlString.
                                                                                        [exEnd]



    A document with document type:
                                                                                        [exBegin]
        |document|

        document := Document newStrict.
        document htmlString.
                                                                                        [exEnd]



    A totaly empty document:
                                                                                        [exBegin]
        |document|

        document := Document new.
        document htmlString.
                                                                                        [exEnd]


    Document with a title:
    (The necessary head gets generated automatically)
                                                                                        [exBegin]
        |document|

        document := Document new.
        document title:'This is a generated title'.
        document htmlString.
                                                                                        [exEnd]
"
! !

!Document class methodsFor:'instance creation'!

newFrameset
    "Create a new document of type HTML 4.01 frameset with webAdress http://www.w3.org/TR/html4/frameset.dtd"

    ^ self new docType:'HTML'; htmlVersion:'4.01'; languageVariant:'Frameset'; webAddress:'http://www.w3.org/TR/html4/frameset.dtd'
"
    self newFrameset printHtmlString     
"
!

newStrict
    "Create a new document of type HTML 4.01 strict with webAdress http://www.w3.org/TR/html4/strict.dtd."

    ^ self new 
        docType:'HTML'; 
        htmlVersion:'4.01'; 
        webAddress:'http://www.w3.org/TR/html4/strict.dtd'

    "
     self newStrict printHtmlString     
    "
!

newTransitional
    "Create a new document of type HTML 4.01 Transitional with webAdress http://www.w3.org/TR/html4/loose.dtd"

    ^ self new 
        docType:'HTML'; 
        htmlVersion:'4.01'; 
        languageVariant:'Transitional'; 
        webAddress:'http://www.w3.org/TR/html4/loose.dtd'

    "
     self newTransitional
    "
! !

!Document class methodsFor:'required protocol'!

tagName
    ^ 'html'.
! !

!Document methodsFor:'accessing'!

docType
    "Return document type or nil."

    ^ docType.
!

docType:something
    docType := something.
!

docTypeString
    "Return document type or nil."

    |stream|

    stream := WriteStream on:''.
    stream nextPutAll:'<!!DOCTYPE HTML PUBLIC "-//W3C//DTD '.
    stream nextPutAll:(docType ? 'HTML').
    stream space.
    stream nextPutAll:(htmlVersion ? '4.01').
    languageVariant notNil ifTrue:[
        stream space.
        stream nextPutAll:languageVariant.
    ].
    stream nextPutAll:'//EN"'.
    webAddress notNil ifTrue:[
        stream space.
        stream nextPut:$".
        webAddress printOn:stream. "/ stream nextPutAll:webAddress.
        stream nextPut:$".
    ].
    stream nextPutAll:'>'.
    ^ stream contents.

    "Modified: / 06-02-2007 / 19:50:40 / cg"
!

fileName:aStringOrFileName
    |path|

    path := aStringOrFileName asFilename pathName.
    self webAddress:(URL fromString:path).

    "Created: / 05-02-2007 / 16:19:18 / cg"
!

htmlVersion
    ^ htmlVersion
!

htmlVersion:something
    htmlVersion := something.
!

languageVariant
    ^ languageVariant
!

languageVariant:something
    languageVariant := something.
!

url
    ^ url
!

url:something
    url := something.
!

webAddress
    ^ webAddress
!

webAddress:something
    webAddress := something.
! !

!Document methodsFor:'interface-accessing-elements'!

addHeading1:aHeading1OrString
    "Add a level one heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading1:aHeading1OrString.
!

addHeading2:aHeading2OrString
    "Add a level two heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading2:aHeading2OrString.
!

addHeading3:aHeading3OrString
    "Add a level three heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading3:aHeading3OrString.
!

addHeading4:aHeading4OrString
    "Add a level four heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading4:aHeading4OrString.
!

addHeading5:aHeading5OrString
    "Add a level five heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading5:aHeading5OrString.
!

addHeading6:aHeading6OrString
    "Add a level six heading to the page.

     The heading will be added as last 
     component in the body."

    ^ self getBody addHeading6:aHeading6OrString.
!

addHorizontalRule
    "Add a horizontal rule to the pages body."

    ^ self getBody addHorizontalRule.
!

addParagraph:aParagraphOrString
    "Add a level paragraph to the pages body.

     The heading will be added as last 
     component in the body."

    ^ self getBody addParagraph:aParagraphOrString.
!

addTable
    "Add a level paragraph to the pages body.

     The heading will be added as last 
     component in the body."

    ^ self getBody addTable.
!

body
    "Return the body of the html-document."

    ^ self children detect:[:child | child isBody] ifNone:nil.
!

body:anElement
    self body notNil ifTrue:[
        HTMLStructureError raiseRequestErrorString:'two bodies in a document'.
        ^ self.
    ].
    self addChild:anElement
!

head
    "Return the head of the html-document."

    ^ self children detect:[:child | child isHead] ifNone:nil.
!

head:anElement
    self head notNil ifTrue:[
        HTMLStructureError raiseRequestErrorString:'two heads in a document'.
        ^ self.
    ].
    self addChild:anElement
!

title:aTitleOrString
    "Set a title for this document.
     The title will be displayed in the window title bar.
     The title can be a simple string or a html-title object."

    self getHead title:aTitleOrString.
! !

!Document methodsFor:'private-accessing'!

getBody
    "Return a body to add elements.
     If no body exists, build one."

    |body|

    body := self body.
    body isNil ifTrue:[
        self getHead.   "/ ensure there is a head and it comes BEFORE the body

        body := HTML::Body new.
        self addChild:body.
    ].

    ^ body.
!

getHead
    "Return a head to add elements.
     If no head exists, build one."

    |head|

    head := self head.
    head isNil ifTrue:[
        head := HTML::Head new.
        self addChild:head.
    ].

    ^ head.
! !

!Document methodsFor:'queries'!

canHaveChild:anotherElement
    "return true, if anotherElement is allowed as my child"

    ^ anotherElement isHead or:[anotherElement isBody]
! !

!Document methodsFor:'required-visiting'!

acceptHTMLVisitor:aVisitor
    "visitor pattern: dispatch me to the visitor"

    aVisitor visitDocument:self.

    "Modified: / 05-11-2007 / 22:07:29 / cg"
! !

!Document class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__Document.st,v 1.33 2009/03/23 17:19:05 cg Exp $'
! !
