"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

Visitor subclass:#Encoder
	instanceVariableNames:'stream indent inOneLine lastWasString'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-Utilities'
!

!Encoder class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!Encoder class methodsFor:'instance creation'!

new
    "Create a new encoder. 
     Build the necessary stream."

    ^ self on:(WriteStream on:(String new:1000)).

    "Modified: / 01-07-2010 / 12:36:26 / cg"
!

on:aStream
    "Create an encoder on a stream."

    |instance|

    instance := self basicNew initialize.
    instance setStream:aStream.
    ^ instance.
! !

!Encoder methodsFor:'accessing'!

htmlString
    "Return the html string.
     Call this after doing a '#visit:'."

    ^ stream contents.
!

stream
    "Return the stream the encoder encodes on."

    ^ stream.
!

stream:aStream
    stream := aStream.
! !

!Encoder methodsFor:'initialization'!

initialize
    "Set the indent depth to a default.
     Attention: Direct instance variable 
                access relies on this setting"

    super initialize.

    indent := 0.
    inOneLine := 0.
    lastWasString := false.

    "Modified: / 19-08-2010 / 16:53:07 / sr"
! !

!Encoder methodsFor:'private-accessing'!

setStream:aStream
    "Set a stream as the stream to print on.
     Private - Use only for instance creation."

    stream := aStream.
! !

!Encoder methodsFor:'private-printing'!

attributeValuesPrintString:aRawObject
    "Print an attribute value or a list of attribute values.
     aRawObject can be a string, a style object or a collection of those."

    |s|

    aRawObject isCollection ifTrue:[
        aRawObject isString ifTrue:[
            ^ aRawObject.
        ].
        s := WriteStream on:''.
        aRawObject do:[:subRawObject|
            s nextPutAll:(self attributeValuesPrintString:subRawObject).
        ].
        ^ s contents.
    ].

    (aRawObject class askFor:#isCSS) ifTrue:[
        ^ aRawObject htmlString.
    ].

    "Whatever you are, try to display yourself."
    ^ aRawObject printString.

    "Created: / 06-08-2007 / 00:45:11 / cg"
!

printAttribute:anAssociation
    "Print a single attribute given as an association 
     with key and value onto a stream.

     The value sometimes can be a collection of values."

    |value sep|

    stream nextPut:Character space.
    anAssociation key printOn:stream.

    value := anAssociation value.
    value isNil ifTrue:[^ self].

    value := self attributeValuesPrintString:value.
    (value includes:$") ifTrue:[
        sep := $'.
    ] ifFalse:[
        sep := $".
    ].
    (value includes:sep) ifTrue:[
        self breakPoint:#cg.
        Transcript showCR:'HTML::Encoder [warning]: attributes include both " and '' - encoding " as &quot;'.
        value := value copyReplaceAll:$" withAll:'&quot;'.
        sep := $".
    ].
    value bitsPerCharacter > 8 ifTrue:[
        value := value utf8Encoded.
    ].
    stream 
        nextPut:$=; 
        nextPut:sep;
        nextPutAll:value;
        nextPut:sep.

    "Modified: / 15-07-2009 / 12:20:24 / sr"
    "Modified: / 24-03-2012 / 21:33:08 / cg"
!

printAttributesOf:anElement
    "Print all set attributes of an 
     item onto the stream.

     Attributes can be nil for no 
     attributes which is the default.

     Otherwise will be key value pairs.
     They are representated as 'key=value' in HTML."

    |attributes|

    attributes := anElement attributes.
    attributes isNil ifTrue:[^ self].

    attributes associationsDo:[:association|
        self printAttribute:association.
    ].
!

printNewLineIndent
    "Print a carriage return and an indent 
     of length indent onto the stream,
     but only if there is no inlining.

     New line and indent are always printed
     together. There is no newline without
     indenting and no intenting witout a new
     line. "

    inOneLine > 0 ifTrue:[^ self].

    stream cr.
    indent timesRepeat:[stream nextPutAll:'  ' "tab"]
!

printScriptElement:aScriptElement
    "A script element gets printed."

    |styleEncoder  scripts|

    scripts := aScriptElement children ? Array new.

    styleEncoder := StyleEncoder on:stream.
    styleEncoder indent:indent + 2.

    self printTagsFor:aScriptElement do:[
        indent := indent + 1.
        self printNewLineIndent.
        stream nextPutAll:'<!!--'.
        stream cr.
        scripts do:[:aScript |
            styleEncoder visit:aScript.    
            self printNewLineIndent.
        ].
        self printNewLineIndent.
        stream nextPutAll:'-->'.
        indent := indent -1.
    ]
!

printStyleElement:aStyleElement
    "A style element gets printed."

    |styleEncoder styleElements|

    styleElements := aStyleElement children.
    styleElements isNil ifTrue:[^ self].

    styleEncoder := StyleEncoder on:stream.
    styleEncoder indent:indent + 2.

    self printTagsFor:aStyleElement do:[
        indent := indent + 1.
        self printNewLineIndent.
        stream nextPutAll:'<!!--'.
        stream cr.

        styleElements do:[:styleElement|
            styleEncoder visit:styleElement
        ] separatedBy:[
            stream cr.
        ].

        self printNewLineIndent.
        stream nextPutAll:'-->'.
        indent := indent -1.
    ].
! !

!Encoder methodsFor:'private-printing-tags'!

printInline:anElement
    "An element gets printed inline."

    inOneLine := inOneLine + 1.

    self printTagsFor:anElement do:[ 
        self visitSubelementsOf:anElement.
    ].

    inOneLine := inOneLine - 1.

    "Modified: / 19-08-2010 / 16:09:41 / sr"
!

printSingle:anElement 
    "Print an element without contents.
     Do not use '<tag/>' or '<tag />', because it is not valid HTML 4.01,
     (the first one is valid only in XHTML)."

    |tag|

    tag := anElement tagName.
    tag notNil ifTrue:[
        stream nextPut:$<; nextPutAll:tag.
        self printAttributesOf:anElement.
        stream nextPut:$>.
    ].
!

printSingle:anElement withSpace:withSpace
    <resource: #obsolete>
    "Print an element without contents.
     Ignore withSpace, since it produces invalid HTML code."

    self printSingle:anElement
!

printTagsFor:anElement do:aBlock
    "Print both tags for an element and evaluate a block which
     is printing other data between the tags."

    |tag|

    tag := anElement tagName.
    tag isNil ifTrue:[
        "/ DummyContainer may have a nil tag
        aBlock value.
        ^ self
    ].

    stream nextPut:$<; nextPutAll:tag.
    self printAttributesOf:anElement.
    stream nextPut:$>.

    anElement hasChildren ifTrue:[
        aBlock value.
        anElement canBeReformatted ifTrue:[
            lastWasString ifFalse:[
                self printNewLineIndent.
            ]
        ]
    ].

    stream nextPutAll:'</'; nextPutAll:tag; nextPut:$>.

    "Modified: / 19-08-2010 / 17:13:00 / sr"
! !

!Encoder methodsFor:'private-queries'!

areChildrenSimpleOf:element
    |children firstChild|

    children := element children.
    children isEmptyOrNil ifTrue:[^ true]. 
    children size == 1 ifTrue:[
        firstChild := children first.
        firstChild isString ifTrue:[
            (firstChild includes:(Character cr)) ifFalse:[^ true].       
        ] ifFalse:[
            element isSpan ifTrue:[
                firstChild isAnchor ifTrue:[^ true].
            ].
        ].
    ].

    ^ false

    "Modified: / 14-02-2011 / 11:50:01 / sr"
! !

!Encoder methodsFor:'visiting'!

visitSubelement:aSubElement 
    "Visit a subelement.
     Care for indent and newlines"

    self visitSubelement:aSubElement indent:true
!

visitSubelement:aSubElement indent:doIndent
    "Visit a subelement.
     Care for indent and newlines"

    |oldIndent|

    aSubElement isNil ifTrue:[^ self].

    aSubElement isString ifTrue:[
        "/ no indent here - otherwise textarea gets mad at you !!
        self visitString:aSubElement.
        lastWasString := true.
        "/ stream cr.
    ] ifFalse:[
        lastWasString ifTrue:[
            "/ without linebreak
            aSubElement acceptHTMLVisitor:self. 
        ] ifFalse:[
            oldIndent := indent.

            doIndent ifTrue:[
                indent := indent + 1.
                self printNewLineIndent.
            ].
            aSubElement acceptHTMLVisitor:self. 
            indent := oldIndent.
        ].
        lastWasString := false.
    ]

    "Modified: / 19-08-2010 / 16:52:53 / sr"
!

visitSubelementsOf:anElement
    "Visit the subelements of an element."

    |wantIndent|

    wantIndent := (anElement isPre or:[anElement isCode or:[anElement isDummyContainer]]) not.
    self visitSubelementsOf:anElement indent:wantIndent.

    "Modified: / 03-07-2009 / 15:15:56 / sr"
!

visitSubelementsOf:anElement indent:wantIndent
    "Visit the subelements of an element."

    anElement subelementsDo:[:subelement| 
        self 
            visitSubelement:subelement 
            indent:wantIndent.
    ].
! !

!Encoder methodsFor:'visiting - required'!

visitComment:aComment 
    stream nextPutAll:'<!!--'; nextPutAll:aComment commentString; nextPutAll:'-->'.

    "Created: / 17-04-2007 / 13:51:53 / cg"
!

visitElement:anElement 
    "The task of this encoder is to print html code.
     So print a default version of every item of the tree."

    anElement canHaveText ifFalse:[
        anElement children isEmptyOrNil ifTrue:[
            anElement mustBeClosed ifFalse:[
                self printSingle:anElement.
                ^ self.
            ].
        ]
    ].

    anElement printInlineWithSimpleElements ifTrue:[
        (self areChildrenSimpleOf:anElement) ifTrue:[
            self printInline:anElement.
            ^ self.
        ].
    ].

    self printTagsFor:anElement do:[ 
        self visitSubelementsOf:anElement 
    ].
!

visitProcessingInstruction:aProcessingInstruction 
    stream nextPutAll:'<?'; nextPutAll:aProcessingInstruction codeString; nextPutAll:'?>'.

    "Created: / 17-04-2007 / 13:52:15 / cg"
!

visitString:aString
    "A string gets visited.
     We like to print it onto our stream."

    aString bitsPerCharacter > 8 ifTrue:[
        stream nextPutAll:aString utf8Encoded.
        ^ self
    ].
    stream nextPutAll:aString.

    "Modified: / 19-08-2010 / 16:44:25 / sr"
    "Modified: / 24-03-2012 / 21:30:56 / cg"
! !

!Encoder methodsFor:'visiting-BlockLevel'!

visitDiv:aDiv
    (self areChildrenSimpleOf:aDiv) ifTrue:[
        self printInline:aDiv.
    ] ifFalse:[
        super visitDiv:aDiv.
    ].

    "Created: / 18-03-2007 / 21:05:04 / cg"
! !

!Encoder methodsFor:'visiting-Form'!

visitOption:anOptionElement
    (self areChildrenSimpleOf:anOptionElement) ifTrue:[
        self printInline:anOptionElement.
    ] ifFalse:[
        super visitOption:anOptionElement.
    ].
!

visitTextArea:anInput
    "A textarea gets visited."

    |tag |

    tag := anInput tagName.
    tag notNil ifTrue:[       
        stream nextPut:$<; nextPutAll:tag.
        self printAttributesOf:anInput.
        stream nextPut:$>.
        anInput subelementsDo:[:subelement| 
            self visitSubelement:subelement indent:true.
        ].

        stream nextPutAll:'</'; nextPutAll:tag; nextPut:$>.
    ].
! !

!Encoder methodsFor:'visiting-Head'!

visitScript:aScript
    "A script gets visited."

    self printScriptElement:aScript.
!

visitStyle:aStyle
    "A style element gets visited."

    self printStyleElement:aStyle.
!

visitStyleElement:anElement
    (self areChildrenSimpleOf:anElement) ifTrue:[
        self printInline:anElement.
    ] ifFalse:[
        super visitStyleElement:anElement.
    ].
!

visitTitle:aTitle
    "A title element gets visited."

    self printInline:aTitle.
! !

!Encoder methodsFor:'visiting-Inline'!

visitAnchor:anAnchor 
    self printInline:anAnchor.
!

visitArea:anArea
    self printSingle:anArea.
!

visitBreak:aLineBreak 
    self printSingle:aLineBreak.
!

visitInput:anInput
    self printSingle:anInput.
! !

!Encoder methodsFor:'visiting-ListDefinition'!

visitDefinitionTerm:aTerm 
    "A definition term gets visited."
    
    self printInline:aTerm.
! !

!Encoder methodsFor:'visiting-Table'!

visitTableDataCell:aTableDataCell
    (self areChildrenSimpleOf:aTableDataCell) ifTrue:[
        self printInline:aTableDataCell.
    ] ifFalse:[
        super visitTableDataCell:aTableDataCell.
    ].
!

visitTableHeaderCell:aTableHeaderCell 
    (self areChildrenSimpleOf:aTableHeaderCell) ifTrue:[
        self printInline:aTableHeaderCell.
    ] ifFalse:[
        super visitTableHeaderCell:aTableHeaderCell.
    ].
! !

!Encoder methodsFor:'visiting-TopLevel'!

visitDocument:aDocument
    "A document gets visited.
     Redefined to display the document type if existent."

    |docType|

    docType := aDocument docTypeString.
    docType notNil ifTrue:[
        stream nextPutAll:docType.
        stream cr.
    ].

    self visitElement:aDocument.
! !

!Encoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__Encoder.st,v 1.89 2012/03/24 20:34:38 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__Encoder.st,v 1.89 2012/03/24 20:34:38 cg Exp $'
! !
