"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

StyleVisitor subclass:#StyleEncoder
	instanceVariableNames:'stream indent'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-Utilities'
!

!StyleEncoder class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!StyleEncoder class methodsFor:'instance creation'!

new
    "Create a new encoder. 
     Build the necessary stream."

    |stream|

    stream := WriteStream on: String new.
    ^ self on:stream.
!

on:aStream
    "Create an encoder on a stream."

    |instance|

    instance := self basicNew initialize.
    instance setStream:aStream.

    ^ instance.
! !

!StyleEncoder methodsFor:'accessing'!

indent
    "Return the indent used while printing."

    indent isNil ifTrue:[
        indent := 0.
    ].

    ^ indent.
!

indent:anIndention
    "Set the indent used while printing."

    indent := anIndention.
! !

!StyleEncoder methodsFor:'printing'!

htmlString
    "Return the html string.
     Call this after doing a '#visit:'."

    ^ stream contents.
! !

!StyleEncoder methodsFor:'private-accessing'!

setStream:aStream
    "Set a stream as the stream to print on.
     Private - Use only for instance creation."

    stream := aStream.
! !

!StyleEncoder methodsFor:'private-printing'!

printProperties:aStyleElement
    "Print the property-list of a style element onto the stream.
     The properties are enclosed in curly-brackets."

    |properties|

    properties := aStyleElement properties. 

    stream nextPut:${.

    properties notEmptyOrNil ifTrue:[
        stream cr.
    ].

    aStyleElement properties do:[
        :property|
        self printProperty:property.
    ].

    indent timesRepeat:[stream tab].
    stream nextPut:$}.
!

printProperty:aProperty
    "Print one property, that could be a string, too."

   (self indent + 1) timesRepeat:[stream tab].

    aProperty isString ifTrue:[
        stream nextPutAll:aProperty.
        stream cr.
        ^ self.
    ].           

    aProperty acceptHTMLVisitor:self.
    stream cr.
!

printPropertyValue:aProperty
    "Print one property value on  the stream."

    stream nextPutAll:aProperty value printString.
!

printPropertyValues:aProperty
    "Print a list of propery-values on the stream."

    |value|

    value := aProperty value.

    value isString ifTrue:[
        self printPropertyValue:value.
        stream nextPut:$;.
        ^ self.
    ].

    value isOrderedCollection ifTrue:[
        value do:[
            :subvalue|
            self printPropertyValue:subvalue.
        ] separatedBy:[
            stream space.
        ].
        stream nextPut:$;.
        ^ self.
    ].

    "Whatever you are display yourself as a string."
    self printPropertyValue:value.

    stream nextPut:$;.
!

printSelectors:aStyleElement
    "The selector-list of a style element gets printed.
     The list is comma-separated."

    self indent timesRepeat:[stream tab].

    aStyleElement selectors do:[:selector|
        stream nextPutAll:selector printString.
    ] separatedBy:[
        stream nextPutAll:', '.
    ].
!

printString:aString 
    "Print a string being included as element."

    |collection|
    collection := aString asCollectionOfSubstringsSeparatedBy:Character cr.

    collection do:[
        :string|
        self indent timesRepeat:[stream tab].
        stream nextPutAll:string.
    ] separatedBy:[
        stream cr.
    ].
!

printStyleElement:aStyleElement
    "Print a style element."


    self printSelectors:aStyleElement.

    self printProperties:aStyleElement.
!

printStyleProperty:aProperty
    "Print a style property."

    stream nextPutAll:aProperty keyName; nextPutAll:': '.

    self printPropertyValues:aProperty.
! !

!StyleEncoder methodsFor:'required-visiting'!

visitString:aString 
    "A string gets visited."

    self printString:aString.
!

visitStyleElement:aStyleElement
    "A style element gets visited."

    self printStyleElement:aStyleElement.
!

visitStyleProperty:aProperty
    "A style property gets visited."

    self printStyleProperty:aProperty.
! !

!StyleEncoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__StyleEncoder.st,v 1.15 2010/06/14 10:50:45 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__StyleEncoder.st,v 1.15 2010/06/14 10:50:45 cg Exp $'
! !
