"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

AbstractHTMLElement subclass:#Table
	instanceVariableNames:'caption head foot'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-ModelTree-Table'
!

!Table class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    HTML table object

    [author:]
        Martin Walser

    [instance variables:]
        -

    [class variables:]
        -

    [see also:]
        TableBody
        TableHead
        TableFoot
        TableRow
        TableDataCell
        TableHeaderCell
        Caption
"
!

examples
"
Building a table:
[exBegin]
    |table|
    table := Table new.
    table border:  1.
    table caption: 'This is the caption of a generated table'.
    table head:    #('Hans' 'Otto' 'Karl' 'Kurt' 'Gerd' 'John' 'Bert').
    table addRow:  #(373 937 363 836 217 832 234).
    table addRow:  #(945 123 534 345 563 343 342).
    table foot:    #(111 222 333 444 555 666 777).
    table printHtmlString.
[exEnd]


Accessing a single cell:
[exBegin]
    |table cell|
    table := HTML::Table new.
    table addBody:  #( #(678 212 131 200 344)
                       #(312 567 112 678 120)
                       #(177 314 234 189 789)).

    cell := table cellAt:3@2.
    cell bgColor:Color red.
[exEnd]
"
! !

!Table class methodsFor:'required protocol'!

tagName
    ^ 'table'.
! !

!Table methodsFor:'interface-attributes'!

bgcolor:aColorSpec
    self attributeAt:#bgcolor put:aColorSpec.
!

cellPadding
    "Return the cellpadding of the table.
     The cellpadding is the spacing within the cells."

    ^ self cellpadding.
!

cellPadding:aLength
    "Set the cellpadding of the table.
     The cellpadding is the spacing within the cells."

    self cellpadding:aLength.
!

cellSpacing
    "Return the cellspacing of the table.
     The cellspacing is the spacing between the cells."

    ^ self cellspacing.
!

cellSpacing:aLength
    "Set the cellspacing of the table..
     The cellspacing is the spacing between the cells."

    self cellspacing:aLength.
!

cellpadding
    "Return the cellpadding of the table.
     The cellpadding is the spacing within the cells."

    ^ self attributeAt:#cellpadding.
!

cellpadding:aLength
    "Set the cellpadding of the table.
     The cellpadding is the spacing within the cells."

    self attributeAt:#cellpadding put:aLength.
!

cellspacing
    "Return the cellspacing of the table.
     The cellspacing is the spacing between the cells."

    ^ self attributeAt:#cellspacing.
!

cellspacing:aLength
    "Set the cellspacing of the table..
     The cellspacing is the spacing between the cells."

    self attributeAt:#cellspacing put:aLength.
!

height
    "Return the height of the table."

    ^ self attributeAt:#height.
!

height:aNumberOrPercentString
    "Set the height of the table."

    self attributeAt:#height put:aNumberOrPercentString.
!

summary
    "Return a text with purpose and structure of table."

    ^ self attributeAt:#summary.
!

summary:aStringOrText
    "Set a text with purpose and structure of table."

    self attributeAt:#summary put:aStringOrText.
!

vspace:verticalSpace
    "Set the vertical spacing (space above and below) of the table."

    self attributeAt:#vspace put:verticalSpace.
!

width
    "Return the width of the table."

    ^ self attributeAt:#width.
!

width:aNumberOrPercentString
    "set the width of the table."

    self attributeAt:#width put:aNumberOrPercentString.
! !

!Table methodsFor:'interface-attributes-frame'!

frame
    "Return the frame of the table.
     The frame is the outer borders.

     Possible frame types are: 
         void, above, below, hsides, 
         lh, rhs, vsides, box, border"

    ^ self attributeAt:#frame
!

frame:aFrameType
    "Set the frame of the table.
     The frame is the outer borders.

     Possible frame types are: 
         void, above, below, hsides, 
         lh, rhs, vsides, box, border"

    self attributeAt:#frame put:aFrameType.
!

frameAbove
    "Set frame to be at above.
     The frame is the outer borders."    

    self frame:#above.
!

frameBelow
    "Set frame to be at below.
     The frame is the outer borders."

    self frame:#below.
!

frameBorder
    "Set the frame to be border.
     The frame is the outer borders."

    self frame:#border.
!

frameBox
    "Set the frame to be as box.
     The frame is the outer borders."

    self frame:#box.
!

frameHorizontalSides
    "Set the frame to be at horizontal sides.
     The frame is the outer borders."

    self frame:#hsides.
!

frameLeftHorizontalSide
    "Set frame to be at left horizontal sides.
     The frame is the outer borders."    

    self frame:#lhs.
!

frameRightHorizontalSide
    "Set frame to be at right horizontal side.
     The frame is the outer borders."    

    self frame:#rhs.
!

frameVerticalSides
    "Set the frame to be at vertical sides.
     The frame is the outer borders."

    self frame:#vsides.
! !

!Table methodsFor:'interface-attributes-rules'!

beRulesAll
    "Set rules to be rows.
     The rules are the inner borders."    

    self rules:#all.
!

beRulesColumns
    "Set rules to be columns.
     The rules are the inner borders."    

    self rules:#cols.
!

beRulesGroups
    "Set rules to be groups.
     The rules are the inner borders."    

    self rules:#groups.
!

beRulesNone
    "Set rules to have no rules.
     The rules are the inner borders."    

    self rules:#none.
!

beRulesRows
    "Set rules to be rows.
     The rules are the inner borders."    

    self rules:#rows.
!

rules
    "Return the rules of the table.
     The rules are the inner borders.

     Possible frame types are: 
          none, groups, rows, cols, all"

    ^ self attributeAt: #rules.
!

rules:aRuleType
    "Set the rules of the table.
     The rules are the inner borders.

     Possible frame types are: 
          none, groups, rows, cols, all"

    self attributeAt:#rules put:aRuleType.
! !

!Table methodsFor:'interface-elements'!

caption
    "Return the caption of the table.
     The caption is the heading of a table."

    ^ caption.
!

caption:aCaptionOrString
    "Set the caption of the table.
     The caption is the heading of a table.
     You can set a caption element or a simple string."

     caption := self buildElement:aCaptionOrString ofType:Caption.

     caption parent:self.
!

foot
    "Return the foot of the table."

    ^ foot.
!

foot:aFoot
    "Set the table foot.
     A foot can be a ready built table foot, a simple string to be wrapped
     by a table foot or a collection of heading cells or strings to be the 
     foot cells of the head."

    foot := self buildElement:   aFoot 
                 ofType:         TableFoot 
                 onCollectionDo: [:foot :rawData|foot addCell:rawData]. 

    foot parent:self.
!

head
    "Return the head of the table."

    ^ head.
!

head:aHead
    "Set the table head. The head is the heading or headline of the table.
     A head can be a ready built table head, a simple string to be wrapped
     by a table head or a collection of heading cells or strings to be the 
     heading cells of the head."

    head := self buildElement:   aHead 
                 ofType:         TableHead 
                 onCollectionDo: [:head :rawData|head addCell:rawData]. 

    head parent:self.
! !

!Table methodsFor:'interface-elements-body'!

addBody
    "Build an empty body and 
     add it to the children."

    ^ self addBody:nil.
!

addBody:aTableBodyOrString
    "Add a table body to the children of this table.
     The passed table body can be a ready built table body,
     a simple string to be wrapped in a body, or a collection 
     of table rows. These table rows can be either ready built
     table rows or simple strings too."

    |body|

    body := self buildElement:   aTableBodyOrString 
                ofType:         TableBody 
                onCollectionDo: [:body :rawData|body addRow:rawData]. 

    ^ self addChild:body.
!

body
    "Return a default body,
     it is the first body found.
     If no body can be found nil."

    ^ self bodyAt:1.
!

bodyAt:anIndex
    "Return an existing body at the given index or nil."

    ^ self childAt:anIndex ifAbsent:nil. 
! !

!Table methodsFor:'interface-elements-cell'!

bodyAt:aBodyIndex rowAt:aRowIndex cellAt:aCellIndex 
    "Find a cell by the given indices.
     Return nil if one of the indices doesn't exist."
    
    |row|

    row := self bodyAt:aBodyIndex rowAt:aRowIndex. 
    row isNil ifTrue:[^ nil].

    ^ row cellAt:aCellIndex.
!

bodyAt:aBodyIndex rowAt:aRowIndex cellAt:aCellIndex put:aBodyOrString
    "Set a cell at the given indices.
     The cell can be a cell element
     or a simple string."

     |body|

     body := self getBodyAt:aBodyIndex.

     body rowAt:  aRowIndex 
          cellAt: aCellIndex 
          put:    aBodyOrString.
!

cellAt:aCellIndexPoint
    "Find a cell by the given indices.
     Return nil if one of the indices doesn't exist."

     ^ self bodyAt: 1 
            rowAt:  aCellIndexPoint y 
            cellAt: aCellIndexPoint x.
!

cellAt:anIndexPoint put:aBodyOrString
    "Set a cell at the given index.
     The cell can be a cell element
     or a simple string."

    self rowAt:  anIndexPoint y 
         cellAt: anIndexPoint x 
         put:    aBodyOrString.
!

rowAt:aRowIndex cellAt:aCellIndex
    "Find a cell by the given indices.
     Return nil if one of the indices doesn't exist."

     ^ self bodyAt: 1 
            rowAt:  aRowIndex 
            cellAt: aCellIndex.
!

rowAt:aRowIndex cellAt:aCellIndex put:aBodyOrString
    "Set a cell at the given indices.
     The cell can be a cell element
     or a simple string."

    self bodyAt: 1 
         rowAt:  aRowIndex 
         cellAt: aCellIndex 
         put:    aBodyOrString.
! !

!Table methodsFor:'interface-elements-foot'!

footCellAt:aCellIndex
    "Return the head cell at a cell index."

    |foot|
    foot := self foot.
    foot isNil ifTrue:[^ nil].

    ^ foot cellAt:aCellIndex.
!

footCellAt:aCellIndex put:aHeadingCellOrString
    "Set the head cell at a cell index."

    ^ self getFoot cellAt:aCellIndex put:aHeadingCellOrString.
! !

!Table methodsFor:'interface-elements-head'!

headCellAt:aCellIndex
    "Return the head cell at a cell index."

    |head|
    head := self head.
    head isNil ifTrue:[^ nil].

    ^ head cellAt:aCellIndex.
!

headCellAt:aCellIndex put:aHeadingCellOrString
    "Set the head cell at a cell index."

    ^ self getHead cellAt:aCellIndex put:aHeadingCellOrString.
! !

!Table methodsFor:'interface-elements-row'!

addRow
    "Build a new empty table row."

    ^ self addRow:nil.
!

addRow:aTableRow
    "Add a table row to the children of this table.
     The passed table row can be a ready built table row,
     a simple string to be wrapped in a row, or a collection 
     of data cells. These data cells can be either ready built
     data cells or simple strings too."

    ^ self getBody addRow:aTableRow.
!

bodyAt:aBodyIndex rowAt:aRowIndex 
    "Find a row by the given indices.
     Return nil if one of the indices doesn't exist."
    
    |body|

    body := self bodyAt:aBodyIndex.
    body isNil ifTrue:[
        ^ nil
    ].

    ^ body rowAt:aRowIndex.
!

bodyAt:aBodyIndex rowAt:aRowIndex put:aBodyOrString
    "Set a row at the given indices."

     |body|

     body := self getBodyAt:aBodyIndex.

     body rowAt:aRowIndex put:aBodyOrString.
!

rowAt:anIndex
    "Try to find a row at the given index.
     Look first in a body if one exists,
     then look at the children for direct rows."

    |row|

    row := self body rowAt:anIndex.
    row notNil ifTrue:[^ row].

    ^ self childAt:anIndex ifAbsent:nil.
! !

!Table methodsFor:'private-accessing'!

getBody
    "Return a body to add rows.
     If no body exists, build one."

    ^ self getElement:TableBody.
!

getBodyAt:aBodyIndex
    "Return a body at the given index.
     If not exists build one."

    ^ self childAt:aBodyIndex 
           ifAbsent:[self addElement:TableBody at:aBodyIndex].
!

getCaption
    "Return a caption.
     If no caption exists, build one."

    caption isNil ifTrue:[
        caption := Caption new.  
        caption parent: self.
    ].

    ^ caption
!

getFoot
    "Return a head to add head cells.
     If no head exists, build one."

    foot isNil ifTrue:[
        foot := TableFoot new.  
        foot parent: self.
    ].
    ^ foot
!

getHead
    "Return a head to add head cells.
     If no head exists, build one."

    head isNil ifTrue:[
        head := TableHead new.
        head parent:self.
    ].

    ^ head
! !

!Table methodsFor:'queries'!

canHaveChild:anElement
    anElement isTableRow ifTrue:[^ true].
    anElement isTableHead ifTrue:[^ true].
    anElement isTableBody ifTrue:[^ true].
"/ self halt.
    ^ false

    "Created: / 06-02-2007 / 20:05:41 / cg"
!

canHaveText
    ^ false

    "Created: / 06-02-2007 / 20:05:41 / cg"
! !

!Table methodsFor:'required-visiting'!

acceptHTMLVisitor:aVisitor
    "visitor pattern: dispatch me to the visitor"

    aVisitor visitTable:self.

    "Modified: / 05-11-2007 / 22:08:25 / cg"
!

subelementsDo:aOneArgumentBlock
    "Execute a one argument block additionally for caption, head and foot"

    caption notNil ifTrue:[aOneArgumentBlock value:caption].
    head    notNil ifTrue:[aOneArgumentBlock value:head].
    foot    notNil ifTrue:[aOneArgumentBlock value:foot].

    super subelementsDo:aOneArgumentBlock.
! !

!Table methodsFor:'testing'!

isBlockElement
    ^ true

    "Created: / 06-02-2007 / 17:32:25 / cg"
!

isTable
    ^ true
! !

!Table class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__Table.st,v 1.35 2007/11/05 21:12:37 cg Exp $'
! !
