"
 COPYRIGHT (c) 2005 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree' }"

"{ NameSpace: HTML }"

AbstractTreeBuilder subclass:#TreeBuilder
	instanceVariableNames:'currentForm'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-Utilities'
!

!TreeBuilder class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2005 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    HTML tree builder - makes it easier to construct valid HTML-trees.
    This class helps to generate html; it provides a message protocol similar to
    html itself (i.e. tr; td; tdEdn; trEnd etc.

    See examples
"
!

examples
"
                                                                                [exBegin]
     |b|

     b := HTML::TreeBuilder new.
     b
        html;
          head;
          headEnd;
          body;
            table;
              tr;
                td; text:'hello'; tdEnd;
                td; text:'world'; tdEnd;
              trEnd;
            tableEnd;
          bodyEnd;
        htmlEnd.
     b htmlString inspect.
                                                                                [exEnd]

                                                                                [exBegin]
     |b|

     b := HTML::TreeBuilder new beginWith:(Document new).
     b 
        head;
        headEnd;
        body;
          table;
            tr;
              td; tdEnd;
              td; tdEnd;
            trEnd;
          tableEnd;
        bodyEnd.
     b rootElement inspect.
     b htmlString inspect.
                                                                                [exEnd]

                                                                                [exBegin]
     |b|

     b := HTML::TreeBuilder new.
     b 
        span;
          table;
            tr;
              td; tdEnd;
              td; tdEnd;
            trEnd;
          tableEnd;
        spanEnd.
     b rootElement inspect.
                                                                                [exEnd]

                                                                                [exBegin]
     |b|

     b := HTML::TreeBuilder new beginWith:(Body new).
     b 
        textArea;
           rawText:'line1';
           rawText:'line2';
           rawText:'line3';
           rawText:'line4
line5
line6';
        textAreaEnd.
     b rootElement printOn:Transcript.
     b rootElement htmlString inspect.
                                                                                [exEnd]

                                                                                [exBegin]
     HTML::TreeBuilder htmlContentsOf:[:html | 
        html
            body;
                h1:'headLine';
                p; text:'Hello world'; pEnd;
            bodyEnd.
     ].  
                                                                                [exEnd]
"
! !

!TreeBuilder class methodsFor:'instance creation'!

newDiv
    ^ self new beginWith:Div new.

    "
     self newDiv
         style:'border-width: 1px;';
         div;
             text:'Hello World';
         divEnd;
         htmlString
    "

    "Created: / 19-11-2010 / 12:25:29 / cg"
!

newDocument
    ^ self new beginDocument.

    "
     self newDocument
         div;
             text:'Hello World';
         divEnd;
         htmlString
    "

    "Created: / 19-11-2010 / 12:32:00 / cg"
! !

!TreeBuilder class methodsFor:'html generation'!

htmlContentsOf:aBlock
    "inject a new builder into a block, return what is generated there"

    |bldr|

    bldr := self new.
    aBlock value:bldr.
    ^ bldr htmlString

    "
     HTML::TreeBuilder htmlContentsOf:[:html | 
        html
            body;
                h1:'headLine';
                p; text:'Hello world'; pEnd;
            bodyEnd.
     ].  
    "

    "Modified: / 11-05-2011 / 12:50:20 / cg"
! !

!TreeBuilder methodsFor:'accessing'!

currentForm
    ^ currentForm
!

htmlString
    "html of the generated tree so far"

    |r|

    r := self rootElement.
    r isNil ifTrue:[^ ''].
    r isString ifTrue:[^ r].
    ^ r htmlString

    "Created: / 24-11-2006 / 17:13:13 / cg"
    "Modified: / 02-07-2009 / 18:15:17 / sr"
! !

!TreeBuilder methodsFor:'attributes'!

accesskey:accessKeyAttribute 
    self attribute:'accesskey' is:accessKeyAttribute
!

action:actionURLText
    attributableElement action:actionURLText
!

align:align
    "The alignment can be left, center, right."

    attributableElement align:align
"/    self assert:(attributableElement isTable
"/                 or:[attributableElement isTableDataCell
"/                 or:[attributableElement isDiv]]).
"/    self attribute:'align' is:align

    "Modified: / 12-04-2007 / 07:35:37 / cg"
!

alt:altText
    self assert:(attributableElement isImg).
    self attribute:'alt' is:altText
!

autocomplete:onOffString
    "if onOffSTring = 'off' give the browser an hint to not store an input field.
     NOTE: this is not W3C standard!!"

    attributableElement autocomplete:onOffString
!

bgcolor:colorText
    attributableElement bgcolor:colorText
"/     self attribute:'bgcolor' is:colorText
!

border:borderWidthInPixels
    attributableElement border:borderWidthInPixels.
"/    self assert:(attributableElement isTable
"/                 or:[attributableElement isImg]).
"/    self attribute:'border' is:border

    "Modified: / 09-05-2007 / 18:11:33 / cg"
!

cellPadding:anInteger
    attributableElement cellPadding:anInteger

    "Created: / 15-01-2008 / 14:31:21 / cg"
!

cellSpacing:anInteger
    attributableElement cellSpacing:anInteger

    "Created: / 15-01-2008 / 14:31:09 / cg"
!

cellpadding:cellpadding
    attributableElement cellpadding:cellpadding
!

cellspacing:cellspacing
    attributableElement cellspacing:cellspacing
"/    self assert:(attributableElement isTable).
"/    self attribute:'cellspacing' is:cellspacing
!

checked:aBoolean
    attributableElement checked:aBoolean
!

class:classString
    classString notNil ifTrue:[
        self assert:(classString isString).
    ].
"/    self assert:(classString includes:$_) not.
    self attribute:'class' is:classString
!

cols:n
    attributableElement cols:n
!

colspan:n
    attributableElement colspan:n
!

content:contentText
    self assert:(attributableElement isMeta).
    self attribute:'content' is:contentText
!

disabled:aBoolean
    attributableElement disabled:aBoolean
!

enctype:enctypeText
    attributableElement enctype:enctypeText
!

for:targetID
    attributableElement for:targetID.
!

frameborder:frameborder
    attributableElement frameborder:frameborder
!

framespacing:framespacing
    attributableElement framespacing:framespacing
!

height:aNumberOrPercentString
    attributableElement height:aNumberOrPercentString    
!

href:href
    attributableElement href:href
!

http_equiv:httpEquivText
    self assert:(attributableElement isMeta).
    self attribute:'http-equiv' is:httpEquivText
!

language:source
    attributableElement language:source
!

marginheight:marginheight
    attributableElement marginheight:marginheight
!

marginwidth:marginwidth
    attributableElement marginwidth:marginwidth
!

maxLength:sz
    attributableElement maxLength:sz
!

media:typeString
    attributableElement media:typeString.
!

method:methodText
    attributableElement method:methodText.
!

multiple:multiple
    attributableElement multiple:multiple
!

nlsValue:nlsString
    "text which is a national-language-translated-string, as value"

    self value:(self nlsTranslated:nlsString).

    "Created: / 03-07-2010 / 10:10:02 / cg"
!

noshade
    attributableElement noshade
!

nowrap
    attributableElement nowrap
!

onBlur:action
    attributableElement onBlur:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:36 / cg"
!

onChange:action
    attributableElement onChange:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:38 / cg"
!

onClick:action
    attributableElement onClick:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:41 / cg"
!

onDoubleClick:action
    attributableElement onDoubleClick:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:43 / cg"
!

onFocus:action
    attributableElement onFocus:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:44 / cg"
!

onKeyPress:action
    attributableElement onKeyPress:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:46 / cg"
!

onKeyUp:action
    attributableElement onKeyUp:(self makeScriptActionText:action)

    "Created: / 25-06-2009 / 17:24:27 / sr"
    "Modified: / 30-06-2010 / 21:29:48 / cg"
!

onLoad:action

    self assert:(attributableElement isBody).
    attributableElement onLoad:action "/ `can we allow: (self makeScriptActionText:action).

    "Modified: / 30-06-2010 / 21:30:40 / cg"
!

onMouseDown:action
    attributableElement onMouseDown:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:55 / cg"
!

onMouseMove:action
    attributableElement onMouseMove:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:29:58 / cg"
!

onMouseOut:action
    attributableElement onMouseOut:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:30:00 / cg"
!

onMouseOver:action
    attributableElement onMouseOver:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:30:02 / cg"
!

onMouseUp:action
    attributableElement onMouseUp:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:30:04 / cg"
!

onSelect:action
    attributableElement onSelect:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:30:06 / cg"
!

onSubmit:action
    self assert:(attributableElement isForm).
    attributableElement onSubmit:(self makeScriptActionText:action)

    "Modified: / 30-06-2010 / 21:30:19 / cg"
!

rel:relText
    attributableElement rel:relText
"/    self assert:(attributableElement isLink).
"/    self attribute:'rel' is:relText
!

rows:n
    attributableElement rows:n
!

rowspan:n
    attributableElement rowspan:n
!

rules:rulesText               

    rulesText isEmptyOrNil ifTrue:[^ self].
    self assert:(attributableElement isTable).
    attributableElement rules:rulesText
!

script:aJavaScript
    |alreadyPresentScript styleAttribute|

    aJavaScript isEmptyOrNil ifTrue:[^ self].

    attributableElement isAnchor ifTrue:[
        attributableElement href notEmptyOrNil ifTrue:[
            self error:'You can define a script action or a href target, but not both!! Do you know what should happen?' mayProceed:true.
        ] ifFalse:[            
            alreadyPresentScript := attributableElement onClick.
            alreadyPresentScript notEmptyOrNil ifTrue:[
                (alreadyPresentScript last == $;) ifTrue:[
                    attributableElement onClick:alreadyPresentScript, aJavaScript.
                ] ifFalse:[
                    attributableElement onClick:alreadyPresentScript, ';', aJavaScript.
                ].
            ] ifFalse:[
                attributableElement onClick:aJavaScript.
            ].

            "/ SR: 
            "/ we got an anchor with a script action. 
            "/ but we don't have a hand cursor caused by the empty href attribute
            "/ lets add the hand cursor
            "/ but do not overwrite an already defined cursor  
            styleAttribute := attributableElement style.
            (styleAttribute isEmptyOrNil or:[(attributableElement style asLowercase includesSubString:'cursor') not]) ifTrue:[
                self style:'cursor:pointer;cursor:hand;'.
            ].
        ].
    ] ifFalse:[
        self error:'not supported, please implement if possible' mayProceed:true.
    ].

    "Created: / 11-06-2010 / 10:38:52 / sr"
!

scrolling:aBooleanOrStringOrSymbol
    aBooleanOrStringOrSymbol == false ifTrue:[
        self scrolling:'no'.
        ^ self.
    ].
    aBooleanOrStringOrSymbol == true ifTrue:[
        self scrolling:'yes'.
        ^ self.
    ].
    attributableElement scrolling:aBooleanOrStringOrSymbol
!

selected
    attributableElement beSelected
!

selected:aBoolean
    attributableElement selected:aBoolean
!

size:sz
    attributableElement size:sz
!

src:source
    attributableElement src:source
!

style:moreStyle
    "set or add-to the style"

"/(((attributableElement attributeAt:#style) ? '') includesString:'position:') ifTrue:[
"/   self assert:(moreStyle includesString:'position:') not
"/].

    attributableElement attributeAt:#style add:moreStyle
!

tabindex:anInteger
    attributableElement tabindex:anInteger.
!

target:targetText
    attributableElement target:targetText
!

title:titleText               
    titleText isEmptyOrNil ifTrue:[^ self].
    attributableElement title:titleText
!

tooltip:tooltipText
    "tooltip: defaults to a title here; can be redefined for nicer tooltips
     (multipline, using a hiver-visible div) in subclasses"

    self title:tooltipText asString

    "Created: / 22-02-2011 / 08:48:47 / cg"
!

type:typeString
    attributableElement type:typeString.
!

valign:align
    "Set the vertical align of the table/cell (top|middle|bottom|baseline)."

    self assert:(attributableElement isTableCell
                 or:[attributableElement isDiv]).
    attributableElement valign:align.
"/    self attribute:'valign' is:align
!

value:valueString
    self assert:(attributableElement isInput or:[attributableElement isOption]).
    self attribute:'value' is:valueString
!

vspace:verticalSpace
    attributableElement vspace:verticalSpace
!

width:aNumberOrPercentString
    aNumberOrPercentString isNil ifTrue:[^ self].
    attributableElement width:aNumberOrPercentString
! !

!TreeBuilder methodsFor:'building'!

a
    self startElement:(Anchor new).
!

a:textOrBlock
    self 
        a;
        textOrBlock:textOrBlock;
        endElementButKeepForAttribute.
!

a:textOrBlock href:href
    self 
        a:textOrBlock;
        href:href
!

aEnd
    self assert:(currentElement isAnchor).
    self endElement
!

blockQuote
    self startElement:(BlockQuote new).

    "Created: / 06-02-2007 / 15:46:30 / cg"
!

blockQuoteEnd
    self assert:(currentElement isBlockQuote).
    self endElement

    "Created: / 06-02-2007 / 15:47:14 / cg"
!

body
    self startElement:(Body new).
!

body:aBlock
    self body.
    aBlock value.
    self bodyEnd.
!

bodyEnd
    self assert:(currentElement isBody).
    self endElement
!

bold
    self startElement:(Bold new).
!

boldEnd
    self assert:(currentElement isBold).
    self endElement
!

br
    currentElement add:(attributableElement := Break new).
!

center
    self startElement:(Center new).
!

centerEnd
    self assert:(currentElement isCenter).
    self endElement
!

code
    self startElement:(Code new).
!

codeEnd
    self assert:(currentElement isCode).
    self endElement
!

comment:aComment
    currentElement add:('<!!-- ' , aComment printString, ' -->').
    attributableElement := nil.
!

commentAttribute:aComment
    attributableElement attributeAt:#comment put:aComment.































!

div
"/ self assert:currentElement isTable not.
    self startElement:(Div new).
!

div:textOrBlock
    self 
        div;
        textOrBlock:textOrBlock;
        endElementButKeepForAttribute.
!

divEnd
    self assert:(currentElement isDiv).
    self endElement
!

doesNotUnderstand:aMessage
    |tag el|

    tag := aMessage selector.
    tag numArgs == 0 ifTrue:[
        el := AbstractHTMLElement newTag:tag.
        el notNil ifTrue:[
            "/ not sure, if its a good idea to fix things on the fly...    
            "/ self halt.
        ].
    ].
    ^ super doesNotUnderstand:aMessage

    "
     |b|

     b := self new beginWith:(Document new).
     b 
        body;
          b;
          text:'hello';
          bEnd;
        bodyEnd.
     b rootElement inspect.
    "
!

embed
    currentElement add:(attributableElement := Embed new).
!

embed:objectSourceURL
    currentElement add:(attributableElement := Embed new source:objectSourceURL).
!

h1
    self assert:(root isNil or:[currentElement isHeading not]).
    self startElement:(Heading1 new).
!

h1:textOrBlock
    self 
        h1; 
        textOrBlock:textOrBlock;
        endElementButKeepForAttribute
!

h1End
    self assert:(currentElement isHeading).
    self assert:(currentElement level == 1).
    self endElement
!

h2
    self assert:(root isNil or:[currentElement isHeading not]).
    self startElement:(Heading2 new).
!

h2:textOrBlock
    self 
        h2; 
        textOrBlock:textOrBlock;
        endElementButKeepForAttribute
!

h2End
    self assert:(currentElement isHeading).
    self assert:(currentElement level == 2).
    self endElement
!

hr
    currentElement add:(attributableElement := HorizontalRule new).
!

html
    self assert:(root isNil).
    self startElement:(Document new).
!

html:aBlock
    self html.
    aBlock value.
    self htmlEnd.
!

htmlEnd
    self endElement
!

iframe
    self startElement:(IFrame new).
!

iframeEnd
    self endElement
!

img:imageSourceURL
    currentElement add:(attributableElement := Image new source:imageSourceURL).
!

noBr
    self startElement:(NoBr new).

    "Created: / 29-11-2011 / 12:19:22 / sr"
!

noBrEnd
    self assert:(currentElement isNoBr).
    self endElement

    "Created: / 29-11-2011 / 12:19:35 / sr"
!

noscript
    self startElement:(NoScript new).
!

noscriptEnd
    self endElement
!

p
    self startElement:(Paragraph new).
!

p:someTextOrBlock
    self p.
    someTextOrBlock isString ifTrue:[
        self text:someTextOrBlock
    ] ifFalse:[
        someTextOrBlock value
    ].
    self pEnd

    "
     HTML::TreeBuilder htmlContentsOf:[:html |
        html p:'hello world'
     ]    
    "
    "
     HTML::TreeBuilder htmlContentsOf:[:html |
        html p:[ html rawText:'&nbsp;' ]
     ]    
    "
    "
     HTML::TreeBuilder htmlContentsOf:[:html |
        html p:[ html center; text:'hello'; centerEnd ]
     ]    
    "
!

pEnd
    self assert:(currentElement isParagraph).
    self endElement
!

pre
    self startElement:(Pre new).

    "Created: / 06-02-2007 / 15:46:22 / cg"
!

preEnd
    self assert:(currentElement isPre).
    self endElement

    "Created: / 06-02-2007 / 15:47:22 / cg"
!

script
    self startElement:(Script new).

    "Created: / 06-02-2007 / 15:46:22 / cg"
!

scriptEnd
    self assert:(currentElement isScript).
    self endElement

    "Created: / 06-02-2007 / 15:47:22 / cg"
!

span
    self startElement:(Span new).
!

span:textOrBlock
    self
        span;
        text:textOrBlock;
        endElementButKeepForAttribute
!

spanEnd
    self assert:(currentElement isSpan).
    self endElement
!

strong
    self startElement:(Strong new).
!

strong:textOrBlock
    self
        strong;
        textOrBlock:textOrBlock;
        endElementButKeepForAttribute
!

strongEnd
    self assert:(currentElement isStrong).
    self endElement
! !

!TreeBuilder methodsFor:'building-forms'!

button
    self startElement:(Button new).
!

buttonEnd
    self assert:(currentElement isButton).
    self endElement
!

form
    self assert:(currentForm isNil).  "/ cannot be nested
    self startElement:(Form new).
    currentForm := currentElement.
!

formEnd
    self assert:(currentElement isForm).
    currentForm := nil.
    self endElement
!

input
    attributableElement := Input new.
    currentElement isNil ifTrue:[
        currentElement := root := attributableElement.
    ] ifFalse:[
        currentElement add:attributableElement.
    ].

    "Modified: / 14-01-2008 / 08:37:29 / cg"
!

label
    self startElement:(Label new).
!

labelEnd
    self assert:(currentElement isLabel).
    self endElement
!

option
    self assert:(root isNil or:[currentElement isSelect]).
    self startElement:(Option new).
!

optionEnd
    self assert:(currentElement isOption).
    self endElement
!

select
    self startElement:(Select new).
!

selectEnd
    self assert:(currentElement isSelect).
    self endElement
!

textArea
    self startElement:(TextArea new).
!

textAreaEnd
    self assert:(currentElement isTextArea).
    self endElement
!

textarea
    self textArea.
!

textareaEnd
    self textAreaEnd
! !

!TreeBuilder methodsFor:'building-head'!

base
    currentElement add:(attributableElement := Base new).
!

head
    self startElement:(Head new).
!

headEnd
    self assert:(currentElement isHead).
    self endElement
!

link
    currentElement add:(attributableElement := Link new).
!

meta
    currentElement add:(attributableElement := Meta new).
! !

!TreeBuilder methodsFor:'building-lists'!

ol
    self startElement:(OrderedList new).
!

olEnd
    self assert:(currentElement isOrderedList).
    self endElement
! !

!TreeBuilder methodsFor:'building-tables'!

caption
    self startElement:(Caption new).
!

captionEnd
    self endElement
!

col
    currentElement add:(attributableElement := Col new).
!

colgroup
    self assert:(currentElement isTable).
    self startElement:(Colgroup new).
!

colgroupEnd
    self assert:(currentElement isColgroup).
    self endElement
!

li
    self assert:(currentElement isUnorderedList).
    self startElement:(ListItem new).
!

liEnd
    self assert:(currentElement isListItem).
    self endElement.
!

table
    self startElement:(Table new).
!

tableEnd
    self assert:(currentElement isTable).
    self endElement
!

tbody
    self assert:(root isNil or:[currentElement isTable or:[currentElement isDummyContainer]]).
    self startElement:(TableBody new).
!

tbodyEnd
    self assert:(currentElement isTableBody).
    self endElement
!

td
    self assert:(root isNil or:[currentElement isTableRow]).
    self startElement:(TableDataCell new).
!

tdEnd
    self assert:(currentElement isTableDataCell).
    self endElement
!

tfoot
    self assert:(currentElement isTable).
    self startElement:(TableFoot new).
!

tfootEnd
    self assert:(currentElement isTableFoot).
    self endElement
!

th
    self assert:(root isNil or:[currentElement isTableRow]).
    self startElement:(TableHeaderCell new).
!

thEnd
    self assert:(currentElement isTableHeaderCell).
    self endElement
!

thead
    self assert:(currentElement isTable).
    self startElement:(TableHead new).
!

theadEnd
    self assert:(currentElement isTableHead).
    self endElement
!

tr
    self assert:(root isNil
                 or:[root isDummyContainer 
                 or:[currentElement isTable 
                 or:[currentElement isTableHead
                 or:[currentElement isTableBody
                 or:[currentElement isTableFoot]]]]]).
    self startElement:(TableRow new).
!

trEnd
    self assert:(currentElement isTableRow).
    self endElement
!

ul
    self startElement:(UnorderedList new).
!

ulEnd
    self assert:(currentElement isUnorderedList).
    self endElement
! !

!TreeBuilder methodsFor:'instance release'!

release
    super release.
    currentForm := nil.
! !

!TreeBuilder methodsFor:'obsolete'!

display:aDisplayStringOrBoolean
    "arg must be one of true, false or the strings: 'block', 'none', 'inline-block' "

    |displayString|

    self obsoleteMethodWarning.
    "sr: no longer supported, 
        - does not work in all browsers!!
        - this mechanism to set style values, which will be interpreted by the browsers as they like,
            is a very ugly solution

     implement this via css class!!"

    aDisplayStringOrBoolean isBoolean ifTrue:[
        aDisplayStringOrBoolean ifTrue:[
            displayString := 'inline-block'.
        ] ifFalse:[
            displayString := 'none'.
        ].
    ] ifFalse:[
        displayString := aDisplayStringOrBoolean.
    ].

    self style:'display: ',displayString.

    "Created: / 09-08-2010 / 10:37:32 / sr"
!

visibility:visibilityStringOrBooleanOrBlock
    "arg must be one of true, false or the strings: 'visible', 'hidden', 'collapse'"

    |visibilityString|

    self obsoleteMethodWarning.
    "sr: no longer supported, 
        - does not work in all browsers!!
        - this mechanism to set style values, which will be interpreted by the browsers as they like,
            is a very ugly solution

     implement this via css class!!"

    visibilityStringOrBooleanOrBlock == true ifTrue:[
        visibilityString := 'visible'
    ] ifFalse:[
        visibilityStringOrBooleanOrBlock == false ifTrue:[
            visibilityString := 'collapse'
        ] ifFalse:[
            visibilityString := visibilityStringOrBooleanOrBlock
        ]
    ].

    self style:'visibility: ',visibilityString.

    "Created: / 02-07-2010 / 15:06:22 / cg"
    "Modified (comment): / 23-02-2012 / 10:11:26 / sr"
! !

!TreeBuilder methodsFor:'private'!

makeScriptActionText:scriptActionText
    "called for onClick: etc. arguments.
     redefinable to allow wrapping of block arguments in more intelligent treeBuilders"

    ^ scriptActionText

    "Created: / 30-06-2010 / 21:29:22 / cg"
! !

!TreeBuilder methodsFor:'setup'!

beginDocument
    self beginWith:(Document new).

    "
     |b|

     b := self new beginDocument.
     b table.
     b tr.
     b td.
     b tdEnd.
     b td.
     b tdEnd.
     b trEnd.
     b tableEnd.
     b
    "

    "Created: / 01-02-2007 / 21:09:14 / cg"
! !

!TreeBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__TreeBuilder.st,v 1.101 2012/02/23 09:10:13 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/HTML__TreeBuilder.st,v 1.101 2012/02/23 09:10:13 sr Exp $'
! !
