"
 COPYRIGHT (c) 2003 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/webServer/htmlTree/tests' }"

"{ NameSpace: HTML }"

TestCase subclass:#Testcase_Table
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-Testcase'
!

!Testcase_Table class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!Testcase_Table methodsFor:'tests'!

test001CreateEmptyTable
    "Create an empty table.
     The table shall not create
     subelements automatically,
     on creation with method new!!"

    |table|

    table := Table new.

    self assert:(table children isEmpty).
    self assert:(table attributes isNil).
    self assert:(table caption isNil).
    self assert:(table head isNil).
    self assert:(table body isNil).
    self assert:(table foot isNil).

    "
    self new test001CreateEmptyTable.
    self run:#test001CreateEmptyTable.
    "
!

test010CaptionConstruction
    "Ensure that method: 'getHead' is building a head."

    |table created|

    table := Table new.

    "The head shall not be automatically created before."
    self assert:(table head isNil).

    "The method 'getHead' must always deliver a head,..."
    created := table getCaption.

    self assert:(created notNil).

    "... and use it later on as head."
    self assert:(created == table caption).
    self assert:(created class == Caption).

    "Ensure correct parent."
    self assert:(created parent == table).

    "
    self new test010CaptionConstruction.
    self run:#test010CaptionConstruction.
    "
!

test020HeadSetting
    "Ensure that table takes and remembers a set head."

    |table created head|

    table := Table new.

    "The head shall not be automatically built before."
    self assert:(table head isNil).

    created := TableHead new.

    table head:created.

    head := table head.
    head parent:table.

    self assert:(head notNil).

    "Is it the one we set?"
    self assert:(created == table head).

    "Ensure correct parent."
    self assert:(head parent == table).

    "
    self new test020HeadSetting.
    self run:#test020HeadSetting.
    "
!

test021HeadConstruction
    "Ensure that method: 'getHead' is building a head."

    |table head|

    table := Table new.

    "The head shall not be automatically built before."
    self assert:(table head isNil).

    "The method 'getHead' must always deliver a head,..."
    head := table getHead.

    self assert:(head notNil).

    "... and use it later on as head."
    self assert:(head == table head).
    self assert:(head class == TableHead).

    "Ensure correct parent."
    self assert:(head parent == table).

    "
    self new test021HeadConstruction.
    self run:#test021HeadConstruction.
    "
!

test022HeaderCellIndexedConstruction
    "Ensure that method: 'headCellAt:put:' is building a head and the needed header cells."

    |size table headCell|
    size := 3.

    table := Table new.

    headCell := table headCellAt:size.
    self assert:(headCell isNil).

    "The method: 'headCellAt:' shall
     not accidentally create a head."
    self assert:(table head isNil).

    headCell := TableHeaderCell new.

    "Set a head cell, build automatically head and needed header cells."
    table headCellAt:size put:headCell.

    "There must have a head now."
    self assert:(table head notNil).

    1 to:size do:[
	:index|
	|cell|
	cell := table headCellAt:index.

	"The constructed head cell must be exactly be the
	 last one and may not occur in any other place."
	index == size
	    ifTrue: [self assert:(cell == headCell)]
	    ifFalse:[self assert:((cell == headCell) not)].

	self assert:(cell class == TableHeaderCell).
	self assert:(cell parent parent== table).
    ].


    "
    self new test022HeaderCellIndexedConstruction.
    self run:#test022HeaderCellIndexedConstruction.
    "
!

test023HeaderCellsAsArray
    "Ensure that method: 'footCellAt:put:' is building a head and the needed header cells."

    |table head cell|

    table := Table new.

    table head:#('Hans' 'Otto' 'Karl' 'Kurt' 'Gerd' 'John' 'Bert').


    head := table head.

    self assert:(head class == TableHead).
    self assert:(head parent == table).


    cell := head cellAt:4.

    self assert:(cell class == TableHeaderCell).
    self assert:(cell parent == head).

    "
    self new test032FootCellsAsArray.
    self run:#test032FootCellsAsArray.
    "
!

test030FootConstruction
    "Check for getFoot building a foot."

    |table created|

    table := Table new.

    "The foot shall not be automatically created before."
    self assert:(table foot isNil).

    "The method 'getFoot' must always deliver a foot,..."
    created := table getFoot.
    self assert:(created notNil).

    "...and use it later on as foot:"
    self assert:(created class == TableFoot).

    "Ensure correct parent."
    self assert:(created parent == table).

    "
    self new test030FootConstruction.
    self run:#test030FootConstruction.
    "
!

test031FootCellIndexedConstruction
    "Ensure that method: 'footCellAt:put:' is building a head and the needed header cells."

    |size table footCell|
    size := 3.

    table := Table new.

    footCell := table footCellAt:size.
    self assert:(footCell isNil).

    "The method: 'footCellAt:' shall
     not accidentally create a foot."
    self assert:(table foot isNil).

    footCell := TableHeaderCell new.

    "The table shall not have a foot at this time."
    self assert:(table foot isNil).

    "Set a foot cell, build automatically foot and needed footer cells."
    table footCellAt:size put:footCell.

    "There must have a foot now."
    self assert:(table foot notNil).

    1 to:size do:[
	:index|
	|cell|
	cell := table footCellAt:index.

	"The constructed head cell must be exactly be the
	 last one and may not occur in any other place."
	index == size
	    ifTrue: [self assert:(cell == footCell)]
	    ifFalse:[self assert:((cell == footCell) not)].

	self assert:(cell class == TableHeaderCell).
	self assert:(cell parent parent== table).
    ].


    "
    self new test031FootCellIndexedConstruction.
    self run:#test031FootCellIndexedConstruction.
    "
!

test032FootCellsAsArray
    "Ensure that method: 'footCellAt:put:' is building a head and the needed header cells."

    |table foot cell|

    table := Table new.

    table foot:#('Hans' 'Otto' 'Karl' 'Kurt' 'Gerd' 'John' 'Bert').


    foot := table foot.

    self assert:(foot class == TableFoot).
    self assert:(foot parent == table).


    cell := foot cellAt:4.

    self assert:(cell class == TableHeaderCell).
    self assert:(cell parent == foot).

    "
    self new test032FootCellsAsArray.
    self run:#test032FootCellsAsArray.
    "
!

test050BodyConstruction
    "Ensure that method: 'getBody' is building a body."

    |table created|

    table := Table new.

    "The body shall not be automatically created before."
    self assert:(table body isNil).


    "The method 'getBody' must always deliver a body,..."
    created := TableBody new.

    table addBody:created.

    self assert:(table body notNil).

    self assert:(created == table body).

    "Ensure correct parent."
    self assert:(created parent == table).

    "
    self new test050BodyConstruction.
    self run:#test050BodyConstruction.
    "
!

test051BodyAutoConstruction
    "Ensure that method: 'getBody' is building a body."

    |table created|

    table := Table new.

    "The body shall not be automatically created before."
    self assert:(table body isNil).


    "The method 'getBody' must always deliver a body,..."
    created := table getBody.

    self assert:(created notNil).

    "...and use it later on as body:"
    self assert:(created == table body).
    self assert:(created class == TableBody).

    "Ensure correct parent."
    self assert:(created parent == table).

    "
    self new test051BodyConstruction.
    self run:#test051BodyConstruction.
    "
!

test052BodyIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |size table created|
    size  := 5.

    table := Table new.

    "Ensure that nothing is autocreated before so far."
    self assert:(table body isNil).

    "The method 'getBodyAt:' must always deliver a body,..."
    created := table getBodyAt:size.

    self assert:(created notNil).

    "... and use it later on as body."
    self assert:(created == (table bodyAt:size)).

    "Ensure correct parent."
    self assert:(created parent == table).

    "All inbetween automatically created objects must be bodies, too."
    1 to:size do:[
	:index|
	|body|
	body := table bodyAt:index.

	"The constructed body must be exactly be the
	 last one and may not occur in any other place."
	index == size
	    ifTrue: [self assert:(body == created)]
	    ifFalse:[self assert:((body == created) not)].

	self assert:(body class == TableBody).
	self assert:(body parent == table).
    ].


    "
    self new test052BodyIndexedConstruction.
    self run:#test052BodyIndexedConstruction.
    "
!

test053BodyCellIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |bodyIndex rowIndex cellIndex table cell|
    bodyIndex  := 3.
    rowIndex   := 4.
    cellIndex  := 5.

    table := Table new.

    cell := table bodyAt:bodyIndex rowAt:rowIndex cellAt:cellIndex.

    self assert:(cell isNil).

    "The method 'bodyAt:rowAt:cellAt:'
     shall not accidentally create a body."
    self assert:(table body isNil).

    cell := TableDataCell new.

    table bodyAt:bodyIndex rowAt:rowIndex cellAt:cellIndex put:cell.

    self assert:(cell == (table bodyAt:bodyIndex rowAt:rowIndex cellAt:cellIndex)).
    self assert:(cell parent parent parent == table).

    cell parent children do:[
	:brother|
	self assert:(brother class == TableDataCell).
    ].

    cell parent parent children do:[
	:brother|
	self assert:(brother class == TableRow).
    ].

    cell parent parent parent children do:[
	:brother|
	self assert:(brother class == TableBody).
    ].

    "
    self new test053BodyCellIndexedConstruction.
    self run:#test053BodyCellIndexedConstruction.
    "
!

test054BodyCellIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |cellIndex table cell|
    cellIndex  := 6@7.

    table := Table new.

    cell := table cellAt:cellIndex.

    self assert:(cell isNil).

    "The method 'bodyAt:rowAt:cellAt:'
     shall not accidentally create a body."
    self assert:(table body isNil).

    cell := TableDataCell new.

    table cellAt:cellIndex put:cell.

    self assert:(cell == (table cellAt:cellIndex)).
    self assert:(cell parent parent parent == table).

    "
    self new test054BodyCellIndexedConstruction.
    self run:#test054BodyCellIndexedConstruction.
    "
!

test055BodyCellIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |rowIndex cellIndex table cell|
    rowIndex  := 4.
    cellIndex := 5.

    table := Table new.

    cell  := table rowAt:rowIndex cellAt:cellIndex.

    self assert:(cell isNil).

    "The method 'bodyAt:rowAt:cellAt:'
     shall not accidentally create a body."
    self assert:(table body isNil).

    cell := TableDataCell new.

    table rowAt:rowIndex cellAt:cellIndex put:cell.

    self assert:(cell == (table rowAt:rowIndex cellAt:cellIndex)).
    self assert:(cell parent parent parent == table).


    "
    self new test055BodyCellIndexedConstruction.
    self run:#test055BodyCellIndexedConstruction.
    "
!

test056BodyRowIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |bodyIndex rowIndex table row|
    bodyIndex  := 3.
    rowIndex   := 4.

    table := Table new.

    row   := table bodyAt:bodyIndex rowAt:rowIndex.

    self assert:(row isNil).

    "The method 'bodyAt:rowAt:cellAt:'
     shall not accidentally create a body."
    self assert:(table body isNil).

    row := TableRow new.

    table bodyAt:bodyIndex rowAt:rowIndex put:row.

    self assert:(row == (table bodyAt:bodyIndex rowAt:rowIndex)).
    self assert:(row parent parent == table).


    "
    self new test056BodyRowIndexedConstruction.
    self run:#test056BodyRowIndexedConstruction.
    "
!

test057BodyRowIndexedConstruction
    "Ensure that method: 'getBodyAt:' is building bodies up to an index."

    |bodyIndex rowIndex table row|
    bodyIndex  := 3.
    rowIndex   := 4.

    table := Table new.

    row   := table bodyAt:bodyIndex rowAt:rowIndex.

    self assert:(row isNil).

    "The method 'bodyAt:rowAt:cellAt:'
     shall not accidentally create a body."
    self assert:(table body isNil).

    row := TableRow new.

    table bodyAt:bodyIndex rowAt:rowIndex put:row.

    self assert:(row == (table bodyAt:bodyIndex rowAt:rowIndex)).
    self assert:(row parent parent == table).


    "
    self new test057BodyRowIndexedConstruction.
    self run:#test057BodyRowIndexedConstruction.
    "
!

test060RowCellSetting
    ""

    |body created row|

    body := TableBody new.

    created := TableRow new.

    body addRow:created.

    row := body rowAt:1.

    self assert:(created == row).

    "Ensure correct parent."
    self assert:(created parent == body).

    "
    self new test060RowCellSetting.
    self run:#test060RowCellSetting.
    "
!

test061RowCellSettingIndexed
    ""

    |body rowIndex created row|

    body := TableBody new.

    rowIndex := 21.

    self assert:((body rowAt:rowIndex) isNil).

    created := TableRow new.

    body rowAt:rowIndex put:created.

    row := body rowAt:rowIndex.

    self assert:(created == row).

    "Ensure correct parent."
    self assert:(created parent == body).

    "
    self new test061RowCellSettingIndexed.
    self run:#test061RowCellSettingIndexed.
    "
!

test062RowCellSettingIndexed
    ""

    |body rowIndex cellIndex created cell|

    body := TableBody new.

    rowIndex  := 4.
    cellIndex := 3.

    self assert:((body rowAt:rowIndex cellAt:cellIndex) isNil).

    created := TableDataCell new.

    body rowAt:rowIndex cellAt:cellIndex put:created.

    cell := body rowAt:rowIndex cellAt:cellIndex.

    self assert:(created == cell).

    "Ensure correct parent."
    self assert:(created parent parent == body).

    "
    self new test062RowCellSettingIndexed.
    self run:#test062RowCellSettingIndexed.
    "
!

test070CellSettingIndexed
    ""

    |row created cell cellIndex|

    row := TableRow new.

    cellIndex := 3.

    cell := row cellAt:cellIndex.

    self assert:(cell isNil).

    created := TableDataCell new.

    row cellAt:cellIndex put:created.

    cell := row cellAt:cellIndex.

    self assert:(created == cell).

    "Ensure correct parent."
    self assert:(created parent == row).

    "
    self new test070CellSettingIndexed.
    self run:#test070CellSettingIndexed.
    "
!

test071CellAdding
    ""

    |row created cell|

    row := TableRow new.

    created := TableDataCell new.

    row addCell:created.

    cell := row cellAt:1.

    self assert:(cell == created).
    self assert:(cell parent == row).

    "
    self new test071CellAdding.
    self run:#test071CellAdding.
    "
! !

!Testcase_Table class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/tests/HTML_Testcase_Table.st,v 1.3 2007/03/13 14:29:06 cg Exp $'
! !
