"
 COPYRIGHT (c) 2003 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/webServer/htmlTree/tests' }"

"{ NameSpace: HTML }"

TestCase subclass:#Testcase_Visitor
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Documents-Testcase'
!

!Testcase_Visitor class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2003 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
     documentation to be added.

    [author:]
	Martin Walser

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!Testcase_Visitor methodsFor:'tests'!

test001VisitorsVisitingNameConformsClassName
    "Check if the encoder for visitors has a
     correct visiting method for all elements."


    AbstractHTMLElement allConcreteSubclasses do:[
	:class|
	|methodName|

	methodName := 'visit', class nameWithoutPrefix, ':'.

	self assert:(Visitor implements:methodName asSymbol).
    ].

    "
    self new test001VisitorsVisitingNameConformsClassName.
    self run:#test001VisitorsVisitingNameConformsClassName
    "
!

test011EncoderHasVisitingMethodForAllElements
    "Check if the encoder for visitors has a
     correct visiting method for all elements."

    |encoder|

    encoder := Encoder new.

    AbstractHTMLElement allConcreteSubclasses do:[:class|
        |el|

        class ~~ Element ifTrue:[
            el := class new.
            "This '#visit:' will call the concrete
             '#visit...'-method in the encoder."
            el visit: encoder.
        ].
    ].

    "
    self new test011EncoderHasVisitingMethodForAllElements.
    self run:#test011EncoderHasVisitingMethodForAllElements
    "
!

test021VisitingNil
    "Check if it is possible to visit nil with the visitor.
     The visitor shall resist this without error."

    Visitor new visit:nil.

    "
    self new test021VisitingNil.
    self run:#test021VisitingNil
    "
!

test022EncodeString
    "Ensure that an encoder can visit simply
     a string without raising an error and
     that the given string will be printed
     completely unchanged."

    |string encoder|
    string := 'I will survive!!'.

    encoder := Encoder new.

    encoder visit:string.

    self assert:(string = encoder htmlString).

    "
    self new test022EncodeString.
    self run:#test022EncodeString
    "
!

test031VisitorInterface
    "Check if the elemnts dispatch to the right visitor method and provide a correct tagName"

    |allElementClasses|

    allElementClasses := AbstractHTMLElement allSubclasses
			select:[:cls | cls class implements:#tagName].
    allElementClasses
	do:[:eachClass |
	    |exception|

	    exception := #(
				(#'HTML::Image'  'img' )
				(#'HTML::Anchor' 'a'   )
				(#'HTML::Bold'   'b'   )
				(#'HTML::Italic' 'i'   )
				(#'HTML::DefinitionDescription' 'dd')
				(#'HTML::DefinitionList'        'dl')
				(#'HTML::DefinitionTerm'        'dt')
				(#'HTML::Paragraph'             'p')
				(#'HTML::Document'              'html')
				(#'HTML::Break'                 'br')
				(#'HTML::ListItem'              'li')
				(#'HTML::OrderedList'           'ol')
				(#'HTML::UnorderedList'         'ul')
				(#'HTML::Underlined'            'u')
				(#'HTML::Heading'               'h%1')
				(#'HTML::Heading1'              'h1')
				(#'HTML::Heading2'              'h2')
				(#'HTML::Heading3'              'h3')
				(#'HTML::Heading4'              'h4')
				(#'HTML::Heading5'              'h5')
				(#'HTML::Heading6'              'h6')
				(#'HTML::HorizontalRule'        'hr')
				(#'HTML::TableBody'             'tbody')
				(#'HTML::TableHead'             'thead')
				(#'HTML::TableRow'              'tr')
				(#'HTML::TableDataCell'         'td')
				(#'HTML::TableHeaderCell'       'th')
				(#'HTML::TableFoot'             'tfoot')

			  ) detect:[:pair | eachClass name == pair first] ifNone:nil.

	    exception notNil ifTrue:[
		self assert:(eachClass tagName = exception second).
	    ] ifFalse:[
		self assert:(eachClass tagName = eachClass nameWithoutPrefix asLowercase).
	    ].
	].

    allElementClasses
	do:[:eachClass |
	    |exception element dummyVisitor expectedVisitorMessage visited visitArg|

	    exception := #(
				(#'HTML::Heading1'  #'visitHeading:' )
				(#'HTML::Heading2'  #'visitHeading:' )
				(#'HTML::Heading3'  #'visitHeading:' )
				(#'HTML::Heading4'  #'visitHeading:' )
				(#'HTML::Heading5'  #'visitHeading:' )
				(#'HTML::Heading6'  #'visitHeading:' )
			  ) detect:[:pair | eachClass name == pair first] ifNone:nil.
	    exception notNil ifTrue:[
		expectedVisitorMessage := exception second.
	    ] ifFalse:[
		expectedVisitorMessage := ('visit' , eachClass nameWithoutPrefix , ':') asSymbol.
	    ].
	    dummyVisitor := Plug new.
	    dummyVisitor respondTo:expectedVisitorMessage with:[:arg | visitArg := arg. visited := true.].

	    element := eachClass new.
	    visited := false.

	    [
		element visit:dummyVisitor.
	    ] on:Error do:[
		self signalFailure:'error in visitor protocol'.
	    ].
	    self assert:visited.
	    self assert:(visitArg == element).
	].

    "
    self new test031VisitorInterface.
    self run:#test031VisitorInterface
    "
! !

!Testcase_Visitor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/htmlTree/tests/HTML_Testcase_Visitor.st,v 1.13 2007/03/13 15:15:53 cg Exp $'
! !
