"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

SwikiAction subclass:#AuthorizedSwikiAction
	instanceVariableNames:'authorizer'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

AuthorizedSwikiAction comment:'A Server with a login name and password for the entire Swiki area.  Can be multiple users each with a different password.  Each sees and can modify the whole Swiki area.

To restart an existing Authorized Swiki:

	AuthorizedSwikiAction new restore: ''SWSecure''.

The front page URL is:  http://serverMachine:80/SWSecure.1

To make a completely new one:
	| a s |
	a := Authorizer new.
	a realm: ''SwikiArea''.
	a mapName: ''viki'' password: ''hard2guess'' to: ''viki''.
	AuthorizedSwikiAction setUp: ''SWSecure''.
	s := AuthorizedSwikiAction new restore: ''SWSecure''.
	s authorizer: a.
'
!


!AuthorizedSwikiAction methodsFor:'URL processing'!

authorizer
	"*** Do not use this method to add or delete users!!  The change will not be recorded on the disk!!  Instead call mapName:password:to: in this class.***"
	^authorizer
!

authorizer: anAuthorizer
    "Smash all old name/password pairs with this new set.  Overwrites the file on the disk"

    | fName refStream |

    authorizer _ anAuthorizer.
    name notNil ifTrue:[
	fName _ ServerAction serverDirectory, name, (ServerAction pathSeparator), 
			    'authorizer'.

	refStream _ BinaryObjectStorage onNew:(fName asFilename writeStream).
	refStream nextPut: authorizer; close.
    ]
!

checkAuthorization: request
    ^ authorizer ifNotNil: [authorizer user: request userID request:request].

    "Modified: / 30.7.1999 / 13:44:12 / cg"
!

mapName: nameString password: pwdString to: aPerson
	"Insert/remove the username:password combination into/from the users Dictionary.  *** Use this method to add or delete users!!  If you ask for the authorizer and talk to it, the change will not be recorded on the disk!! ***   We use encoding per RFC1421."

	authorizer mapName: nameString password: pwdString to: aPerson.
	self authorizer: authorizer.    "force it to be written to the disk"
!

process: request
	self checkAuthorization: request.
	^(super process: request).
!

processSpecial: request
    "Let SwikiAction process this with no authorization check."

    ^ (super process: request).
!

restore: nameOfSwiki
    "Read all files in the directory 'nameOfSwiki'.  Reconstruct the url map."

    | fName f refStream |

    super restore: nameOfSwiki.
    name notNil ifTrue:[
	fName _ ServerAction serverDirectory, name, (ServerAction pathSeparator), 'authorizer'.
	f := fName asFilename.
	(f exists and:[f isDirectory not]) ifTrue: [
	    refStream _ BinaryObjectStorage onOld:(f readStream).
	    authorizer _ refStream next.
	    refStream close.
	]
    ].
! !

!AuthorizedSwikiAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/AuthorizedSwikiAction.st,v 1.10 2003/03/26 10:29:01 stefan Exp $'
! !
