"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#Authorizer
	instanceVariableNames:'users realm'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

Authorizer comment:'The Authorizer does user authorization checking. Each instance of authorizer keeps track of the realm that it is authorizing for, and the table of authorized users. An authorizer can be asked to return the user name/symbol associated with a userID (which concatenates the username and password from the HTTP request) with the user: method.
'
!


!Authorizer methodsFor:'authentication'!

mapFrom:aKey to:aPerson
    "Establish a mapping from a RFC 1421 key to a user."

    |oldKey|

    users isNil ifTrue: [ 
	users := Dictionary new 
    ].
    aPerson isNil ifTrue: [ 
	users removeKey:aKey 
    ] ifFalse: [
	oldKey := users keyAtValue:aPerson ifAbsent:nil.
	oldKey notNil ifTrue:[
	    users removeKey:oldKey ifAbsent:nil.
       ].
	users at:aKey put:aPerson 
    ]
!

mapName:nameString password:pwdString to:aPerson
    "Insert/remove the encoding per RFC1421 of the username:password combination into/from the UserMap.  DO NOT call this directly, use mapName:password:to: in your ServerAction class.  Only it knows how to record the change on the disk!!"

    self 
	mapFrom:(HTTPServer encodeRFC1421:(nameString, ':', pwdString))
	to: aPerson
!

user: userId
    "Return the requesting user."

    |u|

    users notNil ifTrue:[
	u := users at: userId ifAbsent:nil.
	u notNil ifTrue:[^ u].
    ].
    HTTPServer unauthorizedFor: realm.
    ^ self error

    "Modified: / 30.7.1999 / 13:45:11 / cg"
!

user:userId request:request
    "validate & return the requesting user."

    |u|

    users notNil ifTrue:[
	u := users at: userId ifAbsent:nil.
	u notNil ifTrue:[^ u].
    ].
    request noAuth:realm.

    "Created: / 30.7.1999 / 13:45:05 / cg"
! !

!Authorizer methodsFor:'realms'!

realm
	^realm
!

realm: aString
	realm := aString
! !

!Authorizer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/Authorizer.st,v 1.10 2003/03/26 10:26:07 stefan Exp $'
! !
