"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

SwikiAction subclass:#CachedSwikiAction
	instanceVariableNames:'cacheDirectory cacheURL pwsURL'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

CachedSwikiAction comment:'CachedSwikiAction caches SwikiAction pages so that they can be served as plain HTML files (no embedded Squeak code) even by a native webServer. 

You must edit three class methods in CachedSwikiAction to get it to serve appropriately.

* CachedSwikiAction class defaultCacheDirectory is where to store cached pages
* CachedSwikiAction class defaultCacheURL is the URL to precede cached pages
* CachedSwikiAction class defaultPWSURL is where the PWS is that can handle editing and searching.

'
!


!CachedSwikiAction class methodsFor:'initialization'!

setUp: named

    self error:'use setUp: fromServer:'.

"/    | newAction |

"/    super setUp: named.
"/    newAction := HTTPServer actionFor: named.
"/    newAction cacheDirectory: (self defaultCacheDirectory).
"/    newAction cacheURL: (self defaultCacheURL).
"/    newAction source: 'cswiki',(ServerAction pathSeparator).
"/    ^ newAction
!

setUp: named fromServer:aHttpServer
    | newAction |

    super setUp: named fromServer:aHttpServer.
    newAction := aHttpServer actionFor: named.
    newAction cacheDirectory: (self defaultCacheDirectory).
    newAction cacheURL: (self defaultCacheURL).
    newAction source: 'cswiki',(ServerAction pathSeparator).
    ^ newAction
! !

!CachedSwikiAction class methodsFor:'services'!

defaultCacheDirectory
	^'Guz 7600:WebSTAR 2.0:'
!

defaultCacheURL
	^'http://guzdial.cc.gatech.edu/'
!

defaultPWSURL
    ^ 'http://guzdial.cc.gatech.edu:8080/'
! !

!CachedSwikiAction methodsFor:'URL processing'!

browse: pageRef from: request
	"Just reply with a page in HTML format"

	| formattedPage |
	formattedPage _ pageRef copy.
	"Make a copy, then format the text."
	formattedPage formatted: (HTMLformatter swikify: pageRef text
			linkhandler: [:link | urlmap
					linkFor: link
					from: request peerName
					storingTo: OrderedCollection new
					page: formattedPage]).
	request reply: (HTMLformatter evalEmbedded: (self fileContents:
source ,'page.html')
			with: formattedPage).
!

generate
	1 to: (urlmap pages size) do: [:ref |
		self generate: (urlmap atID: ref) from: 'Beginning'.].
	self generateRecent.
!

generate: pageRef from: request
    "Just reply with a page in HTML format"

    | formattedPage peer cacheFile file|

    (request isKindOf: HTTPServer) ifFalse: [
	(request isKindOf: String) ifTrue: [
	    peer _ request
	] ifFalse: [
	    peer _ ' '
	]
    ] ifTrue: [
	peer _ request peerName
    ].

    formattedPage _ pageRef copy.
    "Make a copy, then format the text."
    formattedPage 
	formatted: (HTMLformatter 
			swikify: pageRef text
			linkhandler: [:link | urlmap
				    linkFor: link
				    from: peer
				    storingTo: OrderedCollection new]).
    cacheFile _ (self cacheDirectory),(self name),(ServerAction pathSeparator),(pageRef coreID),'.html'.

    cacheFile := cacheFile asFilename.
    (cacheFile exists and:[cacheFile isDirectory not]) ifTrue: [
	cacheFile delete
    ].
    file _ cacheFile writeStream.
    file nextPutAll: (HTMLformatter 
			    evalEmbedded: (self fileContents:source ,'page.html')
			    with: formattedPage).
    file close.

!

generateRecent
	| file |
	file _ FileStream fileNamed: (self cacheDirectory),(self
name),(ServerAction pathSeparator),'recent.html'.
	file nextPutAll: (HTMLformatter evalEmbedded: (self fileContents:
source, 'recent.html')
					with: urlmap recent).
	file close.
!

inputFrom: request
	"Take user's input and respond with a searchresult or store the edit"

    | coreRef page theText |

    coreRef _ request message size < 2
	    ifTrue: ['1']
	    ifFalse: [request message at: 2].
    coreRef = 'searchresult' ifTrue: [
	    "If contains search string, do search"
	    request 
		reply: HTTPServer crlf;
		reply: (HTMLformatter evalEmbedded: (self fileContents: source, 'results.html')
			    with: (urlmap searchFor: (
				    request fields at: 'searchFor' ifAbsent: ['nothing']))).
	    ^ #return
    ].

    (theText _ request fields at: 'text' ifAbsent: [nil]) ifNotNil: [
	"It's a response from an edit, so store the page"
	page _ urlmap atID: coreRef.
	page user: request peerName.  "Address is machine, user only if logged in"
	 page pageStatus = #new ifTrue: [page pageStatus: #standard].
	page _ urlmap
		storeID: coreRef
		text: (self class withSqueakLineEndings:theText)
		from: request peerName.
	self generate: (urlmap atID: coreRef) from: request.
	self generateRecent.

	^ self        "return self means do serve the edited page afterwards"
    ].

    request fields keys do: [:aTag |
	(aTag startsWith: 'text-') ifTrue: [
	    urlmap
		storeID: coreRef
		text: (self class withSqueakLineEndings:(request fields at: aTag))
		insertAt: (aTag copyFrom: 6 to: aTag size).
	    "string"
	    self generate: (urlmap atID: coreRef) from: request.
	    self generateRecent.
	    ^ self
	]
    ].
    "oops, a new kind!!"
    Transcript show: 'Unknown data from client. '; show: request fields printString; cr.
!

pageURL: aPage
	"make the url suited to aPage"
	^(self url),(self name),'/',aPage coreID,'.html'
! !

!CachedSwikiAction methodsFor:'access'!

cacheDirectory
	^cacheDirectory
!

cacheDirectory: directory
	cacheDirectory _ directory
!

cacheURL
	^cacheURL
!

cacheURL: urlString
	cacheURL _ urlString
!

pwsURL
	^pwsURL 
!

pwsURL: urlString
	pwsURL _ urlString      
!

url
	^cacheURL
! !

!CachedSwikiAction methodsFor:'save/restore'!

restore: nameOfSwiki
    super restore: nameOfSwiki.
    self source: 'cswiki',(ServerAction pathSeparator).
    self cacheDirectory: (self class defaultCacheDirectory).
    self cacheURL: (self class defaultCacheURL).
    self pwsURL: (self class defaultPWSURL).
    self generate.
!

restoreNoGen: nameOfSwiki
    super restore: nameOfSwiki.
    self source: 'cswiki',(ServerAction pathSeparator).
    self cacheDirectory: (self class defaultCacheDirectory).
    self cacheURL: (self class defaultCacheURL).
    self pwsURL: (self class defaultPWSURL).
    "self generate."
! !

!CachedSwikiAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/CachedSwikiAction.st,v 1.8 2003/03/26 10:26:28 stefan Exp $'
! !
