"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

ServerAction subclass:#CodeServer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

CodeServer comment:'Return the source code from Smalltalk, as web page text, or as a raw Squeak file chunk.
URLs are of the form:   
	machine:80/smtlk.Point|min;                     <-- NOTE: use ; instead of :
	machine:80/smtlk.{Class}|{selector}
	machine:80/chunk.{Class}|{selector}

	machine:80/smtlk.{Class}|class|{selector}
	machine:80/chunk.{Class}|class|{selector}

	machine:80/smtlk.{Class}|Definition
	machine:80/chunk.{Class}|Definition

	machine:80/smtlk.{Class}|Hierarchy
	machine:80/chunk.{Class}|Hierarchy

	machine:80/smtlk.{Class}|Comment
	machine:80/chunk.{Class}|Comment
NOTE: use ; semicolon instead of : colon in selector names
'
!


!CodeServer methodsFor:'URL processing'!

chunk: request
    "Return Smalltalk source code as a chunk from the changes file.  
	 URL = machine:80/chunk.<className>|<selector>;  
	 URL = machine:80/chunk.<className>|class/<selector>;  
     included are:  
	Point|at;   
	Point|Comment   
	Point|Hierarchy  
	Point|Definition   
	Point|class|x;y;
     Meant to be received by a Squeak client, not a browser.  
     Reply not in HTML"

    | classAndMethod set strm chunk |

    classAndMethod _ request message atPin: 2.
    classAndMethod _ classAndMethod copyReplaceAll: '|' with: ' '.
    classAndMethod _ classAndMethod copyReplaceAll: ';' with: ':'.

    set _ LinkedMessageSet messageList: (Array with: classAndMethod).
    strm _ WriteStream on: (String new: 300).
    strm nextChunkPutWithStyle: (set selectedMessage). "String or text"
    chunk _ strm contents.

    request
	newReplyData;
	replyHeader: HTTPServer success;
	replyHeader: (HTTPServer content:'text/plain');
	reply: chunk.

    "Modified: / 30.7.1999 / 16:11:13 / cg"
!

process: request
    "Return the source code from Smalltalk, as text or as a chunk.
    URLs are of this form.  Each may have 'chunk' or 'smtlk' as the thing after the slash
    machine:80/smtlk.Point|min;
    machine:80/chunk.{Class}|{selector}             
    machine:80/smtlk.{Class}|{selector}
    machine:80/smtlk.{Class}|class|{selector}
    machine:80/smtlk.{Class}|Definition
    machine:80/smtlk.{Class}|Hierarchy
    machine:80/smtlk.{Class}|Comment
    NOTE: use ; semicolon instead of : colon in selector names!!!!!!"

    | coreRef |

    coreRef _ (request message at: 1) asLowercase.
    request 
	newReplyData;
	replyHeader: HTTPServer success; 
	replyHeader: HTTPServer contentHTML.
    Transcript show: 'In process: ', request message printString; cr.
    coreRef = 'smtlk' ifTrue: [^ self smtlk: request].
    coreRef = 'chunk' ifTrue: [^ self chunk: request].

    "failure"
    request 
	reportBadRequest:'expected smtlk.{Class}|{selector} or chunk.{Class}|{selector}'

    "Modified: / 30.7.1999 / 16:03:03 / cg"
!

smtlk: request
    "Return Smalltalk sourcecode in HTML.  
	URL = machine:80/mySwiki.smtlk.Point|min;  
     included are:  
	Point|min;   
	Point|Comment   
	Point|Hierarchy  
	Point|Definition   
	Point|class|x;y;  
     NOTE: use ; instead of : in selector names!!!!!!
    "

    | classAndMethod set |

    classAndMethod _ request message atPin: 2.
    classAndMethod _ classAndMethod copyReplaceAll: '|' with: ' '.
    classAndMethod _ classAndMethod copyReplaceAll: ';' with: ':'.
    set _ LinkedMessageSet messageList: (Array with: classAndMethod).

    request 
	newReplyData;
	replyHeader: HTTPServer success;
	replyHeader: (HTTPServer content:'text/html');
	reply: 
	    (HTMLformatter 
	       evalEmbedded: (self fileContents: 'swiki',(ServerAction pathSeparator),'smtlk.html')
		       with: set).

    "Modified: / 30.7.1999 / 16:09:25 / cg"
! !

!CodeServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/CodeServer.st,v 1.9 2003/03/26 10:28:40 stefan Exp $'
! !
