"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#Comment
	instanceVariableNames:''
	classVariableNames:'CommentsTable'
	poolDictionaries:''
	category:'PWS-Swiki'
!

Comment comment:'A Comment space is like a bulletin board.  It is a web page with a list statements from many different people.  At the bottom there is a form for you to add your own statement.

Anyone may start a new comment page, just by asking for a page with a new key, and there can be any number of pages.  The default Swiki has a page called ''pws'' already created.

The administrator must take special action to save the accumulated comments (Comment saveTo: ''aFileName'').  Comments are not automatically stored on the disk like regular Swiki pages are.  So, for the moment, it is likely that Comments will get lost when the server is restarted.

URLs are of the form 
machine:80/Comment.{commentKey}
machine:80/Comment.{commentKey}.{number}
machine:80/Comment.{commentKey}.note   -- Does this really work???  -tk
machine:80/Comment.{commentKey}.gif'
!


!Comment class methodsFor:'initialization'!

initialize
	CommentsTable := Dictionary new.
!

readIn: filename
	|f|
	f _ ReferenceStream fileNamed: filename.
	CommentsTable _ f next.
	f close.
!

saveTo: filename
	|f|
	f _ ReferenceStream fileNamed: filename.
	f nextPut: CommentsTable.
	f close.
!

setUpExample
    | newDiscussion |

    newDiscussion _ Discussion new.
    newDiscussion title: 'pws'.
    newDiscussion description: 'Here is a space for talking about the Pluggable Web Server.'.
    CommentsTable at: 'pws' put: newDiscussion.

! !

!Comment class methodsFor:'URL processing'!

comments
	^CommentsTable
!

createComment: request
    "Create a new comment from a Web request"
    | newNote newMap |

    request fields isNil ifTrue: [self error: 'No request to create a comment from!!'].
    newNote := Note new.
    newMap := URLmap new.
    newNote author: (request fields at: 'author' ifAbsent: ['Anonymous']).
    newNote title: (request fields at: 'title' ifAbsent: ['Untitled']).
    newNote text: (HTMLformatter swikify: 
	    (request fields at: 'text' ifAbsent: ['Nothing much to say'])
	    linkhandler: [:phrase | newMap linkFor: phrase from: (request peerName) 
		    storingTo: OrderedCollection new]).
    newNote timestamp: (Date today printString),' ',(Time now printString).
    newNote children: OrderedCollection new. "For later addition of threaded comments"
    ^newNote

    "Modified: / 30.7.1999 / 16:14:32 / cg"
!

process: request
    "URLs are of the form Comment.commentKey or
    Comment.commentKey.note of Comment.commentKey.gif.

    If commentKey is accessed but not created, create an empty one.
    If note is accessed, display it."

    | commentKey noteIndex newNote |

    (request message size > 1) ifTrue: [
	commentKey _ request message at: 2
    ] ifFalse: [
	^ request reply: (self showAllComments: CommentsTable)
    ].

    (CommentsTable includesKey: commentKey) ifFalse:
	[CommentsTable at: commentKey put: Discussion new.
	(CommentsTable at: commentKey) title: commentKey.
	(CommentsTable at: commentKey) description: 'Discussion on ' , commentKey
    ].
    request fields isNil ifFalse:[
	"Are there input fields?"
	newNote _ self createComment: request.
	newNote parent: commentKey.
	(CommentsTable at: commentKey)
		addNote: newNote.
	newNote url: ('Comment.',commentKey,'.',
		(CommentsTable at: commentKey) notes size printString)
    ].
    request message size > 2 ifTrue:[
	"There's a note reference or a request for a status image"
	noteIndex _ request message at: 3.
	noteIndex asUppercase = 'GIF' ifTrue: [
	    request 
		replyHeader: (HTTPServer success);
		replyHeader: (HTTPServer content:'image/gif');
		reply: (HTMLformatter textToGIF:
		    (CommentsTable at: commentKey) status)
	] ifFalse: [
	    request reply: (self showNote:((CommentsTable at: commentKey)
				at:(Number readFrom:noteIndex onError:1)))
	]
    ] ifFalse: [
	request reply: (self showComment: (CommentsTable at: commentKey))
    ]

    "Modified: / 30.7.1999 / 16:12:54 / cg"
!

showAllComments: aDictionaryOfCommentSpaces
    | fileName |

    fileName := (ServerAction serverDirectory) , 'ShowAllComments.html'.
    ^ HTMLformatter 
	evalEmbedded: (FileStream fileNamed: fileName) contentsOfEntireFile 
	with: aDictionaryOfCommentSpaces

    "Modified: / 30.7.1999 / 16:13:48 / cg"
!

showComment: aComment
    | fileName |

    fileName := (ServerAction serverDirectory) , 'ShowComment.html'.
    ^ HTMLformatter 
	evalEmbedded: (FileStream fileNamed: fileName) contentsOfEntireFile 
	with: aComment.

    "Modified: / 30.7.1999 / 16:13:29 / cg"
!

showNote: aNote
    | fileName |

    fileName := (ServerAction serverDirectory) , 'ShowNote.html'.
    ^ HTMLformatter 
	evalEmbedded: (FileStream fileNamed: fileName) contentsOfEntireFile 
	with: aNote.

    "Modified: / 30.7.1999 / 16:13:20 / cg"
! !

!Comment class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/Comment.st,v 1.10 2003/03/26 10:30:43 stefan Exp $'
! !

Comment initialize!
