"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#HTMLformatter
	instanceVariableNames:'formattingBlock specialCharacter'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

HTMLformatter comment:'HTMLformatter class (instances don''t know anything) knows alot about HTML formatting: Creating forms, pages, different widgets, etc.  It also knows how to process Smalltalk embedded within an HTML document.
'
!


!HTMLformatter class methodsFor:'formatting'!

checkbox: buttonname value: b 
	^ '<INPUT TYPE="checkbox" NAME="' , buttonname , '"' , (b
			ifTrue: [' CHECKED ']
			ifFalse: ['']) , '>'
!

formFooter
    "Write the standard footer for a form."

    self reply: '<hr><input type=reset value="Reset"> <input type=submit value="OK"></form>'
!

formHeader: title For: aReference
	"Write the standard header for a page and form for editing anObject."

	self
		title: title;
		reply: '<form action="',aReference,'" method=post><table>'
!

graphic: f
	^self graphic: f standIn: 'Picture' alignment: 'right'
!

graphic: f standIn: s alignment: a 
	^ '<IMG SRC="' , f , '" ALT="' , s , '" ALIGN=' , a printString , '>'
!

hiddenName: n value: v 
	^ '<INPUT TYPE="hidden" NAME="' , n , '" VALUE="' , v printString , '">'
!

linkTo: url label: label
	^'<A HREF="',url,'">',label,'</A>'.
!

select: n values: values selection: selection size: size 
	^ self
		select: n
		values: values
		selections: (Array with: selection)
		size: size
		multiple: false
!

select: buttonname values: values selections: selections size: size multiple: multiple 
	| stream |
	stream _ WriteStream on: ''.
	stream nextPutAll: '<SELECT NAME="' , buttonname , '"' , (multiple
				ifTrue: [' MULTIPLE']
				ifFalse: ['']) , ' SIZE=' , size printString , '>';
	 cr.
	values do: 
		[:value | 
		stream nextPutAll: '<OPTION'.
		(selections includes: value)
			ifTrue: [stream nextPutAll: ' SELECTED'].
		stream nextPutAll: '>'.
		stream nextPutAll: value;
		 cr].
	stream nextPutAll: '</SELECT>'.
	^ stream contents
!

submit: label
	^ self submit: 'submit' label: label
!

submit: buttonName label: v 
	^ '<INPUT TYPE="submit" NAME="' , buttonName , '" VALUE="' , v , '">'
!

text: fieldName
	^ self text: fieldName value: '' length: 80.
!

text: fieldName value: v length: l 
	^ '<INPUT TYPE="text" NAME="' , fieldName , '" VALUE="' , v , '" MAXLENGTH=' , l printString , '>'
!

textArea: fieldName
	^ (self textAreaStart: fieldName rows: 15 cols: 70), self textAreaEnd
!

textArea: fieldName value: value
	^ (self textAreaStart: fieldName rows: 15 cols: 70), value, self textAreaEnd
!

textAreaEnd
	^ '</TEXTAREA>'
!

textAreaStart: fieldName rows: rows cols: cols
       ^ '<TEXTAREA NAME="', fieldName, '" ROWS=',rows,' COLS=',cols,'>' 
! !

!HTMLformatter class methodsFor:'note support'!

show: aCollection
	"Return an HTML list to reference an ordered collection of text. Used in Chat application"
	| ret |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>' ; cr.
	1 to: aCollection size do: [:each |
		ret nextPutAll: '<li>'.
		ret nextPutAll: (aCollection at: each).].
	ret nextPutAll: '</ul>'.
	^ret contents
!

showList: aCollection
	"Return an HTML list to reference an ordered collection of notes. 
	prefix.index will be the reference URL for the note. Used in Comment 
	application. "
	| ret aNote |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>';
	 cr.
	1 to: aCollection size do: 
		[:each | 
		ret nextPutAll: '<li>'.
		aNote _ aCollection at: each.
		ret nextPutAll: (self linkTo: (aNote url) label: aNote title).
		ret nextPutAll: ' ' , aNote author , ' ' , aNote timestamp;
		 cr.
		aNote children size > 0 ifTrue: [ret nextPutAll: (self showList: aNote children)]].
	ret nextPutAll: '</ul>'.
	^ ret contents
!

showList: aCollection parentPrefix: prefix
	"Return an HTML list to reference an ordered collection of notes. prefix.parent.index will be the reference URL for the note."
	| ret aNote |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>' ; cr.
	1 to: aCollection size do: [:each |
		ret nextPutAll: '<li>'.
		aNote := aCollection at: each.
		ret nextPutAll: (self linkTo: (prefix , aNote parent, '.', each printString) label: (aNote title)), '-'.
		ret nextPutAll: (self linkTo: (aNote author homepage) label: aNote author name)
			,'-',aNote timestamp ; cr.
		aNote children size > 0 ifTrue: 
			[ret nextPutAll: (self showList: aNote children prefix: (prefix , each printString))].].
	ret nextPutAll: '</ul>'.
	^ret contents
!

showList: aCollection prefix: prefix
	"Return an HTML list to reference an ordered collection of notes. prefix.index will be the reference URL for the note. Used in Comment application."
	| ret aNote |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>' ; cr.
	1 to: aCollection size do: [:each |
		ret nextPutAll: '<li>'.
		aNote := aCollection at: each.
		ret nextPutAll: (self linkTo: (prefix , each printString) label: (aNote title)).
		ret nextPutAll: ' ',aNote author,' ',aNote timestamp ; cr.
		aNote children size > 0 ifTrue: 
			[ret nextPutAll: (self showList: aNote children prefix: (prefix , each printString))].].
	ret nextPutAll: '</ul>'.
	^ret contents
!

showListWithAuthors: aCollection
	"Return an HTML list to reference an ordered collection of notes. 
	prefix.index will be the reference URL for the note. Special version for 
	linking to author's homepage, if any."
	| ret aNote |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>';
	 cr.
	1 to: aCollection size do: 
		[:each | 
		ret nextPutAll: '<li>'.
		aNote _ aCollection at: each.
		ret nextPutAll: (self linkTo: (aNote url) label: aNote title)
				, '-'.
		ret nextPutAll: (self linkTo: aNote author homepage label: aNote author name)
				, '-' , aNote timestamp;
		 cr.
		aNote children size > 0 ifTrue: [ret nextPutAll: (self showListWithAuthors: aNote children)]].
	ret nextPutAll: '</ul>'.
	^ ret contents
!

showListWithAuthors: aCollection prefix: prefix
	"Return an HTML list to reference an ordered collection of notes. prefix.index will be the reference URL for the note. Special version for linking to author's homepage, if any."
	| ret aNote |
	ret _ WriteStream on: String new.
	ret nextPutAll: '<ul>' ; cr.
	1 to: aCollection size do: [:each |
		ret nextPutAll: '<li>'.
		aNote := aCollection at: each.
		ret nextPutAll: (self linkTo: (prefix , each printString) label: (aNote title)),'-'.
		ret nextPutAll: (self linkTo: (aNote author homepage) label: aNote author name)
			,'-',aNote timestamp ; cr.
		aNote children size > 0 ifTrue: 
			[ret nextPutAll: (self showList: aNote children prefix: (prefix , each printString))].].
	ret nextPutAll: '</ul>'.
	^ret contents
! !

!HTMLformatter class methodsFor:'pages and forms'!

endForm
	^ '</FORM>'
!

endPage
	| stream |
	stream _ WriteStream on: ''.
	stream nextPutAll: '</BODY>';cr;
	 nextPutAll: '</HTML>';cr.
	^stream contents
!

startForm: action
	| stream |
	stream _ WriteStream on: ''.
	stream nextPutAll: '<FORM METHOD="POST" ACTION="', action, '">';
	 cr.
	^ stream contents
!

startPage: title
	| stream |
	stream _ WriteStream on: ''.
	stream nextPutAll: '<!!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">'; cr;
	 nextPutAll: '<HTML>';cr;
	 nextPutAll: '<HEAD>';cr;
	 nextPutAll: '<TITLE>'; nextPutAll: title; nextPutAll: '</TITLE>';cr;
	 nextPutAll: '<BODY>'; cr. 
	^ stream contents
! !

!HTMLformatter class methodsFor:'translating'!

evalEmbedded: stringOrStream with: request
    | formatter |

    formatter _ self forEvaluatingEmbedded: stringOrStream.
    ^ formatter format: request

    "Modified: / 30.7.1999 / 17:50:31 / cg"
!

evalEmbedded: string with: request unlessContains: dangerSet
    dangerSet do: [:each | 
	(string includesSubstring: each caseSensitive: false) ifTrue: [
	    ^ 'Unsafe code!!'
	]
    ].
    ^ self evalEmbedded: string with: request
!

fixEndings: aStringOrStream
    | sourceStream targetStream aLine |

    (aStringOrStream isKindOf: Stream)
	    ifTrue: [sourceStream := aStringOrStream]
	    ifFalse: [sourceStream := ReadStream on: aStringOrStream].
    targetStream := ReadWriteStream on: String new. 
    [sourceStream atEnd] whileFalse:[
	aLine := sourceStream upTo: (Character linefeed).
	targetStream nextPutAll: aLine.
	targetStream nextPut: Character cr.
    ].
    ^ targetStream
!

fixForIE: text
    | inEmbed ch targetStream sourceStream newText|

    inEmbed := false.
    targetStream := WriteStream on: String new.
    sourceStream := ReadStream on: text.
    [sourceStream atEnd] whileFalse:[
	ch := sourceStream next.

	ch == $? ifTrue: [
	    sourceStream peekOrNil == $> ifTrue:[
		targetStream nextPutAll: '?>'.
		sourceStream next.
		inEmbed := false.
	    ] ifFalse:[
		targetStream nextPut: ch
	    ]
	] ifFalse:[
	    inEmbed ifTrue:[
		 targetStream nextPut: ch
	    ] ifFalse:[
		ch == $> ifTrue: [
		    targetStream nextPutAll: '&gt;'
		] ifFalse: [
		    ch == $< ifTrue: [
			sourceStream peekOrNil == $? ifTrue:[
			    targetStream nextPutAll: '<?'.
			    sourceStream next.
			    inEmbed := true.
			] ifFalse:[
			    targetStream nextPutAll: '&lt;'
			].
		    ] ifFalse: [
			ch == $& ifTrue: [
			    targetStream nextPutAll: '&amp;'
			] ifFalse:[
			    targetStream nextPut: ch
			]
		    ]
		].
	    ].
	].
    ].
    newText := targetStream contents.
    newText := newText withoutTrailingSeparators.

    ^ newText

    "Modified: / 30.7.1999 / 19:26:35 / cg"
!

forEvaluatingEmbedded: stringOrStream
    "stringOrStream is text with <?expr?> expressions intermingled.
     This creates a HTLMLformatter instance which will substitute the <?expr?>
     expressions with the value of the argument (named request), and which
     leaves all other text in stringOrStream alone"

    | block blockStream sourceStream doingEval ch |

    blockStream _ WriteStream on: String new.
    blockStream nextPutAll: '[:request :output | output nextPutAll: '''.

    (stringOrStream isKindOf: Stream)
	    ifTrue: [sourceStream := stringOrStream]
	    ifFalse: [sourceStream := ReadStream on: stringOrStream].

    doingEval _ false.

    [sourceStream atEnd] whileFalse:  [
	ch := sourceStream next.
	(doingEval not 
	and: [ ch == $<  
	and: [ sourceStream peek == $? ]]) ifTrue: [
	    "beginning of an <?...?> expression"
	    blockStream nextPutAll: '''.  output nextPutAll: ['.
	    sourceStream next.  "Skip the ?"
	    doingEval _ true
	] ifFalse: [
	    (doingEval 
	    and: [ ch == $? 
	    and: [ sourceStream peek == $> ]]) ifTrue: [
		"end of a <?...?> expression"
		blockStream nextPutAll: '] value printString.  output nextPutAll: '''.
		sourceStream next.  "Skip the >"
		doingEval _ false.
	    ] ifFalse: [
		"normal char"
		blockStream nextPut: ch.
		(doingEval not 
		and: [ ch == $' ]) ifTrue: [
		    "double $' marks"  
		    blockStream nextPut: $'
		] 
	    ] 
	] 
    ].

    "end the block"
    doingEval
	ifTrue: [ blockStream nextPutAll: '] value printString' ]
	ifFalse: [ blockStream nextPutAll: '''' ].
    blockStream nextPutAll: ']'.

    Class nameSpaceQuerySignal answer:(self nameSpace)
    do:[
	|mthd source s|

	source := blockStream contents.
	s := '__m\^ ' withCRs , source.
	mthd := Compiler 
		    compile:s
		    forClass:self
		    inCategory:'temporary'
		    notifying:(WarningCompilationErrorHandler new source:s)
		    install:false.
	mthd == #Error ifTrue:[
	    block := [:request :output | output nextPutAll:stringOrStream].
	] ifFalse:[
	    block := mthd valueWithReceiver:nil arguments:nil.
	].

"/        block := Compiler evaluate: source.
"/        block == #Error ifTrue:[
"/            block := [:request :output | output nextPutAll:stringOrStream].
"/        ]
    ].
    ^ self new formattingBlock:block

    "Modified: / 30.7.1999 / 19:26:58 / cg"
!

oldEvalEmbedded: stringOrStream with: request
    | sourceStream targetStream evalStream currentStream evalValue peekValue ch |

    (stringOrStream isKindOf: Stream)
	    ifTrue: [sourceStream := stringOrStream]

	    ifFalse: [sourceStream := ReadStream on: stringOrStream].

    targetStream := WriteStream on: String new.
    currentStream := targetStream. 
    [sourceStream atEnd] whileFalse:[ 
	ch := sourceStream next.
	ch = $< ifTrue:[
	    peekValue := sourceStream peek. 
	    (peekValue = $?) ifTrue:[
		evalStream := WriteStream on: String new.

		currentStream := evalStream.

		sourceStream next. "Eat the ?"
		ch := sourceStream next.
	    ]
	].
	((currentStream = evalStream) and: [ch = $?]) ifTrue:[
	    peekValue := sourceStream peek.
	    (peekValue = $>) ifTrue:[
		sourceStream next.
		"Eat the >"
		currentStream := targetStream.

		evalValue := (Compiler new 
				evaluate: (evalStream contents) 
				in: thisContext 
				to: self
				notifying: nil ifFail: [^nil]).
		    (evalValue isKindOf: String) ifFalse: [
			evalValue := evalValue printString
		    ].
		    currentStream nextPutAll: evalValue.
	    ]
	] ifFalse: [
	    currentStream nextPut: ch
	].
    ].
    ^ targetStream contents

!

rangesOfAngleBrackets: sourceStrm
    "Return an OrderedCollection of intervals of position within angle 
     brackets < and >.  Caller wants to avoid putting <br> in there."

    | list char intervals start |

    list _ OrderedCollection new: 10.
    [sourceStrm atEnd] whileFalse: [
	(char _ sourceStrm next) == $< ifTrue: [
	    list add: sourceStrm position.            "a start"
	    sourceStrm peek == $? ifTrue:[
		sourceStrm next.
		sourceStrm skipThroughAll:'?>'.
		list add: sourceStrm position negated.
	    ]
	].
	char == $> ifTrue: [
	    list add: sourceStrm position negated    "an end"
	]
    ].

    sourceStrm reset.
    intervals _ OrderedCollection new: 10.
    start _ nil.
    list do: [:each |
	(each > 0) & (start == nil) ifTrue: [
	    start _ each
	].
	(each < 0) & (start ~~ nil) ifTrue: [
	    intervals add: (start to: each negated). start _ nil
	]
    ].
    ^ intervals     

    "       
     HTMLformatter rangesOfAngleBrackets: 
	(ReadStream on: '1234 <good> <456 <good> 567> <ok>')
    "

    "Modified: / 30.7.1999 / 18:37:18 / cg"
!

simpleProcess: aStringOrStream
    | sourceStream targetStream ch |

    (aStringOrStream isKindOf: Stream)
	    ifTrue: [sourceStream := aStringOrStream]
	    ifFalse: [sourceStream := ReadStream on: aStringOrStream].
    targetStream := WriteStream on: String new. 
    [sourceStream atEnd] whileFalse:
    [ch := sourceStream next.
    (ch = Character linefeed) ifTrue:
	    [(sourceStream peek) = (Character linefeed)
	    ifTrue: [sourceStream next. targetStream nextPutAll: '<p>']
	    ifFalse: [targetStream nextPutAll: '<br>']].
    targetStream nextPut: ch].
    ^targetStream contents.
!

swikify: aStringOrStream linkhandler: aBlock
    | formatter |

    formatter _ self new.
    formatter specialCharacter: $*.
    ^ formatter swikify: aStringOrStream linkhandler: aBlock
!

textToGIF: oneLineString
    | form filename s|

    form _ (Form extent: 400@20 depth: Display depth) fillWhite.
    oneLineString displayOn: form at: 2@0. "form display."
"/    filename _ 'f' , (HTTPServer nextSequenceNumber) printString , '.gif'.
"/    GIFReadWriter putForm: form onFileNamed: filename.
"/    ^(FileStream fileNamed: filename) contentsOfEntireFile

    filename := Filename newTemporary withSuffix:'gif'.
    GIFReader new
	save:(Image fromForm:form)
	onStream:(s := filename writeStream).
    s close.
    ^ filename binaryContentsOfEntireFile
! !

!HTMLformatter methodsFor:'accessing'!

specialCharacter
	^specialCharacter
!

specialCharacter: someCharacter
	specialCharacter _ someCharacter
! !

!HTMLformatter methodsFor:'formatting'!

format: requestObject
    "format text with requestObject as the argument for the code blocks"

    ^ String streamContents: [:stream |
	  formattingBlock value: requestObject value: stream 
    ]
! !

!HTMLformatter methodsFor:'private-initialization'!

formattingBlock: aBlock
	formattingBlock _ aBlock
!

initialize
	specialCharacter := $*.
! !

!HTMLformatter methodsFor:'translating'!

rangesOfAngleBrackets: sourceStrm
    "Return an OrderedCollection of intervals of position within angle
     brackets < and >.  Caller wants to avoid putting <br> in there."

    ^ self class
	rangesOfAngleBrackets: sourceStrm

"/    | list char intervals start |
"/
"/    list _ OrderedCollection new: 10.
"/    [sourceStrm atEnd] whileFalse: [
"/        (char _ sourceStrm next) == $< ifTrue: [
"/            list add: sourceStrm position              "a start"
"/        ].
"/        char == $> ifTrue: [
"/            list add: sourceStrm position negated      "an end"
"/        ]
"/    ].
"/
"/    sourceStrm reset.
"/    intervals _ OrderedCollection new: 10.
"/    start _ nil.
"/    list do: [:each |
"/        (each > 0) & (start == nil) ifTrue: [
"/            start _ each
"/        ].
"/        (each < 0) & (start ~~ nil) ifTrue: [
"/            intervals add: (start to: each negated). start _ nil
"/        ]
"/    ].
"/    ^ intervals

    "       
     HTMLformatter rangesOfAngleBrackets: 
	    (ReadStream on: '1234 <good> <456 <good> 567> <ok>')
    "
!

swikify:aStringOrStream linkhandler:aBlock
    | sourceStream aLine targetStream start end forbidden ignore lookAhead nm|

    (aStringOrStream isStream) ifTrue: [
	sourceStream := aStringOrStream
    ] ifFalse: [
	sourceStream := ReadStream on: aStringOrStream
    ].

    forbidden _ self rangesOfAngleBrackets: sourceStream.
    targetStream := WriteStream on: String new.
    [sourceStream atEnd] whileFalse:[
	aLine := sourceStream nextLine. "/ upTo: (Character cr).
	(aLine endsWith:Character return) ifTrue:[
	    aLine := aLine copyWithoutLast:1
	].

	" Now, look for links "
	start _ 1.
	[
	    (start _ aLine indexOf:specialCharacter startingAt: start) ~= 0
	    and: [start < aLine size]
	] whileTrue:[
	    (aLine at: start+1) = specialCharacter ifTrue: [
		"/ ** is made a single * here
		aLine _ aLine copyReplaceFrom: start to: start+1 
			      with: specialCharacter printString.
		start _ start + 1. 
	    ] ifFalse: [
		"/ find string enclosed in *..*
		(end _ aLine indexOf:specialCharacter startingAt: (start+1)) ~= 0
		ifTrue: [
		    nm := (aLine copyFrom: start+1 to: end-1).
		    aLine _ aLine copyReplaceFrom: start to: end
				  with: (aBlock value:nm)
		] ifFalse: [
		    start _ start + 1
		]
	    ]
	].

	"If it's at least 4 dashes, make it a horizontal rule"
	(aLine startsWith: '----') ifTrue: [
	    targetStream nextPutAll: '<hr>'
	] ifFalse: [
	    targetStream nextPutAll: aLine
	].

	"Should there be a <br> after this line?"
	(ignore _ sourceStream peek = $<) ifTrue: [
	    "If just before a tag, ignore the newline"
	    targetStream nextPut: $       "but do put in a separator"
	].

	forbidden do: [:interval |
	    (interval includes: sourceStream position) ifTrue: [
		ignore _ true
	    ]
	].
	ignore ifFalse: [
	    (lookAhead := sourceStream peekOrNil) = (Character cr) ifTrue: [
		sourceStream next. targetStream nextPutAll: '<p>'; cr.
	    ] ifFalse: [
		lookAhead = (Character return) ifTrue: [
		    sourceStream next. targetStream nextPutAll: '<p>'; cr.
		    (sourceStream peekOrNil) = (Character linefeed) ifTrue: [
			sourceStream next.
		    ]
		] ifFalse:[
		    lookAhead notNil ifTrue:[
			targetStream nextPutAll: '<br>'; cr.
		    ]
		]
	    ]
	]
    ].
    ^ targetStream contents.

    "Modified: / 30.7.1999 / 21:43:31 / cg"
! !

!HTMLformatter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/HTMLformatter.st,v 1.14 2003/03/31 15:30:47 mb Exp $'
! !
