"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

SwikiAction subclass:#PSwikiAction
	instanceVariableNames:'authorizer'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

PSwikiAction comment:'A Swiki with the potential for a password on each page.  Requires the special folder ''pswiki'' from the swiki web site.'
!


!PSwikiAction class methodsFor:'initialization'!

mapClass
	"The class of my urlMap"
	^ PURLmap
!

pageClass
	"The class of my pages"
	^ PSwikiPage
!

setUp: actionName
    "Set up a named protected Swiki"

    self error:'use setUp: fromServer:'.
"/    | action auth |
"/
"/    super setUp: actionName.
"/    action _ HTTPServer actions at: actionName.
"/    auth _ Authorizer new.  "*** for now, not restored from the disk***"
"/    auth realm: actionName.
"/    action auth: auth.
"/    ^ action
!

setUp: actionName fromServer:aHttpServer
    "Set up a named protected Swiki"

    | action auth |

    super setUp: actionName fromServer:aHttpServer.
    action := aHttpServer actions at: actionName.
    auth := Authorizer new.  "*** for now, not restored from the disk***"
    auth realm: actionName.
    action auth: auth.
    ^ action
! !

!PSwikiAction methodsFor:'as yet unclassified'!

auth
	^authorizer
!

auth: anAuth
	authorizer _ anAuth.
!

browse: pageRef from: request
    "Check authorization"
    (pageRef privs includesSubString: 'read') ifTrue:[
	(authorizer user: request userID) = pageRef coreID ifFalse: [
	    self error: (HTTPServer unauthorizedFor: authorizer realm)
	]
    ].
    request 
	reply: HTTPServer success; 
	reply: HTTPServer contentHTML;
	reply: HTTPServer crlf.

    ^super browse: pageRef from: request
!

edit: pageRef from: request
    "Check authorization"

    (pageRef privs includesSubString: 'write') ifTrue:[
	(authorizer user: request userID) = pageRef coreID ifFalse: [
	    self error: (HTTPServer unauthorizedFor: authorizer realm)
	]
    ].
    request reply: HTTPServer success; reply: HTTPServer contentHTML.
    request reply: HTTPServer crlf.
    request reply: (HTMLformatter evalEmbedded:(self fileContents: source , 'pedit.html')
					  with: pageRef).
    pageRef noteEditRequest.
    ^ self
!

inputFrom: request
	"Take user's input and respond with a searchresult or store the edit"

    | coreRef page |
    coreRef _ request message size < 2
	    ifTrue: ['1']
	    ifFalse: [request message at: 2].

    coreRef = 'searchresult' ifTrue: [
	"If contains search string, do search"
	request 
	    reply: HTTPServer crlf;
	    reply: (HTMLformatter 
			evalEmbedded: (self fileContents:source, 'results.html')
			with: (urlmap searchFor: (request fields at: 'searchFor' ifAbsent: ['nothing']))).
	^ #return
    ].

    (request fields includesKey: 'text') ifTrue: [
	"It's a response from an edit, so store the page"
	page _ urlmap atID: coreRef.
	page user: request peerName.  "Address is machine, user only if logged in"
	page pageStatus = #new ifTrue: [page pageStatus: #standard].
	page _ urlmap
		storeID: coreRef
		username: (request fields at: 'username' ifAbsent:[''])
		password: (request fields at: 'password' ifAbsent: [''])
		privs: (request fields at: 'privs' ifAbsent: [''])
		text: (request fields at: 'text' ifAbsent: ['blank text'])
		from: request peerName.
	page user: request userID.  "Address is machine, user only if logged in"
	^ self        "return self means do serve the edited page afterwards"
    ].

    request fields keys do: [:aTag |
	(aTag beginsWith: 'text-') ifTrue: [
	    urlmap
		storeID: coreRef
		text: (self class withSqueakLineEndings:(request fields at: aTag))
		insertAt: (aTag copyFrom: 6 to: aTag size).
	    "string"
	    ^ self
	]
    ].

    "oops, a new kind!!"
    Transcript show: 'Unknown data from client. '; show: request fields printString; cr.


!

mapName: nameString password: pwdString to: aPerson
	"Insert/remove the username:password combination into/from the users Dictionary.  *** Use this method to add or delete users!!  If you ask for the authorizer and talk to it, the change will not be recorded on the disk!! ***   We use encoding per RFC1421."

	authorizer mapName: nameString password: pwdString to: aPerson.
	self authorizer: authorizer.    "force it to be written to the disk"
		"*** Authorizer not saved to disk yet for this class ***"
!

process: request
	"URLs are of the form:
	{swikiname} to browse Front Page
	{swikiname}.{coreID} to browse the page
	{swikiname}.{coreID}.edit  to edit the page
	{swikiname}.{coreID}.all  displays the page and all its references
	{swikiname}.{coreID}.versions displays the last three versions of the page
	{swikiname}.searchresult  conducts the search and displays the result
	{swikiname}.recent  to bring up the recent changes list
	{swikiname}.{coreID}.searchresult  returns a search for references to coreID's key
	{swikiname}.{picName}.gif (or jpeg, jpg, jpe, upper or lower case) 
		return the picture stored in the {swikiname} folder."
	"Transcript show: 'Got request ',(request url); cr."

	| coreRef pageRef command formattedPage theLast |
	self log: request.
	theLast _ request message last asLowercase.
	theLast = 'gif' ifTrue: [^ self process: request MIMEtype: 'image/gif'].
	theLast = 'jpeg' ifTrue: [^ self process: request MIMEtype: 'image/jpeg'].
	theLast = 'jpg' ifTrue: [^ self process: request MIMEtype: 'image/jpeg'].
	theLast = 'jpe' ifTrue: [^ self process: request MIMEtype: 'image/jpeg'].
	theLast = 'html' ifTrue: [^ self process: request MIMEtype: 'text/html'].
	request message size < 2
		ifTrue: [coreRef _ '1']
		ifFalse: [coreRef _ request message at: 2].
	request fields ifNotNil: ["Are there input fields?"
		(self inputFrom: request) == #return ifTrue: [^ self]].
	coreRef = 'recent' ifTrue:
			[request reply: HTTPServer crlf, (HTMLformatter
					evalEmbedded: (self fileContents: source, 'recent.html')
					with: urlmap recent).
			^ self].
"       request reply: HTTPServer crlf.                End of header.  Move this when we want to report more header
"       "At this point, coreRef is a page reference"
	pageRef _ urlmap atID: coreRef.
	request message size > 2 ifTrue: ["SearchResult, All, Versions, or Edit"
		command _ request message at: 3.
		command = 'edit' ifTrue:
			[^self edit: pageRef from: request].
		command = 'versions' ifTrue:
					[request reply: ((self formatterFor: 'versions') format: pageRef).
					^self].
		command = 'searchresult' ifTrue:
			[request reply: (HTMLformatter evalEmbedded:
							(self fileContents: source , 'results.html')
						with: (urlmap searchFor: pageRef name)).
			^ self].
		command = 'all' ifTrue:
			[formattedPage _ urlmap allPagesFrom: pageRef for: request.
			request reply: (HTMLformatter evalEmbedded:
							(self fileContents: source, 'page.html')
						with: formattedPage).
			^ self].
		(request message at: 1) = (request message at: 2) ifTrue: [
			request reply: '<h1>Your original url has a slash after it.  Please remove it.</h1>'.
			request message: (request message copyFrom: 2 to: request message size).
			^ self process: request].       "(success and contentHTML will be sent twice!!)"
		Transcript show: 'Unknown command: ', command; cr.
		^ self].
	"Just a browse"
	^ self browse: pageRef from: request
!

restore: nameOfSwiki
	"Read all files in the directory 'nameOfSwiki'.  Reconstruct the url map."

	super restore: nameOfSwiki.
	authorizer _ Authorizer new.    "<- not restored from disk in this implementation"
	authorizer realm: name.
! !

!PSwikiAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/PSwikiAction.st,v 1.8 2003/03/26 10:27:26 stefan Exp $'
! !
