"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

AuthorizedSwikiAction subclass:#RenderingSwikiAction
	instanceVariableNames:'serverDirectory'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

RenderingSwikiAction comment:'An authorized Swiki that can generate cached pages with no link back to the
Swiki

Examples of setting them up:

Here''s one that renders to an external, FTP-accessible website:
| sd auth |
sd _ ServerDirectory new.
sd directory: ''/net/www/fac/mark.guzdial/testSwiki''; server: ''flintstone.com'';
	user: ''fred''; password: ''flintstone''.
RenderingSwikiAction new restore: ''testSwiki''.
(PWS actions at: ''testswiki'') serverDirectory: sd.
auth _ Authorizer new mapName: ''user'' password: ''testSwiki'' to: ''user''.
(PWS actions at: ''testswiki'') authorizer: auth.

Here''s one that renders to a local directory accessible from a webserver:
| sd auth |
sd _ ''Guz 7600:WebSTAR 2.0:testSwiki:''.
RenderingSwikiAction setUp: ''testSwiki''.
(PWS actions at: ''testswiki'') serverDirectory: sd.
auth _ Authorizer new mapName: ''user'' password: ''testSwiki'' to: ''user''.
(PWS actions at: ''testswiki'') authorizer: auth.
'
!


!RenderingSwikiAction methodsFor:'accessing'!

serverDirectory: anObject
	serverDirectory _ anObject. "Should be an instance of
ServerDirectory, or a path string."
! !

!RenderingSwikiAction methodsFor:'process and generate'!

browse: pageRef from: request
	"Just reply with a page in HTML format -- use the rendering page
template rpage.html"

	| formattedPage htmlForUser |
	formattedPage _ pageRef copy.

	"Make a copy, then format the text."
	formattedPage formatted: (formatter swikify: pageRef text
			linkhandler: [:link | urlmap
					linkFor: link
					from: request peerName
					storingTo: OrderedCollection new
					page: formattedPage]).

	"format using the cached formatter"
	htmlForUser _ ((self formatterFor: 'rpage') format: formattedPage).
	htmlForUser size = 0 ifTrue: [self error: 'template file
''rpage.html'' not found'].
	request reply: htmlForUser.
!

generate: pageRef
	"Generate the page to the serverDirectory. Use the render.html page as a template"

	| formattedPage renderedFile newmap file|
	"Make a copy, then format the text."
	formattedPage _ pageRef copy.
	newmap _ urlmap copy.           " Create a new action with different action for cached form."
	newmap action: (RendererSwikiAction new).
	newmap action name: '.'.
	formattedPage formatted: (formatter swikify: pageRef text
			linkhandler: [:link | newmap
					linkFor: link
					from: 'Nowhere'
					storingTo: OrderedCollection new]).
	(serverDirectory isKindOf: String)
	ifTrue: [ "Just save the file into the directory"
		renderedFile _ (serverDirectory),(pageRef coreID),'.html'.
		(StandardFileStream isAFileNamed: renderedFile)
			ifTrue: [FileDirectory deleteFilePath: renderedFile].
		file _ FileStream fileNamed: renderedFile.
		file nextPutAll: (HTMLformatter evalEmbedded: (self fileContents: source ,'render.html')
								with: formattedPage).
		file close.]
	ifFalse: [ "Assume it's FTP and send it off"
		renderedFile _ (pageRef coreID),'.html'.
		(StandardFileStream isAFileNamed: renderedFile)
			ifTrue: [FileDirectory deleteFilePath: renderedFile].
		file _ FileStream fileNamed: renderedFile.
		file nextPutAll: (HTMLformatter evalEmbedded: (self fileContents: source ,'render.html')
								with: formattedPage).
		file close.
		serverDirectory putFile: (FileStream fileNamed: renderedFile) named: renderedFile.
		"FileDirectory deleteFilePath: renderedFile"]
!

pageURL: aPage
	"make the URL suited to aPage"
	^(self name),'.',aPage coreID
!

process: request
	"Handle requests of the form {swikiname}.{coreRef}.render"
	(request message size > 2) ifTrue: [
		((request message at: 3) = 'render') ifTrue:
		[ self generate: (urlmap atID: (request message at: 2)).
		request reply: (HTTPServer success) ; reply: (HTTPServer contentHTML).
		^self browse: (urlmap atID: (request message at: 2)) from:
request].
		((request message at: 3) = 'html') ifTrue: "Handle 1.html refs"
		[^self browse: (urlmap atID: (request message at: 2)) from:
request]].
	super process: request.
! !

!RenderingSwikiAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/RenderingSwikiAction.st,v 1.6 2003/03/26 10:28:06 stefan Exp $'
! !
