"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#ServerAction
	instanceVariableNames:''
	classVariableNames:'DefaultFile Verbose'
	poolDictionaries:''
	category:'PWS-Swiki'
!

ServerAction comment:'The ServerAction class is the default ServerAction (if the first token in the URL does not match any in the PWS ActionTable). Instances of ServerAction act like Mac Web servers -- they parse the URL into a Mac file path (including use of default.html if no final filename is provided), then return the contents of that file as the reply. 

All ServerActions get asked to process: a PWS request. This ServerAction does the processing by asking itself to checkAuthorization, to parse the request, to log the parsed request, and then to reply to the request.  Other ServerActions (like EmbeddedServerAction) can override one or more of those requests.

The ServerAction class provides a ServerDirectory service.'
!


!ServerAction class methodsFor:'System Services'!

defaultFile
    ^ DefaultFile ? 'default.html'
!

pathSeparator
    ^ String with: Filename separator
!

serverDirectory
    ^ UserPreferences current httpServerSwikiServerDirectory

    "
     self serverDirectory
    "
!

serverDirectory:aDirectoryPath
    UserPreferences current httpServerSwikiServerDirectory:aDirectoryPath.
! !

!ServerAction class methodsFor:'support stuff'!

withSqueakLineEndings:aString
    "assume the string is textual, and that CR, LF, and CRLF are all 
     valid line endings.  Replace each occurence with a single CR"

    | cr lf input c crlf inPos outPos outString lineEndPos newOutPos |

    cr _ Character return.
    lf _ Character linefeed.
    crlf _ String with:cr with:lf.

    inPos _ 1.
    outPos _ 1.
    outString _ String new: aString size.

    [ 
	lineEndPos _ aString indexOfAnyOf: crlf startingAt: inPos ifAbsent: [0].
	lineEndPos ~= 0 
    ] whileTrue: [
	newOutPos _ outPos + (lineEndPos - inPos + 1).

	outString replaceFrom: outPos to: newOutPos - 2 with: aString startingAt: inPos.
	outString at: newOutPos-1 put: (Character cr).
	outPos _ newOutPos.

	((aString at: lineEndPos) = cr 
	and: [ lineEndPos < aString size 
	and: [ (aString at: lineEndPos+1) = lf ] ]) ifTrue: [
	    "CRLF ending"
	    inPos _ lineEndPos + 2 
	] ifFalse: [ 
	    "CR or LF ending"
	    inPos _ lineEndPos + 1 
	]. 
    ].

    "no more line endings.  copy the rest"
    newOutPos _ outPos + (aString size - inPos + 1).
    outString replaceFrom: outPos to: newOutPos-1 with: aString startingAt: inPos.

    ^ outString copyFrom: 1 to: newOutPos-1
        
    "
     self withSqueakLineEndings:'foo bar'     
     self withSqueakLineEndings:('foo bar' copyWith:Character return) 
     self withSqueakLineEndings:(('foo bar' copyWith:Character lf) copyWith:Character lf)
     self withSqueakLineEndings:(('foo bar' copyWith:Character return) copyWith:Character lf)
    "
! !

!ServerAction methodsFor:'URL processing'!

checkAuthorization: request
	^self
!

log: pieces to: request
	| tab |
	tab := String with: Character tab.
	request log: ((Time now printString), tab, (Date today printString), tab, (pieces printString),
	Character cr asString).

!

parse: request
    | filename |

    filename := (ServerAction serverDirectory) , 
		((request url copyFrom:2) 
		    copyReplaceAll: '/' 
		    with:(ServerAction pathSeparator)).

    (filename endsWith:ServerAction pathSeparator) ifTrue: [
	filename _ filename , (ServerAction defaultFile)
    ].
    ^ filename

    "Modified: / 30.7.1999 / 16:36:23 / cg"
!

postProcessReplyText:someText with:aRequest
    "to avoid dublicating code - this hook allows for actions
     to be performed on a retrieved files contents (embedded ST, for example)"

    ^ someText

    "Modified: / 30.7.1999 / 17:25:19 / cg"
!

process: request
    | pieces |

    self checkAuthorization: request.
    pieces _ self parse: request.
    self log: pieces to: request.
    self replyTo: pieces from: request.

    "Modified: / 30.7.1999 / 17:26:34 / cg"
!

process: request MIMEtype: imageGif
    "See if there is a .gif file on my disk of this name, and send it out."

    | fName file |

    fName _ ServerAction serverDirectory, (request message at: 1),
	    (ServerAction pathSeparator), (request message at: 2).

    3 to: request message size do: [:part |
	    fName _ fName, '.', (request message at: part)].

    Transcript show: fName; cr.

    file := Filename defaultDirectory construct:fName.
    (file exists and:[file isDirectory not]) ifFalse: [
	request reportNotFound:'file not found'.
	^ self
    ].

    request 
	replyHeader: (HTTPServer success);
	replyHeader: (HTTPServer content: imageGif);
	reply:(file binaryContentsOfEntireFile).

    "Modified: / 30.7.1999 / 21:06:56 / cg"
!

replyTo: pieces from: request
"/    Transcript showCR: 'Now replying with: ', pieces.

    (pieces asFilename isReadable
    and:[pieces asFilename isDirectory not]) ifFalse:[
	request reportNotFound:nil.
	^ self.
    ].

    request 
	replyHeader: HTTPServer success; 
	replyHeader: HTTPServer contentHTML;
	reply: (self
		   postProcessReplyText:(FileStream fileNamed: pieces) contentsOfEntireFile
		   with:request).

    "Modified: / 30.7.1999 / 21:06:30 / cg"
! !

!ServerAction methodsFor:'file input/output'!

fileContents: serverFileRef
    ^ (ServerAction serverDirectory asFilename construct:serverFileRef) 
	 contentsOfEntireFile
!

subdirectory: named
	^named, (ServerAction pathSeparator)
! !

!ServerAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/ServerAction.st,v 1.17 2003/03/26 10:26:17 stefan Exp $'
! !
