"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#Session
	instanceVariableNames:'lastAccess data'
	classVariableNames:'Sessions'
	poolDictionaries:''
	category:'PWS-Swiki'
!

Session comment:'A Session can provide server-side storage of state/data between HTML pages. After creating a new session (which can be manipulated using at: and at:put:), it can be saved on the Sessions list by asking the Session class to store: the session. A random integer is returned as a key to the session, and the key can be stored in an HTML page in a hidden field.

A session can be retrieved by ID by asking Session to recall: an ID. A session can be retired when it is no longer useful.

Each recall: or use at:/at:put: "touches" (updates) the Session''s last use time. A Session that is older than a certain period (default 30 minutes, in viableTime instance method) will generate an error. Every hour, the PWS, as part of its backup procedures, will clear out old sessions.
'
!


!Session class methodsFor:'initialization'!

initialize
	Sessions := Dictionary new.
	HTTPServer addToBackupJob: [Session clearOld].

!

new
	^super new initialize
! !

!Session class methodsFor:'store/recall'!

clearOld
	| aSession |
	Sessions keys do: [:s |
	aSession := Sessions at: s ifAbsent: [nil].
	aSession isNil ifFalse:
		[aSession isViable ifFalse: [self retire: s].]]
!

recall: id
	| aSession |
	aSession := Sessions at: id ifAbsent: [self error: 'Non-existent session'].
	aSession isViable ifFalse: [self error: 'Your session data has expired. Please restart the operation.'].
	aSession touch.
	^aSession

!

retire: id
	Sessions removeKey: id.
!

store: aSession
	| newID |
	[       newID := SmallInteger maxVal atRandom.
		Sessions includesKey: newID ] whileTrue.
	Sessions at: newID put: aSession.
	^newID
! !

!Session methodsFor:'access'!

at: key
	self touch.
	^data at: key ifAbsent: [self error: 'Internal key error. Please alert administrator about key',key printString].
!

at: key put: value
	self touch.
	^data at: key put: value.
!

isViable
	^ (AbsoluteTime now subtractTime: lastAccess) asSeconds > self viableTime
	"/ ^ (Time now subtractTime: lastAccess) asSeconds > self viableTime
!

touch
	lastAccess := AbsoluteTime now.
!

viableTime
	"In seconds"
	^60*30

! !

!Session methodsFor:'initialize'!

initialize
	data := Dictionary new.
	lastAccess := Time now.
	"HTTPServer addToBackupJob: [Session user]."  "not implemented??"
! !

!Session class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/Session.st,v 1.6 2003/03/26 10:26:56 stefan Exp $'
! !

Session initialize!
