"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

ServerAction subclass:#SwikiAction
	instanceVariableNames:'urlmap source name formatters formatter'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!

SwikiAction comment:'SwikiAction is the action to implement a Swiki, a collaborative website.
This is a web server in Squeak.  Each page served has an "Edit this Page"
link.  The syntax rules for editing pages (from any web browser) are in the
file FormattingRules.

The pages may be viewed from any web browser, and the URL is
http://thisMachine:80/OurOwnArea.1  (''OurOwnArea'' is the name of the Swiki
area.)  See http://guzdial.cc.gatech.edu:8080/mySwiki for an example
focused on discussing Swikis.

Swiki is based on the Wiki Server, designed and implemented by Ward
Cunningham.  See the original Wiki at http://c2.com/cgi-bin/wiki, or the
Squeak-focused one at http://c2.com:8080/

See PWS class howToStart.

Modified by Ted Kaehler to have all state on the disk at all times.  Do not
snapshot the image during normal server operation.  Restart the server by
loading all state from the disk.

Each separate Wiki is a folder inside the Server folder.  The name of the
folder is the name of the Wiki.  Each page is a file.  (Inefficient on Macs
because of large minimum page size, but that''s an OS problem.)  All
versions of all pages are kept.  Each page contains chunks, just like a
changes file.  Each chunk is one version of one page:

self name: ''Ted''''s Page'' date: ''1/28/1998'' time: ''12:47:21'' by: ''tk'' text:
''Here is the body of the page'' back: 116!!

The last item is number of bytes in the chunk.  It is the number of bytes
from the start of the chunk to just in front of the number.  This allows
back-chaining to old versions.  On startup, the chunk is executed
explicitly using evaluate:for:logged:.  It supplies the page name to a
SwikiPage and installs it in the URLmap.  By doing this for all files in
the folder, the map (url->page) is built.  During operation, the chunk is
not executed.  Instead, take the file directly, look at ''back:'' at the end,
jump back to the start of the chunk, scan for ''text:'' and read the text.

The looks of the pages served are controlled by templates.  They live in
the ''swiki'' folder in the ''Server'' folder.  See page.html for how to change
the look of a served page.  Beware that templates are cached by
HTMLformatter.  If you change a template, you will not see the effect until
you reload the Swiki.

A symbol *append here* may be added anywhere in the page.  Clicking on it
will bring up a page with a text editing form.  The what you type will be
inserted just before the *append here* marker.  This allows append-only
pages, or multiple questions with places to add to the answer.

'
!


!SwikiAction class methodsFor:'initialization'!

mapClass
    "The class of my map"

    ^ URLmap
!

pageClass
    "The class of my pages"

    ^ SwikiPage
!

restore: actionName from: filename
    "Obsolete.  For old format pages only!!!!!!  Get all the info about a Swiki from its folder."

    | action |

    action _ SwikiAction new.
    action name: actionName.
    action source:#( 'swiki' ).
    action restoreFrom: filename.
    action map pages do: [:each |
	each file: (ServerAction serverDirectory),
		   actionName, (ServerAction pathSeparator),
		   ((each file findTokens: (ServerAction pathSeparator)) last)].
    HTTPServer link: actionName to: action.

    "Modified: / 30.7.1999 / 19:51:08 / cg"
!

setUp
    self setUp: 'mySwiki'

    "
     self setUp
    "
!

setUp:actionName
    "Set up a named Swiki"

    |dir swikiDir actionDir action map page |

    (dir := ServerAction serverDirectory asFilename) exists ifFalse:[
	self warn:'Nonexisiting serverDirectory: ' , dir pathName.
	^ nil
    ].
    (swikiDir := dir construct:'swiki') exists ifFalse:[
	self warn:'Nonexisiting swiki directory: ' , swikiDir pathName.
	^ nil
    ].
    (actionDir := dir construct:actionName) exists ifFalse:[
	Smalltalk isStandAloneApp ifFalse:[
	    (self confirm:('Folder called ', actionDir pathName , ' does not exists.\\Create ?') withCRs)
	    ifFalse:[
		^ nil
	    ]
	].
	actionDir makeDirectory.
	actionDir exists ifFalse:[
	    self warn:('Could not create folder ', actionDir pathName).
	    ^ nil
	]
    ].

    action _ self new.
    map _ self mapClass new.        "URLmap or PURLmap"
    action formatter: (HTMLformatter new initialize).
    action formatter specialCharacter: $*.
    action map: map.
    action name: actionName.
    action source:(Array with:actionName with:'swiki').
    map action: action.
    map pages: (Dictionary new).
    map directory: actionName.

    page _ map newpage: actionName,': Front Page' from: 'Beginning'.
    page pageStatus: #standard.
    page text: (HTMLformatter 
		    evalEmbedded:((dir construct:'FrontPage') contentsOfEntireFile) 
		    with: actionName).

    page _ map newpage: 'Formatting Rules' from: 'Beginning'.
    page pageStatus: #standard.
    page text: (HTMLformatter 
		    evalEmbedded:((dir construct:'FormattingRules') contentsOfEntireFile) 
		    with: actionName).
    HTTPServer link: actionName to: action.
    ^ action

    "Modified: / 30.7.1999 / 19:50:54 / cg"
!

setUp:actionName fromServer:aServer
    "Set up a named Swiki"

    |dir swikiDir actionDir action map page |

    (dir := ServerAction serverDirectory asFilename) exists ifFalse:[
	self warn:'Nonexisiting serverDirectory: ' , dir pathName.
	^ nil
    ].
    (swikiDir := dir construct:'swiki') exists ifFalse:[
	self warn:'Nonexisiting swiki directory: ' , swikiDir pathName.
	^ nil
    ].
    (actionDir := dir construct:actionName) exists ifFalse:[
	Smalltalk isStandAloneApp ifFalse:[
	    (self confirm:('Folder called ', actionDir pathName , ' does not exists.\\Create ?') withCRs)
	    ifFalse:[
		^ nil
	    ]
	].
	actionDir makeDirectory.
	actionDir exists ifFalse:[
	    self warn:('Could not create folder ', actionDir pathName).
	    ^ nil
	]
    ].

    action := self new.
    map := self mapClass new.        "URLmap or PURLmap"
    action formatter: (HTMLformatter new initialize).
    action formatter specialCharacter: $*.
    action map: map.
    action name: actionName.
    action source:(Array with:actionName with:'swiki').
    map action: action.
    map pages: (Dictionary new).
    map directory: actionName.

    page := map newpage: actionName,': Front Page' from: 'Beginning'.
    page pageStatus: #standard.
    page text: (HTMLformatter 
		    evalEmbedded:((dir construct:'FrontPage') contentsOfEntireFile) 
		    with: actionName).

    page := map newpage: 'Formatting Rules' from: 'Beginning'.
    page pageStatus: #standard.
    page text: (HTMLformatter 
		    evalEmbedded:((dir construct:'FormattingRules') contentsOfEntireFile) 
		    with: actionName).
    aServer link: actionName to: action.
    ^ action

    "Modified: / 30.7.1999 / 19:50:54 / cg"
! !

!SwikiAction methodsFor:'URL processing'!

browse: pageRef from: request
    "Just reply with a page in HTML format"

    | pageText withEmbedded formattedPage htmlForUser |

    formattedPage _ pageRef copy.

    pageText := pageRef text.

    "Make a copy, then format the text."
    formattedPage formatted:(formatter 
				swikify: pageText
				linkhandler: [:link | urlmap
						linkFor: link
						from: request peerName
						storingTo: OrderedCollection new
						page: formattedPage
					     ]).

    "allow for subclasses to do special postprocessing (embedd html)"
    withEmbedded := self postProcessReplyText:formattedPage formatted with:request.
    formattedPage formatted:withEmbedded.

    "format using the cached formatter"
    htmlForUser _ (self formatterFor: 'page') format:formattedPage.
    htmlForUser size = 0 ifTrue: [
	self error: 'template file ''page.html'' not found'
    ].

    request reply: htmlForUser.

    "Modified: / 30.7.1999 / 18:38:54 / cg"
!

edit: pageRef from: request
    request reply: (HTMLformatter 
			evalEmbedded:(self findSourceFile:'edit.html') contentsOfEntireFile
			with: pageRef).
    pageRef noteEditRequest.
    ^ self

    "Modified: / 30.7.1999 / 19:58:16 / cg"
!

inputFrom: request
    "Take user's input and respond with a searchresult or store the edit.

    {swikiname}.{coreID}  {with a field named 'text'}  store an edited page
    {swikiname}.{coreID}  {with a field named 'text-placeID'}  store text in a specific place
    {swikiname}.searchresult  {with a field named 'searchFor'} conducts the search for that text and sends back the result
    "

    | coreRef page theText |

    coreRef _ request message size < 2 
		ifTrue: ['1']
		ifFalse: [request message at: 2].

    coreRef = 'searchresult' ifTrue: [
	    "If contains search string, do search"
	    request 
		reply: HTTPServer crlf;
		reply: ((self formatterFor: 'recent') 
			    format: (urlmap 
					searchFor:(request fields at:'searchFor' ifAbsent:['nothing'])
				    )
		       ).
	    ^ #return
    ].

    (theText _ request fields at: 'text' ifAbsent: [nil]) ifNotNil: [
	"It's a response from an edit, so store the page"
	page _ urlmap atID: coreRef.
	page user: request peerName.  "Address is machine, user only if logged in"
	page pageStatus = #new ifTrue: [page pageStatus: #standard].
	page _ urlmap
		storeID: coreRef
		text: (self class withSqueakLineEndings:theText)
		from: request peerName.
	^ self        "return self means do serve the edited page afterwards"
    ].

    request fields keys do: [:aTag |
	(aTag startsWith: 'text-') ifTrue: [
	    urlmap
		storeID: coreRef
		text: (self class withSqueakLineEndings:(request fields at: aTag))
		insertAt: (aTag copyFrom: 6 to: aTag size).
	    "string"
	    ^ self
	]
    ].

    "oops, a new kind!!"
    request log:('Unknown data from client: ' , request fields printString).

    "Modified: / 30.7.1999 / 19:05:56 / cg"
!

log: request
    |requestLog fn logfile|

    "/ OLD:
    "/ logfile _ FileStream fileNamed: name , '-log.txt'.
    "/ NEW: create logfile where HTTPServers logFile is located
    requestLog := request log.
    requestLog isFileStream ifTrue:[
	fn := requestLog pathName asFilename directory construct:(name , '-log.txt').
	logfile := fn writeStream.
    ] ifFalse:[
	logfile := requestLog.
    ].

    logfile isNil ifTrue: [
	request log: 'Failed logging on: ' , name
    ] ifFalse:[
	logfile setToEnd.
	request peerName isNil ifTrue: [
	    logfile nextPutAll:'UnknownPeer'
	] ifFalse: [
	    logfile nextPutAll:request peerName printString 
	].
	logfile 
	    nextPutAll:'-';
	    nextPutAll:request url;
	    nextPutAll:'-';
	    nextPutAll:Time now printString;
	    nextPutAll:'-';
	    nextPutLine:Date today printString.
	logfile close
    ]
!

pageURL:aPage
    "make the URL suited to aPage in this Action"

    ^ (self name) , '.' , aPage coreID

    "Modified: / 30.7.1999 / 21:55:47 / cg"
!

postProcessReplyText:someHTML with:request
    "allow for subclasses to do special postprocessing (embedd html)"
    ^ someHTML

    "Created: / 30.7.1999 / 17:31:58 / cg"
!

process: request
    "URLs are of the form:
    {swikiname}                   browse Front Page
    {swikiname}.{coreID}          browse the page number coreID
    {swikiname}.{coreID}.edit     request to edit the page
    {swikiname}.{coreID}.versions show the last three versions of the page
    {swikiname}.{coreID}  {with a field named 'text'}  store an edited page
    {swikiname}.{coreID}.insert.{placeID}  request to insert text in a specific place in a page
    {swikiname}.{coreID}  {with a field named 'text-placeID'}  store text in a specific place
    -------
    {swikiname}.{coreID}.all           displays the page and all its references
    {swikiname}.searchresult           conducts the search and displays the result
    {swikiname}.recent                 to bring up the recent changes list
    {swikiname}.{coreID}.searchresult  returns a search for references to coreID's key
    {swikiname}.{picName}.gif          or .jpeg, .jpg, .jpe, .html (upper or lower case) return
				       the picture or static page stored in the {swikiname} folder."

    | coreRef pageRef command formattedPage theLast pvtPageRef htmlForUser 
      mime|

    self log: request.

    theLast _ request message last asLowercase.

    ((mime := MIMETypes mimeTypeForSuffix:theLast) notNil
    and:[mime startsWith:'image/']) ifTrue:[
	^ self process: request MIMEtype:mime
    ].

    theLast = 'gif' ifTrue:  [^ self process: request MIMEtype: 'image/gif'].
    theLast = 'jpeg' ifTrue: [^ self process: request MIMEtype: 'image/jpeg'].
    theLast = 'jpg' ifTrue:  [^ self process: request MIMEtype: 'image/jpeg'].
    theLast = 'jpe' ifTrue:  [^ self process: request MIMEtype: 'image/jpeg'].
    theLast = 'html' ifTrue: [^ self process: request MIMEtype: 'text/html'].

    request 
	replyHeader: HTTPServer success;
	replyHeader: HTTPServer contentHTML.

    Verbose == true ifTrue:[
	Transcript showCR: 'In process: ' , request message printString.
    ].

    request message size < 2
	    ifTrue: [coreRef _ '1']
	    ifFalse: [coreRef _ request message at: 2].

    request fields ifNotNil: [
	"Are there input fields? - then its an edit reply (i.e. save)"

	(self inputFrom: request) == #return ifTrue: [^ self]
    ].

    coreRef = 'recent' ifTrue:[
	request 
	    reply: HTTPServer crlf ;
	    reply: ((self formatterFor: 'recent') format: urlmap recent).
	^ self
    ].

    pageRef _ urlmap atID: coreRef.
    pageRef isNil ifTrue:[
	request reportNotFound:'Not found: ' , coreRef.
	^ self
    ].

    "At this point, coreRef is a page reference"

    request message size > 2 ifTrue:[
	"SearchResult, All, Versions, or Edit"
	command _ request message at: 3.

	command = 'edit' ifTrue:[
	    request reply: ((self formatterFor: 'edit') format: pageRef).
	    pageRef noteEditRequest.
	    ^ self
	].

	command = 'versions' ifTrue:[
	    request reply: ((self formatterFor: 'versions') format: pageRef).
	    ^self
	].

	command = 'insert' ifTrue:[
	    pvtPageRef _ pageRef clone.
	    pvtPageRef placeID: (request message at: 4).    "so <?request placeID?> can get it"
	    htmlForUser _ ((self formatterFor: 'insert') format: pvtPageRef).
	    htmlForUser size = 0 ifTrue: [
		self error: 'template file ''insert.html'' not found'
	    ].
	    request reply: htmlForUser.
	    pageRef noteEditRequest.
	    ^ self
	].

	command = 'searchresult' ifTrue:[
	    request reply: ((self formatterFor: 'results')
		    format: (urlmap searchFor: pageRef name)).
	    ^ self
	].

	command = 'all' ifTrue:[
	    formattedPage _ urlmap allPagesFrom: pageRef for: request.
	    request reply: ((self formatterFor: 'page') format: formattedPage).
	    ^ self
	].

	(request message at: 1) = (request message at: 2) ifTrue: [
	    request reply: '<h1>Your original url has a slash after it.  Please remove it.</h1>'.
	    request message: (request message copyFrom: 2 to: request message size).
	    ^ self process: request       "(success and contentHTML will be sent twice!!)"
	].

	Verbose == true ifTrue:[
	    Transcript showCR: 'Unknown command: ' , command.
	].
	^ self
    ].

    "Just a browse"
    ^ self browse: pageRef from: request

    "Modified: / 30.7.1999 / 18:59:57 / cg"
!

process:request MIMEtype:mimeType
    "See if there is a .gif file on my disk of this name, and send it out."

    |relName fName file|

    relName := (request message at:1) , ServerAction pathSeparator ,
	       (request message at:2).

    fName _ ServerAction serverDirectory, relName.
    3 to: request message size do: [:part |
	fName _ fName, '.', (request message at: part)
    ].

    Transcript show: fName; cr.

    (file := fName asFilename) isAbsolute ifFalse:[
	file := Filename defaultDirectory construct:fName.
    ].
    (file exists and:[file isDirectory not]) ifFalse: [
	request reportNotFound:'file not found'.
	^ self.
    ].

    request 
	sendBinaryResponse:(HTTPServer success) 
	data:file binaryContentsOfEntireFile
	type:(HTTPServer content: mimeType)

    "Modified: / 30.7.1999 / 21:07:32 / cg"
!

smtlk: request
    "Return Smalltalk sourcecode.  
     URL = machine:80/mySwiki.smtlk.Point|at;  
     included are:  
	Point|at:
	Point|Comment   
	Point|Hierarchy  
	Point|Definition   
	Point|class|x;y;  
    NOTE:
	use ; instead of : in selector names!!!!!!"

    | classAndMethod set |

    classAndMethod _ request message atPin: 3.
    classAndMethod _ classAndMethod copyReplaceAll: '|' with: ' '.
    classAndMethod _ classAndMethod copyReplaceAll: ';' with: ':'.
    set _ LinkedMessageSet messageList: (Array with: classAndMethod).

    request 
	reply: HTTPServer crlf;
	reply: ((self formatterFor: 'smtlk') format: set).
! !

!SwikiAction methodsFor:'accessing'!

findSourceFile:fileName
    "find some file in the source directory (such as edit.html).
     Changed to allow source to be a list of directories wo search"

    |f|

    source isString ifTrue:[
	f := (ServerAction serverDirectory asFilename construct:source)
		construct:fileName.
	^ f
    ].
    source do:[:aPath |
	f := (ServerAction serverDirectory asFilename construct:aPath).
	f exists ifTrue:[
	    f := f construct:fileName.
	    f exists ifTrue:[
		^ f
	    ]
	]
    ].

    ^ nil

    "Created: / 30.7.1999 / 19:57:56 / cg"
    "Modified: / 30.7.1999 / 20:44:50 / cg"
!

formatter
	^formatter
!

formatter: anHTMLformatter
	formatter _ anHTMLformatter
!

map
	"Answer the receiver's 'urlmap'."

	^urlmap
!

map: anObject
	"Set the receiver's instance variable 'urlmap' to be anObject."

	urlmap := anObject
!

name
	"Answer the receiver's 'name'."

	^name
!

name: anObject
	"Set the receiver's instance variable 'name' to be anObject."

	name := anObject
!

path
	"A complete file path name to the directory where my page data is stored"
	^ (ServerAction serverDirectory) , name, (ServerAction pathSeparator)
!

source
    "Answer the receiver's 'source' path.
     Thats the server-directories, where formatting pages
     are retrieved from."

    ^source

    "Modified: / 30.7.1999 / 19:51:40 / cg"
!

source: aCollectionOfStrings
    "Set the receiver's 'source' path.
     Thats the server-directories, where formatting pages
     are retrieved from."

    source := aCollectionOfStrings

    "Modified: / 30.7.1999 / 19:52:03 / cg"
!

url
	"Answer the receiver's 'name'."

	^name
!

urlmap
	"Answer the receiver's 'urlmap'."

	^urlmap
!

urlmap: anObject
	"Set the receiver's instance variable 'urlmap' to be anObject."

	urlmap := anObject
! !

!SwikiAction methodsFor:'formatters'!

clearFormatterCache
	"remove all formatters"
	formatters _ nil.
!

formatterFor: formatterName
    "get the formatter for the given name.
     uses lazy initialization.  
     Could eventually check whether the on-disk file has been updated."

    | fileName fileModTime formatterInfo |
    
    "first, create the dictionary of formatters if it isn't already"

    formatters == nil ifTrue: [
	formatters _ Dictionary new. 
    ].

    "get the filename for the template source"
    fileName := self findSourceFile:(formatterName, '.html').

    "get its modification time"
"/    fileModTime _ ((FileDirectory on: ServerAction serverDirectory, source) entryAt:fileName) at: 3.
    fileModTime := fileName modificationTime.

    "create the formatter if necessary"

    formatterInfo := formatters at:formatterName ifAbsent:nil.
    (formatterInfo isNil 
    or: [(formatterInfo at: 2) < fileModTime ])    "/ outdated
    ifTrue: [
	Verbose == true ifTrue:[
	    Transcript showCR: '(recompiling formatter ', formatterName, ')'.
	].
	formatterInfo := Array 
			    with: (HTMLformatter forEvaluatingEmbedded: fileName contentsOfEntireFile)
			    with: fileModTime.
	formatters at: formatterName put: formatterInfo
    ].

    "return the formatter"

    ^ formatterInfo at: 1

    "Modified: / 30.7.1999 / 19:59:54 / cg"
! !

!SwikiAction methodsFor:'save and restore'!

convert
	"Make all my pages obey the new format -- all versions in one file, file name is just a single number."

    | old new nn |

    urlmap pages do: [:page |
	old _ page file.   nn _ old size.
	(old at: nn) isDigit & ((old at: nn-1) == $.) ifFalse: [
	    self error: 'May not be an old style page'
	] ifTrue: [
	    new _ old copyFrom: 1 to: nn-2.        "knock off .4"
	    page file: new.
	    page text: (FileStream oldFileNamed: old) contentsOfEntireFile
	]
    ].
        
!

fixAllLineEndings
    "for each page in the swiki, fixes line endings to be CR only.
     This is used to bring older Swikis up to date with the newer convention.
     If a page has no LF's, then it is left alone"

    | origText newText pagesDone |

    pagesDone _ 0.

    ProgressIndicator
	displayProgress:'updating pages...' 
	at:Sensor cursorPoint 
	from:0 to:urlmap pages size  
	during:[:bar |
	    urlmap pages do: [ :page |
		origText _ page text.
		newText _ (self class withSqueakLineEndings:origText).
		origText = newText ifFalse: [ page text: newText ].

		pagesDone _ pagesDone + 1.
		bar value: pagesDone. 
	    ] 
	]

    "
    "
!

restore: nameOfSwiki
    "Read all files in the directory 'nameOfSwiki'.  
     Reconstruct the url map."

    | map page folder dir rep |

    folder _ (ServerAction serverDirectory), nameOfSwiki.

    dir _ folder asFilename.
    (dir exists and:[dir isDirectory]) ifFalse:[
	^ self error:'non existing directory: ' , folder.
    ].

    map _ self class mapClass new.
    self map: map.
    self formatter: (HTMLformatter new initialize).
    self formatter specialCharacter: $*.
    self name: nameOfSwiki.
    self source: (Array with:nameOfSwiki with:'swiki').

    map action: self.
    map pages: (Dictionary new).
    map directory: nameOfSwiki. "This is where the pages are."

    "/ find all files with a numeric fileName
    "/ and generate pages for them.
    dir directoryContents do: [:fName |
	(dir construct:fName) isDirectory ifFalse:[
	    rep _ fName detect: [:char | char isDigit not] ifNone: [$3].
	    rep isDigit ifTrue: ["all are digits"
		page _ self class pageClass new.
		page 
		    fromFileNamed:(folder asFilename construct:fName) name 
		    action: self.
		(page time isNil) ifTrue: [page time: Time now].
		map at: page name put: page
	    ]
	]
    ].
    HTTPServer link: nameOfSwiki to: self.

    "Modified: / 30.7.1999 / 19:50:37 / cg"
!

restore: nameOfSwiki fromServer:aServer
    "Read all files in the directory 'nameOfSwiki'.  
     Reconstruct the url map."

    | map page folder dir rep |

    folder := (ServerAction serverDirectory), nameOfSwiki.

    dir := folder asFilename.
    (dir exists and:[dir isDirectory]) ifFalse:[
	^ self error:'non existing directory: ' , folder.
    ].

    map := self class mapClass new.
    self map: map.
    self formatter: (HTMLformatter new initialize).
    self formatter specialCharacter: $*.
    self name: nameOfSwiki.
    self source: (Array with:nameOfSwiki with:'swiki').

    map action: self.
    map pages: (Dictionary new).
    map directory: nameOfSwiki. "This is where the pages are."

    "/ find all files with a numeric fileName
    "/ and generate pages for them.
    dir directoryContents do: [:fName |
	(dir construct:fName) isDirectory ifFalse:[
	    rep := fName detect: [:char | char isDigit not] ifNone: [$3].
	    rep isDigit ifTrue: ["all are digits"
		page := self class pageClass new.
		page 
		    fromFileNamed:(folder asFilename construct:fName) name 
		    action: self.
		(page time isNil) ifTrue: [page time: Time now].
		map at: page name put: page
	    ]
	]
    ].
    aServer link: nameOfSwiki to: self.

    "Modified: / 30.7.1999 / 19:50:37 / cg"
!

restoreFrom: filename
    "restore my map to aFilename"

    |f|

    f _ BinaryObjectStorage onOld:(filename readStream).
"/    f _ ReferenceStream fileNamed: filename.
    self map: f next.
    self map action: self.
    f close.

    "Modified: / 30.7.1999 / 16:45:42 / cg"
!

rollBack: aDate at: aTime
    "Roll back the entire server to a previous state.  
     This does not erase data, just moves an older page to the end."

    | folder dir rep page |

    folder _ (ServerAction serverDirectory), name.
    dir _ folder asFilename.
    dir directoryContents do: [:fName |
	rep _ fName detect: [:char | char isDigit not] ifNone: [$3].
	rep isDigit ifTrue: ["all are digits"
	    page _ urlmap atID: fName.
	    page rollBack: aDate at: aTime
	]
    ].
!

saveTo: filename
    "save my map to aFilename"

    |f|

"/    f _ ReferenceStream fileNamed: filename.
    f _ BinaryObjectStorage onNew:(filename asFilename writeStream).
    f nextPut: self map; close.

    "Modified: / 30.7.1999 / 16:45:33 / cg"
! !

!SwikiAction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/SwikiAction.st,v 1.27 2003/03/26 10:28:27 stefan Exp $'
! !
