"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#SwikiPage
	instanceVariableNames:'url map formatted address file date time coreID name user
		editReqDate editReqTime temp pageStatus'
	classVariableNames:'OutputFormat'
	poolDictionaries:''
	category:'PWS-Swiki'
!


!SwikiPage class methodsFor:'initialization'!

initialize
    "SwikiPage initialize"

    OutputFormat _ OrderedCollection new.
    OutputFormat
	add: [:thePage | 'name: ''', thePage name printString, '''; '];
	add: [:thePage | 'date: ''', thePage date mmddyyyy printString, '''; '];
	add: [:thePage | 'time: ''', thePage time printString, '''; '];
	add: [:thePage | 'by: ''', thePage by printString, '''; '];
	add: [:thePage | 'pageStatus: #', thePage pageStatus, '; ']";
	add: [:thePage | 'text: ']."
! !

!SwikiPage class methodsFor:'accessing'!

outputFormat

	^OutputFormat
! !

!SwikiPage methodsFor:'accessing'!

address
	"Answer the receiver's 'address'."

	^address
!

address: anObject
	"Set the receiver's instance variable 'address' to be anObject."

	address := anObject
!

by
	| who |
	who _ user class == String
					ifFalse: [address]
					ifTrue: [user].
		who ifNil: [who _ 'unknown user'].
	^who.
!

by: who

	self address: who.
!

coreID
	"Answer the receiver's 'coreID'."

	^coreID
!

coreID: anObject
	"Set the receiver's instance variable 'coreID' to be anObject."

	coreID := anObject
!

date
	"Answer the receiver's 'date'."

	^date
!

date: anObject
    "Set the receiver's instance variable 'date' to be anObject."

    (anObject isKindOf: String)
	ifTrue: [date _ Date readMMDDYYYYFrom:anObject onError:[Date today]]
	ifFalse: [date := anObject]
!

file
	"Answer the receiver's 'file'."

	^file
!

file: anObject
	"Set the receiver's instance variable 'file' to be anObject."

	file := anObject
!

formatted
	^formatted
!

formatted: aString
	"Never set this except on a copy"
	formatted _ aString
!

lastEditTime
    "Describe when this page was edited last"

    | timeNow timeOfAccept tt timeOfReq |

    timeOfAccept _ (tt _ self timeOfAccept) asSeconds + date asSeconds.
    editReqDate ifNotNil: [
	timeOfReq _ editReqTime asSeconds + editReqDate asSeconds.
	(timeOfAccept < timeOfReq) ifTrue: [
	    timeNow := Time now.
	    (timeNow asSeconds + Date today asSeconds - (60*60)) < timeOfReq ifTrue: [
		^ '<b>Someone seems to be editing this page!!</b>  Editing started ', 
		  ((Time fromSeconds:(timeNow - editReqTime)) intervalString), ' ago'
	    ]
	]
    ].
    ^ 'Last edited: ', date printString, ' at ', tt printString
!

map
	"Answer the receiver's 'map'."

	^map
!

map: anObject
	"Set the receiver's instance variable 'map' to be anObject."

	map := anObject
!

name
	"Answer the receiver's 'name'."

	^name
!

name: anObject
	"Set the receiver's instance variable 'name' to be anObject."

	name := anObject.
	^''
!

noteEditRequest
	editReqDate _ Date today.
	editReqTime _ Time now.
!

outputFormat
	^self class outputFormat
!

pageStatus
    "The basic stati ;) are #new and #standard. 
     If the pageStatus is nil, we make make the page #standard, 
     as that is the old behavior. 
     You could add other types to get e.g., different kinds of URLs, 
     or to exclude pages from a search, or alter the formatter, or..."

    pageStatus ifNil: [pageStatus _ #standard].
    ^pageStatus
!

pageStatus: aSymbol
    "While I'm trying to be careful about default behavior if the type is
     unknown, one should still be careful to check all the places that pageType
     is used."

    pageStatus _ aSymbol.
!

placeID
    "Temporarily hold onto the placeID value.  
     For putting into the FORM in insert.html"

    ^ temp
!

placeID: idNum
    "Temporarily hold onto the placeID value.  
     For putting into the FORM in insert.html"

    temp _ idNum
!

text
    | theFile theText real bb gotTo idx|

    theFile _ file asFilename readStream.
    theFile setToEnd.
    bb _ self backupAChunk: theFile.
    bb _ bb - 7.            "before the back:"

    "Be careful about occurances of 'text:' in the name or other field"
    [
	"/ theFile match: 'text:'.
	theFile skipThroughAll:'text:'.    
	gotTo _ theFile position.
	theFile upTo: $'; skip: -1.     
	theText _ theFile nextDelimited: $'.
	real _ (ReadStream on: theText) nextDelimited: $!!.      "Remove double !!"
	theFile skipSeparators.
	theFile position >= bb
    ] whileFalse: [theFile position: gotTo].

    theFile close.

    "/ cut off trailing empty lines
    idx := real size.
    [idx > 0
     and:[(real at:idx) == Character cr]] whileTrue:[
	idx := idx - 1
    ].
    idx < (real size-1) ifTrue:[
	^ real copyTo:idx+1
    ].
    ^ real

    "Modified: / 30.7.1999 / 17:05:11 / cg"
!

text: aString
    "Add onto the end of the file"

    | this aFile start end realString |

    realString _ self uniqueInsertionPoints: aString.

    this _ String streamContents: [:ss |
	ss nextPutAll: self chunk1.
	realString storeOn: ss.
	ss nextPutAll: ' back: '
    ].

    aFile _ file asFilename writeStream.
    aFile setToEnd.
    start _ aFile position.
    aFile nextChunkPut: this; skip: -1.     "undo the !! at end"
    end _ aFile position.
    aFile 
	nextPutAll: (end - start) printString; 
	nextPut: $!!; 
	cr; 
	close.

    "Modified: / 30.7.1999 / 17:11:47 / cg"
!

textVersion: steps
    "One step back gets the original"

    | theFile theText real bb gotTo count |

    theFile _ file asFilename readStream.
    theFile setToEnd.
    bb _ 1. count _ steps.
    [count > 0 and: [bb > 0]] whileTrue: [
	bb _ self backupAChunk: theFile. 
	count := count - 1.
    ].
    bb = 0 ifTrue: ["Went back beyond beginning"
	theFile close.
	^ '<Too Far Back>'.
    ].
    bb _ bb - 7.            "before the back:"

    "Be careful about occurances of 'text:' in the name or other field"
    [
	"/ theFile match: 'text:'.
	theFile skipThroughAll:'text:'.    
	gotTo _ theFile position.
	theFile upTo: $'; skip: -1.
	theText _ theFile nextDelimited: $'.
	real _ (ReadStream on: theText) nextDelimited: $!!.      "Remove double !!"
	theFile skipSeparators.
	theFile position >= bb
    ] whileFalse: [theFile position: gotTo].

    theFile close.
    ^ real

    "Modified: / 30.7.1999 / 17:06:17 / cg"
!

time
	"Answer the receiver's 'time'."

	^time
!

time: anObject
    "Set the receiver's instance variable 'time' to be anObject."
    (anObject isKindOf: String)
	    ifTrue: [time _ Time readFrom:anObject]
	    ifFalse: [time := anObject]

!

timeOfAccept
    "return the Time when the text was written on the file"

    | theFile pp theReturn |

    theFile _ file asFilename readStream.
    theFile setToEnd.
    pp _ self backupAChunk: theFile.
    pp _ pp - 5.            "before the back:"
    theFile upTo: $'; skip: -1.             "name"
    theFile nextDelimited: $'.
    theFile upTo: $'; skip: -1.             "date"
    theFile nextDelimited: $'.
    theFile upTo: $'; skip: -1.             "time"
    theReturn := Time readFromString:(theFile nextDelimited: $').
    theFile close.
    ^ theReturn

    "Modified: / 30.7.1999 / 17:05:48 / cg"
!

url
	"Answer the receiver's 'url'."

	^url
!

url: anObject
	"Set the receiver's instance variable 'url' to be anObject."

	url := anObject
!

user
	^ user
!

user: aString

	aString class == String 
		ifFalse: [user _ aString printString]
		ifTrue: [user _ aString]
! !

!SwikiPage methodsFor:'insertion points'!

unique: aTag in: aSet
    "If aTag is not in aSet, put it in and return aTag.  If it's there,
     make up a new tag, insert it, and return it.  For keeping insertion points
     in text unique, so can tell where to put incoming text."

    | base new |

    ((aSet includes: aTag) not and: [aTag size > 0]) ifTrue: [
	aSet add:aTag.  
	^ aTag
    ].

    base _ 0.
    [
	new _ ((1 to:100) atRandom + base) printString.
	aSet includes: new
    ] whileTrue:[
	base _ base + 50
    ].
    aSet add: new.
    ^ new
!

uniqueInsertionPoints: aString
    "Find all occurances of '*append here'.  Make them lower case.  If
     they don't have a number after them, assign one.  Don't use the same number
     twice in this page.  Place ID can actually be any string."

    | sourceStream targetStream ids char pos inside rest |

    sourceStream := ReadStream on: aString.
    targetStream := WriteStream on: String new.

    ids _ Set new.  "id numbers that have been used"
    [sourceStream atEnd] whileFalse: [
	targetStream nextPut: (char _ sourceStream next).
	char == $* ifTrue: [
	    pos _ sourceStream position.
	    inside _ sourceStream upTo: $*.
	    (inside asLowercase startsWith: 'append here') ifFalse:[
		sourceStream position: pos.   "ignore"
		((inside = '') and: [sourceStream atEnd not]) ifTrue: [
		    targetStream nextPut: (char _ sourceStream next)]
		    "Honor **"
	    ] ifTrue: [
		"See if it has a number"
		targetStream nextPutAll: 'append here '.
		rest _ inside copyFrom: (13 min: inside size +1) to: inside size.
		targetStream nextPutAll: (self unique: rest in: ids); nextPut: $*
	    ]
	]
    ].
    ^ targetStream contents
! !

!SwikiPage methodsFor:'printing & storing'!

displayString
    ^ 'SwikiPage(' , address printString, ' - ' , file printString , ')'
! !

!SwikiPage methodsFor:'save & restore'!

backupAChunk: aFile
    "We are positioned at the end of a chunk.  Read the size of this
     chunk from its end.  Back up one chunk.  Return the position of the space
     before the back number."

    | back pp aPiece isDigitFlag |

    aFile position < 3 ifTrue:[^0].

    aFile skip: -2.
    isDigitFlag _ false.
    [isDigitFlag] whileFalse:[
	aPiece := aFile next. 
	aPiece isNil ifTrue: [^0].
	aPiece isDigit ifFalse: [
	    aFile skip: -2.
	] ifTrue: [
	    isDigitFlag _ true
	].
    ].

    "[aFile next isDigit] whileFalse: [aFile skip: -2]." "This was Ted's elegant version.
The uglier one above is to be able to test backing up too far."
    aFile skip: -2.
    [
	pp _ aFile position + 1. 
	aFile next isDigit
    ] whileTrue: [
	aFile skip: -2.
    ].
    back  _ (aFile upTo: $!!) asNumber.
    aFile position: pp-back.
    ^ pp "now positioned at the end"

    "Modified: / 30.7.1999 / 13:16:50 / cg"
!

chunk1
    "actually, this returns a storeString ... without the actual text"

    ^ self formatForOutput
	"| who |
	^ String streamContents: [:ss |
		ss nextPutAll: 'self name: '; nextPutAll: name printString;
		nextPutAll: ' date: '''; nextPutAll: date mmddyyyy.
		ss nextPutAll: ''' time: '''.  time print24: true on: ss.
		who _ user class == String ifFalse: [address] ifTrue: [user].
		who ifNil: [who _ 'unknown user'].
		ss nextPutAll: ''' by: '; nextPutAll: who printString.
		ss nextPutAll: ' text: ']"
!

condenseChanges
    "Save only the most recent version of this page"

    | theFile recentChunk newFile |

    theFile _ FileStream oldFileNamed: file.
    theFile setToEnd.
    self backupAChunk: theFile.
    recentChunk _ theFile nextChunk.
    theFile close. 

    "/ FileDirectory deleteFilePath: file.
    newFile _ FileStream newFileNamed:(file , '.new').
    newFile nextChunkPut: recentChunk; cr; close.

    file asFilename delete.
    (file , '.new') asFilename renameTo:file.

    "
     SwikiPage allInstances do:[:p |
	p condenseChanges
     ]
    "

!

formatForOutput
    "actually, this returns a storeString ... without the actual text"

    ^ String streamContents: [:ss |
	ss nextPutAll: 'self '.
	self outputFormat do: [:block |
		ss nextPutAll:(block value: self)
		    "ifError: [:msg :rec |
				 ss nextPutAll:'*** ', rec asString, ': ', msg asString ]"].
		ss nextPutAll: 'text: '
	].

"                  nextPutAll: name printString;
		   nextPutAll: ' date: ''';
		   nextPutAll: date mmddyyyy.
		ss nextPutAll: ''' time: '''.
		     Time now print24: true on: ss.
		who _ user class == String
					ifFalse: [address]
					ifTrue: [user].
		who ifNil: [who _ 'unknown user'].
		ss nextPutAll: ''' by: ';
		   nextPutAll: who printString.
		ss nextPutAll: ' text: ']
"
!

fromFileNamed: filePath action: aSwikiAction
    "Fill in all parts of this page from the info in its file"

    map _ aSwikiAction urlmap.
    file _ filePath.
    coreID _ file asFilename baseName. "/ localPathName.
    url _ aSwikiAction name, '.', coreID.
    self scanFrom: file asFilename readStream. "name, date"

    "Modified: / 30.7.1999 / 17:04:31 / cg"
!

name:pageName date:dateString time:timeString by:who text:theString back:bytes
    "Record the name of this page during startup.  When reading the
     page in order to serve it, this is NOT executed.  The text is read from the
     file explicitly."

    name _ pageName.
    date _ dateString asDate.
    time _ timeString asTime.
    address _ who
!

rollBack: aDate at: aTime
    "Roll back this page to a previous state.  
     This does not erase data, just moves an older version to the end."

    | theFile pp dd tt recentChunk page |

    theFile _ FileStream oldFileNamed: file.
    theFile setToEnd.
    [theFile position < 9 "8" "beginning"] whileFalse: [
	self backupAChunk: theFile.
	pp _ theFile position.
	theFile upTo: $'; skip: -1.             "name"
	theFile nextDelimited: $'.
	theFile upTo: $'; skip: -1.             "date"
	dd _ (theFile nextDelimited: $') asDate.
	theFile upTo: $'; skip: -1.             "time"
	tt _ theFile nextDelimited: $'.
	theFile position: pp.
	(dd < aDate) | ((dd = aDate) & (tt asTime < aTime)) ifTrue: [
	    recentChunk _ theFile nextChunk.
	    theFile setToEnd.
	    theFile nextChunkPut: recentChunk; cr; close.
	    date _ dd.      "That's my date now"
	    ^ self
	]
    ].
    "none that early, store out a blank page"
    theFile close.
    page _ map
	    storeID: coreID
	    text: 'blank text (but prevous versions do exist)'
	    from: self peerName.
    page user: 'unknown user' "oldPage userID".
!

scanFrom: aFileStream
    "Read my name off the file for this page.  Other pages contain a ref to me like *myName* "

    |rslt|

    aFileStream setToEnd.
    self backupAChunk: aFileStream.

    "extract all data from the chunk"
    rslt := Compiler evaluate: (aFileStream nextChunk) for: self logged: false.
    rslt == #Error ifTrue:[^ self].

    map at: name asLowercase put: self.
    aFileStream close.

    "Modified: / 14.12.1999 / 17:26:48 / cg"
!

text: theString back: bytes
    "Record the name of this page during startup.  
     When reading the page in order to serve it, this is NOT executed.  
     The text is read from the file explicitly."

    "Which means that you'd better not cascade text & back or you're royally
     screwed ;) BJP"
! !

!SwikiPage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/SwikiPage.st,v 1.15 2003/03/26 10:30:55 stefan Exp $'
! !

SwikiPage initialize!
