"{ Package: 'stx:goodies/webServer/pwsSwiki' }"

"{ NameSpace: PWS }"

Object subclass:#URLmap
	instanceVariableNames:'pages directory action'
	classVariableNames:''
	poolDictionaries:''
	category:'PWS-Swiki'
!


!URLmap methodsFor:'accessing'!

action
	"Answer the receiver's 'action'."

	^action
!

action: anObject
	"Set the receiver's instance variable 'action' to be anObject."

	action := anObject
!

at: key
    "Return page of a given key."

    ^ pages at: key asLowercase ifAbsent: [nil]
!

at: key put: anObject
	"Add anObject to the pages list."

	pages at: key asLowercase put: anObject
!

atID: id
    "Return page of a given key."

    | idString |

    idString _ id isInteger 
		ifTrue: [id printString] 
		ifFalse: [id].
    ^ pages 
	detect: [:page | page coreID = idString] 
	ifNone: [id = '1'
		    ifTrue:[nil]
		    ifFalse:[self atID: '1']]

    "Modified: / 30.7.1999 / 17:05:27 / cg"
!

directory
	"Answer the receiver's 'directory'."

	^directory
!

directory: anObject
	"Set the receiver's instance variable 'directory' to be anObject."

	directory := anObject
!

externalURL: aString
    "make a non-local  URL"
    ^ '<a href="',aString,'">',aString,'</a>'

    "Modified: / 30.7.1999 / 21:48:24 / cg"
!

imageURL:aString
    "make a non-local image URL"

    ^ '<img src="', aString,'" alt="', aString,'">'

    "Modified: / 30.7.1999 / 21:48:01 / cg"
!

localActionURL: aString
    |string|

    "fake up a more ordinary URL by dropping the shriek chars"
    string _ aString copyWithout: $!!.
    ^'<a href="',string,'">',string,'</a>'

    "Modified: / 30.7.1999 / 21:49:24 / cg"
!

localImageURL:aString
    "make a local image URL"

    ^ '<img src="' , action name, '/', aString, '" alt="', action name ,'/',aString,'">'

    "Modified: / 30.7.1999 / 21:48:54 / cg"
!

pageURL:aPage
    ^ self pageURL:aPage urlOnly:false.

    "Modified: / 30.7.1999 / 21:58:14 / cg"
!

pageURL:aPage urlOnly:urlOnly
    |url|

    (aPage pageStatus = #new) ifTrue:[
	"Underlines new but not yet edited pages"
	(action isKindOf: CachedSwikiAction) ifFalse: [
	    url := (action pageURL: aPage) ,'.edit'.
	    urlOnly ifTrue:[^ url].
	] ifTrue: [
	    url := (action pwsURL),(action name),'.',(aPage coreID),'.edit'.
	    urlOnly ifTrue:[^ url].
	].
	^ '<u>',(aPage name),'</u><a href="',url,'">?</a>'
    ].

    url := action pageURL: aPage.
    urlOnly ifTrue:[^ url].
    ^ '<a href="',url,'">',(aPage name),'</a>'

    "Logically, there should be editPageUrl in SwikiAction as well. 
     After all, we might want to define other URL schemes."

    "Created: / 30.7.1999 / 21:57:50 / cg"
    "Modified: / 30.7.1999 / 22:05:24 / cg"
!

pages
	"Answer the receiver's 'pages'."

	^pages
!

pages: anObject
	"Set the receiver's instance variable 'pages' to be anObject."

	pages := anObject
! !

!URLmap methodsFor:'creating'!

allPagesFrom: pageRef
	^self allPagesFrom: pageRef for: '' "Probably won't create new pages here"
!

allPagesFrom: pageRef for: request
    | formattedPage refPages allPages peer |

    (request isKindOf: HTTPServer) ifTrue: [
	peer _ request peerName
    ] ifFalse:[
	peer _ ''
    ].
    formattedPage _ pageRef copy.  "Make a copy, then format the text."
    refPages _ OrderedCollection new.
    formattedPage formatted: (action formatter swikify: (pageRef text)

    linkhandler: [:link | self linkFor: link
			    from: peer
			    storingTo: refPages]).
    "This filtering should be generalized a tad, methinks. 
     Perhaps incorparated into the linkhandler."
    refPages _ refPages reject: [:page | page pageStatus = #new].
    "Now, put all referenced pages into the page"
    allPages _ WriteStream on: String new.
    allPages nextPutAll: '<h2>',(pageRef name),'</h2>', formattedPage formatted.
    refPages do: [:page |
	    allPages nextPutAll: '<h2>',(page name),'</h2>'.
	    allPages nextPutAll: (HTMLformatter swikify: (page text)

    linkhandler: [:link | self linkFor: link
			    from: peer
			    storingTo:(OrderedCollection new)]).].
    formattedPage formatted: allPages contents. "Put all the pages into THE page"
    ^formattedPage

    "Modified: / 30.7.1999 / 15:27:54 / cg"
!

newpage: label from: peer
    | newpage newfile |

    newfile _ ((pages inject: 0 into: [:max :p |
	    p coreID asNumber > max ifTrue: [p coreID asNumber]
			    ifFalse: [max]]) + 1) printString.
    newpage _ action class pageClass new.
    self at: label put: newpage.
    newpage address: peer.
    newpage date: (Date today).
    newpage time: (Time now).
    newpage coreID: newfile.
    newpage name: label.
    newpage file: ((ServerAction serverDirectory),
	    directory, (ServerAction pathSeparator), newfile).
    newpage pageStatus: #new.
    newpage text: 'Describe ',label,' here'.
    newpage map: self.
    newpage url: (action name),'.',newfile.
    ^newpage
!

storeID: id text: text from: peer
	| page |
	page _ self atID: id.
	page date: (Date today).
	page time: (Time now).
	page text: text.
	page address: peer.
	^ page
!

storeID: id text: insertedText insertAt: idString
    "Insert in a place in the text.  Just before '*append here 34*' if idString is '34'."

    | page bigText ind toInsert |

    page _ self atID: id.
    page date: (Date today).
    bigText _ page text.
    ind _ bigText findString: '*append here ',idString,'*' startingAt:1.      "always lower case"
    ind = 0 ifTrue: [^ page].               "could not find that place"
    "Make sure new text surrounded by line feeds"
    toInsert _ (insertedText last == Character linefeed)
	    ifTrue: [insertedText]
	    ifFalse: [insertedText, (String with: Character linefeed)].
    ((insertedText first == Character linefeed) or: [
	    (bigText at: (ind-1 max: 1)) == Character linefeed])
		    ifFalse: [toInsert _ (String with: Character linefeed), toInsert].
    page text: (bigText copyReplaceFrom: ind to: ind-1 with: toInsert).
    "page address: peer.  Don't sign with person who only added"
    ^ page
!

storePage: page text: text from: peer
	page text: text.
	page address: peer.
	page date: (Date today).
! !

!URLmap methodsFor:'fileRendering'!

addToGlossary: myName as: myURL
    |aStream |

    "Add to a CR-delimited file of 'pageName' and 'URL' separated by vertical bars '|'"
    aStream _ directory oldFileNamed: 'glossary'.
    aStream open.
    aStream setToEnd.
    aStream nextPutAll: myName,'|',myURL; cr.
    aStream close.


        
!

readGlossary: aStream
    |aLine pieces name url newpage|

    "Expect a CR-delimited file of 'pageName' and 'URL' separated by vertical bars '|'"
    self pages: (Dictionary new).
    aStream open.
    [aStream atEnd] whileFalse:[
	aLine := aStream upTo: (Character cr).
	pieces _ aLine findTokens: '|'.
	name _ pieces at: 1.
	url _ pieces at: 2.
	newpage _ SwikiPage new.
	newpage coreID: url.
	newpage map: self.
	newpage name: name.
	pages at: name asLowercase put: newpage
    ].
    aStream close.


        
!

writeGlossary
    "Write out the pages dictionary as a CR-delimited file of pagename and URL"

    |aStream |

    aStream _ directory oldFileNamed: 'glossary'.
    aStream open.
    pages keys asSortedCollection do: [:each |
	aStream nextPutAll: (pages at: each) name,'|',(pages at:each) coreID; cr.
    ].
    aStream close.


        
! !

!URLmap methodsFor:'linking'!

linkFor:nameString from:peer storingTo:aList
    "generate an HREF for a given name
     (i.e. nameString is what is enclosed in *...*).
     Return a string.
     New: *!! ...* is expanded to the URL only,
     not to a complete link. This can be used to
     add a link to a swikiPage into a POST or GET action."

    |string uString newpage urlOnly|

    string := nameString.
    urlOnly := (string at:1) == $!!.
    urlOnly ifTrue:[
	string := string copyFrom:2.    
    ].
    uString _ string asUppercase.

    (self isStringRooted: uString) ifTrue: [
	"an external URL"
	(self isStringAnImage: uString) ifTrue: [
	    "Looks like an image URL"
	    ^ self imageURL: string
	] ifFalse: [
	    ^ self externalURL: string
	]
    ] ifFalse: [ 
	"Serious!! Gotta provide-a-link!! But check for image or local action first"
	(self isStringAnImage: uString) ifTrue: [
	    ^ self localImageURL: string
	].
	(self isStringALocalAction: uString) ifTrue:[
	    ^ self localActionURL: string
	].

	newpage _ pages at: string asLowercase ifAbsent: [nil].
	newpage isNil ifTrue: [ 
	    "Create a new page"
	    newpage _ self newpage: string from: peer.
	].
	aList notNil ifTrue:[
	    (aList indexOf: newpage) ~= 0 ifFalse: [
		"Add only if not there"
		aList add: newpage
	    ].
	].
	^ self pageURL:newpage urlOnly:urlOnly
    ]

    "Modified: / 30.7.1999 / 21:57:43 / cg"
!

linkFor:nameString from:peer storingTo:aList page:aPage
    "generate an HREF for a given name
     (i.e. nameString is what is enclosed in *...*).
     Return a string"

    |uString placeID|

    uString _ nameString asUppercase.

    "/ care for *APPEND HERE

    (uString startsWith: 'APPEND HERE') ifTrue: [
	placeID _ uString copyFrom: (13 min: uString size) to:uString size.
	^ '<a href="', aPage url, '.insert.', placeID ,'">','Append Here','</a>'
	"later an image here"
    ].
    ^ self linkFor:nameString from:peer storingTo:aList.

    "Modified: / 30.7.1999 / 21:46:02 / cg"
!

linkFor: string to: peer withPrefix: prefix storingTo: aList
    | uString newpage |

    uString _ string asUppercase.
    (uString indexOfSubCollection: 'HTTP' startingAt: 1) = 1 ifTrue:[
	((uString endsWith: 'GIF') 
	or: [(uString endsWith: 'JPEG') 
	or: [uString endsWith: 'JPG']])
	    ifTrue: [ ^ '<image src="',string,'">' ]
	    ifFalse: [^ '<a href="',string,'">',string,'</a>']
    ] ifFalse: [
	"Serious!! Gotta provide-a-link!!"
	newpage _ pages at: string ifAbsent: [nil].
	newpage isNil ifTrue: [ "Create a new page"
		newpage _ self newpage: string from: peer.].
	aList add: newpage.
	^'<a href="',prefix,'.',newpage coreID,'">',string,'</a>'
    ]
! !

!URLmap methodsFor:'private'!

isStringALocalAction: anUpperCasedString
    "check the string to see if it starts and end with a shriek 
     - code for a Swiki action such as Comment or whatever"

    ^ (anUpperCasedString size >=2) 
      and:[anUpperCasedString first = $!! 
      and:[anUpperCasedString last= $!!]]

    "Modified: / 30.7.1999 / 21:53:09 / cg"
!

isStringAnImage: anUpperCasedString
    "check the string to see if it ends with something that makes it likely 
     to be an image URL"

    "/ rubbish - ask MIMETypes here ...

    ^ (anUpperCasedString endsWith: '.GIF') 
      or:[(anUpperCasedString endsWith: '.JPEG') 
      or:[anUpperCasedString endsWith: '.JPG']]

    "Modified: / 30.7.1999 / 21:52:26 / cg"
!

isStringRooted:anUpperCasedString
    "check the string to see if it starts with something that makes it
     likely to be a rooted URL"

    ^ (anUpperCasedString startsWith: 'HTTP')
      or:[(anUpperCasedString startsWith:'FTP')
      or:[(anUpperCasedString startsWith: 'MAILTO')]]

    "Modified: / 30.7.1999 / 21:51:40 / cg"
! !

!URLmap methodsFor:'searching'!

recent
    | response sortedPages currentDate |

    sortedPages _ pages reject: [:page | page pageStatus = #new].

"We definitely want the reject: first, so we sort a smaller collection.
Having the variable name be 'sortedPages' is a bit of a misnomer, though."

"I think we shouldn't do all this for each call to recent changes. After
all, most of the time, it's *hasn't* changed, and when it *has*, it's
pretty much only added something to the end. At the very least we should
cache the sorted, filtered collection."

"And really, why is this in URLmap? Recent Changes is a kind of page. I'd
rather see it as a swikipage, perhaps standardly defined (that is, when you
set up a swiki, you get a front page, a formatting page, and a Recent
Changes page). That is, SwikiPages could store text (as they do now) or
cgi-ish functions (like recent). That way, we could design custom templates
for them, etc.

At the very least, there needs to be better labeling. Why not call the
category with these page functions, 'dynamic pages', or somethign similar?"

"A different condsideration: It would be nice to have arbitrary filters:
e.g., 'past ten days', 'all pages *not* yet edited', etc., and be able to
combine them. Would work as a 'search for various attributes' feature which
one might or might not expose to the user."

    sortedPages _ sortedPages asSortedCollection: [:a :b | (a date = b date) 
							   ifTrue: [a time > b time]
							   ifFalse: [a date > b date]
						  ].

    response _ WriteStream on: String new.
    response nextPutAll: '<h2>Recent Changes</h2><ul>'.
    currentDate _ Date new.
    sortedPages do: [:page |
	|addr|

	(currentDate ~= page date) ifTrue: [
	    currentDate _ page date.
	    response 
		nextPutAll: '</ul><p><b>';
		nextPutAll: (currentDate printString);
		nextPutAll: '</b><p><ul>'.
	].

	addr := page address.
	addr isString ifFalse:[
	    addr := addr printString , ' [' , addr hostName , ']'
	].
	response 
	    nextPutAll: '<li>';
	    nextPutAll: (self pageURL:page);
	    nextPutAll: '...';
	    nextPutAll: addr.
    ].
    response nextPutAll: '</ul>'.
    ^ response contents

                
!

searchFor: aString
    | hitlines response |

    hitlines _ pages select: [:each | each text includesSubstring: aString caseSensitive: false].
    hitlines isEmpty ifTrue: [^ aString, ' not found'].

    response _ WriteStream on: String new.
    response nextPutAll: '<h2>Search results for ' , aString , '</h2><ul>'.
    hitlines do: [:each |
	response nextPutAll: '<li>',(self pageURL: each) , '...' , (each address printString)
    ].
    response nextPutAll: '</ul>'.
    ^ response contents.
! !

!URLmap class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/pwsSwiki/URLmap.st,v 1.13 2003/03/26 10:29:21 stefan Exp $'
! !
