"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:goodies/webServer/ui' }"

AbstractSettingsApplication subclass:#AbstractServerSettingsApplication
	instanceVariableNames:'httpServerInstance serverStatus subApplications parentApplication
		defaultSettingsApplication'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTTP-Settings'
!

!AbstractServerSettingsApplication class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 2003 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Abstract superclass for HTTP-Server settings applications.
    Instances of me are meant to be used as subcanvas in a 
    SettingsDialog (switched when seleting tree items).

    [author:]
        Christian Penk

    [see also:]
        SettingsDialog
"
! !

!AbstractServerSettingsApplication class methodsFor:'defaults'!

nameInSettingsDialog

    ^ self nameWithoutPrefix
! !

!AbstractServerSettingsApplication methodsFor:'accessing'!

defaultSettingsApplication
    ^ defaultSettingsApplication
!

defaultSettingsApplication:aBoolean
    defaultSettingsApplication := aBoolean.
!

httpServerInstance
    "return the value of the instance variable 'httpServerInstance' (automatically generated)"

    ^ httpServerInstance
!

httpServerInstance:something
    "set the value of the instance variable 'httpServerInstance' (automatically generated)"

    httpServerInstance notNil ifTrue:[
        httpServerInstance serverStatusHolder removeDependent:self.
    ].
    httpServerInstance := something.
    httpServerInstance notNil ifTrue:[
        httpServerInstance serverStatusHolder addDependent:self.
        self serverStatus value:(httpServerInstance serverStatusHolder value).
    ].
!

nameInSettingsDialog

    ^ self class nameInSettingsDialog
!

parentApplication
    "return the value of the instance variable 'parentApplication' (automatically generated)"

    ^ parentApplication
!

parentApplication:something
    "set the value of the instance variable 'parentApplication' (automatically generated)"

    parentApplication := something.
!

subApplications
    "return the value of the instance variable 'subApplications' (automatically generated)"

    subApplications isNil ifTrue:[
        subApplications := OrderedCollection new.
    ].
    ^ subApplications
! !

!AbstractServerSettingsApplication methodsFor:'actions'!

basicSaveSettings
    settingsDialog changeLabelTo:self nameInSettingsDialog application:self.
!

readSettings

    self modifiedChannel value:(self hasUnsavedChanges).
! !

!AbstractServerSettingsApplication methodsFor:'actions-subApplications'!

addSettingsApplicationForService:aService

    |preInitializeBlock settingsApplicationClass|

    settingsApplicationClass := aService settingsApplicationClass.
    settingsApplicationClass notNil ifTrue:[
        preInitializeBlock := [:aSubApplication |
            aSubApplication httpServerInstance:self httpServerInstance.
            aSubApplication serviceInstance:aService.
        ].
        self createServerSubApplication:settingsApplicationClass 
                preInitialize:preInitializeBlock
                expand:true.
    ].
!

createServerSubApplication:aSubApplicationClass preInitialize:aPreInitializeBlock expand:aBoolean
    |itemPathName newItem aSubApplication openSubApplications|

    aSubApplication := aSubApplicationClass basicNew.
    aSubApplication httpServerInstance:self httpServerInstance.
    aPreInitializeBlock notNil ifTrue:[
        aPreInitializeBlock value:aSubApplication.
    ].
    aSubApplication parentApplication:self.
    openSubApplications := self subApplicationFor:aSubApplication.
    openSubApplications notEmpty ifTrue:[
        openSubApplications do:[:eachOpenSubApplication|
            settingsDialog changeLabelTo:eachOpenSubApplication nameInSettingsDialog 
                           application:eachOpenSubApplication.
        ].
        aSubApplication release.        "not used"
    ] ifFalse:[
        " item for this application is not available "
        aSubApplication settingsDialog:self settingsDialog.
        aSubApplication basicInitialize.
        itemPathName := self itemPathNameForSubApplication:aSubApplication.
        newItem := self settingsDialog addApplClass:(aSubApplication className asSymbol) 
                                        forApplication:self 
                                        label:aSubApplication nameInSettingsDialog
                                        expand:aBoolean.
        newItem isNil ifTrue:[
            ^ self
        ].
        newItem application:aSubApplication.
        self subApplications add:aSubApplication.
        aSubApplication initialize.
    ].
    self settingsDialog sortSubApplicationsFor:self.
!

createServerSubApplications:aClassList 

    ^ self createServerSubApplications:aClassList preInitialize:nil expand:false
!

createServerSubApplications:aClassList preInitialize:aPreInitializeBlock

    ^ self createServerSubApplications:aClassList preInitialize:aPreInitializeBlock expand:false
!

createServerSubApplications:aClassList preInitialize:aPreInitializeBlock expand:doExpand

    aClassList do:[:aClass |
        self createServerSubApplication:aClass 
                preInitialize:aPreInitializeBlock
                expand:doExpand.
    ]
!

hasSubApplicationFor:aSettingsApplication

    ^ (self subApplicationFor:aSettingsApplication) notEmpty
!

openAndCloseServiceSettingsApplications
    | openSubApplications removeApplictions serviceInstanceList|

    serviceInstanceList := self httpServerInstance serviceList.
    serviceInstanceList do:[:aService|
        self addSettingsApplicationForService:aService
    ].
    openSubApplications := settingsDialog getAllChildrenAppsForApplication:self.
    removeApplictions := openSubApplications select:[:aApp| (serviceInstanceList includes:aApp serviceInstance) not].
    self removeServerSubApplicationsFor:removeApplictions.
!

removeServerSubApplicationsFor:aApplicationList

    self removeServerSubApplicationsFor:aApplicationList eachDo:nil
!

removeServerSubApplicationsFor:anApplicationList eachDo:aBlock

    anApplicationList copy do:[:anAppl |
        (self hasSubApplicationFor:anAppl) ifTrue:[
            " item for this applicational not available "
            aBlock notNil ifTrue:[
                aBlock value:anAppl.
            ].
            settingsDialog removeApplicationFromTree:anAppl.
            self subApplications remove:anAppl.
        ].
    ]
!

removeSettingsApplicationForService:aService
    |apps|

    apps := self subApplications select:[:anApp| 
        anApp serviceInstance notNil and:[anApp serviceInstance == aService] 
    ].
    apps do:[:anAppl|
        self subApplications remove:anAppl.
        settingsDialog removeApplicationFromTree:anAppl.
    ].
!

subApplicationFor:aSettingsApplication
    |openApps matchBlock|

    matchBlock := [:aSubApplication |
        | sameServer sameClass sameType sameService allSame|
        sameClass := aSubApplication class = aSettingsApplication class.
        sameServer := aSubApplication httpServerInstance hasSameSettingsLike:aSettingsApplication httpServerInstance.
        sameType := (aSubApplication isServiceSettingsApplication = aSettingsApplication isServiceSettingsApplication).
        sameService := true.
        aSubApplication isServiceSettingsApplication ifTrue:[
            sameService := aSettingsApplication serviceInstance == (aSubApplication serviceInstance)
        ].
        allSame := sameServer and:[sameClass and:[sameType and:[sameService]]].
        allSame
    ].
    openApps := settingsDialog getAllChildrenAppsForApplication:self matchBlock:matchBlock.
    ^ openApps 
! !

!AbstractServerSettingsApplication methodsFor:'aspects'!

isServiceSettingsApplication

    ^ false
!

serverStatus

    serverStatus isNil ifTrue:[
        serverStatus := '' asValue.
    ].
    ^ serverStatus.
! !

!AbstractServerSettingsApplication methodsFor:'initialization & release'!

release
    super release.

    httpServerInstance notNil ifTrue:[
        httpServerInstance serverStatusHolder removeDependent:self.
    ].
! !

!AbstractServerSettingsApplication methodsFor:'queries'!

hasChangesToSettingsFile

    |dict serverInst|

    serverInst := self httpServerInstance.    
    dict := serverInst settingsDictionary.
    dict isNil ifTrue:[^ true].
    dict keysAndValuesDo:[:selector :value |
        (serverInst respondsTo:(selector)) ifTrue:[
            ((serverInst perform:(selector asSymbol)) ~= value) ifTrue:[
                ^ true
            ].
        ].
    ].
    ^ false
!

itemPathNameForSubApplication:aSubApplication
    ^ self itemPathName , '/' , aSubApplication nameInSettingsDialog.
!

serverIsRunning
    self httpServerInstance isNil ifTrue:[ ^ false].
    ^ self httpServerInstance isServing.
! !

!AbstractServerSettingsApplication class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/ui/AbstractServerSettingsApplication.st,v 1.38 2006/07/27 16:19:22 cg Exp $'
! !
