"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

"{ Package: 'stx:goodies/xml/indelv' }"

IdNodeImpl subclass:#IdAttributeImpl
	instanceVariableNames:'name value specified'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdAttributeImpl class methodsFor:'class comment'!

classComment
	"This class implements the DOM Attr interface.

	Comment from the DOM specification:

	The Attr interface represents an attribute in an Element object. Typically 
	the allowable values for the attribute are defined in a document type definition.

	Attr objects inherit the Node interface, but since they are not actually child 
	nodes of the element they describe, the DOM does not consider them part of 
	the document tree. Thus, the Node attributes parentNode, previousSibling, 
	and nextSibling have a null value for Attr objects. The DOM takes the view 
	that attributes are properties of elements rather than having a separate identity 
	from the elements they are associated with; this should make it more efficient 
	to implement such features as default attributes associated with all elements 
	of a given type. Furthermore, Attr nodes may not be immediate children of a 
	DocumentFragment. However, they can be associated with Element nodes 
	contained within a DocumentFragment. In short, users and implementors of the 
	DOM need to be aware that Attr nodes have some things in common with other 
	objects inheriting the Node interface, but they also are quite distinct.

	The attribute's effective value is determined as follows: if this attribute has been 
	explicitly assigned any value, that value is the attribute's effective value; otherwise, 
	if there is a declaration for this attribute, and that declaration includes a default 
	value, then that default value is the attribute's effective value; otherwise, the 
	attribute does not exist on this element in the structure model until it has been 
	explicitly added. Note that the nodeValue attribute on the Attr instance can also be 
	used to retrieve the string version of the attribute's value(s). 

	In XML, where the value of an attribute can contain entity references, the child 
	nodes of the Attr node provide a representation in which entity references are not 
	expanded. These child nodes may be either Text or EntityReference nodes. Because 
	the attribute type may be unknown, there are no tokenized attribute values."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"
! !

!IdAttributeImpl class methodsFor:'constructors'!

forName: aName
	"Return a new instance for aName.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aName."

	| newAtt |
	( IdDOMImpl validateXMLName: aName ) ifFalse: [
		IdDOMImpl newExceptionInvalidCharacter signal ].
	newAtt := IdAttributeImpl new.
	newAtt initName: aName.
	^newAtt
! !

!IdAttributeImpl methodsFor:'accessing'!

getName
	"Return the name of this attribute."

	^name
!

getNodeName
	^self getName
!

getNodeType
	"Return an integer representing the receiver's
	node type. All instances share the same node
	type: ATTRIBUTE_NODE."

	^IdDOMConstants ATTRIBUTE_NODE
!

getNodeValue
	^self getValue
!

getSpecified
	"Return true if the receiver was explicitly assigned
	for the Element in which it occurs, else return false.
	Default is false. Note that false is returned for default
	attribute values derived from the DocumentType."

	^specified
!

getValue
	"Returns the value of this attribute as a string. Currently
	this assumes only TEXT values. Need to support entity references.

	Note that an effective value (see class comment) of the null string
	would be returned as a zero length string. If the attribute has no 
	effective value, then this method will return nil."

	value == nil ifTrue: [ ^nil ].
	value isTypeTEXT ifTrue: [ ^value getNodeValue ].
	^nil
!

setSpecified: aBoolean
	"Set the receiver's specified variable to aBoolean.
	This should only be set to true for attributes which
	are explicitly set for the Element in which they occur."

	specified := aBoolean
!

setValue: aString
	"Set the receiver's value to aString. This gets stored
	internally as a Text node in accordance with DOM."

	value ~~ nil ifTrue: [
		value setParent: nil ].
	value := ( IdStringNode asTEXT: aString ).
	value setParent: self
! !

!IdAttributeImpl methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. The argument deep 
	is only relevent to Element nodes and will cause the 
	entire subtree under the receiver to be copied if true. The 
	node returned by this method will not have a parent node.
	Always clone the attribute value node."

	| answer |
	answer := IdAttributeImpl forName: self getNodeName.
	answer
		setSpecified: self getSpecified;
		setValue: self getValue.
	^answer
! !

!IdAttributeImpl methodsFor:'creation'!

initName: aName
	"Initialize the name."

	name := aName
!

initialize
	"Initialize the node type to ATTRIBUTE and the
	specified flag to false."

	super initialize.
	self setSpecified: false
! !

!IdAttributeImpl methodsFor:'services'!

toString
	"Return a string representation of the receiver.
	Only the attribute value will be included as required
	by the DOM spec. Note that if the getNodeValue method
	returns nil (no specified or default value) then this
	method will return an empty string."

	| asString |
	asString := self getNodeValue.
	^asString == nil
		ifTrue: [ String new ]
		ifFalse: [ asString ]
! !

!IdAttributeImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdAttributeImpl.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
