"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdElementNode subclass:#IdCompositeNode
	instanceVariableNames:'childNodes'
	classVariableNames:'DefaultListSize'
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdCompositeNode class methodsFor:'accessing'!

getDefaultListSize
	"Return the default size used to initialize new
	childNodes collections."

	^DefaultListSize
!

setDefaultListSize: anInteger
	"Set the default size used to initialize new
	childNodes collections to anInteger."

	DefaultListSize := anInteger
! !

!IdCompositeNode class methodsFor:'class comment'!

classComment
	"This subclass represents a parent node which stores
	its children in an indexed collection."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdCompositeNode class methodsFor:'class initialization'!

initialize
        "Initialize the default collection size variable after
        loading the class."

        DefaultListSize := 8
!

initializeAfterLoad
        "Initialize the default collection size variable after
        loading the class."

        self initialize
! !

!IdCompositeNode class methodsFor:'constructors'!

forCompositeTag: aTagName
	"Return a new instance formatted with aTagName."

	| newNode |
	newNode := IdCompositeNode new.
	newNode initTagName: aTagName.
	^newNode
! !

!IdCompositeNode methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. If deep is true,
	clone the entire subtree under the receiver. If deep 
	is false, clone only the receiver and its attributes
	while sharing the receiver's child nodes. The node
	returned will not have a parent node.
	Note that all attribute nodes are cloned, including 
	those generated to represent default attributes."

	| answer allChildren childNode |
	answer := IdCompositeNode forCompositeTag: self getTagName.
	self getAttributesOrNil ~~ nil ifTrue: [
		answer cloneAttributesList: self getAttributesOrNil ].
	allChildren := self getLiveNodeCollection.
	1 to: allChildren size do: [ :i |
		childNode := allChildren ideAtIndex: i.
		answer appendChild: ( childNode clone: deep ) ].
	^answer
! !

!IdCompositeNode methodsFor:'editing'!

appendNodeImpl: aNode
	"Adds aNode to the end of the list of children of this node. 
	If aNode is already in the tree, it is first removed. Return aNode.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if aNode is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if aNode was created
	from a different document than the one that created the receiver."

	aNode isTypeDOCUMENT_FRAGMENT ifTrue: [
		^self appendDocFragment: aNode ].
	self confirmChildNodeType: aNode.
	self confirmChildNotAncestor: aNode.
	self confirmSameOwnerDocument: aNode.
	aNode getParent ~~ nil ifTrue: [
		aNode getParent removeChild: aNode ].

	self setLock.
	self getLiveNodeCollection add: aNode.
	aNode setParent: self.
	self resetLock.
	^aNode
!

insertBefore: newChild ref: refChild
	"Insert newChild before the existing refChild and return
	newChild if successful. If refChild is nil, append newChild.
	If newChild is a DocumentFragment object, all of its children 
	are inserted, in the same order, before refChild. If the newChild 
	is already in the tree, it is first removed.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if newChild is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver.
	Raise a NOT_FOUND_ERR exception if refChild is not nil or a 
	current child."

	| newNodes |

	newChild isTypeDOCUMENT_FRAGMENT ifTrue: [
		0 to: newChild getLength - 1 do: [ :i |
			self insertBefore: ( newChild itemImpl: i ) ref: refChild ].
		^newChild ].
	refChild == nil ifTrue: [
		^self appendNodeImpl: newChild ].

	self confirmChildNodeType: newChild.
	self confirmChildNotAncestor: newChild.
	self confirmSameOwnerDocument: newChild.
	newChild getParent ~~ nil ifTrue: [
		newChild getParent removeChild: newChild ].

	self setLock.
	newNodes := self internalInsert: newChild before: refChild.
	newNodes == nil ifTrue: [ 
		IdDOMImpl newExceptionNotFound signal ].
	childNodes := newNodes.
	self resetLock.
	^newChild
!

removeChildImpl: oldChild
	"Remove oldChild from the list of children and return it.
	Raise a NOT_FOUND_ERR exception if oldChild is not a child."

	| answer newNodes child |
	( self == oldChild getParent and: [ self hasChildNodes ] ) ifFalse: [ 
		IdDOMImpl newExceptionNotFound signal ].
	self setLock.
	oldChild setParent: nil.
	newNodes := OrderedCollection new: childNodes size.
	answer := nil.
	1 to: childNodes size do: [ :i |
		child := childNodes ideAtIndex: i.
		child == oldChild
			ifTrue: [ answer := child ]
			ifFalse: [ newNodes add: child ] ].
	childNodes := newNodes.
	self resetLock.
	answer ~~ nil ifTrue: [ ^answer ].
	IdDOMImpl newExceptionNotFound signal
!

replaceChild: newChild old: oldChild
	"Replace oldChild with newChild and return oldChild.
	If newChild is a DocumentFragment object, all of its children 
	are inserted, in the same order, before refChild. If the newChild 
	is already in the tree, it is first removed.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if newChild is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver.
	Raise a NOT_FOUND_ERR exception if refChild is not a child."

	| index |
	index := self indexOfChild: oldChild.
	index == -1 ifTrue: [ IdDOMImpl newExceptionNotFound signal ].
	self insertBefore: newChild ref: oldChild.
	self removeChildImpl: oldChild.
	^oldChild
! !

!IdCompositeNode methodsFor:'internal'!

addInternalByIndex: startIndex to: stopIndex into: aCollection
	"Indexes are not validated - use carefully."

	| child |
	startIndex > stopIndex ifTrue: [ ^self ].
	startIndex to: stopIndex do: [ :i |
		child := childNodes ideAtIndex: i.
		aCollection add: child ]
!

confirmChildNodeType: aNode
	"Raise a HIERARCHY_REQUEST_ERR exception if aNode
	is not a valid child node type."

	( aNode isTypeELEMENT 
		or: [ aNode isTypeTEXT
		or: [ aNode isTypeENTITY_REFERENCE
		or: [ aNode isTypePROCESSING_INSTRUCTION
		or: [ aNode isTypeCDATA_SECTION ]]]] ) ifFalse: [
			IdDOMImpl newExceptionHierarchyRequest signal ]
!

getLiveNodeCollection
	"Return a 'live' collection of all the receiver's 
	child nodes. This method is private to ensure 
	that the returned collections are not modified 
	directly by any calling method. Lazy initialize
	to the default list size if no collection exists yet."

	childNodes == nil ifTrue: [
		self initEmptyChildNodes: DefaultListSize ].
	^childNodes
!

internalInsert: newChild before: refChild
	"Insert newChild before the existing refChild. Return a
	new collection of child nodes if refChild is identified, else
	return nil. If refChild is nil return nil."

	| match newNodes child |
	self hasChildNodes ifFalse: [ ^nil ].
	newChild setParent: self.
	newNodes := OrderedCollection new: childNodes size + 1.
	match := false.
	1 to: childNodes size do: [ :i |
		child := childNodes ideAtIndex: i.
		child == refChild ifTrue: [ 
			match := true.
			newNodes add: newChild ].
		newNodes add: child ].
	^match
		ifTrue: [ newNodes ]
		ifFalse: [ nil ]
! !

!IdCompositeNode methodsFor:'node list api'!

getLength
	"Return the number of nodes stored within the receiver.
	This method overrides the superclass version to return
	zero if the collection has not been initialized yet."

	childNodes == nil ifTrue: [ ^0 ].
	^childNodes size
! !

!IdCompositeNode methodsFor:'queries'!

hasChildNodes
	"Return true if the receiver contains any child nodes,
	else return false."

	^self getLength > 0
!

ideIsCompositeNode
	"Return true if the receiver is a composite node,
	else return false."

	^true
! !

!IdCompositeNode methodsFor:'services'!

initEmptyChildNodes: anInteger
	"Initialize the receiver's childNodes collection to
	a size equal to anInteger."

	childNodes := OrderedCollection new: anInteger
!

normalize
	"Puts all Text nodes in the sub-tree underneath the
	receiver into a 'normal' form where only markup 
	(e.g., tags, comments, PIs, CDATASections, etc.) and 
	entity references separates Text nodes.
	Code is redundant with doc fragment."

	| child newList lastText |
	self getLength > 0 ifFalse: [ ^self ].
	newList := OrderedCollection new: self getLength.
	lastText := nil.
	0 to: self getLength - 1 do: [ :i |
		child := self itemImpl: i.
		child normalize.
		child isTypeTEXT ifTrue: [ 
				lastText == nil ifTrue: [ 
						lastText := child.
						newList add: child ]
					ifFalse: [
						lastText appendData: child getNodeValue ] ]
			ifFalse: [ 
				lastText := nil.
				newList add: child ] ].
	childNodes := newList
! !

!IdCompositeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdCompositeNode.st,v 1.7 2003/04/23 08:44:16 cg Exp $'
! !

IdCompositeNode initialize!
