"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdDOMImpl
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdDOMImpl class methodsFor:'class comment'!

classComment
	"This class implements the DOMImplementation interface. It also
	provides a set of convenience methods which are expected to be
	part of the DOM interface in future revisions. In particular, methods
	used to create new documents, document fragments, text nodes
	and exceptions are defined.

	Comment from the DOM specification:

	The DOMImplementation interface provides a number of methods 
	for performing operations that are independent of any particular 
	instance of the document object model. 

	The DOM Level 1 does not specify a way of creating a document 
	instance, and hence document creation is an operation specific to 
	an implementation. Future Levels of the DOM specification are 
	expected to provide methods for creating documents directly."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdDOMImpl class methodsFor:'convenience'!

createNewDocumentFragment
	"Return a new document fragment."
"
	IdDOMImpl createNewDocumentFragment
"
	^IdDOMImpl createNewXmlDocument createDocFragment
!

createNewTextNode: aString
	^IdDOMImpl createNewXmlDocument createTextNodeImpl: aString
!

createNewXmlDocument
	"Return a new XML document."
"
	IdDOMImpl createNewXmlDocument
"
	^IdDOMImpl new createDocument: 'xml'
! !

!IdDOMImpl class methodsFor:'exception constructors'!

newException: exceptionCode msg: msg
	"Return a new DOM exception for exceptionCode and msg."

	^IdDOMExceptionImpl forExceptionCode: exceptionCode msg: msg
!

newExceptionHierarchyRequest
	"Return a new DOM exception with an exception code
	of HIERARCHY_REQUEST_ERR."

	^IdDOMImpl newException: IdDOMException HIERARCHY_REQUEST_ERR
		msg: 'HIERARCHY_REQUEST_ERR'
!

newExceptionIndexSize
	"Return a new DOM exception with an exception code
	of INDEX_SIZE_ERR."

	^IdDOMImpl newException: IdDOMException INDEX_SIZE_ERR
		msg: 'INDEX_SIZE_ERR'
!

newExceptionInuseAttribute
	"Return a new DOM exception with an exception code
	of INUSE_ATTRIBUTE_ERR."

	^IdDOMImpl newException: IdDOMException INUSE_ATTRIBUTE_ERR
		msg: 'INUSE_ATTRIBUTE_ERR'
!

newExceptionInvalidCharacter
	"Return a new DOM exception with an exception code
	of INVALID_CHARACTER_ERR."

	^IdDOMImpl newException: IdDOMException INVALID_CHARACTER_ERR
		msg: 'INVALID_CHARACTER_ERR'
!

newExceptionNoDataAllowed
	"Return a new DOM exception with an exception code
	of NO_DATA_ALLOWED_ERR."

	^IdDOMImpl newException: IdDOMException NO_DATA_ALLOWED_ERR
		msg: 'NO_DATA_ALLOWED_ERR'
!

newExceptionNoModificationAllowed
	"Return a new DOM exception with an exception code
	of NO_MODIFICATION_ALLOWED_ERR."

	^IdDOMImpl newException: IdDOMException NO_MODIFICATION_ALLOWED_ERR
		msg: 'NO_MODIFICATION_ALLOWED_ERR'
!

newExceptionNotFound
	"Return a new DOM exception with an exception code
	of NOT_FOUND_ERR."

	^IdDOMImpl newException: IdDOMException NOT_FOUND_ERR
		msg: 'NOT_FOUND_ERR'
!

newExceptionNotSupported
	"Return a new DOM exception with an exception code
	of NOT_SUPPORTED_ERR."

	^IdDOMImpl newException: IdDOMException NOT_SUPPORTED_ERR
		msg: 'NOT_SUPPORTED_ERR'
!

newExceptionStringSize
	"Return a new DOM exception with an exception code
	of DOMSTRING_SIZE_ERR."

	^IdDOMImpl newException: IdDOMException DOMSTRING_SIZE_ERR
		msg: 'DOMSTRING_SIZE_ERR'
!

newExceptionWrongDocument
	"Return a new DOM exception with an exception code
	of WRONG_DOCUMENT_ERR."

	^IdDOMImpl newException: IdDOMException WRONG_DOCUMENT_ERR
		msg: 'WRONG_DOCUMENT_ERR'
! !

!IdDOMImpl class methodsFor:'validating'!

validateXMLName: aString
	"Return true if aString is a valid name in the XML
	specification, else return false. This is used to validate
	PI targets, entity references and attribute names."

	| tagSize |
	tagSize := aString ideLength.
	tagSize < 1 ifTrue: [ ^false ].
	1 to: tagSize do: [ :i |
		( IdPlatformUtilities isXMLNameChar: ( aString ideCharAt: i ) ) ifFalse: [ ^false ] ].
	^true
!

validateXMLTagName: tagName
	"Return true if tagName is a valid XML tag name,
	else return false."

	| tagSize |
	tagSize := tagName ideLength.
	tagSize < 1 ifTrue: [ ^false ].
	( IdPlatformUtilities isXMLNameFirstChar: ( tagName ideCharAt: 1 ) ) ifFalse: [ ^false ].
	tagSize == 1 ifTrue: [ ^true ].
	2 to: tagSize do: [ :i |
		( IdPlatformUtilities isXMLNameChar: ( tagName ideCharAt: i ) ) ifFalse: [ ^false ] ].
	^true
! !

!IdDOMImpl methodsFor:'queries'!

hasFeature: feature version: version
	"Test if the DOM implementation implements a specific feature.
	The following is pasted from DOM[REC level-one-core Oct 1998]:

	Parameters 
		feature
			The package name of the feature to test. In Level 1, the legal 
			values are 'HTML' and 'XML' (case-insensitive).
		version
			This is the version number of the package name to test. In Level 1,
			this is the string '1.0'. If the version is not specified, supporting any
			version of the feature will cause the method to return true.

	Return true if the feature is implemented in the specified version, 
	false otherwise.

	This method returns true for 'xml' and false for everything else.
	No version checking is implemented  yet."

	| featureType |
	featureType := feature asLowercase.
	( featureType ideStringTest: 'xml' ) ifTrue: [ ^true ].
	^false
! !

!IdDOMImpl methodsFor:'services'!

createDocument: type
	"Creates a document of the specified type. The only
	allowable type is 'XML' (case-insensitive) and it will 
	result in the return of a new document instance. Any 
	other value for type will result in a nil return value. 
	HTML may be supported later.

	This method is not included in the current DOM, however
	it was in an older specification. It has been kept in this
	class under the assumption that something similar will
	reappear in some future version of DOMImplementation."

	| docType |
	docType := type asLowercase.
	( docType ideStringTest: 'xml' ) ifTrue: [
		^IdDocumentImpl forImpl: self ].
	^nil
! !

!IdDOMImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdDOMImpl.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
