"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdNodeImpl subclass:#IdDocumentFragmentImpl
	instanceVariableNames:'masterDocument childNodes'
	classVariableNames:'DefaultListSize'
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdDocumentFragmentImpl class methodsFor:'accessing'!

getDefaultListSize
	"Return the default size used to initialize new
	childNodes collections."

	^DefaultListSize
!

setDefaultListSize: anInteger
	"Set the default size used to initialize new
	childNodes collections to anInteger."

	DefaultListSize := anInteger
! !

!IdDocumentFragmentImpl class methodsFor:'class comment'!

classComment
	"This class implements the DocumentFragment interface. It also
	serves as the superclass of the DocumentImpl class which implements
	behavior common to both classes.

	Comment from the DOM specification:

	DocumentFragment is a 'lightweight' or 'minimal' Document object. It is 
	very common to want to be able to extract a portion of a document's tree or 
	to create a new fragment of a document. Imagine implementing a user 
	command like cut or rearranging a document by moving fragments around. 
	It is desirable to have an object which can hold such fragments and it is quite 
	natural to use a Node for this purpose. While it is true that a Document object 
	could fulfil this role, a Document object can potentially be a heavyweight object, 
	depending on the underlying implementation. What is really needed for this is 
	a very lightweight object. DocumentFragment is such an object.

	Furthermore, various operations -- such as inserting nodes as children of another 
	Node -- may take DocumentFragment objects as arguments; this results in all the 
	child nodes of the DocumentFragment being moved to the child list of this node.

	The children of a DocumentFragment node are zero or more nodes representing 
	the tops of any sub-trees defining the structure of the document. DocumentFragment 
	nodes do not need to be well-formed XML documents (although they do need to follow 
	the rules imposed upon well-formed XML parsed entities, which can have multiple 
	top nodes). For example, a DocumentFragment might have only one child and that 
	child node could be a Text node. Such a structure model represents neither an HTML 
	document nor a well-formed XML document. 

	When a DocumentFragment is inserted into a Document (or indeed any other Node 
	that may take children) the children of the DocumentFragment and not the DocumentFragment 
	itself are inserted into the Node. This makes the DocumentFragment very useful when 
	the user wishes to create nodes that are siblings; the DocumentFragment acts as the 
	parent of these nodes so that the user can use the standard methods from the Node 
	interface, such as insertBefore() and appendChild()."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdDocumentFragmentImpl class methodsFor:'class initialization'!

initialize
        "Initialize the default collection size variable after
        loading the class."

        DefaultListSize := 8
!

initializeAfterLoad
        "Initialize the default collection size variable after
        loading the class."

        self initialize
! !

!IdDocumentFragmentImpl class methodsFor:'constructors'!

forMaster: aDocument
	"Return a new document fragment which references
	aDocument as its master document."

	| answer |
	answer := IdDocumentFragmentImpl new.
	answer initMaster: aDocument.
	^answer
! !

!IdDocumentFragmentImpl methodsFor:'accessing'!

getCurrentDocument
	"Return the master document."

	^self getMasterDocument
!

getMasterDocument
	"Return the master document. For documents
	this will return a reference to themselves."

	^masterDocument
!

getNodeName
	"Return the node name used for all instances:
	document-fragment."

	^'document-fragment'
!

getNodeType
	"Return an integer representing the receiver's
	node type. All instances share the same node
	type: DOCUMENT_FRAGMENT_NODE."

	^IdDOMConstants DOCUMENT_FRAGMENT_NODE
!

setMasterDocument: aDocument
	"Set the master document to aDocument."

	masterDocument := aDocument
!

setParentNode: aParent
	"Set the receiver's parent node. Override method 
	defined in superclass to prevent this variable from 
	being set for documents and document fragments.
	The getParentNode() method will always return nil."
! !

!IdDocumentFragmentImpl methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. The argument deep 
	is only relevent to Element nodes and will cause the 
	entire subtree under the receiver to be copied if true. The 
	node returned by this method will not have a parent node.
	The fragment returned will share the same master document.
	Clone all child nodes according to the value of deep."

	| answer allChildren childNode |
	answer := IdDocumentFragmentImpl forMaster: self getMasterDocument.
	allChildren := self getLiveNodeCollection.
	1 to: allChildren size do: [ :i |
		childNode := allChildren ideAtIndex: i.
		answer appendNodeImpl: ( childNode clone: deep ) ].
	^answer
! !

!IdDocumentFragmentImpl methodsFor:'creation'!

initMaster: aDocument
	"Initialize the master document to aDocument."

	self setMasterDocument: aDocument
! !

!IdDocumentFragmentImpl methodsFor:'editing'!

appendNodeImpl: aNode
	"Adds aNode to the end of the list of children of this node. 
	If aNode is already in the tree, it is first removed. Return aNode.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if aNode is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if aNode was created
	from a different document than the one that created the receiver."

	aNode isTypeDOCUMENT_FRAGMENT ifTrue: [
		^self appendDocFragment: aNode ].
	self confirmChildNodeType: aNode.
	self confirmChildNotAncestor: aNode.
	self confirmSameOwnerDocument: aNode.
	aNode getParent ~~ nil ifTrue: [
		aNode getParent removeChild: aNode ].

	self getLiveNodeCollection add: aNode.
	aNode setParent: self.
	^aNode
!

removeChildImpl: oldChild
	"Remove oldChild from the list of children and return it.
	Raise a NOT_FOUND_ERR exception if oldChild is not a child."

	self == oldChild getParent ifFalse: [
		IdDOMImpl newExceptionNotFound signal ].
	oldChild setParent: nil.
	childNodes ~~ nil ifTrue: [
		childNodes remove: oldChild ].
	^oldChild
! !

!IdDocumentFragmentImpl methodsFor:'internal'!

confirmChildNodeType: aNode
	"Raise a HIERARCHY_REQUEST_ERR exception if aNode
	is not a valid child node type."

	( aNode isTypeDOCUMENT_FRAGMENT 
		or: [ aNode isTypeATTRIBUTE
		or: [ aNode isTypeDOCUMENT
		or: [ aNode isTypeDOCUMENT_TYPE
		or: [ aNode isTypeENTITY ]]]] ) ifTrue: [
			IdDOMImpl newExceptionHierarchyRequest signal ]
!

getLiveNodeCollection
	"Return a 'live' collection of all the receiver's 
	child nodes. This method is private to ensure 
	that the returned collections are not modified 
	directly by any calling method. Lazy initialize
	to the default list size if no collection exists yet."

	childNodes == nil ifTrue: [
		self initEmptyChildNodes: DefaultListSize ].
	^childNodes
! !

!IdDocumentFragmentImpl methodsFor:'services'!

initEmptyChildNodes: anInteger
	"Initialize the receiver's childNodes collection to
	a size equal to anInteger."

	childNodes := OrderedCollection new: anInteger
!

normalize
	"Puts all Text nodes in the sub-tree underneath the
	receiver into a 'normal' form where only markup 
	(e.g., tags, comments, PIs, CDATASections, etc.) and 
	entity references separates Text nodes.
	Code is redundant with composite."

	| child newList lastText |
	self getLength > 0 ifFalse: [ ^self ].
	newList := OrderedCollection new: self getLength.
	lastText := nil.
	0 to: self getLength - 1 do: [ :i |
		child := self itemImpl: i.
		child normalize.
		child isTypeTEXT ifTrue: [ 
				lastText == nil ifTrue: [ 
						lastText := child.
						newList add: child ]
					ifFalse: [
						lastText appendData: child getNodeValue ] ]
			ifFalse: [ 
				lastText := nil.
				newList add: child ] ].
	childNodes := newList
! !

!IdDocumentFragmentImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdDocumentFragmentImpl.st,v 1.7 2003/04/23 08:44:16 cg Exp $'
! !

IdDocumentFragmentImpl initialize!
