"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdDocumentFragmentImpl subclass:#IdDocumentImpl
	instanceVariableNames:'rootElement impl'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdDocumentImpl class methodsFor:'class comment'!

classComment
	"This class implements the Document interface.

	Comment from the DOM specification:

	The Document interface represents the entire HTML or XML document. 
	Conceptually, it is the root of the document tree, and provides the primary 
	access to the document's data.

	Since elements, text nodes, comments, processing instructions, etc. 
	cannot exist outside the context of a Document, the Document interface 
	also contains the factory methods needed to create these objects. The 
	Node objects created have a ownerDocument attribute which associates 
	them with the Document within whose context they were created."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdDocumentImpl class methodsFor:'constructors'!

forImpl: anObject
	"Return a new document which will maintain a
	reference to anObject as its implementation."

	| answer |
	answer := IdDocumentImpl new.
	answer initImpl: anObject.
	^answer
! !

!IdDocumentImpl methodsFor:'accessing'!

getCurrentDocument
	"Return the receiver."

	^self ideYourself
!

getDoctype
	"Return the receiver's document type if one 
	is stored, else return nil. Currently return nil."

	^nil
!

getDocumentElement
	"Return the document element (root) of this document
	as defined by DOM."

	^self getRootElement
!

getImpl
	^impl
!

getImplementation
	"Return the DOM implementation which was used to
	create the receiver."

	^self getImpl
!

getNodeName
	"Return the node name used for all instances:
	document."

	^'document'
!

getNodeType
	"Return an integer representing the receiver's
	node type. All instances share the same node
	type: DOCUMENT_NODE."

	^IdDOMConstants DOCUMENT_NODE
!

getOwnerDocument
	"Return the Document object associated with this node.
	This is also the Document object used to create new nodes.
	When the receiver is a Document this is nil."

	^nil
!

getRootElement
	"Return the root element of this document."

	^rootElement
!

setRootElement: anElement
	"Set the root element to anElement. Add it to the childNodes
	collection and store a direct reference in this class. If a root
	element already exists remove it from the child nodes."

	rootElement ~~ nil ifTrue: [
		self removeChildImpl: rootElement ].
	rootElement := anElement.
	self appendNodeImpl: anElement
! !

!IdDocumentImpl methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. The argument deep 
	is only relevent to Element nodes and will cause the 
	entire subtree under the receiver to be copied if true. The 
	node returned by this method will not have a parent node.
	The fragment returned will share the same implementation
	and document type. Clone all child nodes including the root
	according to the value of deep."

	| answer allChildren childNode newRoot |
	answer := IdDocumentImpl forImpl: self getImpl.
	allChildren := self getLiveNodeCollection.
	1 to: allChildren size do: [ :i |
		childNode := allChildren ideAtIndex: i.
		childNode == self getRootElement ifTrue: [ 
				newRoot := childNode clone: deep.
				answer setRootElement: newRoot ]
			ifFalse: [ answer appendNodeImpl: ( childNode clone: deep ) ] ].
	^answer
! !

!IdDocumentImpl methodsFor:'creation'!

initImpl: anObject

	impl := anObject
!

initialize
	"Initialize the receiver by storing a reference to 
	itself as the masterDoc. This ensures that the 
	getMasterDoc() method defined in the superclass
	will always return the receiver."

	super initialize.
	self setMasterDocument: self
! !

!IdDocumentImpl methodsFor:'factory'!

createComposite: tagName
	"Return a new composite node with a type equal to tagName.
	Raise the INVALID_CHARACTER_ERR exception if tagName
	contains an invalid character."

	( IdDOMImpl validateXMLTagName: tagName ) ifFalse: [
		IdDOMImpl newExceptionInvalidCharacter signal ].
	^IdCompositeNode forCompositeTag: tagName
!

createDocFragment
	"Return a new document fragment which references the
	receiver as its master document."

	^IdDocumentFragmentImpl forMaster: self
!

createEmpty: tagName
	"Return a new empty element node with a type equal 
	to tagName. Note that the result can not add child nodes.
	Raise the INVALID_CHARACTER_ERR exception if tagName
	contains an invalid character."

	( IdDOMImpl validateXMLTagName: tagName ) ifFalse: [
		IdDOMImpl newExceptionInvalidCharacter signal ].
	^IdElementNode forEmptyTag: tagName
!

createPIImpl: aTarget data: data
	"Return a new ProcessingInstruction node for aTarget
	and contents data.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aTarget.
	Raise the NOT_SUPPORTED_ERR exception if this
	is an HTML document."

	( IdDOMImpl validateXMLName: aTarget ) ifFalse: [
		IdDOMImpl newExceptionInvalidCharacter signal ].
	self isHTMLDocument ifTrue: [
		IdDOMImpl newExceptionNotSupported signal ].
	^IdPINode forTarget: aTarget data: data
!

createTextNodeImpl: aString
	"Return a new Text node containing aString typed
	using internal classes."

	^IdStringNode asTEXT: aString
! !

!IdDocumentImpl methodsFor:'factory - DOM'!

createAttribute: aName
	"Return a new Attribute for aName.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aName."

	^IdAttributeImpl forName: aName
!

createCDATASection: aString
	"Return a new CDATA node containing aString.
	Raise the NOT_SUPPORTED_ERR exception if this
	is an HTML document."

	self isHTMLDocument ifTrue: [
		IdDOMImpl newExceptionNotSupported signal ].
	^IdStringNode asCDATA: aString
!

createComment: aString
	"Return a new Comment node containing aString."

	^IdStringNode asCOMMENT: aString
!

createDocumentFragment
	"Return a new document fragment which references the
	receiver as its master document."

	^self createDocFragment
!

createElement: tagName
	"Return a new element node with a type equal to tagName.
	Raise the INVALID_CHARACTER_ERR exception if tagName
	contains an invalid character.

	This will always return a composite node so that
	it can add more child nodes if required. Some efficiency
	might be gained by using ElementNode for elements
	which you know will be empty if there are large numbers
	of them."

	^self createComposite: tagName
!

createEntityReference: aName
	"Return a new entity reference for aName.
	Not yet implemented.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aName.
	Raise the NOT_SUPPORTED_ERR exception if this
	is an HTML document."

	( IdDOMImpl validateXMLName: aName ) ifFalse: [
		IdDOMImpl newExceptionInvalidCharacter signal ].
	self isHTMLDocument ifTrue: [
		IdDOMImpl newExceptionNotSupported signal ].
	^nil
!

createProcessingInstruction: aTarget data: data
	"Return a new ProcessingInstruction node for aTarget
	and contents data.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aTarget.
	Raise the NOT_SUPPORTED_ERR exception if this
	is an HTML document."

	^self createPIImpl: aTarget data: data
!

createTextNode: aString
	"Return a new Text node containing aString."

	^self createTextNodeImpl: aString
! !

!IdDocumentImpl methodsFor:'internal'!

isHTMLDocument
	"Return true if the receiver is an HTML document,
	else return false. Always return false as the HTML
	DOM functionality is not supported."

	^false
! !

!IdDocumentImpl methodsFor:'services'!

getElementsByTagName: aTagName
	"Returns a node list of all descendant Elements with 
	a given tag name, in the order in which they would be
	encountered in a preorder traversal of the document
	tree. If aTagName is '*', this method will return all 
	elements in the document."

	^self getDocumentElement getElementsByTagName: aTagName
! !

!IdDocumentImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdDocumentImpl.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
