"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdElementBuilder
	instanceVariableNames:'root'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOMUtils'
!


!IdElementBuilder class methodsFor:'class comment'!

classComment
	"A simple utility class used to construct documents and
	elements using source code instead of XML.
	It should only be used for relatively simple tasks as it
	is likely to change as the XML specs continue to evolve.
	If you need to build a production system using similar
	code, we recommend copying the code from this class
	and accessing the DOM api directly.
	This class may be deprecated in future releases."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdElementBuilder class methodsFor:'constructors'!

forTagName: tagName
	"Return a new instance of the receiver initialized
	with a new root composite element using tagName."

	| answer |
	answer := IdElementBuilder new.
	answer initTagName: tagName.
	^answer
! !

!IdElementBuilder methodsFor:'accessing'!

getDocumentFactory
	^self getRoot getCurrentDocument
!

getRoot
	"Return the root element. If none exists lazy initialize
	one now. This enables you to start adding text right
	away without setting up a document."

	root == nil ifTrue: [
		self createNewRoot: self defaultRootType ].
	^root
!

setRoot: anElement
	root := anElement
! !

!IdElementBuilder methodsFor:'create nodes'!

createElement: tagName
	"Return a new element which uses tagName."

	^self getDocumentFactory createComposite: tagName
!

createText: aString
	"Return a new text node for aString."

	^self getDocumentFactory createTextNodeImpl: aString
! !

!IdElementBuilder methodsFor:'creation'!

initTagName: tagName

	self createNewRoot: tagName
! !

!IdElementBuilder methodsFor:'internal'!

defaultRootType
	"Return the type used for default root elements."

	^'xml'
! !

!IdElementBuilder methodsFor:'services'!

addAttributes: attributeInfo to: aNode
	"Add attributes to aNode as stored in attributeInfo. 
	This collection must have an even number of entries
	which are paired in order. ie. 1 is first attribute name, 
	2 is first value, 3 is second name, 4 is second value."

	| entry name |
	aNode isTypeELEMENT ifFalse: [ ^self ].
	name := nil.
	1 to: attributeInfo size do: [ :i |
		entry := attributeInfo ideAtIndex: i.
		name == nil ifTrue: [
				name := entry ]
			ifFalse: [
				aNode setAttribute: name value: entry.
				name := nil ] ]
!

addChild: aNode
	"Add aNode to the current root element.
	Return aNode."

	self getRoot appendChild: aNode.
	^aNode
!

addElement: tagName
	"Add an element for tagName. Return 
	the new element."

	| answer |
	answer := self createElement: tagName.
	self addChild: answer.
	^answer
!

addElementWithText: tagName str: aString
	"Add an element for tagName and add a single
	Text node containing aString to that. Return 
	the new element."

	| anElement |
	anElement := self createElement: tagName.
	anElement appendChild: (
		self createText: aString ).
	self addChild: anElement.
	^anElement
!

addElementWithText: tagName str: aString id: anID
	"Add an element for tagName with an ID attribute of 
	anID. Add a single Text node containing aString to the
	element. Return the element."

	| anElement |
	anElement := self addElementWithText: tagName str: aString.
	anElement setAttribute: 'ID' value: anID.
	^anElement
!

addLinkForEvent: eventName str: aString
	"Add an element with tagName 'link' which is given
	a simple link attribute. Store eventName as the href
	and 'event' as the content-role. Store a text node 
	containing aString as its content. Return the new 
	link element."

	^self addLinkWithRole: 'event'
		ref: eventName
		str: aString
!

addLinkWithRole: contentRole ref: href str: aString
	"Add an element with tagName 'link' which is given
	a simple link attribute. Store href and contentRole as
	link attributes. Store a text node containing aString 
	as its only content. Return the new link element."

	^self addLinkWithRole: contentRole 
		ref: href 
		tag: 'link' 
		str: aString
!

addLinkWithRole: contentRole ref: href tag: tagName str: aString
	"Add an element with tagName which is given
	a simple link attribute. Store href and contentRole as
	link attributes. Store a text node containing aString 
	as its only content. Return the new link element."

	| anElement |
	anElement := self addElementWithText: tagName str: aString.
	anElement 
		setAttribute: 'xml:link' value: 'simple';
		setAttribute: 'href' value: href;
		setAttribute: 'content-role' value: contentRole.
	^anElement
!

createNewRoot: tagName
	"Create and store a new root element using tagName. 
	The root will reside in a standard xml document."

	| rootDoc |
	rootDoc := IdDOMImpl createNewXmlDocument.
	rootDoc setRootElement: ( rootDoc createComposite: tagName ).
	self setRoot: rootDoc getRootElement
! !

!IdElementBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdElementBuilder.st,v 1.6 2003/04/23 08:44:16 cg Exp $'
! !
