"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdNodeImpl subclass:#IdElementNode
	instanceVariableNames:'attributes tagName'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdElementNode class methodsFor:'class comment'!

classComment
	"This class implements the DOM Element interface.
	This class can be used as a concrete class only if the
	instances will not require the ability to store children.
	This provides a slightly more efficient option for empty
	elements identified during a parse. If children are required 
	use the CompositeNode subclass.

	Comment from the DOM specification:

	By far the vast majority of objects (apart from text) that authors 
	encounter when traversing a document are Element nodes. Assume 
	the following XML document: 

		<elementExample id='demo'>
			<subelement1/>
			<subelement2>
				<subsubelement/>
			</subelement2>
		</elementExample>

	When represented using DOM, the top node is an Element node for 
	'elementExample', which contains two child Element nodes, one for 
	'subelement1' and one for 'subelement2'. 'subelement1' contains no 
	child nodes.

	Elements may have attributes associated with them; since the Element 
	interface inherits from Node, the generic Node interface method getAttributes 
	may be used to retrieve the set of all attributes for an element. There are 
	methods on the Element interface to retrieve either an Attr object by name 
	or an attribute value by name. In XML, where an attribute value may contain 
	entity references, an Attr object should be retrieved to examine the possibly 
	fairly complex sub-tree representing the attribute value. On the other hand, 
	in HTML, where all attributes have simple string values, methods to directly 
	access an attribute value can safely be used as a convenience."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdElementNode class methodsFor:'constructors'!

forEmptyTag: aTagName
	"Return a new instance formatted with aTagName
	as type. Note that the result will only be able to 
	represent empty elements because it inherits all 
	the add / remove child logic from NodeImpl (which 
	does nothing). If you need to add child nodes, 
	use CompositeNode instead."

	| newNode |
	newNode := IdElementNode new.
	newNode initTagName: aTagName.
	^newNode
! !

!IdElementNode methodsFor:'accessing'!

getAttributesList
	"Return a named node list containing all of the
	receiver's attributes. Lazy initialize it if needed.
	This method returns the internally defined named
	node list type rather than the DOM interface."

	attributes ~~ nil ifTrue: [ ^attributes ].
	attributes := IdNamedNodeList new.
	^attributes
!

getAttributesOrNil
	"Return a named node list containing all of the
	receiver's attributes, or nil if none is defined."

	^attributes
!

getNodeName
	"Return the receiver's tag name."

	^self getTagName
!

getNodeType
	"Return an integer representing the receiver's
	node type. All instances share the same node
	type: ELEMENT_NODE."

	^IdDOMConstants ELEMENT_NODE
!

getTagName
	"Return the string which is the receiver's type, 
	or name. Note that all operations on a tag name 
	are case sensitive."

	^tagName
! !

!IdElementNode methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. If deep is true,
	clone the entire subtree under the receiver. If deep 
	is false, clone only the receiver and its attributes
	while sharing the receiver's child nodes. The node
	returned will not have a parent node.
	Note that all attribute nodes are cloned, including 
	those generated to represent default attributes."

	| answer |
	answer := IdElementNode forEmptyTag: self getTagName.
	self getAttributesOrNil ~~ nil ifTrue: [
		answer cloneAttributesList: self getAttributesOrNil ].
	^answer
! !

!IdElementNode methodsFor:'creation'!

initTagName: aTagName
	"Initialize the receiver with aTagName as its type."

	tagName := aTagName
! !

!IdElementNode methodsFor:'queries'!

ideIsElementNode
	"Return true if the receiver is an element node,
	else return false."

	^true
! !

!IdElementNode methodsFor:'services'!

addChildrenByTagName: aTagName to: allNodes
	"Add the receiver and all descendant Elements which have
	tag names of aTagName to allNodes. If aTagName is '*', all 
	elements are added."

	( ( aTagName ideStringTest: '*' ) or: [ aTagName ideStringTest: self getTagName ] ) ifTrue: [
		allNodes add: self ].
	0 to: self getLength - 1 do: [ :i |
		( self itemImpl: i ) addChildrenByTagName: aTagName to: allNodes ]
!

cloneAttributesList: anAttributesList
	"Set the receiver's attribute list to a clone of
	anAttributesList. Note that this will replace all 
	current attributes."

	self replaceAllAttributes: ( anAttributesList cloneSelfAndNodes: true )
!

getAttribute: aName
	"Return the first attribute value (String) stored in 
	the receiver which is identified by aName, or nil 
	if none is found. Return an empty string if that 
	attribute does not have a specified or default value. "

	| attr |
	attr := self getAttributeNode: aName.
	attr == nil ifTrue: [ ^nil ].
	^attr getValue
!

getAttributeNode: aName
	"Return the first attribute node stored in the receiver 
	which is identified by aName. or nil if none is found."

	attributes == nil ifTrue: [ ^nil ].
	^attributes getNamedItem: aName
!

getElementsByTagName: aTagName
	"Returns a node list of all descendant Elements with 
	a given tag name, in the order in which they would be
	encountered in a preorder traversal of the document
	tree. If aTagName is '*', this method will return all 
	elements in the document.
	Note that this will not include the receiver in the resulting 
	list even if it has aTagName."

	| allNodes |
	aTagName == nil ifTrue: [ ^IdNodeListImpl forNodes: nil ].
	allNodes := OrderedCollection new.
	0 to: self getLength - 1 do: [ :i |
		( self itemImpl: i ) addChildrenByTagName: aTagName to: allNodes ].
	^IdNodeListImpl forNodes: allNodes
!

removeAttribute: aName
	"Removes the specified attribute by aName. If the
	attribute is not found do nothing."

	attributes == nil ifTrue: [ ^self ].
	attributes removeNamedItem: aName
!

removeAttributeNode: anAttribute
	"Removes anAttribute from the receiver.
	Return anAttribute.
	Raise the NOT_FOUND_ERR exception if the
	attribute is not found."

	| existing |
	existing := self getAttributeNode: anAttribute getName.
	( existing == nil or: [ existing ~~ anAttribute ] ) ifTrue: [
		IdDOMImpl newExceptionNotFound signal ].
	self removeAttribute: anAttribute getNodeName.
	^anAttribute
!

replaceAllAttributes: anAttributesList
	"Replace the receiver's attributes with anAttributesList."

	attributes := anAttributesList
!

setAttribute: aName value: aString
	"Adds a new attribute/value pair to the receiver. 
	If an attribute called aName is already present, 
	it's value is changed to be that of aString.
	Raise the INVALID_CHARACTER_ERR exception if an
	invalid character is specified in aName."

	| att |
	att := IdAttributeImpl forName: aName.
	att setValue: aString.
	self setAttributeNode: att
!

setAttributeNode: anAttribute
	"Adds anAttribute to the receiver. If an attribute by
	the same name is already present, replace it.
	If the newAttr attribute replaces an existing attribute with 
	the same name, the previously existing Attr node is returned, 
	otherwise nil is returned.
	Need to implement exceptions."

	| existing |
	existing := self getAttributeNode: anAttribute getName.
	self getAttributesList setNamedItem: anAttribute.
	^existing
!

toString
	"Return a string representation of the receiver."

	^self ideGetClassName, '<', self getTagName idePrintPrimitive, '>'
! !

!IdElementNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdElementNode.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
