"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdException
	instanceVariableNames:''
	classVariableNames:'HostExceptionLibrary'
	poolDictionaries:''
	category:'XML-Indelv-IdDOMSmalltalk'
!


!IdException class methodsFor:'class comment'!

classComment
	"Smalltalk implementation of a portable exception
	mechanism. Host exception events are stored in a class
	variable dictionary keyed by the class (subclass of this
	class) itself. A signal method is defined in this class.

	In IBM Smalltalk or VisualWorks you can register handler 
	blocks using the following syntax (where IdExceptionSubclass
	is the specific IdException subclass being handled). Note that
	the method ideExitGracefully must be sent to the exception
	to ensure the host exception handler block exits without
	raising another exception. This requirement is dropped under
	the platform specific syntax's shown below.
	See the #runPortableExample class method for a working example.

		[ code which can raise exception ] 
			ideOnException: IdException*
			do: [ :ex |
				optional exception handling code...
				ex ideExitGracefully ]

	Specifying multiple handler blocks is also legal using the
	#ideOnException:do:when:do: and
	#ideOnException:do:when:do:when:do: and
	#ideOnException:do:when:do:when:do:when:do: variants.
	See the #runPortableMultiExample class method for an example.

	In IBM Smalltalk you can also register handler blocks using 
	the following syntax:

		[ code which can raise exception ] 
			when: IdException* getException
			do: [ :ex |
				IBM specific exception handling code ]

	In VisualWorks you can also register handler blocks using 
	the following syntax:

		IdExceptionSubclass getSignal
			handle: [ :ex |
				VisualWorks specific exception handling code ]
			do: [ code which can raise exception ]"
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdException class methodsFor:'class initialization'!

initialize
        "Create a dictionary to hold host exception event signals and 
        store it in a class variable after loading the class. Store one
        Object errorSignal exception for IdException as a default for
        all subclasses. Do not allow this method to
        execute for any other subclass."

        | ex |
        self == IdException ifFalse: [ ^self ].
        HostExceptionLibrary := IdentityDictionary new.
        HostExceptionLibrary at: self put: ( ex := Object errorSignal newSignal ).
        ex notifierString: '(IdException) An error has occurred.'
!

initializeAfterLoad
        "Create a dictionary to hold host exception event signals and 
        store it in a class variable after loading the class. Store one
        Object errorSignal exception for IdException as a default for
        all subclasses. Do not allow this method to
        execute for any other subclass."

        self initialize

! !

!IdException class methodsFor:'examples'!

runPortableExample
	"Run an exception handling example which demonstrates
	the portable handling syntax for a single exception."
"
	IdException runPortableExample
"
	[ IdException signalNew ]
		ideOnException: IdException
		do: [ :ex |
			Transcript cr;show: 'IdException was caught using portable syntax'.
			ex ideExitGracefully ].
	Transcript cr;show: 'Processing can continue after handling an IdException'.
!

runPortableMultiExample
	"Run an exception handling example which demonstrates
	the portable handling syntax for two exceptions. Note that
	when using more than one exception, you should always
	list the more specific exception first (ie. list an exception
	subclass before its superclass) in Smalltalk or in Java.

	In this example only the first handler block will be executed.
	Specifying 3 or 4 handler blocks is also legal using the
	#ideOnException:do:when:do:when:do: and
	#ideOnException:do:when:do:when:do:when:do: variants."
"
	IdException runPortableMultiExample
"
	[ IdDOMImpl newExceptionNotFound signal ]
		ideOnException: IdDOMException
		do: [ :ex |
			Transcript cr;show: 'IdDOMException was caught in IdDOMException handler block'.
			Transcript cr;show: '  query for DOM code: ', ex ideAsExceptionInstance code printString.
			Transcript cr;show: '  query for message: ', ex ideAsExceptionInstance getMessage printString.
			ex ideExitGracefully ]
		when: IdException
		do: [ :ex |
			Transcript cr;show: 'IdException was caught in IdException handler block'.
			ex ideExitGracefully ].
	Transcript cr;show: 'Processing can continue after handling an IdException'.
!

runVisualWorksExample
	"Run an exception handling example which demonstrates
	the VisualWorks syntax for a single exception."
"
	IdException runVisualWorksExample
"
		IdException getSignal
			handle: [ :ex |
				Transcript cr;show: 'IdException was caught using VisualWorks syntax'.
				ex return ]
			do: [ IdException throwNew ].
	Transcript cr;show: 'Processing can continue after handling a IdException'.
! !

!IdException class methodsFor:'host exceptions'!

getException
	"Return the host exception object for this class."

	^self privateGetExceptionFor: self
!

getSignal
	"Convenience method for VisualWorks."

	^self privateGetExceptionFor: self
!

privateGetExceptionFor: aClass
	"Return the host exception for aClass in VisualWorks.
	Add a new entry to HostExceptionLibrary if needed."

	| ex |
	HostExceptionLibrary == nil ifTrue: [
		^self error: 'IdException class was not properly initialized after loading.' ].
	^HostExceptionLibrary at: aClass ifAbsent: [
		ex := aClass superclass getException newSignal.
		ex notifierString: '(', aClass name, ') An error has occurred.'.
		HostExceptionLibrary at: aClass put: ex ]
! !

!IdException class methodsFor:'throw'!

signalNew
	"Create a new exception instance and signal it using
	the host platform exception mechanism."

	self new signal
!

throwNew
	"Create a new exception instance and signal it (throw in Java)
	using the host platform exception mechanism."

	self new signal
! !

!IdException methodsFor:'signalling'!

privateSignalHost: anException
	"Signal anException (the underlying host exception
	for this class) using VisualWorks syntax."

	anException raise
!

signal
	"Signal the underlying host exception for this class."

	self privateSignalHost: self class getException
! !

!IdException class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdException.st,v 1.7 2003/04/23 08:44:16 cg Exp $'
! !

IdException initialize!
