"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdInputSource
	instanceVariableNames:'characterStream systemId publicId'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOMSmalltalk'
!


!IdInputSource class methodsFor:'class comment'!

classComment
	"This class is used as a single input source for an XML entity.
	It allows a SAX application to encapsulate information about an 
	input source in a single object, which may include a public 
	identifier, a system identifier, and/or a character stream. Note
	that a public identifier serves as an alternate URI/URl and will
	only be utilized if a SAX application explicitly uses it to
	resolve entities.

	There are two places that the application will deliver this input 
	source to the parser: as the argument to the Parser>>parse method, 
	or as the return value of the EntityResolver>>resolveEntity method.

	The SAX parser will use the InputSource object to determine how to 
	read XML input. If there is a character stream available, the parser 
	will read that stream directly; if not, the parser will attempt to open a 
	URI/URL connection to the resource identified by the system identifier.

	Note that this Smalltalk version of this SAX class does not support byte 
	streams and different encodings. It relies on the host platform to convert 
	various encodings to standard character streams.

	An InputSource object belongs to the application: the SAX parser shall 
	never modify it in any way (it may modify a copy if necessary)."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdInputSource class methodsFor:'constructors'!

forStream: aStream
	"Create a new input source with a character stream. 
	Application writers may use #setSystemId to provide a 
	base for resolving relative URIs, and #setPublicId to 
	include a public identifier."

	^self new
		setCharacterStream: aStream;
		yourself
!

forSystemId: aSystemId
	"Create a new input source with a system identifier.
	Applications may use #setPublicId to include a public
	identifier as well. If the system identifier is a URI/URL, 
	it must be fully resolved."

	^self new
		setSystemId: aSystemId;
		yourself
! !

!IdInputSource methodsFor:'accessing'!

getCharacterStream
	"Get the character stream for this input source.
	If none was provided return nil."

	^characterStream
!

getPublicId
	"Get the public identifier for this input source."

	^publicId
!

getSystemId
	"Get the system identifier for this input source.
	If the systemId is a URI/URL, it will be fully resolved."

	^systemId
!

setCharacterStream: aStream
	"Set the character stream for this input source
	to aStream. If there is a character stream specified, 
	the SAX parser will not attempt to open a URI/URL
	connection to the system identifier."

	characterStream := aStream
!

setPublicId: aPublicId
	"Set the public identifier for this input source. 
	The public identifier is always optional: if the application 
	writer includes one, it will be provided as part of the 
	location information. The application can then use it to
	resolve entities if so desired."

	publicId := aPublicId
!

setSystemId: aSystemId
	"Set the system identifier for this input source. 
	The system identifier is optional if there is a character 
	stream, but it is still useful to provide one, since the 
	application can use it to resolve relative URIs and can 
	include it in error messages and warnings (the parser 
	will attempt to open a connection to the URI only if there
	is no character stream specified).
	If the system ID is a URL, it must be fully resolved."

	systemId := aSystemId
! !

!IdInputSource class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdInputSource.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
