"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdNodeImpl
	instanceVariableNames:'parent lock'
	classVariableNames:'EmptyChildCollection'
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdNodeImpl class methodsFor:'instance creation'!

new
	"Answer an instance of the receiver and initialize it."

	^super new initialize
! !

!IdNodeImpl class methodsFor:'class comment'!

classComment
	"This class implements the DOM Node interface.
	This is an abstract class which implements the basic behavior
	of a Node object instance. It provides default implementations of 
	several instance methods common to all node types. Many
	methods are reimplemented from subclasses for performance
	reasons to avoid expensive type casting in Java. As a result,
	this is an unusually large class.

	Comment from the DOM specification:

	The Node interface is the primary datatype for the entire Document 
	Object Model. It represents a single node in the document tree. 
	While all objects implementing the Node interface expose methods 
	for dealing with children, not all objects implementing the Node interface 
	may have children. For example, Text nodes may not have children, 
	and adding children to such nodes results in a DOMException being raised. 

	The attributes nodeName, nodeValue and attributes are included as a 
	mechanism to get at node information without casting down to the specific 
	derived interface. In cases where there is no obvious mapping of these 
	attributes for a specific nodeType (e.g., nodeValue for an Element or 
	attributes for a Comment), this returns null. Note that the specialized 
	interfaces may contain additional and more convenient mechanisms to 
	get and set the relevant information."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdNodeImpl class methodsFor:'class initialization'!

initialize
        "Initialize the shared empty node collection after
        loading the class. Note that methods which interact
        directly with EmptyChildCollection are guaranteed
        to not modify it, so it will always be truly empty."

        EmptyChildCollection := OrderedCollection new: 0
!

initializeAfterLoad
        "Initialize the shared empty node collection after
        loading the class. Note that methods which interact
        directly with EmptyChildCollection are guaranteed
        to not modify it, so it will always be truly empty."

        self initialize

! !

!IdNodeImpl class methodsFor:'constants'!

nodeTypeUnknown
	"Return the integer -1 which represents an unknown 
	node type. See the class comment of StringNode for more 
	information on node types."

	^-1
! !

!IdNodeImpl methodsFor:'accessing'!

getAttributes
	"Return a named node map containing all of the
	receiver's attributes, or nil if attributes are not
	supported by the receiver's subclass."

	^self getAttributesList
!

getAttributesList
	"Return a named node list containing all of the
	receiver's attributes. Default is nil."

	^nil
!

getAttributesOrNil
	"Return a named node list containing all of the
	receiver's attributes, or nil if none is defined.
	This method avoids instantiating a default list.
	Default is return nil."

	^nil
!

getChildNodes
	"Return the receiver as a NodeList on its own child 
	nodes if any exist. Methods in this class size()
	and item() operate on the underlying 'live' child
	node collection in subclasses that support it."

	^self ideYourself
!

getFirstChild
	"Return the first child of this node. If there is no such node,
	this returns nil."

	^self item: 0
!

getLastChild
	"Return the last child of this node. If there is no such node,
	this returns nil."

	^self item: self getLength - 1
!

getNextSibling
	"Returns the node immediately following the receiver 
	in a breadth-first traversal, or nil if no such node can be
	located. Defer to a CompositeNode parent if one exists.

	If a non-CompositeNode parent exists return nil for now.
	Later this can be upgraded to use the core DOM api
	to support mixed DOM implementations."

	self getParent == nil ifTrue: [ ^nil ].
	^self getParent getNextSibling: self
!

getNodeName
	"Return the name of this node. The default value
	for all nodes is nil. Subclasses override where needed."

	^nil
!

getNodeType
	"Return an integer representing the receiver's
	node type. Node types are statically defined in
	this class and can be accessed via class methods.
	Default is return the 'unknown' node type (-1).
	This method is redefined by several of the 
	receiver's subclasses."

	^self class nodeTypeUnknown
!

getNodeValue
	"Return the value of this node. The default value
	for all nodes is nil."

	^nil
!

getOwnerDocument
	"Return the Document object associated with this node.
	This is also the Document object used to create new nodes.
	When the receiver is a Document this is nil."

	^self getCurrentDocument
!

getParent
	"Return the receiver's parent or nil if no 
	parent is found."

	^parent
!

getParentNode
	"Return the receiver's parent node typed using
	the DOM Node interface."

	^self getParent
!

getPreviousSibling
	"Returns the node immediately preceding the receiver 
	in a breadth-first traversal, or nil if no such node can be
	located. Defer to a CompositeNode parent if one exists.

	If a non-CompositeNode parent exists return nil for now.
	Later this can be upgraded to use the core DOM api
	to support mixed DOM implementations."

	self getParent == nil ifTrue: [ ^nil ].
	^self getParent getPreviousSibling: self
!

getTarget
	^nil
!

setNodeValue: aString
	"Return the value of this node to aString. This method
	is provided for compatibility with DOM and does nothing.
	It will raise a NO_MODIFICATION_ALLOWED_ERR
	exception if used."

	IdDOMImpl newExceptionNoModificationAllowed signal
!

setParent: aParent
	"Set the receiver's parent to aParent."

	parent := aParent
! !

!IdNodeImpl methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. The argument deep 
	is only relevent to Element nodes and will cause the
	entire subtree under the receiver to be copied if true. The 
	node returned by this method will not have a parent node.
	Default is return nil. This method should be reimplemented
	by all subclasses."

	^nil
!

cloneNode: deep
	"Return a duplicate of the receiver. Cast the
	result of clone(deep) to the w3 Node interface."

	^self clone: deep
! !

!IdNodeImpl methodsFor:'creation'!

initialize
	"Initialize the receiver after it has been created."

	self resetLock
! !

!IdNodeImpl methodsFor:'editing'!

appendChild: aNode
	"Add aNode to the end of the receiver's childNodes.
	Convert from DOM to internal types if possible. See 
	appendNodeImpl() method comment for complete details."

	| typed |
	aNode ideIsNodeImpl ifTrue: [
		typed := aNode.
		^self appendNodeImpl: typed ].
	^nil
!

appendDocFragment: aNode
	"Append a document fragment by adding all of its child
	nodes in order. Return aNode."

	^self appendDocFragment: aNode from: 0
!

appendDocFragment: aNode from: startZeroIndex
	"Append a document fragment by adding all of its child
	nodes in order from startZeroIndex. Return aNode."

	startZeroIndex to: aNode getLength - 1 do: [ :i |
		self appendNodeImpl: ( aNode itemImpl: i ) ].
	^aNode  "is this correct, or return last node in fragment"
!

appendNodeImpl: aNode
	"Adds the node newChild to the end of the list of children of 
	this node. If the newChild is already in the tree, it is first removed.
	Return aNode.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if newChild is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver.
	Raise a NO_MODIFICATION_ALLOWED_ERR exception if the receiver
	is read only.

	Default is raise a NO_MODIFICATION_ALLOWED_ERR exception."

	IdDOMImpl newExceptionNoModificationAllowed signal
!

insertBefore: newChild dom: refChild
	"Insert newChild before the existing refChild and return
	newChild if successful. Convert from DOM to internal types 
	if possible. See internally typed insertBefore() method 
	comment for complete details."

	| typed1 typed2 |
	newChild ideIsNodeImpl ifTrue: [
		typed1 := newChild.
		typed2 := refChild.
		^self insertBefore: typed1 ref: typed2 ].
	^nil
!

insertBefore: newChild ref: refChild
	"Insert newChild before the existing refChild and return
	newChild if successful. If refChild is nil, append newChild.
	If newChild is a DocumentFragment object, all of its children 
	are inserted, in the same order, before refChild. If the newChild 
	is already in the tree, it is first removed.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if newChild is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver.
	Raise a NO_MODIFICATION_ALLOWED_ERR exception if the receiver
	is read only.
	Raise a NOT_FOUND_ERR exception if refChild is not nil or a 
	current child.

	Default is raise a NO_MODIFICATION_ALLOWED_ERR exception."

	IdDOMImpl newExceptionNoModificationAllowed signal
!

removeChild: oldChild
	"Remove oldChild from the list of children and return it.
	Convert from DOM to internal types if possible. See 
	internally typed removeChild() method comment for 
	complete details."

	| typed |
	oldChild ideIsNodeImpl ifTrue: [
		typed := oldChild.
		^self removeChildImpl: typed ].
	^nil
!

removeChildImpl: oldChild
	"Remove oldChild from the list of children and return it.
	Raise a NO_MODIFICATION_ALLOWED_ERR exception if the receiver
	is read only.
	Raise a NOT_FOUND_ERR exception if oldChild is not a child.

	Default is raise a NO_MODIFICATION_ALLOWED_ERR exception."

	IdDOMImpl newExceptionNoModificationAllowed signal
!

replaceChild: newChild dom: oldChild
	"Replace oldChild with newChild and return oldChild.
	Convert from DOM to internal types if possible. See 
	internally typed replaceChild() method comment for 
	complete details."

	| typedNew typedOld |
	( newChild ideIsNodeImpl and: [ oldChild ideIsNodeImpl ] ) ifTrue: [
		typedNew := newChild.
		typedOld := oldChild.
		^self replaceChild: typedNew old: typedOld ].
	^nil
!

replaceChild: newChild old: oldChild
	"Replace oldChild with newChild and return oldChild.
	If newChild is a DocumentFragment object, all of its children 
	are inserted, in the same order, before refChild. If the newChild 
	is already in the tree, it is first removed.

	Raise a HIERARCHY_REQUEST_ERR exception if the receiver
	does not allow children of this type, or if newChild is an ancestor.
	Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver.
	Raise a NO_MODIFICATION_ALLOWED_ERR exception if the receiver
	is read only.
	Raise a NOT_FOUND_ERR exception if refChild is not a child.

	Default is raise a NO_MODIFICATION_ALLOWED_ERR exception."

	IdDOMImpl newExceptionNoModificationAllowed signal
! !

!IdNodeImpl methodsFor:'internal'!

confirmChildNotAncestor: aNode
	"Raise a HIERARCHY_REQUEST_ERR exception if aNode
	is an ancestor of the receiver."

	( self isWithinAncestor: aNode ) ifTrue: [
		IdDOMImpl newExceptionHierarchyRequest signal ]
!

confirmSameOwnerDocument: aNode
	"Raise a WRONG_DOCUMENT_ERR exception if newChild was created
	from a different document than the one that created the receiver. Do not
	raise an exception if aNode has no document - that will happen with new
	instances the first time they are added. This is not entirely consistent with
	DOM but it is safe (because aNode is an internal node type) and it allows
	us to avoid storing an otherwise useless pointer to the document in
	every node instance."

	aNode getOwnerDocument == nil ifTrue: [ ^self ].
	( self getOwnerDocument == aNode getOwnerDocument ) ifFalse: [
		IdDOMImpl newExceptionWrongDocument signal ]
!

getLiveNodeCollection
	"Return a 'live' collection of all the receiver's 
	child nodes. Default is return a shared empty 
	list. This method is private to ensure that the 
	returned collections are not modified directly by
	any calling method. Modification must be done
	with public Node methods. All implementations 
	of this method must never return nil."

	^EmptyChildCollection
! !

!IdNodeImpl methodsFor:'node list api'!

getLength
	"Return the number of nodes stored within the receiver.

	This method is required by the NodeList interface which
	the receiver implements. It is only relevent when used on
	subclasses that can store child nodes."

	^self getLiveNodeCollection size
!

item: anInteger
	"Return the node stored at the (zero-based) index anInteger.
	Valid indexes are in the range 0 through n-1 where n is the
	size of the node list. If anInteger is invalid return nil.

	This method is required by the NodeList interface which
	the receiver implements. It is only relevent when used on
	subclasses that can store child nodes."

	| nodes |
	anInteger < 0 ifTrue: [ ^nil ].
	nodes := self getLiveNodeCollection.
	anInteger >= nodes size ifTrue: [ ^nil ].
	^nodes ideAtIndex: anInteger + 1
!

itemImpl: anInteger
	"Return the node stored at the (zero-based) index anInteger.
	Valid indexes are in the range 0 through n-1 where n is the
	size of the node list. If anInteger is invalid return nil.

	This method works exactly the same as item() except that
	it returns the NodeImpl type instead of the org.w3c.dom.Node  
	which prevents unnecessary type casting in Java."

	| nodes |
	anInteger < 0 ifTrue: [ ^nil ].
	nodes := self getLiveNodeCollection.
	anInteger >= nodes size ifTrue: [ ^nil ].
	^nodes ideAtIndex: anInteger + 1
! !

!IdNodeImpl methodsFor:'queries'!

hasChildNodes
	"Return true if the receiver contains any child nodes,
	else return false. Default is false."

	^false
!

ideIsNodeImpl
	"Return true if the receiver is a NodeImpl instance,
	else return false."

	^true
!

indexInParent
	"Return the index of the receiver in its parent.
	No check is done for the root element, which returns
	its position in the document's node collection.
	Return -1 if a problem is encountered."

	self getParent == nil ifTrue: [ ^-1 ].
	^self getParent indexOfChild: self
!

isLocked
	"Return true if the receiver is locked, else return false."

	^lock
!

isRootElement
	"Return true if the receiver is a root element,
	else return false."

	self getParentNode == nil ifTrue: [ ^false ].
	self getParentNode getNodeType == IdDOMConstants DOCUMENT_NODE ifTrue: [ ^true ].
	self getParentNode getNodeType == IdDOMConstants DOCUMENT_FRAGMENT_NODE ifTrue: [ ^true ].
	^false
!

isWithinAncestor: aNode
	"Return true if the receiver is currently stored
	within aNode or one of its children, else return false."

	self getParent == nil ifTrue: [ ^false ].
	self getParent == aNode ifTrue: [ ^true ].
	^self getParent isWithinAncestor: aNode
! !

!IdNodeImpl methodsFor:'services'!

addChildrenByTagName: aTagName to: allNodes
	"Add the receiver and all descendant Elements which have
	tag names of aTagName to allNodes. If aTagName is '*', all 
	elements are added. Default for all non-Elements is do nothing."
!

getAttribute: aName
	"Return the first attribute value (String) stored in 
	the receiver which is identified by aName, or nil 
	if none is found. Default for all nodes is nil. "

	^nil
!

getNextSibling: aNode
	"Return the node immediately after aChildNode if
	one can be found, else return nil."

	| allNodes child |
	self hasChildNodes ifFalse: [ ^nil ].
	allNodes := self getLiveNodeCollection.
	1 to: allNodes size do: [ :i |
		child := allNodes ideAtIndex: i.
		child == aNode ifTrue: [
			i == allNodes size ifTrue: [ ^nil ].
			child := allNodes ideAtIndex: i + 1.
			^child ] ].
	^nil
!

getPreviousSibling: aNode
	"Return the node immediately before aChildNode if
	one can be found, else return nil."

	| allNodes child |
	self hasChildNodes ifFalse: [ ^nil ].
	allNodes := self getLiveNodeCollection.
	1 to: allNodes size do: [ :i |
		child := allNodes ideAtIndex: i.
		child == aNode ifTrue: [
			i == 1 ifTrue: [ ^nil ].
			child := allNodes ideAtIndex: i - 1.
			^child ] ].
	^nil
!

indexOfChild: aChild
	"Return the index of aChild or -1 if it is not found."

	self hasChildNodes ifFalse: [ ^-1 ].
	0 to: self getLength - 1 do: [ :i |
		( self itemImpl: i ) == aChild ifTrue: [ ^i + 1 ] ].
	^-1
!

normalize
	"Puts all Text nodes in the sub-tree underneath the
	receiver into a 'normal' form where only markup 
	(e.g., tags, comments, PIs, CDATASections, etc.) and 
	entity references separates Text nodes.
	Default is do nothing."
!

printOn: aStream
	"Print a string representation of the receiver to aStream.
	Call the receiver's version of #toString to implement. The
	method #toString is overridden in subclasses instead of 
	#printOn: to make it easier to translate to Java."

	aStream nextPutAll: self toString
!

resetLock
	"Reset the node lock boolean back to false. All edit
	requests which are otherwise valid will be allowed."

	lock := false
!

setAttribute: aName value: aString
	"Adds a new attribute/value pair to the receiver. 
	This default method does nothing and exists to 
	avoid some type casting in Java."
!

setLock
	"Set the node lock boolean to true. This will prevent
	all new edit requests until the lock is reset to false
	using the resetLock() method."

	lock := true
!

toString
	"Return a string representation of the receiver.
	Default just prints the class name."

	^self ideGetClassName
! !

!IdNodeImpl methodsFor:'services alt'!

firstChildAsString
	"Return a string containing the node value of 
	the first child node. If no children are stored
	return an empty string."

	self hasChildNodes ifFalse: [ ^String new ].
	^( self item: 0 ) getNodeValue
!

getChildAtNodeName: aName
	"Returns the first child node which has a node name
	equal to aName, or nil if none is found."

	| child |
	self getLength == 0 ifTrue: [ ^nil ].
	0 to: self getLength - 1 do: [ :i |
		child := self itemImpl: i.
		( child getNodeName ~~ nil
			and: [ aName ideStringTest: child getNodeName ] ) ifTrue: [
				^child ] ].
	^nil
!

getCurrentDocument
	"Return the document in which the receiver currently 
	resides. Relay to the parent."

	^self getParent == nil
		ifTrue: [ nil ]
		ifFalse: [ self getParent getCurrentDocument ]
! !

!IdNodeImpl methodsFor:'test node type'!

isTypeATTRIBUTE
	"Return true if the receiver's node type is ATTRIBUTE_NODE,
	else return false."

	^self getNodeType == IdDOMConstants ATTRIBUTE_NODE
!

isTypeCDATA_SECTION
	"Return true if the receiver's node type is CDATA_SECTION_NODE,
	else return false."

	^self getNodeType == IdDOMConstants CDATA_SECTION_NODE
!

isTypeCOMMENT
	"Return true if the receiver's node type is COMMENT_NODE,
	else return false."

	^self getNodeType == IdDOMConstants COMMENT_NODE
!

isTypeDOCUMENT
	"Return true if the receiver's node type is DOCUMENT_NODE,
	else return false."

	^self getNodeType == IdDOMConstants DOCUMENT_NODE
!

isTypeDOCUMENT_FRAGMENT
	"Return true if the receiver's node type is DOCUMENT_FRAGMENT_NODE,
	else return false."

	^self getNodeType == IdDOMConstants DOCUMENT_FRAGMENT_NODE
!

isTypeDOCUMENT_TYPE
	"Return true if the receiver's node type is DOCUMENT_TYPE_NODE,
	else return false."

	^self getNodeType == IdDOMConstants DOCUMENT_TYPE_NODE
!

isTypeELEMENT
	"Return true if the receiver's node type is ELEMENT_NODE,
	else return false."

	^self getNodeType == IdDOMConstants ELEMENT_NODE
!

isTypeENTITY
	"Return true if the receiver's node type is ENTITY_NODE,
	else return false."

	^self getNodeType == IdDOMConstants ENTITY_NODE
!

isTypeENTITY_REFERENCE
	"Return true if the receiver's node type is ENTITY_REFERENCE_NODE,
	else return false."

	^self getNodeType == IdDOMConstants ENTITY_REFERENCE_NODE
!

isTypeNOTATION
	"Return true if the receiver's node type is NOTATION_NODE,
	else return false."

	^self getNodeType == IdDOMConstants NOTATION_NODE
!

isTypePROCESSING_INSTRUCTION
	"Return true if the receiver's node type is PROCESSING_INSTRUCTION_NODE,
	else return false."

	^self getNodeType == IdDOMConstants PROCESSING_INSTRUCTION_NODE
!

isTypeTEXT
	"Return true if the receiver's node type is TEXT_NODE,
	else return false."

	^self getNodeType == IdDOMConstants TEXT_NODE
! !

!IdNodeImpl class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdNodeImpl.st,v 1.7 2003/04/23 08:44:16 cg Exp $'
! !

IdNodeImpl initialize!
