"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdPlatformUtilities
	instanceVariableNames:''
	classVariableNames:'SaxParserName'
	poolDictionaries:''
	category:'XML-Indelv-IdDOMSmalltalk'
!


!IdPlatformUtilities class methodsFor:'XML parsing'!

getNewInputSource
	^IdInputSource new
!

getNewSAXParser
	"Return a new SAX compliant XML parser instance.
	Return nil if the class name stored in a class variable
	is not loaded."

	| class |
	^( class := Smalltalk at: self getSaxParserName asSymbol ifAbsent: [nil] ) == nil
		ifTrue: [ nil ]
		ifFalse: [ class new ]
!

isXMLNameChar: aChar
	"Return true if aChar is a valid character for an XML name,
	else return false."

	^( self isXMLNameFirstChar: aChar )
		or: [ aChar == $. 
		or: [ aChar == $- ]]
!

isXMLNameFirstChar: aChar
	"Return true if aChar is a valid character to start an XML name,
	else return false."

	^aChar isAlphaNumeric or: [ aChar == $_ or: [ aChar == $: ]]
!

parseXMLDocument: aSystemId
	"Return the XML document located at aSystemId, or nil
	if it can not be located or parsed."

	^IdXMLReader parseDocumentAtURI: aSystemId
!

parseXMLInLocal: directoryPath fileName: aFileName
	"Attempt to parse and return the XML document named
	aFileName at directoryPath, or nil if it can not be found."

	| stream inputSource document |
	inputSource := self getNewInputSource.
	inputSource == nil ifTrue: [ ^nil ].
	stream := self openFileTextReader: directoryPath fileName: aFileName.
	stream == nil ifTrue: [ ^nil ].
	inputSource setCharacterStream: stream.
	self doBlock: [ document := IdXMLReader parseDocument: inputSource ]
		ensure: [ stream close ].
	^document
!

parseXMLLocally: pathName
	"Attempt to parse and return the XML document stored
	locally at pathName, or nil if it can not be found."

	| stream inputSource document asURI |
	inputSource := self getNewInputSource.
	inputSource == nil ifTrue: [ ^nil ].
	stream := self openFileTextReader: pathName.
	stream == nil ifTrue: [ ^nil ].
	asURI := self convertLocalPathToURI: pathName.
	inputSource 
		setSystemId: asURI;
		setCharacterStream: stream.
	self doBlock: [ document := IdXMLReader parseDocument: inputSource ]
		ensure: [ stream close ].
	^document
! !

!IdPlatformUtilities class methodsFor:'XSL'!

getXSLStyleSheetName: anElement
	"Assumes PI is first node. This will change.
	Should add validation that it is an xsl file.

<Customer>
<?xml-stylesheet href='LiteralText.xsl' type='text/xsl' ?>
	...data
</Customer>"

	| piString aName data |
	piString := anElement firstChildAsString.
	piString ideLength == 0 ifTrue: [ ^nil ].
	data := ReadStream on: piString.
	data upTo: $".
	aName := data upTo: $".
	( aName at: aName size - 3 ) = $. ifFalse: [ ^nil ].
	^aName
! !

!IdPlatformUtilities class methodsFor:'array copy'!

copyCharArray: charArray fromZero: start length: length
	"Return a copy of charArray from the zero-based index
	start (where 0 indicates the first entry as per Java arrays) 
	to the length specified."

	^charArray copyFrom: start + 1 to: start + length
! !

!IdPlatformUtilities class methodsFor:'class comment'!

classComment
	"A platform specific utility class which implements many
	useful functions such as reading and parsing files. Refer to
	the individual method comments for more information."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdPlatformUtilities class methodsFor:'class names'!

getSaxParserName
	"Return a Symbol representing the class name used
	to implement SAX compliant Parser objects. Lazy init 
	with system default if necessary."

	^SaxParserName == nil
		ifTrue: [ SaxParserName := #IdStxpParser ]
		ifFalse: [ SaxParserName ]
!

setSaxParserName: aSymbol
	"Set the class name used to implement SAX compliant 
	parser objects to aSymbol."

	SaxParserName := aSymbol
! !

!IdPlatformUtilities class methodsFor:'file access'!

combineIntoUri: directoryPath filename: fileName
	"Return a string containing the arguments provided
	combined into the form of a URI."

	| uri directoryURI |
	directoryURI := self convertLocalPathToURI: directoryPath.
	uri := IdURI forBase: ( IdURI forUriString: directoryURI ) string: fileName.
	^uri toExternalForm
!

convertLocalPathToURI: pathName
	"Return a URI string which represents pathName.
	Currently this does not include the protocol prefix."

	| buffer nextChar |
	buffer := IdStringBuffer new: pathName size.
	1 to: pathName size do: [ :i |
		nextChar := pathName ideCharAt: i.
		nextChar == $\
			ifTrue: [ buffer appendChar: $/ ]
			ifFalse: [ buffer appendChar: nextChar ] ].     
	^buffer convertToString
!

convertURIToLocalPath: systemId
	"May consolidate this with similar methods in 
	a new class."

	| start buffer nextChar |
	start := 1.
	( systemId size > 7 and: [ ( systemId copyFrom: 1 to: 6 ) = 'file:/' ] ) ifTrue: [
		start := 7 ].
	buffer := IdStringBuffer new: systemId size.
	start to: systemId size do: [ :i |
		nextChar := systemId ideCharAt: i.
		nextChar == $/
			ifTrue: [ buffer appendChar: $\ ]
			ifFalse: [ buffer appendChar: nextChar ] ].     
	^buffer convertToString
!

directoryExistsAt: pathName
	"Return true if a directory at local pathName exists, 
	else return false."

	pathName isNil ifTrue: [ ^false ].
	^pathName asFilename definitelyExists
!

fileExistsAt: pathName
	"Return true if the file stored locally at pathName exists, 
	else return false."

	| answer |
	pathName isNil ifTrue: [ ^false ].
	answer := false.
	Object errorSignal
		handle: [:ex | ex return]
		do: [answer := pathName asFilename definitelyExists].
	^answer
!

fileRemoveAt: pathName
	"Remove the file stored locally at pathName if it exists, 
	otherwise do nothing."

	| fileName |
	( fileName := pathName asFilename ) definitelyExists ifTrue: [
		fileName delete ]
!

openFileTextReader: pathName
	"Return a ReadStream on the file stored locally at 
	pathName. If no file exists return nil. Note that the
	ReadStream is open on a file and must be closed
	when no longer required. That the is calling
	method's responsibility."

	( self fileExistsAt: pathName ) ifFalse: [ ^nil ].
	^pathName asFilename readStream
!

openFileTextReader: pathName fileName: aFileName
	"Return a ReadStream on the file stored locally at 
	pathName (the directory) and aFileName. This method
	will concatenate the two together into a single path."

	| buffer |
	buffer := IdStringBuffer new: pathName size + aFileName size + 1.
	buffer appendString: pathName.
	pathName last == $\ ifFalse: [
		buffer appendChar: $\ ].
	buffer appendString: aFileName.
	^self openFileTextReader: buffer convertToString
!

openFileTextWriter: pathName
	"Return a WriteStream on the file stored locally at 
	pathName. Completely replace an existing file of the
	same name if one exists. If no file exists, create a new 
	one. If the file can not be created or written to return nil."

	| fileName |
	fileName := pathName asFilename.
	fileName canBeWritten ifFalse: [ ^nil ].
	^fileName writeStream
! !

!IdPlatformUtilities class methodsFor:'system'!

basicNew: anObject
	"Return a new instance of anObject's class without
	populating any of its instance variables."

	^anObject species new
!

doBlock: aBlock ensure: completionBlock
	aBlock ideEnsureCompletion: completionBlock
! !

!IdPlatformUtilities class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdPlatformUtilities.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
