"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdNodeImpl subclass:#IdStringNode
	instanceVariableNames:'data nodeType'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOM'
!


!IdStringNode class methodsFor:'class comment'!

classComment
	"This class provides support for nodes which contain simple
	string content. It defines an instance variable 'data' which stores
	this String value and provides methods to access and edit the data. 
	It implements the DOM Text interface and several others related
	to different node types (see below).

	This class supports multiple node types (ie. Coment, Text, CDATA, etc.)
	using the nodeType variable. Valid types are defined by the DOM Node 
	interface (or IdDOMConstants in Smalltalk). NodeImpl also provides a 
	static method nodeTypeUnknown() which returns a valid node type (-1) 
	for any node which is otherwise unrecognized."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdStringNode class methodsFor:'constructors'!

forNodeType: anInteger string: aString
	"Return a new instance formatted to represent
	aString with a node type of anInteger."

	| newNode |
	newNode := IdStringNode new.
	newNode initNodeType: anInteger string: aString.
	^newNode
! !

!IdStringNode class methodsFor:'convenience'!

asCDATA: aString
	"Return a new instance formatted to represent a
	CDATA node which contains aString."

	^IdStringNode forNodeType: IdDOMConstants CDATA_SECTION_NODE
		string: aString
!

asCOMMENT: aString
	"Return a new instance formatted to represent a
	COMMENT node which contains aString."

	^IdStringNode forNodeType: IdDOMConstants COMMENT_NODE
		string: aString
!

asTEXT: aString
	"Return a new instance formatted to represent a
	Text node which contains aString."

	^IdStringNode forNodeType: IdDOMConstants TEXT_NODE
		string: aString
! !

!IdStringNode methodsFor:'accessing'!

getData
	"Return the data string stored by the receiver."

	^data
!

getLength
	"Return the length of the receiver's data string."

	self getData == nil ifTrue: [ ^0 ].
	^self getData ideLength
!

getNodeName
	"Return the node name used for the receiver. It will
	be either comment, text or cdata-section."

	self isTypeTEXT ifTrue: [ ^'text' ].
	self isTypeCOMMENT ifTrue: [ ^'comment' ].
	self isTypeCDATA_SECTION ifTrue: [ ^'cdata-section' ].
	^nil
!

getNodeType
	"Return an integer representing the receiver's
	node type. Node types are statically defined in the
	Node class and can be accessed via class methods.
	Instances of this class can have node types equal
	to TEXT, COMMENT or CDATA."

	^nodeType
!

getNodeValue
	"Returns the value of the receiver as a string. Return
	the data string for all instances."

	^self getData
!

setData: aString
	"Set the data string stored by the receiver to aString.
	Raise a NO_MODIFICATION_ALLOWED_ERR DOMException 
	if the receiver is read only (not yet implemented)."

	data := aString
! !

!IdStringNode methodsFor:'cloning'!

clone: deep
	"Return a duplicate of the receiver. The argument deep 
	is only relevent to Element nodes and will cause the 
	entire subtree under the receiver to be copied if true. The 
	node returned by this method will not have a parent node.
	The result will share the same data string as the receiver.
	Since strings are considered immutable, this is safe for
	both deep and shallow copies."

	^IdStringNode forNodeType: self getNodeType string: self getData
! !

!IdStringNode methodsFor:'creation'!

initNodeType: aNodeType string: aString
	"Initialize the receiver to represent aString as 
	a aNodeType (as defined by class variables in the 
	Node class)."

	nodeType := aNodeType.
	self setData: aString
! !

!IdStringNode methodsFor:'editing'!

appendData: aString
	"Append aString to the end of the receiver's data string.
	Raise a NO_MODIFICATION_ALLOWED_ERR DOMException 
	if the receiver is read only (not yet implemented)."

	| buffer mySize |
	self setLock.
	mySize := self getLength.
	buffer := IdStringBuffer new: mySize + aString ideLength.
	buffer appendString: self getData.
	buffer appendString: aString.
	self setData: buffer convertToString.
	self resetLock.
!

deleteData: offset count: count
	"Delete a count number of characters immediately after 
	the character specified by offset. An offset of zero will delete 
	at the beginning of the data string.
	Raise an INDEX_SIZE_ERR exception if the specified offset
	is negative or greater than the number of characters in data."

	| buffer mySize end |
	count == 0 ifTrue: [ ^self ].
	mySize := self getLength.
	( offset < 0 or: [ offset > mySize or: [ count < 0 ]] ) ifTrue: [
		IdDOMImpl newExceptionIndexSize signal ].
	offset == mySize ifTrue: [ ^self ].
	self setLock.
	buffer := IdStringBuffer new: mySize - count.
	offset > 0 ifTrue: [
		buffer appendString: ( self getData copyFrom: 1 to: offset ) ].
	end := offset + count.
	end < mySize ifTrue: [
		buffer appendString: ( self getData copyFrom: end + 1 to: mySize ) ].
	self setData: buffer convertToString.
	self resetLock
!

insertData: offset string: aString
	"Insert aString immediately after the character specified 
	by offset. An offset of zero will insert at the beginning of the
	data string.
	Raise an INDEX_SIZE_ERR exception if the specified offset
	is negative or greater than the number of characters in data."

	| buffer mySize |
	mySize := self getLength.
	( offset < 0 or: [ offset > mySize ] ) ifTrue: [
		IdDOMImpl newExceptionIndexSize signal ].
	offset == mySize ifTrue: [
		self appendData: aString.
		^self ].
	self setLock.
	buffer := IdStringBuffer new: mySize + aString ideLength.
	offset > 0 ifTrue: [
		buffer appendString: ( self getData copyFrom: 1 to: offset ) ].
	buffer appendString: aString.
	buffer appendString: ( self getData copyFrom: offset + 1 to: mySize ).
	self setData: buffer convertToString.
	self resetLock.
!

replaceData: offset count: count string: aString
	"Replace the characters starting at the specified 
	character offset with the specified string.
	Raise an INDEX_SIZE_ERR exception if the specified offset
	is negative or greater than the number of characters in data, or if 
	the specified count is negative."

	self deleteData: offset count: count.
	self insertData: offset string: aString
!

substringData: offset count: count 
	"Return a copy of the characters from offset (a zero-based
	index) for a length of count. An offset of zero will copy from
	the beginning of the data string. 
	Raise an INDEX_SIZE_ERR exception if the specified offset
	is negative or greater than the number of characters in data, or if 
	the specified count is negative."

	| mySize |
	mySize := self getLength.
	( offset < 0 or: [ offset > mySize or: [ count < 0 ]] ) ifTrue: [
		IdDOMImpl newExceptionIndexSize signal ].
	offset == mySize ifTrue: [ ^String new ].
	^self getData copyFrom: offset + 1 to: offset + count
! !

!IdStringNode methodsFor:'services'!

splitText: offset
	"Breaks this Text node into two Text nodes at the specified 
	offset, keeping both in the tree as siblings. This node then only 
	contains all the content up to the offset point. And a new Text 
	node, which is inserted as the next sibling of this node, contains 
	all the content at and after the offset point.
	Raise an INDEX_SIZE_ERR exception if the specified offset
	is negative or greater than the number of characters in data.
	Not yet implemented."

	| mySize |
	mySize := self getLength.
	( offset < 0 or: [ offset > mySize ] ) ifTrue: [
		IdDOMImpl newExceptionIndexSize signal ].
	^nil
!

toString
	"Return the data string stored in the receiver."

	^self getNodeValue
! !

!IdStringNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdStringNode.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
