"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdStxpAttributeList
	instanceVariableNames:'attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdStxp'
!


!IdStxpAttributeList class methodsFor:'instance creation'!

new
	^super new
		initialize;
		yourself
! !

!IdStxpAttributeList class methodsFor:'class comment'!

classComment
	"Instances of this class contain a list of attributes for a parsed 
	element, maintaining the order in which they were encoded. The 
	parser passes an instance to the SAX application as the second
	argument of each startElement event.

	The instance provided will return valid results only during the 
	scope of the startElement invocation (to save it for future use, the 
	application must make a copy).

	An AttributeList includes only attributes that have been specified 
	or defaulted: #IMPLIED attributes will not be included.

	There are two ways for the SAX application to obtain information 
	from an AttributeList. First, it can iterate through the entire list
	accessing each attribute by index position using the #getName:
	and #getValue: methods. This is comparable to the Java SAX
	interface. As an alternative, the application can request the value 
	of attributes by name using #getValueAtName: (this differs
	slightly from the Java implementation because Smalltalk can
	not have two methods which share the same name.

	Also note that 'CDATA' is returned as the type for all attributes
	since the STXP parser does not report attribute types."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdStxpAttributeList methodsFor:'accessing'!

getAttributes
        "Return the attributes in this list."

        ^attributes
!

getLength
	"Return the number of attributes in this list."

	^attributes size
!

getName: zeroBasedIndex
	"Return the name of the attribute stored at zeroBasedIndex.
	Return nil if it is not found."

	| index |
	index := zeroBasedIndex + 1.
	index > attributes size ifTrue: [ ^nil ].
	^( attributes at: index ) key
!

getType: zeroBasedIndex
	"Return the type of the attribute stored at zeroBasedIndex
	in this list. Return 'CDATA' for all attributes. See #getType: 
	for more information."

	^'CDATA'
!

getTypeAtName: aName
	"Return the type of the first attribute named aName 
	in this list. The STXP parser does not report attribute types,
	therefore return the value 'CDATA' for all attributes as 
	required by the XML standard."

	^'CDATA'
!

getValue: zeroBasedIndex
	"Return the value of the attribute stored at zeroBasedIndex.
	Return nil if it is not found."

	| index |
	index := zeroBasedIndex + 1.
	index > attributes size ifTrue: [ ^nil ].
	^( attributes at: index ) value
!

getValueAtName: aName
	"Return the value of the first attribute named aName 
	in this list. Return nil if none is found."

	attributes do: [ :each |
		each key = aName ifTrue: [ ^each value ] ].
	^nil
!

setAttributeAt: aName to: aValue
	"Change the attribute at aName to aValue if one is
	stored, otherwise create a new one and append it to 
	the attribute list."

	attributes do: [ :each |
		each key = aName ifTrue: [ ^each value: aValue ] ].
	attributes add: (
		Association new
			key: aName;
			value: aValue;
			yourself )
! !

!IdStxpAttributeList methodsFor:'creation'!

initialize
	attributes := OrderedCollection new
! !

!IdStxpAttributeList methodsFor:'services'!

printOn: aStream

	| assoc |
	aStream 
		nextPutAll: self class name;
		nextPut: $[.
	1 to: attributes size do: [ :i |
		assoc := attributes at: i.
		i > 1 ifTrue: [
			aStream nextPutAll: ', ' ].
		aStream 
			nextPutAll: assoc key;
			nextPut: $=;
			nextPutAll: assoc value ].
	aStream nextPutAll: ']'
! !

!IdStxpAttributeList class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdStxpAttributeList.st,v 1.6 2003/04/23 08:44:16 cg Exp $'
! !
