"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

IdStxpParser subclass:#IdStxpDoctypeParser
	instanceVariableNames:'isDoneDoctype'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdStxp'
!


!IdStxpDoctypeParser class methodsFor:'class comment'!

classComment
	"An extension of the parser class which handles some
	Doctype specific variations."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdStxpDoctypeParser methodsFor:'internal'!

internalParseDoctype
	"Return the lastChar when done or nil if an error
	is encountered."

	self nextChar.
	[ self nextDoctypeCDATA.
	self atEnd ]
		whileFalse: [ 
			self nextDoctypeAction.
			isDoneDoctype ifTrue: [ ^lastChar ] ].
	^nil
!

nextDoctypeAction

	lastChar == $] ifTrue: [
		self nextChar.
		^isDoneDoctype := true ].
	^self nextAction
!

nextDoctypeCDATA
	"Identify the ] char which should indicate the end of
	the DOCTYPE. We may need to do some more work if
	that char can legally exist in CDATA within a DOCTYPE."

	( lastChar isNil or: [ lastChar == $] ] ) ifTrue: [ ^self ].
	^self nextCDATA
!

syncCoreParserVariables: docParser

	isDoneDoctype := false.
	super syncCoreParserVariables: docParser
! !

!IdStxpDoctypeParser methodsFor:'parsing action'!

mdo
	"Process a markup declaration open. Handle all
	DOCTYPE specific declarations here."

	lastChar == $E ifTrue: [
		self nextChar.
		lastChar == $L ifTrue: [ ^self parseElementDeclaration ].
		lastChar == $N ifTrue: [ ^self parseEntityDeclaration ].
		self fatalError: 'markup declaration must be ENTITY or ELEMENT' ].
	lastChar == $A ifTrue: [ ^self parseAttlistDeclaration ].
	^super mdo
!

parseAttlistDeclaration
	"Parse the current attribute declaration. Only worry 
	about default attributes because this is not a validating
	parser. Default attributes are stored in the DTD."

	| elementType attName attType defaultDec defaultValue |
	self skipUpToSeparator.
	( elementType := self nextXMLToken ) == nil ifTrue: [
		^self fatalError: 'ATTLIST declaration has no element type' ].
	[ lastChar notNil and: [ ( lastChar == $> ) not ] ]
		whileTrue: [ 
			( attName := self nextXMLToken ) == nil ifTrue: [
				^self fatalError: 'ATTLIST declaration has no attribute name' ].
			self skipSeparators.
			lastChar == $( ifTrue: [
					attType := 'ENUMERATED'.
					self skipToCharacter: $).
					lastChar == $) 
						ifFalse: [ ^self fatalError: 'ATTLIST declaration has invalid enumerated types' ]
						ifTrue: [ self nextChar ] ]
				ifFalse: [ attType := self nextXMLToken ].
			attType == nil ifTrue: [
				^self fatalError: 'ATTLIST declaration has no attribute type' ].
			( defaultDec := self nextLiteralOrToken ) == nil ifTrue: [
				^self fatalError: 'ATTLIST declaration has no default declaration' ].
			( defaultDec = '#REQUIRED' or: [ defaultDec = '#IMPLIED' ] ) ifFalse: [
				defaultValue := defaultDec = '#FIXED' 
					ifTrue: [ self nextLiteralOrToken ]
					ifFalse: [ defaultDec ].
				defaultValue == nil ifTrue: [
					^self fatalError: 'ATTLIST declaration has no required default value' ].
				self getDtd addDefaultAttribute: attName
					value: defaultValue
					for: elementType ].
			"self nextChar" ]
!

parseElementDeclaration
	"Skip the entire element declaration since this is not a 
	validating parser."

	[ lastChar notNil and: [ ( lastChar == $> ) not ] ]
		whileTrue: [ self nextChar ]
!

parseEntityDeclaration

	| name value |
	self skipUpToSeparator.
	name := self nextXMLToken.
	name == nil ifTrue: [
		^self fatalError: 'ENTITY declaration contains no entity name' ].
	value := self nextAttributeValueToken.
	value == nil ifTrue: [
		^self fatalError: 'ENTITY declaration contains no entity value' ].
	self getDtd addEntityName: name value: value.
	self skipSeparators
! !

!IdStxpDoctypeParser methodsFor:'services'!

parseExternalDTD: anInputSource for: docParser
	"Should give entityResolver a chance to change input."

	self syncCoreParserVariables: docParser.
	stream := anInputSource getCharacterStream.
	stream ~~ nil ifTrue: [
		^self internalParseDoctype ].
	stream := self retrieveInputStream: anInputSource.
	stream == nil ifTrue: [ ^self ].
	self executeBlock: [ self internalParseDoctype ] 
		ensure: [ stream close ]
!

parseInternalDTD: docParser stream: aStream
	"The receiver is continuing a DOCTYPE parse for docParser.
	Synchronize the receiver to use the same aStream, etc. It should 
	be positioned on top of the [ char which marks a DOCTYPE and 
	will end just after the ] char if successful. Return the current lastChar
	when done or nil if an error is encountered."

	self syncCoreParserVariables: docParser.
	stream := aStream.
	^self internalParseDoctype
! !

!IdStxpDoctypeParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdStxpDoctypeParser.st,v 1.5 2003/04/23 08:44:16 cg Exp $'
! !
