"
The contents of this file are subject to the MOZILLA PUBLIC LICENSE
Version 1.0 (the 'MPL'); you may not use this file except in
compliance with the MPL.  A copy of the MPL has been provided
with this file. If that copy can not be found, you can obtain another 
on the Internet at http://www.mozilla.org/NPL/.

Software distributed under the MPL is distributed on an 'AS IS' basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
for the specific language governing rights and limitations under the
MPL.

The Original Code is

    InDelv Open Source
    STXP - Smalltalk XML Parser
    Beta 1.01
    VisualWorks
    December 21, 1998

The Initial Developer of this code under the MPL is InDelv, Inc.
Portions created by InDelv are
Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.

Contributor(s):
    Exept Software AG
"   


"{ Package: 'stx:goodies/xml/indelv' }"

IdStxpHandlerBase subclass:#IdStxpParseAppExample
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdStxp'
!

!IdStxpParseAppExample class methodsFor:'documentation'!

copyright
"
The contents of this file are subject to the MOZILLA PUBLIC LICENSE
Version 1.0 (the 'MPL'); you may not use this file except in
compliance with the MPL.  A copy of the MPL has been provided
with this file. If that copy can not be found, you can obtain another 
on the Internet at http://www.mozilla.org/NPL/.

Software distributed under the MPL is distributed on an 'AS IS' basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
for the specific language governing rights and limitations under the
MPL.

The Original Code is

    InDelv Open Source
    STXP - Smalltalk XML Parser
    Beta 1.01
    VisualWorks
    December 21, 1998

The Initial Developer of this code under the MPL is InDelv, Inc.
Portions created by InDelv are
Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.

Contributor(s):
    Exept Software AG
"   

! !

!IdStxpParseAppExample class methodsFor:'class comment'!

classComment
	"This class contains several examples of how to use
	the parser. These are stored in class methods."
! !

!IdStxpParseAppExample class methodsFor:'examples'!

runAttributesExample
"
	IdStxpParseAppExample runAttributesExample
"
	"Test element attributes specified with single and double 
	quotation marks."

	self runExampleOnString: 
'<Customers>
	<Customer custID=', $" asSymbol, '001', $" asSymbol, ' lastMod=', $' asSymbol, '1998.05.22', $' asSymbol, ' >
		<Name>
			<First>Bradley</First>
			<Last>Bismark</Last>
		</Name>
	</Customer>
</Customers>'
!

runCharRefsCommentPIExample
"
	IdStxpParseAppExample runCharRefsCommentPIExample
"
	"Test common xml character references, comments and
	processing instructions."

	self runExampleOnString: 
'<?xml version="1.0"?>
<Customers>
	<Customer>
		<?examplePI contents="test"?>
		<!!-- example comment -->
		<Name>
			<First>Bradley &lt;Brad&gt;</First>
			<Last>Bismark</Last>
		</Name>
	</Customer>
</Customers>'
!

runCustomerExample
"
	IdStxpParseAppExample runCustomerExample
"
	"Test a simple well formed document which contains
	several occurences of the same element types."

	self runExampleOnString: 
'<Customers>
	<Customer>
		<Name>
			<First>Bradley</First>
			<Last>Bismark</Last>
		</Name>
		<Order>
			<Number>16273</Number>
			<Date><Day>11</Day><Month>6</Month><Year>1997</Year></Date>
			<Item>
				<Title>Number, the Language of Science</Title>
				<Author>Danzig</Author>
				<Price>5.95</Price>
			</Item>
			<Item>
				<Title>Tales of Grandpa Cat</Title>
				<Author>Wardlaw, Lee</Author>
				<Price>6.58</Price>
			</Item>
		</Order>
	</Customer>
	<Customer>
		<Name>
			<Last>Higginbottom</Last>
			<First>Amy</First>
		</Name>
		<Credit-Limit>5000</Credit-Limit>
		<Order>
			<Number>16182</Number>
			<Date><Month>6</Month><Day>15</Day><Year>1997</Year></Date>
			<Item>
				<Title>Evolution of Complexity in Animal Culture</Title>
				<Publisher>Associated Press</Publisher>
				<Author>Bonner</Author>
				<Price>5.95</Price>
			</Item>
			<Item>
				<Title>When We Were Very Young</Title>
				<Author>Milne, A. A.</Author>
				<Price>12.50</Price>
			</Item>
		</Order>
		<Order>
			<Number>19182</Number>
			<Date><Year>1997</Year><Month>7</Month><Day>17</Day></Date>
			<Item>
				<Title>Learn Java Now</Title>
				<Author>Stephen R. Davis</Author>
				<Publisher>Microsoft Corporation</Publisher>
				<Price>9.95</Price>
			</Item>
		</Order>
	</Customer>
</Customers>'
!

runExampleOn: anInputSource
	"Main method for running examples. anInputSource is
	used to represent an XML resource which is either a stream
	or a systemId (path). Input sources are modelled by the
	class IdInputSource. See that class' comment for more."

	| parser |
	parser := IdStxpParser new.
	parser setDocumentHandler: self new.
	^ parser parse: anInputSource

    "Modified: / 21.4.1999 / 09:10:17 / stefan"
!

runExampleOnString: aString
	"Create an input source containing a character stream 
	over aString and run the example."

	^ self runExampleOn: (
		IdInputSource forStream: ( ReadStream on: aString ) )

    "Modified: / 21.4.1999 / 09:11:23 / stefan"
!

runExampleOnSystemId: aSystemId
	"Run an example by reading the file from aSystemId (path).
	The path must be provided using a URI/URL syntax such as
	'file:/c:/example.xml'. Note that only the 'file' protocol is supported
	in the Smalltalk implementation, not 'http', 'ftp', etc."
"
	IdStxpParseAppExample runExampleOnSystemId: 'file:/c:/example.xml'
"
	^self runExampleOn: (
		IdInputSource forSystemId: aSystemId )
! !

!IdStxpParseAppExample methodsFor:'doc handler api'!

characters: aString
	Transcript cr;show: 'char data: ', aString
!

endDocument
	Transcript cr;show: 'end document'
!

endElement: name
	Transcript cr;show: 'end element: ', name asString
!

processingInstruction: target data: data
	Transcript cr;show: 'PI target: ', target;cr;show: '     data: ', data
!

startDocument
	Transcript cr;show: 'start document'
!

startElement: name atts: atts
	Transcript cr;show: 'start element: ', name asString, '   ', atts printString
! !

!IdStxpParseAppExample class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdStxpParseAppExample.st,v 1.10 2003/04/23 08:44:16 cg Exp $'
! !
