"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"
"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdStxpParser
	instanceVariableNames:'lastChar stream baseURI errHandler docHandler lineNum linePos
		entityResolver dtd isWorking uriClass cdata currentTag tagStack
		streamStack'
	classVariableNames:'CR LF SPACE TAB'
	poolDictionaries:''
	category:'XML-Indelv-IdStxp'
!


!IdStxpParser class methodsFor:'instance creation'!

new
	^super new
		initialize;
		yourself
! !

!IdStxpParser class methodsFor:'class comment'!

classComment
	"The main class used to perform a parse. It accepts a single
	input source parameter in the #parse: method which will parse
	the document (if possible) and inform handlers as needed.
	Nothing is returned. To gather information during a parse
	you must provide a document handler. See IdStxpParseAppExample
	for usage examples. Errors are optionally sent to an error handler
	registered using the #setErrorHandler: method."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdStxpParser class methodsFor:'class initialization'!

initialize
        "Initialize class variables after loading the class."

        CR := Character value: 13.
        LF := Character value: 10.
        SPACE := Character value: 32.
        TAB := Character value: 9
!

initializeAfterLoad
        "Initialize class variables after loading the class."

        self initialize

! !

!IdStxpParser methodsFor:'accessing'!

getBaseURI
	^baseURI
!

getDocumentHandler
	docHandler == nil ifTrue: [
		docHandler := IdStxpHandlerBase new ].
	^docHandler
!

getDtd
	"Return the current DTD and lazy initialize it with
	an empty one if necessary."

	^dtd == nil
		ifTrue: [ dtd := IdStxpDTD new ]
		ifFalse: [ dtd ]
!

getEntityResolver
	entityResolver == nil ifTrue: [
		entityResolver := IdStxpHandlerBase new ].
	^entityResolver
!

getURIClass
	"Return the URI class used for this parse. The default
	is IdURI. An alternate class can be registered using
	#setURIClass: if an application needs a different
	implementation. Note that IdURI only supports the
	'file' protocol - 'http' or 'ftp' is not supported."

	^uriClass == nil
		ifTrue: [ uriClass := IdURI ]
		ifFalse: [ uriClass ]
!

setDocumentHandler: aHandler
	docHandler := aHandler
!

setDtd: aDTD
	dtd := aDTD
!

setEntityResolver: anEntityResolver
	entityResolver := anEntityResolver
!

setErrorHandler: aHandler
	errHandler := aHandler
!

setURIClass: aClass
	"Set the URI class to aClass. See the #getURIClass 
	method comment for more."

	uriClass := aClass
! !

!IdStxpParser methodsFor:'cdata management'!

nextPutCdataAtEnd: elementEnd

	| conts |
	conts := cdata contents.
	conts isEmpty ifFalse: [ self resetCdata ].
	elementEnd ifTrue: [ conts := self trimLastRe: conts ].
	conts isEmpty ifFalse: [
		self getDocumentHandler characters: conts ]
!

nextPutCro
	cdata nextPutAll: '&#'
!

nextPutEro
	cdata nextPut: $&
!

nextPutEtago
	cdata nextPutAll: '</'
!

nextPutMdo
	cdata nextPutAll: '<!!'
!

nextPutRe
	"Encode a return character in cdata. Use a Lf character
	as required by the XML spec."

	cdata nextPut: LF
!

nextPutRefc
	cdata nextPut: $;
!

nextPutStago
	cdata nextPut: $<
!

nextPutTagc
	cdata nextPut: $>
!

resetCdata
	cdata := WriteStream on: (String new: 256).
! !

!IdStxpParser methodsFor:'creation'!

initialize

	streamStack := OrderedCollection new.
	tagStack := OrderedCollection new.
	isWorking := false
! !

!IdStxpParser methodsFor:'error handling'!

fatalError: aString
	"Raise a Fatal Error containing aString."

	self raiseFatalError: aString
!

raiseFatalError: aString
	"Raise an error containing aString using the platform's
	default mechanism. A fatal error can be handled using
	the host platform's exception handling.
	This raises an exception even if an error handler is
	provided which is not precisely what SAX specifies.
	For now record -1 as the line and column numbers.
	These are not easy to calculate precisely without adding
	too much overhead. If anyone has time to solve this problem,
	please send it in to the support team."

	| parseException |
	parseException := IdStxpParseException forException: aString 
		pub: baseURI
		sys: nil
		line: -1 "lineNum"
		col: -1 "linePos".
	errHandler ~~ nil ifTrue: [
		errHandler fatalError: parseException ].
	parseException signal
! !

!IdStxpParser methodsFor:'internal'!

charIsCrOrLf: aChar
	"Return true if aChar is a Cr of Lf."

	^aChar == CR or: [ aChar == LF ]
!

confirmInputSource: anInputSource
	"Return the input source to be used for anInputSource.
	An opportunity is given to the entityResolver to provide 
	an alternate input source. This enables an application to 
	redirect input requests to a database or another URI."

	| newInputSource |
	newInputSource := self getEntityResolver
		resolveEntity: anInputSource publicId 
		system: anInputSource systemId.
	^newInputSource == nil
		ifTrue: [ anInputSource ]
		ifFalse: [ newInputSource ]
!

eventClass
	^IdStxpParserEvent
!

executeBlock: aBlock ensure: completionBlock
        "Execute aBlock and ensure that completionBlock 
        executes on the current platform. Return the result of
        executing aBlock."

        ( Smalltalk at: #CfsFileDescriptor ifAbsent: [nil] ) ~~ nil ifTrue: [ "IBM"
                ^aBlock atEndOrWhenExceptionDo: completionBlock ].
        ( Smalltalk at: #BlockClosure ifAbsent: [nil] ) ~~ nil ifTrue: [  "VW, STX"
                ^aBlock ensure: completionBlock ].
        ( Smalltalk at: #ViewManager ifAbsent: [nil] ) ~~ nil ifTrue: [  "VS"
                ^aBlock ensure: completionBlock ].
        ^aBlock value
!

isPITargetXml: target
	"Return true if target equals 'xml' using any combination
	of upper and lower case characters."

	| aChar |
	target size == 3 ifFalse: [ ^false ].
	aChar := target at: 1.
	( aChar == $x or: [ aChar == $X ] ) ifFalse: [ ^false ].
	aChar := target at: 2.
	( aChar == $m or: [ aChar == $M ] ) ifFalse: [ ^false ].
	aChar := target at: 3.
	( aChar == $l or: [ aChar == $L ] ) ifFalse: [ ^false ].
	^true
!

lastCharIsNormalizedLineFeed
	"Return true if lastChar is a Cr of Lf. Normalize CR-LF
	combinations to a single line feed occurance by advancing
	the scan as needed. The method #nextPutRe takes care
	of actually writing the Lf character."

	lastChar == LF ifTrue: [ ^true ].
	lastChar == CR ifFalse: [ ^false ].
	stream peek == LF ifTrue: [ self nextChar ].
	^true
!

lastCharIsQuoteOrParen
	^lastChar == $" or: [ lastChar == $' ]
!

lastCharIsSeparator
	"Return true if lastChar is a Space, Cr, Lf or Tab."

	( self charIsCrOrLf: lastChar ) ifTrue: [ ^true ].
	^lastChar == SPACE or: [ lastChar == TAB ]
!

normalizeAttValue: aString
	"Return a copy of aString which replaces entity
	references as needed."

	| readStream attChar writeStream eRef eValue |
	readStream := ReadStream on: aString.
	writeStream := WriteStream on: ( String new: aString size ).
	[ readStream atEnd ] whileFalse: [
		attChar := readStream next.
		attChar == $& ifTrue: [
				eRef := readStream upTo: $;.
				eValue := self getDtd getEntityValueAt: eRef.
				eValue ~~ nil ifTrue: [ writeStream nextPutAll: eValue ] ]
			ifFalse: [ writeStream nextPut: attChar ] ].
	^writeStream contents
!

popElementWithEvent: anEvent
	self nextPutCdataAtEnd: true.
	self sendEndElementEvent: anEvent.
	tagStack isEmpty
		ifTrue: [ currentTag := nil ]
		ifFalse: [ currentTag := tagStack removeFirst ].
	^currentTag
!

popStream
	| array |
	streamStack isEmpty ifTrue: [^self fatalError: 'too many end tags'].
	array := streamStack removeFirst.
	stream := array at: 1.
	lastChar := array at: 2
!

privateParseStream
	"This method assumes that the stream has already
	been properly stored."

	stream == nil ifTrue: [ ^self ].
	isWorking := true.
	lineNum := linePos := 0.
"       Transcript cr;cr;cr;show: baseURI toExternalForm printString.
	stream contents inspect.
"
	self startParse.
	self endParse.
	isWorking := false
!

processEmptyTag: aTagName event: anEvent
	"Process an XML empty tag (ie. <element/>) if required 
	and return true, else return false."

	self peekEtagSlash ifFalse: [ ^false ].
	self processEmptyTagEvent: anEvent.
	self nextChar.
	self peekTagc ifTrue: [ 
		self nextChar.
		^true ].
	self fatalError: 'empty XML start tag must end with /> characters'.
	^true
!

pushElement: aTagName event: anEvent
	"Detect an HTML root tag and handle the parse using
	a different class if one is found."

	self sendStartElementEvent: anEvent.
	currentTag == nil ifTrue: [ 
			aTagName asUppercase asSymbol == #HTML ifTrue: [
				^self completeAsHTMLParse: aTagName ] ]
		ifFalse: [ tagStack addFirst: currentTag ].
	currentTag := aTagName
!

pushStream: aStream
	stream isNil ifTrue: [^self fatalError: 'parse not correctly started'].
	streamStack addFirst: (Array with: stream with: lastChar).
	stream := aStream.
	self nextChar.
!

recognizeExternalEntity

	| sysId pubId |
	lastChar == $S ifTrue: [
		self skipUpToSeparator.
		self skipSeparators.
		self lastCharIsQuoteOrParen ifFalse: [
			^self fatalError: 'requires SYSTEM identifier' ].
		( sysId := self nextStringInQuotesOrParen ) == nil ifTrue: [
			^self fatalError: 'invalid SYSTEM identifier' ].
		^IdInputSource forSystemId: sysId ].
	lastChar == $P ifFalse: [
		^self fatalError: 'external entity requires SYSTEM or PUBLIC identifier' ].
	self skipUpToSeparator.
	self skipSeparators.
	self lastCharIsQuoteOrParen ifFalse: [
		^self fatalError: 'requires PUBLIC identifier' ].
	( pubId := self nextStringInQuotesOrParen ) == nil ifTrue: [
		^self fatalError: 'invalid PUBLIC identifier' ].
	self skipSeparators.
	self lastCharIsQuoteOrParen ifFalse: [
		^self fatalError: 'can not identify SYSTEM portion of PUBLIC identifier' ].
	( sysId := self nextStringInQuotesOrParen ) == nil ifTrue: [
		^self fatalError: 'invalid SYSTEM portion of PUBLIC identifier' ].
	^( IdInputSource forSystemId: sysId )
		setPublicId: pubId;
		yourself
!

retrieveInputStream: anInputSource
	"Attempt to retrieve the XML resource represented by
	anInputSource and return a read-only file stream which
	contains it. If the systemId can not be resolved try again
	using the publicId. Return nil if a stream can not be opened."

	| aStream inputURI |
	inputURI := self getURIClass forBase: baseURI string: anInputSource getSystemId.
	( aStream := inputURI openStream ) ~~ nil ifTrue: [ ^aStream ].
	inputURI := self getURIClass forBase: baseURI string: anInputSource getPublicId.
	^inputURI openStream
!

sendEndElementEvent: event
	self getDocumentHandler endElement: event getTagName
!

sendStartElementEvent: event
	self getDocumentHandler startElement: event getTagName
		atts: event getAttributeList
!

stringToInteger: aString
	^aString asNumber
!

syncCoreParserVariables: docParser

	self resetCdata.
	isWorking := true.
	baseURI := docParser getBaseURI.
	uriClass := docParser getURIClass.
	docHandler := docParser getDocumentHandler.
	entityResolver := docParser getEntityResolver.
	dtd := docParser getDtd
! !

!IdStxpParser methodsFor:'parsing'!

completeAsHTMLParse: aTagName
	"The start event for the HTML element has already been
	triggered. Continue to the end of the marked up text and
	then set the stream to its end."

	IdStxpHTMLParser new parseHTMLMarkup: self 
		tag: aTagName
		stream: stream.
	stream setToEnd
!

endParse
	[currentTag notNil]
		whileTrue:
			[self popElementWithEvent: (self eventClass endTag: currentTag)].
	self getDocumentHandler endDocument
!

endTagCloseAction: aTagName
	(currentTag = aTagName or: [tagStack includes: aTagName])
		ifFalse: [^self].
	[currentTag notNil and: [currentTag ~= aTagName]]
		whileTrue:
			[self popElementWithEvent: (self eventClass endTag: currentTag)].
	currentTag = aTagName ifFalse: [^self fatalError: 'end tag does not match: ', aTagName ].
	self popElementWithEvent: (
		self eventClass endTag: aTagName )
!

parseCDATASection
        "The receiver is positioned over the [ in a CDATA section.
        Advance past the last > character."

        'CDATA[' do: [ :char |
                self nextChar.
                lastChar == char ifFalse: [
                        ^self fatalError: 'invalid start to CDATA section' ] ].
        self nextChar.
        [ lastChar == nil ] whileFalse: [
            self writeCDATASegmentToBuffer.
            ( lastChar ~~ nil and: [ stream peek == $] ] ) ifTrue: [
                self nextChar.
                ( lastChar ~~ nil and: [ stream peek == $> ] ) ifTrue: [
                    self nextChar.
                    ^ self nextChar                  "successful CDATA parse"
                ] ifFalse: [
"/ cg: fix for the case, that CDATA ends with ']'
"/ i.e. ']]]>'
                    cdata nextPutAll: ']'.
"/                    cdata nextPutAll: ']]'.
"/                    self nextChar 
                ] 
            ] ifFalse: [ 
                cdata nextPut: $].
                self nextChar 
            ] 
        ].
        self fatalError: 'CDATA section must end with ]]> characters'
!

parseDOCTYPE
	"The receiver is positioned over the D in a DOCTYPE
	declaration such as <!!DOCTYPE. The next token must be
	a tag name which corresponds to the root element (which
	is not validated by this parser). Process an internal or
	external DTD as required."

	| dtdInputSource |
	self skipUpToSeparator.
	self skipSeparators.
	( self isNameFirstChar: lastChar ) ifFalse: [
		^self fatalError: 'invalid DOCTYPE Root Element Type' ].
	self skipUpToSeparator.
	self skipSeparators.
	lastChar == $[ ifTrue: [  "Internal DTD"
		lastChar := IdStxpDoctypeParser new parseInternalDTD: self stream: stream.
		lastChar == nil ifTrue: [
			^self fatalError: 'invalid DOCTYPE - can not parse internal DTD' ].
		self skipSeparators.
		lastChar == $> ifTrue: [ ^self nextChar ].
		^self fatalError: 'invalid DOCTYPE for internal DTD' ].
	( lastChar == $S or: [ lastChar == $P ] ) ifFalse: [
		^self fatalError: 'invalid DOCTYPE - can not locate DTD system identifier' ].
	dtdInputSource := self recognizeExternalEntity.
	dtdInputSource == nil ifTrue: [
		^self fatalError: 'invalid DOCTYPE - can not parse DTD external entity' ].
	IdStxpDoctypeParser new parseExternalDTD: dtdInputSource for: self.
	self skipSeparators.
	lastChar == $> ifTrue: [ ^self nextChar ].
	self fatalError: 'invalid DOCTYPE for external DTD'
!

parseElementAttributes: anEvent
        "Add copies of all default attributes and then replace
        them with any that are parsed.
        Return true if tag is and end tag also, false otherwise."

        | key value |
        self getDtd processDefaultAtts: anEvent.
        [ ( key := self nextXMLToken ) notNil ]
                whileTrue: [
                        self nextAttributeAssignToken isNil
                                ifTrue: [ value := String new ]
                                ifFalse: [ (value := self nextAttributeValueToken ) isNil
                                        ifTrue: [
                                                self fatalError: 'illegal attribute format'.
                                                self skipToTagc.
                                                ^self ] ].
                        anEvent attributeAtName: key put: (
                                self normalizeAttValue: value ) ].
        self skipSeparators.
        self peekEtagSlash ifTrue: [ ^ true ].
        self peekTagc
                ifTrue: [ self nextChar ]
                ifFalse: [ self fatalError: 'tag close expected during attribute parse' ].

        ^ false.
!

parseEndTag: aTagName
	self skipSeparators.
	self peekTagc
		ifTrue: [self nextChar]
		ifFalse:
			[self fatalError: 'tag close expected'.
			self skipToTagc].
	self endTagCloseAction: aTagName
!

parseStartTag: aTagName

        | event etag |
        etag := self parseElementAttributes: (
                event := self eventClass startTag: aTagName ).
        self nextPutCdataAtEnd: false.
        (etag and:[self processEmptyTag: aTagName event: event]) ifFalse: [
                self pushElement: aTagName event: event ]
!

processEmptyTagEvent: anEvent
	self sendStartElementEvent: anEvent.
	self sendEndElementEvent: anEvent
!

processXmlDeclaration: aString
	"Process a legal ?xml declaration which contains aString
	up to the tag close character. Currently do nothing."
!

runBasicParseLoop

	[[self nextCDATA.
	self atEnd]
		whileFalse:
			[self nextAction].
	streamStack isEmpty]
		whileFalse:
			[self popStream]
!

startParse

	self getDocumentHandler startDocument.
	self resetCdata.
	self nextChar.
	self runBasicParseLoop
! !

!IdStxpParser methodsFor:'parsing action'!

cro
        | num |
        num := self nextXMLTokenNoSkip.
        num isNil ifTrue: [^self nextPutCro].
        self peekRefc ifTrue: [self nextChar].
        cdata nextPut: (Character codePoint: (self stringToInteger: num))
!

ero
	| ename entityValue |
	ename := self nextXMLTokenNoSkip.
	ename isNil ifTrue: [^self nextPutEro].
	(entityValue := self getDtd getEntityValueAt: ename) isNil ifTrue:
		[self fatalError: 'entity ', ename, ' does not exist'.
		self nextPutEro.
		cdata nextPutAll: ename.
		self peekRefc ifTrue:
			[self nextPutRefc.
			self nextChar].
		^self].
	self peekRefc ifTrue: [self nextChar].
	self pushStream: (ReadStream on: entityValue)
!

etago
	| name |
	name := self nextXMLTokenNoSkip.
	name isNil ifTrue: [^self nextPutEtago].
	self parseEndTag: name
!

mdo
	"Process a markup declaration open. Receiver is 
	positioned just after <!! and this can be a comment, 
	CDATA section or some kind of markup declaration.
	Note that DOCTYPE specific delarations are handled
	by the IdStxpDoctypeParser subclass."

	self skipSeparators.  "tolerate sloppy HTML syntax"
	lastChar == $- ifTrue: [ ^self skipComment ].
	lastChar == $[ ifTrue: [ ^self parseCDATASection ].
	lastChar == $D ifTrue: [ ^self parseDOCTYPE ].
	lastChar == $% ifTrue: [ ^self parseConditionalSection ].
	lastChar == $N ifTrue: [ ^self parseNotationDeclaration ].
	self fatalError: 'unrecognized Markup Declaration Open'
!

pio
	"The receiver is positioned just after <?. Handle the
	<?xml declaration here if it is in lower case, otherwise
	report an error. Pass all other PI's to the document handler."

	| target dataBuf targetBuf isLooping dSize |
	targetBuf := WriteStream on: ( String new: 20 ).
	dataBuf := WriteStream on: ( String new: 50 ).
	isLooping := true.
	[ isLooping and: [ lastChar notNil ] ] whileTrue: [
		self lastCharIsSeparator ifTrue: [
			target := targetBuf contents.
			self nextChar.
			self appendUpToTagClose: dataBuf.
			self nextChar.
			isLooping := false ].
		targetBuf nextPut: lastChar.
		self nextChar ].
	isLooping ifTrue: [
		^self fatalError: 'invalid PI' ].
	dSize := ( dataBuf contents size - 1 ) max: 1.
	( self isPITargetXml: target ) ifTrue: [
		target = 'xml' ifFalse: [
			^self fatalError: 'invalid PI - ?xml declaration must be lower case' ].
		^self processXmlDeclaration: (
			dataBuf contents copyFrom: 1 to: dSize ) ].
	self getDocumentHandler processingInstruction: target data: (
		dataBuf contents copyFrom: 1 to: dSize )
!

re
	self nextPutRe
!

stago

	| name |
	name := self nextXMLTokenNoSkip.
	name isNil ifTrue: [ ^self nextPutStago ].
	self parseStartTag: name
! !

!IdStxpParser methodsFor:'scanning'!

appendUpToTagClose: aStream
	"Append all characters which are not the tag close '>'
	character to aStream. This will leave the receiver positioned
	over the > char which will also be stored in lastChar."

	[lastChar notNil and: [lastChar ~~ $>]]
		whileTrue:
			[aStream nextPut: lastChar.
			self nextChar]
!

atEnd
	^lastChar isNil
!

isNameChar: aChar
	^( self isNameFirstChar: aChar )
		or: [ aChar == $. 
		or: [ aChar == $- ]]
!

isNameFirstChar: aChar
	^aChar isAlphaNumeric or: [ aChar == $_ or: [ aChar == $: ]]
!

nextAction
	lastChar isNil ifTrue: [^nil].
	lastChar == $& ifTrue:
		[self nextChar == $#
			ifTrue: [self nextChar. ^self cro]
			ifFalse: [^self ero]].
	lastChar == $< ifTrue:
		[self nextChar == $/
			ifTrue: [self nextChar. ^self etago]
			ifFalse: [lastChar == $!!
				ifTrue: [self nextChar. ^self mdo]
				ifFalse: [lastChar == $?
					ifTrue: [self nextChar. ^self pio]
					ifFalse: [^self stago]]]].
	(self lastCharIsNormalizedLineFeed)
		ifTrue: [self nextChar. ^self re].
	self fatalError: 'unexpected character ', lastChar printString.
!

nextAttributeAssignToken
	self skipSeparators.
	lastChar isNil ifTrue: [^nil].
	lastChar == $= ifTrue: [self nextChar. ^'='].
	^nil
!

nextAttributeValueToken

	| value |
	value := self nextLiteralOrToken.
	value isNil ifTrue: [ ^nil ].
	false "if contains $<" ifTrue: [
		self fatalError: 'No < in Attribute Values' ].
	^value "normalized"
!

nextCDATA
	[lastChar isNil or:
	[lastChar == $& or: [lastChar == $< or: [self lastCharIsNormalizedLineFeed]]]]
		whileFalse:
			[cdata nextPut: lastChar.
			self nextChar]
!

nextChar
        "Line number and position counters are difficult to
        calculate accurately. They are commented out here."

"/ can do it faster under ST/X
        ^ lastChar := stream next.

        stream atEnd ifTrue: [ ^lastChar := nil ].
"       linePos := linePos + 1."
        lastChar := stream next.
"       ( self charIsCrOrLf: lastChar ) ifTrue: [
                self peekIsCrOrLf ifFalse: [
                        lineNum := lineNum + 1.
                        linePos := 0 ] ]."
        ^lastChar
!

nextLiteralOrToken

	| write |
	self skipSeparators.
	lastChar isNil ifTrue: [^nil].
	self lastCharIsQuoteOrParen
		ifTrue: [ ^self nextStringInQuotesOrParen ]
		ifFalse: [
			write := WriteStream on: (String new: 16).
			[lastChar isNil or: [self lastCharIsSeparator or: [lastChar == $> or: [lastChar == $/ ]]]]
				whileFalse:
					[write nextPut: lastChar.
					self nextChar].
			^write contents ].
	^nil
!

nextStringInQuotesOrParen
	"Assume the receiver is positioned over a quote or
	parentheses character. Return the surrounded string
	and advance beyond the matching quote or paren."

	| write ch |
	write := WriteStream on: (String new: 50).
	ch := lastChar.
	self nextChar.
	[lastChar isNil or: [lastChar == ch or: [lastChar == $>]]]
		whileFalse:
			[write nextPut: lastChar.
			self nextChar].
	lastChar = ch ifTrue: [self nextChar].
	^write contents
!

nextToTagc

	self appendUpToTagClose: cdata.
	lastChar == $> ifTrue:
		[self nextPutTagc.
		self nextChar]
!

nextXMLToken
	self skipSeparators.
	^self nextXMLTokenNoSkip
!

nextXMLTokenNoSkip
	| write |
	lastChar == $% ifTrue: [  "allows % in parameter entity decl."
		self nextChar.
		self skipSeparators ].
	lastChar isNil ifTrue: [^nil].
	write := WriteStream on: (String new: 16).
	(self isNameFirstChar: lastChar) ifFalse: [^nil].
	write nextPut: lastChar.
	self nextChar.
	[lastChar notNil and: [self isNameChar: lastChar]]
		whileTrue:
			[write nextPut: lastChar.
			self nextChar].
	^write contents
!

peekEtagSlash
	^lastChar == $/
!

peekEtago
	^lastChar == $< and: [stream atEnd not and: [stream peek == $/]]
!

peekIsCrOrLf
	^stream atEnd not and: [ self charIsCrOrLf: stream peek ]
!

peekRe
	^self lastCharIsNormalizedLineFeed
!

peekRefc
	^lastChar == $;
!

peekTagc
	^lastChar == $>
!

peekTago
	^lastChar == $<
!

skipComment
	"The receiver is positioned over the first - in the comment.
	Comments must be <!!-- ... -->. Advance the receiver to the
	character immediately after the last >."

	self nextChar.
	lastChar == $- ifFalse: [
		^self fatalError: 'invalid comment start' ].
	self nextChar.
	[ lastChar ~~ nil ] whileTrue: [
		lastChar == $- ifTrue: [ 
			"       Transcript cr;show: '$- ', stream position printString, '  ', stream size printString,
						(stream copyFrom: stream position to: stream position + 10) printString."
				self nextChar.
				lastChar == $- ifTrue: [ 
					self upcomingChar == $> ifTrue: [
						self nextChar.
						lastChar == $> ifTrue: [ ^self nextChar ].  "normal exit"
						^self fatalError: 'invalid comment termination - requires > char' ] ] ]
			ifFalse: [ self nextChar ] ].
	self fatalError: 'invalid comment - could not find termination'
!

skipSeparators
	[lastChar notNil and: [self lastCharIsSeparator]]
		whileTrue:
			[self nextChar].
!

skipToCharacter: aChar
	"Skip to the next occurance of aChar or the end
	of the stream. The receiver will be positioned over
	the matching character."

	[lastChar notNil and: [lastChar ~~ aChar]]
		whileTrue: [self nextChar]
!

skipToTagc
	self skipToCharacter: $>.
	lastChar == $> ifTrue: [ self nextChar ]
!

skipUpToSeparator
	[ lastChar notNil and: [ self lastCharIsSeparator not ] ]
		whileTrue: [ self nextChar ]
!

trimLastRe: aString
	(aString isEmpty not and: [self charIsCrOrLf: aString last])
		ifTrue: [^aString copyFrom: 1 to: aString size - 1]
		ifFalse:        [^aString]
!

upToSeparators
	| write |
	write := WriteStream on: (String new: 16).
	[lastChar notNil and: [self lastCharIsSeparator]]
		whileTrue:
			[write nextPut: lastChar.
			self nextChar].
	^write contents
!

upcomingChar
	"Should be called peek, but peek is used many times,
	often when its just checking lastChar."

	^stream atEnd 
		ifTrue: [ nil ]
		ifFalse: [ stream peek ]
!

writeCDATASegmentToBuffer
	"Write all characters to the cdata buffer without trying
	to recognize any markup. Continue until the ] character
	or the end of the stream."

	[ lastChar == nil or: [ lastChar == $] ] ] 
		whileFalse: [
			cdata nextPut: lastChar.
			self nextChar ]
! !

!IdStxpParser methodsFor:'services'!

parse: anInputSource
        "Complete a parse of the XML resource represented
        by anInputSource. If the input source can not be resolved
        to a valid character stream, exit without proceeding or
        rasing an error. Raise an error if the receiver is already
        parsing a document."

        isWorking ifTrue: [
                ^self fatalError: 'existing parse is in progress' ].
        Smalltalk isSmalltalkX ifTrue:[
            stream := anInputSource perform:#getCharacterStream ifNotUnderstood:anInputSource.
        ] ifFalse:[
            baseURI := self getURIClass forUriString: anInputSource getSystemId.
            stream := anInputSource getCharacterStream.
        ].
        stream ~~ nil ifTrue: [
                ^self privateParseStream ].
        stream := self retrieveInputStream: anInputSource.
        stream == nil ifTrue: [ ^self ].
        self executeBlock: [ self privateParseStream ] 
                ensure: [ stream close ]

    "Modified: / 6.9.1999 / 09:10:17 / cg"
! !

!IdStxpParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdStxpParser.st,v 1.14 2009/01/22 17:27:43 stefan Exp $'
! !

IdStxpParser initialize!
