"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdURI
	instanceVariableNames:'protocol path'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Indelv-IdDOMSmalltalk'
!


!IdURI class methodsFor:'class comment'!

classComment
	"This is a simplified implementation of URI in Java. It
	is used to resolve single and multiple URI/URL identifiers."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdURI class methodsFor:'constructors'!

for: aString
	"Return a new URI instance from aString. This is
	equivalent to calling #forBase:string: with nil as the
	first argument."

	^self forBase: nil string: aString
!

forBase: baseURI string: aString
	"Return a new URI instance from aString within the
	context of baseURI. If any of the URI components are
	missing from aString, they will be inherited from baseURI
	if it is not nil.
	The argument aString is parsed for a protocol (file, http, etc.)
	which is everything before a colon (:) which occurs before
	the first slash character (/).
	In this class, everything after the protocol (if one exists) is
	considered the URI path."

	| aProtocol aPath looking |
	aProtocol := nil.
	aPath := aString.
	looking := true.
	1 to: aString size do: [ :i |
		( i > 1 and: [ ( ( i + 1 ) < aString size ) and: [ ( aString at: i ) == $: ] ] ) ifTrue: [
			( looking and: [ ( aString at: i + 1 ) == $/ ] ) ifTrue: [
				looking := false.
				aProtocol := aString copyFrom: 1 to: i - 1.
				aPath := aString copyFrom: i + 2 to: aString size ] ] ].
	( aProtocol == nil and: [ baseURI ~~ nil ] ) ifTrue: [
		aProtocol := baseURI getProtocol.
		aPath := self expandPath: aPath baseURI: baseURI ].
	^self new
		initProtocol: aProtocol path: aPath;
		yourself
!

forUriString: aString
	"Return a new URI instance from aString. This is
	equivalent to calling #forBase:string: with nil as the
	first argument."

	^self forBase: nil string: aString
! !

!IdURI class methodsFor:'internal'!

expandPath: aPath baseURI: baseURI
	"Return a fully expanded version of aPath based
	on baseURI. If baseURI is nil return aPath, otherwise
	fill in relative path names accordingly.
	This method assumes a drive name is the first letter
	immediately followed by a colon (:). If a drive is not
	specified, assume aPath is a relative path and append
	it to the first part of baseURI's path (up to the last path
	separator). If a drive is specified return aPath."

	| lastSep goodSep basePath fileName fileNameStart |
	baseURI == nil ifTrue: [ ^aPath ].
	aPath size = 0 ifTrue: [ ^aPath ].
	( aPath size > 2 and: [ ( aPath at: 2 ) == $: ] ) ifTrue: [ ^aPath ].
	basePath := baseURI getPath.
	( basePath == nil or: [ basePath isEmpty ] ) ifTrue: [ ^aPath ].
	lastSep := goodSep := 0.
"       basePath last == $/ ifFalse: [    Sxp fix - basePath could end with file name
		basePath := basePath, '/' ]."
	1 to: basePath size do: [ :i |
		( basePath at: i ) == $/ ifTrue: [
			goodSep := lastSep.
			lastSep := i ] ].
	fileName := aPath.
	fileNameStart := 1.
	( lastSep > 0 
		and: [ fileName size > 2
		and: [ ( ( fileName copyFrom: 1 to: 2 ) = '..' )
		and: [ true ]]] ) ifTrue: [
			goodSep == 0 ifTrue: [ ^aPath ].
			basePath := basePath copyFrom: 1 to: goodSep.
			lastSep := basePath size.
			( fileName at: 3 ) = $/
				ifTrue: [ fileNameStart := 4 ]
				ifFalse: [ fileNameStart := 3 ].
			fileName := fileName copyFrom: fileNameStart to: fileName size ].
	fileName first == $/ ifTrue: [
		lastSep := lastSep - 1 ].
	lastSep < 1 ifTrue: [ ^fileName ].
	^( basePath copyFrom: 1 to: lastSep ), fileName
!

expandPathSxpVersion: aPath baseURI: baseURI
	"Keep only as long as necessary."

	"Return a fully expanded version of aPath based
	on baseURI. If baseURI is nil return aPath, otherwise
	fill in relative path names accordingly.
	This method assumes a drive name is the first letter
	immediately followed by a colon (:). If a drive is not
	specified, assume aPath is a relative path and append
	it to the first part of baseURI's path (up to the last path
	separator). If a drive is specified return aPath."

	| lastSepIndex basePath |
	baseURI == nil ifTrue: [ ^aPath ].
	aPath size = 0 ifTrue: [ ^aPath ].
	( aPath size > 2 and: [ ( aPath at: 2 ) == $: ] ) ifTrue: [ ^aPath ].
	( basePath := baseURI getPath ) == nil ifTrue: [ ^aPath ].
	lastSepIndex := 0.
	1 to: basePath size do: [ :i |
		( basePath at: i ) == $/ ifTrue: [
			lastSepIndex := i ] ].
	aPath first == $/ ifTrue: [
		lastSepIndex := lastSepIndex - 1 ].
	lastSepIndex < 1 ifTrue: [ ^aPath ].
	^( basePath copyFrom: 1 to: lastSepIndex ), aPath
! !

!IdURI methodsFor:'accessing'!

getPath
	"Return the path if one is defined, else return nil."

	^path
!

getProtocol
	"Return the protocol (file, http, etc.) as a String if
	one is defined, else return the default 'file'. In this 
	class, only the file protocol is supported for 
	retrieving files."

	^protocol == nil
		ifTrue: [ 'file' ]
		ifFalse: [ protocol ]
! !

!IdURI methodsFor:'creation'!

initProtocol: aProtocol path: aPath

	protocol := aProtocol.
	path := aPath
! !

!IdURI methodsFor:'services'!

openStream
	"Return a read only file stream on the receiver if
	possible, else return nil. If a stream is returned it is
	the calling object's responsibility to close it when
	done. If the protocol is not 'file' return nil."

	| stream |
	"self getProtocol = 'file' ifFalse: [ ^nil ]."  "protocol could be drive name"
	self getPath == nil ifTrue: [ ^nil ].
	stream := IdPlatformUtilities openFileTextReader: (
		IdPlatformUtilities convertURIToLocalPath: self getPath ).
	stream ~~ nil ifTrue: [ ^stream ].
	^IdPlatformUtilities openFileTextReader: self toExternalForm
!

toExternalForm
	"Return a string representation of this URI."

	| buffer |
	self getPath == nil ifTrue: [ ^String new ].
	buffer := IdStringBuffer new: self getPath ideLength + 10.
	buffer appendString: self getProtocol.
	buffer appendString: ':/'.
	buffer appendString: self getPath.
	^buffer convertToString
! !

!IdURI class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdURI.st,v 1.6 2003/04/23 08:44:16 cg Exp $'
! !
