"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"


"{ Package: 'stx:goodies/xml/indelv' }"

Object subclass:#IdXMLWriter
	instanceVariableNames:'writeStream currentIndent lineNum lineNumStack isInBody'
	classVariableNames:'ExclChar IndentSize'
	poolDictionaries:''
	category:'XML-Indelv-IdDOMUtils'
!


!IdXMLWriter class methodsFor:'accessing'!

getIndentSize
	"Return the default indent size. See the setter method
	comment for more."

	^IndentSize
!

setIndentSize: anInteger
	"Set the default indent size to anInteger. The indent
	size refers to the number of spaces added to the beginning
	of each output line on which a new XML element begins.
	The indent will also be used to position the end tag for
	elements which take up more than one line. Below is 
	some sample XML output:

<Element1>
  <Element2>
	<Element3>Some plain text</Element3>
  </Element2>
</Element1>

	This example demonstrates the default indent size of 2."

	IndentSize := anInteger
! !

!IdXMLWriter class methodsFor:'class comment'!

classComment
	"This is a utility class used to emit an XML encoding of
	a Document. It writes the character based output to a
	PrintWriter in Java or a WriteStream in Smalltalk which
	both can be used to direct output to files or other objects.
	It is invoked using the writeXMLFor() instance method
	which takes a Document and a Stream as parameters.

	Output can be customized by changing the IndentSize
	class variable which will alter the depth of the indented
	XML tag encoding. See the setIndentSize() class method
	comment for more.

	A utility method encodeDocumentLocally() is provided to
	write the XML output to a local disk file. This will only be
	functional in Java if the security manager allows."
!

copyright
"
    InDelv Open Source
    DOM Kernel
    Beta 1.01
    VisualWorks
    December 21, 1998


    The contents of this file are subject to the MOZILLA PUBLIC LICENSE
    Version 1.0 (the 'MPL'); you may not use this file except in
    compliance with the MPL.  A copy of the MPL has been provided
    with this file. If that copy can not be found, you can obtain another 
    on the Internet at http://www.mozilla.org/NPL/.

    Software distributed under the MPL is distributed on an 'AS IS' basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
    for the specific language governing rights and limitations under the
    MPL.

    The Initial Developer of this code under the MPL is InDelv, Inc.
    Portions created by InDelv are
    Copyright (C) 1998 InDelv, Inc.  All Rights Reserved.
"

! !

!IdXMLWriter class methodsFor:'class initialization'!

initialize
        "Initialize class variables after loading the class.
        The ExclChar variable is needed in Smalltalk."

        IndentSize := 2.
        ExclChar := Character value: 33
!

initializeAfterLoad
        "Initialize class variables after loading the class.
        The ExclChar variable is needed in Smalltalk."

        self initialize

! !

!IdXMLWriter class methodsFor:'examples'!

writeXMLDiskExample
	"Write an XML document created from an example method
	in IdXMLReader to a file on disk. This method does not 
	execute, however you can modify the path name shown below
	to refer a valid file name on your network and execute the code
	fragment in a workspace."
"
	IdXMLWriter encodeDocumentLocally: IdXMLReader runSimpleXMLExample
		to: 'c:\xmldocs\TestOutput.xml'
"
!

writeXMLTranscriptExample
	"Write an XML document created from an example method
	in IdXMLReader to the Transcript."
"
	IdXMLWriter writeXMLTranscriptExample
"
	self new writeXMLFor: IdXMLReader runSimpleXMLExample
		to: Transcript.
	( Smalltalk at: #TextCollector ifAbsent: [nil] ) ~~ nil ifTrue: [
		Transcript endEntry ]  "required in VisualWorks"
! !

!IdXMLWriter class methodsFor:'services'!

encodeDocumentLocally: aDocument to: aPath
	"Encode an XML representation of aDocument to the local
	file stored at aPath. Overwrite the contents of an existing file
	or create a new one if required."

	| aStream |
	aStream := IdPlatformUtilities openFileTextWriter: aPath.
	aStream == nil ifTrue: [ ^self ].
	[ IdXMLWriter new writeXMLFor: aDocument to: aStream ]
		ideEnsureCompletion: [ aStream close ]
! !

!IdXMLWriter methodsFor:'encoding'!

encodeCDATA: aNode

	writeStream nextPut: $<.
	writeStream nextPut: ExclChar.
	writeStream nextPutAll: '[CDATA['.
	writeStream nextPutAll: aNode getNodeValue.
	writeStream nextPutAll: ']]>'
!

encodeComment: aNode

	writeStream nextPut: $<.
	writeStream nextPut: ExclChar.
	writeStream nextPutAll: '--'.
	writeStream nextPutAll: aNode getNodeValue.
	writeStream nextPutAll: '-->'
!

encodeComposite: aNode

	| tagName startLine |
	tagName := aNode getNodeName.
	self writeStartTagOpen: tagName attributes: aNode getAttributesList.
	writeStream nextPut: $>.
	lineNumStack add: lineNum ideAsIntegerObject.
	self increaseIndent.
	0 to: aNode getLength - 1 do: [ :i |
		self encodeNode: ( aNode itemImpl: i ) ].
	self decreaseIndent.
	startLine := lineNumStack last.
	startLine ideAsPrimitiveInt < lineNum ifTrue: [
		lineNum := lineNum + 1.
		self startNewIndentedLine ].
	lineNumStack removeAtIndex: lineNumStack size.
	self writeEndTag: tagName
!

encodeDocument: aDocument

	| childNode |
	writeStream nextPutAll: '<?xml version="1.0"?>'.
	"self encodeDocumentType: aDocument getDocumentType."
	0 to: aDocument getLength - 1 do: [ :i |
		childNode := aDocument itemImpl: i.
		childNode isTypePROCESSING_INSTRUCTION ifTrue: [  "what else?"
			self encodePI: childNode ] ].
	writeStream cr;cr.
	self encodeNode: aDocument getRootElement
!

encodeEmpty: aNode

	self writeStartTagOpen: aNode getNodeName
		attributes: aNode getAttributesList.
	writeStream nextPutAll: '/>'
!

encodeNode: aNode
	"Still need to add Entity refs."

	aNode isTypeTEXT ifTrue: [
		self encodeStringNode: aNode.
		^self ].
	aNode isTypeELEMENT ifTrue: [ 
		aNode hasChildNodes
			ifTrue: [ self encodeComposite: aNode ]
			ifFalse: [ self encodeEmpty: aNode ].
		^self ].
	aNode isTypeCDATA_SECTION ifTrue: [
		self encodeCDATA: aNode.
		^self ].
	aNode isTypeCOMMENT ifTrue: [
		self encodeComment: aNode.
		^self ].
	aNode isTypePROCESSING_INSTRUCTION ifTrue: [
		self encodePI: aNode.
		^self ].
!

encodePI: aNode

	writeStream 
		cr;
		nextPutAll: '<?';
		nextPutAll: aNode getTarget;
		nextPut: $ .
	writeStream nextPutAll: aNode getNodeValue.
	writeStream nextPutAll: '?>'
!

encodeStringNode: aNode

	writeStream nextPutAll: (
		self insertEntities: aNode getNodeValue )
! !

!IdXMLWriter methodsFor:'internal'!

decreaseIndent

	currentIndent ideLength <= IndentSize ifTrue: [
		currentIndent := String new.
		^self ].
	currentIndent := currentIndent 
		copyFrom: 1
		to: currentIndent ideLength - IndentSize
!

increaseIndent

	| buffer |
	buffer := IdStringBuffer new: IndentSize + currentIndent ideLength.
	buffer appendString: currentIndent.
	1 to: IndentSize do: [ :i |
		buffer appendChar: $  ].
	currentIndent := buffer convertToString
!

insertEntities: aString

	| buffer nextChar entityString |
	buffer := IdStringBuffer new: aString ideLength.
	1 to: aString ideLength do: [ :i |
		entityString := nil.
		nextChar := aString ideCharAt: i.
		nextChar ideCharValue == 38  "$&" ifTrue: [ entityString := 'amp' ].
		nextChar ideCharValue == 60  "$<" ifTrue: [ entityString := 'lt' ].
		nextChar ideCharValue == 62  "$>" ifTrue: [ entityString := 'gt' ].
		nextChar ideCharValue == 39  "$'" ifTrue: [ entityString := 'apos' ].
		nextChar ideCharValue == 34          ifTrue: [ entityString := 'quot' ].
		entityString ~~ nil ifTrue: [
				buffer
					appendChar: $&;
					appendString: entityString;
					appendChar: $; ]
			ifFalse: [ buffer appendChar: nextChar ] ].
	^buffer convertToString
!

startNewIndentedLine
	"Create a new line and start it with the current indent."

	writeStream cr;nextPutAll: currentIndent
!

writeEndTag: tagName

	writeStream
		nextPutAll: '</';
		nextPutAll: tagName;
		nextPut: $>
!

writeStartTagOpen: tagName attributes: attributeMap

	| nextAtt count indentAtts |
	isInBody ifTrue: [
			lineNum := lineNum + 1.
			writeStream cr ]
		ifFalse: [ isInBody := true ].
	writeStream
		nextPutAll: currentIndent;
		nextPut: $<;
		nextPutAll: tagName.
	count := attributeMap getLength.
	count == 0 ifTrue: [ ^self ].
	indentAtts := count > 1.
	self increaseIndent.
	self increaseIndent.  "increase twice to stand out from children"
	0 to: count - 1 do: [ :i |
		nextAtt := attributeMap item: i.
		indentAtts 
			ifTrue: [ self startNewIndentedLine ]
			ifFalse: [ writeStream nextPut: $  ].
		writeStream nextPutAll: nextAtt getNodeName.
		writeStream nextPutAll: '="'.
		writeStream nextPutAll: nextAtt getNodeValue.
		writeStream nextPut: $" ].
	self decreaseIndent.
	self decreaseIndent.
! !

!IdXMLWriter methodsFor:'services'!

writeXMLFor: aDocument to: aStream
	"Write an XML encoding of aDocument to aStream. Note
	that the calling method is responsible for closing aStream
	when done."

	writeStream := aStream.
	currentIndent := String new.
	lineNumStack := OrderedCollection new.
	lineNum := 1.
	isInBody := false.
	self encodeDocument: aDocument
! !

!IdXMLWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/indelv/IdXMLWriter.st,v 1.7 2003/04/23 08:44:16 cg Exp $'
! !

IdXMLWriter initialize!
