"{ Package: 'stx:goodies/xml/indelv' }"
!

!Array methodsFor:'ID extensions'!

ideArrayLength
	"Return the length of the receiver."

	^self size
! !

!Array methodsFor:'ID extensions'!

ideCharArrayToString
	"Return a String which contains the receiver's contents.
	It is assumed the receiver is an array of characters. This will
	translate to toString() in Java."

	| answer |
	answer := String new: self size.
	answer replaceFrom: 1 to: self size with: self startingAt: 1.
	^answer
! !

!Behavior methodsFor:'ID extensions'!

ideNewInstance
	"Return a new instance of the receiver. This maps
	to newInstance() in Java."

	^self new
! !

!Block methodsFor:'ID extensions'!

ideEnsureCompletion: completionBlock
	"Evaluate the receiver normally and ensure that 
	completionBlock executes no matter what happens."

	self ensure: completionBlock
! !

!Block methodsFor:'ID extensions'!

ideOnException: exceptionClass do: handlerBlock
	"Evaluate the receiver in the context of an exception
	represented by exceptionClass, which is a subclass
	of IdException."

	exceptionClass getSignal
		handle: handlerBlock
		do: self
! !

!Block methodsFor:'ID extensions'!

ideOnException: exceptionClass do: handlerBlock finally: completionBlock
	"Evaluate the receiver as a normal #ideOnException:do:
	and ensure that completionBlock executes no matter
	what happens."

	[
	    self ideOnException: exceptionClass do: handlerBlock
	] ensure: completionBlock
! !

!Block methodsFor:'ID extensions'!

ideOnException: exClass1 do: handler1 when: exClass2 do: handler2
	"Evaluate the receiver in the context of multiple exceptions
	and handler blocks."

	HandlerList new
		on: exClass2 getSignal handle: handler2;
		on: exClass1 getSignal handle: handler1;
		handleDo: self
! !

!Block methodsFor:'ID extensions'!

ideOnException: exClass1 do: handler1 
	when: exClass2 do: handler2 
	when: exClass3 do: handler3
		"Evaluate the receiver in the context of multiple exceptions
		and handler blocks."

	HandlerList new
		on: exClass3 getSignal handle: handler3;
		on: exClass2 getSignal handle: handler2;
		on: exClass1 getSignal handle: handler1;
		handleDo: self
! !

!Block methodsFor:'ID extensions'!

ideOnException: exClass1 do: handler1 
	when: exClass2 do: handler2 
	when: exClass3 do: handler3
	when: exClass4 do: handler4
		"Evaluate the receiver in the context of multiple exceptions
		and handler blocks."

	HandlerList new
		on: exClass4 getSignal handle: handler4;
		on: exClass3 getSignal handle: handler3;
		on: exClass2 getSignal handle: handler2;
		on: exClass1 getSignal handle: handler1;
		handleDo: self
! !

!Boolean methodsFor:'ID extensions'!

ideAsBooleanObject
	"Return the receiver. This method is used to
	convert the primitive boolean in Java to an instance of
	the Boolean Object which can then be stored in
	Vectors, etc."

	^self
! !

!Boolean methodsFor:'ID extensions'!

ideAsPrimitiveBoolean
	"Return the receiver. This method is used to
	convert an Boolean instance in Java to a primitive
	boolean which can then be used in control 
	statements, etc\."

	^self
! !

!Character methodsFor:'ID extensions'!

ideCharValue
	"Return the integer char value of the receiver."

	^self asInteger
! !

!Character methodsFor:'ID extensions'!

idePrintPrimitive
	"Return a string representation of the receiver. This
	calls #asString in Smalltalk and nothing in Java (see
	implementation in Object)."

	^self asString
! !

!Dictionary methodsFor:'ID extensions'!

ideAtKey: aKey put: aValue
	"Store aValue at aKey. No return. Will convert 
	to put() in Java. Use of the normal #at:put: will
	convert to setElementAt() in Java (for collections)."

	self at: aKey put: aValue
! !

!Dictionary methodsFor:'ID extensions'!

ideAtKeyOrNil: aKey
	"Return the value at aKey or nil. Will convert to
	get() in Java which also returns null if not found."

	^self at: aKey ifAbsent: [nil]
! !

!Exception methodsFor:'ID extensions'!

ideAsExceptionInstance
	"Return the IdException instance represented by the receiver."

	^self originator
! !

!Exception methodsFor:'ID extensions'!

ideExitGracefully
	"The receiver is an exception being handled by
	portable ID application code. Have it exit with nil
	as an argument."

	self return
! !

!IdentityDictionary methodsFor:'ID extensions'!

ideAtKey: aKey put: aValue
	"Store aValue at aKey. No return. Will convert 
	to put() in Java. Use of the normal #at:put: will
	convert to setElementAt() in Java (for collections)."

	self at: aKey put: aValue
! !

!IdentityDictionary methodsFor:'ID extensions'!

ideAtKeyOrNil: aKey
	"Return the value at aKey or nil. Will convert to
	get() in Java which also returns null if not found."

	^self at: aKey ifAbsent: [nil]
! !

!Integer methodsFor:'DOM extensions'!

ideAsIntegerObject
	"Return the receiver. This method is used to
	convert the primitive int in Java to an instance of
	the Integer Object which can then be stored in
	Vectors, etc."

	^self
! !

!Integer methodsFor:'DOM extensions'!

ideAsPrimitiveInt
	"Return the receiver. This method is used to
	convert an Integer instance in Java to a primitive
	int which can then be used for mathmatical
	operations, etc\."

	^self
! !

!Number methodsFor:'ID extensions'!

ideAsDoubleObject
	"Return the receiver. This method is used to
	convert the primitive double in Java to an instance of
	the Double Object which can then be stored in
	Vectors, etc."

	^self
! !

!Number methodsFor:'ID extensions'!

ideAsPrimitiveDouble
	"Return the receiver. This method is used to
	convert an Double instance in Java to a primitive
	double which can then be used for mathmatical
	operations, etc\."

	^self
! !

!Object methodsFor:'ID extensions'!

ideAsExceptionInstance
	"Return the exception represented by the receiver. 
	By default return the receiver itself."

	^self ideYourself
! !

!Object methodsFor:'ID extensions'!

ideGetClassName
	"Return the name of the receiver's class."

	^self class name
! !

!Object methodsFor:'ID extensions'!

ideIsCompositeNode
	"Return true if the receiver is a composite node,
	else return false."

	^false
! !

!Object methodsFor:'ID extensions'!

ideIsElementChildIndex
	"Return true if the receiver is an element child index,
	else return false."

	^false
! !

!Object methodsFor:'ID extensions'!

ideIsElementNode
	"Return true if the receiver is an element node,
	else return false."

	^false
! !

!Object methodsFor:'ID extensions'!

ideIsEventMulticaster
	"Return true if the receiver is an event multicaster,
	else return false."

	^false
! !

!Object methodsFor:'ID extensions'!

ideIsNodeImpl
	"Return true if the receiver is a NodeImpl instance,
	else return false."

	^false
! !

!Object methodsFor:'ID extensions'!

idePrintPrimitive
	"Return a string representation of the receiver which
	is assumed to be a primitive data type in Java. This
	calls #printString in Smalltalk and nothing in Java.
	In Java primitives can not reply to toString() but they
	are automatically converted to strings during common
	string operations like concatenation (+). Note that this
	method also works well for strings as it is safe for
	values of nil."

	^self printString
! !

!Object methodsFor:'ID extensions'!

idePrintString
	"Return a string representation of the receiver. This
	calls #printString in Smalltalk and toString() in Java.
	Use for reference objects - for primitive data types
	and strings use #idePrintPrimitive."

	^self printString
! !

!Object methodsFor:'ID extensions'!

ideYourself
	"Return the receiver. This is used to generate an 
	explict 'return this' in java. Normally a ^self in Smalltalk
	generates a 'return' in java with a void return type."

	^self
! !

!OrderedCollection methodsFor:'ID extensions'!

ideAtIndex: anIndex
	"Return the object stored at anIndex. Will convert to
	Vector.elementAt() in Java which throws an exception
	when the index is wrong."

	^self at: anIndex
! !

!OrderedCollection methodsFor:'ID extensions'!

ideAtIndex: anIndex put: anObject
	"Store anObject at anIndex. Will convert to
	Vector.setElementAt() in Java."

	self at: anIndex put: anObject
! !

!ReadStream methodsFor:'ID extensions'!

ideReadNext
	"Return the next character read, or -1 if at the end
	of the stream. This will map to StringReader read() 
	in Java."

	self atEnd ifTrue: [ ^-1 ].
	^self next
! !

!String methodsFor:'ID extensions'!

ideCharAt: anInteger
	"Return the character stored at anInteger. Will convert to
	charAt() in Java."

	^self at: anInteger
! !

!String methodsFor:'ID extensions'!

ideLength
	"Return the receiver's length. Will convert to
	length() in Java."

	^self size
! !

!String methodsFor:'ID extensions'!

idePrintPrimitive
	"Return a string representation of the receiver. This
	calls #asString in Smalltalk (to strip out the single quote
	characters) and nothing in Java (see implementation 
	in Object)."

	^self asString
! !

!String methodsFor:'ID extensions'!

ideStringTest: aString
	"Return true if the receiver equals aString,
	else return false. This translates to equals() in 
	Java. Identity testing two symbols is 50% faster 
	on IBM Smalltalk than this method, however
	sending asSymbol once is 850 times slower than
	sending = once.

	Use == if you know both are Symbols, or this method 
	if you know both are Strings. This method is usually 
	faster overall and provides better unicode string 
	matching support in Java."

	^self = aString asString
! !

!Symbol methodsFor:'ID extensions'!

idePrintPrimitive
	"Return a string representation of the receiver. This
	calls #asString in Smalltalk (to strip out the # character)
	and nothing in Java (see implementation in Object)."

	^self asString
! !

!Symbol methodsFor:'ID extensions'!

ideStringTest: aString
	"Return true if the receiver equals aString,
	else return false. This translates to equals() in 
	Java. This version is very inefficient because it
	must create a new string to test. Use == if you
	are sure both are Symbols, or this method if
	you are sure both are Strings. See comment of 
	String version of this method for more."

	^self asString ideStringTest: aString
! !

