"{ Package: 'stx:goodies/xml/sixx' }"

Object subclass:#SixxXmlUtil
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Core'
!

SixxXmlUtil class instanceVariableNames:'convertCharDict parserAdapter'

"
 No other class instance variables are inherited by this class.
"
!


!SixxXmlUtil class methodsFor:'accessing'!

convertCharDict
	^convertCharDict
!

parserAdapter
        |nameSpace|

        nameSpace := self environment.

        Smalltalk isSmalltalkX ifTrue:[
            parserAdapter isNil
                ifTrue: [parserAdapter := (nameSpace includesKey: #'YAXO::XMLDOMParser')
                                                ifTrue: [nameSpace at: #SixxYaxoXmlParserAdapter]
                                                ifFalse: [(nameSpace includesKey: #'XML::XMLParser')
                                                                ifTrue: [nameSpace at: #SixxVwXmlParserAdapter]]].
        ] ifFalse:[
            parserAdapter isNil
                ifTrue: [parserAdapter := (nameSpace includesKey: #XMLDOMParser)
                                                ifTrue: [nameSpace at: #SixxYaxoXmlParserAdapter]
                                                ifFalse: [(nameSpace includesKey: #VWXMLXMLParser)
                                                                ifTrue: [nameSpace at: #SixxVwXmlParserAdapter]]].
        ].
        ^ parserAdapter
!

parserAdapter: aSixxXmlParserAdapter 
	parserAdapter := aSixxXmlParserAdapter
! !

!SixxXmlUtil class methodsFor:'actions'!

asXmlText: aString 
	|  wStr |
	wStr := WriteStream
				on: (String new: aString size).
	^(self writeXmlText: aString on: wStr) contents
!

contentsWithXmlDeclaration: aString 
	| wStr |
	wStr := WriteStream
				on: (String new: 1024).
	wStr nextPutAll: self xmlDeclaration.
	2
		timesRepeat: [wStr nextPut: Character cr.
			wStr nextPut: Character lf].
	wStr nextPutAll: aString.
	^ wStr contents
!

parseXml: xmlStringOrStream 
        ^[self parserAdapter parseXml: xmlStringOrStream] on: Error
                do: [:ex | SixxPortableUtil signalException: (SixxXmlParseError originalError: ex)]
!

writeXmlText: aString on: aWriteStream 
	SixxSetting useCDATA
		ifTrue: [aWriteStream nextPutAll: '<!![CDATA['.
			aWriteStream nextPutAll: aString.
			aWriteStream nextPutAll: ']]>']
		ifFalse: [self convertWriteXmlText: aString on: aWriteStream]
! !

!SixxXmlUtil class methodsFor:'actions-parse'!

attributeAt: attributeName from: sixxElement 
	^ self parserAdapter attributeAt: attributeName from: sixxElement
!

characterDataFrom: sixxElement
	^ self parserAdapter characterDataFrom: sixxElement
!

elementsFrom: sixxElement
	^ self parserAdapter elementsFrom: sixxElement
!

isTextFrom: sixxElement
	^ self parserAdapter isTextFrom: sixxElement
!

nameFrom: sixxElement 
	^ self parserAdapter nameFrom: sixxElement
! !

!SixxXmlUtil class methodsFor:'class initialization'!

initialize
	"SixxXmlUtil initialize"
	self initConvertCharDict.
	parserAdapter := nil
! !

!SixxXmlUtil class methodsFor:'private-initialize'!

convertWriteXmlText: aString on: aWriteStream
	| cDict wStr |
	cDict := self convertCharDict.
	wStr := aWriteStream.
	aString
		do: [:eachChar | 
			| val | 
			val := eachChar asciiValue.
			val >= 127
				ifTrue: [wStr nextPutAll: '&#';
						 nextPutAll: val printString;
						 nextPut: $;]
				ifFalse: [| conv | 
					conv := cDict
								at: val
								ifAbsent: [].
					conv isNil
						ifTrue: [wStr nextPut: eachChar]
						ifFalse: [wStr nextPutAll: conv]]].
	^wStr
!

initConvertCharDict
        convertCharDict := Dictionary new.
        0 to: 47 do: [:each | convertCharDict at: each put: '&#', each asString, ';'].
        convertCharDict 
                at: $& asciiValue put: '&amp;';
                at: $< asciiValue put: '&lt;';
                at: $> asciiValue put: '&gt;';
                at: $' asciiValue put: '&apos;';
                at: $" asciiValue put: '&quot;'.
        convertCharDict 
                removeKey: Character space asciiValue;
                removeKey: Character tab asciiValue.
        Smalltalk isSmalltalkX ifTrue:[
            convertCharDict 
                removeKey: Character return asciiValue;
                removeKey: Character nl asciiValue.
        ] ifFalse:[
            convertCharDict 
                removeKey: Character cr asciiValue;
                removeKey: Character lf asciiValue.
        ].
        convertCharDict 
                removeKey: $/ asciiValue;
                removeKey: $- asciiValue;
                removeKey: $. asciiValue.
! !

!SixxXmlUtil class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/SixxXmlUtil.st,v 1.3 2003/10/08 09:31:35 cg Exp $'
! !

SixxXmlUtil initialize!
