"{ Package: 'stx:goodies/xml/sixx/examples' }"

Object subclass:#SixxExamples
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Examples'
!


!SixxExamples class methodsFor:'examples'!

example1
	"SixxExamples example1"
	"Create a SIXX string from an object (like #storeString)"
	
	| dict array xmlString |
	dict := Dictionary new.
	dict at: #Smalltalk put: 'Best'.
	dict at: #Ruby put: 'Cool'.
	dict at: #Java put: 'Forget it'.
	array := Array new:3.
	array at: 1 put: 'Language Impressions'.
	array at: 2 put: dict.
	array at: 3 put: Date today.
	xmlString := array sixxString.
	Transcript cr; show: '##This is sixx xml instance##'. 
	Transcript cr; show: xmlString.
	
	^xmlString 
!

example2
	"SixxExamples example2"
	"Read a SIXX string and restore an instance (like #readFrom:)"
	
	| xmlString obj |
	xmlString := SixxExamples example1.
	obj := Object readSixxFrom: xmlString.
	obj inspect. 
!

example3
	"SixxExamples example3"
	"Resolve circular and shared objects"
	
	| rootOrdered sharedDict subOrdered subArray xmlString obj |
	rootOrdered := OrderedCollection new.
	sharedDict := Dictionary new.
	sharedDict at: #hello put: 'Hello'.
	sharedDict at: #recurHello put: sharedDict.
	subOrdered := OrderedCollection new.
	subOrdered add: sharedDict.
	subOrdered add: Time now.
	subArray := Array new: 2.
	subArray at: 1 put: sharedDict.
	subArray at: 2 put: Date today.
	rootOrdered add: subOrdered; add: subArray.
	
	xmlString := rootOrdered sixxString.
	Transcript cr; show: '##This is sixx xml instance##'. 
	Transcript cr; show: xmlString.
	
	obj := Object readSixxFrom: xmlString.
	Transcript cr; show: '##Shared object comparison##'. 
	Transcript cr; show: obj first first = obj last first.
	obj inspect. 
	
	
	

!

example4
	"SixxExamples example4"
	"Save and load Preferences data"
	
	| preferences |
	preferences := Preferences allPreferenceObjects sixxString.
	preferences := Object readSixxFrom: preferences.
	^preferences inspect.

	

!

example5
	"SixxExamples example5"
	"Write object data to an external file"
	
	| sws |
	sws := SixxWriteStream newFileNamed: ('obj.sixx').
	sws nextPut: Color random.
	sws nextPut: Rectangle fromUser.
	sws nextPut: View subclasses.
	sws nextPutAll: View subclasses. "note the differences"
	sws close.
!

example6
	"SixxExamples example6"
	"Read object data from an external file"
	
	| srs objects |
	srs := SixxReadStream readOnlyFileNamed: ('obj.sixx').
	objects := OrderedCollection new.
	[srs atEnd] whileFalse: [objects add: srs next].
	srs close.
	objects inspect.
!

example7
	"SixxExamples example7"

	"Write object data to an external file"

	| sws dic |
	sws := SixxWriteStream newFileNamed: 'obj2.sixx'.
	sws nextPut: Color red.
	sws nextPut: Rectangle fromUser.
	sws nextPut: true.
	sws nextPut: false.
	sws nextPut: $a.
	sws nextPut: $(.
	sws nextPut: (OrderedCollection with: 1 with: Date today).
	sws nextPut: Time now.
	sws nextPut: nil.
	sws nextPut: (ByteArray 
				with: 10
				with: 20
				with: 30).
	dic := Dictionary new.
	dic at: 'first' put: '111'.
	dic at: 'second' put: '222'.
	sws nextPut: dic.
	sws close
! !

!SixxExamples class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/examples/SixxExamples.st,v 1.1 2003/09/29 08:14:34 cg Exp $'
! !
