"{ Package: 'stx:goodies/xml/sixx/examples' }"

Object subclass:#SixxMorphMemento
	instanceVariableNames:'morph'
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Examples'
!


!SixxMorphMemento class methodsFor:'instance creation'!

on: aMorph
	^self new morph: aMorph; yourself
! !

!SixxMorphMemento class methodsFor:'examples'!

example1
	"SixxMorphMemento example1"
	"Create a SIXX string from an StarMorph"

	| morph memento |
	morph := StarMorph new openInWorld.
	memento := SixxMorphMemento on: morph.
	^memento sixxString
!

example2
	"SixxMorphMemento example2"
	"Read a SIXX string and restore a StarMorph. Original StarMorph is still there"

	| xmlMorphString memento morph |
	xmlMorphString := self example1.
	memento := Object readSixxFrom: xmlMorphString.
	morph := memento morph.
	morph openInWorld.
	morph color: Color random.
	morph position: (morph position + (20@20)).
	^morph
!

example3
	"SixxMorphMemento example3"
	"Write object data to an external file"

	| sws  |
	sws := SixxWriteStream newFileNamed: ('morph.sixx').
	sws nextPut: (SixxMorphMemento on: (BouncingAtomsMorph new position: 200@200)).
	sws close.
!

example4
	"SixxMorphMemento example4"
	"Read a Mprph from an external file"

	| srs memento morph |
	srs := SixxReadStream readOnlyFileNamed: ('morph.sixx').
	memento := srs next.
	srs close.
	morph := memento morph.
	morph openInWorld.
	morph inspect.
! !

!SixxMorphMemento methodsFor:'accessing'!

morph
	^morph
!

morph: aMorph
	morph := aMorph
! !

!SixxMorphMemento methodsFor:'factory'!

streamClass
	^ReferenceStream
! !

!SixxMorphMemento methodsFor:'private'!

morphForStore
	| file con |
	self morph allMorphsDo: [:m | m prepareToBeSaved].
	file := RWBinaryOrTextStream on: (ByteArray new: 5000).
	file binary.
	(self streamClass on: file)	nextPut: self morph.
	con := Base64MimeConverter mimeEncode: file reset.
	file close.
	^con contents
!

morphFrom: aString
	| byteStr oStr |
	byteStr := (Base64MimeConverter mimeDecodeToBytes: (ReadStream on: aString)).
	oStr := self streamClass on: byteStr.
	^oStr next
! !

!SixxMorphMemento methodsFor:'writing/reading sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	"override if you like"
	aStream cr.
	self morphForStore
		sixxOn: aStream
		name: 'morph'
		indent: level + 1
		context: dictionary.
	aStream tab: level
!

sixxInstVarNamed: instVarName put: aValue 
	"override if you like"
	instVarName = 'morph' ifTrue: [self morph: (self morphFrom: aValue)]
! !

!SixxMorphMemento class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/examples/SixxMorphMemento.st,v 1.1 2003/09/29 08:14:19 cg Exp $'
! !
