"{ Package: 'stx:goodies/xml/sixx' }"
!

!ArrayedCollection methodsFor:'initializing-sixx'!

initializeFromSixxElement: sixxElement context: aDictionary 
	| idx |
	idx := 1.
	(SixxXmlUtil elementsFrom: sixxElement)
		do: [:elem | | childInst | (SixxXmlUtil isTextFrom: elem)
				ifFalse: [childInst := self class fromSixxElement: elem context: aDictionary.
					self at: idx put: childInst.
					idx := idx + 1]]
! !

!Behavior methodsFor:'private-sixx'!

classFromSixxElement: sixxElement context: dictionary 
	| isClassClass type className retClass |
	isClassClass := false.
	type := self typeFromSixxElement: sixxElement.
	type isNil
		ifTrue: [^ nil].
	className := (type includes: Character space)
				ifTrue: [isClassClass := true.
					type copyUpTo: Character space]
				ifFalse: [type].
	retClass := SixxPortableUtil
				classAt: className
				in: (self envFromSixxElement: sixxElement).
	isClassClass
		ifTrue: [retClass := retClass class].
	^ retClass isNil
		ifTrue: [SixxClassNotFound
				signal: className
				element: sixxElement
				context: dictionary]
		ifFalse: [retClass]
! !

!Behavior methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	"override if you like"
	^ aClass isVariable
		ifTrue: [| nonameElems size | 
			nonameElems := (SixxXmlUtil elementsFrom: sixxElement)
						select: [:each | (self nameFromSixxElement: each) isNil].
			size := nonameElems isEmpty
						ifTrue: [0]
						ifFalse: [(SixxXmlUtil elementsFrom: nonameElems first) size].
			aClass new: size]
		ifFalse: [aClass new]
! !

!Behavior methodsFor:'private-sixx'!

envFromSixxElement: sixxElement 
	| environmentQualifier |
	(SixxXmlUtil isTextFrom: sixxElement)
		ifTrue: [^ nil].
	environmentQualifier := SixxXmlUtil attributeAt: #'sixx.env' from: sixxElement.
	^ environmentQualifier isNil
		ifFalse: [environmentQualifier asSymbol]
! !

!Behavior methodsFor:'private-sixx'!

fromSixxElement: sixxElement context: aDictionary 
	^ (self includesRefInstanceFromSixxElement: sixxElement context: aDictionary)
		ifTrue: [self refInstanceFromSixxElement: sixxElement context: aDictionary]
		ifFalse: [self newInstanceFromSixxElement: sixxElement context: aDictionary]
! !

!Behavior methodsFor:'private-sixx'!

idFromSixxElement: sixxElement 
	| id |
	(SixxXmlUtil isTextFrom: sixxElement) ifTrue: [^nil].
	id := SixxXmlUtil attributeAt: #'sixx.id' from: sixxElement .
	^ id isNil
		ifFalse: [id asSymbol]
! !

!Behavior methodsFor:'private-sixx'!

idrefFromSixxElement: sixxElement
	| idref |
	(SixxXmlUtil isTextFrom: sixxElement) ifTrue: [^nil].
	idref := SixxXmlUtil attributeAt: #'sixx.idref' from: sixxElement .
	^ idref isNil
		ifFalse: [idref asSymbol]
! !

!Behavior methodsFor:'private-sixx'!

includesRefInstanceFromSixxElement: sixxElement context: aDictionary 
	| idref |
	idref := self idrefFromSixxElement: sixxElement.
	^ idref isNil
		ifTrue: [false]
		ifFalse: [aDictionary includesKey: idref]
! !

!Behavior methodsFor:'private-sixx'!

nameFromSixxElement: sixxElement 
	| nm |
	(SixxXmlUtil isTextFrom: sixxElement) ifTrue: [^nil].
	nm := SixxXmlUtil attributeAt: #'sixx.name' from: sixxElement .
	^ nm isNil
		ifFalse: [nm asSymbol]
! !

!Behavior methodsFor:'private-sixx'!

newInstanceFromSixxElement: sixxElement context: aDictionary 
	| class inst |
	class := self classFromSixxElement: sixxElement context: aDictionary .
	class isNil
		ifTrue: [SixxClassNotFound
				signal: (self typeFromSixxElement: sixxElement)
				element: sixxElement
				context: aDictionary].
	inst := class createInstanceOf: class withSixxElement: sixxElement.
	aDictionary
		at: (self idFromSixxElement: sixxElement)
		put: inst.
	inst initializeFromSixxElement: sixxElement context: aDictionary.
	^ inst
! !

!Behavior methodsFor:'instance creation-sixx'!

readSixxFrom: xmlStringOrStream 
	^ self readSixxFrom: xmlStringOrStream context: IdentityDictionary new
! !

!Behavior methodsFor:'instance creation-sixx'!

readSixxFrom: xmlStringOrStream context: aDictionary 
	| topElem |
	topElem := SixxXmlUtil parseXml: xmlStringOrStream.
	^ self readSixxFromSixxElement: topElem context: aDictionary
! !

!Behavior methodsFor:'instance creation-sixx'!

readSixxFromSixxElement: sixxElement context: aDictionary 
	| rootClass typeAttr |
	typeAttr := self typeFromSixxElement: sixxElement.
	^typeAttr isNil
		ifTrue: [(self includesRefInstanceFromSixxElement: sixxElement context: aDictionary)
				ifTrue: [self refInstanceFromSixxElement: sixxElement context: aDictionary]]
		ifFalse: [rootClass := self classFromSixxElement: sixxElement context: aDictionary.
			rootClass isNil
				ifFalse: [rootClass fromSixxElement: sixxElement context: aDictionary]]
! !

!Behavior methodsFor:'private-sixx'!

refInstanceFromSixxElement: sixxElement context: aDictionary 
	| idref |
	idref := self idrefFromSixxElement: sixxElement.
	^aDictionary at: idref
! !

!Behavior methodsFor:'private-sixx'!

typeFromSixxElement: sixxElement 
	| type |
	(SixxXmlUtil isTextFrom: sixxElement)
		ifTrue: [^ nil].
	type := SixxXmlUtil attributeAt: #'sixx.type' from: sixxElement.
	^ type isNil
		ifFalse: [type asSymbol]
! !

!Boolean methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self storeString
! !

!Boolean class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!ByteArray methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: '#( '.
	self do:[:each | aStream nextPutAll: each printString; space].
	aStream nextPutAll: ')'.
! !

!ByteArray class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	| col |
	col := Array readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement)).
	^ col asByteArray
! !

!Character methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self storeString
	
! !

!Character class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!ClassDescription methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	| instVars startIndex  |	
	aStream cr.
	instVars := self class allInstVarNames.
	startIndex := Object class allInstVarNames size + 1.
	startIndex to: instVars size do: [:idx | 
			(self instVarAt: idx)
				sixxOn: aStream
				name: (instVars at: idx)
				indent: level + 1
				context: dictionary].
! !

!Collection methodsFor:'initializing-sixx'!

initializeFromSixxElement: sixxElement context: aDictionary 
	(SixxXmlUtil elementsFrom: sixxElement)
		do: [:elem | 
			| childInst | 
			childInst := self class fromSixxElement: elem context: aDictionary.
			self add: childInst]
! !

!Collection methodsFor:'private-sixx'!

sixxAllContentsOn: aStream indent: level context: dictionary
	"Usually Collection's inst vars are not needed for serialization"
	self sixxElementsOn: aStream indent: level context: dictionary.
! !

!Collection methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream cr.
	self
		do: [:each | each
				sixxOn: aStream
				indent: level + 1
				context: dictionary].
	aStream tab: level.
! !

!Collection methodsFor:'private-sixx'!

sixxElementsOn: aStream indent: level context: dictionary
	"override if you like"
	self size = 0 ifTrue: [^self].
	aStream cr.
	self do: 
			[:each | 
			each 
				sixxOn: aStream
				indent: level
				context: dictionary]

! !

!Collection class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	"override if you like"
	^aClass new: (SixxXmlUtil elementsFrom: sixxElement) size
! !

!Color methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
    Smalltalk isSmalltalkX ifTrue:[
        aStream nextPutAll: '(Color r: ', (self red / 100) printString, 
                                  ' g: ', (self green / 100) printString,
                                  ' b: ', (self blue / 100) printString,
                                  ')'.
        ^ self.
    ].

    aStream nextPutAll: self storeString
! !

!Color class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!Date methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self sixxContentString
! !

!Date methodsFor:'private-sixx'!

sixxContentString
	"Squeak Specific"
	^self printString
! !

!Date class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!Dictionary methodsFor:'printing-six'!

sixxContentOn: aStream indent: level context: dictionary
	aStream cr.
	self
		associationsDo: [:each | each
				sixxOn: aStream
				indent: level + 1
				context: dictionary].
	aStream tab: level.
! !

!FloatArray methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: '#( '.
	self do:[:each | aStream nextPutAll: each printString; space].
	aStream nextPutAll: ')'.
! !

!FloatArray class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	| col |
	col := Array readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement)).
	^ col asFloatArray
! !

!IntegerArray methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: '#( '.
	self do:[:each | aStream nextPutAll: each printString; space].
	aStream nextPutAll: ')'.
! !

!IntegerArray class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	| col |
	col := Array readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement)).
	^ col asIntegerArray
! !

!Interval methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: self printString
! !

!Interval class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!Metaclass methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ aClass isMeta
		ifTrue: [aClass soleInstance]
		ifFalse: [aClass]
! !

!Number methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self asString
	
! !

!Number class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^ (SixxXmlUtil characterDataFrom: sixxElement) asNumber
	
! !

!Object methodsFor:'initializing-sixx'!

initializeFromSixxElement: sixxElement context: aDictionary 
	(SixxXmlUtil elementsFrom: sixxElement)
		do: [:elem | | varName | (SixxXmlUtil isTextFrom: elem)
				ifFalse: [varName := self class nameFromSixxElement: elem.
					varName notNil
						ifTrue: [| childInst | 
							childInst := self class fromSixxElement: elem context: aDictionary.
							self sixxInstVarNamed: varName put: childInst]]].
	self class isVariable
		ifTrue: [| idx nonameElems  | 
			idx := 1.
			nonameElems := (SixxXmlUtil elementsFrom: sixxElement)
						select: [:each | (self class nameFromSixxElement: each) isNil].
			nonameElems isEmpty
				ifFalse: [(SixxXmlUtil elementsFrom: nonameElems first)
						do: [:elem | | childInst | (SixxXmlUtil isTextFrom: elem)
								ifFalse: [childInst := self class fromSixxElement: elem context: aDictionary.
									self at: idx put: childInst.
									idx := idx + 1]]]]
! !

!Object methodsFor:'private-sixx'!

sixxAllContentsOn: aStream indent: level context: dictionary
	self sixxInstVarsOn: aStream indent: level context: dictionary.
	self sixxElementsOn: aStream indent: level context: dictionary.
! !

!Object methodsFor:'private-sixx'!

sixxChildrenContentsFixedOn: aStream indent: level context: dictionary
	self sixxInstVarsOn: aStream indent: level context: dictionary
! !

!Object methodsFor:'private-sixx'!

sixxChildrenContentsOn: aStream indent: level context: dictionary
	self class isVariable
		ifTrue: [self sixxChildrenContentsVariableOn: aStream indent: level context: dictionary]
		ifFalse: 
			[self sixxChildrenContentsFixedOn: aStream indent: level context: dictionary ].
! !

!Object methodsFor:'private-sixx'!

sixxChildrenContentsVariableOn: aStream indent: level context: dictionary
	self class isBits 
		ifTrue: 
			[aStream nextPutAll: '#( '.
			self do: 
					[:each | 
					aStream
						nextPutAll: each asString;
						space].
			aStream nextPutAll: ')']
		ifFalse: 
			[self 
				sixxAllContentsOn: aStream
				indent: level
				context: dictionary]
! !

!Object methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	"override if you like"
	self sixxChildrenContentsOn: aStream indent: level + 1 context: dictionary.
	aStream tab: level.
! !

!Object methodsFor:'private-sixx'!

sixxElementsOn: aStream indent: level context: dictionary 
	"override if you like"
	| size array |
	size := self basicSize.
	size = 0
				ifTrue: [^ self].
	array := Array new: size.
	1
		to: size
		do: [:i | 
			| element | 
			element := self basicAt: i.
			array at: i put: element].
	array
		sixxOn: aStream
		indent: level
		context: dictionary
! !

!Object methodsFor:'private-sixx'!

sixxEnv
	^SixxPortableUtil environmentQualifierOf: self class
! !

!Object methodsFor:'private-sixx'!

sixxEnvAttribString
	^'sixx.env="' , self sixxEnv , '" '
! !

!Object methodsFor:'private-sixx'!

sixxIdAttribString: idString
	^'sixx.id="', idString asString, '" '
! !

!Object methodsFor:'initializing-sixx'!

sixxInstVarNamed: instVarName put: aValue 
	"override if you like"
	self
		instVarAt: (self class allInstVarNames indexOf: instVarName asString)
		put: aValue
! !

!Object methodsFor:'private-sixx'!

sixxInstVarsOn: aStream indent: level context: dictionary
	| instVars ind |
	aStream cr.
	instVars := self class allInstVarNames.
	ind := 0.
	instVars do: 
			[:nm | 
			ind := ind + 1.
			(self instVarAt: ind) 
				sixxOn: aStream
				name: nm
				indent: level
				context: dictionary]
! !

!Object methodsFor:'private-sixx'!

sixxNameAttribString: idString
	^'sixx.name="', idString asString, '" '
! !

!Object methodsFor:'printing-sixx'!

sixxOn: aStream 
	self sixxOn: aStream indent: 0 context: IdentityDictionary new
! !

!Object methodsFor:'printing-sixx'!

sixxOn: aStream indent: level context: dictionary
	^self sixxOn: aStream name: nil indent: level context: dictionary
! !

!Object methodsFor:'printing-sixx'!

sixxOn: aStream name: nameString indent: level context: dictionary 
	aStream tab: level.
	(dictionary includesKey: self)
		ifTrue: [aStream
				nextPutAll: (self
						sixxRefTagStartString: (dictionary at: self) name: nameString)]
		ifFalse: [| idStr | 
			idStr := dictionary size asString.			
			dictionary at: self put: idStr.
			aStream
				nextPutAll: (self sixxTagStartString: idStr name: nameString).
			self
				sixxContentOn: aStream
				indent: level
				context: dictionary].
	aStream nextPutAll: self sixxTagEndString.
	aStream cr
! !

!Object methodsFor:'private-sixx'!

sixxRefTagStartString: idString
	| wStr |
	wStr := (String new: 16) writeStream.
	wStr nextPutAll: '<sixx.object sixx.idref="'.
	idString isNil
		ifFalse: [wStr
				nextPutAll: idString].
	wStr nextPutAll: '" >'.
	^ wStr contents
! !

!Object methodsFor:'private-sixx'!

sixxRefTagStartString: idString name: nameString 
	| wStr |
	wStr := (String new: 16) writeStream.
	wStr nextPutAll: '<sixx.object sixx.idref="'.
	wStr nextPutAll: idString.
	wStr nextPutAll: '" '.
	nameString notNil
		ifTrue: [wStr nextPutAll: 'sixx.name="'.
			wStr nextPutAll: nameString.
			wStr nextPutAll: '" '].
	wStr nextPutAll: '>'.
	^ wStr contents
! !

!Object methodsFor:'printing-sixx'!

sixxString
	| wStr |
	wStr := (String new: 128) writeStream.
	self sixxOn: wStr.
	^wStr contents
! !

!Object methodsFor:'private-sixx'!

sixxTagEndString
	| wStr |
	wStr := (String new: 16) writeStream.
	wStr nextPutAll: '</sixx.object>'.
	^wStr contents
	
! !

!Object methodsFor:'private-sixx'!

sixxTagStartString: idString name: nameString 
	| wStr |
	wStr := (String new: 16) writeStream.
	wStr nextPutAll: '<sixx.object '.
	idString isNil
		ifFalse: [wStr
				nextPutAll: (self sixxIdAttribString: idString)].
	nameString isNil
		ifFalse: [wStr
				nextPutAll: (self sixxNameAttribString: nameString)].
	wStr nextPutAll: self sixxTypeAttribString.
	SixxSetting useEnvironment ifTrue: [wStr nextPutAll: self sixxEnvAttribString.].
	wStr nextPutAll: '>'.
	^ wStr contents
! !

!Object methodsFor:'private-sixx'!

sixxType
	^self class name
! !

!Object methodsFor:'private-sixx'!

sixxTypeAttribString
	^'sixx.type="', self sixxType, '" '
! !

!Point methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: self storeString
! !

!Point class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!RunArray methodsFor:'initializing-sixx'!

initializeFromSixxElement: sixxElement context: aDictionary 
        |runs values|
        (SixxXmlUtil elementsFrom: sixxElement)
                do: [:elem | | varName | (SixxXmlUtil isTextFrom: elem)
                                ifFalse: [varName := self class nameFromSixxElement: elem.
                                        varName notNil
                                                ifTrue: [| childInst | 
                                                        childInst := self class fromSixxElement: elem context: aDictionary.
                                                        varName = 'runs' ifTrue:[ runs := childInst ].
                                                        varName = 'values' ifTrue:[ values := childInst ].
                                                        " self sixxInstVarNamed: varName put: childInst "
                                                        ]]].
        self setRuns:runs setValues:values.
! !

!RunArray methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream cr.
	self runs
		sixxOn: aStream
		name: 'runs'
		indent: level + 1
		context: dictionary.
	self values
		sixxOn: aStream
		name: 'values'
		indent: level + 1
		context: dictionary.
	aStream tab: level
! !

!RunArray class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	"squeak specific"
	
	^aClass basicNew
! !

!String methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	SixxXmlUtil writeXmlText: self on: aStream
! !

!String class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^ SixxXmlUtil characterDataFrom: sixxElement
! !

!Symbol class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^(super createInstanceOf: aClass withSixxElement: sixxElement) asSymbol
! !

!Text methodsFor:'initializing-sixx'!

initializeFromSixxElement: sixxElement context: aDictionary 
	(SixxXmlUtil elementsFrom: sixxElement)
		do: [:elem | | varName | (SixxXmlUtil isTextFrom: elem)
				ifFalse: [varName := self class nameFromSixxElement: elem.
					varName notNil
						ifTrue: [| childInst | 
							childInst := self class fromSixxElement: elem context: aDictionary.
							self sixxInstVarNamed: varName put: childInst ]]]
! !

!Text methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	self sixxChildrenContentsOn: aStream indent: level + 1 context: dictionary.
	aStream tab: level.
! !

!Time methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self sixxContentString
! !

!Time methodsFor:'private-sixx'!

sixxContentString
	"Squeak Specific"
	^self printString
! !

!Time class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!UndefinedObject methodsFor:'private-sixx'!

asSixxKey
	"For some Smalltalks which do not support nil element in IdentityDictionary"
	^#'SIXX-UndefinedObject(98ea8c30-2bf8-4d23-9da2-ad1ee0e29eff)'
! !

!UndefinedObject methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary
	aStream nextPutAll: self storeString
! !

!UndefinedObject methodsFor:'printing-sixx'!

sixxOn: aStream name: nameString indent: level context: dictionary 
	| key |
	key := self asSixxKey.
	aStream tab: level.
	(dictionary includesKey: key )
		ifTrue: [aStream
				nextPutAll: (self
						sixxRefTagStartString: (dictionary at: key) name: nameString)]
		ifFalse: [| idStr | 
			idStr := dictionary size asString.			
			dictionary at: key put: idStr.
			aStream
				nextPutAll: (self sixxTagStartString: idStr name: nameString).
			self
				sixxContentOn: aStream
				indent: level
				context: dictionary].
	aStream nextPutAll: self sixxTagEndString.
	aStream cr
! !

!UndefinedObject class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement
	^ self readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement))
! !

!WordArray methodsFor:'printing-sixx'!

sixxContentOn: aStream indent: level context: dictionary 
	aStream nextPutAll: '#( '.
	self do:[:each | aStream nextPutAll: each printString; space].
	aStream nextPutAll: ')'.
! !

!WordArray class methodsFor:'instance creation-sixx'!

createInstanceOf: aClass withSixxElement: sixxElement 
	| col |
	col := Array readFrom: (ReadStream on: (SixxXmlUtil characterDataFrom: sixxElement)).
	^ col asWordArray
! !

