"{ Package: 'stx:goodies/xml/sixx/tests' }"

SixxAbstractTestCase subclass:#SixxGeneratingTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Test'
!


!SixxGeneratingTestCase methodsFor:'tests'!

testSixxStringForArray
        | arr expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Array" >
        <sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
        <sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
        <sixx.object sixx.id="3" sixx.type="SmallInteger" >30</sixx.object>
</sixx.object>
'.
        Smalltalk isSmalltalkX ifTrue:[
            expectedStr := expectedStr asStringCollection withTabs asString
        ].

        arr := Array new: 3.
        arr at: 1 put: 10.
        arr at: 2 put: 20.
        arr at: 3 put: 30.
        self
                should: [arr sixxString = expectedStr]
!

testSixxStringForBag
	| bag result |
	bag := Bag new.
	bag add: 10.
	bag add: 20.
	bag add: 30.
	result := Object readSixxFrom: bag sixxString.
	self
		should: [result size = 3].
	self
		should: [result includes: 10].
	self
		should: [result includes: 20].
	self
		should: [result includes: 30]
!

testSixxStringForBoolean
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="True" >true</sixx.object>
'.
        self
                should: [true sixxString = expectedStr]
!

testSixxStringForByteArray
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="ByteArray" >#( 10 20 30 )</sixx.object>
'.
        self
                should: [(ByteArray with: 10 with: 20 with: 30) sixxString = expectedStr]
!

testSixxStringForCharacter
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Character" >$A</sixx.object>
'.
        self
                should: [$A sixxString = expectedStr]
!

testSixxStringForClass
	| expectedStr |
	expectedStr := '<sixx.object sixx.id="0" sixx.type="SixxMockClassForUnitTest class" >
	<sixx.object sixx.id="1" sixx.name="instVar1" sixx.type="String" >value1</sixx.object>
	<sixx.object sixx.id="2" sixx.name="instVar2" sixx.type="String" >value2</sixx.object>
	<sixx.object sixx.id="3" sixx.name="instVar3" sixx.type="String" >value3</sixx.object>
</sixx.object>
'.
	SixxMockClassForUnitTest instVar1: 'value1'.
	SixxMockClassForUnitTest instVar2: 'value2'.
	SixxMockClassForUnitTest instVar3: 'value3'.
	self
		should: [SixxMockClassForUnitTest sixxString = expectedStr]
!

testSixxStringForColor
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Color" >(Color r: 0.0 g: 0.0 b: 1.0)</sixx.object>
'.
        self
                should: [Color blue sixxString = expectedStr]
!

testSixxStringForCompositeDictionary
	| dic childDic expected |
	expected := '<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >parent1</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="String" >sample string.</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >parent4</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="Dictionary" >
			<sixx.object sixx.id="7" sixx.type="Association" >
				<sixx.object sixx.id="8" sixx.name="key" sixx.type="Symbol" >child1</sixx.object>
				<sixx.object sixx.id="9" sixx.name="value" sixx.type="String" >abcde</sixx.object>
			</sixx.object>
		</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="10" sixx.type="Association" >
		<sixx.object sixx.id="11" sixx.name="key" sixx.type="Symbol" >parent3</sixx.object>
		<sixx.object sixx.id="12" sixx.name="value" sixx.type="Array" >
			<sixx.object sixx.id="13" sixx.type="SmallInteger" >10</sixx.object>
			<sixx.object sixx.id="14" sixx.type="SmallInteger" >20</sixx.object>
			<sixx.object sixx.id="15" sixx.type="SmallInteger" >30</sixx.object>
		</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="16" sixx.type="Association" >
		<sixx.object sixx.id="17" sixx.name="key" sixx.type="Symbol" >parent2</sixx.object>
		<sixx.object sixx.id="18" sixx.name="value" sixx.type="SmallInteger" >100</sixx.object>
	</sixx.object>
</sixx.object>
'.
	childDic := Dictionary new.
	childDic at: #child1 put: 'abcde'.
	dic := Dictionary new.
	dic at: #parent1 put: 'sample string.'.
	dic at: #parent2 put: 100.
	dic at: #parent3 put: #(10 20 30 ).
	dic at: #parent4 put: childDic.
	self
		should: [(Object readSixxFrom: dic sixxString) = (Object readSixxFrom: expected)]
!

testSixxStringForDate
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Date" >14 May 2002</sixx.object>
'.
        self
                should: [(Date
                                newDay: 14
                                month: 5
                                year: 2002) sixxString = expectedStr]
!

testSixxStringForDictionary
	| dic expected |
	expected := '<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >key2</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="String" >value2</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >key1</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="String" >value1</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="7" sixx.type="Association" >
		<sixx.object sixx.id="8" sixx.name="key" sixx.type="Symbol" >key3</sixx.object>
		<sixx.object sixx.id="9" sixx.name="value" sixx.type="String" >value3</sixx.object>
	</sixx.object>
</sixx.object>
'.
	dic := Dictionary new.
	dic at: #key1 put: 'value1'.
	dic at: #key2 put: 'value2'.
	dic at: #key3 put: 'value3'.
	self
		should: [(Object readSixxFrom: dic sixxString) = (Object readSixxFrom: expected)]
!

testSixxStringForInterval
	| interval expectedStr |
	expectedStr := '<sixx.object sixx.id="0" sixx.type="Interval" >(1 to: 10 by: 3)</sixx.object>
'.
	interval := 1 to: 10 by: 3.
	self should: [interval sixxString = expectedStr]
!

testSixxStringForNumber
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Fraction" >(3/4)</sixx.object>
'.
        self
                should: [(3 / 4) sixxString = expectedStr]
!

testSixxStringForOrdredCollection
	| expectedStr col |
	expectedStr := '<sixx.object sixx.id="0" sixx.type="OrderedCollection" >
	<sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
	<sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
	<sixx.object sixx.id="3" sixx.type="SmallInteger" >30</sixx.object>
</sixx.object>
'.
	col := OrderedCollection new.
	col add: 10.
	col add: 20.
	col add: 30.
	self
		should: [col sixxString = expectedStr]
!

testSixxStringForRunArray
	| arr expectedStr |
	expectedStr := '<sixx.object sixx.id="0" sixx.type="RunArray" >
	<sixx.object sixx.id="1" sixx.name="runs" sixx.type="Array" >
		<sixx.object sixx.id="2" sixx.type="SmallInteger" >1</sixx.object>
		<sixx.object sixx.id="3" sixx.type="SmallInteger" >2</sixx.object>
		<sixx.object sixx.idref="2" ></sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.name="values" sixx.type="Array" >
		<sixx.object sixx.idref="2" ></sixx.object>
		<sixx.object sixx.idref="3" ></sixx.object>
		<sixx.object sixx.id="5" sixx.type="SmallInteger" >3</sixx.object>
	</sixx.object>
</sixx.object>
'.

	arr := RunArray runs: #(1 2 1) values: #(1 2 3).
	self should: [arr sixxString = expectedStr]
!

testSixxStringForSimpleObject
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Object" >
</sixx.object>
'.
        self
                should: [Object new sixxString = expectedStr]
!

testSixxStringForString
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="String" >Hi&#44; I&apos;m String-object</sixx.object>
'.
        self
                should: ['Hi, I''m String-object' sixxString = expectedStr]
!

testSixxStringForStringUsingCDATA
	| originalSetting expectedStr |
	[originalSetting := SixxSetting useCDATA.
	SixxSetting useCDATA: true.
	expectedStr := '<sixx.object sixx.id="0" sixx.type="String" ><!![CDATA[Hi, I''m String-object.
How are you?
]]></sixx.object>
'.
	self
		should: ['Hi, I''m String-object.
How are you?
' sixxString = expectedStr]]
		ensure: [SixxSetting useCDATA: originalSetting]
!

testSixxStringForTime
        | expectedStr |
        expectedStr := '<sixx.object sixx.id="0" sixx.type="Time" >' 
                                , (Time readFrom: '1:30:59 pm' readStream) sixxContentString , '</sixx.object>
'.
        self 
                should: [(Time readFrom: '1:30:59 pm' readStream) sixxString = expectedStr]
!

testSixxStringForUndefinedObject
	| expectedStr |
	expectedStr := '<sixx.object sixx.id="0" sixx.type="UndefinedObject" >nil</sixx.object>
'.
	self
		should: [nil sixxString = expectedStr]
!

testSixxStringForVariableObject
	| expectedStr variableObj |
	expectedStr :='<sixx.object sixx.id="0" sixx.type="SixxMockBinaryData" >
	<sixx.object sixx.id="1" sixx.name="author" sixx.type="String" >MU</sixx.object>
	<sixx.object sixx.id="2" sixx.name="version" sixx.type="String" >0.1</sixx.object>
	<sixx.object sixx.id="3" sixx.type="Array" >
		<sixx.object sixx.id="4" sixx.type="SmallInteger" >1</sixx.object>
		<sixx.object sixx.id="5" sixx.type="SmallInteger" >2</sixx.object>
		<sixx.object sixx.id="6" sixx.type="SmallInteger" >3</sixx.object>
	</sixx.object>
</sixx.object>
'.
	
	variableObj := SixxMockBinaryData author: 'MU' version: '0.1' data: #(1 2 3).
	self
		should: [variableObj sixxString = expectedStr].
	
	
!

testSixxStringForVariableObjectNoData
	| expectedStr variableObj |
	
	expectedStr := '<sixx.object sixx.id="0" sixx.type="SixxMockBinaryData" >
	<sixx.object sixx.id="1" sixx.name="author" sixx.type="String" >MU</sixx.object>
	<sixx.object sixx.id="2" sixx.name="version" sixx.type="String" >0.1</sixx.object>
</sixx.object>
'.

	variableObj := SixxMockBinaryData author: 'MU' version: '0.1' data: #().
	self
		should: [variableObj sixxString = expectedStr].
! !

!SixxGeneratingTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/tests/SixxGeneratingTestCase.st,v 1.2 2003/09/29 10:13:00 cg Exp $'
! !
