"{ Package: 'stx:goodies/xml/sixx/tests' }"

SixxAbstractTestCase subclass:#SixxLoadingTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Test'
!


!SixxLoadingTestCase methodsFor:'tests'!

testReadSixxForArray
	| sixxString arr |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Array" >
	<sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
	<sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
	<sixx.object sixx.id="3" sixx.type="SmallInteger" >30</sixx.object>
</sixx.object>
'.
	arr := Object readSixxFrom: sixxString.
	self
		should: [arr size = 3].
	self
		should: [(arr at: 1)
				= 10].
	self
		should: [(arr at: 2)
				= 20].
	self
		should: [(arr at: 3)
				= 30]
!

testReadSixxForBag
	| sixxString bag |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Bag" >
	<sixx.object sixx.id="1" sixx.type="String" >abcd</sixx.object>
	<sixx.object sixx.id="2" sixx.type="SmallInteger" >10</sixx.object>
	<sixx.object sixx.idref="2" ></sixx.object>
	<sixx.object sixx.id="3" sixx.type="Array" >
		<sixx.object sixx.idref="2" ></sixx.object>
		<sixx.object sixx.id="4" sixx.type="SmallInteger" >20</sixx.object>
	</sixx.object>
</sixx.object>
'.
	bag := Object readSixxFrom: sixxString.
	self
		should: [bag size = 4].
	self
		should: [(bag occurrencesOf: 10)
				= 2].
	self
		should: [(bag occurrencesOf: 'abcd')
				= 1].
	self
		should: [(bag occurrencesOf: #(10 20 ))
				= 1]
!

testReadSixxForBoolean
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="True" >true</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= true]
!

testReadSixxForByteArray
	| sixxString byteArr |
	sixxString := '<sixx.object sixx.id="0" sixx.type="ByteArray" >#( 10 20 30 )</sixx.object>
'.
	byteArr := Object readSixxFrom: sixxString.
	self
		should: [byteArr size = 3].
	self
		should: [(byteArr at: 1)
				= 10].
	self
		should: [(byteArr at: 2)
				= 20].
	self
		should: [(byteArr at: 3)
				= 30]
!

testReadSixxForCharacter
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Character" >$A</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= $A]
!

testReadSixxForClass
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="SixxMockClassForUnitTest class" >
	<sixx.object sixx.id="1" sixx.name="instVar1" sixx.type="SmallInteger" >10</sixx.object>
	<sixx.object sixx.id="2" sixx.name="instVar2" sixx.type="SmallInteger" >20</sixx.object>
	<sixx.object sixx.id="3" sixx.name="instVar3" sixx.type="SmallInteger" >30</sixx.object>
</sixx.object>
'.
	SixxMockClassForUnitTest instVar1: 0;
		 instVar2: 0;
		 instVar3: 0.
	self
		should: [SixxMockClassForUnitTest instVar1 = 0].
	self
		should: [SixxMockClassForUnitTest instVar2 = 0].
	self
		should: [SixxMockClassForUnitTest instVar3 = 0].
	Object readSixxFrom: sixxString.
	self
		should: [SixxMockClassForUnitTest instVar1 = 10].
	self
		should: [SixxMockClassForUnitTest instVar2 = 20].
	self
		should: [SixxMockClassForUnitTest instVar3 = 30]
!

testReadSixxForColor
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Color" >(Color r: 0.0 g: 0.0 b: 1.0)</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= Color blue]
!

testReadSixxForCompositeObject
	| sixxString slipObj headerObj |
	sixxString := '<sixx.object sixx.id="0" sixx.type="SixxMockEnvelope" >
	<sixx.object sixx.id="1" sixx.name="header" sixx.type="SixxMockHeader" >
		<sixx.object sixx.id="2" sixx.name="id" sixx.type="String" >99</sixx.object>
	</sixx.object>
</sixx.object>
'.
	slipObj := Object readSixxFrom: sixxString.
	headerObj := slipObj header.
	self
		should: [headerObj id = '99']
!

testReadSixxForCorruptRecursiveDictionary
	| originalRecoveryRead |
	[ |  sixxString |
	originalRecoveryRead := SixxSetting useRecoveryRead.
	SixxSetting useRecoveryRead: false.
	sixxString := '<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >key2</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="SmallInteger" >100</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >key1</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="String" >value1</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="7" sixx.type="Association" >
		<sixx.object six
'.
	self should:[ Object readSixxFrom: sixxString] raise: SixxXmlParseError.
	
	] ensure: [SixxSetting useRecoveryRead: originalRecoveryRead]
!

testReadSixxForDate
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Date" >14 May 2002</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= (Date
						newDay: 14
						month: 5
						year: 2002)]
!

testReadSixxForDictionary
	| sixxString dic |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >key2</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="SmallInteger" >100</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >key1</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="String" >value1</sixx.object>
	</sixx.object>
</sixx.object>
'.
	dic := Object readSixxFrom: sixxString.
	self
		should: [dic size = 2].
	self
		should: [(dic at: #key1)
				= 'value1'].
	self
		should: [(dic at: #key2)
				= 100]
!

testReadSixxForInterval
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Interval" >(1 to: 10 by: 3)</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= (1 to: 10 by: 3)]
!

testReadSixxForRecursiveArray
	| sixxString arr |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Array" >
	<sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
	<sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
	<sixx.object sixx.idref="0" ></sixx.object>
</sixx.object>
'.
	arr := Object readSixxFrom: sixxString.
	self
		should: [arr size = 3].
	self
		should: [(arr at: 1)
				= 10].
	self
		should: [(arr at: 2)
				= 20].
	self
		should: [(arr at: 3)
				== arr]
!

testReadSixxForRecursiveDictionary
	| sixxString dic |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >key2</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="SmallInteger" >100</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >key1</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="String" >value1</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="7" sixx.type="Association" >
		<sixx.object sixx.id="8" sixx.name="key" sixx.type="Symbol" >key3</sixx.object>
		<sixx.object sixx.idref="0" sixx.name="value" ></sixx.object>
	</sixx.object>
</sixx.object>
'.
	dic := Object readSixxFrom: sixxString.
	self
		should: [dic size = 3].
	self
		should: [(dic at: #key1)
				= 'value1'].
	self
		should: [(dic at: #key2)
				= 100].
	self
		should: [(dic at: #key3)
				== dic]
!

testReadSixxForRunArray
	| sixxString runArray |
	sixxString := '<sixx.object sixx.id="0" sixx.type="RunArray" >
	<sixx.object sixx.id="1" sixx.name="runs" sixx.type="Array" >
		<sixx.object sixx.id="2" sixx.type="SmallInteger" >1</sixx.object>
		<sixx.object sixx.id="3" sixx.type="SmallInteger" >2</sixx.object>
		<sixx.object sixx.idref="2" ></sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.name="values" sixx.type="Array" >
		<sixx.object sixx.idref="2" ></sixx.object>
		<sixx.object sixx.idref="3" ></sixx.object>
		<sixx.object sixx.id="5" sixx.type="SmallInteger" >3</sixx.object>
	</sixx.object>
</sixx.object>
'.
	runArray := Object readSixxFrom: sixxString.

	self should: [runArray runs = #(1 2 1)].
	self should: [runArray values = #(1 2 3)]
	
!

testReadSixxForSimpleObject
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="SixxMockHeader" >
	<sixx.object sixx.id="1" sixx.name="id" sixx.type="String" >1000</sixx.object>
</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString) id = '1000']
!

testReadSixxForStringUsingCDATA
	| originalSetting sixxString string |
	[originalSetting := SixxSetting useCDATA.
	SixxSetting useCDATA: true.
	sixxString := '<sixx.object sixx.id="0" sixx.type="String" ><!![CDATA[Hi, I''m String-object.
How are you?
]]></sixx.object>
'.
	string := Object readSixxFrom: sixxString.
	self
		should: [string = 'Hi, I''m String-object.
How are you?
']]
		ensure: [SixxSetting useCDATA: originalSetting]
!

testReadSixxForTime
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="Time" >4:10:32 pm</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= (Time
						readFrom: (ReadStream on: '4:10:32 pm'))]
!

testReadSixxForUndefinedObject
	| sixxString |
	sixxString := '<sixx.object sixx.id="0" sixx.type="UndefinedObject" >nil</sixx.object>
'.
	self
		should: [(Object readSixxFrom: sixxString)
				= nil]
!

testReadSixxForVariableObject
	| sixxString variableObj |
	sixxString := '<sixx.object sixx.id="0" sixx.type="SixxMockBinaryData" >
	<sixx.object sixx.id="1" sixx.name="author" sixx.type="String" >MU</sixx.object>
	<sixx.object sixx.id="2" sixx.name="version" sixx.type="String" >0.1</sixx.object>
	<sixx.object sixx.id="3" sixx.type="Array" >
		<sixx.object sixx.id="4" sixx.type="SmallInteger" >1</sixx.object>
		<sixx.object sixx.id="5" sixx.type="SmallInteger" >2</sixx.object>
		<sixx.object sixx.id="6" sixx.type="SmallInteger" >3</sixx.object>
	</sixx.object>
</sixx.object>
'.
	
	variableObj := Object readSixxFrom: sixxString.
	self should: [ variableObj author = 'MU'].
	self should: [ variableObj version = '0.1'].
	self should: [ variableObj size = 3].
	self should: [ (variableObj at: 1) = 1].
	self should: [ (variableObj at: 2) = 2].
	self should: [ (variableObj at: 3) = 3].
	
	
!

testReadSixxForVariableObjectNoData
	
	| sixxString variableObj |
	sixxString := '<sixx.object sixx.id="0" sixx.type="SixxMockBinaryData" >
	<sixx.object sixx.id="1" sixx.name="author" sixx.type="String" >MU</sixx.object>
	<sixx.object sixx.id="2" sixx.name="version" sixx.type="String" >0.1</sixx.object>
</sixx.object>
'.
	variableObj := Object readSixxFrom: sixxString.
	self should: [ variableObj author = 'MU'].
	self should: [ variableObj version = '0.1'].
	self should: [ variableObj size = 0].

	
	
!

testReadSixxWithClassDoesNotExist
        | str obj catched |
        str := '<sixx.object sixx.id="0" sixx.type="doesNotExistString" >abcde</sixx.object>
'.

        obj := Object readSixxFrom: str.
        self should: [obj = nil].
        catched := false.
        self should: [[Object readSixxFrom: str]
                                on: SixxClassNotFound
                                do: [:ex | catched := true].
                        catched]
! !

!SixxLoadingTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/tests/SixxLoadingTestCase.st,v 1.2 2003/10/08 16:22:35 cg Exp $'
! !
