"{ Package: 'stx:goodies/xml/sixx/tests' }"

SixxAbstractTestCase subclass:#SixxReadStreamTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Test'
!


!SixxReadStreamTestCase methodsFor:'tests'!

testAtEnd
        | str stream |
        str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
        <sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
        <sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
        stream := SixxReadStream on: str readStream.
        self    should: [stream next = 'a'].
        self    should: [stream atEnd not].
        self    should: [stream next = 'b'].
        self    should: [stream atEnd not].
        self    should: [stream next = 'c'].
        self    should: [stream atEnd].
        self    should: [stream next = nil].
        self    should: [stream atEnd].
!

testCachedObjects
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	stream next; next.
	self
		should: [(stream cachedObjects at: 1)
				= 'a'].
	self
		should: [(stream cachedObjects at: 2)
				= 'b'].
	self
		shouldnt: [stream cachedObjects includesKey: 3].
	stream next.
	self
		should: [(stream cachedObjects at: 3)
				= 'c']
!

testNext
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [stream next = 'a'].
	self
		should: [stream next = 'b'].
	self
		should: [stream next = 'c'].
	self
		should: [stream next = nil]
!

testNextMany
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [(stream next: 2)
				= (OrderedCollection with: 'a' with: 'b')].
	stream reset.
	self
		should: [(stream next: 5)
				= (OrderedCollection
						with: 'a'
						with: 'b'
						with: 'c')]
!

testNextWithIdred
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="Character" >$a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="Character" >$u</sixx.object>
	<sixx.object sixx.idref="0" ></sixx.object>
	<sixx.object sixx.idref="1" ></sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [stream next = $a].
	self
		should: [stream next = $u].
	self
		should: [stream next = $a].
	self
		should: [stream next = $u]
!

testPeek
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [stream position = 0].
	self
		should: [stream peek = 'a'].
	self
		should: [stream position = 0].
	self
		should: [stream next = 'a']
!

testPosition
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	stream next: 3.
	self
		should: [stream position = 3].
	stream position: 1.
	self
		should: [stream next = 'b']
!

testReadEmptyStream
	| stream empStream |
	empStream := '' readStream.
	self
		should: [empStream size = 0].
	stream :=  SixxReadStream on: empStream.
	
	self
		should: [stream position = 0].
	self
		should: [stream atEnd].
	self
		should: [stream next isNil].
	
!

testReadNextContainsIdref
	| str stream firstObj secondObj thirdObj |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="Array" >
		<sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
		<sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
		<sixx.object sixx.id="3" sixx.type="SmallInteger" >30</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="OrderedCollection" >
		<sixx.object sixx.idref="0" ></sixx.object>
	</sixx.object>
	<sixx.object sixx.id="5" sixx.type="Array" >
		<sixx.object sixx.idref="1" ></sixx.object>
		<sixx.object sixx.idref="2" ></sixx.object>
		<sixx.object sixx.idref="3" ></sixx.object>
	</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [stream size = 3].
	firstObj := stream next.
	secondObj := stream next.
	thirdObj := stream next.
	self
		should: [firstObj = #(10 20 30 )].
	self
		should: [(secondObj at: 1)
				== firstObj].
	self
		should: [thirdObj = #(10 20 30 )
				and: [thirdObj ~~ firstObj]]
!

testRecoveryRead
	| originalRecoveryRead   |
	[ |  str stream nextObject |
	originalRecoveryRead := SixxSetting useRecoveryRead.
	SixxSetting useRecoveryRead: true.
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="Dictionary" >
	<sixx.object sixx.id="1" sixx.type="Association" >
		<sixx.object sixx.id="2" sixx.name="key" sixx.type="Symbol" >key2</sixx.object>
		<sixx.object sixx.id="3" sixx.name="value" sixx.type="SmallInteger" >100</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="4" sixx.type="Association" >
		<sixx.object sixx.id="5" sixx.name="key" sixx.type="Symbol" >key1</sixx.object>
		<sixx.object sixx.id="6" sixx.name="value" sixx.type="String" >value1</sixx.object>
	</sixx.object>
	<sixx.object sixx.id="7" sixx.type="Association" >
		<sixx.object six
'.
	stream := SixxReadStream on: str readStream.
	self shouldnt: [nextObject := stream next] raise: SixxXmlParseError.
	self
		should: [nextObject class = Dictionary].
	self
		should: [(nextObject at: #key1)
				= 'value1'].
	self
		should: [(nextObject at: #key2)
				= 100].
	self
		shouldnt: [(nextObject includesKey: #key3)].
	
	] ensure: [SixxSetting useRecoveryRead: originalRecoveryRead]
!

testReset
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	[stream atEnd]
		whileFalse: [stream next].
	self
		should: [stream next = nil].
	stream reset.
	self
		should: [stream position = 0]
!

testSize
	| str stream |
	str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
	<sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
	<sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
	<sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
	stream := SixxReadStream on: str readStream.
	self
		should: [stream size = 3]
!

testSkip
        | str stream |
        str := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="String" >a</sixx.object>
        <sixx.object sixx.id="1" sixx.type="String" >b</sixx.object>
        <sixx.object sixx.id="2" sixx.type="String" >c</sixx.object>
</sixx>'.
        stream := SixxReadStream on: str readStream.
        self     should: [stream position = 0].
        stream skip: 1.
        self     should: [stream position = 1].
        stream next.
        self     should: [stream position = 2].
        stream skip: 1.
        self    should: [stream position = 3].
        stream skip: -1.
        self     should: [stream position = 2].

        "original: (test is bad)"
        false ifTrue:[
            stream skip: 2.
            self     should: [stream position = 3]
        ].

        "fixed:"
        self should:[ stream skip: 2 ] raise:Error.
        self should: [stream position = 2]
! !

!SixxReadStreamTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/tests/SixxReadStreamTestCase.st,v 1.3 2003/10/08 16:18:27 cg Exp $'
! !
