"{ Package: 'stx:goodies/xml/sixx/tests' }"

SixxAbstractTestCase subclass:#SixxWriteStreamTestCase
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Test'
!


!SixxWriteStreamTestCase methodsFor:'tests'!

testCloseTwice
        | st wst expected |
        expected := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="String" >abc</sixx.object>
</sixx>'.

        Smalltalk isSmalltalkX ifTrue:[
            expected := expected asStringCollection withTabs asStringWith:Character cr
        ].
        st := WriteStream on: String new.
        wst := SixxWriteStream onWithDeclaration: st.
        wst nextPut: 'abc'.
        wst close.
        wst close.
        self
                should: [st contents = expected]
!

testKeepStreamOpening
        | st wst expected |
        expected := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="String" >abc</sixx.object>
'.
        Smalltalk isSmalltalkX ifTrue:[
            expected := expected asStringCollection withTabs asString
        ].
        st := WriteStream on: String new.
        wst := SixxWriteStream onWithDeclaration: st.
        wst nextPut: 'abc'.
        self
                should: [st contents = expected]
!

testNeverNextPutAndThenClose
	| st wst expected |
	expected := ''.
	st := WriteStream on: String new.
	wst := SixxWriteStream on: st.
	wst close.
	self
		should: [st contents = expected]
!

testNextPut
        | st wst expected |
        expected := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="String" >abc</sixx.object>
</sixx>'.

        Smalltalk isSmalltalkX ifTrue:[
            expected := expected asStringCollection withTabs asStringWith:Character cr
        ].
        st := WriteStream on: String new.
        wst := SixxWriteStream onWithDeclaration: st.
        wst nextPut: 'abc'.
        wst close.
        self
                should: [st contents = expected]
!

testNextPutAll
        | st wst expected |
        expected := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="Character" >$a</sixx.object>
        <sixx.object sixx.id="1" sixx.type="Character" >$b</sixx.object>
        <sixx.object sixx.id="2" sixx.type="Character" >$c</sixx.object>
</sixx>'.
        Smalltalk isSmalltalkX ifTrue:[
            expected := expected asStringCollection withTabs asStringWith:Character cr
        ].
        st := WriteStream on: String new.
        wst := SixxWriteStream onWithDeclaration: st.
        wst nextPutAll: 'abc'.
        wst close.
        self
                should: [st contents = expected]
!

testWriteRecursiveObject
        | st wst expected arr col |
        expected := '<?xml version="1.0" encoding="utf-8"?>
<sixx sixx.version="1.0">
        <sixx.object sixx.id="0" sixx.type="Array" >
                <sixx.object sixx.id="1" sixx.type="SmallInteger" >10</sixx.object>
                <sixx.object sixx.id="2" sixx.type="SmallInteger" >20</sixx.object>
                <sixx.object sixx.id="3" sixx.type="SmallInteger" >30</sixx.object>
        </sixx.object>
        <sixx.object sixx.id="4" sixx.type="OrderedCollection" >
                <sixx.object sixx.idref="0" ></sixx.object>
        </sixx.object>
        <sixx.object sixx.id="5" sixx.type="Array" >
                <sixx.object sixx.idref="1" ></sixx.object>
                <sixx.object sixx.idref="2" ></sixx.object>
                <sixx.object sixx.idref="3" ></sixx.object>
        </sixx.object>
</sixx>'.
        Smalltalk isSmalltalkX ifTrue:[
            expected := expected asStringCollection withTabs asStringWith:Character cr
        ].

        arr := #(10 20 30 ).
        col := OrderedCollection with: arr.
        st := WriteStream on: String new.
        wst := SixxWriteStream onWithDeclaration: st.
        wst nextPut: arr.
        wst nextPut: col.
        wst nextPut: #(10 20 30 ) copy.
        wst close.
        self
                should: [st contents = expected]
! !

!SixxWriteStreamTestCase class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/tests/SixxWriteStreamTestCase.st,v 1.2 2003/09/29 10:13:07 cg Exp $'
! !
