"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/xml/stx' }"

XML::SAXDriver subclass:#XMLStandardDecoder
	instanceVariableNames:'idToObjectMapping object aspect version stack characterStream
		needWhitespace useSend'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-Presentation'
!

Object subclass:#AbstractConverter
	instanceVariableNames:'id storeSpecification elementClass'
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#Base64Converter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#CharacterConverter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#ContainedObject
	instanceVariableNames:'object'
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#NumberConverter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#StringConverter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

XMLStandardDecoder::AbstractConverter subclass:#SymbolConverter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLStandardDecoder
!

!XMLStandardDecoder class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    XMLStandardDecoder is used to decode the XML representation of objects encoded
    with XMLStandardCoder

    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]
        idToObjectMapping   <Collection> Mapping from object ids to objects
        object              <Object>     The decoded object
        aspect              <String|nil> The aspect used to encode
        version             <String|nil> The version provided by the encoder
        stack               <Stack>      Stack of nested objects       
        characterStream     <WriteStream>Stream to collect character data
        needWhitespace      <Bool>       true, when whitespace should be preserved for the
                                         current element
        useSend             <Bool>       If true, use #perform: insted of 
                                         #instvarNamed:put: to build objects


    [class variables:]

    [see also:]
        ObjectCoder
        XMLCoder
        XMLStandardCoder

"
!

examples
    "Look at examples in XMLStandardCoder"
! !

!XMLStandardDecoder class methodsFor:'instance creation'!

on:readStream
    "redefine to switch off validation"

    ^ super on:readStream beforeScanDo:[:parser| parser validate:false]
!

on:readStream beforeScanDo:aBlock
    "redefine to switch off validation"

    ^ super on:readStream beforeScanDo:[:parser| parser validate:false. 
                                                 aBlock value:parser builder driver]
! !

!XMLStandardDecoder methodsFor:'accessing'!

aspect
    "return the value of the instance variable 'aspect' (automatically generated)"

    ^ aspect
!

next
    "answer the next object from the stream.
     Notice: VALID xml files contain only a single object."

    ^ object
!

object
    ^ object
!

useSend

    ^ useSend
!

useSend:something
    "if something is true, the decoder uses a message send to 
     set the values of an object.
     If something is false, the instance variables are set directly"

    useSend := something.
!

version
    "return the value of the instance variable 'version' (automatically generated)"

    ^ version
! !

!XMLStandardDecoder methodsFor:'building'!

add:obj toParent:parent with:storeSpecifier
    "add an object to its parent object"

    |instVarOffset|

    storeSpecifier isNil ifTrue:[
        "neither index nor name, may be a set or dense collection"
        parent add:obj.
        ^ self.
    ].
    storeSpecifier isInteger ifTrue:[
        "explicit index"
        storeSpecifier > parent size ifTrue:[
            "/ must be done, because (OC new:sz) does not really
            "/ make the collection of size sz.
            parent grow:storeSpecifier
        ].
        "/ cannot take basicAt:put here; parent could be an OC.
        parent at:storeSpecifier put:obj.
        ^ self
    ].

    "named element"
    useSend == true ifTrue:[
        parent perform:(storeSpecifier , ':') asSymbol with:obj.
    ] ifFalse:[
        instVarOffset := parent class instVarOffsetOf:storeSpecifier.
        instVarOffset isNil ifTrue:[
            "/ mhmh - that instance variable seems to be gone;
            "/ what should we do here ?
            (XMLChangedInstanceLayoutException new
                    parent:parent instVarName:storeSpecifier valueToBeStored:obj;
                    errorString:'missing instance variable: ', storeSpecifier) raiseRequest.
            "/ if proceeded, assume the handler fixed it.
        ] ifFalse:[
            parent instVarAt:instVarOffset put:obj
        ]
    ].

    "Modified: / 09-11-2007 / 14:46:25 / cg"
!

remember:obj as:id
    "map an object id to an object.
     We know here that object ids are consecutive integers starting at 1.
     If this changes, we have to use a Dictionary instead of an OrderedCollection"

    idToObjectMapping size < id ifTrue:[
        idToObjectMapping isNil ifTrue:[
            idToObjectMapping := OrderedCollection new:100.
        ].
        idToObjectMapping grow:id.
    ].
    idToObjectMapping at:id put:obj
!

valueForElementName:elementName class:classGiven size:sizeGiven
    "answer an object for a XML element name.
     Only some primitive elements or containers may be resolved immediately.
     Others need the elements text available only when the end tag is
     being processed. In this case push a converter onto the stack and return 0"

    |class obj|

    characterStream reset.
    needWhitespace := false.

    classGiven notNil ifTrue:[
        class := Smalltalk classNamed:classGiven.
        class isNil ifTrue:[
            "/ some special cases ...
            XML::InvalidSignal raiseWith:classGiven errorString:' - cannot resolve class: ', classGiven.
        ].
    ].
    
    elementName = 'integer' ifTrue:[
        class isNil ifTrue:[class := Integer].
        stack push:(NumberConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'string' ifTrue:[
        needWhitespace := true.
        stack push:(StringConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'true' ifTrue:[
        ^ true
    ].
    elementName = 'false' ifTrue:[
        ^ false
    ].
    elementName = 'nil' ifTrue:[
        ^ nil
    ].
    elementName = 'float' ifTrue:[
        class isNil ifTrue:[class := Float].
        stack push:(NumberConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'number' ifTrue:[
        class isNil ifTrue:[class := Object].  "Number cannot convert Fractions!!"
        stack push:(NumberConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'symbol' ifTrue:[
        needWhitespace := true.
        stack push:(SymbolConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'char' ifTrue:[
        stack push:(CharacterConverter new elementClass:class).
        ^ 0.
    ].
    elementName = 'bytes' ifTrue:[
        stack push:(Base64Converter new elementClass:class).
        ^ 0.
    ].
    elementName = 'dictionary' ifTrue:[
        class isNil ifTrue:[
            class := Dictionary
        ].
        ^ (sizeGiven isNil ifTrue:[class new] ifFalse:[class new:sizeGiven]).
    ].
    elementName = 'sequence' ifTrue:[
        class isNil ifTrue:[
            class := Array
        ].
        ^ (sizeGiven isNil ifTrue:[class new] ifFalse:[class new:sizeGiven]).
    ].
    elementName = 'collection' ifTrue:[
        class isNil ifTrue:[
            class := Set
        ].
        ^ (sizeGiven isNil ifTrue:[class new] ifFalse:[class new:sizeGiven]).
    ].
    elementName = 'object' ifTrue:[
        class isNil ifTrue:[
            XML::InvalidSignal raiseWith:elementName errorString:' - unknown element: ', elementName.
            ^ nil
        ].
        class isMeta ifTrue:[
            ^ class theNonMetaclass
        ].
        (class isVariable and:[sizeGiven notNil]) ifTrue:[
            ^ class basicNew:sizeGiven.
        ].
        ^ class new.
    ].

    XML::InvalidSignal raiseWith:elementName errorString:' - unknown non-primitive element: ', elementName.
    ^ nil

    "Modified: / 03-11-2007 / 11:07:14 / cg"
! !

!XMLStandardDecoder methodsFor:'content handler'!

characters:aString
    "process xml character data.
     If characters are separated by ignorable whitespace, this may
     be called more than once for a single element"

    characterStream nextPutAll:aString.
!

endElement:localName namespace:nameSpace prefix:nameSpacePrefix
    "end of xml element.
     If stack is empty, this is the single top element.
     Otherwise this is a subordinate element, that will be added
     to its containing object"

    |currentObjectContainer obj id|

    currentObjectContainer := stack pop.
    currentObjectContainer isContainedObject ifTrue:[
        obj := currentObjectContainer object.
    ] ifFalse:[
        "this is a delayed element. Now, that I know the elements text,
         pop the converter from the stack and create the element"
        obj := currentObjectContainer convertText:characterStream contents.
        id := currentObjectContainer id.
        id notNil ifTrue:[
            self remember:obj as:id
        ].
    ].
    localName = 'object' ifTrue:[
        obj postDecodeFrom:self aspect:aspect
    ].

    stack size == 0 ifTrue:[
        "this is the top xml object"
        object := obj.
    ] ifFalse:[
        "this is a subordinate object"
        self add:obj toParent:stack top object with:currentObjectContainer storeSpecification.
    ].
!

ignorableWhitespace:aString
    "got white space. We need all white space in strings and symbols"

    needWhitespace ifTrue:[
        characterStream nextPutAll:aString.
    ] ifFalse:[
        characterStream nextPut:Character space
    ].
!

startDocument

    stack := Stack new.
    characterStream := '' writeStream.
!

startElement:localName namespace:namespace prefix:nameSpacePrefix attributes:attributes

    |id class nameOrIndex idRef size obj|

    attributes do:[:eachAttribute |
        |type|

        type := eachAttribute tag type.

        type = 'ID' ifTrue:[
            id := Integer readFrom:eachAttribute value.
        ] ifFalse:[type = 'CLASS' ifTrue:[
            class := eachAttribute value.
        ] ifFalse:[type = 'NAME' ifTrue:[
            nameOrIndex := eachAttribute value.
        ] ifFalse:[type = 'INDEX' ifTrue:[
            nameOrIndex := Integer readFrom:eachAttribute value.
        ] ifFalse:[type = 'IDREF' ifTrue:[
            idRef := Integer readFrom:eachAttribute value.
        ] ifFalse:[type = 'SIZE' ifTrue:[
            size := Integer readFrom:eachAttribute value.
        ] ifFalse:[type = 'ASPECT' ifTrue:[
            "aspect may be present only in the start element"
            aspect := eachAttribute value.
        ] ifFalse:[type = 'VERSION' ifTrue:[
            "version may be present only in the start element"
            version := eachAttribute value.
        ] ifFalse:[
            XML::InvalidSignal raiseWith:type errorString:' - unknown attribute: ', type
        ]]]]]]]]
    ].

    "/ the elements value ...
    idRef notNil ifTrue:[
        obj := idToObjectMapping at:idRef ifAbsent:[].
        "there is never a reference to a literal"
        obj isNil ifTrue:[
            XML::InvalidSignal raiseWith:idRef errorString:' - unresolved reference: ', idRef
        ].
    ] ifFalse:[
        obj := self valueForElementName:localName class:class size:size.
        obj == 0 ifTrue:[
            "could not resolve object, because I don't know the text yet.
             Delay until #endElement.
             Converter is on top of stack. Converter is used to remember attributes"
            stack top id:id storeSpecification:nameOrIndex.
            ^ self
        ].
        id notNil ifTrue:[
            "object may have been referenced. Remember by ID-attribute"
            self remember:obj as:id
        ].
    ].
    stack push:(ContainedObject new storeSpecification:nameOrIndex object:obj).
! !

!XMLStandardDecoder methodsFor:'error handler'!

error: anInvalidSignal
        "Called when a validating is on and parser reaches invalid XML. Is resumable
         Also called when a class does not exist"

        anInvalidSignal reject.
!

fatalError: anXMLSignal
        "The XML parsers malformed and invalid signals are sent to this method"

        anXMLSignal reject
!

warning: aWarningSignal

        aWarningSignal resume
! !

!XMLStandardDecoder::AbstractConverter class methodsFor:'documentation'!

documentation
"
    Converters are used to convert the xml text representation to objects.
    The attributes needed to create the object are known, when the start tag is
    processed, the xml text is known, when the end tag is read.


    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]
        id                                      The unique id of the object in the 
                                                xml documnet (used to detect references)
        storeSpecification  (Integer|Symbol)    The index or name of the object in the
                                                parent Object.
        elementClass        (Class|nil)         The class of the element. Nil if default 
                                                class specified by converter.

    [class variables:]

    [see also:]

"
! !

!XMLStandardDecoder::AbstractConverter methodsFor:'accessing'!

elementClass
    ^ elementClass
!

elementClass:something
    elementClass := something.
!

id
    "return the value of the instance variable 'id' (automatically generated)"

    ^ id
!

id:idArg storeSpecification:storeSpecificationArg 
    "set instance variables (automatically generated)"

    id := idArg.
    storeSpecification := storeSpecificationArg.
!

storeSpecification
    "return the value of the instance variable 'storeSpecification' (automatically generated)"

    ^ storeSpecification
! !

!XMLStandardDecoder::AbstractConverter methodsFor:'converting'!

convertText:aString
    "convert the text to the internal smalltalk representation"

    ^ self subclassResponsibility
! !

!XMLStandardDecoder::AbstractConverter methodsFor:'testing'!

isContainedObject
    "answer true, if i already know the final object"

    ^ false
! !

!XMLStandardDecoder::Base64Converter methodsFor:'converting'!

convertText:aString
    "convert the text to a symbol"

    |bytes|

    bytes := (Base64Coder on:aString readStream) upToEnd.

    elementClass notNil ifTrue:[
        |object size|

        elementClass == bytes class ifTrue:[
            ^ bytes
        ].

        size := bytes size.
        object := elementClass new:size.
        object replaceBytesFrom:1 to:size with:bytes startingAt:1.
        ^ object.
   ].
    
    ^ bytes
! !

!XMLStandardDecoder::CharacterConverter methodsFor:'converting'!

convertText:aString
    "convert the text to a character"

    ^ Character value:(Integer readFrom:aString)
! !

!XMLStandardDecoder::ContainedObject methodsFor:'accessing'!

object
    "return the value of the instance variable 'object' (automatically generated)"

    ^ object
!

object:something
    "set the value of the instance variable 'object' (automatically generated)"

    object := something.
!

storeSpecification:storeSpecificationArg object:objectArg 
    "set instance variables (automatically generated)"

    storeSpecification := storeSpecificationArg.
    object := objectArg.
! !

!XMLStandardDecoder::ContainedObject methodsFor:'testing'!

isContainedObject
    "answer true, if i already know the final object"

    ^ true
! !

!XMLStandardDecoder::NumberConverter methodsFor:'converting'!

convertText:aString
    "convert the text to a number"

    ^ elementClass readFromString:aString
! !

!XMLStandardDecoder::StringConverter methodsFor:'converting'!

convertText:aString
    "convert the text to a string"

    ^ aString
! !

!XMLStandardDecoder::SymbolConverter methodsFor:'converting'!

convertText:aString
    "convert the text to a symbol"

    ^ aString asSymbol
! !

!XMLStandardDecoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/stx/XMLStandardDecoder.st,v 1.21 2007/11/09 13:47:31 cg Exp $'
! !
