"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/xml/stx' }"

"{ NameSpace: XML }"

Object subclass:#FastSAXDriver
	instanceVariableNames:'inStream hereChar characters emptyDict'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-STX-SAX'
!

!FastSAXDriver class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2007 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    A stripped-down, but much faster abstract superclass for XML-Parsers.
    To use: subclass from this (FastSAXDriver) instead of from SAXDriver.
    (whitebox API is the same).

    Disadvantages:
        no namespaces;
        no defined entities;
        all hard coded.
        treats '.' and ':' as alpha-characters within a tag.

    Advantages:
        super fast


    Use at your own risk, and only for XML which is known to be valid (i.e., your own).

    [author:]
        Claus Gittinger
"
!

examples
"
    |decoder artefact|

    decoder := NewXMLArtifactDecoder 
                    on:(theArtifact at:2) asString readStream
                    beforeScanDo:[:parser | 
                        parser builder driver dataBase:self.
                        parser validate:false
                    ].
    artefact := decoder decodedObject.
"
! !

!FastSAXDriver class methodsFor:'instance creation'!

on:aStream beforeScanDo:aOneArgBlock
    ^ self new on:aStream beforeScanDo:aOneArgBlock

    "Created: / 17-11-2007 / 17:00:07 / cg"
! !

!FastSAXDriver methodsFor:'api'!

characters:characters
"/    self halt.

    "Created: / 18-11-2007 / 13:58:35 / cg"
!

decodeAttributeValue:value
    ^ value
!

endElement:tag namespace:alwaysNil1 prefix:alwaysNil2 
"/    self halt.

    "Created: / 18-11-2007 / 14:04:11 / cg"
!

startElement:tag namespace:alwaysNil1 prefix:alwaysNil2 attributes:attsOrNil
"/    self halt.

    "Created: / 18-11-2007 / 13:55:46 / cg"
! !

!FastSAXDriver methodsFor:'api-dummy'!

builder 
    ^ self

    "Created: / 18-11-2007 / 14:39:00 / cg"
!

documentLocator:ignored
!

driver
    ^ self

    "Created: / 18-11-2007 / 14:39:02 / cg"
!

tagPosition:aStreamPosition
    "can be redefined to remember the startPosition"
!

validate:doValidate
    self assert:(doValidate == false).

    "Created: / 18-11-2007 / 14:39:37 / cg"
! !

!FastSAXDriver methodsFor:'error handling'!

parseError:message
    MalformedSignal raiseRequestErrorString:message

    "Modified: / 26-04-2010 / 18:39:21 / cg"
! !

!FastSAXDriver methodsFor:'parsing'!

on:aStream beforeScanDo:aOneArgBlock
    emptyDict := Dictionary new.
    inStream := aStream.
    aOneArgBlock value:self.
    self readNextObject.

    "Created: / 17-11-2007 / 17:00:38 / cg"
!

readAttribute:attsIn
    |key value atts peekC|

    key := self readTagOrAttributeName.
    inStream skipSeparators.
    inStream peek == $= ifFalse:[
        self parseError:'= expected'
    ].
    inStream next.

    inStream skipSeparators.
    ((peekC := inStream peek) == $" or:[ peekC == $' ]) ifFalse:[
        self parseError:'" or '' expected'
    ].
    inStream next.

    value := inStream upTo:peekC.
    value := self decodeAttributeValue:value.
    (value includes:$&) ifTrue:[
        value := self unEscape:value.
    ].

    atts := attsIn.
    atts isNil ifTrue:[
        atts := Dictionary new.
    ].
    atts at:key put:value.
    ^ atts.

    "Created: / 18-11-2007 / 13:54:56 / cg"
!

readNextObject
    |tag atts|

    inStream skipSeparators.
    inStream next == $< ifFalse:[
        self parseError:'< expected'.
    ].

    [inStream peek == $?] whileTrue:[
        "/ skip processing instructions...
        inStream next.
        tag := self readTagOrAttributeName.
        inStream skipSeparators.
        [ inStream peek ~~ $? ] whileTrue:[
            atts := self readAttribute:atts.
            inStream skipSeparators.
        ].
        inStream next.
        inStream next == $> ifFalse:[
            self parseError:'> expected'.
        ].
        inStream skipSeparators.
        inStream next == $< ifFalse:[
            self parseError:'< expected'.
        ].
    ].

    ^ self readNextObject2

    "Modified: / 18-11-2007 / 13:53:45 / cg"
!

readNextObject2
    |tag peekc endTag atts isEnd nextChar|

    tag := self readTagOrAttributeName.
    inStream skipSeparators.
    [ 
        peekc := inStream peek.
        peekc ~~ $> and:[peekc ~~ $/]
    ] whileTrue:[
        atts := self readAttribute:atts.
        inStream skipSeparators.
    ].
    isEnd := (peekc == $/).
    isEnd ifFalse:[
        "/ leave insteam positioned after the '>' (for CDATA handling)
        inStream next.
    ].

    self startElement:tag namespace:nil prefix:nil attributes:(atts ? emptyDict).

    isEnd ifTrue:[
        inStream next.   "/ skip over '/'
        (nextChar := inStream next) == $> ifFalse:[
            self parseError:'> expected'.
        ].
        self endElement:tag namespace:nil prefix:nil.
        ^ self.            
    ].

    [true] whileTrue:[
        characters := inStream upTo:$<.
        characters notEmpty ifTrue:[
            (characters includes:$&) ifTrue:[
                characters := self unEscape:characters.
            ].
            self characters:characters
        ].

        self tagPosition:(inStream position-1). "/ remember position before '<' (for CDATA handling)

        inStream peek == $!! ifTrue:[
            inStream throughAll:'-->'.
        ] ifFalse:[
            inStream peek == $/ ifTrue:[
                inStream next.
                endTag := self readTagOrAttributeName.
                endTag = tag ifFalse:[
                    self parseError:'< expected'.
                ].
                nextChar := inStream next.
                nextChar == $> ifFalse:[
                    nextChar isSeparator ifTrue:[
                        inStream skipSeparators.    
                        nextChar := inStream next.
                    ].
                    nextChar == $> ifFalse:[
                        self parseError:'> expected'.
                    ].
                ].
                self endElement:tag namespace:nil prefix:nil.
                ^ self.            
            ].
            self readNextObject2
        ]
    ].

    "Created: / 18-11-2007 / 12:21:53 / cg"
    "Modified: / 18-06-2010 / 18:20:09 / cg"
!

readTagOrAttributeName
    "a tag consiss of:
        alphaNumericWords separated by '-', '.' or ':'s
    "

    |t w c|

    [true] whileTrue:[
        w := inStream nextAlphaNumericWord.
        t isNil ifTrue:[
            t := w.
        ] ifFalse:[
            t := t , w
        ].
        c := inStream peek.
        (c == $- or:[c == $. or:[c == $:]]) ifFalse:[
            ^ t.
        ].
        inStream next.
        t := t copyWith:c.
    ]

    "Created: / 18-11-2007 / 13:33:29 / cg"
!

unEscape:characters
    |w part in out code|

    out := CharacterWriteStream on:''.
    in := characters readStream.
    [in atEnd] whileFalse:[
        part := in upTo:$&.
        out nextPutAll:part.
        in atEnd ifFalse:[
            in peek == $# ifTrue:[
                in next.
                in peek == $x ifTrue:[
                    "/ hex code
                    in next.
                    code := Integer readFrom:in radix:16.
                ] ifFalse:[
                    code := Integer readFrom:in.
                ].
                out nextPut:(Character value:code).
                in peek == $; ifFalse:[
                    self parseError:'; expected after character code'
                ].
                in next.
            ] ifFalse:[
                w := in upTo:$;.
                w = 'amp' ifTrue:[
                    out nextPut:$&
                ] ifFalse:[
                    w = 'lt' ifTrue:[
                        out nextPut:$<
                    ] ifFalse:[
                        w = 'gt' ifTrue:[
                            out nextPut:$>
                        ] ifFalse:[
                            w = 'apos' ifTrue:[
                                out nextPut:$'
                            ] ifFalse:[
                                w = 'quot' ifTrue:[
                                    out nextPut:$"
                                ] ifFalse:[
        "/                    w = 'nbsp' ifTrue:[
        "/                        out nextPut:(Character value:160).
        "/                    ] ifFalse:[
                                  self breakPoint:#cg.
        "/                    ].
                                ]
                            ]
                        ].
                    ].
                ]
            ].
        ].
    ].
    ^ out contents

    "Created: / 18-11-2007 / 14:27:41 / cg"
! !

!FastSAXDriver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/stx/XML__FastSAXDriver.st,v 1.14 2011/09/21 09:30:05 ca Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/stx/XML__FastSAXDriver.st,v 1.14 2011/09/21 09:30:05 ca Exp $'
! !
