"{ Package: 'stx:goodies/xml/xmlrpc' }"

"{ NameSpace: XMLRPC }"

Object subclass:#XMLRPCDateTime
	instanceVariableNames:'date time'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Data'
!

XMLRPCDateTime comment:'DateTime object for sending and receiving dates via XML-RPC.
         See the convenience class methods for easily creating instances of this class.'
!


!XMLRPCDateTime class methodsFor:'instance creation'!

fromDate:d time:t 
    "Get a new XMLRPCDateTime instance from the given date and time."
    
    ^ (XMLRPCDateTime new)
        date:d;
        time:t
!

fromISO8601String:s 
    "Get a new XMLRPCDateTime instance from the given ISO8601 string."
    
    ^ XMLRPCDateTime new decodeISO8601:s
!

fromTimestamp:aTimestamp
    "Get a new XMLRPCDateTime instance from the given date and time."
    
    ^ (XMLRPCDateTime new)
        date:(aTimestamp asDate);
        time:(aTimestamp asTime)

    "
     self fromTimestamp:(Timestamp now)
    "
!

now
    "Get a new XMLRPCDateTime instance representing the current time."
    
    ^ (XMLRPCDateTime new)
        date:(Date today);
        time:(Time now)
! !

!XMLRPCDateTime methodsFor:'*xmlrpc-server'!

asXMLRPCString
    |str|

    str := WriteStream on:(String new:70).
    str
        nextPutAll:'<value><dateTime.iso8601>';
        nextPutAll:self encodeISO8601;
        nextPutAll:'</dateTime.iso8601></value>';
        cr.
    ^ str contents
! !

!XMLRPCDateTime methodsFor:'accessing'!

date
    "Get the date portion of the datetime."
    
    ^ date
!

date:d 
    "Set the date portion of the datetime."
    
    date := d
!

time
    "Get the time portion of the datetime."
    
    ^ time
!

time:t 
    "set the time portion of the datetime."
    
    time := t
! !

!XMLRPCDateTime methodsFor:'converting'!

asTimestamp
    ^ Timestamp fromDate:date andTime:time
! !

!XMLRPCDateTime methodsFor:'printing'!

printOn:aStream 
    "Send it to the stream."
    
    aStream nextPutAll:(self encodeISO8601).
! !

!XMLRPCDateTime methodsFor:'private'!

= anObject 
    "True if the other object is an XMLRPCDateTime representing the same date and time."

    ((self class) == (anObject class)) ifTrue:[
        ((self date) = (anObject date)) ifTrue:[
            ^ (self time) = (anObject time)
        ]
    ].
    ^ false
!

hash
    "Predictable hash for this object."
    
    ^ date hash bitXor:(time hash)
!

pad:in to:preferredSize with:thing 
    "Return a string with in padded on the left with the given number of  
     the given thing."
    
    |rv|

    rv := in asString.
    [
        rv size < preferredSize
    ] whileTrue:[ rv := thing , rv ].
    ^ rv
! !

!XMLRPCDateTime methodsFor:'xmlrpc'!

decodeISO8601:s 
    "Set the date and time in this object by parsing the ISO8601 string."
    
    |d t Y M D|

    Y := (s copyFrom:1 to:4) asInteger.
    M := (s copyFrom:5 to:6) asInteger.
    D := (s copyFrom:7 to:8) asInteger.
    d := Date 
                newDay:D
                month:M
                year:Y.
    t := Time readFrom:((s copyFrom:10 to:17) readStream).
    self date:d.
    self time:t
!

encodeISO8601
    "Encode me into an ISO8601 datetime for XML-RPC"

    ^ String streamContents:[:s |
        s nextPutAll:(date year asString).
        s nextPutAll:(self 
                        pad:(date monthIndex asString)
                        to:2
                        with:'0').
        s nextPutAll:(self 
                        pad:(date dayOfMonth asString)
                        to:2
                        with:'0').
        s nextPutAll:'T'.
        time print24:true on:s.
    ].
! !

!XMLRPCDateTime class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCDateTime.st,v 1.6 2010/03/25 12:06:07 cg Exp $'
! !
