"{ Package: 'stx:goodies/xml/xmlrpc' }"

"{ NameSpace: XMLRPC }"

Object subclass:#XMLRPCDecoder
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Data'
!

XMLRPCDecoder comment:'XML-RPC decoder.
This class will take XML-RPC results and translate them into smalltalk types.  The following translations will take place:
  boolean  -> True or False
      string    -> String
    double   -> Float class
        i4         -> SmallInteger
     struct    -> Dictionary
        array    -> Array
      dateTime.iso8601 -> XMLRPCDateTime
     base64   -> Stream'
!


!XMLRPCDecoder methodsFor:'decoding'!

decode:xml 
    "Decode an XML Element representing an XML-RPC response."
    "xml inspect."
    
    |el0 el0Name|

    xml entities "elements" isEmpty ifTrue:[
        ^ xml contentString
    ].
    "Figure out what to do based on the name of the element (el0 is usually a required argument."
    el0 := xml elements at:1.
    el0Name := el0 name.
    el0Name = #i4 ifTrue:[ ^ Integer readFrom:el0 contentString readStream ].
    el0Name = #i8 ifTrue:[ ^ Integer readFrom:el0 contentString readStream ].
    el0Name = #int ifTrue:[ ^ Integer readFrom:el0 contentString readStream ].
    el0Name = #boolean ifTrue:[ ^ el0 contentString = '1' ].
    el0Name = #string ifTrue:[ ^ el0 contentString ].
    el0Name = #array ifTrue:[ ^ self decodeArray:el0 ].
    el0Name = #struct ifTrue:[ ^ self decodeStruct:el0 ].
    el0Name = #param ifTrue:[ ^ self decode:el0 ].
    el0Name = #value ifTrue:[ ^ self decode:el0 ].
    el0Name = #double ifTrue:[ ^ Float readFrom:el0 contentString readStream ].
    el0Name = #'dateTime.iso8601' ifTrue:[ ^ XMLRPCDateTime fromISO8601String:el0 contentString ].
    el0Name = #base64 ifTrue:[ ^ Base64MimeConverter mimeDecodeToBytes:el0 contentString readStream ].
    el0Name = #nil ifTrue:[ ^ nil ].
    XMLRPCException signal:'Unhandled type!!  ' , xml name.

    "Modified: / 14-07-2011 / 22:31:44 / cg"
! !

!XMLRPCDecoder methodsFor:'private'!

decodeArray:xml 
    "Decode the Array."

    "/ ^ ((xml elements at:1) elements collect:[:el | (self decode:el)]) asArray.
    "/ no: skip cdata...
    ^ ((xml tagElements at:1) tagElements collect:[:el | (self decode:el)]) asArray.
"/
"/    |rv|
"/
"/    rv := OrderedCollection new.
"/    (xml elements at:1) elements do:[:el | 
"/        rv add:(self decode:el)
"/    ].
"/    ^ rv asArray

    "Modified (comment): / 14-07-2011 / 22:03:43 / cg"
!

decodeStruct:xml 
    "Decode the Struct."
    
    |rv|

    rv := Dictionary new.
    xml tagElements do:[:el | 
        rv at:(el elementAt:'name') contentString
            put:(self decode:(el elementAt:'value'))
    ].
    ^ rv

    "Modified: / 14-07-2011 / 22:08:52 / cg"
! !

!XMLRPCDecoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCDecoder.st,v 1.9 2011/07/14 20:32:16 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCDecoder.st,v 1.9 2011/07/14 20:32:16 cg Exp $'
! !
