"{ Package: 'stx:goodies/xml/xmlrpc' }"

"{ NameSpace: XMLRPC }"

Object subclass:#XMLRPCEncoder
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Data'
!

XMLRPCEncoder comment:'Encode objects into proper XML-RPC types.
The following types are understood (smalltalk types in parens):
    Booleans (True and False)
      Strings (String)
       Doubles (Float class)
  Integers (SmallInteger class)
  Struct (Dictionary)
    Array (Array)
  dateTime.iso8601 (XMLRPCDateTime)
      base64 (Stream)'
!


!XMLRPCEncoder methodsFor:'encoding'!

encode:val 
    "Encode the given value for inclusion in an XML-RPC request or response.
     Returns the encoded value."

    "/ cg: what an ugly piece of code - should be done using a visitor...
    |valClass d|

    (val == True or:[val == true]) ifTrue:[
        "/ ^ self wrapVal:'1' type:'boolean'
        ^ '<value><boolean>1</boolean></value>'
    ].
    (val == False or:[val == false]) ifTrue:[
        "/ ^ self wrapVal:'0' type:'boolean'
        ^ '<value><boolean>0</boolean></value>'
    ].
    val isNil ifTrue:[
        ^ '<nil />'
    ].

    valClass := val class.
    val isInteger ifTrue:[
        "/ ^ self wrapVal:val asString type:'i4'
        ((val <= 16r7FFFFFFF) and:[ val >= 16r-80000000]) ifTrue:[
            ^ '<value><i4>',val printString,'</i4></value>'
        ].
        ^ '<value><i8>',val printString,'</i8></value>'
    ].
    val isString ifTrue:[
        "/ ^ self wrapVal:(self normalize:val)
        ^ '<value>',(HTMLUtilities withAllSpecialHTMLCharactersEscaped:val),'</value>'
    ].
    val isLimitedPrecisionReal ifTrue:[
        ^ self wrapVal:(val printfPrintString:'%.14f') type:'double'
    ].
    valClass == XMLRPCDateTime ifTrue:[
        ^ self wrapVal:val encodeISO8601 type:'dateTime.iso8601' 
    ].
    valClass == Timestamp ifTrue:[
        ^ self wrapVal:(XMLRPCDateTime fromTimestamp:val) encodeISO8601 type:'dateTime.iso8601' 
    ].
    val isSequenceable ifTrue:[
        ^ self wrapVal:(self encodeArray:val) type:'array' 
    ].
    (valClass == Dictionary or:[ valClass == OrderedDictionary ]) ifTrue:[
        ^ self wrapVal:(self encodeStruct:val) type:'struct' 
    ].
    val isStream ifTrue:[
        ^ self wrapVal:(Base64Coder decode:val)
        "/ ^ self wrapVal:(Base64MimeConverter mimeEncode:val) contents type:'base64'
    ].

    "/ XMLRPCException signal:'Unhandled type:  ' , val className

    d := Dictionary new.
    valClass instVarNames do:[:each | 
        d at:each asSymbol put:(val instVarNamed:each)
    ].
    d size > 0 ifTrue:[
        ^ self encodeStruct:d
    ].
    ^ '<value>nil</value>'

    "Modified: / 14-07-2011 / 22:31:20 / cg"
    "Modified (format): / 16-07-2011 / 00:05:04 / cg"
! !

!XMLRPCEncoder methodsFor:'private'!

encodeArray:a 
    "Encode an array."
    
    ^ String streamContents:[:s |
        s nextPutAll:'<data>'.
        a do:[:el | 
            s nextPutAll:(self encode:el)
        ].
        s nextPutAll:'</data>']
!

encodeStruct:s 
    "Encode a struct."
    
    ^ String streamContents:[:str |
        s keysAndValuesDo:[:key :val | 
            str 
                nextPutAll:'<member><name>';
                nextPutAll:(self normalize:key);
                nextPutAll:'</name>'; 
                nextPutAll:(self encode:val);
                nextPutAll:'</member>'
        ]]
!

normalize:s 
    "Normalize an XML string."
    
    ^ HTMLUtilities withAllSpecialHTMLCharactersEscaped:s.

"/    |map|
"/
"/
"/    map := Dictionary new.
"/    map at:$< put:'&lt;'.
"/    map at:$> put:'&gt;'.
"/    map at:$& put:'&amp;'.
"/    ^ String streamContents:[:str |
"/        s do:[:c | 
"/            str nextPutAll:(map at:c ifAbsent:[ c ])
"/        ]].
!

wrapVal:val 
    "Wrap the given value in <value> tags, assuming it's already a string."
    
    ^ self wrapVal:val type:'string'
!

wrapVal:val type:t 
    "Wrap the given value in <value> and <t>"
    
    ^ '<value><' , t , '>' , val , '</' , t , '></value>'
! !

!XMLRPCEncoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCEncoder.st,v 1.12 2011/07/15 22:05:25 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCEncoder.st,v 1.12 2011/07/15 22:05:25 cg Exp $'
! !
