"
 COPYRIGHT (c) 2010 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/xml/xmlrpc/server' }"

"{ NameSpace: XMLRPC }"

HTTPService subclass:#XMLRPCService
	instanceVariableNames:'registeredMethods capabilities'
	classVariableNames:'Verbose DebugErrors'
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Server'
!

Object subclass:#ServiceEntry
	instanceVariableNames:'methodName methodHelp methodSignature receiver selector'
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLRPCService
!

HTTPServiceSettingsApplication subclass:#ServicesSettingsAppl
	instanceVariableNames:'selectedServices enableRemoveService serviceTableRowList'
	classVariableNames:'LastServiceName LastClassName LastSelector'
	poolDictionaries:''
	privateIn:XMLRPCService
!

Object subclass:#ServiceItem
	instanceVariableNames:'serviceName receiver selector itemList showRowSeparator
		serviceInstance serviceApplication showClassName
		firstLineOfService'
	classVariableNames:''
	poolDictionaries:''
	privateIn:XMLRPCService::ServicesSettingsAppl
!

!XMLRPCService class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2010 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    XML-RPC Service.

    start via settings, or programatically via:
        | rpcService |

        rpcService := XMLRPCService new.
        rpcService linkName:'/xmlrpc'.
        rpcService registerServiceOn:(HTTPServer serverOnPort:8080).

    add services with:
        rpcService registerMethod:'nameOfMethod' receiver:SomeServiceClass selector:selectorToCall

    or, with additional info for introspection:
        rpcService registerMethod:'nameOfMethod' help:'helptext' signature:'signatureInfo'
                   receiver:SomeServiceClass selector:selectorToCall

    to enable the introspection protocol (listMethods etc.),
        rpcService enableIntrospectionProtocol


    FaultCodes:
        -32700 ---> parse error. not well formed
        -32701 ---> parse error. unsupported encoding
        -32702 ---> parse error. invalid character for encoding
        -32600 ---> server error. invalid xml-rpc. not conforming to spec.
        -32601 ---> server error. requested method not found
        -32602 ---> server error. invalid method parameters
        -32603 ---> server error. internal xml-rpc error
        -32500 ---> application error
        -32400 ---> system error
        -32300 ---> transport error
"
!

versionString
    ^ '1.2'
! !

!XMLRPCService class methodsFor:'debugging'!

debugErrors:aBoolean
    DebugErrors := aBoolean

    "
     self debugErrors:true
     self debugErrors:false
    "
!

verbose:aBoolean
    Verbose := aBoolean

    "
     self verbose:true
     self verbose:false
    "
! !

!XMLRPCService class methodsFor:'defaults'!

canChangeReplyPresenter
    ^ false
!

defaultAllowRobots
    ^ false
!

linkName
    "return the default linkName path (with slash)"

    ^ '/xmlrpc'
! !

!XMLRPCService class methodsFor:'queries-service'!

settingsApplicationClass
    "a SettingsApplication class - or nil (used in the settings dialog if non-nil)"

    ^ ServicesSettingsAppl
! !

!XMLRPCService methodsFor:'faultCodes'!

faultCode_ParseError
    ^ -32700 "/ ---> parse error. not well formed
! !

!XMLRPCService methodsFor:'initialization'!

initialize
    super initialize.

    self initializeCapabilitiesInfo.

    "/ self enableIntrospectionProtocol.  -- not done automatically
    self enableMulticallProtocol.
!

initializeCapabilitiesInfo
    "to be redefined by subclasses to add more capabilities
     (i.e. an expeccoNET-XMLRPC service might add info about its supported protocol)"

    capabilities :=  Dictionary new.
    capabilities
        at:#xmlrpc put:(
                    Dictionary new
                        at:#specUrl put:'http://www.xmlrpc.com/spec';
                        at:#specVersion put:1;
                        yourself);
        at:#faults_interop put:(
                    Dictionary new
                        at:#specUrl put:'http://xmlrpc-epi.sourceforge.net/specs/rfc.fault_codes.php';
                        at:#specVersion put:20010516;
                        yourself);
        at:#'json-rpc' put:(
                    Dictionary new
                        at:#specUrl put:'http://json-rpc.org/wiki/specification';
                        at:#specVersion put:1;
                        yourself).
! !

!XMLRPCService methodsFor:'registration'!

registerMethod:methodName help:methodHelp signature:methodSignature receiver:receiver selector:aSelector 
    registeredMethods isNil ifTrue:[ registeredMethods := Dictionary new ].
    registeredMethods 
        at:methodName asSymbol 
        put:(ServiceEntry new
                methodName:methodName help:methodHelp signature:methodSignature
                receiver:receiver selector:aSelector)

    "
     ((HTTPServer runningServerOnPort:8082)
        serviceForLink:'/xmlrpc')
            registerService:'echo' class:XMLRPCService selector:'echoService'    
    "
!

registerService:serviceName class:aClass selector:aSelector 
    self registerService:serviceName receiver:aClass selector:aSelector

    "
     ((HTTPServer runningServerOnPort:8082)
        serviceForLink:'/xmlrpc')
            registerService:'echo' class:XMLRPCService selector:'echoService'    
    "
!

registerService:methodName receiver:receiver selector:aSelector 
    self registerMethod:methodName help:nil signature:nil receiver:receiver selector:aSelector

    "
     ((HTTPServer runningServerOnPort:8082)
        serviceForLink:'/xmlrpc')
            registerService:'echo' class:XMLRPCService selector:'echoService'    
    "
!

registeredMethods
    ^ registeredMethods
!

removeService:aServiceName 
    registeredMethods removeKey:aServiceName asSymbol
! !

!XMLRPCService methodsFor:'request processing'!

faultDataForException:error
    |errorDict|

    errorDict := Dictionary new.
    (error class canUnderstand:#faultCode) ifTrue:[
        errorDict at:#faultCode put:error faultCode
    ] ifFalse:[
        errorDict at:#faultCode put:"10" -32400 "---> system error"
    ].
    errorDict at:#faultString put:error messageText.
    ^ errorDict
!

faultDataStringForException:error
    |str errorDict|

    errorDict := self faultDataForException:error.

    str := WriteStream on:(String new:1000).
    str
        nextPutAll:'<?xml version="1.0"?><methodResponse><fault>';
        nextPutAll:(XMLRPCEncoder new encode:errorDict); "/ errorDict asXMLRPCString;
        nextPutAll:'</fault></methodResponse>'.
    ^ str contents
!

methodResponseForRPCRequest:xrReq
    |srv ret methodName|

    [
        methodName := xrReq methodCall asSymbol.
        srv := registeredMethods 
                    at:methodName
                    ifAbsent:[ XMLRPCMethodError signal:'Serverside method not found: ',methodName].
        ret := srv receiver 
                    perform:(srv selector) 
                    withOptionalArgument:xrReq. "behaves like an array w.r.t. argument fetches"
    ] on:Error do:[:ex |
        DebugErrors == true ifTrue:[Debugger enter].
        XMLRPCMethodError raiseErrorString:'XMLRPC Execution Error: ',ex description
    ].
    "/ ret isNil ifTrue:[ ret := false ]. - really ????

    ^ xrReq encodeResponse:ret

    "Modified: / 13-05-2010 / 21:44:50 / cg"
!

process:request 
    |xrReq responseData|

    [
        |data dataStream methodCall params|

        data := request data.
        data isEmptyOrNil ifTrue:[
            self error:'invalid request: missing POST-data'
        ].
        dataStream := data readStream.
        dataStream skipSeparators.
        dataStream peek == $< ifTrue:[
            "/ an XML-RPC request
            xrReq := XML_RPCServerRequest fromRequest:data.
        ] ifFalse:[
            "/ a JSON-RPC request
            xrReq := JSON_RPCServerRequest fromRequest:data.
        ].
        xrReq targetURI:request url.
        xrReq request:request.

        Verbose == true ifTrue:[ Transcript showCR:request data ].

        responseData := self methodResponseForRPCRequest:xrReq.

        Verbose == true ifTrue:[ Transcript showCR:responseData ].
    ] on:Error do:[:ex |
        DebugErrors == true ifTrue:[Debugger enter].
        Verbose == true ifTrue:[ Transcript showCR:ex ].
        responseData := self faultDataStringForException:ex.
    ].

    Smalltalk isSmalltalkX ifTrue:[
        request response
            contentType:'text/xml';
            reply:responseData.
    ] ifFalse:[
        ^ HttpResponse 
            fromMIMEDocument:(MIMEDocument contentType:'text/xml' content:responseData)
    ].
! !

!XMLRPCService methodsFor:'service methods'!

enableIntrospectionProtocol
    capabilities
        at:#introspection put:(
                    Dictionary new
                        at:#specUrl put:'http://phpxmlrpc.sourceforge.net/doc-2/ch10.html';
                        at:#specVersion put:2;
                        yourself).

    self 
        registerMethod:'system.versionInfo' 
        help:'Returns information about the XML-RPC version'
        signature:#('struct')   
        receiver:self selector:#'system_versionInfo:'.

    self 
        registerMethod:'system.getCapabilities' 
        help:'This method lists all the capabilites that the XML-RPC server has: the (more or less standard) extensions to the xmlrpc spec that it adheres to' 
        signature:#('struct')   
        receiver:self selector:#'system_getCapabilities'.

    self registerMethod:'system.listMethods' 
        help:'This method lists all the methods that the XML-RPC server knows how to dispatch' 
        signature:#('array')      
        receiver:self selector:#'system_listMethods'.

    self 
        registerMethod:'system.methodHelp' 
        help:'Returns help text if defined for the method passed, otherwise returns an empty string'
        signature:#('string' 'string')      
        receiver:self selector:#'system_methodHelp:'.

    self 
        registerMethod:'system.methodSignature' 
        help:'Returns an array of known signatures (an array of arrays) for the method name passed. If no signatures are known, returns a none-array (test for type !!= array to detect missing signature)'
        signature:#('array' 'string')    
        receiver:self selector:#'system_methodSignature:'.
!

enableMulticallProtocol
    capabilities
        at:#'system.multicall' put:(
                    Dictionary new
                        at:#specUrl put:'http://www.xmlrpc.com/discuss/msgReader$1208';
                        at:#specVersion put:1;
                        yourself).

    self 
        registerMethod:'system.multicall' 
        help:'Boxcar multiple RPC calls in one request. See http://www.xmlrpc.com/discuss/msgReader$1208 for details'
        signature:#('array' 'array')       
        receiver:self selector:#'system_multicall:'.
!

system_getCapabilities
    "service entry for service discovery 
     see proposal in 
        http://tech.groups.yahoo.com/group/xml-rpc/message/2897"

    "/ initialized in #initializeCapabilities
    ^ capabilities
!

system_listMethods
    "service entry for service discovery 
     see proposal in 
        http://www.faqs.org/docs/Linux-HOWTO/XML-RPC-HOWTO.html#xmlrpc-howto-interfaces)
     or: 
        http://xmlrpc-c.sourceforge.net/introspection.html"

    ^ registeredMethods keys asSortedCollection asArray
!

system_methodHelp:params
    "service entry for service discovery 
     see proposal in 
        http://www.faqs.org/docs/Linux-HOWTO/XML-RPC-HOWTO.html#xmlrpc-howto-interfaces)
     or: 
        http://xmlrpc-c.sourceforge.net/introspection.html"

    |methodName entry|

    methodName := params first.
    entry := registeredMethods at:methodName asSymbolIfInterned ifAbsent:[self error:'Can''t introspect: method unknown'].
    ^ entry methodHelp ?''
!

system_methodSignature:params
    "service entry for service discovery 
     see proposal in 
        http://www.faqs.org/docs/Linux-HOWTO/XML-RPC-HOWTO.html#xmlrpc-howto-interfaces)
     or: 
        http://xmlrpc-c.sourceforge.net/introspection.html"

    |methodName entry|

    methodName := params first.
    entry := registeredMethods at:methodName asSymbolIfInterned ifAbsent:[self error:'Can''t introspect: method unknown'].
    ^ Array with:(entry methodSignature)
!

system_multicall:params
    "multicall - gets a vector of call/arg structs,
     returns a vector of results"

    |vector results|

    vector := params first.
    results := vector 
                collect:[:eachCallStruct |
                    |eachResult|

                    [
                        |methodName params srv ret|

                        methodName := eachCallStruct at:'methodName'.
                        params := eachCallStruct at:'params'.

                        srv := registeredMethods 
                                at:methodName asSymbol
                                ifAbsent:[ XMLRPCMethodError signal:'Serverside method not found: ',methodName ].
                        eachResult := srv receiver perform:(srv selector) with:params.
                    ] on:Error do:[:ex |
                        DebugErrors == true ifTrue:[Debugger enter].
                        Verbose == true ifTrue:[ Transcript showCR:ex ].
                        eachResult := self faultDataForException:ex.
                    ].
                    eachResult
                ].

    ^ results.
!

system_versionInfo:params
    "service entry for service discovery 
     see proposal in 
        http://www.faqs.org/docs/Linux-HOWTO/XML-RPC-HOWTO.html#xmlrpc-howto-interfaces)
     or: 
        http://xmlrpc-c.sourceforge.net/introspection.html"

    ^ Dictionary new
        at:'toolkitDocsUrl' put:'www.smalltalk-x.de';
        at:'toolkitName' put:'XML-RPC for ST/X';
        at:'toolkitVersion' put:(self class versionString);
        at:'toolkitOperatingSystem' put:'ST/X';
        yourself
! !

!XMLRPCService::ServiceEntry methodsFor:'accessing'!

methodHelp
    ^ methodHelp
!

methodHelp:something
    methodHelp := something.
!

methodName
    ^ methodName
!

methodName:something
    methodName := something.
!

methodName:methodNameArg help:methodHelpArg signature:methodSignatureArg receiver:receiverArg selector:selectorArg 
    methodName := methodNameArg.
    methodHelp := methodHelpArg.
    methodSignature := methodSignatureArg.
    receiver := receiverArg.
    selector := selectorArg.
!

methodSignature
    ^ methodSignature
!

methodSignature:something
    methodSignature := something.
!

receiver
    ^ receiver
!

receiver:something
    receiver := something.
!

selector
    ^ selector
!

selector:something
    selector := something.
! !

!XMLRPCService::ServicesSettingsAppl class methodsFor:'defaults'!

nameInSettingsDialog
    ^ 'XML-RPC Service'
! !

!XMLRPCService::ServicesSettingsAppl class methodsFor:'image specs'!

defaultIcon
    <resource: #programImage>

    ^ self xmlRpcIcon
!

worldIcon
    "This resource specification was automatically generated
     by the ImageEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the ImageEditor may not be able to read the specification."

    "
     self worldIcon inspect
     ImageEditor openOnClass:self andSelector:#worldIcon
     Icon flushCachedIcons
    "

    <resource: #image>

    ^Icon
        constantNamed:#'HTTPServerSettingsAppl::PluggableServicesSettingsAppl class worldIcon'
        ifAbsentPut:[(Depth8Image new) width: 22; height: 22; photometric:(#palette); bitsPerSample:(#(8 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@DB@0PC@0HBAP@@@@@@@@@@@@@@@@TFA0 IB ,KB00E@@@@@@@@@@@@@@4NC1@QD!!LKB08KE@T@@@@@@@@@@@TUE!!\PDA@XFQ(NF10]AP@@@@@@@@DF
BQ8PDA@PDAH_E $HE"@!!@@@@@@@BH $VDA@PB003IBT%I"\(JP@@@@@@J",WG!!@PB0,3H1@3B3L3L2@@@@@@@@P-DA@PD@,KB2L#B2L#L3LD@@@@@@@1L#L3
DQ@KB2L3G1<3H3L3A@@@@@@@M3 IBP 9B0,KL1<_L2L3DPP@@@@@@C<+BP$IP@,KB2L3G1<3DQDD@@@@@@AOS3HIBQ\KB0,KH1<_H1DTJP@@@@@@S$8@BQ8P
B0,KB2L_G1DQE@@@@@@@@D9N@EIRT%IRT ,KG1<QE@@@@@@@@@AOS0AQTUEQTUERT%HQDP@@@@@@@@@@S4<@TUEQTUEQTUERT @@@@@@@@@@@D=O@EEQTUD@
TUEQTUD@@@@@@@@@@@AOS0APTEEQTP@@@@@@@@@@@@@@@@@@S4<@@EAPTUEQT%IR@@@@@@@@@@@@@D=O@@@@TEAQTUEQTUEQTUAR@@@@@@@@@@@@@@ART%IR
T%IRT%H@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@a') ; colorMapFromArray:#[0 0 0 144 144 144 112 144 112 16 96 16 0 80 0 128 128 128 0 144 0 0 112 64 0 224 16 0 240 0 0 64 160 16 16 224 64 64 192 128 144 128 0 160 0 0 208 32 0 0 240 0 32 208 0 112 128 0 0 224 48 48 96 0 128 32 0 96 128 0 80 160 0 16 224 0 208 0 64 112 64 32 128 32 0 96 112 48 48 112 0 96 144 0 192 32 48 48 64 192 192 192 0 192 0 64 112 144 64 96 144 64 192 64 64 160 96 64 128 112 80 96 80 160 160 160 16 32 64 0 48 192 96 96 96 0 160 80 208 208 208 224 224 224 48 48 48 0 16 48 0 64 176 0 176 64 192 176 192 176 192 176 176 176 176 0 0 80 0 144 80 0 32 192 16 16 16 208 192 208 208 144 144 96 192 96 176 160 176 48 48 128 0 112 112 0 0 144 0 144 64 160 160 64 144 144 80 48 112 48 0 0 208 208 208 224 48 48 80 32 32 144 224 208 224 176 176 224 176 176 240 192 176 208 0 0 255 0 0 132 198 195 198 255 255 255 132 130 132]; mask:((ImageMask new) width: 22; height: 22; photometric:(#blackIs0); bitsPerSample:(#(1 )); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'C? @G?0@O?8@_?<@??>@??>@??>@??>@??>@??>@??>@??>@??<@??8@??0@?? @??0@?? @??<@??>@1?>@@?8@') ; yourself); yourself]
!

xmlRpcIcon
    "This resource specification was automatically generated
     by the ImageEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the ImageEditor may not be able to read the specification."

    "
     self xmlRpcIcon inspect
     ImageEditor openOnClass:self andSelector:#xmlRpcIcon
     Icon flushCachedIcons
    "

    <resource: #image>

    ^Icon
        constantNamed:'XMLRPCService::ServicesSettingsAppl class xmlRpcIcon'
        ifAbsentPut:[(Depth8Image new) width: 22; height: 22; photometric:(#palette); bitsPerSample:(#(8)); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@DB@0PC@0HBAP@@@@@@@@@@@@@@@@TFA0 IB ,KB00E@@@@@@@@@@@@@@4NC1@QD!!LKB08KE@T@@@@@@@@@@@TUE!!\PDA@XFQ(NF10]AP@@@@@@@@DF
BQ8PDA@PDAH_E $HE"@!!@@@@@@@BH $VDA@PB003IBT%I"\(JP@@@@@@J",WG!!@PB0,3H1@3B3L3L2@@@@@@@@P-DA@PD@,KB2L#B2L#L3LD@@@@@@@1L#L3
DQ@KB2L3G1<3H3L3A@@@@@@@M3 IBP 9B0,KL1<_L2L3DPP@@@@@@C<+BP$IP@,KB2L3G1<3DQDD@@@@@@AOS3HIBQ]ZUUUUUUUUUUUUUUUUUU(@S$8@BQ8P
UE)YU5ESTUQTTU]_UEQT@@ENS %RS%1TV5E[WU=QTUEWW5QTW@@@@T8IT$9\WEQQWE5QTUEQU5=\WE0@@@@AS$9NWE][TU-]TUQTTU]_WE1\@@@@@@DAS%]W
TUQQWUETUEEVTUEQU @@@@@@@@EWU5]WU5]WU5]WU5]WU5(@@@@@@@@@U5]^W%!!WW%9XU59^VE)Z@@@@@@@@@E]WW%]^XE9[W%]^XFA X@@@@@@@@@AQTU9^
VEE^W%!!QW%E XF@@@@@@@@@@TUE^TU9QW%EQTU9^VEEQ@@@a') ; colorMapFromArray:#[0 0 0 144 144 144 112 144 112 16 96 16 0 80 0 128 128 128 0 144 0 0 112 64 0 224 16 0 240 0 0 64 160 16 16 224 64 64 192 128 144 128 0 160 0 0 208 32 0 0 240 0 32 208 0 112 128 0 0 224 48 48 96 0 128 32 0 96 128 0 80 160 0 16 224 0 208 0 64 112 64 32 128 32 0 96 112 48 48 112 0 96 144 0 192 32 48 48 64 192 192 192 0 192 0 64 112 144 64 96 144 64 192 64 64 160 96 64 128 112 80 96 80 160 160 160 16 32 64 0 48 192 96 96 96 0 160 80 208 208 208 224 224 224 48 48 48 0 16 48 0 64 176 0 176 64 192 176 192 176 192 176 176 176 176 0 0 80 0 144 80 0 32 192 16 16 16 208 192 208 208 144 144 96 192 96 176 160 176 48 48 128 0 112 112 0 0 144 0 144 64 160 160 64 144 144 80 48 112 48 0 0 208 208 208 224 48 48 80 32 32 144 224 208 224 176 176 224 176 176 240 192 176 208 0 0 255 0 0 132 198 195 198 255 255 255 132 130 132 232 164 89 251 163 27 251 174 60 214 116 3 221 131 13 113 68 4 244 251 255 229 137 43 243 216 184 242 145 16 223 150 66 2 2 2 251 239 223 229 230 235]; mask:((ImageMask new) width: 22; height: 22; photometric:(#blackIs0); bitsPerSample:(#(1)); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'C? @G?0@O?8@_?<@??>@??>@??>@??>@??>@??>@??>@???8???8???8_??8O??8G??8A??8@??8@??8@??8@??8') ; yourself); yourself]
! !

!XMLRPCService::ServicesSettingsAppl class methodsFor:'interface specs'!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:XMLRPCService::ServicesSettingsAppl andSelector:#windowSpec
     XMLRPCService::ServicesSettingsAppl new openInterface:#windowSpec
     XMLRPCService::ServicesSettingsAppl open
    "

    <resource: #canvas>

    ^ 
     #(FullSpec
        name: windowSpec
        window: 
       (WindowSpec
          label: 'Services Settings'
          name: 'Services Settings'
          min: (Point 10 10)
          max: (Point 2048 1024)
          bounds: (Rectangle 0 0 640 611)
        )
        component: 
       (SpecCollection
          collection: (
           (VariableVerticalPanelSpec
              name: 'VariableVerticalPanel1'
              layout: (LayoutFrame 0 0 0 0 0 1 -32 1)
              showHandle: true
              snapMode: both
              handlePosition: left
              component: 
             (SpecCollection
                collection: (
                 (ViewSpec
                    name: 'Box2'
                    component: 
                   (SpecCollection
                      collection: (
                       (LabelSpec
                          label: 'Registered Methods'
                          name: 'Label2'
                          layout: (LayoutFrame 0 0 0 0 0 1 23 0)
                          style: (FontDescription helvetica bold roman 10)
                          translateLabel: true
                          adjust: left
                        )
                       (DataSetSpec
                          name: 'Table1'
                          layout: (LayoutFrame 0 0 23 0 0 1 0 1)
                          model: selectedServices
                          menu: serviceMenu
                          hasHorizontalScrollBar: true
                          hasVerticalScrollBar: true
                          dataList: serviceTableRowList
                          useIndex: false
                          columnHolder: serviceTableColumns
                          multipleSelectOk: true
                        )
                       )
                     
                    )
                  )
                 )
               
              )
              handles: (Any 1.0)
            )
           (HorizontalPanelViewSpec
              name: 'HorizontalPanel2'
              layout: (LayoutFrame 0 0 -32 1.0 0 1 0 1)
              horizontalLayout: center
              verticalLayout: center
              horizontalSpace: 3
              verticalSpace: 3
              component: 
             (SpecCollection
                collection: (
                 (ActionButtonSpec
                    label: 'Remove Service(s)'
                    name: 'Button3'
                    translateLabel: true
                    model: removeService
                    enableChannel: enableRemoveService
                    extent: (Point 200 22)
                  )
                 (ActionButtonSpec
                    label: 'Add Service...'
                    name: 'Button4'
                    translateLabel: true
                    model: addService
                    extent: (Point 200 22)
                  )
                 )
               
              )
            )
           )
         
        )
      )
! !

!XMLRPCService::ServicesSettingsAppl class methodsFor:'menu specs'!

serviceMenu
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:HTTPServerSettingsAppl::PluggableServicesSettingsAppl andSelector:#serviceMenu
     (Menu new fromLiteralArrayEncoding:(HTTPServerSettingsAppl::PluggableServicesSettingsAppl serviceMenu)) startUp
    "

    <resource: #menu>

    ^ 
     #(Menu
        (
         (MenuItem
            enabled: hasServiceSelected
            label: 'Browse'
            itemValue: browseService
            translateLabel: true
          )
         (MenuItem
            label: '-'
          )
         (MenuItem
            enabled: hasServiceSelected
            label: 'Inspect'
            itemValue: inspectService
            translateLabel: true
          )
         )
        nil
        nil
      )
! !

!XMLRPCService::ServicesSettingsAppl class methodsFor:'tableColumns specs'!

serviceTableColumns
    "This resource specification was automatically generated
     by the DataSetBuilder of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the DataSetBuilder may not be able to read the specification."

    "
     DataSetBuilder new openOnClass:HTTPServerSettingsAppl::PluggableServicesSettingsAppl andSelector:#serviceTableColumns
    "

    <resource: #tableColumns>

    ^#(
      (DataSetColumnSpec
         label: 'Method'
         labelButtonType: Button
         width: 220
         model: serviceName
         canSelect: false
       )
      (DataSetColumnSpec
         label: 'Receiver'
         labelButtonType: Button
         model: className
         canSelect: false
       )
      (DataSetColumnSpec
         label: 'Selector'
         labelButtonType: Button
         model: selector
         canSelect: false
       )
      )
! !

!XMLRPCService::ServicesSettingsAppl methodsFor:'actions'!

addService
    |serviceNameHolder classNameHolder selectorHolder 
     box serviceName className class selector|

    serviceNameHolder := (LastServiceName ? '') asValue.
    classNameHolder := (LastClassName ? '') asValue.
    selectorHolder := (LastSelector ? '') asValue.

    box := DialogBox new.
    box 
        addLabelledInputField:(resources string:'Method (name):')
        adjust:#right
        on:serviceNameHolder
        tabable:true
        separateAtX:0.5.
    box 
        addLabelledInputField:(resources string:'Class:')
        adjust:#right
        on:classNameHolder
        tabable:true
        separateAtX:0.5.
    box 
        addLabelledInputField:(resources string:'Selector:')
        adjust:#right
        on:selectorHolder
        tabable:true
        separateAtX:0.5.

    box addAbortAndOkButtons.
    box open.

    (box accepted) ifFalse:[^ self].

    (serviceName := serviceNameHolder value) isEmptyOrNil ifTrue:[^ self].
    (className := classNameHolder value) isEmptyOrNil ifTrue:[^ self].
    (selector := selectorHolder value) isEmptyOrNil ifTrue:[^ self].

    LastServiceName := serviceName.
    LastClassName := className.
    LastSelector := selector.

    selector := selector asSymbol.

    class := Smalltalk classNamed:className.
    class isNil ifTrue:[
        Dialog warn:(resources string:'No such class: %1' with:className).
        ^ self
    ].
    (class theMetaclass implements:selector) ifFalse:[
        Dialog warn:(resources string:'The %1-class does not implement %2 (class protocol)'
                        with:className allBold
                        with:selector allBold).
        ^ self
    ].

    serviceInstance registerService:serviceName class:class selector:selector.
    self readSettings
!

basicSaveSettings

    |serverServices server|

    server := self httpServerInstance.
    serverServices := server serviceList copy.
    self serviceInstanceList do:[:aService|
        self setChangedValuesForService:aService.
        (serverServices includes:aService) ifFalse:[
            aService registerServiceOn:server.    
        ].
    ].
    serverServices do:[:aService|
        (self serviceInstanceList includes:aService) ifFalse:[
            aService unRegisterServiceOn:server.    
        ].
    ].
    self updateModifiedChannel.
    super basicSaveSettings.
    self readSettings.
!

browseService
    (selectedServices value ? #()) do:[:each | 
        |rcvr cls|

        rcvr := each receiver.

        UserPreferences current systemBrowserClass
            openInClass:(rcvr class) selector:(each selector)
    ]
!

inspectService
    (selectedServices value ? #()) do:[:each | 
        |rcvr|

        rcvr := each receiver.
        rcvr inspect
    ]
!

removeService
    self selectedServices value do:[:each |
        serviceInstance removeService:(each serviceName) 
    ].
    self readSettings
! !

!XMLRPCService::ServicesSettingsAppl methodsFor:'aspects'!

enableRemoveService
    enableRemoveService isNil ifTrue:[
        enableRemoveService := false asValue.
    ].
    ^ enableRemoveService.
!

selectedServices

    selectedServices isNil ifTrue:[
        selectedServices := ValueHolder new.
        selectedServices addDependent:self.
    ].
    ^ selectedServices.
!

serviceTableRowList

    serviceTableRowList isNil ifTrue:[
        serviceTableRowList := List new.
        serviceTableRowList addDependent:self.
    ].
    ^ serviceTableRowList.
! !

!XMLRPCService::ServicesSettingsAppl methodsFor:'change & update'!

readSettings
    |entries|

    self serviceTableRowList removeAll.
    serviceInstance registeredMethods notNil ifTrue:[
        entries := OrderedCollection new.
        serviceInstance registeredMethods keysAndValuesDo:[:serviceName :entry |  
            entries add:(ServiceItem new 
                            serviceName:serviceName 
                            receiver:entry receiver 
                            selector:entry selector).
        ].
        entries sort:[:a :b | a serviceName < b serviceName].
        self serviceTableRowList contents:entries
    ].
"/    self openAndCloseServiceSettingsApplications.
"/    self selectedAvailableServiceChanged.
"/    self selectedServerServices value:nil.
"/    self selectedServerServiceChanged.
"/    self updateAvailableServiceList.
"/    self updateModifiedChannel.
    super readSettings.
!

selectedServicesChanged
    self enableRemoveService value:(self selectedServices value notEmptyOrNil)
!

update:something with:aParameter from:changedObject 
    changedObject == self selectedServices ifTrue:[
        self selectedServicesChanged.
        ^ self
    ].

    super 
        update:something
        with:aParameter
        from:changedObject
! !

!XMLRPCService::ServicesSettingsAppl methodsFor:'queries'!

hasServiceSelected
    ^ self selectedServices value notEmptyOrNil
!

hasUnsavedChanges

    ^ false
!

helpFilename
    ^ 'HTTPServer/xmlRpcServiceSettings.html'
! !

!XMLRPCService::ServicesSettingsAppl::ServiceItem methodsFor:'accessing'!

className
    receiver isBehavior ifTrue:[^ receiver name].
    ^ receiver classNameWithArticle
!

receiver
    ^ receiver
!

selector
    ^ selector
!

serviceName
    ^ serviceName
!

serviceName:serviceNameArg receiver:receiverArg selector:selectorArg 
    serviceName := serviceNameArg.
    receiver := receiverArg.
    selector := selectorArg.
! !

!XMLRPCService class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/server/XMLRPCService.st,v 1.19 2011/03/24 14:34:21 stefan Exp $'
! !
