"{ Package: 'stx:goodies/xml/xmlrpc/server/tests' }"

"{ NameSpace: XMLRPC }"

TestCase subclass:#XMLRPCServerTest
	instanceVariableNames:'newServer serverPort service'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Server-Test'
!


!XMLRPCServerTest class methodsFor:'service methods-interopEchoTest'!

echo:params
    ^ params first
!

registerInteropEchoTestsServiceMethodsWith:registrar
    registrar 
        registerMethod: 'interopEchoTests.echoString' 
        help:'Echoes string.'
        signature:#('string' 'string')
        receiver: self selector: #echo:.

    registrar registerService: 'interopEchoTests.echoStringArray' class: self selector: #echo:.
    registrar registerService: 'interopEchoTests.echoInteger' class: self selector: #echo:.
    registrar registerService: 'interopEchoTests.echoIntegerArray' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoFloat' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoFloatArray' class: self  selector: #echo:.
    registrar 
        registerMethod: 'interopEchoTests.echoStruct' 
        help:'Echoes struct.'
        signature:#('struct' 'struct')
        receiver: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoStructArray' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoValue' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoBase64' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.echoDate' class: self  selector: #echo:.
    registrar registerService: 'interopEchoTests.whichToolkit' class: self  selector: #whichToolkit:.
! !

!XMLRPCServerTest class methodsFor:'service methods-registration'!

registerServiceMethodsWith:registrar
    self registerValidator1ServiceMethodsWith:registrar.
    self registerInteropEchoTestsServiceMethodsWith:registrar.

    registrar enableIntrospectionProtocol
! !

!XMLRPCServerTest class methodsFor:'service methods-validator1'!

arrayOfStructsTest:params
    "invoked via xml-rpc from testsuite"

    |ret|

    ret := 0.
    params first do:[:each | 
        ret := ret + (each at:'curly')
    ].
    ^ ret
!

bigArrayTest:params
    "invoked via xml-rpc from testsuite"

    |collectionIn result|

    "/ Transcript show:'2 '; showCR:Timestamp now.
    collectionIn := params first.
    result := collectionIn collect:[:each | each squared].
    "/ Transcript show:'2 '; showCR:Timestamp now.
    ^ result
!

countTheEntities:params 
    "invoked via xml-rpc from testsuite"

    |ret string|

    ret := Dictionary new.
    string := params first.
    ret at:'ctLeftAngleBrackets' put:(string occurrencesOf:$<).
    ret at:'ctRightAngleBrackets' put:(string occurrencesOf:$>).
    ret at:'ctAmpersands' put:(string occurrencesOf:$&).
    ret at:'ctApostrophes' put:(string occurrencesOf:$').
    ret at:'ctQuotes' put:(string occurrencesOf:$").
    ^ ret
!

easyStructTest:params 
    "invoked via xml-rpc from testsuite"

    |ret|

    ret := 0.
    params first valuesDo:[:each | ret := ret + each ].
    ^ ret
!

echoStructTest:params 
    "invoked via xml-rpc from testsuite"

    ^ params first
!

manyTypesTest:params 
    "invoked via xml-rpc from testsuite"

    "simply return the decoded values in aCollection encoded"
    
    |arr x|

    x := 1.
    arr := Array new:6.
    params do:[:each | 
        arr at:x put:each.
        x := x + 1
    ].
    ^ arr
!

moderateSizeArrayCheck:params 
    "invoked via xml-rpc from testsuite"

    |arr|

    arr := params first.
    ^ (arr first) , (arr last)
!

nestedStructTest:params 
    "invoked via xml-rpc from testsuite"

    |struct ret|

    struct := params first.
    ret := 0.
    (((struct at:'2000') at:'04') at:'01') 
        valuesDo:[:each | ret := ret + each ].
    ^ ret
!

registerValidator1ServiceMethodsWith:registrar
    registrar registerService: 'validator1.arrayOfStructsTest' class: self selector: #arrayOfStructsTest:.
    registrar registerService: 'validator1.countTheEntities' class: self selector: #countTheEntities:.
    registrar registerService: 'validator1.echoStructTest' class: self selector: #echoStructTest:.
    registrar registerService: 'validator1.easyStructTest' class: self  selector: #easyStructTest:.
    registrar registerService: 'validator1.bigArrayTest' class: self  selector: #bigArrayTest:.
    registrar registerService: 'validator1.moderateSizeArrayCheck' class: self selector: #moderateSizeArrayCheck:.
    registrar registerService: 'validator1.nestedStructTest' class: self selector: #nestedStructTest:.
    registrar registerService: 'validator1.simpleStructReturnTest' class: self selector: #simpleStructReturnTest:.
    registrar registerService: 'validator1.manyTypesTest' class: self selector: #manyTypesTest:.
!

simpleStructReturnTest:params 
    "invoked via xml-rpc from testsuite"

    |num dict|

    dict := Dictionary new.
    num := params first.
    dict
        at:'times10' put:num * 10;
        at:'times100' put:num * 100;
        at:'times1000' put:num * 1000.
    ^ dict
! !

!XMLRPCServerTest methodsFor:'running'!

setUp
    |server registrar|

    newServer := false.
    Smalltalk isSmalltalkX ifTrue:[
        server := HTTPServer runningServers detect:[:s | s hasServiceForLink:'/xmlrpc'] ifNone:nil.
        server isNil ifTrue:[
            server := HTTPServer serverOnPort:8082.
            XMLRPCService new registerServiceOn:server.
        ].
        serverPort := server port.
        registrar := server serviceForLink:'/xmlrpc'.
    ] ifFalse:[
        (HttpService servicesNamed:'xmlrpc') 
            ifEmpty:[
                XMLRPCHttpModule
                    start;
                    setDebugMode.
                newServer := true
            ].
        registrar := XMLRPCServerRequest.
    ].

    self class registerServiceMethodsWith:registrar

    "
     self new setUp
    "
!

tearDown
    Smalltalk isSmalltalkX ifFalse:[
        #( 
            'validator1.arrayOfStructsTest' 
            'validator1.countTheEntities' 
            'validator1.echoStructTest' 
            'validator1.easyStructTest' 
            'validator1.moderateSizeArrayCheck' 
            'validator1.nestedStructTest' 
            'validator1.simpleStructReturnTest' 
            'validator1.manyTypesTest' 
        ) do:[:xmlrpcService | 
            XMLRPCServerRequest removeService:xmlrpcService asSymbol
        ].

        newServer ifTrue:[
            XMLRPCHttpModule stop
        ]
    ].
! !

!XMLRPCServerTest methodsFor:'tests'!

testBigArray
    |p reply|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  
    "/ Transcript show:'1 '; showCR:Timestamp now.
    reply := p invokeMethod:'validator1.bigArrayTest' with:(1 to:16r7FFF) asArray.
    "/ Transcript show:'1 '; showCR:Timestamp now.
    self assert:(reply = ((1 to:16r7FFF) asArray collect:[:each | each squared])).
!

testDictionary
    |d p r|

    d := Dictionary new.
    d at:'true' put:true.
    d at:'Markus' put:'Fritsche'.
    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.
    r := p invokeMethod:'validator1.echoStructTest' withArgs:{ d }.
    self assert:(r = d)
!

testEcho
    |p ret|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  

    ret := p invokeMethod:'interopEchoTests.echoString' with:'hello'.
    self assert:(ret = 'hello').

    ret := p invokeMethod:'interopEchoTests.echoStringArray' with:#('a' 'b' 'c').
    self assert:(ret = #('a' 'b' 'c')).

    ret := p invokeMethod:'interopEchoTests.echoInteger' with:123.
    self assert:(ret = 123).

    ret := p invokeMethod:'interopEchoTests.echoIntegerArray' with:#(1 2 3).
    self assert:(ret = #(1 2 3)).

    ret := p invokeMethod:'interopEchoTests.echoInteger' with:2147483647.
    self assert:(ret = 2147483647).
    ret := p invokeMethod:'interopEchoTests.echoInteger' with:-2147483648.
    self assert:(ret = -2147483648).

    ret := p invokeMethod:'interopEchoTests.echoFloat' with:1.234.
    self assert:(ret = 1.234).

    ret := p invokeMethod:'interopEchoTests.echoFloatArray' with:#(1.0 2.0 3.0).
    self assert:(ret = #(1.0 2.0 3.0)).

    ret := p invokeMethod:'interopEchoTests.echoDate' with:(Timestamp now).
    ret asTimestamp.
!

testHallo
    |p reply|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  
    reply := p invokeMethod:'validator1.echoStructTest' withArgs:#( 'Hallo' ).

    self assert:(reply = 'Hallo' ).
!

testIntrospection
    "validate correct faultCode;
     see http://xmlrpc-epi.sourceforge.net/specs/rfc.fault_codes.php"

    |p methods info caps helpText sig|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  

    methods := p invokeMethod:'system.listMethods'.
    self assert:(methods includes:'system.listMethods').
    self assert:(methods includes:'system.methodHelp').
    self assert:(methods includes:'system.getCapabilities').
    self assert:(methods includes:'system.versionInfo').
    "/ methods inspect.

    info := p invokeMethod:'system.versionInfo'.
    "/ info inspect.

    helpText := p invokeMethod:'system.methodHelp' with:'system.methodHelp'.
    self assert:(helpText notEmptyOrNil).

    sig := p invokeMethod:'system.methodSignature' with:'system.methodHelp'.
    self assert:(sig = #( #('string' 'string') )).

    sig := p invokeMethod:'system.methodSignature' with:'system.methodSignature'.
    self assert:(sig = #( #('array' 'string') )).

    sig := p invokeMethod:'system.methodSignature' with:'system.listMethods'.
    self assert:(sig = #( #('array') )).

    caps := p invokeMethod:'system.getCapabilities'.
    self assert:(caps includesKey:'xmlrpc').
    self assert:(caps includesKey:'faults_interop').
    self assert:(caps includesKey:'introspection').
    self assert:(caps includesKey:'system.multicall').
    "/ caps inspect.
!

testMultiCall
    |p reply multiCall|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  

    multiCall := OrderedCollection new.
    multiCall add:(Dictionary new
                        at:'methodName' put:'interopEchoTests.echoInteger';
                        at:'params'     put:(Array with:1);
                        yourself).
    multiCall add:(Dictionary new
                        at:'methodName' put:'interopEchoTests.echoFloat';
                        at:'params'     put:(Array with:1.0);
                        yourself).
    multiCall add:(Dictionary new
                        at:'methodName' put:'interopEchoTests.echoFloatArray';
                        at:'params'     put:(Array with:#( 1.0 2.0 3.0 ));
                        yourself).

    reply := p invokeMethod:'system.multicall' with:multiCall.

    self assert:(reply = (Array with:1 with:1.0 with:#( 1.0 2.0 3.0 ))).
!

testUnimplementedMethod
    "validate correct faultCode;
     see http://xmlrpc-epi.sourceforge.net/specs/rfc.fault_codes.php"

    |p faultCode reply|

    p := XMLRPCProxy new url:('http://localhost:',serverPort printString,'/xmlrpc') asUrl.  
    XMLRPCException handle:[:ex |
        faultCode := ex faultCode.
    ] do:[
        reply := p invokeMethod:'unimplemented' withArgs:#( 'Hallo' ).
    ].
    self assert:(faultCode = -32601 ).   
! !

!XMLRPCServerTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/server/tests/XMLRPCServerTest.st,v 1.7 2010/03/26 01:04:43 cg Exp $'
! !
