"{ Package: 'stx:goodies/xml/xmlrpc/tests' }"

"{ NameSpace: XMLRPC }"

XMLRPCSpyTest subclass:#XMLRPCDecoderTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Data-Test'
!

XMLRPCDecoderTest comment:'Test the XMLRPC Decoder.
XMLRPCDecoderTest new testDecoder'
!


!XMLRPCDecoderTest methodsFor:'Testing'!

suite
    ^ (TestSuite named:'XMLRPCDecoderTests') 
        addTest:(XMLRPCDecoderTest selector:#testDecoder)
!

testDecoder
    "Test the decoder."
    
    |e d coll dict|

    e := XMLRPCEncoder new.
    d := XMLRPCDecoder new.
    dict := Dictionary new.
    dict at:'a' put:'A'.
    dict at:'b' put:'B'.
    coll := OrderedCollection new.
    coll add:17.
    coll add:'Some String'.
    coll add:true.
    coll add:false.
    coll add:3.14159265359.
    coll add:XMLRPCDateTime now.
    coll add:#( 1 2 3 'ima g' ).
    coll add:dict.
    coll do:[:item | 
        |xdom er dr|

        er := self xmlIfy:(e encode:item).
        xdom := YAXO::XMLDOMParser parseDocumentFrom:er readStream.
        dr := d decode:(xdom elements at:1).
         "Transcript show: 'Decoded ' , er , ' to ' , dr asString.
         Transcript cr."
        self 
            assertEquals:item
            to:dr
            msg:'Decoding ' , item className
    ]

    "Modified: / 14-07-2011 / 22:29:44 / cg"
!

testDecoder_V3
    "Test the decoder."
    
    |e d coll|

    e := XMLRPCEncoder new.
    d := XMLRPCDecoder new.
    coll := OrderedCollection new.
    coll add:nil.
    coll add:16r100000000.
    coll do:[:item | 
        |xdom er dr|

        er := self xmlIfy:(e encode:item).
        xdom := YAXO::XMLDOMParser parseDocumentFrom:er readStream.
        dr := d decode:(xdom elements at:1).
         "Transcript show: 'Decoded ' , er , ' to ' , dr asString.
         Transcript cr."
        self 
            assertEquals:item
            to:dr
            msg:'Decoding ' , item className
    ]

    "Created: / 14-07-2011 / 22:29:39 / cg"
!

testJSONDecoder
    "Test the decoder."
    
    |request responseData|

    request := JSON_RPCServerRequest fromRequest: '{"jsonrpc": "2.0", "method": "subtract", "params": [42, 23], "id": 1}'.
    self assert:(request methodCall = 'subtract').
    self assert:((request at:1) = 42).
    self assert:((request at:2) = 23).
    self assert:(request id = 1).
    self assert:(request needsResponse).

    responseData := request encodeResponse:'hello world'.
!

testXMLRPCDecoder
    "Test the decoder."
    
    |request responseData|

    request := XML_RPCServerRequest fromRequest: '
<?xml version="1.0" encoding="ISO-8859-1"?>
<methodCall>
   <methodName>sample.sum</methodName>
   <params>
       <param>
            <value><int>17</int></value>
       </param>
       <param>
        <value><int>13</int></value>
       </param>
   </params>
</methodCall>'.
    self assert:(request methodCall = 'sample.sum').
    self assert:((request at:1) = 17).
    self assert:((request at:2) = 13).

    responseData := request encodeResponse:30.
! !

!XMLRPCDecoderTest methodsFor:'private'!

xmlIfy:t 
    ^ '<?xml version="1.0"?><param>' , t , '</param>'
! !

!XMLRPCDecoderTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/tests/XMLRPCDecoderTest.st,v 1.7 2011/07/14 20:32:05 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/tests/XMLRPCDecoderTest.st,v 1.7 2011/07/14 20:32:05 cg Exp $'
! !
