"{ Package: 'stx:goodies/xml/xmlrpc/tests' }"

"{ NameSpace: XMLRPC }"

XMLRPCSpyTest subclass:#XMLRPCTest
	instanceVariableNames:'proxy'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Data-Test'
!


!XMLRPCTest methodsFor:'Testing'!

performCall:method arg:a 
    "Perform method invocation and make sure the result is the same as the  
     passed-in value."
    
    |r|

    r := proxy invokeMethod:method withArgs:(Array with:a).
     "Transcript show: 'Invoked ' , method , ' with ' , a asString , ' which returned ' , r asString;
     cr."
    self 
        assertEquals:a
        to:r
        msg:'Failed on method ' , method.
!

setUp
    "Set up the test suite:"
    
    |url|

    "/ url := Url absoluteFromText:'http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC'.
    url := URL fromString:'http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC'.
    url := URL fromString:'http://bleu.west.spy.net/geo/RPC2'.
    proxy := XMLRPCProxy new url:url.
     "Transcript show: 'Set up XMLRPCTest';
     cr."
    ^ self
!

suite
    ^ (TestSuite named:'XMLRPCTests')
        addTest:(XMLRPCTest selector:#testInt);
        addTest:(XMLRPCTest selector:#testDouble);
        addTest:(XMLRPCTest selector:#testString);
        addTest:(XMLRPCTest selector:#testBoolean);
        addTest:(XMLRPCTest selector:#testArray);
        addTest:(XMLRPCTest selector:#testStruct);
        addTest:(XMLRPCTest selector:#testDate);
        addTest:(XMLRPCTest selector:#testBinary)
!

testArray
    "Test arrays."
    
    self performCall:'echo.echoArray' arg:#( 'A' 1 3 2.525 )
!

testBoolean
    "Test booleans."
    
    self performCall:'echo.echoBoolean' arg:true.
    self performCall:'echo.echoBoolean' arg:false
!

testDateTime
    "Test DateTime."
    
    self performCall:'echo.echoDate' arg:(XMLRPCDateTime now)
!

testDouble
    "Test doubles."
    
    self performCall:'echo.echoDouble' arg:3.14159265358979
!

testGetCurrentTime
    |url proxy r|

    url := URL fromString:'http://www.xmlrpc.com/RPC2'.
    proxy := XMLRPCProxy new url:url.

    r := proxy invokeMethod:'currentTime.getCurrentTime'.
    Transcript showCR:'The current time on time.xmlrpc.com is ',r asTimestamp printString.          
!

testGetZipLookup
    |url proxy r|

    url := URL fromString:'http://bleu.west.spy.net/geo/RPC2'.
    proxy := XMLRPCProxy new url:url.

    r := proxy invokeMethod:'zipcodes.lookupZip' withArgs:(Array with:95051).
    self assert:(r at:'city') = 'Santa Clara'.
    self assert:(r at:'state') = 'California'.
    self assert:(r at:'state_code') = 'CA'.
!

testInt
    "Test ints."
    
    self performCall:'echo.echoInt' arg:19.
!

testString
    "Test strings."
    
    self performCall:'echo.echoString' arg:'This is a test message.'.
    self performCall:'echo.echoString' arg:'Embedded XML <see>.'.
!

testStruct
    "Test structs."
    
    |d|

    d := Dictionary new.
    d at:'a' put:'Value for A'.
    d at:'b' put:'Value for B'.
    self performCall:'echo.echoStruct' arg:d
!

xtestBinary
    "Test Binary data."
    "/ cg: this interface seems to be no longer supported by the server

    |r p msg|

    msg := 'Test for base64.'.
    p := OrderedCollection new.
    p add:msg readStream.
    r := proxy invokeMethod:'echo.echoByte' withArgs:p.
    self 
        assertEquals:msg
        to:(r contents asString)
        msg:'Binary (base64) method failed.'
!

xtestGetPointInfo
    |url proxy r|

    url := URL fromString:'http://bleu.west.spy.net/geo/RPC2'.
    proxy := XMLRPCProxy new url:url.

    r := proxy invokeMethod:'geo.getPointInfo' withArgs:(Array with:47.377677 with:8.526544).
! !

!XMLRPCTest class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/tests/XMLRPCTest.st,v 1.5 2010/03/26 01:04:28 cg Exp $'
! !
