"{ Package: 'stx:goodies/xml/xpath' }"

"{ NameSpace: XML }"

XPathExpression subclass:#XPathBinaryExpression
	instanceVariableNames:'operator argument1 argument2 valueBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-XPath'
!

XPathBinaryExpression class instanceVariableNames:'operators'

"
 No other class instance variables are inherited by this class.
"
!


!XPathBinaryExpression class methodsFor:'instance creation'!

operator: op with: arg1 with: arg2
    ^(operators at: op ifAbsent: [self error: ('Not implemented yet %1' bindWith: op)]) copy
	operator: op with: arg1 with: arg2
! !

!XPathBinaryExpression class methodsFor:'class initialization'!

initialize
    "XPathBinaryExpression initialize"

    operators := Dictionary new.
    self initializeBoolean.
    self initializeComparison.
    self initializeNumeric
!

initializeBoolean
    operators at: #and put: (self new
    			operator: #and with: nil with: nil;
    			valueBlock: [:exp :ns || b1 b2 |
    					b1 := (exp arg1 xpathEvalIn: ns) xpathAsBoolean.
    					b2 := (exp arg2 xpathEvalIn: ns) xpathAsBoolean.
    					b1 & b2]).
    operators at: #or put: (self new
    			operator: #or with: nil with: nil;
    			valueBlock: [:exp :ns || b1 b2 |
    					b1 := (exp arg1 xpathEvalIn: ns) xpathAsBoolean.
    					b2 := (exp arg2 xpathEvalIn: ns) xpathAsBoolean.
    					b1 | b2]).
!

initializeComparison
    operators at: #= put: (self new
                        operator: #= with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareEquality: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 = v2]]).
    operators at: #'!!=' put: (self new
                        operator: #'!!=' with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareEquality: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 ~= v2]]).
    operators at: #< put: (self new
                        operator: #< with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareOrder: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 < v2]]).
    operators at: #> put: (self new
                        operator: #> with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareOrder: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 > v2]]).
    operators at: #<= put: (self new
                        operator: #<= with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareOrder: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 <= v2]]).
    operators at: #>= put: (self new
                        operator: #>= with: nil with: nil;
                        valueBlock: [:exp :ns |
                                        (exp arg1 xpathEvalIn: ns)
                                                xpathCompareOrder: (exp arg2 xpathEvalIn: ns)
                                                using: [:v1 :v2 | v1 >= v2]]).
!

initializeNumeric
    operators at: #+ put: (self new
    			operator: #+ with: nil with: nil;
    			valueBlock: [:exp :ns |
    					(exp arg1 xpathEvalIn: ns) xpathAsNumber +
    						(exp arg2 xpathEvalIn: ns) xpathAsNumber]).
    operators at: #- put: (self new
    			operator: #- with: nil with: nil;
    			valueBlock: [:exp :ns |
    					(exp arg1 xpathEvalIn: ns) xpathAsNumber -
    						(exp arg2 xpathEvalIn: ns) xpathAsNumber]).
    operators at: #* put: (self new
    			operator: #* with: nil with: nil;
    			valueBlock: [:exp :ns |
    					(exp arg1 xpathEvalIn: ns) xpathAsNumber *
    						(exp arg2 xpathEvalIn: ns) xpathAsNumber]).
    operators at: #div put: (self new
    			operator: #div with: nil with: nil;
    			valueBlock: [:exp :ns |
    					(exp arg1 xpathEvalIn: ns) xpathAsNumber /
    						(exp arg2 xpathEvalIn: ns) xpathAsNumber]).
    operators at: #mod put: (self new
    			operator: #mod with: nil with: nil;
    			valueBlock: [:exp :ns |
    					((exp arg1 xpathEvalIn: ns) xpathAsNumber rem:
    						(exp arg2 xpathEvalIn: ns) xpathAsNumber)]).
! !

!XPathBinaryExpression methodsFor:'accessing'!

arg1
    ^argument1
!

arg2
    ^argument2
!

enumerate: aBlock
    super enumerate: aBlock.
    self arg1 enumerate: aBlock.
    self arg2 enumerate: aBlock.
! !

!XPathBinaryExpression methodsFor:'initialize'!

operator: op with: arg1 with: arg2
    operator := op.
    argument1 := arg1.
    argument2 := arg2.
!

valueBlock: aBlock
    valueBlock := aBlock
! !

!XPathBinaryExpression methodsFor:'matching'!

baseValueIn: aNodeContext
    ^valueBlock value: self value: aNodeContext
! !

!XPathBinaryExpression methodsFor:'printing'!

printTestOn: aStream
    argument1 printOn: aStream.
    aStream space; nextPutAll: operator; space.
    argument2 printOn: aStream.
! !

!XPathBinaryExpression methodsFor:'testing'!

xpathMayRequireNodeSet
    ^self arg1 xpathMayRequireNodeSet or: [self arg2 xpathMayRequireNodeSet]
!

xpathMayRequireNodeSetTopLevel
    (#(#'+' #'-' #'*' #'div' #'mod') includes: operator)
        ifTrue: [^true].
    (#(#'=' #'!!=' #'<' #'>' #'<=' #'>=' #'|' #'and' #'or') includes: operator)
        ifTrue: [^self arg1 xpathMayRequireNodeSet or: [self arg2 xpathMayRequireNodeSet]].
    self shouldImplement
!

xpathMayRequireSort
    ^self arg1 xpathMayRequireSort or: [self arg2 xpathMayRequireSort]
!

xpathMayRequireSortTopLevel
    (#(#'+' #'-' #'*' #'div' #'mod') includes: operator)
        ifTrue: [^true].
    (#(#'=' #'!!=' #'<' #'>' #'<=' #'>=' #'|' #'and' #'or') includes: operator)
        ifTrue: [^self arg1 xpathMayRequireSort or: [self arg2 xpathMayRequireSort]].
    self shouldImplement
! !

!XPathBinaryExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xpath/XPathBinaryExpression.st,v 1.2 2003/08/05 20:30:48 stefan Exp $'
! !

XPathBinaryExpression initialize!
