"{ Package: 'stx:goodies/xml/xpath' }"

"{ NameSpace: XML }"

Object subclass:#XPathExpression
	instanceVariableNames:'predicates child'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-XPath'
!


!XPathExpression class methodsFor:'instance creation'!

new
    ^super new initialize
! !

!XPathExpression class methodsFor:'coercing'!

notANumber
    ^FloatD nan
!

stringToNumber: aString
    | s foundDigit numerator denominator ch |
    s := aString readStream.
    s skipSeparators.
    foundDigit := false.
    numerator := 0.
    denominator := 1.
    [(ch := s next) notNil and: [ch isDigit]]
    	whileTrue:
    		[numerator := numerator * 10 + ch digitValue.
    		foundDigit := true].
    ch = $.
    	ifTrue: [[(ch := s next) notNil and: [ch isDigit]]
    		whileTrue:
    			[numerator := numerator * 10 + ch digitValue.
    			denominator := denominator * 10.
    			foundDigit := true]].
    (ch == nil or: [ch isSeparator]) ifFalse: [^self notANumber].
    s skipSeparators.
    s atEnd ifFalse: [^self notANumber].
    foundDigit ifFalse: [^self notANumber].
    ^(numerator / denominator) + 0.0d
! !

!XPathExpression methodsFor:'accessing'!

addPredicate: aPredicate
    predicates := predicates copyWith: aPredicate
!

asUnion
    ^XPathUnion new add: self
!

child
    ^child
!

child: aStep
    child := aStep
!

enumerate: aBlock
    aBlock value: self.
    predicates do: [:p | p enumerate: aBlock].
    child enumerate: aBlock.
!

predicates
    ^predicates
!

usedVarNames
    | list |
    list := OrderedCollection new.
    self enumerate: [:exp |
    	(exp isKindOf: XPathVariable)
    		ifTrue: [list add: exp name]].
    ^list
! !

!XPathExpression methodsFor:'initialize'!

initialize
    predicates := #().
    child := XPathTerminator new.
    child parent: self
! !

!XPathExpression methodsFor:'matching'!

baseValueIn: aNodeContext
    self subclassResponsibility
!

isMatchFor: anXmlNode
    ^self isLocalMatchFor: anXmlNode
!

match: aNodeContext
    | base |
    base := self.
    [base child == nil]
    	whileFalse: [base := base child].
    ^base simpleMatchFor: aNodeContext node isComplex: false do:
    	[:root :complex || ns found |
    	complex not or:
    		[ns := aNodeContext copy add: root; index: 1.
    		found := false.
    		self valueIn: ns do: [:nd | nd == aNodeContext node ifTrue: [found := true]].
    		found]]
!

simpleMatchFor: anXmlNode isComplex: complex do: aBlock
    ^self subclassResponsibility
!

valueIn: aNodeContext do: aBlock
    | result |
    result := self baseValueIn: aNodeContext.
    result xpathIsNodeSet
        ifTrue:
                [1 to: predicates size do: [:i |
                        result := result select: (predicates at: i)].
                result reset.
                [result next notNil]
                        whileTrue: [child valueIn: result do: aBlock]]
        ifFalse: [(predicates isEmpty and: [child isTerminator])
                ifTrue: [aBlock value: result]
                ifFalse: [self error: 'The expression %1 does not represent a node set']].
!

xpathEvalIn:aNodeContext 
    "This is private protocol--see #xpathValueIn: for the client protocol"
    
    |nc ncValue|

    nc := Association new.
    self valueIn:aNodeContext
        do:[:x | x addToXPathHolder:nc for:aNodeContext ].
    ncValue := nc value.
    ncValue isNil ifTrue:[ ^ aNodeContext copy ] ifFalse:[ ^ ncValue ]
!

xpathValueFor: anXmlNode variables: vars
    ^self xpathValueIn: (XPathNodeContext new add: anXmlNode; index: 1; variables: vars)
!

xpathValueIn: aNodeContext
    "This is public protocol only--see #xpathEvalIn: for internal clients"

    aNodeContext baseNode: aNodeContext node.
    ^self xpathEvalIn: aNodeContext
! !

!XPathExpression methodsFor:'printing'!

completeChildPrintOn: aStream
    self completePrintOn: aStream.
!

completePrintOn: aStream
    self printTestOn: aStream.
    predicates do: [:p | self printPredicate: p on: aStream].
    self child isTerminator
    	ifFalse: [aStream nextPut: $/].
    self child completeChildPrintOn: aStream.
!

printOn: aStream
    self completePrintOn: aStream
!

printPredicate: p on: aStream
    aStream nextPutAll: '['.
    aStream print: p.
    aStream nextPutAll: ']'
!

printTestOn: aStream
    self subclassResponsibility
! !

!XPathExpression methodsFor:'testing'!

isXPathAncestor
    "ST/X: does this step refer to an ancestor node?"

    ^ false
!

isXPathRoot
    "ST/X: does this step refer to the root node?"

    ^ false
!

xpathMayRequireNodeSet
    ^self subclassResponsibility
!

xpathMayRequireNodeSetTopLevel
    ^self subclassResponsibility
!

xpathMayRequireSort
    ^self subclassResponsibility
!

xpathMayRequireSortTopLevel
    ^self subclassResponsibility
! !

!XPathExpression class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xpath/XPathExpression.st,v 1.3 2004/03/12 19:20:10 stefan Exp $'
! !
