"{ Package: 'stx:goodies/xml/xpath' }"

"{ NameSpace: XML }"

XPathExpression subclass:#XPathFunction
	instanceVariableNames:'name arguments valueBlock requiresSort requiresNodeSet'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-XPath'
!

XPathFunction class instanceVariableNames:'functions'

"
 No other class instance variables are inherited by this class.
"
!


!XPathFunction class methodsFor:'class initialization'!

baseFunctions
    ^functions
!

initialize
    "XPathFunction initialize"

    functions == nil
        ifTrue: [functions := Dictionary new]
        ifFalse: [functions keys do: [:k | functions removeKey: k]].
    self initializeBoolean.
    self initializeStrings.
    self initializeNodeSets.
    self initializeNumeric.
!

initializeBoolean
    functions at: 'boolean' put: (self new
    		name: 'boolean';
    		valueBlock: [:fn :ns || ns2 |
    				fn arguments size > 1 ifTrue: [self error: 'boolean() only takes one argument'].
    				ns2 := fn arguments size = 1
    					ifTrue: [fn arguments first xpathEvalIn: ns]
    					ifFalse: [ns asSingleNode].
    				ns2 xpathAsBoolean]).
    functions at: 'not' put: (self new
    		name: 'not';
    		valueBlock: [:fn :ns |
    			(fn arguments first xpathEvalIn: ns) xpathAsBoolean not]).
    functions at: 'true' put: (true).
    functions at: 'false' put: (false).
!

initializeNodeSets
    functions at: 'count' put: (self new
                requiresNodeSet: true;
                name: 'count';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size > 1 ifTrue: [self error: 'count() only takes one argument'].
                                ns2 := fn arguments size = 1
                                        ifTrue: [fn arguments first xpathEvalIn: ns]
                                        ifFalse: [ns asSingleNode].
                                ns2 xpathIsNodeSet ifFalse: [self error: 'count() requires a nodeset as an argument'].
                                ns2 size]).
    functions at: 'position' put: (self new
                requiresNodeSet: true;
                requiresSort: true;
                name: 'position';
                valueBlock: [:fn :ns |
                                fn arguments size > 0 ifTrue: [self error: 'position() cannot take any arguments'].
                                ns index]).
    functions at: 'last' put: (self new
                requiresNodeSet: true;
                name: 'last';
                valueBlock: [:fn :ns |
                                fn arguments size > 0 ifTrue: [self error: 'last() cannot take any arguments'].
                                ns size]).
    functions at: 'local-name' put: (self new
                name: 'local-name';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size > 1 ifTrue: [self error: 'local-name() only takes one argument'].
                                ns2 := fn arguments size = 1
                                        ifTrue: [fn arguments first xpathEvalIn: ns]
                                        ifFalse: [ns asSingleNode].
                                ns2 xpathIsNodeSet ifFalse: [self error: 'local-name() requires a nodeset as an argument'].
                                ns2 documentOrder; index: 1.
                                (ns2 node isElement or: [ns2 node isAttribute])
                                        ifTrue: [ns2 node tag type]
                                        ifFalse: ['']]).
    functions at: 'namespace-uri' put: (self new
                name: 'namespace-uri';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size > 1 ifTrue: [self error: 'namespace-uri() only takes one argument'].
                                ns2 := fn arguments size = 1
                                        ifTrue: [fn arguments first xpathEvalIn: ns]
                                        ifFalse: [ns asSingleNode].
                                ns2 xpathIsNodeSet ifFalse: [self error: 'namespace-uri() requires a nodeset as an argument'].
                                ns2 documentOrder; index: 1.
                                (ns2 node isElement or: [ns2 node isAttribute])
                                        ifTrue: [ns2 node tag namespace]
                                        ifFalse: ['']]).
    functions at: 'name' put: (self new
                name: 'name';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size > 1 ifTrue: [self error: 'name() only takes one argument'].
                                ns2 := fn arguments size = 1
                                        ifTrue: [fn arguments first xpathEvalIn: ns]
                                        ifFalse: [ns asSingleNode].
                                ns2 xpathIsNodeSet ifFalse: [self error: 'name() requires a nodeset as an argument'].
                                ns2 size == 0 ifTrue:[''] ifFalse:[
                                ns2 documentOrder; index: 1.
                                (ns2 node isElement or: [ns2 node isAttribute])
                                        ifTrue: [ns2 node tag asString]
                                        ifFalse: ['']]]).
    functions at: 'id' put: (self new
                name: 'id';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size ~= 1 ifTrue: [self error: 'id() only takes one argument'].
                                ns2 := fn arguments first xpathEvalIn: ns.
                                ns2 := ns2 xpathAsString.
                                ns copy add: (ns node document atID: ns2 ifAbsent: [self error: 'ID "', ns2, '" not found'])]).
    functions at: 'current' put: (self new
                requiresNodeSet: true;
                name: 'current';
                valueBlock: [:fn :ns |
                                fn arguments size > 0 ifTrue: [self error: 'current() takes no arguments'].
                                ns copy add: ns baseNode; index: 1]).
!

initializeNumeric
    functions at: 'number' put: (self new
    		name: 'number';
    		valueBlock: [:fn :ns || ns2 |
    				fn arguments size > 1 ifTrue: [self error: 'number() only takes one argument'].
    				ns2 := fn arguments size = 1
    					ifTrue: [fn arguments first xpathEvalIn: ns]
    					ifFalse: [ns asSingleNode].
    				ns2 xpathAsNumber]).
    functions at: 'round' put: (self new
    		name: 'round';
    		valueBlock: [:fn :ns |
    				fn arguments size ~= 1 ifTrue: [self error: 'round() only takes one argument'].
    				(fn arguments first xpathEvalIn: ns) xpathAsNumber rounded]).
    functions at: 'floor' put: (self new
    		name: 'floor';
    		valueBlock: [:fn :ns |
    				fn arguments size ~= 1 ifTrue: [self error: 'floor() only takes one argument'].
    				(fn arguments first xpathEvalIn: ns) xpathAsNumber floor]).
    functions at: 'ceiling' put: (self new
    		name: 'ceiling';
    		valueBlock: [:fn :ns |
    				fn arguments size ~= 1 ifTrue: [self error: 'ceiling() only takes one argument'].
    				(fn arguments first xpathEvalIn: ns) xpathAsNumber ceiling]).
    functions at: 'sum' put: (self new
    		name: 'sum';
    		valueBlock: [:fn :ns |
    				fn arguments size ~= 1 ifTrue: [self error: 'sum() only takes one argument'].
    				(fn arguments first xpathEvalIn: ns) sum]).
!

initializeStrings
    functions at: 'string' put: (self new
                name: 'string';
                valueBlock: [:fn :ns || ns2 |
                                fn arguments size > 1 ifTrue: [self error: 'string() only takes one argument'].
                                ns2 := fn arguments size = 1
                                        ifTrue: [fn arguments first xpathEvalIn: ns]
                                        ifFalse: [ns asSingleNode].
                                ns2 xpathAsString]).
    functions at: 'concat' put: (self new
                name: 'concat';
                valueBlock: [:fn :ns || s |
                        s := ''.
                        fn arguments do: [:exp | s := s, (exp xpathEvalIn: ns) xpathAsString].
                        s]).
    functions at: 'contains' put: (self new
                name: 'contains';
                valueBlock: [:fn :ns || s1 s2 i |
                        fn arguments size = 2 ifFalse: [self error: 'contains() takes two arguments'].
                        s1 := (fn arguments at: 1) xpathEvalIn: ns.
                        s2 := (fn arguments at: 2) xpathEvalIn: ns.
                        i := s1 xpathAsString findString: s2 xpathAsString startingAt: 1.
                        i > 0]).
    functions at: 'translate' put: (self new
                name: 'translate';
                valueBlock: [:fn :ns || s1 s2 s3 |
                        fn arguments size = 3 ifFalse: [self error: 'translate() takes three arguments'].
                        s1 := (fn arguments at: 1) xpathEvalIn: ns.
                        s2 := (fn arguments at: 2) xpathEvalIn: ns.
                        s3 := (fn arguments at: 3) xpathEvalIn: ns.
                        (self
                                        translate: s1 xpathAsString
                                        from: s2 xpathAsString
                                        to: s3 xpathAsString)]).
    functions at: 'string-length' put: (self new
                name: 'string-length';
                valueBlock: [:fn :ns || s1 |
                        fn arguments size < 2 ifFalse: [self error: 'string-length() takes no more than 1 argument'].
                        s1 := (fn arguments size = 0
                                ifTrue: [ns asSingleNode]
                                ifFalse: [fn arguments first xpathEvalIn: ns]) xpathAsString.
                        s1 size]).
    functions at: 'substring' put: (self new
                name: 'substring';
                valueBlock: [:fn :ns || s1 i1 i2 |
                        (fn arguments size between: 2 and: 3) ifFalse: [self error: 'substring() takes two or three arguments'].
                        s1 := ((fn arguments at: 1) xpathEvalIn: ns) xpathAsString.
                        i1 := ((fn arguments at: 2) xpathEvalIn: ns) xpathAsNumber.
                        i2 := fn arguments size = 2
                                ifTrue: [1.0e10]
                                ifFalse: [((fn arguments at: 3) xpathEvalIn: ns) xpathAsNumber].
                        i2 := (i1 + i2 - 1) rounded min: s1 size.
                        i1 := i1 rounded max: 1.
                        (s1 copyFrom: i1 to: i2)]).
    functions at: 'starts-with' put: (self new
                name: 'starts-with';
                valueBlock: [:fn :ns || s1 s2 |
                        fn arguments size = 2 ifFalse: [self error: 'starts-with() takes two arguments'].
                        s1 := ((fn arguments at: 1) xpathEvalIn: ns) xpathAsString.
                        s2 := ((fn arguments at: 2) xpathEvalIn: ns) xpathAsString.
                        (s1 findString: s2 startingAt: 1) = 1]).
    functions at: 'substring-before' put: (self new
                name: 'substring-before';
                valueBlock: [:fn :ns || s1 s2 i |
                        fn arguments size = 2 ifFalse: [self error: 'substring-before() takes two arguments'].
                        s1 := ((fn arguments at: 1) xpathEvalIn: ns) xpathAsString.
                        s2 := ((fn arguments at: 2) xpathEvalIn: ns) xpathAsString.
                        i := s1 findString: s2 startingAt: 1.
                        (i = 0 ifTrue: [''] ifFalse: [s1 copyFrom: 1 to: i - 1])]).
    functions at: 'substring-after' put: (self new
                name: 'substring-after';
                valueBlock: [:fn :ns || s1 s2 i |
                        fn arguments size = 2 ifFalse: [self error: 'substring-after() takes two arguments'].
                        s1 := ((fn arguments at: 1) xpathEvalIn: ns) xpathAsString.
                        s2 := ((fn arguments at: 2) xpathEvalIn: ns) xpathAsString.
                        i := s1 findString: s2 startingAt: 1.
                        (i = 0 ifTrue: [''] ifFalse: [s1 copyFrom: i+s2 size to: s1 size])]).
    functions at: 'normalize-space' put: (self new
                name: 'normalize-space';
                valueBlock: [:fn :ns || s1 |
                                fn arguments size > 1 ifTrue: [self error: 'normalize-space() only takes one argument'].
                                s1 := (fn arguments size = 0
                                        ifTrue: [ns asSingleNode]
                                        ifFalse: [fn arguments first xpathEvalIn: ns]) xpathAsString.
                                self normalizeWhitespace: s1]).
! !

!XPathFunction class methodsFor:'function implementations'!

normalizeWhitespace: aString
    | ch str buffer space |
    str := aString readStream.
    buffer := String new writeStream.
    space := false.
    [str skipSeparators; atEnd]
    	whileFalse:
    		[space ifTrue: [buffer space].
    		[(ch := str next) notNil and: [ch isSeparator not]]
    			whileTrue: [buffer nextPut: ch].
    		space := true].
    ^buffer contents
!

translate: base from: src to: dest
    | dir result c |
    dir := IdentityDictionary new.
    src size to: 1 by: -1 do: [:i |
    	dir at: (src at: i) put: (i > dest size ifTrue: [nil] ifFalse: [dest at: i])].
    result := (String new: base size) writeStream.
    1 to: base size do: [:i |
    	c := base at: i.
    	c := dir at: c ifAbsent: [c].
    	c == nil ifFalse: [result nextPut: c]].
    ^result contents
! !

!XPathFunction methodsFor:'accessing'!

addArgument: arg
    arguments := arguments copyWith: arg
!

arguments
    ^arguments
!

enumerate: aBlock
    super enumerate: aBlock.
    self arguments do: [:i | i enumerate: aBlock].
!

name
    ^name
!

name: nm
    name := nm
!

requiresNodeSet
    ^requiresNodeSet
!

requiresNodeSet: bool
    requiresNodeSet := bool
!

requiresSort
    ^requiresSort
!

requiresSort: bool
    requiresSort := bool
! !

!XPathFunction methodsFor:'initialize'!

initialize
    super initialize.
    arguments := #().
    requiresSort := false.
    requiresNodeSet := false
!

valueBlock: aBlock
    valueBlock := aBlock
! !

!XPathFunction methodsFor:'matching'!

baseValueIn: aNodeContext
    ^valueBlock value: self value: aNodeContext
! !

!XPathFunction methodsFor:'printing'!

printTestOn: aStream
    aStream nextPutAll: name, '('.
    arguments do: [:a | aStream print: a] separatedBy: [aStream nextPutAll: ','].
    aStream nextPutAll: ')'.
! !

!XPathFunction methodsFor:'testing'!

answersNumber
    ^#('sum' 'round' 'count' 'last' 'position' 'string-length' 'floor' 'ceiling') includes: self name
!

xpathMayRequireNodeSet
    ^requiresNodeSet
    	or: [arguments inject: false into: [:b :arg | b or: [arg xpathMayRequireNodeSet]]]
!

xpathMayRequireNodeSetTopLevel
    ^self answersNumber or: [self xpathMayRequireNodeSet]
!

xpathMayRequireSort
    ^requiresSort
    	or: [arguments inject: false into: [:b :arg | b or: [arg xpathMayRequireSort]]]
!

xpathMayRequireSortTopLevel
    ^self answersNumber or: [self xpathMayRequireSort]
! !

!XPathFunction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xpath/XPathFunction.st,v 1.4 2006/09/16 18:10:20 stefan Exp $'
! !

XPathFunction initialize!
