"{ Package: 'stx:goodies/xml/xpath' }"

"{ NameSpace: XML }"

Object subclass:#XPathNodeContext
	instanceVariableNames:'documentOrder nodes index node baseNode variables'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-XPath'
!


!XPathNodeContext class methodsFor:'instance creation'!

new
    ^super new initialize
! !

!XPathNodeContext methodsFor:'accessing'!

baseNode
    ^baseNode
!

baseNode: aNode
    baseNode := aNode
!

documentOrder
    documentOrder := true
!

index
    ^index
!

index: n
    nodes class == IdentitySet ifTrue:[
        self checkSorted.
    ].
"/ at: will fail if n is illegal
"/    (n between:1 and:nodes size) ifFalse: [self error: 'Index out of bounds'].
    index := n.
    node := nodes at: n
!

indexForNode: aNode
    self checkSorted.
    index := nodes identityIndexOf: aNode.
    index = 0 ifTrue: [self error: 'No such node found in the list'].
    node := aNode
!

inverseDocumentOrder
    documentOrder := false
!

node
    ^node
!

size
    ^nodes size
!

sort: aBlock
    nodes := nodes asSortedCollection: aBlock
!

sortedNodes
    ^nodes asSortedCollection: [:n1 :n2 | n1 precedes: n2]
!

unsortedNodes
    ^nodes
!

variables
    ^variables
!

variables: aDictionary
    variables := aDictionary
! !

!XPathNodeContext methodsFor:'adding'!

add: aNode
    nodes add: aNode
!

addAll: collection
    nodes addAll: collection
!

addNodeSet: nodeSet
    nodes addAll: nodeSet unsortedNodes
! !

!XPathNodeContext methodsFor:'coercing'!

xpathAsBoolean
    ^ self size ~~ 0
!

xpathAsNumber
    ^self xpathAsString xpathAsNumber
!

xpathAsString
    | size list |

    size := nodes size.
    size == 0 ifTrue: [^''].
    size == 1 ifTrue:[
        list := nodes
    ] ifFalse:[
        list := nodes asSortedCollection: [:n1 :n2 | n1 precedes: n2].
    ].
    ^ list first xpathStringData
! !

!XPathNodeContext methodsFor:'comparing'!

xpathCompareEquality: aData using: aBlock
    aData isString
        ifTrue: [^nodes contains: [:nd | aBlock value: nd xpathStringData value: aData]].
    aData isNumber
        ifTrue: [^nodes contains: [:nd | aBlock value: nd xpathStringData xpathAsNumber value: aData]].
    aData isBoolean
        ifTrue: [^nodes contains: [:nd | aBlock value: nd xpathStringData xpathAsBoolean value: aData]].
    aData xpathIsNodeSet
        ifTrue: [^nodes contains: [:nd1 |
                        aData unsortedNodes contains: [:nd2 |
                                aBlock value: nd1 xpathStringData value: nd2 xpathStringData]]].
    self error: ('Can''t compare a %1 with a node set' bindWith: aData class printString)
!

xpathCompareOrder:aData using:aBlock 
    |v|

    aData xpathIsNodeSet ifTrue:[
        ^ self unsortedNodes contains:[:nd1 | 
                |v|

                v := nd1 xpathStringData xpathAsNumber.
                aData unsortedNodes 
                    contains:[:nd2 | aBlock value:v value:nd2 xpathStringData xpathAsNumber ]
            ]
    ].
    v := aData xpathAsNumber.
    ^ self unsortedNodes 
        contains:[:nd | aBlock value:nd xpathStringData xpathAsNumber value:v ].
! !

!XPathNodeContext methodsFor:'copying'!

asSingleNode
    ^self copy
    	add: self node;
    	yourself
!

copy
    ^self shallowCopy postCopy
!

postCopy
    nodes := IdentitySet new.
    index := 0.
    node := nil.
    documentOrder := true.
! !

!XPathNodeContext methodsFor:'enumerating'!

addToXPathHolder: anAssociation for: aNodeContext
    self error: 'Should not happen--a NodeSet is being processed as if it were a single XML node'
!

select:aPattern 
    |result|

    aPattern xpathMayRequireSortTopLevel ifTrue:[
        self checkSorted
    ] ifFalse:[
        self checkOrdered
    ].
    result := self copy.
    self reset.
    [ self next notNil ] whileTrue:[ |val|
        val := aPattern xpathEvalIn:self.
        (val isNumber ifTrue:[ val = index ] ifFalse:[ val xpathAsBoolean ]) ifTrue:[
            result add:self node
        ]
    ].
    ^ result
!

selectMatch: aPattern
    | result |
    result := self copy.
    self reset.
    [self next notNil]
        whileTrue:
                [(aPattern match: self)
                        ifTrue: [result add: self node]].
    ^result
! !

!XPathNodeContext methodsFor:'functions'!

sum
    ^(nodes inject: 0.0d into: [:i :nd | i + nd xpathStringData xpathAsNumber])
! !

!XPathNodeContext methodsFor:'initialize'!

checkOrdered
    nodes class == IdentitySet
    	ifTrue: [nodes := nodes asArray].
!

checkSorted

    |size newNodes|

    "nodes is an IndentityDictionary until sorted"
    nodes isSequenceable ifTrue:[
        ^ self.
    ].

    nodes := nodes asArray.

    "sort for small collections"
    size := nodes size.
    size == 1 ifTrue:[
        ^ self.
    ].
"/ This code works, but is it faster?
"/    size < 4 ifTrue:[
"/        documentOrder
"/            ifTrue:  [nodes sort:[:n1 :n2 | n1 precedes: n2]]
"/            ifFalse: [nodes sort:[:n1 :n2 | n2 precedes: n1]].
"/        ^ self.
"/    ].

    "use tree positions for large collections"
    newNodes := nodes collect:[:eachNode | eachNode treePosition].
    newNodes sortWith:nodes.
    "sorting with default sortBlock and reversing is faster
     than sorting with explicit sortBlock"
    documentOrder ifFalse:[nodes reverse].
!

ensureSorted
    nodes class == IdentitySet
    	ifFalse: [self error: 'This collection was already sorted once and may not be in correct sort order'].
    self checkSorted
!

initialize
    nodes := IdentitySet new.
    documentOrder := true
! !

!XPathNodeContext methodsFor:'printing'!

prettyPrintOn: aStream

    nodes do:[:eachNode| eachNode prettyPrintOn:aStream] separatedBy:[aStream cr].
!

printOn: aStream

    nodes do:[:eachNode| eachNode printOn:aStream] separatedBy:[aStream space].
! !

!XPathNodeContext methodsFor:'streaming'!

atEnd
    ^index = nodes size
!

next
    index == nodes size ifTrue:[^ nil].
    (index == 0 and:[nodes class == IdentitySet]) ifTrue:[
        self checkSorted.
    ].
        
    index := index + 1.
    node := nodes at: index.
!

reset
    index := 0.
    node := nil
! !

!XPathNodeContext methodsFor:'testing'!

contains: aBlock
    | match |
    match := nodes detect: aBlock ifNone: [].
    ^match notNil
!

xpathIsNodeSet
    ^true
! !

!XPathNodeContext class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xpath/XPathNodeContext.st,v 1.7 2004/10/26 08:34:50 stefan Exp $'
! !
