"{ Package: 'stx:goodies/xml/xpath' }"

"{ NameSpace: XML }"

Object subclass:#XPathParser
	instanceVariableNames:'pushBack hereChar stack buffer source token tokenType xmlNode
		functions'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-XPath'
!

XPathParser class instanceVariableNames:'nodeTypes typeTable'

"
 No other class instance variables are inherited by this class.
"
!


!XPathParser class methodsFor:'examples'!

examples
    "XPathParser examples"

    | samples |
    samples := #('child::para' 'child::*' 'child::text()' 'child::node()'
                'attribute::name' 'attribute::*' 'descendant::para' 'ancestor::div'
                'ancestor-or-self::div' 'descendant-or-self::para' 'self::para'
                'child::chapter/descendant::para' 'child::*/child::para' '/'
                '/descendant::para' '/descendant::olist/child::item'
                'child::para[position()=1]' 'child::para[position()=last()]'
                'child::para[position()=last()-1]' 'child::para[position()>1]'
                'following-sibling::chapter[position()=1]'
                'preceding-sibling::chapter[position()=1]'
                '/descendant::figure[position()=42]'
                '/child::doc/child::chapter[position()=5]/child::section[position()=2]'
                'child::para[attribute::type="warning"]'
                'child::para[attribute::type=''warning''][position()=5]'
                'child::para[position()=5][attribute::type="warning"]'
                'child::chapter[child::title=''Introduction'']'
                'child::chapter[child::title]' 'child::*[self::chapter or self::appendix]'
                'child::*[self::chapter or self::appendix][position()=last()]').

    ^ samples collect: [:str | (self new parse: str as: #locationPath)]
! !

!XPathParser class methodsFor:'parsing'!

parse: stringOrStream as: construct
    ^self new parse: stringOrStream as: construct
! !

!XPathParser class methodsFor:'private'!

baseTable
    | newTable c selector |
    newTable := Array new: 255.
    1 to: 255 do: [ :each |
        c := each asCharacter.
        selector := #xDefault.
        c isSeparator ifTrue: [ selector := #xDelimiter ].
        c isDigit ifTrue: [ selector := #xDigit ].
        c isLetter ifTrue: [ selector := #xLetter ].
        c == $. ifTrue: [ selector := #xPeriod ].
        c == $" ifTrue: [ selector := #xDoubleQuote ].
        c == $' ifTrue: [ selector := #xSingleQuote ].
        c == $} ifTrue: [ selector := #xEndOfExpression ].
        (':*/+-@=!!<>|' includes: c) ifTrue: [ selector := #xBinary ].
        ('$[]()' includes: c) ifTrue: [ selector := #xCharacter ].
        newTable at: each put: selector
    ].
    ^newTable
!

initialize
    "Compute the character type, reserved word tables, and
    keyword flag from the information associated with each method."
    typeTable := self baseTable.
    nodeTypes := Dictionary new.
    XPathStep allSubclassesDo: [:cls |
        cls axisNames do: [:nm | nodeTypes at: nm put: cls]]
!

nodeTypes
    ^nodeTypes
!

typeTable
    ^typeTable
! !

!XPathParser methodsFor:'construction'!

abbreviatedDescendant
    token = #'/'
        ifTrue: 
                [stack add: (XPathDescendantNode new
                                        axisName: 'descendant-or-self';
                                        baseTest: (XPathTypedNodeTest new type: #node)).
                ^true].
    ^false
!

arg: a1 op: operator arg: a2
    ^XPathBinaryExpression
        operator: operator asSymbol with: a1 with: a2
!

axis: axisName test: test
    | stepClass step |
    stepClass := self class nodeTypes
                        at: axisName
                        ifAbsent: [self error: ('%1 is not an axis' bindWith: axisName)].
    step := stepClass new.
    step axisName: axisName.            "Some classes represent multiple axes, and must be told which"
    step baseTest: test.
    ^step
!

connectParent: parent child: child
    | p |
    p := parent.
    [p child isTerminator] whileFalse: [p := p child].
    p child: child.
    child parent: p.
    ^parent
!

function: aName
    ^self functionNamed: aName
!

function: aFunction arg: anArgument
    ^aFunction addArgument: anArgument
!

isNodeType
    | typeName argument ok |
    typeName := stack at: stack size - 1.
    argument := stack at: stack size.
    ok := typeName = 'processing-instruction'
        ifTrue: [true]
        ifFalse: [(#('node' 'text' 'comment') includes: typeName)
                ifTrue: [argument == nil]
                ifFalse: [false]].
"       ok ifFalse: [stack removeLast]."
    ^ok
!

nodeTestQualifier: qualifier type: typeName
    | ns |
    ns := self namespaceAt: qualifier.
    ^XPathTaggedNodeTest new
        namespace: ns;
        type: typeName
!

nodeTestType: typeName
    ^XPathTaggedNodeTest new
        type: typeName
!

nodeTypeTest: typeName arg: argument
    ^XPathTypedNodeTest new
        type: typeName;
        value: argument
!

number: aValue
    ^aValue
!

selfOrParent
    token = #'.'
        ifTrue: 
                [self scanToken.
                stack add: (XPathCurrentNode new
                                        axisName: 'self';
                                        baseTest: (XPathTypedNodeTest new type: #node)).
                ^true].
    token = #'..'
        ifTrue: 
                [self scanToken.
                stack add: (XPathParentNode new
                                        axisName: 'parent';
                                        baseTest: (XPathTypedNodeTest new type: #node)).
                ^true].
    ^false
!

step: step predicate: predicate
    ^step addPredicate: predicate; yourself
!

string: aValue
    ^aValue
!

union: path1 with: path2
    ^path1 asUnion add: path2
!

variable: aName
    ^XPathVariable new name: aName
! !

!XPathParser methodsFor:'expressions'!

additiveExpr
    | op arg1 arg2 |
    self multiplicativeExpr ifFalse: [ ^false ].
    [
        op := token.
        (token = #+ or: [ token = #- ]) ] whileTrue: [

        self scanToken.
        self multiplicativeExpr ifFalse: [ self error: 'error in operand of +/-' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

andExpr
    | op arg1 arg2 |
    self equalityExpr ifFalse: [ ^false ].
    [
        op := token.
        token = 'and' ] whileTrue: [

        self scanToken.
        self equalityExpr ifFalse: [ self error: 'error in operand of and' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

equalityExpr
    | op arg1 arg2 |
    self relationalExpr ifFalse: [ ^false ].
    [
        op := token.
        (token = #= or: [ token = #'!!=' ]) ] whileTrue: [

        self scanToken.
        self relationalExpr ifFalse: [ self error: 'error in operand of =/!!=' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

expression
    ^self orExpr
!

filterExpr
    | step predicate |
    self primaryExpr ifFalse: [ ^false ].
    [ self predicate ] whileTrue: [
        predicate := stack removeLast.
        step := stack removeLast.
        stack addLast: (self step: step predicate: predicate)
    ].
    ^true
!

multiplicativeExpr
    | op arg1 arg2 |
    self unaryExpr ifFalse: [ ^false ].
    [
        op := token.
        (token = #* or: [ token = 'div' or: [ token = 'mod' ]]) ] whileTrue: [

        self scanToken.
        self unaryExpr ifFalse: [ self error: 'error in operand of */div/mod' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

orExpr
    | op arg1 arg2 |
    self andExpr ifFalse: [ ^false ].
    [
        op := token.
        token = 'or' ] whileTrue: [

        self scanToken.
        self andExpr ifFalse: [ self error: 'error in operand of or' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

pathExpr
    | parent child |
    self locationPath ifTrue: [ ^true ].
    self filterExpr ifFalse: [ ^false ].

    token = #/ ifFalse: [ ^true ].
    self scanToken.
    self relativeLocationPath ifFalse: [ ^false ].

    child := stack removeLast.
    parent := stack removeLast.
    stack addLast: (self connectParent: parent child: child).
    ^true
!

primaryExpr
    | function arg |
    token = $$ ifTrue: [
        self scanToken.
        tokenType = #word ifFalse: [ ^false ].
        stack addLast: (self variable: token).
        self scanToken.
        ^true
    ].
    token = $( ifTrue: [
        self scanToken.
        self expression ifFalse: [ ^false ].
        token = $) ifFalse: [ ^false ].
        self scanToken.
        ^true
    ].
    tokenType = #string ifTrue: [
        stack addLast: (self string: token).
        self scanToken.
        ^true
    ].
    tokenType = #number ifTrue: [
        stack addLast: (self number: token).
        self scanToken.
        ^true
    ].

    tokenType = #word ifTrue: [
        function := token.
        stack addLast: (self function: token).

        self scanToken.
        token = $( ifFalse: [ ^false ].
        self scanToken.
        token = $) ifFalse: [
            [
                self expression ifFalse: [ ^false ].
                arg := stack removeLast.
                function := stack removeLast.
                stack addLast: (self function: function arg: arg).
                token = $,
            ] whileTrue.
            token = $) ifFalse: [ ^false ].
        ].

        self scanToken.
        ^true
    ].

    ^false
!

relationalExpr
    | op arg1 arg2 |
    self additiveExpr ifFalse: [ ^false ].
    [
        op := token.
        (token = #< or: [ token = #> or: [ token = #<= or: [ token = #>= ]]]) ] whileTrue: [

        self scanToken.
        self additiveExpr ifFalse: [ self error: 'error in operand of relational operator' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self arg: arg1 op: op arg: arg2)
    ].
    ^true
!

unaryExpr
    token = #- ifTrue: [ |unaryExpression|
        self scanToken.
        unaryExpression := self unaryExpr.
        unaryExpression ifTrue:[ stack addLast: (self negated: stack removeLast) ].
	^ unaryExpression.
    ].

    ^self unionExpr
!

unionExpr
    | arg1 arg2 |
    self pathExpr ifFalse: [ ^false ].
    [
        token = #| ] whileTrue: [

        self scanToken.
        self pathExpr ifFalse: [ self error: 'error in operand of |' ].

        arg2 := stack removeLast.
        arg1 := stack removeLast.
        stack addLast: (self union: arg1 with: arg2)
    ].
    ^true
! !

!XPathParser methodsFor:'initialize'!

functions: aDictionary
    functions := aDictionary
! !

!XPathParser methodsFor:'paths'!

absoluteLocationPath
    | root child |
    self scanToken. "#/"
    stack addLast: XPathRoot new.
    hereChar isNil ifTrue: [ ^true ].
    self relativeLocationPath ifFalse: [ ^false ].
    child := stack removeLast.
    root := stack removeLast.
    stack addLast: (self connectParent: root child: child).
    ^true
!

anyStep
    self selfOrParent ifTrue: [ ^true ].
    self abbreviatedDescendant ifTrue: [ ^true ].
    ^self stepWithPredicates
!

locationPath
    token = #/ ifTrue: [ ^self absoluteLocationPath ].
    ^self relativeLocationPath
!

nodeTest
    | word word2 arg |
    token = #* ifTrue: [
        self scanToken.
        stack addLast: (self nodeTestType: #*).
        ^true
    ].

    tokenType = #word ifFalse: [ ^false ].
    word := token.
    self scanToken.

    token = $( ifTrue: [
        self scanToken.
        token = $)
            ifTrue: [ self scanToken. arg := nil ]
            ifFalse: [
                tokenType = #string ifFalse: [ ^false ].
                arg := token.
                self scanToken.
                token = $) ifFalse: [ ^false ].
            ].

        stack addLast: (self nodeTypeTest: word arg: arg).
        ^true
    ].

    (self peekFor: #':') ifFalse: [
        self verifyNotFunction ifTrue: [
            stack addLast: (self nodeTestType: word).
            ^true
        ]
    ].

    self scanToken.
    token = #* ifTrue: [
        self scanToken.
        stack addLast: (self nodeTestQualifier: word).
        ^true
    ].

    tokenType = #word ifFalse: [ ^false ].
    word2 := token.
    self scanToken.
    stack addLast: (self nodeTestQualifier: word type: word2)
!

predicate
    (token = $[ and: [
        self scanToken. self expression and: [
            token = $] ]]) ifTrue: [ self scanToken. ^true ].

    ^false
!

relativeLocationPath
    | parent child |
    self anyStep ifFalse: [ ^false ].
    [ token = #/ ] whileTrue: [
        self scanToken.
        self anyStep ifFalse: [ ^false ].
        child := stack removeLast.
        parent := stack removeLast.
        stack addLast: (self connectParent: parent child: child).
    ].
    ^true
!

stepWithPredicates
    | axis word step predicate position |
    axis := 'child'.
    tokenType = #word ifTrue: [
        axis := token.
        self scanToken.

        "???"
        (token = $( and: [
            (#('node' 'text' 'comment' 'processing-instruction')
                includes: axis) not ]) ifTrue: [

            pushBack := #character -> $(.
            tokenType := #word.
            token := axis.
            ^false
        ].

        (self peekFor: #'::') ifFalse: [
            tokenType isNil ifFalse: [ pushBack := tokenType -> token ].
            tokenType := #word.
            token := axis.
            axis := 'child'
        ].
    ].
    token = #@ ifTrue: [
        self scanToken.
        axis := 'attribute'
    ].
    self nodeTest ifFalse: [ ^false ].

    stack addLast: (self axis: axis test: stack removeLast).

    [ self predicate ] whileTrue: [
        predicate := stack removeLast.
        step := stack removeLast.
        stack addLast: (self step: step predicate: predicate).
    ].

    ^true
! !

!XPathParser methodsFor:'private'!

functionNamed: fName
    functions == nil ifTrue: [functions := XPathFunction baseFunctions].
    ^(functions at: fName ifAbsent: [self error: ('Not implemented yet: %1()' bindWith: fName)]) copy

    "Modified: / 03-08-2004 / 15:58:27 / stefan"
!

namespaceAt: aQualifier
    | elm ns |
    elm := xmlNode.
    [elm isDocument]
        whileFalse:
                [ns := elm namespaceAt: aQualifier.
                ns == nil ifFalse: [^ns].
                elm := elm parent].
    aQualifier = 'xml' ifTrue: [^XML_URI].
    self error: ('No namespace binding found for namespace qualifier "%1".'
                bindWith: aQualifier)
!

peekFor: trialValue 
    "Test to see if tokenType matches aType and token equals trialValue. If so, 
    advance to the next token"

    ^token = trialValue
        ifTrue: 
                [self scanToken.
                true]
        ifFalse: [false]
!

scanToken
    | type |
    pushBack isNil ifFalse: [
        tokenType := pushBack key.
        token := pushBack value.
        pushBack := nil.
        ^ self
    ].
    hereChar isNil ifFalse: [
        "figures out what the next token could be based on hereChar then performs
        the needed function"
        type := self class typeTable at: hereChar asInteger ifAbsent: [ #xDefault ].
        self perform: type.
        ^ self
    ].
    tokenType := token := nil
!

step
    hereChar := source next
!

unexpectedError
    ^self error: 'syntax error'

    "Modified: / 03-08-2004 / 16:39:40 / stefan"
!

verifyNotFunction
    ^token ~= $(
!

xBinary
    | char test |
    tokenType := #binary.
    char := hereChar.
    self step.
    char = $. ifTrue: [self halt].
    hereChar == nil
        ifTrue: [token := Symbol internCharacter: char]
        ifFalse:
                [test := String with: char with: hereChar.
                (#('::' '<=' '>=' '!!=') includes: test)
                        ifTrue: [self step. token := Symbol intern: test]
                        ifFalse: [token := Symbol internCharacter: char]]
!

xCharacter
    tokenType := #character.
    token := hereChar.
    self step
!

xDefault
    self error: 'invalid character ', hereChar asString
!

xDelimiter
    source skipSeparators.
    self step.
    self scanToken
!

xDigit
    "form a number"

    | numerator denominator |
    tokenType := #number.
    numerator := 0.
    denominator := 1.
    [hereChar notNil and: [hereChar isDigit]]
        whileTrue:
                [numerator := numerator * 10 + hereChar digitValue.
                self step].
    hereChar = $. ifFalse: [^token := numerator + 0.0].
    self step.
    [hereChar notNil and: [hereChar isDigit]]
        whileTrue:
                [numerator := numerator * 10 + hereChar digitValue.
                denominator := denominator * 10.
                self step].
    token := (numerator / denominator) + 0.0.
!

xDoubleQuote
    "collect string"

    | char |
    buffer reset.
    [(char := source next) == $"]
        whileFalse:
                [char == nil ifTrue: [^self offEnd: 'Unmatched comment quote'].
                buffer nextPut: char].
    tokenType := #string.
    token := buffer contents.

    "Shorten the buffer if it got unreasonably large."
    buffer position > 200 ifTrue: [buffer := WriteStream on: (String new: 40)].

    self step
!

xEndOfExpression
    tokenType := #endOfExpression.
    token := nil
!

xLetter
    "form a word, keyword, or reserved word"

    | char |
    buffer reset.
    buffer nextPut: hereChar.
    [
        char := source next.
        char notNil and:[
            char isAlphaNumeric or: [ char == $- ]
        ]
    ] whileTrue:[
        buffer nextPut: char
    ].
    tokenType := #word.
    hereChar := char.
    token := buffer contents
!

xPeriod
    "form a number"

    | numerator denominator |
    self step.
    hereChar = $.
        ifTrue:
                [self step.
                token := #'..'.
                tokenType := #binary.
                ^self].
    (hereChar notNil and: [hereChar isDigit])
        ifFalse:
                [token := #'.'.
                tokenType := #binary.
                ^self].
    tokenType := #number.
    numerator := 0.
    denominator := 1.
    [hereChar notNil and: [hereChar isDigit]]
        whileTrue:
                [numerator := numerator * 10 + hereChar digitValue.
                denominator := denominator * 10.
                self step].
    token := (numerator / denominator) + 0.0.
!

xSingleQuote
    "collect string"

    | char |
    buffer reset.
    [(char := source next) == $']
        whileFalse:
                [char == nil ifTrue: [^self offEnd: 'Unmatched comment quote'].
                buffer nextPut: char].
    tokenType := #string.
    token := buffer contents.

    "Shorten the buffer if it got unreasonably large."
    buffer position > 200 ifTrue: [buffer := WriteStream on: (String new: 40)].
    self step
! !

!XPathParser methodsFor:'public'!

atEndOfExpression
    ^tokenType == #endOfExpression
!

parse: string as: construct
    self source: string.
    self perform: construct.
    self pastEnd ifFalse: [self error: 'Extra characters which could not be translated at end of stream'].
    ^self result
!

pastEnd
    ^hereChar == nil
!

result
    stack size = 1 ifFalse: [self error: 'Parsing logic error, incorrect number of values on the stack'].
    ^stack first
!

source: streamOrString
    buffer := String new writeStream.
    stack := OrderedCollection new.
    streamOrString isString ifTrue: [ 
        source := XPathReadStream on: streamOrString asString.
    ] ifFalse:[
        source := streamOrString. "then it *should* be a stream'"
    ].

    self step; scanToken
!

xmlNode: aNode
    xmlNode := aNode
! !

!XPathParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xpath/XPathParser.st,v 1.7 2006/09/16 18:10:20 stefan Exp $'
! !

XPathParser initialize!
