"{ Package: 'stx:goodies/xml/xpath' }"
!

!Boolean methodsFor:'converting'!

xpathAsBoolean
    ^self
! !

!Boolean methodsFor:'converting'!

xpathAsNumber
    ^self ifTrue: [1] ifFalse: [0]
! !

!Boolean methodsFor:'converting'!

xpathAsString
    ^self printString
! !

!Boolean methodsFor:'xml support'!

xpathCompareEquality:aData using:aBlock
    aData xpathIsNodeSet ifTrue:[
	^ aData unsortedNodes
	    contains:[:nd2 | aBlock value:self value:nd2 xpathStringData xpathAsBoolean ]
    ].
    ^ aBlock value:self value:aData xpathAsBoolean
! !

!Boolean methodsFor:'xml support'!

xpathIsBoolean
    ^true
! !

!Number methodsFor:'converting'!

xpathAsBoolean
    ^self ~= 0
! !

!Number methodsFor:'converting'!

xpathAsNumber
    ^self
! !

!Number methodsFor:'converting'!

xpathAsString
    | str n num delta n2 found |
"       self isZero ifTrue: [^'0']."
    self isFinite not
	ifTrue: [^self printString].
    str := (String new: 8) writeStream.
    self < 0 ifTrue: [str nextPut: $-].
    n := self abs + 0.0d0.
    num := n truncated.
    str print: num.
    num + 0.0d = n ifTrue:
	[^str contents].
    delta := 1/10.
    found := false.
    [[n2 := num + delta.
    n2 < n] whileTrue:
	[num := n2].
    num = n
	ifTrue: [found := true]
	ifFalse: [n2 = n
		ifTrue: [num := n2. found := true]].
    found]
	whileFalse: [delta := delta / 10].
    num = n ifFalse: [self halt].
    str nextPut: $..
    num := num - num truncated.
    [num = 0]
	whileFalse:
		[num := num * 10.
		str print: num truncated.
		num := num - num truncated].
    ^str contents
! !

!Number methodsFor:'xml support'!

xpathCompareEquality:aData using:aBlock
    aData xpathIsNodeSet ifTrue:[
	^ aData unsortedNodes
	    contains:[:nd2 | aBlock value:self value:nd2 xpathStringData xpathAsNumber ]
    ].
    ^ aBlock value:self xpathAsNumber value:aData
! !

!Number methodsFor:'xml support'!

xpathMayRequireNodeSetTopLevel
    ^true
! !

!Number methodsFor:'xml support'!

xpathMayRequireSortTopLevel
    ^true
! !

!Object methodsFor:'xml support'!

addToXPathHolder: anAssociation for: aNodeContext
    |value|

    value := anAssociation value.
    value isNil
	ifTrue: [^anAssociation value: self].
    value xpathIsNodeSet
	ifTrue: [^self error: 'An XPath expression is answering a combination of Nodes and non-Nodes'].
    self error: 'An XPath expression is answering more than one non-Node value'
! !

!Object methodsFor:'xml support'!

xpathAsBoolean

    self error: ('Can''t convert a %1 to a boolean' bindWith: self className)
! !

!Object methodsFor:'xml support'!

xpathAsNumber

    self error: ('Can''t convert a %1 to a number' bindWith: self className)
! !

!Object methodsFor:'xml support'!

xpathCompareOrder:aData using:aBlock
    |v|

    v := self xpathAsNumber.
    aData xpathIsNodeSet ifTrue:[
	^ aData unsortedNodes
	    contains:[:nd2 | aBlock value:v value:nd2 xpathStringData xpathAsNumber ]
    ].
    ^ aBlock value:v value:aData xpathAsNumber
! !

!Object methodsFor:'xml support'!

xpathEvalIn: aNodeContext
    "This is private protocol--see #xpathValueIn: for the client protocol"

    ^self
! !

!Object methodsFor:'xml support'!

xpathIsBoolean
    ^false
! !

!Object methodsFor:'xml support'!

xpathIsNodeSet
    ^false
! !

!Object methodsFor:'xml support'!

xpathMayRequireNodeSet
    ^false
! !

!Object methodsFor:'xml support'!

xpathMayRequireNodeSetTopLevel
    ^false
! !

!Object methodsFor:'xml support'!

xpathMayRequireSort
    ^false
! !

!Object methodsFor:'xml support'!

xpathMayRequireSortTopLevel
    ^false
! !

!Object methodsFor:'xml support'!

xpathValueIn: aNodeContext
    "This is public protocol--see #xpathEvalIn: for the private internal protocol"

    ^self
! !

!String methodsFor:'converting'!

xpathAsBoolean
    ^self size > 0
! !

!String methodsFor:'converting'!

xpathAsNumber
    | s foundDigit numerator denominator ch |
    s := self readStream.
    s skipSeparators.
    foundDigit := false.
    numerator := 0.
    denominator := (s peekFor: $-) ifTrue: [-1] ifFalse: [1].
    [(ch := s next) notNil and: [ch isDigit]]
	whileTrue:
		[numerator := numerator * 10 + ch digitValue.
		foundDigit := true].
    ch = $.
	ifTrue: [[(ch := s next) notNil and: [ch isDigit]]
		whileTrue:
			[numerator := numerator * 10 + ch digitValue.
			denominator := denominator * 10.
			foundDigit := true]].
    (ch == nil or: [ch isSeparator]) ifFalse: [^FloatD nan].
    s skipSeparators.
    s atEnd ifFalse: [^FloatD nan].
    foundDigit ifFalse: [^FloatD nan].
    ^(numerator / denominator) + 0.0d0
! !

!String methodsFor:'converting'!

xpathAsString
    ^self
! !

!String methodsFor:'xml support'!

xpathCompareEquality: aData using: aBlock
    aData isString
	ifTrue: [^aBlock value: self value: aData].
    aData isNumber
	ifTrue: [^aBlock value: self xpathAsNumber value: aData].
    aData isBoolean
	ifTrue: [^aBlock value: self xpathAsBoolean value: aData].
    aData xpathIsNodeSet
	ifTrue: [^aData unsortedNodes contains: [:nd2 |
				aBlock value: self value: nd2 xpathStringData]].
    self error: ('Can''t compare a %1 with a string' bindWith: aData class printString)
! !

!XML::Attribute methodsFor:'accessing'!

xpathStringData
    ^self characterData
! !

!XML::Comment methodsFor:'accessing'!

xpathStringData
    ^self text
! !

!XML::Document methodsFor:'accessing'!

xpathStringData
    ^self root xpathStringData
! !

!XML::Element methodsFor:'accessing'!

xpathStringData
    ^self characterData
! !

!XML::Node methodsFor:'enumerating'!

addToXPathHolder: anAssociation for: aNodeContext
    |value|

    value := anAssociation value.
    value isNil
	ifTrue: [^ anAssociation value: (aNodeContext copy add: self)].
    value xpathIsNodeSet
	ifTrue: [^ value add: self].
    self error: 'An XPath expression is answering a combination of Nodes and non-Nodes'
! !

!XML::Node methodsFor:'accessing'!

xpathStringData
    ^self subclassResponsibility
! !

!XML::PI methodsFor:'accessing'!

xpathStringData
    ^self text
! !

!XML::Text methodsFor:'accessing'!

xpathStringData
    ^self characterData
! !
