"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

Dictionary variableSubclass:#ChainedDictionary
	instanceVariableNames:'parent'
	classVariableNames:''
	poolDictionaries:''
	category:'XSL-XSL-Nodes-Top Level'
!


!ChainedDictionary methodsFor:'accessing'!

associationAt: anIndex ifAbsent: aBlock
    ^super associationAt: anIndex ifAbsent: [parent associationAt: anIndex ifAbsent: aBlock]
!

at: anIndex ifAbsent: aBlock
    ^super at: anIndex ifAbsent: [parent at: anIndex ifAbsent: aBlock]
!

clone
    ^self class new parent: parent
!

parent
    ^parent
!

parent: aParent
    aParent == nil ifTrue: [self error:'aParent is not expected to be nil!!'].
    parent := aParent
!

size
    | s |
    s := Set new.
    self keysAndValuesDo: [:k :v |
    	s add: k].
    ^s size
! !

!ChainedDictionary methodsFor:'dictionary enumerating'!

associationsDo: aBlock 
    "Evaluate aBlock for each of the receiver's key/value associations."

    self keysAndValuesDo: [:k :v |
    	aBlock value: k -> v]
! !

!ChainedDictionary methodsFor:'dictionary testing'!

includesKey: key 
    "Answer whether the receiver has a key equal to the argument, key."

    ^(super includesKey: key) or: [parent includesKey: key]
! !

!ChainedDictionary methodsFor:'enumerating'!

do: aBlock 
    "Evaluate aBlock with each of the receiver's elements as the 
    argument."

    self keysDo: [:k | aBlock value: (self at: k)]
!

keysAndValuesDo: aBlock 
    "Evaluate aBlock with each of the receiver's key/value pairs as the 
    arguments."

    | keys |
    keys := Set new.
    super keysAndValuesDo: [:k :v | keys add: k].
    parent keysAndValuesDo: [:k :v | keys add: k].
    keys do: [:k |
    	aBlock value: k value: (self at: k)].
! !

!ChainedDictionary methodsFor:'private'!

changeCapacityTo: newCapacity
    | newSelf |

    Smalltalk isSmalltalkX ifTrue:[
        ^ super changeCapacityTo: newCapacity
    ].
    newSelf := self copyEmpty: newCapacity.
    newSelf parent: parent.
    super associationsDo: [:each | newSelf noCheckAdd: each].
    self become: newSelf.
! !

!ChainedDictionary class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/ChainedDictionary.st,v 1.2 2003/05/30 10:01:19 james Exp $'
! !
