"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

XSLCommand subclass:#CountingCommand
	instanceVariableNames:'format prefix postfix lang letterValue digitGroupSep
		digitsPerGroup sequenceSrc'
	classVariableNames:''
	poolDictionaries:''
	category:'XSL-XSL-Nodes-Counting'
!


!CountingCommand methodsFor:'accessing'!

digitGroupSep
    self testPatternInitialized.
    ^digitGroupSep
!

digitsPerGroup
    self testPatternInitialized.
    ^digitsPerGroup
!

format
    self testPatternInitialized.
    ^format
!

lang
    self testPatternInitialized.
    ^lang
!

letterValue
    self testPatternInitialized.
    ^letterValue
!

sequenceSrc
    self testPatternInitialized.
    ^sequenceSrc
! !

!CountingCommand methodsFor:'initialize'!

testPatternInitialized
    userData ifTrue: [^self].
    userData := true.
    format := self readAttribute: 'format' default: ['1'].
    lang := self readAttribute: 'xml:lang' default: [nil].
    letterValue := self readAttribute: 'letter-value' default: [nil].
    digitGroupSep := self readAttribute: 'digit-group-sep' default: [nil].
    digitsPerGroup := self readInteger: 'n-digits-per-group' default: [3].
    sequenceSrc := self readAttribute: 'sequence-src' default: [nil].
! !

!CountingCommand methodsFor:'private'!

format: number by: aFormat
    | n s |
    aFormat size = 1
    	ifFalse: [self error: ('Unrecognized number format = "%1"' bindWith: aFormat)].
    aFormat = 'a'
    	ifTrue:
    		[n := self radix: number base: 26.
    		s := n collect: [:i | (i + $a asInteger) asCharacter].
    		^String withAll: s].
    aFormat = 'A'
    	ifTrue:
    		[n := self radix: number base: 26.
    		s := n collect: [:i | (i + $A asInteger) asCharacter].
    		^String withAll: s].
    aFormat = 'i'
    	ifTrue:
    		[^self romanNumeral: number].
    aFormat = 'I'
    	ifTrue:
    		[^(self romanNumeral: number) asUppercase].
    aFormat = '1'
    	ifTrue:
    		[^number printString].
    self error: 'Unrecognized format'
!

radix: number base: b
    | out n |
    n := number - 1.
    n < b ifTrue: [^Array with: n].
    out := OrderedCollection new.
    n := number.
    [n < b]
    	whileFalse:
    		[out addFirst: n\\b.
    		n := n//b].
    out addFirst: n - 1.
    ^out
!

romanNumeral: number
    | n cycle output idx letters digit |
    n := number.
    cycle := #('ivx' 'xlc' 'cdm').
    output := OrderedCollection new.
    idx := 0.
    [n = 0]
    	whileFalse:
    		[letters := cycle at: (idx := idx + 1).
    		digit := n\\10.
    		digit := #(#() #(1) #(1 1) #(1 1 1) #(1 2) #(2) #(2 1) #(2 1 1) #(2 1 1 1) #(1 3))
    				at: digit + 1.
    		output addAllFirst: (digit collect: [:i | letters at: i]).
    		n := n//10].
    ^String withAll: output
!

tokenizeFormat: aString
    | str isFormat tok tokens t resultFormat |
    str := aString readStream.
    tokens := OrderedCollection new.
    isFormat := [:ch | ch isDigit or: [ch isLetter]].
    [str atEnd] whileFalse:
    	[tok := ''.
    	(isFormat value: str peek)
    		ifTrue: [[str atEnd or: [(isFormat value: str peek) not]]
    			whileFalse: [tok := tok copyWith: str next]]
    		ifFalse: [[str atEnd or: [isFormat value: str peek]]
    			whileFalse: [tok := tok copyWith: str next]].
    	tokens add: tok].
    (tokens isEmpty or: [isFormat value: tokens first first])
    	ifFalse: [prefix := tokens removeFirst].
    (tokens isEmpty or: [isFormat value: tokens last first])
    	ifFalse: [postfix := tokens removeLast].
    tokens size = 0
    	ifTrue: [resultFormat := nil]
    	ifFalse: [tokens size = 1
    		ifTrue:
    			[resultFormat := NumberFormat new format: tokens first; separator: '.'.
    			resultFormat nextLink: resultFormat]
    		ifFalse:
    			[t := (1 to: tokens size by: 2) collect:
    				[:i | NumberFormat new
    						format: (tokens at: i);
    						separator: (tokens at: (i + 1 min: tokens size - 1))].
    			1 to: t size do: [:i | (t at: i) nextLink: (t at: (i + 1 min: t size))].
    			resultFormat := t first]].
    ^resultFormat
! !

!CountingCommand methodsFor:'processing'!

format: countList for: aNodeContext
    | str fmt |
    str := String new writeStream.
    fmt := self processAttributeValue: self format for: aNodeContext.
    fmt := self tokenizeFormat: fmt.
    prefix == nil ifFalse: [str nextPutAll: prefix].
    1 to: countList size do: [:i |
    	str nextPutAll: (self format: (countList at: i) by: fmt format).
    	i = countList size ifFalse: [str nextPutAll: fmt separator].
    	fmt := fmt nextLink].
    postfix == nil ifFalse: [str nextPutAll: postfix].
    ^str contents
! !

!CountingCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/CountingCommand.st,v 1.1 2003/05/28 10:57:08 james Exp $'
! !
