"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

CountingCommand subclass:#NumberCommand
	instanceVariableNames:'level count from'
	classVariableNames:''
	poolDictionaries:''
	category:'XSL-XSL-Nodes-Counting'
!


!NumberCommand class methodsFor:'accessing'!

tag
    ^'number'
! !

!NumberCommand methodsFor:'accessing'!

countFor: aNode
    aNode isElement ifFalse: [self halt: 'Counting things other than elements is not supported yet'].
    self testPatternInitialized.
    count == nil
	ifTrue: [ ^XML::XPathChildNode new axisName: 'child'; baseTest: (XML::XPathTaggedNodeTest new namespace: aNode tag namespace; type: aNode tag type)]
	ifFalse: [ ^ count]
!

from
    self testPatternInitialized.
    ^from
!

level
    self testPatternInitialized.
    ^level
! !

!NumberCommand methodsFor:'initialize'!

testPatternInitialized
    userData ifTrue: [^self].
    super testPatternInitialized.
    level := (self readAttribute: 'level' default: [#single]) asSymbol.
    count := self readMatchPattern: 'count' default: [nil].
    from := self readMatchPattern: 'from' default: [nil].
    userData := true.
! !

!NumberCommand methodsFor:'processing'!

process: aNodeContext into: aProxy
    self level == #single
	ifTrue: [^self processSingle: aNodeContext into: aProxy].
    self level == #multi
	ifTrue: [^self processMulti: aNodeContext into: aProxy].
    self level == #any
	ifTrue: [^self processAny: aNodeContext into: aProxy].
    ^self error: 'Unsupported numbering mode'
!

processAny: aNodeContext into: aProxy
    | n countP |
    n := 0.
    countP := self countFor: aNodeContext node.
    NodeIterator new
	node: aNodeContext node;
	reverseDo: [:nd | (countP match: (aNodeContext copy add: nd; index: 1)) ifTrue: [n := n + 1]]
		until: [:nd | self from notNil and: [self from match: (aNodeContext copy add: nd; index: 1)]].
    aProxy addNode: (XML::Text new text: (self format: (Array with: n) for: aNodeContext))
!

processMulti: aNodeContext into: aProxy
    | allNodes n counts countP sibSelect cnt sibs |
    countP := self countFor: aNodeContext node.
    allNodes := aNodeContext copy.
    n := aNodeContext node.
    [n == nil or: [self from notNil and: [self from match: n]]] whileFalse:
	[allNodes add: n.
	n := n parent].
    allNodes := allNodes selectMatch: countP.
    allNodes documentOrder; index: 1.
    sibSelect := XML::XPathParser parse: '../node()' as: #expression.
    counts := OrderedCollection new.
    allNodes reset.
    [allNodes atEnd] whileFalse:
	[allNodes next.
	cnt := 1.
	sibs := sibSelect xpathValueIn: allNodes.
	sibs reset; next.
	[sibs node == allNodes node]
		whileFalse:
			[(countP match: sibs) ifTrue: [cnt := cnt + 1].
			sibs next].
	counts add: cnt].
    aProxy addNode: (XML::Text new text: (self format: counts asArray for: aNodeContext))
!

processSingle: aNodeContext into: aProxy
    | allNodes n cnt countP sibSelect sibs |
    countP := self countFor: aNodeContext node.
    allNodes := aNodeContext copy.
    n := aNodeContext node.
    [n == nil or: [self from notNil and: [self from match: n]]] whileFalse:
	[allNodes add: n.
	n := n parent].
    allNodes := allNodes selectMatch: countP.
    allNodes size = 0 ifTrue: [^self].
    allNodes inverseDocumentOrder; index: 1.
    sibSelect := XML::XPathParser parse: '../node()' as: #expression.
    sibs := sibSelect xpathValueIn: allNodes.
    sibs reset; next.
    cnt := 1.
    [sibs node == allNodes node]
	whileFalse:
		[(countP match: sibs) ifTrue: [cnt := cnt + 1].
		sibs next].
    aProxy addNode: (XML::Text new text: (self format: (Array with: cnt) for: aNodeContext))
! !

!NumberCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/NumberCommand.st,v 1.4 2009/11/03 14:30:03 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/NumberCommand.st,v 1.4 2009/11/03 14:30:03 cg Exp $'
! !
