"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

XSLDefinition subclass:#Rule
	instanceVariableNames:'pattern name specific priority mode variables'
	classVariableNames:''
	poolDictionaries:''
	category:'XSL-XSL-Nodes-Top Level'
!


!Rule class methodsFor:'accessing'!

tag
    ^'template'
! !

!Rule class methodsFor:'formatting'!

formatText: aNode
    | text tab a |
    a := XML::Attribute new name: 'CLASS' value: 'body'.
    text := aNode characterData.
    tab := String new: 4 withAll: (Character value: 160). "nbsp"
    text := text copyReplaceAll: 9 asCharacter asString with: tab.
    text := text substrings: Character nl.
    ^text collect: [:t |
	t isEmpty
		ifTrue: [XML::Element tag: 'BR']
		ifFalse:
			[XML::Element tag: 'P' attributes: (Array with: a copy)
				elements: (Array with: (XML::Text new text: t))]]
! !

!Rule methodsFor:'accessing'!

defineParameter: aVariable
    | old |
    old := variables detect: [:var | var name = aVariable name] ifNone: [].
    (old == nil or: [old == aVariable])
	ifFalse: [self error: 'The parameter "', aVariable name, '" is shadowing another variable in the same template'].
    old == nil ifTrue: [variables add: aVariable].
!

defineVariable: aVariable
    | old |
    old := variables detect: [:var | var name = aVariable name] ifNone: [].
    (old == nil or: [old == aVariable])
	ifFalse: [self error: 'The variable "', aVariable name, '" is shadowing another variable in the same template'].
    old == nil ifTrue: [variables add: aVariable].
!

mode
    self testPatternInitialized.
    ^mode
!

mode: aMode
    mode := aMode
!

name
    self testPatternInitialized.
    ^name
!

pattern
    self testPatternInitialized.
    ^pattern
!

priority
    self testPatternInitialized.
    ^priority
!

priority: aNumber
    priority := aNumber
!

purgeUnimportant
    ^self
! !

!Rule methodsFor:'initialize'!

computeDefaultPriority: expr
    | list |
    expr class == XML::XPathUnion
	ifTrue:
		[list := Set new.
		expr arguments do: [:expr2 | list add: (self computeDefaultPriority: expr2)].
		list size = 1
			ifTrue: [^ list asArray first]
			ifFalse: [^ #notKnown]]
	ifFalse: [((expr class == XML::XPathChildNode or: [expr class == XML::XPathAttributeNode])
				and: [expr child isTerminator and: [expr predicates isEmpty]])
		ifTrue: [expr baseTest class == XML::XPathTaggedNodeTest
			ifTrue: [expr baseTest type == #*
				ifTrue: [^ expr baseTest namespace == nil ifTrue: [-0.5d] ifFalse: [-0.25d]]
				ifFalse: [^ 0.0d]]
			ifFalse: [('processing-instruction(*)' match: expr printString)
				ifTrue: [^ self halt]
				ifFalse: [^ -0.5d]]]
		ifFalse: [^ 0.5]]
!

initialize
    super initialize.
    priority := 0.
    variables := OrderedCollection new.
!

normalize
    super normalize.
    (self parent == nil or: [self parent isKindOf: RuleSet])
	ifFalse: [self error: self tag asString, ' can only be used at the top level'].
!

testPatternInitialized
    userData ifTrue: [^self].
    userData := true.
    name := self readAttribute: 'name' default: [nil].
    pattern := self readMatchPattern: 'match' default: [nil].
    priority := self readInteger: 'priority' default: [self computeDefaultPriority: self pattern].
    mode := self readAttribute: 'mode' default: [nil]
! !

!Rule methodsFor:'loading'!

addToRuleDB: aDB
    "This is only understood by a small subset of commands."

    self pattern == nil ifFalse: [aDB addRule: self].
    self name == nil ifFalse: [aDB addNamedTemplate: self].
    (self pattern == nil and: [self name == nil])
	ifTrue: [self error: 'Templates must have either a name or match attribute or both'].
! !

!Rule methodsFor:'processing'!

process: aNodeContext into: aProxy
    self shouldNotImplement
!

process: aNodeContext into: aProxy arguments: arguments
    | list |
    list := self xslElements.
    1 to: list size do: [:i || elm |
	elm := list at: i.
	elm process: aNodeContext into: aProxy takeArgumentsFrom: arguments]
! !

!Rule methodsFor:'testing'!

isStylesheetEntry
    ^true
!

match: aNodeContext
    ^self pattern notNil and: [self pattern match: aNodeContext]
!

modeIsLike: aMode
    "We can use #any as the 'accept any mode', because
    normal modes are strings. If this is changed, the marker
    for 'any mode' would need to be changed."

    ^mode = aMode or: [mode == #any]
! !

!Rule class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/Rule.st,v 1.4 2009/11/03 14:30:03 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/Rule.st,v 1.4 2009/11/03 14:30:03 cg Exp $'
! !
