"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

XSLCommand subclass:#Template
	instanceVariableNames:'hasStripped'
	classVariableNames:''
	poolDictionaries:''
	category:'XSL-XSL-Nodes'
!


!Template methodsFor:'accessing'!

importanceHolder: dummy
!

purgeUnimportant
    ^self
! !

!Template methodsFor:'loading'!

topLevelAddToRuleDB: aDB
    "This is only understood by a small subset of commands."
    |transformURI|
    Smalltalk isSmalltalkX ifTrue:[
	transformURI := RuleDatabase xsl_uri.
    ] ifFalse:[
	transformURI := tag namespace =  XSL_URI
    ].

    self tag namespace = transformURI ifTrue: [
	    self error: ('"%1" not recognized as an XSL command'
		bindWith: self tag asString)
    ].
    aDB addRule: self
! !

!Template methodsFor:'processing'!

process: aNodeContext into: aProxy
    | oc list |
    oc := aProxy childProxy.
    oc attributes: (self processAttributes: self attributes for: aNodeContext).
    list := self xslElements.
    1 to: list size do: [:i || elm |
	elm := list at: i.
	elm process: aNodeContext into: oc].
    aProxy addNode: (XML::Element tag: tag attributes: oc attributes elements: oc children)
!

process: aNodeContext into: aProxy arguments: arguments
    "The arguments are ignored because, if I am being used
    as a top-level stylesheet, there's no place to declare the
    top-level <xsl:param> definitions."

    self process: aNodeContext into: aProxy
!

processAttributes: attList for: aNodeContext
    | newList substitution newAtt transformURI|
    Smalltalk isSmalltalkX ifTrue:[
	transformURI := RuleDatabase xsl_uri.
    ] ifFalse:[
	transformURI := tag namespace =  XSL_URI
    ].

    newList := OrderedCollection new.
    attList do: [:att |
	(att tag namespace = transformURI)
		ifTrue:
			[newAtt := self processXSLAttribute: att for: aNodeContext.
			newAtt == nil ifFalse: [newList add: newAtt]]].
    attList do: [:att |
	(att tag namespace = transformURI)
		ifFalse:
			[substitution := self processAttributeValue: att value for: aNodeContext.
			newAtt := XML.Attribute name: att key value: substitution.
			newList add: newAtt]].
    ^newList isEmpty
	ifTrue: [nil]
	ifFalse: [newList asArray]
!

processXSLAttribute: att for: aNodeContext
    att tag type = 'version'
	ifTrue:
		[" aNodeContext db version: att value. "
		^nil].
    att tag type = 'use-attribute-set'
	ifTrue:
		[^self notYetImplementedError].
    ^self notYetImplementedError
! !

!Template methodsFor:'testing'!

importance
    "Really only needs to be > 0 to beat the builtin rule that matches
    against the root of the document, but we throw in a bit of paranoia."

    ^1000
!

match: aNodeContext
    ^aNodeContext node isDocument
!

modeIsLike: aMode
    ^aMode isNil
! !

!Template class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/Template.st,v 1.3 2009/11/03 14:30:03 cg Exp $'
! !
