"{ Package: 'stx:goodies/xml/xsl' }"

"{ NameSpace: XSL }"

XML::Element subclass:#XSLCommand
	instanceVariableNames:'userData'
	classVariableNames:'XPathExtensionFunctions'
	poolDictionaries:''
	category:'XSL-XSL-Nodes'
!


!XSLCommand class methodsFor:'accessing'!

tag
    ^nil
!

xslFunctions
    ^XPathExtensionFunctions
! !

!XSLCommand class methodsFor:'class initialization'!

initialize
    "XSLCommand initialize"

    | functions |
    functions := ChainedDictionary new.
    functions parent: XML.XPathFunction baseFunctions.
    functions at: 'format-number' put: (XML::XPathFunction new
		name: 'format-number';
		valueBlock: [:fn :ns || n s1 s2 |
				(fn arguments size between: 2 and: 3)
					ifFalse: [self error: 'format-number() takes two or three arguments.'].
				n := ((fn arguments at: 1) xpathEvalIn: ns) xpathAsNumber.
				s1 := ((fn arguments at: 2) xpathEvalIn: ns) xpathAsString.
				fn arguments size = 3
					ifTrue: [s2 := ((fn arguments at: 3) xpathEvalIn: ns) xpathAsString]
					ifFalse: [s2 := nil].
				self formatNumber: n pattern: s1 formatName: s2 in: ns]).
    XPathExtensionFunctions := functions.
! !

!XSLCommand class methodsFor:'xpath'!

formatNumber: aNumber pattern: pattern formatName: formatName in: aNodeContext
    ^aNumber xpathAsString
! !

!XSLCommand methodsFor:'accessing'!

defaultTag
    ^'xsl:', self class tag
!

defineVariable: aVariable
    self parent defineVariable: aVariable
!

purgeUnimportant
    self subclassResponsibility
!

sortList
    "Answer a list of sort blocks that take two arguments,
    and return -1 if the arguments are in order, 1 if they
    are in reversed order, and 0 if that particular sort block
    cannot order them."

    ^nil
!

xslElements
    ^self children select: [:i | i isContent]
! !

!XSLCommand methodsFor:'initialize'!

initialize
    super initialize.
    elements := #().
    userData := false
!

normalize
    self stripSpace.
    self xslElements do: [:elm | elm normalize].
!

stripSpace
    self shouldStrip
	ifTrue: [self elements: (self children select: [:t | t isBlankText not])].
!

testPatternInitialized
    userData := true.
! !

!XSLCommand methodsFor:'loading'!

addToRuleDB: aDB
    "This is only understood by a small subset of commands."

    self shouldNotImplement
! !

!XSLCommand methodsFor:'private'!

checkQNameSyntax: aString
    | str mode colons ch valid |
    str := aString readStream.
    mode := #colon.
    colons := 0.
    [str atEnd] whileFalse:
	[ch := str next.
	mode == #colon
		ifTrue:
			[valid := ch = $_ or: [ch isLetter].
			mode := #letter]
		ifFalse: [ch = $:
			ifTrue:
				[valid := true.
				colons := colons + 1.
				mode := #colon]
			ifFalse:
				[valid := ch isLetter or: [ch isDigit or: ['.-_' includes: ch]].
				mode := #letterOrDigit]].
	valid ifFalse: [self error: 'Syntax error in qualified name.']].
    (mode = #colon or: [colons > 1])
	ifTrue: [self error: 'Syntax error in qualified name.'].
!

checkURISyntax: aString
    | n type ch |
    n := aString findLast: [:c | c = $#].
    n = aString size
	ifTrue: [self error: 'The name for an attribute or element, using the x#y syntax, has no type following the #.'].
    type := aString copyFrom: n + 1 to: aString size.
    ch := type at: 1.
    (ch = $_ or: [ch isLetter])
	ifFalse: [self error: ('Type name syntax error in "%1".' bindWith: type)].
    2 to: type size do: [:i |
	ch := type at: i.
	(ch isLetter or: [ch isDigit or: ['.-_' includes: ch]])
		ifFalse: [self error: ('Type name syntax error in "%1".' bindWith: type)]]
!

collate: node1 to: node2 within: aNodeContext
    | list sign |
    (list := self sortList) == nil
	ifFalse:
		[1 to: list size do: [:i |
			sign := (list at: i) collate: node1 to: node2 within: aNodeContext.
			sign = 0 ifFalse: [^sign = -1]]].
    ^node1 precedes: node2
!

readAttribute: attName
    ^self readAttribute: attName
	default: [self error: ('%1 needs to have an attribute named %2'
				bindWith: self tag asString
				with: attName)]
!

readAttribute: attName default: def
    | att |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^att == nil
	ifTrue: [def value]
	ifFalse: [att]
!

readInteger: attName default: def
    | att val |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^att == nil
	ifTrue: [def value]
	ifFalse:
		[att isEmpty ifTrue: [self error: ('The %1 attribute is empty' bindWith: attName)].
		att := att readStream.
		val := Number readFrom: att.
		val = 0 ifTrue: [self error: 'Bad number format, ', (att instVarAt: 1)].
		att atEnd ifFalse: [self error: ('The %1 attribute is not a legal integer value' bindWith: attName)].
		val]
!

readMatchPattern: attName
    ^self readMatchPattern: attName
	default: [self error: ('%1 needs to have an attribute named %2'
				bindWith: self tag asString
				with: attName)]
!

readMatchPattern: attName default: def
    | att |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^att == nil
	ifTrue: [def value]
	ifFalse: [XML.XPathParser new xmlNode: self;
							functions: self class xslFunctions;
							parse: att as: #expression]
!

readSelectPattern: attName
    ^self readSelectPattern: attName
	default: [self error: ('%1 needs to have an attribute named %2'
				bindWith: self tag asString
				with: attName)]
!

readSelectPattern: attName default: def
    | att d |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^(att == nil)
	ifTrue:[
	    d := def value.
	    d == nil ifFalse:[
		d := XML.XPathParser new xmlNode: self;
			    functions: self class xslFunctions;
			    parse: d as: #expression
	    ].
	    d

     ] ifFalse: [XML.XPathParser new
			    xmlNode: self;
			    functions: self class xslFunctions;
			    parse: att as: #expression
    ]
!

readTag: attName
    | att |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^att == nil
	ifTrue: [self error: ('%1 needs to have an attribute named %2'
				bindWith: self tag asString
				with: attName)]
	ifFalse: [att]
!

readTagList: attName default: defaultBlock
    | att str output buffer ch |
    att := self valueOfAttribute: attName ifAbsent: [nil].
    ^att == nil
	ifTrue: [defaultBlock value]
	ifFalse:
		[str := att readStream.
		output := OrderedCollection new.
		buffer := String new writeStream.
		[str skipSeparators; atEnd]
			whileFalse:
				[[(ch := str next) notNil and: [ch isSeparator not]]
					whileTrue: [buffer nextPut: ch].
				output add: buffer contents.
				buffer reset].
		output asArray]
!

xslNodesFrom: aNodeContext
    | list nc |
    list := aNodeContext node
	selectNodes: [:nd | nd isAttribute not or: [nd tag qualifier ~= 'xmlns']].
    nc := aNodeContext copy documentOrder.
    nc addAll: list.
    ^nc
! !

!XSLCommand methodsFor:'processing'!

process: aNodeContext into: aProxy
    self subclassResponsibility
!

process: aNodeContext into: aProxy takeArgumentsFrom: arguments
    self process: aNodeContext into: aProxy
!

processAttributeSets: aNodeContext into: aProxy
    | list vars |
    self useAttrs isEmpty ifTrue: [^self].
    vars := aNodeContext variables.
    aNodeContext variables: vars parent.
    self useAttrs do: [:attSetName |
	list := aNodeContext db attributesForSet: attSetName.
	list do: [:att | att process: aNodeContext into: aProxy]].
    aNodeContext variables: vars.
!

processAttributeValue:aString for:aNodeContext
    |source ch output elm p expr|

    source := XML.XPathReadStream on:aString.
    output := (String new:64) writeStream.
    [source atEnd] whileFalse:[
	ch := source next.
	ch = ${ ifTrue:[
	    (source peekFor:${) ifTrue:[
		output nextPut:${
	    ] ifFalse:[
		p := XML.XPathParser new.
		p
		    initScanner;
		    xmlNode:self;
		    functions:self class xslFunctions;
		    init:source
			notifying:nil
			failBlock:nil.
		p expression.
		p atEndOfExpression ifFalse:[
		    self error:'Syntax error in: ' , aString storeString
		].
		expr := p result.
		elm := expr xpathValueIn:aNodeContext.
		output nextPutAll:elm xpathAsString.
		p pastEnd ifFalse:[
		    source skip:-1
		]
	    ]
	] ifFalse:[
	    ch = $} ifTrue:[
		source next = $} ifFalse:[
		    self error:'Expected doubled }'
		].
		output nextPut:$}
	    ] ifFalse:[
		output nextPut:ch
	    ]
	]
    ].
    ^ output contents
!

resolveComputedTag: nm
    | n type ns qualifier |
    ^(nm includes: $#)
	ifTrue:
		[self checkURISyntax: nm.
		n := nm findLast: [:c | c = $#].
		type := nm copyFrom: n + 1 to: nm size.
		ns := nm copyFrom: 1 to: n - 1.
		qualifier := self findQualifierAtNamespace: 'quote:', ns.
		qualifier == nil
			ifTrue: [qualifier := self findQualifierAtNamespace: ns].
		qualifier == nil ifTrue: [self error: ('The namespace %1 has not been bound to a qualifier in this stylesheet, and automatic creation of qualifiers has not been implemented.' bindWith: ns)].
		XML.NodeTag new qualifier: qualifier ns: ns type: type]
	ifFalse:
		[self checkQNameSyntax: nm.
		self resolveTag: nm].
!

resolveTag: aTagString
    | c qual ns |
    c := aTagString occurrencesOf: $:.
    ^c = 0
	ifTrue: [XML.NodeTag new qualifier: '' ns: '' type: aTagString]
	ifFalse: [c > 1
		ifTrue: [self error: 'A qualified name has too many colons.']
		ifFalse:
			[c := aTagString indexOf: $:.
			(c = 1 or: [c = aTagString size])
				ifTrue: [self error: 'A qualified name cannot begin or end with a colon.'].
			qual := aTagString copyFrom: 1 to: c - 1.
			ns := self findNamespaceAt: qual.
			ns == nil ifTrue: [self error: ('The namespace qualifier %1 has not been bound to a namespace in this stylesheet' bindWith: qual)].
			"Use a # in the match to make sure there's at least one more character"
			('quote:#*' match: ns)
				ifTrue: [ns := ns copyFrom: 'quote:' size + 1 to: ns size].
			XML.NodeTag new qualifier: qual ns: ns type: (aTagString copyFrom: c + 1 to: aTagString size)]]
!

selectAll: startNode withPattern: pattern
    ^pattern xpathValueIn: startNode
!

valueAsVariableIn: aNodeContext
    | new list |
    ^self expression == nil
	ifTrue:
		[new := ElementProxy new.
		list := self xslElements.
		1 to: list size do: [:i || elm |
			elm := list at: i.
			elm process: aNodeContext into: new].
		new]
	ifFalse: [self expression xpathValueIn: aNodeContext]
! !

!XSLCommand methodsFor:'testing'!

generatesAttributes
    ^false
!

isStylesheetEntry
    Transcript nl; tab; show: ('Stylesheet contains a top-level element that is not permitted (%1)'
		bindWith: self tag).
    Transcript nl; tab; show: 'It has been ignored'.
    ^false
!

shouldStrip
    ^true
! !

!XSLCommand class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/xsl/XSLCommand.st,v 1.3 2004/11/18 11:06:43 cg Exp $'
! !

XSLCommand initialize!
