"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

XMLTokenizer subclass:#SAXDriver
	instanceVariableNames:'saxHandler scope useNamespaces validateAttributes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!


!SAXDriver methodsFor:'accessing'!

saxHandler
	^saxHandler
!

saxHandler: aHandler
	saxHandler := aHandler
!

useNamespaces
    ^ useNamespaces
!

useNamespaces:something
    useNamespaces := something.
! !

!SAXDriver methodsFor:'handling tokens'!

handleCData:aString 
    (self saxHandler)
        checkEOD;
        characters:aString
!

handleEndDocument
    self saxHandler endDocument
!

handleEndTag:elementName 
    |namespace localName namespaceURI qualifiedName|

    useNamespaces ifTrue:[
        self 
            splitName:elementName
            into:[:ns :ln | 
                namespace := ns.
                localName := ln
            ].
        "ensure our namespace is defined"
        namespace isNil ifTrue:[
            namespace := self scope defaultNamespace.
            qualifiedName := namespace , ':' , elementName
        ] ifFalse:[
            namespaceURI := self scope namespaceURIOf:namespace.
            namespaceURI 
                ifNil:[
                    self 
                        parseError:'Start tag ' , elementName , ' refers to undefined namespace ' 
                                , namespace asString
                ].
            qualifiedName := elementName
        ].
        "call the handler"
        (self saxHandler)
            checkEOD;
            endElement:localName
                namespace:namespace
                namespaceURI:namespaceURI
                qualifiedName:qualifiedName.
        self scope leaveScope
    ] ifFalse:[
        "call the handler"
        (self saxHandler)
            checkEOD;
            endElement:elementName
                namespace:nil
                namespaceURI:nil
                qualifiedName:elementName
    ]
!

handlePCData:aString 
    (self saxHandler)
        checkEOD;
        characters:aString
!

handlePI:piTarget data:piData 
    (self saxHandler)
        checkEOD;
        processingInstruction:piTarget data:piData
!

handleStartDocument
    self saxHandler startDocument
!

handleStartTag:elementName attributes:attributeList 
    (self saxHandler)
        checkEOD;
        startElement:elementName
            namespaceURI:nil
            namespace:nil
            attributeList:attributeList
!

handleStartTag:elementName attributes:attributeList namespaces:namespaces 
    |localName namespace namespaceURI|

    self usesNamespaces ifTrue:[
        self scope enterScope.
         "declare any namespaces"
        namespaces keysAndValuesDo:[:ns :uri | self scope declareNamespace:ns uri:uri ].
        self 
            splitName:elementName
            into:[:ns :ln | 
                namespace := ns.
                localName := ln
            ].
        "ensure our namespace is defined"
        namespace isNil 
            ifTrue:[ namespace := self scope defaultNamespace ]
            ifFalse:[
                namespaceURI := self scope namespaceURIOf:namespace.
                namespaceURI isNil ifTrue:[
                    self parseError:'Start tag ' , elementName , ' refers to undefined namespace ' , namespace asString
                ]
            ].
        self validatesAttributes ifTrue:[
            self scope validateAttributes:attributeList
        ].
        "call the handler"
        (self saxHandler)
            checkEOD;
            startElement:localName
                namespaceURI:namespaceURI
                namespace:namespace
                attributeList:attributeList
    ] ifFalse:[
        "call the handler"
        (self saxHandler)
            checkEOD;
            startElement:elementName
                namespaceURI:nil
                namespace:nil
                attributeList:attributeList
    ]
!

handleWhitespace: aString
    self saxHandler
        checkEOD; 
        ignorableWhitespace: aString
!

handleXMLDecl: attributes
	self saxHandler
		checkEOD; 
		documentAttributes: attributes
!

handleXMLDecl:attributes namespaces:namespaces 
    self handleXMLDecl:attributes.
    self usesNamespaces ifTrue:[
        namespaces 
            keysAndValuesDo:[:ns :uri | self scope declareNamespace:ns uri:uri ]
    ]
! !

!SAXDriver methodsFor:'initialization'!

initialize
    super initialize.
    useNamespaces := false.
    validateAttributes := false
! !

!SAXDriver methodsFor:'namespaces'!

scope
    scope isNil ifTrue:[ scope := XMLNamespaceScope new ].
    ^ scope
!

splitName:aName into:twoArgsBlock 
    "Split the name into namespace and local name (the block arguments).
     Handle both qualified and unqualified names using the default name space"
    
    |i ns ln|

    i := aName lastIndexOf:$:.
    i = 0 ifTrue:[
        ns := nil.
        ln := aName
    ] ifFalse:[
        ns := aName copyFrom:1 to:(i - 1).
        ln := aName copyFrom:i + 1 to:aName size
    ].
    twoArgsBlock value:ns value:ln
! !

!SAXDriver methodsFor:'testing'!

usesNamespaces
    ^ useNamespaces
!

validatesAttributes
    ^ validateAttributes
! !

!SAXDriver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/SAXDriver.st,v 1.6 2009/11/09 23:11:04 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/SAXDriver.st,v 1.6 2009/11/09 23:11:04 cg Exp $'
! !
