"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

Object subclass:#SAXHandler
	instanceVariableNames:'document driver eod'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!


!SAXHandler class methodsFor:'instance creation'!

new
	^super new initialize
!

on: aStream
        | driver parser |
        driver := YAXO::SAXDriver on: aStream.
        driver validating: true.
        parser := self new driver: driver.
        ^parser
!

parseDTDFrom: aStream
        | driver parser |
        driver := SAXDriver on: aStream.
        driver validating: true.
        driver startParsingMarkup.
        parser := self new driver: driver.
        parser startDocument.
        parser parseDocument.
        ^parser
!

parseDocumentFrom: aStream
        | driver parser stream |
        #swYAXO.        "changed to handle UTF-16"
        aStream atEnd 
                ifTrue: [SAXMalformedException signal: 'XML document must have a top level element'].
        stream := aStream peek codePoint >= 254 
                                ifTrue: [self convertWideStream: aStream]
                                ifFalse: [aStream].
        driver := SAXDriver on: stream.
        driver validating: true.
        parser := self new driver: driver.
        parser startDocument.
        parser parseDocument.
        ^parser
!

parseDocumentFromFileNamed: fileName
	^self parseDocumentFromFileNamed: fileName readIntoMemory: false
!

parseDocumentFromFileNamed: fileName readIntoMemory: readIntoMemory
	| fs stream parser |
	#swYAXO.
	fs := FileStream read: fileName text: true.
	readIntoMemory ifTrue: [stream := stream contents readStream] ifFalse: [stream := fs].
	[parser := self parseDocumentFrom: stream] ensure: [fs close].
	^parser 
! !

!SAXHandler class methodsFor:'helpers'!

convertWideStream: aReadStream
	| wideString |
	#swYAXO.        "added to handle UTF-16"
	aReadStream
		next;
		next.
	wideString := aReadStream upToEnd.
	#cg.
	^ (self unicodeStringFromWideString:wideString length: wideString size // 2)
		asString readStream
"/        ^(UnicodeString fromAddress: wideString yourAddress length: wideString size // 2) 
"/                asString readStream
!

unicodeStringFromWideString:aString length:len
    |newString iSrc code|

    newString := UnicodeString new:len.

    iSrc := 1.
    1 to:len do:[:iDst |
	code := (aString at:iSrc) asciiValue.
	code := code + (((aString at:iSrc+1) asciiValue) bitShift:8).
	newString at:iDst put:(Character value:code).
	iSrc := iSrc + 2.
    ].
    ^ newString
! !

!SAXHandler methodsFor:'accessing'!

document
	^document
!

document: aDocument
	document := aDocument
!

driver
	^driver
!

driver: aDriver
	driver := aDriver.
	driver saxHandler: self
!

eod
	^eod
!

useNamespaces: aBoolean
    self driver useNamespaces: aBoolean
! !

!SAXHandler methodsFor:'content'!

characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."
!

checkEOD
        "Check if the document shouldn't be ended already"
        self eod
                ifTrue: [self driver malformedError: 'No more data expected']
!

documentAttributes: attributeList
!

endDocument
	"This call corresponds to the Java SAX call
	endDocument()."
	eod := true
!

endElement: elementName
!

endElement: elementName namespace:namespace namespaceURI: namespaceURI qualifiedName: qualifiedName
        "This call corresponds to the Java SAX call
        endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
        By default this call is mapped to the following more convenient call:"

        self endElement: elementName
!

endElement: elementName namespaceURI: namespaceURI qualifiedName: qualifiedName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: elementName
!

endPrefixMapping: prefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."
!

ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."
!

processingInstruction: piName data: dataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."
!

skippedEntity: aString
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."
!

startDocument
	"This call corresonds to the Java SAX call
	startDocument()."
!

startElement: elementName attributeList: attributeList
!

startElement: elementName namespaceURI: namespaceURI namespace: qualifiedName attributeList: attributeList
        "This call corresonds to the Java SAX call
        startElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName, Attributes atts).
        By default this call is mapped to the following more convenient call:"

        self startElement: elementName attributeList: attributeList
!

startPrefixMapping: prefix uri: uri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."
! !

!SAXHandler methodsFor:'entity'!

resolveEntity: publicID systemID: systemID
	"This call corresonds to the Java SAX call
	resolveEntity(java.lang.String publicId, java.lang.String systemId)."
! !

!SAXHandler methodsFor:'initialize'!

initialize
        eod := false
! !

!SAXHandler methodsFor:'lexical'!

comment: commentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."
!

endEntity: entityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."
!

startCData
	"This call corresponds to the Java SAX ext call
	startCData()."
!

startDTD: declName publicID: publicID systemID: systemID
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."
!

startEntity: entityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."
! !

!SAXHandler methodsFor:'parsing'!

parseDocument
        [driver nextEntity isNil or: [self eod]] whileFalse
! !

!SAXHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/SAXHandler.st,v 1.9 2010/03/10 11:55:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/SAXHandler.st,v 1.9 2010/03/10 11:55:21 cg Exp $'
! !
