"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

SAXHandler subclass:#XMLDOMParser
	instanceVariableNames:'entity stack incremental'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!


!XMLDOMParser class methodsFor:'instance creation'!

parseDocumentFrom:aStream 
    #swYAXO. "Note that unlike the super class method, this method answers a XMLDocument"
    ^ (super parseDocumentFrom:aStream) document

    "
     self parseDocumentFrom:'c:\zzz.htm' readStream
     self parseDocumentFrom:XMLTokenizer addressBookXMLWithDTD readStream
    "
! !

!XMLDOMParser class methodsFor:'examples'!

addressBookXMLWithDTD
	"XMLDOMParser addressBookXMLWithDTD"
	^self parseDocumentFrom: XMLTokenizer addressBookXMLWithDTD readStream
! !

!XMLDOMParser methodsFor:'accessing'!

incremental
	^incremental
!

incremental: aBoolean
	incremental := aBoolean
! !

!XMLDOMParser methodsFor:'accessing-private'!

pop
	| oldTop |
	oldTop := self stack removeLast.
	entity := oldTop.
	^oldTop
!

push: anObject
	self stack add: anObject.
	entity := anObject
!

stack
	^stack
!

top
        |s|

        ^(s := self stack) isEmpty
                ifTrue: [nil]
                ifFalse: [s last]
! !

!XMLDOMParser methodsFor:'content'!

characters: aString
	| newElement |
	newElement := XMLStringNode string: aString.
	self top addContent: newElement.
!

defaultNamespace
    |t|

    ^ (t := self top) notNil
        ifTrue: [t namespace]
        ifFalse:[nil]
!

documentAttributes: attributeList
	self document version: (attributeList at: 'version' ifAbsent: [nil]).
	self document encoding: (attributeList at: 'encoding' ifAbsent: [nil]).
	self document requiredMarkup: (attributeList at: 'requiredMarkup' ifAbsent: [nil]).
!

domDocument
    [self startDocument; parseDocument]
        ensure: [self driver stream close].
    ^document
!

endDocument
        | currentNode |
        #swYAXO.        "added wellformedness check"
        (currentNode := self pop) isDocument 
                ifFalse: [self driver malformedError: 'document element must be complete'].
        super endDocument
!

endElement: elementName
        | currentElement |
        #swYAXO.
        currentElement := self pop.
        "sw: added wellformedness check"
        currentElement isDocument ifTrue: [self driver malformedError: 'Invalid end tag'].
        "sw: added #asString"
        currentElement name asString = elementName 
                ifFalse: 
                        [self driver 
                                malformedError: 'End tag "' , elementName , '" doesn''t match "' , currentElement name , '".']
!

endElement:localName namespace:namespace namespaceURI:uri qualifiedName:qualifiedName 
    |currentElement|

    currentElement := self pop.
    (currentElement namespace isNil 
    or:[ currentElement namespace = self defaultNamespace ]) 
        ifTrue:[
            currentElement localName = localName ifFalse:[
                self driver 
                    malformedError:'End tag "' , localName , '" doesn''t match "' , currentElement localName , '".'
            ]
        ]
        ifFalse:[
            currentElement qualifiedName = qualifiedName ifFalse:[
                self driver 
                    malformedError:'End tag "' , qualifiedName , '" doesn''t match "' , currentElement qualifiedName , '".'
            ]
        ]
!

processingInstruction: piName data: dataString
	| newElement |
	newElement := XMLPI target: piName data: dataString.
	self top addEntity: newElement
!

startDocument
	self document: XMLDocument new.
	self push: self document 
!

startElement: elementName attributeList: attributeList
        | newElement |
        #swYAXO.
        newElement := XMLElement named: elementName attributes: attributeList.
        self incremental 
                ifFalse: 
                        [self stack isEmpty 
                                ifFalse: 
                                        ["sw: added check for only one document element"

                                        | top |
                                        ((top := self top) isDocument and: [top hasDocumentElement]) 
                                                ifTrue: [self driver malformedError: 'only one document element allowed'].
                                        top addEntity: newElement]].
        self push: newElement
!

startElement:localName namespaceURI:namespaceUri namespace:namespace attributeList:attributeList 
    |newElement|

    "newElement _ namespace = self defaultNamespace
        ifTrue: [XMLElement named: localName namespace: nil uri: nil attributes: attributeList]
        ifFalse: [XMLElement named: localName namespace: namespace uri: namespaceUri attributes: attributeList]."
    newElement := XMLElementWithNamespace 
                named:localName
                namespace:namespace
                uri:namespaceUri
                attributes:attributeList.
    self incremental ifFalse:[
        self stack isEmpty ifFalse:[
            self top addEntity: "addElement:" newElement
        ]
    ].
    self push:newElement
! !

!XMLDOMParser methodsFor:'initialize'!

initialize
	super initialize.
	stack := OrderedCollection new.
	incremental := false
! !

!XMLDOMParser methodsFor:'parsing'!

nextEntity
	| currentTop |
	currentTop := self top.
	[self driver nextEntity isNil
		or: [self top ~~ currentTop]] whileTrue.
	^entity
!

nextEntityStart
	[self driver nextEntity.
	self stack isEmpty] whileTrue.
	^entity
! !

!XMLDOMParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLDOMParser.st,v 1.7 2010/03/20 10:17:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLDOMParser.st,v 1.7 2010/03/20 10:17:41 cg Exp $'
! !
