"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

XMLNodeWithEntities subclass:#XMLElement
	instanceVariableNames:'name contents attributes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!


!XMLElement class methodsFor:'instance creation'!

named: aString
	^self new name: aString
!

named: aString attributes: attributeList
	^self new
		name: aString;
		setAttributes: attributeList
! !

!XMLElement methodsFor:'accessing'!

attributeAt: attributeName
	^self attributeAt: attributeName ifAbsent: [nil]
!

attributeAt: attributeName ifAbsent: aBlock
	^self attributes at: attributeName ifAbsent: [^aBlock value]
!

attributeAt: attributeName put: attributeValue
	self attributes at: attributeName asSymbol put: attributeValue
!

attributes
	^attributes
!

characterData
	^self contentString
!

contentString
	^(self contents size == 1
		and: [self contents first isKindOf: XMLStringNode])
		ifTrue: [self contents first string]
		ifFalse: ['']
!

contentStringAt: entityName
        ^(self entityAt: entityName ifAbsent: [^'']) contentString
!

contents
	contents ifNil: [contents := OrderedCollection new].
	^contents
!

elements
	^super elements , self contents
!

localName
    ^ name
!

name
    ^ self qualifiedName
!

qualifiedName
    |ns|

    ^ (ns := self namespace)
        isNil ifTrue: [self localName]
        ifFalse: [ns , ':' , self localName]
!

tag
	^name asSymbol
!

valueFor: aSymbol 
	^self attributes at: aSymbol ifAbsent: ['']
!

valueFor: aSymbol ifAbsent: aBlock 
	^self attributes at: aSymbol ifAbsent: aBlock
! !

!XMLElement methodsFor:'enumerating'!

contentsDo: aBlock
	#swYAXO.
	contents notNil ifTrue: 
			[
			self contents do: [:each | aBlock value: each]]
! !

!XMLElement methodsFor:'initialize'!

addContent: contentString
	self contents add: contentString
!

name: aString
	name := aString asSymbol
!

setAttributes: newAttributes
	attributes := newAttributes
! !

!XMLElement methodsFor:'printing'!

printXMLOn: writer
        writer startElement: self name attributeList: self attributes.
        (writer canonical not
        and: [self isEmpty 
        and: [self attributes notEmpty]])
                ifTrue: [writer endEmptyTag: self name]
                ifFalse: [
                        writer endTag.
                        self contentsDo: [:content | content printXMLOn: writer].
                        super printXMLOn: writer.
                        writer endTag: self name]
! !

!XMLElement methodsFor:'searching'!

firstTagNamed: aSymbol 
	"Return the first encountered node with the specified tag.
	If it is not the receiver, pass the message on"

	self tag == aSymbol ifTrue: [^self].
	^super firstTagNamed: aSymbol 
!

firstTagNamed: aSymbol with: aBlock
	"Return the first encountered node with the specified tag that allows
	the block to evaluate to true. Pass the message on"

	(self tag == aSymbol and: [aBlock value: self]) ifTrue: [^self].
	^super firstTagNamed: aSymbol with: aBlock.
!

tagsNamed: aSymbol contentsDo: aBlock
	"Evaluate aBlock for all of the contents of the receiver
	if the receiver tag equals aSymbol. Pass the message on"

	self tag == aSymbol ifTrue: [self contentsDo: aBlock].
	super tagsNamed: aSymbol contentsDo: aBlock
!

tagsNamed: aSymbol do: aOneArgumentBlock
	"If the receiver tag equals aSymbol, evaluate aOneArgumentBlock
	with the receiver. Continue the search"

	self tag == aSymbol ifTrue: [aOneArgumentBlock value: self].
	super tagsNamed: aSymbol do: aOneArgumentBlock
!

tagsNamed: aSymbol ifReceiverDo: aOneArgumentBlock
	"If the receiver tag equals aSymbol, evaluate aOneArgumentBlock with the receiver"

	self tag == aSymbol ifTrue: [aOneArgumentBlock value: self]
!

tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock
	"If the receiver tag equals aSymbol, evaluate aOneArgumentBlock
	with the receiver. Then recurse through all the children"

	self tag == aSymbol ifTrue: [aOneArgumentBlock value: self].
	super tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock
!

tagsNamed: aSymbol ifReceiverOrChildDo: aOneArgumentBlock
	"If the receiver tag equals aSymbol, evaluate aOneArgumentBlock with the receiver.
	For each of the receivers children do the same. Do not go beyond direct children"

	self tag == aSymbol ifTrue: [aOneArgumentBlock value: self].
	super tagsNamed: aSymbol ifReceiverDo: aOneArgumentBlock
! !

!XMLElement methodsFor:'testing'!

isEmpty
	^self entities isEmpty
		and: [self contents isEmpty]
!

isTag
	^true
! !

!XMLElement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLElement.st,v 1.8 2009/11/10 12:36:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLElement.st,v 1.8 2009/11/10 12:36:21 cg Exp $'
! !
