"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

Object subclass:#XMLNode
	instanceVariableNames:''
	classVariableNames:'CanonicalTable'
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!


!XMLNode methodsFor:'accessing'!

addContent: contentString
	SAXMalformedException signal: 'Illegal string data.'
! !

!XMLNode methodsFor:'enumerating'!

contentsDo: aBlock
! !

!XMLNode methodsFor:'printing'!

printOn: stream
        #swYAXO.        "sw: changed for debugging"
        "/ ^super printOn: stream
        "/ self printXMLOn: (XMLWriter on: stream)
        self printXMLOn: (XMLPrettyWriter on: stream)
!

printXMLOn: writer
	self subclassResponsibility
! !

!XMLNode methodsFor:'searching'!

firstTagNamed: aSymbol
	"Return the first encountered node with the specified tag. Pass the message on"

	| answer |
	#swYAXO.
	self contentsDo: 
			[:node | 
			(answer := node firstTagNamed: aSymbol) notNil ifTrue: 
					[
					^answer]].
	^nil
!

firstTagNamed: aSymbol with: aBlock
	"Return the first encountered node with the specified tag that
	allows the block to evaluate to true. Pass the message on"

	| answer |
	#swYAXO.
	self contentsDo: 
			[:node | 
			(answer := node firstTagNamed: aSymbol with: aBlock) notNil ifTrue: 
					[
					^answer]].
	^nil
!

tagsNamed: aSymbol childrenDo: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match"

	self contentsDo: [:each | 
		each tagsNamed: aSymbol ifReceiverDo: aOneArgumentBlock]
!

tagsNamed: aSymbol childrenDoAndRecurse: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match and recurse"

	self contentsDo: [:each | 
		each tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock]
!

tagsNamed: aSymbol contentsDo: aBlock
	"Evaluate aBlock for all of the contents of the receiver.
	The receiver has no tag, so pass the message on"

	self contentsDo: [:each | each tagsNamed: aSymbol contentsDo: aBlock]
!

tagsNamed: aSymbol do: aOneArgumentBlock
	"Search for nodes with tag aSymbol. When encountered evaluate aOneArgumentBlock"

	self contentsDo: [:each | each tagsNamed: aSymbol do: aOneArgumentBlock]
!

tagsNamed: aSymbol ifReceiverDo: aOneArgumentBlock
	"Handled only by XMLTagNode subclass"

!

tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock
	"Recurse all children"

	self contentsDo: [:each | each tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock]
!

tagsNamed: aSymbol ifReceiverOrChildDo: aOneArgumentBlock
	"Recurse all children"

	self contentsDo: [:each | each tagsNamed: aSymbol ifReceiverDo: aOneArgumentBlock]
! !

!XMLNode methodsFor:'testing'!

isDocument
	#swYAXO.
	^false
!

isProcessingInstruction
	^false
!

isTag
	^false
!

isText
	^false
! !

!XMLNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLNode.st,v 1.5 2009/10/29 19:15:39 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLNode.st,v 1.5 2009/10/29 19:15:39 cg Exp $'
! !
