"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

Object subclass:#XMLTokenizer
	instanceVariableNames:'stream nestedStreams entities externalEntities parameterEntities
		parsingMarkup peekChar validating buffer contentBuffer'
	classVariableNames:'CharEscapes NameChars NameDelimiters'
	poolDictionaries:''
	category:'XML-YAXO-Parser'
!

XMLTokenizer comment:'XMLTokenizer

bolot@cc.gatech.edu

breaks the stream of characters into a stream of XMLnodes (aka token stream)
token stream is used by XMLparser to generate XMLdocument tree

==========
Dolphin port notes (Steve Waring)
-Methods that have been changed or added are marked with #swYAXO
-Added instance variables: buffer, contentBuffer for buffering.
-Added class variable: NameChars for wellformedness check.'
!


!XMLTokenizer class methodsFor:'instance creation'!

new
        ^self basicNew initialize
!

on: aStream
	^self new parseStream: aStream
! !

!XMLTokenizer class methodsFor:'accessing'!

isCharEscape: aChar
	^CharEscapes includes: aChar
! !

!XMLTokenizer class methodsFor:'class initialization'!

initialize
	"
	XMLTokenizer initialize
	"

	| nameDelimiters |
	#swYAXO.        "added intialization of NameChars"
	CharEscapes := #($& $" $' $> $<) asSet.
	nameDelimiters := #(9 10 12 13 32 61 62 47).
	NameDelimiters := Array new: 256.
	NameDelimiters atAllPut: false.
	nameDelimiters do: [:each | NameDelimiters at: each put: true].

	"sw: can only mark common ASCII/UTF8 characters"
	NameChars := Array new: 256.
	NameChars atAllPut: true.
	1 to: 47 do: [:i | NameChars at: i put: false].
	91 to: 96 do: [:i | NameChars at: i put: false].
	123 to: 127 do: [:i | NameChars at: i put: false].
	'.-_:' do: [:each | NameChars at: each codePoint put: true]
! !

!XMLTokenizer class methodsFor:'examples'!

addressBookXML
	^'<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'
!

addressBookXMLWithDTD
	^'<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE addressbook SYSTEM "addressbook.dtd">
<?xml:stylesheet type="text/xsl" href="demo.xsl"?>
<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'
!

exampleAddressBook
	"
	XMLTokenizer exampleAddressBook
	"

	| tokenizer |
	#swYAXO.
	tokenizer := XMLTokenizer on: self addressBookXML readStream.
	[tokenizer next notNil] whileTrue: [].
	^tokenizer
!

exampleAddressBookWithDTD
	"
	XMLTokenizer exampleAddressBookWithDTD
	"

	| tokenizer |
	tokenizer := XMLTokenizer on: self addressBookXMLWithDTD readStream.
	[tokenizer next notNil] whileTrue: [].
	^tokenizer
! !

!XMLTokenizer methodsFor:'accessing'!

parseStream: aStream
	self stream: aStream
!

validating: aBoolean
	validating := aBoolean
! !

!XMLTokenizer methodsFor:'accessing-private'!

log: aString
	"Transcript show: aString; cr"
!

nestedStreams
	nestedStreams isNil ifTrue: [nestedStreams := OrderedCollection new].
	^nestedStreams
!

stream
	^stream
!

stream: newStream
	"Continue parsing from the new nested stream."
	stream := newStream
! !

!XMLTokenizer methodsFor:'entities'!

entities
	entities isNil ifTrue: [entities := self initEntities].
	^entities
!

entity: refName
	#swYAXO.        "sw: changed to malformedError"
	^self validating 
		ifTrue: 
			[self entities at: refName
				ifAbsentPut: [self malformedError: 'XML undefined entity ' , refName printString]]
		ifFalse: [DTDEntityDeclaration name: refName value: '']
!

entity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already one don't register the new value."

	self entities at: refName ifAbsentPut: [aReference]
!

externalEntities
	externalEntities isNil ifTrue: [externalEntities := Dictionary new].
	^externalEntities
!

externalEntity: refName
	^self entities
		at: refName
		ifAbsentPut: ['']
!

handleEntity: referenceString in: parsingContext 

	| entity entityValue |
	entity := self entity: referenceString.
	entityValue := entity valueForContext: parsingContext.
	(self class isCharEscape: entityValue)
		ifTrue: [entityValue := entity reference].
	self pushStream: (ReadStream on: entityValue asString)
!

initEntities
        | ents |
        #swYAXO.        "sw: changed to use LookupTable"
        ents := Dictionary "LookupTable" new.
        ents
                at: 'amp' put: (DTDEntityDeclaration name: 'amp' value: $&);
                at: 'quot' put: (DTDEntityDeclaration name: 'amp' value: $");
                at: 'apos' put: (DTDEntityDeclaration name: 'apos' value: $');
                at: 'gt' put: (DTDEntityDeclaration name: 'gt' value: $>);
                at: 'lt' put: (DTDEntityDeclaration name: 'lt' value: $<).
        ^ents

    "Modified: / 20-03-2012 / 22:05:40 / cg"
!

parameterEntities
	parameterEntities isNil ifTrue: [parameterEntities := Dictionary new].
	^parameterEntities
!

parameterEntity: refName
	^self parameterEntities
		at: refName
		ifAbsent: [self malformedError: 'XML undefined parameter entity ' , refName printString]
!

parameterEntity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already one don't register the new value."
	self parameterEntities at: refName ifAbsentPut: [aReference]
! !

!XMLTokenizer methodsFor:'errors'!

errorExpected: expectedString
        #swYAXO.        "sw: changed from #parseError"
        self malformedError: 'XML expected "' , expectedString printString , '" got: "' 
                                , (stream nextAvailable: 20) , '"'
!

malformedError: errorString
        SAXMalformedException
            raiseRequestWith:(stream lineNumber)
            errorString: errorString
!

parseError: errorString
    SAXParseException signal: errorString
! !

!XMLTokenizer methodsFor:'handling tokens'!

handleCData: aString
	self log: 'CData: ' , aString
!

handleComment: aString
	self log: 'Comment: ' , aString
!

handleEndDocument
	self log: 'End Doc '
!

handleEndTag: aString
	self log: 'End tag: ' , aString
!

handlePCData: aString
	self log: 'PCData: ' , aString
!

handlePI: piTarget data: piData
	self log: 'PI: ' , piTarget , ' data ' , piData
!

handleStartDocument
	self log: 'Start Doc'
!

handleStartTag: tagName attributes: attributes
	self log: 'Start tag: ' , tagName.
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]
!

handleStartTag: tagName attributes: attributes namespaces:namespaces
        self handleStartTag: tagName attributes: attributes
!

handleXMLDecl: attributes
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]
!

handleXMLDecl: attributes namespaces: namespaces
        self handleXMLDecl: attributes
! !

!XMLTokenizer methodsFor:'initialize'!

initialize
        #swYAXO.        "sw: changed to initialize buffer and contentBuffer"
        "sw: changed to validating := true. IMO, this mode is better named isCheckingWellFormedness. It does *not* validate"
        super initialize.
        parsingMarkup := false.
        validating := true.
        buffer := WriteStream on: (String new: 32).
        contentBuffer := CharacterWriteStream on: (String new: 32)

    "Modified: / 11-07-2011 / 23:56:22 / cg"
! !

!XMLTokenizer methodsFor:'modes-private'!

endParsingMarkup
	parsingMarkup := false
!

startParsingMarkup
	parsingMarkup := true
! !

!XMLTokenizer methodsFor:'streaming'!

atEnd
        nestedStreams notNil ifFalse: [
            ^peekChar isNil and: [self stream atEnd]
        ].
        self stream atEnd ifFalse: [^ false].
        self popNestingLevel.
        ^ self atEnd
!

checkNestedStream
	nestedStreams notNil
		ifTrue: [(peekChar isNil and: [self stream atEnd])
			ifTrue: [
				self popNestingLevel.
				self checkNestedStream]]
!

hasNestedStreams
	^nestedStreams notNil
!

next
        "Return the next character from the current input stream. If the current stream is at end pop to next nesting level if there is one.
        Due to the potential nesting of original document, included documents and replacment texts the streams are held in a stack representing the nested streams. The current stream is the top one."
        | nextChar |
        #swYAXO.
        nestedStreams notNil ifTrue: [self checkNestedStream].
        "sw: changed from ifNil:ifNotNil"
        peekChar isNil
                ifTrue: [
                        nextChar := self stream nextOrNil. "sw: was #next"
"/                        Transcript show:nextChar.
                ]
                ifFalse: [ 
                        nextChar := peekChar.
                        peekChar := nil
                ].
        ^nextChar
!

nextUpTo: delimiter
        | resultStream nextChar |

        (peekChar notNil and:[peekChar = delimiter]) ifTrue:[
            ^ ''
        ].

        resultStream := WriteStream on: (String new: 10).

        [self atEnd or: [(nextChar := self next) == delimiter]]
                whileFalse: [resultStream nextPut: nextChar].
        nextChar == delimiter
                ifFalse: [self malformedError: 'XML no delimiting ' , delimiter printString , ' found'].
        ^resultStream contents
!

nextUpToAll: delimitingString
        | string |

        self unpeek.
        Smalltalk isSmalltalkX ifTrue:[
            string := self stream upToAllExcluding:delimitingString.
        ] ifFalse:[
            string := self stream upToAll: delimitingString.
            Smalltalk isSmalltalkX ifFalse:[
                self stream skip: delimitingString size negated.
            ].
            (self stream next: delimitingString size) = delimitingString
                ifFalse: [self malformedError: 'XML no delimiting ' , delimitingString printString , ' found'
            ].
        ].
        ^ string
!

peek
        "Return the next character from the current input stream. If the current stream poop to next nesting level if there is one.
        Due to the potential nesting of original document, included documents and replacment texts the streams are held in a stack representing the nested streams. The current stream is the top one."

        #swYAXO.
        nestedStreams notNil ifTrue: [self checkNestedStream].
        peekChar isNil ifTrue: [
            peekChar := self stream nextOrNil.       "sw was #next"
"/            Transcript show:peekChar.
        ].
        ^peekChar
!

popNestingLevel
	nestedStreams notNil
		ifTrue: [
			self stream close.
			self stream: self nestedStreams removeLast.
			self nestedStreams size > 0
				ifFalse: [nestedStreams := nil]]
!

pushBack: aString
	| pushBackString |
	#swYAXO.
	pushBackString := peekChar
		isNil ifTrue: [aString]
		ifFalse: [peekChar asString , aString].
	peekChar := nil.
	self pushStream: (ReadStream on: pushBackString)
!

pushSeparators
	| nextChar |
	#swYAXO.        "sw: added for wellformedness checks"
	[(nextChar := self peek) notNil and: [nextChar isSeparator]] 
		whileTrue: [contentBuffer nextPut: self next].
	(nestedStreams notNil and: [self atEnd]) 
		ifTrue: 
			[self checkNestedStream.
			self pushSeparators]
!

pushStream: newStream
	"Continue parsing from the new nested stream."
	self unpeek.
	self nestedStreams addLast: self stream.
	self stream: newStream
!

skipRequiredSeparators
	| nextChar skipped |
	#swYAXO.        "sw: added for wellformedness checks"
	skipped := false.
	[(nextChar := self peek) notNil and: [nextChar isSeparator]] whileTrue: 
			[self next.
			skipped := true].
	skipped ifFalse: [self malformedError: 'Required S not found'].
!

skipSeparators
	| nextChar |
	[(nextChar := self peek) notNil
		and: [nextChar isSeparator]]
		whileTrue: [self next].
	(nestedStreams notNil and: [self atEnd])
		ifTrue: [
			self checkNestedStream.
			self skipSeparators]
!

skipUpTo: delimiter
	| nextChar |
	self unpeek.
	[self atEnd or: [(nextChar := self next) == delimiter]]
		whileFalse: [].
	nextChar == delimiter
		ifFalse: [self malformedError: 'XML no delimiting ' , delimiter printString , ' found']
!

topStream
	^nestedStreams notNil
		ifTrue: [self nestedStreams first]
		ifFalse: [self stream]
!

unpeek
	peekChar
		notNil ifTrue: [ 
			peekChar := nil.
			self stream skip: -1]
! !

!XMLTokenizer methodsFor:'testing'!

validating
	^validating
! !

!XMLTokenizer methodsFor:'testing-private'!

parsingMarkup
	^parsingMarkup
! !

!XMLTokenizer methodsFor:'tokenizing'!

checkAndExpandReference: parsingContext
	| referenceString nextChar |
	nextChar := self peek.
	self validating
		ifFalse: [^nil].
	nextChar == $&
		ifTrue: [
			self next.
			self peek == $#
				ifTrue: [^self pushStream: (ReadStream on: self nextCharReference asString)].
			referenceString := self nextLiteral.
			self next == $;
				ifFalse: [self errorExpected: ';'].
			self handleEntity: referenceString in: parsingContext ]
		ifFalse: [
			((nextChar == $%
				and: [self parsingMarkup])
				and: [parsingContext == #entityValue])
				ifTrue: [
					self skipSeparators.
					referenceString := self nextLiteral.
					self handleEntity: referenceString in: parsingContext]].

	self atEnd ifTrue: [self errorExpected: 'Character expected.'].
	^nextChar
!

conditionalInclude: conditionalKeyword
	conditionalKeyword = 'INCLUDE'
		ifTrue: [^true].
	conditionalKeyword = 'IGNORE'
		ifTrue: [^false].
	^self conditionalInclude: (self parameterEntity: conditionalKeyword) value
!

nextAttributeInto: attributes

	| attrName attrValue |
	#swYAXO. "sw: added check for duplicate attribute"
	attrName := self nextName.
	(attributes includesKey: attrName) ifTrue: [self malformedError: 'Duplicate attribute name'].
	self skipSeparators.
	self next == $=
		ifFalse: [self errorExpected: '='].
	self skipSeparators.
	attrValue := self nextAttributeValue.
	attributes at: attrName put: attrValue
!

nextAttributeInto: attributes namespaces: namespaces

        | attrName attrValue |
        #swYAXO. "sw: added check for duplicate attribute"
        attrName := self nextName.
        (attributes includesKey: attrName) ifTrue: [self malformedError: 'Duplicate attribute name'].
        self skipSeparators.
        self next == $=
                ifFalse: [self errorExpected: '='].
        self skipSeparators.
        attrValue := self nextAttributeValue.

        (self usesNamespaces and: [(attrName findString: 'xmlns') = 1]) 
        ifTrue: [
            attrName size > 6
                ifTrue: [namespaces at: (attrName copyFrom: 7 to: attrName size) put: attrValue]
                ifFalse: [namespaces at: attrName put: attrValue]]
        ifFalse: [
            attributes at: attrName put: attrValue]        
!

nextAttributeValue
        | delimiterChar nextChar nextPeek referenceString entity entityValue |
        #swYAXO.        "Modified to use buffers"
        delimiterChar := self next.
        (delimiterChar == $" or: [delimiterChar == $']) 
                ifFalse: [self errorExpected: 'Attribute value delimiter (" or '')'].
        buffer reset.
        
        [nextPeek := nextChar := self peek.
        nextChar isNil ifTrue: [self errorExpected: 'Character'].
        nextChar == $< ifTrue: [self malformedError: 'Attribute value included <.'].
        nextChar == $& 
                ifTrue: 
                        [self next.
                        self peek == $# 
                                ifTrue: 
                                        [nextPeek := nil.
                                        nextChar := self nextCharReference]
                                ifFalse: 
                                        [referenceString := self nextLiteral.
                                        self next == $; ifFalse: [self errorExpected: ';'].
                                        entity := self entity: referenceString.
                                        entityValue := entity valueForContext: #content.
                                        (self class isCharEscape: entityValue) 
                                                ifTrue: 
                                                        [nextPeek := nil.
                                                        nextChar := entityValue]
                                                ifFalse: 
                                                        [entityValue := entityValue asString.
                                                        entityValue isEmpty 
                                                                ifTrue: [nextPeek := nextChar := nil]
                                                                ifFalse: 
                                                                        [self pushStream: (ReadStream on: entityValue asString).
                                                                        nextPeek := nextChar := self next]]]]
                ifFalse: [self next].
        nextPeek == delimiterChar] 
                        whileFalse: [nextChar notNil ifTrue: [buffer nextPut: nextChar]].
        ^buffer contents
!

nextCDataContent
	| cdata |
	"Skip $[ "
	#swYAXO. "Add check for well-formedness"
	self next.
	cdata := self nextUpToAll: ']]>'.
	('*<!![CDATA[*' match: cdata) ifTrue: [self malformedError: 'Nested CDATA sections'].
	self handleCData: cdata
!

nextCDataOrConditional

        | nextChar conditionalKeyword |
        "Skip ["
        #swYAXO.
        self next.
        "self skipSeparators" "sw: not wellformed"

        nextChar := self peek.
        nextChar == $%
                ifTrue: [
                        self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
                        conditionalKeyword := self nextLiteral.
                        self skipSeparators.
                        self next == $[
                                ifTrue: [
                                                self skipSeparators.
                                                ^ self nextIncludeSection: (self conditionalInclude: conditionalKeyword)]
                                ifFalse: [^ self errorExpected: '[' ]].

        nextChar == $C
                ifTrue: [
                        self nextLiteral = 'CDATA'
                                ifTrue: [^self peek == $[
                                                        ifTrue: [self nextCDataContent]
                                                        ifFalse: [self errorExpected: '[' ]]
                                ifFalse: [^self errorExpected: 'CData']].
        self errorExpected: 'CData or declaration'
!

nextCharReference
    | base numberString charValue |

    #swYAXO.        "changed for Dolphin's Number parsing"

    self next == $# ifFalse: [self errorExpected: '#-character reference'].
    self peek == $x ifTrue: [
        self next.
        base := '16r'
    ] ifFalse: [
        base := '10r'
    ].
    numberString := self nextUpTo: $;.
    charValue := [
                    Number fromString: (base , numberString "asUppercase")
                 ] on: Error do: [:ex | 
                    self breakPoint:#cg. 
                    self errorExpected: 'Number'
                 ].
    ^ charValue > 255 ifTrue: [$-] ifFalse: [charValue asCharacter]

    "
     Number fromString:'16r10' 
    "
!

nextComment
        "Skip first -"

        | string |
        #swYAXO.        "sw: check for comment wellformedness"
        self next.
        self next == $- ifFalse: [self malformedError: 'second comment $-'].
        string := self nextUpToAll: '--'.
        self next == $> ifFalse: [self malformedError: '-- to be followed by >'].
        self handleComment: string
!

nextEndTag
	| string |
	"Skip /"
	#swYAXO. "sw: changed to use buffer"

	Smalltalk isSmalltalkX ifTrue:[
	    contentBuffer position = contentBuffer class zeroPosition ifFalse: [
		self handlePCData: contentBuffer contents
	    ].
	] ifFalse:[
	    contentBuffer position = 0 ifFalse: [
		self handlePCData: contentBuffer contents
	    ].
	].
	self next.
	self skipSeparators.
	string := (self nextUpTo: $>) trimBlanks. "sw: was #withBlanksTrimmed."
	self handleEndTag: string
!

nextEntity
	"return the next XMLnode, or nil if there are no more"

	"branch, depending on what the first character is"

	"nestedStreams notNil ifTrue: [self halt]."

	#swYAXO.        "sw: changed to use buffer"
	contentBuffer reset.
	self pushSeparators.
	self atEnd 
		ifTrue: 
			[self handleEndDocument.
			^nil].
	self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
	^self peek = $< ifTrue: [self nextNode] ifFalse: [self nextPCData]
!

nextEntityValue
        | delimiterChar entityValueStream nextChar nextPeek referenceString entity entityValue |
        #swYAXO.        "sw: changed from ifNil: etc"
        delimiterChar := self next.
        (delimiterChar == $" or: [delimiterChar == $']) 
                ifFalse: [self errorExpected: 'Entity value delimiter'].
        entityValueStream := WriteStream on: String new.
        
        [nextPeek := nextChar := self peek.
        nextChar isNil ifTrue: [self errorExpected: 'Character'].
        nextChar == $& 
                ifTrue: 
                        [self next.
                        self peek == $# 
                                ifTrue: 
                                        [nextPeek := nil.
                                        nextChar := self nextCharReference]
                                ifFalse: 
                                        [referenceString := self nextLiteral.
                                        self next == $; ifFalse: [self errorExpected: ';'].
                                        entity := self entity: referenceString.
                                        entityValue := entity valueForContext: #entityValue.
                                        self pushStream: (ReadStream on: entityValue asString).
                                        nextPeek := nextChar := self next]]
                ifFalse: 
                        [nextChar == $% 
                                ifTrue: 
                                        [self skipSeparators.
                                        referenceString := self nextLiteral.
                                        nextChar := self handleEntity: referenceString in: #entityValue.
                                        nextPeek := nextChar := self next]
                                ifFalse: [self next]].
        nextPeek == delimiterChar] 
                        whileFalse: [nextChar notNil ifTrue: [entityValueStream nextPut: nextChar]].
        ^entityValueStream contents
!

nextIncludeSection: parseSection
	| section |
	"Read the file up to the next include section delimiter and parse it if parseSection is true"

        
	section := self nextUpToAll: ']]>'.
	parseSection
		ifTrue: [
			self pushStream: (ReadStream on: section)]
!

nextLiteral
        | resultStream nextChar validChars resultString |
        #swYAXO.
        validChars := ':-_.'. "/ ##(':-_.' asSet).
        resultStream := (String new: 10) writeStream.
        ((nextChar := self peek) isLetter
                or: [nextChar == $_])
                ifFalse: [self errorExpected: 'Name literal'].
        [
            nextChar := self peek.
            (nextChar isLetter 
            or: [nextChar isDigit 
            or: [validChars includes: nextChar]]) not
                    ifTrue: [
                            resultString := resultStream contents.
                            resultString isEmpty
                                    ifTrue: [self errorExpected: 'Name literal']
                                    ifFalse: [^resultString]]
                    ifFalse: [
                            nextChar == $&
                                    ifTrue: [
                                            nextChar := self next.
                                            self peek == $# ifTrue:[
                                                resultStream nextPut:self nextCharReference
                                            ] ifFalse:[
                                                ^resultStream contents]]
                                    ifFalse: [
                                            resultStream nextPut: self next]].
            nil
        ] repeat
!

nextName
        | nextChar |
        #swYAXO.        "sw: changed to use buffers"
        buffer reset.
        nextChar := self peek.
        nextChar isNil ifTrue: [self errorExpected: 'Character'].
        "sw: changed to check first char of name for wellformedness"
        (nextChar isDigit or: [nextChar == $. or: [nextChar == $-]]) 
                ifTrue: [self malformedError: 'Illegal first character for a name'].
        
        [nextChar := self peek.
        nextChar isNil ifTrue: [self errorExpected: 'Character'].
        NameDelimiters at: nextChar codePoint   "sw: changed from asciiValue"] 
                        whileFalse: 
                                ["sw: changed to check name for wellformedness"

                                (NameChars at: nextChar codePoint) ifFalse: [self malformedError: 'Illegal name'].
                                buffer nextPut: self next].
        ^buffer contents
!

nextNode
	| nextChar |
	"Skip < "
	self next.
	nextChar := self peek.
	nextChar == $!! ifTrue: [
		"Skip !!"
		self next.
		nextChar := self peek.
		nextChar == $- ifTrue: [^self nextComment].
		nextChar == $[ ifTrue: [^self nextCDataOrConditional].
		^self parsingMarkup
			ifTrue: [self nextMarkupDeclaration]
			ifFalse: [self nextDocType]].
	nextChar == $? ifTrue: [^self nextPI].
	^self nextTag
!

nextPCData
        | nextChar referenceString entity entityValue nextPeek |
        #swYAXO.        "sw: changed to use buffers"
        self validating
                ifFalse: [
                        [self peek == $<]
                                whileFalse: [contentBuffer nextPut: self next].
                        ^self handlePCData: contentBuffer contents].

        [
        nextPeek := nextChar := self peek.
        nextChar isNil ifTrue: [self malformedError: 'Character expected.'].
        nextChar == $&
                ifTrue: [
                        self next.
                        self peek == $#
                                ifTrue: [
                                        nextPeek := nil.
                                        nextChar := self nextCharReference]
                                ifFalse: [
                                        referenceString := self nextLiteral.
                                        self next == $;
                                                ifFalse: [self errorExpected: ';'].
                                        entity := self entity: referenceString.
                                        entityValue := entity valueForContext: #content.
                                        (self class isCharEscape: entityValue)
                                                ifTrue: [
                                                        nextPeek := nil.
                                                        nextChar := entityValue]
                                                ifFalse: [
                                                        entityValue := entityValue asString.
                                                        entityValue isEmpty
                                                                ifTrue: [nextPeek := nextChar := nil]
                                                                ifFalse: [
                                                                        self pushStream: (ReadStream on: entityValue asString).
                                                                        nextPeek := nextChar := self peek]]]]
                ifFalse: [nextPeek == $< ifFalse: [self next]].
        nextPeek == $<]
                whileFalse: [
                        nextChar notNil ifTrue: [contentBuffer nextPut: nextChar]].
        self handlePCData: contentBuffer contents
!

nextPI
	| piTarget piData |
	"Skip ?"
	self next.
	piTarget := self nextLiteral.
	piTarget asUppercase = 'XML'
		ifTrue: [^self nextXMLDecl].
	self skipSeparators.
	piData := self nextUpToAll: '?>'.
	self handlePI: piTarget data: piData
!

nextPubidLiteral
	^self nextAttributeValue
!

nextSystemLiteral
	^self nextAttributeValue
!

nextTag
        | tagName attributes nextChar namespaces|
        #swYAXO.
        self peek = $/ ifTrue: [^self nextEndTag].
        tagName := self nextName.
        self skipSeparators.
        "sw: changed from Dictionary"
        "/ attributes := LookupTable new.
        "cg: back to Dictionary"
        attributes := Dictionary new.
        namespaces := Dictionary new: 5.
        [(nextChar := self peek) == $> or: [nextChar == $/]] whileFalse: 
                        [self checkAndExpandReference: #content.
                        self nextAttributeInto: attributes namespaces: namespaces.
                        self skipSeparators].
        self handleStartTag: tagName attributes: attributes namespaces: namespaces.
        self next == $/ 
                ifTrue: 
                        [self handleEndTag: tagName.
                        "sw: added wellformedness check"
                        self next == $> ifFalse: [self malformedError: 'Invalid Empty Element Tag']]
!

nextXMLDecl
        | attributes nextChar namespaces|
        #swYAXO.
        self skipSeparators.
        "sw: changed from Dictionary"
        "/ attributes := LookupTable new.
        "cg: changed to Dolphin::LookupTable"
        "cg: changed to Dictionary"
        "/ attributes := Dolphin::LookupTable new.
        attributes := Dictionary new.
        namespaces _ Dictionary new.
        [(nextChar := self peek) == $?] whileFalse: 
                        [self nextAttributeInto: attributes namespaces: namespaces.
                        self skipSeparators].
        self next.
        self next == $> ifFalse: [self errorExpected: '>'].
        self handleXMLDecl: attributes namespaces: namespaces
! !

!XMLTokenizer methodsFor:'tokenizing dtd'!

endDocTypeDecl
	"Skip ]>"
	self next; next.
	^nil
!

nextAttributeListDeclaration
	#swYAXO. "sw: addition for checking well-formedness"

	self skipSeparators.
	self nextName isEmpty ifTrue: [self malformedError: 'ATTLIST: a name is required'].
	self skipSeparators.
	self skipUpTo: $>

        
!

nextDocType
        | declType |
        declType := self nextLiteral.
        declType = 'DOCTYPE'
                ifTrue: [
                        self startParsingMarkup.
                        ^self nextDocTypeDecl].
        self malformedError: 'markup declaration, not ' , declType printString
!

nextDocTypeDecl
	| nextChar |
	self skipSeparators.
	self nextLiteral.
	self skipSeparators.
	self peek == $[
		ifFalse: [[nextChar := self peek.
				nextChar == $> or: [nextChar == $[ ]] whileFalse: [self next]].
	self peek == $[
		ifTrue: [
			self next.
			[self skipSeparators.
			self peek == $]] whileFalse: [
				self checkAndExpandReference: #dtd.
				self nextNode].
			self next == $] 
				ifFalse: [self errorExpected: ']' ]].
	self skipSeparators.
	self next == $>
		ifFalse: [self errorExpected: '>' ].

	self endParsingMarkup
!

nextElementDeclaration
	#swYAXO.        "sw: addition for checking well-formedness"
	self skipSeparators.
	self nextName isEmpty ifTrue: [self malformedError: 'ELEMENT: a name is required'].
	self skipSeparators.
	self skipUpTo: $>
!

nextEntityDeclaration
        | entityName entityDef referenceClass reference |
        #swYAXO.
        self skipRequiredSeparators.    "sw: check for wellformedness"
        self peek == $% 
                                ifTrue: 
                                        [self next.
                                        self skipRequiredSeparators.    "sw: check for wellformedness"
                                        referenceClass := DTDParameterEntityDeclaration]
                                ifFalse: [referenceClass := DTDEntityDeclaration].
        "sw: changed from #nextLiteral"
        entityName := self nextName.
        self skipSeparators.
        (self peek == $" or: [self peek == $']) 
                                ifTrue: [entityDef := self nextEntityValue]
                                ifFalse: [entityDef := self nextExternalId].
        self skipUpTo: $>.
        reference := referenceClass name: entityName value: entityDef.
        reference registerIn: self.
        ^reference
!

nextExternalId
	| extDefType systemId dir |
	#swYAXO.        "sw: removed parsing of external DTD (not required for a non-validating processor"
	extDefType := self nextLiteral.
	extDefType = 'PUBLIC' 
		ifTrue: 
			[self skipRequiredSeparators.
			self nextPubidLiteral.
			self skipSeparators.
			self peek == $> ifFalse: [systemId := self nextSystemLiteral]].
	extDefType = 'SYSTEM' 
		ifTrue: 
			[self skipRequiredSeparators.
			systemId := self nextSystemLiteral].
	^nil
	"systemId
		ifNil: [^nil].
	dir := self topStream directory.
	^(dir fileExists: systemId)
		ifTrue: [(dir readOnlyFileNamed: systemId) contentsOfEntireFile]
		ifFalse: ['']"
!

nextMarkupDeclaration
	| declType |
	#swYAXO.        "sw: added extra cases to check for wellformedness"
	declType := self nextLiteral.
	self validating ifFalse: [^self skipMarkupDeclaration].
	declType = 'ENTITY' ifTrue: [^self nextEntityDeclaration].
	declType = 'NOTATION' ifTrue: [^self nextNotationDeclaration].
	declType = 'ATTLIST' ifTrue: [^self nextAttributeListDeclaration].
	declType = 'ELEMENT' ifTrue: [^self nextElementDeclaration].
	self malformedError: 'Unknown markup declaration'
!

nextNotationDeclaration
	| idType |
	"Transcript
		show: 'nextNotationDeclaration';
		cr;
		flush."
	self skipSeparators.
	self nextName.
	self skipSeparators.
	idType := self nextLiteral.
	idType  = 'PUBLIC'
		ifTrue: [
			self skipSeparators.
			self nextPubidLiteral.
			self skipSeparators.
			self peek == $>
				ifFalse: [self nextSystemLiteral]].

	idType  = 'SYSTEM'
		ifTrue: [
			self skipSeparators.
			self nextSystemLiteral].
	self skipSeparators.
	self next == $>
		ifFalse: [self errorExpected: '>' ].

!

skipMarkupDeclaration
	self skipUpTo: $>
! !

!XMLTokenizer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLTokenizer.st,v 1.23 2012/03/21 00:12:18 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLTokenizer.st,v 1.23 2012/03/21 00:12:18 cg Exp $'
! !

XMLTokenizer initialize!
