"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

Object subclass:#XMLWriter
	instanceVariableNames:'stream stack scope scanner canonical flushStream physicalStream'
	classVariableNames:'XMLTranslation XMLTranslationMap'
	poolDictionaries:''
	category:'XML-YAXO-Writer'
!


!XMLWriter class methodsFor:'instance creation'!

on: aStream
	^self basicNew initialize stream: aStream
! !

!XMLWriter class methodsFor:'class initialization'!

initialize
        "
        XMLWriter initialize
        "

        XMLTranslation := Dictionary new.
        XMLTranslation
                at: Character cr put: '&#13;';
                at: Character lf put: '&#10;';
                at: Character tab put: '&#9;';
                at: $& put: '&amp;';
                at: $< put: '&lt;';
                at: $> put: '&gt;';
"               at: $' put: '&apos;'; "
                at: $" put: '&quot;';
                yourself.

        XMLTranslationMap := ByteArray new: 256.
        XMLTranslation keysDo:[:ch| XMLTranslationMap at: ch asciiValue+1 put: 1].
! !

!XMLWriter methodsFor:'accessing'!

canonical
	^canonical
!

canonical: aBoolean
	canonical := aBoolean
!

doBuffer:aBoolean
    aBoolean ifTrue:[
        physicalStream := stream.
        stream := WriteStream on:(String new).
    ] ifFalse:[
        stream := physicalStream ? stream.
    ].
!

stream
    ^ physicalStream
!

stream: aStream
    physicalStream := aStream.
    stream isNil ifTrue:[
        stream := physicalStream.
    ].
! !

!XMLWriter methodsFor:'accessing-private'!

stack
	^stack
! !

!XMLWriter methodsFor:'initialize'!

initialize
	stack := OrderedCollection new.
	canonical := false.
	scope := XMLNamespaceScope new
! !

!XMLWriter methodsFor:'namespaces'!

declareNamespace: ns uri: uri
	self scope declareNamespace: ns uri: uri
!

defaultNamespace
	^self scope defaultNamespace
!

defaultNamespace: ns
	"Declare the default namespace."
	self scope defaultNamespace: ns
!

enterScope
	self scope enterScope
!

leaveScope
	self scope leaveScope
! !

!XMLWriter methodsFor:'private'!

scope
	^scope
! !

!XMLWriter methodsFor:'tags-private'!

endCData
        stream nextPutAll: ']]>'
!

endComment
        stream nextPutAll: ' -->'
!

endPI
        stream nextPutAll: ' ?>'
!

popTag: tagName
        | stack stackTop |

        stack := self stack.
        stackTop := stack isEmpty
                ifTrue: ['<empty>']
                ifFalse: [stack last].
        ^ stackTop = tagName
                ifTrue: [stack removeLast]
                ifFalse: [self error: 'Closing tag "' , tagName , '" does not match "' , stackTop]
!

pushTag: tagName
	self stack add: tagName
!

startCData
        stream nextPutAll: '<!![CDATA['
!

startComment
        stream nextPutAll: '<-- '
!

startPI: identifier
        stream
                nextPutAll: '<?';
                nextPutAll: identifier;
                space
! !

!XMLWriter methodsFor:'writing dtd'!

endDecl: type
	self endTag
!

endDeclaration
        stream
                cr;
                nextPut: $].
        self endTag
!

startDecl: type
        stream
                nextPutAll: '<!!';
                nextPutAll: type asUppercase;
                space
!

startDecl: type named: aString
        stream
                nextPutAll: '<!!';
                nextPutAll: type asUppercase;
                space;
                nextPutAll: aString;
                space
!

startDeclaration: dtdName
        self startDecl: 'DOCTYPE' named: dtdName.
        stream
                nextPut: $[;
                cr
! !

!XMLWriter methodsFor:'writing xml'!

attribute: attributeName value: attributeValue
        stream
                space;
                nextPutAll: attributeName.
        self
                eq;
                putAsXMLString: attributeValue.

        flushStream == true ifTrue:[        
            self flush
        ].
!

cdata: aString
        self startCData.
        stream nextPutAll: aString.
        self endCData
!

comment: aString
        self startComment.
        stream nextPutAll: aString.
        self endComment
!

endEmptyTag: tagName
        self popTag: tagName.
        stream nextPutAll: '/>'.
        self canonical
                ifFalse: [stream space]
!

endTag
        stream nextPut: $>.
        "self canonical
                ifFalse: [stream space]"
!

endTag: tagName
	self endTag: tagName xmlns: nil.
!

endTag: tagName xmlns: xmlns
        self popTag: tagName.
        stream
                nextPutAll: '</'.
        (xmlns notNil
                and: [xmlns ~= self defaultNamespace])
                ifTrue: [stream
                        nextPutAll: xmlns;
                        nextPut: $:].
        stream nextPutAll: tagName.
        self endTag.
!

flush
    |data|

    physicalStream ~= stream ifTrue:[
        data := stream contents.

"/        Transcript showCR:'Outgoing:'.
"/        Transcript showCR:data.
        
        physicalStream nextPutAll:data.
        stream := WriteStream on:(String new).
    ].
    physicalStream flush
!

pcData: aString
        aString do: [:c |
            |xlation|

            xlation := XMLTranslation at: c ifAbsent:nil.
            xlation isNil ifTrue:[
                stream nextPut: c
            ] ifFalse:[
                stream nextPutAll: xlation
            ]
        ].

"/pcData: aString
"/        | lastIndex nextIndex |
"/        lastIndex := 1.
"/        "Unroll the first search to avoid copying"
"/        nextIndex := String findFirstInString: aString inSet: XMLTranslationMap startingAt: lastIndex.
"/        nextIndex = 0 ifTrue:[^stream nextPutAll: aString].
"/        [stream nextPutAll: (aString copyFrom: lastIndex to: nextIndex-1).
"/        stream nextPutAll: (XMLTranslation at: (aString at: nextIndex)).
"/        lastIndex := nextIndex + 1.
"/        nextIndex := String findFirstInString: aString inSet: XMLTranslationMap startingAt: lastIndex.
"/        nextIndex = 0] whileFalse.
"/        stream nextPutAll: (aString copyFrom: lastIndex to: aString size).

    "Modified: / 12-07-2011 / 00:02:25 / cg"
!

pi: piTarget data: piData
        self startPI: piTarget.
        stream nextPutAll: piData.
        self endPI
!

startElement: elementName attributeList: attributeList
        self canonical
                ifFalse: [self crIndent].
        self startTag: elementName.
        attributeList keys asSortedCollection do: [:key |
                self attribute: key value: (attributeList at: key)]
!

startTag: tagName
	self startTag: tagName xmlns: nil
!

startTag: tagName xmlns: xmlns
        stream
                nextPut: $<.
        (xmlns notNil
                and: [xmlns ~= self scope defaultNamespace])
                ifTrue: [stream
                        nextPutAll: xmlns;
                        nextPut: $:].
        stream
                nextPutAll: tagName.
        "self canonical
                ifFalse: [stream space]."
        self pushTag: tagName
!

xmlDeclaration: versionString
	self canonical
		ifFalse: [
			self
				startPI: 'XML';
				attribute: 'version' value: versionString;
				endPI]
!

xmlDeclaration: versionString encoding: encodingString
        self canonical
                ifFalse: [
                        self
                                startPI: 'xml';
                                attribute: 'version' value: versionString;
                                attribute: 'encoding' value: encodingString;
                                endPI.
                        self flush]
! !

!XMLWriter methodsFor:'writing xml-private'!

crIndent
    stream cr.
!

eq
        stream nextPut: $=
!

putAsXMLString: aValue
        stream nextPut: $".
        self pcData: aValue.
        stream nextPut: $"
! !

!XMLWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLWriter.st,v 1.13 2011/07/11 22:06:03 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/XMLWriter.st,v 1.13 2011/07/11 22:06:03 cg Exp $'
! !

XMLWriter initialize!
