"{ Package: 'stx:goodies/xml/yaxo' }"

"{ NameSpace: YAXO }"

Object subclass:#XMLNamespaceScope
	instanceVariableNames:'scope currentBindings useNamespaces validateAttributes'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-YAXO-Writer'
!


!XMLNamespaceScope methodsFor:'accessing'!

defaultNamespace
        ^self currentScope first
!

defaultNamespace: ns
        "Declare the default namespace."
        self currentScope at: 1 put: ns
!

namespaceURIOf: ns
	"Retrieve the URI of the given namespace prefix, if it is defined. A nil namespace
	returns the global namespace"

	^ self namespaceURIOf: ns ifAbsent: [ nil ]
!

namespaceURIOf: ns ifAbsent: aBlock
        "Retrieve the URI of the given namespace prefix, if it is defined. 
        A nil namespace returns the default namespace. 
        If no namespace can be found the value of the block is returned"

        ^ns isNil 
            ifTrue: [self defaultNamespace]
            ifFalse: [currentBindings at: ns ifAbsent: aBlock]
!

namespaces
	^currentBindings
! !

!XMLNamespaceScope methodsFor:'private'!

currentScope
        ^self scope last
!

establishLocalBindings
        (self currentScope at: 2)
                isNil ifTrue: [
                        currentBindings := currentBindings copy.
                        self currentScope at: 2 put: currentBindings]
!

initScope
	scope := OrderedCollection new: 20.
	currentBindings := Dictionary new.
	scope addLast: {'http://www.w3.org/TR/REC-xml-names'. currentBindings. nil. }.
!

namespaceAliases: namespace
	"Locate all namespaces that are aliases of the given URI."

	| aliases uri |
	aliases _ Set new.
	uri _ self namespaceURIOf: namespace ifAbsent: [self parseError: 'Attribute refers to undefined namespace ' , namespace asString ].
	currentBindings keysAndValuesDo: [:ns :u |
		(u = uri
			and: [ns ~= namespace])
			ifTrue: [aliases add: ns]].
	^ aliases
!

scope
        scope isNil ifTrue: [self initScope].
        ^scope
! !

!XMLNamespaceScope methodsFor:'scope'!

declareNamespace: ns uri: uri
        "Declare the given name space prefix with the given URL"

        ns = 'xmlns'
                ifTrue: [^self defaultNamespace: uri].
        self establishLocalBindings.
        currentBindings removeKey: ns ifAbsent: [].
        currentBindings at: ns put: uri
!

enterScope
        self scope addLast: (Array with:self defaultNamespace
                                   with:nil 
                                   with:currentBindings )
!

leaveScope
        | leftScope |
        leftScope := self scope removeLast.
        currentBindings := (self currentScope at: 2).
        currentBindings isNil ifTrue: [currentBindings := leftScope at: 3]
! !

!XMLNamespaceScope methodsFor:'validation'!

validateAttributes:attributeList 
    "check all attribute namespaces are defined and not duplicated by aliasing"
    
    |namespace localName|

    attributeList 
        keysDo:[:attrName | 
            self splitName:attrName
                into:[:ns :ln | 
                    namespace := ns.
                    localName := ln
                ].
            namespace 
                ifNotNil:[
                    (self namespaceAliases:namespace) do:[:alias | 
                        (attributeList includesKey:alias , ':' , localName) ifTrue:[
                            self 
                                parseError:'Attributes ' , attrName , ' and ' , alias , ':' , localName 
                                        , ' are aliased to namespace ' , (self namespaceURIOf:namespace)
                        ]
                    ]
                ]
        ]
! !

!XMLNamespaceScope class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/yaxo/YAXO__XMLNamespaceScope.st,v 1.4 2009/11/09 23:50:40 cg Exp $'
! !
