"{ Package: 'stx:libcompat' }"

"{ NameSpace: Dolphin }"

LookupTable subclass:#PluggableLookupTable
	instanceVariableNames:'searchPolicy'
	classVariableNames:''
	poolDictionaries:''
	category:'Compatibility-Dolphin'
!

!PluggableLookupTable class methodsFor:'documentation'!

documentation
"
    Just enough to make Spray happy ...

    Dolphin compatibility class.
    This may be required when existing code has to be ported to ST/X;
    however, it may not be complete and more protocol may be added in the future.
    The code here was created when public domain code (Manchester) had to
    be ported to ST/X and missing classes/methods were encountered, and code added
    by reasoning 'what the original class could probably do there'.

    This is currently not used by ST/X itself.

    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
"
!

examples
"
    |dict|

    dict := PluggableLookupTable new searchPolicy: SearchPolicy caseInsensitive.
    dict at:'abc'   put:1.
    dict at:'hello' put:2.
    dict at:'aBc'.    
    dict at:'HELLO'
"
! !

!PluggableLookupTable methodsFor:'accessing'!

searchPolicy:something
    "set the value of the instance variable 'searchPolicy' (automatically generated)"

    searchPolicy := something.
! !

!PluggableLookupTable methodsFor:'private'!

compareSame:element1 with:element2
    "compare two elements for being the same."

    element2 == DeletedEntry ifTrue:[^ false].
    element1 == DeletedEntry ifTrue:[^ false].
    ^ searchPolicy compareSame:element1 with:element2

    "Modified: / 23-09-2011 / 21:59:40 / cg"
!

find:key ifAbsent:aBlock
    "Look for the key in the receiver.  If it is found, return
     the index of the slot containing the key, otherwise
     return the value of evaluating aBlock."

    |index  "{ Class:SmallInteger }"
     length "{ Class:SmallInteger }"
     startIndex probe|

    length := keyArray basicSize.

    startIndex := index := self initialIndexForKey:key.

    [true] whileTrue:[
        probe := (keyArray basicAt:index).
        (self compareSame:key with:probe) ifTrue:[^ index].    "<<<< different from inherited"
        (self slotIsEmpty:probe) ifTrue:[^ aBlock value].

        index == length ifTrue:[
            index := 1
        ] ifFalse:[
            index := index + 1
        ].
        index == startIndex ifTrue:[^ aBlock value].
    ]

    "Modified: / 23-09-2011 / 21:58:11 / cg"
!

findKeyOrNil:key
    "Look for the key in the receiver.  
     If it is found, return the index of the first unused slot. 
     Grow the receiver, if key was not found, and no unused slots were present"

    |index  "{ Class:SmallInteger }"
     length "{ Class:SmallInteger }"
     startIndex probe 
     delIndex|

    length := keyArray basicSize.
    startIndex := index := self initialIndexForKey:key.

    [true] whileTrue:[
        probe := keyArray basicAt:index.
        (self compareSame:key with:probe) ifTrue:[^ index].
        (self slotIsEmpty:probe) ifTrue:[
            delIndex isNil ifTrue:[^ index].
            keyArray basicAt:delIndex put:nil.
            ^ delIndex
        ].

        probe == DeletedEntry ifTrue:[
            delIndex isNil ifTrue:[
                delIndex := index
            ]
        ].

        index == length ifTrue:[
            index := 1
        ] ifFalse:[
            index := index + 1
        ].
        index == startIndex ifTrue:[
            delIndex notNil ifTrue:[
                keyArray basicAt:delIndex put:nil.
                ^ delIndex
            ].
            ^ self grow findKeyOrNil:key
        ].
    ]

    "Modified: / 30.10.1997 / 16:01:57 / cg"
!

hashFor:aKey
    "return aKeys hash value - must correspond to compareSame:with:"

    ^ searchPolicy hashFor:aKey
! !

!PluggableLookupTable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libcompat/Dolphin__PluggableLookupTable.st,v 1.3 2011/10/01 11:09:18 vrany Exp $'
!

version_SVN
    ^ 'Id: Dolphin__PluggableLookupTable.st,v 1.1 2011/07/05 18:09:55 cg Exp '
! !
