"{ Package: 'stx:libcompat' }"!

!Behavior methodsFor:'Compatibility-Squeak'!

kindOfSubclass
    "Answer a String that is the keyword that describes the receiver's kind
    of subclass, either a regular subclass, a variableSubclass, a
    variableByteSubclass, a variableWordSubclass, or a weakSubclass."

    self isWeakPointers ifTrue:[
	^ ' weakSubclass:'
    ].
    ^ ' ',self firstDefinitionSelectorPart

    "
     Array kindOfSubclass
     WeakArray kindOfSubclass
    "

    "Modified: / 12-09-2010 / 09:46:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Behavior methodsFor:'Compatibility-Squeak'!

setTraitComposition: aTraitComposition
    "
    | oldComposition |
    (self hasTraitComposition not and: [aTraitComposition isEmpty]) ifTrue: [^self].
    aTraitComposition assertValidUser: self.

    oldComposition := self traitComposition.
    self traitComposition: aTraitComposition.
    self applyChangesOfNewTraitCompositionReplacing: oldComposition.

    oldComposition traits do: [:each | each removeUser: self].
    aTraitComposition traits do: [:each | each addUser: self]
    "

    "Now, Smalltalk/X doesnt support traits"
    aTraitComposition notNil ifTrue:[
        self error:'Smalltalk/X doesn''t support traits!!'
    ].

    "Created: / 29-08-2010 / 08:34:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 29-11-2011 / 11:20:36 / cg"
! !

!ChangeSet class methodsFor:'Compatibility-Squeak'!

uniqueNameLike: baseName

    ^self new name: baseName

    "Created: / 12-09-2010 / 09:29:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CharacterArray methodsFor:'Compatibility-Squeak'!

translated

    ^self

    "Created: / 26-11-2010 / 12:24:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CharacterArray methodsFor:'Compatibility-Squeak'!

unescapePercents
    "decode %xx form.  This is the opposite of #encodeForHTTP"
    ^ self unescapePercentsWithTextEncoding: 'utf-8'.

    "Created: / 14-09-2010 / 19:18:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CharacterArray methodsFor:'Compatibility-Squeak'!

unescapePercentsWithTextEncoding: encodingName
    "decode string including %XX form"
    | unescaped char asciiVal specialChars oldPos pos converter |
    unescaped := ReadWriteStream on: String new.
    specialChars := '+%' asSet.
    oldPos := 1.
    [pos := self indexOfAnyOf: specialChars startingAt: oldPos.
    pos > 0]
	whileTrue: [unescaped
		nextPutAll: (self copyFrom: oldPos to: pos - 1).
	    char := self at: pos.
	    (char = $%
		    and: [pos + 2 <= self size])
		ifTrue: [asciiVal := (self at: pos + 1) asUppercase digitValue * 16 + (self at: pos + 2) asUppercase digitValue.
		    asciiVal > 255
			ifTrue: [^ self].
		    unescaped
			nextPut: (Character value: asciiVal).
		    pos := pos + 3.
		    pos <= self size
			ifFalse: [char := nil].
		    oldPos := pos]
		ifFalse: [char = $+
			ifTrue: [unescaped nextPut: Character space]
			ifFalse: [unescaped nextPut: char].
		    oldPos := pos + 1]].
    oldPos <= self size
	ifTrue: [unescaped
		nextPutAll: (self copyFrom: oldPos to: self size)].
    converter := (CharacterEncoder encoderFor: encodingName)
		ifNil: [CharacterEncoder encoderForUTF8].
    ^ [converter decode: unescaped contents]
	on: Error
	do: ["the contents may be squeak-encoded"
	    unescaped contents]

    "Created: / 14-09-2010 / 19:23:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Class methodsFor:'Compatibility-Squeak'!

bindingOf:aName
    "return the binding of a name - either a class var, a private class, or a pool value"

    (self classVarNames includes:aName) ifTrue:[
        ^ self classVarAt:aName
    ].

    self sharedPoolNames do:[:eachPoolName |
        |pool|

        pool := Smalltalk at:eachPoolName.
        (pool classVarNames includes:aName) ifTrue:[
            ^ pool classVarAt:aName
        ].
    ].

    "JV@2011-10-23: In Squeak/Pharo, bindingOf: returns
     nil if no binding is found. Some code depends on this
     behavior"
    self breakPoint:#cg. "/self error:'No such binding'.
    ^nil

    "Created: / 12-09-2011 / 10:09:58 / cg"
    "Modified (format): / 23-10-2011 / 17:05:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Class methodsFor:'Compatibility-Squeak'!

objectForDataStream: stream

    self breakPoint: #jv.

    ^super objectForDataStream: stream

    "Created: / 11-09-2010 / 19:24:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Class methodsFor:'Compatibility-Squeak'!

storeDataOn: stream

    self error: 'Should never happen'

    "Created: / 11-09-2010 / 19:28:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ClassDescription methodsFor:'Compatibility-Squeak'!

compile:code classified:cat withStamp: stamp

    | method |

    method := self compilerClass
		    compile:code
		    forClass:self
		    inCategory:cat.
    "Make it working even if Monticello is not loaded"
    (Annotation respondsTo:#mctimestamp:) ifTrue:
	[method annotateWith: (Annotation mctimestamp: stamp)].
    ^method

    "Created: / 11-09-2010 / 18:35:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 14-09-2010 / 15:40:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ClassDescription methodsFor:'Compatibility-Squeak'!

compileSilently: source classified: category

    Class withoutUpdatingChangesDo:[self compile: source classified: category]

    "Created: / 11-09-2010 / 19:02:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ClassOrganizer methodsFor:'Compatibility-Squeak'!

commentStamp

    ^(class respondsTo: #version_MC)
	ifTrue: [class version_MC]
	ifFalse:[Author changeStamp]

    "Created: / 26-08-2010 / 18:36:27 / Jan Vrany <enter your email here>"
    "Modified: / 14-09-2010 / 15:48:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Collection methodsFor:'Compatibility-Squeak'!

do: block displayingProgress: progress

    self size == 0 ifTrue:[^self].


"/    ProgressIndicator
"/        displayProgress: progress
"/        at:(Screen default center)
"/        from:1 to: self size
"/        during:
"/            [:val |
	    self withIndexDo:[:each :index|
		block value: each.
"/                val value:index.
	    ]
"/    ].

    "
	#(0 1 2 3 4 5 6 6 7 8 9)
	    do:[:i|Delay waitForMilliseconds: 50]
	    displayingProgress: 'Progress'

    "

    "Created: / 12-09-2010 / 09:27:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-08-2011 / 15:19:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Collection methodsFor:'Compatibility-Squeak'!

flatCollectAsSet: aBlock
    "Evaluate aBlock for each of the receiver's elements and answer the
    list of all resulting values flatten one level. Assumes that aBlock returns some kind
    of collection for each element. Equivalent to the lisp's mapcan"

    "original written by a. Kuhn and released under MIT"

    | set |
    self isEmpty ifTrue: [^self copy ].
    set := Set new.
    self do: [ :each |
	set addAll: (aBlock value: each) ].
    ^set

    "Created: / 26-11-2010 / 12:58:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Collection methodsFor:'Compatibility-Squeak'!

ifEmpty:aBlock

    ^self isEmpty ifTrue:[aBlock value] ifFalse:[self]

    "Created: / 27-10-2010 / 00:07:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!FileDirectory methodsFor:'Compatibility-Squeak'!

fileNamesMatching:pattern


    "
	Same as directoryContants, but returns only files
	that matches given patterns. This uses String>>matches:
	for pattern matching
    "
    | names  |

    names := self asFilename directoryContents.
    names ifNil:[^nil].
    ^names select: [:e|e matches: pattern].

    "Created: / 12-09-2010 / 18:41:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GenericException methodsFor:'Compatibility-Squeak'!

signal: aString

    ^self signalWith: aString

    "Created: / 13-09-2010 / 12:03:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Metaclass methodsFor:'Compatibility-Squeak'!

isTrait

    ^false

    "Created: / 12-09-2010 / 16:40:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Method methodsFor:'Compatibility-Squeak'!

numTemps

    ^self numVars

    "Created: / 14-10-2010 / 16:04:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Method methodsFor:'accessing'!

revisionInfo

    ^self containingClass theNonMetaclass revisionInfo

    "Created: / 28-08-2010 / 22:43:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-09-2010 / 16:38:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Method methodsFor:'Compatibility-Squeak'!

timeStamp

    ^(self hasAnnotation: #mctimestamp:)
	ifTrue:[(self annotationAt: #mctimestamp:) timestamp]
	ifFalse:[Author changeStamp].

    "Created: / 28-08-2010 / 22:43:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 14-09-2010 / 16:13:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Object methodsFor:'Compatibility-Squeak'!

assert:aBlock description:aString

    ^self assert: aBlock message: aString

    "Created: / 27-10-2010 / 00:09:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Object methodsFor:'Compatibility-Squeak'!

caseError
    "Report an error from an in-line or explicit case statement."

    self error: 'Case not found, and no otherwise clause'
! !

!Object methodsFor:'Compatibility-Squeak'!

caseOf: cases
    "The elements of cases are associations between blocks.
     Answer the evaluated value of the first association in cases
     whose evaluated key equals the receiver.  If no match is found, report an error."

    ^ self
	caseOf: cases
	otherwise: [self caseError]

    "| z | z _ {[#a]->[1+1]. ['b' asSymbol]->[2+2]. [#c]->[3+3]}. #b caseOf: z"
    "| z | z _ {[#a]->[1+1]. ['d' asSymbol]->[2+2]. [#c]->[3+3]}. #b caseOf: z"
    "The following are compiled in-line:"
    "#b caseOf: {[#a]->[1+1]. ['b' asSymbol]->[2+2]. [#c]->[3+3]}"
    "#b caseOf: {[#a]->[1+1]. ['d' asSymbol]->[2+2]. [#c]->[3+3]}"

    "Created: / 12-09-2011 / 09:17:59 / cg"
! !

!Object methodsFor:'Compatibility-Squeak'!

caseOf: cases otherwise: otherwise
    "The elements of cases are associations between blocks.
     Answer the evaluated value of the first association in cases
     whose evaluated key equals the receiver.  If no match is found, answer the result
     of evaluating the otherwise block."

    cases
	associationsDo: [:eachCase |
	    eachCase key value = self
		ifTrue: [^ eachCase value value].
	    nil].
    ^ otherwise value

    "| z | z _ {[#a]->[1+1]. ['b' asSymbol]->[2+2]. [#c]->[3+3]}. #b caseOf: z otherwise: [0]"
    "| z | z _ {[#a]->[1+1]. ['d' asSymbol]->[2+2]. [#c]->[3+3]}. #b caseOf: z otherwise: [0]"
    "The following are compiled in-line:"
    "#b caseOf: {[#a]->[1+1]. ['b' asSymbol]->[2+2]. [#c]->[3+3]} otherwise: [0]"
    "#b caseOf: {[#a]->[1+1]. ['d' asSymbol]->[2+2]. [#c]->[3+3]} otherwise: [0]"

    "Created: / 12-09-2011 / 09:18:52 / cg"
! !

!Object methodsFor:'Compatibility-Squeak'!

comeFullyUpOnReload: smartRefStream
    "Normally this read-in object is exactly what we want to store. 7/26/96 tk"

    ^ self

    "Created: / 12-09-2010 / 13:08:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Object methodsFor:'Compatibility-Squeak'!

objectForDataStream: stream

    ^self

    "Created: / 11-09-2010 / 19:21:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Object methodsFor:'message sending'!

perform: selector withEnoughArguments: anArray
    "Send the selector, aSymbol, to the receiver with arguments in argArray.
    Only use enough arguments for the arity of the selector; supply nils for missing ones.

    Note: This method is from Pharo 1.4 - do not use in Smalltalk/X code"
    | numArgs args |
    numArgs := selector numArgs.
    anArray size == numArgs
        ifTrue: [ ^self perform: selector withArguments: anArray asArray ].

    args := Array new: numArgs.
    args replaceFrom: 1
        to: (anArray size min: args size)
        with: anArray
        startingAt: 1.

    ^ self perform: selector withArguments: args

    "Created: / 23-10-2011 / 17:17:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Object methodsFor:'Compatibility-Squeak'!

readDataFrom: aDataStream size: varsOnDisk
    "Fill in the fields of self based on the contents of aDataStream.  Return self.
     Read in the instance-variables written by Object>>storeDataOn:.
     NOTE: This method must send beginReference: before reading any objects from aDataStream that might reference it.
     Allow aDataStream to have fewer inst vars.  See SmartRefStream."

    | cntInstVars cntIndexedVars |

    cntInstVars := self class instSize.
    self class isVariable
        ifTrue: [cntIndexedVars := varsOnDisk - cntInstVars.
                cntIndexedVars < 0 ifTrue: [
                    self error: 'Class has changed too much.  Define a convertxxx method']]
        ifFalse: [cntIndexedVars := 0.
                cntInstVars := varsOnDisk].     "OK if fewer than now"

    aDataStream beginReference: self.
    1 to: cntInstVars do:
        [:i | self instVarAt: i put: aDataStream next].
    1 to: cntIndexedVars do:
        [:i | self basicAt: i put: aDataStream next].
    "Total number read MUST be equal to varsOnDisk!!"
    ^ self  "If we ever return something other than self, fix calls
            on (super readDataFrom: aDataStream size: anInteger)"

    "Created: / 12-09-2010 / 13:10:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 25-11-2011 / 14:08:52 / cg"
! !

!Object methodsFor:'Compatibility-Squeak'!

storeDataOn: aDataStream
    "Store myself on a DataStream.  
     Answer self.  
     This is a low-level DataStream/ReferenceStream method. 
     See also objectToStoreOnDataStream.  
     NOTE: This method must send 'aDataStream beginInstance:size:' and then (nextPut:/nextPutWeak:) its subobjects.  
     readDataFrom:size: reads back what we write here."

    | cntInstVars cntIndexedVars |

    cntInstVars := self class instSize.
    cntIndexedVars := self basicSize.
    aDataStream
        beginInstance: self class
        size: cntInstVars + cntIndexedVars.
    1 to: cntInstVars do:
        [:i | aDataStream nextPut: (self instVarAt: i)].

    "Write fields of a variable length object.  When writing to a dummy
        stream, don't bother to write the bytes"
    "1 to: cntInstVars do:
        [:i | aDataStream nextPut: (self instVarAt: i)]."

    "Created: / 11-09-2010 / 19:27:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-09-2010 / 13:14:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 25-11-2011 / 14:08:44 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextInt32
    <resource: #obsolete>
    "Read a 32-bit signed integer from the next 4 bytes, most significant byte first"

    "/ CG: PLEASE do not use this in new code;
    self obsoleteMethodWarning:'use nextLongMSB:'.

    ^ self nextLongMSB: true

    "Original Squeak impl"
    "| s |
    s := 0.
    1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
    (s bitAnd: 16r80000000) = 0
        ifTrue: [^ s]
        ifFalse: [^ -1 - s bitInvert32]
    "

    "Modified: / 14-09-2010 / 13:22:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-09-2011 / 15:22:10 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextInt32Put: i32
    <resource: #obsolete>
    "Write a signed integer to the next 4 bytes, most significant byte first"

    "/ CG: PLEASE do not use this in new code;
    self obsoleteMethodWarning:'use nextLongMSB:'.

    self nextPutLong:i32 MSB:true.
"/    | pos |
"/
"/    pos := i32 < 0
"/            ifTrue: [(0-i32) bitInvert32 + 1]
"/            ifFalse: [i32].
"/    1 to: 4 do: [:i | self nextPut: (pos digitAt: 5-i)].
"/    ^ i32

    "Modified: / 16-09-2011 / 15:23:08 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextLittleEndianNumber: n
    "Answer the next n bytes as a positive Integer or LargePositiveInteger, 
     where the bytes are ordered from least significant to most significant."

    | bytes s |

    bytes := self next: n.
    s := 0.
    n to: 1 by: -1 do: [:i | |b t| 
                              t := (s bitShift: 8).
                              b := bytes at: i.
                              s := t bitOr: b
                       ].
    ^ s

    "
     self assert:( [|s| s := #[1 2 3 4] readStream nextLittleEndianNumber:4] value = 16r04030201)
     self assert:( [|s| s := #[1 2 3 4 5 6 7 8] readStream nextLittleEndianNumber:8] value = 16r0807060504030201) 
    "

    "Modified: / 17-09-2011 / 09:07:03 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextLittleEndianNumber: n put: value
    "write a positive Integer or LargePositiveInteger as the next n bytes ordered from least significant to most significant."

    | bytes |

    bytes := ByteArray new: n.
    1 to: n do: [:i | bytes at: i put: (value digitAt: i)].
    self nextPutAll: bytes

    "Modified (format): / 16-09-2011 / 12:09:33 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextPutSqueakString: s
    "Append the string, s, to the receiver.  
     Only used by DataStream.  
     Max size of 64*256*256*256."

    | length |

    (length := s size) < 192
            ifTrue: [self nextPut: length]
            ifFalse:
                    [self nextPut: (length digitAt: 4)+192.
                    self nextPut: (length digitAt: 3).
                    self nextPut: (length digitAt: 2).
                    self nextPut: (length digitAt: 1)].
    self nextPutAll: s asByteArray.
    ^s

    "Modified: / 16-09-2011 / 15:20:30 / cg"
    "Created: / 08-02-2012 / 14:25:48 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextSqueakString
    "Read a string from the receiver. 
     The first byte is the length of the string, unless it is greater than 192, 
     in which case the first four bytes encode the length.  
     I expect to be in ascii mode when called (caller puts back to binary).
     Max size 1G."

    | length aByteArray |

    "read the length in binary mode"
    self binary.
    length := self next.            "first byte."
    length >= 192 ifTrue: [
        length := length - 192.
        1 to: 3 do: [:ii | length := length * 256 + self next]
    ].
    aByteArray := ByteArray new: length.

    self nextInto: aByteArray.
    ^aByteArray asString.

    "Created: / 26-01-2012 / 17:38:02 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextSqueakStringOld
    "Read a string from the receiver. The first byte is the length of the
     string, unless it is greater than 192, in which case the first *two* bytes
     encode the length.  
     Max size 16K. "

    | aString length |

    length := self next.            "first byte."
    length >= 192 ifTrue: [length := (length - 192) * 256 + self next].
    aString := String new: length.
    1 to: length do: [:ii | aString at: ii put: self next asCharacter].
    ^aString

    "Modified: / 16-09-2011 / 15:20:07 / cg"
    "Created: / 08-02-2012 / 14:27:45 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextString
    <resource: #obsolete>

    "/ CG: PLEASE do not use this in new code;
    "/ it is highly specific to a particular encoding style used by a particular Squeak 
    "/ application. It is not general enough to justify polluting the Stream selector namespace
    "/ with a generic name like nextString - after all, I would expect a 0-terminated string reader under that name
    "/ so please move that algorithm as utility to whoever calls this, and be prepared for this method to vanish
    self obsoleteMethodWarning.
    
    "Read a string from the receiver. 
     The first byte is the length of the string, unless it is greater than 192, 
     in which case the first four bytes encode the length.  
     I expect to be in ascii mode when called (caller puts back to binary)."
    ^ self nextSqueakString

    "Modified (comment): / 26-01-2012 / 17:38:47 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextStringOld
    <resource: #obsolete>
    "/ CG: PLEASE do not use this in new code;
    "/ it is highly specific to a particular encoding style used by a particular Squeak 
    "/ application. It is not general enough to justify polluting the Stream selector namespace
    "/ with a generic name like nextString - after all, I would expect a 0-terminated string reader under that name
    "/ so please move that algorithm as utility to whoever calls this, and be prepared for this method to vanish
    self obsoleteMethodWarning.
    ^ self nextSqueakStringOld

    "Modified: / 08-02-2012 / 14:28:00 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextStringPut: s
    "/ CG: PLEASE do not use this in new code;
    "/ it is highly specific to a particular encoding style used by a particular Squeak 
    "/ application. It is not general enough to justify polluting the Stream selector namespace
    "/ with a generic name like nextStringPut: - after all, I would expect a 0-terminated string reader under that name
    "/ so please move that algorithm as utility to whoever calls this, and be prepared for this method to vanish

    <resource: #obsolete>

    self obsoleteMethodWarning.
    ^ self nextPutSqueakString: s

    "Modified: / 08-02-2012 / 14:26:25 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextWord
    "Answer the next two bytes from the receiver as an Integer, most significant byte first."

    | high low |

    "/ cg: what an ugly rubbish interface: returning false here instead of nil !!!!!!

    (high := self next) == nil ifTrue: [^false].
    (low := self next) == nil ifTrue: [^false].
    ^(high asInteger bitShift: 8) + low asInteger

    "Modified: / 16-09-2011 / 12:12:27 / cg"
    "Modified (format): / 16-09-2011 / 15:21:10 / cg"
! !

!PositionableStream methodsFor:'Compatibility-Squeak'!

nextWordPut: aWord
    "Append to the receiver an Integer as the next two bytes, most significant byte first."

    self nextPut: ((aWord bitShift: -8) bitAnd: 255).
    self nextPut: (aWord bitAnd: 255).
    ^aWord

    "Modified (comment): / 16-09-2011 / 12:11:57 / cg"
! !

!ReadStream methodsFor:'Compatibility-Squeak'!

next: n into: aCollection startingAt: startIndex
    "Read n objects into the given collection.
    Return aCollection or a partial copy if less than
    n elements have been read."
    | max |
    max := (readLimit - position) min: n.
    aCollection
	replaceFrom: startIndex
	to: startIndex+max-1
	with: collection
	startingAt: position+1.
    position := position + max.
    max = n
	ifTrue:[^aCollection]
	ifFalse:[^aCollection copyFrom: 1 to: startIndex+max-1]

    "Modified: / 12-09-2010 / 13:06:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ReadStream methodsFor:'Compatibility-Squeak'!

nextInto:aCollection

    "Read the next elements of the receiver into aCollection.
    Return aCollection or a partial copy if less than aCollection
    size elements have been read."
    ^self next: aCollection size into: aCollection startingAt: 1.

    "Created: / 12-09-2010 / 13:05:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SequenceableCollection methodsFor:'Compatibility-Squeak'!

asTraitComposition
    "For convenience the composition {T1. T2 ...} is the same as T1 + T2 + ..."
    "
    ^self isEmpty
	ifFalse: [
	    self size = 1
		ifTrue: [self first asTraitComposition]
		ifFalse: [
		    self copyWithoutFirst
			inject: self first
			into: [:left :right | left + right]]]
	ifTrue: [
	    TraitComposition new]
    "

    "Now, Smalltalk/X doesn't support traits"
    ^nil

    "Created: / 29-08-2010 / 08:31:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Smalltalk class methodsFor:'Compatibility-Squeak'!

allTraits

    ^#()

    "Created: / 14-09-2010 / 09:44:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Smalltalk class methodsFor:'Compatibility-Squeak'!

globals

    ^self

    "Created: / 12-09-2010 / 13:09:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Smalltalk class methodsFor:'Compatibility-Squeak'!

logChange:aString

    "Nothing to do"

    "Created: / 14-09-2010 / 09:45:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ZipArchive methodsFor:'Compatibility-Squeak'!

contentsOf: fileName

    ^(self extract: fileName) asString

    "Created: / 11-09-2010 / 19:38:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!stx_libcompat class methodsFor:'documentation'!

extensionsVersion_CVS
    ^ '$Header: /cvs/stx/stx/libcompat/extensions.st,v 1.20 2012/02/08 13:39:02 cg Exp $'
! !