"
 COPYRIGHT (c) 2011 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libjavascript' }"

CodeGeneratorTool subclass:#JavaScriptCodeGeneratorTool
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Browsers'
!

!JavaScriptCodeGeneratorTool class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2011 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Code generation facilites for JavaScript

    [author:]
        Claus Gittiner
"
! !

!JavaScriptCodeGeneratorTool methodsFor:'code generation-basic'!

createAccessMethodsFor:aCollectionOfVarNames in:aClass withChange:withChange asValueHolder:asValueHolder readersOnly:readersOnly writersOnly:writersOnly lazyInitialization:lazyInitialization
    "workhorse for creating access methods for instvars."

    |classesClassVars generateCommentsForSetters generateCommentsForGetters|

    self startCollectChanges.

    generateCommentsForSetters := userPreferences generateCommentsForSetters.
    generateCommentsForGetters := userPreferences generateCommentsForGetters.

    classesClassVars := aClass theNonMetaclass allClassVarNames.

    aCollectionOfVarNames do:[:name |
        |source varType methodName defaultMethodName argName|

        varType := (classesClassVars includes:name) 
                        ifTrue:['static'] 
                        ifFalse:[
                            (aClass isMeta ifTrue:['classInstVar'] ifFalse:['instance'])].

        methodName := name.
        name isUppercaseFirst ifTrue:[
            methodName := methodName asLowercaseFirst. 
        ].
        argName := 'something'.

        "/ the GETTER
        writersOnly ifFalse:[
            lazyInitialization ifTrue:[
                defaultMethodName := 'default' , name asUppercaseFirst.
            ].

            "check, if method is not already present"
            (aClass includesSelector:(methodName asSymbol)) ifFalse:[
                asValueHolder ifTrue:[
                    source := methodName , '() {\'.
                    generateComments ifTrue:[
                        source := source , '    // return/create the ''%2'' value holder (automatically generated)\\'. 
                    ].
                    source := source , '    if (%2 == null) {\'.
                    lazyInitialization ifTrue:[
                        source := source
                                   , '        %2 = this.class.%3.asValue;\'.
                    ] ifFalse:[
                        source := source
                                   , '        %2 = new ValueHolder;\'.
                    ].

                    withChange ifTrue:[
                    source := source
                               , '        %2.addDependent(this);\'.
                    ].
                    source := source
                               , '    }\'
                               , '    return %2;\'
                               , '}'.
                ] ifFalse:[
                    source := methodName , '() {\'.
                    lazyInitialization ifTrue:[
                        generateCommentsForGetters ifTrue:[
                            source := source , '    // return the %1 instance variable ''%2'' with lazy instance creation (automatically generated)\\'. 
                        ].
                        source := source
                                    , '    if (%2 == null) {\'
                                    , '        %2 = self.class.%3;\'
                                    , '    }\'
                                    , '    return %2;\'
                                    , '}'.
                    ] ifFalse:[
                        generateCommentsForGetters ifTrue:[
                            source := source , '    // return the %1 instance variable ''%2'' (automatically generated)\\'. 
                        ].
                        source := source
                                    , '    return %2;\'
                                    , '}'.
                    ].
                ].
                source := (source bindWith:varType with:name with:defaultMethodName) withCRs.
                self compile:source forClass:aClass inCategory:(asValueHolder ifTrue:['aspects'] ifFalse:['accessing']).
            ] ifTrue:[
                Transcript showCR:'method ''', methodName , ''' already present'
            ].

            "/ default for lazy on class side
            lazyInitialization ifTrue:[
                (aClass theMetaclass includesSelector:(defaultMethodName asSymbol)) ifFalse:[
                    source := defaultMethodName , '() {\'.
                    generateComments ifTrue:[
                        source := source , '    // default value for the ''%2'' instance variable (automatically generated)\\'. 
                    ].
                    source := source    
                               , '    this.shouldImplement();\'
                               , '    return null;\';
                               , '}'.

                    source := (source bindWith:varType with:name) withCRs.
                    self compile:source forClass:aClass theMetaclass inCategory:'defaults'.
                ].
            ].                                       
        ].

        "/ the SETTER
        readersOnly ifFalse:[
            (aClass includesSelector:((methodName , ':') asSymbol)) ifFalse:[
                ((methodName size > 2) and:[ (methodName startsWith:'is') and:[ (methodName at:3) isUppercase ]])
                ifTrue:[
                    argName := 'aBoolean'
                ].
                asValueHolder ifTrue:[
                    source := methodName , '(%3) {\'.  "/ argName
                    generateComments ifTrue:[
                        source := source , '    // set the ''%2'' value holder' , ' (automatically generated)\\'.
                    ].
                    withChange ifTrue:[
                        source := source
                                  , '    var oldValue, newValue;\\'
                                  , '    if (%2 !!= null) {\'
                                  , '        oldValue = %2.value;\'
                                  , '        %2.removeDependent(this);\'
                                  , '    }\'
                                  , '    %2 = %3;\'        "/ argName
                                  , '    if (%2 !!= null) {\'
                                  , '        %2.addDependent(self);\'
                                  , '    }\'
                                  , '    newValue = %2.value;\'
                                  , '    if (oldValue !!== newValue) {\'
                                  , '        this.update_with_from("value".asSymbol(), newValue, %2);\'
                                  , '    }\'
                                  , '}'.
                    ] ifFalse:[
                        source := source 
                                  , '    %2 = %3;\'  "/ argName
                                  , '}'.
                    ].
                ] ifFalse:[
                    source := methodName , '(%3) {\'.    "/ argName
                    withChange ifTrue:[
                        generateComments ifTrue:[
                            source := source , '    // set the value of the %1 variable ''%2''\'.
                            source := source , '    // and send a change notification (automatically generated)\\'.
                        ].
                        source := source
                                  , '    if (%2 !!== %3) {\'
                                  , '        %2 = %3;\'           "/ argName
                                  , '        this.changed("%2".asSymbol);\'
                                  , '     };\'
                                  , '}'.
                    ] ifFalse:[
                        generateCommentsForSetters ifTrue:[
                            source := source , '    // set the value of the %1 variable ''%2''\'.
                            source := source , '    // (automatically generated)"\\'.
                        ].
                        source := source
                                  , '    %2 = %3;\'          "/ argName
                                 , '}'.
                    ].
                ].
                source := (source bindWith:varType with:name with:argName) withCRs.
                self 
                    compile:source 
                    forClass:aClass 
                    inCategory:(asValueHolder ifTrue:['aspects'] ifFalse:['accessing']).
            ] ifTrue:[
                Transcript showCR:'method ''', methodName , ':'' already present'
            ].
        ].
    ].

    self executeCollectedChangesNamed:('Add Accessors').
! !

!JavaScriptCodeGeneratorTool class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/libjavascript/JavaScriptCodeGeneratorTool.st,v 1.1 2011/01/28 09:23:51 cg Exp $'
! !
