"
 COPYRIGHT (c) 1998 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libjavascript' }"

ParseNode subclass:#JavaScriptFunctionNode
	instanceVariableNames:'functionName arguments locals statements environment
		isReturnedFrom'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-JavaScript-Compiling & Parsing'
!

!JavaScriptFunctionNode class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1998 by eXept Software AG
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    ATTENTION: for protocol compatibility with JavaScriptEnvironment.
    many messages are implemented with an initial underscore.
    These are required as JavaScriptEnvironment catches doesNotUnderstand and
    does some kludgy fiddling there.
"
! !

!JavaScriptFunctionNode methodsFor:'accessing'!

_argVariables
    ^ arguments
!

_argVariables:aDictionary
    arguments := aDictionary
!

_localVariables
    ^ locals
!

_localVariables:aDictionary
    locals := aDictionary
!

_outerEnvironment
    ^ environment

    "Created: / 17.5.1998 / 00:41:00 / cg"
!

_outerEnvironment:anEnvironment
    environment := anEnvironment

    "Created: / 17.5.1998 / 00:41:11 / cg"
!

_topEnvironment
    ^ environment isNil ifTrue:self ifFalse:[environment _topEnvironment]
!

addLocalVariable:aVariable
    locals isNil ifTrue:[
	locals := OrderedCollection new.
    ].
    locals add:aVariable

    "Created: / 14.5.1998 / 21:56:19 / cg"
!

arguments
    "return the value of the instance variable 'arguments' (automatically generated)"

    ^ arguments

    "Created: / 14.5.1998 / 21:26:20 / cg"
!

arguments:something
    "set the value of the instance variable 'arguments' (automatically generated)"

    arguments := something.

    "Created: / 14.5.1998 / 21:26:20 / cg"
!

functionName
    "return the value of the instance variable 'functionName' (automatically generated)"

    ^ functionName

    "Created: / 14.5.1998 / 21:26:18 / cg"
!

functionName:something
    "set the value of the instance variable 'functionName' (automatically generated)"

    functionName := something.

    "Created: / 14.5.1998 / 21:26:18 / cg"
!

isReturnedFrom
    ^ isReturnedFrom
!

isReturnedFrom:something
    isReturnedFrom := something.
!

localVariableNames
    locals isNil ifTrue:[^ #() ].
    ^ (locals asSortedCollection:[:a :b | a index < b index]) collect:[:v | v name].
!

localVariables
    ^ locals ? #()
!

numArgs
    ^ arguments size

    "Created: / 17.5.1998 / 00:36:04 / cg"
!

statements
    "return the value of the instance variable 'statements' (automatically generated)"

    ^ statements

    "Created: / 14.5.1998 / 21:26:22 / cg"
!

statements:something
    "set the value of the instance variable 'statements' (automatically generated)"

    statements := something.

    "Created: / 14.5.1998 / 21:26:22 / cg"
!

xxxx_variables
    |list|

    list := IdentityDictionary new.
    arguments notNil ifTrue:[
	arguments do:[:arg |
	    list at:arg name put:arg
	].
    ].
    locals notNil ifTrue:[
	locals do:[:var |
	    list at:var name put:var
	].
    ].
    ^ list

    "Created: / 17.5.1998 / 00:38:15 / cg"
    "Modified: / 17.5.1998 / 00:40:18 / cg"
! !

!JavaScriptFunctionNode methodsFor:'code generation'!

codeVariableSetupOn:codeStream for:aCompiler
    locals isNil ifTrue:[^ self].

    locals do:[:eachVar |
        "/ generate code to set it up.

        |varNode initExpression setupExpr|

        initExpression := eachVar expressionForSetup.
        initExpression notNil ifTrue:[
            varNode := JavaScriptVariableNode 
                            type:#MethodVariable
                            name:eachVar name
                            token:eachVar
                            index:eachVar index.

            setupExpr := AssignmentNode variable:varNode expression:initExpression.
            setupExpr codeForSideEffectOn:codeStream inBlock:nil for:aCompiler.
        ]
    ]

    "Modified: / 12-07-2006 / 15:07:00 / cg"
! !

!JavaScriptFunctionNode methodsFor:'enumeration'!

messagesDo:aBlock
    statements messagesDo:aBlock

    "Created: / 24-07-2010 / 21:42:25 / cg"
! !

!JavaScriptFunctionNode methodsFor:'evaluation'!

valueWithReceiver:rec arguments:args
    |oldValues val|

    (args size ~~ arguments size) ifTrue:[
        ^ self wrongNumberOfArguments:args size
    ].
    statements isNil ifTrue:[^ nil].

    oldValues := Array new:(arguments size).
    1 to:arguments size do:[:i |
        oldValues at:i put:(arguments at:i) value.
        (arguments at:i) value:(args at:i).
    ].
"/    exitBlock := [:v | 
"/        1 to:argArray size do:[:i |
"/           ( blockArgs at:i) value:(oldValues at:i)
"/        ].
"/        ^ v
"/    ].

    val := statements evaluateAllIn:(Context new receiver:rec).

    1 to:arguments size do:[:i |
        (arguments at:i) value:(oldValues at:i)
    ].
    ^ val

    "Modified: / 17.5.1998 / 00:53:45 / cg"
! !

!JavaScriptFunctionNode methodsFor:'printing'!

printOn:aStream indent:i
    |first stat|

    aStream nextPutAll:'function '.
    functionName notNil ifTrue:[ aStream nextPutAll:functionName ].
    aStream nextPutAll:'('.

    first := true.
    arguments notNil ifTrue:[
        arguments do:[:anArgVar |
            first ifFalse:[aStream nextPutAll:', ' ].
            first := false.

            aStream nextPutAll:anArgVar name.
        ].
    ].
    aStream nextPutAll:')'.
    aStream cr.
    aStream nextPutAll:'{'.
    locals size > 0 ifTrue:[
        locals do:[:v |
            aStream cr; spaces:i+4.
            aStream nextPutAll:'var '; nextPutAll:v name; nextPutAll:';'" ; cr"
        ].
        aStream cr.
    ].
    aStream cr; spaces:i+4.
    (stat := statements) notNil ifTrue:[
        [stat notNil] whileTrue:[
            stat printOn:aStream indent:i+4.
            stat := stat nextStatement.
            stat notNil ifTrue:[aStream cr; spaces:i+4.]
        ].
    ].
    aStream cr; spaces:i; nextPutAll:'}'.

    "Created: / 14.5.1998 / 21:29:02 / cg"
    "Modified: / 15.5.1998 / 15:06:40 / cg"
! !

!JavaScriptFunctionNode methodsFor:'queries'!

_isFunctionEnvironment
    ^ true
!

_isHTMLEnvironment
    ^ false
!

isInnerFunction
    ^ false
! !

!JavaScriptFunctionNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libjavascript/JavaScriptFunctionNode.st,v 1.22 2010/07/24 19:44:01 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/libjavascript/JavaScriptFunctionNode.st,v 1.22 2010/07/24 19:44:01 cg Exp $'
! !
