"{ Package: 'stx:liboracle8' }"

ExternalBytes subclass:#OCI8CallBackData
	instanceVariableNames:'converter'
	classVariableNames:'Debugging UnfinishedCallBacks'
	poolDictionaries:''
	category:'DataBase-Oracle8-Utilities'
!

!OCI8CallBackData primitiveDefinitions!
%{

/*
 * includes, defines, structure definitions
 * and typedefs come here.
 */
#include <oci.h>
#include <stdio.h>
#include <malloc.h>

struct ociCallBackData {
    int iterator;
    int fetchedrows;
    int finished;
    int elementsize;
    dvoid *bufp;
    ub4 *alenp;
    sb2 *indp;
    OCIError *errorp;
};

%}
! !

!OCI8CallBackData primitiveFunctions!
%{
/* ----------------------------------------------------------------- */
/* IN bind callback that does not do any data input.                 */
/* ----------------------------------------------------------------- */
sword 
in_returning(dvoid *ictxp, OCIBind *bindp, ub4 iter, ub4 index,
             dvoid **bufpp, ub4 *alenp, ub1 *piecep, dvoid **indpp)
{
    static sb2 null_ind = -1;
    if (@global(OCI8CallBackData:Debugging) == true) {
        fprintf(stderr,"IN_DATA iter=%d index=%d\n", iter, index);
    }
    /* 
     * This is a dummy input callback function that provides input data
     * for the bind variables in the RETURNING clause.
     */
    *bufpp = 0;
    *alenp = 0;          
    *indpp = (dvoid *) &null_ind;
    *piecep = OCI_ONE_PIECE;
    return(OCI_CONTINUE);
}



/* ----------------------------------------------------------------- */
/* Outbind callback for returning data.                              */
/* ----------------------------------------------------------------- */
static sword 
out_returning(dvoid *octxp, OCIBind *bindp, ub4 iter, ub4 index,
              dvoid **bufpp, ub4 **alenpp, ub1 *piecep,
              dvoid **indpp, ub2 **rcodepp)
{
    /* 
     * This is the callback function that is called to receive the OUT
     * bind values for the bind variables in the RETURNING clause
     * iter     is the number of the iteration 
     *          (so having an array of 5 bind vars causes iter to increment from 0 .. 4)
     * index    the number of the row in the current iteration
     *          (update or delete may affect more than one row in an iteration,
     *           e.g. delete from table where age > 80 returning table.name into :name)
     */

    struct ociCallBackData *ctxp;
    int elementsize;
    int fetchedrows;

    if (@global(OCI8CallBackData:Debugging) == true){
        fprintf(stderr,"BEGIN OUT DATA iter=%d, index= %d ****************\n", iter, index);
    }
    ctxp = (struct ociCallBackData *) octxp;
    elementsize = ctxp->elementsize;
    fetchedrows = ctxp->fetchedrows;

    if (index == 0) {
        dvoid *new_data;
        int new_size;
        ub4 rows = 0;
        ub4 new_rows;
        ub4 attributeSize = sizeof(ub4);

        /* For each new iteration (so index == 0) the OCI_ATTR_ROWS_RETURNED tells us the number
         * of rows returned in that iteration.  So we can use this information
         * to dynamically allocate storage for all the returned rows for that
         * bind.
         */

        if (OCIAttrGet((CONST dvoid *)bindp, OCI_HTYPE_BIND, (dvoid *)&rows, 
                       &attributeSize, OCI_ATTR_ROWS_RETURNED, ctxp->errorp)) {
            if (@global(OCI8CallBackData:Debugging) == true){
                fprintf(stderr, "OCI8 [warning]: OCI_ATTR_ROWS_RETURNED failed\n");
            };
            ctxp->finished = 1;
            return(OCI_ERROR);
        }
        new_rows = rows ? rows : 1;   /* allocate at least one row */

        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"number of returned rows: %d, already fetched rows: %d\n", rows, fetchedrows);
        }
        /* Dynamically allocate storage */
        /* realloc bufp */
        new_size = elementsize * (new_rows+1 + fetchedrows);
        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"bufp before realloc: %x newsize: %d oldsize:%d\n",        
                    ctxp->bufp, new_size, elementsize * fetchedrows);
        }
        new_data = realloc(ctxp->bufp, new_size);
        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"bufp after realloc: %x\n", new_data);
        }
        if (new_data == 0) {
            if (@global(OCI8CallBackData:Debugging) == true){
                fprintf(stderr, "OCI8 [warning]: OCI8CallBackData unable to (re)allocate %d bytes \n", new_size);
            };
            ctxp->finished = 1;
            return(OCI_ERROR);
        }
        ctxp->bufp = new_data;

        /* realloc indp */
        new_size = sizeof(sb2) * (new_rows + fetchedrows);
        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"indp before realloc: %x newsize: %d oldsize:%d\n",        
                    ctxp->indp, new_size, sizeof(sb2) * fetchedrows);
        }
        new_data = realloc(ctxp->indp, new_size);

        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"indp after realloc: %x\n", new_data);
        }
        if (new_data == 0) {
            if (@global(OCI8CallBackData:Debugging) == true){
                fprintf(stderr, "OCI8 [warning]: OCI8CallBackData unable to allocate %d bytes \n", new_size);
            };
            ctxp->finished = 1;
            return OCI_ERROR;
        }
        ctxp->indp = new_data;

        /* realloc alenp */
        new_size = sizeof(ub4) * (new_rows + fetchedrows);
        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"alenp before realloc: %x newsize: %d oldsize:%d\n",        
                    ctxp->alenp, new_size, sizeof(ub4) * fetchedrows);
        }
        new_data = realloc(ctxp->alenp, new_size);
        if (@global(OCI8CallBackData:Debugging) == true){
            fprintf(stderr,"alenp after realloc: %x\n", new_data);
        }
        /* check for realloc */
        if (new_data == 0) {
            if (@global(OCI8CallBackData:Debugging) == true){
                fprintf(stderr, "OCI8 [warning]: OCI8CallBackData unable to allocate %d bytes \n", new_size);
            };
            ctxp->finished = 1;
            return(OCI_ERROR);
        }
        ctxp->alenp = new_data;
    }

    *bufpp   = ctxp->bufp + (fetchedrows * elementsize);
    *indpp   = ctxp->indp + fetchedrows;
    **(sb2 **)indpp = 0;
    *rcodepp = 0;
    *alenpp  = ctxp->alenp + fetchedrows;
    **alenpp = elementsize;
    *piecep  = OCI_ONE_PIECE;

    ctxp->iterator = iter;
    ctxp->fetchedrows++;

    /*
     * XXX called once, set to finished
     */
    ctxp->finished = 1;

    if (@global(OCI8CallBackData:Debugging) == true){
        fprintf(stderr," END OUT DATA bufp:%x indp:%x alenp:%x\n", *bufpp, *indpp, *alenpp);
    }
    return(OCI_CONTINUE);
}

%}
! !

!OCI8CallBackData class methodsFor:'documentation'!

documentation
"
    This utility class is used for a dynamic bind of an OCI8ReturnValueBindVariable.
    In case of a sql statement with a returining clause, the data for the returning
    variables are fetched via callbacks. Those callbacks allocate the memory for the
    returned values. To access the returned values the method OCI8CallBackData>>elements
    can be called. A converter is used to get the correct smalltalk type from the
    returned values.

    [author:]
	 Pierre Schwarz

    [see also:]
	 OCI8ReturnValueBindVariable

    [instance variables:]

	converter <OCI8Converter>
	    used to convert the bytes from an ExternalByte object to a smalltalk value.
	    The type into which the bytes are converted is determined by the concrete
	    class of a converter.

    [class variables:]

	Debugging <Boolean>
	    Swichtes on/off debug traces from C-level
		Debugging := true
		Debugging := false.

	UnfinishedCallBacks <OrderedCollection>
	    holds a collection of all instances of OCI8CallBakcData which
	    haven't at disposed time the finished flag set. Therefore their
	    memory space cannot be released because it could be possible that
	    the out-function of the dynamic bind will access the memory.
	    The memory of the instances in the collection must be freed manually.
"
! !

!OCI8CallBackData class methodsFor:'instance creation'!

elementSize:anElementSize converter:anOCI8Converter errorHandle:anErrorHandle

    |newInst|

    newInst := super unprotectedNew:(self structSize).
    newInst atAllPut:0.
    newInst elementSize:anElementSize.
    newInst converter:anOCI8Converter.
    newInst errorHandle:anErrorHandle.
    ^ newInst

    "
     self elementSize:20 converter:nil errorHandle:ExternalAddress new
    "
!

new

    self shouldNotImplement

    "
     self new
    "
! !

!OCI8CallBackData class methodsFor:'accessing'!

unfinishedCallBacks
"
returns a collection of all instances of the receiver class which
haven't at disposed time the finished flag set. Therefore their
memory space cannot be released because it could be possible that
the out-function of the dynamic bind will access the memory.
The memory of the instances in the collection must be freed manually.

<return: OrderedCollection>
"
    UnfinishedCallBacks isNil ifTrue:[
	UnfinishedCallBacks := OrderedCollection new.
    ].
    ^ UnfinishedCallBacks
! !

!OCI8CallBackData class methodsFor:'constants'!

structSize
"
returns the size of the c struct which is used the c callback functions.

<return: Integer>
"

%{ /* NOCONTEXT */
    RETURN (__mkSmallInteger(sizeof(struct ociCallBackData)));
%}


"
self structSize
"
! !

!OCI8CallBackData methodsFor:'accessing'!

alenp
"
retrieve the alenp vector

<return: ExternalBytes|nil>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ) cd != nil && cd->alenp) {
        RETURN(__MKEXTERNALBYTES_N(cd->alenp, cd->fetchedrows*sizeof(*cd->alenp)));
    };
%}.
    ^ nil
!

bufp
"
retrieve the bufp vector

<return: ExternalBytes|nil>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil && cd->bufp) {
        RETURN(__MKEXTERNALBYTES_N(cd->bufp, cd->fetchedrows*cd->elementsize));
    };
%}.
    ^ nil
!

callBackContext
"retrieve the external address of the receiver. The address of the
receiver doesnt change because the class of the receiver is a subclass of
ExternalBytes. Therefore it make sense to return the address of the receiver.

<return: ExternalAddress>
"

%{  /* NOCONTEXT */

    RETURN(__MKEXTERNALADDRESS(__INST(address_)));
%}.
!

converter
    "return the value of the instance variable 'converter' (automatically generated)"

    ^ converter
!

converter:something
    "set the value of the instance variable 'converter' (automatically generated)"

    converter := something.
!

dump

%{ 
    struct ociCallBackData *ctxp;

    ctxp = (struct ociCallBackData *)(__INST(address_));
    if (ctxp && (OBJ)ctxp != nil) {
        fprintf(stderr, "ctx: %x\n", ctxp);
        fprintf(stderr, "  iterator: %d\n", ctxp->iterator);
        fprintf(stderr, "  fetchedrows: %d\n", ctxp->fetchedrows);
        fprintf(stderr, "  finished: %d\n", ctxp->finished);
        fprintf(stderr, "  elementsize: %d\n", ctxp->elementsize);
        fprintf(stderr, "  bufp: %x\n", ctxp->bufp);
        fprintf(stderr, "  alenp: %x\n", ctxp->alenp);
        fprintf(stderr, "  indp: %x\n", ctxp->indp);
        fprintf(stderr, "  errorp: %x\n", ctxp->errorp);
        fflush(stderr);
    }
%}.
    ^ nil
!

elementSize:anElementSize
"
set the elementsize value in the struct

<return: self>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil && __isSmallInteger(anElementSize)) {
        cd->elementsize = (int) __smallIntegerVal(anElementSize);
        RETURN(self);
    };
%}.
    self primitiveFailed.
!

elementsize
"
retrieve the elementsize value

<return: Integer>
"


%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        RETURN(__mkSmallInteger(cd->elementsize));
    };
%}.
    ^ 0
!

errorHandle:anErrorHandle
"
set the error handle in the struct

<return: self>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        cd->errorp = (OCIError *) __externalAddressVal(anErrorHandle);
    };
%}.
!

errorp
"
retrieve the errorp vector

<return: ExternalAddress|nil>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil && cd->errorp) {
        RETURN(__MKEXTERNALADDRESS(cd->errorp));
    };
%}.
    ^ nil
!

fetchedrows
"
retrieve the fetchedrows value. It holds the number
of elements which are return via the oci dynamic callback out-function.

<return: Integer>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        RETURN(__mkSmallInteger(cd->fetchedrows));
    };
%}.
    ^ 0
!

finished
"
retrieve the finished indicator. This indicator is set in the
out-function of the oci dynamic bind and prevents that the receiver
could be garbage collected before the callback calls are finished.
If the receiver would be garbage collected before the callback calls
are finished, they would write data at the address of the receiver.

<return: Boolean>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd == 0 || (OBJ)cd == nil || cd->finished || 
        (cd->bufp == 0 && cd->indp == 0 && cd->alenp == 0)) {
        RETURN (true);
    } else {
        RETURN(false);
    };
%}.
!

inReturningFunction
"
retrieve the external address of the in returning function

<return: ExternalAddress>
"

%{  /* NOCONTEXT */
    RETURN(__MKEXTERNALADDRESS(in_returning));
%}.
!

indp
"
retrieve the indp vector

<return: ExternalBytes|nil>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil && cd->indp) {
        RETURN(__MKEXTERNALBYTES_N(cd->indp, cd->fetchedrows*sizeof(*cd->indp)));
    };
%}.
    ^ nil
!

outReturningFunction
"
retrieve the external address of the out returning function

<return: ExternalAddress>
"

%{  /* NOCONTEXT */
    RETURN(__MKEXTERNALADDRESS(out_returning));
%}.

    "
     self basicNew outReturningFunction
     self basicNew inReturningFunction
    "
!

setFinished
"
set the finished indicator to true. This method is only provided
for manually free the memory chunks of the receiver.

<return: self>
"

%{  /* NOCONTEXT */
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        cd->finished = 1;
    };
%}.
! !

!OCI8CallBackData methodsFor:'accessing values'!

elements
"
returns a collection of the values received from the callback functions.
The values are converted by the converter to the appropriate smalltalk type.

<return: Collection of:<nil|ByteArray|FixedPoint|Float|Integer|String>>
"
    |fetchedRows elementSize returnArray size
     bufferBytes indicatorBytes elementLengthBytes indicatorStep|

    ((fetchedRows := self fetchedrows) = 0
     or:[(elementSize := self elementsize) = 0
     or:[(bufferBytes := self bufp) isNil 
     or:[(elementLengthBytes := self alenp) isNil 
     or:[(indicatorBytes := self indp) isNil]]]]) ifTrue:[
        ^ #[]
    ].

    returnArray := Array new:fetchedRows.

    indicatorStep := OCI8Lib sb2Size.
    1 to:fetchedRows do:[:index | |value tmpBuffer|
        (indicatorBytes  shortAt:1+((index-1)*indicatorStep)) = -1 ifTrue:[
            value := nil.
        ] ifFalse:[
            size := elementLengthBytes unsignedShortAt:1+((index-1)*indicatorStep).
            tmpBuffer := bufferBytes from:1+((index-1)*elementSize) to:(index*elementSize).
            value := converter getDataFromBuffer:tmpBuffer size:size.
        ].
        returnArray at:index put:value.
    ].
    ^ returnArray
! !

!OCI8CallBackData methodsFor:'instance release'!

finalize
"
handle was finalized; free the associated memory chunks with it. This isnt
done when the finished flag isn't set because it could be possible that
the out-function of the dynamic bind will access the memory.
The memory of the instances in the collection must be freed manually.

<return: self>
"

    self finished ifFalse:[
        self class unfinishedCallBacks add:self.
        Transcript showCR:'**** WARNING: disposed unfinished OCI8CallBackData. Inspect Classvariable UnfinishedCallBacks in this class.'.
        ^ self
    ].
%{  
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        if (cd->bufp) {
            free(cd->bufp);
            cd->bufp = 0;
        }
        if (cd->alenp) {
            free(cd->alenp);
            cd->alenp = 0;
        }
        if (cd->indp) {
            free(cd->indp);
            cd->indp = 0;
        }
        __stx_free(cd);
    };
    __INST(address_) = __INST(size) = nil;
    if (cd && (@global(TraceMalloc) == true)) {
        printf("EXTBYTES: freed memory at: %08x\n", cd);
    }
%}
!

resetCallBackData
"
prepare the receiver for another execution of an statement.
Reset iterator, fetchrows, finished and free bufp, alenp, indp.
Leave the elementsize, errorp, inFunc, outFunc untouched.

<return: self>
"
%{ /* NOCONTEXT */ 
    struct ociCallBackData *cd;

    cd = (struct ociCallBackData *)(__INST(address_));
    if (cd && (OBJ)cd != nil) {
        if (cd->bufp) {
            free(cd->bufp);
            cd->bufp = 0;
        }
        if (cd->alenp) {
            free(cd->alenp);
            cd->alenp = 0;
        }
        if (cd->indp) {
            free(cd->indp);
            cd->indp = 0;
        }
        cd->iterator = 0;
        cd->fetchedrows = 0;
        cd->finished = 0;
    };
%}
! !

!OCI8CallBackData class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8CallBackData.st,v 1.31 2002/04/02 17:53:02 stefan Exp $'
! !
