"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8ColumnDescriptor
	instanceVariableNames:'name scale precision nullValue type size'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Utilities'
!

!OCI8ColumnDescriptor class methodsFor:'documentation'!

documentation
"
    Information on a table column is stored in this class.

    [author:]
         Pierre Schwarz

    [see also:]
         OCI8Lib>>ociStmtGetColumnDescriptor:errorOCIHandle:

    [instance variables:]
        name <String>        
            Name of the column

        scale <Integer>
            When describing a number column the scale of the number is stored here

        precision <Integer>
            When describing a number column the precision of the number is stored here

        nullValue <Boolean>
            If the column can have null values true is stored here, otherwise false.

        type <Symbol>
            Type of the column, possible values are:
                #'CHAR_TYPE'
                #'VARCHAR2_TYPE'
                #'ROWID_DESC_TYPE'
                #'LONG_TYPE'
                #'DATE_TYPE'
                #'NUMBER_TYPE'
                #'RAW_TYPE'
                #'LONG_RAW_TYPE'

        size <Integer>
            Size of the column on the database. For column of type long, long_raw, the size is 0.

    [class variables:]
        none
"
!

history
    "Created: / 30.5.2000 / 14:35:51 / ps"
! !

!OCI8ColumnDescriptor class methodsFor:'instance creation'!

newForParameterHandle:anOCI8ParameterHandle ociLib:anOCI8Lib errorHandle:anOCI8ErrorHandle
"
returns the column descriptors for the parameter handle anOCI8ParameterHandle

<return: OCI8ColumnDescriptor>
"

    |columnDescriptor type|

    columnDescriptor := self new.
    columnDescriptor name:(anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                  attributeType:#'OCI_ATTR_NAME'
                                 errorOCIHandle:anOCI8ErrorHandle).

    columnDescriptor type:(type := anOCI8Lib class symbolForOCIDataType:
                                      (anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                              attributeType:#'OCI_ATTR_DATA_TYPE'
                                             errorOCIHandle:anOCI8ErrorHandle)).

    columnDescriptor nullValue:((anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                        attributeType:#'OCI_ATTR_IS_NULL'
                                       errorOCIHandle:anOCI8ErrorHandle) = 1).

    columnDescriptor precision:(anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                       attributeType:#'OCI_ATTR_PRECISION'
                                      errorOCIHandle:anOCI8ErrorHandle).

    columnDescriptor scale:(anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                   attributeType:#'OCI_ATTR_SCALE'
                                  errorOCIHandle:anOCI8ErrorHandle).

    type == #'CHARACTER_LOB_TYPE' ifTrue:[
        columnDescriptor size:65535000.
    ] ifFalse:[
        columnDescriptor size:(anOCI8Lib ociAttrGet:anOCI8ParameterHandle
                                      attributeType:#'OCI_ATTR_DATA_SIZE'
                                     errorOCIHandle:anOCI8ErrorHandle).
    ].
    ^ columnDescriptor
!

newForStatement:anOCI8Statement
"
returns the column descriptors for the affected columns of the statement anOCI8Statement

<return: OrderedCollection of:OCI8ColumnDescriptor>
"

    ^(self computeParameterForStatement:anOCI8Statement) collect:[:eachHandle|
        self newForParameterHandle:eachHandle
                            ociLib:anOCI8Statement ociLib 
                       errorHandle:anOCI8Statement errorHandle.
     ].



! !

!OCI8ColumnDescriptor class methodsFor:'computing'!

computeParameterForStatement:anOCI8Statement
"

<return: <OrderedCollection of:OCIHandle>>
"
    |parameterColl parameterCount|

    parameterCount := anOCI8Statement ociLib ociAttrGet:anOCI8Statement handle
					  attributeType:#'OCI_ATTR_PARAM_COUNT'
					 errorOCIHandle:anOCI8Statement errorHandle.

    parameterColl := OrderedCollection new:parameterCount.

    1 to:parameterCount do:[:position |
	parameterColl add:(anOCI8Statement ociLib ociParamGet:anOCI8Statement handle
						     position:position
					       errorOCIHandle:anOCI8Statement errorHandle)
    ].
    ^parameterColl


! !

!OCI8ColumnDescriptor methodsFor:'accessing'!

name
"
returns the name of the table column

<return: String>
"
    ^ name
!

name:aString
"
sets the name of the table column

<return: self>
"

    name := aString.
!

nullValue
"
returns if the table column can be NULL or not.

<return: Boolean>
"

    ^ nullValue
!

nullValue:aBoolean
"
sets if the table column can be NULL or not.

<return: self>
"

    nullValue := aBoolean.
!

precision
"
returns the precision of the table column.

<return: Integer>
"

    ^ precision
!

precision:anInteger
"
sets the precision of the table column.

<return: self>
"

    precision := anInteger.
!

scale
"
returns the scale of the table column.

<return: Integer>
"

    ^ scale
!

scale:anInteger
"
sets the scale of the table column.

<return: self>
"

    scale := anInteger.
!

size
"
returns the size of the table column.

<return: Integer>
"

    ^ size
!

size:anInteger
"
sets the size of the table column.

<return: self>
"

    size := anInteger.
!

type
"
returns the type of the table column.

<return: Symbol>
"

    ^ type
!

type:aSymbol
"
sets the type of the table column

<return: self>
"

    type := aSymbol.
! !

!OCI8ColumnDescriptor methodsFor:'generating'!

newConverter
"
returns of the type of the column a converter which handles the
receivment of the data and the conversion to the correct smalltalk type.

<return: OCI8StringConverter|OCI8LongConverter|OCI8FloatConverter|
         OCI8IntegerConverter|OCI8FixedPointConverter|OCI8ByteArrayConverter>
"

    (#(#'CHAR_TYPE' #'VARCHAR2_TYPE' #'ROWID_DESC_TYPE') includes:type) ifTrue:[
        ^ OCI8StringConverter new.
    ].

    (#(#'CHARACTER_LOB_TYPE' #'LONG_TYPE' ) includes:type) ifTrue:[
        ^ OCI8LongConverter new.
    ].

    (#'DATE_TYPE' == type) ifTrue:[
        ^ OCI8DateConverter new.
    ].

    (#'NUMBER_TYPE' == type) ifTrue:[ 
        ^ OCI8NumberConverter new.
    ].

    #'NAMED_OBJECT_TYPE' == type ifTrue:[
        ^ OCI8NamedObjectTypeConverter new.
    ].

    (#(#'BINARY_LOB_TYPE' #'RAW_TYPE' #'LONG_RAW_TYPE') includes:type) ifTrue:[
        ^ OCI8ByteArrayConverter new.
    ].

    self error:'unknown data type'.
! !

!OCI8ColumnDescriptor methodsFor:'printing & storing'!

printOn:aStream

    aStream nextPutAll:self class name; nextPut:$(.
    name printOn:aStream.
    aStream nextPut:$)

! !

!OCI8ColumnDescriptor methodsFor:'testing'!

isPiecewiseType
"
checks if the type must be piecewised fetched.
Here only the long and long raw type must be fetched piecewise. The types raw and
varchar2 can also be fetched piecewise, but for performance issues we will fetch
them en block.

<return: Boolean>
"

    ^  #(#'CHARACTER_LOB_TYPE' #'BINARY_LOB_TYPE' #'LONG_TYPE' #'LONG_RAW_TYPE') includes:type.
! !

!OCI8ColumnDescriptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8ColumnDescriptor.st,v 1.19 2004/03/10 17:40:01 stefan Exp $'
! !
