"{ Package: 'stx:liboracle8' }"

OCI8Statement subclass:#OCI8DDStatement
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Statements'
!

!OCI8DDStatement class methodsFor:'documentation'!

documentation
"
    data definition statements are represented by this class. Those statements
    can only be executed via:
        OCI8Session>>executeStatement:

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8Statement
        OCI8DMStatement
        OCI8QueryStatement

    [instance variables:]
        none

    [class variables:]
        none
"
!

history
    "Created: / 23.3.2000 / 15:45:59 / ps"
! !

!OCI8DDStatement class methodsFor:'examples'!

examples_dropAndCreateTable
"
create the table inserttest for the examples of this class.
                                                                [exBegin]
    |session dropTableStatement createTableStatement|

    session := OCI8Session default.
    dropTableStatement := session newStatementFor:'drop table inserttest'.
    createTableStatement := session newStatementFor:'create table inserttest (id number, floats float, name varchar2(100), binary long raw)'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    OCIErrorSignal handle:[:ex |
    ] do:[
        session executeStatement:dropTableStatement.
    ].
    OCIErrorSignal handle:[:ex |
    ] do:[
        session executeStatement:createTableStatement.
    ].

    session disconnect.
                                                                [exEnd]
"

!

examples_truncateTable
"
create the table inserttest for the examples of this class.
                                                                [exBegin]
    |session truncateStatement|

    session := OCI8Session default.
    truncateStatement := session newStatementFor:'truncate table inserttest'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    session executeStatement:truncateStatement.

    session disconnect.
                                                                [exEnd]
"


! !

!OCI8DDStatement methodsFor:'execution'!

execute
"
executes a data definition statement without any variables in the statement text.


<return: self>
"
    "execute statement"
    super execute.
    "compute how many rows are processed by the statement"
    self computeProcessedRows
! !

!OCI8DDStatement methodsFor:'subclass responsibility'!

iterator
"
returns the number of many times a statement must be executed.
A data definition statement must be executed once. 

<return: Integer>
"
    ^1


! !

!OCI8DDStatement methodsFor:'testing'!

isDDLStatement
"
returns if the statement is a data definition statement

<return: Boolean>
"
    ^true



! !

!OCI8DDStatement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8DDStatement.st,v 1.6 2002/10/11 14:47:37 stefan Exp $'
! !
