"{ Package: 'stx:liboracle8' }"

OCI8Converter subclass:#OCI8DateConverter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Converter'
!

!OCI8DateConverter class methodsFor:'documentation'!

documentation
"
    This class is used to convert bytes from ExternalBytes (representing a date string)
    to a String.

    [author:]
	Pierre Schwarz

    [see also:]
	OCI8ByteArrayConverter
	OCI8FixedPointConverter
	OCI8FloatConverter
	OCI8IntegerConverter
	OCI8LongConverter
	OCI8StringConverter

    [instance variables:]
	none

    [class variables:]
	none
"

!

history
    "Created: / 22.8.2000 / 12:31:30 / ps"
! !

!OCI8DateConverter methodsFor:'accessing'!

basicElementSize
"
returns the maximum size of the external bytes buffer. 
Only used by returning variables.

<return: Integer>
"
    ^ 7
!

dataSizeForColumnDescriptor:anOCI8ColumnDescriptor
"
returns the maximum size of the external bytes buffer. 
Only used by define variables.

<return: Integer>
"
    ^ 7
!

externalCType
"
returns the symbol needed for the bind operation of an OCI8Variable.

<return: Symbol>
"
    ^ self cTypeDate
!

maxSizeForCType
"
returns the maximum possible size of the external bytes buffer. 
Only used by dynamic define variables.

<return: Integer>
"
    ^ 7
! !

!OCI8DateConverter methodsFor:'buffer accessing'!

convertToCTypeValue:aDateOrTime
    "convers a date or time value to the internal oracle format"

    |timestamp bytes year|

    timestamp := aDateOrTime asTimestamp.

    bytes := ByteArray new:7.
    year := timestamp year.
    bytes at:1 put:(year quo:100) + 100.
    bytes at:2 put:(year rem:100) + 100.                                            
    bytes at:3 put:timestamp month.
    bytes at:4 put:timestamp day.
    bytes at:5 put:timestamp hour + 1.
    bytes at:6 put:timestamp minute + 1.
    bytes at:7 put:timestamp second + 1.

    ^ bytes

    "
        self new convertToCTypeValue:TimeStamp now
    "
!

getDataFromBuffer:bytes size:size
    "convert an internal Oracle DATE to an AbsoluteTime value"

    ^ Timestamp year:((bytes at:1)-100)*100 + ((bytes at:2) - 100)
                   month:(bytes at:3)
                   day:(bytes at:4)
                   hour:(bytes at:5)-1
                   minute:(bytes at:6)-1
                   second:(bytes at:7)-1.
     "
        |now converter|

        now := Timestamp now clearMilliseconds.
        converter := self new.
        (converter getDataFromBuffer:(converter convertToCTypeValue:now)) = now
     "
! !

!OCI8DateConverter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8DateConverter.st,v 1.6 2004/03/20 15:31:12 stefan Exp $'
! !
