"{ Package: 'stx:liboracle8' }"

OCI8Variable subclass:#OCI8DefineVariable
	instanceVariableNames:'columnDescriptor position'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8DefineVariable class methodsFor:'documentation'!

documentation
"
    This abstract class is used to return data from a query statement. Each column which is returned
    is represented by a define variable. Two classes are used for query statements:

        OCI8DefaultDefineVariable
            Is used when no piecewise fetch is necessary for the column
            represented by the define variable.

        OCI8DynamicDefineVariable
            Is used when a piecewise fetch is necessary for the column
            represented by the define variable.

     A piecewise fetch must be done for long, long raw cloumns.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8DefaultDefineVariable
        OCI8DynamicDefineVariable
        OCI8Variable

    [instance variables:]

        columnDescriptor <OCI8ColumnDescriptor>
            holds the descriptor of the define variable. The descriptor describes the
            column of a table for which the define variable is generated.

        position <Integer>
            holds the index of the define variable in the select cluase of the statement.

    [class variables:]
        none
"
!

examples
"
    examples to be added.
								[exBegin]
    |session dropTableStatement createTableStatement|

    session := OCI8Session default.
    session username:'scott' password:'tiger' dbname:'secrec'.
    dropTableStatement := session newStatementFor:'drop table numbertest'.
    createTableStatement := session newStatementFor:'create table numbertest (table_intger integer, table_float float, table_number_1_0 number(1,0), table_number_38_0 number(38,0))'.

    session connect.
    OCIErrorSignal handle:[:ex |
    ] do:[
	session executeStatement:dropTableStatement.
    ].
    OCIErrorSignal handle:[:ex |
    ] do:[
	session executeStatement:createTableStatement.
    ].
    Transcript cr.
    Transcript show:'NAME';tab;tab;show:'SCALE';tab;tab;show:'PRECISION'.
    Transcript cr.


    (session columnsOfTable:'numbertest') do:[:eachDescriptor|
	Transcript show:eachDescriptor name;tab;show:eachDescriptor scale printString;tab;tab;show:eachDescriptor precision printString.
	Transcript cr.
    ].

    session disconnect.

								[exEnd]
"
!

history
    "Created: / 21.8.2000 / 13:20:33 / ps"
! !

!OCI8DefineVariable class methodsFor:'instance creation'!

newFor:anOCI8Statement columnDescriptor:anOCI8ColumnDescriptor position:aPositionInteger
    "return a new default or dynamic (for piecewise fetches) define variable. 
     The external buffers are setup and the define variable is bound
     to the statement via an oci call.

     <return: OCI8DynamicDefineVariable|OCI8DefaultDefineVariable>"

    |defineVariable|

    (anOCI8ColumnDescriptor isPiecewiseType) ifTrue:[
        defineVariable := OCI8DynamicDefineVariable new.
    ] ifFalse:[
        defineVariable := OCI8DefaultDefineVariable new.
    ].
    defineVariable columnDescriptor:anOCI8ColumnDescriptor.
    defineVariable position:aPositionInteger.
    defineVariable setupFor:anOCI8Statement.

    ^ defineVariable
! !

!OCI8DefineVariable methodsFor:'accessing'!

columnDescriptor
"
returns the descriptor of the receiver. The descriptor describes the
column of a table for which the receiver is generated.

<return: OCI8ColumnDescriptor>
"

    ^ columnDescriptor
!

columnDescriptor:anOCI8ColumnDescriptor
"
sets the descriptor of the receiver. The descriptor describes the
column of a table for which the receiver is generated.

<return: self>
"
    columnDescriptor := anOCI8ColumnDescriptor.
!

handleType
"
default handle type for the receeiver.

<return: Symbol>
"
    ^#'OCI_HTYPE_DEFINE'.



!

position
"
returns the index where the receiver is defined.

<return: Integer>
"

    ^ position
!

position:anInteger
"
sets the index where the receiver is defined.


<return: self>
"
    position := anInteger.
! !

!OCI8DefineVariable methodsFor:'setup'!

computeConverter
"
sets the converter for the receiver. The converter depends on the
table column.

<return: self>
"
    self converter:(self columnDescriptor newConverter)

! !

!OCI8DefineVariable methodsFor:'testing'!

isDynamic
"
test if the receiver is fetched via a piecevise fetch. Here false.

<return: Boolean>
"
    ^ false
! !

!OCI8DefineVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8DefineVariable.st,v 1.21 2004/11/09 08:32:15 stefan Exp $'
! !
