"{ Package: 'stx:liboracle8' }"

OCI8DefineVariable subclass:#OCI8DynamicDefineVariable
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8DynamicDefineVariable class methodsFor:'documentation'!

documentation
"
    This class is used to return data from a query statement when a piecewise
    fetch is necessary for the column represented by the define variable.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8DefaultDefineVariable
        OCI8DefineVariable
        OCI8Variable

    [instance variables:]
        pieceSize <Integer>
            holds the size of the pieces which are fetched in each iteration
            of a piecewise fetch.

    [class variables:]
        none
"
!

history
    "Created: / 30.8.2000 / 12:08:26 / ps"
! !

!OCI8DynamicDefineVariable methodsFor:'buffer accessing'!

getValue
    "
     access the data form the external bytes aBuffer. The value is stored there in the
     external c type format of the used converter. The value is converted into the smalltalk format
     of the used converter.

     <return: String|ByteArray|nil>
    "

    |size|

    size := dataBuffer size.
    size == 0 ifTrue:[
        ^ nil
    ].

    ^ converter getDataFromBuffer:dataBuffer size:size.
! !

!OCI8DynamicDefineVariable methodsFor:'finalization'!

executor
    "I do not need finalization.
     No indicator array, no elementLengthArray and dataBuffer is a ByteArray"

    ^ nil
! !

!OCI8DynamicDefineVariable methodsFor:'setup'!

setupFor:anOCI8Statement
"
setup the external bytes, the converter etc. and define the receiver.

<return: self>
"
    self computeConverter.
    anOCI8Statement ociLib ociDefineByPos:anOCI8Statement handle 
                             defineHandle:self handle
                                    cType:self externalCType 
                                   buffer:nil
                               bufferSize:converter maxSizeForCType
                                 position:position 
                       nullValueIndicator:nil
                       elementLengthArray:nil 
                          returnCodeArray:nil 
                                     mode:#'OCI_DYNAMIC_FETCH' 
                           errorOCIHandle:anOCI8Statement errorHandle.
! !

!OCI8DynamicDefineVariable methodsFor:'testing'!

isDynamic
"
test if the receiver is fetched via a piecevise fetch. Here true.

<return: Boolean>
"
    ^true

! !

!OCI8DynamicDefineVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8DynamicDefineVariable.st,v 1.20 2004/11/09 08:32:21 stefan Exp $'
! !
