"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Handle
	instanceVariableNames:'handle type'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Utilities'
!

!OCI8Handle class methodsFor:'documentation'!

documentation
"
    Holds information about a pointer to an oci lib handle.
    The pointer is stored as an external address object and
    the type as a symbol.

    [author:]
         Pierre Schwarz

    [see also:]
         OCI8Lib
         OCI8Session
         OCI8Variable

    [instance variables:]

        handle <ExternalAddress>
            Holds the external address which is a pointer to an oci handle.

        type <Symbol>
            Holds the type of the oci pointer

    [class variables:]
        none
"
!

history
    "Created: / 30.5.2000 / 14:49:20 / ps"
! !

!OCI8Handle methodsFor:'accessing'!

handle
"
returns the external address which is a pointer to an oci handle.

<return: ExternalAddress>
"
    handle isNil ifTrue:[
        handle := ExternalAddress new.
    ].
    ^ handle
!

handle:anExternalAddress
"
sets the external address which is a pointer to an oci handle.

<return: self>
"
    handle := anExternalAddress.
!

type
"
returns the type of the oci pointer.

<return: Symbol>
"

    ^ type
!

type:aSymbol
"
sets the type of the oci pointer.

<return: self>
"

    type := aSymbol.
! !

!OCI8Handle methodsFor:'printing & storing'!

printOn:aStream

    aStream nextPutAll:self class name; nextPut:$(.
    type printOn:aStream.
    aStream nextPut:$:.
    self handle notNil ifTrue:[
        self handle address printOn:aStream base:16.
    ].
    aStream nextPut:$).
! !

!OCI8Handle methodsFor:'queries'!

isValid
"
when the address of the receiver is set return true.

<return: Boolean>
"
    ^self handle address ~= 0
!

notValid
"
when the address of the receiver isn't set return true.

<return: Boolean>
"
    ^self handle address == 0
! !

!OCI8Handle class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Handle.st,v 1.8 2000/09/11 23:13:37 stefan Exp $'
! !
