"{ Package: 'stx:liboracle8' }"

OCI8BindVariable subclass:#OCI8MultipleValueBindVariable
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8MultipleValueBindVariable class methodsFor:'documentation'!

documentation
"
    This bind variable must be used, if an array should be insert into the database.
    More then one value is given for each bind variable.

    [author:]
	Pierre Schwarz

    [see also:]
	OCI8BindVariable
	OCI8ReturnValueBindVariable
	OCI8SingleValueBindVariable
	OCI8Variable

    [instance variables:]
	none

    [class variables:]
	none
"
!

history
    "Created: / 30.8.2000 / 11:59:13 / ps"
! !

!OCI8MultipleValueBindVariable class methodsFor:'instance creation'!

name:aName converter:anOCI8Converter
"
returns an instance of OCI8MultipleValueBindVariable class. Used for
array inserts.

<return: OCI8MultipleValueBindVariable>
"
   |inst|

   inst:= super new.
   inst name:aName.
   inst converter:anOCI8Converter.
   ^inst


! !

!OCI8MultipleValueBindVariable methodsFor:'accessing'!

addValue:anObject
"
adds a value anObject of the receiver. Set the state setup to false
because the external bytes must be rearranged. This is done before executing
the statement to which the receiver belongs.

<return: self>
"

    self isSetup ifTrue:[
	self initializeValue
    ].
    self value add:anObject.
    self setup:false.

!

getConvertedValueColl
"
return the value of the receiver. The stored value of the receiver
is converted into the external c type.

<return: Array with:<ByteArray|String|nil>>
"  

    ^ self value collect:[:eachValue |
	eachValue isNil ifTrue:[
	    nil
	] ifFalse:[
	    converter convertToCTypeValue:eachValue
	]
    ]
!

getValue

    |nElements values value tmpBuffer elementSize indicatorStep size|

    elementSize := self maxElementSize.
    nElements := self numberOfElements.
    values := Array new:nElements.
    indicatorStep := OCI8Lib sb2Size.
    1 to:nElements do:[:index |
	(indicatorArray shortAt:1+((index-1)*indicatorStep)) = -1 ifTrue:[
	    value := nil.
	] ifFalse:[
	    size := elementLengthArray unsignedShortAt:1+((index-1)*indicatorStep).
	    tmpBuffer := dataBuffer referenceToBytesFrom:1+((index-1)*elementSize) to:(index*elementSize).
	    value := converter getDataFromBuffer:tmpBuffer size:size.
	].
	values at:index put:value.
    ].
    ^ values
!

initializeValue

    value := OrderedCollection new

!

numberOfElements
"
returns the number of elements which are stored in the value of the
receiver.

<return: Integer>
"    

   ^ value size
!

value
"
return the value coollection of the receiver. These values are
copied into the data buffer.

<return: OrderedCollection>
"
    value isNil ifTrue:[
	self initializeValue
    ].
    ^value

! !

!OCI8MultipleValueBindVariable methodsFor:'setup'!

setupExternalBytes
"
<return: self>
"

    |size sizeArray max numberOfElements bufferSize offset externalValues indicatorStep indicatorOffset|

    externalValues := self getConvertedValueColl.
    max := 0.
    sizeArray := externalValues collect:[:val|
        size := val size.
        max := max max:size.
        size
    ].

    numberOfElements := self numberOfElements.
    indicatorStep := OCI8Lib ub2Size.

    size := numberOfElements * indicatorStep.
    elementLengthArray size < size ifTrue:[   
        |tmpBuffer|
        elementLengthArray notNil ifTrue:[
            tmpBuffer := elementLengthArray.
            elementLengthArray := nil.
            tmpBuffer free.
        ].
        elementLengthArray := OCI8Lib bufferForSize:size.

        indicatorArray notNil ifTrue:[
            tmpBuffer := indicatorArray.
            indicatorArray := nil.
            tmpBuffer free.
        ].
        indicatorArray := OCI8Lib bufferForSize:size.
        needsBind := true.
    ].

    bufferSize := dataBuffer size.

    (maxElementSize isNil or:[max > (maxElementSize ? 0) 
     or:[bufferSize < (maxElementSize * numberOfElements)]]) ifTrue:[
        "maxElementSize changes, need rebind"
        maxElementSize := max.
        needsBind := true.
    ].
    size := maxElementSize * numberOfElements.
    (bufferSize < size) ifTrue:[
        dataBuffer notNil ifTrue:[
            dataBuffer free.
        ].
        dataBuffer := OCI8Lib bufferForSize:size.
        "dataBuffer changes, need rebind"
        needsBind := true.
    ].

    needsBind ifTrue:[
        "some buffers changed"
        self reRegisterForFinalization.
    ].

    offset := 1.
    indicatorOffset := 1.
    externalValues with:sizeArray do:[:eachValue :eachValueSize| 
        eachValue isNil ifTrue:[
            indicatorArray shortAt:indicatorOffset put:-1.
        ] ifFalse:[ |sz|
            indicatorArray shortAt:indicatorOffset put:0.
            "cut to 65535 bytes"
            eachValueSize > 65535 ifTrue:[sz := 65535] ifFalse:[sz := eachValueSize].
            elementLengthArray unsignedShortAt:indicatorOffset put:sz.
            eachValueSize > 0 ifTrue:[
                dataBuffer replaceBytesFrom:offset to:offset+sz-1
                           with:eachValue startingAt:1.
            ]
        ].
        offset := offset + maxElementSize.
        indicatorOffset := indicatorOffset + indicatorStep.
    ]
! !

!OCI8MultipleValueBindVariable methodsFor:'testing'!

isArrayBind
"
returns if the receiver is an array bind.

<return: Boolean>
"

    ^ true
! !

!OCI8MultipleValueBindVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8MultipleValueBindVariable.st,v 1.23 2002/10/01 08:37:53 stefan Exp $'
! !
