"{ Package: 'stx:liboracle8' }"

OCI8MultipleValueBindVariable subclass:#OCI8ReturnMultipleValueBindVariable
	instanceVariableNames:'numberOfElements'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!


!OCI8ReturnMultipleValueBindVariable methodsFor:'accessing'!

basicElementSize
"
returns the maximum size for each element which can be returned by a returning clause.
This size depends on the smalltalk type of the receiver.

<return: Integer>
"
    ^ converter basicElementSize
!

numberOfElements
    "return the value of the instance variable 'numberOfElements' (automatically generated)"

    ^ numberOfElements
!

numberOfElements:something
    "set the value of the instance variable 'numberOfElements' (automatically generated)"

    numberOfElements := something.
!

value

    ^ self values:self numberOfElements
!

values:numberOfValues
"
returns a collection of the values received from the callback functions.
The values are converted in the converter to the approperiate smalltalk type.

<return: Collection of:<nil|ByteArray|FixedPoint|Float|Integer|String>>
"

    |fetchedRows elementSize returnArray size
     indicatorStep bufferOffset indicatorOffset|

    fetchedRows := numberOfValues min:self numberOfElements.

    (fetchedRows == 0
     or:[(elementSize := self basicElementSize) == 0
     or:[dataBuffer size == 0]]) ifTrue:[
        ^ #()
    ].

    returnArray := Array new:fetchedRows.

    indicatorStep := OCI8Lib sb2Size.
    bufferOffset := indicatorOffset := 1.
    1 to:fetchedRows do:[:index | |value tmpBuffer|
        (indicatorArray  shortAt:indicatorOffset) = -1 ifTrue:[
            value := nil.
        ] ifFalse:[
            size := elementLengthArray unsignedShortAt:indicatorOffset.
            tmpBuffer := dataBuffer referenceToBytesFrom:bufferOffset to:bufferOffset+size-1.
            value := converter getDataFromBuffer:tmpBuffer size:size.
        ].
        returnArray at:index put:value.
        bufferOffset := bufferOffset + maxElementSize.
        indicatorOffset := indicatorOffset + indicatorStep.
    ].
    ^ returnArray
! !

!OCI8ReturnMultipleValueBindVariable methodsFor:'setup'!

setupExternalBytes
"
<return: self>
"

    |size|

    size := numberOfElements * OCI8Lib ub2Size.
    elementLengthArray size < size ifTrue:[   
        |tmpBuffer|
        elementLengthArray notNil ifTrue:[
            tmpBuffer := elementLengthArray.
            elementLengthArray := nil.
            tmpBuffer free.
        ].
        elementLengthArray := OCI8Lib bufferForSize:size.

        indicatorArray notNil ifTrue:[
            tmpBuffer := indicatorArray.
            indicatorArray := nil.
            tmpBuffer free.
        ].
        indicatorArray := OCI8Lib bufferForSize:size.
        needsBind := true.
    ].
    elementLengthArray atAllPut:0.
    indicatorArray atAllPut:1.

    maxElementSize := self basicElementSize.
    size := maxElementSize * numberOfElements.
    (dataBuffer size < size) ifTrue:[
        dataBuffer notNil ifTrue:[
            dataBuffer free.
        ].
        dataBuffer := OCI8Lib bufferForSize:size.
        needsBind := true.
    ].
    needsBind ifTrue:[
        self reRegisterForFinalization.
    ].
!

setupForStatement:anOciStatement

    numberOfElements := anOciStatement iterator.
    self setupExternalBytes.
    self setup:true.
    self needsBind ifTrue:[
	self bindForStatement:anOciStatement.
    ].
! !

!OCI8ReturnMultipleValueBindVariable methodsFor:'testing'!

isReturnVariable
"
returns if the receiver is a bind variable which is used to return
values. Used in returning clauses of statements.

<return: Boolean>
"

    ^ true
! !

!OCI8ReturnMultipleValueBindVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8ReturnMultipleValueBindVariable.st,v 1.6 2003/02/19 14:58:29 stefan Exp $'
! !
