"{ Package: 'stx:liboracle8' }"

OCI8BindVariable subclass:#OCI8SingleValueBindVariable
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8SingleValueBindVariable class methodsFor:'documentation'!

documentation
"
    This bind variable must be used, if one value for each bind variable
    should be insert into the database.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8BindVariable
        OCI8MultipleValueBindVariable
        OCI8ReturnValueBindVariable
        OCI8Variable

    [instance variables:]
        none

    [class variables:]
        none
"
!

history
    "Created: / 30.8.2000 / 12:00:42 / ps"
! !

!OCI8SingleValueBindVariable class methodsFor:'instance creation'!

name:aName converter:anOCI8Converter
"
returns an instance of OCI8SingleValueBindVariable class. Used for
single value inserts.

<return: OCI8SingleValueBindVariable>
"
   |inst|

   inst:= super new.
   inst name:aName.
   inst converter:anOCI8Converter.
   ^inst


! !

!OCI8SingleValueBindVariable methodsFor:'accessing'!

addValue:aNewValue
"
set the value of the receiver. If the value is nil the flag nullValue is set
to true. If the receiver is an array bind the null value flag is also set
when the argument aNewValue is an empty collection. Set the state setup to false
because the external bytes must be rearranged. This is done before executing
the statement to which the receiver belongs.

<return: self>
"
    value := aNewValue.
    self setup:false.


!

numberOfElements
"
returns number of elements for a single bind value and that is one element.

<return: Integer>
"
    ^ 1
! !

!OCI8SingleValueBindVariable methodsFor:'setup'!

setupExternalBytes
"
sets up the data buffer of the receiver.

<return: self>
"

    |size externalValue|

    value isNil ifTrue:[
        "A null value is indicated by a size of 0"
        size := 0.
    ] ifFalse:[
        externalValue := converter convertToCTypeValue:value.
        size := externalValue size.
        size > 0 ifTrue:[
            dataBuffer size < size ifTrue:[
                dataBuffer notNil ifTrue:[
                    dataBuffer free.
                ].
                dataBuffer := OCI8Lib bufferForSize:size.
                self reRegisterForFinalization.
            ].
            dataBuffer replaceBytesFrom:1 to:size
                       with:externalValue startingAt:1.
        ].
    ].
    size ~~ maxElementSize ifTrue:[
        maxElementSize := size.
        needsBind := true.
    ].
! !

!OCI8SingleValueBindVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8SingleValueBindVariable.st,v 1.19 2002/10/05 18:24:03 stefan Exp $'
! !
