"{ Package: 'stx:liboracle8' }"

Error subclass:#OCIErrorSignal
	instanceVariableNames:'errorCode errorHandle statement'
	classVariableNames:'ErrorCodeMapping'
	poolDictionaries:''
	category:'DataBase-Oracle8-ErrorSignals'
!

!OCIErrorSignal class methodsFor:'documentation'!

documentation
"
    This ist the mother of all Oracle Signals...

    [author:]
        Stefan Vogel (stefan@zwerg)

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!OCIErrorSignal class methodsFor:'initialization'!

initialize

    NotifierString := 'OCI: error'.
    ErrorCodeMapping := Dictionary new.
! !

!OCIErrorSignal class methodsFor:'queries'!

mayProceed
    ^ true


! !

!OCIErrorSignal class methodsFor:'registering'!

newExceptionForErrorNumber:anErrorNumber
    "answer an exception for the given errorNumber"

    |class|

    class := ErrorCodeMapping at:anErrorNumber ifAbsent:[].
    class isNil ifTrue:[
        class := self.
    ].
    ^ class newException.
!

register:anError asErrorNumber:errorNumber
    "register anError (Signal or Exception class) with a given error number"

    ErrorCodeMapping isNil ifTrue:[
        ErrorCodeMapping := Dictionary new.
    ].

    ErrorCodeMapping at:errorNumber put:anError.
! !

!OCIErrorSignal methodsFor:'accessing'!

errorCode
    "return the value of the instance variable 'errorCode' (automatically generated)"

    ^ errorCode
!

errorCode:something
    "set the value of the instance variable 'errorCode' (automatically generated)"

    errorCode := something.
!

errorHandle
    "return the value of the instance variable 'errorHandle' (automatically generated)"

    ^ errorHandle
!

errorHandle:something
    "set the value of the instance variable 'errorHandle' (automatically generated)"

    errorHandle := something.
!

statement
    "return the value of the instance variable 'statement' (automatically generated)"

    ^ statement
!

statement:something
    "set the value of the instance variable 'statement' (automatically generated)"

    statement := something.
! !

!OCIErrorSignal methodsFor:'getting error parameters'!

getErrorRecord:recordNumber
    "get the error record numbered recordNumber.
     Return nil, if there is none"

    |errorArray|

    errorArray := Array new:2.

    errorCode := OCI8Lib new errorCodeForHandle:errorHandle handle
                             recordNumber:recordNumber
                             errorTextHolder:errorArray.
    errorCode isNil ifTrue:[
        ^ nil.
    ].
    errorArray at:2 put:(errorArray at:1).
    errorArray at:1 put:errorCode.

    ^ errorArray.
!

getRowOffset
    "get the row offset, where the DML error occured"

    ^ OCI8Lib new getRowOffset:errorHandle.
! !

!OCIErrorSignal methodsFor:'testing'!

isEndOfFileCommunication
"
returns false, only an OCIEndOfFileCommunicationSignal intance returns true

<return: Boolean>
"
    ^false





!

isInvalidDatabaseParameterError
"
returns false, only an OCIInvalidDatabaseParameterSignal intance returns true

<return: Boolean>
"
    ^false





!

isInvalidOperation
"
returns false, only an OCIInvalidOperationSignal intance returns true

<return: Boolean>
"
    ^false
!

isInvalidUserParameterError
"
returns false, only an OCIInvalidUserParameterSignal intance returns true

<return: Boolean>
"
    ^false





!

isLogOnError
"
returns false, only an OCILogOnErrorSignal intance returns true

<return: Boolean>
"
    ^false
!

isNeedData
"
returns false, only an OCINeedDataSignal intance returns true

<return: Boolean>
"
    ^ false



!

isNoData
"
returns false, only an OCINoDataSignal intance returns true

<return: Boolean>
"
    ^false







!

isStillExecuting
"
returns false, only an OCIStillExecutingSignal intance returns true

<return: Boolean>
"
    ^ false



!

isSuccessWithInfo
"
returns false, only an OCISuccessWithInfoSignal intance returns true

<return: Boolean>
"
    ^ false



!

isUserInterrupt
"
returns false, only an OCIUserInterruptSignal intance returns true

<return: Boolean>
"
    ^false
! !

!OCIErrorSignal class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCIErrorSignal.st,v 1.13 2003/02/06 17:49:55 stefan Exp $'
! !

OCIErrorSignal initialize!
