/* -----------------------------------------------------------------------
   ffi_common.h - Copyright (c) 1996  Cygnus Solutions

   $Id: ffi_common.h,v 1.3 2006/04/25 16:49:08 cg Exp $

   Common internal definitions and macros. Only necessary for building
   libffi.
   ----------------------------------------------------------------------- */

#ifndef FFI_COMMON_H
#define FFI_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

/* Do not move this. Some versions of AIX are very picky about where
   this is positioned. */
#ifdef __GNUC__
# define alloca __builtin_alloca
#else
# if HAVE_ALLOCA_H
#  include <alloca.h>
# else
#  ifdef _AIX
 #pragma alloca
#  else
#   ifdef NEED_MALLOC_H
#    include <malloc.h>
#   else
#    ifndef alloca /* predefined by HP cc +Olibcalls */
char *alloca ();
#    endif
#   endif
#  endif
# endif
#endif

#ifdef __BORLANDC__
# define ALLOCA(n)       malloc(n)
# define FREEA(ptr)        free(ptr)
#endif

#ifndef ALLOCA
# define ALLOCA(n)       alloca(n)
# define FREEA(n)        /*not needed */
#endif

/* Check for the existence of memcpy. */
#if STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_MEMCPY
#  define memcpy(d, s, n) bcopy ((s), (d), (n))
# endif
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif

#ifndef __cplusplus
/* bool is a keyword in C++ */
/*@-cppnames@*/
typedef int bool;
/*@=cppnames@*/
#endif

#ifdef FFI_DEBUG

/* Debugging functions */
/*@exits@*/ int ffi_assert(/*@temp@*/ char *file, int line);
void ffi_stop_here(void);
bool ffi_type_test(/*@temp@*/ /*@out@*/ ffi_type *a);

#define FFI_ASSERT(x) ((x) ? 0 : ffi_assert(__FILE__,__LINE__))

#else

#define FFI_ASSERT(x)

#endif

/* Perform machine dependent cif processing */
ffi_status ffi_prep_cif_machdep(ffi_cif *cif);

/* Extended cif, used in callback from assembly routine */
typedef struct
{
  /*@dependent@*/ ffi_cif *cif;
  /*@dependent@*/ void *rvalue;
  /*@dependent@*/ void **avalue;
} extended_cif;

#ifdef __cplusplus
}
#endif

#ifndef ORIGINAL
ffi_type *__get_ffi_type_uint();
ffi_type *__get_ffi_type_sint();
ffi_type *__get_ffi_type_uint8();
ffi_type *__get_ffi_type_sint8();
ffi_type *__get_ffi_type_uint16();
ffi_type *__get_ffi_type_sint16();
ffi_type *__get_ffi_type_uint32();
ffi_type *__get_ffi_type_sint32();
ffi_type *__get_ffi_type_uint64();
ffi_type *__get_ffi_type_sint64();
ffi_type *__get_ffi_type_double();
ffi_type *__get_ffi_type_float();


#endif

#endif
