/* -----------------------------------------------------------------------
   ffi.c - Copyright (c) 1996, 1998, 1999, 2001  Red Hat, Inc.
	   Copyright (c) 2002  Ranjit Mathew
	   Copyright (c) 2002  Bo Thorsen
	   Copyright (c) 2002  Roger Sayle

   x86 Foreign Function Interface

   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
   ``Software''), to deal in the Software without restriction, including
   without limitation the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED ``AS IS'', WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL CYGNUS SOLUTIONS BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
   OTHER DEALINGS IN THE SOFTWARE.
   ----------------------------------------------------------------------- */

#if 0
# define DEBUG_puts(x)       puts x
# define DEBUG_printf(x)     printf x
# define DEBUG_fflush(x)     fflush x
#else
# define DEBUG_puts(x)       /* nothing */
# define DEBUG_printf(x)     /* nothing */
# define DEBUG_fflush(x)     /* nothing */
#endif

#ifndef __x86_64__

#include <ffi.h>
#include <ffi_common.h>

#include <stdlib.h>

/* ffi_prep_args is called by the assembly routine once stack space
   has been allocated for the function's arguments */

/*@-exportheader@*/
void ffi_prep_args(char *stack, extended_cif *ecif)
/*@=exportheader@*/
{
  register unsigned int i;
  register void **p_argv;
  register char *argp;
  register ffi_type **p_arg;

  argp = stack;

  if (ecif->cif->rtype->type == FFI_TYPE_STRUCT)
    {
      *(void **) argp = ecif->rvalue;
      argp += 4;
    }

  p_argv = ecif->avalue;

  for (i = ecif->cif->nargs, p_arg = ecif->cif->arg_types;
       i != 0;
       i--, p_arg++)
    {
      size_t z;

      /* Align if necessary */
      if (((*p_arg)->alignment - 1) & (unsigned) argp)
	argp = (char *) ALIGN(argp, (*p_arg)->alignment);

      z = (*p_arg)->size;
      if (z < sizeof(int))
	{
	  z = sizeof(int);
	  switch ((*p_arg)->type)
	    {
	    case FFI_TYPE_SINT8:
	      *(signed int *) argp = (signed int)*(SINT8 *)(* p_argv);
	      break;

	    case FFI_TYPE_UINT8:
	      *(unsigned int *) argp = (unsigned int)*(UINT8 *)(* p_argv);
	      break;

	    case FFI_TYPE_SINT16:
	      *(signed int *) argp = (signed int)*(SINT16 *)(* p_argv);
	      break;

	    case FFI_TYPE_UINT16:
	      *(unsigned int *) argp = (unsigned int)*(UINT16 *)(* p_argv);
	      break;

	    case FFI_TYPE_SINT32:
	      *(signed int *) argp = (signed int)*(SINT32 *)(* p_argv);
	      break;

	    case FFI_TYPE_UINT32:
	      *(unsigned int *) argp = (unsigned int)*(UINT32 *)(* p_argv);
	      break;

	    case FFI_TYPE_STRUCT:
	      *(unsigned int *) argp = (unsigned int)*(UINT32 *)(* p_argv);
	      break;

	    default:
	      FFI_ASSERT(0);
	    }
	}
      else
	{
	  memcpy(argp, *p_argv, z);
	}
      p_argv++;
      argp += z;
    }

  return;
}

/* Perform machine dependent cif processing */
ffi_status ffi_prep_cif_machdep(ffi_cif *cif)
{
  /* Set the return type flag */
  switch (cif->rtype->type)
    {
    case FFI_TYPE_VOID:
    case FFI_TYPE_STRUCT:
    case FFI_TYPE_SINT64:
    case FFI_TYPE_FLOAT:
    case FFI_TYPE_DOUBLE:
#if FFI_TYPE_LONGDOUBLE != FFI_TYPE_DOUBLE
    case FFI_TYPE_LONGDOUBLE:
#endif
      cif->flags = (unsigned) cif->rtype->type;
      break;

    case FFI_TYPE_UINT64:
      cif->flags = FFI_TYPE_SINT64;
      break;

    default:
      cif->flags = FFI_TYPE_INT;
      break;
    }

  return FFI_OK;
}

/*@-declundef@*/
/*@-exportheader@*/
extern void ffi_call_SYSV(void (*)(char *, extended_cif *),
			  /*@out@*/ extended_cif *,
			  unsigned, unsigned,
			  /*@out@*/ unsigned *,
			  void (*fn)());
/*@=declundef@*/
/*@=exportheader@*/

#ifdef X86_WIN32
/*@-declundef@*/
/*@-exportheader@*/
extern void ffi_call_STDCALL(void (*)(char *, extended_cif *),
			  /*@out@*/ extended_cif *,
			  unsigned, unsigned,
			  /*@out@*/ unsigned *,
			  void (*fn)());
/*@=declundef@*/
/*@=exportheader@*/
#endif /* X86_WIN32 */

void ffi_call(/*@dependent@*/ ffi_cif *cif,
	      void (*fn)(),
	      /*@out@*/ void *rvalue,
	      /*@dependent@*/ void **avalue)
{
  void *toFreeA = 0;
  extended_cif ecif;

  ecif.cif = cif;
  ecif.avalue = avalue;

  /* If the return value is a struct and we don't have a return */
  /* value address then we need to make one                     */

  if ((rvalue == NULL) &&
      (cif->rtype->type == FFI_TYPE_STRUCT))
    {
      /*@-sysunrecog@*/
      ecif.rvalue = toFreeA = ALLOCA(cif->rtype->size);
      /*@=sysunrecog@*/
    }
  else
    ecif.rvalue = rvalue;


  switch (cif->abi)
    {
    case FFI_SYSV:
      /*@-usedef@*/
      ffi_call_SYSV(ffi_prep_args, &ecif, cif->bytes,
		    cif->flags, ecif.rvalue, fn);
      /*@=usedef@*/
      break;
#ifdef X86_WIN32
    case FFI_STDCALL:
      /*@-usedef@*/
      ffi_call_STDCALL(ffi_prep_args, &ecif, cif->bytes,
		    cif->flags, ecif.rvalue, fn);
      /*@=usedef@*/
      break;
#endif /* X86_WIN32 */
    default:
      FFI_ASSERT(0);
      break;
    }
    if (toFreeA) {
	FREEA(toFreeA);
    }
}

#ifdef FFI_CLOSURES

/** private members **/

static void ffi_prep_incoming_args_SYSV (char *stack, void **ret,
					 void** args, ffi_cif* cif);
static int ffi_closure_SYSV ();
static void ffi_closure_raw_SYSV ();

/* This function is jumped to by the trampoline, on entry, %ecx (a
 * caller-save register) holds the address of the closure.
 * Clearly, this requires __GNUC__, so perhaps we should translate this
 * into an assembly file if this is to be distributed with ffi.
 */

# if !defined(__GNUC__)

/*
 * dont use ecx/edx to pass args - VISUALC uses cx in its rep-stosl memory-clear prolog
 * dont use eax for the return valu, because VISUALC clobbers it in its epilog
 */
#  if defined(xx__VISUALC__) || defined(xxVISUALC) || defined(__BORLANDC__)
#   define USE_ECX_EDX
#  endif
#  if defined(xx__VISUALC__) || defined(xxVISUALC) || defined(xx__BORLANDC__)
#   define USE_EAX
#  endif

#  if defined(USE_ECX_EDX)
static int
ffi_closure_SYSV ()
{
  ffi_closure *closure;
  int *argp;
#  else
static int
ffi_closure_SYSV (ffi_closure *closure, int *argp)
{
#  endif
  // this is our return value storage
  union {
      char           chars[sizeof(long)*4];
      long double    longDouble;
  } res;

  // our various things...
  ffi_cif       *cif;
  void         **arg_area;
  unsigned short rtype;
  void          *resp;
  void          *args;
  int            retval;

#  if defined(USE_ECX_EDX)
  _asm mov closure, ecx;
  _asm mov argp, edx ;
#  endif

  args = &argp[1];     /* notice: argp[0] is the return address on the stack */
  resp = (void*)&res;

DEBUG_printf(("ffi_closure_SYSV argp=%x closure=%x\n", argp, closure));
DEBUG_fflush((stdout));

  cif         = closure->cif;
DEBUG_printf(("ffi_closure_SYSV cif=%x\n", cif));
DEBUG_fflush((stdout));
DEBUG_printf(("ffi_closure_SYSV nargs=%x\n", cif->nargs));
DEBUG_fflush((stdout));

  arg_area    = (void**) ALLOCA (cif->nargs * sizeof (void*));

  /* this call will initialize ARG_AREA, such that each
   * element in that array points to the corresponding
   * value on the stack; and if the function returns
   * a structure, it will re-set RESP to point to the
   * structure return address.  */

  ffi_prep_incoming_args_SYSV(args, (void**)&resp, arg_area, cif);

DEBUG_printf(("ffi_closure_SYSV calling %x...\n", closure->fun));
DEBUG_fflush((stdout));
  (closure->fun) (cif, resp, arg_area, closure->user_data);
DEBUG_puts(("ffi_closure_SYSV back\n"));
DEBUG_fflush((stdout));

  rtype = cif->flags;
  retval = 0;

  /* now, do a generic return based on the value of rtype */
  if (rtype == FFI_TYPE_INT)
    {
	    retval = *(int *)resp;
	    _asm mov eax, resp ;
	    _asm mov eax, [eax] ;
    }
  else if (rtype == FFI_TYPE_FLOAT)
    {
	    _asm mov eax, resp ;
	    _asm fld DWORD PTR [eax] ;
//      asm ("flds (%0)" : : "r" (resp) : "st" );
    }
  else if (rtype == FFI_TYPE_DOUBLE)
    {
	    _asm mov eax, resp ;
	    _asm fld QWORD PTR [eax] ;
//      asm ("fldl (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_LONGDOUBLE)
    {
	    _asm mov eax, resp ;
	    _asm fld TBYTE PTR [eax] ;
//      asm ("fldt (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_SINT64)
    {
	    _asm mov edx, resp ;
	    _asm mov eax, [edx] ;
	    _asm mov edx, [edx + 4] ;
//      asm ("movl 0(%0),%%eax;"
//         "movl 4(%0),%%edx"
//         : : "r"(resp)
//         : "eax", "edx");
    }
    FREEA(arg_area);
#  ifndef USE_EAX
  return retval;
#  endif
}

# else /* GNUC */

static int
ffi_closure_SYSV ()
{
  // this is our return value storage
  long double    res;

  // our various things...
  void          *args;
  ffi_cif       *cif;
  void         **arg_area;
  ffi_closure   *closure;
  unsigned short rtype;
  void          *resp = (void*)&res;
  int            retval;

  /* grab the trampoline context pointer */
  asm ("movl %%ecx,%0" : "=r" (closure));

  cif         = closure->cif;
  arg_area    = (void**) ALLOCA (cif->nargs * sizeof (void*));
  asm ("leal 8(%%ebp),%0" : "=q" (args));

  /* this call will initialize ARG_AREA, such that each
   * element in that array points to the corresponding
   * value on the stack; and if the function returns
   * a structure, it will re-set RESP to point to the
   * structure return address.  */

  ffi_prep_incoming_args_SYSV(args, (void**)&resp, arg_area, cif);

  (closure->fun) (cif, resp, arg_area, closure->user_data);

  rtype = cif->flags;
  retval = 0;

  /* now, do a generic return based on the value of rtype */
  if (rtype == FFI_TYPE_INT)
    {
      retval = *(int *)resp;
      asm ("movl (%0),%%eax" : : "r" (resp) : "eax");
    }
  else if (rtype == FFI_TYPE_FLOAT)
    {
      asm ("flds (%0)" : : "r" (resp) : "st" );
    }
  else if (rtype == FFI_TYPE_DOUBLE)
    {
      asm ("fldl (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_LONGDOUBLE)
    {
      asm ("fldt (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_SINT64)
    {
      asm ("movl 0(%0),%%eax;"
	   "movl 4(%0),%%edx"
	   : : "r"(resp)
	   : "eax", "edx");
    }
  FREEA(arg_area);
#  ifndef USE_EAX
  return retval;
#  endif
}
# endif /* GNUC */

/*@-exportheader@*/
static void
ffi_prep_incoming_args_SYSV(char *stack, void **rvalue,
			    void **avalue, ffi_cif *cif)
/*@=exportheader@*/
{
  register unsigned int i;
  register void **p_argv;
  register char *argp;
  register ffi_type **p_arg;

  argp = stack;

  if ( cif->rtype->type == FFI_TYPE_STRUCT ) {
    *rvalue = *(void **) argp;
    argp += 4;
  }

  p_argv = avalue;

  for (i = cif->nargs, p_arg = cif->arg_types; (i != 0); i--, p_arg++)
    {
      size_t z;

      /* Align if necessary */
      if (((*p_arg)->alignment - 1) & (unsigned) argp) {
	argp = (char *) ALIGN(argp, (*p_arg)->alignment);
      }

      z = (*p_arg)->size;

      /* because we're little endian, this is what it turns into.   */

      *p_argv = (void*) argp;

      p_argv++;
      argp += z;
    }

  return;
}

# if defined(__VISUALC__) || defined(VISUALC) || defined(__BORLANDC__)

/* the cif must already be prep'ed */
extern void ffi_closure_OUTER();

ffi_status
ffi_prep_closure (ffi_closure* closure,
		  ffi_cif* cif,
		  void (*fun)(ffi_cif*,void*,void**,void*),
		  void *user_data)
{
  short bytes;
  char *tramp;
#  ifdef _WIN64
  int mask;
#  endif
DEBUG_printf(("prep_closure closure=%x cif=%x fun=%x userData=%x\n",
		closure, cif, fun, user_data));
DEBUG_printf(("prep_closure cif->abi=%x cif->nargs=%d\n",
		cif->abi, cif->nargs));
  FFI_ASSERT (cif->abi == FFI_SYSV);

  if (cif->abi == FFI_SYSV)
    bytes = 0;
#  if !defined(_WIN64)
  else if (cif->abi == FFI_STDCALL)
    bytes = cif->bytes;
#  endif
  else
    return FFI_BAD_ABI;

  tramp = &closure->tramp[0];

#define BYTES(text) memcpy(tramp, text, sizeof(text)), tramp += sizeof(text)-1
#define BYTE(x) *(char*)tramp = (x), tramp += sizeof(char)
#define POINTER(x) *(void**)tramp = (void*)(x), tramp += sizeof(void*)
#define SHORT(x) *(short*)tramp = x, tramp += sizeof(short)
#define INT(x) *(int*)tramp = x, tramp += sizeof(int)

#  ifdef _WIN64
  if (cif->nargs >= 1 &&
      (cif->arg_types[0]->type == FFI_TYPE_FLOAT
       || cif->arg_types[0]->type == FFI_TYPE_DOUBLE))
    mask |= 1;
  if (cif->nargs >= 2 &&
      (cif->arg_types[1]->type == FFI_TYPE_FLOAT
       || cif->arg_types[1]->type == FFI_TYPE_DOUBLE))
    mask |= 2;
  if (cif->nargs >= 3 &&
      (cif->arg_types[2]->type == FFI_TYPE_FLOAT
       || cif->arg_types[2]->type == FFI_TYPE_DOUBLE))
    mask |= 4;
  if (cif->nargs >= 4 &&
      (cif->arg_types[3]->type == FFI_TYPE_FLOAT
       || cif->arg_types[3]->type == FFI_TYPE_DOUBLE))
    mask |= 8;

  /* 41 BB ----         mov         r11d,mask */
  BYTES("\x41\xBB"); INT(mask);

  /* 48 B8 --------     mov         rax, closure                        */
  BYTES("\x48\xB8"); POINTER(closure);

  /* 49 BA --------     mov         r10, ffi_closure_OUTER */
  BYTES("\x49\xBA"); POINTER(ffi_closure_OUTER);

  /* 41 FF E2           jmp         r10 */
  BYTES("\x41\xFF\xE2");

#  else /* not WIN64 */

#   if defined(USE_ECX_EDX)
  /* mov ecx, closure */
  BYTES("\xb9"); POINTER(closure);
  /* mov edx, esp */
  BYTES("\x8b\xd4");
#    if 0
  /* call ffi_closure_SYSV */
  BYTES("\xe8"); POINTER((char*)&ffi_closure_SYSV - (tramp + 4));
  /* ret */
  BYTES("\xc3");
#    else
  /* jmp ffi_closure_SYSV */
  BYTES("\xe9"); POINTER((char*)&ffi_closure_SYSV - (tramp + 4));
#    endif

#   else /* do not use ecx/edx */
  /* mov ecx, esp */
  BYTE(0x8B); BYTE(0xCC);       /* 8B  C0|(rd<<3)|rs 11 001 100 */
  /* push ecx */
  BYTE(0x51);
  /* push closure */
  BYTE(0x68); POINTER(closure);
  /* call ffi_closure_SYSV */
  BYTE(0xe8); POINTER((char*)&ffi_closure_SYSV - (tramp + 4));
  /* pop ecx */
  BYTE(0x59);
  /* pop ecx */
  BYTE(0x59);
  /* ret */
  BYTE(0xc3);
#   endif /* use ecx/edx */
#  endif /* win64 */

  if ((tramp - &closure->tramp[0]) > FFI_TRAMPOLINE_SIZE) {
    // FatalError("FFI_TRAMPOLINE_SIZE too small in " __FILE__);
    return FFI_BAD_ABI;
  }

  closure->cif  = cif;
  closure->user_data = user_data;
  closure->fun  = fun;

  return FFI_OK;
}

# else /* not VISUALC/BORLANDC */

/* How to make a trampoline.  Derived from gcc/config/i386/i386.c. */

#define FFI_INIT_TRAMPOLINE(TRAMP,FUN,CTX) \
({ unsigned char *__tramp = (unsigned char*)(TRAMP); \
   unsigned int  __fun = (unsigned int)(FUN); \
   unsigned int  __ctx = (unsigned int)(CTX); \
   unsigned int  __dis = __fun - ((unsigned int) __tramp + 10); \
   *(unsigned char*) &__tramp[0] = 0xb9; \
   *(unsigned int*)  &__tramp[1] = __ctx; \
   *(unsigned char*) &__tramp[5] = 0xe9; \
   *(unsigned int*)  &__tramp[6] = __dis; \
 })


/* the cif must already be prep'ed */

ffi_status
ffi_prep_closure (ffi_closure* closure,
		  ffi_cif* cif,
		  void (*fun)(ffi_cif*,void*,void**,void*),
		  void *user_data)
{
  FFI_ASSERT (cif->abi == FFI_SYSV);

  FFI_INIT_TRAMPOLINE (&closure->tramp[0], \
		       &ffi_closure_SYSV,  \
		       (void*)closure);

  closure->cif  = cif;
  closure->user_data = user_data;
  closure->fun  = fun;

  return FFI_OK;
}
# endif /* not VISUALC/BORLANDC */

/* ------- Native raw API support -------------------------------- */

# if !FFI_NO_RAW_API
#  if !defined(__VISUALC__) && !defined(VISUALC) && !defined(__BORLANDC__)

static void
ffi_closure_raw_SYSV ()
{
  // this is our return value storage
  long double    res;

  // our various things...
  void            *args;
  ffi_raw         *raw_args;
  ffi_cif         *cif;
  ffi_raw_closure *closure;
  unsigned short   rtype;
  void            *resp = (void*)&res;

  /* grab the trampoline context pointer */
  asm ("movl %%ecx,%0" : "=r" (closure));

  /* take the argument pointer */
  asm ("leal 8(%%ebp),%0" : "=q" (args));

  /* get the cif */
  cif = closure->cif;

  /* the SYSV/X86 abi matches the RAW API exactly, well.. almost */
  raw_args = (ffi_raw*) args;

  (closure->fun) (cif, resp, raw_args, closure->user_data);

  rtype = cif->flags;

  /* now, do a generic return based on the value of rtype */
  if (rtype == FFI_TYPE_INT)
    {
      asm ("movl (%0),%%eax" : : "r" (resp) : "eax");
    }
  else if (rtype == FFI_TYPE_FLOAT)
    {
      asm ("flds (%0)" : : "r" (resp) : "st" );
    }
  else if (rtype == FFI_TYPE_DOUBLE)
    {
      asm ("fldl (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_LONGDOUBLE)
    {
      asm ("fldt (%0)" : : "r" (resp) : "st", "st(1)" );
    }
  else if (rtype == FFI_TYPE_SINT64)
    {
      asm ("movl 0(%0),%%eax; movl 4(%0),%%edx"
	   : : "r"(resp)
	   : "eax", "edx");
    }
}




ffi_status
ffi_prep_raw_closure (ffi_raw_closure* closure,
		      ffi_cif* cif,
		      void (*fun)(ffi_cif*,void*,ffi_raw*,void*),
		      void *user_data)
{
  int i;

  FFI_ASSERT (cif->abi == FFI_SYSV);

  // we currently don't support certain kinds of arguments for raw
  // closures.  This should be implemented by a separate assembly language
  // routine, since it would require argument processing, something we
  // don't do now for performance.

  for (i = cif->nargs-1; i >= 0; i--)
    {
      FFI_ASSERT (cif->arg_types[i]->type != FFI_TYPE_STRUCT);
      FFI_ASSERT (cif->arg_types[i]->type != FFI_TYPE_LONGDOUBLE);
    }


  FFI_INIT_TRAMPOLINE (&closure->tramp[0], &ffi_closure_raw_SYSV,
		       (void*)closure);

  closure->cif  = cif;
  closure->user_data = user_data;
  closure->fun  = fun;

  return FFI_OK;
}

#  endif

static void
ffi_prep_args_raw(char *stack, extended_cif *ecif)
{
  memcpy (stack, ecif->avalue, ecif->cif->bytes);
}

# endif

/* we borrow this routine from libffi (it must be changed, though, to
 * actually call the function passed in the first argument.  as of
 * libffi-1.20, this is not the case.)
 */

extern void
ffi_call_SYSV(void (*)(char *, extended_cif *),
	      /*@out@*/ extended_cif *,
	      unsigned, unsigned,
	      /*@out@*/ unsigned *,
	      void (*fn)());

# ifdef X86_WIN32
extern void
ffi_call_STDCALL(void (*)(char *, extended_cif *),
	      /*@out@*/ extended_cif *,
	      unsigned, unsigned,
	      /*@out@*/ unsigned *,
	      void (*fn)());
# endif /* X86_WIN32 */

void
ffi_raw_call(/*@dependent@*/ ffi_cif *cif,
	     void (*fn)(),
	     /*@out@*/ void *rvalue,
	     /*@dependent@*/ ffi_raw *fake_avalue)
{
  extended_cif ecif;
  void **avalue = (void **)fake_avalue;
  void *toFreeA = 0;

  ecif.cif = cif;
  ecif.avalue = avalue;

  /* If the return value is a struct and we don't have a return */
  /* value address then we need to make one                     */

  if ((rvalue == NULL) &&
      (cif->rtype->type == FFI_TYPE_STRUCT))
    {
      /*@-sysunrecog@*/
      ecif.rvalue = toFreeA = ALLOCA(cif->rtype->size);
      /*@=sysunrecog@*/
    }
  else
    ecif.rvalue = rvalue;


  switch (cif->abi)
    {
    case FFI_SYSV:
      /*@-usedef@*/
      ffi_call_SYSV(ffi_prep_args_raw, &ecif, cif->bytes,
		    cif->flags, ecif.rvalue, fn);
      /*@=usedef@*/
      break;
# ifdef X86_WIN32
    case FFI_STDCALL:
      /*@-usedef@*/
      ffi_call_STDCALL(ffi_prep_args_raw, &ecif, cif->bytes,
		    cif->flags, ecif.rvalue, fn);
      /*@=usedef@*/
      break;
# endif /* X86_WIN32 */
    default:
      FFI_ASSERT(0);
      break;
    }
    if (toFreeA) {
	FREEA(toFreeA);
    }
}

#endif /* FFI_CLOSURES */

#endif /* __x86_64__  */
