static char rcsid[] = "$Header: /cvs/stx/stx/librun/main.c,v 1.116 2010/03/09 09:23:32 cg Exp $";

/*
 * COPYRIGHT (c) 1988 by Claus Gittinger
 *              All Rights Reserved
 *
 * This software is furnished under a license and may be used
 * only in accordance with the terms of that license and with the
 * inclusion of the above copyright notice.   This software may not
 * be provided or otherwise made available to, or used by, any
 * other person.  No title to or ownership of the software is
 * hereby transferred.
 *
 * (claus: not much to be copyrighted though :-)
 */

/*
 * this file defines a default main (if none was explicit given)
 * it sets up everything and starts the show with "Smalltalk start"
 * also, some hooks have been plcaed here, to allow customization
 *
 * some of the behavior can be changed, by compiling this file
 * with appropriate defines:
 *    STARTUP_CLASS     - the class to send a start message (instead of Smalltalk)
 *    STARTUP_SELECTOR  - the selector to send (instead of start)
 *    IGNORE_IMAGE      - hard-code a '-I' setting (i.e. never look for an image)
 *    DIRECT_START      - directly go into STARTUPCLASS>>STARTUPSELECTOR
 *                        after all classes have been initialized.
 *                        If defined, its the startup methods responsibility
 *                        to initialize the system
 *                        (i.e. setup the processorScheduler, initialize the
 *                         display etc.)
 *    NOINFO            - infoPrinting initially off
 *    NODEBUG           - noDebug initially on
 */

/* #define SUPERDEBUG /* */
/* #define CMDDEBUG   /* */

/*
 * defaults (if not defined in your Makefile)
 */
#undef RESTART_SELECTOR

#ifndef STARTUP_CLASS
# define STARTUP_CLASS "Smalltalk"
#endif

#ifndef STARTUP_SELECTOR
# define STARTUP_SELECTOR "start"
#endif

#ifndef RESTART_SELECTOR
# define RESTART_SELECTOR "restart"
#endif

#ifndef CONFIGURATION_STRING
# ifdef __VMS__
#  define CONFIGURATION_STRING "vms/standard:standard"
# else
#  ifdef WIN32
#   ifdef __BORLANDC__
#    define CONFIGURATION_STRING "win32/bcc:standard"
#   else
#    ifdef __VISUALC__
#     define CONFIGURATION_STRING "win32/msvc:standard"
#    else
#     ifdef __LCC__
#      define CONFIGURATION_STRING "win32/lcc:standard"
#     else
#      define CONFIGURATION_STRING "win32/unknown:standard"
#     endif
#    endif
#   endif
#  else
#   define CONFIGURATION_STRING "unknown/unknown:standard"
#  endif
# endif
#endif

#ifndef BUILD_DATE
# ifdef WIN32
#  include "buildDate.h"
# endif

# ifndef BUILD_DATE
#  ifdef __VMS__
#   include "buildDate.h"
#  endif
#  ifndef BUILD_DATE
#   define BUILD_DATE "00-000-00 / 00:00"
#  endif
# endif
#endif

#ifdef WIN32
# include "windows.h"
# include "stdio.h"
#endif

#define __IN_VM__
#include "stc.h"

#ifdef SUPERDEBUG
# define SDPRINTF(x)    printf x
# define CMDDEBUG
#else
# define SDPRINTF(x)    /* */
#endif

#ifdef CMDDEBUG
# define CMDDPRINTF(x)  printf x
#else
# define CMDDPRINTF(x)  /* */
#endif

#ifdef IRIX5
  int __Argc = 1;
  char *__Argv[] = {"foo"};
  int __rld_obj_head;
#endif
#ifdef IRIS
/*
 * this kludge was (dont ask me why) required on some
 * IRIS systems I had here. All it does is to reference
 * those two external c variables (to pull in the other code).
 * Maybe its no longer required ...
 */
int __iris_workaround__ = 0;
#endif /* IRIS */

#ifdef NEXT3
/*
 * this kludge was (dont ask me why) required for NeXT v3 systems,
 * to reference some code from the static librun library.
 */
 static
 dummyToForceLoading() {
    __SEND0S();
    __MKREALCONTEXT5();
}
#endif


static int __hasNoConsole__ = 0;

#ifdef WIN32
# include <signal.h>
# define EXTERNAL_SYMBOLS

# ifdef USE_MODULE_TABLE
#  ifndef MODULES_DOT_C_FILE
#   define MODULES_DOT_C_FILE "modules.c"
#  endif
   /*
    * USE_MODULE_TABLE is no longer defined for windows.
    * modules are now loaded via an external ascii file (modules.stx),
    * which lists the dlls to be loaded
    */
#  include MODULES_DOT_C_FILE
# else /* loaded via modules.stx */
  static VOIDFUNC *__modules__[] = { 0 };
# endif
#else /* not WIN32 */
  extern VOIDFUNC *__modules__;
#endif /* not WIN32 */

#ifdef __openVMS__
# define EXTERNAL_SYMBOLS
# include "modules.c"
#endif

#ifdef __hppa__
# define EXTERNAL_SYMBOLS
#endif

#ifdef __hppa64__
extern void __stxMain__( char *, char *, char *, VOIDFUNC *, char *, char *, int, char **, void*, int);
extern void __stxRegisterCustomExitHook__( VOIDFUNC );
extern void __stxRegisterCustomPreInitHook__( VOIDFUNC );
extern void __stxRegisterCustomPostInitHook__( VOIDFUNC );
extern void __stxRegisterCustomPostReInitHook__( VOIDFUNC );
extern void __stxRegisterCustomInterruptHooks__( VOIDFUNC );
extern void __stxRegisterCustomFunctions__( CUSTOMFUNCTION * );
#else
extern void __stxMain__();
extern void __stxRegisterCustomExitHook__();
extern void __stxRegisterCustomPreInitHook__();
extern void __stxRegisterCustomPostInitHook__();
extern void __stxRegisterCustomPostReInitHook__();
extern void __stxRegisterCustomInterruptHooks__();
extern void __stxRegisterCustomFunctions__();
#endif

  static struct __vmData__ *__vmData__;
# define __VMDATA__      (*__vmData__)

/*********************************************************
 * startup and exit
 *********************************************************/

#ifdef WIN32GUI
/*
 * entry point if compiled as BORLAND windows app.
 */

# define MAX_COMMAND_ARGS        64

static void *hInstance = (void *)0,
	    *hPrevInstance = (void *)0;


WINAPI
WinMain(hInst, hPrevInst, lpCmdLine, nCmdShow)
    HINSTANCE hInst;
    HINSTANCE hPrevInst;
    char *lpCmdLine;
    int nCmdShow;
{
    static char *argv[MAX_COMMAND_ARGS];
    static char moduleFileName[MAX_PATH+1];
    int argc;
    char *cp, *tp;
    int quote = 0;
    int i;

    GetModuleFileName(NULL,moduleFileName,MAX_PATH);
    argv[0] = moduleFileName;
    argc = 1;

    cp = lpCmdLine;
    if (cp) {
	CMDDPRINTF(("WinMain: lpCmdLine \'%s\'\n", cp));
	while  (*cp && ((*cp == ' ') || (*cp == '\t'))) cp++;

	if (*cp == '\"') {
	    quote = 1;
	    cp++;
	}

	while (*cp && (argc < MAX_COMMAND_ARGS)) {
	    tp = cp;
	    if (quote) {
		while (*tp && (*tp != '\"'))
		    tp++;
		quote = 0;
	    } else {
		while (*tp && (*tp != ' '))
		    tp++;
	    }
	    if (*tp)
		*tp++ = '\0';

	    if (argc >= MAX_COMMAND_ARGS) {
		printf("WinMain: too many arguments\n");
		exit(1);
	    }
	    CMDDPRINTF(("WinMain: arg %d: %s\n", argc, cp));

	    argv[argc++] = cp;
	    cp = tp;

	    while (*cp && ((*cp == ' ') || (*cp == '\t'))) cp++;
	    if (*cp == '\"') {
		quote = 1;
		cp++;
	    }
	}
    }

    for (i=argc; i<MAX_COMMAND_ARGS; i++) {
	argv[i] = (char *)0;
    }

    hInstance = (void *) hInst;
    hPrevInstance = (void *) hPrevInst;

    __hasNoConsole__ = 1;

    SDPRINTF(("WinMain: calling main...\n"));
    CMDDPRINTF(("argc=%d\n", argc));

    return(main(argc, argv));
}
#endif /* WIN32GUI */

/*
 * your main; you can add all kind of
 * private setup here ....
 */

#ifdef WIN32
static int __argc;
static char **__argv;
static void winProc(void *);
static VOIDFUNC messageHandler = (VOIDFUNC)0;
unsigned long __beginthreadNT(void (*start_address)(void *),
			      unsigned  stack_size,
			      void *arglist,
			      void *security_attr,
			      unsigned long create_flags,
			      unsigned long *thread_id);

int
main(argc, argv)
    char *argv[];
{
#ifdef WIN32GUI
	HANDLE handle;
	DWORD threadId;
	DWORD res;
	SECURITY_ATTRIBUTES saDefault = {
		sizeof(SECURITY_ATTRIBUTES),    // sizeof Structure
		0,                              // no security descriptor
		TRUE,                           // canInherit handle of Thread
	};

	SDPRINTF(("main: begin main thread.\n"));

	__argc = argc;
	__argv = argv;

	handle = CreateThread(&saDefault,
				   0x40000, // Stacksize
				   (LPTHREAD_START_ROUTINE)winProc,
				   NULL,
				   0,       // create in active state
				   &threadId);

	if (handle == INVALID_HANDLE_VALUE) {
# if 0
	    __stxLog__(0, STX_LOG_ERROR, "VM", "threadNTCreate fails in main() with %d\n", GetLastError());
# endif
	    return  -1;
	}

	res = WaitForSingleObject(handle, INFINITE);

	GetExitCodeThread(handle, &res);
	SDPRINTF(("main: main thread finished.\n"));
	return res;
#else /* not WIN32GUI */
	__argc = argc;
	__argv = argv;

	SDPRINTF(("main: calling winProc()\n"));
	winProc(NULL);
	SDPRINTF(("main: after winProc()\n"));
	/* not reached */
	return 0;
#endif
}

# ifdef __VISUALC__

/*
 * entry point for VISUAL-C console app.
 */
int
_tmain(int argc, char* argv[])
{
    return main(argc, argv);
}

# endif

static void
winProc(dummy)
    void *dummy;
{
	int argc;
	char **argv;

	SDPRINTF(("WinProc\n"));

#if 0
#ifdef WIN32
	{
	    int i;

	    /* look for a -L argument, and set the logFile if presenet */
	    for (i=0; __argv[i]; i++) {
	        if (strcmp(__argv[i], "-L") == 0) {
		    if (__argv[i+1] && (strlen(__argv[i+1]) > 0)) {
		        __win32_setLogFile(__argv[i+1], 1);
		    }
	        }
	    }
	}
#endif /* WIN32 */
#endif /* 0 */

#ifdef WIN_LOGFILE
	__win32_setLogFile(WIN_LOGFILE, 0);
#endif
#if 0
	{
	    WSADATA wsaData;
	    WSAStartup(0x202, &wsaData);
	    SDPRINTF(("after WSAStartup\n"));
	}
#endif

	/*
	 * no, I want to handle errors myself
	 */
	SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX | SEM_NOGPFAULTERRORBOX);
	SDPRINTF(("after SetErrorMode\n"));

	argc = __argc;
	argv = __argv;

#else /* not WIN32 */

main(argc, argv)
    char *argv[];
{
#endif /* WIN32 */

	CMDDPRINTF(("argc = %d\n", argc));
	CMDDPRINTF(("argv[0] = %s\n", argv[0]));
	CMDDPRINTF(("argv[1] = %s\n", argv[1]));
	CMDDPRINTF(("argv[2] = %s\n", argv[2]));

#ifdef NODEBUG
	{
	    /*
	     * debuggability initially off (for standAlone apps)
	     * no debugger will come up - instead an error message is printed,
	     * and the program terminates.
	     */
	    __setDebuggerAllowed(1);
	}
#endif
#ifdef NOINFO
	{
	    /*
	     * infoPrinting initially off (for standAlone apps)
	     */
	    __setInfoPrinting(0);
	}
#endif
#ifdef IGNORE_IMAGE
	{
	    /*
	     * force ignore of any image;
	     * useful for standAlone-app's executable
	     */
	    __setIgnoreImage(1);
	}
#endif
#ifdef DIRECT_START
	{
	    /*
	     * force direct start (without initializing other classes)
	     * experts only: the startup message MUST then initialize the system
	     * manually (as done in Smalltalk>>start)
	     */
	    __setDirectStartStandAlone(1);
	}
#endif
#ifdef NO_DISPLAY
	{
	    /*
	     * tell smalltalk that no display connection is wanted
	     */
	    __setNoDisplay(1);
	}
#endif
#ifdef IRIS
	if (__iris_workaround__) {
	    /* dummy code to reference these variables.
	     * Some time ago needed for IRIS
	     */
	    printf("sel=(%s) rsel=(%s)\n", STARTUP_SELECTOR, RESTART_SELECTOR);
	    printf(STARTUP_SELECTOR); printf("\n");
	    printf(RESTART_SELECTOR); printf("\n");
	}
#endif

#ifdef THIS_IS_AN_EXAMPLE_ONLY
	/*
	 * register custom init-hooks & curstom functions
	 */
	__stxRegisterCustomExitHook__(customExitHook);
	__stxRegisterCustomPreInitHook__(customPreInitHook);
	__stxRegisterCustomPostInitHook__(customPostInitHook);
	__stxRegisterCustomPostReInitHook__(customPostReInitHook);
	__stxRegisterCustomInterruptHooks__(enterCustomInterruptHook,
					    leaveCustomInterruptHook);
	__stxRegisterCustomFunctions__(customFunctions);
#endif

	SDPRINTF(("main: calling stxMain.\n"));

	__stxMain__(STARTUP_CLASS, STARTUP_SELECTOR, RESTART_SELECTOR,
		    __modules__, CONFIGURATION_STRING, BUILD_DATE,
		    argc, argv,
#ifdef WIN32
		    GetModuleHandle(0),
#else
		    0,
#endif
		    __hasNoConsole__
		   );
	/* never reached */
}

/*********************************************************
 * custom hooks
 *********************************************************/

/*
 * called right before doing an exit() when smalltalk is shut down
 * if registered with __stxRegisterExitHook__()
 */
static void
customExitHook(exitCode) { }

/*
 * called after VM initialization;
 * if registered with __stxRegisterPreInitHook__();
 * at this time all globals have been setup, but classes have not yet
 * performed their #initialize methods.
 * Shortly after this, ST/X will #initialize all classes.
 * This is not called for image-restart.
 */
static void
customPreInitHook() { }

/*
 * called after initialization;
 * if registered with __stxRegisterPostInitHook__();
 * at this time, all classes have performed their #initialize methods.
 * Shortly after this, Smalltalk takes over in Smalltalk>>start.
 * This is not called for image-restart.
 */
static void
customPostInitHook() { }

/*
 * called after initialization in case of an image restart.
 * if registered with __stxRegisterPostReInitHook__();
 * After this, Smalltalk takes over in Smalltalk>>restart.
 * This is not called for normal (non-image)-start.
 */
static void
customPostReInitHook() { }

/*********************************************************
 * hooks for custom interrupt/signal handling
 * this allows triggering an extra smalltalk interrupt from
 * custom user code.
 *********************************************************/

/*
 * how things work:
 *
 * triggering a custom interrupt:
 *
 *      __stxRegisterCustomInterrupt__();
 *
 * to handle it immediately (only needed, if it can occur during a
 * blocking system call (such as read/write, open ...)
 *
 *      __stxHandleInterrupt__()
 *
 * this can be done in your signal handler - for example.
 *
 *
 * ST/X will (as soon as possible) handle the interrupt, by doing
 * the following steps:
 * (if the Hooks were registered)
 *
 *      call:   __stxEnterCustomInterruptHook();
 *      send:   (ObjectMemory customSignalHandler) customSignal
 *      call:   __stxLeaveCustomSignalHook();
 *
 * with the above scheme, you can either handle these interrupts in
 * C (by defining an stxEnterCustomInterrupt()-function and leaving
 * ObjectMemory:CustomSignalHandler as nil),
 * or handle it on the Smalltalk level, by leaving the C-functions empty
 * and setting ObjectMemory:CustomSignalHandler to a smalltalk handler object.
 * Even a combination is possible.
 */

static void
enterCustomInterruptHook() {}

static void
leaveCustomInterruptHook() {}


#ifdef THIS_IS_AN_EXAMPLE_ONLY

/*********************************************************
 * hooks for special user functions
 *
 * -- this is still being prepared --
 *
 *********************************************************/

/*
 * simple user defined functions
 *
 * this allows calling c-functions from st-code WITHOUT a need
 * to compile primitive code
 * (i.e. if you do not want to use, or cannot use the stc-compiler).
 *
 * the following table defines a set of c-functions, which can be called
 * via
 *    ExternalFunction callCustomFunction:nr
 *    ExternalFunction callCustomFunction:nr with:arg
 *    ExternalFunction callCustomFunction:nr with:arg1 with:arg2
 *    ExternalFunction callCustomFunction:nr with:arg1 with:arg2 with:arg3
 *
 * the arguments are simply passed down to the customFunction in a vector,
 * together with the number of arguments.
 * The custom function has to extract those arguments from the vector itself.
 *
 * Every customFunction should return 1 on success, 0 on failure (will
 * trigger a primitiveFailure in smalltalk.
 * A return value can be put into the cell pointed to by retRef; if left unassigned,
 * self is returned from the callCustomFunction smalltalk method.
 * See examples below.
 */

#define PRIM_OK         1
#define PRIM_FAIL       0

/*
 * exampleFunction 0; to be called with:
 *
 * ExternalFunction callCustomFunction:0
 */
static int
demoFunction0(nargs, retRef, argVector)
    OBJ *retRef;
    OBJ argVector[];
{
    /*
     * dont care for what was passed
     */
    printf("demofunction0 called\n");
    return PRIM_OK;
}

/*
 * exampleFunction 1; to be called with:
 *
 * ExternalFunction callCustomFunction:0 with:aString
 */
static int
demoFunction1(nargs, retRef, argVector)
    OBJ *retRef;
    OBJ argVector[];
{
    /*
     * check for 1 arg, which must be a string
     */
    if (nargs == 1) {
	OBJ arg = argVector[0];

	if (__isStringLike(arg)) {
	    char *stringArg = (char *) __stringVal(arg);

	    printf("demofunction1 called with: %s\n", stringArg);
	    return PRIM_OK;
	}
    }
    /*
     * primitive failure
     */
    return PRIM_FAIL;
}

/*
 * exampleFunction 2; to be called with:
 *
 * ExternalFunction callCustomFunction:2 with:float1 with:float2
 */
static int
demoFunction2(nargs, retRef, argVector)
    OBJ *retRef;
    OBJ argVector[];
{
    /*
     * check for 2 args; both must be floats
     */
    if (nargs == 2) {
	OBJ arg1 = argVector[0];
	OBJ arg2 = argVector[1];

	if (__isFloat(arg1) && __isFloat(arg2)) {
	    double floatArg1 = __floatVal(arg1);
	    double floatArg2 = __floatVal(arg2);

	    printf("demofunction2 called with: %f and: %f; returning %f\n",
			floatArg1, floatArg2, floatArg1 + floatArg2);
	    *retRef = _MKFLOAT(floatArg1 + floatArg2);
	    return PRIM_OK;
	}
    }
    /*
     * primitive failure
     */
    return PRIM_FAIL;
}

/*
 * add more functions here (or include a file with your functions)
 */

CUSTOMFUNCTION customFunctions[] = {
    { demoFunction0, "demoFunction0" },
    { demoFunction1, "demoFunction1" },
    { demoFunction2, "demoFunction2" },
    /*
     * add entries here ..
     */

    /*
     * leave the end-mark below
     */
    { 0, "" }
};

#endif /* THIS_IS_AN_EXAMPLE_ONLY */
