//
// npshell.cpp - Plug-in methods called from Netscape.
//

#include <windows.h>
#include <string.h>
#include <stdio.h>

#ifndef _MSC_VER
#define _MSC_VER
#include "npwin.c"
#endif

#include "..\win32Common\debug.h"
#include "..\win32Common\stxWin32Plugin.c"
static char cvsID[] = "$Header: /cvs/stx/stx/librun/plugin/mozilla/NPStx.c,v 1.7 2005/12/23 15:01:12 cg Exp $";

HANDLE hPluginModule = NULL;

BOOL APIENTRY
DllMain( HANDLE hModule,
	 DWORD  ul_reason_for_call,
	 LPVOID lpReserved)
{
    if(!hPluginModule)
	hPluginModule = hModule;

    switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
	    DEBUG("DllMain - DLL_PROCESS_ATTACH\n");
	    break;

	case DLL_THREAD_ATTACH:
	    DEBUG("DllMain - DLL_THREAD_ATTACH\n");
	    break;

	case DLL_THREAD_DETACH:
	    DEBUG("DllMain - DLL_THREAD_DETACH\n");
	    break;

	case DLL_PROCESS_DETACH:
	    DEBUG("DllMain - DLL_PROCESS_DETACH\n");
	    break;
    }
    return TRUE;
}

//
// NPP_New - Create a new plug-in instance.
//
NPError NP_LOADDS
NPP_New(NPMIMEType pluginType,
	NPP pInstance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
    StxPlugin *stx;
    int i;

    DEBUG("NPP_New\n");

    if(pInstance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    stx = StxPluginNew(hPluginModule, pInstance);
    if(!stx)
	return NPERR_OUT_OF_MEMORY_ERROR;

    if(strncmp(NPN_UserAgent(pInstance), "Microsoft Internet Explorer", 27) == 0)
	StxPluginSetIEMode();

    for(i=0;i<argc;i++) {
	if(stricmp(argn[i],"imageName") == 0) {
	    StxPluginSetImage(stx, argv[i]);
	}
	if(stricmp(argn[i],"vmName") == 0) {
	    StxPluginSetVM(stx, argv[i]);
	}
	if(stricmp(argn[i],"memory") == 0) {
	    StxPluginSetMemory(stx, argv[i]);
	}
	if(stricmp(argn[i],"win32Params") == 0) {
	    StxPluginSetVMParams(stx, argv[i]);
	}
	/* Pass all other args directly */
	StxPluginAddParam(stx, argn[i], argv[i]);
    }

    pInstance->pdata = (void *)stx;

    return NPERR_NO_ERROR;
}

//
// NPP_Destroy - Destroy our plug-in instance.
//
NPError NP_LOADDS
NPP_Destroy (NPP pInstance, NPSavedData** save)
{
    StxPlugin *stx;

    if(pInstance == NULL)
	    return NPERR_INVALID_INSTANCE_ERROR;

    stx = (StxPlugin*) pInstance->pdata;
    StxPluginDestroy(stx);
    pInstance->pdata = NULL;
    return NPERR_NO_ERROR;
}

//
// NPP_SetWindow - A window was created, resized, or destroyed.
//
NPError NP_LOADDS
NPP_SetWindow (NPP pInstance, NPWindow* pNPWindow)
{
    StxPlugin *stx;
    HWND hWnd;

    if(pInstance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    if(pNPWindow == NULL)
	return NPERR_GENERIC_ERROR;

    hWnd = (HWND)(DWORD)pNPWindow->window;
    stx = (StxPlugin*) pInstance->pdata;
    if(stx == NULL)
	return NPERR_GENERIC_ERROR;

    if(hWnd == NULL) // spurious entry
	return NPERR_NO_ERROR;

    if(StxPluginActive(stx)) {
	StxPluginResize(stx, hWnd);
    } else {
	StxPluginRun(stx, hWnd, 1);
    }
    return NPERR_NO_ERROR;
}

//
// NPP_NewStream - A new stream was created.
//
NPError NP_LOADDS
NPP_NewStream(NPP pInstance,
	      NPMIMEType type,
	      NPStream* stream,
	      NPBool seekable,
	      uint16* stype)
{
    StxPlugin *stx = (StxPlugin*) pInstance->pdata;

    if(stx) {
	  *stype = NP_ASFILE;
    }
    return NPERR_NO_ERROR;
}


//
// NPP_WriteReady - Returns amount of data we can handle for the next NPP_Write
//
int32 NP_LOADDS
NPP_WriteReady (NPP pInstance, NPStream *stream)
{
    if(pInstance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;
    return 0x0FFFFFFF;
}


//
// NPP_Write - Here is some data. Return -1 to abort stream.
//
int32 NP_LOADDS
NPP_Write (NPP pInstance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
    return len;
}

//
// NPP_DestroyStream - Stream is done, but audio may still be playing.
//
NPError NP_LOADDS
NPP_DestroyStream (NPP pInstance, NPStream *stream, NPError reason)
{
    StxPlugin *stx = (StxPlugin*) pInstance->pdata;

    if(stx) {
	StxPluginStreamState(stx,(char*)  stream->url, reason == NPRES_DONE);
    }
    return NPERR_NO_ERROR;
}

/**** Unused stubs ***/
NPError
NPP_Initialize(void)
{
    DEBUG("NPP_Initialize\n");
    StxPluginInitialize();
    return NPERR_NO_ERROR;
}


void
NPP_Shutdown(void)
{
    DEBUG("NPP_Shutdown\n");
}

jref NP_LOADDS
NPP_GetJavaClass (void)
{
    return NULL;
}

void NP_LOADDS
NPP_StreamAsFile (NPP pInstance, NPStream* stream, const char* fname)
{
    StxPlugin *stx = (StxPlugin*) pInstance->pdata;
    if(stx) {
	  StxPluginStreamFile(stx, (char*) stream->url, (char*)fname);
    }
}


void NP_LOADDS
NPP_Print (NPP pInstance, NPPrint* printInfo)
{
}

void NP_LOADDS
NPP_URLNotify(NPP pInstance, const char* url, NPReason reason, void* notifyData)
{
    StxPlugin *stx = (StxPlugin*) pInstance->pdata;
    if(stx) {
	  StxPluginNotify(stx, (int)notifyData, (char*) url, reason == NPRES_DONE);
    }
}

#ifdef __cplusplus
extern "C" {
#endif

extern int ieMode;

#ifdef __cplusplus
}
#endif

void StxPluginRequestStream(void *instance, char *url, char *target, int id)
{
    NPError err;

    if(ieMode)
	err = NPN_GetURL((NPP)instance, url, target);
    else
	err = NPN_GetURLNotify((NPP)instance, url, target, (void*) id);
}

void StxPluginPostData(void *instance, char *url, char *target, char *data, int id)
{
    NPError err;
    if(ieMode)
	err = NPN_PostURL((NPP)instance, url, target, data ? strlen(data) : 0, data, 0);
    else
	err = NPN_PostURLNotify((NPP)instance, url, target, data ? strlen(data) : 0, data, 0, (void*) id);
}
