/****************************************************************************
* ST/X plugin for Windows.
*
* Based upon sqWin32Plugin (thanks to Andreas Raab)
*
* Comment in original File:
*   PROJECT: Squeak port for Win32 (NT / Win95)
*   FILE:    sqWin32Plugin.c
*   CONTENT: Generic Squeak Plugin
*
*   AUTHOR:  Andreas Raab (ar)
*   ADDRESS: University of Magdeburg, Germany
*   EMAIL:   raab@isg.cs.uni-magdeburg.de
*   RCSID:   $Id: stxWin32Plugin.c,v 1.6 2006/01/09 14:03:08 sr Exp $
*
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "stxWin32Plugin.h"

static char stxWin32Plugin_ID[] = "$Header: /cvs/stx/stx/librun/plugin/win32Common/stxWin32Plugin.c,v 1.6 2006/01/09 14:03:08 sr Exp $";

const char* gInstanceLookupString = "hWnd->stx";

/* What's the root registry key ?! */
#ifdef SQUEAK
# ifndef HKEY_SQUEAK_ROOT
#  define HKEY_SQUEAK_ROOT "SOFTWARE\\Squeak"
# endif
# define HKEY_ROOT HKEY_SQUEAK_ROOT
#else /* not SQUEAK */
# define HKEY_ROOT  "SOFTWARE\\eXept\\Smalltalk/X\\plugin"
# define HKEY_ROOT2 "SOFTWARE\\eXept\\Smalltalk/X"
#endif

/* are we running IE?! */
int ieMode = 0;

DWORD g_WM_START_SESSION = 0;
DWORD g_WM_QUIT_SESSION = 0;
DWORD g_WM_BWND_SIZE = 0;
DWORD g_WM_REQUEST_DATA = 0;
DWORD g_WM_POST_DATA = 0;
DWORD g_WM_RECEIVE_DATA = 0;
DWORD g_WM_INVALIDATE = 0;

DWORD g_WM_BROWSER_PIPE = 0;
DWORD g_WM_CLIENT_PIPE = 0;

HANDLE hCurrentProcess = NULL;

#define FREE(ptr) if(ptr) { free(ptr); ptr = NULL; }
#define STRDUP(ptr) ((ptr) ? strdup(ptr) : NULL)

/********************************************************************/
/* Initialization stuff                                             */
/********************************************************************/

void
StxPluginInitialize(void)
{
    DEBUG("NPStx [info]: StxPluginInitialize\n");

    DuplicateHandle(
		  GetCurrentProcess(),
		  GetCurrentProcess(),
		  GetCurrentProcess(),
		  &hCurrentProcess,
		  1,
		  0,
		  DUPLICATE_SAME_ACCESS);

    g_WM_START_SESSION = RegisterWindowMessage("StxStartSession");
    g_WM_QUIT_SESSION = RegisterWindowMessage("StxQuitSession");
    g_WM_BWND_SIZE = RegisterWindowMessage("StxSetBrowserWindowSize");
    g_WM_REQUEST_DATA = RegisterWindowMessage("StxRequestData");
    g_WM_POST_DATA = RegisterWindowMessage("StxPostData");
    g_WM_RECEIVE_DATA = RegisterWindowMessage("StxReceiveData");
    g_WM_INVALIDATE = RegisterWindowMessage("StxInvalidateRect");
    g_WM_BROWSER_PIPE = RegisterWindowMessage("StxBrowserPipe");
    g_WM_CLIENT_PIPE = RegisterWindowMessage("StxClientPipe");
}

void
StxPluginSetIEMode(void)
{
    ieMode = 1;
}

/********************************************************************/
/* Window stuff                                                     */
/********************************************************************/

LRESULT CALLBACK
PluginWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    StxPlugin* stx = (StxPlugin*) GetProp(hWnd, gInstanceLookupString);

    if(!stx) {
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    if(uMsg == g_WM_CLIENT_PIPE) {
	HANDLE hProcess;
	HANDLE hPipe;

	DEBUG("NPStx [info]: received g_WM_CLIENT_PIPE\n");
	hProcess = (HANDLE) stx->processInfo.hProcess;
	hPipe = (HANDLE) lParam;
	if(!DuplicateHandle(hProcess, hPipe, GetCurrentProcess(),
			    &stx->hClientPipe, 0, FALSE,
			    DUPLICATE_SAME_ACCESS)) {
	    MessageBox(stx->hWnd, "Can not establish communication channel!",
		       "Stx Plugin Error:", MB_OK | MB_ICONSTOP);
	}
	return 1;
    }

    if(uMsg == g_WM_REQUEST_DATA || uMsg == g_WM_POST_DATA) {
	char *url, *target, *data;
	DWORD urlSize, targetSize, dataSize;
	DWORD nBytes;

	DEBUG("NPStx [info]: received g_WM_REQUEST_DATA/g_WM_POST_DATA\n");
	/* Read the URL from the pipe */
	ReadFile(stx->hParentReadEnd, &urlSize, 4, &nBytes, NULL);
	if(urlSize > 0) {
	    url = (char *) malloc(urlSize+1);
	    ReadFile(stx->hParentReadEnd, url, urlSize, &nBytes, NULL);
	    url[urlSize] = 0;
	} else
	    url = NULL;
	/* Read the target from the pipe */
	ReadFile(stx->hParentReadEnd, &targetSize, 4, &nBytes, NULL);
	if(targetSize > 0) {
	    target = (char *) malloc(targetSize+1);
	    ReadFile(stx->hParentReadEnd, target, targetSize, &nBytes, NULL);
	    target[targetSize] = 0;
	} else
	    target = NULL;

	if(uMsg == g_WM_POST_DATA) {
	    /* Read the data from the pipe */
	    ReadFile(stx->hParentReadEnd, &dataSize, 4, &nBytes, NULL);
	    if(dataSize > 0) {
		data = (char *) malloc(dataSize+1);
		ReadFile(stx->hParentReadEnd, data, dataSize, &nBytes, NULL);
		data[dataSize] = 0;
	    } else
		data = NULL;
	} else data = NULL;

	if(stx->maxStreams < MAX_STREAMS) {
	    /* Request the stream from the browser */
	    stx->maxStreams++;
	    if(ieMode) {/* See comments 1), 3), and 4) on top */
		int i;
		NULL_TEST(url,"PluginWindowProc");
		for(i = 0; i < MAX_STREAMS; i++)
		    if(!stx->requests[i].url) {
			stx->requests[i].url = STRDUP(url);
			stx->requests[i].id = wParam;
			break;
		    }
	    }
	    if(uMsg == g_WM_REQUEST_DATA)
		StxPluginRequestStream(stx->cbData, url, target, wParam);
	    else
		StxPluginPostData(stx->cbData, url, target, data, wParam);
	    if(url) free(url);
	    if(target) free(target);
	    if(data) free(data);
	} else {
	    /* No more space; just abort */
	    if(url) free(url);
	    if(target) free(target);
	    if(data) free(data);
	    DEBUG("NPStx [info]: sending g_WM_RECEIVE_DATA\n");
	    PostThreadMessage(stx->processInfo.dwThreadId,
			      g_WM_RECEIVE_DATA, wParam, 0);
	}
	return 1;
    }

    if(uMsg == WM_PAINT) {
	RECT r;
#ifdef NOT_NEEDED
	if(GetUpdateRect(hWnd, &r, FALSE)) {
	    DEBUG("NPStx [info]: sending g_WM_INVALIDATE\n");
	    PostThreadMessage(stx->processInfo.dwThreadId,
			      g_WM_INVALIDATE, 0, 0);
	}
#endif /* NOT_NEEDED */
	/* FALL THROUGH */
    }
    /* and pass control to the previous window procedure */
    return CallWindowProc((int (__stdcall *)())stx->prevWindowProc, hWnd, uMsg, wParam, lParam);
}


void
UnsubclassWindow(HWND hWnd, StxPlugin *stx)
{
    if(!stx->hWnd) return;
    SetWindowLong(stx->hWnd, GWL_WNDPROC, (DWORD) stx->prevWindowProc);
    stx->hWnd = NULL;
    stx->prevWindowProc = NULL;
    SetProp( hWnd, gInstanceLookupString, (HANDLE)NULL);
}

void
SubclassWindow(HWND hWnd, StxPlugin *stx)
{
    /* subclass the window */
    UnsubclassWindow(hWnd, stx);
    stx->hWnd = hWnd;
    stx->prevWindowProc = (WNDPROC) GetWindowLong(hWnd, GWL_WNDPROC);
    SetWindowLong(hWnd, GWL_WNDPROC, (DWORD) PluginWindowProc);
    SetProp( hWnd, gInstanceLookupString, (HANDLE)stx);
    /* make the window clip its children or else we might have serious
       trouble due to synchronization problems. Remember, Stx is
       running in its own thread so if the window draws atop Stx
       after Stx has drawn itself we'll get damagy looking stuff.
       And believe me, this *does* happen.
    */
    SetWindowLong(hWnd, GWL_STYLE, GetWindowLong(hWnd, GWL_STYLE) | WS_CLIPCHILDREN);
}

/********************************************************************/
/* Startup stuff                                                    */
/********************************************************************/
void
ReadRegistrySettings(HMODULE hPluginModule, StxPlugin *stx) {
    DWORD dwType, dwSize, ok1, ok2;
    TCHAR tmp[MAX_PATH+1];
    HKEY hk = (HKEY)0;

    ok1 = ok2 = RegOpenKey(HKEY_CURRENT_USER, HKEY_ROOT, &hk);
    if (ok1 != ERROR_SUCCESS) {
	ok1 = ok2 = RegOpenKey(HKEY_LOCAL_MACHINE, HKEY_ROOT, &hk);
#ifdef HKEY_ROOT2
	if (ok1 != ERROR_SUCCESS) {
	    ok1 = ok2 = RegOpenKey(HKEY_CURRENT_USER, HKEY_ROOT2, &hk);
	    if (ok1 != ERROR_SUCCESS) {
		ok1 = ok2 = RegOpenKey(HKEY_LOCAL_MACHINE, HKEY_ROOT2, &hk);
	    }
	}
#endif
    }

    // fetch BinDir for VM-directory
    if (ok1 == ERROR_SUCCESS) {
	/* Read the Stx path from the subkey. */
	dwSize = MAX_PATH;
	ok2 = RegQueryValueEx(hk,"BinDir",NULL, &dwType, (LPBYTE) tmp, &dwSize);
    }

    if (ok2 == ERROR_SUCCESS) {
	if(tmp[dwSize-2] != '\\') {
	    tmp[dwSize-1] = '\\';
	    tmp[dwSize] = 0;
	}
    } else {
	/* No base directory found; use stx subdir in plugin location */
	GetModuleFileName(hPluginModule, tmp, MAX_PATH);
	/* MessageBox(0, stx->vmName,"Module name", MB_OK); */
	strrchr(tmp,'\\')[1] = 0;
#ifndef SQUEAK
	strcat(tmp, "stx\\");
#endif
    }
    strcpy(stx->binDir, tmp);
    strcpy(stx->vmName, tmp);

    // fetch LibDir for Image-directory
    if (ok1 == ERROR_SUCCESS) {
	/* Read the Stx path from the subkey. */
	dwSize = MAX_PATH;
	ok2 = RegQueryValueEx(hk,"LibDir",NULL, &dwType, (LPBYTE) tmp, &dwSize);
    }

    if (ok2 == ERROR_SUCCESS) {
	if(tmp[dwSize-2] != '\\') {
	    tmp[dwSize-1] = '\\';
	    tmp[dwSize] = 0;
	}
    } else {
	/* No base directory found; use stx subdir in plugin location */
	GetModuleFileName(hPluginModule, tmp, MAX_PATH);
	/* MessageBox(0, stx->vmName,"Module name", MB_OK); */
	strrchr(tmp,'\\')[1] = 0;
#ifndef SQUEAK
	strcat(tmp, "stx\\");
#endif
    }
    strcpy(stx->libDir, tmp);
    strcpy(stx->imageName, tmp);

    /* Read VM name from the subkey */
    if (ok1 == ERROR_SUCCESS) {
	dwSize = MAX_PATH;
	ok2 = RegQueryValueEx(hk, "VM", NULL, &dwType, (LPBYTE)tmp, &dwSize);
    }
    if (ok2 == ERROR_SUCCESS) {
	tmp[dwSize] = 0;
	strcat(stx->vmName, tmp);
    } else {
	/* No VM name found; use default VM name */
#ifdef SQUEAK
	strcat(stx->vmName, "Squeak.exe");
#else
	strcat(stx->vmName, "winstx.exe");
#endif
    }

    /* Read VM args from the subkey */
    if (ok1 == ERROR_SUCCESS) {
	dwSize = MAX_PATH;
	ok2 = RegQueryValueEx(hk, "VMArgs", NULL, &dwType, (LPBYTE)tmp, &dwSize);
    }
    if (ok2 == ERROR_SUCCESS) {
	tmp[dwSize] = 0;
	stx->vmParams = STRDUP(tmp);
    } else {
	/* No VM params found */
	stx->vmParams = STRDUP("-I --quick");
    }

    /* Read image name from the subkey */
    if (ok1 == ERROR_SUCCESS) {
	dwSize = MAX_PATH;
	ok2 = RegQueryValueEx(hk, "Image", NULL, &dwType, (LPBYTE)tmp, &dwSize);
    }
    if (ok2 == ERROR_SUCCESS) {
	tmp[dwSize] = 0;
	strcat(stx->imageName, tmp);
    } else {
	/* No image name found; use default image name */
#ifdef SQUEAK
      strcat(stx->imageName, "SqueakPlugin.image");
#else
      strcat(stx->imageName, "stxPlugin.img");
#endif
    }
    if (ok1 == ERROR_SUCCESS) {
      RegCloseKey(hk);
    }
}

StxPlugin *
StxPluginNew(HANDLE hPluginModule, void *cbData)
{
    StxPlugin *stx;
    int i;

    stx = (StxPlugin*) calloc(1,sizeof(StxPlugin));
    if (stx) {
	stx->params = (char *) calloc(1,10); /* initial zero terminated string */
	/* Default settings */
	ReadRegistrySettings(hPluginModule, stx);
//        strcpy(stx->params,"\"\" ");
	stx->cbData = cbData;
	if(!CreatePipe(&stx->hParentReadEnd, &stx->hParentWriteEnd, NULL, 4096))
	    return NULL;

	for(i=0; i< MAX_STREAMS; i++) {
	    stx->requests[i].id = -1;
	    stx->requests[i].state = -1;
	}
    }

    DEBUG("NPStx [info]: stx->binDir: "); DEBUG(stx->binDir); DEBUG("\n");
    DEBUG("NPStx [info]: stx->libDir: "); DEBUG(stx->libDir); DEBUG("\n");
    DEBUG("NPStx [info]: stx->vmName: "); DEBUG(stx->vmName); DEBUG("\n");
    DEBUG("NPStx [info]: stx->vmArgs: "); DEBUG(stx->vmParams); DEBUG("\n");
    DEBUG("NPStx [info]: stx->imageName: "); DEBUG(stx->imageName); DEBUG("\n");

    return stx;
}

int
StxPluginActive(StxPlugin *stx)
{
    if(!stx) return 0;
    return stx->processInfo.hProcess != NULL;
}

int
TerminateStxPlugin(StxPlugin *stx)
{
    DWORD exitCode;
    int maxStop = 50; /* Wait at most five seconds to complete */
    if(!stx) return 0;
    if(!stx->processInfo.hProcess) return 0;

    UnsubclassWindow(stx->hWnd, stx);
    /* Gracefully terminate the process */
    do {
	GetExitCodeProcess(stx->processInfo.hProcess, &exitCode);
	if(exitCode == STILL_ACTIVE) {
	    DEBUG("NPStx [info]: sending g_WM_QUIT_SESSION\n");
	    PostThreadMessage(stx->processInfo.dwThreadId, g_WM_QUIT_SESSION, 0, 0);
	    Sleep(100);
	}
    } while(exitCode == STILL_ACTIVE && maxStop-- > 0);
    if(maxStop <= 0) {
	/* Sorry folks - we have to do it the hard way */
	DEBUG("NPStx [info]: terminating the process\n");
	TerminateProcess(stx->processInfo.hProcess,0);
    }
    stx->processInfo.hProcess = NULL;
    return 1;
}

int
StxPluginRun(StxPlugin *stx, HWND hWnd, int terminateOld)
{
    STARTUPINFO sInfo;
    char lbuf[50];
    char *runString;
    int ok, runSize;
    int repeatCount;

    if(!stx) return 0;
    if(!hWnd) return 0;

    if(terminateOld)
	TerminateStxPlugin(stx);

    if(stx->vmName[0] == 0) return 0;
    if(stx->imageName[0] == 0) return 0;

    SubclassWindow(hWnd, stx);

    /* Start Stx */
    ZeroMemory(&sInfo, sizeof(sInfo));
    sInfo.cb = sizeof(sInfo);
    /* setup cmd line */
    runSize = 256; /* some additional space for -browserWindow: etc */
    runSize += strlen(stx->vmName) + 2;
    runSize += strlen(stx->imageName) + 2;
    if(stx->params) runSize += strlen(stx->params);
    if(stx->vmParams) runSize += strlen(stx->vmParams);
    runString = (char *) calloc(1, runSize+1);

    strcat(runString,"\"");
    strcat(runString, stx->vmName);
    strcat(runString,"\"");

    if (stx->vmParams) {
	strcat(runString," ");
	strcat(runString, stx->vmParams);
    }

    if (stx->memory) {
	strcat(runString," -Mold:");
	strcat(runString, ltoa(stx->memory * 1024, lbuf, 10) );
    }

    strcat(runString," -i ");
    strcat(runString,"\"");
    strcat(runString, stx->imageName);
    strcat(runString,"\"");

    strcat(runString, " --browserWindow: ");
    strcat(runString, ltoa((int)hWnd, lbuf, 10) );

    if (stx->params) {
	strcat(runString," --browserParameters: ");
	strcat(runString, stx->params);
    }

    ok = CreateProcess(
		  NULL, /* no image name */
		  runString, /* but full command line */
		  NULL, /* no process security attributes */
		  NULL, /* no thread security attributes */
		  TRUE, /* don't inherit handles */
		  0, /* no special flags */
		  NULL, /* no separate environment */
		  NULL, /* no startup dir */
		  &sInfo, /* startup info */
		  &stx->processInfo); /* process info */

    if (ok == 0) {
	/* Error! */
	DWORD lastError = GetLastError();
	LPVOID lpMsgBuf;
	char msgBuf2[8192];
	char msgBuf3[8192];

	UnsubclassWindow(stx->hWnd, stx);
	stx->processInfo.hProcess = NULL;

	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
		      FORMAT_MESSAGE_FROM_SYSTEM |
		      FORMAT_MESSAGE_IGNORE_INSERTS,
		      NULL,
		      lastError,
		      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		      (LPTSTR) &lpMsgBuf,
		      0,
		      NULL );

	sprintf(msgBuf2, (char *) lpMsgBuf, runString);
	sprintf(msgBuf3, "%s \n\nCode = %x\nCommand:\n%s", msgBuf2, lastError, runString);
	MessageBox(hWnd, msgBuf3, "Stx Plugin Error: Cannot start stx", MB_OK | MB_ICONSTOP);
	free(runString);
	LocalFree( lpMsgBuf );
	return 0;
    }
    WaitForInputIdle(stx->processInfo.hProcess, INFINITE);
    if(!DuplicateHandle(
			GetCurrentProcess(),
			stx->hParentWriteEnd,
			stx->processInfo.hProcess,
			&stx->hParentWriteEnd,
			0,
			FALSE,
			DUPLICATE_SAME_ACCESS | DUPLICATE_CLOSE_SOURCE)) {
	MessageBox(hWnd, "Unable to establish communication channel",
			 "Stx Plugin error", MB_OK | MB_ICONSTOP);
	free(runString);
	return 0;
    }

    DEBUG("NPStx [info]: created process\n");
    repeatCount = 100;
    DEBUG("NPStx [info]: sending g_WM_START_SESSION\n");
    while ((repeatCount > 0) &&
	   (PostThreadMessage(stx->processInfo.dwThreadId, g_WM_START_SESSION, (WPARAM)0, (LPARAM)hWnd) == 0)) {
	DEBUG("NPStx [info]: waiting for ST/X to setup its message queue...\n");
	Sleep(10);
    }
    if (repeatCount == 0) {
	MessageBox(hWnd, "Unable to establish communication channel",
			 "Stx Plugin error", MB_OK | MB_ICONSTOP);
	free(runString);
	return 0;
    }

    WaitForInputIdle(stx->processInfo.hProcess, INFINITE);

    DEBUG("NPStx [info]: sending g_WM_BROWSER_PIPE\n");
    PostThreadMessage(
		  stx->processInfo.dwThreadId,
		  g_WM_BROWSER_PIPE,
		  (WPARAM) hCurrentProcess,
		  (LPARAM) stx->hParentWriteEnd);
    /* MessageBox(0,runString,"Starting Stx",MB_OK); */

    free(runString);
    return 1;
}


int
StxPluginResize(StxPlugin *stx, HWND hWnd)
{
    RECT r;

    if(!stx) return 0;
    if(!stx->processInfo.hProcess) return 0;
    if(!hWnd) return 0;
    if(GetWindowLong(hWnd, GWL_WNDPROC) != (int)PluginWindowProc) {
	SubclassWindow(hWnd, stx);
    }
    GetClientRect(hWnd, &r);
// #ifdef NOT_NEEDED
    DEBUG("NPStx [info]: sending g_WM_BWND_SIZE\n");
    PostThreadMessage(stx->processInfo.dwThreadId,
		      g_WM_BWND_SIZE, (r.right - r.left), (r.bottom - r.top));
// #endif /* NOT_NEEDED */
    return 1;
}


int
StxPluginDestroy(StxPlugin *stx)
{
    if(!stx) return 1;
    TerminateStxPlugin(stx);
    if(stx->vmParams) free(stx->vmParams);
    if(stx->params) free(stx->params);
    free(stx);
    return 1;
}

void
StxPluginSetVM(StxPlugin *stx, char *vmName)
{
#ifndef DISNEY
    strcpy(stx->vmName, vmName);
#endif
}

void
StxPluginSetImage(StxPlugin *stx, char *imageName)
{
#ifndef DISNEY
    strcpy(stx->imageName, imageName);
#endif
}

void
StxPluginSetMemory(StxPlugin *stx, char *src)
{
    char buf[50], *tmp;
    int imageSize, requestedSize;
    int baseSize;
    HANDLE hFile;

    hFile = CreateFile(stx->imageName,
		       GENERIC_READ,
		       FILE_SHARE_READ | FILE_SHARE_WRITE,
		       NULL,
		       OPEN_EXISTING,
		       FILE_ATTRIBUTE_NORMAL,
		       NULL);
    if(hFile) {
	baseSize = (int) SetFilePointer(hFile, 0, NULL, FILE_END);
	CloseHandle(hFile);
    } else {
	baseSize = 0;
    }
    imageSize = 0;
    while(*src) {
	switch(*src) {
	    case ' ': /* white spaces; ignore */
	    case '"':
		src++; break;
	    case '*': /* multiple of image size */
		tmp = buf; src++;
		while(*src && isdigit(*src))
		    *(tmp++) = *(src++); /* integer part */
		if(*src == '.') { /* fraction part */
		    *(tmp++) = *(src++);
		    while(*src && isdigit(*src)) *(tmp++) = *(src++);
		}
		*(tmp++) = 0;
		imageSize += (int) (baseSize * atof(buf));
		break;
	    case '+': /* additional space in bytes */
		tmp = buf; src++;
		while(*src && isdigit(*src))
		    *(tmp++) = *(src++);
		*(tmp++) = 0;
		if (imageSize == 0)
		    imageSize = baseSize;
		requestedSize = atoi(buf);
		imageSize += (requestedSize <= 1000) ? requestedSize*1024*1024 : requestedSize;
		break;
	    default: /* absolute size */
		tmp = buf;
		*(tmp++) = *(src++);
		while(*src && isdigit(*src))
		    *(tmp++) = *(src++);
		*(tmp++) = 0;
		requestedSize = atoi(buf);
		imageSize = (requestedSize <= 1000) ? requestedSize*1024*1024 : requestedSize;
	}
    }
    stx->memory = imageSize / (1024 * 1024);
}
void
StxPluginAddParam(StxPlugin *stx, char *name, char *value)
{
    if(name && value) {
	stx->paramSize += strlen(name) + 3; /* quotes + space */
	stx->paramSize += strlen(value) + 3; /* quotes + space */
	stx->params = (char *) realloc(stx->params, stx->paramSize + 10);
	strcat(stx->params,"\"");
	strcat(stx->params, name);
	strcat(stx->params,"\" \"");
	strcat(stx->params, value);
	strcat(stx->params,"\" ");
    }
}

void
StxPluginSetVMParams(StxPlugin *stx, char *params)
{
    int length;

    if(!params || !*params) return;
    if(stx->vmParams) free(stx->vmParams);
    /* skip double quotes if necessary */
    if(params[0] == '"') {
	params++;
	length = strlen(params);
	if (params[length-1] == '"') params[length-1] = '\0';
    }
    stx->vmParams = STRDUP(params);
}

/********************************************************************/
/* Streaming stuff                                                  */
/********************************************************************/
void
StxReturnRequest(StxPlugin *stx, int index)
{
  char *localName = stx->requests[index].localName;
  DWORD length, written;

  /* This makes the VM aware of data coming in */
  DEBUG("NPStx [info]: sending g_WM_RECEIVE_DATA\n");
  PostThreadMessage(
		stx->processInfo.dwThreadId,
		g_WM_RECEIVE_DATA,
		stx->requests[index].id,
		stx->requests[index].state);

  if(localName)
    length = strlen(localName);
  else
    length = 0;
  WriteFile(stx->hClientPipe, &length, 4, &written, NULL);
  if(length)
    WriteFile(stx->hClientPipe, localName, length, &written, NULL);

  FREE(stx->requests[index].url);
  FREE(stx->requests[index].localName);
  stx->requests[index].id = -1;
  stx->requests[index].state = -1;
  stx->maxStreams--;
}

void
StxPluginStreamFile(StxPlugin *stx, char *url, char *localName)
{
  int i;

  if(!stx->maxStreams) return; /* no streams requested */
  NULL_TEST(url,"PluginStreamFile(url)");
  NULL_TEST(localName, url);
  if(ieMode) {
    /* See comments on top */
    for(i=0; i < MAX_STREAMS; i++) {
      if(stx->requests[i].url && stx->requests[i].id >= 0) {
	if(stricmp(stx->requests[i].url, url) == 0) {
	  stx->requests[i].localName = STRDUP(localName);
	  StxReturnRequest(stx, i);
	  return;
	}
      }
    }
  } else { /* !ieMode */
    /* Cache the URL and the local name */
    for(i = 0; i < MAX_STREAMS; i++)
      if(!stx->requests[i].url) {
	stx->requests[i].url = STRDUP(url);
	stx->requests[i].localName = STRDUP(localName);
	return;
      }
  }
}

/* NOTE: The following is never used iff ieMode != 0 */
void
StxPluginNotify(StxPlugin *stx, int id, char *url, int ok)
{
  int i;

  if(!stx->maxStreams) return; /* no streams requested */
  NULL_TEST(url,"PluginNotify");
  /* See comments 1), 2), and 4) on top */
  for(i = 0; i < MAX_STREAMS; i++) {
    if(stx->requests[i].url && stricmp(stx->requests[i].url, url) == 0) {
      stx->requests[i].id = id;
      stx->requests[i].state = ok;
      StxReturnRequest(stx, i);
      return;
    }
  }
  /* Note: If we come here, NS has never created a stream for the request.
     Handle it as if it were an empty request */
  for(i = 0; i < MAX_STREAMS; i++) {
    if(!stx->requests[i].url) {
      stx->requests[i].url = STRDUP(url);
      stx->requests[i].id = id;
      stx->requests[i].state = ok;
      if(!ok) StxReturnRequest(stx, i);
      return;
    }
  }
}

void
StxPluginStreamState(StxPlugin *stx, char *url, int ok)
{
  int i;

  if(!stx->maxStreams) return; /* no streams requested */
  NULL_TEST(url,"PluginStreamState");
  for(i = 0; i < MAX_STREAMS; i++) {
    if(stx->requests[i].url && stricmp(stx->requests[i].url, url) == 0) {
      stx->requests[i].state = ok;
      if(stx->requests[i].id >= 0) StxReturnRequest(stx, i);
      return;
    }
  }
}
