"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libsnmp' }"

Object subclass:#SNMPSession
	instanceVariableNames:'sessionHandle authenticate lastError responseTimeout'
	classVariableNames:'Lobby DefaultResponseTimeout SessionLock MIBInitialized'
	poolDictionaries:''
	category:'Net-Communication-SNMP'
!

!SNMPSession primitiveDefinitions!
%{


#ifndef __sgi
# include <sys/types.h>
#endif
#include <stdio.h>
#include <netinet/in.h>
#include <sys/time.h>
#include "snmp.h"
#include "mib.h"
#include "asn1.h"
#include "snmp_impl.h"
#include "snmp_api.h"
#include "snmp_client.h"
#include "party.h"
#include "context.h"
#include "view.h"
#include "acl.h"
#ifdef _AIX
# include <sys/select.h>
#endif

#define _snmp_sessionVal(o)  (struct snmp_session *)__externalAddressVal(o)

%}
! !

!SNMPSession primitiveVariables!
%{
int snmp_dump_packet = 0;
%}
! !

!SNMPSession primitiveFunctions!
%{

static
freeSessionData(session)
    struct snmp_session *session;
{
    if (session->srcPartyLen) {
	free(session->srcParty);
	session->srcPartyLen = 0;
	session->srcParty = (oid *)0;
    }
    if (session->dstPartyLen) {
	free(session->dstParty);
	session->dstPartyLen = 0;
	session->dstParty = (oid *)0;
    }
    if (session->contextLen) {
	free(session->context);
	session->contextLen = 0;
	session->context = (oid *)0;
    }
    if (session->peername) {
	free(session->peername);
	session->peername = (char *)0;
    }
}

%}

! !

!SNMPSession class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    interface to libsnmp. Notice, that libsnmp is not too well suited
    as a reusable library: it contains exit() and error printfs all over.
    I removed the exits, and replaced them by returns in most places,
    but still have the printfs in there. These ought to be replaced by
    symbolic error return codes, for a more convenient error report ....

    Since this is for demonstration purposes only, I leave that as an
    excercise to you ...

    For asynchronous get requests, the code from snmp_sync_response was
    splitted and distributed among multiple methods, performing a semaphore
    wait on the file descriptors in between.
    Since (due to the used snmp code) only one request can be outstanding at
    a time, a semaphore is used to interlock multiple sessions.
    Consider this a kludge - the snmp interface should be changed to make proper
    use of asynchronous callbacks .... (another excercise)

    [author:]
	Claus Gittinger

    [see also:]
	Socket

    [start with:]
	(see examples)
	SNMPSession netWalk
	SNMPSession mibDump
"
!

examples
"
  OID translation ...
                                                        [exBegin]
    Transcript showCR:(self rootOID)
                                                        [exEnd]

                                                        [exBegin]
   Transcript showCR:(session OIDFromString:'system.sysDescr.0')
                                                        [exEnd]

   This asks for host/community and dumps the host's MIB:
                                                        [exBegin]
   SNMPSession mibDump
                                                        [exEnd]

   Monitor some (builtin) hosts:
                                                        [exBegin]
   SNMPSession monitor
                                                        [exEnd]


  send a getRequest for system.sysDescr.0 to local host:
                                                        [exBegin]
    |session oId|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    (session open) ifTrue:[
        [
            oId := (self OIDFromString:'system.sysDescr.0').
            oId notNil ifTrue:[
                Transcript showCR:(session getRequest:oId)
            ]
        ] ensure:[
            session close.
        ].
    ]
                                                        [exEnd]


  getRequest for system.sysDescr.0 to local host; show returned value:
                                                        [exBegin]
    |session|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    (session open) ifTrue:[
        [
            Transcript showCR:(session getVariable:'system.sysDescr.0')
        ] ensure:[
            session close.
        ].
    ].
                                                        [exEnd]


  getRequest for system.sysDescr.0 to local host (asynchronous);
  show returned value / error reason:
                                                        [exBegin]
    |session val|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            val := (session asyncGetVariable:'system.sysDescr.0').
            val isNil ifTrue:[
                Transcript showCR:('error:' , session lastError printString)
            ] ifFalse:[
                Transcript showCR:val
            ]
        ] ensure:[
            session close.
        ].
    ].
                                                        [exEnd]


  getRequest for system.sysDescr.0 to some hosts (synchronous);
  show returned value:
                                                        [exBegin]
    |session valString val|

    #('exept' 'ibm' 'exept-brick') do:[:hostname |
        session := SNMPSession new.
        session version:1.
        session community:'public'.
        session peerName:hostname.
        session open ifTrue:[
            [
                valString := session getVariable:'system.sysDescr.0'.
                valString notNil ifTrue:[
                    val := session extractASN1ValueFromString:valString.
                    Transcript showCR:(hostname , ' is: ' , val printString).
                ]
            ] ensure:[
                session close.
            ].
        ].
    ].
                                                        [exEnd]


  same, using simpler interface (synchronous):
                                                        [exBegin]
    |session val type unit amount|

    #('exept' 'ibm' 'exept-brick') do:[:hostname |
        session := SNMPSession new.
        session version:1.
        session community:'public'.
        session peerName:hostname.
        session open ifTrue:[
            [
                val := session getValue:'system.sysDescr.0'.
                val notNil ifTrue:[
                    Transcript showCR:(hostname , ' is: ' , val printString).
                ]
            ] ensure:[
                session close.
            ].
        ].
    ].
                                                        [exEnd]



  same, using asynchronous transfer:
                                                        [exBegin]
    |session val type unit amount|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            val := session asyncGetValue:'system.sysDescr.0'.
            val notNil ifTrue:[
                Transcript showCR:('localhost is: ' , val printString).
            ]
        ] ensure:[
            session close.
        ].
    ].
                                                        [exEnd]


  get next value after 'system':
                                                        [exBegin]
    |session pair|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            pair := session getNextRequest:'system'.
            pair key isNil ifTrue:[
                Transcript showCR:(session lastError printString)
            ] ifFalse:[
                Transcript showCR:pair.
            ]
        ] ensure:[
            session close.
        ].
    ].
                                                        [exEnd]


  walk through the MIB:
                                                        [exBegin]
    |session|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            session
                keysAndValuesDo:[:oId :val |
                    Transcript show:(session stringFromOID:oId); show:' = '; showCR:val.
                ]
        ] ensure:[
            session close.
        ]
    ].
                                                        [exEnd]

  walk through part of the MIB (everything below 'tcp'):
                                                        [exBegin]
    |session|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            session
                keysAndValuesBelow:(self OIDFromString:'tcp')
                do:[:oId :val |
                    Transcript show:(self stringFromOID:oId); show:' = '; showCR:val.
                ]
        ] ensure:[
            session close.
        ].
    ].
                                                        [exEnd]



  walk through part of the MIB (everything after 'tcp'):
                                                        [exBegin]
    |session|

    session := SNMPSession new.
    session version:1.
    session community:'public'.
    session peerName:'localhost'.
    session open ifTrue:[
        [
            session
                keysAndValuesAfter:(self OIDFromString:'tcp')
                do:[:oId :val |
                    Transcript show:(self stringFromOID:oId); show:' = '; showCR:val.
                ]
        ] ensure:[
            session close.
        ]
    ].
                                                        [exEnd]



"
! !

!SNMPSession class methodsFor:'instance creation'!

new
    "return a new (unconnected) SNMPSession"

    ^ (self basicNew) initialize
! !

!SNMPSession class methodsFor:'class initialization'!

deinitialize
    Lobby notNil ifTrue:[
	Lobby do:[:aSession |
	    aSession close
	].
    ]
!

initMib
    "init the mib (in libsnmp)"

%{  /* STACK: 32000 */
    init_mib();
%}

    "
        self initMib
    "
!

initialize
    SessionLock := Semaphore forMutualExclusion name:'sessionLock'.
    DefaultResponseTimeout := 5000   "/ millis
! !

!SNMPSession class methodsFor:'debugging'!

verbose:aBoolean
%{
    snmp_dump_packet = (aBoolean == true) ? 1 : 0;
%}
! !

!SNMPSession class methodsFor:'more examples'!

mibDump
    "dump a mib into a window"

    |host community session text view retrievedLines|

    host := Dialog request:'hostname:'.
    host isEmpty ifTrue:[^ self].
    community := Dialog request:'community:' initialAnswer:'public'.
    community isEmpty ifTrue:[^ self].

    session := SNMPSession new.
    session version:1.
    session community:community.
    session peerName:host.
    session open ifTrue:[
        [
            text := StringCollection new.
            view := EditTextView openWith:nil title:('mibDump of ' , host).
            view waitUntilVisible.
            retrievedLines := 0.
            Processor activeProcess withPriority:7 do:[
                session
                    keysAndValuesDo:[:oId :val |
                        text add:( (self stringFromOID:oId) printString , ' = ' , val displayString ).
                        retrievedLines := retrievedLines + 1.
                        retrievedLines > 50 ifTrue:[
                            view setList:text.
                            retrievedLines := 0.
                        ].
                    ].
                    view setList:text.
            ].
        ] ensure:[
            session close.
        ]
    ] ifFalse:[
        self information:'no snmp response'
    ]

    "
     SNMPSession mibDump
    "

    "Modified: 26.6.1997 / 16:21:29 / cg"
!

monitor
    "monitor a number of hosts"

    |list top hosts monitorProcess|

    hosts := #('dellprinter'    'public'     
               'exept-brick'    'public'
               'smc1'           'public'
               'exeptn'         'public').

    top := StandardSystemView new.
    top label:'SNMP monitor'.

    list := ScrollableView for:ListView in:top.
    list origin:0.0@0.0 corner:1.0@1.0.

    top openAndWait.

    monitorProcess := [
        |l first|

        first := true.
        [top realized] whileTrue:[
            l := OrderedCollection new.
            l add:'                      name users       system'.
            l add:'--------------------------------------------------------------------'.

            hosts pairWiseDo:[:hostname :community|
                |session sysDescr sysName nUsers|

                top realized ifFalse:[monitorProcess terminate].

                top windowGroup withCursor:Cursor wait do:[
                    top label:'SNMP monitor - trying ' , hostname , ' ...'.

                    session := SNMPSession new.
                    session version:1; community:community.
                    session peerName:hostname.
                    session responseTimeout:1000.
                    session open ifTrue:[
                        [
                            sysDescr := session asyncGetValue:'system.sysDescr.0'.
                            sysDescr isNil ifTrue:[
                                sysDescr := '** no SNMP response **'.
                                sysName := ''.
                                nUsers := ''.
                            ] ifFalse:[
                                sysName := session asyncGetValue:'system.sysName.0'.
                                sysName isNil ifTrue:[
                                    sysName := ''
                                ].
                                nUsers := session asyncGetValue:'host.hrSystem.hrSystemNumUsers.0'.
                                nUsers isNil ifTrue:[
                                    nUsers := '  ?'
                                ] ifFalse:[
                                    nUsers := nUsers printString
                                ].
                            ].

                            l add:(
                                    (hostname paddedTo:15 with:Character space)
                                    , ' ' , (sysName leftPaddedTo:10 with:Character space)
                                    , ' ' , (nUsers leftPaddedTo:3 with:Character space)
                                    , '   ' , sysDescr).
                        ] ensure:[
                            session close.
                        ].
                    ].
                    top label:'SNMP monitor'.
                    first ifTrue:[list list:l].
                ].
            ].
            first := false.
            list list:l.
            (Delay forSeconds:10) wait.
        ].
    ] forkAt:(Processor userBackgroundPriority).
    monitorProcess name:'snmp monitor update'.

    "
     SNMPSession monitor
    "
!

netWalk
    "find snmp-reachable hosts in the local network
     (only works with class C networks - needs rewrite for others).
     Uses 'public' community."

    |myAddress list top walkProcess response|

    OperatingSystem getNetworkAddresses detect:[:eachAssoc|
        myAddress := eachAssoc value.
        myAddress isLocal not.
    ].

    myAddress := myAddress hostAddress.

    top := StandardSystemView new.
    top label:'SNMP net walk'.

    list := ScrollableView for:ListView in:top.
    list horizontalScrollable:true.
    list horizontalMini:true.
    list origin:0.0@0.0 corner:1.0@1.0.

    top openAndWait.

    walkProcess := [
        |l low hi direction tryHostID dotHostname hostname session sysDescr
         addr sendBuffer rcvBuffer echoSock n hostString pingable snmpable|

        l := SortedCollection new.

        "/ only works with type C-net

        "/ the code below could simply do 1 to:254 do:[:hostID }
        "/ but, to probe likely hosts earlier, the probing is done
        "/ ping-pong like around my ip-address (assuming, that other machines
        "/ have numbers around my own).

        low := hi := (myAddress at:4).
        direction := 1.

        [low > 0 or:[hi < 255]] whileTrue:[
            direction > 0 ifTrue:[
                hi := hi + 1.
                tryHostID := hi.
                direction := -1.
            ] ifFalse:[
                low := low - 1.
                tryHostID := low.
                direction := 1.
            ].

            (tryHostID between:1 and:254) ifTrue:[
                dotHostname := (myAddress at:1) printString
                        , '.' , (myAddress at:2) printString
                        , '.' , (myAddress at:3) printString
                        , '.' , tryHostID printString.

                top label:'SNMP net walk - trying ' , dotHostname.

                hostname := Socket hostWithIpAddress:(ByteArray
                                                        with:(myAddress at:1)
                                                        with:(myAddress at:2)
                                                        with:(myAddress at:3)
                                                        with:tryHostID).

                hostname ~= dotHostname ifTrue:[
                    hostString := (('[' , dotHostname , ']') paddedTo:18)
                                  ,  (hostname paddedTo:15)
                ] ifFalse:[
                    hostString := (('[' , dotHostname , ']') paddedTo:18)
                                  , (String new:15)
                ].

                top withWaitCursorDo:[
                    response := nil.

                    "/ try a ping first.
                    pingable := false.
                    sendBuffer := (1 to:10) asByteArray.
                    rcvBuffer := ByteArray new:10.
                    addr := IPSocketAddress
                                hostName:dotHostname serviceName:'echo' type:#datagram.
                    echoSock := Socket newUDP:nil.
                    echoSock sendTo:addr buffer:sendBuffer.
                    (echoSock readWaitWithTimeout:1) ifFalse:[
                        "/ got something
                        n := echoSock receiveFrom:(IPSocketAddress new) buffer:rcvBuffer.
                        n ~~ 0 ifTrue:[
                            response := hostString , ' pingable - but no SNMP response'.
                            pingable := true.
                        ].
                    ].
                    echoSock close.

                    snmpable := false.
                    session := SNMPSession new.
                    session responseTimeout:(pingable ifTrue:[1000] ifFalse:[500]).
                    session version:1; community:'public'.
                    session peerName:dotHostname.
                    [
                        session open.
                        sysDescr := session asyncGetVariable:'system.sysDescr.0'.
                        sysDescr notNil ifTrue:[
                            response := hostString , ' ' , sysDescr.
                            snmpable := true.
                        ].
                    ] ensure:[
                        session close.
                    ].

                    response notNil ifTrue:[
                        l add:response.
                        list list:l.
                    ].

                    "/ if more than one SNMP process is running,
                    "/ give others a chance to get through
                    Processor yield.
                ].
            ].
        ].
        top label:'SNMP reachable hosts'.
    ] forkAt:(Processor userBackgroundPriority).
    walkProcess name:'snmp net walker'.

    "
     SNMPSession netWalk
    "

    "Modified: 26.6.1997 / 14:12:03 / cg"
! !

!SNMPSession class methodsFor:'queries'!

OIDFromString:aString
    |resultOID|

%{  /* STACK: 20000 */
    char *s;
    oid o[MAX_NAME_LEN];
    int olen = 0;

    if (! __isString(aString)) {
        RETURN (nil);
    }
    s = __stringVal(aString);

    olen = MAX_NAME_LEN;
/* printf("read_objid (%s)\n", s);  */

    if (!read_objid(s, o, &olen)){
        printf("OIDFromString: Invalid oid: %s\n", s);
        olen = 0;
    }
    if (olen) {
        resultOID = __NEW_INT(@global(SNMPOID), olen * sizeof(oid));
        bcopy(o, __ByteArrayInstPtr(resultOID)->ba_element, olen * sizeof(oid));
    }
%}.
    ^ resultOID
!

contextOIDFromString:aString
    |resultOID|

%{  /* STACK: 20000 */
    struct contextEntry *cxp;
    char *s;
    oid o[MAX_NAME_LEN];
    int olen = 0;

    if (! __isString(aString)) {
        RETURN (nil);
    }
    s = __stringVal(aString);

    context_scanInit();
    for(cxp = context_scanNext(); cxp; cxp = context_scanNext()){
        if (!strcasecmp(cxp->contextName, s)){
            olen = cxp->contextIdentityLen;
            bcopy(cxp->contextIdentity, o, olen * sizeof(oid));
            break;
        }
    }
    if (!cxp){
        olen = MAX_NAME_LEN;
        if (!read_objid(s, o, &olen)){
            printf("Invalid context: %s\n", s);
            olen = 0;
        }
    }
    if (olen) {
        resultOID = __NEW_INT(@global(SNMPOID), olen * sizeof(oid));
        bcopy(o, __ByteArrayInstPtr(resultOID)->ba_element, olen * sizeof(oid));
    }
%}.
    ^ resultOID
!

extractASN1ValueFromString:aValueString
    (aValueString startsWith:$") ifTrue:[
        ^ aValueString copyFrom:2 to:(aValueString lastIndexOf:$")-1
    ].
    (aValueString startsWith:'OID:') ifTrue:[
        ^ self OIDFromString:(aValueString copyFrom:5).
    ].
    (aValueString startsWith:'Gauge:') ifTrue:[
        ^ Number readFromString:(aValueString copyFrom:7) onError:0.
    ].
    (aValueString at:1) isDigit ifTrue:[
        ^ Number readFromString:aValueString onError:0
    ].
    (aValueString endsWith:$)) ifTrue:[
        ^ (aValueString copyFrom:1 to:(aValueString lastIndexOf:$( )-1)
                withoutSpaces asSymbol
    ].
    ^ aValueString
!

partyOIDFromString:aString
    |resultOID|

%{  /* STACK: 20000 */
    struct partyEntry *pp;
    char *s;
    oid o[MAX_NAME_LEN];
    int olen = 0;

    if (! __isString(aString)) {
        RETURN (nil);
    }
    s = __stringVal(aString);

    party_scanInit();
    for(pp = party_scanNext(); pp; pp = party_scanNext()){
        if (!strcasecmp(pp->partyName, s)){
            olen = pp->partyIdentityLen;
            bcopy(pp->partyIdentity, o, olen * sizeof(oid));
            break;
        }
    }
    if (!pp){
        olen = MAX_NAME_LEN;
        if (!read_objid(s, o, &olen)){
            printf("Invalid party: %s\n", s);
            olen = 0;
        }
    }
    if (olen) {
        resultOID = __NEW_INT(@global(SNMPOID), olen * sizeof(oid));
        bcopy(o, __ByteArrayInstPtr(resultOID)->ba_element, olen * sizeof(oid));
    }
%}.
    ^ resultOID
!

rootOID
    ^ self OIDFromString:'.1.3.6.1.2.1'

    "
     SNMPSession rootOID
    "
!

stringFromOID:id

%{  /* STACK: 20000 */
    oid o[MAX_NAME_LEN];
    int l, olen = 0;
    char *bytes;
    char buf[2048];

    if (__Class(id) == @global(SNMPOID)) {
        bytes = __ByteArrayInstPtr(id)->ba_element;
        l = __byteArraySize(id);
        olen = l / sizeof(oid);
        bcopy(bytes, o, l);

        buf[0] = '\0';
        sprint_objid(buf, o, olen);
        RETURN (__MKSTRING(buf));
    }
%}.
    ^ nil
! !

!SNMPSession methodsFor:'accessing'!

lastError
    ^ lastError
!

responseTimeout:millis
    responseTimeout := millis.
! !

!SNMPSession methodsFor:'asynchronous (non blocking) requests'!

asyncGetNextRequest:nameOrOID
    "get the next value as a printed string (for now).
     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    ^ self asyncGetRequest:nameOrOID next:true
!

asyncGetRequest:nameOrOID
    "get a value as a printed string (for now).
     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    ^ self asyncGetRequest:nameOrOID next:false
!

asyncGetRequest:nameOrOID next:next
    |result sema oid|

    SessionLock critical:[
        nameOrOID isString ifTrue:[
            oid := self class OIDFromString:nameOrOID.
"/            'name: ' print. nameOrOID print. ' OID: ' print. oid printCR.
        ].

        (self sendGetRequest:nameOrOID next:next) ifFalse:[^ nil].
        sema := Semaphore new name:'getRequest'.
        Processor signal:sema afterMilliseconds:responseTimeout.

        [self responseArrived] whileFalse:[
           result := self waitForResponsesWithTimeout:50.
"/           (result == #timeout) ifTrue:[
"/               lastError := #timeout.
"/               Processor disableSemaphore:sema.
"/               ^ nil
"/           ].
           sema wouldBlock ifFalse:[
               lastError := #timeout.
               ^ nil
           ].
        ].
        Processor disableSemaphore:sema.
        ^ self getResponseData.
    ]

    "Modified: 26.6.1997 / 14:08:06 / cg"
!

asyncGetValue:aNameString
    |valString val|

    valString := self asyncGetVariable:aNameString.
    valString notNil ifTrue:[
        val := self class extractASN1ValueFromString:valString.
    ].
    ^ val
!

asyncGetVariable:nameOrOID
    ^ (self asyncGetRequest:nameOrOID) value
! !

!SNMPSession methodsFor:'enumeration'!

keysAndValuesAfter:initialOID do:aBlock
    "evaluate aBlock for all of the mibs values.
     The first argument to the block is the oid, the second
     the value.
     This is almost the same as the snmpwalk unix command ..."

    |oId pair val|

    initialOID isString ifTrue:[
        oId := self class OIDFromString:initialOID
    ] ifFalse:[
        oId := initialOID.
    ].

    [oId notNil] whileTrue:[
        pair := self getNextRequest:oId.
        pair notNil ifTrue:[
            oId := pair key.
            oId notNil ifTrue:[
                val := pair value.
                aBlock value:oId value:(self class extractASN1ValueFromString:val)
            ]
        ]
    ]

    "Modified: 26.6.1997 / 15:48:27 / cg"
!

keysAndValuesBelow:initialOID do:aBlock
    "evaluate aBlock for all of the mibs values.
     The first argument to the block is the oid, the second
     the value.
     This is almost the same as the snmpwalk unix command ..."

    |oId pair val|

    initialOID isString ifTrue:[
        oId := self class OIDFromString:initialOID
    ] ifFalse:[
        oId := initialOID.
    ].

"/ oId printCR.

    [oId notNil] whileTrue:[
        pair := self getNextRequest:oId.
        pair notNil ifTrue:[
            oId := pair key.
            oId notNil ifTrue:[
                (oId startsWith:initialOID) ifTrue:[
                    val := pair value.
                    aBlock value:oId value:(self class extractASN1ValueFromString:val)
                ]
            ]
        ]
    ]

    "Modified: 26.6.1997 / 15:49:29 / cg"
!

keysAndValuesDo:aBlock
    "evaluate aBlock for all of the mibs values.
     The first argument to the block is the oid, the second
     the value.
     This is almost the same as the snmpwalk unix command ..."

    self keysAndValuesAfter:self class rootOID do:aBlock
! !

!SNMPSession methodsFor:'initialization'!

community:communityString
    "setup the sessions community parameter;
     the snmpVersion must be 1 in order for this to be allowed"

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    char *community;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (session->version == SNMP_VERSION_1) {
	/*
	 * version 1
	 */
	if (__isString(communityString)) {
	    int l;

	    community = __stringVal(communityString);
	    l = strlen(community);

	    session->community = (char *) malloc(l + 1);
	    strcpy(session->community, community);
	    session->community_len = l;
	    RETURN (true);
	}
    }
%}
!

context:contextOID
    "setup the context session parameter;
     the snmpVersion argument must be 2 in order for this to be allowed"

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    oid *con;
    int contextlen, l;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (session->version == SNMP_VERSION_2) {
	if (__Class(contextOID) != @global(SNMPOID)) {
	    con = (oid *)0;
	    contextlen = 0;
	} else {
	    l = __byteArraySize(contextOID);
	    contextlen = l / sizeof(oid);
	    con = (oid *) malloc(l);
	    bcopy(__ByteArrayInstPtr(contextOID)->ba_element, con, l);
	}

	session->context = con;
	session->contextLen = contextlen;
	RETURN (true);
    }
%}.
    ^ false
!

destinationParty:dstOID
    "setup the destination session parameter;
     the snmpVersion argument must be 2 in order for this to be allowed"

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    oid *dst;
    int dstlen, l;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (session->version == SNMP_VERSION_2) {
	if (__Class(dstOID) != @global(SNMPOID)) {
	    dst = (oid *)0;
	    dstlen = 0;
	} else {
	    l = __byteArraySize(dstOID);
	    dstlen = l / sizeof(oid);
	    dst = (oid *) malloc(l);
	    bcopy(__ByteArrayInstPtr(dstOID)->ba_element, dst, l);
	}

	session->dstParty = dst;
	session->dstPartyLen = dstlen;
	RETURN (true);
    }
%}.
    ^ false
!

initialize
    "create and initialize the underlying snmp session structure"

    responseTimeout := DefaultResponseTimeout.
    MIBInitialized ~~ true ifTrue:[
	self class initMib.
	MIBInitialized := true
    ].

%{
    struct snmp_session *session;
    OBJ sH;

    session = (struct snmp_session *)(malloc(sizeof(struct snmp_session)));
    if (session) {
	memset((char *)session, 0, sizeof(struct snmp_session));
	session->version = SNMP_VERSION_2;
	session->retries = SNMP_DEFAULT_RETRIES;
	if (__isSmallInteger(_INST(responseTimeout))) {
	    session->timeout = __intVal(_INST(responseTimeout)) * 100;
	} else {
	    session->timeout = 2000000L;
	}
	session->authenticator = NULL;

	sH = __MKEXTERNALADDRESS(session); _INST(sessionHandle) = sH; __STORE(self, sH);

	RETURN (self);
    }
%}.
    self primitiveFailed
!

peerName:aHostnameString
    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    char *hostname;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (__isString(aHostnameString)) {
	hostname = __stringVal(aHostnameString);

	session->peername = (char *) malloc(strlen(hostname) + 1);
	strcpy(session->peername, hostname);
	RETURN (true);
    }
%}
!

sourceParty:srcOID
    "setup the sourceParty session parameter;
     the snmpVersion argument must be 2 in order for this to be allowed"

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    oid *src;
    int srclen, l;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (session->version == SNMP_VERSION_2){
	if (__Class(srcOID) != @global(SNMPOID)) {
	    src = (oid *)0;
	    srclen = 0;
	} else {
	    l = __byteArraySize(srcOID);
	    srclen = l / sizeof(oid);
	    src = (oid *) malloc(l);
	    bcopy(__ByteArrayInstPtr(srcOID)->ba_element, src, l);
	}

	session->srcParty = src;
	session->srcPartyLen = srclen;
	RETURN (true);
    }
%}.
    ^ false
!

version:snmpVersion
    "setup the sessions parameters;
     the snmpVersion argument must be 1 or 2.
     The default is 2, if never set explicitely."

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{
    char *community;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (snmpVersion == __MKSMALLINT(1)) {
	session->version = SNMP_VERSION_1;
	RETURN (true);
    }
    if (snmpVersion == __MKSMALLINT(2)) {
	session->version = SNMP_VERSION_2;
	RETURN (true);
    }
%}.
    ^ false
! !

!SNMPSession methodsFor:'nonblocking request helpers'!

getResponseData
    "extract the last responses data"

    |retVarOID retVal|

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{  /* STACK: 20000 */
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));
    struct snmp_pdu *response;
    extern struct synch_state snmp_synch_state;
    struct synch_state *state = &snmp_synch_state;

    int l, status;
    char *bytes;
    struct variable_list *vars;
    char stringBuffer[2048*4];

    if (state->waiting) {
	RETURN (nil);
    }
    response = state->pdu;
    if (! response) {
	RETURN (nil);
    }

    status = state->status;

    switch (status) {
	case STAT_SUCCESS:
	    switch (response->errstat) {
		case SNMP_ERR_NOERROR:
		    for(vars = response->variables; vars; vars = vars->next_variable){
			switch (vars->type) {
			    case SNMP_NOSUCHOBJECT:
				_INST(lastError) = @symbol(noSuchObject);
				break;

			    case SNMP_NOSUCHINSTANCE:
				_INST(lastError) = @symbol(noSuchInstance);
				break;

			    case SNMP_ENDOFMIBVIEW:
				_INST(lastError) = @symbol(endOfMib);
				break;

			    default:
/*
				print_variable(vars->name, vars->name_length, vars);
 */
				retVarOID = __NEW_INT(@global(SNMPOID), vars->name_length * sizeof(oid));
				bcopy(vars->name, __ByteArrayInstPtr(retVarOID)->ba_element, vars->name_length * sizeof(oid));

				sprint_value(stringBuffer, vars->name, vars->name_length, vars);
				retVal = __MKSTRING(stringBuffer);
				break;
			}
		    }
		    break;

		case SNMP_ERR_NOSUCHNAME:
		    _INST(lastError) = @symbol(noSuchName);
		    printf("SNMP [info]: no name [SNMP_ERR_NOSUCHNAME]\n");
		    break;

		default:
		    _INST(lastError) = @symbol(otherError);
		    printf("SNMP [info]: Error in packet.\nReason: %s\n",
					    snmp_errstring(response->errstat));
		    break;
	    }
	    break;

	case STAT_TIMEOUT:
	    _INST(lastError) = @symbol(noResponse);
	    printf("SNMP [info]: No Response from %s [STAT_TIMEOUT]\n", session->peername);
	    break;

	default:
	    _INST(lastError) = @symbol(otherError);
	    printf("SNMP [info]: An error occurred\n");
	    break;
    }

    snmp_free_pdu(response);
%}.

    ^ retVarOID -> retVal
!

readResponseFrom:fd
    "read a response;
     to avoid a blocking read in the snmp-lib,
     the fileDescriptor should have been selected on before."

%{  /* STACK: 20000 */
    fd_set fdset;
    int num = __intVal(fd);

    FD_ZERO(&fdset);
    FD_SET(num, &fdset);
    snmp_read(&fdset);
%}
!

responseArrived
    "return true, if a response arrived"

%{
    extern struct synch_state snmp_synch_state;
    struct synch_state *state = &snmp_synch_state;

    RETURN (state->waiting ? false : true);
%}
!

selectFileDescriptorsAndTimeDelta
    "return a set of filedescriptors and the next
     timeout (in milliseconds),
     the snmp library is waiting for."

    |fds delta|

%{  /* STACK: 20000 */
    int numfds = 0;
    fd_set fdset;
    struct timeval timeout;
    int block = 0;
    int index, i;

    FD_ZERO(&fdset);
    timeout.tv_sec = 2;
    timeout.tv_usec = 0;
    snmp_select_info(&numfds, &fdset, &timeout, &block);

    if (numfds) {
	index = 0;
	for (i=0; i< numfds; i++) {
	    if (FD_ISSET(i, &fdset)) {
		index++;
	    }
	}
	numfds = index;
    }
    if (numfds) {
	fds = __ARRAY_NEW_INT(numfds);
	index = 0;
	for (i=0; i< FD_SETSIZE; i++) {
	    if (FD_ISSET(i, &fdset)) {
		__ArrayInstPtr(fds)->a_element[index++] = __MKSMALLINT(i);
	    }
	}
	delta = _MKSMALLINT((timeout.tv_sec * 1000) + (timeout.tv_usec / 1000));
    }
%}.
    ^ Array with:fds with:delta
!

sendGetRequest:nameOrOID next:aBoolean
    "send a get (getNext) request."

    |variableOID retVarOID retVal|

    sessionHandle isNil ifTrue:[
        ^ self error:'no session handle'.
    ].

    nameOrOID isString ifTrue:[
        variableOID := self class OIDFromString:nameOrOID.
    ] ifFalse:[
        variableOID := nameOrOID
    ].

%{  /* STACK: 20000 */
    struct snmp_pdu *pdu, *response;
    oid name[MAX_NAME_LEN+1];
    int l, name_length, status;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));
    char *bytes;
    struct variable_list *vars;
    char stringBuffer[2048*4];
    extern struct synch_state snmp_synch_state;
    struct synch_state *state = &snmp_synch_state;

    if (__Class(variableOID) == @global(SNMPOID)) {
        bytes = __ByteArrayInstPtr(variableOID)->ba_element;
        l = __byteArraySize(variableOID);
        name_length = l / sizeof(oid);
        bzero(name, sizeof(name));
        bcopy(bytes, name, l);

        pdu = snmp_pdu_create( (aBoolean == true) ? GETNEXT_REQ_MSG : GET_REQ_MSG);
        if (! pdu) {
            RETURN (nil);
        }
        snmp_add_null_var(pdu, name, name_length);

        if ((state->reqid = snmp_send(session, pdu)) == 0){
            snmp_free_pdu(pdu);
            RETURN (false);
        }

        state->waiting = 1;
        RETURN (true);
    }
%}.

    ^ false
!

snmpTimeout
%{  /* STACK: 20000 */
    snmp_timeout();
%}
!

waitForResponsesWithTimeout:millis
    "wait for any outstanding responses"

    |info fds timeout sema wasBlocked didTimeout anyFd|

    info := self selectFileDescriptorsAndTimeDelta.
    fds := info at:1.
    timeout := info at:2.
    didTimeout := true.
    anyFd := false.

    timeout == 0 ifTrue:[
        "/ somehow, libsnmp returns a 0 timeout,
        "/ even if there is nothing to read.
        "/ to avoid busy polling, delay a bit.
        (Delay forMilliseconds:50) wait.
    ] ifFalse:[

"/    'fds = ' print. fds printNL.
"/    'timeout = ' print. timeout printNL.

        fds notNil ifTrue:[
            sema := Semaphore new name:'getResponse'.
            [
                "/
                "/ tell processor to signal this sema on any fd
                "/
                wasBlocked := OperatingSystem blockInterrupts.
                fds do:[:fd |
                    anyFd := true.
                    Processor signal:sema onInput:fd
                ].
                timeout notNil ifTrue:[
                    Processor signal:sema afterMilliseconds:(timeout max:millis)
                ].
                sema wait.
            ] ensure:[
                Processor disableSemaphore:sema.
                wasBlocked ifFalse:[OperatingSystem unblockInterrupts].
            ].
        ].
    ].

    fds do:[:fd |
        (OperatingSystem readCheck:fd) ifTrue:[
            didTimeout := false.
            self readResponseFrom:fd
        ]
    ].

    didTimeout ifTrue:[
"/ 'did timeout' printNL.
        self snmpTimeout
    ].

    anyFd ifFalse:[^ #timeout].
    didTimeout ifTrue:[^ #tryAgain].
    ^ #ok
! !

!SNMPSession methodsFor:'startup & release'!

close
    "close the SNMP session"

"/     'close ...' printNL.
    Lobby unregister:self.
    self closeDown

    "Modified: 18.12.1996 / 17:27:22 / cg"
!

closeDown
    "close an snmp session"

%{  /* STACK: 20000 */
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));

    if (session) {
	_INST(sessionHandle) = nil;
	snmp_close(session);
    }
%}
!

disposed
"/    'force closeDown ...' printNL.
    self closeDown
!

open
    "open a session"

    sessionHandle isNil ifTrue:[
	^ self error:'no session handle'.
    ].

%{  /* STACK: 20000 */
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));
    struct snmp_session *ss;
    OBJ sH;

    snmp_synch_setup(session);
    ss = snmp_open(session);
    if (ss == NULL){
	printf("Couldn't open snmp\n");
	RETURN (false);
    }
    freeSessionData(session);
    free(session);

    sH = __MKEXTERNALADDRESS(ss); _INST(sessionHandle) = sH; __STORE(self, sH);
%}.
    Lobby isNil ifTrue:[
"/        'creating registry' printNL.
	Lobby := Registry new
    ].
    Lobby register:self.

    ^ true
! !

!SNMPSession methodsFor:'synchronous requests'!

getNextRequest:nameOrOID
    "get the next value as a printed string (for now).
     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    ^ self getRequest:nameOrOID next:true
!

getRequest:nameOrOID
    "get a value as a printed string (for now).
     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    ^ self getRequest:nameOrOID next:false
!

getRequest:nameOrOID next:aBoolean
    "get (the next) value as a printed string (for now).
     This synchronously sends a get(next) request, and waits for the
     answer; actually blocking smalltalk for the transmission.

     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    SessionLock critical:[
	^ self primGetRequest:nameOrOID next:aBoolean
    ]
!

getValue:aNameString
    |valString val|

    valString := self getVariable:aNameString.
    valString notNil ifTrue:[
        val := self class extractASN1ValueFromString:valString.
    ].
    ^ val
!

getVariable:nameOrOID
    ^ (self getRequest:nameOrOID) value
!

primGetRequest:nameOrOID next:aBoolean
    "get (the next) value as a printed string (for now).
     This synchronously sends a get(next) request, and waits for the
     answer; actually blocking smalltalk for the transmission.

     It is up to the caller, to extract things from the string.
     (the right way to do it was to create a whole hierarchy of asn1 types
      and return the value as an instance of it; but for most tiny demo
      applications, this is way too much overkill ...
      Use extractASN1ValueFromString: for most simple types ...)
     In case of an error, nil is returned and the reason is left in the
     lastError instance variable."

    |variableOID retVarOID retVal|

    sessionHandle isNil ifTrue:[
        ^ self error:'no session handle'.
    ].

    nameOrOID isString ifTrue:[
        variableOID := self class OIDFromString:nameOrOID.
    ] ifFalse:[
        variableOID := nameOrOID
    ].

%{  /* STACK: 20000 */
    struct snmp_pdu *pdu, *response;
    oid name[MAX_NAME_LEN+1];
    int l, name_length, status;
    struct snmp_session *session = _snmp_sessionVal(_INST(sessionHandle));
    char *bytes;
    struct variable_list *vars;
    char stringBuffer[2048*4];

    if (__Class(variableOID) == @global(SNMPOID)) {
        bytes = __ByteArrayInstPtr(variableOID)->ba_element;
        l = __byteArraySize(variableOID);
        name_length = l / sizeof(oid);
        bzero(name, sizeof(name));
        bcopy(bytes, name, l);

        pdu = snmp_pdu_create( (aBoolean == true) ? GETNEXT_REQ_MSG : GET_REQ_MSG);
        if (! pdu) {
            RETURN (nil);
        }
        snmp_add_null_var(pdu, name, name_length);
        status = snmp_synch_response(session, pdu, &response);

        switch (status) {
            case STAT_SUCCESS:
                switch (response->errstat) {
                    case SNMP_ERR_NOERROR:
                        for(vars = response->variables; vars; vars = vars->next_variable){
                            switch (vars->type) {
                                case SNMP_NOSUCHOBJECT:
                                    _INST(lastError) = @symbol(noSuchObject);
                                    break;

                                case SNMP_NOSUCHINSTANCE:
                                    _INST(lastError) = @symbol(noSuchInstance);
                                    break;

                                case SNMP_ENDOFMIBVIEW:
                                    _INST(lastError) = @symbol(endOfMib);
                                    break;

                                default:
/*
                                    print_variable(vars->name, vars->name_length, vars);
 */
                                    retVarOID = __NEW_INT(@global(SNMPOID), vars->name_length * sizeof(oid));
                                    bcopy(vars->name, __ByteArrayInstPtr(retVarOID)->ba_element, vars->name_length * sizeof(oid));

                                    sprint_value(stringBuffer, vars->name, vars->name_length, vars);
                                    retVal = __MKSTRING(stringBuffer);
                                    break;
                            }
                        }
                        break;

                    case SNMP_ERR_NOSUCHNAME:
                        _INST(lastError) = @symbol(noSuchName);
                        printf("SNMP [info]: no such name: ");
                        {
                            int i;
                            for (i=0; i < name_length; i++) {
                                printf(".%d", name[i]);
                            }
                        }
                        printf("[SNMP_ERR_NOSUCHNAME]\n");
                        break;

                    default:
                        _INST(lastError) = @symbol(otherError);
                        printf("SNMP [info]: Error in packet.\nReason: %s\n",
                                                snmp_errstring(response->errstat));
                        break;
                }
                break;

            case STAT_TIMEOUT:
                _INST(lastError) = @symbol(noResponse);
                printf("SNMP [info]: No Response from %s [STAT_TIMEOUT]\n", session->peername);
                break;

            default:
                _INST(lastError) = @symbol(otherError);
                printf("SNMP [info]: An error occurred\n");
                break;
        }

        if (response) {
            snmp_free_pdu(response);
        }
    }
%}.

    ^ retVarOID -> retVal
! !

!SNMPSession class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libsnmp/SNMPSession.st,v 1.41 2007/02/08 15:21:45 stefan Exp $'
! !

SNMPSession initialize!
