/***********************************************************
        Copyright 1992 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/
/*
 * System dependent routines go here
 */
#include <unistd.h>
#ifdef ultrix4_4
# include <sys/types.h>
#endif
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#ifndef linux
#include <nlist.h>
#endif
#include <sys/file.h>
#ifdef linux
# include <netdb.h>
#endif
#include "system.h"

#define NUM_NETWORKS    32   /* max number of interfaces to check */

#ifndef SIOCGIFFLAGS
/* SYSV.4 */
# include <sys/sockio.h>
#endif

#ifndef IFF_LOOPBACK
#define IFF_LOOPBACK 0
#endif
#ifndef linux
#define LOOPBACK    0x7f000001
#else
#define LOOPBACK    0x0100007f
#endif

u_long get_myaddr(){
    int sd;
    struct ifconf ifc;
    struct ifreq conf[NUM_NETWORKS], *ifrp, ifreq;
    struct sockaddr_in *in_addr;
    int count;
    int interfaces;             /* number of interfaces returned by ioctl */

    if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        return 0;
    ifc.ifc_len = sizeof(conf);
    ifc.ifc_buf = (caddr_t)conf;
    if (ioctl(sd, SIOCGIFCONF, (char *)&ifc) < 0){
        close(sd);
        return 0;
    }
    ifrp = ifc.ifc_req;
    interfaces = ifc.ifc_len / sizeof(struct ifreq);
    for(count = 0; count < interfaces; count++, ifrp++){
        ifreq = *ifrp;
        if (ioctl(sd, SIOCGIFFLAGS, (char *)&ifreq) < 0)
            continue;
        in_addr = (struct sockaddr_in *)&ifrp->ifr_addr;
        if ((ifreq.ifr_flags & IFF_UP)
            && (ifreq.ifr_flags & IFF_RUNNING)
            && !(ifreq.ifr_flags & IFF_LOOPBACK)
            && in_addr->sin_addr.s_addr != LOOPBACK){
                close(sd);
                return in_addr->sin_addr.s_addr;
            }
    }
    close(sd);

#ifdef linux
    /* 
     * this is a hack for all those poor slip/ppp/... using people like me.
     * there's not always an up and running ethnernet imnterface... 
     */
    { char hname [256];
      struct hostent *he;
      if (! gethostname (hname, 256) && ((he = gethostbyname (hname))))
	  return * (u_long *) he->h_addr;
    }
#endif
    return 0;
}

#if !defined(linux) && !defined(SYSV4)

struct nlist nl[] = {
    { "_boottime" },
    { "" }
};

/*
 * Returns uptime in centiseconds(!).
 */
long get_uptime(){
    struct timeval boottime, now, diff;
    int kmem;

    if ((kmem = open("/dev/kmem", 0)) < 0)
        return 0;
    nlist("/vmunix", nl);
    if (nl[0].n_type == 0){
        close(kmem);
        return 0;
    }

    lseek(kmem, (long)nl[0].n_value, L_SET);
    read(kmem, &boottime, sizeof(boottime));
    close(kmem);

    gettimeofday(&now, 0);
    now.tv_sec--;
    now.tv_usec += 1000000L;
    diff.tv_sec = now.tv_sec - boottime.tv_sec;
    diff.tv_usec = now.tv_usec - boottime.tv_usec;
    if (diff.tv_usec > 1000000L){
        diff.tv_usec -= 1000000L;
        diff.tv_sec++;
    }
    return ((diff.tv_sec * 100) + (diff.tv_usec / 10000));
}

#endif /* ! linux  && ! SYSV4 */

#if SYSV
/*
 * strcasecmp.c : Case-insensitive string comparison routines
 *
 * George Ferguson, ferguson@cs.rochester.edu, 23 Apr 1993.
 */

#define ISUPPER(C) ((C) >= 'A' && (C) <= 'Z')
#define TOLOWER(C) ((C) - 'A' + 'a')
#define NORMAL(C) (ISUPPER(C) ? TOLOWER(C) : (C))

int
strcasecmp(s1,s2)
char *s1,*s2;
{
    char c1,c2;

    while (1) {
	c1 = NORMAL(*s1);
	c2 = NORMAL(*s2);
	if (c1 != c2)
	    return(c1-c2);
	else if (c1 == '\0')
	    return(0);
	else {
	    s1 += 1;
	    s2 += 1;
	}
    }
    /*NOTREACHED*/
}

int
strncasecmp(s1,s2,n)
char *s1,*s2;
int n;
{
    char c1,c2;

    while (n-- > 0) {
	c1 = NORMAL(*s1);
	c2 = NORMAL(*s2);
	if (c1 != c2)
	    return(c1-c2);
	else if (c1 == '\0')
	    return(0);
	else {
	    s1 += 1;
	    s2 += 1;
	}
    }
    return(0);
}
#endif /* SYSV */
