"{ Package: 'stx:projects/webServer' }"

StandaloneStartup subclass:#WebServerStartup
	instanceVariableNames:'processCountSema'
	classVariableNames:''
	poolDictionaries:''
	category:'demos-Standalone-applications'
!

!WebServerStartup class methodsFor:'documentation'!

documentation
"
    a non-graphical demo application;
    when built, it creates an executable which starts
    a webServer (HTTPServer - comparable to apache).

    Command-line arguments:
	see WebServerStartup usageLines asString
    [build with:]
	cd projects/webServer; make

    [execute with:]
	cd projects/webServer; webServer

    [start with:]
	WebServerStartup main
"
!

examples

"
    check from non-headless Smalltalk; look at the terminal output

    Example 1:view the help file
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--help'.

	self new initialize startServersFromArguments:args.
									[exEnd]
    Example 2:start from settings file
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--useFile'; add:'./webServerSettings.xml'.

	self new initialize startServersFromArguments:args.

									[exEnd]
    Example 3:start from settings file and overwrite port
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--useFile'; add:'./webServerSettings.xml'.
	args add:'--port'; add:'8082'.

	self new initialize startServersFromArguments:args.
									[exEnd]

    Example 3:start from settings file and overwrite logFile
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--useFile'; add:'./webServerSettings.xml'.
	args add:'--log'; add:'/tmp/foo'.

	self new initialize startServersFromArguments:args.
									[exEnd]

    Example 4:start two servers in one image
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--useFile'; add:'./webServerSettings_8081.xml'.
	args add:self webServerSettingsSeperator.
	args add:' --useFile'; add:'./webServerSettings_8080.xml'.

	self new initialize startServersFromArguments:args.

									[exEnd]

    Example 4:start two servers in one image
									[exBegin]
	| args |
	args := OrderedCollection new.
	args add:'--useFile'; add:'./webServerSettings_8081.xml'.
	args add:self webServerSettingsSeperator.
	args add:' --useFile'; add:'./webServerSettings_8080.xml'.

	self new initialize startServersFromArguments:args.

									[exEnd]

"
! !

!WebServerStartup class methodsFor:'actions'!

setServerSettingsFor:aServer fromArguments:argv
    |argvIdx logFileName|

    argvIdx := argv indexOf:'--port'.
    argvIdx ~~ 0 ifTrue:[
        aServer port:(Integer readFrom:(argv at:(argvIdx+1))).
    ].

    argvIdx := argv indexOf:'--logFile'.
    argvIdx ~~ 0 ifTrue:[
        | nextArg |
        nextArg := argv at:(argvIdx+1) ifAbsent:nil.
        (nextArg notNil and:[(nextArg startsWith:'-') not]) ifTrue:[
            logFileName := nextArg asFilename.
        ].
    ].
    logFileName notNil ifTrue:[
        aServer logFile:logFileName.
    ].


    argvIdx := argv indexOf:'--homeURL'.
    argvIdx ~~ 0 ifTrue:[
        aServer homeURL:argv at:(argvIdx+1).
    ].

    argvIdx := argv indexOf:'--notFoundURL'.
    argvIdx ~~ 0 ifTrue:[
        aServer notFoundURL:argv at:(argvIdx+1).
    ].

    (argv includes:'--verbose') ifTrue:[
        aServer verbose:true.
        aServer verboseData:true.
        aServer verboseErrors:true.
    ].
! !

!WebServerStartup class methodsFor:'helpers'!

redirectStandardStreams
    |logFile|

    logFile := ('/tmp/web%1.out' bindWith:(OperatingSystem getProcessId)) asFilename writeStream.
    logFile buffered:false.
    Stderr := logFile.
    Stdout := logFile.
    Transcript := logFile.
!

serverForSettingsFilename:aFilename

    |serverFileName server|

    serverFileName := aFilename asFilename.
    serverFileName exists not ifTrue:[
	('Settingsfile ', serverFileName asString, ' does not exist') errorPrintCR.
	^ nil.
    ].
    server := HTTPServer serverFromSettingsFile:serverFileName.
    server isNil ifTrue:[
	('can not create server from file ', serverFileName asString) errorPrintCR.
	^ nil.
    ].
    server settingsFilename:serverFileName.
    ^ server
! !

!WebServerStartup class methodsFor:'startup'!

allowDebugOption
    "enable/disable the --debug startup option.
     The default is now false, so standAlone apps are closed by default.
     Can be redefined in subclasses to enable it"

    ^ true
!

isHeadless
    ^ true
!

main:argv

    "/double stack size because of the stc needs more stack in standalone app
"/    Process defaultMaximumStackSize:(Process defaultMaximumStackSize * 2).
"/    Smalltalk beHeadless:true.
"/    OperatingSystem disableSignal:(OperatingSystem sigHUP).
    self verboseInfo:'webServer [info]: entering main'.

    Smalltalk isStandAloneApp ifTrue:[
        Smalltalk loadPackage:'exept:tools/watchDog'.      "no error, if not present"
        WatchDogInterface notNil ifTrue:[
            self verboseInfo:'webServer [info]: start watchdog'.
            WatchDogInterface startup.
        ].
    ].

    self new initialize;
             startServersFromArguments:argv.

    "we come here after all servers have been terminated"
!

usage
    super usage.
    Stderr 
        nextPutLine:'       serverOptions [--verbose] [--port portNr] [--homeURL url] [--notFoundURL url] [--useFile fileName]';
        nextPutLine:'          --verbose ............... verbose (enable trace messages)';
        nextPutAll: '          --port portNr ........... listen on portNr (default: '; nextPutAll:HTTPServer port printString; nextPutLine:')';
        nextPutAll: '          --homeURL url ........... homeURL (default: "'; nextPutAll:(HTTPServer homeURL ? ''); nextPutLine:'")';
        nextPutAll: '          --notFoundURL url ....... notFoundURL (default: "'; nextPutAll:HTTPServer notFoundURL asString; nextPutLine: '")';
        nextPutAll: '          --useFile fileName ...... e.g. '; nextPutLine:'HTTPServer settingsFilename';
        nextPutLine:'          --logFile file .......... log onto file (default is /tmp/web.log)'.


    "
        self usage
    "
! !

!WebServerStartup methodsFor:'actions'!

addRunningServer
    processCountSema consume
!

removeRunningServer
    processCountSema signal.
! !

!WebServerStartup methodsFor:'initialization'!

initialize

    processCountSema := Semaphore new:1.
    processCountSema name:'HTTPserver count'.

"
    self basicNew initialize
"
! !

!WebServerStartup methodsFor:'startup'!

createServerFromArguments:argv
    |idx server|

    idx := argv indexOf:'--useFile'.
    idx ~~ 0 ifTrue:[
        |nextArg|

        nextArg := argv at:(idx + 1) ifAbsent:nil.
        (nextArg notNil and:[ (nextArg startsWith:'-') not ]) ifTrue:[
            "/ start server and search for some parameters
            server := self class serverForSettingsFilename:nextArg.
            server isNil ifTrue:[
                ('server creation from settingsFile ' , nextArg , ' failed') errorPrintCR.
                ^ nil
            ].
        ] ifFalse:[
            ('server creation from settingsFile ' , nextArg , ' failed') errorPrintCR.
            ^ nil
        ].
    ] ifFalse:[
        server := HTTPServer
                    newServerWithDefaultServiceListOn:(HTTPServer defaultPort).
    ].
    ^ server

    "Modified: / 13-07-2009 / 15:48:32 / Administrator"
!

setupErrorHandler
    "setup an error handler to not crash when something unexpected happens.
     e.g. ComancheSwiki does some actions in forked processes, which do not
     handle errors"

    NoHandlerError emergencyHandler:[:ex|
        |activeProcess|

        activeProcess := Processor activeProcess.
        Timestamp now errorPrint.
        (': unhandled exception for <', activeProcess name, '> [', ex description, ']') errorPrintCR.
        (activeProcess name startsWith:'HTTPServer') ifTrue:[
            "This should not happen, since HTTP servers handle errors.
             Anyway, the should be kept alive"
        ] ifFalse:[
            "terminate the failing process"
            ('Terminate failing process...') errorPrintCR.
            Stderr flush.
            [
                activeProcess terminate.
            ] on:Error do:[:ex|
                ('TerminateNoSignal failing process...') errorPrintCR.
                activeProcess terminateNoSignal.
            ].
        ].
    ].
!

startServerFromArguments:argv

    |server serverProcess|

    server := self createServerFromArguments:argv.
    server isNil ifTrue:[
        'cannot create server from arguments' errorPrintCR.
        argv do:[:aString|
            aString errorPrintCR.
        ].
        ^ self.
    ].
    self class setServerSettingsFor:server fromArguments:argv.

    server isNil ifTrue:[
        'can not create server from arguments' errorPrintCR.
        argv do:[:aString|
            aString errorPrintCR.
        ].
        ^ self.
    ].
    'WebServerStartup [info]: Start paramters:' infoPrintCR.
    server parametersPrintStrings do:[:aString|
        aString infoPrintCR.
    ].

    serverProcess := server createServerProcess.
    serverProcess notNil ifTrue:[
        serverProcess addExitAction:[self removeRunningServer].
        self addRunningServer.
        serverProcess resume.
    ].

    'WebServerStartup [info]: You can now talk to me via your preferred web browser.' infoPrintCR.
    server port ~~ 80 ifTrue:[
        ('WebServerStartup [info]: (non-standard port, use: "http://host.domain:<1p>/" to connect to me)' 
          expandMacrosWith:server port printString) infoPrintCR.
    ].
!

startServersFromArguments:argv
    "start one or more servers a specified on the command line
     and wait until the last server has terminated"

    |startIndex endIndex serverArgs|

    startIndex := 0.
    [
        endIndex := argv indexOf:$, startingAt:startIndex + 1.
        endIndex ~~ 0 ifTrue:[
            serverArgs := argv copyFrom:startIndex + 1 to:endIndex - 1.
        ] ifFalse:[
            serverArgs := argv copyFrom:startIndex + 1.
        ].
        self startServerFromArguments:serverArgs.
        startIndex := endIndex.
    ] doUntil:[endIndex = 0].

    self setupErrorHandler.

    "wait until the last server process has finished"
    processCountSema wait.
! !

!WebServerStartup class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/projects/webServer/WebServerStartup.st,v 1.63 2010/05/03 16:41:52 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/projects/webServer/WebServerStartup.st,v 1.63 2010/05/03 16:41:52 stefan Exp $'
! !
