"{ Package: 'stx:goodies/webServer' }"

Object subclass:#HumanReadableImageGenerator
	instanceVariableNames:'form image stringLength string extent characterSet fonts minAngle
		maxAngle numberOfRandomLines numberOfRandomDots colors'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-HTML-Misc'
!

!HumanReadableImageGenerator class methodsFor:'documentation'!

documentation
"
    Generate a form containing a string for the user to type into a web-form
    as confirmation that he is really human.
    I.e. the forms image is designed to make it hard for an OCR program to
    extract the characters.
    At least, we hope that it is so.

    Sometimes, the picture is hard to decipher, even for a real human;
    therefore, add some 'try another picture' button to the web page.

    Caveat: this is a dirty hack

    [usage:]
	HumanReadableImageGenerator new generate inspect

    [author:]
	Claus Gittinger
"
!

examples
"
                                                [exBegin]
    |gen image|

    gen := HumanReadableImageGenerator new.
    gen extent:300@100.
    gen numberOfRandomDots:1000.
    gen numberOfRandomLines:25.
    gen minAngle:-45.
    gen maxAngle:45.
    gen string inspect.
    image := gen generate.
    image inspect.
                                                [exEnd]
"
! !

!HumanReadableImageGenerator class methodsFor:'instance creation'!

new
    ^ self basicNew initialize.
! !

!HumanReadableImageGenerator class methodsFor:'defaults'!

characterMapCharacters
    ^ 'ABCDEFGHHIJKLMNOPQRSTUVWXYZabcdefghhijklmnopqrstuvwxyz0123456789'.
!

numberOfCharMaps
    ^ 3
! !

!HumanReadableImageGenerator class methodsFor:'image specs'!

charMapImage1
"character bitmaps (needed ebcause the webServer has no Display for drawing)"

    "
     self charMapImage1 inspect
     ImageEditor openOnClass:self andSelector:#charMapImage1
     Icon flushCachedIcons
    "

    <resource: #image>

    ^Icon
        constantNamed:'HumanReadableImageGenerator class charMapImage1'
        ifAbsentPut:[(Depth1Image new) width: 3200; height: 53; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@X@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@
@@@@@@@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@<@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@G @@@@@@@@@@@@@@@@@@@@@@O@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@@@@@@C @@@@@@@@@@@C0@@@X@G@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@
@C @@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@O@@@@@@@@0@@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@?@@@@@@@A<@@@@@@@@@@@@@@@@G@@@@@@G8@@@@@@@O?@@@@@0@O@@@@@L@C0@@@@C@@@@@@@@@F@@@@@@X
@F@@@@@A @@@@@@@@@@O@@@@N@A8@@@@@C>@@@@@@C?8@@@@@@O<@@@@@@O<@@@@@@C>@@@@@@@@@@@@@@G @\@@@@8@@@@@@@@@@@@<@@@O@C0@@@@G@A<@
@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@C<@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@C0@@@@@@@^@@@@@@@^@@@@@@@G @@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@A?<@@@@@@A? @@@@@??0@@@@@N@O8@@
@@G??@@@@@@@_?0@@@@^@C0@@@@G @<@@@@A8@@@@@@@@C0@@@@@O@C0@@@@@<@@@@@@@G@@G0@@@G @^@@@@@C?8@@@@@_?? @@@@@G?@@@@@@??8@@@@@G
?8@@@@@@@G<@@@@A8@O@@@@O@@C@@@@@8@@@O@@@C0A<@@@@C8@^@@@@@L@C8@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@A? @@@@@@@@@@
@@@@<@@@@@@@O@@@@@@@A<@@@@@@@G @@@@@@G @@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A>@@@@@@@F@@@@@@@O @@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@C0@@@@@C??@@@@@@A?<@@@@@_?? @@@@O??>@@@@C??0@@@@@@O?<@@@@G @<@@@@A8@O@@@@@^@@@@@@@@@<@@@@@C0A<@@@@@O@@@@@
@@C8@C8@@@A<@G@@@@@A?? @@@@O??<@@@@@G?8@@@@@???@@@@@G??@@@@@@A?? @@@@^@C0@@@C0@@8@@@@N@@@C0@@@<@?@@@@@>@O@@@@@G???@@@@@@
@@@@@@@G @@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@^8@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@^@@@@@@@@0@@@@@@A8@@@@@@@^@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?8@@@@@@G0@@@@@
@O>@@@@@A0_0@@@@@@@^@@@@@@@@^@@@@@@?@@@@@@@@@ @@@@@C0@@@@@@@G @@@@@@A>@@@@@A??8@@@@@@?O@@@@@G??<@@@@C???@@@@@??8@@@@@@O<
F@@@@A8@O@@@@@^@C0@@@@G @@@@@@@@O@@@@@@<@^@@@@@C0@@@@@@@?@A>@@@@_@A0@@@@@>C<@@@@G<@O @@@@C<_@@@@@O0G8@@@@C<C0@@@@C???8@@
@@O@@<@@@@\@@^@@@@C0@@@<@@@G _@@@@@G0G @@@@A???0@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@ON@@@@@@@@@@@@@@C0@@@@@@@<
@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A??@@@@@@G<@@@@@@O?0@@@@@_?>@@@@@@@_ @@@@@A??0@@@@@_0@@@@@G??<@@@@@C?@@@@@@@O<@@@@@
@@_ @@@@@?0N@@@@@@_G0@@@@@<@_ @@@@??8@@@@@O0@@@@@@@G<@@@@@@^@C @@@@G @8@@@@C8@@@@@@@@C0@@@@@O@O @@@@@<@@@@@@@O0@_ @@@G0@
\@@@@@_@O@@@@A? A8@@@@A<C8@@@@C<@_@@@@A<@\@@@@A??0@@@@@C0@O@@@@G @G @@@@<@@@N@@@A8O @@@@@>A8@@@@@_?;8@@@@@@@@@@@@@<@@@@@
@@@@@@@@@@@@G@@@@@@@@@@@@@@C3 @@@@@@@@@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@??8@@@@@C>@@@@@@G8<@@@@@G?? @
@@@@@O0@@@@@@??<@@@@@O0@@@@@A???@@@@@A?8@@@@@@G?0@@@@@@O8@@@@@O0C @@@@@_C8@@@@@O@@<@@@@O@@@@@@@C0@@@@@@@C<@@@@@@G@@8@@@@
A0@N@@@@@<@@@@@@@@@<@@@@@C0G @@@@@N@@@@@@@C<@O8@@@A>@O@@@@@O A8@@@@A8@>@@@@@>@_@@@@@O@C0@@@@<@F@@@@@_?0@@@@@@<@C0@@@@8@C
8@@@@G@@@C @@@OG0@@@@@G0<@@@@@A?!!<@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A0@@@@@@8@@@@@@@80@@@@@@@@@@@@@@N@@@@@@@C @@@@@@@@@@@@@
@@@@@@@@@@A0@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@_@_@@@@@C? @@@@@C8G @@@@@?G0@@@@@@G<@@@@@@_?<@@@@@G0@@@@@@_?? @@@@A>O @@@@@G0<@@@@@@C?@@@@@C0A8@
@@@@G <@@@@@C @O@@@@C0@@@@@@@<@@@@@@@A>@@@@@@A0@^@@@@@\@G @@@@O@@@@@@@@@O@@@@@@<C8@@@@@G @@@@@@A7 G.@@@@_ C @@@@G0@^@@@@
@<@O@@@@@O@C0@@@@C0@<@@@@O@@@@@@@G<<@@@@@@O@A8@@@@O@@<@@@@A8@0@8@@@C38@@@@@A<^@@@@@@@@>@@@@@@@@@@@@@C @@@@@@@F@@@@@@@@^<
@@@@@@?@@@@@@@^H@@@@@@@_@@@@@@C @@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@\C @@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@C>@@@@
@@@@@@@@@@@8@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O C0@@@@C=8@@@@@A<A8@@@@@@C8@@@@@@G?@@@@@@O
@@@@@@@A8@@@@@@@@C0@@@@@>A<@@@@@C8O@@@@@@A90@@@@@<@<@@@@@C0G@@@@@A8@A8@@@@<@@@@@@@O@@@@@@@@>@@@@@@@<@G @@@@O@A8@@@@C0@@@
@@@@@C0@@@@@OA<@@@@@A8@@@@@@@]8C3 @@@O<@8@@@@A8@G @@@@O@G0@@@@G @^@@@@@<@O@@@@C0@@@@@@@@O@@@@@@C0@^@@@@C0@O@@@@@^@^@^@@@
@^<@@@@@@O''@@@@@@@@_@@@@@@G0@@@@@@8@@@@@@@G0@@@@@@@_/@@@@@@_8@@@@@@G @@@@@@@O8@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@N@@@@@@@GA8
@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@_0@@@@@@@G0@@@@@A? @@@@@@@@@@@@@@^@@@@@@@C @@@@@@@@C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@
@@@@@@@@@@G0@>@@@@A<^@@@@@@^@^@@@@@@C8@@@@@@C30@@@@@G @@@@@@@<@@@@@@@@A8@@@@@O@_@@@@@A<C0@@@@@@<^@@@@@O@^@@@@@A<@@@@@@@^
@@^@@@@O@@@@@@@G @@@@@@@_@@@@@@@N@A8@@@@C @^@@@@@<@@@@@@@@@<@@@@@C0^@@@@@@^@@@@@@@G^A=8@@@C?@N@@@@@<@A8@@@@C0G8@@@@C8@G 
@@@@O@G @@@@>@@@@@@@@C0@@@@@@<@G @@@@\@G @@@@G G G @@@C>@@@@@@A?0@@@@@@@O @@@@@G<@@@@@@NC0@@@@@A<@@@@@@@O? @@@@@ON@@@@@@
A8@@@@@@@O>@@@@@@N@@@@@@@C @@@@@@@\@@@@@@@G @@@@@@C0>@@@@@@<@@@@@@@GA0D@@@@A0@@@@@@@O>@@@@@@@C>@@@@@@>8@@@@@@8@@@@@@@O @
@@@@@A8A @@@@@\A0@@@@@N@@@@@@@G@\@0@@@@8C@@@@@@G@<@@@@@@A<@@@@@A8@G @@@@^G @@@@@C@G @@@@@A<@@@@@@A88@@@@@A0@@@@@@@^@@@@@
@@@@^@@@@@C0O @@@@@^A8@@@@@@OG @@@@C0O @@@@@>@@@@@@@G @G @@@C0A8@@@@A8@@@@@@@G @@@@@@C @\@@@@@8@G@@@@@^@@@@@@@@@O@@@@@@<
_@@@@@@G @@@@@@A30>^@@@@=8C @@@@O@@^@@@@@<G<@@@@@<@@<@@@@C0C8@@@@G0@@@@@@@@<@@@@@@^@A8@@@@G A8@@@@@8C<A0@@@@?@@@@@@@O8@@
@@@@@G0@@@@@C? @@@@@C#>@@@@@@? @@@@@@G38@@@@@C# @@@@@@\@@@@@@@G3 @@@@@C N@@@@@@8C @@@@@O@@@@@@@C8@@@@@@@<_@@@@@@N@@@@@@@
C0>C0@@@@\O@@@@@@G?8B@@@@CC?0@@@@@^O@@@@@@NC8@@@@@C0@@@@@@@^C8@@@@@O@\@@@@@G L@@@@@A0G@_A @@OA8@@@@@A0O@@@@@A?? @@@@@>@A
8@@@@@A8@@@@@@@C0@@@@@@<@@@@@@@<^@@@@@@<@@@@@@@O @@@@@@@@O@@@@@@>C0@@@@@O@^@@@@@@G 8@@@@@<_0@@@@@O@@@@@@@A8@C8@@@@??>@@@
@@^@<@@@@@C0@@@@@@@8@_@@@@@N@G0@@@@G @@@@@@@@C0@@@@@OO @@@@@A8@@@@@@@8<OG@@@@N_A8@@@@G @G @@@@NG>@@@@@_@@O@@@@@<C<@@@@A?
 @@@@@@@O@@@@@@G @^@@@@@8@<@@@@@N@?@\@@@@O @@@@@@A>@@@@@@@C8@@@@@A88@@@@@@9?0@@@@@\8@@@@@@A8>@@@@@A88@@@@@@O@@@@@@@C88@@
@@@@8O @@@@@NC8@@@@@C0@@@@@@@<@@@@@@@NO @@@@@G @@@@@@@<_!!>@@@@OG8@@@@@C8?? @@@A9<<@@@@@OC0@@@@@C#?@@@@@A8@H@@@@C??8@@@@@
C0O@C@@@A8G 8@@@@^C0O?8@@C0^@@@@@@<C0@@@@@_?8@@@@@O@@^@@@@@@\@@@@@@@A<@@@@@@_?@@@@@@>G @@@@@_?@@@@@@C0@@@@@@@@G @@@@@G;8
@@@@@C0O @@@@@C0O@@@@@OO?@@@@@G0@@@@@@@^@A<@@@@O?? @@@@G!!?@@@@@A<@C0@@@@O??0@@@@C??<@@@@A8@@@@@@@@@<@@@@@C70@@@@@@\@@@@@
@@NO''10@@@C#0\@@@@A8@C0@@@@C''>@@@@@G @C0@@@@OC>@@@@@G?@@@@@@@G @@@@@A8@G @@@@N@^@@@@@C0_0O@@@@G8@@@@@@@O@@@@@@@A<@@@@@@<
N@L@@@@\>^A @@@GN@L@@@@@<_ B@@@@\\@0@@@@C??@@@@@@<N@@@@@@NG8@@@@@C!!>@@@@@A<@@@@@@@O@@@@@@@C'' @@@@@A8@@@@@@@OO8? @@@C#>@A
@@@A<G?0@@@@^<O@0@@@G <@@@@@@9?0@@@@@\@F@@@@@?? @@@@@@<C0A0@@@^C?>@@@@G <C?<@@@<O@F@@@@N@<@@@@@C8\@@@@@G @G @@@@@G@@@@@@
@@^@@@@@@O?<@@@@@^A8@@@@@G?>@@@@@A8O8@@@@@@C0@@@@@@?<@@@@@@<G0@@@@@@<C0@@@@C3?<@@@@A8@@B@@@@G@@^@@@@C??@@@@@A??0@@@@@^@@
>@@@@C??<@@@@@???@@@@@^@@@@@@@@@O@@@@@@?8@@@@@@G@@@@@@@C!!98\@@@@8^G@@@@@<@@<@@@@A?>@@@@@A8@@<@@@@C?>@@@@@@?<@@@@@@A8@@@@
@@^@A8@@@@C0G @@@@@<G^C @@@C?@@@@@@@C0@@@@@@@>@@@@@@^C G@@@@G_C!!8@@@C!! O@@@@@^G8A @@@ON@\@@@@A?? @@@@@^C @@@@@C#?@C@@@@8
?0@0@@@^@A@@@@@C0@@@@@@@;0@@@@@@^@@@@@@@C3.N8@ @@9?0@0@@@^A78@@@@G.C0<@@@A0O@L@@@@^>@@@@@@G@C@@@@@O<@@@@@@@OA<@8@@@G ?>@
@@@A8_ ?<@@@OC0C @@@C O@@@@@@@N@@@@@A8@C8@@@@@C0@@@@@@@O @@@@@C??@@@@@O_?<@@@@@_?0@@@@@^O? @@@@@@<@@@@@@C<@@@@@@OG<@@@@@
@^G<@@@@@<8_@@@@@^@@A @@@A0@_ @@@A8@@@@@@@_?8@@@@@G @O @@@@?>O@@@@@O?#0@@@@G @@@@@@@@C0@@@@@O<@@@@@@C0@@@@@@A8_<O@@@@^G!!
0@@@@O@@^@@@@@_?@@@@@@<@@_@@@@@;?@@@@@@C?0@@@@@@^@@@@@@G @^@@@@@\C0@@@@@OA3 8@@@A=8@@@@@@@<@@@@@@@_@@@@@@GA0A0@@@A7@8<@@
@@8@G @@@@G@>@0@@@C/@N@@@@@_>@@@@@@O 8@@@@@@130A0@@@L\<@\@@@G @0@@@@@<@@@@@@@N8@@@@@@G A @@@@@93''.@X@@^<<@\@@@O@\@@@@@A?
@8>@@@@<G G@@@@G.@@@@@@A8C0@@@@@O@@@@@@@C _@\@@@@8N^@@@@@^G8\@@@@A98@0@@@A8C0@@@@@@G@@@@@@<@@<@@@@@@<@@@@@@@G0@@@@@@8C8@
@@@G??? @@@@@C>@@@@@O''?8@@@@@@^@@@@@@@?0@@@@@C??@@@@@@O?? @@@@N@A8@@@@O @@8@@@@<@_ @@@@^@@@@@@@G?@@@@@@C0@C0@@@@^@C0@@@@
G @<@@@@A8@@@@@@@@@<@@@@@C?8@@@@@@<@@@@@@@\C>C0@@@G <\@@@@C0@G @@@@G>@@@@@@O@@G @@@@^?8@@@@@@C>@@@@@@G @@@@@A8@G @@@@G 8
@@@@@A088\@@@@>O@@@@@@@O@@@@@@@O @@@@@C0\@8@@@@_ N>@@@@^@C0@@@@C0_ \@@@@? G@@@@@C0@@@@@@C0^@@@@@@\8<@8@@@GNO@N@@@A8@\@@@
@@O@@@@@@@C>@@@@@@A8@X@@@@@N<93 N@@G^O@N@@@C0G@@@@@@_0^_@@@@NC8C @@@A7@@@@@@@OA8@@@@@C @@@@@@@8O0^@@@@NG @@@@@G!!.G@@@@@O
<@X@@@@\A<@@@@@@C0@@@@@O@@_@@@@@@O@@@@@@@C8@@@@@@@@^@@@@A???8@@@@@@O @@@@C38O@@@@@@G @@@@@@_>@@@@@@_; @@@@@G??8@@@@G @^@
@@@C0@@<@@@@O@_0@@@@G @@@@@@A8@@@@@@@<@A<@@@@G@@<@@@@A0@O@@@@@>@@@@@@@@@N@@@@@@?? @@@@@O@@@@@@@G@? <@@@A8G''@@@@@<@C0@@@@
A>@@@@@@C0NC8@@@@G ?@@@@@@@O @@@@@A8@@@@@@^@C0@@@@@8^@@@@@@^NGG@@@@_A8@@@@@@C0@@@@@@G0@@@@@@8O@\@@@@G8C?@@@@G@A8@@@@@8N8
N@@@@O0C @@@@@<@@@@@@@<G @@@@@GNO@\@@@A3#0G@@@@^@N@@@@@C0@@@@@@@? @X@@@@^@L@@@@@C.N88G@@A7#0O@@@A8C0@@@@@G8G?@@@@GA>A8@@
@@]0@@@@@@C0<@@@@@@8@@@@@@@NG^O@@@@C10@@@@@@8;!!0@@@@C>@N@@@@G@?@@@@@@A8@@@@@C0@O @@@@@C0@@@@@@A<@@@@@@@@G @@@@O <N@@@@@@
A8@@@@@<<A0@@@@@C0@@@@@@OG0@@@@@C88@@@@@A?>N@@@@A8@G @@@@<@@^@@@@C0?8@@@@A8@@@@@@@<@@@@@@@O@A>@@@@A0@O@@@@@\@C0@@@@O @@@
@@@@@G @@@@@OG?@@@@@C0@@@@@@A0G0O@@@@^A? @@@@O@A8@@@@@^@@@@@@@<G!!<@@@@A8C<@@@@@@A8@@@@@@^@@@@@@G @<@@@@@NO@@@@@@G''A30@@@
O _@@@@@@@8@@@@@@C8@?8@@@^C0N@@@@A<A?@@@@A0@<@@@@@NGNG@@@@C8A0@@@@@O@@@@@@@^G8@@@@@A7C0N@@@@]0<C @@@G G@@@@@@<@@@@@@@O<@
N@@@@G O@@@@@@?#>NG @@_0\G @@@^A8@@@@@A<C?@@@@A0?@<@@@@G8@@@@@@@^^@@@@@@N@@@@@@@GA3? @@@@\\@@@@@@NN<8@@@@@_@G@@@@A0_0@@@
@@@<@@@@@@<@G0@@@@@@<@@@@@@A>@@@@@@@@A8@@@@@@O@@@@@@@@^@@@@@O^@<@@@@@@<@@@@@@G >@@@@@@@^@@@@@@?8C0@@@@^@C8@@@@O@@O@@@@@?
?8@@@@@<@@0@@@@O@@@@@@@C0@?@@@@@\@C0@@@@G@@<@@@@C0@@@@@@@@A0@@@@@C0_? @@@@8@@@@@@@<@8C0@@@G@O8@@@@C0@<@@@@@G @@@@@@OA<>@
@@@@^@_ @@@@@@>@@@@@@G @@@@@@<@O@@@@@C3 @@@@@A;0^8@@@G0C8@@@@@@^@@@@@@C<O?>@@@GA>O@@@@@^@?@@@@@\@^@@@@@GC3'' @@@@<A8@@@@@
C0@@@@@@G''>@@@@@@_ >O@@@@G8O#0@@@A8G @@@@@O@@@@@@@G? O@@@@A8G @@@@@O0?C30@@G8G#0@@@G >@@@@@@^A?@@@@@\?0^@@@@A<@@@@@@@G?@
@@@@@G @@@@@@A08? @@@@G.@@@@@@C7GN@@@@@G8C @@@@\_<@@@@@@^@@@@@@O@G8@@@@@@OG0@@@@@>@@@@@@@@@<@@@@@@C0@@@@@@@G @@@@C7@^@@@
@@@^@@@@@@C8G @@@@@@G @@@@@_@@>@@@@G @<@@@@C0@O @@@@O?0@@@@@O@_>@@@@C0@@@@@@@>A?0@@@@G@@<@@@@A0@O@@@@@<@@@@@@@N@<@@@@@@<
A?8@@@@N@@@@@@@O@@@<@@@C0A>@@@@@^@^@@@@@A8@@@@@@C8O?@@@@@G C>@@@@@@_@@@@@@A8@@@@@@O G @@@@@]0@@@@@@N8C>@@@A8@_@@@@@@G @@
@@@A????@@@A0?? @@@@G _ @@@@G@_@@@@@A18?0@@@@OA<@@@@@@8@@@@@@A?7 @@@@@G0G? @@@A<A?8@@@@^C8@@@@@C0@@@@@@A9>G @@@@OG0@@@@@
G8O0_8@@A<A?8@@@A8_@@@@@@G#?@@@@@G><O@@@@@_@@@@@@@@? @@@@@A0@@@@@@@\<G @@@@@; @@@@@@=13 @@@@C?#0@@@@G?O@@@@@@O@@@@@@C0C<
@@@@@@C?<@@@@@?@B@@@@@@@^@@@@@@@8@@@@@C G0@@@@@?0O @@@@@G @@@@@@<A8@@@@@@C0@@@@@O0@G @@@A8@_@@@@@<@G0@@@@C?0@@@@@C??? @@
@@<@@@@@@@G??\@@@@C0@O@@@@@<@C0@@@@O@@@@@@@G0^@@@@@@O@C>@@@@G O8@@@@C0@@O@@@@<@_ @@@@G0?@@@@@@^@@@@@@@_@? @@@@A8@O0@@@@@
O @@@@@@^@@@@@@A<G0@@@@@G<@@@@@@C>@?@@@@<@C<@@@@@A8@@@@@@???0@@@@\>_0@@@@A0_ @@@@A8_ @@@@@\<G8@@@@A?>@@@@@@N@@@@@@@_98@@
@@@A<A? @@@@_@_8@@@@G1<@@@@@@<@@@@@@@\O? @@@@C?8@@@@@A>C8G8@@@^@O8@@@@_? @@@@@A;?@@@@@A?OG @@@@G @@@@@@@O0@@@@@@\@@@@@@@
G>@@@@@@@O0@@@@@@G\O0@@@@A9?8@@@@@?# @@@@@G @@@@@@>C>@@@@@@_?>@@@@@_??0@@@@O@?@@@@@@@^@@@@@A<G8@@@@@G<O @@@@@C0@@@@@@O@>
@@@@@@@<@@@@@O8@A<@@@@^@_ @@@@O@G0@@@@@? @@@@@@??? @@@@O@@@@@@@@??N@@@@@<@C0@@@@O@@<@@@@C0@@@@@@A??@@@@@@C0@O@@@@A8?>@@@
@@<@@C0@@@O@C8@@@@@?? @@@@@G @@@@@@C?? @@@@@^@A?@@@@0_0@@@@@@G @@@@@@O?8@@@@@@>@@@@@@@?@G0@@@O@@_ @@@@@^@@@@@@O?0@@@@@G?
C0@@@@@\O @@@@@O? @@@@@G>A<@@@@@O>@@@@@@G @@@@@@C0^@@@@@@^@O @@@@G C8@@@@@?>@@@@@@O@@@@@@@G@?0@@@@@_8@@@@@@_@^@<@@@C@A<@
@@@C? @@@@@@\?@@@@@@OC#0@@@@A8@@@@@@@C0@@@@@@G@@@@@@@@?@@@@@@@A<@@@@@@A>C<@@@@@<O<@@@@@G 8@@@@@C?!! @@@@G?>@@@@@@??<@@@@@
O??<@@@@C?? @@@@@@G @@@@@_?<@@@@@@??0@@@@@@<@@@@@@C??@@@@@@@O@@@@@C<@@O@@@@G''?0@@@@A8O8@@@@@G @@@@@@O?0@@@@@C0@@@@@@@A?G
 @@@@O@@<@@@@C0@O@@@@@>@@@@@@@O? @@@@@@>@@@@@@@_?? @@@@O@@@?@@@C0@^@@@@@G? @@@@@A8@@@@@@@_?<@@@@@G0@O8@@@_?0@@@@@@A8@@@@
@@A?<@@@@@@O @@@@@@G0A8@@@C @C8@@@@@G @@@@@A0@@@@@@@?@@@@@@@GO0@@@@@C?0@@@@@A?@N@@@@@A>@@@@@@A8@@@@@@@@G@@@@@@O@@@@@@@C0
@@@@@@@O>@@@@@@C0@@@@@@A0G0@@@@@C8@@@@@@C0G@@@@@@@@@@@@@@_ @@@@@@GG@@@@@@@A;8@@@@@^@@@@@@@C8@@@@@@A0@@@@@@@O @@@@@@@^@@@
@@@@_ ^@@@@@_@<@@@@@@@^@@@@@A??8@@@@@?>@@@@@@O?0@@@@@G??>@@@@@?? @@@@@@A<@@@@@C?<@@@@@@G?0@@@@@@O@@@@@@@_? @@@@@@C0@@@@@
^@@A @@@@3?8@@@@@_?8@@@@@@@@@@@@@A8@@@@@@@<@@@@@@@@@A8@@@@C @O@@@@@8@C0@@@@G@@@@@@@@_0@@@@@@G @@@@@@G??0@@@@A @@G0@@@<@G
@@@@@@? @@@@@@^@@@@@@@A?O@@@@@@8@@<@@@G?0@@@@@@@N@@@@@@@G8@@@@@@A0@@@@@@A8@N@@@@P@@\@@@@@A8@@@@@@@@@@@@@@G@@@@@@@A''8@@@@
@@O @@@@@@O@@@@@@@@@@@@@@@@^@@@@@@@@A0@@@@@C0@@@@@@@<@@@@@@@A?@@@@@@@<@@@@@@@\@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@A0
@@@@@@@@_<@@@@@G@@@@@@@C<@@@@@@@^@@@@@@@@@@@@@@@@G @@@@@@C0C @@@@G @@@@@@@@G @@@@@??<@@@@@G<@@@@@@A>\@@@@@A>@@@@@@@G?@@@
@@@@@_@@@@@@O<@@@@@@@?0@@@@@@A @@@@@@A?@@@@@@@@X@@@@@@@@@@@@@@@?8@@@@@C?8@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@_@@@@@X@A @@@@
F@@X@@@@@@@@@@@@@@@@@@@@@@0@@@@@@A?8@@@@@@@@@@8@@@F@@@@@@@@@@@@@@@@C @@@@@@@@A8@@@@@@@@@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@N@@@@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@\@@@@@@8@@@@@@@N@@@@@@@@O@@@@@
@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@G>@@@@@A0@@@@@@C>@@@@@@@G0@@@@@@@@@@@@@@@@0@@@@@@@X@@@@@@A8
@@@@@@@@A0@@@@@OA<@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@
@@@@O0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@8@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@A8@@@@@@F@O@@@@@@F@@@@@@@A @@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@A?@@@@
@@@@@@@@@@?@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@\@@@@@C @@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@A0C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@_ @@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@C@O@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
O@@@@@@@N@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@O0@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@K0@@@@@@C8^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@FG @@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O!!8@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@_?@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@A?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@A? @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@<@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@G0@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@_@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@b') ; mask:((Depth1Image new) width: 3200; height: 53; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@X@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@
@@@@@@@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@<@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@G @@@@@@@@@@@@@@@@@@@@@@O@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@@@@@@C @@@@@@@@@@@C0@@@X@G@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@
@C @@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@O@@@@@@@@0@@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@?@@@@@@@A<@@@@@@@@@@@@@@@@G@@@@@@G8@@@@@@@O?@@@@@0@O@@@@@L@C0@@@@C@@@@@@@@@F@@@@@@X
@F@@@@@A @@@@@@@@@@O@@@@N@A8@@@@@C>@@@@@@C?8@@@@@@O<@@@@@@O<@@@@@@C>@@@@@@@@@@@@@@G @\@@@@8@@@@@@@@@@@@<@@@O@C0@@@@G@A<@
@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@C<@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@C0@@@@@@@^@@@@@@@^@@@@@@@G @@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@A?<@@@@@@A? @@@@@??0@@@@@N@O8@@
@@G??@@@@@@@_?0@@@@^@C0@@@@G @<@@@@A8@@@@@@@@C0@@@@@O@C0@@@@@<@@@@@@@G@@G0@@@G @^@@@@@C?8@@@@@_?? @@@@@G?@@@@@@??8@@@@@G
?8@@@@@@@G<@@@@A8@O@@@@O@@C@@@@@8@@@O@@@C0A<@@@@C8@^@@@@@L@C8@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@A? @@@@@@@@@@
@@@@<@@@@@@@O@@@@@@@A<@@@@@@@G @@@@@@G @@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A>@@@@@@@F@@@@@@@O @@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@C0@@@@@C??@@@@@@A?<@@@@@_?? @@@@O??>@@@@C??0@@@@@@O?<@@@@G @<@@@@A8@O@@@@@^@@@@@@@@@<@@@@@C0A<@@@@@O@@@@@
@@C8@C8@@@A<@G@@@@@A?? @@@@O??<@@@@@G?8@@@@@???@@@@@G??@@@@@@A?? @@@@^@C0@@@C0@@8@@@@N@@@C0@@@<@?@@@@@>@O@@@@@G???@@@@@@
@@@@@@@G @@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@^8@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@^@@@@@@@@0@@@@@@A8@@@@@@@^@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?8@@@@@@G0@@@@@
@O>@@@@@A0_0@@@@@@@^@@@@@@@@^@@@@@@?@@@@@@@@@ @@@@@C0@@@@@@@G @@@@@@A>@@@@@A??8@@@@@@?O@@@@@G??<@@@@C???@@@@@??8@@@@@@O<
F@@@@A8@O@@@@@^@C0@@@@G @@@@@@@@O@@@@@@<@^@@@@@C0@@@@@@@?@A>@@@@_@A0@@@@@>C<@@@@G<@O @@@@C<_@@@@@O0G8@@@@C<C0@@@@C???8@@
@@O@@<@@@@\@@^@@@@C0@@@<@@@G _@@@@@G0G @@@@A???0@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@ON@@@@@@@@@@@@@@C0@@@@@@@<
@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A??@@@@@@G<@@@@@@O?0@@@@@_?>@@@@@@@_ @@@@@A??0@@@@@_0@@@@@G??<@@@@@C?@@@@@@@O<@@@@@
@@_ @@@@@?0N@@@@@@_G0@@@@@<@_ @@@@??8@@@@@O0@@@@@@@G<@@@@@@^@C @@@@G @8@@@@C8@@@@@@@@C0@@@@@O@O @@@@@<@@@@@@@O0@_ @@@G0@
\@@@@@_@O@@@@A? A8@@@@A<C8@@@@C<@_@@@@A<@\@@@@A??0@@@@@C0@O@@@@G @G @@@@<@@@N@@@A8O @@@@@>A8@@@@@_?;8@@@@@@@@@@@@@<@@@@@
@@@@@@@@@@@@G@@@@@@@@@@@@@@C3 @@@@@@@@@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@??8@@@@@C>@@@@@@G8<@@@@@G?? @
@@@@@O0@@@@@@??<@@@@@O0@@@@@A???@@@@@A?8@@@@@@G?0@@@@@@O8@@@@@O0C @@@@@_C8@@@@@O@@<@@@@O@@@@@@@C0@@@@@@@C<@@@@@@G@@8@@@@
A0@N@@@@@<@@@@@@@@@<@@@@@C0G @@@@@N@@@@@@@C<@O8@@@A>@O@@@@@O A8@@@@A8@>@@@@@>@_@@@@@O@C0@@@@<@F@@@@@_?0@@@@@@<@C0@@@@8@C
8@@@@G@@@C @@@OG0@@@@@G0<@@@@@A?!!<@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A0@@@@@@8@@@@@@@80@@@@@@@@@@@@@@N@@@@@@@C @@@@@@@@@@@@@
@@@@@@@@@@A0@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@_@_@@@@@C? @@@@@C8G @@@@@?G0@@@@@@G<@@@@@@_?<@@@@@G0@@@@@@_?? @@@@A>O @@@@@G0<@@@@@@C?@@@@@C0A8@
@@@@G <@@@@@C @O@@@@C0@@@@@@@<@@@@@@@A>@@@@@@A0@^@@@@@\@G @@@@O@@@@@@@@@O@@@@@@<C8@@@@@G @@@@@@A7 G.@@@@_ C @@@@G0@^@@@@
@<@O@@@@@O@C0@@@@C0@<@@@@O@@@@@@@G<<@@@@@@O@A8@@@@O@@<@@@@A8@0@8@@@C38@@@@@A<^@@@@@@@@>@@@@@@@@@@@@@C @@@@@@@F@@@@@@@@^<
@@@@@@?@@@@@@@^H@@@@@@@_@@@@@@C @@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@\C @@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@C>@@@@
@@@@@@@@@@@8@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O C0@@@@C=8@@@@@A<A8@@@@@@C8@@@@@@G?@@@@@@O
@@@@@@@A8@@@@@@@@C0@@@@@>A<@@@@@C8O@@@@@@A90@@@@@<@<@@@@@C0G@@@@@A8@A8@@@@<@@@@@@@O@@@@@@@@>@@@@@@@<@G @@@@O@A8@@@@C0@@@
@@@@@C0@@@@@OA<@@@@@A8@@@@@@@]8C3 @@@O<@8@@@@A8@G @@@@O@G0@@@@G @^@@@@@<@O@@@@C0@@@@@@@@O@@@@@@C0@^@@@@C0@O@@@@@^@^@^@@@
@^<@@@@@@O''@@@@@@@@_@@@@@@G0@@@@@@8@@@@@@@G0@@@@@@@_/@@@@@@_8@@@@@@G @@@@@@@O8@@@@@@8@@@@@@@N@@@@@@@@@@@@@@@@N@@@@@@@GA8
@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@_0@@@@@@@G0@@@@@A? @@@@@@@@@@@@@@^@@@@@@@C @@@@@@@@C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@
@@@@@@@@@@G0@>@@@@A<^@@@@@@^@^@@@@@@C8@@@@@@C30@@@@@G @@@@@@@<@@@@@@@@A8@@@@@O@_@@@@@A<C0@@@@@@<^@@@@@O@^@@@@@A<@@@@@@@^
@@^@@@@O@@@@@@@G @@@@@@@_@@@@@@@N@A8@@@@C @^@@@@@<@@@@@@@@@<@@@@@C0^@@@@@@^@@@@@@@G^A=8@@@C?@N@@@@@<@A8@@@@C0G8@@@@C8@G 
@@@@O@G @@@@>@@@@@@@@C0@@@@@@<@G @@@@\@G @@@@G G G @@@C>@@@@@@A?0@@@@@@@O @@@@@G<@@@@@@NC0@@@@@A<@@@@@@@O? @@@@@ON@@@@@@
A8@@@@@@@O>@@@@@@N@@@@@@@C @@@@@@@\@@@@@@@G @@@@@@C0>@@@@@@<@@@@@@@GA0D@@@@A0@@@@@@@O>@@@@@@@C>@@@@@@>8@@@@@@8@@@@@@@O @
@@@@@A8A @@@@@\A0@@@@@N@@@@@@@G@\@0@@@@8C@@@@@@G@<@@@@@@A<@@@@@A8@G @@@@^G @@@@@C@G @@@@@A<@@@@@@A88@@@@@A0@@@@@@@^@@@@@
@@@@^@@@@@C0O @@@@@^A8@@@@@@OG @@@@C0O @@@@@>@@@@@@@G @G @@@C0A8@@@@A8@@@@@@@G @@@@@@C @\@@@@@8@G@@@@@^@@@@@@@@@O@@@@@@<
_@@@@@@G @@@@@@A30>^@@@@=8C @@@@O@@^@@@@@<G<@@@@@<@@<@@@@C0C8@@@@G0@@@@@@@@<@@@@@@^@A8@@@@G A8@@@@@8C<A0@@@@?@@@@@@@O8@@
@@@@@G0@@@@@C? @@@@@C#>@@@@@@? @@@@@@G38@@@@@C# @@@@@@\@@@@@@@G3 @@@@@C N@@@@@@8C @@@@@O@@@@@@@C8@@@@@@@<_@@@@@@N@@@@@@@
C0>C0@@@@\O@@@@@@G?8B@@@@CC?0@@@@@^O@@@@@@NC8@@@@@C0@@@@@@@^C8@@@@@O@\@@@@@G L@@@@@A0G@_A @@OA8@@@@@A0O@@@@@A?? @@@@@>@A
8@@@@@A8@@@@@@@C0@@@@@@<@@@@@@@<^@@@@@@<@@@@@@@O @@@@@@@@O@@@@@@>C0@@@@@O@^@@@@@@G 8@@@@@<_0@@@@@O@@@@@@@A8@C8@@@@??>@@@
@@^@<@@@@@C0@@@@@@@8@_@@@@@N@G0@@@@G @@@@@@@@C0@@@@@OO @@@@@A8@@@@@@@8<OG@@@@N_A8@@@@G @G @@@@NG>@@@@@_@@O@@@@@<C<@@@@A?
 @@@@@@@O@@@@@@G @^@@@@@8@<@@@@@N@?@\@@@@O @@@@@@A>@@@@@@@C8@@@@@A88@@@@@@9?0@@@@@\8@@@@@@A8>@@@@@A88@@@@@@O@@@@@@@C88@@
@@@@8O @@@@@NC8@@@@@C0@@@@@@@<@@@@@@@NO @@@@@G @@@@@@@<_!!>@@@@OG8@@@@@C8?? @@@A9<<@@@@@OC0@@@@@C#?@@@@@A8@H@@@@C??8@@@@@
C0O@C@@@A8G 8@@@@^C0O?8@@C0^@@@@@@<C0@@@@@_?8@@@@@O@@^@@@@@@\@@@@@@@A<@@@@@@_?@@@@@@>G @@@@@_?@@@@@@C0@@@@@@@@G @@@@@G;8
@@@@@C0O @@@@@C0O@@@@@OO?@@@@@G0@@@@@@@^@A<@@@@O?? @@@@G!!?@@@@@A<@C0@@@@O??0@@@@C??<@@@@A8@@@@@@@@@<@@@@@C70@@@@@@\@@@@@
@@NO''10@@@C#0\@@@@A8@C0@@@@C''>@@@@@G @C0@@@@OC>@@@@@G?@@@@@@@G @@@@@A8@G @@@@N@^@@@@@C0_0O@@@@G8@@@@@@@O@@@@@@@A<@@@@@@<
N@L@@@@\>^A @@@GN@L@@@@@<_ B@@@@\\@0@@@@C??@@@@@@<N@@@@@@NG8@@@@@C!!>@@@@@A<@@@@@@@O@@@@@@@C'' @@@@@A8@@@@@@@OO8? @@@C#>@A
@@@A<G?0@@@@^<O@0@@@G <@@@@@@9?0@@@@@\@F@@@@@?? @@@@@@<C0A0@@@^C?>@@@@G <C?<@@@<O@F@@@@N@<@@@@@C8\@@@@@G @G @@@@@G@@@@@@
@@^@@@@@@O?<@@@@@^A8@@@@@G?>@@@@@A8O8@@@@@@C0@@@@@@?<@@@@@@<G0@@@@@@<C0@@@@C3?<@@@@A8@@B@@@@G@@^@@@@C??@@@@@A??0@@@@@^@@
>@@@@C??<@@@@@???@@@@@^@@@@@@@@@O@@@@@@?8@@@@@@G@@@@@@@C!!98\@@@@8^G@@@@@<@@<@@@@A?>@@@@@A8@@<@@@@C?>@@@@@@?<@@@@@@A8@@@@
@@^@A8@@@@C0G @@@@@<G^C @@@C?@@@@@@@C0@@@@@@@>@@@@@@^C G@@@@G_C!!8@@@C!! O@@@@@^G8A @@@ON@\@@@@A?? @@@@@^C @@@@@C#?@C@@@@8
?0@0@@@^@A@@@@@C0@@@@@@@;0@@@@@@^@@@@@@@C3.N8@ @@9?0@0@@@^A78@@@@G.C0<@@@A0O@L@@@@^>@@@@@@G@C@@@@@O<@@@@@@@OA<@8@@@G ?>@
@@@A8_ ?<@@@OC0C @@@C O@@@@@@@N@@@@@A8@C8@@@@@C0@@@@@@@O @@@@@C??@@@@@O_?<@@@@@_?0@@@@@^O? @@@@@@<@@@@@@C<@@@@@@OG<@@@@@
@^G<@@@@@<8_@@@@@^@@A @@@A0@_ @@@A8@@@@@@@_?8@@@@@G @O @@@@?>O@@@@@O?#0@@@@G @@@@@@@@C0@@@@@O<@@@@@@C0@@@@@@A8_<O@@@@^G!!
0@@@@O@@^@@@@@_?@@@@@@<@@_@@@@@;?@@@@@@C?0@@@@@@^@@@@@@G @^@@@@@\C0@@@@@OA3 8@@@A=8@@@@@@@<@@@@@@@_@@@@@@GA0A0@@@A7@8<@@
@@8@G @@@@G@>@0@@@C/@N@@@@@_>@@@@@@O 8@@@@@@130A0@@@L\<@\@@@G @0@@@@@<@@@@@@@N8@@@@@@G A @@@@@93''.@X@@^<<@\@@@O@\@@@@@A?
@8>@@@@<G G@@@@G.@@@@@@A8C0@@@@@O@@@@@@@C _@\@@@@8N^@@@@@^G8\@@@@A98@0@@@A8C0@@@@@@G@@@@@@<@@<@@@@@@<@@@@@@@G0@@@@@@8C8@
@@@G??? @@@@@C>@@@@@O''?8@@@@@@^@@@@@@@?0@@@@@C??@@@@@@O?? @@@@N@A8@@@@O @@8@@@@<@_ @@@@^@@@@@@@G?@@@@@@C0@C0@@@@^@C0@@@@
G @<@@@@A8@@@@@@@@@<@@@@@C?8@@@@@@<@@@@@@@\C>C0@@@G <\@@@@C0@G @@@@G>@@@@@@O@@G @@@@^?8@@@@@@C>@@@@@@G @@@@@A8@G @@@@G 8
@@@@@A088\@@@@>O@@@@@@@O@@@@@@@O @@@@@C0\@8@@@@_ N>@@@@^@C0@@@@C0_ \@@@@? G@@@@@C0@@@@@@C0^@@@@@@\8<@8@@@GNO@N@@@A8@\@@@
@@O@@@@@@@C>@@@@@@A8@X@@@@@N<93 N@@G^O@N@@@C0G@@@@@@_0^_@@@@NC8C @@@A7@@@@@@@OA8@@@@@C @@@@@@@8O0^@@@@NG @@@@@G!!.G@@@@@O
<@X@@@@\A<@@@@@@C0@@@@@O@@_@@@@@@O@@@@@@@C8@@@@@@@@^@@@@A???8@@@@@@O @@@@C38O@@@@@@G @@@@@@_>@@@@@@_; @@@@@G??8@@@@G @^@
@@@C0@@<@@@@O@_0@@@@G @@@@@@A8@@@@@@@<@A<@@@@G@@<@@@@A0@O@@@@@>@@@@@@@@@N@@@@@@?? @@@@@O@@@@@@@G@? <@@@A8G''@@@@@<@C0@@@@
A>@@@@@@C0NC8@@@@G ?@@@@@@@O @@@@@A8@@@@@@^@C0@@@@@8^@@@@@@^NGG@@@@_A8@@@@@@C0@@@@@@G0@@@@@@8O@\@@@@G8C?@@@@G@A8@@@@@8N8
N@@@@O0C @@@@@<@@@@@@@<G @@@@@GNO@\@@@A3#0G@@@@^@N@@@@@C0@@@@@@@? @X@@@@^@L@@@@@C.N88G@@A7#0O@@@A8C0@@@@@G8G?@@@@GA>A8@@
@@]0@@@@@@C0<@@@@@@8@@@@@@@NG^O@@@@C10@@@@@@8;!!0@@@@C>@N@@@@G@?@@@@@@A8@@@@@C0@O @@@@@C0@@@@@@A<@@@@@@@@G @@@@O <N@@@@@@
A8@@@@@<<A0@@@@@C0@@@@@@OG0@@@@@C88@@@@@A?>N@@@@A8@G @@@@<@@^@@@@C0?8@@@@A8@@@@@@@<@@@@@@@O@A>@@@@A0@O@@@@@\@C0@@@@O @@@
@@@@@G @@@@@OG?@@@@@C0@@@@@@A0G0O@@@@^A? @@@@O@A8@@@@@^@@@@@@@<G!!<@@@@A8C<@@@@@@A8@@@@@@^@@@@@@G @<@@@@@NO@@@@@@G''A30@@@
O _@@@@@@@8@@@@@@C8@?8@@@^C0N@@@@A<A?@@@@A0@<@@@@@NGNG@@@@C8A0@@@@@O@@@@@@@^G8@@@@@A7C0N@@@@]0<C @@@G G@@@@@@<@@@@@@@O<@
N@@@@G O@@@@@@?#>NG @@_0\G @@@^A8@@@@@A<C?@@@@A0?@<@@@@G8@@@@@@@^^@@@@@@N@@@@@@@GA3? @@@@\\@@@@@@NN<8@@@@@_@G@@@@A0_0@@@
@@@<@@@@@@<@G0@@@@@@<@@@@@@A>@@@@@@@@A8@@@@@@O@@@@@@@@^@@@@@O^@<@@@@@@<@@@@@@G >@@@@@@@^@@@@@@?8C0@@@@^@C8@@@@O@@O@@@@@?
?8@@@@@<@@0@@@@O@@@@@@@C0@?@@@@@\@C0@@@@G@@<@@@@C0@@@@@@@@A0@@@@@C0_? @@@@8@@@@@@@<@8C0@@@G@O8@@@@C0@<@@@@@G @@@@@@OA<>@
@@@@^@_ @@@@@@>@@@@@@G @@@@@@<@O@@@@@C3 @@@@@A;0^8@@@G0C8@@@@@@^@@@@@@C<O?>@@@GA>O@@@@@^@?@@@@@\@^@@@@@GC3'' @@@@<A8@@@@@
C0@@@@@@G''>@@@@@@_ >O@@@@G8O#0@@@A8G @@@@@O@@@@@@@G? O@@@@A8G @@@@@O0?C30@@G8G#0@@@G >@@@@@@^A?@@@@@\?0^@@@@A<@@@@@@@G?@
@@@@@G @@@@@@A08? @@@@G.@@@@@@C7GN@@@@@G8C @@@@\_<@@@@@@^@@@@@@O@G8@@@@@@OG0@@@@@>@@@@@@@@@<@@@@@@C0@@@@@@@G @@@@C7@^@@@
@@@^@@@@@@C8G @@@@@@G @@@@@_@@>@@@@G @<@@@@C0@O @@@@O?0@@@@@O@_>@@@@C0@@@@@@@>A?0@@@@G@@<@@@@A0@O@@@@@<@@@@@@@N@<@@@@@@<
A?8@@@@N@@@@@@@O@@@<@@@C0A>@@@@@^@^@@@@@A8@@@@@@C8O?@@@@@G C>@@@@@@_@@@@@@A8@@@@@@O G @@@@@]0@@@@@@N8C>@@@A8@_@@@@@@G @@
@@@A????@@@A0?? @@@@G _ @@@@G@_@@@@@A18?0@@@@OA<@@@@@@8@@@@@@A?7 @@@@@G0G? @@@A<A?8@@@@^C8@@@@@C0@@@@@@A9>G @@@@OG0@@@@@
G8O0_8@@A<A?8@@@A8_@@@@@@G#?@@@@@G><O@@@@@_@@@@@@@@? @@@@@A0@@@@@@@\<G @@@@@; @@@@@@=13 @@@@C?#0@@@@G?O@@@@@@O@@@@@@C0C<
@@@@@@C?<@@@@@?@B@@@@@@@^@@@@@@@8@@@@@C G0@@@@@?0O @@@@@G @@@@@@<A8@@@@@@C0@@@@@O0@G @@@A8@_@@@@@<@G0@@@@C?0@@@@@C??? @@
@@<@@@@@@@G??\@@@@C0@O@@@@@<@C0@@@@O@@@@@@@G0^@@@@@@O@C>@@@@G O8@@@@C0@@O@@@@<@_ @@@@G0?@@@@@@^@@@@@@@_@? @@@@A8@O0@@@@@
O @@@@@@^@@@@@@A<G0@@@@@G<@@@@@@C>@?@@@@<@C<@@@@@A8@@@@@@???0@@@@\>_0@@@@A0_ @@@@A8_ @@@@@\<G8@@@@A?>@@@@@@N@@@@@@@_98@@
@@@A<A? @@@@_@_8@@@@G1<@@@@@@<@@@@@@@\O? @@@@C?8@@@@@A>C8G8@@@^@O8@@@@_? @@@@@A;?@@@@@A?OG @@@@G @@@@@@@O0@@@@@@\@@@@@@@
G>@@@@@@@O0@@@@@@G\O0@@@@A9?8@@@@@?# @@@@@G @@@@@@>C>@@@@@@_?>@@@@@_??0@@@@O@?@@@@@@@^@@@@@A<G8@@@@@G<O @@@@@C0@@@@@@O@>
@@@@@@@<@@@@@O8@A<@@@@^@_ @@@@O@G0@@@@@? @@@@@@??? @@@@O@@@@@@@@??N@@@@@<@C0@@@@O@@<@@@@C0@@@@@@A??@@@@@@C0@O@@@@A8?>@@@
@@<@@C0@@@O@C8@@@@@?? @@@@@G @@@@@@C?? @@@@@^@A?@@@@0_0@@@@@@G @@@@@@O?8@@@@@@>@@@@@@@?@G0@@@O@@_ @@@@@^@@@@@@O?0@@@@@G?
C0@@@@@\O @@@@@O? @@@@@G>A<@@@@@O>@@@@@@G @@@@@@C0^@@@@@@^@O @@@@G C8@@@@@?>@@@@@@O@@@@@@@G@?0@@@@@_8@@@@@@_@^@<@@@C@A<@
@@@C? @@@@@@\?@@@@@@OC#0@@@@A8@@@@@@@C0@@@@@@G@@@@@@@@?@@@@@@@A<@@@@@@A>C<@@@@@<O<@@@@@G 8@@@@@C?!! @@@@G?>@@@@@@??<@@@@@
O??<@@@@C?? @@@@@@G @@@@@_?<@@@@@@??0@@@@@@<@@@@@@C??@@@@@@@O@@@@@C<@@O@@@@G''?0@@@@A8O8@@@@@G @@@@@@O?0@@@@@C0@@@@@@@A?G
 @@@@O@@<@@@@C0@O@@@@@>@@@@@@@O? @@@@@@>@@@@@@@_?? @@@@O@@@?@@@C0@^@@@@@G? @@@@@A8@@@@@@@_?<@@@@@G0@O8@@@_?0@@@@@@A8@@@@
@@A?<@@@@@@O @@@@@@G0A8@@@C @C8@@@@@G @@@@@A0@@@@@@@?@@@@@@@GO0@@@@@C?0@@@@@A?@N@@@@@A>@@@@@@A8@@@@@@@@G@@@@@@O@@@@@@@C0
@@@@@@@O>@@@@@@C0@@@@@@A0G0@@@@@C8@@@@@@C0G@@@@@@@@@@@@@@_ @@@@@@GG@@@@@@@A;8@@@@@^@@@@@@@C8@@@@@@A0@@@@@@@O @@@@@@@^@@@
@@@@_ ^@@@@@_@<@@@@@@@^@@@@@A??8@@@@@?>@@@@@@O?0@@@@@G??>@@@@@?? @@@@@@A<@@@@@C?<@@@@@@G?0@@@@@@O@@@@@@@_? @@@@@@C0@@@@@
^@@A @@@@3?8@@@@@_?8@@@@@@@@@@@@@A8@@@@@@@<@@@@@@@@@A8@@@@C @O@@@@@8@C0@@@@G@@@@@@@@_0@@@@@@G @@@@@@G??0@@@@A @@G0@@@<@G
@@@@@@? @@@@@@^@@@@@@@A?O@@@@@@8@@<@@@G?0@@@@@@@N@@@@@@@G8@@@@@@A0@@@@@@A8@N@@@@P@@\@@@@@A8@@@@@@@@@@@@@@G@@@@@@@A''8@@@@
@@O @@@@@@O@@@@@@@@@@@@@@@@^@@@@@@@@A0@@@@@C0@@@@@@@<@@@@@@@A?@@@@@@@<@@@@@@@\@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@A0
@@@@@@@@_<@@@@@G@@@@@@@C<@@@@@@@^@@@@@@@@@@@@@@@@G @@@@@@C0C @@@@G @@@@@@@@G @@@@@??<@@@@@G<@@@@@@A>\@@@@@A>@@@@@@@G?@@@
@@@@@_@@@@@@O<@@@@@@@?0@@@@@@A @@@@@@A?@@@@@@@@X@@@@@@@@@@@@@@@?8@@@@@C?8@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@_@@@@@X@A @@@@
F@@X@@@@@@@@@@@@@@@@@@@@@@0@@@@@@A?8@@@@@@@@@@8@@@F@@@@@@@@@@@@@@@@C @@@@@@@@A8@@@@@@@@@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@N@@@@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@\@@@@@@8@@@@@@@N@@@@@@@@O@@@@@
@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@G>@@@@@A0@@@@@@C>@@@@@@@G0@@@@@@@@@@@@@@@@0@@@@@@@X@@@@@@A8
@@@@@@@@A0@@@@@OA<@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@
@@@@O0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@8@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@A8@@@@@@F@O@@@@@@F@@@@@@@A @@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@A?@@@@
@@@@@@@@@@?@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@\@@@@@C @@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@A0C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@_ @@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@C@O@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
O@@@@@@@N@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@O0@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@K0@@@@@@C8^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@FG @@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O!!8@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@_?@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@A?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@A? @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@<@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@G0@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@_@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@b') ; yourself); yourself]
!

charMapImage1_height
    ^ 53
!

charMapImage1_width
    ^ 22
!

charMapImage2
"character bitmaps (needed ebcause the webServer has no Display for drawing)"

    "
     self charMapImage2 inspect
     ImageEditor openOnClass:self andSelector:#charMapImage2
     Icon flushCachedIcons
    "

    <resource: #image>

    ^Icon
        constantNamed:'HumanReadableImageGenerator class charMapImage2'
        ifAbsentPut:[(Depth1Image new) width: 3200; height: 57; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@C?@@@@@@@@@@@
@@@@\@@@@@@@G@@@@@@@@@@@@@@@@N@@@@@@@G@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@G0@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@A?8@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@N@@@@@@@G0@@@@@@A0@@@@@@@\@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_
 @@@@@@@F@@@@@@@A @@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@^N@@@@@@@@@@@@@@C0@@@@@@@<
@@@@@@@G0@@@@@@A<@@@@@@@<@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@\@@@@@@@_8@@@@@@C?@@@@@C?>@@@@@@@C>@@@@@@A>@@@@@@@_>@@@@@L@C @@@@C@@8@@@@@ @@@@@@@@G@@@@@@G@@@@@@@@0@@@@@@@N@@\@@@@@@@
L@@@@@@? @@@@@@?0@@@@@@C>@@@@@@O<@@@@@@@?@@@@@@@@@@@@@@@0@F@@@@C @A8@@@@@@@@N@@@C@@L@@@@L@@O@@@@@@@A8@@@@@@@@@@@@@^@@@@@
@@@@@@@@@@@@C0@@@@@@@@@@@@@OC @@@@@@@@@@@@@@<@@@@@@@O@@@@@@@A<@@@@@@@_@@@@@@@O@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@A?? @@@@@C?8@@@@A??8@@@@@O?? @@@@A??0@@@@@@_? @@@@G @8@@@@
A8@N@@@@@\@@@@@@@@A0@@@@@A0A @@@@@^@@@@@@@G0@O @@@@8@G@@@@@@?>@@@@@C??@@@@@@C?0@@@@A??0@@@@@A?<@@@@@@@O?@@@@@^@C @@@@<@@
^@@@@F@@@C @@A8@G @@@G0@G0@@@@C??>@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@C08@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@_@@@@@
@@C @@@@@@C0@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@_@@@@@@@A @@@@@@G0@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G0@@@@@A??8@
@@@@A>N@@@@@_??@@@@@C??0@@@@@??8@@@@@@O?8@@@@A8@N@@@@@^@C @@@@G@@@@@@@@@\@@@@@@\@<@@@@@G @@@@@@A<@G8@@@@^@A0@@@@@_?0@@@@
C??8@@@@@A?<@@@@@_?>@@@@@A?? @@@@G???8@@@@G @8@@@@O@@O@@@@C0@@@8@@@^@C8@@@A>@A8@@@@A??? @@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@^
@@@@@@@@@@@@@@8L@@@@@@@@@@@@@@C0@@@@@@@<@@@@@@@C @@@@@@@@@@@@@@@<@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_<@@@@@@A<@@@@@@G?@@@@@@\G<@@@@@@@G @@@@@@
@G@@@@@@G0@@@@@@@A<@@@@@@<@@@@@@@C8@@@@@@A<@@@@@A? O@@@@@@>C @@@@A8A8@@@@@_0@@@@@@O<@@@@@@@G8\@@@@@^@C @@@@G @8@@@@A0@@@
@@@@@G@@@@@@G@_@@@@@A8@@@@@@@_@A>@@@@G0@\@@@@@_ >@@@@@? O@@@@@@>G @@@@C0G0@@@@@>A8@@@@C???>@@@@A8@^@@@@C0@G0@@@@<@@@N@@@
C0A<@@@@O0@<@@@@@_??0@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@^C@@@@@@@@@@@@@@@<@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@O@@
@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@_? @@@@@@?@@@@@@C?8@@@@@G??@@@@@@@G8@@@@@@??8@@@@@G<@@@@@A???@@@@@@?0@@@@@@C? @@@@@@? @@@@@_ C0@@@@@_@8@@@@@^
@O@@@@@C @@@@@@C8@@@@@@@C8C@@@@@G @8@@@@A8@N@@@@@<@@@@@@@@A0@@@@@A0G @@@@@\@@@@@@@G0@? @@@A<@G@@@@@G C @@@@O A0@@@@@^@<@
@@@@8@\@@@@@^@N@@@@@?? @@@@@@\@G @@@@<@A8@@@@O @@C @@@>@>@@@@A<@^@@@@@C?A8@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@
G @@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O#<@@@@@@? @@@@@C<N@@@@@A??0@@@@@@C<@@@@@@_?>@@@@@C<@@@@
@@_??0@@@@@_>@@@@@@C?<@@@@@@O8@@@@@G8@<@@@@@O ^@@@@@G@A0@@@@@8@@@@@@@<@@@@@@@A<@@@@@@A8@N@@@@@^@C @@@@O@@@@@@@@@\@@@@@@<
C8@@@@@O@@@@@@@A>@_0@@@@_@A @@@@C0@\@@@@A8@\@@@@@O@G@@@@@N@G@@@@@G@C@@@@@@@8@@@@@@O@A8@@@@G @<@@@@A8@@A8@@@G _@@@@@G O@@
@@@@@@<@@@@@@@@@@@@@C0@@@@@@@C0@@@@@@@@\@@@@@@O @@@@@A0@@@@@@@@@@@@@@@G @@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@^@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@C@_@@@@@@@<@@@@@@@@@@@@@@@_@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@X@@@@@@@@@@@@@@@@@@@@@@G G
 @@@@@?8@@@@@A>A0@@@@@O!!8@@@@@@A?@@@@@@O?>@@@@@A<@@@@@@G??8@@@@@_C<@@@@@A<G@@@@@@G>@@@@@@^@^@@@@@C0G @@@@A0@N@@@@@N@@@@@
@@O@@@@@@@@>@@@@@@@\@C @@@@G@@8@@@@C0@@@@@@@@G@@@@@@NA<@@@@@C0@@@@@@@? O\@@@@G8@8@@@@A8@G@@@@@^@G@@@@@G A8@@@@C A0@@@@C 
@@@@@@@@N@@@@@@C0@^@@@@A8@O@@@@@^@L@\@@@@<O @@@@A<G0@@@@@@@^@@@@@@C0@@@@@@8G @@@@@A>@@@@@@@@O@@@@@@G<@@@@@@\@@@@@@@@A<@@
@@@A8@@@@@@@^@@@@@@@@@@@@@@@@\@@@@@@@^@\@@@@@G@@@@@@@A @@@@@@@@@@@@@@@@O @@@@@A0O8@@@@@@? @@@@@C >@@@@@@_8@@@@@@G @@@@@@
C@@@@@@@C@A0@@@@@X@@C0@@@G O@@@@@@@G@@@@@@@O @@@@@C0@8@@@@@?.@@@@@@^@\@@@@@@A8@@@@@@A=0@@@@@G @@@@@@@^@@@@@@@@@<@@@@@O@_
@@@@@@<A0@@@@@C30@@@@@G O @@@@A8A0@@@@@\@C @@@@G@@@@@@@C0@@@@@@@_@@@@@@@O@@8@@@@C0@N@@@@@<@@@@@@@@A0@@@@@C >@@@@@@<@@@@@
@@O8C''@@@@A>@N@@@@@<@A0@@@@G@C0@@@@C0@N@@@@A0@<@@@@@8@@@@@@@@C @@@@@@8@G@@@@@^@G @@@@G G G@@@@O''0@@@@@O#8@@@@@@@O@@@@@@C
>@@@@@@NG<@@@@@@?0@@@@@@A;0@@@@@G? @@@@@G@@@@@@@@A? @@@@@^C @@@@@G 8@@@@@@0@@@@@@@O@@@@@@@G _@@@@@A0@@@@@@@\G@G @@@G@@@@
@@@@O<@@@@@@\O?@@@@@@_<@@@@@A8_0@@@@@O>@@@@@@_?<@@@@@A8G@@@@@A8@>@@@@@N@\@<@@@A8C0@@@@@FC0@@@@@_?8@@@@@A8@O@@@@@_#@@@@@@
C@G@@@@@@@<@@@@@@@>X@@@@@C0@@@@@@@O@@@@@@@@@^@@@@@C G0@@@@@^@\@@@@@@<<@@@@@A8G0@@@@@\@@@@@@@G@@8@@@@A0@@@@@@A8@@@@@@@O @
@@@@@C0@\@@@@@<@G@@@@@N@@@@@@@@@8@@@@@@8^@@@@@@O@@@@@@@C/A90@@@@?0C @@@@N@@\@@@@A0A8@@@@@8@C0@@@@\@^@@@@@N@@@@@@@@@8@@@@
@@N@A0@@@@C0A8@@@@A8A8A0@@@A;8@@@@@A=<@@@@@@@G0@@@@@A?8@@@@@G''? @@@@@\<@@@@@@A?8@@@@@C88@@@@@C?<@@@@@@@?<@@@@@G#<@@@@@A8
?@@@@@@\@@@@@@@C0@@@@@@A8O0@@@@@\@@@@@@@GC8C<@@@C0^@@@@@@G? @@@@@GG?0@@@@@OG @@@@@^O<@@@@@O0@@@@@@G??@@@@@@^A0@@@@@^@O @
@@@C O@_@@@@^A8@@@@@C0<@@@@@G?>@@@@@@<@A0@@@@G 0@@@@@@@A0@@@@@@\@@@@@@@^F@@@@@@8@@@@@@@G @@@@@@@@G@@@@@@8C8@@@@@G@N@@@@@
@^O@@@@@@\C8@@@@@O@@@@@@@A0@N@@@@@\G8@@@@@^@@@@@@@C0@@@@@@@<@G@@@@@O@A0@@@@G @@@@@@@@N@@@@@@N_@@@@@@C @@@@@@@90<\@@@@N>@
8@@@@G @G@@@@@\@<@@@@@^@@\@@@@G@O@@@@@C0@@@@@@@@\@@@@@@C @\@@@@@<@<@@@@@^@>@<@@@@O<@@@@@@O>@@@@@@@C8@@@@@@<^@@@@@A;18@@@
@@OO@@@@@@@?>@@@@@@<N@@@@@@??@@@@@@@>O@@@@@A9? @@@@@^_8@@@@@G@@@@@@@@<@@@@@@@^G8@@@@@G@@@@@@@A1?A?@@@@<_0@@@@@C08@@@@@A1
8<@@@@@G!!8@@@@@G''''@@@@@C0@@@@@@A?? @@@@@G <@@@@@G G0@@@@@8C0G @@@G >@@@@@@<O@@@@@A?/@@@@@@N@@\@@@@@@\@@@@@@@@<@@@@@@_>@@
@@@@OC @@@@@^@@@@@@@C8@@@@@@@@C @@@@@OA<@@@@@C C @@@@@G!!8@@@@@GG<@@@@@C @@@@@@@<@C @@@@G?>@@@@@G _@@@@@A8@G @@@@OC?0@@@@
C0?<@@@@A8@@@@@@@@C @@@@@C/ @@@@@@8@@@@@@@N\^G@@@@C/ L@@@@A0@A0@@@@G@?@@@@@G@@G@@@@A0O @@@@@_@@@@@@@@G@@@@@@A8@G@@@@@G@N
@@@@@G0O0N@@@@A>@@@@@@C?@@@@@@@A<@@@@@@^G @@@@@]8^@@@@@G!!0@@@@@@_G @@@@@^C @@@@@O?@@@@@@@^C0@@@@@\?8@@@@@GO>@@@@@A0@@@@@
@@O@@@@@@@G#<@@@@@A0@@@@@@@\?0?8@@@OO<@@@@@A8G@@@@@@\<O@@@@@C0^@@@@@A;0@@@@@A8@@@@@@@C0@@@@@@A0O@@@@@A8C8@@@@@NA<C0@@@@<
_@@@@@@^C0@@@@@O#0@@@@@G @G@@@@@@G@@@@@@@@N@@@@@@G?8@@@@@G 8@@@@@O? @@@@@@<@@@@@@@@A0@@@@@A<>@@@@@@8C8@@@@@C0^@@@@@C3? @
@@@A0@@@@@@@O@@8@@@@A?? @@@@A;?0@@@@@^@A8@@@@C??<@@@@@???@@@@@\@@@@@@@@@8@@@@@@?0@@@@@@N@@@@@@@G''''#0@@@@1<G@@@@@\@@8@@@@
C1?@@@@@C @A0@@@@]?0@@@@@G>@@@@@@@A0@@@@@@\@A0@@@@A8G @@@@@<G<C @@@@O@@@@@@@_ @@@@@@@>@@@@@@OA8@@@@@G<G @@@@A0X@@@@@@OA8
@@@@@OA0@@@@@A0@@@@@@@O@8@@@@@GO^@@@@@A37 @@@@@\@@@@@@@C0@@@@@@A1>@@@@@@X@@@@@@@GO\N>@@@C7'' @@@@@\A0@@@@@G^C0@@@@A8G @@@
@@\8@@@@@@^@@@@@@@@<@@@@@@@<C @@@@@^A<@@@@@C _@<@@@@OO @@@@@G <@@@@@@A8@@@@@A0@A0@@@@@A0@@@@@@@G @@@@@C?>@@@@@G0N@@@@@C?
?@@@@@@^C>@@@@@@@\@@@@@@O>@@@@@@OG<@@@@@A<G0@@@@@=?<@@@@@\@@@@@@@C @\@@@@@??@@@@@@_?8@@@@@G@@^@@@@@???@@@@@O??0@@@@G@@@@
@@@@@N@@@@@@O8@@@@@@G @@@@@@A9;08@@@@LO!!0@@@@N@@N@@@@@?? @@@@@8@@\@@@@G?0@@@@@@_8@@@@@@@\@@@@@@G@@\@@@@@^C0@@@@@OA?@8@@@
@G0@@@@@@C0@@@@@@@_@@@@@@G \@@@@@A>A8@@@@@<@@@@@@@G0^@@@@@C!!8@@@@@@\@@@@@@@G ^@@@@@C7# @@@@@=88@@@@@F@@@@@@@@8@@@@@@@\>@
@@@@@N@@@@@@@A7''G'' @@@=08@@@@@N@\@@@@@A?@<@@@@@\C0@@@@@G\@@@@@@G0@@@@@@@N@@@@@@@NA8@@@@@C ^@@@@@@8O0^@@@@A?0@@@@@A0O@@@@
@@@<@@@@@@\@@<@@@@@@X@@@@@@@C0@@@@@@8C0@@@@C3??@@@@@O?8@@@@@G#?8@@@@@@N@@@@@@A>@@@@@@C??@@@@@@^C<@@@@@O^G @@@@O@@@@@@@@8
@G@@@@@O @@@@@@G?8@@@@@C0@O@@@@@O>C0@@@@C? <@@@@A0@@@@@@@@C @@@@@C?@@@@@@A8@@@@@@@^O8N@@@@GC8\@@@@C @G@@@@@O? @@@@@\@@G@
@@@A/8@@@@@@A? @@@@@@G@@@@@@A0@G@@@@@C0<@@@@@C0=8\@@@@C>@@@@@@@<@@@@@@@O @@@@@A0O@@@@@@_@<@@@@@N@@@@@@@C8G@@@@@A9>@@@@@@
G@@@@@@@A0G @@@@@?08@@@@@O<N@@@@@A @@@@@@@N@@@@@@@G_@@@@@@C @@@@@@@91118@@@N<N@@@@@C G@@@@@@_ ^@@@@@O@<@@@@@A7@@@@@@A>@@
@@@@@G @@@@@@C ^@@@@@@<O@@@@@@NC>G @@@@_8@@@@@@\C0@@@@@@^@@@@@@N@@^@@@@@@F@@@@@@@A8@@@@@@@@\@@@@A???8@@@@@@?@@@@@C9?>@@@
@@@G@@@@@@@O8@@@@@@_;0@@@@@O??@@@@@C0@8@@@@C @@@@@@@N@C0@@@@C @@@@@@A<@@@@@@@8@C0@@@@G@@<@@@@A0@O@@@@@\@@@@@@@@A0@@@@@@?
8@@@@@@^@@@@@@@OC<C @@@A0_G@@@@@8@A0@@@@C?@@@@@@G@@C @@@@8?@@@@@@@C<@@@@@@A0@@@@@@\@C0@@@@@<^@@@@@@<N^G@@@@A;0@@@@@@O@@@
@@@@G0@@@@@@<C0@@@@@O O@@@@@C @@@@@@@<C0@@@@@_>@@@@@@A0@@@@@@@<C0@@@@@N8N@@@@@C.C @@@@@8@@@@@@@G @@@@@@C? @@@@@@8@@@@@@@
N<\8^@@@C>C @@@@A0A0@@@@@G8G @@@@C O@@@@@@? @@@@@@O8@@@@@@A8@@@@@@A8O @@@@@OC0@@@@@A!!;!!0@@@@C<@@@@@@NA<@@@@@@O@@@@@@C @O
 @@@@@A @@@@@@@<@@@@@@@@G@@@@@_??>@@@@@@C0@@@@@<>C0@@@@@A0@@@@@@G?@@@@@@C88@@@@@G??8@@@@@<@N@@@@@8@@@@@@@C A8@@@@@8@@@@@
@@\@@@@@@@N@A8@@@@A0@O@@@@@\@C0@@@@G@@@@@@@@@\@@@@@@N?@@@@@@G@@@@@@@C0^@8@@@@\C90@@@@N@@8@@@@@>@@@@@@A0GA8@@@@NC8@@@@@@@
O @@@@@@\@@@@@@G@@<@@@@@OG@@@@@@OG''3 @@@@<^@@@@@@C0@@@@@@C8@@@@@@NA<@@@@@C0G0@@@@A0@@@@@@@OA<@@@@@G?@@@@@@@\@@@@@@@NA<@@
@@@C<C @@@@@?@8@@@@@N@@@@@@@A8@@@@@@@?8@@@@@@N@@@@@@@C.G^G @@@?@8@@@@@\@8@@@@@A<C0@@@@A8G @@@@@O0@@@@@@@? @@@@@@\@@@@@@@
\C8@@@@@A18@@@@@@\\8<@@@@@>@@@@@@C _@@@@@@G @@@@@@8@G0@@@@@@8@@@@@@@^@@@@@@@@A0@@@@C<GC @@@@@@\@@@@@OO@\@@@@@@8@@@@@@C18
@@@@@@@N@@@@@A?8^@@@@@N@C @@@@N@@@@@@@@8@<@@@@@\@@@@@@@O@@@@@@@C @>@@@@@\@C0@@@@G@@<@@@@A0@@@@@@G G@@@@@@CC8@@@@@A0@@@@@
@@<G@N@@@@G@>8@@@@C @^@@@@@N@@@@@@@\C0<@@@@C _ @@@@@@@8@@@@@@G@@@@@@A0@N@@@@@A;0@@@@@A90<8@@@@^G0@@@@@@8@@@@@@A<@@@@@@G@
?@@@@@@<C8@@@@@\@@@@@@@G _@@@@@A>@0@@@@@G@@@@@@@G@?@@@@@@?@<@@@@@O0O@@@@@C @@@@@@@^@@@@@@@O?@@@@@@C @@@@@@@?!!7A8@@@O N@@
@@@G@N@@@@@@_@<@@@@@\C8@@@@@C<@@@@@@@G<@@@@@@G@@@@@@@GA>@@@@@@^<@@@@@@GOOO@@@@@?0@@@@@@8O0@@@@@C0@@@@@@N@C8@@@@@@NC0@@@@
@_@@@@@@@@@8@@@@@@A0@@@@@@@G@@@@@C7 O@@@@@@N@@@@@@A8O@@@@@@@C @@@@@? C0@@@@C A8@@@@C @0@@@@@N@>@@@@@G@@@@@@@C0@@@@@@@8@_
 @@@@G@@<@@@@A0@O@@@@@\@@@@@@A<C @@@@@A0_ @@@@@\@@@@@@@O@@C @@@A G>@@@@@8@O@@@@@G@@@@@@@G@>^@@@@@8C<@@@@@@@N@@@@@@A0@@@@
@@\@C @@@@@_8@@@@@@^8OL@@@@O@>@@@@@@N@@@@@@@>@@@@@@A0_0@@@@@NA<@@@@@G@F@@@@@A8O0@@@@@\@\@@@@@C0@@@@@@A0_0@@@@@O O@@@@@C8
C0@@@@@8@@@@@@@G @@@@@@C/8@@@@@@8@@@@@@@O0_0_@@@G8C0@@@@A0G @@@@@G ^@@@@@GA>@@@@@@?@@@@@@@@_ @@@@@A0@@@@@@A0? @@@@@G/@@@
@@@A3#; @@@@_>@@@@@@NG<@@@@@A<@@@@@@C A<@@@@@@C''<@@@@@O @@@@@@@@N@@@@@@@\@@@@@@@C0@@@@@=0G @@@@@G@@@@@@@<C0@@@@@@A8@@@@@
_@@>@@@@@8@\@@@@@8@\@@@@@C ?@@@@@A0@@@@@@@<@@@@@@@N@_0@@@@A0@O@@@@@\@C0@@@@G@@@@@@@_!!8@@@@@@\A<@@@@@G@C @@@@G @@8@@@@8@?
 @@@@G@O @@@@A0@@@@@@A8G?@@@@@N@_ @@@@@@G @@@@@@\@@@@@@C A8@@@@@C>@@@@@@G.C?@@@@G G0@@@@@C @@@@@@_@@@@@@@\O<@@@@@C >@@@@
@A0G @@@@@^G\@@@@@G@N@@@@@@<@@@@@@@\_<@@@@@G0C8@@@@A<@>@@@@@N@@@@@@@A8@@@@@@@1?@@@@@@N@@@@@@@C8G8G0@@A<@<@@@@@\C0@@@@@C8
O@@@@@A0? @@@@@O @@@@@@@A8@@@@@@\C@@@@@@\^8@@@@@@? @@@@@@M8_8@@@@O''0@@@@@C''?@@@@@@>@@@@@@@8A>@@@@@@@?<@@@@@O0A0@@@@F@O@@
@@@@@G@@@@@A0A8@@@@@O<C8@@@@@A0@@@@@@N@<@@@@@@@\@@@@@G0@G @@@@^@_@@@@@O@^@@@@@@9? @@@@@<A<@@@@@O@@@@@@@C0_\@@@@@<@C0@@@@
O@@<@@@@A0@@@@@@C?>@@@@@@G@O0@@@@C??<@@@@A8@@N@@@@N@G8@@@@A8O0@@@@@\@@@@@@@O@? @@@@C C>@@@@@@G0@@@@@@G@@@@@@@<@<@@@@@@?@
@@@@@A?@_0@@@C8@>@@@@@@8@@@@@@O!!?8@@@@GO7 @@@@@X_@@@@@@^G0@@@@@G#''L@@@@A8O @@@@@O@@@@@@@G?/@@@@@A<@^@@@@@_@G @@@@C @@@@@
@@^@@@@@@@LO8@@@@@C @@@@@@@>A>@>@@@_@G8@@@@G#8@@@@@@>G @@@@@\?8@@@@@C8@@@@@@@@^@@@@@@GC0@@@@@GO.@@@@@@O0@@@@@@C<G<@@@@C0
>@@@@@@?= @@@@@_O8@@@@@OA?@@@@@@G?0@@@@@G??<@@@@C0O @@@@@@A0@@@@@>A<@@@@@A?C8@@@@@@8@@@@@@C _@@@@@@@G@@@@@C8@A<@@@@G O @
@@@A8_ @@@@@O? @@@@@O?? @@@@C0@@@@@@@_?''@@@@@O@@<@@@@C0@O@@@@@\@@@@@@@_?@@@@@@A0A? @@@@???@@@@@^@@C @@@C A>@@@@@O?0@@@@@
G@@@@@@@A??0@@@@@8@O0@@@@@G8@@@@@@A0@@@@@@G >@@@@@@O @@@@@@_0G8@@@A<@G0@@@@@N@@@@@@G???@@@@A?1<@@@@@G? @@@@@C?8@@@@@A?1?
@@@@@O?0@@@@@C0@@@@@@A?#0@@@@@^@G0@@@@G A<@@@@@8@@@@@@@G @@@@@@GA?@@@@@@8@@@@@@@O@O@O0@@G A>@@@@@?<@@@@@@O?0@@@@@G?\@@@@
@@>@@@@@@@@_@@@@@@A?8@@@@@A?30@@@@@C<@@@@@@@?@?@@@@A8G8@@@@@G<X@@@@@G??@@@@@A??@@@@@@O?8@@@@@C???@@@@@??0@@@@@@@<@@@@@O?
>@@@@@@O?<@@@@@@N@@@@@@@?? @@@@@@A0@@@@@<@@O @@@A9?0@@@@@_? @@@@@G? @@@@@C??8@@@@@<@@@@@@@C?!!0@@@@C0@O@@@@@<@C0@@@@G@@@@
@@@A? @@@@@@\@G8@@@@O?? @@@@G @@8@@@@8@O @@@@A?8@@@@@A0@@@@@@@O?>@@@@@N@A<@@@@G?<@@@@@@@\@@@@@@@??@@@@@@A8@@@@@@C8@>@@@@
^@@<@@@@@C @@@@@C???0@@@@O8_@@@@@A?0@@@@@@?<@@@@@@O8O @@@@A?0@@@@@@<@@@@@@@O 8@@@@@G @8@@@@A8@N@@@@@N@@@@@@@A8@@@@@@A0O>
@@@@@N@@@@@@@C0C C<@@A8@N@@@@@G>@@@@@@C?0@@@@@@?''@@@@@@O @@@@@@O? @@@@@@O<@@@@@@O8^@@@@@@^@@@@@@@G G0@@@@>@>@@@@@@<F@@@@
@C??0@@@@@O?@@@@@@C?<@@@@@A???@@@@@O?0@@@@@@@O @@@@A?>@@@@@@A?<@@@@@@C @@@@@@G?0@@@@@@@\@@@@@F@@A8@@@@^?8@@@@@A?0@@@@@C?
 @@@@@@_?8@@@@@O@@@@@@@@_ <@@@@@8@C0@@@@N@@<@@@@@0@@@@@@@G @@@@@@G@@\@@@@A<@@@@@@@8@@F@@@@N@A0@@@@@O8@@@@@@\@@@@@@@@?'' @
@@@C @N@@@@A?>@@@@@@@C@@@@@@@G? @@@@@@\@@@@@@@>@O@@@@O @F@@@@@@8@@@@@@???<@@@@A8C @@@@@G0@@@@@@C8@@@@@@A8C0@@@@@O0@@@@@@
O@@@@@@@@@N@@@@@@0@@@@@@@L@@@@@@@C @@@@@@@^@@@@@@@\A? @@@@C @@@@@@@<@@@^@@@L@@@@@@@@>@@@@@@@?0@@@@@@G!!0@@@@@A0@@@@@@G?0@
@@@@@A<@@@@@@A8G@@@@@@G @@@@@@A8@8@@@@O@G@@@@@@@C @@@@A?@H@@@@@A?@@@@@@@_''@@@@@@_ @@@@@@@? @@@@@@@C8@@@@@G>@@@@@@@O<@@@@
@@@X@@@@@@@_ @@@@@@@C@@@@@@@@@L@@@@CO8@@@@@@O0@@@@@@>@@@@@@@@@@@@@@@A0@@@@@@@@@O@@@@@F@@P@@@@A @D@@@@@@@@@@@@@@@@@@@@@A 
@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@F@@@@@@@@@@<@@@@@X@@@@@@@_>@@@@@@@@@@@@@@@@_ @@@@@@@@@@@@@@O@A0@@@C0@@@@@@@@L@@@
@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@X@@@@@@@G @@@@@@F@G0@@@@@8@@@@@@@
F@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@\@@@@@@\@@@@@@@? @@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@L@@@@@@C0@@@@@@@@@8@@@@@>@@@@@@@@@@@
@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@G@@@@@@@@@@@@@@
@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@C<
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@A8@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@L@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@A0@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@G@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@0A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@FC0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@
@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
A9<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@C8^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@C?<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O>@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@8@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O>@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C? @@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A? @@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@C @@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ @@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@') ; mask:((Depth1Image new) width: 3200; height: 57; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@C?@@@@@@@@@@@
@@@@\@@@@@@@G@@@@@@@@@@@@@@@@N@@@@@@@G@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@G0@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@A?8@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@N@@@@@@@G0@@@@@@A0@@@@@@@\@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_
 @@@@@@@F@@@@@@@A @@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@^N@@@@@@@@@@@@@@C0@@@@@@@<
@@@@@@@G0@@@@@@A<@@@@@@@<@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@\@@@@@@@_8@@@@@@C?@@@@@C?>@@@@@@@C>@@@@@@A>@@@@@@@_>@@@@@L@C @@@@C@@8@@@@@ @@@@@@@@G@@@@@@G@@@@@@@@0@@@@@@@N@@\@@@@@@@
L@@@@@@? @@@@@@?0@@@@@@C>@@@@@@O<@@@@@@@?@@@@@@@@@@@@@@@0@F@@@@C @A8@@@@@@@@N@@@C@@L@@@@L@@O@@@@@@@A8@@@@@@@@@@@@@^@@@@@
@@@@@@@@@@@@C0@@@@@@@@@@@@@OC @@@@@@@@@@@@@@<@@@@@@@O@@@@@@@A<@@@@@@@_@@@@@@@O@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@A?? @@@@@C?8@@@@A??8@@@@@O?? @@@@A??0@@@@@@_? @@@@G @8@@@@
A8@N@@@@@\@@@@@@@@A0@@@@@A0A @@@@@^@@@@@@@G0@O @@@@8@G@@@@@@?>@@@@@C??@@@@@@C?0@@@@A??0@@@@@A?<@@@@@@@O?@@@@@^@C @@@@<@@
^@@@@F@@@C @@A8@G @@@G0@G0@@@@C??>@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@C08@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@_@@@@@
@@C @@@@@@C0@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@_@@@@@@@A @@@@@@G0@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G0@@@@@A??8@
@@@@A>N@@@@@_??@@@@@C??0@@@@@??8@@@@@@O?8@@@@A8@N@@@@@^@C @@@@G@@@@@@@@@\@@@@@@\@<@@@@@G @@@@@@A<@G8@@@@^@A0@@@@@_?0@@@@
C??8@@@@@A?<@@@@@_?>@@@@@A?? @@@@G???8@@@@G @8@@@@O@@O@@@@C0@@@8@@@^@C8@@@A>@A8@@@@A??? @@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@^
@@@@@@@@@@@@@@8L@@@@@@@@@@@@@@C0@@@@@@@<@@@@@@@C @@@@@@@@@@@@@@@<@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_<@@@@@@A<@@@@@@G?@@@@@@\G<@@@@@@@G @@@@@@
@G@@@@@@G0@@@@@@@A<@@@@@@<@@@@@@@C8@@@@@@A<@@@@@A? O@@@@@@>C @@@@A8A8@@@@@_0@@@@@@O<@@@@@@@G8\@@@@@^@C @@@@G @8@@@@A0@@@
@@@@@G@@@@@@G@_@@@@@A8@@@@@@@_@A>@@@@G0@\@@@@@_ >@@@@@? O@@@@@@>G @@@@C0G0@@@@@>A8@@@@C???>@@@@A8@^@@@@C0@G0@@@@<@@@N@@@
C0A<@@@@O0@<@@@@@_??0@@@@@@@@@@@@@<@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@@^C@@@@@@@@@@@@@@@<@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@O@@
@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@_? @@@@@@?@@@@@@C?8@@@@@G??@@@@@@@G8@@@@@@??8@@@@@G<@@@@@A???@@@@@@?0@@@@@@C? @@@@@@? @@@@@_ C0@@@@@_@8@@@@@^
@O@@@@@C @@@@@@C8@@@@@@@C8C@@@@@G @8@@@@A8@N@@@@@<@@@@@@@@A0@@@@@A0G @@@@@\@@@@@@@G0@? @@@A<@G@@@@@G C @@@@O A0@@@@@^@<@
@@@@8@\@@@@@^@N@@@@@?? @@@@@@\@G @@@@<@A8@@@@O @@C @@@>@>@@@@A<@^@@@@@C?A8@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@
G @@@@@@@@@@@@@@@O@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O#<@@@@@@? @@@@@C<N@@@@@A??0@@@@@@C<@@@@@@_?>@@@@@C<@@@@
@@_??0@@@@@_>@@@@@@C?<@@@@@@O8@@@@@G8@<@@@@@O ^@@@@@G@A0@@@@@8@@@@@@@<@@@@@@@A<@@@@@@A8@N@@@@@^@C @@@@O@@@@@@@@@\@@@@@@<
C8@@@@@O@@@@@@@A>@_0@@@@_@A @@@@C0@\@@@@A8@\@@@@@O@G@@@@@N@G@@@@@G@C@@@@@@@8@@@@@@O@A8@@@@G @<@@@@A8@@A8@@@G _@@@@@G O@@
@@@@@@<@@@@@@@@@@@@@C0@@@@@@@C0@@@@@@@@\@@@@@@O @@@@@A0@@@@@@@@@@@@@@@G @@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@^@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@C@_@@@@@@@<@@@@@@@@@@@@@@@_@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@X@@@@@@@@@@@@@@@@@@@@@@G G
 @@@@@?8@@@@@A>A0@@@@@O!!8@@@@@@A?@@@@@@O?>@@@@@A<@@@@@@G??8@@@@@_C<@@@@@A<G@@@@@@G>@@@@@@^@^@@@@@C0G @@@@A0@N@@@@@N@@@@@
@@O@@@@@@@@>@@@@@@@\@C @@@@G@@8@@@@C0@@@@@@@@G@@@@@@NA<@@@@@C0@@@@@@@? O\@@@@G8@8@@@@A8@G@@@@@^@G@@@@@G A8@@@@C A0@@@@C 
@@@@@@@@N@@@@@@C0@^@@@@A8@O@@@@@^@L@\@@@@<O @@@@A<G0@@@@@@@^@@@@@@C0@@@@@@8G @@@@@A>@@@@@@@@O@@@@@@G<@@@@@@\@@@@@@@@A<@@
@@@A8@@@@@@@^@@@@@@@@@@@@@@@@\@@@@@@@^@\@@@@@G@@@@@@@A @@@@@@@@@@@@@@@@O @@@@@A0O8@@@@@@? @@@@@C >@@@@@@_8@@@@@@G @@@@@@
C@@@@@@@C@A0@@@@@X@@C0@@@G O@@@@@@@G@@@@@@@O @@@@@C0@8@@@@@?.@@@@@@^@\@@@@@@A8@@@@@@A=0@@@@@G @@@@@@@^@@@@@@@@@<@@@@@O@_
@@@@@@<A0@@@@@C30@@@@@G O @@@@A8A0@@@@@\@C @@@@G@@@@@@@C0@@@@@@@_@@@@@@@O@@8@@@@C0@N@@@@@<@@@@@@@@A0@@@@@C >@@@@@@<@@@@@
@@O8C''@@@@A>@N@@@@@<@A0@@@@G@C0@@@@C0@N@@@@A0@<@@@@@8@@@@@@@@C @@@@@@8@G@@@@@^@G @@@@G G G@@@@O''0@@@@@O#8@@@@@@@O@@@@@@C
>@@@@@@NG<@@@@@@?0@@@@@@A;0@@@@@G? @@@@@G@@@@@@@@A? @@@@@^C @@@@@G 8@@@@@@0@@@@@@@O@@@@@@@G _@@@@@A0@@@@@@@\G@G @@@G@@@@
@@@@O<@@@@@@\O?@@@@@@_<@@@@@A8_0@@@@@O>@@@@@@_?<@@@@@A8G@@@@@A8@>@@@@@N@\@<@@@A8C0@@@@@FC0@@@@@_?8@@@@@A8@O@@@@@_#@@@@@@
C@G@@@@@@@<@@@@@@@>X@@@@@C0@@@@@@@O@@@@@@@@@^@@@@@C G0@@@@@^@\@@@@@@<<@@@@@A8G0@@@@@\@@@@@@@G@@8@@@@A0@@@@@@A8@@@@@@@O @
@@@@@C0@\@@@@@<@G@@@@@N@@@@@@@@@8@@@@@@8^@@@@@@O@@@@@@@C/A90@@@@?0C @@@@N@@\@@@@A0A8@@@@@8@C0@@@@\@^@@@@@N@@@@@@@@@8@@@@
@@N@A0@@@@C0A8@@@@A8A8A0@@@A;8@@@@@A=<@@@@@@@G0@@@@@A?8@@@@@G''? @@@@@\<@@@@@@A?8@@@@@C88@@@@@C?<@@@@@@@?<@@@@@G#<@@@@@A8
?@@@@@@\@@@@@@@C0@@@@@@A8O0@@@@@\@@@@@@@GC8C<@@@C0^@@@@@@G? @@@@@GG?0@@@@@OG @@@@@^O<@@@@@O0@@@@@@G??@@@@@@^A0@@@@@^@O @
@@@C O@_@@@@^A8@@@@@C0<@@@@@G?>@@@@@@<@A0@@@@G 0@@@@@@@A0@@@@@@\@@@@@@@^F@@@@@@8@@@@@@@G @@@@@@@@G@@@@@@8C8@@@@@G@N@@@@@
@^O@@@@@@\C8@@@@@O@@@@@@@A0@N@@@@@\G8@@@@@^@@@@@@@C0@@@@@@@<@G@@@@@O@A0@@@@G @@@@@@@@N@@@@@@N_@@@@@@C @@@@@@@90<\@@@@N>@
8@@@@G @G@@@@@\@<@@@@@^@@\@@@@G@O@@@@@C0@@@@@@@@\@@@@@@C @\@@@@@<@<@@@@@^@>@<@@@@O<@@@@@@O>@@@@@@@C8@@@@@@<^@@@@@A;18@@@
@@OO@@@@@@@?>@@@@@@<N@@@@@@??@@@@@@@>O@@@@@A9? @@@@@^_8@@@@@G@@@@@@@@<@@@@@@@^G8@@@@@G@@@@@@@A1?A?@@@@<_0@@@@@C08@@@@@A1
8<@@@@@G!!8@@@@@G''''@@@@@C0@@@@@@A?? @@@@@G <@@@@@G G0@@@@@8C0G @@@G >@@@@@@<O@@@@@A?/@@@@@@N@@\@@@@@@\@@@@@@@@<@@@@@@_>@@
@@@@OC @@@@@^@@@@@@@C8@@@@@@@@C @@@@@OA<@@@@@C C @@@@@G!!8@@@@@GG<@@@@@C @@@@@@@<@C @@@@G?>@@@@@G _@@@@@A8@G @@@@OC?0@@@@
C0?<@@@@A8@@@@@@@@C @@@@@C/ @@@@@@8@@@@@@@N\^G@@@@C/ L@@@@A0@A0@@@@G@?@@@@@G@@G@@@@A0O @@@@@_@@@@@@@@G@@@@@@A8@G@@@@@G@N
@@@@@G0O0N@@@@A>@@@@@@C?@@@@@@@A<@@@@@@^G @@@@@]8^@@@@@G!!0@@@@@@_G @@@@@^C @@@@@O?@@@@@@@^C0@@@@@\?8@@@@@GO>@@@@@A0@@@@@
@@O@@@@@@@G#<@@@@@A0@@@@@@@\?0?8@@@OO<@@@@@A8G@@@@@@\<O@@@@@C0^@@@@@A;0@@@@@A8@@@@@@@C0@@@@@@A0O@@@@@A8C8@@@@@NA<C0@@@@<
_@@@@@@^C0@@@@@O#0@@@@@G @G@@@@@@G@@@@@@@@N@@@@@@G?8@@@@@G 8@@@@@O? @@@@@@<@@@@@@@@A0@@@@@A<>@@@@@@8C8@@@@@C0^@@@@@C3? @
@@@A0@@@@@@@O@@8@@@@A?? @@@@A;?0@@@@@^@A8@@@@C??<@@@@@???@@@@@\@@@@@@@@@8@@@@@@?0@@@@@@N@@@@@@@G''''#0@@@@1<G@@@@@\@@8@@@@
C1?@@@@@C @A0@@@@]?0@@@@@G>@@@@@@@A0@@@@@@\@A0@@@@A8G @@@@@<G<C @@@@O@@@@@@@_ @@@@@@@>@@@@@@OA8@@@@@G<G @@@@A0X@@@@@@OA8
@@@@@OA0@@@@@A0@@@@@@@O@8@@@@@GO^@@@@@A37 @@@@@\@@@@@@@C0@@@@@@A1>@@@@@@X@@@@@@@GO\N>@@@C7'' @@@@@\A0@@@@@G^C0@@@@A8G @@@
@@\8@@@@@@^@@@@@@@@<@@@@@@@<C @@@@@^A<@@@@@C _@<@@@@OO @@@@@G <@@@@@@A8@@@@@A0@A0@@@@@A0@@@@@@@G @@@@@C?>@@@@@G0N@@@@@C?
?@@@@@@^C>@@@@@@@\@@@@@@O>@@@@@@OG<@@@@@A<G0@@@@@=?<@@@@@\@@@@@@@C @\@@@@@??@@@@@@_?8@@@@@G@@^@@@@@???@@@@@O??0@@@@G@@@@
@@@@@N@@@@@@O8@@@@@@G @@@@@@A9;08@@@@LO!!0@@@@N@@N@@@@@?? @@@@@8@@\@@@@G?0@@@@@@_8@@@@@@@\@@@@@@G@@\@@@@@^C0@@@@@OA?@8@@@
@G0@@@@@@C0@@@@@@@_@@@@@@G \@@@@@A>A8@@@@@<@@@@@@@G0^@@@@@C!!8@@@@@@\@@@@@@@G ^@@@@@C7# @@@@@=88@@@@@F@@@@@@@@8@@@@@@@\>@
@@@@@N@@@@@@@A7''G'' @@@=08@@@@@N@\@@@@@A?@<@@@@@\C0@@@@@G\@@@@@@G0@@@@@@@N@@@@@@@NA8@@@@@C ^@@@@@@8O0^@@@@A?0@@@@@A0O@@@@
@@@<@@@@@@\@@<@@@@@@X@@@@@@@C0@@@@@@8C0@@@@C3??@@@@@O?8@@@@@G#?8@@@@@@N@@@@@@A>@@@@@@C??@@@@@@^C<@@@@@O^G @@@@O@@@@@@@@8
@G@@@@@O @@@@@@G?8@@@@@C0@O@@@@@O>C0@@@@C? <@@@@A0@@@@@@@@C @@@@@C?@@@@@@A8@@@@@@@^O8N@@@@GC8\@@@@C @G@@@@@O? @@@@@\@@G@
@@@A/8@@@@@@A? @@@@@@G@@@@@@A0@G@@@@@C0<@@@@@C0=8\@@@@C>@@@@@@@<@@@@@@@O @@@@@A0O@@@@@@_@<@@@@@N@@@@@@@C8G@@@@@A9>@@@@@@
G@@@@@@@A0G @@@@@?08@@@@@O<N@@@@@A @@@@@@@N@@@@@@@G_@@@@@@C @@@@@@@91118@@@N<N@@@@@C G@@@@@@_ ^@@@@@O@<@@@@@A7@@@@@@A>@@
@@@@@G @@@@@@C ^@@@@@@<O@@@@@@NC>G @@@@_8@@@@@@\C0@@@@@@^@@@@@@N@@^@@@@@@F@@@@@@@A8@@@@@@@@\@@@@A???8@@@@@@?@@@@@C9?>@@@
@@@G@@@@@@@O8@@@@@@_;0@@@@@O??@@@@@C0@8@@@@C @@@@@@@N@C0@@@@C @@@@@@A<@@@@@@@8@C0@@@@G@@<@@@@A0@O@@@@@\@@@@@@@@A0@@@@@@?
8@@@@@@^@@@@@@@OC<C @@@A0_G@@@@@8@A0@@@@C?@@@@@@G@@C @@@@8?@@@@@@@C<@@@@@@A0@@@@@@\@C0@@@@@<^@@@@@@<N^G@@@@A;0@@@@@@O@@@
@@@@G0@@@@@@<C0@@@@@O O@@@@@C @@@@@@@<C0@@@@@_>@@@@@@A0@@@@@@@<C0@@@@@N8N@@@@@C.C @@@@@8@@@@@@@G @@@@@@C? @@@@@@8@@@@@@@
N<\8^@@@C>C @@@@A0A0@@@@@G8G @@@@C O@@@@@@? @@@@@@O8@@@@@@A8@@@@@@A8O @@@@@OC0@@@@@A!!;!!0@@@@C<@@@@@@NA<@@@@@@O@@@@@@C @O
 @@@@@A @@@@@@@<@@@@@@@@G@@@@@_??>@@@@@@C0@@@@@<>C0@@@@@A0@@@@@@G?@@@@@@C88@@@@@G??8@@@@@<@N@@@@@8@@@@@@@C A8@@@@@8@@@@@
@@\@@@@@@@N@A8@@@@A0@O@@@@@\@C0@@@@G@@@@@@@@@\@@@@@@N?@@@@@@G@@@@@@@C0^@8@@@@\C90@@@@N@@8@@@@@>@@@@@@A0GA8@@@@NC8@@@@@@@
O @@@@@@\@@@@@@G@@<@@@@@OG@@@@@@OG''3 @@@@<^@@@@@@C0@@@@@@C8@@@@@@NA<@@@@@C0G0@@@@A0@@@@@@@OA<@@@@@G?@@@@@@@\@@@@@@@NA<@@
@@@C<C @@@@@?@8@@@@@N@@@@@@@A8@@@@@@@?8@@@@@@N@@@@@@@C.G^G @@@?@8@@@@@\@8@@@@@A<C0@@@@A8G @@@@@O0@@@@@@@? @@@@@@\@@@@@@@
\C8@@@@@A18@@@@@@\\8<@@@@@>@@@@@@C _@@@@@@G @@@@@@8@G0@@@@@@8@@@@@@@^@@@@@@@@A0@@@@C<GC @@@@@@\@@@@@OO@\@@@@@@8@@@@@@C18
@@@@@@@N@@@@@A?8^@@@@@N@C @@@@N@@@@@@@@8@<@@@@@\@@@@@@@O@@@@@@@C @>@@@@@\@C0@@@@G@@<@@@@A0@@@@@@G G@@@@@@CC8@@@@@A0@@@@@
@@<G@N@@@@G@>8@@@@C @^@@@@@N@@@@@@@\C0<@@@@C _ @@@@@@@8@@@@@@G@@@@@@A0@N@@@@@A;0@@@@@A90<8@@@@^G0@@@@@@8@@@@@@A<@@@@@@G@
?@@@@@@<C8@@@@@\@@@@@@@G _@@@@@A>@0@@@@@G@@@@@@@G@?@@@@@@?@<@@@@@O0O@@@@@C @@@@@@@^@@@@@@@O?@@@@@@C @@@@@@@?!!7A8@@@O N@@
@@@G@N@@@@@@_@<@@@@@\C8@@@@@C<@@@@@@@G<@@@@@@G@@@@@@@GA>@@@@@@^<@@@@@@GOOO@@@@@?0@@@@@@8O0@@@@@C0@@@@@@N@C8@@@@@@NC0@@@@
@_@@@@@@@@@8@@@@@@A0@@@@@@@G@@@@@C7 O@@@@@@N@@@@@@A8O@@@@@@@C @@@@@? C0@@@@C A8@@@@C @0@@@@@N@>@@@@@G@@@@@@@C0@@@@@@@8@_
 @@@@G@@<@@@@A0@O@@@@@\@@@@@@A<C @@@@@A0_ @@@@@\@@@@@@@O@@C @@@A G>@@@@@8@O@@@@@G@@@@@@@G@>^@@@@@8C<@@@@@@@N@@@@@@A0@@@@
@@\@C @@@@@_8@@@@@@^8OL@@@@O@>@@@@@@N@@@@@@@>@@@@@@A0_0@@@@@NA<@@@@@G@F@@@@@A8O0@@@@@\@\@@@@@C0@@@@@@A0_0@@@@@O O@@@@@C8
C0@@@@@8@@@@@@@G @@@@@@C/8@@@@@@8@@@@@@@O0_0_@@@G8C0@@@@A0G @@@@@G ^@@@@@GA>@@@@@@?@@@@@@@@_ @@@@@A0@@@@@@A0? @@@@@G/@@@
@@@A3#; @@@@_>@@@@@@NG<@@@@@A<@@@@@@C A<@@@@@@C''<@@@@@O @@@@@@@@N@@@@@@@\@@@@@@@C0@@@@@=0G @@@@@G@@@@@@@<C0@@@@@@A8@@@@@
_@@>@@@@@8@\@@@@@8@\@@@@@C ?@@@@@A0@@@@@@@<@@@@@@@N@_0@@@@A0@O@@@@@\@C0@@@@G@@@@@@@_!!8@@@@@@\A<@@@@@G@C @@@@G @@8@@@@8@?
 @@@@G@O @@@@A0@@@@@@A8G?@@@@@N@_ @@@@@@G @@@@@@\@@@@@@C A8@@@@@C>@@@@@@G.C?@@@@G G0@@@@@C @@@@@@_@@@@@@@\O<@@@@@C >@@@@
@A0G @@@@@^G\@@@@@G@N@@@@@@<@@@@@@@\_<@@@@@G0C8@@@@A<@>@@@@@N@@@@@@@A8@@@@@@@1?@@@@@@N@@@@@@@C8G8G0@@A<@<@@@@@\C0@@@@@C8
O@@@@@A0? @@@@@O @@@@@@@A8@@@@@@\C@@@@@@\^8@@@@@@? @@@@@@M8_8@@@@O''0@@@@@C''?@@@@@@>@@@@@@@8A>@@@@@@@?<@@@@@O0A0@@@@F@O@@
@@@@@G@@@@@A0A8@@@@@O<C8@@@@@A0@@@@@@N@<@@@@@@@\@@@@@G0@G @@@@^@_@@@@@O@^@@@@@@9? @@@@@<A<@@@@@O@@@@@@@C0_\@@@@@<@C0@@@@
O@@<@@@@A0@@@@@@C?>@@@@@@G@O0@@@@C??<@@@@A8@@N@@@@N@G8@@@@A8O0@@@@@\@@@@@@@O@? @@@@C C>@@@@@@G0@@@@@@G@@@@@@@<@<@@@@@@?@
@@@@@A?@_0@@@C8@>@@@@@@8@@@@@@O!!?8@@@@GO7 @@@@@X_@@@@@@^G0@@@@@G#''L@@@@A8O @@@@@O@@@@@@@G?/@@@@@A<@^@@@@@_@G @@@@C @@@@@
@@^@@@@@@@LO8@@@@@C @@@@@@@>A>@>@@@_@G8@@@@G#8@@@@@@>G @@@@@\?8@@@@@C8@@@@@@@@^@@@@@@GC0@@@@@GO.@@@@@@O0@@@@@@C<G<@@@@C0
>@@@@@@?= @@@@@_O8@@@@@OA?@@@@@@G?0@@@@@G??<@@@@C0O @@@@@@A0@@@@@>A<@@@@@A?C8@@@@@@8@@@@@@C _@@@@@@@G@@@@@C8@A<@@@@G O @
@@@A8_ @@@@@O? @@@@@O?? @@@@C0@@@@@@@_?''@@@@@O@@<@@@@C0@O@@@@@\@@@@@@@_?@@@@@@A0A? @@@@???@@@@@^@@C @@@C A>@@@@@O?0@@@@@
G@@@@@@@A??0@@@@@8@O0@@@@@G8@@@@@@A0@@@@@@G >@@@@@@O @@@@@@_0G8@@@A<@G0@@@@@N@@@@@@G???@@@@A?1<@@@@@G? @@@@@C?8@@@@@A?1?
@@@@@O?0@@@@@C0@@@@@@A?#0@@@@@^@G0@@@@G A<@@@@@8@@@@@@@G @@@@@@GA?@@@@@@8@@@@@@@O@O@O0@@G A>@@@@@?<@@@@@@O?0@@@@@G?\@@@@
@@>@@@@@@@@_@@@@@@A?8@@@@@A?30@@@@@C<@@@@@@@?@?@@@@A8G8@@@@@G<X@@@@@G??@@@@@A??@@@@@@O?8@@@@@C???@@@@@??0@@@@@@@<@@@@@O?
>@@@@@@O?<@@@@@@N@@@@@@@?? @@@@@@A0@@@@@<@@O @@@A9?0@@@@@_? @@@@@G? @@@@@C??8@@@@@<@@@@@@@C?!!0@@@@C0@O@@@@@<@C0@@@@G@@@@
@@@A? @@@@@@\@G8@@@@O?? @@@@G @@8@@@@8@O @@@@A?8@@@@@A0@@@@@@@O?>@@@@@N@A<@@@@G?<@@@@@@@\@@@@@@@??@@@@@@A8@@@@@@C8@>@@@@
^@@<@@@@@C @@@@@C???0@@@@O8_@@@@@A?0@@@@@@?<@@@@@@O8O @@@@A?0@@@@@@<@@@@@@@O 8@@@@@G @8@@@@A8@N@@@@@N@@@@@@@A8@@@@@@A0O>
@@@@@N@@@@@@@C0C C<@@A8@N@@@@@G>@@@@@@C?0@@@@@@?''@@@@@@O @@@@@@O? @@@@@@O<@@@@@@O8^@@@@@@^@@@@@@@G G0@@@@>@>@@@@@@<F@@@@
@C??0@@@@@O?@@@@@@C?<@@@@@A???@@@@@O?0@@@@@@@O @@@@A?>@@@@@@A?<@@@@@@C @@@@@@G?0@@@@@@@\@@@@@F@@A8@@@@^?8@@@@@A?0@@@@@C?
 @@@@@@_?8@@@@@O@@@@@@@@_ <@@@@@8@C0@@@@N@@<@@@@@0@@@@@@@G @@@@@@G@@\@@@@A<@@@@@@@8@@F@@@@N@A0@@@@@O8@@@@@@\@@@@@@@@?'' @
@@@C @N@@@@A?>@@@@@@@C@@@@@@@G? @@@@@@\@@@@@@@>@O@@@@O @F@@@@@@8@@@@@@???<@@@@A8C @@@@@G0@@@@@@C8@@@@@@A8C0@@@@@O0@@@@@@
O@@@@@@@@@N@@@@@@0@@@@@@@L@@@@@@@C @@@@@@@^@@@@@@@\A? @@@@C @@@@@@@<@@@^@@@L@@@@@@@@>@@@@@@@?0@@@@@@G!!0@@@@@A0@@@@@@G?0@
@@@@@A<@@@@@@A8G@@@@@@G @@@@@@A8@8@@@@O@G@@@@@@@C @@@@A?@H@@@@@A?@@@@@@@_''@@@@@@_ @@@@@@@? @@@@@@@C8@@@@@G>@@@@@@@O<@@@@
@@@X@@@@@@@_ @@@@@@@C@@@@@@@@@L@@@@CO8@@@@@@O0@@@@@@>@@@@@@@@@@@@@@@A0@@@@@@@@@O@@@@@F@@P@@@@A @D@@@@@@@@@@@@@@@@@@@@@A 
@@@@@@@@@@@@@@@@@@@@@@@C@@@@@@@@@@@@@@@@F@@@@@@@@@@<@@@@@X@@@@@@@_>@@@@@@@@@@@@@@@@_ @@@@@@@@@@@@@@O@A0@@@C0@@@@@@@@L@@@
@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@X@@@@@@@G @@@@@@F@G0@@@@@8@@@@@@@
F@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@\@@@@@@\@@@@@@@? @@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@L@@@@@@C0@@@@@@@@@8@@@@@>@@@@@@@@@@@
@@@@@@@@@@@@@G @@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C8@@@@@@@@@@@@@@G@@@@@@@@@@@@@@
@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G @@@@@@@@@@@@C<
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@A8@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@L@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@X@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@A0@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L@G@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@0A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@FC0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@
@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
A9<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@C8^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@C?<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O>@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@8@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O>@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C? @@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A? @@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@?@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@C @@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ @@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@<@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@') ; yourself); yourself]
!

charMapImage2_height
    ^ 57
!

charMapImage2_width
    ^ 21
!

charMapImage3
"character bitmaps (needed ebcause the webServer has no Display for drawing)"

    "
     self charMapImage3 inspect
     ImageEditor openOnClass:self andSelector:#charMapImage3
     Icon flushCachedIcons
    "

    <resource: #image>

    ^Icon
        constantNamed:'HumanReadableImageGenerator class charMapImage3'
        ifAbsentPut:[(Depth1Image new) width: 3200; height: 37; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@0@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@L@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@^@@@@@@??8@@@@@@G?>@@@@C?>@@@@@@???@@@@@O??8@@@@@A?? @@@@? C?@@@@O8@?0@@@C?@@@@@@@G?@@@@@@O8G>@@@@C?@@@@@
@A>@@G8@@@_ @?@@@@@A?<@@@@@??8@@@@@@G?0@@@@C?? @@@@@G?0@@@@@O??<@@@@C>@G8@@@C?@A>@@@@? _0C8@@G>@_ @@@C?@G<@@@@G??0@@@@@@
@@@@@@A<@@@@@@@@@@@@@@@@@_@@@@@@@@@@@@@@@? @@@@@@@@@@@@@A<@@@@@@@_@@@@@@@A8@@@@@@@G @@@@@@_@@@@@@@G8@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O8@@@@@@@\@@@@@@
@?0@@@@@@_8@@@@@@@G@@@@@@C?>@@@@@@G>@@@@@@_?8@@@@@A?0@@@@@@_8@@@@@@@G @@@@@C G @@@@@G G0@@@@NA<@@@@@C A0@@@@@8@^@@@@@A8C
8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@\@@@@@N@@@@@@@C @A8@@@@<@C@@@@@A8C0@@@@C G @@@@@G O@@@@@N@^@@@@@C >@@@@@C \G@@@
@@N@@X@@@@O@@F@@@@C A0@X@@@^@C @@@@O@@\@@@@A8@8@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@X<@@@@@@@@@@@@@@G@@@@@@@A0
@@@@@@@^@@@@@@@A<@@@@@@A0@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@GG@@@@@@@_@@@@@@@8N@@@@@@LG @@@@@@A0@@@@@@??@@@@@@C#0@@@@@G?>@@@@@@8N@@@@@@NC @@@@@
@C<@@@@@@8@<@@@@@C @<@@@@C G0@@@@@8@N@@@@@N@A0@@@@@<@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@F@@@@@C @@@@@@@<@@>@@@@O@@
0@@@@@<@N@@@@@8@\@@@@@C0@8@@@@C C0@@@@A0G @@@@@0G@8@@@@C @F@@@@A0@A @@@@8@\@F@@@C0@0@@@@A0@F@@@@@X@^@@@@@@@@@@@@@A0@@@@@
@@@@@@@@@@@@\@@@@@@@@@@@@@@NO@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@G @@@@@@@^@@@@@@@\@@@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C 8@@@@@@?0@@@@@@\A0@@@@@G@8@@
@@@@@<@@@@@@L@@@@@@@A <@@@@@A0C@@@@@@\A0@@@@@G@X@@@@@@@?@@@@@@N@G@@@@@A8@G@@@@@8@^@@@@@N@A @@@@C @L@@@@@\@A @@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C C@@@@@@8@@@@@@@O@@O @@@G8@L@@@@@^@A0@@@@N@G @@@@A8@G@@@@@8@^@@@@@X@8@@@@@\A0F@@@@@8@A @@@@\@@
0@@@@O@G A @@@\@\@@@@@^@C@@@@@N@G@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@C#0@@@@@@@@@@@@@@\@@@@@@@G@@@@@@@@@@@@@@
@@@@@@@@@@G@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@A8G@@@@@@@<@@@@@@G@^@@@@@A0O@@@@@@@_@@@@@@C@@@@@@@@8O@@@@@@XA0@@@@@F@N@@@@@C0G@@@@@@@Y0@@@@@C A8@
@@@@<@@0@@@@N@C0@@@@C @X@@@@@8@C@@@@@O@@X@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8A @@@@@N@@@@@@@G0@G8@@@A/@C@@@@@O@@^@@@@
C @8@@@@@<@A8@@@@N@C @@@@N@F@@@@@F@\A @@@@N@@X@@@@G @L@@@@A0A8@0@@@C F@@@@@C @0@@@@C@C @@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0
@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\A0@@@@@@G@@@@@@C0G @@@@@\A0@@@@@@G0@@@@@@0
@@@@@@@\@@@@@@@F@X@@@@@C C @@@@@8A8@@@@@@F^@@@@@@8@^@@@@@N@@L@@@@C @\@@@@@8@@@@@@@N@@@@@@@C @C@@@@@8@A0@@@@N@@\@@@@C @@@
@@@@C @@@@@@N@0@@@@@C @@@@@@A>@A.@@@@Y0@0@@@@C @C @@@@8@N@@@@@N@@N@@@@C @8@@@@C A @@@@@@G@@@@@@C @F@@@@@8@G@@@@@\@^@L@@@
@<C@@@@@@<@X@@@@@@A8@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@\@@
@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@G@^@@@@@@A0@@@@@@<A8@@@@@G@\@@@@@@C\@@@@@@L@@@@@@@G@@@@@@@C@N@@@@@@8@8@@@@@N@N@@@@@@A'' @@@@@N@G @@@@G @@@@@@@8
@G @@@@N@@@@@@@C @@@@@@A8@@@@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C X@@@@@@8@@@@@@@[ @[ @@@F^@L@@@@A8@@<@@@@N@C @@@@G @C0
@@@@8@N@@@@@8@@@@@@@@A0@@@@@@8@A @@@@O@A @@@@G@G0C@@@@G!! @@@@@G@N@@@@@@@\@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@
G @@@@@@@@@@@@@@@\@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0G @@@@@@\@@@@@@O ^@@@@@@@G@@@@@@A7@@@@@@C@@@@@@@A0@@@@@
@@@C@@@@@@O@N@@@@@C C @@@@@@08@@@@@C A0@@@@A8@@@@@@@N@@8@@@@C C@@@@@@8@@@@@@@\@@@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8
L@@@@@@N@@@@@@@F<@L8@@@A#0C@@@@@^@@G@@@@C @8@@@@A8@@\@@@@N@C @@@@G@@@@@@@@@\@@@@@@N@@X@@@@A0@X@@@@A8C\A @@@@88@@@@@@8C@@
@@@@@O@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@F@@@@@@@@X@@@@@@A0@@@@@@@^@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@8
@@@@@@G@@@@@@A0G @@@@@@C0@@@@@@Y0@@@@@A0@@@@@@@<@@@@@@@@A0@@@@@A0C @@@@@8@8@@@@@@LO@@@@@@8@<@@@@@\@@@@@@@C @O@@@@@8@0@@@
@@N@@@@@@@G@@@@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@NF@@@@@@C @@@@@@A''@CN@@@@X^@0@@@@G@@A0@@@@8@^@@@@@\@@G@@@@C A8@@@@A8
@@@@@@@@G@@@@@@C @F@@@@@\@L@@@@@N@7@X@@@@OL@@@@@@OA @@@@@@C @@@@@A?@@@@@@A18@@@@@@A>@@@@@@@_\@@@@@@G0@@@@@C?8@@@@@@@>O@@
@@@A3<@@@@@@\?@@@@@@_ @@@@@@A>@@@@@@@\O8@@@@@G @@@@@@G78?@@@@A<?@@@@@@@_ @@@@@A38@@@@@@A<@@@@@@_O @@@@@@_ @@@@@C?8@@@@@@
_A<@@@@@O8O0@@@@C<?G8@@@@O;>@@@@@O8_0@@@@@_? @@@@@O@N@@@@@@A0@@@@@@@A0@@@@@@@8@@@@@@L\@@@@@@\@@@@@@@O@@@@@@@@@X@@@@@@^A0
@@@@@N@N@@@@@@FA0@@@@@N@^@@@@@O@@@@@@@@8@C0@@@@N@L@@@@@C C@@@@@C0@@@@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C# @@@@@@8@@@@@
@@Y8A# @@@FC L@@@@A0@@^@@@@N@G@@@@@G@@A8@@@@8@<@@@@@O @@@@@@@A0@@@@@@8@A @@@@G C@@@@@C M8F@@@@A>@@@@@@A08@@@@@@A0@@@@@@0
8@@@@@@]7 @@@@@A00@@@@@@^?@@@@@@GG@@@@@@G @@@@@@@8?0@@@@@]7 @@@@@G]8@@@@@A8@@@@@@@C @@@@@@GA8@@@@@A8@@@@@@@_'']8@@@@G\8@@
@@@@\^@@@@@A=7 @@@@@A9,@@@@@A><@@@@@@LN@@@@@@G @@@@@@A0G@@@@@@<A8@@@@@^G <@@@@A<N@@@@@@<A8@@@@@O@8@@@@@C0C0@@@@@@\@@@@@@
@@\@@@@@@@\@@@@@@CG@@@@@@F?0@@@@@C7<@@@@@@@N@@@@@@C88@@@@@C C @@@@@A ^@@@@@C ^@@@@@C0@@@@@@@N@@<@@@@C G@@@@@@8@0@@@@@<@@
@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@98@@@@@@N@@@@@@@F^@X8@@@A <C@@@@@\@@G @@@C C0@@@@A0@@^@@@@N@^@@@@@A>@@@@@@@@\@@@@
@@N@@X@@@@@8A @@@@@<F^C@@@@@O @@@@@@NL@@@@@@@<@@@@@@\G@@@@@@G0\@@@@@@8N@@@@@@NC0@@@@@C 8@@@@@A8@@@@@@@LG\@@@@@G0\@@@@@A<
G@@@@@@^@@@@@@@@8@@@@@@A0\@@@@@@^@@@@@@@G <N@@@@A<G@@@@@@NA0@@@@@G0\@@@@@@8O@@@@@@_O@@@@@@FA @@@@@A8@@@@@@@\A0@@@@@O@L@@
@@@C 8F@@@@@OC@@@@@@O@X@@@@@C \@@@@@@<@<@@@@@@G@@@@@@@@N@@@@@@@^@@@@@@A!!0@@@@@A<^@@@@@@?''0@@@@@@C@@@@@@@_<@@@@@@8A8@@@@@
@XG @@@@@?? @@@@@<@@@@@@@C @O@@@@@??0@@@@@O?<@@@@@O@@@@@@@@???0@@@@O??<@@@@C @@@@@@@C @@@@@@N7@@@@@@C @@@@@@A# FN@@@@XG 
0@@@@O@@A8@@@@8A0@@@@@<@@G @@@C?>@@@@@@G8@@@@@@@G@@@@@@C @F@@@@@O@X@@@@@GA# 0@@@@C0@@@@@@C&@@@@@@@N@@@@@@OA0@@@@@A8G @@@
@@\C0@@@@@G \@@@@@A0G@@@@@@^@@@@@@@G@8@@@@@A8G@@@@@@^A0@@@@@G @@@@@@@N@@@@@@@\N@@@@@@G @@@@@@A0NA0@@@@^A0@@@@@G@\@@@@@A8
C @@@@@\A0@@@@@G!!0@@@@@A \@@@@@@^@@@@@@@G@\@@@@@A0F@@@@@@8NC@@@@@A1 @@@@@A0F@@@@@@0O@@@@@@N@O@@@@@@A0@@@@@@@G@@@@@@@_ @@
@@@@8\@@@@@@^C0@@@@@O0\@@@@@@A0@@@@@@C>@@@@@@O@^@@@@@@L@8@@@@@N@^@@@@@O@@@@@@@@8@C0@@@@N@\@@@@@C C@@@@@C0@@@@@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C98@@@@@@8@@@@@@@X<CC @@@F@8L@@@@C0@@^@@@@O?8@@@@@O@@A8@@@@8N@@@@@@@_ @@@@@@A0@@@@@@8@A @@@@A0F
@@@@@A0X8L@@@@@^@@@@@@@_ @@@@@@G@@@@@@A0\@@@@@@^@8@@@@@G@<@@@@@A0G@@@@@@8A0@@@@@G @@@@@@A0N@@@@@@\A0@@@@@G@\@@@@@A8@@@@@
@@C @@@@@@GG@@@@@@A8@@@@@@@\C \@@@@G@\@@@@@A0C @@@@@\@8@@@@@G@\@@@@@A0@@@@@@@8C@@@@@@G @@@@@@A0G@@@@@@\A @@@@@GC00@@@@@^
8@@@@@@^A @@@@@LC @@@@@C C0@@@@@@\@@@@@@@A @@@@@@@\@@@@@@LG@@@@@@G@^@@@@@C8G @@@@@@\@@@@@@A?0@@@@@A0O @@@@@C@O@@@@@C A0@
@@@C0@@@@@@@N@@<@@@@C C@@@@@@8@0@@@@@<@@@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@<O@@@@@@N@@@@@@@FG@08@@@A OC@@@@@<@@G @@@
C @@@@@@C0@@^@@@@NC @@@@@@A<@@@@@@@\@@@@@@N@@X@@@@@^C@@@@@@^LOF@@@@@O @@@@@@C0@@@@@@C0@@@@@@@G@@@@@@G@N@@@@@C F@@@@@@\A0
@@@@@N@\@@@@@A8@@@@@@@\C @@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A1 @@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@8@8@@@@@G@O@@@@@C0G@@@@
@@\@@@@@@@G@@@@@@@A8@@@@@@@\A0@@@@@C X@@@@@A0<L@@@@@C,@@@@@@C 0@@@@@@A0@@@@@@<@<@@@@@@G@@@@@@@@0@@@@@@@C0@@@@@FA0@@@@@@@
G @@@@@<@8@@@@@@N@@@@@@@0>@@@@@@_N8@@@@@A??0@@@@@8@^@@@@@<@@@@@@@C @O@@@@@8@0@@@@@N@L@@@@@O@A?0@@@@8@A0@@@@N@@\@@@@C @@@
@@@@C @@@@@@NA8@@@@@C @@@@@@A!!8XN@@@@XA80@@@@G@@A8@@@@8@@@@@@@\@@G @@@C \@@@@@@@O @@@@@@G@@@@@@C @F@@@@@G 0@@@@@C#A1 @@@
@G<@@@@@@@<@@@@@@@8@@@@@@@O0@@@@@A0C @@@@@8@@@@@@@O@\@@@@@C G @@@@@^@@@@@@@G@8@@@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@\8@
@@@@@G @@@@@@A0NA0@@@@\A0@@@@@N@O@@@@@A0C0@@@@@8A0@@@@@G@@@@@@@A<@@@@@@@^@@@@@@@G@\@@@@@@8L@@@@@@\_F@@@@@@^@@@@@@@8L@@@@
@@@<@@@@@@O@O@@@@@@A0@@@@@@@X@@@@@@@@\@@@@@C \@@@@@@@@8@@@@@O@N@@@@@@C @@@@@@\C0@@@@@A?^@@@@@@X@\@@@@@N@C @@@@O@@@@@@@@8
@C0@@@@N@L@@@@@C @@@@@@C0@C0@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C N@@@@@@8@@@@@@@XNFC @@@F@NL@@@@A0@@^@@@@N@@@@@@@G@@A8
@@@@8G@@@@@@@A8@@@@@@A0@@@@@@8@A @@@@@8X@@@@@@80\X@@@@A'' @@@@@@G@@@@@@@\@@@@@@@^\@@@@@@\@8@@@@@N@@@@@@@C0G@@@@@@8A8@@@@@
G @@@@@@A0N@@@@@@\A0@@@@@G@\@@@@@A8@@@@@@@C @@@@@@G^@@@@@@A8@@@@@@@\C \@@@@G@\@@@@@C C0@@@@@\@<@@@@@N@\@@@@@A0@@@@@@@G0@
@@@@@G @@@@@@A0G@@@@@@OC@@@@@@G''9 @@@@@G @@@@@@GF@@@@@@@N@@@@@@C0C @@@@@@\@@@@@@@L@@@@@@@@G @@@@@0G@@@@@@@@N@@@@@C C0@@@
@@@8@@@@@@N@^@@@@@@@G @@@@@F@G @@@@C @<@@@@A8@@@@@@@N@@8@@@@C @@@@@@@8@@@@@@@\@@8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8
C0@@@@@N@@@@@@@FC# 8@@@A C3@@@@@^@@G@@@@C @@@@@@A8@@\@@@@N@8@@@@@@@O@@@@@@@\@@@@@@N@@X@@@@@OF@@@@@@OXG,@@@@@08@@@@@@A0@@
@@@@O@@@@@@@^G@@@@@@G@N@@@@@C @@@@@@@8A0@@@@@O?>@@@@@A8@@@@@@@NG@@@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A=0@@@@@@^@@@@@@@
G@8G@@@@A0G@@@@@@8@<@@@@@G@O@@@@@C G@@@@@@\@@@@@@@@?@@@@@@A8@@@@@@@\A0@@@@@A1 @@@@@@9.X@@@@@@8@@@@@@A1 @@@@@@G@@@@@@@<@8
@@@@@@G@@@@@@@F@@@@@@@@@8@@@@@_??@@@@@@@C @@@@@<@<@@@@@@^@@@@@@C G @@@@@@A8@@@@@C@@8@@@@@8@O@@@@@^@@@@@@@C @^@@@@@8@@@@@
@@N@@@@@@@G@@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@^@@@@@C @@@@@@A <0N@@@@X@^0@@@@G @A0@@@@8@@@@@@@^O@G@@@@C O@@@@@@@
A0@@@@@@G@@@@@@C @F@@@@@A3 @@@@@C6@;@@@@@XG@@@@@@@\@@@@@@C @@@@@@OA0@@@@@A0C @@@@@8@@@@@@@N@\@@@@@C @@@@@@@^@@@@@@@A3 @@
@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@^^@@@@@@G @@@@@@A0NA0@@@@\A0@@@@@N@O@@@@@A0C0@@@@@8A0@@@@@G@@@@@@@@C8@@@@@@^@@@@@@@
G@\@@@@@@\X@@@@@@N[,@@@@@@_@@@@@@@NX@@@@@@C0@@@@@@O@^@@@@@@A0@@@@@@C@@@@@@@@@N@@@@@@@\@@@@@@@@8@@@@@O@O@@@@@@G @@@@@@8@8
@@@@@@@^@@@@@@0@O@@@@@N@C0@@@@G @C@@@@@8@G @@@@N@@@@@@@C @@@@@@A8@C @@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C C0@@@@@8@@@@@
@@XGLC @@@F@C,@@@@@8@@<@@@@N@@@@@@@C/<C0@@@@8A0@@@@@0@\@@@@@@A0@@@@@@8@A @@@@@^0@@@@@@] N0@@@@NA8@@@@@@G@@@@@@A0@@@@@@C 
\@@@@@@\@8@@@@@N@@@@@@@C0G@@@@@@8@@@@@@@G @@@@@@A7 @@@@@@\A0@@@@@G@\@@@@@A8@@@@@@@C @@@@@@GC @@@@@A8@@@@@@@\C \@@@@G@\@@
@@@C C0@@@@@\@<@@@@@N@\@@@@@A0@@@@@@@@O@@@@@@G @@@@@@A0G@@@@@@C.@@@@@@C<;@@@@@@O0@@@@@@C,@@@@@@@8@@@@@@A0G @@@@@@\@@@@@@
A F@@@@@A0C @@@@@@G@@@@@@O@N@@@@@C0C @@@@@A8@@@@@@N@N@@@@@@@G@@@@@@X@C0@@@@C @<@@@@@<@@0@@@@N@C0@@@@C @L@@@@@8@@@@@@@N@@
8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@\@@@@@N@@0@@@@FA>@8@@@A @_@@@@@O@@N@@@@C @@@@@@@?G 8@@@@N@^@@@@@N@G@@@@@@@\@@@@
@@N@@0@@@@@G,@@@@@@G0C8@@@@C@O@@@@@@A0@@@@@@<@L@@@@@8G@@@@@@G@N@@@@@C @@@@@@@<A0@@@@@N@@@@@@@A8@@@@@@@X@@@@@@@G@\@@@@@A0
G@@@@@@^@@@@@@@@8@@@@@@A0\@@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@8@8@@@@@G@N@@@@@C G@@@@@@\@@@@@@@LA0@@@@@A8@@@@@@@\A0@@@@@@;@@@
@@@@_G0@@@@@CN@@@@@@@?@@@@@@@\C@@@@@@\A0@@@@@@G@@@@@@@8A @@@@@<A8@@@@@@A0@@@@@C0G @@@@@\@8@@@@@@^@@@@@@C C @@@@@@C0@@@@@
F@@\@@@@@8@N@@@@@G @X@@@@C @8@@@@@8@G@@@@@N@@@@@@@A0@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@G @@@@C @X@@@@A O N@@@@X@G
0@@@@A0@G@@@@@8@@@@@@@G <\@@@@C C @@@@C C @@@@@@G@@@@@@A0@L@@@@@@>@@@@@@A<@>@@@@A A0@@@@@@\@@@@@@N@C@@@@@NA0@@@@@A0C @@@
@@<@0@@@@@G@\@@@@@C0C@@@@@@^@@@@@@@G@@@@@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@\G @@@@@G @@@@@@A0NA0@@@@\A0@@@@@G@N@@@@@A0
C @@@@@\A0@@@@@G@@@@@@@C@\@@@@@@^@@@@@@@G@\@@@@@@O0@@@@@@G1<@@@@@A#0@@@@@@G0@@@@@@O@0@@@@@G \@@@@@@A0@@@@@@\@X@@@@@O@^@@
@@@@@\@@@@@@<A0@@@@@G@N@@@@@@G @@@@@@8@8@@@@@G 8@@@@@A @G @@@@N@G @@@@A8@L@@@@@8@<@@@@@N@A @@@@C @@@@@@@^@C @@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C @<@@@@@8@F@@@@@XC0C @@@F@@<@@@@@N@C @@@@N@@@@@@@@<G/@@@@@8@<@@@@@\@8@@@@@@A0@@@@@@^@F@@@@@@O 
@@@@@@O@G @@@@8@^@@@@@@G@@@@@@G@A0@@@@C ] @@@@@\A0@@@@@G@L@@@@@A0O@@@@@@\@0@@@@@G @@@@@@A?0@@@@@@\A0@@@@@G@\@@@@@A8@@@@@
@@C @@@@@@G@8@@@@@A8@@@@@@@\C \@@@@G@\@@@@@A0G@@@@@@^A8@@@@@G@\@@@@@A0@@@@@@@0G@@@@@@C#@@@@@@A0O@@@@@@A8@@@@@@A8^@@@@@@8
\@@@@@@A8@@@@@@G \@@@@@@8N@@@@@@@\@@@@@@F@N@@@@@C0G@@@@@@@G@@@@@@G@\@@@@@@8G@@@@@@A8@@@@@@N@^@@@@@A8N@@@@@@0@A8@@@@C C0@
@@@@G F@@@@@N@^@@@@@C @8@@@@@8@@@@@@@A8A8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@G @@@@N@C @@@@F@<@8@@@A @G@@@@@A8A0@@@@
C @@@@@@@G ?@@@@@N@G@@@@@G \@@@@@@@\@@@@@@C0C@@@@@@A0@@@@@@C A0@@@@L@C0@@@@@A0@@@@@C0@8@@@@@<OX@@@@@G 8@@@@@@8F@@@@@@NC8
@@@@@C X@@@@@A8@@@@@@@O?0@@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A0O@@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@NA @@@@@G <@@@@@@8O@@@@
@@\@@@@@@@NA @@@@@@80@@@@@@NG0@@@@@@^@@@@@@@NC @@@@@LC @@@@@@N@@@@@@A0G@@@@@@GC @@@@@@O@@@@@@C?? @@@@@\C @@@@@@A0@@@@@A0
N@@@@@@OA0@@@@@@^@@@@@@A0G@@@@@@^G@@@@@@^@@_@@@@A<C8@@@@@@>O@@@@@G1>@@@@@A<@>@@@@@_@@@@@@@@O!!8@@@@A<@C0@@@@_@@<@@@@C0@@@
@@@@C @@@@@@_@A<@@@@G0C8@@@@A0G@^@@@@\@A0@@@@@GA8@@@@A<@@@@@@@@\O @@@@G0A8@@@@A>N@@@@@@@O@@@@@@@_G @@@@@@\@@@@@@@8@\@@@@
O@@>@@@@@@<@@@@@@8@>@@@@@G/>@@@@@A.\@@@@@@G''@@@@@@A;?@@@@@@^\@@@@@@^@@@@@@@FA<@@@@@A8G @@@@@^A8@@@@@G @@@@@@@N@@@@@@@^A8
@@@@@G @@@@@@A0^C0@@@@^A8@@@@@A10@@@@@A>^@@@@@@G?0@@@@@G@@@@@@@C80@@@@@@O8@@@@@@C?_@@@@@@C@@@@@@@C 8@@@@@O@<@@@@@@C@@@@@
@@8G0@@@@@@;0@@@@@@G8@@@@@@??0@@@@@C!!0@@@@@@@>@@@@@@GG@@@@@@A98@@@@@@G @@@@@@OG @@@@@C'' @@@@@O8@O8@@@@??0@@@@@@A?@@@@@C?
0@@@@@@???@@@@@O<@@@@@@@@_0@@@@@? C?@@@@O8@?0@@@C?@@@@@@@@8@@@@@@O8A? @@@C??<@@@@@?A _8@@@O0@L@@@@@@_0@@@@@?0@@@@@@@A?8@
@@@C>@O @@@@X>@@@@@@@O<@@@@@@@?@@@@@@@G@@@@@@@F@C@@@@G<@?0@@@@@?0@@@@@_?? @@@@@^F@@@@@@X<@@@@@@@_ @@@@@@G&@@@@@@A<@@@@@@
_8@@@@@@C C @@@@A?G<@@@@@_1?@@@@@G>@@@@@@@C @@@@@@_1?@@@@@G>@@@@@@A?O9?@@@@_1?@@@@@@G8@@@@@@\>@@@@@@@^\@@@@@G>@@@@@@@38@
@@@@@@<@@@@@@@OF@@@@@@@0@@@@@@@0L@@@@@G8? @@@@@@0@@@@@@O?8@@@@@@G0@@@@@@O?0@@@@@O?<@@@@@@O0@@@@@@A?<@@@@@@_@@@@@@@G8@@@@
@@A0@@@@@@@? @@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<N@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@OG@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A>@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@N@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C3 @@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A3@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@\@@@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\C @@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G>
@@@@@@@@G<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@a') ; mask:((Depth1Image new) width: 3200; height: 37; photometric:(#whiteIs0); bitsPerSample:(#[1]); samplesPerPixel:(1); bits:(ByteArray fromPackedString:'
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@0@@@@@@@L@@@@@@@@@@@@@@@@@@@@@@@@L@@@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@^@@@@@@??8@@@@@@G?>@@@@C?>@@@@@@???@@@@@O??8@@@@@A?? @@@@? C?@@@@O8@?0@@@C?@@@@@@@G?@@@@@@O8G>@@@@C?@@@@@
@A>@@G8@@@_ @?@@@@@A?<@@@@@??8@@@@@@G?0@@@@C?? @@@@@G?0@@@@@O??<@@@@C>@G8@@@C?@A>@@@@? _0C8@@G>@_ @@@C?@G<@@@@G??0@@@@@@
@@@@@@A<@@@@@@@@@@@@@@@@@_@@@@@@@@@@@@@@@? @@@@@@@@@@@@@A<@@@@@@@_@@@@@@@A8@@@@@@@G @@@@@@_@@@@@@@G8@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O8@@@@@@@\@@@@@@
@?0@@@@@@_8@@@@@@@G@@@@@@C?>@@@@@@G>@@@@@@_?8@@@@@A?0@@@@@@_8@@@@@@@G @@@@@C G @@@@@G G0@@@@NA<@@@@@C A0@@@@@8@^@@@@@A8C
8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@\@@@@@N@@@@@@@C @A8@@@@<@C@@@@@A8C0@@@@C G @@@@@G O@@@@@N@^@@@@@C >@@@@@C \G@@@
@@N@@X@@@@O@@F@@@@C A0@X@@@^@C @@@@O@@\@@@@A8@8@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@@X<@@@@@@@@@@@@@@G@@@@@@@A0
@@@@@@@^@@@@@@@A<@@@@@@A0@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@GG@@@@@@@_@@@@@@@8N@@@@@@LG @@@@@@A0@@@@@@??@@@@@@C#0@@@@@G?>@@@@@@8N@@@@@@NC @@@@@
@C<@@@@@@8@<@@@@@C @<@@@@C G0@@@@@8@N@@@@@N@A0@@@@@<@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@F@@@@@C @@@@@@@<@@>@@@@O@@
0@@@@@<@N@@@@@8@\@@@@@C0@8@@@@C C0@@@@A0G @@@@@0G@8@@@@C @F@@@@A0@A @@@@8@\@F@@@C0@0@@@@A0@F@@@@@X@^@@@@@@@@@@@@@A0@@@@@
@@@@@@@@@@@@\@@@@@@@@@@@@@@NO@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@G @@@@@@@^@@@@@@@\@@@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C 8@@@@@@?0@@@@@@\A0@@@@@G@8@@
@@@@@<@@@@@@L@@@@@@@A <@@@@@A0C@@@@@@\A0@@@@@G@X@@@@@@@?@@@@@@N@G@@@@@A8@G@@@@@8@^@@@@@N@A @@@@C @L@@@@@\@A @@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C C@@@@@@8@@@@@@@O@@O @@@G8@L@@@@@^@A0@@@@N@G @@@@A8@G@@@@@8@^@@@@@X@8@@@@@\A0F@@@@@8@A @@@@\@@
0@@@@O@G A @@@\@\@@@@@^@C@@@@@N@G@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@C#0@@@@@@@@@@@@@@\@@@@@@@G@@@@@@@@@@@@@@
@@@@@@@@@@G@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@A8G@@@@@@@<@@@@@@G@^@@@@@A0O@@@@@@@_@@@@@@C@@@@@@@@8O@@@@@@XA0@@@@@F@N@@@@@C0G@@@@@@@Y0@@@@@C A8@
@@@@<@@0@@@@N@C0@@@@C @X@@@@@8@C@@@@@O@@X@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8A @@@@@N@@@@@@@G0@G8@@@A/@C@@@@@O@@^@@@@
C @8@@@@@<@A8@@@@N@C @@@@N@F@@@@@F@\A @@@@N@@X@@@@G @L@@@@A0A8@0@@@C F@@@@@C @0@@@@C@C @@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0
@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@^@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\A0@@@@@@G@@@@@@C0G @@@@@\A0@@@@@@G0@@@@@@0
@@@@@@@\@@@@@@@F@X@@@@@C C @@@@@8A8@@@@@@F^@@@@@@8@^@@@@@N@@L@@@@C @\@@@@@8@@@@@@@N@@@@@@@C @C@@@@@8@A0@@@@N@@\@@@@C @@@
@@@@C @@@@@@N@0@@@@@C @@@@@@A>@A.@@@@Y0@0@@@@C @C @@@@8@N@@@@@N@@N@@@@C @8@@@@C A @@@@@@G@@@@@@C @F@@@@@8@G@@@@@\@^@L@@@
@<C@@@@@@<@X@@@@@@A8@@@@@@@@@@@@@A0@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@^@@@@@@@@@@@@@@@A0@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@\@@
@@@@@G @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@G@^@@@@@@A0@@@@@@<A8@@@@@G@\@@@@@@C\@@@@@@L@@@@@@@G@@@@@@@C@N@@@@@@8@8@@@@@N@N@@@@@@A'' @@@@@N@G @@@@G @@@@@@@8
@G @@@@N@@@@@@@C @@@@@@A8@@@@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C X@@@@@@8@@@@@@@[ @[ @@@F^@L@@@@A8@@<@@@@N@C @@@@G @C0
@@@@8@N@@@@@8@@@@@@@@A0@@@@@@8@A @@@@O@A @@@@G@G0C@@@@G!! @@@@@G@N@@@@@@@\@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@G@@@@@@@@@@@@@@
G @@@@@@@@@@@@@@@\@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0G @@@@@@\@@@@@@O ^@@@@@@@G@@@@@@A7@@@@@@C@@@@@@@A0@@@@@
@@@C@@@@@@O@N@@@@@C C @@@@@@08@@@@@C A0@@@@A8@@@@@@@N@@8@@@@C C@@@@@@8@@@@@@@\@@@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8
L@@@@@@N@@@@@@@F<@L8@@@A#0C@@@@@^@@G@@@@C @8@@@@A8@@\@@@@N@C @@@@G@@@@@@@@@\@@@@@@N@@X@@@@A0@X@@@@A8C\A @@@@88@@@@@@8C@@
@@@@@O@@@@@@@@@@@@@@G@@@@@@@@@@@@@@@@@A0@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@G@@@@@@@A0@@@@@@@F@@@@@@@@X@@@@@@A0@@@@@@@^@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<@8
@@@@@@G@@@@@@A0G @@@@@@C0@@@@@@Y0@@@@@A0@@@@@@@<@@@@@@@@A0@@@@@A0C @@@@@8@8@@@@@@LO@@@@@@8@<@@@@@\@@@@@@@C @O@@@@@8@0@@@
@@N@@@@@@@G@@@@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@NF@@@@@@C @@@@@@A''@CN@@@@X^@0@@@@G@@A0@@@@8@^@@@@@\@@G@@@@C A8@@@@A8
@@@@@@@@G@@@@@@C @F@@@@@\@L@@@@@N@7@X@@@@OL@@@@@@OA @@@@@@C @@@@@A?@@@@@@A18@@@@@@A>@@@@@@@_\@@@@@@G0@@@@@C?8@@@@@@@>O@@
@@@A3<@@@@@@\?@@@@@@_ @@@@@@A>@@@@@@@\O8@@@@@G @@@@@@G78?@@@@A<?@@@@@@@_ @@@@@A38@@@@@@A<@@@@@@_O @@@@@@_ @@@@@C?8@@@@@@
_A<@@@@@O8O0@@@@C<?G8@@@@O;>@@@@@O8_0@@@@@_? @@@@@O@N@@@@@@A0@@@@@@@A0@@@@@@@8@@@@@@L\@@@@@@\@@@@@@@O@@@@@@@@@X@@@@@@^A0
@@@@@N@N@@@@@@FA0@@@@@N@^@@@@@O@@@@@@@@8@C0@@@@N@L@@@@@C C@@@@@C0@@@@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C# @@@@@@8@@@@@
@@Y8A# @@@FC L@@@@A0@@^@@@@N@G@@@@@G@@A8@@@@8@<@@@@@O @@@@@@@A0@@@@@@8@A @@@@G C@@@@@C M8F@@@@A>@@@@@@A08@@@@@@A0@@@@@@0
8@@@@@@]7 @@@@@A00@@@@@@^?@@@@@@GG@@@@@@G @@@@@@@8?0@@@@@]7 @@@@@G]8@@@@@A8@@@@@@@C @@@@@@GA8@@@@@A8@@@@@@@_'']8@@@@G\8@@
@@@@\^@@@@@A=7 @@@@@A9,@@@@@A><@@@@@@LN@@@@@@G @@@@@@A0G@@@@@@<A8@@@@@^G <@@@@A<N@@@@@@<A8@@@@@O@8@@@@@C0C0@@@@@@\@@@@@@
@@\@@@@@@@\@@@@@@CG@@@@@@F?0@@@@@C7<@@@@@@@N@@@@@@C88@@@@@C C @@@@@A ^@@@@@C ^@@@@@C0@@@@@@@N@@<@@@@C G@@@@@@8@0@@@@@<@@
@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@98@@@@@@N@@@@@@@F^@X8@@@A <C@@@@@\@@G @@@C C0@@@@A0@@^@@@@N@^@@@@@A>@@@@@@@@\@@@@
@@N@@X@@@@@8A @@@@@<F^C@@@@@O @@@@@@NL@@@@@@@<@@@@@@\G@@@@@@G0\@@@@@@8N@@@@@@NC0@@@@@C 8@@@@@A8@@@@@@@LG\@@@@@G0\@@@@@A<
G@@@@@@^@@@@@@@@8@@@@@@A0\@@@@@@^@@@@@@@G <N@@@@A<G@@@@@@NA0@@@@@G0\@@@@@@8O@@@@@@_O@@@@@@FA @@@@@A8@@@@@@@\A0@@@@@O@L@@
@@@C 8F@@@@@OC@@@@@@O@X@@@@@C \@@@@@@<@<@@@@@@G@@@@@@@@N@@@@@@@^@@@@@@A!!0@@@@@A<^@@@@@@?''0@@@@@@C@@@@@@@_<@@@@@@8A8@@@@@
@XG @@@@@?? @@@@@<@@@@@@@C @O@@@@@??0@@@@@O?<@@@@@O@@@@@@@@???0@@@@O??<@@@@C @@@@@@@C @@@@@@N7@@@@@@C @@@@@@A# FN@@@@XG 
0@@@@O@@A8@@@@8A0@@@@@<@@G @@@C?>@@@@@@G8@@@@@@@G@@@@@@C @F@@@@@O@X@@@@@GA# 0@@@@C0@@@@@@C&@@@@@@@N@@@@@@OA0@@@@@A8G @@@
@@\C0@@@@@G \@@@@@A0G@@@@@@^@@@@@@@G@8@@@@@A8G@@@@@@^A0@@@@@G @@@@@@@N@@@@@@@\N@@@@@@G @@@@@@A0NA0@@@@^A0@@@@@G@\@@@@@A8
C @@@@@\A0@@@@@G!!0@@@@@A \@@@@@@^@@@@@@@G@\@@@@@A0F@@@@@@8NC@@@@@A1 @@@@@A0F@@@@@@0O@@@@@@N@O@@@@@@A0@@@@@@@G@@@@@@@_ @@
@@@@8\@@@@@@^C0@@@@@O0\@@@@@@A0@@@@@@C>@@@@@@O@^@@@@@@L@8@@@@@N@^@@@@@O@@@@@@@@8@C0@@@@N@\@@@@@C C@@@@@C0@@@@@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C98@@@@@@8@@@@@@@X<CC @@@F@8L@@@@C0@@^@@@@O?8@@@@@O@@A8@@@@8N@@@@@@@_ @@@@@@A0@@@@@@8@A @@@@A0F
@@@@@A0X8L@@@@@^@@@@@@@_ @@@@@@G@@@@@@A0\@@@@@@^@8@@@@@G@<@@@@@A0G@@@@@@8A0@@@@@G @@@@@@A0N@@@@@@\A0@@@@@G@\@@@@@A8@@@@@
@@C @@@@@@GG@@@@@@A8@@@@@@@\C \@@@@G@\@@@@@A0C @@@@@\@8@@@@@G@\@@@@@A0@@@@@@@8C@@@@@@G @@@@@@A0G@@@@@@\A @@@@@GC00@@@@@^
8@@@@@@^A @@@@@LC @@@@@C C0@@@@@@\@@@@@@@A @@@@@@@\@@@@@@LG@@@@@@G@^@@@@@C8G @@@@@@\@@@@@@A?0@@@@@A0O @@@@@C@O@@@@@C A0@
@@@C0@@@@@@@N@@<@@@@C C@@@@@@8@0@@@@@<@@@@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@<O@@@@@@N@@@@@@@FG@08@@@A OC@@@@@<@@G @@@
C @@@@@@C0@@^@@@@NC @@@@@@A<@@@@@@@\@@@@@@N@@X@@@@@^C@@@@@@^LOF@@@@@O @@@@@@C0@@@@@@C0@@@@@@@G@@@@@@G@N@@@@@C F@@@@@@\A0
@@@@@N@\@@@@@A8@@@@@@@\C @@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A1 @@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@8@8@@@@@G@O@@@@@C0G@@@@
@@\@@@@@@@G@@@@@@@A8@@@@@@@\A0@@@@@C X@@@@@A0<L@@@@@C,@@@@@@C 0@@@@@@A0@@@@@@<@<@@@@@@G@@@@@@@@0@@@@@@@C0@@@@@FA0@@@@@@@
G @@@@@<@8@@@@@@N@@@@@@@0>@@@@@@_N8@@@@@A??0@@@@@8@^@@@@@<@@@@@@@C @O@@@@@8@0@@@@@N@L@@@@@O@A?0@@@@8@A0@@@@N@@\@@@@C @@@
@@@@C @@@@@@NA8@@@@@C @@@@@@A!!8XN@@@@XA80@@@@G@@A8@@@@8@@@@@@@\@@G @@@C \@@@@@@@O @@@@@@G@@@@@@C @F@@@@@G 0@@@@@C#A1 @@@
@G<@@@@@@@<@@@@@@@8@@@@@@@O0@@@@@A0C @@@@@8@@@@@@@O@\@@@@@C G @@@@@^@@@@@@@G@8@@@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@\8@
@@@@@G @@@@@@A0NA0@@@@\A0@@@@@N@O@@@@@A0C0@@@@@8A0@@@@@G@@@@@@@A<@@@@@@@^@@@@@@@G@\@@@@@@8L@@@@@@\_F@@@@@@^@@@@@@@8L@@@@
@@@<@@@@@@O@O@@@@@@A0@@@@@@@X@@@@@@@@\@@@@@C \@@@@@@@@8@@@@@O@N@@@@@@C @@@@@@\C0@@@@@A?^@@@@@@X@\@@@@@N@C @@@@O@@@@@@@@8
@C0@@@@N@L@@@@@C @@@@@@C0@C0@@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C N@@@@@@8@@@@@@@XNFC @@@F@NL@@@@A0@@^@@@@N@@@@@@@G@@A8
@@@@8G@@@@@@@A8@@@@@@A0@@@@@@8@A @@@@@8X@@@@@@80\X@@@@A'' @@@@@@G@@@@@@@\@@@@@@@^\@@@@@@\@8@@@@@N@@@@@@@C0G@@@@@@8A8@@@@@
G @@@@@@A0N@@@@@@\A0@@@@@G@\@@@@@A8@@@@@@@C @@@@@@G^@@@@@@A8@@@@@@@\C \@@@@G@\@@@@@C C0@@@@@\@<@@@@@N@\@@@@@A0@@@@@@@G0@
@@@@@G @@@@@@A0G@@@@@@OC@@@@@@G''9 @@@@@G @@@@@@GF@@@@@@@N@@@@@@C0C @@@@@@\@@@@@@@L@@@@@@@@G @@@@@0G@@@@@@@@N@@@@@C C0@@@
@@@8@@@@@@N@^@@@@@@@G @@@@@F@G @@@@C @<@@@@A8@@@@@@@N@@8@@@@C @@@@@@@8@@@@@@@\@@8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8
C0@@@@@N@@@@@@@FC# 8@@@A C3@@@@@^@@G@@@@C @@@@@@A8@@\@@@@N@8@@@@@@@O@@@@@@@\@@@@@@N@@X@@@@@OF@@@@@@OXG,@@@@@08@@@@@@A0@@
@@@@O@@@@@@@^G@@@@@@G@N@@@@@C @@@@@@@8A0@@@@@O?>@@@@@A8@@@@@@@NG@@@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A=0@@@@@@^@@@@@@@
G@8G@@@@A0G@@@@@@8@<@@@@@G@O@@@@@C G@@@@@@\@@@@@@@@?@@@@@@A8@@@@@@@\A0@@@@@A1 @@@@@@9.X@@@@@@8@@@@@@A1 @@@@@@G@@@@@@@<@8
@@@@@@G@@@@@@@F@@@@@@@@@8@@@@@_??@@@@@@@C @@@@@<@<@@@@@@^@@@@@@C G @@@@@@A8@@@@@C@@8@@@@@8@O@@@@@^@@@@@@@C @^@@@@@8@@@@@
@@N@@@@@@@G@@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@^@@@@@C @@@@@@A <0N@@@@X@^0@@@@G @A0@@@@8@@@@@@@^O@G@@@@C O@@@@@@@
A0@@@@@@G@@@@@@C @F@@@@@A3 @@@@@C6@;@@@@@XG@@@@@@@\@@@@@@C @@@@@@OA0@@@@@A0C @@@@@8@@@@@@@N@\@@@@@C @@@@@@@^@@@@@@@A3 @@
@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@^^@@@@@@G @@@@@@A0NA0@@@@\A0@@@@@N@O@@@@@A0C0@@@@@8A0@@@@@G@@@@@@@@C8@@@@@@^@@@@@@@
G@\@@@@@@\X@@@@@@N[,@@@@@@_@@@@@@@NX@@@@@@C0@@@@@@O@^@@@@@@A0@@@@@@C@@@@@@@@@N@@@@@@@\@@@@@@@@8@@@@@O@O@@@@@@G @@@@@@8@8
@@@@@@@^@@@@@@0@O@@@@@N@C0@@@@G @C@@@@@8@G @@@@N@@@@@@@C @@@@@@A8@C @@@@N@@\@@@@C @G@@@@@8@@@@@@@@8@@@@@@C C0@@@@@8@@@@@
@@XGLC @@@F@C,@@@@@8@@<@@@@N@@@@@@@C/<C0@@@@8A0@@@@@0@\@@@@@@A0@@@@@@8@A @@@@@^0@@@@@@] N0@@@@NA8@@@@@@G@@@@@@A0@@@@@@C 
\@@@@@@\@8@@@@@N@@@@@@@C0G@@@@@@8@@@@@@@G @@@@@@A7 @@@@@@\A0@@@@@G@\@@@@@A8@@@@@@@C @@@@@@GC @@@@@A8@@@@@@@\C \@@@@G@\@@
@@@C C0@@@@@\@<@@@@@N@\@@@@@A0@@@@@@@@O@@@@@@G @@@@@@A0G@@@@@@C.@@@@@@C<;@@@@@@O0@@@@@@C,@@@@@@@8@@@@@@A0G @@@@@@\@@@@@@
A F@@@@@A0C @@@@@@G@@@@@@O@N@@@@@C0C @@@@@A8@@@@@@N@N@@@@@@@G@@@@@@X@C0@@@@C @<@@@@@<@@0@@@@N@C0@@@@C @L@@@@@8@@@@@@@N@@
8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@\@@@@@N@@0@@@@FA>@8@@@A @_@@@@@O@@N@@@@C @@@@@@@?G 8@@@@N@^@@@@@N@G@@@@@@@\@@@@
@@N@@0@@@@@G,@@@@@@G0C8@@@@C@O@@@@@@A0@@@@@@<@L@@@@@8G@@@@@@G@N@@@@@C @@@@@@@<A0@@@@@N@@@@@@@A8@@@@@@@X@@@@@@@G@\@@@@@A0
G@@@@@@^@@@@@@@@8@@@@@@A0\@@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@8@8@@@@@G@N@@@@@C G@@@@@@\@@@@@@@LA0@@@@@A8@@@@@@@\A0@@@@@@;@@@
@@@@_G0@@@@@CN@@@@@@@?@@@@@@@\C@@@@@@\A0@@@@@@G@@@@@@@8A @@@@@<A8@@@@@@A0@@@@@C0G @@@@@\@8@@@@@@^@@@@@@C C @@@@@@C0@@@@@
F@@\@@@@@8@N@@@@@G @X@@@@C @8@@@@@8@G@@@@@N@@@@@@@A0@N@@@@@8@A0@@@@N@@\@@@@C @@@@@@@C @@@@@@N@G @@@@C @X@@@@A O N@@@@X@G
0@@@@A0@G@@@@@8@@@@@@@G <\@@@@C C @@@@C C @@@@@@G@@@@@@A0@L@@@@@@>@@@@@@A<@>@@@@A A0@@@@@@\@@@@@@N@C@@@@@NA0@@@@@A0C @@@
@@<@0@@@@@G@\@@@@@C0C@@@@@@^@@@@@@@G@@@@@@@A0G@@@@@@\A0@@@@@G @@@@@@@N@@@@@@@\G @@@@@G @@@@@@A0NA0@@@@\A0@@@@@G@N@@@@@A0
C @@@@@\A0@@@@@G@@@@@@@C@\@@@@@@^@@@@@@@G@\@@@@@@O0@@@@@@G1<@@@@@A#0@@@@@@G0@@@@@@O@0@@@@@G \@@@@@@A0@@@@@@\@X@@@@@O@^@@
@@@@@\@@@@@@<A0@@@@@G@N@@@@@@G @@@@@@8@8@@@@@G 8@@@@@A @G @@@@N@G @@@@A8@L@@@@@8@<@@@@@N@A @@@@C @@@@@@@^@C @@@@N@@\@@@@
C @G@@@@@8@@@@@@@@8@@@@@@C @<@@@@@8@F@@@@@XC0C @@@F@@<@@@@@N@C @@@@N@@@@@@@@<G/@@@@@8@<@@@@@\@8@@@@@@A0@@@@@@^@F@@@@@@O 
@@@@@@O@G @@@@8@^@@@@@@G@@@@@@G@A0@@@@C ] @@@@@\A0@@@@@G@L@@@@@A0O@@@@@@\@0@@@@@G @@@@@@A?0@@@@@@\A0@@@@@G@\@@@@@A8@@@@@
@@C @@@@@@G@8@@@@@A8@@@@@@@\C \@@@@G@\@@@@@A0G@@@@@@^A8@@@@@G@\@@@@@A0@@@@@@@0G@@@@@@C#@@@@@@A0O@@@@@@A8@@@@@@A8^@@@@@@8
\@@@@@@A8@@@@@@G \@@@@@@8N@@@@@@@\@@@@@@F@N@@@@@C0G@@@@@@@G@@@@@@G@\@@@@@@8G@@@@@@A8@@@@@@N@^@@@@@A8N@@@@@@0@A8@@@@C C0@
@@@@G F@@@@@N@^@@@@@C @8@@@@@8@@@@@@@A8A8@@@@C @G@@@@@8@A0@@@@N@@@@@@@@N@@@@@@@8@G @@@@N@C @@@@F@<@8@@@A @G@@@@@A8A0@@@@
C @@@@@@@G ?@@@@@N@G@@@@@G \@@@@@@@\@@@@@@C0C@@@@@@A0@@@@@@C A0@@@@L@C0@@@@@A0@@@@@C0@8@@@@@<OX@@@@@G 8@@@@@@8F@@@@@@NC8
@@@@@C X@@@@@A8@@@@@@@O?0@@@@@G@\@@@@@A0G@@@@@@^@@@@@@@@8@@@@@@A0O@@@@@@^@@@@@@@G@8G@@@@A0G@@@@@@NA @@@@@G <@@@@@@8O@@@@
@@\@@@@@@@NA @@@@@@80@@@@@@NG0@@@@@@^@@@@@@@NC @@@@@LC @@@@@@N@@@@@@A0G@@@@@@GC @@@@@@O@@@@@@C?? @@@@@\C @@@@@@A0@@@@@A0
N@@@@@@OA0@@@@@@^@@@@@@A0G@@@@@@^G@@@@@@^@@_@@@@A<C8@@@@@@>O@@@@@G1>@@@@@A<@>@@@@@_@@@@@@@@O!!8@@@@A<@C0@@@@_@@<@@@@C0@@@
@@@@C @@@@@@_@A<@@@@G0C8@@@@A0G@^@@@@\@A0@@@@@GA8@@@@A<@@@@@@@@\O @@@@G0A8@@@@A>N@@@@@@@O@@@@@@@_G @@@@@@\@@@@@@@8@\@@@@
O@@>@@@@@@<@@@@@@8@>@@@@@G/>@@@@@A.\@@@@@@G''@@@@@@A;?@@@@@@^\@@@@@@^@@@@@@@FA<@@@@@A8G @@@@@^A8@@@@@G @@@@@@@N@@@@@@@^A8
@@@@@G @@@@@@A0^C0@@@@^A8@@@@@A10@@@@@A>^@@@@@@G?0@@@@@G@@@@@@@C80@@@@@@O8@@@@@@C?_@@@@@@C@@@@@@@C 8@@@@@O@<@@@@@@C@@@@@
@@8G0@@@@@@;0@@@@@@G8@@@@@@??0@@@@@C!!0@@@@@@@>@@@@@@GG@@@@@@A98@@@@@@G @@@@@@OG @@@@@C'' @@@@@O8@O8@@@@??0@@@@@@A?@@@@@C?
0@@@@@@???@@@@@O<@@@@@@@@_0@@@@@? C?@@@@O8@?0@@@C?@@@@@@@@8@@@@@@O8A? @@@C??<@@@@@?A _8@@@O0@L@@@@@@_0@@@@@?0@@@@@@@A?8@
@@@C>@O @@@@X>@@@@@@@O<@@@@@@@?@@@@@@@G@@@@@@@F@C@@@@G<@?0@@@@@?0@@@@@_?? @@@@@^F@@@@@@X<@@@@@@@_ @@@@@@G&@@@@@@A<@@@@@@
_8@@@@@@C C @@@@A?G<@@@@@_1?@@@@@G>@@@@@@@C @@@@@@_1?@@@@@G>@@@@@@A?O9?@@@@_1?@@@@@@G8@@@@@@\>@@@@@@@^\@@@@@G>@@@@@@@38@
@@@@@@<@@@@@@@OF@@@@@@@0@@@@@@@0L@@@@@G8? @@@@@@0@@@@@@O?8@@@@@@G0@@@@@@O?0@@@@@O?<@@@@@@O0@@@@@@A?<@@@@@@_@@@@@@@G8@@@@
@@A0@@@@@@@? @@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@<N@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@0@8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@8@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@OG@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A>@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@N@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@N@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A0@@@@@@@@A0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@F@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C3 @@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@O@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A3@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@\@@@@@@@@@\@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@_0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\C @@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G@@@@@@@@@G@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@_ @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@A?0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G8@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@G>
@@@@@@@@G<@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@C0@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@a') ; yourself); yourself]
!

charMapImage3_height
    ^ 37
!

charMapImage3_width
    ^ 16
!

getCharMapImage:idx
     ^ self perform:('charMapImage',idx printString) asSymbol

     "
      self getCharMapImage:1
      self getCharMapImage:2
      self getCharMapImage:3
     "
! !

!HumanReadableImageGenerator class methodsFor:'support'!

fonts
    |fonts|

    Display notNil ifTrue:[
        fonts := OrderedCollection new.
        fonts add:(Font
                    family:'Segoe Script'
                    face:'bold'
                    style:'roman'
                    size:24
                    encoding:nil).

        fonts add:(Font
                    family:'Segoe Print'
                    face:'bold'
                    style:'roman'
                    size:24
                    encoding:nil).

        fonts add:(Font
                    family:'MS PMincho'
                    face:'bold'
                    style:'roman'
                    size:28
                    encoding:nil).
    ].
    ^ fonts
!

generateFontMaps
    "generates the methods:
        charMapImage1,
        charMapImage2,
        ...
     one for each font"

    |fontIndex chars wCharBox|

    wCharBox := 50.
    chars := self characterMapCharacters.

    fontIndex := 1.
    self fonts do:[:font |
        |charForm x yBaseLine deviceFont image mask|

        deviceFont := font onDevice:Screen current.

        charForm := Form extent:(wCharBox*chars size)@(deviceFont height) depth:1.
        charForm paint:(Color colorId:1) on:(Color colorId:0).
        charForm clear.
        charForm font:deviceFont.

        x := 0.
        yBaseLine := deviceFont ascent.
        chars do:[:eachChar |
            charForm displayString:(eachChar asString) x:x y:yBaseLine.
            x := x + wCharBox.
        ].

        image := Image fromForm:charForm.
        mask := Image fromForm:charForm.
        image mask:mask.

        self 
            generateImageMethod:('charMapImage%1' bindWith:fontIndex) 
            with:image
            forFont:font.

        fontIndex := fontIndex + 1.
    ].

    self class
        compile:'numberOfCharMaps
    ^ ',self fonts size printString
        classified:'defaults'.

    "
     self generateFontMaps
    "
!

generateImageMethod:selector with:image forFont:aFont
    |fontWidth fontHeight|

    fontHeight := (aFont onDevice:Screen default) height.
    fontWidth := (aFont onDevice:Screen default) width.

    SmalltalkCodeGeneratorTool
        createImageSpecMethodFor:image
        comment:'"character bitmaps (needed ebcause the webServer has no Display for drawing)"'
        in:self class
        selector:selector asSymbol.

    SmalltalkCodeGeneratorTool
        compile:('%1_width\    ^ %2' bindWith:selector with:fontWidth) withCRs 
        forClass:self class 
        inCategory:'image specs'.

    SmalltalkCodeGeneratorTool
        compile:('%1_height\    ^ %2' bindWith:selector with:fontHeight) withCRs
        forClass:self class
        inCategory:'image specs'.

    "
     self generateFontMaps
    "

    "Modified: / 31-01-2011 / 18:28:57 / cg"
! !

!HumanReadableImageGenerator methodsFor:'accessing'!

characterSet:something
    characterSet := something.
!

colors:aCollectionOfColors
    colors := aCollectionOfColors.

    "Modified: / 01-02-2011 / 10:30:40 / cg"
!

extent:aPoint
    extent := aPoint.

    "Modified: / 01-02-2011 / 10:30:50 / cg"
!

maxAngle:something
    maxAngle := something.
!

minAngle:something
    minAngle := something.
!

numberOfRandomDots:anInteger
    numberOfRandomDots := anInteger.

    "Modified: / 01-02-2011 / 10:31:04 / cg"
!

numberOfRandomLines:anInteger
    numberOfRandomLines := anInteger.

    "Modified: / 01-02-2011 / 10:31:09 / cg"
!

string
    string isNil ifTrue:[
	self generateString
    ].
    ^ string
!

string:something
    string := something.
!

stringLength:something
    stringLength := something.
! !

!HumanReadableImageGenerator methodsFor:'initialization'!

computeExtent
    |maxFontHeight maxFontWidth wEach minHeight|

    maxFontHeight := maxFontWidth := 0.
    fonts notNil ifTrue:[
        fonts do:[:font |
            maxFontHeight := maxFontHeight max:(font onDevice:Screen default) height.
            maxFontWidth := maxFontWidth max:(font onDevice:Screen default) width.
        ].
    ] ifFalse:[
        "/ what a hack
        maxFontHeight := (self class charMapImage1_height
                         max: self class charMapImage2_height)
                         max: self class charMapImage3_height.
        maxFontWidth := (self class charMapImage1_width
                         max: self class charMapImage2_width)
                         max: self class charMapImage3_width.
    ].

    minHeight := (maxFontHeight * 2) max: 50.
    wEach := (maxFontWidth * 2) max: 20.

    extent := (string size * wEach * 2) @ (minHeight * 2).
!

generateString
    string := String withAll:( (1 to:stringLength) collect:[:i | self randomCharacter]).
!

initialize
    minAngle := -30.
    maxAngle := 45.
    stringLength := 7.
    characterSet := 'ABCDEFGHKMNPRSTUVWXYZaefhmnrt12345678'.
    numberOfRandomLines := 20.
    numberOfRandomDots := 500.
    Display notNil ifTrue:[
        fonts := self class fonts.
    ].

    "
     |gen image|

     gen := HumanReadableImageGenerator new.
     gen numberOfRandomDots:5000.
     gen numberOfRandomLines:10.
     gen minAngle:-45.
     gen maxAngle:45.
     image := gen generate.
     image inspect.
    "
!

randomCharacter
    ^ characterSet atRandom
! !

!HumanReadableImageGenerator methodsFor:'private'!

generate
    ^ self generateImageWithoutUsingDisplay

    "
     HumanReadableImageGenerator new generate inspect
    "
!

generateFormUsingDisplay
    |fontHeight fontWidth fontDescent fontAscent
     xCharBoxLeft yBaseline deltaXBetweenCharacters jitterX jitterY
     depth hasRGBColor hasPaletteColor useColors
     black white|

    string isNil ifTrue:[
	self generateString
    ].
    extent isNil ifTrue:[
	self computeExtent
    ].

    useColors := true.
    hasRGBColor := hasPaletteColor := false.

    useColors ifTrue:[
	depth := Display depth.
	hasRGBColor := (depth == 24).
	hasPaletteColor := (depth == 8).

	(hasRGBColor or:[hasPaletteColor]) ifTrue:[
	    colors isNil ifTrue:[
		colors := OrderedCollection new.
		colors add:(Color black).
		colors add:(Color white).
		colors add:(Color red).
		colors add:(Color green darkened).
		colors add:(Color blue).
		colors add:(Color cyan darkened).
		colors add:(Color magenta darkened).
		colors add:(Color brown).
		colors add:(Color red darkened).
		colors add:(Color green darkened darkened).
		colors add:(Color blue lightened).
		colors := colors asArray.
	    ]
	] ifFalse:[
	    useColors := false.
	].
    ].

    useColors ifFalse:[
	depth := 1.
    ].

    hasPaletteColor ifTrue:[
	black := (Color colorId:0).
	white := (Color colorId:1).
    ] ifFalse:[
	black := (Color colorId:0).
	white := (Color colorId:(1 bitShift:depth)-1).
    ].

    form := Form extent:extent depth:depth.
    form paint:black on:white.
    form clear.

    1 to:numberOfRandomLines do:[:n |
	|clr yLineStart yLineEnd|

	yLineStart := Random nextIntegerBetween:0 and:form height.
	yLineEnd := Random nextIntegerBetween:0 and:form height.
	useColors ifTrue:[
	    clr := colors at:(Random nextIntegerBetween:3 and:colors size).
	    form paint:clr on:white.
	].
	form
	    displayLineFromX:0 y:yLineStart toX:form width y:yLineEnd.
    ].
    1 to:numberOfRandomLines do:[:n |
	|clr xLineStart xLineEnd|

	xLineStart := Random nextIntegerBetween:0 and:form width.
	xLineEnd := Random nextIntegerBetween:0 and:form width.
	useColors ifTrue:[
	    clr := colors at:(Random nextIntegerBetween:3 and:colors size).
	    form paint:clr on:white.
	].
	form
	    displayLineFromX:xLineStart y:0 toX:xLineEnd y:form height.
    ].
    1 to:numberOfRandomDots do:[:n |
	|clr xDot yDot|

	xDot := Random nextIntegerBetween:0 and:form width-1.
	yDot := Random nextIntegerBetween:0 and:form height-1.
	useColors ifTrue:[
	    clr := colors at:(Random nextIntegerBetween:1 and:colors size).
	    form paint:clr on:white.
	].
	form
	    displayPointX:xDot y:yDot.
    ].

    deltaXBetweenCharacters := form width / (string size).

    xCharBoxLeft := 0.

    string do:[:eachCharacter |
	|xCharBoxRight xChar yChar rndXJitter rndYJitter rndAngle
	 charWidth clr font|

	font := fonts atRandom onDevice:Screen current.
	form font:font.

	yBaseline := (form height + font descent) // 2.
	fontHeight := font height.
	fontWidth := font width.
	fontDescent := font descent.
	fontAscent := font ascent.

	charWidth := font widthOf:(eachCharacter asString).

	jitterX := (deltaXBetweenCharacters-charWidth) // 2.
	jitterY := (form height - fontHeight) // 2.

	xCharBoxRight := xCharBoxLeft + deltaXBetweenCharacters.

	rndXJitter := (Random nextIntegerBetween:(jitterX negated) and:jitterX).
	rndYJitter := (Random nextIntegerBetween:(jitterY negated) and:jitterY).
	rndAngle := Random nextIntegerBetween:minAngle and:maxAngle.

	xChar := xCharBoxLeft + (deltaXBetweenCharacters // 2) + rndXJitter.
	yChar := yBaseline + rndYJitter.

	"/ ensure that even after rotation, the result still lies inside the charBox.
	xChar := xChar max:(xCharBoxLeft + (fontHeight*0.7) + charWidth).
	xChar := xChar min:(xCharBoxRight - (fontHeight*0.7) - charWidth).

	yChar := yChar max:(fontAscent).
	yChar := yChar min:(form height - fontHeight).

	useColors ifTrue:[
	    clr := colors at:(Random nextIntegerBetween:3 and:colors size).
	    form paint:clr on:white.
	].

	form
	    displayString:(eachCharacter asString)
	    x:xChar y:yChar
	    angle:rndAngle
	    opaque:false.

	xCharBoxLeft := xCharBoxRight.
    ].

    ^ form

    "
     HumanReadableImageGenerator new generateFormUsingDisplay inspect
    "
!

generateImageWithoutUsingDisplay
    |xCharBoxLeft deltaXBetweenCharacters jitterX jitterY
     depth hasRGBColor hasPaletteColor useColors
     black white  maxDark|

    string isNil ifTrue:[
        self generateString
    ].
    extent isNil ifTrue:[
        self computeExtent
    ].

    useColors := true.
    hasRGBColor := false.
    hasPaletteColor := true.
    depth := 8.

    colors isNil ifTrue:[
"/        colors := OrderedCollection new.
"/        colors add:(Color black).
"/        colors add:(Color white).
"/        colors add:(Color red).
"/        colors add:(Color green darkened).
"/        colors add:(Color blue).
"/        colors add:(Color cyan darkened).
"/        colors add:(Color magenta darkened).
"/        colors add:(Color brown).
"/        colors add:(Color red darkened).
"/        colors add:(Color green darkened darkened).
"/        colors add:(Color blue lightened).
"/        colors := colors asArray.

        colors := OrderedCollection new.
        colors add:(Color black).
        colors add:(Color white).

"/        colors add:(Color green darkened).
"/        colors add:(Color blue).
"/        colors add:(Color blue darkened).
"/        colors add:(Color red darkened).
"/
"/        colors add:(Color red darkened darkened).
"/        colors add:(Color green darkened).
"/
"/        maxDark := colors size.
        colors add:(Color red lightened lightened).
        colors add:(Color green).
        colors add:(Color blue lightened lightened).
        colors add:(Color cyan lightened).
        colors add:(Color magenta lightened).
        colors add:(Color brown lightened lightened).
        colors add:(Color red lightened).
        colors add:(Color green lightened).
        colors add:(Color blue lightened lightened).
        colors add:(Color yellow).
        colors := colors asArray.
    ].

    black := (Color colorId:0).
    white := (Color colorId:1).

    form := Depth8Image extent:extent depth:8 palette:colors.

    1 to:numberOfRandomLines//2 do:[:n |
        |clrIdx yLineStart yLineEnd|

        yLineStart := Random nextIntegerBetween:0 and:form height-1.
        yLineEnd := Random nextIntegerBetween:0 and:form height-1.
        useColors ifTrue:[
            clrIdx := Random nextIntegerBetween:3 and:(maxDark ? colors size).
        ].
        form
            drawLineFrom:(0 @ yLineStart) to:((form width-1) @ yLineEnd) withColor:clrIdx-1.
    ].
    1 to:numberOfRandomLines do:[:n |
        |clrIdx xLineStart xLineEnd|

        xLineStart := Random nextIntegerBetween:0 and:form width-1.
        xLineEnd := Random nextIntegerBetween:0 and:form width-1.
        useColors ifTrue:[
            clrIdx := Random nextIntegerBetween:3 and:(maxDark ? colors size).
        ].
        form
            drawLineFrom:(xLineStart @ 0) to:(xLineEnd @ (form height-1)) withColor:clrIdx-1.
    ].
    1 to:numberOfRandomDots do:[:n |
        |clrIdx xDot yDot|

        xDot := Random nextIntegerBetween:0 and:form width-1.
        yDot := Random nextIntegerBetween:0 and:form height-1.
        useColors ifTrue:[
            clrIdx := Random nextIntegerBetween:3 and:(maxDark ? colors size).
        ].
        form
            atImageAndMask:xDot@yDot put:clrIdx.
    ].

    deltaXBetweenCharacters := form width / (string size).

    xCharBoxLeft := 0.

    string do:[:eachCharacter |
        |charIdx x y xCharBoxRight xChar yChar rndXJitter rndYJitter rndAngle
         charWidth clrIdx font charMap charImage|

        charMap := self class getCharMapImage:(1 to:self class numberOfCharMaps) atRandom.

        charIdx := self class characterMapCharacters indexOf:eachCharacter.
        x := (charIdx-1)*50.
        y := 0.
        charImage := charMap subImageIn:((x@y) extent:(50@charMap height)).
        charImage mask:(charMap mask subImageIn:((x@y) extent:(50@charMap height))).

        useColors ifTrue:[
            charImage := Depth8Image fromImage:charImage.
            charImage colorMap:(Array with:(colors at:(Random nextIntegerBetween:(maxDark ? 3)+1 and:colors size))).
        ].

        jitterX := 20. "/ (deltaXBetweenCharacters) // 2.
        jitterY := (form height - charImage height) // 2.

        xCharBoxRight := xCharBoxLeft + deltaXBetweenCharacters.

        rndXJitter := (Random nextIntegerBetween:(jitterX negated) and:jitterX).
        rndYJitter := (Random nextIntegerBetween:(jitterY negated) and:jitterY).
        rndAngle := Random nextIntegerBetween:minAngle and:maxAngle.

        charImage := charImage rotated:rndAngle.

        xChar := xCharBoxLeft + (deltaXBetweenCharacters // 2) + rndXJitter.
        yChar := ((form height - charImage height) // 2) + rndYJitter.

        xChar := xChar - (rndAngle degreesToRadians sin * 35) rounded.
        yChar := yChar - (rndAngle degreesToRadians cos * 35) rounded.

        "/ ensure that even after rotation, the result still lies inside the charBox.
        xChar := xChar max:xCharBoxLeft.
        xChar := (xChar min:(xCharBoxRight - 30)) max:0.
        xChar := (xChar min:(form width - 60)) max:0.

        yChar := yChar max:5.
        yChar := (yChar min:(form height - charImage height)) max:0.

"/        form
"/            displayLineFromX:xCharBoxRight y:0 toX:xCharBoxRight y:form height.

        form
            copyMaskedFrom:charImage
            x:0 y:0
            toX:xChar rounded y:yChar rounded
            width:charImage width height:charImage height.

        xCharBoxLeft := xCharBoxRight.
    ].

    ^ form

    "
     HumanReadableImageGenerator new generateImageWithoutUsingDisplay inspect
    "

    "Modified: / 01-02-2011 / 10:36:49 / cg"
! !

!HumanReadableImageGenerator methodsFor:'public'!

generateImage
    ^ self generate
! !

!HumanReadableImageGenerator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HumanReadableImageGenerator.st,v 1.13 2011/02/01 09:38:24 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/HumanReadableImageGenerator.st,v 1.13 2011/02/01 09:38:24 cg Exp $'
! !
