"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Lib
	instanceVariableNames:''
	classVariableNames:'Debugging ExecutionLock'
	poolDictionaries:''
	category:'DataBase-Oracle8-Lib'
!

!OCI8Lib primitiveDefinitions!
%{

/*
 * includes, defines, structure definitions
 * and typedefs come here.
 */

#include <oci.h>
/* #include <xa.h> */
#include <stdio.h>
#include <malloc.h>

/*  internal/external datatype codes */
#define VARCHAR2_TYPE           SQLT_CHR
#define NUMBER_TYPE             SQLT_NUM
#define INTEGER_TYPE            SQLT_INT
#define FLOAT_TYPE              SQLT_FLT
#define STRING_TYPE             SQLT_STR
#define VARNUM_TYPE             SQLT_VNU
#define LONG_TYPE               SQLT_LNG
#define VARCHAR_TYPE            SQLT_VCS
#define ROWID_TYPE              SQLT_RID
#define DATE_TYPE               SQLT_DAT
#define VARRAW_TYPE             SQLT_VBI
#define RAW_TYPE                SQLT_BIN
#define LONG_RAW_TYPE           SQLT_LBI
#define UNSIGNED_INT_TYPE       SQLT_UIN
#define LONG_VARCHAR_TYPE       SQLT_LVC
#define LONG_VARRAW_TYPE        SQLT_LVB
#define CHAR_TYPE               SQLT_AFC
#define CHARZ_TYPE              SQLT_AVC
#define ROWID_DESC_TYPE         SQLT_RDD
#define NAMED_DATA_TYPE         SQLT_NTY
#define REF_TYPE                SQLT_REF
#define CHARACTER_LOB_TYPE      SQLT_CLOB
#define BINARY_LOB_TYPE         SQLT_BLOB
#define BINARY_FILE_TYPE        SQLT_FILE
#define OCI_STRING_TYPE         SQLT_VST
#define OCI_DATE_TYPE           SQLT_ODT

static OBJ
__ociReturnSymbol(__retCode)
    int __retCode;
{
    switch (__retCode) {
    case OCI_SUCCESS:           return(@symbol(OCI_SUCCESS));
    case OCI_SUCCESS_WITH_INFO: return(@symbol(OCI_SUCCESS_WITH_INFO));
    case OCI_ERROR:             return(@symbol(OCI_ERROR));
    case OCI_STILL_EXECUTING:   return(@symbol(OCI_STILL_EXECUTING));
    case OCI_NO_DATA:           return(@symbol(OCI_NO_DATA));
    case OCI_NEED_DATA:         return(@symbol(OCI_NEED_DATA));
    case OCI_INVALID_HANDLE:    return(@symbol(OCI_INVALID_HANDLE));
    case OCI_CONTINUE:          return(@symbol(OCI_CONTINUE));       
    default:                    return(@symbol(unknownReturnCode));
    }
}

%}
! !

!OCI8Lib class methodsFor:'documentation'!

documentation
"
    Interface class for the Oracle Call Interface (OCI). All methods start with
    oci (e.g. ociBindDynamic:callBackData:errorHandle:). For every method starting with
    oci a method starting with primOci exists. Those primitive methods include the
    c code which does the oci lib call.

    [author:]
	Pierre Schwarz

    [see also:]
	all OCI8* classes.

    [instance variables:]
	none

    [class variables:]
	Debugging <Boolean>
	    Swichtes on/off debug traces from C-level
		Debugging := true
		Debugging := false.

	Signals <IdentityDictionary>
	    Holds all oci signals for error handling. The keys are the symbols
	    returned by the method OCI8Lib class errorSymbolForStatus:.

	OCISuccessInteger <Integer>             
	    Integer representing the C define OCI_SUCCESS

	OCISuccessWithInfoInteger <Integer>     
	    Integer representing the C define OCI_SUCCESS_WITH_INFO

	ExecutionSignal <RecursionLock>         
	    Lock which prevents that multiple oci primitives are executed parallel.
"



! !

!OCI8Lib class methodsFor:'initialization'!

initialize
"
initialize every class variable

<return: self>
"

    ExecutionLock := RecursionLock new.
    Debugging := false.

"
self initialize
"
! !

!OCI8Lib class methodsFor:'buffer handling'!

bufferForSize:aBufferSize
"
returns an external buffer of the desired size. The buffer is
prevented from garbage collection, hence it dosent change its address and
must be freed by the caller.

<return: ExternalBytes>
"
    ^ ExternalBytes new:aBufferSize

! !

!OCI8Lib class methodsFor:'constants'!

numericValueForAttributes:anOCIAtributeSymbol
"
converts a symbol anOCIAtributeSymbol to its corresponding integer value.

<return: Integer>
"

%{                                      
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_AGENT_ADDRESS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_AGENT_ADDRESS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_AGENT_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_AGENT_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_AGENT_PROTOCOL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_AGENT_PROTOCOL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ALLOC_DURATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ALLOC_DURATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ATTEMPTS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ATTEMPTS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_BUF_ADDR)) {
	RETURN (__MKSMALLINT(OCI_ATTR_BUF_ADDR));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_BUF_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_BUF_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CACHE_ARRAYFLUSH)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CACHE_ARRAYFLUSH));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CACHE_MAX_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CACHE_MAX_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CACHE_OPT_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CACHE_OPT_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CHAR_COUNT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CHAR_COUNT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CHARSET_ID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CHARSET_ID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CHARSET_FORM)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CHARSET_FORM));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_COL_COUNT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_COL_COUNT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CONSUMER_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CONSUMER_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CORRELATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_CORRELATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_COMPLEXOBJECT_LEVEL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_COMPLEXOBJECT_LEVEL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_COMPLEXOBJECT_COLL_OUTOFLINE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_COMPLEXOBJECT_COLL_OUTOFLINE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_COMPLEXOBJECTCOMP_TYPE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_COMPLEXOBJECTCOMP_TYPE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_COMPLEXOBJECTCOMP_TYPE_LEVEL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_COMPLEXOBJECTCOMP_TYPE_LEVEL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_CURSOR_COMMIT_BEHAVIOR)) {
	RETURN (__MKUINT(OCI_ATTR_CURSOR_COMMIT_BEHAVIOR));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DATA_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DATA_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DATA_TYPE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DATA_TYPE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DATEFORMAT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DATEFORMAT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DELAY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DELAY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DEQ_MODE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DEQ_MODE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DEQ_MSGID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DEQ_MSGID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DIRPATH_MODE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DIRPATH_MODE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DIRPATH_NOLOG)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DIRPATH_NOLOG));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DIRPATH_PARALLEL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DIRPATH_PARALLEL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_DML_ROW_OFFSET)) {
	RETURN (__MKSMALLINT(OCI_ATTR_DML_ROW_OFFSET));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ENV)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ENV));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ENQ_TIME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ENQ_TIME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_EXCEPTION_QUEUE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_EXCEPTION_QUEUE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_EXPIRATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_EXPIRATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_EXTERNAL_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_EXTERNAL_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_FOCBK)) {
	RETURN (__MKSMALLINT(OCI_ATTR_FOCBK));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_HEAPALLOC)) {
	RETURN (__MKSMALLINT(OCI_ATTR_HEAPALLOC));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_IN_V8_MODE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_IN_V8_MODE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_INTERNAL_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_INTERNAL_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_IS_NULL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_IS_NULL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LIST_COLUMNS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LIST_COLUMNS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LIST_OBJECTS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LIST_OBJECTS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LIST_SCHEMAS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LIST_SCHEMAS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LIST_TYPE_ATTRS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LIST_TYPE_ATTRS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LIST_TYPE_METHODS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LIST_TYPE_METHODS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_LOBEMPTY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_LOBEMPTY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MAXDATA_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MAXDATA_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MEMPOOL_APPNAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MEMPOOL_APPNAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MEMPOOL_HOMENAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MEMPOOL_HOMENAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MEMPOOL_INSTNAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MEMPOOL_INSTNAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MEMPOOL_MODEL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MEMPOOL_MODEL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MEMPOOL_SIZE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MEMPOOL_SIZE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MIGSESSION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MIGSESSION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_MSG_STATE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_MSG_STATE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NAVIGATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NAVIGATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NONBLOCKING_MODE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NONBLOCKING_MODE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NUM_COLS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NUM_COLS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NUM_DML_ERRORS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NUM_DML_ERRORS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NUM_PARAMS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NUM_PARAMS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_NUM_ROWS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_NUM_ROWS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_OBJECT_DETECTCHANGE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_OBJECT_DETECTCHANGE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_OBJECT_NEWNOTNULL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_OBJECT_NEWNOTNULL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_OBJECT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_OBJECT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ORIGINAL_MSGID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ORIGINAL_MSGID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PARAM)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PARAM));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PARAM_COUNT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PARAM_COUNT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PARSE_ERROR_OFFSET)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PARSE_ERROR_OFFSET));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PASSWORD)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PASSWORD));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PDPRC)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PDPRC));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PDSCL)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PDSCL));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PIN_DURATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PIN_DURATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PINOPTION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PINOPTION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PRECISION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PRECISION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PREFETCH_MEMORY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PREFETCH_MEMORY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PREFETCH_ROWS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PREFETCH_ROWS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_PRIORITY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_PRIORITY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_RECIPIENT_LIST)) {
	RETURN (__MKSMALLINT(OCI_ATTR_RECIPIENT_LIST));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_RELATIVE_MSGID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_RELATIVE_MSGID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ROW_COUNT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ROW_COUNT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ROWID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ROWID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_ROWS_RETURNED)) {
	RETURN (__MKSMALLINT(OCI_ATTR_ROWS_RETURNED));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SCALE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SCALE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SCHEMA_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SCHEMA_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SENDER_ID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SENDER_ID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SERVER)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SERVER));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SERVER_GROUP)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SERVER_GROUP));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SESSION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SESSION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SEQUENCE_DEVIATION)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SEQUENCE_DEVIATION));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SHARED_HEAPALLOC)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SHARED_HEAPALLOC));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SQLFNCODE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SQLFNCODE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_STMT_TYPE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_STMT_TYPE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_STREAM_OFFSET)) {
	RETURN (__MKSMALLINT(OCI_ATTR_STREAM_OFFSET));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUB_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUB_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUBSCR_CALLBACK)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUBSCR_CALLBACK));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUBSCR_CTX)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUBSCR_CTX));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUBSCR_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUBSCR_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUBSCR_NAMESPACE)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUBSCR_NAMESPACE));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_SUBSCR_PAYLOAD)) {
	RETURN (__MKSMALLINT(OCI_ATTR_SUBSCR_PAYLOAD));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_TRANS)) {
	RETURN (__MKSMALLINT(OCI_ATTR_TRANS));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_TRANS_NAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_TRANS_NAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_USERNAME)) {
	RETURN (__MKSMALLINT(OCI_ATTR_USERNAME));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_VISIBILITY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_VISIBILITY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_VISIBILITY)) {
	RETURN (__MKSMALLINT(OCI_ATTR_VISIBILITY));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_WAIT)) {
	RETURN (__MKSMALLINT(OCI_ATTR_WAIT));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_ATTR_XID)) {
	RETURN (__MKSMALLINT(OCI_ATTR_XID));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_CURSOR_CLOSED)) {
	RETURN (__MKUINT(OCI_CURSOR_CLOSED));
    }
    if (anOCIAtributeSymbol == @symbol(OCI_CURSOR_OPEN)) {
	RETURN (__MKUINT(OCI_CURSOR_OPEN));
    }
%}.
    OCIInternalError raiseErrorString:' - bad mode arg: ', anOCIAtributeSymbol printString.
!

numericValueForBindByPosType:anOCIBindByPosTypeSymbol
"
converts a symbol anOCIBindByPosTypeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT      - This is default mode.
	OCI_DATA_AT_EXEC - When this mode is selected, the value_sz parameter defines the maximum
			   size of the data that can be ever provided at runtime. The application must
			   be ready to provide the OCI library runtime IN data buffers at any time and 
			   any number of times. Runtime data is provided in one of the two ways:

			   - callbacks using a user-defined function which must be registered with
			     a subsequent call to OCIBindDynamic(). 
			   - a polling mechanism using calls supplied by the OCI.
			     This mode is assumed if no callbacks are define.

<return: Integer>     
"
%{  
    if (anOCIBindByPosTypeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCIBindByPosTypeSymbol == @symbol(OCI_DATA_AT_EXEC)) {
	RETURN (__MKUINT(OCI_DATA_AT_EXEC));
    }
%}.
    OCIInternalError raiseErrorString:' - bad bind mode arg: ', anOCIBindByPosTypeSymbol printString.


    "
     self numericValueForBindByPosType:#'OCI_DEFAULT' 
     self numericValueForBindByPosType:#'OCI_DATA_AT_EXEC' 
    "
!

numericValueForCDataType:anOCICDataTypeSymbol
"
converts a symbol anOCICDataTypeSymbol to its corresponding integer value.

The possible values are:

    C_TYPE_INTEGER    - The INTEGER datatype converts numbers. An external integer is a signed binary number;
			the size in bytes is system dependent. The host system architecture determines the
			order of the bytes in the variable.

    C_TYPE_FLOAT      - The FLOAT datatype processes numbers that have fractional parts or that exceed
			the capacity of an integer. The number is represented in the host system's
			floating-point format. Normally the length is either four or eight bytes.

    C_TYPE_DATE       - This datatype holds date and time values.

    C_TYPE_STRING     - The null-terminated STRING format behaves like the VARCHAR2 format (datatype code 1),
			except that the string must contain a null terminator character.

    C_TYPE_RAW        - The RAW datatype is used for binary data or byte strings that are not to be interpreted
			by Oracle. The maximum length of a RAW column is 2000 bytes.

    C_TYPE_VARNUM     - The VARNUM datatype is like the external NUMBER datatype, except that the first byte contains
			the length of the number representation. This length does not include the length byte itself.
			Reserve 22 bytes to receive the longest possible VARNUM. Set the length byte when you send a
			VARNUM value to Oracle.

    C_TYPE_LONG_RAW   - The LONG RAW datatype is similar to the RAW datatype, except that it stores raw data with 
			a length up to two gigabytes (2^31-1 bytes).

    C_TYPE_LONG       - The LONG datatype stores character strings longer than 4000 bytes. You can store up to two 
			gigabytes (2^31-1 bytes) in a LONG column. Columns of this type are used only for storage and 
			retrieval of long strings. They cannot be used in functions, expressions, or WHERE clauses.
			LONG column values are generally converted to and from character strings.

<return: Integer>     
"

%{  
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_VARCHAR2)) {
	RETURN (__mkSmallInteger(SQLT_CHR));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_INTEGER)) {
	RETURN (__mkSmallInteger(SQLT_INT));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_NUMBER)) {
	RETURN (__mkSmallInteger(SQLT_NUM));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_FLOAT)) {
	RETURN (__mkSmallInteger(SQLT_FLT));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_STRING)) {
	RETURN (__mkSmallInteger(SQLT_STR));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_RAW)) {
	RETURN (__mkSmallInteger(SQLT_BIN));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_VARNUM)) {
	RETURN (__mkSmallInteger(SQLT_VNU));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_LONG_RAW)) {
	RETURN (__mkSmallInteger(SQLT_LBI));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_LONG)) {
	RETURN (__mkSmallInteger(SQLT_LNG));
    }
    if (anOCICDataTypeSymbol == @symbol(C_TYPE_DATE)) {
	RETURN (__mkSmallInteger(SQLT_DAT));
    }
%}.
    OCIInternalError raiseErrorString:' - bad cdata mode arg: ', anOCICDataTypeSymbol printString.


    "
     self numericValueForCDataType:#'C_TYPE_DATE' 
     self numericValueForCDataType:#'C_TYPE_BAD' 
    "
!

numericValueForCredentialType:anOCICredentialTypeSymbol
"
converts a symbol anOCICredentialTypeSymbol to its corresponding integer value.
Specifies the type of credentials to use for establishing the user session. 

Valid values for anOCICredentialTypeSymbol are: 

	OCI_CRED_RDBMS - authenticate using a database username and password pair as credentials. 
			 The attributes OCI_ATTR_USERNAME and OCI_ATTR_ PASSWORD should be set on 
			 the user session context before this call.

	OCI_CRED_EXT   - authenticate using external credentials. No username or password is provided.

<return: Integer>     
"
%{  
    if (anOCICredentialTypeSymbol == @symbol(OCI_CRED_RDBMS)) {
	RETURN (__MKUINT(OCI_CRED_RDBMS));
    }
    if (anOCICredentialTypeSymbol == @symbol(OCI_CRED_EXT)) {
	RETURN (__MKUINT(OCI_CRED_EXT));
    }
%}.
    OCIInternalError raiseErrorString:' - bad credential mode arg: ', anOCICredentialTypeSymbol printString.


    "
     self numericValueForCredentialType:#'OCI_CRED_RDBMS' 
     self numericValueForCredentialType:#'OCI_CRED_EXT' 
    "
!

numericValueForDefineByPosType:anOCIDefineByPosTypeSymbol
"
converts a symbol anOCIDefineByPosTypeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT       - This is default mode.

	OCI_DYNAMIC_FETCH - For applications requiring dynamically allocated data at the time of fetch,
			    this mode must be used. The user may additionally call OCIDefineDynamic()
			    to set up a callback function that will be invoked to receive the dynamically
			    allocated buffers and. The valuep and value_sz parameters are ignored in this mode.

<return: Integer>     
"
%{  
    if (anOCIDefineByPosTypeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCIDefineByPosTypeSymbol == @symbol(OCI_DYNAMIC_FETCH)) {
	RETURN (__MKUINT(OCI_DYNAMIC_FETCH));
    }
%}.
    OCIInternalError raiseErrorString:' - bad define mode arg: ', anOCIDefineByPosTypeSymbol printString.


    "
     self numericValueForDefineByPosType:#'OCI_DEFAULT' 
     self numericValueForDefineByPosType:#'OCI_DYNAMIC_FETCH' 
    "
!

numericValueForEnvironmentMode:anOCIModeSymbol
"
converts a symbol anOCIModeSymbol to its corresponding integer value.
Specifies initialization of the mode. Valid modes are: 
    #'OCI_DEFAULT'      - uses default mode.
    #'OCI_THREADED'     - uses threaded environment. Internal data structures not exposed to the user 
			  are protected from concurrent accesses by multiple threads. 
    #'OCI_OBJECT'       - uses object features.
    #'OCI_SHARED'       - utilizes shared data structures. 
    #'OCI_EVENTS'       - utilizes publish-subscribe notifications.
    #'OCI_NO_UCB'       - suppresses the calling of the dynamic callback routine. 
			  The default behavior is to allow calling of callback routines at the time that the 
			  environment is created.
    #'OCI_ENV_NO_MUTEX' - no mutexing in this mode. All OCI calls done on the environment handle, or on 
			  handles derived from the environment handle, must be serialized.

<return: Integer>
"
%{  
    if (anOCIModeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCIModeSymbol == @symbol(OCI_THREADED)) {
	RETURN (__MKUINT(OCI_THREADED));
    }
    if (anOCIModeSymbol == @symbol(OCI_OBJECT)) {
	RETURN (__MKUINT(OCI_OBJECT));
    }
    if (anOCIModeSymbol == @symbol(OCI_SHARED)) {
	RETURN (__MKUINT(OCI_SHARED));
    }
    if (anOCIModeSymbol == @symbol(OCI_EVENTS)) {
	RETURN (__MKUINT(OCI_EVENTS));
    }
    if (anOCIModeSymbol == @symbol(OCI_NO_UCB)) {
	RETURN (__MKUINT(OCI_NO_UCB));
    }
    if (anOCIModeSymbol == @symbol(OCI_ENV_NO_MUTEX)) {
	RETURN (__MKUINT(OCI_ENV_NO_MUTEX));
    }
%}.
    OCIInternalError raiseErrorString:' - bad environment mode arg: ', anOCIModeSymbol printString.


    "
     self numericValueForMode:#'OCI_EVENTS' 
     self numericValueForMode:#'OCI_OBJECT' 
     self numericValueForMode:#'OCI_FOO' 
    "
!

numericValueForExecuteMode:anOCIExecuteModeSymbol
"
converts a symbol anOCIExecuteModeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT           - Calling OCIStmtExecute() in this mode executes the statement.
				It also implicitly returns describe information about the select-list.

	OCI_DESCRIBE_ONLY     - This mode is for users who wish to describe a query prior to execution.
				Calling OCIStmtExecute() in this mode does not execute the statement,
				but it does return the select-list description. To maximize performance, 
				it is recommended that applications execute the statement in default mode 
				and use the implicit describe which accompanies the execution.

	OCI_COMMIT_ON_SUCCESS - When a statement is executed in this mode, the current transaction
				is committed after execution, provided that execution completes successfully.

	OCI_EXACT_FETCH       - Used when the application knows in advance exactly how many rows it will be fetching.
				This mode turns prefetching off for Oracle release 8 mode, and requires that defines
				be done before the execute call. Using this mode cancels the cursor after the
				desired rows are fetched and may result in reduced server-side resource usage. 
	OCI_BATCH_ERRORS      - See Batch Error Mode for OCIStmtExecute() for information about this mode.

<return: Integer>
"
%{  
    if (anOCIExecuteModeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCIExecuteModeSymbol == @symbol(OCI_DESCRIBE_ONLY)) {
	RETURN (__MKUINT(OCI_DESCRIBE_ONLY));
    }
    if (anOCIExecuteModeSymbol == @symbol(OCI_COMMIT_ON_SUCCESS)) {
	RETURN (__MKUINT(OCI_COMMIT_ON_SUCCESS));
    }
    if (anOCIExecuteModeSymbol == @symbol(OCI_EXACT_FETCH)) {
	RETURN (__MKUINT(OCI_EXACT_FETCH));
    }
    if (anOCIExecuteModeSymbol == @symbol(OCI_BATCH_ERRORS)) {
	RETURN (__MKUINT(OCI_BATCH_ERRORS));
    }
%}.
    OCIInternalError raiseErrorString:' - bad execute mode arg: ', anOCIExecuteModeSymbol printString.


    "
     self numericValueForExecuteMode:#'OCI_DEFAULT' 
     self numericValueForExecuteMode:#'OCI_BATCH_ERRORS' 
    "
!

numericValueForFetchMode:anOCIFetchModeSymbol
"
converts a symbol anOCIFetchModeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT - only mode

<return: Integer>
"
%{  
    if (anOCIFetchModeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
%}.
    OCIInternalError raiseErrorString:' - bad fetch mode arg: ', anOCIFetchModeSymbol printString.

    "
     self numericValueForFetchMode:#'OCI_DEFAULT' 
     self numericValueForFetchMode:#'bad bad bad' 
    "
!

numericValueForInternalDataType:anOCIInternalDataTypeSymbol
"
converts a symbol anOCIInternalDataTypeSymbol to its corresponding integer value.

The possible values are:

    VARCHAR2_TYPE - The VARCHAR2 datatype is a variable-length string of characters
		    with a maximum length of 4000 bytes.

    NUMBER_TYPE   - You should not need to use NUMBER as an external datatype.
		    If you do use it, Oracle returns numeric values in its internal
		    21-byte binary format and will expect this format on input.

    LONG_TYPE     - The LONG datatype stores character strings longer than 4000 bytes. You can store up
		    to two gigabytes (2^31-1 bytes) in a LONG column. Columns of this type are used only
		    for storage and retrieval of long strings.

    ROWID_TYPE    - The ROWID datatype identifies a particular row in a database table.

    DATE_TYPE     - The DATE datatype can update, insert, or retrieve a date value using the Oracle internal date
		    binary format. A date in binary format contains seven bytes:

		    for 30-NOV-1992, 3:17 PM

		    Byte    | 1       | 2    | 3     | 4   | 5    | 6      | 7      |
		    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		    Meaning | Century | Year | Month | Day | Hour | Minute | Second |
		    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		    Example | 119     | 192  | 11    | 30  | 16   | 18     | 1      |

		    The century and year bytes are in an excess-100 notation. Dates Before Common Era (BCE) are less
		    than 100. The era begins on 01-JAN-4712 BCE, which is Julian day 1. For this date, the century
		    byte is 53, and the year byte is 88. The hour, minute, and second bytes are in excess-1 notation. 
		    The hour byte ranges from 1 to 24, the minute and second bytes from 1 to 60. If no time was specified
		    when the date was created, the time defaults to midnight (1, 1, 1). When you enter a date in binary 
		    format using the DATE external datatype, the database does not do consistency or range checking.
		    All data in this format must be carefully validated before input.

    RAW_TYPE      - The RAW datatype is used for binary data or byte strings that are not to be interpreted
		    by Oracle.The maximum length of a RAW column is 2000 bytes.

    LONG_RAW_TYPE - The LONG RAW datatype is similar to the RAW datatype, except that it stores raw data with
		    a length up to two gigabytes (2^31-1 bytes).

    CHAR_TYPE          - The CHAR datatype is a string of characters, with a maximum length of 2000.
			 CHAR strings are compared using blank-padded comparison semantics.

    REF_TYPE           - This is a reference to a named data type.

    CHARACTER_LOB_TYPE - A LOB (Large OBject) stores character data up to 4 gigabytes in length.
			 Character data (fixed- or varying-width) is stored in a CLOB (Character LOB) or
			 NCLOB (National Character LOB). An NCLOB is a national character version of a CLOB.
			 It stores fixed-width, single- or multi-byte national character set character (NCHAR),
			 or varying-width character sets data.

    BINARY_LOB_TYPE    - A LOB (Large OBject) stores binary data up to 4 gigabytes in length.
			 The BLOB (Binary LOB) datatype stores unstructured binary large objects.
			 BLOBs can be thought of as bitstreams with no character set semantics.

    BINARY_FILE_TYPE   - The FILE datatype provides access to file LOBs that are stored in file systems outside
			 an Oracle database.


<return: Integer>     
"
%{  
    if (anOCIInternalDataTypeSymbol == @symbol(VARCHAR2_TYPE)) {
	RETURN (__MKUINT(VARCHAR2_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(NUMBER_TYPE)) {
	RETURN (__MKUINT(NUMBER_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(LONG_TYPE)) {
	RETURN (__MKUINT(LONG_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(ROWID_TYPE)) {
	RETURN (__MKUINT(ROWID_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(DATE_TYPE)) {
	RETURN (__MKUINT(DATE_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(VARRAW_TYPE)) {
	RETURN (__MKUINT(VARRAW_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(RAW_TYPE)) {
	RETURN (__MKUINT(RAW_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(LONG_RAW_TYPE)) {
	RETURN (__MKUINT(LONG_RAW_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(CHAR_TYPE)) {
	RETURN (__MKUINT(CHAR_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(REF_TYPE)) {
	RETURN (__MKUINT(REF_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(CHARACTER_LOB_TYPE)) {
	RETURN (__MKUINT(CHARACTER_LOB_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(BINARY_LOB_TYPE)) {
	RETURN (__MKUINT(BINARY_LOB_TYPE));
    }
    if (anOCIInternalDataTypeSymbol == @symbol(NAMED_OBJECT_TYPE)) {
	RETURN (__MKUINT(SQLT_NTY));
    }
%}.
    OCIInternalError raiseErrorString:' bad internalData mode arg: ', anOCIInternalDataTypeSymbol printString.


    "
     self numericValueForInternalDataType:#'VARCHAR2_TYPE' 
     self numericValueForInternalDataType:#'ROWID_TYPE' 
     self numericValueForInternalDataType:#'LONG_TYPE' 
     self numericValueForInternalDataType:#'DATE_TYPE' 
    "
!

numericValueForLanguageMode:anOCILanguageModeSymbol
"
converts a symbol anOCILanguageModeSymbol to its corresponding integer value.
Specifies V7, or native syntax. Possible values are:

	OCI_V7_SYNTAX - V7 ORACLE parsing syntax

	OCI_NTV_SYNTAX - syntax depends upon the version of the server. 

<return: Integer>
"
%{  
    if (anOCILanguageModeSymbol == @symbol(OCI_V7_SYNTAX)) {
	RETURN (__MKUINT(OCI_V7_SYNTAX));
    }
    if (anOCILanguageModeSymbol == @symbol(OCI_NTV_SYNTAX)) {
	RETURN (__MKUINT(OCI_NTV_SYNTAX));
    }
%}.
    OCIInternalError raiseErrorString:' - bad language mode arg: ', anOCILanguageModeSymbol printString .


    "
     self numericValueForLanguageMode:#'OCI_V7_SYNTAX' 
     self numericValueForLanguageMode:#'OCI_NTV_SYNTAX' 
    "
!

numericValueForOrientationMode:anOCIOrientationModeSymbol
"
converts a symbol anOCIOrientationModeSymbol to its corresponding integer value.

The possible values are:

	 OCI_FETCH_NEXT - For release 8.0, the only acceptable value is OCI_FETCH_NEXT,
			  which is also the default value.

<return: Integer>
"
%{  
    if (anOCIOrientationModeSymbol == @symbol(OCI_FETCH_NEXT)) {
	RETURN (__MKUINT(OCI_FETCH_NEXT));
    }
%}.
    OCIInternalError raiseErrorString:' - bad orientation mode arg: ', anOCIOrientationModeSymbol printString.


    "
     self numericValueForOrientationMode:#'OCI_FETCH_NEXT' 
    "
!

numericValueForParameterType:anOCIParameterTypeSymbol
"
converts a symbol anOCIParameterTypeSymbol to its corresponding integer value.

The possible values are:

	OCI_DTYPE_PARAM         - for a parameter descriptor
	OCI_HTYPE_COMPLEXOBJECT - for a complex object retrieval handle
	OCI_HTYPE_STMT          - for a statement handle
	OCI_HTYPE_SVCCTX        - for a service context handle
	OCI_HTYPE_SERVER        - for a server context handle
	OCI_HTYPE_SESSION       - for a user session handle
	OCI_HTYPE_TRANS         - for a transaction handle
	OCI_HTYPE_DESCRIBE      - for a describe handle

<return: Integer>
"
%{  
    if (anOCIParameterTypeSymbol == @symbol(OCI_DTYPE_PARAM)) {
	RETURN (__MKUINT(OCI_DTYPE_PARAM));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_COMPLEXOBJECT)) {
	RETURN (__MKUINT(OCI_HTYPE_COMPLEXOBJECT));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_STMT)) {
	RETURN (__MKUINT(OCI_HTYPE_STMT));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_SVCCTX)) {
	RETURN (__MKUINT(OCI_HTYPE_SVCCTX));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_SERVER)) {
	RETURN (__MKUINT(OCI_HTYPE_SERVER));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_SESSION)) {
	RETURN (__MKUINT(OCI_HTYPE_SESSION));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_TRANS)) {
	RETURN (__MKUINT(OCI_HTYPE_TRANS));
    }
    if (anOCIParameterTypeSymbol == @symbol(OCI_HTYPE_DESCRIBE)) {
	RETURN (__MKUINT(OCI_HTYPE_DESCRIBE));
    }
%}.
    OCIInternalError raiseErrorString:' - bad parameter mode arg: ', anOCIParameterTypeSymbol printString.


    "
     self numericValueForParameterType:#'OCI_DTYPE_PARM' 
     self numericValueForParameterType:#'OCI_HTYPE_STMT' 
    "
!

numericValueForPrepareMode:anOCIPrepareModeSymbol
"
converts a symbol anOCIPrepareModeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT    - default mode
	OCI_NO_SHARING - disables sharing mode for the SQL statement.

<return: Integer>
"
%{  
    if (anOCIPrepareModeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCIPrepareModeSymbol == @symbol(OCI_NO_SHARING)) {
	RETURN (__MKUINT(OCI_NO_SHARING));
    }
%}.
    OCIInternalError raiseErrorString:' - bad prepare mode arg: ', anOCIPrepareModeSymbol printString.


    "
     self numericValueForPrepareMode:#'OCI_DEFAULT' 
     self numericValueForPrepareMode:#'OCI_NO_SHARING' 
    "
!

numericValueForSchemaTypes:anOCISchemaTypeSymbol
"
The type of schema object to be described. 
Valid values are:

    OCI_PTYPE_TABLE     - for tables
    OCI_PTYPE_VIEW      - for views 
    OCI_PTYPE_PROC      - for procedures
    OCI_PTYPE_FUNC      - for functions 
    OCI_PTYPE_PKG       - for packages 
    OCI_PTYPE_TYPE      - for types 
    OCI_PTYPE_SYN       - for synonyms 
    OCI_PTYPE_SEQ       - for sequences
    OCI_PTYPE_SCHEMA    - for schemas 
    OCI_PTYPE_DATABASE  - for databases 
    OCI_PTYPE_UNK       - for unknown schema objects

<return: Integer>
"

%{                                      
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_TABLE)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_TABLE));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_VIEW)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_VIEW));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_PROC)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_PROC));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_FUNC)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_FUNC));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_PKG)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_PKG));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_TYPE)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_TYPE));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_SYN)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_SYN));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_SEQ)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_SEQ));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_SCHEMA)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_SCHEMA));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_DATABASE)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_DATABASE));
    }
    if (anOCISchemaTypeSymbol == @symbol(OCI_PTYPE_UNK)) {
	RETURN (__MKSMALLINT(OCI_PTYPE_UNK));
    }
%}.
    OCIInternalError raiseErrorString:' - bad schema mode arg: ', anOCISchemaTypeSymbol printString.
!

numericValueForSessionMode:anOCISessionModeSymbol
"
Specifies the various modes of operation. Valid modes are: 

    OCI_DEFAULT     - in this mode, the user session context returned may only ever 
		      be set with the same server context specified in svchp. 

    OCI_MIGRATE     - in this mode, the new user session context may be set in a service
		      handle with a different server handle. This mode establishes the user 
		      session context. To create a migratable session, the service handle must
		      already be set with a non-migratable user session. A migratable session
		      must have a non-migratable parent session.

    OCI_SYSDBA      - in this mode, the user is authenticated for SYSDBA access.

    OCI_SYSOPER     - in this mode, the user is authenticated for SYSOPER access. 

    OCI_PRELIM_AUTH - this mode may only be used with OCI_SYSDBA or OCI_SYSOPER to 
			 authenticate for certain administration tasks.

<return: Integer>
"
%{  
    if (anOCISessionModeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCISessionModeSymbol == @symbol(OCI_MIGRATE)) {
	RETURN (__MKUINT(OCI_MIGRATE));
    }
    if (anOCISessionModeSymbol == @symbol(OCI_SYSDBA)) {
	RETURN (__MKUINT(OCI_SYSDBA));
    }
    if (anOCISessionModeSymbol == @symbol(OCI_SYSOPER)) {
	RETURN (__MKUINT(OCI_SYSOPER));
    }
    if (anOCISessionModeSymbol == @symbol(OCI_PRELIM_AUTH)) {
	RETURN (__MKUINT(OCI_PRELIM_AUTH));
    }
%}.
    OCIInternalError raiseErrorString:' - bad session mode arg: ', anOCISessionModeSymbol printString.


    "
     self numericValueForSessionMode:#'OCI_DEFAULT' 
     self numericValueForSessionMode:#'OCI_MIGRATE' 
     self numericValueForSessionMode:#'OCI_PRELIM_AUTH' 
    "
!

numericValueForStatement:anOCIStatementSymbol
"
converts a symbol anOCIStatementSymbol to its corresponding integer value.

The possible values are:

	OCI_STMT_SELECT         SELECT statement
	OCI_STMT_UPDATE         UPDATE statement
	OCI_STMT_DELETE         DELETE statement
	OCI_STMT_INSERT         INSERT statement
	OCI_STMT_CREATE         CREATE statement
	OCI_STMT_DROP           DROP statement
	OCI_STMT_ALTER          ALTER statement
	OCI_STMT_BEGIN          BEGIN PL/SQL statement
	OCI_STMT_DECLARE        DECLARE PL/SQL statement

<return: Integer>
"
%{  
    if (anOCIStatementSymbol == @symbol(OCI_STMT_SELECT)) {
	RETURN (__MKUINT(OCI_STMT_SELECT));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_UPDATE)) {
	RETURN (__MKUINT(OCI_STMT_UPDATE));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_DELETE)) {
	RETURN (__MKUINT(OCI_STMT_DELETE));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_INSERT)) {
	RETURN (__MKUINT(OCI_STMT_INSERT));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_CREATE)) {
	RETURN (__MKUINT(OCI_STMT_CREATE));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_DROP)) {
	RETURN (__MKUINT(OCI_STMT_DROP));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_ALTER)) {
	RETURN (__MKUINT(OCI_STMT_ALTER));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_BEGIN)) {
	RETURN (__MKUINT(OCI_STMT_BEGIN));
    }
    if (anOCIStatementSymbol == @symbol(OCI_STMT_DECLARE)) {
	RETURN (__MKUINT(OCI_STMT_DECLARE));
    }
%}.
    OCIInternalError raiseErrorString:' - bad statement type: ', anOCIStatementSymbol printString.


    "
     self numericValueForStatement:#'OCI_STMT_UPDATE' 
     self numericValueForStatement:#'OCI_STMT_SELECT' 
     self numericValueForStatement:#'OCI_STMT_INSERT' 
    "
!

numericValueForTransactionCommitType:anOCITransactionCommitTypeSymbol
"
converts a symbol anOCITransactionCommitTypeSymbol to its corresponding integer value.

The possible values are:

	OCI_DEFAULT           - for non-distributed transactions, its a one phase commit
	OCI_TRANS_TWOPHASE    - for applications managing global transactions 

<return: Integer>     
"
%{  
    if (anOCITransactionCommitTypeSymbol == @symbol(OCI_DEFAULT)) {
	RETURN (__MKUINT(OCI_DEFAULT));
    }
    if (anOCITransactionCommitTypeSymbol == @symbol(OCI_TRANS_TWOPHASE)) {
	RETURN (__MKUINT(OCI_TRANS_TWOPHASE));
    }
%}.
    OCIInternalError raiseErrorString:' - bad transaction commit type: ', anOCITransactionCommitTypeSymbol printString .


    "
     self numericValueForTransactionCommitType:#'OCI_DEFAULT' 
     self numericValueForTransactionCommitType:#'OCI_TRANS_TWOPHASE' 
    "
!

numericValueForTransactionType:anOCITransactionTypeSymbol
"
converts a symbol anOCITransactionTypeSymbol to its corresponding integer value.

The possible values are:

	OCI_TRANS_NEW           - starts a new transaction branch. By default starts 
				  a tightly coupled and migratable branch. 
	OCI_TRANS_TIGHT         - explicitly specifies a tightly coupled branch 
	OCI_TRANS_LOOSE         - specifies a loosely coupled branch 
	OCI_TRANS_RESUME        - resumes an existing transaction branch. 
	OCI_TRANS_READONLY      - start a read-only transaction
	OCI_TRANS_SERIALIZABLE  - start a serializable transaction

<return: Integer>     
"
%{  
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_NEW)) {
	RETURN (__MKUINT(OCI_TRANS_NEW));
    }
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_TIGHT)) {
	RETURN (__MKUINT(OCI_TRANS_TIGHT));
    }
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_LOOSE)) {
	RETURN (__MKUINT(OCI_TRANS_LOOSE));
    }
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_RESUME)) {
	RETURN (__MKUINT(OCI_TRANS_RESUME));
    }
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_READONLY)) {
	RETURN (__MKUINT(OCI_TRANS_READONLY));
    }
    if (anOCITransactionTypeSymbol == @symbol(OCI_TRANS_SERIALIZABLE)) {
	RETURN (__MKUINT(OCI_TRANS_SERIALIZABLE));
    }
%}.
    OCIInternalError raiseErrorString:' - bad transaction type: ', anOCITransactionTypeSymbol printString.


    "
     self numericValueForTransactionType:#'OCI_TRANS_RESUME' 
     self numericValueForTransactionType:#'OCI_TRANS_SERIALIZABLE'
     self numericValueForTransactionType:#'OCI_TRANS_READONLY' 
    "
!

numericValueForTransactionTypeColl:anOCITransactionTypeSymbolColl
"
converts the symbols in anOCITransactionTypeSymbolColl into an integer value. The values
of the symbols are concaternated with bitOr.

<return: Integer>     
"

    |returnValue|

    returnValue := 0.
    anOCITransactionTypeSymbolColl do:[:eachSymbol |
	returnValue := returnValue bitOr:(self numericValueForTransactionType:eachSymbol)
    ].
    ^returnValue


    "
     self numericValueForTransactionTypeColl:#(#'OCI_TRANS_SERIALIZABLE' #'OCI_TRANS_READONLY')
     self numericValueForTransactionType:#'OCI_TRANS_SERIALIZABLE' 
     self numericValueForTransactionType:#'OCI_TRANS_READONLY'      
    "
!

numericValueForType:anOCITypeSymbol
"
converts a symbol anOCITypeSymbol to its corresponding integer value.

	OCI_HTYPE_ENV                   - an environment handle 
	OCI_HTYPE_ERROR                 - an error report handle 
	OCI_HTYPE_SVCCTX                - a service context handle 
	OCI_HTYPE_STMT                  - a statement (application request) handle
	OCI_HTYPE_BIND                  - a bind handle
	OCI_HTYPE_DEFINE                - a define handle
	OCI_HTYPE_DESCRIBE              - a select list description handle 
	OCI_HTYPE_SERVER                - a server handle 
	OCI_HTYPE_SESSION               - a user session handle 
	OCI_HTYPE_TRANS                 - a transaction handle 
	OCI_HTYPE_COMPLEXOBJECT         - a complex object retrieval handle 
	OCI_HTYPE_SECURITY              - a security handle 
	OCI_HTYPE_SUBSCRIPTION          - a subscription handle 
	OCI_HTYPE_DIRPATH_CTX           - direct path context handle 
	OCI_HTYPE_DIRPATH_COLUMN_ARRAY  - direct path column array handle 
	OCI_HTYPE_DIRPATH_STREAM        - direct path stream handle 
	OCI_HTYPE_PROC                  - process handle

<return: Integer>
"
%{  
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_ENV)) {
	RETURN (__MKUINT(OCI_HTYPE_ENV ));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_ERROR)) {
	RETURN (__MKUINT(OCI_HTYPE_ERROR));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_SVCCTX)) {
	RETURN (__MKUINT(OCI_HTYPE_SVCCTX));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_STMT)) {
	RETURN (__MKUINT(OCI_HTYPE_STMT));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_BIND)) {
	RETURN (__MKUINT(OCI_HTYPE_BIND));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_DEFINE)) {
	RETURN (__MKUINT(OCI_HTYPE_DEFINE));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_DESCRIBE)) {
	RETURN (__MKUINT(OCI_HTYPE_DESCRIBE));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_SERVER)) {
	RETURN (__MKUINT(OCI_HTYPE_SERVER));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_SESSION)) {
	RETURN (__MKUINT(OCI_HTYPE_SESSION));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_TRANS)) {
	RETURN (__MKUINT(OCI_HTYPE_TRANS));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_COMPLEXOBJECT)) {
	RETURN (__MKUINT(OCI_HTYPE_COMPLEXOBJECT));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_SECURITY)) {
	RETURN (__MKUINT(OCI_HTYPE_SECURITY));
    }                              
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_SUBSCRIPTION)) {
	RETURN (__MKUINT(OCI_HTYPE_SUBSCRIPTION));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_DIRPATH_CTX)) {
	RETURN (__MKUINT(OCI_HTYPE_DIRPATH_CTX));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_DIRPATH_COLUMN_ARRAY)) {
	RETURN (__MKUINT(OCI_HTYPE_DIRPATH_COLUMN_ARRAY));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_DIRPATH_STREAM)) {
	RETURN (__MKUINT(OCI_HTYPE_DIRPATH_STREAM));
    }
    if (anOCITypeSymbol == @symbol(OCI_HTYPE_PROC)) {
	RETURN (__MKUINT(OCI_HTYPE_PROC));
    }
%}.
    OCIInternalError raiseErrorString:' - bad handle type: ', anOCITypeSymbol printString.


    "
     self numericValueForType:#'OCI_HTYPE_ENV' 
     self numericValueForType:#'OCI_HTYPE_ERROR' 
     self numericValueForType:#'OCI_HTYPE_SESSION' 
     self numericValueForType:#'OCI_HTYPE_SERVER' 
     self numericValueForType:#'OCI_HTYPE_SVCCTX' 
     self numericValueForType:#'OCI_FOO' 
    "
!

sb2Size

%{ /* NOCONTEXT */ 
    RETURN(__mkSmallInteger(sizeof(sb2)))
%}

"
self sb2Size
"
!

symbolForNumericStatementType:anOCINumericStatementType
"
converts an integer value anOCINumericStatementType to its corresponding symbol.

The possible values are:

	OCI_STMT_SELECT         SELECT statement
	OCI_STMT_UPDATE         UPDATE statement
	OCI_STMT_DELETE         DELETE statement
	OCI_STMT_INSERT         INSERT statement
	OCI_STMT_CREATE         CREATE statement
	OCI_STMT_DROP           DROP statement
	OCI_STMT_ALTER          ALTER statement
	OCI_STMT_BEGIN          BEGIN PL/SQL statement
	OCI_STMT_DECLARE        DECLARE PL/SQL statement

<return: Symbol>     
"
"
self symbolForNumericStatementType:(self numericValueForStatement:#'OCI_STMT_SELECT')
"

%{ 
    ub2 __attributetype = (ub2) __unsignedLongIntVal(anOCINumericStatementType);

    switch (__attributetype)
    {
	case OCI_STMT_SELECT:
	    RETURN (@symbol(OCI_STMT_SELECT));
	    break;
	case OCI_STMT_UPDATE:
	    RETURN (@symbol(OCI_STMT_UPDATE));
	    break;
	case OCI_STMT_DELETE:
	    RETURN (@symbol(OCI_STMT_DELETE));
	    break;
	case OCI_STMT_INSERT:
	    RETURN (@symbol(OCI_STMT_INSERT));
	    break;
	case OCI_STMT_CREATE:
	    RETURN (@symbol(OCI_STMT_CREATE));
	    break;
	case OCI_STMT_DROP:
	    RETURN (@symbol(OCI_STMT_DROP));
	    break;
	case OCI_STMT_ALTER:
	    RETURN (@symbol(OCI_STMT_ALTER));
	    break;
	case OCI_STMT_BEGIN:
	    RETURN (@symbol(OCI_STMT_BEGIN));
	    break;
	case OCI_STMT_DECLARE:
	    RETURN (@symbol(OCI_STMT_DECLARE));
	    break;
	default:
	    RETURN (@symbol(OCI_STMT_OTHER));
	    break;
    }
%}.
!

symbolForOCIDataType:anOCIDataType
"
converts an integer anOCIDataType which represents an internal data type of
an oracle db to its corresponding symbol.

The possible values are:

    VARCHAR2_TYPE - The VARCHAR2 datatype is a variable-length string of characters
		    with a maximum length of 4000 bytes.

    NUMBER_TYPE   - You should not need to use NUMBER as an external datatype.
		    If you do use it, Oracle returns numeric values in its internal
		    21-byte binary format and will expect this format on input.

    LONG_TYPE     - The LONG datatype stores character strings longer than 4000 bytes. You can store up
		    to two gigabytes (2^31-1 bytes) in a LONG column. Columns of this type are used only
		    for storage and retrieval of long strings.

    ROWID_TYPE    - The ROWID datatype identifies a particular row in a database table.

    DATE_TYPE     - The DATE datatype can update, insert, or retrieve a date value using the Oracle internal date
		    binary format. A date in binary format contains seven bytes:

		    for 30-NOV-1992, 3:17 PM

		    Byte    | 1       | 2    | 3     | 4   | 5    | 6      | 7      |
		    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		    Meaning | Century | Year | Month | Day | Hour | Minute | Second |
		    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		    Example | 119     | 192  | 11    | 30  | 16   | 18     | 1      |

		    The century and year bytes are in an excess-100 notation. Dates Before Common Era (BCE) are less
		    than 100. The era begins on 01-JAN-4712 BCE, which is Julian day 1. For this date, the century
		    byte is 53, and the year byte is 88. The hour, minute, and second bytes are in excess-1 notation. 
		    The hour byte ranges from 1 to 24, the minute and second bytes from 1 to 60. If no time was specified
		    when the date was created, the time defaults to midnight (1, 1, 1). When you enter a date in binary 
		    format using the DATE external datatype, the database does not do consistency or range checking.
		    All data in this format must be carefully validated before input.

    RAW_TYPE      - The RAW datatype is used for binary data or byte strings that are not to be interpreted
		    by Oracle.The maximum length of a RAW column is 2000 bytes.

    LONG_RAW_TYPE - The LONG RAW datatype is similar to the RAW datatype, except that it stores raw data with
		    a length up to two gigabytes (2^31-1 bytes).

    CHAR_TYPE           - The CHAR datatype is a string of characters, with a maximum length of 2000.
			  CHAR strings are compared using blank-padded comparison semantics.

    REF_TYPE            - This is a reference to a named data type.

    CHARACTER_LOB_TYPE - A LOB (Large OBject) stores character data up to 4 gigabytes in length.
			 Character data (fixed- or varying-width) is stored in a CLOB (Character LOB) or
			 NCLOB (National Character LOB). An NCLOB is a national character version of a CLOB.
			 It stores fixed-width, single- or multi-byte national character set character (NCHAR),
			 or varying-width character sets data.

    BINARY_LOB_TYPE    - A LOB (Large OBject) stores binary data up to 4 gigabytes in length.
			 The BLOB (Binary LOB) datatype stores unstructured binary large objects.
			 BLOBs can be thought of as bitstreams with no character set semantics.

<return: Symbol>     
"
"
self symbolForOCIDataType:(self numericValueForInternalDataType:#'VARCHAR2_TYPE')
"

%{ 
    ub4 __attributetype = (ub4) __unsignedLongIntVal(anOCIDataType);

    switch (__attributetype)
    {
	case VARCHAR2_TYPE:
	    RETURN (@symbol(VARCHAR2_TYPE));
	    break;
	case NUMBER_TYPE:
	    RETURN (@symbol(NUMBER_TYPE));
	    break;
	case LONG_TYPE:
	    RETURN (@symbol(LONG_TYPE));
	    break;
	case ROWID_DESC_TYPE:
	    RETURN (@symbol(ROWID_DESC_TYPE));
	    break;
	case ROWID_TYPE:
	    RETURN (@symbol(ROWID_TYPE));
	    break;
	case DATE_TYPE:
	    RETURN (@symbol(DATE_TYPE));
	    break;
	case RAW_TYPE:
	    RETURN (@symbol(RAW_TYPE));
	    break;
	case LONG_RAW_TYPE:
	    RETURN (@symbol(LONG_RAW_TYPE));
	    break;
	case CHAR_TYPE:
	    RETURN (@symbol(CHAR_TYPE));
	    break;
	case REF_TYPE:
	    RETURN (@symbol(REF_TYPE));
	    break;
	case CHARACTER_LOB_TYPE:
	    RETURN (@symbol(CHARACTER_LOB_TYPE));
	    break;
	case BINARY_LOB_TYPE:
	    RETURN (@symbol(BINARY_LOB_TYPE));
	    break;
	case SQLT_NTY:
	    RETURN (@symbol(NAMED_OBJECT_TYPE));
	    break;
    }
%}.
    OCIInternalError raiseErrorString:' - bad oci data type: ', anOCIDataType printString.
!

ub2Size

%{ /* NOCONTEXT */ 
    RETURN(__mkSmallInteger(sizeof(ub2)))
%}

"
self ub2Size
"
! !

!OCI8Lib class methodsFor:'number conversion'!

asOracle8Varnum:aNumber
"
converts aNumber to the internal oracle 8 number format. Works at the moment only for
integers correct. For other number formats choose integer, float or string type for
the bind variable and not varnum type.

<return: ByteArray>                                     
"
    |negative mantissaBytes exponent index mantissaSize byteArray string stringSize 
     startIndex dstIndex mantissaBytesSize|

    aNumber isInteger ifFalse:[
	OCIErrorSignal raiseErrorString:' - conversion works only with integers'
    ].

    aNumber = 0 ifTrue:[
	"value 0"
	^ #[1 128]
    ].

    "negative number?"
    aNumber < 0 ifTrue:[
	negative := true.
	string := aNumber abs printString.
    ] ifFalse:[
	negative := false.
	string := aNumber printString.
    ].

    "mantissa bytes"
    stringSize := string size.
    mantissaBytes := ByteArray new:20.
    startIndex := 1.
    stringSize odd ifTrue:[
	startIndex := 2.
	mantissaBytes at:1 put:(string at:1) digitValue.
    ].                   
    dstIndex := startIndex.
    startIndex to: stringSize by: 2 do:[:i|  |b|
	b := (string at:i) digitValue.
	b := (b * 10) + (string at:i+1) digitValue.
	mantissaBytes at:dstIndex put:b.
	dstIndex := dstIndex + 1.
    ].
    mantissaBytesSize := dstIndex - 1.

    "exponent"
    exponent := mantissaBytesSize - 1.

    "bytes without trailing 0"
    index := mantissaBytesSize.
    [(mantissaBytes at:index) == 0] whileTrue:[
	index := index - 1.
    ].
    mantissaSize := index.
    "convert manitissa bytes"
    negative ifTrue:[
	"negative number"
	[index > 0] whileTrue:[
	    mantissaBytes at:index put:101-(mantissaBytes at:index).
	    index := index - 1.
	].
	(mantissaSize ~= 20) ifTrue:[
	    byteArray := ByteArray new:(3 + mantissaSize).
	    "byte one: number of bytes for exponent, mantissa and terminator byte"
	    byteArray at:1 put:(2 + mantissaSize).
	    "terminator byte for negative numbers when mantissa size ~= 20"
	    byteArray at:mantissaSize + 3 put:102.
	] ifFalse:[
	    byteArray := ByteArray new:(2 + mantissaSize).
	    "byte one: number of bytes for exponent and mantissa, no terminator byte"
	    byteArray at:1 put:(1+mantissaSize).
	].
	"byte two: exponent byte"
	byteArray at:2 put:(62 - exponent).
    ] ifFalse:[
	"positive number"
	[index > 0] whileTrue:[
	    mantissaBytes at:index put:(mantissaBytes at:index) + 1.
	    index := index - 1.
	].
	byteArray := ByteArray new:(2 + mantissaSize).
	"byte one: number of bytes for exponent and mantissa"
	byteArray at:1 put:(1 + mantissaSize).
	"byte two: exponent byte"
	byteArray at:2 put:(193 + exponent).
    ].
    "followed by mantissa bytes"
    byteArray replaceFrom:3 to:2+mantissaSize with:mantissaBytes startingAt:1.
    ^byteArray.
!

exponentFor:anExponent
"
return 100^anExponent.

<return: Integer>
"
    "kludge: avoid raisedTo:, for common numbers"
    (anExponent between:-2 and:5) ifTrue:[  
	^#(0.0001 0.01 1 100 10000 1000000 100000000 10000000000) at:anExponent+3
    ] ifFalse:[
	^ 100 raisedToInteger:anExponent.
    ].

!

numberFromOracle8Varnum:aByteArray
"
get a number from the internal oracle 8 number format.
The length of the byte array aByteArray must be 22 bytes.

<return: Number>
"
    |numberDescSize mantissaSize exponentByte mantissaBytes exponent mantissa bit8Set exp|

    "first byte is the lenght byte (exponent + mantissa bytes)"
    numberDescSize := aByteArray byteAt:1.
    (numberDescSize == 0) ifTrue:[OCIErrorSignal raise with:'internal oci error while converting number'].
    "number of the mantissa bytes"
    mantissaSize := numberDescSize - 1.
    "second byte is exponent byte"
    exponentByte := aByteArray byteAt:2.
    "a negativ number is terminated by a terminator byte"
    "(could be that there is no terminator byte)"
    (bit8Set := exponentByte isBitSet:8) ifFalse:[
	"positive numbers have the hightest bit of the exponent set, negative not"
	((aByteArray at:(numberDescSize+1)) = 102) ifTrue:[
	    "if terminator byte, then decrement mantissa lenght"
	    mantissaSize := mantissaSize - 1.
	]
    ].
    "get mantissa bytes (starting at byte 3 in buffer)"
    mantissaBytes := ByteArray new:mantissaSize.
    1 to: mantissaSize do:[:i |
	mantissaBytes at:i put:(aByteArray byteAt:2+i)
    ]. 
    "calc mantissa in case of a possitve or negative number"
    bit8Set ifTrue:[
	"positive number"

	exponent := exponentByte - 193.
	mantissa := 0.
	1 to: mantissaBytes size do:[:i |
	    mantissa := (((mantissaBytes at:i)-1)) + (mantissa * 100)
	].
    ] ifFalse:[
	"negative number"
	exponent := 62 - exponentByte.
	mantissa := 0.
	1 to: mantissaBytes size do:[:i |
	    mantissa := ((101-(mantissaBytes at:i))) + (mantissa * 100)
	].
	mantissa := mantissa negated.
    ].
    "return value is mantissa * 100 ^ exponent"
    exp := exponent - (mantissaBytes size - 1).
    ^mantissa * (self exponentFor:exp)                   



! !

!OCI8Lib class methodsFor:'test'!

test
"
self test
"
    |inst envHandle errorHandle serverHandle serviceHandle userSessionHandle|

    inst := self new.
    envHandle := OCI8Handle new type:#'OCI_HTYPE_ENV'.

    100000 timesRepeat:[
	[
	    inst ociEnvCreate:envHandle mode:#(#'OCI_DEFAULT').
	    errorHandle := inst ociHandleAlloc:envHandle type:#'OCI_HTYPE_ERROR'.
	    serverHandle :=  inst ociHandleAlloc:envHandle type:#'OCI_HTYPE_SERVER'.
	    serviceHandle :=  inst ociHandleAlloc:envHandle type:#'OCI_HTYPE_SVCCTX'.
	    userSessionHandle :=  inst ociHandleAlloc:envHandle type:#'OCI_HTYPE_SESSION'.

	    "connect"
	    "server"
	    inst ociServerAttach:serverHandle dbname:'test' errorHandle:errorHandle.
	    inst ociAttrSet:serviceHandle attribute:serverHandle handle attributeType:#'OCI_ATTR_SERVER' errorOCIHandle:errorHandle.
	    "user session"
	    inst ociAttrSet:userSessionHandle attribute:'scott' attributeType:#'OCI_ATTR_USERNAME' errorOCIHandle:errorHandle.
	    "set password attribute in user session handle"
	    inst ociAttrSet:userSessionHandle attribute:'tiger' attributeType:#'OCI_ATTR_PASSWORD' errorOCIHandle:errorHandle.
	    "start session"
	    inst ociSessionBegin:serviceHandle userSession:userSessionHandle credential:#'OCI_CRED_RDBMS' mode:#'OCI_DEFAULT' errorHandle: errorHandle.
	    "set the user session in the service context"
	    inst ociAttrSet:serviceHandle attribute:userSessionHandle handle attributeType:#'OCI_ATTR_SESSION' errorOCIHandle:errorHandle.
	    "unblock connection"
	    inst ociAttrSet:serverHandle attribute:0 attributeType:#'OCI_ATTR_NONBLOCKING_MODE' errorOCIHandle:errorHandle.
	] ensure:[
	    "disconnect"
	    OCIErrorSignal handle:[:ex|
	    ] do:[
		inst ociSessionEnd:serviceHandle userSession:userSessionHandle errorHandle:errorHandle.
	    ].
	    OCIErrorSignal handle:[:ex|
	    ] do:[
		inst ociServerDetach:serverHandle errorHandle:errorHandle.
	    ].
	    inst ociHandleFree:envHandle.
	].
    ].
! !

!OCI8Lib methodsFor:'accessing'!

executionLock
"
returns a lock any oci method must obtain before calling it's primitve method.
It musnt be possible to execute more primitive methods parallel.

<return: RecursionLock>
"
    ^ ExecutionLock
! !

!OCI8Lib methodsFor:'allocate & release buffers'!

bufferForSize:aBufferSize
"
returns an external buffer of the desired size. The buffer is
prevented from garbage collection, hence it dosent change its address and
must be freed by the caller.

<return: ExternalBytes>
"
    ^self class bufferForSize:aBufferSize
! !

!OCI8Lib methodsFor:'bind & define & describe'!

ociBindByName:anOCI8StatementHandle bindHandle:anOCI8BindHandle cType:aCType buffer:anExternalBytes bufferSize:aBufferSize name:aName nullValueIndicator:aNullValueIndicator elementLengthArray:anElementLenghtArray returnCodeArray:aReturnCodeArray mode:aModeSymbol errorOCIHandle:anOCI8ErrorHandle
"
performs a basic bind operation. The bind creates an association between the address of a program variable
and a placeholder in a SQL statement or PL/SQL block. The bind call also specifies the type of data which
is being bound, and may also indicate the method by which data will be provided at runtime.

This function also implicitly allocates the bind handle indicated by the bindpp parameter. If a non-NULL pointer
is passed in **bindpp, the OCI assumes that this points to a valid handle that has been previously allocated
with a call to OCI8Lib>>ociHandleAlloc:type: or this method.

Data in an OCI application can be bound to placeholders statically or dynamically.
Binding is static when all the IN bind data and the OUT bind buffers are well-defined just before the execute.
Binding is dynamic when the IN bind data and the OUT bind buffers are provided by the application on demand
at execute time to the client library. Dynamic binding is indicated by setting the mode parameter
of this call to OCI_DATA_AT_EXEC.

This method take a parameter a bind handle, which is implicitly allocated by the bind call A separate bind
handle is allocated for each placeholder the application is binding.

Additional bind calls may be required to specify particular attributes necessary when binding certain data types
or handling input data in certain ways: 

    If data is being provided dynamically at runtime, and the application will be using user-defined callback
    functions, OCI8Lib>>ociBindDynamic:callBackData:errorHandle: must be called to register the callbacks.

    If a statement with RETURNING clause is used, a call to OCI8Lib>>ociBindDynamic:callBackData:errorHandle: 
    must follow this call.

Parameters:

    anOCI8StatementHandle (IN/OUT)
	The statement handle to the SQL or PL/SQL statement being processed.

    anOCI8BindHandle (IN/OUT)
	An address of a bind handle which is implicitly allocated by this call. The bind handle maintains
	all the bind information for this particular input value. The handle is freed implicitly when the
	statement handle is deallocated. On input, the value of the pointer must be NULL or a valid bind handle.

    aCType (IN)
	The data type of the value(s) being bound.

    anExternalBytes (IN/OUT) 
	An address of a data value or an array of data values of the type specified in the aCType parameter.
	An array of data values can be specified for mapping into a PL/SQL table or for providing data for
	SQL multiple-row operations. When an array of bind values is provided, this is called an array bind
	in OCI terms.
	If the #'OCI_ATTR_CHARSET_ID' is attribute is set to #'OCI_UCS2ID' (Unicode), all data passed to and
	received with the corresponding bind call is assumed to be in UCS-2 encoding. 

    aBufferSize (IN) 
	The size of a data value. In the case of an array bind, this is the maximum size of any element
	possible with the actual sizes being specified in the alenp parameter.
	For descriptors, locators, or REFs, whose size is unknown to client applications use the size
	of the structure you are passing in; e.g., sizeof (OCILobLocator *).

    aName (IN)
	The placeholder attributes are specified by name aName.

    aNullValueIndicator (IN/OUT)
	Pointer to an indicator variable or array. For all data types except SQLT_NTY, this is a pointer to sb2
	or an array of sb2s. This parameter is ignored for dynamic binds.

    anElementLenghtArray (IN/OUT)
	Pointer to array of actual lengths of array elements. Each element in alenp is the length of the data
	in the corresponding element in the bind value array before and after the execute. This parameter is
	ignored for dynamic binds. 

    aReturnCodeArray (OUT)
	Pointer to array of column level return codes. This parameter is ignored for dynamic binds.

    aModeSymbol (IN)
	The valid modes for this parameter are:

	    #'OCI_DEFAULT'      - This is default mode. 
	    #'OCI_DATA_AT_EXEC' - When this mode is selected, the value_sz parameter defines the maximum size
				  of the data that can be ever provided at runtime. The application must be
				  ready to provide the OCI library runtime IN data buffers at any time and
				  any number of times. Runtime data is provided in one of the two ways:

				  1) callbacks using a user-defined function which must be registered with a
				     subsequent call to OCI8Lib>>ociBindDynamic:callBackData:errorHandle:.

				  2) a polling mechanism using calls supplied by the OCI. This mode is assumed
				     if no callbacks are defined.

				  When the allocated buffers are not required any more, they should be freed
				  by the client.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle| 

    errorHandle := anOCI8ErrorHandle handle.

    ^self executePrimitive:[
	self primOciBindByName:anOCI8StatementHandle handle
		     newHandle:anOCI8BindHandle handle
			  name:aName
		    dataBuffer:anExternalBytes
		      dataSize:aBufferSize
		      dataType:(self class numericValueForCDataType:aCType)
	    nullValueIndicator:aNullValueIndicator
	    elementLengthArray:anElementLenghtArray
	       returnCodeArray:aReturnCodeArray
			  mode:(self class numericValueForBindByPosType:aModeSymbol)
		   errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociBindDynamic:anOCI8BindHandle callBackData:anOCI8CallBackData errorHandle:anOCI8ErrorHandle
"
used to register user-defined callback functions for providing or receiving data if #'OCI_DATA_AT_EXEC'
mode was specified in a previous call to OCI8Lib>>ociBindByName:bindHandle:cType:buffer:bufferSize:name:nullValueIndicator:elementLengthArray:returnCodeArray:mode:errorOCIHandle:.

The callback function pointers must return #'OCI_CONTINUE' if it the call is successful. Any return code
other than #'OCI_CONTINUE' signals that the client wishes to abort processing immediately.

When passing the address of a storage area, make sure that the storage area will exist even after 
the application returns from the callback. This means that you should not allocate such storage on the stack.

Parameters:

   anOCI8BindHandle (IN)
       An address of a bind handle. The bind handle maintains all the bind information for this particular input value.

   anOCI8CallBackData (IN)
       Provides the callback function pointers and handles the allocation and deallocation of storage area
       needed by the callback functions. (See OCI8CallBackData documentation for more information.)

   anOCI8ErrorHandle (IN/OUT)
       An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
       for diagnostic information in the event of an error.

<return: self>
"
   |errorHandle|

   errorHandle := anOCI8ErrorHandle handle.

   ^ self executePrimitive:[
       self primOciBindDynamic:anOCI8BindHandle handle
	  inOCICallBackContext:anOCI8CallBackData callBackContext
	 outOCICallBackContext:anOCI8CallBackData callBackContext
	 inOCICallbackFunction:anOCI8CallBackData inReturningFunction
	outOCICallbackFunction:anOCI8CallBackData outReturningFunction
		   errorHandle:errorHandle.
   ] withErrorHandle:anOCI8ErrorHandle.
!

ociDefineByPos:anOCI8StatementHandle defineHandle:anOCI8DefineHandle cType:aDataTypeSymbol buffer:aDataBuffer bufferSize:aBufferSize position:aPositionInteger nullValueIndicator:aNullValueIndicator elementLengthArray:anElementLenghtArray returnCodeArray:aReturnCodeArray mode:aModeSymbol errorOCIHandle:anOCI8ErrorHandle
"
defines an output buffer which will receive data retrieved from Oracle. The define is a local step which
is necessary when a SELECT statement returns data to your OCI application.

This call also implicitly allocates the define handle for the select-list item. If a non-NULL pointer is
passed in anOCI8DefineHandle, the OCI assumes that this points to a valid handle that has been previously
allocated with a call to OCI8Lib>>ociHandleAlloc:type: or this method. This would be true in the case of
an application which is redefining a handle to a different addresses so it can reuse the same define handle
for multiple fetches.

For NCHAR (fixed and varying length), the buffer pointer must point to an array of bytes sufficient for holding
the required NCHAR characters.

Nested table columns are defined and fetched like any other named data type.

When defining an array of descriptors or locators, you should pass in an array of pointers to descriptors or locators.

When doing an array define for character columns, you should pass in an array of character buffers.
If the mode parameter is this call is set to #'OCI_DYNAMIC_FETCH', the client application can fetch 
data dynamically at runtime. Runtime data can be provided in one of two ways: 

    1) a polling mechanism using calls supplied by the OCI. This mode is assumed if no callbacks are defined.
       In this case, the fetch call returns the #'OCI_NEED_ DATA' error code, and a piecewise polling method is
       used to provide the data.

    2) (NOT SUPPORTED BY THE OCI8LIB)
       callbacks using a user-defined function which must be registered with a subsequent call to OCIDefineDynamic().
       When the client library needs a buffer to return the fetched data, the callback will be invoked and the
       runtime buffers provided will return a piece or the whole data.

Parameters:

    anOCI8StatementHandle (IN/OUT)
	A handle to the requested SQL query operation.

    anOCI8DefineHandle (IN/OUT)
	A pointer to a pointer to a define handle. If this parameter is passed as nil, this call implicitly allocates
	the define handle. In the case of a redefine, a non-nil handle can be passed in this parameter.
	This handle is used to store the define information for this column.
	    Note: The user must keep track of this pointer. If a second call to this method is made for the same
		  column position, there is no guarantee that the same pointer is returned.

    cType(IN)
	The data type. Named data type (#'SQLT_NTY') and REF (#'SQLT_REF') are valid only if the environment has been
	initialized with in object mode.

    aDataBuffer (IN/OUT)
	A pointer to a buffer or an array of buffers of the type specified in the dty parameter. A number of buffers
	can be specified when results for more than one row are desired in a single fetch call.

    aBufferSize (IN)
	The size of each valuep buffer in bytes. If the data is stored internally in VARCHAR2 format, the number of
	characters desired, if different from the buffer size in bytes, may be additionally specified by the using
	OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle:. In an NLS conversion environment, a truncation
	error will be generated if the number of bytes specified is insufficient to handle the number of characters
	desired. If the #'OCI_ATTR_CHARSET_ID' is attribute is set to #'OCI_UCS2ID' (Unicode), all data passed to
	and received with the corresponding define call is assumed to be in UCS-2 encoding.

    aPositionInteger (IN)
	The position of this value in the select list. Positions are 1-based and are numbered from left to right.
	For example, in the SELECT statement: 

		SELECT empno, ssn, mgrno FROM employees; 

	empno is at position 1, ssn is at position 2, and mgrno is at position 3.

    aNullValueIndicator (IN)
	Pointer to an indicator variable or array. For scalar data types, pointer to sb2 or an array of sb2s.

    anElementLenghtArray (IN/OUT)
	Pointer to array of length of data fetched. Each element in rlenp is the length of the data in the
	corresponding element in the row after the fetch. 

    aReturnCodeArray (OUT)
	Pointer to array of column-level return codes 

    aModeSymbol (IN)
	The valid modes are:

	    #'OCI_DEFAULT'       - This is the default mode.

	    #'OCI_DYNAMIC_FETCH' - (NOT SUPPORTED BY THE OCI8LIB)
				   For applications requiring dynamically allocated data at the time of fetch,
				   this mode must be used. The user may additionally call OCIDefineDynamic()
				   to set up a callback function that will be invoked to receive the dynamically
				   allocated buffers and. The valuep and value_sz parameters are ignored in this mode.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle  := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciDefineByPos:anOCI8StatementHandle handle
		      newHandle:anOCI8DefineHandle handle  
		       position:aPositionInteger
		       dataSize:aBufferSize
		       dataType:(self class numericValueForCDataType:aDataTypeSymbol)
	       intoDefineBuffer:aDataBuffer   
	     nullValueIndicator:aNullValueIndicator
	     elementLengthArray:anElementLenghtArray
		returnCodeArray:aReturnCodeArray
			   mode:(self class numericValueForDefineByPosType:aModeSymbol)
		    errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociDescribeAny:anOCI8ServiceHandle schemaName:aSchemaString schemaType:aSchemaTypeSymbol newHandle:anOCI8DescribeHandle errorOCIHandle:anOCI8ErrorHandle
"
describes existing schema objects: tables, views, synonyms, procedures, functions, packages, sequences, types,
schemas, and databases. This call also describes sub-schema objects, such as a column in a table. This call
populates the describe handle with the object-specific attributes which can be obtained through an
OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: call.

An OCI8Lib>>ociParamGet:position:errorOCIHandle: on the describe handle returns a parameter descriptor for
a specified position. Parameter positions begin with 1. Calling OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle:
on the parameter descriptor returns the specific attributes of a stored procedure or function parameter,
or a table column descriptor. These subsequent calls do not need an extra round trip to the server because
the entire schema object description is cached on the client side by this method. 
Calling OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: on the describe handle also returns the total number
of positions. 

If the #'OCI_ATTR_DESC_PUBLIC' attribute is set on the describe handle, then the object named is looked up as
a public synonym when the object does not exist in the current schema and only <name1> is specified.

Parameters:

    anOCI8ServiceHandle (IN)
	A service context handle.

    aSchemaString (IN)
	A string containing the name of the object to be described.  The string should be in the format 
	<name1>[.<name2>.<name3>...][@<linkname>], such as scott.emp.empno@mydb. Database links are only
	allowed to Oracle8i databases. The object name is interpreted by the following SQL rules:

	    1) If only <name1> is entered and objtyp is equal to #'OCI_PTYPE_SCHEMA', then the name refers
	       to the named schema. The Oracle database must be release 8.1 or greater.

	    2) If only <name1> is entered and objtyp is equal to #'OCI_PTYPE_DATABASE', then the name refers
	       to the named database. When describing a remote database with database_name@db_link_name,
	       the remote Oracle database must be release 8.1 or greater. 

	    3) If only <name1> is entered and objtyp is not equal to #'OCI_PTYPE_SCHEMA' or #'OCI_PTYPE_DATABASE',
	       then the name refers to the named object (of type table / view / procedure / function / package /
	       type / synonym / sequence) in the current schema of the current user. When connected to an Oracle7
	       Server, the only valid types are procedure and function. 

	    4) If <name1>.<name2>.<name3>... is entered, the object name refers to a schema/sub-schema object in
	       the schema named <name1>. For example, in the string scott.emp.deptno, scott is the name of the schema,
	       emp is the name of a table in the schema, and deptno is the name of a column in the table.

    aSchemaTypeSymbol (IN/OUT)
	The type of schema object being described. Valid values are:

		1) OCI_PTYPE_TABLE, for tables 
		2) OCI_PTYPE_VIEW, for views 
		3) OCI_PTYPE_PROC, for procedures 
		4) OCI_PTYPE_FUNC, for functions 
		5) OCI_PTYPE_PKG, for packages 
		6) OCI_PTYPE_TYPE, for types 
		7) OCI_PTYPE_SYN, for synonyms 
		8) OCI_PTYPE_SEQ, for sequences
		9) OCI_PTYPE_SCHEMA, for schemas 
		10) OCI_PTYPE_DATABASE, for databases 
		11) OCI_PTYPE_UNK, for unknown schema objects

	 A value for this argument must be specified. If OCI_PTYPE_UNK is specified, then the description of an object
	 with the specified name in the current schema is returned, if such an object exists, along with the actual
	 type of the object.

    anOCI8DescribeHandle (IN/OUT) 
	A describe handle that is populated with describe information about the object after the call. 
	Must be non-NULL.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciDescribeAny:anOCI8ServiceHandle handle
		     schemaName:aSchemaString
		     schemaType:(self class numericValueForSchemaTypes:aSchemaTypeSymbol)
		      newHandle:anOCI8DescribeHandle handle
		    errorHandle:errorHandle. 
    ] withErrorHandle:anOCI8ErrorHandle
!

primOciBindByName:aStatementHandle newHandle:aNewBindHandle name:aName dataBuffer:aDataBuffer dataSize:aDataSize dataType:aDataTypeInteger nullValueIndicator:anIndicatorArray elementLengthArray:anElementLengthArray returnCodeArray:aReturnCodeArray mode:aModeInteger errorHandle:anErrorHandle
"
performs a basic bind operation. The bind creates an association between the address of a program variable
and a placeholder in a SQL statement or PL/SQL block. The bind call also specifies the type of data which
is being bound, and may also indicate the method by which data will be provided at runtime.

This function also implicitly allocates the bind handle indicated by the bindpp parameter. If a non-NULL pointer
is passed in **bindpp, the OCI assumes that this points to a valid handle that has been previously allocated
with a call to OCI8Lib>>ociHandleAlloc:type: or this method.

Data in an OCI application can be bound to placeholders statically or dynamically.
Binding is static when all the IN bind data and the OUT bind buffers are well-defined just before the execute.
Binding is dynamic when the IN bind data and the OUT bind buffers are provided by the application on demand
at execute time to the client library. Dynamic binding is indicated by setting the mode parameter
of this call to OCI_DATA_AT_EXEC.

This method take a parameter a bind handle, which is implicitly allocated by the bind call A separate bind
handle is allocated for each placeholder the application is binding.

Additional bind calls may be required to specify particular attributes necessary when binding certain data types
or handling input data in certain ways: 

    If data is being provided dynamically at runtime, and the application will be using user-defined callback
    functions, OCI8Lib>>ociBindDynamic:callBackData:errorHandle: must be called to register the callbacks.

    If a statement with RETURNING clause is used, a call to OCI8Lib>>ociBindDynamic:callBackData:errorHandle: 
    must follow this call.

Parameters:

    aStatementHandle (IN/OUT)
	The statement handle to the SQL or PL/SQL statement being processed.

    aNewBindHandle (IN/OUT)
	An address of a bind handle which is implicitly allocated by this call. The bind handle maintains
	all the bind information for this particular input value. The handle is freed implicitly when the
	statement handle is deallocated. On input, the value of the pointer must be NULL or a valid bind handle.

    aName (IN)
	The placeholder attributes are specified by name aName.

    aDataBuffer (IN/OUT) 
	An address of a data value or an array of data values of the type specified in the aCType parameter.
	An array of data values can be specified for mapping into a PL/SQL table or for providing data for
	SQL multiple-row operations. When an array of bind values is provided, this is called an array bind
	in OCI terms.
	If the #'OCI_ATTR_CHARSET_ID' is attribute is set to #'OCI_UCS2ID' (Unicode), all data passed to and
	received with the corresponding bind call is assumed to be in UCS-2 encoding. 

    aDataSize (IN) 
	The size of a data value. In the case of an array bind, this is the maximum size of any element
	possible with the actual sizes being specified in the alenp parameter.
	For descriptors, locators, or REFs, whose size is unknown to client applications use the size
	of the structure you are passing in; e.g., sizeof (OCILobLocator *).

    aDataTypeInteger (IN)
	The data type of the value(s) being bound.

    anIndicatorArray (IN/OUT)
	Pointer to an indicator variable or array. For all data types except SQLT_NTY, this is a pointer to sb2
	or an array of sb2s. This parameter is ignored for dynamic binds.

    anElementLengthArray (IN/OUT)
	Pointer to array of actual lengths of array elements. Each element in alenp is the length of the data
	in the corresponding element in the bind value array before and after the execute. This parameter is
	ignored for dynamic binds. 

    aReturnCodeArray (OUT)
	Pointer to array of column level return codes. This parameter is ignored for dynamic binds.

    aModeInteger (IN)
	The valid modes for this parameter are:

	    #'OCI_DEFAULT'      - This is default mode. 
	    #'OCI_DATA_AT_EXEC' - When this mode is selected, the value_sz parameter defines the maximum size
				  of the data that can be ever provided at runtime. The application must be
				  ready to provide the OCI library runtime IN data buffers at any time and
				  any number of times. Runtime data is provided in one of the two ways:

				  1) callbacks using a user-defined function which must be registered with a
				     subsequent call to OCI8Lib>>ociBindDynamic:callBackData:errorHandle:.

				  2) a polling mechanism using calls supplied by the OCI. This mode is assumed
				     if no callbacks are defined.

				  When the allocated buffers are not required any more, they should be freed
				  by the client.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmtp   = 0;
    OCIError * __errorp = 0;
    OCIBind * __bindp;
    dvoid * __valuep  = 0;
    dvoid * __indicatorp = 0;
    ub2 * __elementlengthp = 0;
    ub2 * __returncodep = 0;
    text *__name;
    sb4 __nameLen;
    sb4 __value_sz = 0;
    ub2 __datatype;
    ub4 __mode;
    sword __ret;
    double __double_value;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aStatementHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isExternalAddress(aNewBindHandle) ||
       !__isSmallInteger(aDataTypeInteger) || !__isInteger(aModeInteger) || (!__isString(aName) && !__isSymbol(aName))) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIBindByName called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __stmtp    = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorp   = (OCIError *) __externalAddressVal(anErrorHandle);
    __datatype = (ub2) __intVal(aDataTypeInteger);
    __mode = (ub4) __longIntVal(aModeInteger);
    __name = (text *) __stringVal(aName);
    __nameLen = __stringSize(aName);
    __bindp = (OCIBind *) __externalAddressVal(aNewBindHandle);

    if (aDataBuffer != nil) {
	if(!__isExternalBytes(aDataBuffer)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: bind buffer in OCIBindByName must be an external bytes object\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__valuep = (dvoid *)__externalBytesAddress(aDataBuffer);
    }
    if (aDataSize != nil) {
	if(!__isInteger(aDataSize)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: data size in OCIBindByName must be an integer\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__value_sz = __longIntVal(aDataSize);
    }
    if (anIndicatorArray != nil) {
	if (! __isExternalBytes(anIndicatorArray)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: indicator array in OCIBindByName must be an external bytes object\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__indicatorp = (dvoid *) __externalBytesAddress(anIndicatorArray);
    }
    if (aReturnCodeArray != nil) {
	if (! __isExternalBytes(aReturnCodeArray)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: return-code array in OCIBindByName must be an external bytes object\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__returncodep = (ub2 *) __externalBytesAddress(aReturnCodeArray);
    }
    if (anElementLengthArray != nil) {
	if (! __isExternalBytes(anElementLengthArray)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: element length array in OCIBindByName must be an external bytes object\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__elementlengthp = (ub2 *) __externalBytesAddress(anElementLengthArray);
    }

    __ret = OCIBindByName(      __stmtp,
				&__bindp,
				__errorp,
				__name,
				__nameLen,
				__valuep,
				__value_sz,
				__datatype,
				__indicatorp,
				__elementlengthp,
				__returncodep,
				(ub4) 0,   /* only for PL/SQL binds */
				(ub4 *) 0, /* only for PL/SQL binds */
				__mode);

    if (__ret == OCI_SUCCESS) {
	__externalAddressVal(aNewBindHandle) = __bindp;
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIBindByName(%x, %s, %d, %d) -> %x\n", __stmtp, __name, __datatype, __value_sz, __bindp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIBindByName(%x, %s, %d, %d) failed\n", __stmtp, __name, __value_sz, __datatype);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciBindDynamic:aBindHandle inOCICallBackContext:anInOCICallBackContext outOCICallBackContext:anOutOCICallBackContext inOCICallbackFunction:anInOCICallbackFunction outOCICallbackFunction:anOutOCICallbackFunction errorHandle:anErrorHandle
"                                                    
used to register user-defined callback functions for providing or receiving data if #'OCI_DATA_AT_EXEC'
mode was specified in a previous call to OCI8Lib>>ociBindByName:bindHandle:cType:buffer:bufferSize:name:nullValueIndicator:elementLengthArray:returnCodeArray:mode:errorOCIHandle:.

The callback function pointers must return #'OCI_CONTINUE' if it the call is successful. Any return code
other than #'OCI_CONTINUE' signals that the client wishes to abort processing immediately.

When passing the address of a storage area, make sure that the storage area will exist even after 
the application returns from the callback. This means that you should not allocate such storage on the stack.

Parameters:

    aBindHandle (IN)
	An address of a bind handle. The bind handle maintains all the bind information for this particular input value.

    anInOCICallBackContext (IN)
	Pointer to a callback context which handles the allocation and deallocation of storage areas.
	(See OCI8CallBackData documentation for more information.)

    anOutOCICallBackContext (IN)
	Pointer to a callback context which handles the allocation and deallocation of storage areas.
	(See OCI8CallBackData documentation for more information.)

    anInOCICallbackFunction (IN)
	Pointer to a callback function which handles the allocation and deallocation of storage areas.
	(See OCI8CallBackData documentation for more information.)

    anOutOCICallbackFunction (IN)
	Pointer to a callback function which handles the allocation and deallocation of storage areas.
	(See OCI8CallBackData documentation for more information.)

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"


%{  /* UNLIMITEDSTACK */

    OCIBind * __bindp;
    OCIError * __errorp;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aBindHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isExternalAddress(anInOCICallBackContext) || !__isExternalAddress(anInOCICallbackFunction) ||
       !__isExternalAddress(anOutOCICallBackContext) || !__isExternalAddress(anOutOCICallbackFunction)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIBindDynamic called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __bindp = (OCIBind *) __externalAddressVal(aBindHandle);
    __errorp = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCIBindDynamic(     __bindp,
				__errorp,
				(dvoid *) __externalAddressVal(anInOCICallBackContext),
				(dvoid *) __externalAddressVal(anInOCICallbackFunction),
				(dvoid *) __externalAddressVal(anOutOCICallBackContext),
				(dvoid *) __externalAddressVal(anOutOCICallbackFunction));

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIBindDynamic(%x, %x, %x, %x, %x)\n", __bindp, __externalAddressVal(anInOCICallBackContext), __externalAddressVal(anOutOCICallBackContext), __externalAddressVal(anInOCICallbackFunction), __externalAddressVal(anOutOCICallbackFunction));
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIBindDynamic(%x, %x, %x, %x, %x) failed\n", __bindp, __externalAddressVal(anInOCICallBackContext), __externalAddressVal(anOutOCICallBackContext), __externalAddressVal(anInOCICallbackFunction), __externalAddressVal(anOutOCICallbackFunction));
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciDefineByPos:aStatementHandle newHandle:aNewDefineHandle position:aPositionInteger dataSize:aDataSize dataType:aDataTypeInteger intoDefineBuffer:aDefineBuffer nullValueIndicator:anIndicatorArray elementLengthArray:anElementLengthArray returnCodeArray:aReturnCodeArray mode:aModeInteger errorHandle:anErrorHandle 
"
defines an output buffer which will receive data retrieved from Oracle. The define is a local step which
is necessary when a SELECT statement returns data to your OCI application.

This call also implicitly allocates the define handle for the select-list item. If a non-NULL pointer is
passed in anOCI8DefineHandle, the OCI assumes that this points to a valid handle that has been previously
allocated with a call to OCI8Lib>>ociHandleAlloc:type: or this method. This would be true in the case of
an application which is redefining a handle to a different addresses so it can reuse the same define handle
for multiple fetches.

For NCHAR (fixed and varying length), the buffer pointer must point to an array of bytes sufficient for holding
the required NCHAR characters.

Nested table columns are defined and fetched like any other named data type.

When defining an array of descriptors or locators, you should pass in an array of pointers to descriptors or locators.

When doing an array define for character columns, you should pass in an array of character buffers.
If the mode parameter is this call is set to #'OCI_DYNAMIC_FETCH', the client application can fetch 
data dynamically at runtime. Runtime data can be provided in one of two ways: 

    1) a polling mechanism using calls supplied by the OCI. This mode is assumed if no callbacks are defined.
       In this case, the fetch call returns the #'OCI_NEED_ DATA' error code, and a piecewise polling method is
       used to provide the data.

    2) (NOT SUPPORTED BY THE OCI8LIB)
       callbacks using a user-defined function which must be registered with a subsequent call to OCIDefineDynamic().
       When the client library needs a buffer to return the fetched data, the callback will be invoked and the
       runtime buffers provided will return a piece or the whole data.

Parameters:

    aStatementHandle (IN/OUT)
	A handle to the requested SQL query operation.

    aNewDefineHandle (IN/OUT)
	A pointer to a pointer to a define handle. If this parameter is passed as nil, this call implicitly allocates
	the define handle. In the case of a redefine, a non-nil handle can be passed in this parameter.
	This handle is used to store the define information for this column.
	    Note: The user must keep track of this pointer. If a second call to this method is made for the same
		  column position, there is no guarantee that the same pointer is returned.

    aPositionInteger (IN)
	The position of this value in the select list. Positions are 1-based and are numbered from left to right.
	For example, in the SELECT statement: 

		SELECT empno, ssn, mgrno FROM employees; 

	empno is at position 1, ssn is at position 2, and mgrno is at position 3.

    aDataSize (IN)
	The size of each valuep buffer in bytes. If the data is stored internally in VARCHAR2 format, the number of
	characters desired, if different from the buffer size in bytes, may be additionally specified by the using
	OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle:. In an NLS conversion environment, a truncation
	error will be generated if the number of bytes specified is insufficient to handle the number of characters
	desired. If the #'OCI_ATTR_CHARSET_ID' is attribute is set to #'OCI_UCS2ID' (Unicode), all data passed to
	and received with the corresponding define call is assumed to be in UCS-2 encoding.

    aDataTypeInteger(IN)
	The data type. Named data type (#'SQLT_NTY') and REF (#'SQLT_REF') are valid only if the environment has been
	initialized with in object mode.

    aDefineBuffer (IN/OUT)
	A pointer to a buffer or an array of buffers of the type specified in the dty parameter. A number of buffers
	can be specified when results for more than one row are desired in a single fetch call.

    anIndicatorArray (IN)
	Pointer to an indicator variable or array. For scalar data types, pointer to sb2 or an array of sb2s.

    anElementLenghtArray (IN/OUT)
	Pointer to array of length of data fetched. Each element in rlenp is the length of the data in the
	corresponding element in the row after the fetch. 

    aReturnCodeArray (OUT)
	Pointer to array of column-level return codes 

    aModeInteger (IN)
	The valid modes are:

	    #'OCI_DEFAULT'       - This is the default mode.

	    #'OCI_DYNAMIC_FETCH' - (NOT SUPPORTED BY THE OCI8LIB)
				   For applications requiring dynamically allocated data at the time of fetch,
				   this mode must be used. The user may additionally call OCIDefineDynamic()
				   to set up a callback function that will be invoked to receive the dynamically
				   allocated buffers and. The valuep and value_sz parameters are ignored in this mode.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmtp;
    OCIError * __errorp;
    OCIDefine * __definep;
    dvoid * __valuep = 0;
    dvoid * __indicatorp = 0;
    ub2 *__elementlengthp = 0;
    ub2 *__returncodep = 0;
    ub4 __position;
    sb4 __value_sz;
    ub2 __datatype;
    ub4 __mode;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aStatementHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isExternalAddress(aNewDefineHandle) ||
       !__isSmallInteger(aDataTypeInteger) || !__isInteger(aModeInteger) || !__isInteger(aPositionInteger) ||
       !__isInteger(aDataSize)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIDefineByPos called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __stmtp   = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorp = (OCIError *) __externalAddressVal(anErrorHandle);
    __datatype  = (ub2) __intVal(aDataTypeInteger);
    __mode   = (ub4) __unsignedLongIntVal(aModeInteger);
    __position  = __unsignedLongIntVal(aPositionInteger);
    __value_sz = (sb4) __unsignedLongIntVal(aDataSize);

    if (__isExternalBytes(anIndicatorArray)) {
	__indicatorp = (dvoid *) __externalBytesAddress(anIndicatorArray);
    } else if (anIndicatorArray != nil) {
	RETURN(@symbol(attributeError));
    }
    if (__isExternalBytes(anElementLengthArray)) {
	__elementlengthp = (ub2 *) __externalBytesAddress(anElementLengthArray);
    } else if (anElementLengthArray != nil) {
	RETURN(@symbol(attributeError));
    }
    if (__isExternalBytes(aReturnCodeArray)) {
	__returncodep = (ub2 *) __externalBytesAddress(aReturnCodeArray);
    } else if (aReturnCodeArray != nil) {
	RETURN(@symbol(attributeError));
    }

    __definep  = (OCIDefine *) __externalAddressVal(aNewDefineHandle);
    if (__definep == (OCIDefine *) nil)
	__definep = 0;

    if (aDefineBuffer != nil) {
	if (!__isExternalBytes(aDefineBuffer)) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [error]: define buffer in OCIDefineByPos must be an external bytes object.\n");
	    }
	    RETURN(@symbol(attributeError));
	}
	__valuep   = (dvoid *) __externalBytesAddress(aDefineBuffer);
    }

    __ret = OCIDefineByPos(     __stmtp,
				&__definep,
				__errorp,
				__position,
				__valuep,
				__value_sz,
				__datatype,
				__indicatorp,
				__elementlengthp,
				__returncodep,
				__mode);

    if (__ret == OCI_SUCCESS) {
	__externalAddressVal(aNewDefineHandle) = __definep;
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIDefineByPos(%x, %d, %d, %x, %d) -> %x\n", __stmtp, __mode, __position, __datatype, __value_sz, __definep);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIDefineByPos(%x, %d, %d, %x) failed\n", __stmtp, __mode, __position, __datatype);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciDescribeAny:aServiceHandle schemaName:aSchemaString schemaType:aSchemaTypeInteger newHandle:aDescribeHandle errorHandle:anErrorHandle 
"
describes existing schema objects: tables, views, synonyms, procedures, functions, packages, sequences, types,
schemas, and databases. This call also describes sub-schema objects, such as a column in a table. This call
populates the describe handle with the object-specific attributes which can be obtained through an
OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: call.

An OCI8Lib>>ociParamGet:position:errorOCIHandle: on the describe handle returns a parameter descriptor for
a specified position. Parameter positions begin with 1. Calling OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle:
on the parameter descriptor returns the specific attributes of a stored procedure or function parameter,
or a table column descriptor. These subsequent calls do not need an extra round trip to the server because
the entire schema object description is cached on the client side by this method. 
Calling OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: on the describe handle also returns the total number
of positions. 

If the #'OCI_ATTR_DESC_PUBLIC' attribute is set on the describe handle, then the object named is looked up as
a public synonym when the object does not exist in the current schema and only <name1> is specified.

Parameters:

    aServiceHandle (IN)
	A service context handle.

    aSchemaString (IN)
	A string containing the name of the object to be described.  The string should be in the format 
	<name1>[.<name2>.<name3>...][@<linkname>], such as scott.emp.empno@mydb. Database links are only
	allowed to Oracle8i databases. The object name is interpreted by the following SQL rules:

	    1) If only <name1> is entered and objtyp is equal to #'OCI_PTYPE_SCHEMA', then the name refers
	       to the named schema. The Oracle database must be release 8.1 or greater.

	    2) If only <name1> is entered and objtyp is equal to #'OCI_PTYPE_DATABASE', then the name refers
	       to the named database. When describing a remote database with database_name@db_link_name,
	       the remote Oracle database must be release 8.1 or greater. 

	    3) If only <name1> is entered and objtyp is not equal to #'OCI_PTYPE_SCHEMA' or #'OCI_PTYPE_DATABASE',
	       then the name refers to the named object (of type table / view / procedure / function / package /
	       type / synonym / sequence) in the current schema of the current user. When connected to an Oracle7
	       Server, the only valid types are procedure and function. 

	    4) If <name1>.<name2>.<name3>... is entered, the object name refers to a schema/sub-schema object in
	       the schema named <name1>. For example, in the string scott.emp.deptno, scott is the name of the schema,
	       emp is the name of a table in the schema, and deptno is the name of a column in the table.

    aSchemaTypeInteger (IN/OUT)
	The type of schema object being described. Valid values are:

		1) OCI_PTYPE_TABLE, for tables 
		2) OCI_PTYPE_VIEW, for views 
		3) OCI_PTYPE_PROC, for procedures 
		4) OCI_PTYPE_FUNC, for functions 
		5) OCI_PTYPE_PKG, for packages 
		6) OCI_PTYPE_TYPE, for types 
		7) OCI_PTYPE_SYN, for synonyms 
		8) OCI_PTYPE_SEQ, for sequences
		9) OCI_PTYPE_SCHEMA, for schemas 
		10) OCI_PTYPE_DATABASE, for databases 
		11) OCI_PTYPE_UNK, for unknown schema objects

	 A value for this argument must be specified. If OCI_PTYPE_UNK is specified, then the description of an object
	 with the specified name in the current schema is returned, if such an object exists, along with the actual
	 type of the object.

    aDescribeHandle (IN/OUT) 
	A describe handle that is populated with describe information about the object after the call. 
	Must be non-NULL.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svchp;
    OCIError * __errorp;
    OCIDescribe * __describep;
    dvoid * __objptr;
    ub4 __objptr_len;
    ub1 __objtyp;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isExternalAddress(aDescribeHandle) || !__isSmallInteger(aSchemaTypeInteger) ||
       !__isString(aSchemaString)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIDescribeAny called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svchp   = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorp = (OCIError *) __externalAddressVal(anErrorHandle);
    __describep = (OCIDescribe *) __externalAddressVal(aDescribeHandle);
    __objtyp = __intVal(aSchemaTypeInteger);
    __objptr = __stringVal(aSchemaString);
    __objptr_len = __stringSize(aSchemaString);

    __ret = OCIDescribeAny(     __svchp,
				__errorp,
				__objptr,
				__objptr_len,
				OCI_OTYPE_NAME,
				OCI_DEFAULT,
				__objtyp,
				__describep);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIDescribeAny(%x)\n", __svchp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIDescribeAny(%x) failed\n", __svchp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.
! !

!OCI8Lib methodsFor:'connect, authorize, initialize'!

ociEnvCreate:anOCI8EnvironmentHandle mode:aModeSymbolColl 
"
creates an environment for all the OCI calls using the modes specified by the user.
This call should be invoked before any other OCI call. This call initializes an environment handle
which is then used by the remaining OCI functions. There can be multiple environments in OCI,
each with its own environment modes. This function also performs any process level initialization if
required by any mode. For example if the user wants to initialize an environment as #'OCI_THREADED',
then all libraries that are used by OCI are also initialized in the threaded mode.

Parameters:

    anOCI8EnvironmentHandle (OUT)
	A pointer to a handle to the environment.

    aModeSymbolColl (IN)
	Specifies initialization of the mode. Valid modes are: 
	    #'OCI_DEFAULT'      - uses default mode.
	    #'OCI_THREADED'     - uses threaded environment. Internal data structures not exposed to the user 
				  are protected from concurrent accesses by multiple threads. 
	    #'OCI_OBJECT'       - uses object features.
	    #'OCI_SHARED'       - utilizes shared data structures. 
	    #'OCI_EVENTS'       - utilizes publish-subscribe notifications.
	    #'OCI_NO_UCB'       - suppresses the calling of the dynamic callback routine. 
				  The default behavior is to allow calling of callback routines at the time that the 
				  environment is created.
	    #'OCI_ENV_NO_MUTEX' - no mutexing in this mode. All OCI calls done on the environment handle, or on 
				  handles derived from the environment handle, must be serialized.

<return: self>
"
    |modeInteger|

    aModeSymbolColl isEmpty ifTrue:[
	OCIErrorSignal raiseWith:self errorString:'OCI-Error: no mode for environment creation specified'.
    ].
    modeInteger := 0.
    aModeSymbolColl do:[:eachModeSymbol|
	modeInteger := modeInteger bitOr:(self class numericValueForEnvironmentMode:eachModeSymbol).
    ].

    self executePrimitive:[
	self primOciEnvCreate:anOCI8EnvironmentHandle handle mode:modeInteger.
    ] withErrorHandle:nil.
!

ociServerAttach:anOCI8ServerHandle dbname:aDBName errorHandle:anOCI8ErrorHandle
"
creates an association between an OCI application and a particular server. This call initializes
a server context handle, which must have been previously allocated with a call to 
OCI8Lib>>ociHandleAlloc:type:. The server context handle initialized by this call can be associated with
a service context through a call to OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle:.
Once that association has been made, OCI operations can be performed against 
the server. If an application is operating against multiple servers, multiple server context handles 
can be maintained. OCI operations are performed against whichever server context is currently associated
with the service context. When OCI8Lib>>ociServerAttach:dbname:errorHandle: is successfully completed,
an Oracle shadow process is started. OCI8Lib>>ociSessionEnd:userSession:errorHandle: and 
OCI8Lib>>ociServerDetach:errorHandle: should be called to clean up the Oracle shadow process.
Otherwise, the shadow processes accumulate and cause the system to run out of processes.
If the database is restarted and there are not enough processes, the database may not startup.

Parameters:

    anOCI8ServerHandle (IN/OUT)
	An uninitialized server handle, which gets initialized by this call. Passing in an initialized
	server handle causes an error.

    aDBName (IN)
	Specifies the database (server) to use. This string specifies a connect string or a service point.
	If the connect string is nil, then this call attaches to the default host.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    ^ self executePrimitive:[
	self primOciServerAttach:(anOCI8ServerHandle handle)
			  dbname:aDBName
		     errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle
!

ociServerDetach:anOCI8ServerHandle errorHandle:anOCI8ErrorHandle
"
deletes an access to a data source for OCI operations, which was established
by a call to OCI8Lib>>ociServerAttach:dbname:errorHandle:.

Parameters:

    anOCI8ServerHandle (IN)
	A handle to an initialized server context, which gets reset to uninitialized state.
	The handle is not de-allocated.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    ^ self executePrimitive:[
	self primOciServerDetach:(anOCI8ServerHandle handle) 
		     errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle
!

ociSessionBegin:anOCI8ServiceHandle userSession:anOCI8UserSessionHandle credential:aCredSymbol mode:aModeSymbol errorHandle:anOCI8ErrorHandle
"
used to authenticate a user against the server set in the service context handle.

For Oracle8i, this method must be called for any given server handle before requests can be made against it.
Also, this method only supports authenticating the user for access to the Oracle server specified by the
server handle in the service context. In other words, after OCI8Lib>>ociServerAttach:dbname:errorHandle:
is called to initialize a server handle, this method must be called to authenticate the user for that given server.

When this method is called for the first time for a given server handle, the user session may not be created
in migratable (#'OCI_MIGRATE') mode.

After this method has been called for a server handle, the application may call this method again to initialize
another user session handle with different (or the same) credentials and different (or the same) operation modes.
If an application wants to authenticate a user in #'OCI_MIGRATE' mode, the service handle must already be associated
with a non-migratable user handle. The user ID of that user handle becomes the ownership ID of the migratable
user session. Every migratable session must have a non-migratable parent session.

If the #'OCI_MIGRATE' mode is not specified, then the user session context can only be used with the same
server handle set in aServiceHandle. If #'OCI_MIGRATE' mode is specified, then the user authentication may be set
with different server handles. However, the user session context may only be used with server handles which resolve
to the same database instance. Security checking is done during session switching. A process or circuit is allowed
to switch to a migratable session only if the ownership ID of the session matches the user ID of a non-migratable
session currently connected to that same process or circuit, unless it is the creator of the session.

Modes #'OCI_SYSDBA', #'OCI_SYSOPER', and #'OCI_PRELIM_AUTH' may only be used with a primary user session context.

To provide credentials for a call to this method, one of two methods are supported. The first is to provide
a valid username and password pair for database authentication in the user session handle passed to this method.
This involves using OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle: to set the #'OCI_ATTR_USERNAME'
and #'OCI_ATTR_PASSWORD' attributes on the user session handle. Then this method is called with #'OCI_CRED_RDBMS'.

    Note: When the user session handle is terminated using OCI8Lib>>ociSessionEnd:userSession:errorHandle:,
	  the username and password attributes remain unchanged and thus can be re-used in a future call to
	  this method. Otherwise, they must be reset to new values before the next call to this method.

The second type of credentials supported are external credentials. No attributes need to be set on the user 
session handle before calling this method. The credential type is #'OCI_CRED_EXT'. This is equivalent to the Oracle7
'connect /'  syntax. If values have been set for #'OCI_ATTR_USERNAME' and #'OCI_ATTR_ PASSWORD', then these are
ignored if #'OCI_CRED_EXT' is used. Another way of setting credentials is to use the session Id of an already
authenticated user with the #'OCI_MIGSESSION' attribute. This Id can be extracted from the session handle of
an authenticated user using the OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: call.

Parameters:

    anOCI8ServiceHandle (IN)
	A handle to a service context. There must be a valid server handle set in aServiceHandle.

    anOCI8UserSessionHandle (IN/OUT)
	A handle to an user session context, which is initialized by this call.

    aCredSymbol (IN)
	Specifies the type of credentials to use for establishing the user session. Valid values for credt are:
	    #'OCI_CRED_RDBMS' - authenticate using a database username and password pair as credentials.
				The attributes OCI_ATTR_USERNAME and OCI_ATTR_ PASSWORD should be set on
				the user session context before this call. 
	    #'OCI_CRED_EXT'   - authenticate using external credentials. No username or password is provided. 

    aModeSymbol (IN)
	Specifies the various modes of operation. Valid modes are:
	    #'OCI_DEFAULT'     - in this mode, the user session context returned may only ever be set with 
				 the same server context specified in svchp.
	    #'OCI_MIGRATE'     - in this mode, the new user session context may be set in a service handle
				 with a different server handle. This mode establishes the user session context.
				 To create a migratable session, the service handle must already be set with a
				 non-migratable user session. A migratable session must have a non-migratable
				 parent session.
	    #'OCI_SYSDBA'      - in this mode, the user is authenticated for SYSDBA access. 
	    #'OCI_SYSOPER'     - in this mode, the user is authenticated for SYSOPER access. 
	    #'OCI_PRELIM_AUTH' - this mode may only be used with OCI_SYSDBA or OCI_SYSOPER to authenticate 
				 for certain administration tasks.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    ^ self executePrimitive:[
	self primOciSessionBegin:anOCI8ServiceHandle handle 
		     userSession:anOCI8UserSessionHandle handle 
		      credential:(self class numericValueForCredentialType:aCredSymbol)
			    mode:(self class numericValueForSessionMode:aModeSymbol) 
		     errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle
!

ociSessionEnd:anOCI8ServiceHandle userSession:anOCI8UserSessionHandle errorHandle:anOCI8ErrorHandle
"
terminates a user session context created by OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.
The user security context associated with the service context is invalidated by this call.
Storage for the user session context is not freed. The transaction specified by the service
context is implicitly committed. The transaction handle, if explicitly allocated, may be freed
if not being used. Resources allocated on the server for this user are freed. The user session handle
may be reused in a new call to OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.

Parameters:

    anOCI8ServiceHandle (IN/OUT)
	The service context handle. There must be a valid server handle and user session handle
	associated with aServiceHandle.

    anOCI8UserSessionHandle (IN)
	De-authenticate this user. If this parameter is passed as nil, the user in the service context handle
	is de-authenticated.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    ^ self executePrimitive:[
	self primOciSessionEnd:anOCI8ServiceHandle handle 
		   userSession:anOCI8UserSessionHandle handle 
		   errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

primOciEnvCreate:anEnvHandle mode:aModeInteger
"
creates an environment for all the OCI calls using the modes specified by the user.
This call should be invoked before any other OCI call. This call initializes an environment handle
which is then used by the remaining OCI functions. There can be multiple environments in OCI,
each with its own environment modes. This function also performs any process level initialization if
required by any mode. For example if the user wants to initialize an environment as #'OCI_THREADED',
then all libraries that are used by OCI are also initialized in the threaded mode.

Parameters:

    anEnvHandle (OUT)
	A pointer to a handle to the environment.

    aModeInteger (IN)
	Specifies initialization of the mode. Valid modes are: 
	    #'OCI_DEFAULT'      - uses default mode.
	    #'OCI_THREADED'     - uses threaded environment. Internal data structures not exposed to the user 
				  are protected from concurrent accesses by multiple threads. 
	    #'OCI_OBJECT'       - uses object features.
	    #'OCI_SHARED'       - utilizes shared data structures. 
	    #'OCI_EVENTS'       - utilizes publish-subscribe notifications.
	    #'OCI_NO_UCB'       - suppresses the calling of the dynamic callback routine. 
				  The default behavior is to allow calling of callback routines at the time that the 
				  environment is created.
	    #'OCI_ENV_NO_MUTEX' - no mutexing in this mode. All OCI calls done on the environment handle, or on 
				  handles derived from the environment handle, must be serialized.

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */
    OCIEnv *__envhp = 0;
    ub4 __mode = (ub4) __unsignedLongIntVal(aModeInteger);
    sword __ret = OCI_ERROR;

    if (__isExternalAddress(anEnvHandle)) {
	__ret = OCIEnvCreate(   &__envhp,
				__mode,
				0,
				0,
				0,
				0,
				0,
				0 );

	if (__ret == OCI_SUCCESS) {
	    __externalAddressVal(anEnvHandle) = __envhp;
	}
    }
    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIEnvCreate -> %x\n", __envhp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIEnvCreate failed\n");
	}
    };

    RETURN(__ociReturnSymbol(__ret));
%}
!

primOciServerAttach:aServerHandle dbname:aDBName errorHandle:anErrorHandle
"
creates an association between an OCI application and a particular server. This call initializes
a server context handle, which must have been previously allocated with a call to 
OCI8Lib>>ociHandleAlloc:type:. The server context handle initialized by this call can be associated with
a service context through a call to OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle:.
Once that association has been made, OCI operations can be performed against 
the server. If an application is operating against multiple servers, multiple server context handles 
can be maintained. OCI operations are performed against whichever server context is currently associated
with the service context. When OCI8Lib>>ociServerAttach:dbname:errorHandle: is successfully completed,
an Oracle shadow process is started. OCI8Lib>>ociSessionEnd:userSession:errorHandle: and 
OCI8Lib>>ociServerDetach:errorHandle: should be called to clean up the Oracle shadow process.
Otherwise, the shadow processes accumulate and cause the Unix system to run out of processes.
If the database is restarted and there are not enough processes, the database may not startup.

Parameters:

    aServerHandle (IN/OUT)
	An uninitialized server handle, which gets initialized by this call. Passing in an initialized
	server handle causes an error.

    aDBName (IN)
	Specifies the database (server) to use. This string specifies a connect string or a service point.
	If the connect string is nil, then this call attaches to the default host.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIServer * __srvhndlp  = 0;
    OCIError * __errorhndlp = 0;
    sword __ret;
    text * __dbname  = 0;
    sb4 __dbname_len = 0;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServerHandle) || !__isExternalAddress(anErrorHandle) 
       || (aDBName != nil && !__isString(aDBName))) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerAttach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __srvhndlp = (OCIServer *) __externalAddressVal(aServerHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    if (aDBName != nil) {
	 __dbname = __stringVal(aDBName);
	 __dbname_len = __stringSize(aDBName);
    }

    __ret = OCIServerAttach(    __srvhndlp,
				__errorhndlp,
				__dbname,
				__dbname_len,
				OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIServerAttach(%x, ", __srvhndlp);
	    fwrite(__dbname, 1, __dbname_len, stderr);
	    fprintf(stderr,")\n");
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIServerAttach(%x) failed\n", __srvhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciServerDetach:aServerHandle errorHandle:anErrorHandle
"
deletes an access to a data source for OCI operations, which was established
by a call to OCI8Lib>>ociServerAttach:dbname:errorHandle:.

Parameters:

    aServerHandle (IN)
	A handle to an initialized server context, which gets reset to uninitialized state.
	The handle is not de-allocated.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIServer * __srvhndlp  = 0;
    OCIError * __errorhndlp = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aServerHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerDetach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __srvhndlp = (OCIServer *) __externalAddressVal(aServerHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCIServerDetach(    __srvhndlp,
				__errorhndlp,
				OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIServerDetach(%x)\n", __srvhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIServerDetach(%x) failed\n", __srvhndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciSessionBegin:aServiceHandle userSession:aUserSessionHandle credential:aCredInteger mode:aModeInteger errorHandle:anErrorHandle
"
used to authenticate a user against the server set in the service context handle.

    Note:   Check for any errors returned when trying to start a session. 
	    For example, if the password for the account has expired,
	    an ORA-28001 error is returned.

For Oracle8i, this method must be called for any given server handle before requests can be made against it.
Also, this method only supports authenticating the user for access to the Oracle server specified by the
server handle in the service context. In other words, after OCI8Lib>>ociServerAttach:dbname:errorHandle:
is called to initialize a server handle, this method must be called to authenticate the user for that given server.

When this method is called for the first time for a given server handle, the user session may not be created
in migratable (#'OCI_MIGRATE') mode.

After this method has been called for a server handle, the application may
call this method again to initialize another user session handle with different (or the same) credentials and 
different (or the same) operation modes. If an application wants to authenticate a user in #'OCI_MIGRATE' mode,
the service handle must already be associated with a non-migratable user handle. The user ID of that user handle
becomes the ownership ID of the migratable user session. Every migratable session must have a non-migratable
parent session.

If the #'OCI_MIGRATE' mode is not specified, then the user session context can only be used with the same
server handle set in aServiceHandle. If #'OCI_MIGRATE' mode is specified, then the user authentication may be set
with different server handles. However, the user session context may only be used with server handles which resolve
to the same database instance. Security checking is done during session switching. A process or circuit is allowed
to switch to a migratable session only if the ownership ID of the session matches the user ID of a non-migratable
session currently connected to that same process or circuit, unless it is the creator of the session.

#'OCI_SYSDBA', #'OCI_SYSOPER', and #'OCI_PRELIM_AUTH' may only be used with a primary user session context.

To provide credentials for a call to this method, one of two methods are supported. The first is to provide
a valid username and password pair for database authentication in the user session handle passed to this method.
This involves using OCI8Lib>>ociAttrSet:attribute:attributeType:errorOCIHandle: to set the #'OCI_ATTR_USERNAME'
and #'OCI_ATTR_PASSWORD' attributes on the user session handle. Then this method is called with #'OCI_ CRED_RDBMS'.

    Note: When the user session handle is terminated using OCI8Lib>>ociSessionEnd:userSession:errorHandle:,
	  the username and password attributes remain unchanged and thus can be re-used in a future call to
	  this method. Otherwise, they must be reset to new values before the next call to this method.

The second type of credentials supported are external credentials. No attributes need to be set on the user 
session handle before calling this method. The credential type is #'OCI_CRED_EXT'. This is equivalent to the Oracle7
'connect /'  syntax. If values have been set for #'OCI_ATTR_USERNAME' and #'OCI_ATTR_ PASSWORD', then these are
ignored if #'OCI_CRED_EXT' is used. Another way of setting credentials is to use the session Id of an already
authenticated user with the #'OCI_MIGSESSION' attribute. This Id can be extracted from the session handle of
an authenticated user using the OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: call.

Parameters:

    aServiceHandle (IN)
	A handle to a service context. There must be a valid server handle set in aServiceHandle.

    aUserSessionHandle (IN/OUT)
	A handle to an user session context, which is initialized by this call.

    aCredInteger (IN)
	Specifies the type of credentials to use for establishing the user session. Valid values for credt are:
	    #'OCI_CRED_RDBMS' - authenticate using a database username and password pair as credentials.
				The attributes OCI_ATTR_USERNAME and OCI_ATTR_ PASSWORD should be set on
				the user session context before this call. 
	    #'OCI_CRED_EXT'   - authenticate using external credentials. No username or password is provided. 

    aModeInteger (IN)
	Specifies the various modes of operation. Valid modes are:
	    #'OCI_DEFAULT'     - in this mode, the user session context returned may only ever be set with 
				 the same server context specified in svchp.
	    #'OCI_MIGRATE'     - in this mode, the new user session context may be set in a service handle
				 with a different server handle. This mode establishes the user session context.
				 To create a migratable session, the service handle must already be set with a
				 non-migratable user session. A migratable session must have a non-migratable
				 parent session.
	    #'OCI_SYSDBA'      - in this mode, the user is authenticated for SYSDBA access. 
	    #'OCI_SYSOPER'     - in this mode, the user is authenticated for SYSOPER access. 
	    #'OCI_PRELIM_AUTH' - this mode may only be used with OCI_SYSDBA or OCI_SYSOPER to authenticate 
				 for certain administration tasks.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svchndlp;
    OCISession * __usrhndlp;
    OCIError * __errorhndlp;
    sword __ret;
    ub4 __credt;
    ub4 __mode;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || !__isExternalAddress(aUserSessionHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(aCredInteger) || !__isInteger(aModeInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCISessionBegin called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svchndlp  = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __usrhndlp = (OCISession *) __externalAddressVal(aUserSessionHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __credt = __unsignedLongIntVal(aCredInteger);
    __mode  = __unsignedLongIntVal(aModeInteger);

    __ret = OCISessionBegin(    __svchndlp,
				__errorhndlp,
				__usrhndlp,
				__credt,
				__mode);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCISessionBegin(%x, %x, %d, %d)\n", __svchndlp, __usrhndlp, __credt, __mode);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCISessionBegin(%x, %x, %d, %d) failed\n", __svchndlp, __usrhndlp, __credt, __mode);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciSessionEnd:aServiceHandle userSession:aUserSessionHandle errorHandle:anErrorHandle
"
terminates a user session context created by OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.
The user security context associated with the service context is invalidated by this call.
Storage for the user session context is not freed. The transaction specified by the service
context is implicitly committed. The transaction handle, if explicitly allocated, may be freed
if not being used. Resources allocated on the server for this user are freed. The user session handle
may be reused in a new call to OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.

Parameters:

    aServiceHandle (IN/OUT)
	The service context handle. There must be a valid server handle and user session handle
	associated with aServiceHandle.

    aUserSessionHandle (IN)
	De-authenticate this user. If this parameter is passed as nil, the user in the service context handle
	is de-authenticated.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svchndlp;
    OCISession * __usrhndlp;
    OCIError * __errorhndlp;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || 
       !__isExternalAddress(aUserSessionHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCISessionEnd called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svchndlp  = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __usrhndlp = (OCISession *) __externalAddressVal(aUserSessionHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCISessionEnd(      __svchndlp,
				__errorhndlp,
				__usrhndlp,
				OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCISessionEnd(%x, %x)\n", __svchndlp, __usrhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCISessionEnd(%x, %x) failed\n", __svchndlp, __usrhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

! !

!OCI8Lib methodsFor:'error handling'!

createExceptionFor:aStatusSymbol errorHandle:anOCI8ErrorHandle
    "Answer a exception for a status symbol returned by an OCI primitive.

     aStatusSymbol      <Symbol>                returned by OCI primitive operations 
						(OCI_SUCCESS, OCI_ERROR, OCI_INVALID_HANDLE ...)
     anOCI8ErrorHandle  <nil|errorHandle>       used to retrieve the error code and description.
    "
"/    |errorCode textArray errorText exception errorHandle|

    "OCI Errors"

    (aStatusSymbol == #'OCI_ERROR' 
     or:[aStatusSymbol == #'OCI_SUCCESS_WITH_INFO']) ifTrue:[
    |errorCode textArray errorText exception errorHandle|

	(anOCI8ErrorHandle isNil or:[(errorHandle := anOCI8ErrorHandle handle) isNil 
				     or:[errorHandle address == 0]]) ifTrue:[
	    ^ OCIErrorSignal newException messageText:' - got ', aStatusSymbol, ' without error handle'.
	].

	errorCode := self errorCodeForHandle:errorHandle
			  recordNumber:1
			  errorTextHolder:(textArray := Array new:1).

	errorCode isSymbol ifTrue:[
	    ^ OCIInternalError newException
			    messageText:' -- error while retrieving errorCode from error handle: ', errorCode.
	] ifFalse:[
	    aStatusSymbol == #'OCI_SUCCESS_WITH_INFO' ifTrue:[
		exception := OCISuccessWithInfoSignal newException.
	    ] ifFalse:[
		exception := OCIErrorSignal newExceptionForErrorNumber:errorCode.
	    ].
	    errorText := textArray at:1.
	    errorText size > 0 ifTrue:[
		exception messageText:errorText.
	    ].
	].
	exception errorCode:errorCode; errorHandle:anOCI8ErrorHandle.
	^ exception.
    ].
    aStatusSymbol == #'OCI_STILL_EXECUTING' ifTrue:[
	^ OCIStillExecutingSignal newException.
    ].
    aStatusSymbol == #'OCI_NO_DATA' ifTrue:[
	^ OCINoDataSignal newException.
    ].
    aStatusSymbol == #'OCI_NEED_DATA' ifTrue:[
	^ OCINeedDataSignal newException.
    ].
    aStatusSymbol == #'OCI_INVALID_HANDLE' ifTrue:[
	^ OCIInternalError newException messageText:' - invalid handle'.
    ].
    aStatusSymbol == #'OCI_CONTINUE' ifTrue:[
	^ OCIContinueSignal newException.
    ].

    "ST/X Errors"
    aStatusSymbol == #attributeError ifTrue:[
	^ OCIInternalError newException messageText:' - invalid attribute in primitive'.
    ].
    aStatusSymbol == #mallocError ifTrue:[
	^ OCIMallocErrorSignal newException.
    ].
    ^ OCIInternalError newException messageText:' - unknown status: ', aStatusSymbol printString.
!

errorCodeForHandle:anErrorHandle recordNumber:recordNumber errorTextHolder:resultArray
    "Return the errorCode stored in the errorHandler as SmallInteger.
     If errorTextHolder is an array with at least one element,
     the error text is stored in the first element as String.

     More than on error record may be retrieved from the handle.
     recordNumber (1-based) says, which error record should be retrieved.

     <return: SmallInteger|Symbol|nil>
    "
%{  /* UNLIMITEDSTACK */

    text errbuf[512];
    sb4 errcode = 0;
    OBJ __string;
    dvoid *errhp;
    sword __returnCode;
    ub4 __recordno;

    if (!__isExternalAddress(anErrorHandle))
        RETURN(@symbol(badErrorHandle));

    if (!__isSmallInteger(recordNumber))
        RETURN(@symbol(badArgument));

    __recordno = __smallIntegerVal(recordNumber);

    errhp = (dvoid *) __externalAddressVal(anErrorHandle);
    errbuf[0] = '\0';

    __returnCode = OCIErrorGet(errhp, __recordno, (text *) NULL, &errcode,
                               errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);

    if (__returnCode != OCI_SUCCESS) {
        RETURN(nil);
    }

    if (__isArray(resultArray) && __arraySize(resultArray) > 0) {
        int len = strlen(errbuf);
        if (len > 0 && errbuf[len-1] == '\n')
            len--;

        __string = __MKSTRING_L(errbuf, len);
        __ArrayInstPtr(resultArray)->a_element[0] = __string;
        __STORE(resultArray, __string);
    }

    RETURN(__MKSMALLINT(errcode));
%}
!

handleError:aStatusSymbol errorHandle:anOCI8ErrorHandle
    "handle errors from oci operations.

     aStatusSymbol      <Symbol>                returned by OCI primitive operations 
						(OCI_SUCCESS, OCI_ERROR, OCI_INVALID_HANDLE ...)
     anOCI8ErrorHandle  <nil|errorHandle>       used to retrieve the error code and description.
    "

    |exception|

    aStatusSymbol == #'OCI_SUCCESS' ifTrue:[
	"all is well"
	^ self.
    ].

    exception := self createExceptionFor:aStatusSymbol errorHandle:anOCI8ErrorHandle.
    aStatusSymbol == #'OCI_SUCCESS_WITH_INFO' ifTrue:[
	exception raiseRequest.
    ] ifFalse:[
	exception raise.
    ].
! !

!OCI8Lib methodsFor:'execution'!

executePrimitive:aBlock withErrorHandle:anErrorHandle
    "
     evaluates aBlock after obtaining the execution log. 
     The block is repeatently evaluated as long as the result status indicates that the
     operation is still executing.
     This is necessary because the oci lib is driven in non blocking mode. 
     Between two block evaluations wait some time.
     TODO: This should be implemented as exponential backoff!!

     If the result does not indicate a successfull operation, raise a signal.
     anErrorHandle is used to retrieve the error in this case.

     <return: Symbol>
    "

    |statusSymbol delay|

    delay := Delay forMilliseconds:10.

    self executionLock critical:[
	[
	    statusSymbol := aBlock value.
	] doWhile:[statusSymbol == #'OCI_STILL_EXECUTING' and:[delay wait. true]].

	statusSymbol ~~ #'OCI_SUCCESS' ifTrue:[
	    self handleError:statusSymbol errorHandle:anErrorHandle.
	].
    ].
    ^ statusSymbol

! !

!OCI8Lib methodsFor:'handle & descriptor'!

ociAttrGet:anOCI8TargetHandle attributeType:anAttributeTypeSymbol errorOCIHandle:anOCI8ErrorHandle
"
used to get a particular attribute of a handle.

Parameters:

    aTargetOCIHandle (IN)
	Pointer to a handle type. 

    anAttributeTypeSymbol (IN)
	The type of attribute being retrieved.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self | String | Integer | ExternalAddress>
"

    |errorHandle aTypeSymbol aTypeInteger resultHolder statusSymbol|

    errorHandle  := anOCI8ErrorHandle handle.
    aTypeSymbol  := anOCI8TargetHandle type.
    aTypeInteger := self class numericValueForAttributes:anAttributeTypeSymbol.
    resultHolder := Array new:1.

    ^self executePrimitive:[
	aTypeSymbol == #'OCI_HTYPE_SVCCTX' ifTrue:[
	    statusSymbol := self primOciAttrGetForServiceContextHandle:anOCI8TargetHandle handle 
							attributeType:aTypeInteger
							  errorHandle:errorHandle
							  resultHolder:resultHolder.
	]. 
	aTypeSymbol == #'OCI_HTYPE_SERVER' ifTrue:[
	    statusSymbol := self primOciAttrGetForServerHandle:anOCI8TargetHandle handle 
						attributeType:aTypeInteger 
						  errorHandle:errorHandle
						  resultHolder:resultHolder.
	]. 
	aTypeSymbol == #'OCI_HTYPE_SESSION' ifTrue:[
	    statusSymbol := self primOciAttrGetForUserSessionHandle:anOCI8TargetHandle handle 
						     attributeType:aTypeInteger 
						       errorHandle:errorHandle
						       resultHolder:resultHolder.
	].
	aTypeSymbol == #'OCI_HTYPE_STMT' ifTrue:[
	    statusSymbol := self primOciAttrGetForStatementHandle:anOCI8TargetHandle handle 
						   attributeType:aTypeInteger 
						     errorHandle:errorHandle
						     resultHolder:resultHolder.
	].
	aTypeSymbol == #'OCI_HTYPE_ERROR' ifTrue:[
	    statusSymbol := self primOciAttrGetForErrorHandle:anOCI8TargetHandle handle 
						   attributeType:aTypeInteger 
						     errorHandle:errorHandle
						     resultHolder:resultHolder.
	].
	aTypeSymbol == #'OCI_HTYPE_DESCRIBE' ifTrue:[
	    statusSymbol := self primOciAttrGetForDescribeHandle:anOCI8TargetHandle handle 
						  attributeType:aTypeInteger 
						    errorHandle:errorHandle
						    resultHolder:resultHolder.
	]. 
	aTypeSymbol == #'OCI_DTYPE_PARAM' ifTrue:[
	    statusSymbol := self primOciAttrGetForParameterHandle:anOCI8TargetHandle handle 
						   attributeType:aTypeInteger 
						     errorHandle:errorHandle
						     resultHolder:resultHolder.
	].
	statusSymbol isNil ifTrue:[
	    OCIInternalError raiseErrorString:' - unsupported attribute handle type'.
	].
	statusSymbol == #'OCI_SUCCESS' ifTrue:[
	    ^ resultHolder at:1.
	].
	statusSymbol
    ] withErrorHandle:anOCI8ErrorHandle. 
!

ociAttrSet:anOCI8TargetHandle attribute:anAttribute attributeType:anAttributeTypeSymbol errorOCIHandle:anOCI8ErrorHandle
"
used to set a particular attribute of a handle or a descriptor.

Parameters:

    aTargetOCIHandle (IN/OUT) 
	Pointer to a handle type whose attribute gets modified. 

    anAttribute (IN)
	Pointer to an attribute value. The attribute value is copied into the target handle. 
	If the attribute value is a pointer, then only the pointer is copied, not the contents of the pointer.

    anAttributeTypeSymbol (IN) 
	The type of attribute being set.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"

    |errorHandle|

    errorHandle  := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciAttrSet:anOCI8TargetHandle handle 
		  targetType:(self class numericValueForType:anOCI8TargetHandle type) 
		   attribute:anAttribute 
	       attributeType:(self class numericValueForAttributes:anAttributeTypeSymbol) 
		 errorHandle:errorHandle.
    ] withErrorHandle:errorHandle
!

ociHandleAlloc:anOCI8ParentHandle type:aTypeSymbol
"
returns a pointer to an allocated and initialized handle, corresponding to the type specified in aTypeSymbol.
A non-nil handle is returned on success. All handles are allocated with respect to an environment handle
which is passed in as a parent handle. No diagnostics are available on error. 
Handles must be allocated using this method before they can be passed into an OCI call.
To allocate and initialize an environment handle, call OCI8Lib>>ociEnvCreate:mode:.

Parameters:

    anOCI8ParentHandle (IN)
	An environment handle.

    aTypeSymbol (IN)
	Specifies the type of handle to be allocated.
	The allowed types are:
	    #'OCI_HTYPE_ERROR'                 - specifies generation of an error report handle
	    #'OCI_HTYPE_SVCCTX'                - specifies generation of a service context handle
	    #'OCI_HTYPE_STMT'                  - specifies generation of a statement (application request) handle
	    #'OCI_HTYPE_BIND'                  - specifies generation of a bind handle 
	    #'OCI_HTYPE_DEFINE'                - specifies generation of a define handle
	    #'OCI_HTYPE_DESCRIBE'              - specifies generation of a select list description handle
	    #'OCI_HTYPE_SERVER'                - specifies generation of a server context handle
	    #'OCI_HTYPE_SESSION'               - specifies generation of a user session handle
	    #'OCI_HTYPE_TRANS'                 - specifies generation of a transaction context handle
	    #'OCI_HTYPE_COMPLEXOBJECT'         - specifies generation of a complex object retrieval handle
	    #'OCI_HTYPE_SECURITY'              - specifies generation of a security handle
	    #'OCI_HTYPE_SUBSCR'                - specifies generation of a subscription handle
	    #'OCI_HTYPE_DIRPATH_CTX'           - specifies generation of a direct path context handle
	    #'OCI_HTYPE_DIRPATH_COLUMN_ARRAY'  - specifies generation of a direct path column array handle
	    #'OCI_HTYPE_DIRPATH_STREAM'        - specifies generation of a direct path stream handle
	    #'OCI_HTYPE_PROCESS'               - specifies generation of a process handle

<return: OCI8Handle>
"
    |newHandle result|

    newHandle := OCI8Handle new type:aTypeSymbol.

    result := self primOciHandleAlloc:anOCI8ParentHandle handle 
				 type:(self class numericValueForType:aTypeSymbol) 
			    newHandle:newHandle handle.
    result ~~ #'OCI_SUCCESS' ifTrue:[
	self handleError:result errorHandle:nil.
    ].
    ^ newHandle

!

ociHandleFree:anOCI8Handle
"
frees up storage associated with a handle, corresponding to the type specified in the type parameter.
All handles may be explicitly deallocated. The OCI will deallocate a child handle if the parent is deallocated.

Parameters:

    anOCI8Handle (IN)
	A handle allocated by OCI8Lib>>ociHandleAlloc:type:.

<return: self>
"

    |result|

    result := self primOciHandleFree:anOCI8Handle handle 
				type:(self class numericValueForType:anOCI8Handle type).

    result ~~ #'OCI_SUCCESS' ifTrue:[
	self handleError:result errorHandle:nil.
    ].

!

ociParamGet:anOCI8Handle position:aPositionInteger errorOCIHandle:anOCI8ErrorHandle
"
returns a descriptor of a parameter specified by position in the describe handle
or statement handle. Parameter descriptors are always allocated internally by the OCI library.
They are read-only. An OCI8Handle with address 0 may be returned if there are no parameter
descriptors for this position.

Parameters:

    anOCI8Handle (IN)
	 A statement handle or describe handle. The OCIParamGet() function will return a
	 parameter descriptor for this handle.

    aPositionInteger (IN)
	Position number in the statement handle or describe handle. A parameter descriptor
	will be returned for this position or an OCI8Handle with address 0 may be returned
	if there are no parameter descriptors for this position.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: OCI8Handle>
"

    |errorHandle newHandle resultStatus|

    errorHandle := anOCI8ErrorHandle handle.
    newHandle := OCI8Handle new type:#'OCI_DTYPE_PARAM'.

    self executePrimitive:[
	resultStatus := self primOciParamGet:anOCI8Handle handle
			     handleType:(self class numericValueForParameterType:anOCI8Handle type)
			     parameterDescriptorHandle:newHandle handle
			     position:aPositionInteger
			     errorHandle:errorHandle.
	resultStatus == #'OCI_NO_DATA' ifTrue:[
	    ^ newHandle
	].
	resultStatus == #'OCI_SUCCESS' ifTrue:[
	    ^ newHandle
	].
	resultStatus
    ] withErrorHandle:anOCI8ErrorHandle.
!

primOciAttrGetForDescribeHandle:aTargetHandle attributeType:anAttributeTypeInteger 
		    errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved. Vaild types are:

	    OCI_ATTR_PARAM       - Points to the root of the description. Used for subsequent calls
				   to OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: and 
				   OCI8Lib>>ociParamGet:position:errorOCIHandle:.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	boolean boolean;
	ub1 ub1;
	ub2 ub2;
	ub4 ub4;
	sb1 sb1;
	sb2 sb2;
	sb4 sb4;
	text *text;
	dvoid *ptr;
	double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_PARAM:
	    /*
	     * pointer cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_DESCRIBE,
				&__attr.ptr, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/p)\n", __targethndlp, __attr.ptr, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKEXTERNALADDRESS(__attr.ptr); 
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;

	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForErrorHandle:aTargetHandle attributeType:anAttributeTypeInteger 
		    errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved. Vaild types are:

	    OCI_ATTR_PARAM       - Points to the root of the description. Used for subsequent calls
				   to OCI8Lib>>ociAttrGet:attributeType:errorOCIHandle: and 
				   OCI8Lib>>ociParamGet:position:errorOCIHandle:.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	boolean boolean;
	ub1 ub1;
	ub2 ub2;
	ub4 ub4;
	sb1 sb1;
	sb2 sb2;
	sb4 sb4;
	text *text;
	dvoid *ptr;
	double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_DML_ROW_OFFSET:
	    /*
	     * ub4 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_ERROR,
				&__attr.ub4, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/p)\n", __targethndlp, __attr.ptr, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKUINT(__attr.ub4); 
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;

	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForParameterHandle:aTargetHandle attributeType:anAttributeTypeInteger
                     errorHandle:anErrorHandle resultHolder:resultArray

"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
        Pointer to a handle type.

    anAttributeTypeInteger (IN)
        The type of attribute being retrieved. Vaild types are:

            OCI_ATTR_NAME         - Returns the name of a table column.

            OCI_ATTR_IS_NULL      - Returns 1 if the table column can be a NULL value, 0 instead.

            OCI_ATTR_SCALE        - Returns the scale of a table column.

            OCI_ATTR_NUM_COLS     - Returns the number of columns of a table.

            OCI_ATTR_DATA_TYPE    - Returns the type code of a table column.

            OCI_ATTR_PRECISION    - Returns the precision of a table column.

            OCI_ATTR_DATA_SIZE    - Returns the size of a table column.

            OCI_ATTR_LIST_COLUMNS - Retruns a handle to a parameter list. The parameters handels are
                                    fetched with a call to OCI8Lib>>ociParamGet:position:errorOCIHandle:.                                        

    anErrorHandle (IN/OUT)
        An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
        for diagnostic information in the event of an error.

    resultArray (OUT)
        An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary - the oracle doc is wrong, in
     * saying that PRECISION is an ub1 (its something else)
     */
    union {
        boolean boolean;
        ub1 ub1;
        ub2 ub2;
        ub4 ub4;
        sb1 sb1;
        sb2 sb2;
        sb4 sb4;
        text *text;
        dvoid *ptr;
        double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
        if (@global(OCI8Lib:Debugging) == true){
            fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
        }
        RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

        case OCI_ATTR_NAME:
            /*
             *  text case
             */
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.text, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) into %x -> ", __targethndlp, __attr.text);
                    fwrite(__attr.text, 1, __attributesize, stderr);
                    fprintf(stderr, " (size %d)\n", __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                OBJ t;

                if(__attributesize == 0) {
                    t = __MKSTRING("");
                } else {
                    t = __MKSTRING_L(__attr.text,__attributesize);
                }
                __ArrayInstPtr(resultArray)->a_element[0] = t;
                __STORE(resultArray, t);
            }
            break;

        case OCI_ATTR_IS_NULL:
            /*
             * ub1 cases
             */
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.ub1, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/1)\n", __targethndlp, __attr.ub1, __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                __ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub1);
            }
            break;

        case OCI_ATTR_SCALE:
            /*
             * sb1 cases
             */
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.sb1, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %d (size %d/1)\n", __targethndlp, __attr.sb1, __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                __ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.sb1);
            }
            break;

        case OCI_ATTR_PRECISION:
        case OCI_ATTR_NUM_COLS:
        case OCI_ATTR_DATA_TYPE:
            /*
             * ub2 cases
             */
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.ub2, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/2)\n", __targethndlp, __attr.ub2, __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                __ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub2);
            }
            break;

        case OCI_ATTR_DATA_SIZE:
            /*
             * THIS IS UMENTED BY ORACLE TO RETURN ub4. THE SIZE RETURNED IS 4!
             * BUT ub2 is returned!!!!!
             * ub4 cases
             */
            __attr.ub4 = 0;
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.ub4, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/4)\n", __targethndlp, __attr.ub2, __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                OBJ t;
                t = __MKUINT(__attr.ub2);
                __ArrayInstPtr(resultArray)->a_element[0] = t;
                __STORE(resultArray, t);
            }
            break;

        case OCI_ATTR_LIST_COLUMNS:
            /*
             * pointer cases
             */
            __ret = OCIAttrGet( __targethndlp,
                                OCI_DTYPE_PARAM,
                                &__attr.ptr, 
                                &__attributesize,
                                __attributetype,
                                __errorhndlp);
            if (@global(OCI8Lib:Debugging) == true){
                if (__ret == OCI_SUCCESS) {
                    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/4)\n", __targethndlp, __attr.ptr, __attributesize);
                } 
            }
            if (__ret == OCI_SUCCESS) {
                OBJ t;

                t = __MKEXTERNALADDRESS(__attr.ptr); 
                __ArrayInstPtr(resultArray)->a_element[0] = t;
                __STORE(resultArray, t);
            }
            break;

        default:
            if (@global(OCI8Lib:Debugging) == true){
                fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
            }
            RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
        if (__ret != OCI_SUCCESS) {
            fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
        }
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForServerHandle:aTargetHandle attributeType:anAttributeTypeInteger 
		  errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved. Vaild types are:

	    OCI_ATTR_NONBLOCKING_MODE - This attribute determines the blocking mode. 
					When read, the attribute value returns TRUE if
					the server context is in non-blocking mode.

	    OCI_ATTR_ENV              - Returns the environment context associated with the server context.

	    OCI_ATTR_EXTERNAL_NAME    - The external name is the user-friendly global name stored in 
					sys.props$.value$ where name = GLOBAL_DB_NAME. It is not guaranteed
					to be unique unless all databases register their names with a network
					directory service. Database names can be exchanged with the server in
					case of distributed transaction coordination. Server database names can
					only be accessed if the database is open at the time the
					OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle: call
					is issued.

	    OCI_ATTR_INTERNAL_NAME    - Reads the client database name that will be recorded when performing
					global transactions. The name can be used by the DBA to track transactions
					that may be pending in a prepared state due to failures.

	    OCI_ATTR_IN_V8_MODE       - Allows you to determine whether an application has switched to Oracle release 7 mode
					A non-zero (true) return value indicates that the application is currently running in
					Oracle release 8 mode, a zero (false) return value indicates that the application is 
					currently running in Oracle release 7 mode.

	    OCI_ATTR_FOCBK            - Reads 'Application Failover Callbacks' information struct 
				       (returns external address pointing to the struct).

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	boolean boolean;
	ub1 ub1;
	ub2 ub2;
	ub4 ub4;
	sb1 sb1;
	sb2 sb2;
	sb4 sb4;
	text *text;
	dvoid *ptr;
	double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_EXTERNAL_NAME:
	case OCI_ATTR_INTERNAL_NAME:
	    /*
	     *  text case
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SERVER,
				&__attr.text, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> ", __targethndlp);
		    fwrite(__attr.text, 1, __attributesize, stderr);
		    fprintf(stderr, " (size %d)\n", __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		if(__attributesize == 0) {
		    t = __MKSTRING("");
		} else {
		    t = __MKSTRING_L(__attr.text,__attributesize);
		}
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;


	case OCI_ATTR_IN_V8_MODE:
	case OCI_ATTR_NONBLOCKING_MODE:
	    /*
	     * ub1 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SERVER,
				&__attr.ub1, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/1)\n", __targethndlp, __attr.ub1, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		__ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub1);
	    }
	    break;

	case OCI_ATTR_FOCBK:
	case OCI_ATTR_ENV:
	    /*
	     * pointer cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SERVER,
				&__attr.ptr, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/p)\n", __targethndlp, __attr.ptr, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKEXTERNALADDRESS(__attr.ptr); 
		__ArrayInstPtr(resultArray)->a_element[2] = t;
		__STORE(resultArray, t);
	    }
	    break;

	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForServiceContextHandle:aTargetHandle attributeType:anAttributeTypeInteger 
			  errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a service context handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved. Vaild types are:

	    OCI_ATTR_ENV        - Returns the environment context associated with the service context

	    OCI_ATTR_SERVER     - Returns the pointer to the server context attribute of the service context

	    OCI_ATTR_SESSION    - Returns the pointer to the authentication context attribute of the service context

	    OCI_ATTR_TRANS      - Returns the pointer to the transaction context attribute of the service context

	    OCI_ATTR_IN_V8_MODE - Allows you to determine whether an application has switched to Oracle release 7 mode
				  A non-zero (true) return value indicates that the application is currently running in
				  Oracle release 8 mode, a zero (false) return value indicates that the application is 
				  currently running in Oracle release 7 mode.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	ub1 ub1;
	dvoid *ptr;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_IN_V8_MODE:
	    /*
	     * ub1 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SVCCTX,
				&__attr.ub1, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/1)\n", __targethndlp, __attr.ub1, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		__ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub1);
	    }
	    break;

	case OCI_ATTR_ENV:
	case OCI_ATTR_SERVER:
	case OCI_ATTR_SESSION:
	case OCI_ATTR_TRANS:
	    /*
	     * pointer cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SVCCTX,
				&__attr.ptr, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/p)\n", __targethndlp, __attr.ptr, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKEXTERNALADDRESS(__attr.ptr); 
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;


	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForStatementHandle:aTargetHandle attributeType:anAttributeTypeInteger 
		     errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved. Vaild types are:

	    OCI_ATTR_NUM_DML_ERRORS - Returns the number of errors in the DML operation.

	    OCI_ATTR_ROW_COUNT      - Returns the number of rows processed so far. The default value is 1.

	    OCI_ATTR_SQLFNCODE      - Returns the function code of the SQL command associated with the statement.

		    Function codes:
		    01 CREATE TABLE                 43 DROP EXTERNAL DATABASE       85 TRUNCATE TABLE
		    02 SET ROLE                     44 CREATE DATABASE              86 TRUNCATE CLUSTER
		    03 INSERT                       45 ALTER DATABASE               87 CREATE BITMAPFILE
		    04 SELECT                       46 CREATE ROLLBACK SEGMENT      88 ALTER VIEW
		    05 UPDATE                       47 ALTER ROLLBACK SEGMENT       89 DROP BITMAPFILE
		    06 DROP ROLE                    48 DROP ROLLBACK SEGMENT        90 SET CONSTRAINTS
		    07 DROP VIEW                    49 CREATE TABLESPACE            91 CREATE FUNCTION
		    08 DROP TABLE                   50 ALTER TABLESPACE             92 ALTER FUNCTION 
		    09 DELETE                       51 DROP TABLESPACE              93 DROP FUNCTION
		    10 CREATE VIEW                  52 ALTER SESSION                94 CREATE PACKAGE 
		    11 DROP USER                    53 ALTER USER                   95 ALTER PACKAGE
		    12 CREATE ROLE                  54 COMMIT (WORK)                96 DROP PACKAGE 
		    13 CREATE SEQUENCE              55 ROLLBACK                     97 CREATE PACKAGE BODY
		    14 ALTER SEQUENCE               56 SAVEPOINT                    98 ALTER PACKAGE BODY 
		    15 (NOT USED)                   57 CREATE CONTROL FILE          99 DROP PACKAGE BODY
		    16 DROP SEQUENCE                58 ALTER TRACING                157 CREATE DIRECTORY 
		    17 CREATE SCHEMA                59 CREATE TRIGGER               158 DROP DIRECTORY
		    18 CREATE CLUSTER               60 ALTER TRIGGER                159 CREATE LIBRARY 
		    19 CREATE USER                  61 DROP TRIGGER                 160 CREATE JAVA
		    20 CREATE INDEX                 62 ANALYZE TABLE                161 ALTER JAVA 
		    21 DROP INDEX                   63 ANALYZE INDEX                162 DROP JAVA
		    22 DROP CLUSTER                 64 ANALYZE CLUSTER              163 CREATE OPERATOR 
		    23 VALIDATE INDEX               65 CREATE PROFILE               164 CREATE INDEXTYPE
		    24 CREATE PROCEDURE             66 DROP PROFILE                 165 DROP INDEXTYPE
		    25 ALTER PROCEDURE              67 ALTER PROFILE                166 ALTER INDEXTYPE 
		    26 ALTER TABLE                  68 DROP PROCEDURE               167 DROP OPERATOR 
		    27 EXPLAIN                      69 (NOT USED)                   168 ASSOCIATE STATISTICS 
		    28 GRANT                        70 ALTER RESOURCE COST          169 DISASSOCIATE STATISTICS 
		    29 REVOKE                       71 CREATE SNAPSHOT LOG          170 CALL METHOD 
		    30 CREATE SYNONYM               72 ALTER SNAPSHOT LOG           171 CREATE SUMMARY 
		    31 DROP SYNONYM                 73 DROP SNAPSHOT LOG            172 ALTER SUMMARY 
		    32 ALTER SYSTEM SWITCH LOG      74 CREATE SNAPSHOT              173 DROP SUMMARY
		    33 SET TRANSACTION              75 ALTER SNAPSHOT               174 CREATE DIMENSION 
		    34 PL/SQL EXECUTE               76 DROP SNAPSHOT                175 ALTER DIMENSION 
		    35 LOCK                         77 CREATE TYPE                  176 DROP DIMENSION 
		    36 NOOP                         78 DROP TYPE                    177 CREATE CONTEXT 
		    37 RENAME                       79 ALTER ROLE                   178 DROP CONTEXT 
		    38 COMMENT                      80 ALTER TYPE                   179 ALTER OUTLINE 
		    39 AUDIT                        81 CREATE TYPE BODY             180 CREATE OUTLINE 
		    40 NO AUDIT                     82 ALTER TYPE BODY              181 DROP OUTLINE 
		    41 ALTER INDEX                  83 DROP TYPE BODY               182 UPDATE INDEXES
		    42 CREATE EXTERNAL DATABASE     84 DROP LIBRARY                 183 ALTER OPERATOR

	    OCI_ATTR_ENV                - Returns the environment context associated with the statement.

	    OCI_ATTR_STMT_TYPE          - The type of statement associated with the handle. Possible values are:
					  OCI_STMT_SELECT, OCI_STMT_UPDATE, OCI_STMT_DELETE, OCI_STMT_INSERT, 
					  OCI_STMT_CREATE, OCI_STMT_DROP,OCI_STMT_ALTER,OCI_STMT_BEGIN (PL/SQL statement),               
					  OCI_STMT_DECLARE (PL/SQL statement)

	    OCI_ATTR_PARAM_COUNT        - This attribute can be used to get the number of columns in the
					  select-list for the statement associated with the statement handle.

	    OCI_ATTR_PARSE_ERROR_OFFSET - Returns the parse error offset for a statement.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	boolean boolean;
	ub1 ub1;
	ub2 ub2;
	ub4 ub4;
	sb1 sb1;
	sb2 sb2;
	sb4 sb4;
	text *text;
	dvoid *ptr;
	double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_NUM_DML_ERRORS:
	case OCI_ATTR_SQLFNCODE:
	case OCI_ATTR_STMT_TYPE:
	case OCI_ATTR_PARSE_ERROR_OFFSET:
	    /*
	     * ub2 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_STMT,
				&__attr.ub2, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/2)\n", __targethndlp, __attr.ub2, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		__ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub2);
	    }
	    break;

	case OCI_ATTR_ROW_COUNT:
	case OCI_ATTR_PARAM_COUNT:
	    /*
	     * ub4 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_STMT,
				&__attr.ub4, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/4)\n", __targethndlp, __attr.ub4, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKUINT(__attr.ub4);
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;

	case OCI_ATTR_ENV:
	    /*
	     * pointer cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_STMT,
				&__attr.ptr, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %x (size %d/p)\n", __targethndlp, __attr.ptr, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		OBJ t;

		t = __MKEXTERNALADDRESS(__attr.ptr); 
		__ArrayInstPtr(resultArray)->a_element[0] = t;
		__STORE(resultArray, t);
	    }
	    break;

	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrGetForUserSessionHandle:aTargetHandle attributeType:anAttributeTypeInteger 
		       errorHandle:anErrorHandle resultHolder:resultArray
"
used to get a particular attribute of a handle.

Parameters:

    aTargetHandle (IN)
	Pointer to a handle type.

    anAttributeTypeInteger (IN)
	The type of attribute being retrieved.

	    OCI_ATTR_MIGSESSION - Specifies the session identified for the session handle. Allows you
				  to clone a session from one environment to another, in the same process
				  or between processes. These processes can be on the same machine or different
				  machines. For a session to be cloned, the session must be authenticated as
				  migratable.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    /*
     * To avoid misaligned-memory violations,
     * ensure that all of the attr-cells are aligned
     * to a long boundary 
     */
    union {
	boolean boolean;
	ub1 ub1;
	ub2 ub2;
	ub4 ub4;
	sb1 sb1;
	sb2 sb2;
	sb4 sb4;
	text *text;
	dvoid *ptr;
	double *dbl;
    } __attr;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(anAttributeTypeInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __targethndlp = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

   switch (__attributetype) {

	case OCI_ATTR_MIGSESSION:
	    /*
	     * ub1 cases
	     */
	    __ret = OCIAttrGet( __targethndlp,
				OCI_HTYPE_SESSION,
				&__attr.ub1, 
				&__attributesize,
				__attributetype,
				__errorhndlp);
	    if (@global(OCI8Lib:Debugging) == true){
		if (__ret == OCI_SUCCESS) {
		    fprintf(stderr, "OCI8Lib [info]: OCIAttrGet(%x) -> %u (size %d/1)\n", __targethndlp, __attr.ub1, __attributesize);
		} 
	    }
	    if (__ret == OCI_SUCCESS) {
		__ArrayInstPtr(resultArray)->a_element[0] = __MKSMALLINT(__attr.ub1);
	    }
	    break;

	default:
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet unhandled attributeType: %d\n",__attributetype);
	    }
	    RETURN(@symbol(attributeError));
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret != OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrGet(%x) failed\n", __targethndlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciAttrSet:aTargetHandle targetType:aTargetTypeInteger attribute:anAttribute attributeType:anAttributeTypeInteger errorHandle:anErrorHandle
"
used to set a particular attribute of a handle or a descriptor.

Parameters:

    aTargetHandle (IN/OUT) 
	Pointer to a handle type whose attribute gets modified.

    aTargetTypeInteger (IN) 
	The handle type.

    anAttribute (IN)
	Pointer to an attribute value. The attribute value is copied into the target handle. 
	If the attribute value is a pointer, then only the pointer is copied, not the contents of the pointer.

    anAttributeTypeInteger (IN) 
	The type of attribute being set.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __targethndlp;
    OCIError * __errorhndlp;
    dvoid * __attributep = 0;
    ub4 __targettype;
    ub4 __attributetype;
    ub4 __attributesize = 0;
    sword __ret;
    ub4 __ub4Attr;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aTargetHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(aTargetTypeInteger) || !__isInteger(anAttributeTypeInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIAttrSet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __targethndlp   = (dvoid *) __externalAddressVal(aTargetHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __targettype    = (ub4) __unsignedLongIntVal(aTargetTypeInteger);
    __attributetype = (ub4) __unsignedLongIntVal(anAttributeTypeInteger);

    if (__isString(anAttribute)) {
	__attributesize  = __stringSize(anAttribute);
	__attributep = __stringVal(anAttribute);
    }
    else if (__isByteArray(anAttribute)) {
	__attributesize  = __byteArraySize(anAttribute);
	__attributep = __ByteArrayInstPtr(anAttribute)->ba_element;
    }
    else if (__isExternalBytes(anAttribute)) {
	__attributesize  = __intVal(__externalBytesSize(anAttribute));
	__attributep = __externalBytesAddress(anAttribute);
    }
    else if (__isExternalAddress(anAttribute)) {
	__attributesize  = 0;
	__attributep = __externalAddressVal(anAttribute);
    }
    else if (__isSmallInteger(anAttribute)) {
	__attributesize  = sizeof(ub4);
	__attributep = &__ub4Attr;
	__ub4Attr = __intVal(anAttribute);
    }
    else {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrSet attribute must be a string, byte array, external bytes, external address or small integer object\n");
	}
	RETURN(@symbol(attributeError));
    };

    __ret = OCIAttrSet( __targethndlp,
			__targettype,
			__attributep, 
			__attributesize,
			__attributetype,
			__errorhndlp);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIAttrSet(%x, %d, %x, %d, %d)\n", __targethndlp, __targettype,  __attributep, __attributetype, __attributesize);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIAttrSet(%x, %d, %x, %d, %d) failed\n", __targethndlp, __targettype,  __attributep, __attributetype, __attributesize);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciHandleAlloc:aParentHandle type:aTypeInteger newHandle:aNewHandle
"
sets a pointer to an allocated and initialized handle, corresponding to the type specified in aTypeSymbol.
A non-nil handle is returned on success. All handles are allocated with respect to an environment handle
which is passed in as a parent handle. No diagnostics are available on error. 
Handles must be allocated using this method before they can be passed into an OCI call.
To allocate and initialize an environment handle, call OCI8Lib>>ociEnvCreate:mode:.

Parameters:

    aParentHandle (IN)
	An environment handle.

    aTypeInteger (IN)
	Specifies the type of handle to be allocated.
	The allowed types are:
	    #'OCI_HTYPE_ERROR'                 - specifies generation of an error report handle
	    #'OCI_HTYPE_SVCCTX'                - specifies generation of a service context handle
	    #'OCI_HTYPE_STMT'                  - specifies generation of a statement (application request) handle
	    #'OCI_HTYPE_BIND'                  - specifies generation of a bind handle 
	    #'OCI_HTYPE_DEFINE'                - specifies generation of a define handle
	    #'OCI_HTYPE_DESCRIBE'              - specifies generation of a select list description handle
	    #'OCI_HTYPE_SERVER'                - specifies generation of a server context handle
	    #'OCI_HTYPE_SESSION'               - specifies generation of a user session handle
	    #'OCI_HTYPE_TRANS'                 - specifies generation of a transaction context handle
	    #'OCI_HTYPE_COMPLEXOBJECT'         - specifies generation of a complex object retrieval handle
	    #'OCI_HTYPE_SECURITY'              - specifies generation of a security handle
	    #'OCI_HTYPE_SUBSCR'                - specifies generation of a subscription handle
	    #'OCI_HTYPE_DIRPATH_CTX'           - specifies generation of a direct path context handle
	    #'OCI_HTYPE_DIRPATH_COLUMN_ARRAY'  - specifies generation of a direct path column array handle
	    #'OCI_HTYPE_DIRPATH_STREAM'        - specifies generation of a direct path stream handle
	    #'OCI_HTYPE_PROCESS'               - specifies generation of a process handle

    aNewHandle (OUT)
	Returns a handle.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __parenthndlp;
    dvoid * __hndlp;
    ub4 __type;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aParentHandle) || !__isExternalAddress(aNewHandle) || 
	!__isInteger(aTypeInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIHandleAlloc called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __parenthndlp = (dvoid *)__externalAddressVal(aParentHandle);
    __type = (ub4) __unsignedLongIntVal(aTypeInteger);

    __ret = OCIHandleAlloc(     __parenthndlp,
				&__hndlp,
				__type,
				(size_t) 0,
				(dvoid **) 0);

    if (__ret == OCI_SUCCESS) {
	__externalAddressVal(aNewHandle) = __hndlp;
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIHandleAlloc(%x, %d) -> %x\n", __parenthndlp, __type, __hndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIHandleAlloc(%x, %d) failed\n", __parenthndlp, __type);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciHandleFree:aHandle type:aTypeInteger
"
frees up storage associated with a handle, corresponding to the type specified in the type parameter.
All handles may be explicitly deallocated. The OCI will deallocate a child handle if the parent is deallocated.

Parameters:

    aHandle (IN)
	A handle allocated by OCI8Lib>>ociHandleAlloc:type:.

    aTypeInteger (IN)
	Specifies the type of storage to be freed. The specific types are:

	    #'OCI_HTYPE_ENV'                   - an environment handle
	    #'OCI_HTYPE_ERROR'                 - an error report handle
	    #'OCI_HTYPE_SVCCTX'                - a service context handle
	    #'OCI_HTYPE_STMT'                  - a statement (application request) handle
	    #'OCI_HTYPE_DESCRIBE'              - a select list description handle
	    #'OCI_HTYPE_SERVER'                - a server handle
	    #'OCI_HTYPE_SESSION'               - of a user session handle
	    #'OCI_HTYPE_TRANS'                 - a transaction handle
	    #'OCI_HTYPE_COMPLEXOBJECT'         - a complex object retrieval handle
	    #'OCI_HTYPE_SECURITY'              - a security handle
	    #'OCI_HTYPE_SUBSCR'                - a subscription handle
	    #'OCI_HTYPE_DIRPATH_CTX'           - a direct path context handle
	    #'OCI_HTYPE_DIRPATH_COLUMN_ARRAY'  - a direct path column array handle
	    #'OCI_HTYPE_DIRPATH_STREAM'        - a direct path stream handle
	    #'OCI_HTYPE_PROCESS'               - a process handle

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid *__handlp;
    ub4 __type;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aHandle) || !__isInteger(aTypeInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIHandleAlloc called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __handlp = __externalAddressVal(aHandle);
    __type = (ub4) __unsignedLongIntVal(aTypeInteger);

    if (__handlp == 0) {
	RETURN(@symbol(invalidHandle));
    }

    __ret = OCIHandleFree(__handlp, __type);

    if (__ret == OCI_SUCCESS) {
	__externalAddressVal(aHandle) = 0;
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIHandleFree(%x)\n", __handlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIHandleFree(%x) failed\n", __handlp);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciParamGet:aHandle handleType:aTypeInteger parameterDescriptorHandle:aParameterDescriptorHandle position:aPositionInteger errorHandle:anErrorHandle
"
sets a descriptor of a parameter specified by position in the describe handle
or statement handle. Parameter descriptors are always allocated internally by the OCI library.
They are read-only. An OCI8Handle with address 0 may be returned if there are no parameter
descriptors for this position.

Parameters:

    aHandle (IN)
	A statement handle or describe handle. The OCIParamGet() function will return a
	parameter descriptor for this handle.

    aTypeInteger (IN)
	The type of the handle passed in the handle parameter. Valid types are:

	    #'OCI_DTYPE_PARM', for a parameter descriptor 

	    #'OCI_HTYPE_COR', for a complex object retrieval handle 

	    #'OCI_HTYPE_STMT', for a statement handle

    aPositionInteger (IN)
	Position number in the statement handle or describe handle. A parameter descriptor
	will be returned for this position or an OCI8Handle with address 0 may be returned
	if there are no parameter descriptors for this position.

    aParameterDescriptorHandle parmdpp (OUT)
	A descriptor of the parameter at the position given in the pos parameter.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __hndlp;
    dvoid * __paramdhndlp   = 0;
    OCIError * __errorhndlp;
    ub4 __htype;
    ub4 __pos;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isExternalAddress(aParameterDescriptorHandle) ||
       !__isInteger(aTypeInteger) || !__isInteger(aPositionInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIParamGet called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __hndlp      = (dvoid *) __externalAddressVal(aHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __htype = (ub4) __unsignedLongIntVal(aTypeInteger);
    __pos   = (ub4) __unsignedLongIntVal(aPositionInteger);

    __ret = OCIParamGet(__hndlp,
			__htype,
			__errorhndlp, 
			& __paramdhndlp,
			__pos);

    if (__ret == OCI_SUCCESS) {
	__externalAddressVal(aParameterDescriptorHandle) = __paramdhndlp;
    }
    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIParamGet(%x, %x) -> %x\n", __hndlp, __pos ,__paramdhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIParamGet(%x, %x) failed\n", __hndlp, __pos);
	}
    }
    RETURN(__ociReturnSymbol(__ret));
%}.

! !

!OCI8Lib methodsFor:'misc functions'!

getRowOffset:anErrorHandle
    "answer the row offset (anInteger), where the
     error on a batched array DML statement occured"

    ^ self ociAttrGet:anErrorHandle attributeType:#'OCI_ATTR_DML_ROW_OFFSET'
	   errorOCIHandle:anErrorHandle
!

ociBreak:anOCI8Handle errorOCIHandle:anOCI8ErrorHandle
"
performs an immediate (asynchronous) abort of any currently executing OCI function that is
associated with a server. It is normally used to stop a long-running OCI call being processed
on the server. This call can take either the service context handle or the server context handle
as a parameter to identify the function to be aborted.

Parameters:

    anOCI8Handle (IN/OUT)
	The service context handle or the server context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    ^ self executePrimitive:[
	self primOciBreak:anOCI8Handle handle errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociReset:anOCI8Handle errorOCIHandle:anOCI8ErrorHandle
"
resets the interrupted asynchronous operation and protocol. Must be called if a 
OCI8Lib>>ociBreak:errorOCIHandle: call had been issued while a non-blocking operation
was in progress. This call is called in non-blocking mode ONLY.

Parameters:

    anOCI8Handle (IN)
	The service context handle or the server context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    [
	^ self executePrimitive:[
	    self primOciReset:anOCI8Handle handle 
		  errorHandle:errorHandle.
	] withErrorHandle:anOCI8ErrorHandle.
    ] on:OCIUserInterruptSignal do:[:ex| "ignore the exception" ].

!

primOciBreak:aHandle errorHandle:anErrorHandle
"
performs an immediate (asynchronous) abort of any currently executing OCI function that is
associated with a server. It is normally used to stop a long-running OCI call being processed
on the server. This call can take either the service context handle or the server context handle
as a parameter to identify the function to be aborted.

Parameters:

    anOCI8Handle (IN/OUT)
	The service context handle or the server context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIError * __errorhndlp;
    dvoid * __hndlp;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIBreak called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __hndlp = (dvoid *) __externalAddressVal(aHandle);

    __ret = OCIBreak(  __hndlp, __errorhndlp);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIBreak(%x)\n", __hndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIBreak(%x) failed\n", __hndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciReset:aHandle errorHandle:anErrorHandle
"
resets the interrupted asynchronous operation and protocol. Must be called if a 
OCI8Lib>>ociBreak:errorOCIHandle: call had been issued while a non-blocking operation
was in progress. This call is called in non-blocking mode ONLY.

Parameters:

    anOCI8Handle (IN)
	The service context handle or the server context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    dvoid * __hndlp = (dvoid *) __externalAddressVal(aHandle);
    OCIError * __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIReset called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __hndlp = (dvoid *) __externalAddressVal(aHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCIReset(__hndlp, __errorhndlp);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIReset(%x)\n", __hndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIReset(%x) failed\n", __hndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

! !

!OCI8Lib methodsFor:'statement functions'!

ociStatementExecute:anOCI8Statement serviceOCIHandle:anOCI8ServiceHandle errorOCIHandle:anErrorOCIHandle mode:modeSymbol
"
used to execute a prepared SQL statement. Using an execute call, the application associates a request
with a server. If a SELECT statement is executed, the description of the select-list is available implicitly
as a response. This description is buffered on the client side for describes, fetches and define type conversions.
Hence it is optimal to describe a select list only after an execute. Also for SELECT statements, some results are
available implicitly. Rows will be received and buffered at the end of the execute. For queries with small row count,
a prefetch causes memory to be released in the server if the end of fetch is reached, an optimization that may
result in memory usage reduction. Set attribute call has been defined to set the number of rows to be prefetched
per result set. For SELECT statements, at the end of the execute, the statement handle implicitly maintains a
reference to the service context on which it is executed. It is the user s responsibility to maintain the integrity
of the service context. The implicit reference is maintained until the statement handle is freed or the fetch
is cancelled or an end of fetch condition is reached.

Note: If output variables are defined for a SELECT statement before a call to OCIStmtExecute(), the number of
      rows specified by iters will be fetched directly into the defined output buffers and additional rows
      equivalent to the prefetch count will be prefetched. If there are no additional rows, then the fetch is
      complete without calling OCI8Lib>>ociStatementPiecewiseFetch:serviceOCIHandle:pieceSize:numberOfRows:errorOCIHandle:.

Parameters:

    anOCI8Statement (IN/OUT)
	An statement handle. It defines the statement and the associated data to be executed at the server.
	It is invalid to pass in a statement handle that has bind of data types only supported in release
	8 when anOCI8Statement handle points to an Oracle7 server.

    anOCI8ServiceHandle (IN/OUT)
	Service context handle.

    anOCI8ErrorHandle (IN/OUT)
       An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
       for diagnostic information in the event of an error.

<return: self>
"
    |serviceHandle errorHandle abort delay
     statementHandle iterator rowOffset mode status|

    serviceHandle := anOCI8ServiceHandle handle.
    errorHandle := anErrorOCIHandle handle.
    statementHandle := anOCI8Statement handle handle. 
    iterator := anOCI8Statement iterator.
    rowOffset := anOCI8Statement rowOffset.
    mode := self class numericValueForExecuteMode:modeSymbol.   
    abort := false.

    delay := Delay forMilliseconds:10.

    self executionLock critical:[
	anOCI8Statement setExecuting.
	[  
	    status := self primOciStatementExecute:serviceHandle
				   statementHandle:statementHandle
					  iterator:iterator
					    rowOff:rowOffset
					      mode:mode
				       errorHandle:errorHandle.
	] doWhile:[status == #'OCI_STILL_EXECUTING' 
		   and:[delay wait. 
			(abort := anOCI8Statement shouldAbort) not]
	].
            
	abort ifTrue:[
	    self ociBreak:serviceHandle errorOCIHandle:anErrorOCIHandle.
	    self ociReset:serviceHandle errorOCIHandle:anErrorOCIHandle.
	    anOCI8Statement setAborted.
	    ^ #aborted
	].
	anOCI8Statement setExecuted.
    ].
    ^ status
!

ociStatementFetch:anOCI8Statement serviceOCIHandle:anOCI8ServiceHandle numberOfRows:aNumberOfRowsInteger errorOCIHandle:anErrorOCIHandle
"
fetches rows from a query. The fetch call is a local call, if prefetched rows suffice. However,
this is transparent to the application. If LOB columns are being read, LOB locators are fetched
for subsequent LOB operations to be performed on these locators. Prefetching is turned off if
LONG columns are involved. If you call this method with the aNumberOfRowsInteger parameter set to zero,
this cancels the cursor.

Parameters:

    anOCI8Statement (IN)
	A statement (application request) handle.

    anOCI8ServiceHandle (IN/OUT)
	Service context handle.

    aNumberOfRowsInteger (IN)
	Number of rows to be fetched from the current position.

    anOCI8ErrorHandle (IN/OUT)
       An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
       for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle statementHandle status abort delay|

    errorHandle := anErrorOCIHandle handle.
    statementHandle := anOCI8Statement handle handle.
    abort := false.

    delay := Delay forMilliseconds:10.

    self executionLock critical:[
	[  
	    status := self primOciStatementFetch:statementHandle
				    numberOfRows:aNumberOfRowsInteger
				     errorHandle:errorHandle.
	] doWhile:[status == #'OCI_STILL_EXECUTING' 
		   and:[delay wait. 
			(abort := anOCI8Statement shouldAbort) not]
	].

	abort ifTrue:[
	    anOCI8Statement setAborted.
	    ^ self
	].
	status ~~ #'OCI_SUCCESS' ifTrue:[
	    self handleError:status errorHandle:anErrorOCIHandle.
	].
    ].
!

ociStatementPiecewiseFetch:anOCI8Statement serviceOCIHandle:anOCI8ServiceHandle pieceSize:aBufferSize numberOfRows:aNumberOfRowsInteger errorOCIHandle:anErrorOCIHandle
"
picewise fetches rows from a query. The fetch call is a local call, if prefetched rows suffice. However,
this is transparent to the application. Only some datatypes can be manipulated in pieces. Piecewise fetches
can only performed with  of the following data types: VARCHAR2, STRING, LONG, LONG RAW.
Prefetching is turned off if LONG columns are involved. If you call this method with the aNumberOfRowsInteger
parameter set to zero, this cancels the cursor.

Parameters:

    anOCI8Statement (IN)
        A statement (application request) handle.

    anOCI8ServiceHandle (IN/OUT)
        Service context handle.

    aBufferSize (IN)
        The buffer length for the pieces.

    aNumberOfRowsInteger (IN)
        Number of rows to be fetched from the current position.

    anOCI8ErrorHandle (IN/OUT)
       An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
       for diagnostic information in the event of an error.

<return: a Dictionary with (handle->stream)>
"
    |statementHandle errorHandle resultHolder result status abort delay resultHandle currentHandle currentStream|

    statementHandle := anOCI8Statement handle handle.
    errorHandle := anErrorOCIHandle handle.
    resultHolder := Array new:2.

    delay := Delay forMilliseconds:10.
    abort := false.

    self executionLock critical:[
        anOCI8Statement setFetching.
        "first call to statement fetch must return need data signal."
        "if not reject exception"
        [  
            status := self primOciStatementFetch:statementHandle
                                    numberOfRows:aNumberOfRowsInteger
                                     errorHandle:errorHandle.
        ] doWhile:[status == #'OCI_STILL_EXECUTING' 
                   and:[delay wait.(abort := anOCI8Statement shouldAbort) not]
        ].

        result := Dictionary new.

        abort ifTrue:[
            anOCI8Statement setAborted.
            ^ result
        ].
        (status ~~ #'OCI_SUCCESS' and:[status ~~ #'OCI_NEED_DATA']) ifTrue:[
            self handleError:status errorHandle:anErrorOCIHandle.
        ].


        [  
            [
                [
                    status := self primOciStatementPiecewiseFetch:statementHandle
                                                       bufferSize:aBufferSize 
                                                     numberOfRows:aNumberOfRowsInteger 
                                                      errorHandle:errorHandle
                                                     resultHolder:resultHolder.
                ] doWhile:[status == #'OCI_STILL_EXECUTING' 
                           and:[delay wait.(abort := anOCI8Statement shouldAbort) not]
                ].
            ] ifCurtailed:[
                "reset the oci lib only if the statement was executed once 
                 and hasn't finished correctly"
                (status notNil and:[status ~~ #'OCI_SUCCESS']) ifTrue:[
                    anOCI8Statement setAborted.
                ].
            ].
            (status == #'OCI_SUCCESS' or:[status == #'OCI_NEED_DATA']) ifTrue:[
                resultHandle := resultHolder at:2.
                resultHandle ~= currentHandle ifTrue:[
                    currentHandle := resultHandle.
                    currentStream := #[] writeStream.
                    currentHandle notNil ifTrue:[
                        result at:currentHandle put:currentStream.
                    ].
                ].
                currentStream nextPutAll:(resultHolder at:1).
            ].
        ] doWhile:[
            status == #'OCI_NEED_DATA' and:[(abort := anOCI8Statement shouldAbort) not]
        ].

        abort ifTrue:[
            anOCI8Statement setAborted.
            ^ Dictionary new.
        ].
        status ~~ #'OCI_SUCCESS' ifTrue:[
            self handleError:status errorHandle:anErrorOCIHandle.
        ].

    ].

    ^ result
!

ociStatementPrepare:anOCI8StatementHandle textBuffer:aStatementTextBuffer language:aLanguageSymbol mode:aModeSymbol errorOCIHandle:anOCI8ErrorHandle
"
prepares a SQL or PL/SQL statement for execution. This is a purely local call. Data values for this
statement initialized in subsequent bind calls will be stored in a bind handle which will hang off
this statement handle. This call does not create an association between this statement handle and
any particular server.

Parameters:

    anOCI8StatementHandle (IN)
	A statement handle.

    aStatementTextBuffer (IN)
	A pointer to a SQL or PL/SQL statement text to be executed. Must be a null-terminated 
	string. The pointer to the text of the statement must be available as long as the
	statement is executed, or data is fetched from it.

    aLanguageSymbol (IN)
	Specifies V7, or native syntax. Possible values are: 
	    OCI_V7_SYNTAX  - V7 ORACLE parsing syntax 
	    OCI_NTV_SYNTAX - syntax depends upon the version of the server.

    aModeSymbol (IN)
	The possible values are:

	    OCI_DEFAULT    - default mode 

	    OCI_NO_SHARING - disables sharing mode for the SQL statement.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciStatementPrepare:anOCI8StatementHandle handle
			  textBuffer:aStatementTextBuffer
			    language:(self class numericValueForLanguageMode:aLanguageSymbol)
				mode:(self class numericValueForPrepareMode:aModeSymbol)
			 errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle

!

ociStmtGetBindInfo:anOCI8StatementHandle errorOCIHandle:anOCI8ErrorHandle
"
returns information about bind variables after a statement has been prepared. This includes bind names,
indicator names, and whether or not binds are duplicate binds. This call also returns an associated
bind handle if there is one. This function does not include SELECT INTO list variables, because they are
not considered to be binds. The statement must have been prepared with a call to 
OCI8Lib>>ociStatementPrepare:statement:language:mode:errorOCIHandle: prior to this call.

Parameters:

    anOCI8StatementHandle (IN)
	The statement handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: Array of:OCI8BindDescriptor>
"
    |errorHandle returnStatus resultHolder descriptor index|

    errorHandle := anOCI8ErrorHandle handle.
    resultHolder := Array new:1.

    self executionLock critical:[
	"call primitve function to get info about the bind variables of the statement"
	"if there are no bind variables a no data signal is raised and catched and"
	"an empty array is returned else an array with bind desciptors is returned"
	returnStatus := self primOciStmtGetBindInfo:anOCI8StatementHandle handle
					errorHandle:errorHandle
				       resultHolder:resultHolder.
	returnStatus == #'OCI_NO_DATA' ifTrue:[
	    ^ #().
	].
	returnStatus ~~ #'OCI_SUCCESS' ifTrue:[
	    self handleError:returnStatus errorHandle:anOCI8ErrorHandle.
	].
    ].

    index := 1.
    ^ (resultHolder at:1) collect:[:eachElement |
	descriptor := OCI8BindDescriptor new.
	descriptor position:index.
	descriptor name:(eachElement at:1) asSymbol.
	descriptor indicator:(eachElement at:2).
	descriptor duplicate:(eachElement at:3).
	descriptor handle:(eachElement at:4).
	index := index + 1.
	descriptor
    ].
!

primOciStatementExecute:aServiceContextHandle statementHandle:aStatementHandle iterator:anIteratorInteger rowOff:aRowOffInteger mode:aModeInteger errorHandle:anErrorHandle
"
used to execute a prepared SQL statement. Using an execute call, the application associates a request
with a server. If a SELECT statement is executed, the description of the select-list is available implicitly
as a response. This description is buffered on the client side for describes, fetches and define type conversions.
Hence it is optimal to describe a select list only after an execute. Also for SELECT statements, some results are
available implicitly. Rows will be received and buffered at the end of the execute. For queries with small row count,
a prefetch causes memory to be released in the server if the end of fetch is reached, an optimization that may
result in memory usage reduction. Set attribute call has been defined to set the number of rows to be prefetched
per result set. For SELECT statements, at the end of the execute, the statement handle implicitly maintains a
reference to the service context on which it is executed. It is the users responsibility to maintain the integrity
of the service context. The implicit reference is maintained until the statement handle is freed or the fetch
is cancelled or an end of fetch condition is reached.

Note: If output variables are defined for a SELECT statement before a call to OCIStmtExecute(), the number of
      rows specified by iters will be fetched directly into the defined output buffers and additional rows
      equivalent to the prefetch count will be prefetched. If there are no additional rows, then the fetch is
      complete without calling OCI8Lib>>ociStatementPiecewiseFetch:serviceOCIHandle:pieceSize:numberOfRows:errorOCIHandle:.

Parameters:

    anOCI8ServiceHandle (IN/OUT)
	Service context handle.

    anOCI8Statement (IN/OUT)
	An statement handle. It defines the statement and the associated data to be executed at the server.
	It is invalid to pass in a statement handle that has bind of data types only supported in release
	8 when anOCI8Statement handle points to an Oracle7 server.

    anIteratorInteger (IN)
	For non-SELECT statements, the number of times this statement is executed. For SELECT statements,
	if iters is non-zero, then defines must have been done for the statement handle. The execution fetches
	iters rows into these predefined buffers and prefetches more rows depending upon the prefetch row count.
	If you do not know how many rows the SELECT statement will retrieve, set iters to zero.
	This function returns an error if iters=0 for non-SELECT statements.

    aRowOffInteger (IN)
	The starting index from which the data in an array bind is relevant for this multiple row execution.

    aModeInteger (IN)
	The modes are:

	    OCI_DEFAULT           - Calling OCIStmtExecute() in this mode executes the statement. It also implicitly
				    returns describe information about the select-list.

	    OCI_DESCRIBE_ONLY     - This mode is for users who wish to describe a query prior to execution.
				    Calling OCIStmtExecute() in this mode does not execute the statement, but it
				    does return the select-list description. To maximize performance, it is
				    recommended that applications execute the statement in default mode and use 
				    the implicit describe which accompanies the execution.

	    OCI_COMMIT_ON_SUCCESS - When a statement is executed in this mode, the current transaction is committed
				    after execution, provided that execution completes successfully.

	   OCI_EXACT_FETCH        - Used when the application knows in advance exactly how many rows it
				    will be fetching. This mode turns prefetching off for Oracle release 8 mode,
				    and requires that defines be done before the execute call. Using this mode
				    cancels the cursor after the desired rows are fetched and may result in reduced
				    server-side resource usage.

	   OCI_BATCH_ERRORS       - In this mode, the OCI attempts to INSERT, UPDATE, or DELETE all rows, and collects
				    (batches) information about any errors which occurred. The application can then
				    retrieve this error information and re-execute any DML operations which failed
				    during the first call.

	The modes are not mutually exclusive and can be used together.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIStmt * __stmthndlp;
    OCIError * __errorhndlp;
    ub4 __iters;
    ub4 __rowOff;
    ub4 __mode;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceContextHandle) || !__isExternalAddress(aStatementHandle) ||
       !__isExternalAddress(anErrorHandle) || !__isInteger(anIteratorInteger) ||
       !__isInteger(aRowOffInteger) || !__isInteger(aModeInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIStmtExecute called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceContextHandle);
    __stmthndlp   = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __iters  = __unsignedLongIntVal(anIteratorInteger);
    __rowOff = __unsignedLongIntVal(aRowOffInteger);
    __mode   = __unsignedLongIntVal(aModeInteger);

    __ret = OCIStmtExecute(     __svcxtxhndlp,
				__stmthndlp,
				__errorhndlp,
				__iters,
				__rowOff,
				0/*__snap_in*/,
				0/*__snap_out*/,
				__mode);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtExecute(%x, %x, %d, %d, %x)\n", __svcxtxhndlp, __stmthndlp, __iters, __rowOff, __mode);
	} else if (__ret == OCI_STILL_EXECUTING){
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtExecute(%x, %x, %d, %d, %x) still executing\n", __svcxtxhndlp, __stmthndlp, __iters, __rowOff, __mode);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIStmtExecute(%x, %x, %d, %d, %x) failed\n", __svcxtxhndlp, __stmthndlp, __iters, __rowOff, __mode);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciStatementFetch:aStatementHandle numberOfRows:aNumberOfRowsInteger errorHandle:anErrorHandle
"
fetches rows from a query. The fetch call is a local call, if prefetched rows suffice. However,
this is transparent to the application. If LOB columns are being read, LOB locators are fetched
for subsequent LOB operations to be performed on these locators. Prefetching is turned off if
LONG columns are involved. If you call this method with the aNumberOfRowsInteger parameter set to zero,
this cancels the cursor.

Parameters:

    aStatementHandle (IN)
	A statement (application request) handle.

    aNumberOfRowsInteger (IN)
	Number of rows to be fetched from the current position.

    anErrorHandle (IN/OUT)
       An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
       for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmthndlp;
    OCIError * __errorhndlp;
    ub4 __nrows;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aStatementHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(aNumberOfRowsInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIStmtFetch called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __stmthndlp   = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __nrows = __unsignedLongIntVal(aNumberOfRowsInteger);

    __ret = OCIStmtFetch(       __stmthndlp,
				__errorhndlp,
				__nrows,
				OCI_FETCH_NEXT,
				OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtFetch(%x, %d)\n", __stmthndlp, __nrows);
	} else if (__ret == OCI_STILL_EXECUTING) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtFetch(%x, %d) still executing\n", __stmthndlp, __nrows);
	} else if (__ret == OCI_NEED_DATA) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtFetch(%x, %d) need data\n", __stmthndlp, __nrows);
	} else if (__ret == OCI_NO_DATA) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtFetch(%x, %d) no data\n", __stmthndlp, __nrows);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIStmtFetch(%x, %d) failed\n", __stmthndlp, __nrows);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciStatementPiecewiseFetch:aStatementHandle bufferSize:aBufferSize numberOfRows:aNumberOfRowsInteger 
                   errorHandle:anErrorHandle resultHolder:resultArray
"
picewise fetches rows from a query. The fetch call is a local call, if prefetched rows suffice. However,
this is transparent to the application. Only some datatypes can be manipulated in pieces. Piecewise fetches
can only performed with  of the following data types: VARCHAR2, STRING, LONG, LONG RAW.
Prefetching is turned off if LONG columns are involved. If you call this method with the aNumberOfRowsInteger
parameter set to zero, this cancels the cursor.

Parameters:

    aStatementHandle (IN)
        A statement (application request) handle.

    aBufferSize (IN)
        The buffer length for the pieces.

    aNumberOfRowsInteger (IN)
        Number of rows to be fetched from the current position.

    anErrorHandle (IN/OUT)
        An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
        for diagnostic information in the event of an error.

    resultArray (OUT)
        An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmtp;
    OCIError * __errorp;
    dvoid * __hndlp = 0;
    ub4 __type = 0;
    ub1 __in_out = 0;
    ub4 __iter = 0;
    ub4 __index = 0;
    ub1 __piece = OCI_FIRST_PIECE;
    ub2 __rcodep = 0;
    sb2 __indp = 0;
    ub4 __buf_size;
    ub4 __tmp_buf_size;
    dvoid * __buf;
    ub4 __nrows;
    sword __ret;
    OBJ t;
    OBJ __returnCode;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aStatementHandle) || !__isExternalAddress(anErrorHandle) ||
       !__isInteger(aBufferSize) || !__isInteger(aNumberOfRowsInteger) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 2
    ) {
        if (@global(OCI8Lib:Debugging) == true){
            fprintf(stderr, "OCI8Lib [error]: OCIPiecewiseFetch called with wrong attributes.\n");
        }
        RETURN(@symbol(attributeError));
    }

    __stmtp    = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorp   = (OCIError *) __externalAddressVal(anErrorHandle);
    __buf_size = __unsignedLongIntVal(aBufferSize);
    __nrows    = __unsignedLongIntVal(aNumberOfRowsInteger);

    /* allocate memory */
    __buf = (dvoid *) malloc(__buf_size);
    if (!__buf) {
        if (@global(OCI8Lib:Debugging) == true){
           fprintf(stderr, "OCI8Lib [warning]: OCIStmtPiecewiceFetch(%x) unable to alloc %d bytes", __stmtp, __buf_size);
        }
        RETURN(@symbol(mallocError));
    }

    __ret = OCIStmtGetPieceInfo(        __stmtp,
                                        __errorp,
                                        &__hndlp,
                                        &__type,
                                        &__in_out,
                                        &__iter,
                                        &__index,
                                        &__piece);

    if (@global(OCI8Lib:Debugging) == true){
        if (__ret == OCI_SUCCESS) {
           fprintf(stderr, "OCI8Lib [info]: OCIStmtGetPieceInfo(%x) -> %x, %x, %x, %x, %x, %x\n", __stmtp, __hndlp, __type, __in_out, __iter, __index, __piece);
        } else {
           fprintf(stderr, "OCI8Lib [warning]: OCIStmtGetPieceInfo(%x) failed\n", __stmtp);
        }
    };

    if (__ret != OCI_SUCCESS) {
        free(__buf);
        RETURN(__ociReturnSymbol(__ret));
    }

    __tmp_buf_size = __buf_size;

    __ret = OCIStmtSetPieceInfo(        __hndlp,
                                        __type,
                                        __errorp,
                                        __buf,
                                        &__tmp_buf_size,
                                        __piece,
                                        &__indp,
                                        &__rcodep);

    if (@global(OCI8Lib:Debugging) == true){
        if (__ret == OCI_SUCCESS) {
           fprintf(stderr, "OCI8Lib [info]: OCIStmtSetPieceInfo(%x , %x, %x, %d)\n", __hndlp, __type, __piece, __tmp_buf_size);
        } else {
           fprintf(stderr, "OCI8Lib [warning]: OCIStmtSetPieceInfo(%x , %x, %x, %d) failed\n", __hndlp, __type, __piece, __tmp_buf_size);
        }
    };

    if (__ret != OCI_SUCCESS) {
        free(__buf);
        RETURN(__ociReturnSymbol(__ret));
    }

    __ret = OCIStmtFetch(       __stmtp,
                                __errorp,
                                __nrows,
                                OCI_FETCH_NEXT,
                                OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
        if (__ret == OCI_SUCCESS) {
            fprintf(stderr, "OCI8Lib [info]: OCIStmtPiecewiseFetch(%x, %d, %d)\n", __stmtp, __nrows, __tmp_buf_size);
        } else if (__ret == OCI_NEED_DATA) {
            fprintf(stderr, "OCI8Lib [info]: OCIStmtPiecewiseFetch(%x, %d, %d) need data\n", __stmtp, __nrows, __tmp_buf_size);
        } else if (__ret == OCI_NO_DATA) {
            fprintf(stderr, "OCI8Lib [info]: OCIStmtPiecewiseFetch(%x, %d, %d) no data\n", __stmtp, __nrows, __tmp_buf_size);
        } else {
            fprintf(stderr, "OCI8Lib [warning]: OCIStmtPiecewiseFetch(%x, %d, %d) -> %d failed\n", __stmtp, __nrows, __tmp_buf_size, __ret);
        }
    };

    /* make from fetched piece a byte array */
    if ((__ret == OCI_SUCCESS) || (__ret == OCI_NEED_DATA)) {
        t = __MKBYTEARRAY(__buf, __tmp_buf_size);
        __ArrayInstPtr(resultArray)->a_element[0] = t;
        __STORE(resultArray, t);
        t = __MKEXTERNALADDRESS(__hndlp);
        __ArrayInstPtr(resultArray)->a_element[1] = t;
        __STORE(resultArray, t);
    }

    free(__buf);
    RETURN(__ociReturnSymbol(__ret));
%}.
!

primOciStatementPrepare:aStatementHandle textBuffer:aStatementTextBuffer language:aLanguageInteger mode:aModeInteger errorHandle:anErrorHandle
"
prepares a SQL or PL/SQL statement for execution. This is a purely local call. Data values for this
statement initialized in subsequent bind calls will be stored in a bind handle which will hang off
this statement handle. This call does not create an association between this statement handle and
any particular server.

Parameters:

    aStatementHandle (IN)
	A statement handle.

    aStatementTextBuffer (IN)
	A pointer to a SQL or PL/SQL statement text to be executed. Must be a null-terminated 
	string. The pointer to the text of the statement must be available as long as the
	statement is executed, or data is fetched from it.

    aLanguageSymbol (IN)
	Specifies V7, or native syntax. Possible values are: 
	    OCI_V7_SYNTAX  - V7 ORACLE parsing syntax 
	    OCI_NTV_SYNTAX - syntax depends upon the version of the server.

    aModeInteger (IN)
	The possible values are:

	    OCI_DEFAULT    - default mode 

	    OCI_NO_SHARING - disables sharing mode for the SQL statement.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmthndlp;
    OCIError * __errorhndlp;
    ub4 __language;
    ub4 __mode;
    char * __stringp;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aStatementHandle) || !__isExternalAddress(anErrorHandle) ||
	!__isExternalBytes(aStatementTextBuffer) || !__isInteger(aModeInteger) ||
	!__isInteger(aLanguageInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIStmtPrepare called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __stmthndlp  = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorhndlp = (OCIError *) __externalAddressVal(anErrorHandle);
    __language = __unsignedLongIntVal(aLanguageInteger);
    __mode     = __unsignedLongIntVal(aModeInteger);
    __stringp  = (char *)__externalBytesAddress(aStatementTextBuffer);


    __ret = OCIStmtPrepare(     __stmthndlp,
				__errorhndlp,
				__stringp,
				strlen(__stringp),
				__language,
				__mode);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtPrepare(%x, %x, %x) = %s\n", __stmthndlp, __language, __mode, __stringp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIStmtPrepare(%x, %x, %x) = %s failed\n", __stmthndlp, __language, __mode, __stringp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciStmtGetBindInfo:aStatementHandle errorHandle:anErrorHandle resultHolder:resultArray
"
returns information about bind variables after a statement has been prepared. This includes bind names,
indicator names, and whether or not binds are duplicate binds. This call also returns an associated
bind handle if there is one. This function does not include SELECT INTO list variables, because they are
not considered to be binds. The statement must have been prepared with a call to 
OCI8Lib>>ociStatementPrepare:statement:language:mode:errorOCIHandle: prior to this call.
This call is processed locally.

Parameters:

    anOCI8StatementHandle (IN)
	The statement handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

    resultArray (OUT)
	An array of size > 0. The result of the operation is stored in the first element.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"
    |valueArray bindInfo|

%{  /* UNLIMITEDSTACK */

    OCIStmt * __stmtp;
    OCIError * __errorp;
    sb4 __found = -1;
    ub4 __startloc;
    ub4 __numOfExistingBindVars;

#define CHUNK 2
    text *__bvnp[CHUNK];    
    ub1   __bvnl[CHUNK];
    text *__invp[CHUNK];
    ub1   __inpl[CHUNK];
    ub1   __dupl[CHUNK];
    OCIBind *__hndl[CHUNK];
    sword __ret;
    int i, e;


    /* check if all attributes have correct type */
    if(!__isExternalAddress(aStatementHandle) || 
       !__isExternalAddress(anErrorHandle) ||
       !__isArray(resultArray) || __arraySize(resultArray) < 1
    ) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIStmtGetBindInfo called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __stmtp  = (OCIStmt *) __externalAddressVal(aStatementHandle);
    __errorp = (OCIError *) __externalAddressVal(anErrorHandle);

    for (__startloc = 1; __found < 0; __startloc += CHUNK) {
	__ret = OCIStmtGetBindInfo (    __stmtp,
					__errorp,
					CHUNK,
					__startloc,
					&__found,
					__bvnp,    
					__bvnl,
					__invp,
					__inpl,
					__dupl,
					__hndl
				  );

	if (__ret != OCI_SUCCESS) {
	    if (@global(OCI8Lib:Debugging) == true){
		fprintf(stderr, "OCI8Lib [warning]: OCIStmtGetBindInfo(%x) failed\n", __stmtp);
	    }
	    RETURN(__ociReturnSymbol(__ret));
	};

	if (valueArray == nil) {
	    /* calc who many bind vars exists */
	    __numOfExistingBindVars = __found < 0 ? - __found : __found;

	    /* allocate return-value aray */
	    valueArray = __ARRAY_NEW_INT(__numOfExistingBindVars);
	    __ArrayInstPtr(resultArray)->a_element[0] = valueArray;
	    __STORE(resultArray, valueArray);
	}
	if (__startloc-1+CHUNK < __numOfExistingBindVars)
	    e = CHUNK;
	else
	    e = __numOfExistingBindVars - (__startloc-1);

	for (i = 0; i < e; i++) {
	    OBJ t;

	    __ArrayInstPtr(valueArray)->a_element[__startloc-1+i] 
						= bindInfo = __ARRAY_NEW_INT(4);
	    __STORE(valueArray, bindInfo);

	    __ArrayInstPtr(bindInfo)->a_element[0] = t = __MKSTRING_L(__bvnp[i],__bvnl[i]);
	    __STORE(bindInfo, t);    

	    __ArrayInstPtr(bindInfo)->a_element[1] = t = __MKSTRING_L(__invp[i],__inpl[i]);
	    __STORE(bindInfo, t);    

	    __ArrayInstPtr(bindInfo)->a_element[2] = __dupl[i] ? true : false;

	    if (__hndl[i] != 0) {
		__ArrayInstPtr(bindInfo)->a_element[3] = t = __MKEXTERNALADDRESS(__hndl[i]);
		__STORE(bindInfo, t);    
	    }
	}
    }

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtGetBindInfo(%x)\n", __stmtp);
	} else if (__ret == OCI_NO_DATA) {
	    fprintf(stderr, "OCI8Lib [info]: OCIStmtGetBindInfo(%x) NO DATA\n", __stmtp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCIStmtGetBindInfo(%x) failed\n", __stmtp);
	}
    };

    RETURN(__ociReturnSymbol(__ret));
%}.
! !

!OCI8Lib methodsFor:'transaction functions'!

ociTransactionCommit:anOCI8ServiceHandle flag:aTransactionCommitType errorOCIHandle:anOCI8ErrorHandle
"
commits the transaction which is currently associated with the service context. If it is a global transaction
that the server cannot commit, this call additionally retrieves the state of the transaction from
the database to be returned to the user in the error handle.
If the application has defined multiple transactions, this function operates on the transaction currently
associated with the service context. If the application is working with only the implicit local transaction
created when database changes are made, that implicit transaction is committed.
If the application is running in the object mode, then the modified or updated objects in the object cache
for this transaction are also flushed and committed.
Under normal circumstances, this method returns with a status indicating that the transaction has either
been committed or rolled back. With global transactions, it is possible that the transaction is now in-doubt,
meaning that it is neither committed nor aborted. In this case, this method attempts to retrieve the status
of the transaction from the server. The status is returned.

Parameters:

    anOCI8ServiceHandle (IN)
	The service context handle. 

    aTransactionCommitType (IN)
	A flag used for one-phase commit optimization in global transactions.
	If the transaction is non-distributed, the flags parameter is ignored,
	and #'OCI_ DEFAULT' can be passed as its value. OCI applications managing global
	transactions should pass a value of #'OCI_TRANS_TWOPHASE' to the flags parameter
	for a two-phase commit. The default is one-phase commit.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionCommit:anOCI8ServiceHandle handle 
				 flag:(self class numericValueForTransactionCommitType:aTransactionCommitType)
			  errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociTransactionDetach:anOCI8ServiceHandle errorOCIHandle:anOCI8ErrorHandle
"
detaches a global transaction from the service context handle. The transaction currently attached
to the service context handle becomes inactive at the end of this call. The transaction may be resumed
later by calling OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle:, specifying a flags value of #'OCI_TRANS_RESUME'.
When a transaction is detached, the value which was specified in the timeout parameter of 
OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle: when the transaction was started is used to determine
the amount of time the branch can remain inactive before being deleted by the server s PMON process.

    Note: The transaction can be resumed by a different process than the one that detached it,
	  provided that the transaction has the same authorization. If this function is called
	  before a transaction is actually started, this function is a no-op.

Parameters:

    anOCI8ServiceHandle (IN)
	The service context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionDetach:anOCI8ServiceHandle handle errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociTransactionForget:anOCI8ServiceHandle errorOCIHandle:anOCI8ErrorHandle
"
forgets a heuristically completed global transaction. The server deletes the status of the transaction
from the system's pending transaction table.
You set the XID of the transaction to be forgotten as an attribute of the transaction handle (#'OCI_ATTR_XID').

Parameters:

    anOCI8ServiceHandle (IN)
	The service context handle in which the transaction resides.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionForget:anOCI8ServiceHandle handle errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociTransactionPrepare:anOCI8ServiceHandle errorOCIHandle:anOCI8ErrorHandle
"
prepares the specified global transaction for commit. This call is valid only for global transactions.
The call raises an OCISuccessWithInfoSignal-Signal if the transaction has not made any changes.
The error handle will indicate that the transaction is read-only.

Parameters:

    anOCI8ServiceHandle (IN)
	The service context handle.

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionPrepare:anOCI8ServiceHandle handle errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociTransactionRollback:anOCI8ServiceHandle errorOCIHandle:anOCI8ErrorHandle
"
rolls back the current transaction which is defined as the set of statements executed since the last
OCI8Lib>>ociTransactionCommit:flag:errorOCIHandle: or since OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.
If the application is running under object mode then the modified or updated objects in the object cache
for this transaction are also rolled back. Attempting to roll back a global transaction that is not
currently active causes an error.

Parameters:

    anOCI8ServiceHandle (IN)
	A service context handle. The transaction currently set in the service context handle is rolled back. 

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionRollback:anOCI8ServiceHandle handle errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

ociTransactionStart:anOCI8ServiceHandle time:aTimeOutInteger flags:aTransactionTypeColl errorOCIHandle:anOCI8ErrorHandle
"
sets the beginning of a global or serializable transaction. The transaction context currently
associated with the service context handle is initialized at the end of the call if the flags
parameter specifies that a new transaction should be started.

The XID of the transaction is set as an attribute of the transaction handle (#'OCI_ ATTR_XID').

Parameters:

    anOCI8ServiceHandle (IN/OUT)
	The service context handle. The transaction context in the service context handle is initialized
	at the end of the call if the flag specified a new transaction to be started.

    aTimeOutInteger (IN)
	The time, in seconds, to wait for a transaction to become available for resumption
	when #'OCI_TRANS_RESUME' is specified. When #'OCI_TRANS_NEW' is specified, the timeout
	parameter indicates the number of seconds the transaction can be inactive before it
	is automatically aborted by the system. A transaction is inactive between the time
	it is detached with OCI8Lib>>ociTransactionDetach:errorOCIHandle: and the time it is resumed
	with OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle: (with mode = #'OCI_TRANS_RESUME').

    aTransactionTypeColl (IN)
	Specifies whether a new transaction is being started or an existing transaction is
	being resumed. Also specifies serializiability or read-only status. More than a
	single value can be specified. By default, a read/write transaction is started.
	The flag values are:

	    #'OCI_TRANS_NEW'          - starts a new transaction branch. By default starts a tightly coupled 
					and migratable branch.
	    #'OCI_TRANS_TIGHT'        - explicitly specifies a tightly coupled branch
	    #'OCI_TRANS_LOOSE'        - specifies a loosely coupled branch
	    #'OCI_TRANS_RESUME'       - resumes an existing transaction branch.
	    #'OCI_TRANS_READONLY'     - start a read-only transaction
	    #'OCI_TRANS_SERIALIZABLE' - start a serializable transaction

    anOCI8ErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

<return: self>
"
    |errorHandle|

    errorHandle := anOCI8ErrorHandle handle.

    self executePrimitive:[
	self primOciTransactionStart:anOCI8ServiceHandle handle
				time:aTimeOutInteger 
			       flags:(self class numericValueForTransactionTypeColl:aTransactionTypeColl) 
			 errorHandle:errorHandle.
    ] withErrorHandle:anOCI8ErrorHandle.
!

primOciTransactionCommit:aServiceHandle flag:aFlagInteger errorHandle:anErrorHandle
"
commits the transaction which is currently associated with the service context. If it is a global transaction
that the server cannot commit, this call additionally retrieves the state of the transaction from
the database to be returned to the user in the error handle.
If the application has defined multiple transactions, this function operates on the transaction currently
associated with the service context. If the application is working with only the implicit local transaction
created when database changes are made, that implicit transaction is committed.
If the application is running in the object mode, then the modified or updated objects in the object cache
for this transaction are also flushed and committed.
Under normal circumstances, this method returns with a status indicating that the transaction has either
been committed or rolled back. With global transactions, it is possible that the transaction is now in-doubt,
meaning that it is neither committed nor aborted. In this case, this method attempts to retrieve the status
of the transaction from the server. The status is returned.

Parameters

    aServiceHandle (IN)
	The service context handle. 

    aFlagInteger (IN)
	A flag used for one-phase commit optimization in global transactions.
	If the transaction is non-distributed, the flags parameter is ignored,
	and #'OCI_ DEFAULT' can be passed as its value. OCI applications managing global
	transactions should pass a value of #'OCI_TRANS_TWOPHASE' to the flags parameter
	for a two-phase commit. The default is one-phase commit.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    ub4 __flags;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle) || !__isInteger(aFlagInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerAttach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __flags = __unsignedLongIntVal(aFlagInteger);

    __ret = OCITransCommit(     __svcxtxhndlp,
				__errorhndlp,
				__flags);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransCommit(%x, %d)\n", __svcxtxhndlp, __flags);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransCommit(%x, %d) failed\n", __svcxtxhndlp, __flags);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciTransactionDetach:aServiceHandle errorHandle:anErrorHandle
"
detaches a global transaction from the service context handle. The transaction currently attached
to the service context handle becomes inactive at the end of this call. The transaction may be resumed
later by calling OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle:, specifying a flags value of #'OCI_TRANS_RESUME'.
When a transaction is detached, the value which was specified in the timeout parameter of 
OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle: when the transaction was started is used to determine
the amount of time the branch can remain inactive before being deleted by the server's PMON process.

    Note: The transaction can be resumed by a different process than the one that detached it,
	  provided that the transaction has the same authorization. If this function is called
	  before a transaction is actually started, this function is a no-op.

Parameters

    aServiceHandle (IN)
	The service context handle.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    sword __ret;

     /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerAttach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCITransDetach(   __svcxtxhndlp,
			      __errorhndlp,
			      OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransDetach(%x)\n", __svcxtxhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransDetach(%x) failed\n", __svcxtxhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciTransactionForget:aServiceHandle errorHandle:anErrorHandle
"
forgets a heuristically completed global transaction. The server deletes the status of the transaction
from the system's pending transaction table.
You set the XID of the transaction to be forgotten as an attribute of the transaction handle (#'OCI_ATTR_XID').

Parameters:

    aServiceHandle (IN)
	The service context handle in which the transaction resides.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerAttach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }

    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCITransForget(   __svcxtxhndlp,
			      __errorhndlp,
			      OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransForget(%x)\n", __svcxtxhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransForget(%x) failed\n", __svcxtxhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciTransactionPrepare:aServiceHandle errorHandle:anErrorHandle
"
prepares the specified global transaction for commit. This call is valid only for global transactions.
The call raises an OCISuccessWithInfoSignal-Signal if the transaction has not made any changes.
The error handle will indicate that the transaction is read-only.

Parameters:

    aServiceHandle (IN)
	The service context handle.

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    sword __ret;

    /* check if all attributes have correct type */
    if(!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCIServerDetach called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp   = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCITransPrepare(   __svcxtxhndlp,
			       __errorhndlp,
			       OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransPrepare(%x)\n", __svcxtxhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransPrepare(%x) failed\n", __svcxtxhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciTransactionRollback:aServiceHandle errorHandle:anErrorHandle
"
rolls back the current transaction which is defined as the set of statements executed since the last
OCI8Lib>>ociTransactionCommit:flag:errorOCIHandle: or since OCI8Lib>>ociSessionBegin:userSession:credential:mode:errorHandle:.
If the application is running under object mode then the modified or updated objects in the object cache
for this transaction are also rolled back. Attempting to roll back a global transaction that is not
currently active causes an error.

Parameters:

    aServiceHandle (IN)
	A service context handle. The transaction currently set in the service context handle is rolled back. 

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCITransRollback called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);

    __ret = OCITransRollback(   __svcxtxhndlp,
				__errorhndlp,
				OCI_DEFAULT);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransRollback(%x)\n", __svcxtxhndlp);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransRollback(%x) failed\n", __svcxtxhndlp);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

!

primOciTransactionStart:aServiceHandle time:aTimeOutInteger flags:aFlagInteger errorHandle:anErrorHandle
"
sets the beginning of a global or serializable transaction. The transaction context currently
associated with the service context handle is initialized at the end of the call if the flags
parameter specifies that a new transaction should be started.

The XID of the transaction is set as an attribute of the transaction handle (#'OCI_ ATTR_XID').

Parameters:

    aServiceHandle (IN/OUT)
	The service context handle. The transaction context in the service context handle is initialized
	at the end of the call if the flag specified a new transaction to be started.

    aTimeOutInteger (IN)
	The time, in seconds, to wait for a transaction to become available for resumption
	when #'OCI_TRANS_RESUME' is specified. When #'OCI_TRANS_NEW' is specified, the timeout
	parameter indicates the number of seconds the transaction can be inactive before it
	is automatically aborted by the system. A transaction is inactive between the time
	it is detached with OCI8Lib>>ociTransactionDetach:errorOCIHandle: and the time it is resumed
	with OCI8Lib>>ociTransactionStart:time:flags:errorOCIHandle: (with mode = #'OCI_TRANS_RESUME').

    aFlagInteger (IN)
	Specifies whether a new transaction is being started or an existing transaction is
	being resumed. Also specifies serializiability or read-only status. More than a
	single value can be specified. By default, a read/write transaction is started.
	The flag values are:

	    #'OCI_TRANS_NEW'          - starts a new transaction branch. By default starts a tightly coupled 
					and migratable branch.
	    #'OCI_TRANS_TIGHT'        - explicitly specifies a tightly coupled branch
	    #'OCI_TRANS_LOOSE'        - specifies a loosely coupled branch
	    #'OCI_TRANS_RESUME'       - resumes an existing transaction branch.
	    #'OCI_TRANS_READONLY'     - start a read-only transaction
	    #'OCI_TRANS_SERIALIZABLE' - start a serializable transaction

    anErrorHandle (IN/OUT)
	An error handle you can pass to OCI8Lib class>>errorStringForHandle:status:
	for diagnostic information in the event of an error.

Returns:
    Symbol indicating the status of the operation (OCI_SUCCESS, OCI_ERROR, ....)

<return: Symbol>
"

%{  /* UNLIMITEDSTACK */

    OCISvcCtx * __svcxtxhndlp;
    OCIError * __errorhndlp;
    ub4 __flags;
    uword __time;
    sword __ret;

    /* check if all attributes have correct type */
    if (!__isExternalAddress(aServiceHandle) || !__isExternalAddress(anErrorHandle) 
	|| !__isInteger(aFlagInteger) || !__isInteger(aTimeOutInteger)) {
	if (@global(OCI8Lib:Debugging) == true){
	    fprintf(stderr, "OCI8Lib [error]: OCITransStart called with wrong attributes.\n");
	}
	RETURN(@symbol(attributeError));
    }
    __svcxtxhndlp = (OCISvcCtx *) __externalAddressVal(aServiceHandle);
    __errorhndlp  = (OCIError *) __externalAddressVal(anErrorHandle);
    __flags = __unsignedLongIntVal(aFlagInteger);
    __time  = __unsignedLongIntVal(aTimeOutInteger);

    __ret = OCITransStart(      __svcxtxhndlp,
				__errorhndlp,
				__time,
				__flags);

    if (@global(OCI8Lib:Debugging) == true){
	if (__ret == OCI_SUCCESS) {
	    fprintf(stderr, "OCI8Lib [info]: OCITransStart(%x, %d, %d)\n", __svcxtxhndlp, __time, __flags);
	} else {
	    fprintf(stderr, "OCI8Lib [warning]: OCITransStart(%x, %d, %d) failed\n", __svcxtxhndlp, __time, __flags);
	}
    };
    RETURN(__ociReturnSymbol(__ret));
%}.

! !

!OCI8Lib class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Lib.st,v 1.120 2010/02/26 09:46:36 stefan Exp $'
! !

OCI8Lib initialize!
